#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "include.h"
#include "run.h" 
#ifdef USE_MPI
#include "usempi.h"
#endif

#define MOM 1 /* Flag for nonzero P_1 or L  */

/*
  This file runs the lanczos algorithm for finding the lowest
  Eigenvalues of the Hamiltonian.                        
*/

#ifdef __uxp__   /*  Fujitsu vpp */

extern int fcn(integer *m, integer *npar, doublereal *xp, 
	       doublereal *fvecc, integer *iflag){
  return 0;
}
#endif

int MAIN(int argc, char **argv){
  int i;
  sectors x={0};
  /*           1 */
  clock_t t1;
  time_t t2,tf;
  element angle[HINDEX+2];
#if 0
  element beta;
#endif
#if HINDEX==1
#if MOM
  int   ht[HINDEX]={0},np=8,kt=28,cyclic=1,o=1,multi=1,multi2=2,nfig=12;
  element kmax=3.0,ll=3.0;
#else /*  1             0     24    8       1      8   */
  int   ht[HINDEX]={0},kt=28,np=8,cyclic=1,o=1,multi=1,nfig=12;
#endif
#if 1
  element couplings[NPARAMS]={0.1,2.0,-0.2,-0.23,20.0,0.0};
#else
  element couplings[NPARAMS]={0.0324919696232906307,1.0,0.0,0.0,0.0,-2.0};
#endif

#elif HINDEX==2

#if MOM
  /* Profile examples from the end of 1998 use o=-1,multi=9,multi2=8 */
  /*                          6    20                    9        8 */
  int     ht[HINDEX]={0,0},np=6,kt=20,cyclic=1,o=1,multi=9,multi2=8,nfig=14;
  element kmax=4.0,ll=4.0;
#else  /*   1                    24    8       1      8   */
  int     ht[HINDEX]={0,0},np=6,kt=20,cyclic=1,o=1,multi=3,nfig=14;
#endif
#if 1  /* test couplings */
       /* In older results, lambda_5 is scaled differently, use 0.25 or 1.0 */
  element  couplings[NPARAMS]={0.03, 1.0, 0.7,0.03,-0.005, 3.0,0.13,1.0,
                               -2.0,1.0};
#elif 0  /* older test couplings */
  element  couplings[NPARAMS]={0.1, 1.0, 0.2,0.1,0.15,0.0,0.15,0.0,0.0,0.0};
#elif 0 /* couplings used in first 3+1 paper */
  element  couplings[NPARAMS]={0.051777,1.0,0.759751,-0.075265,-0.139860,
			       8.559161,0.157577,1.847000,-3.702121};
#endif
#endif
  size_t j;
  integer nval=5;
  doublereal *val,*vec;
#if MOM
  sectors y={0};
  int ipt[HINDEX];
#if 0
  element transmom[HINDEX];
#endif
#endif

#if defined(USE_MPI) && !ONE_PROCESSOR_SPECTRUM
  int myrank;
  MPI_Init(&argc, &argv);     /* initialize MPI */
  initmpisectors();
  MPI_Comm_rank(MPI_COMM_WORLD,&myrank);
  if(myrank != MASTER){
    slave();
    MPI_Finalize();   /* cleanup MPI */
    return 0;
  }
#endif

#ifdef BABBAGE  /* Routine to initialize Fortran on Babbage */
  if(hf_fint((char*)NULL)) exit(1);
#endif

  
#if MOM
  for(i=0;i<HINDEX;i++)ipt[i]=0;
#endif

  /******  Input parameters for the basis  *************/
#if 0   /* input parameters for basis (instead of defaults.) */
  printf("Input ht[%i], p truncation, kt=2*K, "
	       "cyclic, o, multiplet:\n",HINDEX);
  for(i=0;i<HINDEX;i++)scanf("%i",ht+i);
  scanf("%i %i %i %i %i",&np,&kt,&cyclic,&o,&multi);
#if MOM
  if(kt<0&&!cyclic){
    printf("Input P^+ cutoff kmax and long separation ll:\n");
    scanf("%lg %lg",&kmax,&ll);
  } else {
    printf("Input second multiplet:\n");
    scanf("%f",&multi2);
  }
#endif
#endif

  /* print out basis properties */
  printf("winding=");
  for(i=0;i<HINDEX;i++)printf(" %i",ht[i]);
  printf(", %i particles, K=%i/2, cyclic=%i, o=%i, multi=%i",
	 np,kt,cyclic,o,multi);
#if MOM && !COMPLEX
  if(kt>0&&cyclic)
    printf(" and o=%i multi2=%i",-o,multi2);
  if(kt<0&&!cyclic)
    printf("\n  K_max=%e L=%e",kmax,ll);
  printf("\n");
#endif     

  t1=clock();
  initbasis(&x,ht,np,kt,cyclic,o,multi);
#if 0
  makebasis(&x);
#endif
#if MOM
  if(o==0){
    fprintf(stderr,"Need nonzero o, exititing\n");
    exit(10);
  }
  if(kt<0&&!cyclic)
    initbasis(&y,ht,np,kt,cyclic,-o,multi);
  else
    initbasis(&y,ht,np,kt,cyclic,-o,multi2);
#endif

  printf("Input number of eigenvalues:  ");
  scanf("%li",&nval);
  printf("\n");
  
  /************ Start main loop of program ****************/

  do{

#if 0 /******  Input coupling constants  *************/
    printf("Input %i couplings:  \n",NPARAMS);
    for(i=0;i<NPARAMS;i++) scanf("%le",couplings+i);
#endif
    printf("Couplings:");
    for(i=0;i<NPARAMS;i++) printf(" %.14g",couplings[i]);
    printf("\n");
    t1=clock();

#if MOM
    if(x.kt<0 && !x.cyclic){
      angle[HINDEX]=kmax;
      angle[HINDEX+1]=ll;
    } else {    
      /*  check for nonzero winding number */
      if(!iswinding(x.ht)){
	printf("Input transverse momenta (%i): ",HINDEX);
	for(i=0; i<HINDEX; i++){
	  scanf("%lg",angle+i);
#if 0
	  scanf("%lg",transmom+i);
#endif
	}
	printf("\n");
      } else {
	if(multi!=0)fprintf(stderr,"Need multi=0 for nonzero winding\n");
	printf("Input integer transverse momenta; to (");
	for(i=0; i<HINDEX; i++)printf("%i,",ht[i]-1);
	printf("):");
	for(i=0; i<HINDEX; i++)scanf("%i",ipt+i);
	printf("\n");
	for(i=0; i<HINDEX; i++){
#if 0
	  transmom[i]=2.0*(atan(1.) * 4.0)*(element) ipt[i]/(element) ht[i];
#endif
	  angle[i]=2.0*(atan(1.) * 4.0)*(element) ipt[i]/(element) ht[i];
	}
      }
    }
#endif

    printf("Starting calculation of spectrum.\n");
    t1=clock();
    time(&t2);
#if MOM
    spectrum(&x,&y,&val,&vec,nval,couplings,angle,x.kt,nfig);
#else
    spectrum(&x,&x,&val,&vec,nval,couplings,angle,x.kt,nfig);
#endif
    time(&tf);
    printf("%.2f CPU seconds (%li wall) to construct Hamiltonian "
	   "and run Lanczos\n",
	   (clock()-t1)/(float) CLOCKS_PER_SEC,tf-t2);
    
    if(val==NULL){
      fprintf(stderr,"Error in Spectrum, exiting\n");
      break;
    }
    
    for(j=0; j<nval; j++){
      printf("    %21.14e\n",val[j]);
    } 

    freebasis(&x);
#if MOM
    freebasis(&y);
#endif

    printf("continue? (0=no)");
    scanf("%i",&i);
  }while(i);

#if defined(USE_MPI) && !ONE_PROCESSOR_SPECTRUM
  MasterShutDown();
  MPI_Finalize();             /* cleanup MPI */
#endif
  return 0;
}



















