#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include "include.h"
#include "improved.h"
#include "interact.h"

#define PI 3.1415926535897932385

/*
  This calcualates the determinant operator without
  permuting the legs of a state
  complex_element *x is updated
*/


void baryon_determinant(complex_element *x, 
			partype *left, int *left_markers, int npl, 
			partype *right, int *right_markers, int npr, 
			element *couplings){
#if GLUE_PERIODIC /*  routine assumes quarks are antiperiodic */
  int bits;  /* bits switch left-right for a given leg */
  partype *pl,*pr; /* pointer to beginning of leg */
  int leg; 
  int link; /* position in leg of link to be removed */
  int i;
  int flag;
  partype kill[NCOLORS];
  unsigned int ktotal;
  element prod;

  /* don't even try all left or all right */
  for(bits=1; bits<(1<<NCOLORS)-1; bits++){
    ktotal=0; 
    flag=0;
    for(leg=0; leg<NCOLORS; leg++){
      pl=left+left_markers[leg];
      pr=right+right_markers[leg];
      if(bits&(1<<leg)){
	link=left_markers[leg+1]-left_markers[leg]-2;
	if(link<1)break; /* no link */
	/* right and left legs have different number of spectators */
	if(link != right_markers[leg+1]-right_markers[leg]-1){
	  flag=1;
	  break;
	}
	kill[leg]=pl[link];
	ktotal += kill[leg]&KMASK;  /* assume glue periodic */
      } else {
	link=right_markers[leg+1]-right_markers[leg]-2;
	if(link<1)break; /* no link */
	/* left and right legs have different number of spectators */
	if(link != left_markers[leg+1]-left_markers[leg]-1){
	  flag=1;
	  break;
	}
	kill[leg]=pr[link];
	ktotal -= kill[leg]&KMASK;  /* assume glue periodic */
      }
      for(i=0; i<link; i++)if(pl[i]!=pr[i])break;
      if(i!=link)break; /* spectators don't match */
      if(!H_EQUAL(kill[leg],kill[0]))break; /* helicity doesn't match */
    }

    /* we have a winner! */
    if(leg==NCOLORS && ktotal==0){
      for(leg=0, prod=1.0; leg<NCOLORS; leg++)prod*=GE(kill[leg]);
      /* The color factor is N!/sqrt(N^N).  
	 There are N! possible contractions */
      x->re += couplings[ZETA]*2*PI*FACTORIAL(NCOLORS)*FACTORIAL(NCOLORS)/
	sqrt(pow(4*PI*NCOLORS,NCOLORS)*prod);
#if 0
      if(flag){ 
	printf("match problem\n    ");print_state(stdout,left,TYPE_BARYON,npl);
	printf("  "); print_state(stdout,right,TYPE_BARYON,npr);
	printf("\n");
      }
#endif
    } 
  }
#else
  exit(-1);
#endif
}



/* 
  This calculates matrix elements of the Hamiltonian for a baryon.

  It does either zero or nonzero transverse momentum
  based on the value of current->momflag.
*/

void interact_baryon(complex_element *x, partype *left, int npl, 
		     partype *right, int npr, element *couplings){
  int i,j,k;
  partype lint[2],rint[2];
  int right_markers[NCOLORS+1];
  int left_markers[NCOLORS+1];
  int lpm[NCOLORS+1],rpm[NCOLORS+1];
  partype right_perm[NPMAX];
  partype left_perm[NPMAX];
  int perm,ip;

  x->re=0.0; x->im=0.0;

  find_markers(right_markers,right,npr);
  find_markers(left_markers,left,npl);

  for(perm=0; perm<FACTORIAL(NCOLORS); perm++){
    permute_baryon(perm,right_markers,right_perm,right,npr);
    find_markers(rpm,right_perm,npr);


#if 1  /******** do determinant interaction ***************/
    baryon_determinant(x,left,left_markers,npl,
		       right_perm,rpm,npr,couplings);
#else
    {static int flag=10; if(flag>0 && flag--)
      fprintf(stderr,__FILE__":  determinant interaction off\n");}
#endif

    /******************************************************
      interactions that go across the epsilon,
      they are order 1/N for color factor
      and cannot change the particle number 
    ********************************************************/

    if(npl==npr)
	/* do permutations where spectators are in order */
      for(ip=0; ip<NCOLORS*(NCOLORS-1); ip++){
	permute_baryon(ip,left_markers,left_perm,left,npl);
	find_markers(lpm,left_perm,npl);
	/* check that legs match, lpm==rpm */
	for(i=1; i<NCOLORS; i++)
	  if(lpm[i]!=rpm[i])break;
	if(i<NCOLORS)continue;
 	/* check spectators on other legs match */
	for(i=lpm[2]; i<npl; i++)
	  if(left_perm[i]!=right_perm[i])break;
	if(i<npl)continue;

#if 1	/* Four-quark interaction */
	/*   There is no color factor for this diagram; 
	   however, this is compensated by the 
           N*(N-1) different possible contractions */
	/* check for only quark in each leg */
 	if(lpm[1]==2 && lpm[2]==4){  /* assume lpm==rpm */
	  element z=0;
	  interact_four_quark(&z,left_perm[0],left_perm[2],
			      right_perm[0],right_perm[2],
			      couplings);
	  /* operator is half as big for quark-quark */
	  /*  printf("y=%g%+gi\n",y.re,y.im);*/
	  x->re += z*0.5/NCOLORS;  
	} else
#else
	  {static int flag=10; if(flag>0 && flag--)
	    fprintf(stderr,__FILE__":  four-quark interactions off\n");}
#endif

#if 1  /* interactions between legs involving a link */

	/* Quark-link interactions */
        if(lpm[1]==2 && lpm[2]-lpm[1]>2){ /* assume lpm==rpm */
	  /* check spectators on leg */
	  for(i=lpm[1]; i<lpm[2]-2; i++)
	    if(left_perm[i]!=right_perm[i])break;
	  if(i<lpm[2]-2)continue;
	  lint[0]=left_perm[0]; rint[0]=right_perm[0];
	  lint[1]=left_perm[lpm[2]-2];  rint[1]=right_perm[rpm[2]-2];
#if 1   /* debugging flag to turn off ends of the strings */
	  if(working.momflag) 
	    leg_quark_gluec(x,lint,2,rint,2,couplings);
	  else
	    leg_quark_glue(x,lint,2,rint,2,couplings);
#endif  /* debugging flag to turn off end of string interactions */
	  
	}
	/* link-link interactions */
	else if(lpm[1]>2 && lpm[2]-lpm[1]>2){ /* assume lpm==rpm */
	  /* check spectators on first leg */
	  for(i=lpm[0]; i<lpm[1]-2; i++)
	    if(left_perm[i]!=right_perm[i])break;
	  if(i<lpm[1]-2)continue;
	  /* check spectators on second leg */
	  for(i=lpm[1]; i<lpm[2]-2; i++)
	    if(left_perm[i]!=right_perm[i])break;
	  if(i<lpm[2]-2)continue;
	  lint[0]=left_perm[lpm[1]-2]; rint[0]=right_perm[rpm[1]-2];
	  lint[1]=left_perm[lpm[2]-2]; rint[1]=right_perm[rpm[2]-2];
	  if(working.momflag)
	    leg_glue_gluec(x,lint,2,rint,2,couplings);
	  else
	    leg_glue_glue(&(x->re),lint,2,rint,2,couplings);
	}
#else
    {static int flag=10; if(flag>0 && flag--)
      fprintf(stderr,__FILE__":  between leg interactions turned off\n");}
#endif
      }

#if 1  /****************************************************** 
	     loop through legs and do interactions within a leg 
       ******************************************************/

    for(ip=0; ip<NCOLORS; ip++){
      permute_baryon(ip,left_markers,left_perm,left,npl);
      find_markers(lpm,left_perm,npl);
      /* check other legs match */
      for(j=lpm[1], k=rpm[1]; j<npl && k<npr; j++, k++)
	if(left_perm[j]!=right_perm[k])break;
      if(j<npl || k<npr)continue;

#if 1 /* glue-glue interactions */
      if(left_perm[0]==right_perm[0]){ /* first, check quarks match */
	if(working.momflag)
	  interact_gluec(x,left_perm+1,lpm[1]-2,right_perm+1,rpm[1]-2,
			 couplings);
	else
	  interact_glue(&(x->re),left_perm+1,lpm[1]-2,right_perm+1,rpm[1]-2,
			couplings);
      }
#else
    {static int flag=10; if(flag>0 && flag--)
      fprintf(stderr,__FILE__":  link-link interactions turned off.\n"); }
#endif 

#if 1   /* debugging flag to turn off ends of the strings */
      /* 
	 calculate the contibutions from the end of the string.   
	 Need to put the whole state in so that the kappa self-intertias
	 are turned off at the fock space truncation and
	 spectators are checked.

         The mass shifts MM_i are based on nptrunc-npl.
      */
      if(working.momflag)
	interact_quarkc(x,left_perm,npl,right_perm,npr,couplings,
			lpm[1]-2);
      else
	interact_quark(x,left_perm,npl,right_perm,npr,couplings,lpm[1]-2);
#else
      fprintf(stderr,__FILE__":  quark-link interactions turned off.\n"); 
#endif  
    }
#else
    {static int flag=10; if(flag>0 && flag--)
      fprintf(stderr,__FILE__":  within-leg interactions off.\n");}
#endif 
  }

#if 0 /* debug print */
  if(x->re!=0 || x->im!=0.0){
    printf("   last x=%f%+f*I:  ",x->re,x->im);
    print_state(stdout,left,TYPE_BARYON,npl);
    printf(" - ");
    print_state(stdout,right,TYPE_BARYON,npr);
    printf("\n");
  }
#endif  

  return;
}








