#ifndef OBSERVABLE_INCLUDED
#define OBSERVABLE_INCLUDED

#include "spectrum.h"

/*
   Calculations of Observables

   First, some macros specifying possible observables
*/

enum observable{ /* typedef does not work for enum */
  O_IDENTITY, /* Identity (mainly for debugging) */
  O_NUMBER, /* Spectrum of the number operator */
  O_QUARK_DISTRIBUTION, /* Quark longitudinal distribution function */
  O_QUARK_LINK_DISTRIBUTION /* Quark-adjacent link distribution function */
};

const char *observable_string(enum observable t);

/* Uniform way to return results for an observable */

#define INITIAL_OBSERVABLE_RESULT {-1,0,NULL} /* initialization */

typedef struct {
  enum observable observable;  /* type of observable result */
  int length;  /* length of array containing results */
  element *result;  /* pointer to  array containing results */
} observable_result;


void find_observable(
		     int which_state, /* with eigenstate, starting with 0 */
		     observable_result *x, /* result of calculation */
		     eigensystem *eigen, /* the results for the eigensystem */
		     full_basis *basis   /* pointer to the basis created */
		     );

/* free memory used used to store wavefunction results */
void free_observable_result(observable_result *result);

void print_real_list(FILE *fp,element *x,int n);
void print_complex_list(FILE *fp,complex_element *x,int n);

#endif









