/*
  Print routines.
  The purpose of this file is to remove some of the length
  from christiaan.c.  It is included in the middle of that file.
*/

/* These are all the quantities returned by fcn */
typedef struct {
  element chi2;  /* set to negative to force update */
  doublereal pextrap;
  element wfitting[MAXTH]; element lfitting[MAXTH]; 
  element rescale;  /* G^2 N/\sigma */
  element spectrumvals1[NP1+1][NPERP][NPERPVALS]; 
  element spectrumvals2[NP2+1][NPERP][NPERPVALS];
  element stillvals[NSTILL][NSTILLVALS]; 
  element c2[NP1+NP2][NPERP][NPERPVALS];
  element extra[N_LOG_VALS];
  element chiral[N_FOUR_FERMI_VALS][FOUR_FERMI_K];
  element decay_constants[DECAY_CONSTANTS];
} meson_best;

/*
   print out definitions of bases, etc
*/

#define SQRT_STRING_TENSION 440.0  /* square root of string tension in MeV */

void print_christiaan_header(FILE *fp, doublereal xtol,  int *chargest, 
			     int *multis, 
			     int *multit, int *charge_still, int *multistill){
  int i,j,k;
  fprintf(fp,"********* Eigenvalues for transverse lattice mesons *********\n");
  fprintf(fp,"Couplings:\n    ");
  for(i=0; i<NPARAMS; i++){
    fprintf(fp,"  %i:",i);
    print_coupling_string(fp,i);
    if(i%5==4 && i!=NPARAMS-1)fprintf(fp,"\n    ");
  }
  fprintf(fp,"\nsqrt(string tension) is %g MeV.\n",SQRT_STRING_TENSION);
  fprintf(fp,"Use chi^2 fit with %i criteria and tolerance %g.\n",
	  CHI_DIM,xtol);
  switch(scalemethod){
  case 0:
    fprintf(fp,"Overall scale from glueball data.\n");
    break;
  case 1:
    fprintf(fp,"Overall scale from minimizing chi^2.\n");
    break;
  default:
    fprintf(stderr,__FILE__":  Incorrect scale method = %i, exiting.\n",
	    scalemethod);
    return;
  }
#if KSQUARED
  fprintf(fp,"1/K^2 term used in all extrapolations.\n");
#endif
#if FIX_MASSES>0
  fprintf(fp,"Fit %i masses to experimental values\n  "
	  "(charge,multi,state,mass (MeV),error (MeV)) =",FIX_MASSES);
  for(i=0; i<FIX_MASSES; i++){
    if(i%2==0)fprintf(fp,"\n  ");
    fprintf(fp," (%i,%i,%i,%g,%g)",experiment[i].charge,experiment[i].multi,
	    experiment[i].state,
	    sqrt(experiment[i].value)*SQRT_STRING_TENSION,
	    /* Note we have to convert the error properly */
	    experiment[i].error/(2.0*sqrt(experiment[i].value))*
	    SQRT_STRING_TENSION);
  }
  fprintf(fp,".\n");
#endif
#if FPARITY>0
  fprintf(fp,"Degenerate pairs (charge,multi,state) "
	  "with relative error.\n");
  for(i=0; i<FPARITY; i++)
    fprintf(fp,"  (%2i,%2i,%i) and (%2i,%2i,%i), error %g\n",
	    whichparity[i][0].charge,whichparity[i][0].multi,
	    whichparity[i][0].state,
	    whichparity[i][1].charge,whichparity[i][1].multi,
	    whichparity[i][1].state,parityerr[i]);
#else
  fprintf(fp,"No degenerate pairs.\n");
#endif
#if NP1>0
  fprintf(fp,"Spectrum for P_perp a = (0,0),");
  for(i=0; i<NP1; i++){
    fprintf(fp," (");
    for(j=0; j<HINDEX; j++)fprintf(fp," %g",p1a[i][j]);
    fprintf(fp,")");
  }
  fprintf(fp,"\n  for %i states (multiplet, charge, c^2 error for each) =",
	  NPERPVALS);
  for(i=0; i<NPERP; i++){
    fprintf(fp,"\n  (%2i, %2i,",multis[i],chargest[i]);
    for(j=0; j<FCSQUARED; j++)
      fprintf(fp," %g",c2err[i][j]);
    fprintf(fp,")");
  }
  fprintf(fp,".\n");
#endif
#if NP2>0
  fprintf(fp,"Spectrum for P_perp a = (0,0),");
  for(i=0; i<NP2; i++){
    fprintf(fp," (");
    for(j=0; j<HINDEX; j++)fprintf(fp," %g",p2a[i][j]);
    fprintf(fp,")");
  }
  fprintf(fp,"\n  for %i states (multiplet, charge, c^2 error for each) =",
	  NPERPVALS);
  for(i=0; i<NPERP; i++){
    fprintf(fp,"\n  (%2i, %2i,",multit[i],chargest[i]);
    for(j=0; j<FCSQUARED; j++)
      fprintf(fp," %g",c2err[i][j]);
    fprintf(fp,")");
  }
  fprintf(fp,".\n");
#endif
#if NSTILL>0
  fprintf(fp,"Ordinary spectra multiplets, %i states, (charge,multiplet) =",
	  NSTILLVALS);
  for(i=0; i<NSTILL; i++){
    if(i%6==0)fprintf(fp,"\n ");
    fprintf(fp," (%2i, %2i) ",charge_still[i],multistill[i]);
  }
#endif
  fprintf(fp,".\nAll spectra extrapolated using (K,p) =");
  for(i=0, k=3; i<NS; k++, i++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i/2,%i) ",kt[i],np[i]);
  }
#if PNS
  fprintf(fp,"\np-extrapolation using n=(");
  for(l=0; l<HINDEX; l++)fprintf(fp," %i",pht1[l]);
  fprintf(fp,") and (K,p) =");
  for(j=0, k=4; j<PNS; k++, j++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i/2,%i) ",pkt1,pnp1[j]);
  }
#endif
#if N_LOG_VALS
  fprintf(fp,"\nLog divergence determined using (K,p) =");
  for(j=0, k=4; j<LOG_NS; k++, j++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i/2,%i) ",log_kt[j],log_np[j]);
  }
  fprintf(fp,"\n and (charge,multiplet,error) =");
  for(j=0, k=1; j<N_LOG_VALS; k++, j++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i,%i,%g) ",log_state[j].charge,log_state[j].multi,
	    log_state[j].error);
  }
#endif
#if N_FOUR_FERMI_VALS
  fprintf(fp,"\nfour-fermi found by demanding constant %s (K,p) =",
	  FOUR_FERMI_FROM_DECAY?"decay constant":"mass");
  for(j=0, k=4; j<FOUR_FERMI_K; k++, j++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i/2,%i) ",chiral_kt[j],chiral_np[j]);
  }
  fprintf(fp,"\n and (charge,multiplet) =");
  for(j=0, k=1; j<N_FOUR_FERMI_VALS; k++, j++){
    if(k%5==0)fprintf(fp,"\n ");
    fprintf(fp," (%i,%i) ",four_fermi[j].charge,four_fermi[j].multi);
  }
#endif
#if DECAY_CONSTANTS==1
  fprintf(fp,"\n Pion decay constant fit to experiment.\n");
#endif
  fprintf(fp,"Result format:\n"
	  " Fit info, # steps, chi^2, p damping, scale G^2 N/sigma.\n"
	  " The %i couplings  (G^2 N units);\n"
	  "   which couplings -- if any -- were fit.\n"
	  " Winding potential and heavy souce potential fits (G^2 N units).\n"
#if N_LOG_VALS 
	  " The log(K) coefficients (G^2 N units), \n"
#endif
#if N_FOUR_FERMI_VALS==1
	  "   The pion mass^2 vs K (G^2 N units).\n"
#endif
#if DECAY_CONSTANTS==1
	  "   The meson valence amplitudes.\n"
#endif
	  " The rescaled masses (MeV) for each P_perp*a and c^2 values.\n"
#if NSTILL>0
	  " Finally come the masses (MeV) for the ordinary spectra.\n"
#endif
	  "\n",NPARAMS);
  fflush(fp);
}

/* 
   print out spectrum results
*/

#define MASSOF(A) (A>0?SQRT_STRING_TENSION*sqrt(best->rescale*(A)):0)

void print_christiaan(FILE *fp,meson_best *best, element chi2, integer info, 
		      integer nfev,integer npar){
  int i,j,k;
  fprintf(fp,"%i %i " PDIGITS " " PDIGITS " " PDIGITS "\n",
	  (int) info,(int) nfev,chi2,best->pextrap,best->rescale);
  for(i=0; i<NPARAMS; i++){
    if(i && i%6==0)fprintf(fp,"\n    ");
    fprintf(fp,PDIGITS " ",par[i]);
  }
  if(npar>0)fprintf(fp,"\n");
  for(i=0; i<npar; i++)fprintf(fp," %i",wpar[i]);
  fprintf(fp,"\n");
  for(i=0; i<wth; i++)fprintf(fp,"%g ",best->wfitting[i]);
  fprintf(fp,"    ");
  for(i=0; i<lth; i++)fprintf(fp," %g",best->lfitting[i]);
  fprintf(fp,"\n");
#if N_LOG_VALS
  for(i=0; i<N_LOG_VALS; i++)fprintf(fp," %g",best->extra[i]);
  fprintf(fp," ");
#endif
  for(j=0; j<N_FOUR_FERMI_VALS; j++){
    for(i=0; i<FOUR_FERMI_K; i++)fprintf(fp," %g",best->chiral[j][i]);
    fprintf(fp," ");
  }
#if DECAY_CONSTANTS
  for(i=0; i<DECAY_CONSTANTS; i++)fprintf(fp," %g",best->decay_constants[i]);
#endif
#if N_LOG_VALS ||  N_FOUR_FERMI_VALS || DECAY_CONSTANTS
  fprintf(fp,"\n");
#endif
#if NP1>0
  for(i=0; i<NPERP; i++)
    for(k=0; k<NPERPVALS; k++){
      for(j=0; j<NP1+1; j++)
	fprintf(fp," %f",MASSOF(best->spectrumvals1[j][i][k]));
      for(j=0; j<NP1; j++)
	fprintf(fp," %g",best->c2[j][i][k]); 
      fprintf(fp,"\n");
    }
#endif
#if NP2>0
  for(i=0; i<NPERP; i++)
    for(k=0; k<NPERPVALS; k++){
      for(j=0; j<NP2+1; j++)
	fprintf(fp," %g",MASSOF(best->spectrumvals2[j][i][k]));
      for(j=0; j<NP2; j++)
	fprintf(fp," %g",best->c2[NP1+j][i][k]);
      fprintf(fp,"\n");
    }
#endif
  for(i=0; i<NSTILL; i++){
    for(j=0; j<NSTILLVALS; j++){
      fprintf(fp,"%g ",MASSOF(best->stillvals[i][j]));
    }
    fprintf(fp,"\n");
  }
  fprintf(fp,"\n");
  fflush(fp);
}


