#ifndef INCLUDE_INCLUDED
#define INCLUDE_INCLUDED

/****************************************************************
        Transverse lattice Global definitions
******************************************************************/

/*
  Form of matrix elements to use in calculations 
      1  use internally constructed DLCQ matrix elements.  
      3  use internally constructed improved matrix elements. 
  (historically 0 and 2, were for loading elements from a file)
*/  

#ifndef IMPROVED
#define IMPROVED 1
#endif

#ifndef HINDEX
#define HINDEX 2  /* Number of transverse dimensions */
#endif

#ifndef QUARK_FLAVORS
#define QUARK_FLAVORS 1 /* Number of quark flavors */
#endif

#ifndef NCOLORS
#define NCOLORS 3  /* Number of colors (needed for baryon calculations) */
                   /* Must be 3 or larger  */
#endif

#define NPMAX 16  /* maximum number of particles. This is used for some
                    temporary storage in small arrays.  It is checked in
                    the routines that create the basis states. */

#define QUARK_PERIODIC 0 /* Periodic boundary conditions for quarks? 
                            Only antiperiodic is implemented now. */
#ifndef CONSTRAINED_QUARK
#define CONSTRAINED_QUARK 1 /* include constrained quark fields in the 
                               basis of states */
#endif

#define GLUE_PERIODIC 1  /* Periodic boundary conditions for mesons? 
                            Antiperiodic is for checking against older code;
			    one cannot have quarks.                */

#if GLUE_PERIODIC
#define ADJACENT  0       /* Specify maximum number of adjacent zero modes. 
			    A value of zero means no zero modes. */
#endif

/* 
  structure to store the Hamiltonian matrix
  and definition of types "element" and "complex_element"
*/

#include "elslist.h"

/*
  sizemem is an unsigned integer large enough to express the number of bytes 
  used to store the Hamiltonian matrix in one working group.  
  It is possibly larger than size_t for parallel calculations.  
  See the definition of MPI_SIZE_T
*/

typedef unsigned long int size_mem;
#define SIZE_MEM_PRINT  "%lu" /* Print format for size_mem, either u or lu */
/* Format %zu for size_t is unavailable on TCS, use type conversion */
typedef unsigned long int lu; 

#endif
