#ifndef USEMPI_INCLUDED
#define USEMPI_INCLUDED

#include <mpi.h>
#include "spectrum.h"

/*
   Using a master-slave approach, GROUP_SIZE is the number of processors 
   for each slave.  Each slave performs parallel
   matrix construction and diagonalization.

   GROUP_SIZE=0 means parallel matrix diagonalization across all processors.
   Does not use master-slave approach.

   GROUP_SIZE=1 master slave approach with each slave constructing and 
   diagonalizing matrices locally.  

*/
#ifndef GROUP_SIZE
#define GROUP_SIZE 10  /* only group size zero works now */
#endif

/* processor rank for master */

#define WORLD_MASTER 0

/* offset for message tag numbers tag==TAG_OFFSET means "quit" */

#define TAG_OFFSET 32000

/* mpi_specify_calculation, and mpitype are initialized in
   initmpisectors below: this should be called by the main program.*/

MPI_Datatype mpi_specify_calculation;

/************************ function prototypes ********************/

int master(specify_calculation *base, element *couplings, 
	   element **vals, int njob); /* master for MPI */
void slave(int group_size);                   /* slave for MPI */
/*initialize data structures and groups */ 

void init_mpi_types(int group_size); 

integer mpi_group;  /* group for diagonalizing doing spectrum */

MPI_Comm mpiallslaves;  /* communicator handle of all slaves.  */


/******************* New mpi data type ****************************/


#define MPI_SIZE_MEM MPI_UNSIGNED_LONG /* this should match type size_mem */
         
#endif








