#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "include.h"
#include "group.h"
#include "row.h"

/* 
   This file contains routines to find the remaining
   states in a basis that might have an interaction with
   a given state.

   the routines *_emerge(...) find the possible particles
   that might emerge from an interaction.  Right now, they
   are computed "on the fly"  a possible speed improvement would
   be to use look-up tables.
*/

/*
  given adjacent gluons, find possible gluons that
  may emerge from a general interaction
*/

void gggg_emerge(partype *p, int nin, interact *x){
  int i;
  partype *sp=NULL;
  partype kt=0; /* kt is the total K, rounded down */
  for(i=0; i<nin; i++)kt+=(p[i]&KMASK);
  x->np=4-nin;
  if(nin==3){
#if !GLUE_PERIODIC
    kt+=1;
#endif
    if(H_MINUS(p[0],p[1])){
      row_memory(x,1);
      x->s[0]=kt|(p[2]&(~KMASK));
    } else if(H_MINUS(p[1],p[2])){
      row_memory(x,1);
      x->s[0]=kt|(p[0]&(~KMASK));
    } else if(H_MINUS(p[0],p[2])){
      row_memory(x,1);
      x->s[0]=kt|(p[1]&(~KMASK));
    } else 
      row_memory(x,0);
  } else if(nin==2){
    partype k;
    partype h1=p[0]&(~KMASK), h2=p[1]&(~KMASK);
    if(h1==h2){
      row_memory(x,kt+1);
      for(k=0, sp=x->s; k<=kt; k++){
	sp[0]=k|h1;
	sp[1]=(kt-k)|h2;
	sp+=2;
      }
    } else if((h1^h2) == 1<<HSHIFT){
      partype i;
      row_memory(x,(kt+1)*2*HINDEX);
      for(i=0, sp=x->s; i<2*HINDEX; i++)
	for(k=0; k<=kt; k++){
	  sp[0]=k|(i<<HSHIFT);
	  sp[1]=(kt-k)|((i^1)<<HSHIFT);
	  sp+=2;
	}
    } else {
      /* orthogonal case */
      row_memory(x,(kt+1)*2);
      for(k=0, sp=x->s; k<=kt; k++){
	sp[0]=k|h1;
	sp[1]=(kt-k)|h2;
	sp[2]=k|h2;
	sp[3]=(kt-k)|h1;
	sp+=4;
      }
    }
  } else if(nin==1){
    unsigned int count=0;
    partype k1,k2,k3,h;
    partype h1=p[0]&(~KMASK);
#if !GLUE_PERIODIC
    if(kt==0){
      row_memory(x,0);
      return;
    } else
      kt-=1;
#endif
    row_memory(x,3*(2*HINDEX-1)*(kt+1)*(kt+2)/2);
    for(k1=0, sp=x->s; k1<=kt; k1++)
      for(k2=0; k2<=kt-k1; k2++){
	k3=kt-k1-k2;
	for(h=0; h<2*HINDEX; h++){
	  if(h1==(h<<HSHIFT))continue; /* Don't overcount states */
	  sp[0]=k1|h1;              
	  sp[1]=k2|((h^1)<<HSHIFT);       
	  sp[2]=k3|(h<<HSHIFT);
	  sp[3]=k1|(h<<HSHIFT);     
	  sp[4]=k2|h1;          
	  sp[5]=k3|((h^1)<<HSHIFT);
	  sp[6]=k1|((h^1)<<HSHIFT); 
	  sp[7]=k2|(h<<HSHIFT); 
	  sp[8]=k3|h1;
	  sp+=9;
	  count+=3;
	  if(count>x->length)
	    fprintf(stderr,__FILE__":  count=%i > length=%lu\n",
		    count,(lu) x->length);
	}  
      }
  }
}

/*
  given adjacent quarks, find possible quarks that
  may emerge from a general interaction
*/

void qqqq_emerge(partype *p, int nin, interact *x){
  int i;
  partype *sp=NULL;
  partype kt=0;  /* total K (each rounded down) */
  partype k,h1,h2;
  if(nin!=2){
    fprintf(stderr,__FILE__":  qqq_emerge error\n");
    exit(5);
  }
  for(i=0; i<nin; i++)kt+=p[i]&KMASK;
  x->np=2;
  row_memory(x,(CONSTRAINED_QUARK?12:4)*(kt+1));
  for(h1=0, sp=x->s; h1<(CONSTRAINED_QUARK?4:2); h1++)
    for(h2=0; h2<(CONSTRAINED_QUARK?4:2); h2++){
      if(h1&2 && h2&2)continue; /* not both constrained */ 
      for(k=0; k<=kt; k++){
	sp[0]=k | h1<<HSHIFT | (p[0]&QUARK_FLAVOR_MASK);
	sp[1]=(kt-k) | h2<<HSHIFT | (p[1]&QUARK_FLAVOR_MASK);
	sp+=2;
      }
    }
}

/*
  given adjacent quark and source, find possible quarks that
  may emerge from a general interaction

  kt_left is twice the K cut-off
*/

void ssqq_emerge(partype *p, int nin, int kt_left, interact *x){
  partype *sp=NULL;
  partype k,h1;
  if(nin!=1){
    fprintf(stderr,__FILE__":  ssqq_emerge error\n");
    exit(5);
  }
  x->np=1;
  row_memory(x,1*(kt_left+1));
  for(h1=0, sp=x->s; h1<2; h1++)
    for(k=0; 2*k+1<=kt_left; k++){
      sp[0]=h1<<HSHIFT | k | (p[0]&QUARK_FLAVOR_MASK)
      /* flip the constrained bit */
	| ((~p[0])&QUARK_CONSTRAINED_MASK);
      sp++;
    }
}

/*
       for a general qqgg interaction, find the particles
       that can emerge.  
       This may or may not flip the
       constrained status.
       p[0] is the quark
*/

void qqgg_emerge(partype *p, int nin, interact *x){
  partype k,i,j,h1;
  partype *sp=NULL;
  partype kt=0;  /* kt is total K, each rounded down */
  for(i=0; i<nin; i++)kt+=p[i]&KMASK; 
  x->np=4-nin;
  /* number of spin and constraint bit values, 4 or 2 */
  /* 2 = no constrained states */
#if 1 /* no four particle interactions with constrained fields */
  int nspincon=2;
#else /* general case for 4-particle interactions */
  int nspincon=CONSTRAINED_QUARK && !constrained_quark(p[0])?4:2;
#endif
  if(nin==3){
#if GLUE_PERIODIC
    row_memory(x,nspincon);
    for(h1=0; h1<nspincon; h1++)
      x->s[h1]=kt | h1<<HSHIFT |(p[0]&QUARK_FLAVOR_MASK);
#endif
  }else if(nin==2){
    row_memory(x,nspincon*2*HINDEX*(kt+1));
    for(i=0, sp=x->s; i<(nspincon); i++)
      for(j=0; j<2*HINDEX; j++)
        for(k=0; k<=kt; k++){
          sp[0] = k | i<<HSHIFT | (p[0]&QUARK_FLAVOR_MASK);
          sp[1] = (kt-k) | j<<HSHIFT ;
          sp+=2;
        }
  } else if(nin==1) {
    unsigned int count=0;
    partype k1,k2,k3,h1,h2,j;
    row_memory(x,nspincon*(2*HINDEX)*(2*HINDEX)*
	       (kt+1)*(kt+2)/2);
    for(k1=0, sp=x->s; k1<=kt; k1++)
      for(k2=0; k2<=kt-k1; k2++){
	k3=kt-k1-k2;
	for(j=0; j<nspincon; j++)
	  for(h1=0; h1<2*HINDEX; h1++)
	    for(h2=0; h2<2*HINDEX; h2++){
	      sp[0]=k1 | j<<HSHIFT | (p[0]&QUARK_FLAVOR_MASK);    
	      sp[1]=k2 | h1<<HSHIFT; 
	      sp[2]=k3 | h2<<HSHIFT; 
	      sp+=3;
	      count++;
	      if(count>x->length)
		fprintf(stderr,__FILE__":  count=%i > length=%lu\n",
			count,(lu) x->length);
	    }
      }
  }
}

/*
      ssgg is gives a list of particles that can
      emerge from a static source-gluon interation.

      kt_left twice the K cutoff minus the total light field momenta.

      npin is the number of incoming link fields
*/

void ssgg_emerge(partype *p, int nin, int kt_left, interact *x){
  int i;
  partype *sp=NULL;
  for(i=0; i<nin; i++)kt_left+=2*(p[i]&KMASK)+(GLUE_PERIODIC?0:1);
  x->np=2-nin;
  if(nin==2){
    if(((p[0]^p[1])&(~KMASK))==1<<HSHIFT)
      row_memory(x,1);
    else
      row_memory(x,0);
  } else if(nin==1){
    partype k;
    partype h1=p[0]&(~KMASK);
    row_memory(x,(kt_left+(GLUE_PERIODIC?2:1))/2);
    for(k=0, sp=x->s; 2*k+(GLUE_PERIODIC?0:1)<=kt_left; k++){
      sp[0]=h1|k;
      sp++;
    }
  } else if(nin==0){
    partype k1,k2,h;
    row_memory(x,2*HINDEX*(kt_left/2+(GLUE_PERIODIC?1:0))*
      (kt_left/2+1+(GLUE_PERIODIC?1:0))/2);
    for(k1=0, sp=x->s; k1<=kt_left/2; k1++)
      for(k2=0; k1+k2+(GLUE_PERIODIC?0:1)<=kt_left/2; k2++){
	for(h=0; h<2*HINDEX; h++){
	  sp[0]=(h<<HSHIFT)|k1;    
	  sp[1]=((h^1)<<HSHIFT)|k2;  
	  sp+=2;
	}
      }
  }
}

/*
       for a general qqg interaction, find the particles
       that can emerge. 
       p[0] is the quark
*/

void qqg_emerge(partype *p, int nin, interact *x){
  int i;
  partype h;
  partype *sp=NULL;
  partype kt=0;   /* kt is the total K, each rounded down */
  for(i=0; i<nin; i++)kt+=p[i]&KMASK;
  x->np=3-nin;
  if(nin==2){
#if GLUE_PERIODIC
    row_memory(x,2);
    for(h=0; h<2; h++){
      x->s[h]=kt | (h<<HSHIFT) | (p[0]&QUARK_FLAVOR_MASK)
	/* flip the constrained bit */
	| ((~p[0])&QUARK_CONSTRAINED_MASK);
    }
#endif
  } else if(nin==1){
    partype k1,j,h;
    row_memory(x,2*2*HINDEX*(kt+1));
    for(k1=0, sp=x->s; k1<=kt; k1++)
      for(j=0; j<2; j++)
	for(h=0; h<2*HINDEX; h++){
	  sp[0]=k1 | j<<HSHIFT | (p[0]&QUARK_FLAVOR_MASK)
	    /* flip the constrained bit */
	    | ((~p[0])&QUARK_CONSTRAINED_MASK);
	  sp[1]=(kt-k1) | h<<HSHIFT; 
	  sp+=2;
	}    
  }
}

/*
       for a general qq interaction (mass term), find the particles
       that can emerge. 
       p[0] is the quark
       This is not used.
*/

void qq_emerge(partype *p, int nin, interact *x){
  x->np=1;
  if(nin==1){
    row_memory(x,1);
    x->s[0] = p[0]^ QUARK_CONSTRAINED_MASK;
  }
}
 
/**************************************************************/

/* 
    row_*(...) finds the remaining
    states in the basis bb having a possible nonzero matrix
    element with state x and puts them in y.
    It returns states in the basis occuring after or equal to x.
    It returns the resulting list of locations.  
   
    The calling routine must allocate memory for y.

    Throughout, the following relation should hold:

    if(npout!=npx-nin+s.np) 
      fprintf(stderr,__FILE__":  interaction disaster\n");

*/

#define FOUR 4  /* number of particles in link interactions */
#define THREE 3 /* number of particles in Yukawa 3-point interactions */
#define TWO 2   /* number of particles in source interactions */

/*
  Fill remaining slots with holes, well-order state,
  check if state is valid,
  and add state to sorted basis if it occurs *after* 
  the first state in the basis.
*/

void fix_add(cbase *y, full_basis *bb, partype *z, int np){
  int l;
#if 0       /* debugging print */
  printf("    final:");print_state(stdout,z,bb->type,np);printf("\n");
#endif

#ifdef BARYON_LEG_TRUNCATION /* more restrictive truncation */
  if(!baryon_leg_test(z,np,bb->type,bb->nptrunc))return;
#endif
  if(!zeromodetest(z,bb->type,np))return; /* check for too many zero modes */
  wellorder(z,bb->type,np,bb->charge,bb->o,subgroup[bb->multi]);
  for(l=np; l<bb->nptrunc; l++)z[l]=HOLE;
  for(l=0; l<bb->nptrunc; l++){
    if(z[l]<y->s[l])return;
    else if(z[l]>y->s[l]){
      /* check for non-zero norm 
	 This is a large time user for constructing a
	 matrix.  Perhaps construct a list of zero norm states
	 and perform a search. Or check for zero norm only if state is
	 not found in basis. ?????? */
      if(0==innerproduct(z,z,bb->type,np,bb->charge,bb->o,bb->multi))return;
      y->length=addstate(y->s,z,y->length,bb->nptrunc);
      return;
    }
  }
}

void row_glue(cbase *y, full_basis *bb, partype *x, int npx){
  int nin,npout;
  int i,j,l;
  partype z[NPMAX];
  partype xx[2*NPMAX];
  interact s=INTERACT_INIT; /* object holding results of interactions */ 
  y->length=0;
  y->length=addstate(y->s,x,y->length,bb->nptrunc);
  if(bb->type!=TYPE_GLUE){
    fprintf(stderr,__FILE__":  Wrong type of interaction");
    exit(55);
  }
  memcpy(xx,x,npx*sizeof(partype));
  memcpy(xx+npx,x,npx*sizeof(partype));
  for(nin=1; nin<FOUR; nin++){
    npout=npx+FOUR-2*nin;
    if(nin>npx || npout>bb->nptrunc)continue;
    for(i=0; i<npx; i++){
#if 0        /* debugging print */
      printf("Starting  ");
      print_state(stdout,xx+i,bb->type,npx);
      printf(" nin=%i\n",nin);
#endif
      gggg_emerge(xx+i,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<s.np; l++)
	  z[l]=s.s[s.np*j+l];
	for(l=0; l<npx-nin; l++)
	  z[s.np+l]=xx[l+i+nin];
	fix_add(y,bb,z,npout);
      }
    }
  }

  free_row_memory(&s);
}



void row_source(cbase *y, full_basis *bb, partype *x, int npx){
  int nin,npout;
  int i,j,l;
  partype z[NPMAX];
  partype kt=bb->kt; /* twice the total K cutoff minus light field total K */
  partype xx[NPMAX];
  interact s=INTERACT_INIT; /* object holding results of interactions */ 
  y->length=0;
  y->length=addstate(y->s,x,y->length,bb->nptrunc);
  if(bb->type!=TYPE_SOURCE){
    fprintf(stderr,__FILE__":  Wrong type of interaction");
    exit(55);
  }
  for(i=0; i<npx; i++)xx[i]=x[npx-1-i];
  for(i=0; i<npx; i++)kt-=2*(x[i]&KMASK)+(GLUE_PERIODIC?0:1);

  /* gggg interactions acting on interior of string */

  for(nin=1; nin<FOUR; nin++){
    npout=npx+FOUR-2*nin;
    if(npout>bb->nptrunc)continue;
    for(i=0; i<=npx-nin; i++){
      gggg_emerge(x+i,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<i; l++)
	  z[l]=x[l];
	for(l=0; l<s.np; l++)
	  z[l+i]=s.s[s.np*j+l];
	for(l=i; l<npx-nin; l++)
	  z[s.np+l]=x[l+nin];
	fix_add(y,bb,z,npout);
      }
    }
  }

  /* ssgg interactions */  

  for(nin=0; nin<=TWO; nin++){
    npout=npx+TWO-2*nin;
    if(nin>npx || npout>bb->nptrunc)continue;
#if 0        /* debugging print */
    printf("Starting  ");
    print_state(stdout,x,bb->type,npx);
    printf(" nin=%i kt=%i\n",nin,kt);
#endif
    ssgg_emerge(x,nin,kt,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      for(l=0; l<npx-nin; l++)
	z[s.np+l]=x[l+nin];
      fix_add(y,bb,z,npout);
    }
    ssgg_emerge(xx,nin,kt,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<npx-nin; l++)
	z[l]=x[l];
      for(l=0; l<s.np; l++)
	z[npout-1-l]=s.s[s.np*j+l];
      fix_add(y,bb,z,npout);
    }
  }

  free_row_memory(&s);
}

void row_meson(cbase *y, full_basis *bb, partype *x, int npx){
  int nin,npout;
  int i,j,l;
  partype z[NPMAX];
  partype xx[NPMAX];
  interact s=INTERACT_INIT; /* object holding results of interactions */ 
  y->length=0;
  y->length=addstate(y->s,x,y->length,bb->nptrunc);
  if(bb->type!=TYPE_MESON){
    fprintf(stderr,__FILE__":  Wrong type of interaction");
    exit(55);
  }

  /* qqqq interactions */

  if(npx==2){ 
#if 0        /* debugging print */
    printf("Starting qqqq:  ");
    print_state(stdout,x,bb->type,npx);
    printf("\n");
#endif
    qqqq_emerge(x,npx,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      fix_add(y,bb,z,s.np);
    }
  }

  /* the pseudoscalar four-fermion interaction:
     just use all possible states */
  
  if(npx<=(CONSTRAINED_QUARK?2:3)){
    partype *part;
    int np;
    reduced_basis *ss;
    makereduced(&ss,bb->type,bb->flavor,bb->ht,
		CONSTRAINED_QUARK?2:3,bb->kt,
		bb->charge,bb->o,subgroup[bb->multi]);
    for(j=0, part=ss->s; j<ss->length; j++, part+=ss->nptrunc){
      for(np=0; np<ss->nptrunc && part[np]!=HOLE; np++);
      for(l=0; l<np; l++)
	z[l]=part[l];
      fix_add(y,bb,z,np);
    }
  }

  charge_conjugation(xx,x,bb->type,npx);
  
  /* gggg gauge interactions */
  
  if(!quark_constrained(x[0]) && !quark_constrained(xx[0])){
    for(nin=1; nin<FOUR; nin++){
      npout=npx+FOUR-2*nin;
      if(nin>npx-2 || npout>bb->nptrunc)continue;
      for(i=1; i<npx-nin; i++){
#if 0        /* debugging print */
	printf("Starting gggg:");
	print_state(stdout,x+i,bb->type,npx);
	printf(" nin=%i\n",nin);
#endif
	gggg_emerge(x+i,nin,&s);
	for(j=0; j<s.length; j++){
	  for(l=0; l<i; l++)
	    z[l]=x[l];
	  for(l=0; l<s.np; l++)
	    z[l+i]=s.s[s.np*j+l];
	  for(l=i; l<npx-nin; l++)
	    z[s.np+l]=x[l+nin];
	  fix_add(y,bb,z,npout);
	}
      }
    }
  }
  
  /*  qqgg Yukawa and gauge interactions    */  
  
  /* assume that there are no constrained four-particle
     interactions.  Remove this first if(...) and
     modify qqgg_emerge above. */
  if(!quark_constrained(x[0]) && !quark_constrained(xx[0])){
    for(nin=1; nin<FOUR; nin++){
      npout=npx+FOUR-2*nin;
#if 0        /* debugging print */
      printf("Starting qqgg:  ");
      print_state(stdout,x,bb->type,npx);
      printf(" nin=%i\n",nin);
#endif
      if(nin>npx-1 || npout>bb->nptrunc)continue;
      if(!quark_constrained(xx[0])){
	qqgg_emerge(x,nin,&s);
	for(j=0; j<s.length; j++){
	  for(l=0; l<s.np; l++)
	    z[l]=s.s[s.np*j+l];
	  for(l=0; l<npx-nin; l++)
	    z[s.np+l]=x[l+nin];
	  fix_add(y,bb,z,npout);
	}
      }
      if(!quark_constrained(x[0])){
	qqgg_emerge(xx,nin,&s);
	for(j=0; j<s.length; j++){
	  for(l=0; l<s.np; l++)
	    z[l]=s.s[s.np*j+l];
	  for(l=0; l<npx-nin; l++)
	    z[s.np+l]=xx[l+nin];
	  charge_conjugation(z,z,bb->type,npout);
	  fix_add(y,bb,z,npout);
	}
      }
    }
  }
  
  /* qqg Yukawa interactions */
  
  for(nin=1; nin<THREE; nin++){
    npout=npx+THREE-2*nin;
    if(nin>npx-1 || npout>bb->nptrunc)continue;
#if 0        /* debugging print */
    printf("Starting qqg:  ");
    print_state(stdout,x,bb->type,npx);
    printf(" nin=%i\n",nin);
#endif
    if(!quark_constrained(xx[0])){
      qqg_emerge(x,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<s.np; l++)
	  z[l]=s.s[s.np*j+l];
	for(l=0; l<npx-nin; l++)
	  z[s.np+l]=x[l+nin];
      fix_add(y,bb,z,npout);
      }
    }
    if(!quark_constrained(x[0])){
      qqg_emerge(xx,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<s.np; l++)
	  z[l]=s.s[s.np*j+l];
	for(l=0; l<npx-nin; l++)
	  z[s.np+l]=xx[l+nin];
	charge_conjugation(z,z,bb->type,npout);
	fix_add(y,bb,z,npout);
      }
    }
  }

  
  /*  qq mass term   */  
  
#if CONSTRAINED_QUARK
#if 0        /* debugging print */
  printf("Starting qq:  ");
  print_state(stdout,x,bb->type,npx);
  printf("\n");
#endif
  if(!quark_constrained(xx[0])){
    z[0]=x[0]^QUARK_CONSTRAINED_MASK;
    for(l=1; l<npx; l++)z[l]=x[l];
    fix_add(y,bb,z,npx);
  }
  if(!quark_constrained(x[0])){
    z[0]=xx[0]^QUARK_CONSTRAINED_MASK;
    for(l=1; l<npx; l++)z[l]=xx[l];
    charge_conjugation(z,z,bb->type,npx);
    fix_add(y,bb,z,npx);
  }
#endif

  free_row_memory(&s);
}

void row_heavy_light(cbase *y, full_basis *bb, partype *x, int npx){
  int nin,npout;
  int i,j,l;
  partype z[NPMAX];
  partype kt=bb->kt; /* twice K cutoff minus light field total K */
  partype xx[NPMAX];
  interact s=INTERACT_INIT; /* object holding results of interactions */ 
  y->length=0;
  y->length=addstate(y->s,x,y->length,bb->nptrunc);
  if(bb->type!=TYPE_HEAVY_LIGHT){
    fprintf(stderr,__FILE__":  Wrong type of interaction");
    exit(55);
  }

  for(i=0; i<npx-1; i++)kt-=2*(x[i]&KMASK)+(GLUE_PERIODIC?0:1);
  kt-=2*(x[npx-1]&KMASK)+(QUARK_PERIODIC?0:1);

  /* ssqq interactions */

  if(npx==1){ 
#if 0        /* debugging print */
    printf("Starting qqqq:  ");
    print_state(stdout,x,bb->type,npx);
    printf("\n");
#endif
    ssqq_emerge(x,npx,bb->kt,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      fix_add(y,bb,z,s.np);
    }
  }

  /* gggg gauge interactions */

  for(nin=0; nin<FOUR; nin++){
    npout=npx+FOUR-2*nin;
    if(npout>bb->nptrunc)continue;
    for(i=0; i<npx-nin; i++){
#if 0        /* debugging print */
      printf("Starting gggg:");
      print_state(stdout,x+i,bb->type,npx);
      printf(" nin=%i\n",nin);
#endif
      gggg_emerge(x+i,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<i; l++)
	  z[l]=x[l];
	for(l=0; l<s.np; l++)
	  z[l+i]=s.s[s.np*j+l];
	for(l=i; l<npx-nin; l++)
	  z[s.np+l]=x[l+nin];
	fix_add(y,bb,z,npout);
      }
    }
  }
  
  charge_conjugation(xx,x,bb->type,npx);

  /* qqg Yukawa interactions */
  
  for(nin=0; nin<THREE; nin++){
    npout=npx+THREE-2*nin;
    if(nin>npx || npout>bb->nptrunc)continue;
#if 0        /* debugging print */
    printf("Starting qqg:  ");
    print_state(stdout,x,bb->type,npx);
    printf(" nin=%i\n",nin);
#endif
    qqg_emerge(xx,nin,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      for(l=0; l<npx-nin; l++)
	z[s.np+l]=xx[l+nin];
      charge_conjugation(z,z,bb->type,npout);
      fix_add(y,bb,z,npout);
    }
  }

  /*  qqgg Yukawa and gauge interactions    */  

  for(nin=0; nin<FOUR; nin++){
    npout=npx+FOUR-2*nin;
#if 0        /* debugging print */
    printf("Starting qqgg:  ");
    print_state(stdout,x,bb->type,npx);
    printf(" nin=%i\n",nin);
#endif
    if(nin>npx || npout>bb->nptrunc)continue;
    qqgg_emerge(xx,nin,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      for(l=0; l<npx-nin; l++)
	z[s.np+l]=xx[l+nin];
      charge_conjugation(z,z,bb->type,npout);
      fix_add(y,bb,z,npout);
    }
  }

  for(nin=0; nin<=TWO; nin++){
    npout=npx+TWO-2*nin;
    if(nin>npx || npout>bb->nptrunc)continue;
    ssgg_emerge(x,nin,kt,&s);
    for(j=0; j<s.length; j++){
      for(l=0; l<s.np; l++)
	z[l]=s.s[s.np*j+l];
      for(l=0; l<npx-nin; l++)
	z[s.np+l]=x[l+nin];
	fix_add(y,bb,z,npout);
      }
    }

  free_row_memory(&s);
}

#define DEBUG_BARYON 0 /* debugging flags */

void row_baryon(cbase *y, full_basis *bb, partype *x, int npx){
  int leg;
  int nin,npout;
  int i,j,l,perm;
  int markers[NCOLORS+1];
  int pm[NCOLORS+1];
  partype x_perm[NPMAX];
  partype z[NPMAX];
  partype twin[2];  /* hold particles from two legs */
  interact s=INTERACT_INIT; /* object holding results of interactions */ 
  y->length=0;
  y->length=addstate(y->s,x,y->length,bb->nptrunc);
  if(bb->type!=TYPE_BARYON){
    fprintf(stderr,__FILE__":  Wrong type of interaction");
    exit(55);
  }
  find_markers(markers,x,npx);
    
  /************* determinant interaction *********************/

#if GLUE_PERIODIC /* routine assumes glue is periodic */
  {
    int bits;    /* bits switch add-subtract for a given leg */
    partype kill,kill0;
    partype outk[NCOLORS], *poutk;
    int ktotal, kk;
    int out_legs; /* number particles created by determinant */
    int link;  /* position in leg of link */
    partype *px;  /* pointer to beginning of leg */
#if DEBUG_BARYON
    printf("starting det\n");
    print_state(stdout,x,bb->type,npx);
    printf("\n");
#endif
    for(bits=1; bits<(1<<NCOLORS)-1; bits++){
      ktotal=0; out_legs=0; kill0=HOLE;
      for(leg=0; leg<NCOLORS; leg++){
	px=x+markers[leg];
	if(bits&(1<<leg)){
	  link=markers[leg+1]-markers[leg]-2;
	  if(link<1)break; /* no link */
	  kill=px[link];
	  if(out_legs==leg)kill0=kill; /* first time */
	  ktotal += kill&KMASK;  /* assume glue periodic */
	  if(!H_EQUAL(kill,kill0))break; /* helicity ! match */
	} else
	  out_legs++;
      }
      if(leg<NCOLORS)continue;  /* didn't work */
      npout=npx+2*out_legs-NCOLORS;
      if(npout>bb->nptrunc)continue; /* too many particles */
      if(kill0==HOLE){	fprintf(stderr,"kill0 error\n");exit(0);}
      /* now make a list of new momentum */
      for(j=0; j<out_legs; j++)outk[j]=0; 
      do{
	for(j=0, kk=0; j<out_legs-1; j++)kk+=outk[j]; /* glue periodic */
	if(kk<ktotal){
	  outk[out_legs-1]=ktotal-kk;
	  poutk=outk;
	  /* make outgoing state and add to basis */
	  for(leg=0, l=0; leg<NCOLORS; leg++){
	    if(bits&(1<<leg)){
	      for(j=markers[leg]; j<markers[leg+1]-2; j++)
		z[l++]=x[j];
	    }else {
	      for(j=markers[leg]; j<markers[leg+1]-1; j++)
		z[l++]=x[j];
	      z[l++]=(kill0 & ~(KMASK)) + *poutk;
	      poutk++;
	    }
	    if(l<npout)z[l++]=EPSILON;
	  }
	  if(npout!=l){
	    fprintf(stderr,"Wrong number of particles, npout=%i l=%i\n",
		    npout,l);
	    exit(-111);
	  }
	  /* we have a winner */
#if 0
	  printf("    final:");
	  print_state(stdout,z,bb->type,npout);printf("\n");
#endif
	  fix_add(y,bb,z,npout);
	}

	for(j=0; j<out_legs-1; j++){
	  if(outk[j]<ktotal){
	    outk[j]++;
	    break;
	  }
	  outk[j]=0;
	}
      }while(j<out_legs-1);
    }
  }
#else
  fprintf(stderr,"Need determinant\n"); exit(-1);   
#endif  
   
  /********** Interactions between legs **********/
  for(perm=0; perm<NCOLORS*(NCOLORS-1); perm++){
    permute_baryon(perm,markers,x_perm,x,npx);
    find_markers(pm,x_perm,npx);
        
    /* qqqq interactions */
    
    if(pm[1]==2 && pm[2]==4){ 
#if DEBUG_BARYON  /* debugging print */
      printf("Starting qqqq between:  ");
      print_state(stdout,x_perm,bb->type,npx);
      printf("\n");
#endif
      twin[0]=x_perm[0];
      twin[1]=x_perm[2];
      qqqq_emerge(twin,2,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<npx; l++)z[l]=x_perm[l];
	z[0]=s.s[s.np*j];
	z[2]=s.s[s.np*j+1];
	fix_add(y,bb,z,npx);
      }
    }

    /* qqgg interactions */
    
    if(pm[1]==2 && pm[2]-2>pm[1]){ 
#if DEBUG_BARYON  /* debugging print */
      printf("Starting qqgg between:  ");
      print_state(stdout,x_perm,bb->type,npx);
      printf("\n");
#endif
      twin[0]=x_perm[0];
      twin[1]=x_perm[pm[2]-2];
      qqgg_emerge(twin,2,&s);
      for(j=0; j<s.length; j++){
	/* helicities must match */
	if(!H_EQUAL(twin[1],s.s[s.np*j+1]))continue; 
	for(l=0; l<npx; l++)z[l]=x_perm[l];
	z[0]=s.s[s.np*j];
	z[pm[2]-2]=s.s[s.np*j+1];
	fix_add(y,bb,z,npx);
      }
    }

    /* gggg interactions */
    
    if(pm[1]>2 && pm[2]-2>pm[1]){ 
#if DEBUG_BARYON  /* debugging print */
      printf("Starting gggg between:  ");
      print_state(stdout,x_perm,bb->type,npx);
      printf("\n");
#endif
      twin[0]=x_perm[pm[1]-2];
      twin[1]=x_perm[pm[2]-2];
      gggg_emerge(twin,2,&s);
      for(j=0; j<s.length; j++){
	/* helicities must match */
	if(!H_EQUAL(twin[0],s.s[s.np*j]))continue; 
	if(!H_EQUAL(twin[1],s.s[s.np*j+1]))continue; 
	for(l=0; l<npx; l++)z[l]=x_perm[l];
	z[pm[1]-2]=s.s[s.np*j];
	z[pm[2]-2]=s.s[s.np*j+1];
	fix_add(y,bb,z,npx);
      }
    }
  }

  /*********** interactions within one leg ****************/
  /*   cycle through NCOLORS legs:  */

  for(leg=0; leg<NCOLORS; leg++){
    int link=markers[leg+1]-markers[leg];

    /* gggg gauge interactions */
    
    for(nin=1; nin<FOUR; nin++){
      npout=npx+FOUR-2*nin;
      if(nin>link-2 || npout>bb->nptrunc)continue;
      for(i=markers[leg]+1; i<markers[leg+1]-nin; i++){
#if DEBUG_BARYON  /* debugging print */
	printf("Starting gggg leg=%i:",leg);
	print_state(stdout,x,bb->type,npx);
	printf(" nin=%i\n",nin);
#endif
	gggg_emerge(x+i,nin,&s);
	for(j=0; j<s.length; j++){
	  for(l=0; l<i; l++)
	    z[l]=x[l];
	  for(l=0; l<s.np; l++)
	    z[l+i]=s.s[s.np*j+l];
	  for(l=i; l<npx-nin; l++)
	    z[s.np+l]=x[l+nin];
	  fix_add(y,bb,z,npout);
	}
      }
    }
    
    /* qqg Yukawa interactions */
    
    for(nin=1; nin<THREE; nin++){
      npout=npx+THREE-2*nin;
      if(nin>link-1 || npout>bb->nptrunc)continue;
      i=markers[leg];
#if DEBUG_BARYON  /* debugging print */
      printf("Starting qqg leg=%i:  ",leg);
      print_state(stdout,x,bb->type,npx);
      printf(" npout=%i leg=%i\n",npout,leg);
#endif
      qqg_emerge(x+i,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<i; l++)
	  z[l]=x[l];
	for(l=0; l<s.np; l++)
	  z[l+i]=s.s[s.np*j+l];
	for(l=i; l<npx-nin; l++)
	  z[s.np+l]=x[l+nin];
#if 0
	print_state(stdout,z,bb->type,npout);
	printf("\n");
#endif
	fix_add(y,bb,z,npout);
      }
    }
    
    /*  qqgg Yukawa and gauge interactions */  
    
    for(nin=1; nin<FOUR; nin++){
      npout=npx+FOUR-2*nin;
#if DEBUG_BARYON  /* debugging print */
      printf("Starting qqgg leg=%i:  ",leg);
      print_state(stdout,x,bb->type,npx);
      printf(" nin=%i\n",nin);
#endif
      if(nin>link-1 || npout>bb->nptrunc)continue;
      i=markers[leg];
      qqgg_emerge(x+i,nin,&s);
      for(j=0; j<s.length; j++){
	for(l=0; l<i; l++)
	  z[l]=x[l];
	for(l=0; l<s.np; l++)
	  z[l+i]=s.s[s.np*j+l];
	for(l=i; l<npx-nin; l++)
	  z[s.np+l]=x[l+nin];
	fix_add(y,bb,z,npout);
      }
    }
  }  /* perm loop */

  free_row_memory(&s);

}

/* length is the number of states */

void row_memory(interact *y, size_t length){
  size_t temp_length=length*y->np*sizeof(partype);
  if(temp_length>y->s_bytes){
    y->s=realloc(y->s,y->s_bytes=temp_length);
    if(y->s==NULL){
      fprintf(stderr,__FILE__":  Can't allocate memory in row_memory\n");
      exit(11);
    }
  }
  y->length=length;
}

void free_row_memory(interact *s){
  free(s->s);s->s=NULL;
  s->s_bytes=0;
}


