(*
Construct the Hamiltonian


Brett van de Sande
October 1, 2004


The notation used in this notebook corresponds to the notation introduced in \
the LaTeX notes "endpoint.tex" and to Simon's paper, hep-ph/0101318.


In the following, there are implicit integrations over all momenta, implicit \
sums over repeated flavor indices, and implicit sums over all link \
directions.  The b[...] and d[...] are spinors and vectors in color space.  \
The a[...] are matrices in color space.  sigma[...] and zeta[...] are \
matrices in spinor-space.
*)

(************************************************************************

Definitions

************************************************************************)

Off[General::"spell1"]
(*
All symbols are considered to be c-number unless specified otherwise:
*)
cnumberq[x_] := 
  FreeQ[x, sigma[_]] && FreeQ[x, a[__]] && FreeQ[x, b[__]] && 
    FreeQ[x, d[__]] && FreeQ[x, bb[__]] && FreeQ[x, dd[__]] && 
    FreeQ[x, zeta[_]]
freelistq[x_, y_List] := Apply[And, Map[FreeQ[x, #] &, y]]
prodexpand = {(a__) ** b_Plus :> Map[(a ** #) &, b], 
      b_Plus ** (a__) :> Map[(# ** a) &, b],
      _ ** 0 -> 0, 0 ** _ ->0,
      a_ ** b_Times :> Select[b, cnumberq] a ** Select[b, ! cnumberq[#] &], 
      b_Times ** a_ :> Select[b, cnumberq]*Select[b, ! cnumberq[#] &] ** a};
intsumexpand = {a_ ** int[b_, c_] :> int[a ** b, c] /; freelistq[a, c], 
      int[b_, c_] ** a_ :> int[b ** a, c] /; freelistq[a, c],
      a_ ** sum[b_, c_] :> sum[a ** b, c] /; freelistq[a, c], 
      sum[b_, c_] ** a_ :> sum[b ** a, c] /; freelistq[a, c],
      int[a_Times, b_] :> 
        Module[{vars = Select[a, freelistq[#, b] &]}, 
          vars int[Select[a, ! freelistq[#, b] &], b] /; vars =!= 1],
      sum[a_Times, b_] :> 
        Module[{vars = Select[a, freelistq[#, b] &]}, 
          vars sum[Select[a, ! freelistq[#, b] &], b] /; vars =!= 1],
      int[a_Plus, b_] :> Map[int[#, b] &, a],
      sum[a_Plus, b_] :> Map[sum[#, b] &, a],
      int[d_. sum[a_, b_], c_] -> sum[int[d a, c], b],
      int [a_. int[b_, c_], d_] :> 
        int[Expand[a b], Join[c, d]] /; freelistq[a, c],
      sum [a_. sum[b_, c_], d_] :> 
        sum[Expand[a b], Join[c, d]] /; freelistq[a, c],
      int[a_, b_] :> int[Expand[a], b] /; Expand[a] =!= a}; 
prodfactor = {a_. c_NonCommutativeMultiply + b_. c_NonCommutativeMultiply :> 
        Simplify[Factor[Expand[a] + Expand[b]]] c};
sigmazeta = Join[prodexpand, {
        perp[-a_,b_]->perp[a,b],
        perp[a_,-b_]->perp[a,b],
        sigma12[a_,-b_]->-sigma12[a,b],
        sigma12[-a_,b_]->-sigma12[a,b],
        zeta[-a_]->-zeta[a],
        a[b__] ** sigma[c_] -> sigma[c] ** a[b], 
        Conjugate[a[b__]] ** sigma[c_] -> sigma[c] ** Conjugate[a[b]],
        a[b__] ** zeta[c_] -> zeta[c] ** a[b], 
        Conjugate[a[b__]] ** zeta[c_] -> zeta[c] ** Conjugate[a[b]],
        sigma[a_Integer] ** sigma[a_Integer] ** b_ -> b,
        b_ ** sigma[a_Integer] ** sigma[a_Integer] -> b,
        sigma[2] ** zeta[a_] ** sigma[1] -> sigma[1] ** zeta[a] ** sigma[2], 
        sigma[a_Integer] ** sigma[b_Integer] :> 
          Block[{c = First[Complement[{1, 2, 3}, {a, b}]]}, 
              I Signature[{a, b, c}] sigma[c]] /; a != b,
        sigma[b_] ** zeta[a_] ** sigma[3] -> -1*
            sigma[b] ** sigma[3] ** zeta[a], 
        sigma[3] ** zeta[a_] -> -1*zeta[a] ** sigma[3]}];
(*
Relabel the parameters for implicit sums and integrations.  It acts on an \
expression that is linear in NonCommutativeMultiply[...].
*)
relabel[a_Plus] := Map[relabel, a];
relabel[ain_*bin_NonCommutativeMultiply] := 
  Block[{i1 = 1, i2 = 1, j = 1, krules, kcrules, srules, minussrules, 
         ops, cops}, 
    ops = Cases[Apply[List, bin], 
        a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | bb[_Symbol, ___] |
           dd[_Symbol, ___]]; 
    cops = Cases[Apply[List, bin], 
        Conjugate[
          a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | 
            bb[_Symbol, ___] | dd[_Symbol, ___]]];
    (*Print[{ops, cops}];*) 
    krules = 
      Map[(First[#] -> Symbol["kr" <> ToString[i2++]]) &, Reverse[ops]]; 
    kcrules = 
      Map[(First[First[#]] -> Symbol["kl" <> ToString[i1++]]) &, cops]; 
    srules = Map[(# -> Symbol["s" <> ToString[j++]])&, 
        Union[Map[Last, Cases[              
              Apply[List, bin] /. Conjugate[ex_] -> ex, 
              a[_, _Symbol] | zeta[_Symbol] | sigma[_Symbol]
              ]]]];
    minussrules = Map[(-# -> -Symbol["s" <> ToString[j++]])&, 
        Union[Map[Last, Cases[              
              Apply[List, bin] /. Conjugate[ex_] -> ex, 
              a[_, -_Symbol] | zeta[-_Symbol] | sigma[-_Symbol]
              ]]]];
    ain bin /. Join[krules, kcrules] /. minussrules/. srules]
(*
   Fix summation indices so that minus signs a
*)
(*
simplify an integration over x, assuming the variable is in a delta-function. \
 It acts on an expression that is linear in delta[...] and \
NonCommutativeMultiply[...].
*)
simpint[a_Plus, x_] := Map[simpint[#, x] &, a];
simpint[a_ b_, x_] := a simpint[b, x] /; FreeQ[a, x];
simpint[a_  delta[-x_ + c_], x_] := simpint[a delta[x - c], x];
simpint[a_  delta[x_ + c_], x_] := Simplify[a /. x -> -c]
simpint[a_ , x_] := a /; FreeQ[a, x];
Unprotect[Conjugate];
Conjugate[a_Plus] := Map[Conjugate, a];
Conjugate[a_Times] := Map[Conjugate, a];
Conjugate[a_ ** b_] := Conjugate[b] ** Conjugate[a];
Protect[Conjugate];
sigma /: Conjugate[sigma[a_]] = sigma[a];
zeta /: Conjugate[zeta[a_]] := zeta[a];
int /: Conjugate[int[a_, b_]] := 
    int[Conjugate[a] /. Map[(Conjugate[#] -> #) &, b], b];
sum /: Conjugate[sum[a_, b_]] := 
    sum[Conjugate[a] /. Map[(Conjugate[#] -> #) &, b], b];
intconv[x_] := (int[a_, b_] :> 
      int[Integrate[a, x], Complement[b, {x}]] /; MemberQ[b, x])
(*
A variable x is defined to be positive[x]=True if it is *always* positive.
*)
Unprotect[Positive];
Positive[a_ + b_] := False /; ! Positive[a] && ! Positive[b];
Positive[a_?Positive + b_?Positive] = True;
Positive[a_?Negative b_?Positive] = False;
Protect[Positive];
neverzero[a_] := If[Positive[a], True, True, False];

(*
Find terms in the Hamiltonian
*)
(*
This version leaves all of the integrations over momenta and position \
implicit.  Also, sums are implicit.
*)

u[x_] := 1/
        Sqrt[2 Pi] (b[kf] Exp[-I kf x] +
          sigma[1] ** Conjugate[d[kf]] Exp[I kf x]);
conju[x_] =
    Conjugate[u[x]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks;
v[x_] := 1/
        Sqrt[2 Pi] (sigma[1] ** bb[kf] Exp[-I kf x] -
          Conjugate[dd[kf]] Exp[I kf x]);
conjv[x_] =
    Conjugate[v[x]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks;
m[x_, rho_] :=
    1/Sqrt[4 Pi] (a[ka, -rho] Exp[-I ka x] +
            Conjugate[a[ka, rho]] Exp[I ka x])/Sqrt[ka];

(*
This is to define the momenta variables to be *always * positive.
*)
kl1 /: Positive[kl1] = True; kl2 /: Positive[kl2] = True; kl3 /: 
  Positive[kl3] = True;
kl4 /: Positive[kl4] = True; kl5 /: Positive[kl5] = True; kl6 /: 
  Positive[kl6] = True;
kr1 /: Positive[kr1] = True; kr2 /: Positive[kr2] = True; kr3 /: 
  Positive[kr3] = True;
kr4 /: Positive[kr4] = True; kr5 /: Positive[kr5] = True; kr6 /: 
  Positive[kr6] = True;

On[General::"spell1"]

(******************************************************************

Find Fermion two-link interactions

******************************************************************)
(*
Some simplification rules for indices used before break down
in this case, since more than two objects are being summed over

Only do quarks...
Only to large-N_c meson combinations ....
*)

(*
Block[{mm},
  mm[x_,rho_]=m[x,rho]/.ka->kka;
  expr1 =   relabel[Expand[    
    -kappa1/Sqrt[2] (conju[x]**sigma[1]**m[x,rho]**mm[x,rho]**v[x]+
                    conjv[x]**sigma[1]**m[x,rho]**mm[x,rho]**u[x])
    +kappa2/Sqrt[2] (-conju[x]**zeta[rho]**sigma[1]**
                                             m[x,rho]**mm[x,rho]**v[x]
                   + conjv[x]**sigma[1]**zeta[rho]**
                                              m[x,rho]**mm[x,rho]**u[x])
    -kappa3/Sqrt[2](conju[x]**sigma[1]**m[x,rho]**mm[x,-rho]**v[x]+
                    conjv[x]**sigma[1]**m[x,rho]**mm[x,-rho]**u[x])
    -kappa4 perp[lambda,rho]/Sqrt[2](
                 conju[x]**sigma[1]**m[x,rho]**mm[x,lambda]**v[x]+
                  conjv[x]**sigma[1]**m[x,rho]**mm[x,lambda]**u[x])
    -kappa5/Sqrt[2] I sigma12[rho,lambda] (
                 conju[x]**sigma[2]**m[x,rho]**mm[x,lambda]**v[x]-
                  conjv[x]**sigma[2]**m[x,rho]**mm[x,lambda]**u[x])
    +kappa6 perp[lambda,rho]/Sqrt[2] (-conju[x]**zeta[rho]**sigma[1]**
                         (m[x,rho]**mm[x,lambda]+m[x,lambda]**mm[x,rho])**v[x]
                   + conjv[x]**sigma[1]**zeta[rho]**
                       (m[x,rho]**mm[x,lambda]+m[x,lambda]**mm[x,rho])**u[x])  
     /. {d[_]->0,dd[_]->0}//.prodexpand
     /.{a[__]**Conjugate[a[__]]->0}//.prodexpand]]
   //.sigmazeta //. prodfactor
]
*)

(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the interactions in the Hamiltonian.
*)
(*
expr2 = simpint[
    int[expr1, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /.
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
    kl1]
*)
 (*
These are the 2->2 particle quark interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :>
        Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
          0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2] 
*)
(*
These are the 3->1 particle interactions
*)
(*
expr2 /. a_NonCommutativeMultiply :>
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 1->3 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :>
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)

(******************************************************************

Find Fermion interactions using explicit constrained
Terms in the Hamiltonian P^-

Only show quarks!!!!!!!!!!!!!!!!!!!!!!!!

******************************************************************)

(*
expr1 = relabel[Expand[
       -I/2 (-D[conjv[x], x] ** v[x] + conjv[x] ** D[v[x], x]) -
                       muf/Sqrt[2] (conjv[x] ** sigma[1] ** u[x] +
                            conju[x] ** sigma[1] ** v[x]) -
                      kappaa/Sqrt[
                            2] (-conju[x] ** zeta[rho] ** sigma[1] **
                                m[x, rho] ** v[x] +
                            conjv[x] ** sigma[1] ** zeta[rho] ** m[x, rho] **
                              u[x]) +
                      kappas/Sqrt[
                            2] (conju[x] ** sigma[1] ** m[x, sumss] ** v[x] +
                            conjv[x] ** sigma[1] ** m[x, sumss] **
                              u[x]) /.{d[_]->0,dd[_]->0} //.
                  prodexpand 
        ]]//.sigmazeta //. prodfactor
*)

(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the interactions in the Hamiltonian.
*)
(*
expr2 = simpint[
    int[expr1, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /.
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
    kl1]
*)
 (*
These are the 1->1 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :>
        Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
          0 /; Length[non] != 2 || Count[non, Conjugate[_]] != 1] /. {kl1 ->
        k1, kr1 -> k1} /. delta[0] -> 1
*)
(*
These are the 2->1 particle interactions
*)
(*
expr2 /. a_NonCommutativeMultiply :>
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 1->2 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :>
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 2]
*)


(************************************************************************

Find pseudoscalar 4-fermion interaction, acts on pion

************************************************************************)
(*
Terms in the Hamiltonian kappa_5/2 qq**qq
*)
(*
qq[x_] = relabel[
      Expand[ conjv[x] ** sigma[2] ** u[x] +
                conju[x] ** sigma[2] ** v[x] /. {0 ** _ -> 0, _ ** 0 ->
                  0} //. prodexpand //. sigmazeta]] //. prodfactor
*)

(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch =
  qq[x] /. a_NonCommutativeMultiply :>
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
        0 /; Count[non, Conjugate[_]] != Length[non] &&
            Count[non, Conjugate[_]] != 0]
*)
(*
 (*put in the coupling constant later*)expr1pinch =
  relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1,
                kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** qqpinch //.
        prodexpand] //. prodfactor
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch =
  Factor[kappa5/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /.
              Exp[x_] :> Exp[Factor[x]] /.
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /.
          delta[_?neverzero] -> 0, k11]]
*)

(************************************************************************

Find transverse vector 4-fermion interaction, acts on J_z = -1,+1 of a rho

**************************************************************************)
(*
Terms in the Hamiltonian -kappa_6/2 (qq**qq-rr**rr)
*)
(*
qq[x_] = relabel[
      Expand[ conjv[x] ** sigma[3] ** u[x] +
                conju[x] ** sigma[3] ** v[x] /. {0 ** _ -> 0, _ ** 0 ->
                  0} //. prodexpand //. sigmazeta]] //. prodfactor
*)

(*
rr[x_] = relabel[
      Expand[ conjv[x] ** u[x] -
                conju[x] ** v[x] /. {0 ** _ -> 0, _ ** 0 -> 0} //.
            prodexpand //. sigmazeta]] //. prodfactor
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch =
  qq[x] /. a_NonCommutativeMultiply :>
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
        0 /; Count[non, Conjugate[_]] != Length[non] &&
            Count[non, Conjugate[_]] != 0]
*)
(*
rrpinch =
  rr[x] /. a_NonCommutativeMultiply :>
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}},
        0 /; Count[non, Conjugate[_]] != Length[non] &&
            Count[non, Conjugate[_]] != 0]
*)
(*
 (*put in the coupling constant later*)
  expr1pinch =
    relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1,
                    kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) **
              qqpinch - (rrpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3,
                    kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) **
              rrpinch //. prodexpand] //. prodfactor;
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch =
  Factor[-kappa6/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /.
              Exp[x_] :> Exp[Factor[x]] /.
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /.
          delta[_?neverzero] -> 0, k11]]
*)


(*******************************************************************

Find gauge terms

*********************************************************************)
(*
jplus = Expand[
      I m[x, rr] ** D[m[x, -rr] /. ka -> kaa, x] - 
              I D[m[x, rr], 
                    x] ** (m[x, -rr] /. ka -> kaa) - (Conjugate[
                      u[x]] /. {Conjugate[x] -> x, 
                      Conjugate[kf] -> kf}) ** (u[x] /. kf -> kff) //. 
            prodexpand /. 
          a[b_, -lambda_] ** Conjugate[a[c_, -lambda_]] -> 
            a[b, lambda] ** Conjugate[a[c, lambda]] //. prodexpand] //. 
    prodfactor
*)
(*
I don't use Expand[...] because it makes too many terms.
*)
(*
expr3 = g^2/
          4 relabel[(jplus /. {ka -> la, kaa -> laa, rr -> ll, kf -> lf, 
                    kff -> lff}) ** (I Integrate[I Integrate[jplus, x], 
                    x]) //. prodexpand] //. prodfactor;
*)
(*
For the gauge interactions, the evaluation of the color indices is implicit \
and obeys the following rules:
If there are four adjoint fields, the color indices are evaluated as traces:
\!\(TraditionalForm\`\(a\_1\) \(a\_2\) \(a\_3\) a\_4\  -> \ 
    Tr {\(a\_1\) \(a\_2\) \(a\_3\) a\_4} - 
      Tr {\(a\_1\) a\_2}\ Tr {\(a\_3\) a\_4}/N\)
If there are two adjoint fields and two fundamental fields, the color indices \
are evaluated as:
\!\(TraditionalForm\`\(a\_1\) a\_2\ b\_1\ b\_2\  -> \ 
    b\_1\[CenterDot]a\_1\[CenterDot]a\_2\[CenterDot]b\_2\  - 
      Tr {\(a\_1\) a\_2}\ \ b\_1\[CenterDot]b\_2/N\)
If there are four fundamental fields, the color indices are evaluated as:
\!\(TraditionalForm\`\(\(\ \)\(b\ c\ d\ e\  -> \ 
    b\_i\ c\_j\ d\_k\ e\_l\ \((\[Delta]\_il\ \[Delta]\_kj\  - \ \[Delta]\_ij\ \
\[Delta]\_kl/N)\)\)\)\)
*)
(*
expr4 = Distribute[
      int[expr3, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. delta[_?neverzero] -> 0
*)
(*
These are the 3->1 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)

(**********************************************************************

Gluon effective potential

***********************************************************************)
(*
I only include the terms that act on meson strings in the large N limit.
*)

ka /: Positive[ka] = True; kb /: Positive[kb] = True; kc /: 
  Positive[kc] = True; kd /: Positive[kd] = True;

(*
The \!\(TraditionalForm\`\[Micro]\_B\) term
*)
(*
The factor of 1/2 is from the fact that lambda is summing over four values
*)
(*
term = mub^2/2 m[x, s1] ** (m[x, s1] /. ka -> kb);

expr = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] /. 
        aa_a ** Conjugate[b_] -> Conjugate[b] ** aa];

expr2 = Distribute[
      int[expr, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. delta[_?neverzero] -> 0
*)
(*
The box term \[Beta]
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.  There is an overall factor of \
1/4 since there are no sums in the original expression.
*)
(*
term = m[x, sa] ** (m[x, sb] /. ka -> kb) ** Conjugate[m[x, sa] /. ka -> kc] **
       Conjugate[m[x, -sb] /. ka -> kd];

exprb = relabel[
      Expand[-beta/4 (term + Conjugate[term]) //. 
              prodexpand /. {Conjugate[x] -> x, Conjugate[kk_?Positive] -> kk,
               Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];

exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_1\) term
*)
(*
There is an implicit sum over the whole expression for s1={1,2,-1,-2}.  The \
factor of 1/2 is because s1 takes on all four values
*)
(*
term = lambda1/2 m[x, sa] ** 
        Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sa] /. ka -> kc) ** 
        Conjugate[m[x, sa] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_2\) term
*)
(*
There is an implicit sum over the whole expression for s1={1,2,-1,-2}.  The \
factor of 1/2 is because s1 takes on all four values
*)
(*
term = lambda2/2 m[x, sa] ** (m[x, sa] /. ka -> kb) ** 
        Conjugate[m[x, sa] /. ka -> kc] ** Conjugate[m[x, sa] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_4\) term
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.
*)
(*
term = lambda4 m[x, sa] ** 
        Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sb] /. ka -> kc) ** 
        Conjugate[m[x, sb] /. ka -> kd];

exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** bb_ :> 
            c bb ** aa /; MemberQ[bb, Conjugate[_]], 
          c_ aa_ ** bb_Conjugate :> c bb ** aa /; MemberQ[aa, a[__]]}];
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_4\) term (done more explicitly)
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.
*)
(*
term = lambda4 Sum[
        m[x, sa] ** 
          Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sb] /. ka -> kc) ** 
          Conjugate[m[x, sb] /. ka -> kd], {sa, -1, 1, 2}, {sb, -2, 2, 4}];
*)
(*
exprb = relabel[
        Expand[term //. prodexpand /. {Conjugate[x] -> x, 
                Conjugate[kk_?Positive] -> kk, 
                Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** bb_ :> 
              c bb ** aa /; MemberQ[bb, Conjugate[_]], 
            c_ aa_ ** bb_Conjugate :> c bb ** aa /; MemberQ[aa, a[__]]}] //. 
      prodfactor;
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)

