#include <stdio.h>
#include "blocksparse.h"
#include "lapack.h"

#define DEBUG 0 /* put in some checks */

/*
  Given a factorized matrix stored in block-diagnoal format,
  solve a system of linear equations.
*/

int solve_real_block(struct real_block_matrix *a, integer n, 
		     doublereal *b){
  size_v block;
  int bcol;
  integer err;
  real_block *blistp;
  char uplo='U';
  integer one=1;
  doublereal *block_vector;

  block_vector=malloc(a->largest_block_size*sizeof(*block_vector));
  if(block_vector==NULL){err=-123; goto error;}

  for(blistp=a->blist, block=0; block<a->nblocks; blistp++, block++){
      for(bcol=0; bcol<blistp->n; bcol++){
	block_vector[bcol]=b[blistp->row[bcol]];
#if DEBUG
	printf("input for row %i is %g\n",
	       blistp->row[bcol],block_vector[bcol]);
#endif
      }

    /*
      CALL DSPTRS( UPLO, N, NRHS, AP, IPIV, B, LDB, INFO )
    */

    DSPTRS(&uplo,&(blistp->n),&one,blistp->a,blistp->ipiv,block_vector,
	   &blistp->n,&err);
    if(err)goto error;

    for(bcol=0; bcol<blistp->n; bcol++){
      b[blistp->row[bcol]]=block_vector[bcol];
#if DEBUG
      printf("result for row %i is %g\n",
	     blistp->row[bcol],block_vector[bcol]);
#endif
    }
  }

 error:
  free(block_vector);

  return err;
}

 
