#ifndef SPBLOCK_INCLUDED
#define SPBLOCK_INCLUDED

#include <stdlib.h> /* define size_t */
#include "fortran.h"
#include "elslist.h"

/*
  The array size is limited my memory allocation.
  size_v is assumed to be large enough to index such an array
  Individual blocks are small enough to be indexed by type int.
*/

typedef size_t size_v;

/*
  real_block has data types consistent with FORTRAN
   data types.  These are defined in f2c.h or g2c.h
*/

typedef struct {
  integer n;      /* size of block */
  integer *ipiv;  /* vector of pivots, length n */
  doublereal *a;  /* dense matrix containing factorization, 
		     length n*(n+1)/2 */
  size_v *row;    /* vector of rows of original matrix in this block */
} real_block;      

struct real_block_matrix {
  size_v nblocks;
  int largest_block_size;
  real_block *blist;
};

int sparse_to_real_block_matrix(elslist *els, struct real_block_matrix *a);
int allocate_real_block(real_block *b, integer n);
void free_real_block_matrix(struct real_block_matrix *a);
int  solve_real_block(struct real_block_matrix *a, integer n, 
		      doublereal *b);

#endif
