#include <stdio.h>
#include "blocksparse.h"
#include "lapack.h"

/*
  Given a factorized matrix stored in block-diagnoal format,
  solve a system of linear equations.
*/

#define LENGTH 10

int main(){
  int i=0;
  int row,col;
  int err;
  struct real_block_matrix a;
  elslist els;
  doublereal *rhs;
  doublereal *dense;
  int j;

  /* make sparse matrix */

  els.n=6;
  els.j=malloc((els.n+1)*sizeof(*els.i));
  els.i=malloc(LENGTH*sizeof(*els.j));
  els.r=malloc(LENGTH*sizeof(*els.r));
  els.i[i]=3; els.r[i++]=1.3;
  els.i[i]=4; els.r[i++]=0.1;
  els.i[i]=1; els.r[i++]=1.2;
  els.i[i]=5; els.r[i++]=0.5;
  els.i[i]=2; els.r[i++]=1.1;
  els.i[i]=3; els.r[i++]=0.3;
  els.i[i]=4; els.r[i++]=0.1;
  els.i[i]=3; els.r[i++]=0.7;
  els.i[i]=5; els.r[i++]=0.2; 
  els.i[i]=5; els.r[i++]=2.2; 
  els.j[0]=0; els.j[1]=2; els.j[2]=4; els.j[3]=7; els.j[4]=8; els.j[5]=9;
  els.j[els.n]=LENGTH;
  
  rhs=malloc(els.n*sizeof(*rhs));
  rhs[0]=2.0; rhs[1]=1.0; rhs[2]=0.2; rhs[3]=0.3; rhs[4]=0.4;
  rhs[5]=1.5;

  /* reconstruct dense matrix */

  dense=malloc(els.n*els.n*sizeof(*dense));
  for(i=0; i<els.n*els.n; i++)dense[i]=0.0;
  for(row=0; row<els.n; row++)
    for(col=els.j[row]; col<els.j[row+1]; col++){
      dense[row+els.n*els.i[col]]=els.r[col];
      if(row!=els.i[col])dense[row*els.n+els.i[col]]=els.r[col];
    }
  printf("a={{");
  for(row=0; row<els.n; row++){
    for(col=0; col<els.n; col++)
      printf("%s %4g",col?",":"",dense[row+els.n*col]);
    printf("}%s",row<els.n-1?",\n   {":"};\n");
  }
  free(dense);

  err=sparse_to_real_block_matrix(&els,&a);
  if(err)printf("factorize error %i\n",err);

  /* print out block matrix */
  printf("print block matrix: \n");
  for(i=0; i<a.nblocks; i++){
    printf("%i: ",i);
    for(j=0; j<a.blist[i].n && j<els.n; j++)
      printf(" %i",a.blist[i].row[j]);
    printf("\t");
    for(j=0; j<a.blist[i].n*(a.blist[i].n+1)/2 && j<els.n*els.n; j++)
      printf(" %g",a.blist[i].a[j]);
    printf("\n");
  }

  err=solve_real_block(&a,els.n,rhs);
  if(err)printf("solve error %i\n",err);

  printf("x={");
  for(i=0; i<els.n; i++)
    printf("%s %g",i?",":"",rhs[i]);
  printf("};\n");

  /* deallocate all memory */

  free_real_block_matrix(&a);
  free(rhs);
  free(els.j);
  free(els.i);
  free(els.r);

  return 0;
}

 
