C
C @(#)landr.f	3.16 (BNP) 6/3/89; from landr.f 2.16 6/7/88
C
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pLANDR(N,LANMAX,MAXPRS,CONDM,ENDL,ENDR,EV,KAPPA,
     *   J,NEIG,RITZ,BND,Q,IERR,MSGLVL,MPICOM)
#else
      SUBROUTINE LANDR(N,LANMAX,MAXPRS,CONDM,ENDL,ENDR,EV,KAPPA,
     *   J,NEIG,RITZ,BND,Q,IERR,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LANMAX,MAXPRS,EV,J,NEIG,IERR,MSGLVL
      DOUBLE PRECISION CONDM,ENDL,ENDR,KAPPA,
     *   RITZ(LANMAX),BND(LANMAX),Q(N,LANMAX)
C
C.... The program make a Lanczos run using a linear operator that acts
C.... through a user supplied subroutine called OP in order to compute
C.... either all eigenvalues outside an interval [endl,endr] or the
C.... first MAXPRS eigenvalues, whichever occurs first.  The inner
C.... product is defined implicitly by a user supplied subroutine OPM.
C
C            *****************************************
C            *                                       *
C            *         LANCZOS ALORITHM WITH         *
C            *      SELECTIVE ORTHOGONALIZATION      *
C            *            L  A  N  S  O              *
C            *      (Using Simon's Recurrence)       *
C            *                                       *
C            *****************************************
C
C.... inputs
C.... N      dimension of the eigenproblem
C.... LANMAX upper limit to the number of Lanczos steps
C.... MAXPRS upper limit to the number of wanted eigenpairs
C.... CONDM  estimated effective condition number of M
C.... ENDL   left end of the interval containing the UNwanted eigenvalues
C.... ENDR   right end of the interval containing the UNwanted eigenvalues
C.... EV     .LE.0 means eigenvalues only,
C....        .GT.0 means both eigenvalues and eigenvectors are wanted
C....              and EV becomes the output channel for eigenvectors
C.... KAPPA  relative accuracy of Ritz values acceptable as eigenvalues
C....        (KAPPA will not be touched and/or looked at if EV.LE.0)
C.... Q      The first N elements contain either ZERO
C....        or an initial guess for the first Lanczos vector
C
C.... workspace
C.... Q      Hold all lanczos vectors, length N*LANMAX
C
C.... outputs
C.... J      number of Lanczos steps actually taken
C.... NEIG   number of Ritz values stabilized
C.... RITZ   array to hold the Ritz values
C.... BND    array to hold the error bounds
C.... IERR   error flag
C
C.... subroutines:      MACHAR,LANSO,RITVEC
C
      DOUBLE PRECISION ALF(LANMAX),BET(LANMAX+1)
      INTEGER NTOTAL
C
#ifdef USE_MPI
      INTEGER MPICOM
      INCLUDE 'mpif.h'
      CALL MPI_ALLREDUCE(N,NTOTAL,1,MPI_INTEGER,MPI_SUM,MPICOM,IERR)
      if (ierr .ne. MPI_SUCCESS) call MPI_ABORT(mpicom, ierr)
#else
      NTOTAL=N
#endif
C
C.... check input data
C
      IERR = 0
      IF (NTOTAL.LE.0) IERR = IERR+1
      IF (LANMAX.LE.0) IERR = IERR+2
      IF (ENDR.LE.ENDL) IERR = IERR+4
      IF (MAXPRS.LE.0) IERR = IERR+8
      IF (MAXPRS.GT.LANMAX) IERR = IERR+16
      IF (LANMAX.GT.NTOTAL) IERR = IERR+32
      IF (IERR.GT.0) RETURN
C
C.... compute error constants and fix kappa
C
      CALL FINDEPSILON(NTOTAL,CONDM,KAPPA)
C
#ifdef USE_MPI
      CALL pLANSO(N,LANMAX,MAXPRS,ENDL,ENDR,J,NEIG,RITZ,BND,
     *     ALF,BET,Q,IERR,MSGLVL,MPICOM)
#else
      CALL LANSO(N,LANMAX,MAXPRS,ENDL,ENDR,J,NEIG,RITZ,BND,
     *     ALF,BET,Q,IERR,MSGLVL)
#endif
C
C.... compute eigenvectors
C
      IF (EV.GT.0) THEN
         CALL RITVEC(N,J,MAXPRS,KAPPA,RITZ,BND,
     *      ALF,BET,Q,IERR,MSGLVL)
      ENDIF
      RETURN
      END




