C @(#)plandr2.f	7/31/97; from landr.f 3.16 (BNP) 6/3/89
C
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pLANDR2(N,LANMAX,MAXPRS,LOHI,CONDM,KAPPA,
     *   J,NEIG,RITZ,BND,Q,IERR,MSGLVL,MPICOM)
#else
      SUBROUTINE LANDR2(N,LANMAX,MAXPRS,LOHI,CONDM,KAPPA,
     *   J,NEIG,RITZ,BND,Q,IERR,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LANMAX,MAXPRS,LOHI,J,NEIG,IERR,MSGLVL
      DOUBLE PRECISION CONDM,KAPPA,
     *   RITZ(LANMAX),BND(LANMAX),Q(N,LANMAX)
C
c     this modified version is intended to compute a number of
c     extreme eigenvalues of given OPerator.
c     The alpha and beta arrays are also copied to the beginning
c     of the workspace (W) before returning.
c
C.... The program make a Lanczos run using a linear operator that acts
C.... through a user supplied subroutine called OP in order to compute
C.... first MAXPRS smallest or largest eigenvalues or till maximum
C.... Lanczos steps are taken.  The inner product is defined implicitly
C.... by a user supplied subroutine OPM.
C
C            *****************************************
C            *                                       *
C            *         LANCZOS ALORITHM WITH         *
C            *      SELECTIVE ORTHOGONALIZATION      *
C            *            L  A  N  S  O              *
C            *      (Using Simon's Recurrence)       *
C            *                                       *
C            *****************************************
C
C.... inputs
C.... N      (local) dimension of the eigenproblem
C.... LANMAX upper limit to the number of Lanczos steps
C.... MAXPRS upper limit to the number of wanted eigenpairs
C.... LOHI   which end of spectrum to compute
C....        .LE.0 lower end of the spectrum, otherwise the high end
C.... CONDM  estimated effective condition number of M
C.... KAPPA  relative accuracy of Ritz values acceptable as eigenvalues
C.... Q      The first N elements contain either ZERO
C....        or an initial guess for the first Lanczos vector
C
C.... workspace
C.... Q      Hold all lanczos vectors, length N*LANMAX
C
C.... outputs
C.... J      number of Lanczos steps actually taken
C.... NEIG   number of Ritz values stabilized
C.... RITZ   array to hold the Ritz values
C.... BND    array to hold the error bounds
C.... IERR   error flag
C
C.... NOTE: the j-1 elements produced from the Lanczos algorithm are
C.... stored as beta(2:j) NOT beta(1:j-1) ******
C
C.... subroutines:      MACHAR,LANSO,RITVEC
C
      DOUBLE PRECISION ALF(LANMAX),BET(LANMAX+1)
      INTEGER NTOTAL
C
C
#ifdef USE_MPI
      INTEGER MPICOM
      INCLUDE 'mpif.h'      
#endif
C
C.... check input data
C
#ifdef USE_MPI
      CALL MPI_ALLREDUCE(N,NTOTAL,1,MPI_INTEGER,MPI_SUM,MPICOM,IERR)
      if (ierr .ne. MPI_SUCCESS) call MPI_ABORT(mpicom, ierr)
#else
C  NTOTAL is N in the single processor case.
      NTOTAL = N
      IERR = 0
#endif
      IF (NTOTAL.LE.0) IERR = IERR+1
      IF (LANMAX.GT.NTOTAL) IERR = IERR+32
      IF (LANMAX.LE.0) IERR = IERR+2
      IF (MAXPRS.LE.0) IERR = IERR+8
      IF (MAXPRS.GT.LANMAX) IERR = IERR+16
      IF (IERR.GT.0) RETURN
C
C.... compute error constants and fix kappa
C
      CALL FINDEPSILON(NTOTAL,CONDM,KAPPA)
C
#ifdef USE_MPI
      CALL pLANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     *           ALF,BET,Q,IERR,MSGLVL,MPICOM)
#else
      CALL LANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     *           ALF,BET,Q,IERR,MSGLVL)
#endif
C
      RETURN
      END







