C
C
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pLANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     &   ALF,BET,Q,IERR,MSGLVL,MPICOM)
#else
      SUBROUTINE LANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     &   ALF,BET,Q,IERR,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LANMAX,MAXPRS,LOHI,J,NEIG,IERR,MSGLVL
      DOUBLE PRECISION KAPPA,Q(N,LANMAX),
     &  ALF(LANMAX),BET(LANMAX+1),RITZ(LANMAX),BND(LANMAX)
C
c     This is modified from planso to compute a few extreme eigenvalues
c     of a given operator.  An eigenpair is declared converged if
c     error bound < kappa * max(|Ritz value|, eps^2/3)
c
C.... inputs
C.... N      dimension of the eigenproblem
C.... LANMAX upper limit to the number of Lanczos steps
C.... MAXPRS upper limit to the number of wanted eigenpairs
C.... LOHI   <=0 compute smallest eigenvalue, >0 compute
C....        largest eigenvalues
C.... KAPPA  Relative tolerance on error of Ritz values
C.... Q      The first N elements contain either ZERO
C....        or an initial guess for the first Lanczos vector
C
C.... work space
C.... Q      Array to hold the Lanczos vectors.
C.... ALF    array to hold diagonal of the tridiagonal T
C.... BET    array to hold off-diagonal of T
C
C.... outputs
C.... J      actual number of Lanczos steps taken
C.... NEIG   number of converged Ritz-pairs
C.... RITZ   array to hold the computed Ritz values
C.... BND    array to hold the error bounds
C.... IERR   error flag
C
C.... BLAS routines:    DATX,DAXPY,DCOPY,DDOT,DSCAL
C.... subroutines:      TQLB,ORTBND,PURGE,STARTV,STPONE,
C....                   RITZSORT,LANCONV,DDOTMPI,pSTPONE
C.... user-supplied:    OP,OPM
C
      DOUBLE PRECISION R(N),RA(N),QA(N),WRK(LANMAX),WRK2(2*LANMAX)
      DOUBLE PRECISION ETA(LANMAX),OLDETA(LANMAX) 
C
      DOUBLE PRECISION ZERO,ONE,FOUR
      PARAMETER (ZERO=0.0D0, ONE=1.0D0, FOUR=4.0D0)
C
      LOGICAL ENOUGH
      INTEGER LL,I,L,FIRST,LAST,ID2,ID3
      INTEGER NTOTAL
      DOUBLE PRECISION T
C
C.... common block for machine epsilon and related variables
C
      INCLUDE 'rdata.f'
C
      INTEGER MYRANK
#ifdef USE_MPI
      INTEGER MPICOM
      INCLUDE 'mpif.h'
      DOUBLE PRECISION pSTARTV, DDOTMPI
#else
      DOUBLE PRECISION STARTV, DDOT
#endif
      J = 1
      CALL STPONE(N,ALF,R,RA,Q,QA,MSGLVL)
C     
#ifdef USE_MPI
      CALL MPI_Comm_rank(MPICOM, MYRANK, IERR)
      CALL MPI_ALLREDUCE(N,NTOTAL,1,MPI_INTEGER,MPI_SUM,MPICOM,IERR)      
#else
C.... MYRANK is a dummy variable for the non-MPI case
      MYRANK=0
      NTOTAL=N
#endif
      IF (IERR.NE.0) THEN
         PRINT *,' error in LANSO2 (ierr =',IERR,'), maybe mpi-related'
      ENDIF
 
      IF (NTOTAL.EQ.1) THEN
         NEIG = 1
         RITZ(1) = ALF(1)
         BND(1) = ZERO
      ENDIF
      ETA(1) = EPS1
      OLDETA(1) = EPS1
C
      LL = 0
      FIRST = 2
      LAST = MIN(MAXPRS+MAX(8,MAXPRS),LANMAX)
      ENOUGH = .FALSE.
 100  IF (FIRST.LE.LANMAX .AND. .NOT. ENOUGH) THEN
C
C....    lanczos loop
C
         DO J = FIRST,LAST
            BET(J) = RNM
C
C...        Restart if invariant subspace is found
C 
            IF (BET(J).EQ.ZERO) THEN
               RNM = STARTV(N,J,R,RA,Q,EPS,MSGLVL)
               ENOUGH = RNM.EQ.ZERO
               IF (ENOUGH) EXIT
            ENDIF
C
C....       take a Lanczos step
C
            T = ONE/RNM
            CALL DSCAL(N,T,R,1)
            CALL DSCAL(N,T,RA,1)
            CALL DCOPY(N,RA,1,QA,1)
            CALL DCOPY(N,R,1,Q(1,J),1)
C
C....       check orthogonality
C
            IF(MSGLVL.GT.5)CALL CHECKORTHO(N,J,Q,QA,MSGLVL)
C
C.... at this point, the variables contain:
C
C    R(NQ(2))  q_{j-1}
C    R(NQ(4))  M.q_{j-1}
C    R(NQ(1))  q_j  
C    R(NQ(3))  M.q_j
C    RNM       beta_{j-1}
C    BET(J)    beta_{j-1} before any restart...
C
            CALL OP_(N,QA,Q(1,J),R)
C
            T = DDOT(N,QA,1,R,1)
            CALL DAXPY(N,-T,Q(1,J),1,R,1)
            ALF(J) = T
C
C    ALF(J)    alpha_j = (M.q_j)^\dag p
C    R         p - alpha_j q_j
C
C BvdS .. moved this down to here
C
            IF (BET(J).GT.ZERO) CALL DAXPY(N,-RNM,Q(1,J-1),1,R,1)
C
C    R         r_j = p -  beta_{j-1} q_{j-1} - alpha_j q_j
C
C.... find the norm of r_j and estimate errors
C
            CALL OPM_(N,R,RA)
C
C....               this should be real if M is Hermetian
C
            RNM = SQRT(DDOT(N,RA,1,R,1))
C 
C
C$$$ Rasmus munk Larsen <rmunk@daimi.aau.dk> observed that ANORM is
C$$$ computed based on local estimate rather than global estimate
C$$$ The following is the suggested change -- Aug. 1998
            ANORM = MAX(BET(J)+ABS(ALF(J))+RNM, ANORM)
            TOL = EPSN*ANORM
C
C....       update the orthogonality bounds
C
            CALL ORTBND(J,ALF,BET,ETA,OLDETA)
C
C....       restore the orthogonality state when needed
C           this modifies r_j and q_j to obtain orthogonality
C           against previous vectors.  
C
C   This routine also updates RNM, returning zero if orthogonalization
C   fails.
C
#ifdef USE_MPI
            CALL pPURGE(N,LL,J,ALF(J),BET(J),R,RA,Q,QA,
     &           ETA,OLDETA,MSGLVL,MPICOM)
#else 
            CALL PURGE(N,LL,J,ALF(J),BET(J),R,RA,Q,QA,
     &           ETA,OLDETA,MSGLVL)
#endif
C     
C     R(NQ(4))  M.r_j       
C     
         ENDDO
C     
C.... end of main loop
C
         J = LAST
15       IF (ENOUGH) J = J-1
         FIRST = J+1
         BET(J+1) = RNM
C
C....    Now analyze T
C
         L = 1
         DO ID2 = 1,J
            IF (L.GT.J) EXIT
            DO I = L,J
               IF (BET(I+1).EQ.ZERO) EXIT
            ENDDO
            I = J
C....       Now i is at the end of an unreduced submatrix
C
            CALL DCOPY(I-L+1,ALF(L),1,RITZ(L),1)
            IF (I.GT.L) CALL DCOPY(I-L,BET(L+1),1,WRK(L),1)
            CALL DSTQRB(I-L+1,RITZ(L),WRK(L),BND(L),WRK2,IERR)
            IF (IERR.NE.0) THEN
               PRINT *,' DSTQRB failed to converge (ierr =',IERR,')'
               PRINT *,' L =',L,' I =',I
               PRINT *,(ID3,RITZ(ID3),WRK(ID3),BND(ID3),ID3 = L,I)
            ENDIF
            DO ID3 = L,I
               BND(ID3) = RNM*ABS(BND(ID3))
            ENDDO
            L = I+1
         ENDDO
C
C....    Sort eigenvalues so the wanted ones are in the front
C
         CALL SORTRITZ(LOHI,J,RITZ,BND)
C
C  BvdS:   On some machines, different processes return slightly 
C          different answers for laconv(), presumably due to round-off 
C          errors.  Changed so that this is computed on one processor and
C          the results are shared with the rest.
C
C
         IF(MYRANK.EQ.0) THEN
            call lanconv(j, ritz, bnd, maxprs, eps, kappa, neig)
         END IF
#ifdef USE_MPI
         CALL MPI_BCAST(neig, 1, MPI_INTEGER, 0, mpicom, ierr)
#endif
         IF (MSGLVL.GT.5.AND.MYRANK.eq.0) then
            PRINT *, 'LANSO: STEP ', j, ';  NEIG ', neig,
     &           '   RES ', BND(1)
            if (msglvl.gt.10) then
               do i = 1, min(msglvl, j)
                  print *, i, ritz(i), bnd(i)
               ENDDO
            endif
         ENDIF
         enough = (neig.ge.maxprs)
C
C....    Should we stop?
C
         IF (.NOT.ENOUGH) THEN
            IF (NEIG.EQ.0) THEN
               LAST = FIRST + MAX(8, MAXPRS)
            ELSE
               LAST = FIRST + MIN(128, MAX(MAXPRS-NEIG,
     &              NINT(0.25*J/NEIG)))
            ENDIF
            IF (3+LAST .ge. LANMAX) LAST = LANMAX
         ELSE
            ENOUGH = .TRUE.
         ENDIF
         ENOUGH = ENOUGH.OR.FIRST.GT.LANMAX
         GOTO 100
      ENDIF
      RETURN
      END
c-----------------------------------------------------------------------
      subroutine sortritz(lohi,j,ritz,bnd)
      implicit none
      integer i, j, lohi
      real*8 ritz(j), bnd(j)
c
      if (lohi.le.0) then
         call dsort2(j,ritz,bnd)
      else
         do 10 i = 1, j
            ritz(i) = -ritz(i)
 10      continue
         call dsort2(j,ritz,bnd)
         do 20 i = 1, j
            ritz(i) = -ritz(i)
 20      continue
      endif
      return
      end
c-----------------------------------------------------------------------
      subroutine lanconv(j, ritz, bnd, maxeig, eps, kappa, neig)
      implicit none
      integer j, neig, i, maxeig
      real*8 ritz(j), bnd(j), eps, kappa, gap, gapl, eps23
c
c     an eigenpair is declared converged if
c     bnd(i) < kappa*(eps^2/3, |ritz(i)|)
c
      neig = 0
      eps23 = exp(log(eps)*0.66666667D0)
      gapl = abs(ritz(j) - ritz(1))
      do 10 i = 1, min(j-1, maxeig)
         gap = gapl
         gapl = abs(ritz(i+1) - ritz(i))
         gap = min(gap, gapl)
         if (gap.gt.bnd(i)) bnd(i) = bnd(i)*(bnd(i)/gap)
         if (bnd(i) .le. kappa*max(eps23, abs(ritz(i)))) neig = neig+1
 10   continue
      return
      end
