C
C @(#)purge.f	3.16 (BNP) 5/11/89; from purge.f 2.13 6/25/88
C
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pPURGE(N,LL,J,ALPHA,BETA,R,RA,Q,QA,ETA,OLDETA,
     *     MSGLVL,MPICOM)
#else
      SUBROUTINE PURGE(N,LL,J,ALPHA,BETA,R,RA,Q,QA,ETA,OLDETA,
     *     MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LL,J,MSGLVL
      DOUBLE PRECISION ETA(J),OLDETA(J),ALPHA,BETA
      DOUBLE PRECISION R(N),Q(N,J),RA(N),QA(N)
C
C.... routine to perform a complete or local reorthogonalization
C
C.... This routine examines ETA to decide whether
C.... re-orthogonalization should be performed.
C
C....        RNM is modified by this routine, returning zero if 
C            reorthogonalization fails
C.... ALPHA  alpha_j which is shifted upon reorthogonalization
C.... BETA   beta_{j-1}
C.... N      dimension of the eigenproblem
C.... LL     no. of initial Lanczos vectors in local orthog.
C.... J      current Lanczos step
C.... R      the residual vector to become the next Lanczos vector
C.... Q      matrix of lanczos vectors
C.... RA     the product of the mass matrix and r
C.... QA     the product of the mass matrix and q
C.... WRK    a temporary vector to hold the previous Lanczos vectors
C.... ETA    state of orthogonality between r and previous Lanczos vectors
C.... OLDETA state of orthogonality between q and previous Lanczos vectors
C.... MPICOM MPI Communicator
C
C.... BLAS routines:    DAXPY,DDOT,IDAMAX
C.... subroutines:      DDOTMPI
C.... user-supplied:    OPM,STORE

      INTEGER K,I,LOOP,IDAMAX
      DOUBLE PRECISION T,TQ,TR,ETAJ
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO = 0.0D0)
      INTEGER MY_RANK      
C
C.... common block for machine epsilon and related variables
C
      INCLUDE 'rdata.f'
C
C
#ifdef USE_MPI
      INTEGER MPICOM
      DOUBLE PRECISION DDOTMPI      
      INCLUDE 'mpif.h'
#else
      DOUBLE PRECISION DDOT      
#endif
C  in the non-parallel case, MY_RANK is a dummy variable
      MY_RANK=0      
C
C    don't do anything the first time after a restart.
C
      IF (J.LE.LL+1) RETURN
C
C    criterion for reorthogonalization used by TRLan (Wu and Simon)
C
      ETAJ=SQRT(ALPHA*ALPHA+BETA*BETA)
C
C  global reorthogonalization based on omega-recurrance
C
      K = IDAMAX(J-(LL+1),ETA(LL+1),1)+LL
C
C  Two possible criteria for global reorthogonalization:
C  lanso.F (omega-recurrence): ABS(ETA(K)).GT.REPS
C       In this case, we need Q to be reorthogonalized
C  Wu & Simon paper and trlan: ETAJ.GT.RNM
C  
C  For the transverse lattice, ETAJ.GT.RNM is almost always true
C
      IF (ABS(ETA(K)).GT.REPS.OR.ETAJ.GT.RNM) THEN
         IF (MSGLVL.GT.5) THEN
#ifdef USE_MPI
            CALL MPI_COMM_RANK(MPICOM, MY_RANK, I)
#endif
            IF (MY_RANK.EQ.0)
     *           PRINT 33,J,ABS(ETA(K)),REPS,ETAJ,RNM
 33         FORMAT ('    global reorthog. J=',I3,' ETA=',
     *           D10.4,' REPS=',D10.4,', ETAJ=',D10.4,' RNM=',D10.4)
         ENDIF
C  Up to four trips.
         DO LOOP = 1,4
C
C     RNM is hopelessly small (go find a new random vector)
C     Using criterion in Wu and Simon's paper, pp. 14 & 18
C
            IF (RNM.GT.EPS1*ETAJ) THEN
C
C....       Bring in a Lanczos vector t and orthogonalize both r and q
C....       against it
C
               TQ = ZERO
               TR = ZERO
               DO I = 1,J-1
C
C     This should only make a small improvement
C     Only bother doing once.
C
C     This may not be necessary if many global reorthogonalizations
C     are perfomed.
C
#define ORTHO_Q 1 /* also orthogonalize Q against previous vectors */
#if ORTHO_Q 
                  IF(LOOP.EQ.1)THEN
                     T = DDOT(N,Q(1,I),1,QA,1)
                     TQ = TQ + T*T
                     CALL DAXPY(N,-T,Q(1,I),1,Q(1,J),1)
                  ENDIF
#endif
C
                  T = DDOT(N,Q(1,I),1,RA,1)
                  TR = TR + T*T
                  CALL DAXPY(N,-T,Q(1,I),1,R,1)
               ENDDO
C
C....          restore local orthogonality
C
               T = DDOT(N,Q(1,J),1,RA,1)
               TR = TR +T*T
               CALL DAXPY(N,-T,Q(1,J),1,R,1)
               ALPHA = ALPHA + T
C
#if ORTHO_Q  /* only do this if Q is orthogonalized above */
               IF(LOOP.EQ.1)CALL OPM_(N,Q(1,J),QA)
#endif
               CALL OPM_(N,R,RA)
               RNM = SQRT(DDOT(N,RA,1,R,1))
C See Wu & Simon, p. 15, and TRLan code
C (this was SQRT(TR).LE.EPS1/REPS*RNM in lanso.f)
               IF (SQRT(TR).LE.RNM) GOTO 58
               IF (MSGLVL.GT.5.AND.MY_RANK.EQ.0)
     &              PRINT *, '     re-orthog J=',J,' error=',SQRT(TR)
            ENDIF
         ENDDO
         RNM=ZERO
 58      CONTINUE
         DO I = LL+1,J
            ETA(I) = EPS1
            OLDETA(I) = EPS1
         ENDDO
      ELSE
C     
C.... perform local orthogonalization
C     Wu and Simonm, p. 15 suggest this may not be needed
C     
#if 1
         T = DDOT(N,Q(1,J-1),1,RA,1)
         CALL DAXPY(N,-T,Q(1,J-1),1,R,1)
C     
         T = DDOT(N,Q(1,J),1,RA,1)
         CALL DAXPY(N,-T,Q(1,J),1,R,1)
         ALPHA = ALPHA + T
C     
         CALL OPM_(N,R,RA)
C.... this should be real if M is Hermetian
         RNM = SQRT(DDOT(N,RA,1,R,1))
C  see Wu & Simon, p. 17
         DO I = J-1,J
            ETA(I) = EPS1
            OLDETA(I) = EPS1
         ENDDO
#endif
      ENDIF
C     
C  If RNM is hoplessly small, then give up
C
      IF(RNM.LT.EPS1*ETAJ)RNM = ZERO
      RETURN
      END
C
C********************************************************
C
C  Diagnostic subroutine to check orthogonality of 
C  the Lanczos vectors
C
C
#ifdef USE_MPI
      SUBROUTINE pCHECKORTHO(N,J,Q,QA,MSGLVL,MPICOM)
#else
      SUBROUTINE CHECKORTHO(N,J,Q,QA,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,J,MSGLVL
      DOUBLE PRECISION Q(N,J),QA(N)
C
C.... common block for machine epsilon and related variables
C
      INCLUDE 'rdata.f'
C
#ifdef USE_MPI
      INTEGER MPICOM
      DOUBLE PRECISION DDOTMPI      
      INCLUDE 'mpif.h'
#else
      DOUBLE PRECISION DDOT      
#endif
C
      INTEGER I,MAXI
      DOUBLE PRECISION MAXT,T
      DOUBLE PRECISION ZERO,ONE
      PARAMETER (ZERO=0.0D0,ONE=1.0D0)
C
      MAXI = 0
      MAXT = ZERO
      DO I = 1,J-1
         T = DDOT(N,QA,1,Q(1,I),1)
         IF(ABS(T).GT.MAXT)THEN
            MAXT = T
            MAXI = I
         ENDIF
      ENDDO
C....    Wu & Simon, p. 9, this is "partial orthogonality" 
      IF(MAXT.GT.REPS)THEN
         PRINT *,'   loss of orthognonality for J=',J,',',MAXI,
     *        ': ',MAXT
      ENDIF
C
C   check QA is correct
C
      T = DDOT(N,QA,1,Q(1,J),1)
      IF(ABS(T-ONE).GT.EPS1)THEN
         PRINT *,'   inner=',T
      ENDIF
C
      RETURN
      END



