C
C @(#)startv.f	1.10 (BNP) 10/30/90
C
C
C
#include "mpi_functions.h"
#ifdef USE_MPI
      DOUBLE PRECISION FUNCTION pSTARTV(N,J,R,RA,Q,EPS,MSGLVL,
     *     MPICOM)
#else
      DOUBLE PRECISION FUNCTION STARTV(N,J,R,RA,Q,EPS,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,J,MSGLVL
      DOUBLE PRECISION R(N),RA(N),Q(N,J),EPS
C
C.... This routine delivers a starting vector in R and returns |R|;
C.... it returns ZERO if range is spanned or if no starting vector
C.... within range of operator can be found.
C
C.... N      dimension of the eigenproblem
C.... J      starting index for a Lanczos run
C.... R      an array containing new Lanczos vector
C.... RA     M.R array
C.... Q      array containing previous Lanczos vectors
C
C.... BLAS routines:    DAXPY, DDOT, DSCAL
C.... subroutines:      RANDOM, DDOTMPI
C.... user-supplied:    OP, OPM, STORE
C
      INTEGER RETRQ, DOSCAL
      PARAMETER (RETRQ = 2)
C
      INTEGER IRAND,I,ID,LOOP
      DOUBLE PRECISION RNM2,T,RANDOM
      INTEGER MY_RANK

#ifdef USE_MPI
      INTEGER MPICOM, IERR
      DOUBLE PRECISION DDOTMPI
      INCLUDE 'mpif.h'
#else
      DOUBLE PRECISION DDOT
#endif
C
      DOUBLE PRECISION ZERO,ONE
      PARAMETER (ZERO=0.0D0,ONE=1.0D0)
C
C  in the non-parallel case, MY_RANK is a dummy variable
      MY_RANK=0
      IF (MSGLVL.GT.5) THEN
#ifdef USE_MPI
         call mpi_comm_rank(mpicom, MY_RANK , ierr)
#endif
         IF(MY_RANK.EQ.0)PRINT *,'STARTV: generate random vector, J=',J
      ENDIF
C
#ifdef USE_MPI
      DATA IRAND/0/
C
C.... Initialize the sed to be used to generate random numbers
C.... To limit the number of times MPI_COMM_RANK is called, IRAND is
C.... saved across calls to this routine
C
      IF (IRAND.EQ.0) THEN
         CALL MPI_COMM_RANK(MPI_COMM_WORLD, IRAND, I)
         IF (I .NE. MPI_SUCCESS) STOP 'MPI_COMM_RANK FAILED'
         IRAND = 918272+(J+IRAND)*7219
      ENDIF
#else
      IRAND = 918272+J
#endif
C
C.... get initial vector, default is random
C
      RNM2 = DDOT(N,R,1,R,1)
      DO ID = 1,3
         IF (ID.GT.1.OR.J.GT.1.OR. (.not. RNM2.GT.EPS)) THEN
            DO 20 I = 1,N
               R(I) = RANDOM(IRAND)-0.5D0
20          CONTINUE
         ENDIF
C
C     In the MPI version of Wu and Simon, there is only one call to
C     OPM with the following explanation:
C.... put input vector in the range of M, do not multiply with K
C.... better for restarting and allow zero (0) eigenvalue to be computed
C
C   BvdS  I assume that this improvement is also appropriate for the
C         single processor version.
C
         CALL OPM_(N,R,RA)
         RNM2 = DDOT(N,RA,1,R,1)
         IF (.NOT. RNM2.GT.ZERO) THEN
            EXIT
         ELSE
            if (RNM2.lt.1.0D-8 .or. RNM2.gt.1.0D8) then
               DOSCAL = 1
               RNM2 = ONE / SQRT(RNM2)
               call dscal(N,RNM2,RA,1)
            else
               DOSCAL = 0
            endif
         ENDIF
         CALL OPM_(N,RA,R)
         if (DOSCAL .eq. 1) then
            RNM2 = DDOT(N,RA,1,R,1)
            RNM2 = ONE / SQRT(RNM2)
            call dscal(N,RNM2,RA,1)
         endif
         CALL OPM_(N,R,RA)
         RNM2 = DDOT(N,RA,1,R,1)
         IF (J.GT.1) THEN
            DO LOOP = 1,2
               DO I = 1,J-1
                  T = -DDOT(N,Q(1,I),1,RA,1)
                  CALL DAXPY(N,T,Q(1,I),1,R,1)
               ENDDO
               CALL OPM_(N,R,RA)
               T = DDOT(N,RA,1,R,1)
               IF (T.LE.EPS*RNM2) THEN
                  T = ZERO
                  EXIT
               ENDIF
            ENDDO
           RNM2 = T
         ENDIF
         IF (RNM2.GT.ZERO) EXIT
      ENDDO
#ifdef USE_MPI
      pSTARTV = SQRT(RNM2)
#else
      STARTV = SQRT(RNM2)
#endif
      RETURN
      END





