C
C
#define COMPLEX_VERSION 
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pzLANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     &   ALF,BET,Q,IERR,MSGLVL,MPICOM)
#else
      SUBROUTINE zLANSO2(N,LANMAX,MAXPRS,LOHI,KAPPA,J,NEIG,RITZ,BND,
     &   ALF,BET,Q,IERR,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LANMAX,MAXPRS,LOHI,J,NEIG,IERR,MSGLVL
      DOUBLE PRECISION KAPPA,
     &  ALF(LANMAX),BET(LANMAX+1),RITZ(LANMAX),BND(LANMAX)
      DOUBLE COMPLEX Q(N,LANMAX)
C
C     version for complex matrices.  
C
C     Memory use is not very well optimized.  In particular, if
C     eigenvectors are calculated, the storage for the vectors
C     could be used as work-space.
C
C
c     This is modified from planso to compute a few extreme eigenvalues
c     of a given operator.  An eigenpair is declared converged if
c     error bound < kappa * max(|Ritz value|, eps^2/3)
c
C.... inputs
C.... N      dimension of the eigenproblem
C.... LANMAX upper limit to the number of Lanczos steps
C.... MAXPRS upper limit to the number of wanted eigenpairs
C.... LOHI   <=0 compute smallest eigenvalue, >0 compute
C....        largest eigenvalues
C.... KAPPA  Relative tolerance on error of Ritz values
C.... Q      The first N elements contain either ZERO
C....        or an initial guess for the first Lanczos vector
C
C.... work space
C.... Q      Array to hold the Lanczos vectors.
C.... ALF    array to hold diagonal of the tridiagonal T
C.... BET    array to hold off-diagonal of T
C
C.... outputs
C.... J      actual number of Lanczos steps taken
C.... NEIG   number of converged Ritz-pairs
C.... RITZ   array to hold the computed Ritz values
C.... BND    array to hold the error bounds
C.... IERR   error flag
C
C.... BLAS routines:    DATX,DAXPY,DCOPY,DDOT,DSCAL
C.... subroutines:      TQLB,ORTBND,PURGE,STARTV,STPONE,
C....                   RITZSORT,LANCONV,DDOTMPI,pSTPONE
C.... user-supplied:    OP,OPM (or zOP, zOPM)
C
C
      DOUBLE COMPLEX R(N),RA(N),QA(N)
      DOUBLE PRECISION ETA(LANMAX),OLDETA(LANMAX)
C
      DOUBLE PRECISION REALONLY
      DOUBLE PRECISION ZERO,ONE,FOUR
      PARAMETER (ZERO=0.0D0, ONE=1.0D0, FOUR=4.0D0)
C
      LOGICAL ENOUGH
      INTEGER LL,I,L,FIRST,LAST,ID2,ID3
      INTEGER NTOTAL
      DOUBLE PRECISION  T
C
C.... common block for machine epsilon and related variables
C
      INCLUDE 'rdata.f'
C
      DOUBLE PRECISION DWRK(LANMAX),DWRK2(2*LANMAX)
C
      INTEGER MYRANK
#ifdef USE_MPI
      INTEGER MPICOM,TEMP
      INCLUDE 'mpif.h'
      DOUBLE PRECISION pzSTARTV
      DOUBLE COMPLEX ZDOTCMPI
#else
      DOUBLE PRECISION zSTARTV
      DOUBLE COMPLEX ZDOTC
#endif
C
      J = 1
      CALL zSTPONE(N,ALF,R,RA,Q,QA,MSGLVL)
C
#ifdef USE_MPI
      CALL MPI_Comm_rank(MPICOM, MYRANK, IERR)
      CALL MPI_ALLREDUCE(N,NTOTAL,1,MPI_INTEGER,MPI_SUM,MPICOM,IERR)      
#else
C.... MYRANK is a dummy variable for the non-MPI case
      MYRANK=0
      NTOTAL=N
#endif
      IF (IERR.NE.0) THEN
         PRINT *,' error in LANSO2 (ierr =',IERR,'), maybe mpi-related'
      ENDIF

      IF (N.EQ.1) THEN
         NEIG = 1
         RITZ(1) = ALF(1)
         BND(1) = ZERO
      ENDIF
      ETA(1) = EPS1
      OLDETA(1) = EPS1
C
      LL = 0
      FIRST = 2
      LAST = MIN(MAXPRS+MAX(8,MAXPRS),LANMAX)
      ENOUGH = .FALSE.
      DO WHILE (FIRST.LE.LANMAX .AND. .NOT. ENOUGH)
C
C....    lanczos loop
C
         DO J = FIRST,LAST
            BET(J) = RNM
C
C...        Restart if invariant subspace is found
C 
            IF (BET(J).EQ.ZERO) THEN
               RNM = zSTARTV(N,J,R,RA,Q,EPS,MSGLVL)
               ENOUGH = RNM.EQ.ZERO
               IF (ENOUGH) EXIT
            ENDIF
C
C....       take a Lanczos step
C
            T = ONE/RNM
C.... replacing:  DATX(N,T,R,1,R(NQ(1)),1)
            CALL ZDSCAL(N,T,R,1)
            CALL ZDSCAL(N,T,RA,1)
            CALL ZCOPY(N,R,1,Q(1,J),1)
            CALL ZCOPY(N,RA,1,QA,1)
C
C.... at this point, the variables contain:
C
C    R(NQ(2))  q_{j-1}
C    R(NQ(4))  M.q_{j-1}
C    R(NQ(1))  q_j  
C    R(NQ(3))  M.q_j
C    RNM       beta_{j-1}
C    BET(J)    beta_{j-1} before any restart...
C
            CALL OP_(N,QA,Q(1,J),R)
C
C    R         p = M^{-1}.K.q_j
C
C         This should be real if M and K are Hermetian.
C
            T = REALONLY(ZDOTC(N,QA,1,R,1),IERR)
            CALL ZAXPY(N,-DCMPLX(T,ZERO),Q(1,J),1,R,1)
            ALF(J) = T
C
C    ALF(J)    alpha_j = (M.q_j)^\dag p
C    R         p - alpha_j q_j
C
C BvdS:   moved this here ...
C
            IF (BET(J).GT.ZERO)
     &           CALL ZAXPY(N,-DCMPLX(RNM,ZERO),Q(1,J-1),1,R,1)
C
C    R         r_j = p -  beta_{j-1} q_{j-1} - alpha_j q_j
C          
C.... find the norm of r_j and estimate errors
C
            CALL OPM_(N,R,RA)
C
C....               this should be real if M is Hermetian
C
            RNM = SQRT(REALONLY(ZDOTC(N,RA,1,R,1),IERR))
C$$$ Rasmus munk Larsen <rmunk@daimi.aau.dk> observed that ANORM is
C$$$ computed based on local estimate rather than global estimate
C$$$ The following is the suggested change -- Aug. 1998
            ANORM = MAX(BET(J)+ABS(ALF(J))+RNM, ANORM)
            TOL = EPSN*ANORM
C     
C.... update the orthogonality bounds
C  
C     http://www.nersc.gov/~kewu/ps/trlan.ps
C
C
            CALL ORTBND(J,ALF,BET,ETA,OLDETA)
C
C....       restore the orthogonality state when needed
C           this modifies r_j and q_j to obtain orthogonality
C           against previous vectors.  
C
C   This routine also updates RNM, returning zero if orthogonalization
C   fails.
C
#ifdef USE_MPI
            CALL pzPURGE(N,LL,J,ALF(J),BET(J),R,RA,Q,QA,
     &           ETA,OLDETA,MSGLVL,MPICOM)
#else 
            CALL zPURGE(N,LL,J,ALF(J),BET(J),R,RA,Q,QA,
     &           ETA,OLDETA,MSGLVL)
#endif
C     
C     R(NQ(4))  M.r_j       
C     
        ENDDO
C     
C.... end of main loop
C
         J = LAST
15       IF (ENOUGH) J = J-1
         FIRST = J+1
         BET(J+1) = RNM
C
C....    Now analyze T
C
         L = 1
         DO 40 ID2 = 1,J
            IF (L.GT.J) GOTO 50
            DO 20 I = L,J
               IF (BET(I+1).EQ.ZERO) GOTO 30
20          CONTINUE
            I = J
C....       Now i is at the end of an unreduced submatrix
C
 30         CONTINUE
            CALL DCOPY(I-L+1,ALF(L),1,RITZ(L),1)
            IF (I.GT.L) CALL DCOPY(I-L,BET(L+1),1,DWRK(L),1)
            CALL DSTQRB(I-L+1,RITZ(L),DWRK(L),BND(L),DWRK2,IERR)
            IF (IERR.NE.0) THEN
               PRINT *,' DSTQRB failed to converge (ierr =',IERR,')'
               PRINT *,' L =',L,' I =',I
               PRINT *,(ID3,RITZ(ID3),DWRK(ID3),BND(ID3),ID3 = L,I)
            ENDIF
            DO 35 ID3 = L,I
               BND(ID3) = RNM*ABS(BND(ID3))
35          CONTINUE
            L = I+1
40       CONTINUE
C
C....    Sort eigenvalues so the wanted ones are in the front
C
50       CALL SORTRITZ(LOHI,J,RITZ,BND)
C
C  BvdS:   On some machines, different processes return slightly 
C          different answers for laconv(), presumably due to round-off 
C          errors.  Changed so that results are shared with the rest.
C
C 
         call lanconv(j, ritz, bnd, maxprs, eps, kappa, neig)
#ifdef USE_MPI
         TEMP=NEIG
         CALL MPI_ALLREDUCE(TEMP,NEIG,1,MPI_INTEGER,MPI_MIN,MPICOM,IERR)
#endif
         enough = (neig.ge.maxprs)
         IF (MSGLVL.GT.5.AND.MYRANK.EQ.0) then
            PRINT *, 'LANSO: STEP ', j, ';  NEIG ', neig,
     &           '   RES ', BND(1)
            if (msglvl.gt.10) then
               do 60 i = 1, min(msglvl, j)
                  print *, i, ritz(i), bnd(i)
 60            continue
            endif
         ENDIF
C
C....    Should we stop?
C
         IF (.NOT.ENOUGH) THEN
            IF (NEIG.EQ.0) THEN
               LAST = FIRST + MAX(8, MAXPRS)
            ELSE
               LAST = FIRST + MIN(128, MAX(MAXPRS-NEIG,
     &              NINT(0.25*J/NEIG)))
            ENDIF
            IF (3+LAST .ge. LANMAX) LAST = LANMAX
         ELSE
            ENOUGH = .TRUE.
         ENDIF
         ENOUGH = ENOUGH.OR.FIRST.GT.LANMAX
      ENDDO
      RETURN
      END
c-----------------------------------------------------------------------
C
C  This returns the real part of a complex number.
C
C  In several of the routines, there are dot products
C  which should yield real-valued results.  A complex result
C  indicates that the matrices are not Hermitian 
C  or that there is a bug.
C
C
      DOUBLE PRECISION FUNCTION REALONLY(Z,IERR)
      DOUBLE COMPLEX Z
      IF(ABS(DIMAG(Z)).GE.1.0D-8)THEN
         PRINT *,'Imaginary part should be zero, ',ABS(DIMAG(Z))
         PRINT *,'Your matrices may not be Hermitian'
         IERR = IERR + 256
      ENDIF
      REALONLY=DBLE(Z)
      END
C
