C
C @(#)purge.f	3.16 (BNP) 5/11/89; from purge.f 2.13 6/25/88
C
C
#define COMPLEX_VERSION
#include "mpi_functions.h"
#ifdef USE_MPI
      SUBROUTINE pzPURGE(N,LL,J,ALPHA,BETA,R,RA,Q,QA,ETA,OLDETA,
     *     MSGLVL,MPICOM)
#else
      SUBROUTINE zPURGE(N,LL,J,ALPHA,BETA,R,RA,Q,QA,ETA,OLDETA,
     *     MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,LL,J,MSGLVL
      DOUBLE PRECISION ETA(J),OLDETA(J),ALPHA,BETA
      DOUBLE COMPLEX R(N),Q(N,J),RA(N),QA(N)
C
C.... routine to perform a complete or local reorthogonalization
C.... complex version
C
C.... This routine examines ETA to decide whether
C.... re-orthogonalization should be performed.
C
C....        RNM is modified by this routine, returning zero if 
C            reorthogonalization fails
C.... ALPHA  alpha_j which is shifted upon reorthogonalization
C.... BETA   beta_{j-1}
C.... N      dimension of the eigenproblem
C.... LL     no. of initial Lanczos vectors in local orthog.
C.... J      current Lanczos step
C.... R      the residual vector to become the next Lanczos vector
C.... Q      matrix of lanczos vectors
C.... RA     the product of the mass matrix and r
C.... QA     the product of the mass matrix and q
C.... WRK    a temporary vector to hold the previous Lanczos vectors
C.... ETA    state of orthogonality between r and previous Lanczos vectors
C.... OLDETA state of orthogonality between q and previous Lanczos vectors
C.... MPICOM MPI Communicator
C
C.... BLAS routines:    ZAXPY,ZDOTC,IDAMAX
C.... subroutines:      ZDOTCMPI
C.... user-supplied:    ZOPM,ZSTORE
C
C.... common block for machine epsilon and related variables
C
      INCLUDE 'rdata.f'
C
      DOUBLE PRECISION REALONLY
      INTEGER K,I,LOOP,IDAMAX,IERR
      DOUBLE PRECISION TQ,TR,ETAJ
      DOUBLE PRECISION ZERO
      PARAMETER (ZERO=0.0D0)
      DOUBLE COMPLEX ZT
      INTEGER MY_RANK
C
#ifdef USE_MPI
      INTEGER MPICOM
      DOUBLE  COMPLEX  ZDOTCMPI      
      INCLUDE 'mpif.h'
#else
      DOUBLE COMPLEX ZDOTC
#endif
C  in the non-parallel case, MY_RANK is a dummy variable
      MY_RANK=0
C
C    don't do anything the first time after a restart.
C
      IF (J.LE.LL+1) RETURN
C
C    criterion for reorthogonalization used by TRLan (Wu and Simon)
C
      ETAJ=SQRT(ALPHA*ALPHA+BETA*BETA)
C     
C     global reorthogonalization based on omega-recurrance
C     
      K = IDAMAX(J-(LL+1),ETA(LL+1),1)+LL
      
C     
C  Two possible criteria for global reorthogonalization
C  lanso.F (omega-recurrence): ABS(ETA(K)).GT.REPS
C  Wu & Simon paper and trlan: ETAJ.GT.RNM
C  
C  For the transverse lattice, ETAJ.GT.RNM is almost always true
C  Orthogonality is not maintained for mesons if
C  the omega-recurrence is used.
C     
      IF (ABS(ETA(K)).GT.EPSN.OR.ETAJ.GT.RNM) THEN
         IF (MSGLVL.GT.5) THEN
#ifdef USE_MPI
            CALL MPI_COMM_RANK(MPICOM, MY_RANK, I)
#endif
            IF (MY_RANK.EQ.0)
     *           PRINT 33,J,ABS(ETA(K)),EPSN,ETAJ,RNM
 33         FORMAT ('    global reorthog. J=',I3,' ETA=',
     *           D10.4,' REPS=',D10.4,', ETAJ=',D10.4,' RNM=',D10.4)
         ENDIF
C     Up to four trips.
         DO LOOP = 1,4
C
C     RNM is hopelessly small (go find a new random vector)
C     Using criterion in Wu and Simon's paper, pp. 14 & 18
C
            IF (RNM.GT.EPS1*ETAJ) THEN
C     
C.... Bring in a Lanczos vector t and orthogonalize both r and q
C.... against it
C     
               TQ = 0.0D0
               TR = 0.0D0
               DO I = 1,J-1
C     
C     This should only make a small improvement
C     Only bother doing once.
C
C     This may not be necessary is many global reorthogonalizations
C     are perfomed.
C     
#define ORTHO_Q 0 /* also orthogonalize Q against previous vectors */
#if ORTHO_Q 
                  IF(LOOP.EQ.1)THEN
                     ZT = ZDOTC(N,Q(1,I),1,QA,1)
                     TQ = TQ + ZT*CONJG(ZT)
                     CALL ZAXPY(N,-ZT,WRK,1,Q(1,J),1)
                  ENDIF
#endif
C     
                  ZT = ZDOTC(N,Q(1,I),1,RA,1)
                  TR = TR + ZT*CONJG(ZT)
                  CALL ZAXPY(N,-ZT,Q(1,I),1,R,1)
               ENDDO
C     
C.... restore local orthogonality
C
               ZT = ZDOTC(N,Q(1,J),1,RA,1)
               TR = TR + ZT*CONJG(ZT)
               CALL ZAXPY(N,-ZT,Q(1,J),1,R,1)
               ALPHA = ALPHA + REALONLY(ZT,IERR)
C     
#if ORTHO_Q  /* only do this if Q is orthogonalized above */
               IF(LOOP.EQ.1)CALL OPM_(N,Q(1,J),QA)
#endif
               CALL OPM_(N,R,RA)
               RNM = SQRT(REALONLY(ZDOTC(N,RA,1,R,1),IERR))
C        See Wu & Simon, p. 15, and TRLan code
C       (this was SQRT(TR).LE.EPS1/REPS*RNM in lanso.f)
               IF (SQRT(TR).LE.RNM) GOTO 58
               IF (MSGLVL.GT.5.AND.MY_RANK.EQ.0)
     &              PRINT *, '     re-orthog J=',J,' error=',SQRT(TR)
            ENDIF
         ENDDO
C   set RNM to zero if no convergence is found
         RNM=ZERO
 58      CONTINUE
         DO I = LL+1,J
            ETA(I) = EPS1
            OLDETA(I) = EPS1
         ENDDO
      ELSE
C     
C.... perform local orthogonalization
C     Wu and Simonm, p. 15 suggest this may not be needed
C     
#if 1     
         ZT = ZDOTC(N,Q(1,J-1),1,RA,1)
         CALL ZAXPY(N,-ZT,Q(1,J-1),1,R,1)
C     
         ZT = ZDOTC(N,Q(1,J),1,RA,1)
         CALL ZAXPY(N,-ZT,Q(1,J),1,R,1)
         ALPHA = ALPHA + REALONLY(ZT,IERR)
C     
         CALL OPM_(N,R,RA)
C.... this should be real if M is Hermetian
         RNM = SQRT(REALONLY(ZDOTC(N,RA,1,R,1),IERR))
C  see Wu & Simon, p. 17
         DO I = J-1,J
            ETA(I) = EPS1
            OLDETA(I) = EPS1
         ENDDO
#endif
      ENDIF
C
C  If RNM is hoplessly small, then give up
C
      IF(RNM.LT.EPS1*ETAJ)  RNM = ZERO
      RETURN
      END




