C
C @(#)startv.f	1.10 (BNP) 10/30/90
C
C
#define COMPLEX_VERSION
#include "mpi_functions.h"
#ifdef USE_MPI
      DOUBLE PRECISION FUNCTION pzSTARTV(N,J,R,RA,Q,EPS,MSGLVL,MPICOM)
#else
      DOUBLE PRECISION FUNCTION zSTARTV(N,J,R,RA,Q,EPS,MSGLVL)
#endif
      IMPLICIT NONE
      INTEGER N,J,MSGLVL
      DOUBLE PRECISION EPS
      DOUBLE COMPLEX R(N),RA(N),Q(N,J),ZT
C
C.... This routine delivers a starting vector in R and returns |R|;
C.... it returns ZERO if range is spanned or if no starting vector
C.... within range of operator can be found.
C
C.... N      dimension of the eigenproblem
C.... J      starting index for a Lanczos run
C.... R      an array containing new Lanczos vector
C.... RA     M.R array
C.... Q      array containing previous Lanczos vectors
C
C.... BLAS routines:    ZAXPY, ZDOTC, ZDSCAL
C.... subroutines:      RANDOM, DDOTMPI
C.... user-supplied:    OPM, STORE
C
      DOUBLE PRECISION REALONLY
      INTEGER RETRQ, DOSCAL
      PARAMETER (RETRQ = 2)
C
      INTEGER IRAND,I,ID,LOOP,IERR
      DOUBLE PRECISION T,RANDOM,RNM2
      INTEGER MY_RANK
#ifdef USE_MPI
      INTEGER MPICOM
      DOUBLE COMPLEX ZDOTCMPI
      INCLUDE 'mpif.h'
#else
      DOUBLE COMPLEX ZDOTC
#endif
C
      DOUBLE PRECISION ZERO,ONE
      PARAMETER ( ZERO=0.0D0,ONE=1.0D0)
C
C  in the non-parallel case, MY_RANK is a dummy variable
      MY_RANK=0
      IF (MSGLVL.GT.5) THEN
#ifdef USE_MPI
         call mpi_comm_rank(mpicom, MY_RANK, ierr)
#endif
         IF(MY_RANK.EQ.0)PRINT *,'ZSTARTV: generate random vector, J=',J
      ENDIF
C
#ifdef USE_MPI
      DATA IRAND/0/
C
C.... Initialize the sed to be used to generate random numbers
C.... To limit the number of times MPI_COMM_RANK is called, IRAND is
C.... saved across calls to this routine
C
      IF (IRAND.EQ.0) THEN
         CALL MPI_COMM_RANK(MPI_COMM_WORLD, IRAND, I)
         IF (I .NE. MPI_SUCCESS) STOP 'MPI_COMM_RANK FAILED'
         IRAND = 918272+(J+IRAND)*7219
      ENDIF
#else
      IRAND = 918272+J
#endif
C
C.... get initial vector, default is random
C     There should be no imaginary part.
C
      RNM2 = REALONLY(ZDOTC(N,R,1,R,1),IERR)
      DO 60 ID = 1,3
#ifdef USE_MPI
         IF (ID.GT.1.OR.J.GT.1.OR. (.not. RNM2.GT.EPS)) THEN
#else
         IF (ID.GT.1.OR.J.GT.1.OR.RNM2.LE.EPS) THEN
#endif
            DO I = 1,N
               R(I) = DCMPLX(RANDOM(IRAND)-0.5D0,
     *              (RANDOM(IRAND)-0.5D0))
            ENDDO
         ENDIF
C
C     In the MPI version of Wu and Simon, there is only one call to
C     OPM with the following explanation:
C.... put input vector in the range of M, do not multiply with K
C.... better for restarting and allow zero (0) eigenvalue to be computed
C
C   BvdS  I assume that this improvement is also appropriate for the
C         single processor version.
C
         CALL OPM_(N,R,RA)
C   if the matrix M is hermetian this should be real.
         RNM2 = REALONLY(ZDOTC(N,RA,1,R,1),IERR)
         IF (.NOT. RNM2.GT.ZERO) THEN
            GOTO 60
         ELSE
            if (rnm2.lt.1D-8 .or. rnm2.gt.1D8) then
               doscal = 1
               rnm2 = ONE / sqrt(rnm2)
               call zdscal(n,rnm2,RA,1)
            else
               doscal = 0
            endif
         ENDIF
         CALL OPM_(N,RA,R)
         if (doscal .eq. 1) then
            RNM2 = REALONLY(ZDOTC(N,RA,1,R,1),IERR)
            rnm2 = 1.0D0 / sqrt(rnm2)
            call zdscal(n,rnm2,RA,1)
         endif
         CALL OPM_(N,R,RA)
         RNM2 = REALONLY(ZDOTC(N,RA,1,R,1),IERR)
C
C  If there are previous vectors, then remove overlap
C  of new vector with them.
C
         IF (J.GT.1) THEN
            DO 50 LOOP = 1,2
               DO I = 1,J-1
                  ZT = ZDOTC(N,Q(1,I),1,RA,1)
                  CALL ZAXPY(N,-ZT,Q(1,I),1,R,1)
               ENDDO
               CALL OPM_(N,R,RA)
               T = REALONLY(ZDOTC(N,RA,1,R,1),IERR)
               IF (T.LE.EPS*RNM2) THEN
                  T = ZERO
                  GOTO 55
               ENDIF
 50         CONTINUE
 55         RNM2 = T
         ENDIF
         IF (RNM2.GT.ZERO) EXIT
 60   CONTINUE
#ifdef USE_MPI
 80   pZSTARTV = SQRT(RNM2)
#else
 80   ZSTARTV = SQRT(RNM2)
#endif
      RETURN
      END




