(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     15309,        373]*)
(*NotebookOutlinePosition[     15981,        397]*)
(*  CellTagsIndexPosition[     15937,        393]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Hamiltonian", "Section"],

Cell["Input nonzero matrix elements from laso (using print_els)", "Text"],

Cell[BoxData[
    \(<< "\<~/four/junk.out\>"\)], "Input"],

Cell["List of distinct nonzero matrix elements of the Hamiltonian", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(unique = Union[Map[Chop[#[\([2]\)]] &, list]]\)], "Input"],

Cell[BoxData[
    \({\(-2.263536968418`\), \(-0.752252778064`\), \(-0.432986783174`\), \
\(-0.257238227964`\), \(-0.212206590789`\), \(-0.257238227964`\)\ \
\[ImaginaryI], 
      0.257238227964`\ \[ImaginaryI], \(-0.432986783174`\)\ \[ImaginaryI], 
      0.432986783174`\ \[ImaginaryI], \(-0.752252778064`\)\ \[ImaginaryI], 
      0.752252778064`\ \[ImaginaryI], 0.432986783174`, 0.752252778064`, 
      2.263536968418`, 6.18215969649`, 8.212206590789`, 8.257238227964`, 
      8.445696664908`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Count[Chop[Map[#[\([2]\)] &, list]], #] &, unique]\)], "Input"],

Cell[BoxData[
    \({4, 4, 4, 4, 4, 4, 4, 5, 3, 5, 3, 4, 4, 2, 4, 8, 8, 4}\)], "Output"]
}, Open  ]],

Cell["Reconstruct hamiltonian matrix", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(Block[{ (*switch\ to\ Mma\ indexing*) \ 
        list1 = Map[\((# + {1, 0, 1})\) &, list], 
        pointers1 = pointers + 1}, \  (*fill\ matrix\ with\ zeros*) \ 
      mat = Table[
          0, {Length[pointers1] - 1}, {Length[pointers1] - 
              1}]; \  (*add\ nonzero\ elements*) Do[
        Block[{row = list1[\([j]\)]}, 
          If[row[\([3]\)] == i, mat[\([i, i]\)] = Chop[row[\([2]\)]], 
            mat[\([row[\([3]\)], i]\)] = Conjugate[row[\([2]\)]]; 
            mat[\([i, row[\([3]\)]]\)] = row[\([2]\)]]], {i, 
          Length[pointers1] - 1}, {j, pointers1[\([i]\)], 
          pointers1[\([i + 1]\)] - 1}];]\)\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(mat\)], "Input"],

Cell[BoxData[
    \({{8.257238227964`, 0, 0, \(-0.257238227964`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.432986783174`\ \[ImaginaryI], \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.257238227964`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.257238227964`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.432986783174`\ \[ImaginaryI], 
        0}, {0, 8.212206590789`, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.752252778064`\ \[ImaginaryI], 
        0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.752252778064`\ \[ImaginaryI]}, {0, \(\(0.`\)\(\[InvisibleSpace]\)\
\) + 0.752252778064`\ \[ImaginaryI], 6.18215969649`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, 0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], 0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0}, {\(-0.257238227964`\) + 0.`\ \[ImaginaryI], 0, 0, 
        8.257238227964`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.432986783174`\ \[ImaginaryI], \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.257238227964`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.432986783174`\ \[ImaginaryI], 
        0}, {0, \(-0.212206590789`\) + 
          0.`\ \[ImaginaryI], \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.752252778064`\ \[ImaginaryI], 0, 8.212206590789`, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.752252778064`\ \[ImaginaryI]}, {\(\(0.`\)\(\[InvisibleSpace]\)\) \
- 0.432986783174`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.432986783174`\ \[ImaginaryI], 
        0, 8.445696664908`, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], 0, \(-0.432986783174`\) + 0.`\ \[ImaginaryI], 
        0, 
        0, \(\(2.263536968418`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 0, 0, 0, 0, 0, 0, 
        0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 
        0}, {\(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.257238227964`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 
        0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        8.257238227964`, 0, \(-0.257238227964`\) + 0.`\ \[ImaginaryI], 0, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(-0.432986783174`\) + 0.`\ \[ImaginaryI], 0}, {0, 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 8.212206590789`, 
        0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 0, 
        0, \(-0.752252778064`\) + 
          0.`\ \[ImaginaryI]}, {\(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.257238227964`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.257238227964`\ \[ImaginaryI], 
        0, \(-0.432986783174`\) + 0.`\ \[ImaginaryI], \(-0.257238227964`\) + 
          0.`\ \[ImaginaryI], 0, 8.257238227964`, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 0, 0, 
        0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0}, {0, 0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 0, 0, 0, 
        0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 0, 8.212206590789`, 0, 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI]}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.18215969649`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, 0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.752252778064`\ \[ImaginaryI], 0, 
        0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0}, {0, 0, 0, 0, 
        0, \(\(2.263536968418`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 8.445696664908`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.432986783174`\ \[ImaginaryI], \(-2.263536968418`\) + 
          0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.432986783174`\ \[ImaginaryI], 
        0, 0, \(-0.432986783174`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 
        0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.752252778064`\ \[ImaginaryI], 
        0, 8.212206590789`, 0, 0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.752252778064`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.432986783174`\ \[ImaginaryI], 
        0, 8.257238227964`, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.432986783174`\ \[ImaginaryI], 
        0, \(-0.257238227964`\) + 0.`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.257238227964`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 
        0, 0}, {0, 0, 0, 0, 0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 0, 
        0, 0, 0, 0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.432986783174`\ \[ImaginaryI], 
        8.445696664908`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.432986783174`\ \[ImaginaryI], 
        0, 0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], 
        0, \(-0.432986783174`\) + 
          0.`\ \[ImaginaryI], \(\(2.263536968418`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 0, 0, 8.212206590789`, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.432986783174`\ \[ImaginaryI], 
        0, \(-0.257238227964`\) + 
          0.`\ \[ImaginaryI], \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.432986783174`\ \[ImaginaryI], 0, 8.257238227964`, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.257238227964`\ \[ImaginaryI], 
        0, 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, 0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.752252778064`\ \[ImaginaryI], 0, 
        6.18215969649`, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 
          0.`\ \[ImaginaryI], 0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 
        0, 0, 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 0, 0, 0, 0, 0, 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        8.212206590789`, 0, \(-0.212206590789`\) + 0.`\ \[ImaginaryI], 0, 0, 
        0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(-0.432986783174`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.257238227964`\ \[ImaginaryI], \(\(0.432986783174`\)\(\
\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.257238227964`\ \[ImaginaryI], 
        0, 0, 8.257238227964`, 0, \(-0.257238227964`\) + 0.`\ \[ImaginaryI], 
        0, 0}, {0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 
        0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], \(-0.212206590789`\) + 
          0.`\ \[ImaginaryI], 0, 8.212206590789`, 0, 0, 0}, {0, 0, 0, 0, 0, 
        0, 0, 0, 0, 0, 
        0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
          0.257238227964`\ \[ImaginaryI], \(-0.432986783174`\) + 
          0.`\ \[ImaginaryI], 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 
        0, 0, \(-0.257238227964`\) + 0.`\ \[ImaginaryI], 0, 8.257238227964`, 
        0, 0}, {\(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.432986783174`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.432986783174`\ \[ImaginaryI], 
        0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], \(-0.432986783174`\) + 
          0.`\ \[ImaginaryI], 
        0, \(\(0.432986783174`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, \(-2.263536968418`\) + 0.`\ \[ImaginaryI], 0, 
        0, \(\(2.263536968418`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0, 0, 8.445696664908`, 
        0}, {0, \(\(0.`\)\(\[InvisibleSpace]\)\) + 
          0.752252778064`\ \[ImaginaryI], 0, 
        0, \(\(0.`\)\(\[InvisibleSpace]\)\) - 0.752252778064`\ \[ImaginaryI], 
        0, 0, \(-0.752252778064`\) + 0.`\ \[ImaginaryI], 
        0, \(\(0.752252778064`\)\(\[InvisibleSpace]\)\) + 0.`\ \[ImaginaryI], 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.18215969649`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(mat[\([23, 17]\)]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["Position of given nonzero element in matrix", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(poslist = Position[Chop[mat], unique[\([1]\)]]\)], "Input"],

Cell[BoxData[
    \({{6, 15}, {6, 23}, {12, 15}, {12, 23}, {15, 6}, {15, 12}, {23, 6}, {23, 
        12}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(poslistdiagonal = Select[poslist, Apply[SameQ, #] &]\)], "Input"],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[%]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(eigen = Sort[Chop[Eigenvalues[mat]]] // InputForm\)], "Input"],

Cell[BoxData[
    \(mat >> "\<~/baryon/matrix.m\>"\)], "Input"],

Cell[BoxData[
    \(eigen >> "\<~/baryon/junk.m\>"\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basis", "Section"],

Cell[BoxData[
    \(SetAttributes[State, Orderless]\)], "Input"],

Cell["Input basis from tb", "Text"],

Cell[BoxData[
    \(<< "\<~/baryon/basis.out\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(basis[\([35]\)]\)], "Input"],

Cell[BoxData[
    \(State[{b[1\/2, \(-\(1\/2\)\), u]}, {b[1\/2, \(-\(1\/2\)\), u]}, {b[
          1\/2, 1\/2, u], a[2, \(-1\)]}]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[basis]\)], "Input"],

Cell[BoxData[
    \(954\)], "Output"]
}, Open  ]],

Cell["All", "Text"],

Cell[BoxData[
    \(expr = 
        Sort[Map[Sort, 
            Map[{basis[\([#[\([1]\)]]\)], basis[\([#[\([2]\)]]\)]} &, 
              poslist]]] >> "\<~/baryon/states.out\>"\)], "Input"],

Cell["Diagonal only", "Text"],

Cell[BoxData[
    \(expr2 = 
        Sort[Map[Sort, 
            Map[{basis[\([#[\([1]\)]]\)], basis[\([#[\([2]\)]]\)]} &, 
              poslistdiagonal]]] >> "\<~/baryon/states2.out\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[expr2]\)], "Input"],

Cell[BoxData[
    \(64\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(expr2\)], "Input"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowSize->{808, 667},
WindowMargins->{{Automatic, 86}, {Automatic, 18}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 30, 0, 54, "Section"],
Cell[1772, 53, 73, 0, 32, "Text"],
Cell[1848, 55, 57, 1, 27, "Input"],
Cell[1908, 58, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2008, 62, 78, 1, 27, "Input"],
Cell[2089, 65, 508, 8, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2634, 78, 87, 1, 27, "Input"],
Cell[2724, 81, 88, 1, 27, "Output"]
}, Open  ]],
Cell[2827, 85, 46, 0, 32, "Text"],
Cell[2876, 87, 708, 13, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[3609, 104, 36, 1, 27, "Input"],
Cell[3648, 107, 9520, 154, 635, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13205, 266, 50, 1, 27, "Input"],
Cell[13258, 269, 35, 1, 27, "Output"]
}, Open  ]],
Cell[13308, 273, 59, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[13392, 277, 79, 1, 27, "Input"],
Cell[13474, 280, 119, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13630, 287, 85, 1, 27, "Input"],
Cell[13718, 290, 36, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13791, 296, 42, 1, 27, "Input"],
Cell[13836, 299, 35, 1, 27, "Output"]
}, Open  ]],
Cell[13886, 303, 82, 1, 27, "Input"],
Cell[13971, 306, 63, 1, 27, "Input"],
Cell[14037, 309, 63, 1, 27, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14137, 315, 24, 0, 54, "Section"],
Cell[14164, 317, 64, 1, 27, "Input"],
Cell[14231, 320, 35, 0, 32, "Text"],
Cell[14269, 322, 60, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[14354, 327, 48, 1, 27, "Input"],
Cell[14405, 330, 142, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14584, 337, 46, 1, 27, "Input"],
Cell[14633, 340, 37, 1, 27, "Output"]
}, Open  ]],
Cell[14685, 344, 19, 0, 32, "Text"],
Cell[14707, 346, 189, 4, 27, "Input"],
Cell[14899, 352, 29, 0, 32, "Text"],
Cell[14931, 354, 199, 4, 43, "Input"],

Cell[CellGroupData[{
Cell[15155, 362, 46, 1, 27, "Input"],
Cell[15204, 365, 36, 1, 27, "Output"]
}, Open  ]],
Cell[15255, 369, 38, 1, 27, "Input"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

