(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    135382,       2604]*)
(*NotebookOutlinePosition[    136332,       2634]*)
(*  CellTagsIndexPosition[    136288,       2630]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Construct the Hamiltonian", "Subtitle"],

Cell["\<\
Brett van de Sande
June 20, 2001\
\>", "Subsubtitle"],

Cell["\<\
The notation used in this notebook corresponds to the notation \
introduced in the LaTeX notes \"endpoint.tex\" and to Simon's paper, \
hep-ph/0101318.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Definitions", "Section"],

Cell[BoxData[
    \(Off[General::"\<spell1\>"]\)], "Input",
  InitializationCell->True],

Cell["\<\
All symbols are considered to be c-number unless specified \
otherwise:\
\>", "Text"],

Cell[BoxData[{
    \(cnumberq[x_] := \(FreeQ[x, sigma[_]] && FreeQ[x, a[__]]\) && 
        FreeQ[x, b[__]] && FreeQ[x, d[__]] && 
        FreeQ[x, zeta[_]]\), "\[IndentingNewLine]", 
    \(freelistq[x_, y_List] := Apply[And, Map[FreeQ[x, #] &, y]]\)}], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(prodexpand = {\((a__)\) ** b_Plus \[RuleDelayed] 
            Map[\((a ** #)\) &, b], 
          b_Plus ** \((a__)\) \[RuleDelayed] 
            Map[\((# ** a)\) &, b], \[IndentingNewLine]a_ ** 
              b_Times \[RuleDelayed] 
            Select[b, cnumberq]\ a ** Select[b, \(! cnumberq[#]\) &], 
          b_Times ** a_ \[RuleDelayed] 
            Select[b, cnumberq]\ Select[b, \(! cnumberq[#]\) &] ** 
                a};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(intsumexpand = {a_\  ** \ int[b_, c_] :> 
            int[a ** b, c] /; freelistq[a, c], 
          int[b_, c_] ** a_ :> 
            int[b ** a, c] /; freelistq[a, c], \[IndentingNewLine]a_\  ** \ 
              sum[b_, c_] :> sum[a ** b, c] /; freelistq[a, c], \ 
          sum[b_, c_] ** a_ :> 
            sum[b ** a, c] /; freelistq[a, c], \[IndentingNewLine]int[
              a_Times, b_] :> 
            Module[{vars = Select[a, freelistq[#, b] &]}, 
              vars\ int[Select[a, \(! freelistq[#, b]\) &], b] /; 
                vars =!= 1], \[IndentingNewLine]sum[a_Times, 
              b_] \[RuleDelayed] 
            Module[{vars = Select[a, freelistq[#, b] &]}, 
              vars\ sum[Select[a, \(! freelistq[#, b]\) &], b] /; 
                vars =!= 1], \[IndentingNewLine]int[a_Plus, b_] :> 
            Map[int[#, b] &, a], \[IndentingNewLine]sum[a_Plus, b_] :> 
            Map[sum[#, b] &, a], \[IndentingNewLine]int[d_. \ sum[a_, b_], 
              c_] -> sum[int[d\ a, c], b], \[IndentingNewLine]int\ [
              a_. \ int[b_, c_], d_] :> 
            int[Expand[a\ b], Join[c, d]] /; 
              freelistq[a, c], \[IndentingNewLine]sum\ [a_. \ sum[b_, c_], 
              d_] :> sum[Expand[a\ b], Join[c, d]] /; 
              freelistq[a, c], \[IndentingNewLine]int[a_, b_] :> 
            int[Expand[a], b] /; Expand[a] =!= a};\)\(\ \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(prodfactor = {a_. \ c_NonCommutativeMultiply + 
              b_. \ c_NonCommutativeMultiply \[RuleDelayed] 
            Simplify[Factor[Expand[a] + Expand[b]]]\ c};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(sigmazeta = {a[b__] ** sigma[c_] \[Rule] sigma[c] ** a[b], 
          Conjugate[a[b__]] ** sigma[c_] \[Rule] 
            sigma[c] ** Conjugate[a[b]], \[IndentingNewLine]a[b__] ** 
              zeta[c_] \[Rule] zeta[c] ** a[b], 
          Conjugate[a[b__]] ** zeta[c_] \[Rule] 
            zeta[c] ** Conjugate[a[b]], \[IndentingNewLine]a[b__] ** 
              Conjugate[zeta[c_]] \[Rule] Conjugate[zeta[c]] ** a[b], 
          Conjugate[a[b__]] ** Conjugate[zeta[c_]] \[Rule] 
            Conjugate[zeta[c]] ** Conjugate[a[b]]};\)\)], "Input",
  InitializationCell->True],

Cell["\<\
Relabel the parameters for implicit sums and integrations.  It acts \
on an expression that is linear in NonCommutativeMultiply[...].\
\>", "Text"],

Cell[BoxData[{
    \(\(relabel[a_Plus] := Map[relabel, a];\)\), "\[IndentingNewLine]", 
    \(relabel[ain_\ bin_NonCommutativeMultiply] := 
      Block[{i1 = 1, i2 = 1, j = 1, krules, kcrules, srules, ops, cops}, 
        ops = Cases[Apply[List, bin], 
            a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___]]; 
        cops = Cases[Apply[List, bin], 
            Conjugate[a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___]]]; 
        krules = 
          Map[\((First[#] \[Rule] 
                  Symbol["\<kr\>" <> ToString[\(i2++\)]])\) &, Reverse[ops]]; 
        kcrules = 
          Map[\((First[First[#]] \[Rule] 
                  Symbol["\<kl\>" <> ToString[\(i1++\)]])\) &, cops]; 
        srules = 
          Map[\((# \[Rule] Symbol["\<s\>" <> ToString[\(j++\)]])\) &, 
            Union[Map[Last, 
                Cases[\[IndentingNewLine]\(Apply[List, bin] /. 
                      Conjugate[ex_] \[Rule] ex\) /. 
                    a[_, \(-b_Symbol\)] \[Rule] a[0, b], 
                  a[_, _Symbol] | zeta[_Symbol] | 
                    sigma[_Symbol]\[IndentingNewLine]]]]]; \
\[IndentingNewLine]\(ain\ bin /. Join[krules, kcrules]\) /. 
          srules]\)}], "Input",
  InitializationCell->True],

Cell["\<\
simplify an integration over x, assuming the variable is in a \
delta-function.  It acts on an expression that is linear in delta[...] and \
NonCommutativeMultiply[...].\
\>", "Text"],

Cell[BoxData[{
    \(\(simpint[a_Plus, x_] := 
        Map[simpint[#, x] &, a];\)\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ b_, x_] := 
        a\ simpint[b, x] /; FreeQ[a, x];\)\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ \ delta[\(-x_\) + c_], x_] := 
        simpint[a\ delta[x - c], x];\)\), "\[IndentingNewLine]", 
    \(simpint[a_\ \ delta[x_ + c_], x_] := 
      Simplify[a /. x \[Rule] \(-c\)]\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ , x_] := a /; FreeQ[a, x];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Unprotect[Conjugate];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_Plus] := Map[Conjugate, a];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_Times] := Map[Conjugate, a];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_ ** b_] := 
        Conjugate[b] ** Conjugate[a];\)\), "\[IndentingNewLine]", 
    \(\(Protect[Conjugate];\)\), "\[IndentingNewLine]", 
    \(\(sigma /: Conjugate[sigma[a_]] = 
        sigma[a];\)\), "\[IndentingNewLine]", 
    \(\(sigma /: sigma[a_Integer] ** sigma[a_Integer] ** b_ = 
        b;\)\), "\[IndentingNewLine]", 
    \(\(sigma /: b ** sigma[a_Integer] ** sigma[a_Integer] = 
        b;\)\), "\[IndentingNewLine]", 
    \(\(zeta /: Conjugate[zeta[a_]] := zeta[a];\)\), "\[IndentingNewLine]", 
    \(\(int /: Conjugate[int[a_, b_]] := 
        int[Conjugate[a] /. Map[\((Conjugate[#] \[Rule] #)\) &, b], 
          b];\)\), "\[IndentingNewLine]", 
    \(\(sum /: Conjugate[sum[a_, b_]] := 
        sum[Conjugate[a] /. Map[\((Conjugate[#] \[Rule] #)\) &, b], 
          b];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(intconv[
        x_] := \((int[a_, b_] \[RuleDelayed] 
          int[Integrate[a, x], Complement[b, {x}]] /; 
            MemberQ[b, x])\)\)], "Input",
  InitializationCell->True],

Cell["\<\
A variable x is defined to be positive[x]=True if it is *always* \
positive.\
\>", "Text"],

Cell[BoxData[{
    \(\(Unprotect[Positive];\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_ + b_] := 
        False /; \(! Positive[a]\) && \(! Positive[
                b]\);\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_?Positive + b_?Positive] = 
        True;\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_?Negative\ b_?Positive] = 
        False;\)\), "\[IndentingNewLine]", 
    \(\(Protect[Positive];\)\), "\[IndentingNewLine]", 
    \(\(neverzero[a_] := If[Positive[a], True, True, False];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(On[General::"\<spell1\>"]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Find Yukawa terms (OLD)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(u[x_] := 
      1/Sqrt[2\ Pi]\ int[
          b[kf]\ Exp[\(-I\)\ kf\ x] + 
            sigma[1] ** Conjugate[d[kf]]\ Exp[
                I\ kf\ x], {kf}]\), "\[IndentingNewLine]", 
    \(m[x_, rho_] := 
      1/Sqrt[4\ Pi]\ int[\((a[ka, \(-rho\)]\ Exp[\(-I\)\ ka\ x] + \ 
                Conjugate[a[ka, rho]]\ Exp[I\ ka\ x])\)/
            Sqrt[ka], {ka}]\), "\[IndentingNewLine]", 
    \(kappaa /: Conjugate[kappaa] := kappaa\), "\[IndentingNewLine]", 
    \(kappas /: Conjugate[kappas] := kappas\), "\[IndentingNewLine]", 
    \(muf /: Conjugate[muf] := muf\)}], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kappas\)\" is similar \
to existing symbol \"\!\(kappaa\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(xi = \ 
      Expand[muf/Sqrt[2]\ sigma[1] ** u[x] + 
            kappaa/Sqrt[2]\ sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
            kappas/Sqrt[2]\ sum[sigma[1] ** m[x, rr] ** u[x], {rr}] //. 
          prodexpand (*\(//.\)\(sigmazeta\)*) ]\)], "Input"],

Cell[BoxData[
    \(cxi = \(\(\(Conjugate[xi] /. {Conjugate[x] \[Rule] x, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll}\) //. prodexpand\) //. 
        sigmazeta\)], "Input"],

Cell[BoxData[
    \(expr = 
      Collect[Expand[\(\(\(\((\(-I\)\ cxi ** int[xi, {x}] + 
                        I\ int[cxi, {x}] ** xi)\)/2 //. prodexpand\) /. 
                intconv[x]\) //. prodexpand\) //. sigmazeta], {muf, kappaa, 
          kappas}]\)], "Input"],

Cell[BoxData[
    \(int[expr, {x}] //. prodexpand\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Find terms in the Hamiltonian", "Section"],

Cell["\<\
This version leaves all of the integrations over momenta and \
position implicit.  Also, sums are implicit.\
\>", "Text"],

Cell[BoxData[{
    \(u[x_] := 
      1/Sqrt[2\ Pi]\ \((b[kf]\ Exp[\(-I\)\ kf\ x] + 
            sigma[1] ** Conjugate[d[kf]]\ Exp[
                I\ kf\ x])\)\), "\[IndentingNewLine]", 
    \(m[x_, rho_] := 
      1/Sqrt[4\ Pi]\ \((a[ka, \(-rho\)]\ Exp[\(-I\)\ ka\ x] + \ 
              Conjugate[a[ka, rho]]\ Exp[I\ ka\ x])\)/
          Sqrt[ka]\), "\[IndentingNewLine]", 
    \(kappaa /: Conjugate[kappaa] := kappaa\), "\[IndentingNewLine]", 
    \(kappas /: Conjugate[kappas] := kappas\), "\[IndentingNewLine]", 
    \(muf /: Conjugate[muf] := muf\)}], "Input"],

Cell["\<\
This is to define the momenta variables to be *always * positive.\
\
\>", "Text"],

Cell[BoxData[{
    \(kl1 /: Positive[kl1] = True; kl2 /: Positive[kl2] = True; 
    kl3 /: Positive[kl3] = True; 
    kl4 /: Positive[kl4] = True;\), "\[IndentingNewLine]", 
    \(kr1 /: Positive[kr1] = True; kr2 /: Positive[kr2] = True; 
    kr3 /: Positive[kr3] = True; kr4 /: Positive[kr4] = True;\)}], "Input"],

Cell[CellGroupData[{

Cell["Find Yukawa terms", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(xi = \ 
      Expand[\(\(muf/Sqrt[2]\ sigma[1] ** u[x] + 
                kappaa/Sqrt[2]\ sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
                kappas/Sqrt[2]\ sigma[1] ** m[x, sumrr] ** u[x] //. 
              prodexpand\) /. {zeta[rho_] ** 
                  a[c_, \(-rho_\)] \[Rule] \(-zeta[rho] ** a[c, rho]\), 
              a[b_, \(-sumrr\)] \[Rule] a[b, sumrr]}\) //. 
          prodexpand]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x\)\ muf\ \
Conjugate[d[kf]]\)\/\(2\ \@\[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kf\ x\)\ muf\ sigma[1] ** b[kf]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ ka\ x - \[ImaginaryI]\ kf\ x\)\ kappas\ sigma[1] ** a[ka, \
sumrr] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
ka\ x - \[ImaginaryI]\ kf\ x\)\ kappas\ sigma[1] ** Conjugate[a[ka, sumrr]] ** \
b[kf]\)\/\(4\ \@ka\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ ka\ x \
+ \[ImaginaryI]\ kf\ x\)\ kappas\ sigma[1] ** a[ka, sumrr] ** sigma[1] ** \
Conjugate[d[kf]]\)\/\(4\ \@ka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
ka\ x + \[ImaginaryI]\ kf\ x\)\ kappas\ sigma[1] ** Conjugate[a[ka, sumrr]] ** \
sigma[1] ** Conjugate[d[kf]]\)\/\(4\ \@ka\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ ka\ x - \[ImaginaryI]\ kf\ x\)\ kappaa\ sigma[1] ** zeta[rr] \
** a[ka, rr] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ ka\ x - \[ImaginaryI]\ kf\ x\)\ kappaa\ sigma[1] ** zeta[rr] ** \
Conjugate[a[ka, rr]] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ ka\ x + \[ImaginaryI]\ kf\ x\)\ \
kappaa\ sigma[1] ** zeta[rr] ** a[ka, rr] ** sigma[1] ** Conjugate[d[kf]]\)\/\
\(4\ \@ka\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ ka\ x + \
\[ImaginaryI]\ kf\ x\)\ kappaa\ sigma[1] ** zeta[rr] ** Conjugate[a[ka, rr]] ** \
sigma[1] ** Conjugate[d[kf]]\)\/\(4\ \@ka\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(cxi = \(\(\(Conjugate[xi] /. {Conjugate[x] \[Rule] x, 
                Conjugate[ka] \[Rule] ka, Conjugate[kf] \[Rule] kf, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll, sumrr \[Rule] sumll}\) //. 
          prodexpand\) //. sigmazeta\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ muf\ d[kks]\)\/\(2\ \
\@\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ muf\ \
Conjugate[b[kks]] ** sigma[1]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x - \
\[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** Conjugate[a[kka, sumll]]\)\/\(4\ \
\@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** \
Conjugate[a[kka, sumll]]\)\/\(4\ \@kka\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x + \[ImaginaryI]\ kks\ x\)\ kappaa\ Conjugate[b[kks]] ** \
zeta[ll] ** sigma[1] ** a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\
\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ x\)\ kappaa\ \
Conjugate[b[kks]] ** zeta[ll] ** sigma[1] ** Conjugate[a[kka, ll]]\)\/\(4\ \
\@kka\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ x - \
\[ImaginaryI]\ kks\ x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** sigma[1] ** \
a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ \
x - \[ImaginaryI]\ kks\ x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** \
sigma[1] ** Conjugate[a[kka, ll]]\)\/\(4\ \@kka\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(expr1 = 
        Collect[relabel[
              Expand[\(\((\(-I\)\ cxi ** Integrate[xi, x] + 
                          I\ Integrate[cxi, x] ** xi)\)/2 //. prodexpand\) //. 
                  sigmazeta]], {muf, kappaa, kappas}] //. 
          prodfactor;\)\)], "Input"],

Cell["\<\
Since the delta[...] is removed in the following, the integration \
region can be complicated.  expr2 contians all of the interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      simpint[\(\(\(int[expr1, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
          delta[_?neverzero] \[Rule] 0, k11]\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\) + 
      kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) - \(\((kl1 + kr1 + kr2 - kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** a[kr3, s1] ** a[kr2, \
s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \
\[Pi]\) + \((\((kl1 + kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \(\((kl1 - kr1 + kr2 + kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\) - \((\((\(-kl1\) + kl2 + kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                      kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\))\) + 
      muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                            kl1\^2\ \((2\ kr1 + kr2)\) + 
                            kl1\ \((\(-2\)\ kr1\^2 + 
                                  kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                          b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                            kr2)\)\ \@kr2\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) - \((\((kl1\^2\ \((kl2 - 
                                  2\ kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((kl2\^2 - 
                                  2\ kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl2 + 
                            kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                            2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[
                            kr1] ** Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1 + 
                            kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                            kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + 
                            kr1 + kr2]\ d[kr2] ** a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                                  kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((\(-2\)\ kl2\^2 + 
                                  kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@kr1\ \@\[Pi])\) + \((\((kr1 - 
                            kr2)\)\ \((kl1\^2 + 2\ kr1\ kr2 - 
                            kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                            kr1)\)\ kr1\ \((kl1 - 
                            kr2)\)\ kr2\ \@\[Pi])\))\) + 
            kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                            kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          a[kr2, s1] ** 
                          b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) - \((\@kl2\ \((kl1\^2 + 
                            kl1\ kl2 + 
                            kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                            kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 + 
                            kl2\ \((kl2 - kr1)\) - 
                            kl1\ kr1)\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + 
                            kr2\^2 - 
                            kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                            kr1)\)\ kr2\ \((\(-kl1\) + 
                            kr2)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                            kl1\ \((kl2 - kr1)\) + 
                            kr1\^2)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** 
                          sigma[1] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - 
                            kl1\ kr1 + 
                            kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ kr2\ \((kr1 + kr2)\)\ \@\[Pi])\))\))\) + 
      kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                      kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[
                      kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                      kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr1, s2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                      kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                      kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                      kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                      kr3]\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                      kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)], "Output"]
}, Closed]],

Cell["\<\
These are the 1\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 2 || 
              Count[non, Conjugate[_]] != 1]\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]1 particle interactions\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] != 1]\)], "Input"],

Cell[BoxData[
    \(muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                          kl1\^2\ \((2\ kr1 + kr2)\) + 
                          kl1\ \((\(-2\)\ kr1\^2 + 
                                kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\((kl1 + 
                          kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\((kr1 - 
                          kr2)\)\ \((kl1\^2 + 2\ kr1\ kr2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          kr2)\)\ kr2\ \@\[Pi])\))\) + 
          kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                          kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + kr2\^2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ kr2\ \((\(-kl1\) + 
                          kr2)\)\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - 
                          kl1\ kr1 + 
                          kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\))\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(muf\ \((kappas\ \((\(-\(\((\((kl1\^2\ \((kl2 - 2\ kr1)\) + 
                            kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((kl2\^2 - 
                                  2\ kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\((kl2 + 
                          kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                          2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[
                          kr1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                                kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                          kl1\ \((\(-2\)\ kl2\^2 + 
                                kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                          kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \@\[Pi])\))\) + 
          kappaa\ \((\(-\(\((\@kl2\ \((kl1\^2 + kl1\ kl2 + 
                            kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                            kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\@kr1\ \((kl1\^2 \
+ kl2\ \((kl2 - kr1)\) - kl1\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                          kl1\ \((kl2 - kr1)\) + kr1\^2)\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\))\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\(-\(\(\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** a[kr3, s1] ** 
                      a[kr2, s2] ** 
                      b[kr1]\)\/\(16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((\(-kl1\) + 
                        kr3)\)\ \[Pi]\)\)\) + \(\((kl1 - kr1 + kr2 + kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\) - \((\((kl1 - kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + 
                      kr2 + kr3]\ d[kr3] ** sigma[1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + 
                      kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                      kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                      kr3]\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                      kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + 
                      kl2 - kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                      kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr1, s2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                      kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\(-\(\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
                      Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 - 
                        kr1)\)\ \[Pi])\)\)\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + 
                      kl2 + kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                      kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + kr1)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\(-\(\((\((kl1 + kl2 - kl3 + 
                        kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                        b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[a[kl3, s2]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 - 
                        kr1)\)\ \[Pi])\)\)\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                      kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[
                      kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                      kl3 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Find gauge terms", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(jplus = \ 
      Expand[\(\(I\ m[x, rr] ** D[m[x, \(-rr\)] /. ka \[Rule] kaa, x] - 
                  I\ D[m[x, rr], 
                        x] ** \((m[x, \(-rr\)] /. 
                          ka \[Rule] kaa)\) - \((Conjugate[
                          u[x]] /. {Conjugate[x] \[Rule] x, 
                          Conjugate[kf] \[Rule] kf})\) ** \((u[x] /. 
                        kf \[Rule] kff)\) //. prodexpand\) /. 
              a[b_, \(-lambda_\)] ** Conjugate[a[c_, \(-lambda_\)]] \[Rule] 
                a[b, lambda] ** Conjugate[a[c, lambda]]\) //. prodexpand] //. 
        prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka + kaa)\)\ x\)\ \
\((\(-ka\) + kaa)\)\ a[ka, \(-rr\)] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \
\[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka - kaa)\)\ x\)\ \((ka \
+ kaa)\)\ a[ka, rr] ** Conjugate[a[kaa, rr]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((ka - kaa)\)\ x\)\ \((ka + kaa)\)\ \
Conjugate[a[ka, rr]] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((ka + kaa)\)\ x\)\ \((ka - kaa)\)\ \
Conjugate[a[ka, rr]] ** Conjugate[a[kaa, \(-rr\)]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\
\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x - \[ImaginaryI]\ kff\ x\)\ \
Conjugate[b[kf]] ** b[kff]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kf\ x + \[ImaginaryI]\ kff\ x\)\ d[kf] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x \
+ \[ImaginaryI]\ kff\ x\)\ Conjugate[b[kf]] ** sigma[1] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kf\ x - \[ImaginaryI]\ kff\ x\)\ d[kf] ** sigma[1] ** b[kff]\)\/\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
I don't use Expand[...] because it makes too many terms.\
\>", \
"Text"],

Cell[BoxData[
    \(\(expr3 = 
        g^2/4\ relabel[\((jplus /. {ka \[Rule] la, kaa \[Rule] laa, 
                        rr \[Rule] ll, kf \[Rule] lf, 
                        kff \[Rule] lff})\) ** \((I\ Integrate[
                        I\ Integrate[jplus, x], x])\) //. prodexpand] //. 
          prodfactor;\)\)], "Input"],

Cell[TextData[{
  "For the gauge interactions, the evaluation of the color indices is \
implicit and obeys the following rules:\nIf there are four adjoint fields, \
the color indices are evaluated as traces:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(a\_1\) \(a\_2\) \(a\_3\) a\_4\  \[Rule] \ 
        Tr {\(a\_1\) \(a\_2\) \(a\_3\) a\_4} - 
          Tr {\(a\_1\) a\_2}\ Tr {\(a\_3\) a\_4}/N\)]],
  "\nIf there are two adjoint fields and two fundamental fields, the color \
indices are evaluated as:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(a\_1\) a\_2\ b\_1\ b\_2\  \[Rule] \ 
        b\_1\[CenterDot]a\_1\[CenterDot]a\_2\[CenterDot]b\_2\  - 
          Tr {\(a\_1\) a\_2}\ \ b\_1\[CenterDot]b\_2/N\)]],
  "\nIf there are four fundamental fields, the color indices are evaluated \
as:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(b\ c\ d\ e\  \[Rule] \ 
        b\_i\ c\_j\ d\_k\ e\_l\ \((\[Delta]\_il\ \[Delta]\_kj\  - \ \
\[Delta]\_ij\ \[Delta]\_kl/N)\)\)\)\)]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 = 
      Distribute[\(\(int[expr3, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
        delta[_?neverzero] \[Rule] 0\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                  kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
                Conjugate[a[kl2, s2]] ** 
                Conjugate[
                  a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl1 - kl2)\)\ \((kr1 - 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, \(-s1\)] ** 
              a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
              Conjugate[
                a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) + \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
              Conjugate[
                a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\@kr2\ \@kr3\ \[Pi])\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
              a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(16\ \((kl1 \
- kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) \
+ kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** d[kr1] ** \
Conjugate[d[kl1]]\)\/\(16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) + \
\((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - 
                kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, s1] ** 
              Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) - \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + 
                kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** 
              a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + kr1)\)\ delta[\(-kl1\) - 
                kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl3, 
                  s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) + \
kr1)\)\^2\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(\(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \
\[Pi]\)\)\((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[b[kl3]] ** 
            b[kr1])\)\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
              Conjugate[
                d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(\(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
\((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            b[kr1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\)\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) \
- kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** a[kr1, s1] ** \
Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) \
- \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[b[kl1]] ** b[kr2] ** Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \
\@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + \
kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\
\ \((kl2 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[b[kl1]] ** b[kr2] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \
\((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) - \(\(1\/\(16\ \
\@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \((kl2 - 
              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
            Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\)\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) \
- kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** a[kr1, s1] ** \
Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) \
- \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \
\((kl2 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ d[kr2] ** Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \
\((kl2 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
d[kr2] ** Conjugate[d[kl1]] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \
\((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(\(1\/\(16\ \
\@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\)\((g\^2\ \((kl1 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ a[kr1, s1] ** 
            Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** sigma[1] ** 
            Conjugate[
              d[kl3]])\)\) - \(\(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \
\[Pi]\)\)\((g\^2\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[
              kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
            Conjugate[
              d[kl2]])\)\) + \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) - \(\(1\/\(16\ \
\@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\)\((g\^2\ \((kl1 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** a[kr1, s1] ** Conjugate[b[kl2]] ** sigma[1] ** 
            Conjugate[
              d[kl3]])\)\) - \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) - \(\(1\/\(16\ \
\@kl1\ \@kl2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ \((kl1 - 
              kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** d[kr2] ** 
            sigma[1] ** 
            b[kr1])\)\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** b[kr1] ** Conjugate[b[kl2]] ** sigma[1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \((g\^2\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** a[kr1, s1] ** 
              Conjugate[
                a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) + \(\(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ \
\((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
            sigma[1] ** Conjugate[d[kl2]] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)\) - \(\(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \
\[Pi]\)\)\((g\^2\ \((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\)\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** \
b[kr1]\)\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 - \
kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** \
Conjugate[b[kl2]] ** sigma[1] ** Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\
\ \[Pi]\) - \(\(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \[Pi]\)\)\((g\^2\ \
\((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
            b[kr1] ** Conjugate[a[kl1, s1]] ** 
            Conjugate[
              a[kl2, \(-s1\)]])\)\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ d[kr3] ** Conjugate[d[kl1]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(8\ \
\((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] ** Conjugate[a[kl1, \
s1]]\)\/\(16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl1 \
+ kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** \
Conjugate[a[kl1, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** \
b[kr2] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(8\ \((kl1 - kr1)\)\^2\ \[Pi]\) + \
\(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** \
d[kr1] ** Conjugate[d[kl1]]\)\/\(8\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \
\[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** \
b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(8\ \((kl1 + \
kl2)\)\^2\ \[Pi]\)\)], "Output"]
}, Closed]],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                  kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
                Conjugate[
                  a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\
\ \@kr2\ \@kr3\ \[Pi])\)\)\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
              a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(16\ \((kl1 \
- kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) \
+ kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** d[kr1] ** \
Conjugate[d[kl1]]\)\/\(16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) + \
\((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ Conjugate[b[kl1]] ** b[kr3] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(16\ \
\@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** a[kr2, \
\(-s1\)] ** a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \
\(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, s1] ** \
Conjugate[a[kl1, s1]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(16\ \@kl1\ \((kr1 \
+ kr2)\)\^2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((kl1 + kr1)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] \
** Conjugate[a[kl1, s1]]\)\/\(16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\[Pi]\) - \(g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** \
sigma[1] ** b[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \
\((kl1 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(8\ \
\((kl1 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
d[kr3] ** sigma[1] ** b[kr2] ** d[kr1] ** Conjugate[d[kl1]]\)\/\(8\ \((\(-kl1\
\) + kr1)\)\^2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\((g\^2\ \((kl1 - kl2)\)\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                kr2]\ a[kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
              Conjugate[
                a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) + \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + 
                kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** 
              a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) - \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - 
                kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[b[kl1]] ** b[kr2] ** a[kr1, s1] ** Conjugate[a[kl2, \
s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl2 \
+ kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** \
b[kr2] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\) + \
\(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\
\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** a[kr1, s1] ** Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \
\@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl2 + kr1)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \((kl2 - kr1)\)\^2\ \
\[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \((\(-kl2\) + kr1)\)\
\^2\ \[Pi]\) - \(\(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \
\[Pi]\)\)\((g\^2\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[
              kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
            Conjugate[
              d[kl2]])\)\) - \(\(1\/\(16\ \@kl1\ \@kl2\ \((kr1 + kr2)\)\^2\ \
\[Pi]\)\)\((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            d[kr2] ** sigma[1] ** 
            b[kr1])\)\) + \(\(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\[Pi]\)\)\((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
            a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)\) - \(\(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\[Pi]\)\)\((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
            sigma[1] ** b[kr1] ** Conjugate[a[kl1, s1]] ** 
            Conjugate[
              a[kl2, \(-s1\)]])\)\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\
\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(8\ \((kl1 + kl2)\)\^2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                  kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
                Conjugate[a[kl2, s2]] ** 
                Conjugate[
                  a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl3, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) \
+ kr1)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(\(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \
\[Pi]\)\)\((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[b[kl3]] ** 
            b[kr1])\)\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
              Conjugate[
                d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(\(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
\((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            b[kr1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\)\) - \(\(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ d[kr1] ** Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\)\) + \(\(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \
\@kr1\ \[Pi]\)\)\((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** 
            sigma[1] ** 
            Conjugate[
              d[kl3]])\)\) - \(\(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \
\[Pi]\)\)\((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, s1] ** Conjugate[b[kl2]] ** 
            sigma[1] ** 
            Conjugate[
              d[kl3]])\)\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** b[kr1] ** Conjugate[b[kl2]] ** sigma[1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\) + \((g\^2\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** a[kr1, s1] ** 
              Conjugate[
                a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(\(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \[Pi]\)\)\((g\^2\ \
\((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            sigma[1] ** Conjugate[d[kl2]] ** Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\)\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** \
b[kr1]\)\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 - \
kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** \
Conjugate[b[kl2]] ** sigma[1] ** Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\
\ \[Pi]\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Gluon effective potential", "Subsection"],

Cell["\<\
I only include the terms that act on meson strings in the large N \
limit.\
\>", "Text"],

Cell[BoxData[
    \(ka /: Positive[ka] = True; kb /: Positive[kb] = True; 
    kc /: Positive[kc] = True; kd /: Positive[kd] = True;\)], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Micro]\_B\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
The factor of 1/2 is from the fact that lambda is summing over four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        mub^2/2\ m[x, s1] ** \((m[x, s1] /. ka \[Rule] kb)\);\)\)], "Input"],

Cell[BoxData[
    \(\(expr = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] kk^n}] /. 
            aa_a ** Conjugate[b_] \[Rule] Conjugate[b] ** aa];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      Distribute[\(\(int[expr, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
        delta[_?neverzero] \[Rule] 0\)], "Input"],

Cell[BoxData[
    \(\(mub\^2\ delta[\(-kl1\) + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, \
\(-s1\)]\)\/\(2\ \@kl1\ \@kr1\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The box term \[Beta]", "Subsubsection"],

Cell["\<\
The implicit sums are s1={-1,1} and s2={-2,2}.  There is an overall \
factor of 1/4 since there are no sums in the original expression.\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        m[x, sa] ** \((m[x, sb] /. ka \[Rule] kb)\) ** 
          Conjugate[m[x, sa] /. ka \[Rule] kc] ** 
          Conjugate[m[x, \(-sb\)] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(\(-beta\)/4\ \((term + Conjugate[term])\) //. 
                  prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                  a[kl1, \(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, \(-s1\)] ** 
                a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, s2] ** a[kr2, \(-s1\)] ** a[kr1, s2]\)\/\(32\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, \(-s2\)] ** a[kr2, s1] ** a[kr1, \(-s2\)]\)\/\
\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s2] ** a[kr2, s1] ** a[kr1, s2]\)\
\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, \(-s1\)] ** a[kr2, s2] ** \
a[kr1, s1]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, s1] ** \
a[kr2, s2] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \
\(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** a[kr3, \
\(-s1\)] ** a[kr2, \(-s2\)] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\
\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** \
a[kr3, s1] ** a[kr2, \(-s2\)] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\
\ \@kr3\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                  a[kl1, \(-s1\)]] ** Conjugate[a[kl2, \(-s2\)]] ** 
                a[kr2, \(-s1\)] ** 
                a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)\) - \
\(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** \
Conjugate[a[kl2, s2]] ** a[kr2, \(-s1\)] ** a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s1\)]] ** \
a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\
\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s2] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, \(-s2\)]] ** a[kr2, s1] ** a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr2, \(-s1\
\)] ** Conjugate[a[kl2, \(-s2\)]] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \
\(-s2\)]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s2] ** a[kr1, \
\(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** \
Conjugate[a[kl2, s1]] ** a[kr2, s2] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s2]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** a[kr1, \
\(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, \
\(-s1\)]] ** a[kr2, \(-s2\)] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \
s2]] ** Conjugate[a[kl2, s1]] ** a[kr2, \(-s2\)] ** a[kr1, s1]\)\/\(32\ \@kl1\
\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\)\((beta\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, s1]] ** 
              a[kr1, s2])\)\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, \(-s2\)])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, s2])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s2\)])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s2\)]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s2\)]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, s1])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s2]] ** 
            a[kr1, \(-s1\)])\)\) - \(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
            a[kr1, s1])\)\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_1\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
There is an implicit sum over the whole expression for \
s1={1,2,-1,-2}.  The factor of 1/2 is because s1 takes on all four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda1/2\ m[x, sa] ** 
            Conjugate[
              m[x, sa] /. ka \[Rule] kb] ** \((m[x, sa] /. ka \[Rule] kc)\) ** 
            Conjugate[m[x, sa] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \
\@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)\((lambda1\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            a[kr2, \(-s1\)] ** Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\)\) + \(lambda1\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, s1] ** \
a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda1\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda1\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] \
** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\)\((lambda1\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda1\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_2\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
There is an implicit sum over the whole expression for \
s1={1,2,-1,-2}.  The factor of 1/2 is because s1 takes on all four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda2/2\ m[x, sa] ** \((m[x, sa] /. ka \[Rule] kb)\) ** 
            Conjugate[m[x, sa] /. ka \[Rule] kc] ** 
            Conjugate[m[x, sa] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(16\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-s1\)]] ** a[kr3, s1] ** a[kr2, \(-s1\)] ** a[kr1, \
\(-s1\)]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, \(-s1\)] ** \
a[kr2, s1] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \
\(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, \
s1] ** a[kr2, s1] ** a[kr1, \(-s1\)]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)\((lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** 
            a[kr1, \(-s1\)])\)\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, \(-s1\)] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] \
** a[kr2, s1] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr1, \(-s1\)]\)\/\(16\ \@kl1\
\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\
\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s1] ** a[kr1, \
\(-s1\)]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \
s1]] ** a[kr2, s1] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\
\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\)\((lambda2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s1]] ** 
            a[kr1, \(-s1\)])\)\) + \(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\)\) + \(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\)\) + \(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, \(-s1\)]] ** a[kr1, s1])\)\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_4\)]],
  " term"
}], "Subsubsection"],

Cell["The implicit sums are s1={-1,1} and s2={-2,2}.", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda4\ m[x, sa] ** 
            Conjugate[
              m[x, sa] /. ka \[Rule] kb] ** \((m[x, sb] /. ka \[Rule] kc)\) ** 
            Conjugate[m[x, sb] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ bb_ \[RuleDelayed] 
                c\ bb ** aa /; MemberQ[bb, Conjugate[_]], 
              c_\ aa_\  ** \ bb_Conjugate \[RuleDelayed] 
                c\ bb ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, s2] ** a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s2\)] ** a[kr1, s2]\)\/\(8\ \
\@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** a[kr3, s2] ** \
a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\)\)], \
"Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)\((lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\)\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** a[kr2, s2] ** \
a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
Conjugate[a[kl2, s2]] ** a[kr1, s2]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\
\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** a[kr1, s2]\)\/\(8\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-s2\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, \
\(-s2\)]] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\
\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\)\((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s2]] ** 
            a[kr1, s2])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s2\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s2]] ** Conjugate[a[kl2, \(-s2\)]] ** 
            Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_4\)]],
  " term (done more explicitly)"
}], "Subsubsection"],

Cell["The implicit sums are s1={-1,1} and s2={-2,2}.", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda4\ Sum[
            m[x, sa] ** 
              Conjugate[
                m[x, sa] /. ka \[Rule] kb] ** \((m[x, sb] /. 
                  ka \[Rule] kc)\) ** 
              Conjugate[m[x, sb] /. ka \[Rule] kd], {sa, \(-1\), 1, 
              2}, {sb, \(-2\), 2, 4}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                    Conjugate[kk_?Positive] \[Rule] kk, 
                    Conjugate[kk_?Positive^n_] \[Rule] 
                      kk^n}] //. {c_\ aa_a\  ** \ bb_ \[RuleDelayed] 
                  c\ bb ** aa /; MemberQ[bb, Conjugate[_]], 
                c_\ aa_\  ** \ bb_Conjugate \[RuleDelayed] 
                  c\ bb ** aa /; MemberQ[aa, a[__]]}] //. 
          prodfactor;\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] \
** a[kr3, \(-2\)] ** a[kr2, \(-1\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-2\)]] ** a[kr3, \(-2\)] ** a[kr2, 1] ** a[kr1, \
\(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** a[kr3, \(-1\)] ** a[kr2, \
1] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** a[kr3, 1] ** \
a[kr2, \(-1\)] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) \
+ \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** \
a[kr3, \(-2\)] ** a[kr2, 2] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-1\)]] ** a[kr3, \(-1\)] ** a[kr2, \(-2\)] ** a[kr1, \
2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + \
kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** a[kr3, \(-1\)] ** a[kr2, 2] ** \
a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** a[kr3, 2] ** \
a[kr2, \(-2\)] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) \
+ \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, \(-2\)] ** a[kr2, 2] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 1] ** a[kr2, \(-2\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 1] ** a[kr2, 2] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 2] ** a[kr2, \(-2\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, \(-1\)] ** a[kr2, 1] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 1] ** a[kr2, \(-1\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 2] ** a[kr2, \(-1\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 2] ** a[kr2, 1] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] \
** Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-1\)] ** a[kr1, \(-2\)]\)\/\(8\ \
\@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-2\)]] ** Conjugate[a[kl2, 1]] ** a[kr2, 1] ** \
a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** \
Conjugate[a[kl2, 2]] ** a[kr2, \(-1\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-2\)]] ** Conjugate[a[kl2, 2]] ** a[kr2, 1] ** a[kr1, \(-1\
\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - \
kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** a[kr2, \(-2\)] ** \
Conjugate[a[kl2, \(-2\)]] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-1\)]] ** a[kr2, \(-1\)] ** Conjugate[a[kl2, \(-2\)]] ** \
a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** a[kr2, \
\(-1\)] ** Conjugate[a[kl2, 2]] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-1\)]] ** a[kr2, 2] ** Conjugate[a[kl2, 2]] ** a[kr1, \(-1\
\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - \
kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** Conjugate[a[kl2, \(-2\)]] ** \
a[kr2, \(-2\)] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) \
+ \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** \
Conjugate[a[kl2, 1]] ** a[kr2, \(-2\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-1\)]] ** Conjugate[a[kl2, 1]] ** a[kr2, 2] ** a[kr1, \(-2\
\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - \
kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** Conjugate[a[kl2, 2]] ** a[kr2, \
2] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** a[kr2, \(-2\)] ** \
Conjugate[a[kl2, \(-2\)]] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** \
a[kr2, 1] ** Conjugate[a[kl2, \(-2\)]] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kl2\
\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, 1]] ** a[kr2, 1] ** Conjugate[a[kl2, 2]] ** a[kr1, 2]\)\/\(8\
\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 \
+ kr2]\ Conjugate[a[kl1, 1]] ** a[kr2, 2] ** Conjugate[a[kl2, 2]] ** a[kr1, \
1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - \
kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** Conjugate[a[kl2, \(-2\)]] ** a[kr2, \
\(-2\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** Conjugate[a[kl2, \
\(-1\)]] ** a[kr2, \(-2\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** \
Conjugate[a[kl2, \(-1\)]] ** a[kr2, 2] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kl2\
\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, 1]] ** Conjugate[a[kl2, 2]] ** a[kr2, 2] ** a[kr1, 1]\)\/\(8\
\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 \
+ kr2]\ Conjugate[a[kl1, 2]] ** Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-1\)] ** \
a[kr1, 1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** Conjugate[a[kl2, \
\(-2\)]] ** a[kr2, 1] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** \
Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-1\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kl2\
\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, 2]] ** Conjugate[a[kl2, 1]] ** a[kr2, 1] ** a[kr1, 2]\)\/\(8\
\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\)\((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, \(-2\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-2\)]] ** Conjugate[a[kl2, 1]] ** 
            Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-2\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-2\)]] ** Conjugate[a[kl2, 2]] ** 
            Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-2\)]] ** Conjugate[a[kl2, 2]] ** 
            Conjugate[a[kl3, 1]] ** 
            a[kr1, 1])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-1\)]] ** Conjugate[a[kl2, \(-2\)]] ** 
            Conjugate[a[kl3, 2]] ** 
            a[kr1, \(-1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-1\)]] ** Conjugate[a[kl2, 1]] ** 
            Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-2\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-1\)]] ** Conjugate[a[kl2, 1]] ** 
            Conjugate[a[kl3, 2]] ** 
            a[kr1, 2])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-1\)]] ** Conjugate[a[kl2, 2]] ** 
            Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, 1])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-2\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, 2])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, 1])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-1\)])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, 1])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, 2])\)\) + \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\)\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, 2])\)\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell["Junk", "Section"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->Automatic,
WindowSize->{877, 864},
WindowMargins->{{153, Automatic}, {38, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Landscape",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "hamilton.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 45, 0, 64, "Subtitle"],
Cell[1787, 53, 63, 3, 71, "Subsubtitle"],
Cell[1853, 58, 175, 4, 32, "Text"],

Cell[CellGroupData[{
Cell[2053, 66, 30, 0, 54, "Section"],
Cell[2086, 68, 87, 2, 27, "Input",
  InitializationCell->True],
Cell[2176, 72, 95, 3, 32, "Text"],
Cell[2274, 77, 291, 6, 43, "Input",
  InitializationCell->True],
Cell[2568, 85, 496, 10, 59, "Input",
  InitializationCell->True],
Cell[3067, 97, 1442, 25, 219, "Input",
  InitializationCell->True],
Cell[4512, 124, 230, 4, 43, "Input",
  InitializationCell->True],
Cell[4745, 130, 596, 10, 75, "Input",
  InitializationCell->True],
Cell[5344, 142, 157, 3, 32, "Text"],
Cell[5504, 147, 1224, 24, 251, "Input",
  InitializationCell->True],
Cell[6731, 173, 193, 4, 32, "Text"],
Cell[6927, 179, 527, 10, 91, "Input",
  InitializationCell->True],
Cell[7457, 191, 1077, 20, 187, "Input",
  InitializationCell->True],
Cell[8537, 213, 199, 5, 27, "Input",
  InitializationCell->True],
Cell[8739, 220, 100, 3, 32, "Text"],
Cell[8842, 225, 546, 11, 107, "Input",
  InitializationCell->True],
Cell[9391, 238, 86, 2, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[9514, 245, 42, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[9581, 249, 596, 12, 91, "Input"],
Cell[10180, 263, 185, 3, 23, "Message"]
}, Open  ]],
Cell[10380, 269, 279, 5, 59, "Input"],
Cell[10662, 276, 244, 4, 27, "Input"],
Cell[10909, 282, 270, 5, 43, "Input"],
Cell[11182, 289, 62, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[11281, 295, 48, 0, 34, "Section"],
Cell[11332, 297, 131, 3, 32, "Text"],
Cell[11466, 302, 565, 11, 91, "Input"],
Cell[12034, 315, 91, 3, 32, "Text"],
Cell[12128, 320, 314, 5, 59, "Input"],

Cell[CellGroupData[{
Cell[12467, 329, 39, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[12531, 333, 436, 8, 91, "Input"],
Cell[12970, 343, 1513, 20, 366, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14520, 368, 336, 5, 59, "Input"],
Cell[14859, 375, 1509, 20, 366, "Output"]
}, Open  ]],
Cell[16383, 398, 292, 6, 59, "Input"],
Cell[16678, 406, 180, 4, 32, "Text"],

Cell[CellGroupData[{
Cell[16883, 414, 271, 5, 59, "Input"],
Cell[17157, 421, 24413, 371, 3407, "Output"]
}, Closed]],
Cell[41585, 795, 70, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[41680, 801, 276, 6, 43, "Input"],
Cell[41959, 809, 231, 3, 81, "Output"]
}, Open  ]],
Cell[42205, 815, 69, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[42299, 821, 276, 6, 43, "Input"],
Cell[42578, 829, 2377, 37, 346, "Output"]
}, Open  ]],
Cell[44970, 869, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[45065, 875, 285, 6, 43, "Input"],
Cell[45353, 883, 2485, 37, 347, "Output"]
}, Open  ]],
Cell[47853, 923, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[47948, 929, 285, 6, 43, "Input"],
Cell[48236, 937, 5197, 79, 786, "Output"]
}, Open  ]],
Cell[53448, 1019, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[53543, 1025, 285, 6, 43, "Input"],
Cell[53831, 1033, 8296, 126, 1153, "Output"]
}, Open  ]],
Cell[62142, 1162, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[62237, 1168, 285, 6, 43, "Input"],
Cell[62525, 1176, 5905, 89, 809, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[68479, 1271, 38, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[68542, 1275, 630, 11, 107, "Input"],
Cell[69175, 1288, 1132, 16, 222, "Output"]
}, Open  ]],
Cell[70322, 1307, 82, 3, 32, "Text"],
Cell[70407, 1312, 330, 6, 75, "Input"],
Cell[70740, 1320, 996, 20, 129, "Text"],

Cell[CellGroupData[{
Cell[71761, 1344, 263, 5, 59, "Input"],
Cell[72027, 1351, 12352, 184, 2732, "Output"]
}, Closed]],
Cell[84394, 1538, 70, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[84489, 1544, 285, 6, 43, "Input"],
Cell[84777, 1552, 3173, 43, 734, "Output"]
}, Open  ]],
Cell[87965, 1598, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[88060, 1604, 285, 6, 43, "Input"],
Cell[88348, 1612, 5163, 75, 1167, "Output"]
}, Open  ]],
Cell[93526, 1690, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[93621, 1696, 285, 6, 43, "Input"],
Cell[93909, 1704, 4033, 65, 866, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[97991, 1775, 47, 0, 45, "Subsection"],
Cell[98041, 1777, 98, 3, 32, "Text"],
Cell[98142, 1782, 145, 2, 43, "Input"],

Cell[CellGroupData[{
Cell[98312, 1788, 114, 5, 42, "Subsubsection"],
Cell[98429, 1795, 100, 3, 32, "Text"],
Cell[98532, 1800, 106, 2, 27, "Input"],
Cell[98641, 1804, 302, 5, 75, "Input"],

Cell[CellGroupData[{
Cell[98968, 1813, 262, 5, 59, "Input"],
Cell[99233, 1820, 132, 2, 50, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[99414, 1828, 45, 0, 42, "Subsubsection"],
Cell[99462, 1830, 159, 3, 32, "Text"],
Cell[99624, 1835, 204, 4, 27, "Input"],
Cell[99831, 1841, 519, 9, 107, "Input"],
Cell[100353, 1852, 277, 5, 59, "Input"],
Cell[100633, 1859, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[100728, 1865, 286, 6, 43, "Input"],
Cell[101017, 1873, 1420, 19, 342, "Output"]
}, Open  ]],
Cell[102452, 1895, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[102547, 1901, 286, 6, 43, "Input"],
Cell[102836, 1909, 2247, 31, 510, "Output"]
}, Open  ]],
Cell[105098, 1943, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[105193, 1949, 286, 6, 43, "Input"],
Cell[105482, 1957, 1885, 28, 473, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[107416, 1991, 115, 5, 42, "Subsubsection"],
Cell[107534, 1998, 156, 4, 32, "Text"],
Cell[107693, 2004, 229, 5, 43, "Input"],
Cell[107925, 2011, 463, 8, 107, "Input"],
Cell[108391, 2021, 277, 5, 59, "Input"],
Cell[108671, 2028, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[108766, 2034, 286, 6, 43, "Input"],
Cell[109055, 2042, 371, 5, 90, "Output"]
}, Open  ]],
Cell[109441, 2050, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[109536, 2056, 286, 6, 43, "Input"],
Cell[109825, 2064, 794, 11, 206, "Output"]
}, Open  ]],
Cell[110634, 2078, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[110729, 2084, 286, 6, 43, "Input"],
Cell[111018, 2092, 487, 7, 125, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[111554, 2105, 115, 5, 42, "Subsubsection"],
Cell[111672, 2112, 156, 4, 32, "Text"],
Cell[111831, 2118, 214, 4, 43, "Input"],
Cell[112048, 2124, 463, 8, 107, "Input"],
Cell[112514, 2134, 277, 5, 59, "Input"],
Cell[112794, 2141, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[112889, 2147, 286, 6, 43, "Input"],
Cell[113178, 2155, 717, 10, 174, "Output"]
}, Open  ]],
Cell[113910, 2168, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[114005, 2174, 286, 6, 43, "Input"],
Cell[114294, 2182, 1162, 16, 338, "Output"]
}, Open  ]],
Cell[115471, 2201, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[115566, 2207, 286, 6, 43, "Input"],
Cell[115855, 2215, 975, 15, 241, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[116879, 2236, 115, 5, 42, "Subsubsection"],
Cell[116997, 2243, 62, 0, 32, "Text"],
Cell[117062, 2245, 227, 5, 27, "Input"],
Cell[117292, 2252, 468, 8, 107, "Input"],
Cell[117763, 2262, 277, 5, 59, "Input"],
Cell[118043, 2269, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[118138, 2275, 286, 6, 43, "Input"],
Cell[118427, 2283, 713, 10, 174, "Output"]
}, Open  ]],
Cell[119155, 2296, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[119250, 2302, 286, 6, 43, "Input"],
Cell[119539, 2310, 1156, 16, 322, "Output"]
}, Open  ]],
Cell[120710, 2329, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[120805, 2335, 286, 6, 43, "Input"],
Cell[121094, 2343, 956, 14, 241, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[122099, 2363, 138, 5, 42, "Subsubsection"],
Cell[122240, 2370, 62, 0, 32, "Text"],
Cell[122305, 2372, 325, 8, 59, "Input"],
Cell[122633, 2382, 506, 9, 107, "Input"],
Cell[123142, 2393, 277, 5, 59, "Input"],
Cell[123422, 2400, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[123517, 2406, 286, 6, 43, "Input"],
Cell[123806, 2414, 2691, 35, 678, "Output"]
}, Open  ]],
Cell[126512, 2452, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[126607, 2458, 286, 6, 43, "Input"],
Cell[126896, 2466, 4293, 56, 1014, "Output"]
}, Open  ]],
Cell[131204, 2525, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[131299, 2531, 286, 6, 43, "Input"],
Cell[131588, 2539, 3704, 56, 937, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[135343, 2601, 23, 0, 54, "Section"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

