(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     51773,       1570]*)
(*NotebookOutlinePosition[     52690,       1599]*)
(*  CellTagsIndexPosition[     52646,       1595]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Inchworm model bound state equations", "Subtitle"],

Cell["\<\
Brett van de Sande
July, 2001\
\>", "Subsubtitle"],

Cell[CellGroupData[{

Cell["\<\
Integrals associated with the endpoint analysis integral equation\
\
\>", "Section"],

Cell[CellGroupData[{

Cell["\<\
These are the self-intertia integrals on the last two lines of \
Equation  (2.32).\
\>", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = 
      Integrate[\((x1 + 2  x2 - 
              y)\)/\((2\ \((x1 - y)\)^2\ Sqrt[x2\ \((x1 + x2 - y)\)])\), 
        y]\)], "Input"],

Cell[BoxData[
    \(\(x1 + x2 - y\)\/\(\((x1 - y)\)\ \@\(x2\ \((x1 + x2 - y)\)\)\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[
      Limit[expr, y \[Rule] x1 + x2] - Limit[expr, y \[Rule] 0]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\@\(x1 + x2\)\/\(x1\ \@x2\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Expand[PowerExpand[Simplify[\((Sqrt[1 + x1/x2] - 1)\)/x1]]]\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/x1\)\) + \@\(x1 + x2\)\/\(x1\ \@x2\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["These are the gluon-gluon self-inertia integrals.", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr1 = 
      Integrate[
        1/k2\ \((\((k1 + k2)\)^2/\((k1 - k2)\)^2 + \((k1 - k2)\)^2/\((k1 + 
                      k2)\)^2)\), k2]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(4\ k1\)\/\(\(-k1\) + k2\)\)\) + \(4\ k1\)\/\(k1 + k2\) + 
      2\ Log[k2]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(expr1 = Drop[expr1, \(-1\)];\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(4\ k1\)\/\(\(-k1\) + 
              k2\)\)\) + \(4\ k1\)\/\(k1 + k2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((expr1 /. k2 \[Rule] Infinity)\) - \((expr1 /. 
          k2 \[Rule] 0)\)\)], "Input"],

Cell[BoxData[
    \(\(-8\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apart[
      1/k2\ \((\((k1 + k2)\)^2/\((k1 - k2)\)^2 + \((k1 - k2)\)^2/\((k1 + 
                    k2)\)^2)\), k2]\)], "Input"],

Cell[BoxData[
    \(2\/k2 + \(4\ k1\)\/\((\(-k1\) + k2)\)\^2 - \(4\ k1\)\/\((k1 + \
k2)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      Integrate[\(\((k1 + k2)\)^2/\((k1 - k2)\)^2\)/k2, k2]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(4\ k1\)\/\(\(-k1\) + k2\)\)\) + Log[k2]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(expr2 = Drop[expr2, \(-1\)];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((expr2 /. k2 \[Rule] Infinity)\) - \((expr2 /. 
          k2 \[Rule] 0)\)\)], "Input"],

Cell[BoxData[
    \(\(-4\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "These are the integrals needed to calculate ",
  Cell[BoxData[
      \(f\&~\)], "Input"],
  "(w)."
}], "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = \((\(-2\) + \((2 - w - q)\)\ q^
                a\ \((1 - q)\)^
                  b/\((Sqrt[1 - w]\ Sqrt[1 - q]\ \ w^a\ \((1 - w)\)^
                      b)\))\)\ /\((w - q)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(-2\) + \((1 - q)\)\^\(\(-\(1\/2\)\) + b\)\ q\^a\ \((1 - w)\)\^\(\(-\
\(1\/2\)\) - b\)\ \((2 - q - w)\)\ w\^\(-a\)\)\/\((\(-q\) + w)\)\^2\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = Normal[Simplify[Series[integrand, {q, w, \(-1\)}]]]\)], "Input"],

Cell[BoxData[
    \(\(2\ \((a\ \((\(-1\) + w)\) + b\ w)\)\)\/\(\((q - w)\)\ \((\(-1\) + \
w)\)\ w\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand - pole]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(2 + \((1 - q)\)\^\(\(-\(1\/2\)\) + b\)\ q\^a\ \((1 - w)\)\^\(\(-\
\(1\/2\)\) - b\)\ w\^\(-a\)\ \((\(-2\) + q + 
                  w)\) + \(2\ \((q - w)\)\ \((a\ \((\(-1\) + w)\) + b\ w)\)\)\
\/\(\((\(-1\) + w)\)\ w\)\)\/\((q - w)\)\^2\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FullSimplify[Limit[integrand - pole, q \[Rule] w]]\)], "Input"],

Cell[BoxData[
    \(\(4\ \((\(-1\) + a)\)\ a - 8\ a\ \((\(-1\) + a + b)\)\ w + \((\(-1\) + \
2\ a + 2\ b)\)\^2\ w\^2\)\/\(4\ \((\(-1\) + w)\)\^2\ w\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[pole, {q, 0, 1}, PrincipalValue \[Rule] True, 
      Assumptions \[Rule] {w < 1, w > 0}]\)], "Input"],

Cell[BoxData[
    \(\(2\ \((a\ \((\(-1\) + w)\) + b\ w)\)\ \((Log[1 - w] - \
Log[w])\)\)\/\(\((\(-1\) + w)\)\ w\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(ff[w_, a_, b_] := 
      NIntegrate[
          If[Abs[q - w] < 
              10^\(-10\), \(4\ \((\(-1\) + a)\)\ a - 8\ a\ \((\(-1\) + a + b)\
\)\ w + \((\(-1\) + 2\ a + 2\ b)\)\^2\ w\^2\)\/\(4\ \((\(-1\) + w)\)\^2\ w\^2\
\), \(-\(\(2 + \((1 - q)\)\^\(\(-\(1\/2\)\) + b\)\ q\^a\ \((1 - \
w)\)\^\(\(-\(1\/2\)\) - b\)\ w\^\(-a\)\ \((\(-2\) + q + 
                          w)\) + \(2\ \((q - w)\)\ \((a\ \((\(-1\) + w)\) + b\
\ w)\)\)\/\(\((\(-1\) + w)\)\ w\)\)\/\((q - w)\)\^2\)\)], {q, 0, 
            1}] + \(2\ \((a\ \((\(-1\) + w)\) + b\ w)\)\ \((Log[1 - w] - \
Log[w])\)\)\/\(\((\(-1\) + w)\)\ w\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Integral needed for discrete solution", "Subsection"],

Cell[CellGroupData[{

Cell["Integral needed for constant function.", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[
      1/\((w - u)\)^2\ \((\((2 - w - u)\)/\((Sqrt[1 - w]\ Sqrt[1 - u])\) - 
            2)\), u \[Rule] w]\)], "Input"],

Cell[BoxData[
    \(1\/\(4\ \((\(-1\) + w)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{w = 10^\(-3\)}, 
      NIntegrate[
        If[Abs[u - w] < 10^\(-10\), 
          1\/\(4\ \((\(-1\) + w)\)\^2\), \((\((2 - w - 
                      u)\)/\((Sqrt[1 - w]\ Sqrt[1 - u])\) - 
                2)\)/\((w - u)\)^2\ ], {u, 0, w, 1}]]\)], "Input"],

Cell[BoxData[
    \(1.0012513765506668`\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Now, try with (1-w)", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = 
      1/\((w - q)\)^2\ \((\((2 - w - 
                  q)\)*\((1 - q)\)/\((Sqrt[1 - w]\ Sqrt[1 - q])\) - 
            2\ \((1 - w)\))\)\)], "Input"],

Cell[BoxData[
    \(\(\(-2\)\ \((1 - w)\) + \(\@\(1 - q\)\ \((2 - q - w)\)\)\/\@\(1 - w\)\)\
\/\((\(-q\) + w)\)\^2\)], "Output"]
}, Open  ]],

Cell["\<\
Here, there is a leftover pole piece which we have to add and \
subtract.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = Normal[Simplify[Series[integrand, {q, w, \(-1\)}]]]\)], "Input"],

Cell[BoxData[
    \(\(-\(2\/\(q - w\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand - pole\)], "Input"],

Cell[BoxData[
    \(2\/\(q - w\) + \(\(-2\)\ \((1 - w)\) + \(\@\(1 - q\)\ \((2 - q - w)\)\)\
\/\@\(1 - w\)\)\/\((\(-q\) + w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Limit[integrand - pole, u \[Rule] w]]\)], "Input"],

Cell[BoxData[
    \(\(2\ \((q - w)\) + 2\ \((\(-1\) + w)\) - \(\@\(1 - q\)\ \((\(-2\) + q + \
w)\)\)\/\@\(1 - w\)\)\/\((q - w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[pole, {q, 0, 1}, Assumptions \[Rule] {w > 0, w < 1}, 
      PrincipalValue \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(\(-2\)\ \((Log[1 - w] - Log[w])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{w = 0.01}, 
      NIntegrate[
          If[Abs[u - w] < 10^\(-10\), 1\/\(4 - 4\ w\), 
            2\/\(u - w\) + \(\(-2\)\ \((1 - w)\) + \(\@\(1 - u\)\ \((2 - u - \
w)\)\)\/\@\(1 - w\)\)\/\((\(-u\) + w)\)\^2], {u, 0, 1}] - 
        2\ \((Log[1 - w] - Log[w])\)]\)], "Input"],

Cell[BoxData[
    \(\(-8.960303175310827`\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Now, try with correct endpoint behavior at w=1", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = 
      1/\((w - u)\)^2\ \((\((2 - w - u)\)*\((1 - u)\)^
                  b/\((Sqrt[1 - w]\ Sqrt[1 - u])\) - 
            2\ \((1 - w)\)^b)\)\)], "Input"],

Cell[BoxData[
    \(\(\(-2\)\ \((1 - w)\)\^b + \(\((1 - u)\)\^\(\(-\(1\/2\)\) + b\)\ \((2 - \
u - w)\)\)\/\@\(1 - w\)\)\/\((\(-u\) + w)\)\^2\)], "Output"]
}, Open  ]],

Cell["\<\
Here, there is a leftover pole piece which we have to add and \
subtract.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = Normal[Simplify[Series[integrand, {u, w, \(-1\)}]]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(2\ b\ \((1 - w)\)\^\(\(-1\) + b\)\)\/\(u - w\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand - pole]\)], "Input"],

Cell[BoxData[
    \(\(\(-2\)\ \((1 - w)\)\^b + 2\ b\ \((1 - w)\)\^\(\(-1\) + b\)\ \((u - w)\
\) - \(\((1 - u)\)\^\(\(-\(1\/2\)\) + b\)\ \((\(-2\) + u + w)\)\)\/\@\(1 - \
w\)\)\/\((u - w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[integrand - pole, u \[Rule] w]\)], "Input"],

Cell[BoxData[
    \(1\/4\ \((1 - 2\ b)\)\^2\ \((1 - w)\)\^\(\(-2\) + b\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[pole, {u, 0, 1}, Assumptions \[Rule] {w > 0, w < 1}, 
      PrincipalValue \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(\(-2\)\ b\ \((1 - w)\)\^\(\(-1\) + b\)\ \((Log[1 - w] - 
          Log[w])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{w = 0.995, b = 1}, 
      NIntegrate[
          If[Abs[u - w] < 10^\(-5\), 
            1\/4\ \((1 - 2\ b)\)\^2\ \((1 - w)\)\^\(\(-2\) + b\), \(\(-2\)\ \
\((1 - w)\)\^b + 2\ b\ \((1 - w)\)\^\(\(-1\) + b\)\ \((u - w)\) - \(\((1 - u)\
\)\^\(\(-\(1\/2\)\) + b\)\ \((\(-2\) + u + w)\)\)\/\@\(1 - w\)\)\/\((u - w)\)\
\^2], {u, 0, 1}] - 
        2\ b\ \((1 - w)\)\^\(\(-1\) + b\)\ \((Log[1 - w] - 
              Log[w])\)]\)], "Input"],

Cell[BoxData[
    \(29.86887394612107`\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simon's Pole subtraction", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(phi[n_, x_, b_] = x^n\ \((1 - x)\)^b\)], "Input"],

Cell[BoxData[
    \(\((1 - x)\)\^b\ x\^n\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = \((\((2 - v - w)\)\ \((phi[m, v, b]\ phi[n, w, b] + 
                phi[m, w, b]\ phi[n, v, b] - \ phi[m, v, b]\ phi[n, v, b] - \ 
                phi[m, w, b]\ phi[n, w, b])\))\)/\((Sqrt[1 - v] 
            Sqrt[1 - w] \((w - v)\)^2)\)\)], "Input"],

Cell[BoxData[
    \(\(\((2 - v - w)\)\ \((\(-\((1 - v)\)\^\(2\ b\)\)\ v\^\(m + n\) + \((1 - \
v)\)\^b\ v\^n\ \((1 - w)\)\^b\ w\^m + \((1 - v)\)\^b\ v\^m\ \((1 - w)\)\^b\ w\
\^n - \((1 - w)\)\^\(2\ b\)\ w\^\(m + n\))\)\)\/\(\@\(1 - v\)\ \@\(1 - w\)\ \
\((\(-v\) + w)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = 
      Normal[Simplify[Series[integrand, {w, v, \(-1\)}]]]/2 + 
        Normal[Simplify[Series[integrand, {v, w, \(-1\)}]]]/2\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand - pole]\)], "Input"],

Cell[BoxData[
    \(\(\((\(-2\) + v + w)\)\ \((\((1 - v)\)\^b\ v\^m - \((1 - w)\)\^b\ w\^m)\
\)\ \((\((1 - v)\)\^b\ v\^n - \((1 - w)\)\^b\ w\^n)\)\)\/\(\@\(1 - v\)\ \@\(1 \
- w\)\ \((v - w)\)\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Limit[integrand - pole, w \[Rule] v]]\)], "Input"],

Cell[BoxData[
    \(\(-2\)\ \((1 - v)\)\^\(2\ \((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \
\((m\ \((\(-1\) + v)\) + b\ v)\)\ \((n\ \((\(-1\) + v)\) + 
          b\ v)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr1 = 
      Simplify[Integrate[pole, {w, 0, 1}, PrincipalValue \[Rule] True, 
          Assumptions \[Rule] {v > 0, v < 1, b > 0}]]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = 
      Integrate[expr1, {v, 0, 1}, Assumptions \[Rule] {b > 0}]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr /. {b \[Rule]  .1, m \[Rule] 1, n \[Rule] 2}\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{i, j, expr /. {b \[Rule]  .1, m \[Rule] i, n \[Rule] j}}, {i, 0, 
        3}, {j, 0, 3}]\)], "Input"],

Cell[BoxData[
    \({{{0, 0, 0}, {0, 1, 0}, {0, 2, 0}, {0, 3, 0}}, {{1, 0, 0}, {1, 1, 
          0}, {1, 2, 0}, {1, 3, 0}}, {{2, 0, 0}, {2, 1, 0}, {2, 2, 0}, {2, 3, 
          0}}, {{3, 0, 0}, {3, 1, 0}, {3, 2, 0}, {3, 3, 0}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(findit[m_, n_, b_] := 
      NIntegrate[
          If[Abs[v - w] < 
              10^\(-10\), \[IndentingNewLine]\(-2\)\ \((1 - v)\)\^\(2\ \
\((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \((m\ \((\(-1\) + v)\) + 
                  b\ v)\)\ \((n\ \((\(-1\) + v)\) + 
                  b\ v)\), \[IndentingNewLine]\(\((\(-2\) + v + w)\)\ \((\((1 \
- v)\)\^b\ v\^m - \((1 - w)\)\^b\ w\^m)\)\ \((\((1 - v)\)\^b\ v\^n - \((1 - \
w)\)\^b\ w\^n)\)\)\/\(\@\(1 - v\)\ \@\(1 - w\)\ \((v - w)\)\^2\)\
\[IndentingNewLine]], {v, 0, 1}, {w, 0, v, 1}] - 0\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(findit[0, 0,  .1]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ploss" \(\(:\)\(\ \)\) 
      "Numerical integration stopping due to loss of precision. Achieved \
neither the requested PrecisionGoal nor AccuracyGoal; suspect one of the \
following: highly oscillatory integrand or the true value of the integral is \
0. If your integrand is oscillatory try using the option Method->Oscillatory \
in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(\(-1.4971182313320335`\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Block[{eps = 0.000001}, 
      Plot3D[If[
            Abs[v - w] < 
              10^\(-8\), \[IndentingNewLine]\(-2\)\ \((1 - v)\)\^\(2\ \((\(-1\
\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \((m\ \((\(-1\) + v)\) + 
                  b\ v)\)\ \((n\ \((\(-1\) + v)\) + 
                  b\ v)\), \[IndentingNewLine]\(\((\(-2\) + v + w)\)\ \((\((1 \
- v)\)\^b\ v\^m - \((1 - w)\)\^b\ w\^m)\)\ \((\((1 - v)\)\^b\ v\^n - \((1 - \
w)\)\^b\ w\^n)\)\)\/\(\@\(1 - v\)\ \@\(1 - w\)\ \((v - w)\)\^2\)\
\[IndentingNewLine]] /. {b \[Rule] 0.15, m \[Rule] 0, n \[Rule] 0}, {w, eps, 
          1 - eps}, {v, eps, 1 - eps}, PlotPoints \[Rule] 35]]\)], "Input",
  FontSize->14]
}, Open  ]],

Cell[CellGroupData[{

Cell["My Pole subtraction", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(phi[n_, x_, b_] = x^n\ \((1 - x)\)^b\)], "Input"],

Cell[BoxData[
    \(\((1 - x)\)\^b\ x\^n\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = \((\((2 - v - w)\)\ \((phi[m, v, b]\ phi[n, w, b] + 
                  phi[m, w, b]\ phi[n, v, b])\))\)/\((2\ Sqrt[1 - v] 
              Sqrt[1 - w] \((w - v)\)^2)\) - \((\ 
            phi[m, v, b]\ phi[n, v, b] + \ 
              phi[m, w, b]\ phi[n, w, b])\)/\((w - v)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\((2 - v - w)\)\ \((\((1 - v)\)\^b\ v\^n\ \((1 - w)\)\^b\ w\^m + \((1 \
- v)\)\^b\ v\^m\ \((1 - w)\)\^b\ w\^n)\)\)\/\(2\ \@\(1 - v\)\ \@\(1 - w\)\ \
\((\(-v\) + w)\)\^2\) - \(\((1 - v)\)\^\(2\ b\)\ v\^\(m + n\) + \((1 - w)\)\^\
\(2\ b\)\ w\^\(m + n\)\)\/\((\(-v\) + w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\((1 - v)\)\^\(\(-\(1\/2\)\) + b\)\ \((1 - w)\)\^\(\(-\(1\/2\)\) \
+ b\)\ \((\(-2\) + v + w)\)\ \((v\^n\ w\^m + v\^m\ w\^n)\) + 
            2\ \((\((1 - v)\)\^\(2\ b\)\ v\^\(m + n\) + \((1 - w)\)\^\(2\ b\)\
\ w\^\(m + n\))\)\)\/\(2\ \((v - w)\)\^2\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = 
      Normal[Simplify[Series[integrand, {w, v, 0}]]]/2 + 
        Normal[Simplify[Series[integrand, {v, w, 0}]]]/2\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/8\)\)\ \((1 - v)\)\^\(2\ \((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + \
n\)\ \((4\ m\ \((\(-1\) + v)\)\ \((n\ \((\(-1\) + v)\) + b\ v)\) + 
            v\ \((4\ b\ n\ \((\(-1\) + v)\) - v + 4\ b\^2\ v)\))\) - 
      1\/8\ \((1 - w)\)\^\(2\ \((\(-1\) + b)\)\)\ w\^\(\(-2\) + m + n\)\ \((4\
\ m\ \((\(-1\) + w)\)\ \((n\ \((\(-1\) + w)\) + b\ w)\) + 
            w\ \((4\ b\ n\ \((\(-1\) + w)\) - w + 
                  4\ b\^2\ w)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand - pole]\)], "Input"],

Cell[BoxData[
    \(1\/8\ \((\((1 - v)\)\^\(2\ \((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \
\((4\ m\ \((\(-1\) + v)\)\ \((n\ \((\(-1\) + v)\) + b\ v)\) + 
                v\ \((4\ b\ n\ \((\(-1\) + v)\) - v + 
                      4\ b\^2\ v)\))\) - \(4\ \((1 - v)\)\^\(\(-\(1\/2\)\) + \
b\)\ \((1 - w)\)\^\(\(-\(1\/2\)\) + b\)\ \((\(-2\) + v + w)\)\ \((v\^n\ w\^m \
+ v\^m\ w\^n)\)\)\/\((v - w)\)\^2 - \(8\ \((\((1 - v)\)\^\(2\ b\)\ v\^\(m + n\
\) + \((1 - w)\)\^\(2\ b\)\ w\^\(m + n\))\)\)\/\((v - w)\)\^2 + \((1 - w)\)\^\
\(2\ \((\(-1\) + b)\)\)\ w\^\(\(-2\) + m + n\)\ \((4\ m\ \((\(-1\) + 
                      w)\)\ \((n\ \((\(-1\) + w)\) + b\ w)\) + 
                w\ \((4\ b\ n\ \((\(-1\) + w)\) - w + 
                      4\ b\^2\ w)\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Limit[integrand - pole, w \[Rule] v]]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = Map[Integrate[#, {w, 0, 1}, {v, 0, 1}] &, pole]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((Gamma[\(-1\) + 
                2\ b]\ \((\((\(-1\) + 4\ b\^2 + 4\ m\ n + 
                      4\ b\ \((m + n)\))\)\ Gamma[
                    1 + m + n]\ Gamma[\(-2\) + 2\ b + m + n]\ Gamma[\(-1\) + 
                      2\ b + m + n] - 
                4\ \((\((2\ m\ n + b\ \((m + n)\))\)\ Gamma[
                          m + n]\ Gamma[\(-2\) + 2\ b + m + n] - 
                      m\ n\ Gamma[\(-1\) + m + n]\ Gamma[\(-1\) + 2\ b + m + 
                            n])\)\ Gamma[
                    2\ b + m + n])\))\)/\((4\ Gamma[\(-2\) + 2\ b + m + 
                n]\ Gamma[\(-1\) + 2\ b + m + n]\ Gamma[
              2\ b + m + n])\)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(findit[m_, n_, b_] := 
      NIntegrate[
          If[Abs[v - w] < 
              10^\(-10\), \[IndentingNewLine]0, \[IndentingNewLine]1\/8\ \
\((\((1 - v)\)\^\(2\ \((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \((4\ m\ \
\((\(-1\) + v)\)\ \((n\ \((\(-1\) + v)\) + b\ v)\) + 
                        v\ \((4\ b\ n\ \((\(-1\) + v)\) - v + 
                              4\ b\^2\ v)\))\) - \(4\ \((1 - \
v)\)\^\(\(-\(1\/2\)\) + b\)\ \((1 - w)\)\^\(\(-\(1\/2\)\) + b\)\ \((\(-2\) + \
v + w)\)\ \((v\^n\ w\^m + v\^m\ w\^n)\)\)\/\((v - w)\)\^2 - \(8\ \((\((1 - v)\
\)\^\(2\ b\)\ v\^\(m + n\) + \((1 - w)\)\^\(2\ b\)\ w\^\(m + n\))\)\)\/\((v - \
w)\)\^2 + \((1 - w)\)\^\(2\ \((\(-1\) + b)\)\)\ w\^\(\(-2\) + m + n\)\ \((4\ \
m\ \((\(-1\) + w)\)\ \((n\ \((\(-1\) + w)\) + b\ w)\) + 
                        w\ \((4\ b\ n\ \((\(-1\) + w)\) - w + 
                              4\ b\^2\ w)\))\))\)\[IndentingNewLine]], {v, 0, 
            1}, {w, 0, v, 1}, Method \[Rule] GaussKronrod] - 0\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(findit[1, 1,  .45]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"ncvb" \(\(:\)\(\ \)\) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(13\) \
recursive bisections in \!\(w\) near \!\({v, w}\) = \!\({0.5`, 1.`}\)."\)], \
"Message"],

Cell[BoxData[
    \(\(-6.018110613248000228047208`9.9195*^22359\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{eps = 0.001}, 
      Plot3D[If[
            Abs[v - w] < 
              10^\(-10\), \[IndentingNewLine]0, \[IndentingNewLine]1\/8\ \
\((\((1 - v)\)\^\(2\ \((\(-1\) + b)\)\)\ v\^\(\(-2\) + m + n\)\ \((4\ m\ \
\((\(-1\) + v)\)\ \((n\ \((\(-1\) + v)\) + b\ v)\) + 
                        v\ \((4\ b\ n\ \((\(-1\) + v)\) - v + 
                              4\ b\^2\ v)\))\) - \(4\ \((1 - \
v)\)\^\(\(-\(1\/2\)\) + b\)\ \((1 - w)\)\^\(\(-\(1\/2\)\) + b\)\ \((\(-2\) + \
v + w)\)\ \((v\^n\ w\^m + v\^m\ w\^n)\)\)\/\((v - w)\)\^2 - \(8\ \((\((1 - v)\
\)\^\(2\ b\)\ v\^\(m + n\) + \((1 - w)\)\^\(2\ b\)\ w\^\(m + n\))\)\)\/\((v - \
w)\)\^2 + \((1 - w)\)\^\(2\ \((\(-1\) + b)\)\)\ w\^\(\(-2\) + m + n\)\ \((4\ \
m\ \((\(-1\) + w)\)\ \((n\ \((\(-1\) + w)\) + b\ w)\) + 
                        w\ \((4\ b\ n\ \((\(-1\) + w)\) - w + 
                              4\ b\^2\ w)\))\))\)\[IndentingNewLine]] /. {b \
\[Rule] 0.25, m \[Rule] 2, n \[Rule] 2}, {w, eps, 1 - eps}, {v, eps, 
          1 - eps}, PlotPoints \[Rule] 35]]\)], "Input",
  FontSize->14],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["My Pole subtraction-- change of variables", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(phi[n_, x_, b_] = x^n\ \((1 - x)\)^b\)], "Input"],

Cell[BoxData[
    \(\((1 - x)\)\^b\ x\^n\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(before = \((\((2 - v - w)\)\ \((phi[m, v, b]\ phi[n, w, b] + 
                  phi[m, w, b]\ phi[n, v, b])\))\)/\((2\ Sqrt[1 - v] 
              Sqrt[1 - w] \((w - v)\)^2)\) - \((\ 
            phi[m, v, b]\ phi[n, v, b] + \ 
              phi[m, w, b]\ phi[n, w, b])\)/\((w - v)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\((2 - v - w)\)\ \((\((1 - v)\)\^b\ v\^n\ \((1 - w)\)\^b\ w\^m + \((1 \
- v)\)\^b\ v\^m\ \((1 - w)\)\^b\ w\^n)\)\)\/\(2\ \@\(1 - v\)\ \@\(1 - w\)\ \
\((\(-v\) + w)\)\^2\) - \(\((1 - v)\)\^\(2\ b\)\ v\^\(m + n\) + \((1 - w)\)\^\
\(2\ b\)\ w\^\(m + n\)\)\/\((\(-v\) + w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = 
      Simplify[x\ before /. {v \[Rule] 1 - x\ y, 
            w \[Rule] 1 - x\ \((1 - y)\)}]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(2\ \((1 - 2\ y)\)\^2\ \((x\ y)\)\^\(3/2\)\ \@\(x - x\ \
y\)\)\)\((y\ \((x\ \((x\ y)\)\^b\ \((x - x\ y)\)\^b\ \((\((1 + x\ \((\(-1\) + \
y)\))\)\^n\ \((1 - x\ y)\)\^m + \((1 + x\ \((\(-1\) + y)\))\)\^m\ \((1 - x\ \
y)\)\^n)\) - 
            2\ \((\((x\ y)\)\^\(1\/2 + 2\ b\)\ \((1 - x\ y)\)\^\(m + n\)\ \
\@\(x - x\ y\) + \((1 + x\ \((\(-1\) + y)\))\)\^\(m + n\)\ \@\(x\ y\)\ \((x - \
x\ y)\)\^\(1\/2 + 2\ b\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = Normal[Simplify[Series[integrand, {y, 1/2, \(-1\)}]]]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[integrand - pole]\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(2\ \((1 - 2\ y)\)\^2\ \((x\ y)\)\^\(3/2\)\ \@\(x - x\ \
y\)\)\)\((y\ \((x\ \((x\ y)\)\^b\ \((x - x\ y)\)\^b\ \((\((1 + x\ \((\(-1\) + \
y)\))\)\^n\ \((1 - x\ y)\)\^m + \((1 + x\ \((\(-1\) + y)\))\)\^m\ \((1 - x\ \
y)\)\^n)\) - 
            2\ \((\((x\ y)\)\^\(1\/2 + 2\ b\)\ \((1 - x\ y)\)\^\(m + n\)\ \
\@\(x - x\ y\) + \((1 + x\ \((\(-1\) + y)\))\)\^\(m + n\)\ \@\(x\ y\)\ \((x - \
x\ y)\)\^\(1\/2 + 2\ b\))\))\))\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Limit[integrand - pole, y \[Rule] 1/2]]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(4\^\(-b\)\ \((1 - x\/2)\)\^\(m + n\)\ x\^\(\(-1\) + 2\ b\)\ \
\((\(-4\) + 4\ b\^2\ \((\(-2\) + x)\)\^2 + 4\ x + 
                4\ b\ \((m + n)\)\ \((\(-2\) + x)\)\ x + \((\(-1\) + 
                      4\ m\ n)\)\ x\^2)\)\)\/\((\(-2\) + x)\)\^2\)\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = Map[Integrate[#, {w, 0, 1}, {v, 0, 1}] &, pole]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(findit[m_, n_, b_] := 
      NIntegrate[
          If[Abs[y - 1/2] < 
              10^\(-10\), \[IndentingNewLine]\(-\(\(4\^\(-b\)\ \((1 - x\/2)\)\
\^\(m + n\)\ x\^\(\(-1\) + 2\ b\)\ \((\(-4\) + 4\ b\^2\ \((\(-2\) + x)\)\^2 + 
                        4\ x + 
                        4\ b\ \((m + n)\)\ \((\(-2\) + x)\)\ x + \((\(-1\) + 
                              4\ m\ n)\)\ x\^2)\)\)\/\((\(-2\) + x)\)\^2\)\), \
\[IndentingNewLine]\(1\/\(2\ \((1 - 2\ y)\)\^2\ \((x\ y)\)\^\(3/2\)\ \@\(x - \
x\ y\)\)\) \((y\ \((x\ \((x\ y)\)\^b\ \((x - x\ y)\)\^b\ \((\((1 + x\ \((\(-1\
\) + y)\))\)\^n\ \((1 - x\ y)\)\^m + \((1 + x\ \((\(-1\) + y)\))\)\^m\ \((1 - \
x\ y)\)\^n)\) - 
                      2\ \((\((x\ y)\)\^\(1\/2 + 2\ b\)\ \((1 - x\ y)\)\^\(m \
+ n\)\ \@\(x - x\ y\) + \((1 + x\ \((\(-1\) + y)\))\)\^\(m + n\)\ \@\(x\ y\)\ \
\((x - x\ y)\)\^\(1\/2 + 2\ b\))\))\))\)\[IndentingNewLine]], {x, 0, 
            1/2}, {y, 0, 1}, Method \[Rule] GaussKronrod] - 0\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(findit[0, 0,  .25]\)], "Input"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \(NIntegrate::"inum" \(\(:\)\(\ \)\) 
      "Integrand \!\(Indeterminate\) is not numerical at \!\({x, y}\) = \
\!\({0.2519893299881447`, \
0.9999999999999999999999999999999999999999999999999999999551057839117241775556\
6435650616`71.3024}\)."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"inum" \(\(:\)\(\ \)\) 
      "Integrand \!\(\"System`\"\) is not numerical at \!\({x, y}\) = \!\({x, \
y}\)."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \(NIntegrate::"inum" \(\(:\)\(\ \)\) 
      "Integrand \!\(Indeterminate\) is not numerical at \!\({x, y}\) = \
\!\({0.2519893299881447`, \
0.9999999999999999999999999999999999999999999999999999999551057839117241775556\
6435650616`71.3024}\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(NIntegrate :: \"inum\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(NIntegrate[
      x\ If[Abs[y - 1\/2] < 
            1\/10\^10, \(-\(\(4\^\(-0.25`\)\ \((1 - x\/2)\)\^\(0 + 0\)\ \
x\^\(2\ \((\(-1\) + 0.25`)\)\)\ \((\(-4\) + 
                      4\ 0.25`\^2\ \((\(-2\) + x)\)\^2 + 4\ x + 
                      4\ 0.25`\ \((0 + 0)\)\ \((\(-2\) + 
                            x)\)\ x + \((\(-1\) + 
                            4\ 0\ 0)\)\ x\^2)\)\)\/\((\(-2\) + x)\)\^2\)\), \
\(\(1\/\(2\ \@\(x\ y\)\ \((x - 2\ x\ y)\)\^2\ \@\(x - x\ y\)\)\)\((x\ \((x\ \
y)\)\^0.25`\ \((x - x\ y)\)\^0.25`\ \((\((1 + x\ \((\(-1\) + y)\))\)\^0\ \((1 \
- x\ y)\)\^0 + \((1 + x\ \((\(-1\) + y)\))\)\^0\ \((1 - x\ y)\)\^0)\) - 
              2\ \((\((x\ y)\)\^\(1\/2 + 2\ 0.25`\)\ \((1 - x\ y)\)\^\(0 + \
0\)\ \@\(x - x\ y\) + \((1 + x\ \((\(-1\) + y)\))\)\^\(0 + 0\)\ \@\(x\ y\)\ \
\((x - x\ y)\)\^\(1\/2 + 2\ 0.25`\))\))\)\)], {x, 0, 1\/2}, {y, 0, 1}, 
      Method \[Rule] GaussKronrod]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{eps = 0.001}, 
      Plot3D[x\ If[
              Abs[y - 1/2] < 
                10^\(-10\), \[IndentingNewLine]\(-\(\(4\^\(-b\)\ \((1 - x\/2)\
\)\^\(m + n\)\ x\^\(2\ \((\(-1\) + b)\)\)\ \((\(-4\) + 
                          4\ b\^2\ \((\(-2\) + x)\)\^2 + 4\ x + 
                          4\ b\ \((m + n)\)\ \((\(-2\) + 
                                x)\)\ x + \((\(-1\) + 
                                4\ m\ n)\)\ x\^2)\)\)\/\((\(-2\) + \
x)\)\^2\)\), \[IndentingNewLine]\(1\/\(2\ \@\(x\ y\)\ \((x - 2\ x\ y)\)\^2\ \
\@\(x - x\ y\)\)\) \((x\ \((x\ y)\)\^b\ \((x - x\ y)\)\^b\ \((\((1 + x\ \
\((\(-1\) + y)\))\)\^n\ \((1 - x\ y)\)\^m + \((1 + x\ \((\(-1\) + y)\))\)\^m\ \
\((1 - x\ y)\)\^n)\) - 
                    2\ \((\((x\ y)\)\^\(1\/2 + 2\ b\)\ \((1 - x\ y)\)\^\(m + \
n\)\ \@\(x - x\ y\) + \((1 + x\ \((\(-1\) + y)\))\)\^\(m + n\)\ \@\(x\ y\)\ \
\((x - x\ y)\)\^\(1\/2 + 2\ b\))\))\)\[IndentingNewLine]] /. {b \[Rule] 0.25, 
            m \[Rule] 0, n \[Rule] 0}, {x, eps, 1 - eps}, {y, eps, 1 - eps}, 
        PlotPoints \[Rule] 35]]\)], "Input",
  FontSize->14],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrals associated with variational ansatz", "Section"],

Cell["This is the integrand", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = 
      Simplify[\[IndentingNewLine]\((x + z)\)^a\ \((1 - x - z)\)^
              b\ \((1 - x)\)^c\ \((1 - z)\)^
              d\[IndentingNewLine] - \((x + z)\)^a + 
          z^c\ x^d\[IndentingNewLine] - \((1 - x - z)\)^b\ z^c\ x^
              d\[IndentingNewLine] - \((1 - x - z)\)^
              b \((\ \((1 - z)\)^d\  + \ \((1 - x)\)^c\  - \ x^d\  - \ 
                z^c)\)\[IndentingNewLine]]\)], "Input"],

Cell[BoxData[
    \(x\^d\ z\^c - 
      x\^d\ \((1 - x - z)\)\^b\ z\^c - \((x + z)\)\^a + \((1 - x)\)\^c\ \((1 \
- z)\)\^d\ \((1 - x - z)\)\^b\ \((x + z)\)\^a + \((1 - x - z)\)\^b\ \
\((\(-\((1 - x)\)\^c\) + x\^d - \((1 - z)\)\^d + z\^c)\)\)], "Output"]
}, Open  ]],

Cell["Integrate the first subtraction term", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((x + z)\)^a, {x, 0, 1}, {z, 0, 1 - x}] - 
      Integrate[z^c\ x^d, {x, 0, 1}, {z, 0, 1 - x}]\)], "Input"],

Cell[BoxData[
    \(1\/\(2 + a\) - \(Gamma[2 + c]\ Gamma[1 + d]\)\/\(\((1 + c)\)\ Gamma[3 + \
c + d]\)\)], "Output"]
}, Open  ]],

Cell["Integrate the second subtraction term", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - x - z)\)^b\ z^c\ x^d, {x, 0, 1}, {z, 0, 
        1 - x}]\)], "Input"],

Cell[BoxData[
    \(\(Gamma[1 + b]\ Gamma[1 + c]\ Gamma[1 + d]\)\/Gamma[3 + b + c + d]\)], \
"Output"]
}, Open  ]],

Cell["Under a change of variables, the third subtraction term is:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
        p^\((b + d + 1)\)\ \((1 - w)\)^b\ \((1 - w^d)\), {p, 0, 1}, {w, 0, 
          1}] + Integrate[
        p^\((b + c + 1)\)\ \((1 - w)\)^b\ \((1 - w^c)\), {p, 0, 1}, {w, 0, 
          1}]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(1 + b\) - \(Gamma[1 + b]\ Gamma[1 + c]\)\/Gamma[2 + b + \
c]\)\/\(2 + b + c\) + \(1\/\(1 + b\) - \(Gamma[1 + b]\ Gamma[1 + \
d]\)\/Gamma[2 + b + d]\)\/\(2 + b + d\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FullSimplify[%]\)], "Input"],

Cell[BoxData[
    \(\(4 + 2\ b + c + d\)\/\(\((1 + b)\)\ \((2 + b + c)\)\ \((2 + b + d)\)\) \
+ Gamma[1 + b]\ \((\(-\(Gamma[1 + c]\/Gamma[3 + b + c]\)\) - 
            Gamma[1 + d]\/Gamma[3 + b + d])\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(integrator[a_, b_, c_, d_] := 
      NIntegrate[\[IndentingNewLine]x\^d\ z\^c - 
            x\^d\ \((1 - x - z)\)\^b\ z\^c - \((x + z)\)\^a + \((1 - x)\)\^c\ \
\((1 - z)\)\^d\ \((1 - x - z)\)\^b\ \((x + z)\)\^a + \((1 - x - z)\)\^b\ \
\((\(-\((1 - x)\)\^c\) + x\^d - \((1 - z)\)\^d + 
                  z\^c)\), \[IndentingNewLine]{x, 0, 1}, {z, 0, 
            1 - x}\[IndentingNewLine]] + 
        1\/\(2 + a\) - \(Gamma[2 + c]\ Gamma[1 + d]\)\/\(\((1 + c)\)\ Gamma[3 \
+ c + d]\) + \(Gamma[1 + b]\ Gamma[1 + c]\ Gamma[1 + d]\)\/Gamma[3 + b + c + \
d] + \(4 + 2\ b + c + d\)\/\(\((1 + b)\)\ \((2 + b + c)\)\ \((2 + b + d)\)\) \
+ Gamma[1 + b]\ \((\(-\(Gamma[1 + c]\/Gamma[3 + b + c]\)\) - 
              Gamma[1 + d]\/Gamma[3 + b + d])\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{a =  .05, b =  .02}, 
      integrator[2\ a, 2\ b, 2  a\  - 2\ b\  - 1, 
        2\ a - 2\ b - 1]]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(1.3448509184087136`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{alpha =  .05, beta =  .02, eps = 10^\(-6\)}, 
      Plot3D[If[x + z < 1, 
          integrand /. {a -> 2\ alpha, b -> 2\ beta, 
              c -> 2  alpha\  - 2\ beta\  - 1, d -> 2\ alpha - 2\ beta - 1}, 
          0], {x, eps, 1}, {z, eps, 1}, PlotPoints \[Rule] 35]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - x - z)\)^b\ \((1 - x)\)^c\ \((1 - z)\)^d, {x, 0, 
        1}, {z, 0, 1 - x}]\)], "Input"],

Cell[BoxData[
    \(\(\[Integral]\_0\%1\(\((1 - x)\)\^\(1 + b + c\)\ Hypergeometric2F1[1, \
\(-d\), 2 + b, 1 - x]\) \[DifferentialD]x\)\/\(1 + b\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\((1 - u)\)^b\ u^a\ \((1 - u/2 - v)\)^c\ \((1 - u/2 + v)\)^
          d, {u, 0, 1}, {v, u/2 - 1, 1 - u/2}]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(\((1 + c)\)\ \((1 + d)\)\)\)\((2\^\(1 + a\)\ Gamma[
            1 + b]\ \((\((1 + c)\)\ Hypergeometric2F1[1, \(-c\), 
                  2 + d, \(-1\)] + \((1 + d)\)\ Hypergeometric2F1[1, \(-d\), 
                  2 + c, \(-1\)])\)\ \((\(-\(\(Gamma[
                        1 + a]\ Hypergeometric2F1[1 + a, 2 + a + b + c + d, 
                        2 + a + b, \(-1\)]\)\/Gamma[
                      2 + a + 
                        b]\)\) + \(Gamma[2 + a]\ Hypergeometric2F1[2 + a, 3 + \
a + b + c + d, 3 + a + b, \(-1\)]\)\/Gamma[3 + a + b])\))\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FullSimplify[%]\)], "Input"],

Cell[BoxData[
    \(2\^\(1 + a\)\ Gamma[
        1 + b]\ \((Gamma[1 + d]\ Hypergeometric2F1Regularized[1, \(-c\), 
              2 + d, \(-1\)] + 
          Gamma[1 + c]\ Hypergeometric2F1Regularized[1, \(-d\), 
              2 + c, \(-1\)])\)\ \((Gamma[
              1 + a]\ Hypergeometric2F1Regularized[1 + a, 2 + a + b + c + d, 
              2 + a + b, \(-1\)] - 
          Gamma[2 + a]\ Hypergeometric2F1Regularized[2 + a, 
              3 + a + b + c + d, 3 + a + b, \(-1\)])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Det[{{\((1 - v)\)/2, \((2 - u)\)/2}, {\((1 + v)\)/2, \((u - 2)\)/
            2}}]\)], "Input"],

Cell[BoxData[
    \(\(-1\) + u\/2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Solve[{u \[Equal] x + z, v \[Equal] \((1 - z)\)/\((2 - x - z)\)}, {x, 
        z}]\)], "Input"],

Cell[BoxData[
    \({{x \[Rule] \(-1\) + u + 2\ v - u\ v, 
        z \[Rule] 1 - 2\ v + u\ v}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Solve[{1 - z \[Equal] \((2 - u)\) v, 
        1 - x \[Equal] \((2 - u)\) \((1 - v)\)}, {x, z}]\)], "Input"],

Cell[BoxData[
    \({{x \[Rule] \(-1\) + u + 2\ v - u\ v, 
        z \[Rule] 1 - 2\ v + u\ v}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Solve[{1 - z \[Equal] \((2 - u)\) v, 
        1 - x \[Equal] \((2 - u)\) \((1 - v)\)}, {u, v}]\)], "Input"],

Cell[BoxData[
    \({{u \[Rule] x + z, 
        v \[Rule] \(\(-1\) + z\)\/\(\(-2\) + x + z\)}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Routines for Gauss-Jacobi integration", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(jacobroot[
          1, _, _] := {0};\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(jacobroot[n_, a_, b_] := 
      Block[\[IndentingNewLine]{list = Append[jacobroot[n - 1, a, b], 1], 
          last = \(-1\)}, \[IndentingNewLine]Table[
          x /. FindRoot[
              JacobiP[n, a, b, x] \[Equal] 0, {x, last, 
                last = list[\([i]\)]}], {i, 
            n}]\[IndentingNewLine]]\)}], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(list\)\" is similar to \
existing symbol \"\!\(List\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(last\)\" is similar to \
existing symbols \!\({Last, list}\)."\)], "Message"]
}, Open  ]],

Cell["\<\
This was a little tricky.  I got the formula for normalization from \
the subroutine in numerical recipes.  This however, does not match the \
general expression for the inner product Equation (4.5.9) of NR.\
\>", "Text"],

Cell[BoxData[
    \(weights[n_, a_, b_] := 
      Block[{norm = 
            2\ ^\((a + b)\) 
              Gamma[a + n]\ Gamma[
                b + n] \((2\ n + a + b)\)/\((\ 
                  Gamma[n + 1]*\ Gamma[n + a + b + 1])\)\ }, 
        Map[{#, norm/\((D[JacobiP[n, a, b, x], x] JacobiP[n - 1, a, b, x])\) /. 
                x \[Rule] #} &, jacobroot[n, a, b]]]\)], "Input"],

Cell[BoxData[
    \(jacint[f_, x_, lower_, upper_, a_, b_, n_] := 
      Block[{w = weights[n, a, b]}, \((upper - lower)\)/2*
          Apply[Plus, 
            Map[#[\([2]\)]*f /. 
                  x -> \((#[\([1]\)] + 1)\)*\((upper - lower)\)/2 + lower &, 
              w]]]\)], "Input"],

Cell["Testing:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(weights[5, 0, 0]\)], "Input"],

Cell[BoxData[
    \({{\(-0.9061798460726176`\), 
        0.23692688393049283`}, {\(-0.5384692147813791`\), 
        0.47862891269835484`}, {0.`, 
        0.5688888888888888`}, {0.5384692147813791`, 
        0.47862891269835484`}, {0.9061798460726176`, 
        0.23692688393049283`}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(<< NumericalMath`GaussianQuadrature`\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(GaussianQuadratureWeights[5, \(-1\), 1]\)], "Input"],

Cell[BoxData[
    \({{\(-0.906179845938664`\), 
        0.2369268850561894`}, {\(-0.5384693101056831`\), 
        0.47862867049936625`}, {0, 
        0.5688888888888889`}, {0.5384693101056831`, 
        0.47862867049936625`}, {0.906179845938664`, 
        0.2369268850561894`}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Junk", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Limit[\((u^\((b + 1/2)\) - 
            v^\((b + 1/2)\))\)*\((u^\((b - 1/2)\) - 
              v^\((b - 1/2)\))\)/\((u - v)\)^2, u \[Rule] v]\)], "Input"],

Cell[BoxData[
    \(1\/4\ \((\(-1\) + 4\ b\^2)\)\ v\^\(2\ \((\(-1\) + b)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(f[u_, v_, b_] := 
      If[u \[Equal] v, 
        1\/4\ \((\(-1\) + 
              4\ b\^2)\)\ v\^\(2\ \((\(-1\) + b)\)\), \((u^\((b + 1/2)\) - 
              v^\((b + 1/2)\))\)*\((u^\((b - 1/2)\) - 
                v^\((b - 1/2)\))\)/\((u - v)\)^2]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{eps = 0.01}, 
      Plot3D[f[u, v, 0.35], {u, eps, 1}, {v, eps, 1}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = 
      Simplify[Integrate[
          Exp[\(-t1\)\ x - t2 \((1 - x)\) - u1\ y - u2\ \((1 - y)\)] 
            Cos[z \((x - y)\)], {x, 0, 1}, {y, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(\((\[ExponentialE]\^\(\(-t1\) - t2 - u1 - u2\)\ \((\((\[ExponentialE]\^\
\(t1 + u1\) + \[ExponentialE]\^\(t2 + u2\))\)\ \((t1\ \((u1 - u2)\) + 
                    t2\ \((\(-u1\) + u2)\) + 
                    z\^2)\) + \((\[ExponentialE]\^\(t2 + u1\) + \
\[ExponentialE]\^\(t1 + u2\))\)\ \((\(-t1\)\ u1 + t2\ u1 + t1\ u2 - t2\ u2 - 
                    z\^2)\)\ Cos[
                  z] + \((\[ExponentialE]\^\(t2 + u1\) - \
\[ExponentialE]\^\(t1 + u2\))\)\ \((\(-t1\) + t2 + u1 - u2)\)\ z\ Sin[
                  z])\))\)/\((\((u1 - u2 - \[ImaginaryI]\ z)\)\ \((u1 - 
              u2 + \[ImaginaryI]\ z)\)\ \((\[ImaginaryI]\ t1 - \[ImaginaryI]\ \
t2 + z)\)\ \((\(-\[ImaginaryI]\)\ t1 + \[ImaginaryI]\ t2 + 
              z)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Factor, Apart[z\ expr, z]]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-t1\) - t2 - u1 - u2\)\ \((u1 - u2)\)\ \((\
\[ImaginaryI]\ \[ExponentialE]\^\(t1 + u1\) + \[ImaginaryI]\ \
\[ExponentialE]\^\(t2 + u2\) - \[ImaginaryI]\ \[ExponentialE]\^\(t2 + u1\)\ \
Cos[z] - \[ImaginaryI]\ \[ExponentialE]\^\(t1 + u2\)\ Cos[z] + \
\[ExponentialE]\^\(t2 + u1\)\ Sin[z] - \[ExponentialE]\^\(t1 + u2\)\ \
Sin[z])\)\)\/\(2\ \((t1 - t2 + u1 - u2)\)\ \((u1 - u2 + \[ImaginaryI]\ z)\)\) \
+ \(\[ExponentialE]\^\(\(-t1\) - t2 - u1 - u2\)\ \((u1 - u2)\)\ \((\(-\
\[ImaginaryI]\)\ \[ExponentialE]\^\(t1 + u1\) - \[ImaginaryI]\ \
\[ExponentialE]\^\(t2 + u2\) + \[ImaginaryI]\ \[ExponentialE]\^\(t2 + u1\)\ \
Cos[z] + \[ImaginaryI]\ \[ExponentialE]\^\(t1 + u2\)\ Cos[z] + \
\[ExponentialE]\^\(t2 + u1\)\ Sin[z] - \[ExponentialE]\^\(t1 + u2\)\ \
Sin[z])\)\)\/\(2\ \((t1 - t2 + u1 - u2)\)\ \((u1 - u2 - \[ImaginaryI]\ z)\)\) \
+ \(\[ExponentialE]\^\(\(-t1\) - t2 - u1 - u2\)\ \((t1 - t2)\)\ \((\
\[ImaginaryI]\ \[ExponentialE]\^\(t1 + u1\) + \[ImaginaryI]\ \
\[ExponentialE]\^\(t2 + u2\) - \[ImaginaryI]\ \[ExponentialE]\^\(t2 + u1\)\ \
Cos[z] - \[ImaginaryI]\ \[ExponentialE]\^\(t1 + u2\)\ Cos[z] - \
\[ExponentialE]\^\(t2 + u1\)\ Sin[z] + \[ExponentialE]\^\(t1 + u2\)\ \
Sin[z])\)\)\/\(2\ \((t1 - t2 + u1 - u2)\)\ \((t1 - t2 + \[ImaginaryI]\ z)\)\) \
+ \(\[ExponentialE]\^\(\(-t1\) - t2 - u1 - u2\)\ \((t1 - t2)\)\ \((\(-\
\[ImaginaryI]\)\ \[ExponentialE]\^\(t1 + u1\) - \[ImaginaryI]\ \
\[ExponentialE]\^\(t2 + u2\) + \[ImaginaryI]\ \[ExponentialE]\^\(t2 + u1\)\ \
Cos[z] + \[ImaginaryI]\ \[ExponentialE]\^\(t1 + u2\)\ Cos[z] - \
\[ExponentialE]\^\(t2 + u1\)\ Sin[z] + \[ExponentialE]\^\(t1 + u2\)\ \
Sin[z])\)\)\/\(2\ \((t1 - t2 + u1 - u2)\)\ \((t1 - t2 - \[ImaginaryI]\ z)\)\)\
\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[t1^\((\(-g1\) - 1)\) z\ expr, {t1, 0, Infinity}]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[z\ expr, {z, 0, Infinity}]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      Exp[I\ u\ x - I\ v\ y] \((x^a - y^a)\)/\((x - y)\), {x, 0, 1}, {y, 0, 
        1}]\)], "Input"],

Cell[BoxData[
    \(\[Integral]\_0\%1\(\[Integral]\_0\%1\(\( \[ExponentialE]\^\(\
\[ImaginaryI]\ u\ x - \[ImaginaryI]\ v\ y\)\ \((x\^a - y\^a)\)\)\/\(x - 
                y\)\) \[DifferentialD]y \[DifferentialD]x\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = 
      Simplify[\((x^a1\ \((1 - x)\)^a2 - 
                y^a1 \((1 - y)\)^a2)\)*\((x^b1\ \((1 - x)\)^b2 - 
                  y^b1\ \((1 - y)\)^b2)\)/\((x - y)\)^2 + \((x^
                    a2\ \((1 - x)\)^a1 - 
                y^a2 \((1 - y)\)^a1)\)*\((x^b2\ \((1 - x)\)^b1 - 
                  y^b2\ \((1 - y)\)^b1)\)/\((x - y)\)^2]\)], "Input"],

Cell[BoxData[
    \(\(\((\((1 - x)\)\^a2\ x\^a1 - \((1 - y)\)\^a2\ y\^a1)\)\ \((\((1 - x)\)\
\^b2\ x\^b1 - \((1 - y)\)\^b2\ y\^b1)\) + \((\((1 - x)\)\^a1\ x\^a2 - \((1 - \
y)\)\^a1\ y\^a2)\)\ \((\((1 - x)\)\^b1\ x\^b2 - \((1 - y)\)\^b1\ y\^b2)\)\)\/\
\((x - y)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FullSimplify[Limit[expr, x \[Rule] y]]\)], "Input"],

Cell[BoxData[
    \(\(\((1 - y)\)\^\(a1 + b1\)\ y\^\(a2 + b2\)\ \((a2\ \((\(-1\) + y)\) + \
a1\ y)\)\ \((b2\ \((\(-1\) + y)\) + b1\ y)\) + \((1 - y)\)\^\(a2 + b2\)\ \
y\^\(a1 + b1\)\ \((a1\ \((\(-1\) + y)\) + a2\ y)\)\ \((b1\ \((\(-1\) + y)\) + \
b2\ y)\)\)\/\(\((\(-1\) + y)\)\^2\ y\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NIntegrate[
      If[x \[Equal] 
            y, \(\((1 - y)\)\^\(a1 + b1\)\ y\^\(a2 + b2\)\ \((a2\ \((\(-1\) + \
y)\) + a1\ y)\)\ \((b2\ \((\(-1\) + y)\) + b1\ y)\) + \((1 - y)\)\^\(a2 + \
b2\)\ y\^\(a1 + b1\)\ \((a1\ \((\(-1\) + y)\) + a2\ y)\)\ \((b1\ \((\(-1\) + \
y)\) + b2\ y)\)\)\/\(\((\(-1\) + y)\)\^2\ y\^2\), \(\((\((1 - x)\)\^a2\ x\^a1 \
- \((1 - y)\)\^a2\ y\^a1)\)\ \((\((1 - x)\)\^b2\ x\^b1 - \((1 - y)\)\^b2\ \
y\^b1)\) + \((\((1 - x)\)\^a1\ x\^a2 - \((1 - y)\)\^a1\ y\^a2)\)\ \((\((1 - \
x)\)\^b1\ x\^b2 - \((1 - y)\)\^b1\ y\^b2)\)\)\/\((x - y)\)\^2] /. {a1 \[Rule] 
            0.25, a2 \[Rule] 0.25, b1 \[Rule] 0.25, b2 \[Rule] 0.25}, {x, 0, 
        1/2 + I, 1}, {y, 0, 1/2 - I, 1}]\)], "Input"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \(NIntegrate::"slwcon" \(\(:\)\(\ \)\) 
      "Numerical integration converging too slowly; suspect one of the \
following: singularity, value of the integration being 0, oscillatory \
integrand, or insufficient WorkingPrecision. If your integrand is oscillatory \
try using the option Method->Oscillatory in NIntegrate."\)], "Message"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{eps = 10^\(-3\)}, 
      Plot3D[If[
            x \[Equal] 
              y, \(\((1 - y)\)\^\(a1 + b1\)\ y\^\(a2 + b2\)\ \((a2\ \((\(-1\) \
+ y)\) + a1\ y)\)\ \((b2\ \((\(-1\) + y)\) + b1\ y)\) + \((1 - y)\)\^\(a2 + \
b2\)\ y\^\(a1 + b1\)\ \((a1\ \((\(-1\) + y)\) + a2\ y)\)\ \((b1\ \((\(-1\) + \
y)\) + b2\ y)\)\)\/\(\((\(-1\) + y)\)\^2\ y\^2\), \(\((\((1 - x)\)\^a2\ x\^a1 \
- \((1 - y)\)\^a2\ y\^a1)\)\ \((\((1 - x)\)\^b2\ x\^b1 - \((1 - y)\)\^b2\ \
y\^b1)\) + \((\((1 - x)\)\^a1\ x\^a2 - \((1 - y)\)\^a1\ y\^a2)\)\ \((\((1 - \
x)\)\^b1\ x\^b2 - \((1 - y)\)\^b1\ y\^b2)\)\)\/\((x - y)\)\^2] /. {a1 \[Rule] 
              1 + 0.1, a2 \[Rule] 1 + 0.25, b1 \[Rule] 1 + 0.25, 
            b2 \[Rule] 1 + 0.5}, {x, eps, 1 - eps}, {y, eps, 1 - eps}, 
        PlotPoints \[Rule] 25]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{eps = 10^\(-3\), z}, 
      Plot3D[z = 
          If[x \[Equal] 
                y, \(\((1 - y)\)\^\(a1 + b1\)\ y\^\(a2 + b2\)\ \((a2\ \((\(-1\
\) + y)\) + a1\ y)\)\ \((b2\ \((\(-1\) + y)\) + b1\ y)\) + \((1 - y)\)\^\(a2 \
+ b2\)\ y\^\(a1 + b1\)\ \((a1\ \((\(-1\) + y)\) + a2\ y)\)\ \((b1\ \((\(-1\) \
+ y)\) + b2\ y)\)\)\/\(\((\(-1\) + y)\)\^2\ y\^2\), \(\((\((1 - x)\)\^a2\ \
x\^a1 - \((1 - y)\)\^a2\ y\^a1)\)\ \((\((1 - x)\)\^b2\ x\^b1 - \((1 - \
y)\)\^b2\ y\^b1)\) + \((\((1 - x)\)\^a1\ x\^a2 - \((1 - y)\)\^a1\ y\^a2)\)\ \
\((\((1 - x)\)\^b1\ x\^b2 - \((1 - y)\)\^b1\ y\^b2)\)\)\/\((x - y)\)\^2] /. \
{x \[Rule] \(-x\)\ Exp[\ \(-I\)\ Pi\ x], y \[Rule] \(-y\)\ Exp[I\ Pi\ y], 
              a1 \[Rule] 0.1, a2 \[Rule] 0.25, b1 \[Rule] 0.25, 
              b2 \[Rule] 0.5}; {Abs[z], 
          Hue[ArcTan[Re[z], Im[z]]/\((2\ Pi)\)]}, {x, eps, 1 - eps}, {y, eps, 
          1 - eps}, PlotPoints \[Rule] 35]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = 
      Simplify[Integrate[x^a\ \((1 - x)\)^b\ Cos[c\ x], {x, 0, 1}, 
          Assumptions \[Rule] {a > \(-1\), b > \(-1\), c > 0}]]\)], "Input"],

Cell[BoxData[
    \(\(Gamma[1 + a]\ Gamma[1 + b]\ HypergeometricPFQ[{1\/2 + a\/2, 1 + \
a\/2}, {1\/2, 1 + a\/2 + b\/2, 3\/2 + a\/2 + b\/2}, \
\(-\(c\^2\/4\)\)]\)\/Gamma[2 + a + b]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      Do[expr /. {a \[Rule]  .1, b \[Rule] \(- .2\)}, {c, 0, 
          10,  .1}]]\)], "Input"],

Cell[BoxData[
    \({0.8400000000000034`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
        TrigExpand[
          Cos[m\ Pi\ w\ p]\ Cos[n\ Pi\ \((1 - w)\)\ p] + 
              Cos[n\ Pi\ w\ p]\ Cos[m\ Pi\ \((1 - w)\)\ p] /. 
            w \[Rule] \((1 - z)\)/2]] /. z \[Rule] 1 - 2\ w\)], "Input"],

Cell[BoxData[
    \(2\ Cos[\(m\ p\ \[Pi]\)\/2]\ Cos[\(n\ p\ \[Pi]\)\/2]\ Cos[
          1\/2\ m\ p\ \[Pi]\ \((1 - 2\ w)\)]\ Cos[
          1\/2\ n\ p\ \[Pi]\ \((1 - 2\ w)\)] - 
      2\ Sin[\(m\ p\ \[Pi]\)\/2]\ Sin[\(n\ p\ \[Pi]\)\/2]\ Sin[
          1\/2\ m\ p\ \[Pi]\ \((1 - 2\ w)\)]\ Sin[
          1\/2\ n\ p\ \[Pi]\ \((1 - 2\ w)\)]\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{853, 892},
WindowMargins->{{65, Automatic}, {0, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "inchworm.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 56, 0, 64, "Subtitle"],
Cell[1798, 53, 60, 3, 71, "Subsubtitle"],

Cell[CellGroupData[{
Cell[1883, 60, 94, 3, 54, "Section"],

Cell[CellGroupData[{
Cell[2002, 67, 112, 3, 45, "Subsection"],

Cell[CellGroupData[{
Cell[2139, 74, 158, 4, 27, "Input"],
Cell[2300, 80, 98, 2, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2435, 87, 110, 2, 27, "Input"],
Cell[2548, 91, 70, 1, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2655, 97, 92, 1, 27, "Input"],
Cell[2750, 100, 78, 1, 51, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[2877, 107, 71, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[2973, 111, 172, 4, 27, "Input"],
Cell[3148, 117, 115, 2, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3300, 124, 65, 1, 27, "Input"],
Cell[3368, 127, 110, 2, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3515, 134, 108, 2, 27, "Input"],
Cell[3626, 138, 40, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3703, 144, 149, 3, 27, "Input"],
Cell[3855, 149, 106, 2, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3998, 156, 101, 2, 27, "Input"],
Cell[4102, 160, 80, 1, 44, "Output"]
}, Open  ]],
Cell[4197, 164, 65, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[4287, 169, 108, 2, 27, "Input"],
Cell[4398, 173, 40, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4487, 180, 135, 5, 31, "Subsection"],

Cell[CellGroupData[{
Cell[4647, 189, 223, 4, 27, "Input"],
Cell[4873, 195, 176, 3, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5086, 203, 91, 1, 27, "Input"],
Cell[5180, 206, 113, 2, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5330, 213, 59, 1, 27, "Input"],
Cell[5392, 216, 279, 4, 61, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5708, 225, 83, 1, 27, "Input"],
Cell[5794, 228, 166, 2, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5997, 235, 131, 2, 27, "Input"],
Cell[6131, 239, 127, 2, 46, "Output"]
}, Open  ]],
Cell[6273, 244, 634, 11, 164, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[6944, 260, 59, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[7028, 264, 63, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[7116, 268, 146, 3, 27, "Input"],
Cell[7265, 273, 63, 1, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7365, 279, 282, 6, 87, "Input"],
Cell[7650, 287, 53, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[7752, 294, 44, 0, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[7821, 298, 182, 4, 27, "Input"],
Cell[8006, 304, 128, 2, 69, "Output"]
}, Open  ]],
Cell[8149, 309, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[8271, 316, 91, 1, 27, "Input"],
Cell[8365, 319, 55, 1, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8457, 325, 49, 1, 27, "Input"],
Cell[8509, 328, 143, 2, 69, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8689, 335, 79, 1, 27, "Input"],
Cell[8771, 338, 145, 2, 69, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8953, 345, 131, 2, 27, "Input"],
Cell[9087, 349, 67, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9191, 355, 301, 6, 80, "Input"],
Cell[9495, 363, 57, 1, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9601, 370, 71, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[9697, 374, 186, 4, 27, "Input"],
Cell[9886, 380, 154, 2, 72, "Output"]
}, Open  ]],
Cell[10055, 385, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[10177, 392, 91, 1, 27, "Input"],
Cell[10271, 395, 91, 1, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10399, 401, 59, 1, 27, "Input"],
Cell[10461, 404, 205, 3, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10703, 412, 69, 1, 27, "Input"],
Cell[10775, 415, 86, 1, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10898, 421, 131, 2, 27, "Input"],
Cell[11032, 425, 110, 2, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11179, 432, 456, 9, 112, "Input"],
Cell[11638, 443, 52, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11751, 451, 46, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[11822, 455, 69, 1, 27, "Input"],
Cell[11894, 458, 54, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11985, 464, 282, 4, 59, "Input"],
Cell[12270, 470, 286, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12593, 479, 165, 3, 43, "Input"],
Cell[12761, 484, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12833, 490, 59, 1, 27, "Input"],
Cell[12895, 493, 210, 3, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13142, 501, 79, 1, 27, "Input"],
Cell[13224, 504, 182, 3, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13443, 512, 167, 3, 27, "Input"],
Cell[13613, 517, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13685, 523, 103, 2, 27, "Input"],
Cell[13791, 527, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13863, 533, 82, 1, 27, "Input"],
Cell[13948, 536, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14020, 542, 128, 2, 27, "Input"],
Cell[14151, 546, 240, 3, 43, "Output"]
}, Open  ]],
Cell[14406, 552, 570, 10, 112, "Input"],

Cell[CellGroupData[{
Cell[15001, 566, 50, 1, 27, "Input"],
Cell[15054, 569, 354, 5, 71, "Message"],
Cell[15411, 576, 395, 6, 87, "Message"],
Cell[15809, 584, 58, 1, 27, "Output"]
}, Open  ]],
Cell[15882, 588, 675, 12, 139, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16594, 605, 41, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[16660, 609, 69, 1, 27, "Input"],
Cell[16732, 612, 54, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16823, 618, 320, 5, 59, "Input"],
Cell[17146, 625, 309, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17492, 634, 52, 1, 27, "Input"],
Cell[17547, 637, 295, 4, 54, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17879, 646, 155, 3, 27, "Input"],
Cell[18037, 651, 468, 7, 80, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18542, 663, 59, 1, 27, "Input"],
Cell[18604, 666, 770, 11, 165, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19411, 682, 79, 1, 27, "Input"],
Cell[19493, 685, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19565, 691, 87, 1, 27, "Input"],
Cell[19655, 694, 696, 12, 64, "Output"]
}, Open  ]],
Cell[20366, 709, 1009, 16, 227, "Input"],

Cell[CellGroupData[{
Cell[21400, 729, 51, 1, 27, "Input"],
Cell[21454, 732, 354, 5, 71, "Message"],
Cell[21811, 739, 354, 5, 71, "Message"],
Cell[22168, 746, 221, 4, 39, "Message"],
Cell[22392, 752, 79, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22508, 758, 1072, 18, 299, "Input"],
Cell[23583, 778, 137, 3, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[23769, 787, 63, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[23857, 791, 69, 1, 27, "Input"],
Cell[23929, 794, 54, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24020, 800, 317, 5, 43, "Input"],
Cell[24340, 807, 309, 4, 51, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24686, 816, 138, 3, 27, "Input"],
Cell[24827, 821, 462, 7, 86, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25326, 833, 93, 1, 27, "Input"],
Cell[25422, 836, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25494, 842, 59, 1, 27, "Input"],
Cell[25556, 845, 462, 7, 86, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26055, 857, 81, 1, 27, "Input"],
Cell[26139, 860, 294, 5, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26470, 870, 87, 1, 27, "Input"],
Cell[26560, 873, 35, 1, 27, "Output"]
}, Open  ]],
Cell[26610, 877, 996, 16, 210, "Input"],

Cell[CellGroupData[{
Cell[27631, 897, 51, 1, 27, "Input"],
Cell[27685, 900, 119, 2, 36, "Message"],
Cell[27807, 904, 218, 3, 23, "Message"],
Cell[28028, 909, 276, 5, 39, "Message"],
Cell[28307, 916, 157, 3, 23, "Message"],
Cell[28467, 921, 119, 2, 36, "Message"],
Cell[28589, 925, 218, 3, 23, "Message"],
Cell[28810, 930, 276, 5, 39, "Message"],
Cell[29089, 937, 168, 3, 23, "Message"],
Cell[29260, 942, 938, 15, 209, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[30235, 962, 1104, 19, 231, "Input"],
Cell[31342, 983, 137, 3, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[31540, 993, 63, 0, 54, "Section"],
Cell[31606, 995, 37, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[31668, 999, 446, 8, 107, "Input"],
Cell[32117, 1009, 253, 4, 29, "Output"]
}, Open  ]],
Cell[32385, 1016, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[32462, 1020, 138, 2, 27, "Input"],
Cell[32603, 1024, 116, 2, 46, "Output"]
}, Open  ]],
Cell[32734, 1029, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[32812, 1033, 106, 2, 27, "Input"],
Cell[32921, 1037, 102, 2, 46, "Output"]
}, Open  ]],
Cell[33038, 1042, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[33138, 1046, 236, 5, 43, "Input"],
Cell[33377, 1053, 205, 3, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[33619, 1061, 48, 1, 27, "Input"],
Cell[33670, 1064, 215, 3, 46, "Output"]
}, Open  ]],
Cell[33900, 1070, 773, 12, 142, "Input"],

Cell[CellGroupData[{
Cell[34698, 1086, 138, 3, 27, "Input"],
Cell[34839, 1091, 354, 5, 71, "Message"],
Cell[35196, 1098, 53, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35286, 1104, 309, 5, 59, "Input"],
Cell[35598, 1111, 137, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35772, 1119, 126, 2, 27, "Input"],
Cell[35901, 1123, 160, 2, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36098, 1130, 149, 2, 27, "Input"],
Cell[36250, 1134, 600, 9, 159, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36887, 1148, 48, 1, 27, "Input"],
Cell[36938, 1151, 498, 9, 77, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37473, 1165, 115, 2, 27, "Input"],
Cell[37591, 1169, 47, 1, 41, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37675, 1175, 115, 2, 27, "Input"],
Cell[37793, 1179, 108, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37938, 1186, 127, 2, 27, "Input"],
Cell[38068, 1190, 108, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38213, 1197, 127, 2, 27, "Input"],
Cell[38343, 1201, 108, 2, 44, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[38500, 1209, 56, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[38581, 1213, 444, 9, 107, "Input"],
Cell[39028, 1224, 181, 3, 23, "Message"],
Cell[39212, 1229, 185, 3, 39, "Message"]
}, Open  ]],
Cell[39412, 1235, 231, 4, 50, "Text"],
Cell[39646, 1241, 385, 8, 59, "Input"],
Cell[40034, 1251, 291, 6, 59, "Input"],
Cell[40328, 1259, 24, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[40377, 1263, 49, 1, 27, "Input"],
Cell[40429, 1266, 297, 6, 27, "Output"]
}, Open  ]],
Cell[40741, 1275, 69, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[40835, 1280, 72, 1, 27, "Input"],
Cell[40910, 1283, 291, 6, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[41250, 1295, 23, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[41298, 1299, 174, 3, 27, "Input"],
Cell[41475, 1304, 90, 1, 44, "Output"]
}, Open  ]],
Cell[41580, 1308, 282, 6, 64, "Input"],

Cell[CellGroupData[{
Cell[41887, 1318, 108, 2, 27, "Input"],
Cell[41998, 1322, 137, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42172, 1330, 191, 4, 27, "Input"],
Cell[42366, 1336, 766, 12, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43169, 1353, 63, 1, 27, "Input"],
Cell[43235, 1356, 1745, 25, 171, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45017, 1386, 91, 1, 27, "Input"],
Cell[45111, 1389, 42, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45190, 1395, 69, 1, 27, "Input"],
Cell[45262, 1398, 42, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45341, 1404, 132, 3, 27, "Input"],
Cell[45476, 1409, 228, 3, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45741, 1417, 377, 7, 43, "Input"],
Cell[46121, 1426, 279, 4, 67, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46437, 1435, 71, 1, 27, "Input"],
Cell[46511, 1438, 302, 4, 50, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46850, 1447, 732, 11, 189, "Input"],
Cell[47585, 1460, 354, 5, 71, "Message"],
Cell[47942, 1467, 354, 5, 71, "Message"],
Cell[48299, 1474, 42, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48378, 1480, 819, 13, 208, "Input"],
Cell[49200, 1495, 137, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49374, 1503, 949, 15, 242, "Input"],
Cell[50326, 1520, 137, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50500, 1528, 172, 3, 27, "Input"],
Cell[50675, 1533, 193, 3, 55, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50905, 1541, 123, 3, 27, "Input"],
Cell[51031, 1546, 69, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51137, 1552, 243, 5, 43, "Input"],
Cell[51383, 1559, 350, 6, 80, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

