(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     70221,       1380]*)
(*NotebookOutlinePosition[     71172,       1410]*)
(*  CellTagsIndexPosition[     71128,       1406]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Hamiltonian matrix operator", "Title"],

Cell["\<\
Brett van de Sande
August 1, 2001\
\>", "Subsubtitle"],

Cell["I changed selfkappa to match Simon's (incorrect) form", "Text"],

Cell[CellGroupData[{

Cell["The subroutines", "Section"],

Cell[TextData[{
  "This is the  ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^-\)\)]],
  " operator.  Thus, to get the 2 ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^+\)\)]],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^-\)\)]],
  " operator, we still must multiply by twice the sum of the momenta.\n\
makeham[sl,sr,couplings,npmax] calculates the matrix element \
\[LeftAngleBracket]sl| ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^-\)\)]],
  " |",
  "sr",
  "\[RightAngleBracket]"
}], "Text",
  FontSize->13],

Cell[BoxData[{
    \(\(zetaf[h1_, _[_, 1], 
          h2_] := {{0, \(-I\)}, {I, 0}}[\([\ 3/2 - h1[\([2]\)], \ 
            3/2 - h2[\([2]\)]\ ]\)];\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(zetaf[h1_, _[_, 2], 
          h2_] := \(-{{0, 1}, {1, 0}}[\([\ 3/2 - h1[\([2]\)], \ 
              3/2 - h2[\([2]\)]\ ]\)]\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(zetaf[h1_, _[_, \(-1\)], 
          h2_] := \(-{{0, \(-I\)}, {I, 0}}[\([\ 3/2 - h1[\([2]\)], \ 
              3/2 - h2[\([2]\)]\ ]\)]\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(zetaf[h1_, _[_, \(-2\)], 
          h2_] := {{0, 1}, {1, 0}}[\([\ 3/2 - h1[\([2]\)], \ 
            3/2 - h2[\([2]\)]\ ]\)];\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(zetafzetaf[h1_, a1_, a2_, h2_] := 
        Block[{f1, fa}, 
          Sum[zetaf[h1, a1, f1[fa, hh]]\ zetaf[f1[fa, hh], a2, 
                h2], {hh, \(-1\)/2, 1/2}]];\)\)}], "Input",
  InitializationCell->True],

Cell["\<\
These are the sums used to calculate the self-inertias.  Note, \
these definitions correspond to the case where fermion momenta are \
{1/2,3/2,...} and gluon momenta are {1,2,...}.\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[elfkappa, elfquarkquark, elfquarkglue, 
        elfglueglue];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Correct\ \(way : \ 
          Sum[1/k, {k, 1/2, 
                kl - 1}]/\((2\ Pi\ kl)\)\)*) \[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(elfkappa[kl_] := \(elfkappa[kl] = 
          If[kl > 1/2, Sum[1/k, {k, 1, kl - 1/2}]/\((2\ Pi\ kl)\), 
            0]\);\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(elfquarkquark[k1_, k2_] := \(elfquarkquark[k1, k2] = 
          Sum[If[k \[NotEqual] k1, 1/\((k1 - k)\)^2, 0], {k, 1/2, 
                k1 + k2 - 
                  1/2}]/\((4\ Pi)\)\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(elfquarkglue::usage = "\<elfquarkglue[k1,k2]:  k1 is the quark \
{1/2,3/2,...} and k2 is the link field \
{1,2,3,...}\>";\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(elfquarkglue[k1_, 
          k2_] := \(elfquarkglue[
            kl] = \((Sqrt[1 + k1/k2] - 1)\)/\((4\ Pi\ k1)\) + 
            Sum[If[k1 \[NotEqual] 
                    r, \((k1 + 2\ \ k2 - 
                        r)\)/\((Sqrt[
                          k2\ \((k1 + k2 - r)\)]*\((k1 - r)\)^2)\), 0], {r, 
                  1/2, k1 + k2 - 
                    1}]/\((8\ Pi)\)\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(elfglueglue::usage = "\<This is the link-link expression from the \
c-program \"improved.c\" This is modified for periodic boundary conditions,no \
zero modes.\>";\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(elfglueglue[k1_, k2_] := 
        Block[{kt = k1 + k2}, 
          1/\((16\ Sqrt[k1\ k2])\) + 
            Sum[If[k \[NotEqual] 
                    k1, \((k1 + 
                        k)\)\ \((k2 + kt - 
                          k)\)/\((Sqrt[
                            k1\ k2\ k\ \((kt - k)\)]\ \((k1 - k)\)^2)\), 
                  0], {k, 1, kt - 1}]/\((16\ Pi)\)];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(makeham::usage = \
"\<makeham[leftstate,rightstate,{mub,muf,g2,kappas,kappaa,beta,lambda1,\
lambda2,lambda3,lambda4,lambda5},npmax]  calculates P^- where both leftstate
and rightstate are in |ket> form and npmax is the maximum number of \
particles,
both quarks and gluons, in all states of the \
basis.\>";\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(makeham[sl_, 
        sr_, {mub_, muf_, g2_, kappas_, kappaa_, beta_, lambda1_, lambda2_, 
          lambda3_, lambda4_, lambda5_}, npmax_] := 
      Block[{kl = Map[First, sl], kr = Map[First, sr], 
          hl = Map[Part[#, 2] &, sl], hr = Map[Part[#, 2] &, sr]}, 
        Which[\ \ \[IndentingNewLine]\[IndentingNewLine]Length[sl] \[Equal] 
            Length[sr], \[IndentingNewLine] (*\ 
            diagonal\ interactions\ *) \[IndentingNewLine]If[
              sl === sr, \[IndentingNewLine] (*\ 
                Self - energy\ interactions\ *) \[IndentingNewLine]muf^2/\((2\
\ kl[\([1]\)])\) + muf^2/\((2\ kl[\([\(-1\)]\)])\) + 
                Sum[mub^2/\((2\ kl[\([i]\)])\), {i, 2, Length[sl] - 1}] + 
                Sum[g2\ elfglueglue[kl[\([i]\)], kl[\([i + 1]\)]], {i, 2, 
                    Length[sl] - 2}] + 
                If[Length[sl] < 
                    npmax, \((kappaa^2 + 
                        kappas^2)\)*\((elfkappa[kl[\([1]\)]] + 
                        elfkappa[kl[\([\(-1\)]\)]])\), 0] + 
                g2\ If[Length[sl] \[Equal] 2, \ 
                    elfquarkquark[kl[\([1]\)], kl[\([2]\)]], 
                    elfquarkglue[kl[\([1]\)], kl[\([2]\)]] + 
                      elfquarkglue[kl[\([\(-1\)]\)], 
                        kl[\([\(-2\)]\)]]], \[IndentingNewLine] (*\ 
                non - diagnal\ \((only)\)\ 2 \[Rule] 
                  2\ particle\ interactions\ *) \[IndentingNewLine] (*\ 
                gluon - gluon\  \[Rule] \ 
                  gluon - gluon\ interactions\ *) \[IndentingNewLine]Sum[
                  If[Drop[sl, {i, i + 1}] === 
                      Drop[sr, {i, 
                          i + 1}], \[IndentingNewLine] (*Instantaneous\ gluon \
- gluon\  \[Rule] \ gluon - 
                          gluon\ normal\ interaction\ \
*) \[IndentingNewLine]If[
                      hl[\([i]\)] \[Equal] hr[\([i]\)]\  && \ 
                        hl[\([i + 1]\)] \[Equal] 
                          hr[\([i + 1]\)], \(-g2\)*\((kl[\([i]\)] + 
                            kr[\([i]\)])\)*\((kl[\([i + 1]\)] + 
                              kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                                kl[\([i]\)]*kr[\([i]\)]*kl[\([i + 1]\)]*
                                  kr[\([i + 1]\)]] \((kl[\([i]\)] - 
                                    kr[\([i]\)])\)^2)\), 
                      0], \[IndentingNewLine]0], {i, 2, 
                    Length[sl] - 2}] + \[IndentingNewLine]If[
                  Length[sl] > 2, \[IndentingNewLine] (*\ 
                    quark - gluon\  \[Rule] \ 
                      quark - gluon\ interaction\ *) \[IndentingNewLine]If[
                      Drop[sl, 2] === Drop[sr, 2]\  && \ 
                        hl[\([2]\)] \[Equal] hr[\([2]\)]\  && \ 
                        Rest[sl[\([1]\)]] === 
                          Rest[sr[\([1]\)]], \(-g2\)\ \((kl[\([2]\)] + 
                              kr[\([2]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([2]\)]\ kr[\([2]\)]]\ \((kl[\([1]\)] - 
                                    kr[\([1]\)])\)^2)\), 
                      0] + \[IndentingNewLine] (*\ 
                      antiquark - gluon\  \[Rule] \ 
                        antiquark - 
                          gluon\ interaction\ *) \[IndentingNewLine]If[
                      Drop[sl, \(-2\)] === 
                          Drop[sr, \(-2\)]\  && \ \ hl[\([\(-2\)]\)] \[Equal] 
                          hr[\([\(-2\)]\)]\  && 
                        Rest[sl[\([\(-1\)]\)]] === 
                          Rest[sr[\([\(-1\)]\)]], \(-g2\)\ \
\((kl[\([\(-2\)]\)] + kr[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0], \[IndentingNewLine] (*\ 
                    quark - quark\  \[Rule] \ 
                      quark - quark\ interaction\ *) \[IndentingNewLine]If[\ 
                    Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]] && \ 
                      Rest[sl[\([2]\)]] === 
                        Rest[sr[\([2]\)]], \[IndentingNewLine]\(-g2\)/\((4\ \
Pi\ \((kl[\([1]\)] - kr[\([1]\)])\)^2)\), 
                    0]\[IndentingNewLine]]\[IndentingNewLine]] + \
\[IndentingNewLine] (*\ 
              Both\ diagonal\ and\ non - 
                diagonal\ interactions\ *) \[IndentingNewLine]If[
              Length[sl] > 2, \[IndentingNewLine] (*\ 
                quark - gluon\  \[Rule] \ 
                  quark - gluon\ interactions\ *) \[IndentingNewLine]If[
                  Drop[sl, 2] === Drop[sr, 2], 
                  If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                      kappas^2/\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\), 0] + 
                    kappaa\ kappas\ \((zetaf[sl[\([1]\)], sl[\([2]\)], 
                              sr[\([1]\)]] + 
                            zetaf[sl[\([1]\)], sr[\([2]\)], 
                              sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\) + 
                    kappaa^2\ zetafzetaf[sl[\([1]\)], sl[\([2]\)], 
                          sr[\([2]\)], 
                          sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\), 
                  0] + \[IndentingNewLine] (*\ 
                  antiuark - gluon\  \[Rule] \ 
                    antiquark - gluon\ interactions\ *) \[IndentingNewLine]If[
                  Drop[sl, \(-2\)] === Drop[sr, \(-2\)], 
                  If[Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                      kappas^2/\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\), 0] - 
                    kappaa\ kappas\ Conjugate[
                          zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                              sl[\([\(-1\)]\)]] + 
                            zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                              sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) + 
                    kappaa^2\ Conjugate[
                          zetafzetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                            sl[\([\(-2\)]\)], 
                            sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\), 0], 
              0] + \[IndentingNewLine] (*\ 
              gluon - gluon\  \[Rule] \ 
                gluon - gluon\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i, i + 1}] === 
                  Drop[sr, {i, 
                      i + 1}]\ , \[IndentingNewLine] (*Instantaneous\ \
annihilation\ interaction\ *) \[IndentingNewLine]If[\ 
                    hl[\([i]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                      hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\), 
                    g2*\((kl[\([i]\)] - 
                          kl[\([i + 1]\)])\)*\((kr[\([i]\)] - 
                            kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                              kl[\([i]\)]*kr[\([i]\)]*kl[\([i + 1]\)]*
                                kr[\([i + 1]\)]] \((kl[\([i]\)] + 
                                  kl[\([i + 1]\)])\)^2)\), 
                    0] + \[IndentingNewLine] (*\ 
                    Effective\ potential\ terms\ *) \[IndentingNewLine]If[
                    hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                      hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                      Abs[hl[\([i]\)]] \[NotEqual] 
                        Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 
                                    1]\)]\ kl[\([i]\)]\ kl[\([i + 1]\)]])\), 
                    0] + If[
                    hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                      hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                      hl[\([i]\)] \[Equal] hr[\([i]\)], 
                    lambda1/\((4\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 
                                    1]\)]\ kl[\([i]\)]\ kl[\([i + 1]\)]])\), 
                    0] + If[
                    hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                      hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                      Abs[hl[\([i]\)]] \[Equal] Abs[hr[\([i]\)]], 
                    lambda2/\((8\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 
                                    1]\)]\ kl[\([i]\)]\ kl[\([i + 1]\)]])\), 
                    0] + If[\((hl[\([i]\)] \[Equal] \(-hl[\([i + 
                                    1]\)]\)\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                          Abs[hl[\([i]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]])\) || \((hl[\([i]\)] \[Equal] 
                            hr[\([i]\)]\  && \ 
                          hr[\([i + 1]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                          Abs[hl[\([i]\)]] \[NotEqual] 
                            Abs[hl[\([i + 1]\)]])\), 
                    lambda4/\((8\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 
                                    1]\)]\ kl[\([i]\)]\ kl[\([i + 1]\)]])\), 
                    0], 0], {i, 2, 
                Length[sl] - 
                  2}], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] + 1, \[IndentingNewLine] (*\ 
            Yukawa\ quark\  \[Rule] \ 
              quark - gluon\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, 2] === Drop[sr, 1], 
              If[Rest[sl[\([1]\)]] === 
                    Rest[sr[\([1]\)]], \(-kappas\)\ muf\ \((kl[\([1]\)] + 
                          kr[\([1]\)])\)/\((4\ \ Sqrt[
                            Pi\ kl[\([2]\)]]\ kl[\([1]\)]\ kr[\([1]\)])\), 
                  0] + muf\ kappaa\ Sqrt[
                    kl[\([2]\)]]\ zetaf[sl[\([1]\)], sl[\([2]\)], 
                      sr[\([1]\)]]/\((4\ \ Sqrt[
                          Pi]\ kr[\([1]\)]\ kl[\([1]\)])\), 
              0] + \[IndentingNewLine] (*\ 
              Yukawa\ antiquark\  \[Rule] \ 
                antiquark - gluon\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, \(-2\)] === Drop[sr, \(-1\)], 
              If[Rest[sl[\([\(-1\)]\)]] === 
                    Rest[sr[\([\(-1\)]\)]], \
\(-kappas\)\ \ muf\ \ \((kl[\([\(-1\)]\)] + \ 
                          kr[\([\(-1\)]\)])\)/\((4\ Sqrt[
                            Pi\ kl[\([\(-2\)]\)]]\ kl[\([\(-1\)]\)]\ \
kr[\([\(-1\)]\)]\ )\), 0] - 
                kappaa\ \ muf\ Sqrt[
                    kr[\([\(-1\)]\)] - kl[\([\(-1\)]\)]]\ \ Conjugate[
                      zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                        sl[\([\(-1\)]\)]]]/\((4\ Sqrt[
                          Pi]\ kl[\([\(-1\)]\)]\ kr[\([\(-1\)]\)]\ )\)\ , \
\[IndentingNewLine]0], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] + 2, \[IndentingNewLine] (*\ 
            quark\  \[Rule] \ 
              quark - glue - glue\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, 3] === Drop[sr, 1], 
              If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                  kappas^2\ /\((8\ Pi\ Sqrt[
                          kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([1]\)] + 
                            kl[\([2]\)])\))\), 0] + 
                kappaa\ kappas\ \((zetaf[sl[\([1]\)], sl[\([3]\)], 
                          sr[\([1]\)]] - 
                        zetaf[sl[\([1]\)], sl[\([2]\)], 
                          sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                          kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([3]\)] - 
                            kr[\([1]\)])\))\) + 
                kappaa^2\ zetafzetaf[sl[\([1]\)], sl[\([2]\)], sl[\([3]\)], 
                      sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                          kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([3]\)] - 
                            kr[\([1]\)])\))\) + 
                If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]]\  && \ 
                    hl[\([2]\)] \[Equal] \(-hl[\([3]\)]\), \(-g2\)\ \
\((kl[\([2]\)] - kl[\([3]\)])\)/\((8\ Pi\ Sqrt[
                            kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([1]\)] - 
                                kr[\([1]\)])\)^2)\), 0], 
              0] + \[IndentingNewLine] (*\ 
              antiquark\  \[Rule] \ 
                antiquark - glue - 
                  glue\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, \(-3\)] === Drop[sr, \(-1\)], 
              If[Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                  kappas^2\ /\((8\ Pi\ Sqrt[
                          kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\), 0] + 
                kappaa\ kappas\ Conjugate[
                      zetaf[sr[\([\(-1\)]\)], sl[\([\(-3\)]\)], 
                          sl[\([\(-1\)]\)]] - 
                        zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                          sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                          kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\) - 
                kappaa^2\ Conjugate[
                      zetafzetaf[sr[\([\(-1\)]\)], sl[\([\(-3\)]\)], 
                        sl[\([\(-2\)]\)], 
                        sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                          kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\) + 
                If[Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]]\  && \ 
                    hl[\([\(-2\)]\)] \[Equal] \(-hl[\([\(-3\)]\)]\), 
                  g2\ \((kl[\([\(-3\)]\)] - 
                          kl[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                            kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0], 
              0] + \[IndentingNewLine] (*\ 
              gauge\ glue\  \[Rule] \ 
                glue - glue - glue\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i, i + 2}] === Drop[sr, {i}], 
                If[hl[\([i + 2]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                      hl[\([i]\)] \[Equal] 
                        hr[\([i]\)], \(-g2\)\ \((kl[\([i]\)] + 
                          kr[\([i]\)])\)\ \((kl[\([i + 1]\)] - 
                            kl[\([i + 2]\)])\)/\((16\ Pi\ Sqrt[
                              kl[\([i]\)]\ kr[\([i]\)]\ kl[\([i + 
                                      1]\)]\ kl[\([i + 
                                      2]\)]]\ \((kr[\([i]\)] - 
                                  kl[\([i]\)])\)^2)\), 0] + 
                  If[hl[\([i]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                      hl[\([i + 2]\)] \[Equal] hr[\([i]\)], 
                    g2\ \((kr[\([i]\)] + 
                          kl[\([i + 2]\)])\)\ \((kl[\([i]\)] - 
                            kl[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                              kl[\([i]\)]\ kr[\([i]\)]\ kl[\([i + 
                                      1]\)]\ kl[\([i + 
                                      2]\)]]\ \((kl[\([i + 2]\)] - 
                                  kr[\([i]\)])\)^2)\), 0] + 
                  If[hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                      hl[\([i]\)] \[Equal] \(-hl[\([i + 2]\)]\)\  && \ 
                      Abs[hl[\([i]\)]] \[NotEqual] 
                        Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                            kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0] + 
                  If[hr[\([i]\)] \[Equal] hl[\([i + 2]\)]\  && \ 
                      hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                      hl[\([i]\)] \[Equal] hr[\([i]\)], 
                    lambda1/\((4\ Pi\ Sqrt[
                            kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0] + 
                  If[Abs[hr[\([i]\)]] \[Equal] Abs[hl[\([i]\)]]\  && \ 
                      hl[\([i]\)] \[Equal] \(-hl[\([i + 2]\)]\)\  && \ 
                      hl[\([i + 1]\)] \[Equal] hr[\([i]\)], 
                    lambda2/\((8\ Pi\ Sqrt[
                            kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0] + 
                  If[\((hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                          hl[\([i + 2]\)] \[Equal] hr[\([i]\)] && \ 
                          Abs[hl[\([i + 1]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]])\)\  || \((hl[\([i + 
                                  2]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] hr[\([i]\)] && \ 
                          Abs[hl[\([i + 1]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]])\), 
                    lambda4/\((8\ Pi\ Sqrt[
                            kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0], 0], {i, 2, 
                Length[sr] - 
                  1}], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] - 1, \[IndentingNewLine] (*\ 
            Yukawa\ quark - gluon\  \[Rule] \ 
              quark\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, 1] === Drop[sr, 2], 
              If[Rest[sl[\([1]\)]] === 
                    Rest[sr[\([1]\)]], \(-kappas\)\ \ muf\ \ \((kl[\([1]\)] + 
                          kr[\([1]\)])\)/\((4\ Sqrt[
                            Pi\ kr[\([2]\)]]\ kl[\([1]\)]\ kr[\([1]\)])\)\ , 
                  0] + kappaa\ \ muf\ Sqrt[
                    kr[\([2]\)]]\ \ zetaf[sl[\([1]\)], sr[\([2]\)], 
                      sr[\([1]\)]]/\((4\ Sqrt[
                          Pi]\ kr[\([1]\)]\ kl[\([1]\)])\)\ , 0] +  (*\ 
              Yukawa\ antiquark - gluon\  \[Rule] \ 
                antiquark\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, \(-1\)] === Drop[sr, \(-2\)], 
              If[Rest[sl[\([\(-1\)]\)]] === 
                    Rest[sr[\([\(-1\)]\)]], \
\(-kappas\)\ \ muf\ \ \((kr[\([\(-1\)]\)] + 
                          kl[\([\(-1\)]\)])\)/\((4\ Sqrt[
                            Pi\ kr[\([\(-2\)]\)]]\ kr[\([\(-1\)]\)]\ \
kl[\([\(-1\)]\)])\)\ , 0] - 
                kappaa\ \ muf\ Sqrt[
                    kr[\([\(-2\)]\)]]\ \ Conjugate[
                      zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                        sl[\([\(-1\)]\)]]]/\((4\ Sqrt[
                          Pi]\ kr[\([\(-1\)]\)]\ kl[\([\(-1\)]\)])\)\ , 
              0], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] - 2, \[IndentingNewLine] (*\ 
            quark - glue - glue\  \[Rule] \ 
              quark\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, 1] === Drop[sr, 3], 
              If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                  kappas^2\ /\((8\ Pi\ Sqrt[
                          kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                            kr[\([2]\)])\))\), 0] + 
                kappaa\ kappas\ \((zetaf[sl[\([1]\)], sr[\([2]\)], 
                          sr[\([1]\)]] - 
                        zetaf[sl[\([1]\)], sr[\([3]\)], 
                          sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                          kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                            kr[\([2]\)])\))\) - 
                kappaa^2\ zetafzetaf[sl[\([1]\)], sr[\([3]\)], sr[\([2]\)], 
                      sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                          kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                            kr[\([2]\)])\))\) + 
                If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]]\  && \ 
                    hr[\([2]\)] \[Equal] \(-hr[\([3]\)]\), \(-g2\)\ \
\((kr[\([2]\)] - kr[\([3]\)])\)/\((8\ Pi\ Sqrt[
                            kr[\([2]\)]\ kr[\([3]\)]]\ \((kl[\([1]\)] - 
                                kr[\([1]\)])\)^2)\), 0], 
              0] + \[IndentingNewLine] (*\ 
              antiquark - glue - glue\  \[Rule] \ 
                antiquark\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, \(-1\)] === Drop[sr, \(-3\)], 
              If[Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                  kappas^2\ /\((8\ Pi\ Sqrt[
                          kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\), 0] - 
                kappaa\ kappas\ Conjugate[
                      zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                          sl[\([\(-1\)]\)]] - 
                        zetaf[sr[\([\(-1\)]\)], sr[\([\(-3\)]\)], 
                          sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                          kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) - 
                kappaa^2\ Conjugate[
                      zetafzetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                        sr[\([\(-3\)]\)], 
                        sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                          kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) + 
                If[Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]]\  && \ 
                    hr[\([\(-2\)]\)] === \(-hr[\([\(-3\)]\)]\), 
                  g2\ \((kr[\([\(-3\)]\)] - 
                          kr[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                            kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0], 
              0] + \[IndentingNewLine] (*\ 
              gauge\ glue - glue - glue\  \[Rule] \ 
                glue\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i}] === Drop[sr, {i, i + 2}], 
                If[hr[\([i + 2]\)] == \(-hr[\([i + 1]\)]\)\  && \ 
                      hr[\([i]\)] \[Equal] 
                        hl[\([i]\)], \(-g2\)\ \((kl[\([i]\)] + 
                          kr[\([i]\)])\)\ \((kr[\([i + 1]\)] - 
                            kr[\([i + 2]\)])\)/\((16\ Pi\ Sqrt[
                              kl[\([i]\)]\ kr[\([i]\)]\ kr[\([i + 
                                      1]\)]\ kr[\([i + 
                                      2]\)]]\ \((kr[\([i]\)] - 
                                  kl[\([i]\)])\)^2)\), 0] + 
                  If[hr[\([i]\)] == \(-hr[\([i + 1]\)]\)\  && \ 
                      hr[\([i + 2]\)] \[Equal] hl[\([i]\)], 
                    g2\ \((kl[\([i]\)] + 
                          kr[\([i + 2]\)])\)\ \((kr[\([i]\)] - 
                            kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                              kl[\([i]\)]\ kr[\([i]\)]\ kr[\([i + 
                                      1]\)]\ kr[\([i + 
                                      2]\)]]\ \((kr[\([i + 2]\)] - 
                                  kl[\([i]\)])\)^2)\), 0] + 
                  If[hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                      hr[\([i]\)] \[Equal] \(-hr[\([i + 2]\)]\)\  && \ 
                      Abs[hl[\([i]\)]] \[NotEqual] 
                        Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                    2]\)]\ kl[\([i]\)]])\), 0] + 
                  If[hl[\([i]\)] \[Equal] hr[\([i + 2]\)]\  && \ 
                      hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                      hl[\([i]\)] \[Equal] hr[\([i]\)], 
                    lambda1/\((4\ Pi\ Sqrt[
                            kl[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0] + 
                  If[Abs[hr[\([i]\)]] \[Equal] Abs[hl[\([i]\)]]\  && \ 
                      hr[\([i]\)] \[Equal] \(-hr[\([i + 2]\)]\)\  && \ 
                      hl[\([i]\)] \[Equal] hr[\([i + 1]\)], 
                    lambda2/\((8\ Pi\ Sqrt[
                            kr[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                    2]\)]\ kl[\([i]\)]])\), 0] + 
                  If[\((hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                          hr[\([i + 2]\)] \[Equal] hl[\([i]\)] && \ 
                          Abs[hr[\([i + 1]\)]] \[NotEqual] 
                            Abs[hl[\([i]\)]])\)\  || \((hr[\([i + 
                                  2]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                          hr[\([i]\)] \[Equal] hl[\([i]\)] && \ 
                          Abs[hr[\([i + 1]\)]] \[NotEqual] 
                            Abs[hl[\([i]\)]])\), 
                    lambda4/\((8\ Pi\ Sqrt[
                            kl[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                    2]\)]\ kr[\([i]\)]])\), 0], 0], {i, 2, 
                Length[sl] - 
                  1}], \[IndentingNewLine]\[IndentingNewLine]True, \
\[IndentingNewLine]0\[IndentingNewLine]\[IndentingNewLine]]]\)}], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subroutine for nonzero transverse momentum", "Section"],

Cell[TextData[{
  "This is the  ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^-\)\)]],
  " operator.  Thus, to get the 2 ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^+\)\)]],
  " ",
  Cell[BoxData[
      \(TraditionalForm\`\(P\^-\)\)]],
  " operator, we still must multiply by twice the sum of the momenta.\n\nThe \
definitions of the self-inertias and zeta are given above."
}], "Text",
  FontSize->13],

Cell[BoxData[{
    \(\(If[\(! StringMatchQ[makeham::usage, "\<*aperp*\>"]\), 
        makeham::usage = 
          makeham::usage <> "\<\n\n\
makeham[leftstate,rightstate,aperp,{mub,muf,g2,kappas,kappaa,beta,lambda1,\
lambda2,lambda3,lambda4,lambda5},npmax]  calculates P^- where both leftstate \
and rightstate are in |ket> form and npmax is the maximum number of \
particles, both quarks and gluons, in all states of the basis.  The vector \
aperp is the lattice spacing times transverse momentum.\>"];\)\
\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(choose[x_] := 
        Switch[x[\([2]\)], 1, {1, 0}, 
          2, {0, 1}, \(-1\), {\(-1\), 0}, \(-2\), {0, \(-1\)}, 1/2, 
          Null, \(-1\)/2, 
          Null];\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(makeham[sl_, sr_, 
        aperp_List, {mub_, muf_, g2_, kappas_, kappaa_, beta_, lambda1_, 
          lambda2_, lambda3_, lambda4_, lambda5_}, npmax_] := 
      Block[{kl = Map[First, sl], kr = Map[First, sr], hl = Map[choose, sl], 
          hr = Map[choose, sr], kt}, \[IndentingNewLine]kt = 
          Apply[Plus, 
            kl]; \[IndentingNewLine]Which[\ \ \[IndentingNewLine]\
\[IndentingNewLine]Length[sl] \[Equal] Length[sr], \[IndentingNewLine] (*\ 
            diagonal\ interactions\ *) \[IndentingNewLine]If[
              sl === sr, \[IndentingNewLine] (*\ 
                Self - energy\ interactions\ *) \[IndentingNewLine]muf^2/\((2\
\ kl[\([1]\)])\) + muf^2/\((2\ kl[\([\(-1\)]\)])\) + 
                Sum[mub^2/\((2\ kl[\([i]\)])\), {i, 2, Length[sl] - 1}] + 
                Sum[g2\ elfglueglue[kl[\([i]\)], kl[\([i + 1]\)]], {i, 2, 
                    Length[sl] - 2}] + 
                If[Length[sl] < 
                    npmax, \((kappaa^2 + 
                        kappas^2)\)*\((elfkappa[kl[\([1]\)]] + 
                        elfkappa[kl[\([\(-1\)]\)]])\), 0] + 
                g2\ If[Length[sl] \[Equal] 2, \ 
                    elfquarkquark[kl[\([1]\)], kl[\([2]\)]], 
                    elfquarkglue[kl[\([1]\)], kl[\([2]\)]] + 
                      elfquarkglue[kl[\([\(-1\)]\)], 
                        kl[\([\(-2\)]\)]]], \[IndentingNewLine] (*\ 
                Strictly\ non - diagnal\ 2 \[Rule] 
                  2\ particle\ interactions\ *) \[IndentingNewLine] (*\ 
                gluon - gluon\  \[Rule] \ 
                  gluon - gluon\ interactions\ *) \[IndentingNewLine]Sum[
                  If[Drop[sl, {i, i + 1}] === 
                      Drop[sr, {i, 
                          i + 1}], \[IndentingNewLine] (*Instantaneous\ gluon \
- gluon\  \[Rule] \ gluon - 
                          gluon\ normal\ interaction\ \
*) \[IndentingNewLine]If[
                      hl[\([i]\)] \[Equal] hr[\([i]\)]\  && \ 
                        hl[\([i + 1]\)] \[Equal] 
                          hr[\([i + 1]\)], \(-Exp[
                            I\ aperp\  . \((\((hr[\([i]\)] - 
                                        hl[\([i]\)])\)*\((kr[\([i]\)] + 
                                        kr[\([i + 1]\)])\) + \((hr[\([i]\)] + 
                                        hr[\([i + 1]\)])\)*\((kr[\([i + 
                                        1]\)] - 
                                        kl[\([i + 1]\)])\))\)/\((2\ kt)\)]\)*
                        g2*\((kl[\([i]\)] + 
                            kr[\([i]\)])\)*\((kl[\([i + 1]\)] + 
                              kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                                kl[\([i]\)]*kr[\([i]\)]*kl[\([i + 1]\)]*
                                  kr[\([i + 1]\)]] \((kl[\([i]\)] - 
                                    kr[\([i]\)])\)^2)\), 
                      0], \[IndentingNewLine]0], {i, 2, 
                    Length[sl] - 2}] + \[IndentingNewLine]If[
                  Length[sl] > 2, \[IndentingNewLine] (*\ 
                    quark - gluon\  \[Rule] \ 
                      quark - gluon\ interaction\ *) \[IndentingNewLine]If[
                      Drop[sl, 2] === Drop[sr, 2]\  && \ 
                        hl[\([2]\)] \[Equal] hr[\([2]\)]\  && \ 
                        Rest[sl[\([1]\)]] === 
                          Rest[sr[\([1]\)]], \(-Exp[
                            I\ aperp\  . \((hl[\([2]\)]\ \((kl[\([1]\)] + 
                                        kl[\([2]\)]/2)\) - 
                                      hr[\([2]\)]\ \((kr[\([1]\)] + 
                                        kr[\([2]\)]/2)\))\)/kt]\)*
                        g2*\((kl[\([2]\)] + 
                              kr[\([2]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([2]\)]\ kr[\([2]\)]]\ \((kl[\([1]\)] - 
                                    kr[\([1]\)])\)^2)\), 
                      0] + \[IndentingNewLine] (*\ 
                      antiquark - gluon\  \[Rule] \ 
                        antiquark - 
                          gluon\ interaction\ *) \[IndentingNewLine]If[
                      Drop[sl, \(-2\)] === 
                          Drop[sr, \(-2\)]\  && \ \ hl[\([\(-2\)]\)] \[Equal] 
                          hr[\([\(-2\)]\)]\  && 
                        Rest[sl[\([\(-1\)]\)]] === 
                          Rest[sr[\([\(-1\)]\)]], \(-Exp[\(-I\)\ aperp\  . \
\((hl[\([\(-2\)]\)]\ \((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)]/2)\) - 
                                      hr[\([\(-2\)]\)]\ \((kr[\([\(-1\)]\)] + \
\ kr[\([\(-2\)]\)]/2)\))\)/kt]\)*
                        g2\ \((kl[\([\(-2\)]\)] + 
                              kr[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0], \[IndentingNewLine] (*\ 
                    quark - quark\  \[Rule] \ 
                      quark - quark\ interaction\ *) \[IndentingNewLine]If[\ 
                    Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]] && \ 
                      Rest[sl[\([2]\)]] === 
                        Rest[sr[\([2]\)]], \[IndentingNewLine]\(-g2\)/\((4\ \
Pi\ \((kl[\([1]\)] - kr[\([1]\)])\)^2)\), 
                    0]\[IndentingNewLine]]\[IndentingNewLine]] + \
\[IndentingNewLine] (*\ 
              Both\ diagonal\ and\ non - 
                diagonal\ interactions\ *) \[IndentingNewLine]If[
              Length[sl] > 2, \[IndentingNewLine] (*\ 
                quark - gluon\  \[Rule] \ 
                  quark - gluon\ interactions\ *) \[IndentingNewLine]If[
                  Drop[sl, 2] === Drop[sr, 2], 
                  Exp[I\ aperp\  . \((hl[\([2]\)]\ \((kl[\([1]\)] + 
                                      kl[\([2]\)]/2)\) - 
                                hr[\([2]\)]\ \((kr[\([1]\)] + 
                                      kr[\([2]\)]/2)\))\)/kt]*\((If[
                          Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                          kappas^2/\((8\ Pi\ Sqrt[
                                  kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                    kr[\([2]\)])\))\), 0] + 
                        kappaa\ kappas\ \((zetaf[sl[\([1]\)], sl[\([2]\)], 
                                  sr[\([1]\)]] + 
                                zetaf[sl[\([1]\)], sr[\([2]\)], 
                                  sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                                  kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                    kr[\([2]\)])\))\) + 
                        kappaa^2\ zetafzetaf[sl[\([1]\)], sl[\([2]\)], 
                              sr[\([2]\)], 
                              sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                                  kl[\([2]\)]\ kr[\([2]\)]]\ \((kr[\([1]\)] + 
                                    kr[\([2]\)])\))\))\), 
                  0] + \[IndentingNewLine] (*\ 
                  antiuark - gluon\  \[Rule] \ 
                    antiquark - gluon\ interactions\ *) \[IndentingNewLine]If[
                  Drop[sl, \(-2\)] === Drop[sr, \(-2\)], 
                  Exp[\(-I\)\ aperp\  . \((hl[\([\(-2\)]\)]\ \((kl[\([\(-1\)]\
\)] + kl[\([\(-2\)]\)]/2)\) - hr[\([\(-2\)]\)]\ \((kr[\([\(-1\)]\)] + \ 
                                      kr[\([\(-2\)]\)]/2)\))\)/kt]*\((If[
                          Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                          kappas^2/\((8\ Pi\ Sqrt[
                                  kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \((kr[\
\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\), 0] - 
                        kappaa\ kappas\ Conjugate[
                              zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                                  sl[\([\(-1\)]\)]] + 
                                zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                                  sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                                  kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \((kr[\
\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) + 
                        kappaa^2\ Conjugate[
                              zetafzetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                                sl[\([\(-2\)]\)], 
                                sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                                  kl[\([\(-2\)]\)]\ kr[\([\(-2\)]\)]]\ \((kr[\
\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\))\), 0], 0] + \[IndentingNewLine] (*\ 
              gluon - gluon\  \[Rule] \ 
                gluon - gluon\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i, i + 1}] === 
                  Drop[sr, {i, i + 1}]\ , \[IndentingNewLine]Exp[
                    I\ aperp\  . \((\((hr[\([i]\)] - 
                                    hl[\([i]\)])\)*\((kr[\([i]\)] + 
                                    kr[\([i + 1]\)])\) + \((hr[\([i]\)] + 
                                    hr[\([i + 1]\)])\)*\((kr[\([i + 1]\)] - 
                                    kl[\([i + 
                                        1]\)])\))\)/\((2\ kt)\)]*\((\
\[IndentingNewLine] (*Instantaneous\ annihilation\ interaction\ \
*) \[IndentingNewLine]If[\ 
                        hl[\([i]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\), 
                        g2*\((kl[\([i]\)] - 
                              kl[\([i + 1]\)])\)*\((kr[\([i]\)] - 
                                kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                                  kl[\([i]\)]*kr[\([i]\)]*kl[\([i + 1]\)]*
                                    kr[\([i + 1]\)]] \((kl[\([i]\)] + 
                                      kl[\([i + 1]\)])\)^2)\), 
                        0] + \[IndentingNewLine] (*\ 
                        Effective\ potential\ terms\ *) \[IndentingNewLine]If[
                        hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                          hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                          Abs[hl[\([i]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kl[\([i]\)]\ kl[\([i + 
                                        1]\)]])\), 0] + 
                      If[hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] hr[\([i]\)], 
                        lambda1/\((4\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kl[\([i]\)]\ kl[\([i + 
                                        1]\)]])\), 0] + 
                      If[hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                          hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                          Abs[hl[\([i]\)]] \[Equal] Abs[hr[\([i]\)]], 
                        lambda2/\((8\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kl[\([i]\)]\ kl[\([i + 
                                        1]\)]])\), 0] + 
                      If[\((hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                              hr[\([i]\)] \[Equal] \(-hr[\([i + 
                                        1]\)]\)\  && \ 
                              Abs[hl[\([i]\)]] \[NotEqual] 
                                Abs[hr[\([i]\)]])\) || \((hl[\([i]\)] \
\[Equal] hr[\([i]\)]\  && \ hr[\([i + 1]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                              Abs[hl[\([i]\)]] \[NotEqual] 
                                Abs[hl[\([i + 1]\)]])\), 
                        lambda4/\((8\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kl[\([i]\)]\ kl[\([i + 
                                        1]\)]])\), 0])\), 0], {i, 2, 
                Length[sl] - 
                  2}], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] + 1, \[IndentingNewLine] (*\ 
            Yukawa\ quark\  \[Rule] \ 
              quark - gluon\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, 2] === Drop[sr, 1], 
              Exp[I\ aperp\  . hl[\([2]\)] \((kl[\([1]\)] + kl[\([2]\)]/2)\)/
                      kt]*\((If[
                      Rest[sl[\([1]\)]] === 
                        Rest[sr[\([1]\)]], \(-kappas\)\ muf\ \((kl[\([1]\)] + 
                              kr[\([1]\)])\)/\((4\ \ Sqrt[
                                Pi\ kl[\([2]\)]]\ kl[\([1]\)]\ \
kr[\([1]\)])\), 0] + 
                    muf\ kappaa\ Sqrt[
                        kl[\([2]\)]]\ zetaf[sl[\([1]\)], sl[\([2]\)], 
                          sr[\([1]\)]]/\((4\ \ Sqrt[
                              Pi]\ kr[\([1]\)]\ kl[\([1]\)])\))\), 
              0] + \[IndentingNewLine] (*\ 
              Yukawa\ antiquark\  \[Rule] \ 
                antiquark - gluon\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, \(-2\)] === Drop[sr, \(-1\)], 
              Exp[\(-I\)\ aperp\  . 
                      hl[\([\(-2\)]\)] \((kl[\([\(-1\)]\)] + 
                          kl[\([\(-2\)]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([\(-1\)]\)]] === 
                        Rest[sr[\([\(-1\)]\)]], \
\(-kappas\)\ \ muf\ \ \((kl[\([\(-1\)]\)] + \ 
                              kr[\([\(-1\)]\)])\)/\((4\ Sqrt[
                                Pi\ kl[\([\(-2\)]\)]]\ kl[\([\(-1\)]\)]\ \
kr[\([\(-1\)]\)]\ )\), 0] - 
                    kappaa\ \ muf\ Sqrt[
                        kr[\([\(-1\)]\)] - kl[\([\(-1\)]\)]]\ \ Conjugate[
                          zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                            sl[\([\(-1\)]\)]]]/\((4\ Sqrt[
                              Pi]\ kl[\([\(-1\)]\)]\ kr[\([\(-1\)]\)]\ )\))\)\
\ , \[IndentingNewLine]0], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] + 2, \[IndentingNewLine] (*\ 
            quark\  \[Rule] \ 
              quark - glue - glue\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, 3] === Drop[sr, 1], 
              Exp[I\ aperp\  . \((kl[\([1]\)]\ \((hl[\([2]\)] + 
                                  hl[\([3]\)])\) + 
                            kl[\([2]\)]\ \((hl[\([2]\)]/2 + hl[\([3]\)])\) + 
                            kl[\([3]\)]\ hl[\([3]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                      kappas^2\ /\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([1]\)] + 
                                kl[\([2]\)])\))\), 0] + 
                    kappaa\ kappas\ \((zetaf[sl[\([1]\)], sl[\([3]\)], 
                              sr[\([1]\)]] - 
                            zetaf[sl[\([1]\)], sl[\([2]\)], 
                              sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([3]\)] - 
                                kr[\([1]\)])\))\) + 
                    kappaa^2\ zetafzetaf[sl[\([1]\)], sl[\([2]\)], 
                          sl[\([3]\)], 
                          sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                              kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([3]\)] - 
                                kr[\([1]\)])\))\) + 
                    If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]]\  && \ 
                        hl[\([2]\)] \[Equal] \(-hl[\([3]\)]\), \(-g2\)\ \
\((kl[\([2]\)] - kl[\([3]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([2]\)]\ kl[\([3]\)]]\ \((kl[\([1]\)] - 
                                    kr[\([1]\)])\)^2)\), 0])\), 
              0] + \[IndentingNewLine] (*\ 
              antiquark\  \[Rule] \ 
                antiquark - glue - 
                  glue\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, \(-3\)] === Drop[sr, \(-1\)], 
              Exp[\(-I\)\ aperp\  . \((kl[\([\(-1\)]\)]\ \((hl[\([\(-2\)]\)] \
+ hl[\([\(-3\)]\)])\) + kl[\([\(-2\)]\)]\ \((hl[\([\(-2\)]\)]/2 + 
                                  hl[\([\(-3\)]\)])\) + 
                            kl[\([\(-3\)]\)]\ hl[\([\(-3\)]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                      kappas^2\ /\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\), 0] + 
                    kappaa\ kappas\ Conjugate[
                          zetaf[sr[\([\(-1\)]\)], sl[\([\(-3\)]\)], 
                              sl[\([\(-1\)]\)]] - 
                            zetaf[sr[\([\(-1\)]\)], sl[\([\(-2\)]\)], 
                              sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\) - 
                    kappaa^2\ Conjugate[
                          zetafzetaf[sr[\([\(-1\)]\)], sl[\([\(-3\)]\)], 
                            sl[\([\(-2\)]\)], 
                            sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] + kl[\([\(-2\)]\)])\))\) + 
                    If[Rest[sl[\([\(-1\)]\)]] === 
                          Rest[sr[\([\(-1\)]\)]]\  && \ 
                        hl[\([\(-2\)]\)] \[Equal] \(-hl[\([\(-3\)]\)]\), 
                      g2\ \((kl[\([\(-3\)]\)] - 
                              kl[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                                kl[\([\(-2\)]\)]\ kl[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0])\), 
              0] + \[IndentingNewLine] (*\ 
              gauge\ glue\  \[Rule] \ 
                glue - glue - glue\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i, i + 2}] === Drop[sr, {i}], 
                Exp[\(-I\)\ aperp\  . \((kl[\([i]\)]\ \((hl[\([i]\)] - 
                                    hr[\([i]\)])\) + 
                              kl[\([i + 1]\)]\ \((hl[\([i]\)] - 
                                    hl[\([i + 2]\)])\) + 
                              kl[\([i + 2]\)]\ \((hl[\([i]\)] + 
                                    hl[\([i + 1]\)])\))\)/\((2\ kt)\)]*\((If[
                        hl[\([i + 2]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] 
                            hr[\([i]\)], \(-g2\)\ \((kl[\([i]\)] + 
                              kr[\([i]\)])\)\ \((kl[\([i + 1]\)] - 
                                kl[\([i + 2]\)])\)/\((16\ Pi\ Sqrt[
                                  kl[\([i]\)]\ kr[\([i]\)]\ kl[\([i + 
                                        1]\)]\ kl[\([i + 
                                        2]\)]]\ \((kr[\([i]\)] - 
                                      kl[\([i]\)])\)^2)\), 0] + 
                      If[hl[\([i]\)] == \(-hl[\([i + 1]\)]\)\  && \ 
                          hl[\([i + 2]\)] \[Equal] hr[\([i]\)], 
                        g2\ \((kr[\([i]\)] + 
                              kl[\([i + 2]\)])\)\ \((kl[\([i]\)] - 
                                kl[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                                  kl[\([i]\)]\ kr[\([i]\)]\ kl[\([i + 
                                        1]\)]\ kl[\([i + 
                                        2]\)]]\ \((kl[\([i + 2]\)] - 
                                      kr[\([i]\)])\)^2)\), 0] + 
                      If[hr[\([i]\)] \[Equal] hl[\([i + 1]\)]\  && \ 
                          hl[\([i]\)] \[Equal] \(-hl[\([i + 2]\)]\)\  && \ 
                          Abs[hl[\([i]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                                kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0] + 
                      If[hr[\([i]\)] \[Equal] hl[\([i + 2]\)]\  && \ 
                          hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] hr[\([i]\)], 
                        lambda1/\((4\ Pi\ Sqrt[
                                kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0] + 
                      If[Abs[hr[\([i]\)]] \[Equal] Abs[hl[\([i]\)]]\  && \ 
                          hl[\([i]\)] \[Equal] \(-hl[\([i + 2]\)]\)\  && \ 
                          hl[\([i + 1]\)] \[Equal] hr[\([i]\)], 
                        lambda2/\((8\ Pi\ Sqrt[
                                kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0] + 
                      If[\((hl[\([i]\)] \[Equal] \(-hl[\([i + 1]\)]\)\  && \ 
                              hl[\([i + 2]\)] \[Equal] hr[\([i]\)] && \ 
                              Abs[hl[\([i + 1]\)]] \[NotEqual] 
                                Abs[hr[\([i]\)]])\)\  || \((hl[\([i + 
                                      2]\)] \[Equal] \(-hl[\([i + 
                                        1]\)]\)\  && \ 
                              hl[\([i]\)] \[Equal] hr[\([i]\)] && \ 
                              Abs[hl[\([i + 1]\)]] \[NotEqual] 
                                Abs[hr[\([i]\)]])\), 
                        lambda4/\((8\ Pi\ Sqrt[
                                kl[\([i]\)]\ kl[\([i + 1]\)]\ kl[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0])\), 
                0], {i, 2, 
                Length[sr] - 
                  1}], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] - 1, \[IndentingNewLine] (*\ 
            Yukawa\ quark - gluon\  \[Rule] \ 
              quark\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, 1] === Drop[sr, 2], 
              Exp[\(-I\)\ aperp\  . 
                      hr[\([2]\)] \((kr[\([1]\)] + kr[\([2]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([1]\)]] === 
                        Rest[sr[\([1]\)]], \
\(-kappas\)\ \ muf\ \ \((kl[\([1]\)] + 
                              kr[\([1]\)])\)/\((4\ Sqrt[
                                Pi\ kr[\([2]\)]]\ kl[\([1]\)]\ kr[\([1]\)])\)\
\ , 0] + kappaa\ \ muf\ Sqrt[
                        kr[\([2]\)]]\ \ zetaf[sl[\([1]\)], sr[\([2]\)], 
                          sr[\([1]\)]]/\((4\ Sqrt[
                              Pi]\ kr[\([1]\)]\ kl[\([1]\)])\))\)\ , 
              0] +  (*\ Yukawa\ antiquark - gluon\  \[Rule] \ 
                antiquark\ interactions\ *) \ \[IndentingNewLine]If[
              Drop[sl, \(-1\)] === Drop[sr, \(-2\)], 
              Exp[I\ aperp\  . 
                      hr[\([\(-2\)]\)] \((kr[\([\(-1\)]\)] + 
                          kr[\([\(-2\)]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([\(-1\)]\)]] === 
                        Rest[sr[\([\(-1\)]\)]], \
\(-kappas\)\ \ muf\ \ \((kr[\([\(-1\)]\)] + 
                              kl[\([\(-1\)]\)])\)/\((4\ Sqrt[
                                Pi\ kr[\([\(-2\)]\)]]\ kr[\([\(-1\)]\)]\ \
kl[\([\(-1\)]\)])\)\ , 0] - 
                    kappaa\ \ muf\ Sqrt[
                        kr[\([\(-2\)]\)]]\ \ Conjugate[
                          zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                            sl[\([\(-1\)]\)]]]/\((4\ Sqrt[
                              Pi]\ kr[\([\(-1\)]\)]\ kl[\([\(-1\)]\)])\))\)\ \
, 0], \[IndentingNewLine]\[IndentingNewLine]Length[sl] == 
            Length[sr] - 2, \[IndentingNewLine] (*\ 
            quark - glue - glue\  \[Rule] \ 
              quark\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, 1] === Drop[sr, 3], 
              Exp[\(-I\)\ aperp\  . \((kr[\([1]\)]\ \((hr[\([2]\)] + 
                                  hr[\([3]\)])\) + 
                            kr[\([2]\)]\ \((hr[\([2]\)]/2 + hr[\([3]\)])\) + 
                            kr[\([3]\)]\ hr[\([3]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]], 
                      kappas^2\ /\((8\ Pi\ Sqrt[
                              kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\), 0] + 
                    kappaa\ kappas\ \((zetaf[sl[\([1]\)], sr[\([2]\)], 
                              sr[\([1]\)]] - 
                            zetaf[sl[\([1]\)], sr[\([3]\)], 
                              sr[\([1]\)]])\)/\((8\ Pi\ Sqrt[
                              kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\) - 
                    kappaa^2\ zetafzetaf[sl[\([1]\)], sr[\([3]\)], 
                          sr[\([2]\)], 
                          sr[\([1]\)]]/\((8\ Pi\ Sqrt[
                              kr[\([2]\)]\ kr[\([3]\)]]\ \((kr[\([1]\)] + 
                                kr[\([2]\)])\))\) + 
                    If[Rest[sl[\([1]\)]] === Rest[sr[\([1]\)]]\  && \ 
                        hr[\([2]\)] \[Equal] \(-hr[\([3]\)]\), \(-g2\)\ \
\((kr[\([2]\)] - kr[\([3]\)])\)/\((8\ Pi\ Sqrt[
                                kr[\([2]\)]\ kr[\([3]\)]]\ \((kl[\([1]\)] - 
                                    kr[\([1]\)])\)^2)\), 0])\), 
              0] + \[IndentingNewLine] (*\ 
              antiquark - glue - glue\  \[Rule] \ 
                antiquark\ interactions\ *) \[IndentingNewLine]If[
              Drop[sl, \(-1\)] === Drop[sr, \(-3\)], 
              Exp[I\ aperp\  . \((kr[\([\(-1\)]\)]\ \((hr[\([\(-2\)]\)] + 
                                  hr[\([\(-3\)]\)])\) + 
                            kr[\([\(-2\)]\)]\ \((hr[\([\(-2\)]\)]/2 + 
                                  hr[\([\(-3\)]\)])\) + 
                            kr[\([\(-3\)]\)]\ hr[\([\(-3\)]\)]/2)\)/kt]*\((If[
                      Rest[sl[\([\(-1\)]\)]] === Rest[sr[\([\(-1\)]\)]], 
                      kappas^2\ /\((8\ Pi\ Sqrt[
                              kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\), 0] - 
                    kappaa\ kappas\ Conjugate[
                          zetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                              sl[\([\(-1\)]\)]] - 
                            zetaf[sr[\([\(-1\)]\)], sr[\([\(-3\)]\)], 
                              sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) - 
                    kappaa^2\ Conjugate[
                          zetafzetaf[sr[\([\(-1\)]\)], sr[\([\(-2\)]\)], 
                            sr[\([\(-3\)]\)], 
                            sl[\([\(-1\)]\)]]]/\((8\ Pi\ Sqrt[
                              kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kr[\([\(-1\)]\)] + kr[\([\(-2\)]\)])\))\) + 
                    If[Rest[sl[\([\(-1\)]\)]] === 
                          Rest[sr[\([\(-1\)]\)]]\  && \ 
                        hr[\([\(-2\)]\)] === \(-hr[\([\(-3\)]\)]\), 
                      g2\ \((kr[\([\(-3\)]\)] - 
                              kr[\([\(-2\)]\)])\)/\((8\ Pi\ Sqrt[
                                kr[\([\(-2\)]\)]\ kr[\([\(-3\)]\)]]\ \
\((kl[\([\(-1\)]\)] - kr[\([\(-1\)]\)])\)^2)\), 0])\), 
              0] + \[IndentingNewLine] (*\ 
              gauge\ glue - glue - glue\  \[Rule] \ 
                glue\ interactions\ *) \[IndentingNewLine]Sum[
              If[Drop[sl, {i}] === Drop[sr, {i, i + 2}], 
                Exp[I\ aperp\  . \((kr[\([i]\)]\ \((hr[\([i]\)] - 
                                    hl[\([i]\)])\) + 
                              kr[\([i + 1]\)]\ \((hr[\([i]\)] - 
                                    hr[\([i + 2]\)])\) + 
                              kr[\([i + 2]\)]\ \((hr[\([i]\)] + 
                                    hr[\([i + 1]\)])\))\)/\((2\ kt)\)]*\((If[
                        hr[\([i + 2]\)] == \(-hr[\([i + 1]\)]\)\  && \ 
                          hr[\([i]\)] \[Equal] 
                            hl[\([i]\)], \(-g2\)\ \((kl[\([i]\)] + 
                              kr[\([i]\)])\)\ \((kr[\([i + 1]\)] - 
                                kr[\([i + 2]\)])\)/\((16\ Pi\ Sqrt[
                                  kl[\([i]\)]\ kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kr[\([i + 
                                        2]\)]]\ \((kr[\([i]\)] - 
                                      kl[\([i]\)])\)^2)\), 0] + 
                      If[hr[\([i]\)] == \(-hr[\([i + 1]\)]\)\  && \ 
                          hr[\([i + 2]\)] \[Equal] hl[\([i]\)], 
                        g2\ \((kl[\([i]\)] + 
                              kr[\([i + 2]\)])\)\ \((kr[\([i]\)] - 
                                kr[\([i + 1]\)])\)/\((16\ Pi\ Sqrt[
                                  kl[\([i]\)]\ kr[\([i]\)]\ kr[\([i + 
                                        1]\)]\ kr[\([i + 
                                        2]\)]]\ \((kr[\([i + 2]\)] - 
                                      kl[\([i]\)])\)^2)\), 0] + 
                      If[hl[\([i]\)] \[Equal] hr[\([i + 1]\)]\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 2]\)]\)\  && \ 
                          Abs[hl[\([i]\)]] \[NotEqual] 
                            Abs[hr[\([i]\)]], \(-beta\)/\((8\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                        2]\)]\ kl[\([i]\)]])\), 0] + 
                      If[hl[\([i]\)] \[Equal] hr[\([i + 2]\)]\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] hr[\([i]\)], 
                        lambda1/\((4\ Pi\ Sqrt[
                                kl[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0] + 
                      If[Abs[hr[\([i]\)]] \[Equal] Abs[hl[\([i]\)]]\  && \ 
                          hr[\([i]\)] \[Equal] \(-hr[\([i + 2]\)]\)\  && \ 
                          hl[\([i]\)] \[Equal] hr[\([i + 1]\)], 
                        lambda2/\((8\ Pi\ Sqrt[
                                kr[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                        2]\)]\ kl[\([i]\)]])\), 0] + 
                      If[\((hr[\([i]\)] \[Equal] \(-hr[\([i + 1]\)]\)\  && \ 
                              hr[\([i + 2]\)] \[Equal] hl[\([i]\)] && \ 
                              Abs[hr[\([i + 1]\)]] \[NotEqual] 
                                Abs[hl[\([i]\)]])\)\  || \((hr[\([i + 
                                      2]\)] \[Equal] \(-hr[\([i + 
                                        1]\)]\)\  && \ 
                              hr[\([i]\)] \[Equal] hl[\([i]\)] && \ 
                              Abs[hr[\([i + 1]\)]] \[NotEqual] 
                                Abs[hl[\([i]\)]])\), 
                        lambda4/\((8\ Pi\ Sqrt[
                                kl[\([i]\)]\ kr[\([i + 1]\)]\ kr[\([i + 
                                        2]\)]\ kr[\([i]\)]])\), 0])\), 
                0], {i, 2, 
                Length[sl] - 
                  1}], \[IndentingNewLine]\[IndentingNewLine]True, \
\[IndentingNewLine]0\[IndentingNewLine]\[IndentingNewLine]]]\)}], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Self-inertia expresssions", "Section"],

Cell["These are the old versions of the self-inertias", "Text"],

Cell[BoxData[{
    \(\(Clear[fkappa, fquark, 
        fglue];\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(fkappa[kl_] := \(fkappa[kl] = 
          Sum[1/k, {k, 1/2, 
                kl - 1}]/\((2\ Pi\ kl)\)\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(fquark[kl_] := \(fquark[kl] = 
          Sum[1/\((kl - kr)\)^2, {kr, 1/2, 
                kl - 1}]/\((4\ Pi)\)\);\)\[IndentingNewLine]\), "\
\[IndentingNewLine]", 
    \(\(fglue[
          kl_] := \(fglue[
            kl] = \((1/kl + Sum[1/\((kl - kr)\)^2, {kr, 1, kl - 1}] + 
                Zeta[2])\)/\((4\ Pi)\)\);\)\)}], "Input"],

Cell["\<\
selfglueglue[...] agrees with what we had before for large k1 and \
k2:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[elfglueglue[5, 7]], N[fglue[5] + fglue[7]]/2}\)], "Input"],

Cell[BoxData[
    \({0.2516936993147204`, 0.26052571094281307`}\)], "Output"]
}, Open  ]],

Cell["\<\
This is the quark-link expression from Simon's E-mail.  n1 is the \
quark and n2 is the link.
I added the 1/(8 Pi)\
\>", "Text"],

Cell[BoxData[
    \(selfqgSimon[n1_, 
        n2_] := \((Sqrt[1 + n1/n2] - 
              1)\)/\((4\ Pi\ n1)\) + \((Sum[\((n1 + 
                      2*n2 - \((r - 0.5)\))\)/\((Sqrt[
                        n2*\((n1 + n2 - \((r - 0.5)\))\)]*\((n1 - r + 
                            0.5)\)^2)\), {r, 1, n1 - 0.5}] + 
              Sum[\((n1 + 2*n2 - r - 
                      0.5)\)/\((Sqrt[
                        n2*\((n1 + n2 - r - 0.5)\)]*\((n1 - r - 
                            0.5)\)^2)\), {r, n1 + 0.5, 
                  n1 + n2 - 1.5}])\)/\((8\ Pi)\)\)], "Input"],

Cell["This agrees with what we had before for large k1 and k2:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[elfquarkglue[3/2, 4]], N[fquark[3/2] + fglue[4]/2], 
      selfqgSimon[3/2, 4]}\)], "Input"],

Cell[BoxData[
    \({0.19262900698031313`, 0.20913139279663734`, 
      0.19262900698031313`}\)], "Output"]
}, Open  ]],

Cell["\<\
Quark-antiquark interaction from simon's Email, v is the quark \
momentum and K is the total momentum.
I added the 1/(4 Pi)\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(selfqqSimon[v_, K_] := 
      Module[{ans = 0}, 
        Do[ans = ans + 1/\((i - 0.5 - Abs[v[\([1]\)]])\)^2, {i, 1, 
              Abs[v[\([1]\)]] - 0.5}] + 
          Do[ans = ans + 1/\((i + 0.5 - Abs[v[\([1]\)]])\)^2, {i, 
              Abs[v[\([1]\)]] + 0.5, 
              K - 1}]; \[IndentingNewLine]ans/\((4\ Pi)\)]\)\)\)], "Input"],

Cell["\<\
Compare with version ,where k1 is the quark and k2 is the antiquark \
momentum\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \({N[elfquarkquark[3/2, 5/2]], selfqqSimon[{3/2}, 3/2 + 5/2]}\)], "Input"],

Cell[BoxData[
    \({0.17904931097838225`, 0.17904931097838225`}\)], "Output"]
}, Open  ]],

Cell["\<\
Yukawa self-inertia from Simon. i is the quark momentum and K is \
the total momentum (of what?)
I added the (2 Pi).\
\>", "Text"],

Cell[BoxData[
    \(selfyukSimon[i_, 
        K_] := \(\(If[i > 0.5, 
          ans = K*Sum[1/j, {j, 1, i - 0.5}]/
                i]/\((2\ Pi)\)\)\(\ \)\)\)], "Input"],

Cell["\<\
Compare with my version ,where k1 is the quark momentum (I am \
summing over odd half integers).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[{elfkappa[21/2], selfyukSimon[21/2, 1]}]\)], "Input"],

Cell[BoxData[
    \({0.04439616912174148`, 0.04439616912174148`}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["testing", "Section"],

Cell[BoxData[{
    \(\(state1\  = 
        State[b[1/2, 1/2], d[3/2, \(-1\)/2]];\)\), "\[IndentingNewLine]", 
    \(\(state2\  = State[b[1\/2, 1\/2], d[3\/2, 1\/2]];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(makeham[state1, 
      state2, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 10]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(makeham[state2, 
      state1, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 10]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(\(state1 = 
        State[b[1/2, \(-1\)/2], a[3, \(-2\)], 
          d[1/2, 1/2]];\)\), "\[IndentingNewLine]", 
    \(\(state2 = 
        State[b[1/2, \(-1\)/2], a[1, \(-2\)], a[2, \(-2\)], a[3, \(-1\)], 
          d[1/2, 1/2]];\)\)}], "Input"],

Cell[BoxData[{
    \(\(state1 = 
        State[b[1\/2, 1\/2], a[3, \(-2\)], 
          d[1\/2, 1\/2]];\)\), "\[IndentingNewLine]", 
    \(\(state2 = 
        State[b[1\/2, 1\/2], a[1, 2], a[1, \(-2\)], a[1, \(-1\)], 
          d[1\/2, 1\/2]];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(makeham[state1, 
      state2, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 4]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(makeham[state2, 
      state1, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 4]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(<< "\</home/emwatson/hamiltonian/hamilton.m\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ham[state1, 
      state2, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 4]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ham[state2, 
      state1, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, lambda3, 
        lambda4, lambda5}, 4]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["Test against Beth's matrix code", "Text"],

Cell[BoxData[
    \(<< "\</home/jdbratt/hamiltonianMatrices.m\>"\)], "Input"],

Cell[BoxData[{
    \(\(ll = listOfStates[3, 2, none];\)\), "\[IndentingNewLine]", 
    \(Do[state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
      If[i \[NotEqual] j\  && \ 
          ham[state1, 
              state2, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, 
                lambda3, lambda4, lambda5}, 3] =!= 
            makeham[state1, 
              state2, {mub, muf, g2, kappas, kappaa, beta, lambda1, lambda2, 
                lambda3, lambda4, lambda5}, 3], 
        Print[{i, j, state1, state2}]], {i, Length[ll]}, {j, 
        Length[ll]}]\)}], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{985, 873},
WindowMargins->{{291, Automatic}, {Automatic, 67}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "makeham.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 44, 0, 107, "Title"],
Cell[1786, 53, 64, 3, 67, "Subsubtitle"],
Cell[1853, 58, 69, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1947, 62, 34, 0, 54, "Section"],
Cell[1984, 64, 529, 19, 54, "Text"],
Cell[2516, 85, 980, 21, 155, "Input",
  InitializationCell->True],
Cell[3499, 108, 204, 4, 32, "Text"],
Cell[3706, 114, 1968, 40, 347, "Input",
  InitializationCell->True],
Cell[5677, 156, 25662, 438, 2619, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[31376, 599, 61, 0, 54, "Section"],
Cell[31440, 601, 410, 13, 74, "Text"],
Cell[31853, 616, 31799, 523, 2939, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[63689, 1144, 44, 0, 34, "Section"],
Cell[63736, 1146, 63, 0, 32, "Text"],
Cell[63802, 1148, 615, 14, 123, "Input"],
Cell[64420, 1164, 95, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[64540, 1171, 81, 1, 27, "Input"],
Cell[64624, 1174, 77, 1, 27, "Output"]
}, Open  ]],
Cell[64716, 1178, 138, 4, 50, "Text"],
Cell[64857, 1184, 575, 11, 75, "Input"],
Cell[65435, 1197, 72, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[65532, 1201, 115, 2, 27, "Input"],
Cell[65650, 1205, 107, 2, 27, "Output"]
}, Open  ]],
Cell[65772, 1210, 147, 4, 50, "Text"],
Cell[65922, 1216, 368, 7, 75, "Input"],
Cell[66293, 1225, 102, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[66420, 1232, 92, 1, 27, "Input"],
Cell[66515, 1235, 78, 1, 27, "Output"]
}, Open  ]],
Cell[66608, 1239, 140, 4, 50, "Text"],
Cell[66751, 1245, 168, 4, 27, "Input"],
Cell[66922, 1251, 120, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[67067, 1258, 75, 1, 27, "Input"],
Cell[67145, 1261, 78, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[67272, 1268, 26, 0, 34, "Section"],
Cell[67301, 1270, 181, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[67507, 1277, 159, 3, 27, "Input"],
Cell[67669, 1282, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[67741, 1288, 159, 3, 27, "Input"],
Cell[67903, 1293, 35, 1, 27, "Output"]
}, Open  ]],
Cell[67953, 1297, 265, 6, 43, "Input"],
Cell[68221, 1305, 258, 6, 80, "Input"],

Cell[CellGroupData[{
Cell[68504, 1315, 158, 3, 27, "Input"],
Cell[68665, 1320, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[68737, 1326, 158, 3, 27, "Input"],
Cell[68898, 1331, 35, 1, 27, "Output"]
}, Open  ]],
Cell[68948, 1335, 79, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[69052, 1340, 154, 3, 27, "Input"],
Cell[69209, 1345, 35, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[69281, 1351, 154, 3, 27, "Input"],
Cell[69438, 1356, 35, 1, 27, "Output"]
}, Open  ]],
Cell[69488, 1360, 47, 0, 32, "Text"],
Cell[69538, 1362, 77, 1, 27, "Input"],
Cell[69618, 1365, 575, 11, 91, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

