(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     32804,        624]*)
(*NotebookOutlinePosition[     33915,        659]*)
(*  CellTagsIndexPosition[     33871,        655]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Test example for K=2, one link", "Title"],

Cell["\<\
These are mesons in large N limit with antiperiodic boundary \
conditions for fermions and periodic boundry conditions for bosons (no zero \
modes).
No symmetries.  This uses Simon's form for the Fermion self-intertias.\
\>", \
"Text"],

Cell[BoxData[
    \(<< "\</home/jdbratt/hamiltonianMatrices.m\>"\)], "Input"],

Cell[CellGroupData[{

Cell["Write out matrix ", "Section"],

Cell["These are the |ket> 's with daggers implicit.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(listOfStates[2, 1, none]\)], "Input"],

Cell[BoxData[
    \({State[b[1\/2, \(-\(1\/2\)\)], d[3\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, \(-\(1\/2\)\)], d[3\/2, 1\/2]], 
      State[b[1\/2, 1\/2], d[3\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, 1\/2], d[3\/2, 1\/2]], 
      State[b[3\/2, \(-\(1\/2\)\)], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[3\/2, \(-\(1\/2\)\)], d[1\/2, 1\/2]], 
      State[b[3\/2, 1\/2], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[3\/2, 1\/2], d[1\/2, 1\/2]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, \(-2\)], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, \(-2\)], d[1\/2, 1\/2]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, \(-1\)], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, \(-1\)], d[1\/2, 1\/2]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, 1], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, 1], d[1\/2, 1\/2]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, 2], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, \(-\(1\/2\)\)], a[1, 2], d[1\/2, 1\/2]], 
      State[b[1\/2, 1\/2], a[1, \(-2\)], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, 1\/2], a[1, \(-2\)], d[1\/2, 1\/2]], 
      State[b[1\/2, 1\/2], a[1, \(-1\)], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, 1\/2], a[1, \(-1\)], d[1\/2, 1\/2]], 
      State[b[1\/2, 1\/2], a[1, 1], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, 1\/2], a[1, 1], d[1\/2, 1\/2]], 
      State[b[1\/2, 1\/2], a[1, 2], d[1\/2, \(-\(1\/2\)\)]], 
      State[b[1\/2, 1\/2], a[1, 2], d[1\/2, 1\/2]]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[%]\)], "Input"],

Cell[BoxData[
    \(24\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(generalMatrix[makeham, 2, 1, 0, 
      1, {\[Mu]\_B, \[Mu]\_F, 
        g\^2, \[Kappa]\_S, \[Kappa]\_A, \[Beta], \[Lambda]\_1, \[Lambda]\_2, \
\[Lambda]\_3, \[Lambda]\_4, \[Lambda]\_5}, none]\)], "Input"],

Cell[BoxData[
    \({{\(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\
\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \
\(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \
\(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[Kappa]\_A\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\), 0, 0, 0, 0, 0, 0, 0, 
        0}, {0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\
\ \[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 0, \(-\(g\^2\/\[Pi]\)\), 0, 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \
\[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \[ImaginaryI]\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\
\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 0, 0, 0, 0, 0, 0}, {0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 0, 0, 0, 0, 
        0, 0, 0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\
\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\
\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \
\(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \
\(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[Kappa]\_A\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)}, {0, 0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 0, 0, 0, 0, 
        0, 0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\
\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \[ImaginaryI]\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\
\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\_S\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\)}, {\(-\(g\^2\/\[Pi]\)\), 0, 0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0}, {0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\)}, {0, 
        0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0}, {0, 0, 
        0, \(-\(g\^2\/\[Pi]\)\), 0, 0, 
        0, \(3\ g\^2 + 4\ \[Kappa]\_A\%2 + 4\ \[Kappa]\_S\%2 + 16\ \[Pi]\ \
\[Mu]\_F\%2\)\/\(3\ \[Pi]\), 0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\)}, {\(-\(\(8\ \
\[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \[Kappa]\_A\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\
\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \
\(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \((\[ImaginaryI]\
\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 - \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \((\(-\
\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \
\(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 0, 
        0}, {\(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \
\[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(2\ \
\[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \((\[Kappa]\_A\%2 + \
\[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \
\[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \
\[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \
\[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 
        0}, {\(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(2\ \((\(-\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \
\[Pi]\), \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\
\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \
\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \((\(-\
\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \
\[Pi]\), \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], 
        0, \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \
\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0}, {\(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\
\)\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\
\), \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\
\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + \
3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \
\[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \
\[Kappa]\_S\%2\)\/\(3\ \[Pi]\), 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \
\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\)}, {\(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \
\[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(2\ \((\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \
\[Pi]\), \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\
\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \
\(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \
\((\(-\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \(\
\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], 0, 0, 
        0, \(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0}, {\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\
\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \
\[Kappa]\_S\%2\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \
\((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \
\[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \[Kappa]\_S\%2\)\
\/\(3\ \[Pi]\), 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 0, 
        0, \(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\)}, {\(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\
\(3\ \[Pi]\), \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi], \(2\ \((\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \
\[Pi]\), \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \
\(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \
\[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(2\ \
\[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 0, 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 0, \(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), 
        0}, {\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(8\ \[Kappa]\
\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \
\[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(2\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\
\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\
\/\(3\ \[Pi]\), 0, 0, 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 0, \(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\)}, {0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \[Kappa]\_A\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 0, 
        0, \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\
\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \
\(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \[Kappa]\_S\%2\)\
\/\(3\ \[Pi]\), \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi]\)\), \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 \
+ \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \((\(-\
\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 0}, {0, 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(8\ \
\[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 
        0, \(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \((\
\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \
\[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \((\(-\
\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \
\(2\ \((\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\)}, \
{0, 0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(4\ \[ImaginaryI]\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 0, 
        0, 0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi]\)\), 
        0, \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 - \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \
\((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \
\[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(2\ \[ImaginaryI]\
\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \((\(-\[Kappa]\_A\%2\) + \
\[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(\((1\/3 + \[ImaginaryI]\/3)\
\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]}, {0, 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 
        0, \(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), 0, 
        0, 0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi]\)\), \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi]\)\), \(2\ \((\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\
\%2)\)\)\/\(3\ \[Pi]\), \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\(3\ \[Pi]\)\)\), \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ \
g\^2 + \@6\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \
\[Pi]\), 0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \
\((\(-\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\)}, \
{0, 0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \
\[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(-\(\(4\ \
\[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 0, 
        0, \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \
\[Pi]\)\)\), 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0, \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(-\(\(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi]\)\), \(2\ \((\(-\
\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\
\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \
\(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), \(2\ \
\[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi]}, {0, 
        0, \(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\
\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[ImaginaryI]\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\
\), 0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\[Pi], 0, 0, 
        0, \(-\(\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \
\[Pi]\)\)\), 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), \(2\ \((\(-\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\
\(3\ \[Pi]\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(2\ \[ImaginaryI]\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \
\((\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \
\[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(\((1\/3 + \
\[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \((\[ImaginaryI]\
\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\)}, {0, 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), \(4\ \
\[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(4\ \[Kappa]\_A\ \
\[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 0, \(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), 
        0, \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \(\((1\/3 + \[ImaginaryI]\/3)\
\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \[Kappa]\_S\%2\)\/\(3\ \[Pi]\), \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \((\
\[Kappa]\_A\%2 + \[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \
\[Mu]\_B\%2 + 4\ \[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\), \(2\ \[Kappa]\_A\ \
\[Kappa]\_S\)\/\(3\ \[Pi]\)}, {0, 
        0, \(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\), \(-\(\(8\ \
\[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(4\ \[Kappa]\_A\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 
        0, \(-\(\(8\ \[Kappa]\_S\ \[Mu]\_F\)\/\(3\ \@\[Pi]\)\)\), 0, 0, 
        0, \(-\(\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 
        0, \(-\(\(\((1\/3 + \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\
\[Pi]\)\), 0, \(-\(\(2\ \[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\)\)\), 
        0, \(2\ \((\(-\[Kappa]\_A\%2\) + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \
\(\((1\/3 - \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \((\
\[ImaginaryI]\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \(\((1\/3 \
+ \[ImaginaryI]\/3)\)\ \[Kappa]\_A\ \[Kappa]\_S\)\/\[Pi], \(2\ \((\(-\
\[ImaginaryI]\)\ \[Kappa]\_A\%2 + \[Kappa]\_S\%2)\)\)\/\(3\ \[Pi]\), \(2\ \
\[Kappa]\_A\ \[Kappa]\_S\)\/\(3\ \[Pi]\), \(2\ \((\[Kappa]\_A\%2 + \
\[Kappa]\_S\%2 + 3\ \((\(-2\)\ g\^2 + \@6\ g\^2 + \[Pi]\ \[Mu]\_B\%2 + 4\ \
\[Pi]\ \[Mu]\_F\%2)\))\)\)\/\(3\ \[Pi]\)}}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Eigenvalues\ for\ just\ g\^2\ interactions\)], "Section"],

Cell[BoxData[
    \(\(expr = 
        N[generalMatrix[makeham, 2, 1, 0, 
            1, {0, 0, 1, \(-0.323\), 0.162, 0, 0, 0, 0, 0, 0}, 
            none]];\)\)], "Input"],

Cell["This is a list of the different elements of the matrix", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Union[Flatten[expr]]\)], "Input"],

Cell[BoxData[
    \({\(-0.3183098861837907`\), \(-0.011103922069635357`\), \
\(-0.0055519610348176785`\) - 
        0.0055519610348176785`\ \[ImaginaryI], \(-0.0055519610348176785`\) + 
        0.0055519610348176785`\ \[ImaginaryI], 
      0.`, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
        0.011103922069635357`\ \[ImaginaryI], \
\(\(0.`\)\(\[InvisibleSpace]\)\) + 
        0.011103922069635357`\ \[ImaginaryI], \(\(0.0055519610348176785`\)\(\
\[InvisibleSpace]\)\) - 
        0.0055519610348176785`\ \[ImaginaryI], \(\(0.0055519610348176785`\)\(\
\[InvisibleSpace]\)\) + 0.0055519610348176785`\ \[ImaginaryI], 
      0.011103922069635357`, 
      0.016570151641774198`, \(\(0.0221393014104458`\)\(\[InvisibleSpace]\)\) \
+ 0.`\ \[ImaginaryI], \(\(0.0221393014104458`\)\(\[InvisibleSpace]\)\) - 
        0.005569149768671602`\ \[ImaginaryI], \(\(0.0221393014104458`\)\(\
\[InvisibleSpace]\)\) + 0.005569149768671602`\ \[ImaginaryI], 
      0.3138625089113068`, 0.3737267885420255`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sort[Chop[Eigenvalues[expr]]]\)], "Input"],

Cell[BoxData[
    \({0.05541690235823493`, 0.05541690235823493`, 0.05541690235823493`, 
      0.05541690235823493`, 0.2861540577321894`, 0.28615405773218944`, 
      0.28615405773218944`, 0.28615405773218944`, 0.28615405773218944`, 
      0.2861540577321895`, 0.2861540577321895`, 0.2861540577321895`, 
      0.28615405773218966`, 0.29729235726953257`, 0.29729235726953285`, 
      0.3084306568068759`, 0.3747112633739726`, 0.3858495629113159`, 
      0.38584956291131595`, 0.39698786244865925`, 0.6920366747258164`, 
      0.6920366747258164`, 0.6920366747258164`, 
      0.6920366747258164`}\)], "Output"]
}, Open  ]],

Cell["These are the diagonal matrix elements ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(diag = Table[expr[\([i, i]\)], {i, Length[expr]}]\)], "Input"],

Cell[BoxData[
    \({0.3737267885420255`, 0.3737267885420255`, 0.3737267885420255`, 
      0.3737267885420255`, 0.3737267885420255`, 0.3737267885420255`, 
      0.3737267885420255`, 0.3737267885420255`, 0.3138625089113068`, 
      0.3138625089113068`, 0.3138625089113068`, 0.3138625089113068`, 
      0.3138625089113068`, 0.3138625089113068`, 0.3138625089113068`, 
      0.3138625089113068`, 0.3138625089113068`, 0.3138625089113068`, 
      0.3138625089113068`, 0.3138625089113068`, 0.3138625089113068`, 
      0.3138625089113068`, 0.3138625089113068`, 
      0.3138625089113068`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Eigenvalues\ for\ mass, \ g\^2, \ and\ kappa\ interactions\)], "Section"],

Cell["\<\
The gluon effective potential interactions are not relevant here.\
\
\>", "Text"],

Cell[BoxData[
    \(\(expr = 
        N[generalMatrix[makeham, 2, 1, 0, 
            1, {0.1803, 0.362, 1, \(-0.323\), 0.162, 0, 0, 0, 0, 0, 0}, 
            none]];\)\)], "Input"],

Cell["This is a list of the different elements of the matrix", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Union[Flatten[expr]]\)], "Input"],

Cell[BoxData[
    \({\(-0.3183098861837907`\), \(-0.044115111916766155`\), \
\(-0.011103922069635357`\), \(-0.0055519610348176785`\) - 
        0.0055519610348176785`\ \[ImaginaryI], \(-0.0055519610348176785`\) + 
        0.0055519610348176785`\ \[ImaginaryI], 
      0.`, \(\(0.`\)\(\[InvisibleSpace]\)\) - 
        0.011103922069635357`\ \[ImaginaryI], \
\(\(0.`\)\(\[InvisibleSpace]\)\) + 
        0.011103922069635357`\ \[ImaginaryI], \
\(\(0.`\)\(\[InvisibleSpace]\)\) - 
        0.044115111916766155`\ \[ImaginaryI], \
\(\(0.`\)\(\[InvisibleSpace]\)\) + 
        0.044115111916766155`\ \[ImaginaryI], \(\(0.0055519610348176785`\)\(\
\[InvisibleSpace]\)\) - 
        0.0055519610348176785`\ \[ImaginaryI], \(\(0.0055519610348176785`\)\(\
\[InvisibleSpace]\)\) + 0.0055519610348176785`\ \[ImaginaryI], 
      0.011103922069635357`, 
      0.016570151641774198`, \(\(0.0221393014104458`\)\(\[InvisibleSpace]\)\) \
+ 0.`\ \[ImaginaryI], \(\(0.0221393014104458`\)\(\[InvisibleSpace]\)\) - 
        0.005569149768671602`\ \[ImaginaryI], \(\(0.0221393014104458`\)\(\
\[InvisibleSpace]\)\) + 0.005569149768671602`\ \[ImaginaryI], 
      0.044115111916766155`, 0.17591581665574654`, 1.0726281218753588`, 
      1.4272306889113067`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sort[Chop[Eigenvalues[expr]]]\)], "Input"],

Cell[BoxData[
    \({0.48457118491763124`, 0.4935831869194369`, 0.4935831869194371`, 
      0.5029979224579487`, 1.2806415657826549`, 1.3120196751358046`, 
      1.3120196751358049`, 1.3909380080591502`, 1.3995222377321885`, 
      1.399522237732189`, 1.3995222377321892`, 1.3995222377321894`, 
      1.39952223773219`, 1.39952223773219`, 1.4196471032307374`, 
      1.4196471032307378`, 1.4393196489065516`, 1.4895788701928767`, 
      1.4895788701928774`, 1.5320952790833697`, 1.7393997566075923`, 
      1.7398279261848977`, 1.7398279261848981`, 
      1.7403056820482328`}\)], "Output"]
}, Open  ]],

Cell["These are the diagonal matrix elements ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(diag = Table[expr[\([i, i]\)], {i, Length[expr]}]\)], "Input"],

Cell[BoxData[
    \({1.0726281218753588`, 1.0726281218753588`, 1.0726281218753588`, 
      1.0726281218753588`, 1.0726281218753588`, 1.0726281218753588`, 
      1.0726281218753588`, 1.0726281218753588`, 1.4272306889113067`, 
      1.4272306889113067`, 1.4272306889113067`, 1.4272306889113067`, 
      1.4272306889113067`, 1.4272306889113067`, 1.4272306889113067`, 
      1.4272306889113067`, 1.4272306889113067`, 1.4272306889113067`, 
      1.4272306889113067`, 1.4272306889113067`, 1.4272306889113067`, 
      1.4272306889113067`, 1.4272306889113067`, 
      1.4272306889113067`}\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{803, 943},
WindowMargins->{{Automatic, 136}, {0, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "testcases.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1},
CellLabelAutoDelete->True
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 47, 0, 107, "Title"],
Cell[1789, 53, 245, 6, 50, "Text"],
Cell[2037, 61, 77, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2139, 66, 36, 0, 54, "Section"],
Cell[2178, 68, 61, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2264, 72, 57, 1, 27, "Input"],
Cell[2324, 75, 1483, 24, 404, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3844, 104, 42, 1, 27, "Input"],
Cell[3889, 107, 36, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3962, 113, 224, 4, 31, "Input"],
Cell[4189, 119, 22363, 340, 2893, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[26601, 465, 77, 1, 54, "Section"],
Cell[26681, 468, 171, 4, 27, "Input"],
Cell[26855, 474, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[26950, 478, 53, 1, 27, "Input"],
Cell[27006, 481, 994, 17, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28037, 503, 62, 1, 27, "Input"],
Cell[28102, 506, 607, 9, 59, "Output"]
}, Open  ]],
Cell[28724, 518, 55, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[28804, 522, 82, 1, 27, "Input"],
Cell[28889, 525, 594, 9, 59, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[29532, 540, 93, 1, 54, "Section"],
Cell[29628, 543, 91, 3, 32, "Text"],
Cell[29722, 548, 180, 4, 27, "Input"],
Cell[29905, 554, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[30000, 558, 53, 1, 27, "Input"],
Cell[30056, 561, 1242, 22, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[31335, 588, 62, 1, 27, "Input"],
Cell[31400, 591, 590, 9, 59, "Output"]
}, Open  ]],
Cell[32005, 603, 55, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[32085, 607, 82, 1, 27, "Input"],
Cell[32170, 610, 594, 9, 59, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

