(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    145859,       3023]*)
(*NotebookOutlinePosition[    147348,       3068]*)
(*  CellTagsIndexPosition[    147304,       3064]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Convergence in K", "Subtitle"],

Cell["\<\
Brett van de Sande
July, 2002\
\>", "Subsubtitle"],

Cell["\<\
Numerical convergence in K for various discretizations: periodic \
boundary conditions, antiperiodic boundary conditions, and improved matrix \
elements.\
\>", "Text"],

Cell[BoxData[
    \(rmserror[data_, fit_, var_] := 
      Sqrt[Apply[Plus, 
            Map[\((\((fit /. var \[Rule] #[\([1]\)])\) - #[\([2]\)])\)^2 &, 
              data]]/Length[data]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(nullvalues[mat_?MatrixQ, null_List] := 
      Block[{n = Length[mat], norm, a, b, c, d}, 
        norm = Complement[Range[n], null]; \[IndentingNewLine]a = 
          Map[Part[#, norm] &, Part[mat, norm]]; \[IndentingNewLine]b = 
          Map[Part[#, null] &, Part[mat, norm]]; \[IndentingNewLine]c = 
          Map[Part[#, norm] &, Part[mat, null]]; \[IndentingNewLine]d = 
          Map[Part[#, null] &, Part[mat, null]]; \[IndentingNewLine]Sort[
          Eigenvalues[a - b . Inverse[d] . c]]\[IndentingNewLine]]\)], "Input",\

  InitializationCell->True],

Cell["Input data file.  This divides by two to get the correct K:", "Text"],

Cell[BoxData[
    \(Map[{#[\([1]\)]/2, #[\([2]\)]} &, 
      Map[Partition[#, 2] &, 
        ReadList["\<~/four/k_summary.out\>", Number, RecordLists -> True, 
          RecordSeparators -> "\<\n\n\>"], {1}], {2}]\)], "Input"],

Cell["\<\
Endpoint behavior:  the fermion valence endpoint behavior is \
x^alpha and the link endpoint behavior is x^beta. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[
      1 - Pi\ alpha\ Cot[Pi\ alpha] \[Equal] 2\ Pi\ 0.3349170391, {alpha, 
        0.5}]\)], "Input"],

Cell[BoxData[
    \({alpha \[Rule] 0.6564857107477157`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FindRoot[beta\ Tan[Pi\ beta] == 0.07642881742, {beta, 0.1}]\)], "Input"],

Cell[BoxData[
    \({beta \[Rule] 0.15000000007618638`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Glueballs, winding (2,0), two particles", "Section"],

Cell[CellGroupData[{

Cell["data from ~/three/k_two_*_1.out", "Subsection"],

Cell["\<\
order: anti-periodic, periodic, improved.  Couplings: \
0.001967542671 1 0.3909839522 -0.01250282378 -0.06421231123 353.2492529 \
0.07049127522 160.6273835 -0.7123520719 -0.6519355869 0 0 0 0
Since beta is so small, these points are hard to analyze numerically.\
\>", \
"Text"],

Cell[BoxData[{
    \(\(beta = 0.025;\)\), "\[IndentingNewLine]", 
    \(\(data = {{{12/2, 0.600500677607279`}, {16/2, 
              0.61267581799677`}, {24/2, 0.627139990171567`}, {32/2, 
              0.635810545384629`}, {48/2, 0.646198849802507`}, {64/2, 
              0.652481735683803`}, {96/2, 0.660078271029871`}, {128/2, 
              0.664717826424915`}, {192/2, 0.6703855904512139`}, {256/2, 
              0.6738862105935`}, {384/2, 0.678213816732471`}, {512/2, 
              0.680921273349911`}, {768/2, 0.684313662106626`}}, {{12/2, 
              0.544237116948233`}, {16/2, 0.558194492045904`}, {24/2, 
              0.57656038868097`}, {32/2, 0.588550960471077`}, {48/2, 
              0.603912553487464`}, {64/2, 0.613725699856467`}, {96/2, 
              0.626089383026337`}, {128/2, 0.63388370519763`}, {192/2, 
              0.6436107801062391`}, {256/2, 0.6497010648277309`}, {384/2, 
              0.657271256616406`}, {512/2, 0.662002617079252`}, {768/2, 
              0.667886787074527`}}, {{12/2, 0.800966588309857`}, {16/2, 
              0.799704869561203`}, {24/2, 0.798487376304771`}, {32/2, 
              0.797893962435107`}, {48/2, 0.7973098607228379`}, {64/2, 
              0.797021005723677`}, {96/2, 0.796734062229699`}, {128/2, 
              0.796591234662794`}, {192/2, 0.796448782044152`}, {256/2, 
              0.796377678081713`}, {384/2, 0.796306641908576`}, {512/2, 
              0.796271144618888`}, {768/2, 0.7962356577069689`}, {1024/2, 
              0.796217916973681`}}};\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data from ~/three/k_two_*_2.out", "Subsection"],

Cell[TextData[{
  "order: anti-periodic, periodic, improved.\ncouplings: 0.07642881742 1 \
0.7680984-0.1692461589-0.1857716652 49854.19434 0.0\n2426983029 \
24828.43378-1.547566329-2.239030657 0.9413385404-0.330530446 0.837\n3498804 \
0.1 0.2 0 0\nIn ",
  StyleBox["Mathematica,",
    FontSlant->"Italic"],
  StyleBox[" using compiled code, K=1024 took 484 sec. and 35 Mbytes memory.",
    
    FontVariations->{"CompatibilityType"->0}]
}], "Text"],

Cell[BoxData[{
    \(\(beta = 0.15;\)\), "\[IndentingNewLine]", 
    \(\(data = {{{4, 0.9394318784604669`}, {6, 0.9951337937394269`}, {8, 
              1.03113170812173`}, {12, 1.07764397065232`}, {16, 
              1.10800307873075`}, {24, 1.14749476933118`}, {32, 
              1.17338480007586`}, {48, 1.20713905856584`}, {64, 
              1.22928732261535`}, {96, 1.258156627007`}, {128, 
              1.27708198662843`}, {192, 1.30171443181191`}, {256, 
              1.3178332547006`}, {384, 1.33877034011644`}, {512, 
              1.35244113767536`}, {1024, 1.3817005040863908}}, {{4, 
              0.815849614167641`}, {6, 0.8607441002474581`}, {7, 
              0.8781848829768171`}, {8, 0.8933630496666509`}, {9, 
              0.9067675918168648`}, {12, 0.939398379323538`}, {16, 
              0.9716033261174999`}, {24, 1.01575848642528`}, {32, 
              1.04594593877196`}, {48, 1.08660565277937`}, {64, 
              1.11400820123399`}, {96, 1.15050501625779`}, {128, 
              1.17487568794345`}, {192, 1.20709256165178`}, {256, 
              1.22846800344446`}, {384, 1.25657216579177`}, {512, 
              1.27512826026085`}, {1024, 1.3153943194153728`}}, {{4, 
              1.52759289354871`}, {6, 1.52654520764556`}, {8, 
              1.52604097779003`}, {12, 1.52554831393099`}, {16, 
              1.52530577975902`}, {24, 1.52506536394308`}, {32, 
              1.52494581156736`}, {96/2, 1.52482659088180}, {128/2, 
              1.52476706889591}, {192/2, 1.52470757778977}, {256/2, 
              1.52467783406643}, {384/2, 1.52464808395413}}};\)\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["mathematica data", "Subsection"],

Cell[BoxData[{
    \(\(beta = 0.15;\)\), "\[IndentingNewLine]", 
    \(\(evensimple = {{4, 1.5968788137188905`}, {5, 1.6154283436643004`}, {8, 
            1.6439295792416306`}, {11, 1.6556760985835508`}, {16, 
            1.6629106515001852`}, {22, 1.664547903656986`}, {32, 
            1.662480103739089`}, {45, 1.657901060977981`}, {64, 
            1.6513732376192303`}, {90, 1.6439807537229658`}, {128, 
            1.6357623370029286`}, {181, 1.6274846404906531`}, {256, 
            1.6192809470563305`}, {362, 1.6113450212326288`}, {512, 
            1.6037860151353651`}, {724, 1.5966840536770859`}, {1024, 
            1.5900663907383388`}};\)\)}], "Input"],

Cell[BoxData[
    \(\(datazero = {{4, 1.8606603882241395`}, {6, 1.818129185673524`}, {8, 
            1.7906399531357482`}, {11, 1.762976107169748`}, {16, 
            1.7340385995758345`}, {23, 1.709463635714488`}, {32, 
            1.6897609297554486`}, {45, 1.6717837129021718`}, {64, 
            1.6554308698305105`}, {91, 1.6410564550478381`}, {128, 
            1.6287566063054948`}, {181, 1.6176982011386272`}, {256, 
            1.6078929160003634`}, {362, 1.5991931709934275`}, {512, 
            1.591451139373834`}, {724, 1.5845587694259553`}, {1024, 
            1.5784063095050178`}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["do in mathematica", "Subsection"],

Cell["From <<\"~/four/2001/makeham.m\"", "Text"],

Cell[BoxData[
    \(elfglueglue[k1_, k2_] := 
      Block[{kt = k1 + k2}, 
        1\/\(16\ \@\(k1\ k2\)\) + \(\[Sum]\+\(k = 1\)\%\(kt - 1\)If[k \
\[NotEqual] k1, \(\((k1 + k)\)\ \((k2 + kt - k)\)\)\/\(\@\(k1\ k2\ k\ \((kt - \
k)\)\)\ \((k1 - k)\)\^2\), 0]\)\/\(16\ \[Pi]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(elfodd[k1_, k2_] := 
      Block[{kt = k1 + k2}, 
        1\/\(16\ \@\(k1\ k2\)\) + \(\[Sum]\+\(k = 1/2\)\%\(kt - 1/2\)If[k \
\[NotEqual] k1, \(\((k1 + k)\)\ \((k2 + kt - k)\)\)\/\(\@\(k1\ k2\ k\ \((kt - \
k)\)\)\ \((k1 - k)\)\^2\), 0]\)\/\(16\ \[Pi]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(elfevensimple[k1_, k2_] := 
      Block[{kt = 
            k1 + k2}, \(1\/\(4\ Pi\)\) \((1/k1 + 
                1/k2)\) + \(\[Sum]\+\(k = 1\)\%\(kt - 1\)If[k \[NotEqual] k1, \
1\/\(\(\ \)\(\((k1 - k)\)\^2\)\), 0]\)\/\(4\ \[Pi]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(elfzero[k1_, k2_] := 
      Block[{kt = k1 + k2}, 
        kt/\((4*Pi*If[k1 > 0, k1, 2]*If[k2 > 0, k2, 2])\) + 
          If[k1 > 0 && 
              k2 > 0, \((1/\((2*k1^2)\) + 
                  Sum[If[k \[NotEqual] k1, 1/\((k1 - k)\)^2, 0], {k, 1, 
                      kt - 1}] + 1/\((2*k2^2)\))\)/\((4*Pi)\), 0]]\)], "Input",\

  InitializationCell->True],

Cell["\<\
It is important for speed that the loop variables are real.  \
(Yuck!)\
\>", "Text"],

Cell[BoxData[
    \(\(hamzero = 
        Compile[{{kk, _Integer}, mub2, ggn, lambda2}, 
          Table[\[IndentingNewLine]\t
            2\ kk\ \((If[k1 \[Equal] k2, 
                    2\ ggn\ elfzero[k1, kk - k1] + 
                      mub2\ kk/\((2*
                              If[k1 > 0, k1, 2]\ If[kk - k1 > 0, kk - k1, 
                                2])\), \(-2\)\ ggn\ \((k1 + 
                          k2)\)\ \((2\ kk - k1 - 
                            k2)\)/\((16\ Pi\ If[
                              k1\ k2\ \((kk - k1)\)\ \((kk - k2)\) \[Equal] 
                                0, 2, 1]\ Sqrt[
                              Max[k1, 1]\ Max[k2, 1]\ Max[kk - k1, 1]\ Max[
                                  kk - k2, 1]] \((k1 - 
                                  k2)\)^2)\)] + \ \ 2\ lambda2/\((8\ Pi\ If[
                            k1\ k2\ \((kk - k1)\)\ \((kk - k2)\) \[Equal] 0, 
                            2, 1] Sqrt[
                            Max[k1, 1]\ Max[k2, 1]\ Max[kk - k1, 1]\ Max[
                                kk - k2, 1]])\))\), {k1, 0.0, kk}, {k2, 0.0, 
              kk}]];\)\)], "Input"],

Cell[BoxData[
    \(\(hamevensimple = 
        Compile[{{kk, _Integer}, mub2, ggn, lambda2}, 
          Table[\[IndentingNewLine]\t
            2\ kk\ \((If[k1 \[Equal] k2, 
                    2\ ggn\ elfevensimple[k1, kk - k1] + 
                      mub2/2*\((1/k1 + 
                            1/\((kk - k1)\))\), \(-2\)\ ggn\ \((k1 + 
                          k2)\)\ \((2\ kk - k1 - 
                            k2)\)/\((16\ Pi\ Sqrt[
                              k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)] \((k1 - 
                                  k2)\)^2)\)] + \ \ 2\ lambda2/\((8\ Pi\ Sqrt[
                            k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)])\))\), {k1, 
              1.0, kk - 1.0}, {k2, 1.0, kk - 1.0}]];\)\)], "Input"],

Cell[BoxData[
    \(\(hameven = 
        Compile[{{kk, _Integer}, mub2, ggn, lambda2}, 
          Table[\[IndentingNewLine]\t
            2\ kk\ \((If[k1 \[Equal] k2, 
                    2\ ggn\ elfglueglue[k1, kk - k1] + 
                      mub2/2*\((1/k1 + 
                            1/\((kk - k1)\))\), \(-2\)\ ggn\ \((k1 + 
                          k2)\)\ \((2\ kk - k1 - 
                            k2)\)/\((16\ Pi\ Sqrt[
                              k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)] \((k1 - 
                                  k2)\)^2)\)] + \ \ 2\ lambda2/\((8\ Pi\ Sqrt[
                            k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)])\))\), {k1, 
              1.0, kk - 1.0}, {k2, 1.0, kk - 1.0}]];\)\)], "Input"],

Cell[BoxData[
    \(\(hamodd = 
        Compile[{{kk, _Integer}, mub2, ggn, lambda2}, 
          Table[2\ kk\ \ \((If[k2 \[Equal] k1, 
                    2\ ggn\ elfodd[k1, kk - k1] + 
                      mub2/2*\((1/k1 + 
                            1/\((kk - k1)\))\), \(-2\)\ ggn\ \((k1 + 
                          k2)\)\ \((2\ kk - k1 - 
                            k2)\)/\((16\ Pi\ Sqrt[
                              k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)] \((k1 - 
                                  k2)\)^2)\)] + \ \ 2\ lambda2/\((8\ Pi\ Sqrt[
                            k1\ k2\ \((kk - k1)\)\ \((kk - k2)\)])\))\), {k1, 
              0.5, kk - 0.5}, {k2, 0.5, kk - 0.5}]];\)\)], "Input"],

Cell["Make table of points", "Text"],

Cell[BoxData[
    \(Table[
      k = Floor[2^i + 0.5]; {k, 
        First[Sort[
            Eigenvalues[
              hameven[k, 0.07642881742, 1, \(-0.1857716652\)]]]]}, {i, 2, 10, 
        1/2}]\)], "Input"],

Cell[BoxData[
    \(First[
      Sort[Eigenvalues[
          hameven[9, 0.07642881742, 1, \(-0.1857716652\)]]]]\)], "Input"],

Cell[BoxData[
    \(Table[
      k = Floor[2^i + 0.5]; {k, 
        First[nullvalues[
            hameven[k, 0.07642881742, 1, \(-0.1857716652\)], {1, 
              k + 1}]]}, {i, 2, 10, 1/2}]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analyze", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit0 = 
      Fit[Drop[data[\([1]\)], 7], {1, 1/x^\((4\ beta)\), 1/x^\((2\ beta)\), 
          1/Sqrt[x], 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.5255038811617958`\)\(\[InvisibleSpace]\)\) - 
      0.12099326249962905`\/x - 0.39891435024007826`\/x\^0.6` + 
      1.021577145531488`\/\@x - 1.3550114089892042`\/x\^0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fit0] /. x \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({1.5255038811617958`, \(-0.0012099326249962906`\), \
\(-0.025169793910252154`\), 
      0.10215771455314879`, \(-0.34036347727806604`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit1 = 
      Fit[Drop[data[\([2]\)], 7], {1, 1/x^\((2\ beta)\), 1/x^\((4\ beta)\), 
          1/Sqrt[x], 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.5269609733534784`\)\(\[InvisibleSpace]\)\) + 
      0.34336506649910614`\/x - 1.8039952146481717`\/x\^0.6` + 
      2.8666040627670455`\/\@x - 2.186366875445963`\/x\^0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitprogram = Fit[data[\([2, {2, 3, 5}]\)], {1, 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.9985985615015872`\)\(\[InvisibleSpace]\)\) - 
      0.8323110904229476`\/x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fit1] /. x \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({1.5269609733534784`, 
      0.0034336506649910612`, \(-0.11382440300893867`\), 
      0.2866604062767045`, \(-0.5491905288734711`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitimp = Fit[Drop[data[\([3]\)], 7], {1, 1/x, 1/x^2}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.524588614576325`\)\(\[InvisibleSpace]\)\) + 
      0.0002490033592863483`\/x\^2 + 0.011417569986759218`\/x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitsimple = 
      Fit[Drop[evensimple, 3], {1, 1/x^\((2\ beta)\), 1/x^\((4\ beta)\), 
          1/Sqrt[x], 1/x, 1/x^2}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.528575269097512`\)\(\[InvisibleSpace]\)\) + 
      0.8150314033356216`\/x\^2 - 0.277485921034744`\/x - 
      2.3413741814144373`\/x\^0.6` + 1.7681318765662022`\/\@x + 
      0.3447577654430952`\/x\^0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fitsimple] /. x \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({1.528575269097512`, 
      0.00008150314033356216`, \(-0.00277485921034744`\), \
\(-0.14773072359397396`\), 0.1768131876566202`, 
      0.08659923531740733`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[evensimple, fitsimple, x]\)], "Input"],

Cell[BoxData[
    \(0.0014767202830033455`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitzero = 
      Fit[Drop[datazero, 3], {1, 1/x^\((2\ beta)\), 1/x^\((4\ beta)\), 
          1/Sqrt[x], 1/x, 1/x^2}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.5235677466009092`\)\(\[InvisibleSpace]\)\) - 
      0.1572565327751434`\/x\^2 - 0.10317232808185972`\/x + 
      0.5976459190997279`\/x\^0.6` - 0.38593751111263686`\/\@x + 
      0.461285841478725`\/x\^0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fitzero] /. x \[Rule] 10\)], "Input"],

Cell[BoxData[
    \({1.5235677466009092`, \(-0.001572565327751434`\), \
\(-0.010317232808185972`\), 0.15012186750336787`, \(-0.12204415696124772`\), 
      0.2311905748021506`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[datazero, fitzero, x]\)], "Input"],

Cell[BoxData[
    \(0.0002988320249703984`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fit0 /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fit1 /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fitimp /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fitsimple /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fitzero /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data[\([1]\)]], 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data[\([2]\)]], 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data[\([3]\)]], 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, evensimple], 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, datazero], 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction, Axes \[Rule] False, 
      Frame \[Rule] True, 
      FrameTicks \[Rule] {Flatten[{{Map[{1/Log[#], #, {0.025, 0}} &, {10, 
                    100, 1000}]}, 
              Table[Map[{1/Log[#], "\<\>", {0.0125, 0}} &, 
                  Range[2\ 10^i, 10^\((i + 1)\), 10^i]], {i, 0, 5}]}, 2], 
          Automatic}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Plot for talk, Los Alamos, 2002", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fit0 /. x \[Rule] Exp[1/y], {y, 0, 0.7}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fit1 /. x \[Rule] Exp[1/y], {y, 0, 0.7}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fitimp /. x \[Rule] Exp[1/y], {y, 0, 0.7}, 
        PlotStyle \[Rule] {Hue[2/3]}, DisplayFunction \[Rule] Identity], 
      Plot[fitprogram /. x \[Rule] Exp[1/y], {y, 0, 0.7}, 
        PlotStyle \[Rule] {Hue[0]}, DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data[\([1]\)]], 
        PlotStyle \[Rule] {PointSize[0.01]}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, \[IndentingNewLine] (*\ 
            Only\ plot\ points\ that\ were\ used\ in\ the\ fit\ \
*) \[IndentingNewLine]Drop[Drop[data[\([2]\)], {4}], {1}]], 
        PlotStyle \[Rule] {PointSize[0.01]}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data[\([3]\)]], 
        PlotStyle \[Rule] {PointSize[0.01]}, 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction, Axes \[Rule] False, 
      Frame \[Rule] True, 
      FrameLabel \[Rule] {"\<K\>", \*"\"\<\!\(M\^2\)\>\""}, 
      RotateLabel \[Rule] 
        False, \[IndentingNewLine]Epilog \[Rule] {{Hue[2/3], 
            Text["\<Improved\>", { .5, 1.5}, {0, 1}]}, 
          Text["\<Antiperiodic BC's\>", { .4, 1.1}, {\(-1\), \(-1\)}], 
          Text["\<Periodic BC's\>", { .2, 1.2}, {1, 1}], {Hue[0], 
            Text["\<fit to A+B/K\>", { .2, 0.98}, {1, 1}]}}, 
      TextStyle \[Rule] {FontSize \[Rule] 14, 
          FontFamily \[Rule] "\<Times\>"}, 
      FrameTicks \[Rule] {Flatten[{{Map[{1/Log[#], #, {0.025, 0}} &, {10, 
                    100, 1000}]}, 
              Table[Map[{1/Log[#], "\<\>", {0.0125, 0}} &, 
                  Range[2\ 10^i, 10^\((i + 1)\), 10^i]], {i, 0, 5}]}, 2], 
          Automatic}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(\[IndentingNewLine]Two - 
      link\ glueball\ \((mass)\)\^2\ versus\ K\), "\[IndentingNewLine]", 
    \(\(\(for\)\(\ \)\(various\)\(\ \)\(DLCQ\)\(\ \)\(methods\)\(\
\[IndentingNewLine]\)\)\)}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell["Width 600, left at zero to print ", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["pion", "Section"],

Cell["\<\
Data for zero-, one-, and two-link truncations.
Couplings: 0.07642881742 1 0.76809841 -0.1692461589 -0.1857716652 49854.19434 \
0.02426983029 24828.43378 -1.547566329 -2.239030657 0.359250796 0.6160276139 \
0.3349170391 0 0 0 0
Couplings: 0.07642881742 1 0.76809841 -0.1692461589 -0.1857716652 49854.19434 \
0.02426983029 24828.43378 -1.547566329 -2.239030657 0.359250796 0.6160276139 \
0.3349170391 -0.1651354888 0 0 0
Couplings: 0.07642881742 1 0.76809841 -0.1692461589 -0.1857716652 49854.19434 \
0.02426983029 24828.43378 -1.547566329 -2.239030657 0.359250796 0.6160276139 \
0.3349170391 -0.1651354888 0.01668701189 0 0\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
alpha = 0.6564857107477157; 
beta = 0.15; data0 = 
   {{4, 0.70638777392288}, 
    {6, 0.7520705562469681}, 
    {8, 0.778106364941142}, 
    {12, 0.807404881734627}, 
    {16, 0.823831206326639}, 
    {24, 0.8420409842494869}, 
    {32, 0.8521025897781911}, 
    {48, 0.8631120759311199}, 
    {64, 0.869121759009937}, 
    {96, 0.875630111930261}, 
    {128, 0.8791500383101569}, 
    {192, 0.882933184013001}, 
    {256, 0.8849655876508329}, 
    {384, 0.8871381912487311}, 
    {512, 
     0.8882998518033951}}; 
data1 = {{4, -0.119614338763507}, {6, -0.0948366182804441}, 
    {8, -0.07266694614650081}, {12, -0.0334394206355252}, {16, \
0.00055592353476313}, {24, 0.057132731973138604}, {32, 0.102764799246768}, \
{48, 0.172600902916937}, {64, 0.22397952315214603},{96, 0.29514429584742097}, \
{128, 0.342508765334257}, {192, 0.402386979326936}, \
{256,0.43928292454502393}}; 
data2 = 
   {{4, -0.0928687658756146}, 
    {6, -0.06888377847315849}, 
    {8, -0.0480476497506429}, 
    {12, -0.0112691467612988}, 
    {16, 0.0207954821457309}, 
    {24, 0.0747398557360465}, 
    {32, 0.11893276568830799}, 
    {48, 0.18831464675884}}; \
\>", "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(beta\)\" is similar to \
existing symbol \"\!\(Beta\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analyze", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit0 = 
      Fit[data0, {1, 1/x^\((2\ alpha)\), 1/x^\((4\ alpha)\), 1/x}, 
        x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8909136705919527`\)\(\[InvisibleSpace]\)\) - 
      1.2800441118635635`\/x\^2.6259428429908627` + 
      2.022197786429647`\/x\^1.3129714214954313` - 
      1.9146162771030395`\/x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit0 = Fit[data0, {1, 1/Log[x], 1/Log[x]^2, 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.9239272171951162`\)\(\[InvisibleSpace]\)\) - 
      0.6921489698261452`\/x + 0.2594607352479475`\/Log[x]\^2 - 
      0.2483372152112334`\/Log[x]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[data0, fit0, x]\)], "Input"],

Cell[BoxData[
    \(0.0007303437165651002`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fit0] /. x \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({0.9042787275993115`, 
      0.0025875750031162745`, \(-0.012561261752637679`\), \
\(-0.018330198573990642`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fit0 /. x \[Rule] Exp[1/y], {y, 0, 1}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data0], 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -1.46959 2.32672 [
[.21429 .14662 -9 -9 ]
[.21429 .14662 9 0 ]
[.40476 .14662 -9 -9 ]
[.40476 .14662 9 0 ]
[.59524 .14662 -9 -9 ]
[.59524 .14662 9 0 ]
[.78571 .14662 -9 -9 ]
[.78571 .14662 9 0 ]
[.97619 .14662 -3 -9 ]
[.97619 .14662 3 0 ]
[.01131 .04278 -24 -4.5 ]
[.01131 .04278 0 4.5 ]
[.01131 .27545 -24 -4.5 ]
[.01131 .27545 0 4.5 ]
[.01131 .39179 -18 -4.5 ]
[.01131 .39179 0 4.5 ]
[.01131 .50812 -24 -4.5 ]
[.01131 .50812 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .15912 m
.21429 .16537 L
s
[(0.2)] .21429 .14662 0 1 Mshowa
.40476 .15912 m
.40476 .16537 L
s
[(0.4)] .40476 .14662 0 1 Mshowa
.59524 .15912 m
.59524 .16537 L
s
[(0.6)] .59524 .14662 0 1 Mshowa
.78571 .15912 m
.78571 .16537 L
s
[(0.8)] .78571 .14662 0 1 Mshowa
.97619 .15912 m
.97619 .16537 L
s
[(1)] .97619 .14662 0 1 Mshowa
.125 Mabswid
.07143 .15912 m
.07143 .16287 L
s
.11905 .15912 m
.11905 .16287 L
s
.16667 .15912 m
.16667 .16287 L
s
.2619 .15912 m
.2619 .16287 L
s
.30952 .15912 m
.30952 .16287 L
s
.35714 .15912 m
.35714 .16287 L
s
.45238 .15912 m
.45238 .16287 L
s
.5 .15912 m
.5 .16287 L
s
.54762 .15912 m
.54762 .16287 L
s
.64286 .15912 m
.64286 .16287 L
s
.69048 .15912 m
.69048 .16287 L
s
.7381 .15912 m
.7381 .16287 L
s
.83333 .15912 m
.83333 .16287 L
s
.88095 .15912 m
.88095 .16287 L
s
.92857 .15912 m
.92857 .16287 L
s
.25 Mabswid
0 .15912 m
1 .15912 L
s
.02381 .04278 m
.03006 .04278 L
s
[(0.65)] .01131 .04278 1 0 Mshowa
.02381 .27545 m
.03006 .27545 L
s
[(0.75)] .01131 .27545 1 0 Mshowa
.02381 .39179 m
.03006 .39179 L
s
[(0.8)] .01131 .39179 1 0 Mshowa
.02381 .50812 m
.03006 .50812 L
s
[(0.85)] .01131 .50812 1 0 Mshowa
.125 Mabswid
.02381 .06605 m
.02756 .06605 L
s
.02381 .08932 m
.02756 .08932 L
s
.02381 .11258 m
.02756 .11258 L
s
.02381 .13585 m
.02756 .13585 L
s
.02381 .18238 m
.02756 .18238 L
s
.02381 .20565 m
.02756 .20565 L
s
.02381 .22892 m
.02756 .22892 L
s
.02381 .25219 m
.02756 .25219 L
s
.02381 .29872 m
.02756 .29872 L
s
.02381 .32199 m
.02756 .32199 L
s
.02381 .34525 m
.02756 .34525 L
s
.02381 .36852 m
.02756 .36852 L
s
.02381 .41506 m
.02756 .41506 L
s
.02381 .43832 m
.02756 .43832 L
s
.02381 .46159 m
.02756 .46159 L
s
.02381 .48486 m
.02756 .48486 L
s
.02381 .01951 m
.02756 .01951 L
s
.02381 .53139 m
.02756 .53139 L
s
.02381 .55466 m
.02756 .55466 L
s
.02381 .57793 m
.02756 .57793 L
s
.02381 .60119 m
.02756 .60119 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.03279 .60332 L
.04262 .60332 L
.04517 .60332 L
.04642 .60332 L
.04758 .60332 L
.04822 .60332 L
.04891 .60332 L
.0496 .60332 L
.05035 .60332 L
.05166 .60332 L
.05288 .60332 L
.05412 .60332 L
.05529 .60332 L
.05634 .60332 L
.05749 .60332 L
.05873 .60332 L
.05943 .60332 L
.06006 .60332 L
.06129 .60332 L
.06244 .60332 L
.06371 .60332 L
.06489 .60332 L
.06618 .60332 L
.06691 .60332 L
.06757 .60332 L
.06826 .60332 L
.0689 .60332 L
.06961 .60332 L
.07035 .60332 L
.0716 .60332 L
.07232 .60332 L
.07299 .60332 L
.07417 .60332 L
.07542 .60332 L
.07649 .60332 L
.07765 .60332 L
.07891 .60332 L
.07962 .60332 L
.08026 .60332 L
.08151 .60332 L
.08269 .60332 L
.08395 .60332 L
.08534 .60332 L
.08679 .60332 L
.08815 .60332 L
.08942 .60332 L
.09058 .60332 L
.09319 .60331 L
.09464 .60331 L
Mistroke
.09599 .60331 L
.09759 .60331 L
.09903 .60331 L
.10172 .6033 L
.10319 .60329 L
.10458 .60329 L
.107 .60327 L
.10925 .60326 L
.11172 .60323 L
.11437 .6032 L
.11715 .60316 L
.11978 .60311 L
.1247 .60298 L
.1293 .60282 L
.13356 .60261 L
.13823 .60232 L
.14318 .60192 L
.15258 .60087 L
.15786 .60008 L
.16269 .59921 L
.18099 .59456 L
.19081 .59109 L
.19995 .58722 L
.22066 .57622 L
.25976 .54806 L
.30131 .51092 L
.34134 .47193 L
.37986 .43404 L
.42083 .39494 L
.46029 .35921 L
.49823 .32681 L
.53862 .29436 L
.5775 .2649 L
.61882 .23521 L
.65864 .20791 L
.69694 .18264 L
.73768 .1566 L
.77692 .13219 L
.81464 .10922 L
.85481 .08521 L
.89346 .06245 L
.93457 .03858 L
.97416 .01587 L
.97619 .01472 L
Mfstroke
.008 w
.71081 .17398 Mdot
.55534 .28027 Mdot
.48181 .34085 Mdot
.40708 .40902 Mdot
.36731 .44724 Mdot
.32348 .48961 Mdot
.29861 .51302 Mdot
.26983 .53863 Mdot
.25281 .55262 Mdot
.23247 .56776 Mdot
.22009 .57595 Mdot
.20496 .58475 Mdot
.19556 .58948 Mdot
.18386 .59453 Mdot
.17648 .59724 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000900001Hb000`400?l00000o`00003oo`3ooooo
0?oood80oooo003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo
003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0
oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool0
0?l0ooooo`3ooom20?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo
00<000000?ooo`3oool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3o
ool90?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?oo
o`3oool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00
=P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?ooo`3oool0o`3o
oooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0oooonP3oool00`000000oooo0?ooo`0;
0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooooP0oooo0P00000>0?ooo`00=P3oool00`000000
oooo0?ooo`3o0?ooooL0oooo0P00000?0?ooo`00=P3oool300000?l0oooom@3oool200000140oooo
000f0?ooo`030000003oool0oooo0?l0ooool`3oool2000001<0oooo000f0?ooo`030000003oool0
oooo0?l0ooool@3oool3000001@0oooo000f0?ooo`030000003oool0oooo0?l0ooool03oool20000
01H0oooo000f0?ooo`030000003oool0oooo0?l0ooookP3oool2000001P0oooo000f0?ooo`030000
003oool0oooo0?l0ooook03oool2000001X0oooo000f0?ooo`030000003oool0oooo0?l0oooojP3o
ool3000001/0oooo000f0?ooo`030000003oool0oooo0?l0ooooj@3oool2000001d0oooo00040?oo
o`@000002@3oool2000000T0oooo100000070?ooo`D000002P3oool00`000000oooo0?ooo`3o0?oo
onL0oooo0P00000O0?ooo`000`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000L0
oooo0P0000030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo0?l0ooooi@3oool200000240oooo00030?ooo`030000
003oool0oooo0080oooo0P0000070?ooo`<000001`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooon@0oooo
0P00000R0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o`3ooooR0?ooo`800000903oool00080oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`060?ooo`@00000o`3ooooO0?ooo`8000009P3oool00080oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0>0?ooo`800000103oool2000000/0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`3o0?ooomh0oooo0P00000X0?ooo`000P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`3o0?ooomd0oooo0P00000Y0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool00`0000070?ooo`D000002P3oool00`000000oooo
0?ooo`3o0?ooom/0oooo0P00000[0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0?l0oooof@3oool2000002d0oooo00020?ooo`8000001@3oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3o
0?ooomP0oooo0P00000^0?ooo`000`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
0100oooo0P00000:0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3ooooF0?oo
o`800000<03oool000<0oooo0P0000030?ooo`030000003oool0oooo0140oooo0P0000090?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3ooooD0?ooo`800000<P3oool000@0oooo
1000000F0?ooo`@000001@3oool6000000T0oooo00<000000?ooo`3oool0o`3ooooC0?ooo`800000
<`3oool003H0oooo00<000000?ooo`3oool0o`3ooooA0?ooo`800000=@3oool003H0oooo00<00000
0?ooo`3oool0o`3oooo?0?ooo`800000=`3oool003H0oooo00<000000?ooo`3oool0o`3oooo=0?oo
o`800000>@3oool003H0oooo00<000000?ooo`3oool0o`3oooo<0?ooo`800000>P3oool003H0oooo
0`00003o0?ooolX0oooo0P00000l0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooolP0oooo0P00
000n0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooolH0oooo0`00000o0?ooo`00=P3oool00`00
0000oooo0?ooo`3o0?ooolD0oooo0P0000110?ooo`00=P3oool00`000000oooo0?ooo`3o0?oool<0
oooo0P0000130?ooo`00=P3oool00`000000oooo0?ooo`3o0?oool40oooo0P0000150?ooo`00=P3o
ool00`000000oooo0?ooo`3o0?oookl0oooo0`0000160?ooo`00=P3oool00`000000oooo0?ooo`3o
0?oookh0oooo0P0000180?ooo`00=P3oool00`000000oooo0?ooo`3o0?oook`0oooo0P00001:0?oo
o`00=P3oool00`000000oooo0?ooo`3o0?oookX0oooo0P00001<0?ooo`00=P3oool00`000000oooo
0?ooo`3o0?oookT0oooo0P00001=0?ooo`00=P3oool00`000000oooo0?ooo`3o0?oookL0oooo0P00
001?0?ooo`00=P3oool300000?l0oooo]@3oool200000540oooo000f0?ooo`030000003oool0oooo
0?l0oooo]03oool200000580oooo000f0?ooo`030000003oool0oooo0?l0oooo/P3oool2000005@0
oooo000f0?ooo`030000003oool0oooo0?l0oooo/03oool2000005H0oooo000f0?ooo`030000003o
ool0oooo0?l0oooo[`3oool2000005L0oooo000f0?ooo`030000003oool0oooo0?l0oooo[@3oool2
000005T0oooo000f0?ooo`030000003oool0oooo0?l0oooo[03oool2000005X0oooo000f0?ooo`03
0000003oool0oooo0?l0ooooZP3oool2000005`0oooo000f0?ooo`030000003oool0oooo0?l0oooo
Z03oool2000005h0oooo000f0?ooo`030000003oool0oooo0?l0ooooY`3oool2000005l0oooo000f
0?ooo`030000003oool0oooo0?l0ooooY@3oool200000640oooo000f0?ooo`030000003oool0oooo
0?l0ooooY03oool00`000000oooo0?ooo`1Q0?ooo`00=P3oool300000?l0ooooXP3oool2000006@0
oooo000f0?ooo`030000003oool0oooo0?l0ooooX03oool2000006H0oooo000f0?ooo`030000003o
ool0oooo0?l0ooooW`3oool00`000000oooo0?ooo`1V0?ooo`00=P3oool00`000000oooo0?ooo`1E
0?ooo`@000002@3oool2000000L0oooo200000170?ooo`@000002@3oool2000000T0oooo1P000018
0?ooo`@000002@3oool2000000T0oooo100000190?ooo`@000002@3oool2000000P0oooo1@00001D
0?ooo`P000002P3oool003H0oooo00<000000?ooo`3oool0E03oool2000000<0oooo00<000000?oo
o`3oool01P3oool3000000H0oooo0P0000050?ooo`030000003oool0oooo04@0oooo0P0000030?oo
o`030000003oool0oooo00H0oooo0`00000;0?ooo`030000003oool0oooo04L0oooo0P0000030?oo
o`030000003oool0oooo00H0oooo0`0000070?ooo`8000000`3oool00`000000oooo0?ooo`150?oo
o`8000000`3oool00`000000oooo0?ooo`060?ooo`<000001@3oool3000000@0oooo00<000000?oo
o`3oool0E@3oool00`000000oooo0?ooo`0;0?ooo`00=P3oool00`000000oooo0?ooo`1D0?ooo`03
0000003oool0oooo0080oooo0P0000070?ooo`<00000203oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo04@0oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`00000;0?ooo`03
0000003oool0oooo04L0oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`0000070?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`020?ooo`80
00001`3oool3000000@0oooo00<000000?ooo`0000001P3oool00`000000oooo0?ooo`1D0?ooo`03
0000003oool0oooo00/0oooo000f0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0>0?ooo`P00000AP3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`130?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool02P3oool200000080oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo00/0oooo000f0?ooo`030000003o
ool0oooo05<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo04H0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`130?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0203oool2000000@0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo00/0oooo000f0?ooo`030000003oool0
oooo05<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0
oooo04D0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`050000003oool0
oooo0?ooo`000000B03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo0P00
00040?ooo`800000A@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool01@3oool2000000<0oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?oo
o`0;0?ooo`00=P3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool03`3oool01@000000oooo0?ooo`3oool0000004P0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool0A03o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00D0oooo0P0000090?ooo`@00000F03o
ool00`000000oooo0?ooo`0;0?ooo`00=P3oool00`000000oooo0?ooo`1C0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0503oool2000004D0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000004P0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00<00000A`3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00<0oooo0P00000:0?ooo`8000000`3oool00`000000oooo0?oo
o`1E0?ooo`030000003oool0oooo00/0oooo000f0?ooo`030000003oool0oooo05<0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo04<0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000004P0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo04X0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`1D0?ooo`030000003oool0oooo00/0oooo000f
0?ooo`030000003oool0oooo05<0oooo0P0000050?ooo`030000003oool0oooo01D0oooo00<00000
0?ooo`3oool0@`3oool2000000D0oooo00<000000?ooo`3oool04@3oool00`000000oooo00000018
0?ooo`8000001@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo04T0oooo0P000005
0?ooo`030000003oool0oooo008000003@3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo05@0oooo00<000000?ooo`3oool02`3oool003H0oooo0`00001D0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
04@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool00000
04T0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`800000B`3oool00`00
0000oooo0?ooo`020?ooo`040000003oool00000000000l0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`1@0?ooo`050000003oool0oooo0?ooo`0000003@3oool003H0oooo00<00000
0?ooo`3oool0E03oool2000000<0oooo00<000000?ooo`3oool03`3oool2000000<0oooo0P000017
0?ooo`8000000`3oool00`000000oooo0?ooo`0C0?ooo`800000B@3oool2000000<0oooo00<00000
0?ooo`3oool04@3oool2000004X0oooo0P0000030?ooo`<000004@3oool01@000000oooo0?ooo`3o
ool0000005@0oooo1000000=0?ooo`00=P3oool00`000000oooo0?ooo`1E0?ooo`@00000503oool4
000004T0oooo1000000F0?ooo`800000BP3oool4000001H0oooo100000170?ooo`D00000503oool3
000005L0oooo0P00000=0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooohL0oooo0P00001o0?oo
o`00=P3oool00`000000oooo0?ooo`3o0?ooohH0oooo0P0000200?ooo`00=P3oool00`000000oooo
0?ooo`3o0?oooh@0oooo0P0000220?ooo`00=P3oool00`000000oooo0?ooo`3o0?oooh<0oooo0P00
00230?ooo`00=P3oool00`000000oooo0?ooo`3o0?oooh40oooo0P0000250?ooo`00=P3oool00`00
0000oooo0?ooo`3o0?ooogl0oooo0P0000270?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooogh0
oooo0P0000280?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooog`0oooo0P00002:0?ooo`00=P3o
ool00`000000oooo0?ooo`3o0?ooog/0oooo00<000000?ooo`3oool0RP3oool002T0ooooo`00003o
000001P000000@3oool003H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?oo
o`030000003oool0oooo01L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo0180oooo0P0000020?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01H0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0;0?ooo`00=P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo01H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool04@3oool2000000<0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3o
ool05P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00/0oooo000f0?ooo`030000
003oool0oooo06<0oooo00<000000?ooo`3oool0HP3oool00`000000oooo0?ooo`1S0?ooo`030000
003oool0oooo0480oooo0P00000N0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3oool0
2`3oool003H0oooo00<000000?ooo`3oool0o`3ooomc0?ooo`800000T`3oool003H0oooo00<00000
0?ooo`3oool0o`3oooma0?ooo`800000U@3oool003H0oooo00<000000?ooo`3oool0o`3ooom`0?oo
o`030000003oool0oooo09D0oooo000f0?ooo`030000003oool0oooo0?l0ooooK03oool4000009P0
oooo000f0?ooo`030000003oool0oooo0?l0ooooJ`3oool4000009T0oooo000f0?ooo`030000003o
ool0oooo0?l0ooooJ`3oool4000009T0oooo000f0?ooo`030000003oool0oooo0?l0ooooJ@3oool2
00000080oooo00<000000?ooo`3oool0V03oool003H0oooo00<000000?ooo`3oool0o`3ooomX0?oo
o`800000WP3oool003H0oooo00<000000?ooo`3oool0o`3ooomV0?ooo`800000X03oool003H0oooo
0`00003o0?ooofD0oooo00<000000?ooo`3oool0X03oool003H0oooo00<000000?ooo`3oool0o`3o
oomS0?ooo`800000X`3oool003H0oooo00<000000?ooo`3oool0o`3ooomR0?ooo`030000003oool0
oooo0:<0oooo000f0?ooo`030000003oool0oooo0?l0ooooH03oool200000:H0oooo000f0?ooo`03
0000003oool0oooo0?l0ooooGP3oool200000:P0oooo000f0?ooo`030000003oool0oooo0?l0oooo
G@3oool200000:T0oooo000f0?ooo`030000003oool0oooo0?l0ooooF`3oool200000:/0oooo000f
0?ooo`030000003oool0oooo0?l0ooooFP3oool200000:`0oooo000f0?ooo`030000003oool0oooo
0?l0ooooF03oool200000:h0oooo000f0?ooo`030000003oool0oooo0?l0ooooE`3oool200000:l0
oooo000f0?ooo`030000003oool0oooo0?l0ooooE@3oool200000;40oooo000f0?ooo`030000003o
ool0oooo0?l0ooooE03oool00`000000oooo0?ooo`2a0?ooo`00=P3oool300000?l0ooooDP3oool2
00000;@0oooo000f0?ooo`030000003oool0oooo0?l0ooooD@3oool00`000000oooo0?ooo`2d0?oo
o`00=P3oool00`000000oooo0?ooo`3o0?ooodl0oooo0P00002g0?ooo`00=P3oool00`000000oooo
0?ooo`3o0?ooodh0oooo00<000000?ooo`3oool0]`3oool003H0oooo00<000000?ooo`3oool0o`3o
oom<0?ooo`800000^P3oool003H0oooo00<000000?ooo`3oool0o`3ooom;0?ooo`800000^`3oool0
03H0oooo00<000000?ooo`3oool0o`3ooom90?ooo`800000_@3oool003H0oooo00<000000?ooo`3o
ool0o`3ooom80?ooo`800000_P3oool003H0oooo00<000000?ooo`3oool0o`3ooom60?ooo`800000
`03oool003H0oooo00<000000?ooo`3oool0o`3ooom50?ooo`800000`@3oool003H0oooo00<00000
0?ooo`3oool0o`3ooom30?ooo`800000``3oool003H0oooo00<000000?ooo`3oool0o`3ooom20?oo
o`800000a03oool003H0oooo0`00003o0?oood00oooo0P0000360?ooo`00=P3oool00`000000oooo
0?ooo`3o0?ooocl0oooo0P0000370?ooo`00=P3oool00`000000oooo0?ooo`3o0?oooch0oooo00<0
00000?ooo`3oool0a`3oool003H0oooo00<000000?ooo`3oool0o`3oooll0?ooo`800000bP3oool0
03H0oooo00<000000?ooo`3oool0o`3ooolk0?ooo`030000003oool0oooo0<X0oooo000f0?ooo`03
0000003oool0oooo0?l0oooo>@3oool200000<d0oooo000f0?ooo`030000003oool0oooo0?l0oooo
>03oool200000<h0oooo000f0?ooo`030000003oool0oooo0?l0oooo=`3oool00`000000oooo0?oo
o`3>0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooocD0oooo0P00003A0?ooo`00=P3oool00`00
0000oooo0?ooo`3o0?oooc@0oooo00<000000?ooo`3oool0d@3oool003H0oooo00<000000?ooo`3o
ool0o`3ooolb0?ooo`800000e03oool003H0oooo00<000000?ooo`3oool0o`3ooola0?ooo`800000
e@3oool003H0oooo00<000000?ooo`3oool0o`3oool`0?ooo`030000003oool0oooo0=D0oooo000f
0?ooo`<00000o`3oool^0?ooo`800000f03oool003H0oooo00<000000?ooo`3oool0o`3oool]0?oo
o`800000f@3oool003H0oooo00<000000?ooo`3oool0o`3oool/0?ooo`030000003oool0oooo0=T0
oooo000f0?ooo`030000003oool0oooo0?l0oooo:P3oool200000=`0oooo000f0?ooo`030000003o
ool0oooo0?l0oooo:@3oool00`000000oooo0?ooo`3L0?ooo`00=P3oool00`000000oooo0?ooo`3o
0?ooobL0oooo0P00003O0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooobH0oooo0P00003P0?oo
o`00=P3oool00`000000oooo0?ooo`3o0?ooobD0oooo00<000000?ooo`3oool0h03oool000@0oooo
100000090?ooo`8000002P3oool00`000000oooo0?ooo`070?ooo`D000002P3oool00`000000oooo
0?ooo`3o0?ooob<0oooo0P00003S0?ooo`000`3oool2000000<0oooo00<000000?ooo`3oool01P3o
ool3000000T0oooo0P0000070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`3o0?ooob80oooo0P00003T0?ooo`000`3oool00`000000oooo0?ooo`02
0?ooo`8000001`3oool3000000X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0?l0oooo8@3oool00`000000oooo0?ooo`3T0?ooo`000P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo7`3oool200000>L0oooo00020?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool2000000d0oooo00<000000?ooo`3o
ool01P3oool400000?l0oooo7@3oool200000>P0oooo00020?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0o`3ooolI0?ooo`<0000000<0oooo0000003oool0j@3oool00080oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo603o
ool500000>/0oooo00020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04`3oool2
000000H0oooo1@00000:0?ooo`030000003oool0oooo0?l0oooo6@3oool300000>`0oooo00020?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3ooolI0?ooo`800000k@3oool00080
oooo0P0000050?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0oooo5`3oool200000>l0oooo00030?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`800000
1@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0oooo5P3oool200000?00oooo
00030?ooo`8000000`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0oooo5@3o
ool00`000000oooo0?ooo`3`0?ooo`00103oool400000180oooo200000050?ooo`H000002@3oool0
0`000000oooo0?ooo`3o0?oooa<0oooo0P00003c0?ooo`00=P3oool00`000000oooo0?ooo`3o0?oo
oa80oooo0P00003d0?ooo`00=P3oool00`000000oooo0?ooo`3o0?oooa40oooo00<000000?ooo`3o
ool0m03oool003H0oooo00<000000?ooo`3oool0o`3oool?0?ooo`800000m`3oool003H0oooo00<0
00000?ooo`3oool0o`3oool>0?ooo`800000n03oool003H0oooo0`00003o0?ooo`d0oooo0P00003i
0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooo``0oooo00<000000?ooo`3oool0n@3oool003H0
oooo00<000000?ooo`3oool0o`3oool;0?ooo`030000003oool0oooo0?X0oooo000f0?ooo`030000
003oool0oooo0?l0oooo2@3oool200000?d0oooo000f0?ooo`030000003oool0oooo0?l0oooo203o
ool200000?h0oooo000f0?ooo`030000003oool0oooo0?l0oooo1`3oool200000?l0oooo000f0?oo
o`030000003oool0oooo0?l0oooo1P3oool00`000000oooo0?ooo`3o0?ooo`00=P3oool00`000000
oooo0?ooo`3o0?ooo`@0oooo0P00003o0?ooo`<0oooo000f0?ooo`030000003oool0oooo0?l0oooo
0`3oool200000?l0oooo103oool003H0oooo00<000000?ooo`3oool0o`3oool20?ooo`800000o`3o
ool50?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooo`40oooo00<000000?ooo`3oool0o`3oool5
0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooo`030000003oool0oooo0?l0oooo1P3oool003H0
oooo0`00003m0?ooo`800000o`3oool90?ooo`00=P3oool00`000000oooo0?ooo`3l0?ooo`800000
o`3oool:0?ooo`00=P3oool00`000000oooo0?ooo`3k0?ooo`800000o`3oool;0?ooo`00=P3oool0
0`000000oooo0?ooo`3j0?ooo`030000003oool0oooo0?l0oooo2`3oool003H0oooo00<000000?oo
o`3oool0n@3oool00`000000oooo0?ooo`3o0?ooo``0oooo000f0?ooo`030000003oool0oooo0?L0
oooo0P00003o0?ooo`l0oooo000f0?ooo`030000003oool0oooo0?H0oooo0P00003o0?oooa00oooo
000f0?ooo`030000003oool0oooo0?D0oooo0P00003o0?oooa40oooo000f0?ooo`030000003oool0
oooo0?@0oooo00<000000?ooo`3oool0o`3ooolA0?ooo`00=P3oool00`000000oooo0?ooo`3a0?oo
o`<00000o`3ooolD0?ooo`00=P3oool00`000000oooo0?ooo`3`0?ooo`@00000o`3ooolD0?ooo`00
=P3oool00`000000oooo0?ooo`3`0?ooo`@00000o`3ooolD0?ooo`00=P3oool300000>l0oooo0P00
00000`3oool000000?ooo`3o0?oooa@0oooo000f0?ooo`030000003oool0oooo0>h0oooo0P00003o
0?oooaP0oooo000f0?ooo`030000003oool0oooo0>d0oooo00<000000?ooo`3oool0o`3ooolH0?oo
o`00=P3oool00`000000oooo0?ooo`3/0?ooo`030000003oool0oooo0?l0oooo6@3oool003H0oooo
00<000000?ooo`3oool0jP3oool200000?l0oooo703oool003H0oooo00<000000?ooo`3oool0j@3o
ool200000?l0oooo7@3oool003H0oooo00<000000?ooo`3oool0j03oool200000?l0oooo7P3oool0
03H0oooo00<000000?ooo`3oool0i`3oool00`000000oooo0?ooo`3o0?oooah0oooo000f0?ooo`03
0000003oool0oooo0>H0oooo00<000000?ooo`3oool0o`3ooolO0?ooo`00=P3oool00`000000oooo
0?ooo`3U0?ooo`030000003oool0oooo0?l0oooo803oool003H0oooo00<000000?ooo`3oool0i03o
ool00`000000oooo0?ooo`3o0?ooob40oooo000f0?ooo`030000003oool0oooo0>80oooo0P00003o
0?ooob@0oooo000f0?ooo`030000003oool0oooo0>40oooo0P00003o0?ooobD0oooo000f0?ooo`<0
0000h03oool200000?l0oooo9P3oool003H0oooo00<000000?ooo`3oool0g`3oool200000?l0oooo
9`3oool003H0oooo00<000000?ooo`3oool0gP3oool200000?l0oooo:03oool003H0oooo00<00000
0?ooo`3oool0g@3oool00`000000oooo0?ooo`3o0?ooobP0oooo000f0?ooo`030000003oool0oooo
0=`0oooo00<000000?ooo`3oool0o`3ooolY0?ooo`00=P3oool00`000000oooo0?ooo`3K0?ooo`03
0000003oool0oooo0?l0oooo:P3oool003H0oooo00<000000?ooo`3oool0fP3oool00`000000oooo
0?ooo`3o0?ooob/0oooo000f0?ooo`030000003oool0oooo0=P0oooo0P00003o0?ooobh0oooo000@
0?ooo`@000002@3oool2000000T0oooo1000000:0?ooo`030000003oool0oooo0=L0oooo0P00003o
0?ooobl0oooo000?0?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`<000001`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0eP3oool20000
0?l0oooo<03oool000l0oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`0000060?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3E0?ooo`80
0000o`3ooola0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0=@0oooo
0P00003o0?oooc80oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00H0oooo1000003B0?ooo`030000003o
ool0oooo0?l0oooo<P3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?
0?ooo`8000000`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0=80oooo00<00000
0?ooo`3oool0o`3ooolc0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0100oooo1000000:0?ooo`030000003oool0oooo0=40oooo00<000000?ooo`3oool0o`3ooold0?oo
o`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo0P0000030?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool0d03oool00`000000oooo0?ooo`3o0?ooocD0
oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0c`3oool00`000000oooo
0?ooo`3o0?ooocH0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3>0?ooo`030000003o
ool0oooo0?l0oooo=`3oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3=
0?ooo`030000003oool0oooo0?l0oooo>03oool000l0oooo0P0000030?ooo`030000003oool0oooo
0140oooo00D000000?ooo`3oool0oooo000000090?ooo`030000003oool0oooo0<`0oooo00<00000
0?ooo`3oool0o`3oooli0?ooo`00403oool4000001D0oooo0`00000:0?ooo`030000003oool0oooo
0<T0oooo0`00003o0?oooc`0oooo000f0?ooo`030000003oool0oooo0<P0oooo1000003o0?oooc`0
oooo000f0?ooo`030000003oool0oooo0<P0oooo1000003o0?oooc`0oooo000f0?ooo`030000003o
ool0oooo0<L0oooo0P0000000`3oool000000?ooo`3o0?oooc`0oooo000f0?ooo`030000003oool0
oooo0<H0oooo0P00003o0?oood00oooo000f0?ooo`<00000a@3oool200000?l0oooo@@3oool003H0
oooo00<000000?ooo`3oool0a03oool200000?l0oooo@P3oool003H0oooo00<000000?ooo`3oool0
``3oool200000?l0oooo@`3oool003H0oooo00<000000?ooo`3oool0`P3oool200000?l0ooooA03o
ool003H0oooo00<000000?ooo`3oool0`@3oool200000?l0ooooA@3oool003H0oooo00<000000?oo
o`3oool0`03oool00`000000oooo0?ooo`3o0?ooodD0oooo000f0?ooo`030000003oool0oooo0;l0
oooo00<000000?ooo`3oool0o`3ooom60?ooo`00=P3oool00`000000oooo0?ooo`2n0?ooo`030000
003oool0oooo0?l0ooooA`3oool003H0oooo00<000000?ooo`3oool0_@3oool00`000000oooo0?oo
o`3o0?ooodP0oooo000f0?ooo`030000003oool0oooo0;`0oooo00<000000?ooo`3oool0o`3ooom9
0?ooo`00=P3oool00`000000oooo0?ooo`2k0?ooo`030000003oool0oooo0?l0ooooBP3oool003H0
oooo00<000000?ooo`3oool0^P3oool00`000000oooo0?ooo`3o0?oood/0oooo000f0?ooo`<00000
^@3oool00`000000oooo0?ooo`3o0?oood`0oooo000f0?ooo`030000003oool0oooo0;P0oooo00<0
00000?ooo`3oool0o`3ooom=0?ooo`00=P3oool00`000000oooo0?ooo`2g0?ooo`030000003oool0
oooo0?l0ooooCP3oool003H0oooo00<000000?ooo`3oool0]03oool300000?l0ooooD@3oool003H0
oooo00<000000?ooo`3oool0/`3oool400000?l0ooooD@3oool003H0oooo00<000000?ooo`3oool0
/`3oool400000?l0ooooD@3oool003H0oooo00<000000?ooo`3oool0/P3oool2000000030?ooo`00
0000oooo0?l0ooooD@3oool003H0oooo00<000000?ooo`3oool0/@3oool200000?l0ooooE@3oool0
03H0oooo00<000000?ooo`3oool0/03oool200000?l0ooooEP3oool003H0oooo00<000000?ooo`3o
ool0[`3oool200000?l0ooooE`3oool003H0oooo00<000000?ooo`3oool0[P3oool200000?l0oooo
F03oool003H0oooo00<000000?ooo`3oool0[@3oool200000?l0ooooF@3oool003H0oooo0`00002/
0?ooo`800000o`3ooomJ0?ooo`00=P3oool00`000000oooo0?ooo`2[0?ooo`030000003oool0oooo
0?l0ooooFP3oool003H0oooo00<000000?ooo`3oool0ZP3oool00`000000oooo0?ooo`3o0?oooe/0
oooo000f0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool0o`3ooomL0?ooo`00=P3o
ool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo0?l0ooooG@3oool003H0oooo00<00000
0?ooo`3oool0Y`3oool00`000000oooo0?ooo`3o0?oooeh0oooo000f0?ooo`030000003oool0oooo
0:H0oooo00<000000?ooo`3oool0o`3ooomO0?ooo`00=P3oool00`000000oooo0?ooo`2U0?ooo`03
0000003oool0oooo0?l0ooooH03oool003H0oooo00<000000?ooo`3oool0Y03oool00`000000oooo
0?ooo`3o0?ooof40oooo000f0?ooo`030000003oool0oooo0:<0oooo00<000000?ooo`3oool0o`3o
oomR0?ooo`00=P3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo0?l0ooooH`3oool0
03H0oooo00<000000?ooo`3oool0X@3oool00`000000oooo0?ooo`3o0?ooof@0oooo000f0?ooo`03
0000003oool0oooo0:00oooo00<000000?ooo`3oool0o`3ooomU0?ooo`00=P3oool3000009l0oooo
00<000000?ooo`3oool0o`3ooomV0?ooo`00=P3oool00`000000oooo0?ooo`2L0?ooo`<00000o`3o
oomY0?ooo`00=P3oool00`000000oooo0?ooo`2K0?ooo`@00000o`3ooomY0?ooo`00=P3oool00`00
0000oooo0?ooo`2L0?ooo`<00000o`3ooomY0?ooo`00=P3oool00`000000oooo0?ooo`2K0?ooo`03
0000003oool000000?l0ooooJP3oool003H0oooo00<000000?ooo`3oool0VP3oool00`000000oooo
0?ooo`3o0?ooof/0oooo000f0?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3oool0o`3o
oom/0?ooo`00=P3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo0?l0ooooK@3oool0
00@0oooo100000090?ooo`8000002@3oool4000000L0oooo1@00000:0?ooo`030000003oool0oooo
09L0oooo00<000000?ooo`3oool0o`3ooom^0?ooo`000`3oool2000000<0oooo00<000000?ooo`3o
ool01P3oool3000000L0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2F0?ooo`03
0000003oool0oooo0?l0ooooK`3oool000<0oooo00<000000?ooo`3oool00P3oool2000000L0oooo
0`0000060?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0o`3ooom`0?ooo`000P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0U03o
ool00`000000oooo0?ooo`3o0?ooog40oooo00020?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool01P3oool400000980oooo00<000000?ooo`3oool0o`3ooomb0?ooo`000P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo0P0000030?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2?0?ooo`@00000o`3ooome0?oo
o`000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo100000070?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`2>0?ooo`@0
0000o`3ooomf0?ooo`000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo
0P0000030?ooo`030000003oool0oooo00@0oooo1@00000:0?ooo`030000003oool0oooo08l0oooo
0`00003o0?ooogH0oooo00020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0303o
ool00`000000oooo0?ooo`2>0?ooo`<00000o`3ooomg0?ooo`000P3oool2000000D0oooo00<00000
0?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`2=0?ooo`800000o`3ooomi0?ooo`000`3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0S03oool20000
0?l0ooooNP3oool000<0oooo0P0000030?ooo`030000003oool0oooo0140oooo00D000000?ooo`3o
ool0oooo000000060?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0R`3oool00`00
0000oooo0?ooo`3o0?ooogX0oooo00040?ooo`@000005@3oool3000000L0oooo1P0000090?ooo`03
0000003oool0oooo08X0oooo00<000000?ooo`3oool0o`3ooomk0?ooo`00=P3oool00`000000oooo
0?ooo`290?ooo`030000003oool0oooo0?l0ooooO03oool003H0oooo00<000000?ooo`3oool0Q`3o
ool200000?l0ooooO`3oool003H0oooo00<000000?ooo`3oool0QP3oool200000?l0ooooP03oool0
03H0oooo00<000000?ooo`3oool0Q@3oool200000?l0ooooP@3oool003H0oooo0`0000240?ooo`80
0000o`3ooon20?ooo`00=P3oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo0?l0oooo
PP3oool003H0oooo00<000000?ooo`3oool0P03oool300000?l0ooooQ@3oool003H0oooo00<00000
0?ooo`3oool0O`3oool400000?l0ooooQ@3oool003H0oooo00<000000?ooo`3oool0O`3oool40000
0?l0ooooQ@3oool003H0oooo00<000000?ooo`3oool0OP3oool2000000030?ooo`000000oooo0?l0
ooooQ@3oool003H0oooo00<000000?ooo`3oool0O@3oool200000?l0ooooR@3oool003H0oooo00<0
00000?ooo`3oool0O03oool00`000000oooo0?ooo`3o0?ooohT0oooo000f0?ooo`030000003oool0
oooo07/0oooo00<000000?ooo`3oool0o`3ooon:0?ooo`00=P3oool00`000000oooo0?ooo`1i0?oo
o`800000o`3ooon=0?ooo`00=P3oool00`000000oooo0?ooo`1g0?ooo`<00000o`3ooon>0?ooo`00
=P3oool00`000000oooo0?ooo`1f0?ooo`@00000o`3ooon>0?ooo`00=P3oool3000007D0oooo1@00
003o0?ooohh0oooo000f0?ooo`030000003oool0oooo07@0oooo00D000000?ooo`3oool0oooo0000
003o0?ooohl0oooo000f0?ooo`030000003oool0oooo0780oooo0P00003o0?oooi@0oooo000f0?oo
o`030000003oool0oooo0740oooo0P00003o0?oooiD0oooo000f0?ooo`030000003oool0oooo06l0
oooo0P00003o0?oooiL0oooo000f0?ooo`030000003oool0oooo06h0oooo0P00003o0?oooiP0oooo
000f0?ooo`030000003oool0oooo06`0oooo0`00003o0?oooiT0oooo000f0?ooo`030000003oool0
oooo06/0oooo1000003o0?oooiT0oooo000f0?ooo`030000003oool0oooo06X0oooo1@00003o0?oo
oiT0oooo000f0?ooo`030000003oool0oooo06P0oooo0P0000030?ooo`030000003oool0oooo0?l0
ooooV03oool003H0oooo00<000000?ooo`3oool0I@3oool400000?l0ooooW`3oool003H0oooo00<0
00000?ooo`3oool0I03oool400000?l0ooooX03oool003H0oooo0`00001T0?ooo`@00000o`3ooonP
0?ooo`00=P3oool00`000000oooo0?ooo`1R0?ooo`8000000P3oool00`000000oooo0?ooo`3o0?oo
oil0oooo000f0?ooo`030000003oool0oooo0600oooo0P00003o0?ooojH0oooo000f0?ooo`030000
003oool0oooo05d0oooo0`00003o0?ooojP0oooo000f0?ooo`030000003oool0oooo05`0oooo1000
003o0?ooojP0oooo000f0?ooo`030000003oool0oooo05P0oooo2000003o0?ooojP0oooo000f0?oo
o`030000003oool0oooo05L0oooo100000030?ooo`030000003oool0oooo0?l0ooooY`3oool003H0
oooo00<000000?ooo`3oool0E@3oool600000?l0oooo[@3oool003H0oooo00<000000?ooo`3oool0
DP3oool3000000@0oooo00<000000?ooo`3oool0o`3ooon/0?ooo`00=P3oool00`000000oooo0?oo
o`1>0?ooo`L00000o`3ooonc0?ooo`00=P3oool00`000000oooo0?ooo`1<0?ooo`D0000000@0oooo
0000000000000000o`3ooonc0?ooo`00=P3oool00`000000oooo0?ooo`190?ooo`@0000000@0oooo
00000000000000000P3oool00`000000oooo0?ooo`3o0?oook80oooo000f0?ooo`030000003oool0
oooo04@0oooo1@0000060?ooo`030000003oool0oooo0?l0oooo]P3oool003H0oooo0`00000i0?oo
o`/00000o`3oooo40?ooo`00=P3ooolm00000?l0oooocP3oool003H0oooo00<000000?ooo`3oool0
o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00=P3oool0
0`000000oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?ooo`3oool0o`3ooooo0?oo
o`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00=P3oool00`000000oooo
0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?ooo`3oool0o`3ooooo0?ooo`T0oooo000f
0?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00o`3ooooo0?oood80oooo003o0?ooool0
oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool0
0?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool00?l0ooooo`3o
oom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00
\
\>"],
  ImageRangeCache->{{{91.5625, 320.938}, {255, 113.688}} -> {-0.560228, \
0.851427, 0.00395151, 0.00161744}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit1 = 
      Fit[Drop[data1, 1], {1, 1/x^\((2\ beta)\), 1/Sqrt[x], 1/x, 1/x^2}, 
        x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8416913501749184`\)\(\[InvisibleSpace]\)\) - 
      9.89572943639118`\/x\^2 + 7.320057130286133`\/x - 
      2.597348518756789`\/\@x - 1.4072567722626779`\/x\^0.3`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[data1, fit1, x]\)], "Input"],

Cell[BoxData[
    \(0.01512366368989173`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fit1] /. x \[Rule] 10\)], "Input"],

Cell[BoxData[
    \({0.8416913501749184`, \(-0.0989572943639118`\), 
      0.7320057130286133`, \(-0.8213537196536025`\), \
\(-0.7052991286935758`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fit1 /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data1], 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.141392 0.548808 [
[.2619 .12889 -9 -9 ]
[.2619 .12889 9 0 ]
[.5 .12889 -9 -9 ]
[.5 .12889 9 0 ]
[.7381 .12889 -9 -9 ]
[.7381 .12889 9 0 ]
[.97619 .12889 -9 -9 ]
[.97619 .12889 9 0 ]
[.01131 .03163 -24 -4.5 ]
[.01131 .03163 0 4.5 ]
[.01131 .25115 -18 -4.5 ]
[.01131 .25115 0 4.5 ]
[.01131 .36091 -18 -4.5 ]
[.01131 .36091 0 4.5 ]
[.01131 .47068 -18 -4.5 ]
[.01131 .47068 0 4.5 ]
[.01131 .58044 -18 -4.5 ]
[.01131 .58044 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .14139 m
.2619 .14764 L
s
[(0.2)] .2619 .12889 0 1 Mshowa
.5 .14139 m
.5 .14764 L
s
[(0.4)] .5 .12889 0 1 Mshowa
.7381 .14139 m
.7381 .14764 L
s
[(0.6)] .7381 .12889 0 1 Mshowa
.97619 .14139 m
.97619 .14764 L
s
[(0.8)] .97619 .12889 0 1 Mshowa
.125 Mabswid
.08333 .14139 m
.08333 .14514 L
s
.14286 .14139 m
.14286 .14514 L
s
.20238 .14139 m
.20238 .14514 L
s
.32143 .14139 m
.32143 .14514 L
s
.38095 .14139 m
.38095 .14514 L
s
.44048 .14139 m
.44048 .14514 L
s
.55952 .14139 m
.55952 .14514 L
s
.61905 .14139 m
.61905 .14514 L
s
.67857 .14139 m
.67857 .14514 L
s
.79762 .14139 m
.79762 .14514 L
s
.85714 .14139 m
.85714 .14514 L
s
.91667 .14139 m
.91667 .14514 L
s
.25 Mabswid
0 .14139 m
1 .14139 L
s
.02381 .03163 m
.03006 .03163 L
s
[(-0.2)] .01131 .03163 1 0 Mshowa
.02381 .25115 m
.03006 .25115 L
s
[(0.2)] .01131 .25115 1 0 Mshowa
.02381 .36091 m
.03006 .36091 L
s
[(0.4)] .01131 .36091 1 0 Mshowa
.02381 .47068 m
.03006 .47068 L
s
[(0.6)] .01131 .47068 1 0 Mshowa
.02381 .58044 m
.03006 .58044 L
s
[(0.8)] .01131 .58044 1 0 Mshowa
.125 Mabswid
.02381 .05907 m
.02756 .05907 L
s
.02381 .08651 m
.02756 .08651 L
s
.02381 .11395 m
.02756 .11395 L
s
.02381 .16883 m
.02756 .16883 L
s
.02381 .19627 m
.02756 .19627 L
s
.02381 .22371 m
.02756 .22371 L
s
.02381 .27859 m
.02756 .27859 L
s
.02381 .30603 m
.02756 .30603 L
s
.02381 .33347 m
.02756 .33347 L
s
.02381 .38836 m
.02756 .38836 L
s
.02381 .4158 m
.02756 .4158 L
s
.02381 .44324 m
.02756 .44324 L
s
.02381 .49812 m
.02756 .49812 L
s
.02381 .52556 m
.02756 .52556 L
s
.02381 .553 m
.02756 .553 L
s
.02381 .00419 m
.02756 .00419 L
s
.02381 .60788 m
.02756 .60788 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.02846 .60332 L
.03068 .60332 L
.03178 .60332 L
.03279 .60332 L
.03398 .60332 L
.03527 .60332 L
.03637 .60332 L
.03754 .60332 L
.03878 .60332 L
.03995 .60332 L
.04124 .60332 L
.04196 .60332 L
.04262 .60332 L
.04384 .60332 L
.04514 .60332 L
.04636 .60332 L
.04748 .60332 L
.04879 .60332 L
.04953 .60332 L
.05021 .60332 L
.05146 .60332 L
.05278 .60332 L
.05393 .60331 L
.05516 .60331 L
.05633 .60331 L
.05739 .6033 L
.05863 .60329 L
.05999 .60328 L
.06126 .60326 L
.06244 .60324 L
.06486 .60319 L
.06621 .60315 L
.06745 .6031 L
.06974 .60299 L
.07216 .60283 L
.07481 .6026 L
.07728 .60233 L
.07998 .60194 L
.08288 .60143 L
.08872 .60002 L
.09425 .59816 L
.09921 .59602 L
.10458 .59314 L
.11404 .58663 L
.12446 .57728 L
.1455 .5518 L
.18584 .48451 L
.22466 .4099 L
.26594 .33378 L
Mistroke
.3057 .27092 L
.34395 .22245 L
.38464 .1833 L
.42382 .1559 L
.44384 .14508 L
.46546 .13537 L
.50558 .12164 L
.54663 .11168 L
.58617 .10442 L
.6242 .09852 L
.66467 .09252 L
.70363 .08642 L
.74505 .07912 L
.78494 .07103 L
.82333 .06209 L
.86416 .05128 L
.90348 .03957 L
.94128 .02715 L
.97619 .01472 L
Mfstroke
.008 w
.88256 .07575 Mdot
.68823 .08934 Mdot
.59631 .10151 Mdot
.50289 .12304 Mdot
.45318 .1417 Mdot
.3984 .17275 Mdot
.36731 .19779 Mdot
.33133 .23612 Mdot
.31006 .26431 Mdot
.28463 .30337 Mdot
.26917 .32936 Mdot
.25024 .36222 Mdot
.2385 .38247 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000900001Hb000`400?l00000o`00003oo`3ooooo
0?oood80oooo003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo
003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0
oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool0
0?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool003H0oooo00<0
00000?ooo`3oool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool9
0?ooo`00=P3oool300000?l0ooooo`3oool90?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooool0
oooo2@3oool003H0oooo00<000000?ooo`3oool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003o
ool0oooo0?l0ooooo`3oool90?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool0
03H0oooo00<000000?ooo`3oool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0
ooool`3oool2000001<0oooo000f0?ooo`030000003oool0oooo0?l0ooool03oool3000001D0oooo
000f0?ooo`030000003oool0oooo0?l0ooook@3oool3000001P0oooo000f0?ooo`030000003oool0
oooo0?l0oooojP3oool3000001/0oooo000@0?ooo`@000002@3oool2000000L0oooo200000080?oo
o`030000003oool0oooo0?l0ooooi`3oool3000001h0oooo000?0?ooo`8000000`3oool00`000000
oooo0?ooo`060?ooo`<000001P3oool2000000D0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`3o0?ooon@0oooo0`00000Q0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`800000
1`3oool3000000P0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo0?l0ooooh@3oool3000002@0oooo000>0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0?l0oooogP3o
ool3000002L0oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0oooof`3oool3000002X0oooo000>0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`080?oo
o`@00000o`3ooooG0?ooo`<00000;@3oool000@0oooo100000060?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0?l0
ooooeP3oool3000002l0oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
503oool2000000P0oooo00<000000?ooo`3oool0o`3ooooC0?ooo`<00000<P3oool000h0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o`3oooo@0?ooo`<00000=@3oool000h0oooo0P0000050?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooold0oooo0`00000h
0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooob@3oool40000
03/0oooo000?0?ooo`8000000`3oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool2000000T0
oooo00<000000?ooo`3oool0o`3oooo60?ooo`<00000?`3oool00100oooo1000000D0?ooo`@00000
2P3oool00`000000oooo0?ooo`3o0?oool80oooo100000120?ooo`00=P3oool00`000000oooo0?oo
o`3o0?oookl0oooo0`0000160?ooo`00=P3oool00`000000oooo0?ooo`3o0?oook/0oooo10000019
0?ooo`00=P3oool00`000000oooo0?ooo`3o0?oookP0oooo0`00001=0?ooo`00=P3oool00`000000
oooo0?ooo`3o0?oook@0oooo1000001@0?ooo`00=P3oool00`000000oooo0?ooo`3o0?oook40oooo
0`00001D0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooojd0oooo1000001G0?ooo`00=P3oool3
00000?l0ooooZP3oool3000005/0oooo000f0?ooo`030000003oool0oooo0?l0ooooYP3oool40000
05h0oooo000f0?ooo`030000003oool0oooo0?l0ooooXP3oool400000680oooo000f0?ooo`030000
003oool0oooo0?l0ooooWP3oool5000006D0oooo000f0?ooo`030000003oool0oooo0?l0ooooVP3o
ool5000006T0oooo000f0?ooo`030000003oool0oooo0?l0ooooUP3oool5000006d0oooo000f0?oo
o`030000003oool0oooo0?l0ooooTP3oool500000740oooo000f0?ooo`030000003oool0oooo0?l0
ooooSP3oool5000007D0oooo000f0?ooo`030000003oool0oooo0?l0ooooR@3oool5000003D0oooo
0`0000120?ooo`00=P3oool00`000000oooo0?ooo`3o0?oooh<0oooo1P00000i0?ooo`@00000@P3o
ool003H0oooo00<000000?ooo`3oool0o`3ooomn0?ooo`D00000@03oool300000480oooo000f0?oo
o`030000003oool0oooo0?l0ooooN@3oool5000004H0oooo00<000000?ooo`3oool0@@3oool003H0
oooo00<000000?ooo`3oool0o`3ooomc0?ooo`H00000S`3oool003H0oooo00<000000?ooo`3oool0
o`3ooom^0?ooo`D00000U@3oool003H0oooo00<000000?ooo`3oool0o`3ooomX0?ooo`H00000VP3o
ool003H0oooo0`00003o0?oooe`0oooo0`0000030?ooo`H00000X03oool003H0oooo00<000000?oo
o`3oool0o`3ooomK0?ooo`L00000YP3oool003H0oooo00<000000?ooo`3oool0o`3ooomD0?ooo`L0
000000@0oooo0000000000000000Z@3oool003H0oooo00<000000?ooo`3oool0o`3ooom=0?ooo`L0
00002@3oool00`000000oooo0?ooo`2X0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooodH0oooo
1`00002k0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooocl0oooo1`0000320?ooo`00=P3oool0
0`000000oooo0?ooo`1]0?ooo`@000002@3oool2000000L0oooo2000001O0?ooo`@000002@3oool2
000000T0oooo1P00000S0?ooo`<000002@3oool7000002X0oooo100000090?ooo`8000002@3oool4
00000640oooo100000090?ooo`8000002@3oool4000000H0oooo000f0?ooo`030000003oool0oooo
06`0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0`0000060?ooo`8000001@3oool00`00
0000oooo0?ooo`1L0?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`<000002`3oool00`00
0000oooo0?ooo`0R0?ooo`@000000P3oool700000300oooo0P0000030?ooo`030000003oool0oooo
00H0oooo0`0000070?ooo`8000000`3oool00`000000oooo0?ooo`1M0?ooo`8000000`3oool00`00
0000oooo0?ooo`060?ooo`<000001`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00<0oooo000f0?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool00P3oool2000000L0
oooo0`0000080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0G03oool00`000000
oooo0?ooo`020?ooo`8000001`3oool3000000/0oooo00<000000?ooo`3oool08P3oool7000003H0
oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`0000070?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`020?ooo`8000001`3oool3000000H0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`00=P3oool00`000000oooo
0?ooo`1[0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`1O0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool8000001`0oooo
1P0000020?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0F`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`020?ooo`00=P3oool00`000000oooo0?ooo`1[0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01L0oooo1@00
00120?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool003H0oooo00<00000
0?ooo`3oool0J`3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool0G@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00l0oooo00D00000
0?ooo`3oool0oooo0000000C0?ooo`H00000A`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00h0oooo0P0000040?ooo`800000G@3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00l0oooo0P0000030?ooo`030000003oool0oooo00<0oooo000f0?ooo`030000003oool0
oooo06/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0D0?ooo`030000003oool0
oooo05`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`050000003oool0
oooo0?ooo`0000003P3oool5000004d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0000000<0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0100oooo100000060?ooo`00=P3oool00`000000oooo0?ooo`1[
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool2000005d0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000000T0oooo1@00
001B0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`03000005l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`8000000`3o
ool00`000000oooo0?ooo`030?ooo`00=P3oool3000006/0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0E0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000000D0oooo1000001G0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1R0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003o
ool0oooo0080oooo000f0?ooo`030000003oool0oooo06/0oooo0P0000050?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool0F`3oool2000000D0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo00000005000005/0oooo0P0000050?ooo`030000003oool0oooo00l0oooo00<00000
0?ooo`3oool0H@3oool2000000D0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo000f0?ooo`030000003oool0oooo06`0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0G03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0140oooo1@00001P0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0403oool2000006<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool00P3oool003H0oooo00<000000?ooo`3oool0K03oool2000000<0oooo00<000000?ooo`3o
ool03`3oool2000000<0oooo0P00001O0?ooo`8000000`3oool00`000000oooo0?ooo`0=0?ooo`@0
00000P3oool200000640oooo0P0000030?ooo`030000003oool0oooo0140oooo0P00001R0?ooo`80
00000`3oool00`000000oooo0?ooo`0A0?ooo`050000003oool0oooo0?ooo`0000001@3oool003H0
oooo00<000000?ooo`3oool0K@3oool4000001@0oooo1000001Q0?ooo`@000002`3oool5000000H0
oooo0P00001R0?ooo`@000005P3oool4000005l0oooo1000000E0?ooo`<000001P3oool003H0oooo
00<000000?ooo`3oool0m`3oool500000?l0oooo303oool003H0oooo00<000000?ooo`3oool0m03o
ool300000080oooo0`00003o0?ooo``0oooo000f0?ooo`030000003oool0oooo0?40oooo0`000006
0?ooo`030000003oool0oooo0?l0oooo2`3oool003H0oooo00<000000?ooo`3oool0kP3oool30000
0?l0oooo5`3oool003H0oooo00<000000?ooo`3oool0j`3oool300000?l0oooo6P3oool003H0oooo
00<000000?ooo`3oool0j03oool300000?l0oooo7@3oool003H0oooo00<000000?ooo`3oool0i@3o
ool300000?l0oooo803oool003H0oooo00<000000?ooo`3oool0i03oool200000?l0oooo8P3oool0
03H0oooo00<000000?ooo`3oool0g`3oool500000?l0oooo903oool003H0oooo00<000000?ooo`3o
ool0gP3oool400000?l0oooo9P3oool002X0ooooo`00003o0000018000001P3oool003H0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<0
00000?ooo`3oool07@3oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool05@3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool0703o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool07@3o
ool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool04@3o
ool003H0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool07@3oool4000001/0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool07@3oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0140oooo000f0?ooo`030000003oool0oooo
07/0oooo00<000000?ooo`3oool0FP3oool3000001d0oooo00<000000?ooo`3oool0N`3oool00`00
0000oooo0?ooo`1j0?ooo`030000003oool0oooo0140oooo000f0?ooo`030000003oool0oooo0=H0
oooo0P00003o0?oooc00oooo000f0?ooo`030000003oool0oooo0=@0oooo0P00003o0?oooc80oooo
000f0?ooo`030000003oool0oooo0=40oooo0`00003o0?oooc@0oooo000f0?ooo`030000003oool0
oooo0=00oooo0P00003o0?ooocH0oooo000f0?ooo`030000003oool0oooo0<h0oooo0P00003o0?oo
ocP0oooo000f0?ooo`030000003oool0oooo0<d0oooo0P00003o0?ooocT0oooo000f0?ooo`030000
003oool0oooo0</0oooo0P00003o0?oooc/0oooo000f0?ooo`030000003oool0oooo0<X0oooo0P00
003o0?oooc`0oooo000f0?ooo`030000003oool0oooo0<T0oooo00<000000?ooo`3oool0o`3oooll
0?ooo`00=P3oool00`000000oooo0?ooo`370?ooo`800000o`3ooolo0?ooo`00=P3oool300000<H0
oooo0P00003o0?oood00oooo000f0?ooo`030000003oool0oooo0<80oooo1000003o0?oood80oooo
000f0?ooo`030000003oool0oooo0<40oooo1000003o0?oood<0oooo000f0?ooo`030000003oool0
oooo0<80oooo0`00003o0?oood<0oooo000f0?ooo`030000003oool0oooo0<00oooo0P0000000`3o
ool000000?ooo`3o0?oood<0oooo000f0?ooo`030000003oool0oooo0;l0oooo0P00003o0?ooodL0
oooo000f0?ooo`030000003oool0oooo0;d0oooo0P00003o0?ooodT0oooo000f0?ooo`030000003o
ool0oooo0;`0oooo0P00003o0?ooodX0oooo000f0?ooo`030000003oool0oooo0;/0oooo00<00000
0?ooo`3oool0o`3ooom:0?ooo`00=P3oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo
0?l0ooooB`3oool003H0oooo00<000000?ooo`3oool0^@3oool00`000000oooo0?ooo`3o0?oood`0
oooo000f0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0o`3ooom=0?ooo`00=P3o
ool00`000000oooo0?ooo`2g0?ooo`030000003oool0oooo0?l0ooooCP3oool003H0oooo00<00000
0?ooo`3oool0]P3oool00`000000oooo0?ooo`3o0?ooodl0oooo000f0?ooo`030000003oool0oooo
0;80oooo1000003o0?oooe80oooo000f0?ooo`<00000/@3oool400000?l0ooooD`3oool003H0oooo
00<000000?ooo`3oool0/P3oool300000?l0ooooD`3oool003H0oooo00<000000?ooo`3oool0/@3o
ool300000?l0ooooE03oool003H0oooo00<000000?ooo`3oool0/03oool200000?l0ooooEP3oool0
03H0oooo00<000000?ooo`3oool0[`3oool200000?l0ooooE`3oool003H0oooo00<000000?ooo`3o
ool0[P3oool200000?l0ooooF03oool003H0oooo00<000000?ooo`3oool0[@3oool200000?l0oooo
F@3oool003H0oooo00<000000?ooo`3oool0[03oool200000?l0ooooFP3oool003H0oooo00<00000
0?ooo`3oool0Z`3oool00`000000oooo0?ooo`3o0?oooeX0oooo000f0?ooo`030000003oool0oooo
0:X0oooo00<000000?ooo`3oool0o`3ooomK0?ooo`00=P3oool00`000000oooo0?ooo`2Y0?ooo`03
0000003oool0oooo0?l0ooooG03oool003H0oooo00<000000?ooo`3oool0Z03oool00`000000oooo
0?ooo`3o0?oooed0oooo000f0?ooo`030000003oool0oooo0:L0oooo00<000000?ooo`3oool0o`3o
oomN0?ooo`00=P3oool00`000000oooo0?ooo`2V0?ooo`030000003oool0oooo0?l0ooooG`3oool0
03H0oooo0`00002U0?ooo`030000003oool0oooo0?l0ooooH03oool003H0oooo00<000000?ooo`3o
ool0Y03oool200000?l0ooooHP3oool003H0oooo00<000000?ooo`3oool0Y03oool00`000000oooo
0?ooo`3o0?ooof40oooo000f0?ooo`030000003oool0oooo0:<0oooo00<000000?ooo`3oool0o`3o
oomR0?ooo`00=P3oool00`000000oooo0?ooo`2R0?ooo`030000003oool0oooo0?l0ooooH`3oool0
03H0oooo00<000000?ooo`3oool0W`3oool400000?l0ooooI@3oool003H0oooo00<000000?ooo`3o
ool0WP3oool400000?l0ooooIP3oool003H0oooo00<000000?ooo`3oool0W`3oool300000?l0oooo
IP3oool003H0oooo00<000000?ooo`3oool0W`3oool200000?l0ooooI`3oool003H0oooo00<00000
0?ooo`3oool0WP3oool00`000000oooo0?ooo`3o0?ooofL0oooo000@0?ooo`@000002@3oool20000
00L0oooo200000080?ooo`030000003oool0oooo09d0oooo0P00003o0?ooofT0oooo000?0?ooo`80
00000`3oool00`000000oooo0?ooo`060?ooo`<000001P3oool2000000D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`2M0?ooo`030000003oool0oooo0?l0ooooJ03oool000l0oooo
00<000000?ooo`3oool00P3oool2000000L0oooo0`0000080?ooo`030000003oool0oooo0080oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2L0?ooo`030000003oool0oooo0?l0oooo
J@3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0V`3oool00`000000oooo0?ooo`3o0?ooofX0oooo000>
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo09X0oooo0P00003o0?ooof`0oooo000>0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`080?ooo`@00000V@3oool00`00
0000oooo0?ooo`3o0?ooof/0oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo09T0oooo00<000000?ooo`3o
ool0o`3ooom/0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo
0P0000080?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0o`3ooom]0?ooo`003P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`2G0?ooo`800000o`3ooom_0?ooo`003P3oool2000000D0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo09H0oooo0P00003o
0?ooog00oooo000?0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0U03oool30000
0?l0ooooL@3oool000l0oooo0P0000030?ooo`030000003oool0oooo00l0oooo0P0000030?ooo`80
00002@3oool00`000000oooo0?ooo`2C0?ooo`@00000o`3oooma0?ooo`00403oool4000001@0oooo
1000000:0?ooo`030000003oool0oooo09@0oooo0`00003o0?ooog40oooo000f0?ooo`030000003o
ool0oooo09<0oooo0`00003o0?ooog80oooo000f0?ooo`030000003oool0oooo09<0oooo00<00000
0?ooo`3oool0o`3ooomb0?ooo`00=P3oool00`000000oooo0?ooo`2B0?ooo`030000003oool0oooo
0?l0ooooL`3oool003H0oooo00<000000?ooo`3oool0T@3oool00`000000oooo0?ooo`3o0?ooog@0
oooo000f0?ooo`030000003oool0oooo0940oooo00<000000?ooo`3oool0o`3ooomd0?ooo`00=P3o
ool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo0?l0ooooM@3oool003H0oooo0`00002?
0?ooo`800000o`3ooomg0?ooo`00=P3oool00`000000oooo0?ooo`2?0?ooo`030000003oool0oooo
0?l0ooooMP3oool003H0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`3o0?ooogL0
oooo000f0?ooo`030000003oool0oooo08d0oooo0P00003o0?ooogT0oooo000f0?ooo`030000003o
ool0oooo08d0oooo00<000000?ooo`3oool0o`3ooomh0?ooo`00=P3oool00`000000oooo0?ooo`2<
0?ooo`030000003oool0oooo0?l0ooooN@3oool003H0oooo00<000000?ooo`3oool0S03oool00`00
0000oooo0?ooo`3o0?ooogT0oooo000f0?ooo`030000003oool0oooo08/0oooo00<000000?ooo`3o
ool0o`3ooomj0?ooo`00=P3oool00`000000oooo0?ooo`2:0?ooo`030000003oool0oooo0?l0oooo
N`3oool003H0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`3o0?ooog/0oooo000f
0?ooo`030000003oool0oooo08T0oooo00<000000?ooo`3oool0o`3oooml0?ooo`00=P3oool00`00
0000oooo0?ooo`280?ooo`800000o`3ooomn0?ooo`00=P3oool00`000000oooo0?ooo`260?ooo`<0
0000o`3ooomo0?ooo`00=P3oool00`000000oooo0?ooo`250?ooo`@00000o`3ooomo0?ooo`00=P3o
ool3000008H0oooo0`00003o0?ooogl0oooo000f0?ooo`030000003oool0oooo08H0oooo0P00003o
0?oooh00oooo000f0?ooo`030000003oool0oooo08D0oooo00<000000?ooo`3oool0o`3ooon00?oo
o`00=P3oool00`000000oooo0?ooo`250?ooo`030000003oool0oooo0?l0ooooP03oool003H0oooo
00<000000?ooo`3oool0Q03oool00`000000oooo0?ooo`3o0?oooh40oooo000f0?ooo`030000003o
ool0oooo08<0oooo00<000000?ooo`3oool0o`3ooon20?ooo`00=P3oool00`000000oooo0?ooo`23
0?ooo`030000003oool0oooo0?l0ooooPP3oool003H0oooo00<000000?ooo`3oool0PP3oool00`00
0000oooo0?ooo`3o0?oooh<0oooo000f0?ooo`030000003oool0oooo0840oooo0P00003o0?ooohD0
oooo000f0?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool0o`3ooon40?ooo`00=P3o
ool00`000000oooo0?ooo`200?ooo`030000003oool0oooo0?l0ooooQ@3oool003H0oooo00<00000
0?ooo`3oool0OP3oool300000?l0ooooQ`3oool003H0oooo00<000000?ooo`3oool0O@3oool40000
0?l0ooooQ`3oool003H0oooo00<000000?ooo`3oool0OP3oool300000?l0ooooQ`3oool003H0oooo
00<000000?ooo`3oool0OP3oool200000?l0ooooR03oool003H0oooo0`00001m0?ooo`030000003o
ool0oooo0?l0ooooR03oool003H0oooo00<000000?ooo`3oool0O03oool200000?l0ooooRP3oool0
03H0oooo00<000000?ooo`3oool0O03oool00`000000oooo0?ooo`3o0?ooohT0oooo000f0?ooo`03
0000003oool0oooo07/0oooo00<000000?ooo`3oool0o`3ooon:0?ooo`00=P3oool00`000000oooo
0?ooo`1k0?ooo`030000003oool0oooo0?l0ooooRP3oool003H0oooo00<000000?ooo`3oool0NP3o
ool00`000000oooo0?ooo`3o0?oooh/0oooo000f0?ooo`030000003oool0oooo07X0oooo00<00000
0?ooo`3oool0o`3ooon;0?ooo`00=P3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo
0?l0ooooS03oool003H0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`3o0?oooh`0
oooo000f0?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool0o`3ooon=0?ooo`00403o
ool4000000T0oooo0P0000090?ooo`H00000203oool00`000000oooo0?ooo`1g0?ooo`800000o`3o
oon?0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000/0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo0?l0ooooSP3oool0
00l0oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`00000;0?ooo`030000003oool0oooo
00P0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`3o0?ooohl0oooo000>0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool8000000P0oooo00<000000?ooo`3o
ool0MP3oool00`000000oooo0?ooo`3o0?ooohl0oooo000>0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00P0oooo
1000001c0?ooo`<00000o`3ooonA0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00l0oooo00D000000?ooo`3oool0oooo0000000:0?ooo`030000003oool0oooo07<0oooo
1000003o0?oooi40oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool03`3o
ool01@000000oooo0?ooo`3oool0000000X0oooo00<000000?ooo`3oool0M03oool300000?l0oooo
T@3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`040000003o
ool0oooo000000X0oooo00<000000?ooo`3oool0M03oool200000?l0ooooTP3oool000h0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`3o0?oooi80oooo000>0?ooo`8000001@3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0000000X0oooo00<000000?ooo`3oool0LP3o
ool200000?l0ooooU03oool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0B
0?ooo`030000003oool0000000X0oooo00<000000?ooo`3oool0LP3oool00`000000oooo0?ooo`3o
0?oooi<0oooo000?0?ooo`8000000`3oool00`000000oooo0?ooo`0C0?ooo`8000002P3oool00`00
0000oooo0?ooo`1a0?ooo`800000o`3ooonE0?ooo`00403oool4000001H0oooo0P00000:0?ooo`03
0000003oool0oooo0740oooo00<000000?ooo`3oool0o`3ooonD0?ooo`00=P3oool00`000000oooo
0?ooo`1`0?ooo`030000003oool0oooo0?l0ooooU@3oool003H0oooo00<000000?ooo`3oool0KP3o
ool300000?l0ooooU`3oool003H0oooo00<000000?ooo`3oool0K@3oool400000?l0ooooU`3oool0
03H0oooo00<000000?ooo`3oool0KP3oool300000?l0ooooU`3oool003H0oooo00<000000?ooo`3o
ool0KP3oool200000?l0ooooV03oool003H0oooo00<000000?ooo`3oool0KP3oool00`000000oooo
0?ooo`3o0?oooiL0oooo000f0?ooo`<00000K@3oool00`000000oooo0?ooo`3o0?oooiP0oooo000f
0?ooo`030000003oool0oooo06`0oooo0P00003o0?oooiX0oooo000f0?ooo`030000003oool0oooo
06`0oooo00<000000?ooo`3oool0o`3ooonI0?ooo`00=P3oool00`000000oooo0?ooo`1[0?ooo`03
0000003oool0oooo0?l0ooooVP3oool003H0oooo00<000000?ooo`3oool0J`3oool00`000000oooo
0?ooo`3o0?oooiX0oooo000f0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0o`3o
oonK0?ooo`00=P3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo0?l0ooooV`3oool0
03H0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`3o0?oooi`0oooo000f0?ooo`03
0000003oool0oooo06T0oooo00<000000?ooo`3oool0o`3ooonL0?ooo`00=P3oool00`000000oooo
0?ooo`1X0?ooo`030000003oool0oooo0?l0ooooW@3oool003H0oooo00<000000?ooo`3oool0I`3o
ool200000?l0ooooW`3oool003H0oooo00<000000?ooo`3oool0I`3oool00`000000oooo0?ooo`3o
0?oooih0oooo000f0?ooo`030000003oool0oooo06H0oooo0P00003o0?oooj00oooo000f0?ooo`03
0000003oool0oooo06H0oooo00<000000?ooo`3oool0o`3ooonO0?ooo`00=P3oool3000006D0oooo
0P00003o0?oooj40oooo000f0?ooo`030000003oool0oooo06D0oooo00<000000?ooo`3oool0o`3o
oonP0?ooo`00=P3oool00`000000oooo0?ooo`1T0?ooo`800000o`3ooonR0?ooo`00=P3oool00`00
0000oooo0?ooo`1T0?ooo`030000003oool0oooo0?l0ooooX@3oool003H0oooo00<000000?ooo`3o
ool0H`3oool200000?l0ooooX`3oool003H0oooo00<000000?ooo`3oool0H`3oool00`000000oooo
0?ooo`3o0?oooj80oooo000f0?ooo`030000003oool0oooo0680oooo0P00003o0?oooj@0oooo000f
0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0o`3ooonS0?ooo`00=P3oool00`00
0000oooo0?ooo`1Q0?ooo`800000o`3ooonU0?ooo`00=P3oool00`000000oooo0?ooo`1Q0?ooo`03
0000003oool0oooo0?l0ooooY03oool003H0oooo00<000000?ooo`3oool0H03oool200000?l0oooo
YP3oool003H0oooo00<000000?ooo`3oool0H03oool00`000000oooo0?ooo`3o0?ooojD0oooo000f
0?ooo`030000003oool0oooo05l0oooo0P00003o0?ooojL0oooo000f0?ooo`030000003oool0oooo
05l0oooo00<000000?ooo`3oool0o`3ooonV0?ooo`00=P3oool00`000000oooo0?ooo`1N0?ooo`80
0000o`3ooonX0?ooo`00=P3oool3000005h0oooo00<000000?ooo`3oool0o`3ooonW0?ooo`00=P3o
ool00`000000oooo0?ooo`1M0?ooo`800000o`3ooonY0?ooo`00=P3oool00`000000oooo0?ooo`1M
0?ooo`030000003oool0oooo0?l0ooooZ03oool003H0oooo00<000000?ooo`3oool0G03oool20000
0?l0ooooZP3oool003H0oooo00<000000?ooo`3oool0G03oool00`000000oooo0?ooo`3o0?ooojT0
oooo000f0?ooo`030000003oool0oooo05/0oooo0P00003o0?oooj/0oooo000f0?ooo`030000003o
ool0oooo05/0oooo00<000000?ooo`3oool0o`3ooonZ0?ooo`00=P3oool00`000000oooo0?ooo`1J
0?ooo`800000o`3ooon/0?ooo`00=P3oool00`000000oooo0?ooo`1J0?ooo`030000003oool0oooo
0?l0ooooZ`3oool003H0oooo00<000000?ooo`3oool0F@3oool200000?l0oooo[@3oool00100oooo
100000090?ooo`8000002@3oool4000000X0oooo00<000000?ooo`3oool0F@3oool00`000000oooo
0?ooo`3o0?oooj`0oooo000?0?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`<000001`3o
ool2000000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1H0?ooo`800000o`3o
oon^0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`8000001`3oool3000000L0oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo05P0oooo00<00000
0?ooo`3oool0o`3ooon]0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo
05L0oooo0P00003o0?ooojl0oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00H0oooo1000001F0?ooo`03
0000003oool0oooo0?l0oooo[P3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0>0?ooo`800000103oool2000000P0oooo00<000000?ooo`3oool0EP3oool200000?l0oooo
/03oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0000000<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1F0?ooo`030000003o
ool0oooo0?l0oooo[`3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00<000002P3oool00`000000oooo0?ooo`1E0?ooo`800000o`3ooona
0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo0?l0oooo/03oool000h0
oooo0P0000050?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`1D0?ooo`800000o`3ooonb0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo0100oooo0P00000=0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool0
o`3ooona0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool04@3oool2000000`0oooo00<0
00000?ooo`3oool0D`3oool200000?l0oooo/`3oool00100oooo1000000F0?ooo`@00000203oool0
0`000000oooo0?ooo`1C0?ooo`030000003oool0oooo0?l0oooo/P3oool003H0oooo00<000000?oo
o`3oool0DP3oool00`000000oooo0?ooo`3o0?oook<0oooo000f0?ooo`030000003oool0oooo0580
oooo00<000000?ooo`3oool0o`3ooonc0?ooo`00=P3oool00`000000oooo0?ooo`1A0?ooo`030000
003oool0oooo0?l0oooo]03oool003H0oooo00<000000?ooo`3oool0D03oool200000?l0oooo]P3o
ool003H0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`3o0?oookD0oooo000f0?oo
o`030000003oool0oooo04l0oooo00<000000?ooo`3oool0o`3ooonf0?ooo`00=P3oool3000004l0
oooo00<000000?ooo`3oool0o`3ooonf0?ooo`00=P3oool00`000000oooo0?ooo`1>0?ooo`030000
003oool0oooo0?l0oooo]`3oool003H0oooo00<000000?ooo`3oool0C@3oool00`000000oooo0?oo
o`3o0?oookP0oooo000f0?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0o`3ooonh
0?ooo`00=P3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo0?l0oooo^@3oool003H0
oooo00<000000?ooo`3oool0B`3oool200000?l0oooo^`3oool003H0oooo00<000000?ooo`3oool0
B`3oool00`000000oooo0?ooo`3o0?oookX0oooo000f0?ooo`030000003oool0oooo04X0oooo00<0
00000?ooo`3oool0o`3ooonk0?ooo`00=P3oool00`000000oooo0?ooo`190?ooo`800000o`3ooonm
0?ooo`00=P3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo0?l0oooo_03oool003H0
oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`3o0?oookd0oooo000f0?ooo`030000
003oool0oooo04P0oooo00<000000?ooo`3oool0o`3ooonm0?ooo`00=P3oool00`000000oooo0?oo
o`170?ooo`030000003oool0oooo0?l0oooo_P3oool003H0oooo00<000000?ooo`3oool0AP3oool2
00000?l0oooo`03oool003H0oooo0`0000160?ooo`030000003oool0oooo0?l0oooo_`3oool003H0
oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`3o0?oool00oooo000f0?ooo`030000
003oool0oooo04@0oooo0P00003o0?oool80oooo000f0?ooo`030000003oool0oooo04@0oooo00<0
00000?ooo`3oool0o`3oooo10?ooo`00=P3oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo0?l0oooo`P3oool003H0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`3o0?oo
ol80oooo000f0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0o`3oooo30?ooo`00
=P3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo0?l0ooooa03oool003H0oooo00<0
00000?ooo`3oool0@@3oool00`000000oooo0?ooo`3o0?oool@0oooo000f0?ooo`030000003oool0
oooo0400oooo00<000000?ooo`3oool0o`3oooo50?ooo`00=P3oool00`000000oooo0?ooo`0o0?oo
o`800000o`3oooo70?ooo`00=P3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0?l0
ooooaP3oool003H0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`3o0?ooolL0oooo
000f0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0o`3oooo70?ooo`00=P3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo0?l0oooob03oool003H0oooo0`00000l0?oo
o`030000003oool0oooo0?l0oooob@3oool003H0oooo00<000000?ooo`3oool0>`3oool200000?l0
oooob`3oool003H0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`3o0?ooolX0oooo
000f0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool0o`3oooo;0?ooo`00=P3oool0
0`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0?l0ooooc03oool003H0oooo00<000000?oo
o`3oool0>03oool00`000000oooo0?ooo`3o0?ooold0oooo000f0?ooo`030000003oool0oooo03L0
oooo0P00003o0?oooll0oooo000f0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0
o`3oooo>0?ooo`00=P3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo0?l0ooooc`3o
ool003H0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3o0?ooom00oooo000@0?oo
o`@000002@3oool2000000T0oooo1000000:0?ooo`030000003oool0oooo03@0oooo00<000000?oo
o`3oool0o`3ooooA0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000L0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo03<0
oooo0P00003o0?ooom<0oooo000?0?ooo`030000003oool0oooo0080oooo0P0000070?ooo`<00000
1P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`3o0?ooom80oooo000>0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0<P3oool00`000000oooo0?ooo`3o0?ooom<0oooo000>0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00H0oooo1000000`0?ooo`030000003oool0oooo0?l0ooooe03oool000h0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo0300oooo00<000000?ooo`3oool0o`3ooooE0?ooo`003P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0100oooo1000000:0?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0o`3ooooF0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo00l0oooo0P0000030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0?ooo`3o0?ooomL0oooo000>0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0;@3oool00`000000oooo0?ooo`3o0?ooomP0oooo000>0?ooo`8000001@3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`0[0?ooo`800000o`3ooooK0?ooo`003`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo02T0oooo0P00003o0?ooomd0oooo000?0?ooo`8000000`3oool00`000000
oooo0?ooo`0A0?ooo`050000003oool0oooo0?ooo`0000002@3oool00`000000oooo0?ooo`0W0?oo
o`800000o`3ooooO0?ooo`00403oool4000001D0oooo0`00000:0?ooo`030000003oool0oooo02D0
oooo0P00003o0?ooon40oooo000f0?ooo`030000003oool0oooo02<0oooo0P00003o0?ooon<0oooo
000f0?ooo`030000003oool0oooo0240oooo0P00003o0?ooonD0oooo000f0?ooo`030000003oool0
oooo01`0oooo1@00003o0?ooonL0oooo000f0?oooal00000o`3oooo/0?ooo`00=P3oool00`000000
oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?ooo`3oool0o`3ooooo0?ooo`T0oooo
000f0?ooo`<00000o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool9
0?ooo`00=P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool003H0oooo00<000000?ooo`3o
ool0o`3ooooo0?ooo`T0oooo000f0?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00=P3o
ool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oo
od80oooo003o0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o
0?ooool0oooo@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo
@P3oool00?l0ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo003o0?ooool0oooo@P3oool00?l0
ooooo`3ooom20?ooo`00o`3ooooo0?oood80oooo0000\
\>"],
  ImageRangeCache->{{{0, 575}, {354.875, 0}} -> {-0.0869363, -0.307462, \
0.00318469, 0.00690827}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit2 = Fit[data2, {Log[x], 1, 1/Sqrt[x], 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(-1.2199778011099196`\) - 0.8981228634192538`\/x + 
      1.875064391854904`\/\@x + 0.29873200353538665`\ Log[x]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[data2, fit2, x]\)], "Input"],

Cell[BoxData[
    \(0.00009016790164207473`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fit2a = Fit[data2, {Log[x], 1, 1/x}, x]\)], "Input"],

Cell[BoxData[
    \(\(-0.4838136520344635`\) + 0.6438992890927365`\/x + 
      0.16876744794790582`\ Log[x]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[data2, fit2a, x]\)], "Input"],

Cell[BoxData[
    \(0.004038341221948636`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fit2a] /. x \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({4.212174603192416`, 0.05651356651797549`, 
      1.1160119911152666`, \(-0.05528532770782675`\), 
      0.8398191854294754`, \(-5.554913435245905`\), \
\(-0.28515082459006147`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fit2 /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      Plot[fit2a /. x \[Rule] Exp[1/y], {y, 0, 0.8}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[Map[{1/Log[#[\([1]\)]], #[\([2]\)]} &, data2], 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction, 
      PlotRange \[Rule] {\(-1\)/4, 1/4}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 1.19048 0.309017 1.23607 [
[.2619 .29652 -9 -9 ]
[.2619 .29652 9 0 ]
[.5 .29652 -9 -9 ]
[.5 .29652 9 0 ]
[.7381 .29652 -9 -9 ]
[.7381 .29652 9 0 ]
[.97619 .29652 -9 -9 ]
[.97619 .29652 9 0 ]
[.01131 .0618 -24 -4.5 ]
[.01131 .0618 0 4.5 ]
[.01131 .18541 -24 -4.5 ]
[.01131 .18541 0 4.5 ]
[.01131 .43262 -18 -4.5 ]
[.01131 .43262 0 4.5 ]
[.01131 .55623 -18 -4.5 ]
[.01131 .55623 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .30902 m
.2619 .31527 L
s
[(0.2)] .2619 .29652 0 1 Mshowa
.5 .30902 m
.5 .31527 L
s
[(0.4)] .5 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(0.6)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(0.8)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.08333 .30902 m
.08333 .31277 L
s
.14286 .30902 m
.14286 .31277 L
s
.20238 .30902 m
.20238 .31277 L
s
.32143 .30902 m
.32143 .31277 L
s
.38095 .30902 m
.38095 .31277 L
s
.44048 .30902 m
.44048 .31277 L
s
.55952 .30902 m
.55952 .31277 L
s
.61905 .30902 m
.61905 .31277 L
s
.67857 .30902 m
.67857 .31277 L
s
.79762 .30902 m
.79762 .31277 L
s
.85714 .30902 m
.85714 .31277 L
s
.91667 .30902 m
.91667 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .0618 m
.03006 .0618 L
s
[(-0.2)] .01131 .0618 1 0 Mshowa
.02381 .18541 m
.03006 .18541 L
s
[(-0.1)] .01131 .18541 1 0 Mshowa
.02381 .43262 m
.03006 .43262 L
s
[(0.1)] .01131 .43262 1 0 Mshowa
.02381 .55623 m
.03006 .55623 L
s
[(0.2)] .01131 .55623 1 0 Mshowa
.125 Mabswid
.02381 .08652 m
.02756 .08652 L
s
.02381 .11125 m
.02756 .11125 L
s
.02381 .13597 m
.02756 .13597 L
s
.02381 .16069 m
.02756 .16069 L
s
.02381 .21013 m
.02756 .21013 L
s
.02381 .23485 m
.02756 .23485 L
s
.02381 .25957 m
.02756 .25957 L
s
.02381 .2843 m
.02756 .2843 L
s
.02381 .33374 m
.02756 .33374 L
s
.02381 .35846 m
.02756 .35846 L
s
.02381 .38318 m
.02756 .38318 L
s
.02381 .4079 m
.02756 .4079 L
s
.02381 .45735 m
.02756 .45735 L
s
.02381 .48207 m
.02756 .48207 L
s
.02381 .50679 m
.02756 .50679 L
s
.02381 .53151 m
.02756 .53151 L
s
.02381 .03708 m
.02756 .03708 L
s
.02381 .01236 m
.02756 .01236 L
s
.02381 .58095 m
.02756 .58095 L
s
.02381 .60567 m
.02756 .60567 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.30796 .61803 m
.34508 .50544 L
.36469 .46112 L
.38564 .42189 L
.42714 .36274 L
.46712 .32217 L
.50559 .29339 L
.54651 .27036 L
.58591 .25338 L
.6238 .24048 L
.66414 .22942 L
.70296 .22071 L
.74424 .21299 L
.784 .20668 L
.82225 .2014 L
.86294 .19644 L
.90212 .19218 L
.94376 .18807 L
.97619 .18513 L
s
.30131 .61803 m
.30359 .60994 L
.34227 .50975 L
.38341 .43064 L
.42304 .3734 L
.46115 .33116 L
.50171 .29657 L
.54075 .27096 L
.58225 .25011 L
.62223 .23485 L
.6607 .22369 L
.70162 .21481 L
.74102 .2086 L
.77891 .20438 L
.79835 .20276 L
.81925 .20139 L
.84042 .20034 L
.85016 .19997 L
.86052 .19964 L
.87094 .19938 L
.88037 .19919 L
.88493 .19912 L
.88985 .19906 L
.89416 .19901 L
.89877 .19898 L
.90135 .19896 L
.9037 .19895 L
.90606 .19894 L
.90713 .19894 L
.9083 .19893 L
.90956 .19893 L
.91025 .19893 L
.9109 .19893 L
.91206 .19893 L
.91331 .19893 L
.91397 .19893 L
.91469 .19893 L
.91544 .19893 L
.91615 .19893 L
.9174 .19893 L
.91875 .19894 L
.92148 .19895 L
.92443 .19896 L
.9298 .19899 L
.93502 .19904 L
.93982 .19909 L
.95715 .19936 L
.96699 .19956 L
.97619 .19978 L
s
.008 w
.88256 .19422 Mdot
.68823 .22387 Mdot
.59631 .24963 Mdot
.50289 .29509 Mdot
.45318 .33472 Mdot
.3984 .4014 Mdot
.36731 .45603 Mdot
.33133 .54179 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{397.188, 245.5},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000<J0001jb000`400?l00000o`00003oo`3ooooo
0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?ooool0ooooo`3ooolM0?ooo`00
o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?ooool0ooooo`3ooolM
0?ooo`00o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?ooool0oooo
o`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?oo
ool0ooooo`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool003T0oooo00<000000?ooo`3oool0
o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oo
on00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i
0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ool0ooooh03oool003T0oooo1000003o0?ooool0oooog`3oool003T0oooo00<000000?ooo`3oool0
o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oo
on00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i
0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000
003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03o
ool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo
0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo
00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3o
oooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo1000003o0?oo
ool0oooog`3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000
003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03o
ool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo
0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo
00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3o
oooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?oo
o`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3o
oooo0?ooon00oooo000@0?ooo`@000002@3oool2000000L0oooo2000000;0?ooo`030000003oool0
oooo0?l0ooooo`3ooooP0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool01P3oool30000
00H0oooo0P0000050?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3ooooo0?oo
on00oooo000?0?ooo`030000003oool0oooo0080oooo0P0000070?ooo`<00000203oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00
oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`003P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0303oool600000?l0ooooo`3o
oooM0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01<0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool000@0oooo100000060?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01@0oooo0P00000;0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00
3P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
2@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool000h0oooo0P0000050?ooo`030000003o
ool0oooo01D0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03o
ool000l0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool0
00l0oooo0P0000030?ooo`030000003oool0oooo00l0oooo0P0000030?ooo`800000303oool00`00
0000oooo0?ooo`3o0?ooool0ooooh03oool00100oooo1000000D0?ooo`@000003@3oool00`000000
oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo
000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o
0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`03
0000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0oooo
h03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0
oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0
oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`@00000o`3ooooo0?oooml0oooo
000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o
0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`03
0000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0oooo
h03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0
oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0
oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0oooo
o`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<00000
0?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?oo
o`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0
o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo1000003o0?ooool0oooog`3oool003T0
oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0oooo
o`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<00000
0?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?oo
o`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0
o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oo
on00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i
0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000
003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03o
ool003T0oooo1000003o0?ooool0oooog`3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oo
on00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i
0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000
003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03o
ool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo
0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo
00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3o
oooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?oo
o`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo1000003o0?ooool0oooog`3o
ool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo
0?l0ooooo`3ooooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo
00<000000?ooo`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3o
oooP0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?oo
o`3oool0o`3ooooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3o
oooo0?ooon00oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?ooo`00>@3oool00`00
0000oooo0?ooo`3o0?ooool0ooooh03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooon00
oooo000@0?ooo`@000002@3oool2000000L0oooo2000000;0?ooo`030000003oool0oooo0?l0oooo
o`3ooooP0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000X0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`3o0?ooool0ooooh03oool000l0oooo00<00000
0?ooo`3oool00P3oool2000000L0oooo0`00000:0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0o`3ooooo0?ooon00oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0ooooo`3ooooP0?oo
o`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0303oool600000?l0ooooo`3oooo40?ooo`L000004P3oool000h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
o`3ooooo0?oook/0oooo3000000I0?ooo`00103oool4000000H0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3o
oooo0?ooojl0oooo3000000U0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3o0?ooool0ooooY@3oool:
00000340oooo000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`00
0000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0ooooo`3ooon50?ooo`D000004@3oool:0000
03/0oooo000>0?ooo`8000001@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0o`3ooooo0?ooohD0oooo1@0000080?ooo`T00000A@3oool000l0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo0?ooo`00
00003P3oool00`000000oooo0?ooo`3o0?ooool0ooooQ03oool6000000030?ooo`000000000000D0
0000CP3oool000l0oooo0P0000030?ooo`030000003oool0oooo0100oooo1000000>0?ooo`030000
003oool0oooo0?l0ooooo`3ooon30?ooo`P00000E@3oool00100oooo1000000E0?ooo`8000003P3o
ool00`000000oooo0?ooo`3o0?ooool0ooooO03oool700000080oooo1@00001F0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooool0ooooL`3oool9000000/0oooo00<000000?ooo`3oool0EP3oool0
03T0oooo00<000000?ooo`3oool0o`3ooooo0?ooofT0oooo2P00001]0?ooo`00>@3oool00`000000
oooo0?ooo`3o0?ooool0ooooG`3oool:00000080ooooH`00000B0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?ooool0ooooC`3ooolL000007D0oooo000i0?ooo`030000003oool0oooo0?l0ooooo`3o
oom40?oooa800000RP3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oooc`0oooo20000003
0?ooo`P00000T@3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oooc@0oooo200000040?oo
o`L00000V@3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooobX0oooo2P0000050?ooo`L0
0000X03oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?ooob40oooo2P0000080?ooo`L00000
YP3oool003T0oooo00<000000?ooo`3oool0o`3ooooo0?oooaT0oooo2000000<0?ooo`L00000[03o
ool003T0oooo1000003o0?ooool0oooo403oool8000000h0oooo1`00002b0?ooo`00>@3oool00`00
0000oooo0?ooo`3o0?ooool0oooo2P3oool700000100oooo1`00002h0?ooo`00>@3oool00`000000
oooo0?ooo`3o0?ooool0oooo0P3oool800000180oooo1P00002n0?ooo`00>@3oool00`000000oooo
0?ooo`3o0?oooo/0oooo1P00000E0?ooo`D00000a03oool003T0oooo00<000000?ooo`3oool0o`3o
ooog0?ooo`@000005P3oool500000<T0oooo000i0?ooo`030000003oool0oooo0?l0oooolP3oool5
000001@0oooo1P00003>0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooonh0oooo1000000D0?oo
o`D00000e03oool003T0oooo00<000000?ooo`3oool0o`3ooooZ0?ooo`@000004`3oool500000=T0
oooo000i0?ooo`030000003oool0oooo0?l0ooooi@3oool5000000L0oooo1@0000060?ooo`D00000
gP3oool003T0oooo00<000000?ooo`3oool0o`3ooooQ0?ooo`@00000303oool5000000030?ooo`00
0000000000<00000h`3oool003T0oooo00<000000?ooo`3oool0o`3ooooM0?ooo`@000003`3oool7
00000>P0oooo000i0?ooo`030000003oool0oooo0?l0oooofP3oool300000140oooo2000003Y0?oo
o`00>@3oool00`000000oooo0?ooo`3o0?ooomH0oooo1000000?0?ooo`D000000`3oool500000>T0
oooo000i0?ooo`030000003oool0oooo0?l0ooood`3oool4000000d0oooo1@00000:0?ooo`030000
003oool0oooo0>T0oooo000i0?ooo`030000003oool0oooo0?l0ooood03oool3000000d0oooo1000
003k0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooold0oooo0`00000<0?ooo`@00000o`3oool0
03T0oooo00<000000?ooo`3oool0o`3oooo90?ooo`@00000303oool300000?l0oooo103oool003T0
oooo00<000000?ooo`3oool0o`3oooo60?ooo`@000002`3oool400000?l0oooo1`3oool003T0oooo
00<000000?ooo`3oool0o`3oooo30?ooo`<00000303oool400000?l0oooo2P3oool003T0oooo1000
003o0?oookl0oooo0`00000;0?ooo`@00000o`3oool>0?ooo`00>@3oool00`000000oooo0?ooo`3o
0?oookd0oooo0`00000;0?ooo`@00000o`3ooolA0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ok/0oooo0P00000;0?ooo`<00000o`3ooolE0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oookP0
oooo0`0000090?ooo`@00000o`3ooolH0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oookD0oooo
0`0000090?ooo`<00000o`3ooolL0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oook80oooo0`00
00090?ooo`<00000o`3ooolO0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oook00oooo0`000008
0?ooo`<00000o`3ooolR0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooojd0oooo0`0000090?oo
o`<00000o`3ooolT0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooojX0oooo1`0000050?ooo`<0
0000o`3ooolW0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooojP0oooo0P0000020?ooo`D00000
0P3oool300000?l0oooo:P3oool003T0oooo00<000000?ooo`3oool0o`3ooonU0?ooo`<000000`3o
ool800000?l0oooo;@3oool003T0oooo00<000000?ooo`3oool0o`3ooonR0?ooo`<000001`3oool5
00000?l0oooo;`3oool003T0oooo00<000000?ooo`3oool0o`3ooonP0?ooo`<00000203oool60000
0?l0oooo;`3oool003T0oooo00<000000?ooo`3oool0o`3ooonO0?ooo`8000001`3oool3000000<0
oooo00<000000?ooo`3oool0o`3oool_0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooid0oooo
0P0000060?ooo`<00000o`3ooolh0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooi/0oooo0P00
00060?ooo`800000o`3ooolk0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooiT0oooo0P000005
0?ooo`<00000o`3ooolm0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooiL0oooo0P0000050?oo
o`800000o`3ooom00?ooo`00>@3oool400000?l0ooooU03oool2000000@0oooo0`00003o0?oood80
oooo000i0?ooo`030000003oool0oooo0?l0ooooT`3oool2000000@0oooo0P00003o0?ooodD0oooo
000i0?ooo`030000003oool0oooo0?l0ooooT@3oool2000000<0oooo0`00003o0?ooodL0oooo000i
0?ooo`030000003oool0oooo0?l0ooooS`3oool2000000<0oooo0P00003o0?ooodX0oooo000i0?oo
o`030000003oool0oooo0?l0ooooS@3oool200000080oooo0`00003o0?oood`0oooo000i0?ooo`03
0000003oool0oooo0?l0ooooR`3oool200000080oooo0P00003o0?ooodl0oooo000i0?ooo`030000
003oool0oooo0?l0ooooR@3oool2000000040?ooo`000000000000000?l0ooooD@3oool003T0oooo
00<000000?ooo`3oool0o`3ooon70?ooo`80000000<0oooo000000000000o`3ooomD0?ooo`00>@3o
ool00`000000oooo0?ooo`3o0?ooohD0oooo1@00003o0?oooeH0oooo000i0?ooo`030000003oool0
oooo0?l0ooooQ03oool400000?l0ooooF03oool003T0oooo00<000000?ooo`3oool0o`3ooon20?oo
o`@00000o`3ooomJ0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooh00oooo1000003o0?oooe`0
oooo000i0?ooo`030000003oool0oooo0:80oooo100000090?ooo`8000001`3oool8000009H0oooo
100000090?ooo`8000002@3oool6000000X0oooo100000270?ooo`@000002@3oool2000000T0oooo
1000002H0?ooo`@000002@3oool2000000T0oooo100000050?ooo`00>@3oool00`000000oooo0?oo
o`2Q0?ooo`8000000`3oool00`000000oooo0?ooo`060?ooo`<000001P3oool2000000D0oooo00<0
00000?ooo`3oool0T`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000/0oooo00<0
00000?ooo`3oool0203oool4000008P0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0`00
00070?ooo`8000000`3oool00`000000oooo0?ooo`2D0?ooo`8000000`3oool00`000000oooo0?oo
o`060?ooo`<000001`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo000i
0?ooo`030000003oool0oooo0:40oooo00<000000?ooo`3oool00P3oool2000000L0oooo0`000008
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0T`3oool00`000000oooo0?ooo`02
0?ooo`8000001`3oool3000000/0oooo00<000000?ooo`3oool01`3oool3000008X0oooo00<00000
0?ooo`3oool00P3oool2000000L0oooo0`0000070?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0T`3oool00`000000oooo0?ooo`020?ooo`8000001`3oool3000000H0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`010?ooo`00>@3oool00`000000oooo0?ooo`2P0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`2F0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool8000000D0oooo0`00002;0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0980oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool00@3oool003T0oooo00<000000?ooo`3o
ool0X03oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3o
ool0U@3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`040?ooo`<00000S03oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`2B0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0040oooo000i0?ooo`030000003oool0oooo0:00oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0?0?ooo`050000003oool0oooo0?ooo`000000103oool3000008h0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`800000103oool2000009@0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool00`000000
oooo0?ooo`020?ooo`00>@3oool00`000000oooo0?ooo`2P0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0503oool00`000000oooo0?ooo`2C0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool03`3oool01@000000oooo0?ooo`3oool000000080oooo0`00002@0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo000000030?ooo`03
0000003oool0oooo09<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`@0
00001@3oool003T0oooo1000002O0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
503oool2000009@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0@0?ooo`050000
003oool0oooo0000003oool00`00002A0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`03000009H0oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0?0?ooo`8000000`3oool00`000000oooo0?ooo`020?ooo`00>@3oool00`000000oooo
0?ooo`2P0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`2B0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0403oool00`000000oooo
0?ooo`03000009<0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo09T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool00@3oool003T0oooo00<000000?ooo`3oool0X03o
ool2000000D0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`2B0?ooo`8000001@3o
ool00`000000oooo0?ooo`0A0?ooo`<00000U@3oool2000000D0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`2H0?ooo`8000001@3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool00@3oool003T0oooo00<000000?ooo`3oool0X@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`2C0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04@3oool2
000000030?ooo`000000oooo09D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0@
0?ooo`800000VP3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`010?ooo`00>@3oool00`000000oooo0?ooo`2Q0?oo
o`8000000`3oool00`000000oooo0?ooo`0?0?ooo`8000000`3oool2000009H0oooo0P0000030?oo
o`030000003oool0oooo00l0oooo0`0000000`3oool000000000002F0?ooo`8000000`3oool00`00
0000oooo0?ooo`0A0?ooo`800000V@3oool2000000<0oooo00<000000?ooo`3oool04@3oool01@00
0000oooo0?ooo`3oool0000000@0oooo000i0?ooo`030000003oool0oooo0:80oooo1000000D0?oo
o`@00000V03oool400000140oooo0P0000030?ooo`800000U`3oool4000001H0oooo1000002F0?oo
o`@000005@3oool3000000D0oooo000i0?ooo`030000003oool0oooo0?l0ooooIP3oool600000?l0
ooooM03oool003T0oooo00<000000?ooo`3oool0o`3ooomV0?ooo`D00000o`3ooome0?ooo`00>@3o
ool00`000000oooo0?ooo`3o0?ooofD0oooo1P00003o0?ooogD0oooo000i0?ooo`030000003oool0
oooo0?l0ooooIP3oool500000?l0ooooM@3oool003T0oooo00<000000?ooo`3oool0o`3ooomU0?oo
o`H00000o`3ooome0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooof<0oooo0P0000030?ooo`03
0000003oool0oooo0?l0ooooM@3oool003T0oooo00<000000?ooo`3oool0o`3ooomR0?ooo`800000
o`3oooml0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooof00oooo0`00003o0?ooogd0oooo000i
0?ooo`030000003oool0oooo0?l0ooooG`3oool200000?l0ooooO`3oool003T0oooo00<000000?oo
o`3oool0o`3ooomN0?ooo`800000o`3ooon00?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooe`0
oooo0`00003o0?oooh40oooo000i0?ooo`030000003oool0oooo0?l0ooooF`3oool300000?l0oooo
PP3oool003T0oooo00<000000?ooo`3oool0o`3ooomI0?ooo`@00000o`3ooon30?ooo`009`3ooooo
00000?l00000m@00000003T0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0Z0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0L0?oo
o`040000003oool00000000000X0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo0100oooo000i0?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02T0oooo00<000000?oo
o`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02X0oooo00<000000?oo
o`3oool06P3oool2000000030?ooo`000000oooo00/0oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo0100oooo000i0?ooo`030000003oool0oooo0;00
oooo00<000000?ooo`3oool0X03oool2000000030?ooo`000000oooo00`0oooo00<000000?ooo`3o
ool0/03oool00`000000oooo0?ooo`2a0?ooo`030000003oool0oooo0100oooo000i0?ooo`030000
003oool0oooo0;00oooo00<000000?ooo`3oool0W`3oool010000000oooo0?ooo`00000>0?ooo`03
0000003oool0oooo0;00oooo00<000000?ooo`3oool0/@3oool00`000000oooo0?ooo`0@0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?oooe40oooo0P0000000`3oool000000000003o0?ooohX0oooo
000i0?ooo`030000003oool0oooo0?l0ooooD03oool00`000000oooo0?ooo`0200000?l0ooooR`3o
ool003T0oooo00<000000?ooo`3oool0o`3ooom>0?ooo`8000000P3oool200000?l0ooooS03oool0
03T0oooo00<000000?ooo`3oool0o`3ooom=0?ooo`8000000P3oool00`000000oooo0?ooo`3o0?oo
oh`0oooo000i0?ooo`030000003oool0oooo0?l0ooooC03oool01@000000oooo0?ooo`3oool00000
0?l0ooooS`3oool003T0oooo00<000000?ooo`3oool0o`3ooom;0?ooo`050000003oool0oooo0?oo
o`000000o`3ooon@0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooodX0oooo00D000000?ooo`3o
ool0oooo0000003o0?oooi40oooo000i0?ooo`030000003oool0oooo0?l0ooooB@3oool00`000000
oooo0?ooo`0200000?l0ooooTP3oool003T0oooo00<000000?ooo`3oool0o`3ooom80?ooo`030000
003oool0oooo00800000o`3ooonC0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooodL0oooo00<0
00000?ooo`3oool00P00003o0?oooi@0oooo000i0?ooo`030000003oool0oooo0?l0ooooAP3oool0
10000000oooo0?ooo`00003o0?oooiH0oooo000i0?ooo`030000003oool0oooo0?l0oooo@03oool6
00000080oooo00<000000?ooo`3oool0o`3ooonE0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
od00oooo1@0000020?ooo`030000003oool0oooo0?l0ooooUP3oool003T0oooo1000003o0?oooch0
oooo1P0000000`3oool000000?ooo`3o0?oooiP0oooo000i0?ooo`030000003oool0oooo0?l0oooo
@03oool600000?l0ooooVP3oool003T0oooo00<000000?ooo`3oool0o`3ooom00?ooo`H00000o`3o
oonJ0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oood00oooo1@00003o0?oooi/0oooo000i0?oo
o`030000003oool0oooo0?l0oooo?`3oool2000000030?ooo`00000000000?l0ooooW03oool003T0
oooo00<000000?ooo`3oool0o`3oooln0?ooo`80000000<0oooo000000000000o`3ooonM0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?ooocd0oooo0P0000020?ooo`030000003oool0oooo0?l0oooo
W03oool003T0oooo00<000000?ooo`3oool0o`3oooll0?ooo`8000000P3oool00`000000oooo0?oo
o`3o0?oooid0oooo000i0?ooo`030000003oool0oooo0?l0oooo>`3oool200000080oooo00<00000
0?ooo`3oool0o`3ooonN0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooocX0oooo0P0000020?oo
o`030000003oool0oooo0?l0ooooW`3oool003T0oooo00<000000?ooo`3oool0o`3oooli0?ooo`80
00000P3oool00`000000oooo0?ooo`3o0?oooj00oooo000i0?ooo`030000003oool0oooo0?l0oooo
>03oool200000080oooo00<000000?ooo`3oool0o`3ooonQ0?ooo`00>@3oool00`000000oooo0?oo
o`3o0?ooocL0oooo0P0000020?ooo`030000003oool0oooo0?l0ooooXP3oool003T0oooo00<00000
0?ooo`3oool0o`3ooolg0?ooo`030000003oool0oooo00800000o`3ooonT0?ooo`00>@3oool00`00
0000oooo0?ooo`3o0?ooocH0oooo00<000000?ooo`3oool00P00003o0?ooojD0oooo000i0?ooo`03
0000003oool0oooo0?l0oooo=@3oool00`000000oooo0?ooo`0200000?l0ooooYP3oool003T0oooo
00<000000?ooo`3oool0o`3ooold0?ooo`050000003oool0oooo0?ooo`000000o`3ooonW0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?oooc<0oooo00D000000?ooo`3oool0oooo0000003o0?ooojP0
oooo000i0?ooo`030000003oool0oooo0?l0oooo<P3oool01@000000oooo0?ooo`3oool000000?l0
ooooZ@3oool003T0oooo1000003o0?oooc00oooo00D000000?ooo`3oool0oooo0000003o0?ooojX0
oooo000i0?ooo`030000003oool0oooo0?l0oooo<03oool01@000000oooo0?ooo`3oool000000?l0
ooooZ`3oool003T0oooo00<000000?ooo`3oool0o`3oool_0?ooo`050000003oool0oooo0?ooo`00
0000o`3ooon/0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooobh0oooo00D000000?ooo`3oool0
oooo0000003o0?ooojd0oooo000i0?ooo`030000003oool0oooo0?l0oooo;@3oool010000000oooo
0?ooo`3oool200000?l0oooo[@3oool003T0oooo00<000000?ooo`3oool0o`3oool]0?ooo`030000
003oool0oooo00800000o`3ooon^0?ooo`00>@3oool00`000000oooo0?ooo`3o0?ooob`0oooo00<0
00000?ooo`3oool00P00003o0?ooojl0oooo000i0?ooo`030000003oool0oooo0?l0oooo:`3oool0
0`000000oooo0?ooo`0200000?l0oooo/03oool003T0oooo00<000000?ooo`3oool0o`3ooolZ0?oo
o`8000000P3oool00`000000oooo0?ooo`3o0?ooojl0oooo000i0?ooo`030000003oool0oooo0?l0
oooo:P3oool010000000oooo0?ooo`00003o0?oook80oooo000i0?ooo`030000003oool0oooo0?l0
oooo:@3oool010000000oooo0?ooo`00003o0?oook<0oooo000i0?ooo`030000003oool0oooo0?l0
oooo:03oool2000000030?ooo`000000oooo0?l0oooo/`3oool003T0oooo00<000000?ooo`3oool0
o`3ooolX0?ooo`030000003oool000000?l0oooo]@3oool003T0oooo00<000000?ooo`3oool0o`3o
oolW0?ooo`040000003oool0000000000?l0oooo]@3oool003T0oooo00<000000?ooo`3oool0o`3o
oolV0?ooo`80000000<0oooo0000003oool0o`3ooone0?ooo`00>@3oool00`000000oooo0?ooo`3o
0?ooobH0oooo00<000000?ooo`000000o`3ooong0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
obD0oooo00@000000?ooo`0000000000o`3ooong0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oo
ob@0oooo00@000000?ooo`3oool00000o`3ooonh0?ooo`00>@3oool400000?l0oooo8`3oool00`00
0000oooo0000003o0?oookT0oooo000i0?ooo`030000003oool0oooo0?l0oooo8`3oool010000000
oooo00000000003o0?oookT0oooo000i0?ooo`030000003oool0oooo0?l0oooo8P3oool010000000
oooo0?ooo`00003o0?oookX0oooo000i0?ooo`030000003oool0oooo0?l0oooo8@3oool200000003
0?ooo`000000oooo0?l0oooo^P3oool003T0oooo00<000000?ooo`3oool0o`3ooolQ0?ooo`030000
003oool000000?l0oooo_03oool003T0oooo00<000000?ooo`3oool0o`3ooolP0?ooo`040000003o
ool0oooo00000?l0oooo_03oool003T0oooo00<000000?ooo`3oool0o`3ooolO0?ooo`80000000<0
oooo0000003oool0o`3ooonl0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooal0oooo00<00000
0?ooo`000000o`3ooonn0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooah0oooo00@000000?oo
o`0000000000o`3ooonn0?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooad0oooo0P0000000`3o
ool000000?ooo`3o0?oookh0oooo000i0?ooo`030000003oool0oooo0?l0oooo7@3oool00`000000
oooo0000003o0?oool00oooo000i0?ooo`030000003oool0oooo0?l0oooo703oool010000000oooo
00000000003o0?oool00oooo000i0?ooo`030000003oool0oooo0?l0oooo5`3oool6000000030?oo
o`000000oooo0?l0oooo`03oool003T0oooo00<000000?ooo`3oool0o`3ooolG0?ooo`D0000000<0
oooo0000003oool0o`3oooo10?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooaH0oooo1`00003o
0?oool<0oooo000i0?ooo`030000003oool0oooo0?l0oooo5`3oool600000?l0oooo``3oool003T0
oooo00<000000?ooo`3oool0o`3ooolG0?ooo`D00000o`3oooo40?ooo`00>@3oool00`000000oooo
0?ooo`3o0?oooaP0oooo0`00003o0?ooolD0oooo000i0?ooo`030000003oool0oooo0?l0oooo5`3o
ool010000000oooo00000000003o0?ooolD0oooo000i0?ooo`@00000o`3ooolE0?ooo`80000000<0
oooo0000003oool0o`3oooo50?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooaH0oooo00<00000
0?ooo`000000o`3oooo70?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooaD0oooo00@000000?oo
o`0000000000o`3oooo70?ooo`00>@3oool00`000000oooo0?ooo`3o0?oooa@0oooo0P0000000`3o
ool000000?ooo`3o0?ooolL0oooo000i0?ooo`030000003oool0oooo0?l0oooo503oool00`000000
oooo0000003o0?ooolT0oooo000i0?ooo`030000003oool0oooo0?l0oooo4`3oool00`000000oooo
0000003o0?ooolX0oooo000i0?ooo`030000003oool0oooo0?l0oooo4P3oool2000000030?ooo`00
0000oooo0?l0oooob@3oool003T0oooo00<000000?ooo`3oool0o`3ooolB0?ooo`030000003oool0
00000?l0oooob`3oool003T0oooo00<000000?ooo`3oool0o`3ooolA0?ooo`030000003oool00000
0?l0ooooc03oool003T0oooo00<000000?ooo`3oool0o`3oool@0?ooo`040000003oool000000000
0?l0ooooc03oool003T0oooo00<000000?ooo`3oool0o`3oool@0?ooo`030000003oool000000?l0
ooooc@3oool003T0oooo00<000000?ooo`3oool0o`3oool?0?ooo`030000003oool000000?l0oooo
cP3oool003T0oooo00<000000?ooo`3oool0o`3oool>0?ooo`@00000o`3oooo>0?ooo`00>@3oool0
0`000000oooo0?ooo`3o0?ooo`h0oooo00<000000?ooo`000000o`3oooo?0?ooo`00403oool40000
00T0oooo0P0000070?ooo`P000002`3oool00`000000oooo0?ooo`3o0?ooo`d0oooo00<000000?oo
o`000000o`3oooo@0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool01P3oool3000000X0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3o0?ooo`d0oooo0`00003o0?ooom00
oooo000?0?ooo`030000003oool0oooo0080oooo0P0000070?ooo`<000002P3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo0?l0oooo303oool00`000000oooo0000003o0?ooom40oooo
000>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0?l0oooo303oool200000?l0oooodP3oool000h0oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0o`3oool;0?ooo`<00000o`3ooooB0?ooo`003P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0303oool600000?l0oooo203oool20000
0?l0ooood`3oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0B0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3oool:0?ooo`<00000o`3ooooC0?ooo`00
3P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`3o0?ooo`T0oooo0`00003o0?ooom@0oooo000>0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0?l0oooo2@3oool200000?l0ooooe@3oool000h0oooo0P0000050?ooo`030000003oool0
oooo0180oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`3o0?ooo`P0oooo00<00000
0?ooo`000000o`3ooooE0?ooo`003`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00l0oooo00D000000?ooo`3oool0oooo0000000>0?ooo`030000003oool0oooo0?l0oooo203oool2
00000?l0ooooeP3oool000l0oooo0P0000030?ooo`030000003oool0oooo0100oooo1000000>0?oo
o`030000003oool0oooo0?l0oooo1`3oool00`000000oooo0000003o0?ooomH0oooo000@0?ooo`@0
00005@3oool2000000h0oooo00<000000?ooo`3oool0o`3oool70?ooo`800000o`3ooooG0?ooo`00
>@3oool00`000000oooo0?ooo`3o0?ooo`H0oooo00<000000?ooo`000000o`3ooooG0?ooo`00>@3o
ool00`000000oooo0?ooo`3o0?ooo`H0oooo0P00003o0?ooomP0oooo000i0?ooo`030000003oool0
oooo0?l0oooo1@3oool00`000000oooo0000003o0?ooomP0oooo000i0?ooo`030000003oool0oooo
0?l0oooo103oool300000?l0oooof@3oool003T0oooo00<000000?ooo`3oool0o`3oool40?ooo`03
0000003oool000000?l0oooof@3oool003T0oooo00<000000?ooo`3oool0o`3oool30?ooo`030000
003oool000000?l0oooofP3oool003T0oooo00<000000?ooo`3oool0o`3oool30?ooo`030000003o
ool000000?l0oooofP3oool003T0oooo00<000000?ooo`3oool0oP3oool600000?l0oooof`3oool0
03T0oooo00<000000?ooo`3oool0oP3oool600000?l0oooof`3oool003T0oooo00<000000?ooo`3o
ool0o@3oool600000?l0oooog03oool003T0oooo1000003m0?ooo`D00000o`3ooooL0?ooo`00>@3o
ool00`000000oooo0?ooo`3n0?ooo`D00000o`3ooooL0?ooo`00>@3oool00`000000oooo0?ooo`3n
0?ooo`@00000o`3ooooM0?ooo`00>@3oool00`000000oooo0?ooo`3n0?ooo`030000003oool00000
0?l0oooogP3oool003T0oooo00<000000?ooo`3oool0oP3oool00`000000oooo0000003o0?ooomh0
oooo000i0?ooo`030000003oool0oooo0?d0oooo00<000000?ooo`000000o`3ooooO0?ooo`00>@3o
ool00`000000oooo0?ooo`3m0?ooo`030000003oool000000?l0oooog`3oool003T0oooo00<00000
0?ooo`3oool0o03oool00`000000oooo0000003o0?ooon00oooo000i0?ooo`030000003oool0oooo
0?`0oooo0`00003o0?ooon00oooo000i0?ooo`030000003oool0oooo0?/0oooo00<000000?ooo`00
0000o`3ooooQ0?ooo`00>@3oool00`000000oooo0?ooo`3k0?ooo`<00000o`3ooooQ0?ooo`00>@3o
ool00`000000oooo0?ooo`3j0?ooo`<00000o`3ooooR0?ooo`00>@3oool00`000000oooo0?ooo`3j
0?ooo`<00000o`3ooooR0?ooo`00>@3oool00`000000oooo0?ooo`3j0?ooo`800000o`3ooooS0?oo
o`00>@3oool00`000000oooo0?ooo`3i0?ooo`800000o`3ooooT0?ooo`00>@3oool00`000000oooo
0?ooo`3i0?ooo`800000o`3ooooT0?ooo`00>@3oool00`000000oooo0?ooo`3h0?ooo`800000o`3o
oooU0?ooo`00>@3oool00`000000oooo0?ooo`3h0?ooo`800000o`3ooooU0?ooo`00>@3oool00`00
0000oooo0?ooo`3g0?ooo`800000o`3ooooV0?ooo`00>@3oool400000?H0oooo0P00003o0?ooonH0
oooo000i0?ooo`030000003oool0oooo0?H0oooo0P00003o0?ooonL0oooo000i0?ooo`030000003o
ool0oooo0?H0oooo0P00003o0?ooonL0oooo000i0?ooo`030000003oool0oooo0?H0oooo00<00000
0?ooo`3oool0o`3ooooV0?ooo`00>@3oool00`000000oooo0?ooo`3e0?ooo`800000o`3ooooX0?oo
o`00>@3oool00`000000oooo0?ooo`3e0?ooo`030000003oool0oooo0?l0ooooi`3oool003T0oooo
00<000000?ooo`3oool0m03oool200000?l0ooooj@3oool003T0oooo00<000000?ooo`3oool0m03o
ool00`000000oooo0?ooo`3o0?ooonP0oooo000i0?ooo`030000003oool0oooo0?<0oooo0P00003o
0?ooonX0oooo000i0?ooo`030000003oool0oooo0?<0oooo00<000000?ooo`3oool0o`3ooooY0?oo
o`00>@3oool00`000000oooo0?ooo`3b0?ooo`800000o`3oooo[0?ooo`00>@3oool00`000000oooo
0?ooo`3b0?ooo`030000003oool0oooo0?l0oooojP3oool003T0oooo00<000000?ooo`3oool0lP3o
ool00`000000oooo0?ooo`3o0?ooonX0oooo000i0?ooo`030000003oool0oooo0?40oooo00<00000
0?ooo`3oool0o`3oooo[0?ooo`00>@3oool00`000000oooo0?ooo`3a0?ooo`030000003oool0oooo
0?l0ooooj`3oool003T0oooo00<000000?ooo`3oool0l03oool00`000000oooo0?ooo`3o0?ooon`0
oooo000i0?ooo`030000003oool0oooo0?00oooo00<000000?ooo`3oool0o`3oooo/0?ooo`00>@3o
ool00`000000oooo0?ooo`3_0?ooo`030000003oool0oooo0?l0ooook@3oool003T0oooo00<00000
0?ooo`3oool0kP3oool200000?l0ooook`3oool003T0oooo1000003]0?ooo`800000o`3oooo_0?oo
o`00>@3oool00`000000oooo0?ooo`3]0?ooo`800000o`3oooo`0?ooo`00>@3oool00`000000oooo
0?ooo`3]0?ooo`800000o`3oooo`0?ooo`00>@3oool00`000000oooo0?ooo`3]0?ooo`800000o`3o
ooo`0?ooo`00>@3oool00`000000oooo0?ooo`3/0?ooo`800000o`3ooooa0?ooo`00>@3oool00`00
0000oooo0?ooo`3/0?ooo`800000o`3ooooa0?ooo`00>@3oool00`000000oooo0?ooo`3[0?ooo`03
0000003oool000000?l0ooool@3oool003T0oooo00<000000?ooo`3oool0j`3oool200000?l0oooo
lP3oool003T0oooo00<000000?ooo`3oool0j`3oool200000?l0oooolP3oool003T0oooo00<00000
0?ooo`3oool0jP3oool00`000000oooo0000003o0?oooo80oooo000i0?ooo`030000003oool0oooo
0>X0oooo0P00003o0?oooo<0oooo000i0?ooo`030000003oool0oooo0>T0oooo0`00003o0?oooo<0
oooo000i0?ooo`030000003oool0oooo0>T0oooo00<000000?ooo`000000o`3ooooc0?ooo`00>@3o
ool00`000000oooo0?ooo`3Y0?ooo`800000o`3ooood0?ooo`00>@3oool00`000000oooo0?ooo`3X
0?ooo`030000003oool000000?l0oooom03oool003T0oooo00<000000?ooo`3oool0j03oool00`00
0000oooo0000003o0?oooo@0oooo000i0?ooo`030000003oool0oooo0>P0oooo0P00003o0?ooooD0
oooo000i0?ooo`030000003oool0oooo0>L0oooo00<000000?ooo`000000o`3ooooe0?ooo`00>@3o
ool400000>H0oooo00<000000?ooo`000000o`3ooooe0?ooo`00>@3oool00`000000oooo0?ooo`3V
0?ooo`030000003oool000000?l0oooomP3oool003T0oooo00<000000?ooo`3oool0iP3oool00`00
0000oooo0000003o0?ooooH0oooo000i0?ooo`030000003oool0oooo0>H0oooo00<000000?ooo`00
0000o`3oooof0?ooo`00>@3oool00`000000oooo0?ooo`3U0?ooo`030000003oool000000?l0oooo
m`3oool003T0oooo00<000000?ooo`3oool0h`3oool500000?l0oooom`3oool003T0oooo00<00000
0?ooo`3oool0h`3oool500000?l0oooom`3oool003T0oooo00<000000?ooo`3oool0hP3oool60000
0?l0oooom`3oool003T0oooo00<000000?ooo`3oool0h`3oool500000?l0oooom`3oool003T0oooo
00<000000?ooo`3oool0h`3oool500000?l0oooom`3oool003T0oooo00<000000?ooo`3oool0h`3o
ool00`000000oooo0000003o0?ooooT0oooo000i0?ooo`030000003oool0oooo0><0oooo00<00000
0?ooo`000000o`3ooooi0?ooo`00>@3oool00`000000oooo0?ooo`3R0?ooo`040000003oool0oooo
00000?l0oooon@3oool003T0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0000003o0?oo
ooX0oooo000@0?ooo`@000002@3oool2000000L0oooo2000000;0?ooo`030000003oool0oooo0>40
oooo00@000000?ooo`3oool00000o`3ooooj0?ooo`003`3oool2000000<0oooo00<000000?ooo`3o
ool01P3oool3000000H0oooo0P0000050?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool0h@3oool010000000oooo0?ooo`00003o0?ooooX0oooo000?0?ooo`030000003oool0oooo0080
oooo0P0000070?ooo`<00000203oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0h@3oool00`000000oooo0000003o0?oooo/0oooo000>0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo0>00oooo00@000000?ooo`3oool00000o`3ooook0?ooo`003P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`3P0?ooo`040000003oool0oooo00000?l0oooon`3oool000h0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00/0oooo1P00003L0?ooo`800000
00<0oooo0000003oool0o`3ooook0?ooo`003P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`3O0?ooo`040000003o
ool0oooo00000?l0ooooo03oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`0D0?ooo`8000002`3oool00`000000oooo0?ooo`3O0?ooo`040000003oool0oooo00000?l0oooo
o03oool000h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0E0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0gP3oool010000000oooo0?ooo`00003o0?ooood0oooo
000>0?ooo`8000001@3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0gP3oool010000000oooo0?ooo`00003o0?ooood0oooo000?0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0g@3oool200000080oooo00<000000?ooo`3oool0o`3o
oook0?ooo`003`3oool2000000<0oooo00<000000?ooo`3oool03`3oool2000000<0oooo0P00000<
0?ooo`030000003oool0oooo0=d0oooo00@000000?ooo`3oool00000o`3oooon0?ooo`00403oool4
000001@0oooo1000000=0?ooo`030000003oool0oooo0=d0oooo00@000000?ooo`3oool00000o`3o
ooon0?ooo`00>@3oool00`000000oooo0?ooo`3L0?ooo`050000003oool0oooo0?ooo`000000o`3o
ooon0?ooo`00>@3oool00`000000oooo0?ooo`3L0?ooo`040000003oool0oooo00000?l0ooooo`3o
ool003T0oooo00<000000?ooo`3oool0g03oool010000000oooo0?ooo`00003o0?ooool0oooo000i
0?ooo`030000003oool0oooo0=/0oooo00D000000?ooo`3oool0oooo0000003o0?ooool0oooo000i
0?ooo`030000003oool0oooo0=/0oooo00@000000?ooo`3oool00000o`3ooooo0?ooo`40oooo000i
0?ooo`030000003oool0oooo0=X0oooo00D000000?ooo`3oool0oooo0000003o0?ooool0oooo0@3o
ool003T0oooo00<000000?ooo`3oool0fP3oool01@000000oooo0?ooo`3oool000000?l0ooooo`3o
ool10?ooo`00>@3oool00`000000oooo0?ooo`3J0?ooo`040000003oool0oooo00000?l0ooooo`3o
ool20?ooo`00>@3oool00`000000oooo0?ooo`3I0?ooo`050000003oool0oooo0?ooo`000000o`3o
oooo0?ooo`80oooo000i0?ooo`030000003oool0oooo0=T0oooo00D000000?ooo`3oool0oooo0000
003o0?ooool0oooo0P3oool003T0oooo1000003G0?ooo`8000000P3oool00`000000oooo0?ooo`3o
0?ooool0oooo0@3oool003T0oooo00<000000?ooo`3oool0f03oool01@000000oooo0?ooo`3oool0
00000?l0ooooo`3oool30?ooo`00>@3oool00`000000oooo0?ooo`3H0?ooo`050000003oool0oooo
0?ooo`000000o`3ooooo0?ooo`<0oooo000i0?ooo`030000003oool0oooo0=L0oooo00D000000?oo
o`3oool0oooo0000003o0?ooool0oooo103oool003T0oooo00<000000?ooo`3oool0e`3oool01@00
0000oooo0?ooo`3oool000000?l0ooooo`3oool40?ooo`00>@3oool00`000000oooo0?ooo`3G0?oo
o`050000003oool0oooo0?ooo`000000o`3ooooo0?ooo`@0oooo000i0?ooo`030000003oool0oooo
0=H0oooo00D000000?ooo`3oool0oooo0000003o0?ooool0oooo1@3oool003T0oooo00<000000?oo
o`3oool0eP3oool01@000000oooo0?ooo`3oool000000?l0ooooo`3oool50?ooo`00>@3oool00`00
0000oooo0?ooo`3E0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3ooooo0?oo
o`<0oooo000i0?ooo`030000003oool0oooo0=D0oooo00D000000?ooo`3oool0oooo0000003o0?oo
ool0oooo1P3oool003T0oooo00<000000?ooo`3oool0e@3oool01@000000oooo0?ooo`3oool00000
0?l0ooooo`3oool60?ooo`00>@3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0o`3ooooo0?ooo`@0oooo000i0?ooo`030000003oool0oooo0=@0oooo
00D000000?ooo`3oool0oooo0000003o0?ooool0oooo1`3oool003T0oooo00<000000?ooo`3oool0
d`3oool2000000<0oooo00<000000?ooo`3oool0o`3ooooo0?ooo`D0oooo000i0?ooo`030000003o
ool0oooo0=<0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooool0oooo1@3o
ool003T0oooo00<000000?ooo`3oool0d`3oool01@000000oooo0?ooo`3oool000000?l0ooooo`3o
ool80?ooo`00>@3oool00`000000oooo0?ooo`3B0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0o`3ooooo0?ooo`H0oooo000i0?ooo`030000003oool0oooo0=80oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`3o0?ooool0oooo1P3oool003T0oooo00<000000?ooo`3o
ool0dP3oool01@000000oooo0?ooo`3oool000000?l0ooooo`3oool90?ooo`00>@3oool400000=00
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooool0oooo1`3oool003T0oooo
00<000000?ooo`3oool0d@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0oooo
o`3oool70?ooo`00>@3oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0o`3ooooo0?ooo`P0oooo000i0?ooo`030000003oool0oooo0=00oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`3o0?ooool0oooo203oool003T0oooo00<000000?oo
o`3oool0d03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0ooooo`3oool80?oo
o`00>@3oool00`000000oooo0?ooo`3?0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0o`3ooooo0?ooo`T0oooo000i0?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`3o0?ooool0oooo2@3oool003T0oooo00<000000?ooo`3oool0c`3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?l0ooooo`3oool90?ooo`00>@3oool0
0`000000oooo0?ooo`3>0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3ooooo
0?ooo`X0oooo000i0?ooo`030000003oool0oooo0<h0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`3o0?ooool0oooo2P3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?ooool0oooo
o`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo003o0?oo
ool0ooooo`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oooad0oooo
003o0?ooool0ooooo`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool00?l0ooooo`3ooooo0?oo
oad0oooo003o0?ooool0ooooo`3ooolM0?ooo`00o`3ooooo0?ooool0oooo7@3oool00001\
\>"],
  ImageRangeCache->{{{91.5625, 408.312}, {554.25, 358.875}} -> {-0.319694, \
0.702205, 0.00223318, 0.00215081}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{1140, 1115},
WindowMargins->{{97, Automatic}, {Automatic, 20}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "K_convergence_dlcq.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 36, 0, 132, "Subtitle"],
Cell[1778, 53, 60, 3, 132, "Subsubtitle"],
Cell[1841, 58, 177, 4, 92, "Text"],
Cell[2021, 64, 228, 5, 112, "Input",
  InitializationCell->True],
Cell[2252, 71, 579, 10, 292, "Input",
  InitializationCell->True],
Cell[2834, 83, 75, 0, 60, "Text"],
Cell[2912, 85, 226, 4, 142, "Input"],
Cell[3141, 91, 137, 3, 92, "Text"],

Cell[CellGroupData[{
Cell[3303, 98, 131, 3, 82, "Input"],
Cell[3437, 103, 69, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3543, 109, 92, 1, 52, "Input"],
Cell[3638, 112, 69, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3744, 118, 58, 0, 106, "Section"],

Cell[CellGroupData[{
Cell[3827, 122, 53, 0, 96, "Subsection"],
Cell[3883, 124, 287, 6, 156, "Text"],
Cell[4173, 132, 1544, 22, 1282, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[5754, 159, 53, 0, 60, "Subsection"],
Cell[5810, 161, 448, 10, 188, "Text"],
Cell[6261, 173, 1611, 24, 742, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[7909, 202, 38, 0, 60, "Subsection"],
Cell[7950, 204, 668, 10, 322, "Input"],
Cell[8621, 216, 613, 9, 292, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[9271, 230, 39, 0, 60, "Subsection"],
Cell[9313, 232, 48, 0, 60, "Text"],
Cell[9364, 234, 315, 6, 207, "Input",
  InitializationCell->True],
Cell[9682, 242, 314, 6, 209, "Input",
  InitializationCell->True],
Cell[9999, 250, 291, 6, 179, "Input",
  InitializationCell->True],
Cell[10293, 258, 381, 9, 172, "Input",
  InitializationCell->True],
Cell[10677, 269, 94, 3, 60, "Text"],
Cell[10774, 274, 1131, 20, 382, "Input"],
Cell[11908, 296, 745, 13, 292, "Input"],
Cell[12656, 311, 737, 13, 292, "Input"],
Cell[13396, 326, 699, 12, 292, "Input"],
Cell[14098, 340, 36, 0, 60, "Text"],
Cell[14137, 342, 210, 6, 172, "Input"],
Cell[14350, 350, 124, 3, 82, "Input"],
Cell[14477, 355, 206, 5, 112, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14720, 365, 29, 0, 96, "Subsection"],

Cell[CellGroupData[{
Cell[14774, 369, 147, 3, 82, "Input"],
Cell[14924, 374, 211, 3, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15172, 382, 67, 1, 52, "Input"],
Cell[15242, 385, 170, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15449, 393, 147, 3, 82, "Input"],
Cell[15599, 398, 210, 3, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15846, 406, 88, 1, 52, "Input"],
Cell[15937, 409, 114, 2, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16088, 416, 67, 1, 52, "Input"],
Cell[16158, 419, 167, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16362, 427, 89, 1, 52, "Input"],
Cell[16454, 430, 146, 2, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16637, 437, 156, 3, 82, "Input"],
Cell[16796, 442, 243, 4, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17076, 451, 72, 1, 52, "Input"],
Cell[17151, 454, 192, 4, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17380, 463, 67, 1, 52, "Input"],
Cell[17450, 466, 56, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17543, 472, 152, 3, 82, "Input"],
Cell[17698, 477, 246, 4, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17981, 486, 69, 1, 52, "Input"],
Cell[18053, 489, 190, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18280, 497, 63, 1, 52, "Input"],
Cell[18346, 500, 56, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18439, 506, 1463, 28, 802, "Input"],
Cell[19905, 536, 130, 3, 52, "Output"]
}, Open  ]],
Cell[20050, 542, 47, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[20122, 546, 1971, 36, 865, "Input"],
Cell[22096, 584, 130, 3, 52, "Output"]
}, Open  ]],
Cell[22241, 590, 288, 7, 214, "Text"],
Cell[22532, 599, 49, 0, 60, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22630, 605, 23, 0, 106, "Section"],
Cell[22656, 607, 647, 11, 284, "Text"],

Cell[CellGroupData[{
Cell[23328, 622, 1166, 34, 908, "Input"],
Cell[24497, 658, 181, 3, 79, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24715, 666, 29, 0, 96, "Subsection"],

Cell[CellGroupData[{
Cell[24769, 670, 119, 3, 52, "Input"],
Cell[24891, 675, 219, 4, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25147, 684, 85, 1, 52, "Input"],
Cell[25235, 687, 184, 3, 87, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25456, 695, 57, 1, 52, "Input"],
Cell[25516, 698, 56, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25609, 704, 67, 1, 52, "Input"],
Cell[25679, 707, 144, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25860, 715, 288, 6, 142, "Input"],
Cell[26151, 723, 36297, 718, 372, 4774, 324, "GraphicsData", "PostScript", \
"Graphics"],
Cell[62451, 1443, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[62618, 1451, 125, 3, 82, "Input"],
Cell[62746, 1456, 203, 3, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[62986, 1464, 57, 1, 52, "Input"],
Cell[63046, 1467, 54, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[63137, 1473, 66, 1, 52, "Input"],
Cell[63206, 1476, 163, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[63406, 1484, 290, 6, 142, "Input"],
Cell[63699, 1492, 35311, 666, 372, 4078, 276, "GraphicsData", "PostScript", \
"Graphics"],
Cell[99013, 2160, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99180, 2168, 82, 1, 52, "Input"],
Cell[99265, 2171, 147, 2, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99449, 2178, 57, 1, 52, "Input"],
Cell[99509, 2181, 57, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99603, 2187, 72, 1, 52, "Input"],
Cell[99678, 2190, 121, 2, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99836, 2197, 58, 1, 52, "Input"],
Cell[99897, 2200, 55, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[99989, 2206, 68, 1, 52, "Input"],
Cell[100060, 2209, 213, 4, 82, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100310, 2218, 430, 9, 232, "Input"],
Cell[100743, 2229, 44931, 783, 507, 3935, 273, "GraphicsData", "PostScript", \
"Graphics"],
Cell[145677, 3014, 130, 3, 52, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

