(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    577541,      11929]*)
(*NotebookOutlinePosition[    579064,      11976]*)
(*  CellTagsIndexPosition[    579020,      11972]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Examine meson wavfunctions", "Subtitle"],

Cell["\<\
Brett van de Sande
January 2003\
\>", "Subsubtitle"],

Cell["\<\
These are wavefunctions that were generated on the TCS.  In this \
case, the various observables were evaluated in the C-code.\
\>", "Text"],

Cell[BoxData[
    \(<< Graphics`Graphics`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(rmserror[data_, fit_, var_] := 
      Sqrt[Apply[Plus, 
            Map[\((\((fit /. var \[Rule] #[\([1]\)])\) - #[\([2]\)])\)^2 &, 
              data]]/Length[data]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(NonLinearFit[data_, fit_, var_, coeff_List] := 
      Block[{v = If[Length[coeff] > 1, Map[First, coeff], var]}, 
        fit /. \(Apply[
              FindMinimum[rmserror[data, fit, var], ##, 
                  MaxIterations \[Rule] 100, 
                  AccuracyGoal \[Rule] Infinity] &, 
              coeff]\)[\([2]\)]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Data", "Section"],

Cell[BoxData[
    \(<< "\<jan_3a.m\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< "\<jan_3a_fits.m\>"\)], "Input"],

Cell[BoxData[
    \(\(-0.0036246242228034076`\) + 2.192102384106045`\/k\^2 - 
      1.497432761056963`\/k - 0.2756736584113894`\ k - 
      0.003521740282956598`\ Log[k]\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basis size and run speed", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fittingb = Fit[basisSize, {k^2, k^3, k^4}, k]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(fittingb\)\" is similar \
to existing symbol \"\!\(fitting\)\"."\)], "Message"],

Cell[BoxData[
    \(5.985719362135418`\ k\^2 - 3.322134972878055`\ k\^3 + 
      0.6665562315106752`\ k\^4\)], "Output"]
}, Open  ]],

Cell["Estimates:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((fittingb /. k \[Rule] 100)\)*5\)], "Input"],

Cell[BoxData[
    \(3.169667268590541`*^8\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Show[LogPlot[fittingb, {k, 10, 50}, DisplayFunction \[Rule] Identity], 
      LogListPlot[basisSize, DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((fittingb /. k \[Rule] 100)\)*400/1400*16\)], "Input"],

Cell[BoxData[
    \(2.897981502711352`*^8\)], "Output"]
}, Open  ]],

Cell["Check that the leading power is as expected:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fit[Log[nonzero], {1/x, 1/x^2, 1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(1.6563288176295543`\)\(\[InvisibleSpace]\)\) - 
      2.4716323754921654`\/x\^2 - 1.465261434036889`\/x + 
      4.919450298495392`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = Fit[nonzero, {k^3, k^4, k^5}, k]\)], "Input"],

Cell[BoxData[
    \(15.725062184662798`\ k\^3 - 11.777532781976511`\ k\^4 + 
      2.4665286033489946`\ k\^5\)], "Output"]
}, Open  ]],

Cell["Estimates:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((fitting /. k \[Rule] 100)\)/1400*24\)], "Input"],

Cell[BoxData[
    \(4.029129911567478`*^8\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((fitting /. k \[Rule] 100)\)/1400\)], "Input"],

Cell[BoxData[
    \(1.6788041298197825`*^7\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[LogPlot[fitting, {k, 10, 50}, DisplayFunction \[Rule] Identity], 
      LogListPlot[nonzero, DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
The correct functional form of the Lanczos speed is somewhat \
ambiguous, since it depends somewhat on the number of reorthogonalizations \
and the number of Lanczos steps.  For the parallel code, the small K behavior \
is complicated.\
\>", "Text"],

Cell["First, try to find the functional form for one step:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(oneStep = 
      Table[{klist[\([i]\)], 
          timeLanczos[\([i, 2]\)]/lanczosSteps[\([i, 2]\)]}, {i, 
          Length[klist]}]\)], "Input"],

Cell[BoxData[
    \({{10, 1.7493333333333334`}, {12, 1.6237037037037039`}, {15, 
        2.512592592592593`}, {20, 6.349206349206349`}, {30, 
        42.75358024691358`}, {40, 154.53155555555557`}, {50, 
        510.99733333333336`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fit[Log[Drop[oneStep, 4]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(-12.732943816140756`\) + 4.838262499639623`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting0 = Fit[Drop[oneStep, 3], {k^3, k^4, k^5}, k]\)], "Input"],

Cell[BoxData[
    \(0.002752501395277329`\ k\^3 - 0.00014651986408251123`\ k\^4 + 
      3.4639098643178315`*^-6\ k\^5\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[oneStep, fitting0, k]\)], "Input"],

Cell[BoxData[
    \(1.9314505276156588`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Expand[fitting0*lanczosSteps[\([\(-1\), 2]\)]]\)], "Input"],

Cell[BoxData[
    \(0.24772512557495963`\ k\^3 - 0.01318678776742601`\ k\^4 + 
      0.0003117518877886048`\ k\^5\)], "Output"]
}, Open  ]],

Cell["Now, find the functional form for all of the lanczos steps:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fit[Log[Drop[timeLanczos, 3]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(-9.416065697607284`\) + 5.14951250111363`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting1 = Fit[Drop[timeLanczos, 3], {k^3, k^4, k^5}, k]\)], "Input"],

Cell[BoxData[
    \(0.1422615765067615`\ k\^3 - 0.00850911907180337`\ k\^4 + 
      0.00026040199123118573`\ k\^5\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[timeLanczos, fitting1, k]\)], "Input"],

Cell[BoxData[
    \(119.43836953585996`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fit[Log[Drop[timeSingleLanczos, 2]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(-13.321234954899735`\) + 5.729769497948235`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting2 = Fit[timeSingleLanczos, {k^3, k^4, k^5}, k]\)], "Input"],

Cell[BoxData[
    \(0.00027874313757342625`\ k\^3 - 0.00021471822683960575`\ k\^4 + 
      0.00002503962525751264`\ k\^5\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[timeSingleLanczos, fitting2, k]\)], "Input"],

Cell[BoxData[
    \(0.17029823542640202`\)], "Output"]
}, Open  ]],

Cell["Multiply the single-processor results by 64 for comparision:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      LogPlot[fitting1, {k, 10, 50}, DisplayFunction \[Rule] Identity, 
        PlotStyle \[Rule] {Hue[0]}], 
      LogListPlot[timeLanczos, DisplayFunction \[Rule] Identity, 
        PlotStyle \[Rule] {Hue[0]}], 
      LogPlot[fitting2*64, {k, 10, 50}, DisplayFunction \[Rule] Identity], 
      LogListPlot[Map[{#[\([1]\)], #[\([2]\)]*64} &, timeSingleLanczos], 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction, 
      PlotRange \[Rule] All]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fitting1] /. k \[Rule] 30\)], "Input"],

Cell[BoxData[
    \({3841.0625656825605`, \(-6892.38644816073`\), 
      6327.768386917814`}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Eigenvalue", "Section"],

Cell[BoxData[
    \(<< "\<jan_1.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(-0.5057848663070794`\) + 3.8727746426452203`\/k\^2 - 
      0.1039900081767631`\/k - 0.010358811973834557`\ k + 
      0.23903608536559426`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[fitMassSquared /. k \[Rule] K]\)], "Input"],

Cell[BoxData[
    \(TraditionalForm\`\(-0.010358811973834557`\)\ K + 
      0.23903608536559426`\ \(log(
          K)\) - \(\(0.5057848663070794`\)\(\[InvisibleSpace]\)\) - 
      0.1039900081767631`\/K + 3.8727746426452203`\/K\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitt = Drop[fitMassSquared, {2, 3}]\)], "Input"],

Cell[BoxData[
    \(\(-0.5057848663070794`\) - 0.010358811973834557`\ k + 
      0.23903608536559426`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(1/K^2\)], "Input"],

Cell[BoxData[
    \(1\/K\^2\)], "Output"]
}, Open  ]],

Cell[BoxData[
    RowBox[{"Show", "[", 
      RowBox[{\(Plot[{fitt, fitMassSquared}, {k, 10, 50}, 
          FrameLabel \[Rule] {TraditionalForm[K], TraditionalForm[M^2]}, 
          Frame \[Rule] True, Axes \[Rule] False, RotateLabel \[Rule] False, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          PlotStyle \[Rule] {{Thickness[0.005], 
                GrayLevel[0.6]}, {Thickness[0.005]}}, 
          DisplayFunction \[Rule] Identity]\), 
        ",", \(ListPlot[Transpose[{klist, massSquared}], 
          PlotStyle \[Rule] {PointSize[0.015]}, 
          DisplayFunction \[Rule] Identity]\), 
        ",", \(DisplayFunction \[Rule] $DisplayFunction\), ",", 
        RowBox[{"Epilog", "\[Rule]", 
          RowBox[{"{", 
            RowBox[{\(GrayLevel[0.6]\), ",", 
              RowBox[{"Text", "[", 
                RowBox[{"\"\<\!\(1\/\*
StyleBox[\\\"K\\\",\\nFontSlant->\\\"Italic\\\"]\) and \!\(1\/\*
StyleBox[\\\"K\\\",\\nFontSlant->\\\"Italic\\\"]\^2\) terms dropped\>\"", 
                  ",", \({12, \(-0.05\)}\), ",", \({\(-1\), 0}\)}], "]"}]}], 
            "}"}]}], 
        ",", \(PlotLabel -> 
          TraditionalForm[fitMassSquared /. k \[Rule] K]\)}], "]"}]], "Input"],

Cell[BoxData[
    RowBox[{"\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
        "Pion", " ", \(mass\^2\), " ", "as", " ", "a", " ", "function", " ", 
          "of", " ", 
          StyleBox["K",
            FontSlant->"Italic"]}], ",", 
        " ", \(along\ with\ a\ \(\(fit\)\(.\)\)\)}]}]], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->19],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0238095 0.552262 6.03027 [
[.02381 -0.0125 -9 -14.5 ]
[.02381 -0.0125 9 0 ]
[.2619 -0.0125 -9 -14.5 ]
[.2619 -0.0125 9 0 ]
[.5 -0.0125 -9 -14.5 ]
[.5 -0.0125 9 0 ]
[.7381 -0.0125 -9 -14.5 ]
[.7381 -0.0125 9 0 ]
[.97619 -0.0125 -9 -14.625 ]
[.97619 -0.0125 9 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -7.03125 -29.125 ]
[.5 -0.0125 7.03125 -14.625 ]
[-0.0125 .06984 -38.1875 -7.25 ]
[-0.0125 .06984 0 7.25 ]
[-0.0125 .19045 -38.1875 -7.28125 ]
[-0.0125 .19045 0 7.28125 ]
[-0.0125 .31105 -38.1875 -7.25 ]
[-0.0125 .31105 0 7.25 ]
[-0.0125 .43166 -38.1875 -7.25 ]
[-0.0125 .43166 0 7.25 ]
[-0.0125 .55226 -11 -7.25 ]
[-0.0125 .55226 0 7.25 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -60.625 -8.65625 ]
[-0.01875 .30902 -38.1875 8.65625 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -196.906 0 ]
[.5 .63053 196.906 30.375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -70 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.2619 0 m
.2619 .00625 L
s
gsave
.2619 -0.0125 -70 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.5 0 m
.5 .00625 L
s
gsave
.5 -0.0125 -70 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(30) show
1.000 setlinewidth
grestore
.7381 0 m
.7381 .00625 L
s
gsave
.7381 -0.0125 -70 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -70 -18.625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.625 translate 1 -1 scale
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(50) show
1.000 setlinewidth
grestore
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -68.0312 -33.125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(K) show
1.000 setlinewidth
grestore
0 .06984 m
.00625 .06984 L
s
gsave
-0.0125 .06984 -99.1875 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
72.688 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.08) show
1.000 setlinewidth
grestore
0 .19045 m
.00625 .19045 L
s
gsave
-0.0125 .19045 -99.1875 -11.2812 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5625 translate 1 -1 scale
63.000 14.562 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
72.688 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.06) show
1.000 setlinewidth
grestore
0 .31105 m
.00625 .31105 L
s
gsave
-0.0125 .31105 -99.1875 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
72.688 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.04) show
1.000 setlinewidth
grestore
0 .43166 m
.00625 .43166 L
s
gsave
-0.0125 .43166 -99.1875 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
72.688 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.02) show
1.000 setlinewidth
grestore
0 .55226 m
.00625 .55226 L
s
gsave
-0.0125 .55226 -72 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .09999 m
.00375 .09999 L
s
0 .13014 m
.00375 .13014 L
s
0 .16029 m
.00375 .16029 L
s
0 .2206 m
.00375 .2206 L
s
0 .25075 m
.00375 .25075 L
s
0 .2809 m
.00375 .2809 L
s
0 .3412 m
.00375 .3412 L
s
0 .37135 m
.00375 .37135 L
s
0 .40151 m
.00375 .40151 L
s
0 .46181 m
.00375 .46181 L
s
0 .49196 m
.00375 .49196 L
s
0 .52211 m
.00375 .52211 L
s
0 .03969 m
.00375 .03969 L
s
0 .00954 m
.00375 .00954 L
s
0 .58241 m
.00375 .58241 L
s
0 .61256 m
.00375 .61256 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -121.625 -12.6562 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 25.3125 translate 1 -1 scale
63.250 17.312 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(M) show
75.438 11.688 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 9.938 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.2619 .61178 m
.2619 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.7381 .61178 m
.7381 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .06984 m
1 .06984 L
s
.99375 .19045 m
1 .19045 L
s
.99375 .31105 m
1 .31105 L
s
.99375 .43166 m
1 .43166 L
s
.99375 .55226 m
1 .55226 L
s
.125 Mabswid
.99625 .09999 m
1 .09999 L
s
.99625 .13014 m
1 .13014 L
s
.99625 .16029 m
1 .16029 L
s
.99625 .2206 m
1 .2206 L
s
.99625 .25075 m
1 .25075 L
s
.99625 .2809 m
1 .2809 L
s
.99625 .3412 m
1 .3412 L
s
.99625 .37135 m
1 .37135 L
s
.99625 .40151 m
1 .40151 L
s
.99625 .46181 m
1 .46181 L
s
.99625 .49196 m
1 .49196 L
s
.99625 .52211 m
1 .52211 L
s
.99625 .03969 m
1 .03969 L
s
.99625 .00954 m
1 .00954 L
s
.99625 .58241 m
1 .58241 L
s
.99625 .61256 m
1 .61256 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
gsave
.5 .63053 -257.906 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 38.375 translate 1 -1 scale
63.000 23.875 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
72.688 23.875 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.0103588) show
134.812 23.875 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(K) show
147.625 23.875 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
159.375 23.875 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.239036) show
214.500 23.875 moveto
(log) show
%%IncludeResource: font Math2
%%IncludeFont: Math2
/Math2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
232.375 23.875 moveto
(H) show
236.812 23.875 moveto
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(K) show
%%IncludeResource: font Math2
%%IncludeFont: Math2
/Math2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
246.875 23.875 moveto
(L) show
254.062 23.875 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
265.812 23.875 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.505785) show
318.312 23.875 moveto
0.000 0.000 rmoveto
321.062 23.875 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
336.438 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.10399) show
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
334.875 23.875 moveto
(\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\\
200) show
366.875 23.875 moveto
(\\200\\200\\200\\200) show
374.875 23.875 moveto
(\\200\\200) show
378.875 23.875 moveto
(\\200) show
380.875 23.875 moveto
(\\200) show
381.250 23.875 moveto
(\\200) show
354.125 33.250 moveto
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(K) show
388.312 23.875 moveto
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(+) show
403.688 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3.87277) show
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
402.125 23.875 moveto
(\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\200\\\
200) show
434.125 23.875 moveto
(\\200\\200\\200\\200) show
442.125 23.875 moveto
(\\200\\200) show
446.125 23.875 moveto
(\\200) show
448.125 23.875 moveto
(\\200) show
448.500 23.875 moveto
(\\200) show
418.375 33.375 moveto
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(K) show
428.438 29.625 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 9.938 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.6 g
.005 w
.02381 .19664 m
.06244 .31204 L
.10458 .40578 L
.14415 .47059 L
.18221 .5162 L
.20178 .5343 L
.22272 .55014 L
.24402 .56286 L
.26416 .57201 L
.27386 .57549 L
.28414 .57856 L
.29377 .58088 L
.30258 .58255 L
.31199 .58387 L
.31718 .5844 L
.31957 .5846 L
.32211 .58478 L
.32449 .58491 L
.32665 .58501 L
.32882 .58509 L
.32979 .58512 L
.33086 .58514 L
.33202 .58516 L
.33325 .58518 L
.33439 .58519 L
.33545 .58519 L
.3367 .58518 L
.33784 .58517 L
.33916 .58515 L
.34041 .58512 L
.34278 .58505 L
.345 .58496 L
.35007 .58466 L
.355 .58427 L
.36032 .58372 L
.36993 .58241 L
.37874 .58087 L
.39766 .5765 L
.41837 .57015 L
.45893 .55331 L
.49797 .53214 L
.53947 .50489 L
.57945 .47449 L
.61792 .44179 L
.65884 .40362 L
.69824 .36387 L
.73613 .32311 L
.77647 .27719 L
.81529 .23074 L
.85657 .17911 L
Mistroke
.89633 .12735 L
.93457 .07581 L
.97527 .01921 L
.97619 .01792 L
Mfstroke
0 g
.02381 .36747 m
.06244 .43096 L
.10458 .48917 L
.14415 .53198 L
.18221 .56276 L
.20178 .57489 L
.22272 .5853 L
.23293 .58946 L
.24402 .59331 L
.25361 .59611 L
.26416 .59864 L
.2744 .60056 L
.284 .60189 L
.28929 .60243 L
.29413 .60281 L
.2968 .60298 L
.29972 .60312 L
.30105 .60317 L
.30247 .60322 L
.30379 .60325 L
.30503 .60328 L
.30627 .6033 L
.30693 .60331 L
.30763 .60331 L
.30837 .60332 L
.30907 .60332 L
.31039 .60332 L
.31161 .60331 L
.31275 .60329 L
.31399 .60327 L
.31531 .60324 L
.31804 .60315 L
.3206 .60303 L
.32644 .60267 L
.3315 .60224 L
.33615 .60176 L
.34661 .60034 L
.35641 .59862 L
.36553 .59669 L
.38624 .59116 L
.42528 .5767 L
.46678 .55604 L
.50676 .53151 L
.54523 .50404 L
.58615 .47105 L
.62555 .43592 L
.66344 .3993 L
.70378 .35751 L
.7426 .31477 L
.78388 .26683 L
Mistroke
.82364 .21839 L
.86188 .16986 L
.90258 .11628 L
.94176 .06293 L
.97619 .01472 L
Mfstroke
.015 w
.02381 .36716 Mdot
.07143 .44538 Mdot
.14286 .53 Mdot
.2619 .59783 Mdot
.5 .53725 Mdot
.7381 .31859 Mdot
.97619 .01515 Mdot
.6 g
gsave
.07143 .25075 -61 -20.0312 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 40.0625 translate 1 -1 scale
63.000 23.875 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
0.000 0.000 rmoveto
68.125 14.438 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(1) show
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
65.062 23.875 moveto
(\\200\\200\\200\\200) show
73.062 23.875 moveto
(\\200\\200) show
76.000 23.875 moveto
(\\200) show
66.625 33.250 moveto
%%IncludeResource: font Times-Italic
%%IncludeFont: Times-Italic
%%BeginResource: font Times-Italic-MISO
%%BeginFont: Times-Italic-MISO
/Times-Italic /Times-Italic-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(K) show
83.812 23.875 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(and) show
115.250 14.438 moveto
(1) show
%%IncludeResource: font Math1
%%IncludeFont: Math1
/Math1 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
109.562 23.875 moveto
(\\200\\200\\200\\200) show
117.562 23.875 moveto
(\\200\\200\\200\\200) show
125.750 23.875 moveto
(\\200) show
111.125 35.062 moveto
%%IncludeResource: font Times-Italic-MISO
%%IncludeFont: Times-Italic-MISO
/Times-Italic-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(K) show
120.438 29.625 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 9.938 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(2) show
133.562 23.875 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
(terms) show
168.062 23.875 moveto
(dropped) show
213.875 23.875 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.600 0.600 0.600 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{595.938, 368.312},
  ImageMargins->{{0.25, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000=m0002:2000`400?l00000o`00003oo`3ooooo
0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon00?ooo`00
o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon0
0?ooo`00o`3ooooo0?ooool0ooooP03oool00?l0ooook03oool6000000030?ooo`000000000000@0
0000o`3ooon60?ooo`00o`3oooo^0?ooo`8000001@3oool200000?l0ooooR03oool00?l0ooookP3o
ool2000000D0oooo0P00003o0?ooohP0oooo003o0?ooonh0oooo0`0000030?ooo`800000o`3ooon9
0?ooo`00o`3oooo_0?ooo`8000000P3oool300000?l0ooooR@3oool00?l0ooook`3oool200000080
oooo0P00003o0?ooohX0oooo003o0?ooonl0oooo0P000000103oool0000000000000003o0?ooohX0
oooo003o0?ooonl0oooo1@00003o0?oooh/0oooo003o0?oooo00oooo1000003o0?oooh/0oooo003o
0?oooo00oooo0P0000020?ooo`030000003oool0oooo0?l0ooooR03oool00?l0ooool03oool20000
00<0oooo0P00003o0?ooohP0oooo003o0?oooo40oooo0P0000040?ooo`030000003oool0oooo0?l0
ooooQ@3oool00?l0ooool@3oool2000000D0oooo00<000000?ooo`3oool0o`3ooon40?ooo`00o`3o
ooo_0?ooo`H000000P3oool500000?l0ooooP`3oool00?l0ooooo`3ooooo0?oooh00oooo003o0?oo
ool0ooooo`3ooon00?ooo`00o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo
003o0?ooool0ooooo`3ooon00?ooo`00o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oo
oh00oooo003o0?ooool0ooooo`3ooon00?ooo`00U03oool6000000H0oooo0`00002J0?ooo`P00000
1P3oool3000009/0oooo1P0000080?ooo`<00000W`3oool2000000L0oooo0`00002J0?ooo`H00000
203oool3000002d0oooo002F0?ooo`8000001`3oool01@000000oooo0?ooo`3oool0000009X0oooo
200000040?ooo`050000003oool0oooo0?ooo`000000VP3oool2000000@0oooo00<000000?ooo`3o
ool0103oool01@000000oooo0?ooo`3oool0000009h0oooo0P0000060?ooo`050000003oool0oooo
0?ooo`000000V@3oool2000000<0oooo0`0000050?ooo`050000003oool0oooo0?ooo`000000;03o
ool009H0oooo0P0000060?ooo`030000003oool0oooo0080oooo0P00002J0?ooo`030000003oool0
oooo00@0oooo00@000000?ooo`3oool00000103oool200000:00oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`020?ooo`800000W@3oool2000000D0oooo00<000000?ooo`3oool00P3o
ool2000009h0oooo0P0000040?ooo`030000003oool0oooo0080oooo0P00000[0?ooo`00UP3oool2
000000H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`2N0?ooo`8000000`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo09/0oooo0P0000050?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0W@3oool2000000<0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0Y0?ooo`00UP3oool2000000D0oooo0P0000050?ooo`800000VP3oool00`00
0000oooo0?ooo`050?ooo`8000001@3oool2000009l0oooo0P0000020?ooo`8000001@3oool20000
09H0oooo2P0000020?ooo`8000001@3oool2000009h0oooo0P0000020?ooo`8000001@3oool20000
02X0oooo002F0?ooo`8000001@3oool2000000D0oooo0P00002K0?ooo`030000003oool0oooo00@0
oooo0P0000050?ooo`800000WP3oool300000080oooo0P0000050?ooo`800000U`3oool00`000000
oooo0?ooo`020?ooo`800000103oool2000000D0oooo0P00002N0?ooo`8000000P3oool2000000D0
oooo0P00000Z0?ooo`00UP3oool2000000D0oooo0P0000050?ooo`800000W03oool00`000000oooo
0?ooo`030?ooo`8000001@3oool2000009h0oooo0`0000020?ooo`8000001@3oool2000009L0oooo
00<000000?ooo`3oool00P3oool2000000@0oooo0P0000050?ooo`800000W@3oool300000080oooo
0P0000050?ooo`800000:P3oool009H0oooo0P0000050?ooo`8000001@3oool2000009d0oooo00<0
00000?ooo`3oool00P3oool2000000D0oooo0P00002J0?ooo`H000000`3oool2000000D0oooo0P00
002H0?ooo`040000003oool0oooo0?ooo`800000103oool2000000D0oooo0P00002L0?ooo`@00000
0P3oool2000000D0oooo0P00000Z0?ooo`00UP3oool2000000D0oooo0P0000050?ooo`800000W@3o
ool2000000<0oooo0P0000050?ooo`800000W03oool2000000D0oooo0P0000050?ooo`800000V@3o
ool00`000000oooo0?ooo`02000000@0oooo0P0000050?ooo`800000VP3oool5000000<0oooo0P00
00050?ooo`800000:P3oool009H0oooo0P0000050?ooo`8000001@3oool2000009d0oooo0P000003
0?ooo`8000001@3oool2000009d0oooo0P0000040?ooo`8000001@3oool2000009T0oooo00<00000
0?ooo`3oool00P0000040?ooo`8000001@3oool2000009P0oooo1P0000040?ooo`8000001@3oool2
000002X0oooo002F0?ooo`8000001@3oool2000000D0oooo0P00002G0?ooo`030000003oool0oooo
00<0oooo0P0000030?ooo`8000001@3oool2000009h0oooo0P0000030?ooo`8000001@3oool20000
09X0oooo00@000000?ooo`0000000000103oool2000000D0oooo0P00002H0?ooo`<000001`3oool2
000000D0oooo0P00000Z0?ooo`00UP3oool2000000H0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`2F0?ooo`030000003oool0oooo00<0oooo0P0000040?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`030?ooo`800000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo09X0oooo0`0000050?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0V03oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0:@3oool009<0oooo1@0000060?ooo`8000000`3oool2000009T0
oooo00@000000?ooo`3oool0oooo0`0000040?ooo`8000000`3oool2000009X0oooo00@000000?oo
o`3oool0oooo0`0000040?ooo`8000000`3oool2000009`0oooo0`0000050?ooo`8000000`3oool2
000009X0oooo1P0000040?ooo`8000000`3oool2000002/0oooo002F0?ooo`8000001`3oool50000
09/0oooo100000070?ooo`D00000W03oool5000000H0oooo1@00002N0?ooo`8000001P3oool50000
09`0oooo1@0000050?ooo`D00000;03oool00?l0ooooo`3ooooo0?ooodX0oooo00<000000?ooo`3o
ool0<`3oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon00?ooo`00o`3ooooo
0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon00?ooo`00
o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon0
0?ooo`00o`3ooooo0?ooool0ooooP03oool008/0ooooo`00003o00000=4000008`3oool008/0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0Z03oool00`000000oooo0?ooo`2X0?ooo`030000003oool0oooo0:P0oooo00<00000
0?ooo`3oool0Z03oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0240oooo002;0?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`2X0?oo
o`030000003oool0oooo0:P0oooo00<000000?ooo`3oool0Z03oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3oooo=0?ooo`03
0000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3oooni0?ooo`H00000
3P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool300000?l0ooooo`3ooonh0?ooo`P000002`3o
ool3000002<0oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonh0?ooo`T00000303oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0oooo^03oool90000
00`0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oookL0
oooo2P00000<0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3ooonh0?ooo`T00000303oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`3o0?ooool0oooo^03oool9000000`0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?oookP0oooo2000000=0?ooo`030000003oool0oooo0240oooo002;0?oo
o`030000003oool0oooo0?l0ooooo`3ooonh0?ooo`L000003P3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`3o0?ooool0oooo]`3oool40000000309VIV@000000VIVI00l0
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oookL0oooo
0`00000309VIVA00oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3o
oooo0?oookH0oooo1000000209VIVA40oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?oookD0oooo1000000209VIVA80oooo00<000000?ooo`3oool08@3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?oookD0oooo0`00000309VIVA80oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oook@0oooo0`00000309VIVA<0
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oook<0oooo
1000000209VIVA@0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3o
oooo0?oook80oooo1000000209VIVAD0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?oook80oooo0`00000309VIVAD0oooo00<000000?ooo`3oool08@3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?oook40oooo1000000209VIVAH0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oook00oooo1000000209VIVAL0
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oook00oooo
0`00000209VIVAP0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3o
oooo0?ooojl0oooo1000000209VIVAP0oooo00<000000?ooo`3oool08@3oool008/0oooo0`00003o
0?ooool0oooo[P3oool400000080VIVI5`3oool3000002<0oooo002;0?ooo`030000003oool0oooo
0?l0ooooo`3ooon^0?ooo`<000000P2IVITJ0?ooo`030000003oool0oooo0240oooo002;0?ooo`03
0000003oool0oooo0?l0ooooo`3ooon]0?ooo`@0000000<0VIVI0?ooo`3oool06@3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0oooo[03oool400000080VIVI
6`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0oooo[03o
ool300000080VIVI703oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o
0?ooool0ooooZ`3oool40000000309VIV@3oool0oooo01/0oooo00<000000?ooo`3oool08@3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?ooojX0oooo100000000`2IVIT0oooo0?ooo`0L0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonZ0?ooo`<0
00000P2IVITN0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3ooonY0?ooo`<000000P2IVITO0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0?l0ooooo`3ooonX0?ooo`@0000000<0VIVI0?ooo`3oool07P3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooY`3oool40000000309VIV@3oool0
oooo01l0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oo
ojL0oooo0`00000209VIVB40oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool0o`3ooooo0?ooojH0oooo100000000`2IVIT0oooo0?ooo`0P0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonU0?ooo`@0000000<0VIVI0?ooo`3oool0
8@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooY@3o
ool30000000309VIV@3oool0oooo0280oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?oooj@0oooo100000000`2IVIT0oooo0?ooo`0R0?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonS0?ooo`@0000000<0VIVI0?oo
o`3oool08`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0
ooooX`3oool30000000309VIV@3oool0oooo02@0oooo00<000000?ooo`3oool08@3oool005h0oooo
0`0000060?ooo`8000001@3oool3000000L0oooo1@00000>0?ooo`030000003oool0oooo0?l0oooo
o`3ooonR0?ooo`@000009`3oool00`000000oooo0?ooo`0Q0?ooo`00G@3oool01@000000oooo0?oo
o`3oool0000000D0oooo0P0000040?ooo`050000003oool0oooo0?ooo`0000001@3oool2000000@0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`3o0?ooool0ooooX@3oool400000003
09VIV@3oool0oooo02D0oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool0
0P3oool2000000T0oooo00<000000?ooo`3oool00P3oool2000000<0oooo0P0000050?ooo`800000
303oool00`000000oooo0?ooo`3o0?ooool0ooooX03oool40000000309VIV@3oool0oooo02H0oooo
00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`070?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0oooo0P0000050?ooo`800000
303oool00`000000oooo0?ooo`3o0?ooool0ooooX03oool30000000309VIV@3oool0oooo02L0oooo
00<000000?ooo`3oool08@3oool005/0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P00
00020?ooo`800000103oool3000000`0oooo1@00003o0?ooool0ooooW@3oool4000002H0oooo1@00
000S0?ooo`00C03oool<000000<0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P000003
0?ooo`040000003oool0oooo0?ooo`<000003@3oool00`000000oooo0?ooo`3o0?ooool0ooooWP3o
ool40000000309VIV@3oool0oooo02P0oooo00<000000?ooo`3oool08@3oool005/0oooo0P000005
0?ooo`8000001`3oool2000000D0oooo0P0000040?ooo`030000003oool00000008000003P3oool0
0`000000oooo0?ooo`3o0?ooool0ooooW@3oool40000000309VIV@3oool0oooo02T0oooo00<00000
0?ooo`3oool08@3oool005/0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P0000050?oo
o`<000003`3oool00`000000oooo0?ooo`3o0?ooool0ooooW@3oool30000000309VIV@3oool0oooo
02X0oooo00<000000?ooo`3oool08@3oool005/0oooo0P0000050?ooo`8000001`3oool2000000D0
oooo0P0000040?ooo`@000003`3oool00`000000oooo0?ooo`3o0?ooool0ooooW03oool4000002d0
oooo00<000000?ooo`3oool08@3oool005/0oooo0P0000050?ooo`8000001`3oool2000000D0oooo
0P0000030?ooo`8000000`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0oooo
o`3ooonK0?ooo`@0000000<0VIVI0?ooo`3oool0:`3oool00`000000oooo0?ooo`0Q0?ooo`00F`3o
ool2000000D0oooo0P0000070?ooo`8000001@3oool200000080oooo0P0000040?ooo`8000003@3o
ool00`000000oooo0?ooo`3o0?ooool0ooooVP3oool40000000309VIV@3oool0oooo02`0oooo00<0
00000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00<0oooo00@000000?ooo`3oool0oooo0P0000040?ooo`8000003@3o
ool00`000000oooo0?ooo`3o0?ooool0ooooV@3oool40000000309VIV@3oool0oooo02d0oooo00<0
00000?ooo`3oool08@3oool005`0oooo0P0000030?ooo`8000002@3oool2000000<0oooo0P000003
0?ooo`800000103oool2000000d0oooo00<000000?ooo`3oool0o`3ooooo0?oooiT0oooo0`000000
0`2IVIT0oooo0?ooo`0^0?ooo`030000003oool0oooo0240oooo001M0?ooo`D000002`3oool50000
00H0oooo1@00000>0?ooo`030000003oool0oooo0?l0ooooo`3ooonH0?ooo`@0000000<0VIVI0?oo
o`3oool0;P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0
ooooU`3oool40000000309VIV@3oool0oooo02l0oooo00<000000?ooo`3oool08@3oool008/0oooo
00<000000?ooo`3oool0o`3ooooo0?oooiH0oooo100000000`2IVIT0oooo0?ooo`0`0?ooo`030000
003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonF0?ooo`<000000P2I
VITb0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooonE
0?ooo`@0000000<0VIVI0?ooo`3oool0<@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`3o0?ooool0ooooU03oool40000000309VIV@3oool0oooo0380oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oooi<0oooo100000000`2IVIT0
oooo0?ooo`0c0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3ooonC0?ooo`<000000P2IVITe0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0?l0ooooo`3ooonB0?ooo`@0000000<0VIVI0?ooo`3oool0=03oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooT@3oool40000000309VIV@3oool0
oooo03D0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oo
oi00oooo100000000`2IVIT0oooo0?ooo`0f0?ooo`030000003oool0oooo0240oooo002;0?ooo`03
0000003oool0oooo0?l0ooooo`3ooon@0?ooo`<000000P2IVITh0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooon?0?ooo`@0000000<0VIVI0?ooo`3oool0
=`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool300000?l0ooooo`3ooon>0?ooo`@0000000<0
VIVI0?ooo`3oool0=P3oool3000002<0oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooon=
0?ooo`@0000000<0VIVI0?ooo`3oool0>@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`3o0?ooool0ooooS03oool400000080VIVI>`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooS03oool300000080VIVI?03oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooR`3oool40000000309VI
V@3oool0oooo03/0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3o
oooo0?ooohX0oooo1000000209VIVCd0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?ooohT0oooo1000000209VIVCh0oooo00<000000?ooo`3oool08@3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?ooohT0oooo0`00000209VIVCl0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?ooohP0oooo100000000`2IVIT0
oooo0?ooo`0n0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3ooon70?ooo`@000000P2IVIU00?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0?l0ooooo`3ooon60?ooo`@000000P2IVIU10?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo0?l0ooooo`3ooon60?ooo`<000000P2IVIU20?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooon50?ooo`@0000000<0VIVI0?oo
o`3oool0@@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0
ooooQ03oool400000080VIVI@`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`3o0?ooool0ooooP`3oool400000080VIVIA03oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooP`3oool300000080VIVIA@3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooPP3oool400000080VIVIA@3oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooP@3oool40000
0080VIVIAP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0
ooooP03oool400000080VIVIA`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`3o0?ooool0ooooP03oool300000080VIVIB03oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooO`3oool400000080VIVIB03oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool300000?l0ooooo`3ooomn0?ooo`@000000P2IVIU70?ooo`<000008`3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?ooogd0oooo1000000209VIVDX0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?ooogd0oooo0`00000209VIVD/0
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?ooog`0oooo
100000000`2IVIT0oooo0?ooo`1:0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0?l0ooooo`3ooomk0?ooo`@000000P2IVIU<0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo0?l0ooooo`3ooomj0?ooo`@000000P2IVIU=0?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooomj0?ooo`<000000P2IVIU>0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooomi0?ooo`@0
000000<0VIVI0?ooo`3oool0C@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`3o0?ooool0ooooN03oool400000080VIVIC`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooN03oool300000080VIVID03oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooM`3oool300000080VIVID@3oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooMP3oool40000
000309VIV@3oool0oooo0500oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool0o`3ooooo0?ooogD0oooo100000000`2IVIT0oooo0?ooo`1A0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooome0?ooo`<000000P2IVIUC0?ooo`030000
003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooomd0?ooo`@0000000<0
VIVI0?ooo`3oool0DP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o
0?ooool0ooooL`3oool40000000309VIV@3oool0oooo05<0oooo00<000000?ooo`3oool08@3oool0
08/0oooo00<000000?ooo`3oool0o`3ooooo0?ooog80oooo100000000`2IVIT0oooo0?ooo`1D0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooomb0?ooo`<0
00000P2IVIUF0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3oooma0?ooo`@0000000<0VIVI0?ooo`3oool0E@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooL03oool40000000309VIV@3oool0oooo05H0oooo00<0
00000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?ooofl0oooo10000000
0`2IVIT0oooo0?ooo`1G0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo
0?l0ooooo`3ooom_0?ooo`<0000000<0VIVI0?ooo`3oool0F03oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool300000?l0ooooo`3ooom^0?ooo`@0000000<0VIVI0?ooo`3oool0EP3oool3000002<0
oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooom]0?ooo`@0000000<0VIVI0?ooo`3oool0
F@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooK03o
ool40000000309VIV@3oool0oooo05X0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool0o`3ooooo0?ooof`0oooo0`0000000`2IVIT0oooo0?ooo`1K0?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0?l0ooooo`3ooom[0?ooo`@00000GP3oool00`00
0000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooJP3oool400000003
09VIV@3oool0oooo05`0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0
o`3ooooo0?ooofT0oooo100000000`2IVIT0oooo0?ooo`1M0?ooo`030000003oool0oooo0240oooo
002;0?ooo`030000003oool0oooo0?l0ooooo`3ooomY0?ooo`<0000000<0VIVI0?ooo`3oool0GP3o
ool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooJ03oool4
00000640oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?oo
ofL0oooo100000000`2IVIT0oooo0?ooo`1O0?ooo`030000003oool0oooo0240oooo002;0?ooo`03
0000003oool0oooo0?l0ooooo`3ooomV0?ooo`@0000000<0VIVI0?ooo`3oool0H03oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooIP3oool30000000309VI
V@3oool0oooo0640oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3o
oooo0?ooofD0oooo1000001T0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0
oooo0?l0ooooo`3ooomT0?ooo`@00000I@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`3o0?ooool0ooooH`3oool40000000309VIV@3oool0oooo06<0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0o`3ooooo0?ooof80oooo100000000`2IVIT0
oooo0?ooo`1T0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0?l0oooo
o`3ooomR0?ooo`<0000000<0VIVI0?ooo`3oool0I@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooH@3oool4000006P0oooo00<000000?ooo`3oool08@3o
ool005h0oooo0`0000060?ooo`8000001@3oool3000000L0oooo1@00000>0?ooo`030000003oool0
oooo0?l0ooooo`3ooomP0?ooo`@0000000<0VIVI0?ooo`3oool0IP3oool00`000000oooo0?ooo`0Q
0?ooo`00G@3oool01@000000oooo0?ooo`3oool0000000D0oooo0P0000040?ooo`050000003oool0
oooo0?ooo`0000001@3oool2000000<0oooo0P00000=0?ooo`030000003oool0oooo0?l0ooooo`3o
oomO0?ooo`@0000000<0VIVI0?ooo`3oool0I`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool0
0`000000oooo0?ooo`020?ooo`8000002@3oool00`000000oooo0?ooo`020?ooo`800000103oool2
000000@0oooo0P00000<0?ooo`030000003oool0oooo0?l0ooooo`3ooomO0?ooo`<0000000<0VIVI
0?ooo`3oool0J03oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`3oool2
000000D0oooo0P00000<0?ooo`030000003oool0oooo0?l0ooooo`3ooomN0?ooo`@00000J`3oool0
0`000000oooo0?ooo`0Q0?ooo`00F`3oool2000000D0oooo0P0000070?ooo`8000001@3oool20000
0080oooo0P0000050?ooo`800000303oool500000?l0ooooo`3ooomK0?ooo`@0000000<0VIVI0?oo
o`3oool0I@3oool5000002<0oooo001<0?ooo``000000`3oool2000000D0oooo0P0000070?ooo`80
00001@3oool200000080oooo0P0000050?ooo`800000303oool00`000000oooo0?ooo`3o0?ooool0
ooooG03oool40000000309VIV@3oool0oooo06X0oooo00<000000?ooo`3oool08@3oool005/0oooo
0P0000050?ooo`8000001`3oool2000000D0oooo0P0000020?ooo`8000001@3oool2000000`0oooo
00<000000?ooo`3oool0o`3ooooo0?oooe/0oooo100000000`2IVIT0oooo0?ooo`1[0?ooo`030000
003oool0oooo0240oooo001K0?ooo`8000001@3oool2000000L0oooo0P0000050?ooo`8000000P3o
ool2000000@0oooo0P00000=0?ooo`030000003oool0oooo00d0oooo0`2IVIWo0?ooool0ooooB`3o
ool4000006h0oooo00<000000?ooo`3oool08@3oool005/0oooo0P0000050?ooo`8000001`3oool2
000000D0oooo0P0000020?ooo`L000003P3oool00`000000oooo0?ooo`0=0?ooo`<0VIVIo`3ooooo
0?ooodX0oooo1000001_0?ooo`030000003oool0oooo0240oooo001K0?ooo`8000001@3oool20000
00L0oooo0P0000050?ooo`8000000`3oool200000180oooo00<000000?ooo`3oool03@3oool309VI
VOl0ooooo`3ooom90?ooo`@0000000<0VIVI0?ooo`3oool0K@3oool00`000000oooo0?ooo`0Q0?oo
o`00F`3oool2000000D0oooo0P0000070?ooo`8000001@3oool2000000<0oooo0P00000B0?ooo`03
0000003oool0oooo00h0oooo0`2IVIWo0?ooool0ooooA`3oool40000000309VIV@3oool0oooo06h0
oooo00<000000?ooo`3oool08@3oool005`0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00`3oool200000140oooo
00<000000?ooo`3oool03P3oool309VIVOl0ooooo`3ooom70?ooo`<0000000<0VIVI0?ooo`3oool0
K`3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool2000000<0oooo0P0000090?ooo`8000000`3o
ool2000000H0oooo0P00000@0?ooo`030000003oool0oooo00h0oooo0`2IVIWo0?ooool0ooooAP3o
ool400000780oooo00<000000?ooo`3oool08@3oool005d0oooo1@00000;0?ooo`D00000203oool2
000000l0oooo00<000000?ooo`3oool03`3oool309VIVOl0ooooo`3ooom40?ooo`@0000000<0VIVI
0?ooo`3oool0L03oool00`000000oooo0?ooo`0Q0?ooo`00O03oool2000000d0oooo00<000000?oo
o`3oool03`3oool309VIVOl0ooooo`3ooom30?ooo`@0000000<0VIVI0?ooo`3oool0L@3oool00`00
0000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0?0?ooo`<0VIVIo`3ooooo0?oood<0
oooo0`0000000`2IVIT0oooo0?ooo`1b0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000
003oool0oooo0100oooo0`2IVIWo0?ooool0oooo@@3oool4000007D0oooo00<000000?ooo`3oool0
8@3oool008/0oooo00<000000?ooo`3oool0403oool309VIVOl0ooooo`3ooom00?ooo`@0000000<0
VIVI0?ooo`3oool0L`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0@
0?ooo`<0VIVIo`3ooooo0?ooocl0oooo100000000`2IVIT0oooo0?ooo`1d0?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0140oooo0`2IVIWo0?ooool0oooo?@3oool40000
000309VIV@3oool0oooo07D0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool04@3oool309VIVOl0ooooo`3ooolm0?ooo`<0000000<0VIVI0?ooo`3oool0MP3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0A0?ooo`<0VIVIo`3ooooo0?oooc`0oooo
100000000`2IVIT0oooo0?ooo`1f0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0180oooo0`2IVIWo0?ooool0oooo>P3oool40000000309VIV@3oool0oooo07L0oooo00<0
00000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool04P3oool309VIVOl0ooooo`3oooli
0?ooo`@0000000<0VIVI0?ooo`3oool0N03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`0B0?ooo`<0VIVIo`3ooooo0?ooocT0oooo0`0000000`2IVIT0oooo0?ooo`1i0?oo
o`030000003oool0oooo0240oooo002;0?ooo`<000004`3oool309VIVFD0oooo1P2IVIT00`3oool0
VIVI09VIV@0409VIVOl0ooooa03oool4000007X0oooo0`00000S0?ooo`00R`3oool00`000000oooo
0?ooo`0C0?ooo`<0VIVII`3oool209VIV@D0oooo0P2IVIWo0?ooolD0oooo100000000`2IVIT0oooo
0?ooo`1j0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo01<0oooo0`2I
VIUW0?ooo`80VIVI1@3oool209VIVOl0ooooa03oool40000000309VIV@3oool0oooo07/0oooo00<0
00000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0503oool309VIVB40oooo1P2IVIT0
0`3oool0VIVI09VIV@0409VIVCP0oooo0`2IVIT30?ooo`80VIVIo`3oooo40?ooo`@0000000<0VIVI
0?ooo`3oool0O03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0D0?oo
o`<0VIVI8`3oool209VIV@D0oooo0P2IVITk0?ooo`80VIVI0P3oool309VIVOl0oooo``3oool40000
000309VIV@3oool0oooo07d0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool0503oool409VIVB80oooo0P2IVIT50?ooo`80VIVI>`3oool209VIV@80oooo0P2IVIWo0?oool@0
oooo0`00000209VIVGl0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0
5@3oool309VIVB80oooo0`2IVIT30?ooo`80VIVI?03oool209VIV@040?ooo`2IVIT0VIVI09VIVOl0
oooo``3oool40000000309VIV@3oool0oooo07h0oooo00<000000?ooo`3oool08@3oool008/0oooo
00<000000?ooo`3oool05@3oool309VIVB<0oooo0P2IVIT20?ooo`<0VIVI?03oool509VIVOl0oooo
``3oool40000000309VIV@3oool0oooo07l0oooo00<000000?ooo`3oool08@3oool008/0oooo00<0
00000?ooo`3oool05@3oool409VIVB80oooo0P2IVIT20?ooo`80VIVI?P3oool409VIVOl0oooo`P3o
ool40000000309VIV@3oool0oooo0800oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool05P3oool309VIVB80oooo0P2IVIT0103oool0VIVI09VIV@2IVITn0?ooo`80VIVI0P3o
ool00`2IVIT0oooo0?ooo`040?ooo`D0VIVIo`3ooone0?ooo`@000000P2IVIV20?ooo`030000003o
ool0oooo0240oooo002;0?ooo`030000003oool0oooo01H0oooo0`2IVITR0?ooo`D0VIVI?`3oool2
09VIV@<0oooo0P2IVIT50?ooo`0409VIV@3oool0oooo0?ooo`80VIVIo`3ooonb0?ooo`@000000P2I
VIV30?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo01H0oooo102IVITR
0?ooo`@0VIVI@03oool209VIV@@0oooo00<0VIVI0?ooo`3oool00P3oool00`2IVIT0oooo0?ooo`3o
0?oookD0oooo100000000`2IVIT0oooo0?ooo`220?ooo`030000003oool0oooo0240oooo002;0?oo
o`030000003oool0oooo01L0oooo0`2IVITR0?ooo`80VIVI0P3oool00`2IVIT0oooo0?ooo`0m0?oo
o`80VIVI1@3oool00`2IVIT0oooo0?ooo`020?ooo`0309VIV@3oool0oooo05l0oooo1@2IVIT60?oo
o`D0VIVIo`3ooom40?ooo`@0000000<0VIVI0?ooo`3oool0P`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0G0?ooo`<0VIVI8P3oool209VIV@<0oooo0P2IVITk0?ooo`H0
VIVI0P3oool509VIV@80oooo00<0VIVI0?ooo`3oool0H03oool209VIV@T0oooo0P2IVIWo0?oood@0
oooo100000000`2IVIT0oooo0?ooo`240?ooo`030000003oool0oooo0240oooo002;0?ooo`030000
003oool0oooo01L0oooo102IVITR0?ooo`80VIVI103oool00`2IVIT0oooo0?ooo`180?ooo`0309VI
V@3oool0oooo05l0oooo0P2IVIT90?ooo`80VIVIo`3ooom30?ooo`@0000000<0VIVI0?ooo`3oool0
Q@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0H0?ooo`<0VIVI8P3o
ool209VIV@D0oooo00<0VIVI0?ooo`3oool0A`3oool00`2IVIT0oooo0?ooo`1O0?ooo`80VIVI2@3o
ool209VIVOl0oooo@P3oool400000080VIVIQ`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0H0?ooo`<0VIVI803oool609VIV@80oooo1@2IVIU20?ooo`0509VIV@3oool0
oooo0?ooo`2IVIT0H@3oool209VIV@T0oooo0P2IVIWo0?oood40oooo1000000209VIVHP0oooo00<0
00000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0603oool409VIVCP0oooo0`2IVIT2
0?ooo`80VIVI00<0oooo09VIV@2IVIT00`2IVIT00`3oool0VIVI09VIV@0309VIV@<0oooo0`2IVIT0
0`3oool0VIVI09VIV@0J0?ooo`0509VIV@3oool0oooo0?ooo`2IVIT0403oool309VIV@@0oooo102I
VIT30?ooo`D0VIVI0`3oool509VIV@030?ooo`2IVIT0VIVI0080VIVI00<0oooo09VIV@2IVIT02@2I
VIT90?ooo`<0VIVI00<0oooo09VIV@2IVIT00P3oool509VIV@D0oooo1@2IVIT50?ooo`H0VIVI1@3o
ool609VIV@H0oooo102IVIT60?ooo`<0VIVI00<0oooo09VIV@2IVIT0o`3ooolW0?ooo`@0000000<0
VIVI0?ooo`3oool0Q`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0I
0?ooo`<0VIVI=`3oool209VIV@80oooo0`2IVIT40?ooo`80VIVI0`3oool209VIV@@0oooo2@2IVITI
0?ooo`<0VIVI403oool209VIV@80oooo1@2IVIT30?ooo`0309VIV@3oool0oooo0080oooo0P2IVIT5
0?ooo`80VIVI103oool209VIV@@0oooo0P2IVIT20?ooo`80VIVI0`3oool209VIV@H0oooo2@2IVIT2
0?ooo`80VIVI1@3oool209VIV@<0oooo0P2IVIT40?ooo`80VIVI0`3oool209VIV@@0oooo0P2IVIT3
0?ooo`80VIVI0`3oool309VIV@<0oooo00<0VIVI0?ooo`3oool00P3oool909VIVOl0oooo903oool4
0000000309VIV@3oool0oooo08P0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?oo
o`3oool06@3oool309VIVCL0oooo0P2IVIT30?ooo`80VIVI103oool209VIV@<0oooo0P2IVIT30?oo
o`<0VIVI0`3oool209VIVBh0oooo0P2IVIT40?ooo`80VIVI1@3oool0102IVIT0oooo0?ooo`3oool2
09VIV@D0oooo0P2IVIT40?ooo`80VIVI103oool209VIV@80oooo00<0VIVI0?ooo`3oool00P3oool2
09VIV@D0oooo0`2IVIT30?ooo`80VIVI103oool209VIV@@0oooo0P2IVIT50?ooo`0309VIV@3oool0
oooo0080oooo0P2IVIT40?ooo`0309VIV@3oool0oooo0080oooo0P2IVIT40?ooo`0409VIV@3oool0
oooo0?ooo`80VIVI1@3oool00`2IVIT0oooo0?ooo`0309VIV@<0oooo0P2IVIWo0?ooobD0oooo1000
00000`2IVIT0oooo0?ooo`290?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0
oooo01T0oooo102IVITf0?ooo`80VIVI0`3oool209VIV@@0oooo0P2IVIT30?ooo`80VIVI0`3oool2
09VIV@@0oooo0P2IVIT^0?ooo`80VIVI103oool209VIV@D0oooo00@0VIVI0?ooo`3oool0oooo0P2I
VIT50?ooo`80VIVI103oool209VIV@@0oooo0P2IVIT60?ooo`<0VIVI1@3oool209VIV@@0oooo0P2I
VIT40?ooo`80VIVI103oool209VIV@D0oooo0P2IVIT30?ooo`80VIVI103oool209VIV@<0oooo0P2I
VIT40?ooo`80VIVI0P3oool209VIV@D0oooo00<0VIVI0?ooo`3oool00P2IVIT40?ooo`80VIVIo`3o
oolT0?ooo`@0000000<0VIVI0?ooo`3oool0RP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0J0?ooo`<0VIVI=`3oool209VIV@80oooo0P2IVIT40?ooo`80VIVI0`3oool2
09VIV@<0oooo0P2IVIT40?ooo`80VIVI;P3oool209VIV@@0oooo00<0VIVI0?ooo`3oool0203oool2
09VIV@D0oooo0P2IVIT40?ooo`80VIVI103oool209VIV@@0oooo102IVIT60?ooo`80VIVI103oool2
09VIV@@0oooo0P2IVIT40?ooo`80VIVI1@3oool209VIV@<0oooo0P2IVIT40?ooo`80VIVI0`3oool2
09VIV@@0oooo0P2IVIT20?ooo`0309VIV@3oool0oooo00L0oooo0P2IVIT40?ooo`80VIVIo`3ooolS
0?ooo`@0000000<0VIVI0?ooo`3oool0R`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool30000
01X0oooo0`2IVITI0?oooaD0VIVI2`3oool409VIV@@0oooo0P2IVIT30?ooo`80VIVI0`3oool209VI
V@@0oooo0P2IVIT90?oooa`0VIVI2@3oool209VIV@@0oooo00<0VIVI0?ooo`3oool0203oool209VI
V@D0oooo0P2IVIT40?ooo`80VIVI103oool209VIV@<0oooo0`2IVIT80?ooo`80VIVI103oool209VI
V@@0oooo0P2IVIT40?ooo`80VIVI1@3oool209VIV@<0oooo0P2IVIT40?ooo`80VIVI0`3oool209VI
V@@0oooo0P2IVIT20?ooo`0309VIV@3oool0oooo00L0oooo0P2IVIT40?ooo`80VIVIo`3ooolR0?oo
o`@000000P2IVIV;0?ooo`<000008`3oool008/0oooo00<000000?ooo`3oool06P3oool409VIVCX0
oooo0P2IVIT40?ooo`80VIVI0`3oool209VIV@<0oooo0P2IVIT40?ooo`80VIVI;P3oool209VIV@@0
oooo202IVIT30?ooo`80VIVI1@3oool209VIV@@0oooo0P2IVIT40?ooo`80VIVI0P3oool309VIV@T0
oooo0P2IVIT40?ooo`80VIVI103oool209VIV@@0oooo0P2IVIT50?ooo`80VIVI0`3oool209VIV@@0
oooo0P2IVIT30?ooo`80VIVI103oool209VIV@80oooo202IVIT20?ooo`80VIVI103oool209VIVOl0
oooo8P3oool40000000309VIV@3oool0oooo08`0oooo00<000000?ooo`3oool08@3oool008/0oooo
00<000000?ooo`3oool06`3oool309VIVCD0oooo0P2IVIT30?ooo`80VIVI103oool209VIV@<0oooo
0P2IVIT40?ooo`0309VIV@3oool0oooo0080oooo0P2IVIT^0?ooo`80VIVI103oool00`2IVIT0oooo
0?ooo`020?ooo`<0VIVI0`3oool209VIV@D0oooo0P2IVIT40?ooo`80VIVI103oool209VIV@80oooo
0P2IVIT30?ooo`0309VIV@3oool0oooo00D0oooo00<0VIVI0?ooo`3oool00P3oool209VIV@@0oooo
0P2IVIT50?ooo`0309VIV@3oool0oooo0080oooo0P2IVIT40?ooo`80VIVI103oool209VIV@<0oooo
0P2IVIT40?ooo`80VIVI0P3oool00`2IVIT0oooo0?ooo`020?ooo`<0VIVI0`3oool00`2IVIT0oooo
0?ooo`020?ooo`80VIVIo`3ooolQ0?ooo`@0000000<0VIVI0?ooo`3oool0S@3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0K0?ooo`<0VIVI=@3oool209VIV@<0oooo0P2I
VIT40?ooo`<0VIVI0P3oool209VIV@@0oooo0P2IVIT30?ooo`80VIVI;P3oool209VIV@D0oooo00@0
VIVI0?ooo`3oool0oooo0P2IVIT30?ooo`H0VIVI0P3oool409VIV@050?ooo`2IVIT0VIVI0?ooo`2I
VIT00`3oool0102IVIT0oooo0?ooo`3oool209VIV@<0oooo00<0VIVI0?ooo`3oool01@3oool209VI
V@<0oooo0P2IVIT30?ooo`H0VIVI0P3oool209VIV@<0oooo0P2IVIT30?ooo`@0VIVI0P3oool209VI
V@<0oooo102IVIT20?ooo`80VIVI103oool0102IVIT0oooo0?ooo`3oool209VIV@@0oooo0P2IVIT3
0?ooo`80VIVIo`3ooolP0?ooo`@0000000<0VIVI0?ooo`3oool0SP3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`0K0?ooo`@0VIVI=P3oool409VIV@D0oooo0P2IVIT0103o
ool0VIVI09VIV@2IVIT60?ooo`H0VIVI;@3oool509VIV@@0oooo102IVIT50?ooo`80VIVI00<0oooo
09VIV@2IVIT00P3oool209VIV@80oooo0`2IVIT20?ooo`@0VIVI103oool509VIV@P0oooo1P2IVIT4
0?ooo`80VIVI00<0oooo09VIV@2IVIT00`3oool509VIV@D0oooo0P2IVIT0103oool0VIVI09VIV@2I
VIT50?ooo`80VIVI00@0oooo09VIV@2IVIT0VIVI1P3oool409VIV@H0oooo1P2IVIWo0?oooal0oooo
100000000`2IVIT0oooo0?ooo`2?0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo01`0oooo0`2IVIU?0?ooo`80VIVI;P3oool209VIVCP0oooo0P2IVITk0?ooo`80VIVIo`3o
oolN0?ooo`@0000000<0VIVI0?ooo`3oool0T03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0L0?ooo`<0VIVIC`3oool209VIVBl0oooo00<0VIVI0?ooo`3oool0=P3oool2
09VIVC/0oooo0P2IVIWo0?oooah0oooo0`0000000`2IVIT0oooo0?ooo`2A0?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo01`0oooo102IVIU>0?ooo`80VIVI;`3oool00`2I
VIT0oooo0?ooo`0f0?ooo`80VIVI>`3oool209VIVOl0oooo7@3oool40000000309VIV@3oool0oooo
0940oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool07@3oool309VIVDh0
oooo0P2IVIUX0?ooo`80VIVI>`3oool209VIVOl0oooo703oool40000000309VIV@3oool0oooo0980
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool07@3oool309VIVAh0oooo
1P2IVIT[0?ooo`0309VIV@3oool0oooo0180oooo1P2IVIU?0?ooo`0309VIV@3oool0oooo03X0oooo
00<0VIVI0?ooo`3oool0o`3ooolI0?ooo`@0000000<0VIVI0?ooo`3oool0T`3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0M0?ooo`@0VIVI7`3oool209VIVD@0oooo0P2I
VIWo0?ooojT0oooo100000000`2IVIT0oooo0?ooo`2D0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo01h0oooo0`2IVITO0?ooo`80VIVIA03oool209VIVOl0ooooZ03oool4
0000000309VIV@3oool0oooo09D0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?oo
o`3oool07P3oool309VIVAl0oooo0P2IVIU40?ooo`80VIVIo`3ooonW0?ooo`D00000V03oool00`00
0000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0N0?ooo`@0VIVI7P3oool209VIVD@0
oooo0P2IVIWo0?ooojL0oooo100000000`2IVIT0oooo0?ooo`2F0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo01l0oooo0`2IVITN0?ooo`80VIVIA03oool209VIVOl0oooo
YP3oool40000000309VIV@3oool0oooo09L0oooo00<000000?ooo`3oool08@3oool008/0oooo00<0
00000?ooo`3oool07`3oool309VIVAh0oooo0P2IVIU40?ooo`80VIVIo`3ooonU0?ooo`@0000000<0
VIVI0?ooo`3oool0V03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0O
0?ooo`@0VIVI7@3oool209VIVD@0oooo0P2IVIWo0?oooj@0oooo100000000`2IVIT0oooo0?ooo`2I
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0200oooo0`2IVITM0?oo
o`80VIVIA03oool209VIVOl0ooooX`3oool40000000309VIV@3oool0oooo09X0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0803oool309VIVAd0oooo0P2IVIU40?ooo`80
VIVIo`3ooonR0?ooo`@0000000<0VIVI0?ooo`3oool0V`3oool00`000000oooo0?ooo`0Q0?ooo`00
R`3oool00`000000oooo0?ooo`0P0?ooo`@0VIVI703oool209VIVD@0oooo0P2IVIWo0?oooj40oooo
100000000`2IVIT0oooo0?ooo`2L0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0240oooo0`2IVITL0?ooo`80VIVIA03oool209VIVOl0ooooX03oool400000080VIVIWP3o
ool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0Q0?ooo`<0VIVI6@3oool5
09VIVD40oooo1@2IVIWo0?oooj00oooo100000000`2IVIT0oooo0?ooo`2M0?ooo`030000003oool0
oooo0240oooo002;0?ooo`<000008@3oool409VIVA/0oooo0P2IVIU40?ooo`80VIVIo`3ooonO0?oo
o`@0000000<0VIVI0?ooo`3oool0W03oool3000002<0oooo002;0?ooo`030000003oool0oooo0280
oooo0`2IVIWo0?ooool0oooo0P3oool40000000309VIV@3oool0oooo09l0oooo00<000000?ooo`3o
ool08@3oool008/0oooo00<000000?ooo`3oool08P3oool309VIVOl0ooooo`3oool10?ooo`@00000
00<0VIVI0?ooo`3oool0X03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`0S0?ooo`<0VIVIo`3oooon0?ooo`@0000000<0VIVI0?ooo`3oool0X@3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0S0?ooo`<0VIVIo`3oooom0?ooo`@0000000<0VIVI
0?ooo`3oool0XP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0S0?oo
o`<0VIVIo`3ooool0?ooo`@0000000<0VIVI0?ooo`3oool0X`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0T0?ooo`<0VIVIo`3ooooj0?ooo`@000000P2IVIVU0?ooo`03
0000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo02@0oooo0`2IVIWo0?ooooT0oooo
1@0000000`2IVIT0oooo0?ooo`2T0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo02@0oooo0`2IVIWo0?ooooT0oooo100000000`2IVIT0oooo0?ooo`2U0?ooo`030000003o
ool0oooo0240oooo002;0?ooo`030000003oool0oooo02D0oooo0`2IVIWo0?ooooL0oooo10000000
0`2IVIT0oooo0?ooo`2V0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo
02D0oooo0`2IVIWo0?ooooH0oooo100000000`2IVIT0oooo0?ooo`2W0?ooo`030000003oool0oooo
0240oooo002;0?ooo`030000003oool0oooo02D0oooo0`2IVIWo0?ooooD0oooo100000000`2IVIT0
oooo0?ooo`2X0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo02H0oooo
0`2IVIWo0?oooo<0oooo100000000`2IVIT0oooo0?ooo`2Y0?ooo`030000003oool0oooo0240oooo
002;0?ooo`030000003oool0oooo02H0oooo0`2IVIWo0?oooo80oooo100000000`2IVIT0oooo0?oo
o`2Z0?ooo`030000003oool0oooo0240oooo000V0?ooo`@000000P3oool010000000oooo0?ooo`3o
ool6000005D0oooo00<000000?ooo`3oool09P3oool309VIVOl0ooool@3oool400000080VIVI[03o
ool00`000000oooo0?ooo`0Q0?ooo`009`3oool00`000000oooo0?ooo`020?ooo`800000103oool2
000005L0oooo00<000000?ooo`3oool09`3oool309VIVOl0ooook`3oool400000080VIVI[@3oool0
0`000000oooo0?ooo`0Q0?ooo`00:03oool010000000oooo0?ooo`3oool3000000<0oooo0`00001F
0?ooo`030000003oool0oooo02L0oooo0`2IVIWo0?ooonl0oooo100000000`2IVIT0oooo0?ooo`2/
0?ooo`030000003oool0oooo0240oooo000X0?ooo`040000003oool0oooo0?ooo`<00000103oool2
000002T0oooo0`0000060?ooo`8000001@3oool3000000X0oooo0P00000>0?ooo`030000003oool0
oooo02L0oooo0`2IVIWo0?ooonh0oooo100000000`2IVIT0oooo0?ooo`2]0?ooo`030000003oool0
oooo0240oooo000X0?ooo`040000003oool0oooo0?ooo`80000000<0oooo0000003oool00P3oool2
000002P0oooo00D000000?ooo`3oool0oooo000000050?ooo`800000103oool01@000000oooo0?oo
o`3oool0000000T0oooo0P00000>0?ooo`030000003oool0oooo02P0oooo0`2IVIWo0?ooon`0oooo
100000000`2IVIT0oooo0?ooo`2^0?ooo`030000003oool0oooo0240oooo000X0?ooo`040000003o
ool0oooo0?ooo`80000000<0oooo0000000000000P3oool2000002L0oooo00<000000?ooo`3oool0
0P3oool2000000T0oooo00<000000?ooo`3oool00P3oool2000000P0oooo0P00000>0?ooo`030000
003oool0oooo02P0oooo0`2IVIWo0?ooon/0oooo100000000`2IVIT0oooo0?ooo`2_0?ooo`030000
003oool0oooo0240oooo000Y0?ooo`030000003oool0oooo008000000P3oool00`000000oooo0?oo
o`03000002H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool01P3oool2000000h0oooo00<000000?ooo`3oool0:03o
ool309VIVOl0oooojP3oool40000000309VIV@3oool0oooo0;00oooo00<000000?ooo`3oool08@3o
ool002T0oooo00<000000?ooo`3oool00P0000030?ooo`030000003oool0oooo008000009@3oool2
000000D0oooo0P0000070?ooo`8000001@3oool2000000030?ooo`000000000000P00000303oool5
000002L0oooo0`2IVIWo0?ooonP0oooo100000000`2IVIT0oooo0?ooo`2]0?ooo`D000008`3oool0
02T0oooo00@000000?ooo`3oool000001@3oool010000000oooo0000000000030?ooo`D000003P3o
ool<000000<0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P0000020?ooo`030000003o
ool0oooo0080oooo0P00000>0?ooo`030000003oool0oooo02T0oooo0`2IVIWo0?ooonL0oooo1000
000209VIVK<0oooo00<000000?ooo`3oool08@3oool002X0oooo0`0000050?ooo`040000003oool0
0000000000@0oooo00@000000?ooo`3oool0oooo0P00000K0?ooo`8000001@3oool2000000L0oooo
0P0000050?ooo`8000000P3oool00`000000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo
0?ooo`0Y0?ooo`@0VIVIo`3ooooP0?ooo`T000000P2IVIVd0?ooo`030000003oool0oooo0240oooo
000Z0?ooo`<000001P3oool010000000oooo0000000000030?ooo`030000003oool0oooo01h0oooo
0P0000050?ooo`8000001`3oool2000000D0oooo0P0000030?ooo`040000003oool0oooo0?ooo`80
00003P3oool00`000000oooo0?ooo`0Z0?ooo`<0VIVIo`3ooooO0?ooo`X0000000<0VIVI0?ooo`3o
ool0/`3oool00`000000oooo0?ooo`0Q0?ooo`00:P3oool3000000L0oooo0`0000040?ooo`030000
003oool0oooo01d0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P0000040?ooo`030000
003oool0oooo008000003P3oool00`000000oooo0?ooo`0Z0?ooo`<0VIVIo`3ooooO0?ooo`T00000
00<0VIVI0?ooo`3oool0]03oool00`000000oooo0?ooo`0Q0?ooo`00:P3oool3000000L0oooo0`00
00050?ooo`030000003oool0oooo01`0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P00
00040?ooo`030000003oool0oooo008000003P3oool00`000000oooo0?ooo`0[0?ooo`<0VIVIo`3o
oooN0?ooo`T00000]`3oool00`000000oooo0?ooo`0Q0?ooo`00:@3oool4000000P0oooo10000004
0?ooo`030000003oool0oooo01/0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P000005
0?ooo`040000003oool00000000000h0oooo00<000000?ooo`3oool0:`3oool309VIVOl0oooog@3o
ool:00000;L0oooo00<000000?ooo`3oool08@3oool003d0oooo00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`050?ooo`<000003P3oool00`000000oooo0?ooo`0/0?ooo`<0VIVIo`3ooooM
0?ooo`T00000]`3oool00`000000oooo0?ooo`0Q0?ooo`00>@3oool01@000000oooo0?ooo`3oool0
000001h0oooo0P0000030?ooo`8000002@3oool2000000<0oooo0P0000070?ooo`<000003P3oool0
0`000000oooo0?ooo`0/0?ooo`<0VIVIo`3ooooM0?ooo`T00000]`3oool00`000000oooo0?ooo`0Q
0?ooo`00>@3oool01@000000oooo0?ooo`3oool0000001l0oooo1@00000;0?ooo`D000002@3oool2
000000h0oooo00<000000?ooo`3oool0;03oool409VIVOl0oooog03oool800000;P0oooo00<00000
0?ooo`3oool08@3oool003X0oooo0`00001>0?ooo`030000003oool0oooo02d0oooo0`2IVIWo0?oo
om/0oooo2000002i0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo02d0
oooo102IVIWo0?ooomX0oooo100000000`2IVIT000000?ooo`2j0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo02h0oooo0`2IVIWo0?ooomT0oooo100000000`2IVIT0oooo
0?ooo`2k0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo02h0oooo102I
VIWo0?ooomL0oooo100000000`2IVIT0oooo0?ooo`2l0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo02l0oooo0`2IVIWo0?ooomH0oooo100000000`2IVIT0oooo0?ooo`2m
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo02l0oooo0`2IVIWo0?oo
omD0oooo100000000`2IVIT0oooo0?ooo`2n0?ooo`030000003oool0oooo0240oooo002;0?ooo`03
0000003oool0oooo0300oooo0`2IVIWo0?ooom<0oooo100000000`2IVIT0oooo0?ooo`2o0?ooo`03
0000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0300oooo0`2IVIWo0?ooom80oooo
100000000`2IVIT0oooo0?ooo`300?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003o
ool0oooo0300oooo102IVIWo0?ooom00oooo100000000`2IVIT0oooo0?ooo`310?ooo`030000003o
ool0oooo0240oooo002;0?ooo`030000003oool0oooo0340oooo0`2IVIWo0?ooom00oooo10000034
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0340oooo102IVIWo0?oo
olh0oooo100000350?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0380
oooo0`2IVIWo0?ooold0oooo100000360?ooo`030000003oool0oooo0240oooo002;0?ooo`<00000
<P3oool409VIVOl0oooob`3oool400000<D0oooo0`00000S0?ooo`00R`3oool00`000000oooo0?oo
o`0c0?ooo`<0VIVIo`3oooo:0?ooo`@0000000<0VIVI0?ooo`3oool0a@3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0c0?ooo`<0VIVIo`3oooo90?ooo`@0000000<0VIVI
0?ooo`3oool0aP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0d0?oo
o`<0VIVIo`3oooo70?ooo`@0000000<0VIVI0?ooo`3oool0a`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0d0?ooo`<0VIVIo`3oooo60?ooo`@0000000<0VIVI0?ooo`3o
ool0b03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0d0?ooo`@0VIVI
o`3oooo50?ooo`@00000b`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`0e0?ooo`<0VIVIo`3oooo40?ooo`@00000c03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0e0?ooo`@0VIVIo`3oooo20?ooo`@00000c@3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`0f0?ooo`<0VIVIo`3oooo10?ooo`@00000cP3oool00`00
0000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0f0?ooo`@0VIVIo`3ooono0?ooo`@0
0000c`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0g0?ooo`<0VIVI
o`3ooonn0?ooo`@00000d03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`0g0?ooo`<0VIVIo`3ooonm0?ooo`@00000d@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0h0?ooo`<0VIVIo`3ooonk0?ooo`@00000dP3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`0h0?ooo`<0VIVIo`3ooonj0?ooo`D00000dP3oool00`00
0000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0i0?ooo`<0VIVIo`3ooonh0?ooo`D0
0000d`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0;0?ooo`H00000
:03oool309VIVOl0oooo]`3oool00`2IVIT000000000000200000=@0oooo00<000000?ooo`3oool0
8@3oool008/0oooo00<000000?ooo`3oool02P3oool8000002L0oooo102IVIWo0?oookD0oooo00<0
VIVI0000000000000P00003E0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0
oooo00X0oooo2@00000W0?ooo`<0VIVIo`3ooond0?ooo`0309VIV@000000000000800000eP3oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0:0?ooo`T000009`3oool409VI
VOl0oooo/P3oool00`2IVIT000000000000200000=L0oooo00<000000?ooo`3oool08@3oool008/0
oooo00<000000?ooo`3oool02@3oool:000002P0oooo0`2IVIWo0?oook40oooo00<0VIVI00000000
00000P00003H0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo00X0oooo
2@00000X0?ooo`@0VIVIo`3ooon_0?ooo`0309VIV@000000000000800000f@3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0:0?ooo`T00000:@3oool309VIVOl0oooo[P3o
ool00`2IVIT000000000000200000=X0oooo00<000000?ooo`3oool08@3oool008/0oooo0`00000:
0?ooo`P00000:P3oool309VIVOl0oooo[@3oool00`2IVIT000000000000200000=T0oooo0`00000S
0?ooo`00R`3oool00`000000oooo0?ooo`0;0?ooo`P00000:P3oool309VIVOl0ooooZ`3oool00`2I
VIT000000000000200000=`0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo00000002000002X0oooo0`2IVIWo0?ooojX0oooo00<0VIVI00000000
00000P00003M0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0100oooo
1000000Y0?ooo`@0VIVIo`3ooonX0?ooo`0309VIV@000000000000800000gP3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0A0?ooo`@00000:@3oool309VIVOl0ooooY`3o
ool00`2IVIT000000000000200000=l0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool04P3oool3000002T0oooo102IVIWo0?ooojD0oooo00<0VIVI0000000000000P00003P
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0180oooo1000000Y0?oo
o`<0VIVIo`3ooonT0?ooo`0309VIV@000000000000800000h@3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0C0?ooo`<00000:@3oool409VIVOl0ooooXP3oool00`2IVIT0
00000000000200000>80oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0
503oool3000002T0oooo0`2IVIWo0?oooj40oooo00<0VIVI0000000000000P00003S0?ooo`030000
003oool0oooo0240oooo002;0?ooo`030000003oool0oooo01@0oooo1000000X0?ooo`<0VIVIo`3o
oonP0?ooo`0309VIV@000000000000800000i03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0E0?ooo`<00000:@3oool309VIVOl0ooooWP3oool00`2IVIT0000000000002
00000>D0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool05@3oool40000
02P0oooo0`2IVIWo0?oooid0oooo00<0VIVI0000000000000P00003V0?ooo`030000003oool0oooo
0240oooo002;0?ooo`030000003oool0oooo01H0oooo1000000W0?ooo`@0VIVIo`3ooonK0?ooo`03
09VIV@000000000000800000i`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`0G0?ooo`<00000:03oool309VIVOl0ooooVP3oool00`2IVIT000000000000200000>P0oooo
00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool05`3oool4000002L0oooo102I
VIWo0?oooiP0oooo00<0VIVI0000000000000P00003Y0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo01P0oooo0`00000X0?ooo`<0VIVIo`3ooonG0?ooo`0309VIV@000000
000000800000jP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0I0?oo
o`<000009`3oool409VIVOl0ooooU@3oool00`2IVIT000000000000200000>/0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool06@3oool4000002L0oooo0`2IVIWo0?oooi@0
oooo00<0VIVI0000000000000P00003/0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000
003oool0oooo01X0oooo0`00000W0?ooo`<0VIVIo`3ooonC0?ooo`0309VIV@000000000000<00000
k03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0J0?ooo`@000009`3o
ool309VIVOl0ooooT@3oool00`2IVIT000000000000300000>d0oooo00<000000?ooo`3oool08@3o
ool008/0oooo00<000000?ooo`3oool06`3oool4000002H0oooo0`2IVIWo0?oooi00oooo00<0VIVI
0000000000000`00003^0?ooo`030000003oool0oooo0240oooo002;0?ooo`<00000703oool30000
02L0oooo0`2IVIWo0?ooohh0oooo00<0VIVI0000000000000`00003]0?ooo`<000008`3oool008/0
oooo00<000000?ooo`3oool0703oool4000002H0oooo0`2IVIWo0?ooohd0oooo00<0VIVI00000000
00000`00003`0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo01d0oooo
0`00000V0?ooo`@0VIVIo`3ooon;0?ooo`0309VIV@000000000000800000lP3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0N0?ooo`<000009P3oool309VIVOl0ooooRP3o
ool00`2IVIT000000000000200000?<0oooo00<000000?ooo`3oool08@3oool008/0oooo00<00000
0?ooo`3oool07P3oool4000002D0oooo102IVIWo0?ooohP0oooo00<0VIVI0000000000000P00003d
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo01l0oooo0`00000V0?oo
o`<0VIVIo`3ooon70?ooo`0309VIV@000000000000800000m@3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0O0?ooo`@000009P3oool309VIVOl0ooooQ@3oool00`2IVIT0
00000000000200000?H0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0
803oool4000002D0oooo102IVIWo0?oooh<0oooo00<0VIVI0000000000000P00003g0?ooo`030000
003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0240oooo0`00000V0?ooo`<0VIVIo`3o
oon20?ooo`0309VIV@000000000000800000n03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0Q0?ooo`@000009@3oool409VIVOl0ooooP03oool00`2IVIT0000000000002
00000?T0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool08P3oool30000
02H0oooo0`2IVIWo0?ooogl0oooo1@00003j0?ooo`030000003oool0oooo0240oooo002;0?ooo`03
0000003oool0oooo02<0oooo0`00000V0?ooo`<0VIVIo`3ooomm0?ooo`D00000n`3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0S0?ooo`@000009@3oool409VIVOl0oooo
N`3oool500000?`0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0903o
ool3000002H0oooo0`2IVIWo0?ooogT0oooo00<0VIVI0000000000000`00003m0?ooo`030000003o
ool0oooo0240oooo002;0?ooo`030000003oool0oooo02@0oooo1000000U0?ooo`@0VIVIo`3ooomg
0?ooo`0309VIV@000000000000<00000oP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`0U0?ooo`@000009@3oool309VIVOl0ooooMP3oool00`2IVIT00000000000020000
0?l0oooo0@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0V0?ooo`<0
00009@3oool409VIVOl0ooooM03oool00`2IVIT000000000000200000?l0oooo0P3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0V0?ooo`@000009@3oool409VIVOl0oooo
LP3oool00`2IVIT000000000000200000?l0oooo0`3oool00`000000oooo0?ooo`0Q0?ooo`00GP3o
ool3000000H0oooo0P0000050?ooo`<000001@3oool8000000d0oooo00<000000?ooo`3oool09`3o
ool3000002H0oooo0`2IVIWo0?ooog40oooo00<0VIVI0000000000000P00003o0?ooo`@0oooo00<0
00000?ooo`3oool08@3oool005d0oooo00D000000?ooo`3oool0oooo000000050?ooo`800000103o
ool01@000000oooo0?ooo`3oool0000000D0oooo2000000<0?ooo`030000003oool0oooo02L0oooo
1000000U0?ooo`@0VIVIo`3ooom_0?ooo`0309VIV@000000000000800000o`3oool50?ooo`030000
003oool0oooo0240oooo001L0?ooo`030000003oool0oooo0080oooo0P0000090?ooo`030000003o
ool0oooo0080oooo0P0000050?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0X0?ooo`@000009@3oool309VIVOl0ooooKP3oool00`2IVIT000000000
000200000?l0oooo1P3oool00`000000oooo0?ooo`0Q0?ooo`00G03oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0:@3oool3000002D0oooo102IVIWo
0?ooof`0oooo00<0VIVI0000000000000P00003o0?ooo`L0oooo00<000000?ooo`3oool08@3oool0
05/0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P0000050?ooo`030000003oool0oooo
00l0oooo1@00000W0?ooo`@000009@3oool409VIVOl0ooooJP3oool00`2IVIT00000000000020000
0?l0oooo103oool5000002<0oooo001<0?ooo``000000`3oool2000000D0oooo0P0000070?ooo`80
00001@3oool2000000H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0Z0?ooo`@0
00009@3oool309VIVOl0ooooJ@3oool500000?l0oooo2@3oool00`000000oooo0?ooo`0Q0?ooo`00
F`3oool2000000D0oooo0P0000070?ooo`8000001@3oool2000000L0oooo00<000000?ooo`3oool0
3@3oool00`000000oooo0?ooo`0[0?ooo`<000009@3oool409VIVOl0ooooI`3oool500000?l0oooo
2P3oool00`000000oooo0?ooo`0Q0?ooo`00F`3oool2000000D0oooo0P0000070?ooo`8000001@3o
ool2000000P0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0/0?ooo`<000009@3o
ool309VIVOl0ooooIP3oool500000?l0oooo2`3oool00`000000oooo0?ooo`0Q0?ooo`00F`3oool2
000000D0oooo0P0000070?ooo`8000001@3oool2000000P0oooo0P00000=0?ooo`030000003oool0
oooo02`0oooo1000000T0?ooo`@0VIVIo`3ooomT0?ooo`D00000o`3oool<0?ooo`030000003oool0
oooo0240oooo001K0?ooo`8000001@3oool2000000L0oooo0P0000050?ooo`800000203oool20000
00d0oooo00<000000?ooo`3oool0;@3oool4000002@0oooo102IVIWo0?ooof40oooo00<0VIVI0000
000000000`00003o0?ooo`d0oooo00<000000?ooo`3oool08@3oool005/0oooo0P0000050?ooo`80
00001`3oool2000000D0oooo0P0000020?ooo`030000003oool0oooo00<0oooo0P00000=0?ooo`03
0000003oool0oooo02d0oooo1P00000S0?ooo`<0VIVIo`3ooomP0?ooo`0309VIV@00000000000080
0000o`3oool?0?ooo`030000003oool0oooo0240oooo001L0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`00
00001@3oool2000000d0oooo00<000000?ooo`3oool0;03oool800000280oooo102IVIWo0?oooeh0
oooo00<0VIVI0000000000000P00003o0?oooa00oooo00<000000?ooo`3oool08@3oool005`0oooo
0P0000030?ooo`8000002@3oool2000000<0oooo0P0000040?ooo`040000003oool0oooo0?ooo`<0
00003@3oool00`000000oooo0?ooo`0/0?ooo`T000008P3oool309VIVOl0ooooG@3oool00`2IVIT0
00000000000200000?l0oooo4@3oool00`000000oooo0?ooo`0Q0?ooo`00G@3oool5000000/0oooo
1@0000060?ooo`@000003`3oool00`000000oooo0?ooo`0/0?ooo`T000008P3oool409VIVOl0oooo
F`3oool00`2IVIT000000000000200000?l0oooo4P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`0[0?ooo`X000008`3oool409VIVOl0ooooF@3oool500000?l0oooo4`3o
ool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0/0?ooo`T00000903oool3
09VIVOl0ooooF03oool500000?l0oooo503oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`0/0?ooo`X000008`3oool409VIVOl0ooooE@3oool00`2IVIT00000000000030000
0?l0oooo5@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0/0?ooo`X0
0000903oool309VIVOl0ooooE03oool00`2IVIT000000000000300000?l0oooo5P3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0]0?ooo`X000008`3oool409VIVOl0oooo
DP3oool00`2IVIT000000000000200000?l0oooo603oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`0`0?ooo`040000003oool0oooo0?ooo`@000008`3oool409VIVOl0oooo
D03oool00`2IVIT000000000000200000?l0oooo6@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`0e0?ooo`<00000903oool309VIVOl0ooooC`3oool00`2IVIT000000000
000200000?l0oooo6P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0f
0?ooo`<000008`3oool409VIVOl0ooooC@3oool00`2IVIT000000000000200000?l0oooo6`3oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0f0?ooo`@000008`3oool309VI
VOl0ooooB`3oool00`2IVIT000000000000300000?l0oooo703oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`0g0?ooo`@000008P3oool409VIVOl0ooooB@3oool00`2IVIT0
00000000000300000?l0oooo7@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`0h0?ooo`<000008`3oool309VIVOl0ooooB03oool00`2IVIT000000000000300000?l0oooo
7P3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0h0?ooo`@000008`3o
ool309VIVOl0ooooAP3oool00`2IVIT000000000000300000?l0oooo7`3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool3000003T0oooo1000000R0?ooo`@0VIVIo`3ooom40?ooo`0309VIV@000000
000000800000o`3ooolO0?ooo`<000008`3oool008/0oooo00<000000?ooo`3oool0>P3oool30000
02<0oooo0`2IVIWo0?oood<0oooo00<0VIVI0000000000000P00003o0?ooob80oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0>`3oool300000280oooo102IVIWo0?oood00
oooo0P2IVIT400000?l0oooo8`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo
0?ooo`0k0?ooo`@000008P3oool309VIVOl0oooo?`3oool209VIV@@00000o`3ooolT0?ooo`030000
003oool0oooo0240oooo002;0?ooo`030000003oool0oooo03`0oooo1000000R0?ooo`<0VIVIo`3o
oolm0?ooo`0309VIV@000000000000<00000o`3ooolU0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo03d0oooo0`00000R0?ooo`@0VIVIo`3ooolk0?ooo`0309VIV@000000
000000<00000o`3ooolV0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo
03d0oooo1000000R0?ooo`<0VIVIo`3ooolj0?ooo`0309VIV@000000000000<00000o`3ooolW0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo03h0oooo1000000Q0?ooo`@0
VIVIo`3ooolh0?ooo`0309VIV@000000000000<00000o`3ooolX0?ooo`030000003oool0oooo0240
oooo002;0?ooo`030000003oool0oooo03l0oooo0`00000R0?ooo`@0VIVIo`3ooole0?ooo`80VIVI
1000003o0?ooobX0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0@03o
ool300000280oooo102IVIWo0?oooc<0oooo0P2IVIT400000?l0oooo:`3oool00`000000oooo0?oo
o`0Q0?ooo`00R`3oool00`000000oooo0?ooo`100?ooo`@000008P3oool409VIVOl0oooo<@3oool0
0`2IVIT000000000000300000?l0oooo;03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`00
0000oooo0?ooo`110?ooo`@000008P3oool409VIVOl0oooo;P3oool209VIV@D00000o`3oool]0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0480oooo0`00000S0?ooo`@0
VIVIo`3oool/0?ooo`80VIVI1@00003o0?ooobh0oooo00<000000?ooo`3oool08@3oool008/0oooo
00<000000?ooo`3oool0@P3oool400000280oooo102IVIWo0?ooob/0oooo0P2IVIT400000?l0oooo
<03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`130?ooo`@000008P3o
ool409VIVOl0oooo:03oool209VIV@D00000o`3ooola0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo04@0oooo0`00000S0?ooo`@0VIVIo`3ooolV0?ooo`80VIVI1@00003o
0?oooc80oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0A@3oool30000
02<0oooo102IVIWo0?ooob@0oooo0P2IVIT500000?l0oooo<`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`150?ooo`@000008`3oool409VIVOl0oooo8@3oool309VIV@D0
0000o`3ooold0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo04H0oooo
1000000S0?ooo`@0VIVIo`3ooolO0?ooo`<0VIVI1000003o0?ooocH0oooo00<000000?ooo`3oool0
8@3oool008/0oooo00<000000?ooo`3oool0A`3oool4000002<0oooo0`2IVIWo0?oooah0oooo0P2I
VIT500000?l0oooo=`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`17
0?ooo`@000008`3oool409VIVOl0oooo703oool209VIV@D00000o`3ooolh0?ooo`030000003oool0
oooo0240oooo002;0?ooo`<00000B03oool4000002<0oooo102IVIWo0?oooaT0oooo0`2IVIT50000
0?l0oooo=`3oool3000002<0oooo002;0?ooo`030000003oool0oooo04T0oooo1000000S0?ooo`@0
VIVIo`3ooolG0?ooo`<0VIVI1@00003o0?ooocX0oooo00<000000?ooo`3oool08@3oool008/0oooo
00<000000?ooo`3oool0BP3oool4000002<0oooo102IVIWo0?oooaD0oooo0`2IVIT400000?l0oooo
?03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`1;0?ooo`@000008`3o
ool409VIVOl0oooo4P3oool309VIV@D00000o`3ooolm0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo04`0oooo1000000S0?ooo`@0VIVIo`3oool@0?ooo`<0VIVI1@00003o
0?oooch0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0C@3oool40000
02<0oooo0`2IVIWo0?ooo`l0oooo0`2IVIT500000?l0oooo?`3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`1>0?ooo`@000008P3oool409VIVOl0oooo303oool409VIV@@0
0000o`3ooom10?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo04l0oooo
1000000R0?ooo`@0VIVIo`3oool:0?ooo`@0VIVI1000003o0?oood80oooo00<000000?ooo`3oool0
8@3oool008/0oooo00<000000?ooo`3oool0D03oool400000280oooo102IVIWo0?ooo`P0oooo0`2I
VIT500000?l0oooo@`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`1@
0?ooo`@000008`3oool409VIVOl0oooo1@3oool409VIV@D00000o`3ooom40?ooo`030000003oool0
oooo0240oooo002;0?ooo`030000003oool0oooo0540oooo1000000S0?ooo`@0VIVIo`3oool30?oo
o`@0VIVI1@00003o0?ooodD0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3o
ool0DP3oool4000002<0oooo102IVIWo0?ooo`@0VIVI1@00003o0?ooodL0oooo00<000000?ooo`3o
ool08@3oool008/0oooo00<000000?ooo`3oool0D`3oool400000280oooo102IVIWn0?ooo`@0VIVI
1@00003o0?ooodP0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0E03o
ool400000280oooo102IVIWk0?ooo`@0VIVI1@00003o0?ooodX0oooo00<000000?ooo`3oool08@3o
ool008/0oooo00<000000?ooo`3oool0E@3oool400000280oooo102IVIWi0?ooo`@0VIVI1@00003o
0?oood/0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0EP3oool40000
0280oooo102IVIWf0?ooo`D0VIVI1@00003o0?oood`0oooo00<000000?ooo`3oool08@3oool008/0
oooo00<000000?ooo`3oool0E`3oool400000280oooo102IVIWd0?ooo`@0VIVI1@00003o0?ooodh0
oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0F03oool400000280oooo
102IVIWa0?ooo`D0VIVI1@00003o0?ooodl0oooo00<000000?ooo`3oool08@3oool008/0oooo00<0
00000?ooo`3oool0F@3oool400000280oooo102IVIW_0?ooo`@0VIVI1@00003o0?oooe40oooo00<0
00000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0F@3oool400000280oooo1@2IVIW/
0?ooo`D0VIVI1@00003o0?oooe80oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?oo
o`3oool0FP3oool400000280oooo1@2IVIWY0?ooo`H0VIVI1@00003o0?oooe<0oooo00<000000?oo
o`3oool08@3oool008/0oooo00<000000?ooo`3oool0F`3oool4000002<0oooo102IVIWW0?ooo`D0
VIVI1@00003o0?oooeD0oooo00<000000?ooo`3oool08@3oool008/0oooo0`00001L0?ooo`@00000
8`3oool409VIVN@0oooo1P2IVIT500000?l0ooooE03oool3000002<0oooo002;0?ooo`030000003o
ool0oooo05d0oooo1000000S0?ooo`@0VIVIhP3oool509VIV@D00000o`3ooomH0?ooo`030000003o
ool0oooo0240oooo002;0?ooo`030000003oool0oooo05h0oooo2000000O0?ooo`@0VIVIg`3oool6
09VIV@D00000o`3ooomI0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo
05l0oooo2000000O0?ooo`@0VIVIg@3oool509VIV@030?ooo`000000000000<00000o`3ooomJ0?oo
o`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo05l0oooo2@00000O0?ooo`@0
VIVIfP3oool609VIV@D00000o`3ooomL0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000
003oool0oooo05l0oooo2@00000P0?ooo`@0VIVIf03oool509VIV@030?ooo`000000000000<00000
o`3ooomM0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo05h0oooo2P00
000Q0?ooo`@0VIVIe@3oool609VIV@D00000o`3ooomO0?ooo`030000003oool0oooo0240oooo002;
0?ooo`030000003oool0oooo05l0oooo2@00000R0?ooo`D0VIVId@3oool00`2IVIT0000000000009
00000?l0ooooH03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`1O0?oo
o`T000008`3oool509VIVLh0oooo0P2IVIT;00000?l0ooooH@3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`1O0?ooo`X000008`3oool509VIVL`0oooo0`2IVIT900000?l0
ooooH`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`1P0?ooo`/00000
8P3oool509VIVLT0oooo1@2IVIT900000?l0ooooH`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`1S0?ooo`040000003oool0oooo0?ooo`D000008`3oool509VIVLD0oooo
1P2IVIT:00000?l0ooooH`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`1X0?ooo`D000008`3oool509VIVL80oooo1P2IVIT00`3oool000000000000900000?l0ooooH`3o
ool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`1Z0?ooo`D000008P3oool5
09VIVKl0oooo1P2IVIT00`3oool000000000000;00000?l0ooooH`3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`1[0?ooo`D000008`3oool509VIVK/0oooo1`2IVIT00`3o
ool000000000000;00000?l0ooooI03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000
oooo0?ooo`1/0?ooo`D000008`3oool509VIVKP0oooo1`2IVIT00`3oool000000000000400000080
oooo1P00003o0?ooofD0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0
KP3oool4000002@0oooo1@2IVIVe0?ooo`H0VIVI00<0oooo000000000000100000070?ooo`030000
003oool0oooo0?l0ooooI@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?oo
o`1_0?ooo`D000008`3oool509VIVK80oooo1P2IVIT00`3oool000000000000400000?l0ooooL@3o
ool00`000000oooo0?ooo`0Q0?ooo`00N03oool300000100oooo00<000000?ooo`3oool0L03oool5
000002<0oooo1@2IVIV_0?ooo`H0VIVI0P3oool600000?l0ooooLP3oool00`000000oooo0?ooo`0Q
0?ooo`00M`3oool01@000000oooo0?ooo`3oool0000000l0oooo00<000000?ooo`3oool0L@3oool5
000002@0oooo1@2IVIV[0?ooo`H0VIVI0P3oool600000?l0ooooM03oool00`000000oooo0?ooo`0Q
0?ooo`00MP3oool00`000000oooo0?ooo`020?ooo`8000003P3oool00`000000oooo0?ooo`1c0?oo
o`@00000903oool509VIVJP0oooo1P2IVIT20?ooo`H00000o`3ooomf0?ooo`030000003oool0oooo
0240oooo001f0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`1d0?ooo`D000008`3oool609VIVJ@0oooo1`2IVIT20?ooo`H00000o`3ooomg0?ooo`03
0000003oool0oooo0240oooo001e0?ooo`8000001@3oool2000000d0oooo1@00001c0?ooo`D00000
903oool609VIVJ00oooo1`2IVIT20?ooo`H00000o`3ooome0?ooo`D000008`3oool007D0oooo0P00
00050?ooo`8000003@3oool00`000000oooo0?ooo`1f0?ooo`D000009@3oool509VIVI`0oooo202I
VIT20?ooo`H00000o`3ooomk0?ooo`030000003oool0oooo0240oooo001e0?ooo`8000001@3oool2
000000d0oooo00<000000?ooo`3oool0N03oool5000002@0oooo1P2IVIVH0?ooo`P0VIVI0P3oool6
00000?l0ooooO@3oool00`000000oooo0?ooo`0Q0?ooo`00M@3oool2000000D0oooo0P00000=0?oo
o`030000003oool0oooo07T0oooo1@00000U0?ooo`H0VIVIU03oool709VIV@<0oooo1`00003o0?oo
ogh0oooo00<000000?ooo`3oool08@3oool007D0oooo0P0000050?ooo`8000003@3oool00`000000
oooo0?ooo`1j0?ooo`D000009P3oool509VIVI00oooo202IVIT30?ooo`L00000o`3ooon00?ooo`03
0000003oool0oooo0240oooo001e0?ooo`8000001@3oool2000000d0oooo00<000000?ooo`3oool0
O03oool4000002H0oooo1P2IVIV<0?ooo`P0VIVI0`3oool700000?l0ooooPP3oool00`000000oooo
0?ooo`0Q0?ooo`00M@3oool2000000D0oooo0P00000=0?ooo`030000003oool0oooo07d0oooo1@00
000V0?ooo`H0VIVIR03oool709VIV@@0oooo1`00003o0?oooh@0oooo00<000000?ooo`3oool08@3o
ool007H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo07h0oooo1@00000W0?ooo`H0VIVIQ03oool709VIV@@0oooo1`00003o0?ooohH0oooo00<00000
0?ooo`3oool08@3oool007H0oooo0P0000030?ooo`8000003P3oool00`000000oooo0?ooo`1o0?oo
o`H000009P3oool709VIVGl0oooo202IVIT40?ooo`L00000o`3ooon80?ooo`030000003oool0oooo
0240oooo001g0?ooo`D000003`3oool00`000000oooo0?ooo`210?ooo`D000009`3oool709VIVG/0
oooo202IVIT40?ooo`L00000o`3ooon:0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000
003oool0oooo0880oooo1P00000W0?ooo`L0VIVIM`3oool709VIV@D0oooo1`00003o0?oooh`0oooo
00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0Q03oool5000002T0oooo1`2I
VIUa0?ooo`P0VIVI1@3oool700000?l0ooooSP3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`260?ooo`D00000:@3oool709VIVF`0oooo2@2IVIT50?ooo`L00000o`3ooon@
0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo08L0oooo1P00000Y0?oo
o`L0VIVII`3oool909VIV@H0oooo1`00003o0?oooi80oooo00<000000?ooo`3oool08@3oool008/0
oooo00<000000?ooo`3oool0R@3oool5000002X0oooo2@2IVIUO0?ooo`/0VIVI1P3oool700000?l0
ooooU03oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`2:0?ooo`H00000
:P3oool:09VIVEP0oooo302IVIT70?ooo`L00000o`3ooonF0?ooo`030000003oool0oooo0240oooo
002;0?ooo`030000003oool0oooo08`0oooo1P00000[0?ooo`T0VIVIDP3oool<09VIV@T0oooo1`00
003o0?oooiP0oooo00<000000?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0S@3oool7
000002d0oooo202IVIU;0?ooo``0VIVI2P3oool800000?l0ooooVP3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`2?0?ooo`L00000;@3oool:09VIVD80oooo3P2IVIT;0?oo
o`P00000o`3ooonL0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0940
oooo1`00000^0?ooo`h0VIVI=P3oool?09VIV@`0oooo2@00003o0?oooih0oooo00<000000?ooo`3o
ool08@3oool008/0oooo00<000000?ooo`3oool0U03oool7000002d0oooo4P2IVITZ0?oooa00VIVI
3`3oool800000?l0ooooX@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool3000009H0oooo1`00
000`0?oooa<0VIVI7P3ooolB09VIVA00oooo2000003o0?oooj80oooo0`00000S0?ooo`00R`3oool0
0`000000oooo0?ooo`2H0?ooo`L00000=03ooolh09VIVA<0oooo2000003o0?ooojH0oooo00<00000
0?ooo`3oool08@3oool008/0oooo00<000000?ooo`3oool0VP3oool7000003T0oooo:P2IVITG0?oo
o`P00000o`3ooonY0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo09`0
oooo2000000k0?ooob00VIVI6P3oool800000?l0ooooZ`3oool00`000000oooo0?ooo`0Q0?ooo`00
R`3oool00`000000oooo0?ooo`2N0?ooo`P00000L03oool800000?l0oooo[P3oool00`000000oooo
0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`2Q0?ooo`P00000JP3oool900000?l0oooo/03o
ool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`2S0?ooo`P00000I@3oool9
00000?l0oooo/`3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`2V0?oo
o`P00000203oool600000540oooo2P00003o0?oookD0oooo00<000000?ooo`3oool08@3oool008/0
oooo00<000000?ooo`3oool0Z03oool:000000<0oooo2000001<0?ooo`/00000o`3ooonh0?ooo`03
0000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0:/0oooo4`0000170?ooo``00000
o`3ooonk0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0:h0oooo4000
00120?ooo`d00000o`3ooono0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0
oooo0;80oooo3@00000k0?ooo`l00000o`3oooo30?ooo`030000003oool0oooo0240oooo002;0?oo
o`030000003oool0oooo0;D0oooo4@00000/0?oooa@00000o`3oooo60?ooo`030000003oool0oooo
0240oooo002;0?ooo`030000003oool0oooo0;D0oooo5`00000Q0?oooa@00000o`3oooo;0?ooo`03
0000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0;D0oooo200000020?oooc/00000
o`3ooooB0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0oooo0;H0oooo1P00
000:0?ooobd00000o`3ooooI0?ooo`030000003oool0oooo0240oooo002;0?ooo`030000003oool0
oooo0;T0oooo00<000000?ooo`3oool03`3ooolR00000?l0oooog`3oool00`000000oooo0?ooo`0Q
0?ooo`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooc@3oool00`000000oooo0?ooo`0Q0?oo
o`00R`3oool00`000000oooo0?ooo`3o0?ooool0ooooc@3oool00`000000oooo0?ooo`0Q0?ooo`00
R`3oool00`000000oooo0?ooo`3o0?ooool0ooooc@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3o
ool00`000000oooo0?ooo`3o0?ooool0ooooc@3oool00`000000oooo0?ooo`0Q0?ooo`00R`3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0:P0oooo00<000000?ooo`3oool0Z03oool0
0`000000oooo0?ooo`2X0?ooo`030000003oool0oooo0:P0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0Q0?ooo`00R`3oool3000000h0oooo00<000000?ooo`3oool0Z03oool00`00
0000oooo0?ooo`2X0?ooo`030000003oool0oooo0:P0oooo00<000000?ooo`3oool0Z03oool00`00
0000oooo0?ooo`0<0?ooo`<000008`3oool008/0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool03P3oool00`000000
oooo0?ooo`0Q0?ooo`00R`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0240oooo
002;0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool08@3oool008/0ooooo`00003o
00000=4000008`3oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon00?ooo`00
o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0ooooo`3ooon0
0?ooo`00o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?ooool0oooo
o`3ooon00?ooo`00o`3ooooo0?ooool0ooooP03oool00?l0ooooo`3ooooo0?oooh00oooo003o0?oo
ool0ooooi@3oool6000000030?ooo`000000000000@00000S@3oool00?l0ooooo`3ooon40?ooo`H0
000000<0oooo0000000000001000001F0?ooo`8000001@3oool2000008l0oooo003o0?ooool0oooo
QP3oool2000000D0oooo0P00001H0?ooo`8000001@3oool2000008l0oooo003o0?ooool0ooooQP3o
ool2000000D0oooo0P00001H0?ooo`<000000`3oool200000900oooo003o0?ooool0ooooQP3oool3
000000<0oooo0P00001J0?ooo`8000000P3oool300000900oooo003o0?ooool0ooooQ`3oool20000
0080oooo0`00001J0?ooo`8000000P3oool200000940oooo003o0?ooool0ooooQ`3oool200000080
oooo0P00001K0?ooo`80000000@0oooo00000000000000001`3oool5000008D0oooo003o0?ooool0
ooooQ`3oool2000000040?ooo`0000000000000005/0oooo1@0000090?ooo`040000003oool0oooo
0?ooo`800000P`3oool00?l0ooooo`3ooon70?ooo`D00000G@3oool4000000T0oooo00<000000?oo
o`3oool0QP3oool00?l0ooooo`3ooon80?ooo`@00000G@3oool200000080oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`250?ooo`00o`3oooo50?ooo`D00000_@3oool200000080oooo
00<000000?ooo`3oool0FP3oool2000000<0oooo0P0000080?ooo`030000003oool0oooo08@0oooo
003o0?oool<0oooo0`0000040?ooo`8000001P3oool00`000000oooo0?ooo`0?0?ooo`030000003o
ool0oooo0:00oooo0P0000030?ooo`800000F`3oool2000000@0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`230?ooo`00o`3oooo30?ooo`8000001`3oool00`000000oooo0?ooo`02
0?ooo`8000004@3oool200000:80oooo0P0000040?ooo`030000003oool0oooo05P0oooo0P000005
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0P`3oool00?l0ooooa03oool00`00
0000oooo0?ooo`050?ooo`040000003oool0oooo0?ooo`8000004`3oool200000:40oooo0P000005
0?ooo`030000003oool0oooo05D0oooo1P0000020?ooo`H000000`3oool00`000000oooo0?ooo`23
0?ooo`00o`3oooo50?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0W@3oool600000080oooo1@00001Q0?ooo`050000003oool0oooo
0?ooo`000000Q@3oool00>00oooo0`0000060?ooo`8000001@3oool3000000L0oooo1P0000060?oo
o`<000001@3oool6000000D0oooo1P0000070?ooo`D000001P3oool5000000<0oooo1P0000000`3o
ool0000000000004000000h0oooo00<000000?ooo`3oool02`3oool3000000H0oooo0P0000020?oo
o`P000000`3oool6000000L0oooo0P00000:0?ooo`<000001@3oool6000000L0oooo1@0000040?oo
o`H000000`3oool5000000D0oooo1`0000030?ooo`8000000P3oool6000000030?ooo`0000000000
00@000001P3oool2000001/0oooo0`0000060?ooo`8000000P3oool6000000P0oooo0`0000050?oo
o`H00000203oool00`000000oooo0?ooo`070?ooo`D00000103oool600000740oooo00<000000?oo
o`3oool0=03oool3000008H0oooo003O0?ooo`050000003oool0oooo0?ooo`0000001@3oool20000
00@0oooo00D000000?ooo`3oool0oooo000000080?ooo`8000001`3oool01@000000oooo0?ooo`3o
ool0000000@0oooo0P0000040?ooo`030000003oool0oooo0080oooo0P0000030?ooo`<00000103o
ool2000000@0oooo00<000000?ooo`3oool00P3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool2000000D0oooo0P00000@0?ooo`030000003oool0oooo00X0oooo00D000000?ooo`3oool0oooo
000000050?ooo`8000000`3oool800000080oooo0P0000040?ooo`030000003oool0oooo00H0oooo
0P0000070?ooo`050000003oool0oooo0?ooo`000000103oool2000000@0oooo00<000000?ooo`3o
ool00`3oool2000000<0oooo0P0000050?ooo`800000103oool2000000<0oooo0P0000030?ooo`<0
0000203oool00`000000oooo0?ooo`030?ooo`8000001@3oool2000000T0oooo00<000000?ooo`3o
ool0603oool01@000000oooo0?ooo`3oool0000000D0oooo0P0000020?ooo`8000000`3oool30000
00D0oooo00D000000?ooo`3oool0oooo000000040?ooo`8000000`3oool3000000H0oooo0P000007
0?ooo`800000103oool010000000oooo0?ooo`3oool2000000<0oooo0`00001_0?ooo`030000003o
ool0oooo0;d0oooo003N0?ooo`030000003oool0oooo0080oooo0P0000090?ooo`030000003oool0
oooo0080oooo0P0000070?ooo`8000001P3oool00`000000oooo0?ooo`020?ooo`8000002P3oool0
0`000000oooo0?ooo`070?ooo`8000000`3oool2000000D0oooo0P0000020?ooo`8000001@3oool2
000000<0oooo0P0000050?ooo`800000403oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0080oooo0P00000:0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool01P3oool0
0`000000oooo0?ooo`060?ooo`8000001@3oool00`000000oooo0?ooo`020?ooo`8000002P3oool0
0`000000oooo0?ooo`020?ooo`800000103oool2000000@0oooo0P0000030?ooo`8000001@3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo00H0oooo0P0000050?ooo`8000001@3oool2
000000T0oooo0P00000H0?ooo`030000003oool0oooo0080oooo0P00000>0?ooo`800000103oool0
0`000000oooo0?ooo`020?ooo`8000002@3oool2000000H0oooo0P0000060?ooo`8000001@3oool2
000000P0oooo0P00001_0?ooo`030000003oool0oooo0;d0oooo003N0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00D0oooo0P0000060?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool20000
00T0oooo0P0000020?ooo`8000001@3oool200000080oooo0P0000050?ooo`8000000`3oool30000
00<0oooo0P00000A0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo00d0oooo0P0000080?ooo`800000103oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo0P0000020?ooo`8000001@3oool20000
00@0oooo0P0000030?ooo`8000001@3oool2000000@0oooo100000040?ooo`8000001@3oool30000
00<0oooo0P00000:0?ooo`800000603oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo
00d0oooo0P0000030?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool20000
00H0oooo00<000000?ooo`3oool0103oool2000000D0oooo0P0000090?ooo`800000KP3oool00`00
0000oooo0?ooo`2m0?ooo`00g@3oool2000000D0oooo0P0000070?ooo`8000001@3oool2000000H0
oooo0P0000050?ooo`8000001@3oool2000000T0oooo0P0000090?ooo`8000000P3oool2000000@0
oooo0`0000020?ooo`800000103oool3000000@0oooo0P0000020?ooo`<000004@3oool00`000000
oooo0?ooo`080?ooo`8000001@3oool2000000X0oooo00<000000?ooo`3oool0303oool2000000P0
oooo0P0000030?ooo`8000001@3oool2000000T0oooo0P0000020?ooo`8000001@3oool2000000@0
oooo0P0000030?ooo`8000001@3oool2000000<0oooo0P0000030?ooo`040000003oool0oooo0?oo
o`8000001P3oool200000080oooo0`00000:0?ooo`8000005`3oool2000000D0oooo0P00000>0?oo
o`8000000P3oool2000000D0oooo0P0000090?ooo`8000001P3oool2000000D0oooo0P0000040?oo
o`<000002@3oool2000006h0oooo00<000000?ooo`3oool0_@3oool00<l0oooo300000020?ooo`80
00001@3oool2000000L0oooo0P0000050?ooo`8000001P3oool2000000D0oooo0P0000050?ooo`80
0000203oool3000000T0oooo0P0000030?ooo`040000003oool0oooo0?ooo`<00000103oool01000
0000oooo0?ooo`3oool3000000D0oooo0P0000020?ooo`800000303oool<000000D0oooo0P000005
0?ooo`8000002`3oool00`000000oooo0?ooo`0:0?ooo`<00000103oool300000080oooo00@00000
0?ooo`3oool0oooo0P0000050?ooo`800000203oool300000080oooo0P0000050?ooo`800000103o
ool2000000<0oooo0P0000050?ooo`8000000P3oool3000000<0oooo0P0000020?ooo`8000001P3o
ool200000080oooo0P00000;0?ooo`8000001@3oool<000000H0oooo0P0000050?ooo`8000003P3o
ool200000080oooo0P0000050?ooo`8000002@3oool2000000H0oooo0P0000060?ooo`040000003o
ool0oooo0?ooo`<000002P3oool2000000<0oooo300000080?ooodT00000203oool<000000L0oooo
BP00001Y0?ooo`00g@3oool2000000D0oooo0P0000070?ooo`8000001@3oool2000000H0oooo0P00
00050?ooo`8000001@3oool2000000P0oooo0`0000080?ooo`<00000103oool00`000000oooo0000
0002000000H0oooo00<000000?ooo`0000000P0000060?ooo`80000000@0oooo0000000000000000
4P3oool00`000000oooo0?ooo`080?ooo`8000001@3oool2000000`0oooo00<000000?ooo`3oool0
2@3oool3000000<0oooo0P0000020?ooo`@000000P3oool2000000D0oooo0P0000080?ooo`<00000
0P3oool2000000D0oooo0P0000040?ooo`8000000`3oool2000000D0oooo0P0000020?ooo`800000
103oool200000080oooo0P0000060?ooo`80000000@0oooo00000000000000002`3oool2000001L0
oooo0P0000050?ooo`8000003@3oool300000080oooo0P0000050?ooo`800000203oool3000000L0
oooo00<000000?ooo`3oool01@3oool00`000000oooo00000002000000X0oooo0`00001^0?ooo`03
0000003oool0oooo0;d0oooo003M0?ooo`8000001@3oool2000000L0oooo0P0000050?ooo`800000
1P3oool2000000D0oooo0P0000050?ooo`800000103oool6000000P0oooo100000050?ooo`<00000
203oool3000000L0oooo1@00000C0?ooo`030000003oool0oooo00P0oooo0P0000050?ooo`800000
3@3oool00`000000oooo0?ooo`040?ooo`H000000`3oool2000000D0oooo0P0000020?ooo`800000
1@3oool2000000@0oooo1P0000030?ooo`800000103oool2000000D0oooo0P0000040?ooo`030000
003oool0oooo0080oooo0P0000030?ooo`800000103oool200000080oooo0P0000060?ooo`D00000
303oool2000001L0oooo0P0000050?ooo`800000303oool400000080oooo0P0000050?ooo`800000
1`3oool4000000L0oooo0P0000070?ooo`<000002P3oool4000006h0oooo00<000000?ooo`3oool0
_@3oool00=d0oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P0000060?ooo`8000001@3o
ool2000000D0oooo0P0000060?ooo`800000203oool5000000D0oooo100000070?ooo`@00000203o
ool4000001<0oooo00<000000?ooo`3oool0203oool2000000D0oooo0P00000=0?ooo`8000001`3o
ool2000000D0oooo0P0000050?ooo`8000000P3oool2000000D0oooo0P0000060?ooo`8000001@3o
ool7000000H0oooo0P0000040?ooo`8000000`3oool2000000@0oooo0P0000020?ooo`@0000000<0
oooo0000000000001`3oool4000000`0oooo0P00000G0?ooo`8000001@3oool2000000X0oooo1@00
00030?ooo`8000001@3oool2000000D0oooo1@0000090?ooo`030000003oool0oooo00@0oooo1000
00080?ooo`D00000K`3oool00`000000oooo0?ooo`2m0?ooo`00g@3oool2000000D0oooo0P000007
0?ooo`8000001@3oool2000000H0oooo0P0000050?ooo`8000001@3oool2000000L0oooo0P000005
0?ooo`H000001@3oool2000000<0oooo00<000000?ooo`3oool00`3oool2000000<0oooo00<00000
0?ooo`3oool01@3oool200000080oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`08
0?ooo`8000001@3oool2000000d0oooo0P0000080?ooo`800000103oool2000000D0oooo0P000002
0?ooo`8000001@3oool2000000L0oooo0P0000050?ooo`8000002P3oool2000000D0oooo1@000006
0?ooo`D00000103oool00`000000oooo0?ooo`050?ooo`8000000P3oool00`000000oooo0?ooo`09
0?ooo`030000003oool0oooo01H0oooo0P0000050?ooo`800000203oool6000000@0oooo0P000005
0?ooo`8000000`3oool6000000X0oooo00<000000?ooo`3oool00`3oool2000000<0oooo00<00000
0?ooo`3oool00`3oool600000700oooo00<000000?ooo`3oool0_@3oool00=d0oooo0P0000050?oo
o`8000001`3oool2000000D0oooo0P0000060?ooo`8000001@3oool2000000D0oooo0P0000080?oo
o`800000103oool3000000L0oooo0P0000040?ooo`8000000`3oool2000000@0oooo0P0000060?oo
o`8000000`3oool200000100oooo00<000000?ooo`3oool0203oool2000000D0oooo0P0000070?oo
o`030000003oool0oooo00<0oooo0P0000090?ooo`8000000`3oool2000000D0oooo0P0000020?oo
o`8000001@3oool2000000P0oooo0P0000040?ooo`8000002P3oool2000001T0oooo0P0000060?oo
o`8000000`3oool2000000P0oooo0P00000H0?ooo`8000001@3oool2000000P0oooo0`0000070?oo
o`8000001@3oool2000000<0oooo0`00000=0?ooo`8000000`3oool2000000@0oooo0P0000040?oo
o`<00000L`3oool00`000000oooo0?ooo`2m0?ooo`00gP3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`050?ooo`80
00001P3oool00`000000oooo0?ooo`030?ooo`050000003oool0oooo0?ooo`0000001@3oool20000
00D0oooo00<000000?ooo`3oool01P3oool2000000@0oooo0P0000030?ooo`800000103oool20000
00L0oooo0P0000040?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`030000003oool0oooo
00<0oooo0P0000030?ooo`8000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
00<0oooo00D000000?ooo`3oool0oooo000000050?ooo`8000001@3oool2000000T0oooo0P00000J
0?ooo`030000003oool0oooo00D0oooo0P0000040?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool01`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo0?ooo`800000103o
ool2000000D0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`00gP3oool2000000<0oooo0P000009
0?ooo`8000000`3oool2000000@0oooo1@0000060?ooo`8000000`3oool2000000@0oooo00@00000
0?ooo`3oool0oooo0`0000050?ooo`H000000`3oool2000000@0oooo0P0000030?ooo`800000103o
ool2000000L0oooo0P0000050?ooo`030000003oool0oooo01P0oooo0P0000030?ooo`8000002@3o
ool010000000oooo0?ooo`3oool3000000@0oooo00@000000?ooo`3oool0oooo0`0000040?ooo`03
0000003oool0oooo0080oooo0P0000040?ooo`8000000`3oool2000000@0oooo00@000000?ooo`3o
ool0oooo0`0000060?ooo`800000203oool2000001X0oooo0P0000060?ooo`8000001@3oool00`00
0000oooo0?ooo`030?ooo`8000006P3oool2000000<0oooo0P00000:0?ooo`H00000103oool20000
00<0oooo0P0000050?ooo`H000000`3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo
0?ooo`800000103oool2000000D0oooo1P00003o0?oooc00oooo003O0?ooo`D000002`3oool50000
00P0oooo0P0000070?ooo`D000001P3oool5000000L0oooo1@0000050?ooo`D000001P3oool50000
00H0oooo1P0000020?ooo`D00000603oool5000000/0oooo100000070?ooo`D000001P3oool40000
00L0oooo1@0000060?ooo`D00000203oool2000000L0oooo0P00000K0?ooo`030000003oool0oooo
0080oooo1P0000020?ooo`D000000P3oool00`000000oooo0?ooo`0J0?ooo`D00000303oool50000
00D0oooo1@0000070?ooo`D000000`3oool9000000@0oooo1@0000070?ooo`D00000o`3oool`0?oo
o`00o`3ooolR0?ooo`030000003oool0oooo08P0oooo0P0000060?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0K0?ooo`<000001P3oool2000000@0
oooo1P0000060?ooo`<000001@3oool6000000L0oooo0P0000090?ooo`8000009@3oool6000000H0
oooo0P0000040?ooo`D000001`3oool00`000000oooo0?ooo`050?ooo`P000001P3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo06d0oooo003o0?ooool0ooooK@3oool01@000000oooo
0?ooo`3oool0000000D0oooo0P0000060?ooo`8000001`3oool01@000000oooo0?ooo`3oool00000
00@0oooo0P0000040?ooo`030000003oool0oooo00H0oooo0P0000090?ooo`8000008`3oool20000
00@0oooo00<000000?ooo`3oool00`3oool2000000<0oooo0P0000040?ooo`030000003oool0oooo
00@0oooo0P0000070?ooo`P000001@3oool2000000T0oooo0P00001^0?ooo`00o`3ooooo0?ooof`0
oooo00<000000?ooo`3oool00P3oool2000000`0oooo0P0000060?ooo`030000003oool0oooo0080
oooo0P00000:0?ooo`030000003oool0oooo00H0oooo0P0000090?ooo`800000:@3oool00`000000
oooo0?ooo`060?ooo`8000001@3oool2000000D0oooo0P0000080?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool00P3oool2000000T0oooo0P00001^0?ooo`00o`3ooooo0?ooof`0oooo
00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`8000001P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00P0oooo0P0000080?ooo`8000002@3oool2000002P0oooo
0P0000070?ooo`8000001@3oool2000000H0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0K03oool00?l0ooooo`3o
oom[0?ooo`8000001@3oool2000000/0oooo0P0000050?ooo`8000001@3oool2000000T0oooo0P00
00080?ooo`8000002@3oool2000002P0oooo0P0000070?ooo`800000103oool3000000H0oooo0P00
00080?ooo`030000003oool0oooo00T0oooo0P0000090?ooo`800000K@3oool00?l0ooooo`3ooom[
0?ooo`8000001@3oool2000000/0oooo0P0000050?ooo`8000001@3oool2000000P0oooo0`000004
0?ooo`<000000P3oool00`000000oooo0?ooo`030?ooo`<000000P3oool00`000000oooo0?ooo`0U
0?ooo`<00000203oool010000000oooo0?ooo`3oool3000000L0oooo0P0000090?ooo`030000003o
ool0oooo00P0oooo0P0000090?ooo`800000K@3oool00?l0ooooo`3ooom[0?ooo`8000001@3oool2
000000/0oooo0P0000050?ooo`8000001@3oool2000000P0oooo0`0000030?ooo`8000000P3oool4
000000<0oooo0P0000020?ooo`@000009P3oool3000000T0oooo00<000000?ooo`0000000P000009
0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`08
0?ooo`030000003oool0oooo06/0oooo003o0?ooool0ooooJ`3oool2000000D0oooo0P00000;0?oo
o`8000001@3oool2000000D0oooo0P0000040?ooo`H000000`3oool2000000D0oooo0P0000020?oo
o`8000001@3oool200000280oooo1P00000;0?ooo`<000002P3oool2000000X0oooo00<000000?oo
o`3oool01`3oool2000000T0oooo0P00001/0?ooo`00o`3ooooo0?ooof/0oooo0P0000050?ooo`80
00002`3oool2000000D0oooo0P0000050?ooo`8000001P3oool2000000D0oooo0P0000050?ooo`80
00000P3oool2000000D0oooo0P00000T0?ooo`800000303oool4000000/0oooo00<000000?ooo`3o
ool0203oool2000000T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1Z0?ooo`00
o`3ooooo0?ooof/0oooo0P0000050?ooo`8000002`3oool2000000D0oooo0P0000050?ooo`800000
1`3oool2000000@0oooo0P0000050?ooo`8000000P3oool2000000D0oooo0P00000U0?ooo`800000
2P3oool2000000<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`800000
2@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo06X0oooo003o0?ooool0ooooJ`3o
ool2000000D0oooo0P00000;0?ooo`8000001@3oool2000000D0oooo0P0000080?ooo`8000000`3o
ool2000000D0oooo0P0000020?ooo`8000001@3oool2000002H0oooo0P0000080?ooo`800000103o
ool2000000T0oooo0P0000030?ooo`030000003oool0oooo00<0oooo0P0000090?ooo`8000002@3o
ool2000006/0oooo003o0?ooool0ooooK03oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00X0oooo0P0000060?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo0000
00050?ooo`8000000`3oool2000000D0oooo00@000000?ooo`3oool0oooo0P0000050?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool00`3oool2000000P0oooo0P0000040?ooo`800000
0`3oool00`000000oooo0?ooo`040?ooo`050000003oool0oooo0?ooo`0000001@3oool2000000<0
oooo00<000000?ooo`3oool0103oool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?oo
o`3oool0J@3oool00?l0ooooo`3ooom/0?ooo`8000000`3oool2000000T0oooo1@0000060?ooo`80
00000`3oool2000000@0oooo00@000000?ooo`3oool0oooo0`0000040?ooo`030000003oool0oooo
0080oooo0P0000040?ooo`030000003oool0oooo0080oooo0P00000R0?ooo`040000003oool0oooo
0?ooo`<00000203oool2000000@0oooo0P0000030?ooo`030000003oool0oooo00@0oooo00<00000
0?ooo`3oool00P3oool010000000oooo0?ooo`3oool3000000<0oooo00<000000?ooo`3oool0103o
ool01@000000oooo0?ooo`3oool0000000H0oooo00<000000?ooo`3oool0J@3oool00?l0ooooo`3o
oom]0?ooo`D000003@3oool2000000L0oooo1@0000060?ooo`D000001P3oool4000000L0oooo1000
000U0?ooo`D000002`3oool5000000@0oooo2@0000040?ooo`@000001@3oool900000080oooo2@00
001Z0?ooo`00o`3ooooo0?ooool0ooooP03oool00001\
\>"],
  ImageRangeCache->{{{57.375, 533.125}, {446.625, 153}} -> {-5.46154, \
-0.0374391, 0.0878205, 0.000346744}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell["Width 600, left at zero to print ", "Text"],

Cell[BoxData[
    RowBox[{"\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
        RowBox[{
        "Pion", " ", \(mass\^2\), " ", "as", " ", "a", " ", "function", " ", 
          "of", " ", 
          StyleBox["K",
            FontSlant->"Italic"]}], ",", 
        " ", \(along\ with\ a\ \(\(fit\)\(.\)\)\)}]}]], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->19],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan4b\)], "Input"],

Cell[BoxData[
    \(\(-0.17959798072993982`\) + 19.910598034026645`\/k\^2 - 
      4.443333990208367`\/k - 0.006238855759253004`\ k + 
      0.3234239036795359`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan4a\)], "Input"],

Cell[BoxData[
    \(\(\(0.2884081574812196`\)\(\[InvisibleSpace]\)\) - 
      0.018843801931274218`\/k\^2 - 0.6224854180168093`\/k - 
      0.03981135577203391`\ k - 0.3715266139153625`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan3b\)], "Input"],

Cell[BoxData[
    \(\(\(0.3814062497992905`\)\(\[InvisibleSpace]\)\) + 
      9.21474765359311`\/k\^2 - 2.8054347238814357`\/k - 
      0.0007731066750167008`\ k + 0.42168678100536616`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan3a\)], "Input"],

Cell[BoxData[
    \(\(-0.0036246242228034076`\) + 2.192102384106045`\/k\^2 - 
      1.497432761056963`\/k - 0.2756736584113894`\ k - 
      0.003521740282956598`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan2b\)], "Input"],

Cell[BoxData[
    \(\(-0.9534409760154304`\) - 0.5528773337710792`\/k\^2 + 
      1.8704370615470611`\/k - 0.008819841471860067`\ k + 
      0.38426683326540906`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan2a\)], "Input"],

Cell[BoxData[
    \(\(\(0.1938216019493156`\)\(\[InvisibleSpace]\)\) + 
      9.588074716240127`\/k\^2 - 2.9474159800975333`\/k - 
      0.011873126834583421`\ k - 0.00845009639940476`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(jan1\)], "Input"],

Cell[BoxData[
    \(\(-0.5057848663070794`\) + 3.8727746426452203`\/k\^2 - 
      0.1039900081767631`\/k - 0.010358811973834557`\ k + 
      0.23903608536559426`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Transpose[{klist, massSquared}], fitting, k]\)], "Input"],

Cell[BoxData[
    \(0.0003920469077573383`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      ListPlot[Transpose[{klist, massSquared}], 
        DisplayFunction \[Rule] Identity], 
      Plot[fitting, {k, 10, 50}, DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0238095 -0.853593 0.750985 [
[.2619 .03509 -6 -9 ]
[.2619 .03509 6 0 ]
[.5 .03509 -6 -9 ]
[.5 .03509 6 0 ]
[.7381 .03509 -6 -9 ]
[.7381 .03509 6 0 ]
[.97619 .03509 -6 -9 ]
[.97619 .03509 6 0 ]
[.01131 .19779 -18 -4.5 ]
[.01131 .19779 0 4.5 ]
[.01131 .34798 -18 -4.5 ]
[.01131 .34798 0 4.5 ]
[.01131 .49818 -18 -4.5 ]
[.01131 .49818 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .04759 m
.2619 .05384 L
s
[(20)] .2619 .03509 0 1 Mshowa
.5 .04759 m
.5 .05384 L
s
[(30)] .5 .03509 0 1 Mshowa
.7381 .04759 m
.7381 .05384 L
s
[(40)] .7381 .03509 0 1 Mshowa
.97619 .04759 m
.97619 .05384 L
s
[(50)] .97619 .03509 0 1 Mshowa
.125 Mabswid
.07143 .04759 m
.07143 .05134 L
s
.11905 .04759 m
.11905 .05134 L
s
.16667 .04759 m
.16667 .05134 L
s
.21429 .04759 m
.21429 .05134 L
s
.30952 .04759 m
.30952 .05134 L
s
.35714 .04759 m
.35714 .05134 L
s
.40476 .04759 m
.40476 .05134 L
s
.45238 .04759 m
.45238 .05134 L
s
.54762 .04759 m
.54762 .05134 L
s
.59524 .04759 m
.59524 .05134 L
s
.64286 .04759 m
.64286 .05134 L
s
.69048 .04759 m
.69048 .05134 L
s
.78571 .04759 m
.78571 .05134 L
s
.83333 .04759 m
.83333 .05134 L
s
.88095 .04759 m
.88095 .05134 L
s
.92857 .04759 m
.92857 .05134 L
s
.25 Mabswid
0 .04759 m
1 .04759 L
s
.02381 .19779 m
.03006 .19779 L
s
[(1.4)] .01131 .19779 1 0 Mshowa
.02381 .34798 m
.03006 .34798 L
s
[(1.6)] .01131 .34798 1 0 Mshowa
.02381 .49818 m
.03006 .49818 L
s
[(1.8)] .01131 .49818 1 0 Mshowa
.125 Mabswid
.02381 .08514 m
.02756 .08514 L
s
.02381 .12269 m
.02756 .12269 L
s
.02381 .16024 m
.02756 .16024 L
s
.02381 .23533 m
.02756 .23533 L
s
.02381 .27288 m
.02756 .27288 L
s
.02381 .31043 m
.02756 .31043 L
s
.02381 .38553 m
.02756 .38553 L
s
.02381 .42308 m
.02756 .42308 L
s
.02381 .46063 m
.02756 .46063 L
s
.02381 .01004 m
.02756 .01004 L
s
.02381 .53573 m
.02756 .53573 L
s
.02381 .57328 m
.02756 .57328 L
s
.02381 .61083 m
.02756 .61083 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.02381 .01472 Mdot
.07143 .08533 Mdot
.14286 .17197 Mdot
.2619 .28187 Mdot
.5 .43003 Mdot
.7381 .52941 Mdot
.97619 .60332 Mdot
.5 Mabswid
.02381 .01473 m
.06244 .07285 L
.10458 .12801 L
.14415 .17341 L
.18221 .21229 L
.22272 .24943 L
.26171 .28172 L
.30316 .31293 L
.34309 .34041 L
.3815 .36481 L
.42237 .38885 L
.46172 .41038 L
.49955 .42975 L
.53984 .44911 L
.57861 .46663 L
.61984 .4842 L
.65954 .5002 L
.69774 .5148 L
.73838 .52956 L
.77751 .5431 L
.81909 .55681 L
.85916 .56943 L
.89771 .58105 L
.93871 .5929 L
.97619 .6033 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00803oool00`000000oooo0?ooo`3o0?ooohh0oooo000P0?ooo`030000003oool0oooo0?l0
ooooSP3oool00200oooo00<000000?ooo`3oool0o`3ooon>0?ooo`00803oool00`000000oooo0?oo
o`1F0?ooo`H000001@3oool3000005<0oooo1@0000060?ooo`<00000E@3oool4000000D0oooo0`00
001E0?ooo`@000001P3oool3000000D0oooo000P0?ooo`800000F03oool01@000000oooo0?ooo`3o
ool0000000@0oooo0P0000020?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool00P3o
ool200000080oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`030?ooo`8000000P3o
ool00`000000oooo0?ooo`1F0?ooo`030000003oool0oooo0080oooo0P0000020?ooo`030000003o
ool0oooo0080oooo000O0?ooo`800000FP3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0E03oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0C`3oool6000000@0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`1F0?ooo`050000003oool0oooo0?ooo`000000103oool00`000000oooo0?oo
o`010?ooo`007`3oool2000005/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo04l0oooo00D000000?ooo`3oool0oooo000000050?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool0EP3oool01@000000oooo0?ooo`3oool0000000@0oooo
00<000000?ooo`3oool00@3oool00200oooo0P00001K0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`1C0?ooo`800000103oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0500oooo00@000000?ooo`3oool000001@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0040oooo000P0?ooo`800000G03oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0DP3oool2000000D0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`1A0?ooo`030000003oool0000000D0oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`1A0?ooo`@000001@3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo0040oooo000P0?ooo`030000003oool0000005/0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo05@0oooo00<00000
0?ooo`3oool00P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0540oooo00<00000
0?ooo`0000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0540oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0040oooo000P0?oo
o`030000003oool0000005H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`020?oo
o`050000003oool0oooo0?ooo`000000E`3oool00`000000oooo0?ooo`020?ooo`050000003oool0
oooo0?ooo`000000E@3oool2000000D0oooo00D000000?ooo`3oool0oooo0000001D0?ooo`030000
003oool0oooo00H0oooo00D000000?ooo`3oool0oooo000000040?ooo`00803oool010000000oooo
0?ooo`00001E0?ooo`D000001P3oool3000005@0oooo100000060?ooo`<00000E`3oool00`000000
oooo0?ooo`040?ooo`<00000E@3oool5000000D0oooo0`0000050?ooo`00803oool01@000000oooo
0?ooo`3oool000000?l0ooooS03oool00200oooo00D000000?ooo`3oool0oooo0000003o0?oooh`0
oooo000P0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0o`3ooon90?ooo`00803o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0ooooR03oool00200oooo00<00000
0?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooohP0oooo000P0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0o`3ooon70?ooo`00803oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0ooooQ`3oool00200oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`3o0?ooohH0oooo000F0?ooool00000VP0000010?ooo`00803oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0A
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`08
0?ooo`00803oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo05@0oooo00<000000?oo
o`3oool0GP3oool00`000000oooo0?ooo`1N0?ooo`030000003oool0oooo05l0oooo00<000000?oo
o`3oool0203oool00200oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`3o0?oooh<0
oooo000P0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0o`3ooon30?ooo`00803o
ool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0?l0ooooPP3oool00200oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`3o0?oooh80oooo000P0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool0o`3ooon10?ooo`00803oool00`000000oooo0?ooo`0;0?ooo`03
0000003oool0oooo0?l0ooooP03oool00200oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`3o0?oooh00oooo000P0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0o`3o
oomo0?ooo`00803oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo0?l0ooooOP3oool0
0200oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3o0?ooogd0oooo000P0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool0o`3ooomm0?ooo`00803oool00`000000oooo
0?ooo`0?0?ooo`<00000o`3oooml0?ooo`00803oool200000140oooo0P00003o0?ooog`0oooo000P
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0o`3ooomj0?ooo`00803oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0?l0ooooNP3oool00200oooo00<000000?ooo`3o
ool04P3oool00`000000oooo0?ooo`3o0?ooogT0oooo000P0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0o`3ooomh0?ooo`00803oool00`000000oooo0?ooo`0D0?ooo`030000003o
ool0oooo0?l0ooooM`3oool00200oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`3o
0?ooogH0oooo000P0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0o`3ooomf0?oo
o`00803oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0?l0ooooM@3oool00200oooo
00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3o0?ooog@0oooo000P0?ooo`030000003o
ool0oooo01P0oooo00<000000?ooo`3oool0o`3ooomc0?ooo`00803oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo0?l0ooooLP3oool00200oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`3o0?ooog80oooo000P0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool0o`3oooma0?ooo`00803oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0?l0oooo
L03oool00200oooo0P00000M0?ooo`030000003oool0oooo0?l0ooooK`3oool00200oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`3o0?ooofl0oooo000P0?ooo`030000003oool0oooo
01d0oooo00<000000?ooo`3oool0o`3ooom^0?ooo`00803oool00`000000oooo0?ooo`0N0?ooo`03
0000003oool0oooo0?l0ooooK@3oool00200oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`3o0?ooof`0oooo000P0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0o`3o
oom[0?ooo`00803oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0?l0ooooJP3oool0
0200oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`3o0?ooofT0oooo000P0?ooo`03
0000003oool0oooo0280oooo00<000000?ooo`3oool0o`3ooomY0?ooo`00803oool00`000000oooo
0?ooo`0S0?ooo`030000003oool0oooo0?l0ooooJ03oool00200oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`3o0?ooofL0oooo000P0?ooo`030000003oool0oooo02D0oooo00<00000
0?ooo`3oool0o`3ooomV0?ooo`00803oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo
0?l0ooooI@3oool00200oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`3o0?ooof@0
oooo000P0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0o`3ooomS0?ooo`00803o
ool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo0?l0ooooHP3oool00200oooo0P00000[
0?ooo`030000003oool0oooo0?l0ooooH@3oool00200oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`3o0?ooof40oooo000P0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool0o`3ooomP0?ooo`00803oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo0?l0oooo
G`3oool00200oooo00<000000?ooo`3oool0;@3oool200000?l0ooooG`3oool00200oooo00<00000
0?ooo`3oool0;@3oool200000?l0ooooG`3oool00200oooo00<000000?ooo`3oool0;`3oool00`00
0000oooo0?ooo`3o0?oooe`0oooo000P0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3o
ool0o`3ooomK0?ooo`00803oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0?l0oooo
FP3oool00200oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`3o0?oooeT0oooo000P
0?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0o`3ooomH0?ooo`00803oool00`00
0000oooo0?ooo`0d0?ooo`030000003oool0oooo0?l0ooooE`3oool00080oooo1P0000050?ooo`80
00001P3oool4000000L0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3o0?oooeH0
oooo00040?ooo`030000003oool0oooo00H0oooo0P0000080?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3o0?oooeH0oooo00040?ooo`030000003o
ool0oooo00`0oooo1P0000070?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0o`3o
oomE0?ooo`00103oool00`000000oooo0?ooo`0<0?ooo`050000003oool0oooo0?ooo`000000203o
ool3000003L0oooo00<000000?ooo`3oool0o`3ooomD0?ooo`00103oool00`000000oooo0?ooo`0=
0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?oo
o`3o0?oooe<0oooo00040?ooo`030000003oool0oooo00h0oooo00<000000?ooo`000000203oool0
0`000000oooo0?ooo`0i0?ooo`030000003oool0oooo0?l0ooooDP3oool000@0oooo00<000000?oo
o`3oool03P3oool00`000000oooo000000080?ooo`030000003oool0oooo03X0oooo00<000000?oo
o`3oool0o`3ooomA0?ooo`00103oool00`000000oooo0?ooo`0?0?ooo`800000203oool00`000000
oooo0?ooo`0k0?ooo`030000003oool0oooo0?l0ooooD03oool00080oooo0`00000B0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`3o0?ooodl0oooo
000P0?ooo`030000003oool0oooo03d0oooo00<000000?ooo`3oool0o`3ooom>0?ooo`00803oool0
0`000000oooo0?ooo`0n0?ooo`030000003oool0oooo0?l0ooooC@3oool00200oooo00<000000?oo
o`3oool0?`3oool00`000000oooo0?ooo`3o0?oood`0oooo000P0?ooo`030000003oool0oooo0400
oooo00<000000?ooo`3oool0o`3ooom;0?ooo`00803oool00`000000oooo0?ooo`110?ooo`800000
o`3ooom;0?ooo`00803oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo0?l0ooooB03o
ool00200oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`3o0?ooodL0oooo000P0?oo
o`030000003oool0oooo04D0oooo00<000000?ooo`3oool0o`3ooom60?ooo`00803oool00`000000
oooo0?ooo`160?ooo`030000003oool0oooo0?l0ooooA@3oool00200oooo0P0000180?ooo`030000
003oool0oooo0?l0ooooA03oool00200oooo00<000000?ooo`3oool0B03oool00`000000oooo0?oo
o`3o0?oood<0oooo000P0?ooo`030000003oool0oooo04T0oooo0P00003o0?oood<0oooo000P0?oo
o`030000003oool0oooo04/0oooo00<000000?ooo`3oool0o`3ooom00?ooo`00803oool00`000000
oooo0?ooo`1<0?ooo`030000003oool0oooo0?l0oooo?`3oool00200oooo00<000000?ooo`3oool0
C@3oool00`000000oooo0?ooo`3o0?oooch0oooo000P0?ooo`030000003oool0oooo04h0oooo00<0
00000?ooo`3oool0o`3ooolm0?ooo`00803oool00`000000oooo0?ooo`1?0?ooo`030000003oool0
oooo0?l0oooo?03oool00200oooo00<000000?ooo`3oool0D03oool200000?l0oooo?03oool00200
oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`3o0?ooocT0oooo000P0?ooo`030000
003oool0oooo05<0oooo00<000000?ooo`3oool0o`3ooolh0?ooo`00803oool00`000000oooo0?oo
o`1D0?ooo`030000003oool0oooo0?l0oooo=`3oool00200oooo00<000000?ooo`3oool0E@3oool0
0`000000oooo0?ooo`3o0?ooocH0oooo000P0?ooo`030000003oool0oooo05H0oooo0P00003o0?oo
ocH0oooo000P0?ooo`030000003oool0oooo05P0oooo00<000000?ooo`3oool0o`3ooolc0?ooo`00
803oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo0?l0oooo<P3oool00200oooo0P00
001K0?ooo`030000003oool0oooo0?l0oooo<@3oool00200oooo00<000000?ooo`3oool0F`3oool2
00000?l0oooo<@3oool00200oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`3o0?oo
obh0oooo000P0?ooo`030000003oool0oooo05h0oooo0P00003o0?ooobh0oooo000P0?ooo`030000
003oool0oooo05h0oooo0P00003o0?ooobh0oooo000P0?ooo`030000003oool0oooo0600oooo0P00
003o0?ooob`0oooo000P0?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0o`3ooolY
0?ooo`00803oool00`000000oooo0?ooo`1S0?ooo`030000003oool0oooo0?l0oooo:03oool00200
oooo00<000000?ooo`3oool0I03oool200000?l0oooo:03oool00200oooo00<000000?ooo`3oool0
IP3oool00`000000oooo0?ooo`3o0?ooobD0oooo000P0?ooo`030000003oool0oooo06L0oooo00<0
00000?ooo`3oool0o`3ooolT0?ooo`00803oool00`000000oooo0?ooo`1X0?ooo`800000o`3ooolT
0?ooo`00803oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo0?l0oooo8@3oool00200
oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`3o0?ooob00oooo000P0?ooo`030000
003oool0oooo06`0oooo0P00003o0?ooob00oooo000P0?ooo`800000K`3oool00`000000oooo0?oo
o`3o0?oooad0oooo000P0?ooo`030000003oool0oooo06l0oooo00<000000?ooo`3oool0o`3ooolL
0?ooo`00803oool00`000000oooo0?ooo`1`0?ooo`800000o`3ooolL0?ooo`00803oool00`000000
oooo0?ooo`1b0?ooo`030000003oool0oooo0?l0oooo6@3oool00200oooo00<000000?ooo`3oool0
L`3oool200000?l0oooo6@3oool00200oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?oo
o`3o0?oooaH0oooo000P0?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3oool0o`3ooolE
0?ooo`00803oool00`000000oooo0?ooo`1g0?ooo`800000o`3ooolE0?ooo`00803oool00`000000
oooo0?ooo`1i0?ooo`030000003oool0oooo0?l0oooo4P3oool00200oooo00<000000?ooo`3oool0
NP3oool200000?l0oooo4P3oool00200oooo00<000000?ooo`3oool0O03oool00`000000oooo0?oo
o`3o0?ooo`l0oooo000P0?ooo`030000003oool0oooo07d0oooo0P00003o0?ooo`l0oooo00020?oo
o`H000001@3oool2000000H0oooo0`0000080?ooo`030000003oool0oooo07l0oooo00<000000?oo
o`3oool0o`3oool<0?ooo`00103oool00`000000oooo0?ooo`060?ooo`8000001@3oool200000080
oooo0P0000060?ooo`030000003oool0oooo0800oooo0P00003o0?ooo``0oooo00040?ooo`030000
003oool0oooo00d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0880oooo00<000000?ooo`3oool0o`3oool90?ooo`00103oool00`000000oooo0?oo
o`0=0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool3000008<0oooo0P00
003o0?ooo`T0oooo00040?ooo`030000003oool0oooo00d0oooo0P0000020?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0Q@3oool200000?l0oooo1`3oool000@0oooo00<000000?oo
o`3oool03@3oool4000000P0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo0?ooo`3o0?oo
o`@0oooo00040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool02@3oool00`000000
oooo0?ooo`280?ooo`800000o`3oool40?ooo`00103oool00`000000oooo0?ooo`0>0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`3o0?ooo`40oooo
00020?ooo`<000004@3oool4000000H0oooo00<000000?ooo`3oool0R`3oool200000?l0oooo0@3o
ool00200oooo00<000000?ooo`3oool0S@3oool200000?h0oooo000P0?ooo`030000003oool0oooo
08l0oooo00<000000?ooo`3oool0n`3oool00200oooo00<000000?ooo`3oool0T03oool200000?/0
oooo000P0?ooo`030000003oool0oooo0980oooo00<000000?ooo`3oool0n03oool00200oooo00<0
00000?ooo`3oool0T`3oool200000?P0oooo000P0?ooo`030000003oool0oooo09D0oooo0P00003f
0?ooo`00803oool00`000000oooo0?ooo`2G0?ooo`030000003oool0oooo0?<0oooo000P0?ooo`03
0000003oool0oooo09P0oooo0P00003c0?ooo`00803oool00`000000oooo0?ooo`2J0?ooo`030000
003oool0oooo0?00oooo000P0?ooo`030000003oool0oooo09/0oooo0P00003`0?ooo`00803oool2
000009h0oooo0P00003^0?ooo`00803oool00`000000oooo0?ooo`2O0?ooo`030000003oool0oooo
0>/0oooo000P0?ooo`030000003oool0oooo0:00oooo0P00003[0?ooo`00803oool00`000000oooo
0?ooo`2R0?ooo`800000j@3oool00200oooo00<000000?ooo`3oool0Y03oool200000>L0oooo000P
0?ooo`030000003oool0oooo0:H0oooo0P00003U0?ooo`00803oool00`000000oooo0?ooo`2X0?oo
o`800000h`3oool00200oooo00<000000?ooo`3oool0ZP3oool200000>40oooo000P0?ooo`030000
003oool0oooo0:`0oooo0P00003O0?ooo`00803oool00`000000oooo0?ooo`2^0?ooo`800000g@3o
ool00200oooo00<000000?ooo`3oool0/03oool00`000000oooo0?ooo`3J0?ooo`00803oool00`00
0000oooo0?ooo`2a0?ooo`800000fP3oool00200oooo00<000000?ooo`3oool0/`3oool200000=P0
oooo000P0?ooo`030000003oool0oooo0;D0oooo0P00003F0?ooo`00803oool00`000000oooo0?oo
o`2g0?ooo`800000e03oool00200oooo0P00002j0?ooo`800000dP3oool00200oooo00<000000?oo
o`3oool0^`3oool200000=00oooo000P0?ooo`030000003oool0oooo0;d0oooo1000003<0?ooo`00
803oool00`000000oooo0?ooo`2o0?ooo`800000c03oool00200oooo00<000000?ooo`3oool0`03o
ool200000</0oooo000P0?ooo`030000003oool0oooo0<80oooo0P0000390?ooo`00803oool00`00
0000oooo0?ooo`340?ooo`800000a`3oool00200oooo00<000000?ooo`3oool0aP3oool200000<D0
oooo000P0?ooo`030000003oool0oooo0<P0oooo0P0000330?ooo`00803oool00`000000oooo0?oo
o`3:0?ooo`800000`@3oool00200oooo00<000000?ooo`3oool0c03oool200000;l0oooo000P0?oo
o`030000003oool0oooo0<h0oooo0`00002l0?ooo`00803oool00`000000oooo0?ooo`3A0?ooo`80
0000^P3oool00200oooo00<000000?ooo`3oool0d`3oool200000;P0oooo000P0?ooo`030000003o
ool0oooo0=D0oooo0P00002f0?ooo`00803oool00`000000oooo0?ooo`3G0?ooo`<00000/`3oool0
0200oooo0P00003K0?ooo`800000/@3oool00200oooo00<000000?ooo`3oool0g03oool200000:l0
oooo000P0?ooo`030000003oool0oooo0=h0oooo0`00002/0?ooo`00803oool00`000000oooo0?oo
o`3Q0?ooo`800000ZP3oool00200oooo00<000000?ooo`3oool0h`3oool300000:L0oooo000P0?oo
o`030000003oool0oooo0>H0oooo0P00002U0?ooo`00803oool00`000000oooo0?ooo`3X0?ooo`80
0000X`3oool00200oooo00<000000?ooo`3oool0jP3oool300000:00oooo000P0?ooo`030000003o
ool0oooo0>d0oooo0P00002N0?ooo`00803oool00`000000oooo0?ooo`3_0?ooo`<00000V`3oool0
0200oooo00<000000?ooo`3oool0lP3oool2000009T0oooo00020?ooo`H000001@3oool2000000H0
oooo100000070?ooo`030000003oool0oooo0?@0oooo0P00002G0?ooo`00103oool00`000000oooo
0?ooo`060?ooo`8000001@3oool010000000oooo0?ooo`3oool2000000H0oooo00<000000?ooo`3o
ool0mP3oool2000009D0oooo00040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?P0oooo0`00002B0?ooo`00103o
ool00`000000oooo0?ooo`0=0?ooo`040000003oool0oooo0?ooo`8000001P3oool00`000000oooo
0?ooo`3k0?ooo`800000T03oool000@0oooo00<000000?ooo`3oool03P3oool3000000P0oooo0`00
003m0?ooo`800000SP3oool000@0oooo00<000000?ooo`3oool03@3oool200000080oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`3o0?ooo`<00000R`3oool000@0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0o`3oool30?ooo`800000R@3oool000@0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool50?ooo`<0
0000QP3oool00080oooo0`00000@0?ooo`@000001`3oool00`000000oooo0?ooo`3o0?ooo`P0oooo
0`0000230?ooo`00803oool00`000000oooo0?ooo`3o0?ooo`/0oooo0P0000210?ooo`00803oool0
0`000000oooo0?ooo`3o0?ooo`d0oooo0`00001n0?ooo`00803oool00`000000oooo0?ooo`3o0?oo
oa00oooo0`00001k0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo0`00001h0?ooo`00
803oool00`000000oooo0?ooo`3o0?oooaH0oooo0`00001e0?ooo`00803oool00`000000oooo0?oo
o`3o0?oooaT0oooo0P00001c0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa/0oooo0`00001`
0?ooo`00803oool00`000000oooo0?ooo`3o0?oooah0oooo1@00001[0?ooo`00803oool00`000000
oooo0?ooo`3o0?ooob40oooo0`00001Z0?ooo`00803oool00`000000oooo0?ooo`3o0?ooob@0oooo
0`00001W0?ooo`00803oool200000?l0oooo:03oool3000006@0oooo000P0?ooo`030000003oool0
oooo0?l0oooo:P3oool400000600oooo000P0?ooo`030000003oool0oooo0?l0oooo;P3oool30000
05d0oooo000P0?ooo`030000003oool0oooo0?l0oooo<@3oool3000005X0oooo000P0?ooo`030000
003oool0oooo0?l0oooo=03oool3000005L0oooo000P0?ooo`030000003oool0oooo0?l0oooo=`3o
ool3000005@0oooo000P0?ooo`030000003oool0oooo0?l0oooo>P3oool200000580oooo000P0?oo
o`030000003oool0oooo0?l0oooo?03oool3000004l0oooo000P0?ooo`030000003oool0oooo0?l0
oooo?`3oool3000004`0oooo000P0?ooo`030000003oool0oooo0?l0oooo@P3oool3000004T0oooo
000P0?ooo`030000003oool0oooo0?l0ooooA@3oool3000004H0oooo000P0?ooo`030000003oool0
oooo0?l0ooooB03oool3000004<0oooo000P0?ooo`030000003oool0oooo0?l0ooooB`3oool40000
03l0oooo000P0?ooo`030000003oool0oooo0?l0ooooC`3oool3000003`0oooo000P0?ooo`030000
003oool0oooo0?l0ooooDP3oool3000003T0oooo000P0?ooo`030000003oool0oooo0?l0ooooE@3o
ool3000003H0oooo000P0?ooo`800000o`3ooomI0?ooo`<00000<`3oool00200oooo00<000000?oo
o`3oool0o`3ooomK0?ooo`@00000;`3oool00200oooo00<000000?ooo`3oool0o`3ooomO0?ooo`<0
0000;03oool00200oooo00<000000?ooo`3oool0o`3ooomR0?ooo`<00000:@3oool00200oooo00<0
00000?ooo`3oool0o`3ooomU0?ooo`<000009P3oool00200oooo00<000000?ooo`3oool0o`3ooomX
0?ooo`<000008`3oool00200oooo00<000000?ooo`3oool0o`3ooom[0?ooo`@000007`3oool00200
oooo00<000000?ooo`3oool0o`3ooom_0?ooo`<00000703oool00200oooo00<000000?ooo`3oool0
o`3ooomb0?ooo`<000006@3oool00200oooo00<000000?ooo`3oool0o`3ooome0?ooo`@000005@3o
ool00200oooo00<000000?ooo`3oool0o`3ooomi0?ooo`@000004@3oool00200oooo00<000000?oo
o`3oool0o`3ooomm0?ooo`@000000P3oool2000000T0oooo000P0?ooo`030000003oool0oooo0?l0
ooooP@3oool4000000T0oooo000P0?ooo`030000003oool0oooo0?l0ooooSP3oool00200oooo00<0
00000?ooo`3oool0o`3ooon>0?ooo`00803oool200000?l0ooooS`3oool00200oooo00<000000?oo
o`3oool0o`3ooon>0?ooo`00803oool00`000000oooo0?ooo`3o0?ooohh0oooo000P0?ooo`030000
003oool0oooo0?l0ooooSP3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0
oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {6.69962, 1.1149, 0.154179, \
0.00488813}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fitting] /. k \[Rule] 10\)], "Input"],

Cell[BoxData[
    \({0.5795163981006775`, 0.12866503074124358`, \(-0.3963108591423432`\), 
      0.8444854609274249`}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Number distribution", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(aa = Drop[Transpose[Map[number, klist]], 2]\)], "Input"],

Cell[BoxData[
    \({{0.027518342999`, 0.030854945552`, 0.034940537584`, 0.039974954459`, 
        0.046137612785`, 0.049606198665`, 0.051722161963`}, {0.433154488295`, 
        0.421318604453`, 0.409977154441`, 0.401095000018`, 0.399547863297`, 
        0.405021970235`, 0.412013288664`}, {0.392091159363`, 0.378911710894`, 
        0.36211457224`, 0.339655210695`, 0.307078539666`, 0.284017683363`, 
        0.266636641749`}, {0.147236009344`, 0.1689147391`, 0.192967735735`, 
        0.219274834827`, 0.247235984252`, 0.261354147736`, 
        0.269627907624`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListPlot3D[aa, ViewPoint -> {2.3, 1. , 0.3}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .43956 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-7.19368e-17 1.10285 -0.276874 1.10285 [
[.34727 .45031 -3.03851 0 ]
[.34727 .45031 2.96149 9 ]
[.23163 .44321 -3.06035 0 ]
[.23163 .44321 2.93965 9 ]
[.08669 .43432 -3.08775 0 ]
[.08669 .43432 2.91225 9 ]
[.01191 .07251 -2.89576 -9 ]
[.01191 .07251 3.10424 0 ]
[.24495 .04726 -2.94069 -9 ]
[.24495 .04726 3.05931 0 ]
[.50831 .01872 -2.99164 -9 ]
[.50831 .01872 3.00836 0 ]
[.80833 -0.01378 -3.04991 -9 ]
[.80833 -0.01378 2.95009 0 ]
[-0.00255 .09542 -6 -4.02189 ]
[-0.00255 .09542 0 4.97811 ]
[-0.00505 .16638 -18 -4.10557 ]
[-0.00505 .16638 0 4.89443 ]
[-0.00756 .23799 -18 -4.19013 ]
[-0.00756 .23799 0 4.80987 ]
[-0.0101 .31026 -18 -4.27558 ]
[-0.0101 .31026 0 4.72442 ]
[-0.01265 .3832 -18 -4.36194 ]
[-0.01265 .3832 0 4.63806 ]
[ 0 0 0 0 ]
[ 1 .43956 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.39688 .43956 m
0 .41519 L
s
.34745 .43652 m
.34754 .42963 L
s
[(2)] .34727 .45031 .01284 -1 Mshowa
.2319 .42943 m
.23204 .42254 L
s
[(4)] .23163 .44321 .02012 -1 Mshowa
.08709 .42054 m
.08729 .41365 L
s
[(6)] .08669 .43432 .02925 -1 Mshowa
.125 Mabswid
.32081 .43489 m
.32087 .43075 L
s
.29275 .43316 m
.29282 .42903 L
s
.26316 .43135 m
.26323 .42721 L
s
.19884 .4274 m
.19893 .42326 L
s
.16381 .42525 m
.16391 .42111 L
s
.12663 .42297 m
.12674 .41883 L
s
.37278 .43808 m
.37282 .43394 L
s
.04497 .41795 m
.0451 .41382 L
s
.25 Mabswid
.01143 .08629 m
.80856 0 L
s
.01143 .08629 m
.01119 .09318 L
s
[(1)] .01191 .07251 -0.03475 1 Mshowa
.24468 .06104 m
.24454 .06793 L
s
[(2)] .24495 .04726 -0.01977 1 Mshowa
.50827 .03251 m
.50826 .0394 L
s
[(3)] .50831 .01872 -0.00279 1 Mshowa
.80856 0 m
.80867 .00689 L
s
[(4)] .80833 -0.01378 .01664 1 Mshowa
.125 Mabswid
.0559 .08147 m
.05577 .08561 L
s
.10143 .07655 m
.10131 .08068 L
s
.14804 .0715 m
.14793 .07563 L
s
.19577 .06633 m
.19568 .07047 L
s
.29479 .05562 m
.29472 .05975 L
s
.34615 .05005 m
.3461 .05419 L
s
.39882 .04435 m
.39878 .04849 L
s
.45285 .03851 m
.45282 .04264 L
s
.56516 .02635 m
.56517 .03048 L
s
.62357 .02002 m
.62359 .02416 L
s
.68357 .01353 m
.6836 .01767 L
s
.74521 .00686 m
.74526 .01099 L
s
.25 Mabswid
.01143 .08629 m
0 .41519 L
s
.01116 .09396 m
.01802 .09323 L
s
[(0)] -0.00255 .09542 1 -0.10625 Mshowa
.00869 .16517 m
.01555 .16457 L
s
[(0.1)] -0.00505 .16638 1 -0.08765 Mshowa
.00619 .23704 m
.01307 .23657 L
s
[(0.2)] -0.00756 .23799 1 -0.06886 Mshowa
.00367 .30957 m
.01055 .30923 L
s
[(0.3)] -0.0101 .31026 1 -0.04987 Mshowa
.00113 .38278 m
.00802 .38257 L
s
[(0.4)] -0.01265 .3832 1 -0.03068 Mshowa
.125 Mabswid
.01067 .10815 m
.01478 .10773 L
s
.01017 .12237 m
.01429 .12196 L
s
.00968 .13661 m
.0138 .13622 L
s
.00918 .15088 m
.0133 .1505 L
s
.00819 .17949 m
.01231 .17915 L
s
.00769 .19384 m
.01181 .19351 L
s
.00719 .20822 m
.01132 .2079 L
s
.00669 .22262 m
.01082 .22232 L
s
.00569 .25149 m
.00982 .25123 L
s
.00518 .26597 m
.00931 .26572 L
s
.00468 .28048 m
.00881 .28024 L
s
.00418 .29501 m
.00831 .29479 L
s
.00316 .32416 m
.00729 .32397 L
s
.00266 .33878 m
.00679 .3386 L
s
.00215 .35342 m
.00628 .35326 L
s
.00164 .36808 m
.00577 .36794 L
s
.00061 .3975 m
.00475 .39739 L
s
.0001 .41225 m
.00424 .41216 L
s
.25 Mabswid
.39918 .2038 m
.39688 .43956 L
s
.39688 .43956 m
1 .43094 L
s
1 .43094 m
.99241 .16207 L
s
.99241 .16207 m
.39918 .2038 L
s
.01143 .08629 m
.80856 0 L
s
.80856 0 m
.81517 .39712 L
s
.81517 .39712 m
0 .41519 L
s
0 .41519 m
.01143 .08629 L
s
.39918 .2038 m
.39688 .43956 L
s
.39688 .43956 m
0 .41519 L
s
0 .41519 m
.01143 .08629 L
s
.01143 .08629 m
.39918 .2038 L
s
.99241 .16207 m
.80856 0 L
s
.80856 0 m
.81517 .39712 L
s
.81517 .39712 m
1 .43094 L
s
1 .43094 m
.99241 .16207 L
s
0 0 m
1 0 L
1 .43956 L
0 .43956 L
closepath
clip
newpath
.5 Mabswid
.823 .347 .054 r
.39899 .2234 .58036 .43099 .53837 .42053 .35034 .21144 Metetra
.812 .329 .037 r
.35034 .21144 .53837 .42053 .49174 .40922 .29657 .19864 Metetra
.801 .311 .019 r
.29657 .19864 .49174 .40922 .43964 .39817 .23682 .18498 Metetra
.789 .293 .003 r
.23682 .18498 .43964 .39817 .38099 .39075 .17003 .17044 Metetra
.779 .276 0 r
.17003 .17044 .38099 .39075 .31444 .38746 .09494 .15227 Metetra
.78 .277 0 r
.09494 .15227 .31444 .38746 .23827 .38489 .00988 .13071 Metetra
.322 .794 .923 r
.58036 .43099 .78018 .40434 .74718 .39113 .53837 .42053 Metetra
.323 .795 .922 r
.53837 .42053 .74718 .39113 .71034 .37408 .49174 .40922 Metetra
.326 .8 .915 r
.49174 .40922 .71034 .37408 .66894 .35112 .43964 .39817 Metetra
.33 .809 .893 r
.43964 .39817 .66894 .35112 .62214 .31792 .38099 .39075 Metetra
.393 .85 .886 r
.38099 .39075 .62214 .31792 .56888 .28741 .31444 .38746 Metetra
.47 .891 .897 r
.31444 .38746 .56888 .28741 .50765 .25701 .23827 .38489 Metetra
.721 .978 .916 r
.78018 .40434 .99502 .25448 .97351 .25386 .74718 .39113 Metetra
.686 .965 .937 r
.74718 .39113 .97351 .25386 .94937 .25473 .71034 .37408 Metetra
.641 .942 .96 r
.71034 .37408 .94937 .25473 .92207 .25731 .66894 .35112 Metetra
.584 .902 .984 r
.66894 .35112 .92207 .25731 .89091 .26151 .62214 .31792 Metetra
.434 .842 .972 r
.62214 .31792 .89091 .26151 .85476 .2551 .56888 .28741 Metetra
.318 .78 .943 r
.56888 .28741 .85476 .2551 .81259 .24249 .50765 .25701 Metetra
0 g
.25 Mabswid
.01143 .08629 m
.80856 0 L
s
.80856 0 m
.81517 .39712 L
s
.81517 .39712 m
0 .41519 L
s
0 .41519 m
.01143 .08629 L
s
.99241 .16207 m
.80856 0 L
s
.80856 0 m
.81517 .39712 L
s
.81517 .39712 m
1 .43094 L
s
1 .43094 m
.99241 .16207 L
s
.01143 .08629 m
0 .41519 L
s
.01116 .09396 m
.01802 .09323 L
s
[(0)] -0.00255 .09542 1 -0.10625 Mshowa
.00869 .16517 m
.01555 .16457 L
s
[(0.1)] -0.00505 .16638 1 -0.08765 Mshowa
.00619 .23704 m
.01307 .23657 L
s
[(0.2)] -0.00756 .23799 1 -0.06886 Mshowa
.00367 .30957 m
.01055 .30923 L
s
[(0.3)] -0.0101 .31026 1 -0.04987 Mshowa
.00113 .38278 m
.00802 .38257 L
s
[(0.4)] -0.01265 .3832 1 -0.03068 Mshowa
.125 Mabswid
.01067 .10815 m
.01478 .10773 L
s
.01017 .12237 m
.01429 .12196 L
s
.00968 .13661 m
.0138 .13622 L
s
.00918 .15088 m
.0133 .1505 L
s
.00819 .17949 m
.01231 .17915 L
s
.00769 .19384 m
.01181 .19351 L
s
.00719 .20822 m
.01132 .2079 L
s
.00669 .22262 m
.01082 .22232 L
s
.00569 .25149 m
.00982 .25123 L
s
.00518 .26597 m
.00931 .26572 L
s
.00468 .28048 m
.00881 .28024 L
s
.00418 .29501 m
.00831 .29479 L
s
.00316 .32416 m
.00729 .32397 L
s
.00266 .33878 m
.00679 .3386 L
s
.00215 .35342 m
.00628 .35326 L
s
.00164 .36808 m
.00577 .36794 L
s
.00061 .3975 m
.00475 .39739 L
s
.0001 .41225 m
.00424 .41216 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 126.562},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`0000_B000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooomB0?ooo`@00000F`3oool00?l0ooooE03oool00`000000oooo0?ooo`1J0?ooo`00
o`3ooom@0?ooo`H00000F`3oool00?l0ooooD03oool01@000000oooo0?ooo`3oool0000005`0oooo
003o0?oooe40oooo00@000000?ooo`3oool00000G03oool00?l0ooooDP3oool00`000000oooo0000
001L0?ooo`00o`3ooomB0?ooo`030000003oool0000005`0oooo003o0?oooe<0oooo0P00001L0?oo
o`00o`3ooomD0?ooo`030000003oool0oooo05X0oooo003o0?oook40oooo003o0?oook40oooo003o
0?oook40oooo003V0?ooo`D00000a@3oool00>/0oooo00<000000?ooo`3oool0`P3oool00>/0oooo
00<000000?ooo`3oool0`P3oool00>/0oooo00<000000?ooo`3oool0`P3oool00>X0oooo0P000034
0?ooo`00j@3oool2000006<0oooo1@00001M0?ooo`00j`3oool00`000000oooo0?ooo`1G0?ooo`T0
0000103oool2000005`0oooo003[0?ooo`030000003oool0oooo04d0oooo2P00000=0?ooo`030000
003oool0000005/0oooo003W0?ooo`@00000A`3oool9000000030?ooo`000000oooo01@0oooo00@0
00000?ooo`3oool00000FP3oool008X0oooo1P00002I0?ooo`T000002P3oool00`000000oooo0?oo
o`0C0?ooo`040000003oool0oooo0?ooo`800000F03oool008/0oooo00D000000?ooo`3oool0oooo
0000002@0?ooo`T00000:@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo05D0oooo
002<0?ooo`030000003oool0oooo08L0oooo2P0000070?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`1D0?ooo`00S@3oool00`000000oooo0?ooo`1m
0?ooo`T00000?03oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo05<0oooo002>0?oo
o`030000003oool0oooo07<0oooo2@0000050?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`1B0?ooo`00S`3oool00`000000oooo0?ooo`1X0?ooo`X0
0000CP3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0540oooo002?0?ooo`030000
003oool0oooo05l0oooo2@0000030?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`1@0?ooo`00RP3oool00`000000oooo0?ooo`020?ooo`030000003o
ool0oooo05H0oooo2@00000<0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool02@3o
ool200000500oooo002:0?ooo`D00000D03oool900000080oooo00<000000?ooo`3oool0I@3oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo04d0oooo000j0?ooo`H00000U@3oool:0000
00/0oooo00<000000?ooo`3oool0I@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
04`0oooo000l0?ooo`030000003oool0oooo08d0oooo2@0000000`3oool000000?ooo`0B0?ooo`03
0000003oool0oooo06D0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`1;0?ooo`00
?03oool00`000000oooo0?ooo`240?ooo`T000002P3oool00`000000oooo0?ooo`1i0?ooo`030000
003oool0oooo00h0oooo00<000000?ooo`3oool0BP3oool003`0oooo00<000000?ooo`3oool0N`3o
ool:000008h0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`190?ooo`00?03oool0
0`000000oooo0?ooo`1a0?ooo`X000002@3oool00`000000oooo0?ooo`2<0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0B03oool003`0oooo00<000000?ooo`3oool0J03oool90000
00030?ooo`000000oooo09l0oooo00<000000?ooo`3oool04@3oool2000004P0oooo000l0?ooo`03
0000003oool0oooo05l0oooo2@00000:0?ooo`030000003oool0oooo09h0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`150?ooo`00?03oool00`000000oooo0?ooo`1F0?ooo`T00000
00<0oooo0000003oool0/@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo04@0oooo
000j0?ooo`<00000CP3oool:000000X0oooo00<000000?ooo`3oool0/03oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo04<0oooo00220?ooo`T000000`3oool00`000000oooo0?ooo`31
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0@P3oool007T0oooo2@00000<0?oo
o`030000003oool0oooo0<40oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`110?oo
o`00K`3oool:000000@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`320?ooo`03
0000003oool0oooo01L0oooo00<000000?ooo`3oool0@03oool006H0oooo2@00000>0?ooo`030000
003oool0oooo0=<0oooo00<000000?ooo`3oool0603oool200000400oooo001M0?ooo`T000001P3o
ool00`000000oooo0?ooo`3T0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0?@3o
ool005@0oooo2@00003f0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0?03oool0
04X0oooo2P0000080?ooo`030000003oool0oooo0?@0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0k0?ooo`00@@3oool900000080oooo00<000000?ooo`3oool0o`3oool50?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool0>P3oool003`0oooo1@00000;0?ooo`030000
003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo03T0oooo
000a0?ooo`<00000203oool00`000000oooo0000000200000?l0oooo4`3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo03P0oooo000`0?ooo`8000000P3oool00`000000oooo0?ooo`05
0?ooo`050000003oool000000000003oool01000003o0?ooo`l0oooo00<000000?ooo`3oool0803o
ool2000003P0oooo000`0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool2
000000L0oooo0`00003o0?ooo``0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0e
0?ooo`00<03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool02@3oool300000?l0oooo2@3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
03@0oooo000`0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`0<0?ooo`@00000o`3oool50?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3o
ool0<`3oool00300oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0100oooo0`00003o0?ooo`80oooo00<000000?ooo`3oool09@3oool00`000000oooo
0?ooo`0b0?ooo`00<03oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool04`3oool300000?h0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`0a0?ooo`00<03oool01@000000oooo0?ooo`3oool0000000L0oooo0`00000F0?ooo`@00000nP3o
ool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo0300oooo000a0?ooo`<00000203oool0
0`000000oooo0?ooo`0J0?ooo`<00000m`3oool00`000000oooo0?ooo`0X0?ooo`800000<03oool0
03`0oooo00<000000?ooo`3oool07@3oool300000?@0oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0]0?ooo`00?03oool00`000000oooo0?ooo`0P0?ooo`@00000l03oool00`000000
oooo0?ooo`0[0?ooo`030000003oool0oooo02`0oooo000l0?ooo`030000003oool0oooo02@0oooo
0`00003]0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0:`3oool003`0oooo0P00
000X0?ooo`<00000jP3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02X0oooo000l
0?ooo`030000003oool0oooo02X0oooo1000003V0?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool0:@3oool003/0oooo00<000000?ooo`3oool0;`3oool300000><0oooo00<000000?oo
o`3oool0;`3oool00`000000oooo0?ooo`0X0?ooo`00>`3oool400000340oooo0`00003P0?ooo`03
0000003oool0oooo0300oooo0P00000X0?ooo`00>`3oool010000000oooo00000037AP0400000300
oooo1000003L0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool09@3oool003/0oooo
0P0000000`3oool000000<M600030<M600@00000<03oool300000=T0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`0T0?ooo`00>`3oool01@000000oooo0?ooo`3oool0000000L0adH0
1000000_0?ooo`<00000eP3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo02<0oooo
000k0?ooo`030000003oool0oooo0080oooo00<000000<M60037AP002037AP04000002h0oooo0`00
003C0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool08P3oool003/0oooo00<00000
0?ooo`3oool00P3oool00`000000adH00<M6000<0<M600@00000;@3oool400000<l0oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0Q0?ooo`00>`3oool00`000000oooo000000030?oo
o`0300000037AP00adH000l0adH01000000]0?ooo`<00000c03oool00`000000oooo0?ooo`0g0?oo
o`030000003oool0oooo0200oooo000k0?ooo`8000001@3oool00`000000adH00<M6000B0<M600D0
0000:`3oool300000<T0oooo00<000000?ooo`3oool0>03oool200000200oooo000k0?ooo`030000
003oool0oooo00D0oooo00<000000<M60037AP005037AP000`000000adH000000003000002X0oooo
100000350?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool07@3oool001l0oooo0`00
00070?ooo`8000001@3oool6000000D0oooo00<000000?ooo`3oool01P3oool00`000000adH00<M6
000D0<M6000300000037AP00adH00080adH01@00000Y0?ooo`<00000`P3oool00`000000oooo0?oo
o`0k0?ooo`030000003oool0oooo01`0oooo000N0?ooo`8000000P3oool00`000000oooo0?ooo`04
0?ooo`8000001`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo00<00000
0<M60037AP005037AP000`000000adH00<M600060<M600@00000:03oool300000;l0oooo00<00000
0?ooo`3oool0=P3oool7000001d0oooo000N0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`060?ooo`040000003oool00000000000L0oooo00<00000
0<M60037AP005037AP000`000000adH00<M600090<M600D000009P3oool400000;/0oooo00<00000
0?ooo`3oool0:03oool>000000H0oooo00<000000?ooo`3oool06`3oool001h0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo0P00000:0?oo
o`0300000037AP00adH001@0adH000<000000<M60037AP003@37AP04000002H0oooo0`00002h0?oo
o`030000003oool0oooo01X0oooo3P00000D0?ooo`030000003oool0oooo01/0oooo000N0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo00X0oooo00<000000<M60037AP005037AP000`000000adH00<M6000@0<M600D0
0000903oool300000;D0oooo00<000000?ooo`3oool0303oool>00000280oooo00<000000?ooo`3o
ool06`3oool001h0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool02`3oool00`000000adH00<M6000D0<M600030000
0037AP00adH00180adH000<000000<U;000000000`00000S0?ooo`@00000/@3oool00`000000oooo
0000000<00000300oooo00<000000?ooo`3oool06`3oool001h0oooo00<000000?ooo`3oool00P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool02`3o
ool00`000000adH00<M6000E0<M6000300000037AP00adH00180adH000<000000<U;0039B`000P39
B`0500000280oooo0`00002R0?ooo`h00000?@3oool00`000000oooo0?ooo`0K0?ooo`007P3oool0
1@000000oooo0?ooo`3oool0000000l0oooo00<000000?ooo`3oool01P3oool3000000`0oooo00<0
00000<M60037AP005037AP000`000000adH00<M6000C0<M6000300000039B`00bD/000H0bD/01@00
000P0?ooo`<00000T@3oool>000000`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?oo
o`0K0?ooo`007`3oool3000000h0oooo0`0000080?ooo`030000003oool0oooo00d0oooo00<00000
0<M60037AP005037AP000`000000adH00<M6000C0<M6000300000039B`00bD/000X0bD/01000000O
0?ooo`@00000O`3oool>000001X0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0K
0?ooo`00>`3oool00`000000oooo0?ooo`0>0?ooo`0300000037AP00adH001@0adH000<000000<M6
0037AP004`37AP000`000000bD/00<U;000=0<U;00D000007P3oool3000006l0oooo3@00000X0?oo
o`030000003oool0oooo03`0oooo00<000000?ooo`3oool06`3oool003/0oooo00<000000?ooo`3o
ool03`3oool00`000000adH00<M6000D0<M6000300000037AP00adH001<0adH000<000000<U;0039
B`003`39B`000`000000cDl400000003000001d0oooo0`00001N0?ooo`h00000=@3oool00`000000
oooo0?ooo`0l0?ooo`030000003oool0oooo01/0oooo000k0?ooo`030000003oool0oooo0100oooo
00<000000<M60037AP005037AP000`000000adH00<M6000C0<M6000300000039B`00bD/000l0bD/0
00<000000<e?103=C`@00P3=C`@4000001`0oooo1000001<0?ooo`h00000@`3oool00`000000oooo
0?ooo`0l0?ooo`030000003oool0oooo01/0oooo000k0?ooo`8000004P3oool00`000000adH00<M6
000D0<M6000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`000`000000cDl40<e?
10050<e?10@00000703oool3000003`0oooo3@00001A0?ooo`030000003oool0oooo03`0oooo00<0
00000?ooo`3oool06`3oool003/0oooo00<000000?ooo`3oool04P3oool00`000000adH00<M6000D
0<M6000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`000`000000cDl40<e?1008
0<e?10@000006`3oool3000002/0oooo3P00001N0?ooo`030000003oool0oooo03`0oooo00<00000
0?ooo`3oool06`3oool003/0oooo00<000000?ooo`3oool04`3oool00`000000adH00<M6000D0<M6
000300000037AP00adH00180adH000<000000<U;0039B`004039B`000`000000cDl40<e?100;0<e?
10@000006P3oool4000001T0oooo3P00001/0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool06`3oool003/0oooo00<000000?ooo`3oool0503oool00`000000adH00<M6000D0<M60003
00000037AP00adH00180adH000<000000<U;0039B`004039B`000`000000cDl40<e?100=0<e?10D0
00006P3oool3000000P0oooo3P00001k0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3o
ool06`3oool003/0oooo00<000000?ooo`3oool0503oool00`000000adH00<M6000E0<M600030000
0037AP00adH00180adH000<000000<U;0039B`004039B`000`000000cDl40<e?100=0<e?10040000
003?E0T0ce@90<mD2@@000006@3oool8000008T0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`0K0?ooo`00>`3oool2000001H0oooo00<000000<M60037AP005@37AP000`000000adH0
0<M6000B0<M6000300000039B`00bD/00100bD/000<000000<e?103=C`@03@3=C`@00`000000ce@9
0<mD2@040<mD2@@000005P3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo03`0oooo
00<000000?ooo`3oool06P3oool003/0oooo00<000000?ooo`3oool05P3oool00`000000adH00<M6
000E0<M6000300000037AP00adH00180adH000<000000<U;0039B`004039B`000`000000cDl40<e?
100=0<e?10030000003?E0T0ce@900L0ce@91000000B0?ooo`030000003oool0oooo08d0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0H
0?ooo`0300000037AP00adH001D0adH000<000000<M60037AP004P37AP000`000000bD/00<U;000?
0<U;00030000003=C`@0cDl400h0cDl400<000000<mD2@3?E0T02P3?E0T3000000l0oooo00<00000
0?ooo`3oool0S@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01X0oooo000j0?oo
o`030000003oool0oooo01T0oooo00<000000<M60037AP005@37AP000`000000adH00<M6000B0<M6
000300000039B`00bD/000l0bD/000<000000<e?103=C`@03P3=C`@00`000000ce@90<mD2@0;0<mD
2@D000002`3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo03`0oooo00<000000?oo
o`3oool06P3oool003X0oooo00<000000?ooo`3oool06P3oool00`000000adH00<M6000E0<M60003
00000037AP00adH00180adH000<000000<U;0039B`003`39B`000`000000cDl40<e?100=0<e?1003
0000003?E0T0ce@900`0ce@900@000000=9H3@3BF0d0dUP=100000070?ooo`030000003oool0oooo
08d0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000
oooo0?ooo`0K0?ooo`0300000037AP00adH001D0adH000<000000<M60037AP004P37AP000`000000
bD/00<U;000?0<U;00030000003=C`@0cDl400d0cDl400<000000<mD2@3?E0T0303?E0T00`000000
dUP=0=9H3@040=9H3@@000000`3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo03`0
oooo00<000000?ooo`3oool06P3oool003X0oooo0`00000L0?ooo`0300000037AP00adH001@0adH0
00<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0cDl400d0cDl4
00<000000<mD2@3?E0T0303?E0T00`000000dUP=0=9H3@070=9H3@@00000S`3oool00`000000oooo
0?ooo`0l0?ooo`030000003oool0oooo01X0oooo000j0?ooo`030000003oool0oooo01d0oooo00<0
00000<M60037AP005037AP000`000000adH00<M6000C0<M6000300000039B`00bD/000l0bD/000<0
00000<e?103=C`@03@3=C`@00`000000ce@90<mD2@0;0<mD2@030000003BF0d0dUP=00X0dUP=00<0
00000?ooo`3oool0S@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01X0oooo000N
0?ooo`<000001`3oool2000000@0oooo1P0000060?ooo`030000003oool0oooo01h0oooo00<00000
0<M60037AP005037AP000`000000adH00<M6000C0<M6000300000039B`00bD/000l0bD/000<00000
0<e?103=C`@03@3=C`@00`000000ce@90<mD2@0;0<mD2@030000003BF0d0dUP=00X0dUP=00<00000
0?ooo`3oool0S03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo01X0oooo000M0?oo
o`8000000P3oool00`000000oooo0?ooo`040?ooo`8000001@3oool01@000000oooo0?ooo`3oool0
000000H0oooo00<000000?ooo`3oool07P3oool00`000000adH00<M6000E0<M6000300000037AP00
adH001<0adH000<000000<U;0039B`003`39B`000`000000cDl40<e?100=0<e?10030000003?E0T0
ce@900/0ce@900<000000=9H3@3BF0d02P3BF0d00`000000oooo0?ooo`2;0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool06P3oool001d0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo00L0oooo00@000000?ooo`00000000007P3oool0
0`000000adH00<M6000E0<M6000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`00
0`000000cDl40<e?100=0<e?10030000003?E0T0ce@900/0ce@900<000000=9H3@3BF0d02P3BF0d0
0`000000oooo0?ooo`2:0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool06P3oool0
01d0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo
00H0oooo0P00000Q0?ooo`0300000037AP00adH001D0adH000<000000<M60037AP004`37AP000`00
0000bD/00<U;000?0<U;00030000003=C`@0cDl400d0cDl400<000000<mD2@3?E0T02`3?E0T00`00
0000dUP=0=9H3@090=9H3@030000003oool0oooo08X0oooo00<000000?ooo`3oool0?03oool00`00
0000oooo0?ooo`0J0?ooo`007@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00d0
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Q0?ooo`0300000037AP00adH001D0
adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0cDl400d0
cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@090=9H3@030000003oool0oooo08T0
oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0J0?ooo`007@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0R0?ooo`0300000037AP00adH001D0adH000<000000<M60037AP004`37AP000`000000bD/0
0<U;000?0<U;00030000003=C`@0cDl400d0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=
0=9H3@090=9H3@030000003oool0oooo07h0oooo3000000m0?ooo`030000003oool0oooo01X0oooo
000M0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo02<0oooo00<000000<M60037AP005@37AP000`000000adH00<M6
000C0<M6000300000039B`00bD/000l0bD/000<000000<e?103=C`@03@3=C`@00`000000ce@90<mD
2@0;0<mD2@030000003BF0d0dUP=00T0dUP=00<000000?ooo`3oool0I`3ooolF000000X0DLOa00<0
00000577l@0000000`00000i0?ooo`030000003oool0oooo01X0oooo000M0?ooo`050000003oool0
oooo0?ooo`000000303oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0903oool00`000000adH00<M6000E0<M6000300000037AP00adH001<0adH000<0
00000<U;0039B`003`39B`000`000000cDl40<e?100=0<e?10030000003?E0T0ce@900/0ce@900<0
00000=9H3@3BF0d02@3BF0d00`000000oooo0?ooo`1A0?oooaD00000801Aao400`000000DLOa0577
l@030577l@<00000=P3oool00`000000oooo0?ooo`0J0?ooo`007P3oool3000000d0oooo1@000007
0?ooo`<000009@3oool00`000000adH00<M6000E0<M6000300000037AP00adH00180adH000<00000
0<U;0039B`004039B`000`000000cDl40<e?100<0<e?10030000003?E0T0ce@900/0ce@900<00000
0=9H3@3BF0d02P3BF0d00`000000oooo0?ooo`0k0?oooaD00000=@1Aao400`000000DLOa0577l@06
0577l@@00000<P3oool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0V0?oo
o`0300000037AP00adH001D0adH000<000000<M60037AP004P37AP000`000000bD/00<U;000@0<U;
00030000003=C`@0cDl400`0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@0:0=9H
3@030000003oool0oooo02@0oooo5P00001:0577l@030000001Aao40DLOa00L0DLOa2000000I0?oo
oaD00000703oool003X0oooo00<000000?ooo`3oool09P3oool00`000000adH00<M6000F0<M60003
00000037AP00adH00180adH000<000000<U;0039B`004039B`000`000000cDl40<e?100<0<e?1003
0000003?E0T0ce@900/0ce@900<000000=9H3@3BF0d02P3BF0d00`000000oooo0?ooo`0G0?ooo``0
0000H01Aao4:000000P0KmOh1P0000090?ooo`X0000000<0Y?7e000000000000202_mnl2000000D0
^?[Z0`00000L0?ooo`00>P3oool00`000000oooo0?ooo`0W0?ooo`0300000037AP00adH001H0adH0
00<000000<M60037AP004P37AP000`000000bD/00<U;000@0<U;00030000003=C`@0cDl400`0cDl4
00<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@0:0=9H3@030000003oool0oooo01@0oooo
0P00000307STi@800000G`1Aao49000001L0KmOh2@000000102Ei_/000000000000000050:Cam@80
0000202_mnl2000000H0^?[Z00@000000?ooo`3oool00000703oool003X0oooo00<000000?ooo`3o
ool0:03oool00`000000adH00<M6000F0<M6000300000037AP00adH00180adH000<000000<U;0039
B`004039B`000`000000cDl40<e?100<0<e?10030000003?E0T0ce@900/0ce@900<000000=9H3@3B
F0d02@3BF0d00`000000oooo0?ooo`0B0?ooo`8000001`1hi>D2000005@0DLOa2@00000806oGn003
0000001_eoP0KmOh0140KmOh1@00000709GVn`8000001P2TlOD2000000T0[oO_00<000000;SjjP2h
n^X0102hn^X2000000<0oooo00<000000?ooo`3oool06P3oool003X0oooo0`00000Y0?ooo`030000
0037AP00adH001H0adH000<000000<M60037AP004P37AP000`000000bD/00<U;000?0<U;00030000
003=C`@0cDl400d0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@090=9H3@030000
003oool0oooo00l0oooo0P00000;07STi@800000B@1Aao4900000140KmOh00<0000006oGn01_eoP0
3@1_eoP4000000T0UNKk0`0000060:Cam@8000002@2_mnl2000000H0^?[Z00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0Z0?ooo`0300000037
AP00adH001D0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=
C`@0cDl400d0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@090=9H3@030000003o
ool0oooo00`0oooo0P00000?07STi@800000?P1Aao49000001X0KmOh00<0000006oGn01_eoP0201_
eoP5000000X0UNKk0`0000070:Cam@8000002@2_mnl2000000L0^?[Z00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0[0?ooo`0300000037AP00
adH001D0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0
cDl400d0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@090=9H3@030000003oool0
oooo00T0oooo0P00000C07STi@<00000<P1Aao49000002<0KmOh00<0000006oGn01_eoP00`1_eoP5
000000`0UNKk0`0000080:Cam@8000002P2_mnl00`000000^?[Z0;SjjP050;SjjP8000001`3oool0
0`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0/0?ooo`0300000037AP00adH0
01D0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0cDl4
00d0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@090=9H3@030000003oool0oooo
00H0oooo0P00000H07STi@8000009`1Aao49000002`0KmOh00<0000006oGn00000000`00000?09GV
n`8000002@2TlOD2000000X0[oO_0P0000070;SjjP030000003oool0oooo00L0oooo00<000000?oo
o`3oool06P3oool003X0oooo00<000000?ooo`3oool0;@3oool00`000000adH00<M6000E0<M60003
00000037AP00adH001<0adH000<000000<U;0039B`003`39B`000`000000cDl40<e?100=0<e?1003
0000003?E0T0ce@900X0ce@900<000000=9H3@3BF0d02P3BF0d00`000000oooo0?ooo`020?ooo`<0
0000701hi>D2000001`0DLOa2@00000b06oGn0D00000402Ei_/3000000T0Y?7e0P00000:0:ogk`80
00001`2hn^X2000000X0oooo00<000000?ooo`3oool06P3oool003X0oooo0P00000_0?ooo`030000
0037AP00adH001D0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000
003=C`@0cDl400d0cDl400<000000<mD2@3?E0T02P3?E0T00`000000dUP=0=9H3@0:0=9H3@040000
003oool0000000000240N>CU0P00000A0577l@T00000=P1_eoP5000000<0UNKk00<0000009GVn`2E
i_/0302Ei_/3000000X0Y?7e0P00000;0:ogk`030000002hn^X0^?[Z00H0^?[Z00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0_0?ooo`030000
0037AP00adH001D0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000
003=C`@0cDl400d0cDl400<000000<mD2@3?E0T02P3?E0T00`000000dUP=0=9H3@090=9H3@800000
9@1hi>D2000000H0DLOa2@00000k06oGn0@00000202Ei_/00`000000UNKk09GVn`0909GVn`<00000
2`2TlOD2000000/0[oO_0P0000070;SjjP8000003@3oool00`000000oooo0?ooo`0J0?ooo`00>P3o
ool00`000000oooo0?ooo`0_0?ooo`0300000037AP00adH001H0adH000<000000<M60037AP004`37
AP000`000000bD/00<U;000?0<U;00030000003=C`@0cDl400`0cDl400<000000<mD2@3?E0T02`3?
E0T00`000000dUP=0=9H3@060=9H3@800000:@1hi>D6000003l0KmOh1@00000<09GVn`030000002E
i_/0UNKk00L0UNKk0P00000<0:Cam@8000002`2_mnl2000000P0^?[Z00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0J0?ooo`00>P3oool00`000000oooo0?ooo`0`0?ooo`0300000037AP00
adH001H0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0
cDl400`0cDl400<000000<mD2@3?E0T02`3?E0T00`000000dUP=0=9H3@030=9H3@800000:@1hi>D2
00000080I=WR0P00000l06oGn0D000004@2Ei_/00`000000UNKk09GVn`0409GVn`<00000302TlOD2
000000/0[oO_0P0000080;SjjP800000403oool00`000000oooo0?ooo`0J0?ooo`00>@3oool00`00
0000oooo0?ooo`0b0?ooo`0300000037AP00adH001H0adH000<000000<M60037AP004`37AP000`00
0000bD/00<U;000?0<U;00030000003=C`@0cDl400`0cDl400<000000<mD2@3?E0T02`3?E0T00`00
0000dUP=0=9H3@02000002P0N>CU0`00000606CIhP800000=@1_eoP5000001H0UNKk00@0000009GV
n`2Ei_/0UNKk0`00000=0:Cam@800000302_mnl00`000000^?[Z0;SjjP070;SjjP030000003oool0
oooo0100oooo00<000000?ooo`3oool06P3oool003T0oooo0`00000c0?ooo`0300000037AP00adH0
01H0adH000<000000<M60037AP004`37AP000`000000bD/00<U;000?0<U;00030000003=C`@0cDl4
00`0cDl400<000000<mD2@3?E0T02`3?E0T2000002P0N>CU0P00000;06CIhP030000001_eoP0KmOh
02h0KmOh1000000K09GVn`@000003P2TlOD2000000`0[oO_0P0000090;SjjP030000003oool0oooo
0140oooo00<000000?ooo`3oool06P3oool003T0oooo00<000000?ooo`3oool0=03oool00`000000
adH00<M6000F0<M6000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`000`000000
cDl40<e?100<0<e?10030000003?E0T0ce@900P0ce@90P00000W07STi@<000003P1TfN82000002T0
KmOh1@00000N09GVn`8000003`2TlOD2000000`0[oO_0P0000090;SjjP8000005@3oool00`000000
oooo0?ooo`0I0?ooo`007@3oool3000000L0oooo0P0000040?ooo`D000001`3oool00`000000oooo
0?ooo`0e0?ooo`0300000037AP00adH001H0adH000<000000<M60037AP004`37AP000`000000bD/0
0<U;000?0<U;00030000003=C`@0cDl400`0cDl400<000000<mD2@3?E0T0103?E0T3000002L0N>CU
0P00000C06CIhP030000001_eoP0KmOh0240KmOh1@00000P09GVn`<0000000<0Y?7e0000002TlOD0
302TlOD2000000d0[oO_00<000000;SjjP2hn^X0202hn^X00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo01T0oooo000L0?ooo`8000000P3oool00`000000oooo0?ooo`040?ooo`8000002@3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03H0oooo00<000000<M60037AP005P37
AP000`000000adH00<M6000C0<M6000300000039B`00bD/000l0bD/000<000000<e?103=C`@0303=
C`@010000000ce@90<mD2@3?E0T2000002L0N>CU0`00000F06CIhP8000007@1_eoP400000280UNKk
0`0000040:Cam@030000002TlOD0Y?7e00T0Y?7e0P00000=0:ogk`8000002@2hn^X2000001P0oooo
00<000000?ooo`3oool06@3oool001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?oo
o`0>0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0=`3oool00`000000adH00<M6
000F0<M6000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`000`000000cDl40<e?
100<0<e?10<000009P1hi>D3000001/0I=WR0P00000F06oGn0D000008`2Ei_/3000000L0Y?7e00<0
00000:Cam@2TlOD01`2TlOD2000000d0[oO_0P00000:0;SjjP030000003oool0oooo01P0oooo00<0
00000?ooo`3oool06@3oool001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo00@0oooo1000000g0?ooo`0300000037AP00adH001H0adH000<00000
0<M60037AP004P37AP000`000000bD/00<U;000@0<U;00030000003=C`@0cDl400X0cDl40P00000V
07STi@800000801TfN800`000000KmOh06oGn00>06oGn0D000009P2Ei_/2000000X0Y?7e00<00000
0:Cam@2TlOD01@2TlOD2000000h0[oO_00<000000;SjjP2hn^X0202hn^X2000001/0oooo00<00000
0?ooo`3oool06@3oool001`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0=0?oo
o`8000001P3oool00`000000oooo0?ooo`0i0?ooo`0300000037AP00adH001H0adH000<000000<M6
0037AP004P37AP000`000000bD/00<U;000@0<U;00030000003=C`@0cDl400L0cDl40P00000U07ST
i@<000008`1TfN82000000X0KmOh1000000X09GVn`<00000302TlOD00`000000Y?7e0:Cam@030:Ca
m@8000003P2_mnl2000000X0^?[Z00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0I0?oo
o`00703oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo0P0000070?ooo`03
0000003oool0oooo03T0oooo00<000000<M60037AP005`37AP000`000000adH00<M6000B0<M60003
00000039B`00bD/00100bD/000<000000<e?103=C`@0103=C`@2000002D0N>CU0P00000X06CIhP80
00000`1_eoP5000002T0UNKk0`00000?0:Cam@040000002TlOD0Y?7e0:Cam@8000003P2_mnl20000
00X0^?[Z0P00000N0?ooo`030000003oool0oooo01T0oooo000L0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo03X0
oooo00<000000<M60037AP005P37AP000`000000adH00<M6000C0<M6000300000039B`00bD/000l0
bD/000<000000<e?103=C`@00P3=C`@2000002@0N>CU0`00000[06CIhP@00000:`2Ei_/300000180
Y?7e00@000000:Cam@00000000003`2_mnl00`000000^?[Z0;SjjP090;SjjP030000003oool0oooo
01h0oooo00<000000?ooo`3oool06@3oool001`0oooo00D000000?ooo`3oool0oooo0000000A0?oo
o`030000003oool0oooo00@0oooo00<000000?ooo`3oool0>`3oool00`000000adH00<M6000F0<M6
000300000037AP00adH001<0adH000<000000<U;0039B`003`39B`04000002@0N>CU0P00000Z06CI
hP@000000P1Dcn@2000002T0UNKk0P00000E0:Cam@8000003`2_mnl2000000/0^?[Z00<000000?oo
o`3oool07`3oool00`000000oooo0?ooo`0I0?ooo`007@3oool3000000h0oooo100000070?ooo`<0
0000?03oool00`000000adH00<M6000F0<M6000300000037AP00adH001<0adH000<000000<U;0039
B`003@39B`02000002@0N>CU0`00000Y06CIhP<00000201Dcn@00`000000UNKk09GVn`0S09GVn`<0
00005@2TlOD3000000h0[oO_0P00000;0;SjjP8000008P3oool00`000000oooo0?ooo`0I0?ooo`00
>@3oool00`000000oooo0?ooo`0m0?ooo`0300000037AP00adH001H0adH000<000000<M60037AP00
4`37AP000`000000bD/00<U;000:0<U;00800000901hi>D2000002T0I=WR0`00000<05C?i0800000
802Ei_/3000001H0Y?7e0P0000020:ogk`030000002_mnl0[oO_00/0[oO_00<000000;SjjP2hn^X0
2P2hn^X00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo01T0oooo000i0?ooo`030000003o
ool0oooo03h0oooo00<000000<M60037AP005P37AP000`000000adH00<M6000C0<M6000300000039
B`00bD/000L0bD/00P00000S07STi@<000009`1TfN8400000140E<oT00<0000009GVn`2Ei_/06P2E
i_/3000001L0Y?7e0P0000040:ogk`030000002_mnl0[oO_00T0[oO_0P00000;0;SjjP8000009@3o
ool00`000000oooo0?ooo`0I0?ooo`00>@3oool00`000000oooo0?ooo`0o0?ooo`0300000037AP00
adH001H0adH000<000000<M60037AP004`37AP000`000000bD/00<U;00040<U;008000008`1hi>D2
000002L0I=WR0`00000F05C?i0800000602Ei_/2000001P0Y?7e0P0000070:ogk`030000002_mnl0
[oO_00H0[oO_0P00000<0;SjjP030000003oool0oooo02D0oooo00<000000?ooo`3oool06@3oool0
03T0oooo00<000000?ooo`3oool0@03oool00`000000adH00<M6000F0<M6000300000037AP00adH0
01<0adH000@000000<U;0039B`00bD/00P00000R07STi@<000009P1TfN83000001/0E<oT0P00000C
09GVn`<00000602TlOD2000000T0[oO_00<000000:ogk`2_mnl01@2_mnl00`000000^?[Z0;SjjP0:
0;SjjP800000:03oool00`000000oooo0?ooo`0I0?ooo`00>@3oool300000440oooo00<000000<M6
0037AP005P37AP000`000000adH00<M6000C0<M600<000008@1hi>D3000002D0I=WR1000000P05C?
i0030000002Ei_/0UNKk00d0UNKk0`00000I0:Cam@8000002`2_mnl00`000000[oO_0:ogk`030:og
k`800000302hn^X00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01T0oooo000i0?ooo`03
0000003oool0oooo0480oooo00<000000<M60037AP005P37AP000`000000adH00<M6000@0<M600<0
00008@1hi>D2000002D0I=WR0`00000U05C?i08000002P2Ei_/3000001X0Y?7e0P00000=0:ogk`04
0000002_mnl0[oO_0:ogk`8000003@2hn^X00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo
01T0oooo000i0?ooo`030000003oool0oooo0480oooo00<000000<M60037AP005`37AP000`000000
adH00<M6000=0<M6008000008@1hi>D3000002@0I=WR0`00000Z05C?i0030000002Ei_/0UNKk00D0
UNKk0P00000K0:Cam@8000003`2_mnl010000000[oO_0:ogk`00000=0;SjjP800000;03oool00`00
0000oooo0?ooo`0I0?ooo`00>@3oool00`000000oooo0?ooo`130?ooo`0300000037AP00adH001L0
adH000<000000<M60037AP002P37AP0200000240N>CU0P00000S06CIhP@00000;P1Dcn@200000080
UNKk0`00000K0:Cam@8000004@2_mnl3000000d0^?[Z00<000000?ooo`3oool0;03oool00`000000
oooo0?ooo`0I0?ooo`00>@3oool00`000000oooo0?ooo`140?ooo`0300000037AP00adH001L0adH0
00<000000<M60037AP001`37AP0200000200N>CU0`00000R06CIhP<00000<P1Dcn@4000001`0Y?7e
0P00000B0:ogk`8000003@2hn^X2000002l0oooo00<000000?ooo`3oool06@3oool003T0oooo0`00
00150?ooo`0300000037AP00adH001L0adH000<000000<M60037AP001037AP0200000200N>CU0P00
000Q06CIhP@00000<01Dcn@5000000<0DlcZ0P00000I0:Cam@8000004`2_mnl00`000000^?[Z0000
000<0;SjjP030000003oool0oooo02l0oooo00<000000?ooo`3oool06@3oool003T0oooo00<00000
0?ooo`3oool0AP3oool00`000000adH00<M6000G0<M6000400000037AP00adH00<M6008000007`1h
i>D300000200I=WR0`00000`05C?i0@000002P1Cc>X2000001D0Y?7e0P00000C0:ogk`8000000P2h
n^X00`000000^?[Z0;SjjP080;SjjP800000<P3oool00`000000oooo0?ooo`0I0?ooo`00>@3oool0
0`000000oooo0?ooo`170?ooo`0300000037AP00adH001L0adH00`00000O07STi@800000801TfN83
000002h0E<oT1@00000@05?<jP8000004@2TlOD2000001<0[oO_0P0000040;SjjP030000002hn^X0
^?[Z00L0^?[Z00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0I0?ooo`00>@3oool00`00
0000oooo0?ooo`180?ooo`0300000037AP00adH001D0adH00P00000N07STi@<000007P1TfN840000
02`0E<oT1@00000G05?<jP030000002TlOD0Y?7e00`0Y?7e0P00000C0:ogk`8000001P2hn^X00`00
0000^?[Z0;SjjP050;SjjP800000=@3oool00`000000oooo0?ooo`0I0?ooo`00>@3oool00`000000
oooo0?ooo`190?ooo`0300000037AP00adH00140adH00`00000N07STi@8000007P1TfN83000002`0
E<oT1000000M05?<jP8000002P2TlOD2000001@0[oO_00<000000;SjjP2hn^X01P2hn^X00`000000
^?[Z0;SjjP040;SjjP030000003oool0oooo03D0oooo00<000000?ooo`3oool06@3oool003T0oooo
0P00001;0?ooo`0300000037AP00adH000h0adH00P00000N07STi@<000007@1TfN83000002X0E<oT
1@00000S05?<jP8000001P2TlOD2000001@0[oO_0P0000090;SjjP030000002hn^X0^?[Z00<0^?[Z
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0I0?ooo`00>@3oool00`000000oooo0?oo
o`1;0?ooo`0300000037AP00adH000/0adH00P00000M07STi@<00000701TfN84000002P0E<oT1@00
000Z05?<jP8000000P2TlOD2000001@0[oO_0P00000;0;SjjP040000002hn^X0^?[Z0;SjjP800000
>@3oool00`000000oooo0?ooo`0I0?ooo`00703oool3000000L0oooo0P0000060?ooo`@000001`3o
ool00`000000oooo0?ooo`1;0?ooo`0300000037AP00adH000T0adH00P00000M07STi@800000701T
fN83000002L0E<oT1@00000_05?<jP@000005@2_mnl00`000000^?[Z0;SjjP0;0;SjjP040000002h
n^X0^?[Z000003/0oooo00<000000?ooo`3oool06@3oool001/0oooo0P0000020?ooo`030000003o
ool0oooo00@0oooo0P0000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0C@3o
ool00`000000adH00<M600060<M600800000701hi>D3000001/0I=WR0`00000V05C?i0@00000;P1C
c>X6000000@0D/_/0P00000A0:ogk`8000003P2hn^X3000003`0oooo00<000000?ooo`3oool06@3o
ool001/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`H000001P3oool0
0`000000oooo0?ooo`1>0?ooo`0300000037AP00adH000<0adH00P00000L07STi@8000006P1TfN84
000002@0E<oT1@00000/05?<jPH00000301Bbn`2000000d0[oO_0P00000@0;SjjP030000003oool0
oooo03`0oooo00<000000?ooo`3oool06@3oool001/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`090?ooo`050000003oool0oooo0?ooo`0000001`3oool4000004h0oooo00<00000
0<M60037AP000P00000K07STi@<000006@1TfN83000002<0E<oT1@00000[05?<jPH00000501Bbn`2
000000X0[oO_00<000000;SjjP2hn^X03P2hn^X3000003h0oooo00<000000?ooo`3oool06@3oool0
01/0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`040000003oool0oooo
000000L0oooo00<000000?ooo`3oool0D03oool>000000l0N>CU0P00000I06CIhP<000008P1Dcn@4
000002X0DlcZ1P00000L05;;k08000001P2_mnl200000100^?[Z00@000000?ooo`3oool00000?P3o
ool00`000000oooo0?ooo`0I0?ooo`006`3oool00`000000oooo0?ooo`020?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`0000001`3oool00`000000oooo0?ooo`1N0?oooaT000003@1TfN84
00000200E<oT1@00000X05?<jPH00000901Bbn`200000080[oO_0P00000@0;SjjP8000000`3oool0
0`000000oooo0?ooo`0m0?ooo`030000003oool0oooo01P0oooo000K0?ooo`030000003oool0oooo
0080oooo00<000000?ooo`3oool02`3oool00`000000oooo000000070?ooo`030000003oool0oooo
07L0oooo3`00000M05C?i0D000009`1Cc>X6000002X0D/_/1@00000@0;SjjP030000003oool0oooo
00<0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0H0?ooo`006`3oool01@000000
oooo0?ooo`3oool0000000l0oooo0P0000070?ooo`030000003oool0oooo08H0oooo1`00000B05C?
i0@000009`1Cc>X5000002X0D/_/1P00000505;;k08000003@2hn^X00`000000oooo0?ooo`040?oo
o`030000003oool0oooo03d0oooo00<000000?ooo`3oool0603oool001`0oooo0`00000A0?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`020?ooo`L0
00001P1Dcn@5000002D0DlcZ1P00000X05;;k0L000003@1Bbn`G000003d0oooo00<000000?ooo`3o
ool0603oool003P0oooo0`0000280?ooo`030000003oool0oooo00T0oooo1P00000T05?<jPH00000
6@1Bbn`Y00000080D/_/0P0000060;SjjP030000003oool0oooo00X0oooo1@00000h0?ooo`030000
003oool0oooo01P0oooo000h0?ooo`030000003oool0oooo08P0oooo00<000000?ooo`3oool03`3o
ool4000001L0DlcZ:000000805;;k0H000007`1Bbn`200000080^?[Z0P00000B0?ooo`H00000<P3o
ool00`000000oooo0?ooo`0H0?ooo`00>03oool00`000000oooo0?ooo`280?ooo`040000003oool0
oooo0?ooob`000009P1Bbn`7000002D0D/_/1000000J0?ooo`D00000;@3oool00`000000oooo0?oo
o`0H0?ooo`00>03oool00`000000oooo0?ooo`1S0?ooobT000005`3oool4000000D0DlcZ1P00000U
05;;k0L00000901Bbn`8000002<0oooo1@00000X0?ooo`030000003oool0oooo01P0oooo000h0?oo
o`030000003oool0oooo03/0oooo:000000U0?ooo`030000003oool0oooo01`0oooo1@00000U05;;
k0H000008`1Bbn`800000300oooo1P00000R0?ooo`030000003oool0oooo01P0oooo000h0?ooo`<0
00003@3oool00`000000oooo0?ooo`020?ooobT00000C@3oool00`000000oooo0?ooo`0P0?ooo`@0
00006`1Bbn`700000280D/_/1`00000n0?ooo`D000007@3oool00`000000oooo0?ooo`0H0?ooo`00
>03ooolE000000X0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0T0?ooo`@00000
401Bbn`700000240D/_/2000001:0?ooo`D00000603oool00`000000oooo0?ooo`0H0?ooo`00@@3o
oolB000000@0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1K0?ooo`030000003o
ool0oooo02P0oooo1000000605;;k0H00000801Bbn`8000005L0oooo1P00000B0?ooo`030000003o
ool0oooo01P0oooo001C0?oooa40000000<0oooo0000003oool02`3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0;03oool6000001l0D/_/1`00001U
0?ooo`D000003@3oool00`000000oooo0?ooo`0H0?ooo`00I03ooolA000000T0oooo00<000000?oo
o`3oool0203oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0340oooo1@00000C05;;
k0P00000L@3oool5000000P0oooo00<000000?ooo`3oool0603oool007D0oooo4P0000020?ooo`03
0000003oool0oooo00P0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0f0?ooo`D0
00001P1Bbn`8000007h0oooo1P0000020?ooo`030000003oool0oooo01P0oooo00270?oooa400000
1P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0
3P3oool00`000000oooo0?ooo`0k0?ooo`H00000J`3ooolT000001X0oooo002H0?oooa4000002@3o
ool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0IP3o
oom6000003h0oooo002X0?oooa<00000203oool00`000000oooo0?ooo`0P0?ooodH00000Q03oool0
0;X0oooo;000003:0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00E@3oool300000?l0ooooF@3o
ool005@0oooo0P0000020?ooo`800000o`3ooomG0?ooo`00E03oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0?l0ooooE@3oool005@0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`3o0?oooeD0oooo001D0?ooo`8000000P3oool00`000000oooo0?ooo`0]0?ooo`@00000
o`3ooolU0?ooo`00E03oool400000380oooo00<000000?ooo`3oool0o`3ooolT0?ooo`00E03oool0
0`000000oooo0?ooo`0_0?ooo`H000008P3oool600000?`0oooo001E0?ooo`030000003oool0oooo
02h0oooo00D000000?ooo`3oool0oooo0000000T0?ooo`050000003oool0oooo0?ooo`000000o03o
ool005H0oooo1000000]0?ooo`040000003oool0oooo000002D0oooo00<000000?ooo`3oool0o@3o
ool008P0oooo00<000000?ooo`0000009P3oool00`000000oooo0?ooo`3l0?ooo`00R03oool00`00
0000oooo0000000W0?ooo`030000003oool0oooo0?/0oooo00290?ooo`800000:03oool00`000000
oooo0?ooo`3j0?ooo`00RP3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0?X0oooo
002^0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0nP3oool00:h0oooo1@00003m
0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 431}, {188.812, 0}} -> {-0.147908, 0.203448, \
0.00390094, 0.00390094}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
The linear in K term is clearly not necessary.  The 1/k^2 and \
Log[K] terms are needed.  Adding a 1/K^3 term will give a better fit than \
adding a K term. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(bb = Transpose[{klist, aa[\([4]\)]}]\)], "Input"],

Cell[BoxData[
    \({{10, 0.147236009344`}, {12, 0.1689147391`}, {15, 
        0.192967735735`}, {20, 0.219274834827`}, {30, 0.247235984252`}, {40, 
        0.261354147736`}, {50, 0.269627907624`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = Fit[bb, {1, 1/k, 1/k^2, 1/Log[k]}, k]\)], "Input"],

Cell[BoxData[
    \(\(\(0.2471238008704588`\)\(\[InvisibleSpace]\)\) + 
      5.2081953650639665`\/k\^2 - 2.8275478118136417`\/k + 
      0.3011862426198493`\/Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[bb, fitting, k]\)], "Input"],

Cell[BoxData[
    \(0.00002648050891177026`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[bb, DisplayFunction \[Rule] Identity], 
      Plot[fitting, {k, 10, 50}, DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0238095 -0.693264 4.80846 [
[.2619 .25593 -6 -9 ]
[.2619 .25593 6 0 ]
[.5 .25593 -6 -9 ]
[.5 .25593 6 0 ]
[.7381 .25593 -6 -9 ]
[.7381 .25593 6 0 ]
[.97619 .25593 -6 -9 ]
[.97619 .25593 6 0 ]
[.01131 .07609 -24 -4.5 ]
[.01131 .07609 0 4.5 ]
[.01131 .17226 -24 -4.5 ]
[.01131 .17226 0 4.5 ]
[.01131 .3646 -24 -4.5 ]
[.01131 .3646 0 4.5 ]
[.01131 .46077 -24 -4.5 ]
[.01131 .46077 0 4.5 ]
[.01131 .55694 -24 -4.5 ]
[.01131 .55694 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .26843 m
.2619 .27468 L
s
[(20)] .2619 .25593 0 1 Mshowa
.5 .26843 m
.5 .27468 L
s
[(30)] .5 .25593 0 1 Mshowa
.7381 .26843 m
.7381 .27468 L
s
[(40)] .7381 .25593 0 1 Mshowa
.97619 .26843 m
.97619 .27468 L
s
[(50)] .97619 .25593 0 1 Mshowa
.125 Mabswid
.07143 .26843 m
.07143 .27218 L
s
.11905 .26843 m
.11905 .27218 L
s
.16667 .26843 m
.16667 .27218 L
s
.21429 .26843 m
.21429 .27218 L
s
.30952 .26843 m
.30952 .27218 L
s
.35714 .26843 m
.35714 .27218 L
s
.40476 .26843 m
.40476 .27218 L
s
.45238 .26843 m
.45238 .27218 L
s
.54762 .26843 m
.54762 .27218 L
s
.59524 .26843 m
.59524 .27218 L
s
.64286 .26843 m
.64286 .27218 L
s
.69048 .26843 m
.69048 .27218 L
s
.78571 .26843 m
.78571 .27218 L
s
.83333 .26843 m
.83333 .27218 L
s
.88095 .26843 m
.88095 .27218 L
s
.92857 .26843 m
.92857 .27218 L
s
.25 Mabswid
0 .26843 m
1 .26843 L
s
.02381 .07609 m
.03006 .07609 L
s
[(0.16)] .01131 .07609 1 0 Mshowa
.02381 .17226 m
.03006 .17226 L
s
[(0.18)] .01131 .17226 1 0 Mshowa
.02381 .3646 m
.03006 .3646 L
s
[(0.22)] .01131 .3646 1 0 Mshowa
.02381 .46077 m
.03006 .46077 L
s
[(0.24)] .01131 .46077 1 0 Mshowa
.02381 .55694 m
.03006 .55694 L
s
[(0.26)] .01131 .55694 1 0 Mshowa
.125 Mabswid
.02381 .10013 m
.02756 .10013 L
s
.02381 .12417 m
.02756 .12417 L
s
.02381 .14822 m
.02756 .14822 L
s
.02381 .1963 m
.02756 .1963 L
s
.02381 .22034 m
.02756 .22034 L
s
.02381 .24439 m
.02756 .24439 L
s
.02381 .29247 m
.02756 .29247 L
s
.02381 .31651 m
.02756 .31651 L
s
.02381 .34056 m
.02756 .34056 L
s
.02381 .38864 m
.02756 .38864 L
s
.02381 .41268 m
.02756 .41268 L
s
.02381 .43672 m
.02756 .43672 L
s
.02381 .48481 m
.02756 .48481 L
s
.02381 .50885 m
.02756 .50885 L
s
.02381 .53289 m
.02756 .53289 L
s
.02381 .05205 m
.02756 .05205 L
s
.02381 .02801 m
.02756 .02801 L
s
.02381 .00396 m
.02756 .00396 L
s
.02381 .58098 m
.02756 .58098 L
s
.02381 .60502 m
.02756 .60502 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.02381 .01472 Mdot
.07143 .11896 Mdot
.14286 .23461 Mdot
.2619 .36111 Mdot
.5 .49556 Mdot
.7381 .56345 Mdot
.97619 .60323 Mdot
.5 Mabswid
.02381 .0148 m
.06244 .10102 L
.10458 .17759 L
.14415 .23646 L
.18221 .28379 L
.22272 .3263 L
.26171 .3611 L
.30316 .39282 L
.34309 .41923 L
.3815 .44146 L
.42237 .46229 L
.46172 .48003 L
.49955 .49528 L
.53984 .50984 L
.57861 .52245 L
.61984 .53457 L
.65954 .54513 L
.69774 .5544 L
.73838 .56343 L
.77751 .57139 L
.81909 .57915 L
.85916 .58603 L
.89771 .59216 L
.93871 .59819 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40006`00012R000`400?l00000o`00003oo`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona
0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00o`3ooona0?ooo`00:03oool00`000000oooo0?oo
o`3o0?ooohH0oooo000X0?ooo`030000003oool0oooo0?l0ooooQP3oool002P0oooo0`00003o0?oo
ohH0oooo000X0?ooo`030000003oool0oooo0?l0ooooQP3oool002P0oooo00<000000?ooo`3oool0
o`3ooon60?ooo`00:03oool200000?l0ooooQ`3oool002P0oooo0P00003o0?ooohL0oooo000X0?oo
o`030000003oool0oooo0?l0ooooQP3oool002P0oooo0P00003o0?ooohL0oooo000X0?ooo`800000
o`3ooon70?ooo`00:03oool00`000000oooo0000003o0?ooohH0oooo000X0?ooo`<00000o`3ooon6
0?ooo`00:03oool010000000oooo0?ooo`00003o0?ooohD0oooo000X0?ooo`040000003oool0oooo
00000?l0ooooQ@3oool002P0oooo00D000000?ooo`3oool0oooo0000003o0?oooh@0oooo000X0?oo
o`050000003oool0oooo0?ooo`000000o`3ooon40?ooo`00:03oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0?l0ooooP@3oool002P0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`3o0?oooh40oooo000X0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
o`3ooon00?ooo`00:03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0ooooP03o
ool002P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?ooogl0oooo000X0?oo
o`<00000103oool00`000000oooo0?ooo`3o0?ooogl0oooo000X0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0o`3ooomn0?ooo`00:03oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo0?l0ooooOP3oool002P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`3o0?ooogh0oooo000X0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3ooomm
0?ooo`00:03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0ooooO@3oool000<0
oooo0`0000070?ooo`8000001@3oool6000000@0oooo0`0000070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0o`3oooml0?ooo`000P3oool200000080oooo00<000000?ooo`3oool0
103oool2000000L0oooo00<000000?ooo`3oool0103oool200000080oooo0P0000050?ooo`030000
003oool0oooo00L0oooo00<000000?ooo`3oool0o`3oooml0?ooo`000P3oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`3o0?ooog/0oooo00020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0303oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo0?l0ooooN`3oool00080oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo0P0000020?oo
o`030000003oool0oooo00@0oooo100000080?ooo`030000003oool0oooo0?l0ooooNP3oool00080
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0
oooo100000070?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0o`3ooomj0?ooo`00
0P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
o`3ooomi0?ooo`000P3oool01@000000oooo0?ooo`3oool0000000l0oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
o`3ooomi0?ooo`000`3oool3000000h0oooo0`0000080?ooo`@000001@3oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo0?l0ooooN03oool002P0oooo00<000000?ooo`3oool02`3oool0
0`000000oooo0?ooo`3o0?ooogP0oooo000X0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0o`3ooomg0?ooo`00:03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0?l0
ooooM`3oool002P0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`3o0?ooogH0oooo
000X0?ooo`<000003@3oool00`000000oooo0?ooo`3o0?ooogH0oooo000X0?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0o`3ooome0?ooo`00:03oool00`000000oooo0?ooo`0>0?oo
o`030000003oool0oooo0?l0ooooM@3oool002P0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`3o0?ooog@0oooo000X0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
o`3ooomd0?ooo`00:03oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0?l0ooooL`3o
ool002P0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`3o0?ooog<0oooo000X0?oo
o`030000003oool0oooo0100oooo0P00003o0?ooog@0oooo000X0?ooo`030000003oool0oooo0100
oooo0P00003o0?ooog@0oooo000X0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
o`3oooma0?ooo`00:03oool300000180oooo00<000000?ooo`3oool0o`3oooma0?ooo`00:03oool0
0`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0?l0ooooL03oool002P0oooo00<000000?oo
o`3oool0503oool00`000000oooo0?ooo`3o0?ooofl0oooo000X0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0o`3ooom_0?ooo`00:03oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo0?l0ooooKP3oool002P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`3o0?ooofh0oooo000X0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0o`3ooom]
0?ooo`00:03oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0?l0ooooK@3oool002P0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`3o0?ooof`0oooo000X0?ooo`<00000
5`3oool00`000000oooo0?ooo`3o0?ooof`0oooo000X0?ooo`030000003oool0oooo01P0oooo00<0
00000?ooo`3oool0o`3ooom[0?ooo`00:03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo0?l0ooooJP3oool002P0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`3o0?oo
ofX0oooo000X0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0o`3ooomY0?ooo`00
:03oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo0?l0ooooJ@3oool000<0oooo0`00
00070?ooo`8000001@3oool6000000@0oooo100000060?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool0o`3ooomX0?ooo`000P3oool200000080oooo00<000000?ooo`3oool0103oool2
000000L0oooo00<000000?ooo`3oool0103oool010000000oooo0?ooo`3oool2000000D0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`3o0?ooofP0oooo00020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo0?l0ooooI`3oool00080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0<0?oo
o`030000003oool0oooo00@0oooo00@000000?ooo`3oool0oooo0P0000050?ooo`030000003oool0
oooo01`0oooo00<000000?ooo`3oool0o`3ooomW0?ooo`000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool01@3oool3000000L0oooo1000000L0?oo
o`030000003oool0oooo0?l0ooooIP3oool00080oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo0P0000020?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`3o0?ooofH0oooo00020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0N0?ooo`030000
003oool0oooo0?l0ooooI@3oool00080oooo00D000000?ooo`3oool0oooo0000000?0?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0o`3ooomT0?ooo`000`3oool3000000h0oooo0`00
00070?ooo`@000001P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0?l0ooooI03o
ool002P0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`3o0?ooof<0oooo000X0?oo
o`030000003oool0oooo0240oooo00<000000?ooo`3oool0o`3ooomR0?ooo`00:03oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo0?l0ooooHP3oool002P0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`3o0?ooof40oooo000X0?ooo`030000003oool0oooo02<0oooo00<0
00000?ooo`3oool0o`3ooomP0?ooo`00:03oool3000002<0oooo00<000000?ooo`3oool0o`3ooomP
0?ooo`00:03oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0?l0ooooG`3oool002P0
oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3o0?oooeh0oooo000X0?ooo`030000
003oool0oooo02D0oooo00<000000?ooo`3oool0o`3ooomN0?ooo`00:03oool00`000000oooo0?oo
o`0V0?ooo`030000003oool0oooo0?l0ooooG@3oool002P0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`3o0?oooe`0oooo000X0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0o`3ooomL0?ooo`00:03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0?l0
ooooF`3oool002P0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3o0?oooeX0oooo
000X0?ooo`<00000:@3oool00`000000oooo0?ooo`3o0?oooeX0oooo000X0?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool0o`3ooomI0?ooo`00:03oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo02L0oooo1P0000050?ooo`<00000D@3oool5000000H0oooo0`00001C0?oo
o`@000001@3oool300000580oooo100000060?ooo`<000001@3oool002P0oooo00<000000?ooo`3o
ool0:`3oool00`000000oooo0?ooo`0X0?ooo`050000003oool0oooo0?ooo`000000103oool20000
0080oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`020?ooo`8000000P3oool00`00
0000oooo0?ooo`1B0?ooo`030000003oool0oooo00<0oooo0P0000020?ooo`030000003oool0oooo
05<0oooo00<000000?ooo`3oool00P3oool200000080oooo00<000000?ooo`3oool00P3oool002P0
oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1=0?ooo`H00000103oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo05<0oooo00D000000?ooo`3oool0oooo000000040?oo
o`030000003oool0oooo0040oooo000X0?ooo`030000003oool0oooo02d0oooo0P00000Y0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1B0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1=0?ooo`05
0000003oool0oooo0?ooo`0000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
05<0oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0oooo0040oooo000X0?oo
o`030000003oool0oooo02d0oooo0P00000Z0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool00P3oool00`000000oooo0?ooo`1A0?ooo`800000103oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo04h0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0580oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo0040oooo000X0?ooo`030000003oool0oooo02h0oooo00<000000?oo
o`3oool0:@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0D03oool2000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1?0?oo
o`030000003oool0000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1>0?oo
o`@000001@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0040oooo000X0?ooo`03
0000003oool0oooo02l0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0C`3oool00`000000oooo000000050?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0CP3oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool00@3oool002P0oooo00<000000?ooo`3oool0
<03oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0P3oool01@000000oooo0?ooo`3oool0000005D0oooo00<000000?ooo`3oool00P3oool01@000000
oooo0?ooo`3oool0000005<0oooo0P0000050?ooo`050000003oool0oooo0?ooo`000000D@3oool0
0`000000oooo0?ooo`060?ooo`050000003oool0oooo0?ooo`000000103oool002P0oooo0`00000`
0?ooo`030000003oool0oooo0280oooo1@0000060?ooo`<00000DP3oool4000000H0oooo0`00001E
0?ooo`030000003oool0oooo00@0oooo0`00001B0?ooo`D000001@3oool3000000D0oooo000X0?oo
o`030000003oool0oooo0340oooo00<000000?ooo`3oool0o`3ooomB0?ooo`00:03oool00`000000
oooo0?ooo`0b0?ooo`030000003oool0oooo0?l0ooooD@3oool002P0oooo00<000000?ooo`3oool0
<`3oool00`000000oooo0?ooo`3o0?oooe00oooo000X0?ooo`030000003oool0oooo03@0oooo00<0
00000?ooo`3oool0o`3ooom?0?ooo`00:03oool00`000000oooo0?ooo`0e0?ooo`030000003oool0
oooo0?l0ooooCP3oool002P0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`3o0?oo
odh0oooo000X0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0o`3ooom=0?ooo`00
:03oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0?l0ooooC03oool001l0ooooo`00
002A00000040oooo000X0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool000000100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`080?ooo`00:03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0403oool010000000oooo0?ooo`00000?0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo0100oooo00<000000?ooo`3oool0203oool002P0oooo00<000000?ooo`3oool0>P3oool00`00
0000oooo0?ooo`0P0?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool0G03oool00`00
0000oooo0?ooo`1L0?ooo`030000003oool0oooo00P0oooo000X0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool0o`3ooom80?ooo`00:03oool00`000000oooo0?ooo`0l0?ooo`030000
003oool0oooo0?l0ooooA`3oool002P0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?oo
o`3o0?ooodH0oooo000X0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0o`3ooom5
0?ooo`00:03oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo0?l0ooooA03oool002P0
oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`3o0?oood<0oooo000X0?ooo`<00000
@@3oool00`000000oooo0?ooo`3o0?oood80oooo000X0?ooo`030000003oool0oooo0480oooo00<0
00000?ooo`3oool0o`3ooom10?ooo`00:03oool00`000000oooo0?ooo`130?ooo`030000003oool0
oooo0?l0oooo@03oool002P0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`3o0?oo
ocl0oooo000X0?ooo`030000003oool0oooo04D0oooo00<000000?ooo`3oool0o`3oooln0?ooo`00
:03oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo0?l0oooo?P3oool002P0oooo00<0
00000?ooo`3oool0AP3oool00`000000oooo0?ooo`3o0?ooocd0oooo000X0?ooo`030000003oool0
oooo04L0oooo00<000000?ooo`3oool0o`3oooll0?ooo`00:03oool00`000000oooo0?ooo`180?oo
o`030000003oool0oooo0?l0oooo>`3oool002P0oooo00<000000?ooo`3oool0B@3oool00`000000
oooo0?ooo`3o0?ooocX0oooo000X0?ooo`<00000BP3oool00`000000oooo0?ooo`3o0?ooocT0oooo
000X0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0o`3ooolh0?ooo`00:03oool0
0`000000oooo0?ooo`1<0?ooo`030000003oool0oooo0?l0oooo=`3oool002P0oooo00<000000?oo
o`3oool0C@3oool00`000000oooo0?ooo`3o0?ooocH0oooo000X0?ooo`030000003oool0oooo04h0
oooo00<000000?ooo`3oool0o`3ooole0?ooo`00:03oool00`000000oooo0?ooo`1?0?ooo`030000
003oool0oooo0?l0oooo=03oool002P0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?oo
o`3o0?oooc<0oooo000X0?ooo`030000003oool0oooo0540oooo0P00003o0?oooc<0oooo000X0?oo
o`030000003oool0oooo05<0oooo00<000000?ooo`3oool0o`3oool`0?ooo`00:03oool3000005@0
oooo00<000000?ooo`3oool0o`3oool_0?ooo`00:03oool00`000000oooo0?ooo`1E0?ooo`030000
003oool0oooo0?l0oooo;P3oool002P0oooo00<000000?ooo`3oool0EP3oool00`000000oooo0?oo
o`3o0?ooobd0oooo000X0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool0o`3oool/
0?ooo`00:03oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo0?l0oooo:`3oool002P0
oooo00<000000?ooo`3oool0F@3oool200000?l0oooo:`3oool000<0oooo0`0000070?ooo`800000
103oool6000000<0oooo1P0000060?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool0
o`3ooolX0?ooo`000P3oool200000080oooo00<000000?ooo`3oool0103oool2000000D0oooo00D0
00000?ooo`3oool0oooo000000040?ooo`050000003oool0oooo0?ooo`0000001P3oool00`000000
oooo0?ooo`1L0?ooo`<00000o`3ooolW0?ooo`000P3oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo05d0oooo0P00003o0?ooobL0oooo00020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0GP3oool00`000000oooo0?ooo`3o0?ooobD0oooo00020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00D0oooo1000001N0?ooo`800000o`3ooolU0?ooo`000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0640oooo00<000000?ooo`3oool0o`3ooolR0?ooo`000P3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0680oooo00<000000?ooo`3oool0o`3ooolQ0?ooo`00
0P3oool01@000000oooo0?ooo`3oool0000000`0oooo00<000000?ooo`3oool00P3oool01@000000
oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1S0?oo
o`030000003oool0oooo0?l0oooo803oool000<0oooo0`00000=0?ooo`D00000103oool5000000L0
oooo00<000000?ooo`3oool0I03oool00`000000oooo0?ooo`3o0?oooal0oooo000X0?ooo`030000
003oool0oooo06D0oooo0P00003o0?oooal0oooo000X0?ooo`030000003oool0oooo06L0oooo00<0
00000?ooo`3oool0o`3ooolL0?ooo`00:03oool00`000000oooo0?ooo`1X0?ooo`030000003oool0
oooo0?l0oooo6`3oool002P0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`3o0?oo
oaX0oooo000X0?ooo`<00000JP3oool200000?l0oooo6P3oool002P0oooo00<000000?ooo`3oool0
K03oool00`000000oooo0?ooo`3o0?oooaL0oooo000X0?ooo`030000003oool0oooo06d0oooo00<0
00000?ooo`3oool0o`3ooolF0?ooo`00:03oool00`000000oooo0?ooo`1^0?ooo`800000o`3ooolF
0?ooo`00:03oool00`000000oooo0?ooo`1`0?ooo`030000003oool0oooo0?l0oooo4`3oool002P0
oooo00<000000?ooo`3oool0L@3oool200000?l0oooo4`3oool002P0oooo00<000000?ooo`3oool0
L`3oool00`000000oooo0?ooo`3o0?oooa00oooo000X0?ooo`030000003oool0oooo07@0oooo00<0
00000?ooo`3oool0o`3oool?0?ooo`00:03oool00`000000oooo0?ooo`1e0?ooo`800000o`3oool?
0?ooo`00:03oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo0?l0oooo303oool002P0
oooo0`00001h0?ooo`800000o`3oool<0?ooo`00:03oool00`000000oooo0?ooo`1j0?ooo`030000
003oool0oooo0?l0oooo2@3oool002P0oooo00<000000?ooo`3oool0N`3oool200000?l0oooo2@3o
ool002P0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`3o0?ooo`H0oooo000X0?oo
o`030000003oool0oooo07h0oooo0P00003o0?ooo`H0oooo000X0?ooo`030000003oool0oooo0800
oooo0P00003o0?ooo`@0oooo000X0?ooo`030000003oool0oooo0880oooo0P00003o0?ooo`80oooo
000X0?ooo`030000003oool0oooo08@0oooo0P00003o0?ooo`00:03oool00`000000oooo0?ooo`26
0?ooo`800000o@3oool002P0oooo00<000000?ooo`3oool0R03oool200000?/0oooo000X0?ooo`<0
0000RP3oool200000?T0oooo000X0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool0
mP3oool002P0oooo00<000000?ooo`3oool0S@3oool200000?H0oooo000X0?ooo`030000003oool0
oooo08l0oooo0P00003d0?ooo`00:03oool00`000000oooo0?ooo`2A0?ooo`800000lP3oool002P0
oooo00<000000?ooo`3oool0T`3oool200000?00oooo00030?ooo`<000001`3oool2000000@0oooo
1P0000050?ooo`@000001P3oool00`000000oooo0?ooo`2E0?ooo`800000kP3oool00080oooo0P00
00020?ooo`030000003oool0oooo00@0oooo0P0000050?ooo`050000003oool0oooo0?ooo`000000
1`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo09L0oooo0P00003/0?ooo`000P3o
ool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0103o
ool6000000H0oooo00<000000?ooo`3oool0V@3oool200000>X0oooo00020?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`030?ooo`050000003oool0
oooo0?ooo`0000001`3oool4000009X0oooo0P00003X0?ooo`000P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`00
00070?ooo`030000003oool0oooo09d0oooo0P00003V0?ooo`000P3oool00`000000oooo0?ooo`02
0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo00000007
0?ooo`030000003oool0oooo09l0oooo0P00003T0?ooo`000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo000000070?oo
o`030000003oool0oooo0:40oooo0P00003R0?ooo`000P3oool01@000000oooo0?ooo`3oool00000
00`0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`8000001`3oool00`00
0000oooo0?ooo`2S0?ooo`800000h03oool000<0oooo0`00000=0?ooo`D00000203oool00`000000
oooo0?ooo`050?ooo`030000003oool0oooo0:D0oooo0P00003N0?ooo`00:03oool00`000000oooo
0?ooo`2W0?ooo`800000g03oool002P0oooo00<000000?ooo`3oool0Z@3oool200000=X0oooo000X
0?ooo`030000003oool0oooo0:/0oooo0`00003G0?ooo`00:03oool00`000000oooo0?ooo`2^0?oo
o`800000e@3oool002P0oooo0`00002`0?ooo`<00000dP3oool002P0oooo00<000000?ooo`3oool0
/`3oool300000<l0oooo000X0?ooo`030000003oool0oooo0;H0oooo0P00003=0?ooo`00:03oool0
0`000000oooo0?ooo`2h0?ooo`<0000000<0oooo000000000000a`3oool002P0oooo00<000000?oo
o`3oool0^`3oool300000<L0oooo000X0?ooo`030000003oool0oooo0;h0oooo0P0000350?ooo`00
:03oool00`000000oooo0?ooo`300?ooo`<00000`P3oool002P0oooo00<000000?ooo`3oool0``3o
ool300000;l0oooo000X0?ooo`030000003oool0oooo0<H0oooo0P00002m0?ooo`00:03oool30000
0<P0oooo0`00002j0?ooo`00:03oool00`000000oooo0?ooo`3;0?ooo`<00000]`3oool002P0oooo
00<000000?ooo`3oool0cP3oool300000;@0oooo000X0?ooo`030000003oool0oooo0=40oooo0`00
002a0?ooo`00:03oool00`000000oooo0?ooo`3D0?ooo`<00000[P3oool002P0oooo00<000000?oo
o`3oool0e`3oool300000:/0oooo000X0?ooo`030000003oool0oooo0=X0oooo0`00002X0?ooo`00
:03oool00`000000oooo0?ooo`3M0?ooo`@00000Y03oool002P0oooo00<000000?ooo`3oool0h@3o
ool300000:40oooo000X0?ooo`030000003oool0oooo0>@0oooo0`00002N0?ooo`00:03oool30000
0>L0oooo1000002J0?ooo`00:03oool00`000000oooo0?ooo`3[0?ooo`<00000U`3oool002P0oooo
00<000000?ooo`3oool0kP3oool4000009<0oooo000X0?ooo`030000003oool0oooo0?80oooo1000
002?0?ooo`00:03oool00`000000oooo0?ooo`3f0?ooo`@00000R`3oool002P0oooo00<000000?oo
o`3oool0nP3oool3000008P0oooo00030?ooo`<000001`3oool2000000@0oooo1P0000050?ooo`<0
00001`3oool00`000000oooo0?ooo`3m0?ooo`@00000Q03oool00080oooo0P0000020?ooo`030000
003oool0oooo00@0oooo0P0000050?ooo`050000003oool0oooo0?ooo`000000103oool200000080
oooo0P0000050?ooo`030000003oool0oooo0?l0oooo0P3oool400000800oooo00020?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`050?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`H0oooo
1000001l0?ooo`000P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0o`3oool:0?ooo`D00000M`3oool00080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00<0oooo0P0000020?ooo`030000003oool0
oooo00@0oooo1000003o0?ooo`h0oooo1@00001b0?ooo`000P3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo00h0oooo00<000000?ooo`3oool00P3oool4000000L0oooo00<000000?oo
o`3oool0o`3ooolD0?ooo`H000000P3oool2000006P0oooo00020?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0o`3ooolJ0?ooo`D00000I`3oool00080oooo00D000000?ooo`3oool0
oooo0000000<0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo0?l0oooo7`3oool500000680oooo00030?ooo`<00000
3@3oool5000000L0oooo100000050?ooo`030000003oool0oooo0?l0oooo903oool6000005`0oooo
000X0?ooo`030000003oool0oooo0?l0oooo:P3oool5000005L0oooo000X0?ooo`030000003oool0
oooo0?l0oooo;`3oool500000580oooo000X0?ooo`030000003oool0oooo0?l0oooo=03oool60000
04`0oooo000X0?ooo`030000003oool0oooo0?l0oooo>P3oool5000004L0oooo000X0?ooo`<00000
o`3ooolo0?ooo`D00000@P3oool002P0oooo00<000000?ooo`3oool0o`3ooom40?ooo`H00000?03o
ool002P0oooo00<000000?ooo`3oool0o`3ooom:0?ooo`D00000=`3oool002P0oooo00<000000?oo
o`3oool0o`3ooom?0?ooo`D00000<P3oool002P0oooo00<000000?ooo`3oool0o`3ooomD0?ooo`H0
0000;03oool002P0oooo00<000000?ooo`3oool0o`3ooomJ0?ooo`H000009P3oool002P0oooo00<0
00000?ooo`3oool0o`3ooomP0?ooo`P000007P3oool002P0oooo00<000000?ooo`3oool0o`3ooomX
0?ooo`P000005P3oool002P0oooo00<000000?ooo`3oool0o`3ooom`0?ooo`P000000`3oool20000
00T0oooo000X0?ooo`030000003oool0oooo0?l0ooooN03oool5000000T0oooo000X0?ooo`<00000
o`3ooon60?ooo`00:03oool00`000000oooo0?ooo`3o0?ooohH0oooo000X0?ooo`030000003oool0
oooo0?l0ooooQP3oool002P0oooo00<000000?ooo`3oool0o`3ooon60?ooo`00:03oool00`000000
oooo0?ooo`3o0?ooohH0oooo000X0?ooo`030000003oool0oooo0?l0ooooQP3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3o
ool00?l0oooo/@3oool00?l0oooo/@3oool00?l0oooo/@3oool00001\
\>"],
  ImageRangeCache->{{{0, 431}, {265.875, 0}} -> {5.69024, 0.13924, 0.157692, \
0.000780823}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[List, fitting] /. k \[Rule] 100\)], "Input"],

Cell[BoxData[
    \({\(-0.38490856861804856`\), \(-0.00030834460809311606`\), 
      0.015883431076298582`, 0.5682144607570515`}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Put fits in files", "Section"],

Cell[CellGroupData[{

Cell["Fill fit files 1", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_1.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(-0.5057848663070794`\) + 3.8727746426452203`\/k\^2 - 
      0.1039900081767631`\/k - 0.010358811973834557`\ k + 
      0.23903608536559426`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.5490699243884504`\)\((\((1 - x)\)\^0.23682976030567204`\ \
\((\(\(0.12208043107847259`\)\(\[InvisibleSpace]\)\) + 
              0.06850568755522249`\ \((\(\(1.383288808141729`\)\(\
\[InvisibleSpace]\)\) + 3.0060305947669654`\ \((\(-2\) + 2\ x)\) + 
                    1.2389765964358739`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.10696000093377754`\ \((\(\(0.7858996846941224`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6877598359172215`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6313142704174352`\)\
\((\((1 - x)\)\^0.20288929998927166`\ \((\(\(0.0983259146162377`\)\(\
\[InvisibleSpace]\)\) + 
              0.07205268894491265`\ \((\(\(1.324915984008976`\)\(\
\[InvisibleSpace]\)\) + 2.832447558381696`\ \((\(-2\) + 2\ x)\) + 
                    1.1480716452861532`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.11817841823911729`\ \((\(\(0.8342035704067068`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5715750295718363`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5731543075622646`\)\
\((\((1 - x)\)\^0.22541086044733036`\ \((\(\(0.11618153607016828`\)\(\
\[InvisibleSpace]\)\) + 
              0.08323495818817901`\ \((\(\(1.3635213186747985`\)\(\
\[InvisibleSpace]\)\) + 2.951180268741512`\ \((\(-2\) + 2\ x)\) + 
                    1.2108401719008546`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.13137708558179614`\ \((\(\(0.798565168009595`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6522565528850657`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5020531169318292`\)\
\((\((1 - x)\)\^0.2565649845354224`\ \((\(\(0.143937508742433`\)\(\
\[InvisibleSpace]\)\) + 
              0.10137409818005326`\ \((\(\(1.4177602724479645`\)\(\
\[InvisibleSpace]\)\) + 3.10786751496077`\ \((\(-2\) + 2\ x)\) + 
                    1.2927309370465785`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.15261573672787182`\ \((\(\(0.7586181014672516`\)\(\
\[InvisibleSpace]\)\) + 
                    1.7545118676035933`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.4235298003447337`\)\
\((\((1 - x)\)\^0.2990305342309952`\ \((\(\(0.18627874425107938`\)\(\
\[InvisibleSpace]\)\) + 
              0.13354190618922787`\ \((\(\(1.4932554315477298`\)\(\
\[InvisibleSpace]\)\) + 3.305431994204075`\ \((\(-2\) + 2\ x)\) + 
                    1.393000650558336`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.18862635148799503`\ \((\(\(0.7225603345757289`\)\(\
\[InvisibleSpace]\)\) + 
                    1.8755007338862615`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.3042117132777062`\)\
\((\((1 - x)\)\^0.33777927937009616`\ \((\(\(0.24919860591690632`\)\(\
\[InvisibleSpace]\)\) + 
              0.15642905798687137`\ \((\(\(1.563716339841035`\)\(\
\[InvisibleSpace]\)\) + 3.5459056992899822`\ \((\(-2\) + 2\ x)\) + 
                    1.5295124680175123`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.21696495705150468`\ \((\(\(0.6419909926478023`\)\(\
\[InvisibleSpace]\)\) + 
                    2.03356756609239`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.24331815477971777`\)\((\((1 - x)\)\^0.35587133484555067`\ \
\((\(\(0.28935572899233186`\)\(\[InvisibleSpace]\)\) + 
              0.1750323191275868`\ \((\(\(1.5971292057507032`\)\(\
\[InvisibleSpace]\)\) + 3.6663874075497294`\ \((\(-2\) + 2\ x)\) + 
                    1.6000675598504706`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.2385040612249342`\ \((\(\(0.5991894896252684`\)\(\
\[InvisibleSpace]\)\) + 
                    2.112553180065833`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(sfit\)\" is similar to \
existing symbol \"\!\(fit\)\"."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.24362224213562889`\ x\^0.23503751881237414`\ \
\((\(\(0.3320289203040242`\)\(\[InvisibleSpace]\)\) + 
            0.037380796560604675`\ \((\(\(1.395109261635039`\)\(\
\[InvisibleSpace]\)\) + 3.9023992062425754`\ \((\(-2\) + 2\ x)\) + 
                  1.947466686673353`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.04855581165205054`\ \((\(\(0.008584723323254745`\)\(\
\[InvisibleSpace]\)\) + 
                  2.478659760948003`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.2435206248257603`\ \
x\^0.08551820881923135`\ \((\(\(0.2971760878281575`\)\(\[InvisibleSpace]\)\) \
+ 0.037778294564126305`\ \((\(\(1.3949320845964046`\)\(\[InvisibleSpace]\)\) \
+ 3.7343836420642154`\ \((\(-2\) + 2\ x)\) + 
                  1.8014422986951748`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.07501023151130602`\ \((\(\(0.15800241600652898`\)\(\
\[InvisibleSpace]\)\) + 
                  2.3290388336449914`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.2676001637488393`\ \
x\^0.3090892485199922`\ \((\(\(0.38423721766389346`\)\(\[InvisibleSpace]\)\) \
+ 0.03767482060425719`\ \((\(\(1.4372051694424617`\)\(\[InvisibleSpace]\)\) + 
                  4.055250748469771`\ \((\(-2\) + 2\ x)\) + 
                  2.0461745705130987`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.07342218758809432`\ \((\(-0.04148908477115287`\) + 
                  2.5766894122688315`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.271594546131041`\ \
x\^0.37328160446964725`\ \((\(\(0.43078452362182335`\)\(\[InvisibleSpace]\)\) \
+ 0.024713436313440855`\ \((\(\(1.4442736179406246`\)\(\[InvisibleSpace]\)\) \
+ 4.139840392513814`\ \((\(-2\) + 2\ x)\) + 
                  2.1162497879772975`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.09305258430407475`\ \((\(-0.10168705833860625`\) + 
                  2.6448761506006884`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.31153647984398625`\ \
x\^0.38473403005633194`\ \((\(\(0.497024864719585`\)\(\[InvisibleSpace]\)\) + 
            0.027237576110146065`\ \((\(\(1.5158322089027705`\)\(\
\[InvisibleSpace]\)\) + 4.272032061503059`\ \((\(-2\) + 2\ x)\) + 
                  2.1698357740323844`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.14104113233404852`\ \((\(-0.0731975502123457`\) + 
                  2.696270509900318`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.3427352577077333`\ \
x\^0.3948669970628788`\ \((\(\(0.5506396750724021`\)\(\[InvisibleSpace]\)\) + 
            0.03921572126243174`\ \((\(\(1.572836614999593`\)\(\
\[InvisibleSpace]\)\) + 4.378106290769517`\ \((\(-2\) + 2\ x)\) + 
                  2.213409108704622`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.17865941778843658`\ \((\(-0.052131739355145534`\) + 
                  2.737602254770612`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.36457551255297754`\ \
x\^0.4050195728858076`\ \((\(\(0.5940450273301505`\)\(\[InvisibleSpace]\)\) + 
            0.056148216650967586`\ \((\(\(1.6133209210060995`\)\(\
\[InvisibleSpace]\)\) + 4.4567461156343`\ \((\(-2\) + 2\ x)\) + 
                  2.247430274200378`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.20739093006248863`\ \((\(-0.040444060332830045`\) + 
                  2.769595085438785`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_1_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 2a", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_2a.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(\(0.1938216019493156`\)\(\[InvisibleSpace]\)\) + 
      9.588074716240127`\/k\^2 - 2.9474159800975333`\/k - 
      0.011873126834583421`\ k - 0.00845009639940476`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.4740911436992787`\)\((\((1 - x)\)\^0.21574751304522177`\ \
\((\(\(0.15293110385710867`\)\(\[InvisibleSpace]\)\) + 
              0.08626775996042924`\ \((\(\(1.3468947642604316`\)\(\
\[InvisibleSpace]\)\) + 3.0374091410014326`\ \((\(-2\) + 2\ x)\) + 
                    1.28229200211514`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.12164487247163723`\ \((\(\(0.6898386567445005`\)\(\
\[InvisibleSpace]\)\) + 
                    1.741656369345943`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6184956178871859`\)\
\((\((1 - x)\)\^0.1717298311990992`\ \((\(\(0.10945101233600711`\)\(\
\[InvisibleSpace]\)\) + 
              0.09215811102194228`\ \((\(\(1.2723403142604843`\)\(\
\[InvisibleSpace]\)\) + 2.7724674535438236`\ \((\(-2\) + 2\ x)\) + 
                    1.1340298951673022`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.13638240818081818`\ \((\(\(0.7902254490862851`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5532342133119135`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5163133956737938`\)\
\((\((1 - x)\)\^0.20807671316244805`\ \((\(\(0.14384813353570997`\)\(\
\[InvisibleSpace]\)\) + 
              0.10651574712057069`\ \((\(\(1.333763029023916`\)\(\
\[InvisibleSpace]\)\) + 2.971809949345797`\ \((\(-2\) + 2\ x)\) + 
                    1.2421691343582726`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.1517449802783527`\ \((\(\(0.7243901088362418`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6917633174886542`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.46202546778661835`\)\((\((1 - x)\)\^0.22289238350328291`\ \
\((\(\(0.1692973929020288`\)\(\[InvisibleSpace]\)\) + 
              0.12459507422200865`\ \((\(\(1.3591790825668117`\)\(\
\[InvisibleSpace]\)\) + 3.0685550194063866`\ \((\(-2\) + 2\ x)\) + 
                    1.297906630251939`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.17406797456768`\ \((\(\(0.6849178512899012`\)\(\
\[InvisibleSpace]\)\) + 
                    1.7608669157166643`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.33800303030287987`\)\((\((1 - x)\)\^0.2513249120268771`\ \
\((\(\(0.23223812230008514`\)\(\[InvisibleSpace]\)\) + 
              0.15186470234618044`\ \((\(\(1.4085694737429741`\)\(\
\[InvisibleSpace]\)\) + 3.279417064539127`\ \((\(-2\) + 2\ x)\) + 
                    1.4250957835319813`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.20838047340922447`\ \((\(\(0.589327942329757`\)\(\
\[InvisibleSpace]\)\) + 
                    1.9133218817239972`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.2493597494697057`\)\
\((\((1 - x)\)\^0.2615704949902495`\ \((\(\(0.2833960589665773`\)\(\
\[InvisibleSpace]\)\) + 
              0.17007274353668672`\ \((\(\(1.4265653044100963`\)\(\
\[InvisibleSpace]\)\) + 3.4061634733809227`\ \((\(-2\) + 2\ x)\) + 
                    1.5107030401187467`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.23177740765200686`\ \((\(\(0.5109302444599552`\)\(\
\[InvisibleSpace]\)\) + 
                    2.012210745520544`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.16877606575623633`\)\((\((1 - x)\)\^0.2620436755701154`\ \
\((\(\(0.3304380705833112`\)\(\[InvisibleSpace]\)\) + 
              0.181663228768953`\ \((\(\(1.427398957308321`\)\(\
\[InvisibleSpace]\)\) + 3.498553216812686`\ \((\(-2\) + 2\ x)\) + 
                    1.5826965144671934`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.24753487216551584`\ \((\(\(0.4308197413263517`\)\(\
\[InvisibleSpace]\)\) + 
                    2.093267609813879`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.2618111010755085`\ x\^0.3703115497169773`\ \
\((\(\(0.4140258786611632`\)\(\[InvisibleSpace]\)\) + 
            0.05523178765130234`\ \((\(\(1.4269891779364479`\)\(\
\[InvisibleSpace]\)\) + 4.107587666015124`\ \((\(-2\) + 2\ x)\) + 
                  2.10305470014904`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.06124382613774467`\ \((\(-0.1085004486414688`\) + 
                  2.632122650792486`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.25470457694730053`\ \
x\^0.2646695269869917`\ \((\(\(0.38952143365116004`\)\(\[InvisibleSpace]\)\) \
+ 0.055125709794280484`\ \((\(\(1.4144940761799023`\)\(\[InvisibleSpace]\)\) \
+ 3.9675744500652264`\ \((\(-2\) + 2\ x)\) + 
                  1.9881710234221994`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.08548463882887174`\ \((\(-0.009964950039691167`\) + 
                  2.5193741039342923`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.25211969796845857`\ \
x\^0.24550888512620886`\ \((\(\(0.3983057290217595`\)\(\[InvisibleSpace]\)\) \
+ 0.05388489805361122`\ \((\(\(1.4099617180045412`\)\(\[InvisibleSpace]\)\) + 
                  3.9385391140825052`\ \((\(-2\) + 2\ x)\) + 
                  1.9663792860469347`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.10691417132777567`\ \((\(\(0.006610812842249708`\)\(\
\[InvisibleSpace]\)\) + 
                  2.4976285830946674`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.26622640726732316`\ \
x\^0.3766083071465775`\ \((\(\(0.47958348193656053`\)\(\[InvisibleSpace]\)\) \
+ 0.05272319447062818`\ \((\(\(1.4347778608642179`\)\(\[InvisibleSpace]\)\) + 
                  4.1277441135574495`\ \((\(-2\) + 2\ x)\) + 
                  2.1141349338691136`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.12364525665061969`\ \((\(-0.11038189987925434`\) + 
                  2.6428347144139006`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.2894289223208781`\ \
x\^0.38435165060420745`\ \((\(\(0.5427434259891802`\)\(\[InvisibleSpace]\)\) \
+ 0.07368009417840027`\ \((\(\(1.4760279340192295`\)\(\[InvisibleSpace]\)\) + 
                  4.205429748957628`\ \((\(-2\) + 2\ x)\) + 
                  2.146305533865857`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.16907842237407422`\ \((\(-0.09492272828332937`\) + 
                  2.6737805729250854`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.30389366222428377`\ \
x\^0.39065499172328644`\ \((\(\(0.5903226600494803`\)\(\[InvisibleSpace]\)\) \
+ 0.10428156799236821`\ \((\(\(1.5020161723064693`\)\(\[InvisibleSpace]\)\) + 
                  4.255923614304533`\ \((\(-2\) + 2\ x)\) + 
                  2.1680298012916714`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.20014007943507794`\ \((\(-0.08676132949900267`\) + 
                  2.69454865394757`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.31436513858793264`\ \
x\^0.5284238757308586`\ \((\(\(0.6976101348208088`\)\(\[InvisibleSpace]\)\) + 
            0.12451004264540563`\ \((\(\(1.520960428061604`\)\(\
\[InvisibleSpace]\)\) + 4.446808464844048`\ \((\(-2\) + 2\ x)\) + 
                  2.326227052860997`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.20902529391828722`\ \((\(-0.21405873714292595`\) + 
                  2.8427890143187913`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_2a_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 2b", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_2b.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(-0.9534409760154304`\) - 0.5528773337710792`\/k\^2 + 
      1.8704370615470611`\/k - 0.008819841471860067`\ k + 
      0.38426683326540906`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.9131199316256897`\)\((\((1 - x)\)\^0.1463870726531503`\ \
\((\(\(0.027831138724562506`\)\(\[InvisibleSpace]\)\) + 
              0.0620486551362835`\ \((\(\(1.2302951964997049`\)\(\
\[InvisibleSpace]\)\) + 2.396727860641201`\ \((\(-2\) + 2\ x)\) + 
                    0.9025936580275534`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.10765947665963733`\ \((\(\(1.05950700427884`\)\(\
\[InvisibleSpace]\)\) + 
                    1.2332671410274607`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6583197925421836`\)\
\((\((1 - x)\)\^0.19288213036339433`\ \((\(\(0.08692202177505408`\)\(\
\[InvisibleSpace]\)\) + 
              0.057057306353393435`\ \((\(\(1.3079249536518522`\)\(\
\[InvisibleSpace]\)\) + 2.7789982294501008`\ \((\(-2\) + 2\ x)\) + 
                    1.1198210727653666`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.1073508983304509`\ \((\(\(0.851201922905578`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5345623378212108`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6504228550866071`\)\
\((\((1 - x)\)\^0.21025189582725043`\ \((\(\(0.08960780033695093`\)\(\
\[InvisibleSpace]\)\) + 
              0.06855054066576219`\ \((\(\(1.3374807735903522`\)\(\
\[InvisibleSpace]\)\) + 2.8289334951453298`\ \((\(-2\) + 2\ x)\) + 
                    1.1390692198199757`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.1201584778251714`\ \((\(\(0.8606747509138576`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5598290407406434`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5628005739281482`\)\
\((\((1 - x)\)\^0.2513911097493645`\ \((\(\(0.11821698273240346`\)\(\
\[InvisibleSpace]\)\) + 
              0.08450630303532292`\ \((\(\(1.4086854096545556`\)\(\
\[InvisibleSpace]\)\) + 3.0265344150520836`\ \((\(-2\) + 2\ x)\) + 
                    1.2396387006410787`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.13876251222782002`\ \((\(\(0.8141916836775127`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6885905358212163`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.3739103441340603`\)\
\((\((1 - x)\)\^0.32763183857654105`\ \((\(\(0.1959073607508685`\)\(\
\[InvisibleSpace]\)\) + 
              0.1118508234817568`\ \((\(\(1.545119068689334`\)\(\
\[InvisibleSpace]\)\) + 3.4375880963760994`\ \((\(-2\) + 2\ x)\) + 
                    1.4597740201467502`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.1729170772917914`\ \((\(\(0.7015421827106014`\)\(\
\[InvisibleSpace]\)\) + 
                    1.9537214944424806`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.32258188075046085`\)\((\((1 - x)\)\^0.3626613150301711`\ \
\((\(\(0.23172193259314824`\)\(\[InvisibleSpace]\)\) + 
              0.13641431110150476`\ \((\(\(1.609753587254963`\)\(\
\[InvisibleSpace]\)\) + 3.591339036995739`\ \((\(-2\) + 2\ x)\) + 
                    1.535270300126269`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.20098466775938767`\ \((\(\(0.685243195780632`\)\(\
\[InvisibleSpace]\)\) + 
                    2.04007943427971`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.2685352380280675`\)\
\((\((1 - x)\)\^0.391160516444282`\ \((\(\(0.2687657689849864`\)\(\
\[InvisibleSpace]\)\) + 
              0.1571359361783303`\ \((\(\(1.6632440494789018`\)\(\
\[InvisibleSpace]\)\) + 3.733349136699843`\ \((\(-2\) + 2\ x)\) + 
                    1.6091767384775197`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.22384669937981655`\ \((\(\(0.6596957544723495`\)\(\
\[InvisibleSpace]\)\) + 
                    2.1226252784162147`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.26597161873196745`\ x\^0.36866549098465046`\ \
\((\(\(0.35894680181019617`\)\(\[InvisibleSpace]\)\) + 
            0.044128797172999286`\ \((\(\(1.4343278790834026`\)\(\
\[InvisibleSpace]\)\) + 4.117992267503922`\ \((\(-2\) + 2\ x)\) + 
                  2.105653003630723`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.024572028140105763`\ \((\(-0.102693872252683`\) + 
                  2.6346371097166177`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.23993523226922164`\ \
x\^0.19196474855639212`\ \((\(\(0.30998548601592096`\)\(\[InvisibleSpace]\)\) \
+ 0.03148174040097481`\ \((\(\(1.388687306245875`\)\(\[InvisibleSpace]\)\) + 
                  3.843616840337679`\ \((\(-2\) + 2\ x)\) + 
                  1.9012296824020578`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.049670006627664236`\ \((\(\(0.047970483712829526`\)\(\
\[InvisibleSpace]\)\) + 
                  2.4318999808256136`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.2513419813630658`\ \
x\^0.32087145800987826`\ \((\(\(0.36028778613521606`\)\(\[InvisibleSpace]\)\) \
+ 0.027857327551689355`\ \((\(\(1.4085993678423545`\)\(\[InvisibleSpace]\)\) \
+ 4.021137041224828`\ \((\(-2\) + 2\ x)\) + 
                  2.041615286976203`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.05275773605127468`\ \((\(-0.06952947664681247`\) + 
                  2.5722134393729443`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.25903853188954545`\ \
x\^0.37307406524065867`\ \((\(\(0.39991323660088174`\)\(\[InvisibleSpace]\)\) \
+ 0.013632954974763507`\ \((\(\(1.4221082783360637`\)\(\[InvisibleSpace]\)\) \
+ 4.102541154539271`\ \((\(-2\) + 2\ x)\) + 
                  2.103044314420265`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.07109963126849533`\ \((\(-0.11403553335111322`\) + 
                  2.632112597130204`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.3048227982270658`\ \
x\^0.38125133968326436`\ \((\(\(0.4621450617666021`\)\(\[InvisibleSpace]\)\) \
+ 0.006669888782667256`\ \((\(\(1.5036926665000883`\)\(\[InvisibleSpace]\)\) \
+ 4.247873854505453`\ \((\(-2\) + 2\ x)\) + 
                  2.1591520860102142`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.11648755048062026`\ \((\(-0.07642854145619854`\) + 
                  2.6860741379103303`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.3443242217390505`\ \
x\^0.39555672918544565`\ \((\(\(0.5175012545963422`\)\(\[InvisibleSpace]\)\) \
+ 0.00897243758386524`\ \((\(\(1.5757659174466772`\)\(\[InvisibleSpace]\)\) + 
                  4.383746749836385`\ \((\(-2\) + 2\ x)\) + 
                  2.215823809751551`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.1538080258275252`\ \((\(-0.05123250744639518`\) + 
                  2.7398809509244964`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.374821538403637`\ \
x\^0.4104939638715799`\ \((\(\(0.5642944689776406`\)\(\[InvisibleSpace]\)\) + 
            0.01661591095927965`\ \((\(\(1.6324779004310903`\)\(\
\[InvisibleSpace]\)\) + 4.494724392228183`\ \((\(-2\) + 2\ x)\) + 
                  2.2642411192550367`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.18396048196563072`\ \((\(-0.03567242546794286`\) + 
                  2.7853155022752167`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_2b_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 3a", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_3a.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(-0.0036246242228034076`\) + 2.192102384106045`\/k\^2 - 
      1.497432761056963`\/k - 0.2756736584113894`\ k - 
      0.003521740282956598`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.8786081408306836`\)\((\((1 - x)\)\^0.4502679077515215`\ \
\((\(\(0.03381285601221247`\)\(\[InvisibleSpace]\)\) + 
              0.035823337367357065`\ \((\(\(1.7767724560027485`\)\(\
\[InvisibleSpace]\)\) + 3.1506276982709425`\ \((\(-2\) + 2\ x)\) + 
                    1.1481367154650217`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0017329701951257046`\ \((\(\(1.328876048582205`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5716597669208376`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.9867049900842563`\)\
\((\((1 - x)\)\^0.5677758946730004`\ \((\(\(0.021517644500456554`\)\(\
\[InvisibleSpace]\)\) + 
              0.023751797912972012`\ \((\(\(2.0128485752953633`\)\(\
\[InvisibleSpace]\)\) + 3.313805825622406`\ \((\(-2\) + 2\ x)\) + 
                    1.1553747398879126`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.0001321488652643999`\ \((\(\(1.5544808847572567`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5810709045887439`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.9282459216852015`\)\
\((\((1 - x)\)\^0.5248213156921931`\ \((\(\(0.019891983598993756`\)\(\
\[InvisibleSpace]\)\) + 
              0.02411596429395116`\ \((\(\(1.924950680240732`\)\(\
\[InvisibleSpace]\)\) + 3.2779444512953533`\ \((\(-2\) + 2\ x)\) + 
                    1.1673473963461944`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0032578316283990534`\ \((\(\(1.4530672373773945`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5965753940069916`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^1.0518686578555063`\)\
\((\((1 - x)\)\^0.5700113843416539`\ \((\(\(0.010555785707813872`\)\(\
\[InvisibleSpace]\)\) + 
              0.012712227655048819`\ \((\(\(2.017473565652025`\)\(\
\[InvisibleSpace]\)\) + 3.2358277372332647`\ \((\(-2\) + 2\ x)\) + 
                    1.1073981896801544`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0018525679913708787`\ \((\(\(1.6218800421971602`\)\(\
\[InvisibleSpace]\)\) + 
                    1.5181427264861473`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.9743580734124896`\)\
\((\((1 - x)\)\^0.5788206945746069`\ \((\(\(0.00858374821764128`\)\(\
\[InvisibleSpace]\)\) + 
              0.013445651749213429`\ \((\(\(2.0357477400958257`\)\(\
\[InvisibleSpace]\)\) + 3.358221052849447`\ \((\(-2\) + 2\ x)\) + 
                    1.1734610207740956`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0034702210460013207`\ \((\(\(1.5531787679870965`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6044626211621176`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^1.306841614277314`\)\((\((1 - x)\)\^0.6758387459568216`\ \
\((\(\(0.002312268826287923`\)\(\[InvisibleSpace]\)\) - 
              0.011006955553982463`\ \((\(\(2.242137124203477`\)\(\
\[InvisibleSpace]\)\) + 3.1695271570043`\ \((\(-2\) + 2\ x)\) + 
                    0.9976430676981551`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0034904771044353333`\ \((\(\(1.9826803602341356`\)\(\
\[InvisibleSpace]\)\) + 
                    1.3689971316795073`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^1.1469911558181947`\)\
\((\((1 - x)\)\^0.5650221678757529`\ \((\(\(0.0027966099632372407`\)\(\
\[InvisibleSpace]\)\) + 
              0.00348247354966751`\ \((\(\(2.007158276909137`\)\(\
\[InvisibleSpace]\)\) + 3.101151574269339`\ \((\(-2\) + 2\ x)\) + 
                    1.0331131234162072`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0008303812923981396`\ \((\(\(1.7120133236939474`\)\(\
\[InvisibleSpace]\)\) + 
                    1.418031012057558`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.21589337089758087`\ x\^0.4697236534439679`\ \
\((\(\(0.07402630522272383`\)\(\[InvisibleSpace]\)\) + 
            0.054583098147805764`\ \((\(\(1.3471450301451318`\)\(\
\[InvisibleSpace]\)\) + 4.083467165952853`\ \((\(-2\) + 2\ x)\) + 
                  2.158673734307225`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.015502808021020987`\ \((\(-0.253830282546387`\) + 
                  2.685617024341549`\ \((\(-1\) + 
                        2\ x)\))\))\), \(\(1\/\(\((1 - \
x)\)\^0.17303542125969307`\ \
x\^0.18052724477505264`\)\)\((\(\(0.028510123745762336`\)\(\[InvisibleSpace]\)\
\) + 0.02865365233144496`\ \
\((\(\(0.7554174966157202`\)\(\[InvisibleSpace]\)\) + 
                2.417473634505226`\ \((\(-2\) + 2\ x)\) + 
                1.2062584870712971`\ \((\(-2\) + 2\ x)\)\^2)\) - 
          0.009415259199471086`\ \((\(\(0.007491823515359569`\)\(\
\[InvisibleSpace]\)\) + 
                1.6464373339652545`\ \((\(-1\) + 
                      2\ x)\))\))\)\), \(\(1\/\((1 - \
x)\)\^0.1314285484890035`\)\((x\^0.04850913519729678`\ \
\((\(\(0.021764215065975447`\)\(\[InvisibleSpace]\)\) + 
              0.02437404751061822`\ \((\(\(0.8114939089454579`\)\(\
\[InvisibleSpace]\)\) + 2.7253867530400324`\ \((\(-2\) + 2\ x)\) + 
                    1.4283049670073364`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.00676009836201301`\ \((\(-0.17993768368630028`\) + 
                    1.9170805867082934`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/\((1 - \
x)\)\^0.24842034061161694`\)\((x\^0.6107665935110764`\ \
\((\(\(0.012266338684706174`\)\(\[InvisibleSpace]\)\) + 
              0.015539296195946179`\ \((\(\(0.6582258218973704`\)\(\
\[InvisibleSpace]\)\) + 2.9447086521997203`\ \((\(-2\) + 2\ x)\) + 
                    1.8334648221608116`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.003307161172483544`\ \((\(-0.8591869341226934`\) + 
                    2.362346252899459`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/\((1 - \
x)\)\^0.6020241309772485`\)\((x\^0.6590313841076785`\ \
\((\(\(0.0031700363338182836`\)\(\[InvisibleSpace]\)\) + 
              0.005373486500963243`\ \((\(\(0.2781803306735829`\)\(\
\[InvisibleSpace]\)\) + 2.1368111856519336`\ \((\(-2\) + 2\ x)\) + 
                    1.5502875748528109`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0006794808997284946`\ \((\(-1.261055515084927`\) + 
                    2.05700725313043`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/\((1 - \
x)\)\^0.7939737253136304`\)\((x\^0.6624328085986046`\ \
\((\(\(0.0013267092527985977`\)\(\[InvisibleSpace]\)\) + 
              0.0025361836081332022`\ \((\(\(0.1242365502737565`\)\(\
\[InvisibleSpace]\)\) + 1.7297185111522282`\ \((\(-2\) + 2\ x)\) + 
                    1.387064574470631`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.00011041496804865302`\ \((\(-1.4564065339122352`\) + 
                    1.8684590832849743`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/\((1 - \
x)\)\^0.4666986032738429`\)\((x\^0.6053674398101602`\ \
\((\(\(0.0017681285849478119`\)\(\[InvisibleSpace]\)\) + 
              0.0042064620795593515`\ \((\(\(0.40885588823811353`\)\(\
\[InvisibleSpace]\)\) + 2.4062626554609987`\ \((\(-2\) + 2\ x)\) + 
                    1.6237388627475697`\ \((\(-2\) + 2\ x)\)\^2)\) - 
              0.0001313461916160462`\ \((\(-1.072066043084003`\) + 
                    2.138668836536317`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_3a_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 3b", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_3b.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(\(0.3814062497992905`\)\(\[InvisibleSpace]\)\) + 
      9.21474765359311`\/k\^2 - 2.8054347238814357`\/k - 
      0.0007731066750167008`\ k + 0.42168678100536616`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/\(\((1 - x)\)\^0.2654210779524651`\ \
x\^0.088370676333097`\)\)\((\(\(0.1520982522881667`\)\(\[InvisibleSpace]\)\) \
- 0.19310199999118188`\ \((\(\(0.6370925573820266`\)\(\[InvisibleSpace]\)\) + 
                2.295028523182324`\ \((\(-2\) + 2\ x)\) + 
                1.2060782906751901`\ \((\(-2\) + 2\ x)\)\^2)\) - 
          0.07198397004294234`\ \((\(-0.1770504016193681`\) + 
                1.646208245714438`\ \((\(-1\) + 
                      2\ x)\))\))\)\), \(\(1\/\(\((1 - \
x)\)\^0.1652611042446479`\ \
x\^0.027183805683102264`\)\)\((\(\(0.18692081524883714`\)\(\[InvisibleSpace]\)\
\) - 0.20645354667646432`\ \
\((\(\(0.7657639599211084`\)\(\[InvisibleSpace]\)\) + 
                2.575565262865739`\ \((\(-2\) + 2\ x)\) + 
                1.3362400842328561`\ \((\(-2\) + 2\ x)\)\^2)\) - 
          0.06756306211754848`\ \((\(-0.13807729856154563`\) + 
                1.80755509007225`\ \((\(-1\) + 
                      2\ x)\))\))\)\), \((1 - x)\)\^0.09491514114148715`\ \
x\^0.12879642368404653`\ \((\(\(0.29724072725630774`\)\(\[InvisibleSpace]\)\) \
- 0.21833335059717993`\ \((\(\(1.146877153721185`\)\(\[InvisibleSpace]\)\) + 
                  3.376701083912964`\ \((\(-2\) + 2\ x)\) + 
                  1.7020034772519281`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.05900691642059036`\ \((\(-0.033881282542559385`\) + 
                  2.2237115648255337`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.3351584880715021`\ \
x\^0.2829601021398929`\ \((\(\(0.44395893908256845`\)\(\[InvisibleSpace]\)\) \
- 0.24737985064879078`\ \((\(\(1.558903338170441`\)\(\[InvisibleSpace]\)\) + 
                  4.224440168390719`\ \((\(-2\) + 2\ x)\) + 
                  2.088612590380586`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.05628953973910925`\ \((\(\(0.05219838593160919`\)\(\
\[InvisibleSpace]\)\) + 
                  2.6181185902113953`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.6458851394739555`\ \
x\^0.49510239824174024`\ \((\(\(0.7325466206578141`\)\(\[InvisibleSpace]\)\) \
- 0.3002809201958812`\ \((\(\(2.177411515907579`\)\(\[InvisibleSpace]\)\) + 
                  5.478288694394403`\ \((\(-2\) + 2\ x)\) + 
                  2.6610956656540496`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.06104523871588112`\ \((\(\(0.1507827412322153`\)\(\
\[InvisibleSpace]\)\) + 
                  3.140987537715696`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.8443062784427525`\ \
x\^0.6347276290809983`\ \((\(\(1.0008216783880737`\)\(\[InvisibleSpace]\)\) - 
            0.34462404638695876`\ \((\(\(2.622885963573054`\)\(\
\[InvisibleSpace]\)\) + 6.36987213226389`\ \((\(-2\) + 2\ x)\) + 
                  3.067597331533904`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.07024747621794933`\ \((\(\(0.2095786493617542`\)\(\
\[InvisibleSpace]\)\) + 
                  3.4790339075237506`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.9879527682745488`\ \
x\^0.734911802937713`\ \((\(\(1.2476551239806086`\)\(\[InvisibleSpace]\)\) - 
            0.382303022821899`\ \((\(\(2.9699544885824953`\)\(\
\[InvisibleSpace]\)\) + 7.055848134869734`\ \((\(-2\) + 2\ x)\) + 
                  3.3785392911530305`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.07974220819830065`\ \((\(\(0.25304096533683584`\)\(\
\[InvisibleSpace]\)\) + 
                  3.722864571212262`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.5202452390146334`\ x\^0.30748639158513635`\ \
\((\(\(0.4718881975622957`\)\(\[InvisibleSpace]\)\) - 
            0.19590453654431558`\ \((\(\(1.9156954128806467`\)\(\
\[InvisibleSpace]\)\) + 4.823411209122395`\ \((\(-2\) + 2\ x)\) + 
                  2.3099076333117177`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.0041946716954915095`\ \((\(\(0.21275884742949702`\)\(\
\[InvisibleSpace]\)\) + 
                  2.8277316305997697`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.6309909250962163`\ \
x\^0.5013388534777242`\ \((\(\(0.6410977132967124`\)\(\[InvisibleSpace]\)\) - 
            0.19262945687084992`\ \((\(\(2.1455611614212136`\)\(\
\[InvisibleSpace]\)\) + 5.436061073466448`\ \((\(-2\) + 2\ x)\) + 
                  2.651059897182862`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.028212840171776058`\ \((\(\(0.12965207161849213`\)\(\
\[InvisibleSpace]\)\) + 
                  3.1323297785739403`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.7856203712802688`\ \
x\^0.4717467001310815`\ \((\(\(0.7146262704803211`\)\(\[InvisibleSpace]\)\) - 
            0.23908251826680613`\ \((\(\(2.4870302408056775`\)\(\
\[InvisibleSpace]\)\) + 5.929704221070638`\ \((\(-2\) + 2\ x)\) + 
                  2.7978176815186258`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.02482963292303991`\ \((\(\(0.31387367114918735`\)\(\
\[InvisibleSpace]\)\) + 
                  3.2573670714113505`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.8836406159733493`\ \
x\^0.33655681255118325`\ \((\(\(0.6623879162231372`\)\(\[InvisibleSpace]\)\) \
- 0.3140880292417905`\ \((\(\(2.715871293058904`\)\(\[InvisibleSpace]\)\) + 
                  6.084766356159815`\ \((\(-2\) + 2\ x)\) + 
                  2.753782970531202`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.10651340662801657`\ \((\(\(0.5470838034221661`\)\(\
\[InvisibleSpace]\)\) + 
                  3.2201974285245325`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.149196385804488`\ \
x\^0.5347306315160106`\ \((\(\(1.0375533219804611`\)\(\[InvisibleSpace]\)\) - 
            0.37430317394366763`\ \((\(\(3.3841207452797804`\)\(\
\[InvisibleSpace]\)\) + 7.3753030171588545`\ \((\(-2\) + 2\ x)\) + 
                  3.327887415113175`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.13215067142766812`\ \((\(\(0.6144657542884774`\)\(\
\[InvisibleSpace]\)\) + 
                  3.6839270173204985`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.32338572142428`\ \
x\^0.5111915315684401`\ \((\(\(1.1295383230864224`\)\(\[InvisibleSpace]\)\) - 
            0.43638605230517735`\ \((\(\(3.860753465971251`\)\(\
\[InvisibleSpace]\)\) + 8.033582505859313`\ \((\(-2\) + 2\ x)\) + 
                  3.52596430851842`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.22661450284489854`\ \((\(\(0.8121941898558398`\)\(\
\[InvisibleSpace]\)\) + 
                  3.83457725299272`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.3651821601017016`\ \
x\^0.45342955851614813`\ \((\(\(1.073526948440747`\)\(\[InvisibleSpace]\)\) - 
            0.4933654939476331`\ \((\(\(3.9796344052825265`\)\(\
\[InvisibleSpace]\)\) + 8.107753095974893`\ \((\(-2\) + 2\ x)\) + 
                  3.5047038266773893`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.276010816531603`\ \((\(\(0.9117526015855535`\)\(\
\[InvisibleSpace]\)\) + 
                  3.81861171861785`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_3b_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 4a", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_4a.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(\(0.2884081574812196`\)\(\[InvisibleSpace]\)\) - 
      0.018843801931274218`\/k\^2 - 0.6224854180168093`\/k - 
      0.03981135577203391`\ k - 0.3715266139153625`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.5798780147652112`\)\((\((1 - x)\)\^0.33875562486008653`\ \
\((\(\(0.09381090127341765`\)\(\[InvisibleSpace]\)\) + 
              0.03862976358454148`\ \((\(\(1.5655111239773036`\)\(\
\[InvisibleSpace]\)\) + 3.2261702644549715`\ \((\(-2\) + 2\ x)\) + 
                    1.2962854096972107`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.0631530375738298`\ \((\(\(0.9186336396252976`\)\(\
\[InvisibleSpace]\)\) + 
                    1.7588776100948755`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5364091458759781`\)\
\((\((1 - x)\)\^0.31069601101450195`\ \((\(\(0.09768284232615874`\)\(\
\[InvisibleSpace]\)\) + 
              0.039389878311105624`\ \((\(\(1.5143100221519146`\)\(\
\[InvisibleSpace]\)\) + 3.2052667963427566`\ \((\(-2\) + 2\ x)\) + 
                    1.3088693094023327`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.06275151224785122`\ \((\(\(0.84710515689048`\)\(\
\[InvisibleSpace]\)\) + 
                    1.7742868651385237`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5727581469663234`\)\
\((\((1 - x)\)\^0.24402025199958838`\ \((\(\(0.08396406248461184`\)\(\
\[InvisibleSpace]\)\) + 
              0.03467049437803012`\ \((\(\(1.3958033196923538`\)\(\
\[InvisibleSpace]\)\) + 2.9971831310468486`\ \((\(-2\) + 2\ x)\) + 
                    1.2258629173525017`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.06522408466923271`\ \((\(\(0.8167783989659119`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6712621050332648`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5083889401518744`\)\
\((\((1 - x)\)\^0.20694138636578135`\ \((\(\(0.09168754940819895`\)\(\
\[InvisibleSpace]\)\) + 
              0.03779535856123818`\ \((\(\(1.3318244482441677`\)\(\
\[InvisibleSpace]\)\) + 2.977773538414045`\ \((\(-2\) + 2\ x)\) + 
                    1.247592218897621`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.06604013085710347`\ \((\(\(0.7153303265176558`\)\(\
\[InvisibleSpace]\)\) + 
                    1.6985524462139068`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.3683137540892005`\)\
\((\((1 - x)\)\^0.19612010962035079`\ \((\(\(0.11590515294836948`\)\(\
\[InvisibleSpace]\)\) + 
              0.05433044258934114`\ \((\(\(1.3134117131292753`\)\(\
\[InvisibleSpace]\)\) + 3.1051012017470967`\ \((\(-2\) + 2\ x)\) + 
                    1.3530368924891893`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.06945028527646303`\ \((\(\(0.5644338637095513`\)\(\
\[InvisibleSpace]\)\) + 
                    1.8278063555311501`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.2776743375131187`\)\
\((\((1 - x)\)\^0.17827546117983586`\ \((\(\(0.1326528524875063`\)\(\
\[InvisibleSpace]\)\) + 
              0.07032223366124296`\ \((\(\(1.2833042617991954`\)\(\
\[InvisibleSpace]\)\) + 3.159154125176934`\ \((\(-2\) + 2\ x)\) + 
                    1.4142610002854175`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.07310049319528243`\ \((\(\(0.4559497986929546`\)\(\
\[InvisibleSpace]\)\) + 
                    1.900601123666717`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.20422424798969296`\)\((\((1 - x)\)\^0.16363795104002926`\ \
\((\(\(0.1470340152255389`\)\(\[InvisibleSpace]\)\) + 
              0.08718977210112012`\ \((\(\(1.2588456160703334`\)\(\
\[InvisibleSpace]\)\) + 3.2015499003738084`\ \((\(-2\) + 2\ x)\) + 
                    1.4646928961065553`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.07679046788672343`\ \((\(\(0.3678621990297222`\)\(\
\[InvisibleSpace]\)\) + 
                    1.9594137030503365`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.48584283080898677`\ x\^0.3319325054816671`\ \
\((\(\(0.3218932659671258`\)\(\[InvisibleSpace]\)\) + 
            0.06349753318799259`\ \((\(\(1.8467858743377252`\)\(\
\[InvisibleSpace]\)\) + 4.745194724678746`\ \((\(-2\) + 2\ x)\) + 
                  2.299147981834984`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.01515635090120347`\ \((\(\(0.15391032532731969`\)\(\
\[InvisibleSpace]\)\) + 
                  2.817775336290654`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.4725002095042292`\ \
x\^0.3366909325166047`\ \((\(\(0.316047859436367`\)\(\[InvisibleSpace]\)\) + 
            0.06975254793623192`\ \((\(\(1.820378538247114`\)\(\
\[InvisibleSpace]\)\) + 4.7091129483440834`\ \((\(-2\) + 2\ x)\) + 
                  2.2898910373088524`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.01980994378313111`\ \((\(\(0.13580927698762452`\)\(\
\[InvisibleSpace]\)\) + 
                  2.809191142020834`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.4250136861795415`\ \
x\^0.30517626042940255`\ \((\(\(0.2919452266682618`\)\(\[InvisibleSpace]\)\) \
+ 0.06724993212984264`\ \((\(\(1.727838845989273`\)\(\[InvisibleSpace]\)\) + 
                  4.522880836288011`\ \((\(-2\) + 2\ x)\) + 
                  2.2055633730489226`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.02696978592608358`\ \((\(\(0.11983742575013895`\)\(\
\[InvisibleSpace]\)\) + 
                  2.730189946608944`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.43253640518666436`\ \
x\^0.5451758829512611`\ \((\(\(0.35253017166735506`\)\(\[InvisibleSpace]\)\) \
+ 0.08923502942768913`\ \((\(\(1.7423484786858978`\)\(\[InvisibleSpace]\)\) + 
                  4.837964975126925`\ \((\(-2\) + 2\ x)\) + 
                  2.474988416917672`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.022584284550952592`\ \((\(-0.11263947776459671`\) + 
                  2.9777122881379254`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.43889285492409486`\ \
x\^0.8634605721132901`\ \((\(\(0.44747503642921055`\)\(\[InvisibleSpace]\)\) \
+ 0.13275921769546448`\ \((\(\(1.7546527514378536`\)\(\[InvisibleSpace]\)\) + 
                  5.246489516279835`\ \((\(-2\) + 2\ x)\) + 
                  2.8515748047722145`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.02153911678448839`\ \((\(-0.4245677171891952`\) + 
                  3.3023534270373847`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.4532571824083778`\ \
x\^1.2069404127128918`\ \((\(\(0.5673937299618241`\)\(\[InvisibleSpace]\)\) + 
            0.18453704872514032`\ \((\(\(1.7826068103149575`\)\(\
\[InvisibleSpace]\)\) + 5.716331610836753`\ \((\(-2\) + 2\ x)\) + 
                  3.297204902586917`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.018473380785886492`\ \((\(-0.753683230304514`\) + 
                  3.6601975951212697`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.46296846053569024`\ \
x\^1.4919996480512094`\ \((\(\(0.6801249737507158`\)\(\[InvisibleSpace]\)\) + 
            0.2379282969534568`\ \((\(\(1.8016225885289288`\)\(\
\[InvisibleSpace]\)\) + 6.101965087204858`\ \((\(-2\) + 2\ x)\) + 
                  3.6883346332125178`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.01916715108934858`\ \((\(-1.0290311875155191`\) + 
                  3.9549681085868995`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_4a_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 4b", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_4b.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(-0.17959798072993982`\) + 19.910598034026645`\/k\^2 - 
      4.443333990208367`\/k - 0.006238855759253004`\ k + 
      0.3234239036795359`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.687781866843172`\)\((\((1 - x)\)\^0.09272731336916928`\ \
\((\(\(0.08169772416804397`\)\(\[InvisibleSpace]\)\) + 
              0.07176636414368294`\ \((\(\(1.143390147376086`\)\(\
\[InvisibleSpace]\)\) + 2.5164475115538836`\ \((\(-2\) + 2\ x)\) + 
                    1.0235885059115155`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.10079375475825555`\ \((\(\(0.7805091802123413`\)\(\
\[InvisibleSpace]\)\) + 
                    1.4049454465259972`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6449483301833201`\)\
\((\((1 - x)\)\^0.12932819000145113`\ \((\(\(0.09301828754196195`\)\(\
\[InvisibleSpace]\)\) + 
              0.08615809104696621`\ \((\(\(1.2023551753667023`\)\(\
\[InvisibleSpace]\)\) + 2.6450300350913`\ \((\(-2\) + 2\ x)\) + 
                    1.082065393461011`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.11489581208309697`\ \((\(\(0.7742765201847712`\)\(\
\[InvisibleSpace]\)\) + 
                    1.484379859818131`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6860582526184681`\)\
\((\((1 - x)\)\^0.1343196001370175`\ \((\(\(0.0853236841466904`\)\(\
\[InvisibleSpace]\)\) + 
              0.10487803913748832`\ \((\(\(1.2105002776960103`\)\(\
\[InvisibleSpace]\)\) + 2.6126860901333533`\ \((\(-2\) + 2\ x)\) + 
                    1.0552806216589865`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.13595214365745573`\ \((\(\(0.8203778527554856`\)\(\
\[InvisibleSpace]\)\) + 
                    1.4482613475185495`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.6769076696142261`\)\
\((\((1 - x)\)\^0.16019781860460341`\ \((\(\(0.09434384052369194`\)\(\
\[InvisibleSpace]\)\) + 
              0.13452544702428346`\ \((\(\(1.2531283984497417`\)\(\
\[InvisibleSpace]\)\) + 2.6821989814056564`\ \((\(-2\) + 2\ x)\) + 
                    1.0812525141328784`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.16550937519612174`\ \((\(\(0.8371054882188296`\)\(\
\[InvisibleSpace]\)\) + 
                    1.4832901489903771`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.5008803972219025`\)\
\((\((1 - x)\)\^0.2534238501777136`\ \((\(\(0.17063831107886038`\)\(\
\[InvisibleSpace]\)\) + 
              0.17680861443340698`\ \((\(\(1.4122475991860188`\)\(\
\[InvisibleSpace]\)\) + 3.1013235327705715`\ \((\(-2\) + 2\ x)\) + 
                    1.2911298641707138`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.2154165016061856`\ \((\(\(0.7543042473996161`\)\(\
\[InvisibleSpace]\)\) + 
                    1.7525434529558113`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.4505851709392276`\)\
\((\((1 - x)\)\^0.29998880657906785`\ \((\(\(0.20551181971833443`\)\(\
\[InvisibleSpace]\)\) + 
              0.2066230768723448`\ \((\(\(1.4949798519049686`\)\(\
\[InvisibleSpace]\)\) + 3.276798233698667`\ \((\(-2\) + 2\ x)\) + 
                    1.3710630893046725`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.24784885744684668`\ \((\(\(0.7505739775182955`\)\(\
\[InvisibleSpace]\)\) + 
                    1.8494036356398402`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.4208557975071381`\)\
\((\((1 - x)\)\^0.3305333773991985`\ \((\(\(0.2291854033702962`\)\(\
\[InvisibleSpace]\)\) + 
              0.22664510678047395`\ \((\(\(1.5504262228862582`\)\(\
\[InvisibleSpace]\)\) + 3.390550358704285`\ \((\(-2\) + 2\ x)\) + 
                    1.4219876498523225`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.26974765091498254`\ \((\(\(0.7513891749063366`\)\(\
\[InvisibleSpace]\)\) + 
                    1.9096775798920604`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.15104495279598457`\ x\^0.40544805283661`\ \
\((\(\(0.3346893942265031`\)\(\[InvisibleSpace]\)\) + 
            0.044512423843356894`\ \((\(\(1.2379747180765475`\)\(\
\[InvisibleSpace]\)\) + 3.8250881647101065`\ \((\(-2\) + 2\ x)\) + 
                  2.02564193809327`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.02981773265316675`\ \((\(-0.25440310004062544`\) + 
                  2.5564930056325945`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.14905624037255957`\ \
x\^0.26450934897817685`\ \((\(\(0.3152969381200767`\)\(\[InvisibleSpace]\)\) \
+ 0.038310962299516284`\ \((\(\(1.2346932419558403`\)\(\[InvisibleSpace]\)\) \
+ 3.667972215857617`\ \((\(-2\) + 2\ x)\) + 
                  1.883249452768772`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.05695429116060907`\ \((\(-0.11545310860561728`\) + 
                  2.4135655893507364`\ \((\(-1\) + 
                        2\ x)\))\))\), \
\(\(1\/x\^0.0030612341935196408`\)\((\((1 - x)\)\^0.13777215769262743`\ \((\(\
\(0.2600659749389298`\)\(\[InvisibleSpace]\)\) + 
              0.03184781287073241`\ \((\(\(1.2161488202565822`\)\(\
\[InvisibleSpace]\)\) + 3.3506488673356682`\ \((\(-2\) + 2\ x)\) + 
                    1.6201404371754675`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.10231425280365053`\ \((\(\(0.14083339188614707`\)\(\
\[InvisibleSpace]\)\) + 
                    2.134710923499108`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \((1 - \
x)\)\^0.15341839089813658`\ x\^0.3874711899225363`\ \
\((\(\(0.4093081625020677`\)\(\[InvisibleSpace]\)\) - 
            0.004589799160534156`\ \((\(\(1.2418961876800914`\)\(\
\[InvisibleSpace]\)\) + 3.812508371739415`\ \((\(-2\) + 2\ x)\) + 
                  2.009848575548134`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.09852997579688347`\ \((\(-0.23405279902439974`\) + 
                  2.540889580820673`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.16840375068523203`\ \
x\^0.7266542024362818`\ \((\(\(0.6020199286980914`\)\(\[InvisibleSpace]\)\) - 
            0.0631578731447712`\ \((\(\(1.2667855376502748`\)\(\
\[InvisibleSpace]\)\) + 4.223029137342516`\ \((\(-2\) + 2\ x)\) + 
                  2.3833168014120836`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.09096799348362625`\ \((\(-0.5582504517510497`\) + 
                  2.8950579531215137`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.15977506372429257`\ \
x\^1.261824163190071`\ \((\(\(0.8639489152628816`\)\(\[InvisibleSpace]\)\) - 
            0.08780762100745851`\ \((\(\(1.2524266310804897`\)\(\
\[InvisibleSpace]\)\) + 4.774829876036127`\ \((\(-2\) + 2\ x)\) + 
                  2.9965173687955073`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.013167208238467992`\ \((\(-1.1020490994657786`\) + 
                  3.421599226914364`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.20590587067700192`\ \
x\^1.0489544729098323`\ \((\(\(0.8556205961938209`\)\(\[InvisibleSpace]\)\) - 
            0.12002655671508487`\ \((\(\(1.33005741980513`\)\(\
\[InvisibleSpace]\)\) + 4.692910705414482`\ \((\(-2\) + 2\ x)\) + 
                  2.7948371108768386`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.06491617353525518`\ \((\(-0.8430486022328304`\) + 
                  3.2548603435868344`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_4b_fits.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Fill fit files 5", "Subsection"],

Cell[BoxData[
    \(<< "\<jan_5.m\>"\)], "Input"],

Cell["\<\
This really needs a linear K term for a decent fit.  Both the K and \
Log[K] terms are more important than the 1/K^2 term.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitMassSquared = 
      Fit[Transpose[{klist, massSquared}], {1, 1/k, 1/k^2, k, Log[k]}, 
        k]\)], "Input"],

Cell[BoxData[
    \(\(\(1.7745483417447325`\)\(\[InvisibleSpace]\)\) + 
      19.460177541742247`\/k\^2 - 8.455827979374511`\/k - 
      0.00032246272439975874`\ k + 0.3364744924482843`\ Log[k]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitAll]; 
    Map[\((sfitAll[#] = 
            NonLinearFit[
              Drop[Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                    Table[i, {i, 
                            1/2, # - 
                              1/2}]/#\ quark[#]\ *#}], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, \(-0.5\)}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmcv\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.03978978952863379`\ x\^0.12142265991172982`\ \
\((\(\(0.30454365319031934`\)\(\[InvisibleSpace]\)\) - 
            0.2165732970818871`\ \((\(\(1.0604762979683173`\)\(\
\[InvisibleSpace]\)\) + 3.2241044384496282`\ \((\(-2\) + 2\ x)\) + 
                  1.6443095749921381`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.0580640454428664`\ \((\(-0.08163287038309602`\) + 
                  2.1612124494403635`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.38937346356987956`\ \
x\^0.26139797171256635`\ \((\(\(0.48122459197591066`\)\(\[InvisibleSpace]\)\) \
- 0.2348868747345723`\ \((\(\(1.6598660424210214`\)\(\[InvisibleSpace]\)\) + 
                  4.361528194511399`\ \((\(-2\) + 2\ x)\) + 
                  2.122362938494587`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.035984127000796506`\ \((\(\(0.12797549185731322`\)\(\
\[InvisibleSpace]\)\) + 
                  2.650771435282446`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^0.7935234446596492`\ \
x\^0.43568840672746834`\ \((\(\(0.7832231657988966`\)\(\[InvisibleSpace]\)\) \
- 0.27048073271215956`\ \((\(\(2.5051248956017313`\)\(\[InvisibleSpace]\)\) + 
                  5.907201229641176`\ \((\(-2\) + 2\ x)\) + 
                  2.764430591912546`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.014257473037426917`\ \((\(\(0.35783503793218085`\)\(\
\[InvisibleSpace]\)\) + 
                  3.2292118513871175`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.1489507964306724`\ \
x\^0.6152891985541963`\ \((\(\(1.205314909943926`\)\(\[InvisibleSpace]\)\) - 
            0.3494755866520794`\ \((\(\(3.3834701609553477`\)\(\
\[InvisibleSpace]\)\) + 7.501178663297233`\ \((\(-2\) + 2\ x)\) + 
                  3.432777840599786`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.01819320265369923`\ \((\(\(0.5336615978764762`\)\(\
\[InvisibleSpace]\)\) + 
                  3.764239994984869`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.5452312366464247`\ \
x\^0.8643722394040402`\ \((\(\(2.0266868605329615`\)\(\[InvisibleSpace]\)\) - 
            0.48764763494053276`\ \((\(\(4.511716642323556`\)\(\
\[InvisibleSpace]\)\) + 9.589147610582593`\ \((\(-2\) + 2\ x)\) + 
                  4.334176655518466`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            
            0.07428784267744523`\ \((\(\(0.6808589972423845`\)\(\
\[InvisibleSpace]\)\) + 
                  4.4096034760504645`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.7246775747473952`\ \
x\^0.9667876693218064`\ \((\(\(2.532873306535515`\)\(\[InvisibleSpace]\)\) - 
            0.595931705165548`\ \
\((\(\(5.074272730539372`\)\(\[InvisibleSpace]\)\) + 
                  10.599436481019382`\ \((\(-2\) + 2\ x)\) + 
                  4.760530233564611`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.11575746323409662`\ \((\(\(0.7578899054255889`\)\(\
\[InvisibleSpace]\)\) + 
                  4.691465244069201`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^1.8507663419489957`\ \
x\^1.0580998775396053`\ \((\(\(3.019861903233209`\)\(\[InvisibleSpace]\)\) - 
            0.675959437672001`\ \
\((\(\(5.488817539169126`\)\(\[InvisibleSpace]\)\) + 
                  11.376831578543056`\ \((\(-2\) + 2\ x)\) + 
                  5.1029457774127644`\ \((\(-2\) + 2\ x)\)\^2)\) - 
            0.16763590059047934`\ \((\(\(0.7926664644093904`\)\(\
\[InvisibleSpace]\)\) + 
                  4.908866219488601`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[
                  Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                      Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
                  Floor[#*0.3]], \(-1\)], \((a + \ 
                    b\ JacobiP[1, d, e, 2\ x - 1] + \ 
                    c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
              x, {{a, 0.5}, {b, 0}, {c, 0}, {d, 0.3}, {e, 0.3}}])\) &, 
      klist]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\(\(1\/x\^0.4182255250881273`\)\((\((1 - x)\)\^0.7210843163961644`\ \
\((\(\(0.10668904979657146`\)\(\[InvisibleSpace]\)\) - 
              0.20577228535335826`\ \((\(\(2.341607770270509`\)\(\
\[InvisibleSpace]\)\) + 4.4936786281497465`\ \((\(-2\) + 2\ x)\) + 
                    1.7764668733286033`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.25870972702346146`\ \((\(\(1.1393098414842917`\)\(\
\[InvisibleSpace]\)\) + 
                    2.3028587913080374`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \
\(\(1\/x\^0.22557440376965543`\)\((\((1 - x)\)\^1.0639159806438807`\ \
\((\(\(0.27619725354866254`\)\(\[InvisibleSpace]\)\) - 
              0.16989238913529492`\ \((\(\(3.161832577900536`\)\(\
\[InvisibleSpace]\)\) + 5.880178048277385`\ \((\(-2\) + 2\ x)\) + 
                    2.321400954704442`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.2864425339593471`\ \((\(\(1.2894903844135361`\)\(\
\[InvisibleSpace]\)\) + 
                    2.838341576874225`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.2131780509310224`\)\
\((\((1 - x)\)\^1.0628445877531152`\ \((\(\(0.2429594081534169`\)\(\
\[InvisibleSpace]\)\) - 
              0.3035421845912926`\ \((\(\(3.1590861904877174`\)\(\
\[InvisibleSpace]\)\) + 5.895465158479913`\ \((\(-2\) + 2\ x)\) + 
                    2.333699872693738`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.29778166324337524`\ \((\(\(1.2760226386841376`\)\(\
\[InvisibleSpace]\)\) + 
                    2.8496665368220926`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \(\(1\/x\^0.010426491225314104`\
\)\((\((1 - x)\)\^1.4274135038534341`\ \((\(\(0.42863178855680917`\)\(\
\[InvisibleSpace]\)\) - 
              0.4046743431269386`\ \((\(\(4.15987491127172`\)\(\
\[InvisibleSpace]\)\) + 7.569420466713428`\ \((\(-2\) + 2\ x)\) + 
                    2.9908451602942`\ \((\(-2\) + 2\ x)\)\^2)\) + 
              0.38227237991681995`\ \((\(\(1.4378399950787482`\)\(\
\[InvisibleSpace]\)\) + 
                    3.41698701262812`\ \((\(-1\) + 
                          2\ x)\))\))\))\)\), \((1 - x)\)\^2.164997891588645`\
\ x\^0.38329352118911275`\ \((\(\(1.232022494214666`\)\(\[InvisibleSpace]\)\) \
- 0.5327075658173742`\ \((\(\(6.5911047726746075`\)\(\[InvisibleSpace]\)\) + 
                  11.554311018069374`\ \((\(-2\) + 2\ x)\) + 
                  4.541478626735145`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.6493789690980901`\ \((\(\(1.7817043703995323`\)\(\
\[InvisibleSpace]\)\) + 
                  4.548291412777758`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^2.3796592250247506`\ \
x\^0.5346462946616269`\ \((\(\(1.7159735879678022`\)\(\[InvisibleSpace]\)\) - 
            0.7163653591951548`\ \((\(\(7.400877851159825`\)\(\
\[InvisibleSpace]\)\) + 12.951321364454625`\ \((\(-2\) + 2\ x)\) + 
                  5.111664412484892`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.7598390067003175`\ \((\(\(1.8450129303631235`\)\(\
\[InvisibleSpace]\)\) + 
                  4.914305519686378`\ \((\(-1\) + 
                        2\ x)\))\))\), \((1 - x)\)\^2.309309772127683`\ \
x\^0.5158653377931376`\ \((\(\(1.5823393952344922`\)\(\[InvisibleSpace]\)\) - 
            0.873304735355999`\ \
\((\(\(7.130420470013731`\)\(\[InvisibleSpace]\)\) + 
                  12.551242012768373`\ \((\(-2\) + 2\ x)\) + 
                  4.969730021395233`\ \((\(-2\) + 2\ x)\)\^2)\) + 
            0.7173213891451563`\ \((\(\(1.7934444343345453`\)\(\
\[InvisibleSpace]\)\) + 
                  4.825175109920821`\ \((\(-1\) + 2\ x)\))\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit, sfitAll, 
        fitMassSquared] >> "\<jan_5_fits.m\>"\)], "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(unpolarized) quark distribution", "Section"],

Cell["Normalization (should be 1):", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, quark[10]]\)], "Input"],

Cell[BoxData[
    \(0.999999999999`\)], "Output"]
}, Open  ]],

Cell["\<\
This is the one that Simon used (he says that it is not \
well-justified):\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[Drop[Drop[str, 1], \(-1\)], 
        a*\((x^b)\)*\((1 + c*Sqrt[x])\)*\((1 - x)\)^d, 
        x, {{a, 0.2}, {b, 0.1}, {c, 0.1}, {d, 0.1}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(\(0.1787060983502144`\ \((1 + 0.36853653596896474`\ \@x)\)\ \
x\^0.07673168978493321`\)\/\((1 - x)\)\^0.06719316001734296`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 5], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.0062527810802487874`\)], "Output"]
}, Open  ]],

Cell["This should work better:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[
        Drop[Drop[str, 
            2], \(-1\)], \((a + \ b\ JacobiP[1, d, e, 2\ x - 1] + \ 
              c\ JacobiP[2, d, e, 2\ x - 1])\)\ \((1 - x)\)^d\ x^e, 
        x, {{a, 0.3}, {b, 0}, {c, 0}, {d, 0.25}, {e, 0.25}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(\((1 - x)\)\^0.2476906439108751`\ x\^0.004015476597149985`\ \
\((\(\(0.27132245844412634`\)\(\[InvisibleSpace]\)\) + 
          0.04347031483026967`\ \((\(\(1.4022112934068047`\)\(\
\[InvisibleSpace]\)\) + 3.6544147119068087`\ \((\(-2\) + 2\ x)\) + 
                1.7281623518321723`\ \((\(-2\) + 2\ x)\)\^2)\) + 
          0.06997319941590245`\ \((\(\(0.24367516731372513`\)\(\
\[InvisibleSpace]\)\) + 
                2.2517061205080253`\ \((\(-1\) + 2\ x)\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 1], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.003576669333409623`\)], "Output"]
}, Open  ]],

Cell["Check I did the Jacobi stuff right:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{e = 0.3, d = 0.8}, 
      Integrate[
        JacobiP[2, d, e, 2\ x - 1]\ JacobiP[4, d, e, 2\ x - 1]\ x^
            e\ \((1 - x)\)^d, {x, 0, 1}]]\)], "Input"],

Cell[BoxData[
    \(4.333628123020961`*^-14\)], "Output"]
}, Open  ]],

Cell["Input fits from files for the following:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Fit[Map[{#, sfit[#]} &, klist], {1, 1/kvalue, 1/kvalue^2}, kvalue] /. 
        kvalue \[Rule] Infinity, {x, 0, 1}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Put all the fits together:", "Text"],

Cell[BoxData[
    \(<< "\<jan_5.m\>"; \ << "\<jan_5_fits.m\>";\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[
      Show[Plot[\((Fit[
                  Map[{#, sfitAll[#]} &, klist], {1, 1/kvalue, 1/kvalue^2}, 
                  kvalue] /. kvalue \[Rule] Infinity)\), {x, 0, 1}, 
            DisplayFunction \[Rule] Identity, 
            PlotStyle \[Rule] {Thickness[0.007]}], ##, Axes \[Rule] False, \ 
          Frame \[Rule] True, PlotRange \[Rule] {0, 0.6}, 
          RotateLabel \[Rule] False, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          FrameLabel \[Rule] {"\<x\>", "\<x q(x)\>"}, 
          DisplayFunction \[Rule] $DisplayFunction] &, 
      Map[{ListPlot[
              Transpose[{Table[i, {i, 1/2, # - 1/2}]/#, 
                  Table[i, {i, 1/2, # - 1/2}]/#\ quark[#]\ *#}], 
              DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[1 - #/50]}], 
            Plot[sfit[#], {x, 0, 1}, DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[1 - #/50], PointSize[0.1]}], 
            Plot[sfitAll[#], {x, 0, 1}, DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[1 - #/50], PointSize[0.1]}]} &, 
        klist]]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 -2.18738e-17 1.03006 [
[.02381 -0.0125 -5.5 -14.5 ]
[.02381 -0.0125 5.5 0 ]
[.21429 -0.0125 -10.75 -14.5 ]
[.21429 -0.0125 10.75 0 ]
[.40476 -0.0125 -10.75 -14.5 ]
[.40476 -0.0125 10.75 0 ]
[.59524 -0.0125 -10.75 -14.5625 ]
[.59524 -0.0125 10.75 0 ]
[.78571 -0.0125 -10.75 -14.5 ]
[.78571 -0.0125 10.75 0 ]
[.97619 -0.0125 -5.5 -14.5 ]
[.97619 -0.0125 5.5 0 ]
[ 0 0 -0.125 0 ]
[.5 -0.0125 -5.5 -29.0625 ]
[.5 -0.0125 5.5 -14.5625 ]
[-0.0125 .10301 -21.5 -7.25 ]
[-0.0125 .10301 0 7.25 ]
[-0.0125 .20601 -21.5 -7.25 ]
[-0.0125 .20601 0 7.25 ]
[-0.0125 .30902 -21.5 -7.25 ]
[-0.0125 .30902 0 7.25 ]
[-0.0125 .41202 -21.5 -7.25 ]
[-0.0125 .41202 0 7.25 ]
[-0.0125 .51503 -21.5 -7.3125 ]
[-0.0125 .51503 0 7.3125 ]
[ 0 0 -0.125 0 ]
[-0.01875 .30902 -58.875 -7.25 ]
[-0.01875 .30902 -21.5 7.25 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
gsave
.02381 -0.0125 -66.5 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0) show
1.000 setlinewidth
grestore
.21429 0 m
.21429 .00625 L
s
gsave
.21429 -0.0125 -71.75 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.40476 0 m
.40476 .00625 L
s
gsave
.40476 -0.0125 -71.75 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.59524 0 m
.59524 .00625 L
s
gsave
.59524 -0.0125 -71.75 -18.5625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5625 translate 1 -1 scale
63.000 14.562 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.78571 0 m
.78571 .00625 L
s
gsave
.78571 -0.0125 -71.75 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.97619 0 m
.97619 .00625 L
s
gsave
.97619 -0.0125 -66.5 -18.5 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.2619 0 m
.2619 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.5 0 m
.5 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.7381 0 m
.7381 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
gsave
.5 -0.0125 -66.5 -33.0625 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
70.000 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
0 .10301 m
.00625 .10301 L
s
gsave
-0.0125 .10301 -82.5 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.1) show
1.000 setlinewidth
grestore
0 .20601 m
.00625 .20601 L
s
gsave
-0.0125 .20601 -82.5 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
0 .30902 m
.00625 .30902 L
s
gsave
-0.0125 .30902 -82.5 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.3) show
1.000 setlinewidth
grestore
0 .41202 m
.00625 .41202 L
s
gsave
-0.0125 .41202 -82.5 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
0 .51503 m
.00625 .51503 L
s
gsave
-0.0125 .51503 -82.5 -11.3125 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.625 translate 1 -1 scale
63.000 14.625 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
gsave
-0.01875 .30902 -119.875 -11.25 Mabsadd m
1 1 Mabs scale
currentpoint translate
/MISOfy
{
    /newfontname exch def
    /oldfontname exch def

    oldfontname findfont
    dup length dict begin
        {1 index /FID ne {def} {pop pop} ifelse} forall
        /Encoding ISOLatin1Encoding def
        currentdict
    end

    newfontname exch definefont pop
} def

0 22.5 translate 1 -1 scale
63.000 14.500 moveto
%%IncludeResource: font Times-Roman
%%IncludeFont: Times-Roman
%%BeginResource: font Times-Roman-MISO
%%BeginFont: Times-Roman-MISO
/Times-Roman /Times-Roman-MISO MISOfy
%%EndFont
%%EndResource
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
73.500 14.500 moveto
(q) show
%%IncludeResource: font Math2
%%IncludeFont: Math2
/Math2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
80.500 14.500 moveto
(H) show
84.938 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(x) show
%%IncludeResource: font Math2
%%IncludeFont: Math2
/Math2 findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
91.938 14.500 moveto
(L) show
96.375 14.500 moveto
%%IncludeResource: font Times-Roman-MISO
%%IncludeFont: Times-Roman-MISO
/Times-Roman-MISO findfont 14.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
.02381 .61178 m
.02381 .61803 L
s
.21429 .61178 m
.21429 .61803 L
s
.40476 .61178 m
.40476 .61803 L
s
.59524 .61178 m
.59524 .61803 L
s
.78571 .61178 m
.78571 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.07143 .61428 m
.07143 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.16667 .61428 m
.16667 .61803 L
s
.2619 .61428 m
.2619 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.35714 .61428 m
.35714 .61803 L
s
.45238 .61428 m
.45238 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.54762 .61428 m
.54762 .61803 L
s
.64286 .61428 m
.64286 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.7381 .61428 m
.7381 .61803 L
s
.83333 .61428 m
.83333 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.92857 .61428 m
.92857 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.007 w
.02381 .01309 m
.02499 .01867 L
.02605 .01596 L
.02729 .01404 L
.02846 .01293 L
.02954 .0123 L
.03053 .01199 L
.03163 .01188 L
.03279 .01198 L
.03395 .01227 L
.0352 .01277 L
.03638 .01339 L
.03746 .01407 L
.04262 .01852 L
.04748 .02414 L
.05278 .03149 L
.06244 .0474 L
.10458 .14037 L
.14523 .24739 L
.18435 .35135 L
.22594 .45129 L
.266 .52927 L
.28438 .55748 L
.30455 .5824 L
.32449 .60054 L
.33539 .60766 L
.34556 .61253 L
.35056 .6143 L
.3532 .61506 L
.35601 .61574 L
.35845 .61623 L
.36115 .61666 L
.36233 .61681 L
.36361 .61694 L
.3648 .61704 L
.36591 .61712 L
.367 .61717 L
.36816 .6172 L
.36945 .61722 L
.37063 .61721 L
.37185 .61717 L
.37254 .61714 L
.37318 .6171 L
.37435 .61702 L
.37559 .61691 L
.37777 .61665 L
.37982 .61634 L
.38447 .61539 L
.38945 .61398 L
.39487 .61201 L
Mistroke
.40475 .60725 L
.41437 .60121 L
.42329 .59439 L
.44385 .57446 L
.46247 .55168 L
.50259 .48956 L
.54119 .41695 L
.58225 .33178 L
.62179 .24796 L
.65981 .17111 L
.70029 .09868 L
.73925 .04238 L
.7591 .01978 L
.78066 .00024 L
Mfstroke
.78066 .00024 m
.78099 0 L
s
.8 0 1 r
.008 w
.07143 .18149 Mdot
.16667 .24822 Mdot
.2619 .32371 Mdot
.35714 .38138 Mdot
.45238 .40464 Mdot
.54762 .38541 Mdot
.64286 .32821 Mdot
.7381 .2483 Mdot
.83333 .16547 Mdot
.92857 .00085 Mdot
.5 Mabswid
.15906 0 m
.1819 .0899 L
.20331 .15357 L
.22348 .20371 L
.26204 .2786 L
.28158 .308 L
.30304 .33474 L
.34253 .37107 L
.36204 .38372 L
.38295 .39389 L
.39312 .39767 L
.40419 .40097 L
.41464 .40335 L
.4193 .40419 L
.42431 .40494 L
.42933 .40554 L
.43394 .40596 L
.43834 .40625 L
.44055 .40636 L
.44172 .4064 L
.44298 .40644 L
.4437 .40646 L
.44437 .40648 L
.44568 .4065 L
.44687 .40651 L
.44814 .40651 L
.44879 .40651 L
.44949 .40651 L
.45076 .40649 L
.45153 .40648 L
.45227 .40646 L
.45365 .40643 L
.45611 .40634 L
.4584 .40623 L
.46358 .40587 L
.46867 .4054 L
.47417 .40474 L
.48414 .40317 L
.50276 .39903 L
.52274 .39295 L
.54382 .38485 L
.58184 .36631 L
.62232 .3418 L
.66128 .31432 L
.70269 .28172 L
.74259 .24763 L
.78097 .21289 L
.8218 .1743 L
.86112 .13581 L
.89892 .09753 L
Mistroke
.93918 .05424 L
.97619 0 L
Mfstroke
.02381 .02318 m
.02499 .08159 L
.02605 .08892 L
.02729 .09458 L
.02846 .09874 L
.03279 .11008 L
.0375 .1194 L
.04262 .12803 L
.06244 .15578 L
.10458 .20448 L
.14545 .24526 L
.1848 .27978 L
.22263 .30871 L
.26292 .33492 L
.30169 .35558 L
.3214 .36434 L
.34292 .37255 L
.36374 .37913 L
.38262 .38394 L
.4023 .38775 L
.4131 .38933 L
.42327 .39048 L
.43303 .39127 L
.43795 .39156 L
.44077 .39169 L
.44335 .39178 L
.44572 .39185 L
.44796 .3919 L
.44919 .39192 L
.45033 .39193 L
.45158 .39194 L
.45221 .39195 L
.4529 .39195 L
.45406 .39195 L
.45512 .39195 L
.45628 .39194 L
.4575 .39193 L
.45857 .39191 L
.45956 .39189 L
.46182 .39184 L
.46445 .39176 L
.46687 .39167 L
.47234 .39139 L
.47717 .39106 L
.48228 .39064 L
.49113 .3897 L
.50087 .38837 L
.52031 .3848 L
.54121 .3796 L
.58248 .36516 L
Mistroke
.62224 .34598 L
.66049 .32266 L
.70119 .29261 L
.74037 .25861 L
.782 .21709 L
.82212 .17195 L
.86072 .12394 L
.90178 .06833 L
.92224 .03912 L
.94132 .01127 L
Mfstroke
.94132 .01127 m
.94903 0 L
s
.56 0 1 r
.008 w
.06349 .15604 Mdot
.14286 .22841 Mdot
.22222 .30547 Mdot
.30159 .37163 Mdot
.38095 .41426 Mdot
.46032 .42346 Mdot
.53968 .39663 Mdot
.61905 .33961 Mdot
.69841 .2646 Mdot
.77778 .18627 Mdot
.85714 .11751 Mdot
.93651 .0006 Mdot
.5 Mabswid
.12919 0 m
.1458 .06572 L
.18551 .18012 L
.22371 .26277 L
.26435 .3279 L
.30348 .3725 L
.32489 .39046 L
.34506 .40365 L
.35483 .40883 L
.36514 .41348 L
.38361 .41983 L
.39309 .42214 L
.39831 .42315 L
.40325 .42394 L
.40785 .42453 L
.41027 .42479 L
.41284 .42502 L
.415 .42519 L
.41738 .42533 L
.41955 .42543 L
.42159 .4255 L
.42276 .42553 L
.42402 .42555 L
.4252 .42557 L
.42628 .42557 L
.42755 .42556 L
.42826 .42556 L
.42892 .42555 L
.43013 .42552 L
.43142 .42549 L
.43396 .42539 L
.43635 .42527 L
.43896 .42509 L
.44177 .42486 L
.44673 .42435 L
.45144 .42373 L
.46027 .42225 L
.46988 .42018 L
.48012 .41747 L
.49852 .41132 L
.54015 .39198 L
.58027 .36717 L
.61887 .33859 L
.65993 .30411 L
.69947 .26784 L
.73749 .2309 L
.77797 .1902 L
.81693 .15043 L
.85834 .10838 L
.89824 .06894 L
Mistroke
.93662 .03294 L
.97619 0 L
Mfstroke
.02381 .00287 m
.02499 .04276 L
.02605 .05103 L
.02729 .05773 L
.02846 .06278 L
.03279 .0768 L
.0375 .08844 L
.04262 .0992 L
.06244 .13361 L
.10458 .19367 L
.14545 .24397 L
.1848 .28639 L
.22263 .3216 L
.26292 .3529 L
.30169 .37683 L
.3214 .38663 L
.34292 .39548 L
.36351 .40214 L
.38262 .40674 L
.39319 .40863 L
.40315 .40999 L
.40749 .41045 L
.41209 .41086 L
.41712 .4112 L
.41934 .41132 L
.42175 .41142 L
.42308 .41147 L
.4243 .41151 L
.42545 .41154 L
.42671 .41156 L
.42784 .41158 L
.4289 .41159 L
.43002 .4116 L
.43123 .4116 L
.43247 .4116 L
.43316 .41159 L
.43381 .41159 L
.43513 .41157 L
.43653 .41154 L
.43908 .41147 L
.44139 .41138 L
.44592 .41115 L
.45082 .41081 L
.45973 .40993 L
.46919 .40864 L
.4796 .40681 L
.50065 .4018 L
.52009 .39562 L
.54098 .38737 L
.58226 .36631 L
Mistroke
.62202 .34038 L
.66027 .31061 L
.70096 .2743 L
.74015 .23549 L
.78178 .19102 L
.8219 .14616 L
.8605 .10254 L
.90156 .05784 L
.92202 .03731 L
.9411 .02008 L
.95805 .0073 L
.96293 .00431 L
.96754 .0019 L
.96955 .00102 L
.97169 .00024 L
Mfstroke
.97169 .00024 m
.97252 0 L
s
.2 0 1 r
.008 w
.05556 .1278 Mdot
.11905 .20202 Mdot
.18254 .27712 Mdot
.24603 .34704 Mdot
.30952 .40445 Mdot
.37302 .44075 Mdot
.43651 .44977 Mdot
.5 .42992 Mdot
.56349 .38461 Mdot
.62698 .32132 Mdot
.69048 .24974 Mdot
.75397 .17953 Mdot
.81746 .11845 Mdot
.88095 .07117 Mdot
.94444 .00037 Mdot
.5 Mabswid
.14643 0 m
.18458 .14738 L
.22428 .25935 L
.26248 .33762 L
.28215 .36855 L
.30312 .39536 L
.32453 .41678 L
.34471 .43195 L
.35447 .43768 L
.36478 .44265 L
.37445 .44635 L
.38326 .44894 L
.3878 .45 L
.39273 .45094 L
.39544 .45136 L
.39795 .4517 L
.4005 .45198 L
.40289 .4522 L
.40493 .45235 L
.40712 .45247 L
.4083 .45252 L
.40955 .45256 L
.41062 .45258 L
.41178 .45259 L
.41302 .4526 L
.41415 .45259 L
.41545 .45257 L
.41668 .45254 L
.41781 .4525 L
.41886 .45245 L
.42123 .45232 L
.42367 .45213 L
.4259 .45192 L
.43092 .45132 L
.43557 .45059 L
.43999 .44975 L
.44987 .44739 L
.46063 .44407 L
.48034 .4361 L
.50133 .42517 L
.54296 .39704 L
.58308 .36338 L
.62168 .3265 L
.66274 .284 L
.70228 .24131 L
.7403 .19987 L
.78078 .15661 L
.81974 .11697 L
.86115 .07822 L
.90105 .04536 L
Mistroke
.93943 .01898 L
.97619 0 L
Mfstroke
.02381 .00022 m
.02499 .01972 L
.02605 .02634 L
.02729 .03216 L
.02846 .03677 L
.03279 .05042 L
.04262 .07396 L
.06244 .1124 L
.10458 .18292 L
.14299 .24031 L
.18386 .29447 L
.22321 .33921 L
.26105 .37475 L
.28209 .39119 L
.30133 .40409 L
.32144 .41535 L
.34011 .42378 L
.35909 .43033 L
.36912 .43298 L
.37981 .43518 L
.38482 .43599 L
.3901 .4367 L
.39286 .43701 L
.39589 .4373 L
.39868 .43752 L
.40123 .43768 L
.40381 .43781 L
.40626 .4379 L
.4076 .43794 L
.40885 .43796 L
.40949 .43797 L
.41021 .43798 L
.41096 .43799 L
.41166 .43799 L
.41285 .43799 L
.4141 .43798 L
.41517 .43796 L
.41634 .43793 L
.41761 .4379 L
.41896 .43785 L
.4214 .43774 L
.42399 .43759 L
.42673 .43739 L
.43164 .43693 L
.43693 .4363 L
.44274 .43543 L
.45315 .43343 L
.46289 .43105 L
.48318 .42457 L
.50172 .41688 L
Mistroke
.54148 .39504 L
.57972 .36773 L
.62042 .33278 L
.6596 .29444 L
.70124 .24994 L
.74136 .20483 L
.77996 .16088 L
.82102 .11538 L
.86056 .07483 L
.89858 .04107 L
.91976 .02543 L
.93906 .01374 L
.9482 .00919 L
.95814 .00508 L
.96752 .00207 L
.97619 0 L
Mfstroke
0 .4 1 r
.008 w
.04762 .09687 Mdot
.09524 .16686 Mdot
.14286 .23444 Mdot
.19048 .3003 Mdot
.2381 .36246 Mdot
.28571 .41632 Mdot
.33333 .45666 Mdot
.38095 .47911 Mdot
.42857 .48118 Mdot
.47619 .46271 Mdot
.52381 .42588 Mdot
.57143 .37485 Mdot
.61905 .315 Mdot
.66667 .25207 Mdot
.71429 .1914 Mdot
.7619 .13722 Mdot
.80952 .09229 Mdot
.85714 .05778 Mdot
.90476 .03343 Mdot
.95238 .00018 Mdot
.5 Mabswid
.14927 0 m
.18292 .13675 L
.22379 .26588 L
.26314 .35822 L
.30098 .42078 L
.32039 .44391 L
.34126 .4626 L
.35137 .46947 L
.3624 .47545 L
.36744 .47767 L
.37282 .4797 L
.37752 .48118 L
.38249 .48247 L
.38784 .48353 L
.39022 .4839 L
.39277 .48423 L
.39552 .4845 L
.39674 .48459 L
.39808 .48467 L
.39889 .48472 L
.39964 .48475 L
.40035 .48477 L
.40112 .48479 L
.40182 .48481 L
.40247 .48482 L
.40316 .48482 L
.4039 .48482 L
.40517 .4848 L
.40635 .48477 L
.40764 .48472 L
.40903 .48465 L
.41185 .48444 L
.41442 .48418 L
.41939 .48349 L
.42407 .48262 L
.43379 .48016 L
.44408 .47661 L
.46254 .46801 L
.48211 .45601 L
.50042 .44239 L
.53925 .4071 L
.58052 .36226 L
.62029 .31446 L
.65853 .2664 L
.69923 .21519 L
.73841 .16772 L
.78004 .12116 L
.82016 .08169 L
.85877 .04994 L
.87835 .03649 L
Mistroke
.89982 .02395 L
.91892 .01482 L
.92942 .01064 L
.93936 .00724 L
.9486 .00459 L
.95864 .00229 L
.96306 .00148 L
.96776 .00076 L
.97177 .0003 L
.97287 .0002 L
.97405 .0001 L
.97517 4e-05 L
.97619 0 L
Mfstroke
.02381 2e-05 m
.06244 .09421 L
.10458 .17309 L
.14415 .2426 L
.18221 .3034 L
.22272 .35952 L
.26171 .40364 L
.28158 .42205 L
.30316 .43879 L
.32216 .45067 L
.33296 .45622 L
.34309 .46062 L
.35344 .46433 L
.36292 .46703 L
.37305 .46917 L
.37864 .47002 L
.38138 .47036 L
.38395 .47062 L
.38637 .47083 L
.38898 .471 L
.39013 .47106 L
.39135 .47112 L
.3925 .47116 L
.39354 .47118 L
.39475 .47121 L
.39606 .47122 L
.39727 .47122 L
.39841 .47122 L
.39905 .47121 L
.39974 .47119 L
.40097 .47116 L
.4023 .47112 L
.40376 .47105 L
.40614 .47091 L
.40832 .47075 L
.41324 .47026 L
.41758 .46969 L
.42214 .46894 L
.43169 .46692 L
.44189 .46407 L
.46025 .45721 L
.4816 .44653 L
.50175 .43394 L
.54022 .40375 L
.58114 .36403 L
.62054 .32002 L
.65843 .27406 L
.69877 .22316 L
.73759 .17432 L
.77887 .12488 L
Mistroke
.81863 .08203 L
.85687 .04739 L
.87823 .03159 L
.89757 .01981 L
.90727 .01487 L
.91776 .0103 L
.92759 .00676 L
.93675 .00412 L
.94591 .00213 L
.95075 .00134 L
.95594 .00069 L
.95855 .00044 L
.96139 .00023 L
.96269 .00015 L
.96407 8e-05 L
.96526 4e-05 L
Mfstroke
.96526 4e-05 m
.96633 0 L
s
0 1 .4 r
.008 w
.03968 .06412 Mdot
.07143 .12146 Mdot
.10317 .17427 Mdot
.13492 .2259 Mdot
.16667 .27765 Mdot
.19841 .32901 Mdot
.23016 .37845 Mdot
.2619 .42385 Mdot
.29365 .46295 Mdot
.3254 .49357 Mdot
.35714 .51393 Mdot
.38889 .52284 Mdot
.42063 .51978 Mdot
.45238 .50501 Mdot
.48413 .47946 Mdot
.51587 .4447 Mdot
.54762 .40276 Mdot
.57937 .35596 Mdot
.61111 .30671 Mdot
.64286 .25735 Mdot
.6746 .20993 Mdot
.70635 .16615 Mdot
.7381 .12727 Mdot
.76984 .09404 Mdot
.80159 .06675 Mdot
.83333 .04528 Mdot
.86508 .02917 Mdot
.89683 .0177 Mdot
.92857 .01005 Mdot
.96032 6e-05 Mdot
.5 Mabswid
.14054 0 m
.14536 .01834 L
.18463 .16347 L
.22238 .28493 L
.26258 .38811 L
.30127 .45972 L
.32131 .4862 L
.33151 .49697 L
.34241 .50652 L
.35275 .51377 L
.36215 .51886 L
.36668 .52081 L
.3716 .52258 L
.3759 .52382 L
.38052 .52485 L
.38307 .52529 L
.38541 .5256 L
.38653 .52573 L
.38775 .52584 L
.3889 .52593 L
.38998 .52599 L
.39064 .52602 L
.39136 .52605 L
.3926 .52608 L
.39331 .52609 L
.39405 .52609 L
.39469 .52608 L
.39538 .52607 L
.39664 .52603 L
.39734 .526 L
.39798 .52597 L
.40039 .52579 L
.4017 .52566 L
.40309 .52549 L
.40558 .52514 L
.41121 .52403 L
.41607 .52274 L
.42144 .52098 L
.43156 .51671 L
.44082 .51177 L
.46169 .49727 L
.50137 .45872 L
.53953 .41131 L
.58014 .35377 L
.61924 .2952 L
.66079 .23324 L
.70082 .17693 L
.73934 .12821 L
.78031 .08417 L
.81977 .05049 L
Mistroke
.85771 .02662 L
.87719 .01757 L
.8981 .01016 L
.90826 .00738 L
.91931 .00491 L
.92976 .00309 L
.93943 .00182 L
.94376 .00138 L
.94843 .00097 L
.95247 .00068 L
.9569 .00043 L
.96179 .00023 L
.96447 .00014 L
.96696 8e-05 L
.96816 6e-05 L
.96929 4e-05 L
.97033 3e-05 L
.97142 2e-05 L
.97261 1e-05 L
.97371 0 L
.97489 0 L
.97619 0 L
Mfstroke
.02381 0 m
.06244 .07674 L
.10458 .16246 L
.14415 .24315 L
.18221 .31599 L
.22272 .38421 L
.26171 .43781 L
.28158 .4599 L
.30316 .47961 L
.32392 .49425 L
.33381 .49972 L
.34309 .50396 L
.35211 .50725 L
.36181 .50989 L
.36733 .51097 L
.37005 .51139 L
.37258 .51172 L
.37505 .51198 L
.37729 .51216 L
.37849 .51224 L
.37978 .51231 L
.38049 .51234 L
.38115 .51237 L
.38244 .5124 L
.3837 .51242 L
.38489 .51242 L
.38597 .51242 L
.38713 .51239 L
.3884 .51235 L
.38975 .5123 L
.39217 .51215 L
.3944 .51196 L
.39649 .51175 L
.40126 .5111 L
.40645 .51015 L
.4112 .50906 L
.42192 .50584 L
.43173 .50199 L
.44085 .49766 L
.46155 .48523 L
.50059 .45279 L
.54209 .40741 L
.58207 .35567 L
.62054 .30115 L
.66146 .2411 L
.70086 .18427 L
.73875 .13326 L
.77909 .08554 L
.81791 .04829 L
.83766 .03315 L
Mistroke
.85919 .01982 L
.86918 .01478 L
.87982 .01021 L
.88979 .00667 L
.89895 .00404 L
.90926 .00176 L
.91468 .00084 L
.9204 7e-05 L
Mfstroke
.9204 7e-05 m
.92107 0 L
s
.8 1 0 r
.008 w
.03571 .04761 Mdot
.05952 .09493 Mdot
.08333 .13755 Mdot
.10714 .17858 Mdot
.13095 .21965 Mdot
.15476 .26127 Mdot
.17857 .30326 Mdot
.20238 .34502 Mdot
.22619 .38565 Mdot
.25 .42403 Mdot
.27381 .45901 Mdot
.29762 .48942 Mdot
.32143 .51418 Mdot
.34524 .5324 Mdot
.36905 .54342 Mdot
.39286 .54682 Mdot
.41667 .54251 Mdot
.44048 .53067 Mdot
.46429 .51177 Mdot
.4881 .48654 Mdot
.5119 .45591 Mdot
.53571 .42096 Mdot
.55952 .38289 Mdot
.58333 .34291 Mdot
.60714 .30223 Mdot
.63095 .26198 Mdot
.65476 .22316 Mdot
.67857 .18663 Mdot
.70238 .15304 Mdot
.72619 .12289 Mdot
.75 .09645 Mdot
.77381 .07384 Mdot
.79762 .05499 Mdot
.82143 .0397 Mdot
.84524 .02768 Mdot
.86905 .01854 Mdot
.89286 .01185 Mdot
.91667 .00718 Mdot
.94048 .00409 Mdot
.96429 3e-05 Mdot
.5 Mabswid
.14029 0 m
.14233 .00727 L
.18253 .15829 L
.22122 .29005 L
.26236 .40381 L
.28289 .4486 L
.30199 .48277 L
.32041 .5089 L
.34009 .52953 L
.35034 .53737 L
.35573 .54072 L
.36144 .5437 L
.36643 .54583 L
.37192 .54768 L
.37433 .54832 L
.37692 .5489 L
.37937 .54935 L
.38159 .54967 L
.38282 .54981 L
.38414 .54994 L
.38487 .55 L
.38553 .55004 L
.38685 .55011 L
.38798 .55014 L
.38922 .55016 L
.39051 .55015 L
.39172 .55011 L
.39241 .55008 L
.39316 .55004 L
.39451 .54994 L
.39583 .54982 L
.39706 .54968 L
.39983 .54929 L
.40277 .54874 L
.4077 .54753 L
.41233 .54607 L
.42287 .5416 L
.43273 .53605 L
.44309 .52888 L
.46161 .51293 L
.49977 .46927 L
.54038 .4111 L
.57948 .34855 L
.62103 .28 L
.66106 .21612 L
.69958 .15989 L
.74055 .10827 L
.78001 .06813 L
.81795 .03901 L
.83743 .02762 L
Mistroke
.85834 .01794 L
.8685 .01413 L
.87955 .01061 L
.88911 .00805 L
.89967 .00573 L
.90917 .00404 L
.91942 .00262 L
.92506 .002 L
.93024 .00151 L
.93516 .00113 L
.94042 .0008 L
.94505 .00056 L
.94926 .00039 L
.95349 .00025 L
.95749 .00016 L
.95976 .00011 L
.96216 8e-05 L
.96421 5e-05 L
.96646 3e-05 L
.96773 2e-05 L
.96888 2e-05 L
.97022 1e-05 L
.97149 1e-05 L
.97265 0 L
.97374 0 L
.97492 0 L
.97619 0 L
Mfstroke
.02381 0 m
.06244 .06903 L
.10458 .157 L
.14415 .24326 L
.18221 .32269 L
.22272 .3978 L
.26171 .45689 L
.28158 .48111 L
.30316 .50254 L
.31323 .51076 L
.32392 .5182 L
.33393 .52399 L
.34309 .52827 L
.3525 .53167 L
.35769 .53311 L
.36262 .53419 L
.36482 .53458 L
.36716 .53494 L
.36918 .5352 L
.37137 .53542 L
.37271 .53554 L
.37395 .53562 L
.37511 .53569 L
.37638 .53574 L
.37752 .53578 L
.37859 .5358 L
.37971 .5358 L
.38092 .53579 L
.38214 .53577 L
.38283 .53574 L
.38348 .53572 L
.38479 .53565 L
.3862 .53556 L
.38876 .53533 L
.39109 .53507 L
.39648 .53422 L
.40216 .53299 L
.41177 .53013 L
.42228 .5259 L
.44189 .51504 L
.46284 .49941 L
.50434 .45758 L
.54432 .40614 L
.58279 .34943 L
.62371 .28494 L
.66311 .22235 L
.701 .16495 L
.74134 .11005 L
.78016 .06591 L
.79991 .04739 L
Mistroke
.82144 .03052 L
.84207 .01771 L
.8612 .00876 L
.87179 .005 L
.88176 .00219 L
.89071 .00027 L
Mfstroke
.89071 .00027 m
.89228 0 L
s
1 0 0 r
.008 w
.03333 .03782 Mdot
.05238 .07791 Mdot
.07143 .11359 Mdot
.09048 .1475 Mdot
.10952 .18108 Mdot
.12857 .21501 Mdot
.14762 .24952 Mdot
.16667 .28457 Mdot
.18571 .31986 Mdot
.20476 .35498 Mdot
.22381 .38937 Mdot
.24286 .42238 Mdot
.2619 .45335 Mdot
.28095 .48159 Mdot
.3 .50644 Mdot
.31905 .5273 Mdot
.3381 .54365 Mdot
.35714 .55509 Mdot
.37619 .56132 Mdot
.39524 .5622 Mdot
.41429 .55771 Mdot
.43333 .54798 Mdot
.45238 .53328 Mdot
.47143 .51398 Mdot
.49048 .49058 Mdot
.50952 .46365 Mdot
.52857 .43384 Mdot
.54762 .40184 Mdot
.56667 .36834 Mdot
.58571 .33405 Mdot
.60476 .29963 Mdot
.62381 .26572 Mdot
.64286 .23286 Mdot
.6619 .20156 Mdot
.68095 .17221 Mdot
.7 .14513 Mdot
.71905 .12053 Mdot
.7381 .09855 Mdot
.75714 .07922 Mdot
.77619 .06253 Mdot
.79524 .04838 Mdot
.81429 .0366 Mdot
.83333 .02701 Mdot
.85238 .01938 Mdot
.87143 .01346 Mdot
.89048 .00901 Mdot
.90952 .00578 Mdot
.92857 .00352 Mdot
.94762 .00202 Mdot
.96667 2e-05 Mdot
.5 Mabswid
.14685 0 m
.18347 .15043 L
.22461 .30142 L
.26423 .41754 L
.2842 .46345 L
.30234 .49761 L
.32195 .52649 L
.33285 .53899 L
.3429 .54833 L
.35222 .55516 L
.36209 .56052 L
.36721 .56256 L
.37199 .56402 L
.37474 .56467 L
.37724 .56514 L
.37863 .56535 L
.38014 .56554 L
.38156 .56568 L
.38289 .56578 L
.38418 .56585 L
.38483 .56588 L
.38554 .56589 L
.3868 .5659 L
.38798 .56588 L
.38869 .56586 L
.38937 .56583 L
.39064 .56576 L
.392 .56564 L
.39348 .56549 L
.39599 .56513 L
.39866 .56464 L
.40345 .56347 L
.40802 .56201 L
.4123 .56034 L
.42207 .5555 L
.43193 .54921 L
.44229 .54117 L
.46081 .52346 L
.49897 .47569 L
.53958 .41287 L
.57868 .34609 L
.62023 .27378 L
.66026 .20736 L
.69878 .14985 L
.73975 .09817 L
.77921 .05914 L
.81715 .03192 L
.83663 .0217 L
.85754 .01334 L
.8677 .01018 L
Mistroke
.87875 .00733 L
.88831 .00535 L
.89887 .00361 L
.90905 .00233 L
.91862 .00145 L
.92389 .00108 L
.92872 .0008 L
.93403 .00055 L
.93962 .00035 L
.94441 .00023 L
.94877 .00015 L
.95118 .00011 L
.9538 8e-05 L
.95606 6e-05 L
.95856 4e-05 L
.96086 3e-05 L
.96301 2e-05 L
.96528 1e-05 L
.96656 1e-05 L
.96773 0 L
.9688 0 L
.9698 0 L
.9709 0 L
.97207 0 L
.97318 0 L
.97423 0 L
.97517 0 L
.97619 0 L
Mfstroke
.02381 0 m
.06244 .06389 L
.10458 .15288 L
.14415 .2425 L
.18221 .32617 L
.22272 .4059 L
.26171 .46881 L
.28158 .49457 L
.30316 .51727 L
.31323 .52593 L
.32392 .53374 L
.33393 .53975 L
.34309 .54415 L
.3525 .54758 L
.35769 .549 L
.36262 .55003 L
.36482 .5504 L
.36716 .55072 L
.36918 .55094 L
.37137 .55113 L
.37271 .55122 L
.37395 .55127 L
.37513 .55131 L
.37638 .55133 L
.3775 .55134 L
.37854 .55133 L
.37969 .5513 L
.38092 .55126 L
.38214 .55119 L
.38348 .5511 L
.3862 .55085 L
.38873 .55054 L
.39109 .55017 L
.39642 .54912 L
.40216 .5476 L
.41266 .54385 L
.42228 .53932 L
.44121 .52749 L
.46191 .51036 L
.50247 .46565 L
.54151 .41141 L
.58301 .34572 L
.62299 .27874 L
.66146 .21473 L
.70238 .15106 L
.74178 .09747 L
.77967 .05563 L
.80075 .03708 L
.82001 .02326 L
.83992 .01215 L
Mistroke
.85883 .00451 L
.86844 .00166 L
Mfstroke
.86844 .00166 m
.87583 0 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{599.125, 370.312},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000>20002:b000`400?l00000o`00003oo`3ooooo
0?ooool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00
o`3ooooo0?ooool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon5
0?ooo`00o`3ooooo0?ooool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0oooo
o`3ooon50?ooo`00o`3ooooo0?ooool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?oo
ool0ooooo`3ooon50?ooo`00o`3oooo`0?ooo`@000000P3oool400000?l0ooooRP3oool00?l0oooo
lP3oool00`000000oooo0?ooo`020?ooo`800000o`3ooon;0?ooo`00o`3ooooc0?ooo`030000003o
ool0oooo00800000o`3ooon<0?ooo`00o`3ooooc0?ooo`040000003oool0000000000?l0ooooS@3o
ool00?l0oooom03oool300000?l0ooooS@3oool00?l0oooom03oool200000?l0ooooSP3oool00?l0
ooool`3oool300000?l0ooooSP3oool00?l0ooool`3oool2000000030?ooo`000000oooo0?l0oooo
S03oool00?l0oooolP3oool2000000<0oooo00<000000?ooo`3oool0o`3ooon:0?ooo`00o`3ooooa
0?ooo`@0000000@0oooo0000000000000000o`3ooon;0?ooo`00o`3ooooo0?ooool0ooooQ@3oool0
0?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?ooool0oooo
Q@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?oo
ool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3o
oooo0?ooool0ooooQ@3oool00?l0ooooo`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?oo
o`00N03oool3000008d0oooo0`0000060?ooo`8000000P3oool800000880oooo0`0000060?ooo`80
00001`3oool2000008<0oooo0`0000060?ooo`800000103oool5000008@0oooo0`0000060?ooo`80
0000103oool5000008X0oooo1P00000@0?ooo`00M`3oool01@000000oooo0?ooo`3oool0000008/0
oooo00D000000?ooo`3oool0oooo000000050?ooo`8000000`3oool800000800oooo00D000000?oo
o`3oool0oooo000000050?ooo`8000001`3oool200000880oooo00D000000?ooo`3oool0oooo0000
00050?ooo`8000000`3oool2000000<0oooo0P0000220?ooo`050000003oool0oooo0?ooo`000000
1@3oool2000000<0oooo0P0000040?ooo`030000003oool0oooo08T0oooo0P00000B0?ooo`00MP3o
ool00`000000oooo0?ooo`020?ooo`800000R@3oool00`000000oooo0?ooo`020?ooo`8000002P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo07`0oooo00<000000?ooo`3oool00P3o
ool2000000d0oooo0P0000210?ooo`030000003oool0oooo0080oooo0P0000090?ooo`800000103o
ool200000800oooo00<000000?ooo`3oool00P3oool2000000P0oooo0P0000050?ooo`800000RP3o
ool200000180oooo001f0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0Q`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0P`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo00/0oooo0P0000210?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool01P3oool2000000D0oooo0P0000200?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool01P3oool2000000D0oooo0P00002:0?ooo`8000004P3oool0
07D0oooo0P0000050?ooo`800000Q`3oool2000000D0oooo0P00000:0?ooo`030000003oool0oooo
0840oooo0P0000050?ooo`8000001P3oool:000007h0oooo0P0000050?ooo`8000001`3oool20000
00D0oooo0P00001o0?ooo`8000001@3oool2000000L0oooo0P0000040?ooo`<00000RP3oool20000
0180oooo001e0?ooo`8000001@3oool2000008L0oooo0P0000050?ooo`8000002`3oool00`000000
oooo0?ooo`200?ooo`8000001@3oool2000000L0oooo00<000000?ooo`3oool00P3oool200000800
oooo0P0000050?ooo`8000001`3oool2000000D0oooo0P00001o0?ooo`8000001@3oool2000000P0
oooo00@000000?ooo`3oool0oooo0`00002;0?ooo`8000004P3oool007D0oooo0P0000050?ooo`80
0000Q`3oool2000000D0oooo0P00000<0?ooo`030000003oool0oooo07l0oooo0P0000050?ooo`80
00001`3oool00`000000oooo0?ooo`020?ooo`800000P03oool2000000D0oooo0P0000070?ooo`80
00001@3oool2000007l0oooo0P0000050?ooo`8000002@3oool00`000000oooo00000002000008`0
oooo0P00000B0?ooo`00M@3oool2000000D0oooo0P0000270?ooo`8000001@3oool2000000d0oooo
00<000000?ooo`3oool0OP3oool2000000D0oooo0P0000080?ooo`040000003oool0oooo0?ooo`80
0000P03oool2000000D0oooo0P0000070?ooo`800000103oool200000800oooo0P0000050?ooo`80
00002P3oool3000008d0oooo0P00000B0?ooo`00M@3oool2000000D0oooo0P0000270?ooo`800000
1@3oool2000000d0oooo0P00001o0?ooo`8000001@3oool2000000T0oooo00<000000?ooo`3oool0
0P0000200?ooo`8000001@3oool2000000L0oooo1`0000210?ooo`8000001@3oool2000000T0oooo
1000002=0?ooo`8000004P3oool007D0oooo0P0000050?ooo`800000Q`3oool2000000D0oooo0P00
000=0?ooo`800000O`3oool2000000D0oooo0P0000090?ooo`030000003oool0oooo00800000P03o
ool2000000D0oooo0P0000080?ooo`800000Q@3oool2000000D0oooo0P0000080?ooo`8000000`3o
ool00`000000oooo0?ooo`2:0?ooo`8000004P3oool007D0oooo0P0000050?ooo`800000Q`3oool2
000000D0oooo0P0000070?ooo`030000003oool0oooo00<0oooo0P00001o0?ooo`8000001@3oool2
000000X0oooo00@000000?ooo`0000000000P03oool2000000D0oooo0P0000080?ooo`800000Q@3o
ool2000000D0oooo0P0000070?ooo`800000103oool2000008/0oooo0P00000B0?ooo`00MP3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo08L0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo00<0oooo0P0000200?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool02P3oool300000840oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`080?ooo`800000Q@3oool00`000000oooo0?ooo`030?ooo`030000003oool0
oooo00H0oooo0P0000040?ooo`800000R`3oool200000180oooo001f0?ooo`8000000`3oool20000
08T0oooo0P0000030?ooo`8000002@3oool010000000oooo0?ooo`3oool300000800oooo0P000003
0?ooo`800000303oool300000840oooo0P0000030?ooo`8000002`3oool2000008@0oooo0P000003
0?ooo`800000203oool2000000@0oooo0P0000280?ooo`D000004P3oool007L0oooo1@00002;0?oo
o`D000002`3oool4000008<0oooo1@00000>0?ooo`800000PP3oool5000000d0oooo0P0000240?oo
o`D000002`3oool5000008`0oooo0P00000B0?ooo`00o`3ooooo0?oood/0oooo0P00003o0?ooocP0
oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?ooool0ooooQ@3oool00?l0ooooo`3ooooo
0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?ooool0ooooQ@3oool00?l0oooo
o`3ooooo0?ooohD0oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?ooomD0oooo00<00000
0?ooo`3oool0Q03oool5001Vo`040?ooo`00ofH00?mV003oIP80c?l01@3o000H0?ooo`00o`3ooooo
0?ooom<0oooo1@00001a0?ooo`D0c03o00<0oooo08l0o`2?0?l00`2?0?l00`3oool0<`3o03<0o`03
03<0o`80oooo1@00I_l0103oool00?mV003oIP00ofH20<co00D0o`00603oool00?l0ooooo`3ooooC
0?ooo`D00000L@3oool50<`0o`030?ooo`2?0?l0S`3o00<0S`3o00@0oooo03<0o`0c0?l0<`3o1@3o
0004001Vo`<00?mV0P3<o`060?l001P0oooo001W0?oooa<0000000<0o`00000000000000D0000000
0`2?0?l0000000000006000000030<co0000000000000080000000<0o`0000000000I_l01`000000
0`3<0?l000000000003o00000?l00000=P0000030?l000P000001@3<o`0?000000P00?mV;03o000C
00000000I`3oool00`000000oooo0?ooo`0@0?ooo`03003oIP3o0000oooo0280oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`050?ooo`0308l0o`3oool0oooo00H0oooo00<0c?l00?oo
o`3oool00P3oool00`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo00<0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?oo
o`3oool06`3oool7000000030?ooo`000000oooo02@0oooo00<000000?ooo`3oool05`3oool50?l0
00H0oooo1@3<o`0>0?ooo`h0o`00103<o`050?l000040?ooo`00ofH00?mV003oIPD0c?l01P3o0005
001Vo`<00?mV0P3<o`050?l0000503<0o`3oool0<`3o03<0o`0000004P3oool100000000I`3oool0
0`000000oooo0?ooo`0@0?ooo`0303<0o`3o0000oooo0280oooo00<000000?ooo`3oool08`3oool0
0`000000oooo0?ooo`060?ooo`0308l0o`3oool0oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0
000c0?l006Ko00P0oooo00<0c03o0?ooo`3oool00P3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0I0?ooo`T00000
00<0oooo0000003oool0903oool00`000000oooo0?ooo`0C0?ooo`@0o`00203oool30<co00030000
003oool0oooo00X0oooo1P3o0002003oIP030?ooo`3o0000o`0000<0o`001P3<o`020?ooo`03003o
IP3o0000o`0000@0o`0000<00?mV08l0o`2?0?l01P3<o`000`0c0?l0o`000?l000030?l000<006Ko
103oool01P00ofH0oooo03<0o`3<o`00S`3o0?l00080oooo0P0c0?l00`2?0?l0c03o0000000B0?oo
o`400000001W0?ooo`030000003oool0oooo0100oooo00<0<`3o003oIP3o00008P3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo00H0oooo00<0S`3o0?ooo`3oool01P3oool00`3<o`00
oooo0?ooo`020?ooo`030?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool00P3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0R0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0H0?ooo`P000000`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0100
oooo0`3o000:0?ooo`80c?l00`3oool00`000000oooo0?ooo`060?ooo`@0o`001000ofH20?ooo`<0
c?l01@3o00050<co00<00?mV0P3oool50?l000040?ooo`00I_l006Ko001Vo`D0c?l000<0<`3o0?l0
003o00000`3o00070?ooo`80<`3o00@0S`3o0?ooo`3oool0oooo0P0c0?l208l0o`030<`0o`3oool0
00000180oooo0@00000006L0oooo00<000000?ooo`3oool0403oool00`0c0?l0oooo0?l0001A0?oo
o`0308l0o`3oool0oooo00H0oooo00<00?mV0<co003oool00P3oool00`3o0000oooo001Vo`070?oo
o`030<`0o`3oool0oooo02P0oooo00<000000?ooo`3oool0U03oool00`000000oooo0?ooo`2E0?oo
o`030000003oool0oooo08P0oooo200000040?ooo`030000003oool0oooo03@0oooo0P3o000:0?oo
o`<0c?l0203oool80?l000800?mV1P3<o`020?ooo`H0o`001@3<o`050?ooo`D0o`0000@006Ko0?oo
o`3oool0oooo1@3<o`02001Vo`030?ooo`3o0000oooo00D0oooo0`0c0?l0102?0?l0oooo0?ooo`3o
ool203<0o`040?ooo`2?0?l0oooo0<`0o`80oooo00<000000?ooo`3oool0403oool100000000I`3o
ool00`000000oooo0?ooo`0@0?ooo`0403<0o`3oool00?mV0?l00500oooo00<0S`3o0?ooo`3oool0
1P3oool00`00ofH0c?l00?ooo`020?ooo`030?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0
:03oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo09D0oooo00<000000?ooo`3oool0
Q`3oool8000000D0oooo00<000000?ooo`3oool0<@3oool30?l000T0oooo0`3<o`060?ooo`D0o`00
0`3oool50?l00080c?l0203oool00`00ofH0o`000?l000030?l000D0c?l0103oool01000I_l00?mV
003oIP3o0002003oIP@0oooo0P00I_l0103<o`00oooo0?ooo`3<0?l60?ooo`<0<`3o00<0oooo08l0
o`3oool00P3oool203<0o`80oooo00@0S`3o0?ooo`3oool0c03o0P3oool00`000000oooo0?ooo`0@
0?ooo`400000001W0?ooo`030000003oool0oooo0100oooo00D0<`3o0?ooo`00I_l0c?l00?l0001@
0?ooo`0308l0o`3oool0oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0000c0?l006Ko00P0oooo
00<0c03o0?ooo`3oool0o`3ooooP0?ooo`L00000>@3oool20?l000X0oooo0P3<o`060?ooo`<0o`00
1P3oool00`00ofH0o`000?l000040?l000L0oooo0`00ofH00`3oool0o`000?l000030?l000D0c?l0
0P3oool2001Vo`030?ooo`00ofH00?mV00<00?mV00@0oooo001Vo`00I_l006Ko103oool00`3<0?l0
oooo0?ooo`030?ooo`80<`3o0`3oool00`2?0?l0oooo0?ooo`0203<0o`<0oooo00@0S`3o0?ooo`3o
ool0c03o5P3oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`0503<0o`3oool0oooo003o
IP3o00001@00001;0?ooo`0308l0o`3oool0oooo00H0oooo00<00?mV0<co003oool00P3oool00`3o
0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0oooog`3oool7000003L0oooo0`3o00090?oo
o`<0c?l01@3oool30?l000H0oooo103<o`000`3oool0o`000?l000030?l00080c?l00`3oool2003o
IPH0oooo00<0o`000?ooo`3oool01@3<o`02001Vo`80oooo1P00ofH00`00I_l0oooo0?ooo`050?oo
o`040<`0o`3oool0oooo0?ooo`80<`3o0`3oool208l0o`030?ooo`0c0?l0<`3o00@0oooo00@0S`3o
0?ooo`3oool0c03o5`3oool100000000I`3oool00`000000oooo0?ooo`0>0?ooo`80000000H0S`3o
03<0o`00000006Ko0<co003o0006000004T0oooo00<0S`3o0?ooo`3oool01P3oool00`00ofH0c?l0
0?ooo`020?ooo`030?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooooN0?ooo`L00000
=P3oool20?l000X0oooo0P3<o`040?ooo`P0o`000P3oool30<co0003003oIP3oool0oooo0080oooo
1@3o00020<co00<00?mV3@3oool00`3<o`0006Ko001Vo`050?ooo`D00?mV1`3oool00`3<0?l0oooo
0?ooo`0203<0o`@0oooo00@0S`3o0?ooo`0c0?l0<`3o103oool208l0o`<0oooo00<0c03o0?ooo`3o
ool05@3oool100000000I`3oool00`000000oooo0?ooo`0>0?ooo`80000000H0S`3o03<0o`000000
0000003oIP3o0007000004P0oooo00<0S`3o0?ooo`3oool01`3oool01`3<o`00oooo0?ooo`3oool0
o`000?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?ooomd0oooo1`00000e0?ooo`80o`002P3o
ool20<co0080oooo103o00030?ooo`D0o`000P3<o`000`00ofH0oooo0?ooo`060?ooo`030<co003o
0000c?l000<0c?l03@3oool3001Vo`H0oooo0P00I_l5003oIPH0oooo00@0c03o0?ooo`0c0?l0<`3o
1@3oool00`2?0?l0<`3o03<0o`050?ooo`0308l0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool0
5P3oool100000000I`3oool00`000000oooo0?ooo`0>0?ooo`80000000L0S`3o03<0o`0000000000
001Vo`00ofH0o`0000L00000B03oool00`2?0?l0oooo0?ooo`060?ooo`070<co003oool0oooo0?oo
o`3o0000<`3o001Vo`080?ooo`030<`0o`3oool0oooo0?l0oooofP3oool8000003@0oooo0P3o000:
0?ooo`80c?l000@0oooo0?l0003o0000o`001P3oool60?l000/0oooo00<00?mV0<co003<o`000`3<
o`0:0?ooo`<006Ko1`3oool2001Vo`@0oooo00<00?mV0?ooo`3oool01@3oool00`3<0?l0<`3o03<0
o`050?ooo`0308l0o`0c0?l0<`3o00H0oooo00<0S`3o0?ooo`3oool00P3oool00`3<0?l0oooo0?oo
o`0G0?ooo`400000001W0?ooo`030000003oool0oooo00h0oooo0P000000202?0?l0<`3o00000000
000006Ko003oIP3<o`00o`001`0000170?ooo`0308l0o`3oool0oooo00H0oooo00<00?mV0<co003o
ool00P3oool00`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0oooof@3oool8000003<0
oooo0P3o000:0?ooo`80c?l000<0oooo0?l0003o00002@3oool00`3<o`00o`000?l000030?l000T0
oooo0P00ofH00`3oool0c?l00<co00030<co00<00?mV1@3oool2001Vo`L0oooo0`00I_l>0?ooo`03
03<0o`3oool0oooo00@0oooo0P0c0?l70?ooo`0308l0o`3oool0oooo00<0oooo00<0c03o0?ooo`3o
ool05`3oool100000000I`3oool00`000000oooo0?ooo`0>0?ooo`80000000<0S`3o03<0o`000000
0P0000000`00I_l00?mV0?l00009000004D0oooo00<0S`3o0?ooo`3oool01P3oool00`00ofH0c?l0
0?ooo`020?ooo`030?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooooH0?ooo`P00000
<P3oool20?l000X0oooo0P3<o`030?l000P0oooo103<o`050?l000H0oooo0`00ofH50?ooo`030<co
0000ofH00?mV00<00?mV0`3oool2001Vo`L0oooo0P00I_l?0?ooo`80<`3o1@3oool203<0o`P0oooo
00<0S`3o0?ooo`3oool00`3oool00`3<0?l0oooo0?ooo`0H0?ooo`400000001W0?ooo`030000003o
ool0oooo00l0oooo00<0000008l0o`0c0?l00P0000001@3oool006Ko003oIP3<o`00o`0000T00000
A03oool00`2?0?l0oooo0?ooo`070?ooo`070<co003oool0oooo0?ooo`3o0000oooo001Vo`070?oo
o`030<`0o`3oool0oooo0?l0ooooe`3oool800000340oooo0P3o000:0?ooo`80c?l01`3o00030?oo
o`<0c?l000<00?mV0?ooo`3<o`000`3<o`000`3o0000oooo0?ooo`030?ooo`<00?mV203oool6003o
IP030?ooo`00I_l006Ko00L0oooo0P00I_l?0?ooo`80<`3o00<0c03o0?ooo`3oool00P3oool203<0
o`0308l0o`3oool0oooo00D0oooo0P2?0?l50?ooo`030<`0o`3oool0oooo01T0oooo0@00000006L0
oooo00<000000?ooo`3oool03`3oool00`000000S`3o03<0o`0200000080oooo00<006Ko003oIP3o
00000P3oool7000004D0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3oool0o`00
03<0o`00I_l0203oool00`3<0?l0oooo0?ooo`3o0?ooomD0oooo1`00000b0?ooo`030?l0003oool0
oooo00P0oooo0P3<o`020?l00080oooo1@3o00030<co00800?mV0P3oool60<co0080oooo0`00ofH<
0?ooo`D00?mV00<006Ko0?ooo`3oool01@3oool2001Vo`l0oooo0P0c0?l20?ooo`030<`0o`3oool0
oooo0080<`3o0P2?0?l70?ooo`0308l0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool06@3oool1
00000000I`3oool00`000000oooo0?ooo`0?0?ooo`030000002?0?l0<`3o008000000P3oool01000
I_l00?mV0<co003o00020?ooo`L00000A03oool00`2?0?l0oooo0?ooo`060?ooo`03003oIP3<o`00
oooo0080oooo00<0o`000?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?ooom@0oooo1`00000a
0?ooo`80o`002@3oool0103<o`00o`000?l0003o00030?ooo`H0o`000P3oool00`00ofH0oooo0?oo
o`030?ooo`D0c?l00P00ofH?0?ooo`D00?mV1P3oool2001Vo`l0oooo0P0c0?l30?ooo`060<`0o`3o
ool0oooo03<0o`3oool0S`3o203oool00`2?0?l0oooo0?ooo`050?ooo`030<`0o`3oool0oooo01X0
oooo0@00000006L0oooo1000000?0?ooo`0308l0o`0c0?l0000000<0oooo00D006Ko0?ooo`00ofH0
c?l00?l000030?ooo`H00000@`3oool00`2?0?l0oooo0?ooo`070?ooo`070<co003oool0oooo0?oo
o`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0ooood`3oool700000300oooo0P3o0009
0?ooo`030<co003o0000o`0000L0oooo1@3o0002003oIPH0oooo1@3<o`0B0?ooo`03001Vo`00ofH0
oooo00D0oooo0P00I_l@0?ooo`0303<0o`3oool0oooo0080oooo00H0c03o0?ooo`0c0?l0<`3o0?oo
o`2?0?l80?ooo`0308l0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool0603oool400000000I`3o
ool00`000000oooo0?ooo`0@0?ooo`0308l0o`0c0?l0oooo00@0oooo00@006Ko003oIP3<o`00o`00
0`3oool6000004@0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3oool0o`000?oo
o`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?ooom80oooo1`00000_0?ooo`80o`002P3oool20?l0
00P0oooo00<0c?l00?l0003o00000`3o00090?ooo`03003oIP3<o`00oooo0140oooo0P00I_l60?oo
o`8006Ko403oool203<0o`D0oooo00D0c03o03<0o`3oool0oooo08l0o`080?ooo`0308l0o`3oool0
oooo00H0oooo00<0c03o0?ooo`3oool06`3oool100000000I`3oool00`000000oooo0?ooo`0@0?oo
o`0308l0o`0c0?l0oooo00@0oooo00D006Ko0?ooo`00ofH0c?l00?l000030?ooo`H00000@`3oool0
0`2?0?l0oooo0?ooo`060?ooo`070<co003oool0oooo0?ooo`3o0000<`3o001Vo`080?ooo`030<`0
o`3oool0oooo0?l0ooood03oool7000002l0oooo00<0o`000?ooo`3oool0203oool20?l000P0oooo
0P3<o`030?ooo`030?l00000ofH0oooo00L0oooo0P00ofHC0?ooo`03001Vo`3oool0oooo00@0oooo
0P00I_l@0?ooo`80<`3o1P3oool203<0o`80oooo00<0S`3o0?ooo`3oool01@3oool208l0o`P0oooo
00<0c03o0?ooo`3oool0703oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`0c
0?l0oooo00D0oooo00@006Ko003oIP3oool0o`00103oool600000480oooo00<0S`3o0?ooo`3oool0
1P3oool00`00ofH0c?l00?ooo`020?ooo`030?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0
o`3oooo@0?ooo`H00000;P3oool20?l000T0oooo1@3o00040?ooo`D0c?l00P3oool00`00ofH0oooo
0?ooo`060?ooo`800?mV4`3oool2001Vo`H0oooo00<006Ko0?ooo`3oool03P3oool203<0o`L0oooo
00<0<`3o0?ooo`3oool00P2?0?l70?ooo`0308l0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool0
7@3oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`0c0?l0oooo00D0oooo00D0
06Ko0?ooo`00ofH0c?l00?l000030?ooo`L00000@@3oool00`2?0?l0oooo0?ooo`070?ooo`070<co
003oool0oooo0?ooo`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0ooooc`3oool60000
02h0oooo00<0o`000?ooo`3oool01`3oool70?l000<0oooo1P3<o`02003oIPL0oooo1`00ofH?0?oo
o`03001Vo`3oool0oooo00@0oooo0P00I_l@0?ooo`0303<0o`3oool0oooo00D0oooo0P0c0?l00`3<
0?l0oooo08l0o`080?ooo`0308l0o`3oool0oooo00P0oooo00<0c03o0?ooo`3oool07@3oool10000
0000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`0c0?l0oooo00H0oooo00D006Ko003oIP3o
ool0c?l00?l000030?ooo`H00000@P3oool00`2?0?l0oooo0?ooo`060?ooo`070<co003oool0oooo
0?ooo`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0oooocP3oool7000002d0oooo00<0
o`000?ooo`3oool01P3oool20?l000030?ooo`3o0000o`0000@0o`0000<0oooo0<co003<o`001P3<
o`070?ooo`800?mV0P3oool5003oIPd0oooo0P00I_l50?ooo`8006Ko403oool203<0o`L0oooo00D0
<`3o0?ooo`3<0?l0oooo08l0o`080?ooo`0308l0o`3oool0oooo00P0oooo00<0c03o0?ooo`3oool0
7P3oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`0c0?l0oooo00H0oooo00D0
06Ko0?ooo`00ofH0c?l00?l000040?ooo`H00000@@3oool00`2?0?l0oooo0?ooo`060?ooo`07003o
IP3<o`00oooo0?ooo`3o0000<`3o001Vo`080?ooo`030<`0o`3oool0oooo0?l0ooooc03oool70000
02`0oooo0P3o00070?ooo`80o`0000@0c?l00?ooo`3oool0oooo1@3o0000103<o`00oooo0?ooo`3o
ool50<co00D0oooo0P00ofH30?ooo`H00?mV2`3oool2001Vo`H0oooo00<006Ko0?ooo`3oool03`3o
ool00`0c0?l0oooo0?ooo`050?ooo`80<`3o00@0oooo0<`0o`3oool0S`3o203oool00`2?0?l0oooo
0?ooo`080?ooo`030<`0o`3oool0oooo01l0oooo0@00000006L0oooo00<000000?ooo`3oool0403o
ool00`3<0?l0<`3o0?ooo`060?ooo`06001Vo`3oool00?mV0?ooo`3<o`00o`00103oool600000400
oooo00<0S`3o0?ooo`3oool01P3oool00`00ofH0c?l00?ooo`020?ooo`030?l0003oool006Ko00L0
oooo00<0c03o0?ooo`3oool0o`3oooo;0?ooo`L00000;03oool00`3o0000oooo0?ooo`050?ooo`80
o`0000<0c?l00?ooo`3oool00`3oool50?l000@0oooo1@3<o`040?ooo`03003oIP3oool0oooo00@0
oooo1@00ofH:0?ooo`03001Vo`3oool0oooo00@0oooo0P00I_lA0?ooo`0303<0o`3oool0oooo00D0
oooo00H0<`3o0?ooo`3oool0c03o0?ooo`2?0?l70?ooo`80S`3o2P3oool00`3<0?l0oooo0?ooo`0P
0?ooo`400000001W0?ooo`030000003oool0oooo0100oooo00<0c03o08l0o`0c0?l01`3oool01@00
I_l0oooo003oIP3<o`00o`0000@0oooo1`00000o0?ooo`0308l0o`3oool0oooo00L0oooo00L0c?l0
0?ooo`3oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3oooo:0?ooo`L00000
;03oool00`3o0000oooo0?ooo`040?ooo`80o`0000<0c?l00?ooo`3oool01P3oool00`3<o`00o`00
0?ooo`040?ooo`04003oIP3oool0oooo0<co00@0oooo0P00ofH70?ooo`D00?mV203oool2001Vo`D0
oooo0P00I_l:0?ooo`D006Ko0P3oool203<0o`H0oooo0P0c0?l30?ooo`030<`0o`2?0?l0oooo00H0
oooo00<0S`3o0?ooo`3oool02P3oool00`3<0?l0oooo0?ooo`0P0?ooo`400000001W0?ooo`030000
003oool0oooo0100oooo00<0c03o08l0o`0c0?l01`3oool01P00I_l0oooo003oIP3oool0c?l00?l0
00@0oooo1P0000100?ooo`0308l0o`3oool0oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0003o
ool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3oooo90?ooo`L00000:`3oool20?l000H0oooo00<0
o`000?ooo`3<o`001`3oool20<co00D0oooo0P00ofH60?ooo`800?mV2`3oool00`00ofH0oooo0?oo
o`070?ooo`03001Vo`3oool0oooo00<0oooo0P00I_l<0?ooo`D006Ko00<0oooo03<0o`3oool01P3o
ool00`0c0?l0oooo0?ooo`020?ooo`030<`0o`2?0?l0oooo00H0oooo00<0S`3o0?ooo`3oool02P3o
ool00`3<0?l0oooo0?ooo`0Q0?ooo`400000001W0?ooo`030000003oool0oooo0100oooo00<0c03o
08l0o`0c0?l0203oool01P00I_l0oooo003oIP3<o`00oooo0?l000@0oooo1P00000o0?ooo`0308l0
o`3oool0oooo00H0oooo00L00?mV0<co003oool0oooo0?l0000c0?l006Ko00P0oooo00<0c03o0?oo
o`3oool0o`3oooo70?ooo`L00000:`3oool00`3o0000oooo0?ooo`020?ooo`D0o`0000<0c?l00?oo
o`3oool0103oool20<co00H0oooo00<00?mV0?ooo`3oool01@3oool00`00ofH0oooo0?ooo`0C0?oo
o`8006Ko1@3oool00`00I_l0oooo0?ooo`0;0?ooo`H006Ko00<0<`3o0?ooo`3oool01@3oool00`0c
0?l0oooo0?ooo`020?ooo`80S`3o1`3oool00`2?0?l0oooo0?ooo`0:0?ooo`030<`0o`3oool0oooo
0280oooo0@00000006L0oooo00<000000?ooo`3oool0403oool00`3<0?l0S`3o03<0o`030?ooo`D0
o`0000H006Ko0?ooo`00ofH0oooo0<co003o00050?ooo`H00000?P3oool00`2?0?l0oooo0?ooo`07
0?ooo`070<co003oool0oooo0?ooo`3o0000oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0oooo
aP3oool7000002/0oooo00<0o`000?ooo`3oool00`3oool50?l000H0oooo00<0c?l00?ooo`3oool0
103oool2003oIPH0oooo0P00ofHE0?ooo`03001Vo`3oool0oooo00<0oooo0P00I_l?0?ooo`D006Ko
1P3oool203<0o`@0oooo00<0S`3o0<`0o`3oool01P3oool00`2?0?l0oooo0?ooo`0;0?ooo`030<`0
o`3oool0oooo0280oooo0@00000006L0oooo00<000000?ooo`3oool0403oool0103<0?l0S`3o0?oo
o`0c0?l20?ooo`D0o`0000L006Ko0?ooo`3oool00?mV0?ooo`3<o`00o`0000@0oooo1P00000o0?oo
o`0308l0o`3oool0oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0003oool006Ko00L0oooo00<0
c03o0?ooo`3oool0o`3oooo60?ooo`H00000:P3oool20?l000D0oooo1P3o00040?ooo`80c?l01P3o
ool00`00ofH0oooo0?ooo`050?ooo`03003oIP3oool0oooo01@0oooo00<006Ko0?ooo`3oool00P3o
ool2001Voa40oooo1@00I_l50?ooo`0303<0o`3oool0oooo00<0oooo00<0S`3o0<`0o`3oool01P3o
ool00`2?0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo02<0oooo0@00000006L0oooo00<00000
0?ooo`3oool0403oool01@3<0?l0S`3o0?ooo`0c0?l0oooo00H0o`0000L0oooo001Vo`3oool00?mV
0?ooo`3<o`00o`0000D0oooo1P00000n0?ooo`0308l0o`3oool0oooo00H0oooo00L0c?l00?ooo`3o
ool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3oooo50?ooo`H00000:P3oool0
0`3o0000oooo0?ooo`040?ooo`L0o`0000<0oooo0<co003<o`000`3<o`050?ooo`03003oIP3oool0
oooo00@0oooo0P00ofHE0?ooo`8006Ko103oool00`00I_l0oooo0?ooo`0A0?ooo`0303<0o`3oool0
06Ko00D0oooo0P0c0?l50?ooo`0308l0o`3<0?l0oooo00H0oooo00<0S`3o0?ooo`3oool02`3oool0
0`3<0?l0oooo0?ooo`0T0?ooo`400000001W0?ooo`030000003oool0oooo0100oooo00@0c03o08l0
o`3oool0<`3o0P3oool50?l000080?ooo`00I_l0oooo0?ooo`00ofH0oooo0<co003o00050?ooo`D0
0000?P3oool00`2?0?l0oooo0?ooo`060?ooo`07003oIP3<o`00oooo0?ooo`3o0000<`3o001Vo`08
0?ooo`030<`0o`3oool0oooo0?l0oooo``3oool7000002T0oooo00<0o`000?ooo`3oool0103oool0
0`3o0000oooo0<co00050?l000H0c?l00`3oool2003oIPD0oooo0P00ofHF0?ooo`03001Vo`3oool0
oooo0080oooo0P00I_lC0?ooo`0303<0o`3oool0oooo00D0oooo00<0<`3o0?ooo`3oool0103oool0
0`2?0?l0oooo0<`0o`060?ooo`0308l0o`3oool0oooo00`0oooo00<0c03o0?ooo`3oool0903oool1
00000000I`3oool00`000000oooo0?ooo`0@0?ooo`060<`0o`2?0?l0oooo0?ooo`0c0?l0oooo1@3o
00020?ooo`07001Vo`3oool00?mV0?ooo`3<o`00oooo0?l000040?ooo`H00000?@3oool00`2?0?l0
oooo0?ooo`070?ooo`070<co003oool0oooo0?ooo`3o0000oooo001Vo`070?ooo`030<`0o`3oool0
oooo0?l0oooo`P3oool7000002T0oooo00<0o`000?ooo`3oool0103oool00`3o0000oooo0<co0003
0?ooo`030?l0003oool0c?l000H0c?l00P3oool00`00ofH0oooo0?ooo`040?ooo`03003oIP3oool0
oooo01@0oooo0P00I_l40?ooo`03001Vo`3oool0oooo0140oooo0P0c0?l60?ooo`80<`3o1P3oool0
0`2?0?l0oooo0<`0o`050?ooo`80S`3o3P3oool00`3<0?l0oooo0?ooo`0U0?ooo`400000001W0?oo
o`@000003`3oool01@3<0?l0S`3o0?ooo`3oool0<`3o00<0oooo00<0o`000?ooo`3oool00P3oool0
1`00I_l0oooo0?ooo`00ofH0oooo0<co003o00001@3oool5000003h0oooo00<0S`3o0?ooo`3oool0
1P3oool01`3<o`00oooo0?ooo`3oool0o`000?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?oo
ol40oooo1`00000Y0?ooo`030?l0003oool0oooo00<0oooo0P3o00020<co00D0oooo00<0c?l00?oo
o`3oool01@3<o`000`3oool00?mV0?ooo`040?ooo`800?mV5P3oool00`00I_l0oooo0?ooo`020?oo
o`8006Ko4`3oool00`0c0?l0oooo0?ooo`050?ooo`0303<0o`3oool0oooo00D0oooo00<0S`3o0?oo
o`3<0?l01@3oool00`2?0?l0oooo0?ooo`0=0?ooo`030<`0o`3oool0oooo02<0oooo1000000006L0
oooo00<000000?ooo`3oool0403oool01@3<0?l0S`3o0?ooo`3oool0<`3o00P0oooo00P006Ko0?oo
o`3oool00?mV0?ooo`3<o`00oooo0?l000@0oooo1P00000m0?ooo`0308l0o`3oool0oooo00H0oooo
00L0c?l00?ooo`3oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3oooo00?oo
o`L00000:03oool20?l000D0oooo00<0o`000?ooo`3<o`001@3oool20<co00<0oooo1@3<o`001000
ofH0oooo0?ooo`3oool5003oIQ@0oooo0P00I_l40?ooo`03001Vo`3oool0oooo0180oooo00<0<`3o
0?ooo`3oool0103oool203<0o`L0oooo00@0S`3o0?ooo`3oool0c03o103oool00`2?0?l0oooo0?oo
o`0>0?ooo`030<`0o`3oool0oooo02H0oooo0@00000006L0oooo00<000000?ooo`3oool0403oool0
0`3<0?l0S`3o0?ooo`020?ooo`0303<0o`3oool0oooo00H0oooo00L006Ko0?ooo`3oool00?mV0?oo
o`3<o`00o`0000D0oooo1P00000l0?ooo`0308l0o`3oool0oooo00H0oooo00L00?mV0<co003oool0
oooo0?l0000c0?l006Ko00P0oooo00<0c03o0?ooo`3oool0o`3ooono0?ooo`H00000:03oool00`3o
0000oooo0?ooo`040?ooo`030?l0003oool0c?l000D0oooo00<0c?l00?ooo`3oool01@3oool00`3<
o`000?mV0?ooo`040?ooo`D00?mV4`3oool00`00I_l0oooo0?ooo`020?ooo`8006Ko503oool00`0c
0?l0oooo0?ooo`040?ooo`0303<0o`3oool0oooo00H0oooo00@0S`3o0?ooo`3oool0c03o103oool0
0`2?0?l0oooo0?ooo`0>0?ooo`030<`0o`3oool0oooo02L0oooo0@00000006L0oooo00<000000?oo
o`3oool0403oool00`3<0?l0S`3o0?ooo`020?ooo`0303<0o`3oool0c?l000@0c?l00P3oool02000
I_l0oooo0?ooo`00ofH0oooo0<co003oool0o`00103oool6000003`0oooo00<0S`3o0?ooo`3oool0
1`3oool01`3<o`00oooo0?ooo`3oool0o`000?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?oo
okh0oooo1P00000X0?ooo`030?l0003oool0oooo00<0oooo0P3o00020<co00D0oooo00<0c?l00?oo
o`3oool01P3oool00`00ofH0oooo0?ooo`030?ooo`H00?mV4P3oool00`00I_l0oooo0?ooo`020?oo
o`03001Vo`3oool0oooo01<0oooo00<0<`3o0?ooo`3oool00`3oool203<0o`P0oooo00@0S`3o0?oo
o`3oool0c03o103oool00`2?0?l0oooo0?ooo`0>0?ooo`030<`0o`3oool0oooo02P0oooo0@000000
06L0oooo00<000000?ooo`3oool0403oool00`3<0?l0S`3o0?ooo`030?ooo`0303<0o`3<o`00c?l0
00<0c?l00`3oool02000I_l0oooo0?ooo`00ofH0oooo0<co003oool0o`00103oool6000003`0oooo
00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3oool0o`000?ooo`00I_l01`3oool00`3<
0?l0oooo0?ooo`3o0?oookd0oooo1P00000X0?ooo`030?l0003oool0o`0000@0o`0000<0oooo0<co
003oool0103oool20<co00L0oooo0P00ofH40?ooo`800?mV00<0oooo003oIP00ofH00`00ofH@0?oo
o`8006Ko0`3oool2001Voa@0oooo0P0c0?l50?ooo`0303<0o`3oool0oooo00L0oooo00D0S`3o0?oo
o`3oool0oooo0<`0o`030?ooo`0308l0o`3oool0oooo00l0oooo00<0c03o0?ooo`3oool0:03oool1
00000000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`2?0?l0oooo00<0oooo1P3<o`030?oo
o`04001Vo`3oool0oooo003oIP80oooo00<0c?l00?l0003oool0103oool5000003`0oooo00<0S`3o
0?ooo`3oool01P3oool01`00ofH0c?l00?ooo`3oool0o`000?ooo`00I_l01`3oool00`3<0?l0oooo
0?ooo`3o0?oookd0oooo1P00000W0?ooo`030?l0003oool0oooo00D0o`0000<0c?l00?ooo`3oool0
0`3oool00`3<o`00oooo0?ooo`060?ooo`03003oIP3oool0oooo0080oooo0P00ofH30?ooo`D00?mV
3`3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo01<0oooo00<0<`3o0?ooo`3oool0
103oool00`0c0?l0oooo0?ooo`070?ooo`0508l0o`3oool0oooo0?ooo`3<0?l00`3oool00`2?0?l0
oooo0?ooo`0?0?ooo`030<`0o`3oool0oooo02T0oooo0@00000006L0oooo00<000000?ooo`3oool0
403oool00`3<0?l0S`3o0?ooo`030?ooo`0303<0o`3<o`00c?l000<0c?l00`3oool02@00I_l0oooo
0?ooo`3oool00?mV0?ooo`3<o`00oooo0?l000040?ooo`H00000>`3oool00`2?0?l0oooo0?ooo`06
0?ooo`07003oIP3<o`00oooo0?ooo`3o0000<`3o001Vo`080?ooo`030<`0o`3oool0oooo0?l0oooo
^`3oool6000002L0oooo00<0o`000?ooo`3oool01P3o00050?ooo`030<co003oool0oooo00H0oooo
00<00?mV0?ooo`3oool00P3oool00`00ofH0oooo0?ooo`050?ooo`03003oIP3oool0oooo00h0oooo
00@006Ko0?ooo`3oool0oooo0P00I_lE0?ooo`0303<0o`3oool0oooo00@0oooo00<0<`3o0?ooo`3o
ool0203oool0102?0?l0oooo0?ooo`3<0?l30?ooo`0308l0o`3oool0oooo00l0oooo00<0c03o0?oo
o`3oool0:P3oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`030<`0o`2?0?l0oooo00@0
oooo1@3<o`040?ooo`04001Vo`3oool0oooo003oIP80oooo00<0c?l00?l0003oool0103oool50000
03/0oooo00<0S`3o0?ooo`3oool01`3oool01`3<o`00oooo0?ooo`3oool0o`000?ooo`00I_l01`3o
ool00`3<0?l0oooo0?ooo`3o0?oookX0oooo1P00000W0?ooo`030?l0003oool0oooo0080oooo1@3o
00030?ooo`80c?l0203oool01000ofH0oooo0?ooo`3oool2003oIQP0oooo00D006Ko0?ooo`3oool0
oooo001Vo`0F0?ooo`0303<0o`3oool0oooo00<0oooo0P0c0?l:0?ooo`0408l0o`3oool0oooo0<`0
o`<0oooo00<0S`3o0?ooo`3oool0403oool00`3<0?l0oooo0?ooo`0Z0?ooo`400000001W0?ooo`03
0000003oool0oooo0100oooo00<0c03o08l0o`3oool0103oool00`0c0?l0oooo0<co00060?ooo`09
001Vo`3oool0oooo0?ooo`00ofH0oooo0<co003oool0o`0000@0oooo1P00000k0?ooo`0308l0o`3o
ool0oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3o
ool0o`3ooonj0?ooo`H000009@3oool20?l000<0oooo1`3o00020?ooo`030<co003oool0oooo00L0
oooo00D00?mV0?ooo`3oool0oooo003oIP0I0?ooo`05001Vo`3oool0oooo0?ooo`00I_l05P3oool0
0`0c0?l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo00T0oooo00H0S`3o0?ooo`3oool0oooo0<`0
o`3oool208l0oa80oooo00<0c03o0?ooo`3oool0:`3oool100000000I`3oool00`000000oooo0?oo
o`0@0?ooo`030<`0o`2?0?l0oooo00@0oooo00<0<`3o0?ooo`3oool01`3oool01000I_l0oooo0?oo
o`00ofH20?ooo`030<co003oool0o`0000@0oooo1@00000k0?ooo`0308l0o`3oool0oooo00H0oooo
00L00?mV0<co003oool0oooo0?l0000c0?l006Ko00L0oooo00<0c03o0?ooo`3oool0o`3oooni0?oo
o`H000009@3oool00`3o0000oooo0?ooo`020?ooo`060?l0003oool0oooo0?ooo`3<o`00o`00103<
o`090?ooo`05003oIP3oool0oooo0?ooo`00ofH06@3oool00`00I_l0oooo0?ooo`02001VoaH0oooo
00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`090?ooo`0708l0o`3oool0oooo0?ooo`3<
0?l0oooo08l0o`0C0?ooo`030<`0o`3oool0oooo02`0oooo0@00000006L0oooo00<000000?ooo`3o
ool04@3oool00`2?0?l0oooo0?ooo`040?ooo`0303<0o`3oool0oooo00H0oooo00T006Ko0?ooo`3o
ool0oooo003oIP3oool0c?l00?ooo`3o0000103oool6000003X0oooo00<0S`3o0?ooo`3oool01`3o
ool01P3<o`00oooo0?ooo`3o0000<`3o001Vo`P0oooo00<0c03o0?ooo`3oool0o`3ooong0?ooo`H0
00009@3oool00`3o0000oooo0?ooo`020?ooo`060?l0003oool0oooo0?ooo`3<o`00oooo1@3<o`08
0?ooo`05003oIP3oool0oooo0?ooo`00ofH06@3oool01000I_l0oooo0?ooo`00I_lG0?ooo`0303<0
o`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool02@3oool01`2?0?l0oooo0?ooo`3oool0c03o0?oo
o`2?0?l0503oool00`3<0?l0oooo0?ooo`0/0?ooo`400000001W0?ooo`030000003oool0oooo0140
oooo00<0c03o08l0o`3oool0103oool00`0c0?l0oooo0?ooo`060?ooo`05001Vo`3oool0oooo0?oo
o`00ofH00P3oool00`3<o`00oooo0?l000040?ooo`D00000>`3oool00`2?0?l0oooo0?ooo`060?oo
o`070<co003oool0oooo0?l0000c0?l0oooo001Vo`070?ooo`030<`0o`3oool0oooo0?l0oooo]`3o
ool6000002@0oooo00@0o`000?ooo`3oool0oooo0P3o00030?ooo`030<co003oool0c?l000D0c?l0
1`3oool00`00ofH0oooo0?ooo`02003oIQT0oooo00D006Ko0?ooo`3oool006Ko0?ooo`05001Voa40
oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo0080
oooo00<0c03o08l0o`3oool04`3oool00`3<0?l0oooo0?ooo`0]0?ooo`400000001W0?ooo`030000
003oool0oooo0140oooo00<0c03o08l0o`3oool01@3oool00`0c0?l0oooo0?ooo`060?ooo`09001V
o`3oool0oooo0?ooo`00ofH0oooo0<co003oool0o`0000@0oooo1@00000k0?ooo`0308l0o`3oool0
oooo00H0oooo00L0c?l00?ooo`3oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0
o`3ooonf0?ooo`H00000903oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo0P3<o`030?ooo`D0
c?l01P3oool01000ofH0oooo0?ooo`00ofHI0?ooo`8006Ko0P3oool00`00I_l0oooo0?ooo`05001V
oa00oooo00<0<`3o0?ooo`3oool00P3oool203<0o`/0oooo00<0S`3o0?ooo`3oool00P3oool00`3<
0?l0S`3o0?ooo`0C0?ooo`030<`0o`3oool0oooo02h0oooo0@00000006L0oooo00<000000?ooo`3o
ool04@3oool00`3<0?l0S`3o0?ooo`050?ooo`0303<0o`3oool0oooo00H0oooo00D006Ko0?ooo`3o
ool0oooo003oIP020?ooo`030<co003oool0o`0000<0oooo1P00000j0?ooo`0308l0o`3oool0oooo
00H0oooo00L00?mV0<co003oool0oooo0?l0000c0?l006Ko00L0oooo00<0c03o0?ooo`3oool0o`3o
oone0?ooo`H00000903oool01@3o0000oooo0?ooo`3oool0o`0000<0oooo00<0c?l00?ooo`3oool0
0P3oool60<co00D0oooo00@00?mV0?ooo`3oool00?mV6@3oool00`00I_l0oooo0?ooo`02001Vo`80
oooo1P00I_l>0?ooo`80<`3o103oool00`0c0?l0oooo0?ooo`0:0?ooo`0308l0o`3oool0oooo0080
oooo00<0c03o08l0o`3oool04`3oool00`3<0?l0oooo0?ooo`0_0?ooo`400000001W0?ooo`030000
003oool0oooo0140oooo00<0c03o08l0o`3oool01P3oool00`0c0?l0oooo0?ooo`060?ooo`05001V
o`3oool0oooo0?ooo`00ofH00P3oool00`3<o`00oooo0?l000030?ooo`D00000>P3oool00`2?0?l0
oooo0?ooo`070?ooo`060<co003oool0oooo0?l0000c0?l006Ko203oool00`3<0?l0oooo0?ooo`3o
0?oook@0oooo1P00000S0?ooo`030?l0003oool0oooo0080o`000`3oool0103<o`00oooo0?ooo`3o
ool20<co00<0oooo00<0c?l00?ooo`3oool0103oool01000ofH0oooo0?ooo`00ofHI0?ooo`04001V
o`3oool0oooo001Vo`D0oooo1@00I_l=0?ooo`0303<0o`3oool0oooo00<0oooo00<0<`3o0?ooo`3o
ool02P3oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo0180oooo00<0c03o0?ooo`3o
ool0<03oool100000000I`3oool00`000000oooo0?ooo`0A0?ooo`030<`0o`2?0?l0oooo00H0oooo
00<0<`3o0?ooo`3oool01P3oool01@00I_l0oooo0?ooo`3oool00?mV0080oooo00<0c?l00?ooo`3o
00000`3oool6000003X0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3o0000<`3o
0?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?oook<0oooo1P00000R0?ooo`D0o`00103oool0
1@3<o`00oooo0?ooo`3oool0c?l000/0oooo00@00?mV0?ooo`3oool00?mV6@3oool01000I_l0oooo
0?ooo`00I_l60?ooo`D006Ko303oool00`0c0?l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo00X0
oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0B0?ooo`030<`0o`3oool0oooo0340
oooo0@00000006L0oooo00<000000?ooo`3oool04@3oool00`3<0?l0oooo08l0o`070?ooo`D00?mV
0`3oool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0oooo0<co003oool0o`000`3oool50000
03X0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3oool0o`000?ooo`00I_l01`3o
ool00`3<0?l0oooo0?ooo`3o0?oook80oooo1P00000S0?ooo`D0o`000`3oool01@3<o`00oooo0?oo
o`3oool0c?l000/0oooo00@00?mV0?ooo`00ofH00?mV6@3oool01000I_l0oooo0?ooo`00I_l90?oo
o`03001Vo`3oool0oooo00/0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`0:0?oo
o`0308l0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool04P3oool00`3<0?l0oooo0?ooo`0b0?oo
o`400000001W0?ooo`@00000403oool00`3<0?l0oooo08l0o`070?ooo`D00?mV103oool01@00I_l0
oooo0?ooo`3oool00?mV0080oooo00<0c?l00?ooo`3o00000`3oool6000003T0oooo00<0S`3o0?oo
o`3oool01P3oool01`00ofH0c?l00?ooo`3oool0o`0003<0o`00I_l01`3oool00`3<0?l0oooo0?oo
o`3o0?oook40oooo1`00000R0?ooo`H0o`000P3oool00`3<o`00oooo0?ooo`020<co00/0oooo00<0
0?mV0?ooo`00ofH06P3oool01000I_l0oooo001Vo`00I_lG0?ooo`0303<0o`3oool0oooo0080oooo
0P0c0?l<0?ooo`0308l0o`3oool0oooo0080oooo0P2?0?lD0?ooo`030<`0o`3oool0oooo0300oooo
1000000006L0oooo00<000000?ooo`3oool04@3oool00`3<0?l0oooo08l0o`060?ooo`H00?mV103o
ool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0oooo0<co003oool0o`000`3oool5000003T0
oooo00<0S`3o0?ooo`3oool01`3oool01P3<o`00oooo0?ooo`3o0000<`3o001Vo`P0oooo00<0c03o
0?ooo`3oool0o`3ooon`0?ooo`H000008`3oool60?l000050?ooo`3<o`00oooo0?ooo`3<o`002P3o
ool5003oIQX0oooo00<006Ko0?ooo`00I_l0603oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3o
ool0oooo00/0oooo00<0S`3o0?ooo`3oool00P3oool00`2?0?l0oooo0<`0o`0C0?ooo`030<`0o`3o
ool0oooo03@0oooo0@00000006L0oooo00<000000?ooo`3oool04@3oool0103<0?l0oooo0?ooo`2?
0?l60?ooo`D00?mV1@3oool01@00I_l0oooo0?ooo`3oool00?mV0080oooo00<0c?l00?ooo`3o0000
0`3oool6000003T0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo0?ooo`3o0000<`3o0?oo
o`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?ooojl0oooo1P00000S0?ooo`030?l0003oool0o`00
00@0o`0000@0c?l00?ooo`3oool0c?l02`3oool5003oIQT0oooo00<006Ko0?ooo`00I_l0603oool0
0`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00/0oooo00<0S`3o0?ooo`3oool00P3oool0
0`2?0?l0oooo0<`0o`0C0?ooo`030<`0o`3oool0oooo03D0oooo0@00000006L0oooo00<000000?oo
o`3oool04@3oool0103<0?l0oooo0?ooo`2?0?l60?ooo`D00?mV1@3oool00`00I_l0oooo0?ooo`02
0?ooo`06003oIP3oool0oooo0<co003oool0o`000`3oool5000003T0oooo00<0S`3o0?ooo`3oool0
1P3oool01`00ofH0c?l00?ooo`3oool0o`000?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?oo
ojh0oooo1P00000S0?ooo`030?l0003oool0o`000080oooo00@0o`000?ooo`3<o`00oooo0P3<o`0;
0?ooo`H00?mV603oool00`00I_l0oooo001Vo`0H0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o
0?ooo`3oool02`3oool00`2?0?l0oooo0?ooo`020?ooo`0308l0o`3oool0c03o01@0oooo00<0c03o
0?ooo`3oool0=@3oool100000000I`3oool00`000000oooo0?ooo`0A0?ooo`040<`0o`3oool0oooo
08l0o`P0oooo00<00?mV0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0
oooo0<co003oool0o`000`3oool6000003P0oooo00<0S`3o0?ooo`3oool01`3oool01P3<o`00oooo
0?ooo`3o0000<`3o001Vo`L0oooo00<0c03o0?ooo`3oool0o`3ooon^0?ooo`H000008P3oool00`3o
0000oooo0?l000040?ooo`030<co003oool0c?l000d0oooo1P00ofHG0?ooo`03001Vo`3oool006Ko
01P0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`0;0?ooo`0308l0o`3oool0oooo
0080oooo00@0S`3o0?ooo`3oool0c03o4`3oool00`3<0?l0oooo0?ooo`0f0?ooo`400000001W0?oo
o`030000003oool0oooo0140oooo00D0c03o0?ooo`3oool0oooo08l0o`070?ooo`0303<0o`3oool0
oooo00H0oooo00<006Ko0?ooo`3oool00P3oool01P00ofH0oooo0?ooo`3<o`00oooo0?l000<0oooo
1@00000i0?ooo`0308l0o`3oool0oooo00H0oooo00H0c?l00?ooo`3oool0o`0003<0o`00I_l80?oo
o`030<`0o`3oool0oooo0?l0oooo[03oool600000280oooo00<0o`000?ooo`3o0000103oool00`3<
o`00oooo0<co000=0?ooo`L00?mV5P3oool3001VoaL0oooo0P0c0?l30?ooo`80<`3o0`3oool503<0
o`D0oooo00<0S`3o0?ooo`3oool00P3oool0102?0?l0oooo0?ooo`3<0?lC0?ooo`030<`0o`3oool0
oooo03L0oooo0@00000006L0oooo00<000000?ooo`3oool04@3oool01@3<0?l0oooo0?ooo`3oool0
S`3o00P0oooo00<0<`3o0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0
oooo0<co003oool0o`000`3oool6000003P0oooo00<0S`3o0?ooo`3oool01P3oool01`3<o`00oooo
0?ooo`3o0000<`3o0?ooo`00I_l01`3oool00`3<0?l0oooo0?ooo`3o0?oooj/0oooo1P00000R0?oo
o`030?l0003oool0o`0000@0oooo00<0c?l00?ooo`3<o`003@3oool2003oIP<0oooo00<00?mV0?oo
o`3oool05@3oool2001VoaP0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`030?oo
o`D0<`3o1@3oool01@2?0?l0oooo0?ooo`3oool0S`3o0080oooo00<0c03o0?ooo`3oool04@3oool0
0`3<0?l0oooo0?ooo`0h0?ooo`400000001W0?ooo`030000003oool0oooo0140oooo00D0c03o0?oo
o`3oool0oooo08l0o`080?ooo`0303<0o`3oool0oooo00H0oooo00<006Ko0?ooo`3oool00P3oool0
1P00ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1P00000h0?ooo`0308l0o`3oool0oooo00H0oooo
00L00?mV0<co003oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooon[0?oo
o`H000008@3oool00`3o0000oooo0?l000020?ooo`D0c?l03@3oool2003oIQ/0oooo0P00I_lH0?oo
o`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool00`3oool603<0o`@0oooo00D0S`3o0?oo
o`3oool0oooo08l0o`030?ooo`030<`0o`3oool0oooo0100oooo00<0c03o0?ooo`3oool0>@3oool1
00000000I`3oool00`000000oooo0?ooo`0A0?ooo`030<`0o`3oool0oooo0080oooo00<0S`3o0?oo
o`3oool01P3oool00`0c0?l0oooo0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo00H00?mV0?oo
o`3oool0c?l00?ooo`3o00030?ooo`D00000>@3oool00`2?0?l0oooo0?ooo`060?ooo`060<co003o
ool0oooo0?l0000c0?l006Ko1`3oool00`3<0?l0oooo0?ooo`3o0?ooojX0oooo1P00000Q0?ooo`03
0?l0003oool0o`0000<0oooo1@3<o`0<0?ooo`800?mV6`3oool2001VoaP0oooo00<0<`3o0?ooo`3o
ool00P3oool00`0c0?l0oooo0?ooo`050?ooo`D0<`3o0`3oool01@2?0?l0oooo0?ooo`3oool0S`3o
00<0oooo00<0c03o0?ooo`3oool0403oool00`3<0?l0oooo0?ooo`0j0?ooo`400000001W0?ooo`03
0000003oool0oooo0140oooo00<0c03o0?ooo`3oool00P3oool00`2?0?l0oooo0?ooo`060?ooo`03
03<0o`3oool0oooo00H0oooo00<006Ko0?ooo`3oool00P3oool01P00ofH0oooo0?ooo`3<o`00oooo
0?l00080oooo1P00000h0?ooo`0308l0o`3oool0oooo00H0oooo00H0c?l00?ooo`3oool0o`0003<0
o`00I_l80?ooo`030<`0o`3oool0oooo0?l0ooooZ03oool600000240oooo00<0o`000?ooo`3o0000
0`3oool60<co00/0oooo0P00ofHJ0?ooo`<006Ko603oool00`0c0?l0oooo0?ooo`020?ooo`0303<0
o`3oool0oooo00H0oooo1@0c0?l20?ooo`0508l0o`3oool0oooo0?ooo`2?0?l00`3oool00`3<0?l0
oooo0?ooo`0@0?ooo`030<`0o`3oool0oooo03/0oooo0@00000006L0oooo00<000000?ooo`3oool0
4@3oool00`3<0?l0oooo0?ooo`020?ooo`0308l0o`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool0
0`3oool50?l000<0oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00030?ooo`D00000>03oool00`2?
0?l0oooo0?ooo`060?ooo`070<co003oool0oooo0?l0000c0?l0oooo001Vo`070?ooo`030<`0o`3o
ool0oooo0?l0ooooZ03oool600000200oooo00<0o`000?ooo`3o00001@3oool50<co00X0oooo00<0
0?mV0?ooo`3oool06@3oool2001VoaT0oooo00@0<`3o0?ooo`3oool0oooo0P0c0?l;0?ooo`0503<0
o`3oool0oooo0?ooo`2?0?l00`3oool01@2?0?l0oooo0?ooo`3oool0c03o0180oooo00<0c03o0?oo
o`3oool0?03oool100000000I`3oool00`000000oooo0?ooo`0A0?ooo`030<`0o`3oool0oooo00<0
oooo00<0S`3o0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`020?ooo`D0o`00103oool01P00ofH0
oooo0?ooo`3<o`00oooo0?l00080oooo1P00000h0?ooo`0308l0o`3oool0oooo00D0oooo00L00?mV
0<co003oool0oooo0?l0003oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooonW0?ooo`H00000
6`3oool60?l000030?ooo`3o0000oooo00@0oooo1P3<o`090?ooo`03003oIP3oool0oooo01T0oooo
0P00I_lI0?ooo`0503<0o`3oool0oooo0?ooo`0c0?l0403oool01@2?0?l0oooo0?ooo`3oool0S`3o
00@0oooo00<0c03o0?ooo`3oool03`3oool00`3<0?l0oooo0?ooo`0m0?ooo`400000001W0?ooo`03
0000003oool0oooo0140oooo00<0c03o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`060?ooo`04
03<0o`3oool0oooo0?ooo`H0o`00103oool01P00ofH0oooo0?ooo`3<o`00oooo0?l000<0oooo1@00
000h0?ooo`0308l0o`3oool0oooo00H0oooo00H0c?l00?ooo`3oool0o`0003<0o`00I_l70?ooo`03
0<`0o`3oool0oooo0?l0ooooYP3oool6000001`0oooo1`3o00050?ooo`80c?l00P3oool00`3<o`00
oooo0?ooo`080?ooo`03003oIP3oool0oooo01T0oooo0P00I_lI0?ooo`0503<0o`3oool0oooo0?oo
o`0c0?l0403oool01@2?0?l0oooo0?ooo`3oool0S`3o00@0oooo00<0c03o0?ooo`3oool03`3oool0
0`3<0?l0oooo0?ooo`0n0?ooo`400000001W0?ooo`030000003oool0oooo0140oooo00<0c03o0?oo
o`3oool00`3oool00`2?0?l0oooo0?ooo`070?ooo`0403<0o`3oool0oooo0?ooo`D0o`00103oool0
1`00ofH0oooo0?ooo`3oool0c?l00?ooo`3o00000P3oool6000003L0oooo00<0S`3o0?ooo`3oool0
1P3oool01P3<o`00oooo0?ooo`3o0000<`3o001Vo`P0oooo00<0c03o0?ooo`3oool0o`3ooonT0?oo
o`L000006`3oool70?l000D0oooo0P3<o`0=0?ooo`03003oIP3oool0oooo01T0oooo0P00I_lI0?oo
o`0503<0o`3oool0oooo0?ooo`0c0?l0403oool01@2?0?l0oooo0?ooo`3oool0S`3o00@0oooo00<0
c03o0?ooo`3oool03`3oool00`3<0?l0oooo0?ooo`0o0?ooo`400000001W0?ooo`030000003oool0
oooo0140oooo00<0c03o0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`060?ooo`0403<0o`3oool0
oooo0?ooo`D0o`0000<006Ko0?ooo`3oool00P3oool01P00ofH0oooo0?ooo`3<o`00oooo0?l000<0
oooo1@00000h0?ooo`0308l0o`3oool0oooo00D0oooo00L0c?l00?ooo`3oool0o`0003<0o`3oool0
06Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooonT0?ooo`H000007@3oool50?l000H0oooo00<0c?l0
0?ooo`3oool02P3oool2003oIQ/0oooo00<006Ko0?ooo`3oool0603oool01@0c0?l0oooo0?ooo`3o
ool0<`3o0100oooo00D0S`3o0?ooo`3oool0oooo08l0o`040?ooo`030<`0o`3oool0oooo00l0oooo
00<0c03o0?ooo`3oool0@03oool100000000I`3oool00`000000oooo0?ooo`0A0?ooo`030<`0o`3o
ool0oooo00@0oooo00<0S`3o0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`020?ooo`040?l0003o
ool0oooo001Vo`@0oooo00L00?mV0?ooo`3oool0oooo0<co003oool0o`000080oooo1P00000g0?oo
o`0308l0o`3oool0oooo00D0oooo00L00?mV0<co003oool0oooo0?l0003oool006Ko00L0oooo00<0
c03o0?ooo`3oool0o`3ooonS0?ooo`H000007P3oool50?l000D0oooo00<0c?l00?ooo`3oool02P3o
ool2003oIQ/0oooo00<006Ko0?ooo`3oool0603oool01@0c0?l0oooo0?ooo`3oool0<`3o0100oooo
00D0S`3o0?ooo`3oool0oooo08l0o`050?ooo`030<`0o`3oool0oooo00h0oooo00<0c03o0?ooo`3o
ool0@@3oool100000000I`3oool400000100oooo00<0c03o0?ooo`3oool0103oool00`2?0?l0oooo
0?ooo`070?ooo`0303<0o`3oool0oooo00D0oooo00<006Ko0?ooo`3oool00`3oool01P00ofH0oooo
0?ooo`3<o`00oooo0?l000<0oooo1@00000g0?ooo`0308l0o`3oool0oooo00H0oooo00H0c?l00?oo
o`3oool0o`0003<0o`00I_l70?ooo`030<`0o`3oool0oooo0?l0ooooXP3oool600000200oooo0P3o
00060?ooo`030<co003oool0oooo00X0oooo0P00ofHK0?ooo`03001Vo`3oool0oooo01P0oooo00D0
<`3o0?ooo`3oool0oooo03<0o`0@0?ooo`0508l0o`3oool0oooo0?ooo`2?0?l01@3oool00`3<0?l0
oooo0?ooo`0>0?ooo`030<`0o`3oool0oooo03l0oooo1000000006L0oooo00<000000?ooo`3oool0
4@3oool00`3<0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo00L0oooo00<0<`3o0?ooo`3oool0
1@3oool00`00I_l0oooo0?ooo`020?ooo`07003oIP3oool0oooo0?ooo`3<o`00oooo0?l000020?oo
o`H00000=`3oool00`2?0?l0oooo0?ooo`050?ooo`060<co003oool0oooo0?l0000c0?l006Ko203o
ool00`3<0?l0oooo0?ooo`3o0?oooj40oooo1P00000O0?ooo`030?l0003oool0oooo00D0oooo00<0
c?l00?ooo`3oool02P3oool2003oIQ/0oooo00<006Ko0?ooo`3oool0603oool01@0c0?l0oooo0?oo
o`3oool0<`3o0100oooo00D0S`3o0?ooo`3oool0oooo08l0o`050?ooo`030<`0o`3oool0oooo00h0
oooo00<0c03o0?ooo`3oool0@`3oool100000000I`3oool00`000000oooo0?ooo`0A0?ooo`030<`0
o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`050?ooo`03001V
o`3oool0oooo00<0oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00030?ooo`D00000=`3oool00`2?
0?l0oooo0?ooo`050?ooo`07003oIP3<o`00oooo0?l0000c0?l0oooo001Vo`070?ooo`030<`0o`3o
ool0oooo0?l0ooooX03oool6000001l0oooo00<0o`000?ooo`3oool01@3oool00`3<o`00oooo0?oo
o`0:0?ooo`800?mV6`3oool00`00I_l0oooo0?ooo`0H0?ooo`0503<0o`3oool0oooo0?ooo`0c0?l0
403oool01@2?0?l0oooo0?ooo`3oool0S`3o00D0oooo00<0c03o0?ooo`3oool03P3oool00`3<0?l0
oooo0?ooo`140?ooo`400000001W0?ooo`030000003oool0oooo0140oooo00<0c03o0?ooo`3oool0
1@3oool00`2?0?l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00D0oooo00<006Ko0?ooo`3oool0
0P3oool01`00ofH0oooo0?ooo`3oool0c?l00?ooo`3o00000P3oool5000003L0oooo00<0S`3o0?oo
o`3oool01P3oool01P3<o`00oooo0?ooo`3o0000oooo001Vo`L0oooo00<0c03o0?ooo`3oool0o`3o
oonO0?ooo`H000007`3oool00`3o0000oooo0?ooo`050?ooo`030<co003oool0oooo00X0oooo0P00
ofHK0?ooo`03001Vo`3oool0oooo01P0oooo00D0<`3o0?ooo`3oool0oooo03<0o`0@0?ooo`0508l0
o`3oool0oooo0?ooo`2?0?l01P3oool00`3<0?l0oooo0?ooo`0=0?ooo`030<`0o`3oool0oooo04D0
oooo0@00000006L0oooo00<000000?ooo`3oool04@3oool00`3<0?l0oooo0?ooo`060?ooo`0308l0
o`3oool0oooo00L0oooo00<0<`3o0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`030?ooo`06003o
IP3oool0oooo0<co003oool0o`000P3oool6000003L0oooo00<0S`3o0?ooo`3oool01@3oool01P3<
o`00oooo0?ooo`3o0000<`3o001Vo`L0oooo00<0c03o0?ooo`3oool0o`3ooonO0?ooo`H000007P3o
ool00`3o0000oooo0?ooo`050?ooo`030<co003oool0oooo00X0oooo0P00ofHK0?ooo`03001Vo`3o
ool0oooo01P0oooo00D0<`3o0?ooo`3oool0oooo03<0o`0@0?ooo`0508l0o`3oool0oooo0?ooo`2?
0?l01P3oool00`3<0?l0oooo0?ooo`0>0?ooo`030<`0o`3oool0oooo04D0oooo0@00000006L0oooo
00<000000?ooo`3oool04@3oool00`3<0?l0oooo0?ooo`060?ooo`0308l0o`3oool0oooo00P0oooo
00<0<`3o0?ooo`3oool0103oool00`00I_l0oooo0?ooo`030?ooo`07003oIP3oool0oooo0?ooo`3<
o`00oooo0?l000020?ooo`D00000=`3oool00`2?0?l0oooo0?ooo`050?ooo`060<co003oool0oooo
0?l0000c0?l006Ko203oool00`3<0?l0oooo0?ooo`3o0?oooid0oooo1P00000N0?ooo`030?l0003o
ool0oooo00H0oooo00<0c?l00?ooo`3oool02@3oool2003oIQ/0oooo00<006Ko0?ooo`3oool0603o
ool00`0c0?l0oooo0?ooo`0203<0oa00oooo00D0S`3o0?ooo`3oool0oooo08l0o`060?ooo`030<`0
o`3oool0oooo00h0oooo00<0c03o0?ooo`3oool0AP3oool100000000I`3oool00`000000oooo0?oo
o`0A0?ooo`030<`0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool01`3oool00`0c0?l0oooo0?oo
o`050?ooo`03001Vo`3oool0oooo00<0oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00020?ooo`H0
0000=P3oool00`2?0?l0oooo0?ooo`050?ooo`07003oIP3<o`00oooo0?l0000c0?l0oooo001Vo`07
0?ooo`030<`0o`3oool0oooo0?l0ooooW03oool6000001h0oooo00<0o`000?ooo`3oool01P3oool0
0`3<o`00oooo0?ooo`090?ooo`800?mV603oool5001VoaT0oooo00@0<`3o0?ooo`3oool0<`3o4@3o
ool01@2?0?l0oooo0?ooo`3oool0S`3o00H0oooo00<0c03o0?ooo`3oool03P3oool00`3<0?l0oooo
0?ooo`170?ooo`400000001W0?ooo`030000003oool0oooo0140oooo00<0c03o0?ooo`3oool01`3o
ool00`2?0?l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00@0oooo00<006Ko0?ooo`3oool00`3o
ool01`00ofH0oooo0?ooo`3oool0c?l00?ooo`3o00000P3oool5000003L0oooo00<0S`3o0?ooo`3o
ool01@3oool01P3<o`00oooo0?ooo`3o0000oooo001Vo`L0oooo00<0c03o0?ooo`3oool0o`3ooonL
0?ooo`H000007@3oool20?l000L0oooo00<0c?l00?ooo`3oool01@3oool6003oIQT0oooo1@00I_lH
0?ooo`0403<0o`3oool0oooo03<0oa40oooo00D0S`3o0?ooo`3oool0oooo08l0o`070?ooo`030<`0
o`3oool0oooo00d0oooo00<0c03o0?ooo`3oool0B03oool100000000I`3oool00`000000oooo0?oo
o`0B0?ooo`030<`0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool01`3oool0100c0?l0oooo0?oo
o`3oool50<co00D0oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00020?ooo`H00000=P3oool00`2?
0?l0oooo0?ooo`050?ooo`060<co003oool0oooo0?l0000c0?l006Ko203oool00`3<0?l0oooo0?oo
o`3o0?oooiX0oooo1P00000M0?ooo`80o`001`3oool00`3<o`00oooo0?ooo`060?ooo`H00?mV603o
ool6001VoaL0oooo00@0<`3o0?ooo`3oool0<`3o4P3oool0102?0?l0oooo0?ooo`2?0?l70?ooo`03
0<`0o`3oool0oooo00d0oooo00<0c03o0?ooo`3oool0B@3oool100000000I`3oool00`000000oooo
0?ooo`0B0?ooo`030<`0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool00P3oool5001Vo`040?oo
o`0c0?l0oooo0?ooo`D0c?l01@3oool01P00ofH0oooo0?ooo`3oool0c?l00?l000<0oooo1@00000f
0?ooo`0308l0o`3oool0oooo00D0oooo00H0c?l00?ooo`3oool0o`0003<0o`00I_l80?ooo`030<`0
o`3oool0oooo0?l0ooooV@3oool6000001d0oooo0P3o00000`3oool0c?l00<co00030<co00030?oo
o`3<o`00oooo00L0oooo1P00ofHJ0?ooo`D006Ko5P3oool0100c0?l0oooo0?ooo`0c0?lB0?ooo`04
08l0o`3oool0oooo08l0o`L0oooo00<0c03o0?ooo`3oool03@3oool00`3<0?l0oooo0?ooo`1:0?oo
o`400000001W0?ooo`030000003oool0oooo0180oooo00<0c03o0?ooo`3oool0203oool0102?0?l0
oooo0?ooo`3oool5001Vo`030?ooo`0c0?l0oooo00H0c?l000<006Ko0?ooo`3oool00`3oool01P00
ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1P00000f0?ooo`0308l0o`3oool0oooo00@0oooo00L0
0?mV0<co003oool0o`0003<0o`3oool006Ko00L0oooo00<0c03o0?ooo`3oool0o`3ooonI0?ooo`H0
00005P3oool50?l000030?ooo`3o0000o`000080oooo1P3<o`0:0?ooo`D00?mV6@3oool6001VoaH0
oooo00<0<`3o0?ooo`0c0?l04P3oool0102?0?l0oooo0?ooo`2?0?l70?ooo`030<`0o`3oool0oooo
00d0oooo00<0c03o0?ooo`3oool0B`3oool100000000I`3oool00`000000oooo0?ooo`0B0?ooo`03
0<`0o`3oool0oooo00T0oooo00<0S`3o0?ooo`00I_l01@00I_l20?ooo`0303<0o`3oool0c?l000@0
c?l000<006Ko0?ooo`3oool00`3oool01P00ofH0oooo0?ooo`3oool0c?l00?l000<0oooo1@00000f
0?ooo`0308l0o`3oool0oooo00D0oooo00H0c?l00?ooo`3oool0o`000?ooo`00I_l80?ooo`030<`0
o`3oool0oooo0?l0ooooU`3oool6000001L0oooo1`3o00020?ooo`H0c?l02`3oool5003oIQP0oooo
0P00I_l20?ooo`03001Vo`3oool0oooo01D0oooo00<0<`3o0?ooo`0c0?l04P3oool0102?0?l0oooo
0?ooo`2?0?l80?ooo`030<`0o`3oool0oooo00`0oooo00<0c03o0?ooo`3oool0C03oool100000000
I`3oool00`000000oooo0?ooo`0C0?ooo`030<`0o`3oool0oooo00P0oooo00<0S`3o0?ooo`3oool0
1@00I_l20?ooo`0303<0o`3oool0c?l000@0c?l000<0oooo001Vo`3oool0103oool01P00ofH0oooo
0?ooo`3<o`00oooo0?l00080oooo1P00000f0?ooo`0308l0o`3oool0oooo00@0oooo00H0c?l00?oo
o`3oool0o`0003<0o`00I_l80?ooo`030<`0o`3oool0oooo0?l0ooooUP3oool6000001L0oooo1`3o
00040?ooo`D0c?l02`3oool3003oIQT0oooo0P00I_lJ0?ooo`0303<0o`3oool0<`3o0180oooo00@0
S`3o0?ooo`3oool0S`3o203oool00`3<0?l0oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo04d0oooo
0@00000004@0oooo0`0000060?ooo`800000103oool6000000h0oooo00<000000?ooo`3oool04`3o
ool00`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool006Ko00@006Ko0`3oool0100c0?l0oooo0?oo
o`3<o`030?ooo`03001Vo`3oool0oooo00<0oooo00H00?mV0?ooo`3oool0oooo0<co003o00030?oo
o`D00000=P3oool00`2?0?l0oooo0?ooo`040?ooo`060<co003oool0oooo0?l0000c0?l006Ko203o
ool00`3<0?l0oooo0?ooo`3o0?oooiH0oooo1P00000H0?ooo`D0o`001@3oool50<co00X0oooo0P00
ofHJ0?ooo`8006Ko6P3oool00`0c0?l0oooo03<0o`0B0?ooo`0408l0o`3oool0oooo08l0o`P0oooo
00<0c03o0?ooo`3oool0303oool00`3<0?l0oooo0?ooo`1>0?ooo`40000000130?ooo`050000003o
ool0oooo0?ooo`0000001@3oool2000000H0oooo0P00000@0?ooo`030000003oool0oooo01<0oooo
00<0c03o0?ooo`3oool02P3oool0102?0?l0oooo0?ooo`00I_l50?ooo`0303<0o`3oool0oooo00D0
oooo00<006Ko0?ooo`3oool00`3oool01P00ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1P00000e
0?ooo`0308l0o`3oool0oooo00D0oooo00H0c?l00?ooo`3o0000<`3o0?ooo`00I_l80?ooo`030<`0
o`3oool0oooo0?l0ooooU03oool6000001T0oooo1@3o00060?ooo`80c?l02`3oool2003oIQX0oooo
0P00I_lJ0?ooo`0303<0o`3oool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o203oool00`3<0?l0
oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo04l0oooo0@0000000480oooo00<000000?ooo`3oool0
0P3oool2000000`0oooo0P00000@0?ooo`030000003oool0oooo01@0oooo00<0c03o0?ooo`3oool0
2@3oool00`2?0?l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00@0oooo00<006Ko0?ooo`3oool0
0`3oool01P00ofH0oooo0?ooo`3oool0c?l00?l000<0oooo1@00000f0?ooo`0308l0o`3oool0oooo
00@0oooo00H0c?l00?ooo`3o0000oooo03<0o`00I_l80?ooo`030<`0o`3oool0oooo0?l0ooooU03o
ool5000001/0oooo0`3o00060?ooo`80c?l02`3oool2003oIQX0oooo0P00I_lJ0?ooo`0303<0o`3o
ool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o2@3oool00`3<0?l0oooo0?ooo`0;0?ooo`030<`0
o`3oool0oooo0500oooo0@0000000480oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`0:0?ooo`800000403oool600000140oooo00<0c03o0?ooo`3oool02P3oool00`2?0?l0oooo0?oo
o`060?ooo`0303<0o`3oool0oooo00D0oooo00<006Ko0?ooo`3oool00`3oool01P00ofH0oooo0?oo
o`3<o`00oooo0?l00080oooo1@00000f0?ooo`0308l0o`3oool0oooo00@0oooo00H0c?l00?ooo`3o
ool0o`0003<0o`00I_l80?ooo`030<`0o`3oool0oooo0?l0ooooT`3oool6000001/0oooo0P3o0006
0?ooo`80c?l02`3oool2003oIQX0oooo0P00I_lJ0?ooo`0303<0o`3oool0<`3o0180oooo00@0S`3o
0?ooo`3oool0S`3o2@3oool00`3<0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo04`0oooo1P00
00000440oooo0P0000050?ooo`8000002`3oool200000100oooo00<000000?ooo`3oool05@3oool0
0`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00L0oooo00<0<`3o0?ooo`3oool0103oool0
0`00I_l0oooo0?ooo`030?ooo`06003oIP3oool0oooo0?ooo`3<o`00o`000P3oool6000003D0oooo
00<0S`3o0?ooo`3oool0103oool01P00ofH0c?l00?ooo`3o0000<`3o001Vo`T0oooo00<0c03o0?oo
o`3oool0o`3ooonA0?ooo`H000006`3oool20?l000H0oooo0P3<o`0;0?ooo`03003oIP3oool00?mV
01T0oooo0P00I_lJ0?ooo`0303<0o`3oool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o2@3oool0
0`3<0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo0580oooo0@0000000440oooo0P0000050?oo
o`8000002`3oool200000100oooo00<000000?ooo`3oool05@3oool00`3<0?l0oooo0?ooo`0:0?oo
o`0308l0o`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`030?oo
o`06003oIP3oool0oooo0<co003oool0o`000P3oool5000003H0oooo00<0S`3o0?ooo`3oool0103o
ool01P3<o`00oooo0?l0000c0?l0oooo001Vo`P0oooo00<0c03o0?ooo`3oool0o`3ooonA0?ooo`H0
00006P3oool00`3o0000oooo0?l000050?ooo`80c?l02`3oool00`00ofH0oooo003oIP0I0?ooo`80
06Ko6P3oool00`0c0?l0oooo03<0o`0B0?ooo`0408l0o`3oool0oooo08l0o`T0oooo00<0c03o0?oo
o`3oool02`3oool00`3<0?l0oooo0?ooo`1C0?ooo`40000000110?ooo`8000001@3oool2000000/0
oooo0P00000@0?ooo`030000003oool0oooo01H0oooo00<0c03o0?ooo`3oool02@3oool00`2?0?l0
oooo0?ooo`070?ooo`0303<0o`3oool0oooo00@0oooo00<006Ko0?ooo`3oool00`3oool01P00ofH0
oooo0?ooo`3oool0c?l00?l00080oooo1P00000e0?ooo`0308l0o`3oool0oooo00@0oooo00H0c?l0
0?ooo`3o0000oooo03<0o`00I_l80?ooo`030<`0o`3oool0oooo0?l0ooooT03oool6000001X0oooo
00<0o`000?ooo`3o00001@3oool00`3<o`00oooo0<co000:0?ooo`03003oIP3oool00?mV01T0oooo
0P00I_lJ0?ooo`0303<0o`3oool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o2P3oool00`3<0?l0
oooo0?ooo`0:0?ooo`030<`0o`3oool0oooo05@0oooo0@0000000440oooo0P0000050?ooo`800000
2`3oool200000100oooo00<000000?ooo`3oool05P3oool00`3<0?l0oooo0?ooo`0:0?ooo`0308l0
o`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`030?ooo`06003o
IP3oool0oooo0<co003oool0o`000P3oool5000003D0oooo00<0S`3o0?ooo`3oool0103oool01P3<
o`00oooo0?ooo`3o0000<`3o001Vo`T0oooo00<0c03o0?ooo`3oool0o`3ooon?0?ooo`D000006`3o
ool20?l000D0oooo00<0c?l00?ooo`3<o`002P3oool00`00ofH0oooo003oIP0I0?ooo`8006Ko6P3o
ool303<0oa<0oooo00<0S`3o0?ooo`2?0?l02P3oool00`3<0?l0oooo0?ooo`0:0?ooo`030<`0o`3o
ool0oooo05D0oooo0@0000000440oooo0P0000050?ooo`8000002`3oool200000100oooo00<00000
0?ooo`3oool05`3oool00`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00L0oooo00<0<`3o
0?ooo`3oool0103oool00`00I_l0oooo0?ooo`030?ooo`06003oIP3oool0oooo0?ooo`3<o`00o`00
0P3oool6000003D0oooo00<0S`3o0?ooo`3oool00`3oool01P00ofH0c?l00?ooo`3o0000<`3o001V
o`T0oooo00<0c03o0?ooo`3oool0o`3ooon>0?ooo`H000006P3oool00`3o0000oooo0?l000040?oo
o`030<co003oool0c?l000X0oooo00<00?mV0?ooo`00ofH06@3oool2001VoaX0oooo0P0c0?lD0?oo
o`0308l0o`3oool0S`3o00X0oooo00<0c03o0?ooo`3oool02P3oool00`3<0?l0oooo0?ooo`1F0?oo
o`40000000110?ooo`8000001@3oool2000000/0oooo0P00000@0?ooo`030000003oool0oooo01L0
oooo00<0c03o0?ooo`3oool02P3oool00`2?0?l0oooo0?ooo`060?ooo`0303<0o`3oool0oooo00@0
oooo1@3o00020?ooo`06003oIP3oool0oooo0<co003oool0o`000P3oool5000003D0oooo00<0S`3o
0?ooo`3oool0103oool01P3<o`00oooo0?l0000c0?l0oooo001Vo`P0oooo00<0c03o0?ooo`3oool0
o`3ooon>0?ooo`D000006P3oool00`3o0000oooo0?l000040?ooo`030<co003oool0c?l000X0oooo
00<00?mV0?ooo`00ofH06@3oool2001VoaX0oooo0P0c0?lD0?ooo`0308l0o`3oool0S`3o00X0oooo
00<0c03o0?ooo`3oool02P3oool00`3<0?l0oooo0?ooo`1G0?ooo`40000000110?ooo`8000001@3o
ool2000000/0oooo0P00000@0?ooo`030000003oool0oooo01L0oooo00<0c03o0?ooo`3oool02P3o
ool00`2?0?l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00<0oooo1@3o00020?ooo`06003oIP3o
ool0oooo0?ooo`3<o`00o`000P3oool6000003@0oooo00<0S`3o0?ooo`3oool0103oool01P3<o`00
oooo0?l0003oool0<`3o001Vo`T0oooo00<0c03o0?ooo`3oool0o`3ooon<0?ooo`H000006@3oool0
0`3o0000oooo0?l000040?ooo`030<co003oool0c?l000X0oooo00<00?mV0?ooo`00ofH06@3oool2
001VoaX0oooo0P0c0?lD0?ooo`0308l0o`3oool0S`3o00/0oooo00<0c03o0?ooo`3oool02@3oool0
0`3<0?l0oooo0?ooo`1H0?ooo`40000000120?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool02P3oool200000100oooo00<000000?ooo`3oool0603oool00`3<0?l0oooo0?ooo`0:0?oo
o`0308l0o`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool00P3oool60?l000<0oooo00H00?mV0?oo
o`3oool0c?l00?ooo`3o00020?ooo`D00000=@3oool00`2?0?l0oooo0?ooo`030?ooo`060<co003o
ool0oooo0?l0000c0?l006Ko2@3oool00`3<0?l0oooo0?ooo`3o0?oooh`0oooo1@00000I0?ooo`03
0?l0003oool0o`0000D0oooo00<0c?l00?ooo`3<o`002@3oool00`00ofH0oooo003oIP0I0?ooo`80
06Ko6P3oool203<0oa@0oooo00<0S`3o0?ooo`2?0?l02`3oool00`3<0?l0oooo0?ooo`090?ooo`03
0<`0o`3oool0oooo05T0oooo0@0000000480oooo0P0000030?ooo`8000002@3oool500000100oooo
00<000000?ooo`3oool0603oool00`3<0?l0oooo0?ooo`0:0?ooo`0308l0o`3oool0oooo00L0oooo
00<0<`3o0?ooo`3oool00P3oool50?l000<0oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00020?oo
o`H00000=03oool00`2?0?l0oooo0?ooo`030?ooo`06003oIP3<o`00oooo0?l0000c0?l006Ko2@3o
ool00`3<0?l0oooo0?ooo`3o0?oooh/0oooo1P00000I0?ooo`030?l0003oool0o`0000@0oooo00<0
c?l00?ooo`3<o`002@3oool00`00ofH0oooo003oIP0I0?ooo`8006Ko6P3oool203<0oa@0oooo00<0
S`3o0?ooo`2?0?l02`3oool00`3<0?l0oooo0?ooo`090?ooo`030<`0o`3oool0oooo05X0oooo0@00
000004<0oooo1@00000=0?ooo`800000403oool00`000000oooo0?ooo`0I0?ooo`030<`0o`3oool0
oooo00X0oooo00<0S`3o0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`020?ooo`D0o`00103oool0
1P00ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1@00000d0?ooo`0308l0o`3oool0oooo00@0oooo
00H0c?l00?ooo`3o0000<`3o0?ooo`00I_l90?ooo`030<`0o`3oool0oooo0?l0ooooR@3oool60000
01T0oooo00<0o`000?ooo`3o0000103oool00`3<o`00oooo0<co00090?ooo`03003oIP3oool00?mV
01T0oooo00<006Ko0?ooo`00I_l0503oool703<0oa@0oooo00<0S`3o0?ooo`2?0?l00P3oool508l0
o`@0oooo00<0c03o0?ooo`3oool02@3oool00`3<0?l0oooo0?ooo`1K0?ooo`400000001W0?ooo`03
0000003oool0oooo01T0oooo00<0c03o0?ooo`3oool02P3oool00`2?0?l0oooo0?ooo`070?ooo`03
03<0o`3oool0oooo00<0oooo00<0o`000?ooo`00I_l0103oool01P00ofH0oooo0?ooo`3<o`00oooo
0?l00080oooo1P00000d0?ooo`0308l0o`3oool0oooo00<0oooo00H0c?l00?ooo`3o0000oooo03<0
o`00I_l90?ooo`030<`0o`3oool0oooo0?l0ooooR@3oool6000001P0oooo00<0o`000?ooo`3o0000
103oool00`3<o`00oooo0<co00090?ooo`04003oIP3oool0oooo003oIQP0oooo00<006Ko0?ooo`00
I_l05@3oool603<0oaD0oooo0P2?0?l30?ooo`D0S`3o103oool00`3<0?l0oooo0?ooo`080?ooo`03
0<`0o`3oool0oooo05`0oooo0@00000006L0oooo00<000000?ooo`3oool06P3oool00`3<0?l0oooo
0?ooo`0:0?ooo`0308l0o`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool01@3oool00`00I_l0oooo
0?ooo`030?ooo`07003oIP3oool0oooo0<co003oool0o`000?ooo`06000003@0oooo00<0S`3o0?oo
o`3oool00`3oool01P3<o`00oooo0?ooo`3o0000<`3o001Vo`T0oooo00<0c03o0?ooo`3oool0o`3o
oon80?ooo`H000004P3oool50?l000040?ooo`3o0000oooo0?l000@0oooo00@0c?l00?ooo`3oool0
c?l02@3oool00`00ofH0oooo003oIP0H0?ooo`03001Vo`3oool006Ko01D0oooo1P0c0?lE0?ooo`80
S`3o0`3oool608l0o`<0oooo00<0c03o0?ooo`3oool0203oool00`3<0?l0oooo0?ooo`1M0?ooo`40
0000001W0?ooo`030000003oool0oooo01X0oooo00<0c03o0?ooo`3oool02`3oool00`2?0?l0oooo
0?ooo`060?ooo`0303<0o`3oool0oooo00D00?mV00<006Ko0?ooo`3oool00P3oool01P00ofH0oooo
0?ooo`3<o`00oooo0?l00080oooo1@00000d0?ooo`0308l0o`3oool0oooo00@0oooo00D0c?l00?oo
o`3o0000<`3o001Vo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooQ`3oool600000180oooo1@3o0000
103oool0o`000?ooo`3o00040?ooo`030<co003oool0c?l000T0oooo00<00?mV0?ooo`00ofH0603o
ool00`00I_l0oooo001Vo`0G0?ooo`D0<`3o503oool208l0o`D0oooo1@2?0?l20?ooo`030<`0o`3o
ool0oooo00P0oooo00<0c03o0?ooo`3oool0GP3oool100000000I`3oool00`000000oooo0?ooo`0K
0?ooo`030<`0o`3oool0oooo00X0oooo00<0S`3o0?ooo`3oool01`3oool00`0c0?l0oooo003oIP04
003oIP03001Vo`3oool0oooo00<0oooo00L00?mV0?ooo`3oool0c?l00?ooo`3o0000oooo00H00000
=03oool00`2?0?l0oooo0?ooo`030?ooo`060<co003oool0o`0003<0o`3oool006Ko2@3oool00`3<
0?l0oooo0?ooo`3o0?ooohH0oooo1P00000B0?ooo`L0o`0000<0c?l00?l0003<o`000P3<o`00103o
ool0c?l00?ooo`3<o`090?ooo`03003oIP3oool00?mV01P0oooo00<006Ko0?ooo`00I_l0603oool5
03<0oa<0oooo0P2?0?l60?ooo`D0S`3o00<0oooo0<`0o`3oool02@3oool00`3<0?l0oooo0?ooo`1O
0?ooo`400000001W0?ooo`030000003oool0oooo01/0oooo00<0c03o0?ooo`3oool02`3oool00`2?
0?l0oooo0?ooo`060?ooo`0303<0o`00ofH00?mV00@00?mV00<0oooo001Vo`3oool00`3oool01P00
ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1@00000d0?ooo`0308l0o`3oool0oooo00<0oooo00H0
c?l00?ooo`3o0000oooo03<0o`00I_l90?ooo`030<`0o`3oool0oooo0?l0ooooQP3oool5000001@0
oooo1@3o00000`3oool0o`000<co00040<co00030?ooo`3<o`00oooo00P0oooo00<00?mV0?ooo`00
ofH0603oool00`00I_l0oooo001Vo`0I0?ooo`<0<`3o503oool208l0o`T0oooo00D0S`3o0?ooo`3o
ool0oooo0<`0o`090?ooo`030<`0o`3oool0oooo0600oooo0@00000006L0oooo00<000000?ooo`3o
ool0703oool00`3<0?l0oooo0?ooo`0:0?ooo`0308l0o`3oool0oooo00L0oooo00<0<`3o003oIP00
ofH00`00ofH00`3oool006Ko0?ooo`040?ooo`07003oIP3oool0oooo0<co003oool0o`000?ooo`06
000003<0oooo00<0S`3o0?ooo`3oool00`3oool01P00ofH0c?l00?ooo`3o0000<`3o001Vo`X0oooo
00<0c03o0?ooo`3oool0o`3ooon40?ooo`H00000503oool50?l000030<co003o0000c?l000<0c?l0
0P3oool00`3<o`00oooo0?ooo`060?ooo`03003oIP3oool00?mV01P0oooo00<006Ko0?ooo`00I_l0
6P3oool00`0c0?l0oooo0?ooo`0C0?ooo`80S`3o3@3oool00`3<0?l0oooo0?ooo`070?ooo`030<`0
o`3oool0oooo0640oooo0@00000006L0oooo1000000L0?ooo`030<`0o`3oool0oooo00X0oooo00<0
S`3o0?ooo`3oool01P3oool00`0c0?l00?mV003oIP03003oIP80oooo00<006Ko0?ooo`3oool00P3o
ool01P00ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1@00000d0?ooo`0308l0o`3oool0oooo00<0
oooo00D0c?l00?ooo`3o0000<`3o001Vo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooQ03oool50000
01L0oooo00@0o`000?ooo`3oool0o`001@3<o`00103oool0c?l00?ooo`3oool5003oIP040?ooo`00
ofH0oooo003oIQP0oooo00<006Ko0?ooo`00I_l06P3oool00`0c0?l0oooo0?ooo`0C0?ooo`80S`3o
3@3oool00`3<0?l0oooo0?ooo`070?ooo`030<`0o`3oool0oooo05l0oooo1000000006L0oooo00<0
00000?ooo`3oool07P3oool00`3<0?l0oooo0?ooo`060?ooo`D0<`3o203oool00`0c0?l0oooo003o
IP040?ooo`03001Vo`3oool0oooo00<0oooo00D00?mV0?ooo`3oool0c?l00?l000020?ooo`H00000
<`3oool00`2?0?l0oooo0?ooo`030?ooo`060<co003oool0o`0003<0o`3oool006Ko2@3oool00`3<
0?l0oooo0?ooo`3o0?oooh<0oooo1P00000G0?ooo`040?l0003oool0o`000?ooo`H0c?l00`3oool6
003oIP030?ooo`00ofH0oooo01L0oooo00<006Ko0?ooo`00I_l06P3oool00`0c0?l0oooo0?ooo`0D
0?ooo`0308l0o`3oool0oooo00/0oooo00<0c03o0?ooo`3oool01`3oool00`3<0?l0oooo0?ooo`1S
0?ooo`400000001W0?ooo`030000003oool0oooo01h0oooo00<0c03o0?ooo`3oool01P3oool503<0
o`T0oooo00<0<`3o0?ooo`3oool0103oool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0oooo
0<co003oool0o`000P3oool5000003<0oooo00<0S`3o0?ooo`3oool00`3oool01P3<o`00oooo0?l0
003oool0<`3o001Vo`X0oooo00<0c03o0?ooo`3oool0o`3ooon20?ooo`D000005`3oool00`3o0000
oooo0?l000040?ooo`040<co003oool0oooo0<co0080oooo1P00ofH00`3oool00?mV0?ooo`0G0?oo
o`03001Vo`3oool006Ko01X0oooo00<0<`3o0?ooo`3oool0503oool00`2?0?l0oooo0?ooo`0;0?oo
o`030<`0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool0I03oool100000000I`3oool00`000000
oooo0?ooo`0O0?ooo`030<`0o`3oool0oooo00@0oooo1P0c0?l90?ooo`0303<0o`3oool0oooo00D0
oooo00<006Ko0?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3<o`00o`000080oooo1P00000c0?oo
o`0308l0o`3oool0oooo0080oooo00H00?mV0<co003oool0o`0003<0o`00I_l:0?ooo`030<`0o`3o
ool0oooo0?l0ooooP@3oool6000001H0oooo00@0o`000?ooo`3oool0o`000`3oool0103<o`00oooo
0?ooo`3<o`040?ooo`D00?mV00<0oooo003oIP3oool05`3oool2001VoaX0oooo00<0<`3o0?ooo`3o
ool0503oool00`2?0?l0oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo00H0oooo00<0c03o0?ooo`3o
ool0I@3oool100000000I`3oool00`000000oooo0?ooo`0P0?ooo`030<`0o`3oool0oooo00@0oooo
1@0c0?l00`2?0?l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00@0oooo00<006Ko0?ooo`3oool0
0P3oool01P00ofH0oooo0?ooo`3<o`00oooo0?l00080oooo1@00000c0?ooo`0308l0o`3oool0oooo
00<0oooo00D0c?l00?ooo`3o0000<`3o001Vo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooP03oool6
000001H0oooo00@0o`000?ooo`3oool0o`000`3oool0103<o`00oooo0?ooo`3<o`050?ooo`H00?mV
603oool00`00I_l0oooo001Vo`0I0?ooo`0303<0o`3oool0oooo01@0oooo00<0S`3o0?ooo`3oool0
303oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo06H0oooo0@00000006L0oooo00<0
00000?ooo`3oool0803oool00`3<0?l0oooo0?ooo`040?ooo`D0<`3o00<0S`3o0?ooo`3oool01`3o
ool00`0c0?l0oooo0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo00D00?mV0?ooo`3oool0c?l0
0?l000020?ooo`H00000<P3oool00`2?0?l0oooo0?ooo`030?ooo`060<co003oool0o`0003<0o`3o
ool006Ko2P3oool00`3<0?l0oooo0?ooo`3o0?ooogl0oooo1P00000E0?ooo`040?l0003oool0oooo
0?l000<0oooo00@0c?l00?ooo`3oool0c?l01`3oool2003oIP030?ooo`00ofH0oooo01L0oooo00<0
06Ko0?ooo`00I_l06@3oool00`0c0?l0oooo0?ooo`0D0?ooo`80S`3o3@3oool00`3<0?l0oooo0?oo
o`060?ooo`030<`0o`3oool0oooo06L0oooo0@00000006L0oooo00<000000?ooo`3oool08@3oool0
0`3<0?l0oooo0?ooo`050?ooo`0503<0o`3oool0oooo0?ooo`2?0?l02@3oool00`0c0?l0oooo0?oo
o`040?ooo`03001Vo`3oool0oooo0080oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o00020?ooo`D0
0000<`3oool00`2?0?l0oooo0?ooo`020?ooo`060<co003oool0o`000?ooo`0c0?l006Ko2P3oool0
0`3<0?l0oooo0?ooo`3o0?ooogh0oooo1P00000F0?ooo`040?l0003oool0oooo0?l000<0oooo00@0
c?l00?ooo`3oool0c?l01`3oool00`00ofH0oooo003oIP0H0?ooo`03001Vo`3oool006Ko01T0oooo
00<0<`3o0?ooo`3oool0503oool208l0o`d0oooo00<0c03o0?ooo`3oool01P3oool00`3<0?l0oooo
0?ooo`1X0?ooo`400000001W0?ooo`030000003oool0oooo0280oooo00<0c03o0?ooo`3oool0203o
ool00`2?0?l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00@0oooo00<006Ko0?ooo`3oool00P3o
ool01@00ofH0oooo0?ooo`3<o`00o`000080oooo1@00000c0?ooo`0308l0o`3oool0oooo00<0oooo
00D0c?l00?ooo`3o0000<`3o001Vo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooOP3oool5000001H0
oooo00@0o`000?ooo`3oool0o`000`3oool0103<o`00oooo0?ooo`3<o`070?ooo`03003oIP3oool0
0?mV01P0oooo00<006Ko0?ooo`00I_l06@3oool00`0c0?l0oooo0?ooo`0D0?ooo`80S`3o3@3oool0
0`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo06T0oooo0@00000006L0oooo00<000000?oo
o`3oool08P3oool00`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00L0oooo00<0<`3o0?oo
o`3oool00P3oool50<co0080oooo00L00?mV0?ooo`3oool0c?l00?ooo`3o0000oooo00H00000<P3o
ool00`2?0?l0oooo0?ooo`030?ooo`050<co003oool0o`0003<0o`00I_l02`3oool00`3<0?l0oooo
0?ooo`3o0?ooog`0oooo1P00000E0?ooo`040?l0003oool0oooo0?l000<0oooo00@0c?l00?ooo`3o
ool0c?l01`3oool00`00ofH0oooo003oIP0A0?ooo`D006Ko0P3oool00`00I_l0oooo001Vo`0I0?oo
o`0303<0o`3oool0oooo01D0oooo00<0S`3o0?ooo`3oool0303oool00`3<0?l0oooo0?ooo`050?oo
o`030<`0o`3oool0oooo06X0oooo0@00000006L0oooo00<000000?ooo`3oool08`3oool00`3<0?l0
oooo0?ooo`090?ooo`0308l0o`3oool0oooo00L0oooo00@0<`3o0?ooo`3oool0oooo1@3<o`030?oo
o`05003oIP3oool0oooo0<co003o00000P3oool5000003<0oooo00<0S`3o0?ooo`3oool00P3oool0
1P3<o`00oooo0?l0000c0?l0oooo001Vo`X0oooo00<0c03o0?ooo`3oool0o`3oooml0?ooo`D00000
5@3oool0103o0000oooo0?ooo`3o00030?ooo`040<co003oool0oooo0<co00L0oooo00@00?mV0?oo
o`3oool00?mV4@3oool5001Vo`040?ooo`00I_l0oooo001VoaT0oooo00<0<`3o0?ooo`3oool05@3o
ool00`2?0?l0oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool0J`3o
ool100000000I`3oool00`000000oooo0?ooo`0T0?ooo`030<`0o`3oool0oooo00P0oooo00<0S`3o
0?ooo`3oool0203oool00`0c0?l0oooo0<co00050<co00<0oooo00L00?mV0?ooo`3oool0c?l00?oo
o`3o0000oooo00H00000<P3oool00`2?0?l0oooo0?ooo`020?ooo`060<co003oool0o`000?ooo`0c
0?l006Ko2P3oool00`3<0?l0oooo0?ooo`3o0?ooog/0oooo1P00000E0?ooo`040?l0003oool0oooo
0?l00080oooo00D0c?l00?ooo`3oool0oooo0<co00060?ooo`04003oIP3oool0oooo003oIQ40oooo
1`00I_l20?ooo`03001Vo`3oool0oooo01H0oooo00<0<`3o0?ooo`3oool05@3oool00`2?0?l0oooo
0?ooo`0<0?ooo`030<`0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool0K03oool100000000I`3o
ool00`000000oooo0?ooo`0T0?ooo`030<`0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3oool01`3o
ool00`0c0?l0oooo0?ooo`050<co00@0oooo00D00?mV0?ooo`3oool0c?l00?l000020?ooo`D00000
<P3oool00`2?0?l0oooo0?ooo`030?ooo`050<co003oool0o`0003<0o`00I_l02`3oool00`3<0?l0
oooo0?ooo`3o0?ooogX0oooo1@00000E0?ooo`040?l0003oool0oooo0?l00080oooo00D0c?l00?oo
o`3oool0oooo0<co00070?ooo`03003oIP3oool00?mV01<0oooo1@00I_l20?ooo`03001Vo`3oool0
oooo01H0oooo0P0c0?lF0?ooo`0308l0o`3oool0oooo00`0oooo00<0c03o0?ooo`3oool01@3oool0
0`3<0?l0oooo0?ooo`1]0?ooo`400000001W0?ooo`030000003oool0oooo02D0oooo00<0c03o0?oo
o`3oool02@3oool00`2?0?l0oooo0?ooo`070?ooo`0303<0o`3oool0c?l000@0c?l0103oool01`00
ofH0oooo0?ooo`3<o`00oooo0?l0003oool01P00000b0?ooo`0308l0o`3oool0oooo0080oooo00H0
c?l00?ooo`3o0000<`3o0?ooo`00I_l:0?ooo`030<`0o`3oool0oooo0?l0ooooN@3oool6000001@0
oooo00@0o`000?ooo`3oool0o`000`3oool0103<o`00oooo0?ooo`3<o`070?ooo`03003oIP3oool0
0?mV01@0oooo1@00I_l00`3oool006Ko0?ooo`0G0?ooo`80<`3o5P3oool00`2?0?l0oooo0?ooo`0=
0?ooo`030<`0o`3oool0oooo00@0oooo00<0c03o0?ooo`3oool0KP3oool100000000I`3oool00`00
0000oooo0?ooo`0V0?ooo`030<`0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3oool01P3oool01@0c
0?l0oooo0?ooo`3oool0c?l00080oooo00<006Ko0?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3<
o`00o`000080oooo1@00000b0?ooo`0308l0o`3oool0oooo0080oooo00H0c?l00?ooo`3o0000<`3o
0?ooo`00I_l:0?ooo`030<`0o`3oool0oooo0?l0ooooN03oool6000001@0oooo00@0o`000?ooo`3o
ool0o`000`3oool01@3<o`00oooo0?ooo`3oool0c?l000H0oooo00@00?mV0?ooo`3oool00?mV5P3o
ool2001Vo`030?ooo`00I_l0oooo01L0oooo0P0c0?lF0?ooo`0308l0o`3oool0oooo00d0oooo00<0
c03o0?ooo`3oool0103oool00`3<0?l0oooo0?ooo`1_0?ooo`400000001W0?ooo`030000003oool0
oooo02H0oooo00<0c03o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`070?ooo`0303<0o`3oool0
oooo00@0oooo00D006Ko0?ooo`3oool0oooo003oIP020?ooo`040<co003oool0o`000?ooo`H00000
<@3oool00`2?0?l0oooo0?ooo`020?ooo`06003oIP3<o`00o`000?ooo`0c0?l006Ko2`3oool00`3<
0?l0oooo0?ooo`3o0?ooogL0oooo1P00000=0?ooo`D0o`0000<0oooo0?l0003oool00P3oool0103o
0000oooo0?ooo`3<o`030?ooo`030<co003oool0oooo00@0oooo00@00?mV0?ooo`3oool00?mV5`3o
ool00`00I_l0oooo001Vo`0H0?ooo`80<`3o5P3oool00`2?0?l0oooo0?ooo`0=0?ooo`030<`0o`3o
ool0oooo00@0oooo00<0c03o0?ooo`3oool0L03oool100000000I`3oool00`000000oooo0?ooo`0W
0?ooo`030<`0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`03
0?ooo`03001Vo`3oool0oooo0080oooo00D00?mV0?ooo`3oool0c?l00?l000020?ooo`D00000<P3o
ool00`2?0?l0oooo0?ooo`020?ooo`060<co003oool0o`0003<0o`3oool006Ko2P3oool00`3<0?l0
oooo0?ooo`3o0?ooogH0oooo1P00000>0?ooo`D0o`0000D0oooo0?l0003oool0oooo0?l000020?oo
o`050<co003oool0oooo0?ooo`3<o`001P3oool01000ofH0oooo0?ooo`00ofHG0?ooo`04001Vo`3o
ool0oooo001VoaL0oooo0P0c0?lF0?ooo`80S`3o3P3oool00`3<0?l0oooo0?ooo`040?ooo`030<`0
o`3oool0oooo0740oooo0@00000006L0oooo1000000W0?ooo`030<`0o`3oool0oooo00T0oooo00<0
S`3o0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`030?ooo`D0o`0000L00?mV0?ooo`3oool0c?l0
0?ooo`3o0000oooo00D00000<P3oool00`2?0?l0oooo0?ooo`020?ooo`060<co003oool0o`0003<0
o`3oool006Ko2P3oool00`3<0?l0oooo0?ooo`3o0?ooogH0oooo1@00000>0?ooo`L0o`000P3oool0
103o0000oooo0?ooo`3<o`030?ooo`030<co003oool0oooo00D0oooo00<00?mV0?ooo`00ofH05`3o
ool01000I_l0oooo0?ooo`00I_lG0?ooo`80<`3o5P3oool208l0o`h0oooo00<0c03o0?ooo`3oool0
103oool00`3<0?l0oooo0?ooo`1_0?ooo`@00000001W0?ooo`030000003oool0oooo02T0oooo00<0
c03o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`060?ooo`0303<0o`3oool0oooo0080oooo1@3o
00001`3oool00?mV0?ooo`3oool0c?l00?l0003oool01P00000a0?ooo`0308l0o`3oool0oooo0080
oooo00H0c?l00?ooo`3o0000<`3o0?ooo`00I_l;0?ooo`030<`0o`3oool0oooo0?l0ooooM03oool6
000000l0oooo1@3o00030?ooo`040?l0003oool0oooo0<co00<0oooo00<0c?l00?ooo`3oool0103o
ool01000ofH0oooo0?ooo`00ofHF0?ooo`04001Vo`3oool0oooo001VoaL0oooo0P0c0?lF0?ooo`80
S`3o3`3oool00`3<0?l0oooo0?ooo`030?ooo`030<`0o`3oool0oooo07<0oooo0@00000006L0oooo
00<000000?ooo`3oool0:@3oool00`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00H0oooo
00@0<`3o0?ooo`3oool0oooo1P3o0000203oool00?mV0?ooo`3oool0c?l00?ooo`3o0000oooo1@00
000b0?ooo`0:08l0o`3oool0oooo0?ooo`00ofH0c?l00?l0003oool0<`3o001Vo`/0oooo00<0c03o
0?ooo`3oool0o`3ooomd0?ooo`D00000403oool50?l00080oooo00@0o`000?ooo`3oool0c?l00`3o
ool00`3<o`00oooo0?ooo`040?ooo`04003oIP3oool0oooo003oIQH0oooo00@006Ko0?ooo`3oool0
06Ko5`3oool203<0oaH0oooo0P2?0?l?0?ooo`030<`0o`3oool0oooo00<0oooo00<0c03o0?ooo`3o
ool0M03oool100000000I`3oool00`000000oooo0?ooo`0Z0?ooo`030<`0o`3oool0oooo00T0oooo
00<0S`3o0?ooo`3oool01P3oool0100c0?l0oooo0?ooo`3oool50?l00080oooo00H00?mV0?ooo`3o
ool0c?l00?l0003oool500000380oooo00<0S`3o0?ooo`3oool00P3oool01P3<o`00oooo0?l0000c
0?l0oooo001Vo`X0oooo00<0c03o0?ooo`3oool0o`3ooomc0?ooo`H000004P3oool20?l00080oooo
00@0o`000?ooo`3oool0c?l00`3oool00`3<o`00oooo0?ooo`040?ooo`04003oIP3oool0oooo003o
IQH0oooo00@006Ko0?ooo`3oool006Ko5`3oool203<0oaH0oooo0P2?0?l?0?ooo`030<`0o`3oool0
oooo00<0oooo00<0c03o0?ooo`3oool0M@3oool100000000I`3oool00`000000oooo0?ooo`0[0?oo
o`030<`0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`050?l0
0080oooo00H00?mV0?ooo`3oool0c?l00?ooo`3o000600000340oooo00<0S`3o0?ooo`3oool00P3o
ool01P3<o`00oooo0?l0000c0?l0oooo001Vo`/0oooo00<0c03o0?ooo`3oool0o`3ooomb0?ooo`D0
00004`3oool0103o0000oooo0?ooo`3o00020?ooo`050<co003oool0oooo0?ooo`3<o`001P3oool0
1000ofH0oooo0?ooo`00ofHF0?ooo`04001Vo`3oool0oooo001VoaL0oooo0P0c0?lF0?ooo`80S`3o
3`3oool00`3<0?l0oooo0?ooo`030?ooo`030<`0o`3oool0oooo07H0oooo0@00000006L0oooo00<0
00000?ooo`3oool0:`3oool00`3<0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00H0oooo00<0
<`3o0?ooo`3oool00P3oool00`3o000006Ko0?ooo`030?ooo`06003oIP3oool0oooo0<co003o0000
oooo1@00000b0?ooo`0:08l0o`3oool0oooo0?ooo`3<o`00oooo0?l0003oool0<`3o001Vo`/0oooo
00<0c03o0?ooo`3oool0o`3oooma0?ooo`H000004P3oool01@3o0000c?l00<co003<o`00o`000080
c?l0103oool00`3<o`00oooo0?ooo`040?ooo`04003oIP3oool0oooo003oIQD0oooo00D006Ko0?oo
o`3oool0oooo001Vo`0F0?ooo`80<`3o5P3oool208l0o`l0oooo00<0c03o0?ooo`3oool00`3oool0
0`3<0?l0oooo0?ooo`1g0?ooo`400000001W0?ooo`030000003oool0oooo02`0oooo00<0c03o0?oo
o`3oool02@3oool00`2?0?l0oooo0?ooo`060?ooo`0303<0o`3oool0oooo00<0oooo00D006Ko0?oo
o`3oool0oooo003oIP020?ooo`030<co003oool0o`0000H00000<@3oool00`2?0?l0oooo0?ooo`02
0?ooo`060<co003o0000oooo03<0o`3oool006Ko2P3oool00`3<0?l0oooo0?ooo`3o0?ooog00oooo
1P00000B0?ooo`050?l0003oool0c?l00<co003o00000`3<o`030?ooo`030<co003oool0oooo00@0
oooo00@00?mV0?ooo`3oool00?mV5P3oool01000I_l0oooo0?ooo`00I_lF0?ooo`80<`3o5P3oool2
08l0oa00oooo00<0c03o0?ooo`3oool00P3oool00`3<0?l0oooo0?ooo`1h0?ooo`400000001W0?oo
o`030000003oool0oooo02d0oooo00<0c03o08l0o`2?0?l00`2?0?l60?ooo`0308l0o`3oool0oooo
00D0oooo00<0<`3o0?ooo`3oool00`3oool00`00I_l0oooo0?ooo`020?ooo`06003oIP3oool0oooo
0<co003o0000oooo1@00000b0?ooo`0:08l0o`3oool0oooo0?ooo`3<o`00oooo0?l0000c0?l0oooo
001Vo`/0oooo00<0c03o0?ooo`3oool0o`3ooom_0?ooo`H000004P3oool0103o0000c?l00<co003o
00030<co00<0oooo00<0c?l00?ooo`3oool0103oool01000ofH0oooo0?ooo`00ofHF0?ooo`04001V
o`3oool0oooo001VoaH0oooo00<0<`3o0?ooo`0c0?l05@3oool208l0oa00oooo00<0c03o0?ooo`3o
ool00P3oool00`3<0?l0oooo0?ooo`1i0?ooo`400000001W0?ooo`030000003oool0oooo02d0oooo
00<0c03o08l0o`2?0?l00`2?0?l70?ooo`0308l0o`3oool0oooo00D0oooo00<0<`3o0?ooo`3oool0
0`3oool01@00I_l0oooo0?ooo`3oool00?mV0080oooo00<0c?l00?l0003oool01@00000b0?ooo`0:
08l0o`3oool0oooo0?ooo`3<o`00oooo0?l0003oool0<`3o001Vo`/0oooo00<0c03o0?ooo`3oool0
o`3ooom^0?ooo`H000004P3oool01@3o0000oooo0?ooo`3<o`00o`0000<0c?l00`3oool00`3<o`00
oooo0?ooo`030?ooo`05003oIP3oool0oooo0?ooo`00ofH05@3oool01000I_l0oooo0?ooo`00I_lF
0?ooo`0303<0o`3oool0<`3o01D0oooo00<0S`3o0?ooo`2?0?l03`3oool0103<0?l0oooo0?ooo`3o
ool20<`0og`0oooo0@00000006L0oooo00<000000?ooo`3oool0;@3oool608l0o`L0oooo00<0S`3o
0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo0080oooo00D00?mV
0?ooo`3oool0c?l00?l0000600000340oooo00/0S`3o0?ooo`3oool0oooo0<co003oool0o`000?oo
o`0c0?l0oooo001Vo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooKP3oool600000140oooo00D0o`00
0?ooo`3oool0oooo0?l000040<co0080oooo00<0c?l00?ooo`3oool0103oool01000ofH0oooo0?oo
o`00ofHE0?ooo`04001Vo`3oool0oooo001VoaH0oooo00<0<`3o0?ooo`0c0?l05@3oool00`2?0?l0
oooo08l0o`0?0?ooo`050<`0o`3oool0oooo0?ooo`3<0?l0OP3oool100000000I`3oool00`000000
oooo0?ooo`0^0?ooo`D0S`3o203oool00`2?0?l0oooo0?ooo`050?ooo`0303<0o`3oool0oooo00<0
oooo00D006Ko0?ooo`3oool0oooo003oIP020?ooo`030<co003o0000oooo00D00000<P3oool01P2?
0?l0oooo0?ooo`3oool0c?l00?l00080oooo00<0<`3o001Vo`3oool02P3oool00`3<0?l0oooo0?oo
o`3o0?ooof`0oooo1P00000B0?ooo`050?l0003oool0oooo0?l0003oool00P3<o`030?ooo`030<co
003oool0oooo00@0oooo00@00?mV0?ooo`3oool00?mV5@3oool01000I_l0oooo0?ooo`00I_lF0?oo
o`0303<0o`3oool0<`3o01D0oooo00<0S`3o0?ooo`2?0?l03`3oool01@3<0?l0oooo0?ooo`3oool0
c03o07l0oooo0@00000006L0oooo00<000000?ooo`3oool0;P3oool508l0o`T0oooo00<0S`3o0?oo
o`3oool01@3oool00`0c0?l0oooo0?ooo`030?ooo`09001Vo`3oool0oooo0?ooo`00ofH0oooo0<co
003oool0o`0000D00000<P3oool02P2?0?l0oooo0?ooo`3oool0c?l00?ooo`3o0000oooo03<0o`00
I_l;0?ooo`030<`0o`3oool0oooo0?l0ooooK03oool500000180oooo00L0o`000?ooo`3oool0oooo
0?l0003oool0c?l000<0oooo00<0c?l00?ooo`3oool0103oool01000ofH0oooo0?ooo`00ofHE0?oo
o`05001Vo`3oool0oooo0?ooo`00I_l05@3oool00`0c0?l0oooo03<0o`0E0?ooo`0308l0o`3oool0
S`3o0100oooo00@0c03o0?ooo`3oool0c03oP03oool100000000I`3oool00`000000oooo0?ooo`0`
0?ooo`0308l0o`3<0?l0oooo00T0oooo00<0S`3o0?ooo`3oool01@3oool00`0c0?l0oooo0?ooo`03
0?ooo`05001Vo`3oool0oooo0?ooo`00ofH00P3oool00`3<o`00o`000000000500000340oooo00/0
S`3o0?ooo`3oool0oooo0<co003oool0o`000?ooo`0c0?l0oooo001Vo`0;0?ooo`030<`0o`3oool0
oooo0?l0ooooJP3oool600000140oooo00L0o`000?ooo`3oool0oooo0?l0003oool0c?l000@0oooo
00<0c?l00?ooo`3oool00`3oool01@00ofH0oooo0?ooo`3oool00?mV01@0oooo00D006Ko0?ooo`3o
ool0oooo001Vo`0E0?ooo`0303<0o`3oool0<`3o01D0oooo00<0S`3o0?ooo`2?0?l0403oool0103<
0?l0oooo0?ooo`3<0?n10?ooo`400000001W0?ooo`030000003oool0oooo0340oooo00<0c03o0?oo
o`3oool02@3oool00`2?0?l0oooo0?ooo`050?ooo`0303<0o`3oool0oooo00<0oooo00T006Ko0?oo
o`3oool0oooo003oIP3oool0c?l00?ooo`3o00001@00000b0?ooo`0608l0o`3oool0oooo003oIP3<
o`00o`000P3oool00`0c0?l006Ko0?ooo`0:0?ooo`030<`0o`3oool0oooo0?l0ooooJP3oool50000
0140oooo00L0o`000?ooo`3oool0oooo0?l0003oool0c?l000@0oooo00<0c?l00?ooo`3oool00`3o
ool01@00ofH0oooo0?ooo`3oool00?mV01@0oooo00D006Ko0?ooo`3oool0oooo001Vo`0E0?ooo`03
03<0o`3oool0<`3o01D0oooo00<0S`3o0?ooo`2?0?l0403oool0103<0?l0oooo0?ooo`3<0?n20?oo
o`400000001W0?ooo`030000003oool0oooo0380oooo00<0c03o0?ooo`3oool02@3oool00`2?0?l0
oooo0?ooo`050?ooo`0303<0o`3oool0oooo0080oooo00D006Ko0?ooo`3oool0oooo003oIP020?oo
o`030<co003o0000000000D00000<@3oool01P2?0?l0oooo0?ooo`3oool0c?l00?l00080oooo00<0
<`3o001Vo`3oool02P3oool00`3<0?l0oooo0?ooo`3o0?ooofT0oooo1P00000A0?ooo`060?l0003o
ool0oooo0?ooo`3o0000c?l0103oool00`3<o`00oooo0?ooo`040?ooo`04003oIP3oool0oooo003o
IQ@0oooo00D006Ko0?ooo`3oool0oooo001Vo`0E0?ooo`0303<0o`3oool0<`3o01D0oooo00<0S`3o
0?ooo`2?0?l0403oool0103<0?l0oooo0?ooo`3<0?n30?ooo`400000001W0?ooo`030000003oool0
oooo03<0oooo00<0c03o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`040?ooo`0303<0o`3oool0
oooo00<0oooo00T006Ko0?ooo`3oool0oooo003oIP3oool0c?l00?l0003oool01@00000a0?ooo`0;
08l0o`3oool0oooo0?ooo`3<o`00oooo0?l0003oool0<`3o0?ooo`00I_l02`3oool00`3<0?l0oooo
0?ooo`3o0?ooofP0oooo1@00000A0?ooo`070?l0003oool0oooo0?ooo`3o0000oooo0<co00030?oo
o`030<co0000ofH00?mV00<00?mV00D0oooo003oIP3oool0oooo003oIP0E0?ooo`04001Vo`3oool0
oooo001VoaD0oooo00<0<`3o0?ooo`0c0?l05@3oool00`2?0?l0oooo08l0o`0A0?ooo`H0c03oP@3o
ool100000000I`3oool00`000000oooo0?ooo`0d0?ooo`030<`0o`3oool0oooo00P0oooo00<0S`3o
0?ooo`3oool01@3oool6001Vo`<0oooo00D00?mV0?ooo`3oool0c?l00?l0000500000380oooo00H0
S`3o0?ooo`3oool0c?l00?ooo`3o00020?ooo`0303<0o`00I_l0oooo00X0oooo00<0c03o0?ooo`3o
ool0o`3ooomW0?ooo`H00000403oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3<o`00103oool0
0`3<o`000?mV003oIP04003oIP<0oooo00<00?mV0?ooo`3oool04P3oool01@00I_l0oooo0?ooo`3o
ool006Ko01@0oooo00<0<`3o0?ooo`0c0?l05@3oool00`2?0?l0oooo08l0o`0A0?ooo`030<`0o`3o
ool0c03o00@0c03oP@3oool100000000I`3oool4000003@0oooo00<0c03o0?ooo`3oool0203oool0
0`2?0?l0oooo0?ooo`040?ooo`D006Ko00<0oooo001Vo`3oool00P3oool01000ofH0oooo0<co003o
000600000340oooo00H0S`3o0?ooo`3oool0oooo0<co003o00020?ooo`0303<0o`00I_l0oooo00/0
oooo00<0c03o0?ooo`3oool0o`3ooomU0?ooo`H000004@3oool01P3o0000oooo0?ooo`3oool0o`00
0<co00@0oooo00<0c?l0003oIP00ofH01000ofH30?ooo`03003oIP3oool0oooo0180oooo00D006Ko
0?ooo`3oool0oooo001Vo`0D0?ooo`0403<0o`3oool0oooo03<0oa@0oooo00<0S`3o0?ooo`2?0?l0
4@3oool00`3<0?l0oooo0<`0o`050<`0ogh0oooo1000000006L0oooo00<000000?ooo`3oool0=P3o
ool00`3<0?l0oooo0?ooo`080?ooo`0308l0o`3oool0oooo0080oooo1P00I_l00`3oool006Ko0?oo
o`020?ooo`05003oIP3oool0oooo0<co003o00001@00000a0?ooo`0:08l0o`3oool0oooo0?ooo`3<
o`00oooo0?l0003oool0<`3o001Vo``0oooo00<0c03o0?ooo`3oool0o`3ooomU0?ooo`H00000403o
ool01`3o0000oooo0?ooo`3oool0o`000?ooo`3<o`000`3oool00`3<o`00oooo0?ooo`05003oIP80
oooo00<00?mV0?ooo`3oool04P3oool01@00I_l0oooo0?ooo`3oool006Ko01@0oooo00@0<`3o0?oo
o`3oool0<`3o503oool00`2?0?l0oooo08l0o`0A0?ooo`040<`0o`3oool0c03o0?ooo`D0c03oP@3o
ool100000000I`3oool00`000000oooo0?ooo`0g0?ooo`030<`0o`3oool0oooo00P0oooo00<0S`3o
0?ooo`3oool00P3oool5001Vo`80oooo00P006Ko0?ooo`3oool0oooo003oIP3oool0c?l00?l000D0
0000<P3oool01P2?0?l0oooo0?ooo`3<o`00oooo0?l00080oooo00<0<`3o001Vo`3oool02`3oool0
0`3<0?l0oooo0?ooo`3o0?ooof<0oooo1P00000@0?ooo`070?l0003oool0oooo0?ooo`3o0000oooo
0<co00040?ooo`030<co003oool0oooo00D00?mV0P3oool00`00ofH0oooo0?ooo`0A0?ooo`05001V
o`3oool0oooo0?ooo`00I_l05@3oool00`0c0?l0oooo03<0o`0D0?ooo`0308l0o`3oool0S`3o0140
oooo00<0c03o0?ooo`3<0?l00P3oool50<`0oh40oooo0@00000006L0oooo00<000000?ooo`3oool0
>03oool00`3<0?l0oooo0?ooo`070?ooo`0308l0o`3oool0oooo0080oooo1@00I_l20?ooo`09001V
o`3oool0oooo0?ooo`00ofH0oooo0<co003oool0o`0000D00000<@3oool01P2?0?l0oooo0?ooo`3o
ool0c?l00?l00080oooo00<0<`3o001Vo`3oool02`3oool00`3<0?l0oooo0?ooo`3o0?ooof<0oooo
1@00000;0?ooo`D0o`0000<0oooo0?l0003oool00P3oool00`3o0000c?l00?ooo`030?ooo`030<co
003oool0oooo00<0oooo00D00?mV0?ooo`3oool0oooo003oIP0C0?ooo`05001Vo`3oool0oooo0?oo
o`00I_l05@3oool00`0c0?l0oooo03<0o`0D0?ooo`0408l0o`3oool0oooo08l0oa40oooo0P3<0?l5
0?ooo`030<`0o`3oool0oooo0840oooo0@00000006L0oooo00<000000?ooo`3oool0>03oool00`3<
0?l0oooo0?ooo`080?ooo`0308l0o`3oool0oooo00<0oooo00<006Ko03<0o`3oool00`3oool02000
I_l0oooo0?ooo`3oool00?mV0?ooo`3<o`00o`001@00000a0?ooo`0608l0o`3oool0oooo0?ooo`3<
o`00o`000P3oool00`0c0?l006Ko0?ooo`0;0?ooo`030<`0o`3oool0oooo0?l0ooooHP3oool60000
00/0oooo1P3o00030?ooo`030?l0003oool0c?l000<0oooo00<0c?l00?ooo`3oool00`3oool01@00
ofH0oooo0?ooo`3oool00?mV01<0oooo00D006Ko0?ooo`3oool0oooo001Vo`0E0?ooo`0303<0o`3o
ool0<`3o01D0oooo00<0S`3o0?ooo`2?0?l04@3oool20<`0ohX0oooo0@00000006L0oooo00<00000
0?ooo`3oool0>@3oool00`3<0?l0oooo0?ooo`080?ooo`0308l0o`3oool0oooo00@0oooo00<0<`3o
0?ooo`00ofH01000ofH01P3oool00?mV0?ooo`3<o`00oooo0?l000D00000<@3oool01P2?0?l0oooo
0?ooo`3<o`00oooo0?l00080oooo00<0<`3o001Vo`3oool02`3oool00`3<0?l0oooo0?ooo`3o0?oo
of40oooo1@00000;0?ooo`H0o`000`3oool00`3o0000oooo0<co00040?ooo`030<co003oool0oooo
0080oooo00D00?mV0?ooo`3oool0oooo003oIP0D0?ooo`05001Vo`3oool0oooo0?ooo`00I_l0503o
ool00`0c0?l0oooo03<0o`0E0?ooo`0308l0o`3oool0S`3o0140oooo0P3<0?n;0?ooo`400000001W
0?ooo`030000003oool0oooo03X0oooo00<0c03o0?ooo`3oool01`3oool00`2?0?l0oooo0?ooo`04
0?ooo`0303<0o`3oool00?mV00@00?mV0P3oool01000ofH0oooo0<co003o000500000340oooo00H0
S`3o0?ooo`3oool0c?l00?ooo`3o00020?ooo`0303<0o`00I_l0oooo00/0oooo00<0c03o0?ooo`3o
ool0o`3ooomP0?ooo`H00000303oool50?l000<0oooo00<0o`000<co003oool00`3oool00`3<o`00
oooo0?ooo`020?ooo`03003oIP3oool0oooo0080oooo00<00?mV0?ooo`3oool04@3oool01@00I_l0
oooo0?ooo`3oool006Ko01@0oooo00<0<`3o0?ooo`0c0?l05@3oool00`2?0?l0oooo08l0o`0A0?oo
o`80c03oS03oool100000000I`3oool00`000000oooo0?ooo`0k0?ooo`030<`0o`3oool0oooo00L0
oooo00<0S`3o0?ooo`3oool0103oool6003oIP80oooo00@00?mV0?ooo`3<o`00o`001@00000a0?oo
o`0608l0o`3oool0oooo0?ooo`3<o`00o`000`3oool00`00I_l0oooo0?ooo`0;0?ooo`030<`0o`3o
ool0oooo0?l0ooooG`3oool5000000d0oooo1@3o00020?ooo`030?l0003oool0c?l000<0oooo00<0
c?l00?ooo`3oool00`3oool01@00ofH0oooo0?ooo`3oool00?mV01<0oooo00D006Ko0?ooo`3oool0
oooo001Vo`0D0?ooo`0403<0o`3oool0oooo03<0oa@0oooo00<0S`3o0?ooo`2?0?l04@3oool20<`0
ohd0oooo0@00000006L0oooo00<000000?ooo`3oool0?03oool00`3<0?l0oooo0?ooo`070?ooo`03
08l0o`3oool0oooo00<0oooo00@0<`3o003oIP00ofH00?mV1@3<o`001000ofH0oooo0<co003o0005
00000340oooo00D0S`3o0?ooo`3oool0c?l00?l000030?ooo`0303<0o`00I_l0oooo00/0oooo00<0
c03o0?ooo`3oool0o`3ooomN0?ooo`H000003`3oool01`3o0000oooo0?ooo`3oool0o`000?ooo`3<
o`000`3oool00`3<o`00oooo0?ooo`030?ooo`05003oIP3oool0oooo0?ooo`00ofH04`3oool01@00
I_l0oooo0?ooo`3oool006Ko01@0oooo00@0<`3o0?ooo`3oool0<`3o503oool00`2?0?l0oooo08l0
o`0A0?ooo`80c03oSP3oool100000000I`3oool00`000000oooo0?ooo`0m0?ooo`030<`0o`3oool0
oooo00L0oooo00<0S`3o0?ooo`3oool00`3oool3003oIPD0c?l000@00?mV0?ooo`3<o`00o`001@00
000a0?ooo`0608l0o`3oool0oooo0<co003oool0o`000P3oool00`0c0?l006Ko0?ooo`0;0?ooo`03
0<`0o`3oool0oooo0?l0ooooGP3oool500000100oooo00H0o`000?ooo`3oool0oooo0?l0003<o`04
0?ooo`030<co003oool0oooo0080oooo00D00?mV0?ooo`3oool0oooo003oIP0C0?ooo`05001Vo`3o
ool0oooo0?ooo`00I_l0303oool503<0o`<0oooo00@0<`3o0?ooo`3oool0<`3o503oool00`2?0?l0
oooo08l0o`0A0?ooo`80c03oS`3oool100000000I`3oool00`000000oooo0?ooo`0d0?ooo`D0c03o
1@3oool00`3<0?l0oooo0?ooo`060?ooo`0308l0o`3oool0oooo00@0oooo00@0<`3o0<co003<o`00
c?l01@3o00000`3oool0c?l00?l0000400000340oooo00H0S`3o0?ooo`3oool0oooo0<co003o0003
0?ooo`03001Vo`3oool0oooo00/0oooo00<0c03o0?ooo`3oool0o`3ooomL0?ooo`H000003`3oool0
1P3o0000oooo0?ooo`3oool0o`000<co00@0oooo00<0c?l00?ooo`3oool00P3oool00`00ofH0oooo
0?ooo`020?ooo`03003oIP3oool0oooo0100oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo
0?ooo`0:0?ooo`D0<`3o0P3oool0100c0?l0oooo0?ooo`0c0?lD0?ooo`0308l0o`3oool0S`3o0180
oooo00<0c03o0?ooo`3oool0SP3oool100000000I`3oool00`000000oooo0?ooo`0d0?ooo`D0c03o
1P3oool00`3<0?l0oooo0?ooo`060?ooo`0308l0o`3oool0oooo00<0oooo00@0<`3o0?ooo`3<o`00
c?l01@3o00000`3oool0c?l00?l0000500000340oooo00D0S`3o0?ooo`3oool0c?l00?l000030?oo
o`0303<0o`00I_l0oooo00/0oooo00<0c03o0?ooo`3oool0o`3ooomL0?ooo`D000003`3oool01`3o
0000oooo0?ooo`3oool0o`000?ooo`3<o`000`3oool00`3<o`00oooo0?ooo`030?ooo`05003oIP3o
ool0oooo0?ooo`00ofH04`3oool01@00I_l0oooo0?ooo`3oool006Ko00`0oooo1P0c0?l01@3oool0
<`3o0?ooo`3oool0<`3o01@0oooo00<0S`3o0?ooo`2?0?l04P3oool00`3<0?l0oooo0?ooo`2?0?oo
o`400000001W0?ooo`030000003oool0oooo03<0oooo1P3<0?l60?ooo`030<`0o`3oool0oooo00L0
oooo00<0S`3o0?ooo`3oool00`3oool00`0c0?l0c?l00?l000050?l00004003oIP3oool0c?l00?l0
00@00000<@3oool01P2?0?l0oooo0?ooo`3<o`00oooo0?l00080oooo00<0<`3o001Vo`3oool0303o
ool00`3<0?l0oooo0?ooo`3o0?oooeX0oooo1P00000>0?ooo`030?l0003oool0oooo0080oooo00<0
o`000<co003oool00`3oool00`3<o`00oooo0?ooo`020?ooo`05003oIP3oool0oooo0?ooo`00ofH0
4`3oool01@00I_l0oooo0?ooo`3oool006Ko00h0oooo1P0c0?l20?ooo`0303<0o`3oool0oooo0180
oooo00<0S`3o0?ooo`2?0?l04P3oool00`3<0?l0oooo0?ooo`2@0?ooo`400000001W0?ooo`030000
003oool0oooo03@0oooo1@3<0?l70?ooo`030<`0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool0
0`3oool00`0c0?l0oooo0?ooo`050?l00004003oIP3oool0c?l00?l000D00000<03oool01P2?0?l0
oooo0?ooo`3<o`000?mV0?l000<0oooo00<006Ko0?ooo`3oool02`3oool00`3<0?l0oooo0?ooo`3o
0?oooeX0oooo1@00000?0?ooo`060?l0003oool0oooo0?ooo`3o0000c?l0103oool00`3<o`00oooo
0?ooo`020?ooo`05003oIP3oool0oooo0?ooo`00ofH04`3oool01@00I_l0oooo0?ooo`3oool006Ko
00l0oooo1@0c0?l20?ooo`0303<0o`3oool0oooo0180oooo00<0S`3o0?ooo`2?0?l04P3oool00`3<
0?l0oooo0?ooo`2A0?ooo`400000001W0?ooo`030000003oool0oooo03@0oooo1@3<0?l80?ooo`03
0<`0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?l000040?l00004
0?ooo`00ofH0c?l00?l000D00000<@3oool01@2?0?l0oooo0?ooo`3<o`00o`0000<0oooo00<006Ko
0?ooo`3oool02`3oool00`3<0?l0oooo0?ooo`3o0?oooeT0oooo1P00000>0?ooo`030?l0003<o`00
c?l00080c?l000<0o`000<co003oool00P3oool00`3<o`00oooo0?ooo`030?ooo`05003oIP3oool0
oooo0?ooo`00ofH04P3oool01@00I_l0oooo0?ooo`3oool006Ko0180oooo0P0c0?l30?ooo`0303<0
o`3oool0oooo00T0oooo1@2?0?l30?ooo`0308l0o`3oool0S`3o0180oooo00<0c03o0?ooo`3oool0
TP3oool100000000I`3oool00`000000oooo0?ooo`0f0?ooo`030<`0o`3oool0oooo00T0oooo00<0
c03o0?ooo`3oool01P3oool00`2?0?l0oooo0?ooo`030?ooo`0503<0o`3oool0oooo0?l00000I_l0
0P3oool01000ofH0oooo0<co003o000400000340oooo00D0S`3o0?ooo`3oool0c?l00?l000030?oo
o`0303<0o`00I_l0oooo00`0oooo00<0c03o0?ooo`3oool0o`3ooomH0?ooo`D000003P3oool00`3o
0000c?l00<co00020<co00030?l0003<o`00oooo00<0oooo00<0c?l00?ooo`3oool00P3oool01@00
ofH0oooo0?ooo`3oool00?mV0180oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0@
0?ooo`0503<0o`3oool0oooo0?ooo`0c0?l0303oool508l0o`80oooo00@0S`3o0?ooo`3oool0S`3o
4@3oool00`3<0?l0oooo0?ooo`2C0?ooo`400000001W0?ooo`@00000@P3oool00`3<0?l0oooo0?oo
o`060?ooo`0308l0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool00P3oool01P00I_l0oooo0?oo
o`00ofH0c?l00?l000D00000<@3oool01@2?0?l0oooo0<co0000ofH0o`0000<0oooo00<006Ko0?oo
o`3oool02`3oool00`3<0?l0oooo0?ooo`3o0?oooeL0oooo1P00000>0?ooo`060?l0003<o`00c?l0
0<co003o0000c?l0103oool00`3<o`00oooo0?ooo`020?ooo`05003oIP3oool0oooo0?ooo`00ofH0
4`3oool01@00I_l0oooo0?ooo`3oool006Ko0180oooo00D0<`3o0?ooo`3oool0oooo03<0o`0<0?oo
o`H0S`3o00D0oooo08l0o`3oool0oooo08l0o`0A0?ooo`030<`0o`3oool0oooo0940oooo10000000
06L0oooo00<000000?ooo`3oool0A03oool00`3<0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo
00<0oooo00D0<`3o0?ooo`3oool0oooo001Vo`020?ooo`04003oIP3oool0c?l00?l000@00000<@3o
ool01@2?0?l0oooo0?ooo`3<o`00o`0000<0oooo00<006Ko0?ooo`3oool0303oool00`3<0?l0oooo
0?ooo`3o0?oooeH0oooo1@00000>0?ooo`030?l0003oool0c?l00080c?l000<0o`000<co003oool0
0P3oool00`3<o`00oooo0?ooo`020?ooo`03003oIP3oool0oooo0080oooo00<00?mV0?ooo`3oool0
403oool01@00I_l0oooo0?ooo`3oool006Ko0180oooo00D0<`3o0?ooo`3oool0oooo03<0o`0>0?oo
o`H0S`3o0P3oool00`2?0?l0oooo0?ooo`0?0?ooo`030<`0o`3oool0oooo09D0oooo0@00000006L0
oooo00<000000?ooo`3oool0A@3oool00`3<0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo00<0
oooo00D0<`3o0?ooo`3oool0oooo001Vo`020?ooo`03003oIP3<o`00o`0000@00000<P3oool0102?
0?l0oooo0<co003o00040?ooo`03001Vo`3oool0oooo00/0oooo00<0c03o0?ooo`3oool0o`3ooomE
0?ooo`H000003@3oool00`3o0000oooo0?ooo`020<co00030?l0003<o`00c?l000<0oooo00<0c?l0
0?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV00X0oooo1@00I_l30?ooo`05001Vo`3o
ool0oooo0?ooo`00I_l04P3oool01@0c0?l0oooo0?ooo`3oool0<`3o00l0oooo1@2?0?l20?ooo`03
08l0o`3oool0oooo00l0oooo00<0c03o0?ooo`3oool0UP3oool100000000I`3oool00`000000oooo
0?ooo`160?ooo`030<`0o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool00P3oool01@0c0?l0oooo
0?ooo`3oool006Ko0080oooo00@00?mV0?ooo`3<o`00o`001000000a0?ooo`0508l0o`3oool0c?l0
0?ooo`3o00000`3oool00`00I_l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo0?l0ooooE@3oool5
000000h0oooo00H0o`000?ooo`3oool0oooo0?l0003<o`040?ooo`030<co003oool0oooo0080oooo
00D00?mV0?ooo`3oool0oooo003oIP0;0?ooo`D006Ko0P3oool01@00I_l0oooo0?ooo`3oool006Ko
01<0oooo00@0<`3o0?ooo`3oool0<`3o4P3oool208l0o`80oooo00<0S`3o0?ooo`3oool03`3oool2
0<`0oiP0oooo0@00000006L0oooo00<000000?ooo`3oool0AP3oool00`3<0?l0oooo0?ooo`060?oo
o`0308l0o`3oool0oooo0080oooo00D0<`3o0?ooo`3oool0oooo001Vo`020?ooo`03003oIP3<o`00
o`0000@00000<P3oool0102?0?l0c?l0003oIP3o00030?ooo`03001Vo`3oool0oooo00`0oooo00<0
c03o0?ooo`3oool0o`3ooomC0?ooo`H000003@3oool00`3o0000oooo0?ooo`020?ooo`030?l0003<
o`00oooo0080oooo00<0c?l00?ooo`3oool00P3oool00`00ofH0oooo0?ooo`020?ooo`03003oIP3o
ool0oooo00P0oooo1P00I_l00`3oool006Ko0?ooo`030?ooo`03001Vo`3oool0oooo0100oooo00@0
<`3o0?ooo`3oool0<`3o4`3oool0102?0?l0oooo0?ooo`2?0?lA0?ooo`80c03oV@3oool100000000
I`3oool00`000000oooo0?ooo`170?ooo`030<`0o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool0
0`3oool0100c0?l0oooo0?ooo`00I_l20?ooo`04003oIP3oool0c?l00?l000@00000<@3oool0102?
0?l0oooo0<co003o00040?ooo`03001Vo`3oool0oooo00/0oooo00<0c03o0?ooo`3oool0o`3ooomC
0?ooo`D000003@3oool00`3o0000oooo0?ooo`020?ooo`030?l0003<o`00oooo00<0oooo00<0c?l0
0?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV00`0oooo1@00I_l00`3oool006Ko0?oo
o`020?ooo`03001Vo`3oool0oooo0100oooo00D0<`3o0?ooo`3oool0oooo03<0o`0B0?ooo`0408l0
o`3oool0oooo08l0oa40oooo0P3<0?nJ0?ooo`400000001W0?ooo`030000003oool0oooo04P0oooo
00<0c03o0?ooo`3oool01@3oool00`2?0?l0oooo0?ooo`020?ooo`0503<0o`3oool0oooo0?ooo`00
I_l00P3oool00`00ofH0c?l00?l0000400000380oooo00<0S`3o0<co003o0000103oool00`00I_l0
oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo0?l0ooooD@3oool6000000d0oooo00H0o`000?ooo`3o
ool0oooo0?l0003<o`040?ooo`030<co003oool0oooo0080oooo00D00?mV0?ooo`3oool0oooo003o
IP0=0?ooo`H006Ko0`3oool00`00I_l0oooo0?ooo`0@0?ooo`0503<0o`3oool0oooo0?ooo`0c0?l0
4P3oool0102?0?l0oooo0?ooo`2?0?l@0?ooo`<0c03oV`3oool100000000I`3oool00`000000oooo
0?ooo`190?ooo`030<`0o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool00P3oool02@0c0?l0oooo
0?ooo`3oool006Ko0?ooo`00ofH0c?l00?l0000400000380oooo00@0S`3o0<co0000ofH0o`000`3o
ool00`00I_l0oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo0?l0ooooD@3oool5000000d0oooo00<0
o`000?ooo`3oool00P3oool00`3o0000c?l00?ooo`020?ooo`030<co003oool0oooo0080oooo00D0
0?mV0?ooo`3oool0oooo003oIP0@0?ooo`03001Vo`3oool006Ko00<0oooo00<006Ko0?ooo`3oool0
403oool01@0c0?l0oooo0?ooo`3oool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o403oool00`3<
0?l0oooo0<`0o`2L0?ooo`400000001W0?ooo`030000003oool0oooo04X0oooo00<0c03o0?ooo`3o
ool01@3oool00`2?0?l0oooo0?ooo`020?ooo`0403<0o`3oool0oooo001Vo`80oooo00<00?mV0<co
003o00001000000b0?ooo`0308l0o`3<o`00o`0000@0oooo00<006Ko0?ooo`3oool0303oool00`3<
0?l0oooo0?ooo`3o0?ooodl0oooo1P00000<0?ooo`030?l0003oool0oooo0080oooo00<0o`000<co
003oool00P3oool00`3<o`00oooo0?ooo`030?ooo`05003oIP3oool0oooo0?ooo`00ofH04@3oool0
1@00I_l0oooo0?ooo`3oool006Ko0180oooo00D0<`3o0?ooo`3oool0oooo03<0o`0B0?ooo`0408l0
o`3oool0oooo08l0oa00oooo00<0c03o0?ooo`3<0?l0W@3oool100000000I`3oool00`000000oooo
0?ooo`1;0?ooo`030<`0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool00P3oool02@0c0?l0oooo
0?ooo`3oool006Ko0?ooo`00ofH0c?l00?l0000400000380oooo00<0S`3o0<co003o0000103oool0
0`00I_l0oooo0?ooo`0<0?ooo`030<`0o`3oool0oooo0?l0ooooC`3oool5000000d0oooo00H0o`00
0?ooo`3oool0oooo0?l0003<o`040?ooo`030<co003oool0oooo0080oooo00D00?mV0?ooo`3oool0
oooo003oIP0A0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool03`3oool01@0c0?l0
oooo0?ooo`3oool0<`3o0180oooo00@0S`3o0?ooo`3oool0S`3o403oool00`3<0?l0oooo0<`0o`2N
0?ooo`400000001W0?ooo`030000003oool0oooo04`0oooo00<0c03o0?ooo`3oool0103oool00`2?
0?l0oooo0?ooo`020?ooo`0403<0o`3oool0oooo001Vo`80oooo00<00?mV0<co003o00000`00000b
0?ooo`0308l0o`3<o`00o`0000@0oooo00<006Ko0?ooo`3oool0303oool00`3<0?l0oooo0?ooo`3o
0?ooodh0oooo1P0000070?ooo`H0o`00103oool00`3o0000c?l00?ooo`020?ooo`030<co003oool0
oooo0080oooo00D00?mV0?ooo`3oool0oooo003oIP0A0?ooo`03001Vo`3oool0oooo0080oooo00<0
06Ko0?ooo`3oool03`3oool01@0c0?l0oooo0?ooo`3oool0<`3o0180oooo00D0S`3o0?ooo`3oool0
oooo08l0o`0?0?ooo`030<`0o`3oool0c03o09l0oooo0@00000006L0oooo00<000000?ooo`3oool0
C@3oool00`3<0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo0080oooo0`0c0?l01@00I_l0<`3o
003oIP3<o`00o`0000@00000<P3oool00`3<o`000?mV0?l000040?ooo`03001Vo`3oool0oooo00`0
oooo00<0c03o0?ooo`3oool0o`3ooom=0?ooo`D00000203oool50?l000@0oooo00<0o`000<co003o
ool00P3oool00`3<o`00oooo0?ooo`030?ooo`05003oIP3oool0oooo0?ooo`00ofH04@3oool01@00
I_l0oooo0?ooo`3oool006Ko0140oooo00D0<`3o0?ooo`3oool0oooo03<0o`0B0?ooo`0508l0o`3o
ool0oooo0?ooo`2?0?l03`3oool00`3<0?l0oooo0<`0o`2P0?ooo`40000000140?ooo`<000001P3o
ool200000080oooo2000000>0?ooo`030000003oool0oooo04d0oooo00<0c03o0?ooo`3oool01@3o
ool0102?0?l0oooo0?ooo`3oool303<0o`06001Vo`0c0?l0<`3o003oIP3<o`00o`000`00000b0?oo
o`0308l0o`3<o`00o`0000@0oooo00<006Ko0?ooo`3oool0303oool00`3<0?l0oooo0?ooo`3o0?oo
od`0oooo1P0000070?ooo`H0o`000`3oool00`3o0000c?l00?ooo`030?ooo`030<co003oool0oooo
0080oooo00D00?mV0?ooo`3oool0oooo003oIP0A0?ooo`05001Vo`3oool0oooo0?ooo`00I_l04@3o
ool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00l0oooo00D0S`3o0?ooo`3oool0oooo
08l0o`0?0?ooo`040<`0o`3oool0oooo0<`0oj00oooo0@00000004<0oooo00D000000?ooo`3oool0
oooo000000050?ooo`8000000`3oool8000000d0oooo00<000000?ooo`3oool0CP3oool00`3<0?l0
oooo0?ooo`050?ooo`0308l0o`3oool0oooo00@0<`3o00D006Ko03<0o`00ofH0c?l00?l000040000
0380oooo00<0c?l00?l0003oool00`3oool00`00I_l0oooo0?ooo`0=0?ooo`030<`0o`3oool0oooo
0?l0ooooB`3oool5000000T0oooo1@3o00030?ooo`030?l0003<o`00oooo0080oooo00<0c?l00?oo
o`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV0140oooo00<006Ko0?ooo`3oool00P3oool0
0`00I_l0oooo0?ooo`0>0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool03`3oool0
1@2?0?l0oooo0?ooo`3oool0S`3o00l0oooo00@0c03o0?ooo`3oool0c03oX@3oool100000000@P3o
ool00`000000oooo0?ooo`020?ooo`8000002P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0C`3oool00`3<0?l0oooo0?ooo`040?ooo`0408l0o`3o
ool0oooo0?ooo`@0<`3o00@006Ko003oIP3oool0o`001000000b0?ooo`030<co0000ofH0o`0000<0
oooo00<006Ko03<0o`3oool03@3oool00`3<0?l0oooo0?ooo`3o0?ooodX0oooo1P0000090?ooo`D0
o`000P3oool00`3o0000c?l00?ooo`020?ooo`030<co003oool0oooo00<0oooo00D00?mV0?ooo`3o
ool0oooo003oIP0@0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool03P3oool00`0c
0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00l0oooo00D0S`3o0?ooo`3oool0oooo08l0o`0?
0?ooo`040<`0o`3oool0oooo0<`0oj80oooo0@0000000480oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0D03oool0
0`3<0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo00@0<`3o00D006Ko0?ooo`00ofH0c?l00?l0
0003000003<0oooo00<0c?l00?l0003oool00`3oool00`00I_l0oooo0?ooo`0=0?ooo`030<`0o`3o
ool0oooo0?l0ooooB@3oool5000000/0oooo0P3o00030?ooo`030?l0003<o`00oooo00<0oooo00<0
c?l00?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV0100oooo00<006Ko0?ooo`3oool0
0P3oool00`00I_l0oooo0?ooo`0>0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0
3`3oool01@2?0?l0oooo0?ooo`3oool0S`3o00l0oooo00@0c03o0?ooo`3oool0c03oX`3oool10000
0000@@3oool2000000D0oooo0P00000:0?ooo`030000003oool0oooo0100oooo1P00001>0?ooo`03
0<`0o`3oool0oooo00@0oooo00D0S`3o0?ooo`3oool0oooo03<0o`020?ooo`04001Vo`00ofH00000
0?l000@00000<P3oool00`3<o`00o`000?ooo`030?ooo`03001Vo`0c0?l0oooo00d0oooo00<0c03o
0?ooo`3oool0o`3ooom80?ooo`H000002`3oool00`3o0000oooo0?ooo`020?ooo`030?l0003oool0
oooo0080oooo00<0c?l00?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV0140oooo00D0
06Ko0?ooo`3oool0oooo001Vo`0@0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0
3`3oool01@2?0?l0oooo0?ooo`3oool0S`3o00l0oooo00@0c03o0?ooo`3oool0c03oW`3oool60000
0000@@3oool2000000D0oooo0P00000;0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0DP3oool00`3<0?l0oooo0?ooo`040?ooo`0;08l0o`3oool0oooo0?ooo`0c0?l0oooo001Vo`3o
ool00?mV0<co003o00000`00000b0?ooo`030<co003o0000oooo00<0oooo00<006Ko03<0o`3oool0
3P3oool00`3<0?l0oooo0?ooo`3o0?ooodL0oooo1P00000:0?ooo`030?l0003oool0oooo0080oooo
00<0o`000<co003oool00P3oool00`3<o`000?mV003oIP03003oIP@0oooo00<00?mV0?ooo`3oool0
3P3oool01@00I_l0oooo0?ooo`3oool006Ko0140oooo00D0<`3o0?ooo`3oool0oooo03<0o`0A0?oo
o`0508l0o`3oool0oooo0?ooo`2?0?l03`3oool0103<0?l0oooo0?ooo`3<0?nU0?ooo`4000000011
0?ooo`8000001@3oool2000000`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1C
0?ooo`030<`0o`3oool0oooo00@0oooo00@0S`3o0?ooo`3oool0<`3o0P3oool01000I_l00?mV0<co
003o000400000340oooo00<0c?l0003oIP3o0000103oool00`00I_l0oooo0?ooo`0=0?ooo`030<`0
o`3oool0oooo0?l0ooooAP3oool6000000X0oooo00<0o`000?ooo`3oool00P3oool00`3o0000c?l0
0?ooo`030?ooo`030<co0000ofH00?mV00<00?mV0`3oool00`00ofH0oooo0?ooo`0>0?ooo`03001V
o`3oool0oooo0080oooo00<006Ko0?ooo`3oool03P3oool01@0c0?l0oooo0?ooo`3oool0<`3o0140
oooo00D0S`3o0?ooo`3oool0oooo08l0o`0?0?ooo`040<`0o`3oool0oooo0<`0ojH0oooo0@000000
0440oooo0P0000050?ooo`8000003@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
05@0oooo00<0c03o0?ooo`3oool00`3oool02`2?0?l0oooo0?ooo`3oool0<`3o0?ooo`00I_l0oooo
003oIP3<o`00o`0000<00000<P3oool00`3<o`00o`000?ooo`030?ooo`03001Vo`0c0?l0oooo00h0
oooo00<0c03o0?ooo`3oool0o`3ooom50?ooo`H000002P3oool00`3o0000oooo0?ooo`020?ooo`03
0?l0003oool0oooo0080oooo00<0c?l0003oIP00ofH00`00ofH30?ooo`03003oIP3oool0oooo00h0
oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0>0?ooo`0503<0o`3oool0oooo0?oo
o`0c0?l04@3oool01@2?0?l0oooo0?ooo`3oool0S`3o00l0oooo00@0c03o0?ooo`3oool0c03oY`3o
ool100000000@@3oool2000000D0oooo0P00000=0?ooo`8000003P3oool00`000000oooo0?ooo`1E
0?ooo`030<`0o`3oool0oooo00<0oooo00X0S`3o0?ooo`3oool0oooo03<0o`3oool006Ko003oIP3<
o`00o`000`00000b0?ooo`030<co003o0000oooo00<0oooo00<006Ko03<0o`3oool03P3oool00`3<
0?l0oooo0?ooo`3o0?oood@0oooo1P00000:0?ooo`030?l0003oool0oooo0080oooo00<0o`000<co
003oool00P3oool00`3<o`00oooo003oIP04003oIP<0oooo00<00?mV0?ooo`3oool03@3oool00`00
I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00h0oooo00<0<`3o0?ooo`3oool00P3oool00`0c
0?l0oooo0?ooo`0=0?ooo`80S`3o0`3oool00`2?0?l0oooo0?ooo`0=0?ooo`040<`0o`3oool0oooo
0<`0ojP0oooo0@0000000440oooo0P0000050?ooo`8000003@3oool2000000h0oooo00<000000?oo
o`3oool0EP3oool00`3<0?l0oooo0?ooo`030?ooo`0508l0o`3oool0oooo03<0o`3oool01@3o0003
00000340oooo00<0c?l00?l0002?0?l0103oool00`00I_l0oooo0?ooo`0>0?ooo`030<`0o`3oool0
oooo0?l0oooo@`3oool6000000X0oooo00H0o`000?ooo`3oool0oooo0?l0003<o`040?ooo`030<co
003oool00?mV00@00?mV0P3oool00`00ofH0oooo0?ooo`0=0?ooo`03001Vo`3oool0oooo0080oooo
00<006Ko0?ooo`3oool03P3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00d0oooo
00<0S`3o0?ooo`3oool00P3oool00`2?0?l0oooo0?ooo`0=0?ooo`050<`0o`3oool0oooo0?ooo`3<
0?l0Z03oool100000000@@3oool2000000D0oooo0P0000070?ooo`030000003oool0oooo00<0oooo
0P00000>0?ooo`030000003oool0oooo05L0oooo00<0c03o0?ooo`3oool00`3oool0102?0?l0oooo
0?ooo`0c0?l50?l000<00000<P3oool00`3<o`00o`000?ooo`030?ooo`03001Vo`0c0?l0oooo00h0
oooo00<0c03o0?ooo`3oool0o`3ooom20?ooo`H000002P3oool00`3o0000oooo0?ooo`020?ooo`03
0?l0003oool0oooo0080oooo00<0c?l00?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV
0100oooo00D006Ko0?ooo`3oool0oooo001Vo`0@0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o
0?ooo`3oool03@3oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00`0oooo00D0c03o
0?ooo`3oool0oooo0<`0o`2Y0?ooo`40000000120?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`030?ooo`8000003P3oool00`000000oooo0?ooo`1H
0?ooo`030<`0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3o00001@3o000300000380oooo00<0c?l0
0?l0003oool00`3oool00`00I_l0<`3o0?ooo`0?0?ooo`030<`0o`3oool0oooo0?l0oooo@@3oool6
000000T0oooo00<0o`000?ooo`3oool00P3oool00`3o0000c?l00?ooo`020?ooo`030<co003oool0
oooo0080oooo00D00?mV0?ooo`3oool0oooo003oIP0@0?ooo`03001Vo`3oool0oooo0080oooo00<0
06Ko0?ooo`3oool03@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00d0oooo00<0
S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0<0?ooo`050<`0o`3oool0oooo0?ooo`3<0?l0
ZP3oool100000000@P3oool2000000<0oooo0P0000090?ooo`040000003oool0oooo0?ooo`<00000
3P3oool00`000000oooo0?ooo`1I0?ooo`030<`0o`3oool0oooo0080oooo00<0S`3o0?ooo`3oool0
1P3o000300000340oooo00<0c?l00?l0002?0?l0103oool00`00I_l0oooo0?ooo`0>0?ooo`030<`0
o`3oool0oooo0?l0oooo@@3oool5000000X0oooo00H0o`000?ooo`3oool0oooo0?l0003<o`030?oo
o`030<co003oool0oooo00<0oooo00D00?mV0?ooo`3oool0oooo003oIP0?0?ooo`03001Vo`3oool0
oooo0080oooo00<006Ko0?ooo`3oool03@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0
oooo00d0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0<0?ooo`050<`0o`3oool0
oooo0?ooo`3<0?l0Z`3oool100000000@`3oool5000000/0oooo1000000@0?ooo`030000003oool0
oooo05X0oooo00<0c03o0?ooo`3oool00P3oool00`2?0?l0oooo0?l000050?l000030<co00000000
00000340oooo00<0c?l0003oIP3o0000103oool00`00I_l0<`3o0?ooo`0?0?ooo`030<`0o`3oool0
oooo0?l0oooo?`3oool6000000T0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`02
0?ooo`030<co003oool0oooo0080oooo00D00?mV0?ooo`3oool0oooo003oIP0?0?ooo`03001Vo`3o
ool0oooo0080oooo00<006Ko0?ooo`3oool03@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3o
ool0oooo00d0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0<0?ooo`050<`0o`3o
ool0oooo0?ooo`3<0?l0[03oool100000000I`3oool00`000000oooo0?ooo`1K0?ooo`030<`0o`3o
ool0oooo0080oooo00@0S`3o0?ooo`3oool0o`000`3<o`00103o0000000000000000000a0?ooo`03
0<co003o0000S`3o00<0oooo00<006Ko03<0o`3oool03`3oool00`3<0?l0oooo0?ooo`3o0?ooocl0
oooo1@00000:0?ooo`060?l0003oool0oooo0?ooo`3o0000c?l00`3oool00`3<o`00oooo0?ooo`02
0?ooo`05003oIP3oool0oooo0?ooo`00ofH03`3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3o
ool0oooo00d0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`0=0?ooo`0308l0o`3o
ool0oooo00<0oooo00<0S`3o0?ooo`3oool02`3oool20<`0o`<0oooo00<0c03o0?ooo`3oool0Z`3o
ool100000000I`3oool00`000000oooo0?ooo`1L0?ooo`030<`0o`3oool0oooo0080oooo00<0S`3o
0<co003<o`000`3<o`00103o0000000000000000000a0?ooo`030<co003o0000S`3o00@0oooo00<0
06Ko0?ooo`3oool03`3oool00`3<0?l0oooo0?ooo`3o0?ooocd0oooo1P0000090?ooo`030?l0003<
o`00c?l00080c?l000D0o`000?ooo`3oool0oooo0<co00050?ooo`05003oIP3oool0oooo0?ooo`00
ofH03`3oool01@00I_l0oooo0?ooo`3oool006Ko00l0oooo00<0<`3o0?ooo`3oool00P3oool00`0c
0?l0oooo0?ooo`0=0?ooo`0308l0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool02`3oool00`3<
0?l0oooo0?ooo`020?ooo`030<`0o`3oool0oooo0:`0oooo0@00000006L0oooo00<000000?ooo`3o
ool0G@3oool00`3<0?l0oooo0?ooo`020?ooo`0308l0o`3<o`00c?l000<0c?l000<0o`0000000000
0000<@3oool0103<o`00o`000?ooo`2?0?l30?ooo`03001Vo`0c0?l0oooo00l0oooo00<0c03o0?oo
o`3oool0o`3ooolm0?ooo`D000002@3oool00`3o0000c?l00<co00020<co00030?l0003oool0oooo
0080oooo00<0c?l00?ooo`3oool00P3oool01@00ofH0oooo0?ooo`3oool00?mV00l0oooo00<006Ko
0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0=0?ooo`0503<0o`3oool0oooo0?ooo`0c0?l03`3o
ool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00/0oooo00<0c03o0?ooo`3oool00P3o
ool00`3<0?l0oooo0?ooo`2]0?ooo`400000001W0?ooo`030000003oool0oooo05h0oooo00D0c03o
0?ooo`3oool0oooo08l0o`050<co00040?l000000000000000000340oooo00<0c?l00?l0002?0?l0
0`3oool00`00I_l0<`3o0?ooo`0@0?ooo`030<`0o`3oool0oooo0?l0oooo>`3oool6000000T0oooo
00H0o`000<co003<o`00c?l00?l0003<o`030?ooo`030<co003oool0oooo0080oooo00D00?mV0?oo
o`3oool0oooo003oIP0?0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool03@3oool0
0`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00`0oooo00<0S`3o0?ooo`3oool00`3oool0
0`2?0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool0[P3oool1
00000000I`3oool00`000000oooo0?ooo`1O0?ooo`070<`0o`3oool0oooo0?ooo`2?0?l0oooo0<co
0002003oIP040?l00000ofH00?mV00000340oooo00@0c?l00?l0003oool0S`3o0P3oool00`00I_l0
oooo03<0o`0@0?ooo`030<`0o`3oool0oooo0?l0oooo>`3oool5000000T0oooo00<0o`000?ooo`3<
o`000P3<o`001@3o0000c?l00?ooo`3oool0c?l000D0oooo00D00?mV0?ooo`3oool0oooo003oIP0>
0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool03@3oool00`0c0?l0oooo0?ooo`02
0?ooo`0303<0o`3oool0oooo00`0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0;
0?ooo`030<`0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool0[`3oool100000000I`3oool00`00
0000oooo0?ooo`1P0?ooo`060<`0o`3oool0oooo0?ooo`2?0?l0oooo0P00ofH0103<o`00o`00003o
IP00000a0?ooo`040<co003o0000oooo08l0o`<0oooo00<006Ko03<0o`3oool0403oool00`3<0?l0
oooo0?ooo`3o0?ooocT0oooo1P0000090?ooo`050?l0003oool0c?l00<co003o00000P3<o`020?oo
o`030<co003oool0oooo0080oooo00D00?mV0?ooo`3oool0oooo003oIP0>0?ooo`03001Vo`3oool0
oooo0080oooo00<006Ko0?ooo`3oool03@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0
oooo00`0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0
oooo0080oooo00<0c03o0?ooo`3oool0/03oool100000000I`3oool00`000000oooo0?ooo`1Q0?oo
o`040<`0o`3oool0oooo0?ooo`@00?mV00@0o`00003oIP00000000001@2?0?l[0?ooo`050<co003o
0000oooo0?ooo`2?0?l00P3oool00`00I_l0<`3o0?ooo`0@0?ooo`030<`0o`3oool0oooo0?l0oooo
>@3oool5000000T0oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3oool0c?l000@0
oooo00D00?mV0?ooo`3oool0oooo003oIP0?0?ooo`05001Vo`3oool0oooo0?ooo`00I_l03`3oool0
0`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00`0oooo00<0S`3o0?ooo`3oool00`3oool0
0`2?0?l0oooo0?ooo`0;0?ooo`030<`0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool0/@3oool1
00000000I`3oool400000640oooo00@0c03o0?ooo`3oool0oooo0`00ofH0103<o`00o`0000000000
000508l0ob`0oooo00@0c?l00?l0003oool0S`3o0P3oool00`00I_l0oooo03<0o`0A0?ooo`030<`0
o`3oool0oooo0?l0oooo=`3oool6000000P0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo
0?ooo`020?ooo`050<co003oool0oooo0?ooo`00ofH0103oool00`00ofH0oooo0?ooo`0<0?ooo`03
001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool0303oool00`0c0?l0oooo0?ooo`020?ooo`03
03<0o`3oool0oooo00`0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`0;0?ooo`03
0<`0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool0[`3oool400000000I`3oool00`000000oooo
0?ooo`1S0?ooo`030<`0o`3oool0oooo00@00?mV00<0o`000000002?0?l01@2?0?l/0?ooo`050<co
003o0000oooo0?ooo`2?0?l00P3oool00`00I_l0<`3o0?ooo`0@0?ooo`030<`0o`3oool0oooo0?l0
oooo=`3oool5000000T0oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`3oool0c?l0
00@0oooo00D00?mV0?ooo`3oool0oooo003oIP0>0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko
0?ooo`3oool0303oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00`0oooo00<0S`3o
0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`0:0?ooo`030<`0o`3oool0oooo0080oooo00<0c03o
0?ooo`3oool0/`3oool100000000I`3oool00`000000oooo0?ooo`1T0?ooo`090<`0o`3oool0oooo
08l0o`00ofH006Ko0<co003o0000000000D0S`3o;03oool01@3<o`00o`000?ooo`3oool0S`3o0080
oooo00<006Ko03<0o`3oool04@3oool00`3<0?l0oooo0?ooo`3o0?ooocD0oooo1P0000040?ooo`D0
o`00103oool01@3o0000oooo0?ooo`3oool0c?l000@0oooo00D00?mV0?ooo`3oool0oooo003oIP0>
0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool0303oool00`0c0?l0oooo0?ooo`02
0?ooo`0303<0o`3oool0oooo00`0oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`09
0?ooo`80c03o103oool00`3<0?l0oooo0?ooo`2d0?ooo`400000001W0?ooo`030000003oool0oooo
06D0oooo00P0c03o0?ooo`3oool0S`3o03<0o`00ofH0o`00000000D006Ko;@3oool0103<o`00o`00
0?ooo`2?0?l20?ooo`03001Vo`3oool0<`3o0140oooo00<0c03o0?ooo`3oool0o`3ooole0?ooo`D0
00001@3oool50?l000<0oooo00<0o`000?ooo`3oool00P3oool01@3<o`00oooo0?ooo`3oool00?mV
00@0oooo00<00?mV0?ooo`3oool02`3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo
00`0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`0<0?ooo`0308l0o`3oool0oooo
00@0oooo00<0S`3o0?ooo`3oool02@3oool00`3<0?l0oooo0?ooo`030?ooo`030<`0o`3oool0oooo
0;D0oooo0@00000006L0oooo00<000000?ooo`3oool0IP3oool01`3<0?l0oooo0?ooo`2?0?l006Ko
0<co003o00001@00I_l]0?ooo`050<co003o0000oooo0?ooo`2?0?l00P3oool00`00I_l0<`3o0?oo
o`0A0?ooo`030<`0o`3oool0oooo0?l0oooo<`3oool6000000@0oooo1P3o00030?ooo`050?l0003o
ool0oooo0?ooo`3<o`00103oool01@00ofH0oooo0?ooo`3oool00?mV00h0oooo00D006Ko0?ooo`3o
ool0oooo001Vo`0>0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0303oool00`2?
0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo00T0oooo00<0c03o0?ooo`3oool00`3oool00`3<
0?l0oooo0?ooo`2f0?ooo`400000001W0?ooo`030000003oool0oooo06L0oooo00H0c03o0?ooo`3o
ool0S`3o0<co003o0005001Vobd0oooo00D0c?l00?l0003oool0oooo08l0o`020?ooo`03001Vo`0c
0?l0oooo0140oooo00<0c03o0?ooo`3oool0o`3ooolc0?ooo`D000001P3oool50?l00080oooo00D0
o`000?ooo`3oool0oooo0<co00040?ooo`05003oIP3oool0oooo0?ooo`00ofH03P3oool00`00I_l0
oooo0?ooo`020?ooo`03001Vo`3oool0oooo00/0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0
oooo0?ooo`0;0?ooo`0308l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool02@3oool00`3<0?l0
oooo0?ooo`030?ooo`030<`0o`3oool0oooo0;L0oooo0@00000006L0oooo00<000000?ooo`3oool0
J03oool01@3<0?l0oooo08l0o`0c0?l0o`0000D006Ko;@3oool00`3<o`00o`00003oIP020?ooo`05
08l0o`3oool006Ko0?ooo`0c0?l04P3oool00`3<0?l0oooo0?ooo`3o0?oooc40oooo1P0000060?oo
o`D0o`000P3oool01@3o0000oooo0?ooo`3oool0c?l000<0oooo00D00?mV0?ooo`3oool0oooo003o
IP0>0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool0303oool00`0c0?l0oooo0?oo
o`020?ooo`0303<0o`3oool0oooo00/0oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo0?oo
o`090?ooo`030<`0o`3oool0oooo00<0oooo00<0c03o0?ooo`3oool0^03oool100000000I`3oool0
0`000000oooo0?ooo`1Y0?ooo`050<`0o`000000S`3o0<co003o00001000I_l^0?ooo`050<co003o
0000oooo0?ooo`2?0?l00P3oool00`00I_l0<`3o0?ooo`0A0?ooo`030<`0o`3oool0oooo0?l0oooo
<@3oool5000000P0oooo0P3o00030?ooo`050?l0003oool0oooo0?ooo`3<o`00103oool01@00ofH0
oooo0?ooo`3oool00?mV00d0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0<0?oo
o`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool02`3oool00`2?0?l0oooo0?ooo`040?oo
o`0308l0o`3oool0oooo00T0oooo00<0c03o0?ooo`3oool00`3oool00`3<0?l0oooo0?ooo`2i0?oo
o`400000001W0?ooo`030000003oool0oooo06X0oooo00H0c03o0000002?0?l0o`0000000000I_l`
0?ooo`030<co003o0000oooo0080oooo00D0S`3o0?ooo`00I_l0oooo03<0o`0B0?ooo`030<`0o`3o
ool0oooo0?l0oooo;`3oool6000000P0oooo00D0o`000?ooo`3oool0oooo0?l000040?ooo`050<co
003oool0oooo0?ooo`00ofH00`3oool00`00ofH0oooo0?ooo`0;0?ooo`03001Vo`3oool0oooo0080
oooo00<006Ko0?ooo`3oool0303oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00/0
oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`090?ooo`030<`0o`3oool0oooo00<0
oooo00<0c03o0?ooo`3oool0^P3oool100000000I`3oool00`000000oooo0?ooo`1[0?ooo`040<`0
o`000000c?l00?l000800000;`3oool00`3<o`00o`00003oIP020?ooo`0508l0o`3oool006Ko0?oo
o`0c0?l04P3oool00`3<0?l0oooo0?ooo`3o0?ooobl0oooo1@0000080?ooo`030?l0003oool0oooo
0080oooo00D0o`000?ooo`3oool0oooo0<co00030?ooo`05003oIP3oool0oooo0?ooo`00ofH03P3o
ool01@00I_l0oooo0?ooo`3oool006Ko00h0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo
0?ooo`0;0?ooo`0308l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool02@3oool00`3<0?l0oooo
0?ooo`030?ooo`030<`0o`3oool0oooo0;/0oooo0@00000006L0oooo00<000000?ooo`3oool0K03o
ool00`3<0?l000000?l0000200000300oooo00<0c?l00?l0003oool00P3oool0102?0?l006Ko0?oo
o`0c0?lC0?ooo`030<`0o`3oool0oooo0?l0oooo;@3oool6000000L0oooo00<0o`000?ooo`3oool0
0P3oool01@3o0000oooo0?ooo`3oool0c?l000@0oooo00D00?mV0?ooo`3oool0oooo003oIP0=0?oo
o`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool02`3oool00`0c0?l0oooo0?ooo`020?oo
o`0303<0o`3oool0oooo00/0oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`090?oo
o`030<`0o`3oool0oooo00<0oooo00<0c03o0?ooo`3oool0_03oool100000000I`3oool00`000000
oooo0?ooo`1/0?ooo`050000003<0?l0c?l00?l000000000<03oool00`3<o`00o`000?ooo`020?oo
o`0508l0o`3oool006Ko0?ooo`0c0?l04P3oool00`3<0?l0oooo0?ooo`3o0?ooobd0oooo1@000008
0?ooo`060?l0003oool0oooo0?ooo`3<o`00o`000`3oool01@3<o`00oooo0?ooo`3oool00?mV00<0
oooo00<00?mV0?ooo`3oool02`3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00/0
oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`0;0?ooo`0308l0o`3oool0oooo00@0
oooo00<0S`3o0?ooo`3oool0203oool20<`0o`D0oooo00<0c03o0?ooo`3oool0_@3oool100000000
I`3oool00`000000oooo0?ooo`1/0?ooo`80000000@0c03o0?l0000c0?l00000;`3oool00`3<o`00
o`000?ooo`030?ooo`0408l0o`00I_l0oooo03<0oa<0oooo00<0c03o0?ooo`3oool0o`3oool[0?oo
o`H000001`3oool00`3o0000oooo0?ooo`020?ooo`050?l0003oool0oooo0?ooo`3<o`000`3oool0
1@00ofH0oooo0?ooo`3oool00?mV00d0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?oo
o`0;0?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool02`3oool00`2?0?l0oooo0?oo
o`040?ooo`0308l0o`3oool0oooo00P0oooo00<0c03o0?ooo`3oool0103oool00`3<0?l0oooo0?oo
o`2n0?ooo`400000001W0?ooo`030000003oool0oooo06d0oooo0P0000000`3o000006Ko0000000_
0?ooo`030<co003o00000?mV00<0oooo00@0S`3o001Vo`3oool0<`3o503oool00`3<0?l0oooo0?oo
o`3o0?ooobX0oooo1@0000070?ooo`030?l0003oool0oooo0080oooo00D0o`000?ooo`3oool0oooo
0<co00040?ooo`05003oIP3oool0oooo0?ooo`00ofH0303oool00`00I_l0oooo0?ooo`020?ooo`03
001Vo`3oool0oooo00/0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`0;0?ooo`03
08l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool0203oool00`3<0?l0oooo0?ooo`040?ooo`03
0<`0o`3oool0oooo0;l0oooo0@00000006L0oooo00<000000?ooo`3oool0K@3oool2000000030<co
003o0000<`3o00d0oooo1@3<0?lN0?ooo`030<co003o0000oooo00<0oooo00@0S`3o001Vo`3oool0
<`3o4`3oool00`3<0?l0oooo0?ooo`3o0?ooobT0oooo1P0000070?ooo`030?l0003oool0oooo0080
oooo00D0o`000?ooo`3oool0oooo0<co00030?ooo`05003oIP3oool0oooo0?ooo`00ofH03@3oool0
1@00I_l0oooo0?ooo`3oool006Ko00d0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?oo
o`0:0?ooo`0308l0o`3oool0oooo00<0oooo0P2?0?l20?ooo`D0c03o0`3oool00`3<0?l0oooo0?oo
o`040?ooo`030<`0o`3oool0oooo0<00oooo0@00000006L0oooo00<000000?ooo`3oool0K@3oool3
000000D0o`002P3oool50<`0oah0oooo00<0c?l00?l0003oool00`3oool0102?0?l006Ko0?ooo`0c
0?lD0?ooo`030<`0o`3oool0oooo0?l0oooo:03oool5000000L0oooo00<0o`000?ooo`3oool00P3o
ool01@3o0000oooo0?ooo`3oool0c?l000<0oooo00D00?mV0?ooo`3oool0oooo003oIP0=0?ooo`03
001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool02P3oool00`0c0?l0oooo0?ooo`030?ooo`03
03<0o`3oool0oooo00X0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`020?ooo`D0
c03o0P3oool00`3<0?l0oooo0?ooo`040?ooo`030<`0o`3oool0oooo0<40oooo0@00000006L0oooo
1000001]0?ooo`8000001@3o00090?ooo`H0c03o7P3oool00`3<o`00o`00003oIP030?ooo`0408l0
o`00I_l0oooo03<0oa@0oooo00<0c03o0?ooo`3oool0o`3ooolW0?ooo`H000001`3oool01P3o0000
oooo0?ooo`3oool0c?l00?l000<0oooo00D0c?l00?ooo`3oool0oooo003oIP030?ooo`03003oIP3o
ool0oooo00X0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`040?ooo`D0<`3o00<0
oooo03<0o`3oool0103oool00`0c0?l0oooo0?ooo`0:0?ooo`0308l0o`3oool0oooo00<0oooo00<0
S`3o0?ooo`3oool00P3oool60<`0o`030?ooo`3<0?l0oooo00D0oooo00<0c03o0?ooo`3oool0_`3o
ool400000000I`3oool00`000000oooo0?ooo`1^0?ooo`030000003o0000o`0000@0o`002P3oool5
0<`0oal0oooo00<0o`00003oIP3oool00`3oool0102?0?l006Ko0?ooo`0c0?lD0?ooo`030<`0o`3o
ool0oooo0?l0oooo9P3oool5000000L0oooo00<0o`000?ooo`3oool00P3oool01@3o0000oooo0?oo
o`3oool0c?l000<0oooo00D00?mV0?ooo`3oool0oooo003oIP060?ooo`D006Ko00<0oooo001Vo`3o
ool00`3oool00`00I_l0oooo0?ooo`050?ooo`H0<`3o1@3oool00`0c0?l0oooo0?ooo`090?ooo`80
S`3o1@3oool00`2?0?l0oooo0?ooo`040?ooo`H0c03o1P3oool00`3<0?l0oooo0?ooo`330?ooo`40
0000001W0?ooo`030000003oool0oooo06l0oooo00<000000?l0003o00000`3o00000`2?0?l0oooo
0?ooo`070?ooo`D0c03o7`3oool00`3<o`00o`000?ooo`030?ooo`0408l0o`00I_l0oooo03<0oa@0
oooo00<0c03o0?ooo`3oool0o`3ooolU0?ooo`H000001P3oool00`3o0000oooo0?ooo`020?ooo`05
0<co003o0000oooo0?ooo`3<o`000`3oool01@00ofH0oooo0?ooo`3oool00?mV00L0oooo1P00I_l4
0?ooo`03001Vo`3oool0oooo00D0oooo1P0c0?l50?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o
0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`050?ooo`D0c03o1P3oool00`3<0?l0oooo0?ooo`34
0?ooo`400000001W0?ooo`030000003oool0oooo06l0oooo00<000000?l0003o00000`3o00000`0c
0?l0c03o0?ooo`090?ooo`030<`0o`3oool0oooo01l0oooo00<0c?l00?l00000ofH0103oool00`00
I_l0oooo03<0o`0E0?ooo`030<`0o`3oool0oooo0?l0oooo903oool5000000L0oooo00H0o`000?oo
o`3oool0oooo0<co003o00030?ooo`050<co003oool0oooo0?ooo`00ofH00`3oool00`00ofH0oooo
0?ooo`040?ooo`L006Ko0`3oool00`00I_l0oooo0?ooo`070?ooo`D0<`3o103oool00`0c0?l0oooo
0?ooo`090?ooo`0308l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool0203oool20<`0o`H0oooo
00<0c03o0?ooo`3oool0a@3oool100000000I`3oool00`000000oooo0?ooo`1_0?ooo`<000000P3o
00001000I_l0oooo03<0o`3<0?l[0?ooo`030<co003o00000?mV00@0oooo00@0S`3o001Vo`3oool0
<`3o503oool00`3<0?l0oooo0?ooo`3o0?ooob<0oooo1P0000060?ooo`030?l0003oool0oooo0080
oooo00D0o`000?ooo`3oool0oooo0<co00030?ooo`05003oIP3oool0oooo0?ooo`00ofH0203oool5
001Vo`@0oooo00<006Ko0?ooo`3oool01`3oool503<0o`<0oooo00<0<`3o0?ooo`3oool02@3oool0
0`2?0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo00P0oooo00<0c03o0?ooo`3oool01@3oool0
0`3<0?l0oooo0?ooo`360?ooo`400000001W0?ooo`030000003oool0oooo0700oooo0`0000001P3o
00000000001Vo`0c0?l0S`3o0<`0ob/0oooo00<0c?l00?l0003oool0103oool00`00I_l0oooo03<0
o`0E0?ooo`030<`0o`3oool0oooo0?l0oooo8P3oool5000000L0oooo00H0o`000?ooo`3oool0oooo
0<co003o00030?ooo`040<co003oool0oooo003oIP<0oooo00<00?mV0?ooo`3oool01`3oool5001V
o`<0oooo00<006Ko0?ooo`3oool02P3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo
00T0oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo0?ooo`080?ooo`030<`0o`3oool0oooo
00D0oooo00<0c03o0?ooo`3oool0a`3oool100000000I`3oool00`000000oooo0?ooo`1`0?ooo`<0
000000L0o`00003oIP00I_l0oooo03<0o`2?0?l0c03o02X0oooo00<0c?l00?l00000ofH0103oool0
1000I_l0oooo0?ooo`0c0?lD0?ooo`030<`0o`3oool0oooo0?l0oooo8@3oool6000000H0oooo00<0
o`000?ooo`3oool00P3oool01@3o0000oooo0?ooo`00ofH0c?l000@00?mV0`3oool00`00ofH0oooo
0?ooo`090?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool02P3oool00`0c0?l0oooo
0?ooo`020?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3oool0103oool00`2?0?l0oooo
0?ooo`080?ooo`030<`0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool0b03oool100000000I`3o
ool00`000000oooo0?ooo`1`0?ooo`@0000000L0o`0000000000I_l0oooo03<0o`2?0?l0c03o02T0
oooo00<0c?l00?l00000ofH01@3oool00`00I_l0oooo03<0o`0E0?ooo`030<`0o`3oool0oooo0?l0
oooo803oool5000000H0oooo00<0o`000?ooo`3oool00P3oool01@3<o`00o`000?ooo`3oool0c?l0
00@00?mV0`3oool00`00ofH0oooo0?ooo`090?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?oo
o`3oool02P3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o0?oo
o`3oool0103oool00`2?0?l0oooo0?ooo`080?ooo`030<`0o`3oool0oooo00D0oooo00<0c03o0?oo
o`3oool0b@3oool100000000I`3oool00`000000oooo0?ooo`1a0?ooo`<0000000<0o`00003oIP00
I_l00P3oool00`0c0?l0S`3o0<`0o`0Y0?ooo`030?l00000ofH0oooo00@0oooo00<006Ko0?ooo`0c
0?l05@3oool00`3<0?l0oooo0?ooo`3o0?oooal0oooo1P0000060?ooo`060?l0003oool0oooo0?oo
o`3<o`00o`000P3oool00`00ofH0c?l0003oIP03003oIP80oooo00<00?mV0?ooo`3oool02P3oool0
1@00I_l0oooo0?ooo`3oool006Ko00`0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?oo
o`090?ooo`0308l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool01`3oool20<`0o`L0oooo00<0
c03o0?ooo`3oool0bP3oool100000000I`3oool00`000000oooo0?ooo`1a0?ooo`@0000000<0o`00
0<co003<o`000`3<o`000`2?0?l0c03o0?ooo`0W0?ooo`030<co003o00000?mV00@0oooo00@006Ko
08l0o`3oool0<`3o5@3oool00`3<0?l0oooo0?ooo`3o0?oooah0oooo1@0000060?ooo`030?l0003<
o`00c?l00080c?l000D0o`000?ooo`3oool0oooo0<co0005003oIP80oooo00<00?mV0?ooo`3oool0
2@3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00T0oooo00<0<`3o0?ooo`3oool0
0P3oool00`0c0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool0
1`3oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo0</0oooo0@00000006L0oooo00<0
00000?ooo`3oool0LP3oool3000000030?l0003<o`00c?l000<0c?l000<0oooo08l0o`3<0?l09`3o
ool00`3<o`00o`00003oIP040?ooo`04001Vo`2?0?l0oooo03<0oaD0oooo00<0c03o0?ooo`3oool0
o`3ooolM0?ooo`H000001P3oool00`3o0000c?l00<co00020<co00050?l0003oool0oooo0<co003o
ool01@00ofH00`3oool00?mV0?ooo`0:0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3o
ool02@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o0?ooo`3o
ool0103oool00`2?0?l0oooo0?ooo`070?ooo`030<`0o`3oool0oooo00H0oooo00<0c03o0?ooo`3o
ool0c03oool100000000I`3oool00`000000oooo0?ooo`1b0?ooo`<0000000<0c?l00?l0003<o`00
0`3<o`00100c0?l0oooo08l0o`3<0?lV0?ooo`030<co003o00000?mV00D0oooo00<006Ko08l0o`0c
0?l05P3oool00`3<0?l0oooo0?ooo`3o0?oooa`0oooo1@0000060?ooo`030?l0003<o`00c?l00080
c?l000D0o`000?ooo`3oool0oooo0<co00020?ooo`800?mV0P3oool00`00ofH0oooo0?ooo`090?oo
o`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool02@3oool00`0c0?l0oooo0?ooo`030?oo
o`0403<0o`3oool0oooo0?ooo`D0S`3o0P3oool00`2?0?l0oooo0?ooo`040?ooo`0308l0o`3oool0
oooo00L0oooo00<0c03o0?ooo`3oool01P3oool00`3<0?l0oooo0?ooo`3=0?ooo`400000001W0?oo
o`030000003oool0oooo0780oooo100000000`3o0000c?l00<co00020<co00040?ooo`0c0?l0oooo
08l0o`80c03o9@3oool00`3o00000?mV0?ooo`040?ooo`04001Vo`2?0?l0oooo03<0oaD0oooo00<0
c03o0?ooo`3oool0o`3ooolK0?ooo`H000000P3oool50?l000@0c?l000@0o`000?ooo`3oool0c?l0
0`3oool01@00ofH0oooo0?ooo`3oool00?mV00X0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0
oooo0?ooo`090?ooo`0303<0o`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool00P3oool508l0o`03
0?ooo`2?0?l0oooo00D0oooo00<0S`3o0?ooo`3oool01`3oool00`3<0?l0oooo0?ooo`050?ooo`80
c03od03oool100000000I`3oool00`000000oooo0?ooo`1c0?ooo`<0000000<0c?l00?l0003<o`00
0P3<o`020?ooo`0503<0o`3oool0S`3o0?ooo`3<0?l0903oool00`3<o`00o`00003oIP040?ooo`04
001Vo`3oool0S`3o03<0oaH0oooo00<0c03o0?ooo`3oool0o`3ooolJ0?ooo`D000000`3oool50?l0
00<0c?l000D0o`000<co003oool0oooo0<co00020?ooo`05003oIP3oool0oooo0?ooo`00ofH02`3o
ool01@00I_l0oooo0?ooo`3oool006Ko00/0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo
0?ooo`020?ooo`L0S`3o1@3oool208l0o`P0oooo0P3<0?l70?ooo`030<`0o`3oool0oooo0=00oooo
0@00000006L0oooo00<000000?ooo`3oool0L`3oool4000000040?l0003<o`00oooo001Vo`80oooo
00<0<`3o0?ooo`2?0?l00P3oool00`3<0?l0oooo0?ooo`0Q0?ooo`030<co003o00000?mV00D0oooo
00@006Ko08l0o`3oool0<`3o5@3oool00`3<0?l0oooo0?ooo`3o0?oooaT0oooo1P0000020?ooo`H0
o`000P3oool01@3o0000oooo0?ooo`3oool0c?l00080oooo00D00?mV0?ooo`3oool0oooo003oIP0;
0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`03
0?ooo`0303<0o`3oool0oooo00@0oooo1@2?0?l50?ooo`0308l0o`3oool0oooo00L0oooo00<0c03o
0?ooo`3oool01P3oool00`3<0?l0oooo0?ooo`3A0?ooo`400000001W0?ooo`030000003oool0oooo
07<0oooo1@000000103o0000oooo0?ooo`00I_l20?ooo`0303<0o`3oool0S`3o0080oooo00<0c03o
0?ooo`3oool08@3oool00`3o00000?mV0?ooo`040?ooo`04001Vo`3oool0S`3o03<0oaH0oooo00<0
c03o0?ooo`3oool0o`3ooolH0?ooo`D00000103oool50?l000030?ooo`3<o`00o`000080oooo00@0
c?l00?ooo`3oool00?mV103oool00`00ofH0oooo0?ooo`080?ooo`03001Vo`3oool0oooo0080oooo
00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo00D0oooo
1@2?0?l40?ooo`0308l0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool01P3oool00`3<0?l0oooo
0?ooo`3B0?ooo`400000001W0?ooo`@00000L`3oool4000000040?l00000ofH0oooo001Vo`<0oooo
00<0<`3o0?ooo`2?0?l00P3oool00`3<0?l0oooo0?ooo`0P0?ooo`030<co003o00000?mV00@0oooo
00D006Ko0?ooo`3oool0S`3o03<0o`0E0?ooo`030<`0o`3oool0oooo0?l0oooo5`3oool6000000@0
oooo1@3o00000`3<o`00o`000?ooo`020?ooo`040<co003oool0oooo003oIP<0oooo00<00?mV0?oo
o`3oool0203oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00P0oooo00<0<`3o0?oo
o`3oool00`3oool00`0c0?l0oooo0?ooo`080?ooo`0308l0o`3oool0oooo00<0oooo00<0S`3o0?oo
o`3oool01`3oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo0=00oooo1000000006L0
oooo00<000000?ooo`3oool0M03oool4000000050?l0003<o`00oooo0?ooo`00I_l00`3oool00`0c
0?l0oooo08l0o`020?ooo`030<`0o`3oool0oooo01l0oooo00<0c?l00?l00000ofH01@3oool01000
I_l0oooo08l0o`0c0?lF0?ooo`030<`0o`3oool0oooo0?l0oooo5P3oool6000000@0oooo00<0o`00
0?ooo`3o00000P3oool01@3<o`00o`000?ooo`3oool0c?l00080oooo00D00?mV0?ooo`3oool0oooo
003oIP0:0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo
0?ooo`030?ooo`0303<0o`3oool0oooo00P0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo
0?ooo`070?ooo`030<`0o`3oool0oooo00H0oooo00<0c03o0?ooo`3oool0e03oool100000000I`3o
ool00`000000oooo0?ooo`1e0?ooo`@0000000@0o`00003oIP3oool006Ko0`3oool0100c0?l0oooo
0?ooo`2?0?l20?ooo`030<`0o`3oool0oooo01h0oooo00<0c?l00?l00000ofH01@3oool01@00I_l0
oooo0?ooo`2?0?l0<`3o01D0oooo00<0c03o0?ooo`3oool0o`3ooolE0?ooo`H000001@3oool01P3o
0000oooo0?ooo`3oool0c?l00?l00080oooo00@0c?l00?ooo`3oool00?mV103oool00`00ofH0oooo
0?ooo`080?ooo`05001Vo`3oool0oooo0?ooo`00I_l02P3oool00`0c0?l0oooo0?ooo`030?ooo`03
03<0o`3oool0oooo00P0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`070?ooo`03
0<`0o`3oool0oooo00H0oooo00<0c03o0?ooo`3oool0e@3oool100000000I`3oool00`000000oooo
0?ooo`1e0?ooo`@0000000D0o`00003oIP3oool0oooo001Vo`030?ooo`0403<0o`3oool0oooo08l0
o`80oooo00<0c03o0?ooo`3oool07P3oool00`3o0000oooo003oIP050?ooo`04001Vo`3oool0S`3o
03<0oaH0oooo00<0c03o0?ooo`3oool0o`3ooolD0?ooo`H00000103oool01`3o0000oooo0?ooo`3o
ool0c?l00?ooo`3o00000P3oool0103<o`00oooo0?ooo`00ofH30?ooo`03003oIP3oool0oooo00P0
oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00<0
oooo00<0<`3o0?ooo`3oool0203oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00H0
oooo0P3<0?l70?ooo`80c03of03oool100000000I`3oool00`000000oooo0?ooo`1e0?ooo`D00000
00D0o`00003oIP3oool0oooo001Vo`030?ooo`0403<0o`3oool0oooo08l0o`80oooo00<0c03o0?oo
o`3oool07@3oool00`3<o`00o`00003oIP050?ooo`05001Vo`3oool0oooo08l0o`0c0?l05@3oool0
0`3<0?l0oooo0?ooo`3o0?oooa@0oooo1@0000050?ooo`060?l0003oool0oooo0?ooo`3<o`00o`00
0P3oool0103<o`00oooo0?ooo`00ofH30?ooo`03003oIP3oool0oooo00P0oooo00<006Ko0?ooo`3o
ool00P3oool00`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00<0oooo00<0<`3o0?ooo`3o
ool0203oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00H0oooo00<0c03o0?ooo`3o
ool01P3oool00`3<0?l0oooo0?ooo`3H0?ooo`400000001W0?ooo`030000003oool0oooo07H0oooo
100000001@3o00000?mV0?ooo`3oool006Ko00@0oooo00@0<`3o0?ooo`3oool0S`3o0P3oool20<`0
o`030?ooo`0c0?l0<`3o00<0<`3o5`3oool00`3<o`00o`00003oIP060?ooo`04001Vo`3oool0oooo
03<0oaH0oooo00<0c03o0?ooo`3oool0o`3ooolB0?ooo`H00000103oool01P3o0000oooo0?ooo`3o
ool0c?l00?l000<0oooo00<0c?l00?ooo`00ofH0103oool00`00ofH0oooo0?ooo`070?ooo`03001V
o`3oool0oooo0080oooo00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`020?ooo`0303<0
o`3oool0oooo00P0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`060?ooo`030<`0
o`3oool0oooo00H0oooo00<0c03o0?ooo`3oool0f@3oool100000000I`3oool00`000000oooo0?oo
o`1f0?ooo`D0000000@0o`00003oIP3oool0oooo1@00ofH30?ooo`0508l0o`3oool0oooo0?ooo`3<
0?l01@0c0?lH0?ooo`030?l0003oool00?mV00D0oooo00D006Ko0?ooo`3oool0S`3o03<0o`0E0?oo
o`030<`0o`3oool0oooo0?l0oooo4P3oool5000000@0oooo00<0o`000?ooo`3oool00P3oool01@3<
o`00o`000?ooo`3oool0c?l00080oooo00D00?mV0?ooo`3oool0oooo003oIP090?ooo`03001Vo`3o
ool0oooo0080oooo00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3o
ool0oooo00P0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`060?ooo`030<`0o`3o
ool0oooo00H0oooo00<0c03o0?ooo`3oool0fP3oool100000000I`3oool00`000000oooo0?ooo`1f
0?ooo`D0000000@0o`00003oIP3oool0oooo1@00ofH01@0c0?l0oooo0?ooo`3oool0S`3o0080oooo
1P0c0?lH0?ooo`030?l0003oool00?mV00H0oooo00@006Ko0?ooo`3oool0<`3o5P3oool00`3<0?l0
oooo0?ooo`3o0?oooa00oooo1P0000040?ooo`060?l0003oool0oooo0?ooo`3<o`00o`000P3oool0
103<o`00oooo0?ooo`00ofH30?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool00P3o
ool00`00I_l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool01`3o
ool208l0o`D0oooo00<0S`3o0?ooo`3oool01P3oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3o
ool0oooo0=/0oooo0@00000006L0oooo00<000000?ooo`3oool0M`3oool5000000030?l00000ofH0
0?mV00D00?mV00<0oooo03<0o`3oool00P3oool00`2?0?l0oooo0?ooo`0503<0oaP0oooo00<0c?l0
0?l00000ofH01P3oool01@00I_l0oooo0?ooo`3oool0<`3o01D0oooo00<0c03o0?ooo`3oool0o`3o
ool@0?ooo`D00000103oool01`3o0000oooo0?ooo`3oool0c?l00?ooo`3o00000P3oool00`3<o`00
oooo003oIP040?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0
oooo0?ooo`070?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool01`3oool00`2?0?l0
oooo0?ooo`040?ooo`0308l0o`3oool0oooo00D0oooo0P3<0?l80?ooo`030<`0o`3oool0oooo0=`0
oooo0@00000006L0oooo00<000000?ooo`3oool0M`3oool5000000030?l00000ofH0oooo00D00?mV
0P3oool01P0c0?l0oooo0?ooo`3oool0S`3o0?ooo`D0<`3o603oool00`3<o`00o`00003oIP060?oo
o`05001Vo`3oool0oooo0?ooo`0c0?l05P3oool00`3<0?l0oooo0?ooo`3o0?ooo`h0oooo1P000004
0?ooo`060?l0003oool0oooo0?ooo`3<o`00o`000P3oool0103<o`00oooo0?ooo`00ofH30?ooo`03
003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`070?ooo`03
03<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool01`3oool00`2?0?l0oooo0?ooo`040?ooo`03
08l0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool01`3oool00`3<0?l0oooo0?ooo`3M0?ooo`40
0000001W0?ooo`030000003oool0oooo07P0oooo100000000`3o0000c?l0003oIP05003oIP<0oooo
00@0<`3o0?ooo`3oool0S`3o0`3oool00`0c0?l0c03o0?ooo`0I0?ooo`030?l0003oool00?mV00H0
oooo00D006Ko0?ooo`3oool0oooo03<0o`0F0?ooo`030<`0o`3oool0oooo0?l0oooo3@3oool50000
00@0oooo00H0o`000?ooo`3oool0oooo0<co003o00020?ooo`040<co003oool0oooo003oIP<0oooo
00<00?mV0?ooo`3oool01`3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00L0oooo
00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`070?ooo`0308l0o`3oool0oooo00<0oooo
0P2?0?l70?ooo`030<`0o`3oool0oooo00H0oooo0P3<0?oP0?ooo`400000001W0?ooo`030000003o
ool0oooo07P0oooo1@0000000`3o0000oooo0?l000040?l000<0oooo00D0<`3o0?ooo`3oool0oooo
08l0o`040?ooo`80c03o603oool00`3<o`00o`00003oIP060?ooo`06001Vo`3oool0oooo0?ooo`0c
0?l0S`3o5@3oool00`3<0?l0oooo0?ooo`3o0?ooo``0oooo1P0000040?ooo`060?l0003oool0oooo
0<co003oool0o`000P3oool00`3<o`00oooo003oIP040?ooo`03003oIP3oool0oooo00H0oooo00<0
06Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo0080oooo00<0
<`3o0?ooo`3oool01`3oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00H0oooo00<0
c03o0?ooo`3oool01P3oool00`3<0?l0oooo0?ooo`3P0?ooo`400000001W0?ooo`030000003oool0
oooo07P0oooo1@0000000`3o0000c?l00?l000040?l000@0oooo00D0<`3o0?ooo`3oool0oooo08l0
o`050?ooo`030<`0o`3oool0oooo01D0oooo00<0c?l00?l00000ofH01`3oool01@00I_l0oooo0?oo
o`3oool0<`3o01H0oooo00<0c03o0?ooo`3oool0o`3oool;0?ooo`D00000103oool01P3o0000oooo
0?ooo`3oool0c?l00?l00080oooo00@0c?l00?ooo`3oool00?mV0`3oool00`00ofH0oooo0?ooo`06
0?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`02
0?ooo`0303<0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`06
0?ooo`030<`0o`3oool0oooo00H0oooo00<0c03o0?ooo`3oool0h@3oool100000000I`3oool00`00
0000oooo0?ooo`1i0?ooo`D000001P3o00050?ooo`0503<0o`3oool0oooo0?ooo`2?0?l01@3oool0
0`3<0?l0oooo0?ooo`0E0?ooo`030?l0003oool00?mV00H0oooo00H006Ko0?ooo`3oool0oooo03<0
o`2?0?lF0?ooo`030<`0o`3oool0oooo0?l0oooo2@3oool6000000<0oooo00L0o`000?ooo`3oool0
oooo0<co003oool0o`000080oooo00<0c?l00?ooo`00ofH00`3oool00`00ofH0oooo0?ooo`070?oo
o`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`030?oo
o`0303<0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`050?oo
o`80c03o203oool00`3<0?l0oooo0?ooo`3R0?ooo`400000001W0?ooo`030000003oool0oooo07T0
oooo1@0000060?l00003001Vo`3oool0oooo00<0oooo00D0<`3o0?ooo`3oool0oooo08l0o`050?oo
o`80c03o5@3oool00`3o0000oooo003oIP070?ooo`05001Vo`3oool0oooo0?ooo`0c0?l05P3oool0
0`3<0?l0oooo0?ooo`3o0?ooo`T0oooo1@0000040?ooo`060?l0003oool0oooo0?ooo`3<o`00o`00
0P3oool00`3<o`00oooo003oIP040?ooo`03003oIP3oool0oooo00H0oooo00<006Ko0?ooo`3oool0
0P3oool00`00I_l0oooo0?ooo`060?ooo`0303<0o`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool0
1P3oool00`2?0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool0
1`3oool00`3<0?l0oooo0?ooo`3S0?ooo`400000001W0?ooo`030000003oool0oooo07T0oooo1P00
00050?l00003001Vo`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool00P3oool00`2?0?l0oooo0?oo
o`040?ooo`030<`0o`3oool0oooo0180oooo00<0c?l00?l00000ofH01`3oool01P00I_l0oooo0?oo
o`3oool0<`3o08l0oaH0oooo00<0c03o0?ooo`3oool0o`3oool70?ooo`H000000`3oool01P3o0000
oooo0?ooo`3oool0c?l00?l00080oooo00@0c?l00?ooo`3oool00?mV0`3oool00`00ofH0oooo0?oo
o`060?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01P3oool00`0c0?l0oooo0?oo
o`030?ooo`0303<0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?oo
o`050?ooo`030<`0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool0i03oool100000000I`3oool0
0`000000oooo0?ooo`1j0?ooo`D0000000<0o`000<co003o00000`3oool00`00I_l0oooo0?ooo`03
0?ooo`0303<0o`3oool0oooo0080oooo00<0S`3o0?ooo`3oool0103oool00`3<0?l0oooo0?ooo`0A
0?ooo`040<co003o0000oooo003oIPH0oooo00<006Ko0?ooo`3oool00P3oool00`0c0?l0S`3o0?oo
o`0E0?ooo`030<`0o`3oool0oooo0?l0oooo1P3oool5000000@0oooo00H0o`000?ooo`3oool0c?l0
0?ooo`3o00020?ooo`030<co003oool00?mV00<0oooo00<00?mV0?ooo`3oool01P3oool00`00I_l0
oooo0?ooo`020?ooo`03001Vo`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0
oooo0?ooo`060?ooo`0308l0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool01@3oool00`3<0?l0
oooo0?ooo`070?ooo`030<`0o`3oool0oooo0>D0oooo0@00000006L0oooo1000001i0?ooo`D00000
00@0oooo0?l0003oool00?mV0`3oool00`00I_l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo0080
oooo00<0S`3o0?ooo`3oool0103oool20<`0oa80oooo00<0o`000?ooo`00ofH01`3oool01P00I_l0
oooo0?ooo`3oool0<`3o08l0oaH0oooo00<0c03o0?ooo`3oool0o`3oool50?ooo`H000000`3oool0
1P3o0000oooo0?ooo`3oool0c?l00?l00080oooo00<0c?l00?ooo`00ofH0103oool00`00ofH0oooo
0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01P3oool00`0c0?l0oooo
0?ooo`030?ooo`0303<0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo
0?ooo`040?ooo`80c03o203oool20<`0onD0oooo1000000006L0oooo00<000000?ooo`3oool0N`3o
ool5000000030?l0003<o`000?mV00<0oooo00<006Ko0?ooo`3oool0103oool00`0c0?l0oooo0?oo
o`020?ooo`0308l0o`3oool0oooo00D0oooo00<0c03o0?ooo`3oool03`3oool0103<o`00o`000?oo
o`00ofH60?ooo`03001Vo`3oool0oooo0080oooo00<0<`3o08l0o`3oool05@3oool00`3<0?l0oooo
0?ooo`3o0?ooo`@0oooo1@0000030?ooo`090?l0003oool0oooo0?ooo`3<o`00oooo0?l0003oool0
c?l00080oooo00D00?mV0?ooo`3oool0oooo003oIP080?ooo`05001Vo`3oool0oooo0?ooo`00I_l0
203oool00`0c0?l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool0
0`3oool00`2?0?l0oooo0?ooo`040?ooo`030<`0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool0
j03oool100000000I`3oool00`000000oooo0?ooo`1k0?ooo`D0000000@0o`000<co003oool00?mV
0`3oool00`00I_l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool0
1@3oool00`3<0?l0oooo0?ooo`0>0?ooo`040<co003o0000oooo003oIPL0oooo00L006Ko0?ooo`3o
ool0oooo03<0o`3oool0S`3o01H0oooo00<0c03o0?ooo`3oool0o`3oool20?ooo`H000000`3oool0
1P3o0000oooo0?ooo`3oool0c?l00?l00080oooo00<0c?l00?ooo`00ofH00`3oool00`00ofH0oooo
0?ooo`060?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01@3oool00`0c0?l0oooo
0?ooo`030?ooo`0303<0o`3oool0oooo00H0oooo00<0S`3o0?ooo`3oool00P3oool208l0o`H0oooo
00<0c03o0?ooo`3oool01`3oool00`3<0?l0oooo0?ooo`3Y0?ooo`400000001W0?ooo`030000003o
ool0oooo07/0oooo1P0000000`3o0000c?l0003oIP030?ooo`03001Vo`3oool0oooo00@0oooo00<0
<`3o0?ooo`3oool00`3oool208l0o`H0oooo0P3<0?l?0?ooo`030?l0003oool00?mV00L0oooo00<0
06Ko0?ooo`3oool00P3oool00`0c0?l0S`3o0?ooo`0E0?ooo`030<`0o`3oool0oooo0?l0oooo0P3o
ool5000000<0oooo00H0o`000?ooo`3oool0oooo0<co003o00020?ooo`030<co003oool00?mV00@0
oooo00<00?mV0?ooo`3oool01@3oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00D0
oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`060?ooo`0308l0o`3oool0oooo0080
oooo00<0S`3o0?ooo`3oool0103oool20<`0o`T0oooo00<0c03o0?ooo`3oool0jP3oool100000000
I`3oool00`000000oooo0?ooo`1l0?ooo`D0000000@0o`000<co003oool00?mV0`3oool00`00I_l0
oooo0?ooo`040?ooo`0303<0o`3oool0oooo00@0oooo00<0S`3o0?ooo`3oool01@3oool00`3<0?l0
oooo0?ooo`0<0?ooo`040?l0003oool0oooo003oIPL0oooo00L006Ko0?ooo`3oool0oooo03<0o`3o
ool0S`3o01H0oooo00<0c03o0?ooo`3oool0o`3oool6000000<0oooo00H0o`000?ooo`3oool0c?l0
0?ooo`3o00020?ooo`030<co003oool00?mV00<0oooo00<00?mV0?ooo`3oool01@3oool00`00I_l0
oooo0?ooo`020?ooo`03001Vo`3oool0oooo00D0oooo00<0<`3o0?ooo`3oool00`3oool00`0c0?l0
oooo0?ooo`060?ooo`0308l0o`3oool0oooo0080oooo00<0S`3o0?ooo`3oool0103oool00`3<0?l0
oooo0?ooo`070?ooo`80c03ok@3oool100000000I`3oool00`000000oooo0?ooo`1l0?ooo`D00000
00@0oooo0?l0003<o`000?mV103oool00`00I_l0oooo0?ooo`040?ooo`0303<0o`3oool0oooo00@0
oooo00<0S`3o0?ooo`3oool01@3oool00`3<0?l0oooo0?ooo`0;0?ooo`040<co003o0000oooo003o
IPL0oooo00<006Ko0?ooo`3oool00P3oool00`0c0?l0S`3o0?ooo`0E0?ooo`030<`0o`3oool0oooo
0?l0oooo1@0000030?ooo`060?l0003oool0oooo0?ooo`3<o`00o`000P3oool00`3<o`00oooo003o
IP030?ooo`03003oIP3oool0oooo00D0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?oo
o`060?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool01@3oool208l0o`@0oooo00<0
S`3o0?ooo`3oool0103oool00`3<0?l0oooo0?ooo`070?ooo`030<`0o`3oool0oooo0>d0oooo0@00
000006L0oooo00<000000?ooo`3oool0O03oool6000000040?l0003<o`00oooo003oIP<0oooo00<0
06Ko0?ooo`3oool01@3oool00`0c0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo00D0oooo0P3<
0?l<0?ooo`040?l0003oool0oooo003oIPL0oooo00L006Ko0?ooo`3oool0oooo03<0o`3oool0S`3o
01H0oooo00<0c03o0?ooo`3oool0o@3oool5000000D0o`000P3oool01`3<o`00oooo0?l0003oool0
c?l00?ooo`00ofH0103oool00`00ofH0oooo0?ooo`040?ooo`03001Vo`3oool0oooo0080oooo00<0
06Ko0?ooo`3oool01P3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00D0oooo00<0
S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`040?ooo`030<`0o`3oool0oooo00L0oooo00<0
c03o0?ooo`3oool0kP3oool1000000005`3oool6000004X0oooo00<000000?ooo`3oool0O@3oool5
000000040?ooo`3o0000oooo003oIP@0oooo00<006Ko0?ooo`3oool01@3oool5001Vo`80oooo00<0
S`3o0?ooo`3oool01P3oool00`3<0?l0oooo0?ooo`090?ooo`040?l0003oool0oooo003oIPL0oooo
00<006Ko0?ooo`3oool00P3oool00`0c0?l0oooo08l0o`0F0?ooo`030<`0o`3oool0oooo0?`0oooo
1@0000050?l000040?ooo`3<o`00oooo0?l00080oooo00<0c?l00?ooo`00ofH00`3oool00`00ofH0
oooo0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01@3oool00`0c0?l0
oooo0?ooo`020?ooo`0303<0o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0
oooo0?ooo`030?ooo`80c03o203oool20<`0oo40oooo0@00000001T0oooo0P0000070?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`1m0?ooo`D00000
00D0oooo0?l0003<o`00oooo003oIP040?ooo`03001Vo`3oool0oooo00@0oooo1@00I_l30?ooo`03
08l0o`3oool0oooo00H0oooo00<0c03o0?ooo`3oool0203oool0103o0000c?l00?ooo`00ofH70?oo
o`03001Vo`3oool0oooo0080oooo00<0<`3o0?ooo`2?0?l05P3oool00`3<0?l0oooo0?ooo`3k0?oo
o`D000001P3o00020<co00060?l0003oool0oooo0<co003oool00?mV0`3oool00`00ofH0oooo0?oo
o`050?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool01@3oool00`0c0?l0oooo0?oo
o`020?ooo`0303<0o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?oo
o`030?ooo`030<`0o`3oool0oooo00L0oooo00<0c03o0?ooo`3oool0l@3oool1000000006@3oool2
000000H0oooo0P00000<0?ooo`800000=P3oool00`000000oooo0?ooo`1n0?ooo`D0000000@0oooo
0?l0003oool00?mV0P3oool50<co00<0oooo1P00I_l40?ooo`0308l0o`3oool0oooo00H0oooo0P3<
0?l90?ooo`040?l0003oool0oooo003oIPL0oooo00D006Ko0?ooo`3oool0oooo03<0o`020?ooo`03
08l0o`3oool0oooo01@0oooo00<0c03o0?ooo`3oool0nP3oool5000000030?ooo`3o0000o`0000<0
o`000P3<o`001@3o0000oooo0<co003oool00?mV00@0oooo00<00?mV0?ooo`3oool0103oool00`00
I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00D0oooo00<0<`3o0?ooo`3oool00P3oool00`0c
0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool00`3oool00`3<
0?l0oooo0?ooo`070?ooo`030<`0o`3oool0oooo0?80oooo0@00000001T0oooo0P0000050?ooo`80
00003P3oool2000003D0oooo00<000000?ooo`3oool0OP3oool5000000060?ooo`3o0000c?l00?oo
o`00ofH0oooo1@3<o`040?ooo`D006Ko1@3oool00`2?0?l0oooo0?ooo`070?ooo`030<`0o`3oool0
oooo00H0oooo00@0o`000?ooo`3oool00?mV1`3oool00`00I_l0oooo0?ooo`020?ooo`0403<0o`3o
ool0oooo08l0oaH0oooo00<0c03o0?ooo`3oool0n03oool6000000030?ooo`3o0000o`0000<0o`00
00L0c?l00?l0003oool0oooo0<co003oool00?mV00<0oooo00<00?mV0?ooo`3oool0103oool00`00
I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00D0oooo00<0<`3o0?ooo`3oool00P3oool00`0c
0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool00`3oool00`3<
0?l0oooo0?ooo`060?ooo`80c03om@3oool1000000006@3oool2000000D0oooo00<000000?ooo`3o
ool03P3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo07h0oooo1P000000103o0000
c?l00?ooo`00ofH60<co00@0oooo1@00I_l60?ooo`0308l0o`3oool0oooo00L0oooo00<0c03o0?oo
o`3oool00P3oool308l0o`040?l0003<o`00oooo003oIPP0oooo00D006Ko0?ooo`3oool0oooo03<0
o`020?ooo`0308l0o`3oool0oooo01@0oooo00<0c03o0?ooo`3oool0n03oool5000000<0oooo0P3o
00030<co00050?l0003oool0c?l00?ooo`00ofH00`3oool00`00ofH0oooo0?ooo`040?ooo`03001V
o`3oool0oooo0080oooo00<006Ko0?ooo`3oool01@3oool00`0c0?l0oooo0?ooo`020?ooo`0303<0
o`3oool0oooo00D0oooo00<0S`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`020?ooo`80c03o
203oool00`3<0?l0oooo0?ooo`3e0?ooo`40000000030?ooo`@000000P3oool4000000L0oooo1000
00000`3oool00000000000040?ooo`8000000P3oool400000080oooo100000040?ooo`8000004@3o
ool3000000H0oooo0P0000020?ooo`H00000403oool00`000000oooo0?ooo`1o0?ooo`D0000000D0
oooo0?l0003<o`00oooo003oIP050<co00H0oooo00<006Ko0?ooo`0c0?l01`3oool208l0o`P0oooo
0P3<0?l20?ooo`@0S`3o00@0o`000?ooo`3oool00?mV1`3oool00`00I_l0oooo0?ooo`020?ooo`04
03<0o`3oool0oooo08l0oaH0oooo00<0c03o0?ooo`3oool0mP3oool600000080oooo00<0o`000?oo
o`3<o`000P3<o`001@3o0000c?l0003oIP3<o`000?mV00@0oooo00<00?mV0?ooo`3oool0103oool0
0`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00@0oooo00<0<`3o0?ooo`3oool00P3oool0
0`0c0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo0080oooo0P2?0?l40?ooo`030<`0o`3oool0
oooo00H0oooo0P3<0?oh0?ooo`40000000050?ooo`030000003oool0oooo0080oooo0P0000070?oo
o`<000000P3oool3000000@0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`020?oo
o`8000001P3oool00`000000oooo0?ooo`0>0?ooo`050000003oool0oooo0?ooo`0000001@3oool2
00000080oooo0P0000040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0O`3oool5
000000050?ooo`3o0000c?l00?ooo`00ofH01@3<o`000`00I_l0oooo0?ooo`060?ooo`0303<0o`3o
ool0oooo00H0oooo00<0S`3o0?ooo`3oool01`3oool00`3<0?l0S`3o08l0o`0308l0o`040?l0003o
ool0oooo003oIPP0oooo00D006Ko0?ooo`3oool0oooo03<0o`030?ooo`0308l0o`3oool0oooo01@0
oooo00<0c03o0?ooo`3oool0m@3oool5000000<0oooo00D0o`000?ooo`3oool0c?l00?l00002003o
IP030<co0000ofH00?mV00<0oooo00<00?mV0?ooo`3oool0103oool00`00I_l0oooo0?ooo`020?oo
o`03001Vo`3oool0oooo00@0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`050?oo
o`0308l0o`3oool0oooo0080oooo00<0S`3o0?ooo`3oool00`3oool00`3<0?l0oooo0?ooo`060?oo
o`030<`0o`3oool0oooo0?P0oooo0@00000000H0oooo00<000000?ooo`3oool00P0000070?ooo`<0
0000103oool2000000<0oooo0P0000060?ooo`030000003oool0oooo008000001`3oool2000000h0
oooo00<000000?ooo`3oool00P3oool2000000l0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`1o0?ooo`H0000000L0oooo0?l0003<o`00oooo003oIP3oool0c?l00080oooo00<006Ko
0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`060?ooo`0308l0o`3oool0oooo00L0oooo00<0c03o
08l0o`2?0?l00P2?0?l01@3o0000c?l00?ooo`3oool00?mV00L0oooo00<006Ko0?ooo`3oool00P3o
ool0100c0?l0oooo0?ooo`2?0?lF0?ooo`030<`0o`3oool0oooo0?@0oooo1P0000020?ooo`080?l0
003oool0oooo0<co003oool0o`00003oIP3<o`03003oIP80oooo00<00?mV0?ooo`3oool0103oool0
0`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00@0oooo00<0<`3o0?ooo`3oool00P3oool0
0`0c0?l0oooo0?ooo`050?ooo`0308l0o`3oool0oooo0080oooo00<0S`3o0?ooo`3oool00P3oool2
0<`0o`P0oooo00<0c03o0?ooo`3oool0n@3oool1000000001P3oool010000000oooo000000000008
0?ooo`8000001@3oool2000000<0oooo0P0000060?ooo`040000003oool00000000000P0oooo0P00
000>0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool03@3oool2000000d0oooo00<0
00000?ooo`3oool0P03oool5000000050?ooo`3o0000c?l00?ooo`00ofH01@3oool00`00I_l0oooo
0?ooo`060?ooo`0303<0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool01`3oool508l0o`040?l0
003oool0oooo003oIPL0oooo00<006Ko0?ooo`3oool00P3oool01@0c0?l0oooo0?ooo`3oool0S`3o
01H0oooo00<0c03o0?ooo`3oool0l`3oool500000080oooo00T0o`000?ooo`3oool0oooo0<co003o
0000oooo003oIP3<o`000`00ofH00`3oool00?mV0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo
00<006Ko0?ooo`3oool0103oool00`0c0?l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo00@0oooo
0P2?0?l40?ooo`0308l0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool01P3oool20<`0oo`0oooo
0@00000000L0oooo0`0000080?ooo`8000001@3oool2000000<0oooo0P0000070?ooo`<00000203o
ool2000000d0oooo0P0000050?ooo`8000003P3oool2000000d0oooo1P00001m0?ooo`D000000P3o
ool0103o0000c?l00?ooo`00ofH50?ooo`03001Vo`3oool0oooo00H0oooo00<0<`3o0?ooo`3oool0
1`3oool00`2?0?l0oooo0?ooo`080?ooo`0408l0o`3oool0oooo0?l00080oooo00<00?mV0?ooo`3o
ool01P3oool00`00I_l0oooo0?ooo`020?ooo`0403<0o`3oool0oooo08l0oaL0oooo00<0c03o0?oo
o`3oool0l@3oool600000080oooo00P0o`000?ooo`3oool0c?l00?ooo`3o0000oooo0<co00@00?mV
00<0oooo003oIP3oool0103oool00`00I_l0oooo0?ooo`020?ooo`03001Vo`3oool0oooo00@0oooo
00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo00<0oooo
00<0S`3o0?ooo`3oool00P3oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo0?L0oooo
1P00000000L0oooo0P0000090?ooo`8000001@3oool2000000<0oooo0P0000070?ooo`8000002@3o
ool2000000d0oooo0P0000050?ooo`8000003@3oool3000000d0oooo00<000000?ooo`3oool0P@3o
ool5000000050?ooo`3o0000c?l00?ooo`00ofH01P3oool00`00I_l0oooo0?ooo`060?ooo`0303<0
o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool0203oool20<`0o`050?l0003<o`00oooo0?ooo`00
ofH01`3oool00`00I_l0oooo0?ooo`020?ooo`0503<0o`3oool0oooo0?ooo`2?0?l05`3oool00`3<
0?l0oooo0?ooo`3`0?ooo`H0000000X0oooo0?l0003oool0oooo0<co003oool0o`000?ooo`3<o`00
oooo0P00ofH20?ooo`03003oIP3oool0oooo00<0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0
oooo0?ooo`040?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0103oool00`2?0?l0
oooo0?ooo`030?ooo`0308l0o`3oool0oooo0080oooo00<0c03o0?ooo`3oool01P3oool00`3<0?l0
oooo0?ooo`3m0?ooo`40000000060?ooo`<000002@3oool2000000D0oooo0P0000030?ooo`800000
1P3oool3000000T0oooo0P00000=0?ooo`8000001@3oool2000000d0oooo0`00000=0?ooo`030000
003oool0oooo0840oooo1@0000020?ooo`040?l0003<o`00oooo003oIPD0oooo00<006Ko0?ooo`3o
ool01`3oool00`0c0?l0oooo0?ooo`070?ooo`0308l0o`3oool0oooo00T0oooo00D0c03o0?l0003o
ool0oooo003oIP080?ooo`03001Vo`3oool0oooo0080oooo00D0<`3o0?ooo`3oool0oooo08l0o`0F
0?ooo`030<`0o`3oool0oooo0>l0oooo1P0000020?ooo`050?l0003oool0oooo0<co003o00000P3o
ool00`3<o`000?mV0?ooo`020?ooo`03003oIP3oool0oooo00D006Ko103oool00`00I_l0oooo0?oo
o`030?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0103oool00`2?0?l0oooo0?oo
o`030?ooo`0408l0o`3oool0oooo0?ooo`80c03o1`3oool20<`0ool0oooo0@3oool1000000001P3o
ool2000000030?ooo`000000oooo00P0oooo0P0000040?ooo`8000000`3oool2000000H0oooo0P00
00000`3oool000000?ooo`070?ooo`8000003@3oool2000000D0oooo0P0000090?ooo`H000003P3o
ool00`000000oooo0?ooo`210?ooo`H0000000D0oooo0?l0003<o`00oooo003oIP060?ooo`03001V
o`3oool0oooo00L0oooo00<0<`3o0?ooo`3oool01`3oool00`2?0?l0oooo0?ooo`090?ooo`050?l0
003<o`00oooo0?ooo`00ofH01`3oool00`00I_l0oooo0?ooo`020?ooo`0503<0o`3oool0oooo0?oo
o`2?0?l05`3oool00`3<0?l0oooo0?ooo`3^0?ooo`H0000000/0oooo0?l0003oool0oooo0<co003o
ool0o`000?ooo`3<o`00oooo003oIP030?ooo`03003oIP3oool0oooo00D006Ko0`3oool00`00I_l0
oooo0?ooo`030?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool0103oool00`2?0?l0
oooo0?ooo`020?ooo`80S`3o0`3oool00`3<0?l0oooo0?ooo`060?ooo`030<`0o`3oool0oooo0?l0
oooo0@3oool1000000001@3oool2000000<0oooo00<000000?ooo`3oool01`3oool2000000<0oooo
0P0000030?ooo`8000001@3oool2000000<0oooo00<000000?ooo`3oool01@3oool2000000d0oooo
0P0000050?ooo`8000002`3oool200000100oooo00<000000?ooo`3oool0PP3oool500000080oooo
00@0o`000<co003oool00?mV1P3oool00`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00L0
oooo00<0S`3o0?ooo`3oool0203oool01@3o0000c?l00<`0o`3oool00?mV00P0oooo00<006Ko0?oo
o`3oool00P3oool01@0c0?l0oooo0?ooo`3oool0S`3o01L0oooo00<0c03o0?ooo`3oool0k@3oool5
000000030?ooo`3o0000oooo0080oooo00H0c?l00?l0003oool0oooo0<co0000ofH30?ooo`03003o
IP3oool0oooo00H006Ko0P3oool00`00I_l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo0080oooo
00<0<`3o0?ooo`3oool00`3oool208l0o`@0oooo00<0S`3o0?ooo`3oool00P3oool00`3<0?l0oooo
0?ooo`050?ooo`80c03oo`3oool40?ooo`40000000040?ooo`@0000000@0oooo0000000000000000
2@3oool4000000030?ooo`000000oooo00<0oooo00<000000?ooo`3oool00P3oool4000000040?oo
o`0000000000000000H0oooo00<000000?ooo`3oool0303oool2000000D0oooo0P00000<0?ooo`80
00003`3oool00`000000oooo0?ooo`220?ooo`D000000P3oool0103o0000c?l00?ooo`00ofH70?oo
o`03001Vo`3oool0oooo00L0oooo00<0<`3o0?ooo`3oool01`3oool208l0o`T0oooo00@0o`000?oo
o`3<0?l00?mV203oool00`00I_l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo0080oooo00<0S`3o
0?ooo`3oool05@3oool00`3<0?l0oooo0?ooo`3[0?ooo`H0000000H0oooo0?l0003oool0oooo0<co
003o00020?ooo`030<co0000ofH0oooo0080oooo00<00?mV0?ooo`3oool00P3oool5001Vo`030?oo
o`00I_l0oooo00@0oooo00<0<`3o0?ooo`3oool00P3oool00`0c0?l0oooo0?ooo`030?ooo`0308l0
o`3oool0oooo00<0oooo00@0S`3o0?ooo`3oool0oooo0P3<0?l70?ooo`030<`0o`3oool0oooo0?l0
oooo103oool1000000007`3oool200000100oooo0P00000>0?ooo`8000001@3oool2000000d0oooo
0P00000>0?ooo`030000003oool0oooo0880oooo1P0000001`3oool0o`000?ooo`3<o`00oooo003o
IP3oool01@3o00000`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o0?oo
o`3oool01P3oool01@3o0000c?l00?ooo`3oool00?mV00P0oooo00<006Ko0?ooo`3oool00P3oool0
1@0c0?l0oooo0?ooo`3oool0S`3o01P0oooo00<0c03o0?ooo`3oool0jP3oool50000000;0?ooo`3o
0000oooo0?ooo`3<o`00oooo0?l0003oool0c?l00?ooo`00ofH00P3oool00`00ofH0oooo0?ooo`03
0?ooo`H006Ko1P3oool01@0c0?l0oooo0?ooo`3oool0<`3o00D0oooo00<0S`3o0?ooo`3oool00`3o
ool01@2?0?l0oooo0?ooo`3oool0c03o00P0oooo00<0c03o0?ooo`3oool0o`3oool50?ooo`400000
000P0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool00`3oool2000000h0oooo00<0
00000?ooo`3oool0P`3oool500000080oooo00D0o`000<co003oool0oooo003oIP050?l000030?oo
o`00I_l0oooo00P0oooo00<0<`3o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`060?ooo`040?l0
003oool0oooo003oIP80c03o1P3oool00`00I_l0oooo0?ooo`020?ooo`0303<0o`3oool0oooo0080
oooo00<0S`3o0?ooo`3oool05@3oool00`3<0?l0oooo0?ooo`3Y0?ooo`H0000000H0oooo0?l0003o
ool0oooo0<co003o00020?ooo`030<co0000ofH0oooo0080oooo00<00?mV0?ooo`3oool00`3oool0
0`00I_l0oooo001Vo`020?ooo`03001Vo`0c0?l0<`3o00@0<`3o0`3oool00`0c0?l0oooo0?ooo`03
0?ooo`0308l0o`3oool0oooo0080oooo0P2?0?l30?ooo`030<`0o`3oool0oooo00D0oooo0P3<0?oo
0?ooo`P0oooo0@0000000200oooo0P00000>0?ooo`800000403oool2000000<0oooo0P0000090?oo
o`040000003oool0oooo0?ooo`<000003P3oool00`000000oooo0?ooo`230?ooo`D000000P3oool0
103o0000oooo0<co003oool60?l00080oooo00<006Ko0?ooo`3oool01`3oool00`0c0?l0oooo0?oo
o`090?ooo`0308l0o`3oool0oooo00D0oooo00L0o`000?ooo`3oool0oooo003oIP3oool0c03o00D0
oooo00<006Ko0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`020?ooo`0308l0o`3oool0oooo01D0
oooo00<0c03o0?ooo`3oool0j03oool50000000:0?ooo`3o0000oooo0?ooo`3<o`00oooo0?l0003o
ool0c?l0003oIP<0oooo00<00?mV0?ooo`3oool00`3oool00`00I_l0oooo0?ooo`020?ooo`03001V
o`3oool0<`3o00@0<`3o0`3oool00`0c0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo0080oooo
00@0S`3o0?ooo`3oool0oooo0P3<0?l70?ooo`030<`0o`3oool0oooo0?l0oooo203oool100000000
8@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo1@00000;0?ooo`D00000
3`3oool00`000000oooo0?ooo`230?ooo`H000000P3oool0103o0000c?l00?ooo`3oool50?l000<0
oooo00<006Ko0?ooo`3oool01`3oool00`0c0?l0oooo0?ooo`090?ooo`0308l0o`3oool0oooo00@0
oooo00D0o`000<co003oool0oooo003oIP020?ooo`80c03o103oool00`00I_l0oooo0?ooo`020?oo
o`0303<0o`3oool0oooo0080oooo00<0S`3o0<`0o`3<0?l00P3<0?lD0?ooo`030<`0o`3oool0oooo
0>H0oooo1P0000002P3oool0o`000?ooo`3<o`00oooo0?l0003oool0c?l00?ooo`00ofH20?ooo`03
003oIP3oool0oooo00<0oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo03<0o`0503<0o`80
oooo00<0<`3o0?ooo`3oool00`3oool00`2?0?l0oooo0?ooo`020?ooo`0508l0o`3oool0oooo0?oo
o`3<0?l0203oool00`3<0?l0oooo0?ooo`3o0?ooo`T0oooo0@0000000280oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo08@0oooo1@0000020?ooo`04
0?l0003oool0c?l00?ooo`D0o`000`3oool00`00I_l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo
00T0oooo00<0S`3o0?ooo`3oool0103oool0103o0000oooo0?ooo`00ofH40?ooo`050<`0o`3oool0
oooo0?ooo`00I_l01@3oool01P0c0?l0oooo0?ooo`3oool0c03o08l0o`<0c03o5@3oool00`3<0?l0
oooo0?ooo`3U0?ooo`D0000000H0oooo0?l0003oool0oooo0<co003o00020?ooo`050<co0000ofH0
oooo0?ooo`00ofH01@3oool00`00I_l0oooo0?ooo`020?ooo`04001Vo`3oool0oooo0?ooo`D0<`3o
00<0oooo03<0o`3oool00`3oool208l0o`@0oooo00<0S`3o0?ooo`3oool00P3<0?l70?ooo`80c03o
o`3oool<0?ooo`400000001W0?ooo`030000003oool0oooo08@0oooo1P0000020?ooo`060?l0003<
o`00oooo0?ooo`00ofH0o`001P3oool00`00I_l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00T0
oooo00<0S`3o0?ooo`3oool00`3oool01@3o0000c?l00?ooo`3oool00?mV00@0oooo0P3<0?l20?oo
o`03001Vo`3oool0oooo0080oooo00<0<`3o0?ooo`3oool00P3<0?l0102?0?l0c03o0<`0o`3<0?lF
0?ooo`030<`0o`3oool0oooo0><0oooo1P0000002@3o0000oooo0?ooo`3<o`00oooo0?l0003oool0
c?l0003oIP030?ooo`03003oIP3oool0oooo0080oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0
oooo0?ooo`020?ooo`H0<`3o103oool00`2?0?l0oooo0?ooo`020?ooo`80S`3o0P3oool00`3<0?l0
oooo0?ooo`060?ooo`030<`0o`3oool0oooo0?l0oooo303oool100000000I`3oool00`000000oooo
0?ooo`250?ooo`D000000P3oool00`3o0000oooo0<co00020?ooo`03003oIP3oool0oooo00D0oooo
00<006Ko0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`090?ooo`80S`3o0`3oool01@3o0000c?l0
0?ooo`3oool00?mV00H0oooo0P3<0?l00`00I_l0oooo0?ooo`030?ooo`0303<0o`3oool0oooo0080
c03o00<0S`3o0<`0o`3<0?l05`3oool00`3<0?l0oooo0?ooo`3R0?ooo`D0000000H0oooo0?l0003o
ool0oooo0<co003o00020?ooo`050<co0000ofH0oooo0?ooo`00ofH0103oool00`00I_l0oooo0?oo
o`020?ooo`03001Vo`3oool0oooo00<0oooo00D0<`3o0?ooo`0c0?l0oooo03<0o`040?ooo`0308l0
o`3oool0oooo0080oooo00D0S`3o0?ooo`3oool0oooo0<`0o`070?ooo`80c03oo`3oool?0?ooo`40
0000001W0?ooo`030000003oool0oooo08D0oooo1@0000030?ooo`050?l0003<o`00oooo0?ooo`00
ofH0203oool00`00I_l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00X0oooo00D0S`3o0?ooo`3o
ool0oooo0?l000020?ooo`03003oIP3oool0oooo00H0oooo00<0c03o001Vo`3oool00`3oool00`0c
0?l0oooo0?ooo`030<`0o`0308l0o`3<0?l0oooo01H0oooo00<0c03o0?ooo`3oool0h@3oool60000
00090?l0003oool0oooo0<co003oool0o`000?ooo`3<o`000?mV0080oooo00<00?mV0?ooo`3oool0
0`3oool01@00I_l0oooo0?ooo`3oool006Ko00D0oooo00D0<`3o0?ooo`3oool0oooo03<0o`040?oo
o`0308l0o`3oool0oooo0080oooo00<0S`3o0<`0o`3<0?l00P3<0?l60?ooo`80c03oo`3ooolA0?oo
o`400000001W0?ooo`030000003oool0oooo08D0oooo1P0000020?ooo`030?l0003oool0c?l00080
oooo00<00?mV0?ooo`3oool0103oool5003oIPL0oooo00<0<`3o0?ooo`3oool02`3oool01@2?0?l0
oooo0?ooo`3o0000c?l00080oooo00<00?mV0?ooo`3oool01P3oool00`00I_l0c03o0?ooo`030?oo
o`0603<0o`3oool0oooo0?ooo`3<0?l0S`3o6@3oool00`3<0?l0oooo0?ooo`3P0?ooo`D0000000X0
oooo0?l0003oool0c?l00?ooo`3o0000oooo0<co003oool00?mV0P3oool00`00ofH0oooo0?ooo`02
0?ooo`05001Vo`3oool0oooo0?ooo`00I_l01@3oool01@0c0?l0oooo0?ooo`3oool0<`3o00@0oooo
00<0S`3o0?ooo`3oool00P3oool00`2?0?l0c03o0<`0o`030<`0o`D0oooo00<0c03o0?ooo`3oool0
o`3ooolA0?ooo`400000001W0?ooo`030000003oool0oooo08H0oooo1@0000030?ooo`050?l0003<
o`00oooo0?ooo`00ofH01P3oool5003oIPP0oooo00<0<`3o0?ooo`3oool02`3oool208l0o`050?l0
003<o`00oooo0?ooo`00ofH0203oool00`00I_l0oooo0<`0o`030?ooo`0303<0o`3oool0oooo00<0
oooo00<0S`3o0?ooo`3oool05`3oool00`3<0?l0oooo0?ooo`3N0?ooo`H0000000D0o`000?ooo`3o
ool0c?l00?l000020?ooo`050<co0000ofH0oooo0?ooo`00ofH0103oool00`00I_l0oooo0?ooo`02
0?ooo`03001Vo`3oool0oooo0080oooo00D0<`3o0?ooo`3oool0oooo03<0o`040?ooo`0408l0o`3o
ool0oooo0?ooo`80S`3o1P3<0?l30?ooo`80c03oo`3ooolD0?ooo`400000001W0?ooo`@00000Q@3o
ool5000000<0oooo00<0o`000?ooo`3<o`000P3oool00`00ofH0oooo0?ooo`020?ooo`H00?mV2@3o
ool00`0c0?l0oooo0?ooo`0<0?ooo`0308l0o`3o0000oooo0080oooo00<00?mV0?ooo`3oool01P3o
ool01000I_l0oooo0<`0o`3<0?l20?ooo`0303<0o`3oool0oooo00<0oooo00<0S`3o0?ooo`3oool0
5`3oool00`3<0?l0oooo0?ooo`3M0?ooo`D0000000T0oooo0?l0003oool0c?l00?ooo`3o0000oooo
0<co0000ofH00P3oool00`00ofH0oooo0?ooo`020?ooo`03001Vo`3oool0oooo0080oooo00<006Ko
0?ooo`3oool00P3oool01@0c0?l0oooo0?ooo`3oool0<`3o00<0oooo0P2?0?l30?ooo`0308l0o`3o
ool0oooo00H0c03o00<0oooo0<`0o`3<0?l0o`3ooolC0?ooo`@00000001W0?ooo`030000003oool0
oooo08H0oooo1P0000030?ooo`030?l0003<o`00oooo0080oooo00<00?mV0?ooo`3oool00P3oool5
003oIPX0oooo00<0<`3o0?ooo`3oool0303oool01@3o0000c?l00?ooo`3oool00?mV00P0oooo00L0
06Ko0?ooo`3oool0oooo0<`0o`3oool0<`3o00H0oooo00<0S`3o0?ooo`3oool05`3oool00`3<0?l0
oooo0?ooo`3K0?ooo`H0000000/0o`000?ooo`3oool0c?l00?l0003oool0c?l00?ooo`00ofH0oooo
003oIP040?ooo`03001Vo`3oool0oooo0080oooo00<006Ko0?ooo`3oool00P3oool01@0c0?l0oooo
0?ooo`3oool0<`3o00<0oooo00<0S`3o0?ooo`3oool00P3oool01@2?0?l0oooo0<`0o`3<0?l0oooo
00H0c03oo`3ooolH0?ooo`400000001W0?ooo`030000003oool0oooo08L0oooo1@0000030?ooo`03
0?l0003oool0c?l00080oooo00<00?mV0?ooo`3oool00P3oool5003oIP/0oooo00<0<`3o0?ooo`3o
ool02`3oool01@3o0000c?l00?ooo`3oool00?mV00T0oooo00@006Ko0?ooo`3oool0oooo0P3<0?l0
0`0c0?l0oooo0?ooo`040?ooo`0308l0o`3oool0oooo01H0oooo00<0c03o0?ooo`3oool0f`3oool3
000000D0o`0000H0c?l00?ooo`3o0000oooo0<co0000ofH20?ooo`03003oIP3oool0oooo0080oooo
00D006Ko0?ooo`3oool0oooo001Vo`040?ooo`0503<0o`3oool0oooo0?ooo`0c0?l00`3oool00`2?
0?l0oooo0?ooo`020?ooo`0308l0o`3oool0c03o00D0oooo0`3<0?oo0?oooaT0oooo0@00000006L0
oooo00<000000?ooo`3oool0Q`3oool6000000<0oooo00<0o`000<co003oool00P3oool00`00ofH0
oooo0?ooo`030?ooo`04003oIP3oool0oooo001Vo`/0oooo00<0<`3o0?ooo`3oool02`3oool01@3o
0000S`3o08l0o`3oool00?mV00P0oooo00<006Ko0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`05
0?ooo`0308l0o`3oool0oooo01H0oooo00<0c03o0?ooo`3oool0f@3oool4000000D0o`0000D0oooo
0?l0003oool0c?l0003oIP020?ooo`03003oIP3oool0oooo0080oooo00D006Ko0?ooo`3oool0oooo
001Vo`040?ooo`0503<0o`3oool0oooo0?ooo`0c0?l00`3oool0102?0?l0oooo0?ooo`3oool208l0
o`030?ooo`3<0?l0oooo00@0oooo0P3<0?oo0?oooa/0oooo0@00000006L0oooo00<000000?ooo`3o
ool0R03oool5000000<0oooo00<0o`000?ooo`3<o`000`3oool00`00ofH0oooo0?ooo`060?ooo`03
001Vo`3oool0oooo00T0oooo00<0<`3o0?ooo`3oool02P3oool01@3o0000c?l00?ooo`2?0?l00?mV
00T0oooo00<006Ko0?ooo`3oool00`3oool00`0c0?l0c03o0?ooo`040?ooo`0308l0o`3oool0oooo
01L0oooo00<0c03o0?ooo`3oool0f03oool3000000L0o`000P3oool0103<o`000?mV0?ooo`00ofH4
0?ooo`05001Vo`3oool0oooo0?ooo`00I_l0103oool01@0c0?l0oooo0?ooo`3oool0<`3o00<0oooo
00H0S`3o0?ooo`3oool0oooo08l0o`3oool20<`0o`D0oooo00<0c03o0?ooo`3oool0o`3ooolK0?oo
o`400000001W0?ooo`030000003oool0oooo08P0oooo1@0000040?ooo`030?l0003<o`00oooo0080
oooo00<00?mV0?ooo`3oool01`3oool00`00I_l0oooo0?ooo`080?ooo`0303<0o`3oool0oooo00X0
oooo00H0o`000<co003oool0oooo08l0o`00ofH80?ooo`03001Vo`3oool0oooo00<0oooo00<0<`3o
0?ooo`3<0?l01@3oool00`2?0?l0oooo0?ooo`0G0?ooo`030<`0o`3oool0oooo0=H0oooo1@000006
0?l000030?ooo`3<o`000?mV0080oooo00D00?mV0?ooo`3oool0oooo001Vo`030?ooo`04001Vo`3o
ool0oooo0?ooo`80<`3o0`3oool00`0c0?l0oooo0?ooo`0208l0o`<0oooo00<0S`3o0?ooo`3<0?l0
1@3oool20<`0ool0oooo7P3oool100000000I`3oool00`000000oooo0?ooo`280?ooo`H000000`3o
ool00`3o0000oooo0<co00030?ooo`03003oIP3oool0oooo00H0oooo00<006Ko0?ooo`3oool02@3o
ool00`0c0?l0oooo0?ooo`0:0?ooo`050?l0003<o`00oooo0?ooo`00ofH02@3oool00`00I_l0oooo
0?ooo`030?ooo`0403<0o`3oool0c03o0<`0o`@0oooo00<0S`3o0?ooo`3oool05`3oool00`3<0?l0
oooo0?ooo`3E0?ooo`D000001@3o00000`3oool0c?l0003oIP020?ooo`05003oIP3oool0oooo0?oo
o`00I_l00`3oool01@00I_l0oooo0?ooo`3oool0<`3o00<0oooo0P0c0?l408l0o`<0oooo00<0S`3o
0<`0o`3<0?l0103oool20<`0ool0oooo803oool100000000I`3oool00`000000oooo0?ooo`290?oo
o`D00000103oool00`3o0000oooo0<co00020?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?oo
o`3oool02@3oool00`0c0?l0oooo0?ooo`090?ooo`050?l0003<o`00oooo0?ooo`00ofH00P2?0?l7
0?ooo`03001Vo`3oool0oooo00<0oooo00@0<`3o0?ooo`3oool0oooo0P3<0?l30?ooo`0308l0o`3o
ool0oooo01L0oooo00<0c03o0?ooo`3oool0d`3oool50000000;0?l0003oool0oooo0?l0003oool0
o`000?ooo`3<o`000?mV0?ooo`00ofH0103oool01000I_l0oooo0?ooo`00I_l30?ooo`0603<0o`3o
ool0oooo0?ooo`0c0?l0oooo1@2?0?l0103oool0S`3o08l0o`3<0?l50?ooo`030<`0o`3oool0oooo
0?l0oooo803oool100000000I`3oool00`000000oooo0?ooo`290?ooo`D00000103oool00`3o0000
oooo0<co00030?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool02@3oool00`0c0?l0
oooo0?ooo`080?ooo`030?l0003<o`00oooo0080oooo00<00?mV0?ooo`2?0?l01P3oool00`00I_l0
oooo0?ooo`040?ooo`0303<0o`3oool0oooo0080oooo0P3<0?l20?ooo`0308l0o`3oool0oooo01L0
oooo00<0c03o0?ooo`3oool0dP3oool50000000:0?l0003oool0c?l00?ooo`3o0000oooo0<co0000
ofH0oooo003oIP@0oooo00@006Ko0?ooo`3oool006Ko0`3oool01P0c0?l0oooo0?ooo`3oool0<`3o
0?ooo`L0S`3o00<0oooo0<`0o`3oool00`3oool20<`0ool0oooo8`3oool100000000I`3oool00`00
0000oooo0?ooo`290?ooo`H00000103oool00`3o0000oooo0<co00030?ooo`03003oIP3oool0oooo
00L0oooo00<006Ko0?ooo`3oool02@3oool00`0c0?l0oooo0?ooo`080?ooo`050?l0003<o`00oooo
0?ooo`00ofH00P3oool00`2?0?l0oooo0?ooo`040?ooo`03001Vo`3oool0oooo00<0oooo00<0<`3o
0?ooo`3oool0103oool20<`0o`030?ooo`2?0?l0oooo01P0oooo0P3<0?oA0?ooo`D0000000P0o`00
0<co003<o`00c?l00?l0003oool0c?l0003oIP80oooo00D00?mV0?ooo`3oool0oooo001Vo`020?oo
o`05001Vo`3oool0oooo0?ooo`0c0?l00`3oool00`0c0?l0oooo0?ooo`0608l0o`80c03o0`3oool2
0<`0ool0oooo9@3oool100000000I`3oool00`000000oooo0?ooo`2:0?ooo`D00000103oool00`3o
0000oooo0<co00030?ooo`03003oIP3oool0oooo00D0c?l00P3oool00`00I_l0oooo0?ooo`0:0?oo
o`0303<0o`3oool0oooo00L0oooo00<0o`000<co003oool00P3oool00`00ofH0oooo0?ooo`0208l0
o`@0oooo00<006Ko0?ooo`3oool0103oool00`0c0?l0oooo0?ooo`050?ooo`80c03o00<0S`3o0?oo
o`3oool0603oool00`3<0?l0oooo0?ooo`3>0?ooo`D0000000X0o`000<co003<o`00c?l00?l0003o
ool0c?l0003oIP3oool00?mV0`3oool01000I_l0oooo0?ooo`00I_l30?ooo`0503<0o`3oool0oooo
0?ooo`0c0?l00P3oool00`2?0?l0oooo08l0o`0408l0o`@0oooo00<0c03o0?ooo`3oool0o`3ooolU
0?ooo`400000001W0?ooo`030000003oool0oooo08X0oooo1P0000040?ooo`030?l0003oool0c?l0
00<0oooo00<00?mV0?ooo`3<o`00103<o`030?ooo`03001Vo`3oool0oooo00X0oooo00<0<`3o0?oo
o`3oool01P3oool00`3o0000c?l00?ooo`020?ooo`03003oIP3oool0oooo0080oooo00<0S`3o0?oo
o`3oool00P3oool00`00I_l0oooo0?ooo`040?ooo`0303<0o`3oool0oooo00H0oooo00<0c03o08l0
o`3oool06@3oool00`3<0?l0oooo0?ooo`3=0?ooo`@0000000D0o`000<co003<o`00c?l00?l00002
0<co0003003oIP3oool00?mV00<0oooo00D006Ko0?ooo`3oool0oooo001Vo`020?ooo`0303<0o`3o
ool0oooo0080<`3o0P3oool0102?0?l0oooo0?ooo`3oool208l0o`040<`0o`3oool0oooo0?ooo`80
c03oo`3ooolX0?ooo`400000001W0?ooo`030000003oool0oooo08/0oooo1@0000040?ooo`030?l0
003oool0c?l000<0oooo00<00?mV0<co003<o`00103<o`040?ooo`03001Vo`3oool0oooo00X0oooo
00<0<`3o0?ooo`3oool01P3oool01@3o0000c?l00?ooo`3oool00?mV00@0<`3o00<0oooo08l0o`3o
ool00P3oool00`00I_l0oooo0?ooo`040?ooo`0303<0o`3oool0oooo00P0oooo00<0S`3o0<`0o`3o
ool0603oool00`3<0?l0oooo0?ooo`3;0?ooo`D0000000D0o`000<co003<o`00c?l00?l000020<co
00030?ooo`00ofH0oooo0080oooo00D006Ko0?ooo`3oool0oooo001Vo`020?ooo`0403<0o`3oool0
oooo03<0o`80oooo0P2?0?l20?ooo`80S`3o00@0c03o0?ooo`3oool0oooo0P3<0?oo0?ooobX0oooo
0@00000006L0oooo00<000000?ooo`3oool0R`3oool5000000D0oooo00<0o`000?ooo`3<o`000`3o
ool00`00ofH0c?l00<co00030<co00D0oooo00<006Ko0?ooo`3oool02P3oool00`0c0?l0oooo0?oo
o`050?ooo`060?l0003<o`00oooo0?ooo`0c0?l00?mV0`0c0?l20?ooo`0508l0o`3oool0oooo0?oo
o`00I_l01P3oool00`0c0?l0oooo0?ooo`070?ooo`0308l0o`3oool0c03o01T0oooo00<0c03o0?oo
o`3oool0bP3oool4000000050?l0003oool0c?l00<co003o00000P3<o`000`00ofH0oooo003oIP03
0?ooo`04001Vo`3oool0oooo001Vo`80oooo00@0<`3o0?ooo`3oool0<`3o0P3oool01P2?0?l0oooo
0?ooo`3oool0S`3o0<`0o`<0oooo0P3<0?oo0?ooob`0oooo0@00000006L0oooo00<000000?ooo`3o
ool0R`3oool6000000@0oooo00<0o`000?ooo`3<o`00103oool50<co00D0oooo00<006Ko0?ooo`3o
ool02`3oool00`0c0?l0oooo0?ooo`050?ooo`050?l0003oool0<`3o03<0o`00ofH00`0c0?l30?oo
o`80S`3o00<0oooo001Vo`3oool01P3oool00`0c0?l0oooo0?ooo`070?ooo`0408l0o`3oool0c03o
0<`0oaP0oooo00<0c03o0?ooo`3oool0b03oool4000000050?l000000000oooo0<co003o00000P3<
o`000`00ofH0oooo003oIP030?ooo`04001Vo`3oool0oooo001Vo`80oooo00@0<`3o0?ooo`3oool0
<`3o0P3oool00`2?0?l0oooo0?ooo`0208l0o`030<`0o`3oool0oooo0080c03oo`3oool^0?ooo`40
0000001W0?ooo`030000003oool0oooo08`0oooo1@0000050?ooo`030?l0003oool0c?l000<0oooo
00<00?mV0?ooo`3<o`00203oool00`00I_l0oooo0?ooo`0;0?ooo`80<`3o1@3oool00`3o0000c?l0
0?ooo`0203<0o`03003oIP0c0?l0<`3o00D0oooo00<0S`3o001Vo`3oool01P3oool00`0c0?l0oooo
0?ooo`080?ooo`0308l0o`3oool0oooo0080c03o5`3oool00`3<0?l0oooo0?ooo`370?ooo`@00000
00T0o`000?ooo`3<o`00oooo0?l0003oool0c?l00?ooo`00ofH00`3oool01000I_l0oooo0?ooo`00
I_l20?ooo`0503<0o`3oool0oooo03<0o`3oool00P2?0?l20?ooo`0408l0o`3<0?l0oooo0?ooo`80
c03oo`3oool`0?ooo`400000001W0?ooo`030000003oool0oooo08`0oooo1@0000050?ooo`040?l0
003oool0oooo0<co00<0oooo00<00?mV0?ooo`3oool0203oool00`00I_l0oooo0?ooo`0<0?ooo`03
03<0o`3oool0oooo0080oooo00<0o`000<co003oool00P0c0?l00`00ofH0<`3o03<0o`060?ooo`03
08l0o`00I_l0oooo00H0oooo00<0<`3o0?ooo`3oool0203oool0102?0?l0oooo0?ooo`3oool20<`0
oaH0oooo00<0c03o0?ooo`3oool0a@3oool40000000:0?l000000000c?l00?ooo`3o0000oooo0<co
0000ofH0oooo003oIP80oooo00@006Ko0?ooo`3oool006Ko0P3oool01P0c0?l0oooo0?ooo`0c0?l0
oooo08l0o`80oooo0P2?0?l0103<0?l0oooo0<`0o`3<0?oo0?oooc80oooo0@00000006L0oooo1000
002;0?ooo`H000001@3oool00`3o0000oooo0<co00030?ooo`03003oIP3oool0oooo00T0oooo00<0
06Ko0?ooo`3oool0303oool00`0c0?l0oooo0?ooo`020?ooo`050?l0003<o`00oooo0?ooo`00ofH0
2@3oool00`00I_l0S`3o0?ooo`060?ooo`0303<0o`3oool0oooo00P0oooo00<0S`3o0?ooo`3oool0
0P3oool20<`0oaD0oooo00<0c03o0?ooo`3oool0a03oool4000000090?l0003oool0c?l00?l0003o
ool0c?l0003oIP3oool00?mV0080oooo00@006Ko0?ooo`3oool006Ko0P3oool01P0c0?l0oooo03<0
o`0c0?l0oooo08l0o`80oooo00<0S`3o0<`0o`3oool00P3<0?oo0?oooc40oooo1000000006L0oooo
00<000000?ooo`3oool0S@3oool5000000D0oooo00@0o`000?ooo`3oool0c?l00`3oool50?l000H0
oooo00<006Ko0?ooo`3oool03@3oool01P0c0?l0oooo0?ooo`3oool0o`000<co00<0oooo00<00?mV
0?ooo`3oool01`3oool00`00I_l0S`3o0?ooo`050?ooo`0303<0o`3oool0oooo00T0oooo00<0S`3o
0?ooo`3oool00`3oool20<`0oa@0oooo00<0c03o0?ooo`3oool0`P3oool4000000090?l000000000
c?l00?ooo`3o0000oooo0<co003oool00?mV00<0oooo00<006Ko0?ooo`00I_l00P3oool0100c0?l0
oooo03<0o`3oool208l0o`040?ooo`2?0?l0S`3o0?ooo`80c03oo`3ooolf0?ooo`400000001W0?oo
o`030000003oool0oooo08d0oooo1P0000050?ooo`030?l0003oool0c?l000<0oooo1@3o00070?oo
o`03001Vo`3oool0oooo00d0oooo00D0<`3o0?ooo`3oool0o`000<co00030?ooo`03003oIP3oool0
oooo00L0oooo00<006Ko0?ooo`2?0?l01P3oool00`0c0?l0oooo0?ooo`090?ooo`0308l0o`3oool0
oooo00@0oooo0P3<0?lC0?ooo`80c03o`P3oool4000000060?l0003oool0c?l00?l00000ofH0c?l0
0P00ofH30?ooo`03001Vo`3oool006Ko0080oooo00H0<`3o0?ooo`0c0?l0oooo08l0o`3oool208l0
o`<0c03oo`3ooolh0?ooo`400000001W0?ooo`030000003oool0oooo08h0oooo1@0000060?ooo`04
0?l0003oool0c?l00?ooo`H0o`00203oool00`00I_l0oooo0?ooo`0=0?ooo`0503<0o`3oool0oooo
0?l0003<o`000P3oool00`00ofH0oooo0?ooo`080?ooo`04001Vo`3oool0S`3o08l0o`D0oooo00<0
<`3o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`050?ooo`80c03o4`3oool00`3<0?l0oooo0?oo
o`2n0?ooo`@0000000H0o`000000003<o`000?mV0?l0003<o`03003oIP80oooo00@006Ko0?ooo`00
I_l0oooo0P0c0?l01P3oool0<`3o08l0o`2?0?l0oooo08l0o`<0c03oo`3ooolj0?ooo`400000001W
0?ooo`030000003oool0oooo08h0oooo1@0000060?ooo`030?l0003oool0c?l00080oooo1@3o0009
0?ooo`03001Vo`3oool0oooo00d0oooo00@0<`3o0?ooo`3o0000c?l00`3oool00`00ofH0oooo0?oo
o`070?ooo`05001Vo`3oool0oooo0?ooo`2?0?l0103oool00`0c0?l0oooo0?ooo`090?ooo`0308l0
o`3oool0oooo00L0oooo0P3<0?lB0?ooo`030<`0o`3oool0oooo0;d0oooo0`0000001`3o00000000
0<co0000ofH0o`00003oIP3<o`000P00ofH20?ooo`05001Vo`3oool006Ko0?ooo`0c0?l00P3oool0
0`0c0?l0S`3o0?ooo`0208l0o`80c03oo`3oooll0?ooo`400000001W0?ooo`030000003oool0oooo
08h0oooo1P0000060?ooo`040?l0003oool0c?l00?ooo`D0o`002@3oool00`00I_l0oooo0?ooo`0>
0?ooo`0303<0o`3o0000c?l000<0oooo00<00?mV0?ooo`3oool0203oool01000I_l0oooo0?ooo`3o
ool208l0o`<0oooo00<0<`3o0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`080?ooo`80c03o4@3o
ool00`3<0?l0oooo0?ooo`2k0?ooo`@0000000H0o`000000003<o`00o`00003oIP3<o`03003oIP06
0?ooo`00I_l0oooo001Vo`3oool0<`3o0P3oool0100c0?l0S`3o0?ooo`2?0?l20<`0ool0oooo?P3o
ool100000000I`3oool00`000000oooo0?ooo`2?0?ooo`D000001P3oool00`3o0000oooo0<co0003
0?ooo`030?l00000ofH0oooo00X0oooo00<006Ko0?ooo`3oool03P3oool00`0c0?l0o`000<co0003
0?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool00`3oool208l0o`80oooo00<0<`3o
0?ooo`3oool02@3oool00`2?0?l0oooo0?ooo`090?ooo`80c03o403oool00`3<0?l0oooo0?ooo`2j
0?ooo`<0000000H0o`000000003<o`00oooo0?l0003<o`04003oIP06001Vo`3oool0oooo001Vo`0c
0?l0oooo0P0c0?l308l0o`030<`0o`3oool0oooo0?l0oooo?P3oool100000000I`3oool00`000000
oooo0?ooo`2?0?ooo`H000001P3oool00`3o0000oooo0<co00040?ooo`03003oIP3oool0oooo00T0
oooo00<006Ko0?ooo`3oool01`3oool5001Vo`80oooo00<0o`000<co003oool00P3oool00`00ofH0
oooo0?ooo`080?ooo`03001Vo`3oool0oooo00@0oooo00<0S`3o0?ooo`0c0?l0303oool00`2?0?l0
oooo0?ooo`0:0?ooo`80c03o3`3oool00`3<0?l0oooo0?ooo`2h0?ooo`@0000000P0o`000<co003o
ool0o`000<co0000ofH0oooo003oIP80oooo00X006Ko0?ooo`00I_l0<`3o0?ooo`0c0?l0S`3o0?oo
o`2?0?l0c03oo`3ooom20?ooo`400000001W0?ooo`030000003oool0oooo0900oooo1@0000060?oo
o`030?l0003oool0c?l000@0oooo00<00?mV0?ooo`3oool02P3oool00`00I_l0oooo0?ooo`060?oo
o`D006Ko0P3oool00`3o0000<`3o0<co00020?ooo`03003oIP3oool0oooo00P0oooo00<006Ko0?oo
o`3oool01@3oool208l0o`0303<0o`3oool0oooo00X0oooo00<0S`3o0?ooo`3oool02`3oool30<`0
o`d0oooo00<0c03o0?ooo`3oool0]`3oool3000000080?l000000000c?l00?l0003oool0c?l00?oo
o`00ofH20?ooo`06001Vo`3oool006Ko03<0o`3oool0<`3o0`2?0?oo0?oood@0oooo0@00000006L0
oooo00<000000?ooo`3oool0T03oool6000000H0oooo00<0o`000?ooo`3<o`00103oool00`00ofH0
oooo0?ooo`0:0?ooo`03001Vo`3oool0oooo00@0oooo1P00I_l30?ooo`030?l0003<o`00oooo0080
oooo00<00?mV0?ooo`3oool0203oool00`00I_l0oooo0?ooo`060?ooo`0308l0o`0c0?l0oooo00/0
oooo00<0S`3o0?ooo`3oool03@3oool20<`0o``0oooo00<0c03o0?ooo`3oool0]@3oool400000005
0?l0003<o`00oooo0?l0003<o`000P00ofH20?ooo`06001Vo`3oool006Ko03<0o`3oool0<`3o0P2?
0?l00`3<0?l0oooo0?ooo`3o0?oood<0oooo0@00000006L0oooo00<000000?ooo`3oool0T@3oool5
000000H0oooo00@0o`000?ooo`3oool0c?l0103oool00`00ofH0oooo0?ooo`0:0?ooo`03001Vo`3o
ool0oooo00@0oooo1@00I_l30?ooo`030?l0003<o`00<`3o0080oooo00<00?mV0?ooo`3oool0203o
ool00`00I_l0oooo0?ooo`070?ooo`0303<0o`2?0?l0oooo00/0oooo00<0S`3o0?ooo`3oool03P3o
ool20<`0o`/0oooo00<0c03o0?ooo`3oool0]03oool3000000050?l000000000c?l00?l0003<o`00
0P00ofH20?ooo`06001Vo`3oool006Ko03<0o`3oool0<`3o0P2?0?oo0?ooodL0oooo0@00000006L0
oooo00<000000?ooo`3oool0T@3oool5000000L0oooo00<0o`000?ooo`3<o`00103oool00`00ofH0
oooo0?ooo`0;0?ooo`03001Vo`3oool0oooo00<0oooo1@00I_l30?ooo`040?l0003oool0c?l003<0
o`80oooo00<00?mV0?ooo`3oool0203oool00`00I_l0oooo0?ooo`070?ooo`0303<0o`2?0?l0oooo
00/0oooo00<0S`3o0?ooo`3oool03`3oool30<`0o`T0oooo0P3<0?nd0?ooo`80000000`0o`000000
003<o`00o`000?ooo`3<o`00oooo003oIP3oool006Ko0?ooo`00I_l303<0o`0308l0o`3oool0oooo
0?l0ooooA`3oool100000000I`3oool00`000000oooo0?ooo`2A0?ooo`H000001P3oool0103o0000
oooo0?ooo`3<o`040?ooo`03003oIP3oool0oooo00X0oooo00<006Ko0?ooo`3oool01@3oool00`00
I_l0oooo0?ooo`040?ooo`030?l0003<o`00oooo0080<`3o00<00?mV0?ooo`3oool02@3oool00`00
I_l0oooo0?ooo`070?ooo`0303<0o`2?0?l0S`3o00/0oooo00<0S`3o0?ooo`3oool04@3oool20<`0
o`T0oooo00<0c03o0?ooo`3oool0/03oool00`000000o`000?l000040?l000070<co003oool00?mV
0?ooo`00I_l0oooo001Vo`0203<0o`80S`3oo`3ooom:0?ooo`400000001W0?ooo`030000003oool0
oooo0980oooo1@0000070?ooo`030?l0003oool0c?l000@0oooo00<00?mV0?ooo`3oool02`3oool0
0`00I_l0oooo0?ooo`0;0?ooo`030?l0003<o`00oooo0080oooo00<00?mV0?ooo`3oool02@3oool0
0`00I_l0oooo0?ooo`070?ooo`0303<0o`3oool0oooo0080S`3o2@3oool00`2?0?l0oooo0?ooo`0C
0?ooo`<0c03o1`3oool00`3<0?l0oooo0?ooo`2_0?ooo`030000003o0000o`0000<0o`0000L0c?l0
003oIP00ofH0oooo001Vo`3<0?l006Ko0080<`3o00<0S`3o0?ooo`3oool0o`3ooom:0?ooo`400000
001W0?ooo`030000003oool0oooo0980oooo1P0000060?ooo`040?l0003oool0oooo0<co00@0oooo
00<00?mV0?ooo`3oool02`3oool00`00I_l0oooo0?ooo`0:0?ooo`030?l0003oool0c?l000<0oooo
00<00?mV0?ooo`3oool02@3oool00`00I_l0oooo0?ooo`070?ooo`0503<0o`3oool0oooo0?ooo`2?
0?l02@3oool508l0oa<0oooo0P3<0?l60?ooo`030<`0o`3oool0oooo0:d0oooo00<000000?l0003o
0000103o0000203<o`000?mV0<`0o`00I_l0c03o001Vo`2?0?l0<`3oo`3ooom>0?ooo`400000001W
0?ooo`030000003oool0oooo09<0oooo1@0000070?ooo`030?l0003oool0c?l000D0oooo00<00?mV
0?ooo`3oool02`3oool00`00I_l0oooo0?ooo`0:0?ooo`030?l0003<o`00oooo0080oooo00<00?mV
03<0o`3oool02@3oool00`00I_l0oooo0?ooo`080?ooo`0403<0o`3oool0oooo0?ooo`80S`3o1`3o
ool508l0oaD0oooo0P3<0?l50?ooo`030<`0o`3oool0oooo0:`0oooo0P0000050?l00003003oIP3<
0?l0oooo008006Ko0P0c0?oo0?ooodl0oooo0@00000006L0oooo1000002B0?ooo`D000001`3oool0
103o0000oooo0?ooo`3<o`040?ooo`03003oIP3oool0oooo00`0oooo00<006Ko0?ooo`3oool02@3o
ool00`3o0000c?l00?ooo`030?ooo`03003oIP0c0?l0oooo00T0oooo00<006Ko0?ooo`3oool01`3o
ool00`0c0?l0oooo0?ooo`030?ooo`0308l0o`3oool0oooo00<0oooo1P2?0?lG0?ooo`<0c03o0`3o
ool00`3<0?l0oooo0?ooo`2Z0?ooo`<000001@3o00001@00ofH0oooo001Vo`00I_l0<`3o0?l0oooo
CP3oool400000000I`3oool00`000000oooo0?ooo`2C0?ooo`H000001`3oool00`3o0000oooo0<co
00050?ooo`03003oIP3oool0oooo00`0oooo00<006Ko0?ooo`3oool0203oool00`3o0000oooo0<co
00030?ooo`03003oIP3oool0<`3o00X0oooo00<006Ko0?ooo`3oool01`3oool00`0c0?l0oooo0?oo
o`030?ooo`80S`3o103oool508l0oaX0oooo0P3<0?l20?ooo`80c03oZP3oool020000000c03o0?l0
003<o`0000000?l0003<o`000?mV1000I_oo0?oooe80oooo0@00000006L0oooo00<000000?ooo`3o
ool0U03oool5000000L0oooo00@0o`000?ooo`3oool0c?l0103oool00`00ofH0oooo0?ooo`0=0?oo
o`03001Vo`3oool0oooo00P0oooo00<0o`000<co003oool00P3oool01000ofH0oooo0?ooo`0c0?l9
0?ooo`03001Vo`3oool0oooo00P0oooo00<0<`3o0?ooo`3oool0103oool208l0o`80oooo1@2?0?lL
0?ooo`<0c03o00<0oooo0<`0o`3oool0Y`3oool20<`0o`060?l000000000c?l00?l0003<o`000?mV
1@00I_oo0?oooe80oooo0@00000006L0oooo00<000000?ooo`3oool0U03oool6000000L0oooo00@0
o`000?ooo`3oool0c?l0103oool00`00ofH0oooo0?ooo`0=0?ooo`03001Vo`3oool0oooo00L0oooo
00<0o`000?ooo`3<o`000`3oool01000ofH0oooo0?ooo`0c0?l90?ooo`03001Vo`3oool0oooo00P0
oooo00<0<`3o0?ooo`3oool01@3oool01@2?0?l0oooo0?ooo`3oool0S`3o0080oooo00<0S`3o0?oo
o`3oool0703oool40<`0oj<0oooo0`3<0?l2000000050?l0003<o`00o`000<co0000ofH01P00I_oo
0?oooe80oooo0@00000006L0oooo00<000000?ooo`3oool0U@3oool5000000L0oooo00@0o`000?oo
o`3oool0c?l01@3oool00`00ofH0oooo0?ooo`050?ooo`D00?mV0`3oool00`00I_l0oooo0?ooo`07
0?ooo`030?l0003<o`00oooo0080oooo00D00?mV0?ooo`3oool0oooo03<0o`080?ooo`03001Vo`3o
ool0oooo00P0oooo00<0<`3o0?ooo`3oool01P3oool208l0o`D0oooo00<0S`3o0?ooo`3oool07`3o
ool30<`0oid0oooo0`3<0?l20?ooo`80000000H0o`000<co002?0?l0o`000<co0000ofH6001Vool0
ooooDP3oool100000000I`3oool00`000000oooo0?ooo`2E0?ooo`H000001`3oool0103o0000oooo
0?ooo`3<o`040?ooo`03003oIP3oool0oooo00D0oooo1@00ofH30?ooo`03001Vo`3oool0oooo00L0
oooo00<0o`000<co003oool00P3oool00`00ofH0oooo0?ooo`020?ooo`80<`3o1`3oool00`00I_l0
oooo0?ooo`080?ooo`0303<0o`3oool0oooo00L0oooo00<0S`3o0?ooo`3oool00`3oool208l0ob00
oooo1@3<0?nG0?ooo`<0c03o1@3oool01`000000c03o0?l0003<o`00o`000<co0000ofH01`00I_oo
0?oooe80oooo0@00000006L0oooo00<000000?ooo`3oool0UP3oool5000000L0oooo00@0o`000?oo
o`3oool0c?l01@3oool00`00ofH0oooo0?ooo`030?ooo`H00?mV103oool00`00I_l0oooo0?ooo`06
0?ooo`030?l0003oool0c?l000<0oooo00<00?mV0?ooo`3oool00`3oool00`0c0?l0oooo0?ooo`05
0?ooo`03001Vo`3oool0oooo00P0oooo00<0<`3o0?ooo`3oool01`3oool208l0o`D0oooo00<0S`3o
0?ooo`3oool07`3oool70<`0oi00oooo0`3<0?l70?ooo`070<`0o`2?0?l0o`000<co002?0?l0o`00
003oIP02001Vo`<0oooo00<006Ko0?ooo`3oool0o`3ooomB0?ooo`400000001W0?ooo`030000003o
ool0oooo09H0oooo1@0000080?ooo`040?l0003oool0oooo0<co00D0oooo00<00?mV0?ooo`3oool0
0`3oool5003oIPD0oooo00<006Ko0?ooo`3oool01P3oool00`3o0000c?l00?ooo`020?ooo`03003o
IP3oool0oooo00@0oooo00<0<`3o0?ooo`3oool0103oool00`00I_l0oooo0?ooo`090?ooo`0303<0
o`3oool0oooo00P0oooo0P2?0?l40?ooo`0308l0o`3oool0oooo01l0oooo1@3<0?l0103oool0c03o
0<`0o`3<0?n90?ooo`@0c03o203oool0203<0?l0S`3o08l0o`3<o`00o`000<co003o0000c?l00P00
I_oo0?oooeT0oooo0@00000006L0oooo00<000000?ooo`3oool0UP3oool6000000P0oooo00<0o`00
0?ooo`3<o`001@3oool00`00ofH0oooo0?ooo`030?ooo`D00?mV1P3oool00`00I_l0oooo0?ooo`05
0?ooo`030?l0003<o`00oooo00<0oooo00<00?mV0?ooo`3oool0103oool203<0o`D0oooo00<006Ko
0?ooo`3oool0203oool00`0c0?l0oooo0?ooo`0:0?ooo`80S`3o0`3oool00`2?0?l0oooo0?ooo`0M
0?ooo`H0c03o103oool40<`0oh40oooo103<0?l:0?ooo`030<`0o`2?0?l0S`3o0080000000<0o`00
0<co003o00000P3<o`000`00I_l0oooo0?ooo`3o0?oooeP0oooo0@00000006L0oooo00<000000?oo
o`3oool0U`3oool5000000P0oooo00@0o`000?ooo`3oool0c?l01@3oool00`00ofH0oooo0?ooo`04
0?ooo`03003oIP3oool0oooo00L0oooo00<006Ko0?ooo`3oool0103oool00`3o0000oooo0<co0003
0?ooo`03003oIP3oool0oooo00H0oooo00<0<`3o0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`09
0?ooo`0303<0o`3oool0oooo00/0oooo0P2?0?l20?ooo`0308l0o`3oool0oooo01d0oooo1@3<0?l8
0?ooo`@0c03oN@3oool40<`0o`T0oooo103<0?l208l0o`80oooo00D0S`3o0?l0003<o`00c?l00?l0
00020<co00<0<`3oo`3ooomH0?ooo`400000001W0?ooo`030000003oool0oooo09L0oooo1P000008
0?ooo`030?l0003oool0c?l000D0oooo00<00?mV0?ooo`3<o`00103<o`0;0?ooo`03001Vo`3oool0
oooo00@0oooo00<0o`000<co003oool00P3oool00`00ofH0oooo0?ooo`070?ooo`0303<0o`3oool0
oooo0080oooo00<006Ko0?ooo`3oool02@3oool00`0c0?l0oooo0?ooo`0<0?ooo`80S`3o00<0oooo
08l0o`3oool07@3oool70<`0o`X0oooo103<0?ma0?ooo`@0c03o3@3oool20<`0o`80S`3o00P0c03o
0?ooo`3oool0S`3o0000003o0000c?l00?l000<0c?l00`0c0?oo0?oooeP0oooo0@00000006L0oooo
00<000000?ooo`3oool0V03oool5000000P0oooo00@0o`000?ooo`3oool0c?l01@3oool00`00ofH0
c?l00<co00030<co00`0oooo00<006Ko0?ooo`3oool00`3oool00`3o0000c?l00?ooo`030?ooo`03
003oIP3oool0oooo00L0oooo0P0c0?l30?ooo`03001Vo`3oool0oooo00P0oooo00<0<`3o0?ooo`3o
ool03P3oool308l0oal0oooo00<0c03o0?ooo`3oool00P3oool20<`0o``0oooo103<0?mX0?ooo`D0
c03o3`3oool20<`0o`80S`3o0`3<0?l208l0o`040000003o0000c?l00?l000@0c?l00`0c0?oo0?oo
oeP0oooo0@00000006L0oooo00<000000?ooo`3oool0V03oool5000000T0oooo00<0o`000?ooo`3<
o`00103oool50?l00080c?l03@3oool00`00I_l0oooo0?ooo`020?ooo`030?l0003oool0c?l000<0
oooo00<00?mV0?ooo`3oool02@3oool0100c0?l0oooo0?ooo`00I_l;0?ooo`0303<0o`3oool0oooo
00l0oooo0`2?0?lT0?ooo`80c03o3P3oool50<`0oeh0oooo1@3<0?lB0?ooo`80c03o0P2?0?l40<`0
o`0908l0o`3oool0oooo03<0o`3o0000<`3o0?l00000I_l0c?l000D0<`3oo`3ooomH0?ooo`400000
001W0?ooo`030000003oool0oooo09P0oooo1P0000080?ooo`040?l0003oool0oooo0<co00<0oooo
1@3o00020<co00d0oooo00<006Ko0?ooo`3oool00`3oool00`3o0000c?l00?ooo`030?ooo`03003o
IP3oool0oooo00T0oooo00@0<`3o0?ooo`3oool006Ko2`3oool00`0c0?l0oooo0?ooo`0@0?ooo`<0
S`3o903oool20<`0oa40oooo1@3<0?mD0?ooo`D0c03o503oool20<`0o`<0S`3o0P3oool20<`0o`80
S`3o00T0c03o03<0o`0c0?l0o`000<co003o000006Ko0000003oool01@0c0?oo0?oooeP0oooo0@00
000006L0oooo00<000000?ooo`3oool0V@3oool5000000T0oooo00D0o`000?ooo`3oool0c?l00?oo
o`060?l00080c?l03P3oool00`00I_l0oooo0?ooo`020?ooo`030?l0003oool0c?l000<0oooo00<0
0?mV0?ooo`3oool02P3oool203<0o`03001Vo`3oool0oooo00X0oooo00<0<`3o0?ooo`3oool04@3o
ool308l0ob@0oooo0P3<0?lD0?ooo`L0c03oAP3oool70<`0oaL0oooo0P3<0?l208l0o`D0oooo0P2?
0?l02@3<0?l0oooo03<0o`3oool0o`000<co003o0000c?l0000000040?ooo`0303<0o`3oool0oooo
0?l0ooooF03oool100000000I`3oool00`000000oooo0?ooo`2I0?ooo`H00000203oool0103o0000
oooo0?ooo`3<o`020?ooo`D0o`004@3oool01`00I_l0oooo0?ooo`3oool0o`000?ooo`3<o`00103o
ool00`00ofH0oooo0?ooo`0;0?ooo`0303<0o`00I_l0oooo00X0oooo00<0<`3o0?ooo`3oool04P3o
ool00`2?0?l0oooo08l0o`0208l0ob<0oooo0P3<0?lI0?ooo`P0c03o=`3oool70<`0oa`0oooo0P3<
0?l208l0o`H0oooo00<0S`3o0?ooo`3oool00P0c0?l01@3oool0<`3o0?l0003<o`00o`0000800000
o`3ooomO0?ooo`400000001W0?ooo`@00000V@3oool5000000T0oooo00D0o`000?ooo`3oool0c?l0
0?ooo`050?l00003003oIP3oool0oooo00l0oooo00H006Ko0?ooo`3oool0oooo0?l0003<o`040?oo
o`03003oIP3oool0oooo00`0oooo00<0<`3o001Vo`3oool02P3oool00`0c0?l0oooo0?ooo`0B0?oo
o`0408l0o`3oool0oooo0?ooo`80S`3o8`3oool20<`0oal0oooo3P3<0?lK0?ooo`h0c03o803oool2
0<`0o`<0S`3o1P3oool208l0o`80oooo00L0<`3o0?ooo`3oool0<`3o0?l0003<o`00o`0000800000
o`3ooomM0?ooo`@00000001W0?ooo`030000003oool0oooo09X0oooo1P0000080?ooo`040?l0003o
ool0oooo0<co00<0oooo00@0o`000?ooo`3oool00?mV4P3oool01P00I_l0oooo0?ooo`3o0000oooo
0<co00@0oooo00<00?mV0?ooo`3oool0303oool00`00I_l0<`3o0?ooo`0:0?ooo`0303<0o`3oool0
oooo0180oooo0P2?0?l30?ooo`80S`3o8`3oool20<`0ob/0oooo6`3<0?l/0?ooo`80c03o0P2?0?l7
0?ooo`80S`3o0`3oool0200c0?l0oooo0?ooo`0c0?l000000?l00000ofH0o`000P00003o0?ooof00
oooo0@00000006L0oooo00<000000?ooo`3oool0V`3oool5000000T0oooo00@0o`000?ooo`3oool0
c?l01P3oool00`00ofH0oooo0?ooo`0@0?ooo`05001Vo`3oool0o`000?ooo`3<o`00103oool00`00
ofH0oooo0?ooo`0=0?ooo`03001Vo`0c0?l0oooo00X0oooo00<0<`3o0?ooo`3oool04`3oool00`2?
0?l0oooo0?ooo`020?ooo`<0S`3o8P3oool30<`0of`0oooo0`3<0?l208l0o`L0oooo1@2?0?l203<0
o`070?ooo`0c0?l0<`3o0?ooo`3o0000c?l00?l0000200000?l0ooooH@3oool100000000I`3oool0
0`000000oooo0?ooo`2K0?ooo`D000002@3oool0103o0000oooo0?ooo`3<o`070?ooo`03003oIP3o
ool0oooo0100oooo00@006Ko0?ooo`3o0000c?l01@3oool00`00ofH0oooo0?ooo`0<0?ooo`03001V
o`3oool0<`3o00X0oooo00<0<`3o0?ooo`3oool0503oool208l0o`D0oooo0P2?0?lS0?ooo`80c03o
I`3oool30<`0o`030?ooo`2?0?l0S`3o00L0oooo0P2?0?l0203oool0S`3o08l0o`2?0?l0<`3o08l0
o`3oool0<`3o0P3oool00`00I_l0o`000?l0000300000?l0ooooH@3oool100000000I`3oool00`00
0000oooo0?ooo`2K0?ooo`H000002@3oool0103o0000oooo0?ooo`3<o`070?ooo`03003oIP3oool0
oooo0100oooo00@006Ko0?l0003oool0c?l0103oool00`00ofH0oooo0?ooo`0=0?ooo`04001Vo`3o
ool0<`3o03<0o`T0oooo00<0<`3o0?ooo`3oool05@3oool208l0o`D0oooo0`2?0?lR0?ooo`<0c03o
H03oool40<`0o`040?ooo`2?0?l0S`3o08l0o`L0oooo0P2?0?l20?ooo`80S`3o0P0c0?l208l0o`07
03<0o`3oool0oooo001Vo`3o00000?mV0?l0000200000?l0ooooHP3oool100000000I`3oool00`00
0000oooo0?ooo`2L0?ooo`D000002@3oool0103o0000oooo0?ooo`3<o`070?ooo`03003oIP3oool0
oooo0140oooo00<0o`000?ooo`3<o`001@3oool00`00ofH0oooo0?ooo`0=0?ooo`04001Vo`3oool0
oooo03<0o`T0oooo00<0<`3o0?ooo`3oool05P3oool00`2?0?l0oooo0?ooo`050?ooo`<0S`3o8P3o
ool30<`0oeX0oooo0`3<0?l20?ooo`<0S`3o203oool208l0o`@0oooo0P0c0?l308l0o`0703<0o`3o
ool0oooo001Vo`3o0000c?l00?l0000300000?l0ooooHP3oool100000000I`3oool00`000000oooo
0?ooo`2L0?ooo`H000002@3oool0103o0000oooo0?ooo`3<o`070?ooo`03003oIP3oool0oooo0100
oooo00@006Ko0?l0003oool0c?l0103oool00`00ofH0oooo0?ooo`0=0?ooo`05001Vo`3oool0oooo
0?ooo`0c0?l02@3oool00`0c0?l0oooo0?ooo`0F0?ooo`80S`3o203oool308l0ob80oooo0`3<0?mD
0?ooo`<0c03o00<0oooo08l0o`2?0?l00P2?0?l80?ooo`<0S`3o103oool203<0o`030?ooo`2?0?l0
S`3o0080<`3o00<0S`3o0?ooo`3o0000103o000200000?l0ooooH`3oool100000000I`3oool00`00
0000oooo0?ooo`2M0?ooo`D000002@3oool01@3o0000oooo0?ooo`3oool0c?l000L0oooo00<00?mV
0?ooo`3oool0403oool00`3o0000oooo0<co00050?ooo`03003oIP3oool0oooo00d0oooo00@006Ko
0?ooo`3oool0oooo0P0c0?l80?ooo`0303<0o`3oool0oooo01L0oooo00<0S`3o0?ooo`3oool0203o
ool408l0ob40oooo0`3<0?m=0?ooo`@0c03o00@0oooo08l0o`2?0?l0S`3o2P3oool208l0o`D0oooo
0P0c0?l40?ooo`0503<0o`2?0?l0oooo0?ooo`00I_l01@3o000200000?l0ooooH`3oool100000000
I`3oool00`000000oooo0?ooo`2M0?ooo`H000002@3oool0103o0000oooo0?ooo`3<o`080?ooo`03
003oIP3oool0oooo00l0oooo00<0o`00001Vo`3<o`001@3oool00`00ofH0oooo0?ooo`0=0?ooo`03
001Vo`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool01P3oool503<0oaD0oooo0P2?0?l<0?ooo`<0
S`3o8@3oool30<`0odD0oooo1@3<0?l20?ooo`<0S`3o2`3oool208l0o`H0oooo00<0<`3o0?ooo`3o
ool00P3oool203<0o`<0oooo00<006Ko0?l0003o0000103o00000`000000oooo0?ooo`3o0?ooof80
oooo0@00000006L0oooo00<000000?ooo`3oool0W@3oool6000000T0oooo00D0o`000?ooo`3oool0
oooo0<co00070?ooo`03003oIP3oool0oooo0100oooo00<0o`00001Vo`3<o`001@3oool00`00ofH0
oooo0?ooo`0=0?ooo`03001Vo`3oool0oooo00<0oooo00<0<`3o0?ooo`3oool01@3oool503<0oaL0
oooo0P2?0?l=0?ooo`<0S`3o8@3oool40<`0oah0oooo1@3<0?lI0?ooo`D0c03o0`3oool408l0o``0
oooo0P2?0?l60?ooo`80<`3o103oool00`0c0?l0oooo0?ooo`020?ooo`03001Vo`3oool00?mV00D0
o`0000<000000?ooo`3oool0o`3ooomR0?ooo`400000001W0?ooo`030000003oool0oooo09h0oooo
1@00000:0?ooo`050?l0003oool0oooo0?ooo`3<o`001`3oool00`00ofH0oooo0?ooo`0?0?ooo`03
0?l0003oool0c?l000H0oooo00<00?mV0?ooo`3oool03@3oool00`00I_l0oooo0?ooo`030?ooo`80
<`3o103oool603<0oaT0oooo00<0S`3o0?ooo`3oool03@3oool408l0ob40oooo103<0?lJ0?ooo`D0
c03o4P3oool70<`0o`@0oooo102?0?l>0?ooo`80S`3o1P3oool203<0o`@0oooo0P0c0?l40?ooo`04
001Vo`3oool006Ko003oIPD0o`00o`3ooomU0?ooo`400000001W0?ooo`030000003oool0oooo09h0
oooo1P0000090?ooo`050?l0003oool0oooo0?ooo`3<o`00203oool00`00ofH0oooo0?ooo`0?0?oo
o`030?l0003<o`0006Ko00D0oooo00<00?mV0?ooo`3oool03@3oool00`00I_l0oooo0?ooo`050?oo
o`0303<0o`3oool0oooo0080oooo1@0c0?lJ0?ooo`80S`3o4@3oool408l0ob40oooo203<0?lA0?oo
o`H0c03o303oool60<`0o`L0oooo102?0?l?0?ooo`<0S`3o1P3oool203<0o`D0oooo00<0<`3o0?oo
o`3oool00`3oool01000I_l0oooo001Vo`3oool20?l00004003oIP3o00000?mV003oI_l0ooooI@3o
ool100000000I`3oool00`000000oooo0?ooo`2O0?ooo`D000002P3oool01@3o0000oooo0?ooo`3o
ool0c?l000P0oooo00<00?mV0?ooo`3oool03P3oool0103o0000oooo0<co0000I_l50?ooo`03003o
IP3oool0oooo00d0oooo00<006Ko0?ooo`3oool01@3oool203<0o`80oooo1@0c0?lL0?ooo`80S`3o
4`3oool408l0obD0oooo8`3<0?l90?ooo`@0S`3o4@3oool208l0o`P0oooo00<0<`3o0?ooo`3oool0
0`3oool203<0o`@0oooo0P00I_l02@3oool006Ko0?ooo`3o0000c?l00?l0000000000?mV0000003o
0?ooofH0oooo0@00000004@0oooo0`0000060?ooo`8000001`3oool2000000l0oooo00<000000?oo
o`3oool0W`3oool6000000X0oooo00D0o`000?ooo`3oool0oooo0<co00070?ooo`03003oIP3oool0
oooo00h0oooo00D0o`000?ooo`3<o`00oooo001Vo`040?ooo`03003oIP3oool0oooo00d0oooo00<0
06Ko0?ooo`3oool01`3oool203<0o`80oooo00@0<`3o0?ooo`3oool0<`3o7@3oool208l0oaD0oooo
1P2?0?la0?ooo`D0c03o3`3oool608l0oa<0oooo0P2?0?l80?ooo`80<`3o1@3oool00`0c0?l0oooo
0?ooo`030?ooo`06001Vo`3oool0oooo001Vo`3oool00?mV0P3o000400000?l0ooooIP3oool10000
0000@`3oool01@000000oooo0?ooo`3oool0000000D0oooo0P0000070?ooo`8000003`3oool00`00
0000oooo0?ooo`2P0?ooo`D000002P3oool01@3o0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV
0?ooo`3oool03P3oool01@3o0000oooo0<co003oool006Ko00@0oooo00<00?mV0?ooo`3oool03@3o
ool00`00I_l0oooo0?ooo`080?ooo`80<`3o103oool00`0c0?l0oooo0?ooo`0L0?ooo`80S`3o6@3o
ool808l0ob/0oooo00<0c03o0?ooo`3oool01`3oool808l0oaL0oooo0P2?0?l80?ooo`80<`3o1@3o
ool203<0o`D0oooo00H006Ko0?ooo`3oool006Ko0?ooo`00ofH20?l000D00000o`3ooomV0?ooo`40
000000120?ooo`030000003oool0oooo0080oooo0P00000=0?ooo`8000003`3oool00`000000oooo
0?ooo`2P0?ooo`D000002`3oool01@3o0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV0?ooo`3o
ool03@3oool00`3o0000oooo0<co00020?ooo`05001Vo`3oool0oooo0?ooo`00ofH0403oool00`00
I_l0oooo0?ooo`090?ooo`0303<0o`3oool0oooo0080oooo00<0<`3o0?ooo`3oool07@3oool208l0
oal0oooo1P2?0?lX0?ooo`L0S`3o7@3oool208l0o`P0oooo0P0c0?l60?ooo`0303<0o`3oool0oooo
00<0oooo0P00I_l20?ooo`04001Vo`3oool00?mV003oIP80o`001000003o0?ooofL0oooo0@000000
0480oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0;0?ooo`8000003`3oool00`00
0000oooo0?ooo`2P0?ooo`H000002P3oool00`3o0000oooo0?ooo`020?ooo`030<co003oool0oooo
00D0oooo00<00?mV0?ooo`3oool03@3oool0103o0000oooo0?ooo`3<o`020?ooo`05001Vo`3oool0
oooo0?ooo`00ofH03`3oool00`00I_l0oooo0?ooo`0:0?ooo`80<`3o0`3oool00`0c0?l0oooo0?oo
o`0N0?ooo`80S`3o3P3oool508l0oa00oooo4P2?0?l50?oooa40S`3o8@3oool308l0o`P0oooo0P0c
0?l60?ooo`80<`3o1@3oool01P00I_l0oooo0?ooo`3oool006Ko0?ooo`800?mV0P3o000500000?l0
ooooI`3oool100000000@@3oool2000000D0oooo0P0000060?ooo`X000003@3oool6000009h0oooo
1@00000;0?ooo`050?l0003oool0oooo0?ooo`3<o`00203oool00`00ofH0oooo0?ooo`0=0?ooo`03
0?l0003oool0c?l000<0oooo00@006Ko0?ooo`3oool00?mV403oool00`00I_l0oooo0?ooo`0;0?oo
o`80<`3o0P3oool00`0c0?l0oooo0?ooo`0O0?ooo`80S`3o303oool508l0ob80oooo1@2?0?l_0?oo
o`<0S`3o2P3oool00`0c0?l0oooo0?ooo`050?ooo`0303<0o`3oool0oooo00@0oooo00H006Ko0?oo
o`3oool0oooo001Vo`3oool2003oIP030<co003o0000000000@00000o`3ooomS0?ooo`H000000011
0?ooo`8000001@3oool2000000L0oooo00<000000?ooo`3oool00P3oool2000000l0oooo00<00000
0?ooo`3oool0X@3oool6000000/0oooo00D0o`000?ooo`3oool0oooo0<co00080?ooo`03003oIP3o
ool0oooo00`0oooo00@0o`000?ooo`3oool0c?l00`3oool01000I_l0oooo0?ooo`00ofH50?ooo`D0
06Ko1@3oool00`00I_l0oooo0?ooo`0=0?ooo`80<`3o00<0oooo03<0o`3oool08@3oool308l0o`P0
oooo1P2?0?mC0?ooo`<0S`3o2`3oool203<0o`H0oooo0P0c0?l60?ooo`03001Vo`3oool0oooo0080
06Ko00@0oooo003oIP00ofH0c?l00P3o000500000?l0ooooJ03oool100000000@@3oool2000000D0
oooo0P0000070?ooo`030000003oool0oooo0080oooo0P00000?0?ooo`030000003oool0oooo0:80
oooo1@00000;0?ooo`030?l0003oool0oooo0080oooo00<0c?l00?ooo`3oool01P3oool00`00ofH0
oooo0?ooo`0;0?ooo`040?l0003oool0oooo0<co00@0oooo00<006Ko0?ooo`00ofH01@3oool5001V
o`H0oooo00<006Ko0?ooo`3oool03P3oool303<0ob@0oooo0`2?0?l60?ooo`D0S`3oD03oool308l0
o``0oooo0P0c0?l70?ooo`0303<0o`3oool0oooo00@0oooo0P00I_l20?ooo`04001Vo`3oool0oooo
0?ooo`800?mV0P3o000500000?l0ooooJ@3oool100000000@@3oool2000000D0oooo0P0000080?oo
o`040000003oool0oooo0?ooo`8000003`3oool00`000000oooo0?ooo`2R0?ooo`H000002`3oool0
1@3o0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV0?ooo`3oool0303oool00`3o0000oooo0<co
00050?ooo`03001Vo`3oool00?mV00<0oooo1P00I_l70?ooo`03001Vo`3oool0oooo00l0oooo0P0c
0?lV0?ooo`80S`3o103oool508l0odd0oooo0`2?0?l=0?ooo`80<`3o1`3oool203<0o`H0oooo00D0
06Ko0?ooo`3oool0oooo001Vo`030?ooo`800?mV00<0c?l00?l0000000001@00003o0?ooofT0oooo
0@0000000440oooo0P0000050?ooo`8000002@3oool00`000000oooo0?ooo`02000000l0oooo00<0
00000?ooo`3oool0X`3oool5000000`0oooo00D0o`000?ooo`3oool0oooo0<co00080?ooo`03003o
IP3oool0oooo00/0oooo00@0o`000?ooo`3oool0c?l01@3oool00`00I_l0oooo003oIP030?ooo`D0
06Ko1`3oool00`00I_l0oooo0?ooo`0A0?ooo`80<`3o9P3oool308l0o`<0oooo00<0S`3o0?ooo`3o
ool0B@3oool408l0o`h0oooo0P0c0?l80?ooo`0303<0o`3oool0oooo00D0oooo00D006Ko0?ooo`3o
ool0oooo001Vo`030?ooo`800?mV00<0c?l00?l0003oool01@00003o0?ooofX0oooo0@0000000440
oooo0P0000050?ooo`8000002@3oool00`000000oooo0?ooo`02000000l0oooo00<000000?ooo`3o
ool0X`3oool5000000`0oooo00<0o`000?ooo`3oool00P3oool00`3<o`00oooo0?ooo`060?ooo`03
003oIP3oool0oooo00X0oooo00@0o`000?ooo`3oool0c?l01P3oool00`00I_l00?mV0?ooo`020?oo
o`D006Ko203oool00`00I_l0oooo0?ooo`0A0?ooo`@0<`3o9P3oool308l0odP0oooo102?0?l@0?oo
o`80<`3o203oool203<0o`H0oooo0P00I_l30?ooo`04001Vo`3oool0oooo0?ooo`800?mV00<0c?l0
0?l0003o00000P3<o`0400000?l0ooooJP3oool100000000@@3oool2000000D0oooo0P00000:0?oo
o`040000003oool00000000000l0oooo00<000000?ooo`3oool0X`3oool6000000`0oooo00D0o`00
0?ooo`3oool0oooo0<co00020?ooo`D0o`000P3oool00`00ofH0oooo0?ooo`0:0?ooo`040?l0003o
ool0oooo0<co00H0oooo00<006Ko003oIP3oool00`3oool00`00I_l0oooo0?ooo`080?ooo`03001V
o`3oool0oooo01<0oooo00@0<`3o0?ooo`0c0?l0<`3o9`3oool308l0od40oooo102?0?lA0?ooo`<0
<`3o2@3oool00`0c0?l0oooo0?ooo`050?ooo`03001Vo`3oool0oooo0080oooo00@006Ko0?ooo`3o
ool0oooo0P00ofH20<co00040?l0003<o`00c?l00<co00<00000o`3ooom[0?ooo`40000000120?oo
o`030000003oool0oooo00<0oooo00<000000?ooo`3oool02P3oool3000000l0oooo00<000000?oo
o`3oool0Y03oool5000000`0oooo00<0o`000?ooo`3oool00P3oool00`3<o`00oooo0?l000040?l0
00030?ooo`3<o`00c?l000<0c?l0103oool5003oIP040?l0003oool0oooo0<co00L0oooo00<00?mV
0?ooo`3oool03P3oool00`00I_l0oooo0?ooo`0C0?ooo`0303<0o`3oool0oooo00<0<`3o9`3oool4
08l0ocH0oooo1`2?0?lC0?ooo`80<`3o2P3oool203<0o`L0oooo00<006Ko0?ooo`3oool00P3oool0
2@00I_l0oooo0?ooo`3oool00?mV0?ooo`00ofH0c?l00?l000040<co00<00000o`3ooom[0?ooo`40
000000120?ooo`8000000`3oool2000000`0oooo0`00000?0?ooo`030000003oool0oooo0:@0oooo
1P00000<0?ooo`030?l0003oool0oooo0080oooo1P3o00000`3oool0c?l00<co00030<co00@0oooo
1@00ofH01@3oool0o`000?ooo`3oool0c?l000L0oooo00<00?mV0?ooo`3oool03P3oool00`00I_l0
oooo0?ooo`0C0?ooo`80<`3o0`3oool203<0obT0oooo102?0?l`0?ooo`L0S`3o4P3oool303<0o`/0
oooo00<0<`3o0?ooo`3oool01@3oool2001Vo`<0oooo1@00I_l01000ofH0oooo003oIP3<o`020?l0
00@0c?l00P00003o0?ooof`0oooo0@00000004<0oooo1@00000>0?ooo`8000003`3oool00`000000
oooo0?ooo`2U0?ooo`D000003@3oool01@3o0000oooo0?ooo`3oool0c?l000D0o`001P3<o`030?oo
o`H00?mV00D0oooo0?l0003oool0oooo0<co00070?ooo`03003oIP00I_l006Ko00l0oooo00<006Ko
0?ooo`3oool0503oool00`0c0?l0oooo0?ooo`020?ooo`<0<`3o:P3oool808l0ob<0oooo1@2?0?l0
0`3oool0S`3o08l0o`0408l0oa00oooo0P0c0?l<0?ooo`80<`3o1`3oool00`00I_l0oooo0?ooo`03
0?ooo`@006Ko00<00?mV0?ooo`00ofH00P3<o`000`3o0000c?l00<co00030<co00800000o`3ooom/
0?ooo`400000001W0?ooo`030000003oool0oooo0:D0oooo1@00000=0?ooo`030?l0003oool0oooo
0080oooo1@3o00000`3oool0c?l00<co00030<co00@0oooo1@00ofH00`3oool0o`000?ooo`020?oo
o`030<co003oool0oooo00D0oooo00<00?mV0?ooo`00I_l03P3oool00`00I_l0oooo0?ooo`0E0?oo
o`80<`3o1@3oool303<0obl0oooo2P2?0?l;0?ooo`h0S`3o1`3oool508l0o`d0oooo0`0c0?l=0?oo
o`0303<0o`3oool0oooo00H0oooo00<006Ko0?ooo`3oool00P3oool5001Vo`03003oIP00I_l00?mV
0080c?l000D0o`000?ooo`00000000000<co000300000?l0ooooK@3oool100000000I`3oool00`00
0000oooo0?ooo`2U0?ooo`H000003@3oool00`3o0000oooo0?ooo`020?ooo`030<co003o0000oooo
0080oooo1@3<o`040?ooo`D00?mV0P3oool0103o0000oooo0?ooo`3<o`070?ooo`04003oIP3oool0
oooo001Vo`h0oooo00<006Ko0?ooo`3oool05P3oool00`0c0?l0oooo0?ooo`050?ooo`<0<`3o=P3o
ool;08l0oaD0oooo1@2?0?l:0?ooo`<0<`3o3P3oool203<0o`P0oooo00<006Ko0?ooo`3oool00P3o
ool00`00I_l0oooo0?ooo`02001Vo`06003oIP00I_l006Ko003oIP3<o`00o`000P3oool600000?l0
ooooK@3oool100000000I`3oool00`000000oooo0?ooo`2V0?ooo`D000003P3oool01@3o0000oooo
0?ooo`3oool0c?l000H0oooo00@0c?l00?ooo`3oool00?mV1@3oool00`00ofH0oooo0?ooo`020?oo
o`040?l0003oool0oooo0<co00P0oooo00<00?mV0?ooo`3oool00P00I_l=0?ooo`03001Vo`3oool0
oooo01H0oooo0P0c0?l80?ooo`<0<`3oE@3oool00`2?0?l0oooo0?ooo`060?ooo`@0<`3o3`3oool6
03<0o`@0oooo0P00I_l40?ooo`06001Vo`3oool0oooo0?ooo`00I_l00?mV0P00I_l01@00ofH0c?l0
0<co003o0000oooo00H00000o`3ooom^0?ooo`400000001W0?ooo`030000003oool0oooo0:H0oooo
1P00000=0?ooo`030?l0003oool0oooo0080oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo
0?ooo`080?ooo`040?l0003oool0oooo0<co00L0oooo00<00?mV0?ooo`3oool00P3oool00`00I_l0
oooo0?ooo`0;0?ooo`03001Vo`3oool0oooo01L0oooo00<0<`3o0?ooo`3oool0203oool303<0oeP0
oooo0`0c0?lA0?ooo`80<`3o00<0oooo03<0o`0c0?l00`0c0?l30?ooo`03001Vo`3oool0oooo00<0
oooo00<006Ko0?ooo`3oool00P3oool01000ofH0oooo001Vo`00ofH20<co00030?l0003oool0oooo
00D00000o`3ooom_0?ooo`400000001W0?ooo`030000003oool0oooo0:L0oooo1@00000>0?ooo`03
0?l0003oool0oooo0080oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo0?ooo`070?ooo`04
0?l0003oool0oooo0<co00P0oooo00<00?mV0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0;0?oo
o`03001Vo`3oool0oooo01L0oooo00<0<`3o0?ooo`3oool02P3oool303<0oe80oooo0`0c0?lB0?oo
o`80<`3o0P3oool603<0o`80oooo00<006Ko0?ooo`3oool00`3oool00`00I_l0oooo0?ooo`030?oo
o`05003oIP3oool00?mV0<co003oool00P3o00000`3oool000000000000400000?l0ooooK`3oool1
00000000I`3oool00`000000oooo0?ooo`2W0?ooo`H000003@3oool00`3o0000oooo0?ooo`030?oo
o`030<co003oool0oooo00H0oooo00<00?mV0?ooo`3oool01`3oool01@3o0000oooo0?ooo`3oool0
c?l000L0oooo00<00?mV0?ooo`3oool00`3oool2001Vo``0oooo00<006Ko0?ooo`3oool05`3oool2
03<0o`d0oooo0P0c0?m=0?ooo`<0<`3o4`3oool203<0o`D0oooo1@0c0?l2001Vo`D0oooo00<006Ko
0?ooo`3oool00`3oool01@00ofH0oooo003oIP3<o`00oooo00D0o`001000003o0?ooog00oooo0@00
000006L0oooo1000002W0?ooo`D000003P3oool00`3o0000oooo0?ooo`020?ooo`030<co003oool0
oooo00L0oooo00<00?mV0?ooo`3oool01`3oool0103o0000oooo0?ooo`3<o`080?ooo`03003oIP3o
ool0oooo00@0oooo00<006Ko0?ooo`3oool02@3oool00`00I_l0oooo0?ooo`0I0?ooo`0303<0o`3o
ool0oooo00`0oooo0`0c0?m60?ooo`@0<`3o503oool203<0o`L0oooo100c0?l00`00I_l0oooo0?oo
o`040?ooo`03001Vo`3oool0oooo00<0oooo00@00?mV0?ooo`3oool00?mV0P3<o`050?l000@00000
o`3ooom]0?ooo`@00000001W0?ooo`030000003oool0oooo0:P0oooo1@00000?0?ooo`030?l0003o
ool0oooo0080oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo0?ooo`060?ooo`050?l0003o
ool0oooo0?ooo`3<o`00203oool00`00ofH0oooo0?ooo`040?ooo`8006Ko2P3oool00`00I_l0oooo
0?ooo`0I0?ooo`80<`3o3`3oool503<0oc/0oooo1P0c0?lF0?ooo`80<`3o2`3oool00`0c0?l006Ko
0?ooo`050?ooo`03001Vo`3oool0oooo00<0oooo00@00?mV0?ooo`3oool00?mV0P3<o`060?l000<0
0000o`3oooma0?ooo`400000001W0?ooo`030000003oool0oooo0:P0oooo1P00000>0?ooo`030?l0
003oool0oooo00<0oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo0?ooo`060?ooo`040?l0
003oool0oooo0<co00P0oooo00<00?mV0?ooo`3oool01P3oool00`00I_l0oooo0?ooo`080?ooo`03
001Vo`3oool0oooo01X0oooo0P0c0?lB0?ooo`H0<`3o;P3oool703<0oaX0oooo0P0c0?l<0?ooo`80
06Ko1P3oool00`00I_l0oooo0?ooo`030?ooo`06003oIP3oool0oooo003oIP3<o`00oooo1`3o0003
00000?l0ooooL@3oool100000000I`3oool00`000000oooo0?ooo`2Y0?ooo`D000003`3oool00`3o
0000oooo0?ooo`020?ooo`030<co003oool0oooo00L0oooo00<00?mV0?ooo`3oool01P3oool01@3o
0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV0?ooo`3oool01P3oool00`00I_l0oooo0?ooo`08
0?ooo`03001Vo`3oool0oooo01/0oooo0P0c0?lF0?ooo`P0<`3o8P3oool403<0oal0oooo0P0c0?l<
0?ooo`8006Ko1P3oool2001Vo`D0oooo00T00?mV0?ooo`3oool00?mV0<co003oool0c?l00?l0003o
ool01@3o000200000?l0ooooLP3oool100000000I`3oool00`000000oooo0?ooo`2Y0?ooo`H00000
3P3oool00`3o0000oooo0?ooo`030?ooo`030<co003oool0oooo00L0oooo00<00?mV0?ooo`3oool0
1@3oool01@3o0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV0?ooo`3oool01`3oool2001Vo`T0
oooo00<006Ko0?ooo`3oool0703oool00`0c0?l0oooo0?ooo`0:0?ooo`D0<`3o303ooolR03<0ob40
oooo0P0c0?l=0?ooo`03001Vo`3oool0oooo00D0oooo00<006Ko0?ooo`3oool0103oool01`00ofH0
oooo0?ooo`00ofH0c?l00?ooo`3<o`000P3o00020?ooo`030000003o0000000000<00000o`3ooomb
0?ooo`400000001W0?ooo`030000003oool0oooo0:X0oooo1@00000?0?ooo`030?l0003oool0oooo
00<0oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo0?ooo`050?ooo`040?l0003oool0oooo
0<co00T0oooo00<00?mV0?ooo`3oool0203oool00`00I_l0oooo0?ooo`070?ooo`03001Vo`3oool0
oooo01`0oooo0P0c0?l:0?ooo`D0<`3oC03oool303<0o`d0oooo0P00I_l70?ooo`03001Vo`3oool0
oooo00@0oooo00L00?mV0?ooo`3oool0oooo003oIP3<o`00oooo0080o`000P3oool600000?l0oooo
L`3oool100000000I`3oool00`000000oooo0?ooo`2Z0?ooo`D00000403oool00`3o0000oooo0?oo
o`020?ooo`030<co003oool0oooo00P0oooo00<00?mV0?ooo`3oool0103oool01@3o0000oooo0?oo
o`3oool0c?l000P0oooo00<00?mV0?ooo`3oool02@3oool2001Vo`L0oooo00<006Ko0?ooo`3oool0
7P3oool203<0o`L0oooo1P0c0?m:0?ooo`80<`3o3`3oool00`00I_l0oooo0?ooo`060?ooo`03001V
o`3oool0oooo00@0oooo00L00?mV0?ooo`3oool0oooo003oIP3<o`00oooo0080o`000`3oool60000
0?l0ooooL`3oool100000000I`3oool00`000000oooo0?ooo`2Z0?ooo`H000003`3oool00`3o0000
oooo0?ooo`030?ooo`030<co003oool0oooo00L0oooo00<00?mV0?ooo`3oool01@3oool0103o0000
oooo0?ooo`3<o`090?ooo`03003oIP3oool0oooo00X0oooo00<006Ko0?ooo`3oool01@3oool00`00
I_l0oooo0?ooo`0O0?ooo`80<`3o1P3oool503<0odL0oooo0`0c0?l?0?ooo`8006Ko203oool00`00
I_l0oooo0?ooo`040?ooo`03003oIP3oool0oooo00800?mV00<0c?l0003oIP3<o`000P3o00020?oo
o`H00000o`3ooomd0?ooo`400000001W0?ooo`030000003oool0oooo0:/0oooo1@00000@0?ooo`03
0?l0003oool0oooo00<0oooo00<0c?l00?ooo`3oool01`3oool00`00ofH0oooo0?ooo`040?ooo`05
0?l0003oool0oooo0?ooo`3<o`00203oool00`00ofH0oooo0?ooo`0;0?ooo`8006Ko1P3oool00`00
I_l0oooo0?ooo`0P0?ooo`<0<`3o0`3oool503<0odD0oooo0P0c0?l@0?ooo`8006Ko2@3oool00`00
I_l0oooo0?ooo`040?ooo`06003oIP3oool0oooo0?ooo`00ofH0c?l00P00ofH20?l000<0oooo1@00
003o0?ooogD0oooo0@00000006L0oooo00<000000?ooo`3oool0Z`3oool600000100oooo00<0o`00
0?ooo`3oool00P3oool00`3<o`00oooo0?ooo`080?ooo`03003oIP3oool0oooo00@0oooo00@0o`00
0?ooo`3oool0c?l02@3oool00`00ofH0oooo0?ooo`0<0?ooo`03001Vo`3oool0oooo00@0oooo00<0
06Ko0?ooo`3oool08P3oool203<0o`<0oooo00<0<`3o0?ooo`3oool0@P3oool303<0oa40oooo00<0
06Ko0?ooo`3oool0203oool00`00I_l0oooo0?ooo`040?ooo`04003oIP3oool0oooo0?ooo`800?mV
00@0c?l0003oIP3o0000o`000`3oool600000?l0ooooM@3oool100000000I`3oool00`000000oooo
0?ooo`2/0?ooo`D00000403oool00`3o0000oooo0?ooo`030?ooo`030<co003oool0oooo00P0oooo
00<00?mV0?ooo`3oool00`3oool01@3o0000oooo0?ooo`3oool0c?l000P0oooo00<00?mV0?ooo`3o
ool03@3oool2001Vo`D0oooo00<006Ko0?ooo`3oool08`3oool303<0od80oooo0`0c0?lB0?ooo`80
06Ko2@3oool2001Vo`L0oooo00P00?mV0?ooo`3oool0oooo003oIP3<o`000?mV0<co0080o`000`3o
ool500000?l0ooooMP3oool100000000I`3oool00`000000oooo0?ooo`2/0?ooo`H00000403oool0
0`3o0000oooo0?ooo`030?ooo`030<co003oool0oooo00P0oooo00<00?mV0?ooo`3oool00P3oool0
1@3o0000oooo0?ooo`3oool0c?l000T0oooo00<00?mV0?ooo`3oool03P3oool00`00I_l0oooo0?oo
o`030?ooo`03001Vo`3oool0oooo02D0oooo0`0c0?l^0?ooo`D0<`3o2@3oool303<0oa@0oooo00<0
06Ko0?ooo`3oool0203oool00`00I_l0oooo0?ooo`060?ooo`06003oIP3oool0oooo0?ooo`00ofH0
c?l00P00ofH20?l00003003oIP3oool0oooo00H00000o`3ooomf0?ooo`400000001W0?ooo`030000
003oool0oooo0:d0oooo1@00000@0?ooo`030?l0003oool0oooo00<0oooo00<0c?l00?ooo`3oool0
2@3oool00`00ofH0oooo0?ooo`020?ooo`050?l0003oool0oooo0?ooo`3<o`002@3oool00`00ofH0
oooo0?ooo`0>0?ooo`8006Ko103oool00`00I_l0oooo0?ooo`0W0?ooo`@0<`3o:P3oool503<0o`D0
oooo100c0?lE0?ooo`8006Ko2P3oool00`00I_l0oooo0?ooo`060?ooo`06003oIP3oool0oooo0?oo
o`00ofH0c?l00P3oool20?l000@0oooo1@00003o0?ooogL0oooo0@00000006L0oooo00<000000?oo
o`3oool0[@3oool500000140oooo00<0o`000?ooo`3oool00`3oool00`3<o`00oooo0?ooo`080?oo
o`03003oIP3oool0oooo0080oooo00D0o`000?ooo`3oool0oooo0<co00090?ooo`03003oIP3oool0
oooo0100oooo00<006Ko0?ooo`3oool00P3oool2001Vob/0oooo0`0c0?lV0?ooo`H0<`3o00<0oooo
03<0o`0c0?l00P0c0?lG0?ooo`8006Ko2P3oool2001Vo`P0oooo00H00?mV0?ooo`3oool0oooo003o
IP3<o`020?ooo`030?l0003oool0o`0000<0oooo1P00003o0?ooogL0oooo0@00000006L0oooo00<0
00000?ooo`3oool0[@3oool600000140oooo00<0o`000?ooo`3oool00`3oool00`3<o`00o`000?l0
00030?l000D0oooo00<00?mV0?ooo`3oool00P3oool01@3o0000oooo0?ooo`3oool0c?l000T0oooo
00<00?mV0?ooo`3oool0403oool2001Vo`@0oooo00<006Ko0?ooo`3oool0:`3oool403<0ob80oooo
1`0c0?lI0?ooo`8006Ko2`3oool00`00I_l0oooo0?ooo`070?ooo`03003oIP3oool0oooo0080oooo
00@00?mV0<co003oool0c?l00P3o00040?ooo`D00000o`3ooomh0?ooo`400000001W0?ooo`030000
003oool0oooo0:h0oooo1@00000A0?ooo`030?l0003oool0oooo00<0oooo00<0c?l00?l0003o0000
0`3o00060?ooo`05003oIP3oool0oooo0?ooo`3o00000`3oool00`3<o`00oooo0?ooo`070?ooo`03
003oIP3oool0oooo0180oooo00<006Ko0?ooo`3oool00P3oool00`00I_l0oooo0?ooo`0^0?ooo`L0
<`3o5`3oool403<0o`030?ooo`0c0?l0<`3o00<0<`3o603oool2001Vo``0oooo00<006Ko0?ooo`3o
ool01`3oool00`00ofH0oooo0?ooo`020?ooo`04003oIP3<o`00oooo0?ooo`80o`00103oool60000
0?l0ooooN03oool100000000I`3oool00`000000oooo0?ooo`2^0?ooo`H000004@3oool00`3o0000
oooo0?ooo`020?ooo`H0o`001`3oool01000ofH0oooo0?ooo`3o00040?ooo`030<co003oool0oooo
00L0oooo00<00?mV0?ooo`3oool04P3oool2001Vo`<0oooo00<006Ko0?ooo`3oool0=03ooolG03<0
o`L0oooo00<0<`3o0?ooo`3oool05P3oool2001Vo`d0oooo00<006Ko0?ooo`3oool01`3oool00`00
ofH0oooo0?ooo`020?ooo`08003oIP3<o`00oooo0<co003o0000c?l00?l0003<o`020?ooo`H00000
o`3ooomi0?ooo`400000001W0?ooo`@00000[P3oool500000180oooo00<0o`000?ooo`3oool00P3o
ool50?l000P0oooo00@00?mV0?ooo`3oool0o`000`3oool00`3<o`00oooo0?ooo`070?ooo`03003o
IP3oool0oooo01@0oooo0P00I_l20?ooo`D006KoIP3oool2001Vo`d0oooo0P00I_l90?ooo`03003o
IP3oool0oooo0080oooo00L00?mV0<co003oool0oooo0?l0003<o`00o`000080c?l00P3oool60000
0?l0ooooMP3oool400000000I`3oool00`000000oooo0?ooo`2_0?ooo`H000004@3oool00`3o0000
oooo0?ooo`020?ooo`D0o`002@3oool00`00ofH0oooo0?l000040?ooo`030<co003oool0oooo00L0
oooo00<00?mV0?ooo`3oool05@3oool7001Vof@0oooo0P00I_l>0?ooo`03001Vo`3oool0oooo00P0
oooo00<00?mV0?ooo`3oool00P3oool01@00ofH0oooo0<co003oool0c?l00080o`000`3<o`000`3o
ool000000000000400000?l0ooooNP3oool100000000I`3oool00`000000oooo0?ooo`2`0?ooo`D0
00004P3oool00`3o0000oooo0?ooo`030?ooo`030?l0003oool0oooo00D0oooo1@3<o`020?ooo`03
0?l0003oool0oooo0080oooo00<0c?l00?ooo`3oool01P3oool00`00ofH0oooo0?ooo`0F0?ooo`H0
06KoHP3oool2001Vo`l0oooo00<006Ko0?ooo`3oool01`3oool2003oIPD0oooo00L00?mV0<co003o
ool0oooo0?l0003<o`00o`0000<0c?l000<0oooo0000000000000`00003o0?ooog/0oooo0@000000
06L0oooo00<000000?ooo`3oool0/03oool600000140oooo00<0o`000?ooo`3oool0103oool00`3<
o`00oooo0?ooo`040?ooo`D0c?l000<00?mV0?ooo`3o0000103oool00`3<o`00oooo0?ooo`070?oo
o`03003oIP3oool0oooo01H0oooo1@00I_mP0?ooo`8006Ko3`3oool2001Vo`T0oooo00<00?mV0?oo
o`3oool0103oool01`00ofH0c?l00?ooo`3oool0o`000?ooo`3o0000103<o`0600000?l0ooooN`3o
ool100000000I`3oool00`000000oooo0?ooo`2a0?ooo`D000004P3oool00`3o0000oooo0?ooo`03
0?ooo`030<co003oool0oooo00<0oooo1P3<o`000`3oool00?mV0?l000050?ooo`030<co003oool0
oooo00H0oooo00<00?mV0?ooo`3oool05P3oool8001VoeX0oooo0`00I_l?0?ooo`8006Ko2P3oool0
0`00ofH0oooo0?ooo`040?ooo`03003oIP3<o`00oooo0080oooo0P3o00020?ooo`030<co003oool0
oooo00D00000o`3oooml0?ooo`400000001W0?ooo`030000003oool0oooo0;40oooo1P00000B0?oo
o`030?l0003oool0oooo00<0oooo00<0c?l00?ooo`3oool00`3oool50<co0080oooo00<00?mV0?l0
003oool00`3oool00`3<o`00oooo003oIP04003oIP<0oooo00<00?mV0?ooo`3oool05`3oool01P00
I_l0oooo0?ooo`3oool006Ko0?ooo`<006KoE03oool3001Vo`l0oooo1@00I_l90?ooo`03003oIP3o
ool0oooo00@0oooo00<00?mV0<co003oool00P3oool60?l000030?ooo`000000000000@00000o`3o
ooml0?ooo`400000001W0?ooo`030000003oool0oooo0;80oooo1@00000B0?ooo`030?l0003oool0
oooo00@0oooo00<0c?l00?ooo`3oool00P3oool50<co00<0oooo00<0o`000?ooo`3oool00`3oool0
0`3<o`000?mV003oIP03003oIP@0oooo00<00?mV0?ooo`3oool06`3oool01000I_l0oooo0?ooo`3o
ool2001Voe00oooo0P00I_lA0?ooo`H006Ko203oool00`00ofH0oooo0?ooo`040?ooo`03003oIP3<
o`00oooo0080oooo00<0o`000?ooo`3o0000103o00000`3oool000000000000300000?l0ooooO@3o
ool100000000I`3oool00`000000oooo0?ooo`2b0?ooo`H000004P3oool00`3o0000oooo0?ooo`04
0?ooo`030<co003oool0oooo00<0oooo00<0c?l00?ooo`3oool0103oool00`3o0000oooo0?ooo`02
0?ooo`03003oIP3<o`000?mV00<00?mV1@3oool00`00ofH0oooo0?ooo`0K0?ooo`03001Vo`3oool0
oooo0080oooo0`00I_m:0?ooo`<006Ko4P3oool7001Vo`H0oooo0P00ofH60?ooo`03003oIP3<o`00
oooo0080oooo00<0o`000?ooo`3o00001@3o000600000?l0ooooO@3oool100000000I`3oool00`00
0000oooo0?ooo`2c0?ooo`D000004`3oool00`3o0000oooo0?ooo`030?ooo`030<co003oool0oooo
00X0oooo00<0o`00003oIP3oool00`3oool00`3<o`000?mV003oIP02003oIPD0oooo00<00?mV0?oo
o`3oool0703oool2001Vo`D0oooo0`00I_m30?ooo`@006Ko4`3oool2001Vo`80oooo1@00I_l50?oo
o`03003oIP3oool0oooo00D0oooo00<00?mV0<co003oool00`3oool00`3o0000oooo0?l000050?l0
00D00000o`3ooomn0?ooo`400000001W0?ooo`030000003oool0oooo0;<0oooo1P00000B0?ooo`03
0?l0003oool0oooo00@0oooo00<0c?l00?ooo`3oool02P3oool00`3o00000?mV003oIP020?ooo`03
003oIP3<o`000?mV00800?mV1P3oool00`00ofH0oooo0?ooo`0M0?ooo`03001Vo`3oool0oooo00D0
oooo0P00I_lm0?ooo`@006Ko5@3oool2001Vo`@0oooo1@00I_l40?ooo`03003oIP3oool0oooo00D0
oooo00<00?mV0<co003oool00`3oool0103o0000oooo0?l0003oool50?l000D00000o`3ooomn0?oo
o`400000001W0?ooo`030000003oool0oooo0;@0oooo1P00000B0?ooo`030?l0003oool0oooo00@0
oooo00<0c?l00?ooo`3oool02@3oool0103o0000oooo0?ooo`00ofH20?ooo`030<co0000ofH0oooo
00P0oooo00<00?mV0?ooo`3oool07@3oool2001Vo`L0oooo0`00I_lf0?ooo`@006Ko5`3oool2001V
o`P0oooo00<006Ko0?ooo`3oool00`3oool00`00ofH0oooo0?ooo`050?ooo`03003oIP3<o`00oooo
00<0oooo00@0o`000?ooo`3oool0o`000`3oool00`3o000000000000000400000?l0ooooO`3oool1
00000000I`3oool00`000000oooo0?ooo`2d0?ooo`H000004P3oool00`3o0000oooo0?ooo`050?oo
o`030<co003oool0oooo00T0oooo00@0o`000?ooo`3oool00?mV0P3oool00`3<o`00oooo0?ooo`08
0?ooo`03003oIP3oool0oooo01h0oooo0P00I_l80?ooo`D006Ko;@3oool4001VoaT0oooo0P00I_l?
0?ooo`03003oIP3oool0oooo00D0oooo00<00?mV0<co003oool00`3oool0103o0000c?l00?ooo`3o
00050?ooo`H00000o`3ooomo0?ooo`400000001W0?ooo`030000003oool0oooo0;D0oooo1P00000B
0?ooo`030?l0003oool0oooo00@0oooo00<0c?l00?ooo`3oool02@3oool01`3o0000oooo0?ooo`3o
ool00?mV0?ooo`3<o`002P3oool00`00ofH0oooo0?ooo`0P0?ooo`03001Vo`3oool0oooo00X0oooo
1P00I_lS0?ooo`@006Ko6`3oool2001Vo`l0oooo0P00ofH70?ooo`03003oIP3<o`00oooo00<0oooo
00@0o`000<co003oool0o`001@3oool600000?l0ooooP03oool100000000I`3oool00`000000oooo
0?ooo`2e0?ooo`H000004`3oool00`3o0000oooo0?ooo`040?ooo`030<co003oool0oooo00T0oooo
00L0o`000?ooo`3oool0oooo003oIP3oool0c?l000X0oooo00<00?mV0?ooo`3oool0803oool2001V
oa00oooo2000I_lD0?ooo`L006Ko7@3oool2001Voa00oooo00<00?mV0?ooo`3oool01P3oool00`00
ofH0c?l00?ooo`030?ooo`050?l0003<o`00oooo0?ooo`3o00001@3oool600000?l0ooooP03oool1
00000000I`3oool00`000000oooo0?ooo`2f0?ooo`H000004P3oool00`3o0000oooo0?ooo`050?oo
o`030<co003oool0oooo00P0oooo00<0o`000?ooo`3oool00P3oool00`00ofH0oooo0<co000:0?oo
o`03003oIP3oool0oooo0240oooo0P00I_lF0?oooa@006Ko8P3oool2001Voa40oooo00<00?mV0?oo
o`3oool01P3oool00`00ofH0c?l00?ooo`040?ooo`040?l0003oool0oooo0?l000D0oooo1P00003o
0?oooh40oooo0@00000006L0oooo00<000000?ooo`3oool0]P3oool6000001<0oooo00<0o`000?oo
o`3oool01@3oool00`3<o`00oooo0?ooo`080?ooo`030?l0003oool0oooo0080oooo00<00?mV0<co
003oool02P3oool00`00ofH0oooo0?ooo`0R0?ooo`8006KoB03oool2001Voa80oooo00<00?mV0?oo
o`3oool01P3oool00`00ofH0c?l00?ooo`040?ooo`040?l0003oool0oooo0?l000H0oooo1@00003o
0?oooh80oooo0@00000006L0oooo1000002f0?ooo`H000004`3oool00`3o0000oooo0?ooo`040?oo
o`030<co003oool0oooo00P0oooo00<0o`000?ooo`3oool00`3oool00`00ofH0c?l00?ooo`090?oo
o`03003oIP3oool0oooo02@0oooo0P00I_m40?ooo`8006Ko4`3oool00`00ofH0oooo0?ooo`060?oo
o`03003oIP3<o`00oooo00@0oooo00D0o`000?ooo`3oool0oooo0?l000050?ooo`H00000o`3ooomo
0?ooo`@00000001W0?ooo`030000003oool0oooo0;L0oooo1P00000D0?ooo`030?l0003oool0oooo
00@0oooo00<0c?l00?ooo`3oool0203oool00`3o0000oooo0?ooo`030?ooo`030<co0000ofH0oooo
00T0oooo00<00?mV0?ooo`3oool09@3oool00`00I_l0oooo0?ooo`0o0?ooo`8006Ko4`3oool2003o
IPP0oooo00<00?mV0<co003oool0103oool01@3o0000c?l00?ooo`3oool0o`0000H0oooo1@00003o
0?oooh<0oooo0@00000006L0oooo00<000000?ooo`3oool0^03oool6000001<0oooo00<0o`000?oo
o`3oool01@3oool00`3<o`00oooo0?ooo`080?ooo`030?l0003oool0oooo00<0oooo00<0c?l0003o
IP3oool02@3oool00`00ofH0oooo0?ooo`0U0?ooo`8006Ko2P3oool5001Vobd0oooo0`00I_lD0?oo
o`03003oIP3oool0oooo00H0oooo0P00ofH00`3<o`000?mV003oIP030?ooo`050?l0003<o`00oooo
0?ooo`3o00001P3oool600000?l0ooooP`3oool100000000I`3oool00`000000oooo0?ooo`2i0?oo
o`D00000503oool00`3o0000oooo0?ooo`050?ooo`030<co003oool0oooo00L0oooo00<0o`000?oo
o`3oool0103oool00`3<o`000?mV0?ooo`090?ooo`03003oIP3oool0oooo02H0oooo0P00I_l80?oo
o`D006Ko803oool5001Vo`D0oooo0`00I_lF0?ooo`03003oIP3oool0oooo00H0oooo0P00ofH0103<
o`000?mV003oIP00ofH20?ooo`050?l0003<o`00oooo0?ooo`3o00001`3oool500000?l0ooooQ03o
ool100000000I`3oool00`000000oooo0?ooo`2i0?ooo`H00000503oool00`3o0000oooo0?ooo`04
0?ooo`D0o`001P3oool00`3o0000oooo0?ooo`030?ooo`030<co003oool00?mV00T0oooo00<00?mV
0?ooo`3oool0:03oool2001Vo`D0oooo1P00I_lP0?ooo`D006Ko0`3oool2001VoaP0oooo00<00?mV
0?ooo`3oool01P3oool2003oIP030<co0000ofH00?mV00800?mV00<0oooo0?l0003<o`000`3oool0
0`3o0000oooo0?ooo`040?ooo`H00000o`3ooon40?ooo`400000001W0?ooo`030000003oool0oooo
0;X0oooo1@00000E0?ooo`030?l0003oool0oooo00<0oooo1@3o00060?ooo`030?l0003oool0oooo
00@0oooo00<0c?l00?ooo`00ofH02@3oool00`00ofH0oooo0?ooo`0Y0?ooo`<006Ko0`3oool5001V
oal0oooo2@00I_lI0?ooo`03003oIP3oool0oooo00H0oooo00<00?mV0?ooo`3<o`001@00ofH00`3o
0000c?l00?ooo`020?ooo`030?l0003oool0oooo00@0oooo1P00003o0?ooohD0oooo0@00000006L0
oooo00<000000?ooo`3oool0^P3oool6000001@0oooo00<0o`000?ooo`3oool00P3oool60?l000L0
oooo00<0o`000?ooo`3oool00`3oool00`3<o`00oooo0?ooo`02003oIPP0oooo00<00?mV0?ooo`3o
ool0:`3oool8001Vob00oooo1@00I_lJ0?ooo`800?mV203oool01000ofH0oooo0<co003oool4003o
IP030?l0003<o`00oooo0080oooo00<0o`000?ooo`3oool01@3oool600000?l0ooooQ@3oool10000
0000I`3oool00`000000oooo0?ooo`2k0?ooo`D000005@3oool00`3o0000oooo0?ooo`020?ooo`D0
o`001`3oool00`3o0000oooo0?ooo`040?ooo`050<co003oool0oooo0?ooo`00ofH0203oool00`00
ofH0oooo0?ooo`0^0?ooo`@006Ko7@3oool8001VoaT0oooo00<00?mV0?ooo`3oool01`3oool00`00
ofH0oooo0<co00040?ooo`03003oIP3oool0o`0000<0oooo00<0o`000?ooo`3oool01@3oool60000
0?l0ooooQP3oool100000000I`3oool00`000000oooo0?ooo`2k0?ooo`H000005@3oool0103o0000
oooo0?ooo`3oool50?l000P0oooo00<0o`000?ooo`3oool0103oool0103<o`00oooo0?ooo`3oool2
003oIPH0oooo00<00?mV0?ooo`3oool0<P3oool:001Vo`X0oooo2@00I_l50?ooo`03001Vo`3oool0
oooo01P0oooo00<00?mV0?ooo`3oool01`3oool00`00ofH0oooo0<co00060?ooo`050?l0003<o`00
c?l00<co003o0000203oool600000?l0ooooQP3oool100000000I`3oool00`000000oooo0?ooo`2l
0?ooo`D000005P3oool00`3o0000oooo0?ooo`020?ooo`040?l0003oool0oooo0<co00L0oooo00<0
o`000?ooo`3oool0103oool00`3<o`00oooo0?ooo`030?ooo`03003oIP3oool0oooo00@0oooo00<0
0?mV0?ooo`3oool0>`3oool:001VobP0oooo00<00?mV0?ooo`3oool01`3oool00`00ofH0oooo0<co
00060?ooo`030?l0003<o`00c?l00080c?l000<0o`000?ooo`3oool01@3oool600000?l0ooooQ`3o
ool100000000I`3oool00`000000oooo0?ooo`2l0?ooo`H000005@3oool00`3o0000oooo0?ooo`06
0?ooo`030<co003oool0oooo00@0oooo00<0c?l00?l0003<o`000P3<o`030?ooo`030<co003oool0
oooo00<0oooo00<00?mV0?ooo`3oool0103oool00`00ofH0oooo0?ooo`1Z0?ooo`800?mV2@3oool0
0`00ofH0oooo0<co00060?ooo`030?l0003<o`00c?l00080c?l000<0o`000<co003oool01P3oool5
00000?l0ooooR03oool100000000I`3oool00`000000oooo0?ooo`2m0?ooo`D000005P3oool00`3o
0000oooo0?ooo`060?ooo`030<co003oool0oooo00<0oooo0P3<o`000`3o0000c?l00<co00040?oo
o`030<co003oool0oooo00<0oooo0P00ofH50?ooo`03003oIP3oool0oooo06L0oooo0P00ofH:0?oo
o`03003oIP3oool0c?l000H0oooo00<0o`000<co003oool00P3<o`050?l000@0oooo1P00003o0?oo
ohP0oooo0@00000006L0oooo00<000000?ooo`3oool0_@3oool6000001H0oooo00<0o`000?ooo`3o
ool01P3oool0103<o`00oooo0?ooo`3oool30<co00030?l0003<o`00c?l000@0oooo00<0c?l00?oo
o`3oool01@3oool00`00ofH0oooo0?ooo`030?ooo`03003oIP3oool0oooo06D0oooo00<00?mV0?oo
o`3oool02@3oool00`00ofH0oooo0<co00060?ooo`050?l0003<o`00oooo0?ooo`3<o`001P3o0004
0?ooo`D00000o`3ooon90?ooo`400000001W0?ooo`030000003oool0oooo0;h0oooo1@00000G0?oo
o`030?l0003oool0oooo00H0oooo00@0c?l00?ooo`3oool0oooo0`3<o`000`3o0000c?l00?ooo`04
0?ooo`030<co003oool0oooo00D0oooo0P00ofH30?ooo`D00?mVH@3oool2003oIP/0oooo00<00?mV
0?ooo`3<o`001P3oool00`3o0000c?l00?ooo`030?ooo`H0o`000`3oool600000?l0ooooR@3oool1
00000000I`3oool00`000000oooo0?ooo`2n0?ooo`H000005P3oool00`3o0000oooo0?ooo`070?oo
o`030<co003oool0oooo00<0c?l000<0o`000<co003oool01@3oool00`3<o`00oooo0?ooo`060?oo
o`03003oIP3oool0oooo00D00?mVH03oool00`00ofH0oooo0?ooo`0:0?ooo`03003oIP3oool0c?l0
00H0oooo00<0o`000<co003oool00`3oool00`3o0000oooo0?l000040?l00080oooo1P00003o0?oo
ohX0oooo0@00000006L0oooo00<000000?ooo`3oool0_`3oool5000001L0oooo00<0o`000?ooo`3o
ool01`3oool20<co0080oooo00<0c?l00?ooo`3o00001P3oool00`3<o`00oooo0?ooo`070?ooo`L0
0?mVGP3oool2003oIP`0oooo00<00?mV0?ooo`3<o`001P3oool00`3o0000c?l00?ooo`030?ooo`03
0?l0003oool0oooo00D0o`000P3oool600000?l0ooooRP3oool100000000I`3oool400000;h0oooo
1P00000G0?ooo`030?l0003oool0oooo00P0oooo00D0c?l00?ooo`3oool0oooo0?l000070?ooo`03
0<co003oool0oooo00P0oooo1@00ofIL0?ooo`800?mV3@3oool00`00ofH0oooo0<co00070?ooo`03
0?l0003oool0oooo0080oooo00<0o`000?ooo`3oool00`3oool0103o0000oooo0?ooo`3oool60000
0?l0ooooR03oool400000000I`3oool00`000000oooo0?ooo`300?ooo`D00000603oool00`3o0000
oooo0?ooo`080?ooo`050<co003oool0oooo0?ooo`3o00001`3oool00`3<o`00oooo0?ooo`070?oo
o`H00?mVFP3oool00`00ofH0oooo0?ooo`0<0?ooo`03003oIP3oool0c?l000L0oooo00<0o`000?oo
o`3oool00P3oool00`3o0000oooo0?ooo`070?ooo`H00000o`3ooon<0?ooo`400000001W0?ooo`03
0000003oool0oooo0<00oooo1P00000H0?ooo`030?l0003oool0oooo00P0oooo00@0c?l00?ooo`3o
ool0o`001`3oool00`3<o`00oooo0?ooo`090?ooo`800?mV0P3oool00`00ofH0oooo0?ooo`1E0?oo
o`800?mV3P3oool00`00ofH0oooo0<co00070?ooo`030?l0003<o`00oooo00<0oooo00<0o`000?oo
o`3oool01`3oool600000?l0ooooS03oool100000000I`3oool00`000000oooo0?ooo`310?ooo`D0
00006@3oool00`3o0000oooo0?ooo`080?ooo`040<co003oool0oooo0?l000L0oooo00<0c?l00?oo
o`3oool02P3oool2003oIP030?ooo`00ofH0oooo05<0oooo0P00ofH?0?ooo`03003oIP3oool0c?l0
00L0oooo00<0o`000<co003oool00`3oool00`3o0000oooo0?ooo`070?ooo`H00000o`3ooon=0?oo
o`400000001W0?ooo`030000003oool0oooo0<40oooo1P00000I0?ooo`030?l0003oool0oooo00P0
oooo00@0c?l00?ooo`3oool0o`001`3oool00`3<o`00oooo0?ooo`0;0?ooo`<00?mVD@3oool2003o
IPl0oooo0P00ofH00`3oool0c?l00?ooo`060?ooo`030?l0003<o`00oooo00<0oooo00<0o`000?oo
o`3oool01`3oool600000?l0ooooSP3oool100000000I`3oool00`000000oooo0?ooo`320?ooo`D0
00006P3oool00`3o0000oooo0?ooo`080?ooo`030<co003oool0o`0000L0oooo00<0c?l00?ooo`3o
ool03@3oool2003oITh0oooo0P00ofH@0?ooo`04003oIP3oool0oooo0<co00L0oooo00<0o`000<co
003oool00`3oool00`3o0000oooo0?ooo`080?ooo`H00000o`3ooon>0?ooo`400000001W0?ooo`03
0000003oool0oooo0<80oooo1P00000J0?ooo`030?l0003oool0oooo00P0oooo00<0c?l00?ooo`3o
00001`3oool00`3<o`00oooo0?ooo`0>0?ooo`800?mVB@3oool3003oIP`0oooo1P00ofH00`3oool0
c?l00<co00070?ooo`030?l0003<o`00oooo00<0oooo00<0o`000?ooo`3oool0203oool600000?l0
ooooS`3oool100000000I`3oool00`000000oooo0?ooo`330?ooo`D000006`3oool00`3o0000oooo
0?ooo`030?ooo`D0o`0000<0c?l00?ooo`3o00001`3oool00`3<o`00oooo0?ooo`0?0?ooo`800?mV
A03oool3003oIPl0oooo1@00ofH00`3oool0c?l00?ooo`070?ooo`030?l0003<o`00oooo00@0oooo
00<0o`000?ooo`3oool01`3oool600000?l0ooooT03oool100000000I`3oool00`000000oooo0?oo
o`330?ooo`H000006`3oool00`3o0000oooo0?ooo`020?ooo`D0o`0000<0oooo0<co003o00001`3o
ool00`3<o`00oooo0?ooo`0@0?ooo`@00?mV?`3oool2003oIQ40oooo1P00ofH00`3<o`00oooo0?oo
o`060?ooo`030?l0003<o`00oooo00@0oooo00<0o`000?ooo`3oool0203oool600000?l0ooooT03o
ool100000000I`3oool00`000000oooo0?ooo`340?ooo`H000006`3oool00`3o0000oooo0?ooo`06
0?l00080oooo00<0c?l00?l0003oool01P3oool00`3<o`00oooo0?ooo`0@0?ooo`03003oIP3oool0
oooo00@00?mV>03oool3003oIQ@0oooo1000ofH00`3<o`00oooo0?ooo`070?ooo`030?l0003oool0
oooo00<0oooo00<0o`000?ooo`3oool0203oool600000?l0ooooT@3oool100000000I`3oool00`00
0000oooo0?ooo`340?ooo`H00000703oool00`3o0000oooo0?ooo`050?l000@0oooo00<0o`000?oo
o`3oool01@3oool00`3<o`00oooo0?ooo`0@0?ooo`03003oIP3oool0oooo00<0oooo0`00ofHb0?oo
o`<00?mV5P3oool3003oIP80c?l000<00?mV0?ooo`3oool01P3oool00`3o0000oooo0?ooo`030?oo
o`030?l0003oool0oooo00T0oooo1P00003o0?oooi40oooo0@00000006L0oooo00<000000?ooo`3o
ool0a@3oool6000001`0oooo00<0o`000?ooo`3o0000103o00040?ooo`030?l0003<o`00oooo00D0
oooo00<0c?l00?ooo`3oool04@3oool00`00ofH0oooo0?ooo`050?ooo`800?mV;@3oool3003oIQL0
oooo0P00ofH20?ooo`030<co0000ofH0oooo00P0oooo00<0o`000<co003oool00`3oool00`3o0000
oooo0?ooo`090?ooo`H00000o`3ooonB0?ooo`400000001W0?ooo`030000003oool0oooo0<D0oooo
1P00000M0?ooo`040?l0003oool0oooo0?l000L0oooo00<0o`000<co003<o`001@3oool00`3<o`00
oooo0?ooo`0A0?ooo`800?mV1`3oool3003oIRL0oooo0`00ofHI0?ooo`05003oIP3oool0oooo0?oo
o`3<o`002@3oool00`3<o`00o`000<co00020<co00<0oooo00<0o`000?ooo`3oool0203oool60000
0?l0ooooT`3oool100000000A03oool3000000H0oooo0P0000020?ooo`H00000403oool00`000000
oooo0?ooo`360?ooo`H000007@3oool00`3o0000oooo0?ooo`080?ooo`030?l0003oool0c?l000D0
oooo00<0c?l00?ooo`3oool04P3oool00`00ofH0oooo0?ooo`030?ooo`/00?mV703oool4003oIQX0
oooo0P00ofH30?ooo`030<co003oool0oooo00P0oooo00<0o`000<co003<o`000P3<o`000`3oool0
o`000?l000030?l000P0oooo1P00003o0?oooi<0oooo0@00000004<0oooo00D000000?ooo`3oool0
oooo000000050?ooo`8000000P3oool2000000<0oooo0`00000>0?ooo`030000003oool0oooo0<H0
oooo1P00000N0?ooo`030?l0003oool0oooo00L0oooo00@0o`000?ooo`3oool0c?l00P3oool50<co
01<0oooo00<00?mV0?ooo`3oool00P3oool5003oIPH0oooo2000ofH90?ooo`/00?mV703oool2003o
IP@0oooo00<0c?l00?ooo`3oool0203oool00`3o0000c?l00<co00030<co00030?ooo`3o0000o`00
00<0o`001`3oool600000?l0ooooU03oool100000000@P3oool00`000000oooo0?ooo`020?ooo`80
00003P3oool2000000h0oooo00<000000?ooo`3oool0a`3oool6000001h0oooo00<0o`000?ooo`3o
ool01`3oool00`3o0000oooo0?ooo`070<co01@0oooo0P00ofH00`3oool00?mV003oIP04003oIPh0
oooo2@00ofHU0?ooo`800?mV103oool20<co00T0oooo0P3o00060<co00H0o`001P3oool600000?l0
ooooU@3oool100000000@P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00d0oooo
0P00000=0?ooo`030000003oool0oooo0<L0oooo1P00000O0?ooo`030?l0003oool0oooo00H0oooo
00@0o`000?ooo`3oool0oooo1`3<o`0E0?ooo`03003oIP3oool00?mV00@00?mV>P3oool2003oIPD0
oooo00<0c?l00?ooo`3oool0203oool0103o0000oooo0<co003oool50<co00H0o`001P3oool60000
0?l0ooooU@3oool100000000@@3oool2000000D0oooo0P00000>0?ooo`8000003@3oool600000<D0
oooo1P00000O0?ooo`030?l0003oool0oooo00H0oooo00@0o`000?ooo`3oool0oooo1@3<o`000`3o
ool0c?l00?ooo`0D0?ooo`H00?mV>03oool2003oIPD0oooo0P3<o`0:0?ooo`030?l0003oool0c?l0
00@0oooo00@0c?l00?ooo`3o0000oooo1@3o00050?ooo`H00000o`3ooonA0?ooo`H0000000110?oo
o`8000001@3oool2000000h0oooo0P00000=0?ooo`030000003oool0oooo0<T0oooo1@00000P0?oo
o`030?l0003oool0oooo00H0oooo00<0o`000?ooo`3oool01@3<o`020?ooo`030<co003oool0oooo
01@0oooo0P00ofH_0?ooo`D00?mV103oool2003oIPD0oooo0P3<o`0;0?ooo`030?l0003oool0c?l0
00H0oooo00<0o`000?ooo`3oool00P3oool00`3o0000oooo0?ooo`040?ooo`H00000o`3ooonG0?oo
o`40000000110?ooo`8000001@3oool2000000d0oooo0`00000=0?ooo`030000003oool0oooo0<T0
oooo1P00000P0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3oool00P3oool00`3<o`00
oooo0<co00030?ooo`030<co003oool0oooo01D0oooo0P00ofH]0?ooo`D00?mV00@0oooo003oIP00
ofH00?mV1@3oool20<co00`0oooo00<0o`000?ooo`3<o`001P3oool00`3o0000oooo0?ooo`0:0?oo
o`H00000o`3ooonG0?ooo`40000000110?ooo`8000001@3oool2000000`0oooo1000000=0?ooo`03
0000003oool0oooo0<X0oooo1@00000Q0?ooo`030?l0003oool0oooo00D0oooo00<0o`000?ooo`3o
ool0103oool20<co0080oooo00<0c?l00?ooo`3oool05P3oool2003oIRX0oooo1`00ofH60?ooo`80
c?l0303oool20?l000030?ooo`3<o`00oooo00D0oooo00<0o`000?ooo`3oool02P3oool600000?l0
ooooV03oool100000000@@3oool2000000D0oooo0P00000:0?ooo`D000003P3oool00`000000oooo
0?ooo`3:0?ooo`H000008@3oool00`3o0000oooo0?ooo`050?ooo`030?l0003oool0oooo00D0oooo
00@0c?l00?ooo`3oool0c?l06@3oool3003oIPl0oooo1@00ofHD0?ooo`D00?mV1@3oool20<co00d0
oooo00@0o`000?ooo`3<o`00c?l01`3oool00`3o0000oooo0?ooo`0:0?ooo`D00000o`3ooonI0?oo
o`40000000110?ooo`8000001@3oool2000000P0oooo1P00000?0?ooo`030000003oool0oooo0</0
oooo1@00000R0?ooo`80o`001@3oool00`3o0000oooo0?ooo`060?ooo`80c?l000<0oooo0<co003o
ool06P3oool2003oIPd0oooo1@00ofHC0?ooo`H00?mV0`3oool20<co00h0oooo00<0o`000?ooo`3<
o`00203oool00`3o0000oooo0?ooo`0:0?ooo`H00000o`3ooonI0?ooo`40000000110?ooo`800000
1@3oool2000000P0oooo0`00000B0?ooo`030000003oool0oooo0</0oooo1P00000S0?ooo`030?l0
003oool0oooo00<0oooo00<0o`000?ooo`3oool01`3oool30<co01`0oooo0`00ofH90?ooo`H00?mV
403oool3003oIP<0oooo00@00?mV0?ooo`3oool0oooo0P3<o`0?0?ooo`030?l0003oool0c?l000P0
oooo00<0o`000?ooo`3oool02P3oool600000?l0ooooVP3oool100000000@P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`3<0?ooo`D00000903oool0103o0000oooo0?ooo`3oool50?l000P0oooo0P3<o`0N0?ooo`H0
0?mV103oool5003oIP`0oooo1000ofH80?ooo`80c?l0403oool00`3o0000oooo0<co00080?ooo`03
0?l0003oool0oooo00/0oooo1P00003o0?oooiX0oooo0@0000000480oooo0P0000030?ooo`800000
2P3oool6000000h0oooo00<000000?ooo`3oool0c03oool6000002@0oooo00<0o`000?ooo`3oool0
1@3o000:0?ooo`80c?l08P3ooolE003oIPX0oooo0P3<o`0@0?ooo`80o`0000<0oooo0<co003oool0
1`3oool00`3o0000oooo0?ooo`0;0?ooo`H00000o`3ooonK0?ooo`40000000130?ooo`D00000303o
ool5000000h0oooo00<000000?ooo`3oool0c@3oool5000002D0oooo1`3o000;0?ooo`<0c?l09P3o
ool00`00ofH0oooo0?ooo`0D0?ooo`80c?l04@3oool0103o0000oooo0?ooo`3<o`080?ooo`030?l0
003oool0oooo00/0oooo1P00003o0?oooi`0oooo0@00000005T0oooo00<000000?ooo`3oool02`3o
ool00`000000oooo0?ooo`3=0?ooo`H000009@3oool60?l000`0oooo00<0c?l00?ooo`3<o`000P3<
o`0h0?ooo`80c?l0403oool20<co00040?l0003<o`00c?l00<co00P0oooo00<0o`000?ooo`3oool0
303oool600000?l0ooooW03oool100000000I`3oool00`000000oooo0?ooo`3>0?ooo`H000009@3o
ool60?l000`0oooo00<0c?l00?ooo`3<o`00103<o`0c0?ooo`80c?l04P3oool20?l000<0c?l0203o
ool00`3o0000oooo0?ooo`0<0?ooo`H00000o`3ooonM0?ooo`400000001W0?ooo`030000003oool0
oooo0<h0oooo1P00000V0?ooo`80o`000`3oool00`3o0000oooo0?ooo`0:0?ooo`P0c?l0;P3oool3
0<co0180oooo0P3o00050<co00@0oooo1@3o000=0?ooo`H00000o`3ooonN0?ooo`400000001W0?oo
o`030000003oool0oooo0<l0oooo1P00000W0?ooo`050?l0003oool0oooo0?ooo`3o00002`3oool6
0<co0080oooo0`3<o`0W0?ooo`@0c?l0503oool00`3o0000oooo0?ooo`050<co00@0oooo1@3o000=
0?ooo`H00000o`3ooonN0?ooo`400000001W0?ooo`030000003oool0oooo0=00oooo1P00000W0?oo
o`040?l0003oool0oooo0?l000`0oooo1@3<o`050?ooo`@0c?l07`3oool40<co01H0oooo0P3o0000
0`3oool0c?l00<co00050<co00<0oooo1P3o000<0?ooo`H00000o`3ooonO0?ooo`400000001W0?oo
o`030000003oool0oooo0=00oooo1P00000X0?ooo`040?l0003oool0oooo0?l000/0oooo1@3<o`09
0?ooo`H0c?l05@3oool40<co01P0oooo0P3o00000`3oool0c?l00<co00040?ooo`030<co003oool0
oooo00<0oooo1P3o000;0?ooo`H00000o`3ooonP0?ooo`400000001W0?ooo`@00000d03oool60000
02P0oooo0P3o00000`3oool0o`000?ooo`0;0?ooo`040<co003oool0c?l00<co00h0oooo5@3<o`0K
0?ooo`040?l0003oool0oooo0<co00/0oooo00<0o`000?ooo`3o0000103o000;0?ooo`H00000o`3o
oonM0?ooo`@00000001W0?ooo`030000003oool0oooo0=80oooo1P00000Y0?ooo`<0o`003`3oool0
0`3<o`00oooo0?ooo`0i0?ooo`80o`0000<0oooo0<co003<o`002`3oool00`3o0000oooo0?ooo`02
0?ooo`030?l0003oool0oooo00X0oooo1P00003o0?oooj40oooo0@00000006L0oooo00<000000?oo
o`3oool0dP3oool6000002/0oooo0P3o000?0?ooo`030<co003oool0oooo03H0oooo0P3o00020?oo
o`030<co003oool0oooo00X0oooo00<0o`000?ooo`3oool0403oool600000?l0ooooX@3oool10000
0000I`3oool00`000000oooo0?ooo`3C0?ooo`H00000;03oool20?l000h0oooo0P3<o`0_0?ooo`D0
c?l00P3o00030?ooo`030<co003oool0oooo00X0oooo00<0o`000?ooo`3oool0403oool600000?l0
ooooXP3oool100000000I`3oool00`000000oooo0?ooo`3D0?ooo`H00000;03oool70?l000X0oooo
0P3<o`070?ooo`D0c?l08@3oool30<co0080o`000`3oool20<co00`0oooo00<0o`000?ooo`3oool0
403oool600000?l0ooooX`3oool100000000I`3oool00`000000oooo0?ooo`3D0?ooo`H00000;@3o
ool60?l000`0oooo0P3<o`050?ooo`D0c?l0803oool20<co0080o`000P3<o`000`3oool0c?l00<co
000=0?ooo`030?l0003oool0oooo0140oooo1P00003o0?oooj<0oooo0@00000006L0oooo00<00000
0?ooo`3oool0e@3oool6000002`0oooo1P3o000>0?ooo`80c?l00P3oool60<co0200oooo0P3o0005
0<co00h0oooo00<0o`000?ooo`3oool04@3oool600000?l0ooooY03oool100000000I`3oool00`00
0000oooo0?ooo`3F0?ooo`H00000;03oool70?l000h0oooo0P3<o`000`3oool0c?l00<co00030<co
00h0oooo1@3<o`0;0?ooo`80o`0000<0oooo0<co003<o`000`3<o`080?ooo`L0o`004`3oool60000
0?l0ooooY@3oool100000000I`3oool00`000000oooo0?ooo`3F0?ooo`H00000;03oool50?l00080
oooo0P3o000>0?ooo`H0c?l03P3oool50<co00P0oooo0`3o00030<co0080oooo00<0c?l00?ooo`3o
ool0203oool50?l001D0oooo1P00003o0?ooojD0oooo0@00000006L0oooo00<000000?ooo`3oool0
e`3oool6000002d0oooo00<0o`000?ooo`3o0000103oool30?l000d0oooo0P3<o`0?0?ooo`H0c?l0
1P3oool20?l000<0c?l03`3oool60?l001@0oooo1P00003o0?ooojH0oooo0@00000006L0oooo00<0
00000?ooo`3oool0f03oool6000002l0oooo00<0o`000?ooo`3oool0103oool30?l000`0oooo103<
o`0<0?ooo`D0c?l00P3oool40?l000030<co003oool0oooo0180oooo1@3o000C0?ooo`H00000o`3o
oonW0?ooo`400000001W0?ooo`030000003oool0oooo0=P0oooo1P00000`0?ooo`030?l0003oool0
oooo00H0oooo1`3o00090?ooo`T0c?l00`3oool30<co00@0o`000P3<o`0G0?ooo`D0o`004`3oool6
00000?l0ooooY`3oool100000000I`3oool00`000000oooo0?ooo`3I0?ooo`H00000<03oool00`3o
0000oooo0?ooo`0<0?ooo`X0o`000P3oool<0?l001`0oooo0P3o00000`3oool0o`000?ooo`0C0?oo
o`H00000o`3ooonX0?ooo`400000001W0?ooo`030000003oool0oooo0=X0oooo1P00000`0?ooo`80
o`001@3oool50?l000`0oooo0P3o000V0?ooo`80o`005`3oool600000?l0ooooZ@3oool100000000
I`3oool00`000000oooo0?ooo`3J0?ooo`H00000<P3oool00`3o0000oooo0?ooo`020?ooo`D0o`00
<`3oool00`3o0000oooo0?ooo`0F0?ooo`L00000o`3ooonY0?ooo`400000001W0?ooo`030000003o
ool0oooo0=/0oooo1P00000b0?ooo`030?l0003oool0oooo00H0o`00:03oool50?l000@0oooo0P3o
000I0?ooo`H00000o`3ooonZ0?ooo`400000001W0?ooo`@00000f`3oool600000380oooo00<0o`00
0?ooo`3oool01@3o000X0?ooo`D0o`000P3oool20?l001X0oooo1P00003o0?ooojP0oooo10000000
06L0oooo00<000000?ooo`3oool0g03oool700000380oooo1`3o000W0?ooo`P0o`006`3oool60000
0?l0oooo[03oool100000000I`3oool00`000000oooo0?ooo`3M0?ooo`L00000<`3oool30?l000`0
oooo1@3o000I0?ooo`D0o`00703oool700000?l0oooo[03oool100000000I`3oool00`000000oooo
0?ooo`3N0?ooo`H00000=@3oool20?l000/0oooo1@3o000:0?ooo`D0o`002P3oool50?l001/0oooo
1`00003o0?ooojd0oooo0@00000006L0oooo00<000000?ooo`3oool0g`3oool6000003H0oooo0`3o
00070?ooo`H0o`002P3oool50?l000T0oooo0P3o00000`3oool0o`000?ooo`0K0?ooo`L00000o`3o
oon^0?ooo`400000001W0?ooo`030000003oool0oooo0=l0oooo1`00000h0?ooo`<0o`001@3oool5
0?l000T0oooo1P3o00050?ooo`@0o`00803oool600000?l0oooo[`3oool100000000I`3oool00`00
0000oooo0?ooo`3P0?ooo`L00000>P3oool20?l000<0oooo1@3o000:0?ooo`D0o`0000<0oooo0?l0
003o00000P3o000S0?ooo`H00000o`3ooon`0?ooo`400000001W0?ooo`030000003oool0oooo0>40
oooo1`00000k0?ooo`H0o`00303oool60?l002H0oooo1P00003o0?oook40oooo0@00000006L0oooo
00<000000?ooo`3oool0hP3oool600000440oooo3@3o00000`3oool0o`000?ooo`0W0?ooo`L00000
o`3ooona0?ooo`400000001W0?ooo`030000003oool0oooo0><0oooo1P00001f0?ooo`L00000o`3o
oonb0?ooo`400000001W0?ooo`030000003oool0oooo0><0oooo1`00001d0?ooo`L00000o`3ooonc
0?ooo`400000001W0?ooo`030000003oool0oooo0>@0oooo1`00001c0?ooo`H00000o`3ooond0?oo
o`400000001W0?ooo`030000003oool0oooo0>D0oooo1`00001a0?ooo`H00000o`3ooone0?ooo`40
0000001W0?ooo`030000003oool0oooo0>H0oooo1P00001`0?ooo`H00000o`3ooonf0?ooo`400000
001W0?ooo`030000003oool0oooo0>L0oooo1P00001^0?ooo`L00000o`3ooonf0?ooo`400000001W
0?ooo`030000003oool0oooo0>L0oooo1`00001/0?ooo`L00000o`3ooong0?ooo`400000001W0?oo
o`030000003oool0oooo0>P0oooo1`00001Z0?ooo`L00000o`3ooonh0?ooo`400000001W0?ooo`@0
0000j03oool7000006P0oooo1`00003o0?oookH0oooo1000000006L0oooo00<000000?ooo`3oool0
jP3oool6000006L0oooo1`00003o0?oookX0oooo0@00000006L0oooo00<000000?ooo`3oool0j`3o
ool6000006D0oooo1`00003o0?oook/0oooo0@00000006L0oooo00<000000?ooo`3oool0j`3oool7
000006<0oooo1`00003o0?oook`0oooo0@00000006L0oooo00<000000?ooo`3oool0k03oool70000
0640oooo1`00003o0?oookd0oooo0@00000006L0oooo00<000000?ooo`3oool0k@3oool7000005l0
oooo1`00003o0?oookh0oooo0@00000006L0oooo00<000000?ooo`3oool0kP3oool7000005d0oooo
1`00003o0?oookl0oooo0@00000006L0oooo00<000000?ooo`3oool0k`3oool7000005/0oooo1`00
003o0?oool00oooo0@00000006L0oooo00<000000?ooo`3oool0l03oool7000005T0oooo1`00003o
0?oool40oooo0@00000006L0oooo00<000000?ooo`3oool0l@3oool7000005L0oooo1`00003o0?oo
ol80oooo0@00000006L0oooo00<000000?ooo`3oool0lP3oool7000005D0oooo1`00003o0?oool<0
oooo0@00000006L0oooo00<000000?ooo`3oool0l`3oool7000005<0oooo1`00003o0?oool@0oooo
0@00000006L0oooo00<000000?ooo`3oool0m03oool700000540oooo1`00003o0?ooolD0oooo0@00
000006L0oooo00<000000?ooo`3oool0m@3oool7000004l0oooo1`00003o0?ooolH0oooo0@000000
06L0oooo00<000000?ooo`3oool0mP3oool7000004`0oooo2000003o0?ooolL0oooo0@00000006L0
oooo00<000000?ooo`3oool0m`3oool7000004X0oooo2000003o0?ooolP0oooo0@00000006L0oooo
1000003g0?ooo`L00000A`3oool900000?l0ooooaP3oool400000000I`3oool00`000000oooo0?oo
o`3i0?ooo`L00000A@3oool900000?l0oooobP3oool100000000I`3oool00`000000oooo0?ooo`3j
0?ooo`P00000@@3oool900000?l0ooooc03oool100000000I`3oool00`000000oooo0?ooo`3k0?oo
o`T00000?P3oool900000?l0ooooc@3oool100000000I`3oool00`000000oooo0?ooo`3l0?ooo`T0
0000>`3oool900000?l0ooooc`3oool100000000I`3oool00`000000oooo0?ooo`3m0?ooo`X00000
>03oool900000?l0ooood03oool100000000I`3oool00`000000oooo0?ooo`3o0?ooo`X00000=03o
ool900000?l0oooodP3oool100000000I`3oool00`000000oooo0?ooo`3o0?ooo`80oooo2P00000`
0?ooo`X00000o`3ooooC0?ooo`400000001W0?ooo`030000003oool0oooo0?l0oooo0`3oool;0000
02`0oooo2P00003o0?ooomD0oooo0@00000006L0oooo00<000000?ooo`3oool0o`3oool50?ooo`/0
0000:03oool;00000?l0ooooeP3oool100000000I`3oool00`000000oooo0?ooo`3o0?ooo`L0oooo
2`00000S0?ooo``00000o`3ooooH0?ooo`400000001W0?ooo`030000003oool0oooo0?l0oooo2@3o
ool;000001d0oooo3P00003o0?ooomX0oooo0@00000006L0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`2D0?ooo`030000003oool0oooo0600oooo3@00000E0?oooa0000000P3oool0
0`000000oooo0?ooo`2E0?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool0U03oool0
0`000000oooo0?ooo`0@0?ooo`400000001W0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool0U03oool00`000000oooo0?ooo`1R0?oooa8000002P3ooolB000000@0oooo00<000000?oo
o`3oool0U@3oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo09@0oooo00<000000?oo
o`3oool0403oool100000000I`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01P0
oooo:`0000050?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool08`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo0100oooo0@00000006L0oooo00<000000?ooo`3oool0
403oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0
8`3oool00`000000oooo0?ooo`0J0?ooobH00000203oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo0280oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo02<0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0@0?ooo`400000
001W0?ooool00000o`00003o000001h00000003o0?oooh40oooo5`00003o0?ooon`0oooo003o0?oo
ohL0oooo3@00003o0?oooo00oooo003o0?ooool0ooooo`3ooon50?ooo`00o`3ooooo0?ooool0oooo
Q@3oool00?l0ooooo`3ooooo0?ooohD0oooo0000\
\>"],
  ImageRangeCache->{{{0, 897.688}, {554.438, 0}} -> {-0.161437, -0.0715426, \
0.00198291, 0.00183338}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{"Pion", " ", "distribution", " ", "function", " ", 
      StyleBox["x",
        FontSlant->"Italic"], 
      StyleBox[" ",
        FontSlant->"Italic"], 
      StyleBox["q",
        FontSlant->"Italic"], 
      RowBox[{"(", 
        StyleBox["x",
          FontSlant->"Italic"], ")"}], " ", "with", " ", \(m\_0\%2\), " ", 
      "shifted", " ", "up"}], "\[IndentingNewLine]", 
    RowBox[{\(Three\ link\ truncation\), ",", " ", 
      RowBox[{
        StyleBox["K",
          FontSlant->"Italic"], "=", "10"}], ",", "12", ",", "15", ",", "20", 
      ",", "30", ",", "40", ",", "50", ",", 
      RowBox[{
        RowBox[{
        "along", " ", "with", " ", "fits", " ", "and", " ", "an", " ", 
          "extrapolation", " ", "to", " ", 
          StyleBox["K",
            FontSlant->"Italic"]}], 
        "=", \(\(\[Infinity]\)\(.\)\)}]}]}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[{
    RowBox[{"Pion", " ", "distribution", " ", "function", " ", 
      StyleBox["x",
        FontSlant->"Italic"], 
      StyleBox[" ",
        FontSlant->"Italic"], 
      StyleBox["q",
        FontSlant->"Italic"], 
      RowBox[{"(", 
        StyleBox["x",
          FontSlant->"Italic"], ")"}]}], "\[IndentingNewLine]", 
    RowBox[{\(Three\ link\ truncation\), ",", " ", 
      RowBox[{
        StyleBox["K",
          FontSlant->"Italic"], "=", "10"}], ",", "12", ",", "15", ",", "20", 
      ",", "30", ",", "40", ",", "50", ",", 
      RowBox[{
        RowBox[{
        "along", " ", "with", " ", "fits", " ", "and", " ", "an", " ", 
          "extrapolation", " ", "to", " ", 
          StyleBox["K",
            FontSlant->"Italic"]}], 
        "=", \(\(\[Infinity]\)\(.\)\)}]}]}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell["Width 600, left at zero to print ", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Compute 2-d distributions", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p1 = 
      Block[{kk = 10}, 
        ListPlot3D[
          kk^2*Map[Rest, 
              Drop[Partition[Map[If[# \[Equal] 0, 1, #] &, quarkLink[kk]], 
                  kk], \(-1\)]], ViewPoint \[Rule] {\(-2.3\), 0.8, 1}, 
          PlotRange \[Rule] {0, 1}, ClipFill \[Rule] None, 
          MeshRange \[Rule] {{0, 1}, {0, 1}}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p2 = 
      Block[{kk = 50}, 
        ListPlot3D[
          kk^2*Map[Rest, 
              Drop[Partition[Map[If[# \[Equal] 0, 1, #] &, quarkLink[kk]], 
                  kk], \(-1\)]], ViewPoint \[Rule] {\(-2.3\), 0.8, 1}, 
          PlotRange \[Rule] {0, 1}, ClipFill \[Rule] None, 
          MeshRange \[Rule] {{0, 1}, {0, 1}}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[p1, p2]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Check normalization (the numbers should be equal):", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{kk = 50}, {1 - \(number[kk]\)[\([3]\)], 
        Apply[Plus, quarkLink[kk]]}]\)], "Input"],

Cell[BoxData[
    \({0.900194398609`, 0.9001943985989985`}\)], "Output"]
}, Open  ]],

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c \>" <> 
          datadir <> "\<meson_5_40+4_5.m.gz\>", "\<\>"];\)\), "\n", 
    \(\(kk = Apply[Plus, Map[Last, basis[\([1]\)]]];\)\), "\n", 
    \(\(dist = Table[0, {i, 1/2, kk - 1/2}, {i, 1/2, kk - 1/2}];\)\), "\n", 
    \(\(Do[
        dist[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 
              basis[\([i, \(-1\), \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2; 
        dist[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 
              basis[\([i, 1, \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2, {i, 
          Length[basis]}];\)\), "\n", 
    \(\(twod[kk] = dist;\)\)}], "Input"],

Cell["This is the normalization:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(total = Apply[Plus, dist, {0, 1}]/kk^2\)], "Input"],

Cell[BoxData[
    \(0.9999999999991946`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ptwo = 
      Block[{range = {{1/\((2\ kk)\), 1 - 1/\((2\ kk)\)}, {1/\((2\ kk)\), 
                1 - 1/\((2\ kk)\)}}}, 
        Show[ListPlot3D[Map[If[# \[Equal] 0, \(-0.1\), #] &, dist, {2}], 
            MeshRange \[Rule] range, ClipFill \[Rule] None, 
            DisplayFunction \[Rule] Identity], PlotRange \[Rule] {0, 4}, 
          ViewPoint \[Rule] {2.4, \(-0.2\), 1.8}, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          DisplayFunction \[Rule] $DisplayFunction, 
          AxesLabel \[Rule] {"\<x\>", "\<y\>", "\<q(x,y)\>"}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<2d_struct.eps\>", ptwo, ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("2d_struct.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["junk", "Section"],

Cell[BoxData[
    \(\(RunThrough["\<gunzip -c ~/public_html/four/meson_5_20+4_5.m.gz\>", \
"\<\>"];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Map[\((Abs[#]^2)\) &, wave[0]]]\)], "Input"],

Cell[BoxData[
    \(1.0000000000001734`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowToolbars->{},
WindowSize->{708, 641},
WindowMargins->{{77, Automatic}, {36, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"public_html", "four"}, "jan_1_mass_squared.ps", CharacterEncoding -> \
"ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 46, 0, 64, "Subtitle"],
Cell[1788, 53, 62, 3, 71, "Subsubtitle"],
Cell[1853, 58, 150, 3, 32, "Text"],
Cell[2006, 63, 82, 2, 27, "Input",
  InitializationCell->True],
Cell[2091, 67, 228, 5, 43, "Input",
  InitializationCell->True],
Cell[2322, 74, 387, 8, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2734, 86, 23, 0, 54, "Section"],
Cell[2760, 88, 50, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[2835, 93, 55, 1, 27, "Input"],
Cell[2893, 96, 183, 3, 43, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[3125, 105, 43, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[3193, 109, 78, 1, 39, "Input"],
Cell[3274, 112, 188, 3, 60, "Message"],
Cell[3465, 117, 120, 2, 43, "Output"]
}, Open  ]],
Cell[3600, 122, 26, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[3651, 126, 66, 1, 39, "Input"],
Cell[3720, 129, 55, 1, 43, "Output"]
}, Open  ]],
Cell[3790, 133, 217, 3, 62, "Input"],

Cell[CellGroupData[{
Cell[4032, 140, 76, 1, 39, "Input"],
Cell[4111, 143, 55, 1, 43, "Output"]
}, Open  ]],
Cell[4181, 147, 60, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[4266, 151, 73, 1, 39, "Input"],
Cell[4342, 154, 172, 3, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4551, 162, 75, 1, 39, "Input"],
Cell[4629, 165, 122, 2, 43, "Output"]
}, Open  ]],
Cell[4766, 170, 26, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[4817, 174, 71, 1, 39, "Input"],
Cell[4891, 177, 55, 1, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4983, 183, 68, 1, 39, "Input"],
Cell[5054, 186, 56, 1, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5147, 192, 214, 3, 62, "Input"],
Cell[5364, 197, 130, 3, 39, "Output"]
}, Open  ]],
Cell[5509, 203, 259, 5, 71, "Text"],
Cell[5771, 210, 68, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[5864, 214, 165, 4, 39, "Input"],
Cell[6032, 220, 247, 4, 62, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6316, 229, 70, 1, 39, "Input"],
Cell[6389, 232, 82, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6508, 238, 85, 1, 39, "Input"],
Cell[6596, 241, 132, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6765, 248, 63, 1, 39, "Input"],
Cell[6831, 251, 53, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6921, 257, 79, 1, 39, "Input"],
Cell[7003, 260, 127, 2, 43, "Output"]
}, Open  ]],
Cell[7145, 265, 75, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[7245, 269, 74, 1, 39, "Input"],
Cell[7322, 272, 80, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7439, 278, 89, 1, 39, "Input"],
Cell[7531, 281, 127, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7695, 288, 67, 1, 39, "Input"],
Cell[7765, 291, 53, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7855, 297, 80, 1, 39, "Input"],
Cell[7938, 300, 82, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8057, 306, 86, 1, 39, "Input"],
Cell[8146, 309, 134, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8317, 316, 73, 1, 39, "Input"],
Cell[8393, 319, 54, 1, 39, "Output"]
}, Open  ]],
Cell[8462, 323, 76, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[8563, 327, 524, 10, 131, "Input"],
Cell[9090, 339, 130, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9257, 347, 70, 1, 39, "Input"],
Cell[9330, 350, 106, 2, 39, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[9485, 358, 29, 0, 34, "Section"],
Cell[9517, 360, 49, 1, 27, "Input"],
Cell[9569, 363, 146, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[9740, 370, 133, 3, 27, "Input"],
Cell[9876, 375, 183, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10096, 383, 79, 1, 27, "Input"],
Cell[10178, 386, 244, 4, 45, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10459, 395, 68, 1, 27, "Input"],
Cell[10530, 398, 123, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10690, 405, 38, 1, 27, "Input"],
Cell[10731, 408, 41, 1, 43, "Output"]
}, Open  ]],
Cell[10787, 412, 1252, 24, 181, "Input"],
Cell[12042, 438, 398, 11, 100, "Text"],
Cell[12443, 451, 113090, 2340, 377, 25384, 1253, "GraphicsData", \
"PostScript", "Graphics"],
Cell[125536, 2793, 130, 3, 27, "Output"],
Cell[125669, 2798, 49, 0, 32, "Text"],
Cell[125721, 2800, 398, 11, 100, "Text"],

Cell[CellGroupData[{
Cell[126144, 2815, 38, 1, 27, "Input"],
Cell[126185, 2818, 182, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126404, 2826, 38, 1, 27, "Input"],
Cell[126445, 2829, 207, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126689, 2837, 38, 1, 27, "Input"],
Cell[126730, 2840, 206, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126973, 2848, 38, 1, 27, "Input"],
Cell[127014, 2851, 183, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127234, 2859, 38, 1, 27, "Input"],
Cell[127275, 2862, 183, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127495, 2870, 38, 1, 27, "Input"],
Cell[127536, 2873, 206, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127779, 2881, 37, 1, 27, "Input"],
Cell[127819, 2884, 183, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128039, 2892, 86, 1, 27, "Input"],
Cell[128128, 2895, 56, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128221, 2901, 248, 5, 59, "Input"],
Cell[128472, 2908, 22986, 464, 186, 3016, 213, "GraphicsData", "PostScript", \
"Graphics"],
Cell[151461, 3374, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[151628, 3382, 69, 1, 27, "Input"],
Cell[151700, 3385, 131, 2, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[151880, 3393, 38, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[151943, 3397, 76, 1, 39, "Input"],
Cell[152022, 3400, 578, 8, 108, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[152637, 3413, 77, 1, 39, "Input"],
Cell[152717, 3416, 42493, 874, 202, 6843, 429, "GraphicsData", "PostScript", \
"Graphics"],
Cell[195213, 4292, 137, 3, 39, "Output"]
}, Open  ]],
Cell[195365, 4298, 181, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[195571, 4306, 69, 1, 39, "Input"],
Cell[195643, 4309, 211, 3, 62, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[195891, 4317, 80, 1, 39, "Input"],
Cell[195974, 4320, 179, 3, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[196190, 4328, 57, 1, 39, "Input"],
Cell[196250, 4331, 57, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[196344, 4337, 203, 3, 62, "Input"],
Cell[196550, 4342, 26948, 540, 279, 3485, 246, "GraphicsData", "PostScript", \
"Graphics"],
Cell[223501, 4884, 130, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223668, 4892, 70, 1, 39, "Input"],
Cell[223741, 4895, 142, 2, 39, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[223932, 4903, 36, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[223993, 4907, 38, 0, 45, "Subsection"],
Cell[224034, 4909, 49, 1, 39, "Input"],
Cell[224086, 4912, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[224257, 4919, 133, 3, 39, "Input"],
Cell[224393, 4924, 183, 3, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[224613, 4932, 528, 11, 200, "Input"],
Cell[225144, 4945, 163, 3, 60, "Message"],
Cell[225310, 4950, 163, 3, 60, "Message"],
Cell[225476, 4955, 163, 3, 60, "Message"],
Cell[225642, 4960, 169, 3, 60, "Message"],
Cell[225814, 4965, 3709, 63, 536, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[229560, 5033, 517, 11, 223, "Input"],
Cell[230080, 5046, 180, 3, 60, "Message"],
Cell[230263, 5051, 163, 3, 60, "Message"],
Cell[230429, 5056, 163, 3, 60, "Message"],
Cell[230595, 5061, 163, 3, 60, "Message"],
Cell[230761, 5066, 169, 3, 60, "Message"],
Cell[230933, 5071, 3361, 51, 70, "Output"]
}, Open  ]],
Cell[234309, 5125, 53, 0, 46, "Text"],
Cell[234365, 5127, 105, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[234507, 5134, 39, 0, 29, "Subsection"],
Cell[234549, 5136, 50, 1, 39, "Input"],
Cell[234602, 5139, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[234773, 5146, 133, 3, 39, "Input"],
Cell[234909, 5151, 206, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[235152, 5159, 528, 11, 200, "Input"],
Cell[235683, 5172, 168, 3, 70, "Message"],
Cell[235854, 5177, 163, 3, 70, "Message"],
Cell[236020, 5182, 163, 3, 70, "Message"],
Cell[236186, 5187, 163, 3, 70, "Message"],
Cell[236352, 5192, 169, 3, 70, "Message"],
Cell[236524, 5197, 3706, 63, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[240267, 5265, 517, 11, 223, "Input"],
Cell[240787, 5278, 163, 3, 70, "Message"],
Cell[240953, 5283, 163, 3, 70, "Message"],
Cell[241119, 5288, 163, 3, 70, "Message"],
Cell[241285, 5293, 169, 3, 70, "Message"],
Cell[241457, 5298, 3366, 50, 70, "Output"]
}, Open  ]],
Cell[244838, 5351, 53, 0, 46, "Text"],
Cell[244894, 5353, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[245037, 5360, 39, 0, 29, "Subsection"],
Cell[245079, 5362, 50, 1, 39, "Input"],
Cell[245132, 5365, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[245303, 5372, 133, 3, 39, "Input"],
Cell[245439, 5377, 183, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[245659, 5385, 528, 11, 200, "Input"],
Cell[246190, 5398, 163, 3, 70, "Message"],
Cell[246356, 5403, 163, 3, 70, "Message"],
Cell[246522, 5408, 163, 3, 70, "Message"],
Cell[246688, 5413, 169, 3, 70, "Message"],
Cell[246860, 5418, 3706, 63, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[250603, 5486, 517, 11, 223, "Input"],
Cell[251123, 5499, 163, 3, 70, "Message"],
Cell[251289, 5504, 163, 3, 70, "Message"],
Cell[251455, 5509, 163, 3, 70, "Message"],
Cell[251621, 5514, 169, 3, 70, "Message"],
Cell[251793, 5519, 3335, 50, 70, "Output"]
}, Open  ]],
Cell[255143, 5572, 53, 0, 46, "Text"],
Cell[255199, 5574, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[255342, 5581, 39, 0, 29, "Subsection"],
Cell[255384, 5583, 50, 1, 39, "Input"],
Cell[255437, 5586, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[255608, 5593, 133, 3, 39, "Input"],
Cell[255744, 5598, 183, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[255964, 5606, 528, 11, 200, "Input"],
Cell[256495, 5619, 163, 3, 70, "Message"],
Cell[256661, 5624, 163, 3, 70, "Message"],
Cell[256827, 5629, 163, 3, 70, "Message"],
Cell[256993, 5634, 169, 3, 70, "Message"],
Cell[257165, 5639, 168, 3, 70, "Message"],
Cell[257336, 5644, 3733, 63, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[261106, 5712, 517, 11, 223, "Input"],
Cell[261626, 5725, 163, 3, 70, "Message"],
Cell[261792, 5730, 163, 3, 70, "Message"],
Cell[261958, 5735, 163, 3, 70, "Message"],
Cell[262124, 5740, 169, 3, 70, "Message"],
Cell[262296, 5745, 3549, 58, 70, "Output"]
}, Open  ]],
Cell[265860, 5806, 53, 0, 46, "Text"],
Cell[265916, 5808, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[266059, 5815, 39, 0, 29, "Subsection"],
Cell[266101, 5817, 50, 1, 39, "Input"],
Cell[266154, 5820, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[266325, 5827, 133, 3, 39, "Input"],
Cell[266461, 5832, 206, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[266704, 5840, 528, 11, 200, "Input"],
Cell[267235, 5853, 168, 3, 70, "Message"],
Cell[267406, 5858, 163, 3, 70, "Message"],
Cell[267572, 5863, 3453, 55, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[271062, 5923, 517, 11, 223, "Input"],
Cell[271582, 5936, 163, 3, 70, "Message"],
Cell[271748, 5941, 163, 3, 70, "Message"],
Cell[271914, 5946, 163, 3, 70, "Message"],
Cell[272080, 5951, 169, 3, 70, "Message"],
Cell[272252, 5956, 3492, 56, 70, "Output"]
}, Open  ]],
Cell[275759, 6015, 53, 0, 46, "Text"],
Cell[275815, 6017, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[275958, 6024, 39, 0, 29, "Subsection"],
Cell[276000, 6026, 50, 1, 39, "Input"],
Cell[276053, 6029, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[276224, 6036, 133, 3, 39, "Input"],
Cell[276360, 6041, 207, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[276604, 6049, 528, 11, 200, "Input"],
Cell[277135, 6062, 163, 3, 70, "Message"],
Cell[277301, 6067, 163, 3, 70, "Message"],
Cell[277467, 6072, 163, 3, 70, "Message"],
Cell[277633, 6077, 169, 3, 70, "Message"],
Cell[277805, 6082, 3721, 63, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[281563, 6150, 517, 11, 223, "Input"],
Cell[282083, 6163, 163, 3, 70, "Message"],
Cell[282249, 6168, 163, 3, 70, "Message"],
Cell[282415, 6173, 163, 3, 70, "Message"],
Cell[282581, 6178, 169, 3, 70, "Message"],
Cell[282753, 6183, 3412, 52, 70, "Output"]
}, Open  ]],
Cell[286180, 6238, 53, 0, 46, "Text"],
Cell[286236, 6240, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[286379, 6247, 39, 0, 29, "Subsection"],
Cell[286421, 6249, 50, 1, 39, "Input"],
Cell[286474, 6252, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[286645, 6259, 133, 3, 39, "Input"],
Cell[286781, 6264, 182, 3, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[287000, 6272, 528, 11, 200, "Input"],
Cell[287531, 6285, 163, 3, 70, "Message"],
Cell[287697, 6290, 163, 3, 70, "Message"],
Cell[287863, 6295, 163, 3, 70, "Message"],
Cell[288029, 6300, 169, 3, 70, "Message"],
Cell[288201, 6305, 3712, 63, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[291950, 6373, 517, 11, 223, "Input"],
Cell[292470, 6386, 163, 3, 70, "Message"],
Cell[292636, 6391, 163, 3, 70, "Message"],
Cell[292802, 6396, 163, 3, 70, "Message"],
Cell[292968, 6401, 169, 3, 70, "Message"],
Cell[293140, 6406, 3365, 52, 70, "Output"]
}, Open  ]],
Cell[296520, 6461, 53, 0, 46, "Text"],
Cell[296576, 6463, 106, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[296719, 6470, 38, 0, 29, "Subsection"],
Cell[296760, 6472, 49, 1, 39, "Input"],
Cell[296812, 6475, 146, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[296983, 6482, 133, 3, 39, "Input"],
Cell[297119, 6487, 207, 3, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[297363, 6495, 528, 11, 200, "Input"],
Cell[297894, 6508, 168, 3, 60, "Message"],
Cell[298065, 6513, 168, 3, 60, "Message"],
Cell[298236, 6518, 168, 3, 60, "Message"],
Cell[298407, 6523, 169, 3, 60, "Message"],
Cell[298579, 6528, 163, 3, 60, "Message"],
Cell[298745, 6533, 3524, 58, 360, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[302306, 6596, 517, 11, 223, "Input"],
Cell[302826, 6609, 163, 3, 60, "Message"],
Cell[302992, 6614, 168, 3, 60, "Message"],
Cell[303163, 6619, 163, 3, 60, "Message"],
Cell[303329, 6624, 168, 3, 60, "Message"],
Cell[303500, 6629, 163, 3, 60, "Message"],
Cell[303666, 6634, 169, 3, 60, "Message"],
Cell[303838, 6639, 3604, 60, 461, "Output"]
}, Open  ]],
Cell[307457, 6702, 53, 0, 46, "Text"],
Cell[307513, 6704, 105, 2, 39, "Input"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[307667, 6712, 51, 0, 54, "Section"],
Cell[307721, 6714, 44, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[307790, 6718, 55, 1, 27, "Input"],
Cell[307848, 6721, 49, 1, 27, "Output"]
}, Open  ]],
Cell[307912, 6725, 98, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[308035, 6732, 200, 4, 43, "Input"],
Cell[308238, 6738, 168, 3, 37, "Message"],
Cell[308409, 6743, 156, 2, 53, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[308602, 6750, 81, 1, 27, "Input"],
Cell[308686, 6753, 56, 1, 27, "Output"]
}, Open  ]],
Cell[308757, 6757, 40, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[308822, 6761, 286, 6, 59, "Input"],
Cell[309111, 6769, 168, 3, 37, "Message"],
Cell[309282, 6774, 502, 8, 62, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[309821, 6787, 81, 1, 27, "Input"],
Cell[309905, 6790, 55, 1, 27, "Output"]
}, Open  ]],
Cell[309975, 6794, 51, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[310051, 6798, 185, 4, 43, "Input"],
Cell[310239, 6804, 57, 1, 28, "Output"]
}, Open  ]],
Cell[310311, 6808, 56, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[310392, 6812, 159, 3, 43, "Input"],
Cell[310554, 6817, 130, 3, 27, "Output"]
}, Open  ]],
Cell[310699, 6823, 42, 0, 32, "Text"],
Cell[310744, 6825, 75, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[310844, 6830, 1185, 22, 267, "Input"],
Cell[312032, 6854, 259432, 4865, 379, 35335, 2094, "GraphicsData", \
"PostScript", "Graphics"],
Cell[571467, 11721, 130, 3, 27, "Output"]
}, Open  ]],
Cell[571612, 11727, 942, 26, 83, "Text"],
Cell[572557, 11755, 877, 25, 83, "Text"],
Cell[573437, 11782, 130, 3, 27, "Output"],
Cell[573570, 11787, 49, 0, 32, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[573656, 11792, 44, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[573725, 11796, 363, 8, 108, "Input"],
Cell[574091, 11806, 137, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[574265, 11814, 363, 8, 108, "Input"],
Cell[574631, 11824, 137, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[574805, 11832, 45, 1, 39, "Input"],
Cell[574853, 11835, 132, 3, 39, "Output"]
}, Open  ]],
Cell[575000, 11841, 66, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[575091, 11845, 117, 2, 39, "Input"],
Cell[575211, 11849, 72, 1, 39, "Output"]
}, Open  ]],
Cell[575298, 11853, 675, 13, 177, "Input"],
Cell[575976, 11868, 42, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[576043, 11872, 71, 1, 39, "Input"],
Cell[576117, 11875, 53, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[576207, 11881, 629, 11, 131, "Input"],
Cell[576839, 11894, 137, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[577013, 11902, 90, 1, 39, "Input"],
Cell[577106, 11905, 49, 1, 39, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[577204, 11912, 23, 0, 34, "Section"],
Cell[577230, 11914, 114, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[577369, 11920, 76, 1, 27, "Input"],
Cell[577448, 11923, 53, 1, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

