(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    529425,       7843]*)
(*NotebookOutlinePosition[    530781,       7884]*)
(*  CellTagsIndexPosition[    530737,       7880]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Spectra mesons transverse lattice
Two-dimensional scans\
\>", "Subtitle"],

Cell["Brett van de Sande", "Subsubtitle"],

Cell["All of the following are for spectra computed in 2002.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cross[x_, y_, z_] := {Line[{{x, y - z}, {x, y + z}}], 
        Line[{{x - 0.1, y}, {x + 0.1, y}}]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(cross\)\" is similar to \
existing symbol \"\!\(Cross\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(<< \ Statistics`LinearRegression`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(data = 
      Block[\n\t\t{x, fp = OpenRead["\<~/four/junk2.out\>"], 
          info}, \n\t\tinfo = 
          Read[fp, Record, RecordSeparators -> "\<\n\n\>"]; \n\t\tx = 
          ReadList[fp, Number, 
            RecordLists -> True, \n\t\t\tRecordSeparators -> "\<\n\n\>"]; 
        Close[fp]; \n\t\t{info, Chop[\ x]}\n\ ]\)], "Input"],

Cell[BoxData[
    \(\(d3 = data;\)\)], "Input"],

Cell[BoxData[
    \(\(d5 = data;\)\)], "Input"],

Cell[BoxData[
    \(data = {d3[\([1]\)], 
        Table[If[d3[\([2, i, 3]\)] < d5[\([2, i, 3]\)], d3[\([2, i]\)], 
            d5[\([2, i]\)]], {i, Length[d3[\([2]\)]]}]}\)], "Input"],

Cell[CellGroupData[{

Cell["data three/ytraj_m0_m1_1.out", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for transverse lattice mesons \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\n\
sqrt(string tension) is 440 MeV.\nUse chi^2 fit with 11 criteria and \
tolerance 0.001.\nOverall scale from glueball data.\n0 nonstandard fit \
criteria (for log divergence).\nFit 3 masses to experimental values\n  \
(charge,multi,state,mass (MeV),error (MeV)) =\n   (1,4,0,139.557,69.3623) \
(-1,3,0,785.988,73.8942)\n   (-1,7,0,785.988,73.8942).\nNo degenerate pairs.\n\
Spectrum for P_perp a = (0,0), ( 0.25 0.25)\n  for 2 states (multiplet, \
charge, c^2 error for each) =\n  (17,  1, 5 5)\n  (17, -1, 0.2 0.2)\n  (18, \
-1, 0.2 5)\n  (18,  1, 0.2 5).\nOrdinary spectra multiplets, 2 states, \
(charge,multiplet) =\n  ( 1,  3)  (-1,  3)  ( 1,  4)  (-1,  4)  ( 1,  5)  \
(-1,  5) \n  ( 1,  6)  (-1,  6)  (-1,  7)  ( 1,  7) .\nAll spectra \
extrapolated using (K,p) = (12/2,4)  (14/2,4) \n  (18/2,4) \nLog divergence \
determined using pion (K,p) = (20/2,4) \n  (24/2,4)  (28/2,4)  (36/2,4) .\n\
Result format:\n Fit info, # steps, chi^2, p damping, scale G^2 N/sigma.\n \
The 17 couplings  (G^2 N units);\n   which couplings -- if any -- were fit.\n \
Winding potential and heavy souce potential fits (G^2 N units).\n The log(K) \
coefficient of the pion mass^2 (G^2 N units).\n The rescaled masses (MeV) for \
each P_perp*a and c^2 values.\n Finally come the masses (MeV) for the \
ordinary spectra.\>", {{3, 77, 26.26858193`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1846873158`, 0.3756823285`, 
              0.8745442022`, \(-0.5`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.462849`\), 1585.6`, 
              1588.66`, 0.851045`, 1610.74`, 1613.39`, 0.749528`, 452.208`, 
              466.627`, 1.16079`, 644.937`, 649.696`, 0.539826`, 452.208`, 
              464.164`, 0.959917`, 1642.82`, 1644.81`, 0.573416`, 191.18`, 
              228.965`, 1.39093`, 1610.74`, 1613.65`, 0.824068`, 1585.6`, 
              2447.28`, 644.937`, 2216, 191.18`, 2163.46`, 1642.82`, 
              2679.65`, 2296.25`, 2752.39`, 2305.36`, 2746.15`, 2293.97`, 
              2306.77`, 2744.13`, 2754.55`, 452.208`, 2184.81`, 1610.74`, 
              2374.89`}, {2, 35, 63.61161767`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1455837848`, 0.02340108396`, 
              0.7637967231`, \(-0.5`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.144483`\), 1404.95`, 
              1406.56`, 0.396272`, 1405.16`, 1406.77`, 0.394881`, 0, 
              69.1422`, 0.710237`, 0, 76.8552`, 0.707053`, 0, 69.0576`, 
              0.709212`, 1405.38`, 1406.98`, 0.394176`, 0, 60.3678`, 
              0.712389`, 1405.16`, 1406.77`, 0.39557`, 1404.95`, 2016.9`, 0, 
              1979.1`, 0, 1978.7`, 1405.38`, 2469.58`, 2016.16`, 2494.34`, 
              2018.11`, 2492.45`, 2016.1`, 2018.16`, 2492.42`, 2494.37`, 0, 
              1978.9`, 1405.16`, 2016.54`}, {3, 22, 
              25.92729233`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2073236909`, 0.4435322063`, 
              0.808136633`, \(-0.5`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.383595`\), 1551.63`, 
              1554.47`, 0.772335`, 1579.69`, 1582.06`, 0.657756`, 466.814`, 
              480.511`, 1.13694`, 650.65`, 655.355`, 0.538342`, 466.814`, 
              478.132`, 0.937106`, 1618.3`, 1619.66`, 0.38528`, 189.602`, 
              228.636`, 1.43039`, 1579.69`, 1582.26`, 0.713336`, 1551.63`, 
              2643.2`, 650.65`, 2183.12`, 189.602`, 2144, 1618.3`, 2651.8`, 
              2459.54`, 2913.84`, 2474.72`, 2903.27`, 2455.88`, 2476.85`, 
              2899.81`, 2917.6`, 466.814`, 2157.55`, 1579.69`, 2642.4`}, {3, 
              17, 25.46075966`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.218389142`, 0.4733623574`, 
              0.7729549518`, \(-0.5`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32646`\), 1526.13`, 
              1528.96`, 0.75756`, 1558.44`, 1560.65`, 0.603438`, 479.435`, 
              492.479`, 1.11076`, 654.749`, 659.491`, 0.546036`, 479.435`, 
              490.221`, 0.916363`, 1601.74`, 1602.78`, 0.289743`, 188.193`, 
              228.237`, 1.46103`, 1558.44`, 1560.87`, 0.663847`, 1526.13`, 
              2601.03`, 654.749`, 2156.33`, 188.193`, 2113.47`, 1601.74`, 
              2624.82`, 2520.99`, 2976.04`, 2540.37`, 2962.38`, 2516.36`, 
              2543.02`, 2957.85`, 2981.04`, 479.435`, 2128.05`, 1558.44`, 
              2607.89`}, {3, 21, 24.75822129`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2297920166`, 0.4988968154`, 
              0.7399186411`, \(-0.5`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.248693`\), 1472.81`, 
              1475.78`, 0.765343`, 1513.49`, 1515.51`, 0.535194`, 495.988`, 
              508.188`, 1.07344`, 660.017`, 664.893`, 0.565978`, 495.988`, 
              506.129`, 0.890464`, 1564.64`, 1565.31`, 0.18474`, 186.278`, 
              227.639`, 1.5`, 1513.49`, 1515.82`, 0.618615`, 1472.81`, 
              2472.24`, 660.017`, 2074.44`, 186.278`, 2024.59`, 1564.64`, 
              2497.68`, 2575.05`, 3031.15`, 2599.43`, 3013.7`, 2569.23`, 
              2602.79`, 3007.77`, 3037.85`, 495.988`, 2041.07`, 1513.49`, 
              2478.46`}, {3, 21, 23.94141816`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2433530414`, 0.5223990331`, 0.7134659492`, \(-0.5`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.1515`\), 1380.49`, 
              1383.59`, 0.751914`, 1429.83`, 1431.64`, 0.453907`, 513.286`, 
              524.592`, 1.02813`, 663.897`, 669.081`, 0.60544`, 513.286`, 
              522.81`, 0.864622`, 1488.37`, 1488.72`, 0.0918843`, 184.436`, 
              227.175`, 1.54134`, 1429.83`, 1432.07`, 0.56199`, 1380.49`, 
              2341.5`, 663.897`, 1943.99`, 184.436`, 1895.53`, 1488.37`, 
              2357.02`, 2626.46`, 3082.57`, 2656.51`, 3060.64`, 2619.22`, 
              2660.88`, 3052.9`, 3091.61`, 513.286`, 1909.33`, 1429.83`, 
              2341.04`}, {2, 22, 23.27030625`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2603066503`, 0.5468217318`, 0.6941272956`, \(-0.5`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.048939`\), 1272.06`, 
              1275.26`, 0.713224`, 1327.26`, 1328.9`, 0.382844`, 527.322`, 
              537.771`, 0.975115`, 662.338`, 668.044`, 0.665105`, 527.322`, 
              536.342`, 0.84063`, 1390.15`, 1390.31`, 0.0372671`, 183.451`, 
              227.239`, 1.57567`, 1327.26`, 1329.43`, 0.506286`, 1272.06`, 
              2264.8`, 662.338`, 1825.11`, 183.451`, 1781.44`, 1390.15`, 
              2272.02`, 2671.48`, 3125.64`, 2707.5`, 3098.99`, 2662.66`, 
              2713.19`, 3089, 3137.79`, 527.322`, 1790.67`, 1327.26`, 
              2258.02`}, {2, 23, 22.98813521`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2805907558`, 0.5724482126`, 0.6798130879`, \(-0.5`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.041384`, 1168.9`, 
              1172.17`, 0.671462`, 1227.85`, 1229.39`, 0.330865`, 536.382`, 
              545.988`, 0.910919`, 653.438`, 659.885`, 0.741841`, 536.382`, 
              545.008`, 0.817301`, 1293.2`, 1293.26`, 0.0138426`, 183.677`, 
              227.95`, 1.59671`, 1227.85`, 1230.01`, 0.464388`, 1168.9`, 
              2233.65`, 653.438`, 1741.69`, 183.677`, 1701.08`, 1293.2`, 
              2235.24`, 2705.03`, 3155.58`, 2746.65`, 3124.78`, 2694.65`, 
              2753.89`, 3112.09`, 3171.34`, 536.382`, 1706.73`, 1227.85`, 
              2221.58`}, {2, 25, 23.35666343`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3034628641`, 0.5977440209`, 0.6687483713`, \(-0.5`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.104514`, 1077.75`, 
              1081.1`, 0.635012`, 1139.21`, 1140.67`, 0.29271`, 539.76`, 
              548.459`, 0.829389`, 636.698`, 644.11`, 0.831742`, 539.76`, 
              548.077`, 0.792686`, 1206, 1206.04`, 0.00788244`, 185.343`, 
              229.325`, 1.59794`, 1139.21`, 1141.38`, 0.434735`, 1077.75`, 
              2230.85`, 636.698`, 1691.36`, 185.343`, 1651.19`, 1206, 
              2227.87`, 2726.56`, 3173.01`, 2772.91`, 3139.07`, 2714.83`, 
              2781.69`, 3123.37`, 3192.33`, 539.76`, 1655.1`, 1139.21`, 
              2213.88`}, {3, 54, 24.792966`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3270699143`, 0.6190605549`, 0.6600796365`, \(-0.5`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.131888`, 999.179`, 
              1002.61`, 0.601682`, 1061.82`, 1063.22`, 0.261253`, 535.558`, 
              543.197`, 0.721975`, 610.896`, 619.489`, 0.926353`, 535.558`, 
              543.634`, 0.763564`, 1128.92`, 1128.97`, 0.0107501`, 188.69`, 
              231.245`, 1.56573`, 1061.82`, 1064.02`, 0.410075`, 999.179`, 
              2244.28`, 610.896`, 1666.27`, 188.69`, 1625.2`, 1128.92`, 
              2236.97`, 2739.93`, 3182.37`, 2789.97`, 3146.2`, 2727.12`, 
              2800.1`, 3127.37`, 3204.77`, 535.558`, 1629.06`, 1061.82`, 
              2222.77`}, {2, 25, 28.77028094`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3280477695`, 0.6759529135`, 0.6472380354`, \(-0.5`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.062704`, 860.935`, 
              864.761`, 0.578464`, 937.307`, 938.4`, 0.179666`, 531.419`, 
              536.525`, 0.477765`, 598.665`, 607.256`, 0.90777`, 531.419`, 
              537.697`, 0.588073`, 1015.68`, 1015.3`, \(-0.0676662`\), 0, 
              96.1972`, 1.56394`, 937.307`, 939.425`, 0.348254`, 860.935`, 
              2292.63`, 598.665`, 1665.61`, 0, 1621.32`, 1015.68`, 2282.69`, 
              2768.26`, 3199.6`, 2822.64`, 3162.29`, 2754.58`, 2834.09`, 
              3138.05`, 3228.19`, 531.419`, 1624.37`, 937.307`, 
              2262.01`}, {2, 31, 50.79063955`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1068448657`, 0.1035503236`, 
              0.7386324692`, \(-0.4`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.23493`\), 1492.47`, 
              1492.31`, \(-0.0430099`\), 1498.28`, 1497.97`, \(-0.0822467`\), 
              311.609`, 323.803`, 0.678848`, 391.948`, 399.661`, 0.534993`, 
              311.609`, 323.148`, 0.641715`, 1491.09`, 1504.84`, 3.60965`, 
              212.486`, 232.17`, 0.766878`, 1498.28`, 
              1497.48`, \(-0.209481`\), 1492.47`, 1511.37`, 391.948`, 
              1755.21`, 212.486`, 1527.54`, 1491.09`, 2040.83`, 1503.51`, 
              2024.38`, 1505.01`, 2023.44`, 1503.04`, 1505.38`, 2023.17`, 
              2024.67`, 311.609`, 1504.87`, 1498.28`, 1501.09`}, {2, 36, 
              73.37325775`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1011203678`, 0.0001760777039`, 
              0.6796420092`, \(-0.4`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.119614`\), 1363.17`, 
              1367.13`, 0.949059`, 1363.17`, 1367.13`, 0.949055`, 0, 
              74.2014`, 0.528063`, 0, 74.2023`, 0.528063`, 0, 74.2014`, 
              0.528063`, 1363.17`, 1367.13`, 0.949062`, 0, 74.2005`, 
              0.528063`, 1363.17`, 1367.13`, 0.949058`, 1363.17`, 1587.9`, 0, 
              1742.52`, 0, 1587.9`, 1363.17`, 2101.18`, 1587.9`, 2101.92`, 
              1588.79`, 2101.18`, 1587.9`, 1588.79`, 2101.18`, 2101.92`, 0, 
              1587.9`, 1363.17`, 1587.9`}, {3, 22, 
              26.17394913`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2082907645`, 0.4230382537`, 
              0.7610261034`, \(-0.4`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384747`\), 1555.49`, 
              1558.34`, 0.77723`, 1580.43`, 1582.98`, 0.707976`, 459.126`, 
              473.281`, 1.15643`, 648.07`, 652.787`, 0.537616`, 459.126`, 
              470.698`, 0.942783`, 1615.8`, 1617.45`, 0.468991`, 190.421`, 
              228.793`, 1.4094`, 1580.43`, 1583.15`, 0.755852`, 1555.49`, 
              2521.72`, 648.07`, 2177.33`, 190.421`, 2131.01`, 1615.8`, 
              2629.14`, 2313.59`, 2775.38`, 2328.93`, 2764.93`, 2309.82`, 
              2331.19`, 2761.47`, 2779.12`, 459.126`, 2148.09`, 1580.43`, 
              2412.82`}, {3, 17, 25.76842088`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2215789138`, 0.4588196708`, 
              0.7180807621`, \(-0.4`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.327458`\), 1531.7`, 
              1534.46`, 0.740943`, 1559.68`, 1562.05`, 0.649215`, 471.255`, 
              484.796`, 1.13426`, 652.446`, 657.155`, 0.540274`, 471.255`, 
              482.297`, 0.922517`, 1599.73`, 1601, 0.355805`, 189.067`, 
              228.4`, 1.43868`, 1559.68`, 1562.22`, 0.693738`, 1531.7`, 
              2609.64`, 652.446`, 2156.83`, 189.067`, 2111.46`, 1599.73`, 
              2628.12`, 2372.2`, 2836.77`, 2392.58`, 2822.8`, 2367.29`, 
              2395.45`, 2818.13`, 2841.91`, 471.255`, 2127.22`, 1559.68`, 
              2606.92`}, {3, 21, 24.98469644`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2350282189`, 0.4914590736`, 
              0.6741913589`, \(-0.4`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.244935`\), 1481.36`, 
              1484.21`, 0.742143`, 1517.9`, 1520.04`, 0.570824`, 490.358`, 
              502.937`, 1.09475`, 659.33`, 664.118`, 0.555177`, 490.358`, 
              500.632`, 0.892077`, 1566.93`, 1567.75`, 0.224466`, 186.768`, 
              227.596`, 1.48227`, 1517.9`, 1520.3`, 0.639401`, 1481.36`, 
              2486.53`, 659.33`, 2084.05`, 186.768`, 2030.3`, 1566.93`, 
              2512.02`, 2417.78`, 2887, 2444.6`, 2868.5`, 2411.45`, 2448.29`, 
              2862.13`, 2894.21`, 490.358`, 2048.27`, 1517.9`, 2492.27`}, {3, 
              21, 23.88400008`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2511116851`, 0.5226393137`, 0.6373507224`, \(-0.4`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130041`\), 1372.51`, 
              1375.54`, 0.728326`, 1420.06`, 1421.92`, 0.463974`, 513.963`, 
              525.353`, 1.0372`, 665.69`, 670.824`, 0.601248`, 513.963`, 
              523.397`, 0.857504`, 1479.1`, 1479.46`, 0.094885`, 184.106`, 
              226.817`, 1.53776`, 1420.06`, 1422.34`, 0.567421`, 1372.51`, 
              2324, 665.69`, 1928.24`, 184.106`, 1875.27`, 1479.1`, 2336.95`, 
              2460.9`, 2934.78`, 2495.28`, 2910.8`, 2452.82`, 2500.17`, 
              2902.05`, 2945.15`, 513.963`, 1889.74`, 1420.06`, 
              2321.09`}, {3, 22, 23.01799589`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2729128998`, 0.5552635402`, 0.611455029`, \(-0.4`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.004653`\), 1236.7`, 
              1239.82`, 0.677439`, 1291.1`, 1292.74`, 0.371005`, 532.801`, 
              542.991`, 0.960501`, 661.563`, 667.484`, 0.689552`, 532.801`, 
              541.565`, 0.824968`, 1355.36`, 1355.47`, 0.0245629`, 183.027`, 
              227.122`, 1.5846`, 1291.1`, 1293.28`, 0.494099`, 1236.7`, 
              2238.65`, 661.563`, 1786.32`, 183.027`, 1738.06`, 1355.36`, 
              2241.04`, 2498.88`, 2974.72`, 2541.02`, 2945.24`, 2488.87`, 
              2547.57`, 2933.25`, 2989.64`, 532.801`, 1746.39`, 1291.1`, 
              2227}, {2, 25, 23.11892632`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3002706509`, 0.5875940165`, 0.5934012883`, \(-0.4`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.094527`, 1113.23`, 
              1116.43`, 0.626046`, 1171, 1172.5`, 0.307674`, 541.215`, 
              550.152`, 0.854594`, 641.742`, 648.926`, 0.812408`, 541.215`, 
              549.515`, 0.793282`, 1237.34`, 1237.35`, 0.00268614`, 184.679`, 
              228.865`, 1.60098`, 1171, 1173.15`, 0.442663`, 1113.23`, 
              2216.83`, 641.742`, 1700.65`, 184.679`, 1652.84`, 1237.34`, 
              2212.11`, 2523.35`, 2997.59`, 2572.16`, 2963.93`, 2511.54`, 
              2580.62`, 2948.02`, 3017.81`, 541.215`, 1657.53`, 1171, 
              2197.79`}, {2, 21, 25.27188514`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3288660511`, 0.6124245451`, 0.5805139546`, \(-0.4`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.142555`, 1010.24`, 
              1013.51`, 0.580701`, 1068.62`, 1069.99`, 0.257179`, 535.301`, 
              542.677`, 0.696641`, 604.461`, 613.335`, 0.946905`, 535.301`, 
              543.27`, 0.753042`, 1134.6`, 1134.61`, 0.00208019`, 190.081`, 
              232.117`, 1.55498`, 1068.62`, 1070.76`, 0.401359`, 1010.24`, 
              2227.43`, 604.461`, 1660.66`, 190.081`, 1610.62`, 1134.6`, 
              2216.67`, 2537.59`, 3007.59`, 2591.34`, 2971.48`, 2524.38`, 
              2601.51`, 2951.37`, 3032.61`, 535.301`, 1614.9`, 1068.62`, 
              2202.09`}, {2, 47, 31.58984013`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3531931573`, 0.6246059948`, 0.5697003513`, \(-0.4`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.105535`, 887.233`, 
              890.676`, 0.536252`, 945.967`, 947.182`, 0.2016`, 465.761`, 
              471.381`, 0.461454`, 508.07`, 519.897`, 1.06515`, 465.761`, 
              474.094`, 0.686191`, 1011.25`, 1011.28`, 0.00454595`, 0, 
              86.9188`, 1.42297`, 945.967`, 948.104`, 0.354689`, 887.233`, 
              2245.93`, 508.07`, 1638.08`, 0, 1587.3`, 1011.25`, 2229.98`, 
              2547.63`, 3010.4`, 2605.01`, 2973.61`, 2533.46`, 2616.59`, 
              2948.64`, 3040.01`, 465.761`, 1593.05`, 945.967`, 
              2215.98`}, {2, 21, 38.84726757`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.334085901`, 0.7013488433`, 0.5558519666`, \(-0.4`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.015221`\), 777.805`, 
              781.838`, 0.551111`, 850.645`, 851.087`, 0.0658874`, 522.536`, 
              523.712`, 0.107747`, 552.729`, 563.954`, 1.09835`, 522.536`, 
              527.566`, 0.462733`, 928.134`, 927.464`, \(-0.108943`\), 0, 
              56.639`, 1.46909`, 850.645`, 852.472`, 0.272647`, 777.805`, 
              2333.09`, 552.729`, 1686.79`, 0, 1631.74`, 928.134`, 2314.7`, 
              2598.26`, 3033.68`, 2659.52`, 2998.05`, 2583.42`, 2672.85`, 
              2964.44`, 3074.03`, 522.536`, 1635.48`, 850.645`, 
              2289.07`}, {3, 47, 26.21686701`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1796750458`, 0.3195199506`, 
              0.804187631`, \(-0.3`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.468812`\), 1608.57`, 
              1612.04`, 0.977836`, 1633.01`, 1635.94`, 0.8399`, 444.292`, 
              459.163`, 1.17715`, 642.168`, 647.031`, 0.549342`, 444.292`, 
              456.457`, 0.960053`, 1645.31`, 1650.33`, 1.45186`, 191.957`, 
              228.895`, 1.36204`, 1633.01`, 1636.81`, 1.08844`, 1608.57`, 
              2112.08`, 642.168`, 2216.95`, 191.957`, 2103.94`, 1645.31`, 
              2561.13`, 1985.64`, 2460.57`, 1993.61`, 2455.28`, 1983.5`, 
              1995.01`, 2453.51`, 2462.45`, 444.292`, 1992.73`, 1633.01`, 
              2044.05`}, {2, 28, 111.1302206`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.02671976057`, 0.01154715932`, 
              0.6041944725`, \(-0.3`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323553`\), 372.545`, 
              372.586`, 0.00265441`, 372.721`, 372.725`, 0.000287529`, 0, 0, 
              0.151331`, 41.0305`, 57.4388`, 0.141564`, 0, 0, 0.150576`, 
              372.697`, 372.703`, 0.000403042`, 0, 0, 0.160055`, 372.572`, 
              372.581`, 0.000587437`, 372.851`, 1313.87`, 41.0305`, 951.531`, 
              0, 373.762`, 1312.94`, 1345.57`, 372.572`, 1345.24`, 372.706`, 
              1345.21`, 372.545`, 372.728`, 1345.2`, 1345.25`, 0, 372.697`, 
              372.721`, 1313.4`}, {3, 40, 26.40393243`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2091616798`, 0.396773754`, 
              0.7152882417`, \(-0.3`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384343`\), 1560.98`, 
              1563.89`, 0.797863`, 1582.83`, 1585.6`, 0.770507`, 451.104`, 
              465.725`, 1.17446`, 644.687`, 649.456`, 0.540709`, 451.104`, 
              462.926`, 0.946745`, 1613.14`, 1615.24`, 0.592821`, 191.313`, 
              228.993`, 1.3876`, 1582.83`, 1585.77`, 0.815551`, 1560.98`, 
              2337.93`, 644.687`, 2165.83`, 191.313`, 2112.83`, 1613.14`, 
              2600.26`, 2159.69`, 2628.74`, 2174.83`, 2618.57`, 2155.84`, 
              2177.22`, 2615.16`, 2632.41`, 451.104`, 2172.79`, 1582.83`, 
              2253.86`}, {3, 34, 26.15638443`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2257996742`, 0.4380887504`, 
              0.6634954471`, \(-0.3`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32563`\), 1535.92`, 
              1538.64`, 0.732021`, 1559.18`, 1561.78`, 0.712406`, 461.162`, 
              475.302`, 1.1602`, 648.749`, 653.465`, 0.538144`, 461.162`, 
              472.517`, 0.928896`, 1594.6`, 1596.23`, 0.456077`, 190.213`, 
              228.698`, 1.41252`, 1559.18`, 1561.87`, 0.736913`, 1535.92`, 
              2442.61`, 648.749`, 2146.85`, 190.213`, 2094.8`, 1594.6`, 
              2600.97`, 2215.47`, 2687.66`, 2236.59`, 2673.49`, 2210.26`, 
              2239.74`, 2668.69`, 2692.92`, 461.162`, 2113.89`, 1559.18`, 
              2327.18`}, {2, 17, 25.40495886`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.243329782`, 0.4797536321`, 
              0.6043937893`, \(-0.3`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.237711`\), 1489.92`, 
              1492.63`, 0.708147`, 1520.43`, 1522.79`, 0.62747`, 481.046`, 
              494.2`, 1.124`, 656.823`, 661.525`, 0.543159`, 481.046`, 
              491.551`, 0.895183`, 1565.56`, 1566.64`, 0.293931`, 187.75`, 
              227.74`, 1.45579`, 1520.43`, 1522.94`, 0.667694`, 1489.92`, 
              2498.24`, 656.823`, 2090.9`, 187.75`, 2030.2`, 1565.56`, 
              2527.47`, 2247.37`, 2726.74`, 2277.01`, 2706.92`, 2240.32`, 
              2281.21`, 2700, 2734.59`, 481.046`, 2051.48`, 1520.43`, 
              2491.11`}, {3, 17, 23.84402731`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2644082026`, 0.5255455929`, 0.5481405511`, \(-0.3`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.093021`\), 1354.72`, 
              1357.62`, 0.690954`, 1399.73`, 1401.68`, 0.477972`, 515.224`, 
              526.723`, 1.04979`, 668.562`, 673.609`, 0.593512`, 515.224`, 
              524.505`, 0.845555`, 1459.73`, 1460.11`, 0.0975892`, 183.522`, 
              226.222`, 1.53293`, 1399.73`, 1402.06`, 0.572479`, 1354.72`, 
              2291.67`, 668.562`, 1898.14`, 183.522`, 1836.97`, 1459.73`, 
              2300.77`, 2272.45`, 2760.77`, 2313.42`, 2733.57`, 2263.07`, 
              2319.18`, 2723.15`, 2773.41`, 515.224`, 1853.19`, 1399.73`, 
              2284.7`}, {3, 23, 22.75006481`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2978225357`, 0.5750077706`, 0.5101611544`, \(-0.3`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.071631`, 1169.8`, 
              1172.82`, 0.6201`, 1222.51`, 1224.11`, 0.343767`, 541.472`, 
              551.071`, 0.91886`, 655.041`, 661.556`, 0.751598`, 541.472`, 
              549.813`, 0.797503`, 1288.97`, 1288.99`, 0.00322677`, 183.093`, 
              227.537`, 1.59901`, 1222.51`, 1224.7`, 0.469191`, 1169.8`, 
              2202.35`, 655.041`, 1725.36`, 183.093`, 1665.91`, 1288.97`, 
              2195.91`, 2293.9`, 2786.66`, 2346.08`, 2752.89`, 2281.92`, 
              2354.2`, 2736.94`, 2807.19`, 541.472`, 1673.44`, 1222.51`, 
              2181.5`}, {2, 21, 25.92881432`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3369965301`, 0.6077236816`, 0.4872758014`, \(-0.3`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.160313`, 1018.07`, 
              1021.23`, 0.564963`, 1070.34`, 1071.6`, 0.237369`, 533.95`, 
              541, 0.663932`, 595.628`, 604.838`, 0.968656`, 533.95`, 
              541.766`, 0.736666`, 1135.39`, 1135.34`, \(-0.0091488`\), 
              192.967`, 233.94`, 1.53256`, 1070.34`, 1072.43`, 0.391561`, 
              1018.07`, 2204.05`, 595.628`, 1653.41`, 192.967`, 1588.16`, 
              1135.39`, 2187.64`, 2303.07`, 2792.69`, 2362.81`, 2755.82`, 
              2289.08`, 2373.28`, 2733.42`, 2821.08`, 533.95`, 1594.11`, 
              1070.34`, 2172.92`}, {3, 29, 28.1438379`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4276921031`, 0.6948470058`, 0.4624954651`, \(-0.3`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.094759`, 892.684`, 
              900.685`, 1.25726`, 947.488`, 944.468`, \(-0.500542`\), 
              530.51`, 540.052`, 0.89505`, 543.388`, 552.897`, 0.913285`, 
              543.388`, 551.108`, 0.74027`, 1019.66`, 
              1019.63`, \(-0.00581349`\), 193.428`, 235.873`, 1.59651`, 
              947.488`, 949.853`, 0.393117`, 892.684`, 2245.81`, 530.51`, 
              1643.17`, 193.428`, 1539.06`, 1019.66`, 2206.95`, 2249.34`, 
              2749.65`, 2317.97`, 2705.71`, 2232.86`, 2330.36`, 2680.63`, 
              2780.34`, 543.388`, 1552.39`, 947.488`, 2191.66`}, {3, 42, 
              34.5336063`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3636125331`, 0.7331054693`, 0.4524853262`, \(-0.3`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.013926`\), 795.505`, 
              806.986`, 1.61198`, 855.31`, 848.141`, \(-1.06995`\), 560.633`, 
              567.416`, 0.670354`, 576.808`, 582.919`, 0.620953`, 576.808`, 
              581.219`, 0.447503`, 929.883`, 929.043`, \(-0.136807`\), 
              195.331`, 235.397`, 1.51203`, 855.31`, 856.949`, 0.245795`, 
              795.505`, 2346.57`, 560.633`, 1706.69`, 195.331`, 1623.35`, 
              929.883`, 2310.68`, 2352.82`, 2802.64`, 2421.97`, 2764.72`, 
              2336.55`, 2436.44`, 2726.11`, 2848.56`, 576.808`, 1626.41`, 
              855.31`, 2283.35`}, {1, 77, 35.74751313`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5314504922`, 0.8700987377`, 0.4266936727`, \(-0.3`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.391972`\), 752.475`, 
              762.94`, 1.3895`, 804.243`, 798.291`, \(-0.835758`\), 449.427`, 
              460.109`, 0.851311`, 582.674`, 590.41`, 0.795085`, 582.674`, 
              587.479`, 0.492549`, 886.808`, 886.008`, \(-0.124161`\), 
              198.953`, 239.578`, 1.56092`, 804.243`, 806.283`, 0.287814`, 
              752.475`, 2411.97`, 449.427`, 1785.09`, 198.953`, 1612.56`, 
              886.808`, 2335.45`, 2220.41`, 2714.41`, 2298.8`, 2664.05`, 
              2201.47`, 2312.94`, 2632.65`, 2749.7`, 582.674`, 1633.27`, 
              804.243`, 2326.33`}, {2, 25, 79.72615082`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2651859411`, 0.1666281368`, 
              0.8310352022`, \(-0.2`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.388346`\), 1649.37`, 
              1656.09`, 1.94573`, 1651.7`, 1658.78`, 2.05378`, 215.709`, 
              256.723`, 1.69767`, 320.284`, 346.529`, 1.53333`, 215.709`, 
              254.321`, 1.59012`, 1630.03`, 1649.18`, 5.50226`, 0, 120.499`, 
              1.74037`, 1651.7`, 1659.98`, 2.40132`, 1649.37`, 1935.05`, 
              320.284`, 2182.78`, 0, 1971.07`, 1630.03`, 2451.05`, 1918.3`, 
              2421.39`, 1924.86`, 2415.78`, 1916.72`, 1926.23`, 2414.77`, 
              2422.42`, 215.709`, 1924.37`, 1651.7`, 1916.68`}, {3, 40, 
              33.64498053`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1502208406`, 0.1904132838`, 
              0.6743800469`, \(-0.2`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.332505`\), 1566.47`, 
              1569.15`, 0.735865`, 1572.04`, 1570.81`, \(-0.339406`\), 
              380.812`, 394.02`, 0.896681`, 520.789`, 526.783`, 0.550153`, 
              380.812`, 392.34`, 0.780921`, 1567.9`, 1577.47`, 2.63717`, 
              204.62`, 231.697`, 1.03513`, 1572.35`, 
              1570.13`, \(-0.609625`\), 1566.47`, 1620.28`, 520.789`, 
              1876.72`, 204.62`, 1641.03`, 1567.9`, 2135.53`, 1573.43`, 
              2086.17`, 1577.94`, 2083.51`, 1572.04`, 1578.93`, 2082.59`, 
              2087.13`, 380.812`, 1577.45`, 1572.35`, 1591.98`}, {3, 26, 
              26.54960701`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096226057`, 0.3653438598`, 
              0.67236945`, \(-0.2`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384021`\), 1571.94`, 
              1575.08`, 0.868158`, 1592.24`, 1595.24`, 0.83717`, 443.899`, 
              458.919`, 1.18814`, 641.204`, 646.049`, 0.546453`, 443.899`, 
              455.91`, 0.946909`, 1612.53`, 1615.73`, 0.905374`, 192.111`, 
              229.132`, 1.36636`, 1592.24`, 1595.64`, 0.94967`, 1571.94`, 
              2162.64`, 641.204`, 2152.09`, 192.111`, 2119.62`, 1612.53`, 
              2601.82`, 1997.35`, 2476.59`, 2011.81`, 2467, 1993.51`, 
              2014.28`, 2463.74`, 2480.09`, 443.899`, 2009.98`, 1592.24`, 
              2081.42`}, {3, 30, 26.56192936`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2308971884`, 0.4099250503`, 
              0.6108696233`, \(-0.2`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321356`\), 1541.6`, 
              1544.38`, 0.750854`, 1560.53`, 1563.44`, 0.796308`, 450.332`, 
              465.106`, 1.18497`, 643.976`, 648.752`, 0.54095`, 450.332`, 
              462.001`, 0.932747`, 1588.65`, 1590.91`, 0.629582`, 191.471`, 
              229.063`, 1.3851`, 1560.53`, 1563.5`, 0.812745`, 1541.6`, 
              2249.11`, 643.976`, 2120.53`, 191.471`, 2065.61`, 1588.65`, 
              2543.66`, 2050.53`, 2531.81`, 2071.82`, 2517.68`, 2045.05`, 
              2075.27`, 2512.83`, 2537.09`, 450.332`, 2068.73`, 1560.53`, 
              2156.66`}, {3, 21, 26.12136431`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2572075476`, 0.460191744`, 
              0.5320263066`, \(-0.2`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.224487`\), 1496.97`, 
              1499.49`, 0.661756`, 1518.97`, 1521.68`, 0.723196`, 466.56`, 
              480.565`, 1.16219`, 650.816`, 655.49`, 0.534888`, 466.56`, 
              477.431`, 0.899153`, 1556.65`, 1558.21`, 0.424567`, 189.566`, 
              228.329`, 1.41928`, 1518.97`, 1521.63`, 0.707969`, 1496.97`, 
              2331.7`, 650.816`, 2081.27`, 189.566`, 2008.96`, 1556.65`, 
              2526.99`, 2062.16`, 2552.35`, 2094.73`, 2530.96`, 2054.18`, 
              2099.7`, 2523.43`, 2560.9`, 466.56`, 2036.13`, 1518.97`, 
              2195.6`}, {2, 18, 23.72891678`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2950403358`, 0.5445546645`, 0.4283570163`, \(-0.2`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.002394`\), 1294.31`, 
              1297.03`, 0.619068`, 1336.03`, 1338.09`, 0.48095`, 523.369`, 
              534.779`, 1.05788`, 676.056`, 680.971`, 0.584283`, 523.369`, 
              532.155`, 0.812558`, 1399.63`, 1399.95`, 0.0787874`, 181.405`, 
              224.63`, 1.53774`, 1336.03`, 1338.44`, 0.563045`, 1294.31`, 
              2210.85`, 676.056`, 1818.15`, 181.405`, 1738.87`, 1399.63`, 
              2216.34`, 2041.38`, 2544.5`, 2095.53`, 2511.09`, 2029.39`, 
              2103.13`, 2496.89`, 2562.62`, 523.369`, 1758.65`, 1336.03`, 
              2185.24`}, {3, 25, 28.27663806`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4758561769`, 0.7160187793`, 0.339883876`, \(-0.2`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.170517`, 956.409`, 
              973.865`, 2.95224`, 993.91`, 981.342`, \(-2.17515`\), 540.195`, 
              550.159`, 0.951898`, 551.957`, 561.809`, 0.961397`, 551.957`, 
              559.782`, 0.762226`, 1070.26`, 1070.05`, \(-0.0396355`\), 
              188.743`, 233.174`, 1.64247`, 993.91`, 996.327`, 0.421501`, 
              956.409`, 2165.61`, 540.195`, 1658.62`, 188.743`, 1489.56`, 
              1070.26`, 2130.41`, 1933.76`, 2455.18`, 2015.77`, 2408.4`, 
              1915.35`, 2028.61`, 2380.88`, 2487.89`, 551.957`, 1514.2`, 
              993.91`, 2099.47`}, {1, 53, 36.73004522`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.447152894`, 0.8145314141`, 0.3076869907`, \(-0.2`\), 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.075076`\), 807.509`, 
              813.46`, 0.845089`, 835.96`, 833.011`, \(-0.43125`\), 490.307`, 
              499.376`, 0.786378`, 587.096`, 593.724`, 0.68572`, 587.096`, 
              591.458`, 0.450453`, 915.168`, 913.757`, \(-0.226125`\), 
              199.216`, 240.102`, 1.57377`, 835.96`, 837.396`, 0.210591`, 
              807.509`, 2285.39`, 490.307`, 1777.88`, 199.216`, 1606.15`, 
              915.168`, 2278.63`, 2001.91`, 2467.1`, 2084.97`, 2423.79`, 
              1982.78`, 2101.65`, 2377.21`, 2519.05`, 587.096`, 1612.88`, 
              835.96`, 2194.41`}, {2, 49, 50.40180104`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6658425659`, 0.0004532400367`, 0.355542392`, \(-0.2`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.13861`, 806.612`, 
              807.179`, 0.0800828`, 806.612`, 807.179`, 0.0800828`, 226.03`, 
              249.978`, 0.998774`, 226.03`, 249.978`, 0.998773`, 226.03`, 
              249.978`, 0.998774`, 806.612`, 807.179`, 0.0800825`, 226.031`, 
              249.978`, 0.998773`, 806.612`, 807.179`, 0.0800826`, 806.612`, 
              2025.23`, 226.03`, 1720.32`, 226.031`, 1720.32`, 806.612`, 
              2285.83`, 2025.23`, 2371.74`, 2059.67`, 2472.93`, 2025.23`, 
              2059.67`, 2371.74`, 2500.54`, 226.03`, 1720.32`, 806.612`, 
              2025.23`}, {2, 67, 60.44436513`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.000776728751`, 0.7273160363`, 0.3260522196`, \(-0.2`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.015926`, 742.109`, 
              748.277`, 0.80531`, 780.314`, 775.106`, \(-0.709794`\), 
              590.555`, 596.562`, 0.624718`, 607.246`, 608.233`, 0.105038`, 
              607.246`, 608.234`, 0.105209`, 845.786`, 
              843.012`, \(-0.410403`\), 192.499`, 232.364`, 1.48398`, 
              780.314`, 780.19`, \(-0.0169975`\), 742.109`, 2351.85`, 
              590.555`, 1793.84`, 192.499`, 1765.89`, 845.786`, 2527.3`, 
              2351.85`, 2453.03`, 2385.1`, 2507.32`, 2343.7`, 2380.12`, 
              2527.3`, 2552.7`, 607.246`, 1732.18`, 780.314`, 2354.14`}, {1, 
              56, 51.88141016`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3098915498`, 0.8485481716`, 0.2964806869`, \(-0.2`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.269107`\), 712.307`, 
              714.359`, 0.256548`, 726.591`, 725.395`, \(-0.15211`\), 
              456.982`, 466.236`, 0.748515`, 586.25`, 590.262`, 0.413647`, 
              586.25`, 588.479`, 0.229414`, 794.304`, 
              791.768`, \(-0.352509`\), 200.598`, 240.687`, 1.54998`, 
              726.591`, 726.83`, 0.030432`, 712.307`, 2276.26`, 456.982`, 
              1931.23`, 200.598`, 1818.1`, 794.304`, 2448.89`, 2150.54`, 
              2509.74`, 2213.81`, 2451.59`, 2132.18`, 2239.41`, 2401.49`, 
              2577.16`, 586.25`, 1777.27`, 726.591`, 2261.28`}, {2, 48, 
              35.05823125`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2087688852`, 0.4271119199`, 
              0.8009787658`, \(-0.1`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.630712`\), 1659.13`, 
              1663.31`, 1.21737`, 1691.9`, 1695.22`, 0.983639`, 495.713`, 
              512.464`, 1.47964`, 769.267`, 772.517`, 0.439047`, 495.713`, 
              508.344`, 1.1112`, 1717.75`, 1735.1`, 5.24976`, 171.7`, 
              219.197`, 1.62674`, 1691.9`, 1697.02`, 1.52026`, 1659.13`, 
              2242.73`, 769.267`, 2315.6`, 171.7`, 2183.21`, 1717.75`, 
              2646.72`, 2021.94`, 2490.87`, 2034.61`, 2482.7`, 2018.93`, 
              2036.36`, 2480.04`, 2493.73`, 495.713`, 2033.16`, 1691.9`, 
              2128.89`}, {3, 50, 27.58885019`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1734911392`, 0.2411954651`, 
              0.6773970031`, \(-0.1`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.39866`\), 1604.11`, 
              1605.63`, 0.427758`, 1607.66`, 1610.29`, 0.741185`, 416.258`, 
              430.575`, 1.0623`, 592.682`, 598.029`, 0.557773`, 416.258`, 
              428.109`, 0.876797`, 1608.82`, 1615.6`, 1.91509`, 198.109`, 
              230.333`, 1.20963`, 1609.63`, 1606.9`, \(-0.769141`\), 
              1604.11`, 1691.59`, 592.682`, 1946.96`, 198.109`, 1708.21`, 
              1608.82`, 2192.06`, 1609.63`, 2117.68`, 1616.27`, 2113.8`, 
              1607.66`, 1617.61`, 2112.45`, 2119.12`, 416.258`, 1615.52`, 
              1611.36`, 1645.87`}, {2, 32, 28.12056887`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2231675752`, 0.3864876941`, 
              0.6520640614`, \(-0.1`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.424855`\), 1602.47`, 
              1606.44`, 1.11705`, 1627.21`, 1630.35`, 0.897716`, 460.681`, 
              476.694`, 1.31516`, 684.21`, 688.724`, 0.542935`, 460.681`, 
              473.032`, 1.01045`, 1639.53`, 1653.87`, 4.13851`, 187.476`, 
              227.855`, 1.46936`, 1627.21`, 1632.01`, 1.37057`, 1602.47`, 
              2114.22`, 684.21`, 2186, 187.476`, 2087.57`, 1639.53`, 
              2553.57`, 1924.82`, 2409.28`, 1941.2`, 2398.68`, 1920.51`, 
              1943.93`, 2395.06`, 2413.18`, 460.681`, 1939.14`, 1627.21`, 
              2015.64`}, {1, 33, 43.26661077`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1676719311`, 0.1959958972`, 
              0.4922454312`, \(-0.1`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.225698`\), 1503.73`, 
              1502.74`, \(-0.262375`\), 1504.8`, 1507.52`, 0.718132`, 
              356.909`, 368.765`, 0.75384`, 479.739`, 485.186`, 0.460502`, 
              356.909`, 367.181`, 0.651694`, 1497.07`, 1513.93`, 4.44914`, 
              206.073`, 228.956`, 0.872213`, 1507.19`, 
              1504.34`, \(-0.751405`\), 1503.73`, 1553.91`, 479.739`, 
              1761.9`, 206.073`, 1584.74`, 1497.07`, 2082.32`, 1507.19`, 
              2028.44`, 1514.95`, 2024.14`, 1504.8`, 1516.6`, 2022.64`, 
              2030.05`, 356.909`, 1513.91`, 1508.55`, 1523.52`}, {1, 22, 
              27.12894717`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2809326076`, 0.4282694616`, 
              0.462817049`, \(-0.1`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.203491`\), 1504.44`, 
              1506.92`, 0.656001`, 1517.46`, 1520.78`, 0.882978`, 447.831`, 
              462.929`, 1.20477`, 640.291`, 645.071`, 0.538261`, 447.831`, 
              459.139`, 0.898559`, 1540.16`, 1543.16`, 0.810415`, 192.239`, 
              229.524`, 1.37783`, 1517.46`, 1520.6`, 0.834849`, 1504.44`, 
              2095.12`, 640.291`, 2013.59`, 192.239`, 1948.97`, 1540.16`, 
              2458.14`, 1863.71`, 2370.98`, 1898.25`, 2348.83`, 1854.67`, 
              1904.12`, 2341, 2379.79`, 447.831`, 1892.37`, 1517.46`, 
              1990.03`}, {1, 50, 57.27527321`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.002715459081`, 0.668989628`, 0.119027714`, \(-0.1`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.140587`, 788.246`, 
              787.781`, \(-0.0641307`\), 791.389`, 789.862`, \(-0.21155`\), 
              480.978`, 490.274`, 0.791049`, 544.601`, 546.466`, 0.178219`, 
              544.601`, 546.465`, 0.178182`, 853.768`, 
              850.307`, \(-0.516662`\), 215.823`, 246.557`, 1.24511`, 
              788.246`, 787.781`, \(-0.0641046`\), 791.389`, 1774.93`, 
              480.978`, 1709.89`, 215.823`, 1709.9`, 853.768`, 1973.46`, 
              1775.61`, 1981.29`, 1815.3`, 2058.67`, 1785.88`, 1835.22`, 
              1971.77`, 1973.8`, 544.601`, 1709.9`, 788.246`, 1777.31`}, {1, 
              55, 50.56199921`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6287379951`, 0.0001503176027`, 0.1351022271`, \(-0.1`\), 
              0.1`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.105968`, 773.142`, 
              770.497`, \(-0.357709`\), 773.142`, 770.497`, \(-0.357709`\), 
              225.848`, 249.773`, 0.997033`, 225.848`, 249.773`, 0.997032`, 
              225.848`, 249.773`, 0.997033`, 773.142`, 
              770.497`, \(-0.357709`\), 225.848`, 249.773`, 0.997032`, 
              773.142`, 770.497`, \(-0.357709`\), 773.142`, 1784.1`, 
              225.848`, 1793.29`, 225.848`, 1784.1`, 773.142`, 2003.85`, 
              1749.58`, 1784.1`, 1833.86`, 1956.21`, 1784.1`, 1833.86`, 
              1749.58`, 2003.85`, 225.848`, 1749.58`, 773.142`, 1784.1`}, {3, 
              68, 50.56395455`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6276684631`, 0.0006285387724`, 0.1301240918`, \(-0.1`\), 
              0.2`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.103302`, 765.377`, 
              762.667`, \(-0.362804`\), 765.377`, 762.667`, \(-0.362805`\), 
              225.857`, 249.796`, 0.997669`, 225.858`, 249.797`, 0.997668`, 
              225.858`, 249.797`, 0.997669`, 765.377`, 
              762.667`, \(-0.362804`\), 225.858`, 249.797`, 0.997668`, 
              765.377`, 762.667`, \(-0.362805`\), 765.377`, 1783.5`, 
              225.857`, 1780.54`, 225.858`, 1780.54`, 765.377`, 1992.6`, 
              1736.97`, 1783.5`, 1830.54`, 1992.59`, 1783.5`, 1830.55`, 
              1736.97`, 1992.6`, 225.858`, 1736.97`, 765.377`, 1783.5`}, {3, 
              59, 68.07597193`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.00090590053`, 0.7012434924`, 0.07864987404`, \(-0.1`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.023796`, 700.186`, 
              699.607`, \(-0.0709744`\), 740.291`, 737.252`, \(-0.393353`\), 
              281.681`, 302.945`, 1.08921`, 487.615`, 489.888`, 0.194699`, 
              487.615`, 489.888`, 0.194699`, 749.858`, 
              746.459`, \(-0.445525`\), 220.481`, 248.99`, 1.17271`, 
              700.186`, 699.607`, \(-0.0709743`\), 740.291`, 1695.54`, 
              281.681`, 1626.03`, 220.481`, 1626.03`, 749.858`, 1901.66`, 
              1695.54`, 1909.88`, 1736.16`, 1900.35`, 1694.87`, 1735.81`, 
              1901.66`, 1918.57`, 487.615`, 1626.03`, 700.186`, 1698.2`}, {2, 
              49, 68.67997337`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.00003826815809`, 
              0.7071041291`, \(-0.07370579325`\), \(-0.1`\), 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.014166`, 695.655`, 
              695.057`, \(-0.0728811`\), 738.666`, 735.475`, \(-0.412165`\), 
              267.011`, 289.881`, 1.11588`, 485.481`, 487.809`, 0.198562`, 
              485.481`, 487.809`, 0.198562`, 745.668`, 
              742.205`, \(-0.451443`\), 219.793`, 248.53`, 1.17916`, 
              695.655`, 695.057`, \(-0.0728811`\), 738.666`, 1687.77`, 
              267.011`, 1617.65`, 219.793`, 1617.65`, 745.668`, 1894.83`, 
              1687.77`, 1908.77`, 1728.36`, 1893.87`, 1687.28`, 1728.1`, 
              1894.83`, 1915.27`, 485.481`, 1617.65`, 695.655`, 
              1690.47`}, {3, 46, 69.05428703`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.000035424050889999996`, 
              0.7129072919`, \(-0.06910850417`\), \(-0.1`\), 0.5`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.006553`, 692.788`, 
              692.171`, \(-0.0748767`\), 738.221`, 734.894`, \(-0.42938`\), 
              256.475`, 280.692`, 1.13974`, 484.716`, 487.095`, 0.202564`, 
              484.716`, 487.095`, 0.202564`, 743.532`, 
              739.993`, \(-0.45995`\), 219.128`, 248.173`, 1.18916`, 
              692.788`, 692.171`, \(-0.0748767`\), 738.221`, 1680.97`, 
              256.475`, 1610.27`, 219.128`, 1610.27`, 743.532`, 1888.88`, 
              1680.97`, 1907.79`, 1721.51`, 1888.16`, 1680.59`, 1721.32`, 
              1888.88`, 1912.79`, 484.716`, 1610.27`, 692.788`, 1683.7`}, {2, 
              95, 44.51729737`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2216510102`, 0.4772267698`, 0.7997970973`, 0, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.686503`\), 1672.03`, 
              1676.57`, 1.3321`, 1703.19`, 1706.92`, 1.11581`, 480.878`, 
              500.663`, 1.70148`, 788.292`, 792.089`, 0.525747`, 480.878`, 
              495.472`, 1.24838`, 1733.57`, 1750.61`, 5.20152`, 0, 140.754`, 
              1.80365`, 1703.19`, 1708.92`, 1.71318`, 1672.03`, 2298.31`, 
              788.292`, 2364.35`, 0, 2219.26`, 1733.57`, 2693.18`, 2041.87`, 
              2510.74`, 2056.78`, 2500.99`, 2038.25`, 2058.95`, 2497.82`, 
              2514.16`, 480.878`, 2055.08`, 1703.19`, 2168.98`}, {2, 48, 
              26.27757052`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1905642732`, 0.2734638388`, 0.6761735867`, 0, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.441522`\), 1607.62`, 
              1606.43`, \(-0.334637`\), 1616.14`, 1619.36`, 0.912625`, 
              436.37`, 451.581`, 1.18334`, 637.785`, 642.768`, 0.559057`, 
              436.37`, 448.498`, 0.940263`, 1617.32`, 1617.52`, 0.0550042`, 
              192.865`, 228.699`, 1.32357`, 1610.02`, 
              1608.26`, \(-0.496401`\), 1616.14`, 1727.42`, 637.785`, 
              1976.64`, 192.865`, 1731.74`, 1630.29`, 2210.55`, 1610.02`, 
              2117.33`, 1618.27`, 2112.56`, 1607.62`, 1619.87`, 2110.89`, 
              2119.11`, 436.37`, 1617.32`, 1619.72`, 1672.1`}, {2, 42, 
              38.29667287`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2489533289`, 0.4702504947`, 0.6530327832`, 0, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.510294`\), 1634.81`, 
              1639.24`, 1.27313`, 1660.49`, 1664.07`, 1.04292`, 495.759`, 
              513.427`, 1.56226`, 772.189`, 775.809`, 0.490945`, 495.759`, 
              508.637`, 1.13329`, 1685.8`, 1708.72`, 6.81522`, 175.456`, 
              223.445`, 1.67723`, 1660.49`, 1666.19`, 1.66261`, 1634.81`, 
              2235.38`, 772.189`, 2269.16`, 175.456`, 2147.05`, 1685.8`, 
              2637.41`, 1968.13`, 2448.47`, 1990.08`, 2434.39`, 1962.72`, 
              1993.31`, 2429.65`, 2453.62`, 495.759`, 1987.27`, 1660.49`, 
              2102.5`}, {3, 25, 29.55478551`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2164391544`, 0.2732926492`, 0.4974735395`, 0, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.290259`\), 1554.17`, 
              1553.12`, \(-0.286762`\), 1565.73`, 1568.68`, 0.809267`, 
              406.808`, 421.059`, 1.03368`, 580.39`, 585.486`, 0.520617`, 
              406.808`, 418.122`, 0.817756`, 1561.64`, 1570.25`, 2.36325`, 
              199.522`, 230.214`, 1.15564`, 1558.26`, 1556.4`, \(-0.50827`\), 
              1565.73`, 1665.2`, 580.39`, 1894.79`, 199.522`, 1692.27`, 
              1561.64`, 2172.29`, 1558.26`, 2075.77`, 1571.98`, 2068.09`, 
              1554.17`, 1574.7`, 2065.36`, 2078.69`, 406.808`, 1570.13`, 
              1569.01`, 1608.11`}, {1, 41, 33.69157599`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3616239192`, 0.3936072421`, 0.4690119333`, 0, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.199931`\), 1557.23`, 
              1562.58`, 1.46189`, 1564.63`, 1568.5`, 1.06357`, 428.474`, 
              447.236`, 1.4396`, 614.438`, 621.706`, 0.787128`, 428.474`, 
              442.363`, 1.05977`, 1563.92`, 1588.19`, 6.70231`, 195.638`, 
              236.167`, 1.53335`, 1564.63`, 1571.18`, 1.80031`, 1557.23`, 
              1961.75`, 614.438`, 2013.01`, 195.638`, 1994.62`, 1563.92`, 
              2420.35`, 1755.18`, 2287.8`, 1789.92`, 2263.49`, 1744.7`, 
              1797.8`, 2255.6`, 2296.34`, 428.474`, 1784.47`, 1564.63`, 
              1860.41`}, {3, 26, 33.32775482`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6623453098`, 0.3754364803`, 0.5267070012`, 0, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.073551`\), 1532.48`, 
              1536.38`, 1.04744`, 1537.64`, 1543.57`, 1.60261`, 405.156`, 
              422.787`, 1.27896`, 562.81`, 568.916`, 0.605456`, 405.156`, 
              416.703`, 0.831442`, 1547.48`, 1556.19`, 2.36693`, 201.731`, 
              237.473`, 1.3754`, 1532.48`, 1536.6`, 1.10745`, 1537.64`, 
              1907.44`, 562.81`, 2008.3`, 201.731`, 1869.27`, 1547.48`, 
              2337.52`, 1689.13`, 2280.14`, 1733.84`, 2244.68`, 1676.34`, 
              1744.2`, 2236.07`, 2288.99`, 405.156`, 1725.73`, 1532.48`, 
              1813.19`}, {3, 26, 34.64549013`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8236509644`, 0.3570715068`, 0.6813938503`, 0, 0.1`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130333`\), 1589.18`, 
              1594.49`, 1.47907`, 1592.24`, 1600.51`, 2.31477`, 400.371`, 
              417.92`, 1.25817`, 547.739`, 553.909`, 0.59554`, 400.371`, 
              411.859`, 0.81754`, 1602.76`, 1612.1`, 2.6305`, 202.7`, 
              238.419`, 1.3805`, 1589.18`, 1594.82`, 1.57128`, 1592.24`, 
              2006.34`, 547.739`, 2112.12`, 202.7`, 1968.29`, 1602.76`, 
              2456.64`, 1813.1`, 2413.43`, 1853.4`, 2378.82`, 1803.07`, 
              1861.69`, 2372.31`, 2419.98`, 400.371`, 1846.78`, 1589.18`, 
              1920.89`}, {3, 36, 35.39778722`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9259931776`, 0.3494453078`, 0.7944182674`, 0, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.162399`\), 1623.46`, 
              1631.27`, 2.22663`, 1626.33`, 1635.43`, 2.60321`, 397.795`, 
              415.29`, 1.24628`, 539.711`, 545.952`, 0.593729`, 397.795`, 
              409.271`, 0.811484`, 1632, 1642.46`, 3.00072`, 203.395`, 
              239.063`, 1.38271`, 1623.46`, 1630.99`, 2.14605`, 1626.33`, 
              2104.28`, 539.711`, 2176.71`, 203.395`, 2064.28`, 1632, 
              2556.39`, 1931.46`, 2519.74`, 1969.08`, 2485.69`, 1923.18`, 
              1975.98`, 2480.38`, 2525.06`, 397.795`, 1963.25`, 1623.46`, 
              2025.63`}, {3, 39, 36.04480578`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              1.00695157`, 0.3461612174`, 0.8872255019`, 0, 0.3`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.178783`\), 1643.81`, 
              1654.28`, 3.02679`, 1647.22`, 1656.92`, 2.80659`, 395.816`, 
              413.292`, 1.23893`, 534.044`, 540.355`, 0.594055`, 395.816`, 
              407.287`, 0.807164`, 1648.79`, 1660.48`, 3.3904`, 203.948`, 
              239.545`, 1.38321`, 1643.81`, 1653.1`, 2.68439`, 1647.22`, 
              2196.15`, 534.044`, 2214.98`, 203.948`, 2146.98`, 1648.79`, 
              2634.94`, 2037.48`, 2601.72`, 2073.21`, 2567.8`, 2030.36`, 
              2079.16`, 2563.25`, 2606.26`, 395.816`, 2067.72`, 1643.81`, 
              2122.64`}, {1, 48, 150.356082`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.0007559633739`, 0.07693755963`, 0.0002411903818`, 0, 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.020739`, 1150.65`, 
              1150.65`, \(-0.0000805121`\), 1150.66`, 
              1150.66`, \(-0.000239786`\), 226.93`, 226.944`, 0.000550983`, 
              227.121`, 227.123`, 0.0000857277`, 227.121`, 227.123`, 
              0.0000857277`, 1150.66`, 1150.66`, \(-0.00023979`\), 226.93`, 
              226.944`, 0.000550987`, 1150.65`, 1150.65`, \(-0.0000805121`\), 
              1150.66`, 1556.67`, 226.93`, 1554.54`, 226.93`, 1554.54`, 
              1150.66`, 1755.86`, 1556.67`, 1558.85`, 1617.27`, 1745.47`, 
              1556.67`, 1617.27`, 1558.85`, 1755.86`, 227.121`, 1554.54`, 
              1150.65`, 1556.68`}, {1, 71, 150.3564104`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.001665004568`, 0.07709538971`, 0.0004326947951`, 0, 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.020831`, 1150.74`, 
              1150.74`, \(-0.000081128`\), 1150.76`, 
              1150.75`, \(-0.000241607`\), 227.438`, 227.452`, 0.000555104`, 
              227.631`, 227.633`, 0.0000863748`, 227.631`, 227.633`, 
              0.00008637529999999999`, 1150.76`, 1150.75`, \(-0.000241617`\), 
              227.439`, 227.453`, 0.000555114`, 1150.74`, 
              1150.74`, \(-0.00008112830000000001`\), 1150.76`, 1556.69`, 
              227.438`, 1554.54`, 227.439`, 1554.54`, 1150.76`, 1755.87`, 
              1556.68`, 1558.88`, 1617.28`, 1745.48`, 1556.69`, 1617.28`, 
              1558.87`, 1755.87`, 227.631`, 1554.54`, 1150.74`, 
              1556.69`}, {3, 51, 27.96377973`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1981690921`, 0.3111400963`, 0.7495354154`, 0.1`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.541664`\), 1648.49`, 
              1648.28`, \(-0.0626825`\), 1649.86`, 1658.18`, 2.412`, 
              464.642`, 480.805`, 1.33894`, 702.074`, 706.366`, 0.529547`, 
              464.642`, 477.055`, 1.02418`, 1659.74`, 1660.05`, 0.0889279`, 
              183.847`, 224.982`, 1.47347`, 1652.23`, 
              1649.98`, \(-0.648972`\), 1648.49`, 1800.2`, 702.074`, 
              2032.79`, 183.847`, 1794.84`, 1667.52`, 2265.49`, 1652.23`, 
              2151.9`, 1660.69`, 2146.99`, 1649.86`, 1662.21`, 2145.29`, 
              2153.72`, 464.642`, 1659.74`, 1658.73`, 1724.77`}, {2, 38, 
              26.23439234`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1955529823`, 0.2654909245`, 0.6650560358`, 0.1`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.447971`\), 1513.95`, 
              1513.96`, 0.00286528`, 1526.07`, 1526.05`, \(-0.00640337`\), 
              436.036`, 451.4`, 1.19458`, 640.062`, 645.05`, 0.561722`, 
              436.036`, 448.189`, 0.941498`, 1523.54`, 1523.85`, 0.0828431`, 
              192.702`, 228.61`, 1.32551`, 1516.36`, 
              1516.26`, \(-0.0261082`\), 1562.09`, 1686.65`, 640.062`, 
              1921.58`, 192.702`, 1639.49`, 1623.49`, 2127.56`, 1516.36`, 
              2034.47`, 1524.46`, 2029.96`, 1513.95`, 1526.07`, 2028.37`, 
              2036.16`, 436.036`, 1523.54`, 1545.19`, 1648.68`}, {3, 55, 
              26.64150476`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2150226928`, 0.2849849125`, 0.5875073145`, 0.1`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.396103`\), 1537.67`, 
              1537.51`, \(-0.0421222`\), 1554.66`, 1554.55`, \(-0.0304689`\), 
              432.995`, 448.552`, 1.20164`, 636.513`, 641.516`, 0.560186`, 
              432.995`, 445.086`, 0.930232`, 1551.03`, 1551.32`, 0.0799803`, 
              193.241`, 228.95`, 1.32088`, 1541.05`, 
              1540.64`, \(-0.110184`\), 1580.59`, 1697.22`, 636.513`, 
              1942.53`, 193.241`, 1682.2`, 1612.73`, 2163.46`, 1541.05`, 
              2058.37`, 1552.41`, 2051.99`, 1537.67`, 1554.66`, 2049.74`, 
              2060.77`, 432.995`, 1551.03`, 1568.7`, 1647.38`}, {1, 21, 
              27.34014421`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2553684009`, 0.3243325355`, 0.5000972694`, 0.1`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323175`\), 1574.8`, 
              1573.81`, \(-0.272979`\), 1593.86`, 1596.77`, 0.81392`, 
              434.263`, 450.357`, 1.24743`, 642.469`, 647.412`, 0.558732`, 
              434.263`, 446.277`, 0.92688`, 1594.23`, 1596.72`, 0.695192`, 
              192.57`, 229.094`, 1.34936`, 1580.34`, 
              1578.44`, \(-0.523668`\), 1593.86`, 1739.65`, 642.469`, 
              1979.35`, 192.57`, 1759.02`, 1594.23`, 2228.84`, 1580.34`, 
              2099.4`, 1599.01`, 2088.7`, 1574.8`, 1602.7`, 2084.92`, 
              2103.44`, 434.263`, 1596.5`, 1596.6`, 1659.54`}, {1, 40, 
              30.1932564`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3771860313`, 0.3352920112`, 0.379494855`, 0.1`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153249`\), 1498.38`, 
              1497.89`, \(-0.127452`\), 1533.15`, 1535.34`, 0.58849`, 
              419.824`, 435.544`, 1.17807`, 609.38`, 614, 0.495283`, 
              419.824`, 430.458`, 0.792184`, 1529.63`, 1537.32`, 2.06569`, 
              201.341`, 234.226`, 1.25498`, 1508.65`, 
              1506.69`, \(-0.516005`\), 1540.29`, 1688.94`, 609.38`, 
              1932.65`, 201.341`, 1734.36`, 1529.63`, 2192.94`, 1508.65`, 
              2063.56`, 1542.21`, 2042.99`, 1498.38`, 1549.72`, 2036.45`, 
              2070.63`, 419.824`, 1537.05`, 1533.15`, 1605.91`}, {3, 26, 
              34.35651904`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.7092882236`, 0.3401052295`, 0.5943701433`, 0.1`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.149105`\), 1592.17`, 
              1592.24`, 0.0191064`, 1593.66`, 1603, 2.61367`, 408.681`, 
              426.167`, 1.27909`, 568.809`, 574.687`, 0.588845`, 408.681`, 
              420.075`, 0.827385`, 1602.27`, 1623.45`, 5.98521`, 199.518`, 
              235.849`, 1.38586`, 1592.17`, 1594.92`, 0.768109`, 1593.66`, 
              1888.19`, 568.809`, 2093.8`, 199.518`, 1889.45`, 1602.27`, 
              2362.3`, 1698.26`, 2299.44`, 1738.46`, 2266.31`, 1687.7`, 
              1747.22`, 2259.48`, 2306.38`, 408.681`, 1732.44`, 1592.17`, 
              1802.51`}, {3, 21, 35.06524065`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8458668867`, 0.3316445815`, 0.7327794565`, 0.1`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.190092`\), 1634.82`, 
              1640.61`, 1.65938`, 1636.64`, 1646.49`, 2.83124`, 403.743`, 
              421.113`, 1.25536`, 553.107`, 559.131`, 0.587065`, 403.743`, 
              415.114`, 0.815838`, 1638.35`, 1656.16`, 5.14101`, 201.946`, 
              237.875`, 1.38454`, 1634.82`, 1641.96`, 2.04983`, 1636.64`, 
              2002.47`, 553.107`, 2174.27`, 201.946`, 1998.24`, 1638.35`, 
              2484.26`, 1837.4`, 2437.39`, 1874.86`, 2404.37`, 1828.98`, 
              1881.91`, 2399.02`, 2442.76`, 403.743`, 1869.68`, 1634.82`, 
              1925.13`}, {3, 25, 35.90865264`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9413351703`, 0.3305256986`, 0.8373529084`, 0.1`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.210726`\), 1657.17`, 
              1668.13`, 3.19289`, 1660.81`, 1670.84`, 2.92706`, 401.551`, 
              418.88`, 1.24569`, 545.853`, 551.939`, 0.58542`, 401.551`, 
              412.882`, 0.808546`, 1657.21`, 1674.62`, 5.08192`, 202.828`, 
              238.672`, 1.38655`, 1657.17`, 1667.55`, 3.02548`, 1660.81`, 
              2107.69`, 545.853`, 2218.97`, 202.828`, 2098.25`, 1657.21`, 
              2578.26`, 1957.52`, 2540.07`, 1993.18`, 2507.02`, 1950.37`, 
              1999.19`, 2502.5`, 2544.59`, 401.551`, 1988.47`, 1657.17`, 
              2036.42`}, {3, 34, 36.78498011`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              1.019360674`, 0.3308649298`, 0.9249580244`, 0.1`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220071`\), 1670.15`, 
              1685.04`, 4.37503`, 1675.14`, 1685.88`, 3.16134`, 399.695`, 
              417.022`, 1.23991`, 540.47`, 546.624`, 0.586111`, 399.695`, 
              411.011`, 0.803749`, 1668.68`, 1686.47`, 5.22981`, 203.399`, 
              239.175`, 1.38731`, 1670.15`, 1682.96`, 3.76362`, 1675.14`, 
              2203.47`, 540.47`, 2241.14`, 203.399`, 2185.09`, 1668.68`, 
              2648.65`, 2062.87`, 2616.99`, 2097.1`, 2583.74`, 2056.6`, 
              2102.38`, 2579.78`, 2620.94`, 399.695`, 2092.64`, 1670.15`, 
              2136.52`}, {5, 101, 16355.71195`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.487591316`, 0.3854697625`, \(-0.002251717398`\), 0.1`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.185289`, 1583.15`, 
              1583.81`, 0.183379`, 1620.75`, 1620.7`, \(-0.0125282`\), 
              1565.51`, 1565.51`, 0.00178118`, 1593.28`, 1596.39`, 0.866728`, 
              1565.51`, 1565.93`, 0.116781`, 1593.28`, 1593.53`, 0.068885`, 
              1565.19`, 1565.52`, 0.0892121`, 1568.57`, 1568.86`, 0.0782433`, 
              1620.75`, 1906.9`, 1602.76`, 1656.9`, 1565.19`, 1623.34`, 
              1818.89`, 2083.72`, 1568.57`, 1699.91`, 1668, 1820.01`, 
              1583.15`, 1738.88`, 1615.92`, 1859.16`, 1565.51`, 1593.28`, 
              1635.33`, 1822.52`}, {1, 95, 15817.63522`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1926831175`, 1.431867211`, \(-0.00003018768987`\), 0.1`, 
              0.5`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196679`, 1635.25`, 
              1630.56`, \(-1.34317`\), 1644.93`, 1645.61`, 0.194305`, 
              1556.58`, 1560.29`, 1.01092`, 1558.91`, 1562.19`, 0.897076`, 
              1556.58`, 1559.96`, 0.92174`, 1593.38`, 1593.83`, 0.125782`, 
              1554.56`, 1558.07`, 0.955652`, 1642.79`, 
              1641.85`, \(-0.27022`\), 1644.93`, 2044.89`, 1558.91`, 1686.8`, 
              1554.56`, 1643.66`, 1876.78`, 2041.56`, 1642.79`, 2016.19`, 
              1677.72`, 1876.86`, 1635.25`, 1687.46`, 1868.07`, 2024.59`, 
              1556.58`, 1593.38`, 1652.05`, 1871.22`}, {2, 34, 
              30.50016708`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2092252755`, 0.3355061661`, 0.7487793192`, 0.2`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.57002`\), 1654.66`, 
              1653.19`, \(-0.425954`\), 1654.74`, 1665.58`, 3.15132`, 
              475.004`, 491.744`, 1.4179`, 730.773`, 734.714`, 0.50612`, 
              475.004`, 487.542`, 1.05741`, 1665.85`, 1666.23`, 0.112856`, 
              179.282`, 222.897`, 1.5369`, 1657.31`, 
              1655.18`, \(-0.617292`\), 1654.74`, 1830.13`, 730.773`, 
              2050.64`, 179.282`, 1817.8`, 1677.22`, 2284.41`, 1657.31`, 
              2155.08`, 1666.93`, 2149.54`, 1654.66`, 1668.61`, 2147.61`, 
              2157.14`, 475.004`, 1665.85`, 1665.99`, 1742.82`}, {2, 40, 
              500.6655654`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1188163722`, 1.043251361`, 0.6694502796`, 0.2`, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-2.508545`\), 1705.81`, 
              1703.01`, \(-0.836744`\), 1732.97`, 1735, 0.615742`, 0, 0, 
              0.427727`, 1000.47`, 987.908`, \(-2.18907`\), 0, 0, 0.367399`, 
              1958.01`, 2031.08`, 25.5389`, 253.332`, 
              215.283`, \(-1.56222`\), 1732.97`, 1734.11`, 0.345217`, 
              1705.81`, 2883.69`, 1000.47`, 2659.49`, 253.332`, 2559.61`, 
              1958.01`, 2851.08`, 2704.71`, 2899.99`, 2747.45`, 2889.41`, 
              2708.51`, 2735.92`, 2886.6`, 2904.69`, 0, 2596.96`, 1732.97`, 
              2960.62`}, {2, 35, 26.71911473`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2232862324`, 0.2777022081`, 0.5775434833`, 0.2`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.40023`\), 1454.81`, 
              1455.16`, 0.0908473`, 1471.91`, 1472.01`, 0.0271735`, 431.837`, 
              447.566`, 1.21193`, 636.952`, 641.979`, 0.563298`, 431.837`, 
              443.95`, 0.929464`, 1468.23`, 1468.59`, 0.0925981`, 193.294`, 
              229.026`, 1.32216`, 1458.25`, 1458.6`, 0.0887981`, 1524.55`, 
              1670.27`, 636.952`, 1900.98`, 193.294`, 1602.02`, 1606.79`, 
              2092.82`, 1458.25`, 1987.48`, 1469.59`, 1981.29`, 1454.81`, 
              1471.91`, 1979.11`, 1989.8`, 431.837`, 1468.23`, 1497.89`, 
              1632.27`}, {1, 26, 27.46827683`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.26562493`, 0.3002590686`, 0.4824286264`, 0.2`, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322253`\), 1454.03`, 
              1454.53`, 0.126285`, 1480.74`, 1480.88`, 0.0387202`, 428.163`, 
              444.223`, 1.22761`, 632.721`, 637.727`, 0.557234`, 428.163`, 
              440.055`, 0.904699`, 1474.75`, 1475.13`, 0.0981436`, 194.079`, 
              229.601`, 1.3186`, 1459.44`, 1459.87`, 0.108117`, 1532.02`, 
              1670.28`, 632.721`, 1920.75`, 194.079`, 1632.86`, 1588.44`, 
              2116.45`, 1459.44`, 1994.87`, 1477.04`, 1985.15`, 1454.03`, 
              1480.74`, 1981.78`, 1998.47`, 428.163`, 1474.75`, 1504.43`, 
              1622.39`}, {1, 25, 31.18197015`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4871543697`, 0.3251594621`, 0.4405083275`, 0.2`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.165801`\), 1512.16`, 
              1510.14`, \(-0.534989`\), 1545.3`, 1553.7`, 2.28044`, 423.83`, 
              440.495`, 1.26209`, 609.781`, 614.747`, 0.532783`, 423.83`, 
              434.651`, 0.813934`, 1555.47`, 1555.85`, 0.103144`, 201.172`, 
              236.097`, 1.33804`, 1523.29`, 1518.51`, \(-1.27375`\), 
              1558.33`, 1725.92`, 609.781`, 2052.87`, 201.172`, 1749.5`, 
              1574.74`, 2208.57`, 1523.29`, 2103.88`, 1560.66`, 2076.39`, 
              1512.16`, 1569.63`, 2069.23`, 2111.35`, 423.83`, 1555.47`, 
              1545.3`, 1649.31`}, {1, 66, 38.98860016`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.7440244693`, 0.3011079734`, 0.659060567`, 0.2`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.213419`\), 1638.73`, 
              1636.5`, \(-0.639476`\), 1643.26`, 1650.24`, 2.01465`, 
              395.986`, 413.523`, 1.24382`, 546.079`, 552.493`, 0.617385`, 
              395.986`, 407.851`, 0.835594`, 1637.47`, 1681.21`, 12.7181`, 
              195.704`, 231.915`, 1.35668`, 1643.26`, 
              1641.75`, \(-0.43377`\), 1638.73`, 1872.54`, 546.079`, 
              2163.89`, 195.704`, 1891.3`, 1637.47`, 2385.08`, 1724.69`, 
              2330.86`, 1760.98`, 2299.78`, 1716.46`, 1767.94`, 2294.63`, 
              2336.05`, 395.986`, 1756.19`, 1643.26`, 1796.39`}, {3, 33, 
              37.13181161`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8656635245`, 0.3137543666`, 0.7785384391`, 0.2`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.24017`\), 1675.05`, 
              1680.04`, 1.46569`, 1675.84`, 1686.39`, 3.10709`, 404.773`, 
              422.066`, 1.25282`, 554.598`, 560.591`, 0.585609`, 404.773`, 
              416.129`, 0.816805`, 1663.08`, 1695.25`, 9.46646`, 200.447`, 
              236.672`, 1.38739`, 1675.84`, 1683.06`, 2.12359`, 1675.05`, 
              2002.57`, 554.598`, 2222.29`, 200.447`, 2020.24`, 1663.08`, 
              2507.69`, 1863.4`, 2461.09`, 1898.88`, 2429.11`, 1856.16`, 
              1905.01`, 2424.58`, 2465.64`, 404.773`, 1894.56`, 1675.84`, 
              1929.12`}, {3, 29, 37.56112654`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9564318651`, 0.3190872458`, 0.8752979353`, 0.2`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253539`\), 1688.2`, 
              1701.51`, 3.95191`, 1692.05`, 1703.31`, 3.3498`, 405.995`, 
              423.189`, 1.2492`, 553.26`, 559.147`, 0.573766`, 405.995`, 
              417.145`, 0.804212`, 1676.5`, 1704.35`, 8.24719`, 201.733`, 
              237.953`, 1.39532`, 1688.2`, 1701.18`, 3.85246`, 1692.05`, 
              2113.12`, 553.26`, 2250.15`, 201.733`, 2124.38`, 1676.5`, 
              2596.09`, 1981.17`, 2557.15`, 2015.49`, 2524.72`, 1974.79`, 
              2020.89`, 2520.73`, 2561.14`, 405.995`, 2011.48`, 1688.2`, 
              2045.64`}, {5, 100, 15886.93239`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5925580674`, 1.84331453`, \(-0.002272353285`\), 0.2`, 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196624`, 1615.74`, 
              1613.47`, \(-0.643937`\), 1647.8`, 1648.24`, 0.127529`, 
              1563.79`, 1567.39`, 0.985242`, 1577.04`, 1578.31`, 0.348712`, 
              1563.79`, 1566.07`, 0.623022`, 1595.14`, 1595.63`, 0.137462`, 
              1555.38`, 1558.14`, 0.752418`, 1636.73`, 1637.16`, 0.122321`, 
              1647.8`, 2058.97`, 1577.04`, 1690.36`, 1555.38`, 1642.11`, 
              1882.54`, 2042.34`, 1636.73`, 1987.48`, 1680.46`, 1882.18`, 
              1615.74`, 1711.44`, 1841.9`, 2021.06`, 1563.79`, 1595.14`, 
              1666.53`, 1878.52`}, {1, 28, 15830.05108`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5756147536`, 2.921972179`, 0.0003657871351`, 0.2`, 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196645`, 1631.63`, 
              1627.68`, \(-1.12822`\), 1649.81`, 1650.51`, 0.201855`, 
              1559.14`, 1563.32`, 1.14313`, 1564.76`, 1567.75`, 0.821968`, 
              1559.14`, 1562.5`, 0.919804`, 1594.08`, 1594.63`, 0.15257`, 
              1554.67`, 1558.36`, 1.0057`, 1645.44`, 
              1644.35`, \(-0.313186`\), 1649.81`, 2049.34`, 1564.76`, 
              1690.15`, 1554.67`, 1646.62`, 1888.53`, 2041.66`, 1645.44`, 
              2011.25`, 1680.43`, 1888.26`, 1631.63`, 1697.73`, 1872.12`, 
              2024.2`, 1559.14`, 1594.08`, 1661.29`, 1883.79`}, {5, 101, 
              16511.92643`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6195163215`, 0.6534086022`, 0.005906987103`, 0.2`, 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196842`, 1583.91`, 
              1584.53`, 0.173299`, 1635.81`, 1635.84`, 0.00919565`, 1570.95`, 
              1571.31`, 0.10006`, 1599.12`, 1597.76`, \(-0.381745`\), 
              1570.95`, 1571.41`, 0.127495`, 1599.12`, 1599.32`, 0.0555849`, 
              1567.74`, 1568.17`, 0.119666`, 1585.67`, 
              1585.59`, \(-0.0222251`\), 1635.81`, 2089.19`, 1610.35`, 
              1692.45`, 1567.74`, 1630.41`, 1848.67`, 2055.45`, 1585.67`, 
              1792.47`, 1677.2`, 1848.73`, 1583.91`, 1743.69`, 1673.85`, 
              1933.68`, 1570.95`, 1599.12`, 1665.26`, 1850.39`}, {3, 28, 
              26.2169557`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1954145559`, 0.2504208148`, 0.7263097895`, 0.3`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.489099`\), 1396.1`, 
              1396.48`, 0.09444`, 1405.95`, 1406.08`, 0.0317713`, 436.258`, 
              451.72`, 1.20292`, 642.328`, 647.322`, 0.564226`, 436.258`, 
              448.485`, 0.947792`, 1403.9`, 1404.25`, 0.0861988`, 192.444`, 
              228.55`, 1.3318`, 1398.08`, 1398.54`, 0.112661`, 1463.97`, 
              1660.27`, 642.328`, 1854.89`, 192.444`, 1517.3`, 1613.92`, 
              2023.65`, 1398.08`, 1934.4`, 1404.6`, 1930.89`, 1396.1`, 
              1405.95`, 1929.67`, 1935.7`, 436.258`, 1403.9`, 1434.97`, 
              1632.2`}, {3, 50, 26.47934522`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096437448`, 0.2608428192`, 0.653773566`, 0.3`, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.452309`\), 1398.7`, 
              1399.15`, 0.11032`, 1411.43`, 1411.58`, 0.0370473`, 433.776`, 
              449.407`, 1.20959`, 639.909`, 644.923`, 0.564431`, 433.776`, 
              445.972`, 0.940085`, 1408.74`, 1409.11`, 0.0912009`, 192.848`, 
              228.789`, 1.32771`, 1401.27`, 1401.8`, 0.12926`, 1472.26`, 
              1660.3`, 639.909`, 1864.12`, 192.848`, 1532.1`, 1609.51`, 
              2035.05`, 1401.27`, 1938.4`, 1409.68`, 1933.87`, 1398.7`, 
              1411.43`, 1932.29`, 1940.09`, 433.776`, 1408.74`, 1441.44`, 
              1629.72`}, {3, 34, 26.89839887`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2330073646`, 0.2743439186`, 0.5716264973`, 0.3`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401797`\), 1397.38`, 
              1397.95`, 0.138438`, 1414.99`, 1415.18`, 0.0473136`, 430.511`, 
              446.381`, 1.21929`, 636.318`, 641.358`, 0.564163`, 430.511`, 
              442.63`, 0.927111`, 1411.18`, 1411.56`, 0.0953514`, 193.432`, 
              229.168`, 1.32319`, 1400.96`, 1401.61`, 0.158076`, 1479.02`, 
              1659.37`, 636.318`, 1876.43`, 193.432`, 1548.99`, 1601.61`, 
              2047.86`, 1400.96`, 1941.02`, 1412.55`, 1934.76`, 1397.38`, 
              1414.99`, 1932.58`, 1943.34`, 430.511`, 1411.18`, 1445.75`, 
              1625.03`}, {1, 25, 27.75148677`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2834310067`, 0.292208027`, 0.4758103914`, 0.3`, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32274`\), 1392.07`, 
              1392.88`, 0.197596`, 1419.76`, 1420.05`, 0.0716558`, 426.39`, 
              442.608`, 1.23483`, 630.385`, 635.422`, 0.558615`, 426.39`, 
              438.244`, 0.898057`, 1413.51`, 1413.91`, 0.0984197`, 194.438`, 
              229.911`, 1.31889`, 1397.74`, 1398.6`, 0.212221`, 1484.91`, 
              1658.81`, 630.385`, 1903.78`, 194.438`, 1574.78`, 1588.17`, 
              2067.92`, 1397.74`, 1947.24`, 1415.78`, 1937.23`, 1392.07`, 
              1419.76`, 1933.86`, 1950.82`, 426.39`, 1413.51`, 1448.7`, 
              1618.16`}, {1, 29, 31.88433096`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5701346198`, 0.300438937`, 0.5163481355`, 0.3`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.216022`\), 1532.03`, 
              1529.08`, \(-0.792588`\), 1562.93`, 1568.03`, 1.3999`, 
              418.421`, 435.374`, 1.26825`, 593.14`, 598.538`, 0.563574`, 
              418.421`, 429.555`, 0.827242`, 1573.47`, 1574.06`, 0.160539`, 
              200.63`, 236.126`, 1.35835`, 1541.81`, 1535.62`, \(-1.67085`\), 
              1578.79`, 1746.14`, 593.14`, 2110.23`, 200.63`, 1744.84`, 
              1617.73`, 2225.55`, 1541.81`, 2136.31`, 1578.04`, 2108.15`, 
              1532.03`, 1586.23`, 2102.04`, 2142.57`, 418.421`, 1573.47`, 
              1562.93`, 1681.07`}, {2, 63, 48.14545078`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.780097974`, 0.2656937523`, 0.7127181469`, 0.3`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.260336`\), 1673.55`, 
              1667.11`, \(-1.88601`\), 1683.31`, 1683.77`, 0.138129`, 
              383.382`, 399.992`, 1.14007`, 520.512`, 526.877`, 0.584102`, 
              383.382`, 394.85`, 0.781989`, 1658.87`, 1723.33`, 19.1018`, 
              199.929`, 233.065`, 1.25709`, 1683.31`, 
              1674.02`, \(-2.73161`\), 1673.55`, 1849.16`, 520.512`, 
              2213.11`, 199.929`, 1880.94`, 1658.87`, 2393.63`, 1741.45`, 
              2351.53`, 1775.39`, 2321.6`, 1735.08`, 1780.86`, 2317.73`, 
              2355.42`, 383.382`, 1771.25`, 1683.31`, 1780.77`}, {3, 29, 
              42.31020688`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8831608117`, 0.3008577154`, 0.8207296374`, 0.3`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.28368`\), 1711.8`, 
              1716.58`, 1.43463`, 1716.72`, 1723.77`, 2.12369`, 403.476`, 
              420.943`, 1.26167`, 553.388`, 559.532`, 0.59907`, 403.476`, 
              415.044`, 0.829606`, 1682.11`, 1733.55`, 15.3942`, 196.362`, 
              233.524`, 1.3997`, 1716.72`, 1721.49`, 1.43786`, 1711.8`, 
              2005.33`, 553.388`, 2262.03`, 196.362`, 2042.84`, 1682.11`, 
              2530.21`, 1891.82`, 2485.2`, 1925.8`, 2453.96`, 1885.48`, 
              1931.19`, 2450.03`, 2489.14`, 403.476`, 1922.05`, 1716.72`, 
              1932.57`}, {3, 35, 40.8579568`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9708517175`, 0.3104475356`, 0.9102777335`, 0.3`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.29135`\), 1719.79`, 
              1733.39`, 4.11424`, 1722.49`, 1734.85`, 3.74525`, 409.038`, 
              426.19`, 1.25519`, 558.638`, 564.409`, 0.567812`, 409.038`, 
              420.101`, 0.803734`, 1692.55`, 1734.16`, 12.4931`, 199.836`, 
              236.599`, 1.40575`, 1719.79`, 1733.89`, 4.26749`, 1722.49`, 
              2118.07`, 558.638`, 2275.01`, 199.836`, 2147.26`, 1692.55`, 
              2611.49`, 2004.2`, 2572.55`, 2037.46`, 2540.56`, 1998.43`, 
              2042.36`, 2536.97`, 2576.13`, 409.038`, 2033.94`, 1719.79`, 
              2052.2`}, {5, 102, 15896.77294`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5925159497`, 1.765004335`, 0.002179089959`, 0.3`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196717`, 1614.08`, 
              1611.95`, \(-0.599822`\), 1647.5`, 1647.92`, 0.120621`, 
              1564.27`, 1567.77`, 0.960131`, 1578.46`, 1579.51`, 0.28986`, 
              1564.27`, 1566.43`, 0.591997`, 1595.27`, 1595.75`, 0.134435`, 
              1555.51`, 1558.16`, 0.722928`, 1635.53`, 1635.9`, 0.106586`, 
              1647.5`, 2060.1`, 1578.46`, 1690.73`, 1555.51`, 1641.61`, 
              1881.98`, 2042.46`, 1635.53`, 1983.93`, 1680.44`, 1881.32`, 
              1614.08`, 1712.86`, 1837.79`, 2020.41`, 1564.27`, 1595.27`, 
              1667.1`, 1877.77`}, {5, 103, 15870.54174`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6265822075`, 2.115903863`, \(-0.001127263524`\), 0.3`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196663`, 1618.96`, 
              1616.39`, \(-0.730827`\), 1648.58`, 1649.07`, 0.142754`, 
              1562.96`, 1566.74`, 1.03619`, 1574.64`, 1576.26`, 0.448669`, 
              1562.96`, 1565.46`, 0.683729`, 1594.89`, 1595.41`, 0.14401`, 
              1555.16`, 1558.14`, 0.81183`, 1639.08`, 1639.62`, 0.15368`, 
              1648.58`, 2057.04`, 1574.64`, 1690.79`, 1555.16`, 1643.22`, 
              1885.01`, 2042.12`, 1639.08`, 1993.85`, 1680.62`, 1884.4`, 
              1618.96`, 1708.95`, 1849.91`, 2022.08`, 1562.96`, 1594.89`, 
              1666.1`, 1880.59`}, {5, 103, 15857.11053`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6787564168`, 2.513273752`, 0.001091976418`, 0.3`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196639`, 1622.44`, 
              1619.53`, \(-0.827803`\), 1649.35`, 1649.9`, 0.159442`, 
              1562.07`, 1566.02`, 1.08461`, 1572.14`, 1574.14`, 0.552342`, 
              1562.07`, 1564.81`, 0.750652`, 1594.69`, 1595.23`, 0.149766`, 
              1554.99`, 1558.2`, 0.874275`, 1641.38`, 1642.04`, 0.188992`, 
              1649.35`, 2055.06`, 1572.14`, 1690.88`, 1554.99`, 1644.42`, 
              1887.12`, 2041.96`, 1641.38`, 1999.65`, 1680.78`, 1886.55`, 
              1622.44`, 1706.31`, 1857.62`, 2022.91`, 1562.07`, 1594.69`, 
              1665.5`, 1882.6`}, {2, 38, 26.37566396`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2019705775`, 0.251271744`, 0.7240026627`, 0.4`, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.48845`\), 1364.81`, 
              1365.28`, 0.112094`, 1375.06`, 1375.22`, 0.0387087`, 434.901`, 
              450.469`, 1.20763`, 641.172`, 646.183`, 0.565216`, 434.901`, 
              447.15`, 0.946666`, 1372.91`, 1373.27`, 0.0873718`, 192.634`, 
              228.717`, 1.33211`, 1366.88`, 1367.46`, 0.136913`, 1437.98`, 
              1655.55`, 641.172`, 1842.26`, 192.634`, 1490.5`, 1609.11`, 
              2001.67`, 1366.88`, 1909.95`, 1373.63`, 1906.33`, 1364.81`, 
              1375.06`, 1905.08`, 1911.29`, 434.901`, 1372.91`, 1406.26`, 
              1627.54`}, {3, 54, 26.65742179`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2174913802`, 0.2608489147`, 0.6508524945`, 0.4`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.452064`\), 1362.55`, 
              1363.1`, 0.132312`, 1375.8`, 1375.99`, 0.045863`, 432.43`, 
              448.174`, 1.21479`, 638.809`, 643.837`, 0.56502`, 432.43`, 
              444.641`, 0.938363`, 1372.98`, 1373.37`, 0.092372`, 193.02`, 
              228.95`, 1.32836`, 1365.25`, 1365.91`, 0.159612`, 1442.23`, 
              1655.29`, 638.809`, 1850.06`, 193.02`, 1500.74`, 1604.93`, 
              2009.35`, 1365.25`, 1910.33`, 1373.94`, 1905.66`, 1362.55`, 
              1375.8`, 1904.04`, 1912.05`, 432.43`, 1372.98`, 1408.22`, 
              1625.17`}, {3, 48, 27.11187757`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2435130222`, 0.272746629`, 0.5680796661`, 0.4`, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401848`\), 1356.61`, 
              1357.31`, 0.167112`, 1374.95`, 1375.19`, 0.0591776`, 429.151`, 
              445.141`, 1.22484`, 635.015`, 640.066`, 0.564329`, 429.151`, 
              441.268`, 0.924067`, 1370.96`, 1371.36`, 0.0955524`, 193.625`, 
              229.354`, 1.32411`, 1360.36`, 1361.19`, 0.197511`, 1445.11`, 
              1654.8`, 635.015`, 1862.52`, 193.625`, 1513.21`, 1598.4`, 
              2018.76`, 1360.36`, 1910.06`, 1372.35`, 1903.58`, 1356.61`, 
              1374.95`, 1901.36`, 1912.42`, 429.151`, 1370.96`, 1408.13`, 
              1621.64`}, {1, 21, 28.08118957`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.303344631`, 0.2871781503`, 0.4733538375`, 0.4`, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322802`\), 1353.28`, 
              1354.29`, 0.240052`, 1382.51`, 1382.88`, 0.0899221`, 425.326`, 
              441.645`, 1.23959`, 628.454`, 633.478`, 0.555458`, 425.326`, 
              437.101`, 0.889772`, 1375.9`, 1376.29`, 0.0939663`, 194.73`, 
              230.184`, 1.31994`, 1359.28`, 1360.4`, 0.266719`, 1452.36`, 
              1658.63`, 628.454`, 1899.42`, 194.73`, 1540.86`, 1590.91`, 
              2041.79`, 1359.28`, 1921.54`, 1378.18`, 1910.83`, 1353.28`, 
              1382.51`, 1907.37`, 1925.2`, 425.326`, 1375.9`, 1412.49`, 
              1620.62`}, {3, 33, 32.33289113`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.625461168`, 0.2875249167`, 0.5762418815`, 0.4`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.266856`\), 1554.59`, 
              1551.31`, \(-0.891137`\), 1584.38`, 1588.18`, 1.05672`, 
              418.987`, 435.857`, 1.26357`, 590.443`, 595.788`, 0.555516`, 
              418.987`, 430.066`, 0.824207`, 1594.36`, 1595.05`, 0.19318`, 
              200.686`, 236.41`, 1.36811`, 1563.22`, 1556.68`, \(-1.78809`\), 
              1601.4`, 1766.73`, 590.443`, 2150.61`, 200.686`, 1755.23`, 
              1648.58`, 2253.35`, 1563.22`, 2170.45`, 1598.39`, 2142.23`, 
              1554.59`, 1605.7`, 2136.93`, 2175.83`, 418.987`, 1594.36`, 
              1584.38`, 1706.52`}, {2, 30, 49.62622684`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.813660091`, 0.3376590872`, 0.6954800688`, 0.4`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291276`\), 1668.03`, 
              1647.37`, \(-6.00129`\), 1674.55`, 1679.83`, 1.55045`, 
              472.324`, 486.476`, 1.18886`, 672.366`, 673.768`, 0.165308`, 
              472.324`, 479.403`, 0.590314`, 1679.44`, 1720.01`, 12.0845`, 
              215.05`, 248.528`, 1.35977`, 1668.03`, 1650.37`, \(-5.13464`\), 
              1674.86`, 1890.86`, 672.366`, 2221.08`, 215.05`, 1916.12`, 
              1679.44`, 2394.93`, 1683.75`, 2304.19`, 1722.91`, 2271.27`, 
              1674.55`, 1730.59`, 2265.54`, 2309.94`, 472.324`, 1718.88`, 
              1668.03`, 1797.53`}, {3, 29, 51.82450535`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8991082865`, 0.2886666655`, 0.8619492945`, 0.4`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32186`\), 1748.22`, 
              1751.6`, 1.03718`, 1757.83`, 1759.45`, 0.499725`, 398.066`, 
              416.012`, 1.28003`, 546.211`, 552.817`, 0.636127`, 398.066`, 
              410.195`, 0.858972`, 1698.17`, 1770.68`, 22.0373`, 190.23`, 
              228.851`, 1.41808`, 1757.83`, 1758.21`, 0.11554`, 1748.22`, 
              2009.48`, 546.211`, 2296.7`, 190.23`, 2066.71`, 1698.17`, 
              2552.41`, 1924.09`, 2510.79`, 1956.7`, 2480.14`, 1918.54`, 
              1961.45`, 2476.74`, 2514.19`, 398.066`, 1953.38`, 1757.83`, 
              1936.9`}, {2, 65, 16206.88193`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6534367899`, 1.072073111`, 0.001839506183`, 0.4`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.197445`, 1592, 
              1591.78`, \(-0.0632694`\), 1642.95`, 1643.1`, 0.0439088`, 
              1569.94`, 1571.63`, 0.465615`, 1598.19`, 
              1595.15`, \(-0.850907`\), 1569.94`, 1570.85`, 0.249329`, 
              1598.19`, 1598.4`, 0.0577042`, 1560.14`, 1561.29`, 0.314791`, 
              1611.55`, 1611.29`, \(-0.0735161`\), 1642.95`, 2079.07`, 
              1601.02`, 1692.74`, 1560.14`, 1635.14`, 1868.94`, 2047.08`, 
              1611.55`, 1900.24`, 1680.14`, 1867.54`, 1592, 1734.06`, 
              1757.61`, 1994.63`, 1569.94`, 1598.19`, 1675.11`, 
              1866.73`}, {5, 101, 15863.67493`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5790304018`, 2.084442545`, 0.002904297284`, 0.4`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196769`, 1621.05`, 
              1618.26`, \(-0.791657`\), 1648.55`, 1649.07`, 0.151569`, 
              1562.3`, 1566.13`, 1.04958`, 1572.65`, 1574.54`, 0.522445`, 
              1562.3`, 1564.93`, 0.720718`, 1594.76`, 1595.28`, 0.14437`, 
              1555.11`, 1558.19`, 0.84013`, 1640.09`, 1640.69`, 0.173102`, 
              1648.55`, 2055.49`, 1572.65`, 1690.84`, 1555.11`, 1643.65`, 
              1885.02`, 2042.06`, 1640.09`, 1997.18`, 1680.48`, 1884.36`, 
              1621.05`, 1706.86`, 1853.43`, 2022.65`, 1562.3`, 1594.76`, 
              1664.72`, 1880.36`}, {5, 101, 15847.75661`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6643902918`, 2.65511043`, \(-0.000111983055`\), 0.4`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196674`, 1624.91`, 
              1621.73`, \(-0.903526`\), 1649.53`, 1650.12`, 0.17144`, 
              1561.28`, 1565.31`, 1.10586`, 1570.06`, 1572.34`, 0.630039`, 
              1561.28`, 1564.19`, 0.79747`, 1594.5`, 1595.04`, 0.151334`, 
              1554.88`, 1558.22`, 0.912521`, 1642.62`, 1642.71`, 0.0238127`, 
              1649.53`, 2053.44`, 1570.06`, 1690.96`, 1554.88`, 1645.04`, 
              1887.78`, 2041.85`, 1642.62`, 2003.17`, 1680.7`, 1887.17`, 
              1624.91`, 1704.05`, 1862.01`, 2023.36`, 1561.28`, 1594.5`, 
              1664.42`, 1883.07`}, {1, 29, 15840.34872`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6871322254`, 2.980817472`, \(-0.0003360803953`\), 0.4`, 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196666`, 1627.55`, 
              1624.09`, \(-0.98543`\), 1649.89`, 1650.53`, 0.184407`, 
              1560.52`, 1564.65`, 1.13048`, 1568.12`, 1570.69`, 0.705906`, 
              1560.52`, 1563.62`, 0.847935`, 1594.36`, 1594.9`, 0.153565`, 
              1554.79`, 1558.29`, 0.954818`, 1643.99`, 
              1643.5`, \(-0.142219`\), 1649.89`, 2051.93`, 1568.12`, 
              1690.94`, 1554.79`, 1645.83`, 1888.76`, 2041.77`, 1643.99`, 
              2006.59`, 1680.72`, 1888.21`, 1627.55`, 1701.87`, 1866.69`, 
              2023.75`, 1560.52`, 1594.36`, 1663.62`, 1884}, {2, 22, 
              94.58203292`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.27853279`, 0.6478329186`, 0.7856359548`, 0.5`, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.974161`\), 1693.71`, 
              1698.28`, 1.35943`, 1725.31`, 1730.8`, 1.66115`, 487.307`, 
              512.133`, 2.17394`, 948.155`, 948.894`, 0.122775`, 487.307`, 
              503.987`, 1.44866`, 1790.98`, 1831.56`, 12.8797`, 0, 0, 
              1.99002`, 1725.31`, 1733.02`, 2.3378`, 1693.71`, 2540.31`, 
              948.155`, 2369.07`, 0, 2229.85`, 1790.98`, 2749.09`, 2097.45`, 
              2552.22`, 2122.2`, 2536.62`, 2092.19`, 2124.95`, 2531.73`, 
              2557.61`, 487.307`, 2119.33`, 1725.31`, 2328.6`}, {3, 58, 
              26.83925141`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2254872088`, 0.2612229013`, 0.6488786238`, 0.5`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.451064`\), 1334.54`, 
              1335.18`, 0.149602`, 1348.35`, 1348.57`, 0.0526715`, 431.042`, 
              446.888`, 1.21889`, 637.286`, 642.332`, 0.565782`, 431.042`, 
              443.266`, 0.936383`, 1345.4`, 1345.8`, 0.0928298`, 193.234`, 
              229.139`, 1.32872`, 1337.36`, 1338.14`, 0.182628`, 1418.79`, 
              1652.62`, 637.286`, 1841.12`, 193.234`, 1477.48`, 1601.91`, 
              1990.92`, 1337.36`, 1889.61`, 1346.38`, 1884.76`, 1334.54`, 
              1348.35`, 1883.1`, 1891.38`, 431.042`, 1345.4`, 1382.44`, 
              1622.51`}, {1, 34, 27.33077478`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2545446051`, 0.2714364743`, 0.5660493042`, 0.5`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401137`\), 1326.52`, 
              1327.33`, 0.189549`, 1345.7`, 1345.99`, 0.0682032`, 427.619`, 
              443.715`, 1.22884`, 632.763`, 637.844`, 0.565734`, 427.619`, 
              439.737`, 0.920937`, 1341.51`, 1341.91`, 0.0941822`, 193.924`, 
              229.62`, 1.32465`, 1330.46`, 1331.43`, 0.226548`, 1419.4`, 
              1653.48`, 632.763`, 1855.25`, 193.924`, 1487.89`, 1597.17`, 
              1999.2`, 1330.46`, 1888.98`, 1342.92`, 1882.19`, 1326.52`, 
              1345.7`, 1879.91`, 1891.39`, 427.619`, 1341.51`, 1380.09`, 
              1620.82`}, {1, 21, 28.42959538`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3255246668`, 0.2828152758`, 0.4739315306`, 0.5`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323503`\), 1329.42`, 
              1330.6`, 0.274639`, 1360.53`, 1360.97`, 0.103797`, 424.554`, 
              440.935`, 1.24223`, 626.187`, 631.188`, 0.550893`, 424.554`, 
              436.23`, 0.880616`, 1353.51`, 1353.88`, 0.0871693`, 195.058`, 
              230.488`, 1.32102`, 1335.78`, 1337.08`, 0.305304`, 1431.02`, 
              1663.94`, 626.187`, 1903.44`, 195.058`, 1521.53`, 1596.85`, 
              2029.14`, 1335.78`, 1910.04`, 1355.82`, 1898.3`, 1329.42`, 
              1360.53`, 1894.71`, 1913.82`, 424.554`, 1353.51`, 1389.55`, 
              1627.36`}, {1, 36, 32.72387227`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6668599155`, 0.2781494628`, 0.6287020135`, 0.5`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.313375`\), 1582.24`, 
              1578.04`, \(-1.16121`\), 1609.92`, 1613.96`, 1.14094`, 
              419.679`, 436.436`, 1.25692`, 588.84`, 594.121`, 0.547314`, 
              419.679`, 430.696`, 0.820829`, 1620.46`, 1621.21`, 0.213772`, 
              200.92`, 236.75`, 1.37399`, 1589.89`, 1582.49`, \(-2.05779`\), 
              1626.66`, 1784.64`, 588.84`, 2186.23`, 200.92`, 1772.97`, 
              1672.14`, 2282.79`, 1589.89`, 2205.18`, 1624.03`, 2177.09`, 
              1582.24`, 1630.56`, 2172.45`, 2209.87`, 419.679`, 1620.46`, 
              1609.92`, 1727.37`}, {3, 52, 52.369068`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8320648882`, 0.3365857905`, 0.7245803718`, 0.5`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.329998`\), 1685.78`, 
              1662.92`, \(-6.70849`\), 1688.41`, 1698.85`, 3.09776`, 
              484.294`, 498.061`, 1.18493`, 691.151`, 691.947`, 0.0964855`, 
              484.294`, 490.866`, 0.561513`, 1695.81`, 1730.39`, 10.3797`, 
              218.728`, 251.972`, 1.37104`, 1688.41`, 
              1666.02`, \(-6.58145`\), 1697.5`, 1898.38`, 691.151`, 2248.62`, 
              218.728`, 1931.8`, 1695.81`, 2411.96`, 1694.54`, 2316.73`, 
              1733, 2284.15`, 1685.78`, 1740.33`, 2278.72`, 2322.17`, 
              484.294`, 1729.34`, 1688.41`, 1805.45`}, {2, 38, 
              64.95563385`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9047284713`, 0.3249455065`, 0.8952771947`, 0.5`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.379019`\), 1782.69`, 
              1786.36`, 1.14748`, 1791.93`, 1793.69`, 0.553546`, 386.024`, 
              408.228`, 1.5452`, 571.46`, 578.85`, 0.744794`, 386.024`, 
              400.768`, 1.01639`, 1713.91`, 1802.03`, 27.1442`, 0, 131.945`, 
              1.69386`, 1791.93`, 1792.75`, 0.258598`, 1782.69`, 2067.33`, 
              571.46`, 2330.23`, 0, 2142.84`, 1713.91`, 2604.08`, 1968.67`, 
              2545.05`, 2001.26`, 2514.29`, 1962.53`, 2006.49`, 2510.46`, 
              2548.88`, 386.024`, 1998.3`, 1791.93`, 1980.26`}, {5, 101, 
              15879.30411`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6166726823`, 1.983792441`, 0.0008473514213`, 0.5`, 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196767`, 1617.12`, 
              1614.71`, \(-0.680512`\), 1648.22`, 1648.68`, 0.13462`, 
              1563.47`, 1567.16`, 1.00996`, 1576.09`, 1577.49`, 0.389172`, 
              1563.47`, 1565.84`, 0.64847`, 1595.02`, 1595.52`, 0.140969`, 
              1555.27`, 1558.13`, 0.779034`, 1637.83`, 1638.31`, 0.136312`, 
              1648.22`, 2058.19`, 1576.09`, 1691.09`, 1555.27`, 1642.63`, 
              1884.26`, 2042.23`, 1637.83`, 1990.42`, 1680.56`, 1883.39`, 
              1617.12`, 1710.44`, 1845.66`, 2021.52`, 1563.47`, 1595.02`, 
              1666.63`, 1879.68`}, {5, 103, 15869.84264`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6404545348`, 2.171337687`, 0.0005938863592`, 0.5`, 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196737`, 1619.17`, 
              1616.57`, \(-0.735939`\), 1648.7`, 1649.2`, 0.14417`, 1562.95`, 
              1566.75`, 1.04134`, 1574.57`, 1576.21`, 0.452614`, 1562.95`, 
              1565.46`, 0.687538`, 1594.88`, 1595.4`, 0.144872`, 1555.15`, 
              1558.14`, 0.816604`, 1639.29`, 1639.83`, 0.156065`, 1648.7`, 
              2056.98`, 1574.57`, 1691.13`, 1555.15`, 1643.34`, 1885.57`, 
              2042.11`, 1639.29`, 1994.27`, 1680.66`, 1884.74`, 1619.17`, 
              1708.87`, 1850.57`, 2022.13`, 1562.95`, 1594.88`, 1666.29`, 
              1880.94`}, {5, 102, 15836.85416`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6704495757`, 3.060616125`, 0.0008835116564`, 0.5`, 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196689`, 1629.01`, 
              1625.39`, \(-1.0338`\), 1649.97`, 1650.63`, 0.191468`, 
              1560.07`, 1564.23`, 1.13881`, 1566.98`, 1569.7`, 0.747948`, 
              1560.07`, 1563.26`, 0.874715`, 1594.27`, 1594.82`, 0.153874`, 
              1554.75`, 1558.33`, 0.975282`, 1644.61`, 
              1643.86`, \(-0.215082`\), 1649.97`, 2051.04`, 1566.98`, 
              1691.01`, 1554.75`, 1646.19`, 1889.02`, 2041.73`, 1644.61`, 
              2008.33`, 1680.67`, 1888.47`, 1629.01`, 1700.52`, 1868.89`, 
              2023.93`, 1560.07`, 1594.27`, 1662.99`, 1884.19`}, {1, 29, 
              15834.76926`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.7065472553`, 3.316999765`, \(-0.0002931072334`\), 0.5`, 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196677`, 1629.9`, 
              1626.18`, \(-1.0619`\), 1650.16`, 1650.84`, 0.196004`, 
              1559.84`, 1564.03`, 1.14872`, 1566.43`, 1569.24`, 0.771484`, 
              1559.84`, 1563.1`, 0.892245`, 1594.23`, 1594.79`, 0.155013`, 
              1554.73`, 1558.36`, 0.990678`, 1645.09`, 
              1644.19`, \(-0.260423`\), 1650.16`, 2050.62`, 1566.43`, 
              1691.03`, 1554.73`, 1646.5`, 1889.54`, 2041.71`, 1645.09`, 
              2009.34`, 1680.71`, 1889, 1629.9`, 1699.88`, 1870.49`, 
              2024.02`, 1559.84`, 1594.23`, 1662.89`, 
              1884.7`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data three/ytraj_m0_m1_2.out", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for transverse lattice mesons \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\n\
sqrt(string tension) is 440 MeV.\nUse chi^2 fit with 11 criteria and \
tolerance 0.001.\nOverall scale from glueball data.\n0 nonstandard fit \
criteria (for log divergence).\nFit 3 masses to experimental values\n  \
(charge,multi,state,mass (MeV),error (MeV)) =\n   (1,4,0,139.557,69.3623) \
(-1,3,0,785.988,73.8942)\n   (-1,7,0,785.988,73.8942).\nNo degenerate pairs.\n\
Spectrum for P_perp a = (0,0), ( 0.25 0.25)\n  for 2 states (multiplet, \
charge, c^2 error for each) =\n  (17,  1, 5 5)\n  (17, -1, 0.2 0.2)\n  (18, \
-1, 0.2 5)\n  (18,  1, 0.2 5).\nOrdinary spectra multiplets, 2 states, \
(charge,multiplet) =\n  ( 1,  3)  (-1,  3)  ( 1,  4)  (-1,  4)  ( 1,  5)  \
(-1,  5) \n  ( 1,  6)  (-1,  6)  (-1,  7)  ( 1,  7) .\nAll spectra \
extrapolated using (K,p) = (12/2,4)  (14/2,4) \n  (18/2,4) \nLog divergence \
determined using pion (K,p) = (20/2,4) \n  (24/2,4)  (28/2,4)  (36/2,4) .\n\
Result format:\n Fit info, # steps, chi^2, p damping, scale G^2 N/sigma.\n \
The 17 couplings  (G^2 N units);\n   which couplings -- if any -- were fit.\n \
Winding potential and heavy souce potential fits (G^2 N units).\n The log(K) \
coefficient of the pion mass^2 (G^2 N units).\n The rescaled masses (MeV) for \
each P_perp*a and c^2 values.\n Finally come the masses (MeV) for the \
ordinary spectra.\>", {{3, 13, 26.26858264`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1846847583`, 0.3756367876`, 
              0.8745349493`, \(-0.5`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.462799`\), 1585.59`, 
              1588.65`, 0.851019`, 1610.72`, 1613.37`, 0.74951`, 452.183`, 
              466.603`, 1.16076`, 644.891`, 649.651`, 0.539911`, 452.183`, 
              464.14`, 0.959915`, 1642.8`, 1644.79`, 0.573461`, 191.186`, 
              228.968`, 1.39087`, 1610.72`, 1613.64`, 0.824042`, 1585.59`, 
              2447.19`, 644.891`, 2215.97`, 191.186`, 2163.44`, 1642.8`, 
              2679.62`, 2296.19`, 2752.33`, 2305.31`, 2746.09`, 2293.91`, 
              2306.71`, 2744.07`, 2754.5`, 452.183`, 2184.79`, 1610.72`, 
              2374.82`}, {3, 13, 26.18288012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.196048301`, 0.4102974277`, 
              0.8421873012`, \(-0.5`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.427103`\), 1569.35`, 
              1572.29`, 0.809302`, 1595.65`, 1598.18`, 0.705766`, 457.985`, 
              472.129`, 1.1527`, 647.381`, 652.102`, 0.537476`, 457.985`, 
              469.688`, 0.951264`, 1631.12`, 1632.79`, 0.477929`, 190.561`, 
              228.865`, 1.40763`, 1595.65`, 1598.4`, 0.768906`, 1569.35`, 
              2602.66`, 647.381`, 2199.89`, 190.561`, 2154.99`, 1631.12`, 
              2661.39`, 2385, 2839.57`, 2396.78`, 2831.44`, 2382.11`, 
              2398.51`, 2828.8`, 2842.41`, 457.985`, 2172.04`, 1595.65`, 
              2475.14`}, {2, 9, 25.92729451`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2073197805`, 0.4435655935`, 
              0.8081357079`, \(-0.5`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.383626`\), 1551.64`, 
              1554.48`, 0.772335`, 1579.7`, 1582.07`, 0.657747`, 466.834`, 
              480.531`, 1.13692`, 650.686`, 655.39`, 0.538242`, 466.834`, 
              478.152`, 0.937073`, 1618.32`, 1619.68`, 0.38521`, 189.592`, 
              228.629`, 1.43041`, 1579.7`, 1582.27`, 0.71333`, 1551.64`, 
              2643.2`, 650.686`, 2183.14`, 189.592`, 2144.01`, 1618.32`, 
              2651.81`, 2459.56`, 2913.86`, 2474.74`, 2903.28`, 2455.9`, 
              2476.87`, 2899.83`, 2917.62`, 466.834`, 2157.56`, 1579.7`, 
              2642.41`}, {3, 13, 25.4607596`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2183915838`, 0.4733467663`, 
              0.772956802`, \(-0.5`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.326448`\), 1526.13`, 
              1528.96`, 0.757556`, 1558.44`, 1560.65`, 0.603444`, 479.424`, 
              492.469`, 1.11077`, 654.732`, 659.474`, 0.546082`, 479.424`, 
              490.211`, 0.916381`, 1601.74`, 1602.77`, 0.289781`, 188.2`, 
              228.242`, 1.46101`, 1558.44`, 1560.87`, 0.66385`, 1526.13`, 
              2601.03`, 654.732`, 2156.33`, 188.2`, 2113.47`, 1601.74`, 
              2624.82`, 2520.98`, 2976.03`, 2540.36`, 2962.38`, 2516.36`, 
              2543.02`, 2957.85`, 2981.03`, 479.424`, 2128.05`, 1558.44`, 
              2607.89`}, {3, 21, 24.75822128`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2297919988`, 0.4988969312`, 
              0.7399186213`, \(-0.5`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.248693`\), 1472.81`, 
              1475.78`, 0.765343`, 1513.49`, 1515.51`, 0.535194`, 495.988`, 
              508.188`, 1.07344`, 660.017`, 664.893`, 0.565978`, 495.988`, 
              506.129`, 0.890464`, 1564.64`, 1565.31`, 0.184739`, 186.278`, 
              227.639`, 1.5`, 1513.49`, 1515.82`, 0.618614`, 1472.81`, 
              2472.24`, 660.017`, 2074.44`, 186.278`, 2024.59`, 1564.64`, 
              2497.68`, 2575.05`, 3031.15`, 2599.43`, 3013.7`, 2569.23`, 
              2602.79`, 3007.77`, 3037.85`, 495.988`, 2041.07`, 1513.49`, 
              2478.46`}, {3, 13, 23.94141815`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2433533918`, 0.5223987797`, 0.7134661265`, \(-0.5`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.151501`\), 1380.49`, 
              1383.59`, 0.751914`, 1429.83`, 1431.64`, 0.453909`, 513.286`, 
              524.592`, 1.02813`, 663.897`, 669.081`, 0.605443`, 513.286`, 
              522.81`, 0.864624`, 1488.37`, 1488.72`, 0.0918868`, 184.437`, 
              227.176`, 1.54134`, 1429.83`, 1432.07`, 0.561992`, 1380.49`, 
              2341.5`, 663.897`, 1944, 184.437`, 1895.53`, 1488.37`, 
              2357.02`, 2626.46`, 3082.57`, 2656.51`, 3060.64`, 2619.21`, 
              2660.88`, 3052.9`, 3091.61`, 513.286`, 1909.33`, 1429.83`, 
              2341.04`}, {3, 17, 23.27030547`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2603186135`, 0.5467907374`, 0.6941343245`, \(-0.5`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0489565`\), 1272.1`, 
              1275.3`, 0.713218`, 1327.29`, 1328.94`, 0.382898`, 527.286`, 
              537.737`, 0.975187`, 662.292`, 667.999`, 0.665208`, 527.286`, 
              536.308`, 0.840716`, 1390.17`, 1390.33`, 0.0373743`, 183.467`, 
              227.252`, 1.57561`, 1327.29`, 1329.47`, 0.506323`, 1272.1`, 
              2264.82`, 662.292`, 1825.15`, 183.467`, 1781.47`, 1390.17`, 
              2272.04`, 2671.47`, 3125.63`, 2707.49`, 3098.98`, 2662.65`, 
              2713.18`, 3088.99`, 3137.78`, 527.286`, 1790.71`, 1327.29`, 
              2258.05`}, {2, 17, 22.98813868`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2805797359`, 0.5724733713`, 0.6798080265`, \(-0.5`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0413998`, 1168.86`, 
              1172.13`, 0.671463`, 1227.82`, 1229.36`, 0.330821`, 536.409`, 
              546.014`, 0.910849`, 653.473`, 659.919`, 0.74176`, 536.409`, 
              545.034`, 0.817221`, 1293.18`, 1293.24`, 0.0137656`, 183.659`, 
              227.936`, 1.59675`, 1227.82`, 1229.98`, 0.464358`, 1168.86`, 
              2233.64`, 653.473`, 1741.67`, 183.659`, 1701.06`, 1293.18`, 
              2235.24`, 2705.04`, 3155.58`, 2746.66`, 3124.79`, 2694.66`, 
              2753.9`, 3112.1`, 3171.36`, 536.409`, 1706.71`, 1227.82`, 
              2221.57`}, {2, 39, 23.35663452`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3034943467`, 0.5976044673`, 0.6687674752`, \(-0.5`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.10448`, 1077.86`, 
              1081.22`, 0.634944`, 1139.29`, 1140.75`, 0.292845`, 539.585`, 
              548.288`, 0.829605`, 636.498`, 643.914`, 0.831919`, 539.585`, 
              547.907`, 0.792962`, 1206.05`, 1206.08`, 0.00815293`, 185.32`, 
              229.298`, 1.59759`, 1139.29`, 1141.46`, 0.434794`, 1077.86`, 
              2230.82`, 636.498`, 1691.39`, 185.32`, 1651.23`, 1206.05`, 
              2227.84`, 2726.53`, 3172.99`, 2772.88`, 3139.05`, 2714.8`, 
              2781.66`, 3123.35`, 3192.3`, 539.585`, 1655.15`, 1139.29`, 
              2213.86`}, {2, 46, 24.79296645`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3270666232`, 0.6190731346`, 0.660077957`, \(-0.5`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.131885`, 999.166`, 
              1002.6`, 0.601687`, 1061.81`, 1063.21`, 0.26124`, 535.57`, 
              543.208`, 0.721951`, 610.911`, 619.503`, 0.926337`, 535.57`, 
              543.645`, 0.763535`, 1128.91`, 1128.97`, 0.0107281`, 188.682`, 
              231.239`, 1.56576`, 1061.81`, 1064.01`, 0.41007`, 999.166`, 
              2244.29`, 610.911`, 1666.27`, 188.682`, 1625.2`, 1128.91`, 
              2236.98`, 2739.93`, 3182.37`, 2789.97`, 3146.21`, 2727.13`, 
              2800.11`, 3127.37`, 3204.77`, 535.57`, 1629.06`, 1061.81`, 
              2222.78`}, {5, 100, 48.04299227`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 1.0965093`, 
              0.3931507154`, 0.8552985539`, \(-0.5`\), 0.5`, 0, 0, 10, 11, 
              12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.112454`, 1433.74`, 
              1441.83`, 2.03706`, 1438.74`, 1444.46`, 1.4444`, 218.86`, 
              245.572`, 1.08697`, 267.124`, 284.559`, 0.842785`, 218.86`, 
              241.347`, 0.906698`, 1453.7`, 1458.25`, 1.16222`, 128.574`, 
              172.957`, 1.17256`, 1438.74`, 1443.63`, 1.23278`, 1433.74`, 
              2290.53`, 267.124`, 1993.79`, 128.574`, 1929.44`, 1453.7`, 
              2560.56`, 2099.43`, 2656.3`, 2139.72`, 2618.85`, 2090.23`, 
              2147.17`, 2612.7`, 2662.43`, 218.86`, 1952.86`, 1438.74`, 
              2203.12`}, {3, 13, 26.2876898`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1822822432`, 0.3481938492`, 
              0.8375962778`, \(-0.4`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.464865`\), 1593.52`, 
              1596.68`, 0.88254`, 1616.61`, 1619.44`, 0.801264`, 447.416`, 
              462.095`, 1.16977`, 642.722`, 647.54`, 0.544687`, 447.416`, 
              459.508`, 0.960848`, 1642.83`, 1645.3`, 0.71147`, 191.699`, 
              229.001`, 1.37497`, 1616.61`, 1619.75`, 0.889555`, 1593.52`, 
              2284.04`, 642.722`, 2205.51`, 191.699`, 2188.36`, 1642.83`, 
              2658.55`, 2143.5`, 2607.53`, 2152.12`, 2601.7`, 2141.27`, 
              2153.54`, 2599.79`, 2609.57`, 447.416`, 2151.14`, 1616.61`, 
              2215.28`}, {3, 13, 26.32333993`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1951084523`, 0.3855830586`, 
              0.8006101398`, \(-0.4`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.428555`\), 1574.76`, 
              1577.76`, 0.828773`, 1598.53`, 1601.23`, 0.757521`, 451.666`, 
              466.176`, 1.16684`, 644.882`, 649.645`, 0.540198`, 451.666`, 
              463.572`, 0.954723`, 1629.97`, 1632, 0.57947`, 191.236`, 
              228.972`, 1.38932`, 1598.53`, 1601.45`, 0.820137`, 1574.76`, 
              2400.04`, 644.882`, 2194.69`, 191.236`, 2141.85`, 1629.97`, 
              2644.7`, 2236.95`, 2698.43`, 2248.46`, 2690.62`, 2234.05`, 
              2250.25`, 2688.06`, 2701.18`, 451.666`, 2184.18`, 1598.53`, 
              2322.41`}, {2, 9, 26.1739472`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2082939496`, 0.4229996884`, 
              0.7610254731`, \(-0.4`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384711`\), 1555.48`, 
              1558.33`, 0.777232`, 1580.42`, 1582.97`, 0.707978`, 459.103`, 
              473.26`, 1.15643`, 648.028`, 652.747`, 0.537724`, 459.103`, 
              470.677`, 0.942811`, 1615.78`, 1617.44`, 0.469062`, 190.43`, 
              228.799`, 1.40937`, 1580.42`, 1583.14`, 0.755853`, 1555.48`, 
              2521.62`, 648.028`, 2177.32`, 190.43`, 2131, 1615.78`, 
              2629.12`, 2313.56`, 2775.35`, 2328.9`, 2764.9`, 2309.79`, 
              2331.16`, 2761.45`, 2779.09`, 459.103`, 2148.08`, 1580.42`, 
              2412.78`}, {3, 9, 25.76842062`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2215883239`, 0.4587567992`, 
              0.7180872739`, \(-0.4`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32741`\), 1531.7`, 
              1534.45`, 0.740943`, 1559.67`, 1562.04`, 0.649234`, 471.211`, 
              484.754`, 1.1343`, 652.373`, 657.084`, 0.540462`, 471.211`, 
              482.255`, 0.922584`, 1599.7`, 1600.97`, 0.355947`, 189.08`, 
              228.409`, 1.43862`, 1559.67`, 1562.2`, 0.693749`, 1531.7`, 
              2609.64`, 652.373`, 2156.81`, 189.08`, 2111.45`, 1599.7`, 
              2628.11`, 2372.17`, 2836.74`, 2392.55`, 2822.78`, 2367.26`, 
              2395.42`, 2818.1`, 2841.89`, 471.211`, 2127.21`, 1559.67`, 
              2606.89`}, {3, 13, 24.98469643`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2350283549`, 0.4914587888`, 
              0.6741914647`, \(-0.4`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.244935`\), 1481.36`, 
              1484.22`, 0.742143`, 1517.9`, 1520.04`, 0.570825`, 490.358`, 
              502.937`, 1.09475`, 659.329`, 664.117`, 0.555178`, 490.358`, 
              500.632`, 0.892077`, 1566.93`, 1567.75`, 0.224468`, 186.768`, 
              227.596`, 1.48227`, 1517.9`, 1520.3`, 0.639401`, 1481.36`, 
              2486.53`, 659.329`, 2084.05`, 186.768`, 2030.3`, 1566.93`, 
              2512.02`, 2417.78`, 2887, 2444.6`, 2868.5`, 2411.45`, 2448.29`, 
              2862.13`, 2894.21`, 490.358`, 2048.27`, 1517.9`, 2492.27`}, {3, 
              13, 23.88400007`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2511109211`, 0.5226381234`, 0.6373505709`, \(-0.4`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130039`\), 1372.51`, 
              1375.54`, 0.728323`, 1420.06`, 1421.92`, 0.46397`, 513.962`, 
              525.352`, 1.0372`, 665.688`, 670.823`, 0.601244`, 513.962`, 
              523.396`, 0.857499`, 1479.1`, 1479.46`, 0.0948829`, 184.105`, 
              226.817`, 1.53775`, 1420.06`, 1422.33`, 0.567417`, 1372.51`, 
              2324, 665.688`, 1928.24`, 184.105`, 1875.27`, 1479.1`, 
              2336.95`, 2460.9`, 2934.78`, 2495.28`, 2910.8`, 2452.82`, 
              2500.17`, 2902.05`, 2945.15`, 513.962`, 1889.73`, 1420.06`, 
              2321.09`}, {3, 21, 23.01799539`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2729177078`, 0.5552530441`, 0.6114576798`, \(-0.4`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0046601`\), 
              1236.71`, 1239.84`, 0.677439`, 1291.11`, 1292.75`, 0.371025`, 
              532.789`, 542.98`, 0.960529`, 661.547`, 667.469`, 0.689586`, 
              532.789`, 541.553`, 0.824999`, 1355.37`, 1355.48`, 0.0246`, 
              183.033`, 227.126`, 1.58458`, 1291.11`, 1293.3`, 0.494113`, 
              1236.71`, 2238.66`, 661.547`, 1786.34`, 183.033`, 1738.07`, 
              1355.37`, 2241.05`, 2498.88`, 2974.72`, 2541.01`, 2945.23`, 
              2488.87`, 2547.56`, 2933.25`, 2989.64`, 532.789`, 1746.4`, 
              1291.11`, 2227.01`}, {2, 17, 23.11893049`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3002643962`, 0.5876104286`, 0.59339821`, \(-0.4`\), 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0945329`, 1113.21`, 
              1116.41`, 0.626049`, 1170.98`, 1172.48`, 0.30765`, 541.233`, 
              550.169`, 0.854557`, 641.764`, 648.947`, 0.812375`, 541.233`, 
              549.533`, 0.793239`, 1237.33`, 1237.34`, 0.00264452`, 184.671`, 
              228.858`, 1.60101`, 1170.98`, 1173.14`, 0.442648`, 1113.21`, 
              2216.84`, 641.764`, 1700.64`, 184.671`, 1652.84`, 1237.33`, 
              2212.12`, 2523.36`, 2997.6`, 2572.17`, 2963.94`, 2511.55`, 
              2580.63`, 2948.02`, 3017.81`, 541.233`, 1657.52`, 1170.98`, 
              2197.79`}, {3, 42, 25.27187954`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.328917537`, 0.6124152075`, 0.5805173385`, \(-0.4`\), 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.142543`, 1010.29`, 
              1013.56`, 0.580765`, 1068.66`, 1070.03`, 0.257267`, 535.266`, 
              542.644`, 0.696827`, 604.412`, 613.29`, 0.947191`, 535.266`, 
              543.238`, 0.75327`, 1134.64`, 1134.65`, 0.00220426`, 190.067`, 
              232.109`, 1.55508`, 1068.66`, 1070.8`, 0.401463`, 1010.29`, 
              2227.41`, 604.412`, 1660.66`, 190.067`, 1610.61`, 1134.64`, 
              2216.64`, 2537.53`, 3007.55`, 2591.28`, 2971.44`, 2524.32`, 
              2601.45`, 2951.33`, 3032.57`, 535.266`, 1614.89`, 1068.66`, 
              2202.07`}, {2, 36, 29.58706619`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3210851293`, 0.6821396077`, 0.5631609837`, \(-0.4`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0669424`, 848.469`, 
              852.212`, 0.557698`, 921.713`, 922.539`, 0.133454`, 542.259`, 
              545.969`, 0.353769`, 596.866`, 605.909`, 0.953024`, 542.259`, 
              547.781`, 0.527351`, 1000.48`, 999.857`, \(-0.108735`\), 0, 
              92.8686`, 1.5635`, 921.713`, 923.632`, 0.310322`, 848.469`, 
              2289.96`, 596.866`, 1664.93`, 0, 1610.89`, 1000.48`, 2275.97`, 
              2581.09`, 3029.18`, 2640.35`, 2993.46`, 2566.91`, 2652.21`, 
              2965.17`, 3064.51`, 542.259`, 1612.01`, 921.713`, 
              2251.74`}, {3, 25, 28.15937643`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4347452461`, 0.717048001`, 0.5548177239`, \(-0.4`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.00096243`, 844.769`, 
              850.231`, 0.811139`, 910.382`, 910.153`, \(-0.0364281`\), 
              530.285`, 539.79`, 0.891219`, 543.56`, 552.988`, 0.905773`, 
              543.56`, 551.153`, 0.728288`, 984.965`, 985.052`, 0.0150872`, 
              193.736`, 235.993`, 1.59105`, 910.382`, 912.909`, 0.403754`, 
              844.769`, 2305.81`, 530.285`, 1661.04`, 193.736`, 1579.65`, 
              984.965`, 2270.27`, 2488.28`, 2969.11`, 2551.29`, 2925.39`, 
              2472.41`, 2563.65`, 2902.2`, 2996.65`, 543.56`, 1591.77`, 
              910.382`, 2253.34`}, {3, 13, 26.21687069`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1796567617`, 0.3191907904`, 
              0.8041163245`, \(-0.3`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.468471`\), 1608.54`, 
              1612.01`, 0.978155`, 1632.96`, 1635.9`, 0.839919`, 444.122`, 
              458.996`, 1.17698`, 641.824`, 646.696`, 0.550093`, 444.122`, 
              456.292`, 0.960101`, 1645.16`, 1650.21`, 1.45764`, 191.984`, 
              228.91`, 1.36173`, 1632.96`, 1636.77`, 1.08915`, 1608.54`, 
              2111.29`, 641.824`, 2216.61`, 191.984`, 2103.34`, 1645.16`, 
              2560.52`, 1985.11`, 2460.09`, 1993.07`, 2454.8`, 1982.97`, 
              1994.46`, 2453.04`, 2461.97`, 444.122`, 1992.19`, 1632.96`, 
              2043.35`}, {3, 13, 26.39078729`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1936566587`, 0.3564089841`, 
              0.7611370639`, \(-0.3`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.429803`\), 1583.85`, 
              1586.99`, 0.871795`, 1605.67`, 1608.57`, 0.816003`, 445.9`, 
              460.726`, 1.17772`, 642.166`, 646.995`, 0.545431`, 445.9`, 
              457.965`, 0.955502`, 1629.85`, 1632.52`, 0.763528`, 191.866`, 
              229.047`, 1.37119`, 1605.67`, 1608.89`, 0.905818`, 1583.85`, 
              2232.07`, 642.166`, 2180.08`, 191.866`, 2166.73`, 1629.85`, 
              2650.49`, 2080.45`, 2550.61`, 2091.37`, 2543.28`, 2077.59`, 
              2093.19`, 2540.84`, 2553.22`, 445.9`, 2090.07`, 1605.67`, 
              2157.91`}, {3, 9, 26.40393314`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2091619827`, 0.3967544827`, 
              0.715286503`, \(-0.3`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384326`\), 1560.97`, 
              1563.89`, 0.797862`, 1582.82`, 1585.6`, 0.770503`, 451.094`, 
              465.714`, 1.17445`, 644.667`, 649.436`, 0.540753`, 451.094`, 
              462.916`, 0.946751`, 1613.14`, 1615.23`, 0.592848`, 191.315`, 
              228.995`, 1.38758`, 1582.82`, 1585.76`, 0.815547`, 1560.97`, 
              2337.9`, 644.667`, 2165.82`, 191.315`, 2112.83`, 1613.14`, 
              2600.25`, 2159.67`, 2628.72`, 2174.81`, 2618.55`, 2155.82`, 
              2177.2`, 2615.15`, 2632.4`, 451.094`, 2172.77`, 1582.82`, 
              2253.83`}, {2, 9, 26.15639567`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2258218239`, 0.4379503759`, 
              0.6635103626`, \(-0.3`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.325525`\), 1535.91`, 
              1538.63`, 0.732065`, 1559.15`, 1561.75`, 0.712445`, 461.077`, 
              475.22`, 1.16027`, 648.593`, 653.314`, 0.538586`, 461.077`, 
              472.436`, 0.929044`, 1594.55`, 1596.18`, 0.456384`, 190.247`, 
              228.724`, 1.41245`, 1559.15`, 1561.84`, 0.736949`, 1535.91`, 
              2442.29`, 648.593`, 2146.8`, 190.247`, 2094.78`, 1594.55`, 
              2600.92`, 2215.39`, 2687.6`, 2236.51`, 2673.43`, 2210.19`, 
              2239.66`, 2668.63`, 2692.86`, 461.077`, 2113.86`, 1559.15`, 
              2327.05`}, {3, 13, 25.40495879`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.243328855`, 0.4797589597`, 
              0.6043928271`, \(-0.3`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.237713`\), 1489.92`, 
              1492.63`, 0.708148`, 1520.43`, 1522.79`, 0.627468`, 481.052`, 
              494.206`, 1.12399`, 656.831`, 661.533`, 0.543145`, 481.052`, 
              491.556`, 0.895178`, 1565.57`, 1566.64`, 0.293918`, 187.753`, 
              227.743`, 1.4558`, 1520.43`, 1522.94`, 0.667694`, 1489.92`, 
              2498.24`, 656.831`, 2090.91`, 187.753`, 2030.2`, 1565.57`, 
              2527.47`, 2247.37`, 2726.74`, 2277.01`, 2706.92`, 2240.33`, 
              2281.21`, 2700, 2734.6`, 481.052`, 2051.48`, 1520.43`, 
              2491.11`}, {3, 17, 23.84402726`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2644071325`, 0.5255457409`, 0.5481400853`, \(-0.3`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0930198`\), 
              1354.71`, 1357.62`, 0.690952`, 1399.73`, 1401.68`, 0.477968`, 
              515.224`, 526.724`, 1.04978`, 668.563`, 673.61`, 0.593505`, 
              515.224`, 524.506`, 0.845549`, 1459.73`, 1460.11`, 0.0975833`, 
              183.522`, 226.222`, 1.53293`, 1399.73`, 1402.06`, 0.572475`, 
              1354.71`, 2291.67`, 668.563`, 1898.14`, 183.522`, 1836.97`, 
              1459.73`, 2300.77`, 2272.45`, 2760.77`, 2313.42`, 2733.57`, 
              2263.07`, 2319.18`, 2723.15`, 2773.41`, 515.224`, 1853.18`, 
              1399.73`, 2284.7`}, {2, 29, 22.75007648`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2977829561`, 0.5750790498`, 0.5101419814`, \(-0.3`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0716781`, 1169.7`, 
              1172.72`, 0.620086`, 1222.42`, 1224.03`, 0.343617`, 541.557`, 
              551.152`, 0.918652`, 655.142`, 661.655`, 0.751404`, 541.557`, 
              549.894`, 0.797279`, 1288.91`, 1288.92`, 0.00298445`, 183.057`, 
              227.511`, 1.59913`, 1222.42`, 1224.61`, 0.469089`, 1169.7`, 
              2202.34`, 655.142`, 1725.3`, 183.057`, 1665.86`, 1288.91`, 
              2195.9`, 2293.95`, 2786.69`, 2346.13`, 2752.91`, 2281.97`, 
              2354.25`, 2736.97`, 2807.23`, 541.557`, 1673.37`, 1222.42`, 
              2181.48`}, {3, 33, 25.92878847`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3371052854`, 0.6078275869`, 0.4872634792`, \(-0.3`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.160288`, 1018.09`, 
              1021.25`, 0.565222`, 1070.36`, 1071.63`, 0.237398`, 533.989`, 
              541.04`, 0.664143`, 595.646`, 604.861`, 0.96931`, 533.989`, 
              541.808`, 0.736976`, 1135.43`, 1135.38`, \(-0.00905988`\), 
              192.896`, 233.895`, 1.53309`, 1070.36`, 1072.45`, 0.391764`, 
              1018.09`, 2204.03`, 595.646`, 1653.4`, 192.896`, 1588.09`, 
              1135.43`, 2187.6`, 2302.95`, 2792.61`, 2362.7`, 2755.72`, 
              2288.95`, 2373.16`, 2733.33`, 2820.99`, 533.989`, 1594.05`, 
              1070.36`, 2172.88`}, {3, 50, 28.14383943`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4276951708`, 0.6948766091`, 0.4624917266`, \(-0.3`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0947346`, 892.676`, 
              900.68`, 1.25758`, 947.485`, 944.463`, \(-0.50086`\), 530.533`, 
              540.075`, 0.895057`, 543.412`, 552.92`, 0.913313`, 543.412`, 
              551.131`, 0.740248`, 1019.66`, 1019.63`, \(-0.00583987`\), 
              193.414`, 235.864`, 1.5966`, 947.485`, 949.849`, 0.393123`, 
              892.676`, 2245.82`, 530.533`, 1643.18`, 193.414`, 1539.06`, 
              1019.66`, 2206.96`, 2249.34`, 2749.65`, 2317.96`, 2705.71`, 
              2232.86`, 2330.36`, 2680.63`, 2780.33`, 543.412`, 1552.39`, 
              947.485`, 2191.67`}, {2, 55, 35.1210135`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.590967449`, 0.8537295972`, 0.4300765888`, \(-0.3`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.294754`\), 803.296`, 
              814.934`, 1.65001`, 853.943`, 847.518`, \(-0.957894`\), 
              419.125`, 434.411`, 1.14309`, 567.035`, 575.375`, 0.834794`, 
              567.035`, 573.799`, 0.676098`, 939.688`, 
              939.259`, \(-0.0705752`\), 194.801`, 237.222`, 1.60571`, 
              853.943`, 856.521`, 0.386403`, 803.296`, 2324.93`, 419.125`, 
              1726.8`, 194.801`, 1517.19`, 939.688`, 2247.23`, 2134.39`, 
              2658.89`, 2215.61`, 2605.29`, 2115.06`, 2228.4`, 2580.62`, 
              2688.67`, 567.035`, 1551.62`, 853.943`, 2240.49`}, {2, 26, 
              47.81114303`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2814717673`, 0.5606607074`, 0.4702023019`, \(-0.3`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0790444`, 751.779`, 
              754.157`, 0.313675`, 789.679`, 789.947`, 0.0370386`, 378.105`, 
              376.236`, \(-0.123522`\), 379.852`, 392.37`, 0.847021`, 
              378.105`, 383.02`, 0.327782`, 834.478`, 
              833.979`, \(-0.0729993`\), 0, 82.5228`, 0.839228`, 789.679`, 
              790.64`, 0.133053`, 751.779`, 2278.8`, 379.852`, 1657.54`, 0, 
              1629.42`, 834.478`, 2266.35`, 2480.96`, 2818.59`, 2547.66`, 
              2829.5`, 2467.29`, 2564.98`, 2762.33`, 2895.02`, 378.105`, 
              1631.28`, 789.679`, 2253.14`}, {2, 26, 
              28.33016243`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1551279279`, 0.2154371855`, 
              0.7487065457`, \(-0.2`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.410097`\), 1603.65`, 
              1608.87`, 1.46895`, 1611.49`, 1609.94`, \(-0.436787`\), 
              409.639`, 423.611`, 1.02007`, 575.968`, 581.557`, 0.566827`, 
              409.639`, 421.512`, 0.864569`, 1610.19`, 1617.13`, 1.96377`, 
              199.759`, 230.91`, 1.17544`, 1611.49`, 
              1609.73`, \(-0.498225`\), 1603.65`, 1677.53`, 575.968`, 
              1940.01`, 199.759`, 1692.42`, 1610.19`, 2180.25`, 1613.02`, 
              2120.13`, 1617.57`, 2117.44`, 1611.65`, 1618.51`, 2116.51`, 
              2121.12`, 409.639`, 1617.08`, 1611.49`, 1640.68`}, {3, 17, 
              26.51499893`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1949038817`, 0.3386522358`, 
              0.7301063951`, \(-0.2`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.443935`\), 1604.23`, 
              1607.9`, 1.03337`, 1629.52`, 1632.49`, 0.84796`, 447.333`, 
              462.526`, 1.21117`, 651.941`, 656.705`, 0.546252`, 447.333`, 
              459.519`, 0.968199`, 1638.73`, 1647.67`, 2.57393`, 190.973`, 
              228.602`, 1.38331`, 1629.52`, 1633.76`, 1.21168`, 1604.23`, 
              2087.66`, 651.941`, 2191.36`, 190.973`, 2078.02`, 1638.73`, 
              2537.06`, 1942.95`, 2422.91`, 1953.59`, 2415.93`, 1940.09`, 
              1955.44`, 2413.57`, 2425.43`, 447.333`, 1952.35`, 1629.52`, 
              2009.16`}, {3, 13, 26.54960679`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096207049`, 0.3653694643`, 
              0.6723708579`, \(-0.2`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384043`\), 1571.94`, 
              1575.09`, 0.86814`, 1592.24`, 1595.24`, 0.837159`, 443.913`, 
              458.933`, 1.18814`, 641.232`, 646.077`, 0.546376`, 443.913`, 
              455.923`, 0.946891`, 1612.54`, 1615.74`, 0.905295`, 192.107`, 
              229.129`, 1.36637`, 1592.24`, 1595.64`, 0.949652`, 1571.94`, 
              2162.69`, 641.232`, 2152.1`, 192.107`, 2119.63`, 1612.54`, 
              2601.85`, 1997.38`, 2476.61`, 2011.84`, 2467.02`, 1993.54`, 
              2014.31`, 2463.76`, 2480.11`, 443.913`, 2010, 1592.24`, 
              2081.45`}, {3, 13, 26.56192969`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2308919087`, 0.4099634451`, 
              0.6108666888`, \(-0.2`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321384`\), 1541.61`, 
              1544.38`, 0.750832`, 1560.54`, 1563.45`, 0.796301`, 450.354`, 
              465.127`, 1.18497`, 644.02`, 648.794`, 0.540831`, 450.354`, 
              462.022`, 0.932714`, 1588.66`, 1590.92`, 0.629506`, 191.466`, 
              229.059`, 1.38512`, 1560.54`, 1563.51`, 0.812735`, 1541.61`, 
              2249.17`, 644.02`, 2120.54`, 191.466`, 2065.62`, 1588.66`, 
              2543.67`, 2050.55`, 2531.83`, 2071.85`, 2517.7`, 2045.08`, 
              2075.29`, 2512.85`, 2537.11`, 450.354`, 2068.76`, 1560.54`, 
              2156.71`}, {3, 13, 26.1213643`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2572044477`, 0.4602103432`, 
              0.5320217844`, \(-0.2`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.224495`\), 1496.97`, 
              1499.49`, 0.661753`, 1518.97`, 1521.69`, 0.723195`, 466.573`, 
              480.578`, 1.16219`, 650.839`, 655.512`, 0.534837`, 466.573`, 
              477.444`, 0.899137`, 1556.66`, 1558.21`, 0.424528`, 189.564`, 
              228.327`, 1.4193`, 1518.97`, 1521.63`, 0.70797`, 1496.97`, 
              2331.74`, 650.839`, 2081.28`, 189.564`, 2008.96`, 1556.66`, 
              2526.99`, 2062.17`, 2552.36`, 2094.74`, 2530.97`, 2054.18`, 
              2099.7`, 2523.43`, 2560.91`, 466.573`, 2036.14`, 1518.97`, 
              2195.61`}, {1, 13, 23.72891814`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.295045754`, 0.5444454486`, 0.4283884321`, \(-0.2`\), 0, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.00243562`\), 
              1294.38`, 1297.11`, 0.619015`, 1336.09`, 1338.14`, 0.481046`, 
              523.266`, 534.678`, 1.05786`, 675.928`, 680.843`, 0.584367`, 
              523.266`, 532.054`, 0.812617`, 1399.65`, 1399.97`, 0.0790358`, 
              181.433`, 224.645`, 1.53745`, 1336.09`, 1338.49`, 0.563042`, 
              1294.38`, 2210.89`, 675.928`, 1818.22`, 181.433`, 1738.95`, 
              1399.65`, 2216.41`, 2041.36`, 2544.49`, 2095.49`, 2511.08`, 
              2029.37`, 2103.1`, 2496.88`, 2562.6`, 523.266`, 1758.74`, 
              1336.09`, 2185.28`}, {1, 35, 29.39443454`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3527883889`, 0.5981272746`, 0.3672617011`, \(-0.2`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.200619`, 996.507`, 
              1000.66`, 0.727404`, 1036.38`, 1036.19`, \(-0.03475`\), 
              518.604`, 524.467`, 0.535899`, 564.849`, 574.304`, 0.94361`, 
              518.604`, 525.809`, 0.659339`, 1097.8`, 
              1097.63`, \(-0.032306`\), 206.662`, 242.08`, 1.39252`, 
              1036.38`, 1038.28`, 0.344266`, 996.507`, 2167.88`, 564.849`, 
              1644.27`, 206.662`, 1554.36`, 1097.8`, 2148.39`, 2027.79`, 
              2523.32`, 2098.27`, 2487.97`, 2012.43`, 2109.82`, 2458.61`, 
              2559.19`, 518.604`, 1564.83`, 1036.38`, 2129.66`}, {3, 66, 
              50.37998225`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6713639483`, 0.00009034343372`, 0.368591689`, \(-0.2`\), 
              0.2`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.214857`, 876.395`, 
              877.415`, 0.156701`, 876.395`, 877.415`, 0.156701`, 225.822`, 
              249.807`, 0.999506`, 225.822`, 249.807`, 0.999506`, 225.822`, 
              249.807`, 0.999506`, 876.395`, 877.415`, 0.156701`, 225.822`, 
              249.807`, 0.999506`, 876.395`, 877.415`, 0.156701`, 876.395`, 
              1917.63`, 225.822`, 1669.43`, 225.822`, 1669.43`, 876.395`, 
              2238.12`, 1917.63`, 2405.28`, 1950.95`, 2419.29`, 1917.63`, 
              1950.95`, 2419.29`, 2405.28`, 225.822`, 1669.43`, 876.395`, 
              1917.63`}, {2, 60, 50.40233634`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6652239162`, 0.0006126644402`, 0.3556966877`, \(-0.2`\), 
              0.3`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.139648`, 807.149`, 
              807.722`, 0.0811128`, 807.149`, 807.722`, 0.0811126`, 225.869`, 
              249.788`, 0.996849`, 225.87`, 249.789`, 0.996848`, 225.87`, 
              249.789`, 0.996849`, 807.149`, 807.722`, 0.0811122`, 225.87`, 
              249.789`, 0.996848`, 807.149`, 807.722`, 0.0811124`, 807.149`, 
              2025.33`, 225.869`, 1719.56`, 225.87`, 1719.56`, 807.149`, 
              2285.16`, 2025.33`, 2372.14`, 2059.75`, 2473.32`, 2025.33`, 
              2059.75`, 2372.14`, 2500.71`, 225.87`, 1719.56`, 807.149`, 
              2025.33`}, {1, 44, 48.22170205`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3490456863`, 0.8500702797`, 0.2970036582`, \(-0.2`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.243264`\), 730.862`, 
              733.672`, 0.360491`, 748.271`, 746.773`, \(-0.19614`\), 
              466.034`, 474.55`, 0.701837`, 591.509`, 596.41`, 0.510143`, 
              591.509`, 594.09`, 0.268101`, 820.066`, 
              817.683`, \(-0.341929`\), 200.164`, 240.768`, 1.56867`, 
              748.271`, 748.727`, 0.0598725`, 730.862`, 2277.15`, 466.034`, 
              1907.33`, 200.164`, 1775.28`, 820.066`, 2437.61`, 2112.63`, 
              2497.28`, 2182.2`, 2445.42`, 2093.77`, 2205.96`, 2392.4`, 
              2565.89`, 591.509`, 1748.94`, 748.271`, 2251.24`}, {1, 60, 
              51.88199134`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3105511136`, 0.8482787206`, 0.2964742957`, \(-0.2`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.269397`\), 712.113`, 
              714.167`, 0.256689`, 726.338`, 725.143`, \(-0.151944`\), 
              456.275`, 465.532`, 0.747666`, 585.836`, 589.851`, 0.413645`, 
              585.836`, 588.069`, 0.229652`, 793.977`, 
              791.446`, \(-0.351562`\), 200.892`, 240.869`, 1.54732`, 
              726.338`, 726.578`, 0.0306007`, 712.113`, 2276.23`, 456.275`, 
              1930.99`, 200.892`, 1817.68`, 793.977`, 2448.78`, 2150.1`, 
              2509.47`, 2213.47`, 2451.47`, 2131.73`, 2239.06`, 2401.26`, 
              2576.98`, 585.836`, 1777.17`, 726.338`, 2261.16`}, {2, 34, 
              26.08454411`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1719761912`, 0.2526820847`, 
              0.7500233823`, \(-0.1`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.464253`\), 1626.13`, 
              1630.19`, 1.16003`, 1632.34`, 1633.87`, 0.43828`, 433.649`, 
              448.476`, 1.14589`, 627.294`, 632.409`, 0.564609`, 433.649`, 
              445.764`, 0.933436`, 1636.7`, 1639.51`, 0.806398`, 193.924`, 
              229.034`, 1.30111`, 1634.07`, 1631.27`, \(-0.803078`\), 
              1626.13`, 1727.45`, 627.294`, 1982.93`, 193.924`, 1736.62`, 
              1636.7`, 2217.37`, 1634.07`, 2138.1`, 1640.02`, 2134.6`, 
              1632.34`, 1641.19`, 2133.39`, 2139.39`, 433.649`, 1639.38`, 
              1634.96`, 1676.96`}, {3, 34, 27.58883592`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1734611306`, 0.2412676182`, 
              0.6773872263`, \(-0.1`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.398737`\), 1604.12`, 
              1605.65`, 0.430977`, 1607.68`, 1610.3`, 0.740856`, 416.325`, 
              430.638`, 1.06213`, 592.822`, 598.163`, 0.557273`, 416.325`, 
              428.172`, 0.876583`, 1608.84`, 1615.62`, 1.91476`, 198.098`, 
              230.321`, 1.20954`, 1609.65`, 1606.92`, \(-0.768764`\), 
              1604.12`, 1691.65`, 592.822`, 1946.87`, 198.098`, 1708.25`, 
              1608.84`, 2192.09`, 1609.65`, 2117.68`, 1616.29`, 2113.8`, 
              1607.68`, 1617.62`, 2112.45`, 2119.12`, 416.325`, 1615.54`, 
              1611.38`, 1645.91`}, {1, 13, 28.12048177`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2232059941`, 0.3863091699`, 
              0.6520758487`, \(-0.1`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.424683`\), 1602.48`, 
              1606.46`, 1.11756`, 1627.2`, 1630.35`, 0.897757`, 460.585`, 
              476.603`, 1.31533`, 683.998`, 688.519`, 0.543745`, 460.585`, 
              472.942`, 1.0107`, 1639.48`, 1653.84`, 4.14392`, 187.489`, 
              227.867`, 1.46946`, 1627.2`, 1632, 1.37108`, 1602.48`, 2113.9`, 
              683.998`, 2185.92`, 187.489`, 2087.44`, 1639.48`, 2553.38`, 
              1924.67`, 2409.16`, 1941.04`, 2398.56`, 1920.36`, 1943.77`, 
              2394.94`, 2413.05`, 460.585`, 1938.98`, 1627.2`, 2015.39`}, {3, 
              13, 26.90030147`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2369845324`, 0.3769254387`, 
              0.5633644741`, \(-0.1`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.31819`\), 1556.8`, 
              1560.22`, 0.934082`, 1575.72`, 1578.82`, 0.857135`, 441.439`, 
              456.789`, 1.20808`, 640.478`, 645.344`, 0.548259`, 441.439`, 
              453.345`, 0.933376`, 1587.25`, 1593.74`, 1.80737`, 192.462`, 
              229.359`, 1.36366`, 1575.72`, 1579.78`, 1.12277`, 1556.8`, 
              2064.41`, 640.478`, 2088.6`, 192.462`, 2037.97`, 1587.25`, 
              2516.2`, 1880.33`, 2374.28`, 1901.01`, 2360.9`, 1874.73`, 
              1904.64`, 2356.25`, 2379.31`, 441.439`, 1898.12`, 1575.72`, 
              1972.09`}, {3, 17, 27.12894621`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2809249479`, 0.4283042823`, 
              0.4628040945`, \(-0.1`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.203502`\), 1504.43`, 
              1506.92`, 0.655964`, 1517.46`, 1520.78`, 0.882981`, 447.852`, 
              462.949`, 1.20477`, 640.332`, 645.11`, 0.53815`, 447.852`, 
              459.159`, 0.898525`, 1540.17`, 1543.17`, 0.810383`, 192.232`, 
              229.519`, 1.37785`, 1517.46`, 1520.6`, 0.834841`, 1504.43`, 
              2095.16`, 640.332`, 2013.6`, 192.232`, 1948.97`, 1540.17`, 
              2458.14`, 1863.72`, 2370.99`, 1898.26`, 2348.84`, 1854.68`, 
              1904.13`, 2341, 2379.8`, 447.852`, 1892.38`, 1517.46`, 
              1990.06`}, {3, 48, 32.9391364`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.5921448625`, 0.4493266263`, 0.4347804085`, \(-0.1`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0244069`, 1444.85`, 
              1448.33`, 0.882521`, 1452.36`, 1455.89`, 0.899038`, 407.27`, 
              424.899`, 1.28535`, 557.603`, 564.166`, 0.645038`, 407.27`, 
              419.1`, 0.856578`, 1464.77`, 1467.8`, 0.780558`, 203.21`, 
              239.135`, 1.39236`, 1444.85`, 1447.55`, 0.683429`, 1452.36`, 
              1977.34`, 557.603`, 1916.28`, 203.21`, 1794.07`, 1464.77`, 
              2313.45`, 1701.49`, 2275.47`, 1756.61`, 2234.82`, 1685.49`, 
              1768.83`, 2223.15`, 2287.99`, 407.27`, 1741.46`, 1444.85`, 
              1861.35`}, {1, 48, 64.53491534`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.00001473049813`, 0.6781057225`, 0.099128158`, \(-0.1`\), 
              0.1`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0728192`, 728.849`, 
              728.346`, \(-0.0642356`\), 754.453`, 752.106`, \(-0.309842`\), 
              357.345`, 372.376`, 0.961055`, 503.206`, 505.246`, 0.180313`, 
              503.206`, 505.246`, 0.180313`, 780.311`, 
              777.079`, \(-0.441042`\), 221.461`, 249.674`, 1.16464`, 
              728.849`, 728.346`, \(-0.0642356`\), 754.453`, 1732.25`, 
              357.345`, 1665.37`, 221.461`, 1665.37`, 780.311`, 1934.21`, 
              1732.25`, 1915.39`, 1772.91`, 1930.42`, 1730.38`, 1771.9`, 
              1934.21`, 1937.23`, 503.206`, 1665.37`, 728.849`, 
              1734.73`}, {1, 55, 50.56383347`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6277767151`, 0.000148565998`, 0.1300729928`, \(-0.1`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.103225`, 765.372`, 
              762.66`, \(-0.363103`\), 765.372`, 762.66`, \(-0.363103`\), 
              225.824`, 249.776`, 0.998078`, 225.824`, 249.776`, 0.998078`, 
              225.824`, 249.776`, 0.998078`, 765.372`, 
              762.66`, \(-0.363103`\), 225.824`, 249.776`, 0.998078`, 
              765.372`, 762.66`, \(-0.363103`\), 765.372`, 1783.41`, 
              225.824`, 1780.43`, 225.824`, 1780.43`, 765.372`, 1992.5`, 
              1736.84`, 1783.41`, 1830.44`, 1992.5`, 1783.41`, 1830.44`, 
              1736.84`, 1992.5`, 225.824`, 1736.84`, 765.372`, 1783.41`}, {5, 
              102, 57.5319485`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6922212096`, 0.1689555235`, \(-0.08123924483`\), \(-0.1`\), 
              0.3`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.035576`, 753.588`, 
              749.809`, \(-0.497866`\), 759.365`, 755.319`, \(-0.537046`\), 
              146.373`, 185.909`, 1.15105`, 206.54`, 234.026`, 1.06101`, 
              206.54`, 236.138`, 1.14799`, 748.065`, 744.23`, \(-0.501424`\), 
              250.316`, 273.405`, 1.0595`, 753.588`, 
              749.485`, \(-0.540447`\), 759.365`, 1698.1`, 146.373`, 
              1683.22`, 250.316`, 1672.82`, 748.065`, 1907.54`, 1625.68`, 
              1713.44`, 1738.79`, 1897.33`, 1674.95`, 1759.86`, 1639.54`, 
              1912.36`, 206.54`, 1631.14`, 753.588`, 1701.47`}, {5, 103, 
              85.64697278`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.02898999148`, 0.5505212913`, 0.1530940173`, \(-0.1`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.158662`, 757.143`, 
              756.918`, \(-0.0298376`\), 765.461`, 764.539`, \(-0.123695`\), 
              346.478`, 355.589`, 0.560477`, 442.327`, 443.545`, 0.0945517`, 
              442.327`, 443.531`, 0.0934738`, 782.182`, 
              780.66`, \(-0.208529`\), 257.759`, 272.983`, 0.707936`, 
              757.143`, 756.923`, \(-0.0292218`\), 765.461`, 1851.39`, 
              346.478`, 1797.08`, 257.759`, 1796.44`, 782.182`, 2038.81`, 
              1850.33`, 1956.29`, 1893.86`, 2033.34`, 1846.29`, 1893.48`, 
              1977.64`, 2039.79`, 442.327`, 1757.31`, 757.143`, 
              1852.98`}, {3, 51, 69.05473641`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.0002018459789`, 
              0.7118816687`, \(-0.06980074199`\), \(-0.1`\), 0.5`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.00744013`, 692.92`, 
              692.306`, \(-0.0745015`\), 738.043`, 734.736`, \(-0.426763`\), 
              257.219`, 281.297`, 1.13608`, 484.515`, 486.886`, 0.201805`, 
              484.515`, 486.886`, 0.201805`, 743.446`, 
              739.923`, \(-0.457815`\), 219.316`, 248.273`, 1.18632`, 
              692.92`, 692.306`, \(-0.0745015`\), 738.043`, 1681.96`, 
              257.219`, 1611.36`, 219.316`, 1611.36`, 743.446`, 1889.74`, 
              1681.96`, 1907.95`, 1722.52`, 1889, 1681.57`, 1722.32`, 
              1889.74`, 1913.05`, 484.515`, 1611.36`, 692.92`, 1684.69`}, {3, 
              37, 25.95334978`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1780157057`, 0.2545579208`, 0.7416157605`, 0, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.481203`\), 1561.22`, 
              1560.86`, \(-0.0982758`\), 1570.39`, 1570.23`, \(-0.0420493`\), 
              438.676`, 453.762`, 1.17963`, 640.641`, 645.624`, 0.561537`, 
              438.676`, 450.83`, 0.947304`, 1568.51`, 1568.77`, 0.0738347`, 
              192.452`, 228.438`, 1.32704`, 1563.03`, 
              1562.45`, \(-0.158996`\), 1590.55`, 1701.38`, 640.641`, 
              1943.22`, 192.452`, 1671.32`, 1636.18`, 2157.51`, 1563.03`, 
              2074.13`, 1569.18`, 2070.63`, 1561.22`, 1570.39`, 2069.41`, 
              2075.43`, 438.676`, 1568.51`, 1582.27`, 1662.63`}, {3, 37, 
              26.24850993`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.192760654`, 0.2821307923`, 0.6786643596`, 0, \(-0.4`\), 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.447882`\), 1626.18`, 
              1627.56`, 0.391648`, 1629.39`, 1634.75`, 1.53193`, 440.148`, 
              455.464`, 1.20186`, 645.942`, 650.82`, 0.554161`, 440.148`, 
              452.291`, 0.949486`, 1634.76`, 1639.68`, 1.4121`, 191.919`, 
              228.374`, 1.34243`, 1631.88`, 1629.28`, \(-0.74497`\), 
              1626.18`, 1748.25`, 645.942`, 1994.52`, 191.919`, 1758.16`, 
              1634.76`, 2234.12`, 1631.88`, 2136.76`, 1640.52`, 2131.74`, 
              1629.39`, 1642.16`, 2129.98`, 2138.64`, 440.148`, 1639.52`, 
              1635.53`, 1685.02`}, {3, 21, 27.04715719`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2017406863`, 0.2747645514`, 0.5963250204`, 0, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.375129`\), 1596.71`, 
              1595.27`, \(-0.403253`\), 1601.42`, 1606.61`, 1.46098`, 
              423.631`, 438.59`, 1.13003`, 612.825`, 617.986`, 0.556504`, 
              423.631`, 435.559`, 0.897915`, 1604.61`, 1609.08`, 1.25781`, 
              195.944`, 229.766`, 1.26155`, 1599.79`, 
              1597.57`, \(-0.620971`\), 1601.42`, 1709.83`, 612.825`, 
              1957.47`, 195.944`, 1727.37`, 1604.61`, 2205.91`, 1599.79`, 
              2110.66`, 1610.21`, 2104.65`, 1596.71`, 1612.27`, 2102.53`, 
              2112.92`, 423.631`, 1608.94`, 1607.15`, 1651.32`}, {2, 27, 
              36.03345629`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2797192055`, 0.4564103502`, 0.5638331788`, 0, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.379559`\), 1605.42`, 
              1610.24`, 1.35916`, 1624, 1627.51`, 1.00033`, 479.131`, 
              497.097`, 1.53663`, 727.672`, 732.41`, 0.606179`, 479.131`, 
              492.189`, 1.1112`, 1639.48`, 1665.91`, 7.65541`, 182.736`, 
              228.546`, 1.65075`, 1624, 1630.03`, 1.71886`, 1605.42`, 
              2151.27`, 727.672`, 2164.81`, 182.736`, 2084.54`, 1639.48`, 
              2583.46`, 1893.68`, 2390.22`, 1921.54`, 2372.11`, 1886.27`, 
              1926.31`, 2365.87`, 2397.02`, 479.131`, 1917.68`, 1624, 
              2024.13`}, {2, 17, 33.69159428`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3613891773`, 0.3936286685`, 0.468640571`, 0, \(-0.1`\), 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.199787`\), 1557.05`, 
              1562.42`, 1.46768`, 1564.48`, 1568.34`, 1.06025`, 428.535`, 
              447.28`, 1.43843`, 614.582`, 621.834`, 0.785605`, 428.535`, 
              442.408`, 1.05864`, 1563.73`, 1588.4`, 6.81202`, 195.657`, 
              236.156`, 1.53225`, 1564.48`, 1571.05`, 1.80421`, 1557.05`, 
              1961.37`, 614.582`, 2012.54`, 195.657`, 1994.07`, 1563.73`, 
              2420.03`, 1754.73`, 2287.35`, 1789.49`, 2263.04`, 1744.25`, 
              1797.36`, 2255.15`, 2295.89`, 428.535`, 1784.03`, 1564.48`, 
              1859.96`}, {3, 30, 33.32775662`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.662317569`, 0.3753799595`, 0.5267271697`, 0, 0, 0, 0, 10, 11, 
              12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0735689`\), 
              1532.49`, 1536.39`, 1.04733`, 1537.64`, 1543.58`, 1.60297`, 
              405.123`, 422.754`, 1.27892`, 562.742`, 568.85`, 0.605637`, 
              405.123`, 416.671`, 0.831536`, 1547.49`, 1556.19`, 2.36718`, 
              201.743`, 237.482`, 1.37535`, 1532.49`, 1536.61`, 1.10772`, 
              1537.64`, 1907.4`, 562.742`, 2008.31`, 201.743`, 1869.27`, 
              1547.49`, 2337.53`, 1689.16`, 2280.17`, 1733.86`, 2244.71`, 
              1676.38`, 1744.22`, 2236.11`, 2289.02`, 405.123`, 1725.75`, 
              1532.49`, 1813.18`}, {1, 86, 150.2709504`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.07349253627`, 0.00133054853`, 0.01966916637`, 0, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0195229`, 1152.81`, 
              1152.8`, \(-0.000251129`\), 1152.81`, 
              1152.8`, \(-0.000251205`\), 228.79`, 228.811`, 0.000803791`, 
              228.791`, 228.811`, 0.000803552`, 228.791`, 228.811`, 
              0.000803804`, 1152.81`, 1152.8`, \(-0.000251141`\), 228.791`, 
              228.811`, 0.000803566`, 1152.81`, 1152.8`, \(-0.000251216`\), 
              1152.81`, 1561, 228.79`, 1562.78`, 228.791`, 1561, 1152.81`, 
              1759.95`, 1559.19`, 1561, 1621.45`, 1749.23`, 1561, 1621.45`, 
              1559.19`, 1759.95`, 228.791`, 1559.19`, 1152.81`, 1561}, {3, 
              73, 35.39778692`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9260006591`, 0.3494504171`, 0.7944149337`, 0, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.162398`\), 1623.46`, 
              1631.27`, 2.22654`, 1626.32`, 1635.43`, 2.60321`, 397.803`, 
              415.296`, 1.24624`, 539.722`, 545.963`, 0.593657`, 397.803`, 
              409.278`, 0.811427`, 1632, 1642.46`, 3.00068`, 203.398`, 
              239.064`, 1.38268`, 1623.46`, 1630.99`, 2.14597`, 1626.32`, 
              2104.27`, 539.722`, 2176.71`, 203.398`, 2064.27`, 1632, 
              2556.38`, 1931.45`, 2519.73`, 1969.07`, 2485.68`, 1923.17`, 
              1975.97`, 2480.36`, 2525.05`, 397.803`, 1963.24`, 1623.46`, 
              2025.62`}, {3, 47, 36.04480606`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.006951252`, 0.3461700033`, 0.8872229384`, 0, 0.3`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.178788`\), 1643.81`, 
              1654.28`, 3.02683`, 1647.22`, 1656.92`, 2.80653`, 395.821`, 
              413.298`, 1.23895`, 534.054`, 540.365`, 0.594047`, 395.821`, 
              407.292`, 0.807167`, 1648.79`, 1660.48`, 3.3904`, 203.945`, 
              239.544`, 1.38324`, 1643.81`, 1653.1`, 2.68437`, 1647.22`, 
              2196.16`, 534.054`, 2214.98`, 203.945`, 2146.98`, 1648.79`, 
              2634.94`, 2037.48`, 2601.72`, 2073.21`, 2567.8`, 2030.36`, 
              2079.16`, 2563.25`, 2606.26`, 395.821`, 2067.72`, 1643.81`, 
              2122.64`}, {3, 25, 36.70190339`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.076545803`, 0.3448022844`, 0.967703659`, 0, 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.186189`\), 1657.08`, 
              1670.11`, 3.79995`, 1661.1`, 1671.7`, 3.09451`, 394.234`, 
              411.711`, 1.23411`, 529.758`, 536.129`, 0.595005`, 394.234`, 
              405.702`, 0.803739`, 1659.79`, 1672.89`, 3.82636`, 204.397`, 
              239.923`, 1.38305`, 1657.08`, 1668.15`, 3.22389`, 1661.1`, 
              2282.29`, 529.758`, 2234.08`, 204.397`, 2212.58`, 1659.79`, 
              2702.73`, 2133.36`, 2663.82`, 2167.57`, 2629.79`, 2127.08`, 
              2172.83`, 2625.79`, 2667.81`, 394.234`, 2161.99`, 1657.08`, 
              2212.54`}, {5, 103, 1059.04132`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.03761307032`, 0.2586261235`, 0.06705035555`, 0, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.210149`, 1381.24`, 
              1381.18`, \(-0.0136189`\), 1382.81`, 1382.66`, \(-0.0356727`\), 
              788.576`, 789.03`, 0.0628413`, 794.951`, 795.037`, 0.0120551`, 
              794.951`, 795.037`, 0.0119983`, 1382.99`, 
              1382.83`, \(-0.0388757`\), 788.228`, 788.715`, 0.0672747`, 
              1381.24`, 1381.18`, \(-0.0135629`\), 1382.81`, 1628.18`, 
              788.576`, 1607.85`, 788.228`, 1606.88`, 1382.99`, 1821.65`, 
              1627.1`, 1653.39`, 1685.34`, 1815.74`, 1627.04`, 1685.94`, 
              1653.45`, 1821.69`, 794.951`, 1607.34`, 1381.24`, 
              1628.39`}, {2, 41, 25.96580932`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1831086754`, 0.251205778`, 0.7343728558`, 0.1`, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.486362`\), 1489.52`, 
              1489.61`, 0.0244255`, 1498.79`, 1498.8`, 0.00422369`, 438.438`, 
              453.66`, 1.18984`, 642.688`, 647.665`, 0.562586`, 438.438`, 
              450.617`, 0.948716`, 1496.87`, 1497.18`, 0.0817805`, 192.281`, 
              228.365`, 1.32993`, 1491.36`, 1491.41`, 0.0132971`, 1539.4`, 
              1679.59`, 642.688`, 1901.51`, 192.281`, 1602.51`, 1627.99`, 
              2096.46`, 1491.36`, 2011.71`, 1497.55`, 2008.29`, 1489.52`, 
              1498.79`, 2007.09`, 2012.98`, 438.438`, 1496.87`, 1520.03`, 
              1648.32`}, {2, 37, 26.23440803`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1955463588`, 0.2652870125`, 0.6650310168`, 0.1`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.447801`\), 1513.57`, 
              1513.58`, 0.00328498`, 1525.69`, 1525.67`, \(-0.00621782`\), 
              435.914`, 451.279`, 1.19436`, 639.792`, 644.787`, 0.562266`, 
              435.914`, 448.07`, 0.941509`, 1523.15`, 1523.46`, 0.0827407`, 
              192.725`, 228.623`, 1.32525`, 1515.98`, 
              1515.88`, \(-0.0254956`\), 1561.77`, 1686.39`, 639.792`, 
              1921.37`, 192.725`, 1639.01`, 1623.4`, 2127.16`, 1515.98`, 
              2034.17`, 1524.08`, 2029.66`, 1513.57`, 1525.69`, 2028.08`, 
              2035.86`, 435.914`, 1523.15`, 1544.82`, 1648.53`}, {2, 37, 
              26.64150465`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2150272632`, 0.2849855398`, 0.5875103847`, 0.1`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.396101`\), 1537.68`, 
              1537.53`, \(-0.0421644`\), 1554.68`, 1554.56`, \(-0.0304873`\), 
              432.993`, 448.551`, 1.20167`, 636.51`, 641.513`, 0.560221`, 
              432.993`, 445.085`, 0.930253`, 1551.04`, 1551.34`, 0.079975`, 
              193.242`, 228.951`, 1.3209`, 1541.06`, 1540.65`, \(-0.11025`\), 
              1580.6`, 1697.23`, 636.51`, 1942.55`, 193.242`, 1682.21`, 
              1612.74`, 2163.48`, 1541.06`, 2058.38`, 1552.43`, 2052.01`, 
              1537.68`, 1554.68`, 2049.76`, 2060.78`, 432.993`, 1551.04`, 
              1568.71`, 1647.38`}, {1, 13, 27.34014751`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2552943043`, 0.3244826659`, 0.5000463918`, 0.1`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323266`\), 1574.8`, 
              1573.81`, \(-0.27239`\), 1593.85`, 1596.78`, 0.818793`, 
              434.353`, 450.442`, 1.24729`, 642.696`, 647.631`, 0.557936`, 
              434.353`, 446.361`, 0.926618`, 1594.24`, 1596.72`, 0.693741`, 
              192.539`, 229.065`, 1.34927`, 1580.33`, 
              1578.44`, \(-0.522714`\), 1593.85`, 1739.79`, 642.696`, 
              1979.2`, 192.539`, 1759.07`, 1594.24`, 2228.87`, 1580.33`, 
              2099.33`, 1599.01`, 2088.64`, 1574.8`, 1602.7`, 2084.86`, 
              2103.38`, 434.353`, 1596.51`, 1596.61`, 1659.59`}, {1, 21, 
              30.19332668`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3773458671`, 0.3347855239`, 0.3797689265`, 0.1`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153305`\), 1498.46`, 
              1497.97`, \(-0.129593`\), 1533.14`, 1535.34`, 0.590003`, 
              419.55`, 435.274`, 1.17762`, 608.725`, 613.364`, 0.49668`, 
              419.55`, 430.195`, 0.79248`, 1529.7`, 1537.37`, 2.06198`, 
              201.412`, 234.277`, 1.25456`, 1508.72`, 
              1506.75`, \(-0.519712`\), 1540.28`, 1688.58`, 608.725`, 
              1932.8`, 201.412`, 1734.02`, 1529.7`, 2192.75`, 1508.72`, 
              2063.82`, 1542.25`, 2043.23`, 1498.46`, 1549.76`, 2036.7`, 
              2070.88`, 419.55`, 1537.1`, 1533.14`, 1605.79`}, {3, 47, 
              34.35651896`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7092899813`, 0.3401057759`, 0.5943691092`, 0.1`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.149103`\), 1592.17`, 
              1592.24`, 0.0190755`, 1593.66`, 1602.99`, 2.61367`, 408.682`, 
              426.168`, 1.27908`, 568.811`, 574.688`, 0.588831`, 408.682`, 
              420.076`, 0.827374`, 1602.27`, 1623.44`, 5.9852`, 199.518`, 
              235.849`, 1.38585`, 1592.17`, 1594.92`, 0.768069`, 1593.66`, 
              1888.19`, 568.811`, 2093.79`, 199.518`, 1889.45`, 1602.27`, 
              2362.3`, 1698.26`, 2299.44`, 1738.46`, 2266.3`, 1687.7`, 
              1747.21`, 2259.48`, 2306.38`, 408.682`, 1732.44`, 1592.17`, 
              1802.51`}, {2, 44, 35.0652417`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8458571861`, 0.3316695178`, 0.7327738165`, 0.1`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.190101`\), 1634.83`, 
              1640.61`, 1.65962`, 1636.65`, 1646.49`, 2.83095`, 403.759`, 
              421.13`, 1.25548`, 553.135`, 559.159`, 0.587112`, 403.759`, 
              415.131`, 0.815908`, 1638.36`, 1656.17`, 5.141`, 201.947`, 
              237.879`, 1.38466`, 1634.83`, 1641.97`, 2.04981`, 1636.65`, 
              2002.5`, 553.135`, 2174.28`, 201.947`, 1998.27`, 1638.36`, 
              2484.28`, 1837.41`, 2437.4`, 1874.87`, 2404.38`, 1828.99`, 
              1881.93`, 2399.02`, 2442.77`, 403.759`, 1869.69`, 1634.83`, 
              1925.15`}, {3, 34, 35.90865251`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9413385404`, 0.330530446`, 0.8373498804`, 0.1`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.210727`\), 1657.17`, 
              1668.13`, 3.1928`, 1660.81`, 1670.84`, 2.92704`, 401.557`, 
              418.886`, 1.24567`, 545.862`, 551.948`, 0.585377`, 401.557`, 
              412.887`, 0.808514`, 1657.21`, 1674.62`, 5.08191`, 202.829`, 
              238.673`, 1.38654`, 1657.17`, 1667.55`, 3.02539`, 1660.81`, 
              2107.69`, 545.862`, 2218.97`, 202.829`, 2098.25`, 1657.21`, 
              2578.25`, 1957.51`, 2540.06`, 1993.17`, 2507.01`, 1950.36`, 
              1999.19`, 2502.49`, 2544.58`, 401.557`, 1988.46`, 1657.17`, 
              2036.42`}, {2, 30, 36.78498095`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.019356548`, 0.3308975161`, 0.9249485459`, 0.1`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220089`\), 1670.16`, 
              1685.04`, 4.37511`, 1675.15`, 1685.89`, 3.16099`, 399.716`, 
              417.044`, 1.24002`, 540.509`, 546.662`, 0.586102`, 399.716`, 
              411.031`, 0.803781`, 1668.69`, 1686.48`, 5.22985`, 203.39`, 
              239.171`, 1.38743`, 1670.16`, 1682.97`, 3.76345`, 1675.15`, 
              2203.49`, 540.509`, 2241.14`, 203.39`, 2185.1`, 1668.69`, 
              2648.66`, 2062.87`, 2616.99`, 2097.1`, 2583.74`, 2056.59`, 
              2102.37`, 2579.78`, 2620.94`, 399.716`, 2092.63`, 1670.16`, 
              2136.53`}, {2, 51, 37.69559744`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.087343769`, 0.3313929401`, 1.001922107`, 0.1`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.222543`\), 1679.4`, 
              1697.18`, 5.25987`, 1685.16`, 1697.38`, 3.62067`, 397.796`, 
              415.15`, 1.23604`, 535.71`, 541.948`, 0.58899`, 397.796`, 
              409.127`, 0.801084`, 1677.2`, 1695.9`, 5.52817`, 203.812`, 
              239.521`, 1.38704`, 1679.4`, 1694.33`, 4.41271`, 1685.16`, 
              2291.87`, 535.71`, 2247.29`, 203.812`, 2258.42`, 1677.2`, 
              2701.42`, 2157.97`, 2674.55`, 2190.92`, 2640.99`, 2152.36`, 
              2195.64`, 2637.46`, 2678.07`, 397.796`, 2186.51`, 1679.4`, 
              2228.01`}, {3, 47, 38.67098982`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 1.1487595`, 
              0.332654225`, \(-1.071042531`\), 0.1`, 0.5`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220908`\), 1687.47`, 
              1707.36`, 5.91485`, 1693.69`, 1708.13`, 4.30413`, 396.636`, 
              414, 1.23329`, 532.576`, 538.857`, 0.589621`, 396.636`, 
              407.95`, 0.797619`, 1684.86`, 1704.86`, 5.94049`, 204.152`, 
              239.805`, 1.38683`, 1687.47`, 1704.44`, 5.04065`, 1693.69`, 
              2374.7`, 532.576`, 2241.54`, 204.152`, 2313.66`, 1684.86`, 
              2745.17`, 2245.16`, 2717.17`, 2276.95`, 2683.24`, 2240.05`, 
              2281.24`, 2680.03`, 2720.37`, 396.636`, 2272.27`, 1687.47`, 
              2312.84`}, {3, 41, 26.07195283`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1890446567`, 0.2500488188`, 0.7295721504`, 0.2`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.488515`\), 1436.09`, 
              1436.37`, 0.0694738`, 1445.61`, 1445.69`, 0.0219029`, 437.417`, 
              452.765`, 1.19705`, 642.785`, 647.771`, 0.563801`, 437.417`, 
              449.623`, 0.948624`, 1443.63`, 1443.96`, 0.0844759`, 192.332`, 
              228.437`, 1.33103`, 1438, 1438.31`, 0.0775408`, 1496.84`, 
              1667.53`, 642.785`, 1873.46`, 192.332`, 1552.89`, 1620.17`, 
              2053.61`, 1438, 1966.83`, 1444.32`, 1963.4`, 1436.09`, 
              1445.61`, 1962.2`, 1968.11`, 437.417`, 1443.63`, 1471.58`, 
              1638.78`}, {3, 32, 26.32440319`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2021560688`, 0.2621409886`, 0.6581824283`, 0.2`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.451379`\), 1447.29`, 
              1447.59`, 0.0764347`, 1459.63`, 1459.72`, 0.0232991`, 435.157`, 
              450.658`, 1.20312`, 640.746`, 645.739`, 0.562811`, 435.157`, 
              447.329`, 0.941167`, 1457.04`, 1457.38`, 0.0889932`, 192.693`, 
              228.639`, 1.32694`, 1449.77`, 1450.09`, 0.0808667`, 1511.83`, 
              1669.46`, 640.746`, 1886.38`, 192.693`, 1576.34`, 1615.89`, 
              2072.45`, 1449.77`, 1977.86`, 1457.97`, 1973.4`, 1447.29`, 
              1459.63`, 1971.83`, 1979.53`, 435.157`, 1457.04`, 1485.9`, 
              1637.07`}, {3, 26, 26.71906996`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2232652621`, 0.2778638143`, 0.5775425924`, 0.2`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.400335`\), 1454.98`, 
              1455.34`, 0.0907427`, 1472.09`, 1472.2`, 0.0271145`, 431.932`, 
              447.659`, 1.21202`, 637.182`, 642.202`, 0.562738`, 431.932`, 
              444.042`, 0.929382`, 1468.41`, 1468.77`, 0.0927201`, 193.262`, 
              229.002`, 1.32228`, 1458.43`, 1458.78`, 0.0886553`, 1524.74`, 
              1670.4`, 637.182`, 1901.02`, 193.262`, 1602.28`, 1606.83`, 
              2093.02`, 1458.43`, 1987.59`, 1469.77`, 1981.4`, 1454.98`, 
              1472.09`, 1979.22`, 1989.92`, 431.932`, 1468.41`, 1498.09`, 
              1632.34`}, {3, 13, 27.46824182`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2657375295`, 0.2998978272`, 0.4824916919`, 0.2`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322108`\), 1453.89`, 
              1454.38`, 0.12634`, 1480.58`, 1480.73`, 0.0387809`, 427.951`, 
              444.018`, 1.22748`, 632.204`, 637.226`, 0.55856`, 427.951`, 
              439.853`, 0.904964`, 1474.59`, 1474.97`, 0.0978271`, 194.131`, 
              229.64`, 1.31843`, 1459.3`, 1459.72`, 0.108106`, 1531.81`, 
              1670.03`, 632.204`, 1920.92`, 194.131`, 1632.51`, 1588.43`, 
              2116.21`, 1459.3`, 1994.86`, 1476.88`, 1985.15`, 1453.89`, 
              1480.58`, 1981.77`, 1998.46`, 427.951`, 1474.59`, 1504.22`, 
              1622.3`}, {1, 25, 31.18199101`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4871382349`, 0.3251927304`, 0.4404673962`, 0.2`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.165776`\), 1512.13`, 
              1510.11`, \(-0.534406`\), 1545.28`, 1553.67`, 2.27742`, 
              423.853`, 440.518`, 1.26209`, 609.833`, 614.797`, 0.53263`, 
              423.853`, 434.673`, 0.813862`, 1555.44`, 1555.82`, 0.103118`, 
              201.171`, 236.096`, 1.33804`, 1523.26`, 
              1518.48`, \(-1.27285`\), 1558.31`, 1725.93`, 609.833`, 
              2052.87`, 201.171`, 1749.51`, 1574.73`, 2208.56`, 1523.26`, 
              2103.85`, 1560.63`, 2076.36`, 1512.13`, 1569.61`, 2069.2`, 
              2111.32`, 423.853`, 1555.44`, 1545.28`, 1649.3`}, {1, 84, 
              38.98865026`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7440289446`, 0.3010605302`, 0.6590786053`, 0.2`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.213417`\), 1638.73`, 
              1636.5`, \(-0.638717`\), 1643.26`, 1650.24`, 2.01336`, 
              395.956`, 413.492`, 1.2437`, 546.018`, 552.434`, 0.617438`, 
              395.956`, 407.821`, 0.83558`, 1637.47`, 1681.21`, 12.7178`, 
              195.718`, 231.923`, 1.35655`, 1643.26`, 
              1641.75`, \(-0.433983`\), 1638.73`, 1872.51`, 546.018`, 
              2163.89`, 195.718`, 1891.26`, 1637.47`, 2385.07`, 1724.7`, 
              2330.86`, 1760.98`, 2299.8`, 1716.47`, 1767.95`, 2294.65`, 
              2336.06`, 395.956`, 1756.19`, 1643.26`, 1796.37`}, {3, 47, 
              37.13181112`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8656617307`, 0.313742479`, 0.7785446887`, 0.2`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.240169`\), 1675.05`, 
              1680.04`, 1.46598`, 1675.85`, 1686.39`, 3.10704`, 404.762`, 
              422.055`, 1.25281`, 554.579`, 560.573`, 0.585654`, 404.762`, 
              416.119`, 0.816828`, 1663.08`, 1695.25`, 9.46648`, 200.445`, 
              236.671`, 1.38737`, 1675.85`, 1683.06`, 2.1238`, 1675.05`, 
              2002.57`, 554.579`, 2222.29`, 200.445`, 2020.24`, 1663.08`, 
              2507.69`, 1863.41`, 2461.1`, 1898.89`, 2429.12`, 1856.17`, 
              1905.01`, 2424.59`, 2465.64`, 404.762`, 1894.56`, 1675.85`, 
              1929.12`}, {3, 30, 37.56112677`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.956428616`, 0.3190666391`, 0.8753078055`, 0.2`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253533`\), 1688.2`, 
              1701.51`, 3.95227`, 1692.05`, 1703.31`, 3.35002`, 405.976`, 
              423.172`, 1.24918`, 553.228`, 559.116`, 0.573849`, 405.976`, 
              417.128`, 0.804257`, 1676.5`, 1704.35`, 8.24717`, 201.735`, 
              237.954`, 1.39529`, 1688.2`, 1701.18`, 3.85284`, 1692.05`, 
              2113.12`, 553.228`, 2250.15`, 201.735`, 2124.38`, 1676.5`, 
              2596.1`, 1981.18`, 2557.16`, 2015.51`, 2524.74`, 1974.8`, 
              2020.9`, 2520.74`, 2561.16`, 405.976`, 2011.5`, 1688.2`, 
              2045.64`}, {2, 21, 41.01760896`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.023229421`, 0.3783396698`, 0.9471802626`, 0.2`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.298069`\), 1699.42`, 
              1718.84`, 5.81606`, 1709.29`, 1719.14`, 2.95827`, 397.79`, 
              418.793`, 1.50264`, 585.901`, 591.865`, 0.615444`, 397.79`, 
              410.657`, 0.911346`, 1687.87`, 1713.76`, 7.71576`, 0, 134.685`, 
              1.67426`, 1699.42`, 1714.8`, 4.60069`, 1709.29`, 2257.38`, 
              585.901`, 2268.15`, 0, 2251, 1687.87`, 2685.94`, 2086.22`, 
              2632.1`, 2121.14`, 2598.38`, 2079, 2127.16`, 2593.75`, 
              2636.73`, 397.79`, 2117.41`, 1699.42`, 2173.04`}, {3, 58, 
              39.68412177`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.098708393`, 0.3241209782`, \(-1.032405983`\), 0.2`, 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.257536`\), 1701.06`, 
              1722.6`, 6.4596`, 1708.73`, 1724.31`, 4.68377`, 403.378`, 
              420.591`, 1.24272`, 545.113`, 551.099`, 0.574935`, 403.378`, 
              414.468`, 0.794728`, 1693.08`, 1718.97`, 7.7416`, 202.732`, 
              238.867`, 1.39813`, 1701.06`, 1720.32`, 5.77314`, 1708.73`, 
              2302.13`, 545.113`, 2254.13`, 202.732`, 2291.57`, 1693.08`, 
              2708.18`, 2178.84`, 2682.07`, 2210.89`, 2648.78`, 2173.68`, 
              2215.25`, 2645.56`, 2685.29`, 403.378`, 2207.19`, 1701.06`, 
              2241.8`}, {5, 102, 15866.46231`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6732580146`, 2.339036915`, \(-0.001527705942`\), 0.2`, 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196567`, 1620.22`, 
              1617.53`, \(-0.762951`\), 1649.08`, 1649.59`, 0.148587`, 
              1562.73`, 1566.6`, 1.05983`, 1573.97`, 1575.71`, 0.480931`, 
              1562.73`, 1565.32`, 0.708211`, 1594.86`, 1595.39`, 0.147309`, 
              1555.11`, 1558.18`, 0.837329`, 1640.09`, 1640.67`, 0.165208`, 
              1649.08`, 2056.5`, 1573.97`, 1690.59`, 1555.11`, 1643.79`, 
              1886.11`, 2042.07`, 1640.09`, 1996.13`, 1680.8`, 1885.7`, 
              1620.22`, 1708.24`, 1853.17`, 2022.42`, 1562.73`, 1594.86`, 
              1666.27`, 1881.87`}, {2, 30, 26.21729474`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1953510022`, 0.2510560568`, 0.7263297882`, 0.3`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.489712`\), 1396.98`, 
              1397.36`, 0.0941812`, 1406.85`, 1406.98`, 0.0316268`, 436.698`, 
              452.15`, 1.20333`, 643.348`, 648.312`, 0.561758`, 436.698`, 
              448.909`, 0.947454`, 1404.8`, 1405.15`, 0.0867018`, 192.318`, 
              228.459`, 1.33239`, 1398.97`, 1399.43`, 0.112321`, 1465.01`, 
              1660.75`, 643.348`, 1855.16`, 192.318`, 1518.49`, 1614.16`, 
              2024.56`, 1398.97`, 1935, 1405.5`, 1931.48`, 1396.98`, 
              1406.85`, 1930.26`, 1936.31`, 436.698`, 1404.8`, 1435.97`, 
              1632.51`}, {3, 30, 26.47934932`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096496395`, 0.2607912221`, 0.653772803`, 0.3`, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.452269`\), 1398.63`, 
              1399.08`, 0.110341`, 1411.37`, 1411.52`, 0.0370596`, 433.741`, 
              449.374`, 1.20955`, 639.83`, 644.846`, 0.564619`, 433.741`, 
              445.939`, 0.940109`, 1408.67`, 1409.04`, 0.0911581`, 192.856`, 
              228.794`, 1.32766`, 1401.21`, 1401.73`, 0.129287`, 1472.18`, 
              1660.26`, 639.83`, 1864.11`, 192.856`, 1532.01`, 1609.5`, 
              2034.98`, 1401.21`, 1938.36`, 1409.62`, 1933.83`, 1398.63`, 
              1411.37`, 1932.25`, 1940.04`, 433.741`, 1408.67`, 1441.37`, 
              1629.7`}, {3, 25, 26.89840048`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2330090762`, 0.274335411`, 0.5716269531`, 0.3`, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401792`\), 1397.37`, 
              1397.94`, 0.138441`, 1414.98`, 1415.18`, 0.0473154`, 430.506`, 
              446.376`, 1.21929`, 636.306`, 641.346`, 0.564195`, 430.506`, 
              442.625`, 0.927116`, 1411.17`, 1411.56`, 0.0953439`, 193.434`, 
              229.169`, 1.32318`, 1400.95`, 1401.6`, 0.158079`, 1479.01`, 
              1659.37`, 636.306`, 1876.43`, 193.434`, 1548.98`, 1601.61`, 
              2047.85`, 1400.95`, 1941.01`, 1412.54`, 1934.75`, 1397.37`, 
              1414.98`, 1932.57`, 1943.33`, 430.506`, 1411.17`, 1445.74`, 
              1625.03`}, {1, 13, 27.75149864`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2834523561`, 0.2921577865`, 0.4758225096`, 0.3`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322726`\), 1392.07`, 
              1392.88`, 0.197603`, 1419.75`, 1420.04`, 0.0716611`, 426.362`, 
              442.581`, 1.23481`, 630.312`, 635.351`, 0.558805`, 426.362`, 
              438.218`, 0.898095`, 1413.5`, 1413.9`, 0.0983691`, 194.45`, 
              229.921`, 1.31887`, 1397.73`, 1398.6`, 0.212211`, 1484.89`, 
              1658.79`, 630.312`, 1903.82`, 194.45`, 1574.75`, 1588.18`, 
              2067.9`, 1397.73`, 1947.26`, 1415.78`, 1937.25`, 1392.07`, 
              1419.75`, 1933.88`, 1950.84`, 426.362`, 1413.5`, 1448.68`, 
              1618.16`}, {1, 25, 31.88437621`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.5700738066`, 0.3006268475`, 0.5160974987`, 0.3`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.215849`\), 1531.79`, 
              1528.85`, \(-0.788272`\), 1562.76`, 1567.91`, 1.41167`, 
              418.579`, 435.527`, 1.26822`, 593.46`, 598.845`, 0.562595`, 
              418.579`, 429.703`, 0.826733`, 1573.26`, 1573.84`, 0.160303`, 
              200.628`, 236.125`, 1.35833`, 1541.58`, 1535.4`, \(-1.66522`\), 
              1578.67`, 1746.17`, 593.46`, 2110.08`, 200.628`, 1744.85`, 
              1617.66`, 2225.44`, 1541.58`, 2136.07`, 1577.83`, 2107.9`, 
              1531.79`, 1586.02`, 2101.78`, 2142.33`, 418.579`, 1573.26`, 
              1562.76`, 1681.04`}, {1, 33, 48.03924732`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.795547672`, 0.3506320616`, 0.6553224993`, 0.3`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.243209`\), 1638.08`, 
              1622.73`, \(-4.38407`\), 1645.72`, 1653.01`, 2.10582`, 
              470.961`, 485.266`, 1.19854`, 671.529`, 673.013`, 0.174905`, 
              470.961`, 478.107`, 0.594226`, 1659.81`, 1701.07`, 12.1472`, 
              216.369`, 249.616`, 1.35742`, 1638.08`, 
              1625.28`, \(-3.65857`\), 1645.72`, 1889.95`, 671.529`, 
              2181.06`, 216.369`, 1903.73`, 1659.81`, 2373.72`, 1663.37`, 
              2282.05`, 1704.48`, 2248.08`, 1653.1`, 1712.97`, 2241.6`, 
              2288.56`, 470.961`, 1699.83`, 1638.08`, 1792.86`}, {3, 34, 
              42.31020728`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8831445908`, 0.3008919651`, 0.8207262082`, 0.3`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.283695`\), 1711.81`, 
              1716.59`, 1.43482`, 1716.73`, 1723.78`, 2.12445`, 403.495`, 
              420.964`, 1.2619`, 553.425`, 559.569`, 0.599187`, 403.495`, 
              415.064`, 0.829753`, 1682.12`, 1733.55`, 15.3936`, 196.352`, 
              233.521`, 1.39992`, 1716.73`, 1721.51`, 1.43856`, 1711.81`, 
              2005.38`, 553.425`, 2262.05`, 196.352`, 2042.89`, 1682.12`, 
              2530.24`, 1891.85`, 2485.22`, 1925.82`, 2453.98`, 1885.5`, 
              1931.22`, 2450.05`, 2489.16`, 403.495`, 1922.07`, 1716.73`, 
              1932.61`}, {3, 29, 40.85795749`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9708542651`, 0.3104767919`, 0.9102629088`, 0.3`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291358`\), 1719.79`, 
              1733.39`, 4.11342`, 1722.49`, 1734.85`, 3.74489`, 409.065`, 
              426.216`, 1.25522`, 558.685`, 564.454`, 0.567699`, 409.065`, 
              420.127`, 0.803676`, 1692.55`, 1734.16`, 12.4932`, 199.835`, 
              236.599`, 1.40579`, 1719.79`, 1733.89`, 4.26674`, 1722.49`, 
              2118.07`, 558.685`, 2275.02`, 199.835`, 2147.27`, 1692.55`, 
              2611.49`, 2004.18`, 2572.53`, 2037.44`, 2540.54`, 1998.4`, 
              2042.34`, 2536.95`, 2576.12`, 409.065`, 2033.92`, 1719.79`, 
              2052.19`}, {5, 101, 15937.30243`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6822921571`, 1.7705001`, \(-0.00009586744928`\), 0.3`, 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196605`, 1609.17`, 
              1607.5`, \(-0.469272`\), 1647.59`, 1647.95`, 0.102164`, 
              1565.96`, 1569.22`, 0.897517`, 1583.96`, 1584.11`, 0.0422089`, 
              1565.96`, 1567.79`, 0.50509`, 1595.9`, 1596.35`, 0.127439`, 
              1556.03`, 1558.39`, 0.644957`, 1632.37`, 1632.58`, 0.0597166`, 
              1647.59`, 2064.49`, 1583.96`, 1691.29`, 1556.03`, 1640.68`, 
              1881.95`, 2042.96`, 1632.37`, 1973.26`, 1680.9`, 1881.15`, 
              1609.17`, 1718, 1826.82`, 2018.1`, 1565.96`, 1595.9`, 1670.98`, 
              1878.14`}, {5, 102, 15876.46056`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6698257385`, 2.190934726`, \(-0.001051920529`\), 0.3`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196635`, 1618.01`, 
              1615.53`, \(-0.701459`\), 1648.77`, 1649.25`, 0.138873`, 
              1563.38`, 1567.14`, 1.03254`, 1575.81`, 1577.28`, 0.40697`, 
              1563.38`, 1565.81`, 0.666217`, 1595.02`, 1595.54`, 0.144453`, 
              1555.23`, 1558.17`, 0.799866`, 1638.72`, 1639.23`, 0.144581`, 
              1648.77`, 2057.95`, 1575.81`, 1690.96`, 1555.23`, 1643.15`, 
              1885.46`, 2042.19`, 1638.72`, 1992.33`, 1680.8`, 1884.81`, 
              1618.01`, 1710.12`, 1848.51`, 2021.81`, 1563.38`, 1595.02`, 
              1667.14`, 1881.11`}, {5, 103, 15842.30045`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6276474525`, 2.665562465`, 0.001006676449`, 0.3`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196662`, 1626.7`, 
              1623.33`, \(-0.961519`\), 1649.54`, 1650.16`, 0.179544`, 
              1560.67`, 1564.75`, 1.11546`, 1568.51`, 1571, 0.685321`, 
              1560.67`, 1563.71`, 0.830405`, 1594.38`, 1594.92`, 0.151425`, 
              1554.82`, 1558.25`, 0.937165`, 1643.38`, 
              1643.09`, \(-0.0845689`\), 1649.54`, 2052.24`, 1568.51`, 
              1690.68`, 1554.82`, 1645.43`, 1887.77`, 2041.79`, 1643.38`, 
              2005.52`, 1680.59`, 1887.3`, 1626.7`, 1702.29`, 1864.76`, 
              2023.64`, 1560.67`, 1594.38`, 1663.39`, 1883.08`}, {2, 26, 
              26.37610434`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2018577825`, 0.2520211052`, 0.7240034911`, 0.4`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.489121`\), 1365.68`, 
              1366.15`, 0.111857`, 1375.95`, 1376.11`, 0.0385716`, 435.43`, 
              450.985`, 1.20803`, 642.409`, 647.383`, 0.562114`, 435.43`, 
              447.659`, 0.946174`, 1373.8`, 1374.16`, 0.0879887`, 192.492`, 
              228.613`, 1.33272`, 1367.76`, 1368.33`, 0.136631`, 1439.1`, 
              1656.01`, 642.409`, 1842.41`, 192.492`, 1491.74`, 1609.33`, 
              2002.59`, 1367.76`, 1910.49`, 1374.52`, 1906.86`, 1365.68`, 
              1375.95`, 1905.6`, 1911.83`, 435.43`, 1373.8`, 1407.3`, 
              1627.83`}, {1, 30, 26.65743495`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2175260691`, 0.260660191`, 0.6508576361`, 0.4`, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.45193`\), 1362.37`, 
              1362.92`, 0.132372`, 1375.61`, 1375.8`, 0.045898`, 432.306`, 
              448.053`, 1.21469`, 638.51`, 643.547`, 0.56577`, 432.306`, 
              444.521`, 0.938483`, 1372.8`, 1373.18`, 0.0922046`, 193.058`, 
              228.978`, 1.32822`, 1365.06`, 1365.73`, 0.159677`, 1441.98`, 
              1655.19`, 638.51`, 1850.05`, 193.058`, 1500.47`, 1604.9`, 
              2009.15`, 1365.06`, 1910.23`, 1373.76`, 1905.56`, 1362.37`, 
              1375.61`, 1903.94`, 1911.95`, 432.306`, 1372.8`, 1407.99`, 
              1625.11`}, {2, 21, 27.1119413`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2436062887`, 0.2724051249`, 0.5681124253`, 0.4`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401684`\), 1356.41`, 
              1357.11`, 0.167219`, 1374.74`, 1374.98`, 0.0592386`, 428.946`, 
              444.941`, 1.22467`, 634.496`, 639.563`, 0.565662`, 428.946`, 
              441.072`, 0.924304`, 1370.75`, 1371.15`, 0.0952294`, 193.703`, 
              229.414`, 1.3239`, 1360.16`, 1360.99`, 0.197598`, 1444.77`, 
              1654.66`, 634.496`, 1862.61`, 193.703`, 1512.82`, 1598.39`, 
              2018.5`, 1360.16`, 1910, 1372.13`, 1903.53`, 1356.41`, 
              1374.74`, 1901.31`, 1912.36`, 428.946`, 1370.75`, 1407.84`, 
              1621.59`}, {1, 25, 28.08117697`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3033754804`, 0.2871306462`, 0.4733710997`, 0.4`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322796`\), 1353.3`, 
              1354.31`, 0.240061`, 1382.52`, 1382.89`, 0.0899255`, 425.298`, 
              441.618`, 1.23959`, 628.381`, 633.408`, 0.555656`, 425.298`, 
              437.075`, 0.889819`, 1375.91`, 1376.3`, 0.0939145`, 194.738`, 
              230.191`, 1.31994`, 1359.29`, 1360.41`, 0.266702`, 1452.36`, 
              1658.62`, 628.381`, 1899.47`, 194.738`, 1540.85`, 1590.92`, 
              2041.79`, 1359.29`, 1921.58`, 1378.2`, 1910.87`, 1353.3`, 
              1382.52`, 1907.41`, 1925.23`, 425.298`, 1375.91`, 1412.49`, 
              1620.63`}, {3, 35, 32.33289021`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6254653251`, 0.2875019791`, 0.5762507532`, 0.4`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.266858`\), 1554.59`, 
              1551.31`, \(-0.891157`\), 1584.38`, 1588.18`, 1.05661`, 
              418.986`, 435.855`, 1.2635`, 590.421`, 595.767`, 0.555541`, 
              418.986`, 430.065`, 0.824192`, 1594.36`, 1595.05`, 0.193194`, 
              200.721`, 236.438`, 1.36804`, 1563.22`, 
              1556.68`, \(-1.78804`\), 1601.4`, 1766.71`, 590.421`, 2150.61`, 
              200.721`, 1755.21`, 1648.58`, 2253.34`, 1563.22`, 2170.45`, 
              1598.39`, 2142.23`, 1554.59`, 1605.7`, 2136.94`, 2175.83`, 
              418.986`, 1594.36`, 1584.38`, 1706.51`}, {1, 60, 
              49.62603699`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8136854988`, 0.3378389991`, 0.6953338684`, 0.4`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291217`\), 1667.93`, 
              1647.26`, \(-6.00551`\), 1674.36`, 1679.76`, 1.58434`, 
              472.579`, 486.726`, 1.18904`, 672.752`, 674.144`, 0.164251`, 
              472.579`, 479.65`, 0.589915`, 1679.46`, 1719.85`, 12.0278`, 
              215.241`, 248.699`, 1.36002`, 1667.93`, 
              1650.25`, \(-5.14008`\), 1674.81`, 1890.97`, 672.752`, 
              2221.01`, 215.241`, 1916.19`, 1679.46`, 2394.89`, 1683.57`, 
              2304.03`, 1722.74`, 2271.1`, 1674.36`, 1730.43`, 2265.36`, 
              2309.79`, 472.579`, 1718.71`, 1667.93`, 1797.57`}, {3, 25, 
              51.82451361`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8991240551`, 0.2886073519`, 0.8619662269`, 0.4`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321842`\), 1748.21`, 
              1751.59`, 1.03698`, 1757.83`, 1759.44`, 0.49819`, 398.017`, 
              415.962`, 1.27977`, 546.128`, 552.734`, 0.636093`, 398.017`, 
              410.147`, 0.85886`, 1698.16`, 1770.67`, 22.0378`, 190.233`, 
              228.846`, 1.4178`, 1757.83`, 1758.2`, 0.114516`, 1748.21`, 
              2009.42`, 546.128`, 2296.69`, 190.233`, 2066.65`, 1698.16`, 
              2552.37`, 1924.08`, 2510.78`, 1956.7`, 2480.14`, 1918.54`, 
              1961.44`, 2476.74`, 2514.18`, 398.017`, 1953.37`, 1757.83`, 
              1936.87`}, {5, 102, 15858.64884`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6790938445`, 2.477991156`, \(-0.0001092583036`\), 0.4`, 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196674`, 1621.93`, 
              1619.07`, \(-0.813091`\), 1649.29`, 1649.83`, 0.157286`, 
              1562.21`, 1566.15`, 1.07942`, 1572.54`, 1574.48`, 0.53631`, 
              1562.21`, 1564.92`, 0.740923`, 1594.71`, 1595.25`, 0.149315`, 
              1555.01`, 1558.18`, 0.866059`, 1641.09`, 1641.84`, 0.214605`, 
              1649.29`, 2055.37`, 1572.54`, 1691.06`, 1555.01`, 1644.27`, 
              1887.06`, 2041.97`, 1641.09`, 1998.88`, 1680.77`, 1886.37`, 
              1621.93`, 1706.74`, 1856.66`, 2022.79`, 1562.21`, 1594.71`, 
              1665.72`, 1882.45`}, {5, 102, 15875.73188`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6451119775`, 2.118072112`, \(-0.001047719699`\), 0.4`, 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196708`, 1618.03`, 
              1615.55`, \(-0.703569`\), 1648.59`, 1649.07`, 0.138885`, 
              1563.3`, 1567.05`, 1.02802`, 1575.58`, 1577.08`, 0.413088`, 
              1563.3`, 1565.73`, 0.6661`, 1594.99`, 1595.5`, 0.143529`, 
              1555.23`, 1558.15`, 0.797724`, 1638.6`, 1639.11`, 0.144948`, 
              1648.59`, 2057.79`, 1575.58`, 1691.06`, 1555.23`, 1643.04`, 
              1885.14`, 2042.18`, 1638.6`, 1992.27`, 1680.7`, 1884.37`, 
              1618.03`, 1709.91`, 1848.18`, 2021.81`, 1563.3`, 1594.99`, 
              1666.78`, 1880.64`}, {5, 102, 15847.63844`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.625101401`, 2.497620544`, \(-0.00004723264639`\), 0.4`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196685`, 1624.75`, 
              1621.58`, \(-0.900949`\), 1649.29`, 1649.88`, 0.170294`, 
              1561.24`, 1565.25`, 1.09801`, 1569.96`, 1572.25`, 0.628985`, 
              1561.24`, 1564.14`, 0.793406`, 1594.48`, 1595.01`, 0.149942`, 
              1554.87`, 1558.2`, 0.906457`, 1642.38`, 1642.52`, 0.041691`, 
              1649.29`, 2053.37`, 1569.96`, 1690.87`, 1554.87`, 1644.86`, 
              1887.19`, 2041.85`, 1642.38`, 2002.91`, 1680.58`, 1886.59`, 
              1624.75`, 1703.94`, 1861.31`, 2023.33`, 1561.24`, 1594.48`, 
              1664.05`, 1882.46`}, {5, 103, 15829.96654`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.5005670901`, 2.551263159`, \(-0.0008061323063`\), 0.4`, 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196693`, 1631.38`, 
              1627.44`, \(-1.12603`\), 1649.34`, 1650.04`, 0.2004`, 1559.06`, 
              1563.18`, 1.12718`, 1564.56`, 1567.54`, 0.817872`, 1559.06`, 
              1562.39`, 0.910975`, 1594.05`, 1594.58`, 0.149762`, 1554.68`, 
              1558.32`, 0.993296`, 1644.99`, 1643.98`, \(-0.289459`\), 
              1649.34`, 2049.19`, 1564.56`, 1690.52`, 1554.68`, 1646.25`, 
              1887.49`, 2041.66`, 1644.99`, 2011.04`, 1680.2`, 1887.1`, 
              1631.38`, 1697.39`, 1870.92`, 2024.2`, 1559.06`, 1594.05`, 
              1660.66`, 1882.56`}, {3, 30, 26.5358766`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2085541656`, 0.2523097178`, 0.7223324593`, 0.5`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.487144`\), 1339.66`, 
              1340.2`, 0.126349`, 1350.33`, 1350.52`, 0.0442383`, 433.492`, 
              449.157`, 1.21141`, 639.674`, 644.706`, 0.566349`, 433.492`, 
              445.763`, 0.945294`, 1348.09`, 1348.46`, 0.0882108`, 192.842`, 
              228.893`, 1.3321`, 1341.83`, 1342.5`, 0.156017`, 1417.06`, 
              1652.53`, 639.674`, 1833.56`, 192.842`, 1469.77`, 1605.57`, 
              1985.13`, 1341.83`, 1891.03`, 1348.82`, 1887.29`, 1339.66`, 
              1350.33`, 1886, 1892.41`, 433.492`, 1348.09`, 1383.22`, 
              1624.35`}, {2, 22, 26.84013375`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2252721927`, 0.2621364132`, 0.6488297093`, 0.5`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.451616`\), 1335.26`, 
              1335.9`, 0.149324`, 1349.09`, 1349.31`, 0.052518`, 431.7`, 
              447.528`, 1.21931`, 638.787`, 643.788`, 0.56202`, 431.7`, 
              443.897`, 0.935734`, 1346.14`, 1346.54`, 0.0936707`, 193.187`, 
              229.114`, 1.32932`, 1338.08`, 1338.86`, 0.182361`, 1419.89`, 
              1653.04`, 638.787`, 1841.04`, 193.187`, 1478.68`, 1602.02`, 
              1991.75`, 1338.08`, 1889.93`, 1347.13`, 1885.08`, 1335.26`, 
              1349.09`, 1883.41`, 1891.7`, 431.7`, 1346.14`, 1383.41`, 
              1622.7`}, {2, 21, 27.33080295`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2545628685`, 0.2713642313`, 0.5660558299`, 0.5`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401109`\), 1326.47`, 
              1327.29`, 0.189573`, 1345.65`, 1345.94`, 0.0682157`, 427.58`, 
              443.677`, 1.22878`, 632.658`, 637.743`, 0.565986`, 427.58`, 
              439.7`, 0.92097`, 1341.46`, 1341.86`, 0.0941148`, 193.949`, 
              229.639`, 1.32458`, 1330.41`, 1331.39`, 0.226569`, 1419.33`, 
              1653.45`, 632.658`, 1855.26`, 193.949`, 1487.81`, 1597.17`, 
              1999.15`, 1330.41`, 1888.96`, 1342.87`, 1882.18`, 1326.47`, 
              1345.65`, 1879.9`, 1891.38`, 427.58`, 1341.46`, 1380.03`, 
              1620.81`}, {1, 18, 28.42954934`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3258041071`, 0.2824874861`, 0.4740856013`, 0.5`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323511`\), 1329.64`, 
              1330.82`, 0.274727`, 1360.76`, 1361.19`, 0.103819`, 424.368`, 
              440.756`, 1.2422`, 625.689`, 630.706`, 0.552268`, 424.368`, 
              436.054`, 0.880956`, 1353.73`, 1354.1`, 0.0868136`, 195.122`, 
              230.542`, 1.32099`, 1336, 1337.3`, 0.305161`, 1431.03`, 
              1663.97`, 625.689`, 1903.9`, 195.122`, 1521.53`, 1597, 
              2029.25`, 1336, 1910.41`, 1356.04`, 1898.66`, 1329.64`, 
              1360.76`, 1895.06`, 1914.19`, 424.368`, 1353.73`, 1389.64`, 
              1627.48`}, {3, 39, 32.72386653`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6668310813`, 0.2782793072`, 0.6285105317`, 0.5`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.313275`\), 1581.98`, 
              1577.81`, \(-1.15556`\), 1609.73`, 1613.78`, 1.14242`, 
              419.812`, 436.562`, 1.25685`, 589.095`, 594.365`, 0.546468`, 
              419.812`, 430.819`, 0.820377`, 1620.22`, 1620.98`, 0.213619`, 
              200.915`, 236.745`, 1.37397`, 1589.65`, 
              1582.27`, \(-2.05179`\), 1626.51`, 1784.66`, 589.095`, 
              2186.06`, 200.915`, 1772.91`, 1672.1`, 2282.66`, 1589.65`, 
              2204.94`, 1623.8`, 2176.85`, 1581.98`, 1630.34`, 2172.21`, 
              2209.64`, 419.812`, 1620.22`, 1609.73`, 1727.35`}, {3, 55, 
              52.36904577`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8320634473`, 0.3365607577`, 0.7245972047`, 0.5`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.330001`\), 1685.8`, 
              1662.93`, \(-6.71027`\), 1688.42`, 1698.86`, 3.09675`, 
              484.276`, 498.043`, 1.18488`, 691.11`, 691.908`, 0.0966008`, 
              484.276`, 490.848`, 0.56154`, 1695.81`, 1730.41`, 10.3851`, 
              218.737`, 251.979`, 1.37098`, 1688.42`, 
              1666.03`, \(-6.58088`\), 1697.51`, 1898.36`, 691.11`, 2248.63`, 
              218.737`, 1931.78`, 1695.81`, 2411.96`, 1694.56`, 2316.75`, 
              1733.01`, 2284.17`, 1685.8`, 1740.34`, 2278.74`, 2322.19`, 
              484.276`, 1729.36`, 1688.42`, 1805.44`}, {3, 48, 
              110.4748928`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9514614266`, 0.4023702337`, 0.7914407531`, 0.5`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.358943`\), 1726.67`, 
              1709.11`, \(-5.28604`\), 1737.88`, 1747.62`, 2.97539`, 555.73`, 
              567.766`, 1.18481`, 809.492`, 806.499`, \(-0.423729`\), 
              555.73`, 559.043`, 0.323656`, 1714.17`, 1804.21`, 27.7569`, 
              239.687`, 271.814`, 1.43979`, 1726.67`, 
              1711.24`, \(-4.64686`\), 1737.88`, 2020.82`, 809.492`, 
              2280.41`, 239.687`, 2068.68`, 1714.17`, 2509.88`, 1766.72`, 
              2385.95`, 1808.48`, 2350.38`, 1756.59`, 1816.77`, 2343.95`, 
              2392.36`, 555.73`, 1804.71`, 1726.67`, 1893.37`}, {5, 102, 
              15857.31187`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6194801725`, 2.285110666`, 0.0008580338071`, 0.5`, 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196726`, 1622.07`, 
              1619.18`, \(-0.820845`\), 1648.94`, 1649.48`, 0.157485`, 
              1562.03`, 1565.93`, 1.07033`, 1572.04`, 1574.03`, 0.548976`, 
              1562.03`, 1564.74`, 0.742285`, 1594.65`, 1595.18`, 0.147332`, 
              1554.99`, 1558.16`, 0.862715`, 1640.9`, 1641.63`, 0.211087`, 
              1648.94`, 2055, 1572.04`, 1691.01`, 1554.99`, 1644.08`, 
              1886.25`, 2041.96`, 1640.9`, 1998.98`, 1680.58`, 1885.52`, 
              1622.07`, 1706.21`, 1856.2`, 2022.83`, 1562.03`, 1594.65`, 
              1664.94`, 1881.52`}, {5, 102, 15854.02033`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6172530946`, 2.339027839`, \(-0.0009905252751`\), 0.5`, 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196723`, 1622.96`, 
              1619.98`, \(-0.847179`\), 1649.04`, 1649.6`, 0.161675`, 
              1561.76`, 1565.7`, 1.07912`, 1571.32`, 1573.41`, 0.576297`, 
              1561.76`, 1564.53`, 0.759041`, 1594.59`, 1595.12`, 0.148109`, 
              1554.96`, 1558.17`, 0.876891`, 1641.38`, 1642, 0.17657`, 
              1649.04`, 2054.44`, 1571.32`, 1690.99`, 1554.96`, 1644.33`, 
              1886.53`, 2041.92`, 1641.38`, 2000.32`, 1680.57`, 1885.82`, 
              1622.96`, 1705.43`, 1857.88`, 2023.01`, 1561.76`, 1594.59`, 
              1664.6`, 1881.78`}, {1, 95, 15829.79746`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4842894556`, 2.45990978`, \(-0.0001111120852`\), 0.5`, 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.196697`, 1631.18`, 
              1627.25`, \(-1.12125`\), 1649.18`, 1649.87`, 0.199342`, 
              1559.06`, 1563.16`, 1.1219`, 1564.57`, 1567.54`, 0.813651`, 
              1559.06`, 1562.38`, 0.906316`, 1594.03`, 1594.56`, 0.148906`, 
              1554.67`, 1558.3`, 0.988087`, 1644.8`, 
              1643.83`, \(-0.278309`\), 1649.18`, 2049.21`, 1564.57`, 
              1690.57`, 1554.67`, 1646.1`, 1887.15`, 2041.66`, 1644.8`, 
              2010.83`, 1680.13`, 1886.72`, 1631.18`, 1697.39`, 1870.39`, 
              2024.18`, 1559.06`, 1594.03`, 1660.49`, 1882.16`}, {5, 101, 
              15824.37613`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3494299885`, 2.022279942`, 0.000816394416`, 0.5`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.1967`, 1633.34`, 
              1629.08`, \(-1.2173`\), 1648.14`, 1648.85`, 0.204517`, 
              1558.05`, 1562.06`, 1.09527`, 1562.2`, 1565.37`, 0.86851`, 
              1558.05`, 1561.46`, 0.930635`, 1593.8`, 1594.31`, 0.14248`, 
              1554.63`, 1558.27`, 0.993091`, 1644.7`, 
              1643.62`, \(-0.311499`\), 1648.14`, 2047.4`, 1562.2`, 1690.05`, 
              1554.63`, 1645.78`, 1884.64`, 2041.61`, 1644.7`, 2013.31`, 
              1679.47`, 1884.33`, 1633.34`, 1693.86`, 1871.23`, 2024.39`, 
              1558.05`, 1593.8`, 1657.76`, 1879.44`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data three/ytraj_m0_m1_3.out", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for transverse lattice mesons \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\n\
sqrt(string tension) is 440 MeV.\nUse chi^2 fit with 11 criteria and \
tolerance 0.001.\nOverall scale from glueball data.\n0 nonstandard fit \
criteria (for log divergence).\nFit 3 masses to experimental values\n  \
(charge,multi,state,mass (MeV),error (MeV)) =\n   (1,4,0,139.557,69.3623) \
(-1,3,0,785.988,73.8942)\n   (-1,7,0,785.988,73.8942).\nNo degenerate pairs.\n\
Spectrum for P_perp a = (0,0), ( 0.25 0.25)\n  for 2 states (multiplet, \
charge, c^2 error for each) =\n  (17,  1, 5 5)\n  (17, -1, 0.2 0.2)\n  (18, \
-1, 0.2 5)\n  (18,  1, 0.2 5).\nOrdinary spectra multiplets, 2 states, \
(charge,multiplet) =\n  ( 1,  3)  (-1,  3)  ( 1,  4)  (-1,  4)  ( 1,  5)  \
(-1,  5) \n  ( 1,  6)  (-1,  6)  (-1,  7)  ( 1,  7) .\nAll spectra \
extrapolated using (K,p) = (12/2,4)  (14/2,4) \n  (18/2,4) \nLog divergence \
determined using pion (K,p) = (20/2,4) \n  (24/2,4)  (28/2,4)  (36/2,4) .\n\
Result format:\n Fit info, # steps, chi^2, p damping, scale G^2 N/sigma.\n \
The 17 couplings  (G^2 N units);\n   which couplings -- if any -- were fit.\n \
Winding potential and heavy souce potential fits (G^2 N units).\n The log(K) \
coefficient of the pion mass^2 (G^2 N units).\n The rescaled masses (MeV) for \
each P_perp*a and c^2 values.\n Finally come the masses (MeV) for the \
ordinary spectra.\>", {{1, 38, 26.15638484`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2257946972`, 0.4381263674`, 
              0.6634923493`, \(-0.3`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.325659`\), 1535.93`, 
              1538.64`, 0.732012`, 1559.18`, 1561.79`, 0.712401`, 461.186`, 
              475.326`, 1.16019`, 648.791`, 653.507`, 0.538033`, 461.186`, 
              472.54`, 0.928863`, 1594.61`, 1596.25`, 0.456001`, 190.207`, 
              228.693`, 1.41255`, 1559.18`, 1561.88`, 0.736909`, 1535.93`, 
              2442.7`, 648.791`, 2146.86`, 190.207`, 2094.81`, 1594.61`, 
              2600.98`, 2215.49`, 2687.68`, 2236.62`, 2673.51`, 2210.28`, 
              2239.77`, 2668.7`, 2692.94`, 461.186`, 2113.9`, 1559.18`, 
              2327.22`}, {3, 21, 25.40495877`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2433282825`, 0.4797613829`, 
              0.6043922139`, \(-0.3`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.237714`\), 1489.92`, 
              1492.63`, 0.708149`, 1520.44`, 1522.79`, 0.627467`, 481.054`, 
              494.208`, 1.12399`, 656.834`, 661.536`, 0.543137`, 481.054`, 
              491.558`, 0.895175`, 1565.57`, 1566.64`, 0.293911`, 187.753`, 
              227.742`, 1.4558`, 1520.44`, 1522.94`, 0.667693`, 1489.92`, 
              2498.24`, 656.834`, 2090.9`, 187.753`, 2030.2`, 1565.57`, 
              2527.47`, 2247.37`, 2726.74`, 2277.01`, 2706.92`, 2240.33`, 
              2281.21`, 2700, 2734.6`, 481.054`, 2051.48`, 1520.44`, 
              2491.11`}, {3, 26, 23.84402715`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2644014832`, 0.5255373782`, 0.5481391419`, \(-0.3`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0930116`\), 1354.7`, 
              1357.61`, 0.690933`, 1399.72`, 1401.67`, 0.477943`, 515.219`, 
              526.718`, 1.04974`, 668.555`, 673.602`, 0.593475`, 515.219`, 
              524.501`, 0.845519`, 1459.72`, 1460.1`, 0.0975691`, 183.523`, 
              226.221`, 1.53288`, 1399.72`, 1402.05`, 0.572449`, 1354.7`, 
              2291.67`, 668.555`, 1898.13`, 183.523`, 1836.96`, 1459.72`, 
              2300.77`, 2272.46`, 2760.77`, 2313.42`, 2733.58`, 2263.07`, 
              2319.18`, 2723.16`, 2773.41`, 515.219`, 1853.18`, 1399.72`, 
              2284.7`}, {3, 52, 22.75006599`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2978151588`, 0.5750211814`, 0.5101575555`, \(-0.3`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0716395`, 1169.78`, 
              1172.81`, 0.620097`, 1222.49`, 1224.1`, 0.343739`, 541.488`, 
              551.086`, 0.918821`, 655.06`, 661.575`, 0.751562`, 541.488`, 
              549.828`, 0.797461`, 1288.96`, 1288.98`, 0.00318139`, 183.086`, 
              227.532`, 1.59903`, 1222.49`, 1224.68`, 0.469172`, 1169.78`, 
              2202.35`, 655.06`, 1725.35`, 183.086`, 1665.9`, 1288.96`, 
              2195.91`, 2293.91`, 2786.66`, 2346.09`, 2752.89`, 2281.93`, 
              2354.21`, 2736.95`, 2807.2`, 541.488`, 1673.43`, 1222.49`, 
              2181.49`}, {3, 30, 37.41206821`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.919014194`, 0.539280296`, 0.6216090011`, \(-0.3`\), 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0167871`, 1431.46`, 
              1433.73`, 0.571484`, 1439.26`, 1447.57`, 2.10151`, 400.483`, 
              418.244`, 1.27402`, 519.163`, 525.826`, 0.61007`, 400.483`, 
              411.286`, 0.76831`, 1468.19`, 1470.17`, 0.508615`, 204.296`, 
              240.824`, 1.4246`, 1431.46`, 1434.11`, 0.666096`, 1439.26`, 
              2183.94`, 519.163`, 1947.12`, 204.296`, 1808.2`, 1468.19`, 
              2348.44`, 1817.01`, 2420.06`, 1875.02`, 2374.41`, 1800.31`, 
              1887.31`, 2362.53`, 2432, 400.483`, 1825.01`, 1431.46`, 
              2030.66`}, {2, 22, 37.58681992`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.984571322`, 0.4694715871`, 0.7429146685`, \(-0.3`\), 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0415118`\), 
              1510.78`, 1513.81`, 0.801881`, 1514.28`, 1523.85`, 2.54641`, 
              388.404`, 406.374`, 1.25134`, 513.217`, 519.841`, 0.59955`, 
              388.404`, 399.849`, 0.790455`, 1534.51`, 1538.14`, 0.97797`, 
              205.143`, 240.932`, 1.39874`, 1510.78`, 1514.6`, 1.01136`, 
              1514.28`, 2221.08`, 513.217`, 2041.32`, 205.143`, 1935.18`, 
              1534.51`, 2509.93`, 1935.67`, 2527.01`, 1982.81`, 2486.98`, 
              1922.84`, 1992.85`, 2478.2`, 2535.77`, 388.404`, 1941.83`, 
              1510.78`, 2096.04`}, {3, 38, 37.65742644`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.054631639`, 0.4358815791`, 0.842072018`, \(-0.3`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0681205`\), 
              1557.35`, 1561.06`, 1.01344`, 1558.78`, 1569.11`, 2.83218`, 
              384.577`, 402.537`, 1.23855`, 509.414`, 516.093`, 0.600173`, 
              384.577`, 396.223`, 0.796686`, 1574.83`, 1579.49`, 1.28851`, 
              205.636`, 241.141`, 1.38987`, 1557.35`, 1562.04`, 1.28039`, 
              1558.78`, 2278.86`, 509.414`, 2113.67`, 205.636`, 2030.33`, 
              1574.83`, 2665.27`, 2037.56`, 2608.03`, 2079.51`, 2570.26`, 
              2027.15`, 2087.86`, 2563.27`, 2615, 384.577`, 2038.16`, 
              1557.35`, 2170.39`}, {3, 45, 37.74048355`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.117780155`, 0.4171575832`, 0.9270763062`, \(-0.3`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0831215`\), 
              1587.01`, 1591.86`, 1.35121`, 1587.95`, 1598.65`, 2.987`, 
              382.65`, 400.589`, 1.23105`, 506.642`, 513.379`, 0.60217`, 
              382.65`, 394.383`, 0.798794`, 1600.19`, 1605.86`, 1.593`, 
              206.007`, 241.35`, 1.38533`, 1587.01`, 1592.59`, 1.55565`, 
              1587.95`, 2345.12`, 506.642`, 2165.4`, 206.007`, 2101.88`, 
              1600.19`, 2790.42`, 2131.17`, 2671, 2169.9`, 2634.25`, 
              2122.37`, 2177.04`, 2628.4`, 2676.83`, 382.65`, 2112.4`, 
              1587.01`, 2247.24`}, {3, 45, 37.87097492`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.175338015`, 0.405617032`, 1.002509981`, \(-0.3`\), 0.6`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0910184`\), 1606.8`, 
              1613.18`, 1.79722`, 1607.92`, 1619.04`, 3.14501`, 381.439`, 
              399.365`, 1.2263`, 504.51`, 511.298`, 0.604101`, 381.439`, 
              393.214`, 0.799227`, 1616.83`, 1623.68`, 1.94477`, 206.313`, 
              241.543`, 1.38243`, 1606.8`, 1613.48`, 1.88364`, 1607.92`, 
              2414.18`, 504.51`, 2198.03`, 206.313`, 2150.02`, 1616.83`, 
              2882.9`, 2218.6`, 2719.29`, 2254.98`, 2682.98`, 2210.94`, 
              2261.22`, 2677.93`, 2724.32`, 381.439`, 2161.59`, 1606.8`, 
              2323.57`}, {3, 49, 38.0756481`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.228712843`, 0.3979828018`, 1.070966809`, \(-0.3`\), 0.7`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0939949`\), 
              1621.13`, 1629.29`, 2.32419`, 1622.63`, 1634.46`, 3.37661`, 
              380.623`, 398.541`, 1.22317`, 502.878`, 509.705`, 0.605657`, 
              380.623`, 392.418`, 0.798833`, 1628.78`, 1637.04`, 2.36208`, 
              206.574`, 241.719`, 1.38041`, 1621.13`, 1629.14`, 2.28145`, 
              1622.63`, 2483.73`, 502.878`, 2214.85`, 206.574`, 2178.29`, 
              1628.78`, 2944.7`, 2301.14`, 2755.85`, 2335.62`, 2719.68`, 
              2294.33`, 2341.18`, 2715.21`, 2760.29`, 380.623`, 2189.23`, 
              1621.13`, 2398.36`}, {3, 37, 38.37904358`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.278837419`, 0.3926978514`, 1.134041355`, \(-0.3`\), 0.8`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0933928`\), 
              1632.65`, 1642.79`, 2.91131`, 1634.59`, 1647.46`, 3.70182`, 
              380.089`, 398.001`, 1.22109`, 501.676`, 508.531`, 0.606753`, 
              380.089`, 391.887`, 0.797953`, 1638.49`, 1648.37`, 2.84459`, 
              206.794`, 241.874`, 1.37901`, 1632.65`, 1642.23`, 2.74816`, 
              1634.59`, 2552.62`, 501.676`, 2218.89`, 206.794`, 2190.62`, 
              1638.49`, 2981.13`, 2379.62`, 2783.12`, 2412.5`, 2746.9`, 
              2373.49`, 2417.52`, 2742.88`, 2787.09`, 380.089`, 2200.22`, 
              1632.65`, 2471.17`}, {3, 25, 26.56192937`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2308974985`, 0.4099228077`, 
              0.6108697967`, \(-0.2`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321355`\), 1541.6`, 
              1544.38`, 0.750855`, 1560.53`, 1563.44`, 0.796309`, 450.331`, 
              465.105`, 1.18498`, 643.974`, 648.75`, 0.540957`, 450.331`, 
              462, 0.932749`, 1588.65`, 1590.91`, 0.629587`, 191.472`, 
              229.063`, 1.3851`, 1560.53`, 1563.5`, 0.812745`, 1541.6`, 
              2249.11`, 643.974`, 2120.53`, 191.472`, 2065.61`, 1588.65`, 
              2543.66`, 2050.53`, 2531.8`, 2071.82`, 2517.68`, 2045.05`, 
              2075.26`, 2512.83`, 2537.09`, 450.331`, 2068.73`, 1560.53`, 
              2156.66`}, {3, 23, 26.12136428`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2572047293`, 0.4602084662`, 
              0.5320222209`, \(-0.2`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.224494`\), 1496.97`, 
              1499.49`, 0.661754`, 1518.97`, 1521.69`, 0.723195`, 466.572`, 
              480.577`, 1.16219`, 650.837`, 655.509`, 0.534842`, 466.572`, 
              477.443`, 0.899138`, 1556.66`, 1558.21`, 0.424532`, 189.564`, 
              228.328`, 1.4193`, 1518.97`, 1521.63`, 0.70797`, 1496.97`, 
              2331.73`, 650.837`, 2081.28`, 189.564`, 2008.96`, 1556.66`, 
              2526.99`, 2062.17`, 2552.36`, 2094.74`, 2530.97`, 2054.18`, 
              2099.7`, 2523.43`, 2560.91`, 466.572`, 2036.14`, 1518.97`, 
              2195.61`}, {3, 26, 23.72891556`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2950641059`, 0.5445215478`, 0.4283751388`, \(-0.2`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.00243022`\), 
              1294.37`, 1297.1`, 0.619085`, 1336.09`, 1338.14`, 0.481073`, 
              523.327`, 534.74`, 1.058`, 676.007`, 680.923`, 0.584408`, 
              523.327`, 532.115`, 0.812669`, 1399.67`, 1399.99`, 0.0789593`, 
              181.414`, 224.637`, 1.53772`, 1336.09`, 1338.49`, 0.563117`, 
              1294.37`, 2210.88`, 676.007`, 1818.21`, 181.414`, 1738.92`, 
              1399.67`, 2216.38`, 2041.36`, 2544.49`, 2095.5`, 2511.08`, 
              2029.37`, 2103.11`, 2496.88`, 2562.61`, 523.327`, 1758.71`, 
              1336.09`, 2185.26`}, {3, 31, 35.76335365`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7930393923`, 0.4939956657`, 0.54473512`, \(-0.2`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0374759`, 1438, 
              1440.69`, 0.680088`, 1446.63`, 1452.88`, 1.58758`, 400.098`, 
              417.93`, 1.27804`, 533.102`, 539.66`, 0.616414`, 400.098`, 
              411.402`, 0.803707`, 1467.09`, 1469.48`, 0.614914`, 203.488`, 
              239.678`, 1.4052`, 1438, 1440.58`, 0.650944`, 1446.63`, 
              2083.34`, 533.102`, 1939.6`, 203.488`, 1802.29`, 1467.09`, 
              2328.17`, 1759.02`, 2356.16`, 1816.05`, 2311.89`, 1742.5`, 
              1828.46`, 2300.11`, 2368.25`, 400.098`, 1801.86`, 1438, 
              1947.55`}, {3, 18, 36.30581695`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.899928289`, 0.4338763815`, 0.6880479569`, \(-0.2`\), 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.047081`\), 1521.93`, 
              1525.43`, 0.934335`, 1525.89`, 1534.29`, 2.25362`, 392.2`, 
              410.08`, 1.25682`, 526.527`, 533.004`, 0.601249`, 392.2`, 
              403.735`, 0.80442`, 1543.34`, 1547.45`, 1.11175`, 204.218`, 
              239.928`, 1.38965`, 1521.93`, 1525.86`, 1.04767`, 1525.89`, 
              2128.2`, 526.527`, 2039.81`, 204.218`, 1924.37`, 1543.34`, 
              2467.4`, 1870.17`, 2466.95`, 1916.36`, 2428.09`, 1857.61`, 
              1926.33`, 2419.59`, 2475.49`, 392.2`, 1898.7`, 1521.93`, 
              2015.9`}, {3, 29, 36.6049871`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9864421313`, 0.4067884333`, 0.7973431258`, \(-0.2`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0856974`\), 
              1570.29`, 1574.56`, 1.17464`, 1572.16`, 1581.7`, 2.63533`, 
              389.393`, 407.212`, 1.24368`, 521.524`, 528.044`, 0.599567`, 
              389.393`, 401.01`, 0.804475`, 1586.35`, 1591.58`, 1.45599`, 
              204.742`, 240.287`, 1.38599`, 1570.29`, 1575.23`, 1.35982`, 
              1572.16`, 2196.97`, 521.524`, 2117.46`, 204.742`, 2023.18`, 
              1586.35`, 2594.27`, 1976.04`, 2558.72`, 2017.32`, 2521.94`, 
              1965.92`, 2025.52`, 2515.23`, 2565.42`, 389.393`, 2001.83`, 
              1570.29`, 2098.16`}, {3, 33, 36.85548356`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.058788281`, 0.3925204838`, 0.8884413581`, \(-0.2`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.106687`\), 1600.2`, 
              1605.82`, 1.58113`, 1601.68`, 1611.69`, 2.81888`, 387.608`, 
              405.393`, 1.23568`, 517.624`, 524.201`, 0.600395`, 387.608`, 
              399.265`, 0.803657`, 1612.05`, 1618.41`, 1.79922`, 205.164`, 
              240.599`, 1.38397`, 1600.2`, 1606.17`, 1.67971`, 1601.68`, 
              2271.55`, 517.624`, 2171.29`, 205.164`, 2098.8`, 1612.05`, 
              2721.78`, 2074.27`, 2631.77`, 2112.56`, 2595.88`, 2065.74`, 
              2119.54`, 2590.28`, 2637.36`, 387.608`, 2091.58`, 1600.2`, 
              2181.58`}, {3, 41, 37.12415075`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.122490999`, 0.3841540044`, 0.9680356541`, \(-0.2`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.117779`\), 1619.61`, 
              1626.99`, 2.09832`, 1621.34`, 1631.85`, 2.99408`, 386.276`, 
              404.045`, 1.23043`, 514.528`, 521.158`, 0.601701`, 386.276`, 
              397.953`, 0.802372`, 1628.33`, 1635.98`, 2.18829`, 205.529`, 
              240.872`, 1.38235`, 1619.61`, 1626.82`, 2.04999`, 1621.34`, 
              2346.6`, 514.528`, 2204.58`, 205.529`, 2151.33`, 1628.33`, 
              2832.82`, 2165.39`, 2688.24`, 2201.51`, 2652.69`, 2157.98`, 
              2207.6`, 2647.86`, 2693.06`, 386.276`, 2156.49`, 1619.61`, 
              2262.8`}, {3, 37, 37.4541733`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.180390568`, 0.3789000695`, 1.03954113`, \(-0.2`\), 0.6`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.122581`\), 1633.39`, 
              1642.75`, 2.68625`, 1635.55`, 1646.84`, 3.24766`, 385.274`, 
              403.037`, 1.22689`, 512.11`, 518.784`, 0.602889`, 385.274`, 
              396.959`, 0.800851`, 1639.78`, 1648.94`, 2.63947`, 205.844`, 
              241.111`, 1.38105`, 1633.39`, 1642.06`, 2.4866`, 1635.55`, 
              2420.58`, 512.11`, 2221.37`, 205.844`, 2182.83`, 1639.78`, 
              2914.93`, 2250.74`, 2731.18`, 2285.1`, 2695.7`, 2244.16`, 
              2290.52`, 2691.42`, 2735.44`, 385.274`, 2192.85`, 1633.39`, 
              2341.25`}, {2, 21, 37.87841999`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.234046127`, 0.375485674`, 1.104952612`, \(-0.2`\), 0.7`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.123087`\), 1644.39`, 
              1655.89`, 3.32277`, 1647, 1659.43`, 3.60045`, 384.552`, 
              402.314`, 1.22452`, 510.281`, 516.988`, 0.603719`, 384.552`, 
              396.234`, 0.799165`, 1649.04`, 1659.91`, 3.1519`, 206.098`, 
              241.305`, 1.38013`, 1644.39`, 1654.73`, 2.98817`, 1647, 
              2492.94`, 510.281`, 2225.06`, 206.098`, 2196.96`, 1649.04`, 
              2964.89`, 2331.44`, 2763.42`, 2364.28`, 2727.83`, 2325.5`, 
              2369.18`, 2723.99`, 2767.24`, 384.552`, 2207.29`, 1644.39`, 
              2416.95`}, {3, 29, 10314.46893`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              2.249654374`, 0.2005640183`, 1.377059601`, \(-0.2`\), 0.8`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-2.2886`\), 1135.06`, 
              1204.03`, 14.134`, 1203.06`, 1416.11`, 48.8906`, 1283.13`, 
              1238.64`, \(-9.82873`\), 1374.71`, 1367.79`, \(-1.6615`\), 
              1200.95`, 1239.11`, 8.15957`, 1283.13`, 1500.87`, 53.1141`, 
              1137.7`, 1115.37`, \(-4.4089`\), 1154.11`, 1442.13`, 65.5189`, 
              1442.75`, 1481.14`, 1419.65`, 1967.97`, 1154.11`, 1436.08`, 
              1605.17`, 2259.3`, 1137.7`, 1903.13`, 1200.95`, 1854.44`, 
              1135.06`, 1203.06`, 1853.2`, 1904.31`, 1283.13`, 1374.71`, 
              1490.82`, 1358.76`}, {3, 21, 26.90029993`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2369967722`, 0.3767948556`, 
              0.5633678111`, \(-0.1`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.318103`\), 1556.79`, 
              1560.21`, 0.934293`, 1575.71`, 1578.81`, 0.857119`, 441.369`, 
              456.722`, 1.20805`, 640.332`, 645.202`, 0.548624`, 441.369`, 
              453.278`, 0.933444`, 1587.21`, 1593.7`, 1.80826`, 192.48`, 
              229.372`, 1.36358`, 1575.71`, 1579.77`, 1.12295`, 1556.79`, 
              2064.19`, 640.332`, 2088.52`, 192.48`, 2037.89`, 1587.21`, 
              2516.04`, 1880.22`, 2374.19`, 1900.89`, 2360.82`, 1874.62`, 
              1904.53`, 2356.16`, 2379.22`, 441.369`, 1898.01`, 1575.71`, 
              1971.92`}, {3, 21, 27.12894625`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2809259821`, 0.42830032`, 
              0.4628057299`, \(-0.1`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.203501`\), 1504.43`, 
              1506.92`, 0.655969`, 1517.46`, 1520.78`, 0.882981`, 447.85`, 
              462.947`, 1.20477`, 640.327`, 645.106`, 0.538164`, 447.85`, 
              459.156`, 0.898529`, 1540.17`, 1543.17`, 0.810388`, 192.232`, 
              229.519`, 1.37785`, 1517.46`, 1520.6`, 0.834842`, 1504.43`, 
              2095.16`, 640.327`, 2013.6`, 192.232`, 1948.97`, 1540.17`, 
              2458.14`, 1863.72`, 2370.99`, 1898.26`, 2348.84`, 1854.68`, 
              1904.13`, 2341, 2379.79`, 447.85`, 1892.38`, 1517.46`, 
              1990.06`}, {1, 34, 32.93913716`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.592181852`, 0.4493530874`, 0.4347765751`, \(-0.1`\), 0, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0244262`, 1444.84`, 
              1448.33`, 0.882597`, 1452.36`, 1455.89`, 0.89897`, 407.287`, 
              424.915`, 1.28536`, 557.633`, 564.195`, 0.64494`, 407.287`, 
              419.116`, 0.856512`, 1464.76`, 1467.8`, 0.780531`, 203.212`, 
              239.137`, 1.39237`, 1444.84`, 1447.54`, 0.683384`, 1452.36`, 
              1977.34`, 557.633`, 1916.28`, 203.212`, 1794.05`, 1464.76`, 
              2313.43`, 1701.47`, 2275.45`, 1756.59`, 2234.8`, 1685.47`, 
              1768.82`, 2223.13`, 2287.98`, 407.287`, 1741.45`, 1444.84`, 
              1861.35`}, {3, 22, 34.95715651`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7997214319`, 0.402110307`, 0.6201133992`, \(-0.1`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0583885`\), 
              1529.54`, 1533.9`, 1.17069`, 1534.38`, 1541.23`, 1.84724`, 
              397.275`, 415.046`, 1.26478`, 541.415`, 547.741`, 0.603685`, 
              397.275`, 408.84`, 0.816778`, 1549.12`, 1554.22`, 1.38628`, 
              203.2`, 238.873`, 1.38174`, 1529.54`, 1533.61`, 1.09318`, 
              1534.38`, 2025.77`, 541.415`, 2031.84`, 203.2`, 1905, 1549.12`, 
              2416.02`, 1789.76`, 2388.5`, 1835.19`, 2350.94`, 1777.25`, 
              1845.25`, 2342.53`, 2397.01`, 397.275`, 1825.2`, 1529.54`, 
              1924.14`}, {3, 18, 35.60338086`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9111608864`, 0.3804715923`, 0.7452435318`, \(-0.1`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.107562`\), 1581.2`, 
              1586.36`, 1.4309`, 1583.74`, 1592.41`, 2.41262`, 394.575`, 
              412.255`, 1.24987`, 533.908`, 540.265`, 0.598252`, 394.575`, 
              406.136`, 0.811113`, 1596.2`, 1602.6`, 1.79211`, 203.801`, 
              239.414`, 1.38296`, 1581.2`, 1586.55`, 1.48336`, 1583.74`, 
              2107.68`, 533.908`, 2117.81`, 203.801`, 2008.78`, 1596.2`, 
              2525.89`, 1902.79`, 2495.76`, 1943.51`, 2460.01`, 1892.82`, 
              1951.67`, 2453.47`, 2502.3`, 394.575`, 1935.38`, 1581.2`, 
              2016.69`}, {3, 25, 36.06342558`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9955469967`, 0.36990087`, 0.8449390205`, \(-0.1`\), 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.133719`\), 1612.31`, 
              1619.12`, 1.92741`, 1614.5`, 1623.86`, 2.65343`, 392.561`, 
              410.198`, 1.24051`, 528.363`, 534.782`, 0.597916`, 392.561`, 
              404.133`, 0.8078`, 1622.85`, 1630.51`, 2.184`, 204.323`, 
              239.859`, 1.38298`, 1612.31`, 1618.95`, 1.87977`, 1614.5`, 
              2192.33`, 528.363`, 2175.3`, 204.323`, 2091.9`, 1622.85`, 
              2620.39`, 2008.72`, 2582.55`, 2046.63`, 2547.55`, 2000.38`, 
              2053.53`, 2542.14`, 2587.95`, 392.561`, 2039.05`, 1612.31`, 
              2108.87`}, {3, 29, 36.49663584`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.066722691`, 0.3642605793`, 0.9300350116`, \(-0.1`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.14737`\), 1631.82`, 
              1640.65`, 2.5324`, 1634.33`, 1644.27`, 2.85438`, 390.958`, 
              408.579`, 1.23441`, 524.114`, 530.593`, 0.598722`, 390.958`, 
              402.537`, 0.805025`, 1639.05`, 1648.09`, 2.60587`, 204.772`, 
              240.227`, 1.38238`, 1631.82`, 1639.87`, 2.30816`, 1634.33`, 
              2274.63`, 524.114`, 2210.24`, 204.772`, 2154.12`, 1639.05`, 
              2720.52`, 2105.67`, 2649.84`, 2141.57`, 2615.09`, 2098.45`, 
              2147.57`, 2610.43`, 2654.49`, 390.958`, 2133.1`, 1631.82`, 
              2196.59`}, {3, 25, 36.97288006`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.129889073`, 0.3611097449`, 1.005398301`, \(-0.1`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153522`\), 1645.25`, 
              1656.27`, 3.18724`, 1648.24`, 1659.05`, 3.13325`, 389.714`, 
              407.332`, 1.23037`, 520.861`, 527.39`, 0.599665`, 389.714`, 
              401.294`, 0.802534`, 1650.14`, 1660.75`, 3.07912`, 205.141`, 
              240.527`, 1.38173`, 1645.25`, 1654.89`, 2.78811`, 1648.24`, 
              2353.92`, 520.861`, 2227.7`, 205.141`, 2193.64`, 1650.14`, 
              2821.76`, 2195.24`, 2701.03`, 2229.5`, 2666.26`, 2188.84`, 
              2234.83`, 2662.14`, 2705.14`, 389.714`, 2213.45`, 1645.25`, 
              2279.82`}, {2, 33, 37.53573188`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.187540298`, 0.3593384931`, 1.073686218`, \(-0.1`\), 0.6`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.154746`\), 1655.8`, 
              1669.09`, 3.87201`, 1659.25`, 1671.29`, 3.51539`, 388.777`, 
              406.398`, 1.22761`, 518.38`, 524.947`, 0.600365`, 388.777`, 
              400.35`, 0.800145`, 1659.01`, 1671.37`, 3.60751`, 205.451`, 
              240.778`, 1.38116`, 1655.8`, 1667.21`, 3.32331`, 1659.25`, 
              2430.34`, 518.38`, 2231.56`, 205.451`, 2212.09`, 1659.01`, 
              2903.38`, 2279.09`, 2739.56`, 2311.92`, 2704.62`, 2273.3`, 
              2316.72`, 2700.91`, 2743.25`, 388.777`, 2253.09`, 1655.8`, 
              2359.16`}, {2, 31, 6509.813968`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              2.155151168`, 0.1265774601`, 1.332831705`, \(-0.1`\), 0.7`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.91017`\), 1143.27`, 
              1195.6`, 10.7229`, 1206.58`, 1437.89`, 53.5952`, 1230.29`, 
              1214.32`, \(-3.42033`\), 1263.47`, 1268.64`, 1.14687`, 
              1205.65`, 1214.41`, 1.85616`, 1230.29`, 1372.27`, 32.3747`, 
              1144.41`, 1161.21`, 3.39372`, 1207.6`, 1305.78`, 21.6215`, 
              1482.44`, 1361.01`, 1320.94`, 1986.03`, 1207.6`, 1283.5`, 
              1605.1`, 2137.21`, 1144.41`, 1916.37`, 1205.65`, 1869.01`, 
              1143.27`, 1206.58`, 1868.48`, 1916.87`, 1263.47`, 1230.29`, 
              1522.91`, 1283.93`}, {2, 27, 920.2129738`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.963231433`, 0.5901224166`, 1.183398831`, \(-0.1`\), 0.8`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.40785`\), 1511.13`, 
              1520.13`, 2.38862`, 1532.43`, 1577.2`, 12.1984`, 959.792`, 
              952.554`, \(-1.21284`\), 1603.42`, 1618.51`, 4.26068`, 
              959.792`, 946.429`, \(-2.23193`\), 1603.42`, 1605.83`, 
              0.678923`, 0, 0, 1.13906`, 1511.13`, 1497.61`, \(-3.56659`\), 
              1604.47`, 2370.89`, 1688.2`, 2043.14`, 0, 1952.57`, 1681.84`, 
              2548.76`, 1546.36`, 2206.14`, 1606.91`, 2157.51`, 1532.43`, 
              1617.25`, 2148.93`, 2214.28`, 959.792`, 1603.42`, 1511.13`, 
              2019.7`}, {3, 31, 29.5547911`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2164224113`, 0.2733366805`, 0.4974641805`, 0, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.290284`\), 1554.18`, 
              1553.13`, \(-0.286662`\), 1565.73`, 1568.69`, 0.810667`, 
              406.84`, 421.089`, 1.03364`, 580.46`, 585.554`, 0.520409`, 
              406.84`, 418.152`, 0.817684`, 1561.65`, 1570.26`, 2.36358`, 
              199.515`, 230.208`, 1.15563`, 1558.27`, 
              1556.41`, \(-0.508115`\), 1565.73`, 1665.24`, 580.46`, 
              1894.76`, 199.515`, 1692.29`, 1561.65`, 2172.31`, 1558.27`, 
              2075.76`, 1571.99`, 2068.08`, 1554.18`, 1574.71`, 2065.35`, 
              2078.69`, 406.84`, 1570.13`, 1569.02`, 1608.13`}, {3, 22, 
              33.69132999`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3614936416`, 0.3937945667`, 0.4687962874`, 0, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.199894`\), 1557.15`, 
              1562.51`, 1.46408`, 1564.57`, 1568.44`, 1.06239`, 428.608`, 
              447.361`, 1.43925`, 614.708`, 621.963`, 0.786113`, 428.608`, 
              442.486`, 1.05922`, 1563.86`, 1588.32`, 6.75589`, 195.649`, 
              236.169`, 1.53306`, 1564.57`, 1571.13`, 1.80212`, 1557.15`, 
              1961.8`, 614.708`, 2012.81`, 195.649`, 1994.41`, 1563.86`, 
              2420.35`, 1755.04`, 2287.64`, 1789.8`, 2263.32`, 1744.55`, 
              1797.67`, 2255.42`, 2296.18`, 428.608`, 1784.34`, 1564.57`, 
              1860.36`}, {3, 23, 33.32775512`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.662333151`, 0.3754259181`, 0.5267126739`, 0, 0, 0, 0, 10, 11, 
              12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0735598`\), 
              1532.49`, 1536.38`, 1.04747`, 1537.64`, 1543.58`, 1.60267`, 
              405.149`, 422.78`, 1.27898`, 562.795`, 568.902`, 0.605528`, 
              405.149`, 416.696`, 0.831491`, 1547.49`, 1556.19`, 2.367`, 
              201.734`, 237.476`, 1.37542`, 1532.49`, 1536.6`, 1.10755`, 
              1537.64`, 1907.44`, 562.795`, 2008.31`, 201.734`, 1869.27`, 
              1547.49`, 2337.53`, 1689.14`, 2280.15`, 1733.85`, 2244.69`, 
              1676.36`, 1744.21`, 2236.09`, 2289.01`, 405.149`, 1725.74`, 
              1532.49`, 1813.19`}, {3, 17, 34.64549012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8236493275`, 0.3570706905`, 0.6813944266`, 0, 0.1`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130334`\), 1589.18`, 
              1594.49`, 1.47909`, 1592.24`, 1600.51`, 2.31477`, 400.37`, 
              417.919`, 1.25818`, 547.737`, 553.907`, 0.595553`, 400.37`, 
              411.858`, 0.81755`, 1602.76`, 1612.1`, 2.63051`, 202.7`, 
              238.419`, 1.38051`, 1589.18`, 1594.82`, 1.5713`, 1592.24`, 
              2006.34`, 547.737`, 2112.12`, 202.7`, 1968.29`, 1602.76`, 
              2456.64`, 1813.1`, 2413.43`, 1853.41`, 2378.82`, 1803.07`, 
              1861.69`, 2372.31`, 2419.99`, 400.37`, 1846.78`, 1589.18`, 
              1920.9`}, {3, 13, 35.39778694`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9260015919`, 0.3494524559`, 0.794414115`, 0, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.162398`\), 1623.46`, 
              1631.27`, 2.22653`, 1626.32`, 1635.43`, 2.6032`, 397.805`, 
              415.298`, 1.24624`, 539.725`, 545.966`, 0.593646`, 397.805`, 
              409.279`, 0.81142`, 1632, 1642.46`, 3.00068`, 203.397`, 
              239.064`, 1.38268`, 1623.46`, 1630.99`, 2.14595`, 1626.32`, 
              2104.27`, 539.725`, 2176.71`, 203.397`, 2064.27`, 1632, 
              2556.38`, 1931.45`, 2519.73`, 1969.07`, 2485.68`, 1923.17`, 
              1975.97`, 2480.36`, 2525.05`, 397.805`, 1963.24`, 1623.46`, 
              2025.62`}, {3, 22, 36.0448058`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.006951941`, 0.3461627946`, 0.8872249`, 0, 0.3`, 0, 0, 10, 11, 
              12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.178784`\), 1643.81`, 
              1654.28`, 3.02679`, 1647.22`, 1656.92`, 2.80658`, 395.817`, 
              413.294`, 1.23893`, 534.046`, 540.357`, 0.594049`, 395.817`, 
              407.288`, 0.80716`, 1648.79`, 1660.48`, 3.3904`, 203.948`, 
              239.545`, 1.38321`, 1643.81`, 1653.1`, 2.68438`, 1647.22`, 
              2196.15`, 534.046`, 2214.98`, 203.948`, 2146.98`, 1648.79`, 
              2634.94`, 2037.48`, 2601.72`, 2073.21`, 2567.8`, 2030.36`, 
              2079.16`, 2563.25`, 2606.26`, 395.817`, 2067.72`, 1643.81`, 
              2122.64`}, {3, 26, 36.70190351`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.076545289`, 0.3448087063`, 0.9677018873`, 0, 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.186193`\), 1657.08`, 
              1670.12`, 3.79998`, 1661.1`, 1671.7`, 3.09447`, 394.238`, 
              411.715`, 1.23412`, 529.765`, 536.136`, 0.595002`, 394.238`, 
              405.705`, 0.803743`, 1659.79`, 1672.89`, 3.82637`, 204.394`, 
              239.922`, 1.38307`, 1657.08`, 1668.15`, 3.22387`, 1661.1`, 
              2282.3`, 529.765`, 2234.08`, 204.394`, 2212.58`, 1659.79`, 
              2702.73`, 2133.36`, 2663.82`, 2167.57`, 2629.79`, 2127.08`, 
              2172.83`, 2625.79`, 2667.81`, 394.238`, 2161.99`, 1657.08`, 
              2212.54`}, {2, 22, 37.42798211`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.138872013`, 0.3445142349`, 1.039635558`, 0, 0.5`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.188037`\), 1667.18`, 
              1682.68`, 4.55155`, 1671.69`, 1683.58`, 3.49622`, 393.055`, 
              410.539`, 1.231`, 526.577`, 532.994`, 0.59575`, 393.055`, 
              404.513`, 0.800722`, 1668.39`, 1683.11`, 4.32148`, 204.74`, 
              240.214`, 1.38294`, 1667.18`, 1680.11`, 3.79186`, 1671.69`, 
              2363.82`, 526.577`, 2238.74`, 204.74`, 2255.21`, 1668.39`, 
              2782.92`, 2221.67`, 2710.44`, 2254.53`, 2676.18`, 2216, 
              2259.27`, 2672.58`, 2714.03`, 393.055`, 2247.83`, 1667.18`, 
              2296.75`}, {2, 23, 38.26010511`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.196006931`, 0.3446760137`, 1.105268326`, 0, 0.6`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.186004`\), 1676.07`, 
              1693.99`, 5.2925`, 1680.95`, 1694.46`, 3.99637`, 392.078`, 
              409.571`, 1.22868`, 524.015`, 530.47`, 0.596383`, 392.078`, 
              403.526`, 0.797967`, 1676.26`, 1692.76`, 4.87005`, 205.009`, 
              240.433`, 1.38254`, 1676.07`, 1690.96`, 4.39412`, 1680.95`, 
              2441.48`, 524.015`, 2232.24`, 205.009`, 2272.05`, 1676.26`, 
              2861, 2304.33`, 2745.24`, 2335.98`, 2710.66`, 2299.16`, 
              2340.29`, 2707.38`, 2748.49`, 392.078`, 2319.83`, 1676.07`, 
              2376.45`}, {3, 29, 39.2180637`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.24917003`, 0.3450378724`, 1.165998774`, 0, 0.7`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.181092`\), 1684.69`, 
              1704.97`, 6.02548`, 1689.85`, 1705.2`, 4.56792`, 391.278`, 
              408.778`, 1.22671`, 521.897`, 528.382`, 0.59682`, 391.278`, 
              402.71`, 0.795311`, 1684.14`, 1702.53`, 5.45794`, 205.308`, 
              240.67`, 1.38178`, 1684.69`, 1701.62`, 5.02333`, 1689.85`, 
              2515.75`, 521.897`, 2216.98`, 205.308`, 2269.21`, 1684.14`, 
              2920.03`, 2382.55`, 2770.9`, 2413.07`, 2735.95`, 2377.79`, 
              2417.03`, 2732.93`, 2773.87`, 391.278`, 2340.96`, 1684.69`, 
              2452.34`}, {2, 25, 40.30773578`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.299127546`, 0.3457053736`, 1.222725176`, 0, 0.8`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.174178`\), 1693.43`, 
              1716.02`, 6.74908`, 1698.8`, 1716.13`, 5.18402`, 390.688`, 
              408.198`, 1.22561`, 520.306`, 526.814`, 0.597012`, 390.688`, 
              402.103`, 0.792889`, 1692.32`, 1712.67`, 6.07023`, 205.471`, 
              240.804`, 1.38155`, 1693.43`, 1712.43`, 5.66801`, 1698.8`, 
              2587.02`, 520.306`, 2194.72`, 205.471`, 2254.76`, 1692.32`, 
              2949.02`, 2456.98`, 2789.52`, 2486.47`, 2753.94`, 2452.55`, 
              2490.14`, 2751.13`, 2792.04`, 390.688`, 2329.29`, 1693.43`, 
              2524.93`}, {1, 29, 27.34015395`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2553889188`, 0.3242911589`, 0.5001113095`, 0.1`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32315`\), 1574.8`, 
              1573.81`, \(-0.273143`\), 1593.86`, 1596.77`, 0.812592`, 
              434.238`, 450.334`, 1.24747`, 642.405`, 647.352`, 0.558951`, 
              434.238`, 446.253`, 0.926953`, 1594.23`, 1596.72`, 0.695644`, 
              192.577`, 229.1`, 1.34939`, 1580.34`, 1578.44`, \(-0.523934`\), 
              1593.86`, 1739.62`, 642.405`, 1979.4`, 192.577`, 1759.01`, 
              1594.23`, 2228.83`, 1580.34`, 2099.42`, 1599.01`, 2088.72`, 
              1574.8`, 1602.7`, 2084.94`, 2103.46`, 434.238`, 1596.5`, 
              1596.6`, 1659.52`}, {1, 37, 30.19326758`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3772897498`, 0.3349661325`, 0.3796707161`, 0.1`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153285`\), 1498.43`, 
              1497.94`, \(-0.128828`\), 1533.14`, 1535.33`, 0.589464`, 
              419.643`, 435.365`, 1.17778`, 608.955`, 613.587`, 0.49618`, 
              419.643`, 430.284`, 0.792373`, 1529.67`, 1537.35`, 2.06336`, 
              201.376`, 234.249`, 1.25471`, 1508.69`, 
              1506.73`, \(-0.518389`\), 1540.28`, 1688.71`, 608.955`, 
              1932.75`, 201.376`, 1734.14`, 1529.67`, 2192.82`, 1508.69`, 
              2063.73`, 1542.24`, 2043.14`, 1498.43`, 1549.75`, 2036.61`, 
              2070.79`, 419.643`, 1537.08`, 1533.14`, 1605.83`}, {3, 17, 
              34.35651913`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7092883433`, 0.3400988038`, 0.5943723738`, 0.1`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.149105`\), 1592.17`, 
              1592.24`, 0.0190524`, 1593.66`, 1603, 2.61373`, 408.677`, 
              426.163`, 1.27907`, 568.802`, 574.679`, 0.588853`, 408.677`, 
              420.071`, 0.827384`, 1602.27`, 1623.44`, 5.98527`, 199.519`, 
              235.85`, 1.38584`, 1592.17`, 1594.92`, 0.76812`, 1593.66`, 
              1888.18`, 568.802`, 2093.8`, 199.519`, 1889.45`, 1602.27`, 
              2362.3`, 1698.26`, 2299.44`, 1738.46`, 2266.31`, 1687.7`, 
              1747.22`, 2259.48`, 2306.38`, 408.677`, 1732.44`, 1592.17`, 
              1802.51`}, {3, 13, 35.06524039`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8458646265`, 0.3316392679`, 0.7327823772`, 0.1`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.190092`\), 1634.82`, 
              1640.61`, 1.65944`, 1636.64`, 1646.49`, 2.83127`, 403.737`, 
              421.108`, 1.25536`, 553.098`, 559.122`, 0.587094`, 403.737`, 
              415.109`, 0.815857`, 1638.35`, 1656.16`, 5.14103`, 201.945`, 
              237.874`, 1.38454`, 1634.82`, 1641.96`, 2.04992`, 1636.64`, 
              2002.47`, 553.098`, 2174.27`, 201.945`, 1998.24`, 1638.35`, 
              2484.26`, 1837.41`, 2437.4`, 1874.86`, 2404.38`, 1828.99`, 
              1881.92`, 2399.02`, 2442.77`, 403.737`, 1869.68`, 1634.82`, 
              1925.14`}, {2, 9, 35.90865319`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9413470946`, 0.330555421`, 0.8373388692`, 0.1`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.210734`\), 1657.16`, 
              1668.12`, 3.19257`, 1660.81`, 1670.83`, 2.92684`, 401.573`, 
              418.901`, 1.24566`, 545.897`, 551.981`, 0.585254`, 401.573`, 
              412.902`, 0.808438`, 1657.21`, 1674.61`, 5.08189`, 202.815`, 
              238.662`, 1.38655`, 1657.16`, 1667.55`, 3.02509`, 1660.81`, 
              2107.69`, 545.897`, 2218.97`, 202.815`, 2098.24`, 1657.21`, 
              2578.25`, 1957.49`, 2540.04`, 1993.15`, 2506.99`, 1950.34`, 
              1999.16`, 2502.47`, 2544.56`, 401.573`, 1988.44`, 1657.16`, 
              2036.41`}, {2, 17, 39.27659015`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.01861345`, 0.3574699201`, 0.9227201455`, 0.1`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.244031`\), 1664.8`, 
              1680.39`, 4.56974`, 1671.53`, 1681.23`, 2.84985`, 365.684`, 
              386.216`, 1.35258`, 535.551`, 541.935`, 0.60273`, 365.684`, 
              378.739`, 0.851497`, 1662.92`, 1680.84`, 5.25119`, 0, 127.434`, 
              1.51048`, 1664.8`, 1677.39`, 3.68897`, 1671.53`, 2223.63`, 
              535.551`, 2240.11`, 0, 2197.89`, 1662.92`, 2658.3`, 2062.09`, 
              2616.95`, 2097.18`, 2583.26`, 2055.1`, 2103.02`, 2578.79`, 
              2621.41`, 365.684`, 2092.85`, 1664.8`, 2146.94`}, {3, 22, 
              37.6955966`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.087347015`, 0.3313691682`, 1.001928556`, 0.1`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.222526`\), 1679.39`, 
              1697.17`, 5.25976`, 1685.16`, 1697.38`, 3.62096`, 397.783`, 
              415.136`, 1.23595`, 535.683`, 541.921`, 0.58899`, 397.783`, 
              409.114`, 0.801056`, 1677.19`, 1695.9`, 5.52814`, 203.822`, 
              239.527`, 1.38694`, 1679.39`, 1694.32`, 4.41281`, 1685.16`, 
              2291.86`, 535.683`, 2247.28`, 203.822`, 2258.41`, 1677.19`, 
              2701.41`, 2157.97`, 2674.55`, 2190.92`, 2640.99`, 2152.36`, 
              2195.64`, 2637.46`, 2678.07`, 397.783`, 2186.51`, 1679.39`, 
              2228}, {2, 21, 38.6709909`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.148758051`, 0.3326931933`, 1.071030765`, 0.1`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220937`\), 1687.48`, 
              1707.37`, 5.91503`, 1693.69`, 1708.13`, 4.30364`, 396.66`, 
              414.025`, 1.23339`, 532.623`, 538.903`, 0.58957`, 396.66`, 
              407.974`, 0.797619`, 1684.87`, 1704.87`, 5.94052`, 204.138`, 
              239.796`, 1.38695`, 1687.48`, 1704.44`, 5.04047`, 1693.69`, 
              2374.71`, 532.623`, 2241.55`, 204.138`, 2313.66`, 1684.87`, 
              2745.18`, 2245.15`, 2717.17`, 2276.94`, 2683.24`, 2240.04`, 
              2281.23`, 2680.02`, 2720.37`, 396.66`, 2272.26`, 1687.48`, 
              2312.84`}, {2, 21, 39.76834356`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.205268141`, 0.334008485`, 1.134415862`, 0.1`, 0.6`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.216243`\), 1695.43`, 
              1717.48`, 6.59172`, 1701.9`, 1718.54`, 4.98749`, 395.694`, 
              413.07`, 1.23126`, 530.075`, 536.389`, 0.590067`, 395.694`, 
              406.991`, 0.794485`, 1692.54`, 1714.09`, 6.4323`, 204.419`, 
              240.023`, 1.38646`, 1695.43`, 1714.43`, 5.67554`, 1701.9`, 
              2453.09`, 530.075`, 2226.65`, 204.419`, 2342.81`, 1692.54`, 
              2804.28`, 2326.81`, 2748.74`, 2357.51`, 2714.4`, 2322.1`, 
              2361.45`, 2711.44`, 2751.67`, 395.694`, 2351.19`, 1695.43`, 
              2392.69`}, {2, 25, 40.99605593`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.257958996`, 0.3352460068`, 1.193307446`, 0.1`, 0.7`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.209198`\), 1703.62`, 
              1727.85`, 7.28267`, 1710.22`, 1729.04`, 5.67166`, 394.821`, 
              412.208`, 1.22936`, 527.867`, 534.213`, 0.590495`, 394.821`, 
              406.102`, 0.791583`, 1700.52`, 1723.77`, 6.9747`, 204.651`, 
              240.2`, 1.38557`, 1703.62`, 1724.66`, 6.31902`, 1710.22`, 
              2527.68`, 527.867`, 2204.5`, 204.651`, 2346.19`, 1700.52`, 
              2863.49`, 2404.09`, 2771.82`, 2433.78`, 2737, 2399.73`, 
              2437.42`, 2734.26`, 2774.48`, 394.821`, 2407.62`, 1703.62`, 
              2468.48`}, {3, 29, 42.35026795`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.307530086`, 0.3363974736`, 1.248546612`, 0.1`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.200315`\), 1712.13`, 
              1738.52`, 7.97879`, 1718.8`, 1739.76`, 6.35031`, 394.05`, 
              411.448`, 1.22786`, 525.935`, 532.31`, 0.591103`, 394.05`, 
              405.317`, 0.789101`, 1708.86`, 1733.88`, 7.54625`, 204.909`, 
              240.397`, 1.38461`, 1712.13`, 1735.19`, 6.96532`, 1718.8`, 
              2598.92`, 525.935`, 2176.44`, 204.909`, 2333.88`, 1708.86`, 
              2903.63`, 2477.72`, 2789.28`, 2506.48`, 2752.81`, 2473.65`, 
              2509.87`, 2750.24`, 2790.55`, 394.05`, 2408.25`, 1712.13`, 
              2540.77`}, {1, 41, 27.4682556`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2657558288`, 0.2998356025`, 0.4825019412`, 0.2`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322083`\), 1453.86`, 
              1454.35`, 0.126353`, 1480.55`, 1480.7`, 0.0387929`, 427.916`, 
              443.984`, 1.22745`, 632.116`, 637.141`, 0.558782`, 427.916`, 
              439.82`, 0.905007`, 1474.56`, 1474.94`, 0.0977736`, 194.144`, 
              229.65`, 1.31839`, 1459.27`, 1459.69`, 0.10811`, 1531.77`, 
              1669.98`, 632.116`, 1920.94`, 194.144`, 1632.45`, 1588.43`, 
              2116.17`, 1459.27`, 1994.86`, 1476.85`, 1985.14`, 1453.86`, 
              1480.55`, 1981.77`, 1998.46`, 427.916`, 1474.56`, 1504.18`, 
              1622.28`}, {3, 21, 31.48306342`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4944828822`, 0.3098505885`, 0.4578206094`, 0.2`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.17615`\), 1524.79`, 
              1521.76`, \(-0.809379`\), 1552.22`, 1565.5`, 3.62728`, 
              412.473`, 429.524`, 1.25784`, 586.179`, 591.946`, 0.595308`, 
              412.473`, 423.965`, 0.842201`, 1566.14`, 1566.56`, 0.117381`, 
              200.693`, 235.611`, 1.33484`, 1535.24`, 1529, \(-1.67718`\), 
              1562.86`, 1721.77`, 586.179`, 2053.64`, 200.693`, 1743.15`, 
              1580.01`, 2209.45`, 1535.24`, 2115.84`, 1571.07`, 2088.89`, 
              1524.79`, 1579.65`, 2082.21`, 2122.78`, 412.473`, 1566.14`, 
              1552.22`, 1650.85`}, {1, 45, 38.98858574`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7440251912`, 0.301119644`, 0.6590543042`, 0.2`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.213417`\), 1638.73`, 
              1636.5`, \(-0.639779`\), 1643.25`, 1650.24`, 2.015`, 395.996`, 
              413.533`, 1.24384`, 546.096`, 552.51`, 0.617351`, 395.996`, 
              407.86`, 0.835581`, 1637.47`, 1681.21`, 12.7182`, 195.704`, 
              231.915`, 1.3567`, 1643.25`, 1641.75`, \(-0.433815`\), 
              1638.73`, 1872.55`, 546.096`, 2163.89`, 195.704`, 1891.3`, 
              1637.47`, 2385.08`, 1724.69`, 2330.85`, 1760.97`, 2299.78`, 
              1716.45`, 1767.94`, 2294.63`, 2336.05`, 395.996`, 1756.18`, 
              1643.25`, 1796.39`}, {3, 13, 37.13181086`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8656610654`, 0.313727018`, 0.7785515806`, 0.2`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.240166`\), 1675.05`, 
              1680.04`, 1.4663`, 1675.85`, 1686.39`, 3.10694`, 404.749`, 
              422.043`, 1.25278`, 554.556`, 560.551`, 0.585694`, 404.749`, 
              416.107`, 0.816841`, 1663.08`, 1695.25`, 9.46654`, 200.447`, 
              236.671`, 1.38734`, 1675.85`, 1683.06`, 2.12399`, 1675.05`, 
              2002.56`, 554.556`, 2222.29`, 200.447`, 2020.24`, 1663.08`, 
              2507.69`, 1863.42`, 2461.11`, 1898.9`, 2429.13`, 1856.18`, 
              1905.02`, 2424.6`, 2465.65`, 404.749`, 1894.57`, 1675.85`, 
              1929.12`}, {3, 13, 37.56112653`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9564317894`, 0.319086696`, 0.875298127`, 0.2`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253539`\), 1688.2`, 
              1701.51`, 3.95191`, 1692.05`, 1703.31`, 3.34981`, 405.994`, 
              423.189`, 1.24919`, 553.26`, 559.147`, 0.573767`, 405.994`, 
              417.145`, 0.804213`, 1676.5`, 1704.35`, 8.24719`, 201.734`, 
              237.954`, 1.39532`, 1688.2`, 1701.18`, 3.85247`, 1692.05`, 
              2113.12`, 553.26`, 2250.15`, 201.734`, 2124.38`, 1676.5`, 
              2596.09`, 1981.17`, 2557.15`, 2015.49`, 2524.72`, 1974.79`, 
              2020.89`, 2520.73`, 2561.14`, 405.994`, 2011.48`, 1688.2`, 
              2045.64`}, {2, 17, 40.90177432`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.031956868`, 0.345982102`, 0.9565258241`, 0.2`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.27987`\), 1689.22`, 
              1708.21`, 5.6512`, 1697.14`, 1708.9`, 3.50824`, 371.253`, 
              391.414`, 1.34714`, 543.612`, 549.69`, 0.582221`, 371.253`, 
              383.92`, 0.838088`, 1678.84`, 1704.88`, 7.71925`, 0, 127.802`, 
              1.50982`, 1689.22`, 1705.41`, 4.81532`, 1697.14`, 2228.77`, 
              543.612`, 2256.9`, 0, 2227.42`, 1678.84`, 2669.18`, 2083.46`, 
              2628.21`, 2117.35`, 2594.99`, 2077.13`, 2122.66`, 2590.99`, 
              2632.21`, 371.253`, 2113.63`, 1689.22`, 2156.85`}, {2, 21, 
              39.68412203`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.098707422`, 0.3241428879`, 1.032397912`, 0.2`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.257549`\), 1701.07`, 
              1722.6`, 6.45957`, 1708.74`, 1724.31`, 4.6834`, 403.399`, 
              420.612`, 1.24277`, 545.145`, 551.131`, 0.574894`, 403.399`, 
              414.489`, 0.794719`, 1693.08`, 1718.98`, 7.74163`, 202.736`, 
              238.873`, 1.39819`, 1701.07`, 1720.32`, 5.77293`, 1708.74`, 
              2302.14`, 545.145`, 2254.14`, 202.736`, 2291.58`, 1693.08`, 
              2708.18`, 2178.83`, 2682.07`, 2210.88`, 2648.78`, 2173.67`, 
              2215.24`, 2645.56`, 2685.28`, 403.399`, 2207.18`, 1701.07`, 
              2241.8`}, {2, 21, 40.91378519`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.159180447`, 0.3259189599`, 1.099337603`, 0.2`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253157`\), 1707.5`, 
              1731.47`, 7.22373`, 1715.62`, 1733.7`, 5.46515`, 401.935`, 
              419.166`, 1.23964`, 541.501`, 547.544`, 0.576623`, 401.935`, 
              413.017`, 0.791322`, 1700.55`, 1727, 7.94431`, 203.106`, 
              239.157`, 1.39696`, 1707.5`, 1728.97`, 6.46666`, 1715.62`, 
              2385.87`, 541.501`, 2239.64`, 203.106`, 2355.91`, 1700.55`, 
              2742.29`, 2265.27`, 2721.38`, 2296.26`, 2687.63`, 2260.54`, 
              2300.24`, 2684.68`, 2724.31`, 401.935`, 2292.51`, 1707.5`, 
              2327.34`}, {3, 29, 42.26779926`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.215004496`, 0.3274650436`, 1.16099729`, 0.2`, 0.6`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.246128`\), 1714.62`, 
              1740.86`, 7.94501`, 1722.86`, 1743.31`, 6.20947`, 400.579`, 
              417.827`, 1.23681`, 538.292`, 544.388`, 0.578307`, 400.579`, 
              411.657`, 0.788348`, 1708.33`, 1735.81`, 8.29409`, 203.438`, 
              239.396`, 1.39516`, 1714.62`, 1738.16`, 7.12161`, 1722.86`, 
              2464.68`, 538.292`, 2217.61`, 203.438`, 2399.54`, 1708.33`, 
              2766.71`, 2346.31`, 2750.3`, 2376.29`, 2716.08`, 2341.94`, 
              2379.96`, 2713.35`, 2752.99`, 400.579`, 2372.07`, 1714.62`, 
              2407.45`}, {2, 26, 955.0632337`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              2.294605777`, 0.3760300953`, 1.352323284`, 0.2`, 0.7`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-2.72793`\), 776.956`, 
              880.101`, 14.9751`, 891.801`, 1069.49`, 30.534`, 882.577`, 
              911.752`, 4.58666`, 934.129`, 917.126`, \(-2.75781`\), 
              882.577`, 884.015`, 0.222551`, 883.409`, 935.662`, 8.32802`, 0, 
              0, 1.07934`, 788.509`, 970.025`, 27.9674`, 1107.88`, 1961, 
              1718.4`, 1756.21`, 0, 1510.17`, 1534.53`, 2167.4`, 788.509`, 
              1710.88`, 883.409`, 1654.99`, 776.956`, 891.801`, 1650.62`, 
              1714.94`, 934.129`, 882.577`, 1032.64`, 1613.63`}, {2, 25, 
              45.3334248`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.316280618`, 0.3301870161`, 1.272450343`, 0.2`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.22658`\), 1730.56`, 
              1761.07`, 9.33282`, 1738.69`, 1763.53`, 7.62185`, 398.27`, 
              415.553`, 1.23237`, 533.05`, 539.237`, 0.581302`, 398.27`, 
              409.341`, 0.783402`, 1724.97`, 1755.2`, 9.21863`, 203.97`, 
              239.758`, 1.39135`, 1730.56`, 1758.01`, 8.391`, 1738.69`, 
              2610.6`, 533.05`, 2156.43`, 203.97`, 2408.13`, 1724.97`, 
              2841.11`, 2496.05`, 2789.79`, 2524.21`, 2750.4`, 2492.24`, 
              2527.39`, 2748.01`, 2787.92`, 398.27`, 2474.17`, 1730.56`, 
              2555.34`}, {3, 25, 27.75147624`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2833601926`, 0.2923635321`, 0.4757701638`, 0.3`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322784`\), 1392.08`, 
              1392.89`, 0.19758`, 1419.77`, 1420.06`, 0.0716418`, 426.48`, 
              442.696`, 1.23488`, 630.616`, 635.646`, 0.558007`, 426.48`, 
              438.331`, 0.897925`, 1413.52`, 1413.92`, 0.0985795`, 194.411`, 
              229.89`, 1.31895`, 1397.75`, 1398.61`, 0.212263`, 1484.98`, 
              1658.86`, 630.616`, 1903.66`, 194.411`, 1574.88`, 1588.16`, 
              2067.97`, 1397.75`, 1947.18`, 1415.79`, 1937.17`, 1392.08`, 
              1419.77`, 1933.8`, 1950.77`, 426.48`, 1413.52`, 1448.76`, 
              1618.16`}, {1, 13, 31.8843615`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.5700835497`, 0.300597456`, 0.5161360226`, 0.3`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.215876`\), 1531.83`, 
              1528.88`, \(-0.788932`\), 1562.79`, 1567.93`, 1.40984`, 
              418.554`, 435.502`, 1.26822`, 593.409`, 598.797`, 0.562744`, 
              418.554`, 429.679`, 0.82681`, 1573.29`, 1573.87`, 0.160339`, 
              200.627`, 236.123`, 1.35833`, 1541.62`, 
              1535.44`, \(-1.66608`\), 1578.69`, 1746.16`, 593.409`, 2110.1`, 
              200.627`, 1744.85`, 1617.67`, 2225.45`, 1541.62`, 2136.1`, 
              1577.86`, 2107.94`, 1531.83`, 1586.05`, 2101.82`, 2142.37`, 
              418.554`, 1573.29`, 1562.79`, 1681.05`}, {1, 13, 
              48.03922786`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7955479622`, 0.3506006803`, 0.6553376931`, 0.3`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.243213`\), 1638.08`, 
              1622.74`, \(-4.38392`\), 1645.72`, 1653.01`, 2.10625`, 
              470.939`, 485.244`, 1.19845`, 671.484`, 672.969`, 0.174984`, 
              470.939`, 478.085`, 0.594224`, 1659.81`, 1701.07`, 12.1505`, 
              216.373`, 249.617`, 1.35733`, 1638.08`, 
              1625.29`, \(-3.65826`\), 1645.72`, 1889.92`, 671.484`, 
              2181.07`, 216.373`, 1903.7`, 1659.81`, 2373.71`, 1663.38`, 
              2282.06`, 1704.48`, 2248.09`, 1653.11`, 1712.98`, 2241.61`, 
              2288.57`, 470.939`, 1699.84`, 1638.08`, 1792.85`}, {3, 17, 
              42.31020629`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8831603785`, 0.3008897993`, 0.8207145193`, 0.3`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.283686`\), 1711.8`, 
              1716.58`, 1.43405`, 1716.72`, 1723.77`, 2.12397`, 403.503`, 
              420.969`, 1.26174`, 553.436`, 559.578`, 0.59899`, 403.503`, 
              415.07`, 0.829581`, 1682.11`, 1733.55`, 15.3941`, 196.358`, 
              233.522`, 1.39978`, 1716.72`, 1721.49`, 1.43754`, 1711.8`, 
              2005.35`, 553.436`, 2262.03`, 196.358`, 2042.86`, 1682.11`, 
              2530.21`, 1891.81`, 2485.19`, 1925.78`, 2453.95`, 1885.46`, 
              1931.18`, 2450.02`, 2489.12`, 403.503`, 1922.03`, 1716.72`, 
              1932.57`}, {1, 13, 40.8579623`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.970863348`, 0.3105120777`, 0.9102411781`, 0.3`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291367`\), 1719.79`, 
              1733.38`, 4.11201`, 1722.49`, 1734.85`, 3.74446`, 409.101`, 
              426.25`, 1.25518`, 558.746`, 564.512`, 0.567482`, 409.101`, 
              420.16`, 0.803537`, 1692.55`, 1734.16`, 12.4935`, 199.833`, 
              236.597`, 1.40579`, 1719.79`, 1733.89`, 4.26556`, 1722.49`, 
              2118.06`, 558.746`, 2275.01`, 199.833`, 2147.26`, 1692.55`, 
              2611.48`, 2004.14`, 2572.5`, 2037.4`, 2540.51`, 1998.36`, 
              2042.3`, 2536.92`, 2576.09`, 409.101`, 2033.88`, 1719.79`, 
              2052.16`}, {3, 25, 41.62540242`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.044734325`, 0.3177427251`, 0.9885495371`, 0.3`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.293805`\), 1720.77`, 
              1741.88`, 6.40442`, 1728.19`, 1743.4`, 4.62613`, 411.457`, 
              428.521`, 1.25586`, 560.438`, 566.057`, 0.554557`, 411.457`, 
              422.286`, 0.791064`, 1700.7`, 1737.43`, 11.0654`, 200.667`, 
              237.503`, 1.41419`, 1720.77`, 1740.61`, 6.01688`, 1728.19`, 
              2219.88`, 560.438`, 2272.21`, 200.667`, 2238.36`, 1700.7`, 
              2671.54`, 2104.34`, 2637.95`, 2136.74`, 2605.33`, 2099.03`, 
              2141.24`, 2602.03`, 2641.24`, 411.457`, 2133.4`, 1720.77`, 
              2158.32`}, {3, 25, 42.91619558`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.110175145`, 0.3213571294`, 1.059613598`, 0.3`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291147`\), 1723.42`, 
              1748.65`, 7.67609`, 1732.91`, 1751.58`, 5.69913`, 410.752`, 
              427.817`, 1.25381`, 557.661`, 563.298`, 0.553689`, 410.752`, 
              421.522`, 0.785334`, 1708.16`, 1742.79`, 10.4703`, 201.089`, 
              237.896`, 1.41567`, 1723.42`, 1746.9`, 7.14034`, 1732.91`, 
              2311.82`, 557.661`, 2258.11`, 201.089`, 2318.14`, 1708.16`, 
              2715.14`, 2196.55`, 2687.11`, 2227.95`, 2653.96`, 2191.67`, 
              2232.07`, 2650.93`, 2690.12`, 410.752`, 2224.73`, 1723.42`, 
              2253.17`}, {2, 21, 44.37813734`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.169760418`, 0.3231703192`, 1.124830502`, 0.3`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.284699`\), 1727.98`, 
              1756.24`, 8.62789`, 1738.14`, 1759.76`, 6.62652`, 408.871`, 
              425.959`, 1.24988`, 553.276`, 558.992`, 0.557029`, 408.871`, 
              419.646`, 0.782113`, 1715.75`, 1749.79`, 10.3334`, 201.537`, 
              238.21`, 1.41297`, 1727.98`, 1754.1`, 7.97091`, 1738.14`, 
              2396.49`, 553.276`, 2235.81`, 201.537`, 2387.24`, 1715.75`, 
              2746.36`, 2282.45`, 2723.61`, 2312.84`, 2689.95`, 2277.96`, 
              2316.62`, 2687.17`, 2726.37`, 408.871`, 2309.66`, 1727.98`, 
              2339.85`}, {3, 33, 45.95357884`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.224909035`, 0.3243675356`, 1.185201106`, 0.3`, 0.6`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.275684`\), 1734.03`, 
              1764.77`, 9.42192`, 1744.25`, 1768.48`, 7.45881`, 406.819`, 
              423.933`, 1.24569`, 548.873`, 554.675`, 0.561023`, 406.819`, 
              417.614`, 0.779729`, 1723.73`, 1758, 10.4558`, 201.963`, 
              238.475`, 1.40896`, 1734.03`, 1762.34`, 8.67156`, 1744.25`, 
              2475.73`, 548.873`, 2207.3`, 201.963`, 2441.4`, 1723.73`, 
              2768.07`, 2363.19`, 2750.33`, 2392.61`, 2716.16`, 2359.04`, 
              2396.09`, 2713.58`, 2752.86`, 406.819`, 2389.28`, 1734.03`, 
              2420.53`}, {2, 19, 1039.911237`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              2.123685986`, 0.4422411571`, 1.275037224`, 0.3`, 0.7`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.99019`\), 1188.96`, 
              1258.51`, 14.9153`, 1262.13`, 1275.95`, 3.07288`, 947.806`, 
              928.108`, \(-3.2377`\), 1264.98`, 1286.67`, 4.8492`, 947.806`, 
              935.687`, \(-1.99997`\), 1264.98`, 1266.64`, 0.368271`, 0, 0, 
              0.355671`, 1199.71`, 1246.6`, 10.0497`, 1345.24`, 2109.06`, 
              1719.56`, 1858.42`, 0, 1768.95`, 1573.13`, 2306.7`, 1199.71`, 
              1959.26`, 1266.95`, 1909.19`, 1188.96`, 1275.18`, 1903.67`, 
              1964.46`, 947.806`, 1264.98`, 1262.13`, 1772.78`}, {2, 25, 
              49.40811951`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.325183076`, 0.3264055309`, 1.294644096`, 0.3`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.252723`\), 1748.78`, 
              1783.71`, 10.8115`, 1758.55`, 1787.33`, 8.9423`, 403.239`, 
              420.407`, 1.23891`, 541.501`, 547.455`, 0.568011`, 403.239`, 
              414.076`, 0.776029`, 1740.57`, 1776.56`, 11.0905`, 202.627`, 
              238.854`, 1.40131`, 1748.78`, 1780.87`, 9.92309`, 1758.55`, 
              2621.74`, 541.501`, 2135.99`, 202.627`, 2472.92`, 1740.57`, 
              2788.24`, 2512.34`, 2789.47`, 2540.02`, 2747.02`, 2508.73`, 
              2543.04`, 2744.76`, 2784.41`, 403.239`, 2523.78`, 1748.78`, 
              2568.59`}, {3, 25, 28.08116804`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3034090893`, 0.28707858`, 0.4733899052`, 0.4`, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322789`\), 1353.32`, 
              1354.33`, 0.24007`, 1382.54`, 1382.91`, 0.0899294`, 425.268`, 
              441.589`, 1.23958`, 628.302`, 633.331`, 0.555872`, 425.268`, 
              437.046`, 0.88987`, 1375.93`, 1376.32`, 0.0938579`, 194.748`, 
              230.199`, 1.31993`, 1359.31`, 1360.43`, 0.266683`, 1452.35`, 
              1658.62`, 628.302`, 1899.53`, 194.748`, 1540.83`, 1590.94`, 
              2041.8`, 1359.31`, 1921.62`, 1378.21`, 1910.91`, 1353.32`, 
              1382.54`, 1907.45`, 1925.27`, 425.268`, 1375.93`, 1412.49`, 
              1620.64`}, {3, 13, 32.33289016`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6254649209`, 0.2875036834`, 0.5762483553`, 0.4`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.266856`\), 1554.59`, 
              1551.31`, \(-0.891103`\), 1584.38`, 1588.18`, 1.05663`, 
              418.987`, 435.857`, 1.2635`, 590.424`, 595.77`, 0.555531`, 
              418.987`, 430.067`, 0.824187`, 1594.36`, 1595.05`, 0.193192`, 
              200.721`, 236.438`, 1.36804`, 1563.22`, 
              1556.68`, \(-1.78798`\), 1601.4`, 1766.71`, 590.424`, 2150.6`, 
              200.721`, 1755.21`, 1648.58`, 2253.34`, 1563.22`, 2170.45`, 
              1598.39`, 2142.23`, 1554.59`, 1605.7`, 2136.94`, 2175.83`, 
              418.987`, 1594.36`, 1584.38`, 1706.51`}, {2, 22, 
              49.62641708`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.813649419`, 0.3375731097`, 0.6955302414`, 0.4`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291281`\), 1668.07`, 
              1647.42`, \(-6.00039`\), 1674.62`, 1679.86`, 1.53976`, 
              472.278`, 486.43`, 1.18869`, 672.243`, 673.648`, 0.165672`, 
              472.278`, 479.359`, 0.590407`, 1679.44`, 1720.06`, 12.0997`, 
              215.116`, 248.58`, 1.35957`, 1668.07`, 1650.42`, \(-5.13327`\), 
              1674.88`, 1890.8`, 672.243`, 2221.11`, 215.116`, 1916.07`, 
              1679.44`, 2394.93`, 1683.81`, 2304.24`, 1722.96`, 2271.33`, 
              1674.62`, 1730.64`, 2265.6`, 2309.99`, 472.278`, 1718.93`, 
              1668.07`, 1797.5`}, {2, 17, 51.82457178`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8991900438`, 0.2885952892`, 0.8619003253`, 0.4`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321798`\), 1748.16`, 
              1751.52`, 1.03194`, 1757.78`, 1759.38`, 0.494417`, 398.062`, 
              415.993`, 1.27895`, 546.184`, 552.78`, 0.635114`, 398.062`, 
              410.178`, 0.857996`, 1698.12`, 1770.65`, 22.0417`, 190.275`, 
              228.862`, 1.41705`, 1757.78`, 1758.13`, 0.107751`, 1748.16`, 
              2009.24`, 546.184`, 2296.62`, 190.275`, 2066.47`, 1698.12`, 
              2552.22`, 1923.88`, 2510.62`, 1956.5`, 2479.98`, 1918.33`, 
              1961.24`, 2476.58`, 2514.03`, 398.062`, 1953.17`, 1757.78`, 
              1936.67`}, {3, 17, 46.34383585`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9845525212`, 0.3006370503`, 0.9445873029`, 0.4`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.324404`\), 1752.69`, 
              1764.34`, 3.59052`, 1752.91`, 1765.83`, 3.98335`, 407.835`, 
              425.126`, 1.26194`, 557.358`, 563.265`, 0.579995`, 407.835`, 
              419.069`, 0.813955`, 1706.79`, 1764.54`, 17.5644`, 196.895`, 
              234.314`, 1.41375`, 1752.69`, 1766.01`, 4.10427`, 1752.91`, 
              2121.95`, 557.358`, 2296.35`, 196.895`, 2168.19`, 1706.79`, 
              2625.27`, 2029.1`, 2588.05`, 2061.33`, 2556.44`, 2023.92`, 
              2065.76`, 2553.24`, 2591.24`, 407.835`, 2058.19`, 1752.69`, 
              2057.25`}, {3, 25, 46.07831181`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.057152617`, 0.3147466155`, 1.016376483`, 0.4`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.326163`\), 1748.13`, 
              1770.23`, 6.81244`, 1755.42`, 1772.23`, 5.19642`, 417.315`, 
              434.28`, 1.26586`, 570.619`, 575.965`, 0.537098`, 417.315`, 
              427.905`, 0.784271`, 1714.87`, 1764.1`, 15.0042`, 198.624`, 
              236.166`, 1.43017`, 1748.13`, 1769.78`, 6.67371`, 1755.42`, 
              2226.04`, 570.619`, 2283.89`, 198.624`, 2259.95`, 1714.87`, 
              2680.48`, 2122.24`, 2645.35`, 2153.99`, 2612.89`, 2117.24`, 
              2158.23`, 2609.8`, 2648.43`, 417.315`, 2151, 1748.13`, 
              2164.69`}, {3, 25, 47.43045239`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.12151056`, 0.3214840955`, 1.084121927`, 0.4`, 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323202`\), 1747.24`, 
              1775.4`, 8.68971`, 1758.13`, 1779.22`, 6.53828`, 419.316`, 
              436.227`, 1.26763`, 572.308`, 577.537`, 0.526738`, 419.316`, 
              429.713`, 0.773427`, 1722.68`, 1767.46`, 13.695`, 199.009`, 
              236.663`, 1.43736`, 1747.24`, 1774.17`, 8.3085`, 1758.13`, 
              2320.08`, 572.308`, 2261.77`, 199.009`, 2341.13`, 1722.68`, 
              2721.25`, 2211.73`, 2690.27`, 2242.65`, 2657.18`, 2207.04`, 
              2246.61`, 2654.28`, 2693.16`, 419.316`, 2239.79`, 1747.24`, 
              2261.62`}, {3, 25, 49.13792663`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.180272258`, 0.3238361632`, 1.147679154`, 0.4`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.315739`\), 1749.48`, 
              1781.73`, 9.97865`, 1761.68`, 1786.36`, 7.6743`, 417.594`, 
              434.526`, 1.2641`, 568.155`, 573.454`, 0.530015`, 417.594`, 
              427.983`, 0.769643`, 1730.58`, 1773.26`, 13.1033`, 199.413`, 
              236.947`, 1.435`, 1749.48`, 1779.92`, 9.41371`, 1761.68`, 
              2406.02`, 568.155`, 2232.34`, 199.413`, 2412.67`, 1730.58`, 
              2750.27`, 2296.86`, 2724.21`, 2326.82`, 2690.58`, 2292.53`, 
              2330.48`, 2687.89`, 2726.87`, 417.594`, 2324.03`, 1749.48`, 
              2349.92`}, {2, 23, 1032.547601`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.849397603`, 0.4910092149`, 1.137054836`, 0.4`, 0.6`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.32008`\), 1438.11`, 
              1453.48`, 3.89506`, 1463.39`, 1498.32`, 9.06204`, 941.857`, 
              931.745`, \(-1.65996`\), 1506.4`, 1523.45`, 4.5265`, 941.857`, 
              930.459`, \(-1.86981`\), 1506.4`, 1508.15`, 0.461417`, 
              243.214`, 258.934`, 0.691657`, 1450, 1434.22`, \(-3.98754`\), 
              1548.83`, 2231.99`, 1639.24`, 2025.04`, 243.214`, 1988.18`, 
              1666.32`, 2472.55`, 1450, 2142.73`, 1508.99`, 2095.78`, 
              1438.11`, 1518.12`, 2088.89`, 2149.29`, 941.857`, 1506.4`, 
              1463.39`, 1905}, {3, 41, 52.78657333`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.285872112`, 0.3248004849`, 1.262690004`, 0.4`, 0.7`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.292798`\), 1760.05`, 
              1797.59`, 11.7021`, 1772.12`, 1802.47`, 9.50387`, 411.955`, 
              428.959`, 1.25283`, 556.893`, 562.437`, 0.543726`, 411.955`, 
              422.447`, 0.767035`, 1747.15`, 1789.01`, 12.9717`, 200.473`, 
              237.52`, 1.42169`, 1760.05`, 1795.11`, 10.9199`, 1772.12`, 
              2561.01`, 556.893`, 2158.78`, 200.473`, 2514.15`, 1747.15`, 
              2783.11`, 2453.92`, 2767.13`, 2482.03`, 2731.86`, 2450.19`, 
              2485.15`, 2729.55`, 2768.81`, 411.955`, 2478.93`, 1760.05`, 
              2508.11`}, {2, 24, 56.7675313`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.330920929`, 0.3541018829`, 1.311666161`, 0.4`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.315175`\), 1764.99`, 
              1804.87`, 12.474`, 1778.87`, 1809.99`, 9.78492`, 382.907`, 
              403.254`, 1.40151`, 554.619`, 560.629`, 0.587199`, 382.907`, 
              395.204`, 0.838357`, 1751.75`, 1794.17`, 13.1812`, 0, 131.036`, 
              1.58656`, 1764.99`, 1801.24`, 11.3276`, 1778.87`, 2650.8`, 
              554.619`, 2117.69`, 0, 2515.31`, 1751.75`, 2798.94`, 2526.6`, 
              2790.07`, 2554.39`, 2744.76`, 2522.6`, 2557.71`, 2742.24`, 
              2782.66`, 382.907`, 2549.98`, 1764.99`, 2589.93`}, {1, 21, 
              28.4295812`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3258974233`, 0.2823705009`, 0.4741369224`, 0.5`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323513`\), 1329.71`, 
              1330.89`, 0.274759`, 1360.82`, 1361.26`, 0.103827`, 424.303`, 
              440.693`, 1.24217`, 625.513`, 630.535`, 0.552737`, 424.303`, 
              435.992`, 0.881064`, 1353.8`, 1354.16`, 0.08669`, 195.146`, 
              230.562`, 1.32096`, 1336.06`, 1337.37`, 0.305118`, 1431.02`, 
              1663.98`, 625.513`, 1904.05`, 195.146`, 1521.51`, 1597.05`, 
              2029.27`, 1336.06`, 1910.52`, 1356.1`, 1898.77`, 1329.71`, 
              1360.82`, 1895.17`, 1914.3`, 424.303`, 1353.8`, 1389.66`, 
              1627.52`}, {1, 13, 32.72386423`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6668535004`, 0.2781790962`, 0.6286592355`, 0.5`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.313353`\), 1582.18`, 
              1577.99`, \(-1.15995`\), 1609.88`, 1613.92`, 1.14128`, 
              419.708`, 436.463`, 1.25691`, 588.897`, 594.175`, 0.547125`, 
              419.708`, 430.722`, 0.820729`, 1620.4`, 1621.16`, 0.213738`, 
              200.916`, 236.747`, 1.37399`, 1589.84`, 
              1582.44`, \(-2.05646`\), 1626.63`, 1784.65`, 588.897`, 
              2186.19`, 200.916`, 1772.96`, 1672.13`, 2282.76`, 1589.84`, 
              2205.12`, 1623.98`, 2177.04`, 1582.18`, 1630.51`, 2172.4`, 
              2209.82`, 419.708`, 1620.4`, 1609.88`, 1727.37`}, {3, 21, 
              52.36903832`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8320629694`, 0.3365506995`, 0.7246038902`, 0.5`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.330002`\), 1685.81`, 
              1662.94`, \(-6.71097`\), 1688.43`, 1698.86`, 3.09635`, 
              484.268`, 498.035`, 1.18486`, 691.094`, 691.892`, 0.0966459`, 
              484.268`, 490.841`, 0.561549`, 1695.81`, 1730.41`, 10.3872`, 
              218.741`, 251.982`, 1.37095`, 1688.43`, 
              1666.04`, \(-6.58066`\), 1697.51`, 1898.36`, 691.094`, 
              2248.63`, 218.741`, 1931.78`, 1695.81`, 2411.96`, 1694.57`, 
              2316.76`, 1733.02`, 2284.18`, 1685.81`, 1740.35`, 2278.75`, 
              2322.2`, 484.268`, 1729.36`, 1688.43`, 1805.44`}, {3, 25, 
              65.55127638`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.914419096`, 0.2796577407`, 0.9006689657`, 0.5`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.356227`\), 1783.93`, 
              1784.76`, 0.258413`, 1797.76`, 1793.14`, \(-1.45353`\), 
              392.746`, 411.318`, 1.30841`, 539.756`, 546.891`, 0.679378`, 
              392.746`, 405.546`, 0.895282`, 1712.97`, 1805.75`, 28.6041`, 
              183.06`, 223.635`, 1.44582`, 1797.76`, 1792.65`, \(-1.6049`\), 
              1783.93`, 2015.58`, 539.756`, 2327.25`, 183.06`, 2092.12`, 
              1712.97`, 2573.39`, 1956.41`, 2534.8`, 1987.93`, 2504.58`, 
              1951.48`, 1992.17`, 2501.58`, 2537.81`, 392.746`, 1984.93`, 
              1797.76`, 1942.34`}, {1, 17, 54.43096574`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9979333629`, 0.289161034`, 0.9788236188`, 0.5`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.353521`\), 1783.52`, 
              1794.46`, 3.42747`, 1786.51`, 1796.11`, 3.01475`, 402.404`, 
              420.023`, 1.2696`, 549.51`, 555.811`, 0.610224`, 402.404`, 
              414.076`, 0.835007`, 1720.17`, 1795.07`, 23.07`, 192.982`, 
              231.172`, 1.41925`, 1786.51`, 1797.25`, 3.37248`, 1783.52`, 
              2125.24`, 549.51`, 2315.53`, 192.982`, 2187.97`, 1720.17`, 
              2637.58`, 2056.01`, 2603.61`, 2087.22`, 2572.34`, 2051.4`, 
              2091.17`, 2569.53`, 2606.42`, 402.404`, 2084.36`, 1786.51`, 
              2061.88`}, {3, 25, 52.01318012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.06928605`, 0.3102531596`, 1.043572326`, 0.5`, 0.3`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.354898`\), 1776.94`, 
              1798.4`, 6.72252`, 1782.96`, 1800.51`, 5.51265`, 420.214`, 
              437.159`, 1.27288`, 576.008`, 581.233`, 0.529735`, 420.214`, 
              430.709`, 0.782423`, 1728.29`, 1791.4`, 19.4646`, 196.235`, 
              234.423`, 1.44096`, 1776.94`, 1798.76`, 6.83468`, 1782.96`, 
              2230.29`, 576.008`, 2294.93`, 196.235`, 2279.16`, 1728.29`, 
              2687.86`, 2140.62`, 2652.25`, 2171.72`, 2619.94`, 2135.94`, 
              2175.7`, 2617.07`, 2655.12`, 420.214`, 2169.01`, 1776.94`, 
              2168.93`}, {3, 25, 53.12483753`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.132633515`, 0.3221543273`, 1.106969782`, 0.5`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.352962`\), 1772.67`, 
              1802.44`, 9.32403`, 1784.15`, 1806.82`, 7.13412`, 427.428`, 
              444.196`, 1.28058`, 586.239`, 591.079`, 0.499278`, 427.428`, 
              437.47`, 0.76104`, 1736.62`, 1792.79`, 17.3721`, 196.759`, 
              235.278`, 1.45809`, 1772.67`, 1801.77`, 9.11123`, 1784.15`, 
              2326.33`, 586.239`, 2266.35`, 196.759`, 2361.53`, 1736.62`, 
              2726.26`, 2225.61`, 2692.32`, 2256.11`, 2659.25`, 2221.08`, 
              2259.94`, 2656.46`, 2695.1`, 427.428`, 2253.52`, 1772.67`, 
              2267.2`}, {3, 25, 55.11143833`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.190596147`, 0.3266255031`, 1.168352306`, 0.5`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.345894`\), 1772.33`, 
              1807.75`, 11.1106`, 1786.27`, 1813.27`, 8.51363`, 427.484`, 
              444.249`, 1.28048`, 585.025`, 589.853`, 0.496976`, 427.484`, 
              437.438`, 0.754313`, 1744.94`, 1797.27`, 16.243`, 196.965`, 
              235.51`, 1.46056`, 1772.33`, 1806.26`, 10.6413`, 1786.27`, 
              2413.9`, 585.025`, 2230.71`, 196.965`, 2434.68`, 1744.94`, 
              2753.61`, 2309.08`, 2723.6`, 2338.71`, 2689.94`, 2304.82`, 
              2342.3`, 2687.31`, 2726.2`, 427.484`, 2336.21`, 1772.33`, 
              2357.27`}, {2, 25, 846.2054032`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              2.198380183`, 0.3552088392`, 1.30419364`, 0.5`, 0.6`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-2.38954`\), 875.817`, 
              967.99`, 14.8904`, 976.694`, 1027.34`, 8.89358`, 909.944`, 
              882.636`, \(-4.28901`\), 968.037`, 994.93`, 4.62531`, 909.944`, 
              902.198`, \(-1.22991`\), 968.037`, 969.578`, 0.261555`, 0, 0, 
              0.00537293`, 885.538`, 991.316`, 17.3946`, 1073.15`, 1974.98`, 
              1662.29`, 1730.16`, 0, 1592.84`, 1503.45`, 2149.31`, 885.538`, 
              1769.92`, 969.385`, 1716.8`, 875.817`, 976.694`, 1712.83`, 
              1773.64`, 909.944`, 968.037`, 991.299`, 1643.53`}, {3, 37, 
              566.7414323`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 1.7950036`, 
              0.5888832324`, 1.12064284`, 0.5`, 0.7`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.13996`\), 1602.84`, 
              1588.44`, \(-4.02661`\), 1609.83`, 1661.38`, 14.7744`, 
              939.986`, 937.752`, \(-0.367673`\), 1671.5`, 1674.83`, 
              0.97519`, 939.986`, 932.342`, \(-1.2541`\), 1671.5`, 
              1668.94`, \(-0.751393`\), 0, 120.734`, 1.4849`, 1609.83`, 
              1573.81`, \(-10.0474`\), 1724.54`, 2452.02`, 1726.38`, 
              2138.02`, 0, 2137.17`, 1718.72`, 2657.43`, 1616.77`, 2265.16`, 
              1674.92`, 2217.89`, 1602.84`, 1685.32`, 2209.13`, 2273.52`, 
              939.986`, 1671.5`, 1609.83`, 2032.9`}, {3, 25, 
              61.309781`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.342939573`, 0.3263195707`, 1.333802448`, 0.5`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.306011`\), 1785.24`, 
              1829.42`, 13.9924`, 1799, 1835.27`, 11.5503`, 417.097`, 
              433.998`, 1.26034`, 565.086`, 570.368`, 0.525502`, 417.097`, 
              427.283`, 0.753593`, 1770.15`, 1819.31`, 15.4584`, 198.706`, 
              236.339`, 1.43446`, 1785.24`, 1826.83`, 13.162`, 1799, 
              2641.79`, 565.086`, 2098.74`, 198.706`, 2573.26`, 1770.15`, 
              2789.35`, 2538.76`, 2785.56`, 2565.78`, 2737.94`, 2535.36`, 
              2568.62`, 2735.82`, 2775.29`, 417.097`, 2562.62`, 1785.24`, 
              2590.63`}, {3, 36, 28.79725629`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3506538118`, 0.2783906328`, 0.4773051938`, 0.6`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.325887`\), 1316.31`, 
              1317.64`, 0.306335`, 1349.61`, 1350.1`, 0.115613`, 424.088`, 
              440.498`, 1.24315`, 623.742`, 628.707`, 0.544869`, 424.088`, 
              435.646`, 0.870629`, 1342.13`, 1342.47`, 0.0788678`, 195.454`, 
              230.851`, 1.32213`, 1323.03`, 1324.48`, 0.335495`, 1418.12`, 
              1673.15`, 623.742`, 1913.69`, 195.454`, 1512.18`, 1605.9`, 
              2025.74`, 1323.03`, 1908.91`, 1344.48`, 1895.78`, 1316.31`, 
              1349.61`, 1892.01`, 1912.85`, 424.088`, 1342.13`, 1376.25`, 
              1637.61`}, {3, 13, 33.12466738`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7000230012`, 0.2719232783`, 0.674360726`, 0.6`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.353636`\), 1610.63`, 
              1605.13`, \(-1.54853`\), 1636.27`, 1640.22`, 1.13345`, 
              421.131`, 437.752`, 1.25073`, 589.232`, 594.398`, 0.535795`, 
              421.131`, 432.038`, 0.815312`, 1647.62`, 1648.4`, 0.225307`, 
              201.16`, 237.065`, 1.37858`, 1617.53`, 1608.96`, \(-2.42129`\), 
              1652.35`, 1800.36`, 589.232`, 2217.71`, 201.16`, 1794.15`, 
              1691.23`, 2310.58`, 1617.53`, 2237.32`, 1650.83`, 2209.37`, 
              1610.63`, 1656.74`, 2205.23`, 2241.5`, 421.131`, 1647.62`, 
              1636.27`, 1745.21`}, {1, 17, 54.72948312`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8482376904`, 0.3316843479`, 0.7551919876`, 0.6`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.365252`\), 1702.67`, 
              1680.93`, \(-6.44389`\), 1709.66`, 1718.31`, 2.59845`, 
              489.135`, 502.66`, 1.17527`, 698.248`, 698.75`, 0.0615213`, 
              489.135`, 495.448`, 0.544615`, 1709.84`, 1746, 10.9504`, 
              219.024`, 252.281`, 1.37334`, 1709.66`, 
              1684.13`, \(-7.59207`\), 1719.62`, 1903.51`, 698.248`, 
              2273.14`, 219.024`, 1945.83`, 1709.84`, 2428.24`, 1710.85`, 
              2333.35`, 1748.33`, 2301.24`, 1702.67`, 1755.2`, 2296.2`, 
              2338.39`, 489.135`, 1745.01`, 1709.66`, 1813.12`}, {3, 37, 
              82.61433792`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9293875709`, 0.2757241882`, 0.934914959`, 0.6`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.387813`\), 1817.7`, 
              1815.12`, \(-0.820584`\), 1834.89`, 1824.18`, \(-3.43403`\), 
              391.221`, 410.329`, 1.34193`, 539.689`, 547.141`, 0.709621`, 
              391.221`, 404.497`, 0.925592`, 1727.07`, 1838.45`, 34.7946`, 
              175.813`, 218.661`, 1.48093`, 1834.89`, 
              1824.01`, \(-3.48578`\), 1817.7`, 2022.85`, 539.689`, 2353.7`, 
              175.813`, 2117.63`, 1727.07`, 2591.76`, 1984.6`, 2554.34`, 
              2015.35`, 2524.33`, 1980.06`, 2019.26`, 2521.58`, 2557.09`, 
              391.221`, 2012.62`, 1834.89`, 1947.54`}, {3, 21, 
              65.32194442`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.01110141`, 0.2764536968`, 1.013142444`, 0.6`, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.379629`\), 1814.38`, 
              1823.84`, 3.01596`, 1820.6`, 1825.64`, 1.61002`, 393.314`, 
              411.491`, 1.28172`, 536.21`, 543.168`, 0.658048`, 393.314`, 
              405.715`, 0.868168`, 1733.29`, 1825.15`, 28.64`, 187.962`, 
              227.17`, 1.42608`, 1820.6`, 1827.51`, 2.2087`, 1814.38`, 
              2129.07`, 536.21`, 2333.31`, 187.962`, 2207.75`, 1733.29`, 
              2648.87`, 2085.01`, 2619.19`, 2115.22`, 2588.2`, 2080.96`, 
              2118.7`, 2585.76`, 2621.63`, 393.314`, 2112.57`, 1820.6`, 
              2067.52`}, {3, 25, 59.58524242`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.081238261`, 0.3031813276`, 1.071021129`, 0.6`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.38013`\), 1806.61`, 
              1826.04`, 6.1844`, 1810.52`, 1828.03`, 5.58312`, 419.102`, 
              436.136`, 1.27645`, 574.941`, 580.25`, 0.537402`, 419.102`, 
              429.701`, 0.788232`, 1741.15`, 1818.95`, 24.2678`, 193.4`, 
              232.156`, 1.44503`, 1806.61`, 1827.12`, 6.52832`, 1810.52`, 
              2233.13`, 574.941`, 2305.55`, 193.4`, 2296.48`, 1741.15`, 
              2693.77`, 2160.51`, 2659.19`, 2190.92`, 2627.04`, 2156.2`, 
              2194.59`, 2624.4`, 2661.82`, 419.102`, 2188.43`, 1806.61`, 
              2172.24`}, {3, 25, 59.90003695`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.143575671`, 0.3213241362`, 1.129273046`, 0.6`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.379749`\), 1799.35`, 
              1829.27`, 9.51322`, 1810.43`, 1833.88`, 7.48801`, 433.301`, 
              449.963`, 1.28946`, 596.459`, 601.014`, 0.477976`, 433.301`, 
              443.082`, 0.751051`, 1749.91`, 1818.46`, 21.4318`, 194.487`, 
              233.746`, 1.47301`, 1799.35`, 1829.17`, 9.48162`, 1810.43`, 
              2330.51`, 596.459`, 2271.92`, 194.487`, 2379.64`, 1749.91`, 
              2730.01`, 2239.55`, 2693.93`, 2269.62`, 2660.9`, 2235.2`, 
              2273.31`, 2658.22`, 2696.59`, 433.301`, 2267.24`, 1799.35`, 
              2270.6`}, {3, 25, 62.11081142`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.200717925`, 0.3296888317`, 1.187661711`, 0.6`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.374315`\), 1796.49`, 
              1833.88`, 11.8934`, 1811.58`, 1840.04`, 9.10525`, 437.142`, 
              453.742`, 1.29575`, 601.485`, 605.855`, 0.462187`, 437.142`, 
              446.669`, 0.737745`, 1758.69`, 1821.58`, 19.7295`, 194.482`, 
              234.035`, 1.48502`, 1796.49`, 1832.76`, 11.5342`, 1811.58`, 
              2419.8`, 601.485`, 2231.39`, 194.482`, 2454.24`, 1758.69`, 
              2756.19`, 2320.08`, 2722.27`, 2349.43`, 2688.55`, 2315.89`, 
              2352.96`, 2685.97`, 2724.81`, 437.142`, 2347.14`, 1796.49`, 
              2362.01`}, {2, 20, 936.9002087`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.996117434`, 0.4113302895`, 1.228406632`, 0.6`, 0.6`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-1.75971`\), 1225.28`, 
              1265.85`, 8.85354`, 1267.46`, 1294.28`, 6.01938`, 910.374`, 
              889.042`, \(-3.36331`\), 1296.96`, 1316.37`, 4.44399`, 
              910.374`, 895.142`, \(-2.40963`\), 1296.96`, 1298.27`, 
              0.297119`, 0, 0, \(-0.136523`\), 1234.99`, 1242.33`, 1.59339`, 
              1341.59`, 2098.89`, 1630.75`, 1872.69`, 0, 1848.17`, 1568.57`, 
              2340.65`, 1234.99`, 1988.45`, 1298.72`, 1940.12`, 1225.28`, 
              1306.27`, 1935.12`, 1993.2`, 910.374`, 1296.96`, 1267.46`, 
              1768}, {2, 25, 66.98908952`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.304130685`, 0.3314482651`, 1.29901723`, 0.6`, 0.7`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.349881`\), 1799.61`, 
              1845.7`, 14.7192`, 1816.34`, 1852.76`, 11.7079`, 431.117`, 
              447.81`, 1.2855`, 589.529`, 594.161`, 0.480361`, 431.117`, 
              440.749`, 0.73577`, 1775.79`, 1833.4`, 18.2185`, 195.162`, 
              234.285`, 1.47206`, 1799.61`, 1843.44`, 13.9888`, 1816.34`, 
              2578.56`, 589.529`, 2136.08`, 195.162`, 2574.4`, 1775.79`, 
              2783.92`, 2475.83`, 2762.92`, 2503.45`, 2723.78`, 2472.19`, 
              2506.5`, 2721.52`, 2760.81`, 431.117`, 2501.22`, 1799.61`, 
              2525.38`}, {3, 25, 69.24214533`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.351587019`, 0.3300116563`, 1.350734961`, 0.6`, 0.8`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.333727`\), 1803.92`, 
              1852.56`, 15.5804`, 1820.06`, 1859.51`, 12.7175`, 426.506`, 
              443.258`, 1.27653`, 581.121`, 585.953`, 0.494108`, 426.506`, 
              436.265`, 0.73765`, 1784.07`, 1840.66`, 17.9716`, 195.904`, 
              234.596`, 1.45941`, 1803.92`, 1850.02`, 14.7579`, 1820.06`, 
              2650.39`, 581.121`, 2084.03`, 195.904`, 2609.9`, 1784.07`, 
              2788.57`, 2548.9`, 2782.27`, 2575.71`, 2732.52`, 2545.53`, 
              2578.53`, 2730.42`, 2769.96`, 426.506`, 2573.27`, 1803.92`, 
              2599.08`}, {1, 43, 29.20281638`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3794000017`, 0.2741440504`, 0.4835999672`, 0.7`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.330753`\), 1311.21`, 
              1312.68`, 0.336265`, 1347, 1347.54`, 0.126003`, 424.258`, 
              440.652`, 1.24235`, 622.091`, 626.972`, 0.534165`, 424.258`, 
              435.656`, 0.858823`, 1339.06`, 1339.36`, 0.0702443`, 195.774`, 
              231.151`, 1.32328`, 1318.3`, 1319.85`, 0.358822`, 1411.86`, 
              1685.58`, 622.091`, 1928.09`, 195.774`, 1510.31`, 1617.53`, 
              2029.09`, 1318.3`, 1915.34`, 1341.45`, 1900.41`, 1311.21`, 
              1347, 1896.43`, 1919.47`, 424.258`, 1339.06`, 1370.3`, 
              1650.59`}, {1, 18, 33.55731291`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7281566287`, 0.2673426238`, 0.7153743215`, 0.7`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.388875`\), 1638.99`, 
              1631.94`, \(-2.02095`\), 1662.68`, 1666.21`, 1.03138`, 
              422.673`, 439.165`, 1.24532`, 590.131`, 595.183`, 0.524722`, 
              422.673`, 433.467`, 0.809624`, 1674.95`, 1675.74`, 0.230526`, 
              201.413`, 237.371`, 1.3824`, 1645.28`, 1635.36`, \(-2.85192`\), 
              1677.94`, 1814.39`, 590.131`, 2246.23`, 201.413`, 1816.65`, 
              1708, 2336.26`, 1645.28`, 2267.25`, 1677.88`, 2239.43`, 
              1638.99`, 1683.29`, 2235.68`, 2271.02`, 422.673`, 1674.95`, 
              1662.68`, 1761.36`}, {3, 21, 56.67624186`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8631426366`, 0.3338398975`, 0.7784617299`, 0.7`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.395655`\), 1713.07`, 
              1693.37`, \(-5.8785`\), 1725.2`, 1733.24`, 2.43792`, 500.312`, 
              513.57`, 1.17778`, 716.625`, 716.588`, \(-0.00465923`\), 
              500.312`, 506.21`, 0.520179`, 1723.3`, 1755.99`, 9.96362`, 
              221.427`, 254.736`, 1.38965`, 1721.02`, 
              1696.57`, \(-7.32061`\), 1737.62`, 1913.57`, 716.625`, 2291.1`, 
              221.427`, 1962.33`, 1723.3`, 2442.46`, 1721.02`, 2343.92`, 
              1758.13`, 2311.94`, 1713.07`, 1764.83`, 2307.06`, 2348.81`, 
              500.312`, 1755.04`, 1725.2`, 1823.8`}, {3, 25, 
              123.0306074`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9701555048`, 0.3903158384`, 0.8400463439`, 0.7`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.417464`\), 1770.38`, 
              1744.32`, \(-8.02661`\), 1780.82`, 1786.54`, 1.78827`, 
              563.183`, 574.868`, 1.16518`, 820.91`, 
              817.491`, \(-0.490801`\), 563.183`, 566.151`, 0.293679`, 
              1736.95`, 1831.71`, 29.629`, 238.453`, 270.794`, 1.44299`, 
              1770.38`, 1747.07`, \(-7.18575`\), 1780.82`, 2019.98`, 820.91`, 
              2319.5`, 238.453`, 2092.22`, 1736.95`, 2533.2`, 1794.27`, 
              2409.93`, 1833.93`, 2375.42`, 1785.21`, 1841.41`, 2369.72`, 
              2415.59`, 563.183`, 1830.73`, 1770.38`, 1896.69`}, {3, 26, 
              78.9843801`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.024094567`, 0.264303827`, 1.04673365`, 0.7`, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.403844`\), 1845.4`, 
              1852.51`, 2.30276`, 1854.51`, 1854.37`, \(-0.0447633`\), 
              382.67`, 401.61`, 1.30148`, 521.019`, 528.803`, 0.716006`, 
              382.67`, 396.013`, 0.91036`, 1746.49`, 1854.44`, 34.0583`, 
              181.795`, 222.43`, 1.43915`, 1854.51`, 1856.74`, 0.726061`, 
              1845.4`, 2134.16`, 521.019`, 2350.11`, 181.795`, 2228.34`, 
              1746.49`, 2659.33`, 2114.82`, 2633.95`, 2144.12`, 2603.19`, 
              2111.27`, 2147.18`, 2601.06`, 2636.07`, 382.67`, 2141.64`, 
              1854.51`, 2074.4`}, {3, 35, 68.98055337`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.093112986`, 0.2937865335`, 1.098882356`, 0.7`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.402407`\), 1836.63`, 
              1852.98`, 5.28753`, 1838.03`, 1854.75`, 5.41194`, 414.163`, 
              431.414`, 1.27805`, 567.84`, 573.441`, 0.560064`, 414.163`, 
              425.074`, 0.802274`, 1753.66`, 1846.36`, 29.2398`, 190.008`, 
              229.31`, 1.44392`, 1836.63`, 1854.64`, 5.82606`, 1838.03`, 
              2235.23`, 567.84`, 2315.86`, 190.008`, 2312.54`, 1753.66`, 
              2698.37`, 2182.05`, 2666.16`, 2211.71`, 2634.15`, 2178.13`, 
              2215.07`, 2631.78`, 2668.53`, 414.163`, 2209.41`, 1836.63`, 
              2175.46`}, {3, 29, 67.772693`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.154409631`, 0.3180072685`, 1.151785291`, 0.7`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.403383`\), 1826.72`, 
              1855.51`, 9.28756`, 1836.56`, 1860.11`, 7.62838`, 435.875`, 
              452.488`, 1.29309`, 601.266`, 605.689`, 0.467674`, 435.875`, 
              445.533`, 0.745848`, 1762.59`, 1844.19`, 25.7854`, 192.126`, 
              231.956`, 1.47997`, 1826.72`, 1855.97`, 9.43917`, 1836.56`, 
              2333.02`, 601.266`, 2278.08`, 192.126`, 2395.74`, 1762.59`, 
              2732.42`, 2254.49`, 2695.51`, 2284.08`, 2662.52`, 2250.35`, 
              2287.59`, 2659.98`, 2698.02`, 435.875`, 2281.88`, 1826.72`, 
              2272.72`}, {3, 29, 69.96177904`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.210684466`, 0.331530837`, 1.206417101`, 0.7`, 0.5`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.400312`\), 1821.69`, 
              1859.72`, 12.2672`, 1837.09`, 1866.24`, 9.4583`, 445.202`, 
              461.654`, 1.30715`, 615.256`, 619.238`, 0.43065`, 445.202`, 
              454.363`, 0.722032`, 1771.75`, 1845.97`, 23.5263`, 192.117`, 
              232.549`, 1.50436`, 1821.69`, 1859.01`, 12.036`, 1837.09`, 
              2423.73`, 615.256`, 2234.02`, 192.117`, 2471.68`, 1771.75`, 
              2757.83`, 2330.87`, 2720.59`, 2359.94`, 2686.83`, 2326.76`, 
              2363.4`, 2684.3`, 2723.06`, 445.202`, 2357.82`, 1821.69`, 
              2364.62`}, {3, 33, 72.94774011`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.263294236`, 0.3366709071`, 1.26126984`, 0.7`, 0.6`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.391598`\), 1819.9`, 
              1864.63`, 14.4379`, 1838.12`, 1872.22`, 11.0865`, 445.979`, 
              462.441`, 1.31025`, 615.467`, 619.419`, 0.42751`, 445.979`, 
              455.039`, 0.715207`, 1780.64`, 1850.12`, 22.1035`, 191.915`, 
              232.515`, 1.5098`, 1819.9`, 1863.04`, 13.9189`, 1838.12`, 
              2507.44`, 615.467`, 2183.99`, 191.915`, 2539.11`, 1780.64`, 
              2774.56`, 2408.24`, 2740.28`, 2436.54`, 2705.75`, 2404.33`, 
              2439.83`, 2703.33`, 2742.48`, 445.979`, 2434.49`, 1819.9`, 
              2450.62`}, {3, 25, 75.8219105`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.312923389`, 0.3373533795`, 1.314665516`, 0.7`, 0.7`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.378461`\), 1820.67`, 
              1870.05`, 15.9675`, 1839.7`, 1878.07`, 12.4989`, 442.658`, 
              459.178`, 1.30537`, 609.1`, 613.196`, 0.438692`, 442.658`, 
              451.785`, 0.715259`, 1789.19`, 1855.66`, 21.2298`, 192.048`, 
              232.458`, 1.50298`, 1820.67`, 1867.89`, 15.2594`, 1839.7`, 
              2585.13`, 609.1`, 2129.91`, 192.048`, 2596.77`, 1789.19`, 
              2784.09`, 2484, 2760.21`, 2511.48`, 2718.89`, 2480.34`, 
              2514.54`, 2716.62`, 2756.04`, 442.658`, 2509.46`, 1820.67`, 
              2530.69`}, {3, 25, 78.42212504`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.360005759`, 0.3356837924`, 1.36602955`, 0.7`, 0.8`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.362015`\), 1823.23`, 
              1875.87`, 17.0608`, 1841.9`, 1883.89`, 13.7076`, 437.503`, 
              454.092`, 1.2959`, 599.794`, 604.112`, 0.455472`, 437.503`, 
              446.774`, 0.718225`, 1797.37`, 1861.98`, 20.7148`, 192.699`, 
              232.661`, 1.4893`, 1823.23`, 1873.36`, 16.2377`, 1841.9`, 
              2657.74`, 599.794`, 2072.96`, 192.699`, 2639.45`, 1797.37`, 
              2787.72`, 2557.16`, 2778.32`, 2583.85`, 2726.69`, 2553.78`, 
              2586.67`, 2724.58`, 2764.26`, 437.503`, 2581.77`, 1823.23`, 
              2605.57`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data three/ytraj_m0_m1_?.out composite", "Section"],

Cell["\<\
This was hard to do because the third data set had to be shifted \
over by hand.\
\>", "Text"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for transverse lattice mesons \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\n\
sqrt(string tension) is 440 MeV.\nUse chi^2 fit with 11 criteria and \
tolerance 0.001.\nOverall scale from glueball data.\n0 nonstandard fit \
criteria (for log divergence).\nFit 3 masses to experimental values\n  \
(charge,multi,state,mass (MeV),error (MeV)) =\n   (1,4,0,139.557,69.3623) \
(-1,3,0,785.988,73.8942)\n   (-1,7,0,785.988,73.8942).\nNo degenerate pairs.\n\
Spectrum for P_perp a = (0,0), ( 0.25 0.25)\n  for 2 states (multiplet, \
charge, c^2 error for each) =\n  (17,  1, 5 5)\n  (17, -1, 0.2 0.2)\n  (18, \
-1, 0.2 5)\n  (18,  1, 0.2 5).\nOrdinary spectra multiplets, 2 states, \
(charge,multiplet) =\n  ( 1,  3)  (-1,  3)  ( 1,  4)  (-1,  4)  ( 1,  5)  \
(-1,  5) \n  ( 1,  6)  (-1,  6)  (-1,  7)  ( 1,  7) .\nAll spectra \
extrapolated using (K,p) = (12/2,4)  (14/2,4) \n  (18/2,4) \nLog divergence \
determined using pion (K,p) = (20/2,4) \n  (24/2,4)  (28/2,4)  (36/2,4) .\n\
Result format:\n Fit info, # steps, chi^2, p damping, scale G^2 N/sigma.\n \
The 17 couplings  (G^2 N units);\n   which couplings -- if any -- were fit.\n \
Winding potential and heavy souce potential fits (G^2 N units).\n The log(K) \
coefficient of the pion mass^2 (G^2 N units).\n The rescaled masses (MeV) for \
each P_perp*a and c^2 values.\n Finally come the masses (MeV) for the \
ordinary spectra.\>", {{3, 77, 26.26858193`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1846873158`, 0.3756823285`, 
              0.8745442022`, \(-0.5`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.462849`\), 1585.6`, 
              1588.66`, 0.851045`, 1610.74`, 1613.39`, 0.749528`, 452.208`, 
              466.627`, 1.16079`, 644.937`, 649.696`, 0.539826`, 452.208`, 
              464.164`, 0.959917`, 1642.82`, 1644.81`, 0.573416`, 191.18`, 
              228.965`, 1.39093`, 1610.74`, 1613.65`, 0.824068`, 1585.6`, 
              2447.28`, 644.937`, 2216, 191.18`, 2163.46`, 1642.82`, 
              2679.65`, 2296.25`, 2752.39`, 2305.36`, 2746.15`, 2293.97`, 
              2306.77`, 2744.13`, 2754.55`, 452.208`, 2184.81`, 1610.74`, 
              2374.89`}, {3, 13, 26.18288012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.196048301`, 0.4102974277`, 
              0.8421873012`, \(-0.5`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.427103`\), 1569.35`, 
              1572.29`, 0.809302`, 1595.65`, 1598.18`, 0.705766`, 457.985`, 
              472.129`, 1.1527`, 647.381`, 652.102`, 0.537476`, 457.985`, 
              469.688`, 0.951264`, 1631.12`, 1632.79`, 0.477929`, 190.561`, 
              228.865`, 1.40763`, 1595.65`, 1598.4`, 0.768906`, 1569.35`, 
              2602.66`, 647.381`, 2199.89`, 190.561`, 2154.99`, 1631.12`, 
              2661.39`, 2385, 2839.57`, 2396.78`, 2831.44`, 2382.11`, 
              2398.51`, 2828.8`, 2842.41`, 457.985`, 2172.04`, 1595.65`, 
              2475.14`}, {3, 22, 25.92729233`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2073236909`, 0.4435322063`, 
              0.808136633`, \(-0.5`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.383595`\), 1551.63`, 
              1554.47`, 0.772335`, 1579.69`, 1582.06`, 0.657756`, 466.814`, 
              480.511`, 1.13694`, 650.65`, 655.355`, 0.538342`, 466.814`, 
              478.132`, 0.937106`, 1618.3`, 1619.66`, 0.38528`, 189.602`, 
              228.636`, 1.43039`, 1579.69`, 1582.26`, 0.713336`, 1551.63`, 
              2643.2`, 650.65`, 2183.12`, 189.602`, 2144, 1618.3`, 2651.8`, 
              2459.54`, 2913.84`, 2474.72`, 2903.27`, 2455.88`, 2476.85`, 
              2899.81`, 2917.6`, 466.814`, 2157.55`, 1579.69`, 2642.4`}, {3, 
              13, 25.4607596`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2183915838`, 0.4733467663`, 
              0.772956802`, \(-0.5`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.326448`\), 1526.13`, 
              1528.96`, 0.757556`, 1558.44`, 1560.65`, 0.603444`, 479.424`, 
              492.469`, 1.11077`, 654.732`, 659.474`, 0.546082`, 479.424`, 
              490.211`, 0.916381`, 1601.74`, 1602.77`, 0.289781`, 188.2`, 
              228.242`, 1.46101`, 1558.44`, 1560.87`, 0.66385`, 1526.13`, 
              2601.03`, 654.732`, 2156.33`, 188.2`, 2113.47`, 1601.74`, 
              2624.82`, 2520.98`, 2976.03`, 2540.36`, 2962.38`, 2516.36`, 
              2543.02`, 2957.85`, 2981.03`, 479.424`, 2128.05`, 1558.44`, 
              2607.89`}, {3, 21, 24.75822128`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2297919988`, 0.4988969312`, 
              0.7399186213`, \(-0.5`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.248693`\), 1472.81`, 
              1475.78`, 0.765343`, 1513.49`, 1515.51`, 0.535194`, 495.988`, 
              508.188`, 1.07344`, 660.017`, 664.893`, 0.565978`, 495.988`, 
              506.129`, 0.890464`, 1564.64`, 1565.31`, 0.184739`, 186.278`, 
              227.639`, 1.5`, 1513.49`, 1515.82`, 0.618614`, 1472.81`, 
              2472.24`, 660.017`, 2074.44`, 186.278`, 2024.59`, 1564.64`, 
              2497.68`, 2575.05`, 3031.15`, 2599.43`, 3013.7`, 2569.23`, 
              2602.79`, 3007.77`, 3037.85`, 495.988`, 2041.07`, 1513.49`, 
              2478.46`}, {3, 13, 23.94141815`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2433533918`, 0.5223987797`, 0.7134661265`, \(-0.5`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.151501`\), 1380.49`, 
              1383.59`, 0.751914`, 1429.83`, 1431.64`, 0.453909`, 513.286`, 
              524.592`, 1.02813`, 663.897`, 669.081`, 0.605443`, 513.286`, 
              522.81`, 0.864624`, 1488.37`, 1488.72`, 0.0918868`, 184.437`, 
              227.176`, 1.54134`, 1429.83`, 1432.07`, 0.561992`, 1380.49`, 
              2341.5`, 663.897`, 1944, 184.437`, 1895.53`, 1488.37`, 
              2357.02`, 2626.46`, 3082.57`, 2656.51`, 3060.64`, 2619.21`, 
              2660.88`, 3052.9`, 3091.61`, 513.286`, 1909.33`, 1429.83`, 
              2341.04`}, {3, 17, 23.27030547`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2603186135`, 0.5467907374`, 0.6941343245`, \(-0.5`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0489565`\), 1272.1`, 
              1275.3`, 0.713218`, 1327.29`, 1328.94`, 0.382898`, 527.286`, 
              537.737`, 0.975187`, 662.292`, 667.999`, 0.665208`, 527.286`, 
              536.308`, 0.840716`, 1390.17`, 1390.33`, 0.0373743`, 183.467`, 
              227.252`, 1.57561`, 1327.29`, 1329.47`, 0.506323`, 1272.1`, 
              2264.82`, 662.292`, 1825.15`, 183.467`, 1781.47`, 1390.17`, 
              2272.04`, 2671.47`, 3125.63`, 2707.49`, 3098.98`, 2662.65`, 
              2713.18`, 3088.99`, 3137.78`, 527.286`, 1790.71`, 1327.29`, 
              2258.05`}, {2, 23, 22.98813521`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2805907558`, 0.5724482126`, 0.6798130879`, \(-0.5`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.041384`, 1168.9`, 
              1172.17`, 0.671462`, 1227.85`, 1229.39`, 0.330865`, 536.382`, 
              545.988`, 0.910919`, 653.438`, 659.885`, 0.741841`, 536.382`, 
              545.008`, 0.817301`, 1293.2`, 1293.26`, 0.0138426`, 183.677`, 
              227.95`, 1.59671`, 1227.85`, 1230.01`, 0.464388`, 1168.9`, 
              2233.65`, 653.438`, 1741.69`, 183.677`, 1701.08`, 1293.2`, 
              2235.24`, 2705.03`, 3155.58`, 2746.65`, 3124.78`, 2694.65`, 
              2753.89`, 3112.09`, 3171.34`, 536.382`, 1706.73`, 1227.85`, 
              2221.58`}, {2, 39, 23.35663452`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3034943467`, 0.5976044673`, 0.6687674752`, \(-0.5`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.10448`, 1077.86`, 
              1081.22`, 0.634944`, 1139.29`, 1140.75`, 0.292845`, 539.585`, 
              548.288`, 0.829605`, 636.498`, 643.914`, 0.831919`, 539.585`, 
              547.907`, 0.792962`, 1206.05`, 1206.08`, 0.00815293`, 185.32`, 
              229.298`, 1.59759`, 1139.29`, 1141.46`, 0.434794`, 1077.86`, 
              2230.82`, 636.498`, 1691.39`, 185.32`, 1651.23`, 1206.05`, 
              2227.84`, 2726.53`, 3172.99`, 2772.88`, 3139.05`, 2714.8`, 
              2781.66`, 3123.35`, 3192.3`, 539.585`, 1655.15`, 1139.29`, 
              2213.86`}, {3, 54, 24.792966`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3270699143`, 0.6190605549`, 0.6600796365`, \(-0.5`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.131888`, 999.179`, 
              1002.61`, 0.601682`, 1061.82`, 1063.22`, 0.261253`, 535.558`, 
              543.197`, 0.721975`, 610.896`, 619.489`, 0.926353`, 535.558`, 
              543.634`, 0.763564`, 1128.92`, 1128.97`, 0.0107501`, 188.69`, 
              231.245`, 1.56573`, 1061.82`, 1064.02`, 0.410075`, 999.179`, 
              2244.28`, 610.896`, 1666.27`, 188.69`, 1625.2`, 1128.92`, 
              2236.97`, 2739.93`, 3182.37`, 2789.97`, 3146.2`, 2727.12`, 
              2800.1`, 3127.37`, 3204.77`, 535.558`, 1629.06`, 1061.82`, 
              2222.77`}, {2, 25, 28.77028094`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3280477695`, 0.6759529135`, 0.6472380354`, \(-0.5`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.062704`, 860.935`, 
              864.761`, 0.578464`, 937.307`, 938.4`, 0.179666`, 531.419`, 
              536.525`, 0.477765`, 598.665`, 607.256`, 0.90777`, 531.419`, 
              537.697`, 0.588073`, 1015.68`, 1015.3`, \(-0.0676662`\), 0, 
              96.1972`, 1.56394`, 937.307`, 939.425`, 0.348254`, 860.935`, 
              2292.63`, 598.665`, 1665.61`, 0, 1621.32`, 1015.68`, 2282.69`, 
              2768.26`, 3199.6`, 2822.64`, 3162.29`, 2754.58`, 2834.09`, 
              3138.05`, 3228.19`, 531.419`, 1624.37`, 937.307`, 
              2262.01`}, {3, 13, 26.2876898`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1822822432`, 0.3481938492`, 
              0.8375962778`, \(-0.4`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.464865`\), 1593.52`, 
              1596.68`, 0.88254`, 1616.61`, 1619.44`, 0.801264`, 447.416`, 
              462.095`, 1.16977`, 642.722`, 647.54`, 0.544687`, 447.416`, 
              459.508`, 0.960848`, 1642.83`, 1645.3`, 0.71147`, 191.699`, 
              229.001`, 1.37497`, 1616.61`, 1619.75`, 0.889555`, 1593.52`, 
              2284.04`, 642.722`, 2205.51`, 191.699`, 2188.36`, 1642.83`, 
              2658.55`, 2143.5`, 2607.53`, 2152.12`, 2601.7`, 2141.27`, 
              2153.54`, 2599.79`, 2609.57`, 447.416`, 2151.14`, 1616.61`, 
              2215.28`}, {3, 13, 26.32333993`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1951084523`, 0.3855830586`, 
              0.8006101398`, \(-0.4`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.428555`\), 1574.76`, 
              1577.76`, 0.828773`, 1598.53`, 1601.23`, 0.757521`, 451.666`, 
              466.176`, 1.16684`, 644.882`, 649.645`, 0.540198`, 451.666`, 
              463.572`, 0.954723`, 1629.97`, 1632, 0.57947`, 191.236`, 
              228.972`, 1.38932`, 1598.53`, 1601.45`, 0.820137`, 1574.76`, 
              2400.04`, 644.882`, 2194.69`, 191.236`, 2141.85`, 1629.97`, 
              2644.7`, 2236.95`, 2698.43`, 2248.46`, 2690.62`, 2234.05`, 
              2250.25`, 2688.06`, 2701.18`, 451.666`, 2184.18`, 1598.53`, 
              2322.41`}, {2, 9, 26.1739472`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2082939496`, 0.4229996884`, 
              0.7610254731`, \(-0.4`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384711`\), 1555.48`, 
              1558.33`, 0.777232`, 1580.42`, 1582.97`, 0.707978`, 459.103`, 
              473.26`, 1.15643`, 648.028`, 652.747`, 0.537724`, 459.103`, 
              470.677`, 0.942811`, 1615.78`, 1617.44`, 0.469062`, 190.43`, 
              228.799`, 1.40937`, 1580.42`, 1583.14`, 0.755853`, 1555.48`, 
              2521.62`, 648.028`, 2177.32`, 190.43`, 2131, 1615.78`, 
              2629.12`, 2313.56`, 2775.35`, 2328.9`, 2764.9`, 2309.79`, 
              2331.16`, 2761.45`, 2779.09`, 459.103`, 2148.08`, 1580.42`, 
              2412.78`}, {3, 9, 25.76842062`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2215883239`, 0.4587567992`, 
              0.7180872739`, \(-0.4`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32741`\), 1531.7`, 
              1534.45`, 0.740943`, 1559.67`, 1562.04`, 0.649234`, 471.211`, 
              484.754`, 1.1343`, 652.373`, 657.084`, 0.540462`, 471.211`, 
              482.255`, 0.922584`, 1599.7`, 1600.97`, 0.355947`, 189.08`, 
              228.409`, 1.43862`, 1559.67`, 1562.2`, 0.693749`, 1531.7`, 
              2609.64`, 652.373`, 2156.81`, 189.08`, 2111.45`, 1599.7`, 
              2628.11`, 2372.17`, 2836.74`, 2392.55`, 2822.78`, 2367.26`, 
              2395.42`, 2818.1`, 2841.89`, 471.211`, 2127.21`, 1559.67`, 
              2606.89`}, {3, 13, 24.98469643`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2350283549`, 0.4914587888`, 
              0.6741914647`, \(-0.4`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.244935`\), 1481.36`, 
              1484.22`, 0.742143`, 1517.9`, 1520.04`, 0.570825`, 490.358`, 
              502.937`, 1.09475`, 659.329`, 664.117`, 0.555178`, 490.358`, 
              500.632`, 0.892077`, 1566.93`, 1567.75`, 0.224468`, 186.768`, 
              227.596`, 1.48227`, 1517.9`, 1520.3`, 0.639401`, 1481.36`, 
              2486.53`, 659.329`, 2084.05`, 186.768`, 2030.3`, 1566.93`, 
              2512.02`, 2417.78`, 2887, 2444.6`, 2868.5`, 2411.45`, 2448.29`, 
              2862.13`, 2894.21`, 490.358`, 2048.27`, 1517.9`, 2492.27`}, {3, 
              13, 23.88400007`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2511109211`, 0.5226381234`, 0.6373505709`, \(-0.4`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130039`\), 1372.51`, 
              1375.54`, 0.728323`, 1420.06`, 1421.92`, 0.46397`, 513.962`, 
              525.352`, 1.0372`, 665.688`, 670.823`, 0.601244`, 513.962`, 
              523.396`, 0.857499`, 1479.1`, 1479.46`, 0.0948829`, 184.105`, 
              226.817`, 1.53775`, 1420.06`, 1422.33`, 0.567417`, 1372.51`, 
              2324, 665.688`, 1928.24`, 184.105`, 1875.27`, 1479.1`, 
              2336.95`, 2460.9`, 2934.78`, 2495.28`, 2910.8`, 2452.82`, 
              2500.17`, 2902.05`, 2945.15`, 513.962`, 1889.73`, 1420.06`, 
              2321.09`}, {3, 21, 23.01799539`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2729177078`, 0.5552530441`, 0.6114576798`, \(-0.4`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0046601`\), 
              1236.71`, 1239.84`, 0.677439`, 1291.11`, 1292.75`, 0.371025`, 
              532.789`, 542.98`, 0.960529`, 661.547`, 667.469`, 0.689586`, 
              532.789`, 541.553`, 0.824999`, 1355.37`, 1355.48`, 0.0246`, 
              183.033`, 227.126`, 1.58458`, 1291.11`, 1293.3`, 0.494113`, 
              1236.71`, 2238.66`, 661.547`, 1786.34`, 183.033`, 1738.07`, 
              1355.37`, 2241.05`, 2498.88`, 2974.72`, 2541.01`, 2945.23`, 
              2488.87`, 2547.56`, 2933.25`, 2989.64`, 532.789`, 1746.4`, 
              1291.11`, 2227.01`}, {2, 25, 23.11892632`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3002706509`, 0.5875940165`, 0.5934012883`, \(-0.4`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.094527`, 1113.23`, 
              1116.43`, 0.626046`, 1171, 1172.5`, 0.307674`, 541.215`, 
              550.152`, 0.854594`, 641.742`, 648.926`, 0.812408`, 541.215`, 
              549.515`, 0.793282`, 1237.34`, 1237.35`, 0.00268614`, 184.679`, 
              228.865`, 1.60098`, 1171, 1173.15`, 0.442663`, 1113.23`, 
              2216.83`, 641.742`, 1700.65`, 184.679`, 1652.84`, 1237.34`, 
              2212.11`, 2523.35`, 2997.59`, 2572.16`, 2963.93`, 2511.54`, 
              2580.62`, 2948.02`, 3017.81`, 541.215`, 1657.53`, 1171, 
              2197.79`}, {3, 42, 25.27187954`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.328917537`, 0.6124152075`, 0.5805173385`, \(-0.4`\), 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.142543`, 1010.29`, 
              1013.56`, 0.580765`, 1068.66`, 1070.03`, 0.257267`, 535.266`, 
              542.644`, 0.696827`, 604.412`, 613.29`, 0.947191`, 535.266`, 
              543.238`, 0.75327`, 1134.64`, 1134.65`, 0.00220426`, 190.067`, 
              232.109`, 1.55508`, 1068.66`, 1070.8`, 0.401463`, 1010.29`, 
              2227.41`, 604.412`, 1660.66`, 190.067`, 1610.61`, 1134.64`, 
              2216.64`, 2537.53`, 3007.55`, 2591.28`, 2971.44`, 2524.32`, 
              2601.45`, 2951.33`, 3032.57`, 535.266`, 1614.89`, 1068.66`, 
              2202.07`}, {2, 36, 29.58706619`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3210851293`, 0.6821396077`, 0.5631609837`, \(-0.4`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0669424`, 848.469`, 
              852.212`, 0.557698`, 921.713`, 922.539`, 0.133454`, 542.259`, 
              545.969`, 0.353769`, 596.866`, 605.909`, 0.953024`, 542.259`, 
              547.781`, 0.527351`, 1000.48`, 999.857`, \(-0.108735`\), 0, 
              92.8686`, 1.5635`, 921.713`, 923.632`, 0.310322`, 848.469`, 
              2289.96`, 596.866`, 1664.93`, 0, 1610.89`, 1000.48`, 2275.97`, 
              2581.09`, 3029.18`, 2640.35`, 2993.46`, 2566.91`, 2652.21`, 
              2965.17`, 3064.51`, 542.259`, 1612.01`, 921.713`, 
              2251.74`}, {3, 25, 28.15937643`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.4347452461`, 0.717048001`, 0.5548177239`, \(-0.4`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.00096243`, 844.769`, 
              850.231`, 0.811139`, 910.382`, 910.153`, \(-0.0364281`\), 
              530.285`, 539.79`, 0.891219`, 543.56`, 552.988`, 0.905773`, 
              543.56`, 551.153`, 0.728288`, 984.965`, 985.052`, 0.0150872`, 
              193.736`, 235.993`, 1.59105`, 910.382`, 912.909`, 0.403754`, 
              844.769`, 2305.81`, 530.285`, 1661.04`, 193.736`, 1579.65`, 
              984.965`, 2270.27`, 2488.28`, 2969.11`, 2551.29`, 2925.39`, 
              2472.41`, 2563.65`, 2902.2`, 2996.65`, 543.56`, 1591.77`, 
              910.382`, 2253.34`}, {3, 47, 26.21686701`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1796750458`, 0.3195199506`, 
              0.804187631`, \(-0.3`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.468812`\), 1608.57`, 
              1612.04`, 0.977836`, 1633.01`, 1635.94`, 0.8399`, 444.292`, 
              459.163`, 1.17715`, 642.168`, 647.031`, 0.549342`, 444.292`, 
              456.457`, 0.960053`, 1645.31`, 1650.33`, 1.45186`, 191.957`, 
              228.895`, 1.36204`, 1633.01`, 1636.81`, 1.08844`, 1608.57`, 
              2112.08`, 642.168`, 2216.95`, 191.957`, 2103.94`, 1645.31`, 
              2561.13`, 1985.64`, 2460.57`, 1993.61`, 2455.28`, 1983.5`, 
              1995.01`, 2453.51`, 2462.45`, 444.292`, 1992.73`, 1633.01`, 
              2044.05`}, {3, 13, 26.39078729`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1936566587`, 0.3564089841`, 
              0.7611370639`, \(-0.3`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.429803`\), 1583.85`, 
              1586.99`, 0.871795`, 1605.67`, 1608.57`, 0.816003`, 445.9`, 
              460.726`, 1.17772`, 642.166`, 646.995`, 0.545431`, 445.9`, 
              457.965`, 0.955502`, 1629.85`, 1632.52`, 0.763528`, 191.866`, 
              229.047`, 1.37119`, 1605.67`, 1608.89`, 0.905818`, 1583.85`, 
              2232.07`, 642.166`, 2180.08`, 191.866`, 2166.73`, 1629.85`, 
              2650.49`, 2080.45`, 2550.61`, 2091.37`, 2543.28`, 2077.59`, 
              2093.19`, 2540.84`, 2553.22`, 445.9`, 2090.07`, 1605.67`, 
              2157.91`}, {3, 40, 26.40393243`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2091616798`, 0.396773754`, 
              0.7152882417`, \(-0.3`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384343`\), 1560.98`, 
              1563.89`, 0.797863`, 1582.83`, 1585.6`, 0.770507`, 451.104`, 
              465.725`, 1.17446`, 644.687`, 649.456`, 0.540709`, 451.104`, 
              462.926`, 0.946745`, 1613.14`, 1615.24`, 0.592821`, 191.313`, 
              228.993`, 1.3876`, 1582.83`, 1585.77`, 0.815551`, 1560.98`, 
              2337.93`, 644.687`, 2165.83`, 191.313`, 2112.83`, 1613.14`, 
              2600.26`, 2159.69`, 2628.74`, 2174.83`, 2618.57`, 2155.84`, 
              2177.22`, 2615.16`, 2632.41`, 451.104`, 2172.79`, 1582.83`, 
              2253.86`}, {3, 34, 26.15638443`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2257996742`, 0.4380887504`, 
              0.6634954471`, \(-0.3`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32563`\), 1535.92`, 
              1538.64`, 0.732021`, 1559.18`, 1561.78`, 0.712406`, 461.162`, 
              475.302`, 1.1602`, 648.749`, 653.465`, 0.538144`, 461.162`, 
              472.517`, 0.928896`, 1594.6`, 1596.23`, 0.456077`, 190.213`, 
              228.698`, 1.41252`, 1559.18`, 1561.87`, 0.736913`, 1535.92`, 
              2442.61`, 648.749`, 2146.85`, 190.213`, 2094.8`, 1594.6`, 
              2600.97`, 2215.47`, 2687.66`, 2236.59`, 2673.49`, 2210.26`, 
              2239.74`, 2668.69`, 2692.92`, 461.162`, 2113.89`, 1559.18`, 
              2327.18`}, {3, 21, 25.40495877`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2433282825`, 0.4797613829`, 
              0.6043922139`, \(-0.3`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.237714`\), 1489.92`, 
              1492.63`, 0.708149`, 1520.44`, 1522.79`, 0.627467`, 481.054`, 
              494.208`, 1.12399`, 656.834`, 661.536`, 0.543137`, 481.054`, 
              491.558`, 0.895175`, 1565.57`, 1566.64`, 0.293911`, 187.753`, 
              227.742`, 1.4558`, 1520.44`, 1522.94`, 0.667693`, 1489.92`, 
              2498.24`, 656.834`, 2090.9`, 187.753`, 2030.2`, 1565.57`, 
              2527.47`, 2247.37`, 2726.74`, 2277.01`, 2706.92`, 2240.33`, 
              2281.21`, 2700, 2734.6`, 481.054`, 2051.48`, 1520.44`, 
              2491.11`}, {3, 26, 23.84402715`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2644014832`, 0.5255373782`, 0.5481391419`, \(-0.3`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0930116`\), 1354.7`, 
              1357.61`, 0.690933`, 1399.72`, 1401.67`, 0.477943`, 515.219`, 
              526.718`, 1.04974`, 668.555`, 673.602`, 0.593475`, 515.219`, 
              524.501`, 0.845519`, 1459.72`, 1460.1`, 0.0975691`, 183.523`, 
              226.221`, 1.53288`, 1399.72`, 1402.05`, 0.572449`, 1354.7`, 
              2291.67`, 668.555`, 1898.13`, 183.523`, 1836.96`, 1459.72`, 
              2300.77`, 2272.46`, 2760.77`, 2313.42`, 2733.58`, 2263.07`, 
              2319.18`, 2723.16`, 2773.41`, 515.219`, 1853.18`, 1399.72`, 
              2284.7`}, {3, 23, 22.75006481`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2978225357`, 0.5750077706`, 0.5101611544`, \(-0.3`\), 0.1`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.071631`, 1169.8`, 
              1172.82`, 0.6201`, 1222.51`, 1224.11`, 0.343767`, 541.472`, 
              551.071`, 0.91886`, 655.041`, 661.556`, 0.751598`, 541.472`, 
              549.813`, 0.797503`, 1288.97`, 1288.99`, 0.00322677`, 183.093`, 
              227.537`, 1.59901`, 1222.51`, 1224.7`, 0.469191`, 1169.8`, 
              2202.35`, 655.041`, 1725.36`, 183.093`, 1665.91`, 1288.97`, 
              2195.91`, 2293.9`, 2786.66`, 2346.08`, 2752.89`, 2281.92`, 
              2354.2`, 2736.94`, 2807.19`, 541.472`, 1673.44`, 1222.51`, 
              2181.5`}, {3, 33, 25.92878847`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3371052854`, 0.6078275869`, 0.4872634792`, \(-0.3`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.160288`, 1018.09`, 
              1021.25`, 0.565222`, 1070.36`, 1071.63`, 0.237398`, 533.989`, 
              541.04`, 0.664143`, 595.646`, 604.861`, 0.96931`, 533.989`, 
              541.808`, 0.736976`, 1135.43`, 1135.38`, \(-0.00905988`\), 
              192.896`, 233.895`, 1.53309`, 1070.36`, 1072.45`, 0.391764`, 
              1018.09`, 2204.03`, 595.646`, 1653.4`, 192.896`, 1588.09`, 
              1135.43`, 2187.6`, 2302.95`, 2792.61`, 2362.7`, 2755.72`, 
              2288.95`, 2373.16`, 2733.33`, 2820.99`, 533.989`, 1594.05`, 
              1070.36`, 2172.88`}, {3, 29, 28.1438379`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4276921031`, 0.6948470058`, 0.4624954651`, \(-0.3`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.094759`, 892.684`, 
              900.685`, 1.25726`, 947.488`, 944.468`, \(-0.500542`\), 
              530.51`, 540.052`, 0.89505`, 543.388`, 552.897`, 0.913285`, 
              543.388`, 551.108`, 0.74027`, 1019.66`, 
              1019.63`, \(-0.00581349`\), 193.428`, 235.873`, 1.59651`, 
              947.488`, 949.853`, 0.393117`, 892.684`, 2245.81`, 530.51`, 
              1643.17`, 193.428`, 1539.06`, 1019.66`, 2206.95`, 2249.34`, 
              2749.65`, 2317.97`, 2705.71`, 2232.86`, 2330.36`, 2680.63`, 
              2780.34`, 543.388`, 1552.39`, 947.488`, 2191.66`}, {3, 42, 
              34.5336063`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3636125331`, 0.7331054693`, 0.4524853262`, \(-0.3`\), 0.4`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.013926`\), 795.505`, 
              806.986`, 1.61198`, 855.31`, 848.141`, \(-1.06995`\), 560.633`, 
              567.416`, 0.670354`, 576.808`, 582.919`, 0.620953`, 576.808`, 
              581.219`, 0.447503`, 929.883`, 929.043`, \(-0.136807`\), 
              195.331`, 235.397`, 1.51203`, 855.31`, 856.949`, 0.245795`, 
              795.505`, 2346.57`, 560.633`, 1706.69`, 195.331`, 1623.35`, 
              929.883`, 2310.68`, 2352.82`, 2802.64`, 2421.97`, 2764.72`, 
              2336.55`, 2436.44`, 2726.11`, 2848.56`, 576.808`, 1626.41`, 
              855.31`, 2283.35`}, {1, 77, 35.74751313`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5314504922`, 0.8700987377`, 0.4266936727`, \(-0.3`\), 0.5`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.391972`\), 752.475`, 
              762.94`, 1.3895`, 804.243`, 798.291`, \(-0.835758`\), 449.427`, 
              460.109`, 0.851311`, 582.674`, 590.41`, 0.795085`, 582.674`, 
              587.479`, 0.492549`, 886.808`, 886.008`, \(-0.124161`\), 
              198.953`, 239.578`, 1.56092`, 804.243`, 806.283`, 0.287814`, 
              752.475`, 2411.97`, 449.427`, 1785.09`, 198.953`, 1612.56`, 
              886.808`, 2335.45`, 2220.41`, 2714.41`, 2298.8`, 2664.05`, 
              2201.47`, 2312.94`, 2632.65`, 2749.7`, 582.674`, 1633.27`, 
              804.243`, 2326.33`}, {2, 26, 28.33016243`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1551279279`, 0.2154371855`, 
              0.7487065457`, \(-0.2`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.410097`\), 1603.65`, 
              1608.87`, 1.46895`, 1611.49`, 1609.94`, \(-0.436787`\), 
              409.639`, 423.611`, 1.02007`, 575.968`, 581.557`, 0.566827`, 
              409.639`, 421.512`, 0.864569`, 1610.19`, 1617.13`, 1.96377`, 
              199.759`, 230.91`, 1.17544`, 1611.49`, 
              1609.73`, \(-0.498225`\), 1603.65`, 1677.53`, 575.968`, 
              1940.01`, 199.759`, 1692.42`, 1610.19`, 2180.25`, 1613.02`, 
              2120.13`, 1617.57`, 2117.44`, 1611.65`, 1618.51`, 2116.51`, 
              2121.12`, 409.639`, 1617.08`, 1611.49`, 1640.68`}, {3, 17, 
              26.51499893`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1949038817`, 0.3386522358`, 
              0.7301063951`, \(-0.2`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.443935`\), 1604.23`, 
              1607.9`, 1.03337`, 1629.52`, 1632.49`, 0.84796`, 447.333`, 
              462.526`, 1.21117`, 651.941`, 656.705`, 0.546252`, 447.333`, 
              459.519`, 0.968199`, 1638.73`, 1647.67`, 2.57393`, 190.973`, 
              228.602`, 1.38331`, 1629.52`, 1633.76`, 1.21168`, 1604.23`, 
              2087.66`, 651.941`, 2191.36`, 190.973`, 2078.02`, 1638.73`, 
              2537.06`, 1942.95`, 2422.91`, 1953.59`, 2415.93`, 1940.09`, 
              1955.44`, 2413.57`, 2425.43`, 447.333`, 1952.35`, 1629.52`, 
              2009.16`}, {3, 13, 26.54960679`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096207049`, 0.3653694643`, 
              0.6723708579`, \(-0.2`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.384043`\), 1571.94`, 
              1575.09`, 0.86814`, 1592.24`, 1595.24`, 0.837159`, 443.913`, 
              458.933`, 1.18814`, 641.232`, 646.077`, 0.546376`, 443.913`, 
              455.923`, 0.946891`, 1612.54`, 1615.74`, 0.905295`, 192.107`, 
              229.129`, 1.36637`, 1592.24`, 1595.64`, 0.949652`, 1571.94`, 
              2162.69`, 641.232`, 2152.1`, 192.107`, 2119.63`, 1612.54`, 
              2601.85`, 1997.38`, 2476.61`, 2011.84`, 2467.02`, 1993.54`, 
              2014.31`, 2463.76`, 2480.11`, 443.913`, 2010, 1592.24`, 
              2081.45`}, {3, 30, 26.56192936`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2308971884`, 0.4099250503`, 
              0.6108696233`, \(-0.2`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.321356`\), 1541.6`, 
              1544.38`, 0.750854`, 1560.53`, 1563.44`, 0.796308`, 450.332`, 
              465.106`, 1.18497`, 643.976`, 648.752`, 0.54095`, 450.332`, 
              462.001`, 0.932747`, 1588.65`, 1590.91`, 0.629582`, 191.471`, 
              229.063`, 1.3851`, 1560.53`, 1563.5`, 0.812745`, 1541.6`, 
              2249.11`, 643.976`, 2120.53`, 191.471`, 2065.61`, 1588.65`, 
              2543.66`, 2050.53`, 2531.81`, 2071.82`, 2517.68`, 2045.05`, 
              2075.27`, 2512.83`, 2537.09`, 450.332`, 2068.73`, 1560.53`, 
              2156.66`}, {3, 23, 26.12136428`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2572047293`, 0.4602084662`, 
              0.5320222209`, \(-0.2`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.224494`\), 1496.97`, 
              1499.49`, 0.661754`, 1518.97`, 1521.69`, 0.723195`, 466.572`, 
              480.577`, 1.16219`, 650.837`, 655.509`, 0.534842`, 466.572`, 
              477.443`, 0.899138`, 1556.66`, 1558.21`, 0.424532`, 189.564`, 
              228.328`, 1.4193`, 1518.97`, 1521.63`, 0.70797`, 1496.97`, 
              2331.73`, 650.837`, 2081.28`, 189.564`, 2008.96`, 1556.66`, 
              2526.99`, 2062.17`, 2552.36`, 2094.74`, 2530.97`, 2054.18`, 
              2099.7`, 2523.43`, 2560.91`, 466.572`, 2036.14`, 1518.97`, 
              2195.61`}, {3, 26, 23.72891556`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2950641059`, 0.5445215478`, 0.4283751388`, \(-0.2`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.00243022`\), 
              1294.37`, 1297.1`, 0.619085`, 1336.09`, 1338.14`, 0.481073`, 
              523.327`, 534.74`, 1.058`, 676.007`, 680.923`, 0.584408`, 
              523.327`, 532.115`, 0.812669`, 1399.67`, 1399.99`, 0.0789593`, 
              181.414`, 224.637`, 1.53772`, 1336.09`, 1338.49`, 0.563117`, 
              1294.37`, 2210.88`, 676.007`, 1818.21`, 181.414`, 1738.92`, 
              1399.67`, 2216.38`, 2041.36`, 2544.49`, 2095.5`, 2511.08`, 
              2029.37`, 2103.11`, 2496.88`, 2562.61`, 523.327`, 1758.71`, 
              1336.09`, 2185.26`}, {3, 25, 28.27663806`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4758561769`, 0.7160187793`, 0.339883876`, \(-0.2`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.170517`, 956.409`, 
              973.865`, 2.95224`, 993.91`, 981.342`, \(-2.17515`\), 540.195`, 
              550.159`, 0.951898`, 551.957`, 561.809`, 0.961397`, 551.957`, 
              559.782`, 0.762226`, 1070.26`, 1070.05`, \(-0.0396355`\), 
              188.743`, 233.174`, 1.64247`, 993.91`, 996.327`, 0.421501`, 
              956.409`, 2165.61`, 540.195`, 1658.62`, 188.743`, 1489.56`, 
              1070.26`, 2130.41`, 1933.76`, 2455.18`, 2015.77`, 2408.4`, 
              1915.35`, 2028.61`, 2380.88`, 2487.89`, 551.957`, 1514.2`, 
              993.91`, 2099.47`}, {3, 18, 36.30581695`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.899928289`, 0.4338763815`, 0.6880479569`, \(-0.2`\), 0.2`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.047081`\), 1521.93`, 
              1525.43`, 0.934335`, 1525.89`, 1534.29`, 2.25362`, 392.2`, 
              410.08`, 1.25682`, 526.527`, 533.004`, 0.601249`, 392.2`, 
              403.735`, 0.80442`, 1543.34`, 1547.45`, 1.11175`, 204.218`, 
              239.928`, 1.38965`, 1521.93`, 1525.86`, 1.04767`, 1525.89`, 
              2128.2`, 526.527`, 2039.81`, 204.218`, 1924.37`, 1543.34`, 
              2467.4`, 1870.17`, 2466.95`, 1916.36`, 2428.09`, 1857.61`, 
              1926.33`, 2419.59`, 2475.49`, 392.2`, 1898.7`, 1521.93`, 
              2015.9`}, {3, 29, 36.6049871`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9864421313`, 0.4067884333`, 0.7973431258`, \(-0.2`\), 0.3`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0856974`\), 
              1570.29`, 1574.56`, 1.17464`, 1572.16`, 1581.7`, 2.63533`, 
              389.393`, 407.212`, 1.24368`, 521.524`, 528.044`, 0.599567`, 
              389.393`, 401.01`, 0.804475`, 1586.35`, 1591.58`, 1.45599`, 
              204.742`, 240.287`, 1.38599`, 1570.29`, 1575.23`, 1.35982`, 
              1572.16`, 2196.97`, 521.524`, 2117.46`, 204.742`, 2023.18`, 
              1586.35`, 2594.27`, 1976.04`, 2558.72`, 2017.32`, 2521.94`, 
              1965.92`, 2025.52`, 2515.23`, 2565.42`, 389.393`, 2001.83`, 
              1570.29`, 2098.16`}, {3, 33, 36.85548356`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.058788281`, 0.3925204838`, 0.8884413581`, \(-0.2`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.106687`\), 1600.2`, 
              1605.82`, 1.58113`, 1601.68`, 1611.69`, 2.81888`, 387.608`, 
              405.393`, 1.23568`, 517.624`, 524.201`, 0.600395`, 387.608`, 
              399.265`, 0.803657`, 1612.05`, 1618.41`, 1.79922`, 205.164`, 
              240.599`, 1.38397`, 1600.2`, 1606.17`, 1.67971`, 1601.68`, 
              2271.55`, 517.624`, 2171.29`, 205.164`, 2098.8`, 1612.05`, 
              2721.78`, 2074.27`, 2631.77`, 2112.56`, 2595.88`, 2065.74`, 
              2119.54`, 2590.28`, 2637.36`, 387.608`, 2091.58`, 1600.2`, 
              2181.58`}, {3, 41, 37.12415075`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.122490999`, 0.3841540044`, 0.9680356541`, \(-0.2`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.117779`\), 1619.61`, 
              1626.99`, 2.09832`, 1621.34`, 1631.85`, 2.99408`, 386.276`, 
              404.045`, 1.23043`, 514.528`, 521.158`, 0.601701`, 386.276`, 
              397.953`, 0.802372`, 1628.33`, 1635.98`, 2.18829`, 205.529`, 
              240.872`, 1.38235`, 1619.61`, 1626.82`, 2.04999`, 1621.34`, 
              2346.6`, 514.528`, 2204.58`, 205.529`, 2151.33`, 1628.33`, 
              2832.82`, 2165.39`, 2688.24`, 2201.51`, 2652.69`, 2157.98`, 
              2207.6`, 2647.86`, 2693.06`, 386.276`, 2156.49`, 1619.61`, 
              2262.8`}, {2, 34, 26.08454411`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1719761912`, 0.2526820847`, 
              0.7500233823`, \(-0.1`\), \(-0.5`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.464253`\), 1626.13`, 
              1630.19`, 1.16003`, 1632.34`, 1633.87`, 0.43828`, 433.649`, 
              448.476`, 1.14589`, 627.294`, 632.409`, 0.564609`, 433.649`, 
              445.764`, 0.933436`, 1636.7`, 1639.51`, 0.806398`, 193.924`, 
              229.034`, 1.30111`, 1634.07`, 1631.27`, \(-0.803078`\), 
              1626.13`, 1727.45`, 627.294`, 1982.93`, 193.924`, 1736.62`, 
              1636.7`, 2217.37`, 1634.07`, 2138.1`, 1640.02`, 2134.6`, 
              1632.34`, 1641.19`, 2133.39`, 2139.39`, 433.649`, 1639.38`, 
              1634.96`, 1676.96`}, {3, 34, 27.58883592`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1734611306`, 0.2412676182`, 
              0.6773872263`, \(-0.1`\), \(-0.4`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.398737`\), 1604.12`, 
              1605.65`, 0.430977`, 1607.68`, 1610.3`, 0.740856`, 416.325`, 
              430.638`, 1.06213`, 592.822`, 598.163`, 0.557273`, 416.325`, 
              428.172`, 0.876583`, 1608.84`, 1615.62`, 1.91476`, 198.098`, 
              230.321`, 1.20954`, 1609.65`, 1606.92`, \(-0.768764`\), 
              1604.12`, 1691.65`, 592.822`, 1946.87`, 198.098`, 1708.25`, 
              1608.84`, 2192.09`, 1609.65`, 2117.68`, 1616.29`, 2113.8`, 
              1607.68`, 1617.62`, 2112.45`, 2119.12`, 416.325`, 1615.54`, 
              1611.38`, 1645.91`}, {1, 13, 28.12048177`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2232059941`, 0.3863091699`, 
              0.6520758487`, \(-0.1`\), \(-0.3`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.424683`\), 1602.48`, 
              1606.46`, 1.11756`, 1627.2`, 1630.35`, 0.897757`, 460.585`, 
              476.603`, 1.31533`, 683.998`, 688.519`, 0.543745`, 460.585`, 
              472.942`, 1.0107`, 1639.48`, 1653.84`, 4.14392`, 187.489`, 
              227.867`, 1.46946`, 1627.2`, 1632, 1.37108`, 1602.48`, 2113.9`, 
              683.998`, 2185.92`, 187.489`, 2087.44`, 1639.48`, 2553.38`, 
              1924.67`, 2409.16`, 1941.04`, 2398.56`, 1920.36`, 1943.77`, 
              2394.94`, 2413.05`, 460.585`, 1938.98`, 1627.2`, 2015.39`}, {3, 
              21, 26.90029993`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2369967722`, 0.3767948556`, 
              0.5633678111`, \(-0.1`\), \(-0.2`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.318103`\), 1556.79`, 
              1560.21`, 0.934293`, 1575.71`, 1578.81`, 0.857119`, 441.369`, 
              456.722`, 1.20805`, 640.332`, 645.202`, 0.548624`, 441.369`, 
              453.278`, 0.933444`, 1587.21`, 1593.7`, 1.80826`, 192.48`, 
              229.372`, 1.36358`, 1575.71`, 1579.77`, 1.12295`, 1556.79`, 
              2064.19`, 640.332`, 2088.52`, 192.48`, 2037.89`, 1587.21`, 
              2516.04`, 1880.22`, 2374.19`, 1900.89`, 2360.82`, 1874.62`, 
              1904.53`, 2356.16`, 2379.22`, 441.369`, 1898.01`, 1575.71`, 
              1971.92`}, {3, 17, 27.12894621`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2809249479`, 0.4283042823`, 
              0.4628040945`, \(-0.1`\), \(-0.1`\), 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.203502`\), 1504.43`, 
              1506.92`, 0.655964`, 1517.46`, 1520.78`, 0.882981`, 447.852`, 
              462.949`, 1.20477`, 640.332`, 645.11`, 0.53815`, 447.852`, 
              459.159`, 0.898525`, 1540.17`, 1543.17`, 0.810383`, 192.232`, 
              229.519`, 1.37785`, 1517.46`, 1520.6`, 0.834841`, 1504.43`, 
              2095.16`, 640.332`, 2013.6`, 192.232`, 1948.97`, 1540.17`, 
              2458.14`, 1863.72`, 2370.99`, 1898.26`, 2348.84`, 1854.68`, 
              1904.13`, 2341, 2379.8`, 447.852`, 1892.38`, 1517.46`, 
              1990.06`}, {3, 48, 32.9391364`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.5921448625`, 0.4493266263`, 0.4347804085`, \(-0.1`\), 0, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 0.0244069`, 1444.85`, 
              1448.33`, 0.882521`, 1452.36`, 1455.89`, 0.899038`, 407.27`, 
              424.899`, 1.28535`, 557.603`, 564.166`, 0.645038`, 407.27`, 
              419.1`, 0.856578`, 1464.77`, 1467.8`, 0.780558`, 203.21`, 
              239.135`, 1.39236`, 1444.85`, 1447.55`, 0.683429`, 1452.36`, 
              1977.34`, 557.603`, 1916.28`, 203.21`, 1794.07`, 1464.77`, 
              2313.45`, 1701.49`, 2275.47`, 1756.61`, 2234.82`, 1685.49`, 
              1768.83`, 2223.15`, 2287.99`, 407.27`, 1741.46`, 1444.85`, 
              1861.35`}, {3, 22, 34.95715651`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7997214319`, 0.402110307`, 0.6201133992`, \(-0.1`\), 0.1`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.0583885`\), 
              1529.54`, 1533.9`, 1.17069`, 1534.38`, 1541.23`, 1.84724`, 
              397.275`, 415.046`, 1.26478`, 541.415`, 547.741`, 0.603685`, 
              397.275`, 408.84`, 0.816778`, 1549.12`, 1554.22`, 1.38628`, 
              203.2`, 238.873`, 1.38174`, 1529.54`, 1533.61`, 1.09318`, 
              1534.38`, 2025.77`, 541.415`, 2031.84`, 203.2`, 1905, 1549.12`, 
              2416.02`, 1789.76`, 2388.5`, 1835.19`, 2350.94`, 1777.25`, 
              1845.25`, 2342.53`, 2397.01`, 397.275`, 1825.2`, 1529.54`, 
              1924.14`}, {3, 18, 35.60338086`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9111608864`, 0.3804715923`, 0.7452435318`, \(-0.1`\), 0.2`, 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.107562`\), 1581.2`, 
              1586.36`, 1.4309`, 1583.74`, 1592.41`, 2.41262`, 394.575`, 
              412.255`, 1.24987`, 533.908`, 540.265`, 0.598252`, 394.575`, 
              406.136`, 0.811113`, 1596.2`, 1602.6`, 1.79211`, 203.801`, 
              239.414`, 1.38296`, 1581.2`, 1586.55`, 1.48336`, 1583.74`, 
              2107.68`, 533.908`, 2117.81`, 203.801`, 2008.78`, 1596.2`, 
              2525.89`, 1902.79`, 2495.76`, 1943.51`, 2460.01`, 1892.82`, 
              1951.67`, 2453.47`, 2502.3`, 394.575`, 1935.38`, 1581.2`, 
              2016.69`}, {3, 25, 36.06342558`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9955469967`, 0.36990087`, 0.8449390205`, \(-0.1`\), 0.3`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.133719`\), 1612.31`, 
              1619.12`, 1.92741`, 1614.5`, 1623.86`, 2.65343`, 392.561`, 
              410.198`, 1.24051`, 528.363`, 534.782`, 0.597916`, 392.561`, 
              404.133`, 0.8078`, 1622.85`, 1630.51`, 2.184`, 204.323`, 
              239.859`, 1.38298`, 1612.31`, 1618.95`, 1.87977`, 1614.5`, 
              2192.33`, 528.363`, 2175.3`, 204.323`, 2091.9`, 1622.85`, 
              2620.39`, 2008.72`, 2582.55`, 2046.63`, 2547.55`, 2000.38`, 
              2053.53`, 2542.14`, 2587.95`, 392.561`, 2039.05`, 1612.31`, 
              2108.87`}, {3, 29, 36.49663584`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.066722691`, 0.3642605793`, 0.9300350116`, \(-0.1`\), 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.14737`\), 1631.82`, 
              1640.65`, 2.5324`, 1634.33`, 1644.27`, 2.85438`, 390.958`, 
              408.579`, 1.23441`, 524.114`, 530.593`, 0.598722`, 390.958`, 
              402.537`, 0.805025`, 1639.05`, 1648.09`, 2.60587`, 204.772`, 
              240.227`, 1.38238`, 1631.82`, 1639.87`, 2.30816`, 1634.33`, 
              2274.63`, 524.114`, 2210.24`, 204.772`, 2154.12`, 1639.05`, 
              2720.52`, 2105.67`, 2649.84`, 2141.57`, 2615.09`, 2098.45`, 
              2147.57`, 2610.43`, 2654.49`, 390.958`, 2133.1`, 1631.82`, 
              2196.59`}, {3, 25, 36.97288006`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.129889073`, 0.3611097449`, 1.005398301`, \(-0.1`\), 0.5`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153522`\), 1645.25`, 
              1656.27`, 3.18724`, 1648.24`, 1659.05`, 3.13325`, 389.714`, 
              407.332`, 1.23037`, 520.861`, 527.39`, 0.599665`, 389.714`, 
              401.294`, 0.802534`, 1650.14`, 1660.75`, 3.07912`, 205.141`, 
              240.527`, 1.38173`, 1645.25`, 1654.89`, 2.78811`, 1648.24`, 
              2353.92`, 520.861`, 2227.7`, 205.141`, 2193.64`, 1650.14`, 
              2821.76`, 2195.24`, 2701.03`, 2229.5`, 2666.26`, 2188.84`, 
              2234.83`, 2662.14`, 2705.14`, 389.714`, 2213.45`, 1645.25`, 
              2279.82`}, {3, 37, 25.95334978`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1780157057`, 0.2545579208`, 0.7416157605`, 0, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.481203`\), 1561.22`, 
              1560.86`, \(-0.0982758`\), 1570.39`, 1570.23`, \(-0.0420493`\), 
              438.676`, 453.762`, 1.17963`, 640.641`, 645.624`, 0.561537`, 
              438.676`, 450.83`, 0.947304`, 1568.51`, 1568.77`, 0.0738347`, 
              192.452`, 228.438`, 1.32704`, 1563.03`, 
              1562.45`, \(-0.158996`\), 1590.55`, 1701.38`, 640.641`, 
              1943.22`, 192.452`, 1671.32`, 1636.18`, 2157.51`, 1563.03`, 
              2074.13`, 1569.18`, 2070.63`, 1561.22`, 1570.39`, 2069.41`, 
              2075.43`, 438.676`, 1568.51`, 1582.27`, 1662.63`}, {3, 37, 
              26.24850993`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.192760654`, 0.2821307923`, 0.6786643596`, 0, \(-0.4`\), 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.447882`\), 1626.18`, 
              1627.56`, 0.391648`, 1629.39`, 1634.75`, 1.53193`, 440.148`, 
              455.464`, 1.20186`, 645.942`, 650.82`, 0.554161`, 440.148`, 
              452.291`, 0.949486`, 1634.76`, 1639.68`, 1.4121`, 191.919`, 
              228.374`, 1.34243`, 1631.88`, 1629.28`, \(-0.74497`\), 
              1626.18`, 1748.25`, 645.942`, 1994.52`, 191.919`, 1758.16`, 
              1634.76`, 2234.12`, 1631.88`, 2136.76`, 1640.52`, 2131.74`, 
              1629.39`, 1642.16`, 2129.98`, 2138.64`, 440.148`, 1639.52`, 
              1635.53`, 1685.02`}, {3, 21, 27.04715719`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2017406863`, 0.2747645514`, 0.5963250204`, 0, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.375129`\), 1596.71`, 
              1595.27`, \(-0.403253`\), 1601.42`, 1606.61`, 1.46098`, 
              423.631`, 438.59`, 1.13003`, 612.825`, 617.986`, 0.556504`, 
              423.631`, 435.559`, 0.897915`, 1604.61`, 1609.08`, 1.25781`, 
              195.944`, 229.766`, 1.26155`, 1599.79`, 
              1597.57`, \(-0.620971`\), 1601.42`, 1709.83`, 612.825`, 
              1957.47`, 195.944`, 1727.37`, 1604.61`, 2205.91`, 1599.79`, 
              2110.66`, 1610.21`, 2104.65`, 1596.71`, 1612.27`, 2102.53`, 
              2112.92`, 423.631`, 1608.94`, 1607.15`, 1651.32`}, {3, 25, 
              29.55478551`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2164391544`, 0.2732926492`, 0.4974735395`, 0, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.290259`\), 1554.17`, 
              1553.12`, \(-0.286762`\), 1565.73`, 1568.68`, 0.809267`, 
              406.808`, 421.059`, 1.03368`, 580.39`, 585.486`, 0.520617`, 
              406.808`, 418.122`, 0.817756`, 1561.64`, 1570.25`, 2.36325`, 
              199.522`, 230.214`, 1.15564`, 1558.26`, 1556.4`, \(-0.50827`\), 
              1565.73`, 1665.2`, 580.39`, 1894.79`, 199.522`, 1692.27`, 
              1561.64`, 2172.29`, 1558.26`, 2075.77`, 1571.98`, 2068.09`, 
              1554.17`, 1574.7`, 2065.36`, 2078.69`, 406.808`, 1570.13`, 
              1569.01`, 1608.11`}, {3, 22, 33.69132999`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3614936416`, 0.3937945667`, 0.4687962874`, 0, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.199894`\), 1557.15`, 
              1562.51`, 1.46408`, 1564.57`, 1568.44`, 1.06239`, 428.608`, 
              447.361`, 1.43925`, 614.708`, 621.963`, 0.786113`, 428.608`, 
              442.486`, 1.05922`, 1563.86`, 1588.32`, 6.75589`, 195.649`, 
              236.169`, 1.53306`, 1564.57`, 1571.13`, 1.80212`, 1557.15`, 
              1961.8`, 614.708`, 2012.81`, 195.649`, 1994.41`, 1563.86`, 
              2420.35`, 1755.04`, 2287.64`, 1789.8`, 2263.32`, 1744.55`, 
              1797.67`, 2255.42`, 2296.18`, 428.608`, 1784.34`, 1564.57`, 
              1860.36`}, {3, 26, 33.32775482`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.6623453098`, 0.3754364803`, 0.5267070012`, 0, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.073551`\), 1532.48`, 
              1536.38`, 1.04744`, 1537.64`, 1543.57`, 1.60261`, 405.156`, 
              422.787`, 1.27896`, 562.81`, 568.916`, 0.605456`, 405.156`, 
              416.703`, 0.831442`, 1547.48`, 1556.19`, 2.36693`, 201.731`, 
              237.473`, 1.3754`, 1532.48`, 1536.6`, 1.10745`, 1537.64`, 
              1907.44`, 562.81`, 2008.3`, 201.731`, 1869.27`, 1547.48`, 
              2337.52`, 1689.13`, 2280.14`, 1733.84`, 2244.68`, 1676.34`, 
              1744.2`, 2236.07`, 2288.99`, 405.156`, 1725.73`, 1532.48`, 
              1813.19`}, {3, 17, 34.64549012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8236493275`, 0.3570706905`, 0.6813944266`, 0, 0.1`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.130334`\), 1589.18`, 
              1594.49`, 1.47909`, 1592.24`, 1600.51`, 2.31477`, 400.37`, 
              417.919`, 1.25818`, 547.737`, 553.907`, 0.595553`, 400.37`, 
              411.858`, 0.81755`, 1602.76`, 1612.1`, 2.63051`, 202.7`, 
              238.419`, 1.38051`, 1589.18`, 1594.82`, 1.5713`, 1592.24`, 
              2006.34`, 547.737`, 2112.12`, 202.7`, 1968.29`, 1602.76`, 
              2456.64`, 1813.1`, 2413.43`, 1853.41`, 2378.82`, 1803.07`, 
              1861.69`, 2372.31`, 2419.99`, 400.37`, 1846.78`, 1589.18`, 
              1920.9`}, {3, 73, 35.39778692`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9260006591`, 0.3494504171`, 0.7944149337`, 0, 0.2`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.162398`\), 1623.46`, 
              1631.27`, 2.22654`, 1626.32`, 1635.43`, 2.60321`, 397.803`, 
              415.296`, 1.24624`, 539.722`, 545.963`, 0.593657`, 397.803`, 
              409.278`, 0.811427`, 1632, 1642.46`, 3.00068`, 203.398`, 
              239.064`, 1.38268`, 1623.46`, 1630.99`, 2.14597`, 1626.32`, 
              2104.27`, 539.722`, 2176.71`, 203.398`, 2064.27`, 1632, 
              2556.38`, 1931.45`, 2519.73`, 1969.07`, 2485.68`, 1923.17`, 
              1975.97`, 2480.36`, 2525.05`, 397.803`, 1963.24`, 1623.46`, 
              2025.62`}, {3, 39, 36.04480578`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              1.00695157`, 0.3461612174`, 0.8872255019`, 0, 0.3`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.178783`\), 1643.81`, 
              1654.28`, 3.02679`, 1647.22`, 1656.92`, 2.80659`, 395.816`, 
              413.292`, 1.23893`, 534.044`, 540.355`, 0.594055`, 395.816`, 
              407.287`, 0.807164`, 1648.79`, 1660.48`, 3.3904`, 203.948`, 
              239.545`, 1.38321`, 1643.81`, 1653.1`, 2.68439`, 1647.22`, 
              2196.15`, 534.044`, 2214.98`, 203.948`, 2146.98`, 1648.79`, 
              2634.94`, 2037.48`, 2601.72`, 2073.21`, 2567.8`, 2030.36`, 
              2079.16`, 2563.25`, 2606.26`, 395.816`, 2067.72`, 1643.81`, 
              2122.64`}, {3, 25, 36.70190339`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.076545803`, 0.3448022844`, 0.967703659`, 0, 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.186189`\), 1657.08`, 
              1670.11`, 3.79995`, 1661.1`, 1671.7`, 3.09451`, 394.234`, 
              411.711`, 1.23411`, 529.758`, 536.129`, 0.595005`, 394.234`, 
              405.702`, 0.803739`, 1659.79`, 1672.89`, 3.82636`, 204.397`, 
              239.923`, 1.38305`, 1657.08`, 1668.15`, 3.22389`, 1661.1`, 
              2282.29`, 529.758`, 2234.08`, 204.397`, 2212.58`, 1659.79`, 
              2702.73`, 2133.36`, 2663.82`, 2167.57`, 2629.79`, 2127.08`, 
              2172.83`, 2625.79`, 2667.81`, 394.234`, 2161.99`, 1657.08`, 
              2212.54`}, {2, 22, 37.42798211`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.138872013`, 0.3445142349`, 1.039635558`, 0, 0.5`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.188037`\), 1667.18`, 
              1682.68`, 4.55155`, 1671.69`, 1683.58`, 3.49622`, 393.055`, 
              410.539`, 1.231`, 526.577`, 532.994`, 0.59575`, 393.055`, 
              404.513`, 0.800722`, 1668.39`, 1683.11`, 4.32148`, 204.74`, 
              240.214`, 1.38294`, 1667.18`, 1680.11`, 3.79186`, 1671.69`, 
              2363.82`, 526.577`, 2238.74`, 204.74`, 2255.21`, 1668.39`, 
              2782.92`, 2221.67`, 2710.44`, 2254.53`, 2676.18`, 2216, 
              2259.27`, 2672.58`, 2714.03`, 393.055`, 2247.83`, 1667.18`, 
              2296.75`}, {2, 41, 25.96580932`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1831086754`, 0.251205778`, 0.7343728558`, 0.1`, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.486362`\), 1489.52`, 
              1489.61`, 0.0244255`, 1498.79`, 1498.8`, 0.00422369`, 438.438`, 
              453.66`, 1.18984`, 642.688`, 647.665`, 0.562586`, 438.438`, 
              450.617`, 0.948716`, 1496.87`, 1497.18`, 0.0817805`, 192.281`, 
              228.365`, 1.32993`, 1491.36`, 1491.41`, 0.0132971`, 1539.4`, 
              1679.59`, 642.688`, 1901.51`, 192.281`, 1602.51`, 1627.99`, 
              2096.46`, 1491.36`, 2011.71`, 1497.55`, 2008.29`, 1489.52`, 
              1498.79`, 2007.09`, 2012.98`, 438.438`, 1496.87`, 1520.03`, 
              1648.32`}, {2, 38, 26.23439234`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1955529823`, 0.2654909245`, 0.6650560358`, 0.1`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.447971`\), 1513.95`, 
              1513.96`, 0.00286528`, 1526.07`, 1526.05`, \(-0.00640337`\), 
              436.036`, 451.4`, 1.19458`, 640.062`, 645.05`, 0.561722`, 
              436.036`, 448.189`, 0.941498`, 1523.54`, 1523.85`, 0.0828431`, 
              192.702`, 228.61`, 1.32551`, 1516.36`, 
              1516.26`, \(-0.0261082`\), 1562.09`, 1686.65`, 640.062`, 
              1921.58`, 192.702`, 1639.49`, 1623.49`, 2127.56`, 1516.36`, 
              2034.47`, 1524.46`, 2029.96`, 1513.95`, 1526.07`, 2028.37`, 
              2036.16`, 436.036`, 1523.54`, 1545.19`, 1648.68`}, {2, 37, 
              26.64150465`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2150272632`, 0.2849855398`, 0.5875103847`, 0.1`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.396101`\), 1537.68`, 
              1537.53`, \(-0.0421644`\), 1554.68`, 1554.56`, \(-0.0304873`\), 
              432.993`, 448.551`, 1.20167`, 636.51`, 641.513`, 0.560221`, 
              432.993`, 445.085`, 0.930253`, 1551.04`, 1551.34`, 0.079975`, 
              193.242`, 228.951`, 1.3209`, 1541.06`, 1540.65`, \(-0.11025`\), 
              1580.6`, 1697.23`, 636.51`, 1942.55`, 193.242`, 1682.21`, 
              1612.74`, 2163.48`, 1541.06`, 2058.38`, 1552.43`, 2052.01`, 
              1537.68`, 1554.68`, 2049.76`, 2060.78`, 432.993`, 1551.04`, 
              1568.71`, 1647.38`}, {1, 21, 27.34014421`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2553684009`, 0.3243325355`, 0.5000972694`, 0.1`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323175`\), 1574.8`, 
              1573.81`, \(-0.272979`\), 1593.86`, 1596.77`, 0.81392`, 
              434.263`, 450.357`, 1.24743`, 642.469`, 647.412`, 0.558732`, 
              434.263`, 446.277`, 0.92688`, 1594.23`, 1596.72`, 0.695192`, 
              192.57`, 229.094`, 1.34936`, 1580.34`, 
              1578.44`, \(-0.523668`\), 1593.86`, 1739.65`, 642.469`, 
              1979.35`, 192.57`, 1759.02`, 1594.23`, 2228.84`, 1580.34`, 
              2099.4`, 1599.01`, 2088.7`, 1574.8`, 1602.7`, 2084.92`, 
              2103.44`, 434.263`, 1596.5`, 1596.6`, 1659.54`}, {1, 40, 
              30.1932564`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.3771860313`, 0.3352920112`, 0.379494855`, 0.1`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.153249`\), 1498.38`, 
              1497.89`, \(-0.127452`\), 1533.15`, 1535.34`, 0.58849`, 
              419.824`, 435.544`, 1.17807`, 609.38`, 614, 0.495283`, 
              419.824`, 430.458`, 0.792184`, 1529.63`, 1537.32`, 2.06569`, 
              201.341`, 234.226`, 1.25498`, 1508.65`, 
              1506.69`, \(-0.516005`\), 1540.29`, 1688.94`, 609.38`, 
              1932.65`, 201.341`, 1734.36`, 1529.63`, 2192.94`, 1508.65`, 
              2063.56`, 1542.21`, 2042.99`, 1498.38`, 1549.72`, 2036.45`, 
              2070.63`, 419.824`, 1537.05`, 1533.15`, 1605.91`}, {3, 47, 
              34.35651896`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7092899813`, 0.3401057759`, 0.5943691092`, 0.1`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.149103`\), 1592.17`, 
              1592.24`, 0.0190755`, 1593.66`, 1602.99`, 2.61367`, 408.682`, 
              426.168`, 1.27908`, 568.811`, 574.688`, 0.588831`, 408.682`, 
              420.076`, 0.827374`, 1602.27`, 1623.44`, 5.9852`, 199.518`, 
              235.849`, 1.38585`, 1592.17`, 1594.92`, 0.768069`, 1593.66`, 
              1888.19`, 568.811`, 2093.79`, 199.518`, 1889.45`, 1602.27`, 
              2362.3`, 1698.26`, 2299.44`, 1738.46`, 2266.3`, 1687.7`, 
              1747.21`, 2259.48`, 2306.38`, 408.682`, 1732.44`, 1592.17`, 
              1802.51`}, {3, 13, 35.06524039`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8458646265`, 0.3316392679`, 0.7327823772`, 0.1`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.190092`\), 1634.82`, 
              1640.61`, 1.65944`, 1636.64`, 1646.49`, 2.83127`, 403.737`, 
              421.108`, 1.25536`, 553.098`, 559.122`, 0.587094`, 403.737`, 
              415.109`, 0.815857`, 1638.35`, 1656.16`, 5.14103`, 201.945`, 
              237.874`, 1.38454`, 1634.82`, 1641.96`, 2.04992`, 1636.64`, 
              2002.47`, 553.098`, 2174.27`, 201.945`, 1998.24`, 1638.35`, 
              2484.26`, 1837.41`, 2437.4`, 1874.86`, 2404.38`, 1828.99`, 
              1881.92`, 2399.02`, 2442.77`, 403.737`, 1869.68`, 1634.82`, 
              1925.14`}, {3, 34, 35.90865251`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9413385404`, 0.330530446`, 0.8373498804`, 0.1`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.210727`\), 1657.17`, 
              1668.13`, 3.1928`, 1660.81`, 1670.84`, 2.92704`, 401.557`, 
              418.886`, 1.24567`, 545.862`, 551.948`, 0.585377`, 401.557`, 
              412.887`, 0.808514`, 1657.21`, 1674.62`, 5.08191`, 202.829`, 
              238.673`, 1.38654`, 1657.17`, 1667.55`, 3.02539`, 1660.81`, 
              2107.69`, 545.862`, 2218.97`, 202.829`, 2098.25`, 1657.21`, 
              2578.25`, 1957.51`, 2540.06`, 1993.17`, 2507.01`, 1950.36`, 
              1999.19`, 2502.49`, 2544.58`, 401.557`, 1988.46`, 1657.17`, 
              2036.42`}, {3, 34, 36.78498011`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              1.019360674`, 0.3308649298`, 0.9249580244`, 0.1`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220071`\), 1670.15`, 
              1685.04`, 4.37503`, 1675.14`, 1685.88`, 3.16134`, 399.695`, 
              417.022`, 1.23991`, 540.47`, 546.624`, 0.586111`, 399.695`, 
              411.011`, 0.803749`, 1668.68`, 1686.47`, 5.22981`, 203.399`, 
              239.175`, 1.38731`, 1670.15`, 1682.96`, 3.76362`, 1675.14`, 
              2203.47`, 540.47`, 2241.14`, 203.399`, 2185.09`, 1668.68`, 
              2648.65`, 2062.87`, 2616.99`, 2097.1`, 2583.74`, 2056.6`, 
              2102.38`, 2579.78`, 2620.94`, 399.695`, 2092.64`, 1670.15`, 
              2136.52`}, {3, 22, 37.6955966`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.087347015`, 0.3313691682`, 1.001928556`, 0.1`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.222526`\), 1679.39`, 
              1697.17`, 5.25976`, 1685.16`, 1697.38`, 3.62096`, 397.783`, 
              415.136`, 1.23595`, 535.683`, 541.921`, 0.58899`, 397.783`, 
              409.114`, 0.801056`, 1677.19`, 1695.9`, 5.52814`, 203.822`, 
              239.527`, 1.38694`, 1679.39`, 1694.32`, 4.41281`, 1685.16`, 
              2291.86`, 535.683`, 2247.28`, 203.822`, 2258.41`, 1677.19`, 
              2701.41`, 2157.97`, 2674.55`, 2190.92`, 2640.99`, 2152.36`, 
              2195.64`, 2637.46`, 2678.07`, 397.783`, 2186.51`, 1679.39`, 
              2228}, {3, 47, 38.67098982`, \(-1.764202771`\), 7.563874923`, 
              0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 1.1487595`, 
              0.332654225`, \(-1.071042531`\), 0.1`, 0.5`, 0, 0, 10, 11, 12, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.220908`\), 1687.47`, 
              1707.36`, 5.91485`, 1693.69`, 1708.13`, 4.30413`, 396.636`, 
              414, 1.23329`, 532.576`, 538.857`, 0.589621`, 396.636`, 
              407.95`, 0.797619`, 1684.86`, 1704.86`, 5.94049`, 204.152`, 
              239.805`, 1.38683`, 1687.47`, 1704.44`, 5.04065`, 1693.69`, 
              2374.7`, 532.576`, 2241.54`, 204.152`, 2313.66`, 1684.86`, 
              2745.17`, 2245.16`, 2717.17`, 2276.95`, 2683.24`, 2240.05`, 
              2281.24`, 2680.03`, 2720.37`, 396.636`, 2272.27`, 1687.47`, 
              2312.84`}, {3, 41, 26.07195283`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.1890446567`, 0.2500488188`, 0.7295721504`, 0.2`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.488515`\), 1436.09`, 
              1436.37`, 0.0694738`, 1445.61`, 1445.69`, 0.0219029`, 437.417`, 
              452.765`, 1.19705`, 642.785`, 647.771`, 0.563801`, 437.417`, 
              449.623`, 0.948624`, 1443.63`, 1443.96`, 0.0844759`, 192.332`, 
              228.437`, 1.33103`, 1438, 1438.31`, 0.0775408`, 1496.84`, 
              1667.53`, 642.785`, 1873.46`, 192.332`, 1552.89`, 1620.17`, 
              2053.61`, 1438, 1966.83`, 1444.32`, 1963.4`, 1436.09`, 
              1445.61`, 1962.2`, 1968.11`, 437.417`, 1443.63`, 1471.58`, 
              1638.78`}, {3, 32, 26.32440319`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2021560688`, 0.2621409886`, 0.6581824283`, 0.2`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.451379`\), 1447.29`, 
              1447.59`, 0.0764347`, 1459.63`, 1459.72`, 0.0232991`, 435.157`, 
              450.658`, 1.20312`, 640.746`, 645.739`, 0.562811`, 435.157`, 
              447.329`, 0.941167`, 1457.04`, 1457.38`, 0.0889932`, 192.693`, 
              228.639`, 1.32694`, 1449.77`, 1450.09`, 0.0808667`, 1511.83`, 
              1669.46`, 640.746`, 1886.38`, 192.693`, 1576.34`, 1615.89`, 
              2072.45`, 1449.77`, 1977.86`, 1457.97`, 1973.4`, 1447.29`, 
              1459.63`, 1971.83`, 1979.53`, 435.157`, 1457.04`, 1485.9`, 
              1637.07`}, {3, 26, 26.71906996`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2232652621`, 0.2778638143`, 0.5775425924`, 0.2`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.400335`\), 1454.98`, 
              1455.34`, 0.0907427`, 1472.09`, 1472.2`, 0.0271145`, 431.932`, 
              447.659`, 1.21202`, 637.182`, 642.202`, 0.562738`, 431.932`, 
              444.042`, 0.929382`, 1468.41`, 1468.77`, 0.0927201`, 193.262`, 
              229.002`, 1.32228`, 1458.43`, 1458.78`, 0.0886553`, 1524.74`, 
              1670.4`, 637.182`, 1901.02`, 193.262`, 1602.28`, 1606.83`, 
              2093.02`, 1458.43`, 1987.59`, 1469.77`, 1981.4`, 1454.98`, 
              1472.09`, 1979.22`, 1989.92`, 431.932`, 1468.41`, 1498.09`, 
              1632.34`}, {3, 13, 27.46824182`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2657375295`, 0.2998978272`, 0.4824916919`, 0.2`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322108`\), 1453.89`, 
              1454.38`, 0.12634`, 1480.58`, 1480.73`, 0.0387809`, 427.951`, 
              444.018`, 1.22748`, 632.204`, 637.226`, 0.55856`, 427.951`, 
              439.853`, 0.904964`, 1474.59`, 1474.97`, 0.0978271`, 194.131`, 
              229.64`, 1.31843`, 1459.3`, 1459.72`, 0.108106`, 1531.81`, 
              1670.03`, 632.204`, 1920.92`, 194.131`, 1632.51`, 1588.43`, 
              2116.21`, 1459.3`, 1994.86`, 1476.88`, 1985.15`, 1453.89`, 
              1480.58`, 1981.77`, 1998.46`, 427.951`, 1474.59`, 1504.22`, 
              1622.3`}, {1, 25, 31.18197015`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.4871543697`, 0.3251594621`, 0.4405083275`, 0.2`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.165801`\), 1512.16`, 
              1510.14`, \(-0.534989`\), 1545.3`, 1553.7`, 2.28044`, 423.83`, 
              440.495`, 1.26209`, 609.781`, 614.747`, 0.532783`, 423.83`, 
              434.651`, 0.813934`, 1555.47`, 1555.85`, 0.103144`, 201.172`, 
              236.097`, 1.33804`, 1523.29`, 1518.51`, \(-1.27375`\), 
              1558.33`, 1725.92`, 609.781`, 2052.87`, 201.172`, 1749.5`, 
              1574.74`, 2208.57`, 1523.29`, 2103.88`, 1560.66`, 2076.39`, 
              1512.16`, 1569.63`, 2069.23`, 2111.35`, 423.83`, 1555.47`, 
              1545.3`, 1649.31`}, {1, 45, 38.98858574`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7440251912`, 0.301119644`, 0.6590543042`, 0.2`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.213417`\), 1638.73`, 
              1636.5`, \(-0.639779`\), 1643.25`, 1650.24`, 2.015`, 395.996`, 
              413.533`, 1.24384`, 546.096`, 552.51`, 0.617351`, 395.996`, 
              407.86`, 0.835581`, 1637.47`, 1681.21`, 12.7182`, 195.704`, 
              231.915`, 1.3567`, 1643.25`, 1641.75`, \(-0.433815`\), 
              1638.73`, 1872.55`, 546.096`, 2163.89`, 195.704`, 1891.3`, 
              1637.47`, 2385.08`, 1724.69`, 2330.85`, 1760.97`, 2299.78`, 
              1716.45`, 1767.94`, 2294.63`, 2336.05`, 395.996`, 1756.18`, 
              1643.25`, 1796.39`}, {3, 13, 37.13181086`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8656610654`, 0.313727018`, 0.7785515806`, 0.2`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.240166`\), 1675.05`, 
              1680.04`, 1.4663`, 1675.85`, 1686.39`, 3.10694`, 404.749`, 
              422.043`, 1.25278`, 554.556`, 560.551`, 0.585694`, 404.749`, 
              416.107`, 0.816841`, 1663.08`, 1695.25`, 9.46654`, 200.447`, 
              236.671`, 1.38734`, 1675.85`, 1683.06`, 2.12399`, 1675.05`, 
              2002.56`, 554.556`, 2222.29`, 200.447`, 2020.24`, 1663.08`, 
              2507.69`, 1863.42`, 2461.11`, 1898.9`, 2429.13`, 1856.18`, 
              1905.02`, 2424.6`, 2465.65`, 404.749`, 1894.57`, 1675.85`, 
              1929.12`}, {3, 13, 37.56112653`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9564317894`, 0.319086696`, 0.875298127`, 0.2`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253539`\), 1688.2`, 
              1701.51`, 3.95191`, 1692.05`, 1703.31`, 3.34981`, 405.994`, 
              423.189`, 1.24919`, 553.26`, 559.147`, 0.573767`, 405.994`, 
              417.145`, 0.804213`, 1676.5`, 1704.35`, 8.24719`, 201.734`, 
              237.954`, 1.39532`, 1688.2`, 1701.18`, 3.85247`, 1692.05`, 
              2113.12`, 553.26`, 2250.15`, 201.734`, 2124.38`, 1676.5`, 
              2596.09`, 1981.17`, 2557.15`, 2015.49`, 2524.72`, 1974.79`, 
              2020.89`, 2520.73`, 2561.14`, 405.994`, 2011.48`, 1688.2`, 
              2045.64`}, {2, 17, 40.90177432`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.031956868`, 0.345982102`, 0.9565258241`, 0.2`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.27987`\), 1689.22`, 
              1708.21`, 5.6512`, 1697.14`, 1708.9`, 3.50824`, 371.253`, 
              391.414`, 1.34714`, 543.612`, 549.69`, 0.582221`, 371.253`, 
              383.92`, 0.838088`, 1678.84`, 1704.88`, 7.71925`, 0, 127.802`, 
              1.50982`, 1689.22`, 1705.41`, 4.81532`, 1697.14`, 2228.77`, 
              543.612`, 2256.9`, 0, 2227.42`, 1678.84`, 2669.18`, 2083.46`, 
              2628.21`, 2117.35`, 2594.99`, 2077.13`, 2122.66`, 2590.99`, 
              2632.21`, 371.253`, 2113.63`, 1689.22`, 2156.85`}, {3, 58, 
              39.68412177`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.098708393`, 0.3241209782`, \(-1.032405983`\), 0.2`, 0.4`, 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.257536`\), 1701.06`, 
              1722.6`, 6.4596`, 1708.73`, 1724.31`, 4.68377`, 403.378`, 
              420.591`, 1.24272`, 545.113`, 551.099`, 0.574935`, 403.378`, 
              414.468`, 0.794728`, 1693.08`, 1718.97`, 7.7416`, 202.732`, 
              238.867`, 1.39813`, 1701.06`, 1720.32`, 5.77314`, 1708.73`, 
              2302.13`, 545.113`, 2254.13`, 202.732`, 2291.57`, 1693.08`, 
              2708.18`, 2178.84`, 2682.07`, 2210.89`, 2648.78`, 2173.68`, 
              2215.25`, 2645.56`, 2685.29`, 403.378`, 2207.19`, 1701.06`, 
              2241.8`}, {2, 21, 40.91378519`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.159180447`, 0.3259189599`, 1.099337603`, 0.2`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.253157`\), 1707.5`, 
              1731.47`, 7.22373`, 1715.62`, 1733.7`, 5.46515`, 401.935`, 
              419.166`, 1.23964`, 541.501`, 547.544`, 0.576623`, 401.935`, 
              413.017`, 0.791322`, 1700.55`, 1727, 7.94431`, 203.106`, 
              239.157`, 1.39696`, 1707.5`, 1728.97`, 6.46666`, 1715.62`, 
              2385.87`, 541.501`, 2239.64`, 203.106`, 2355.91`, 1700.55`, 
              2742.29`, 2265.27`, 2721.38`, 2296.26`, 2687.63`, 2260.54`, 
              2300.24`, 2684.68`, 2724.31`, 401.935`, 2292.51`, 1707.5`, 
              2327.34`}, {3, 28, 26.2169557`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.1954145559`, 0.2504208148`, 0.7263097895`, 0.3`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.489099`\), 1396.1`, 
              1396.48`, 0.09444`, 1405.95`, 1406.08`, 0.0317713`, 436.258`, 
              451.72`, 1.20292`, 642.328`, 647.322`, 0.564226`, 436.258`, 
              448.485`, 0.947792`, 1403.9`, 1404.25`, 0.0861988`, 192.444`, 
              228.55`, 1.3318`, 1398.08`, 1398.54`, 0.112661`, 1463.97`, 
              1660.27`, 642.328`, 1854.89`, 192.444`, 1517.3`, 1613.92`, 
              2023.65`, 1398.08`, 1934.4`, 1404.6`, 1930.89`, 1396.1`, 
              1405.95`, 1929.67`, 1935.7`, 436.258`, 1403.9`, 1434.97`, 
              1632.2`}, {3, 50, 26.47934522`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2096437448`, 0.2608428192`, 0.653773566`, 0.3`, \(-0.4`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.452309`\), 1398.7`, 
              1399.15`, 0.11032`, 1411.43`, 1411.58`, 0.0370473`, 433.776`, 
              449.407`, 1.20959`, 639.909`, 644.923`, 0.564431`, 433.776`, 
              445.972`, 0.940085`, 1408.74`, 1409.11`, 0.0912009`, 192.848`, 
              228.789`, 1.32771`, 1401.27`, 1401.8`, 0.12926`, 1472.26`, 
              1660.3`, 639.909`, 1864.12`, 192.848`, 1532.1`, 1609.51`, 
              2035.05`, 1401.27`, 1938.4`, 1409.68`, 1933.87`, 1398.7`, 
              1411.43`, 1932.29`, 1940.09`, 433.776`, 1408.74`, 1441.44`, 
              1629.72`}, {3, 34, 26.89839887`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2330073646`, 0.2743439186`, 0.5716264973`, 0.3`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401797`\), 1397.38`, 
              1397.95`, 0.138438`, 1414.99`, 1415.18`, 0.0473136`, 430.511`, 
              446.381`, 1.21929`, 636.318`, 641.358`, 0.564163`, 430.511`, 
              442.63`, 0.927111`, 1411.18`, 1411.56`, 0.0953514`, 193.432`, 
              229.168`, 1.32319`, 1400.96`, 1401.61`, 0.158076`, 1479.02`, 
              1659.37`, 636.318`, 1876.43`, 193.432`, 1548.99`, 1601.61`, 
              2047.86`, 1400.96`, 1941.02`, 1412.55`, 1934.76`, 1397.38`, 
              1414.99`, 1932.58`, 1943.34`, 430.511`, 1411.18`, 1445.75`, 
              1625.03`}, {3, 25, 27.75147624`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2833601926`, 0.2923635321`, 0.4757701638`, 0.3`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322784`\), 1392.08`, 
              1392.89`, 0.19758`, 1419.77`, 1420.06`, 0.0716418`, 426.48`, 
              442.696`, 1.23488`, 630.616`, 635.646`, 0.558007`, 426.48`, 
              438.331`, 0.897925`, 1413.52`, 1413.92`, 0.0985795`, 194.411`, 
              229.89`, 1.31895`, 1397.75`, 1398.61`, 0.212263`, 1484.98`, 
              1658.86`, 630.616`, 1903.66`, 194.411`, 1574.88`, 1588.16`, 
              2067.97`, 1397.75`, 1947.18`, 1415.79`, 1937.17`, 1392.08`, 
              1419.77`, 1933.8`, 1950.77`, 426.48`, 1413.52`, 1448.76`, 
              1618.16`}, {1, 29, 31.88433096`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.5701346198`, 0.300438937`, 0.5163481355`, 0.3`, \(-0.1`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.216022`\), 1532.03`, 
              1529.08`, \(-0.792588`\), 1562.93`, 1568.03`, 1.3999`, 
              418.421`, 435.374`, 1.26825`, 593.14`, 598.538`, 0.563574`, 
              418.421`, 429.555`, 0.827242`, 1573.47`, 1574.06`, 0.160539`, 
              200.63`, 236.126`, 1.35835`, 1541.81`, 1535.62`, \(-1.67085`\), 
              1578.79`, 1746.14`, 593.14`, 2110.23`, 200.63`, 1744.84`, 
              1617.73`, 2225.55`, 1541.81`, 2136.31`, 1578.04`, 2108.15`, 
              1532.03`, 1586.23`, 2102.04`, 2142.57`, 418.421`, 1573.47`, 
              1562.93`, 1681.07`}, {1, 13, 48.03922786`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.7955479622`, 0.3506006803`, 0.6553376931`, 0.3`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.243213`\), 1638.08`, 
              1622.74`, \(-4.38392`\), 1645.72`, 1653.01`, 2.10625`, 
              470.939`, 485.244`, 1.19845`, 671.484`, 672.969`, 0.174984`, 
              470.939`, 478.085`, 0.594224`, 1659.81`, 1701.07`, 12.1505`, 
              216.373`, 249.617`, 1.35733`, 1638.08`, 
              1625.29`, \(-3.65826`\), 1645.72`, 1889.92`, 671.484`, 
              2181.07`, 216.373`, 1903.7`, 1659.81`, 2373.71`, 1663.38`, 
              2282.06`, 1704.48`, 2248.09`, 1653.11`, 1712.98`, 2241.61`, 
              2288.57`, 470.939`, 1699.84`, 1638.08`, 1792.85`}, {3, 17, 
              42.31020629`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8831603785`, 0.3008897993`, 0.8207145193`, 0.3`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.283686`\), 1711.8`, 
              1716.58`, 1.43405`, 1716.72`, 1723.77`, 2.12397`, 403.503`, 
              420.969`, 1.26174`, 553.436`, 559.578`, 0.59899`, 403.503`, 
              415.07`, 0.829581`, 1682.11`, 1733.55`, 15.3941`, 196.358`, 
              233.522`, 1.39978`, 1716.72`, 1721.49`, 1.43754`, 1711.8`, 
              2005.35`, 553.436`, 2262.03`, 196.358`, 2042.86`, 1682.11`, 
              2530.21`, 1891.81`, 2485.19`, 1925.78`, 2453.95`, 1885.46`, 
              1931.18`, 2450.02`, 2489.12`, 403.503`, 1922.03`, 1716.72`, 
              1932.57`}, {3, 35, 40.8579568`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9708517175`, 0.3104475356`, 0.9102777335`, 0.3`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.29135`\), 1719.79`, 
              1733.39`, 4.11424`, 1722.49`, 1734.85`, 3.74525`, 409.038`, 
              426.19`, 1.25519`, 558.638`, 564.409`, 0.567812`, 409.038`, 
              420.101`, 0.803734`, 1692.55`, 1734.16`, 12.4931`, 199.836`, 
              236.599`, 1.40575`, 1719.79`, 1733.89`, 4.26749`, 1722.49`, 
              2118.07`, 558.638`, 2275.01`, 199.836`, 2147.26`, 1692.55`, 
              2611.49`, 2004.2`, 2572.55`, 2037.46`, 2540.56`, 1998.43`, 
              2042.36`, 2536.97`, 2576.13`, 409.038`, 2033.94`, 1719.79`, 
              2052.2`}, {3, 25, 41.62540242`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.044734325`, 0.3177427251`, 0.9885495371`, 0.3`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.293805`\), 1720.77`, 
              1741.88`, 6.40442`, 1728.19`, 1743.4`, 4.62613`, 411.457`, 
              428.521`, 1.25586`, 560.438`, 566.057`, 0.554557`, 411.457`, 
              422.286`, 0.791064`, 1700.7`, 1737.43`, 11.0654`, 200.667`, 
              237.503`, 1.41419`, 1720.77`, 1740.61`, 6.01688`, 1728.19`, 
              2219.88`, 560.438`, 2272.21`, 200.667`, 2238.36`, 1700.7`, 
              2671.54`, 2104.34`, 2637.95`, 2136.74`, 2605.33`, 2099.03`, 
              2141.24`, 2602.03`, 2641.24`, 411.457`, 2133.4`, 1720.77`, 
              2158.32`}, {3, 25, 42.91619558`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.110175145`, 0.3213571294`, 1.059613598`, 0.3`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291147`\), 1723.42`, 
              1748.65`, 7.67609`, 1732.91`, 1751.58`, 5.69913`, 410.752`, 
              427.817`, 1.25381`, 557.661`, 563.298`, 0.553689`, 410.752`, 
              421.522`, 0.785334`, 1708.16`, 1742.79`, 10.4703`, 201.089`, 
              237.896`, 1.41567`, 1723.42`, 1746.9`, 7.14034`, 1732.91`, 
              2311.82`, 557.661`, 2258.11`, 201.089`, 2318.14`, 1708.16`, 
              2715.14`, 2196.55`, 2687.11`, 2227.95`, 2653.96`, 2191.67`, 
              2232.07`, 2650.93`, 2690.12`, 410.752`, 2224.73`, 1723.42`, 
              2253.17`}, {2, 21, 44.37813734`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.169760418`, 0.3231703192`, 1.124830502`, 0.3`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.284699`\), 1727.98`, 
              1756.24`, 8.62789`, 1738.14`, 1759.76`, 6.62652`, 408.871`, 
              425.959`, 1.24988`, 553.276`, 558.992`, 0.557029`, 408.871`, 
              419.646`, 0.782113`, 1715.75`, 1749.79`, 10.3334`, 201.537`, 
              238.21`, 1.41297`, 1727.98`, 1754.1`, 7.97091`, 1738.14`, 
              2396.49`, 553.276`, 2235.81`, 201.537`, 2387.24`, 1715.75`, 
              2746.36`, 2282.45`, 2723.61`, 2312.84`, 2689.95`, 2277.96`, 
              2316.62`, 2687.17`, 2726.37`, 408.871`, 2309.66`, 1727.98`, 
              2339.85`}, {2, 38, 26.37566396`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2019705775`, 0.251271744`, 0.7240026627`, 0.4`, \(-0.5`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.48845`\), 1364.81`, 
              1365.28`, 0.112094`, 1375.06`, 1375.22`, 0.0387087`, 434.901`, 
              450.469`, 1.20763`, 641.172`, 646.183`, 0.565216`, 434.901`, 
              447.15`, 0.946666`, 1372.91`, 1373.27`, 0.0873718`, 192.634`, 
              228.717`, 1.33211`, 1366.88`, 1367.46`, 0.136913`, 1437.98`, 
              1655.55`, 641.172`, 1842.26`, 192.634`, 1490.5`, 1609.11`, 
              2001.67`, 1366.88`, 1909.95`, 1373.63`, 1906.33`, 1364.81`, 
              1375.06`, 1905.08`, 1911.29`, 434.901`, 1372.91`, 1406.26`, 
              1627.54`}, {3, 54, 26.65742179`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2174913802`, 0.2608489147`, 0.6508524945`, 0.4`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.452064`\), 1362.55`, 
              1363.1`, 0.132312`, 1375.8`, 1375.99`, 0.045863`, 432.43`, 
              448.174`, 1.21479`, 638.809`, 643.837`, 0.56502`, 432.43`, 
              444.641`, 0.938363`, 1372.98`, 1373.37`, 0.092372`, 193.02`, 
              228.95`, 1.32836`, 1365.25`, 1365.91`, 0.159612`, 1442.23`, 
              1655.29`, 638.809`, 1850.06`, 193.02`, 1500.74`, 1604.93`, 
              2009.35`, 1365.25`, 1910.33`, 1373.94`, 1905.66`, 1362.55`, 
              1375.8`, 1904.04`, 1912.05`, 432.43`, 1372.98`, 1408.22`, 
              1625.17`}, {3, 48, 27.11187757`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2435130222`, 0.272746629`, 0.5680796661`, 0.4`, \(-0.3`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401848`\), 1356.61`, 
              1357.31`, 0.167112`, 1374.95`, 1375.19`, 0.0591776`, 429.151`, 
              445.141`, 1.22484`, 635.015`, 640.066`, 0.564329`, 429.151`, 
              441.268`, 0.924067`, 1370.96`, 1371.36`, 0.0955524`, 193.625`, 
              229.354`, 1.32411`, 1360.36`, 1361.19`, 0.197511`, 1445.11`, 
              1654.8`, 635.015`, 1862.52`, 193.625`, 1513.21`, 1598.4`, 
              2018.76`, 1360.36`, 1910.06`, 1372.35`, 1903.58`, 1356.61`, 
              1374.95`, 1901.36`, 1912.42`, 429.151`, 1370.96`, 1408.13`, 
              1621.64`}, {3, 25, 28.08116804`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3034090893`, 0.28707858`, 0.4733899052`, 0.4`, \(-0.2`\), 0, 
              0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.322789`\), 1353.32`, 
              1354.33`, 0.24007`, 1382.54`, 1382.91`, 0.0899294`, 425.268`, 
              441.589`, 1.23958`, 628.302`, 633.331`, 0.555872`, 425.268`, 
              437.046`, 0.88987`, 1375.93`, 1376.32`, 0.0938579`, 194.748`, 
              230.199`, 1.31993`, 1359.31`, 1360.43`, 0.266683`, 1452.35`, 
              1658.62`, 628.302`, 1899.53`, 194.748`, 1540.83`, 1590.94`, 
              2041.8`, 1359.31`, 1921.62`, 1378.21`, 1910.91`, 1353.32`, 
              1382.54`, 1907.45`, 1925.27`, 425.268`, 1375.93`, 1412.49`, 
              1620.64`}, {3, 13, 32.33289016`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6254649209`, 0.2875036834`, 0.5762483553`, 0.4`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.266856`\), 1554.59`, 
              1551.31`, \(-0.891103`\), 1584.38`, 1588.18`, 1.05663`, 
              418.987`, 435.857`, 1.2635`, 590.424`, 595.77`, 0.555531`, 
              418.987`, 430.067`, 0.824187`, 1594.36`, 1595.05`, 0.193192`, 
              200.721`, 236.438`, 1.36804`, 1563.22`, 
              1556.68`, \(-1.78798`\), 1601.4`, 1766.71`, 590.424`, 2150.6`, 
              200.721`, 1755.21`, 1648.58`, 2253.34`, 1563.22`, 2170.45`, 
              1598.39`, 2142.23`, 1554.59`, 1605.7`, 2136.94`, 2175.83`, 
              418.987`, 1594.36`, 1584.38`, 1706.51`}, {1, 60, 
              49.62603699`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8136854988`, 0.3378389991`, 0.6953338684`, 0.4`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.291217`\), 1667.93`, 
              1647.26`, \(-6.00551`\), 1674.36`, 1679.76`, 1.58434`, 
              472.579`, 486.726`, 1.18904`, 672.752`, 674.144`, 0.164251`, 
              472.579`, 479.65`, 0.589915`, 1679.46`, 1719.85`, 12.0278`, 
              215.241`, 248.699`, 1.36002`, 1667.93`, 
              1650.25`, \(-5.14008`\), 1674.81`, 1890.97`, 672.752`, 
              2221.01`, 215.241`, 1916.19`, 1679.46`, 2394.89`, 1683.57`, 
              2304.03`, 1722.74`, 2271.1`, 1674.36`, 1730.43`, 2265.36`, 
              2309.79`, 472.579`, 1718.71`, 1667.93`, 1797.57`}, {3, 29, 
              51.82450535`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.8991082865`, 0.2886666655`, 0.8619492945`, 0.4`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.32186`\), 1748.22`, 
              1751.6`, 1.03718`, 1757.83`, 1759.45`, 0.499725`, 398.066`, 
              416.012`, 1.28003`, 546.211`, 552.817`, 0.636127`, 398.066`, 
              410.195`, 0.858972`, 1698.17`, 1770.68`, 22.0373`, 190.23`, 
              228.851`, 1.41808`, 1757.83`, 1758.21`, 0.11554`, 1748.22`, 
              2009.48`, 546.211`, 2296.7`, 190.23`, 2066.71`, 1698.17`, 
              2552.41`, 1924.09`, 2510.79`, 1956.7`, 2480.14`, 1918.54`, 
              1961.45`, 2476.74`, 2514.19`, 398.066`, 1953.38`, 1757.83`, 
              1936.9`}, {3, 17, 46.34383585`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9845525212`, 0.3006370503`, 0.9445873029`, 0.4`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.324404`\), 1752.69`, 
              1764.34`, 3.59052`, 1752.91`, 1765.83`, 3.98335`, 407.835`, 
              425.126`, 1.26194`, 557.358`, 563.265`, 0.579995`, 407.835`, 
              419.069`, 0.813955`, 1706.79`, 1764.54`, 17.5644`, 196.895`, 
              234.314`, 1.41375`, 1752.69`, 1766.01`, 4.10427`, 1752.91`, 
              2121.95`, 557.358`, 2296.35`, 196.895`, 2168.19`, 1706.79`, 
              2625.27`, 2029.1`, 2588.05`, 2061.33`, 2556.44`, 2023.92`, 
              2065.76`, 2553.24`, 2591.24`, 407.835`, 2058.19`, 1752.69`, 
              2057.25`}, {3, 25, 46.07831181`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.057152617`, 0.3147466155`, 1.016376483`, 0.4`, 0.3`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.326163`\), 1748.13`, 
              1770.23`, 6.81244`, 1755.42`, 1772.23`, 5.19642`, 417.315`, 
              434.28`, 1.26586`, 570.619`, 575.965`, 0.537098`, 417.315`, 
              427.905`, 0.784271`, 1714.87`, 1764.1`, 15.0042`, 198.624`, 
              236.166`, 1.43017`, 1748.13`, 1769.78`, 6.67371`, 1755.42`, 
              2226.04`, 570.619`, 2283.89`, 198.624`, 2259.95`, 1714.87`, 
              2680.48`, 2122.24`, 2645.35`, 2153.99`, 2612.89`, 2117.24`, 
              2158.23`, 2609.8`, 2648.43`, 417.315`, 2151, 1748.13`, 
              2164.69`}, {3, 25, 47.43045239`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.12151056`, 0.3214840955`, 1.084121927`, 0.4`, 0.4`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323202`\), 1747.24`, 
              1775.4`, 8.68971`, 1758.13`, 1779.22`, 6.53828`, 419.316`, 
              436.227`, 1.26763`, 572.308`, 577.537`, 0.526738`, 419.316`, 
              429.713`, 0.773427`, 1722.68`, 1767.46`, 13.695`, 199.009`, 
              236.663`, 1.43736`, 1747.24`, 1774.17`, 8.3085`, 1758.13`, 
              2320.08`, 572.308`, 2261.77`, 199.009`, 2341.13`, 1722.68`, 
              2721.25`, 2211.73`, 2690.27`, 2242.65`, 2657.18`, 2207.04`, 
              2246.61`, 2654.28`, 2693.16`, 419.316`, 2239.79`, 1747.24`, 
              2261.62`}, {3, 25, 49.13792663`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.180272258`, 0.3238361632`, 1.147679154`, 0.4`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.315739`\), 1749.48`, 
              1781.73`, 9.97865`, 1761.68`, 1786.36`, 7.6743`, 417.594`, 
              434.526`, 1.2641`, 568.155`, 573.454`, 0.530015`, 417.594`, 
              427.983`, 0.769643`, 1730.58`, 1773.26`, 13.1033`, 199.413`, 
              236.947`, 1.435`, 1749.48`, 1779.92`, 9.41371`, 1761.68`, 
              2406.02`, 568.155`, 2232.34`, 199.413`, 2412.67`, 1730.58`, 
              2750.27`, 2296.86`, 2724.21`, 2326.82`, 2690.58`, 2292.53`, 
              2330.48`, 2687.89`, 2726.87`, 417.594`, 2324.03`, 1749.48`, 
              2349.92`}, {3, 30, 26.5358766`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.2085541656`, 0.2523097178`, 0.7223324593`, 0.5`, \(-0.5`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.487144`\), 1339.66`, 
              1340.2`, 0.126349`, 1350.33`, 1350.52`, 0.0442383`, 433.492`, 
              449.157`, 1.21141`, 639.674`, 644.706`, 0.566349`, 433.492`, 
              445.763`, 0.945294`, 1348.09`, 1348.46`, 0.0882108`, 192.842`, 
              228.893`, 1.3321`, 1341.83`, 1342.5`, 0.156017`, 1417.06`, 
              1652.53`, 639.674`, 1833.56`, 192.842`, 1469.77`, 1605.57`, 
              1985.13`, 1341.83`, 1891.03`, 1348.82`, 1887.29`, 1339.66`, 
              1350.33`, 1886, 1892.41`, 433.492`, 1348.09`, 1383.22`, 
              1624.35`}, {3, 58, 26.83925141`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2254872088`, 0.2612229013`, 0.6488786238`, 0.5`, \(-0.4`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.451064`\), 1334.54`, 
              1335.18`, 0.149602`, 1348.35`, 1348.57`, 0.0526715`, 431.042`, 
              446.888`, 1.21889`, 637.286`, 642.332`, 0.565782`, 431.042`, 
              443.266`, 0.936383`, 1345.4`, 1345.8`, 0.0928298`, 193.234`, 
              229.139`, 1.32872`, 1337.36`, 1338.14`, 0.182628`, 1418.79`, 
              1652.62`, 637.286`, 1841.12`, 193.234`, 1477.48`, 1601.91`, 
              1990.92`, 1337.36`, 1889.61`, 1346.38`, 1884.76`, 1334.54`, 
              1348.35`, 1883.1`, 1891.38`, 431.042`, 1345.4`, 1382.44`, 
              1622.51`}, {1, 34, 27.33077478`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.2545446051`, 0.2714364743`, 0.5660493042`, 0.5`, \(-0.3`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.401137`\), 1326.52`, 
              1327.33`, 0.189549`, 1345.7`, 1345.99`, 0.0682032`, 427.619`, 
              443.715`, 1.22884`, 632.763`, 637.844`, 0.565734`, 427.619`, 
              439.737`, 0.920937`, 1341.51`, 1341.91`, 0.0941822`, 193.924`, 
              229.62`, 1.32465`, 1330.46`, 1331.43`, 0.226548`, 1419.4`, 
              1653.48`, 632.763`, 1855.25`, 193.924`, 1487.89`, 1597.17`, 
              1999.2`, 1330.46`, 1888.98`, 1342.92`, 1882.19`, 1326.52`, 
              1345.7`, 1879.91`, 1891.39`, 427.619`, 1341.51`, 1380.09`, 
              1620.82`}, {1, 18, 28.42954934`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.3258041071`, 0.2824874861`, 0.4740856013`, 0.5`, \(-0.2`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.323511`\), 1329.64`, 
              1330.82`, 0.274727`, 1360.76`, 1361.19`, 0.103819`, 424.368`, 
              440.756`, 1.2422`, 625.689`, 630.706`, 0.552268`, 424.368`, 
              436.054`, 0.880956`, 1353.73`, 1354.1`, 0.0868136`, 195.122`, 
              230.542`, 1.32099`, 1336, 1337.3`, 0.305161`, 1431.03`, 
              1663.97`, 625.689`, 1903.9`, 195.122`, 1521.53`, 1597, 
              2029.25`, 1336, 1910.41`, 1356.04`, 1898.66`, 1329.64`, 
              1360.76`, 1895.06`, 1914.19`, 424.368`, 1353.73`, 1389.64`, 
              1627.48`}, {1, 13, 32.72386423`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.6668535004`, 0.2781790962`, 0.6286592355`, 0.5`, \(-0.1`\), 
              0, 0, 10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.313353`\), 1582.18`, 
              1577.99`, \(-1.15995`\), 1609.88`, 1613.92`, 1.14128`, 
              419.708`, 436.463`, 1.25691`, 588.897`, 594.175`, 0.547125`, 
              419.708`, 430.722`, 0.820729`, 1620.4`, 1621.16`, 0.213738`, 
              200.916`, 236.747`, 1.37399`, 1589.84`, 
              1582.44`, \(-2.05646`\), 1626.63`, 1784.65`, 588.897`, 
              2186.19`, 200.916`, 1772.96`, 1672.13`, 2282.76`, 1589.84`, 
              2205.12`, 1623.98`, 2177.04`, 1582.18`, 1630.51`, 2172.4`, 
              2209.82`, 419.708`, 1620.4`, 1609.88`, 1727.37`}, {3, 21, 
              52.36903832`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.8320629694`, 0.3365506995`, 0.7246038902`, 0.5`, 0, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.330002`\), 1685.81`, 
              1662.94`, \(-6.71097`\), 1688.43`, 1698.86`, 3.09635`, 
              484.268`, 498.035`, 1.18486`, 691.094`, 691.892`, 0.0966459`, 
              484.268`, 490.841`, 0.561549`, 1695.81`, 1730.41`, 10.3872`, 
              218.741`, 251.982`, 1.37095`, 1688.43`, 
              1666.04`, \(-6.58066`\), 1697.51`, 1898.36`, 691.094`, 
              2248.63`, 218.741`, 1931.78`, 1695.81`, 2411.96`, 1694.57`, 
              2316.76`, 1733.02`, 2284.18`, 1685.81`, 1740.35`, 2278.75`, 
              2322.2`, 484.268`, 1729.36`, 1688.43`, 1805.44`}, {2, 38, 
              64.95563385`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 0, 
              0.02426983029`, 0, \(-1.547566329`\), \(-2.239030657`\), 
              0.9047284713`, 0.3249455065`, 0.8952771947`, 0.5`, 0.1`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.379019`\), 1782.69`, 
              1786.36`, 1.14748`, 1791.93`, 1793.69`, 0.553546`, 386.024`, 
              408.228`, 1.5452`, 571.46`, 578.85`, 0.744794`, 386.024`, 
              400.768`, 1.01639`, 1713.91`, 1802.03`, 27.1442`, 0, 131.945`, 
              1.69386`, 1791.93`, 1792.75`, 0.258598`, 1782.69`, 2067.33`, 
              571.46`, 2330.23`, 0, 2142.84`, 1713.91`, 2604.08`, 1968.67`, 
              2545.05`, 2001.26`, 2514.29`, 1962.53`, 2006.49`, 2510.46`, 
              2548.88`, 386.024`, 1998.3`, 1791.93`, 1980.26`}, {1, 17, 
              54.43096574`, \(-1.764202771`\), 7.563874923`, 0.07642881742`, 
              1, 0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              0.9979333629`, 0.289161034`, 0.9788236188`, 0.5`, 0.2`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.353521`\), 1783.52`, 
              1794.46`, 3.42747`, 1786.51`, 1796.11`, 3.01475`, 402.404`, 
              420.023`, 1.2696`, 549.51`, 555.811`, 0.610224`, 402.404`, 
              414.076`, 0.835007`, 1720.17`, 1795.07`, 23.07`, 192.982`, 
              231.172`, 1.41925`, 1786.51`, 1797.25`, 3.37248`, 1783.52`, 
              2125.24`, 549.51`, 2315.53`, 192.982`, 2187.97`, 1720.17`, 
              2637.58`, 2056.01`, 2603.61`, 2087.22`, 2572.34`, 2051.4`, 
              2091.17`, 2569.53`, 2606.42`, 402.404`, 2084.36`, 1786.51`, 
              2061.88`}, {3, 25, 52.01318012`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.06928605`, 0.3102531596`, 1.043572326`, 0.5`, 0.3`, 0, 0, 10, 
              11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.354898`\), 1776.94`, 
              1798.4`, 6.72252`, 1782.96`, 1800.51`, 5.51265`, 420.214`, 
              437.159`, 1.27288`, 576.008`, 581.233`, 0.529735`, 420.214`, 
              430.709`, 0.782423`, 1728.29`, 1791.4`, 19.4646`, 196.235`, 
              234.423`, 1.44096`, 1776.94`, 1798.76`, 6.83468`, 1782.96`, 
              2230.29`, 576.008`, 2294.93`, 196.235`, 2279.16`, 1728.29`, 
              2687.86`, 2140.62`, 2652.25`, 2171.72`, 2619.94`, 2135.94`, 
              2175.7`, 2617.07`, 2655.12`, 420.214`, 2169.01`, 1776.94`, 
              2168.93`}, {3, 25, 53.12483753`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.132633515`, 0.3221543273`, 1.106969782`, 0.5`, 0.4`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.352962`\), 1772.67`, 
              1802.44`, 9.32403`, 1784.15`, 1806.82`, 7.13412`, 427.428`, 
              444.196`, 1.28058`, 586.239`, 591.079`, 0.499278`, 427.428`, 
              437.47`, 0.76104`, 1736.62`, 1792.79`, 17.3721`, 196.759`, 
              235.278`, 1.45809`, 1772.67`, 1801.77`, 9.11123`, 1784.15`, 
              2326.33`, 586.239`, 2266.35`, 196.759`, 2361.53`, 1736.62`, 
              2726.26`, 2225.61`, 2692.32`, 2256.11`, 2659.25`, 2221.08`, 
              2259.94`, 2656.46`, 2695.1`, 427.428`, 2253.52`, 1772.67`, 
              2267.2`}, {3, 25, 55.11143833`, \(-1.764202771`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.7680984`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 
              1.190596147`, 0.3266255031`, 1.168352306`, 0.5`, 0.5`, 0, 0, 
              10, 11, 12, 0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, \(-0.345894`\), 1772.33`, 
              1807.75`, 11.1106`, 1786.27`, 1813.27`, 8.51363`, 427.484`, 
              444.249`, 1.28048`, 585.025`, 589.853`, 0.496976`, 427.484`, 
              437.438`, 0.754313`, 1744.94`, 1797.27`, 16.243`, 196.965`, 
              235.51`, 1.46056`, 1772.33`, 1806.26`, 10.6413`, 1786.27`, 
              2413.9`, 585.025`, 2230.71`, 196.965`, 2434.68`, 1744.94`, 
              2753.61`, 2309.08`, 2723.6`, 2338.71`, 2689.94`, 2304.82`, 
              2342.3`, 2687.31`, 2726.2`, 427.484`, 2336.21`, 1772.33`, 
              2357.27`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["analyze", "Section"],

Cell[BoxData[
    \(data[\([1]\)]\)], "Input"],

Cell[BoxData[
    \(\(dat2 = Transpose[Partition[data[\([2]\)], 11]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(mesh2 = {\({Min[#], Max[#]} &\)[
          Map[#[\([19]\)] &, data[\([2]\)]]], \({Min[#], Max[#]} &\)[
          Map[#[\([20]\)] &, data[\([2]\)]]]}\)], "Input"],

Cell[BoxData[
    \({{\(-0.5`\), 0.5`}, {\(-0.5`\), 0.5`}}\)], "Output"]
}, Open  ]],

Cell["Chi^2", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot3D[Map[\((#[\([3]\)])\) &, dat2, {2}], 
        PlotRange \[Rule] {22, 50}, ClipFill \[Rule] None, 
        BoxRatios \[Rule] {1, 1, 0.4}, MeshRange \[Rule] mesh2, 
        AxesLabel \[Rule] {\*"\"\<\!\(m\_0\^2\)\>\"", \
\*"\"\<\!\(m\_1\^2\)\>\"", \*"\"\<\!\(\[Chi]\^2\)\>\""}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ListContourPlot[
      Map[\((#[\([3]\)])\) &, dat2, {2}], \[IndentingNewLine]\t
      PlotRange \[Rule] {20, 50}, MeshRange \[Rule] mesh2, 
      FrameLabel \[Rule] {\*"\"\<\!\(m\_0\^2\)\>\"", \
\*"\"\<\!\(m\_1\^2\)\>\""}, ColorFunction \[Rule] Hue, 
      RotateLabel \[Rule] False]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  ContourGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["log(K) coefficient", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(logk = 
      ListPlot3D[Map[\((#[\([32]\)])\) &, dat2, {2}], 
        PlotRange -> {\(-0.5\), 0}, ClipFill \[Rule] None, 
        BoxRatios \[Rule] {1, 1, 0.25}, MeshRange \[Rule] mesh2, 
        AxesLabel \[Rule] {\*"\"\<\!\(m\_0\^2\)\>\"", \
\*"\"\<\!\(m\_1\^2\)\>\"", "\<log(K)\>"}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["rho mass", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(rhomass = 
      ListPlot3D[Map[\((#[\([\(-4\)]\)])\) &, dat2, {2}], 
        PlotRange -> {400, 600}, ClipFill \[Rule] None, 
        BoxRatios \[Rule] {1, 1, 0.25}, MeshRange \[Rule] mesh2, 
        AxesLabel \[Rule] {\*"\"\<\!\(m\_0\^2\)\>\"", \
\*"\"\<\!\(m\_1\^2\)\>\"", "\<rho\>"}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Chi]\^2\ fit\)], "Section"],

Cell["Select which points to use for fitting (ssdata).", "Text"],

Cell[BoxData[
    \(\(ssdata = 
        Select[data[\([2]\)], \((\ #[\([3]\)] < 
                30\  (*\(&&\)\(\ \)\(#[\([19]\)] > \(-0.15\)\)\(\ \)\(&&\)\(\ \
\)\(#[\([20]\)] > \(-0.15\)\)*) )\) &];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[{#[\([19]\)], #[\([20]\)]} &, ssdata]\)], "Input"],

Cell[BoxData[
    \({{\(-0.5`\), \(-0.5`\)}, {\(-0.5`\), \(-0.4`\)}, {\(-0.5`\), \
\(-0.3`\)}, {\(-0.5`\), \(-0.2`\)}, {\(-0.5`\), \(-0.1`\)}, {\(-0.5`\), 
        0}, {\(-0.5`\), 0.1`}, {\(-0.5`\), 0.2`}, {\(-0.5`\), 
        0.3`}, {\(-0.5`\), 0.4`}, {\(-0.5`\), 
        0.5`}, {\(-0.4`\), \(-0.5`\)}, {\(-0.4`\), \(-0.4`\)}, {\(-0.4`\), \
\(-0.3`\)}, {\(-0.4`\), \(-0.2`\)}, {\(-0.4`\), \(-0.1`\)}, {\(-0.4`\), 
        0}, {\(-0.4`\), 0.1`}, {\(-0.4`\), 0.2`}, {\(-0.4`\), 
        0.3`}, {\(-0.4`\), 0.4`}, {\(-0.4`\), 
        0.5`}, {\(-0.3`\), \(-0.5`\)}, {\(-0.3`\), \(-0.4`\)}, {\(-0.3`\), \
\(-0.3`\)}, {\(-0.3`\), \(-0.2`\)}, {\(-0.3`\), \(-0.1`\)}, {\(-0.3`\), 
        0}, {\(-0.3`\), 0.1`}, {\(-0.3`\), 0.2`}, {\(-0.3`\), 
        0.3`}, {\(-0.2`\), \(-0.5`\)}, {\(-0.2`\), \(-0.4`\)}, {\(-0.2`\), \
\(-0.3`\)}, {\(-0.2`\), \(-0.2`\)}, {\(-0.2`\), \(-0.1`\)}, {\(-0.2`\), 
        0}, {\(-0.2`\), 
        0.1`}, {\(-0.1`\), \(-0.5`\)}, {\(-0.1`\), \(-0.4`\)}, {\(-0.1`\), \
\(-0.3`\)}, {\(-0.1`\), \(-0.2`\)}, {\(-0.1`\), \(-0.1`\)}, {0, \(-0.5`\)}, \
{0, \(-0.4`\)}, {0, \(-0.3`\)}, {0, \(-0.2`\)}, {0.1`, \(-0.5`\)}, {0.1`, \
\(-0.4`\)}, {0.1`, \(-0.3`\)}, {0.1`, \(-0.2`\)}, {0.2`, \(-0.5`\)}, {0.2`, \
\(-0.4`\)}, {0.2`, \(-0.3`\)}, {0.2`, \(-0.2`\)}, {0.3`, \(-0.5`\)}, {0.3`, \
\(-0.4`\)}, {0.3`, \(-0.3`\)}, {0.3`, \(-0.2`\)}, {0.4`, \(-0.5`\)}, {0.4`, \
\(-0.4`\)}, {0.4`, \(-0.3`\)}, {0.4`, \(-0.2`\)}, {0.5`, \(-0.5`\)}, {0.5`, \
\(-0.4`\)}, {0.5`, \(-0.3`\)}, {0.5`, \(-0.2`\)}}\)], "Output"]
}, Open  ]],

Cell["\<\
Using ssdata, fit all couplings to either a linear or constant \
function.  Here, x is the mass.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Chop[Table[
          Fit[Map[{#[\([19]\)], #[\([20]\)], #[\([i]\)]} &, ssdata], 
            If[MemberQ[{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, 16} + 6, 
                i], {1}, {1, x, y}], {x, y}], {i, 6, 15 + 7}]]\)], "Input"],

Cell[BoxData[
    \({0.07642881742000002`, 1.0000000000000002`, 
      0.7680984100000002`, \(-0.16924615890000008`\), \(-0.18577166520000002`\
\), 0, 0.024269830290000006`, 
      0, \(-1.5475663289999997`\), \(-2.239030657`\), \(\(0.5724245495234672`\
\)\(\[InvisibleSpace]\)\) + 0.5344445384078449`\ x + 
        1.6049446034984582`\ y, \
\(\(0.3716796191922236`\)\(\[InvisibleSpace]\)\) - 0.2412067644395341`\ x - 
        0.08461359975642356`\ y, \(\(0.5556834014859715`\)\(\[InvisibleSpace]\
\)\) + 0.3352308341762594`\ x + 1.1979638266618706`\ y, 1.`\ x, 
      1.0000000000000002`\ y, 0, 0}\)], "Output"]
}, Open  ]],

Cell["Fit weighted by chi^2 values of data", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Block[{selected}, 
        Chop[Table[\[IndentingNewLine]BestFit /. 
              Regress[Map[{#[\([19]\)], #[\([20]\)], #[\([i]\)]} &, ssdata], 
                If[MemberQ[{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, 16} + 6, 
                    i], {1}, {1, x, y, x^2, y^2, x\ y}], {x, y}, 
                Weights \[Rule] Map[\((1/#[\([3]\)])\) &, ssdata], 
                RegressionReport \[Rule] BestFit], {i, 6, 
              15 + 7}]]]\)], "Input"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(DesignedRegress :: \"badfit\"\) will be \
suppressed during this calculation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \({0.07642881742`, 0.9999999999999997`, 
      0.7680984038671962`, \(-0.16924615889999997`\), \(-0.18577166519999996`\
\), 30574.60026783461`, 0.024269830289999995`, 
      15226.791810590552`, \(-1.547566329`\), \(-2.239030657`\), \
\(\(0.3281787565415082`\)\(\[InvisibleSpace]\)\) + 0.19699066364411094`\ x + 
        0.040151756419765254`\ x\^2 + 0.39101739617961145`\ y + 
        0.41648625836959`\ x\ y + 
        0.16015399375981057`\ y\^2, \(\(0.4528247213120223`\)\(\
\[InvisibleSpace]\)\) - 0.13522988999054417`\ x + 
        0.07499224522179224`\ x\^2 + 0.395147092375987`\ y + 
        0.07898859944707212`\ x\ y + 
        0.004059974384711878`\ y\^2, \(\(0.3948510807805496`\)\(\
\[InvisibleSpace]\)\) - 0.44535235500682047`\ x + 0.3875440096791963`\ x\^2 - 
        0.5761157899486702`\ y - 0.6592836235942773`\ x\ y + 
        0.22446666427781303`\ y\^2, 1.`\ x, 1.`\ y, 0, 0}\)], "Output"]
}, Open  ]],

Cell["Plot out data and fits", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pc = 
      Table[Show[
          ListPlot3D[Map[\((#[\([i]\)])\) &, dat2, {2}], 
            PlotRange \[Rule] All, DisplayFunction -> Identity, 
            MeshRange \[Rule] mesh2], \n\t
          Plot3D[couplefitting[\([i - 5]\)], {x, \(-0.5\), 
              0.5}, {y, \(-0.5\), 0.5}, DisplayFunction -> Identity], 
          DisplayFunction -> $DisplayFunction, FormatType -> OutputForm, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
          AxesLabel -> {\*"\"\<\!\(mu\_0\^2\)\>\"", \
\*"\"\<\!\(mu\_1\^2\)\>\"", {"\<b\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\(l\_2\
\)\>\"", \*"\"\<\!\(l\_3\)\>\"", \*"\"\<\!\(l\_4\)\>\"", \
\*"\"\<\!\(l\_5\)\>\"", \*"\"\<\!\(t\_1\)\>\"", \*"\"\<\!\(t\_2\)\>\"", \*"\"\
\<\!\(kappa\_S\)\>\"", \*"\"\<\!\(kappa\_A\)\>\"", \*"\"\<\!\(mu\_F\)\>\"", \
\*"\"\<\!\(mu\_0\^2\)\>\"", \*"\"\<\!\(mu\_1\^2\)\>\"", \*"\"\<\!\(mu\_2\^2\)\
\>\"", \*"\"\<\!\(mu\_3\^2\)\>\""}[\([i - 7]\)]}], {i, 16, 
          11 + 7}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Open  ]],

Cell["Just plot out data", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pcd = 
      Table[ListPlot3D[Map[\((#[\([i]\)])\) &, dat2, {2}], 
          MeshRange \[Rule] mesh2, 
          AxesLabel -> {\*"\"\<\!\(m\_0\^2\)\>\"", \*"\"\<\!\(m\_1\^2\)\>\"", \
{"\<b\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\(l\_2\)\>\"", \*"\"\<\!\(l\_3\)\>\
\"", \*"\"\<\!\(l\_4\)\>\"", \*"\"\<\!\(l\_5\)\>\"", \*"\"\<\!\(t\_1\)\>\"", \
\*"\"\<\!\(t\_2\)\>\"", \*"\"\<\!\(\[Kappa]\_S\)\>\"", \*"\"\<\!\(\[Kappa]\_A\
\)\>\"", \*"\"\<\!\(m\_F\)\>\"", \*"\"\<\!\(m\_0\^2\)\>\"", \
\*"\"\<\!\(m\_1\^2\)\>\"", \*"\"\<\!\(m\_2\^2\)\>\"", \*"\"\<\!\(mu\_3\^2\)\>\
\""}[\([i - 7]\)]}], {i, 16, 11 + 7}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  
            SurfaceGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  
            SurfaceGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  
            SurfaceGraphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put plots", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/four/scan_m0_m1_123.chi2.eps\>", chi2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/four/scan_m0_m1_123.logk.eps\>", logk, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/four/scan_m0_m1_123.rhomass.eps\>", rhomass, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This was printed to a postscript file \"ttraj60.scaling.ps\", size \
616 by 779, and moved to the left such that left=0. (You need to adjust the \
Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(Show[GraphicsArray[Partition[pcd, 1]]]\)\)\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Create data file", "Section"],

Cell["\<\
This is a fit to the scan_m0_m1_1.out points emphasizing the \
mu_0^2<0 region.\
\>", "Text"],

Cell[BoxData[
    \(\(couplefitting = {0.07642881742, 1, 
          0.7680984, \(-0.1692461589\), \(-0.1857716652\), 49854.19434, \ 
          0.02426983029, 24828.43378, \(-1.547566329\), \(-2.239030657\), 
          0.3279806025625138`\[InvisibleSpace] + 0.15636205804742298`\ x + 
            0.226087138639397`\ y, 
          0.4986340298455228`\[InvisibleSpace] - 0.0541782556531582`\ x + 
            0.37347013469656215`\ y, 
          Sqrt[Max[\((0.3710731803579951`\[InvisibleSpace] - 
                    0.7169102368827279`\ x - 
                    0.3520126883030118`\ y)\)^2, \(-x\), \(-y\)]], x, y, 0, 
          0};\)\)], "Input"],

Cell["\<\
This is a fit to the scan_m0_m1_1.out points emphasizing the \
mu_0^2>0, mu_1^2>0 region.\
\>", "Text"],

Cell[BoxData[
    \(\(couplefitting = {0.07642881742, 1, 
          0.7680984, \(-0.1692461589\), \(-0.1857716652\), 49854.19434, \ 
          0.02426983029, 24828.43378, \(-1.547566329\), \(-2.239030657\), 
          0.5673269760746644`\[InvisibleSpace] + 0.5510892642838314`\ x + 
            1.6314496771000009`\ y, 
          0.3725704358379123`\[InvisibleSpace] - 0.24477442766003837`\ x - 
            0.08821944524478163`\ y, 
          Sqrt[Max[\((0.5565869563550221`\[InvisibleSpace] + 
                    0.3250539160123411`\ x + 
                    1.1966319111273385`\ y)\)^2, \(-x\), \(-y\)]], \ x, \ y, 
          0, 0};\)\)], "Input"],

Cell[BoxData[
    \(expr2 = 
      TableForm[
        Flatten[Table[{3, 
              Map[NumberForm[Chop[#], 12] &, 
                couplefitting /. {x \[Rule] 0.1*i - 0.3, 
                    y \[Rule] 0.1*j - 0.2}\[IndentingNewLine]], {10, 11, 
                12}}, {i, 0, 10}, {j, 0, 10}], 2], \n
        TableSpacing -> {0, 1}]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/four/scan_m0_m1_3.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<scan_m0_m1_3.out\>"]; \n\t\tWrite[fp, "\<100\>"]; \n\t\tWrite[
        fp, "\<0\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/scan_m0_m1_3.in"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{1042, 1043},
WindowMargins->{{151, Automatic}, {46, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "scan_m0_m1_123.couplings.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 83, 3, 194, "Subtitle"],
Cell[1825, 56, 41, 0, 101, "Subsubtitle"],
Cell[1869, 58, 70, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[1964, 62, 159, 3, 112, "Input",
  InitializationCell->True],
Cell[2126, 67, 183, 3, 112, "Message"]
}, Open  ]],
Cell[2324, 73, 94, 2, 52, "Input",
  InitializationCell->True],
Cell[2421, 77, 359, 7, 232, "Input"],
Cell[2783, 86, 47, 1, 52, "Input"],
Cell[2833, 89, 47, 1, 52, "Input"],
Cell[2883, 92, 183, 3, 142, "Input"],

Cell[CellGroupData[{
Cell[3091, 99, 47, 0, 106, "Section"],
Cell[3141, 101, 125222, 1751, 58282, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[128400, 1857, 47, 0, 62, "Section"],
Cell[128450, 1859, 129700, 1872, 60472, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[258187, 3736, 47, 0, 62, "Section"],
Cell[258237, 3738, 128068, 1837, 59782, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[386342, 5580, 57, 0, 62, "Section"],
Cell[386402, 5582, 104, 3, 60, "Text"],
Cell[386509, 5587, 127796, 1825, 59572, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[514342, 7417, 26, 0, 62, "Section"],
Cell[514371, 7419, 46, 1, 52, "Input"],
Cell[514420, 7422, 84, 1, 52, "Input"],

Cell[CellGroupData[{
Cell[514529, 7427, 181, 3, 82, "Input"],
Cell[514713, 7432, 72, 1, 52, "Output"]
}, Open  ]],
Cell[514800, 7436, 21, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[514846, 7440, 332, 6, 145, "Input"],
Cell[515181, 7448, 137, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[515355, 7456, 317, 6, 145, "Input"],
Cell[515675, 7464, 137, 3, 52, "Output"]
}, Open  ]],
Cell[515827, 7470, 34, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[515886, 7474, 319, 6, 145, "Input"],
Cell[516208, 7482, 137, 3, 52, "Output"]
}, Open  ]],
Cell[516360, 7488, 24, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[516409, 7492, 320, 6, 145, "Input"],
Cell[516732, 7500, 137, 3, 52, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[516918, 7509, 49, 1, 107, "Section"],
Cell[516970, 7512, 64, 0, 60, "Text"],
Cell[517037, 7514, 215, 4, 82, "Input"],

Cell[CellGroupData[{
Cell[517277, 7522, 74, 1, 52, "Input"],
Cell[517354, 7525, 1520, 22, 532, "Output"]
}, Open  ]],
Cell[518889, 7550, 120, 3, 60, "Text"],

Cell[CellGroupData[{
Cell[519034, 7557, 268, 5, 142, "Input"],
Cell[519305, 7564, 612, 10, 172, "Output"]
}, Open  ]],
Cell[519932, 7577, 52, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[520009, 7581, 492, 9, 202, "Input"],
Cell[520504, 7592, 156, 3, 112, "Message"],
Cell[520663, 7597, 156, 3, 112, "Message"],
Cell[520822, 7602, 156, 3, 112, "Message"],
Cell[520981, 7607, 175, 3, 112, "Message"],
Cell[521159, 7612, 119, 2, 64, "Message"],
Cell[521281, 7616, 218, 3, 79, "Message"],
Cell[521502, 7621, 119, 2, 64, "Message"],
Cell[521624, 7625, 218, 3, 79, "Message"],
Cell[521845, 7630, 925, 15, 262, "Output"]
}, Open  ]],
Cell[522785, 7648, 38, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[522848, 7652, 1005, 17, 423, "Input"],
Cell[523856, 7671, 438, 11, 52, "Output"]
}, Open  ]],
Cell[524309, 7685, 34, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[524368, 7689, 631, 10, 242, "Input"],
Cell[525002, 7701, 492, 14, 82, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[525543, 7721, 28, 0, 106, "Section"],

Cell[CellGroupData[{
Cell[525596, 7725, 143, 2, 82, "Input"],
Cell[525742, 7729, 137, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[525916, 7737, 143, 2, 82, "Input"],
Cell[526062, 7741, 137, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[526236, 7749, 149, 2, 82, "Input"],
Cell[526388, 7753, 137, 3, 52, "Output"]
}, Open  ]],
Cell[526540, 7759, 208, 4, 92, "Text"],

Cell[CellGroupData[{
Cell[526773, 7767, 85, 1, 52, "Input"],
Cell[526861, 7770, 135, 3, 52, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[527045, 7779, 35, 0, 106, "Section"],
Cell[527083, 7781, 103, 3, 60, "Text"],
Cell[527189, 7786, 646, 11, 352, "Input"],
Cell[527838, 7799, 113, 3, 60, "Text"],
Cell[527954, 7804, 651, 11, 352, "Input"],
Cell[528608, 7817, 349, 8, 232, "Input"],

Cell[CellGroupData[{
Cell[528982, 7829, 333, 6, 232, "Input"],
Cell[529318, 7837, 67, 1, 52, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

