(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     20790,        641]*)
(*NotebookOutlinePosition[     22300,        687]*)
(*  CellTagsIndexPosition[     22256,        683]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Examine meson wavfunctions.
Spin parallel/antiparallel case\
\>", "Subtitle"],

Cell[BoxData[{
    \(We\ define\ \(q\_parallel\) \((x)\)\ to\ be\ the\ probability\ of\ \
finding\ a\ quark\ with\ momentum\), "\[IndentingNewLine]", 
    \(fraction\ x\ and\ the\ same\ spin\ as\ the\ \(\(antiquark\)\(.\)\)\)}], \
"Text",
  FontFamily->"Times"],

Cell[BoxData[{
    \(We\ define\ \(q\_anti\) \((x)\)\ to\ be\ the\ probability\ of\ finding\ \
a\ quark\ with\ momentum\), "\[IndentingNewLine]", 
    \(fraction\ x\ and\ the\ opposite\ spin\ as\ the\ \
\(\(antiquark\)\(.\)\)\)}], "Text",
  FontFamily->"Times"],

Cell[BoxData[
    \(rmserror[data_, fit_, var_] := 
      Sqrt[Apply[Plus, 
            Map[\((\((fit /. var \[Rule] #[\([1]\)])\) - #[\([2]\)])\)^2 &, 
              data]]/Length[data]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(NonLinearFit[data_, fit_, var_, coeff_List] := 
      Block[{v = If[Length[coeff] > 1, Map[First, coeff], var]}, 
        fit /. \(Apply[
              FindMinimum[rmserror[data, fit, var], ##, 
                  MaxIterations \[Rule] 100, 
                  AccuracyGoal \[Rule] Infinity] &, 
              coeff]\)[\([2]\)]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Compute spin parallel/antiparallel quark distributions", "Section"],

Cell[BoxData[
    \(Clear[data, parallel, anti]\)], "Input"],

Cell["\<\
Use data files from web page.  Since they are pretty large, we \
don't try to save them in Mathematica.
The idea here is to cycle through the various values of K (by hand) and then \
dump the results for all K in a data file.\
\>", "Text"],

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c meson_5_40+4_5.m.gz\>", "\<\>"];\)\), "\n", 
    \(\(kk = Apply[Plus, Map[Last, basis[\([\)\(1\)\(]\)]]];\)\), "\n", 
    \(\(pp = Table[{i/kk, 0}, {i, 1/2, kk - 1/2}];\)\), "\n", 
    \(aa = Table[{i/kk, 0}, {i, 1/2, kk - 1/2}]; 
    Do[If[basis[\([\)\(i, 1, 1\)\(]\)] \[Equal] 
          basis[\([\)\(i, \(-1\), 1\)\(]\)], 
        pp[\([\)\(basis[\([\)\(i, 1, \(-1\)\)\(]\)] + 1/2, 2\)\(]\)] += 
          Abs[\(wave[0]\)[\([\)\(i\)\(]\)]]^2\ basis[\([\)\(i, 
                1, \(-1\)\)\(]\)]/2; 
        pp[\([\)\(basis[\([\)\(i, \(-1\), \(-1\)\)\(]\)] + 1/2, 2\)\(]\)] += 
          Abs[\(wave[
                    0]\)[\([\)\(i\)\(]\)]]^2\ basis[\([\)\(i, \(-1\), \
\(-1\)\)\(]\)]/2, 
        aa[\([\)\(basis[\([\)\(i, 1, \(-1\)\)\(]\)] + 1/2, 2\)\(]\)] += 
          Abs[\(wave[0]\)[\([\)\(i\)\(]\)]]^2\ basis[\([\)\(i, 
                1, \(-1\)\)\(]\)]/2; 
        aa[\([\)\(basis[\([\)\(i, \(-1\), \(-1\)\)\(]\)] + 1/2, 2\)\(]\)] += 
          Abs[\(wave[
                    0]\)[\([\)\(i\)\(]\)]]^2\ basis[\([\)\(i, \(-1\), \
\(-1\)\)\(]\)]/2], {i, Length[basis]}]; \), "\n", 
    \(data[kk, parallel] = pp; \), "\n", 
    \(\(data[kk, anti] = aa;\)\)}], "Input"],

Cell["Output to data file.", "Text"],

Cell[BoxData[
    \(Definition[data] >> "\<meson_5+4_5_structure.m\>"\)], "Input"],

Cell["This is the proper normalization:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Map[\((#[\([\(-1\)]\)]/#[\([1]\)])\) &, p]]/kk\)], "Input"],

Cell[BoxData[
    \(0.44549066600907594`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Map[\((#[\([\(-1\)]\)]/#[\([1]\)])\) &, a]]/kk\)], "Input"],

Cell[BoxData[
    \(0.554509333990119`\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Compute spin parallel/antiparallel 2-d distributions", "Section"],

Cell[BoxData[
    \(Clear[data, parallel, anti]\)], "Input"],

Cell["\<\
Use data files from web page.  Since they are pretty large, we \
don't try to save them in Mathematica.
The idea here is to cycle through the various values of K (by hand) and then \
dump the results for all K in a data file.\
\>", "Text"],

Cell[BoxData[
    \(\(datadir = "\<~/public_html/four/\>";\)\)], "Input"],

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c \>" <> 
          datadir <> "\<meson_5_40+4_5.m.gz\>", "\<\>"];\)\), "\n", 
    \(\(kk = Apply[Plus, Map[Last, basis[\([1]\)]]];\)\), "\n", 
    \(\(pp = Table[0, {i, 1/2, kk - 1/2}, {i, 1/2, kk - 1/2}];\)\), "\n", 
    \(aa = Table[0, {i, 1/2, kk - 1/2}, {i, 1/2, kk - 1/2}]; 
    Do[If[basis[\([i, 1, 1]\)] \[Equal] basis[\([i, \(-1\), 1]\)], 
        pp[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 
              basis[\([i, \(-1\), \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2; 
        pp[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 
              basis[\([i, 1, \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2, 
        aa[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 
              basis[\([i, \(-1\), \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2; 
        aa[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 
              basis[\([i, 1, \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2], {i, 
        Length[basis]}];\), "\n", 
    \(\(twod[kk, parallel] = pp;\)\), "\n", 
    \(\(twod[kk, anti] = aa;\)\)}], "Input"],

Cell["Output to data file.", "Text"],

Cell[BoxData[
    \(Definition[data] >> "\<meson_5+4_5_twod.m\>"\)], "Input"],

Cell["This is the normalization (the following should add to 1):", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(totalpara = Apply[Plus, pp, {0, 1}]/kk^2\)], "Input"],

Cell[BoxData[
    \(0.44549066600907594`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(totalanti = Apply[Plus, aa, {0, 1}]/kk^2\)], "Input"],

Cell[BoxData[
    \(0.5545093339901191`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(totalpara + totalanti\)], "Input"],

Cell[BoxData[
    \(0.9999999999991951`\)], "Output"]
}, Open  ]],

Cell["This is not so useful:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{range = {{1/\((2\ kk)\), 1 - 1/\((2\ kk)\)}, {1/\((2\ kk)\), 
              1 - 1/\((2\ kk)\)}}}, 
      Show[ListPlot3D[pp, MeshRange \[Rule] range, ClipFill \[Rule] None, 
          DisplayFunction \[Rule] Identity], 
        ListPlot3D[aa, MeshRange \[Rule] range, ClipFill \[Rule] None, 
          DisplayFunction \[Rule] Identity], 
        ViewPoint \[Rule] {1.0, \(-1.4\), 2.6}, 
        DisplayFunction \[Rule] $DisplayFunction, 
        PlotRange \[Rule] {0.1, 2}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Find the fractional spin parallel/antiparallel asymmetry", "Text"],

Cell[BoxData[
    \(\(percent = 
        Array[If[
              aa[\([##]\)] > 
                0, \((aa[\([##]\)] - pp[\([##]\)])\)/\((aa[\([##]\)] + 
                    pp[\([##]\)])\), 10] &, Dimensions[aa]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pdist = 
      Block[{range = {{1/\((2\ kk)\), 1 - 1/\((2\ kk)\)}, {1/\((2\ kk)\), 
                1 - 1/\((2\ kk)\)}}}, 
        Show[ListPlot3D[percent, MeshRange \[Rule] range, 
            ClipFill \[Rule] None, 
            AxesLabel \[Rule] {"\<x\>", "\<y\>", "\<\>"}, 
            TextStyle \[Rule] {FontSize \[Rule] 14, 
                FontFamily \[Rule] "\<Times\>"}, 
            DisplayFunction \[Rule] Identity], 
          ViewPoint \[Rule] {0.9, \(-1.4\), 0.6}, PlotRange \[Rule] {0, 0.6}, 
          DisplayFunction \[Rule] $DisplayFunction]]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(pdist\)\" is similar to \
existing symbol \"\!\(dist\)\"."\)], "Message"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<relative_20.eps\>", pdist, ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("relative_20.eps"\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analyze spin parallel/antiparallel quark distributions", "Section"],

Cell[BoxData[
    \(\(<< "\<meson_5+4_5_structure.m\>";\)\)], "Input"],

Cell["\<\
This is the one that Simon used (he says that it is not \
well-justified):\
\>", "Text"],

Cell[BoxData[
    \(\(str = data[10, parallel];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[Drop[Drop[str, 1], \(-1\)], 
        x^beta*\((a + b*Sqrt[x] + c\ x)\)*\((1 - x)\)^alpha, 
        x, {{alpha, 0.19}, {beta, 0.001}, {a, 0.1}, {b, \(-0.12\)}, {c, 
            0.4}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(\(\((1 - x)\)\^0.3052679455525083`\ \((\(\(0.10822691190140109`\)\(\
\[InvisibleSpace]\)\) - 0.28181251106291944`\ \@x + 0.4265060435629802`\ x)\)\
\)\/x\^0.052667406060032895`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 1], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.0007783895995448066`\)], "Output"]
}, Open  ]],

Cell["Another arbitraray choice:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[
        Drop[Drop[str, 
            2], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
              c\ LegendreP[2, 2\ x - 1])\)\ x^e\ \((1 - x)\)^d, 
        x, {{a, 0.12}, {b, 0.1}, {d, 0.2}, {c, 0}, {e, 0.2}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(\(\((1 - x)\)\^0.36806349896742074`\ \((\(\(0.12997829888895682`\)\(\
\[InvisibleSpace]\)\) + 0.13306233652701388`\ \((\(-1\) + 2\ x)\) + \
0.037273504834986405`\ \((1 - 6\ x + 6\ \
x\^2)\))\)\)\/x\^0.2376015176823101`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[
        Drop[Drop[str, 
            2], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
              c\ LegendreP[2, 2\ x - 1])\)\ x^e\ \((1 - x)\)^d, 
        x, {{a, 0.14}, {b, 2.77}, {c, 1}, {d, 0.4}, {e, 0.5}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(\((1 - x)\)\^2.1390712172925186`\ x\^5.911877783349889`\ \
\((\(\(75.19629189353596`\)\(\[InvisibleSpace]\)\) - 
          106.44965717243423`\ \((\(-1\) + 2\ x)\) + 
          81.95214099538633`\ \((1 - 6\ x + 6\ x\^2)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 1], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.0004728143918490532`\)], "Output"]
}, Open  ]],

Cell["Compare a fit with the actual data:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[str, DisplayFunction \[Rule] Identity], 
      Plot[fitting, {x, 0, 1}, DisplayFunction \[Rule] Identity], 
      Axes \[Rule] False, \ Frame \[Rule] True, 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Do fits for all K at once (this takes some time):", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#, parallel] = 
            NonLinearFit[Drop[Drop[data[#, parallel], 1], \(-1\)], 
              x^beta*\((a + b*Sqrt[x] + c\ x)\)*\((1 - x)\)^alpha, 
              x, {{alpha, 0.19}, {beta, 0.1}, {a, 0.14}, {b, \(-0.3\)}, {c, 
                  0.6}}])\) &, {10, 12, 15, 20}]; 
    Map[\((sfit[#, anti] = 
            NonLinearFit[Drop[Drop[data[#, anti], 1], \(-1\)], 
              x^beta*\((a + b*Sqrt[x] + c\ x)\)*\((1 - x)\)^alpha, 
              x, {{alpha, 0.19}, {beta, 0.1}, {a, 0.14}, {b, \(-0.3\)}, {c, 
                  0.6}}])\) &, {10, 12, 15, 20}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.18237952703944763`\ \((\(\(0.2059090479323971`\)\(\
\[InvisibleSpace]\)\) - 0.4110639248139638`\ \@x + 
            0.49967696843422926`\ x)\)\ x\^0.04538081438028371`, \((1 - \
x)\)\^0.1790105495200314`\ \((\(\(0.19082560279251992`\)\(\[InvisibleSpace]\)\
\) - 0.39132714064969765`\ \@x + 
            0.510820310359387`\ x)\)\ x\^0.065630873957132`, \((1 - \
x)\)\^0.1939655920858285`\ \((\(\(0.16622897960816596`\)\(\[InvisibleSpace]\)\
\) - 0.38891841899323165`\ \@x + 
            0.5668750884545795`\ x)\)\ x\^0.040524943349059143`, \((1 - \
x)\)\^0.21527445914697796`\ \
\((\(\(0.16714298680066836`\)\(\[InvisibleSpace]\)\) - 
            0.42621872843741765`\ \@x + 
            0.6596720141093798`\ x)\)\ x\^0.09468789327672536`}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfit] >>> "\<meson_5+4_5_structure.m\>"\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Analyze spin parallel/antiparallel quark distributions", "Section"],

Cell[BoxData[
    \(\(<< "\<meson_5+4_5_structure.m\>";\)\)], "Input"],

Cell["Extrapolate result to infinite K and divide by x:", "Text"],

Cell[BoxData[
    \(\(extrapolatedparallel = 
        Table[{x, \((Fit[
                    Map[{#, sfit[#, parallel]} &, {10, 12, 15, 20}], {1, 
                      1/kvalue, 1/kvalue^2}, kvalue] /. 
                  kvalue \[Rule] Infinity)\)/x}, {x, 0.1, 0.9999, 
            0.01}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitextrapolatedparallel = 
      NonLinearFit[extrapolatedparallel, 
        x^beta*\((a + b*Sqrt[x] + c\ x)\)*\((1 - x)\)^alpha, 
        x, {{alpha, 0.19}, {beta, 0.1}, {a, 0.14}, {b, \(-0.3\)}, {c, 
            0.6}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmcv" \(\(:\)\(\ \)\) 
      "Failed to converge to the requested accuracy or precision within \
\!\(100\) iterations."\)], "Message"],

Cell[BoxData[
    \(\(\((1 - x)\)\^0.3621713463244486`\ \((\(\(0.12026154255675296`\)\(\
\[InvisibleSpace]\)\) - 0.423528886982434`\ \@x + 0.7939302318572398`\ x)\)\)\
\/x\^0.7815053851773061`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(extrapolatedanti = 
        Table[{x, \((Fit[
                    Map[{#, sfit[#, anti]} &, {10, 12, 15, 20}], {1, 
                      1/kvalue, 1/kvalue^2}, kvalue] /. 
                  kvalue \[Rule] Infinity)\)/x}, {x, 0.1, 0.9999, 
            0.01}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitextrapolatedanti = 
      NonLinearFit[extrapolatedanti, 
        x^beta*\((a + b*Sqrt[x] + c\ x)\)*\((1 - x)\)^alpha, 
        x, {{alpha, 0.19}, {beta, 0.1}, {a, 0.14}, {b, \(-0.3\)}, {c, 
            0.6}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(\(\((1 - x)\)\^0.31204489730923934`\ \((\(\(0.1493040120873917`\)\(\
\[InvisibleSpace]\)\) - 0.5668934637582177`\ \@x + 1.1262484860309139`\ \
x)\)\)\/x\^0.7225646955467799`\)], "Output"]
}, Open  ]],

Cell["Compare the fits with the K->infinity extrapolation.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[extrapolatedparallel, DisplayFunction \[Rule] Identity], 
      Plot[fitextrapolatedparallel, {x, 0.1, 1}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[extrapolatedanti, DisplayFunction \[Rule] Identity], 
      Plot[fitextrapolatedanti, {x, 0.1, 1}, 
        DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p1 = 
      Apply[Show[
            Plot[fitextrapolatedparallel, {x, 0.1, 1}, 
              DisplayFunction \[Rule] Identity], ##, Axes \[Rule] False, \ 
            Frame \[Rule] True, RotateLabel \[Rule] False, 
            TextStyle \[Rule] {FontSize \[Rule] 14, 
                FontFamily \[Rule] "\<Times\>"}, PlotRange \[Rule] {0, 0.5}, 
            FrameLabel \[Rule] {"\<x\>", "\<q(x)\>"}, 
            DisplayFunction \[Rule] $DisplayFunction] &, 
        Map[{ListPlot[
                Map[{#[\([1]\)], #[\([2]\)]/#[\([1]\)]} &, 
                  data[#, parallel]], 
                DisplayFunction \[Rule] Identity]} &, {10, 12, 15, 
            20}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p2 = 
      Apply[Show[
            Plot[fitextrapolatedanti, {x, 0.1, 1}, 
              DisplayFunction \[Rule] Identity], ##, Axes \[Rule] False, \ 
            Frame \[Rule] True, RotateLabel \[Rule] False, 
            TextStyle \[Rule] {FontSize \[Rule] 14, 
                FontFamily \[Rule] "\<Times\>"}, PlotRange \[Rule] {0, 0.5}, 
            FrameLabel \[Rule] {"\<x\>", "\<q(x)\>"}, 
            DisplayFunction \[Rule] $DisplayFunction] &, 
        Map[{ListPlot[
                Map[{#[\([1]\)], #[\([2]\)]/#[\([1]\)]} &, data[#, anti]], 
                DisplayFunction \[Rule] Identity]} &, {10, 12, 15, 
            20}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Put all the fits together:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(p3 = Show[p1, p2]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<spin_struct_parallel.eps\>", p1, 
      ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("spin_struct_parallel.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<spin_struct_anti.eps\>", p2, 
      ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("spin_struct_anti.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<spin_struct_both.eps\>", p3, 
      ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("spin_struct_both.eps"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{1067, 1092},
WindowMargins->{{Automatic, 115}, {Automatic, 33}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"public_html", "four"}, "meson_5-7_1_polarized.ps", CharacterEncoding -> \
"ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 87, 3, 146, "Subtitle"],
Cell[1829, 56, 261, 5, 70, "Text"],
Cell[2093, 63, 261, 5, 67, "Text"],
Cell[2357, 70, 228, 5, 62, "Input",
  InitializationCell->True],
Cell[2588, 77, 387, 8, 131, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[3000, 89, 73, 0, 77, "Section"],
Cell[3076, 91, 60, 1, 39, "Input"],
Cell[3139, 94, 249, 5, 71, "Text"],
Cell[3391, 101, 1223, 22, 292, "Input"],
Cell[4617, 125, 36, 0, 46, "Text"],
Cell[4656, 127, 82, 1, 39, "Input"],
Cell[4741, 130, 49, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[4815, 134, 91, 1, 39, "Input"],
Cell[4909, 137, 54, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5000, 143, 91, 1, 39, "Input"],
Cell[5094, 146, 52, 1, 39, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5195, 153, 71, 0, 45, "Section"],
Cell[5269, 155, 60, 1, 39, "Input"],
Cell[5332, 158, 249, 5, 71, "Text"],
Cell[5584, 165, 73, 1, 39, "Input"],
Cell[5660, 168, 1143, 21, 292, "Input"],
Cell[6806, 191, 36, 0, 46, "Text"],
Cell[6845, 193, 77, 1, 39, "Input"],
Cell[6925, 196, 74, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[7024, 200, 73, 1, 39, "Input"],
Cell[7100, 203, 54, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7191, 209, 73, 1, 39, "Input"],
Cell[7267, 212, 53, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7357, 218, 54, 1, 39, "Input"],
Cell[7414, 221, 53, 1, 39, "Output"]
}, Open  ]],
Cell[7482, 225, 38, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[7545, 229, 514, 9, 131, "Input"],
Cell[8062, 240, 132, 3, 39, "Output"]
}, Open  ]],
Cell[8209, 246, 72, 0, 46, "Text"],
Cell[8284, 248, 228, 5, 62, "Input"],

Cell[CellGroupData[{
Cell[8537, 257, 592, 11, 131, "Input"],
Cell[9132, 270, 182, 3, 60, "Message"],
Cell[9317, 275, 137, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9491, 283, 93, 1, 39, "Input"],
Cell[9587, 286, 51, 1, 39, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9687, 293, 73, 0, 77, "Section"],
Cell[9763, 295, 70, 1, 39, "Input"],
Cell[9836, 298, 98, 3, 46, "Text"],
Cell[9937, 303, 62, 1, 39, "Input"],

Cell[CellGroupData[{
Cell[10024, 308, 245, 5, 85, "Input"],
Cell[10272, 315, 163, 3, 60, "Message"],
Cell[10438, 320, 210, 3, 69, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10685, 328, 81, 1, 39, "Input"],
Cell[10769, 331, 56, 1, 39, "Output"]
}, Open  ]],
Cell[10840, 335, 42, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[10907, 339, 279, 6, 85, "Input"],
Cell[11189, 347, 163, 3, 60, "Message"],
Cell[11355, 352, 252, 4, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11644, 361, 280, 6, 85, "Input"],
Cell[11927, 369, 163, 3, 60, "Message"],
Cell[12093, 374, 258, 4, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12388, 383, 81, 1, 39, "Input"],
Cell[12472, 386, 56, 1, 39, "Output"]
}, Open  ]],
Cell[12543, 390, 51, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[12619, 394, 251, 4, 85, "Input"],
Cell[12873, 400, 130, 3, 39, "Output"]
}, Open  ]],
Cell[13018, 406, 65, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[13108, 410, 628, 11, 246, "Input"],
Cell[13739, 423, 163, 3, 60, "Message"],
Cell[13905, 428, 163, 3, 60, "Message"],
Cell[14071, 433, 163, 3, 60, "Message"],
Cell[14237, 438, 169, 3, 60, "Message"],
Cell[14409, 443, 788, 13, 146, "Output"]
}, Open  ]],
Cell[15212, 459, 53, 0, 46, "Text"],
Cell[15268, 461, 83, 1, 39, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15388, 467, 73, 0, 77, "Section"],
Cell[15464, 469, 70, 1, 39, "Input"],
Cell[15537, 472, 65, 0, 46, "Text"],
Cell[15605, 474, 304, 6, 131, "Input"],

Cell[CellGroupData[{
Cell[15934, 484, 253, 5, 85, "Input"],
Cell[16190, 491, 168, 3, 60, "Message"],
Cell[16361, 496, 206, 3, 69, "Output"]
}, Open  ]],
Cell[16582, 502, 296, 6, 131, "Input"],

Cell[CellGroupData[{
Cell[16903, 512, 245, 5, 85, "Input"],
Cell[17151, 519, 163, 3, 60, "Message"],
Cell[17317, 524, 207, 3, 69, "Output"]
}, Open  ]],
Cell[17539, 530, 68, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[17632, 534, 405, 7, 131, "Input"],
Cell[18040, 543, 130, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18207, 551, 700, 14, 200, "Input"],
Cell[18910, 567, 130, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19077, 575, 673, 13, 200, "Input"],
Cell[19753, 590, 130, 3, 39, "Output"]
}, Open  ]],
Cell[19898, 596, 42, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[19965, 600, 50, 1, 39, "Input"],
Cell[20018, 603, 130, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20185, 611, 106, 2, 39, "Input"],
Cell[20294, 615, 60, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20391, 621, 102, 2, 39, "Input"],
Cell[20496, 625, 56, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20589, 631, 102, 2, 39, "Input"],
Cell[20694, 635, 56, 1, 39, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

