(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     33892,        893]*)
(*NotebookOutlinePosition[     34534,        916]*)
(*  CellTagsIndexPosition[     34490,        912]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Test Cases for nonzero transverse momentum", "Subtitle"],

Cell[TextData[{
  "When outputting to a file, it seems that \">>\" and \">>>\" don't work \
correctly.  There is a bizarre situation where \"!!file\" gives the desired \
contents, but, outside ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", the file is empty."
}], "Text"],

Cell["\<\
The following uses Simon's for for the Fermion self-inertias.  The \
results for these can be found on the web page.\
\>", "Text"],

Cell[BoxData[
    \(<< "\</home/jdbratt/hamiltonianMatrices.m\>"\)], "Input"],

Cell["Check the nonzero momentum matrix for Hermicity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{npmax = 5, ll, apperp = {0.1234, \(-0.321\)}, 
        couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
            0.688, \(-0.038\), \(-0.09\), 374, 0.158, 143}}, 
      ll = listOfStates[4, npmax - 2, none]; \[IndentingNewLine]Do[
        state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
        If[Chop[\ 
              makeham[state1, state2, apperp, couplings, npmax] - 
                Conjugate[
                  makeham[state2, state1, apperp, couplings, 
                    npmax]]] \[NotEqual] 0, 
          Print[{i, j, state1, state2}]], {i, Length[ll]}, {j, 
          i}]]\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["K=3, 1 link: test for lattice symmetry (completed)", "Section"],

Cell[BoxData[
    \(<< "\</home/jdbratt/hamiltonianMatrices.m\>"\)], "Input"],

Cell[BoxData[
    \(mat[pperp_] := 
      Block[{npmax = 3, ll}, \[IndentingNewLine]ll = 
          listOfStates[3, npmax - 2, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          makeham[state1, state2, 
            pperp, {0.1803, 0.362, 1, \(-0.323\), 0.162, 
              0.688, \(-0.038\), \(-0.09\), 374, 0.158, 143}, npmax], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sort[Chop[Eigenvalues[mat[{0, 0}]]]]\)], "Input"],

Cell[BoxData[
    \({0.07112316999458074`, 0.07306860602365517`, 0.07306860602365517`, 
      0.07512639587717844`, 0.2591280664982728`, 0.26288453250622096`, 
      0.26288453250622107`, 0.2722554141155533`, 0.27225541411555343`, 
      0.2722554141155535`, 0.2722554141155535`, 0.27225541411555354`, 
      0.27225541411555365`, 0.27377731285944346`, 0.2770333915156983`, 
      0.2770333915156983`, 0.28164966534280383`, 0.2851101321334162`, 
      0.2851101321334162`, 0.290854211409482`, 0.31669714707331287`, 
      0.3199927853836556`, 0.31999278538365566`, 0.324740834517384`, 
      0.3419836626664321`, 0.34198366266643215`, 0.3419836626664322`, 
      0.3419836626664324`, 0.34198366266643243`, 0.3419836626664325`, 
      0.3439124573656134`, 0.34391245736561343`, 0.3450747552283526`, 
      0.3450747552283527`, 0.3457291150153539`, 0.3468183397091866`, 
      0.37460891795173645`, 0.3751086683650853`, 0.37510866836508533`, 
      0.37571546875088546`, 0.3811504767190335`, 0.38646587560374474`, 
      0.38646587560374485`, 0.39179881416343165`, 0.546323965855134`, 
      0.546323965855134`, 0.5463239658551342`, 0.5463239658551343`, 
      0.5463239658551343`, 0.5463239658551344`, 0.5474764325052935`, 
      0.547476432505294`, 0.5476208586913353`, 0.5476208586913355`, 
      0.5485322918482732`, 0.5488971630010477`, 0.5824560326946675`, 
      0.583199635693365`, 0.5831996356933652`, 
      0.5839667533305584`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e1 = Sort[Chop[Eigenvalues[mat[{\(-0.11234\),  .04567}]]]]\)], "Input"],

Cell[BoxData[
    \({0.07119765480570618`, 0.0731344340795941`, 0.0731421321177777`, 
      0.07519074286932868`, 0.25880872534701493`, 0.26249800969320736`, 
      0.262564819160153`, 0.27183507687133185`, 0.27225571313704655`, 
      0.27225571939171694`, 0.27226434680061135`, 0.2724291092212207`, 
      0.27243227633684863`, 0.27269029293140157`, 0.27703317727809434`, 
      0.2785669917866764`, 0.2814618849323598`, 0.2854516053869725`, 
      0.28552706932937083`, 0.2910234962916297`, 0.3163868633237746`, 
      0.31960925152912545`, 0.31966136113146254`, 0.3242702903223771`, 
      0.34198383683829486`, 0.34198385274234633`, 0.34198731803165905`, 
      0.34199161410423434`, 0.3420901171762285`, 0.3420916320541006`, 
      0.3439141514237979`, 0.34409085381293975`, 0.345044253587449`, 
      0.3451670573785385`, 0.3457855679717493`, 0.346803014398064`, 
      0.37456857973511737`, 0.3750475991131365`, 0.3750671132718923`, 
      0.37566240137690765`, 0.3814202740634526`, 0.3866622423885659`, 
      0.3866889848102566`, 0.3919626067407357`, 0.5463239870337077`, 
      0.5463239872147696`, 0.5463261788919864`, 0.5463263678411894`, 
      0.5463360365607076`, 0.5463360404579859`, 0.5474761912161079`, 
      0.5474995002347175`, 0.5476212403236264`, 0.5476454381857948`, 
      0.5485409618833361`, 0.5489075173532627`, 0.5824212561242377`, 
      0.5831618607631354`, 0.5831650020541006`, 
      0.583929013377296`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e2 = Sort[Chop[Eigenvalues[mat[{0.11234,  .04567}]]]]\)], "Input"],

Cell[BoxData[
    \({0.07119765480570632`, 0.0731344340795944`, 0.07314213211777802`, 
      0.07519074286932909`, 0.2588087253470147`, 0.26249800969320725`, 
      0.262564819160153`, 0.2718350768713319`, 0.27225571313704655`, 
      0.2722557193917168`, 0.27226434680061146`, 0.2724291092212207`, 
      0.2724322763368487`, 0.2726902929314015`, 0.2770331772780944`, 
      0.2785669917866763`, 0.2814618849323598`, 0.2854516053869725`, 
      0.28552706932937083`, 0.2910234962916297`, 0.3163868633237745`, 
      0.3196092515291254`, 0.31966136113146243`, 0.3242702903223771`, 
      0.34198383683829503`, 0.3419838527423465`, 0.34198731803165916`, 
      0.3419916141042344`, 0.34209011717622856`, 0.34209163205410065`, 
      0.3439141514237979`, 0.3440908538129398`, 0.34504425358744895`, 
      0.3451670573785384`, 0.34578556797174914`, 0.34680301439806394`, 
      0.3745685797351175`, 0.37504759911313656`, 0.37506711327189224`, 
      0.3756624013769079`, 0.38142027406345247`, 0.38666224238856567`, 
      0.38668898481025643`, 0.3919626067407354`, 0.5463239870337077`, 
      0.5463239872147696`, 0.5463261788919868`, 0.5463263678411895`, 
      0.5463360365607076`, 0.5463360404579859`, 0.5474761912161079`, 
      0.5474995002347176`, 0.5476212403236264`, 0.5476454381857948`, 
      0.5485409618833358`, 0.5489075173532625`, 0.5824212561242381`, 
      0.5831618607631357`, 0.583165002054101`, 
      0.5839290133772962`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e3 = Sort[Chop[Eigenvalues[mat[{ .04567, 0.11234}]]]]\)], "Input"],

Cell[BoxData[
    \({0.07119765480570692`, 0.0731344340795945`, 0.07314213211777812`, 
      0.07519074286932949`, 0.25880872534701477`, 0.2624980096932072`, 
      0.26256481916015295`, 0.27183507687133174`, 0.2722557131370464`, 
      0.2722557193917168`, 0.2722643468006112`, 0.2724291092212206`, 
      0.2724322763368486`, 0.2726902929314014`, 0.27703317727809423`, 
      0.27856699178667615`, 0.2814618849323597`, 0.28545160538697234`, 
      0.28552706932937066`, 0.2910234962916297`, 0.3163868633237746`, 
      0.31960925152912545`, 0.31966136113146254`, 0.3242702903223771`, 
      0.3419838368382947`, 0.3419838527423462`, 0.34198731803165866`, 
      0.3419916141042341`, 0.3420901171762285`, 0.3420916320541005`, 
      0.34391415142379783`, 0.34409085381293963`, 0.3450442535874489`, 
      0.34516705737853837`, 0.34578556797174903`, 0.34680301439806394`, 
      0.3745685797351173`, 0.37504759911313645`, 0.3750671132718922`, 
      0.37566240137690754`, 0.3814202740634524`, 0.3866622423885657`, 
      0.3866889848102564`, 0.39196260674073546`, 0.5463239870337079`, 
      0.5463239872147696`, 0.546326178891986`, 0.5463263678411893`, 
      0.5463360365607077`, 0.546336040457986`, 0.547476191216108`, 
      0.5474995002347177`, 0.5476212403236262`, 0.5476454381857948`, 
      0.5485409618833362`, 0.5489075173532629`, 0.582421256124238`, 
      0.5831618607631355`, 0.5831650020541007`, 
      0.5839290133772961`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(e4 = Sort[Chop[Eigenvalues[mat[{0.11234, \(- .04567\)}]]]]\)], "Input"],

Cell[BoxData[
    \({0.07119765480570642`, 0.07313443407959375`, 0.07314213211777737`, 
      0.07519074286932885`, 0.2588087253470146`, 0.2624980096932072`, 
      0.26256481916015295`, 0.2718350768713317`, 0.2722557131370464`, 
      0.27225571939171667`, 0.2722643468006111`, 0.2724291092212204`, 
      0.27243227633684836`, 0.2726902929314013`, 0.2770331772780944`, 
      0.2785669917866763`, 0.28146188493235974`, 0.2854516053869724`, 
      0.2855270693293707`, 0.2910234962916297`, 0.3163868633237746`, 
      0.3196092515291254`, 0.3196613611314625`, 0.3242702903223771`, 
      0.3419838368382948`, 0.34198385274234644`, 0.34198731803165894`, 
      0.3419916141042343`, 0.3420901171762285`, 0.3420916320541004`, 
      0.34391415142379783`, 0.34409085381293963`, 0.3450442535874489`, 
      0.3451670573785383`, 0.3457855679717489`, 0.34680301439806377`, 
      0.3745685797351173`, 0.37504759911313645`, 0.3750671132718922`, 
      0.37566240137690765`, 0.38142027406345236`, 0.3866622423885657`, 
      0.3866889848102564`, 0.39196260674073574`, 0.5463239870337074`, 
      0.5463239872147693`, 0.5463261788919859`, 0.546326367841189`, 
      0.5463360365607075`, 0.5463360404579858`, 0.5474761912161079`, 
      0.5474995002347175`, 0.5476212403236261`, 0.5476454381857946`, 
      0.5485409618833361`, 0.5489075173532626`, 0.5824212561242378`, 
      0.5831618607631354`, 0.5831650020541006`, 
      0.5839290133772966`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({e1 \[Equal] e2, e1 \[Equal] e3, e1 \[Equal] e4}\)], "Input"],

Cell[BoxData[
    \({True, True, True}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["K=5, 3 links: test for lattice symmetry (Completed)", "Section"],

Cell["\<\
The following was run as a batch job (It took several days).  The \
largest error was 10^-15.  This shows the dispersion obeys the lattice \
symmetries.
Each matrix takes 31,000 seconds on bvds.\
\>", "Text"],

Cell[BoxData[
    \( (*<< "\</home/jdbratt/hamiltonianMatrices.m\>"\[IndentingNewLine]\
\[IndentingNewLine]mat[pperp_] := 
        Block[{nglue = 3, ll}, 
            ll = listOfStates[5, nglue, none]; \[IndentingNewLine]Table[
              state1 = ll[\([i]\)]; \ state2 = ll[\([j]\)]; \ 
              makeham[state1, state2, 
                pperp, {0.1803, 0.362, 1, \(-0.323\), 0.162, 
                  0.688, \(-0.038\), \(-0.09\), 374, 0.158, 143}, 
                nglue + 2], {i, Length[ll]}, {j, 
                Length[ll]}]]\[IndentingNewLine]\[IndentingNewLine]\(e1 = 
            Sort[Chop[
                Eigenvalues[
                  mat[{\(-0.11234\),  .24567}]]]]\); \[IndentingNewLine]\
\[IndentingNewLine]Save["\<junk.dat\>", 
          e1]\[IndentingNewLine]\[IndentingNewLine]\(e2 = 
          Sort[Chop[
              Eigenvalues[
                mat[{0.11234,  .24567}]]]]\); \[IndentingNewLine]\
\[IndentingNewLine]Max[
          Abs[e1 - 
              e2]]\[IndentingNewLine]\[IndentingNewLine]Save["\<junk.dat\>", 
          e2]\[IndentingNewLine]\[IndentingNewLine]\(e3 = 
          Sort[Chop[
              Eigenvalues[
                mat[{ .24567, 
                    0.11234}]]]]\); \[IndentingNewLine]\[IndentingNewLine]Max[
          Abs[e1 - 
              e3]]\[IndentingNewLine]\[IndentingNewLine]Save["\<junk.dat\>", 
          e3]\[IndentingNewLine]\[IndentingNewLine]\(e4 = 
          Sort[Chop[
              Eigenvalues[
                mat[{0.11234, \(- .24567\)}]]]]\); \[IndentingNewLine]\
\[IndentingNewLine]Max[
          Abs[e1 - 
              e4]]\[IndentingNewLine]\[IndentingNewLine]Save["\<junk.dat\>", 
          e4]*) \)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=2, 1 link: eigenvalues", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k2onelinktrans1.txt\>"\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(file\)\" is similar to \
existing symbol \"\!\(File\)\"."\)], "Message"],

Cell[BoxData[
    \("/home/bvds/four/k2onelinktrans1.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 2, nglue = 1, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=2, one link truncation, nonzero transverse momentumn\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=2, 1 link: eigenvalues for various couplings", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k2onelinktrans2.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k2onelinktrans2.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 2, nglue = 1, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=2, one link truncation, nonzero transverse momentum\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=4, 1 link: eigenvalues", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k4onelinktrans1.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k4onelinktrans1.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 4, nglue = 1, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=4, one link truncation, nonzero transverse momentumn\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=3, 2 links: eigenvalues", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k3twolinktrans1.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k3twolinktrans1.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 3, nglue = 2, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=3, one link truncation, nonzero transverse momentumn\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=3, 2 links: eigenvalues for various couplings", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k3twolinktrans2.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k3twolinktrans2.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 3, nglue = 2, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=3, two link truncation, nonzero transverse momentum\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=3, 2 links: eigenvalues for more combinations", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k3twolinktrans3.txt\>"\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(file\)\" is similar to \
existing symbol \"\!\(File\)\"."\)], "Message"],

Cell[BoxData[
    \("/home/bvds/four/k3twolinktrans3.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 3, nglue = 2, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=3, two link truncation, nonzero transverse momentum\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 0, \(-0.323\), 0, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, 0, 0, 0, 0, 0, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=4, 2 links: eigenvalues", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k4twolinktrans1.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k4twolinktrans1.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 4, nglue = 2, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=4, two link truncation, nonzero transverse momentumn\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=5, 2 links: eigenvalues", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(file = "\</home/bvds/four/k5twolinktrans1.txt\>"\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/k5twolinktrans1.txt"\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 5, nglue = 2, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=5, two link truncation, nonzero transverse momentumn\n\
\>" <> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n      \
     lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=4, 3 links: eigenvalues", "Section"],

Cell["do as batch job", "Text"],

Cell[BoxData[
    \(file = "\</home/bvds/four/k4threelinktrans.txt\>"\)], "Input"],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 4, nglue = 3, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=4, three link truncation, nonzero transverse \
momentumn\n\>" \
<> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n          \
 lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["K=5, 3 links: eigenvalues", "Section"],

Cell["do as batch job", "Text"],

Cell[BoxData[
    \(file = "\</home/bvds/four/k5threelinktrans.txt\>"\)], "Input"],

Cell[BoxData[
    \(mat[pperp_, cc_] := 
      Block[{k = 5, nglue = 3, ll}, \[IndentingNewLine]ll = 
          listOfStates[k, nglue, none]; \[IndentingNewLine]Table[
          state1 = ll[\([i]\)]; state2 = ll[\([j]\)]; 
          2\ k\ makeham[state1, state2, pperp, cc, nglue + 2], {i, 
            Length[ll]}, {j, Length[ll]}]]\)], "Input"],

Cell[BoxData[
    \(\(Put[
        OutputForm["\<K=5, three link truncation, nonzero transverse \
momentumn\n\>" \
<> \[IndentingNewLine]"\<couplings={mub,muf,g2,kappas,kappaa,beta,\n          \
 lambda1,lambda2,lambda3,lambda4,lambda5}\n\>"], file];\)\)], "Input"],

Cell[BoxData[{
    \(\(couplings = {0.1803, 0.362, 1, \(-0.323\), 0.162, 
          0.688, \(-0.038\), \(-0.09\), 374, 0.158, 
          143};\)\), "\[IndentingNewLine]", 
    \(Save[file, couplings]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0.11234, 0.24567};\)\), "\n", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"],

Cell[BoxData[{
    \(\(ptrans = {0, 0};\)\), "\[IndentingNewLine]", 
    \(PutAppend[
      OutputForm["\<\nEigenvalues for a*Pperp=\>" <> 
          ToString[ptrans]], \[IndentingNewLine]Sort[
        Chop[Eigenvalues[mat[ptrans, couplings]]]], file]\)}], "Input"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
WindowSize->{1017, 938},
WindowMargins->{{250, Automatic}, {70, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 62, 0, 194, "Subtitle"],
Cell[1804, 53, 284, 7, 124, "Text"],
Cell[2091, 62, 140, 3, 92, "Text"],
Cell[2234, 67, 77, 1, 52, "Input"],
Cell[2314, 70, 63, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[2402, 74, 627, 12, 322, "Input"],
Cell[3032, 88, 42, 1, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3111, 94, 69, 0, 106, "Section"],
Cell[3183, 96, 77, 1, 27, "Input"],
Cell[3263, 99, 438, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[3726, 111, 69, 1, 27, "Input"],
Cell[3798, 114, 1450, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5285, 140, 91, 1, 27, "Input"],
Cell[5379, 143, 1451, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6867, 169, 86, 1, 27, "Input"],
Cell[6956, 172, 1452, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8445, 198, 86, 1, 27, "Input"],
Cell[8534, 201, 1451, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10022, 227, 91, 1, 27, "Input"],
Cell[10116, 230, 1450, 21, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11603, 256, 81, 1, 27, "Input"],
Cell[11687, 259, 52, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[11788, 266, 70, 0, 62, "Section"],
Cell[11861, 268, 218, 5, 124, "Text"],
Cell[12082, 275, 1692, 36, 952, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[13811, 316, 43, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[13879, 320, 81, 1, 52, "Input"],
Cell[13963, 323, 181, 3, 145, "Message"],
Cell[14147, 328, 71, 1, 52, "Output"]
}, Open  ]],
Cell[14233, 332, 346, 6, 232, "Input"],
Cell[14582, 340, 261, 4, 232, "Input"],
Cell[14846, 346, 213, 4, 142, "Input"],
Cell[15062, 352, 260, 5, 232, "Input"],
Cell[15325, 359, 265, 5, 232, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[15627, 369, 65, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[15717, 373, 81, 1, 27, "Input"],
Cell[15801, 376, 71, 1, 27, "Output"]
}, Open  ]],
Cell[15887, 380, 346, 6, 75, "Input"],
Cell[16236, 388, 260, 4, 43, "Input"],
Cell[16499, 394, 169, 3, 43, "Input"],
Cell[16671, 399, 260, 5, 59, "Input"],
Cell[16934, 406, 265, 5, 59, "Input"],
Cell[17202, 413, 173, 3, 43, "Input"],
Cell[17378, 418, 260, 5, 59, "Input"],
Cell[17641, 425, 265, 5, 59, "Input"],
Cell[17909, 432, 213, 4, 43, "Input"],
Cell[18125, 438, 260, 5, 59, "Input"],
Cell[18388, 445, 265, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[18690, 455, 43, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[18758, 459, 81, 1, 52, "Input"],
Cell[18842, 462, 71, 1, 52, "Output"]
}, Open  ]],
Cell[18928, 466, 346, 6, 172, "Input"],
Cell[19277, 474, 261, 4, 232, "Input"],
Cell[19541, 480, 213, 4, 112, "Input"],
Cell[19757, 486, 260, 5, 202, "Input"],
Cell[20020, 493, 265, 5, 202, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[20322, 503, 44, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[20391, 507, 81, 1, 27, "Input"],
Cell[20475, 510, 71, 1, 27, "Output"]
}, Open  ]],
Cell[20561, 514, 346, 6, 75, "Input"],
Cell[20910, 522, 261, 4, 43, "Input"],
Cell[21174, 528, 213, 4, 43, "Input"],
Cell[21390, 534, 260, 5, 59, "Input"],
Cell[21653, 541, 265, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[21955, 551, 66, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[22046, 555, 81, 1, 27, "Input"],
Cell[22130, 558, 71, 1, 27, "Output"]
}, Open  ]],
Cell[22216, 562, 346, 6, 75, "Input"],
Cell[22565, 570, 260, 4, 43, "Input"],
Cell[22828, 576, 169, 3, 43, "Input"],
Cell[23000, 581, 260, 5, 59, "Input"],
Cell[23263, 588, 265, 5, 59, "Input"],
Cell[23531, 595, 173, 3, 43, "Input"],
Cell[23707, 600, 260, 5, 59, "Input"],
Cell[23970, 607, 265, 5, 59, "Input"],
Cell[24238, 614, 213, 4, 43, "Input"],
Cell[24454, 620, 260, 5, 59, "Input"],
Cell[24717, 627, 265, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[25019, 637, 66, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[25110, 641, 81, 1, 27, "Input"],
Cell[25194, 644, 181, 3, 23, "Message"],
Cell[25378, 649, 71, 1, 27, "Output"]
}, Open  ]],
Cell[25464, 653, 346, 6, 75, "Input"],
Cell[25813, 661, 260, 4, 43, "Input"],
Cell[26076, 667, 169, 3, 43, "Input"],
Cell[26248, 672, 260, 5, 59, "Input"],
Cell[26511, 679, 265, 5, 59, "Input"],
Cell[26779, 686, 160, 3, 43, "Input"],
Cell[26942, 691, 260, 5, 59, "Input"],
Cell[27205, 698, 265, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[27507, 708, 44, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[27576, 712, 81, 1, 27, "Input"],
Cell[27660, 715, 71, 1, 27, "Output"]
}, Open  ]],
Cell[27746, 719, 346, 6, 75, "Input"],
Cell[28095, 727, 261, 4, 43, "Input"],
Cell[28359, 733, 213, 4, 43, "Input"],
Cell[28575, 739, 260, 5, 59, "Input"],
Cell[28838, 746, 265, 5, 59, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[29140, 756, 44, 0, 62, "Section"],

Cell[CellGroupData[{
Cell[29209, 760, 81, 1, 52, "Input"],
Cell[29293, 763, 71, 1, 52, "Output"]
}, Open  ]],
Cell[29379, 767, 346, 6, 172, "Input"],
Cell[29728, 775, 261, 4, 232, "Input"],
Cell[29992, 781, 213, 4, 112, "Input"],
Cell[30208, 787, 260, 5, 202, "Input"],
Cell[30471, 794, 265, 5, 202, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[30773, 804, 44, 0, 62, "Section"],
Cell[30820, 806, 31, 0, 60, "Text"],
Cell[30854, 808, 82, 1, 82, "Input"],
Cell[30939, 811, 346, 6, 232, "Input"],
Cell[31288, 819, 265, 5, 232, "Input"],
Cell[31556, 826, 213, 4, 142, "Input"],
Cell[31772, 832, 260, 5, 232, "Input"],
Cell[32035, 839, 265, 5, 232, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[32337, 849, 44, 0, 62, "Section"],
Cell[32384, 851, 31, 0, 60, "Text"],
Cell[32418, 853, 82, 1, 82, "Input"],
Cell[32503, 856, 346, 6, 232, "Input"],
Cell[32852, 864, 265, 5, 232, "Input"],
Cell[33120, 871, 213, 4, 142, "Input"],
Cell[33336, 877, 260, 5, 232, "Input"],
Cell[33599, 884, 265, 5, 232, "Input"]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

