(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     24695,        727]*)
(*NotebookOutlinePosition[     26205,        773]*)
(*  CellTagsIndexPosition[     26161,        769]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Examine meson wavfunctions.", "Subtitle"],

Cell[BoxData[
    \(rmserror[data_, fit_, var_] := 
      Sqrt[Apply[Plus, 
            Map[\((\((fit /. var \[Rule] #[\([1]\)])\) - #[\([2]\)])\)^2 &, 
              data]]/Length[data]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(NonLinearFit[data_, fit_, var_, coeff_List] := 
      Block[{v = If[Length[coeff] > 1, Map[First, coeff], var]}, 
        fit /. \(Apply[
              FindMinimum[rmserror[data, fit, var], ##, 
                  MaxIterations \[Rule] 100, 
                  AccuracyGoal \[Rule] Infinity] &, 
              coeff]\)[\([2]\)]]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Compute (unpolarized) quark distributions", "Section"],

Cell[BoxData[
    \(Clear[data]\)], "Input"],

Cell["\<\
Use data files from web page.  Since they are pretty large, we \
don't try to save them in Mathematica.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c ~/public_html/four/meson_5_16-7_1.m.gz\>", \
"\<\>"];\)\), "\[IndentingNewLine]", 
    \(kk = Apply[Plus, Map[Last, basis[\([1]\)]]]\)}], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]],

Cell["Here, we calculate x q(x).", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(str = Table[{i/kk, 0}, {i, 1/2, kk - 1/2}]; 
    Do[str[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 2]\)] += 
        Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, 1, \(-1\)]\)]/2; 
      str[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 2]\)] += 
        Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, \(-1\), \(-1\)]\)]/2, {i, 
        Length[basis]}];\), "\[IndentingNewLine]", 
    \(data[kk] = str\)}], "Input"],

Cell[BoxData[
    \({{1\/16, 0.2707878103752186`}, {3\/16, 0.23776132766991487`}, {5\/16, 
        0.24287066480440317`}, {7\/16, 0.2487155125607057`}, {9\/16, 
        0.24944801825412832`}, {11\/16, 0.24004542370401588`}, {13\/16, 
        0.21158009464770208`}, {15\/16, 0.0005901547656735716`}}\)], "Output"]
}, Open  ]],

Cell["Output to data file.", "Text"],

Cell[BoxData[
    \(Definition[
        data] >> "\<~/public_html/four/meson_5-7_1_structure.m\>"\)], "Input"],

Cell["This is the proper normalization:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Map[\((#[\([\(-1\)]\)]/#[\([1]\)])\) &, str]]/kk\)], "Input"],

Cell[BoxData[
    \(1.000000000000521`\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Analyze (unpolarized) quark distributions", "Section"],

Cell[BoxData[
    \(\(<< "\<~/public_html/four/meson_5-7_1_structure.m\>";\)\)], "Input"],

Cell[BoxData[
    \(\(str = data[21];\)\)], "Input"],

Cell["\<\
This is the one that Simon used (he says that it is not \
well-justified):\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[Drop[Drop[str, 1], \(-1\)], 
        a*\((x^b)\)*\((1 + c*Sqrt[x])\)*\((1 - x)\)^d, 
        x, {{a, 0.19}, {b, 0.001}, {c, 1.9}, {d, 0.5}}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(0.16174224148538333`\ \((1 + 
          1.9037110673808681`\ \@x)\)\ \((1 - x)\)\^0.5840217156929998`\ \
x\^0.08715003963917348`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 1], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.003445950889124595`\)], "Output"]
}, Open  ]],

Cell["This works tons better:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = 
      NonLinearFit[
        Drop[Drop[str, 
            1], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
              c\ LegendreP[2, 2\ x - 1])\)\ \((1 - x)\)^d, 
        x, {{a, 0.14}, {b, 2.77}, {c, 1}, {d, 0.5}}]\)], "Input"],

Cell[BoxData[
    \(\((1 - x)\)\^0.5647564753457877`\ \((\(\(0.4544134787416207`\)\(\
\[InvisibleSpace]\)\) + 0.37750613214348644`\ \((\(-1\) + 2\ x)\) + 
          0.07225125873357553`\ \((1 - 6\ x + 6\ x\^2)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rmserror[Drop[Drop[str, 1], \(-1\)], fitting, x]\)], "Input"],

Cell[BoxData[
    \(0.0013204652186438967`\)], "Output"]
}, Open  ]],

Cell["Do all fits at once:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfit]; 
    Map[\((sfit[#] = 
            NonLinearFit[
              Drop[Drop[data[#], 
                  1], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
                    c\ LegendreP[2, 2\ x - 1])\)\ \((1 - x)\)^d, 
              x, {{a, 0.14}, {b, 2.77}, {c, 1}, {d, 0.5}}])\) &, {8, 11, 15, 
        21}]\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \({\((1 - x)\)\^0.5230332800444796`\ \((\(\(0.3822322957207141`\)\(\
\[InvisibleSpace]\)\) + 0.19417458146372754`\ \((\(-1\) + 2\ x)\) + 
            0.0467913534123938`\ \((1 - 6\ x + 
                  6\ x\^2)\))\), \((1 - x)\)\^0.5169522373386033`\ \
\((\(\(0.38685540111811845`\)\(\[InvisibleSpace]\)\) + 
            0.2317621654560932`\ \((\(-1\) + 2\ x)\) + 
            0.045451473863520046`\ \((1 - 6\ x + 
                  6\ x\^2)\))\), \((1 - x)\)\^0.5317604336474622`\ \
\((\(\(0.4100206504574703`\)\(\[InvisibleSpace]\)\) + 
            0.28919725384119127`\ \((\(-1\) + 2\ x)\) + 
            0.052248684237694784`\ \((1 - 6\ x + 
                  6\ x\^2)\))\), \((1 - x)\)\^0.5647564753457877`\ \
\((\(\(0.4544134787416207`\)\(\[InvisibleSpace]\)\) + 
            0.37750613214348644`\ \((\(-1\) + 2\ x)\) + 
            0.07225125873357553`\ \((1 - 6\ x + 6\ x\^2)\))\)}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[
        sfit] >>> "\<~/public_html/four/meson_5-7_1_structure.m\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[str, DisplayFunction \[Rule] Identity], 
      Plot[fitting, {x, 0, 1}, DisplayFunction \[Rule] Identity], 
      Axes \[Rule] False, \ Frame \[Rule] True, 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Extrapolate result to infinite K:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      Fit[Map[{#, sfit[#]} &, {8, 11, 15, 21}], {1, 1/kvalue, 1/kvalue^2}, 
          kvalue] /. kvalue \[Rule] Infinity, {x, 0, 1}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["Put all the fits together:", "Text"],

Cell[BoxData[
    \(Apply[
      Show[Plot[
            Fit[Map[{#, sfit[#]} &, {8, 11, 15, 21}], {1, 1/kvalue, 
                  1/kvalue^2}, kvalue] /. kvalue \[Rule] Infinity, {x, 0, 1}, 
            DisplayFunction \[Rule] Identity], ##, Axes \[Rule] False, \ 
          Frame \[Rule] True, RotateLabel \[Rule] False, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          FrameLabel \[Rule] {"\<x\>", "\<x q(x)\>"}, 
          DisplayFunction \[Rule] $DisplayFunction] &, 
      Map[{ListPlot[data[#], DisplayFunction \[Rule] Identity], 
            Plot[sfit[#], {x, 0, 1}, 
              DisplayFunction \[Rule] Identity]} &, {8, 11, 15, 
          21}]]\)], "Input"],

Cell[BoxData[{
    RowBox[{"\[IndentingNewLine]", 
      RowBox[{"rho", " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              StyleBox["J",
                FontSlant->"Italic"], 
              StyleBox["z",
                FontSlant->"Italic"]], "=", \(1\ component\)}], ")"}], 
        "unpolarized", " ", "distribution", " ", "function", " ", 
        StyleBox["x",
          FontSlant->"Italic"], " ", 
        StyleBox["q",
          FontSlant->"Italic"], 
        RowBox[{"(", 
          StyleBox["x",
            FontSlant->"Italic"], 
          ")"}]}]}], "\[IndentingNewLine]", \(Three\ link\ truncation, \ 
    K = 8, 11, 15, 
    21, \[IndentingNewLine]along\ with\ extrapolation\ to\ K = \
\(\(\[Infinity]\)\(.\)\)\)}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell[BoxData[{
    RowBox[{"\[IndentingNewLine]", 
      RowBox[{"Pion", " ", "distribution", " ", "function", " ", 
        StyleBox["x",
          FontSlant->"Italic"], " ", 
        StyleBox["q",
          FontSlant->"Italic"], 
        RowBox[{"(", 
          StyleBox["x",
            FontSlant->"Italic"], 
          ")"}]}]}], "\[IndentingNewLine]", \(Three\ link\ truncation, \ 
    K = 8, 11, 15, 
    21, \[IndentingNewLine]along\ with\ extrapolation\ to\ K = \
\(\(\[Infinity]\)\(.\)\)\)}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell["Width 600, left at zero to print ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[
      Show[Plot[\((Fit[
                    Map[{#, sfit[#]} &, {8, 11, 15, 21}], {1, 1/kvalue, 
                      1/kvalue^2}, kvalue] /. kvalue \[Rule] Infinity)\)/
              x, {x, 0, 1}, DisplayFunction \[Rule] Identity], ##, 
          Axes \[Rule] False, \ Frame \[Rule] True, 
          PlotRange \[Rule] {0, 1.5}, RotateLabel \[Rule] False, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          FrameLabel \[Rule] {"\<x\>", "\<q(x)\>"}, 
          DisplayFunction \[Rule] $DisplayFunction] &, 
      Map[{ListPlot[Map[{#[\([1]\)], #[\([2]\)]/#[\([1]\)]} &, data[#]], 
              DisplayFunction \[Rule] Identity], 
            Plot[sfit[#]/x, {x, 0, 1}, 
              DisplayFunction \[Rule] Identity]} &, {8, 11, 15, 
          21}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[BoxData[{
    RowBox[{"\[IndentingNewLine]", 
      RowBox[{"Pion", " ", "distribution", " ", "function", " ", 
        StyleBox["q",
          FontSlant->"Italic"], 
        RowBox[{"(", 
          StyleBox["x",
            FontSlant->"Italic"], 
          ")"}]}]}], "\[IndentingNewLine]", \(Three\ link\ truncation, \ 
    K = 8, 11, 15, 
    21, \[IndentingNewLine]along\ with\ extrapolation\ to\ K = \
\(\(\[Infinity]\)\(.\)\)\)}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell["Width 600, left at zero to print ", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Compute polarized quark distributions", "Section"],

Cell[BoxData[
    \(Clear[datap, datam]\)], "Input"],

Cell["\<\
Use data files from web page.  Since they are pretty large, we \
don't try to save them in Mathematica.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c ~/public_html/four/meson_5_16-7_1.m.gz\>", \
"\<\>"];\)\), "\[IndentingNewLine]", 
    \(kk = Apply[Plus, Map[Last, basis[\([\)\(1\)\(]\)]]]\)}], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]],

Cell["Calculate x q(X)", "Text"],

Cell[BoxData[{
    \(strp = Table[{i/kk, 0}, {i, 1/2, kk - 1/2}]; 
    strm = Table[{i/kk, 0}, {i, 1/2, kk - 1/2}]; 
    Do[If[basis[\([i, 1, 1]\)] \[Equal] 1, 
        strp[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, 1, \(-1\)]\)]/2, 
        strm[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, 1, \(-1\)]\)]/2]; 
      If[basis[\([i, \(-1\), 1]\)] \[Equal] 1, 
        strp[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, \(-1\), \(-1\)]\)]/2, 
        strm[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ basis[\([i, \(-1\), \(-1\)]\)]/2], {i, 
        Length[basis]}];\), "\[IndentingNewLine]", 
    \(datap[kk] = strp; \ datam[kk] = strm;\)}], "Input"],

Cell["This is the proper normalization:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((Apply[Plus, Map[\((#[\([\(-1\)]\)]/#[\([1]\)])\) &, strp]] + 
          Apply[Plus, 
            Map[\((#[\([\)\(-1\)\(]\)]/#[\([\)\(1\)\(]\)])\) &, strm]])\)/
      kk\)], "Input"],

Cell[BoxData[
    \(1.0000000000005314`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Definition[datap, 
        datam] >> "\<~/public_html/four/meson_5-7_1_polarized.m\>"\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Analyze polarized quark distributions for ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox[\(J\_z\),
            FontSlant->"Italic"], "=", "1"}], TraditionalForm]]]
}], "Section"],

Cell[BoxData[
    \(\(<< "\<~/public_html/four/meson_5-7_1_polarized.m\>";\)\)], "Input"],

Cell["Do all fits at once:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[sfitp, 
      sfitm]; {Map[\((sfitp[#] = 
              NonLinearFit[
                Drop[Drop[datap[#], 
                    1], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
                      c\ LegendreP[2, 2\ x - 1] + \ 
                      e\ LegendreP[3, 2\ x - 1])\)\ \((1 - x)\)^d, 
                x, {{a, 0.1}, {b, \(-0.05\)}, {c, 0.1}, {d, 0.5}, {e, 
                    0.01}}])\) &, {8, 11, 15, 21}], 
      Map[\((sfitm[#] = 
              NonLinearFit[
                Drop[Drop[datam[#], 
                    1], \(-1\)], \((a + \ b\ LegendreP[1, 2\ x - 1] + \ 
                      c\ LegendreP[2, 2\ x - 1] + \ 
                      e\ LegendreP[3, 2\ x - 1])\)\ \((1 - x)\)^d, 
                x, {{a, 0.2}, {b, 0.1}, {c, \(-0.1\)}, {d, 0.5}, {e, 
                    0.01}}])\) &, {8, 11, 15, 21}]}\)], "Input"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(FindMinimum::"fmmp" \(\(:\)\(\ \)\) 
      "Machine precision is insufficient to achieve the requested accuracy or \
precision."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(FindMinimum :: \"fmmp\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \({{\((1 - x)\)\^0.7407495753215071`\ \((\(\(0.3121793471132725`\)\(\
\[InvisibleSpace]\)\) + 0.19973607353258213`\ \((\(-1\) + 2\ x)\) + 
              0.11879803932970964`\ \((1 - 6\ x + 6\ x\^2)\) + 
              0.023767549098581208`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.5728779310814613`\ \
\((\(\(0.26283481118342816`\)\(\[InvisibleSpace]\)\) + 
              0.13801288716409596`\ \((\(-1\) + 2\ x)\) + 
              0.06592300354538129`\ \((1 - 6\ x + 6\ x\^2)\) + 
              0.004795124383921212`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.5498362958095816`\ \
\((\(\(0.26414207438343956`\)\(\[InvisibleSpace]\)\) + 
              0.15478546153952089`\ \((\(-1\) + 2\ x)\) + 
              0.0572020848121228`\ \((1 - 6\ x + 6\ x\^2)\) + 
              0.0031158982616324034`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.5646610085177741`\ \
\((\(\(0.2820719882129288`\)\(\[InvisibleSpace]\)\) + 
              0.19532520323195704`\ \((\(-1\) + 2\ x)\) + 
              0.060236292790362335`\ \((1 - 6\ x + 6\ x\^2)\) + 
              0.00371155191071929`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\)}, {\((1 - x)\)\^0.2987991205606181`\ \((\(\
\(0.10477973100051097`\)\(\[InvisibleSpace]\)\) + 
              0.05841708908065279`\ \((\(-1\) + 2\ x)\) - 
              0.031566574086314164`\ \((1 - 6\ x + 6\ x\^2)\) - 
              0.0004529105098959131`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.25421841247165833`\ \
\((\(\(0.1020651417333651`\)\(\[InvisibleSpace]\)\) + 
              0.05779655716734929`\ \((\(-1\) + 2\ x)\) - 
              0.03465921948081873`\ \((1 - 6\ x + 6\ x\^2)\) - 
              0.006854756570974994`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.370798836344239`\ \
\((\(\(0.12311208744949041`\)\(\[InvisibleSpace]\)\) + 
              0.09431209045650475`\ \((\(-1\) + 2\ x)\) - 
              0.025611789100691704`\ \((1 - 6\ x + 6\ x\^2)\) - 
              0.010234350158553623`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\), \((1 - x)\)\^0.392720588361047`\ \
\((\(\(0.13655767411639996`\)\(\[InvisibleSpace]\)\) + 
              0.11779642994891412`\ \((\(-1\) + 2\ x)\) - 
              0.024659358470451507`\ \((1 - 6\ x + 6\ x\^2)\) - 
              0.017268988659984334`\ \((\(-1\) + 12\ x - 30\ x\^2 + 
                    20\ x\^3)\))\)}}\)], "Output"]
}, Open  ]],

Cell["Append all the fits to the data file:", "Text"],

Cell[BoxData[
    \(Definition[sfitp, 
        sfitm] >>> "\<~/public_html/four/meson_5-7_1_polarized.m\>"\)], \
"Input"],

Cell["Put all the fits together:", "Text"],

Cell[BoxData[
    \(Apply[
      Show[Plot[
            Fit[Map[{#, sfitp[#]} &, {8, 11, 15, 21}], {1, 1/kvalue, 
                  1/kvalue^2}, kvalue] /. kvalue \[Rule] Infinity, {x, 0, 1}, 
            DisplayFunction \[Rule] Identity], 
          Plot[Fit[
                Map[{#, sfitm[#]} &, {8, 11, 15, 21}], {1, 1/kvalue, 
                  1/kvalue^2}, kvalue] /. kvalue \[Rule] Infinity, {x, 0, 1}, 
            DisplayFunction \[Rule] Identity], ##, Axes \[Rule] False, \ 
          Frame \[Rule] True, RotateLabel \[Rule] False, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          FrameLabel \[Rule] {"\<x\>", "\<x q(x)\>"}, 
          DisplayFunction \[Rule] $DisplayFunction] &, 
      Map[{ListPlot[datap[#], DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[0]}], 
            ListPlot[datam[#], DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[2/3]}], 
            Plot[sfitp[#], {x, 0, 1}, DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[0]}], 
            Plot[sfitm[#], {x, 0, 1}, DisplayFunction \[Rule] Identity, 
              PlotStyle \[Rule] {Hue[2/3]}]} &, {8, 11, 15, 21}]]\)], "Input"],

Cell[BoxData[{
    RowBox[{"\[IndentingNewLine]", 
      RowBox[{"rho", " ", 
        RowBox[{"(", 
          RowBox[{
            SubscriptBox[
              StyleBox["J",
                FontSlant->"Italic"], 
              StyleBox["z",
                FontSlant->"Italic"]], "=", \(1\ component\)}], ")"}], 
        "polarized", " ", "distribution", " ", "functions", " ", 
        StyleBox["x",
          FontSlant->"Italic"], " ", 
        StyleBox["q",
          FontSlant->"Italic"], 
        RowBox[{"(", 
          StyleBox["x",
            FontSlant->"Italic"], 
          ")"}]}]}], "\[IndentingNewLine]", \(Three\ link\ truncation, \ 
    K = 8, 11, 15, 
    21, \[IndentingNewLine]along\ with\ extrapolation\ to\ K = \
\(\(\[Infinity]\)\(.\)\)\)}], "Text",
  TextAlignment->Center,
  FontFamily->"Times",
  FontSize->15],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"],

Cell["Width 600, left at zero to print ", "Text"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell["Compute 2-d distributions", "Section"],

Cell[BoxData[
    \(Clear[data, parallel, anti]\)], "Input"],

Cell["\<\
Use data files from web page.  Since they are pretty large, we \
don't try to save them in Mathematica.
The idea here is to cycle through the various values of K (by hand) and then \
dump the results for all K in a data file.\
\>", "Text"],

Cell[BoxData[
    \(\(datadir = "\<~/public_html/four/\>";\)\)], "Input"],

Cell[BoxData[{
    \(\(RunThrough["\<gunzip -c \>" <> 
          datadir <> "\<meson_5_40+4_5.m.gz\>", "\<\>"];\)\), "\n", 
    \(\(kk = Apply[Plus, Map[Last, basis[\([1]\)]]];\)\), "\n", 
    \(\(dist = Table[0, {i, 1/2, kk - 1/2}, {i, 1/2, kk - 1/2}];\)\), "\n", 
    \(\(Do[
        dist[\([basis[\([i, 1, \(-1\)]\)] + 1/2, 
              basis[\([i, \(-1\), \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2; 
        dist[\([basis[\([i, \(-1\), \(-1\)]\)] + 1/2, 
              basis[\([i, 1, \(-1\)]\)] + 1/2]\)] += 
          Abs[\(wave[0]\)[\([i]\)]]^2\ kk^2/2, {i, 
          Length[basis]}];\)\), "\n", 
    \(\(twod[kk] = dist;\)\)}], "Input"],

Cell["This is the normalization:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(total = Apply[Plus, dist, {0, 1}]/kk^2\)], "Input"],

Cell[BoxData[
    \(0.9999999999991946`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ptwo = 
      Block[{range = {{1/\((2\ kk)\), 1 - 1/\((2\ kk)\)}, {1/\((2\ kk)\), 
                1 - 1/\((2\ kk)\)}}}, 
        Show[ListPlot3D[Map[If[# \[Equal] 0, \(-0.1\), #] &, dist, {2}], 
            MeshRange \[Rule] range, ClipFill \[Rule] None, 
            DisplayFunction \[Rule] Identity], PlotRange \[Rule] {0, 4}, 
          ViewPoint \[Rule] {2.4, \(-0.2\), 1.8}, 
          TextStyle \[Rule] {FontSize \[Rule] 14, 
              FontFamily \[Rule] "\<Times\>"}, 
          DisplayFunction \[Rule] $DisplayFunction, 
          AxesLabel \[Rule] {"\<x\>", "\<y\>", "\<q(x,y)\>"}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  SurfaceGraphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<2d_struct.eps\>", ptwo, ImageSize \[Rule] 6*72]\)], "Input"],

Cell[BoxData[
    \("2d_struct.eps"\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["junk", "Section"],

Cell[BoxData[
    \(\(RunThrough["\<gunzip -c ~/public_html/four/meson_5_20+4_5.m.gz\>", \
"\<\>"];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Map[\((Abs[#]^2)\) &, wave[0]]]\)], "Input"],

Cell[BoxData[
    \(1.0000000000001734`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{1238, 762},
WindowMargins->{{149, Automatic}, {103, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"public_html", "four"}, "meson_5-7_1_polarized.ps", CharacterEncoding -> \
"ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 47, 0, 99, "Subtitle"],
Cell[1789, 53, 228, 5, 62, "Input",
  InitializationCell->True],
Cell[2020, 60, 387, 8, 108, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2432, 72, 60, 0, 77, "Section"],
Cell[2495, 74, 44, 1, 39, "Input"],
Cell[2542, 77, 127, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[2694, 84, 193, 3, 62, "Input"],
Cell[2890, 89, 35, 1, 39, "Output"]
}, Open  ]],
Cell[2940, 93, 42, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[3007, 97, 413, 7, 108, "Input"],
Cell[3423, 106, 312, 4, 108, "Output"]
}, Open  ]],
Cell[3750, 113, 36, 0, 46, "Text"],
Cell[3789, 115, 110, 2, 39, "Input"],
Cell[3902, 119, 49, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[3976, 123, 93, 1, 39, "Input"],
Cell[4072, 126, 52, 1, 39, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4173, 133, 60, 0, 45, "Section"],
Cell[4236, 135, 89, 1, 39, "Input"],
Cell[4328, 138, 52, 1, 39, "Input"],
Cell[4383, 141, 98, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[4506, 148, 203, 4, 62, "Input"],
Cell[4712, 154, 163, 3, 35, "Message"],
Cell[4878, 159, 162, 3, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5077, 167, 81, 1, 39, "Input"],
Cell[5161, 170, 55, 1, 39, "Output"]
}, Open  ]],
Cell[5231, 174, 39, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[5295, 178, 265, 6, 62, "Input"],
Cell[5563, 186, 228, 3, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5828, 194, 81, 1, 39, "Input"],
Cell[5912, 197, 56, 1, 39, "Output"]
}, Open  ]],
Cell[5983, 201, 36, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[6044, 205, 356, 8, 131, "Input"],
Cell[6403, 215, 163, 3, 35, "Message"],
Cell[6569, 220, 163, 3, 35, "Message"],
Cell[6735, 225, 923, 15, 121, "Output"]
}, Open  ]],
Cell[7673, 243, 53, 0, 46, "Text"],
Cell[7729, 245, 111, 2, 39, "Input"],

Cell[CellGroupData[{
Cell[7865, 251, 251, 4, 62, "Input"],
Cell[8119, 257, 130, 3, 39, "Output"]
}, Open  ]],
Cell[8264, 263, 49, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[8338, 267, 171, 3, 62, "Input"],
Cell[8512, 272, 130, 3, 39, "Output"]
}, Open  ]],
Cell[8657, 278, 42, 0, 46, "Text"],
Cell[8702, 280, 730, 14, 177, "Input"],
Cell[9435, 296, 835, 24, 161, "Text"],
Cell[10273, 322, 130, 3, 39, "Output"],
Cell[10406, 327, 573, 16, 161, "Text"],
Cell[10982, 345, 130, 3, 39, "Output"],
Cell[11115, 350, 49, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[11189, 354, 850, 16, 200, "Input"],
Cell[12042, 372, 130, 3, 39, "Output"]
}, Open  ]],
Cell[12187, 378, 513, 14, 161, "Text"],
Cell[12703, 394, 130, 3, 39, "Output"],
Cell[12836, 399, 49, 0, 46, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[12922, 404, 56, 0, 45, "Section"],
Cell[12981, 406, 52, 1, 39, "Input"],
Cell[13036, 409, 127, 3, 46, "Text"],

Cell[CellGroupData[{
Cell[13188, 416, 201, 3, 62, "Input"],
Cell[13392, 421, 35, 1, 39, "Output"]
}, Open  ]],
Cell[13442, 425, 32, 0, 46, "Text"],
Cell[13477, 427, 856, 14, 177, "Input"],
Cell[14336, 443, 49, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[14410, 447, 204, 4, 62, "Input"],
Cell[14617, 453, 53, 1, 39, "Output"]
}, Open  ]],
Cell[14685, 457, 118, 2, 39, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[14840, 464, 220, 7, 49, "Section"],
Cell[15063, 473, 89, 1, 39, "Input"],
Cell[15155, 476, 36, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[15216, 480, 869, 17, 223, "Input"],
Cell[16088, 499, 163, 3, 35, "Message"],
Cell[16254, 504, 163, 3, 35, "Message"],
Cell[16420, 509, 163, 3, 35, "Message"],
Cell[16586, 514, 169, 3, 35, "Message"],
Cell[16758, 519, 2583, 40, 225, "Output"]
}, Open  ]],
Cell[19356, 562, 53, 0, 46, "Text"],
Cell[19412, 564, 121, 3, 39, "Input"],
Cell[19536, 569, 42, 0, 46, "Text"],
Cell[19581, 571, 1241, 22, 269, "Input"],
Cell[20825, 595, 834, 24, 161, "Text"],
Cell[21662, 621, 130, 3, 39, "Output"],
Cell[21795, 626, 49, 0, 46, "Text"],
Cell[21847, 628, 130, 3, 39, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[22014, 636, 44, 0, 45, "Section"],
Cell[22061, 638, 60, 1, 39, "Input"],
Cell[22124, 641, 249, 5, 71, "Text"],
Cell[22376, 648, 73, 1, 39, "Input"],
Cell[22452, 651, 675, 13, 177, "Input"],
Cell[23130, 666, 42, 0, 46, "Text"],

Cell[CellGroupData[{
Cell[23197, 670, 71, 1, 39, "Input"],
Cell[23271, 673, 53, 1, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23361, 679, 629, 11, 131, "Input"],
Cell[23993, 692, 137, 3, 39, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[24167, 700, 90, 1, 39, "Input"],
Cell[24260, 703, 49, 1, 39, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[24358, 710, 23, 0, 77, "Section"],
Cell[24384, 712, 114, 2, 39, "Input"],

Cell[CellGroupData[{
Cell[24523, 718, 76, 1, 39, "Input"],
Cell[24602, 721, 53, 1, 39, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

