(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    538690,       9414]*)
(*NotebookOutlinePosition[    539655,       9445]*)
(*  CellTagsIndexPosition[    539611,       9441]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Construct the Hamiltonian", "Subtitle"],

Cell["\<\
Brett van de Sande
June 20, 2001\
\>", "Subsubtitle"],

Cell["\<\
The notation used in this notebook corresponds to the notation \
introduced in the LaTeX notes \"endpoint.tex\" and to Simon's paper, \
hep-ph/0101318.\
\>", "Text"],

Cell["\<\
In the following, there are implicit integrations over all momenta, \
implicit sums over repeated flavor indices, and implicit sums over all link \
directions.  The b[...] and d[...] are spinors and vectors in color space.  \
The a[...] are matrices in color space.  sigma[...] and zeta[...] are \
matrices in spinor-space.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Definitions", "Section"],

Cell[BoxData[
    \(Off[General::"\<spell1\>"]\)], "Input",
  InitializationCell->True],

Cell["\<\
All symbols are considered to be c-number unless specified \
otherwise:\
\>", "Text"],

Cell[BoxData[{
    \(cnumberq[x_] := 
      FreeQ[x, sigma[_]] && FreeQ[x, a[__]] && FreeQ[x, b[__]] && 
        FreeQ[x, d[__]] && FreeQ[x, bb[__]] && FreeQ[x, dd[__]] && 
        FreeQ[x, zeta[_]]\), "\[IndentingNewLine]", 
    \(freelistq[x_, y_List] := Apply[And, Map[FreeQ[x, #] &, y]]\)}], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(prodexpand = {\((a__)\) ** b_Plus \[RuleDelayed] 
            Map[\((a ** #)\) &, b], 
          b_Plus ** \((a__)\) \[RuleDelayed] 
            Map[\((# ** a)\) &, b], \[IndentingNewLine]a_ ** 
              b_Times \[RuleDelayed] 
            Select[b, cnumberq]\ a ** Select[b, \(! cnumberq[#]\) &], 
          b_Times ** a_ \[RuleDelayed] 
            Select[b, cnumberq]*
              Select[b, \(! cnumberq[#]\) &] ** a};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(intsumexpand = {a_\  ** \ int[b_, c_] :> 
            int[a ** b, c] /; freelistq[a, c], 
          int[b_, c_] ** a_ :> 
            int[b ** a, c] /; freelistq[a, c], \[IndentingNewLine]a_\  ** \ 
              sum[b_, c_] :> sum[a ** b, c] /; freelistq[a, c], \ 
          sum[b_, c_] ** a_ :> 
            sum[b ** a, c] /; freelistq[a, c], \[IndentingNewLine]int[
              a_Times, b_] :> 
            Module[{vars = Select[a, freelistq[#, b] &]}, 
              vars\ int[Select[a, \(! freelistq[#, b]\) &], b] /; 
                vars =!= 1], \[IndentingNewLine]sum[a_Times, 
              b_] \[RuleDelayed] 
            Module[{vars = Select[a, freelistq[#, b] &]}, 
              vars\ sum[Select[a, \(! freelistq[#, b]\) &], b] /; 
                vars =!= 1], \[IndentingNewLine]int[a_Plus, b_] :> 
            Map[int[#, b] &, a], \[IndentingNewLine]sum[a_Plus, b_] :> 
            Map[sum[#, b] &, a], \[IndentingNewLine]int[d_. \ sum[a_, b_], 
              c_] -> sum[int[d\ a, c], b], \[IndentingNewLine]int\ [
              a_. \ int[b_, c_], d_] :> 
            int[Expand[a\ b], Join[c, d]] /; 
              freelistq[a, c], \[IndentingNewLine]sum\ [a_. \ sum[b_, c_], 
              d_] :> sum[Expand[a\ b], Join[c, d]] /; 
              freelistq[a, c], \[IndentingNewLine]int[a_, b_] :> 
            int[Expand[a], b] /; Expand[a] =!= a};\)\(\ \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(prodfactor = {a_. \ c_NonCommutativeMultiply + 
              b_. \ c_NonCommutativeMultiply \[RuleDelayed] 
            Simplify[Factor[Expand[a] + Expand[b]]]\ c};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(sigmazeta = 
        Join[prodexpand, {\[IndentingNewLine]a[b__] ** sigma[c_] \[Rule] 
              sigma[c] ** a[b], 
            Conjugate[a[b__]] ** sigma[c_] \[Rule] 
              sigma[c] ** Conjugate[a[b]], \[IndentingNewLine]a[b__] ** 
                zeta[c_] \[Rule] zeta[c] ** a[b], 
            Conjugate[a[b__]] ** zeta[c_] \[Rule] 
              zeta[c] ** 
                Conjugate[a[b]], \[IndentingNewLine]sigma[a_Integer] ** 
                sigma[a_Integer] ** b_ -> 
              b, \[IndentingNewLine]b_ ** sigma[a_Integer] ** 
                sigma[a_Integer] -> 
              b, \[IndentingNewLine]sigma[2] ** zeta[a_] ** sigma[1] \[Rule] 
              sigma[1] ** zeta[a] ** sigma[2], 
            sigma[a_Integer] ** sigma[b_Integer] \[RuleDelayed] 
              Block[{c = First[Complement[{1, 2, 3}, {a, b}]]}, 
                  I\ Signature[{a, b, c}]\ sigma[c]] /; 
                a \[NotEqual] b, \[IndentingNewLine]sigma[b_] ** zeta[a_] ** 
                sigma[3] \[Rule] \(-1\)*sigma[b] ** sigma[3] ** zeta[a], 
            sigma[3] ** zeta[a_] \[Rule] \(-1\)*
                zeta[a] ** sigma[3]}];\)\)], "Input",
  InitializationCell->True],

Cell["\<\
Relabel the parameters for implicit sums and integrations.  It acts \
on an expression that is linear in NonCommutativeMultiply[...].\
\>", "Text"],

Cell[BoxData[{
    \(\(relabel[a_Plus] := Map[relabel, a];\)\), "\[IndentingNewLine]", 
    \(relabel[ain_*bin_NonCommutativeMultiply] := 
      Block[{i1 = 1, i2 = 1, j = 1, krules, kcrules, srules, ops, cops}, 
        ops = Cases[Apply[List, bin], 
            a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | 
              bb[_Symbol, ___] | dd[_Symbol, ___]]; 
        cops = Cases[Apply[List, bin], 
            Conjugate[
              a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | 
                bb[_Symbol, ___] | 
                dd[_Symbol, ___]]]; \[IndentingNewLine] (*\(Print[{ops, 
                cops}];\)*) \ \[IndentingNewLine]krules = 
          Map[\((First[#] \[Rule] 
                  Symbol["\<kr\>" <> ToString[\(i2++\)]])\) &, Reverse[ops]]; 
        kcrules = 
          Map[\((First[First[#]] \[Rule] 
                  Symbol["\<kl\>" <> ToString[\(i1++\)]])\) &, cops]; 
        srules = 
          Map[\((# \[Rule] Symbol["\<s\>" <> ToString[\(j++\)]])\) &, 
            Union[Map[Last, 
                Cases[\[IndentingNewLine]\(Apply[List, bin] /. 
                      Conjugate[ex_] \[Rule] ex\) /. 
                    a[_, \(-b_Symbol\)] \[Rule] a[0, b], 
                  a[_, _Symbol] | zeta[_Symbol] | 
                    sigma[_Symbol]\[IndentingNewLine]]]]]; \
\[IndentingNewLine]\(ain\ bin /. Join[krules, kcrules]\) /. 
          srules]\)}], "Input",
  InitializationCell->True],

Cell["\<\
simplify an integration over x, assuming the variable is in a \
delta-function.  It acts on an expression that is linear in delta[...] and \
NonCommutativeMultiply[...].\
\>", "Text"],

Cell[BoxData[{
    \(\(simpint[a_Plus, x_] := 
        Map[simpint[#, x] &, a];\)\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ b_, x_] := 
        a\ simpint[b, x] /; FreeQ[a, x];\)\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ \ delta[\(-x_\) + c_], x_] := 
        simpint[a\ delta[x - c], x];\)\), "\[IndentingNewLine]", 
    \(simpint[a_\ \ delta[x_ + c_], x_] := 
      Simplify[a /. x \[Rule] \(-c\)]\), "\[IndentingNewLine]", 
    \(\(simpint[a_\ , x_] := a /; FreeQ[a, x];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Unprotect[Conjugate];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_Plus] := Map[Conjugate, a];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_Times] := Map[Conjugate, a];\)\), "\[IndentingNewLine]", 
    \(\(Conjugate[a_ ** b_] := 
        Conjugate[b] ** Conjugate[a];\)\), "\[IndentingNewLine]", 
    \(\(Protect[Conjugate];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(sigma /: Conjugate[sigma[a_]] = 
        sigma[a];\)\), "\[IndentingNewLine]", 
    \(\(zeta /: Conjugate[zeta[a_]] := zeta[a];\)\), "\[IndentingNewLine]", 
    \(\(int /: Conjugate[int[a_, b_]] := 
        int[Conjugate[a] /. Map[\((Conjugate[#] \[Rule] #)\) &, b], 
          b];\)\), "\[IndentingNewLine]", 
    \(\(sum /: Conjugate[sum[a_, b_]] := 
        sum[Conjugate[a] /. Map[\((Conjugate[#] \[Rule] #)\) &, b], 
          b];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(intconv[
        x_] := \((int[a_, b_] \[RuleDelayed] 
          int[Integrate[a, x], Complement[b, {x}]] /; 
            MemberQ[b, x])\)\)], "Input",
  InitializationCell->True],

Cell["\<\
A variable x is defined to be positive[x]=True if it is *always* \
positive.\
\>", "Text"],

Cell[BoxData[{
    \(\(Unprotect[Positive];\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_ + b_] := 
        False /; \(! Positive[a]\) && \(! Positive[
                b]\);\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_?Positive + b_?Positive] = 
        True;\)\), "\[IndentingNewLine]", 
    \(\(Positive[a_?Negative\ b_?Positive] = 
        False;\)\), "\[IndentingNewLine]", 
    \(\(Protect[Positive];\)\), "\[IndentingNewLine]", 
    \(\(neverzero[a_] := If[Positive[a], True, True, False];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(On[General::"\<spell1\>"]\)], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Find Yukawa terms (OLD)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(u[x_] := 
      1/Sqrt[2\ Pi]\ int[
          b[kf]\ Exp[\(-I\)\ kf\ x] + 
            sigma[1] ** Conjugate[d[kf]]\ Exp[
                I\ kf\ x], {kf}]\), "\[IndentingNewLine]", 
    \(m[x_, rho_] := 
      1/Sqrt[4\ Pi]\ int[\((a[ka, \(-rho\)]\ Exp[\(-I\)\ ka\ x] + \ 
                Conjugate[a[ka, rho]]\ Exp[I\ ka\ x])\)/
            Sqrt[ka], {ka}]\), "\[IndentingNewLine]", 
    \(kappaa /: Conjugate[kappaa] := kappaa\), "\[IndentingNewLine]", 
    \(kappas /: Conjugate[kappas] := kappas\), "\[IndentingNewLine]", 
    \(muf /: Conjugate[muf] := muf\)}], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kappas\)\" is similar \
to existing symbol \"\!\(kappaa\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(xi[x_] = \ 
      Expand[muf/Sqrt[2]\ sigma[1] ** u[x] + 
            kappaa/Sqrt[2]\ sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
            kappas/Sqrt[2]\ sum[sigma[1] ** m[x, rr] ** u[x], {rr}] //. 
          prodexpand (*\(//.\)\(sigmazeta\)*) ]\)], "Input"],

Cell[BoxData[
    \(cxi[
        x_] = \(\(\(Conjugate[xi[x]] /. {Conjugate[x] \[Rule] x, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll}\) //. prodexpand\) //. 
        sigmazeta\)], "Input"],

Cell[BoxData[
    \(expr = 
      Collect[Expand[\(\(\(\((\(-I\)\ cxi[x] ** int[xi[x], {x}] + 
                        I\ int[cxi[x], {x}] ** xi[x])\)/2 //. prodexpand\) /. 
                intconv[x]\) //. prodexpand\) //. sigmazeta], {muf, kappaa, 
          kappas}]\)], "Input"],

Cell[BoxData[
    \(int[expr, {x}] //. prodexpand\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Find terms in the Hamiltonian", "Section"],

Cell["\<\
This version leaves all of the integrations over momenta and \
position implicit.  Also, sums are implicit.\
\>", "Text"],

Cell[BoxData[{
    \(\(u[x_] := 
        1/Sqrt[2\ Pi]\ \((b[kf]\ Exp[\(-I\)\ kf\ x] + 
              sigma[1] ** Conjugate[d[kf]]\ Exp[
                  I\ kf\ x])\);\)\), "\[IndentingNewLine]", 
    \(\(conju[
          x_] = \(Conjugate[u[x]] /. {Conjugate[x] \[Rule] x, 
              Conjugate[kf] \[Rule] kf}\) /. kf \[Rule] kks;\)\)}], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(m[x_, rho_] := 
        1/Sqrt[4\ Pi]\ \((a[ka, \(-rho\)]\ Exp[\(-I\)\ ka\ x] + \ 
                Conjugate[a[ka, rho]]\ Exp[I\ ka\ x])\)/
            Sqrt[ka];\)\), "\[IndentingNewLine]", 
    \(\(kappaa /: Conjugate[kappaa] := kappaa;\)\), "\[IndentingNewLine]", 
    \(\(kappas /: Conjugate[kappas] := kappas;\)\), "\[IndentingNewLine]", 
    \(muf /: Conjugate[muf] := muf\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kappas\)\" is similar \
to existing symbol \"\!\(kappaa\)\"."\)], "Message"]
}, Open  ]],

Cell["\<\
Xi is used for the Yukawa interactions and the 4-fermion \
interactions\
\>", "Text"],

Cell[BoxData[
    \(\(xi[x_] = \ 
        Expand[\(\(muf/Sqrt[2]\ sigma[1] ** u[x] + 
                  kappaa/Sqrt[2]\ sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
                  kappas/Sqrt[2]\ sigma[1] ** m[x, sumrr] ** u[x] //. 
                prodexpand\) /. {zeta[rho_] ** 
                    a[c_, \(-rho_\)] \[Rule] \(-zeta[rho] ** a[c, rho]\), 
                a[b_, \(-sumrr\)] \[Rule] a[b, sumrr]}\) //. 
            prodexpand];\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(cxi[
        x_] = \(\(\(Conjugate[xi[x]] /. {Conjugate[x] \[Rule] x, 
                Conjugate[ka] \[Rule] ka, Conjugate[kf] \[Rule] kf, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll, sumrr \[Rule] sumll}\) //. 
          prodexpand\) //. sigmazeta\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ muf\ d[kks]\)\/\(2\ \
\@\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ muf\ \
Conjugate[b[kks]] ** sigma[1]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x - \
\[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** Conjugate[a[kka, sumll]]\)\/\(4\ \
\@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** \
Conjugate[a[kka, sumll]]\)\/\(4\ \@kka\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x + \[ImaginaryI]\ kks\ x\)\ kappaa\ Conjugate[b[kks]] ** \
zeta[ll] ** sigma[1] ** a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\
\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ x\)\ kappaa\ \
Conjugate[b[kks]] ** zeta[ll] ** sigma[1] ** Conjugate[a[kka, ll]]\)\/\(4\ \
\@kka\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ x - \
\[ImaginaryI]\ kks\ x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** sigma[1] ** \
a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ \
x - \[ImaginaryI]\ kks\ x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** \
sigma[1] ** Conjugate[a[kka, ll]]\)\/\(4\ \@kka\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
This is to define the momenta variables to be *always * positive.\
\
\>", "Text"],

Cell[BoxData[
    \(kl1 /: Positive[kl1] = True; \ kl2 /: Positive[kl2] = True; \ 
    kl3 /: Positive[kl3] = True;\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(kl4 /: Positive[kl4] = True; \ kl5 /: Positive[kl5] = True; \ 
    kl6 /: Positive[kl6] = True;\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(kr1 /: Positive[kr1] = True; \ kr2 /: Positive[kr2] = True; \ 
    kr3 /: Positive[kr3] = True;\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(kr4 /: Positive[kr4] = True; \ kr5 /: Positive[kr5] = True; \ 
    kr6 /: Positive[kr6] = True;\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Find Yukawa terms", "Subsection"],

Cell[BoxData[
    \(\(expr1 = 
        Collect[relabel[
              Expand[\(\((\(-I\)\ cxi[x] ** Integrate[xi[x], x] + 
                          I\ Integrate[cxi[x], x] ** xi[x])\)/2 //. 
                    prodexpand\) //. sigmazeta]], {muf, kappaa, kappas}] //. 
          prodfactor;\)\)], "Input"],

Cell["\<\
Since the delta[...] is removed in the following, the integration \
region can be complicated.  expr2 contians all of the interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      simpint[\(\(\(int[expr1, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
          delta[_?neverzero] \[Rule] 0, k11]\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\) + 
      kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) - \(\((kl1 + kr1 + kr2 - kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** a[kr3, s1] ** a[kr2, \
s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \
\[Pi]\) + \((\((kl1 + kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \(\((kl1 - kr1 + kr2 + kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\) - \((\((\(-kl1\) + kl2 + kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                      kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\))\) + 
      muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                            kl1\^2\ \((2\ kr1 + kr2)\) + 
                            kl1\ \((\(-2\)\ kr1\^2 + 
                                  kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                          b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                            kr2)\)\ \@kr2\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) - \((\((kl1\^2\ \((kl2 - 
                                  2\ kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((kl2\^2 - 
                                  2\ kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl2 + 
                            kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                            2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[
                            kr1] ** Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1 + 
                            kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                            kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + 
                            kr1 + kr2]\ d[kr2] ** a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                                  kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((\(-2\)\ kl2\^2 + 
                                  kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@kr1\ \@\[Pi])\) + \((\((kr1 - 
                            kr2)\)\ \((kl1\^2 + 2\ kr1\ kr2 - 
                            kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                            kr1)\)\ kr1\ \((kl1 - 
                            kr2)\)\ kr2\ \@\[Pi])\))\) + 
            kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                            kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          a[kr2, s1] ** 
                          b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                            kr2)\)\ \@\[Pi])\) - \((\@kl2\ \((kl1\^2 + 
                            kl1\ kl2 + 
                            kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                            kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 + 
                            kl2\ \((kl2 - kr1)\) - 
                            kl1\ kr1)\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + 
                            kr2\^2 - 
                            kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                            kr1)\)\ kr2\ \((\(-kl1\) + 
                            kr2)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                            kl1\ \((kl2 - kr1)\) + 
                            kr1\^2)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** 
                          sigma[1] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - 
                            kl1\ kr1 + 
                            kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                            kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ kr2\ \((kr1 + kr2)\)\ \@\[Pi])\))\))\) + 
      kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                      kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[
                      kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                      kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr1, s2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                      kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                      kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                      kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                      kr3]\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                      kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                      kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)], "Output"]
}, Closed]],

Cell["\<\
These are the 1\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 2 || 
              Count[non, Conjugate[_]] != 1]\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]1 particle interactions\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] != 1]\)], "Input"],

Cell[BoxData[
    \(muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                          kl1\^2\ \((2\ kr1 + kr2)\) + 
                          kl1\ \((\(-2\)\ kr1\^2 + 
                                kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\((kl1 + 
                          kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\((kr1 - 
                          kr2)\)\ \((kl1\^2 + 2\ kr1\ kr2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          kr2)\)\ kr2\ \@\[Pi])\))\) + 
          kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                          kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + kr2\^2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ kr2\ \((\(-kl1\) + 
                          kr2)\)\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - 
                          kl1\ kr1 + 
                          kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\))\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(muf\ \((kappas\ \((\(-\(\((\((kl1\^2\ \((kl2 - 2\ kr1)\) + 
                            kl2\ \((kl2 - kr1)\)\ kr1 + 
                            kl1\ \((kl2\^2 - 
                                  2\ kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                            kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\((kl2 + 
                          kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                          2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[
                          kr1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                                kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                          kl1\ \((\(-2\)\ kl2\^2 + 
                                kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                          kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \@\[Pi])\))\) + 
          kappaa\ \((\(-\(\((\@kl2\ \((kl1\^2 + kl1\ kl2 + 
                            kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                            kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\@kr1\ \((kl1\^2 \
+ kl2\ \((kl2 - kr1)\) - kl1\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                          kl1\ \((kl2 - kr1)\) + kr1\^2)\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\))\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\(-\(\(\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** a[kr3, s1] ** 
                      a[kr2, s2] ** 
                      b[kr1]\)\/\(16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((\(-kl1\) + 
                        kr3)\)\ \[Pi]\)\)\) + \(\((kl1 - kr1 + kr2 + kr3)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\) - \((\((kl1 - kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + 
                      kr2 + kr3]\ d[kr3] ** sigma[1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + 
                      kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                      kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                      kr3]\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                      kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                      kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + 
                      kl2 - kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                      kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr1, s2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                      kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                      kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                      kr1)\)\ \@kr1\ \((kl1 - 
                      kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                      kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                      kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(kappas\^2\ \((\(-\(\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
                      Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 - 
                        kr1)\)\ \[Pi])\)\)\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + 
                      kl2 + kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                      kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + kr1)\)\ \[Pi])\))\) + 
      kappaa\ kappas\ \((\(-\(\((\((kl1 + kl2 - kl3 + 
                        kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                        b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[a[kl3, s2]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 - 
                        kr1)\)\ \[Pi])\)\)\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                      kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[
                      kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\) + 
      kappaa\^2\ \((\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                      kl3 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                      b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                      kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Find gauge terms", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(jplus = \ 
      Expand[\(\(I\ m[x, rr] ** D[m[x, \(-rr\)] /. ka \[Rule] kaa, x] - 
                  I\ D[m[x, rr], 
                        x] ** \((m[x, \(-rr\)] /. 
                          ka \[Rule] kaa)\) - \((Conjugate[
                          u[x]] /. {Conjugate[x] \[Rule] x, 
                          Conjugate[kf] \[Rule] kf})\) ** \((u[x] /. 
                        kf \[Rule] kff)\) //. prodexpand\) /. 
              a[b_, \(-lambda_\)] ** Conjugate[a[c_, \(-lambda_\)]] \[Rule] 
                a[b, lambda] ** Conjugate[a[c, lambda]]\) //. prodexpand] //. 
        prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka + kaa)\)\ x\)\ \
\((\(-ka\) + kaa)\)\ a[ka, \(-rr\)] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \
\[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka - kaa)\)\ x\)\ \((ka \
+ kaa)\)\ a[ka, rr] ** Conjugate[a[kaa, rr]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((ka - kaa)\)\ x\)\ \((ka + kaa)\)\ \
Conjugate[a[ka, rr]] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((ka + kaa)\)\ x\)\ \((ka - kaa)\)\ \
Conjugate[a[ka, rr]] ** Conjugate[a[kaa, \(-rr\)]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\
\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x - \[ImaginaryI]\ kff\ x\)\ \
Conjugate[b[kf]] ** b[kff]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kf\ x + \[ImaginaryI]\ kff\ x\)\ d[kf] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x \
+ \[ImaginaryI]\ kff\ x\)\ Conjugate[b[kf]] ** sigma[1] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kf\ x - \[ImaginaryI]\ kff\ x\)\ d[kf] ** sigma[1] ** b[kff]\)\/\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["I don't use Expand[...] because it makes too many terms.", "Text"],

Cell[BoxData[
    \(\(expr3 = 
        g^2/4\ relabel[\((jplus /. {ka \[Rule] la, kaa \[Rule] laa, 
                        rr \[Rule] ll, kf \[Rule] lf, 
                        kff \[Rule] lff})\) ** \((I\ Integrate[
                        I\ Integrate[jplus, x], x])\) //. prodexpand] //. 
          prodfactor;\)\)], "Input"],

Cell[TextData[{
  "For the gauge interactions, the evaluation of the color indices is \
implicit and obeys the following rules:\nIf there are four adjoint fields, \
the color indices are evaluated as traces:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(a\_1\) \(a\_2\) \(a\_3\) a\_4\  \[Rule] \ 
        Tr {\(a\_1\) \(a\_2\) \(a\_3\) a\_4} - 
          Tr {\(a\_1\) a\_2}\ Tr {\(a\_3\) a\_4}/N\)]],
  "\nIf there are two adjoint fields and two fundamental fields, the color \
indices are evaluated as:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(a\_1\) a\_2\ b\_1\ b\_2\  \[Rule] \ 
        b\_1\[CenterDot]a\_1\[CenterDot]a\_2\[CenterDot]b\_2\  - 
          Tr {\(a\_1\) a\_2}\ \ b\_1\[CenterDot]b\_2/N\)]],
  "\nIf there are four fundamental fields, the color indices are evaluated \
as:\n",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\ \)\(b\ c\ d\ e\  \[Rule] \ 
        b\_i\ c\_j\ d\_k\ e\_l\ \((\[Delta]\_il\ \[Delta]\_kj\  - \ \
\[Delta]\_ij\ \[Delta]\_kl/N)\)\)\)\)]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 = 
      Distribute[\(\(int[expr3, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
        delta[_?neverzero] \[Rule] 0\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                  kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
                Conjugate[a[kl2, s2]] ** 
                Conjugate[
                  a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl1 - kl2)\)\ \((kr1 - 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, \(-s1\)] ** 
              a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
              Conjugate[
                a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) + \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
              Conjugate[
                a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\@kr2\ \@kr3\ \[Pi])\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
              a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(16\ \((kl1 \
- kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) \
+ kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** d[kr1] ** \
Conjugate[d[kl1]]\)\/\(16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) + \
\((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - 
                kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, s1] ** 
              Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) - \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + 
                kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** 
              a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + kr1)\)\ delta[\(-kl1\) - 
                kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl3, 
                  s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) + \
kr1)\)\^2\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \[Pi]\)\) \((g\
\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[b[kl3]] ** 
              b[kr1])\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
              Conjugate[
                d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \
\((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              b[kr1] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[
                a[kl3, \(-s1\)]])\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) \
- kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** a[kr1, s1] ** \
Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) \
- \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[b[kl1]] ** b[kr2] ** Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \
\@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + \
kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\
\ \((kl2 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[b[kl1]] ** b[kr2] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \
\((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) - \(1\/\(16\ \
\@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - 
                kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
              Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[
                a[kl3, \(-s1\)]])\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) \
- kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** a[kr1, s1] ** \
Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) \
- \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \
\((kl2 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ d[kr2] ** Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \
\((kl2 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
d[kr2] ** Conjugate[d[kl1]] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \
\((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(1\/\(16\ \
\@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ a[kr1, s1] ** 
              Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** sigma[1] ** 
              Conjugate[
                d[kl3]])\) - \(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \
\[Pi]\)\) \((g\^2\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[
                kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[
                d[kl2]])\) + \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) - \(1\/\(16\ \@kl1\ \
\((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** a[kr1, s1] ** Conjugate[b[kl2]] ** sigma[1] ** 
              Conjugate[
                d[kl3]])\) - \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) - \(1\/\(16\ \@kl1\ \
\@kl2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl1 - 
                kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** d[kr2] ** 
              sigma[1] ** 
              b[kr1])\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** b[kr1] ** Conjugate[b[kl2]] ** sigma[1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \((g\^2\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** a[kr1, s1] ** 
              Conjugate[
                a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) + \(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\) \((g\^2\ \
\((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** a[kr2, \(-s1\)] ** 
              a[kr1, s1])\) - \(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \
\[Pi]\)\) \((g\^2\ \((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
              Conjugate[a[kl3, s1]] ** 
              a[kr1, s1])\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** \
b[kr1]\)\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 - \
kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** \
Conjugate[b[kl2]] ** sigma[1] ** Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\
\ \[Pi]\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \[Pi]\)\) \((g\^2\ \
\((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
              b[kr1] ** Conjugate[a[kl1, s1]] ** 
              Conjugate[
                a[kl2, \(-s1\)]])\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ d[kr3] ** Conjugate[d[kl1]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(8\ \
\((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] ** Conjugate[a[kl1, \
s1]]\)\/\(16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl1 \
+ kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** \
Conjugate[a[kl1, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** \
b[kr2] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(8\ \((kl1 - kr1)\)\^2\ \[Pi]\) + \
\(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** \
d[kr1] ** Conjugate[d[kl1]]\)\/\(8\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \
\[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** \
b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(8\ \((kl1 + \
kl2)\)\^2\ \[Pi]\)\)], "Output"]
}, Closed]],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                  kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
                Conjugate[
                  a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\
\ \@kr2\ \@kr3\ \[Pi])\)\)\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
              a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(16\ \((kl1 \
- kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) \
+ kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** d[kr1] ** \
Conjugate[d[kl1]]\)\/\(16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi]\) + \
\((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
                kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + 
                kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ Conjugate[b[kl1]] ** b[kr3] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(16\ \
\@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((\(-kr1\) + kr2)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** a[kr2, \
\(-s1\)] ** a[kr1, s1]\)\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \
\(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, s1] ** \
Conjugate[a[kl1, s1]] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(16\ \@kl1\ \((kr1 \
+ kr2)\)\^2\ \@kr3\ \[Pi]\) - \(g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ \((kl1 + kr1)\)\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] \
** Conjugate[a[kl1, s1]]\)\/\(16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\[Pi]\) - \(g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** \
sigma[1] ** b[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \
\((kl1 - kr1)\)\^2\ \@kr1\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** Conjugate[b[kl1]] ** b[kr1]\)\/\(8\ \
\((kl1 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
d[kr3] ** sigma[1] ** b[kr2] ** d[kr1] ** Conjugate[d[kl1]]\)\/\(8\ \((\(-kl1\
\) + kr1)\)\^2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\((g\^2\ \((kl1 - kl2)\)\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                kr2]\ a[kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
              Conjugate[
                a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
              Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) + \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + 
                kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** 
              a[kr1, s2] ** 
              Conjugate[
                a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\
\)\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
                kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \(g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(16\ \
\@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi]\) - \(g\^2\ \((kl1 + kr2)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \
\[Pi]\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - 
                kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \(g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[b[kl1]] ** b[kr2] ** a[kr1, s1] ** Conjugate[a[kl2, \
s1]]\)\/\(16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl2 \
+ kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** \
b[kr2] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\) + \
\(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** \
d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\
\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** a[kr1, s1] ** Conjugate[a[kl2, s1]]\)\/\(16\ \@kl2\ \
\@kr1\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\) - \(g\^2\ \((kl2 + kr1)\)\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\
\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** b[kr1]\)\/\(8\ \((kl2 - kr1)\)\^2\ \
\[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** \
Conjugate[d[kl1]] ** d[kr1] ** Conjugate[d[kl2]]\)\/\(8\ \((\(-kl2\) + kr1)\)\
\^2\ \[Pi]\) - \(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \[Pi]\)\) \((g\^2\
\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, \(-s1\)] ** 
              a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[
                d[kl2]])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kr1 + kr2)\)\^2\ \
\[Pi]\)\) \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                kr2]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              d[kr2] ** sigma[1] ** 
              b[kr1])\) + \(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\[Pi]\)\) \((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                kr2]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
              a[kr2, \(-s1\)] ** 
              a[kr1, s1])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\[Pi]\)\) \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                kr2] ** sigma[1] ** b[kr1] ** Conjugate[a[kl1, s1]] ** 
              Conjugate[
                a[kl2, \(-s1\)]])\) + \(g\^2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\
\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(8\ \((kl1 + kl2)\)\^2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr4 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                  kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
                Conjugate[a[kl2, s2]] ** 
                Conjugate[
                  a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
              Conjugate[
                a[kl3, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) \
+ kr1)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s2]] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \[Pi]\)\) \((g\
\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[b[kl3]] ** 
              b[kr1])\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
              Conjugate[
                d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \
\((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              b[kr1] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[
                a[kl3, \(-s1\)]])\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                kr1]\ d[kr1] ** Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[
                a[kl3, \(-s1\)]])\) + \(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \
\@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
              Conjugate[b[kl2]] ** sigma[1] ** 
              Conjugate[
                d[kl3]])\) - \(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \
\[Pi]\)\) \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, s1] ** 
              Conjugate[b[kl2]] ** sigma[1] ** 
              Conjugate[
                d[kl3]])\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** b[kr1] ** Conjugate[b[kl2]] ** sigma[1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\) + \((g\^2\ \((kl3 + 
                kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** a[kr1, s1] ** 
              Conjugate[
                a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \
\((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[d[kl2]] ** Conjugate[a[kl3, s1]] ** 
              a[kr1, s1])\) + \(g\^2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** \
b[kr1]\)\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\) + \(g\^2\ delta[\(-kl1\) - kl2 - \
kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr1] ** \
Conjugate[d[kl3]]\)\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\) + \(g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** \
Conjugate[b[kl2]] ** sigma[1] ** Conjugate[d[kl3]]\)\/\(8\ \((kl2 + kl3)\)\^2\
\ \[Pi]\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Gluon effective potential", "Subsection"],

Cell["\<\
I only include the terms that act on meson strings in the large N \
limit.\
\>", "Text"],

Cell[BoxData[
    \(ka /: Positive[ka] = True; kb /: Positive[kb] = True; 
    kc /: Positive[kc] = True; kd /: Positive[kd] = True;\)], "Input"],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Micro]\_B\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
The factor of 1/2 is from the fact that lambda is summing over four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        mub^2/2\ m[x, s1] ** \((m[x, s1] /. ka \[Rule] kb)\);\)\)], "Input"],

Cell[BoxData[
    \(\(expr = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] kk^n}] /. 
            aa_a ** Conjugate[b_] \[Rule] Conjugate[b] ** aa];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      Distribute[\(\(int[expr, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
        delta[_?neverzero] \[Rule] 0\)], "Input"],

Cell[BoxData[
    \(\(mub\^2\ delta[\(-kl1\) + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, \
\(-s1\)]\)\/\(2\ \@kl1\ \@kr1\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The box term \[Beta]", "Subsubsection"],

Cell["\<\
The implicit sums are s1={-1,1} and s2={-2,2}.  There is an overall \
factor of 1/4 since there are no sums in the original expression.\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        m[x, sa] ** \((m[x, sb] /. ka \[Rule] kb)\) ** 
          Conjugate[m[x, sa] /. ka \[Rule] kc] ** 
          Conjugate[m[x, \(-sb\)] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(\(-beta\)/4\ \((term + Conjugate[term])\) //. 
                  prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                  a[kl1, \(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, \(-s1\)] ** 
                a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, s2] ** a[kr2, \(-s1\)] ** a[kr1, s2]\)\/\(32\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, \(-s2\)] ** a[kr2, s1] ** a[kr1, \(-s2\)]\)\/\
\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s2] ** a[kr2, s1] ** a[kr1, s2]\)\
\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + \
kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, \(-s1\)] ** a[kr2, s2] ** \
a[kr1, s1]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, s1] ** \
a[kr2, s2] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) - \
\(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** a[kr3, \
\(-s1\)] ** a[kr2, \(-s2\)] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\
\ \[Pi]\) - \(beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** \
a[kr3, s1] ** a[kr2, \(-s2\)] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kr1\ \@kr2\
\ \@kr3\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                  a[kl1, \(-s1\)]] ** Conjugate[a[kl2, \(-s2\)]] ** 
                a[kr2, \(-s1\)] ** 
                a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)\) - \
\(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** \
Conjugate[a[kl2, s2]] ** a[kr2, \(-s1\)] ** a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s1\)]] ** \
a[kr1, \(-s2\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\
\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s2] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, \(-s2\)]] ** a[kr2, s1] ** a[kr1, s2]\)\/\(32\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr2, \(-s1\
\)] ** Conjugate[a[kl2, \(-s2\)]] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \
\(-s2\)]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s2] ** a[kr1, \
\(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** \
Conjugate[a[kl2, s1]] ** a[kr2, s2] ** a[kr1, s1]\)\/\(32\ \@kl1\ \@kl2\ \
\@kr1\ \@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, s2]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** a[kr1, \
\(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) - \(beta\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, \
\(-s1\)]] ** a[kr2, \(-s2\)] ** a[kr1, \(-s1\)]\)\/\(32\ \@kl1\ \@kl2\ \@kr1\ \
\@kr2\ \[Pi]\) - \(beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \
s2]] ** Conjugate[a[kl2, s1]] ** a[kr2, \(-s2\)] ** a[kr1, s1]\)\/\(32\ \@kl1\
\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((beta\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
                Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, s1]] ** 
                a[kr1, s2])\)\)\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** 
              Conjugate[a[kl3, s1]] ** 
              a[kr1, \(-s2\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, \(-s1\)]] ** 
              a[kr1, s2])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s1\)]] ** 
              a[kr1, \(-s2\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-s2\)]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, \(-s2\)]] ** 
              a[kr1, \(-s1\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-s2\)]] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[a[kl3, \(-s2\)]] ** 
              a[kr1, s1])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s2]] ** 
              a[kr1, \(-s1\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              a[kr1, s1])\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_1\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
There is an implicit sum over the whole expression for \
s1={1,2,-1,-2}.  The factor of 1/2 is because s1 takes on all four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda1/2\ m[x, sa] ** 
            Conjugate[
              m[x, sa] /. ka \[Rule] kb] ** \((m[x, sa] /. ka \[Rule] kc)\) ** 
            Conjugate[m[x, sa] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \
\@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda1\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
              a[kr2, \(-s1\)] ** Conjugate[a[kl2, \(-s1\)]] ** 
              a[kr1, \(-s1\)])\) + \(lambda1\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, s1] ** \
a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda1\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
Conjugate[a[kl2, s1]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda1\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] \
** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda1\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
              a[kr1, \(-s1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda1\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_2\)]],
  " term"
}], "Subsubsection"],

Cell["\<\
There is an implicit sum over the whole expression for \
s1={1,2,-1,-2}.  The factor of 1/2 is because s1 takes on all four \
values\
\>", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda2/2\ m[x, sa] ** \((m[x, sa] /. ka \[Rule] kb)\) ** 
            Conjugate[m[x, sa] /. ka \[Rule] kc] ** 
            Conjugate[m[x, sa] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ b_ \[RuleDelayed] 
                c\ b ** aa /; MemberQ[b, Conjugate[_]], 
              c_\ aa_\  ** \ b_Conjugate \[RuleDelayed] 
                c\ b ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(16\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-s1\)]] ** a[kr3, s1] ** a[kr2, \(-s1\)] ** a[kr1, \
\(-s1\)]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, \(-s1\)] ** \
a[kr2, s1] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \
\(lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, \
s1] ** a[kr2, s1] ** a[kr1, \(-s1\)]\)\/\(16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** 
              a[kr1, \(-s1\)])\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, \(-s1\)] ** \
a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, \(-s1\)] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \
\[Pi]\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] \
** a[kr2, s1] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr1, \(-s1\)]\)\/\(16\ \@kl1\
\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\
\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s1] ** a[kr1, \
\(-s1\)]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \
s1]] ** a[kr2, s1] ** a[kr1, s1]\)\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\
\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s1]] ** 
              a[kr1, \(-s1\)])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[a[kl3, s1]] ** 
              a[kr1, s1])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, \(-s1\)]] ** 
              a[kr1, \(-s1\)])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[a[kl3, \(-s1\)]] ** a[kr1, s1])\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_4\)]],
  " term"
}], "Subsubsection"],

Cell["The implicit sums are s1={-1,1} and s2={-2,2}.", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda4\ m[x, sa] ** 
            Conjugate[
              m[x, sa] /. ka \[Rule] kb] ** \((m[x, sb] /. ka \[Rule] kc)\) ** 
            Conjugate[m[x, sb] /. ka \[Rule] kd];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                  Conjugate[kk_?Positive] \[Rule] kk, 
                  Conjugate[kk_?Positive^n_] \[Rule] 
                    kk^n}] //. {c_\ aa_a\  ** \ bb_ \[RuleDelayed] 
                c\ bb ** aa /; MemberQ[bb, Conjugate[_]], 
              c_\ aa_\  ** \ bb_Conjugate \[RuleDelayed] 
                c\ bb ** aa /; MemberQ[aa, a[__]]}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \
\(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, s2] ** a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \
\@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s2\)] ** a[kr1, s2]\)\/\(8\ \
\@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + \
kr3]\ Conjugate[a[kl1, \(-s2\)]] ** a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** a[kr3, s2] ** \
a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\)\)], \
"Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
              a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s2\)]] ** 
              a[kr1, \(-s1\)])\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + \
kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** a[kr2, s2] ** \
a[kr1, \(-s1\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** \
Conjugate[a[kl2, s2]] ** a[kr1, s2]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\
\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** a[kr1, s2]\)\/\(8\ \@kl1\ \
\@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ \
Conjugate[a[kl1, \(-s2\)]] ** Conjugate[a[kl2, s1]] ** a[kr2, s1] ** a[kr1, \
\(-s2\)]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** Conjugate[a[kl2, \
\(-s2\)]] ** a[kr2, \(-s1\)] ** a[kr1, s1]\)\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\
\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s2\)]] ** 
              a[kr1, \(-s1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
              Conjugate[a[kl3, s2]] ** 
              a[kr1, s2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s2\)]] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
              a[kr1, \(-s2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, s2]] ** Conjugate[a[kl2, \(-s2\)]] ** 
              Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\_4\)]],
  " term (done more explicitly)"
}], "Subsubsection"],

Cell["The implicit sums are s1={-1,1} and s2={-2,2}.", "Text"],

Cell[BoxData[
    \(\(term = 
        lambda4\ Sum[
            m[x, sa] ** 
              Conjugate[
                m[x, sa] /. ka \[Rule] kb] ** \((m[x, sb] /. 
                  ka \[Rule] kc)\) ** 
              Conjugate[m[x, sb] /. ka \[Rule] kd], {sa, \(-1\), 1, 
              2}, {sb, \(-2\), 2, 4}];\)\)], "Input"],

Cell[BoxData[
    \(\(exprb = 
        relabel[Expand[\(term //. prodexpand\) /. {Conjugate[x] \[Rule] x, 
                    Conjugate[kk_?Positive] \[Rule] kk, 
                    Conjugate[kk_?Positive^n_] \[Rule] 
                      kk^n}] //. {c_\ aa_a\  ** \ bb_ \[RuleDelayed] 
                  c\ bb ** aa /; MemberQ[bb, Conjugate[_]], 
                c_\ aa_\  ** \ bb_Conjugate \[RuleDelayed] 
                  c\ bb ** aa /; MemberQ[aa, a[__]]}] //. 
          prodfactor;\)\)], "Input"],

Cell[BoxData[
    \(\(exprbb = 
        Distribute[\(\(int[exprb, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]] /. 
          delta[_?neverzero] \[Rule] 0;\)\)], "Input"],

Cell["\<\
These are the 3\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 1]\)], "Input"],

Cell[BoxData[
    \(\(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] \
** a[kr3, \(-2\)] ** a[kr2, \(-1\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-2\)]] ** a[kr3, \(-2\)] ** a[kr2, 1] ** a[kr1, \
\(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\
\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** a[kr3, \(-1\)] ** a[kr2, \
1] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** a[kr3, 1] ** \
a[kr2, \(-1\)] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) \
+ \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** \
a[kr3, \(-2\)] ** a[kr2, 2] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ \
Conjugate[a[kl1, \(-1\)]] ** a[kr3, \(-1\)] ** a[kr2, \(-2\)] ** a[kr1, \
2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ delta[\(-kl1\) + \
kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** a[kr3, \(-1\)] ** a[kr2, 2] ** \
a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) + \(lambda4\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** a[kr3, 2] ** \
a[kr2, \(-2\)] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi]\) \
+ \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, \(-2\)] ** a[kr2, 2] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 1] ** a[kr2, \(-2\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 1] ** a[kr2, 2] ** a[kr1, \(-2\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** \
a[kr3, 2] ** a[kr2, \(-2\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, \(-1\)] ** a[kr2, 1] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 1] ** a[kr2, \(-1\)] ** a[kr1, 2]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 2] ** a[kr2, \(-1\)] ** a[kr1, 1]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\) + \(lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** \
a[kr3, 2] ** a[kr2, 1] ** a[kr1, \(-1\)]\)\/\(8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, \(-2\)]] ** Conjugate[a[kl2, \(-1\)]] ** 
              a[kr2, \(-1\)] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
              Conjugate[a[kl2, 1]] ** a[kr2, 1] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
              Conjugate[a[kl2, 2]] ** a[kr2, \(-1\)] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
              Conjugate[a[kl2, 2]] ** a[kr2, 1] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              a[kr2, \(-2\)] ** Conjugate[a[kl2, \(-2\)]] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              a[kr2, \(-1\)] ** Conjugate[a[kl2, \(-2\)]] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              a[kr2, \(-1\)] ** Conjugate[a[kl2, 2]] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              a[kr2, 2] ** Conjugate[a[kl2, 2]] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-2\)] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, 1]] ** a[kr2, \(-2\)] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, 1]] ** a[kr2, 2] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, 2]] ** a[kr2, 2] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              a[kr2, \(-2\)] ** Conjugate[a[kl2, \(-2\)]] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              a[kr2, 1] ** Conjugate[a[kl2, \(-2\)]] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              a[kr2, 1] ** Conjugate[a[kl2, 2]] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              a[kr2, 2] ** Conjugate[a[kl2, 2]] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-2\)] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-2\)] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-1\)]] ** a[kr2, 2] ** 
              a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, 2]] ** a[kr2, 2] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-1\)] ** 
              a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-2\)]] ** a[kr2, 1] ** 
              a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-1\)] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, 1]] ** a[kr2, 1] ** 
              a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)], \
"Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]3 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(exprbb /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 4 || 
              Count[non, Conjugate[_]] \[NotEqual] 3]\)], "Input"],

Cell[BoxData[
    \(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
              Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
              a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-2\)]] ** Conjugate[a[kl2, 1]] ** 
              Conjugate[a[kl3, \(-1\)]] ** 
              a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-2\)]] ** Conjugate[a[kl2, 2]] ** 
              Conjugate[a[kl3, \(-1\)]] ** 
              a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-2\)]] ** Conjugate[a[kl2, 2]] ** 
              Conjugate[a[kl3, 1]] ** 
              a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 2]] ** 
              a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-1\)]] ** Conjugate[a[kl2, 1]] ** 
              Conjugate[a[kl3, \(-2\)]] ** 
              a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                a[kl1, \(-1\)]] ** Conjugate[a[kl2, 1]] ** 
              Conjugate[a[kl3, 2]] ** 
              a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
              Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-2\)]] ** 
              a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 2]] ** 
              a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, \(-2\)]] ** 
              a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 2]] ** 
              a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
              Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-2\)]] ** 
              a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, \(-1\)]] ** 
              a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 1]] ** 
              a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
              a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
              Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, \(-1\)]] ** 
              a[kr1, 2])\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Find pseudoscalar 4-fermion interaction to leading order, acts on \
pion\
\>", "Subsection"],

Cell["\<\
The chiral four-fermion interaction in the Hamiltonian is kappa5 \
qq**qq/2 where qq is:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** sigma[2] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** sigma[2] ** u[x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\@2\ kl1\ kl2\ \[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** \
b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\
\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                      a[kr1, s1] ** 
                      Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + 
                        kr1)\)\ \((\(-kl2\) + 
                        kr1)\)\ \[Pi]\^\(3/2\)\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                      kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + \
kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\
\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \
\((kl1 + 2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[a[kl2, \
s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + \
kl2)\)\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ \
d[kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \
\((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\
\ kr1 + kr2)\)\ d[kr2] ** sigma[3] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(4\ \
\@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\
\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\
\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                      kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + \
kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kl1 + kr1 + kr2)\
\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \
\@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\
\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\
\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                      kl2 - kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** 
                    sigma[2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                      kl2)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\@2\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \@2\ kr1\ \
kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + 
                        kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ \
d[kr3] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + \
kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                      sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + 
                        kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ \
d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \
\((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        Collect[relabel[
              Map[Distribute, 
                    qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                        kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                        kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                  Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
              kappas}] //. prodfactor;\)\)\)], "Input"],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      kappa5/2*simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
            delta[_?neverzero] \[Rule] 0, k11]\)\)\)], "Input"],

Cell[BoxData[
    \(1\/2\ kappa5\ \((muf\^2\ \((\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\
\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                        b[kr1])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                          kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[d[kl2]])\))\) + 
          kappas\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
          muf\ \((kappas\ \((\(-\(\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 
                                  kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                  kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                                Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                                d[kr2] ** sigma[2] ** 
                                b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                  kl2)\)\ \((kl2 + 
                                  kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                              a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                                2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ d[kr2] ** sigma[2] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
                kappaa\ \((\(-\(\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + 
                                  kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                  kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                                Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                                zeta[s1] ** a[kr2, s1] ** 
                                b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                  kr2)\)\ \((kr2 + 
                                  kr3)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\((kl1 + 
                                2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                              sigma[2] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                                kr1 + kr2]\ d[kr2] ** sigma[2] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ d[kr3] ** sigma[2] ** zeta[s1] ** 
                              a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[2] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
          kappaa\ kappas\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 
                          2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** sigma[2] ** zeta[s2] ** a[kr2, s2] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
          kappaa\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                        zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\))\))\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Select terms that act on a baryon (no anti-quarks)", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqbaryon = \(qq[x] /. d[_] \[Rule] 0\) /. {0 ** a_ \[Rule] 0, 
          a_ ** 0 \[Rule] 0}\)], "Input"],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\
\ \((kl1 - kr1)\)\ muf\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ \
kl1\ kr1\ \[Pi]\) + 
      kappas\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** 
                      sigma[3] ** a[kr2, s1] ** 
                      b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) + 
                        kr2)\)\ \((kr1 + 
                        kr2)\)\ \[Pi]\^\(3/2\)\)\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 \
- kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \
\[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) \
+ kr2)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                      2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1baryon \
= Collect[relabel[
              Map[Distribute, 
                    qqbaryon /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                        kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                        kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                  Map[Distribute, qqbaryon] //. prodexpand], {muf, kappaa, 
              kappas}] //. prodfactor;\)\)\)], "Input"],

Cell["\<\
Since the delta[...] is removed in the following, the integration \
region can be complicated.  expr2 contians all of the leading baryon \
interactions in the Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(\ \)\(expr2baryon = 
      kappa5/2*simpint[\(\(\(int[expr1baryon, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
            delta[_?neverzero] \[Rule] 0, k11]\)\)\)], "Input"],

Cell[BoxData[
    \(1\/2\ kappa5\ \((\(-\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\((kl2 \
- kr1)\)\ \((kl1 - kr2)\)\ muf\^2\ delta[\(-kl1\) - kl2 + kr1 + 
                      kr2]\ Conjugate[b[kl1]] ** sigma[3] ** b[kr2] ** 
                    Conjugate[b[kl2]] ** sigma[3] ** b[kr1])\)\)\) + 
          kappas\^2\ \((\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                          2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr3, s2] ** 
                        b[kr2] ** Conjugate[b[kl2]] ** sigma[3] ** 
                        Conjugate[a[kl3, s1]] ** 
                        b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                          kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((\(-kl2\) + kr1 + 
                          2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                          2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr4, s2] ** 
                        b[kr3] ** Conjugate[b[kl2]] ** sigma[3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + 
                          kr2)\)\ \((kr1 + kr2)\)\ \@kr4\ \((\(-kl1\) + 
                          kr4)\)\ \((kr3 + kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 
                          2\ kl4 - kr1)\)\ \((kl1 + 2\ kl2 - 
                          kr2)\)\ delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr2] ** Conjugate[b[kl3]] ** 
                        sigma[3] ** Conjugate[a[kl4, s1]] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                          kr1)\)\ \((kl2 - 
                          kr2)\)\ \[Pi]\^2)\) + \((\((\(-kl3\) + kr1 + 
                          2\ kr2)\)\ \((kl1 + 2\ kl2 - 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr3] ** Conjugate[b[kl3]] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                          kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\))\) + 
          muf\ \((kappas\ \((\((\((kl2 - kr1)\)\ \((\(-kl1\) + kr2 + 
                                2\ kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              a[kr3, s1] ** b[kr2] ** Conjugate[b[kl2]] ** 
                              sigma[3] ** 
                              b[kr1])\)/\((8\ kl2\ kr1\ \@kr3\ \((\(-kl1\) + 
                                kr3)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl2 + 
                                2\ kl3 - kr1)\)\ \((kl1 - 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              b[kr2] ** Conjugate[b[kl2]] ** sigma[3] ** 
                              Conjugate[a[kl3, s1]] ** 
                              b[kr1])\)/\((8\ kl1\ \@kl3\ \((kl2 + 
                                kl3)\)\ \((kl3 - 
                                kr1)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\((\(-kl2\
\) + kr1 + 2\ kr2)\)\ \((kl1 - kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              b[kr3] ** Conjugate[b[kl2]] ** sigma[3] ** 
                              a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ \@kr2\ \((\(-kl2\) + 
                                kr2)\)\ \((kr1 + 
                                kr2)\)\ kr3\ \[Pi]\^\(3/2\))\) - \((\((kl3 - 
                                kr1)\)\ \((kl1 + 2\ kl2 - 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[a[kl2, s1]] ** b[kr2] ** 
                              Conjugate[b[kl3]] ** sigma[3] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kl3\ kr1\ \((kl2 - 
                                kr2)\)\ \[Pi]\^\(3/2\))\))\) + 
                kappaa\ \((\(-\(\((\((kl2 + 2\ kl3 - kr1)\)\ \((kl1 - 
                                  kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                  kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                                b[kr2] ** Conjugate[b[kl2]] ** zeta[s1] ** 
                                sigma[3] ** Conjugate[a[kl3, s1]] ** 
                                b[kr1])\)/\((8\ kl1\ \@kl3\ \((kl2 + 
                                  kl3)\)\ \((kl3 - 
                                  kr1)\)\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \((\((\
\(-kl2\) + kr1 + 2\ kr2)\)\ \((kl1 - kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              b[kr3] ** Conjugate[b[kl2]] ** zeta[s1] ** 
                              sigma[3] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ \@kr2\ \((\(-kl2\) + 
                                kr2)\)\ \((kr1 + 
                                kr2)\)\ kr3\ \[Pi]\^\(3/2\))\) - \((\((kl2 - 
                                kr1)\)\ \((\(-kl1\) + kr2 + 
                                2\ kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                              sigma[3] ** a[kr3, s1] ** b[kr2] ** 
                              Conjugate[b[kl2]] ** sigma[3] ** 
                              b[kr1])\)/\((8\ kl2\ kr1\ \@kr3\ \((\(-kl1\) + 
                                kr3)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl3 - 
                                kr1)\)\ \((kl1 + 2\ kl2 - 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                              sigma[3] ** Conjugate[a[kl2, s1]] ** b[kr2] ** 
                              Conjugate[b[kl3]] ** sigma[3] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kl3\ kr1\ \((kl2 - 
                                kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
          kappaa\ kappas\ \((\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                          2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr3, s2] ** 
                        b[kr2] ** Conjugate[b[kl2]] ** zeta[s1] ** sigma[3] ** 
                        Conjugate[a[kl3, s1]] ** 
                        b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                          kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((\(-kl2\) + kr1 + 
                          2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                          2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr4, s2] ** 
                        b[kr3] ** Conjugate[b[kl2]] ** zeta[s1] ** sigma[3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + 
                          kr2)\)\ \((kr1 + kr2)\)\ \@kr4\ \((\(-kl1\) + 
                          kr4)\)\ \((kr3 + kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 
                          2\ kl4 - kr1)\)\ \((kl1 + 2\ kl2 - 
                          kr2)\)\ delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr2] ** Conjugate[b[kl3]] ** 
                        zeta[s1] ** sigma[3] ** Conjugate[a[kl4, s1]] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                          kr1)\)\ \((kl2 - 
                          kr2)\)\ \[Pi]\^2)\) - \((\((\(-kl3\) + kr1 + 
                          2\ kr2)\)\ \((kl1 + 2\ kl2 - 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr3] ** Conjugate[b[kl3]] ** 
                        zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                          kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\) - \((\((kl2 + 
                          2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                          2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        a[kr3, s2] ** b[kr2] ** Conjugate[b[kl2]] ** 
                        sigma[3] ** Conjugate[a[kl3, s1]] ** 
                        b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                          kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((\(-kl2\) + kr1 + 
                          2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                          2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        a[kr4, s2] ** b[kr3] ** Conjugate[b[kl2]] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + 
                          kr2)\)\ \((kr1 + kr2)\)\ \@kr4\ \((\(-kl1\) + 
                          kr4)\)\ \((kr3 + kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 
                          2\ kl4 - kr1)\)\ \((kl1 + 2\ kl2 - 
                          kr2)\)\ delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr2] ** Conjugate[b[kl3]] ** 
                        sigma[3] ** Conjugate[a[kl4, s1]] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                          kr1)\)\ \((kl2 - 
                          kr2)\)\ \[Pi]\^2)\) + \((\((\(-kl3\) + kr1 + 
                          2\ kr2)\)\ \((kl1 + 2\ kl2 - 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr3] ** Conjugate[b[kl3]] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                          kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\))\) + 
          kappaa\^2\ \((\(-\(\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                            2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                            kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                          sigma[3] ** a[kr3, s2] ** b[kr2] ** 
                          Conjugate[b[kl2]] ** zeta[s1] ** sigma[3] ** 
                          Conjugate[a[kl3, s1]] ** 
                          b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                            kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                            kr3)\)\ \[Pi]\^2)\)\)\) - \((\((\(-kl2\) + kr1 + 
                          2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                          2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        a[kr4, s2] ** b[kr3] ** Conjugate[b[kl2]] ** 
                        zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + 
                          kr2)\)\ \((kr1 + kr2)\)\ \@kr4\ \((\(-kl1\) + 
                          kr4)\)\ \((kr3 + kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 
                          2\ kl4 - kr1)\)\ \((kl1 + 2\ kl2 - 
                          kr2)\)\ delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr2] ** Conjugate[b[kl3]] ** 
                        zeta[s1] ** sigma[3] ** Conjugate[a[kl4, s1]] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                          kr1)\)\ \((kl2 - 
                          kr2)\)\ \[Pi]\^2)\) - \((\((\(-kl3\) + kr1 + 
                          2\ kr2)\)\ \((kl1 + 2\ kl2 - 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** b[kr3] ** Conjugate[b[kl3]] ** 
                        zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                          kr2)\)\ \((kl2 - 
                          kr3)\)\ \[Pi]\^2)\))\))\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Testing", "Subsection"],

Cell[BoxData[{
    \(\(v[x_] = I*Integrate[xi[x], x];\)\), "\[IndentingNewLine]", 
    \(\(conjv[x_] = \(-I\)*Integrate[cxi[x], x];\)\)}], "Input"],

Cell["psibar psi:  0^++", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(\(-\((conju[x] ** sigma[1] ** v[x] + 
                        conjv[x] ** sigma[1] ** u[x])\)\)/Sqrt[2] //. 
                prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
      prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 + kr1)\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
d[kr1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((\(-kl1\) + kl2)\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((\(-kr1\) + kr2)\)\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\
\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                      b[kr1]\)\/\(8\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^\(3/2\)\)\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] \
** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - \
kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ \
x\)\ \((kl1 + kr2)\)\ d[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \
\((kl1 - kr1)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 - kl2)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(8\ \
\((kl1 - kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 - kl3)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]]\)\
\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + kl3)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((kr1 - kr2)\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\
\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 - \
kr3)\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + \
kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\
\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                      a[kr2, s1] ** 
                      b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \[Pi]\^\(3/2\)\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - kr1)\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \
\@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                      kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[1] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                      kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kl1 + kr1 + kr2)\
\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \
\@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ \
d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 \
+ kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** 
                    sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                      kl2)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^1 psi:  1^--", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(\((conju[x] ** sigma[3] ** v[x] + 
                      conjv[x] ** sigma[3] ** u[x])\)/Sqrt[2] //. 
                prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
      prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                      b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\)\)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 + kl2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) \
+ \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 - kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \
\[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \
\((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** \
sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 \
+ kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 \
- kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - \
2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\
\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((\(-kl2\) + kr1)\)\ \[Pi]\^\(3/2\)\) - \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                      2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 \
- kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[kr2] ** sigma[2] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\
\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\
\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 \
+ kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** sigma[3] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\
\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** \
zeta[s1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl2 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^\
\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[2] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\
\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr1, \
s1] ** Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\(1\/\(8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** \
sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \
\@kl1\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \
\@kl1\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** \
sigma[2] ** zeta[s1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((kr1 \
+ kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ \
x\)\ d[kr2] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\
\ \@kr1\ \((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 \
- kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 \
+ kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** sigma[3] ** \
a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\(1\
\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\)\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\)\) + \(\(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\)\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** \
sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \
\@kl1\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^2 psi:  1^+-", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(\(-I\)\ \((conju[x] ** v[x] - conjv[x] ** u[x])\)/
                    Sqrt[2] //. prodexpand\) //. sigmazeta]], {muf, kappas, 
          kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x\)\ \((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                      Conjugate[
                        d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \
\((kr1 + kr2)\)\ d[kr2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \(\[ImaginaryI]\
\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 \
- kr1)\)\ d[kr1] ** sigma[1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \
\[Pi]\))\) + 
      kappas\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((\(-kl2\) + kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \
\((kl2 + kl3)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ \
d[kr2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] \
** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** \
sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 \
+ kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 \
- kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \
\((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[kr2] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr1, \
s1] ** Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, \
s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** a[kr2, s1] \
** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + \
kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl2, \
s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \
\((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] \
** b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x \
- \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** zeta[s1] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x \
- \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\(1\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\)\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(\(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\)\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \
\@kl1\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^+ psi:  0^+-", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(conju[x] ** u[x] //. prodexpand\) //. 
              sigmazeta]], {muf, kappas, kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell["psibar gamma^- psi:  0^+-", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(conjv[x] ** v[x] //. prodexpand\) //. 
              sigmazeta]], {muf, kappas, kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\^2\ \((\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** a[kr2, \
s1] ** Conjugate[a[kl2, s2]] ** b[kr1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\ \@kr2\
\ \((\(-kl1\) + kr2)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** \
a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\
\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** b[kr1]\
\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - kr1)\)\ \[Pi]\^2\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]]\)\/\(16\ \@kl1\ \@kl2\ \((kl2 + \
kl3)\)\ \((kl1 - kr1)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** a[kr1, \
s1] ** Conjugate[a[kl1, s2]] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((kl1 + \
kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** Conjugate[d[kl2]]\)\/\(16\ \
\@kl1\ \@kr1\ \((\(-kl2\) + kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^2\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 \
+ kr3)\)\ x\)\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x \
+ \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \
\((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, \
s2] ** Conjugate[d[kl2]]\)\/\(16\ \@kr1\ \((\(-kl2\) + kr1)\)\ \@kr2\ \
\((\(-kl1\) + kr2)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \
s2]] ** b[kr1]\)\/\(16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\
\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** \
Conjugate[a[kl1, s2]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 \
+ kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + \
kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** \
sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
      muf\ \((kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** \
a[kr2, s1] ** b[kr1]\)\/\(8\ kl1\ kr1\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((\(-kl1\)\ kl2 + 2\ kl1\ kr1 + kl2\ kr1)\)\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - \
kr1)\)\ kr1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 + kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ kl2\ \((kl1 + \
kl2)\)\ \((kl1 - kr1)\)\ kr1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((kr1 + 2\ \
kr2)\))\)\ d[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ kl1\ \((kl1 - \
kr1)\)\ \@kr1\ kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ \
kl2 - kl1\ kr1 - kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            
                            kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl2\ kl3 + 
                            kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kr1\ kr2 + \
kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(8\ \@kl1\ kr1\ \((\(-kl1\) + kr1)\)\ \((kl1 - kr2)\)\ kr2\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** \
a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + \
kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
            kappaa\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kl1 + kr1)\)\ \@kr2\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, \
s1] ** b[kr1]\)\/\(8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \@kl2\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\
\ kr1\ \[Pi]\^\(3/2\)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((kl1 - kl2)\)\ \@kr1\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \@kl2\ \
\((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \@kl1\ \((kr1 - kr2)\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \
\((kl1 - kr1)\)\ kr1\ \((kl1 - kr2)\)\ kr2\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \
\((kr1 - kr3)\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl1\ \
\((kl2 + kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \@kr1\ \((kl1 + 
                            kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                          sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ kr2\ \((kr1 + 
                            kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
      kappaa\ kappas\ \((\(-\(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                        b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                      Conjugate[a[kl2, s2]] ** 
                      b[kr1]\)\/\(16\ \@kl2\ \((kl2 - 
                        kr1)\)\ \@kr2\ \((\(-kl1\) + 
                        kr2)\)\ \[Pi]\^2\)\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^2\
\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \
\[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, \
s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\
\) + kr3)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** \
zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^2\) - \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((\(-kl3\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** \
a[kr2, s1] ** b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \
\[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, \
s2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\
\ \((kr2 + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\
\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, \
s2]] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - \
kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\
\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s1] ** \
a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((kr3 + kr4)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 \
+ kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** \
zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ \
d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ \
d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
      kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\
\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^5 psi:  0^-+", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(I \((conju[x] ** sigma[2] ** v[x] + 
                        conjv[x] ** sigma[2] ** u[x])\)/Sqrt[2] //. 
                prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
      prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x\)\ \((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** \
Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((\(-kl1\) + kr1)\)\ Conjugate[b[kl1]] ** \
sigma[3] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** 
                      sigma[2] ** a[kr1, s1] ** 
                      Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + 
                        kr1)\)\ \((\(-kl2\) + 
                        kr1)\)\ \[Pi]\^\(3/2\)\)\)\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                      2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \
\((\(-kl1\) + kr2)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - kr1)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \
\@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ \
d[kr1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \
\@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ \
kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\
\(8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\
\ kr1 + kr2)\)\ d[kr2] ** sigma[3] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \
\@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ \
x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\
\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[2] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                      2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \
\@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                      2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ \
kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ \
x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \
\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                      kl2)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                      2\ kr1 + kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                    sigma[2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^5 gamma^1 psi:  1^++", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(\(-\((conju[x] ** v[x] + conjv[x] ** u[x])\)\)/
                  Sqrt[2] //. prodexpand\) //. sigmazeta]], {muf, kappas, 
          kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((\(-kl1\) + kl2)\)\ Conjugate[b[kl1]] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\
\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \
\((\(-kr1\) + kr2)\)\ d[kr2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
d[kr1] ** sigma[1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\))\) + 
      kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\
\)\ \((kl1 - kl2)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\
\(8\ \((kl1 - kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 - kl3)\)\
\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]]\)\/\(8\ \
\@kl2\ \((kl1 + kl2)\)\ \((kl2 + kl3)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ \
d[kr2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 - kr3)\)\ d[kr3] ** a[kr2, s1] ** b[kr1]\)\
\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr1)\)\
\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \((kl1 - kr2)\
\)\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] \
** sigma[1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\
\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 - kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \
\((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr2)\)\ d[kr2] ** sigma[1] ** a[kr1, s1] \
** Conjugate[d[kl1]]\)\/\(8\ \((kl1 - kr1)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^\
\(3/2\)\))\) + 
      kappaa\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                      a[kr1, s1] ** 
                      Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + 
                        kr1)\)\ \[Pi]\^\(3/2\)\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]]\)\
\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \
\@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl2 \
- kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
+ \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^\
\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\
\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ d[kr1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, \
s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\
\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \
\((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) - \(\(1\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(\(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** \
zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 \
- kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** \
sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\
\)\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^5 gamma^2 psi:  1^-+", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(I \((conju[x] ** sigma[3] ** v[x] - 
                        conjv[x] ** sigma[3] ** u[x])\)/Sqrt[2] //. 
                prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
      prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 + kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** b[kr1]\)\/\(4\ kl1\ kr1\ \
\[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ \
x\)\ \((kl1 - kl2)\)\ Conjugate[b[kl1]] ** sigma[3] ** \
Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ d[kr1] ** sigma[2] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ \
d[kr2] ** sigma[3] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\
\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** a[kr2, s1] ** 
                      b[kr1]\)\/\(8\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^\(3/2\)\)\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] \
** sigma[2] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\
\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((\(-kl1\) + kl2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(8\ \
\@kr1\ \((\(-kl1\) + kr1)\)\ \((\(-kl2\) + kr1)\)\ \[Pi]\^\(3/2\)\) - \((\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \
\((kl1 - kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** \
sigma[2] ** Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 \
+ kl2)\)\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr2)\)\ d[kr2] ** \
sigma[2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\
\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((kr1 - kr2)\)\ d[kr2] ** sigma[3] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\
\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\
\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 - \
kr3)\)\ d[kr3] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + \
kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      b[kr1]\)\/\(8\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^\(3/2\)\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(8\ \@kl2\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \
\@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(8\ \@kl2\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(8\ \@kr1\ \((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) + \
\(\(1\/\(8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\)\)\)\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[
                      kl3]])\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x \
- \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[2] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ d[kr1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, \
s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\
\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** sigma[2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ \@kr1\ \
\((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] \
** zeta[s1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \
\((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\(1\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(\(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** \
zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 \
- kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** \
sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\
\)\))\)\)], "Output"]
}, Closed]],

Cell["psibar gamma^+ gamma^5 psi:  1^-+", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(conju[x] ** sigma[3] ** u[x] //. prodexpand\) //. 
              sigmazeta]], {muf, kappas, kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\
\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["psibar gamma^- gamma^5 psi:  1^-+", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Collect[
        relabel[Expand[\(conjv[x] ** sigma[3] ** v[x] //. prodexpand\) //. 
              sigmazeta]], {muf, kappas, kappaa}] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\
\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(4\ kl1\ \
kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ d[kr1] ** sigma[3] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \
\(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\
\))\) + kappas\^2\ \((\(-\(\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                        b[kl1]] ** sigma[2] ** a[kr1, s1] ** 
                      Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr1\ \((\(-kl1\) + 
                        kr1)\)\ \[Pi]\^2)\)\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 \
- kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl2]]\)\/\(16\ \@kr1\ \((\(-kl2\) + kr1)\)\ \@kr2\ \((\(-kl1\) + \
kr2)\)\ \[Pi]\^2\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** \
b[kr1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^2\
\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \
\[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr3, \
s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\
\) + kr3)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\
\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** \
Conjugate[a[kl2, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\
\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                      b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ \
d[kr1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) \
+ kr2)\)\ x\)\ d[kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ d[kr2] ** sigma[3] ** a[kr1, s1] ** Conjugate[a[kl1, \
s2]] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((kl1 + kl2)\)\ \@kr1\ \((kr1 + \
kr2)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \
\[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** a[kr1, s2] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \
\@kr1\ \((\(-kl2\) + kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^2\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** \
Conjugate[a[kl1, s2]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^2\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[2] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \
\((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\^2\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[2] ** \
a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
      muf\ \((kappas\ \((\(-\(\((\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 - kl1\ kr1 - 
                              kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                            a[kr1, s1] ** 
                            Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                              kr1)\)\ \((kl2 - 
                              kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\)\)\) - \((\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \
\((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                          Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kr1\)\ kr2 + kl1\ \((2\ kr1 \
+ kr2)\))\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \
kl1\ kr1\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1\ \((kl2 \
- 2\ kr1)\) - kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 \
+ kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[2] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ kr1\ \((\(-kl1\) + kr1)\)\ \
\((kl1 - kr2)\)\ kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** \
sigma[3] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ kl1\ \((kl1 - kr1)\)\ \
\@kr1\ kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ \
kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \
\[Pi]\^\(3/2\)\))\) + 
            kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 - kl1\ kr1 - 
                            kl2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          sigma[2] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) - \((\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \
\((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          sigma[2] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 \
- kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                          a[kr2, s1] ** 
                          b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                            kr2)\)\ \@kr2\ \((kr1 + 
                            kr2)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1\ \
\((kl2 - 2\ kr1)\) - kl2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                          Conjugate[a[kl2, s1]] ** 
                          b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\
\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\
\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[2] ** zeta[s1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                            kr1)\)\ kr1\ \((kl1 - 
                            kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ \
kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, \
s1] ** b[kr1]\)\/\(8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \
\[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 - kr1)\)\ x\)\ \((kl1\ \((kl2 - kr1)\) - 2\ kl2\ kr1)\)\ d[kr1] ** 
                          sigma[1] ** zeta[s1] ** sigma[2] ** 
                          Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\
\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kr1\)\ \
kr2 + kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                          a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                            kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
      kappaa\ kappas\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\
\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \
\[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[3] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) \
+ kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) \
+ kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, \
s1] ** Conjugate[a[kl1, s2]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\
\ \((kr2 + kr3)\)\ \[Pi]\^2\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2\) - \
\(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + \
kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** a[kr2, s1] ** \
b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) - \
\(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[2] ** zeta[s1] ** a[kr3, \
s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + \
kr4)\)\ \[Pi]\^2\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[2] ** \
zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ \
x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ \
d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ \
d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
      kappaa\^2\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** zeta[s2] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** a[kr3, s1] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) \
+ kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi]\^2)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[2] ** zeta[s1] ** zeta[s2] ** a[kr3, \
s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + \
kr4)\)\ \[Pi]\^2\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                    sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
d[kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ \
d[kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\))\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Find pseudoscalar 4-fermion interaction to leading order, with \
flavor\
\>", "Subsection"],

Cell[BoxData[{
    \(\(u[x_, f_] := 
        1/Sqrt[2\ Pi]\ \((b[kf, f]\ Exp[\(-I\)\ kf\ x] + 
              sigma[1] ** Conjugate[d[kf, f]]\ Exp[
                  I\ kf\ x])\);\)\), "\[IndentingNewLine]", 
    \(kappaa /: Conjugate[kappaa[a_]] := kappaa[a]\), "\[IndentingNewLine]", 
    \(kappas /: Conjugate[kappas[a_]] := kappas[a]\), "\[IndentingNewLine]", 
    \(muf /: Conjugate[muf[a_]] := muf[a]\)}], "Input",
  InitializationCell->True],

Cell["\<\
Xi is used for the Yukawa interactions and the 4-fermion \
interactions\
\>", "Text"],

Cell[BoxData[
    \(\(xi[x_, f_] = \ 
        Expand[\(\(muf[f]/Sqrt[2]\ sigma[1] ** u[x, f] + 
                  kappaa[f]/Sqrt[2]\ sigma[1] ** zeta[rr] ** m[x, rr] ** 
                      u[x, f] - 
                  kappas[f]/Sqrt[2]\ sigma[1] ** m[x, sumrr] ** u[x, f] //. 
                prodexpand\) /. {zeta[rho_] ** 
                    a[c_, \(-rho_\)] \[Rule] \(-zeta[rho] ** a[c, rho]\), 
                a[b_, \(-sumrr\)] \[Rule] a[b, sumrr]}\) //. 
            prodexpand];\)\)], "Input",
  InitializationCell->True],

Cell["We need the definition of xi from the Yukawa interactions", "Text"],

Cell[BoxData[
    \(cxi[x_, 
        f_] = \(\(\(Conjugate[xi[x, f]] /. {Conjugate[x] \[Rule] x, 
                Conjugate[ka] \[Rule] ka, Conjugate[kf] \[Rule] kf, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll, sumrr \[Rule] sumll}\) //. 
          prodexpand\) //. sigmazeta\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(conju[x_, 
        f_] = \(Conjugate[u[x, f]] /. {Conjugate[x] \[Rule] x, 
            Conjugate[kf] \[Rule] kf}\) /. kf \[Rule] kks\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ Conjugate[b[kks, f]] + \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ d[kks, f] ** \
sigma[1]\)\/\@\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell["\<\
The chiral four-fermion interaction in the Hamiltonian is kappa5 \
qq**qq/2 where qq is:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(qq[x_, f1_, f2_] = 
      relabel[Expand[\(I\ conju[x, f1] ** sigma[2] ** 
                      Integrate[xi[x, f2], x] - 
                  I\ Integrate[cxi[x, f1], x] ** sigma[2] ** u[x, f2] //. 
                prodexpand\) //. sigmazeta]] //. prodfactor\)\)\)], "Input"],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Clear[qqpinch]; 
    qqpinch[x_, f1_, f2_] = 
      qq[x, f1, f2] /. 
        a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(qq[x, f1, f2]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        relabel[Map[Distribute, 
                  qqpinch[x, f1, f2] /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                      kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                      kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                Map[Distribute, qqpinch[x, f2, f1]] //. prodexpand] //. 
          prodfactor;\)\)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl1\)\" is similar to \
existing symbol \"\!\(kl1\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl2\)\" is similar to \
existing symbol \"\!\(kl2\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl3\)\" is similar to \
existing symbol \"\!\(kl3\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell1\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr1\)\" is similar to \
existing symbols \!\({kkl1, kr1}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr2\)\" is similar to \
existing symbols \!\({kkl2, kr2}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr3\)\" is similar to \
existing symbols \!\({kkl3, kr3}\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell\"\) will be suppressed during \
this calculation."\)], "Message"]
}, Open  ]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ \
constant*) \)\(\[IndentingNewLine]\)\(expr2pinch = 
      kappa5/2*simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
            delta[_?neverzero] \[Rule] 0, k11]\)\)\)], "Input"],

Cell[BoxData[
    \(1\/2\ kappa5\ int[
        relabel[qq[x, f1, f2] ** qq[x, f2, f1]], {x}]\)], "Output"]
}, Open  ]],

Cell["Just look at the term that acts in the two particle sector.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2pinch /. {kappas[_] \[Rule] 0, kappaa[_] \[Rule] 0}\)], "Input"],

Cell[BoxData[
    \(1\/2\ kappa5\ int[
        relabel[qq[x, f1, f2] ** qq[x, f2, f1]], {x}]\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Select terms that act on a baryon (no anti-quarks)", "Subsubsection"],

Cell[BoxData[{
    \(\(Clear[qqbaryon];\)\), "\[IndentingNewLine]", 
    \(qqbaryon[x_, f1_, 
        f2_] = \(qq[x, f1, f2] /. d[_] \[Rule] 0\) /. {0 ** a_ \[Rule] 0, 
          a_ ** 0 \[Rule] 0}\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1baryon \
= relabel[Map[Distribute, 
                  qqbaryon[x, f1, f2] /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                      kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                      kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                Map[Distribute, qqbaryon[x, f2, f1]] //. prodexpand] //. 
          prodfactor;\)\)\)], "Input"],

Cell[BoxData[
    \($Aborted\)], "Output"]
}, Open  ]],

Cell["\<\
Since the delta[...] is removed in the following, the integration \
region can be complicated.  expr2 contians all of the leading baryon \
interactions in the Hamiltonian.\
\>", "Text"],

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(\ \)\(expr2baryon = 
      kappa5/2*simpint[\(\(\(int[expr1baryon, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
            delta[_?neverzero] \[Rule] 0, k11]\)\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Find transverse vector 4-fermion interaction to leading order, acts \
on rho J_z = -1,1\
\>", "Subsection"],

Cell["\<\
The vector four-fermion interaction in the Hamiltonian is -kappa6 \
(qq**qq-rr**rr)/2 where qq is:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** sigma[3] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** sigma[3] ** u[x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                      b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 + kl2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \
\[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ \
x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[d[kl1]]\)\/\(2\ \@2\ \
kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \
\[Pi]\))\) + 
      kappas\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** \
sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \
\((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - kr1)\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \
\@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \
\@kr1\ \((\(-kl1\) + kr1)\)\ \((\(-kl2\) + kr1)\)\ \[Pi]\^\(3/2\)\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                      2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 \
- kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \[Pi]\
\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 \
- kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[kr2] ** sigma[2] ** a[kr1, \
s1] ** Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\
\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - \
kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** sigma[3] ** a[kr2, s1] \
** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \
\[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\
\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** \
zeta[s1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl2 - kr1)\
\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] \
** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[2] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\
\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[3] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + \
kr1)\)\ \[Pi]\^\(3/2\)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\
\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr1)\)\ x\)\ d[kr1] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ d[kr1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, \
s1]] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\
\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** a[kr1, \
s1] ** Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
- kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** sigma[2] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + \
kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \((\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \
\((kl2 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, \
s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \
\((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, \
s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(rr[x_] = 
      Collect[relabel[
            Expand[\(\(-I\)\ conju[x] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** u[x] //. prodexpand\) //. 
                sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                      Conjugate[
                        d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** \
sigma[1] ** b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[1] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\))\) + 
      kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\
\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((\(-kl2\) + \
kr1)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** Conjugate[a[kl2, \
s1]] ** Conjugate[d[kl3]]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + \
kl3)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\
\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[kr2] ** Conjugate[a[kl1, \
s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \
\@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + \
kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\
\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 \
- kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\ \[Pi]\^\(3/2\)\) \
+ \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + \
kl2 - kr1)\)\ d[kr1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - kr1)\)\ \[Pi]\
\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \
\((\(-kl1\) + 2\ kr1 + kr2)\)\ d[kr2] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) \
+ \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\
\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** \
Conjugate[d[kl3]]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \
\((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\
\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
\((\(-kl2\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl2 - kr1)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x + \
\[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\
\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 - \
kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** zeta[s1] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + \
kl2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** a[kr1, \
s1] ** Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((\(-kl1\) + kr1)\)\ \[Pi]\^\(3/2\)\) \
- \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - \
kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                      b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \
\@2\ \@kl1\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \
\@2\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ kr1\ \
kr2\ \[Pi]\))\) + 
      kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + 
                        kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ \
d[kr3] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + \
kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x \
+ \[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                        kl2)\)\ \[Pi]\^\(3/2\))\)\)\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rrpinch = 
      rr[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                      Conjugate[
                        d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\
\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** \
Conjugate[d[kl3]]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + kl3)\)\ \[Pi]\
\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ \
x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \
\@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, \
s1]] ** Conjugate[d[kl3]]\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\
\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \
\@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                      b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        Collect[relabel[
                Map[Distribute, 
                      qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, 
                          kr2 \[Rule] kkr2, kr3 \[Rule] kkr3, 
                          s1 \[Rule] ss1}] ** Map[Distribute, qqpinch] //. 
                  prodexpand] - 
              relabel[Map[Distribute, 
                      rrpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, 
                          kr2 \[Rule] kkr2, kr3 \[Rule] kkr3, 
                          s1 \[Rule] ss1}] ** Map[Distribute, rrpinch] //. 
                  prodexpand], {muf, kappaa, kappas}] //. 
          prodfactor;\)\)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl1\)\" is similar to \
existing symbol \"\!\(kl1\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl2\)\" is similar to \
existing symbol \"\!\(kl2\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl3\)\" is similar to \
existing symbol \"\!\(kl3\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell1\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr1\)\" is similar to \
existing symbols \!\({kkl1, kr1}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr2\)\" is similar to \
existing symbols \!\({kkl2, kr2}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr3\)\" is similar to \
existing symbols \!\({kkl3, kr3}\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell\"\) will be suppressed during \
this calculation."\)], "Message"]
}, Open  ]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa6\ is\ the\ coupling\ constant*) \)\(expr2pinch = \(-kappa6\
\)/2*\((simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11])\)\)\)\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/2\)\)\ kappa6\ \((muf\^2\ \((\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\)\)\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** Conjugate[d[kl2]] ** 
                      d[kr2] ** 
                      b[kr1])\)\) + \(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\)\
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                        kr2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      Conjugate[
                        d[kl2]])\)\) + \(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\)\)\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[d[kl2]] ** d[kr2] ** sigma[3] ** 
                      b[kr1])\)\) + \(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\)\
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                        kr2] ** sigma[3] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[d[kl2]])\)\))\) + 
          kappas\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s2] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 
                          2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[3] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[3] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
          muf\ \((kappas\ \((\(-\(\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 
                                  kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                  kr2]\ Conjugate[b[kl1]] ** 
                                Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                                d[kr2] ** 
                                b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                  kl2)\)\ \((kl2 + 
                                  kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ Conjugate[b[kl1]] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                                2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ d[kr2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ d[kr3] ** a[kr2, s1] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                                2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** sigma[3] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** sigma[3] ** 
                              a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                                2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ d[kr2] ** sigma[3] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                                kr2 + kr3]\ d[kr3] ** sigma[3] ** a[kr2, s1] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
                kappaa\ \((\(-\(\((\((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + 
                                  kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                                Conjugate[d[kl2]] ** d[kr3] ** zeta[s1] ** 
                                a[kr2, s1] ** 
                                b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                  kr2)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[kr3] ** 
                              zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \[Pi]\^\(3/2\))\) + \((\((kl1 + 
                                kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** Conjugate[d[kl2]] ** 
                              d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                              a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
                              zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \
\((\((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ Conjugate[
                                b[kl1]] ** sigma[3] ** Conjugate[d[kl2]] ** 
                              d[kr3] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \[Pi]\^\(3/2\))\) - \((\((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                              sigma[3] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** sigma[3] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \
\((\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                              zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \
\((\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** 
                              sigma[3] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \
\((\((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[kr3] ** 
                              sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kl1 + 
                                kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ d[kr3] ** zeta[s1] ** sigma[3] ** 
                              a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[3] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                              sigma[1] ** zeta[s1] ** sigma[2] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** sigma[3] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\
\[ImaginaryI]\ \((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** sigma[1] ** 
                              zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[3] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                              zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\
\[ImaginaryI]\ \((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[
                                kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                              a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[3] ** 
                              Conjugate[
                                d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
          kappaa\ kappas\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ delta[\(-kl1\) - 
                          kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s2] ** 
                        sigma[1] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** sigma[1] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                          kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** sigma[3] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                          kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ d[kr3] ** zeta[s2] ** sigma[3] ** a[kr2, s2] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \((kr1 + 
                          2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                          kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                        zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \((kl1 + 
                          2\ kl2 + kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                          kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \((kl1 + 
                          2\ kl2 + kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                          kr2 + kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** 
                        sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \((kr1 + 
                          2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                          kr2 + kr3]\ d[kr3] ** sigma[3] ** a[kr2, s2] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                        sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\))\) + 
          kappaa\^2\ \((\((delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        sigma[1] ** zeta[s2] ** sigma[1] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** zeta[s1] ** sigma[3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                        zeta[s2] ** sigma[1] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** 
                        a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[1] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** 
                        sigma[3] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                        zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** zeta[s1] ** sigma[3] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                        sigma[3] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                        sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                        zeta[s2] ** sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** 
                        sigma[3] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                        sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        sigma[1] ** zeta[s2] ** sigma[1] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                        sigma[1] ** zeta[s2] ** sigma[2] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                        sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                        zeta[s2] ** sigma[1] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                        zeta[s2] ** sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                        sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\))\))\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Find longitudinal vector 4-fermion interaction to leading order, \
acts on rho J_z = 0\
\>", "Subsection"],

Cell["This couples to the J_z =0 component of the rho", "Text"],

Cell["\<\
The vector four-fermion interaction in the Hamiltonian is -kappa7 \
(qq**rr+rr**qq) where qq is:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\ \(conju[x] ** u[x] //. prodexpand\) //. 
                sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(rr[x_] = 
      Collect[relabel[
            Expand[\(Integrate[cxi[x], x] ** Integrate[xi[x], x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\^2\ \((\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** a[kr2, \
s1] ** Conjugate[a[kl2, s2]] ** b[kr1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\ \@kr2\
\ \((\(-kl1\) + kr2)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** \
a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\
\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** b[kr1]\
\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - kr1)\)\ \[Pi]\^2\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]]\)\/\(16\ \@kl1\ \@kl2\ \((kl2 + \
kl3)\)\ \((kl1 - kr1)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** a[kr1, \
s1] ** Conjugate[a[kl1, s2]] ** Conjugate[d[kl2]]\)\/\(16\ \@kl1\ \((kl1 + \
kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** Conjugate[d[kl2]]\)\/\(16\ \
\@kl1\ \@kr1\ \((\(-kl2\) + kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^2\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 \
+ kr3)\)\ x\)\ d[kr3] ** a[kr2, s1] ** a[kr1, s2] ** \
Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \@kr2\ \((kr2 + \
kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x \
+ \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \
\((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, \
s2] ** Conjugate[d[kl2]]\)\/\(16\ \@kr1\ \((\(-kl2\) + kr1)\)\ \@kr2\ \
\((\(-kl1\) + kr2)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, \
s2]] ** b[kr1]\)\/\(16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\
\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** \
Conjugate[a[kl1, s2]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \
\((kr2 + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 \
+ kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + \
kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** \
sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
      muf\ \((kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** \
a[kr2, s1] ** b[kr1]\)\/\(8\ kl1\ kr1\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((\(-kl1\)\ kl2 + 2\ kl1\ kr1 + kl2\ kr1)\)\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - \
kr1)\)\ kr1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 + kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(8\ \@kl1\ kl2\ \((kl1 + \
kl2)\)\ \((kl1 - kr1)\)\ kr1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((kr1 + 2\ \
kr2)\))\)\ d[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(8\ kl1\ \((kl1 - \
kr1)\)\ \@kr1\ kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ \
kl2 - kl1\ kr1 - kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            
                            kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl2\ kl3 + 
                            kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kr1\ kr2 + \
kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(8\ \@kl1\ kr1\ \((\(-kl1\) + kr1)\)\ \((kl1 - kr2)\)\ kr2\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** \
a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + \
kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
            kappaa\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kl1 + kr1)\)\ \@kr2\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, \
s1] ** b[kr1]\)\/\(8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \@kl2\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - kr1)\)\
\ kr1\ \[Pi]\^\(3/2\)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((kl1 - kl2)\)\ \@kr1\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \@kl2\ \
\((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \@kl1\ \((kr1 - kr2)\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \
\((kl1 - kr1)\)\ kr1\ \((kl1 - kr2)\)\ kr2\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \
\((kr1 - kr3)\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl1\ \
\((kl2 + kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl1, s1]] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                            kl2)\)\ \((kl1 - 
                            kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \@kr1\ \((kl1 + 
                            kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                          sigma[1] ** a[kr1, s1] ** 
                          Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                            kr1)\)\ kr2\ \((kr1 + 
                            kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
      kappaa\ kappas\ \((\(-\(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                        b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                      Conjugate[a[kl2, s2]] ** 
                      b[kr1]\)\/\(16\ \@kl2\ \((kl2 - 
                        kr1)\)\ \@kr2\ \((\(-kl1\) + 
                        kr2)\)\ \[Pi]\^2\)\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(16\ \@kl2\ \((kl2 - kr1)\)\ \@kr2\ \((\(-kl1\) + kr2)\)\ \[Pi]\^2\
\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \
\[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, \
s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\
\) + kr3)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ Conjugate[b[kl1]] ** \
zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^2\) - \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((\(-kl3\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** \
a[kr2, s1] ** b[kr1]\)\/\(16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \
\[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** \
b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, \
s2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\
\ \((kr2 + kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** b[kr1]\)\/\(16\
\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - kr3)\)\ \[Pi]\^2\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, \
s2]] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - \
kr3)\)\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\
\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s1] ** \
a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((kr3 + kr4)\)\ \[Pi]\^2\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 \
+ kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** \
zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + \
kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ \
d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ \
d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                    Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
      kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\
\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                    a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((\(-kl3\) + 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                    Conjugate[a[kl1, s2]] ** 
                    b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                      kr3)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \
\((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                      kl3)\)\ \((kl1 - 
                      kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                      kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                      kl2)\)\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ \
d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                    Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \((kl1 - 
                      kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                      kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    a[kr2, s1] ** a[kr1, s2] ** 
                    Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \@kr2\ \((kr2 + 
                      kr3)\)\ \[Pi]\^2)\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rrpinch = 
      rr[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[
                        d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
      muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\
\ kr3)\))\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \@kr2\ \
\((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
            kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ \
d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \((kr1 + \
kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\))\) + 
      kappas\^2\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\
\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\)\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \
\((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
      kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** 
                      zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[a[kl3, s2]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \
\@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, \
s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \
\[Pi]\^2\))\) + 
      kappaa\^2\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \
\[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\)\
\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Distribute, rrpinch]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ muf\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
                Conjugate[
                  d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ muf\^2\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + 
      kappas\ muf\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                      kl2)\)\ kl3\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\ \
kr3)\))\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \@kr2\ \
\((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 \
+ kl4)\)\ x\)\ kappas\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + kl4)\)\ \[Pi]\^2)\) + 
      kappaa\ muf\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                      kl2)\)\ kl3\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[kr3] ** \
sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \((kr1 + kr2)\)\ \
kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
kappas\^2\ d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \
\@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 \
+ kl4)\)\ x\)\ kappaa\ kappas\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
              Conjugate[a[kl3, s1]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ \
d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \
\@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[kr4] ** sigma[1] ** zeta[s1] ** \
a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((kr3 + kr4)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\^2\ Conjugate[
                b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ kappaa\^2\ d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
              a[kr3, s1] ** a[kr2, s2] ** 
              b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                kr4)\)\ \[Pi]\^2)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Distribute, rrpinch, 3]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ muf\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
                Conjugate[
                  d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ muf\^2\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ kappas\ \((kl2\ kl3 + 
                kl1\ \((kl2 + 2\ kl3)\))\)\ muf\ Conjugate[b[kl1]] ** 
              sigma[1] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                kl2)\)\ kl3\ \((kl2 + 
                kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ kappas\ \((kr2\ kr3 + kr1\ \
\((kr2 + 2\ kr3)\))\)\ muf\ d[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\
\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 \
+ kl4)\)\ x\)\ kappas\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ kappaa\ \@kl2\ \((kl1 - kl3)\)\ muf\ Conjugate[
                b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
              Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + kl2)\)\ kl3\ \((kl2 + 
                kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ kappaa\ \@kr2\ \((kr1 - kr3)\)\ \
muf\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ kr1\ \
\((kr1 + kr2)\)\ kr3\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ \
x\)\ kappas\^2\ d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** \
b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \
\((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ Conjugate[b[kl1]] ** zeta[s1] ** 
              sigma[1] ** Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
              Conjugate[a[kl3, s1]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ \
d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \
\@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[kr4] ** sigma[1] ** zeta[s1] ** \
a[kr3, s2] ** a[kr2, s1] ** b[kr1]\)\/\(16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \
\((kr3 + kr4)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\^2\ Conjugate[
                b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
              Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
              Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                kl2)\)\ \@kl3\ \((kl3 + 
                kl4)\)\ \[Pi]\^2)\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ kappaa\^2\ d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
              a[kr3, s1] ** a[kr2, s2] ** 
              b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                kr4)\)\ \[Pi]\^2)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        Collect[relabel[\((qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kl4 \[Rule] kkl4, 
                          kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                          kr3 \[Rule] kkr3, kr4 \[Rule] kkr4, 
                          s1 \[Rule] ss1})\) ** Map[Distribute, rrpinch, 3] //. 
                  prodexpand] + 
              relabel[Map[Distribute, 
                      rrpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kl4 \[Rule] kkl4, 
                          kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                          kr3 \[Rule] kkr3, kr4 \[Rule] kkr4, 
                          s1 \[Rule] ss1}, 3] ** qqpinch //. 
                  prodexpand], {muf, kappaa, kappas}] //. 
          prodfactor;\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr1pinch\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(-\(\(1\/\(8\ kl1\ kl2\ kl3\ kl4\ \[Pi]\^2\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3 + kl4)\)\ x\)\ \((kl1\ \
kl2 + kl3\ kl4)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                    Conjugate[b[kl3]] ** sigma[1] ** 
                    Conjugate[
                      d[kl4]])\)\)\) - \(\(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\^2\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1 - kr2)\)\ \
x\)\ \((kl1\ kl2 + kr1\ kr2)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)\) - \(\(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\^2\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1 - kr2)\)\ \
x\)\ \((kl1\ kl2 + kr1\ kr2)\)\ d[kr2] ** sigma[1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2 + kr3 + kr4)\)\ x\)\ \((kr1\ kr2 + kr3\ kr4)\)\ d[kr4] ** \
sigma[1] ** b[kr3] ** d[kr2] ** sigma[1] ** b[kr1]\)\/\(8\ kr1\ kr2\ kr3\ kr4\
\ \[Pi]\^2\))\) + 
      muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x + \[ImaginaryI]\ kl4\ x + \[ImaginaryI]\ kl5\ x\)\ \((kl2\ kl3 + 
                            kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]] ** Conjugate[b[kl4]] ** sigma[1] ** 
                          Conjugate[d[kl5]])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \((kl2\ kl3 + 
                            kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                          b[kr1])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl3 + kl4 + kl5)\)\ x\)\ \((kl4\ kl5 + 
                            kl3\ \((kl4 + 2\ kl5)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                          sigma[1] ** Conjugate[a[kl4, s1]] ** 
                          Conjugate[d[kl5]])\)/\((16\ kl3\ \@kl4\ \((kl3 + 
                            kl4)\)\ kl5\ \((kl4 + 
                            kl5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + 
                            kr1\ \((kr2 + 2\ kr3)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** 
                          sigma[1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \((kl2\ kl3 + 
                            kl1\ \((kl2 + 2\ kl3)\))\)\ d[kr2] ** sigma[1] ** 
                          b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                          Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + 
                            kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** 
                          a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                          sigma[1] ** 
                          Conjugate[d[kl2]])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ \((kr3 + kr4 + kr5)\)\ x\)\ \((kr4\ kr5 + 
                            kr3\ \((kr4 + 2\ kr5)\))\)\ d[kr5] ** sigma[1] ** 
                          a[kr4, s1] ** b[kr3] ** d[kr2] ** sigma[1] ** 
                          b[kr1])\)/\((16\ kr3\ \@kr4\ \((kr3 + 
                            kr4)\)\ kr5\ \((kr4 + 
                            kr5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x - \
\[ImaginaryI]\ kr4\ x - \[ImaginaryI]\ kr5\ x\)\ \((kr2\ kr3 + 
                            kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr5] ** sigma[1] ** 
                          b[kr4] ** d[kr3] ** sigma[1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\))\) + 
            kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4 + kl5)\)\ x\)\ \@kl4\ \
\((kl3 - kl5)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                          Conjugate[b[kl3]] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl4, s1]] ** 
                          Conjugate[d[kl5]])\)/\((16\ kl3\ \((kl3 + 
                            kl4)\)\ kl5\ \((kl4 + 
                            kl5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ Conjugate[
                            b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                          d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kr1\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x + \[ImaginaryI]\
\ kl4\ x + \[ImaginaryI]\ kl5\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[
                            b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                          Conjugate[b[kl4]] ** sigma[1] ** 
                          Conjugate[d[kl5]])\)/\((16\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[
                            b[kl1]] ** zeta[s1] ** sigma[1] ** 
                          Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                          d[kr2] ** sigma[1] ** 
                          b[kr1])\)/\((16\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \@kl2\ \((kl1 - kl3)\)\ d[kr2] ** 
                          sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                          zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((16\ kl1\ \((kl1 + 
                            kl2)\)\ kl3\ \((kl2 + 
                            kl3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[kr3] ** 
                          sigma[1] ** zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                          Conjugate[b[kl1]] ** sigma[1] ** 
                          Conjugate[d[kl2]])\)/\((16\ kr1\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x - \
\[ImaginaryI]\ kr4\ x - \[ImaginaryI]\ kr5\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[
                            kr5] ** sigma[1] ** b[kr4] ** d[kr3] ** sigma[1] ** 
                          zeta[s1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kr1\ \((kr1 + 
                            kr2)\)\ kr3\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ \((kr3 + kr4 + kr5)\)\ x\)\ \@kr4\ \((kr3 - kr5)\)\ d[kr5] ** 
                          sigma[1] ** zeta[s1] ** a[kr4, s1] ** b[kr3] ** 
                          d[kr2] ** sigma[1] ** 
                          b[kr1])\)/\((16\ kr3\ \((kr3 + 
                            kr4)\)\ kr5\ \((kr4 + 
                            kr5)\)\ \[Pi]\^\(5/2\))\))\))\) + 
      kappas\^2\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\
\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ kl5\ x + \
\[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                      Conjugate[d[kl4]] ** Conjugate[b[kl5]] ** sigma[1] ** 
                      Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^3)\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                    sigma[1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + \
kl4)\)\ x + \[ImaginaryI]\ \((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                    sigma[1] ** Conjugate[a[kl4, s1]] ** 
                    Conjugate[a[kl5, s2]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                      kl4)\)\ \@kl5\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** 
                    b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** 
                    a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                    sigma[1] ** 
                    Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                    a[kr5, s2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** sigma[1] ** 
                    b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                    b[kr5] ** d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\))\) + 
      kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ \
\((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                      Conjugate[b[kl3]] ** zeta[s1] ** sigma[1] ** 
                      Conjugate[a[kl4, s1]] ** Conjugate[a[kl5, s2]] ** 
                      Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                        kl4)\)\ \@kl5\ \((kl5 + 
                        kl6)\)\ \[Pi]\^3)\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + \
kl4)\)\ x + \[ImaginaryI]\ \((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                    zeta[s1] ** sigma[1] ** Conjugate[a[kl4, s2]] ** 
                    Conjugate[a[kl5, s1]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                      kl4)\)\ \@kl5\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                    zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                    zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \
\[ImaginaryI]\ kl5\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** Conjugate[d[kl4]] ** 
                    Conjugate[b[kl5]] ** sigma[1] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                    sigma[1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \
\[ImaginaryI]\ kl5\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** 
                    Conjugate[b[kl5]] ** sigma[1] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                    sigma[1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** 
                    b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** 
                    b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** 
                    zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1] ** 
                    Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** 
                    zeta[s2] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                    Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                    b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                    b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                    zeta[s2] ** a[kr5, s1] ** a[kr4, s2] ** b[kr3] ** d[kr2] ** 
                    sigma[1] ** 
                    b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                    zeta[s2] ** a[kr5, s2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** 
                    sigma[1] ** 
                    b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\))\) + 
      kappaa\^2\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ \
\((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                    Conjugate[b[kl3]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                    Conjugate[a[kl4, s1]] ** Conjugate[a[kl5, s2]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                      kl4)\)\ \@kl5\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** 
                    sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                    zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \
\[ImaginaryI]\ kl5\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]] ** Conjugate[b[kl5]] ** sigma[1] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                    Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** 
                    b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                    sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + \
kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** 
                    zeta[s2] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                    b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((kr3 + 
                      
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                    b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                    a[kr3, s1] ** a[kr2, s2] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                      kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                    zeta[s2] ** zeta[s1] ** a[kr5, s2] ** a[kr4, s1] ** 
                    b[kr3] ** d[kr2] ** sigma[1] ** 
                    b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\))\)\)], "Output"]
}, Open  ]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa7\ is\ the\ coupling\ constant*) \)\(expr2pinch = \(-kappa7\
\)*\((simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11])\)\)\)\)], "Input"],

Cell[BoxData[
    \(\(-kappa7\)\ \((muf\^2\ \((\(-\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\)\
\((\((kl1\ kl2 + kr1\ kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                          b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr2] ** 
                        sigma[1] ** 
                        b[kr1])\)\)\) - \(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\
\)\)\((\((kl1\ kl2 + kr1\ kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\)\))\) + 
          muf\ \((kappas\ \((\((\((kl2\ kl3 + 
                                kl1\ \((kl2 + 2\ kl3)\))\)\ delta[\(-kl1\) - 
                                kl2 - kl3 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                              sigma[1] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                              b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                                kl2)\)\ kl3\ \((kl2 + 
                                kl3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr2\ kr3 + 
                                kr1\ \((kr2 + 2\ kr3)\))\)\ delta[\(-kl1\) - 
                                kl2 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                              sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** 
                              sigma[1] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                                kr2)\)\ kr3\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kl2\ kl3 + 
                                kl1\ \((kl2 + 2\ kl3)\))\)\ delta[\(-kl1\) - 
                                kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                              Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                                kl2)\)\ kl3\ \((kl2 + 
                                kl3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr2\ kr3 + 
                                kr1\ \((kr2 + 2\ kr3)\))\)\ delta[\(-kl1\) - 
                                kl2 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                              a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[1] ** 
                              Conjugate[d[kl2]])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                                kr2)\)\ kr3\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
                kappaa\ \((\((\@kr2\ \((kr1 - kr3)\)\ delta[\(-kl1\) - kl2 + 
                                kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                              sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** 
                              sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kr1\ \((kr1 + 
                                kr2)\)\ kr3\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\) + \((\@kl2\ \((kl1 \
- kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                              zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                              b[kr1])\)/\((8\ kl1\ \((kl1 + 
                                kl2)\)\ kl3\ \((kl2 + 
                                kl3)\)\ \[Pi]\^\(3/2\))\) + \((\@kl2\ \((kl1 \
- kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                              b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                              sigma[1] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                                kl2)\)\ kl3\ \((kl2 + 
                                kl3)\)\ \[Pi]\^\(3/2\))\) + \((\@kr2\ \((kr1 \
- kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                              zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** sigma[1] ** 
                              Conjugate[d[kl2]])\)/\((8\ kr1\ \((kr1 + 
                                kr2)\)\ kr3\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
          kappas\^2\ \((\(-\(\((delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                            kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
                          Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                          Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                          b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                            kl2)\)\ \@kl3\ \((kl3 + 
                            kl4)\)\ \[Pi]\^2)\)\)\) - \((delta[\(-kl1\) - 
                          kl2 + kr1 + kr2 + kr3 + kr4]\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                        a[kr3, s1] ** a[kr2, s2] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                        Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + 
                          kr1 + kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** 
                        a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2)\))\) + 
          kappaa\ kappas\ \((\((delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                        a[kr3, s1] ** a[kr2, s2] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + 
                          kr1 + kr2 + kr3 + kr4]\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                        zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[a[kl3, s2]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                        sigma[1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                        sigma[1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                        Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                        Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + 
                          kr1 + kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** 
                        zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 + 
                          kr1 + kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** 
                        zeta[s2] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2)\))\) + 
          kappaa\^2\ \((\((delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                          kr4]\ Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                        zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                        Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - 
                          kl3 - kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[a[kl3, s2]] ** 
                        Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 + 
                          kr1 + kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** 
                        zeta[s2] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@kr3\ \((kr3 + 
                          kr4)\)\ \[Pi]\^2)\))\))\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["kappa_5 term for nonzero transverse momentum.", "Section"],

Cell["\<\
This version leaves all of the integrations over momenta and \
position implicit.  Also, sums are implicit.\
\>", "Text"],

Cell["\<\
Phase factors put in by hand, keeping only the terms that \
contribute to leading order in N\
\>", "Text"],

Cell[BoxData[
    \(xi[x_] := \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kf\ x\)\ muf\ \
sigma[1] ** b[kf]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ ka\ x - \[ImaginaryI]\ kf\ x + I\ ka\ pperp[sumrr]/2 + I\ kf\ \
pperp[sumrr]\)\ kappas\ sigma[1] ** a[ka, sumrr] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\
\) - \(0\ kappas\ sigma[1] ** Conjugate[a[ka, sumrr]] ** b[kf]\)\/\(4\ \@ka\ \
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x\)\ muf\ sigma[1] ** \
sigma[1] ** Conjugate[d[kf]]\)\/\(2\ \@\[Pi]\) - \(0\ kappas\ sigma[1] ** \
a[ka, sumrr] ** sigma[1] ** Conjugate[d[kf]]\)\/\(4\ \@ka\ \[Pi]\) - \
\(\(1\/\(4\ \@ka\ \[Pi]\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ ka\ x + \
\[ImaginaryI]\ kf\ x + I\ ka\ pperp[sumrr]/2 + I\ kf\ pperp[sumrr]\)\ kappas\ \
sigma[1] ** Conjugate[a[ka, sumrr]] ** sigma[1] ** 
              Conjugate[
                d[kf]])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ ka\ x \
- \[ImaginaryI]\ kf\ x + I\ ka\ pperp[rr]/2 + I\ kf\ pperp[rr]\)\ kappaa\ \
sigma[1] ** zeta[rr] ** a[ka, rr] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) + \(0\ \
kappaa\ sigma[1] ** zeta[rr] ** Conjugate[a[ka, rr]] ** b[kf]\)\/\(4\ \@ka\ \
\[Pi]\) - \(1\/\(4\ \@ka\ \[Pi]\)\) \((0\ kappaa\ sigma[1] ** zeta[rr] ** 
                a[ka, rr] ** sigma[1] ** 
                Conjugate[
                  d[kf]])\) + \(\(1\/\(4\ \@ka\ \[Pi]\)\)\((\[ExponentialE]\^\
\(\[ImaginaryI]\ ka\ x + \[ImaginaryI]\ kf\ x + I\ ka\ pperp[rr]/2 + I\ kf\ \
pperp[rr]\)\ kappaa\ sigma[1] ** zeta[rr] ** Conjugate[a[ka, rr]] ** 
              sigma[1] ** Conjugate[d[kf]])\)\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cxi[
        x_] = \(\(\(Conjugate[
                xi[x]] /. {Conjugate[pperp[a_]] \[Rule] pperp[a], 
                Conjugate[x] \[Rule] x, Conjugate[ka] \[Rule] ka, 
                Conjugate[kf] \[Rule] kf, 
                Conjugate[ka^n_] \[Rule] ka^n}\) /. {ka \[Rule] kka, 
              kf \[Rule] kks, rr \[Rule] ll, sumrr \[Rule] sumll}\) //. 
          prodexpand\) //. sigmazeta\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ muf\ d[kks]\)\/\(2\ \
\@\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ muf\ \
Conjugate[b[kks]] ** sigma[1]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x - 1\/2\ \[ImaginaryI]\ kka\ \
pperp[sumll] - \[ImaginaryI]\ kks\ pperp[sumll]\)\ kappas\ d[kks] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \
\[ImaginaryI]\ kks\ x - 1\/2\ \[ImaginaryI]\ kka\ pperp[sumll] - \
\[ImaginaryI]\ kks\ pperp[sumll]\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** \
Conjugate[a[kka, sumll]]\)\/\(4\ \@kka\ \[Pi]\) - \(\(1\/\(4\ \@kka\ \
\[Pi]\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ x \
- 1\/2\ \[ImaginaryI]\ kka\ pperp[ll] - \[ImaginaryI]\ kks\ pperp[ll]\)\ \
kappaa\ Conjugate[b[kks]] ** zeta[ll] ** sigma[1] ** 
            Conjugate[
              a[kka, ll]])\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\
\ x - \[ImaginaryI]\ kks\ x - 1\/2\ \[ImaginaryI]\ kka\ pperp[ll] - \
\[ImaginaryI]\ kks\ pperp[ll]\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** \
sigma[1] ** a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Find pseudoscalar 4-fermion interaction to leading order, acts on \
pion\
\>", "Subsection"],

Cell["\<\
The chiral four-fermion interaction in the Hamiltonian is kappa5 \
qq**qq/2 where qq is:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** sigma[2] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** sigma[2] ** u[x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\@2\ kl1\ kl2\ \[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** \
b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
      kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x \
+ 1\/2\ \[ImaginaryI]\ \((2\ kr1 + kr2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** \
sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\
\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x - \[ImaginaryI]\ kr1\ x - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s1]\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** b[kr1]\
\)\/\(4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x \
+ 1\/2\ \[ImaginaryI]\ \((kl1 + 2\ kl2)\)\ pperp[s1]\)\ d[kr1] ** sigma[3] ** \
Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \((kl1 + \
kl2)\)\ \[Pi]\^\(3/2\)\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - 1\/2\ \[ImaginaryI]\ \((kr1 + 2\ \
kr2)\)\ pperp[s1]\)\ d[kr2] ** sigma[3] ** a[kr1, s1] ** \
Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \((\
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ \
x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \
\((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 \
+ kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ \((x + \
pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
      kappaa\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x \
+ 1\/2\ \[ImaginaryI]\ \((2\ kr1 + kr2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x - 1\/2\ \[ImaginaryI]\ \((2\ kl1 \
+ kl2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x + 1\/2\ \
\[ImaginaryI]\ \((kl1 + 2\ kl2)\)\ pperp[s1]\)\ d[kr1] ** sigma[1] ** 
                    zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((kr1 + 2\ kr2)\)\ pperp[s1]\)\ d[kr2] ** sigma[1] ** 
                    zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\@2\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \@2\ kr1\ \
kr2\ \[Pi]\))\) + 
      kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
      kappaa\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^\(3/2\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
      Collect[relabel[
            Map[Distribute, 
                  qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                      kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                      kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
            kappas}] //. prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(muf\^2\ \((\(\(1\/\(8\ kl1\ kl2\ kl3\ kl4\ \[Pi]\^2\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 \
+ kl4)\)\ x\)\ \((kl1 + kl2)\)\ \((kl3 + kl4)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                  sigma[2] ** 
                  Conjugate[
                    d[kl4]])\)\) + \(\(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \[Pi]\^2\)\
\)\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ \((kl1 + kl2)\)\ \((kr1 + kr2)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                  b[kr1])\)\) + \(\(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\^2\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \((kl1 + kl2)\)\ \((kr1 + kr2)\)\ d[
                    kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  sigma[2] ** 
                  Conjugate[
                    d[kl2]])\)\) + \(\(1\/\(8\ kr1\ kr2\ kr3\ kr4\ \[Pi]\^2\)\
\)\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \((kr1 + kr2)\)\ \((kr3 + kr4)\)\ d[
                    kr4] ** sigma[2] ** b[kr3] ** d[kr2] ** sigma[2] ** 
                  b[kr1])\)\))\) + 
      muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + kl5)\)\
\ x - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kl4 + kl5)\)\ Conjugate[
                            b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]] ** Conjugate[b[kl4]] ** sigma[2] ** 
                          Conjugate[d[kl5]])\)/\((16\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 + 
                            kl3)\)\ kl4\ kl5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ Conjugate[
                            b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                          b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                            kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((2\ kl3 + kl4)\)\ pperp[s1]\)\ \((kl1 + 
                            kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ \((x + pperp[s1])\)\)\)\ \((kl3 + 
                                  kl4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ x\)\ \((kl4 + kl5)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                          sigma[2] ** Conjugate[a[kl4, s1]] ** 
                          Conjugate[
                            d[kl5]])\)/\((16\ kl1\ kl2\ \@kl4\ \((kl3 + 
                            kl4)\)\ \((kl4 + 
                            kl5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\
\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 \
+ kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ \
x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[
                            b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
                          d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                            kr2)\)\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ d[kr2] ** 
                          sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                          sigma[2] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 + 
                            kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                            kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                          sigma[2] ** a[kr2, s1] ** b[kr1] ** 
                          Conjugate[b[kl1]] ** sigma[2] ** 
                          Conjugate[
                            d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                            kr2)\)\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr3 + kr4 + kr5)\)\ x + \((kr4 + 2\ kr5)\)\ \
pperp[s1])\)\)\ \((kr1 + 
                            kr2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ x\)\)\ \((kr3 + 
                                  kr4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ \((x + pperp[s1])\)\)\ \((kr4 + kr5)\))\)\ d[kr5] ** 
                          sigma[2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** 
                          sigma[2] ** 
                          b[kr1])\)/\((16\ kr1\ kr2\ \@kr4\ \((kr3 + 
                            kr4)\)\ \((kr4 + 
                            kr5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr4 + kr5)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\
\ x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ \((kr4 + 
                            kr5)\)\ d[kr5] ** sigma[2] ** b[kr4] ** d[kr3] ** 
                          sigma[2] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                            kr3)\)\ kr4\ kr5\ \[Pi]\^\(5/2\))\))\) + 
            kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ \
x - 1\/2\ \[ImaginaryI]\ \((2\ kl3 + kl4)\)\ pperp[s1]\)\ \((kl1 + 
                            kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ \((x + pperp[s1])\)\)\)\ \((kl3 + 
                                  kl4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ x\)\ \((kl4 + kl5)\))\)\ Conjugate[b[kl1]] ** 
                          sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                          zeta[s1] ** sigma[2] ** Conjugate[a[kl4, s1]] ** 
                          Conjugate[
                            d[kl5]])\)/\((16\ kl1\ kl2\ \@kl4\ \((kl3 + 
                            kl4)\)\ \((kl4 + 
                            kl5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\
\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 \
+ kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ \
x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[
                            b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
                          d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                            kr2)\)\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + kl5)\)\ x - 1\/2\ \[ImaginaryI]\
\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kl4 + kl5)\)\ Conjugate[
                            b[kl1]] ** zeta[s1] ** sigma[2] ** 
                          Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                          Conjugate[b[kl4]] ** sigma[2] ** 
                          Conjugate[d[kl5]])\)/\((16\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 + 
                            kl3)\)\ kl4\ kl5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ Conjugate[
                            b[kl1]] ** zeta[s1] ** sigma[2] ** 
                          Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                          d[kr2] ** sigma[2] ** 
                          b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                            kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                  kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ d[kr2] ** 
                          sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                          zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                          Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                            kl2)\)\ \((kl2 + 
                            kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                            kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                          sigma[2] ** zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                          Conjugate[b[kl1]] ** sigma[2] ** 
                          Conjugate[
                            d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                            kr2)\)\ \((kr2 + 
                            kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr4 + kr5)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\
\ x\)\)\ \((kr1 + 
                                  kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ \((kr4 + 
                            kr5)\)\ d[kr5] ** sigma[2] ** b[kr4] ** d[kr3] ** 
                          sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                          b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                            kr3)\)\ kr4\ kr5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr3 + kr4 + kr5)\)\ x + \((kr4 + 2\ kr5)\)\ \
pperp[s1])\)\)\ \((kr1 + 
                            kr2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ x\)\)\ \((kr3 + 
                                  kr4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ \((x + pperp[s1])\)\)\ \((kr4 + kr5)\))\)\ d[kr5] ** 
                          sigma[2] ** zeta[s1] ** a[kr4, s1] ** b[kr3] ** 
                          d[kr2] ** sigma[2] ** 
                          b[kr1])\)/\((16\ kr1\ kr2\ \@kr4\ \((kr3 + 
                            kr4)\)\ \((kr4 + 
                            kr5)\)\ \[Pi]\^\(5/2\))\))\))\) + 
      kappas\^2\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                            kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 \
+ kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    Conjugate[b[kl4]] ** sigma[2] ** Conjugate[a[kl5, s1]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + 
                      kl5)\)\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                    sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                      kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                            kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                            kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 \
+ kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** 
                    sigma[2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\))\) + 
      kappaa\ kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((2\ kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                            kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 \
+ kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    Conjugate[b[kl4]] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl5, s1]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + 
                      kl5)\)\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                    sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                      kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((2\ kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                            kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 \
+ kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** zeta[s2] ** 
                    sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    Conjugate[b[kl4]] ** sigma[2] ** Conjugate[a[kl5, s1]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + 
                      kl5)\)\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                      kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                    zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                    Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                            kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                            kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 \
+ kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** 
                    sigma[2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** 
                    zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                            kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                            kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 \
+ kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** 
                    sigma[2] ** zeta[s2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** 
                    sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\))\) + 
      kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                            kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 \
+ kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** zeta[s2] ** 
                    sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                    Conjugate[b[kl4]] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl5, s1]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + 
                      kl5)\)\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                    zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                      kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                            kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 \
+ kl2 + kl3)\)\ x\)\ \((kl2 + 
                            kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                    sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                    Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \((kr2 + 
                      kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\
\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                            kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 \
+ kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                            kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                            kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 \
+ kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** 
                    sigma[2] ** zeta[s2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** 
                    sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                      kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                      kr6)\)\ \[Pi]\^3)\))\)\)], "Output"]
}, Closed]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = \(expr = \
\(\(\(\(kappa5/2*int[expr1pinch, {x}] //. intsumexpand\) /. 
                Exp[y_] \[RuleDelayed] 
                  Exp[Collect[y, x]]\) /. {int[Exp[a_\ b_ + c_], {b_}] :> 
                  2\ Pi\ delta[I\ a]\ Exp[c] /; FreeQ[c, b], 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]}\) /. {Exp[
                  a_] \[RuleDelayed] Exp[Simplify[a]], 
              delta[a_] :> delta[Expand[a]]}\) /. 
          delta[_?neverzero] \[Rule] 0\)\)\)\)], "Input"],

Cell[BoxData[
    \(1\/2\ kappa5\ \((muf\^2\ \((\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\)\)\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                      b[kr1])\)\) + \(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\)\
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                        kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[2] ** Conjugate[d[kl2]])\)\))\) + 
          kappas\^2\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\) + \((\((2\ \[ExponentialE]\^\(1\
\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                        sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^3)\))\) + 
          muf\ \((kappas\ \((\((\((kr1 + 
                                kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                      kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2])\)\ Conjugate[
                                b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                              b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \
\((\((kl1 + 
                                kl2)\)\ \((\(-2\)\ \
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ \
pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                              sigma[2] ** Conjugate[d[kl2]] ** d[kr3] ** 
                              sigma[2] ** a[kr2, s1] ** 
                              b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(5/2\))\) + \((\((kr1 + 
                                kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                      kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2])\)\ d[kr2] ** 
                              sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[2] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \
\((\((kl1 + 
                                kl2)\)\ \((\(-2\)\ \
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ \
pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3])\)\ d[kr3] ** sigma[2] ** 
                              a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              sigma[2] ** 
                              Conjugate[
                                d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(5/2\))\))\) + 
                kappaa\ \((\((\((kl1 + 
                                kl2)\)\ \((\(-2\)\ \
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ \
pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                              sigma[2] ** Conjugate[d[kl2]] ** d[kr3] ** 
                              sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                              b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(5/2\))\) + \((\((kr1 + 
                                kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                      kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2])\)\ Conjugate[
                                b[kl1]] ** zeta[s1] ** sigma[2] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** sigma[2] ** 
                              b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \
\((\((kr1 + 
                                kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                      kr1 + kr2] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 - kl3 + kr1 + kr2])\)\ d[kr2] ** 
                              sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                              zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                              Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \
\((\((kl1 + 
                                kl2)\)\ \((\(-2\)\ \
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ \
pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                      kl2 + kr1 + kr2 + kr3] - 
                                2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((2\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + 
                                      kr2 + kr3])\)\ d[kr3] ** sigma[2] ** 
                              zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                              Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[
                                d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(5/2\))\))\))\) + 
          kappaa\ kappas\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\) + \((\((2\ \
\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ \
pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\
\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\) + \((\((2\ \[ExponentialE]\^\(1\
\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                        sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\) + \((\((2\ \[ExponentialE]\^\(1\
\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                        sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^3)\))\) + 
          kappaa\^2\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                        zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                        Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\) + \((\((2\ \[ExponentialE]\^\(1\
\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                                kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((\((kl2 + 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                          2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                        sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^3)\))\))\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Other terms of the Hamiltonian we probably don't care about", "Section"],

Cell[CellGroupData[{

Cell["\<\
Find scalar 4-fermion interaction to leading order, acts on \
a_0\
\>", "Subsection"],

Cell["\<\
This acts only on the lowest J^{P_1 C} = 0^{++} state, the a_0.  \
This state has a node in the wavefunction.\
\>", "Text"],

Cell["\<\
The chiral four-fermion interaction in the Hamiltonian is -kappa0 \
qq**qq/2 where qq is:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** sigma[1] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** sigma[1] ** u[x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        Collect[relabel[
              Map[Distribute, 
                    qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                        kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                        kr3 \[Rule] kkr3, s1 \[Rule] ss1}] ** 
                  Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
              kappas}] //. prodfactor;\)\)\)], "Input"],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[BoxData[
    \(\(\( (*kappa0\ is\ the\ coupling\ constant*) \)\(expr2pinch = \(-kappa0\
\)/2*simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
              int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
            delta[_?neverzero] \[Rule] 0, k11]\)\)\)], "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Find transverse axial-vector 4-fermion interaction to leading \
order, acts on a_1 J_z = -1,1\
\>", "Subsection"],

Cell["\<\
This appears to not act on the lowest state since it acts on a \
wavefunction with a node.
It acts on the J_z=-1,1 components of the a_1.\
\>", "Text"],

Cell["\<\
The vector four-fermion interaction in the Hamiltonian is kappa6 \
(qq**qq-rr**rr)/2 where qq is:\
\>", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(qq[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** sigma[3] ** Integrate[xi[x], x] + 
                    I\ Integrate[cxi[x], x] ** sigma[3] ** u[x] //. 
                  prodexpand\) //. sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(\(\(\ \)\(rr[x_] = 
      Collect[relabel[
            Expand[\(I\ conju[x] ** Integrate[xi[x], x] - 
                    I\ Integrate[cxi[x], x] ** u[x] //. prodexpand\) //. 
                sigmazeta]], {muf, kappas, kappaa}] //. 
        prodfactor\)\)\)], "Input"],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(rrpinch = 
      rr[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        Collect[relabel[
                Map[Distribute, 
                      qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, 
                          kr2 \[Rule] kkr2, kr3 \[Rule] kkr3, 
                          s1 \[Rule] ss1}] ** Map[Distribute, qqpinch] //. 
                  prodexpand] - 
              relabel[Map[Distribute, 
                      rrpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                          kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, 
                          kr2 \[Rule] kkr2, kr3 \[Rule] kkr3, 
                          s1 \[Rule] ss1}] ** Map[Distribute, rrpinch] //. 
                  prodexpand], {muf, kappaa, kappas}] //. 
          prodfactor;\)\)\)], "Input"],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[BoxData[
    \(\(\( (*\ kappa6\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      kappa6/2*\((simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11])\)\)\)\)], "Input"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Find fermion interactions using explicit constrained", "Section"],

Cell["\<\
This version leaves all of the integrations over momenta and \
position implicit.  Also, sums are implicit.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(u[x_] := 
        1/Sqrt[2\ Pi]\ \((b[kf]\ Exp[\(-I\)\ kf\ x] + 
              sigma[1] ** Conjugate[d[kf]]\ Exp[
                  I\ kf\ x])\);\)\), "\[IndentingNewLine]", 
    \(\(conju[
          x_] = \(Conjugate[u[x]] /. {Conjugate[x] \[Rule] x, 
              Conjugate[kf] \[Rule] kf}\) /. 
          kf \[Rule] kks;\)\), "\[IndentingNewLine]", 
    \(\(v[x_] := 
        1/Sqrt[2\ Pi]\ \((sigma[1] ** bb[kf]\ Exp[\(-I\)\ kf\ x] - 
              Conjugate[dd[kf]]\ Exp[
                  I\ kf\ x])\);\)\), "\[IndentingNewLine]", 
    \(\(conjv[
          x_] = \(Conjugate[v[x]] /. {Conjugate[x] \[Rule] x, 
              Conjugate[kf] \[Rule] kf}\) /. 
          kf \[Rule] kks;\)\), "\[IndentingNewLine]", 
    \(\(m[x_, rho_] := 
        1/Sqrt[4\ Pi]\ \((a[ka, \(-rho\)]\ Exp[\(-I\)\ ka\ x] + \ 
                Conjugate[a[ka, rho]]\ Exp[I\ ka\ x])\)/
            Sqrt[ka];\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(conjv\)\" is similar to \
existing symbol \"\!\(conju\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[{
    \(\(kappaa /: Conjugate[kappaa] := kappaa;\)\), "\[IndentingNewLine]", 
    \(\(kappas /: Conjugate[kappas] := kappas;\)\), "\[IndentingNewLine]", 
    \(\(kappa5 /: Conjugate[kappa5] := kappa5;\)\), "\[IndentingNewLine]", 
    \(\(muf /: Conjugate[muf] := muf;\)\)}], "Input",
  InitializationCell->True],

Cell["\<\
This is to define the momenta variables to be *always * positive.\
\
\>", "Text"],

Cell[BoxData[
    \(kl1 /: Positive[kl1] = True; \ kl2 /: Positive[kl2] = True; \ 
    kl3 /: Positive[kl3] = True;\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\(kl4 /: Positive[kl4] = True\) \)\(;\)\(\ \)\(\(kl5 /: 
        Positive[kl5] = True\) \)\(;\)\(\ \)\(\(kl6 /: Positive[kl6] = 
        True\) \)\(;\)\(\ \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(kr1 /: Positive[kr1] = True; \ kr2 /: Positive[kr2] = True; \ 
    kr3 /: Positive[kr3] = True;\), "\[IndentingNewLine]", 
    \(kr4 /: Positive[kr4] = True; \ kr5 /: Positive[kr5] = True; \ 
    kr6 /: Positive[kr6] = True;\)}], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Find Yukawa terms", "Subsection"],

Cell["Terms in the Hamiltonian P^-", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr1 = 
      Collect[relabel[
            Expand[\(\(\(\(\(-I\)/
                              2\ \((\(-D[conjv[x], x] ** v[x]\) + 
                                conjv[x] ** D[v[x], x])\) - 
                          muf/Sqrt[2]\ \((conjv[x] ** sigma[1] ** u[x] + 
                                conju[x] ** sigma[1] ** v[x])\) - 
                          kappaa/
                              Sqrt[2]\ \((\(-conju[x] ** zeta[rho] ** 
                                    sigma[1] ** m[x, rho] ** v[x]\) + 
                                conjv[x] ** sigma[1] ** zeta[rho] ** 
                                  m[x, rho] ** u[x])\) + 
                          kappas/
                              Sqrt[2]\ \((conju[x] ** sigma[1] ** 
                                  m[x, sumss] ** v[x] + 
                                conjv[x] ** sigma[1] ** m[x, sumss] ** 
                                  u[x])\) /. {0 ** _ \[Rule] 
                            0, _ ** 0 \[Rule] 0}\) //. 
                      prodexpand\) /. {zeta[rho_] ** ss_ ** 
                          a[c_, \(-rho_\)] \[Rule] \(-zeta[rho] ** ss ** 
                            a[c, rho]\), 
                      zeta[rho_] ** 
                          a[c_, \(-rho_\)] \[Rule] \(-zeta[rho] ** 
                            a[c, rho]\), 
                      a[b_, \(-sumss\)] \[Rule] a[b, sumss]}\) /. 
                  prodexpand\) //. sigmazeta]], {muf, kappaa, kappas}] //. 
        prodfactor\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + 
                  kr1)\)\ Conjugate[bb[kl1]] ** 
                bb[kr1]\)\/\(4\ \[Pi]\)\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ dd[kr1] ** \
Conjugate[dd[kl1]]\)\/\(4\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x\)\ \((kl1 - kl2)\)\ Conjugate[bb[kl1]] ** sigma[1] ** \
Conjugate[dd[kl2]]\)\/\(4\ \[Pi]\) + 
      muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x\)\ Conjugate[b[kl1]] ** 
                      bb[kr1]\)\/\(2\ \@2\ \[Pi]\)\)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[bb[kl1]] ** b[kr1]\)\/\(2\ \@2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** Conjugate[dd[kl1]]\
\)\/\(2\ \@2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** Conjugate[d[kl1]]\)\/\(2\ \@2\ \[Pi]\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[dd[kl2]]\)\/\(2\ \@2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** bb[kr1]\)\/\(2\ \@2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
b[kr1]\)\/\(2\ \@2\ \[Pi]\))\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ dd[kr2] ** sigma[1] ** bb[kr1]\)\/\(4\ \
\[Pi]\) + 
      kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** a[kr2, s1] ** \
bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[dd[kl2]]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\
\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(4\ \
\@2\ \@kr1\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** a[kr1, s1] ** Conjugate[dd[kl2]]\)\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\
\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, \
s1]] ** Conjugate[dd[kl3]]\)\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[a[kl2, \
s1]] ** Conjugate[d[kl3]]\)\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
bb[kr1]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x\)\ \
d[kr3] ** sigma[1] ** a[kr2, s1] ** bb[kr1]\)\/\(4\ \@2\ \@kr2\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** Conjugate[a[kl1, \
s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ kr3\ x\)\ dd[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \
\@2\ \@kr2\ \[Pi]\^\(3/2\)\))\) + 
      kappaa\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\)\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, \
s1]] ** bb[kr1]\)\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
Conjugate[bb[kl1]] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, \
s1] ** Conjugate[dd[kl2]]\)\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\) - \(\(1\/\(4\ \
\@2\ \@kl2\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    dd[kl3]])\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** \
zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\) - \(\(1\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[bb[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x \
- \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
zeta[s1] ** Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\
\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ \
x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** \
bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
dd[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \
\@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ \
x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x\)\ dd[kr3] ** sigma[1] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) - \(\(1\
\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\
\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\) + \(\(1\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x \
- \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\))\)\)], "Output"]
}, Closed]],

Cell["\<\
Since the delta[...] is removed in the following, the integration \
region can be complicated.  expr2 contians all of the interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      simpint[\(\(\(int[expr1, {x}] //. intsumexpand\) /. 
              Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
            int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
          delta[_?neverzero] \[Rule] 0, k11]\)], "Input"],

Cell[BoxData[
    \(\(-\(1\/2\)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[
            bb[kl1]] ** bb[kr1] + 
      muf\ \((\(-\(\(delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** 
                      bb[kr1]\)\/\@2\)\) - \(delta[\(-kl1\) + kr1]\ \
Conjugate[bb[kl1]] ** b[kr1]\)\/\@2 + \(delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[dd[kl1]]\)\/\@2 + \(delta[\(-kl1\) + kr1]\ dd[kr1] ** \
Conjugate[d[kl1]]\)\/\@2)\) + 
      1\/2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ dd[kr1] ** 
          Conjugate[dd[kl1]] + 
      kappas\ \((\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, \
s1] ** bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + \
kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\
\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[bb[kl1]] ** a[kr2, \
s1] ** b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + kr1]\
\ Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(2\ \@kl2\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[dd[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + \
kr2]\ d[kr2] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + \
kr2]\ dd[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[dd[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + \
kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(2\ \@kl1\ \
\@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\))\) + 
      kappaa\ \((\(-\(\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\)\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[bb[kl1]] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ \
Conjugate[bb[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(2\ \
\@kl2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** \
zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[dd[kl2]]\)\/\(2\ \@kr1\ \@\(2\
\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** \
sigma[1] ** a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(\(1\/\(2\ \
\@kl1\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(2\ \
\@kr1\ \@\(2\ \[Pi]\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** sigma[1] ** zeta[s1] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    d[kl2]])\)\) - \(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** \
sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \
\@kr1\ \@\(2\ \[Pi]\)\))\)\)], "Output"]
}, Closed]],

Cell["\<\
These are the 1\[Rule]1 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(expr2 /. 
          a_NonCommutativeMultiply \[RuleDelayed] 
            Block[{non = 
                  a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
              0 /; Length[non] \[NotEqual] 2 || 
                  Count[non, Conjugate[_]] != 1]\) /. {kl1 \[Rule] k1, 
          kr1 \[Rule] k1}\) /. delta[0] \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(\(-k1\)\ Conjugate[bb[k1]] ** bb[k1] + 
      muf\ \((\(-\(Conjugate[b[k1]] ** bb[k1]\/\@2\)\) - 
            Conjugate[bb[k1]] ** b[k1]\/\@2 + 
            d[k1] ** Conjugate[dd[k1]]\/\@2 + 
            dd[k1] ** Conjugate[d[k1]]\/\@2)\) + 
      k1\ dd[k1] ** Conjugate[dd[k1]]\)], "Output"]
}, Open  ]],

Cell["\<\
These are the 2\[Rule]1 particle interactions\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] != 1]\)], "Input"],

Cell[BoxData[
    \(kappas\ \((\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, \
s1] ** bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + \
kr2]\ Conjugate[bb[kl1]] ** a[kr2, s1] ** b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** a[kr1, s1] ** \
Conjugate[dd[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + \
kr2]\ dd[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \
\(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\))\) + 
      kappaa\ \((\(-\(\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\)\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[bb[kl1]] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ \
d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(2\ \
\@kl1\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** \
zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) - \
\(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\))\)\)], \
"Output"]
}, Open  ]],

Cell["\<\
These are the 1\[Rule]2 particle interactions:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 /. 
      a_NonCommutativeMultiply \[RuleDelayed] 
        Block[{non = 
              a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
          0 /; Length[non] \[NotEqual] 3 || 
              Count[non, Conjugate[_]] \[NotEqual] 2]\)], "Input"],

Cell[BoxData[
    \(kappas\ \((\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ \
d[kr1] ** Conjugate[a[kl1, s1]] ** Conjugate[dd[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) - kl2 + \
kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[dd[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + \
kr1]\ Conjugate[bb[kl1]] ** sigma[1] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\))\) + 
      kappaa\ \((\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] \
** Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) - \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** \
Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[dd[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** sigma[1] ** \
a[kr1, s1] ** Conjugate[d[kl2]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) - \(\(1\/\(2\ \
\@kl1\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** sigma[1] ** zeta[s1] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)\))\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Testing", "Subsection"],

Cell["Terms in the Hamiltonian kappa_5/2 qq**qq", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qq[x_] = 
      relabel[Expand[\ \(\(conjv[x] ** 
                    v[x] /. {0 ** _ \[Rule] 0, _ ** 0 \[Rule] 0}\) //. 
                prodexpand\) //. sigmazeta]] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[bb[kl1]] ** bb[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** \
Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** bb[kr1]\)\/\(2\ \
\[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ Conjugate[bb[kl1]] ** sigma[1] ** 
                Conjugate[
                  dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
bb[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
      relabel[\((qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                    kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                    kr3 \[Rule] kkr3, s1 \[Rule] ss1})\) ** qqpinch //. 
            prodexpand] //. prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[dd[kl2]] ** Conjugate[bb[kl3]] ** 
            sigma[1] ** 
            Conjugate[
              dd[kl4]])\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** 
            Conjugate[dd[kl2]] ** dd[kr2] ** sigma[1] ** 
            bb[kr1])\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** bb[kr1] ** 
            Conjugate[bb[kl1]] ** sigma[1] ** 
            Conjugate[
              dd[kl2]])\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ \
dd[kr4] ** sigma[1] ** bb[kr3] ** dd[kr2] ** sigma[1] ** bb[kr1]\)\/\(4\ \
\[Pi]\^2\)\)], "Output"]
}, Open  ]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      Factor[kappa5/2*
          simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11]]\)\)\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \[Pi]\)\)\((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
            kr2]\ \((Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[dd[kl2]] ** 
              dd[kr2] ** sigma[1] ** bb[kr1] + 
            dd[kr2] ** sigma[1] ** bb[kr1] ** Conjugate[bb[kl1]] ** sigma[1] ** 
              Conjugate[dd[kl2]])\))\)\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Find gamma^+ gamma^5, acts on pion", "Subsection"],

Cell["Terms in the Hamiltonian kappa_5/2 qq**qq", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qq[x_] = 
      relabel[Expand[\ \(\(conju[x] ** sigma[3] ** 
                    u[x] /. {0 ** _ \[Rule] 0, _ ** 0 \[Rule] 0}\) //. 
                prodexpand\) //. sigmazeta]] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\
\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\
\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ \
x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
      relabel[\((qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                    kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                    kr3 \[Rule] kkr3, s1 \[Rule] ss1})\) ** qqpinch //. 
            prodexpand] //. prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x \
+ \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
              sigma[2] ** 
              Conjugate[
                d[kl4]])\)\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** \
b[kr1]\)\/\(4\ \[Pi]\^2\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** \
Conjugate[d[kl2]]\)\/\(4\ \[Pi]\^2\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ d[kr4] ** sigma[2] ** b[kr3] \
** d[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ \[Pi]\^2\)\)], "Output"]
}, Open  ]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      Factor[kappa5/2*
          simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11]]\)\)\)], "Input"],

Cell[BoxData[
    \(\(\(1\/\(4\ \[Pi]\)\)\((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
            kr2]\ \((Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              d[kr2] ** sigma[2] ** b[kr1] + 
            d[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[d[kl2]])\))\)\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Find pseudoscalar 4-fermion interaction, acts on pion", "Subsection"],

Cell["Terms in the Hamiltonian kappa_5/2 qq**qq", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qq[x_] = 
      relabel[Expand[\ \(\(conjv[x] ** sigma[2] ** u[x] + 
                    conju[x] ** sigma[2] ** v[x] /. {0 ** _ \[Rule] 
                      0, _ ** 0 \[Rule] 0}\) //. prodexpand\) //. 
              sigmazeta]] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                Conjugate[
                  dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** sigma[3] ** bb[kr1]\)\/\(2\ \[Pi]\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** sigma[3] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** bb[kr1]\)\/\(2\ \[Pi]\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                Conjugate[
                  dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** \
bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)], \
"Output"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
      relabel[\((qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                    kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                    kr3 \[Rule] kkr3, s1 \[Rule] ss1})\) ** qqpinch //. 
            prodexpand] //. prodfactor\)\)\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl1\)\" is similar to \
existing symbol \"\!\(kl1\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl2\)\" is similar to \
existing symbol \"\!\(kl2\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkl3\)\" is similar to \
existing symbol \"\!\(kl3\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell1\"\) will be suppressed \
during this calculation."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr1\)\" is similar to \
existing symbols \!\({kkl1, kr1}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr2\)\" is similar to \
existing symbols \!\({kkl2, kr2}\)."\)], "Message"],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(kkr3\)\" is similar to \
existing symbols \!\({kkl3, kr3}\)."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    \(\(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** Conjugate[b[kl3]] ** 
              sigma[2] ** 
              Conjugate[
                dd[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
              Conjugate[bb[kl3]] ** sigma[2] ** 
              Conjugate[
                d[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
              d[kr2] ** sigma[2] ** 
              bb[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
              dd[kr2] ** sigma[2] ** 
              b[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              Conjugate[b[kl3]] ** sigma[2] ** 
              Conjugate[
                dd[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              Conjugate[bb[kl3]] ** sigma[2] ** 
              Conjugate[
                d[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              d[kr2] ** sigma[2] ** 
              bb[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              dd[kr2] ** sigma[2] ** 
              b[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[b[kl1]] ** 
              sigma[2] ** 
              Conjugate[
                dd[kl2]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[bb[kl1]] ** 
              sigma[2] ** 
              Conjugate[
                d[kl2]])\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ \
d[kr4] ** sigma[2] ** bb[kr3] ** d[kr2] ** sigma[2] ** bb[kr1]\)\/\(4\ \
\[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \
kr2\ x - \[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ d[kr4] ** sigma[2] ** \
bb[kr3] ** dd[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ \[Pi]\^2\) + \(1\/\(4\ \[Pi]\
\^2\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** 
              b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[
                dd[kl2]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[bb[kl1]] ** 
              sigma[2] ** 
              Conjugate[
                d[kl2]])\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ \
dd[kr4] ** sigma[2] ** b[kr3] ** d[kr2] ** sigma[2] ** bb[kr1]\)\/\(4\ \
\[Pi]\^2\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \
kr2\ x - \[ImaginaryI]\ kr3\ x - \[ImaginaryI]\ kr4\ x\)\ dd[kr4] ** sigma[2] \
** b[kr3] ** dd[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ \[Pi]\^2\)\)], "Output"]
}, Closed]],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      Factor[kappa5/2*
          simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11]]\)\)\)], "Input"],

Cell[BoxData[
    \(\(1\/\(4\ \[Pi]\)\) \((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ \((Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
                d[kr2] ** sigma[2] ** bb[kr1] + 
              Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** dd[kr2] ** 
                sigma[2] ** b[kr1] + 
              Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** d[kr2] ** 
                sigma[2] ** bb[kr1] + 
              Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** dd[kr2] ** 
                sigma[2] ** b[kr1] + 
              d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                Conjugate[dd[kl2]] + 
              d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[2] ** Conjugate[d[kl2]] + 
              dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                Conjugate[dd[kl2]] + 
              dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[2] ** Conjugate[d[kl2]])\))\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Find transverse vector 4-fermion interaction, acts on J_z = -1,+1 \
of a rho\
\>", "Subsection"],

Cell["Terms in the Hamiltonian -kappa_6/2 (qq**qq-rr**rr)", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qq[x_] = 
      relabel[Expand[\ \(\(conjv[x] ** sigma[3] ** u[x] + 
                    conju[x] ** sigma[3] ** v[x] /. {0 ** _ \[Rule] 
                      0, _ ** 0 \[Rule] 0}\) //. prodexpand\) //. 
              sigmazeta]] //. prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** bb[kr1]\)\/\(2\ \
\[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ \
x\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** sigma[2] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[3] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
sigma[2] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[3] ** \
b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rr[x_] = 
      relabel[Expand[\ \(\(conjv[x] ** u[x] - 
                    conju[x] ** v[x] /. {0 ** _ \[Rule] 0, _ ** 0 \[Rule] 
                      0}\) //. prodexpand\) //. sigmazeta]] //. 
        prodfactor\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
dd[kr2] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** bb[kr1]\)\/\(2\ \
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[1] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[1] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** sigma[1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Select terms that are leading order in N for mesons (\"pinching\" \
terms):\
\>", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(qqpinch = 
      qq[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\
\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                Conjugate[
                  dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[3] ** \
bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\)\)], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rrpinch = 
      rr[x] /. a_NonCommutativeMultiply \[RuleDelayed] 
          Block[{non = 
                a //. {sigma[_] ** c_ \[Rule] c, zeta[_] ** c_ \[Rule] c}}, 
            0 /; Count[non, Conjugate[_]] != Length[non] && 
                Count[non, Conjugate[_]] != 0]\)], "Input"],

Cell[BoxData[
    \(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
dd[kr2] ** b[kr1]\)\/\(2\ \[Pi]\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\(\ \)\( (*put\ in\ the\ coupling\ constant\ later*) \)\(expr1pinch = 
        relabel[\((qqpinch /. {kl1 \[Rule] kkl1, kl2 \[Rule] kkl2, 
                        kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, kr2 \[Rule] kkr2, 
                        kr3 \[Rule] kkr3, s1 \[Rule] ss1})\) ** 
                  qqpinch - \((rrpinch /. {kl1 \[Rule] kkl1, 
                        kl2 \[Rule] kkl2, kl3 \[Rule] kkl3, kr1 \[Rule] kkr1, 
                        kr2 \[Rule] kkr2, kr3 \[Rule] kkr3, 
                        s1 \[Rule] ss1})\) ** rrpinch //. prodexpand] //. 
          prodfactor;\)\)\)], "Input"],

Cell["\<\
expr2 contians all of the \"pinching\" interactions in the \
Hamiltonian.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\( (*kappa5\ is\ the\ coupling\ constant*) \)\(expr2pinch = 
      Factor[\(-kappa6\)/2*
          simpint[\(\(\(int[expr1pinch, {x}] //. intsumexpand\) /. 
                  Exp[x_] \[RuleDelayed] Exp[Factor[x]]\) /. 
                int[Exp[a_\ b_], {b_}] \[Rule] 2\ Pi\ delta[I\ a]\) /. 
              delta[_?neverzero] \[Rule] 0, k11]]\)\)\)], "Input"],

Cell[BoxData[
    \(\(-\(\(1\/\(4\ \[Pi]\)\)\((kappa6\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ \((Conjugate[b[kl1]] ** Conjugate[dd[kl2]] ** d[kr2] ** 
                bb[kr1] + 
              Conjugate[b[kl1]] ** Conjugate[dd[kl2]] ** dd[kr2] ** b[kr1] + 
              Conjugate[bb[kl1]] ** Conjugate[d[kl2]] ** d[kr2] ** bb[kr1] + 
              Conjugate[bb[kl1]] ** Conjugate[d[kl2]] ** dd[kr2] ** b[kr1] + 
              d[kr2] ** bb[kr1] ** Conjugate[b[kl1]] ** Conjugate[dd[kl2]] + 
              d[kr2] ** bb[kr1] ** Conjugate[bb[kl1]] ** Conjugate[d[kl2]] + 
              dd[kr2] ** b[kr1] ** Conjugate[b[kl1]] ** Conjugate[dd[kl2]] + 
              dd[kr2] ** b[kr1] ** Conjugate[bb[kl1]] ** Conjugate[d[kl2]] + 
              Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]] ** d[kr2] ** 
                sigma[3] ** bb[kr1] + 
              Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]] ** dd[kr2] ** 
                sigma[3] ** b[kr1] + 
              Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]] ** d[kr2] ** 
                sigma[3] ** bb[kr1] + 
              Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]] ** dd[kr2] ** 
                sigma[3] ** b[kr1] + 
              d[kr2] ** sigma[3] ** bb[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                Conjugate[dd[kl2]] + 
              d[kr2] ** sigma[3] ** bb[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[3] ** Conjugate[d[kl2]] + 
              dd[kr2] ** sigma[3] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                Conjugate[dd[kl2]] + 
              dd[kr2] ** sigma[3] ** b[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[3] ** Conjugate[d[kl2]])\))\)\)\)\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell["Junk", "Section"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
NotebookAutoSave->True,
AutoGeneratedPackage->Automatic,
WindowSize->{668, 697},
WindowMargins->{{Automatic, 77}, {Automatic, 7}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Landscape",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds"}, \
"four-fermi.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 45, 0, 64, "Subtitle"],
Cell[1787, 53, 63, 3, 71, "Subsubtitle"],
Cell[1853, 58, 175, 4, 50, "Text"],
Cell[2031, 64, 347, 6, 68, "Text"],

Cell[CellGroupData[{
Cell[2403, 74, 30, 0, 54, "Section"],
Cell[2436, 76, 87, 2, 27, "Input",
  InitializationCell->True],
Cell[2526, 80, 95, 3, 32, "Text"],
Cell[2624, 85, 334, 7, 59, "Input",
  InitializationCell->True],
Cell[2961, 94, 493, 10, 75, "Input",
  InitializationCell->True],
Cell[3457, 106, 1442, 25, 235, "Input",
  InitializationCell->True],
Cell[4902, 133, 230, 4, 59, "Input",
  InitializationCell->True],
Cell[5135, 139, 1206, 22, 219, "Input",
  InitializationCell->True],
Cell[6344, 163, 157, 3, 32, "Text"],
Cell[6504, 168, 1440, 28, 315, "Input",
  InitializationCell->True],
Cell[7947, 198, 193, 4, 50, "Text"],
Cell[8143, 204, 527, 10, 91, "Input",
  InitializationCell->True],
Cell[8673, 216, 399, 7, 91, "Input",
  InitializationCell->True],
Cell[9075, 225, 502, 10, 75, "Input",
  InitializationCell->True],
Cell[9580, 237, 199, 5, 27, "Input",
  InitializationCell->True],
Cell[9782, 244, 100, 3, 32, "Text"],
Cell[9885, 249, 546, 11, 107, "Input",
  InitializationCell->True],
Cell[10434, 262, 86, 2, 27, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[10557, 269, 42, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[10624, 273, 596, 12, 107, "Input"],
Cell[11223, 287, 185, 3, 37, "Message"]
}, Open  ]],
Cell[11423, 293, 283, 5, 59, "Input"],
Cell[11709, 300, 260, 5, 59, "Input"],
Cell[11972, 307, 282, 5, 75, "Input"],
Cell[12257, 314, 62, 1, 27, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[12356, 320, 48, 0, 34, "Section"],
Cell[12407, 322, 131, 3, 32, "Text"],
Cell[12541, 327, 380, 8, 43, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[12946, 339, 442, 8, 75, "Input",
  InitializationCell->True],
Cell[13391, 349, 185, 3, 70, "Message"]
}, Open  ]],
Cell[13591, 355, 95, 3, 32, "Text"],
Cell[13689, 360, 487, 9, 91, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[14201, 373, 380, 7, 59, "Input",
  InitializationCell->True],
Cell[14584, 382, 1509, 20, 70, "Output"]
}, Open  ]],
Cell[16108, 405, 91, 3, 32, "Text"],
Cell[16202, 410, 156, 3, 27, "Input",
  InitializationCell->True],
Cell[16361, 415, 156, 3, 27, "Input",
  InitializationCell->True],
Cell[16520, 420, 156, 3, 27, "Input",
  InitializationCell->True],
Cell[16679, 425, 156, 3, 27, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[16860, 432, 39, 0, 45, "Subsection"],
Cell[16902, 434, 306, 6, 91, "Input"],
Cell[17211, 442, 180, 4, 32, "Text"],

Cell[CellGroupData[{
Cell[17416, 450, 271, 5, 59, "Input"],
Cell[17690, 457, 24413, 371, 3407, "Output"]
}, Closed]],
Cell[42118, 831, 70, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[42213, 837, 276, 6, 59, "Input"],
Cell[42492, 845, 231, 3, 81, "Output"]
}, Open  ]],
Cell[42738, 851, 69, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[42832, 857, 276, 6, 59, "Input"],
Cell[43111, 865, 2377, 37, 328, "Output"]
}, Open  ]],
Cell[45503, 905, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[45598, 911, 285, 6, 59, "Input"],
Cell[45886, 919, 2485, 37, 369, "Output"]
}, Open  ]],
Cell[48386, 959, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[48481, 965, 285, 6, 59, "Input"],
Cell[48769, 973, 5197, 79, 784, "Output"]
}, Open  ]],
Cell[53981, 1055, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[54076, 1061, 285, 6, 43, "Input"],
Cell[54364, 1069, 8296, 126, 1287, "Output"]
}, Open  ]],
Cell[62675, 1198, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[62770, 1204, 285, 6, 59, "Input"],
Cell[63058, 1212, 5905, 89, 933, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[69012, 1307, 38, 0, 29, "Subsection"],

Cell[CellGroupData[{
Cell[69075, 1311, 630, 11, 123, "Input"],
Cell[69708, 1324, 1132, 16, 304, "Output"]
}, Open  ]],
Cell[70855, 1343, 72, 0, 32, "Text"],
Cell[70930, 1345, 330, 6, 75, "Input"],
Cell[71263, 1353, 996, 20, 130, "Text"],

Cell[CellGroupData[{
Cell[72284, 1377, 263, 5, 59, "Input"],
Cell[72550, 1384, 12388, 184, 2732, "Output"]
}, Closed]],
Cell[84953, 1571, 70, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[85048, 1577, 285, 6, 59, "Input"],
Cell[85336, 1585, 3173, 43, 942, "Output"]
}, Open  ]],
Cell[88524, 1631, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[88619, 1637, 285, 6, 59, "Input"],
Cell[88907, 1645, 5175, 75, 1363, "Output"]
}, Open  ]],
Cell[94097, 1723, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[94192, 1729, 285, 6, 59, "Input"],
Cell[94480, 1737, 4053, 65, 970, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[98582, 1808, 47, 0, 29, "Subsection"],
Cell[98632, 1810, 98, 3, 32, "Text"],
Cell[98733, 1815, 145, 2, 43, "Input"],

Cell[CellGroupData[{
Cell[98903, 1821, 114, 5, 42, "Subsubsection"],
Cell[99020, 1828, 100, 3, 32, "Text"],
Cell[99123, 1833, 106, 2, 27, "Input"],
Cell[99232, 1837, 302, 5, 75, "Input"],

Cell[CellGroupData[{
Cell[99559, 1846, 262, 5, 59, "Input"],
Cell[99824, 1853, 132, 2, 49, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[100005, 1861, 45, 0, 42, "Subsubsection"],
Cell[100053, 1863, 159, 3, 50, "Text"],
Cell[100215, 1868, 204, 4, 43, "Input"],
Cell[100422, 1874, 519, 9, 123, "Input"],
Cell[100944, 1885, 277, 5, 59, "Input"],
Cell[101224, 1892, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[101319, 1898, 286, 6, 59, "Input"],
Cell[101608, 1906, 1420, 19, 450, "Output"]
}, Open  ]],
Cell[103043, 1928, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[103138, 1934, 286, 6, 59, "Input"],
Cell[103427, 1942, 2247, 31, 662, "Output"]
}, Open  ]],
Cell[105689, 1976, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[105784, 1982, 286, 6, 59, "Input"],
Cell[106073, 1990, 1903, 28, 473, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[108025, 2024, 115, 5, 42, "Subsubsection"],
Cell[108143, 2031, 156, 4, 32, "Text"],
Cell[108302, 2037, 229, 5, 43, "Input"],
Cell[108534, 2044, 463, 8, 107, "Input"],
Cell[109000, 2054, 277, 5, 59, "Input"],
Cell[109280, 2061, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[109375, 2067, 286, 6, 59, "Input"],
Cell[109664, 2075, 371, 5, 101, "Output"]
}, Open  ]],
Cell[110050, 2083, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[110145, 2089, 286, 6, 59, "Input"],
Cell[110434, 2097, 795, 11, 203, "Output"]
}, Open  ]],
Cell[111244, 2111, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[111339, 2117, 286, 6, 59, "Input"],
Cell[111628, 2125, 489, 7, 141, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[112166, 2138, 115, 5, 42, "Subsubsection"],
Cell[112284, 2145, 156, 4, 32, "Text"],
Cell[112443, 2151, 214, 4, 43, "Input"],
Cell[112660, 2157, 463, 8, 107, "Input"],
Cell[113126, 2167, 277, 5, 59, "Input"],
Cell[113406, 2174, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[113501, 2180, 286, 6, 59, "Input"],
Cell[113790, 2188, 717, 10, 238, "Output"]
}, Open  ]],
Cell[114522, 2201, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[114617, 2207, 286, 6, 59, "Input"],
Cell[114906, 2215, 1163, 16, 347, "Output"]
}, Open  ]],
Cell[116084, 2234, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[116179, 2240, 286, 6, 59, "Input"],
Cell[116468, 2248, 983, 15, 273, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[117500, 2269, 115, 5, 42, "Subsubsection"],
Cell[117618, 2276, 62, 0, 32, "Text"],
Cell[117683, 2278, 227, 5, 43, "Input"],
Cell[117913, 2285, 468, 8, 107, "Input"],
Cell[118384, 2295, 277, 5, 59, "Input"],
Cell[118664, 2302, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[118759, 2308, 286, 6, 59, "Input"],
Cell[119048, 2316, 713, 10, 238, "Output"]
}, Open  ]],
Cell[119776, 2329, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[119871, 2335, 286, 6, 59, "Input"],
Cell[120160, 2343, 1157, 16, 347, "Output"]
}, Open  ]],
Cell[121332, 2362, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[121427, 2368, 286, 6, 59, "Input"],
Cell[121716, 2376, 962, 14, 257, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[122727, 2396, 138, 5, 42, "Subsubsection"],
Cell[122868, 2403, 62, 0, 32, "Text"],
Cell[122933, 2405, 325, 8, 59, "Input"],
Cell[123261, 2415, 506, 9, 107, "Input"],
Cell[123770, 2426, 277, 5, 59, "Input"],
Cell[124050, 2433, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[124145, 2439, 286, 6, 59, "Input"],
Cell[124434, 2447, 2691, 35, 874, "Output"]
}, Open  ]],
Cell[127140, 2485, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[127235, 2491, 286, 6, 59, "Input"],
Cell[127524, 2499, 5040, 74, 1298, "Output"]
}, Open  ]],
Cell[132579, 2576, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[132674, 2582, 286, 6, 59, "Input"],
Cell[132963, 2590, 3700, 54, 937, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[136724, 2651, 102, 3, 29, "Subsection"],
Cell[136829, 2656, 112, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[136966, 2663, 314, 6, 91, "Input"],
Cell[137283, 2671, 5363, 77, 990, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[142683, 2753, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[142816, 2760, 308, 6, 75, "Input"],
Cell[143127, 2768, 1549, 23, 338, "Output"]
}, Open  ]],
Cell[144691, 2794, 490, 8, 123, "Input"],
Cell[145184, 2804, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[145306, 2811, 349, 5, 75, "Input"],
Cell[145658, 2818, 9085, 132, 1366, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[154792, 2956, 75, 0, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[154892, 2960, 124, 2, 27, "Input"],
Cell[155019, 2964, 1453, 22, 271, "Output"]
}, Open  ]],
Cell[156487, 2989, 486, 8, 123, "Input"],
Cell[156976, 2999, 195, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[157196, 3007, 357, 5, 91, "Input"],
Cell[157556, 3014, 13703, 200, 1983, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[171320, 3221, 29, 0, 29, "Subsection"],
Cell[171352, 3223, 147, 2, 43, "Input"],
Cell[171502, 3227, 33, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[171560, 3231, 266, 5, 75, "Input"],
Cell[171829, 3238, 4822, 69, 946, "Output"]
}, Closed]],
Cell[176666, 3310, 41, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[176732, 3314, 259, 5, 59, "Input"],
Cell[176994, 3321, 6655, 93, 1337, "Output"]
}, Closed]],
Cell[183664, 3417, 41, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[183730, 3421, 228, 4, 43, "Input"],
Cell[183961, 3427, 6513, 91, 1315, "Output"]
}, Closed]],
Cell[190489, 3521, 41, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[190555, 3525, 170, 3, 43, "Input"],
Cell[190728, 3530, 538, 8, 85, "Output"]
}, Open  ]],
Cell[191281, 3541, 41, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[191347, 3545, 170, 3, 43, "Input"],
Cell[191520, 3550, 25721, 379, 4104, "Output"]
}, Closed]],
Cell[217256, 3932, 41, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[217322, 3936, 263, 5, 75, "Input"],
Cell[217588, 3943, 5342, 77, 979, "Output"]
}, Closed]],
Cell[222945, 4023, 49, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[223019, 4027, 223, 4, 43, "Input"],
Cell[223245, 4033, 6065, 85, 1294, "Output"]
}, Closed]],
Cell[229325, 4121, 49, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[229399, 4125, 263, 5, 75, "Input"],
Cell[229665, 4132, 6571, 95, 1337, "Output"]
}, Closed]],
Cell[236251, 4230, 49, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[236325, 4234, 182, 3, 43, "Input"],
Cell[236510, 4239, 592, 8, 124, "Output"]
}, Open  ]],
Cell[237117, 4250, 49, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[237191, 4254, 182, 3, 43, "Input"],
Cell[237376, 4259, 28079, 413, 4377, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[265504, 4678, 101, 3, 45, "Subsection"],
Cell[265608, 4683, 447, 8, 75, "Input",
  InitializationCell->True],
Cell[266058, 4693, 95, 3, 32, "Text"],
Cell[266156, 4698, 532, 10, 91, "Input",
  InitializationCell->True],
Cell[266691, 4710, 73, 0, 32, "Text"],
Cell[266767, 4712, 359, 6, 59, "Input"],

Cell[CellGroupData[{
Cell[267151, 4722, 165, 3, 27, "Input"],
Cell[267319, 4727, 195, 3, 51, "Output"]
}, Open  ]],
Cell[267529, 4733, 112, 3, 32, "Text"],
Cell[267644, 4738, 305, 5, 91, "Input"],

Cell[CellGroupData[{
Cell[267974, 4747, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[268107, 4754, 360, 8, 91, "Input"],
Cell[268470, 4764, 47, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[268554, 4770, 453, 7, 107, "Input"],
Cell[269010, 4779, 180, 3, 37, "Message"],
Cell[269193, 4784, 180, 3, 37, "Message"],
Cell[269376, 4789, 180, 3, 37, "Message"],
Cell[269559, 4794, 167, 3, 22, "Message"],
Cell[269729, 4799, 184, 3, 37, "Message"],
Cell[269916, 4804, 184, 3, 37, "Message"],
Cell[270103, 4809, 184, 3, 37, "Message"],
Cell[270290, 4814, 166, 3, 22, "Message"]
}, Open  ]],
Cell[270471, 4820, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[270593, 4827, 374, 6, 91, "Input"],
Cell[270970, 4835, 106, 2, 43, "Output"]
}, Open  ]],
Cell[271091, 4840, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[271191, 4844, 89, 1, 27, "Input"],
Cell[271283, 4847, 106, 2, 43, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[271438, 4855, 75, 0, 42, "Subsubsection"],
Cell[271516, 4857, 211, 4, 43, "Input"],

Cell[CellGroupData[{
Cell[271752, 4865, 449, 7, 107, "Input"],
Cell[272204, 4874, 42, 1, 27, "Output"]
}, Open  ]],
Cell[272261, 4878, 195, 4, 50, "Text"],
Cell[272459, 4884, 357, 5, 91, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[272865, 4895, 117, 3, 29, "Subsection"],
Cell[272985, 4900, 122, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[273132, 4907, 314, 6, 91, "Input"],
Cell[273449, 4915, 6743, 94, 1293, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[280229, 5014, 293, 6, 72, "Input"],
Cell[280525, 5022, 6215, 86, 1283, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[286777, 5113, 108, 3, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[286910, 5120, 308, 6, 75, "Input"],
Cell[287221, 5128, 2049, 30, 435, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[289307, 5163, 308, 6, 72, "Input"],
Cell[289618, 5171, 1817, 26, 379, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[291472, 5202, 866, 15, 168, "Input"],
Cell[292341, 5219, 180, 3, 37, "Message"],
Cell[292524, 5224, 180, 3, 37, "Message"],
Cell[292707, 5229, 180, 3, 37, "Message"],
Cell[292890, 5234, 167, 3, 22, "Message"],
Cell[293060, 5239, 184, 3, 37, "Message"],
Cell[293247, 5244, 184, 3, 37, "Message"],
Cell[293434, 5249, 184, 3, 37, "Message"],
Cell[293621, 5254, 166, 3, 22, "Message"]
}, Open  ]],
Cell[293802, 5260, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[293924, 5267, 361, 5, 75, "Input"],
Cell[294288, 5274, 28435, 419, 4203, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[322784, 5700, 116, 3, 29, "Subsection"],
Cell[322903, 5705, 63, 0, 32, "Text"],
Cell[322969, 5707, 120, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[323114, 5714, 217, 5, 43, "Input"],
Cell[323334, 5721, 538, 8, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[323909, 5734, 244, 5, 59, "Input"],
Cell[324156, 5741, 25721, 379, 4079, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[349914, 6125, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[350047, 6132, 308, 6, 75, "Input"],
Cell[350358, 6140, 298, 4, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[350693, 6149, 308, 6, 75, "Input"],
Cell[351004, 6157, 4269, 64, 793, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[355310, 6226, 57, 1, 27, "Input"],
Cell[355370, 6229, 4111, 62, 742, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[359518, 6296, 60, 1, 27, "Input"],
Cell[359581, 6299, 4082, 61, 698, "Output"]
}, Open  ]],
Cell[363678, 6363, 930, 15, 171, "Input"],

Cell[CellGroupData[{
Cell[364633, 6382, 43, 1, 27, "Input"],
Cell[364679, 6385, 23831, 359, 3192, "Output"]
}, Open  ]],
Cell[388525, 6747, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[388647, 6754, 359, 5, 75, "Input"],
Cell[389009, 6761, 11074, 163, 1666, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[400156, 6932, 64, 0, 34, "Section"],
Cell[400223, 6934, 131, 3, 32, "Text"],
Cell[400357, 6939, 116, 3, 32, "Text"],
Cell[400476, 6944, 1580, 23, 564, "Input"],

Cell[CellGroupData[{
Cell[402081, 6971, 453, 9, 75, "Input",
  InitializationCell->True],
Cell[402537, 6982, 1202, 17, 333, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[403776, 7004, 102, 3, 45, "Subsection"],
Cell[403881, 7009, 112, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[404018, 7016, 314, 6, 123, "Input"],
Cell[404335, 7024, 5273, 75, 1222, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[409645, 7104, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[409778, 7111, 308, 6, 75, "Input"],
Cell[410089, 7119, 2569, 39, 591, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[412695, 7163, 475, 8, 107, "Input"],
Cell[413173, 7173, 29348, 447, 3722, "Output"]
}, Closed]],
Cell[442536, 7623, 97, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[442658, 7630, 573, 9, 123, "Input"],
Cell[443234, 7641, 41208, 592, 6197, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[484515, 8241, 78, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[484618, 8245, 95, 3, 45, "Subsection"],
Cell[484716, 8250, 133, 3, 32, "Text"],
Cell[484852, 8255, 113, 3, 32, "Text"],
Cell[484968, 8260, 314, 6, 91, "Input"],

Cell[CellGroupData[{
Cell[485307, 8270, 108, 3, 42, "Subsubsection"],
Cell[485418, 8275, 308, 6, 59, "Input"],
Cell[485729, 8283, 490, 8, 123, "Input"],
Cell[486222, 8293, 97, 3, 32, "Text"],
Cell[486322, 8298, 349, 5, 75, "Input"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[486720, 8309, 123, 3, 29, "Subsection"],
Cell[486846, 8314, 161, 4, 50, "Text"],
Cell[487010, 8320, 121, 3, 32, "Text"],
Cell[487134, 8325, 314, 6, 91, "Input"],
Cell[487451, 8333, 288, 6, 75, "Input"],

Cell[CellGroupData[{
Cell[487764, 8343, 108, 3, 42, "Subsubsection"],
Cell[487875, 8348, 308, 6, 59, "Input"],
Cell[488186, 8356, 308, 6, 59, "Input"],
Cell[488497, 8364, 866, 15, 171, "Input"],
Cell[489366, 8381, 97, 3, 32, "Text"],
Cell[489466, 8386, 363, 5, 75, "Input"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[489890, 8398, 71, 0, 54, "Section"],
Cell[489964, 8400, 131, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[490120, 8407, 954, 21, 155, "Input",
  InitializationCell->True],
Cell[491077, 8430, 183, 3, 33, "Message"]
}, Open  ]],
Cell[491275, 8436, 324, 5, 75, "Input",
  InitializationCell->True],
Cell[491602, 8443, 91, 3, 32, "Text"],
Cell[491696, 8448, 156, 3, 43, "Input",
  InitializationCell->True],
Cell[491855, 8453, 224, 4, 43, "Input",
  InitializationCell->True],
Cell[492082, 8459, 287, 5, 75, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[492394, 8468, 39, 0, 45, "Subsection"],
Cell[492436, 8470, 44, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[492505, 8474, 1506, 27, 219, "Input"],
Cell[494014, 8503, 8568, 122, 1903, "Output"]
}, Closed]],
Cell[502597, 8628, 180, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[502802, 8636, 271, 5, 59, "Input"],
Cell[503076, 8643, 3906, 56, 1207, "Output"]
}, Closed]],
Cell[506997, 8702, 70, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[507092, 8708, 373, 7, 43, "Input"],
Cell[507468, 8717, 313, 6, 110, "Output"]
}, Open  ]],
Cell[507796, 8726, 69, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[507890, 8732, 276, 6, 59, "Input"],
Cell[508169, 8740, 1623, 23, 514, "Output"]
}, Open  ]],
Cell[509807, 8766, 70, 2, 32, "Text"],

Cell[CellGroupData[{
Cell[509902, 8772, 285, 6, 59, "Input"],
Cell[510190, 8780, 1819, 26, 556, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[512058, 8812, 29, 0, 29, "Subsection"],
Cell[512090, 8814, 57, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[512172, 8818, 214, 4, 59, "Input"],
Cell[512389, 8824, 546, 8, 167, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[512972, 8837, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[513105, 8844, 308, 6, 75, "Input"],
Cell[513416, 8852, 347, 6, 87, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[513800, 8863, 362, 5, 75, "Input"],
Cell[514165, 8870, 1099, 18, 250, "Output"]
}, Open  ]],
Cell[515279, 8891, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[515401, 8898, 374, 6, 91, "Input"],
Cell[515778, 8906, 343, 5, 94, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[516182, 8918, 56, 0, 45, "Subsection"],
Cell[516241, 8920, 57, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[516323, 8924, 226, 4, 59, "Input"],
Cell[516552, 8930, 592, 8, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[517181, 8943, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[517314, 8950, 308, 6, 75, "Input"],
Cell[517625, 8958, 330, 5, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[517992, 8968, 362, 5, 59, "Input"],
Cell[518357, 8975, 1009, 15, 197, "Output"]
}, Open  ]],
Cell[519381, 8993, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[519503, 9000, 374, 6, 91, "Input"],
Cell[519880, 9008, 335, 5, 77, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[520276, 9020, 75, 0, 29, "Subsection"],
Cell[520354, 9022, 57, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[520436, 9026, 278, 5, 75, "Input"],
Cell[520717, 9033, 1208, 18, 241, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[521962, 9056, 108, 3, 28, "Subsubsection"],

Cell[CellGroupData[{
Cell[522095, 9063, 308, 6, 75, "Input"],
Cell[522406, 9071, 616, 10, 124, "Output"]
}, Closed]],

Cell[CellGroupData[{
Cell[523059, 9086, 362, 5, 56, "Input"],
Cell[523424, 9093, 180, 3, 37, "Message"],
Cell[523607, 9098, 180, 3, 37, "Message"],
Cell[523790, 9103, 180, 3, 37, "Message"],
Cell[523973, 9108, 167, 3, 22, "Message"],
Cell[524143, 9113, 184, 3, 37, "Message"],
Cell[524330, 9118, 184, 3, 37, "Message"],
Cell[524517, 9123, 184, 3, 37, "Message"],
Cell[524704, 9128, 166, 3, 22, "Message"],
Cell[524873, 9133, 4295, 67, 758, "Output"]
}, Closed]],
Cell[529183, 9203, 97, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[529305, 9210, 374, 6, 91, "Input"],
Cell[529682, 9218, 1058, 17, 173, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[530801, 9242, 106, 3, 29, "Subsection"],
Cell[530910, 9247, 67, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[531002, 9251, 278, 5, 91, "Input"],
Cell[531283, 9258, 1163, 16, 327, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[532483, 9279, 248, 5, 75, "Input"],
Cell[532734, 9286, 1051, 14, 287, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[533822, 9305, 108, 3, 42, "Subsubsection"],

Cell[CellGroupData[{
Cell[533955, 9312, 308, 6, 75, "Input"],
Cell[534266, 9320, 616, 10, 167, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[534919, 9335, 308, 6, 75, "Input"],
Cell[535230, 9343, 521, 7, 127, "Output"]
}, Open  ]],
Cell[535766, 9353, 620, 9, 123, "Input"],
Cell[536389, 9364, 97, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[536511, 9371, 379, 6, 91, "Input"],
Cell[536893, 9379, 1707, 26, 334, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[538651, 9411, 23, 0, 54, "Section"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

