(*
Construct the Hamiltonian
*)
(*
Brett van de Sande
June 20, 2001
*)
(*
The notation used in this notebook corresponds to the notation introduced in \
the LaTeX notes "endpoint.tex" and to Simon's paper, hep-ph/0101318.
*)
(*
In the following, there are implicit integrations over all momenta, implicit \
sums over repeated flavor indices, and implicit sums over all link \
directions.  The b[...] and d[...] are spinors and vectors in color space.  \
The a[...] are matrices in color space.  sigma[...] and zeta[...] are \
matrices in spinor-space.
*)
(*
Definitions
*)
Off[General::"spell1"]
(*
All symbols are considered to be c-number unless specified otherwise:
*)
cnumberq[x_] := 
  FreeQ[x, sigma[_]] && FreeQ[x, a[__]] && FreeQ[x, b[__]] && 
    FreeQ[x, d[__]] && FreeQ[x, bb[__]] && FreeQ[x, dd[__]] && 
    FreeQ[x, zeta[_]]
freelistq[x_, y_List] := Apply[And, Map[FreeQ[x, #] &, y]]
prodexpand = {(a__) ** b_Plus :> Map[(a ** #) &, b], 
      b_Plus ** (a__) :> Map[(# ** a) &, b],
      a_ ** b_Times :> Select[b, cnumberq] a ** Select[b, ! cnumberq[#] &], 
      b_Times ** a_ :> Select[b, cnumberq]*Select[b, ! cnumberq[#] &] ** a};
intsumexpand = {a_ ** int[b_, c_] :> int[a ** b, c] /; freelistq[a, c], 
      int[b_, c_] ** a_ :> int[b ** a, c] /; freelistq[a, c],
      a_ ** sum[b_, c_] :> sum[a ** b, c] /; freelistq[a, c], 
      sum[b_, c_] ** a_ :> sum[b ** a, c] /; freelistq[a, c],
      int[a_Times, b_] :> 
        Module[{vars = Select[a, freelistq[#, b] &]}, 
          vars int[Select[a, ! freelistq[#, b] &], b] /; vars =!= 1],
      sum[a_Times, b_] :> 
        Module[{vars = Select[a, freelistq[#, b] &]}, 
          vars sum[Select[a, ! freelistq[#, b] &], b] /; vars =!= 1],
      int[a_Plus, b_] :> Map[int[#, b] &, a],
      sum[a_Plus, b_] :> Map[sum[#, b] &, a],
      int[d_. sum[a_, b_], c_] -> sum[int[d a, c], b],
      int [a_. int[b_, c_], d_] :> 
        int[Expand[a b], Join[c, d]] /; freelistq[a, c],
      sum [a_. sum[b_, c_], d_] :> 
        sum[Expand[a b], Join[c, d]] /; freelistq[a, c],
      int[a_, b_] :> int[Expand[a], b] /; Expand[a] =!= a}; 
prodfactor = {a_. c_NonCommutativeMultiply + b_. c_NonCommutativeMultiply :> 
        Simplify[Factor[Expand[a] + Expand[b]]] c};
sigmazeta = Join[prodexpand, {
        a[b__] ** sigma[c_] -> sigma[c] ** a[b], 
        Conjugate[a[b__]] ** sigma[c_] -> sigma[c] ** Conjugate[a[b]],
        a[b__] ** zeta[c_] -> zeta[c] ** a[b], 
        Conjugate[a[b__]] ** zeta[c_] -> zeta[c] ** Conjugate[a[b]],
        sigma[a_Integer] ** sigma[a_Integer] ** b_ -> b,
        b_ ** sigma[a_Integer] ** sigma[a_Integer] -> b,
        sigma[2] ** zeta[a_] ** sigma[1] -> sigma[1] ** zeta[a] ** sigma[2], 
        sigma[a_Integer] ** sigma[b_Integer] :> 
          Block[{c = First[Complement[{1, 2, 3}, {a, b}]]}, 
              I Signature[{a, b, c}] sigma[c]] /; a != b,
        sigma[b_] ** zeta[a_] ** sigma[3] -> -1*
            sigma[b] ** sigma[3] ** zeta[a], 
        sigma[3] ** zeta[a_] -> -1*zeta[a] ** sigma[3]}];
(*
Relabel the parameters for implicit sums and integrations.  It acts on an \
expression that is linear in NonCommutativeMultiply[...].
*)
relabel[a_Plus] := Map[relabel, a];
relabel[ain_*bin_NonCommutativeMultiply] := 
  Block[{i1 = 1, i2 = 1, j = 1, krules, kcrules, srules, ops, cops}, 
    ops = Cases[Apply[List, bin], 
        a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | bb[_Symbol, ___] |
           dd[_Symbol, ___]]; 
    cops = Cases[Apply[List, bin], 
        Conjugate[
          a[_Symbol, _] | b[_Symbol, ___] | d[_Symbol, ___] | 
            bb[_Symbol, ___] | dd[_Symbol, ___]]];
    (*Print[{ops, cops}];*) 
    krules = 
      Map[(First[#] -> Symbol["kr" <> ToString[i2++]]) &, Reverse[ops]]; 
    kcrules = 
      Map[(First[First[#]] -> Symbol["kl" <> ToString[i1++]]) &, cops]; 
    srules = Map[(# -> Symbol["s" <> ToString[j++]]) &, 
        Union[Map[Last, Cases[
              
              Apply[List, bin] /. Conjugate[ex_] -> ex /. 
                a[_, -b_Symbol] -> a[0, b], 
              a[_, _Symbol] | zeta[_Symbol] | sigma[_Symbol]
              ]]]];
    ain bin /. Join[krules, kcrules] /. srules]
(*
simplify an integration over x, assuming the variable is in a delta-function. \
 It acts on an expression that is linear in delta[...] and \
NonCommutativeMultiply[...].
*)
simpint[a_Plus, x_] := Map[simpint[#, x] &, a];
simpint[a_ b_, x_] := a simpint[b, x] /; FreeQ[a, x];
simpint[a_  delta[-x_ + c_], x_] := simpint[a delta[x - c], x];
simpint[a_  delta[x_ + c_], x_] := Simplify[a /. x -> -c]
simpint[a_ , x_] := a /; FreeQ[a, x];
Unprotect[Conjugate];
Conjugate[a_Plus] := Map[Conjugate, a];
Conjugate[a_Times] := Map[Conjugate, a];
Conjugate[a_ ** b_] := Conjugate[b] ** Conjugate[a];
Protect[Conjugate];
sigma /: Conjugate[sigma[a_]] = sigma[a];
zeta /: Conjugate[zeta[a_]] := zeta[a];
int /: Conjugate[int[a_, b_]] := 
    int[Conjugate[a] /. Map[(Conjugate[#] -> #) &, b], b];
sum /: Conjugate[sum[a_, b_]] := 
    sum[Conjugate[a] /. Map[(Conjugate[#] -> #) &, b], b];
intconv[x_] := (int[a_, b_] :> 
      int[Integrate[a, x], Complement[b, {x}]] /; MemberQ[b, x])
(*
A variable x is defined to be positive[x]=True if it is *always* positive.
*)
Unprotect[Positive];
Positive[a_ + b_] := False /; ! Positive[a] && ! Positive[b];
Positive[a_?Positive + b_?Positive] = True;
Positive[a_?Negative b_?Positive] = False;
Protect[Positive];
neverzero[a_] := If[Positive[a], True, True, False];
On[General::"spell1"]
(*
Find Yukawa terms (OLD)
*)
(*
u[x_] := 1/Sqrt[2 Pi] int[
      b[kf] Exp[-I kf x] + sigma[1] ** Conjugate[d[kf]] Exp[I kf x], {kf}]
m[x_, rho_] := 
  1/Sqrt[4 Pi] int[(a[ka, -rho] Exp[-I ka x] + 
            Conjugate[a[ka, rho]] Exp[I ka x])/Sqrt[ka], {ka}]
kappaa /: Conjugate[kappaa] := kappaa
kappas /: Conjugate[kappas] := kappas
muf /: Conjugate[muf] := muf
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kappas\)\" \
is similar to existing symbol \"\!\(kappaa\)\"."
*)
(*
xi[x_] = Expand[
    muf/Sqrt[2] sigma[1] ** u[x] + 
        kappaa/Sqrt[2] sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
        kappas/Sqrt[2] sum[sigma[1] ** m[x, rr] ** u[x], {rr}] //. 
      prodexpand(*//. sigmazeta*)]
*)
(*
cxi[x_] = 
  Conjugate[xi[x]] /. {Conjugate[x] -> x, Conjugate[ka^n_] -> ka^n} /. {ka -> 
            kka, kf -> kks, rr -> ll} //. prodexpand //. sigmazeta
*)
(*
expr = Collect[
    Expand[(-I cxi[x] ** int[xi[x], {x}] + I int[cxi[x], {x}] ** xi[x])/2 //. 
              prodexpand /. intconv[x] //. prodexpand //. sigmazeta], {muf, 
      kappaa, kappas}]
*)
(*
int[expr, {x}] //. prodexpand
*)
(*
Find terms in the Hamiltonian
*)
(*
This version leaves all of the integrations over momenta and position \
implicit.  Also, sums are implicit.
*)
u[x_] := 1/
        Sqrt[2 Pi] (b[kf] Exp[-I kf x] + 
          sigma[1] ** Conjugate[d[kf]] Exp[I kf x]);
conju[x_] = 
    Conjugate[u[x]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks;
m[x_, rho_] := 
    1/Sqrt[4 Pi] (a[ka, -rho] Exp[-I ka x] + 
            Conjugate[a[ka, rho]] Exp[I ka x])/Sqrt[ka];
kappaa /: Conjugate[kappaa] := kappaa;
kappas /: Conjugate[kappas] := kappas;
muf /: Conjugate[muf] := muf
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kappas\)\" \
is similar to existing symbol \"\!\(kappaa\)\"."
*)
(*
Xi is used for the Yukawa interactions and the 4-fermion interactions
*)
xi[x_] = Expand[
      muf/Sqrt[2] sigma[1] ** u[x] + 
              kappaa/Sqrt[2] sigma[1] ** zeta[rr] ** m[x, rr] ** u[x] - 
              kappas/Sqrt[2] sigma[1] ** m[x, sumrr] ** u[x] //. 
            prodexpand /. {zeta[rho_] ** 
                a[c_, -rho_] -> -zeta[rho] ** a[c, rho], 
            a[b_, -sumrr] -> a[b, sumrr]} //. prodexpand];
cxi[x_] = 
  Conjugate[xi[x]] /. {Conjugate[x] -> x, Conjugate[ka] -> ka, 
            Conjugate[kf] -> kf, Conjugate[ka^n_] -> ka^n} /. {ka -> kka, 
          kf -> kks, rr -> ll, sumrr -> sumll} //. prodexpand //. sigmazeta
(*
\!\(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ muf\ d[kks]\)\/\(2\ \@\
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ muf\ \
Conjugate[b[kks]] ** sigma[1]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x - \
\[ImaginaryI]\ kks\ x\)\ kappas\ d[kks] ** Conjugate[a[kka, sumll]]\)\/\(4\ \
\@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \
\[ImaginaryI]\ kks\ x\)\ kappas\ Conjugate[b[kks]] ** sigma[1] ** \
Conjugate[a[kka, sumll]]\)\/\(4\ \@kka\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x + \[ImaginaryI]\ kks\ x\)\ kappaa\ Conjugate[b[kks]] ** \
zeta[ll] ** sigma[1] ** a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\(1\/\(4\ \@kka\
\ \[Pi]\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ \
x\)\ kappaa\ Conjugate[b[kks]] ** zeta[ll] ** sigma[1] ** 
          Conjugate[
            a[kka, ll]])\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kka\ \
x - \[ImaginaryI]\ kks\ x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** \
sigma[1] ** a[kka, ll]\)\/\(4\ \@kka\ \[Pi]\) - \(\(1\/\(4\ \@kka\ \
\[Pi]\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x - \[ImaginaryI]\ kks\ \
x\)\ kappaa\ d[kks] ** sigma[1] ** zeta[ll] ** sigma[1] ** 
          Conjugate[a[kka, ll]])\)\)\)
*)
(*
This is to define the momenta variables to be *always * positive.
*)
kl1 /: Positive[kl1] = True; kl2 /: Positive[kl2] = True; kl3 /: 
  Positive[kl3] = True;
kl4 /: Positive[kl4] = True; kl5 /: Positive[kl5] = True; kl6 /: 
  Positive[kl6] = True;
kr1 /: Positive[kr1] = True; kr2 /: Positive[kr2] = True; kr3 /: 
  Positive[kr3] = True;
kr4 /: Positive[kr4] = True; kr5 /: Positive[kr5] = True; kr6 /: 
  Positive[kr6] = True;
(*
Find Yukawa terms
*)
(*
expr1 = Collect[
        relabel[Expand[(-I cxi[x] ** Integrate[xi[x], x] + 
                      I Integrate[cxi[x], x] ** xi[x])/2 //. prodexpand //. 
              sigmazeta]], {muf, kappaa, kappas}] //. prodfactor;
*)
(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the interactions in the Hamiltonian.
*)
(*
expr2 = simpint[
    int[expr1, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0, 
    k11]
*)
(*
\!\(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\) + 
    kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                    kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                    b[kl1]] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                    kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kr1 + 
                    kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\))\) + 
    muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                          kl1\^2\ \((2\ kr1 + kr2)\) + 
                          kl1\ \((\(-2\)\ kr1\^2 + 
                                kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) - \((\((kl1\^2\ \((kl2 - 
                                2\ kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                          kl1\ \((kl2\^2 - 2\ kr1\^2)\))\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl2 + 
                          kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                          2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[
                          kr1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1 + 
                          kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                                kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                          kl1\ \((\(-2\)\ kl2\^2 + 
                                kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                          kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@kr1\ \@\[Pi])\) + \((\((kr1 - 
                          kr2)\)\ \((kl1\^2 + 2\ kr1\ kr2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          kr2)\)\ kr2\ \@\[Pi])\))\) + 
          kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                          kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                          kr2)\)\ \@\[Pi])\) - \((\@kl2\ \((kl1\^2 + 
                          kl1\ kl2 + 
                          kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 + 
                          kl2\ \((kl2 - kr1)\) - 
                          kl1\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + kr2\^2 - 
                          kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ kr2\ \((\(-kl1\) + 
                          kr2)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                          kl1\ \((kl2 - kr1)\) + kr1\^2)\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** 
                        sigma[1] ** Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - 
                          kl1\ kr1 + 
                          kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                          kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ kr2\ \((kr1 + kr2)\)\ \@\[Pi])\))\))\) + 
    kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + kr2)\)\ delta[\(-kl1\) - 
                    kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                    kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 + 
                    kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                    kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                    kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                    kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                    kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                    kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
    kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                    kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((kl1 + kl2 + 
                    kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) - \((\((kl1 - kr1 + 
                    kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)
*)
(*
These are the 1->1 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 2 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(muf\^2\ \((\(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** \
b[kr1]\)\/\(4\ kl1\ kr1\) - \(\((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\))\)\)
*)
(*
These are the 2->1 particle interactions
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(muf\ \((kappas\ \((\((\((kr1\ kr2\ \((kr1 + kr2)\) - 
                        kl1\^2\ \((2\ kr1 + kr2)\) + 
                        kl1\ \((\(-2\)\ kr1\^2 + 
                              kr2\^2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                      b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                        kr2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@\[Pi])\) + \((\((kl1 + 
                        kr2)\)\ \((\(-kr1\)\ \((kr1 + kr2)\) + 
                        kl1\ \((kr1 + 2\ kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ d[kr2] ** a[kr1, s1] ** 
                      Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                        kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                        kr2)\)\ \@\[Pi])\) + \((\((kr1 - kr2)\)\ \((kl1\^2 + 
                        2\ kr1\ kr2 - 
                        kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                      b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                        kr2)\)\ kr2\ \@\[Pi])\))\) + 
        kappaa\ \((\((\@kr2\ \((kl1\^2 - kl1\ kr2 + 
                        kr1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                        kr2)\)\ \@\[Pi])\) + \((\@kl1\ \((kr1\^2 + kr2\^2 - 
                        kl1\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                      Conjugate[a[kl1, s1]] ** 
                      b[kr1])\)/\((8\ kr1\ \((\(-kl1\) + 
                        kr1)\)\ kr2\ \((\(-kl1\) + 
                        kr2)\)\ \@\[Pi])\) + \((\@kr1\ \((kl1\^2 - kl1\ kr1 + 
                        kr2\ \((kr1 + kr2)\))\)\ delta[\(-kl1\) + kr1 + 
                        kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                      a[kr1, s1] ** 
                      Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                        kr1)\)\ kr2\ \((kr1 + kr2)\)\ \@\[Pi])\))\))\)\)
*)
(*
These are the 1->2 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(muf\ \((kappas\ \((\(-\(\((\((kl1\^2\ \((kl2 - 2\ kr1)\) + 
                          kl2\ \((kl2 - kr1)\)\ kr1 + 
                          kl1\ \((kl2\^2 - 2\ kr1\^2)\))\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\((kl2 + 
                        kr1)\)\ \((kl1\^2 + kl1\ \((kl2 - kr1)\) - 
                        2\ kl2\ kr1)\)\ delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** 
                      Conjugate[a[kl1, s1]] ** 
                      Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                        kl2)\)\ \((kl1 - 
                        kr1)\)\ kr1\ \@\[Pi])\) + \((\((kl1\^2\ \((2\ kl2 - 
                              kr1)\) + kl2\ \((kl2 - kr1)\)\ kr1 + 
                        kl1\ \((\(-2\)\ kl2\^2 + 
                              kr1\^2)\))\)\ delta[\(-kl1\) - kl2 + 
                        kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                      Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                        kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \@\[Pi])\))\) + 
        kappaa\ \((\(-\(\((\@kl2\ \((kl1\^2 + kl1\ kl2 + 
                          kr1\ \((\(-kl2\) + kr1)\))\)\ delta[\(-kl1\) - 
                          kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \@\[Pi])\)\)\) + \((\@kr1\ \((kl1\^2 + 
                        kl2\ \((kl2 - kr1)\) - kl1\ kr1)\)\ delta[\(-kl1\) - 
                        kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                      sigma[1] ** a[kr1, s1] ** 
                      Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                        kr1)\)\ \((kl2 - 
                        kr1)\)\ \@\[Pi])\) - \((\@kl1\ \((kl2\^2 + 
                        kl1\ \((kl2 - kr1)\) + kr1\^2)\)\ delta[\(-kl1\) - 
                        kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** 
                      sigma[1] ** Conjugate[a[kl1, s1]] ** 
                      Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                        kl2)\)\ \((kl1 - kr1)\)\ kr1\ \@\[Pi])\))\))\)\)
*)
(*
These are the 3->1 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(kappas\^2\ \((\(-\(\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                      kr2 + kr3]\ Conjugate[b[kl1]] ** a[kr3, s1] ** 
                    a[kr2, s2] ** 
                    b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                      kr2)\)\ \@kr3\ \((\(-kl1\) + 
                      kr3)\)\ \[Pi])\)\)\) + \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) - \((\((kl1 - 
                    kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                    kr3] ** sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\))\) + 
    kappaa\ kappas\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                    kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) - \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\) + \((\((kl1 - 
                    kr1 + kr2 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
                    kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\) + 
    kappaa\^2\ \((\((\((kl1 + kr1 + kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                    kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                  a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi])\) + \((\((kl1 + kr1 + kr2 - 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi])\) - \((\((kl1 - kr1 + kr2 + 
                    kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi])\))\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(kappas\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                    kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + 
                    kl2 - kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                    kr2]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\))\) + 
    kappaa\ kappas\ \((\((\((\(-kl1\) + kl2 - kr1 + kr2)\)\ delta[\(-kl1\) - 
                    kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** a[kr2, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((\(-kl1\) + kl2 - kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 + 
                    kl2 + kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + 
                    kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\) + 
    kappaa\^2\ \((\((\((kl1 - kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + 
                    kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \((kl2 - 
                    kr1)\)\ \@kr1\ \((kl1 - 
                    kr2)\)\ \@kr2\ \[Pi])\) + \((\((kl1 - kl2 + kr1 - 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi])\) - \((\((kl1 + kl2 + kr1 + 
                    kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + kr2)\)\ \[Pi])\) + \((\((kl1 - 
                    kl2 + kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - kr2)\)\ \[Pi])\))\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(kappas\^2\ \((\(-\(\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                      kl3 + kr1]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\)\)\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + 
                    kl2 + kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
                    kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + kr1)\)\ \[Pi])\))\) + 
    kappaa\ kappas\ \((\(-\(\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - 
                      kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                      kr1)\)\ \[Pi])\)\)\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - 
                    kl3 - kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\) + 
    kappaa\^2\ \((\((\((kl1 + kl2 - kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                    kl3 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi])\) - \((\((\(-kl1\) + kl2 + kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 + kl2 - kl3 + 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi])\) + \((\((kl1 - kl2 - kl3 - 
                    kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - kr1)\)\ \[Pi])\))\)\)
*)
(*
Find gauge terms
*)
(*
jplus = Expand[
      I m[x, rr] ** D[m[x, -rr] /. ka -> kaa, x] - 
              I D[m[x, rr], 
                    x] ** (m[x, -rr] /. ka -> kaa) - (Conjugate[
                      u[x]] /. {Conjugate[x] -> x, 
                      Conjugate[kf] -> kf}) ** (u[x] /. kf -> kff) //. 
            prodexpand /. 
          a[b_, -lambda_] ** Conjugate[a[c_, -lambda_]] -> 
            a[b, lambda] ** Conjugate[a[c, lambda]] //. prodexpand] //. 
    prodfactor
*)
(*
\!\(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka + kaa)\)\ x\)\ \((\(-ka\) \
+ kaa)\)\ a[ka, \(-rr\)] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((ka - kaa)\)\ x\)\ \((ka + kaa)\)\ \
a[ka, rr] ** Conjugate[a[kaa, rr]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((ka - kaa)\)\ x\)\ \((ka + kaa)\)\ \
Conjugate[a[ka, rr]] ** a[kaa, rr]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((ka + kaa)\)\ x\)\ \((ka - kaa)\)\ \
Conjugate[a[ka, rr]] ** Conjugate[a[kaa, \(-rr\)]]\)\/\(4\ \@ka\ \@kaa\ \[Pi]\
\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x - \[ImaginaryI]\ kff\ x\)\ \
Conjugate[b[kf]] ** b[kff]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kf\ x + \[ImaginaryI]\ kff\ x\)\ d[kf] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x \
+ \[ImaginaryI]\ kff\ x\)\ Conjugate[b[kf]] ** sigma[1] ** \
Conjugate[d[kff]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kf\ x - \[ImaginaryI]\ kff\ x\)\ d[kf] ** sigma[1] ** b[kff]\)\/\(2\ \
\[Pi]\)\)
*)
(*
I don't use Expand[...] because it makes too many terms.
*)
(*
expr3 = g^2/
          4 relabel[(jplus /. {ka -> la, kaa -> laa, rr -> ll, kf -> lf, 
                    kff -> lff}) ** (I Integrate[I Integrate[jplus, x], 
                    x]) //. prodexpand] //. prodfactor;
*)
(*
For the gauge interactions, the evaluation of the color indices is implicit \
and obeys the following rules:
If there are four adjoint fields, the color indices are evaluated as traces:
\!\(TraditionalForm\`\(a\_1\) \(a\_2\) \(a\_3\) a\_4\  -> \ 
    Tr {\(a\_1\) \(a\_2\) \(a\_3\) a\_4} - 
      Tr {\(a\_1\) a\_2}\ Tr {\(a\_3\) a\_4}/N\)
If there are two adjoint fields and two fundamental fields, the color indices \
are evaluated as:
\!\(TraditionalForm\`\(a\_1\) a\_2\ b\_1\ b\_2\  -> \ 
    b\_1\[CenterDot]a\_1\[CenterDot]a\_2\[CenterDot]b\_2\  - 
      Tr {\(a\_1\) a\_2}\ \ b\_1\[CenterDot]b\_2/N\)
If there are four fundamental fields, the color indices are evaluated as:
\!\(TraditionalForm\`\(\(\ \)\(b\ c\ d\ e\  -> \ 
    b\_i\ c\_j\ d\_k\ e\_l\ \((\[Delta]\_il\ \[Delta]\_kj\  - \ \[Delta]\_ij\ \
\[Delta]\_kl/N)\)\)\)\)
*)
(*
expr4 = Distribute[
      int[expr3, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. delta[_?neverzero] -> 0
*)
(*
\!\(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl1 - kl2)\)\ \((kr1 - 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, \(-s1\)] ** 
            a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
            Conjugate[
              a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
            Conjugate[
              a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\
\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi])\) + \
\((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** d[kr1] ** 
            Conjugate[
              d[kl2]])\)/\((16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \[Pi])\
\) - \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
              kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
            Conjugate[
              a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\@kr2\ \@kr3\ \[Pi])\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
              kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
            a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
              kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** 
            b[kr1])\)/\((16\ \((kl1 - kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi])\) - \
\((g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
            a[kr2, s1] ** d[kr1] ** 
            Conjugate[
              d[kl1]])\)/\((16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi])\
\) + \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + 
              kr2 + kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** 
            a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - 
              kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, s1] ** 
            Conjugate[a[kl2, s2]] ** 
            Conjugate[
              a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\
\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** a[kr2, s1] ** a[kr1, s2] ** 
            Conjugate[
              a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\
\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi])\) - \
\((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** a[kr2, s1] ** d[kr1] ** 
            Conjugate[
              d[kl2]])\)/\((16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \[Pi])\
\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + 
              kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** 
            a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + kr1)\)\ delta[\(-kl1\) - 
              kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
            Conjugate[
              a[kl3, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) + \
kr1)\)\^2\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \[Pi]\)\) \((g\
\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** Conjugate[b[kl3]] ** 
            b[kr1])\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
              kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
            Conjugate[
              d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \[Pi])\
\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - 
              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            b[kr1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, \(-s1\)]])\) + \((g\^2\ \((kl2 + 
              kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
            b[kr2] ** a[kr1, s1] ** 
            Conjugate[
              a[kl2, s1]])\)/\((16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[a[kl2, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) - kl2 + 
            kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[b[kl2]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** d[kr1] ** 
          Conjugate[
            d[kl2]])\)\) + \((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + 
              kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** 
            a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi])\) - \
\(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - 
              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** 
            Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, \(-s1\)]])\) + \((g\^2\ \((kl2 + 
              kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
            Conjugate[d[kl1]] ** a[kr1, s1] ** 
            Conjugate[
              a[kl2, s1]])\)/\((16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
              kr2] ** Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) - kl2 + 
            kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** d[kr1] ** 
          Conjugate[
            d[kl2]])\)\) + \((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + 
              kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** 
            a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi])\) + \
\(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ a[kr1, s1] ** 
            Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** sigma[1] ** 
            Conjugate[
              d[kl3]])\) - \(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \[Pi]\
\)\) \((g\^2\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[
              kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
            Conjugate[d[kl2]])\) + \((g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + 
              kr1 + kr2 + kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** 
            d[kr2] ** sigma[1] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi])\) - \(1\
\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** a[kr1, s1] ** Conjugate[b[kl2]] ** sigma[1] ** 
            Conjugate[d[kl3]])\) - \((g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + 
              kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** a[kr3, s1] ** 
            d[kr2] ** sigma[1] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi])\) - \(1\
\/\(16\ \@kl1\ \@kl2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl1 - 
              kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** d[kr2] ** 
            sigma[1] ** 
            b[kr1])\) + \(\(1\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** b[kr1] ** 
          Conjugate[b[kl2]] ** sigma[1] ** 
          Conjugate[
            d[kl3]])\)\) + \(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** 
          sigma[1] ** 
          b[kr1])\)\) + \((g\^2\ \((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
              kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** a\
[kr1, s1] ** Conjugate[
              a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) + \(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi]\)\) \((g\^2\ \
\((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
            sigma[1] ** Conjugate[d[kl2]] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\) - \(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \
\[Pi]\)\) \((g\^2\ \((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\) + \(\(1\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** d[kr1] ** 
          Conjugate[
            d[kl3]])\)\) + \(\(1\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** 
          Conjugate[b[kl2]] ** sigma[1] ** 
          Conjugate[
            d[kl3]])\)\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \[Pi]\
\)\) \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
            sigma[1] ** b[kr1] ** Conjugate[a[kl1, s1]] ** 
            Conjugate[
              a[kl2, \(-s1\)]])\) + \(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\
\((g\^2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** 
          d[kr2] ** sigma[1] ** 
          b[kr1])\)\) + \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + 
              kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] ** 
            Conjugate[
              a[kl1, s1]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
              kr3] ** sigma[1] ** b[kr2] ** Conjugate[a[kl1, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl1 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) + kr1 + 
            kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** Conjugate[b[kl1]] ** 
          b[
            kr1])\)\) + \(\(1\/\(8\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** d[kr1] ** 
          Conjugate[
            d[kl1]])\)\) + \(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((kl1 + kl2)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
          Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]])\)\)\)
*)
(*
These are the 3->1 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\(-\(\((g\^2\ \((kl1 + kr1)\)\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + 
                kr2 + kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** a[kr1, s2] ** 
              Conjugate[
                a[kl1, s2]])\)/\((32\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\@kr2\ \@kr3\ \[Pi])\)\)\) + \((g\^2\ \((kl1 + kr1)\)\ \((kr2 - 
              kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
            a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \@kr2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + 
              kr3]\ a[kr3, \(-s1\)] ** a[kr2, s1] ** Conjugate[b[kl1]] ** 
            b[kr1])\)/\((16\ \((kl1 - kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi])\) - \
\((g\^2\ \((kr2 - kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, \(-s1\)] ** 
            a[kr2, s1] ** d[kr1] ** 
            Conjugate[
              d[kl1]])\)/\((16\ \((\(-kl1\) + kr1)\)\^2\ \@kr2\ \@kr3\ \[Pi])\
\) + \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + 
              kr2 + kr3]\ a[kr3, s1] ** Conjugate[a[kl1, s1]] ** 
            a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) - \((g\^2\ \((\(-kr1\) + kr2)\)\ \((kl1 + 
              kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, s1]] ** a[kr3, s1] ** a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\@kr3\ \[Pi])\) + \((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + kr1 + kr2 + 
              kr3]\ Conjugate[b[kl1]] ** b[kr3] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi])\) + \
\((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** 
            Conjugate[d[kl1]] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \[Pi])\) + \
\((g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ a[kr3, s1] ** 
            Conjugate[a[kl1, s1]] ** d[kr2] ** sigma[1] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi])\) - \
\((g\^2\ \((kl1 + kr3)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, s1]] ** a[kr3, s1] ** d[kr2] ** sigma[1] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kr1 + kr2)\)\^2\ \@kr3\ \[Pi])\) + \
\(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) + kr1 + 
            kr2 + kr3]\ Conjugate[b[kl1]] ** b[kr3] ** d[kr2] ** sigma[1] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** Conjugate[d[kl1]] ** d[kr2] ** 
          sigma[1] ** 
          b[kr1])\)\) + \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + 
              kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** a[kr1, s1] ** 
            Conjugate[
              a[kl1, s1]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl1\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[
              kr3] ** sigma[1] ** b[kr2] ** Conjugate[a[kl1, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl1 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) + kr1 + 
            kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** Conjugate[b[kl1]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl1\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** b[kr2] ** d[kr1] ** 
          Conjugate[d[kl1]])\)\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\((g\^2\ \((kl1 - kl2)\)\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ a[kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
            Conjugate[
              a[kl2, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\@kr1\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
            Conjugate[
              a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\
\^2\ \@kr2\ \[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) + \((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ a[kr2, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi])\) + \
\((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[kr2, s1] ** 
            Conjugate[a[kl1, s1]] ** d[kr1] ** 
            Conjugate[
              d[kl2]])\)/\((16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \[Pi])\
\) - \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** a[kr1, s2] ** 
            Conjugate[
              a[kl2, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\
\^2\ \@kr2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ \((kl1 + 
              kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \
\@kr2\ \[Pi])\) - \((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, s1]] ** a[kr2, s1] ** Conjugate[b[kl2]] ** 
            b[kr1])\)/\((16\ \@kl1\ \((kl2 - kr1)\)\^2\ \@kr2\ \[Pi])\) - \
\((g\^2\ \((kl1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** a[kr2, s1] ** d[kr1] ** 
            Conjugate[
              d[kl2]])\)/\((16\ \@kl1\ \((\(-kl2\) + kr1)\)\^2\ \@kr2\ \[Pi])\
\) - \((g\^2\ \((kl1 - kl2)\)\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + 
              kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \((kr1 + \
kr2)\)\^2\ \[Pi])\) + \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[b[kl1]] ** b[kr2] ** a[kr1, s1] ** 
            Conjugate[
              a[kl2, s1]])\)/\((16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[a[kl2, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) - kl2 + 
            kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** Conjugate[b[kl2]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** b[kr2] ** d[kr1] ** 
          Conjugate[
            d[kl2]])\)\) + \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + 
              kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** a[kr1, s1] ** 
            Conjugate[
              a[kl2, s1]])\)/\((16\ \@kl2\ \@kr1\ \((\(-kl2\) + kr1)\)\^2\ \
\[Pi])\) - \((g\^2\ \((kl2 + kr1)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
              kr2] ** Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl2\ \((kl2 - kr1)\)\^2\ \@kr1\ \[Pi])\) + \
\(\(1\/\(8\ \((kl2 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ delta[\(-kl1\) - kl2 + 
            kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** Conjugate[b[kl2]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl2\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** Conjugate[d[kl1]] ** d[kr1] ** 
          Conjugate[
            d[kl2]])\)\) - \(1\/\(16\ \((kl1 + kl2)\)\^2\ \@kr1\ \@kr2\ \[Pi]\
\)\) \((g\^2\ \((kr1 - kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ a[
              kr2, \(-s1\)] ** a[kr1, s1] ** Conjugate[b[kl1]] ** sigma[1] ** 
            Conjugate[
              d[kl2]])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kr1 + kr2)\)\^2\ \[Pi]\
\)\) \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** d[kr2] ** 
            sigma[1] ** 
            b[kr1])\) + \(1\/\(16\ \@kr1\ \@kr2\ \((kr1 + kr2)\)\^2\ \
\[Pi]\)\) \((g\^2\ \((\(-kr1\) + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
            a[kr2, \(-s1\)] ** 
            a[kr1, s1])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl1 + kl2)\)\^2\ \
\[Pi]\)\) \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[
              kr2] ** sigma[1] ** b[kr1] ** Conjugate[a[kl1, s1]] ** 
            Conjugate[
              a[kl2, \(-s1\)]])\) + \(\(1\/\(8\ \((kr1 + kr2)\)\^2\ \[Pi]\)\)\
\((g\^2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((kl1 + kl2)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
          Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]])\)\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
expr4 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(-\(\((g\^2\ \((kl2 - kl3)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - 
                kl3 + kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** 
              Conjugate[a[kl2, s2]] ** 
              Conjugate[
                a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\
\)\^2\ \@kr1\ \[Pi])\)\)\) + \((g\^2\ \((kl2 - kl3)\)\ \((kl1 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
            Conjugate[
              a[kl3, \(-s2\)]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\
\^2\ \@kr1\ \[Pi])\) - \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** a[kr1, s2] ** 
            Conjugate[
              a[kl3, s2]])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \((\(-kl3\) + \
kr1)\)\^2\ \[Pi])\) + \((g\^2\ \((kl1 - kl2)\)\ \((kl3 + 
              kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s2]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kl3\ \((kl3 - kr1)\)\^2\ \
\@kr1\ \[Pi])\) - \(1\/\(16\ \@kl1\ \@kl2\ \((kl3 - kr1)\)\^2\ \[Pi]\)\) \((g\
\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** Conjugate[b[kl3]] ** 
            b[kr1])\) - \((g\^2\ \((kl1 - kl2)\)\ delta[\(-kl1\) - kl2 - 
              kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** d[kr1] ** 
            Conjugate[
              d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((\(-kl3\) + kr1)\)\^2\ \[Pi])\
\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - 
              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            b[kr1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\) - \(1\/\(16\ \@kl2\ \@kl3\ \((kl2 + \
kl3)\)\^2\ \[Pi]\)\) \((g\^2\ \((kl2 - kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ d[kr1] ** Conjugate[d[kl1]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[
              a[kl3, \(-s1\)]])\) + \(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \
\@kr1\ \[Pi]\)\) \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + 
              kr1]\ a[kr1, s1] ** Conjugate[a[kl1, s1]] ** Conjugate[b[kl2]] ** 
            sigma[1] ** 
            Conjugate[
              d[kl3]])\) - \(1\/\(16\ \@kl1\ \((kl2 + kl3)\)\^2\ \@kr1\ \[Pi]\
\)\) \((g\^2\ \((kl1 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** a[kr1, s1] ** Conjugate[b[kl2]] ** sigma[1] ** 
            Conjugate[
              d[kl3]])\) + \(\(1\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** b[kr1] ** 
          Conjugate[b[kl2]] ** sigma[1] ** 
          Conjugate[
            d[kl3]])\)\) + \((g\^2\ \((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - 
              kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
            a[kr1, s1] ** 
            Conjugate[
              a[kl3, s1]])\)/\((16\ \@kl3\ \@kr1\ \((\(-kl3\) + kr1)\)\^2\ \
\[Pi])\) - \(1\/\(16\ \@kl3\ \((kl3 - kr1)\)\^2\ \@kr1\ \[Pi]\)\) \((g\^2\ \
\((kl3 + kr1)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** 
            sigma[1] ** Conjugate[d[kl2]] ** Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\) + \(\(1\/\(8\ \((kl3 - kr1)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
          b[kr1])\)\) + \(\(1\/\(8\ \((\(-kl3\) + kr1)\)\^2\ \[Pi]\)\)\((g\^2\
\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
          Conjugate[d[kl2]] ** d[kr1] ** 
          Conjugate[
            d[kl3]])\)\) + \(\(1\/\(8\ \((kl2 + kl3)\)\^2\ \[Pi]\)\)\((g\^2\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ d[kr1] ** Conjugate[d[kl1]] ** 
          Conjugate[b[kl2]] ** sigma[1] ** Conjugate[d[kl3]])\)\)\)
*)
(*
Gluon effective potential
*)
(*
I only include the terms that act on meson strings in the large N limit.
*)
(*
ka /: Positive[ka] = True; kb /: Positive[kb] = True; kc /: 
  Positive[kc] = True; kd /: Positive[kd] = True;
*)
(*
The \!\(TraditionalForm\`\[Micro]\_B\) term
*)
(*
The factor of 1/2 is from the fact that lambda is summing over four values
*)
(*
term = mub^2/2 m[x, s1] ** (m[x, s1] /. ka -> kb);
*)
(*
expr = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] /. 
        aa_a ** Conjugate[b_] -> Conjugate[b] ** aa];
*)
(*
expr2 = Distribute[
      int[expr, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. delta[_?neverzero] -> 0
*)
(*
\!\(\(mub\^2\ delta[\(-kl1\) + kr1]\ Conjugate[a[kl1, s1]] ** a[kr1, \(-s1\)]\
\)\/\(2\ \@kl1\ \@kr1\)\)
*)
(*
The box term \[Beta]
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.  There is an overall factor of \
1/4 since there are no sums in the original expression.
*)
(*
term = m[x, sa] ** (m[x, sb] /. ka -> kb) ** Conjugate[m[x, sa] /. ka -> kc] **
       Conjugate[m[x, -sb] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[-beta/4 (term + Conjugate[term]) //. 
              prodexpand /. {Conjugate[x] -> x, Conjugate[kk_?Positive] -> kk,
               Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\(-\(\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
                a[kl1, \(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, \(-s1\)] ** 
              a[kr1, \(-s2\)])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \
\[Pi])\)\)\) - \((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, \(-s1\)]] ** a[kr3, s2] ** a[kr2, \(-s1\)] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, \(-s2\)] ** a[kr2, s1] ** 
            a[kr1, \(-s2\)])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, s2] ** a[kr2, s1] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** 
            a[kr3, \(-s1\)] ** a[kr2, s2] ** 
            a[kr1, s1])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** 
            a[kr3, s1] ** a[kr2, s2] ** 
            a[kr1, \(-s1\)])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** 
            a[kr3, \(-s1\)] ** a[kr2, \(-s2\)] ** 
            a[kr1, s1])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** 
            a[kr3, s1] ** a[kr2, \(-s2\)] ** 
            a[kr1, \(-s1\)])\)/\((32\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\(-\(\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                a[kl1, \(-s1\)]] ** Conjugate[a[kl2, \(-s2\)]] ** 
              a[kr2, \(-s1\)] ** 
              a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, s2]] ** a[kr2, \(-s1\)] ** 
            a[kr1, \(-s2\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, \(-s2\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, s2] ** Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s2\)]] ** a[kr2, s1] ** 
            a[kr1, s2])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, s2]] ** a[kr2, s1] ** 
            a[kr1, \(-s2\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** 
            a[kr2, \(-s1\)] ** Conjugate[a[kl2, \(-s2\)]] ** 
            a[kr1, s1])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s2] ** 
            a[kr1, \(-s1\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
            a[kr1, s1])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** 
            a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, \(-s1\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
            a[kr1, \(-s1\)])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) - \
\((beta\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, s1]] ** a[kr2, \(-s2\)] ** 
            a[kr1, s1])\)/\((32\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(-\(\(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((beta\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
              Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, s1]] ** 
              a[kr1, s2])\)\)\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, \(-s2\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s2\)]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, s2])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s2\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, s1])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s2]] ** 
            a[kr1, \(-s1\)])\) - \(1\/\(32\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((beta\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** a[kr1, s1])\)\)
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_1\) term
*)
(*
There is an implicit sum over the whole expression for s1={1,2,-1,-2}.  The \
factor of 1/2 is because s1 takes on all four values
*)
(*
term = lambda1/2 m[x, sa] ** 
        Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sa] /. ka -> kc) ** 
        Conjugate[m[x, sa] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\((lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, \(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, s1] ** 
            a[kr1, \(-s1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda1\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, s1] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda1\ delta[\(-kl1\
\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** a[kr2, \(-s1\)] ** 
            Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\) + \((lambda1\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** 
            a[kr2, s1] ** 
            a[kr1, \(-s1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda1\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, s1] ** Conjugate[a[kl2, s1]] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda1\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda1\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda1\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_2\) term
*)
(*
There is an implicit sum over the whole expression for s1={1,2,-1,-2}.  The \
factor of 1/2 is because s1 takes on all four values
*)
(*
term = lambda2/2 m[x, sa] ** (m[x, sa] /. ka -> kb) ** 
        Conjugate[m[x, sa] /. ka -> kc] ** Conjugate[m[x, sa] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** b_ :> 
            c b ** aa /; MemberQ[b, Conjugate[_]], 
          c_ aa_ ** b_Conjugate :> c b ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\((lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, \(-s1\)]] ** a[kr3, \(-s1\)] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s1\)]] ** 
            a[kr3, s1] ** a[kr2, \(-s1\)] ** 
            a[kr1, \(-s1\)])\)/\((16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, \(-s1\)] ** a[kr2, s1] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, s1] ** a[kr2, s1] ** 
            a[kr1, \(-s1\)])\)/\((16\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\(1\/\(16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda2\ delta[\(-kl1\
\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s1\)] ** 
            a[kr1, \(-s1\)])\) + \((lambda2\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** 
            a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, \(-s1\)] ** Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, s1] ** Conjugate[a[kl2, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, s1] ** 
            a[kr1, \(-s1\)])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda2\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, s1]] ** a[kr2, s1] ** 
            a[kr1, s1])\)/\((16\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda2\ delta[\(-kl1\
\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, s1]] ** 
            a[kr1, \(-s1\)])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s1]] ** 
            a[kr1, s1])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, \(-s1\)])\) + \(1\/\(16\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\
\)\) \((lambda2\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[kr1, s1])\)\)
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_4\) term
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.
*)
(*
term = lambda4 m[x, sa] ** 
        Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sb] /. ka -> kc) ** 
        Conjugate[m[x, sb] /. ka -> kd];
*)
(*
exprb = relabel[
      Expand[term //. prodexpand /. {Conjugate[x] -> x, 
              Conjugate[kk_?Positive] -> kk, 
              Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** bb_ :> 
            c bb ** aa /; MemberQ[bb, Conjugate[_]], 
          c_ aa_ ** bb_Conjugate :> c bb ** aa /; MemberQ[aa, a[__]]}];
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[
              a[kl1, \(-s1\)]] ** a[kr3, \(-s2\)] ** a[kr2, s2] ** 
            a[kr1, \(-s1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s1]] ** 
            a[kr3, s1] ** a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-s2\)]] ** 
            a[kr3, \(-s1\)] ** a[kr2, s1] ** 
            a[kr1, \(-s2\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, s2]] ** 
            a[kr3, s2] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\(1\/\(8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s1\)]] ** 
            a[kr2, \(-s2\)] ** Conjugate[a[kl2, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\) + \((lambda4\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ Conjugate[a[kl1, \(-s1\)]] ** Conjugate[a[kl2, s2]] ** 
            a[kr2, s2] ** 
            a[kr1, \(-s1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
            a[kr1, s2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s1]] ** 
            Conjugate[a[kl2, \(-s1\)]] ** a[kr2, \(-s2\)] ** 
            a[kr1, s2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, s1]] ** a[kr2, s1] ** 
            a[kr1, \(-s2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, s2]] ** 
            Conjugate[a[kl2, \(-s2\)]] ** a[kr2, \(-s1\)] ** 
            a[kr1, s1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s1\)]] ** 
            Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, \(-s2\)]] ** 
            a[kr1, \(-s1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s1]] ** Conjugate[a[kl2, \(-s1\)]] ** 
            Conjugate[a[kl3, s2]] ** 
            a[kr1, s2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-s2\)]] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, \(-s1\)]] ** 
            a[
              kr1, \(-s2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \
\[Pi]\)\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[
              a[kl1, s2]] ** Conjugate[a[kl2, \(-s2\)]] ** 
            Conjugate[a[kl3, s1]] ** a[kr1, s1])\)\)
*)
(*
The \!\(TraditionalForm\`\[Lambda]\_4\) term (done more explicitly)
*)
(*
The implicit sums are s1={-1,1} and s2={-2,2}.
*)
(*
term = lambda4 Sum[
        m[x, sa] ** 
          Conjugate[m[x, sa] /. ka -> kb] ** (m[x, sb] /. ka -> kc) ** 
          Conjugate[m[x, sb] /. ka -> kd], {sa, -1, 1, 2}, {sb, -2, 2, 4}];
*)
(*
exprb = relabel[
        Expand[term //. prodexpand /. {Conjugate[x] -> x, 
                Conjugate[kk_?Positive] -> kk, 
                Conjugate[kk_?Positive^n_] -> kk^n}] //. {c_ aa_a ** bb_ :> 
              c bb ** aa /; MemberQ[bb, Conjugate[_]], 
            c_ aa_ ** bb_Conjugate :> c bb ** aa /; MemberQ[aa, a[__]]}] //. 
      prodfactor;
*)
(*
exprbb = Distribute[
        int[exprb, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]] /. 
      delta[_?neverzero] -> 0;
*)
(*
These are the 3->1 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** 
            a[kr3, \(-2\)] ** a[kr2, \(-1\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** 
            a[kr3, \(-2\)] ** a[kr2, 1] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** 
            a[kr3, \(-1\)] ** a[kr2, 1] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-2\)]] ** 
            a[kr3, 1] ** a[kr2, \(-1\)] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr3, \(-2\)] ** a[kr2, 2] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr3, \(-1\)] ** a[kr2, \(-2\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr3, \(-1\)] ** a[kr2, 2] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr3, 2] ** a[kr2, \(-2\)] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** 
            a[kr3, \(-2\)] ** a[kr2, 2] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** 
            a[kr3, 1] ** a[kr2, \(-2\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** 
            a[kr3, 1] ** a[kr2, 2] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 1]] ** 
            a[kr3, 2] ** a[kr2, \(-2\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** 
            a[kr3, \(-1\)] ** a[kr2, 1] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** 
            a[kr3, 1] ** a[kr2, \(-1\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** 
            a[kr3, 2] ** a[kr2, \(-1\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) + kr1 + kr2 + kr3]\ Conjugate[a[kl1, 2]] ** 
            a[kr3, 2] ** a[kr2, 1] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kr1\ \@kr2\ \@kr3\ \[Pi])\)\)
*)
(*
These are the 2->2 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-1\)] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 1]] ** a[kr2, 1] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 2]] ** a[kr2, \(-1\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 2]] ** a[kr2, 1] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr2, \(-2\)] ** Conjugate[a[kl2, \(-2\)]] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr2, \(-1\)] ** Conjugate[a[kl2, \(-2\)]] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr2, \(-1\)] ** Conjugate[a[kl2, 2]] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            a[kr2, 2] ** Conjugate[a[kl2, 2]] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-2\)] ** 
            a[
              kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 1]] ** a[kr2, \(-2\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 1]] ** a[kr2, 2] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 2]] ** a[kr2, 2] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            a[kr2, \(-2\)] ** Conjugate[a[kl2, \(-2\)]] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            a[kr2, 1] ** Conjugate[a[kl2, \(-2\)]] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            a[kr2, 1] ** Conjugate[a[kl2, 2]] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            a[kr2, 2] ** Conjugate[a[kl2, 2]] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-2\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-2\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** a[kr2, 2] ** 
            a[kr1, \(-2\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, 2]] ** a[kr2, 2] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** a[kr2, \(-1\)] ** 
            a[kr1, 1])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** a[kr2, 1] ** 
            a[kr1, \(-1\)])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-1\)]] ** a[kr2, \(-1\)] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\) + \
\((lambda4\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, 1]] ** a[kr2, 1] ** 
            a[kr1, 2])\)/\((8\ \@kl1\ \@kl2\ \@kr1\ \@kr2\ \[Pi])\)\)
*)
(*
These are the 1->3 particle interactions:
*)
(*
exprbb /. 
  a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 4 || Count[non, Conjugate[_]] != 3]
*)
(*
\!\(\(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \((lambda4\ \
delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-2\)]] ** 
            Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, \(-1\)]] ** 
            Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, \(-2\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 2]] ** 
            a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 1]] ** 
            Conjugate[a[kl2, 2]] ** Conjugate[a[kl3, \(-2\)]] ** 
            a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, \(-1\)])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\
\) \((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-2\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, 1])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, \(-1\)]] ** Conjugate[a[kl3, 1]] ** 
            a[kr1, 2])\) + \(1\/\(8\ \@kl1\ \@kl2\ \@kl3\ \@kr1\ \[Pi]\)\) \
\((lambda4\ delta[\(-kl1\) - kl2 - kl3 + kr1]\ Conjugate[a[kl1, 2]] ** 
            Conjugate[a[kl2, 1]] ** Conjugate[a[kl3, \(-1\)]] ** 
            a[kr1, 2])\)\)
*)
(*
Find pseudoscalar 4-fermion interaction to leading order, acts on pion
*)
(*
The chiral four-fermion interaction in the Hamiltonian is kappa5 qq**qq/2 \
where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** sigma[2] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** sigma[2] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 \
+ kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\
\ kl2\ \[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \
\@2\ kl1\ kr1\ \[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - \
kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                    kl2 - kr1)\)\ d[kr1] ** sigma[3] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[
                    kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    2\ kr1 + kr2)\)\ d[kr2] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] \
** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[3] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kl1 + kr1 + 
                    kr2)\)\ d[kr2] ** sigma[2] ** zeta[s1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[
                    kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 \
+ kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\
\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \
\[Pi]\))\) + 
    kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + \
2\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \
\@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    sigma[2] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 \
+ 2\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    Collect[relabel[
          Map[Distribute, 
                qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                    kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
              Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
          kappas}] //. prodfactor;
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  kappa5/2*simpint[
      int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
       k11]
*)
(*
\!\(1\/2\ kappa5\ \((muf\^2\ \((\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                      b[kr1])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\
\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[d[kl2]])\))\) + 
        kappas\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                      a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
        muf\ \((kappas\ \((\(-\(\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** sigma[2] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \((\
\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                            Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                            a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                              2\ kl2 + kl3)\)\ \((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ d[kr2] ** sigma[2] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** sigma[2] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                            b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
              kappaa\ \((\(-\(\((\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + 
                                kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                              zeta[s1] ** a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \((kr2 + 
                                kr3)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\((kl1 + 
                              2\ kl2 + kl3)\)\ \((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[2] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ 2\ kl2 + kl3)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ d[kr2] ** sigma[2] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** sigma[2] ** zeta[s1] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[2] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
        kappaa\ kappas\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                      a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                      sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 
                        2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** sigma[2] ** zeta[s2] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
        kappaa\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[2] ** 
                      zeta[s2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\))\)\)
*)
(*
Select terms that act on a baryon (no anti-quarks)
*)
(*
qqbaryon = qq[x] /. d[_] -> 0 /. {0 ** a_ -> 0, a_ ** 0 -> 0}
*)
(*
\!\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 - kr1)\)\ muf\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ \
kl1\ kr1\ \[Pi]\) + 
    kappas\ \((\(-\(\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** 
                    sigma[3] ** a[kr2, s1] ** 
                    b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\)\)\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
 (*put in the coupling constant later*)
  expr1baryon = 
    Collect[relabel[
          Map[Distribute, 
                qqbaryon /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                    kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
              Map[Distribute, qqbaryon] //. prodexpand], {muf, kappaa, 
          kappas}] //. prodfactor;
*)
(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the leading baryon interactions in the \
Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*) expr2baryon = 
  kappa5/2*simpint[
      int[expr1baryon, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
       k11]
*)
(*
\!\(1\/2\ kappa5\ \((\(-\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\((kl2 - 
                    kr1)\)\ \((kl1 - kr2)\)\ muf\^2\ delta[\(-kl1\) - kl2 + 
                    kr1 + kr2]\ Conjugate[b[kl1]] ** sigma[3] ** b[kr2] ** 
                  Conjugate[b[kl2]] ** sigma[3] ** b[kr1])\)\)\) + 
        kappas\^2\ \((\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                        2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr3, s2] ** 
                      b[kr2] ** Conjugate[b[kl2]] ** sigma[3] ** 
                      Conjugate[a[kl3, s1]] ** 
                      b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                        kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((\(-kl2\) + kr1 + 
                        2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                        2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr4, s2] ** 
                      b[kr3] ** Conjugate[b[kl2]] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \@kr4\ \((\(-kl1\) + kr4)\)\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 2\ kl4 - 
                        kr1)\)\ \((kl1 + 2\ kl2 - kr2)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** b[kr2] ** 
                      Conjugate[b[kl3]] ** sigma[3] ** Conjugate[a[kl4, s1]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                        kr1)\)\ \((kl2 - 
                        kr2)\)\ \[Pi]\^2)\) + \((\((\(-kl3\) + kr1 + 
                        2\ kr2)\)\ \((kl1 + 2\ kl2 - kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** b[kr3] ** 
                      Conjugate[b[kl3]] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\))\) + 
        muf\ \((kappas\ \((\((\((kl2 - kr1)\)\ \((\(-kl1\) + kr2 + 
                              2\ kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                            a[kr3, s1] ** b[kr2] ** Conjugate[b[kl2]] ** 
                            sigma[3] ** 
                            b[kr1])\)/\((8\ kl2\ kr1\ \@kr3\ \((\(-kl1\) + 
                              kr3)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl2 + 
                              2\ kl3 - kr1)\)\ \((kl1 - 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** sigma[3] ** b[kr2] ** 
                            Conjugate[b[kl2]] ** sigma[3] ** 
                            Conjugate[a[kl3, s1]] ** 
                            b[kr1])\)/\((8\ kl1\ \@kl3\ \((kl2 + 
                              kl3)\)\ \((kl3 - 
                              kr1)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\((\(-kl2\) \
+ kr1 + 2\ kr2)\)\ \((kl1 - kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** sigma[3] ** b[kr3] ** 
                            Conjugate[b[kl2]] ** sigma[3] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ \@kr2\ \((\(-kl2\) + 
                              kr2)\)\ \((kr1 + 
                              kr2)\)\ kr3\ \[Pi]\^\(3/2\))\) - \((\((kl3 - 
                              kr1)\)\ \((kl1 + 2\ kl2 - 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[a[kl2, s1]] ** b[kr2] ** 
                            Conjugate[b[kl3]] ** sigma[3] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kl3\ kr1\ \((kl2 - 
                              kr2)\)\ \[Pi]\^\(3/2\))\))\) + 
              kappaa\ \((\(-\(\((\((kl2 + 2\ kl3 - kr1)\)\ \((kl1 - 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                              b[kr2] ** Conjugate[b[kl2]] ** zeta[s1] ** 
                              sigma[3] ** Conjugate[a[kl3, s1]] ** 
                              b[kr1])\)/\((8\ kl1\ \@kl3\ \((kl2 + 
                                kl3)\)\ \((kl3 - 
                                kr1)\)\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \
\((\((\(-kl2\) + kr1 + 2\ kr2)\)\ \((kl1 - kr3)\)\ delta[\(-kl1\) - kl2 + 
                              kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                            sigma[3] ** b[kr3] ** Conjugate[b[kl2]] ** 
                            zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ \@kr2\ \((\(-kl2\) + 
                              kr2)\)\ \((kr1 + 
                              kr2)\)\ kr3\ \[Pi]\^\(3/2\))\) - \((\((kl2 - 
                              kr1)\)\ \((\(-kl1\) + kr2 + 
                              2\ kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[3] ** a[kr3, s1] ** b[kr2] ** 
                            Conjugate[b[kl2]] ** sigma[3] ** 
                            b[kr1])\)/\((8\ kl2\ kr1\ \@kr3\ \((\(-kl1\) + 
                              kr3)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl3 - 
                              kr1)\)\ \((kl1 + 2\ kl2 - 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[3] ** Conjugate[a[kl2, s1]] ** b[kr2] ** 
                            Conjugate[b[kl3]] ** sigma[3] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kl3\ kr1\ \((kl2 - 
                              kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
        kappaa\ kappas\ \((\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                        2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr3, s2] ** 
                      b[kr2] ** Conjugate[b[kl2]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl3, s1]] ** 
                      b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                        kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((\(-kl2\) + kr1 + 
                        2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                        2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** sigma[3] ** a[kr4, s2] ** 
                      b[kr3] ** Conjugate[b[kl2]] ** zeta[s1] ** sigma[3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \@kr4\ \((\(-kl1\) + kr4)\)\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 2\ kl4 - 
                        kr1)\)\ \((kl1 + 2\ kl2 - kr2)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** b[kr2] ** 
                      Conjugate[b[kl3]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl4, s1]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                        kr1)\)\ \((kl2 - 
                        kr2)\)\ \[Pi]\^2)\) - \((\((\(-kl3\) + kr1 + 
                        2\ kr2)\)\ \((kl1 + 2\ kl2 - kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** b[kr3] ** 
                      Conjugate[b[kl3]] ** zeta[s1] ** sigma[3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\) - \((\((kl2 + 
                        2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                        2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                      a[kr3, s2] ** b[kr2] ** Conjugate[b[kl2]] ** sigma[3] ** 
                      Conjugate[a[kl3, s1]] ** 
                      b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                        kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((\(-kl2\) + kr1 + 
                        2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                        2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                      a[kr4, s2] ** b[kr3] ** Conjugate[b[kl2]] ** sigma[3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \@kr4\ \((\(-kl1\) + kr4)\)\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 2\ kl4 - 
                        kr1)\)\ \((kl1 + 2\ kl2 - kr2)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                      b[kr2] ** Conjugate[b[kl3]] ** sigma[3] ** 
                      Conjugate[a[kl4, s1]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                        kr1)\)\ \((kl2 - 
                        kr2)\)\ \[Pi]\^2)\) + \((\((\(-kl3\) + kr1 + 
                        2\ kr2)\)\ \((kl1 + 2\ kl2 - kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                      b[kr3] ** Conjugate[b[kl3]] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\))\) + 
        kappaa\^2\ \((\(-\(\((\((kl2 + 2\ kl3 - kr1)\)\ \((\(-kl1\) + kr2 + 
                          2\ kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                          kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                        a[kr3, s2] ** b[kr2] ** Conjugate[b[kl2]] ** 
                        zeta[s1] ** sigma[3] ** Conjugate[a[kl3, s1]] ** 
                        b[kr1])\)/\((16\ \@kl3\ \((kl2 + kl3)\)\ \((kl3 - 
                          kr1)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^2)\)\)\) - \((\((\(-kl2\) + kr1 + 
                        2\ kr2)\)\ \((\(-kl1\) + kr3 + 
                        2\ kr4)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                      a[kr4, s2] ** b[kr3] ** Conjugate[b[kl2]] ** zeta[s1] ** 
                      sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((\(-kl2\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \@kr4\ \((\(-kl1\) + kr4)\)\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((\((kl3 + 2\ kl4 - 
                        kr1)\)\ \((kl1 + 2\ kl2 - kr2)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 - kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                      b[kr2] ** Conjugate[b[kl3]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl4, s1]] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl4\ \((kl3 + kl4)\)\ \((kl4 - 
                        kr1)\)\ \((kl2 - 
                        kr2)\)\ \[Pi]\^2)\) - \((\((\(-kl3\) + kr1 + 
                        2\ kr2)\)\ \((kl1 + 2\ kl2 - kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                      b[kr3] ** Conjugate[b[kl3]] ** zeta[s1] ** sigma[3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((\(-kl3\) + kr2)\)\ \((kr1 + 
                        kr2)\)\ \((kl2 - kr3)\)\ \[Pi]\^2)\))\))\)\)
*)
(*
Testing
*)
(*
v[x_] = I*Integrate[xi[x], x];
conjv[x_] = -I*Integrate[cxi[x], x];
*)
(*
psibar psi:  0^++
*)
(*
Collect[relabel[
      Expand[-(conju[x] ** sigma[1] ** v[x] + conjv[x] ** sigma[1] ** u[x])/
              Sqrt[2] //. prodexpand //. sigmazeta]], {muf, kappas, 
      kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 \
+ kr1)\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
d[kr1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((\(-kl1\) + kl2)\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((\(-kr1\) + kr2)\)\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\
\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** a[kr2, s1] ** 
                    b[kr1]\)\/\(8\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\)\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr2)\)\ d[kr2] ** a[kr1, \
s1] ** Conjugate[d[kl1]]\)\/\(8\ \((kl1 - kr1)\)\ \@kr1\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\
\ x\)\ \((kl1 - kl2)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \((kl1 - kr1)\)\ \((kl2 - 
                    kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 - kl3)\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ d[kr2] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 - kr3)\)\ d[kr3] ** \
sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + \
kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    a[kr2, s1] ** 
                    b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - 
                    kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 
                    2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((\(-2\)\ kl1 + kr1 + 
                    kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[
                    kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^1 psi:  1^--
*)
(*
Collect[relabel[
      Expand[(conju[x] ** sigma[3] ** v[x] + conjv[x] ** sigma[3] ** u[x])/
              Sqrt[2] //. prodexpand //. sigmazeta]], {muf, kappas, 
      kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\)\)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 + kl2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) \
+ \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 - kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \
\[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \
\((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 
                    2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                    kl2 - kr1)\)\ d[kr1] ** sigma[2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    2\ kr1 + kr2)\)\ d[kr2] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[
                    kr2] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] \
** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + \
kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + \
kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** zeta[s1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - \
kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \(1\/\(8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ \
x\)\ d[kr2] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** 
                  zeta[s1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \
\[Pi]\^\(3/2\)\) + \(1\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\) \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\) + \(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\) \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^2 psi:  1^+-
*)
(*
Collect[relabel[
      Expand[-I (conju[x] ** v[x] - conjv[x] ** u[x])/Sqrt[2] //. 
            prodexpand //. sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x\)\ \((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                    Conjugate[
                      d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) + \(\[ImaginaryI]\
\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + \
kr2)\)\ d[kr2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 \
- kr1)\)\ d[kr1] ** sigma[1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \
\[Pi]\))\) + 
    kappas\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                    2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ \
kl1 - kr1 - kr2)\)\ d[kr2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \
\((kl1 - kr1)\)\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\
\ kr2 + kr3)\)\ d[kr3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\
\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 
                    2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                    kl2 - kr1)\)\ d[kr1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    2\ kr1 + kr2)\)\ d[kr2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** \
b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \[Pi]\^\(3/2\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + \
kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - \
kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr1)\)\ x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ \
x\)\ d[kr2] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \(1\/\(8\ \@kr1\ \((\(-kl2\) \
+ kr1)\)\ \[Pi]\^\(3/2\)\)\) \((\[ImaginaryI]\ \[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\) - \(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\) \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^+ psi:  0^+-
*)
(*
Collect[relabel[Expand[conju[x] ** u[x] //. prodexpand //. sigmazeta]], {muf, 
      kappas, kappaa}] //. prodfactor
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \
\[Pi]\)\)
*)
(*
psibar gamma^- psi:  0^+-
*)
(*
Collect[relabel[Expand[conjv[x] ** v[x] //. prodexpand //. sigmazeta]], {muf, 
      kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\^2\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \
\((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** a[kr2, \
s1] ** a[kr1, s2] ** Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \
\((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
    muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((\(-kl1\)\ kl2 + 2\ kl1\ kr1 + 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 + 
                          kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 - kl1\ kr1 - 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl2\ kl3 + 
                          kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ \((kl1 - 
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + 
                          kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
          kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\
\)\ x\)\ \((kl1 + kr1)\)\ \@kr2\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl2\ \((kl1 + kr1)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 - 
                          kl2)\)\ \@kr1\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\@kl1\ \((kr1 - kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[
                          kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl1\ \((kl2 + kr1)\)\ d[kr1] ** 
                        sigma[1] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \@kr1\ \((kl1 + kr2)\)\ d[
                          kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
    kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\)\)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
    kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\)\)
*)
(*
psibar gamma^5 psi:  0^-+
*)
(*
Collect[relabel[
      Expand[I(conju[x] ** sigma[2] ** v[x] + conjv[x] ** sigma[2] ** u[x])/
                Sqrt[2] //. prodexpand //. sigmazeta]], {muf, kappas, 
      kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x\)\ \((kl1 + kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** \
Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((\(-kl1\) + kr1)\)\ Conjugate[b[kl1]] ** \
sigma[3] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\(-\(\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** 
                    sigma[2] ** a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                    2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 
                    2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - 
                    kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** 
                  sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ \
kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                    kr2] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\
\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \
\((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 
                    2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 
                    2\ kr2)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - 
                    kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ \
kl1 - kr1 - kr2)\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 
                    2\ kr2 + kr3)\)\ d[kr3] ** sigma[2] ** zeta[s1] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                    kl2 - kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    2\ kr1 + kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                  sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^5 gamma^1 psi:  1^++
*)
(*
Collect[relabel[
      Expand[-(conju[x] ** v[x] + conjv[x] ** u[x])/Sqrt[2] //. 
            prodexpand //. sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((\(-kl1\) + kl2)\)\ Conjugate[b[kl1]] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\
\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \
\((\(-kr1\) + kr2)\)\ d[kr2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
Conjugate[b[kl1]] ** sigma[1] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
d[kr1] ** sigma[1] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\))\) + 
    kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\
\ \((kl1 - kl2)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** \
Conjugate[d[kl2]]\)\/\(8\ \((kl1 - kr1)\)\ \((kl2 - kr1)\)\ \@kr1\ \
\[Pi]\^\(3/2\)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\
\ x\)\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ d[kr2] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - kr2)\
\)\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 \
+ kr3)\)\ x\)\ \((kr1 - kr3)\)\ d[kr3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \
\((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr2)\)\ d[kr2] ** 
                  sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \((kl1 - kr1)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                    a[kr1, s1] ** 
                    Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(8\ \@kl1\ \((kl1 - kr1)\)\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(8\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** 
                  zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \(1\/\(8\ \@kr1\ \((\(-kl2\) \
+ kr1)\)\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\) - \(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) - \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^5 gamma^2 psi:  1^-+
*)
(*
Collect[relabel[
      Expand[I(conju[x] ** sigma[3] ** v[x] - conjv[x] ** sigma[3] ** u[x])/
                Sqrt[2] //. prodexpand //. sigmazeta]], {muf, kappas, 
      kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 \
+ kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \
\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 - kl2)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(4\ \
kl1\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
\((kl1 + kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \
\[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + \
kr2)\)\ x\)\ \((kr1 - kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(4\ kr1\ kr2\ \
\[Pi]\))\) + 
    kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** a[kr2, s1] ** 
                    b[kr1])\)/\((8\ \((kl1 - kr2)\)\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ Conjugate[
                    b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((\(-kl1\) + 
                    kl2)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 - 
                    kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl2 + kr1)\)\ d[kr1] ** 
                  sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + kl2)\)\ \((kl1 - 
                    
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kl1 + kr2)\)\ d[kr2] ** 
                  sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((kr1 - kr2)\)\ d[kr2] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 - \
kr3)\)\ d[kr3] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \@kr2\ \((kr1 + \
kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    zeta[s1] ** a[kr2, s1] ** 
                    b[kr1])\)/\((8\ \@kr2\ \((kr1 + 
                      kr2)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \(1\/\(8\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\)\)\) \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) - \((\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x \
- \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** sigma[2] ** zeta[s1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** 
                  zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((8\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[2] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** 
                  zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((8\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** zeta[s1] ** sigma[3] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
kr3\ x\)\ d[kr3] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(8\ \
\@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \(1\/\(8\ \@kr1\ \((\(-kl2\) + 
                      kr1)\)\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  a[kr1, s1] ** 
                  Conjugate[
                    d[kl2]])\) - \(1\/\(8\ \@kl2\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) - \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** sigma[1] ** 
                  zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((8\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
psibar gamma^+ gamma^5 psi:  1^-+
*)
(*
Collect[relabel[
      Expand[conju[x] ** sigma[3] ** u[x] //. prodexpand //. 
          sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
psibar gamma^- gamma^5 psi:  1^-+
*)
(*
Collect[relabel[
      Expand[conjv[x] ** sigma[3] ** v[x] //. prodexpand //. 
          sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\^2\ \((\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\
\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(4\ kl1\ \
kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ d[kr1] ** sigma[3] ** Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \
\(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\
\))\) + kappas\^2\ \((\(-\(\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                      b[kl1]] ** sigma[2] ** a[kr1, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                      kl3)\)\ \@kr1\ \((\(-kl1\) + 
                      kr1)\)\ \[Pi]\^2)\)\)\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 \
- kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** sigma[3] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[3] ** Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[3] ** a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[3] ** Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[3] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ \
x\)\ d[kr4] ** sigma[2] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\))\) + 
    muf\ \((kappas\ \((\(-\(\((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 \
- kl1\ kr1 - kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** a[kr1, s1] ** 
                          Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                            kr1)\)\ \((kl2 - 
                            kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\)\)\) - \((\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \
\((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kr1\)\ kr2 + 
                          kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1\ \((kl2 - 2\ kr1)\) - 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 + 
                          kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** sigma[3] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[2] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ \((kl1 - 
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** sigma[3] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ \
kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
          kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 - kl1\ kr1 - 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        sigma[2] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\
\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl2\ kl3 + 
                          kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                        zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - 
                          kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** 
                        zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1\ \((kl2 - 2\ kr1)\) - 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        sigma[3] ** Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[2] ** 
                        zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ \
kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[2] ** zeta[s1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1\ \((kl2 \
- kr1)\) - 2\ kl2\ kr1)\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kr1\)\ \
kr2 + kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
    kappaa\ kappas\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr1, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s2] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr2, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ \
x\)\ d[kr4] ** sigma[2] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ \
x\)\ d[kr4] ** sigma[2] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ \
x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ \
x\)\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                    kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                    kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                    kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
    kappaa\^2\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl2 \
+ kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** zeta[s2] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr3)\)\ x\)\ d[kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** 
                  Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ \
x\)\ d[kr4] ** sigma[2] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ \
x\)\ d[kr1] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[
                    kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ \
x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \[ExponentialE]\^\
\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[
                    kr3] ** sigma[2] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\)\)
*)
(*
Find pseudoscalar 4-fermion interaction to leading order, with flavor
*)
u[x_, f_] := 
    1/Sqrt[2 Pi] (b[kf, f] Exp[-I kf x] + 
          sigma[1] ** Conjugate[d[kf, f]] Exp[I kf x]);
kappaa /: Conjugate[kappaa[a_]] := kappaa[a]
kappas /: Conjugate[kappas[a_]] := kappas[a]
muf /: Conjugate[muf[a_]] := muf[a]
(*
Xi is used for the Yukawa interactions and the 4-fermion interactions
*)
xi[x_, f_] = 
    Expand[muf[f]/Sqrt[2] sigma[1] ** u[x, f] + 
              kappaa[f]/Sqrt[2] sigma[1] ** zeta[rr] ** m[x, rr] ** u[x, f] - 
              kappas[f]/Sqrt[2] sigma[1] ** m[x, sumrr] ** u[x, f] //. 
            prodexpand /. {zeta[rho_] ** 
                a[c_, -rho_] -> -zeta[rho] ** a[c, rho], 
            a[b_, -sumrr] -> a[b, sumrr]} //. prodexpand];
(*
We need the definition of xi from the Yukawa interactions
*)
(*
cxi[x_, f_] = 
  Conjugate[xi[x, f]] /. {Conjugate[x] -> x, Conjugate[ka] -> ka, 
            Conjugate[kf] -> kf, Conjugate[ka^n_] -> ka^n} /. {ka -> kka, 
          kf -> kks, rr -> ll, sumrr -> sumll} //. prodexpand //. sigmazeta
*)
(*
conju[x_, f_] = 
  Conjugate[u[x, f]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ Conjugate[b[kks, f]] + \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ d[kks, f] ** \
sigma[1]\)\/\@\(2\ \[Pi]\)\)
*)
(*
The chiral four-fermion interaction in the Hamiltonian is kappa5 qq**qq/2 \
where qq is:
*)
(*
 qq[x_, f1_, f2_] = 
  relabel[Expand[
        I conju[x, f1] ** sigma[2] ** Integrate[xi[x, f2], x] - 
              I Integrate[cxi[x, f1], x] ** sigma[2] ** u[x, f2] //. 
            prodexpand //. sigmazeta]] //. prodfactor
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
Clear[qqpinch]; 
qqpinch[x_, f1_, f2_] = 
  qq[x, f1, f2] /. 
    a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
qq[x, f1, f2]
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    relabel[Map[Distribute, 
              qqpinch[x, f1, f2] /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                  kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
            Map[Distribute, qqpinch[x, f2, f1]] //. prodexpand] //. 
      prodfactor;
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl1\)\" \
is similar to existing symbol \"\!\(kl1\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl2\)\" \
is similar to existing symbol \"\!\(kl2\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl3\)\" \
is similar to existing symbol \"\!\(kl3\)\"."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell1\"\) will be \
suppressed during this calculation."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr1\)\" is \
similar to existing symbols \!\({kkl1, kr1}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr2\)\" is \
similar to existing symbols \!\({kkl2, kr2}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr3\)\" is \
similar to existing symbols \!\({kkl3, kr3}\)."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell\"\) will be \
suppressed during this calculation."
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)
expr2pinch = 
  kappa5/2*simpint[
      int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
       k11]
*)
(*
\!\(1\/2\ kappa5\ int[relabel[qq[x, f1, f2] ** qq[x, f2, f1]], {x}]\)
*)
(*
Just look at the term that acts in the two particle sector.
*)
(*
expr2pinch /. {kappas[_] -> 0, kappaa[_] -> 0}
*)
(*
\!\(1\/2\ kappa5\ int[relabel[qq[x, f1, f2] ** qq[x, f2, f1]], {x}]\)
*)
(*
Select terms that act on a baryon (no anti-quarks)
*)
(*
Clear[qqbaryon];
qqbaryon[x_, f1_, f2_] = 
  qq[x, f1, f2] /. d[_] -> 0 /. {0 ** a_ -> 0, a_ ** 0 -> 0}
*)
(*
 (*put in the coupling constant later*)
  expr1baryon = 
    relabel[Map[Distribute, 
              qqbaryon[x, f1, f2] /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                  kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
            Map[Distribute, qqbaryon[x, f2, f1]] //. prodexpand] //. 
      prodfactor;
*)
(*
$Aborted
*)
(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the leading baryon interactions in the \
Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*) expr2baryon = 
  kappa5/2*simpint[
      int[expr1baryon, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
       k11]
*)
(*
Find transverse vector 4-fermion interaction to leading order, acts on rho \
J_z = -1,1
*)
(*
The vector four-fermion interaction in the Hamiltonian is -kappa6 \
(qq**qq-rr**rr)/2 where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** sigma[3] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** sigma[3] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                    b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 + kl2)\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \
\[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ \
x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[2] ** Conjugate[d[kl1]]\)\/\(2\ \@2\ \
kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \
\[Pi]\))\) + 
    kappas\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1 - kr2)\)\ x\)\ \((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 
                    2\ kl2 - kr1)\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + kl2 - 
                    2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + 
                    kl2 - kr1)\)\ d[kr1] ** sigma[2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 
                    2\ kr1 + kr2)\)\ d[kr2] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ d[
                    kr2] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] \
** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \
\((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + \
kr1)\)\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** zeta[s1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - \
kr2)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr1)\)\ x\)\ d[kr1] ** sigma[2] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ \
x\)\ d[kr2] ** sigma[2] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ \
x\)\ d[kr2] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** 
                  zeta[s1] ** sigma[3] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** \
zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + \
kr2)\)\ \[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
 rr[x_] = 
  Collect[relabel[
        Expand[-I conju[x] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                    Conjugate[
                      d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** \
sigma[1] ** b[kr1]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[1] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\))\) + 
    kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ \
x\)\ \((kl1 + kl2 - 2\ kr1)\)\ Conjugate[b[kl1]] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl1 + 2\ kl2 + 
                    kl3)\)\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \((2\ kl1 - kr1 - kr2)\)\ \
d[kr2] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\)\
\ \((kl1 - kr2)\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\
\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\) \
- \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \
\((\(-kl1\) + kr1 + 2\ kr2)\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + kr2)\)\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 + 2\ kl2 - 
                    kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1 + kl2 - kr1)\)\ d[kr1] ** 
                  sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((\(-kl1\) + 2\ kr1 + kr2)\)\ d[
                    kr2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** \
zeta[s1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \((kl1 - kr1)\
\)\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\
\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\
\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((\(-kl2\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl2 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 - 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** 
                  zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl1\ \((kl1 - 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 \
+ kl2)\)\ Conjugate[b[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\
\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \
\[Pi]\))\) + 
    kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \((kl2 + 
                      kl3)\)\ \[Pi]\^\(3/2\))\)\)\) - \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + \
2\ kr2 + kr3)\)\ d[kr3] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \
\@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                    Conjugate[a[kl2, s1]] ** 
                    Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                      kl2)\)\ \[Pi]\^\(3/2\))\)\)\) + \
\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
kr3\ x\)\ d[kr3] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \
\@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\(3/2\)\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \((kl2 + kl3)\)\ \
x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + \
kr3)\)\ x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
rrpinch = 
  rr[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \
\((kl1 + kl2)\)\ Conjugate[b[kl1]] ** 
                    Conjugate[
                      d[kl2]]\)\/\(2\ \@2\ kl1\ kl2\ \[Pi]\)\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ \
d[kr2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
x\)\ \((kl1 + 2\ kl2 + kl3)\)\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr1 + 2\ kr2 + kr3)\)\ d[kr3] \
** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + kr3)\)\ \
\[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \[Pi]\^\
\(3/2\)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ \
\((kl2 + kl3)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    Collect[relabel[
            Map[Distribute, 
                  qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
                Map[Distribute, qqpinch] //. prodexpand] - 
          relabel[Map[Distribute, 
                  rrpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
                Map[Distribute, rrpinch] //. prodexpand], {muf, kappaa, 
          kappas}] //. prodfactor;
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl1\)\" \
is similar to existing symbol \"\!\(kl1\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl2\)\" \
is similar to existing symbol \"\!\(kl2\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl3\)\" \
is similar to existing symbol \"\!\(kl3\)\"."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell1\"\) will be \
suppressed during this calculation."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr1\)\" is \
similar to existing symbols \!\({kkl1, kr1}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr2\)\" is \
similar to existing symbols \!\({kkl2, kr2}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr3\)\" is \
similar to existing symbols \!\({kkl3, kr3}\)."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell\"\) will be \
suppressed during this calculation."
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa6 is the coupling constant*)expr2pinch = -kappa6/
      2*(simpint[
        int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11])
*)
(*
\!\(\(-\(1\/2\)\)\ kappa6\ \((muf\^2\ \((\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \
\[Pi]\)\) \((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** Conjugate[d[kl2]] ** 
                      d[kr2] ** 
                      b[kr1])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\
\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** 
                      Conjugate[
                        d[kl2]])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[d[kl2]] ** d[kr2] ** sigma[3] ** 
                      b[kr1])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\
\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      sigma[3] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[d[kl2]])\))\) + 
        kappas\^2\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 
                        2\ kl2 + kl3)\)\ \((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[3] ** 
                      a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
        muf\ \((kappas\ \((\(-\(\((\((kl1 + 2\ kl2 + kl3)\)\ \((kr1 + 
                                kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                                kr2]\ Conjugate[b[kl1]] ** 
                              Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                              d[kr2] ** 
                              b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                                kl2)\)\ \((kl2 + 
                                kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\)\)\) - \((\
\((kl1 + kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** Conjugate[d[kl2]] ** 
                            d[kr3] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                              2\ kl2 + kl3)\)\ \((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ d[kr2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** a[kr2, s1] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                              2\ kl2 + kl3)\)\ \((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                            d[kr2] ** sigma[3] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[d[kl2]] ** d[kr3] ** sigma[3] ** 
                            a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\((kl1 + 
                              2\ kl2 + kl3)\)\ \((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ d[kr2] ** sigma[3] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ \((kr1 + 2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** sigma[3] ** a[kr2, s1] ** 
                            b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
              kappaa\ \((\(-\(\((\((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + 
                                kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                              Conjugate[d[kl2]] ** d[kr3] ** zeta[s1] ** 
                              a[kr2, s1] ** 
                              b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                                kr2)\)\ \[Pi]\^\(3/2\))\)\)\) - \((\((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                            d[kr2] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kr1 \
+ kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** zeta[s1] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kl1 \
+ kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s1] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \[Pi]\^\(3/2\))\) + \((\((kl1 + 
                              kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** Conjugate[d[kl2]] ** 
                            d[kr3] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                            a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
                            zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \((\((kl1 \
+ kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                            sigma[3] ** Conjugate[d[kl2]] ** d[kr3] ** 
                            zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \[Pi]\^\(3/2\))\) - \((\((kr1 + 
                              kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[3] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** sigma[3] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \((\((kr1 \
+ kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                            sigma[1] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\((kr1 \
+ kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[3] ** 
                            b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[3] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl1 + 
                              kl2)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) + \((\((kl1 \
+ kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                            zeta[s1] ** sigma[1] ** a[kr2, s1] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kl1 + 
                              kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** zeta[s1] ** sigma[3] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[3] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                            sigma[1] ** zeta[s1] ** sigma[2] ** 
                            Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                            d[kr2] ** sigma[3] ** 
                            b[kr1])\)/\((8\ \@kl2\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\
\[ImaginaryI]\ \((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                            Conjugate[d[kl2]] ** d[kr3] ** sigma[1] ** 
                            zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                              
                              kr3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[3] ** 
                            b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                            zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ \@kl2\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(3/2\))\) - \((\
\[ImaginaryI]\ \((kl1 + kl2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3]\ d[
                              kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[3] ** 
                            Conjugate[
                              d[kl2]])\)/\((8\ kl1\ kl2\ \@kr2\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
        kappaa\ kappas\ \((\((\((kl1 + 2\ kl2 + kl3)\)\ delta[\(-kl1\) - 
                        kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                      sigma[1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[1] ** zeta[s2] ** 
                      sigma[1] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                      d[kr3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[3] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** sigma[1] ** 
                      a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\((kr1 + 2\ kr2 + 
                        kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** sigma[3] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\((kl1 + 2\ kl2 + 
                        kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ d[kr3] ** zeta[s2] ** sigma[3] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \((kr1 + 
                        2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                        kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \((kl1 + 
                        2\ kl2 + kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                        kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[3] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[1] ** zeta[s1] ** sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \((kl1 + 
                        2\ kl2 + kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                        kr2 + kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** 
                      sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[3] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((\[ImaginaryI]\ \((kr1 + 
                        2\ kr2 + kr3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                        kr2 + kr3]\ d[kr3] ** sigma[3] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                      sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\))\) + 
        kappaa\^2\ \((\((delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                        kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** d[kr3] ** 
                      sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                      d[kr3] ** zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** 
                      sigma[1] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** a[kr2, s2] ** 
                      b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** zeta[s1] ** 
                      sigma[1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** sigma[3] ** 
                      a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                      sigma[3] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** zeta[s1] ** sigma[3] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      sigma[3] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                      d[kr3] ** sigma[1] ** zeta[s1] ** sigma[2] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                      zeta[s2] ** sigma[2] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** sigma[3] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((\[ImaginaryI]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3]\ d[kr3] ** zeta[s2] ** 
                      sigma[3] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[1] ** zeta[s1] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                      sigma[1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                      sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** 
                      sigma[1] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[1] ** zeta[s1] ** sigma[1] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + 
                        kr3)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 + 
                        kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** zeta[s2] ** 
                      sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[1] ** zeta[s1] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\))\)\)
*)
(*
Find longitudinal vector 4-fermion interaction to leading order, acts on rho \
J_z = 0
*)
(*
This couples to the J_z =0 component of the rho
*)
(*
The vector four-fermion interaction in the Hamiltonian is -kappa7 \
(qq**rr+rr**qq) where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[ conju[x] ** u[x] //. prodexpand //. sigmazeta]], {muf, kappas,
         kappaa}] //. prodfactor
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \
\[Pi]\)\)
*)
(*
 rr[x_] = 
  Collect[relabel[
        Expand[Integrate[cxi[x], x] ** Integrate[xi[x], x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\^2\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** b[kr1]\)\/\(4\ kl1\ kr1\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
Conjugate[d[kl1]]\)\/\(4\ kl1\ kr1\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** \
sigma[1] ** Conjugate[d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\) - \(\[ExponentialE]\^\
\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                  a[kr2, s1] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \
\((kr1 + kr2)\)\ \@kr3\ \((\(-kl1\) + kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  a[kr1, s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** a[kr2, \
s1] ** a[kr1, s2] ** Conjugate[d[kl1]]\)\/\(16\ \@kr1\ \((\(-kl1\) + kr1)\)\ \
\@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** b[kr1]\)\/\(16\ \@kr2\ \
\((kr1 + kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2\))\) + 
    muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - \
kr2)\)\ x\)\ \((kr1\ kr2 - kl1\ \((2\ kr1 + kr2)\))\)\ Conjugate[b[kl1]] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - 
                          kr2)\)\ \@kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((\(-kl1\)\ kl2 + 2\ kl1\ kr1 + 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** 
                        Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl2\ kr1 + 
                          kl1\ \((\(-kl2\) + kr1)\))\)\ d[kr1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ \@kl1\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \((kr1\ kr2 - 
                          kl1\ \((kr1 + 2\ kr2)\))\)\ d[kr2] ** a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ \@kr1\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((2\ kl1\ kl2 - kl1\ kr1 - 
                          kl2\ kr1)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \@kr1\ \[Pi]\^\(3/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \((kl2\ kl3 + 
                          kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\((\(-2\)\ kr1\ kr2 + kl1\ \((kr1 + kr2)\))\)\ d[kr2] ** sigma[1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \@kl1\ kr1\ \((\(-kl1\) + 
                          kr1)\)\ \((kl1 - 
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + 
                          kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
          kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\
\)\ x\)\ \((kl1 + kr1)\)\ \@kr2\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kl1\ kr1\ \((kl1 - kr2)\)\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl2\ \((kl1 + kr1)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                        b[kr1])\)/\((8\ kl1\ \((kl1 + kl2)\)\ \((kl2 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \((kl1 - 
                          kl2)\)\ \@kr1\ Conjugate[b[kl1]] ** zeta[s1] ** 
                        sigma[1] ** a[kr1, s1] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl1\ kl2\ \((kl1 - 
                          kr1)\)\ \((kl2 - 
                          kr1)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl1\) + kr1 + kr2)\)\ x\)\ \
\@kl1\ \((kr1 - kr2)\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        b[kr1])\)/\((8\ \((kl1 - kr1)\)\ kr1\ \((kl1 - 
                          kr2)\)\ kr2\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[
                          kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 - kr1)\)\ x\)\ \@kl1\ \((kl2 + kr1)\)\ d[kr1] ** 
                        sigma[1] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl1, s1]] ** 
                        Conjugate[d[kl2]])\)/\((8\ kl2\ \((kl1 + 
                          kl2)\)\ \((kl1 - 
                          kr1)\)\ kr1\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\
\^\(\[ImaginaryI]\ \((kl1 - kr1 - kr2)\)\ x\)\ \@kr1\ \((kl1 + kr2)\)\ d[
                          kr2] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                        a[kr1, s1] ** 
                        Conjugate[d[kl1]])\)/\((8\ kl1\ \((kl1 - 
                          kr1)\)\ kr2\ \((kr1 + 
                          kr2)\)\ \[Pi]\^\(3/2\))\))\))\) + 
    kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                      b[kl1]] ** zeta[s1] ** a[kr2, s1] ** 
                    Conjugate[a[kl2, s2]] ** 
                    b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                      kr1)\)\ \@kr2\ \((\(-kl1\) + 
                      kr2)\)\ \[Pi]\^2)\)\)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ \
x\)\ Conjugate[b[kl1]] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s2] ** Conjugate[a[kl1, s1]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s2]] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr1, s2] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s2]] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** sigma[1] ** a[kr2, s2] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\) + 
    kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((\(-kl2\) + \
kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl2 - 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x + \[ImaginaryI]\ \((kl1 - kr3)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((\(-kl1\) + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((\(-kl3\) + kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x + \[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr1, s1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl2 + 
                    kl3)\)\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x + \[ImaginaryI]\ \((kl1 - kr2)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  a[kr2, s1] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \@kr2\ \((\(-kl1\) + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 - kr1)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((\(-kl3\) + 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[
                    b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl2\) + kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + \
kr2)\)\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kl2\ \((kl2 - kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((\(-kl1\) + kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr2, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  b[kr1])\)/\((16\ \@kl1\ \((kl1 - kr1)\)\ \@kr2\ \((kr2 + 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr3)\)\ \
x\)\ d[kr3] ** sigma[1] ** zeta[s1] ** zeta[s2] ** Conjugate[a[kl1, s1]] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kl1\ \@kr2\ \((kr1 + kr2)\)\ \((kl1 - 
                    kr3)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 + kl3)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr1)\)\ x\)\ d[kr1] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]])\)/\((16\ \@kl1\ \@kl2\ \((kl2 + 
                    kl3)\)\ \((kl1 - 
                    kr1)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr1, s1] ** 
                  Conjugate[a[kl1, s2]] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \((kl1 + 
                    kl2)\)\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl2 - kr1)\)\ x - \[ImaginaryI]\ \((\(-kl1\) + kr2)\)\ x\)\ d[kr2] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** a[kr1, s2] ** 
                  Conjugate[d[kl2]])\)/\((16\ \@kl1\ \@kr1\ \((\(-kl2\) + 
                    kr1)\)\ \((kl1 - 
                    kr2)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 - kr1)\)\ x - \[ImaginaryI]\ \((kr2 + kr3)\)\ x\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** zeta[s2] ** sigma[1] ** a[kr2, s1] ** 
                  a[kr1, s2] ** 
                  Conjugate[d[kl1]])\)/\((16\ \@kr1\ \((\(-kl1\) + 
                    kr1)\)\ \@kr2\ \((kr2 + kr3)\)\ \[Pi]\^2)\))\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
rrpinch = 
  rr[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(muf\^2\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[
                      d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\))\) + 
    muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ \
kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
          kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\
\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \
\((kr1 - kr3)\)\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
    kappas\^2\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ \
x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                    Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\)\)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\))\) + 
    kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                    Conjugate[a[kl3, s2]] ** 
                    Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^2)\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\))\) + 
    kappaa\^2\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \
\[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  zeta[s2] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[
                    kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^2)\))\)\)
*)
(*
Map[Distribute, rrpinch]
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ muf\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[
                d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ muf\^2\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + 
    kappas\ muf\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                    kl2)\)\ kl3\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \((kr2\ kr3 + 
                    kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ \
kappas\^2\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + kl4)\)\ \[Pi]\^2)\) + 
    kappaa\ muf\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                    kl2)\)\ kl3\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[kr3] ** 
                  sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
kappas\^2\ d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
            Conjugate[a[kl3, s1]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[
              kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[
              kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\^2\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\^2\ d[kr4] ** 
            sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\)\)
*)
(*
Map[Distribute, rrpinch, 3]
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ muf\^2\ Conjugate[b[kl1]] ** sigma[1] ** 
              Conjugate[
                d[kl2]]\)\/\(4\ kl1\ kl2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ muf\^2\ d[kr2] ** \
sigma[1] ** b[kr1]\)\/\(4\ kr1\ kr2\ \[Pi]\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x\)\ kappas\ \((kl2\ kl3 + 
              kl1\ \((kl2 + 2\ kl3)\))\)\ muf\ Conjugate[b[kl1]] ** sigma[1] ** 
            Conjugate[a[kl2, s1]] ** 
            Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
              kl2)\)\ kl3\ \((kl2 + 
              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ kappas\ \((kr2\ kr3 + 
              kr1\ \((kr2 + 2\ kr3)\))\)\ muf\ d[kr3] ** sigma[1] ** 
            a[kr2, s1] ** 
            b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappas\^2\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ kappaa\ \@kl2\ \((kl1 - kl3)\)\ muf\ Conjugate[
              b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + kl2)\)\ kl3\ \((kl2 + 
              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x\)\ kappaa\ \@kr2\ \((kr1 - 
              kr3)\)\ muf\ d[kr3] ** sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
            b[kr1])\)/\((8\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
              kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ \
kappas\^2\ d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
            Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\ kappas\ \
Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
            Conjugate[a[kl3, s1]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[
              kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) - \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\ kappas\ d[
              kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x\)\ kappaa\^2\ \
Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
            Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
            Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
              kl2)\)\ \@kl3\ \((kl3 + 
              kl4)\)\ \[Pi]\^2)\) + \((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ kappaa\^2\ d[kr4] ** 
            sigma[1] ** zeta[s1] ** zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
            b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
              kr4)\)\ \[Pi]\^2)\)\)
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    Collect[relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kl4 -> kkl4, kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, 
                      kr4 -> kkr4, s1 -> ss1}) ** 
                Map[Distribute, rrpinch, 3] //. prodexpand] + 
          relabel[Map[Distribute, 
                  rrpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kl4 -> kkl4, kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, 
                      kr4 -> kkr4, s1 -> ss1}, 3] ** qqpinch //. 
              prodexpand], {muf, kappaa, kappas}] //. prodfactor;
*)
(*
expr1pinch
*)
(*
\!\(muf\^2\ \((\(-\(\(1\/\(8\ kl1\ kl2\ kl3\ kl4\ \[Pi]\^2\)\) \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3 + kl4)\)\ x\)\ \((kl1\ \
kl2 + kl3\ kl4)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                    Conjugate[b[kl3]] ** sigma[1] ** 
                    Conjugate[
                      d[kl4]])\)\)\) - \(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \[Pi]\^2\
\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1 - kr2)\)\ x\)\ \
\((kl1\ kl2 + kr1\ kr2)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\) - \(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \[Pi]\^2\)\) \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 - kr1 - kr2)\)\ x\)\ \((kl1\ \
kl2 + kr1\ kr2)\)\ d[kr2] ** sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  sigma[1] ** 
                  Conjugate[
                    d[kl2]])\) - \(\(1\/\(8\ kr1\ kr2\ kr3\ kr4\ \[Pi]\^2\)\)\
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3 + kr4)\)\ x\)\ \
\((kr1\ kr2 + kr3\ kr4)\)\ d[kr4] ** sigma[1] ** b[kr3] ** d[kr2] ** 
                sigma[1] ** b[kr1])\)\))\) + 
    muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ x + \[ImaginaryI]\ kl4\ x + \[ImaginaryI]\ kl5\ x\)\ \((kl2\ kl3 + 
                          kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** Conjugate[b[kl4]] ** sigma[1] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ Conjugate[
                          b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4 + \
kl5)\)\ x\)\ \((kl4\ kl5 + kl3\ \((kl4 + 2\ kl5)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                        sigma[1] ** Conjugate[a[kl4, s1]] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl3\ \@kl4\ \((kl3 + 
                          kl4)\)\ kl5\ \((kl4 + 
                          kl5)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** sigma[1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ \((kl2\ kl3 + kl1\ \((kl2 + 2\ kl3)\))\)\ d[kr2] ** 
                        sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ kl1\ \@kl2\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \((kr2\ kr3 + kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr3] ** sigma[1] ** 
                        a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ \((kr3 + kr4 + kr5)\)\ x\)\ \((kr4\ kr5 + 
                          kr3\ \((kr4 + 2\ kr5)\))\)\ d[kr5] ** sigma[1] ** 
                        a[kr4, s1] ** b[kr3] ** d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ kr3\ \@kr4\ \((kr3 + 
                          kr4)\)\ kr5\ \((kr4 + 
                          kr5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x - \
\[ImaginaryI]\ kr4\ x - \[ImaginaryI]\ kr5\ x\)\ \((kr2\ kr3 + 
                          kr1\ \((kr2 + 2\ kr3)\))\)\ d[kr5] ** sigma[1] ** 
                        b[kr4] ** d[kr3] ** sigma[1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kr1\ \@kr2\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\))\) + 
          kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4 + kl5)\)\ x\)\ \@kl4\ \
\((kl3 - kl5)\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                        Conjugate[b[kl3]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl4, s1]] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl3\ \((kl3 + 
                          kl4)\)\ kl5\ \((kl4 + 
                          kl5)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[d[kl2]] ** d[kr3] ** sigma[1] ** zeta[s1] ** 
                        a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x + \[ImaginaryI]\ kl4\ x + \
\[ImaginaryI]\ kl5\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** 
                        zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** Conjugate[b[kl4]] ** sigma[1] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl1\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ \@kl2\ \((kl1 - kl3)\)\ Conjugate[b[kl1]] ** 
                        zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ kl1\ \((kl1 + kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ \@kl2\ \((kl1 - kl3)\)\ d[kr2] ** sigma[1] ** 
                        b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ kl1\ \((kl1 + 
                          kl2)\)\ kl3\ \((kl2 + 
                          kl3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2 + \
kr3)\)\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[kr3] ** sigma[1] ** zeta[s1] ** 
                        a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                        sigma[1] ** 
                        Conjugate[d[kl2]])\)/\((16\ kr1\ \((kr1 + 
                          kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2 + kr3)\)\ x - \
\[ImaginaryI]\ kr4\ x - \[ImaginaryI]\ kr5\ x\)\ \@kr2\ \((kr1 - kr3)\)\ d[
                          kr5] ** sigma[1] ** b[kr4] ** d[kr3] ** sigma[1] ** 
                        zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kr1\ \((kr1 + kr2)\)\ kr3\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ \((kr3 + kr4 + kr5)\)\ x\)\ \@kr4\ \((kr3 - kr5)\)\ d[kr5] ** 
                        sigma[1] ** zeta[s1] ** a[kr4, s1] ** b[kr3] ** 
                        d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ kr3\ \((kr3 + kr4)\)\ kr5\ \((kr4 + 
                          kr5)\)\ \[Pi]\^\(5/2\))\))\))\) + 
    kappas\^2\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ \
x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ kl5\ x + \[ImaginaryI]\
\ kl6\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                    Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** 
                    Conjugate[b[kl5]] ** sigma[1] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                      kl2)\)\ \@kl3\ \((kl3 + 
                      kl4)\)\ \[Pi]\^3)\)\)\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \
\[ImaginaryI]\ \((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** sigma[1] ** 
                  Conjugate[a[kl4, s1]] ** Conjugate[a[kl5, s2]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                    kl4)\)\ \@kl5\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** a[kr3, s2] ** 
                  a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((kr3 + 
                    
                    kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                  a[kr5, s2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                  b[kr5] ** d[kr4] ** sigma[1] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\))\) + 
    kappaa\ kappas\ \((\(-\(\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ \
\((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                    Conjugate[b[kl3]] ** zeta[s1] ** sigma[1] ** 
                    Conjugate[a[kl4, s1]] ** Conjugate[a[kl5, s2]] ** 
                    Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                      kl4)\)\ \@kl5\ \((kl5 + 
                      kl6)\)\ \[Pi]\^3)\)\)\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ \((kl3 + \
kl4)\)\ x + \[ImaginaryI]\ \((kl5 + kl6)\)\ x\)\ Conjugate[b[kl1]] ** 
                  sigma[1] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl4, s2]] ** 
                  Conjugate[a[kl5, s1]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                    kl4)\)\ \@kl5\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                  a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                  a[kr3, s2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ kl5\
\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]] ** Conjugate[b[kl5]] ** sigma[1] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ kl5\
\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]] ** Conjugate[b[kl5]] ** sigma[1] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                  a[kr3, s1] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  sigma[1] ** 
                  Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                  a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  sigma[1] ** 
                  Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                  b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s1] ** 
                  a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                  b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** a[kr3, s2] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) - \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                  zeta[s2] ** a[kr5, s1] ** a[kr4, s2] ** b[kr3] ** d[kr2] ** 
                  sigma[1] ** 
                  b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                  zeta[s2] ** a[kr5, s2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** 
                  sigma[1] ** 
                  b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\))\) + 
    kappaa\^2\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ \((kl5 + kl6)\)\
\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** 
                  Conjugate[b[kl3]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl4, s1]] ** Conjugate[a[kl5, s2]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl4\ \((kl3 + 
                    kl4)\)\ \@kl5\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                  zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x + \[ImaginaryI]\ kl5\
\ x + \[ImaginaryI]\ kl6\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** zeta[s1] ** 
                  sigma[1] ** Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]] ** Conjugate[b[kl5]] ** sigma[1] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** zeta[s2] ** zeta[s1] ** 
                  sigma[1] ** Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                  Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \((kl3 + kl4)\)\ x - \[ImaginaryI]\ kr1\
\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                  Conjugate[d[kl4]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \@kl3\ \((kl3 + 
                    kl4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - \
\[ImaginaryI]\ \((kr3 + kr4)\)\ x\)\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                  zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                  Conjugate[b[kl1]] ** sigma[1] ** 
                  Conjugate[d[kl2]])\)/\((32\ \@kr2\ \((kr1 + 
                    kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \((kr3 + kr4)\)\ x - \
\[ImaginaryI]\ kr5\ x - \[ImaginaryI]\ kr6\ x\)\ d[kr6] ** sigma[1] ** 
                  b[kr5] ** d[kr4] ** sigma[1] ** zeta[s1] ** zeta[s2] ** 
                  a[kr3, s1] ** a[kr2, s2] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \@kr3\ \((kr3 + 
                    kr4)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ \((kr3 + \
kr4)\)\ x - \[ImaginaryI]\ \((kr5 + kr6)\)\ x\)\ d[kr6] ** sigma[1] ** 
                  zeta[s2] ** zeta[s1] ** a[kr5, s2] ** a[kr4, s1] ** b[kr3] ** 
                  d[kr2] ** sigma[1] ** 
                  b[kr1])\)/\((32\ \@kr4\ \((kr3 + kr4)\)\ \@kr5\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\))\)\)
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa7 is the coupling constant*)expr2pinch = -kappa7*(simpint[
        int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11])
*)
(*
\!\(\(-kappa7\)\ \((muf\^2\ \((\(-\(\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \
\((\((kl1\ kl2 + kr1\ kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ Conjugate[
                          b[kl1]] ** sigma[1] ** Conjugate[d[kl2]] ** d[kr2] ** 
                        sigma[1] ** 
                        b[kr1])\)\)\) - \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\
\) \((\((kl1\ kl2 + kr1\ kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      sigma[1] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\))\) + 
        muf\ \((kappas\ \((\((\((kl2\ kl3 + 
                              kl1\ \((kl2 + 2\ kl3)\))\)\ delta[\(-kl1\) - 
                              kl2 - kl3 + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                            sigma[1] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                            b[kr1])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                              kl2)\)\ kl3\ \((kl2 + 
                              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr2\ kr3 + 
                              kr1\ \((kr2 + 2\ kr3)\))\)\ delta[\(-kl1\) - 
                              kl2 + kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                            sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** 
                            sigma[1] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                              kr2)\)\ kr3\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) + \((\((kl2\ kl3 + 
                              kl1\ \((kl2 + 2\ kl3)\))\)\ delta[\(-kl1\) - 
                              kl2 - kl3 + kr1 + kr2]\ d[kr2] ** sigma[1] ** 
                            b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ kl1\ \@kl2\ \((kl1 + 
                              kl2)\)\ kl3\ \((kl2 + 
                              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\((kr2\ kr3 + 
                              kr1\ \((kr2 + 2\ kr3)\))\)\ delta[\(-kl1\) - 
                              kl2 + kr1 + kr2 + kr3]\ d[kr3] ** sigma[1] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[1] ** 
                            Conjugate[d[kl2]])\)/\((8\ kr1\ \@kr2\ \((kr1 + 
                              kr2)\)\ kr3\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
              kappaa\ \((\((\@kr2\ \((kr1 - kr3)\)\ delta[\(-kl1\) - kl2 + 
                              kr1 + kr2 + kr3]\ Conjugate[b[kl1]] ** 
                            sigma[1] ** Conjugate[d[kl2]] ** d[kr3] ** 
                            sigma[1] ** zeta[s1] ** a[kr2, s1] ** 
                            b[kr1])\)/\((8\ kr1\ \((kr1 + 
                              kr2)\)\ kr3\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\) + \((\@kl2\ \((kl1 - 
                              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                            sigma[1] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** sigma[1] ** 
                            b[kr1])\)/\((8\ kl1\ \((kl1 + 
                              kl2)\)\ kl3\ \((kl2 + 
                              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\@kl2\ \((kl1 - 
                              kl3)\)\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
                            Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                            Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((8\ kl1\ \((kl1 + 
                              kl2)\)\ kl3\ \((kl2 + 
                              kl3)\)\ \[Pi]\^\(3/2\))\) + \((\@kr2\ \((kr1 - 
                              kr3)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                              kr3]\ d[kr3] ** sigma[1] ** zeta[s1] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[1] ** 
                            Conjugate[d[kl2]])\)/\((8\ kr1\ \((kr1 + 
                              kr2)\)\ kr3\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(3/2\))\))\))\) + 
        kappas\^2\ \((\(-\(\((delta[\(-kl1\) - kl2 - kl3 - kl4 + kr1 + 
                          kr2]\ Conjugate[b[kl1]] ** sigma[1] ** 
                        Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                        Conjugate[d[kl4]] ** d[kr2] ** sigma[1] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \@kl3\ \((kl3 + 
                          kl4)\)\ \[Pi]\^2)\)\)\) - \((delta[\(-kl1\) - kl2 + 
                        kr1 + kr2 + kr3 + kr4]\ Conjugate[b[kl1]] ** 
                      sigma[1] ** Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** 
                      a[kr3, s1] ** a[kr2, s2] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[a[kl3, s2]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + kr1 + 
                        kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** a[kr3, s2] ** 
                      a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2)\))\) + 
        kappaa\ kappas\ \((\((delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                      a[kr3, s1] ** a[kr2, s2] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + kr1 + 
                        kr2 + kr3 + kr4]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                      a[kr3, s2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      sigma[1] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[a[kl3, s2]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                      sigma[1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      sigma[1] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                      sigma[1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                      Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                      Conjugate[a[kl2, s2]] ** Conjugate[a[kl3, s1]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) - \((delta[\(-kl1\) - kl2 + kr1 + 
                        kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                      a[kr3, s1] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 + kr1 + 
                        kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                      a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2)\))\) + 
        kappaa\^2\ \((\((delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3 + 
                        kr4]\ Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]] ** d[kr4] ** sigma[1] ** zeta[s1] ** 
                      zeta[s2] ** a[kr3, s1] ** a[kr2, s2] ** 
                      b[kr1])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + 
                        kr4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ Conjugate[b[kl1]] ** zeta[s2] ** 
                      zeta[s1] ** sigma[1] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[a[kl3, s1]] ** Conjugate[d[kl4]] ** d[kr2] ** 
                      sigma[1] ** 
                      b[kr1])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 - kl3 - 
                        kl4 + kr1 + kr2]\ d[kr2] ** sigma[1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** zeta[s2] ** sigma[1] ** 
                      Conjugate[a[kl2, s1]] ** Conjugate[a[kl3, s2]] ** 
                      Conjugate[d[kl4]])\)/\((16\ \@kl2\ \((kl1 + 
                        kl2)\)\ \@kl3\ \((kl3 + 
                        kl4)\)\ \[Pi]\^2)\) + \((delta[\(-kl1\) - kl2 + kr1 + 
                        kr2 + kr3 + kr4]\ d[kr4] ** sigma[1] ** zeta[s2] ** 
                      zeta[s1] ** a[kr3, s2] ** a[kr2, s1] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[1] ** 
                      Conjugate[d[kl2]])\)/\((16\ \@kr2\ \((kr1 + 
                        kr2)\)\ \@kr3\ \((kr3 + kr4)\)\ \[Pi]\^2)\))\))\)\)
*)
(*
kappa_5 term for nonzero transverse momentum.
*)
(*
This version leaves all of the integrations over momenta and position \
implicit.  Also, sums are implicit.
*)
(*
Phase factors put in by hand, keeping only the terms that contribute to \
leading order in N
*)
(*
\!\(xi[x_] := \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kf\ x\)\ muf\ sigma[1] \
** b[kf]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ ka\ x - \
\[ImaginaryI]\ kf\ x + I\ ka\ pperp[sumrr]/2 + I\ kf\ pperp[sumrr]\)\ kappas\ \
sigma[1] ** a[ka, sumrr] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) - \(0\ kappas\ \
sigma[1] ** Conjugate[a[ka, sumrr]] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kf\ x\)\ muf\ sigma[1] ** sigma[1] ** \
Conjugate[d[kf]]\)\/\(2\ \@\[Pi]\) - \(0\ kappas\ sigma[1] ** a[ka, sumrr] ** \
sigma[1] ** Conjugate[d[kf]]\)\/\(4\ \@ka\ \[Pi]\) - \(1\/\(4\ \@ka\ \
\[Pi]\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ ka\ x + \[ImaginaryI]\ kf\ x + \
I\ ka\ pperp[sumrr]/2 + I\ kf\ pperp[sumrr]\)\ kappas\ sigma[1] ** 
              Conjugate[a[ka, sumrr]] ** sigma[1] ** 
              Conjugate[
                d[kf]])\) - \(\(1\/\(4\ \@ka\ \[Pi]\)\)\((\[ExponentialE]\^\(\
\(-\[ImaginaryI]\)\ ka\ x - \[ImaginaryI]\ kf\ x + I\ ka\ pperp[rr]/2 + I\ kf\
\ pperp[rr]\)\ kappaa\ sigma[1] ** zeta[rr] ** a[ka, rr] ** 
            b[kf])\)\) + \(0\ kappaa\ sigma[1] ** zeta[rr] ** Conjugate[a[ka, \
rr]] ** b[kf]\)\/\(4\ \@ka\ \[Pi]\) - \(1\/\(4\ \@ka\ \[Pi]\)\) \((0\ kappaa\ \
sigma[1] ** zeta[rr] ** a[ka, rr] ** sigma[1] ** 
              Conjugate[
                d[kf]])\) + \(1\/\(4\ \@ka\ \[Pi]\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ ka\ x + \[ImaginaryI]\ kf\ x + I\ ka\ pperp[rr]/2 + I\ kf\ \
pperp[rr]\)\ kappaa\ sigma[1] ** zeta[rr] ** Conjugate[a[ka, rr]] ** 
              sigma[1] ** Conjugate[d[kf]])\)\)
*)
cxi[x_] = 
  Conjugate[xi[x]] /. {Conjugate[pperp[a_]] -> pperp[a], Conjugate[x] -> x, 
            Conjugate[ka] -> ka, Conjugate[kf] -> kf, 
            Conjugate[ka^n_] -> ka^n} /. {ka -> kka, kf -> kks, rr -> ll, 
          sumrr -> sumll} //. prodexpand //. sigmazeta
(*
\!\(\(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kks\ x\)\ muf\ d[kks]\)\/\(2\ \@\
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kks\ x\)\ muf\ \
Conjugate[b[kks]] ** sigma[1]\)\/\(2\ \@\[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x - 1\/2\ \[ImaginaryI]\ kka\ \
pperp[sumll] - \[ImaginaryI]\ kks\ pperp[sumll]\)\ kappas\ d[kks] ** a[kka, \
sumll]\)\/\(4\ \@kka\ \[Pi]\) - \(\(1\/\(4\ \@kka\ \
\[Pi]\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ x \
- 1\/2\ \[ImaginaryI]\ kka\ pperp[sumll] - \[ImaginaryI]\ kks\ pperp[sumll]\)\
\ kappas\ Conjugate[b[kks]] ** sigma[1] ** 
          Conjugate[
            a[kka, sumll]])\)\) - \(1\/\(4\ \@kka\ \[Pi]\)\) \((\
\[ExponentialE]\^\(\[ImaginaryI]\ kka\ x + \[ImaginaryI]\ kks\ x - 1\/2\ \
\[ImaginaryI]\ kka\ pperp[ll] - \[ImaginaryI]\ kks\ pperp[ll]\)\ kappaa\ \
Conjugate[b[kks]] ** zeta[ll] ** sigma[1] ** 
            Conjugate[
              a[kka, ll]])\) + \(\(1\/\(4\ \@kka\ \[Pi]\)\)\((\[ExponentialE]\
\^\(\(-\[ImaginaryI]\)\ kka\ x - \[ImaginaryI]\ kks\ x - 1\/2\ \[ImaginaryI]\ \
kka\ pperp[ll] - \[ImaginaryI]\ kks\ pperp[ll]\)\ kappaa\ d[kks] ** sigma[1] ** 
          zeta[ll] ** sigma[1] ** a[kka, ll])\)\)\)
*)
(*
Find pseudoscalar 4-fermion interaction to leading order, acts on pion
*)
(*
The chiral four-fermion interaction in the Hamiltonian is kappa5 qq**qq/2 \
where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** sigma[2] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** sigma[2] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 \
+ kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\
\ kl2\ \[Pi]\) + \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - \
kr1)\)\ x\)\ \((kl1 - kr1)\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \
\@2\ kl1\ kr1\ \[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 - kr1)\)\ x\)\ \((kl1 - kr1)\)\ d[kr1] ** sigma[3] ** \
Conjugate[d[kl1]]\)\/\(2\ \@2\ kl1\ kr1\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \[Pi]\))\) + 
    kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ kl1 \
+ kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x \
+ 1\/2\ \[ImaginaryI]\ \((2\ kr1 + kr2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** 
                  sigma[3] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x \
- 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** 
                  sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x \
+ 1\/2\ \[ImaginaryI]\ \((kl1 + 2\ kl2)\)\ pperp[s1]\)\ d[kr1] ** sigma[3] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x \
- 1\/2\ \[ImaginaryI]\ \((kr1 + 2\ kr2)\)\ pperp[s1]\)\ d[kr2] ** sigma[3] ** 
                  a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ kl1 \
+ kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x \
+ 1\/2\ \[ImaginaryI]\ \((2\ kr1 + kr2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[3] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\) + \((\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x \
- 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[3] ** Conjugate[a[kl2, s1]] ** 
                  b[kr1])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ kr1\ x + 1\/2\ \
\[ImaginaryI]\ \((kl1 + 2\ kl2)\)\ pperp[s1]\)\ d[kr1] ** sigma[1] ** 
                  zeta[s1] ** sigma[2] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[d[kl2]])\)/\((4\ \@2\ \@kl1\ \((kl1 + 
                    kl2)\)\ \[Pi]\^\(3/2\))\) - \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((kr1 + 2\ kr2)\)\ pperp[s1]\)\ d[kr2] ** sigma[1] ** 
                  zeta[s1] ** sigma[2] ** a[kr1, s1] ** 
                  Conjugate[d[kl1]])\)/\((4\ \@2\ \@kr1\ \((kr1 + 
                    kr2)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. 
    a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(muf\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 \
+ kl2)\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ kl1\
\ kl2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ \
x\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \@2\ kr1\ kr2\ \
\[Pi]\))\) + 
    kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ kl1 \
+ kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\) + 
    kappaa\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ kl1 \
+ kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + \
kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((4\ \@2\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + 
                    kl3)\)\ \[Pi]\^\(3/2\))\) + \
\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\
\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((4\ \@2\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^\(3/2\))\))\)\)
*)
(*
 (*put in the coupling constant later*)expr1pinch = 
  Collect[relabel[
        Map[Distribute, 
              qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1, 
                  kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
            Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
        kappas}] //. prodfactor
*)
(*
\!\(muf\^2\ \((\(1\/\(8\ kl1\ kl2\ kl3\ kl4\ \[Pi]\^2\)\) \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x + \[ImaginaryI]\ \
\((kl3 + kl4)\)\ x\)\ \((kl1 + kl2)\)\ \((kl3 + kl4)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                  sigma[2] ** 
                  Conjugate[
                    d[kl4]])\) + \(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \[Pi]\^2\)\) \
\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \
\((kr1 + kr2)\)\ x\)\ \((kl1 + kl2)\)\ \((kr1 + kr2)\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                  b[kr1])\) + \(1\/\(8\ kl1\ kl2\ kr1\ kr2\ \[Pi]\^2\)\) \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - \[ImaginaryI]\ \((kr1 \
+ kr2)\)\ x\)\ \((kl1 + kl2)\)\ \((kr1 + kr2)\)\ d[kr2] ** sigma[2] ** 
                  b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[
                    d[kl2]])\) + \(1\/\(8\ kr1\ kr2\ kr3\ kr4\ \[Pi]\^2\)\) \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - \[ImaginaryI]\ \
\((kr3 + kr4)\)\ x\)\ \((kr1 + kr2)\)\ \((kr3 + kr4)\)\ d[kr4] ** sigma[2] ** 
                  b[kr3] ** d[kr2] ** sigma[2] ** b[kr1])\))\) + 
    muf\ \((kappas\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + kl5)\)\ \
x - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kl4 + kl5)\)\ Conjugate[
                          b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** Conjugate[b[kl4]] ** sigma[2] ** 
                        Conjugate[d[kl5]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + 
                          kl3)\)\ kl4\ kl5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ Conjugate[
                          b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((2\ kl3 + kl4)\)\ pperp[s1]\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ \((x + pperp[s1])\)\)\)\ \((kl3 + 
                                kl4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ x\)\ \((kl4 + kl5)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                        sigma[2] ** Conjugate[a[kl4, s1]] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl1\ kl2\ \@kl4\ \((kl3 + 
                          kl4)\)\ \((kl4 + 
                          kl5)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + \
kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[
                          b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** d[kr3] ** 
                        sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ d[kr2] ** 
                        sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + 
                          kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                        sigma[2] ** a[kr2, s1] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr3 + kr4 + kr5)\)\ x + \((kr4 + 2\ kr5)\)\ \
pperp[s1])\)\)\ \((kr1 + 
                          kr2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ x\)\)\ \((kr3 + 
                                kr4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ \((x + pperp[s1])\)\)\ \((kr4 + kr5)\))\)\ d[kr5] ** 
                        sigma[2] ** a[kr4, s1] ** b[kr3] ** d[kr2] ** 
                        sigma[2] ** 
                        b[kr1])\)/\((16\ kr1\ kr2\ \@kr4\ \((kr3 + 
                          kr4)\)\ \((kr4 + 
                          kr5)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr4 + kr5)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\
\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ \((kr4 + 
                          kr5)\)\ d[kr5] ** sigma[2] ** b[kr4] ** d[kr3] ** 
                        sigma[2] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ kr4\ kr5\ \[Pi]\^\(5/2\))\))\) + 
          kappaa\ \((\((\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x \
- 1\/2\ \[ImaginaryI]\ \((2\ kl3 + kl4)\)\ pperp[s1]\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ \((x + pperp[s1])\)\)\)\ \((kl3 + 
                                kl4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl3 + kl4 + kl5)\)\ x\)\ \((kl4 + kl5)\))\)\ Conjugate[b[kl1]] ** 
                        sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
                        zeta[s1] ** sigma[2] ** Conjugate[a[kl4, s1]] ** 
                        Conjugate[d[kl5]])\)/\((16\ kl1\ kl2\ \@kl4\ \((kl3 + 
                          kl4)\)\ \((kl4 + 
                          kl5)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + \
kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[
                          b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** d[kr3] ** 
                        sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \((\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl4 + kl5)\)\ x - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kl4 + kl5)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                        Conjugate[b[kl4]] ** sigma[2] ** 
                        Conjugate[d[kl5]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + 
                          kl3)\)\ kl4\ kl5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ Conjugate[
                          b[kl1]] ** zeta[s1] ** sigma[2] ** 
                        Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** d[kr2] ** 
                        sigma[2] ** 
                        b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                          kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ \((\(-\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + pperp[s1])\)\)\)\ \((kl1 + 
                                kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl1 + kl2 + kl3)\)\ x\)\ \((kl2 + kl3)\))\)\ \((kr1 + kr2)\)\ d[kr2] ** 
                        sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                        sigma[2] ** Conjugate[a[kl2, s1]] ** 
                        Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                          kl2)\)\ \((kl2 + 
                          kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2)\)\ x - 1\/2\ \[ImaginaryI]\ \
\((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s1])\)\)\ \((kl1 + 
                          kl2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** 
                        sigma[2] ** zeta[s1] ** a[kr2, s1] ** b[kr1] ** 
                        Conjugate[b[kl1]] ** sigma[2] ** 
                        Conjugate[d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                          kr2)\)\ \((kr2 + 
                          kr3)\)\ \[Pi]\^\(5/2\))\) + \
\((\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr4 + kr5)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + kr2 + kr3)\)\
\ x\)\)\ \((kr1 + 
                                kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ \((kr4 + 
                          kr5)\)\ d[kr5] ** sigma[2] ** b[kr4] ** d[kr3] ** 
                        sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                        b[kr1])\)/\((16\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                          kr3)\)\ kr4\ kr5\ \[Pi]\^\(5/2\))\) + \((\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \((kr1 + kr2)\)\ x - 1\/2\ \
\[ImaginaryI]\ \((4\ \((kr3 + kr4 + kr5)\)\ x + \((kr4 + 2\ kr5)\)\ \
pperp[s1])\)\)\ \((kr1 + 
                          kr2)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ x\)\)\ \((kr3 + 
                                kr4)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr3 + kr4 + kr5)\)\ \((x + pperp[s1])\)\)\ \((kr4 + kr5)\))\)\ d[kr5] ** 
                        sigma[2] ** zeta[s1] ** a[kr4, s1] ** b[kr3] ** 
                        d[kr2] ** sigma[2] ** 
                        b[kr1])\)/\((16\ kr1\ kr2\ \@kr4\ \((kr3 + 
                          kr4)\)\ \((kr4 + kr5)\)\ \[Pi]\^\(5/2\))\))\))\) + 
    kappas\^2\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                          kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + \
kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  Conjugate[b[kl4]] ** sigma[2] ** Conjugate[a[kl5, s1]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + kl5)\)\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                          kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                          kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 + \
kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** sigma[2] ** 
                  a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\))\) + 
    kappaa\ kappas\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((2\ kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                          kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + \
kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                  Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  Conjugate[b[kl4]] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl5, s1]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + kl5)\)\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((2\ kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ \
kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ \((x + pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                          kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + \
kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** zeta[s2] ** 
                  sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  Conjugate[b[kl4]] ** sigma[2] ** Conjugate[a[kl5, s1]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + kl5)\)\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                  zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** zeta[s1] ** 
                  sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  zeta[s2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                          kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                          kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 + \
kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** sigma[2] ** 
                  a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                          kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                          kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 + \
kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** sigma[2] ** 
                  zeta[s2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\))\) + 
    kappaa\^2\ \((\((\[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((2\ \
kl4 + kl5)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ \
pperp[s2]\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \
\((x + pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kl4 + kl5 + kl6)\)\ \((x + pperp[s1])\)\)\)\ \((kl4 + 
                          kl5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl4 + \
kl5 + kl6)\)\ x\)\ \((kl5 + kl6)\))\)\ Conjugate[b[kl1]] ** zeta[s2] ** 
                  sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                  Conjugate[b[kl4]] ** zeta[s1] ** sigma[2] ** 
                  Conjugate[a[kl5, s1]] ** 
                  Conjugate[d[kl6]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kl5\ \((kl4 + kl5)\)\ \((kl5 + 
                    kl6)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s2]\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s2])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + kr3)\))\)\ Conjugate[b[kl1]] ** 
                  zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                  Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                  a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                    kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1] - 1\/2\ \[ImaginaryI]\ \((4\ \
\((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ \((\(-\
\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + kl2 + kl3)\)\ \((x + \
pperp[s1])\)\)\)\ \((kl1 + 
                          kl2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 + \
kl2 + kl3)\)\ x\)\ \((kl2 + 
                          kl3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr1 + kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s2])\)\)\ \((kr2 + kr3)\))\)\ d[kr3] ** sigma[2] ** 
                  zeta[s2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                  zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                  Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                    kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \[Pi]\^3)\) + \((\[ExponentialE]\^\(\(-\(1\/2\)\)\
\ \[ImaginaryI]\ \((4\ \((kr1 + kr2 + kr3)\)\ x + \((kr2 + 2\ kr3)\)\ \
pperp[s1])\) - 1\/2\ \[ImaginaryI]\ \((4\ \((kr4 + kr5 + kr6)\)\ x + \((kr5 + \
2\ kr6)\)\ pperp[s2])\)\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ x\)\)\ \((kr1 + 
                          kr2)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr1 + \
kr2 + kr3)\)\ \((x + pperp[s1])\)\)\ \((kr2 + 
                          kr3)\))\)\ \((\(-\[ExponentialE]\^\(\[ImaginaryI]\ \
\((kr4 + kr5 + kr6)\)\ x\)\)\ \((kr4 + 
                          kr5)\) - \[ExponentialE]\^\(\[ImaginaryI]\ \((kr4 + \
kr5 + kr6)\)\ \((x + pperp[s2])\)\)\ \((kr5 + kr6)\))\)\ d[kr6] ** sigma[2] ** 
                  zeta[s2] ** a[kr5, s2] ** b[kr4] ** d[kr3] ** sigma[2] ** 
                  zeta[s1] ** a[kr2, s1] ** 
                  b[kr1])\)/\((32\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                    kr3)\)\ \@kr5\ \((kr4 + kr5)\)\ \((kr5 + 
                    kr6)\)\ \[Pi]\^3)\))\)\)
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  expr = kappa5/2*int[expr1pinch, {x}] //. intsumexpand /. 
            Exp[y_] :> Exp[Collect[y, x]] /. {int[Exp[a_ b_ + c_], {b_}] :> 
              2 Pi delta[I a] Exp[c] /; FreeQ[c, b], 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a]} /. {Exp[a_] :> 
            Exp[Simplify[a]], delta[a_] :> delta[Expand[a]]} /. 
      delta[_?neverzero] -> 0
*)
(*
\!\(1\/2\ kappa5\ \((muf\^2\ \((\(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \
\((\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + 
                        kr2]\ Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[d[kl2]] ** d[kr2] ** sigma[2] ** 
                      b[kr1])\) + \(1\/\(4\ kl1\ kl2\ kr1\ kr2\ \[Pi]\)\) \((\
\((kl1 + kl2)\)\ \((kr1 + kr2)\)\ delta[\(-kl1\) - kl2 + kr1 + kr2]\ d[kr2] ** 
                      sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
                      Conjugate[d[kl2]])\))\) + 
        kappas\^2\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                              kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                      sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                      d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^3)\) + \((\((2\ \
\[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \
\((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - 
                              kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr1\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                      sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^3)\))\) + 
        muf\ \((kappas\ \((\((\((kr1 + 
                              kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                    kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2])\)\ Conjugate[
                              b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]] ** d[kr2] ** sigma[2] ** 
                            b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\((kl1 \
+ kl2)\)\ \((\(-2\)\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 \
+ 2\ kr3)\)\ pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 + kr1 + kr2 + kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3])\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                            Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                            a[kr2, s1] ** 
                            b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(5/2\))\) + \((\((kr1 + 
                              kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                    kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2])\)\ d[kr2] ** 
                            sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[2] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\((kl1 \
+ kl2)\)\ \((\(-2\)\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 \
+ 2\ kr3)\)\ pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 + kr1 + kr2 + kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3])\)\ d[kr3] ** sigma[2] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[2] ** 
                            Conjugate[
                              d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^\(5/2\))\))\) + 
              kappaa\ \((\((\((kl1 + 
                              kl2)\)\ \((\(-2\)\ \[ExponentialE]\^\(\(-\(1\/2\
\)\)\ \[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr1\ \[Pi]\ \
delta[\(-kl1\) - kl2 + kr1 + kr2 + kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 + kr1 + kr2 + kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3])\)\ Conjugate[b[kl1]] ** sigma[2] ** 
                            Conjugate[d[kl2]] ** d[kr3] ** sigma[2] ** 
                            zeta[s1] ** a[kr2, s1] ** 
                            b[kr1])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(5/2\))\) + \((\((kr1 + 
                              kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \
\[ImaginaryI]\ \((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                    kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2])\)\ Conjugate[
                              b[kl1]] ** zeta[s1] ** sigma[2] ** 
                            Conjugate[a[kl2, s1]] ** Conjugate[d[kl3]] ** 
                            d[kr2] ** sigma[2] ** 
                            b[kr1])\)/\((16\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\((kr1 \
+ kr2)\)\ \((\(-2\)\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((kl2 + 2\ kl3)\
\)\ pperp[s1]\)\ kl1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \
\((kl2 + 2\ kl3)\)\ pperp[s1]\)\ kl2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + 
                                    kr1 + kr2] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((2\ kl1 + kl2)\)\ pperp[s1]\)\ kl3\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 - kl3 + kr1 + kr2])\)\ d[kr2] ** 
                            sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                            Conjugate[d[kl3]])\)/\((16\ \@kl2\ \((kl1 + 
                              kl2)\)\ \((kl2 + 
                              kl3)\)\ kr1\ kr2\ \[Pi]\^\(5/2\))\) + \((\((kl1 \
+ kl2)\)\ \((\(-2\)\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \((kr2 \
+ 2\ kr3)\)\ pperp[s1]\)\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3] - 
                              2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((kr2 + 2\ kr3)\)\ pperp[s1]\)\ kr2\ \[Pi]\ delta[\(-kl1\) - 
                                    kl2 + kr1 + kr2 + kr3] - 
                              
                              2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((2\
\ kr1 + kr2)\)\ pperp[s1]\)\ kr3\ \[Pi]\ delta[\(-kl1\) - kl2 + kr1 + kr2 + 
                                    kr3])\)\ d[kr3] ** sigma[2] ** zeta[s1] ** 
                            a[kr2, s1] ** b[kr1] ** Conjugate[b[kl1]] ** 
                            sigma[2] ** 
                            Conjugate[
                              d[kl2]])\)/\((16\ kl1\ kl2\ \@kr2\ \((kr1 + 
                              kr2)\)\ \((kr2 + 
                              kr3)\)\ \[Pi]\^\(5/2\))\))\))\) + 
        kappaa\ kappas\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                              kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                      sigma[2] ** Conjugate[a[kl2, s2]] ** Conjugate[d[kl3]] ** 
                      d[kr3] ** sigma[2] ** zeta[s1] ** a[kr2, s1] ** 
                      b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^3)\) + \((\((2\ \[ExponentialE]\^\(\(-\
\(1\/2\)\)\ \[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ \
kl3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + 
                              kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** a[kr2, s1] ** 
                      b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^3)\) + \((\((2\ \
\[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \
\((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - 
                              kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr1\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                      sigma[2] ** a[kr2, s2] ** b[kr1] ** Conjugate[b[kl1]] ** 
                      zeta[s1] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^3)\) + \((\((2\ \
\[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \
\((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - 
                              kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr1\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                      sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** sigma[2] ** Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^3)\))\) + 
        kappaa\^2\ \((\((\((2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \
\[ImaginaryI]\ \((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ \
pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + 
                              kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] - \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr2\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((kr2 + 2\ kr3)\)\ pperp[s1] + \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] + \((kl2 + 2\ kl3)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((2\ \
kr1 + kr2)\)\ pperp[s1] - \((2\ kl1 + kl2)\)\ pperp[s2])\)\)\ kl3\ kr3\ \[Pi]\
\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ Conjugate[b[kl1]] ** 
                      zeta[s2] ** sigma[2] ** Conjugate[a[kl2, s2]] ** 
                      Conjugate[d[kl3]] ** d[kr3] ** sigma[2] ** zeta[s1] ** 
                      a[kr2, s1] ** 
                      b[kr1])\)/\((32\ \@kl2\ \((kl1 + kl2)\)\ \((kl2 + 
                        kl3)\)\ \@kr2\ \((kr1 + kr2)\)\ \((kr2 + 
                        kr3)\)\ \[Pi]\^3)\) + \((\((2\ \
\[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 + 2\ kl3)\)\ pperp[s1] - \
\((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - 
                              kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr1\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr1\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl1\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] - \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ kr2\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl2\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] + \((kr2 + 2\ kr3)\)\ pperp[s2])\)\)\ kl3\ \
kr2\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl1\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        
                        2\ \[ExponentialE]\^\(1\/2\ \[ImaginaryI]\ \((\((kl2 \
+ 2\ kl3)\)\ pperp[s1] + \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl2\ kr3\ \[Pi]\ \
delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3] + 
                        2\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \[ImaginaryI]\ \
\((\((2\ kl1 + kl2)\)\ pperp[s1] - \((2\ kr1 + kr2)\)\ pperp[s2])\)\)\ kl3\ \
kr3\ \[Pi]\ delta[\(-kl1\) - kl2 - kl3 + kr1 + kr2 + kr3])\)\ d[kr3] ** 
                      sigma[2] ** zeta[s2] ** a[kr2, s2] ** b[kr1] ** 
                      Conjugate[b[kl1]] ** zeta[s1] ** sigma[2] ** 
                      Conjugate[a[kl2, s1]] ** 
                      Conjugate[d[kl3]])\)/\((32\ \@kl2\ \((kl1 + 
                        kl2)\)\ \((kl2 + kl3)\)\ \@kr2\ \((kr1 + 
                        kr2)\)\ \((kr2 + kr3)\)\ \[Pi]\^3)\))\))\)\)
*)
(*
Other terms of the Hamiltonian we probably don't care about
*)
(*
Find scalar 4-fermion interaction to leading order, acts on a_0
*)
(*
This acts only on the lowest J^{P_1 C} = 0^{++} state, the a_0.  This state \
has a node in the wavefunction.
*)
(*
The chiral four-fermion interaction in the Hamiltonian is -kappa0 qq**qq/2 \
where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** sigma[1] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** sigma[1] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    Collect[relabel[
          Map[Distribute, 
                qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                    kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
              Map[Distribute, qqpinch] //. prodexpand], {muf, kappaa, 
          kappas}] //. prodfactor;
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa0 is the coupling constant*)expr2pinch = -kappa0/2*
    simpint[
      int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
          int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0,
       k11]
*)
(*
Find transverse axial-vector 4-fermion interaction to leading order, acts on \
a_1 J_z = -1,1
*)
(*
This appears to not act on the lowest state since it acts on a wavefunction \
with a node.
It acts on the J_z=-1,1 components of the a_1.
*)
(*
The vector four-fermion interaction in the Hamiltonian is kappa6 \
(qq**qq-rr**rr)/2 where qq is:
*)
(*
 qq[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** sigma[3] ** Integrate[xi[x], x] + 
                I Integrate[cxi[x], x] ** sigma[3] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
 rr[x_] = 
  Collect[relabel[
        Expand[I conju[x] ** Integrate[xi[x], x] - 
                I Integrate[cxi[x], x] ** u[x] //. prodexpand //. 
            sigmazeta]], {muf, kappas, kappaa}] //. prodfactor
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
rrpinch = 
  rr[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    Collect[relabel[
            Map[Distribute, 
                  qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
                Map[Distribute, qqpinch] //. prodexpand] - 
          relabel[Map[Distribute, 
                  rrpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                      kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}] ** 
                Map[Distribute, rrpinch] //. prodexpand], {muf, kappaa, 
          kappas}] //. prodfactor;
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(* kappa6 is the coupling constant*)expr2pinch = 
  kappa6/2*(simpint[
        int[expr1pinch, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11])
*)
(*
Find fermion interactions using explicit constrained
*)
(*
This version leaves all of the integrations over momenta and position \
implicit.  Also, sums are implicit.
*)
u[x_] := 1/
        Sqrt[2 Pi] (b[kf] Exp[-I kf x] + 
          sigma[1] ** Conjugate[d[kf]] Exp[I kf x]);
conju[x_] = 
    Conjugate[u[x]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks;
v[x_] := 1/
        Sqrt[2 Pi] (sigma[1] ** bb[kf] Exp[-I kf x] - 
          Conjugate[dd[kf]] Exp[I kf x]);
conjv[x_] = 
    Conjugate[v[x]] /. {Conjugate[x] -> x, Conjugate[kf] -> kf} /. kf -> kks;
m[x_, rho_] := 
    1/Sqrt[4 Pi] (a[ka, -rho] Exp[-I ka x] + 
            Conjugate[a[ka, rho]] Exp[I ka x])/Sqrt[ka];
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(conjv\)\" \
is similar to existing symbol \"\!\(conju\)\"."
*)
kappaa /: Conjugate[kappaa] := kappaa;
kappas /: Conjugate[kappas] := kappas;
kappa5 /: Conjugate[kappa5] := kappa5;
muf /: Conjugate[muf] := muf;
(*
This is to define the momenta variables to be *always * positive.
*)
kl1 /: Positive[kl1] = True; kl2 /: Positive[kl2] = True; kl3 /: 
  Positive[kl3] = True;
kl4 /: Positive[kl4] = True; kl5 /: Positive[kl5] = True; 
kl6 /: Positive[kl6] = True; 
kr1 /: Positive[kr1] = True; kr2 /: Positive[kr2] = True; kr3 /: 
  Positive[kr3] = True;
kr4 /: Positive[kr4] = True; kr5 /: Positive[kr5] = True; kr6 /: 
  Positive[kr6] = True;
(*
Find Yukawa terms
*)
(*
Terms in the Hamiltonian P^-
*)
(*
expr1 = Collect[
      relabel[Expand[-I/2 (-D[conjv[x], x] ** v[x] + conjv[x] ** D[v[x], x]) -
                       muf/Sqrt[2] (conjv[x] ** sigma[1] ** u[x] + 
                            conju[x] ** sigma[1] ** v[x]) - 
                      kappaa/Sqrt[
                            2] (-conju[x] ** zeta[rho] ** sigma[1] ** 
                                m[x, rho] ** v[x] + 
                            conjv[x] ** sigma[1] ** zeta[rho] ** m[x, rho] ** 
                              u[x]) + 
                      kappas/Sqrt[
                            2] (conju[x] ** sigma[1] ** m[x, sumss] ** v[x] + 
                            conjv[x] ** sigma[1] ** m[x, sumss] ** 
                              u[x]) /. {0 ** _ -> 0, _ ** 0 -> 0} //. 
                  prodexpand /. {zeta[rho_] ** ss_ ** 
                      a[c_, -rho_] -> -zeta[rho] ** ss ** a[c, rho], 
                  zeta[rho_] ** a[c_, -rho_] -> -zeta[rho] ** a[c, rho], 
                  a[b_, -sumss] -> a[b, sumss]} /. prodexpand //. 
            sigmazeta]], {muf, kappaa, kappas}] //. prodfactor
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + 
                kr1)\)\ Conjugate[bb[kl1]] ** 
              bb[kr1]\)\/\(4\ \[Pi]\)\)\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ \((kl1 - kr1)\)\ x\)\ \((kl1 + kr1)\)\ \
dd[kr1] ** Conjugate[dd[kl1]]\)\/\(4\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ \((kl1 + kl2)\)\ x\)\ \((kl1 - kl2)\)\ Conjugate[bb[kl1]] ** \
sigma[1] ** Conjugate[dd[kl2]]\)\/\(4\ \[Pi]\) + 
    muf\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x\)\ Conjugate[b[kl1]] ** 
                    bb[kr1]\)\/\(2\ \@2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** b[kr1]\
\)\/\(2\ \@2\ \[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x\)\ d[kr1] ** Conjugate[dd[kl1]]\)\/\(2\ \@2\ \[Pi]\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] \
** Conjugate[d[kl1]]\)\/\(2\ \@2\ \[Pi]\) + \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** sigma[1] ** Conjugate[dd[kl2]]\)\/\(2\ \@2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[d[kl2]]\)\/\(2\ \@2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[1] ** bb[kr1]\)\/\(2\ \@2\ \[Pi]\) + \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
b[kr1]\)\/\(2\ \@2\ \[Pi]\))\) + \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
\((kr1 + kr2)\)\ x\)\ \((kr1 - kr2)\)\ dd[kr2] ** sigma[1] ** bb[kr1]\)\/\(4\ \
\[Pi]\) + 
    kappas\ \((\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** a[kr2, s1] ** \
bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** a[kr2, s1] ** \
b[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[dd[kl2]]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\
\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(4\ \
\@2\ \@kr1\ \[Pi]\^\(3/2\)\) - \(\(1\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) - \(\(1\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** 
                Conjugate[a[kl2, s1]] ** 
                Conjugate[
                  dd[kl3]])\)\) + \(\(1\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** 
                a[kr1, s1] ** 
                Conjugate[
                  d[kl2]])\)\) + \(\(1\/\(4\ \@2\ \@kl2\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x + \[ImaginaryI]\ kl3\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** 
                Conjugate[a[kl2, s1]] ** 
                Conjugate[
                  d[kl3]])\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) + \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ kr3\ x\)\ d[kr3] ** sigma[1] ** a[kr2, s1] ** bb[kr1]\)\/\(4\ \
\@2\ \@kr2\ \[Pi]\^\(3/2\)\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \
\[ImaginaryI]\ kr3\ x\)\ dd[kr3] ** sigma[1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \
\@2\ \@kr2\ \[Pi]\^\(3/2\)\))\) + 
    kappaa\ \((\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** a[kr2, s1] ** 
                    bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\)\)\) + \
\(\(1\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[
                  b[kl1]] ** zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                bb[kr1])\)\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \
\[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) - \(\(1\
\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\
\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** 
                zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                b[kr1])\)\) + \(\(1\/\(4\ \@2\ \@kr1\ \[Pi]\^\(3/2\)\)\)\((\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) - \(1\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\)\) \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[b[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    dd[kl3]])\) + \(\(1\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[bb[kl1]] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  d[kl2]])\)\) - \(1\/\(4\ \@2\ \@kl2\ \[Pi]\^\(3/2\)\)\) \((\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \
\[ImaginaryI]\ kl3\ x\)\ Conjugate[bb[kl1]] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl2, s1]] ** 
                  Conjugate[
                    d[kl3]])\) + \(\(1\/\(4\ \@2\ \@kl1\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                Conjugate[a[kl1, s1]] ** 
                bb[kr1])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ \
x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x\)\ d[kr3] ** sigma[1] ** \
zeta[s1] ** a[kr2, s1] ** bb[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) + \
\(\(1\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\
\ kl1\ x - \[ImaginaryI]\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** 
                sigma[1] ** zeta[s1] ** Conjugate[a[kl1, s1]] ** 
                b[kr1])\)\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x \
- \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x\)\ dd[kr3] ** sigma[1] ** \
zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(4\ \@2\ \@kr2\ \[Pi]\^\(3/2\)\) - \(1\/\
\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** sigma[1] ** 
                  zeta[s1] ** sigma[1] ** Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\) + \(\(1\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl1]])\)\) + \(1\/\(4\ \@2\ \@kl1\ \[Pi]\^\(3/2\)\)\) \
\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \
\[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    d[kl2]])\) - \(\(1\/\(4\ \@2\ \@kr1\ \
\[Pi]\^\(3/2\)\)\)\((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]])\)\))\)\)
*)
(*
Since the delta[...] is removed in the following, the integration region can \
be complicated.  expr2 contians all of the interactions in the Hamiltonian.
*)
(*
expr2 = simpint[
    int[expr1, {x}] //. intsumexpand /. Exp[x_] :> Exp[Factor[x]] /. 
        int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. delta[_?neverzero] -> 0, 
    k11]
*)
(*
\!\(\(-\(1\/2\)\)\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ Conjugate[
          bb[kl1]] ** bb[kr1] + 
    muf\ \((\(-\(\(delta[\(-kl1\) + kr1]\ Conjugate[b[kl1]] ** 
                    bb[kr1]\)\/\@2\)\) - \(delta[\(-kl1\) + kr1]\ \
Conjugate[bb[kl1]] ** b[kr1]\)\/\@2 + \(delta[\(-kl1\) + kr1]\ d[kr1] ** \
Conjugate[dd[kl1]]\)\/\@2 + \(delta[\(-kl1\) + kr1]\ dd[kr1] ** \
Conjugate[d[kl1]]\)\/\@2)\) + 
    1\/2\ \((kl1 + kr1)\)\ delta[\(-kl1\) + kr1]\ dd[kr1] ** 
        Conjugate[dd[kl1]] + 
    kappas\ \((\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] \
** bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + kr1]\ \
Conjugate[b[kl1]] ** Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\(2\ \
\[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[bb[kl1]] ** a[kr2, s1] ** \
b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) - kl2 + kr1]\ \
Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** b[kr1]\)\/\(2\ \@kl2\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[dd[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + \
kr2]\ d[kr2] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + \
kr2]\ dd[kr2] ** a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\) - \(\(1\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + 
                  kr1]\ Conjugate[b[kl1]] ** sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** sigma[1] ** 
                a[kr1, s1] ** 
                Conjugate[
                  d[kl2]])\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** \
sigma[1] ** Conjugate[a[kl1, s1]] ** bb[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) \
- \(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] \
** b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\))\) + 
    kappaa\ \((\(-\(\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** a[kr2, s1] ** 
                    bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\)\)\) + \(\(1\/\(2\ \
\@kl2\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** 
                zeta[s1] ** Conjugate[a[kl2, s1]] ** 
                bb[kr1])\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ \
Conjugate[bb[kl1]] ** zeta[s1] ** a[kr2, s1] ** b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \
\[Pi]\)\) - \(\(1\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + 
                  kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** 
                Conjugate[a[kl2, s1]] ** 
                b[kr1])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  d[kl2]])\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                Conjugate[a[kl1, s1]] ** 
                bb[kr1])\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\)\((delta[\
\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** 
                Conjugate[a[kl1, s1]] ** 
                b[kr1])\)\) - \(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\) \
\((delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl1]])\)\) + \(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\) \
\((delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    d[kl2]])\) - \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]])\)\))\)\)
*)
(*
These are the 1->1 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
        Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
          0 /; Length[non] != 2 || Count[non, Conjugate[_]] != 1] /. {kl1 -> 
        k1, kr1 -> k1} /. delta[0] -> 1
*)
(*
\!\(\(-k1\)\ Conjugate[bb[k1]] ** bb[k1] + 
    muf\ \((\(-\(Conjugate[b[k1]] ** bb[k1]\/\@2\)\) - 
          Conjugate[bb[k1]] ** b[k1]\/\@2 + d[k1] ** Conjugate[dd[k1]]\/\@2 + 
          dd[k1] ** Conjugate[d[k1]]\/\@2)\) + 
    k1\ dd[k1] ** Conjugate[dd[k1]]\)
*)
(*
These are the 2->1 particle interactions
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 1]
*)
(*
\!\(kappas\ \((\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** a[kr2, s1] \
** bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(delta[\(-kl1\) + kr1 + kr2]\ \
Conjugate[bb[kl1]] ** a[kr2, s1] ** b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) - \
\(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** a[kr1, s1] ** Conjugate[dd[kl1]]\)\/\
\(2\ \@kr1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** \
a[kr1, s1] ** Conjugate[d[kl1]]\)\/\(2\ \@kr1\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** \
bb[kr1]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) + kr1 + kr2]\ \
dd[kr2] ** sigma[1] ** Conjugate[a[kl1, s1]] ** b[kr1]\)\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\))\) + 
    kappaa\ \((\(-\(\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[b[kl1]] ** 
                    zeta[s1] ** a[kr2, s1] ** 
                    bb[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\)\)\) + \
\(delta[\(-kl1\) + kr1 + kr2]\ Conjugate[bb[kl1]] ** zeta[s1] ** a[kr2, s1] ** \
b[kr1]\)\/\(2\ \@kr2\ \@\(2\ \[Pi]\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                Conjugate[a[kl1, s1]] ** 
                bb[kr1])\)\) + \(\(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\)\((delta[\
\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** 
                Conjugate[a[kl1, s1]] ** 
                b[kr1])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ d[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl1]])\)\) - \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) + kr1 + kr2]\ dd[kr2] ** sigma[1] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** Conjugate[d[kl1]])\)\))\)\)
*)
(*
These are the 1->2 particle interactions:
*)
(*
expr2 /. a_NonCommutativeMultiply :> 
    Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
      0 /; Length[non] != 3 || Count[non, Conjugate[_]] != 2]
*)
(*
\!\(kappas\ \((\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** \
Conjugate[a[kl2, s1]] ** bb[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) + \
\(delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** Conjugate[a[kl2, s1]] ** \
b[kr1]\)\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ \
d[kr1] ** Conjugate[a[kl1, s1]] ** Conjugate[dd[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \
\[Pi]\)\) - \(delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** Conjugate[a[kl1, s1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\) - \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** sigma[1] ** 
                a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** sigma[1] ** 
                a[kr1, s1] ** Conjugate[d[kl2]])\)\))\) + 
    kappaa\ \((\(\(1\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + 
                  kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                Conjugate[a[kl2, s1]] ** 
                bb[kr1])\)\) - \(\(1\/\(2\ \@kl2\ \@\(2\ \[Pi]\)\)\)\((delta[\
\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** 
                Conjugate[a[kl2, s1]] ** 
                b[kr1])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[b[kl1]] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  dd[kl2]])\)\) + \(\(1\/\(2\ \@kr1\ \@\(2\ \
\[Pi]\)\)\)\((delta[\(-kl1\) - kl2 + kr1]\ Conjugate[bb[kl1]] ** zeta[s1] ** 
                sigma[1] ** a[kr1, s1] ** 
                Conjugate[
                  d[kl2]])\)\) - \(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\) \
\((delta[\(-kl1\) - kl2 + kr1]\ d[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** 
                  Conjugate[
                    dd[kl2]])\) + \(1\/\(2\ \@kl1\ \@\(2\ \[Pi]\)\)\) \
\((delta[\(-kl1\) - kl2 + kr1]\ dd[kr1] ** sigma[1] ** zeta[s1] ** sigma[1] ** 
                  Conjugate[a[kl1, s1]] ** Conjugate[d[kl2]])\))\)\)
*)
(*
Testing
*)
(*
Terms in the Hamiltonian kappa_5/2 qq**qq
*)
(*
qq[x_] = relabel[
      Expand[ conjv[x] ** v[x] /. {0 ** _ -> 0, _ ** 0 -> 0} //. 
            prodexpand //. sigmazeta]] //. prodfactor
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[bb[kl1]] ** bb[kr1]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** \
Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** bb[kr1]\)\/\(2\ \
\[Pi]\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ Conjugate[bb[kl1]] ** sigma[1] ** 
              Conjugate[
                dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** \
bb[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
 (*put in the coupling constant later*)expr1pinch = 
  relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1, 
                kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** qqpinch //. 
        prodexpand] //. prodfactor
*)
(*
\!\(\(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[dd[kl2]] ** Conjugate[bb[kl3]] ** 
            sigma[1] ** 
            Conjugate[
              dd[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[1] ** 
            Conjugate[dd[kl2]] ** dd[kr2] ** sigma[1] ** 
            bb[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[1] ** bb[kr1] ** 
            Conjugate[bb[kl1]] ** sigma[1] ** 
            Conjugate[
              dd[kl2]])\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ dd[kr4] ** sigma[1] ** bb[kr3] ** dd[kr2] ** 
          sigma[1] ** bb[kr1])\)\)\)
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  Factor[kappa5/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /. 
              Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11]]
*)
(*
\!\(\(1\/\(4\ \[Pi]\)\) \((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
            kr2]\ \((Conjugate[bb[kl1]] ** sigma[1] ** Conjugate[dd[kl2]] ** 
              dd[kr2] ** sigma[1] ** bb[kr1] + 
            dd[kr2] ** sigma[1] ** bb[kr1] ** Conjugate[bb[kl1]] ** sigma[1] ** 
              Conjugate[dd[kl2]])\))\)\)
*)
(*
Find gamma^+ gamma^5, acts on pion
*)
(*
Terms in the Hamiltonian kappa_5/2 qq**qq
*)
(*
qq[x_] = relabel[
      Expand[ conju[x] ** sigma[3] ** u[x] /. {0 ** _ -> 0, _ ** 0 -> 0} //. 
            prodexpand //. sigmazeta]] //. prodfactor
*)
(*
\!\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[b[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; 
          Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\
\ kl2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \
\[Pi]\) - \(\[ImaginaryI]\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
 (*put in the coupling constant later*)expr1pinch = 
  relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1, 
                kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** qqpinch //. 
        prodexpand] //. prodfactor
*)
(*
\!\(\(-\(\(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** Conjugate[b[kl3]] ** 
              sigma[2] ** 
              Conjugate[
                d[kl4]])\)\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
          d[kr2] ** sigma[2] ** 
          b[kr1])\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
          sigma[2] ** 
          Conjugate[
            d[kl2]])\)\) - \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ d[kr4] ** sigma[2] ** b[kr3] ** d[kr2] ** sigma[2] ** 
          b[kr1])\)\)\)
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  Factor[kappa5/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /. 
              Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11]]
*)
(*
\!\(\(1\/\(4\ \[Pi]\)\) \((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
            kr2]\ \((Conjugate[b[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
              d[kr2] ** sigma[2] ** b[kr1] + 
            d[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[d[kl2]])\))\)\)
*)
(*
Find pseudoscalar 4-fermion interaction, acts on pion
*)
(*
Terms in the Hamiltonian kappa_5/2 qq**qq
*)
(*
qq[x_] = relabel[
      Expand[ conjv[x] ** sigma[2] ** u[x] + 
                conju[x] ** sigma[2] ** v[x] /. {0 ** _ -> 0, _ ** 0 -> 
                  0} //. prodexpand //. sigmazeta]] //. prodfactor
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[
                dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ \
Conjugate[b[kl1]] ** sigma[3] ** bb[kr1]\)\/\(2\ \[Pi]\) - \
\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** sigma[3] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[2] ** bb[kr1]\)\/\(2\ \[Pi]\) + \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
sigma[3] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** \
b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[
                dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
sigma[2] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** \
bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
 (*put in the coupling constant later*)expr1pinch = 
  relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1, 
                kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** qqpinch //. 
        prodexpand] //. prodfactor
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl1\)\" \
is similar to existing symbol \"\!\(kl1\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl2\)\" \
is similar to existing symbol \"\!\(kl2\)\"."
*)
(*
General::"spell1": "Possible spelling error: new symbol name \"\!\(kkl3\)\" \
is similar to existing symbol \"\!\(kl3\)\"."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell1\"\) will be \
suppressed during this calculation."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr1\)\" is \
similar to existing symbols \!\({kkl1, kr1}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr2\)\" is \
similar to existing symbols \!\({kkl2, kr2}\)."
*)
(*
General::"spell": "Possible spelling error: new symbol name \"\!\(kkr3\)\" is \
similar to existing symbols \!\({kkl3, kr3}\)."
*)
(*
General::"stop": "Further output of \!\(General :: \"spell\"\) will be \
suppressed during this calculation."
*)
(*
\!\(\(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \
\[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \[ImaginaryI]\ kl4\ x\)\ \
Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** Conjugate[b[kl3]] ** 
            sigma[2] ** 
            Conjugate[
              dd[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
            Conjugate[bb[kl3]] ** sigma[2] ** 
            Conjugate[
              d[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
            d[kr2] ** sigma[2] ** 
            bb[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
            dd[kr2] ** sigma[2] ** 
            b[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
            Conjugate[b[kl3]] ** sigma[2] ** 
            Conjugate[
              dd[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x + \[ImaginaryI]\ kl3\ x + \
\[ImaginaryI]\ kl4\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
            Conjugate[bb[kl3]] ** sigma[2] ** 
            Conjugate[
              d[kl4]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
            d[kr2] ** sigma[2] ** 
            bb[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** 
            dd[kr2] ** sigma[2] ** 
            b[kr1])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[b[kl1]] ** 
            sigma[2] ** 
            Conjugate[
              dd[kl2]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[bb[kl1]] ** 
            sigma[2] ** 
            Conjugate[
              d[kl2]])\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ d[kr4] ** sigma[2] ** bb[kr3] ** d[kr2] ** sigma[2] ** 
          bb[kr1])\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ d[kr4] ** sigma[2] ** bb[kr3] ** dd[kr2] ** 
          sigma[2] ** 
          b[kr1])\)\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** 
            sigma[2] ** 
            Conjugate[
              dd[kl2]])\) + \(1\/\(4\ \[Pi]\^2\)\) \((\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x - \[ImaginaryI]\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[bb[kl1]] ** 
            sigma[2] ** 
            Conjugate[
              d[kl2]])\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ dd[kr4] ** sigma[2] ** b[kr3] ** d[kr2] ** sigma[2] ** 
          bb[kr1])\)\) + \(\(1\/\(4\ \[Pi]\^2\)\)\((\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x - \[ImaginaryI]\ kr3\ x - \
\[ImaginaryI]\ kr4\ x\)\ dd[kr4] ** sigma[2] ** b[kr3] ** dd[kr2] ** 
          sigma[2] ** b[kr1])\)\)\)
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  Factor[kappa5/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /. 
              Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11]]
*)
(*
\!\(\(1\/\(4\ \[Pi]\)\) \((kappa5\ delta[\(-kl1\) - kl2 + kr1 + 
            kr2]\ \((Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** 
              d[kr2] ** sigma[2] ** bb[kr1] + 
            Conjugate[b[kl1]] ** sigma[2] ** Conjugate[dd[kl2]] ** dd[kr2] ** 
              sigma[2] ** b[kr1] + 
            Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** d[kr2] ** 
              sigma[2] ** bb[kr1] + 
            Conjugate[bb[kl1]] ** sigma[2] ** Conjugate[d[kl2]] ** dd[kr2] ** 
              sigma[2] ** b[kr1] + 
            d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[dd[kl2]] + 
            d[kr2] ** sigma[2] ** bb[kr1] ** Conjugate[bb[kl1]] ** sigma[2] ** 
              Conjugate[d[kl2]] + 
            dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[2] ** 
              Conjugate[dd[kl2]] + 
            dd[kr2] ** sigma[2] ** b[kr1] ** Conjugate[bb[kl1]] ** sigma[2] ** 
              Conjugate[d[kl2]])\))\)\)
*)
(*
Find transverse vector 4-fermion interaction, acts on J_z = -1,+1 of a rho
*)
(*
Terms in the Hamiltonian -kappa_6/2 (qq**qq-rr**rr)
*)
(*
qq[x_] = relabel[
      Expand[ conjv[x] ** sigma[3] ** u[x] + 
                conju[x] ** sigma[3] ** v[x] /. {0 ** _ -> 0, _ ** 0 -> 
                  0} //. prodexpand //. sigmazeta]] //. prodfactor
*)
(*
\!\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\
\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[2] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) - \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[2] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) + \(\
\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ d[kr1] ** sigma[2] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
d[kr2] ** sigma[3] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ImaginaryI]\ \
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** \
sigma[2] ** Conjugate[d[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[3] ** \
b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
rr[x_] = relabel[
      Expand[ conjv[x] ** u[x] - 
                conju[x] ** v[x] /. {0 ** _ -> 0, _ ** 0 -> 0} //. 
            prodexpand //. sigmazeta]] //. prodfactor
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
dd[kr2] ** b[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ \
x - \[ImaginaryI]\ kr1\ x\)\ Conjugate[b[kl1]] ** sigma[1] ** bb[kr1]\)\/\(2\ \
\[Pi]\) + \(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ \
x\)\ Conjugate[bb[kl1]] ** sigma[1] ** b[kr1]\)\/\(2\ \[Pi]\) + \(\
\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ d[kr1] ** \
sigma[1] ** Conjugate[dd[kl1]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x - \[ImaginaryI]\ kr1\ x\)\ dd[kr1] ** sigma[1] ** \
Conjugate[d[kl1]]\)\/\(2\ \[Pi]\)\)
*)
(*
Select terms that are leading order in N for mesons ("pinching" terms):
*)
(*
qqpinch = 
  qq[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(-\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\
\ Conjugate[b[kl1]] ** sigma[3] ** 
              Conjugate[
                dd[kl2]]\)\/\(2\ \[Pi]\)\)\) - \(\[ExponentialE]\^\(\
\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
sigma[3] ** Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\
\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** sigma[3] ** \
bb[kr1]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \
\[ImaginaryI]\ kr2\ x\)\ dd[kr2] ** sigma[3] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
rrpinch = 
  rr[x] /. a_NonCommutativeMultiply :> 
      Block[{non = a //. {sigma[_] ** c_ -> c, zeta[_] ** c_ -> c}}, 
        0 /; Count[non, Conjugate[_]] != Length[non] && 
            Count[non, Conjugate[_]] != 0]
*)
(*
\!\(\(\[ExponentialE]\^\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ \
Conjugate[b[kl1]] ** Conjugate[dd[kl2]]\)\/\(2\ \[Pi]\) + \(\[ExponentialE]\^\
\(\[ImaginaryI]\ kl1\ x + \[ImaginaryI]\ kl2\ x\)\ Conjugate[bb[kl1]] ** \
Conjugate[d[kl2]]\)\/\(2\ \[Pi]\) - \(\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \
kr1\ x - \[ImaginaryI]\ kr2\ x\)\ d[kr2] ** bb[kr1]\)\/\(2\ \[Pi]\) - \(\
\[ExponentialE]\^\(\(-\[ImaginaryI]\)\ kr1\ x - \[ImaginaryI]\ kr2\ x\)\ \
dd[kr2] ** b[kr1]\)\/\(2\ \[Pi]\)\)
*)
(*
 (*put in the coupling constant later*)
  expr1pinch = 
    relabel[(qqpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, kr1 -> kkr1, 
                    kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** 
              qqpinch - (rrpinch /. {kl1 -> kkl1, kl2 -> kkl2, kl3 -> kkl3, 
                    kr1 -> kkr1, kr2 -> kkr2, kr3 -> kkr3, s1 -> ss1}) ** 
              rrpinch //. prodexpand] //. prodfactor;
*)
(*
expr2 contians all of the "pinching" interactions in the Hamiltonian.
*)
(*
(*kappa5 is the coupling constant*)expr2pinch = 
  Factor[-kappa6/2*
      simpint[int[expr1pinch, {x}] //. intsumexpand /. 
              Exp[x_] :> Exp[Factor[x]] /. 
            int[Exp[a_ b_], {b_}] -> 2 Pi delta[I a] /. 
          delta[_?neverzero] -> 0, k11]]
*)
(*
\!\(\(-\(\(1\/\(4\ \[Pi]\)\) \((kappa6\ delta[\(-kl1\) - kl2 + kr1 + 
              kr2]\ \((Conjugate[b[kl1]] ** Conjugate[dd[kl2]] ** d[kr2] ** 
                bb[kr1] + 
              Conjugate[b[kl1]] ** Conjugate[dd[kl2]] ** dd[kr2] ** b[kr1] + 
              Conjugate[bb[kl1]] ** Conjugate[d[kl2]] ** d[kr2] ** bb[kr1] + 
              Conjugate[bb[kl1]] ** Conjugate[d[kl2]] ** dd[kr2] ** b[kr1] + 
              d[kr2] ** bb[kr1] ** Conjugate[b[kl1]] ** Conjugate[dd[kl2]] + 
              d[kr2] ** bb[kr1] ** Conjugate[bb[kl1]] ** Conjugate[d[kl2]] + 
              dd[kr2] ** b[kr1] ** Conjugate[b[kl1]] ** Conjugate[dd[kl2]] + 
              dd[kr2] ** b[kr1] ** Conjugate[bb[kl1]] ** Conjugate[d[kl2]] + 
              Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]] ** d[kr2] ** 
                sigma[3] ** bb[kr1] + 
              Conjugate[b[kl1]] ** sigma[3] ** Conjugate[dd[kl2]] ** dd[kr2] ** 
                sigma[3] ** b[kr1] + 
              Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]] ** d[kr2] ** 
                sigma[3] ** bb[kr1] + 
              Conjugate[bb[kl1]] ** sigma[3] ** Conjugate[d[kl2]] ** dd[kr2] ** 
                sigma[3] ** b[kr1] + 
              d[kr2] ** sigma[3] ** bb[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                Conjugate[dd[kl2]] + 
              d[kr2] ** sigma[3] ** bb[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[3] ** Conjugate[d[kl2]] + 
              dd[kr2] ** sigma[3] ** b[kr1] ** Conjugate[b[kl1]] ** sigma[3] ** 
                Conjugate[dd[kl2]] + 
              dd[kr2] ** sigma[3] ** b[kr1] ** Conjugate[bb[kl1]] ** 
                sigma[3] ** Conjugate[d[kl2]])\))\)\)\)\)
*)
(*
Junk
*)
