(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    117634,       3698]*)
(*NotebookOutlinePosition[    118584,       3728]*)
(*  CellTagsIndexPosition[    118540,       3724]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Small Momentum Analysis for the Inchworm Model", "Subtitle"],

Cell["\<\
Brett van de Sande
August 7, 2001\
\>", "Subsubtitle"],

Cell["\<\
These are routines for solving for the functions f(w) in the \
endpoint analysis of the inchworm model.  Also, there is a routine for \
finding \[Alpha].\
\>", "Text"],

Cell[CellGroupData[{

Cell["Plot wavefunction (not used in paper)", "Section"],

Cell["\<\
This was too complicated for a paper, but may make a good slide in \
a talk?
Plot of an example wavefunction\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(psi[x_, y_] = 
      Block[{a = 0.25, \ beta = 0.4}, 
        x^a\ y^beta\ \((1 - x - y)\)^
              a/\((\((x + y)\)^\((beta + 1/2)\)\ \((1 - x)\)^\((beta + 
                      1/2)\))\)]\)], "Input"],

Cell[BoxData[
    \(\(x\^0.25`\ \((1 - x - y)\)\^0.25`\ y\^0.4`\)\/\(\((1 - x)\)\^0.9`\ \
\((x + y)\)\^0.9`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c1 = 
      Block[{y = 0.58}, 
        ParametricPlot3D[{x, y, 
            psi[x, y] + 0.01, {Thickness[0.01], Hue[0.6]}}, {x, 
            0,  .25}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c2 = 
      Block[{x = 0.5}, 
        ParametricPlot3D[{x, y, 
            psi[x, y] + 0.01, {Thickness[0.01], Hue[0.05]}}, {y, 
            0,  .25}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(c3 = 
      Block[{p = 0.25, x, y}, 
        ParametricPlot3D[
          Evaluate[x = w\ p; \ 
            y = \((1 - w)\)\ p; {x, y, 
              psi[x, y] + 0.01, {Thickness[0.01]}}], {w, 0, 1}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(s1 = 
      Block[{np = 20, x, y}, 
        ParametricPlot3D[
          Evaluate[x = w\ p; \ 
            y = \((1 - w)\) p; {x, y, 
              Which[x + y < 1/np\  || \ 1 - x\  < \ 0.5/np, \ 10, True, 
                psi[x, y]]}], {p, 0, 1}, {w, 0, 1}, 
          PlotRange \[Rule] {{0, 1 - 1/np}, {0, 1}, {0, 1.5}}, 
          PlotPoints \[Rule] np, 
          AxesEdge \[Rule] {{1, 1}, {\(-1\), 1}, {1, 1}}, 
          AxesLabel \[Rule] {"\<x\>", "\<y\>", "\<\>"}, 
          TextStyle \[Rule] {FontFamily \[Rule] "\<Times\>", 
              FontSize \[Rule] 11}, BoxRatios \[Rule] {1, 1, 0.5}, 
          ViewPoint \[Rule] {0.7, \(-1.4\), 1.6}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pp = Show[s1, c1, c2, c3]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Definitions", "Section"],

Cell[BoxData[
    \(matrix[kk_, muf2_, mub2_, gg2_, alpha_] := 
      Block[{w, q}, 
        Table[\[IndentingNewLine]w = 
            N[\((i - 1/2)\)/kk]; \[IndentingNewLine]\ 
          If[\[IndentingNewLine]i \[Equal] 
              j, \[IndentingNewLine]muf2/
                w + \((mub2 + gg2/Pi)\)/\((1 - w)\) + 
              gg2\ \((1 + w)\)/\((2\ Pi\ kk)\)\ Sum[
                  If[k \[NotEqual] i, q = \((k - 1/2)\)/kk; 1/\((q - w)\)^2, 
                    0], {k, kk}], \ 
            q = N[\((j - 1/2)\)/
                  kk]; \[IndentingNewLine]\(-gg2\)\ \((\((2 - w - 
                        q)\) + \((w + q - 2\ q\ w)\)\ \((w/q)\)^
                        alpha)\)/\((4\ Pi\ kk\ Sqrt[\((1 - w)\)\ \((1 - 
                            q)\)]\ \((w - 
                          q)\)^2)\)\[IndentingNewLine]], \
\[IndentingNewLine]{i\ , kk}, {j, kk}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(vectors[kk_, muf2_, mub2_, gg2_, alpha_] := 
      Block[{w}, 
        Table[w = N[\((i - 1/2)\)/kk]; \ 
          muf2\ {\((w + 1)\)/\ Sqrt[1 - w], Sqrt[1 - w]}/w - 
            muf2\ \((1 - w)\)/w - mub2 - gg2/Pi + 
            gg2/\((4\ Pi)\)*\((NIntegrate[
                    If[Abs[q - w] < 
                          10^\(-8\), \[IndentingNewLine]\((\(-4\)\ alpha\^2\ \
\((\(-1\) + w)\)\^2 + w + 8\ alpha\ \((\(-1\) + w)\)\ w - 
                              3\ w\^2)\)/\((4\ \((\(-1\) + 
                                  w)\)\ w)\), \(1\/\((q - w)\)\^2\) \((\(-2\)\
\ \((1 - w\^2)\) + \@\(\(1 - q\)\/\(1 - w\)\)\ \((2 - q - 
                                      w + \((w\/q)\)\^alpha\ \((q + w - 
                                        2\ q\ w)\))\))\) + \(1 + 2\ alpha\ \
\((1 - w)\) + 3\ w\)\/\(q - w\)\[IndentingNewLine]] - \((w/q)\)^
                          alpha/\((w\ Sqrt[1 - w])\), {q, 0, 
                      1}] - \((1 + 2\ alpha\ \((1 - w)\) + 
                        3\ w)\)\ Log[\((1 - w)\)/w] + 
                  w^\((alpha - 1)\)/\((\((1 - alpha)\)\ Sqrt[
                          1 - w])\))\), {i, kk}]]\)], "Input",
  InitializationCell->True],

Cell["\<\
muf is the \"canonical\" fermion mass and mu02 is the shift in \
(mass)^2 in the zero-link sector.\
\>", "Text"],

Cell[BoxData[
    \(findalpha[kk_, mub2_, gg2_, kappas_, kappaa_, muf_, mu02_] := 
      Block[{ft, alpha0, alpha1}, \[IndentingNewLine]alpha /. 
          FindRoot[\[IndentingNewLine]Print[alpha]; \[IndentingNewLine]ft = 
              LinearSolve[matrix[kk, muf^2, mub2, gg2, alpha], 
                vectors[kk, muf^2, mub2, gg2, alpha]]; 
            0 \[Equal] 
              muf^2 + mu02 - 
                gg2/\((2\ Pi)\)\ \((1 - 
                      Pi\ alpha\ Cot[Pi\ alpha])\) + {kappas^2, kappaa^2} . 
                    Sum[q = 
                        N[\((i - 1/2)\)/
                            kk]\ ; {1 - \((1 + 
                                  q)\)\ \((1 - q + ft[\([i, 1]\)])\)/
                                Sqrt[1 - q], 
                          1 - Sqrt[1 - q]\ \((1 - q + ft[\([i, 2]\)])\)}/
                        q, {i, kk}]/\((Pi\ kk)\), \[IndentingNewLine]{alpha, 
              0.1, 0.5}\[IndentingNewLine]]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculations for definitions", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = \((\((1 - 
                  q)\)\ \((\((2 - q - w)\) + \((w + q - 2\ q\ w)\)\ \((w/q)\)^
                        alpha)\)/Sqrt[\((1 - q)\)\ \((1 - w)\)] - 
            2\ \((1 + w)\)\ \((1 - w)\))\)/\((q - w)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(-2\)\ \((1 - w)\)\ \((1 + w)\) + \(\((1 - q)\)\ \((2 - q - w + \((w\
\/q)\)\^alpha\ \((q + w - 2\ q\ w)\))\)\)\/\@\(\((1 - q)\)\ \((1 - w)\)\)\)\/\
\((q - w)\)\^2\)], "Output"]
}, Open  ]],

Cell["Equivalently:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrand = \((\ \((\((2 - q - w)\) + \((w + q - 2\ q\ w)\)\ \((w/q)\)^
                      alpha)\)\ Sqrt[\((1 - q)\)/\ \((1 - w)\)] - 
            2\ \((1 - w^2)\))\)/\((q - w)\)^2\)], "Input"],

Cell[BoxData[
    \(\(\(-2\)\ \((1 - w\^2)\) + \@\(\(1 - q\)\/\(1 - w\)\)\ \((2 - q - w + \
\((w\/q)\)\^alpha\ \((q + w - 2\ q\ w)\))\)\)\/\((q - w)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pole = 
      Normal[Simplify[
          PowerExpand[Series[integrand, {q, w, \(-1\)}]]]]\)], "Input"],

Cell[BoxData[
    \(\(\(-1\) + 2\ alpha\ \((\(-1\) + w)\) - 3\ w\)\/\(q - w\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[pole, {q, 0, 1}, Assumptions \[Rule] {w > 0, w < 1}, 
      PrincipalValue \[Rule] True]\)], "Input"],

Cell[BoxData[
    \(\((\(-1\) + 2\ alpha\ \((\(-1\) + w)\) - 3\ w)\)\ \((Log[1 - w] - 
          Log[w])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Normal[
      Simplify[PowerExpand[Series[integrand - pole, {q, w, 0}]]]]\)], "Input"],

Cell[BoxData[
    \(\(\(-4\)\ alpha\^2\ \((\(-1\) + w)\)\^2 + w + 8\ alpha\ \((\(-1\) + \
w)\)\ w - 3\ w\^2\)\/\(4\ \((\(-1\) + w)\)\ w\)\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example calculations", "Section"],

Cell[BoxData[""], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[
      findalpha[50, 0.07642881742, 1, 0.359250796, \ 0.6160276139, \ 
        0.3349170391, \(-0.1651354888\)]]\)], "Input"],

Cell[BoxData[
    \(0.1`\)], "Print"],

Cell[BoxData[
    \(0.1`\)], "Print"],

Cell[BoxData[
    \(0.5`\)], "Print"],

Cell[BoxData[
    \(0.5`\)], "Print"],

Cell[BoxData[
    \(0.4958061198087839`\)], "Print"],

Cell[BoxData[
    \(0.4979600070466262`\)], "Print"],

Cell[BoxData[
    \(0.49796908272962004`\)], "Print"],

Cell[BoxData[
    \({29.48`\ Second, 0.49796908272962004`}\)], "Output"]
}, Open  ]],

Cell["Calculate a rescaled version of f_S(w) and f_A(w).", "Text"],

Cell["\<\
Block[{kk=200,muf2=1.65,mub2=0.5,gg2=N[4 Pi],alpha=0.4},
ft=LinearSolve[matrix[kk,muf2,mub2,gg2,alpha],vectors[kk,muf2,mub2,gg2,alpha]]\
;
fsscaled=Table[{k/kk,1/Sqrt[Pi]*(1-k/kk+ft[[k+1/2,1]])},{k,1/2,kk-1/2}];
fascaled=Table[{k/kk,-1/Sqrt[Pi]*(1-k/kk+ft[[k+1/2,2]])},{k,1/2,kk-1/2}];
]\
\>", "Input",
  PageWidth->Infinity,
  ShowSpecialCharacters->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(p1 = 
      ListPlot[fsscaled, \ Axes \[Rule] False, Frame \[Rule] True, 
        PlotRange \[Rule] {{0, 1}, {0, 1/Sqrt[Pi]}}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1 2.12233e-17 1.09544 [
[.2 -0.0125 -9 -9 ]
[.2 -0.0125 9 0 ]
[.4 -0.0125 -9 -9 ]
[.4 -0.0125 9 0 ]
[.6 -0.0125 -9 -9 ]
[.6 -0.0125 9 0 ]
[.8 -0.0125 -9 -9 ]
[.8 -0.0125 9 0 ]
[1 -0.0125 -3 -9 ]
[1 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .10954 -18 -4.5 ]
[-0.0125 .10954 0 4.5 ]
[-0.0125 .21909 -18 -4.5 ]
[-0.0125 .21909 0 4.5 ]
[-0.0125 .32863 -18 -4.5 ]
[-0.0125 .32863 0 4.5 ]
[-0.0125 .43817 -18 -4.5 ]
[-0.0125 .43817 0 4.5 ]
[-0.0125 .54772 -18 -4.5 ]
[-0.0125 .54772 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2 0 m
.2 .00625 L
s
[(0.2)] .2 -0.0125 0 1 Mshowa
.4 0 m
.4 .00625 L
s
[(0.4)] .4 -0.0125 0 1 Mshowa
.6 0 m
.6 .00625 L
s
[(0.6)] .6 -0.0125 0 1 Mshowa
.8 0 m
.8 .00625 L
s
[(0.8)] .8 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(1)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.05 0 m
.05 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.25 0 m
.25 .00375 L
s
.3 0 m
.3 .00375 L
s
.35 0 m
.35 .00375 L
s
.45 0 m
.45 .00375 L
s
.5 0 m
.5 .00375 L
s
.55 0 m
.55 .00375 L
s
.65 0 m
.65 .00375 L
s
.7 0 m
.7 .00375 L
s
.75 0 m
.75 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.95 0 m
.95 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .10954 m
.00625 .10954 L
s
[(0.1)] -0.0125 .10954 1 0 Mshowa
0 .21909 m
.00625 .21909 L
s
[(0.2)] -0.0125 .21909 1 0 Mshowa
0 .32863 m
.00625 .32863 L
s
[(0.3)] -0.0125 .32863 1 0 Mshowa
0 .43817 m
.00625 .43817 L
s
[(0.4)] -0.0125 .43817 1 0 Mshowa
0 .54772 m
.00625 .54772 L
s
[(0.5)] -0.0125 .54772 1 0 Mshowa
.125 Mabswid
0 .02191 m
.00375 .02191 L
s
0 .04382 m
.00375 .04382 L
s
0 .06573 m
.00375 .06573 L
s
0 .08763 m
.00375 .08763 L
s
0 .13145 m
.00375 .13145 L
s
0 .15336 m
.00375 .15336 L
s
0 .17527 m
.00375 .17527 L
s
0 .19718 m
.00375 .19718 L
s
0 .241 m
.00375 .241 L
s
0 .2629 m
.00375 .2629 L
s
0 .28481 m
.00375 .28481 L
s
0 .30672 m
.00375 .30672 L
s
0 .35054 m
.00375 .35054 L
s
0 .37245 m
.00375 .37245 L
s
0 .39436 m
.00375 .39436 L
s
0 .41627 m
.00375 .41627 L
s
0 .46008 m
.00375 .46008 L
s
0 .48199 m
.00375 .48199 L
s
0 .5039 m
.00375 .5039 L
s
0 .52581 m
.00375 .52581 L
s
0 .56963 m
.00375 .56963 L
s
0 .59154 m
.00375 .59154 L
s
0 .61344 m
.00375 .61344 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
.2 .61178 m
.2 .61803 L
s
.4 .61178 m
.4 .61803 L
s
.6 .61178 m
.6 .61803 L
s
.8 .61178 m
.8 .61803 L
s
.125 Mabswid
.05 .61428 m
.05 .61803 L
s
.1 .61428 m
.1 .61803 L
s
.15 .61428 m
.15 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.3 .61428 m
.3 .61803 L
s
.35 .61428 m
.35 .61803 L
s
.45 .61428 m
.45 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.55 .61428 m
.55 .61803 L
s
.65 .61428 m
.65 .61803 L
s
.7 .61428 m
.7 .61803 L
s
.75 .61428 m
.75 .61803 L
s
.85 .61428 m
.85 .61803 L
s
.9 .61428 m
.9 .61803 L
s
.95 .61428 m
.95 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10954 m
1 .10954 L
s
.99375 .21909 m
1 .21909 L
s
.99375 .32863 m
1 .32863 L
s
.99375 .43817 m
1 .43817 L
s
.99375 .54772 m
1 .54772 L
s
.125 Mabswid
.99625 .02191 m
1 .02191 L
s
.99625 .04382 m
1 .04382 L
s
.99625 .06573 m
1 .06573 L
s
.99625 .08763 m
1 .08763 L
s
.99625 .13145 m
1 .13145 L
s
.99625 .15336 m
1 .15336 L
s
.99625 .17527 m
1 .17527 L
s
.99625 .19718 m
1 .19718 L
s
.99625 .241 m
1 .241 L
s
.99625 .2629 m
1 .2629 L
s
.99625 .28481 m
1 .28481 L
s
.99625 .30672 m
1 .30672 L
s
.99625 .35054 m
1 .35054 L
s
.99625 .37245 m
1 .37245 L
s
.99625 .39436 m
1 .39436 L
s
.99625 .41627 m
1 .41627 L
s
.99625 .46008 m
1 .46008 L
s
.99625 .48199 m
1 .48199 L
s
.99625 .5039 m
1 .5039 L
s
.99625 .52581 m
1 .52581 L
s
.99625 .56963 m
1 .56963 L
s
.99625 .59154 m
1 .59154 L
s
.99625 .61344 m
1 .61344 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.0025 .61162 Mdot
.0075 .60842 Mdot
.0125 .60611 Mdot
.0175 .60418 Mdot
.0225 .60247 Mdot
.0275 .6009 Mdot
.0325 .59944 Mdot
.0375 .59806 Mdot
.0425 .59675 Mdot
.0475 .59549 Mdot
.0525 .59428 Mdot
.0575 .5931 Mdot
.0625 .59195 Mdot
.0675 .59083 Mdot
.0725 .58974 Mdot
.0775 .58867 Mdot
.0825 .58762 Mdot
.0875 .58658 Mdot
.0925 .58556 Mdot
.0975 .58455 Mdot
.1025 .58356 Mdot
.1075 .58258 Mdot
.1125 .58161 Mdot
.1175 .58065 Mdot
.1225 .5797 Mdot
.1275 .57875 Mdot
.1325 .57782 Mdot
.1375 .57689 Mdot
.1425 .57597 Mdot
.1475 .57505 Mdot
.1525 .57414 Mdot
.1575 .57324 Mdot
.1625 .57233 Mdot
.1675 .57144 Mdot
.1725 .57055 Mdot
.1775 .56966 Mdot
.1825 .56877 Mdot
.1875 .56789 Mdot
.1925 .56701 Mdot
.1975 .56614 Mdot
.2025 .56526 Mdot
.2075 .56439 Mdot
.2125 .56352 Mdot
.2175 .56266 Mdot
.2225 .56179 Mdot
.2275 .56093 Mdot
.2325 .56006 Mdot
.2375 .5592 Mdot
.2425 .55834 Mdot
.2475 .55748 Mdot
.2525 .55663 Mdot
.2575 .55577 Mdot
.2625 .55491 Mdot
.2675 .55406 Mdot
.2725 .5532 Mdot
.2775 .55234 Mdot
.2825 .55149 Mdot
.2875 .55063 Mdot
.2925 .54977 Mdot
.2975 .54892 Mdot
.3025 .54806 Mdot
.3075 .5472 Mdot
.3125 .54634 Mdot
.3175 .54549 Mdot
.3225 .54463 Mdot
.3275 .54377 Mdot
.3325 .5429 Mdot
.3375 .54204 Mdot
.3425 .54118 Mdot
.3475 .54031 Mdot
.3525 .53944 Mdot
.3575 .53858 Mdot
.3625 .53771 Mdot
.3675 .53683 Mdot
.3725 .53596 Mdot
.3775 .53509 Mdot
.3825 .53421 Mdot
.3875 .53333 Mdot
.3925 .53245 Mdot
.3975 .53156 Mdot
.4025 .53068 Mdot
.4075 .52979 Mdot
.4125 .5289 Mdot
.4175 .528 Mdot
.4225 .5271 Mdot
.4275 .5262 Mdot
.4325 .5253 Mdot
.4375 .52439 Mdot
.4425 .52349 Mdot
.4475 .52257 Mdot
.4525 .52166 Mdot
.4575 .52074 Mdot
.4625 .51981 Mdot
.4675 .51889 Mdot
.4725 .51795 Mdot
.4775 .51702 Mdot
.4825 .51608 Mdot
.4875 .51514 Mdot
.4925 .51419 Mdot
.4975 .51324 Mdot
.5025 .51228 Mdot
.5075 .51132 Mdot
.5125 .51035 Mdot
.5175 .50938 Mdot
.5225 .5084 Mdot
.5275 .50742 Mdot
.5325 .50643 Mdot
.5375 .50543 Mdot
.5425 .50443 Mdot
.5475 .50343 Mdot
.5525 .50241 Mdot
.5575 .5014 Mdot
.5625 .50037 Mdot
.5675 .49934 Mdot
.5725 .4983 Mdot
.5775 .49725 Mdot
.5825 .4962 Mdot
.5875 .49514 Mdot
.5925 .49407 Mdot
.5975 .49299 Mdot
.6025 .49191 Mdot
.6075 .49081 Mdot
.6125 .48971 Mdot
.6175 .4886 Mdot
.6225 .48748 Mdot
.6275 .48635 Mdot
.6325 .48521 Mdot
.6375 .48406 Mdot
.6425 .4829 Mdot
.6475 .48173 Mdot
.6525 .48055 Mdot
.6575 .47935 Mdot
.6625 .47815 Mdot
.6675 .47693 Mdot
.6725 .4757 Mdot
.6775 .47446 Mdot
.6825 .4732 Mdot
.6875 .47193 Mdot
.6925 .47064 Mdot
.6975 .46934 Mdot
.7025 .46803 Mdot
.7075 .4667 Mdot
.7125 .46535 Mdot
.7175 .46399 Mdot
.7225 .4626 Mdot
.7275 .4612 Mdot
.7325 .45978 Mdot
.7375 .45834 Mdot
.7425 .45688 Mdot
.7475 .4554 Mdot
.7525 .4539 Mdot
.7575 .45237 Mdot
.7625 .45082 Mdot
.7675 .44924 Mdot
.7725 .44764 Mdot
.7775 .44601 Mdot
.7825 .44435 Mdot
.7875 .44267 Mdot
.7925 .44095 Mdot
.7975 .43919 Mdot
.8025 .43741 Mdot
.8075 .43558 Mdot
.8125 .43372 Mdot
.8175 .43182 Mdot
.8225 .42987 Mdot
.8275 .42788 Mdot
.8325 .42585 Mdot
.8375 .42376 Mdot
.8425 .42162 Mdot
.8475 .41942 Mdot
.8525 .41716 Mdot
.8575 .41484 Mdot
.8625 .41245 Mdot
.8675 .40999 Mdot
.8725 .40744 Mdot
.8775 .40481 Mdot
.8825 .40209 Mdot
.8875 .39926 Mdot
.8925 .39633 Mdot
.8975 .39328 Mdot
.9025 .39009 Mdot
.9075 .38676 Mdot
.9125 .38328 Mdot
.9175 .37961 Mdot
.9225 .37574 Mdot
.9275 .37165 Mdot
.9325 .3673 Mdot
.9375 .36266 Mdot
.9425 .35767 Mdot
.9475 .35228 Mdot
.9525 .34641 Mdot
.9575 .33996 Mdot
.9625 .33278 Mdot
.9675 .32468 Mdot
.9725 .31537 Mdot
.9775 .30436 Mdot
.9825 .29084 Mdot
.9875 .27318 Mdot
.9925 .2473 Mdot
.9975 .19519 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00@P3oool2000000@0oooo00<000000?ooo`3oool0
0P3oool4000002H0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
903oool2000000@0oooo00<000000?ooo`3oool00`3oool2000002L0oooo0P0000040?ooo`030000
003oool0oooo00<0oooo0P00000/0?ooo`@000000P3oool00440oooo00@000000?ooo`3oool00000
203oool00`000000oooo0?ooo`0V0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool08`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002D0oooo00@00000
0?ooo`3oool00000203oool010000000oooo0?ooo`00000]0?ooo`030000003oool0oooo0040oooo
00110?ooo`040000003oool0oooo000000T0oooo00<000000?ooo`3oool09@3oool010000000oooo
0?ooo`0000070?ooo`D00000903oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo
000002D0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000]0?ooo`030000
003oool0oooo0040oooo00110?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0
903oool010000000oooo0?ooo`0000070?ooo`040000003oool0oooo000002D0oooo00@000000?oo
o`3oool00000203oool3000002H0oooo00@000000?ooo`3oool000002@3oool2000002h0oooo00<0
00000?ooo`3oool00@3oool00440oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?oo
o`0S0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000009@3oool010000000oooo
0?ooo`0000080?ooo`030000003oool0oooo02H0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`00000]0?ooo`030000003oool0oooo0040oooo00110?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000009@3oool010000000oooo0?ooo`0000090?ooo`800000
9@3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02H0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`00000[0?ooo`<000000`3oool00480oooo0P00000:0?oo
o`8000009`3oool2000000/0oooo00<000000?ooo`3oool0903oool2000000X0oooo0`00000V0?oo
o`8000002P3oool2000002h0oooo00<000000?ooo`3oool00@3oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool001D0ooooo`000009000000<0oooo000E0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0<@3oool00`000000
oooo0?ooo`0b0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool00@3oool001D0oooo
00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003o
ool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o
0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo0P00003o0?ooo`D0oooo0P0000030?oo
o`005@3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo
00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003o
ool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o
0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool50?oo
o`030000003oool0oooo0040oooo000E0?ooo`800000o`3oool50?ooo`8000000`3oool001D0oooo
00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003o
ool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o
0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool50?oo
o`030000003oool0oooo0040oooo000E0?ooo`800000o`3oool50?ooo`8000000`3oool001D0oooo
00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003o
ool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o
0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool50?oo
o`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0?l0oooo1@3oool00`000000
oooo0?ooo`010?ooo`005@3oool200000?l0oooo1@3oool2000000<0oooo000E0?ooo`030000003o
ool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o
0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool50?oo
o`030000003oool0oooo0040oooo00020?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@0
0000103oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool000050?oo
o`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?l0
oooo1@3oool00`000000oooo0?ooo`010?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`03
0000003oool0oooo00<0oooo0P00003o0?ooo`@0oooo0`0000030?ooo`0000D0oooo0000003oool0
oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0o`3oool50?ooo`03
0000003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool000050?oo
o`000000oooo0?ooo`000000203oool3000000D0oooo00<000000?ooo`3oool0o`3oool50?ooo`03
0000003oool0oooo0040oooo00020?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`030000
003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo0P00003o0?ooo`D0oooo0P000003
0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0
oooo00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000
003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo0P00003o0?ooo`D0oooo0P000003
0?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0
oooo00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000
003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool5
0?ooo`030000003oool0oooo0040oooo000E0?ooo`800000o`3oool50?ooo`8000000`3oool001D0
oooo00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000
003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?oo
o`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool5
0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0?l0oooo0`3oool30000
00<0oooo000E0?ooo`800000o`3oool50?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0
o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0?l0oooo1@3o
ool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<00000
0?ooo`3oool00@3oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo100000040?oo
o`030000003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0o`3oool50?oo
o`030000003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000T0oooo00<000000?oo
o`3oool0103oool200000?l0oooo103oool3000000<0oooo00001@3oool000000?ooo`3oool00000
00X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3o0?ooo`D0oooo00<000000?oo
o`3oool00@3oool000050?ooo`000000oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`0000D0oooo0000003o
ool0oooo000000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0o`3oool5
0?ooo`030000003oool0oooo0040oooo00020?ooo`8000002P3oool2000000D0oooo00<000000?oo
o`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`800000o`3oool50?ooo`80
00000`3oool001D0oooo00<000000?ooo`3oool0o`3oool20?ooo`80000000<0oooo0000003oool0
0P3oool001D0oooo00<000000?ooo`3oool0o`3oool30?ooo`030000003oool0000000<0oooo000E
0?ooo`030000003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`00
0000oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3o
ool0o`3oool50?ooo`030000003oool0oooo0040oooo000E0?ooo`800000o`3oool50?ooo`800000
0`3oool001D0oooo00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo0040oooo000E
0?ooo`030000003oool0oooo0?l0oooo0@3oool200000080oooo00<000000?ooo`3oool00@3oool0
01D0oooo00<000000?ooo`3oool0o`3oool20?ooo`040000003oool0oooo000000<0oooo000E0?oo
o`030000003oool0oooo0?l0oooo1@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000
oooo0?ooo`3o0?ooo`D0oooo00<000000?ooo`3oool00@3oool001D0oooo0P00003o0?ooo`D0oooo
0P0000030?ooo`005@3oool00`000000oooo0?ooo`3n0?ooo`800000103oool00`000000oooo0?oo
o`010?ooo`005@3oool00`000000oooo0?ooo`3o0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0o`3oool50?ooo`030000003oool0oooo
0040oooo000E0?ooo`030000003oool0oooo0?d0oooo0P0000050?ooo`030000003oool0oooo0040
oooo000E0?ooo`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`010?ooo`005@3oool200000?l0oooo1@3oool2000000<0oooo000E0?ooo`030000003oool0
oooo0?`0oooo0P0000060?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo
0?d0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`010?ooo`000P3oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000000D0oooo00<000000?ooo`3oool0o`3oool50?ooo`03
0000003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000P0oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`3j0?ooo`800000203oool00`000000oooo0?ooo`010?oo
o`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo0P00003l0?oo
o`030000003oool0oooo00@0oooo0`0000030?ooo`0000D0oooo0000003oool0oooo000000090?oo
o`8000001@3oool00`000000oooo0?ooo`3i0?ooo`8000002@3oool00`000000oooo0?ooo`010?oo
o`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0nP3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo0040oooo00001@3oool0
00000?ooo`3oool0000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`3h
0?ooo`8000002P3oool00`000000oooo0?ooo`010?ooo`000P3oool2000000X0oooo0P0000050?oo
o`030000003oool0oooo0?L0oooo0`00000:0?ooo`030000003oool0oooo0040oooo000E0?ooo`03
0000003oool0oooo0?P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`010?ooo`00
5@3oool200000?H0oooo0P00000<0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0m03o
ool3000000d0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0m@3oool0
0`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo
0?<0oooo0P00000?0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0?40
oooo0P0000000`3oool000000?ooo`0>0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000
003oool0oooo0?00oooo0`00000A0?ooo`030000003oool0oooo0040oooo000E0?ooo`800000l03o
ool300000140oooo0P0000030?ooo`005@3oool00`000000oooo0?ooo`3]0?ooo`80000000<0oooo
0000003oool04P3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3/0?oo
o`<000005@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3[0?ooo`<0
00005P3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`3Y0?ooo`800000
00<0oooo0000003oool05P3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?oo
o`3X0?ooo`<000006@3oool00`000000oooo0?ooo`010?ooo`005@3oool200000>H0oooo1@00000I
0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0i03oool3000000030?ooo`000000oooo
01X0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0h`3oool3000001h0
oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0h03oool5000001l0oooo
00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0gP3oool4000000030?ooo`00
0000oooo01l0oooo00<000000?ooo`3oool00@3oool001D0oooo0P00003M0?ooo`80000000<0oooo
0000000000008P3oool2000000<0oooo000E0?ooo`030000003oool0oooo0=X0oooo1000000V0?oo
o`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0=L0oooo0`0000000`3oool0
00000000000W0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo0=@0oooo
1P00000Z0?ooo`030000003oool0oooo0040oooo00020?ooo`800000103oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0dP3oool4000000030?ooo`00
0000oooo02/0oooo00<000000?ooo`3oool00@3oool000050?ooo`000000oooo0?ooo`0000002P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0<l0oooo0`0000000`3oool000000000
000_0?ooo`030000003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000L0oooo1@00
00040?ooo`800000c@3oool600000300oooo0`0000030?ooo`0000D0oooo0000003oool0oooo0000
00070?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3oool0bP3oool4000000030?oo
o`000000oooo03<0oooo00<000000?ooo`3oool00@3oool000050?ooo`000000oooo0?ooo`000000
203oool00`000000oooo000000050?ooo`030000003oool0oooo0<L0oooo1P00000g0?ooo`030000
003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000T0oooo0P0000050?ooo`030000
003oool0oooo0<<0oooo1P0000000`3oool000000?ooo`0h0?ooo`030000003oool0oooo0040oooo
00020?ooo`8000002`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0<40oooo1000
00000`3oool000000000000l0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0
oooo0;d0oooo100000000`3oool00000000000100?ooo`030000003oool0oooo0040oooo000E0?oo
o`800000^P3oool4000000040?ooo`000000000000000480oooo0P0000030?ooo`005@3oool00`00
0000oooo0?ooo`2e0?ooo`@0000000@0oooo0000000000000000A`3oool00`000000oooo0?ooo`01
0?ooo`005@3oool00`000000oooo0?ooo`2a0?ooo`P00000B`3oool00`000000oooo0?ooo`010?oo
o`005@3oool00`000000oooo0?ooo`2]0?ooo`L0000000<0oooo0000003oool0C@3oool00`000000
oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`2Y0?ooo`L0000000<0oooo0000003oool0
D@3oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`2U0?ooo`L0000000<0
oooo0000003oool0E@3oool00`000000oooo0?ooo`010?ooo`005@3oool200000:80oooo1`000000
0`3oool000000?ooo`1H0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0W03oool80000
00030?ooo`000000oooo05d0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3o
ool0V03oool7000000030?ooo`00000000000680oooo00<000000?ooo`3oool00@3oool001D0oooo
00<000000?ooo`3oool0T`3oool8000000030?ooo`000000oooo06H0oooo00<000000?ooo`3oool0
0@3oool001D0oooo00<000000?ooo`3oool0S@3oool6000000060?ooo`00000000000000003oool0
0000J`3oool00`000000oooo0?ooo`010?ooo`005@3oool2000008X0oooo1P000000103oool00000
00000000001`0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0Q03oool7000000030?oo
o`000000000007H0oooo00<000000?ooo`3oool00@3oool001D0oooo00<000000?ooo`3oool0O`3o
ool7000000040?ooo`0000000000000007X0oooo00<000000?ooo`3oool00@3oool001D0oooo00<0
00000?ooo`3oool0NP3oool8000000030?ooo`000000000007l0oooo00<000000?ooo`3oool00@3o
ool001D0oooo00<000000?ooo`3oool0M03oool6000000060?ooo`00000000000000003oool00000
Q03oool00`000000oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`1^0?ooo`P0000000@0
oooo0000000000000000RP3oool00`000000oooo0?ooo`010?ooo`005@3oool2000006X0oooo2000
0000103oool0000000000000002>0?ooo`8000000`3oool001D0oooo00<000000?ooo`3oool0H`3o
ool6000000060?ooo`00000000000000003oool00000U@3oool00`000000oooo0?ooo`010?ooo`00
5@3oool00`000000oooo0?ooo`1N0?ooo`L0000000@0oooo0000000000000000V`3oool00`000000
oooo0?ooo`010?ooo`005@3oool00`000000oooo0?ooo`1H0?ooo`T0000000<0oooo000000000000
X03oool00`000000oooo0?ooo`010?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool3000000D0oooo00<000000?ooo`3oool0DP3oool6000000050?ooo`00000000000000003oool0
0P00002U0?ooo`030000003oool0oooo0040oooo00001@3oool000000?ooo`3oool0000000/0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1<0?ooo`P0000000@0oooo000000000000
0000[03oool00`000000oooo0?ooo`010?ooo`0000D0oooo0000003oool0oooo0000000;0?ooo`03
0000003oool0oooo0080oooo0P0000180?ooo`D0000000D0oooo00000000000000000?ooo`020000
0:l0oooo0`0000030?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<000001@3oool00`00
0000oooo0?ooo`100?ooo`L0000000<0oooo0000000000000P00002h0?ooo`030000003oool0oooo
0040oooo00001@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0k0?ooo`P0000000@0oooo0000000000000000_@3oool00`000000oooo0?ooo`01
0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0=@3oool6000000060?ooo`00000000000000003oool00000``3oool00`000000oooo
0?ooo`010?ooo`000P3oool2000000T0oooo100000040?ooo`030000003oool0oooo02l0oooo2000
0000103oool000000000000000390?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003o
ool0oooo02X0oooo2@0000000`3oool000000000003>0?ooo`030000003oool0oooo0040oooo000E
0?ooo`8000009@3oool6000000060?ooo`00000000000000003oool00000d`3oool2000000<0oooo
000E0?ooo`030000003oool0oooo01l0oooo1`000000103oool0000000000000003J0?ooo`030000
003oool0oooo0040oooo000E0?ooo`030000003oool0oooo01X0oooo200000000`3oool000000000
003O0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo01D0oooo2@000000
0`3oool000000?ooo`3S0?ooo`030000003oool0oooo0040oooo000E0?ooo`030000003oool0oooo
00l0oooo1P0000001P3oool00000000000000000oooo00000>T0oooo00<000000?ooo`3oool00@3o
ool001D0oooo0P00000<0?ooo`H0000000@0oooo0000000000000000kP3oool2000000<0oooo000E
0?ooo`030000003oool0oooo00L0oooo1P0000000`3oool000000000003d0?ooo`030000003oool0
oooo0040oooo000E0?ooo`030000003oool0oooo00@0oooo1@0000000`3oool000000000003h0?oo
o`030000003oool0oooo0040oooo000E0?ooo`040000003oool0oooo0?ooo`<0000000@0oooo0000
000000000000o03oool00`000000oooo0?ooo`010?ooo`005@3oool700000?l0oooo0@3oool00`00
0000oooo0?ooo`010?ooo`005@3oool3000000030?ooo`000000oooo0?l0oooo0P3oool00`000000
oooo0?ooo`010?ooo`005@3oool2000000/0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool2000000<0oooo
000E0?ooool000002@0000030?ooo`00o`3ooolQ0?ooo`00\
\>"],
  ImageRangeCache->{{{91.5625, 320.938}, {318.875, 177.562}} -> {-0.517212, \
0.725572, 0.00381046, 0.00347847}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<fsscaled.eps\>", p1, ImageSize \[Rule] 72*2.5]\)], "Input"],

Cell[BoxData[
    \("fsscaled.eps"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(p2 = 
      ListPlot[fascaled, \ Axes \[Rule] False, Frame \[Rule] True, 
        PlotRange \[Rule] {{0, 1}, {\(-1\)/Sqrt[Pi], 0}}]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1 0.618034 1.09544 [
[.2 -0.0125 -9 -9 ]
[.2 -0.0125 9 0 ]
[.4 -0.0125 -9 -9 ]
[.4 -0.0125 9 0 ]
[.6 -0.0125 -9 -9 ]
[.6 -0.0125 9 0 ]
[.8 -0.0125 -9 -9 ]
[.8 -0.0125 9 0 ]
[1 -0.0125 -3 -9 ]
[1 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .07032 -24 -4.5 ]
[-0.0125 .07032 0 4.5 ]
[-0.0125 .17986 -24 -4.5 ]
[-0.0125 .17986 0 4.5 ]
[-0.0125 .2894 -24 -4.5 ]
[-0.0125 .2894 0 4.5 ]
[-0.0125 .39895 -24 -4.5 ]
[-0.0125 .39895 0 4.5 ]
[-0.0125 .50849 -24 -4.5 ]
[-0.0125 .50849 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2 0 m
.2 .00625 L
s
[(0.2)] .2 -0.0125 0 1 Mshowa
.4 0 m
.4 .00625 L
s
[(0.4)] .4 -0.0125 0 1 Mshowa
.6 0 m
.6 .00625 L
s
[(0.6)] .6 -0.0125 0 1 Mshowa
.8 0 m
.8 .00625 L
s
[(0.8)] .8 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(1)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.05 0 m
.05 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.25 0 m
.25 .00375 L
s
.3 0 m
.3 .00375 L
s
.35 0 m
.35 .00375 L
s
.45 0 m
.45 .00375 L
s
.5 0 m
.5 .00375 L
s
.55 0 m
.55 .00375 L
s
.65 0 m
.65 .00375 L
s
.7 0 m
.7 .00375 L
s
.75 0 m
.75 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.95 0 m
.95 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07032 m
.00625 .07032 L
s
[(-0.5)] -0.0125 .07032 1 0 Mshowa
0 .17986 m
.00625 .17986 L
s
[(-0.4)] -0.0125 .17986 1 0 Mshowa
0 .2894 m
.00625 .2894 L
s
[(-0.3)] -0.0125 .2894 1 0 Mshowa
0 .39895 m
.00625 .39895 L
s
[(-0.2)] -0.0125 .39895 1 0 Mshowa
0 .50849 m
.00625 .50849 L
s
[(-0.1)] -0.0125 .50849 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(0)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .09222 m
.00375 .09222 L
s
0 .11413 m
.00375 .11413 L
s
0 .13604 m
.00375 .13604 L
s
0 .15795 m
.00375 .15795 L
s
0 .20177 m
.00375 .20177 L
s
0 .22368 m
.00375 .22368 L
s
0 .24559 m
.00375 .24559 L
s
0 .26749 m
.00375 .26749 L
s
0 .31131 m
.00375 .31131 L
s
0 .33322 m
.00375 .33322 L
s
0 .35513 m
.00375 .35513 L
s
0 .37704 m
.00375 .37704 L
s
0 .42086 m
.00375 .42086 L
s
0 .44276 m
.00375 .44276 L
s
0 .46467 m
.00375 .46467 L
s
0 .48658 m
.00375 .48658 L
s
0 .5304 m
.00375 .5304 L
s
0 .55231 m
.00375 .55231 L
s
0 .57422 m
.00375 .57422 L
s
0 .59613 m
.00375 .59613 L
s
0 .04841 m
.00375 .04841 L
s
0 .0265 m
.00375 .0265 L
s
0 .00459 m
.00375 .00459 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 .61178 m
0 .61803 L
s
.2 .61178 m
.2 .61803 L
s
.4 .61178 m
.4 .61803 L
s
.6 .61178 m
.6 .61803 L
s
.8 .61178 m
.8 .61803 L
s
.125 Mabswid
.05 .61428 m
.05 .61803 L
s
.1 .61428 m
.1 .61803 L
s
.15 .61428 m
.15 .61803 L
s
.25 .61428 m
.25 .61803 L
s
.3 .61428 m
.3 .61803 L
s
.35 .61428 m
.35 .61803 L
s
.45 .61428 m
.45 .61803 L
s
.5 .61428 m
.5 .61803 L
s
.55 .61428 m
.55 .61803 L
s
.65 .61428 m
.65 .61803 L
s
.7 .61428 m
.7 .61803 L
s
.75 .61428 m
.75 .61803 L
s
.85 .61428 m
.85 .61803 L
s
.9 .61428 m
.9 .61803 L
s
.95 .61428 m
.95 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 .07032 m
1 .07032 L
s
.99375 .17986 m
1 .17986 L
s
.99375 .2894 m
1 .2894 L
s
.99375 .39895 m
1 .39895 L
s
.99375 .50849 m
1 .50849 L
s
.125 Mabswid
.99625 .09222 m
1 .09222 L
s
.99625 .11413 m
1 .11413 L
s
.99625 .13604 m
1 .13604 L
s
.99625 .15795 m
1 .15795 L
s
.99625 .20177 m
1 .20177 L
s
.99625 .22368 m
1 .22368 L
s
.99625 .24559 m
1 .24559 L
s
.99625 .26749 m
1 .26749 L
s
.99625 .31131 m
1 .31131 L
s
.99625 .33322 m
1 .33322 L
s
.99625 .35513 m
1 .35513 L
s
.99625 .37704 m
1 .37704 L
s
.99625 .42086 m
1 .42086 L
s
.99625 .44276 m
1 .44276 L
s
.99625 .46467 m
1 .46467 L
s
.99625 .48658 m
1 .48658 L
s
.99625 .5304 m
1 .5304 L
s
.99625 .55231 m
1 .55231 L
s
.99625 .57422 m
1 .57422 L
s
.99625 .59613 m
1 .59613 L
s
.99625 .04841 m
1 .04841 L
s
.99625 .0265 m
1 .0265 L
s
.99625 .00459 m
1 .00459 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.0025 .03993 Mdot
.0075 .05964 Mdot
.0125 .07326 Mdot
.0175 .08429 Mdot
.0225 .09379 Mdot
.0275 .10223 Mdot
.0325 .10989 Mdot
.0375 .11695 Mdot
.0425 .12352 Mdot
.0475 .12968 Mdot
.0525 .1355 Mdot
.0575 .14101 Mdot
.0625 .14627 Mdot
.0675 .1513 Mdot
.0725 .15612 Mdot
.0775 .16076 Mdot
.0825 .16523 Mdot
.0875 .16954 Mdot
.0925 .17372 Mdot
.0975 .17777 Mdot
.1025 .18169 Mdot
.1075 .18551 Mdot
.1125 .18923 Mdot
.1175 .19284 Mdot
.1225 .19637 Mdot
.1275 .19981 Mdot
.1325 .20317 Mdot
.1375 .20646 Mdot
.1425 .20967 Mdot
.1475 .21282 Mdot
.1525 .2159 Mdot
.1575 .21892 Mdot
.1625 .22188 Mdot
.1675 .22479 Mdot
.1725 .22764 Mdot
.1775 .23044 Mdot
.1825 .23319 Mdot
.1875 .2359 Mdot
.1925 .23856 Mdot
.1975 .24117 Mdot
.2025 .24375 Mdot
.2075 .24628 Mdot
.2125 .24878 Mdot
.2175 .25124 Mdot
.2225 .25366 Mdot
.2275 .25605 Mdot
.2325 .25841 Mdot
.2375 .26073 Mdot
.2425 .26302 Mdot
.2475 .26528 Mdot
.2525 .26751 Mdot
.2575 .26971 Mdot
.2625 .27189 Mdot
.2675 .27404 Mdot
.2725 .27616 Mdot
.2775 .27826 Mdot
.2825 .28033 Mdot
.2875 .28238 Mdot
.2925 .2844 Mdot
.2975 .28641 Mdot
.3025 .28839 Mdot
.3075 .29035 Mdot
.3125 .29229 Mdot
.3175 .29421 Mdot
.3225 .29611 Mdot
.3275 .29799 Mdot
.3325 .29985 Mdot
.3375 .30169 Mdot
.3425 .30352 Mdot
.3475 .30532 Mdot
.3525 .30711 Mdot
.3575 .30889 Mdot
.3625 .31065 Mdot
.3675 .31239 Mdot
.3725 .31412 Mdot
.3775 .31583 Mdot
.3825 .31753 Mdot
.3875 .31921 Mdot
.3925 .32088 Mdot
.3975 .32254 Mdot
.4025 .32418 Mdot
.4075 .32581 Mdot
.4125 .32743 Mdot
.4175 .32903 Mdot
.4225 .33063 Mdot
.4275 .33221 Mdot
.4325 .33378 Mdot
.4375 .33533 Mdot
.4425 .33688 Mdot
.4475 .33842 Mdot
.4525 .33994 Mdot
.4575 .34146 Mdot
.4625 .34297 Mdot
.4675 .34446 Mdot
.4725 .34595 Mdot
.4775 .34743 Mdot
.4825 .34889 Mdot
.4875 .35035 Mdot
.4925 .3518 Mdot
.4975 .35325 Mdot
.5025 .35468 Mdot
.5075 .35611 Mdot
.5125 .35753 Mdot
.5175 .35894 Mdot
.5225 .36034 Mdot
.5275 .36174 Mdot
.5325 .36312 Mdot
.5375 .36451 Mdot
.5425 .36588 Mdot
.5475 .36725 Mdot
.5525 .36862 Mdot
.5575 .36997 Mdot
.5625 .37132 Mdot
.5675 .37267 Mdot
.5725 .37401 Mdot
.5775 .37534 Mdot
.5825 .37667 Mdot
.5875 .378 Mdot
.5925 .37932 Mdot
.5975 .38063 Mdot
.6025 .38194 Mdot
.6075 .38325 Mdot
.6125 .38455 Mdot
.6175 .38585 Mdot
.6225 .38715 Mdot
.6275 .38844 Mdot
.6325 .38973 Mdot
.6375 .39101 Mdot
.6425 .3923 Mdot
.6475 .39358 Mdot
.6525 .39486 Mdot
.6575 .39613 Mdot
.6625 .39741 Mdot
.6675 .39868 Mdot
.6725 .39995 Mdot
.6775 .40122 Mdot
.6825 .40248 Mdot
.6875 .40375 Mdot
.6925 .40502 Mdot
.6975 .40629 Mdot
.7025 .40755 Mdot
.7075 .40882 Mdot
.7125 .41008 Mdot
.7175 .41135 Mdot
.7225 .41262 Mdot
.7275 .41389 Mdot
.7325 .41516 Mdot
.7375 .41644 Mdot
.7425 .41771 Mdot
.7475 .41899 Mdot
.7525 .42027 Mdot
.7575 .42156 Mdot
.7625 .42284 Mdot
.7675 .42414 Mdot
.7725 .42544 Mdot
.7775 .42674 Mdot
.7825 .42805 Mdot
.7875 .42936 Mdot
.7925 .43068 Mdot
.7975 .43201 Mdot
.8025 .43335 Mdot
.8075 .4347 Mdot
.8125 .43605 Mdot
.8175 .43742 Mdot
.8225 .4388 Mdot
.8275 .44018 Mdot
.8325 .44159 Mdot
.8375 .443 Mdot
.8425 .44444 Mdot
.8475 .44588 Mdot
.8525 .44735 Mdot
.8575 .44884 Mdot
.8625 .45034 Mdot
.8675 .45187 Mdot
.8725 .45343 Mdot
.8775 .45501 Mdot
.8825 .45662 Mdot
.8875 .45827 Mdot
.8925 .45995 Mdot
.8975 .46167 Mdot
.9025 .46344 Mdot
.9075 .46526 Mdot
.9125 .46713 Mdot
.9175 .46906 Mdot
.9225 .47106 Mdot
.9275 .47314 Mdot
.9325 .47531 Mdot
.9375 .47759 Mdot
.9425 .47999 Mdot
.9475 .48253 Mdot
.9525 .48524 Mdot
.9575 .48817 Mdot
.9625 .49135 Mdot
.9675 .49488 Mdot
.9725 .49884 Mdot
.9775 .50343 Mdot
.9825 .50893 Mdot
.9875 .51595 Mdot
.9925 .52599 Mdot
.9975 .54567 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00A`3oool2000000@0oooo00<000000?ooo`3oool00P3oool4000002@0
oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0903oool2000000@0
oooo00<000000?ooo`3oool00`3oool2000002D0oooo0P0000040?ooo`030000003oool0oooo00<0
oooo0P00000[0?ooo`@000000P3oool004H0oooo00@000000?ooo`3oool00000203oool00`000000
oooo0?ooo`0T0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08`3oool01000
0000oooo0?ooo`0000080?ooo`040000003oool0oooo000002<0oooo00@000000?ooo`3oool00000
203oool010000000oooo0?ooo`00000/0?ooo`030000003oool0oooo0040oooo00160?ooo`040000
003oool0oooo000000T0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000070?oo
o`D00000903oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002<0oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`00000/0?ooo`030000003oool0oooo0040
oooo00160?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool08P3oool010000000
oooo0?ooo`0000070?ooo`040000003oool0oooo000002D0oooo00@000000?ooo`3oool00000203o
ool3000002@0oooo00@000000?ooo`3oool000002@3oool2000002d0oooo00<000000?ooo`3oool0
0@3oool004H0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`0Q0?ooo`040000
003oool0oooo000000P0oooo00<000000?ooo`0000009@3oool010000000oooo0?ooo`0000080?oo
o`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00
000/0?ooo`030000003oool0oooo0040oooo00160?ooo`040000003oool0oooo000000P0oooo00@0
00000?ooo`3oool000008`3oool010000000oooo0?ooo`0000090?ooo`8000009@3oool010000000
oooo0?ooo`0000080?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000203oool0
10000000oooo0?ooo`00000Z0?ooo`<000000`3oool004L0oooo0P00000:0?ooo`8000009@3oool2
000000/0oooo00<000000?ooo`3oool0903oool2000000X0oooo0`00000T0?ooo`8000002P3oool2
000002d0oooo00<000000?ooo`3oool00@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo
8@3oool00?l0oooo8@3oool001/0ooooo`000003000000<0oooo000K0?ooo`800000<@3oool00`00
0000oooo0?ooo`0`0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0<03oool00`00
0000oooo0?ooo`0`0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0oP3oool00`000000
oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo
000K0?ooo`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo00<00000
0?ooo`3oool0oP3oool00`000000oooo0?ooo`010?ooo`006`3oool200000?h0oooo0P0000030?oo
o`006`3oool00`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000
003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo0P00003o0?ooo`030000003o
ool0oooo0040oooo000K0?ooo`800000o`3oool00`000000oooo0?ooo`010?ooo`006`3oool00`00
0000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo000K0?ooo`800000oP3oool2000000<0
oooo000K0?ooo`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo0`00
003n0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool000000?h0oooo00<00000
0?ooo`3oool00@3oool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo0`0000050?oo
o`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool000L0oooo00@000000?ooo`3o
ool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0?h0oooo00<000000?oo
o`3oool00@3oool000L0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?oo
o`@00000n`3oool3000000<0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3o
ool00000203oool3000000D0oooo00@000000?ooo`3oool00000o@3oool00`000000oooo0?ooo`01
0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0oP3oool00`000000oooo0?ooo`010?ooo`001`3oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00D0oooo00@000000?ooo`3oool0oooo0P00003k0?ooo`030000003o
ool0oooo0040oooo00080?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`3i0?ooo`030000003oool0oooo0040oooo000K0?ooo`8000000`3oool20000
0?T0oooo0P0000030?ooo`006`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0?P0
oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool00`3oool200000?T0oooo
00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`3g0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo00D0oooo0P00003g
0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0m@3oool00`000000oooo0?ooo`010?ooo`006`3oool2000000L0oooo0P00003e0?ooo`80
00000`3oool001/0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`3d0?ooo`030000
003oool0oooo0040oooo000K0?ooo`030000003oool0oooo00L0oooo0P00003e0?ooo`030000003o
ool0oooo0040oooo000K0?ooo`030000003oool0oooo00P0oooo0P00003d0?ooo`030000003oool0
oooo0040oooo000K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0lP3oool00`00
0000oooo0?ooo`010?ooo`006`3oool2000000/0oooo0P00003a0?ooo`8000000`3oool001/0oooo
00<000000?ooo`3oool02`3oool200000?40oooo00<000000?ooo`3oool00@3oool001/0oooo00<0
00000?ooo`3oool0303oool00`000000oooo0?ooo`3_0?ooo`030000003oool0oooo0040oooo000K
0?ooo`030000003oool0oooo00`0oooo0P00003`0?ooo`030000003oool0oooo0040oooo000K0?oo
o`030000003oool0oooo00d0oooo0`00003^0?ooo`030000003oool0oooo0040oooo000K0?ooo`03
0000003oool0oooo00l0oooo0P00003]0?ooo`030000003oool0oooo0040oooo000K0?ooo`800000
4@3oool200000>/0oooo0P0000030?ooo`006`3oool00`000000oooo0?ooo`0A0?ooo`800000j`3o
ool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo0>T0oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool04`3oool20000
0>T0oooo00<000000?ooo`3oool00@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0D0?ooo`800000j03oool00`000000
oooo0?ooo`010?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0
oooo00<000000?ooo`3oool05@3oool200000>L0oooo00<000000?ooo`3oool00@3oool000L0oooo
00@000000?ooo`3oool000001`3oool5000000@0oooo0P00000G0?ooo`<00000h`3oool3000000<0
oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool000001`3oool010000000
oooo0?ooo`0000050?ooo`030000003oool0oooo01P0oooo0P00003T0?ooo`030000003oool0oooo
0040oooo00070?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`0000001@3oool00`00
0000oooo0?ooo`0I0?ooo`800000h`3oool00`000000oooo0?ooo`010?ooo`001`3oool010000000
oooo0?ooo`0000090?ooo`8000001@3oool00`000000oooo0?ooo`0J0?ooo`@00000h03oool00`00
0000oooo0?ooo`010?ooo`00203oool2000000/0oooo00<000000?ooo`3oool00`3oool00`000000
oooo0?ooo`0K0?ooo`040000003oool0000000000=l0oooo00<000000?ooo`3oool00@3oool001/0
oooo0P00000O0?ooo`800000g@3oool2000000<0oooo000K0?ooo`030000003oool0oooo01l0oooo
0`00003L0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0240oooo0P00
003K0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0280oooo0`00003I
0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo02<0oooo1000003G0?oo
o`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo02H0oooo0P00003F0?ooo`03
0000003oool0oooo0040oooo000K0?ooo`800000:03oool400000=80oooo0P0000030?ooo`006`3o
ool00`000000oooo0?ooo`0X0?ooo`040000003oool0000000000=80oooo00<000000?ooo`3oool0
0@3oool001/0oooo00<000000?ooo`3oool0:`3oool200000=40oooo00<000000?ooo`3oool00@3o
ool001/0oooo00<000000?ooo`3oool0;03oool400000<h0oooo00<000000?ooo`3oool00@3oool0
01/0oooo00<000000?ooo`3oool0;@3oool010000000oooo00000000003=0?ooo`030000003oool0
oooo0040oooo000K0?ooo`030000003oool0oooo0300oooo0`00003;0?ooo`030000003oool0oooo
0040oooo000K0?ooo`800000<P3oool400000<P0oooo0P0000030?ooo`006`3oool00`000000oooo
0?ooo`0d0?ooo`<00000a`3oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?oo
o`0e0?ooo`D00000a03oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`0h
0?ooo`<00000``3oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`0j0?oo
o`@00000`03oool00`000000oooo0?ooo`010?ooo`006`3oool2000003`0oooo00<000000?ooo`00
00000P00002m0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0?P3oool500000;/0oooo
00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0@@3oool400000;T0oooo00<0
00000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0@`3oool400000;L0oooo00<00000
0?ooo`3oool00@3oool000P0oooo0P0000040?ooo`030000003oool0oooo00<0oooo0P0000050?oo
o`030000003oool0oooo04H0oooo0`00002e0?ooo`030000003oool0oooo0040oooo00070?ooo`04
0000003oool0oooo000000P0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`17
0?ooo`D00000/P3oool00`000000oooo0?ooo`010?ooo`001`3oool010000000oooo0?ooo`00000;
0?ooo`030000003oool0oooo0080oooo0P00001;0?ooo`@00000[P3oool3000000<0oooo00000`3o
ool000000000000200000080oooo00@000000?ooo`3oool000002@3oool2000000D0oooo00<00000
0?ooo`3oool0C03oool500000:d0oooo00<000000?ooo`3oool00@3oool000L0oooo00@000000?oo
o`3oool000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo04l0oooo1@00002Z
0?ooo`030000003oool0oooo0040oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000103oool00`000000oooo0?ooo`1A0?ooo`030000003oool0000000<00000Y`3o
ool00`000000oooo0?ooo`010?ooo`00203oool2000000X0oooo0P0000050?ooo`030000003oool0
oooo05@0oooo1P00002T0?ooo`030000003oool0oooo0040oooo000K0?ooo`800000F@3oool50000
0:00oooo0P0000030?ooo`006`3oool00`000000oooo0?ooo`1J0?ooo`030000003oool000000080
0000W`3oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`1M0?ooo`H00000
V`3oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`1O0?ooo`030000003o
ool0000000@00000V03oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`1S
0?ooo`030000003oool0000000800000UP3oool00`000000oooo0?ooo`010?ooo`006`3oool00`00
0000oooo0?ooo`1V0?ooo`H00000TP3oool00`000000oooo0?ooo`010?ooo`006`3oool2000006T0
oooo00<000000?ooo`0000001000002>0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0
K03oool00`000000oooo00000003000008`0oooo00<000000?ooo`3oool00@3oool001/0oooo00<0
00000?ooo`3oool0K`3oool7000008P0oooo00<000000?ooo`3oool00@3oool001/0oooo00<00000
0?ooo`3oool0L`3oool6000008D0oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?oo
o`3oool0M`3oool600000840oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3o
ool0N@3oool2000000030?ooo`000000000000<00000O@3oool00`000000oooo0?ooo`010?ooo`00
6`3oool2000007h0oooo0P000000103oool0000000000000001j0?ooo`8000000`3oool001/0oooo
00<000000?ooo`3oool0P@3oool6000007L0oooo00<000000?ooo`3oool00@3oool001/0oooo00<0
00000?ooo`3oool0P`3oool00`000000oooo00000005000007<0oooo00<000000?ooo`3oool00@3o
ool001/0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo00000005000006l0oooo00<00000
0?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0R`3oool2000000030?ooo`0000000000
00<00000J`3oool00`000000oooo0?ooo`010?ooo`006`3oool200000900oooo0P0000000`3oool0
000000000002000006L0oooo0P0000030?ooo`006`3oool00`000000oooo0?ooo`2C0?ooo`L00000
I03oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`2G0?ooo`L00000H03o
ool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`2J0?ooo`030000003oool0
000000D00000G03oool00`000000oooo0?ooo`010?ooo`00203oool2000000@0oooo00<000000?oo
o`3oool00P3oool4000000@0oooo00<000000?ooo`3oool0WP3oool00`000000oooo000000050000
05P0oooo00<000000?ooo`3oool00@3oool000L0oooo00@000000?ooo`3oool00000203oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0:80oooo0P0000000`3oool00000000000030000
05@0oooo00<000000?ooo`3oool00@3oool000L0oooo00@000000?ooo`3oool000002@3oool00`00
0000oooo0?ooo`040?ooo`800000Y`3oool2000000030?ooo`000000000000800000C`3oool30000
00<0oooo00000`3oool000000000000200000080oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0:X0oooo1`00001=0?ooo`030000003oool0oooo
0040oooo00070?ooo`040000003oool0oooo000000/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`2^0?ooo`L00000B@3oool00`000000oooo0?ooo`010?ooo`001`3oool010000000
oooo0?ooo`0000080?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool0/P3oool7
000004D0oooo00<000000?ooo`3oool00@3oool000P0oooo0P00000:0?ooo`8000001@3oool00`00
0000oooo0?ooo`2e0?ooo`030000003oool0000000D00000@@3oool00`000000oooo0?ooo`010?oo
o`006`3oool00`000000oooo0?ooo`2i0?ooo`030000003oool0000000D00000?@3oool00`000000
oooo0?ooo`010?ooo`006`3oool200000;h0oooo0P0000000`3oool0000000000003000003P0oooo
0P0000030?ooo`006`3oool00`000000oooo0?ooo`310?ooo`80000000<0oooo0000000000000P00
000f0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0<D0oooo1`00000b
0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0<T0oooo1`00000^0?oo
o`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0<d0oooo1`00000Z0?ooo`03
0000003oool0oooo0040oooo000K0?ooo`800000d@3oool00`000000oooo00000004000002H0oooo
0P0000030?ooo`006`3oool00`000000oooo0?ooo`3D0?ooo`030000003oool0000000<00000903o
ool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3G0?ooo`L00000803oool0
0`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3K0?ooo`H000007@3oool00`00
0000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3O0?ooo`@000006`3oool00`000000
oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3Q0?ooo`D00000603oool00`000000oooo
0?ooo`010?ooo`006`3oool200000>D0oooo1P00000C0?ooo`8000000`3oool001/0oooo00<00000
0?ooo`3oool0iP3oool00`000000oooo00000002000001<0oooo00<000000?ooo`3oool00@3oool0
01/0oooo00<000000?ooo`3oool0jP3oool400000100oooo00<000000?ooo`3oool00@3oool001/0
oooo00<000000?ooo`3oool0j`3oool00`000000oooo00000002000000h0oooo00<000000?ooo`3o
ool00@3oool001/0oooo00<000000?ooo`3oool0kP3oool4000000`0oooo00<000000?ooo`3oool0
0@3oool001/0oooo0P00003b0?ooo`<000002@3oool2000000<0oooo000K0?ooo`030000003oool0
oooo0?80oooo0`0000090?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo
0?@0oooo0`0000070?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo0?H0
oooo0P0000060?ooo`030000003oool0oooo0040oooo00080?ooo`800000103oool00`000000oooo
0?ooo`020?ooo`@00000103oool00`000000oooo0?ooo`3g0?ooo`8000001@3oool00`000000oooo
0?ooo`010?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo00<0oooo
00<000000?ooo`3oool0n03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0040oooo
00070?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool200000?X0oooo
0P000000103oool000000000000000030?ooo`0000<0oooo0000000000000P0000020?ooo`040000
003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`3j0?ooo`05
0000003oool0oooo0?ooo`0000000`3oool000L0oooo00@000000?ooo`3oool000002P3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo0?X0oooo0P0000020?ooo`030000003oool0oooo
0040oooo00070?ooo`040000003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo
0?/0oooo00@000000?ooo`3oool000000`3oool000P0oooo0P00000;0?ooo`030000003oool0oooo
00<0oooo00<000000?ooo`3oool0n`3oool2000000030?ooo`000000oooo0080oooo000K0?ooo`03
0000003oool0oooo0?`0oooo00<000000?ooo`0000000`3oool001/0oooo0P00003n0?ooo`800000
0`3oool001/0oooo00<000000?ooo`3oool0oP3oool00`000000oooo0?ooo`010?ooo`006`3oool0
0`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo000K0?ooo`030000003oool0oooo
0?`0oooo0`0000030?ooo`006`3oool00`000000oooo0?ooo`3m0?ooo`8000000`3oool001/0oooo
0P00003n0?ooo`8000000`3oool001/0oooo00<000000?ooo`3oool0oP3oool00`000000oooo0?oo
o`010?ooo`006`3oool00`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo000K0?oo
o`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3o
ool0oP3oool00`000000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3n0?ooo`030000
003oool0oooo0040oooo000K0?ooo`800000oP3oool2000000<0oooo000K0?ooo`030000003oool0
oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo00<000000?ooo`3oool0oP3oool00`00
0000oooo0?ooo`010?ooo`006`3oool00`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040
oooo000K0?ooo`030000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001/0oooo00<0
00000?ooo`3oool0oP3oool00`000000oooo0?ooo`010?ooo`006`3oool200000?h0oooo0P000003
0?ooo`006`3oool00`000000oooo0?ooo`3n0?ooo`030000003oool0oooo0040oooo000K0?ooo`03
0000003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001@0oooo0P0000050?ooo`030000
003oool0oooo0?h0oooo00<000000?ooo`3oool00@3oool001<0oooo00@000000?ooo`3oool00000
103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0
0@3oool001<0oooo00@000000?ooo`3oool00000103ooooo000000<000000`3oool001<0oooo00@0
00000?ooo`3oool00000o`3oool:0?ooo`004`3oool010000000oooo0?ooo`00003o0?ooo`X0oooo
000C0?ooo`040000003oool0oooo00000?l0oooo2P3oool001@0oooo0P00003o0?ooo`/0oooo003o
0?ooob40oooo0000\
\>"],
  ImageRangeCache->{{{91.5625, 320.938}, {562.562, 421.25}} -> {-0.552263, \
1.26382, 0.00389849, 0.00355883}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<fascaled.eps\>", p2, ImageSize \[Rule] 72*2.5]\)], "Input"],

Cell[BoxData[
    \("fascaled.eps"\)], "Output"]
}, Open  ]],

Cell[BoxData[""], "Input"],

Cell[CellGroupData[{

Cell["\<\
Block[{kk=50,muf2=1.65,mub2=0.5,gg2=N[4 Pi],alpha=0.396},
ft=LinearSolve[matrix[kk,muf2,mub2,gg2,alpha],vectors[kk,muf2,mub2,gg2,alpha]]\
;
ftssave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[1]]&,ft]}];
ftasave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[2]]&,ft]}];
]\
\>", "Input",
  PageWidth->Infinity,
  ShowSpecialCharacters->False],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(ftasave\)\" is similar \
to existing symbol \"\!\(ftssave\)\"."\)], "Message"]
}, Open  ]],

Cell["\<\
Block[{kk=100,muf2=1.65,mub2=0.5,gg2=N[4 Pi],alpha=0.396},
ft=LinearSolve[matrix[kk,muf2,mub2,gg2,alpha],vectors[kk,muf2,mub2,gg2,alpha]]\
;
ftssave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[1]]&,ft]}];
ftasave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[2]]&,ft]}];
]\
\>", "Input",
  PageWidth->Infinity,
  ShowSpecialCharacters->False],

Cell["\<\
Block[{kk=200,muf2=1.65,mub2=0.5,gg2=N[4 Pi],alpha=0.396},
ft=LinearSolve[matrix[kk,muf2,mub2,gg2,alpha],vectors[kk,muf2,mub2,gg2,alpha]]\
;
ftssave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[1]]&,ft]}];
ftasave[kk]=Transpose[{Table[(k-1/2)/kk,{k,kk}],Map[#[[2]]&,ft]}];
]\
\>", "Input",
  PageWidth->Infinity,
  ShowSpecialCharacters->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[ftssave[50], DisplayFunction \[Rule] Identity], 
      ListPlot[ftssave[100], DisplayFunction \[Rule] Identity], 
      ListPlot[ftssave[200], 
        DisplayFunction \[Rule] 
          Identity], \[IndentingNewLine]DisplayFunction \[Rule] \
$DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.954768 0.0294197 1.07859 [
[.21476 .01692 -9 -9 ]
[.21476 .01692 9 0 ]
[.40572 .01692 -9 -9 ]
[.40572 .01692 9 0 ]
[.59667 .01692 -9 -9 ]
[.59667 .01692 9 0 ]
[.78762 .01692 -9 -9 ]
[.78762 .01692 9 0 ]
[.97858 .01692 -3 -9 ]
[.97858 .01692 3 0 ]
[.01131 .13728 -18 -4.5 ]
[.01131 .13728 0 4.5 ]
[.01131 .24514 -18 -4.5 ]
[.01131 .24514 0 4.5 ]
[.01131 .353 -18 -4.5 ]
[.01131 .353 0 4.5 ]
[.01131 .46086 -18 -4.5 ]
[.01131 .46086 0 4.5 ]
[.01131 .56871 -18 -4.5 ]
[.01131 .56871 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21476 .02942 m
.21476 .03567 L
s
[(0.2)] .21476 .01692 0 1 Mshowa
.40572 .02942 m
.40572 .03567 L
s
[(0.4)] .40572 .01692 0 1 Mshowa
.59667 .02942 m
.59667 .03567 L
s
[(0.6)] .59667 .01692 0 1 Mshowa
.78762 .02942 m
.78762 .03567 L
s
[(0.8)] .78762 .01692 0 1 Mshowa
.97858 .02942 m
.97858 .03567 L
s
[(1)] .97858 .01692 0 1 Mshowa
.125 Mabswid
.07155 .02942 m
.07155 .03317 L
s
.11929 .02942 m
.11929 .03317 L
s
.16702 .02942 m
.16702 .03317 L
s
.2625 .02942 m
.2625 .03317 L
s
.31024 .02942 m
.31024 .03317 L
s
.35798 .02942 m
.35798 .03317 L
s
.45346 .02942 m
.45346 .03317 L
s
.50119 .02942 m
.50119 .03317 L
s
.54893 .02942 m
.54893 .03317 L
s
.64441 .02942 m
.64441 .03317 L
s
.69215 .02942 m
.69215 .03317 L
s
.73989 .02942 m
.73989 .03317 L
s
.83536 .02942 m
.83536 .03317 L
s
.8831 .02942 m
.8831 .03317 L
s
.93084 .02942 m
.93084 .03317 L
s
.25 Mabswid
0 .02942 m
1 .02942 L
s
.02381 .13728 m
.03006 .13728 L
s
[(0.1)] .01131 .13728 1 0 Mshowa
.02381 .24514 m
.03006 .24514 L
s
[(0.2)] .01131 .24514 1 0 Mshowa
.02381 .353 m
.03006 .353 L
s
[(0.3)] .01131 .353 1 0 Mshowa
.02381 .46086 m
.03006 .46086 L
s
[(0.4)] .01131 .46086 1 0 Mshowa
.02381 .56871 m
.03006 .56871 L
s
[(0.5)] .01131 .56871 1 0 Mshowa
.125 Mabswid
.02381 .05099 m
.02756 .05099 L
s
.02381 .07256 m
.02756 .07256 L
s
.02381 .09414 m
.02756 .09414 L
s
.02381 .11571 m
.02756 .11571 L
s
.02381 .15885 m
.02756 .15885 L
s
.02381 .18042 m
.02756 .18042 L
s
.02381 .20199 m
.02756 .20199 L
s
.02381 .22357 m
.02756 .22357 L
s
.02381 .26671 m
.02756 .26671 L
s
.02381 .28828 m
.02756 .28828 L
s
.02381 .30985 m
.02756 .30985 L
s
.02381 .33142 m
.02756 .33142 L
s
.02381 .37457 m
.02756 .37457 L
s
.02381 .39614 m
.02756 .39614 L
s
.02381 .41771 m
.02756 .41771 L
s
.02381 .43928 m
.02756 .43928 L
s
.02381 .48243 m
.02756 .48243 L
s
.02381 .504 m
.02756 .504 L
s
.02381 .52557 m
.02756 .52557 L
s
.02381 .54714 m
.02756 .54714 L
s
.02381 .00785 m
.02756 .00785 L
s
.02381 .59029 m
.02756 .59029 L
s
.02381 .61186 m
.02756 .61186 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.03336 .01472 Mdot
.05245 .02296 Mdot
.07155 .03438 Mdot
.09064 .04709 Mdot
.10974 .06053 Mdot
.12883 .07445 Mdot
.14793 .0887 Mdot
.16702 .10319 Mdot
.18612 .11786 Mdot
.20522 .13267 Mdot
.22431 .14758 Mdot
.24341 .16257 Mdot
.2625 .17761 Mdot
.2816 .19268 Mdot
.30069 .20777 Mdot
.31979 .22286 Mdot
.33888 .23794 Mdot
.35798 .253 Mdot
.37707 .26802 Mdot
.39617 .28299 Mdot
.41526 .2979 Mdot
.43436 .31274 Mdot
.45346 .32749 Mdot
.47255 .34215 Mdot
.49165 .35669 Mdot
.51074 .3711 Mdot
.52984 .38536 Mdot
.54893 .39947 Mdot
.56803 .41339 Mdot
.58712 .4271 Mdot
.60622 .44058 Mdot
.62531 .4538 Mdot
.64441 .46673 Mdot
.6635 .47933 Mdot
.6826 .49154 Mdot
.70169 .50333 Mdot
.72079 .51461 Mdot
.73989 .52532 Mdot
.75898 .53536 Mdot
.77808 .54459 Mdot
.79717 .55286 Mdot
.81627 .55996 Mdot
.83536 .5656 Mdot
.85446 .56935 Mdot
.87355 .57062 Mdot
.89265 .56843 Mdot
.91174 .56108 Mdot
.93084 .54522 Mdot
.94993 .5124 Mdot
.96903 .42538 Mdot
.02858 .0179 Mdot
.03813 .02011 Mdot
.04768 .02457 Mdot
.05723 .02995 Mdot
.06677 .03588 Mdot
.07632 .04217 Mdot
.08587 .04872 Mdot
.09542 .05547 Mdot
.10496 .06237 Mdot
.11451 .06939 Mdot
.12406 .07651 Mdot
.13361 .08372 Mdot
.14316 .091 Mdot
.1527 .09835 Mdot
.16225 .10574 Mdot
.1718 .11318 Mdot
.18135 .12066 Mdot
.19089 .12817 Mdot
.20044 .13571 Mdot
.20999 .14328 Mdot
.21954 .15087 Mdot
.22908 .15847 Mdot
.23863 .16609 Mdot
.24818 .17373 Mdot
.25773 .18137 Mdot
.26728 .18903 Mdot
.27682 .19668 Mdot
.28637 .20435 Mdot
.29592 .21201 Mdot
.30547 .21968 Mdot
.31501 .22734 Mdot
.32456 .235 Mdot
.33411 .24266 Mdot
.34366 .25031 Mdot
.3532 .25796 Mdot
.36275 .26559 Mdot
.3723 .27322 Mdot
.38185 .28084 Mdot
.3914 .28844 Mdot
.40094 .29602 Mdot
.41049 .30359 Mdot
.42004 .31115 Mdot
.42959 .31868 Mdot
.43913 .3262 Mdot
.44868 .33369 Mdot
.45823 .34116 Mdot
.46778 .3486 Mdot
.47732 .35602 Mdot
.48687 .36341 Mdot
.49642 .37076 Mdot
.50597 .37809 Mdot
.51551 .38538 Mdot
.52506 .39263 Mdot
.53461 .39985 Mdot
.54416 .40702 Mdot
.55371 .41415 Mdot
.56325 .42124 Mdot
.5728 .42828 Mdot
.58235 .43526 Mdot
.5919 .44219 Mdot
.60144 .44906 Mdot
.61099 .45588 Mdot
.62054 .46262 Mdot
.63009 .4693 Mdot
.63963 .4759 Mdot
.64918 .48243 Mdot
.65873 .48887 Mdot
.66828 .49523 Mdot
.67783 .50149 Mdot
.68737 .50765 Mdot
.69692 .5137 Mdot
.70647 .51963 Mdot
.71602 .52544 Mdot
.72556 .53112 Mdot
.73511 .53665 Mdot
.74466 .54203 Mdot
.75421 .54723 Mdot
.76375 .55225 Mdot
.7733 .55707 Mdot
.78285 .56167 Mdot
.7924 .56603 Mdot
.80195 .57012 Mdot
.81149 .57391 Mdot
.82104 .57737 Mdot
.83059 .58045 Mdot
.84014 .58312 Mdot
.84968 .5853 Mdot
.85923 .58693 Mdot
.86878 .58792 Mdot
.87833 .58815 Mdot
.88787 .58747 Mdot
.89742 .5857 Mdot
.90697 .58256 Mdot
.91652 .57768 Mdot
.92607 .57049 Mdot
.93561 .56011 Mdot
.94516 .54497 Mdot
.95471 .522 Mdot
.96426 .48344 Mdot
.9738 .39517 Mdot
.0262 .02078 Mdot
.03097 .02052 Mdot
.03574 .02184 Mdot
.04052 .02382 Mdot
.04529 .02619 Mdot
.05007 .02881 Mdot
.05484 .03162 Mdot
.05961 .03458 Mdot
.06439 .03765 Mdot
.06916 .04082 Mdot
.07393 .04406 Mdot
.07871 .04737 Mdot
.08348 .05074 Mdot
.08826 .05416 Mdot
.09303 .05762 Mdot
.0978 .06112 Mdot
.10258 .06465 Mdot
.10735 .06821 Mdot
.11213 .0718 Mdot
.1169 .07542 Mdot
.12167 .07906 Mdot
.12645 .08272 Mdot
.13122 .0864 Mdot
.13599 .0901 Mdot
.14077 .09381 Mdot
.14554 .09754 Mdot
.15032 .10128 Mdot
.15509 .10503 Mdot
.15986 .1088 Mdot
.16464 .11257 Mdot
.16941 .11636 Mdot
.17419 .12015 Mdot
.17896 .12396 Mdot
.18373 .12777 Mdot
.18851 .13159 Mdot
.19328 .13542 Mdot
.19805 .13925 Mdot
.20283 .14309 Mdot
.2076 .14693 Mdot
.21238 .15078 Mdot
.21715 .15463 Mdot
.22192 .15849 Mdot
.2267 .16235 Mdot
.23147 .16622 Mdot
.23625 .17008 Mdot
.24102 .17396 Mdot
.24579 .17783 Mdot
.25057 .18171 Mdot
.25534 .18558 Mdot
.26011 .18946 Mdot
.26489 .19335 Mdot
.26966 .19723 Mdot
.27444 .20111 Mdot
.27921 .205 Mdot
.28398 .20888 Mdot
.28876 .21277 Mdot
.29353 .21666 Mdot
.29831 .22054 Mdot
.30308 .22443 Mdot
.30785 .22831 Mdot
.31263 .2322 Mdot
.3174 .23608 Mdot
.32217 .23997 Mdot
.32695 .24385 Mdot
.33172 .24773 Mdot
.3365 .25161 Mdot
.34127 .25549 Mdot
.34604 .25936 Mdot
.35082 .26324 Mdot
.35559 .26711 Mdot
.36037 .27098 Mdot
.36514 .27485 Mdot
.36991 .27871 Mdot
.37469 .28257 Mdot
.37946 .28643 Mdot
.38423 .29028 Mdot
.38901 .29414 Mdot
.39378 .29798 Mdot
.39856 .30183 Mdot
.40333 .30567 Mdot
.4081 .3095 Mdot
.41288 .31334 Mdot
.41765 .31716 Mdot
.42243 .32099 Mdot
.4272 .32481 Mdot
.43197 .32862 Mdot
.43675 .33243 Mdot
.44152 .33623 Mdot
.44629 .34003 Mdot
.45107 .34382 Mdot
.45584 .3476 Mdot
.46062 .35138 Mdot
.46539 .35516 Mdot
.47016 .35892 Mdot
.47494 .36268 Mdot
.47971 .36644 Mdot
.48449 .37018 Mdot
.48926 .37392 Mdot
.49403 .37765 Mdot
.49881 .38137 Mdot
.50358 .38509 Mdot
.50835 .38879 Mdot
.51313 .39249 Mdot
.5179 .39618 Mdot
.52268 .39986 Mdot
.52745 .40353 Mdot
.53222 .40719 Mdot
.537 .41085 Mdot
.54177 .41449 Mdot
.54654 .41812 Mdot
.55132 .42174 Mdot
.55609 .42535 Mdot
.56087 .42894 Mdot
.56564 .43253 Mdot
.57041 .4361 Mdot
.57519 .43967 Mdot
.57996 .44321 Mdot
.58474 .44675 Mdot
.58951 .45027 Mdot
.59428 .45378 Mdot
.59906 .45727 Mdot
.60383 .46075 Mdot
.6086 .46422 Mdot
.61338 .46767 Mdot
.61815 .4711 Mdot
.62293 .47452 Mdot
.6277 .47791 Mdot
.63247 .4813 Mdot
.63725 .48466 Mdot
.64202 .488 Mdot
.6468 .49133 Mdot
.65157 .49464 Mdot
.65634 .49792 Mdot
.66112 .50119 Mdot
.66589 .50443 Mdot
.67066 .50765 Mdot
.67544 .51084 Mdot
.68021 .51402 Mdot
.68499 .51716 Mdot
.68976 .52029 Mdot
.69453 .52338 Mdot
.69931 .52645 Mdot
.70408 .52949 Mdot
.70886 .5325 Mdot
.71363 .53548 Mdot
.7184 .53842 Mdot
.72318 .54134 Mdot
.72795 .54422 Mdot
.73272 .54706 Mdot
.7375 .54987 Mdot
.74227 .55263 Mdot
.74705 .55536 Mdot
.75182 .55805 Mdot
.75659 .56069 Mdot
.76137 .56329 Mdot
.76614 .56583 Mdot
.77092 .56833 Mdot
.77569 .57078 Mdot
.78046 .57317 Mdot
.78524 .57551 Mdot
.79001 .57778 Mdot
.79478 .57999 Mdot
.79956 .58214 Mdot
.80433 .58421 Mdot
.80911 .58622 Mdot
.81388 .58814 Mdot
.81865 .58998 Mdot
.82343 .59173 Mdot
.8282 .59339 Mdot
.83298 .59496 Mdot
.83775 .59641 Mdot
.84252 .59776 Mdot
.8473 .59898 Mdot
.85207 .60008 Mdot
.85684 .60104 Mdot
.86162 .60184 Mdot
.86639 .60249 Mdot
.87117 .60296 Mdot
.87594 .60324 Mdot
.88071 .60332 Mdot
.88549 .60316 Mdot
.89026 .60276 Mdot
.89504 .60208 Mdot
.89981 .60108 Mdot
.90458 .59974 Mdot
.90936 .59801 Mdot
.91413 .59582 Mdot
.9189 .59313 Mdot
.92368 .58984 Mdot
.92845 .58585 Mdot
.93323 .58103 Mdot
.938 .57519 Mdot
.94277 .56809 Mdot
.94755 .55938 Mdot
.95232 .54855 Mdot
.9571 .53478 Mdot
.96187 .51665 Mdot
.96664 .49131 Mdot
.97142 .45166 Mdot
.97619 .36639 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00@P3oool2000000@0oooo00<0
00000?ooo`3oool00P3oool4000002D0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0903oool2000000@0oooo00<000000?ooo`3oool00`3oool2000002H0oooo0P00
00040?ooo`030000003oool0oooo00<0oooo0P00000[0?ooo`@000001@3oool00440oooo00@00000
0?ooo`3oool00000203oool00`000000oooo0?ooo`0U0?ooo`040000003oool0oooo000000X0oooo
00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo0000
02@0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000/0?ooo`030000003o
ool0oooo00@0oooo000E0?ooo`030000003oool0oooo02T0oooo00@000000?ooo`3oool000002@3o
ool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo000000L0oooo1@00000T0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool00000903oool010000000oooo0?ooo`000008
0?ooo`040000003oool0oooo000002`0oooo00<000000?ooo`3oool0103oool001D0oooo00<00000
0?ooo`3oool0:@3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0oooo02<0oooo00@0
00000?ooo`3oool000001`3oool010000000oooo0?ooo`00000U0?ooo`040000003oool0oooo0000
00P0oooo0`00000U0?ooo`040000003oool0oooo000000T0oooo0P00000]0?ooo`030000003oool0
oooo00@0oooo000E0?ooo`800000:P3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0
oooo0280oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000U0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool09@3oool010000000oooo0?ooo`0000080?ooo`04
0000003oool0oooo000002`0oooo00<000000?ooo`3oool0103oool001D0oooo00<000000?ooo`3o
ool0:@3oool010000000oooo0?ooo`0000080?ooo`040000003oool0oooo000002@0oooo00@00000
0?ooo`3oool000002@3oool2000002D0oooo00@000000?ooo`3oool00000203oool00`000000oooo
0?ooo`0U0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool00000:P3oool30000
00H0oooo000E0?ooo`040000003oool00000000002T0oooo0P00000:0?ooo`8000009P3oool20000
00/0oooo00<000000?ooo`3oool0903oool2000000X0oooo0`00000U0?ooo`8000002P3oool20000
02d0oooo00<000000?ooo`3oool0103oool001D0oooo1@00003o0?ooo`L0oooo000E0?ooo`T00000
o`3oool30?ooo`005@3oool010000000oooo0?ooo`3oool500000?l0oooo0`3oool000h0ooooo`00
000C000000005@3oool00`000000oooo0?ooo`040?ooo`030000003oool0000000@00000303oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`040?ooo`005@3oool00`000000oooo0?ooo`070?ooo`@000009P3oool00`00
0000oooo0?ooo`0a0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0<@3oool00`00
0000oooo0?ooo`0a0?ooo`030000003oool0oooo00@0oooo000E0?ooo`030000003oool0oooo00P0
oooo1@00003k0?ooo`005@3oool00`000000oooo0?ooo`0:0?ooo`<00000n`3oool001D0oooo00<0
00000?ooo`3oool02`3oool600000?L0oooo000E0?ooo`8000003@3oool500000?L0oooo000E0?oo
o`030000003oool0oooo00h0oooo1000003f0?ooo`005@3oool00`000000oooo0?ooo`0@0?ooo`80
00000P3oool200000?80oooo000E0?ooo`030000003oool0oooo0140oooo0`0000000`3oool00000
0?ooo`3a0?ooo`005@3oool00`000000oooo0?ooo`0C0?ooo`800000l`3oool001D0oooo00<00000
0?ooo`3oool0503oool300000?40oooo000E0?ooo`8000005P3oool3000000030?ooo`0000000000
0>d0oooo000E0?ooo`030000003oool0oooo01L0oooo1000003]0?ooo`005@3oool00`000000oooo
0?ooo`0H0?ooo`@00000k03oool001D0oooo00<000000?ooo`3oool06@3oool300000>`0oooo000E
0?ooo`030000003oool0oooo01/0oooo1@00003X0?ooo`005@3oool00`000000oooo0?ooo`0L0?oo
o`@00000j03oool001D0oooo0P00000N0?ooo`@00000i`3oool001D0oooo00<000000?ooo`3oool0
7P3oool300000>L0oooo000E0?ooo`030000003oool0oooo0200oooo1@00003S0?ooo`005@3oool0
0`000000oooo0?ooo`0Q0?ooo`@00000h`3oool001D0oooo00<000000?ooo`3oool08P3oool30000
0><0oooo000E0?ooo`030000003oool0oooo02@0oooo0`00003Q0?ooo`005@3oool2000002H0oooo
0P0000020?ooo`800000g@3oool001D0oooo00<000000?ooo`3oool09P3oool3000000030?ooo`00
0000oooo0=`0oooo000E0?ooo`030000003oool0oooo02P0oooo0P00003N0?ooo`005@3oool00`00
0000oooo0?ooo`0Y0?ooo`<00000g03oool00080oooo0P0000040?ooo`030000003oool0oooo0080
oooo100000040?ooo`030000003oool0oooo02T0oooo0`0000020?ooo`800000f03oool000050?oo
o`000000oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02X0
oooo100000000`3oool000000?ooo`3G0?ooo`0000D0oooo0000003oool0oooo0000000:0?ooo`03
0000003oool0oooo00<0oooo0`00000[0?ooo`<00000fP3oool000050?ooo`000000oooo0?ooo`00
00002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02d0oooo1000003G0?ooo`00
00D0oooo0000003oool0oooo0000000:0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0;P3oool300000080oooo0P00003C0?ooo`0000D0oooo0000003oool0oooo000000080?ooo`<0
00001@3oool00`000000oooo0?ooo`0_0?ooo`H00000d`3oool00080oooo0P00000;0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0<@3oool300000=@0oooo000E0?ooo`800000<`3o
ool400000=80oooo000E0?ooo`030000003oool0oooo03<0oooo0`0000020?ooo`800000cP3oool0
01D0oooo00<000000?ooo`3oool0=@3oool500000<h0oooo000E0?ooo`030000003oool0oooo03H0
oooo0`00003?0?ooo`005@3oool00`000000oooo0?ooo`0g0?ooo`@00000c@3oool001D0oooo00<0
00000?ooo`3oool0>03oool300000080oooo0P0000390?ooo`005@3oool2000003/0oooo1@000039
0?ooo`005@3oool00`000000oooo0?ooo`0k0?ooo`<00000bP3oool001D0oooo00<000000?ooo`3o
ool0?03oool500000<L0oooo000E0?ooo`030000003oool0oooo03h0oooo0`0000000`3oool00000
000000340?ooo`005@3oool00`000000oooo0?ooo`0o0?ooo`D00000a03oool001D0oooo00<00000
0?ooo`3oool0@03oool300000<D0oooo000E0?ooo`800000@`3oool400000<80oooo000E0?ooo`03
0000003oool0oooo04<0oooo00<000000?ooo`0000000P3oool200000;h0oooo000E0?ooo`030000
003oool0oooo04<0oooo0P0000001@3oool000000000003oool000000;h0oooo000E0?ooo`030000
003oool0oooo04@0oooo0P0000000`3oool000000?ooo`2o0?ooo`005@3oool00`000000oooo0?oo
o`150?ooo`<0000000<0oooo000000000000_@3oool001D0oooo00<000000?ooo`3oool0A`3oool2
000000040?ooo`000000oooo0?ooo`800000^@3oool001D0oooo0P0000190?ooo`800000103oool0
0`000000oooo0?ooo`2g0?ooo`005@3oool00`000000oooo0?ooo`190?ooo`D00000^P3oool001D0
oooo00<000000?ooo`3oool0B`3oool300000;X0oooo000E0?ooo`030000003oool0oooo04`0oooo
100000020?ooo`800000]03oool00080oooo0P0000040?ooo`030000003oool0oooo0080oooo1000
00040?ooo`030000003oool0oooo04d0oooo0`0000030?ooo`030000003oool0oooo0;80oooo0000
1@3oool000000?ooo`3oool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`1?0?ooo`@00000]@3oool000050?ooo`000000oooo0?ooo`0000002@3oool00`000000oooo0?oo
o`040?ooo`<00000D03oool300000;D0oooo00001@3oool000000?ooo`3oool0000000X0oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`1A0?ooo`@00000/`3oool000050?ooo`000000
oooo0?ooo`0000002`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0580oooo0`00
00020?ooo`800000[`3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?oo
o`0000040?ooo`030000003oool0oooo05@0oooo1@00002_0?ooo`000P3oool2000000X0oooo0P00
00050?ooo`030000003oool0oooo05D0oooo0`00002`0?ooo`005@3oool00`000000oooo0?ooo`1F
0?ooo`D00000[@3oool001D0oooo0P00001I0?ooo`<0000000<0oooo000000000000ZP3oool001D0
oooo00<000000?ooo`3oool0F@3oool500000:X0oooo000E0?ooo`030000003oool0oooo05X0oooo
00<000000?ooo`000000Z`3oool001D0oooo00<000000?ooo`3oool0F`3oool2000000030?ooo`00
000000000:P0oooo000E0?ooo`030000003oool0oooo05`0oooo0P000000103oool000000?ooo`3o
ool200000:@0oooo000E0?ooo`030000003oool0oooo05d0oooo0P0000001@3oool000000000003o
ool000000:@0oooo000E0?ooo`800000G`3oool2000000030?ooo`000000oooo0:D0oooo000E0?oo
o`030000003oool0oooo05l0oooo0`0000000`3oool000000000002S0?ooo`005@3oool00`000000
oooo0?ooo`1Q0?ooo`80000000@0oooo0000003oool0oooo0P00002O0?ooo`005@3oool00`000000
oooo0?ooo`1R0?ooo`8000000P3oool3000009l0oooo000E0?ooo`030000003oool0oooo06<0oooo
0`0000000`3oool000000?ooo`2O0?ooo`005@3oool2000006H0oooo0P0000000`3oool000000000
002N0?ooo`005@3oool00`000000oooo0?ooo`1V0?ooo`80000000@0oooo0000003oool0oooo0P00
002J0?ooo`005@3oool00`000000oooo0?ooo`1W0?ooo`8000000P3oool3000009X0oooo000E0?oo
o`030000003oool0oooo06P0oooo0`0000000`3oool000000?ooo`2J0?ooo`005@3oool00`000000
oooo0?ooo`1Z0?ooo`80000000<0oooo000000000000V@3oool001D0oooo00<000000?ooo`3oool0
J`3oool2000000040?ooo`000000oooo0?ooo`800000U@3oool001D0oooo0P00001]0?ooo`<00000
0`3oool00`000000oooo0?ooo`2C0?ooo`005@3oool00`000000oooo0?ooo`1^0?ooo`@00000UP3o
ool001D0oooo00<000000?ooo`3oool0K`3oool3000009H0oooo000E0?ooo`030000003oool0oooo
0700oooo1@0000000`3oool000000000002@0?ooo`000P3oool2000000@0oooo00<000000?ooo`3o
ool00`3oool2000000D0oooo00<000000?ooo`3oool0LP3oool300000080oooo00<000000?ooo`3o
ool0SP3oool000050?ooo`000000oooo0?ooo`000000203oool010000000oooo0?ooo`0000040?oo
o`030000003oool0oooo07<0oooo1000002A0?ooo`0000D0oooo0000003oool0oooo0000000;0?oo
o`030000003oool0oooo0080oooo0`00001d0?ooo`<00000T@3oool000050?ooo`000000oooo0?oo
o`0000002@3oool2000000D0oooo00<000000?ooo`3oool0M@3oool500000080oooo0P00002:0?oo
o`0000D0oooo0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0M`3oool3000000<0oooo00<000000?ooo`3oool0R03oool000050?ooo`000000oooo0?oo
o`000000203oool010000000oooo0?ooo`0000040?ooo`030000003oool0oooo07P0oooo10000023
0?ooo`8000001`3oool00080oooo0P00000:0?ooo`8000001@3oool00`000000oooo0?ooo`1i0?oo
o`<00000Q03oool00`000000oooo0?ooo`050?ooo`005@3oool00`000000oooo0?ooo`1k0?ooo`@0
00000P3oool2000008D0oooo000E0?ooo`800000O@3oool3000000<0oooo00<000000?ooo`3oool0
P`3oool001D0oooo00<000000?ooo`3oool0O@3oool5000008H0oooo000E0?ooo`030000003oool0
oooo07l0oooo0`0000260?ooo`005@3oool00`000000oooo0?ooo`200?ooo`@000000P3oool20000
0800oooo000E0?ooo`030000003oool0oooo0840oooo00<000000?ooo`0000000`3oool00`000000
oooo0?ooo`1n0?ooo`005@3oool00`000000oooo0?ooo`210?ooo`8000000P3oool2000007P0oooo
0P0000070?ooo`005@3oool2000008<0oooo0`0000000`3oool000000?ooo`030?ooo`800000L`3o
ool00`000000oooo0?ooo`050?ooo`005@3oool00`000000oooo0?ooo`240?ooo`80000000<0oooo
0000000000000`3oool00`000000oooo0?ooo`1i0?ooo`005@3oool00`000000oooo0?ooo`250?oo
o`80000000<0oooo0000003oool00P00001l0?ooo`005@3oool00`000000oooo0?ooo`260?ooo`@0
000000<0oooo0000003oool0N`3oool001D0oooo00<000000?ooo`3oool0R03oool300000080oooo
0P0000000`3oool000000000001f0?ooo`005@3oool00`000000oooo0?ooo`2:0?ooo`<0000000D0
oooo0000003oool0oooo0000001f0?ooo`005@3oool2000008d0oooo0P0000000`3oool000000000
001g0?ooo`005@3oool00`000000oooo0?ooo`2=0?ooo`80000000<0oooo0000003oool0K03oool2
000000P0oooo000E0?ooo`030000003oool0oooo08h0oooo0P0000020?ooo`8000000P3oool20000
06L0oooo00<000000?ooo`3oool01P3oool001D0oooo00<000000?ooo`3oool0S`3oool300000003
0?ooo`000000oooo0080oooo00<000000?ooo`3oool0KP3oool001D0oooo00<000000?ooo`3oool0
T@3oool2000000030?ooo`00000000000780oooo000E0?ooo`030000003oool0oooo0980oooo0P00
00000`3oool000000?ooo`1a0?ooo`005@3oool2000009@0oooo0`0000000`3oool0000000000002
0?ooo`800000J`3oool001D0oooo00<000000?ooo`3oool0U@3oool2000000030?ooo`000000oooo
0080oooo00<000000?ooo`3oool0J@3oool001D0oooo00<000000?ooo`3oool0UP3oool200000080
oooo0P00001/0?ooo`005@3oool00`000000oooo0?ooo`2G0?ooo`<0000000<0oooo0000003oool0
0`3oool2000005`0oooo0P0000080?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo09T0oooo0P0000000`3oool000000000
00030?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool01P3oool000050?ooo`000000
oooo0?ooo`0000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo09X0oooo0P00
00000`3oool000000?ooo`1Y0?ooo`0000D0oooo0000003oool0oooo000000070?ooo`D00000103o
ool3000009/0oooo0P0000020?ooo`800000I`3oool000050?ooo`000000oooo0?ooo`0000001`3o
ool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo09`0oooo0`000000103oool00000
0000000000020?ooo`800000H@3oool000050?ooo`000000oooo0?ooo`000000203oool00`000000
oooo000000050?ooo`030000003oool0oooo09h0oooo0P0000020?ooo`050000003oool0oooo0?oo
o`000000H@3oool000050?ooo`000000oooo0?ooo`0000002@3oool2000000D0oooo00<000000?oo
o`3oool0W`3oool2000000<0oooo0P00001R0?ooo`000P3oool2000000/0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`2P0?ooo`<000000P3oool00`000000oooo0?ooo`020?ooo`80
0000G03oool001D0oooo0P00002S0?ooo`8000000P3oool2000000<0oooo00<000000?ooo`3oool0
CP3oool2000000X0oooo000E0?ooo`030000003oool0oooo0:<0oooo1@00001E0?ooo`030000003o
ool0oooo00P0oooo000E0?ooo`030000003oool0oooo0:@0oooo00D000000?ooo`00000000000?oo
o`02000000D0oooo0P00001;0?ooo`8000002@3oool001D0oooo00<000000?ooo`3oool0Y`3oool2
000000030?ooo`000000oooo008000000`3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0
oooo00L0oooo000E0?ooo`030000003oool0oooo0:P0oooo0P0000030?ooo`030000003oool0oooo
05P0oooo000E0?ooo`030000003oool0oooo0:T0oooo0`0000020?ooo`800000F03oool001D0oooo
0P00002/0?ooo`8000000P3oool00`000000oooo0?ooo`030?ooo`800000D@3oool001D0oooo00<0
00000?ooo`3oool0[03oool2000000<0oooo0P0000030?ooo`030000003oool0oooo04l0oooo000E
0?ooo`030000003oool0oooo0:d0oooo0`0000020?ooo`<00000@`3oool2000000h0oooo000E0?oo
o`030000003oool0oooo0:l0oooo0`0000020?ooo`030000003oool0oooo00<0oooo0P00000m0?oo
o`030000003oool0oooo008000002P3oool001D0oooo00<000000?ooo`3oool0/03oool400000080
oooo0P0000030?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0203oool001D0oooo
00<000000?ooo`3oool0/`3oool200000080oooo00<000000?ooo`3oool0@03oool2000000`0oooo
000E0?ooo`800000]@3oool2000000<0oooo0P0000040?ooo`800000>P3oool00`000000oooo0?oo
o`0:0?ooo`005@3oool00`000000oooo0?ooo`2e0?ooo`8000000`3oool3000000<0oooo00<00000
0?ooo`3oool0A@3oool001D0oooo00<000000?ooo`3oool0]P3oool400000080oooo00<000000?oo
o`3oool01@3oool2000003@0oooo0P00000<0?ooo`005@3oool00`000000oooo0?ooo`2h0?ooo`<0
00000`3oool2000000D0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0:0?ooo`00
5@3oool00`000000oooo0?ooo`2j0?ooo`<000000P3oool3000004H0oooo000E0?ooo`030000003o
ool0oooo0;`0oooo0P0000030?ooo`030000003oool0oooo00H0oooo0P00000W0?ooo`8000000P3o
ool2000000l0oooo000E0?ooo`800000_P3oool400000080oooo0P0000060?ooo`030000003oool0
oooo02H0oooo00@000000?ooo`3oool0oooo0`00000=0?ooo`005@3oool00`000000oooo0?ooo`2n
0?ooo`040000003oool0000000000080oooo00@000000?ooo`00000000001`3oool2000002T0oooo
00<000000?ooo`3oool02`3oool001D0oooo00<000000?ooo`3oool0`@3oool2000000@0oooo0`00
00060?ooo`030000003oool0oooo03D0oooo000E0?ooo`030000003oool0oooo0<80oooo10000003
0?ooo`030000003oool0oooo00P0oooo0P00000H0?ooo`800000103oool200000080oooo0P00000>
0?ooo`000P3oool2000000@0oooo00<000000?ooo`3oool00P3oool3000000D0oooo00<000000?oo
o`3oool0``3oool010000000oooo0000000000040?ooo`800000203oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo00<0oooo00D000000?ooo`3oool0oooo0000000>0?ooo`0000D0oooo
0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0aP3o
ool400000080oooo0`00000:0?ooo`8000003P3oool2000000/0oooo0P00000@0?ooo`0000D0oooo
0000003oool0oooo0000000;0?ooo`030000003oool0oooo0080oooo0`0000370?ooo`040000003o
ool00000000000<0oooo00@000000?ooo`0000000000203oool010000000oooo0?ooo`3oool20000
00@0oooo0P0000040?ooo`030000003oool0oooo00D0oooo0P0000030?ooo`030000003oool0oooo
00h0oooo00001@3oool000000?ooo`3oool0000000P0oooo0`0000050?ooo`030000003oool0oooo
0<X0oooo100000030?ooo`040000003oool00000000000X0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0;0?ooo`040000003oool0000000000140oooo00001@3oool000000?ooo`3o
ool0000000P0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`3;0?ooo`040000003o
ool00000000000D0oooo0`00000H0?ooo`8000001@3oool00`000000oooo0?ooo`0?0?ooo`0000D0
oooo0000003oool0oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
cP3oool3000000D0oooo00<000000?ooo`0000000`00000A0?ooo`80000000<0oooo0000003oool0
0P3oool200000180oooo00020?ooo`8000002@3oool4000000@0oooo00<000000?ooo`3oool0c`3o
ool5000000D0oooo00@000000?ooo`000000oooo0P0000000`3oool00000000000080?ooo`800000
0P3oool010000000oooo0?ooo`3oool2000000030?ooo`000000oooo0140oooo000E0?ooo`030000
003oool0oooo0=80oooo1@0000070?ooo`030000003oool0oooo00<0000000<0oooo000000000000
0P0000020?ooo`030000003oool0oooo00<0oooo0`00000D0?ooo`005@3oool200000=D0oooo00<0
00000?ooo`0000000P00000:0?ooo`060000003oool0oooo0000003oool000001`3oool3000001D0
oooo000E0?ooo`030000003oool0oooo0=L0oooo1P00000@0?ooo`D000005P3oool001D0oooo00<0
00000?ooo`3oool0f@3oool00`000000oooo00000006000000T0oooo100000000`3oool000000?oo
o`0F0?ooo`005@3oool00`000000oooo0?ooo`3M0?ooo`030000003oool0000000/0000000<0oooo
0000000000006P3oool001D0oooo00<000000?ooo`3oool0h`3oool4000000040?ooo`0000000000
000001d0oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool001D0oooo0P00003o0?ooo`X0
oooo000E0?ooo`030000003oool0oooo0?l0oooo2@3oool00?l0oooo8@3oool00?l0oooo8@3oool0
0001\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.0823894, -0.049739, \
0.00384959, 0.00340766}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[ListPlot[ftasave[50], DisplayFunction \[Rule] Identity], 
      ListPlot[ftasave[100], DisplayFunction \[Rule] Identity], 
      ListPlot[ftasave[200], 
        DisplayFunction \[Rule] 
          Identity], \[IndentingNewLine]DisplayFunction \[Rule] \
$DisplayFunction]\)], "Input"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.954768 0.341502 1.57321 [
[.21476 .329 -9 -9 ]
[.21476 .329 9 0 ]
[.40572 .329 -9 -9 ]
[.40572 .329 9 0 ]
[.59667 .329 -9 -9 ]
[.59667 .329 9 0 ]
[.78762 .329 -9 -9 ]
[.78762 .329 9 0 ]
[.97858 .329 -3 -9 ]
[.97858 .329 3 0 ]
[.01131 .02686 -24 -4.5 ]
[.01131 .02686 0 4.5 ]
[.01131 .10552 -30 -4.5 ]
[.01131 .10552 0 4.5 ]
[.01131 .18418 -24 -4.5 ]
[.01131 .18418 0 4.5 ]
[.01131 .26284 -30 -4.5 ]
[.01131 .26284 0 4.5 ]
[.01131 .42016 -24 -4.5 ]
[.01131 .42016 0 4.5 ]
[.01131 .49882 -18 -4.5 ]
[.01131 .49882 0 4.5 ]
[.01131 .57748 -24 -4.5 ]
[.01131 .57748 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21476 .3415 m
.21476 .34775 L
s
[(0.2)] .21476 .329 0 1 Mshowa
.40572 .3415 m
.40572 .34775 L
s
[(0.4)] .40572 .329 0 1 Mshowa
.59667 .3415 m
.59667 .34775 L
s
[(0.6)] .59667 .329 0 1 Mshowa
.78762 .3415 m
.78762 .34775 L
s
[(0.8)] .78762 .329 0 1 Mshowa
.97858 .3415 m
.97858 .34775 L
s
[(1)] .97858 .329 0 1 Mshowa
.125 Mabswid
.07155 .3415 m
.07155 .34525 L
s
.11929 .3415 m
.11929 .34525 L
s
.16702 .3415 m
.16702 .34525 L
s
.2625 .3415 m
.2625 .34525 L
s
.31024 .3415 m
.31024 .34525 L
s
.35798 .3415 m
.35798 .34525 L
s
.45346 .3415 m
.45346 .34525 L
s
.50119 .3415 m
.50119 .34525 L
s
.54893 .3415 m
.54893 .34525 L
s
.64441 .3415 m
.64441 .34525 L
s
.69215 .3415 m
.69215 .34525 L
s
.73989 .3415 m
.73989 .34525 L
s
.83536 .3415 m
.83536 .34525 L
s
.8831 .3415 m
.8831 .34525 L
s
.93084 .3415 m
.93084 .34525 L
s
.25 Mabswid
0 .3415 m
1 .3415 L
s
.02381 .02686 m
.03006 .02686 L
s
[(-0.2)] .01131 .02686 1 0 Mshowa
.02381 .10552 m
.03006 .10552 L
s
[(-0.15)] .01131 .10552 1 0 Mshowa
.02381 .18418 m
.03006 .18418 L
s
[(-0.1)] .01131 .18418 1 0 Mshowa
.02381 .26284 m
.03006 .26284 L
s
[(-0.05)] .01131 .26284 1 0 Mshowa
.02381 .42016 m
.03006 .42016 L
s
[(0.05)] .01131 .42016 1 0 Mshowa
.02381 .49882 m
.03006 .49882 L
s
[(0.1)] .01131 .49882 1 0 Mshowa
.02381 .57748 m
.03006 .57748 L
s
[(0.15)] .01131 .57748 1 0 Mshowa
.125 Mabswid
.02381 .04259 m
.02756 .04259 L
s
.02381 .05832 m
.02756 .05832 L
s
.02381 .07406 m
.02756 .07406 L
s
.02381 .08979 m
.02756 .08979 L
s
.02381 .12125 m
.02756 .12125 L
s
.02381 .13698 m
.02756 .13698 L
s
.02381 .15272 m
.02756 .15272 L
s
.02381 .16845 m
.02756 .16845 L
s
.02381 .19991 m
.02756 .19991 L
s
.02381 .21564 m
.02756 .21564 L
s
.02381 .23138 m
.02756 .23138 L
s
.02381 .24711 m
.02756 .24711 L
s
.02381 .27857 m
.02756 .27857 L
s
.02381 .29431 m
.02756 .29431 L
s
.02381 .31004 m
.02756 .31004 L
s
.02381 .32577 m
.02756 .32577 L
s
.02381 .35723 m
.02756 .35723 L
s
.02381 .37297 m
.02756 .37297 L
s
.02381 .3887 m
.02756 .3887 L
s
.02381 .40443 m
.02756 .40443 L
s
.02381 .43589 m
.02756 .43589 L
s
.02381 .45163 m
.02756 .45163 L
s
.02381 .46736 m
.02756 .46736 L
s
.02381 .48309 m
.02756 .48309 L
s
.02381 .51456 m
.02756 .51456 L
s
.02381 .53029 m
.02756 .53029 L
s
.02381 .54602 m
.02756 .54602 L
s
.02381 .56175 m
.02756 .56175 L
s
.02381 .01113 m
.02756 .01113 L
s
.02381 .59322 m
.02756 .59322 L
s
.02381 .60895 m
.02756 .60895 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.03336 .16172 Mdot
.05245 .09736 Mdot
.07155 .06309 Mdot
.09064 .04177 Mdot
.10974 .02822 Mdot
.12883 .02003 Mdot
.14793 .01581 Mdot
.16702 .01472 Mdot
.18612 .01614 Mdot
.20522 .01965 Mdot
.22431 .02494 Mdot
.24341 .03175 Mdot
.2625 .03989 Mdot
.2816 .04919 Mdot
.30069 .05952 Mdot
.31979 .07078 Mdot
.33888 .08286 Mdot
.35798 .09569 Mdot
.37707 .10919 Mdot
.39617 .12329 Mdot
.41526 .13796 Mdot
.43436 .15313 Mdot
.45346 .16875 Mdot
.47255 .18479 Mdot
.49165 .20121 Mdot
.51074 .21798 Mdot
.52984 .23505 Mdot
.54893 .2524 Mdot
.56803 .26999 Mdot
.58712 .28779 Mdot
.60622 .30578 Mdot
.62531 .32392 Mdot
.64441 .34217 Mdot
.6635 .36052 Mdot
.6826 .37891 Mdot
.70169 .3973 Mdot
.72079 .41566 Mdot
.73989 .43393 Mdot
.75898 .45205 Mdot
.77808 .46995 Mdot
.79717 .48753 Mdot
.81627 .50467 Mdot
.83536 .52121 Mdot
.85446 .53693 Mdot
.87355 .55151 Mdot
.89265 .56441 Mdot
.91174 .57475 Mdot
.93084 .58075 Mdot
.94993 .57794 Mdot
.96903 .5466 Mdot
.02858 .2083 Mdot
.03813 .15451 Mdot
.04768 .1223 Mdot
.05723 .0993 Mdot
.06677 .08179 Mdot
.07632 .06805 Mdot
.08587 .05711 Mdot
.09542 .04836 Mdot
.10496 .04138 Mdot
.11451 .03588 Mdot
.12406 .03163 Mdot
.13361 .02846 Mdot
.14316 .02623 Mdot
.1527 .02484 Mdot
.16225 .0242 Mdot
.1718 .02423 Mdot
.18135 .02486 Mdot
.19089 .02605 Mdot
.20044 .02775 Mdot
.20999 .02991 Mdot
.21954 .03251 Mdot
.22908 .0355 Mdot
.23863 .03886 Mdot
.24818 .04258 Mdot
.25773 .04661 Mdot
.26728 .05095 Mdot
.27682 .05557 Mdot
.28637 .06047 Mdot
.29592 .06561 Mdot
.30547 .071 Mdot
.31501 .07661 Mdot
.32456 .08243 Mdot
.33411 .08846 Mdot
.34366 .09468 Mdot
.3532 .10109 Mdot
.36275 .10767 Mdot
.3723 .11441 Mdot
.38185 .12131 Mdot
.3914 .12836 Mdot
.40094 .13556 Mdot
.41049 .14289 Mdot
.42004 .15035 Mdot
.42959 .15794 Mdot
.43913 .16564 Mdot
.44868 .17346 Mdot
.45823 .18139 Mdot
.46778 .18942 Mdot
.47732 .19754 Mdot
.48687 .20576 Mdot
.49642 .21407 Mdot
.50597 .22247 Mdot
.51551 .23094 Mdot
.52506 .23949 Mdot
.53461 .24811 Mdot
.54416 .2568 Mdot
.55371 .26556 Mdot
.56325 .27437 Mdot
.5728 .28324 Mdot
.58235 .29216 Mdot
.5919 .30114 Mdot
.60144 .31015 Mdot
.61099 .31921 Mdot
.62054 .32831 Mdot
.63009 .33744 Mdot
.63963 .34659 Mdot
.64918 .35578 Mdot
.65873 .36498 Mdot
.66828 .3742 Mdot
.67783 .38343 Mdot
.68737 .39266 Mdot
.69692 .4019 Mdot
.70647 .41114 Mdot
.71602 .42036 Mdot
.72556 .42957 Mdot
.73511 .43875 Mdot
.74466 .4479 Mdot
.75421 .45702 Mdot
.76375 .46608 Mdot
.7733 .47509 Mdot
.78285 .48403 Mdot
.7924 .49288 Mdot
.80195 .50165 Mdot
.81149 .51029 Mdot
.82104 .51881 Mdot
.83059 .52717 Mdot
.84014 .53535 Mdot
.84968 .54332 Mdot
.85923 .55103 Mdot
.86878 .55844 Mdot
.87833 .56549 Mdot
.88787 .5721 Mdot
.89742 .57817 Mdot
.90697 .58355 Mdot
.91652 .58803 Mdot
.92607 .59133 Mdot
.93561 .59297 Mdot
.94516 .59212 Mdot
.95471 .58714 Mdot
.96426 .5739 Mdot
.9738 .53421 Mdot
.0262 .24358 Mdot
.03097 .20114 Mdot
.03574 .17425 Mdot
.04052 .15397 Mdot
.04529 .1376 Mdot
.05007 .12391 Mdot
.05484 .11222 Mdot
.05961 .10207 Mdot
.06439 .09317 Mdot
.06916 .08531 Mdot
.07393 .07833 Mdot
.07871 .07211 Mdot
.08348 .06656 Mdot
.08826 .06159 Mdot
.09303 .05715 Mdot
.0978 .05318 Mdot
.10258 .04964 Mdot
.10735 .04648 Mdot
.11213 .04369 Mdot
.1169 .04122 Mdot
.12167 .03906 Mdot
.12645 .03718 Mdot
.13122 .03557 Mdot
.13599 .0342 Mdot
.14077 .03306 Mdot
.14554 .03214 Mdot
.15032 .03143 Mdot
.15509 .03091 Mdot
.15986 .03057 Mdot
.16464 .0304 Mdot
.16941 .0304 Mdot
.17419 .03056 Mdot
.17896 .03087 Mdot
.18373 .03132 Mdot
.18851 .0319 Mdot
.19328 .03262 Mdot
.19805 .03346 Mdot
.20283 .03442 Mdot
.2076 .0355 Mdot
.21238 .03668 Mdot
.21715 .03798 Mdot
.22192 .03937 Mdot
.2267 .04087 Mdot
.23147 .04246 Mdot
.23625 .04414 Mdot
.24102 .04591 Mdot
.24579 .04776 Mdot
.25057 .0497 Mdot
.25534 .05172 Mdot
.26011 .05382 Mdot
.26489 .05599 Mdot
.26966 .05823 Mdot
.27444 .06055 Mdot
.27921 .06293 Mdot
.28398 .06538 Mdot
.28876 .06789 Mdot
.29353 .07047 Mdot
.29831 .07311 Mdot
.30308 .07581 Mdot
.30785 .07856 Mdot
.31263 .08137 Mdot
.3174 .08424 Mdot
.32217 .08716 Mdot
.32695 .09013 Mdot
.33172 .09315 Mdot
.3365 .09622 Mdot
.34127 .09933 Mdot
.34604 .1025 Mdot
.35082 .10571 Mdot
.35559 .10896 Mdot
.36037 .11226 Mdot
.36514 .1156 Mdot
.36991 .11898 Mdot
.37469 .1224 Mdot
.37946 .12586 Mdot
.38423 .12935 Mdot
.38901 .13289 Mdot
.39378 .13646 Mdot
.39856 .14007 Mdot
.40333 .14371 Mdot
.4081 .14738 Mdot
.41288 .15109 Mdot
.41765 .15483 Mdot
.42243 .15861 Mdot
.4272 .16241 Mdot
.43197 .16624 Mdot
.43675 .17011 Mdot
.44152 .174 Mdot
.44629 .17792 Mdot
.45107 .18187 Mdot
.45584 .18584 Mdot
.46062 .18984 Mdot
.46539 .19387 Mdot
.47016 .19792 Mdot
.47494 .202 Mdot
.47971 .2061 Mdot
.48449 .21022 Mdot
.48926 .21436 Mdot
.49403 .21853 Mdot
.49881 .22272 Mdot
.50358 .22693 Mdot
.50835 .23116 Mdot
.51313 .23541 Mdot
.5179 .23968 Mdot
.52268 .24397 Mdot
.52745 .24828 Mdot
.53222 .25261 Mdot
.537 .25695 Mdot
.54177 .26131 Mdot
.54654 .26569 Mdot
.55132 .27008 Mdot
.55609 .27449 Mdot
.56087 .27891 Mdot
.56564 .28335 Mdot
.57041 .2878 Mdot
.57519 .29227 Mdot
.57996 .29674 Mdot
.58474 .30124 Mdot
.58951 .30574 Mdot
.59428 .31025 Mdot
.59906 .31478 Mdot
.60383 .31932 Mdot
.6086 .32387 Mdot
.61338 .32842 Mdot
.61815 .33299 Mdot
.62293 .33756 Mdot
.6277 .34215 Mdot
.63247 .34674 Mdot
.63725 .35134 Mdot
.64202 .35594 Mdot
.6468 .36056 Mdot
.65157 .36517 Mdot
.65634 .3698 Mdot
.66112 .37442 Mdot
.66589 .37906 Mdot
.67066 .38369 Mdot
.67544 .38833 Mdot
.68021 .39297 Mdot
.68499 .39761 Mdot
.68976 .40225 Mdot
.69453 .4069 Mdot
.69931 .41154 Mdot
.70408 .41618 Mdot
.70886 .42082 Mdot
.71363 .42546 Mdot
.7184 .4301 Mdot
.72318 .43473 Mdot
.72795 .43935 Mdot
.73272 .44397 Mdot
.7375 .44859 Mdot
.74227 .45319 Mdot
.74705 .45779 Mdot
.75182 .46238 Mdot
.75659 .46696 Mdot
.76137 .47152 Mdot
.76614 .47608 Mdot
.77092 .48061 Mdot
.77569 .48514 Mdot
.78046 .48964 Mdot
.78524 .49413 Mdot
.79001 .49859 Mdot
.79478 .50304 Mdot
.79956 .50746 Mdot
.80433 .51185 Mdot
.80911 .51622 Mdot
.81388 .52056 Mdot
.81865 .52486 Mdot
.82343 .52913 Mdot
.8282 .53335 Mdot
.83298 .53754 Mdot
.83775 .54168 Mdot
.84252 .54577 Mdot
.8473 .54981 Mdot
.85207 .55379 Mdot
.85684 .55771 Mdot
.86162 .56156 Mdot
.86639 .56533 Mdot
.87117 .56902 Mdot
.87594 .57261 Mdot
.88071 .57611 Mdot
.88549 .57949 Mdot
.89026 .58275 Mdot
.89504 .58587 Mdot
.89981 .58884 Mdot
.90458 .59163 Mdot
.90936 .59421 Mdot
.91413 .59657 Mdot
.9189 .59867 Mdot
.92368 .60046 Mdot
.92845 .60188 Mdot
.93323 .60287 Mdot
.938 .60332 Mdot
.94277 .60311 Mdot
.94755 .60205 Mdot
.95232 .59987 Mdot
.9571 .59613 Mdot
.96187 .59006 Mdot
.96664 .58015 Mdot
.97142 .56261 Mdot
.97619 .52067 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3o
ool00`000000oooo0?ooo`3l0?ooo`008@3oool00`000000oooo0?ooo`3l0?ooo`008@3oool20000
01h0oooo0P0000020?ooo`8000000`3oool200000=@0oooo000Q0?ooo`030000003oool0oooo01P0
oooo0P0000040?ooo`050000003oool0oooo0?ooo`000000103oool010000000oooo0?ooo`3oool2
00000<l0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00@0oooo0P00000:0?ooo`030000003oool0oooo
0<d0oooo000=0?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0K0?ooo`80000000H0oooo000000000000oooo0000003oool4000000030?ooo`00
0000000000800000103oool200000<X0oooo000=0?ooo`040000003oool0oooo000000T0oooo00<0
00000?ooo`3oool0103oool2000001@0oooo0P000000103oool000000000003oool2000000030?oo
o`000000000000`0000000<0oooo0000003oool00P0000020?ooo`040000003oool0oooo0?ooo`80
0000a@3oool000L0oooo100000020?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00T0000000<0oooo00000000
00000P0000000`3oool0000000000009000000030?ooo`00000000000080oooo00<000000?ooo`3o
ool0``3oool000d0oooo00@000000?ooo`3oool000002`3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo01@0oooo1`0000000`3oool000000000000;0?ooo`030000003oool0000000H0
000000@0oooo0000000000000000aP3oool000d0oooo00@000000?ooo`3oool00000203oool01000
0000oooo0?ooo`0000040?ooo`030000003oool0oooo00h0oooo0P000000103oool000000000003o
ool2000000030?ooo`000000000001D0oooo00<000000?ooo`0000000`0000000`3oool000000?oo
o`02000000030?ooo`00000000000<00oooo000>0?ooo`8000002P3oool2000000D0oooo0P00000@
0?ooo`030000003oool0oooo00@00000703oool5000000030?ooo`000000000000800000`03oool0
0240oooo00<000000?ooo`3oool03`3oool6000001l0oooo00<000000?ooo`0000000P0000000`3o
ool000000?ooo`02000000030?ooo`00000000000;/0oooo000Q0?ooo`030000003oool0oooo00l0
oooo0P0000000`3oool000000?ooo`0S0?ooo`D0000000D0oooo0000003oool0oooo0000002k0?oo
o`008@3oool00`000000oooo0?ooo`0<0?ooo`D00000:03oool00`000000oooo0000000400000;`0
oooo000Q0?ooo`8000003P3oool3000002`0oooo10000000103oool000000000003oool200000;H0
oooo000Q0?ooo`030000003oool0oooo00T0oooo0P0000030?ooo`030000003oool0oooo02d0oooo
2000002f0?ooo`008@3oool00`000000oooo0?ooo`0:0?ooo`@00000<P3oool3000000030?ooo`00
0000oooo0;H0oooo000Q0?ooo`030000003oool0oooo00/0oooo0`00000d0?ooo`D000000P3oool2
00000;40oooo000Q0?ooo`8000003@3oool00`000000oooo0?ooo`0d0?ooo`050000003oool00000
0000003oool00`00002a0?ooo`008@3oool00`000000oooo0?ooo`070?ooo`@00000>`3oool40000
0;80oooo000Q0?ooo`030000003oool0oooo00P0oooo0`00000l0?ooo`D000000P3oool200000:`0
oooo000Q0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0?P3oool2000000040?oo
o`000000000000000:`0oooo000Q0?ooo`800000203oool200000480oooo1000002]0?ooo`008@3o
ool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0440oooo1@0000020?ooo`800000Y`3o
ool000P0oooo0P0000040?ooo`030000003oool0oooo0080oooo100000020?ooo`<000001@3oool0
0`000000oooo0?ooo`040?ooo`<00000B03oool2000000040?ooo`000000000000000:L0oooo0007
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?oo
o`020?ooo`030000003oool0oooo00D0oooo0`0000180?ooo`80000000<0oooo0000003oool0Y`3o
ool000L0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0080oooo0P0000080?ooo`030000003oool0oooo04L0oooo0P0000000`3oool000000000
002V0?ooo`0000<0oooo0000000000000P0000020?ooo`040000003oool0oooo000000X0oooo00@0
00000?ooo`3oool0oooo0`0000050?ooo`030000003oool0oooo00@0oooo0P00001<0?ooo`@00000
0P3oool200000:80oooo00070?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0
oooo000000070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0B`3oool00`000000
oooo0000000400000:80oooo00070?ooo`040000003oool0oooo000000P0oooo0`0000030?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0EP3oool300000:<0oooo00080?ooo`800000
2`3oool010000000oooo0?ooo`3oool4000000@0oooo0P0000030?ooo`<00000DP3oool400000080
oooo0P00002M0?ooo`008@3oool00`000000oooo0?ooo`030?ooo`800000D`3oool3000000<0oooo
00<000000?ooo`3oool0V`3oool00240oooo00<000000?ooo`3oool0FP3oool4000009h0oooo000Q
0?ooo`030000003oool0oooo05/0oooo1@00002L0?ooo`008@3oool2000000<0oooo0P00001H0?oo
o`80000000@0oooo0000003oool0oooo0P00002H0?ooo`008@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo05L0oooo0`000000103oool0000000000000002H0?ooo`008@3oool00`00
0000oooo0?ooo`1O0?ooo`80000000<0oooo0000003oool0V03oool00240oooo00<000000?ooo`3o
ool0H03oool5000009L0oooo000Q0?ooo`80000000@0oooo0000000000000000GP3oool400000080
oooo0P00002C0?ooo`008@3oool010000000oooo0?ooo`3oool200000640oooo1@00002C0?ooo`00
8@3oool010000000oooo00000000001T0?ooo`<00000U03oool00240oooo00@000000?ooo`3oool0
0000I@3oool400000980oooo000Q0?ooo`800000J03oool300000080oooo0P00002>0?ooo`008@3o
ool010000000oooo00000000001X0?ooo`D00000SP3oool00240oooo00@000000?ooo`3oool00000
J@3oool3000008l0oooo000>0?ooo`800000103oool00`000000oooo0?ooo`020?ooo`@00000103o
ool00`000000oooo0?ooo`1[0?ooo`@00000S@3oool000d0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06`0oooo00<000000?ooo`0000000P3o
ool2000008T0oooo000=0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3o
ool2000006h0oooo0P000000103oool000000000000000290?ooo`001`3oool400000080oooo00@0
00000?ooo`3oool000002P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06h0oooo
0P0000000`3oool000000?ooo`290?ooo`003@3oool010000000oooo0?ooo`00000:0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0K`3oool2000008/0oooo000=0?ooo`040000003o
ool0oooo000000P0oooo0`0000050?ooo`<00000L03oool4000008P0oooo000>0?ooo`8000002`3o
ool00`000000oooo0?ooo`030?ooo`<00000L@3oool300000080oooo0P0000240?ooo`008@3oool2
000007@0oooo1@0000240?ooo`008@3oool2000007D0oooo0`0000250?ooo`008@3oool00`000000
oooo0?ooo`1e0?ooo`@00000P`3oool00240oooo0P00001g0?ooo`<000000P3oool2000007l0oooo
000Q0?ooo`030000003oool0oooo07L0oooo1P00001o0?ooo`008@3oool00`000000oooo0?ooo`1i
0?ooo`<00000P03oool00240oooo00<000000?ooo`3oool0NP3oool4000007h0oooo000Q0?ooo`80
0000O03oool300000080oooo0P00001j0?ooo`008@3oool00`000000oooo0?ooo`1l0?ooo`030000
003oool0oooo00<00000NP3oool00240oooo00<000000?ooo`3oool0O@3oool2000000030?ooo`00
0000oooo07X0oooo000Q0?ooo`800000O`3oool2000007`0oooo000Q0?ooo`800000P03oool40000
07T0oooo000Q0?ooo`030000003oool0oooo0800oooo0`0000020?ooo`800000M@3oool000P0oooo
0P0000040?ooo`030000003oool0oooo00<0oooo0P0000030?ooo`<000001@3oool00`000000oooo
0?ooo`210?ooo`H00000M@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000050?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0P`3oool3000007H0
oooo00070?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool000001@3oool00`00
0000oooo0?ooo`020?ooo`800000Q@3oool4000007@0oooo00000`3oool000000000000200000080
oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?ooo`<000001@3oool0
0`000000oooo0?ooo`250?ooo`<000000P3oool200000700oooo00070?ooo`040000003oool0oooo
000000P0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo08H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1^0?ooo`001`3oool0
10000000oooo0?ooo`0000080?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0
1@3oool00`000000oooo0?ooo`270?ooo`<00000LP3oool000P0oooo0P00000:0?ooo`8000000`3o
ool4000000@0oooo0P0000290?ooo`800000LP3oool00240oooo00<000000?ooo`3oool0R@3oool4
000006l0oooo000Q0?ooo`030000003oool0oooo08X0oooo0`0000020?ooo`800000J`3oool00240
oooo00<000000?ooo`3oool0R`3oool4000000030?ooo`000000oooo06X0oooo000Q0?ooo`800000
SP3oool2000006d0oooo000Q0?ooo`030000003oool0oooo08h0oooo00@000000?ooo`0000000000
JP3oool00240oooo00<000000?ooo`3oool09`3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool4000002<0oooo0P0000040?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool08P3o
ool2000000050?ooo`00000000000?ooo`0000001@3oool2000002<0oooo0P0000040?ooo`030000
003oool0oooo00<0oooo0P00000Y0?ooo`@000001@3oool00240oooo00<000000?ooo`3oool09P3o
ool010000000oooo0?ooo`0000080?ooo`030000003oool0oooo02<0oooo00@000000?ooo`3oool0
00002P3oool00`000000oooo0?ooo`0Q0?ooo`050000003oool0oooo0000003oool00P0000030?oo
o`<000000P3oool00`000000oooo0?ooo`0O0?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool00000:P3oool00`000000oooo0?ooo`040?ooo`008@3oool2000002L0oooo00@00000
0?ooo`3oool000002@3oool00`000000oooo0?ooo`0R0?ooo`040000003oool0oooo000000L0oooo
1@00000R0?ooo`040000003oool0oooo00000080oooo100000000`3oool00000000000020?ooo`03
0000003oool0oooo01l0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`00000Z
0?ooo`030000003oool0oooo00@0oooo000Q0?ooo`030000003oool0oooo02H0oooo00@000000?oo
o`3oool000002P3oool00`000000oooo0?ooo`0Q0?ooo`040000003oool0oooo000000L0oooo00@0
00000?ooo`3oool000008`3oool010000000oooo0?ooo`0000040?ooo`8000000P3oool300000280
oooo00@000000?ooo`3oool000002@3oool2000002/0oooo00<000000?ooo`3oool0103oool00240
oooo00<000000?ooo`3oool09P3oool010000000oooo0?ooo`00000;0?ooo`030000003oool0oooo
0200oooo00@000000?ooo`3oool00000203oool00`000000oooo0000000S0?ooo`040000003oool0
oooo000000D0oooo1000000T0?ooo`040000003oool0oooo000000P0oooo00@000000?ooo`3oool0
0000:P3oool00`000000oooo0?ooo`040?ooo`008@3oool00`000000oooo0?ooo`0V0?ooo`040000
003oool0oooo000000P0oooo00@000000?ooo`3oool000008P3oool010000000oooo0?ooo`000009
0?ooo`8000008`3oool010000000oooo0?ooo`0000060?ooo`<000000P3oool200000200oooo00@0
00000?ooo`3oool00000203oool010000000oooo0?ooo`00000X0?ooo`<000001P3oool00240oooo
0P00000X0?ooo`8000002P3oool2000002@0oooo0P00000;0?ooo`030000003oool0oooo0280oooo
0P0000080?ooo`030000003oool0000000<000008@3oool2000000X0oooo0P00000[0?ooo`030000
003oool0oooo00@0oooo000Q0?ooo`030000003oool0oooo09H0oooo0`00001S0?ooo`008@3oool0
0`000000oooo0?ooo`2G0?ooo`800000H`3oool00240oooo00<000000?ooo`3oool0V03oool40000
0600oooo000K0?ooool000001P0000000240oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01@3oool6000000X0oooo
00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo
00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool0103oool00240oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?oo
o`0_0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0000
000U0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0103oool00240oooo00<00000
0?ooo`3oool0W03oool2000005h0oooo000Q0?ooo`030000003oool0oooo09d0oooo1000001K0?oo
o`008@3oool2000009l0oooo0`0000020?ooo`800000E`3oool00240oooo00<000000?ooo`3oool0
W`3oool4000000030?ooo`000000oooo05H0oooo000Q0?ooo`030000003oool0oooo0:00oooo0`00
001I0?ooo`008@3oool00`000000oooo0?ooo`2R0?ooo`040000003oool00000000005H0oooo000Q
0?ooo`800000X`3oool2000000030?ooo`000000oooo05D0oooo000Q0?ooo`030000003oool0oooo
0:<0oooo0P0000020?ooo`800000D`3oool00240oooo00<000000?ooo`3oool0Y03oool5000005<0
oooo000Q0?ooo`030000003oool0oooo0:D0oooo0`00001D0?ooo`008@3oool200000:P0oooo1000
001A0?ooo`008@3oool00`000000oooo0?ooo`2X0?ooo`040000003oool000000?ooo`800000CP3o
ool00240oooo00<000000?ooo`3oool0Z03oool2000000<0oooo00<000000?ooo`3oool0C03oool0
0240oooo00<000000?ooo`3oool0Z@3oool4000004l0oooo000Q0?ooo`800000Z`3oool3000004l0
oooo000Q0?ooo`030000003oool0oooo0:`0oooo1000001<0?ooo`00203oool2000000@0oooo00<0
00000?ooo`3oool00`3oool2000000<0oooo0`0000050?ooo`030000003oool0oooo0:d0oooo0`00
00000`3oool00000000000190?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`040000003o
ool0oooo000000D0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`2^0?ooo`030000
003oool0000000800000B@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo
0?ooo`0000050?ooo`030000003oool0oooo0080oooo0P00002_0?ooo`80000000<0oooo0000003o
ool0B@3oool000L0oooo00@000000?ooo`3oool00000203oool010000000oooo0?ooo`0000020?oo
o`<000001@3oool00`000000oooo0?ooo`2_0?ooo`<00000BP3oool000L0oooo00@000000?ooo`3o
ool00000203oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool0/@3oool4000004L0oooo00070?ooo`040000003oool0oooo000000P0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0;80oooo0`00
00000`3oool00000000000140?ooo`00203oool2000000X0oooo0P0000030?ooo`@00000103oool2
00000;@0oooo1@0000140?ooo`008@3oool00`000000oooo0?ooo`2d0?ooo`030000003oool00000
04D0oooo000Q0?ooo`030000003oool0oooo0;D0oooo0P0000150?ooo`008@3oool00`000000oooo
0?ooo`2f0?ooo`@00000@P3oool00240oooo0P00002h0?ooo`<0000000<0oooo000000000000?`3o
ool00240oooo00<000000?ooo`3oool0^03oool5000003l0oooo000Q0?ooo`030000003oool0oooo
0;T0oooo0`0000100?ooo`008@3oool00`000000oooo0?ooo`2j0?ooo`030000003oool0oooo0080
0000?@3oool00240oooo0P00002l0?ooo`80000000<0oooo0000003oool00P00000j0?ooo`008@3o
ool00`000000oooo0?ooo`2l0?ooo`8000000`3oool00`000000oooo0?ooo`0h0?ooo`008@3oool0
0`000000oooo0?ooo`2m0?ooo`@00000>`3oool00240oooo00<000000?ooo`3oool0_P3oool30000
03/0oooo000Q0?ooo`800000`03oool5000003P0oooo000Q0?ooo`030000003oool0oooo0<40oooo
0`0000000`3oool000000000000e0?ooo`003P3oool2000000@0oooo00<000000?ooo`3oool00P3o
ool4000000@0oooo00<000000?ooo`3oool0`P3oool00`000000oooo00000002000003D0oooo000=
0?ooo`040000003oool0oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?oo
o`320?ooo`80000000<0oooo0000003oool0=@3oool000d0oooo00@000000?ooo`3oool000002P3o
ool00`000000oooo0?ooo`030?ooo`800000a03oool2000003L0oooo000=0?ooo`040000003oool0
oooo000000X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`340?ooo`D0000000<0
oooo000000000000<03oool000d0oooo00@000000?ooo`3oool000002P3oool00`000000oooo0?oo
o`030?ooo`030000003oool0oooo0<H0oooo0`0000020?ooo`030000003oool0oooo02h0oooo000=
0?ooo`040000003oool0oooo000000P0oooo0`0000050?ooo`030000003oool0oooo0<L0oooo1000
000a0?ooo`003P3oool2000000/0oooo00<000000?ooo`3oool00`3oool200000<T0oooo0`00000a
0?ooo`008@3oool00`000000oooo0?ooo`390?ooo`040000003oool00000000002l0oooo000Q0?oo
o`030000003oool0oooo0<X0oooo0`0000020?ooo`8000008`3oool2000000H0oooo000Q0?ooo`03
0000003oool0oooo0</0oooo0P000000103oool0000000000000000T0?ooo`030000003oool0oooo
00@0oooo000Q0?ooo`030000003oool0oooo0<`0oooo0P0000000`3oool000000?ooo`0[0?ooo`00
8@3oool200000<h0oooo0P0000000`3oool000000000000Q0?ooo`8000001`3oool00240oooo00<0
00000?ooo`3oool0cP3oool400000080oooo0P00000N0?ooo`030000003oool0oooo00D0oooo000Q
0?ooo`030000003oool0oooo0=00oooo0P000000103oool0000000000000000V0?ooo`008@3oool0
0`000000oooo0?ooo`3A0?ooo`80000000<0oooo0000003oool0703oool2000000P0oooo000Q0?oo
o`800000d`3oool2000000030?ooo`000000000001`0oooo00<000000?ooo`3oool01P3oool00240
oooo00<000000?ooo`3oool0d`3oool400000080oooo0P00000Q0?ooo`008@3oool00`000000oooo
0?ooo`3E0?ooo`80000000@0oooo00000000000000008@3oool00240oooo00<000000?ooo`3oool0
eP3oool2000000030?ooo`000000oooo01P0oooo0P0000070?ooo`008@3oool200000=P0oooo0P00
00000`3oool00000000000020?ooo`800000503oool00`000000oooo0?ooo`050?ooo`008@3oool0
0`000000oooo0?ooo`3H0?ooo`@000000`3oool00`000000oooo0?ooo`0J0?ooo`00203oool20000
00@0oooo00<000000?ooo`3oool00P3oool400000080oooo0`0000050?ooo`030000003oool0oooo
0=X0oooo0P0000000`3oool000000000000B0?ooo`8000002@3oool000L0oooo00@000000?ooo`3o
ool000002P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0080oooo00<000000?oo
o`3oool0f`3oool2000000040?ooo`000000000000000080oooo0P0000080?ooo`8000000`3oool0
0`000000oooo0?ooo`070?ooo`001`3oool010000000oooo0?ooo`00000:0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool00P3oool200000=d0oooo0P0000020?ooo`050000003oool0
oooo0?ooo`0000000`3oool2000000@0oooo00<000000?ooo`3oool00P0000090?ooo`001`3oool0
10000000oooo0?ooo`00000:0?ooo`040000003oool0oooo0?ooo`<000001@3oool00`000000oooo
0?ooo`3M0?ooo`@0000000<0oooo0000000000001@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo00L0oooo00070?ooo`040000003oool0oooo000000X0oooo00D000000?ooo`3oool0
oooo000000070?ooo`030000003oool0oooo0=h0oooo00D000000?ooo`00000000000?ooo`030000
00P0oooo0P00000<0?ooo`001`3oool010000000oooo0?ooo`0000080?ooo`<000000`3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo0>40oooo0P0000020?ooo`030000003oool00000
00<0000000@0oooo000000000000oooo0`00000:0?ooo`00203oool2000000/0oooo00@000000?oo
o`3oool0oooo100000040?ooo`800000h`3oool400000080oooo00<000000?ooo`0000000P3oool0
0`000000oooo00000002000000X0oooo000Q0?ooo`030000003oool0oooo0><0oooo00<000000?oo
o`0000000P0000060?ooo`<000002`3oool00240oooo00<000000?ooo`3oool0iP3oool800000003
0?ooo`000000oooo00/0oooo000Q0?ooo`030000003oool0oooo0>P0oooo00<000000?ooo`000000
0`00000>0?ooo`008@3oool200000?d0oooo000Q0?ooo`030000003oool0oooo0?`0oooo000Q0?oo
o`030000003oool0oooo0?`0oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o
0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo003o0?ooob40oooo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.133977, -0.236989, \
0.00402934, 0.00244537}}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowSize->{742, 598},
WindowMargins->{{Automatic, 65}, {Automatic, 29}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "endpoint.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 66, 0, 64, "Subtitle"],
Cell[1808, 53, 64, 3, 71, "Subsubtitle"],
Cell[1875, 58, 177, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[2077, 66, 56, 0, 54, "Section"],
Cell[2136, 68, 132, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[2293, 76, 229, 5, 43, "Input"],
Cell[2525, 83, 123, 2, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2685, 90, 184, 5, 43, "Input"],
Cell[2872, 97, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3041, 105, 184, 5, 43, "Input"],
Cell[3228, 112, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3397, 120, 232, 6, 59, "Input"],
Cell[3632, 128, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3801, 136, 686, 14, 123, "Input"],
Cell[4490, 152, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4659, 160, 58, 1, 27, "Input"],
Cell[4720, 163, 132, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[4901, 172, 30, 0, 34, "Section"],
Cell[4934, 174, 907, 18, 171, "Input",
  InitializationCell->True],
Cell[5844, 194, 1198, 21, 322, "Input",
  InitializationCell->True],
Cell[7045, 217, 122, 3, 32, "Text"],
Cell[7170, 222, 986, 18, 203, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[8193, 245, 47, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[8265, 249, 258, 4, 59, "Input"],
Cell[8526, 255, 200, 3, 71, "Output"]
}, Open  ]],
Cell[8741, 261, 29, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[8795, 265, 217, 3, 43, "Input"],
Cell[9015, 270, 167, 2, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9219, 277, 122, 3, 27, "Input"],
Cell[9344, 282, 91, 1, 46, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9472, 288, 131, 2, 27, "Input"],
Cell[9606, 292, 120, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9763, 299, 106, 2, 27, "Input"],
Cell[9872, 303, 151, 2, 46, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[10072, 311, 39, 0, 34, "Section"],
Cell[10114, 313, 26, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[10165, 317, 152, 3, 43, "Input"],
Cell[10320, 322, 37, 1, 23, "Print"],
Cell[10360, 325, 37, 1, 23, "Print"],
Cell[10400, 328, 37, 1, 23, "Print"],
Cell[10440, 331, 37, 1, 23, "Print"],
Cell[10480, 334, 52, 1, 23, "Print"],
Cell[10535, 337, 52, 1, 23, "Print"],
Cell[10590, 340, 53, 1, 23, "Print"],
Cell[10646, 343, 72, 1, 27, "Output"]
}, Open  ]],
Cell[10733, 347, 66, 0, 32, "Text"],
Cell[10802, 349, 367, 9, 87, "Input"],

Cell[CellGroupData[{
Cell[11194, 362, 159, 3, 27, "Input"],
Cell[11356, 367, 25363, 806, 186, 7801, 585, "GraphicsData", "PostScript", \
"Graphics"],
Cell[36722, 1175, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36889, 1183, 89, 1, 27, "Input"],
Cell[36981, 1186, 48, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37066, 1192, 164, 3, 27, "Input"],
Cell[37233, 1197, 24386, 796, 186, 7911, 588, "GraphicsData", "PostScript", \
"Graphics"],
Cell[61622, 1995, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61789, 2003, 89, 1, 27, "Input"],
Cell[61881, 2006, 48, 1, 27, "Output"]
}, Open  ]],
Cell[61944, 2010, 26, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[61995, 2014, 355, 9, 87, "Input"],
Cell[62353, 2025, 187, 3, 37, "Message"]
}, Open  ]],
Cell[62555, 2031, 356, 9, 87, "Input"],
Cell[62914, 2042, 356, 9, 87, "Input"],

Cell[CellGroupData[{
Cell[63295, 2055, 307, 6, 75, "Input"],
Cell[63605, 2063, 26251, 783, 186, 9719, 574, "GraphicsData", "PostScript", \
"Graphics"],
Cell[89859, 2848, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[90026, 2856, 307, 6, 75, "Input"],
Cell[90336, 2864, 27125, 824, 186, 10193, 610, "GraphicsData", "PostScript", \
"Graphics"],
Cell[117464, 3690, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

