(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     23328,        696]*)
(*NotebookOutlinePosition[     24667,        737]*)
(*  CellTagsIndexPosition[     24623,        733]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Spectra mesons transverse lattice
One-dimensional scans\
\>", "Subtitle"],

Cell["Brett van de Sande", "Subsubtitle"],

Cell["All of the following are for spectra computed in 2002.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cross[x_, y_, z_] := {Line[{{x, y - z}, {x, y + z}}], 
        Line[{{x - 0.1, y}, {x + 0.1, y}}]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(cross\)\" is similar to \
existing symbol \"\!\(Cross\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(<< \ Statistics`LinearRegression`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(data = 
      Block[\n\t\t{x, fp = OpenRead["\<~/four/junk.out\>"], 
          info}, \n\t\tinfo = 
          Read[fp, Record, RecordSeparators -> "\<\n\n\>"]; \n\t\tx = 
          ReadList[fp, Number, 
            RecordLists -> True, \n\t\t\tRecordSeparators -> "\<\n\n\>"]; 
        Close[fp]; \n\t\t{info, Chop[\ x]}\n\ ]\)], "Input"],

Cell[BoxData[
    \(\(d3 = data; \)\)], "Input"],

Cell[BoxData[
    \(\(d5 = data; \)\)], "Input"],

Cell[BoxData[
    \(data = {d3[\([1]\)], 
        Table[If[d3[\([2, i, 3]\)] < d5[\([2, i, 3]\)], d3[\([2, i]\)], 
            d5[\([2, i]\)]], {i, Length[d3[\([2]\)]]}]}\)], "Input"],

Cell["data three/ytraj_1_1?.out; periodic BC's", "Section"],

Cell[CellGroupData[{

Cell["analyze", "Section"],

Cell[BoxData[
    \(data[\([1]\)]\)], "Input"],

Cell["Chi^2", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot[Map[{#[\([20]\)], #[\([3]\)]} &, data[\([2]\)]], 
        PlotJoined -> True, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        FrameLabel -> {"\<m\>", "\<\>"} (*\(,\)\(PlotRange -> {0, 
              40}\)*) ]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Select which points to plot (sdata) and which points to use for \
fitting (ssdata).\
\>", "Text"],

Cell[BoxData[{
    \(\(sdata = 
        Select[data[\([2]\)], \((\ #[\([3]\)] < 100)\) &];\)\), "\n", 
    \(\(ssdata = 
        Select[data[\([2]\)], \((#[\([3]\)] < 20 && \ #[\([38]\)] > 
                  8)\) &];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([20]\)])\) &, sdata]\)], "Input"],

Cell[BoxData[
    \({\(-1\), \(-0.9`\), \(-0.8`\), \(-0.7`\), \(-0.6`\), \(-0.5`\), \(-0.4`\
\), \(-0.3`\), \(-0.2`\), \(-0.1`\), 0, 0.1`, 0.2`, 0.3`, 0.6`, 
      0.8`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([6]\)])\) &, ssdata]\)], "Input"],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell["\<\
Using sdata, fit all couplings to either a linear or constant \
function.  Here, x is the mass.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Chop[Table[
          Fit[Map[{#[\([20]\)], #[\([i]\)]} &, sdata], 
            If[MemberQ[{11, 13}, i], {1}, {1, x, x^2}], x], {i, 6, 
            15 + 7}]]\)], "Input"],

Cell[BoxData[
    \({0.07642881741999999`, 0.9999999999999999`, 
      0.7680984099999998`, \(-0.16924615889999997`\), \(-0.18577166519999994`\
\), 49854.194339999995`, 0.024269830289999995`, 
      24828.433780000003`, \(-1.547566329`\), \(-2.2390306569999994`\), \
\(\(0.43477404269125536`\)\(\[InvisibleSpace]\)\) - 0.10798033682164579`\ x - 
        16.83634939376625`\ x\^2, \
\(\(0.4908465386760173`\)\(\[InvisibleSpace]\)\) + 0.4643012146640685`\ x - 
        8.564302814718621`\ x\^2, \(\(0.29734237205718606`\)\(\
\[InvisibleSpace]\)\) + 0.22028122898874475`\ x + 
        17.24103339844156`\ x\^2, \(-0.1148921034578225`\) - 
        0.7753670166566236`\ x - 8.909626006199135`\ x\^2, 
      1.0000000000000002`\ x, 0, 0}\)], "Output"]
}, Open  ]],

Cell["Fit weighted by chi^2 values of data", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Block[{selected}, 
        Chop[Table[\[IndentingNewLine]selected = 
              Select[sdata, \((Abs[#[\([i]\)]] < 10000000)\) &]; 
            BestFit /. 
              Regress[Map[{#[\([20]\)], #[\([i]\)]} &, selected], 
                If[MemberQ[{1, 2, 3, 4, 5, 6, 7, 13} + 6, i], {1, x}, {1, x, 
                    x^2}], x, 
                Weights \[Rule] Map[\((1/#[\([3]\)])\) &, selected], 
                RegressionReport \[Rule] BestFit], {i, 6, 
              15 + 7}]]]\)], "Input"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(DesignedRegress :: \"badfit\"\) will be \
suppressed during this calculation."\)], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \(Power::"infy" \(\(:\)\(\ \)\) 
      "Infinite expression \!\(1\/0.`\) encountered."\)], "Message"],

Cell[BoxData[
    RowBox[{\(\[Infinity]::"indet"\), \(\(:\)\(\ \)\), "\<\"Indeterminate \
expression \\!\\(0.`\\\\ \\*InterpretationBox[\\\"ComplexInfinity\\\", \
DirectedInfinity[]]\\) encountered.\"\>"}]], "Message"],

Cell[BoxData[
    \({0.07642881742`, 1.0000000000000002`, 
      0.7680984100000002`, \(-0.16924615890000005`\), \(-0.18577166520000007`\
\), 49854.19434000002`, 0.02426983029000001`, 
      24828.433780000003`, \(-1.547566329`\), \(-2.2390306570000003`\), \
\(\(0.19484838237260443`\)\(\[InvisibleSpace]\)\) + 0.3932588419001844`\ x + 
        0.5580616106333188`\ x\^2, \(\(0.3771254980408863`\)\(\
\[InvisibleSpace]\)\) + 0.2323122029007473`\ x + 
        0.06117854220049925`\ x\^2, \(\(0.6342994770793252`\)\(\
\[InvisibleSpace]\)\) + 0.204956972436839`\ x + 
        0.7393867149720776`\ x\^2, \(-0.3215336226740564`\) - 
        3.1976859446381427`\ x, 1.0000000000000002`\ x, 0, 0}\)], "Output"]
}, Open  ]],

Cell["change mu_F by hand", "Text"],

Cell[BoxData[
    \(\(couplefitting = {0.07642881742000004`, 1.`, 
          0.76809841`, \(-0.16924615890000003`\), \(-0.1857716652`\), 
          49854.19434`, 0.024269830290000002`, 
          24828.43378`, \(-1.5475663290000004`\), \(-2.2390306570000003`\), 
          0.3600892491665967`, 0.44440106366780385`, 
          0.1 + 0.3723128897038414`, \(-0.05\) - 0.1415284266539087`, 
          1.0000000000000004`\ x, 0, 0};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting[\([14]\)]\)], "Input"],

Cell[BoxData[
    \(\(-0.1385284266539087`\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[
      couplefitting[\([13]\)]^2 + couplefitting[\([15]\)], {x, \(-0.25\), 
        0.25}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pc = 
      Table[Show[
          ListPlot[Map[{#[\([20]\)], #[\([i]\)]} &, sdata], 
            PlotStyle -> {PointSize[ .02], Hue[2/3]}, PlotRange \[Rule] All, 
            DisplayFunction -> Identity], \n\t (*\(ListPlot[tau1, 
              PlotStyle -> {PointSize[ .02]}, 
              DisplayFunction -> Identity]\)\(,\)*) Plot[
            couplefitting[\([i - 5]\)], {x, \(-1\), 1}, 
            PlotStyle -> {Hue[0]}, DisplayFunction -> Identity], 
          DisplayFunction -> $DisplayFunction, Axes -> False, 
          RotateLabel -> False, FormatType -> OutputForm, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
          Frame -> True, 
          FrameLabel -> {"\<m\>", {"\<b\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\
\(l\_2\)\>\"", \*"\"\<\!\(l\_3\)\>\"", \*"\"\<\!\(l\_4\)\>\"", \
\*"\"\<\!\(l\_5\)\>\"", \*"\"\<\!\(t\_1\)\>\"", \*"\"\<\!\(t\_2\)\>\"", \*"\"\
\<\!\(kappa\_S\)\>\"", \*"\"\<\!\(kappa\_A\)\>\"", \*"\"\<\!\(mu\_F\)\>\"", \
\*"\"\<\!\(mu\_0\^2\)\>\"", \*"\"\<\!\(mu\_1\^2\)\>\"", \*"\"\<\!\(mu\_2\^2\)\
\>\"", \*"\"\<\!\(mu\_3\^2\)\>\""}[\([i - 7]\)]}], {i, 8, 15 + 7}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \( .28* .28\)], "Input"],

Cell[BoxData[
    \(0.07840000000000001`\)], "Output"]
}, Open  ]],

Cell["\<\
a^2 sigma vs m using two different methods: using the overall scale \
and using the longitudinal potential. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita2 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(2.022478482413753`\)\(\[InvisibleSpace]\)\) + 
      1.137984830339073`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita3 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(2.0226799306464702`\)\(\[InvisibleSpace]\)\) + 
      0.6435286581292277`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fita2, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t\tPlot[fita3, {x, 0,  .5}, 
        DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0]}], \n\t\t (*\(Plot[1 + 2\ x, {x, 0,  .5}, 
          DisplayFunction -> Identity]\)\(,\)*) \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          sdata], PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          sdata], PlotStyle -> {PointSize[0.01], Hue[0]}, 
        DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitag = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]]*#[\([5]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(5.091866692483492`\)\(\[InvisibleSpace]\)\) - 
      2.2778471841273324`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fitag, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], 
              Sqrt[#[\([24]\)]]*#[\([5]\)]} &, sdata], 
        PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \nFrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(exam = Sqrt[Partition[Take[data[\([2, 2]\)], \(-40\)], 4]]\)], "Input"],

Cell[BoxData[
    \({{3.4963372262984014`, 4.97000714285201`, 5.6980637062075745`, 
        6.5280748310662`}, {9.069016539845983`, 9.33762223480903`, 
        10.272320088470764`, 11.039075640650355`}, {8.236730722805014`, 
        9.651410363257797`, 10.844002351530545`, 
        10.756670767481916`}, {5.746020535988364`, 8.318284198078352`, 
        9.541465348676796`, 9.76838067440044`}, {4.968623753113129`, 
        5.7064206820037375`, 7.104791974435281`, 
        7.352763698093391`}, {8.391401968681992`, 9.11649537925622`, 
        10.096977319970566`, 10.367887875551123`}, {6.067179492976947`, 
        9.165112547044908`, 10.0852764463846`, 
        9.670015563586235`}, {8.36810558011788`, 9.59763460442207`, 
        9.768000972563424`, 9.546210504697662`}, {5.367422100040205`, 
        7.376504117805398`, 8.006552691389722`, 
        8.870453426967531`}, {5.6207531523809156`, 7.788696617534926`, 
        7.985614065305185`, 8.355544865536897`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec1 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(\(0\^++\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(0\^\(-+\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(\(1\^++\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(1\^\(-+\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(\(2\^++\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(2\^\(-+\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([1]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([3]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([10]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([5]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([7]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 7}}, 
          FormatType -> OutputForm, FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec2 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(0\^\(+-\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(\(0\^--\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(1\^\(+-\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(\(1\^--\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(2\^\(+-\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(\(2\^--\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([2]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([4]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([9]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([6]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([8]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 9.5}}, 
          FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put plots", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.spec1.eps\>", pspec1, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.spec2.eps\>", pspec2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.chi2.eps\>", chi2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pass = 
      Show[\n\t\t (*
          Plot[Sqrt[fita2], {x, 0,  .5}, DisplayFunction -> Identity], *) \n\t
        ListPlot[
          Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]*#[\([5]\)]]}&, 
            sdata], PlotStyle -> {PointSize[0.015]}, 
          DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
        PlotRange -> All, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.ass.eps\>", pass, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This was printed to a postscript file \"ttraj60.scaling.ps\", size \
616 by 779, and moved to the left such that left=0. (You need to adjust the \
Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ Show[GraphicsArray[Partition[pc, \ 2]]]\)\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This prints out scaling graphs to separate files (You need to \
adjust the Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{couplelist = {"\<b\>", "\<l1\>", "\<l2\>", "\<l3\>", "\<l4\>", "\
\<l5\>", "\<t1\>", "\<t2\>"}}, \[IndentingNewLine]Do[
        Print["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>"]; 
        Display["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>", 
          pc[\([i]\)], "\<EPS\>", ImageSize -> 360, 
          ImageOffset -> {0, 0}], {i, Length[pc]}]]\)], "Input"],

Cell[BoxData[
    \("~/three/ttraj60.b.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l2.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l3.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l4.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l5.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t2.eps"\)], "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Create data file", "Section"],

Cell["\<\
Average previous value with fit. (In the file, I set lamda_3 and \
lambda_5  = 100.)\
\>", "Text"],

Cell[BoxData[
    \(expr2 = 
      Block[{mf, ll = Length[data[\([2]\)]]}, 
        TableForm[
          Flatten[Table[{4, 
                Map[NumberForm[#, 12] &, mf = 0.1*i - 1; 
                  couplefitting /. x \[Rule] mf\[IndentingNewLine]], {10, 11, 
                  12, 13}}, {i, 0, 20}], 1], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/four/scan_m1_8.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<scan_m1_8.out\>"]; \n\t\tWrite[fp, "\<100\>"]; \n\t\tWrite[
        fp, "\<0\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/four/scan_m1_8.in"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowSize->{696, 681},
WindowMargins->{{Automatic, 130}, {Automatic, 6}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "spectrum.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 83, 3, 93, "Subtitle"],
Cell[1825, 56, 41, 0, 53, "Subsubtitle"],
Cell[1869, 58, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1964, 62, 159, 3, 27, "Input",
  InitializationCell->True],
Cell[2126, 67, 183, 3, 37, "Message"]
}, Open  ]],
Cell[2324, 73, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[2421, 77, 358, 7, 123, "Input"],
Cell[2782, 86, 48, 1, 27, "Input"],
Cell[2833, 89, 48, 1, 27, "Input"],
Cell[2884, 92, 183, 3, 43, "Input"],
Cell[3070, 97, 59, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[3154, 101, 26, 0, 54, "Section"],
Cell[3183, 103, 46, 1, 27, "Input"],
Cell[3232, 106, 21, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[3278, 110, 324, 6, 59, "Input"],
Cell[3605, 118, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3750, 124, 107, 3, 32, "Text"],
Cell[3860, 129, 232, 5, 43, "Input"],

Cell[CellGroupData[{
Cell[4117, 138, 64, 1, 27, "Input"],
Cell[4184, 141, 183, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4404, 149, 64, 1, 27, "Input"],
Cell[4471, 152, 36, 1, 27, "Output"]
}, Open  ]],
Cell[4522, 156, 119, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[4666, 163, 213, 5, 59, "Input"],
Cell[4882, 170, 745, 12, 61, "Output"]
}, Open  ]],
Cell[5642, 185, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[5719, 189, 544, 11, 107, "Input"],
Cell[6266, 202, 156, 3, 37, "Message"],
Cell[6425, 207, 156, 3, 37, "Message"],
Cell[6584, 212, 156, 3, 37, "Message"],
Cell[6743, 217, 175, 3, 37, "Message"],
Cell[6921, 222, 119, 2, 35, "Message"],
Cell[7043, 226, 218, 3, 22, "Message"],
Cell[7264, 231, 119, 2, 35, "Message"],
Cell[7386, 235, 218, 3, 22, "Message"],
Cell[7607, 240, 703, 11, 61, "Output"]
}, Open  ]],
Cell[8325, 254, 35, 0, 32, "Text"],
Cell[8363, 256, 443, 7, 91, "Input"],

Cell[CellGroupData[{
Cell[8831, 267, 56, 1, 27, "Input"],
Cell[8890, 270, 58, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8985, 276, 128, 3, 27, "Input"],
Cell[9116, 281, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9283, 289, 1156, 19, 215, "Input"],
Cell[10442, 310, 1920, 47, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12399, 362, 42, 1, 27, "Input"],
Cell[12444, 365, 54, 1, 27, "Output"]
}, Open  ]],
Cell[12513, 369, 132, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[12670, 376, 154, 3, 43, "Input"],
Cell[12827, 381, 112, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12976, 388, 155, 3, 43, "Input"],
Cell[13134, 393, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13285, 400, 882, 14, 187, "Input"],
Cell[14170, 416, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14337, 424, 160, 3, 43, "Input"],
Cell[14500, 429, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14650, 436, 533, 9, 123, "Input"],
Cell[15186, 447, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15353, 455, 91, 1, 27, "Input"],
Cell[15447, 458, 981, 15, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16465, 478, 1016, 17, 155, "Input"],
Cell[17484, 497, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17651, 505, 991, 17, 155, "Input"],
Cell[18645, 524, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[18824, 533, 28, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[18877, 537, 140, 2, 43, "Input"],
Cell[19020, 541, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19187, 549, 140, 2, 43, "Input"],
Cell[19330, 553, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19497, 561, 137, 2, 43, "Input"],
Cell[19637, 565, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19804, 573, 534, 10, 123, "Input"],
Cell[20341, 585, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20508, 593, 136, 2, 43, "Input"],
Cell[20647, 597, 130, 3, 27, "Output"]
}, Open  ]],
Cell[20792, 603, 208, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[21025, 611, 78, 1, 27, "Input"],
Cell[21106, 614, 135, 3, 27, "Output"]
}, Open  ]],
Cell[21256, 620, 136, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[21417, 627, 419, 6, 75, "Input"],
Cell[21839, 635, 56, 1, 23, "Print"],
Cell[21898, 638, 57, 1, 23, "Print"],
Cell[21958, 641, 57, 1, 23, "Print"],
Cell[22018, 644, 57, 1, 23, "Print"],
Cell[22078, 647, 57, 1, 23, "Print"],
Cell[22138, 650, 57, 1, 23, "Print"],
Cell[22198, 653, 57, 1, 23, "Print"],
Cell[22258, 656, 57, 1, 23, "Print"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[22364, 663, 35, 0, 54, "Section"],
Cell[22402, 665, 108, 3, 32, "Text"],
Cell[22513, 670, 356, 8, 107, "Input"],

Cell[CellGroupData[{
Cell[22894, 682, 327, 6, 123, "Input"],
Cell[23224, 690, 64, 1, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

