(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     24337,        606]*)
(*NotebookOutlinePosition[     25288,        636]*)
(*  CellTagsIndexPosition[     25244,        632]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Hamiltonian", "Section"],

Cell["Input nonzero matrix elements from laso (using print_els)", "Text"],

Cell[BoxData[
    \(<< "\<~/four/laso2.out\>"\)], "Input"],

Cell["List of distinct nonzero matrix elements of the Hamiltonian", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(unique = Union[Map[Chop[#[\([2]\)]] &, list]]\)], "Input"],

Cell[BoxData[
    \({\(-0.182377155102`\), \(-0.167174953615`\), \(-0.150450555613`\), \
\(-0.107151877295`\), \(-0.106103295395`\) - 
        0.06107165822`\ \[ImaginaryI], \(-0.106103295395`\) + 
        0.06107165822`\ \[ImaginaryI], \(-0.075225277806`\), \
\(-0.06107165822`\) - 0.106103295395`\ \[ImaginaryI], \(-0.06107165822`\) + 
        0.106103295395`\ \[ImaginaryI], \(-0.045031637174`\), \
\(-0.031926599489`\), \(-0.022635369684`\), \(-0.031926599489`\)\ \
\[ImaginaryI], 
      0.031926599489`\ \[ImaginaryI], \(-0.045031637174`\)\ \[ImaginaryI], 
      0.045031637174`\ \[ImaginaryI], \(-0.075225277806`\)\ \[ImaginaryI], 
      0.075225277806`\ \[ImaginaryI], \(-0.107151877295`\)\ \[ImaginaryI], 
      0.107151877295`\ \[ImaginaryI], \(-0.167174953615`\)\ \[ImaginaryI], 
      0.167174953615`\ \[ImaginaryI], 0.022635369684`, 0.031926599489`, 
      0.045031637174`, \(\(0.06107165822`\)\(\[InvisibleSpace]\)\) - 
        0.106103295395`\ \[ImaginaryI], \(\(0.06107165822`\)\(\
\[InvisibleSpace]\)\) + 0.106103295395`\ \[ImaginaryI], 
      0.075225277806`, \(\(0.106103295395`\)\(\[InvisibleSpace]\)\) - 
        0.06107165822`\ \[ImaginaryI], \(\(0.106103295395`\)\(\
\[InvisibleSpace]\)\) + 0.06107165822`\ \[ImaginaryI], 0.107151877295`, 
      0.167174953615`, \(\(0.212206590789`\)\(\[InvisibleSpace]\)\) - 
        0.257238227964`\ \[ImaginaryI], \(\(0.212206590789`\)\(\
\[InvisibleSpace]\)\) + 0.257238227964`\ \[ImaginaryI], 0.257238227964`, 
      0.549444818753`, 1.750986059647`, 1.773621429331`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Count[Chop[Map[#[\([2]\)] &, list]], #] &, unique]\)], "Input"],

Cell[BoxData[
    \({16, 4, 16, 4, 4, 4, 4, 4, 4, 28, 4, 4, 4, 4, 8, 8, 3, 5, 3, 5, 4, 4, 
      2, 20, 12, 4, 4, 4, 4, 4, 4, 4, 4, 4, 8, 16, 4, 4}\)], "Output"]
}, Open  ]],

Cell["Reconstruct hamiltonian matrix", "Text"],

Cell[BoxData[
    \(\(\(\ \)\(Block[{ (*switch\ to\ Mma\ indexing*) \ 
        list1 = Map[\((# + {1, 0, 1})\) &, list], 
        pointers1 = pointers + 1}, \  (*fill\ matrix\ with\ zeros*) \ 
      mat = Table[
          0, {Length[pointers1] - 1}, {Length[pointers1] - 
              1}]; \  (*add\ nonzero\ elements*) Do[
        Block[{row = list1[\([j]\)]}, 
          If[row[\([3]\)] == i, mat[\([i, i]\)] = Chop[row[\([2]\)]], 
            mat[\([row[\([3]\)], i]\)] = Conjugate[row[\([2]\)]]; 
            mat[\([i, row[\([3]\)]]\)] = row[\([2]\)]]], {i, 
          Length[pointers1] - 1}, {j, pointers1[\([i]\)], 
          pointers1[\([i + 1]\)] - 1}];]\)\)\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sort[Chop[Eigenvalues[mat]]]\)], "Input"],

Cell[BoxData[
    \({0.00014141655899154714`, 0.00014141655899176377`, 
      0.07999999999699996`, 0.07999999999899987`, 0.07999999999900004`, 
      0.08000000000099974`, 0.08000000000099998`, 0.0800000000010001`, 
      0.4397959704917283`, 0.48651216380520834`, 0.4865121638052085`, 
      0.7482394140361088`, 0.8770267659744643`, 1.2442993562033082`, 
      1.2442993562033091`, 1.7417506021109082`, 1.750986059647`, 
      1.7688870774207912`, 1.7688870774207925`, 1.8027856568315384`, 
      1.931573008769891`, 1.9397848316206991`, 1.9397848316207003`, 
      2.238139886882364`}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(mat\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[mat]\)], "Input"],

Cell[BoxData[
    \({24, 24}\)], "Output"]
}, Open  ]],

Cell["Position of given nonzero element in matrix", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(unique[\([13]\)]\)], "Input"],

Cell[BoxData[
    \(\(-0.921317731924`\)\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(poslist = Position[Chop[mat], unique[\([13]\)]]\)], "Input"],

Cell[BoxData[
    \({{57, 244}, {60, 61}, {61, 121}, {118, 1103}, {119, 183}, {119, 
        1104}, {120, 122}, {122, 59}, {183, 58}, {244, 118}, {365, 
        949}, {366, 950}, {425, 612}, {425, 919}, {426, 920}, {428, 
        429}, {429, 489}, {487, 551}, {488, 490}, {490, 427}, {551, 
        426}, {612, 486}, {733, 796}, {734, 797}, {765, 733}, {766, 
        734}, {919, 486}, {920, 487}, {980, 365}, {981, 366}, {1103, 
        57}, {1104, 58}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(poslistdiagonal = Select[poslist, Apply[SameQ, #] &]\)], "Input"],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[%]\)], "Input"],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(nonzeros = 
      Select[Range[
          Length[inner]], \((inner[\([#]\)] \[NotEqual] 0)\) &]\)], "Input"],

Cell[BoxData[
    \({1, 2, 5, 6, 9, 10, 13, 14}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(zerolist = Complement[Range[Length[inner]], nonzeros]\)], "Input"],

Cell[BoxData[
    \({3, 4, 7, 8, 11, 12, 15, 16, 17, 18, 19, 20, 21, 22, 23, 
      24}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(inner\)], "Input"],

Cell[BoxData[
    \({1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
      0}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[mat]\)], "Input"],

Cell[BoxData[
    \({24, 24}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(aa = mat[\([nonzeros, nonzeros]\)];\)\)], "Input"],

Cell[BoxData[
    \(\(bb = mat[\([nonzeros, zerolist]\)];\)\)], "Input"],

Cell[BoxData[
    \(\(cc = mat[\([zerolist, zerolist]\)];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Chop[Eigenvalues[cc]]\)], "Input"],

Cell[BoxData[
    \({\(-6.000000000000002`\), \(-6.`\), \(-6.`\), \(-6.`\), \(-6.`\), \
\(-6.`\), \(-5.9999999999999964`\), \(-4.691507283741782`\), \
\(-2.0000000000000004`\), \(-2.`\), \(-2.`\), \(-2.`\), \(-2.`\), \
\(-1.9999999999999998`\), \(-1.9999999999999991`\), 
      1.784465462685783`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Chop[
      Sort[Eigenvalues[
          aa - bb . Inverse[cc] . Transpose[Conjugate[bb]]]]]\)], "Input"],

Cell[BoxData[
    \({\(-0.0764468213508016`\), 0.05333333333347701`, 0.05333333333347701`, 
      0.05333333333347701`, 0.05333333333347701`, 0.05333333333347701`, 
      0.05333333333347702`, 0.05333333333347703`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[%]\)], "Input"],

Cell[BoxData[
    \(8\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basis", "Section"],

Cell["Input basis from tb", "Text"],

Cell[BoxData[
    \(<< "\<~/four/basis2.out\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(basis[\([35]\)]\)], "Input"],

Cell[BoxData[
    \(State[b[1\/2, 1\/2, u], a[1, \(-1\)], a[2, 2], 
      dd[1\/2, \(-\(1\/2\)\), u]]\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(remat = mat[\([reorder, reorder]\)];\)\)], "Input"],

Cell[BoxData[
    \(Union[Flatten[remat - Conjugate[Transpose[remat]]]]\)], "Input"],

Cell[BoxData[
    \(Definition[remat]\  >> \ "\</home/bvds/four/junk.m\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(pairlist = 
      Block[{a = {}}, 
        Do[If[Abs[mat[\([i, j]\)] - remat[\([i, j]\)]] > 10^\(-10\), 
            a = Append[a, {i, j}]], {i, 10}, {j, 10}]; a]\)], "Input"],

Cell[BoxData[
    \({{1, 1}, {2, 2}, {3, 3}, {4, 4}, {4, 9}, {5, 5}, {6, 6}, {7, 7}, {8, 
        8}, {8, 9}, {9, 4}, {9, 8}, {9, 9}, {10, 10}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Sort[basis]\)[\([{49, 356}]\)]\)], "Input"],

Cell[BoxData[
    \({State[b[1\/2, \(-\(1\/2\)\), u], a[1, \(-2\)], 
        d[5\/2, \(-\(1\/2\)\), u]], 
      State[b[1\/2, \(-\(1\/2\)\), u], a[1, \(-2\)], a[2, \(-2\)], 
        dd[1\/2, 1\/2, u]]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(remat[\([49, 356]\)]\)], "Input"],

Cell[BoxData[
    \(\(\(1.128379167096`\)\(\[InvisibleSpace]\)\) + 
      0.`\ \[ImaginaryI]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((\(Sort[basis]\)[\([#[\([1]\)], 1]\)] === \(Sort[
                basis]\)[\([#[\([2]\)], 1]\)])\) &, pairlist]\)], "Input"],

Cell[BoxData[
    \({True, True, True, True, False, True, True, True, True, False, False, 
      False, True, True}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(reorder = \(Transpose[
          Sort[Transpose[{basis, 
                Range[1, Length[basis]]}]]]\)[\([2]\)]\)], "Input"],

Cell[BoxData[
    \({490, 429, 612, 551, 122, 61, 244, 183, 662, 637, 712, 687, 294, 269, 
      344, 319, 722, 717, 732, 727, 354, 349, 364, 359, 734, 733, 736, 735, 
      366, 365, 368, 367, 982, 951, 798, 767, 1068, 1055, 884, 871, 1098, 
      1095, 914, 911, 1104, 1103, 920, 919, 508, 499, 526, 517, 386, 377, 
      404, 395, 447, 438, 465, 456, 569, 560, 587, 578, 536, 531, 546, 541, 
      414, 409, 424, 419, 475, 470, 485, 480, 597, 592, 607, 602, 548, 547, 
      550, 549, 426, 425, 428, 427, 487, 486, 489, 488, 609, 608, 611, 610, 
      140, 131, 158, 149, 18, 9, 36, 27, 79, 70, 97, 88, 201, 192, 219, 210, 
      168, 163, 178, 173, 46, 41, 56, 51, 107, 102, 117, 112, 229, 224, 239, 
      234, 180, 179, 182, 181, 58, 57, 60, 59, 119, 118, 121, 120, 241, 240, 
      243, 242, 672, 667, 682, 677, 622, 617, 632, 627, 647, 642, 657, 652, 
      697, 692, 707, 702, 684, 683, 686, 685, 634, 633, 636, 635, 659, 658, 
      661, 660, 709, 708, 711, 710, 304, 299, 314, 309, 254, 249, 264, 259, 
      279, 274, 289, 284, 329, 324, 339, 334, 316, 315, 318, 317, 266, 265, 
      268, 267, 291, 290, 293, 292, 341, 340, 343, 342, 724, 723, 726, 725, 
      714, 713, 716, 715, 719, 718, 721, 720, 729, 728, 731, 730, 356, 355, 
      358, 357, 346, 345, 348, 347, 351, 350, 353, 352, 361, 360, 363, 362, 
      992, 987, 930, 925, 961, 956, 1022, 1017, 1006, 1003, 944, 941, 975, 
      972, 1036, 1033, 1012, 1011, 950, 949, 981, 980, 1042, 1041, 808, 803, 
      746, 741, 777, 772, 838, 833, 822, 819, 760, 757, 791, 788, 852, 849, 
      828, 827, 766, 765, 797, 796, 858, 857, 1074, 1071, 1048, 1045, 1061, 
      1058, 1086, 1083, 1080, 1079, 1054, 1053, 1067, 1066, 1092, 1091, 890, 
      887, 864, 861, 877, 874, 902, 899, 896, 895, 870, 869, 883, 882, 908, 
      907, 1100, 1099, 1094, 1093, 1097, 1096, 1102, 1101, 916, 915, 910, 
      909, 913, 912, 918, 917, 510, 509, 512, 511, 492, 491, 494, 493, 501, 
      500, 503, 502, 519, 518, 521, 520, 514, 513, 516, 515, 496, 495, 498, 
      497, 505, 504, 507, 506, 523, 522, 525, 524, 388, 387, 390, 389, 370, 
      369, 372, 371, 379, 378, 381, 380, 397, 396, 399, 398, 392, 391, 394, 
      393, 374, 373, 376, 375, 383, 382, 385, 384, 401, 400, 403, 402, 449, 
      448, 451, 450, 431, 430, 433, 432, 440, 439, 442, 441, 458, 457, 460, 
      459, 453, 452, 455, 454, 435, 434, 437, 436, 444, 443, 446, 445, 462, 
      461, 464, 463, 571, 570, 573, 572, 553, 552, 555, 554, 562, 561, 564, 
      563, 580, 579, 582, 581, 575, 574, 577, 576, 557, 556, 559, 558, 566, 
      565, 568, 567, 584, 583, 586, 585, 538, 537, 540, 539, 528, 527, 530, 
      529, 533, 532, 535, 534, 543, 542, 545, 544, 416, 415, 418, 417, 406, 
      405, 408, 407, 411, 410, 413, 412, 421, 420, 423, 422, 477, 476, 479, 
      478, 467, 466, 469, 468, 472, 471, 474, 473, 482, 481, 484, 483, 599, 
      598, 601, 600, 589, 588, 591, 590, 594, 593, 596, 595, 604, 603, 606, 
      605, 142, 141, 144, 143, 124, 123, 126, 125, 133, 132, 135, 134, 151, 
      150, 153, 152, 146, 145, 148, 147, 128, 127, 130, 129, 137, 136, 139, 
      138, 155, 154, 157, 156, 20, 19, 22, 21, 2, 1, 4, 3, 11, 10, 13, 12, 
      29, 28, 31, 30, 24, 23, 26, 25, 6, 5, 8, 7, 15, 14, 17, 16, 33, 32, 35, 
      34, 81, 80, 83, 82, 63, 62, 65, 64, 72, 71, 74, 73, 90, 89, 92, 91, 85, 
      84, 87, 86, 67, 66, 69, 68, 76, 75, 78, 77, 94, 93, 96, 95, 203, 202, 
      205, 204, 185, 184, 187, 186, 194, 193, 196, 195, 212, 211, 214, 213, 
      207, 206, 209, 208, 189, 188, 191, 190, 198, 197, 200, 199, 216, 215, 
      218, 217, 170, 169, 172, 171, 160, 159, 162, 161, 165, 164, 167, 166, 
      175, 174, 177, 176, 48, 47, 50, 49, 38, 37, 40, 39, 43, 42, 45, 44, 53, 
      52, 55, 54, 109, 108, 111, 110, 99, 98, 101, 100, 104, 103, 106, 105, 
      114, 113, 116, 115, 231, 230, 233, 232, 221, 220, 223, 222, 226, 225, 
      228, 227, 236, 235, 238, 237, 674, 673, 676, 675, 664, 663, 666, 665, 
      669, 668, 671, 670, 679, 678, 681, 680, 624, 623, 626, 625, 614, 613, 
      616, 615, 619, 618, 621, 620, 629, 628, 631, 630, 649, 648, 651, 650, 
      639, 638, 641, 640, 644, 643, 646, 645, 654, 653, 656, 655, 699, 698, 
      701, 700, 689, 688, 691, 690, 694, 693, 696, 695, 704, 703, 706, 705, 
      306, 305, 308, 307, 296, 295, 298, 297, 301, 300, 303, 302, 311, 310, 
      313, 312, 256, 255, 258, 257, 246, 245, 248, 247, 251, 250, 253, 252, 
      261, 260, 263, 262, 281, 280, 283, 282, 271, 270, 273, 272, 276, 275, 
      278, 277, 286, 285, 288, 287, 331, 330, 333, 332, 321, 320, 323, 322, 
      326, 325, 328, 327, 336, 335, 338, 337, 994, 993, 984, 983, 989, 988, 
      998, 997, 996, 995, 986, 985, 991, 990, 1000, 999, 932, 931, 922, 921, 
      927, 926, 936, 935, 934, 933, 924, 923, 929, 928, 938, 937, 963, 962, 
      953, 952, 958, 957, 967, 966, 965, 964, 955, 954, 960, 959, 969, 968, 
      1024, 1023, 1014, 1013, 1019, 1018, 1028, 1027, 1026, 1025, 1016, 1015, 
      1021, 1020, 1030, 1029, 1008, 1007, 1002, 1001, 1005, 1004, 1010, 1009, 
      946, 945, 940, 939, 943, 942, 948, 947, 977, 976, 971, 970, 974, 973, 
      979, 978, 1038, 1037, 1032, 1031, 1035, 1034, 1040, 1039, 810, 809, 
      800, 799, 805, 804, 814, 813, 812, 811, 802, 801, 807, 806, 816, 815, 
      748, 747, 738, 737, 743, 742, 752, 751, 750, 749, 740, 739, 745, 744, 
      754, 753, 779, 778, 769, 768, 774, 773, 783, 782, 781, 780, 771, 770, 
      776, 775, 785, 784, 840, 839, 830, 829, 835, 834, 844, 843, 842, 841, 
      832, 831, 837, 836, 846, 845, 824, 823, 818, 817, 821, 820, 826, 825, 
      762, 761, 756, 755, 759, 758, 764, 763, 793, 792, 787, 786, 790, 789, 
      795, 794, 854, 853, 848, 847, 851, 850, 856, 855, 1076, 1075, 1070, 
      1069, 1073, 1072, 1078, 1077, 1050, 1049, 1044, 1043, 1047, 1046, 1052, 
      1051, 1063, 1062, 1057, 1056, 1060, 1059, 1065, 1064, 1088, 1087, 1082, 
      1081, 1085, 1084, 1090, 1089, 892, 891, 886, 885, 889, 888, 894, 893, 
      866, 865, 860, 859, 863, 862, 868, 867, 879, 878, 873, 872, 876, 875, 
      881, 880, 904, 903, 898, 897, 901, 900, 906, 905}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[basis[\([#]\)] &, poslist, {2}]\)], "Input"],

Cell[BoxData[
    \({{State[b[1\/2, 1\/2, u], a[3, \(-1\)], d[1\/2, 1\/2, u]], 
        State[b[1\/2, 1\/2, u], dd[7\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, 1\/2, u], a[3, \(-1\)], dd[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, 1\/2, u], d[7\/2, 1\/2, u]]}, {State[b[1\/2, 1\/2, u], 
          d[7\/2, 1\/2, u]], 
        State[b[1\/2, 1\/2, u], a[3, 1], dd[1\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, 1\/2, u], a[3, 1], d[1\/2, 1\/2, u]], 
        State[bb[7\/2, \(-\(1\/2\)\), u], d[1\/2, 1\/2, u]]}, {State[
          b[1\/2, 1\/2, u], a[3, 1], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, 1\/2, u], dd[7\/2, 1\/2, u]]}, {State[b[1\/2, 1\/2, u], 
          a[3, 1], d[1\/2, \(-\(1\/2\)\), u]], 
        State[bb[7\/2, \(-\(1\/2\)\), u], d[1\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, 1\/2, u], a[3, 1], dd[1\/2, 1\/2, u]], 
        State[b[1\/2, 1\/2, u], d[7\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, 1\/2, u], d[7\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, 1\/2, u], a[3, \(-1\)], dd[1\/2, 1\/2, u]]}, {State[
          b[1\/2, 1\/2, u], dd[7\/2, 1\/2, u]], 
        State[b[1\/2, 1\/2, u], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\), u]]}, {State[b[1\/2, 1\/2, u], 
          dd[7\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, 1\/2, u], a[3, 1], d[1\/2, 1\/2, u]]}, {State[
          b[7\/2, 1\/2, u], d[1\/2, 1\/2, u]], 
        State[bb[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          d[1\/2, 1\/2, u]]}, {State[b[7\/2, 1\/2, u], 
          d[1\/2, \(-\(1\/2\)\), u]], 
        State[bb[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\), u]]}, {State[b[1\/2, \(-\(1\/2\)\), u], 
          a[3, \(-1\)], d[1\/2, 1\/2, u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], dd[7\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], d[1\/2, 1\/2, u]], 
        State[bb[7\/2, 1\/2, u], d[1\/2, 1\/2, u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\), u]], 
        State[bb[7\/2, 1\/2, u], d[1\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          dd[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], d[7\/2, 1\/2, u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], d[7\/2, 1\/2, u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, 1], 
          dd[1\/2, \(-\(1\/2\)\), u]]}, {State[b[1\/2, \(-\(1\/2\)\), u], 
          a[3, 1], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], dd[7\/2, 1\/2, u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], a[3, 1], dd[1\/2, 1\/2, u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], d[7\/2, \(-\(1\/2\)\), u]]}, {State[
          b[1\/2, \(-\(1\/2\)\), u], d[7\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          dd[1\/2, 1\/2, u]]}, {State[b[1\/2, \(-\(1\/2\)\), u], 
          dd[7\/2, 1\/2, u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\), u]]}, {State[b[1\/2, \(-\(1\/2\)\), u], 
          dd[7\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, 1], d[1\/2, 1\/2, u]]}, {State[
          b[7\/2, \(-\(1\/2\)\), u], d[1\/2, 1\/2, u]], 
        State[bb[1\/2, 1\/2, u], a[3, 1], d[1\/2, 1\/2, u]]}, {State[
          b[7\/2, \(-\(1\/2\)\), u], d[1\/2, \(-\(1\/2\)\), u]], 
        State[bb[1\/2, 1\/2, u], a[3, 1], d[1\/2, \(-\(1\/2\)\), u]]}, {State[
          bb[1\/2, 1\/2, u], a[3, \(-1\)], d[1\/2, 1\/2, u]], 
        State[b[7\/2, \(-\(1\/2\)\), u], d[1\/2, 1\/2, u]]}, {State[
          bb[1\/2, 1\/2, u], a[3, \(-1\)], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[7\/2, \(-\(1\/2\)\), u], d[1\/2, \(-\(1\/2\)\), u]]}, {State[
          bb[7\/2, 1\/2, u], d[1\/2, 1\/2, u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, 1], d[1\/2, 1\/2, u]]}, {State[
          bb[7\/2, 1\/2, u], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, \(-\(1\/2\)\), u], a[3, 1], 
          d[1\/2, \(-\(1\/2\)\), u]]}, {State[bb[1\/2, \(-\(1\/2\)\), u], 
          a[3, 1], d[1\/2, 1\/2, u]], 
        State[b[7\/2, 1\/2, u], d[1\/2, 1\/2, u]]}, {State[
          bb[1\/2, \(-\(1\/2\)\), u], a[3, 1], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[7\/2, 1\/2, u], d[1\/2, \(-\(1\/2\)\), u]]}, {State[
          bb[7\/2, \(-\(1\/2\)\), u], d[1\/2, 1\/2, u]], 
        State[b[1\/2, 1\/2, u], a[3, \(-1\)], d[1\/2, 1\/2, u]]}, {State[
          bb[7\/2, \(-\(1\/2\)\), u], d[1\/2, \(-\(1\/2\)\), u]], 
        State[b[1\/2, 1\/2, u], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\), u]]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[Sort[%], OrderedQ]\)], "Input"],

Cell[BoxData[
    \({{State[b[1\/2, \(-\(1\/2\)\)], d[7\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, \(-1\)], dd[1\/2, 1\/2]]}, {State[
          b[1\/2, \(-\(1\/2\)\)], d[7\/2, 1\/2]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, 1], 
          dd[1\/2, \(-\(1\/2\)\)]]}, {State[b[1\/2, \(-\(1\/2\)\)], 
          dd[7\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, 1], d[1\/2, 1\/2]]}, {State[
          b[1\/2, \(-\(1\/2\)\)], dd[7\/2, 1\/2]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\)]]}, {State[b[1\/2, 1\/2], 
          d[7\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, 1\/2], a[3, \(-1\)], dd[1\/2, 1\/2]]}, {State[
          b[1\/2, 1\/2], d[7\/2, 1\/2]], 
        State[b[1\/2, 1\/2], a[3, 1], dd[1\/2, \(-\(1\/2\)\)]]}, {State[
          b[1\/2, 1\/2], dd[7\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, 1\/2], a[3, 1], d[1\/2, 1\/2]]}, {State[b[1\/2, 1\/2], 
          dd[7\/2, 1\/2]], 
        State[b[1\/2, 1\/2], a[3, \(-1\)], d[1\/2, \(-\(1\/2\)\)]]}, {State[
          b[7\/2, \(-\(1\/2\)\)], d[1\/2, \(-\(1\/2\)\)]], 
        State[bb[1\/2, 1\/2], a[3, 1], d[1\/2, \(-\(1\/2\)\)]]}, {State[
          b[7\/2, \(-\(1\/2\)\)], d[1\/2, 1\/2]], 
        State[bb[1\/2, 1\/2], a[3, 1], d[1\/2, 1\/2]]}, {State[b[7\/2, 1\/2], 
          d[1\/2, \(-\(1\/2\)\)]], 
        State[bb[1\/2, \(-\(1\/2\)\)], a[3, \(-1\)], 
          d[1\/2, \(-\(1\/2\)\)]]}, {State[b[7\/2, 1\/2], d[1\/2, 1\/2]], 
        State[bb[1\/2, \(-\(1\/2\)\)], a[3, \(-1\)], d[1\/2, 1\/2]]}, {State[
          bb[7\/2, \(-\(1\/2\)\)], d[1\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, 1\/2], a[3, \(-1\)], d[1\/2, \(-\(1\/2\)\)]]}, {State[
          bb[7\/2, \(-\(1\/2\)\)], d[1\/2, 1\/2]], 
        State[b[1\/2, 1\/2], a[3, \(-1\)], d[1\/2, 1\/2]]}, {State[
          bb[7\/2, 1\/2], d[1\/2, \(-\(1\/2\)\)]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, 1], 
          d[1\/2, \(-\(1\/2\)\)]]}, {State[bb[7\/2, 1\/2], d[1\/2, 1\/2]], 
        State[b[1\/2, \(-\(1\/2\)\)], a[3, 1], d[1\/2, 1\/2]]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[%]\)], "Input"],

Cell[BoxData[
    \(16\)], "Output"]
}, Open  ]],

Cell["All", "Text"],

Cell[BoxData[
    \(expr = 
        Sort[Map[Sort, 
            Map[{basis[\([#[\([1]\)]]\)], basis[\([#[\([2]\)]]\)]} &, 
              poslist]]] >> "\<~/four/states.out\>"\)], "Input"],

Cell["Diagonal only", "Text"],

Cell[BoxData[
    \(expr2 = 
        Sort[Map[Sort, 
            Map[{basis[\([#[\([1]\)]]\)], basis[\([#[\([2]\)]]\)]} &, 
              poslistdiagonal]]] >> "\<~/baryon/states2.out\>"\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Length[expr2]\)], "Input"],

Cell[BoxData[
    \(64\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(expr2\)], "Input"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowSize->{808, 667},
WindowMargins->{{91, Automatic}, {Automatic, 19}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"four"}, "test_laso.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 30, 0, 54, "Section"],
Cell[1772, 53, 73, 0, 32, "Text"],
Cell[1848, 55, 58, 1, 27, "Input"],
Cell[1909, 58, 75, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2009, 62, 78, 1, 27, "Input"],
Cell[2090, 65, 1542, 23, 107, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3669, 93, 87, 1, 27, "Input"],
Cell[3759, 96, 161, 2, 43, "Output"]
}, Open  ]],
Cell[3935, 101, 46, 0, 32, "Text"],
Cell[3984, 103, 708, 13, 107, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[4717, 120, 61, 1, 27, "Input"],
Cell[4781, 123, 602, 9, 59, "Output"]
}, Open  ]],
Cell[5398, 135, 36, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[5459, 140, 48, 1, 27, "Input"],
Cell[5510, 143, 42, 1, 27, "Output"]
}, Open  ]],
Cell[5567, 147, 59, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[5651, 151, 49, 1, 27, "Input"],
Cell[5703, 154, 69, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5809, 160, 80, 1, 27, "Input"],
Cell[5892, 163, 468, 7, 75, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6397, 175, 85, 1, 27, "Input"],
Cell[6485, 178, 36, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6558, 184, 42, 1, 27, "Input"],
Cell[6603, 187, 35, 1, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[6687, 194, 24, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[6736, 198, 128, 3, 27, "Input"],
Cell[6867, 203, 61, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6965, 209, 86, 1, 27, "Input"],
Cell[7054, 212, 101, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7192, 219, 38, 1, 27, "Input"],
Cell[7233, 222, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7383, 229, 48, 1, 27, "Input"],
Cell[7434, 232, 42, 1, 27, "Output"]
}, Open  ]],
Cell[7491, 236, 72, 1, 27, "Input"],
Cell[7566, 239, 72, 1, 27, "Input"],
Cell[7641, 242, 72, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[7738, 247, 54, 1, 27, "Input"],
Cell[7795, 250, 311, 5, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8143, 260, 124, 3, 27, "Input"],
Cell[8270, 265, 228, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8535, 273, 42, 1, 27, "Input"],
Cell[8580, 276, 35, 1, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8664, 283, 24, 0, 54, "Section"],
Cell[8691, 285, 35, 0, 32, "Text"],
Cell[8729, 287, 59, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[8813, 292, 48, 1, 27, "Input"],
Cell[8864, 295, 115, 2, 43, "Output"]
}, Open  ]],
Cell[8994, 300, 73, 1, 27, "Input"],
Cell[9070, 303, 84, 1, 27, "Input"],
Cell[9157, 306, 86, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[9268, 311, 195, 4, 59, "Input"],
Cell[9466, 317, 157, 2, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9660, 324, 65, 1, 27, "Input"],
Cell[9728, 327, 215, 4, 43, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9980, 336, 53, 1, 27, "Input"],
Cell[10036, 339, 106, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10179, 346, 150, 2, 27, "Input"],
Cell[10332, 350, 129, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10498, 357, 144, 3, 27, "Input"],
Cell[10645, 362, 6097, 79, 907, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16779, 446, 68, 1, 27, "Input"],
Cell[16850, 449, 4565, 74, 1159, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21452, 528, 58, 1, 27, "Input"],
Cell[21513, 531, 2069, 34, 583, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23619, 570, 42, 1, 27, "Input"],
Cell[23664, 573, 36, 1, 27, "Output"]
}, Open  ]],
Cell[23715, 577, 19, 0, 32, "Text"],
Cell[23737, 579, 187, 4, 27, "Input"],
Cell[23927, 585, 29, 0, 32, "Text"],
Cell[23959, 587, 199, 4, 43, "Input"],

Cell[CellGroupData[{
Cell[24183, 595, 46, 1, 27, "Input"],
Cell[24232, 598, 36, 1, 27, "Output"]
}, Open  ]],
Cell[24283, 602, 38, 1, 27, "Input"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

