(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      4732,        156]*)
(*NotebookOutlinePosition[      5651,        185]*)
(*  CellTagsIndexPosition[      5607,        181]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["\<\
The data is from the file \"laso.prof1.nb\" run on bvds.geneva.edu \
on April 2000.\
\>", "Text"],

Cell["\<\
Data: {number of states, number of nonzero matrix elements, number \
of matrix multiplies, time to construct matrix, time to diagonalize matrix}\
\
\>", "Text"],

Cell[BoxData[
    \(\(data = {{8615, 339076, 92*3, 8.3, 14.36}, \[IndentingNewLine]{14439, 
            639674, 102*3, 15.53, 30.36}, \[IndentingNewLine]{23442, \ 
            1137265, 122*3, 27.91, 66.05}, \[IndentingNewLine]{36022, 
            1916632, 126*3, 46.0, 115.87}, \[IndentingNewLine]{54019, 
            3101540, 229*3, 74.32, 346.96}};\)\)], "Input"],

Cell["Number of matrix elements created per second:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([2]\)]/#[\([4]\)])\) &, data]\)], "Input"],

Cell[BoxData[
    \({40852.530120481926`, 41189.568576947844`, 40747.58151200287`, 
      41665.91304347826`, 41732.23896663079`}\)], "Output"]
}, Open  ]],

Cell["\<\
Number of iterations per second of the line:  (vector element) = \
(matrix element)*(vector element).\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(dl = 
      Map[{Log[N[#[\([2]\)]]], \((2*#[\([2]\)]\  + #[\([1]\)])\) \
#[\([3]\)]/#[\([\(-1\)]\)]} &, data]\)], "Input"],

Cell[BoxData[
    \({{12.733979550026596`, 1.31997`*^7}, {13.368713950560338`, 
        1.3040145652173914`*^7}, {13.944136815057291`, 
        1.2733652566237697`*^7}, {14.46608003698533`, 
        1.2622681522395788`*^7}, {14.947409320297474`, 
        1.2389402274037354`*^7}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitting = Fit[dl, {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(1.791245551651095`*^7 - 368220.2398167023`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      Plot[fitting, {x, dl[\([1, 1]\)], dl[\([\(-1\), 1]\)]}, 
        DisplayFunction \[Rule] Identity], 
      ListPlot[dl, DisplayFunction \[Rule] Identity], 
      DisplayFunction \[Rule] $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Assuming that the routine spends all its time doing matrix-vector \
multiplications (actually, it is 97% or better, see file \"laso.prof2\"), \
there are:
2 stride one reads
1 random read
0.5 stride one writes
0.5  random writes
one double precision add
one double precision multiply
Each involves eight bytes.\
\>", "Text"],

Cell["\<\
Thus, the random read proceeds at a rate of 100 Mbytes/sec. and the \
stride one reads proceed at a rate of 200 Mbytes/sec.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(1.2622681522395788`*^7*8\)], "Input"],

Cell[BoxData[
    \(1.009814521791663`*^8\)], "Output"]
}, Open  ]],

Cell["\<\
The measured maximum fro stride 1read and writes is about 300 \
Mbytes/sec. as measured by the benchmark \"stream.\"  This is identical to \
the total memory access rate above.\
\>", "Text"]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 800}, {0, 600}},
WindowSize->{520, 455},
WindowMargins->{{53, Automatic}, {Automatic, 50}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"three"}, "laso.prof1.nb.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 107, 3, 32, "Text"],
Cell[1827, 54, 170, 4, 50, "Text"],
Cell[2000, 60, 365, 5, 91, "Input"],
Cell[2368, 67, 61, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[2454, 71, 73, 1, 27, "Input"],
Cell[2530, 74, 143, 2, 27, "Output"]
}, Open  ]],
Cell[2688, 79, 125, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[2838, 86, 142, 3, 59, "Input"],
Cell[2983, 91, 293, 5, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3313, 101, 61, 1, 27, "Input"],
Cell[3377, 104, 79, 1, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3493, 110, 248, 5, 75, "Input"],
Cell[3744, 117, 130, 3, 27, "Output"]
}, Open  ]],
Cell[3889, 123, 334, 11, 176, "Text"],
Cell[4226, 136, 147, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[4398, 143, 57, 1, 27, "Input"],
Cell[4458, 146, 55, 1, 29, "Output"]
}, Open  ]],
Cell[4528, 150, 200, 4, 50, "Text"]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

