(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    116371,       1970]*)
(*NotebookOutlinePosition[    117570,       2009]*)
(*  CellTagsIndexPosition[    117526,       2005]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Spectra for 3+1 transverse lattice", "Subtitle"],

Cell["Brett van de Sande", "Subsubtitle"],

Cell["All of the following are for spectra computed in 2002.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cross[x_, y_, z_] := {Line[{{x, y - z}, {x, y + z}}], 
        Line[{{x - 0.1, y}, {x + 0.1, y}}]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(cross\)\" is similar to \
existing symbol \"\!\(Cross\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(<< \ Statistics`LinearRegression`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(data = 
      Block[\n\t\t{x, fp = OpenRead["\<~/three/ttraj_2_composite.out\>"], 
          info}, \n\t\tinfo = 
          Read[fp, Record, RecordSeparators -> "\<\n\n\>"]; \n\t\tx = 
          ReadList[fp, Number, 
            RecordLists -> True, \n\t\t\tRecordSeparators -> "\<\n\n\>"]; 
        Close[fp]; \n\t\t{info, Chop[\ x]}\n\ ]\)], "Input"],

Cell[BoxData[
    \(\(d3 = data; \)\)], "Input"],

Cell[BoxData[
    \(\(d5 = data; \)\)], "Input"],

Cell[BoxData[
    \(data = {d3[\([1]\)], 
        Table[If[d3[\([2, i, 3]\)] < d5[\([2, i, 3]\)], d3[\([2, i]\)], 
            d5[\([2, i]\)]], {i, Length[d3[\([2]\)]]}]}\)], "Input"],

Cell[CellGroupData[{

Cell["data three/ytraj_1_1?.out; periodic BC's", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for the 3+1 transverse lattice \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2\nUse chi^2 fit \
with 40 criteria and tolerance 0.01.\nOverall scale from minimizing chi^2.\n\
Parity doublets (charge,multi,state) with error for difference\n  over \
average.\n  ( 1, 5,0) and ( 1, 6,0), error 0.25\n  (-1, 5,0) and (-1, 6,0), \
error 0.25\n  (-1, 7,0) and (-1, 4,0), error 0.25\nSpectrum for P_perp a = \
(0,0), ( 0.25 0)\n  using 4 states (multiplet, charge, c^2 error for each) =\n\
  (15,  1, 0.25 2 2 2)\n  (15, -1, 0.25 2 2 2)\n  (16, -1, 0.25 2 2 2)\n  \
(16,  1, 0.25 2 2 2)\nSpectrum for P_perp a = (0,0), ( 0.25 0.25)\n  using 4 \
states (multiplet, charge, c^2 error for each) =\n  (17,  1, 0.25 2 2 2)\n  \
(17, -1, 0.25 2 2 2)\n  (18, -1, 0.25 2 2 2)\n  (18,  1, 0.25 2 2 2).\n\
Ordinary spectra multiplets, 4 states, (charge,multiplet) = ( 1,  3) \n  (-1, \
 3)  ( 1,  4)  (-1,  4)  ( 1,  5)  (-1,  5)  ( 1,  6) \n  (-1,  6)  (-1,  7)  \
( 1,  7) .\nAll spectra extrapolated using (K,p) = (12/2,8)  (12/2,6) \n  \
(14/2,6)  (18/2,6) .\nWinding potential using (n,K,p) = ( 2 0,16/2,6)  ( 2 \
0,16/2,4) \n  ( 2 0,26/2,4)  ( 3 0,18/2,7)  ( 3 0,18/2,5)  ( 3 0,26/2,5) \n  \
( 4 0,16/2,8)  ( 4 0,16/2,6)  ( 4 0,26/2,6) .\nRoundness of winding using \
(n,K,p) = ( 2 2,14/2,8) \n  ( 2 2,14/2,6)  ( 2 2,22/2,6) \n  with error 1; in \
G^2 N units.\nHeavy potential determined using (n,K,p,K_max) =\n  ( 0 \
0,26/2,2,3.5)  ( 0 0,26/2,4,3.5)  ( 0 0,26/2,2,3) \n  ( 0 0,42/2,2,3.5)  ( 0 \
0,42/2,2,4.25)  ( 0 0,70/2,2,4) \n  ( 0 0,70/2,2,4.5) ,\n L = 3.577-2.726*m \
5.058-3.856*m 7.154-5.452*m\n (all in G^2 N units); relative scale error=0.1.\
\nRoundness determined using (n,K,p,K_max) = ( 1 0,14/2,3,3.5) \n  ( 1 \
0,14/2,5,3.5)  ( 1 0,14/2,3,3)  ( 1 0,24/2,3,4) \n  ( 1 0,24/2,3,4.5)  ( 1 \
0,36/2,3,5)  ( 1 0,36/2,3,6) \n L=1.265-0.964*m and error 0.1;\n  ( 1 \
0,14/2,3,3.5)  ( 1 0,14/2,5,3.5)  ( 1 0,14/2,3,3) \n  ( 1 0,24/2,3,4)  ( 1 \
0,24/2,3,4.5)  ( 1 0,36/2,3,5) \n  ( 1 0,36/2,3,6) \n L=5.058-3.856*m and \
error 0.1;\n  ( 1 1,22/2,2,5)  ( 1 1,22/2,4,5)  ( 1 1,22/2,2,4.5) \n  ( 1 \
1,44/2,2,5)  ( 1 1,44/2,2,6)  ( 1 1,70/2,2,6) \n  ( 1 1,70/2,2,7) \n L=0+0*m \
and error 0.2; all in G^2 N units.\np-extrapolation using n=( 1 0) and (K,p) \
= (14/2,3) \n  (22/2,3)  (46/2,3)  (14/2,5)  (28/2,5)  (14/2,7) \n  (16/2,7) \
.\nResult format:\n Fit info, # steps, chi^2, and p damping, scale G^2 \
N/sigma.\n The 16 couplings  (G^2 N units);\n   which couplings -- if any -- \
were fit.\n Winding potential and heavy souce potential fits.\n Roundness of \
winding potential, 1 values, and roundness of\n   heavy source potential, 3 \
values, (G^2 N units)\n   showing measured value and derived value for each.\n\
 The rescaled spectrum for each P_perp*a and c^2 values.\n Finally come the \
states for the ordinary spectra.\>", {{2, 93, 
              77.74064998`, \(-8.719198472`\), 10.85717086`, 0, 
              1, \(-0.09322407979`\), 0.2630434656`, \(-0.1959827684`\), 
              4.125790871`, \(-0.2507965869`\), 
              0.963421868`, \(-1.694691785`\), \(-2.573457059`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.148365`, \(-0.384917`\), 
              0.239529`, 0.107477`, \(-0.196382`\), 0.37413`, 1.610182`, 
              0.831947`, 0.448302`, 0.358829`, 0.542833`, 0.565992`, 
              1.07444`, 0.502521`, 17.977756`, 18.01602`, 0.986182`, 
              22.94905`, 22.950565`, 0.039033`, 33.208187`, 33.245352`, 
              0.957858`, 35.665956`, 35.711642`, 1.177468`, 32.434295`, 
              32.453444`, 0.493539`, 48.116038`, 48.119004`, 0.07646`, 
              55.215593`, 55.208964`, \(-0.170857`\), 56.867021`, 56.870369`, 
              0.086288`, 20.678434`, 20.678454`, 0.00052`, 32.434295`, 
              32.436134`, 0.047408`, 48.116038`, 48.10006`, \(-0.411799`\), 
              50.583505`, 50.591917`, 0.2168`, 35.769827`, 35.770326`, 
              0.012873`, 39.559889`, 39.560246`, 0.009206`, 48.262844`, 
              48.263102`, 0.006663`, 55.013726`, 55.020498`, 0.174534`, 
              17.977756`, 18.053952`, 0.981914`, 22.94905`, 22.952478`, 
              0.044175`, 35.769827`, 35.74446`, \(-0.326891`\), 39.559889`, 
              39.55927`, \(-0.007972`\), 32.434295`, 32.471971`, 0.485512`, 
              48.116038`, 48.104004`, \(-0.15507`\), 51.208479`, 51.218117`, 
              0.124211`, 55.215593`, 55.224609`, 0.116184`, 20.678434`, 
              20.678444`, 0.000129`, 32.434295`, 32.438168`, 0.049907`, 
              48.116038`, 48.107029`, \(-0.116088`\), 50.583505`, 50.597868`, 
              0.18508`, 33.208187`, 33.277816`, 0.897283`, 35.665956`, 
              35.761045`, 1.225373`, 35.769827`, 35.776986`, 0.092266`, 
              48.262844`, 48.385444`, 1.579911`, 17.977756`, 22.94905`, 
              41.647389`, 47.230924`, 63.426534`, 62.221165`, 68.667568`, 
              62.629968`, 62.021436`, 64.472984`, 68.590065`, 76.28353`, 
              20.678434`, 50.583505`, 53.897017`, 59.419484`, 33.208187`, 
              35.665956`, 54.647512`, 51.199564`, 56.867021`, 59.055556`, 
              61.703044`, 65.252665`, 39.559889`, 56.787512`, 66.558733`, 
              75.754449`, 51.208479`, 63.693887`, 67.638807`, 74.285931`, 
              32.434295`, 48.116038`, 55.215593`, 58.117943`, 35.769827`, 
              48.262844`, 55.013726`, 58.083944`}, {2, 12, 
              104.6388092`, \(-0.25`\), 6.908512945`, 0.001967542671`, 
              1, \(-0.2859175775`\), \(-0.02304343231`\), \
\(-0.04617080277`\), 105.8441254`, 0.04714841419`, 
              41.51103398`, \(-0.7033299165`\), \(-0.5553591676`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.179423`, 
              0.032144`, \(-0.814782`\), 0.182172`, \(-0.022895`\), 
              0.135414`, 2.299824`, 1.36568`, 0.643067`, 0.59753`, 1.089321`, 
              1.03855`, 1.218054`, 0.763736`, 15.111727`, 15.120337`, 
              0.170748`, 18.289772`, 18.293292`, 0.069799`, 18.463436`, 
              18.462482`, \(-0.018918`\), 20.012137`, 
              19.999599`, \(-0.248673`\), 21.352172`, 21.359106`, 0.137525`, 
              34.271297`, 34.2875`, 0.321361`, 37.639672`, 
              37.615517`, \(-0.479066`\), 38.942955`, 38.952319`, 0.185718`, 
              21.352172`, 21.376417`, 0.480849`, 21.533315`, 
              21.513957`, \(-0.383918`\), 34.271297`, 
              34.268208`, \(-0.061249`\), 37.385563`, 
              37.337456`, \(-0.954108`\), 20.34719`, 20.350486`, 0.065368`, 
              21.081235`, 21.078258`, \(-0.059043`\), 35.168891`, 35.173403`, 
              0.08949`, 36.177941`, 36.174515`, \(-0.067957`\), 15.111727`, 
              15.128988`, 0.171165`, 18.463436`, 18.461079`, \(-0.023368`\), 
              20.015612`, 20.00606`, \(-0.094726`\), 20.34719`, 
              20.343972`, \(-0.031919`\), 21.352172`, 21.365369`, 0.130871`, 
              34.271297`, 34.269332`, \(-0.019479`\), 37.639672`, 
              37.633769`, \(-0.058535`\), 39.229684`, 39.244451`, 0.146433`, 
              21.352172`, 21.401543`, 0.489577`, 21.533315`, 
              21.494519`, \(-0.384713`\), 34.271297`, 34.300308`, 0.287686`, 
              37.385563`, 37.215029`, \(-1.691082`\), 18.289772`, 18.297206`, 
              0.073711`, 20.012137`, 19.99442`, \(-0.175688`\), 20.34719`, 
              20.358941`, 0.116521`, 34.722502`, 34.725791`, 0.032621`, 
              15.111727`, 18.463436`, 20.015612`, 25.694861`, 44.3849`, 
              46.227036`, 50.457269`, 45.996292`, 37.57672`, 39.347668`, 
              43.712704`, 45.202439`, 21.533315`, 37.385563`, 40.459752`, 
              45.312319`, 18.289772`, 20.012137`, 34.722502`, 36.172085`, 
              38.942955`, 45.730601`, 47.710275`, 44.725067`, 21.081235`, 
              40.23551`, 43.915252`, 45.391061`, 39.229684`, 43.972219`, 
              44.146551`, 50.019275`, 21.352172`, 34.271297`, 37.639672`, 
              41.365761`, 20.34719`, 35.168891`, 36.177941`, 39.232526`}, {2, 
              12, 115.7049474`, \(-9.005861658`\), 4.540098951`, 
              0.007919222016`, 1, 
              0.4766636174`, \(-0.03420033848`\), \(-0.08235225798`\), 
              8753.029993`, 0.1010361659`, 
              4193.815861`, \(-0.7551743188`\), \(-0.6304824078`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.187182`, \(-0.24278`\), \(-0.297378`\), 0.162433`, 
              0.138064`, \(-0.193217`\), 1.967611`, 1.217501`, 0.640258`, 
              0.425867`, 1.054655`, 0.929886`, 1.0438`, 0.519726`, 8.194479`, 
              8.205569`, 0.150789`, 11.478891`, 11.483413`, 0.061485`, 
              11.805708`, 11.803681`, \(-0.02756`\), 13.516896`, 
              13.502063`, \(-0.201688`\), 14.179256`, 14.183984`, 0.064297`, 
              21.745041`, 21.760969`, 0.216571`, 24.879085`, 
              24.856971`, \(-0.300698`\), 25.987886`, 26.200616`, 2.892531`, 
              14.179256`, 14.277147`, 1.33105`, 15.332993`, 
              15.242713`, \(-1.227559`\), 21.745041`, 
              21.742656`, \(-0.032432`\), 24.879085`, 
              24.733089`, \(-1.98514`\), 13.941576`, 13.947234`, 0.076929`, 
              14.879038`, 14.874472`, \(-0.062084`\), 22.79476`, 22.798083`, 
              0.045191`, 23.829103`, 23.820301`, \(-0.119685`\), 8.194479`, 
              8.216706`, 0.151112`, 11.805708`, 11.801239`, \(-0.030382`\), 
              13.526776`, 13.516366`, \(-0.070775`\), 13.941576`, 13.943791`, 
              0.015058`, 14.179256`, 14.189568`, 0.07011`, 21.745041`, 
              21.740668`, \(-0.029733`\), 24.879085`, 
              24.862458`, \(-0.113041`\), 25.090089`, 25.122759`, 0.222107`, 
              14.179256`, 14.364854`, 1.261804`, 15.332993`, 
              15.161956`, \(-1.162812`\), 21.745041`, 21.778237`, 0.225688`, 
              24.879085`, 24.615195`, \(-1.794081`\), 11.478891`, 11.4882`, 
              0.063287`, 13.516896`, 13.493948`, \(-0.156014`\), 13.941576`, 
              13.959182`, 0.119699`, 20.431596`, 20.451581`, 0.135872`, 
              8.194479`, 11.805708`, 13.526776`, 19.604149`, 29.957303`, 
              30.497453`, 30.586312`, 28.026623`, 24.87686`, 25.798683`, 
              29.95999`, 31.770545`, 15.332993`, 24.996866`, 27.01854`, 
              31.959614`, 11.478891`, 13.516896`, 20.431596`, 22.952507`, 
              26.189849`, 31.768682`, 31.544048`, 25.987886`, 14.879038`, 
              26.514363`, 30.093006`, 32.119862`, 25.090089`, 29.281336`, 
              30.270436`, 33.390648`, 14.179256`, 21.745041`, 24.879085`, 
              27.493374`, 13.941576`, 22.79476`, 23.829103`, 25.530603`}, {2, 
              12, 165.2901122`, \(-9.544393374`\), 2.731481242`, 
              0.01800590693`, 1, 
              0.5509797784`, \(-0.03715845381`\), \(-0.1044116999`\), 
              2754.663256`, 0.09105807955`, 
              1251.064888`, \(-0.7555401436`\), \(-0.683216902`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.200062`, \(-0.304618`\), \(-0.278805`\), 0.158956`, 
              0.181141`, \(-0.268979`\), 1.941335`, 1.261027`, 0.648919`, 
              0.285031`, 1.036896`, 0.871361`, 1.06043`, 0.30011`, 5.020508`, 
              5.02756`, 0.061653`, 7.305521`, 7.309481`, 0.034627`, 
              7.393786`, 7.396593`, 0.024539`, 8.856781`, 
              8.843645`, \(-0.114855`\), 8.788581`, 8.791406`, 0.024701`, 
              13.495326`, 13.505467`, 0.088669`, 15.546982`, 
              15.534244`, \(-0.111374`\), 16.43161`, 16.438195`, 0.057577`, 
              8.788581`, 8.955022`, 1.455267`, 9.761787`, 
              9.601186`, \(-1.404206`\), 13.495326`, 
              13.492437`, \(-0.025259`\), 15.546982`, 
              15.482602`, \(-0.562907`\), 8.953497`, 8.957865`, 0.038196`, 
              9.668161`, 9.664805`, \(-0.029337`\), 14.308957`, 14.311409`, 
              0.021445`, 14.988607`, 14.982244`, \(-0.055636`\), 5.020508`, 
              5.03465`, 0.061822`, 7.393786`, 7.398985`, 0.022727`, 
              8.861421`, 8.853683`, \(-0.033827`\), 8.953497`, 
              8.948518`, \(-0.021767`\), 8.788581`, 8.79531`, 0.029415`, 
              13.495326`, 13.491554`, \(-0.016488`\), 15.546982`, 
              15.539727`, \(-0.031718`\), 15.880191`, 15.900788`, 0.090045`, 
              8.788581`, 9.060718`, 1.189706`, 9.761787`, 
              9.500376`, \(-1.142815`\), 13.495326`, 13.514621`, 0.084354`, 
              15.546982`, 15.416998`, \(-0.568256`\), 7.305521`, 7.313814`, 
              0.036257`, 8.856781`, 8.837862`, \(-0.082711`\), 8.953497`, 
              8.967788`, 0.062477`, 12.802662`, 12.814058`, 0.049822`, 
              5.020508`, 7.393786`, 8.861421`, 12.744516`, 18.911626`, 
              19.669737`, 19.942527`, 18.361898`, 15.729046`, 16.515002`, 
              19.512362`, 20.435683`, 9.761787`, 15.729287`, 17.143664`, 
              20.582518`, 7.305521`, 8.856781`, 12.802662`, 14.440942`, 
              16.43161`, 20.056631`, 20.276975`, 17.150033`, 9.668161`, 
              16.744565`, 19.485824`, 20.736836`, 15.880191`, 18.343811`, 
              19.648711`, 20.171188`, 8.788581`, 13.495326`, 15.546982`, 
              17.588016`, 8.953497`, 14.308957`, 14.988607`, 16.253034`}, {2, 
              12, 182.6951451`, \(-9.528504676`\), 2.143134132`, 
              0.03249196962`, 1, 
              0.6768237831`, \(-0.06998201759`\), \(-0.1064815138`\), 
              245.5300598`, 0.1411662678`, 
              106.9875529`, \(-0.9009852566`\), \(-0.8218889087`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.224051`, \(-0.394418`\), \(-0.186941`\), 0.168834`, 
              0.051185`, \(-0.025036`\), 2.098257`, 1.374625`, 0.656033`, 
              0.285931`, 1.00619`, 0.801856`, 1.096595`, 0.275947`, 
              3.949076`, 3.956023`, 0.053371`, 6.000692`, 6.00414`, 0.02649`, 
              6.221194`, 6.221082`, \(-0.000861`\), 7.283016`, 
              7.274686`, \(-0.063995`\), 7.207523`, 7.209479`, 0.015031`, 
              11.085926`, 11.095057`, 0.070154`, 13.012876`, 
              13.000848`, \(-0.092404`\), 13.855348`, 13.861855`, 0.049992`, 
              7.207523`, 7.413894`, 1.585501`, 8.750601`, 
              8.549733`, \(-1.543223`\), 11.085926`, 
              11.085031`, \(-0.006875`\), 13.012876`, 
              12.971861`, \(-0.315107`\), 7.662204`, 7.666733`, 0.03479`, 
              8.297797`, 8.294224`, \(-0.027453`\), 11.763355`, 11.765938`, 
              0.019845`, 12.47229`, 12.467756`, \(-0.034838`\), 3.949076`, 
              3.963003`, 0.053497`, 6.221194`, 6.220753`, \(-0.001693`\), 
              7.285447`, 7.278605`, \(-0.026282`\), 7.662204`, 7.665072`, 
              0.011015`, 7.207523`, 7.212463`, 0.018977`, 11.085926`, 
              11.084024`, \(-0.007303`\), 13.012876`, 
              13.008788`, \(-0.015702`\), 13.511424`, 13.52567`, 0.054724`, 
              7.207523`, 7.434612`, 0.872336`, 8.750601`, 
              8.533607`, \(-0.833556`\), 11.085926`, 11.105076`, 0.073562`, 
              13.012876`, 12.918626`, \(-0.362047`\), 6.000692`, 6.007674`, 
              0.026819`, 7.283016`, 7.273126`, \(-0.037992`\), 7.662204`, 
              7.669487`, 0.027975`, 10.34796`, 10.360895`, 0.04969`, 
              3.949076`, 6.221194`, 7.285447`, 11.409171`, 15.629398`, 
              17.098287`, 17.349328`, 16.148049`, 13.327062`, 13.671665`, 
              16.502353`, 17.918271`, 8.750601`, 13.317873`, 14.79845`, 
              17.975204`, 6.000692`, 7.283016`, 10.34796`, 11.764782`, 
              13.855348`, 16.646914`, 17.931777`, 15.813869`, 8.297797`, 
              14.462848`, 16.786214`, 17.22439`, 13.511424`, 14.792587`, 
              16.950676`, 17.252814`, 7.207523`, 11.085926`, 13.012876`, 
              15.024649`, 7.662204`, 11.763355`, 12.47229`, 13.761143`}, {2, 
              30, 45.64654345`, \(-9.144992176`\), 6.583307193`, 
              0.0517766953`, 1, 0.9578897449`, 
              0.03425649655`, \(-0.3472168608`\), 2895.562018`, 
              0.1665341647`, 
              1397.532039`, \(-1.215021615`\), \(-1.708512441`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.208`, \(-0.947846`\), 
              0.700515`, 0.168986`, \(-0.088872`\), 0.218189`, 1.373494`, 
              0.803718`, 0.60408`, 0.517015`, 0.80005`, 0.823294`, 1.0994`, 
              0.680044`, 10.001546`, 10.048212`, 1.022413`, 17.97084`, 
              18.009193`, 0.840284`, 18.267475`, 18.304982`, 0.82176`, 
              27.746295`, 27.691983`, \(-1.189925`\), 22.770112`, 22.77945`, 
              0.204591`, 33.871182`, 33.913444`, 0.925929`, 41.28775`, 
              41.258902`, \(-0.63204`\), 45.660914`, 45.675621`, 0.322216`, 
              22.770112`, 22.821259`, 1.120606`, 28.788765`, 
              28.772241`, \(-0.36202`\), 33.871182`, 
              33.870433`, \(-0.016401`\), 41.28775`, 
              41.214289`, \(-1.609479`\), 27.746295`, 27.778722`, 0.710448`, 
              33.139484`, 33.126136`, \(-0.292433`\), 39.071094`, 39.082862`, 
              0.257825`, 41.474744`, 41.448973`, \(-0.564633`\), 10.001546`, 
              10.0951`, 1.024843`, 18.267475`, 18.338181`, 0.774556`, 
              27.746295`, 27.679508`, \(-0.731626`\), 31.184403`, 
              31.144431`, \(-0.437873`\), 22.770112`, 22.794168`, 0.263531`, 
              33.871182`, 33.863823`, \(-0.080611`\), 41.28775`, 
              41.281437`, \(-0.069159`\), 43.205538`, 43.291546`, 0.942191`, 
              22.770112`, 22.841915`, 0.786579`, 28.788765`, 
              28.781265`, \(-0.082152`\), 33.871182`, 33.9662`, 1.040889`, 
              41.28775`, 41.093175`, \(-2.131493`\), 17.97084`, 18.049818`, 
              0.865175`, 27.746295`, 27.769788`, 0.257356`, 30.684822`, 
              30.751652`, 0.732104`, 30.930692`, 30.906147`, \(-0.268886`\), 
              10.001546`, 18.267475`, 31.184403`, 39.52469`, 50.019972`, 
              54.243821`, 57.604641`, 55.560973`, 44.58144`, 50.652728`, 
              63.744626`, 64.040135`, 28.788765`, 45.73942`, 50.402403`, 
              57.606619`, 17.97084`, 30.930692`, 30.684822`, 36.471731`, 
              45.660914`, 53.34084`, 53.892058`, 57.520692`, 33.139484`, 
              49.138722`, 54.878518`, 60.077214`, 43.205538`, 47.637036`, 
              60.66584`, 57.262956`, 22.770112`, 33.871182`, 41.28775`, 
              51.754509`, 27.746295`, 39.071094`, 41.474744`, 
              46.924288`}, {2, 85, 42.35093819`, \(-9.411714392`\), 
              8.00777585`, 0.07642881742`, 1, 
              0.6239550522`, \(-0.1415750473`\), \(-0.1586469446`\), 
              572.1825645`, 0.01698361638`, 
              269.611929`, \(-1.668335646`\), \(-2.376032931`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.282685`, \(-0.237609`\), \(-0.643102`\), 
              0.142234`, \(-0.188426`\), 0.371906`, 3.005376`, 1.94348`, 
              0.606104`, 0.51862`, 0.683371`, 0.705421`, 1.239267`, 
              0.729748`, 21.481433`, 21.509792`, 1.027161`, 27.097336`, 
              27.121669`, 0.881291`, 27.173185`, 27.193494`, 0.735564`, 
              34.294231`, 34.293094`, \(-0.041151`\), 30.740724`, 30.748466`, 
              0.280391`, 51.089311`, 51.113404`, 0.872589`, 57.071895`, 
              57.054199`, \(-0.640934`\), 59.536196`, 59.548812`, 0.45695`, 
              30.740724`, 30.768921`, 1.02125`, 38.69174`, 
              38.679059`, \(-0.459265`\), 51.089311`, 
              51.082364`, \(-0.251625`\), 57.071895`, 
              57.041196`, \(-1.111877`\), 34.294231`, 34.308101`, 0.502379`, 
              37.180058`, 37.169374`, \(-0.386992`\), 53.350249`, 53.366028`, 
              0.571513`, 55.243495`, 55.238313`, \(-0.187663`\), 21.481433`, 
              21.538376`, 1.031211`, 27.173185`, 27.20808`, 0.631935`, 
              34.294231`, 34.264555`, \(-0.537415`\), 34.361675`, 
              34.309028`, \(-0.953413`\), 30.740724`, 30.758078`, 0.314273`, 
              51.089311`, 51.087881`, \(-0.025906`\), 57.071895`, 57.074251`, 
              0.042678`, 65.433594`, 65.524275`, 1.642189`, 30.740724`, 
              30.790864`, 0.907996`, 38.69174`, 38.671012`, \(-0.375365`\), 
              51.089311`, 51.126078`, 0.665823`, 57.071895`, 
              56.974874`, \(-1.756995`\), 27.097336`, 27.149281`, 0.940679`, 
              34.294231`, 34.291212`, \(-0.054669`\), 34.36161`, 
              34.338779`, \(-0.413451`\), 49.314667`, 49.408098`, 1.691968`, 
              21.481433`, 27.173185`, 34.361675`, 47.817988`, 69.312712`, 
              74.681092`, 80.583588`, 80.926375`, 59.222789`, 66.419063`, 
              75.879843`, 79.257193`, 38.69174`, 59.113525`, 68.649475`, 
              76.160661`, 27.097336`, 34.36161`, 49.314667`, 49.958563`, 
              59.536196`, 71.608117`, 81.472951`, 79.871801`, 37.180058`, 
              67.690083`, 74.174068`, 75.724885`, 65.433594`, 66.418069`, 
              78.200187`, 79.26629`, 30.740724`, 51.089311`, 57.071895`, 
              67.550016`, 34.294231`, 53.350249`, 55.243495`, 
              63.379479`}, {2, 12, 123.1219523`, \(-9.414094251`\), 
              5.087145263`, 0.1072401379`, 1, 
              1.087510466`, \(-0.02310432945`\), \(-0.1375635295`\), 
              174.0868912`, 0.234849293`, 
              46.82532111`, \(-1.078488251`\), \(-1.07425762`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.341449`, \(-0.541986`\), \(-0.296357`\), 
              0.196447`, \(-0.057497`\), 0.127347`, 3.109602`, 2.152558`, 
              0.734876`, 0.596327`, 1.012087`, 0.915955`, 1.281452`, 
              0.798638`, 14.930116`, 14.952219`, 0.614269`, 24.258644`, 
              24.268909`, 0.285285`, 24.768963`, 24.76442`, \(-0.126257`\), 
              28.205943`, 28.17294`, \(-0.917209`\), 20.819749`, 20.822668`, 
              0.08112`, 34.913869`, 34.937554`, 0.658243`, 42.255109`, 
              42.220722`, \(-0.955676`\), 45.392174`, 45.414715`, 0.626444`, 
              20.819749`, 20.809216`, \(-0.29273`\), 31.332366`, 31.360373`, 
              0.778379`, 34.913869`, 34.91206`, \(-0.050268`\), 42.255109`, 
              42.189806`, \(-1.814888`\), 29.000564`, 29.016931`, 0.454878`, 
              31.123217`, 31.11011`, \(-0.364277`\), 38.290168`, 38.301746`, 
              0.321791`, 41.663242`, 41.650784`, \(-0.346226`\), 14.930116`, 
              14.974559`, 0.617573`, 24.768963`, 24.758822`, \(-0.140913`\), 
              28.257505`, 28.243006`, \(-0.201482`\), 29.000564`, 29.023294`, 
              0.31586`, 20.819749`, 20.828478`, 0.121308`, 34.913869`, 
              34.91082`, \(-0.042369`\), 42.255109`, 
              42.246593`, \(-0.11834`\), 44.390664`, 
              44.336962`, \(-0.746252`\), 20.819749`, 
              20.797131`, \(-0.314292`\), 31.332366`, 31.387387`, 0.764568`, 
              34.913869`, 34.962331`, 0.673427`, 42.255109`, 
              42.071246`, \(-2.554962`\), 24.258644`, 24.279687`, 0.292414`, 
              28.205943`, 28.148467`, \(-0.798691`\), 29.000564`, 29.053803`, 
              0.739816`, 30.608476`, 30.606823`, \(-0.022971`\), 14.930116`, 
              24.768963`, 28.257505`, 31.858564`, 48.448747`, 56.004683`, 
              56.036422`, 61.32199`, 45.079232`, 49.391306`, 62.080803`, 
              62.804549`, 31.332366`, 44.586736`, 54.55291`, 54.110091`, 
              24.258644`, 28.205943`, 30.608476`, 36.919951`, 45.392174`, 
              51.110252`, 58.322058`, 58.82878`, 31.123217`, 52.254941`, 
              52.614782`, 55.521441`, 44.390664`, 49.405559`, 53.342025`, 
              61.561725`, 20.819749`, 34.913869`, 42.255109`, 50.135164`, 
              29.000564`, 38.290168`, 41.663242`, 49.927181`}, {1, 49, 
              98.86447681`, \(-9.196356516`\), 5.347649532`, 0.1453085056`, 
              1, 0.6946309105`, \(-0.2074597729`\), \(-0.2404269963`\), 
              25.22459288`, 0.00498530267`, 
              7.237598566`, \(-0.9155508478`\), \(-1.646686583`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.374393`, \(-0.174928`\), \(-0.874965`\), 
              0.201327`, \(-0.035632`\), 0.090739`, 4.531741`, 2.710846`, 
              0.759737`, 0.674205`, 0.937828`, 0.960706`, 1.439988`, 
              0.915606`, 21.205271`, 21.232778`, 0.881176`, 23.471464`, 
              23.500202`, 0.920582`, 24.006548`, 24.021974`, 0.494143`, 
              30.977182`, 30.937391`, \(-1.274679`\), 24.205477`, 24.210136`, 
              0.149252`, 43.910857`, 43.92513`, 0.457219`, 47.671913`, 
              47.667958`, \(-0.126693`\), 49.699414`, 49.707901`, 0.271879`, 
              24.205477`, 24.199708`, \(-0.184802`\), 35.066915`, 35.081103`, 
              0.454499`, 43.910857`, 43.91369`, 0.090763`, 47.671913`, 
              47.665505`, \(-0.205268`\), 30.977182`, 30.986337`, 0.293274`, 
              33.669241`, 33.662266`, \(-0.223419`\), 45.505288`, 45.519931`, 
              0.469069`, 46.503503`, 46.502682`, \(-0.026316`\), 21.205271`, 
              21.260565`, 0.88565`, 24.006548`, 24.03847`, 0.51129`, 
              30.977182`, 30.91609`, \(-0.97851`\), 31.241909`, 
              31.212789`, \(-0.466415`\), 24.205477`, 24.214304`, 0.141378`, 
              43.910857`, 43.91662`, 0.092313`, 47.671913`, 47.678611`, 
              0.107288`, 54.789408`, 54.832289`, 0.686816`, 24.205477`, 
              24.194958`, \(-0.168486`\), 35.066915`, 35.094942`, 0.44891`, 
              43.910857`, 43.939593`, 0.460268`, 47.671913`, 
              47.645`, \(-0.431064`\), 23.471464`, 23.527422`, 0.896267`, 
              30.977182`, 30.977075`, \(-0.001715`\), 31.241502`, 
              31.213507`, \(-0.448398`\), 33.053077`, 33.125814`, 1.165025`, 
              21.205271`, 24.006548`, 31.241909`, 36.80969`, 57.307026`, 
              60.489969`, 66.274758`, 68.895131`, 50.203222`, 61.660742`, 
              68.126756`, 70.438919`, 35.066915`, 50.52299`, 63.729879`, 
              62.509047`, 23.471464`, 31.241502`, 33.053077`, 43.685871`, 
              49.699414`, 58.490801`, 67.017132`, 68.350964`, 33.669241`, 
              62.103059`, 62.967983`, 63.197147`, 54.789408`, 62.257383`, 
              64.414445`, 66.822346`, 24.205477`, 43.910857`, 47.671913`, 
              56.222194`, 30.977182`, 45.505288`, 46.503503`, 
              53.604842`}, {2, 75, 108.3790739`, \(-8.886925453`\), 
              4.528365995`, 0.1921681542`, 1, 0.7972161339`, 
              0.07225360472`, \(-0.3292082263`\), 37.64374935`, 
              0.1917739707`, 
              11.15794503`, \(-0.4604891759`\), \(-1.470994475`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.429226`, 
              0.01329`, \(-1.223817`\), 0.234159`, \(-0.038862`\), 0.056752`, 
              6.040366`, 3.294118`, 0.831632`, 0.7017`, 1.025858`, 1.024706`, 
              1.587724`, 0.957115`, 26.369952`, 26.39977`, 0.927296`, 
              30.61292`, 30.64391`, 0.963757`, 30.639428`, 30.654979`, 
              0.483606`, 31.321071`, 31.330465`, 0.29212`, 22.504828`, 
              22.508067`, 0.100732`, 45.097267`, 45.118101`, 0.647931`, 
              49.137656`, 49.20586`, 2.121063`, 49.228081`, 49.252711`, 
              0.765952`, 22.504828`, 22.499859`, \(-0.154528`\), 38.06589`, 
              38.076945`, 0.343792`, 45.097267`, 45.109697`, 0.386571`, 
              49.137656`, 49.120702`, \(-0.527252`\), 37.709954`, 37.717195`, 
              0.225204`, 40.611229`, 40.605944`, \(-0.164385`\), 48.114298`, 
              48.127633`, 0.414724`, 50.126995`, 50.124504`, \(-0.077454`\), 
              26.369952`, 26.429842`, 0.931262`, 31.321071`, 31.340096`, 
              0.295818`, 32.54931`, 32.52616`, \(-0.359976`\), 37.709954`, 
              37.65685`, \(-0.825733`\), 22.504828`, 22.50911`, 0.06659`, 
              45.097267`, 45.111146`, 0.215814`, 49.137656`, 49.139969`, 
              0.035965`, 49.228081`, 49.223293`, \(-0.074448`\), 22.504828`, 
              22.49753`, \(-0.113474`\), 38.06589`, 38.08781`, 0.340853`, 
              45.097267`, 45.150232`, 0.823588`, 49.137656`, 49.15367`, 
              0.248997`, 30.61292`, 30.670202`, 0.890706`, 30.639428`, 
              30.675032`, 0.553625`, 37.709954`, 37.697585`, \(-0.192333`\), 
              39.796154`, 39.779086`, \(-0.26541`\), 26.369952`, 31.321071`, 
              32.54931`, 39.76699`, 56.896833`, 62.794595`, 63.606188`, 
              68.627898`, 55.085188`, 68.517443`, 70.141588`, 69.165278`, 
              38.06589`, 53.781473`, 63.814687`, 69.098959`, 30.61292`, 
              30.639428`, 39.796154`, 47.355586`, 52.175154`, 58.645712`, 
              65.885481`, 66.525581`, 40.611229`, 62.335375`, 66.049909`, 
              72.736996`, 55.756347`, 63.407846`, 66.785437`, 67.446273`, 
              22.504828`, 45.097267`, 49.228081`, 49.137656`, 37.709954`, 
              48.114298`, 50.126995`, 56.187361`}, {2, 11, 
              242.3567924`, \(-9.976383834`\), 2.173030567`, 0.249999`, 1, 
              1.793042746`, 0.498668268`, \(-0.2733701498`\), 7.106598741`, 
              0.3982458139`, 
              1.190444993`, \(-0.1557649057`\), \(-0.371849808`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.546196`, \(-1.051941`\), 
              0.156837`, 0.243773`, \(-0.04321`\), 0.053144`, 4.955383`, 
              3.337233`, 0.911258`, 0.42208`, 1.238032`, 0.829481`, 
              1.573396`, 0.533845`, 10.82649`, 10.840349`, 0.263173`, 
              11.068001`, 11.082665`, 0.278484`, 15.333425`, 
              15.327502`, \(-0.112463`\), 20.730785`, 20.771652`, 0.776081`, 
              11.502708`, 11.503455`, 0.014179`, 22.866966`, 22.875049`, 
              0.153514`, 23.796981`, 23.793962`, \(-0.057337`\), 28.205064`, 
              28.195025`, \(-0.190642`\), 11.502708`, 
              11.492581`, \(-0.19232`\), 21.33911`, 21.401332`, 1.181633`, 
              22.866966`, 22.816988`, \(-0.949088`\), 23.796981`, 
              23.775664`, \(-0.404824`\), 23.659518`, 23.669325`, 0.186231`, 
              25.07225`, 25.062707`, \(-0.181232`\), 25.986664`, 25.997301`, 
              0.201988`, 29.327283`, 29.732302`, 7.691468`, 10.82649`, 
              10.853954`, 0.260772`, 15.333425`, 15.321452`, \(-0.113677`\), 
              21.461642`, 21.428008`, \(-0.319364`\), 23.659518`, 23.676771`, 
              0.16382`, 11.502708`, 11.505086`, 0.022582`, 22.866966`, 
              22.860402`, \(-0.062322`\), 23.796981`, 
              23.795013`, \(-0.018693`\), 28.091526`, 
              27.748438`, \(-3.257691`\), 11.502708`, 
              11.482407`, \(-0.192762`\), 21.33911`, 21.473519`, 1.276241`, 
              22.866966`, 22.780176`, \(-0.824088`\), 23.796981`, 
              23.753828`, \(-0.409752`\), 11.068001`, 11.099419`, 0.298323`, 
              20.730785`, 20.811086`, 0.762474`, 21.132129`, 21.145122`, 
              0.123372`, 23.659518`, 23.658285`, \(-0.01171`\), 10.82649`, 
              15.333425`, 21.461642`, 28.041167`, 31.743108`, 34.698308`, 
              35.901095`, 38.347717`, 32.7314`, 38.089727`, 37.070315`, 
              29.327283`, 21.33911`, 30.771042`, 34.367583`, 37.325154`, 
              11.068001`, 20.730785`, 21.132129`, 25.276864`, 30.333338`, 
              33.296717`, 34.792299`, 35.985348`, 25.07225`, 30.126966`, 
              37.902992`, 33.295023`, 28.091526`, 31.716652`, 37.072656`, 
              38.403605`, 11.502708`, 22.866966`, 23.796981`, 28.205064`, 
              23.659518`, 25.986664`, 29.738725`, 34.420121`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data three/ytraj_1_2?.out; periodic BC's", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for the 3+1 transverse lattice \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\nUse \
chi^2 fit with 40 criteria and tolerance 0.01.\nOverall scale from minimizing \
chi^2.\nParity doublets (charge,multi,state) with error for difference\n  \
over average.\n  ( 1, 5,0) and ( 1, 6,0), error 0.25\n  (-1, 5,0) and (-1, \
6,0), error 0.25\n  (-1, 7,0) and (-1, 4,0), error 0.25\nSpectrum for P_perp \
a = (0,0), ( 0.25 0)\n  using 4 states (multiplet, charge, c^2 error for \
each) =\n  (15,  1, 0.25 2 2 2)\n  (15, -1, 0.25 2 2 2)\n  (16, -1, 0.25 2 2 \
2)\n  (16,  1, 0.25 2 2 2)\nSpectrum for P_perp a = (0,0), ( 0.25 0.25)\n  \
using 4 states (multiplet, charge, c^2 error for each) =\n  (17,  1, 0.25 2 2 \
2)\n  (17, -1, 0.25 2 2 2)\n  (18, -1, 0.25 2 2 2)\n  (18,  1, 0.25 2 2 2).\n\
Ordinary spectra multiplets, 4 states, (charge,multiplet) = ( 1,  3) \n  (-1, \
 3)  ( 1,  4)  (-1,  4)  ( 1,  5)  (-1,  5)  ( 1,  6) \n  (-1,  6)  (-1,  7)  \
( 1,  7) .\nAll spectra extrapolated using (K,p) = (12/2,8)  (12/2,6) \n  \
(14/2,6)  (18/2,6) .\nWinding potential using (n,K,p) = ( 2 0,16/2,6)  ( 2 \
0,16/2,4) \n  ( 2 0,26/2,4)  ( 3 0,18/2,7)  ( 3 0,18/2,5)  ( 3 0,26/2,5) \n  \
( 4 0,16/2,8)  ( 4 0,16/2,6)  ( 4 0,26/2,6) .\nRoundness of winding using \
(n,K,p) = ( 2 2,14/2,8) \n  ( 2 2,14/2,6)  ( 2 2,22/2,6) \n  with error 1; in \
G^2 N units.\nHeavy potential determined using (n,K,p,K_max) =\n  ( 0 \
0,26/2,2,3.5)  ( 0 0,26/2,4,3.5)  ( 0 0,26/2,2,3) \n  ( 0 0,42/2,2,3.5)  ( 0 \
0,42/2,2,4.25)  ( 0 0,70/2,2,4) \n  ( 0 0,70/2,2,4.5) ,\n L = 3.577-2.726*m \
5.058-3.856*m 7.154-5.452*m\n (all in G^2 N units); relative scale error=0.1.\
\nRoundness determined using (n,K,p,K_max) = ( 1 0,14/2,3,3.5) \n  ( 1 \
0,14/2,5,3.5)  ( 1 0,14/2,3,3)  ( 1 0,24/2,3,4) \n  ( 1 0,24/2,3,4.5)  ( 1 \
0,36/2,3,5)  ( 1 0,36/2,3,6) \n L=1.265-0.964*m and error 0.1;\n  ( 1 \
0,14/2,3,3.5)  ( 1 0,14/2,5,3.5)  ( 1 0,14/2,3,3) \n  ( 1 0,24/2,3,4)  ( 1 \
0,24/2,3,4.5)  ( 1 0,36/2,3,5) \n  ( 1 0,36/2,3,6) \n L=5.058-3.856*m and \
error 0.1;\n  ( 1 1,22/2,2,5)  ( 1 1,22/2,4,5)  ( 1 1,22/2,2,4.5) \n  ( 1 \
1,44/2,2,5)  ( 1 1,44/2,2,6)  ( 1 1,70/2,2,6) \n  ( 1 1,70/2,2,7) \n L=0+0*m \
and error 0.2; all in G^2 N units.\np-extrapolation using n=( 1 0) and (K,p) \
= (14/2,3) \n  (22/2,3)  (46/2,3)  (14/2,5)  (28/2,5)  (14/2,7) \n  (16/2,7) \
.\nResult format:\n Fit info, # steps, chi^2, and p damping, scale G^2 \
N/sigma.\n The 17 couplings  (G^2 N units);\n   which couplings -- if any -- \
were fit.\n Winding potential and heavy souce potential fits.\n Roundness of \
winding potential, 1 values, and roundness of\n   heavy source potential, 3 \
values, (G^2 N units)\n   showing measured value and derived value for each.\n\
 The rescaled spectrum for each P_perp*a and c^2 values.\n Finally come the \
states for the ordinary spectra.\>", {{2, 20, 
              104.2882809`, \(-7.773123274`\), 12.64585014`, 0, 
              1, \(-0.1855392676`\), 0.4867288493`, \(-0.2218701369`\), 
              16391.09856`, \(-0.1544536648`\), 
              5838.981137`, \(-2.016321431`\), \(-2.174321371`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.141553`, \(-0.350119`\), 
              0.093728`, 0.10158`, \(-0.227256`\), 0.420931`, 1.578076`, 
              0.794022`, 0.481882`, 0.358337`, 0.652735`, 0.538743`, 
              1.128351`, 0.518793`, 25.857655`, 25.873629`, 0.457498`, 
              41.907303`, 41.912`, 0.134528`, 49.01444`, 49.027594`, 
              0.376741`, 52.356319`, 52.358733`, 0.06914`, 37.876668`, 
              37.893226`, 0.474243`, 58.471238`, 58.477214`, 0.171154`, 
              74.212171`, 74.215707`, 0.101279`, 75.292114`, 
              75.291271`, \(-0.024157`\), 27.946858`, 
              27.944902`, \(-0.056037`\), 37.876668`, 37.884607`, 0.227381`, 
              58.471238`, 58.441536`, \(-0.850712`\), 63.376092`, 63.390836`, 
              0.422289`, 52.356319`, 52.358163`, 0.052799`, 57.358524`, 
              57.359878`, 0.038769`, 59.028329`, 59.029978`, 0.047242`, 
              76.070836`, 76.079306`, 0.242596`, 25.857655`, 25.88956`, 
              0.456893`, 41.907303`, 41.917056`, 0.139665`, 52.356319`, 
              52.371806`, 0.221781`, 55.038172`, 55.032136`, \(-0.08643`\), 
              37.876668`, 37.907876`, 0.44692`, 58.471238`, 
              58.449706`, \(-0.308357`\), 63.376092`, 63.39756`, 0.307442`, 
              74.212171`, 74.227273`, 0.216274`, 27.946858`, 
              27.942873`, \(-0.057068`\), 37.876668`, 37.894397`, 0.253898`, 
              58.471238`, 58.454873`, \(-0.234359`\), 63.525737`, 63.539762`, 
              0.200846`, 49.01444`, 49.040398`, 0.371722`, 52.356319`, 
              52.350065`, \(-0.089569`\), 59.028329`, 
              58.983805`, \(-0.637598`\), 63.272653`, 63.299382`, 0.382772`, 
              25.857655`, 41.907303`, 55.038172`, 63.449251`, 83.691205`, 
              86.937851`, 92.846154`, 78.725995`, 81.555789`, 88.767332`, 
              95.746498`, 104.422883`, 27.946858`, 63.525737`, 77.496334`, 
              78.906029`, 49.01444`, 63.272653`, 71.473745`, 72.272115`, 
              75.292114`, 80.515097`, 89.519461`, 88.020922`, 57.358524`, 
              77.823663`, 97.992202`, 102.485192`, 63.376092`, 84.754996`, 
              81.174466`, 93.236578`, 37.876668`, 58.471238`, 74.212171`, 
              78.637859`, 52.356319`, 59.028329`, 76.070836`, 
              80.916302`}, {2, 12, 106.5946382`, \(-0.25`\), 9.338354732`, 
              0.001967542671`, 1, 0.05128983526`, 
              0.08588843657`, \(-0.1282959758`\), 
              1570.962466`, \(-0.04986973483`\), 
              746.0818937`, \(-1.171488692`\), \(-1.458193253`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.165701`, \(-0.044264`\), \(-0.378639`\), 0.134722`, 
              0.013724`, 0.043809`, 2.605082`, 1.234011`, 0.56265`, 
              0.562636`, 0.84197`, 0.854906`, 1.096217`, 0.74612`, 
              24.622056`, 24.634855`, 0.316886`, 26.384326`, 
              26.383624`, \(-0.017383`\), 26.945771`, 26.958056`, 0.304145`, 
              29.344045`, 29.330706`, \(-0.330251`\), 28.81674`, 28.829165`, 
              0.307617`, 48.969547`, 48.972267`, 0.067333`, 51.249363`, 
              51.249013`, \(-0.00867`\), 52.715338`, 52.715978`, 0.01583`, 
              26.03884`, 26.038444`, \(-0.009805`\), 28.81674`, 28.817369`, 
              0.015569`, 48.969547`, 48.968942`, \(-0.014992`\), 51.249363`, 
              51.246742`, \(-0.064896`\), 29.799013`, 29.799134`, 0.002984`, 
              32.059245`, 32.059095`, \(-0.003717`\), 49.140287`, 49.140745`, 
              0.011322`, 51.36118`, 51.360753`, \(-0.010569`\), 24.622056`, 
              24.647697`, 0.317415`, 26.384326`, 26.383301`, \(-0.012692`\), 
              29.344045`, 29.318439`, \(-0.316981`\), 29.799013`, 29.824054`, 
              0.309983`, 28.81674`, 28.837247`, 0.253855`, 48.969547`, 
              48.969964`, 0.005162`, 51.249363`, 51.250728`, 0.016905`, 
              53.277414`, 53.27093`, \(-0.080267`\), 26.03884`, 
              26.038062`, \(-0.009635`\), 28.81674`, 28.822278`, 0.068562`, 
              48.969547`, 48.973302`, 0.046486`, 51.249363`, 
              51.242028`, \(-0.090798`\), 26.945771`, 26.969756`, 0.296906`, 
              29.799013`, 29.796757`, \(-0.027929`\), 34.817236`, 
              34.804116`, \(-0.162421`\), 49.140287`, 
              49.133389`, \(-0.085398`\), 24.622056`, 26.384326`, 29.344045`, 
              34.959514`, 59.238982`, 64.135798`, 68.001871`, 63.330645`, 
              55.011921`, 58.803114`, 60.880132`, 63.007471`, 26.03884`, 
              52.739668`, 55.027261`, 57.075832`, 26.945771`, 34.817236`, 
              50.228414`, 55.193923`, 52.715338`, 58.219056`, 65.269486`, 
              65.92971`, 32.059245`, 57.683701`, 61.178954`, 62.569946`, 
              55.222024`, 59.134365`, 62.896853`, 65.90236`, 28.81674`, 
              48.969547`, 51.249363`, 53.277414`, 29.799013`, 49.140287`, 
              51.36118`, 53.333664`}, {1, 67, 
              84.18575565`, \(-9.762227907`\), 9.740554961`, 0.007919222016`, 
              1, 0.1150469766`, 0.187011881`, \(-0.311648554`\), 
              9518.799799`, \(-0.1896301303`\), 
              3765.928167`, \(-1.487075386`\), \(-2.369580444`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.137313`, \(-0.399544`\), 
              0.307499`, 0.122261`, \(-0.164481`\), 0.354114`, 1.806527`, 
              0.737397`, 0.486953`, 0.393023`, 0.590882`, 0.621121`, 
              1.06418`, 0.528974`, 16.834422`, 16.880943`, 0.995552`, 
              23.253257`, 23.244894`, \(-0.17896`\), 25.956539`, 25.984364`, 
              0.595463`, 32.463481`, 32.446639`, \(-0.360423`\), 29.907426`, 
              29.929116`, 0.464183`, 46.928616`, 46.936007`, 0.15817`, 
              51.541852`, 51.55121`, 0.200255`, 53.179658`, 
              53.164806`, \(-0.31783`\), 22.559004`, 
              22.558705`, \(-0.006382`\), 29.907426`, 29.909912`, 0.053216`, 
              46.928616`, 46.912459`, \(-0.345772`\), 49.459133`, 49.466989`, 
              0.168124`, 32.463481`, 32.464499`, 0.02178`, 38.133011`, 
              38.133463`, 0.009668`, 47.314719`, 47.314434`, \(-0.006101`\), 
              52.111298`, 52.114993`, 0.079072`, 16.834422`, 16.927573`, 
              0.996722`, 23.253257`, 23.2372`, \(-0.171814`\), 32.463481`, 
              32.441133`, \(-0.239134`\), 33.879803`, 33.894608`, 0.158414`, 
              29.907426`, 29.94461`, 0.397872`, 46.928616`, 
              46.918666`, \(-0.106465`\), 50.103954`, 50.11492`, 0.117342`, 
              51.541852`, 51.563407`, 0.230642`, 22.559004`, 
              22.558379`, \(-0.006678`\), 29.907426`, 29.918364`, 0.117042`, 
              46.928616`, 46.923816`, \(-0.051368`\), 49.459133`, 49.473574`, 
              0.154522`, 25.956539`, 26.011275`, 0.585677`, 32.463481`, 
              32.455388`, \(-0.086598`\), 44.922105`, 
              44.89644`, \(-0.27462`\), 47.314719`, 
              47.253677`, \(-0.653157`\), 16.834422`, 23.253257`, 33.879803`, 
              45.503198`, 60.22656`, 63.552317`, 64.860064`, 56.135731`, 
              58.799886`, 60.534548`, 65.205737`, 69.993487`, 22.559004`, 
              49.459133`, 52.868278`, 56.073887`, 25.956539`, 44.922105`, 
              48.142168`, 50.136646`, 53.179658`, 56.448027`, 63.751889`, 
              57.811014`, 38.133011`, 55.254581`, 63.57813`, 69.028365`, 
              50.103954`, 60.394376`, 65.714368`, 72.841781`, 29.907426`, 
              46.928616`, 51.541852`, 54.582363`, 32.463481`, 47.314719`, 
              52.111298`, 54.394244`}, {2, 12, 
              142.0316076`, \(-9.575588235`\), 3.776583788`, 0.01800590693`, 
              1, 0.5092963519`, \(-0.0353246424`\), \(-0.1443702018`\), 
              1570.962466`, 0.06676113425`, 
              746.0818937`, \(-1.268005047`\), \(-1.411868608`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.190814`, \(-0.305384`\), \(-0.239806`\), 
              0.141323`, \(-0.047473`\), 0.160165`, 1.941755`, 1.191153`, 
              0.588093`, 0.31554`, 0.848158`, 0.668412`, 1.065405`, 0.35089`, 
              7.050855`, 7.062297`, 0.131924`, 9.608441`, 9.616095`, 
              0.088244`, 9.662917`, 9.671243`, 0.096002`, 12.280786`, 
              12.265858`, \(-0.172114`\), 12.144806`, 12.149452`, 0.05357`, 
              18.71564`, 18.729722`, 0.162359`, 21.254846`, 
              21.240216`, \(-0.168689`\), 22.439088`, 22.446032`, 0.080064`, 
              12.144806`, 12.272391`, 1.471054`, 13.218244`, 
              13.098147`, \(-1.384716`\), 18.71564`, 
              18.713163`, \(-0.028567`\), 21.254846`, 
              21.202034`, \(-0.608924`\), 12.280786`, 12.286607`, 0.067123`, 
              13.513178`, 13.509146`, \(-0.046498`\), 19.906146`, 19.910427`, 
              0.049354`, 20.60498`, 20.597221`, \(-0.089456`\), 7.050855`, 
              7.073797`, 0.132257`, 9.662917`, 9.678077`, 0.087396`, 
              12.280786`, 12.258936`, \(-0.125961`\), 12.607988`, 
              12.595485`, \(-0.072083`\), 12.144806`, 12.154799`, 0.057611`, 
              18.71564`, 18.712078`, \(-0.020534`\), 21.254846`, 
              21.249108`, \(-0.03308`\), 21.952827`, 21.978062`, 0.145483`, 
              12.144806`, 12.375438`, 1.329598`, 13.218244`, 
              13.00204`, \(-1.246415`\), 18.71564`, 18.743776`, 0.162201`, 
              21.254846`, 21.133768`, \(-0.698017`\), 9.608441`, 9.625083`, 
              0.095938`, 12.280786`, 12.28455`, 0.021705`, 12.601366`, 
              12.587615`, \(-0.079271`\), 17.92193`, 17.938586`, 0.096023`, 
              7.050855`, 9.662917`, 12.607988`, 17.00395`, 26.032014`, 
              26.870288`, 27.217529`, 24.843296`, 21.910712`, 22.988518`, 
              27.11336`, 27.851651`, 13.218244`, 21.765781`, 23.53735`, 
              27.944093`, 9.608441`, 12.601366`, 17.92193`, 19.783557`, 
              22.439088`, 27.399967`, 27.505294`, 23.017908`, 13.513178`, 
              23.022709`, 26.698779`, 28.528882`, 21.952827`, 25.583689`, 
              27.013239`, 28.37415`, 12.144806`, 18.71564`, 21.254846`, 
              24.04609`, 12.280786`, 19.906146`, 20.60498`, 22.308944`}, {2, 
              12, 196.3066789`, \(-9.533503455`\), 1.672469174`, 
              0.03249196962`, 1, 
              0.6751164286`, \(-0.07375746079`\), \(-0.157741533`\), 
              1570.962466`, 0.1076113598`, 
              746.0818937`, \(-1.268982104`\), \(-1.410344977`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.213562`, \(-0.475326`\), \(-0.052082`\), 
              0.151422`, \(-0.080847`\), 0.217556`, 1.90343`, 1.226659`, 
              0.60773`, 0.28433`, 0.858063`, 0.639182`, 1.087565`, 0.283701`, 
              2.898004`, 2.904159`, 0.035171`, 4.32123`, 4.325531`, 
              0.024582`, 4.392443`, 4.396167`, 0.021279`, 5.774958`, 
              5.763067`, \(-0.067952`\), 5.595266`, 5.597146`, 0.010744`, 
              8.50031`, 8.507725`, 0.042375`, 9.915264`, 
              9.907351`, \(-0.04522`\), 10.578749`, 10.582926`, 0.023872`, 
              5.595266`, 5.865846`, 1.546312`, 6.60033`, 
              6.33457`, \(-1.518772`\), 8.50031`, 8.499281`, \(-0.005877`\), 
              9.915264`, 9.890705`, \(-0.140352`\), 5.826448`, 5.830516`, 
              0.023247`, 6.484629`, 6.481762`, \(-0.016387`\), 9.125232`, 
              9.127431`, 0.012565`, 9.550861`, 9.546887`, \(-0.022708`\), 
              2.898004`, 2.910345`, 0.035264`, 4.392443`, 4.399545`, 
              0.020292`, 5.777899`, 5.769856`, \(-0.022981`\), 5.826448`, 
              5.821419`, \(-0.014371`\), 5.595266`, 5.59989`, 0.013211`, 
              8.50031`, 8.498457`, \(-0.005294`\), 9.915264`, 
              9.91245`, \(-0.008039`\), 10.282831`, 10.296069`, 0.037828`, 
              5.595266`, 5.84958`, 0.726676`, 6.60033`, 
              6.354883`, \(-0.701344`\), 8.50031`, 8.51562`, 0.043748`, 
              9.915264`, 9.856211`, \(-0.168738`\), 4.32123`, 4.330057`, 
              0.025222`, 5.774958`, 5.759892`, \(-0.043048`\), 5.826448`, 
              5.837322`, 0.031071`, 7.860307`, 7.870853`, 0.030135`, 
              2.898004`, 4.392443`, 5.777899`, 8.673673`, 12.074924`, 
              12.854888`, 13.205278`, 12.207176`, 10.25556`, 10.666114`, 
              12.896543`, 13.64629`, 6.60033`, 10.248831`, 11.284238`, 
              13.719762`, 4.32123`, 5.774958`, 7.860307`, 8.942925`, 
              10.578749`, 12.845135`, 13.287688`, 11.832535`, 6.484629`, 
              10.942205`, 12.898007`, 13.462772`, 10.282831`, 11.510473`, 
              13.064703`, 13.230093`, 5.595266`, 8.50031`, 9.915264`, 
              11.412502`, 5.826448`, 9.125232`, 9.550861`, 10.459202`}, {2, 
              48, 46.71315582`, \(-9.048213093`\), 6.79882144`, 
              0.0517766953`, 1, 
              0.9255540976`, \(-0.06069435614`\), \(-0.2698300162`\), 
              5185.455993`, 0.1899764074`, 
              2161.887365`, \(-1.28538792`\), \(-1.791512503`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.222691`, \(-0.843632`\), 
              0.55516`, 0.163657`, \(-0.104597`\), 0.24695`, 1.693483`, 
              1.00729`, 0.599651`, 0.520839`, 0.781115`, 0.804705`, 
              1.114853`, 0.692393`, 10.209193`, 10.252325`, 1.044859`, 
              17.499417`, 17.527644`, 0.683791`, 18.365852`, 18.389459`, 
              0.571862`, 27.128062`, 27.145058`, 0.411732`, 23.728702`, 
              23.737276`, 0.207723`, 35.238826`, 35.280338`, 1.005603`, 
              42.807173`, 42.770095`, \(-0.89822`\), 46.839998`, 46.859647`, 
              0.475993`, 23.728702`, 23.773228`, 1.078625`, 30.874809`, 
              30.860294`, \(-0.351636`\), 35.238826`, 35.24315`, 0.104749`, 
              42.807173`, 42.727606`, \(-1.927479`\), 27.128062`, 27.158145`, 
              0.728763`, 31.74785`, 31.730404`, \(-0.422617`\), 39.427341`, 
              39.438906`, 0.280162`, 41.915912`, 41.897387`, \(-0.448752`\), 
              10.209193`, 10.295613`, 1.046744`, 18.365852`, 18.411632`, 
              0.554489`, 27.128062`, 27.10757`, \(-0.248202`\), 27.906989`, 
              27.853115`, \(-0.652533`\), 23.728702`, 23.750182`, 0.260173`, 
              35.238826`, 35.236386`, \(-0.029553`\), 42.807173`, 
              42.799232`, \(-0.096184`\), 45.045658`, 45.116259`, 0.855142`, 
              23.728702`, 23.798641`, 0.847132`, 30.874809`, 
              30.860986`, \(-0.167429`\), 35.238826`, 35.337002`, 1.189136`, 
              42.807173`, 42.586645`, \(-2.67111`\), 17.499417`, 17.557346`, 
              0.70165`, 27.128062`, 27.181572`, 0.648138`, 27.830543`, 
              27.758108`, \(-0.877348`\), 30.858613`, 30.934449`, 0.918554`, 
              10.209193`, 18.365852`, 27.906989`, 40.799255`, 50.965485`, 
              54.808472`, 59.640791`, 57.494653`, 45.460576`, 48.887058`, 
              61.397341`, 64.953297`, 30.874809`, 46.521983`, 51.761246`, 
              59.792855`, 17.499417`, 27.830543`, 30.858613`, 37.140464`, 
              46.839998`, 54.403218`, 55.780991`, 59.822692`, 31.74785`, 
              50.116204`, 57.873265`, 59.285953`, 45.045658`, 48.358335`, 
              61.033223`, 57.736244`, 23.728702`, 35.238826`, 42.807173`, 
              51.779762`, 27.128062`, 39.427341`, 41.915912`, 
              46.988865`}, {2, 57, 39.69398236`, \(-9.180373479`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 2.430894`, 1.680288`, 
              0.611141`, 0.528937`, 0.71216`, 0.733464`, 1.201923`, 
              0.734706`, 15.880884`, 15.914083`, 1.126294`, 22.903951`, 
              22.929273`, 0.859073`, 22.963928`, 22.988224`, 0.824247`, 
              30.912199`, 30.946653`, 1.16886`, 28.683755`, 28.69176`, 
              0.271583`, 45.567465`, 45.594481`, 0.916504`, 52.27398`, 
              52.254324`, \(-0.666836`\), 55.087284`, 55.102457`, 0.514725`, 
              28.683755`, 28.712611`, 0.978958`, 35.931338`, 
              35.925961`, \(-0.182424`\), 45.567465`, 
              45.554409`, \(-0.442923`\), 52.27398`, 
              52.230872`, \(-1.462461`\), 30.912199`, 30.932793`, 0.698634`, 
              34.342464`, 34.327499`, \(-0.507695`\), 48.251991`, 48.268106`, 
              0.546683`, 50.491023`, 50.478944`, \(-0.409787`\), 15.880884`, 
              15.947544`, 1.13073`, 22.963928`, 23.003966`, 0.679149`, 
              30.912199`, 30.868313`, \(-0.744429`\), 30.976108`, 
              30.940954`, \(-0.596306`\), 28.683755`, 28.704935`, 0.359266`, 
              45.567465`, 45.559264`, \(-0.139108`\), 52.27398`, 
              52.273502`, \(-0.008117`\), 57.990367`, 58.038021`, 0.808336`, 
              28.683755`, 28.727813`, 0.747345`, 35.931338`, 
              35.929399`, \(-0.032895`\), 45.567465`, 45.605335`, 0.642362`, 
              52.27398`, 52.151894`, \(-2.070909`\), 22.903951`, 22.955543`, 
              0.875134`, 30.912199`, 30.912751`, 0.009355`, 30.976025`, 
              30.950573`, \(-0.431737`\), 43.49992`, 43.55507`, 0.935498`, 
              15.880884`, 22.963928`, 30.976108`, 46.263632`, 63.643324`, 
              69.752069`, 71.634437`, 73.417569`, 54.828004`, 59.185229`, 
              70.233878`, 74.021273`, 35.931338`, 54.454524`, 61.92048`, 
              70.448125`, 22.903951`, 30.976025`, 43.49992`, 46.42929`, 
              55.087284`, 66.220994`, 72.402802`, 72.739651`, 34.342464`, 
              60.321597`, 68.102494`, 69.973197`, 57.990367`, 60.142587`, 
              72.783184`, 72.055893`, 28.683755`, 45.567465`, 52.27398`, 
              62.334198`, 30.912199`, 48.251991`, 50.491023`, 58.14851`}, {2, 
              56, 98.56749724`, \(-8.678295519`\), 5.149387628`, 
              0.1072401379`, 1, 
              0.9511099567`, \(-0.0773283664`\), \(-0.412850594`\), 
              8437.647236`, 0.1209403565`, 
              3655.724334`, \(-0.7290905206`\), \(-1.437938054`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.279549`, \(-0.730019`\), 
              0.11829`, 0.210061`, \(-0.040558`\), 0.093812`, 2.751778`, 
              1.52116`, 0.710374`, 0.597659`, 0.931472`, 0.960682`, 
              1.287035`, 0.792613`, 13.452365`, 13.492002`, 0.91291`, 
              17.658699`, 17.686909`, 0.649739`, 17.766405`, 17.802732`, 
              0.836683`, 27.208516`, 27.151595`, \(-1.311`\), 20.959184`, 
              20.966522`, 0.169016`, 34.375079`, 34.405146`, 0.692498`, 
              39.348689`, 39.335712`, \(-0.298893`\), 42.924783`, 42.936192`, 
              0.262777`, 20.959184`, 20.950057`, \(-0.210208`\), 29.746372`, 
              29.774718`, 0.652861`, 34.375079`, 34.385418`, 0.238131`, 
              39.348689`, 39.328448`, \(-0.466194`\), 27.208516`, 27.229028`, 
              0.472431`, 31.810589`, 31.799933`, \(-0.245432`\), 38.280113`, 
              38.297434`, 0.398935`, 39.587408`, 39.583191`, \(-0.097122`\), 
              13.452365`, 13.531844`, 0.915277`, 17.766405`, 17.832168`, 
              0.757324`, 27.208516`, 27.124313`, \(-0.969685`\), 29.483518`, 
              29.437826`, \(-0.526197`\), 20.959184`, 20.973323`, 0.162825`, 
              34.375079`, 34.38633`, 0.129569`, 39.348689`, 39.356759`, 
              0.092931`, 46.314799`, 46.425655`, 1.276629`, 20.959184`, 
              20.942522`, \(-0.191872`\), 29.746372`, 29.802324`, 0.644337`, 
              34.375079`, 34.446324`, 0.820453`, 39.348689`, 
              39.274821`, \(-0.850673`\), 17.658699`, 17.722136`, 0.730546`, 
              27.208516`, 27.22074`, 0.140771`, 29.431663`, 
              29.38657`, \(-0.51929`\), 31.574215`, 
              31.56782`, \(-0.073641`\), 13.452365`, 17.766405`, 29.483518`, 
              33.59692`, 47.623036`, 53.608486`, 54.930607`, 58.300178`, 
              46.458462`, 48.119289`, 61.107939`, 63.227139`, 29.746372`, 
              44.777152`, 51.972912`, 53.605716`, 17.658699`, 29.431663`, 
              31.574215`, 35.2381`, 42.924783`, 49.65973`, 56.45671`, 
              56.577957`, 31.810589`, 51.635595`, 51.53213`, 57.24668`, 
              46.314799`, 46.716986`, 54.176656`, 60.527921`, 20.959184`, 
              34.375079`, 39.348689`, 51.090857`, 27.208516`, 38.280113`, 
              39.587408`, 47.574967`}, {2, 11, 
              105.5101431`, \(-9.127174982`\), 5.122562159`, 0.1453085056`, 
              1, 0.8507976573`, \(-0.05324967878`\), \(-0.2567129115`\), 
              1570.962466`, 0.1354421572`, 
              746.0818937`, \(-0.8885773624`\), \(-1.579965316`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.371339`, \(-0.172546`\), \(-0.902569`\), 
              0.208654`, \(-0.037434`\), 0.082522`, 4.499799`, 2.685348`, 
              0.761823`, 0.665689`, 0.950508`, 0.972259`, 1.434141`, 
              0.902376`, 21.750151`, 21.777564`, 0.834335`, 27.150912`, 
              27.172522`, 0.657685`, 27.454315`, 27.469618`, 0.465745`, 
              33.60402`, 33.565947`, \(-1.158769`\), 23.128195`, 23.132219`, 
              0.12249`, 42.369417`, 42.389821`, 0.620997`, 47.409568`, 
              47.392059`, \(-0.532889`\), 50.305271`, 50.320682`, 0.46904`, 
              23.128195`, 23.120981`, \(-0.219554`\), 35.743711`, 35.761294`, 
              0.535144`, 42.369417`, 42.377221`, 0.237524`, 47.409568`, 
              47.389177`, \(-0.620626`\), 33.60402`, 33.615388`, 0.345991`, 
              36.421649`, 36.413222`, \(-0.256472`\), 45.393119`, 45.408142`, 
              0.457239`, 47.299393`, 47.296822`, \(-0.078252`\), 21.750151`, 
              21.8052`, 0.837724`, 27.454315`, 27.483057`, 0.437385`, 
              33.60402`, 33.553576`, \(-0.767643`\), 34.264126`, 34.317367`, 
              0.810204`, 23.128195`, 23.135365`, 0.109114`, 42.369417`, 
              42.37828`, 0.134878`, 47.409568`, 47.410672`, 0.016799`, 
              53.238243`, 53.440043`, 3.070932`, 23.128195`, 
              23.115379`, \(-0.195023`\), 35.743711`, 35.77842`, 0.528181`, 
              42.369417`, 42.417606`, 0.733335`, 47.409568`, 
              47.333846`, \(-1.152308`\), 27.150912`, 27.194654`, 0.665646`, 
              33.60402`, 33.59976`, \(-0.064831`\), 33.606469`, 33.664268`, 
              0.879574`, 34.305384`, 34.288287`, \(-0.260172`\), 21.750151`, 
              27.454315`, 34.346842`, 34.264126`, 55.623662`, 61.124424`, 
              63.849895`, 68.158774`, 51.840964`, 61.17475`, 71.649943`, 
              70.703227`, 35.743711`, 51.155584`, 62.850143`, 63.126676`, 
              27.150912`, 34.305384`, 33.606469`, 44.051435`, 50.305271`, 
              57.448808`, 65.520789`, 66.41757`, 36.421649`, 60.935285`, 
              63.466912`, 63.479376`, 53.238243`, 61.90092`, 61.22987`, 
              66.857063`, 23.128195`, 42.369417`, 47.409568`, 53.467711`, 
              33.60402`, 45.393119`, 47.299393`, 54.662098`}, {2, 39, 
              115.1822331`, \(-8.651401712`\), 4.111407276`, 0.1921681542`, 
              1, 0.8425444011`, 0.3138475204`, \(-0.7348296173`\), 
              2795.783725`, \(-0.07134050254`\), 1109.048525`, 
              0.04750237355`, \(-1.474386678`\), 0, 0, 0, 0, 0, 0, 0, 2, 3, 
              4, 5, 6, 7, 8, 9, 0.344097`, \(-0.411833`\), \(-0.561083`\), 
              0.244887`, \(-0.049367`\), 0.059182`, 4.423382`, 2.270807`, 
              0.81822`, 0.599396`, 1.015033`, 0.979283`, 1.522281`, 
              0.803225`, 20.177444`, 20.221699`, 1.001728`, 25.867429`, 
              25.910253`, 0.969345`, 26.372046`, 26.393954`, 0.495901`, 
              28.551609`, 28.542625`, \(-0.203352`\), 20.054202`, 20.059872`, 
              0.128335`, 38.784747`, 38.791906`, 0.162047`, 42.066603`, 
              42.082254`, 0.354262`, 43.754543`, 43.755953`, 0.031913`, 
              20.054202`, 20.04651`, \(-0.174106`\), 30.000642`, 30.018238`, 
              0.398303`, 38.784747`, 38.792411`, 0.173493`, 42.066603`, 
              42.065932`, \(-0.015185`\), 34.924054`, 34.932431`, 0.189619`, 
              40.495566`, 40.49503`, \(-0.012135`\), 42.307718`, 42.318358`, 
              0.240834`, 45.809185`, 45.80676`, \(-0.054885`\), 20.177444`, 
              20.266355`, 1.006275`, 26.372046`, 26.430106`, 0.657113`, 
              29.293324`, 29.258565`, \(-0.393403`\), 34.924054`, 
              34.71253`, \(-2.393977`\), 20.054202`, 20.063267`, 0.102595`, 
              38.784747`, 38.793666`, 0.100946`, 42.066603`, 42.08449`, 
              0.202437`, 43.754543`, 43.744354`, \(-0.115323`\), 20.054202`, 
              20.041496`, \(-0.143802`\), 30.000642`, 30.035646`, 0.396168`, 
              38.784747`, 38.805902`, 0.239431`, 42.066603`, 42.079383`, 
              0.144636`, 25.867429`, 25.938744`, 0.807123`, 28.551609`, 
              28.533542`, \(-0.204479`\), 34.924054`, 34.928898`, 0.05482`, 
              41.289957`, 41.269411`, \(-0.232534`\), 20.177444`, 26.372046`, 
              29.293324`, 38.780277`, 52.781728`, 54.140515`, 58.58913`, 
              59.486618`, 54.486073`, 60.727243`, 67.702248`, 68.794136`, 
              30.000642`, 49.702454`, 54.55029`, 55.952248`, 25.867429`, 
              28.551609`, 41.289957`, 43.715492`, 45.297368`, 53.502412`, 
              57.017547`, 58.60669`, 40.495566`, 53.577973`, 62.278241`, 
              61.632023`, 49.422398`, 55.838707`, 55.666241`, 61.074654`, 
              20.054202`, 38.784747`, 42.066603`, 43.754543`, 34.924054`, 
              42.307718`, 45.809185`, 50.673403`}, {2, 20, 
              114.8343292`, \(-7.755023921`\), 4.996369388`, 0.25`, 1, 
              0.5946568016`, 0.04850895148`, \(-0.185564638`\), 
              1586.82275`, \(-0.03797029929`\), 
              741.2164`, \(-0.7525902581`\), \(-1.626786855`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.550197`, 
              0.361703`, \(-1.989266`\), 0.226025`, \(-0.035393`\), 
              0.059832`, 7.690713`, 4.514617`, 0.898109`, 0.836558`, 
              1.059241`, 1.07338`, 1.722601`, 1.160658`, 37.044129`, 
              37.06007`, 0.70117`, 37.480696`, 37.502441`, 0.956423`, 
              38.750019`, 38.748167`, \(-0.081448`\), 44.45582`, 
              44.454156`, \(-0.073197`\), 27.612499`, 27.614472`, 0.086759`, 
              58.146189`, 58.142892`, \(-0.145039`\), 58.2333`, 58.237383`, 
              0.179553`, 63.290299`, 63.295693`, 0.237285`, 27.612499`, 
              27.609349`, \(-0.138537`\), 45.764072`, 45.770584`, 0.286429`, 
              58.146189`, 58.131031`, \(-0.666694`\), 58.2333`, 58.241293`, 
              0.351547`, 47.003864`, 47.007267`, 0.149666`, 48.767626`, 
              48.764747`, \(-0.126645`\), 61.181074`, 61.18957`, 0.373676`, 
              64.063005`, 64.061003`, \(-0.088015`\), 37.044129`, 37.077291`, 
              0.729295`, 38.750019`, 38.745312`, \(-0.103515`\), 44.45582`, 
              44.464682`, 0.194882`, 47.003864`, 46.959837`, \(-0.968246`\), 
              27.612499`, 27.616029`, 0.077637`, 58.146189`, 
              58.133292`, \(-0.283638`\), 58.2333`, 58.244851`, 0.254029`, 
              63.290299`, 63.298937`, 0.189963`, 27.612499`, 
              27.606828`, \(-0.124723`\), 45.764072`, 45.777048`, 0.285373`, 
              58.146189`, 58.122534`, \(-0.520232`\), 58.2333`, 58.247139`, 
              0.304336`, 37.480696`, 37.519884`, 0.861812`, 44.457313`, 
              44.474197`, 0.371301`, 47.003864`, 47.002874`, \(-0.021786`\), 
              49.624364`, 49.607853`, \(-0.363111`\), 37.044129`, 38.750019`, 
              44.45582`, 49.610592`, 73.660937`, 75.137436`, 82.533662`, 
              84.11153`, 66.608657`, 85.350036`, 82.877247`, 83.573379`, 
              45.764072`, 65.148842`, 77.288341`, 82.162231`, 37.480696`, 
              44.457313`, 49.624364`, 61.610622`, 64.601203`, 74.322189`, 
              81.337388`, 82.982411`, 48.767626`, 77.431717`, 80.363613`, 
              87.699949`, 69.832094`, 79.454722`, 78.956993`, 87.131306`, 
              27.612499`, 58.146189`, 58.2333`, 63.290299`, 47.003864`, 
              61.181074`, 64.063005`, 67.262516`}}};\)\)], "Input"]
}, Closed]],

Cell["data three/ytraj_1_3?.out; periodic BC's", "Section"],

Cell[CellGroupData[{

Cell["composite of three/ytraj_1_??.out; periodic BC's", "Section"],

Cell[BoxData[
    \(\(data = {"\<********* Eigenvalues for the 3+1 transverse lattice \
*********\nCouplings:\n      0:mu^2  1:g^2 N  2:beta  3:lambda_1  4:lambda_2\n\
      5:lambda_3  6:lambda_4  7:lambda_5  8:tau_1  9:tau_2\n      10:kappa_S  \
11:kappa_A  12:mu_F  13:mu_0^2  14:mu_1^2\n      15:mu_2^2  16:mu_3^2\nUse \
chi^2 fit with 40 criteria and tolerance 0.01.\nOverall scale from minimizing \
chi^2.\nParity doublets (charge,multi,state) with error for difference\n  \
over average.\n  ( 1, 5,0) and ( 1, 6,0), error 0.25\n  (-1, 5,0) and (-1, \
6,0), error 0.25\n  (-1, 7,0) and (-1, 4,0), error 0.25\nSpectrum for P_perp \
a = (0,0), ( 0.25 0)\n  using 4 states (multiplet, charge, c^2 error for \
each) =\n  (15,  1, 0.25 2 2 2)\n  (15, -1, 0.25 2 2 2)\n  (16, -1, 0.25 2 2 \
2)\n  (16,  1, 0.25 2 2 2)\nSpectrum for P_perp a = (0,0), ( 0.25 0.25)\n  \
using 4 states (multiplet, charge, c^2 error for each) =\n  (17,  1, 0.25 2 2 \
2)\n  (17, -1, 0.25 2 2 2)\n  (18, -1, 0.25 2 2 2)\n  (18,  1, 0.25 2 2 2).\n\
Ordinary spectra multiplets, 4 states, (charge,multiplet) = ( 1,  3) \n  (-1, \
 3)  ( 1,  4)  (-1,  4)  ( 1,  5)  (-1,  5)  ( 1,  6) \n  (-1,  6)  (-1,  7)  \
( 1,  7) .\nAll spectra extrapolated using (K,p) = (12/2,8)  (12/2,6) \n  \
(14/2,6)  (18/2,6) .\nWinding potential using (n,K,p) = ( 2 0,16/2,6)  ( 2 \
0,16/2,4) \n  ( 2 0,26/2,4)  ( 3 0,18/2,7)  ( 3 0,18/2,5)  ( 3 0,26/2,5) \n  \
( 4 0,16/2,8)  ( 4 0,16/2,6)  ( 4 0,26/2,6) .\nRoundness of winding using \
(n,K,p) = ( 2 2,14/2,8) \n  ( 2 2,14/2,6)  ( 2 2,22/2,6) \n  with error 1; in \
G^2 N units.\nHeavy potential determined using (n,K,p,K_max) =\n  ( 0 \
0,26/2,2,3.5)  ( 0 0,26/2,4,3.5)  ( 0 0,26/2,2,3) \n  ( 0 0,42/2,2,3.5)  ( 0 \
0,42/2,2,4.25)  ( 0 0,70/2,2,4) \n  ( 0 0,70/2,2,4.5) ,\n L = 3.577-2.726*m \
5.058-3.856*m 7.154-5.452*m\n (all in G^2 N units); relative scale error=0.1.\
\nRoundness determined using (n,K,p,K_max) = ( 1 0,14/2,3,3.5) \n  ( 1 \
0,14/2,5,3.5)  ( 1 0,14/2,3,3)  ( 1 0,24/2,3,4) \n  ( 1 0,24/2,3,4.5)  ( 1 \
0,36/2,3,5)  ( 1 0,36/2,3,6) \n L=1.265-0.964*m and error 0.1;\n  ( 1 \
0,14/2,3,3.5)  ( 1 0,14/2,5,3.5)  ( 1 0,14/2,3,3) \n  ( 1 0,24/2,3,4)  ( 1 \
0,24/2,3,4.5)  ( 1 0,36/2,3,5) \n  ( 1 0,36/2,3,6) \n L=5.058-3.856*m and \
error 0.1;\n  ( 1 1,22/2,2,5)  ( 1 1,22/2,4,5)  ( 1 1,22/2,2,4.5) \n  ( 1 \
1,44/2,2,5)  ( 1 1,44/2,2,6)  ( 1 1,70/2,2,6) \n  ( 1 1,70/2,2,7) \n L=0+0*m \
and error 0.2; all in G^2 N units.\np-extrapolation using n=( 1 0) and (K,p) \
= (14/2,3) \n  (22/2,3)  (46/2,3)  (14/2,5)  (28/2,5)  (14/2,7) \n  (16/2,7) \
.\nResult format:\n Fit info, # steps, chi^2, and p damping, scale G^2 \
N/sigma.\n The 17 couplings  (G^2 N units);\n   which couplings -- if any -- \
were fit.\n Winding potential and heavy souce potential fits.\n Roundness of \
winding potential, 1 values, and roundness of\n   heavy source potential, 3 \
values, (G^2 N units)\n   showing measured value and derived value for each.\n\
 The rescaled spectrum for each P_perp*a and c^2 values.\n Finally come the \
states for the ordinary spectra.\>", {{2, 93, 
              77.74064998`, \(-8.719198472`\), 10.85717086`, 0, 
              1, \(-0.09322407979`\), 0.2630434656`, \(-0.1959827684`\), 
              4.125790871`, \(-0.2507965869`\), 
              0.963421868`, \(-1.694691785`\), \(-2.573457059`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.148365`, \(-0.384917`\), 
              0.239529`, 0.107477`, \(-0.196382`\), 0.37413`, 1.610182`, 
              0.831947`, 0.448302`, 0.358829`, 0.542833`, 0.565992`, 
              1.07444`, 0.502521`, 17.977756`, 18.01602`, 0.986182`, 
              22.94905`, 22.950565`, 0.039033`, 33.208187`, 33.245352`, 
              0.957858`, 35.665956`, 35.711642`, 1.177468`, 32.434295`, 
              32.453444`, 0.493539`, 48.116038`, 48.119004`, 0.07646`, 
              55.215593`, 55.208964`, \(-0.170857`\), 56.867021`, 56.870369`, 
              0.086288`, 20.678434`, 20.678454`, 0.00052`, 32.434295`, 
              32.436134`, 0.047408`, 48.116038`, 48.10006`, \(-0.411799`\), 
              50.583505`, 50.591917`, 0.2168`, 35.769827`, 35.770326`, 
              0.012873`, 39.559889`, 39.560246`, 0.009206`, 48.262844`, 
              48.263102`, 0.006663`, 55.013726`, 55.020498`, 0.174534`, 
              17.977756`, 18.053952`, 0.981914`, 22.94905`, 22.952478`, 
              0.044175`, 35.769827`, 35.74446`, \(-0.326891`\), 39.559889`, 
              39.55927`, \(-0.007972`\), 32.434295`, 32.471971`, 0.485512`, 
              48.116038`, 48.104004`, \(-0.15507`\), 51.208479`, 51.218117`, 
              0.124211`, 55.215593`, 55.224609`, 0.116184`, 20.678434`, 
              20.678444`, 0.000129`, 32.434295`, 32.438168`, 0.049907`, 
              48.116038`, 48.107029`, \(-0.116088`\), 50.583505`, 50.597868`, 
              0.18508`, 33.208187`, 33.277816`, 0.897283`, 35.665956`, 
              35.761045`, 1.225373`, 35.769827`, 35.776986`, 0.092266`, 
              48.262844`, 48.385444`, 1.579911`, 17.977756`, 22.94905`, 
              41.647389`, 47.230924`, 63.426534`, 62.221165`, 68.667568`, 
              62.629968`, 62.021436`, 64.472984`, 68.590065`, 76.28353`, 
              20.678434`, 50.583505`, 53.897017`, 59.419484`, 33.208187`, 
              35.665956`, 54.647512`, 51.199564`, 56.867021`, 59.055556`, 
              61.703044`, 65.252665`, 39.559889`, 56.787512`, 66.558733`, 
              75.754449`, 51.208479`, 63.693887`, 67.638807`, 74.285931`, 
              32.434295`, 48.116038`, 55.215593`, 58.117943`, 35.769827`, 
              48.262844`, 55.013726`, 58.083944`}, {2, 12, 
              104.6388092`, \(-0.25`\), 6.908512945`, 0.001967542671`, 
              1, \(-0.2859175775`\), \(-0.02304343231`\), \
\(-0.04617080277`\), 105.8441254`, 0.04714841419`, 
              41.51103398`, \(-0.7033299165`\), \(-0.5553591676`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.179423`, 
              0.032144`, \(-0.814782`\), 0.182172`, \(-0.022895`\), 
              0.135414`, 2.299824`, 1.36568`, 0.643067`, 0.59753`, 1.089321`, 
              1.03855`, 1.218054`, 0.763736`, 15.111727`, 15.120337`, 
              0.170748`, 18.289772`, 18.293292`, 0.069799`, 18.463436`, 
              18.462482`, \(-0.018918`\), 20.012137`, 
              19.999599`, \(-0.248673`\), 21.352172`, 21.359106`, 0.137525`, 
              34.271297`, 34.2875`, 0.321361`, 37.639672`, 
              37.615517`, \(-0.479066`\), 38.942955`, 38.952319`, 0.185718`, 
              21.352172`, 21.376417`, 0.480849`, 21.533315`, 
              21.513957`, \(-0.383918`\), 34.271297`, 
              34.268208`, \(-0.061249`\), 37.385563`, 
              37.337456`, \(-0.954108`\), 20.34719`, 20.350486`, 0.065368`, 
              21.081235`, 21.078258`, \(-0.059043`\), 35.168891`, 35.173403`, 
              0.08949`, 36.177941`, 36.174515`, \(-0.067957`\), 15.111727`, 
              15.128988`, 0.171165`, 18.463436`, 18.461079`, \(-0.023368`\), 
              20.015612`, 20.00606`, \(-0.094726`\), 20.34719`, 
              20.343972`, \(-0.031919`\), 21.352172`, 21.365369`, 0.130871`, 
              34.271297`, 34.269332`, \(-0.019479`\), 37.639672`, 
              37.633769`, \(-0.058535`\), 39.229684`, 39.244451`, 0.146433`, 
              21.352172`, 21.401543`, 0.489577`, 21.533315`, 
              21.494519`, \(-0.384713`\), 34.271297`, 34.300308`, 0.287686`, 
              37.385563`, 37.215029`, \(-1.691082`\), 18.289772`, 18.297206`, 
              0.073711`, 20.012137`, 19.99442`, \(-0.175688`\), 20.34719`, 
              20.358941`, 0.116521`, 34.722502`, 34.725791`, 0.032621`, 
              15.111727`, 18.463436`, 20.015612`, 25.694861`, 44.3849`, 
              46.227036`, 50.457269`, 45.996292`, 37.57672`, 39.347668`, 
              43.712704`, 45.202439`, 21.533315`, 37.385563`, 40.459752`, 
              45.312319`, 18.289772`, 20.012137`, 34.722502`, 36.172085`, 
              38.942955`, 45.730601`, 47.710275`, 44.725067`, 21.081235`, 
              40.23551`, 43.915252`, 45.391061`, 39.229684`, 43.972219`, 
              44.146551`, 50.019275`, 21.352172`, 34.271297`, 37.639672`, 
              41.365761`, 20.34719`, 35.168891`, 36.177941`, 39.232526`}, {2, 
              29, 83.23419647`, \(-9.963903787`\), 9.909704447`, 
              0.007919222016`, 1, 0.06665684768`, 
              0.07485119468`, \(-0.2869147126`\), 
              4.587635016`*^6, \(-0.1783436366`\), 
              2.018051428`*^6, \(-1.515377013`\), \(-2.505935987`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.142899`, \(-0.432393`\), 
              0.382641`, 0.119397`, \(-0.203517`\), 0.419575`, 1.998408`, 
              0.758632`, 0.476327`, 0.372226`, 0.555377`, 0.585152`, 
              1.081329`, 0.508217`, 14.526847`, 14.571608`, 1.014171`, 
              22.302796`, 22.330994`, 0.638894`, 23.372501`, 
              23.368932`, \(-0.080864`\), 29.397801`, 
              29.375848`, \(-0.49741`\), 30.464208`, 30.487745`, 0.53328`, 
              48.103167`, 48.108803`, 0.127702`, 51.064582`, 51.075361`, 
              0.244214`, 52.464418`, 52.454892`, \(-0.215846`\), 23.370028`, 
              23.369989`, \(-0.000886`\), 30.464208`, 30.46508`, 0.019765`, 
              48.103167`, 48.097757`, \(-0.122574`\), 50.405772`, 
              50.400728`, \(-0.114274`\), 29.397801`, 29.398266`, 0.010538`, 
              34.797761`, 34.797792`, 0.000688`, 48.282782`, 
              48.282373`, \(-0.009264`\), 50.517041`, 50.517229`, 0.004274`, 
              14.526847`, 14.616604`, 1.016836`, 23.372501`, 
              23.365365`, \(-0.080839`\), 29.397801`, 
              29.365172`, \(-0.369652`\), 30.600698`, 30.617711`, 0.192739`, 
              30.464208`, 30.504121`, 0.452159`, 48.103167`, 
              48.101753`, \(-0.016012`\), 51.064582`, 51.079418`, 0.168077`, 
              52.381156`, 52.387532`, 0.07223`, 23.370028`, 
              23.369936`, \(-0.001042`\), 30.464208`, 30.472841`, 0.097799`, 
              48.103167`, 48.106094`, 0.03316`, 50.405772`, 
              50.395609`, \(-0.115126`\), 22.302796`, 22.358779`, 0.634218`, 
              29.397801`, 29.388763`, \(-0.102391`\), 39.405699`, 
              39.375235`, \(-0.345119`\), 46.348576`, 
              46.336357`, \(-0.138426`\), 14.526847`, 23.372501`, 30.600698`, 
              39.960734`, 59.50033`, 63.972496`, 64.060372`, 53.885757`, 
              56.687334`, 59.154692`, 62.492118`, 66.0927`, 23.370028`, 
              50.405772`, 53.035513`, 54.356281`, 22.302796`, 39.405699`, 
              46.348576`, 49.752785`, 52.464418`, 55.658893`, 61.185199`, 
              53.793453`, 34.797761`, 55.02416`, 59.692318`, 65.573923`, 
              52.381156`, 59.350755`, 65.273948`, 71.940898`, 30.464208`, 
              48.103167`, 51.064582`, 52.985337`, 29.397801`, 48.282782`, 
              50.517041`, 53.865367`}, {2, 10, 
              80.08569985`, \(-9.703171222`\), 6.356633011`, 0.01800590693`, 
              1, 0.4603567256`, 0.03769360804`, \(-0.2405940089`\), 
              112.769386`, 0.01205535126`, 
              53.92982256`, \(-1.429490563`\), \(-1.932714669`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.169863`, \(-0.355078`\), \(-0.108878`\), 
              0.133708`, \(-0.125048`\), 0.292707`, 1.77569`, 0.990212`, 
              0.543864`, 0.35925`, 0.713516`, 0.631708`, 1.06373`, 5, 
              0.449344`, 12.012017`, 12.039268`, 0.470784`, 15.966203`, 
              16.029753`, 1.097894`, 16.176335`, 16.197644`, 0.368121`, 
              21.653985`, 21.604145`, \(-0.861041`\), 20.321559`, 20.331906`, 
              0.178758`, 31.603987`, 31.62819`, 0.418129`, 35.293833`, 
              35.280396`, \(-0.232154`\), 37.432578`, 37.437187`, 0.079623`, 
              20.321559`, 20.403176`, 1.410018`, 20.917357`, 
              20.848164`, \(-1.195387`\), 31.603987`, 
              31.598609`, \(-0.092925`\), 35.293833`, 
              35.244077`, \(-0.859603`\), 21.653985`, 21.663003`, 0.155803`, 
              24.777185`, 24.773371`, \(-0.065887`\), 34.044008`, 34.050862`, 
              0.118418`, 35.320474`, 35.304445`, \(-0.276918`\), 12.012017`, 
              12.066642`, 0.471845`, 16.176335`, 16.217207`, 0.353052`, 
              21.653985`, 21.568326`, \(-0.739919`\), 24.777185`, 
              24.767814`, \(-0.080945`\), 20.321559`, 20.34165`, 0.173543`, 
              31.603987`, 31.598272`, \(-0.04937`\), 35.293833`, 
              35.289664`, \(-0.036015`\), 36.639249`, 36.688458`, 0.425069`, 
              20.321559`, 20.482508`, 1.390284`, 20.917357`, 
              20.782063`, \(-1.168669`\), 31.603987`, 31.649886`, 0.396477`, 
              35.293833`, 35.17562`, \(-1.021133`\), 15.966203`, 16.079569`, 
              0.979262`, 21.653985`, 21.658962`, 0.042992`, 24.679381`, 
              24.795604`, 1.003937`, 30.721608`, 30.973755`, 2.17805`, 
              12.012017`, 16.176335`, 24.995026`, 27.263539`, 43.791562`, 
              45.252047`, 46.021699`, 40.545531`, 37.997333`, 40.887041`, 
              46.629394`, 50.769335`, 20.917357`, 36.92427`, 39.336162`, 
              46.631324`, 15.966203`, 24.679381`, 30.721608`, 32.045367`, 
              37.432578`, 45.29047`, 45.632384`, 37.642332`, 24.777185`, 
              38.926604`, 46.138776`, 50.992615`, 36.639249`, 44.333501`, 
              47.456357`, 48.276081`, 20.321559`, 31.603987`, 35.293833`, 
              41.14892`, 21.653985`, 34.044008`, 35.320474`, 37.98443`}, {2, 
              21, 165.9667079`, \(-9.511617143`\), 2.045016211`, 
              0.03249196962`, 1, 0.7019188198`, 
              0.047421379`, \(-0.2825886398`\), 148.0935677`, 0.04387922894`, 
              72.23754226`, \(-1.304816549`\), \(-2.090869737`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.189452`, \(-0.666824`\), 
              0.228272`, 0.149544`, \(-0.093524`\), 0.239133`, 1.381544`, 
              0.87733`, 0.549857`, 0.28688`, 0.686553`, 0.626072`, 1.06173`, 
              0, 0.284691`, 3.234384`, 3.245747`, 0.070443`, 5.288532`, 
              5.403403`, 0.712073`, 5.512683`, 5.521645`, 0.055559`, 
              7.634248`, 7.616964`, \(-0.107145`\), 6.73839`, 6.741475`, 
              0.019118`, 10.174696`, 10.184386`, 0.060065`, 11.882004`, 
              11.876305`, \(-0.035326`\), 12.840402`, 12.843387`, 0.018502`, 
              6.73839`, 6.992869`, 1.577498`, 7.413846`, 
              7.167412`, \(-1.52763`\), 10.174696`, 
              10.170255`, \(-0.027531`\), 11.882004`, 
              11.854316`, \(-0.171633`\), 7.634248`, 7.640253`, 0.037225`, 
              8.894476`, 8.892259`, \(-0.013748`\), 11.296229`, 11.299104`, 
              0.017825`, 11.975971`, 11.962855`, \(-0.081306`\), 3.234384`, 
              3.257189`, 0.070685`, 5.512683`, 5.530115`, 0.054031`, 
              7.634248`, 7.608104`, \(-0.081032`\), 8.823215`, 8.845103`, 
              0.067842`, 6.73839`, 6.746045`, 0.023724`, 10.174696`, 
              10.169459`, \(-0.016233`\), 11.882004`, 
              11.87441`, \(-0.023537`\), 12.157062`, 12.187796`, 0.095259`, 
              6.73839`, 7.074252`, 1.04099`, 7.413846`, 
              7.092698`, \(-0.995387`\), 10.174696`, 10.191608`, 0.052419`, 
              11.882004`, 11.824962`, \(-0.1768`\), 5.288532`, 5.45144`, 
              0.504927`, 7.634248`, 7.63799`, 0.011597`, 7.74651`, 8.645863`, 
              2.787513`, 9.355962`, 9.504287`, 0.459727`, 3.234384`, 
              5.512683`, 8.823215`, 10.458001`, 14.8773`, 15.569695`, 
              16.113281`, 14.51848`, 12.643234`, 14.314231`, 16.919349`, 
              18.256738`, 7.413846`, 12.545537`, 13.698848`, 16.638092`, 
              5.288532`, 7.74651`, 9.355962`, 9.641917`, 12.840402`, 
              15.750763`, 15.652731`, 13.983914`, 8.894476`, 13.292112`, 
              16.543872`, 16.121862`, 12.157062`, 14.50306`, 16.86288`, 
              16.292445`, 6.73839`, 10.174696`, 11.882004`, 14.378016`, 
              7.634248`, 11.296229`, 11.975971`, 13.015345`}, {2, 47, 
              45.55962782`, \(-9.369954152`\), 6.903648624`, 0.0517766953`, 
              1, 0.9512419514`, 0.1401678041`, \(-0.3340921541`\), 
              45626.48396`, 0.1750495074`, 
              19441.14541`, \(-1.438535821`\), \(-1.966143731`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.212016`, \(-0.853742`\), 
              0.52896`, 0.160528`, \(-0.12921`\), 0.282729`, 1.468508`, 
              0.908506`, 0.584132`, 0.492464`, 0.73798`, 0.767697`, 
              1.103589`, 0.655337`, 12.588568`, 12.634563`, 1.077148`, 
              22.8738`, 22.904079`, 0.709094`, 23.108835`, 23.14241`, 
              0.786302`, 31.855151`, 31.798189`, \(-1.333997`\), 23.953446`, 
              23.961978`, 0.199811`, 36.801188`, 36.841209`, 0.937256`, 
              44.993731`, 44.96484`, \(-0.676588`\), 49.762608`, 49.781496`, 
              0.442344`, 23.953446`, 23.999377`, 1.075658`, 30.419709`, 
              30.408281`, \(-0.267622`\), 36.801188`, 
              36.797641`, \(-0.08305`\), 44.993731`, 
              44.915822`, \(-1.824531`\), 31.855151`, 31.885311`, 0.706302`, 
              37.127367`, 37.115578`, \(-0.276077`\), 42.414935`, 42.428408`, 
              0.315513`, 45.955897`, 45.930341`, \(-0.598493`\), 12.588568`, 
              12.680809`, 1.080092`, 23.108835`, 23.171279`, 0.731189`, 
              31.855151`, 31.784283`, \(-0.829832`\), 36.686351`, 36.760632`, 
              0.869798`, 23.953446`, 23.97591`, 0.263045`, 36.801188`, 
              36.792284`, \(-0.104258`\), 44.993731`, 
              44.984168`, \(-0.111979`\), 47.044988`, 47.131445`, 1.012361`, 
              23.953446`, 24.017717`, 0.75258`, 30.419709`, 
              30.41952`, \(-0.002217`\), 36.801188`, 36.887745`, 1.013542`, 
              44.993731`, 44.794277`, \(-2.335499`\), 22.8738`, 22.939156`, 
              0.765287`, 31.855151`, 31.8726`, 0.204322`, 35.464283`, 
              35.507039`, 0.500656`, 36.12793`, 36.114773`, \(-0.154059`\), 
              12.588568`, 23.108835`, 36.686351`, 42.277968`, 54.294113`, 
              60.971083`, 62.630258`, 61.869363`, 49.484211`, 55.882008`, 
              70.396403`, 67.915396`, 30.419709`, 49.794092`, 54.47242`, 
              61.451672`, 22.8738`, 36.12793`, 35.464283`, 40.544158`, 
              49.762608`, 57.792931`, 60.822745`, 63.114159`, 37.127367`, 
              54.401822`, 58.038702`, 66.968835`, 47.044988`, 51.558144`, 
              63.248357`, 64.922916`, 23.953446`, 36.801188`, 44.993731`, 
              58.043419`, 31.855151`, 42.414935`, 45.955897`, 
              52.630098`}, {2, 57, 39.69398236`, \(-9.180373479`\), 
              7.563874923`, 0.07642881742`, 1, 
              0.76809841`, \(-0.1692461589`\), \(-0.1857716652`\), 
              49854.19434`, 0.02426983029`, 
              24828.43378`, \(-1.547566329`\), \(-2.239030657`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.280322`, \(-0.531459`\), \(-0.246649`\), 
              0.148495`, \(-0.166355`\), 0.340103`, 2.430894`, 1.680288`, 
              0.611141`, 0.528937`, 0.71216`, 0.733464`, 1.201923`, 
              0.734706`, 15.880884`, 15.914083`, 1.126294`, 22.903951`, 
              22.929273`, 0.859073`, 22.963928`, 22.988224`, 0.824247`, 
              30.912199`, 30.946653`, 1.16886`, 28.683755`, 28.69176`, 
              0.271583`, 45.567465`, 45.594481`, 0.916504`, 52.27398`, 
              52.254324`, \(-0.666836`\), 55.087284`, 55.102457`, 0.514725`, 
              28.683755`, 28.712611`, 0.978958`, 35.931338`, 
              35.925961`, \(-0.182424`\), 45.567465`, 
              45.554409`, \(-0.442923`\), 52.27398`, 
              52.230872`, \(-1.462461`\), 30.912199`, 30.932793`, 0.698634`, 
              34.342464`, 34.327499`, \(-0.507695`\), 48.251991`, 48.268106`, 
              0.546683`, 50.491023`, 50.478944`, \(-0.409787`\), 15.880884`, 
              15.947544`, 1.13073`, 22.963928`, 23.003966`, 0.679149`, 
              30.912199`, 30.868313`, \(-0.744429`\), 30.976108`, 
              30.940954`, \(-0.596306`\), 28.683755`, 28.704935`, 0.359266`, 
              45.567465`, 45.559264`, \(-0.139108`\), 52.27398`, 
              52.273502`, \(-0.008117`\), 57.990367`, 58.038021`, 0.808336`, 
              28.683755`, 28.727813`, 0.747345`, 35.931338`, 
              35.929399`, \(-0.032895`\), 45.567465`, 45.605335`, 0.642362`, 
              52.27398`, 52.151894`, \(-2.070909`\), 22.903951`, 22.955543`, 
              0.875134`, 30.912199`, 30.912751`, 0.009355`, 30.976025`, 
              30.950573`, \(-0.431737`\), 43.49992`, 43.55507`, 0.935498`, 
              15.880884`, 22.963928`, 30.976108`, 46.263632`, 63.643324`, 
              69.752069`, 71.634437`, 73.417569`, 54.828004`, 59.185229`, 
              70.233878`, 74.021273`, 35.931338`, 54.454524`, 61.92048`, 
              70.448125`, 22.903951`, 30.976025`, 43.49992`, 46.42929`, 
              55.087284`, 66.220994`, 72.402802`, 72.739651`, 34.342464`, 
              60.321597`, 68.102494`, 69.973197`, 57.990367`, 60.142587`, 
              72.783184`, 72.055893`, 28.683755`, 45.567465`, 52.27398`, 
              62.334198`, 30.912199`, 48.251991`, 50.491023`, 58.14851`}, {2, 
              11, 98.50276386`, \(-8.759288452`\), 5.335730856`, 
              0.1072401379`, 1, 
              0.9533021314`, \(-0.1131319213`\), \(-0.3557856176`\), 
              20228.99226`, 0.09830611755`, 
              9750.568781`, \(-0.9425853121`\), \(-1.681989874`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.292031`, \(-0.70427`\), 
              0.002652`, 0.196074`, \(-0.050713`\), 0.124779`, 2.732998`, 
              1.632311`, 0.693443`, 0.585609`, 0.877212`, 0.910024`, 
              1.280816`, 0.779434`, 13.616507`, 13.653267`, 0.916475`, 
              18.418231`, 18.455632`, 0.932461`, 18.45588`, 18.479462`, 
              0.587911`, 27.365051`, 27.314108`, \(-1.270056`\), 21.736203`, 
              21.743039`, 0.170432`, 35.513817`, 35.541446`, 0.688819`, 
              40.7788`, 40.765922`, \(-0.321056`\), 44.145252`, 44.15744`, 
              0.303854`, 21.736203`, 21.726799`, \(-0.23446`\), 30.442837`, 
              30.472234`, 0.732895`, 35.513817`, 35.519764`, 0.148261`, 
              40.7788`, 40.757073`, \(-0.541683`\), 27.365051`, 27.385758`, 
              0.516252`, 31.613278`, 31.601371`, \(-0.296874`\), 39.172405`, 
              39.190598`, 0.453572`, 40.599425`, 40.593309`, \(-0.152476`\), 
              13.616507`, 13.690272`, 0.91952`, 18.418231`, 18.480385`, 
              0.774789`, 27.365051`, 27.29104`, \(-0.922589`\), 28.963671`, 
              28.920827`, \(-0.534079`\), 21.736203`, 21.750792`, 0.181865`, 
              35.513817`, 35.520591`, 0.084441`, 40.7788`, 40.786017`, 
              0.089972`, 47.090391`, 47.252152`, 2.016436`, 21.736203`, 
              21.717534`, \(-0.232713`\), 30.442837`, 30.500892`, 0.723692`, 
              35.513817`, 35.575788`, 0.772495`, 40.7788`, 
              40.703232`, \(-0.941999`\), 18.45588`, 18.515353`, 0.741367`, 
              27.365051`, 27.379584`, 0.181167`, 28.939051`, 
              28.892377`, \(-0.581827`\), 32.555171`, 32.557279`, 0.026273`, 
              13.616507`, 18.418231`, 28.963671`, 35.065387`, 49.302211`, 
              55.170868`, 56.774443`, 60.10574`, 46.790474`, 49.211354`, 
              61.739908`, 64.119516`, 30.442837`, 45.471309`, 53.064306`, 
              55.12776`, 18.45588`, 28.939051`, 32.555171`, 36.241533`, 
              44.145252`, 51.337622`, 58.273336`, 58.43214`, 31.613278`, 
              52.455506`, 52.867509`, 58.164317`, 47.090391`, 48.347601`, 
              55.875636`, 61.66681`, 21.736203`, 35.513817`, 40.7788`, 
              51.878997`, 27.365051`, 39.172405`, 40.599425`, 
              48.650404`}, {2, 13, 115.3333111`, \(-8.863029652`\), 
              4.747518371`, 0.1453085056`, 1, 
              0.938967997`, \(-0.1210019785`\), \(-0.3291059662`\), 
              24618.21`, 0.05026764128`, 
              12001.97987`, \(-0.9059424461`\), \(-1.83522371`\), 0, 0, 0, 0, 
              0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 
              0.358261`, \(-0.472661`\), \(-0.49297`\), 
              0.203897`, \(-0.036127`\), 0.087529`, 3.732347`, 2.331802`, 
              0.739195`, 0.600858`, 0.892286`, 0.916245`, 1.392216`, 
              0.805047`, 16.58621`, 16.617034`, 0.838815`, 21.711923`, 
              21.75372`, 1.137435`, 21.754993`, 21.760768`, 0.157166`, 
              28.805203`, 28.759262`, \(-1.25022`\), 21.23765`, 21.242398`, 
              0.129221`, 37.103009`, 37.12051`, 0.476253`, 41.905854`, 
              41.900034`, \(-0.158387`\), 44.552568`, 44.563462`, 0.296457`, 
              21.23765`, 21.227961`, \(-0.263663`\), 31.21916`, 31.24293`, 
              0.646873`, 37.103009`, 37.105346`, 0.063574`, 41.905854`, 
              41.894928`, \(-0.297342`\), 28.805203`, 28.819767`, 0.396314`, 
              32.189557`, 32.180118`, \(-0.256866`\), 40.228024`, 40.244518`, 
              0.448875`, 41.824084`, 41.820983`, \(-0.084381`\), 16.58621`, 
              16.648171`, 0.843092`, 21.711923`, 21.7595`, 0.647366`, 
              28.805203`, 28.736316`, \(-0.93734`\), 30.230307`, 
              30.194755`, \(-0.483744`\), 21.23765`, 21.247721`, 0.137034`, 
              37.103009`, 37.1079`, 0.066545`, 41.905854`, 41.913039`, 
              0.097759`, 47.337141`, 47.702036`, 4.965054`, 21.23765`, 
              21.218534`, \(-0.260103`\), 31.21916`, 31.266151`, 0.639396`, 
              37.103009`, 37.13862`, 0.48455`, 41.905854`, 
              41.86592`, \(-0.543374`\), 21.754993`, 21.802102`, 0.641003`, 
              28.805203`, 28.811857`, 0.09053`, 30.218534`, 
              30.181892`, \(-0.498574`\), 30.757476`, 
              30.748884`, \(-0.11691`\), 16.58621`, 21.711923`, 30.230307`, 
              31.538228`, 50.397804`, 54.51761`, 57.81371`, 60.953035`, 
              46.770824`, 53.199021`, 63.696136`, 64.633981`, 31.21916`, 
              45.77961`, 56.097926`, 54.65642`, 21.754993`, 30.218534`, 
              30.757476`, 38.114427`, 44.552568`, 51.868455`, 58.779833`, 
              59.895269`, 32.189557`, 54.165905`, 54.553833`, 57.295334`, 
              47.337141`, 52.930117`, 56.049081`, 61.206945`, 21.23765`, 
              37.103009`, 41.905854`, 48.937815`, 28.805203`, 40.228024`, 
              41.824084`, 49.003707`}, {1, 12, 
              108.6539535`, \(-9.024532028`\), 4.46942687`, 0.1921681542`, 1, 
              0.8482403441`, \(-0.0752115788`\), \(-0.4267124885`\), 
              2.531198471`*^6, \(-0.001729744394`\), \(-309204.0658`\), \
\(-0.6241302381`\), \(-1.944987495`\), 0, 0, 0, 0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 
              8, 9, 0.408884`, \(-0.253955`\), \(-0.864921`\), 
              0.224408`, \(-0.031499`\), 0.057554`, 5.070648`, 2.909005`, 
              0.798062`, 0.656443`, 0.927922`, 0.972716`, 1.53629`, 
              0.889738`, 21.559131`, 21.593681`, 1.010225`, 25.545235`, 
              25.582017`, 1.075492`, 25.68787`, 25.70109`, 0.386543`, 
              31.684403`, 31.745268`, 1.779681`, 22.073468`, 22.077871`, 
              0.128736`, 41.848482`, 41.860061`, 0.338561`, 45.396137`, 
              45.39823`, 0.061191`, 47.929385`, 47.931971`, 0.075597`, 
              22.073468`, 22.066557`, \(-0.20208`\), 34.228917`, 34.244783`, 
              0.463911`, 41.848482`, 41.854805`, 0.184884`, 45.396137`, 
              45.393326`, \(-0.082191`\), 33.085384`, 33.094905`, 0.278409`, 
              36.86485`, 36.859147`, \(-0.16674`\), 44.835412`, 44.85177`, 
              0.478302`, 46.191867`, 46.19087`, \(-0.029134`\), 21.559131`, 
              21.628607`, 1.015716`, 25.68787`, 25.736202`, 0.706604`, 
              32.857372`, 32.880473`, 0.337729`, 33.085384`, 
              32.980417`, \(-1.534602`\), 22.073468`, 22.081123`, 0.11191`, 
              41.848482`, 41.856491`, 0.117091`, 45.396137`, 45.40649`, 
              0.151367`, 47.933649`, 47.92579`, \(-0.114899`\), 22.073468`, 
              22.061285`, \(-0.178122`\), 34.228917`, 34.260404`, 0.460334`, 
              41.848482`, 41.876581`, 0.410808`, 45.396137`, 
              45.385028`, \(-0.162407`\), 25.545235`, 25.596898`, 0.755313`, 
              31.684403`, 31.728918`, 0.650807`, 33.085384`, 
              33.083752`, \(-0.023861`\), 35.603073`, 
              35.503538`, \(-1.455176`\), 21.559131`, 25.68787`, 32.857372`, 
              35.031716`, 54.810819`, 57.32993`, 62.231171`, 64.328428`, 
              51.735132`, 62.360262`, 66.955386`, 69.525748`, 34.228917`, 
              50.206668`, 60.101265`, 62.105196`, 25.545235`, 31.684403`, 
              35.603073`, 42.560718`, 47.929385`, 55.785167`, 62.582984`, 
              63.532059`, 36.86485`, 58.654951`, 62.717313`, 62.724467`, 
              52.514035`, 60.572524`, 61.497216`, 64.000775`, 22.073468`, 
              41.848482`, 45.396137`, 47.933649`, 33.085384`, 44.835412`, 
              46.191867`, 52.217771`}, {2, 48, 
              114.3508317`, \(-8.276615831`\), 4.53865958`, 0.25`, 1, 
              0.6192218155`, 0.1354806948`, \(-0.376938921`\), 
              10277.45805`, \(-0.0687354657`\), 
              5020.839498`, \(-0.3784761485`\), \(-1.590636546`\), 0, 0, 0, 
              0, 0, 0, 0, 2, 3, 4, 5, 6, 7, 8, 9, 0.509808`, 
              0.225605`, \(-1.703753`\), 0.238431`, \(-0.039004`\), 
              0.053296`, 7.280213`, 4.091099`, 0.89698`, 0.771884`, 
              1.063317`, 1.047051`, 1.712201`, 1.065343`, 32.619724`, 
              32.643664`, 0.886292`, 33.881509`, 33.909765`, 1.046096`, 
              35.021145`, 35.020109`, \(-0.038346`\), 39.056863`, 39.062739`, 
              0.217562`, 24.983769`, 24.986562`, 0.103427`, 52.476822`, 
              52.476837`, 0.000563`, 52.758316`, 52.751312`, \(-0.259307`\), 
              56.175208`, 56.193532`, 0.678406`, 24.983769`, 
              24.980508`, \(-0.120729`\), 40.911142`, 40.918091`, 0.257271`, 
              52.476822`, 52.485257`, 0.312274`, 52.758316`, 
              52.748591`, \(-0.360017`\), 43.618472`, 43.621995`, 0.130407`, 
              46.748866`, 46.746581`, \(-0.084569`\), 55.477269`, 55.486656`, 
              0.347522`, 58.418129`, 58.415886`, \(-0.083039`\), 32.619724`, 
              32.668454`, 0.902032`, 35.021145`, 35.018297`, \(-0.052715`\), 
              39.056863`, 39.078327`, 0.397323`, 43.618472`, 
              43.516258`, \(-1.892057`\), 24.983769`, 24.988236`, 0.082685`, 
              52.476822`, 52.484457`, 0.141335`, 52.758316`, 
              52.746723`, \(-0.214602`\), 56.175208`, 56.194519`, 0.35747`, 
              24.983769`, 24.978572`, \(-0.096195`\), 40.911142`, 40.924995`, 
              0.256442`, 52.476822`, 52.487207`, 0.192244`, 52.758316`, 
              52.736746`, \(-0.39928`\), 33.881509`, 33.926198`, 0.82723`, 
              39.061915`, 39.089292`, 0.506767`, 43.618472`, 
              43.617848`, \(-0.011562`\), 48.711655`, 
              48.681401`, \(-0.560022`\), 32.619724`, 35.021145`, 39.056863`, 
              47.746627`, 66.908569`, 67.817012`, 74.974527`, 75.691662`, 
              63.085132`, 81.051873`, 78.749324`, 75.413481`, 40.911142`, 
              60.39469`, 69.718843`, 74.950834`, 33.881509`, 39.061915`, 
              48.711655`, 55.387386`, 58.248584`, 67.445223`, 73.275063`, 
              74.976641`, 46.748866`, 70.13714`, 74.983033`, 80.591386`, 
              63.627105`, 72.094781`, 72.586057`, 79.865123`, 24.983769`, 
              52.758316`, 52.476822`, 56.175208`, 43.618472`, 55.477269`, 
              58.418129`, 61.762892`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["analyze", "Section"],

Cell[BoxData[
    \(data[\([1]\)]\)], "Input"],

Cell["Chi^2", "Text"],

Cell[BoxData[""], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot[Map[{Sqrt[#[\([6]\)]], #[\([3]\)]} &, data[\([2]\)]], 
        PlotJoined -> True, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        FrameLabel -> {"\<m\>", "\<\>"} (*\(,\)\(PlotRange -> {0, 
              40}\)*) ]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Select which points to plot (sdata) and which points to use for \
fitting (ssdata).\
\>", "Text"],

Cell[BoxData[{
    \(\(sdata = Select[data[\([2]\)], \((#[\([3]\)] < 100)\) &];\)\), "\n", 
    \(\(ssdata = 
        Select[data[\([2]\)], \((#[\([3]\)] < 60)\) &];\)\)}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([6]\)])\) &, sdata]\)], "Input"],

Cell[BoxData[
    \({0, 0.001967542671`, 0.01800590693`, 0.03249196962`, 0.0517766953`, 
      0.07642881742`, 0.1072401379`, 0.1453085056`, 0.1921681542`, 
      0.25`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([6]\)])\) &, ssdata]\)], "Input"],

Cell[BoxData[
    \({0.01800590693`, 0.0517766953`, 0.07642881742`, 0.1072401379`, 
      0.1453085056`, 0.1921681542`}\)], "Output"]
}, Open  ]],

Cell["\<\
Using sdata, fit all couplings to either a linear or constant \
function.  Here, x is the mass.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Chop[Table[
          Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([i]\)]/#[\([7]\)]} &, 
              ssdata], If[MemberQ[{11, 13}, i], {1}, {1, x, x^2}], x], {i, 6, 
            6 + 9}]]\)], "Input"],

Cell[BoxData[
    \({0.9999999999999983`\ x\^2, 
      1.`, \(\(0.6371636454163107`\)\(\[InvisibleSpace]\)\) - 
        1.538847707659005`\ x + 
        7.655775859873223`\ x\^2, \
\(\(0.5234545166697914`\)\(\[InvisibleSpace]\)\) - 5.071185450318936`\ x + 
        10.723542815056934`\ x\^2, \(-0.382674137573457`\) + 
        2.8730211153388234`\ x - 6.064665708896461`\ x\^2, 
      639968.0679225`, \(-0.3244786449506592`\) + 3.6664721169102092`\ x - 
        7.1441786252760835`\ x\^2, 
      111671.81861775`, \(\(0.9061887759540128`\)\(\[InvisibleSpace]\)\) - 
        25.930279752410556`\ x + 
        55.0630602291444`\ x\^2, \(\(1.7385557236232931`\)\(\[InvisibleSpace]\
\)\) - 38.18233920659946`\ x + 83.12022199722941`\ x\^2}\)], "Output"]
}, Open  ]],

Cell["Fit weighted by chi^2 values of data", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Block[{selected}, 
        Chop[Table[\[IndentingNewLine]selected = 
              Select[ssdata, \((Abs[#[\([i]\)]/#[\([7]\)]] < 50000)\) &]; 
            BestFit /. 
              Regress[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], \
#[\([i]\)]/#[\([7]\)]} &, selected], 
                If[MemberQ[{11, 13}, i], {1}, {1, x, x^2}], x, 
                Weights \[Rule] Map[\((1/#[\([3]\)])\) &, selected], 
                RegressionReport \[Rule] BestFit], {i, 6, 
              6 + 9}]]]\)], "Input"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \({0.9999999999999998`\ x\^2, 
      1.0000000000000004`, \(\(0.48268867040166236`\)\(\[InvisibleSpace]\)\) \
- 0.09954202082336724`\ x + 
        4.709333786804126`\ x\^2, \(\(0.11639557739125364`\)\(\
\[InvisibleSpace]\)\) - 0.8875453837992442`\ x + 
        1.1826371945307124`\ x\^2, \(-0.23833639975538995`\) + 
        1.5906967976328137`\ x - 3.686498759255846`\ x\^2, 
      2880.884842933725`, \(-0.0736221667334823`\) + 1.0827424746671677`\ x - 
        1.2736725055270803`\ x\^2, 
      1298.3903289740977`, \(-0.30599821663084614`\) - 
        14.094981324777148`\ x + 
        29.725907064818944`\ x\^2, \(-0.4779947920169752`\) - 
        15.736783597096107`\ x + 32.904706507020585`\ x\^2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pc = 
      Table[Show[
          Plot[couplefitting[\([i - 5]\)], {x, 0, 1/2}, 
            PlotStyle -> {Hue[0]}, DisplayFunction -> Identity], 
          ListPlot[
            Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([i]\)]/#[\([7]\)]} &, 
              sdata], PlotStyle -> {PointSize[ .02], Hue[2/3]}, 
            DisplayFunction -> Identity], \n\t (*\(ListPlot[tau1, 
              PlotStyle -> {PointSize[ .02]}, 
              DisplayFunction -> 
                Identity]\)\(,\)*) \[IndentingNewLine]DisplayFunction -> \
$DisplayFunction, Axes -> False, RotateLabel -> False, 
          FormatType -> OutputForm, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
          Frame -> True, 
          FrameLabel -> {"\<m\>", {"\<b\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\
\(l\_2\)\>\"", \*"\"\<\!\(l\_3\)\>\"", \*"\"\<\!\(l\_4\)\>\"", \
\*"\"\<\!\(l\_5\)\>\"", \*"\"\<\!\(t\_1\)\>\"", \*"\"\<\!\(t\_2\)\>\""}[\([i \
- 7]\)]}], {i, 8, 15}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
a^2 sigma vs m using two different methods: using the overall scale \
and using the longitudinal potential. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita2 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.`\)\(\[InvisibleSpace]\)\) + 0.`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita3 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(2.0226799306464702`\)\(\[InvisibleSpace]\)\) + 
      0.6435286581292277`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fita2, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t\tPlot[fita3, {x, 0,  .5}, 
        DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0]}], \n\t\t (*\(Plot[1 + 2\ x, {x, 0,  .5}, 
          DisplayFunction -> Identity]\)\(,\)*) \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          sdata], PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          sdata], PlotStyle -> {PointSize[0.01], Hue[0]}, 
        DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitag = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]]*#[\([5]\)]} &, 
          Drop[ssdata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(5.091866692483492`\)\(\[InvisibleSpace]\)\) - 
      2.2778471841273324`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fitag, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], 
              Sqrt[#[\([24]\)]]*#[\([5]\)]} &, sdata], 
        PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \nFrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(exam = Sqrt[Partition[Take[data[\([2, 2]\)], \(-40\)], 4]]\)], "Input"],

Cell[BoxData[
    \({{3.4963372262984014`, 4.97000714285201`, 5.6980637062075745`, 
        6.5280748310662`}, {9.069016539845983`, 9.33762223480903`, 
        10.272320088470764`, 11.039075640650355`}, {8.236730722805014`, 
        9.651410363257797`, 10.844002351530545`, 
        10.756670767481916`}, {5.746020535988364`, 8.318284198078352`, 
        9.541465348676796`, 9.76838067440044`}, {4.968623753113129`, 
        5.7064206820037375`, 7.104791974435281`, 
        7.352763698093391`}, {8.391401968681992`, 9.11649537925622`, 
        10.096977319970566`, 10.367887875551123`}, {6.067179492976947`, 
        9.165112547044908`, 10.0852764463846`, 
        9.670015563586235`}, {8.36810558011788`, 9.59763460442207`, 
        9.768000972563424`, 9.546210504697662`}, {5.367422100040205`, 
        7.376504117805398`, 8.006552691389722`, 
        8.870453426967531`}, {5.6207531523809156`, 7.788696617534926`, 
        7.985614065305185`, 8.355544865536897`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec1 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(\(0\^++\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(0\^\(-+\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(\(1\^++\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(1\^\(-+\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(\(2\^++\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(2\^\(-+\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([1]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([3]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([10]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([5]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([7]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 7}}, 
          FormatType -> OutputForm, FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec2 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(0\^\(+-\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(\(0\^--\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(1\^\(+-\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(\(1\^--\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(2\^\(+-\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(\(2\^--\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([2]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([4]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([9]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([6]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([8]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 9.5}}, 
          FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put plots", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.spec1.eps\>", pspec1, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.spec2.eps\>", pspec2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.chi2.eps\>", chi2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pass = 
      Show[\n\t\t (*
          Plot[Sqrt[fita2], {x, 0,  .5}, DisplayFunction -> Identity], *) \n\t
        ListPlot[
          Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]*#[\([5]\)]]}&, 
            sdata], PlotStyle -> {PointSize[0.015]}, 
          DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
        PlotRange -> All, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.ass.eps\>", pass, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This was printed to a postscript file \"ttraj60.scaling.ps\", size \
616 by 779, and moved to the left such that left=0. (You need to adjust the \
Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ Show[GraphicsArray[Partition[pc, \ 2]]]\)\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This prints out scaling graphs to separate files (You need to \
adjust the Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{couplelist = {"\<b\>", "\<l1\>", "\<l2\>", "\<l3\>", "\<l4\>", "\
\<l5\>", "\<t1\>", "\<t2\>"}}, \[IndentingNewLine]Do[
        Print["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>"]; 
        Display["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>", 
          pc[\([i]\)], "\<EPS\>", ImageSize -> 360, 
          ImageOffset -> {0, 0}], {i, Length[pc]}]]\)], "Input"],

Cell[BoxData[
    \("~/three/ttraj60.b.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l2.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l3.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l4.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l5.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t2.eps"\)], "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Create data file", "Section"],

Cell["\<\
Average previous value with fit. (In the file, I set lamda_3 and \
lambda_5  = 100.)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      Block[{beta, ll = Length[data[\([2]\)]]}, 
        TableForm[
          Flatten[Table[{8, 
                Map[NumberForm[#, 12] &, 
                  beta = 0.25\ \((i - 1)\)/\((ll - 1)\); 
                  couplefitting /. 
                    x -> Sqrt[
                        beta\ Tan[
                            Pi\ beta]]]\[IndentingNewLine], \
\[IndentingNewLine]Table[0, {32 - Length[couplefitting]}], {2, 3, 4, 5, 6, 7, 
                  8, 9}}, {i, 1, 11}], 1], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(beta\)\" is similar to \
existing symbol \"\!\(Beta\)\"."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/three/ttraj_2_50.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<ttraj_2_50.out\>"]; \n\t\tWrite[fp, "\<100\>"]; \n\t\tWrite[
        fp, "\<3\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/three/ttraj_2_50.in"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
AutoGeneratedPackage->Automatic,
WindowSize->{790, 688},
WindowMargins->{{Automatic, 60}, {Automatic, 3}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 54, 0, 64, "Subtitle"],
Cell[1796, 53, 41, 0, 53, "Subsubtitle"],
Cell[1840, 55, 70, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[1935, 59, 159, 3, 27, "Input",
  InitializationCell->True],
Cell[2097, 64, 183, 3, 33, "Message"]
}, Open  ]],
Cell[2295, 70, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[2392, 74, 372, 7, 123, "Input"],
Cell[2767, 83, 48, 1, 27, "Input"],
Cell[2818, 86, 48, 1, 27, "Input"],
Cell[2869, 89, 183, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[3077, 96, 59, 0, 54, "Section"],
Cell[3139, 98, 31900, 448, 12922, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[35076, 551, 59, 0, 34, "Section"],
Cell[35138, 553, 31792, 443, 12892, "Input"]
}, Closed]],
Cell[66945, 999, 59, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[67029, 1003, 67, 0, 54, "Section"],
Cell[67099, 1005, 31813, 445, 12922, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[98949, 1455, 26, 0, 34, "Section"],
Cell[98978, 1457, 46, 1, 27, "Input"],
Cell[99027, 1460, 21, 0, 32, "Text"],
Cell[99051, 1462, 26, 0, 27, "Input"],

Cell[CellGroupData[{
Cell[99102, 1466, 329, 6, 59, "Input"],
Cell[99434, 1474, 130, 3, 27, "Output"]
}, Open  ]],
Cell[99579, 1480, 107, 3, 32, "Text"],
Cell[99689, 1485, 181, 3, 43, "Input"],

Cell[CellGroupData[{
Cell[99895, 1492, 63, 1, 27, "Input"],
Cell[99961, 1495, 183, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100181, 1503, 64, 1, 27, "Input"],
Cell[100248, 1506, 133, 2, 27, "Output"]
}, Open  ]],
Cell[100396, 1511, 119, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[100540, 1518, 242, 5, 59, "Input"],
Cell[100785, 1525, 750, 13, 65, "Output"]
}, Open  ]],
Cell[101550, 1541, 52, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[101627, 1545, 536, 11, 91, "Input"],
Cell[102166, 1558, 156, 3, 20, "Message"],
Cell[102325, 1563, 736, 13, 65, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103098, 1581, 996, 19, 155, "Input"],
Cell[104097, 1602, 1052, 26, 43, "Output"]
}, Open  ]],
Cell[105164, 1631, 132, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[105321, 1638, 154, 3, 27, "Input"],
Cell[105478, 1643, 75, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105590, 1649, 155, 3, 27, "Input"],
Cell[105748, 1654, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105899, 1661, 882, 14, 171, "Input"],
Cell[106784, 1677, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[106951, 1685, 160, 3, 27, "Input"],
Cell[107114, 1690, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107264, 1697, 533, 9, 107, "Input"],
Cell[107800, 1708, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[107967, 1716, 91, 1, 27, "Input"],
Cell[108061, 1719, 981, 15, 91, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[109079, 1739, 1016, 17, 139, "Input"],
Cell[110098, 1758, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[110265, 1766, 991, 17, 139, "Input"],
Cell[111259, 1785, 130, 3, 27, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[111438, 1794, 28, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[111491, 1798, 140, 2, 112, "Input"],
Cell[111634, 1802, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[111801, 1810, 140, 2, 112, "Input"],
Cell[111944, 1814, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[112111, 1822, 137, 2, 112, "Input"],
Cell[112251, 1826, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[112418, 1834, 534, 10, 412, "Input"],
Cell[112955, 1846, 130, 3, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[113122, 1854, 136, 2, 112, "Input"],
Cell[113261, 1858, 130, 3, 52, "Output"]
}, Open  ]],
Cell[113406, 1864, 208, 4, 124, "Text"],

Cell[CellGroupData[{
Cell[113639, 1872, 78, 1, 52, "Input"],
Cell[113720, 1875, 135, 3, 52, "Output"]
}, Open  ]],
Cell[113870, 1881, 136, 3, 92, "Text"],

Cell[CellGroupData[{
Cell[114031, 1888, 419, 6, 292, "Input"],
Cell[114453, 1896, 56, 1, 44, "Print"],
Cell[114512, 1899, 57, 1, 44, "Print"],
Cell[114572, 1902, 57, 1, 44, "Print"],
Cell[114632, 1905, 57, 1, 44, "Print"],
Cell[114692, 1908, 57, 1, 44, "Print"],
Cell[114752, 1911, 57, 1, 44, "Print"],
Cell[114812, 1914, 57, 1, 44, "Print"],
Cell[114872, 1917, 57, 1, 44, "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[114978, 1924, 35, 0, 34, "Section"],
Cell[115016, 1926, 108, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[115149, 1933, 562, 13, 139, "Input"],
Cell[115714, 1948, 181, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[115932, 1956, 330, 6, 91, "Input"],
Cell[116265, 1964, 66, 1, 27, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

