(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    942898,      14397]*)
(*NotebookOutlinePosition[    944234,      14438]*)
(*  CellTagsIndexPosition[    944190,      14434]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Spectra for 2+1 transverse lattice", "Subtitle"],

Cell["\<\
Brett van de Sande
August 2000
(This is a modified version of the 3+1 file)\
\>", "Subsubtitle"],

Cell["\<\
All of the following are for spectra computed in 2000.  These all \
have the new form of the tau-interactions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(cross[x_, y_, z_] := {Line[{{x, y - z}, {x, y + z}}], 
        Line[{{x - 0.1, y}, {x + 0.1, y}}]}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(cross\)\" is similar to \
existing symbol \"\!\(Cross\)\"."\)], "Message"]
}, Open  ]],

Cell[BoxData[
    \(<< Graphics`Graphics3D`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(<< Statistics`LinearRegression`\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Exit[]\)], "Input"],

Cell[BoxData[
    \(data = 
      Block[\n\t\t{x, fp = OpenRead["\<~/dave/junk2.out\>"], 
          info}, \n\t\tinfo = 
          Read[fp, Record, RecordSeparators -> "\<\n\n\>"]; \n\t\tx = 
          ReadList[fp, Number, 
            RecordLists -> True, \n\t\t\tRecordSeparators -> "\<\n\n\>"]; 
        Close[fp]; \n\t\t{info, Chop[\ x]}\n\ ]\)], "Input"],

Cell[BoxData[
    \(data[\([2]\)]\)], "Input"],

Cell[BoxData[
    \(\(d3 = data;\)\)], "Input"],

Cell[BoxData[
    \(\(d5 = data;\)\)], "Input"],

Cell[BoxData[
    \(data = {d3[\([1]\)], 
        Table[If[d3[\([2, i, 3]\)] < d5[\([2, i, 3]\)], d3[\([2, i]\)], 
            d5[\([2, i]\)]], {i, Length[d3[\([2]\)]]}]}\)], "Input"],

Cell[CellGroupData[{

Cell["Scaling trajectory", "Section"],

Cell[CellGroupData[{

Cell["data btraj1*.out", "Subsection"],

Cell[BoxData[
    \(\(data = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from minimizing \
chi^2.\n2 parity doublets with fractional errors 1 0.1.\nSpectrum for P_perp \
a = (0)  ( 0.25) \n using (# states, o, multiplet, c^2 error for each) =\n  \
(4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & 1, 0.25 0.25 1 1).\n\
Spectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  (26/2,6)  (32/2,6) \
.\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( 2,20/2,6) \n  ( 2,24/2,4) \
 ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \n  ( 3,35/2,5)  ( \
4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy potential \
determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  ( \
1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{1, 63, 6.370164754`, \(-1.715868283`\), 7.572460367`, 
              0, 1, \(-0.00388139087`\), \(-0.114934935`\), 
              2421.240119`, \(-0.6625789889`\), \(-0.6704465208`\), 2, 3, 4, 
              5, 6, 0.191058`, \(-0.421972`\), 0.252975`, 0.132542`, 
              0.234986`, \(-0.224797`\), 0.621269`, 0.605777`, 0.710037`, 
              0.730573`, 1.004086`, 0.99226`, 17.989366`, 18.034693`, 
              1.049257`, 28.320188`, 28.290514`, \(-0.686909`\), 35.825838`, 
              35.85182`, 0.601441`, 46.630916`, 46.6384`, 0.17324`, 
              24.061804`, 24.094341`, 0.753176`, 45.992042`, 46.029674`, 
              0.871142`, 60.250317`, 60.254106`, 0.087695`, 64.082798`, 
              64.117432`, 0.801732`, 17.989366`, 28.320188`, 35.825838`, 
              52.046823`, 60.250317`, 77.265373`, 91.82883`, 98.060247`, 
              24.061804`, 45.992042`, 64.082798`, 66.978856`, 46.630916`, 
              67.777089`, 75.335254`, 83.222567`}, {1, 51, 
              6.546390059`, \(-1.715868283`\), 7.572460367`, 0, 
              1, \(-0.005773821378`\), \(-0.1045757157`\), 4213.671756`, 
              1.110502258`, \(-0.8343633171`\), 2, 3, 4, 5, 6, 
              0.191058`, \(-0.421972`\), 0.252975`, 0.132542`, 
              0.234986`, \(-0.224797`\), 0.621269`, 0.605777`, 0.710037`, 
              0.730573`, 1.004086`, 0.99226`, 17.989366`, 18.034693`, 
              1.049257`, 28.320188`, 28.290514`, \(-0.686909`\), 35.825838`, 
              35.85182`, 0.601441`, 46.630916`, 46.6384`, 0.17324`, 
              24.061804`, 24.094341`, 0.753176`, 45.992042`, 46.029674`, 
              0.871142`, 60.250317`, 60.254106`, 0.087695`, 64.082798`, 
              64.117432`, 0.801732`, 17.989366`, 28.320188`, 35.825838`, 
              52.046823`, 60.250317`, 77.265373`, 91.82883`, 98.060247`, 
              24.061804`, 45.992042`, 64.082798`, 66.978856`, 46.630916`, 
              67.777089`, 75.335254`, 83.222567`}, {3, 36, 
              7.221793145`, \(-1.460665053`\), 7.347981832`, 0.001967542671`, 
              1, \(-0.05204096037`\), \(-0.108108538`\), 
              1047.692376`, \(-0.7368299707`\), \(-0.774257767`\), 2, 3, 4, 
              5, 6, 0.221422`, \(-0.447331`\), 0.286189`, 0.136835`, 
              0.222056`, \(-0.211328`\), 0.656068`, 0.634063`, 0.735942`, 
              0.75637`, 1.031029`, 1.019126`, 17.612442`, 17.652802`, 
              1.050644`, 27.074878`, 27.044137`, \(-0.800257`\), 36.3845`, 
              36.40821`, 0.617233`, 46.785291`, 46.790898`, 0.145967`, 
              25.206115`, 25.235088`, 0.754215`, 47.328836`, 47.360841`, 
              0.833177`, 61.406704`, 61.40516`, \(-0.04021`\), 64.772982`, 
              64.792092`, 0.497494`, 17.612442`, 27.074878`, 36.3845`, 
              51.815035`, 61.406704`, 78.208694`, 91.839317`, 101.709321`, 
              25.206115`, 47.328836`, 64.772982`, 66.722226`, 46.785291`, 
              66.537981`, 76.068306`, 78.110084`}, {2, 36, 
              7.409986269`, \(-1.460665053`\), 7.347981832`, 0.001967542671`, 
              1, \(-0.05140513834`\), \(-0.0989481855`\), 801.0438393`, 
              1.168299077`, \(-0.9282915879`\), 2, 3, 4, 5, 6, 
              0.221422`, \(-0.447331`\), 0.286189`, 0.136835`, 
              0.222056`, \(-0.211328`\), 0.656068`, 0.634063`, 0.735942`, 
              0.75637`, 1.031029`, 1.019126`, 17.612442`, 17.652802`, 
              1.050644`, 27.074878`, 27.044137`, \(-0.800257`\), 36.3845`, 
              36.40821`, 0.617233`, 46.785291`, 46.790898`, 0.145967`, 
              25.206115`, 25.235088`, 0.754215`, 47.328836`, 47.360841`, 
              0.833177`, 61.406704`, 61.40516`, \(-0.04021`\), 64.772982`, 
              64.792092`, 0.497494`, 17.612442`, 27.074878`, 36.3845`, 
              51.815035`, 61.406704`, 78.208694`, 91.839317`, 101.709321`, 
              25.206115`, 47.328836`, 64.772982`, 66.722226`, 46.785291`, 
              66.537981`, 76.068306`, 78.110084`}, {2, 44, 
              8.017525812`, \(-1.357983891`\), 7.2635494`, 0.007919222016`, 
              1, \(-0.0858035476`\), \(-0.1057199312`\), 
              3285.646128`, \(-0.8568898887`\), \(-0.8971987453`\), 2, 3, 4, 
              5, 6, 0.254921`, \(-0.464866`\), 0.301875`, 0.138571`, 
              0.208906`, \(-0.19523`\), 0.688086`, 0.663858`, 0.756309`, 
              0.779953`, 1.050452`, 1.036666`, 18.720482`, 18.755829`, 
              1.047172`, 27.879574`, 27.851213`, \(-0.840241`\), 37.726643`, 
              37.747287`, 0.611613`, 49.113923`, 49.118106`, 0.123932`, 
              26.66611`, 26.691219`, 0.743865`, 50.363616`, 50.391844`, 
              0.836271`, 64.387418`, 64.380325`, \(-0.210119`\), 66.391154`, 
              66.398824`, 0.227226`, 18.720482`, 27.879574`, 37.726643`, 
              54.304682`, 64.387418`, 81.433187`, 96.59497`, 105.615107`, 
              26.66611`, 50.363616`, 66.391154`, 68.628022`, 49.113923`, 
              69.865748`, 79.109677`, 81.634381`}, {\(-2\), 38, 
              8.186556211`, \(-1.357983891`\), 7.2635494`, 0.007919222016`, 
              1, \(-0.08546984463`\), \(-0.09958038171`\), 3161.643744`, 
              1.253247934`, \(-1.030973626`\), 2, 3, 4, 5, 6, 
              0.254921`, \(-0.464866`\), 0.301875`, 0.138571`, 
              0.208906`, \(-0.19523`\), 0.688086`, 0.663858`, 0.756309`, 
              0.779953`, 1.050452`, 1.036666`, 18.720482`, 18.755829`, 
              1.047172`, 27.879574`, 27.851213`, \(-0.840241`\), 37.726643`, 
              37.747287`, 0.611613`, 49.113923`, 49.118106`, 0.123932`, 
              26.66611`, 26.691219`, 0.743865`, 50.363616`, 50.391844`, 
              0.836271`, 64.387418`, 64.380325`, \(-0.210119`\), 66.391154`, 
              66.398824`, 0.227226`, 18.720482`, 27.879574`, 37.726643`, 
              54.304682`, 64.387418`, 81.433187`, 96.59497`, 105.615107`, 
              26.66611`, 50.363616`, 66.391154`, 68.628022`, 49.113923`, 
              69.865748`, 79.109677`, 81.634381`}, {2, 38, 
              8.532763521`, \(-1.557054463`\), 7.848962264`, 0.01800590693`, 
              1, \(-0.1034342375`\), \(-0.08832654646`\), 
              3476.586169`, \(-1.137693126`\), \(-1.109529497`\), 2, 3, 4, 5, 
              6, 0.307121`, \(-0.457813`\), 0.26089`, 0.127282`, 
              0.20975`, \(-0.188951`\), 0.724092`, 0.719958`, 0.766236`, 
              0.810664`, 1.05019`, 1.024767`, 25.759382`, 25.786281`, 
              1.037443`, 34.22855`, 34.206917`, \(-0.834357`\), 43.141544`, 
              43.159863`, 0.706534`, 59.517802`, 59.520144`, 0.090316`, 
              31.09892`, 31.117991`, 0.735543`, 61.558903`, 61.582864`, 
              0.924166`, 75.001228`, 74.99126`, \(-0.384482`\), 76.238712`, 
              76.242954`, 0.163625`, 25.759382`, 34.22855`, 43.141544`, 
              68.980143`, 76.238712`, 95.297004`, 116.076143`, 122.326015`, 
              31.09892`, 61.558903`, 75.001228`, 79.30121`, 59.517802`, 
              85.415761`, 91.947551`, 97.616199`}, {2, 49, 
              8.529634699`, \(-1.567106314`\), 7.90918129`, 0.01800590693`, 
              1, \(-0.1033621529`\), \(-0.0868150849`\), 1880.125265`, 
              1.472924885`, \(-1.208273142`\), 2, 3, 4, 5, 6, 
              0.308015`, \(-0.457542`\), 0.26057`, 0.12734`, 
              0.170763`, \(-0.143708`\), 0.724233`, 0.696985`, 0.747814`, 
              0.785575`, 1.018132`, 0.996409`, 26.072937`, 26.099428`, 
              1.032584`, 34.514738`, 34.493465`, \(-0.829202`\), 43.478458`, 
              43.496866`, 0.717494`, 60.053998`, 60.056283`, 0.089081`, 
              31.356953`, 31.375895`, 0.738332`, 62.140839`, 62.164687`, 
              0.929583`, 75.598597`, 75.589142`, \(-0.368533`\), 76.89382`, 
              76.896534`, 0.105783`, 26.072937`, 34.514738`, 43.478458`, 
              69.694412`, 76.89382`, 96.103651`, 117.0513`, 123.348179`, 
              31.356953`, 62.140839`, 75.598597`, 79.939992`, 60.053998`, 
              86.133812`, 92.711888`, 98.437401`}, {2, 47, 
              8.161369264`, \(-1.20889115`\), 7.295667415`, 0.03249196962`, 
              1, \(-0.1532867816`\), \(-0.109567404`\), 
              1442.774385`, \(-1.156569305`\), \(-1.181045074`\), 2, 3, 4, 5, 
              6, 0.326617`, \(-0.477225`\), 0.28069`, 0.138145`, 
              0.187048`, \(-0.167782`\), 0.75009`, 0.722802`, 0.790862`, 
              0.824134`, 1.079823`, 1.060647`, 21.786049`, 21.813101`, 
              1.031424`, 32.181861`, 32.159619`, \(-0.847971`\), 41.577104`, 
              41.592885`, 0.601682`, 56.294229`, 56.296876`, 0.100915`, 
              30.30174`, 30.321386`, 0.749039`, 58.635157`, 58.659809`, 
              0.939907`, 71.500195`, 71.492152`, \(-0.306632`\), 73.055395`, 
              73.058191`, 0.106608`, 21.786049`, 32.181861`, 41.577104`, 
              62.794966`, 73.055395`, 91.005187`, 111.767942`, 116.379385`, 
              30.30174`, 58.635157`, 71.500195`, 75.04742`, 56.294229`, 
              80.132889`, 88.133033`, 93.174586`}, {2, 47, 
              8.281041052`, \(-1.20889115`\), 7.295667415`, 0.03249196962`, 
              1, \(-0.1526984279`\), \(-0.1079790789`\), 650.1533596`, 
              1.474628783`, \(-1.266133686`\), 2, 3, 4, 5, 6, 
              0.326617`, \(-0.477225`\), 0.28069`, 0.138145`, 
              0.187048`, \(-0.167782`\), 0.75009`, 0.722802`, 0.790862`, 
              0.824134`, 1.079823`, 1.060647`, 21.786049`, 21.813101`, 
              1.031424`, 32.181861`, 32.159619`, \(-0.847971`\), 41.577104`, 
              41.592885`, 0.601682`, 56.294229`, 56.296876`, 0.100915`, 
              30.30174`, 30.321386`, 0.749039`, 58.635157`, 58.659809`, 
              0.939907`, 71.500195`, 71.492152`, \(-0.306632`\), 73.055395`, 
              73.058191`, 0.106608`, 21.786049`, 32.181861`, 41.577104`, 
              62.794966`, 73.055395`, 91.005187`, 111.767942`, 116.379385`, 
              30.30174`, 58.635157`, 71.500195`, 75.04742`, 56.294229`, 
              80.132889`, 88.133033`, 93.174586`}, {2, 30, 
              8.188396598`, \(-0.9934829794`\), 6.767551048`, 0.0517766953`, 
              1, \(-0.2022680316`\), \(-0.1324153951`\), 
              666.6271335`, \(-1.162711072`\), \(-1.246086125`\), 2, 3, 4, 5, 
              6, 0.349526`, \(-0.49635`\), 0.293483`, 0.150273`, 
              0.162142`, \(-0.144587`\), 0.777596`, 0.727252`, 0.818646`, 
              0.840464`, 1.114641`, 1.101879`, 18.671907`, 18.699074`, 
              1.028168`, 30.710798`, 30.688107`, \(-0.858815`\), 39.997697`, 
              40.011167`, 0.509789`, 53.731891`, 53.734721`, 0.107081`, 
              29.471583`, 29.490971`, 0.733752`, 56.22639`, 56.251475`, 
              0.949378`, 68.081455`, 68.075142`, \(-0.238956`\), 70.288543`, 
              70.292055`, 0.132898`, 18.671907`, 30.710798`, 39.997697`, 
              58.721759`, 70.288543`, 87.209028`, 108.023802`, 111.181342`, 
              29.471583`, 56.22639`, 68.081455`, 71.287785`, 53.731891`, 
              73.571182`, 87.13242`, 89.52469`}, {2, 42, 
              8.382298802`, \(-0.9934829794`\), 6.767551048`, 0.0517766953`, 
              1, \(-0.2022450579`\), \(-0.131541675`\), 528.1516904`, 
              1.457808888`, \(-1.314471419`\), 2, 3, 4, 5, 6, 
              0.349526`, \(-0.49635`\), 0.293483`, 0.150273`, 
              0.162142`, \(-0.144587`\), 0.777596`, 0.727252`, 0.818646`, 
              0.840464`, 1.114641`, 1.101879`, 18.671907`, 18.699074`, 
              1.028168`, 30.710798`, 30.688107`, \(-0.858815`\), 39.997697`, 
              40.011167`, 0.509789`, 53.731891`, 53.734721`, 0.107081`, 
              29.471583`, 29.490971`, 0.733752`, 56.22639`, 56.251475`, 
              0.949378`, 68.081455`, 68.075142`, \(-0.238956`\), 70.288543`, 
              70.292055`, 0.132898`, 18.671907`, 30.710798`, 39.997697`, 
              58.721759`, 70.288543`, 87.209028`, 108.023802`, 111.181342`, 
              29.471583`, 56.22639`, 68.081455`, 71.287785`, 53.731891`, 
              73.571182`, 87.13242`, 89.52469`}, {2, 27, 
              9.20918081`, \(-0.9395667301`\), 6.147348633`, 0.07642881742`, 
              1, \(-0.2424623993`\), \(-0.1545038879`\), 
              101.907866`, \(-1.023571927`\), \(-1.133129755`\), 2, 3, 4, 5, 
              6, 0.384951`, \(-0.488521`\), 0.237572`, 0.173734`, 
              0.126901`, \(-0.11831`\), 0.81083`, 0.758519`, 0.88324`, 
              0.893256`, 1.210532`, 1.200644`, 18.097221`, 18.124007`, 
              1.014215`, 30.610661`, 30.588253`, \(-0.848446`\), 37.854116`, 
              37.86489`, 0.407955`, 52.247156`, 52.24979`, 0.099728`, 
              28.328111`, 28.345199`, 0.646971`, 54.673758`, 54.697907`, 
              0.914353`, 63.974027`, 63.973408`, \(-0.023423`\), 67.901972`, 
              67.907362`, 0.204071`, 18.097221`, 30.610661`, 37.854116`, 
              57.761389`, 67.901972`, 83.56743`, 96.715087`, 107.402173`, 
              28.328111`, 54.673758`, 63.974027`, 68.050695`, 52.247156`, 
              73.778541`, 78.491491`, 89.691806`}, {2, 46, 
              9.321543729`, \(-0.9395667301`\), 6.147348633`, 0.07642881742`, 
              1, \(-0.2399057092`\), \(-0.1531785757`\), 181.5301198`, 
              1.346187518`, \(-1.203156602`\), 2, 3, 4, 5, 6, 
              0.384951`, \(-0.488521`\), 0.237572`, 0.173734`, 
              0.126901`, \(-0.11831`\), 0.81083`, 0.758519`, 0.88324`, 
              0.893256`, 1.210532`, 1.200644`, 18.097221`, 18.124007`, 
              1.014215`, 30.610661`, 30.588253`, \(-0.848446`\), 37.854116`, 
              37.86489`, 0.407955`, 52.247156`, 52.24979`, 0.099728`, 
              28.328111`, 28.345199`, 0.646971`, 54.673758`, 54.697907`, 
              0.914353`, 63.974027`, 63.973408`, \(-0.023423`\), 67.901972`, 
              67.907362`, 0.204071`, 18.097221`, 30.610661`, 37.854116`, 
              57.761389`, 67.901972`, 83.56743`, 96.715087`, 107.402173`, 
              28.328111`, 54.673758`, 63.974027`, 68.050695`, 52.247156`, 
              73.778541`, 78.491491`, 89.691806`}, {2, 7, 
              9.604729214`, \(-0.3926699882`\), 5.640507282`, 0.1453085056`, 
              1, \(-0.4133270229`\), \(-0.2157804044`\), 
              50, \(-1.086034176`\), \(-1.479585058`\), 2, 3, 4, 5, 6, 
              0.412995`, \(-0.758464`\), 0.68288`, 0.18718`, 
              0.074139`, \(-0.048847`\), 0.861536`, 0.739166`, 0.898897`, 
              0.887269`, 1.215623`, 1.222205`, 3.246196`, 3.27193`, 0.95915`, 
              23.220179`, 23.197906`, \(-0.830184`\), 35.777006`, 35.785802`, 
              0.327835`, 44.784104`, 44.805578`, 0.80038`, 27.860273`, 
              27.879908`, 0.731832`, 48.502349`, 48.533102`, 1.146194`, 
              60.27355`, 60.254222`, \(-0.720422`\), 60.312602`, 
              60.302677`, \(-0.369906`\), 3.246196`, 23.220179`, 35.777006`, 
              44.784104`, 62.385767`, 77.460306`, 85.802128`, 96.682957`, 
              27.860273`, 48.502349`, 60.27355`, 60.312602`, 45.16946`, 
              65.649594`, 61.304792`, 72.213506`}, {2, 51, 
              9.611153005`, \(-0.3926699882`\), 5.640507282`, 0.1453085056`, 
              1, \(-0.413244753`\), \(-0.2167825992`\), 17.64823786`, 
              1.288822823`, \(-1.494785695`\), 2, 3, 4, 5, 6, 
              0.412995`, \(-0.758464`\), 0.68288`, 0.18718`, 
              0.074139`, \(-0.048847`\), 0.861536`, 0.739166`, 0.898897`, 
              0.887269`, 1.215623`, 1.222205`, 3.246196`, 3.27193`, 0.95915`, 
              23.220179`, 23.197906`, \(-0.830184`\), 35.777006`, 35.785802`, 
              0.327835`, 44.784104`, 44.805578`, 0.80038`, 27.860273`, 
              27.879908`, 0.731832`, 48.502349`, 48.533102`, 1.146194`, 
              60.27355`, 60.254222`, \(-0.720422`\), 60.312602`, 
              60.302677`, \(-0.369906`\), 3.246196`, 23.220179`, 35.777006`, 
              44.784104`, 62.385767`, 77.460306`, 85.802128`, 96.682957`, 
              27.860273`, 48.502349`, 60.27355`, 60.312602`, 45.16946`, 
              65.649594`, 61.304792`, 72.213506`}, {2, 41, 
              16.43896473`, \(-0.8415201463`\), 6.962079971`, 0.1921681542`, 
              1, \(-0.4316287667`\), \(-0.1635591678`\), 
              228.1681143`, \(-1.903696669`\), \(-1.969498337`\), 2, 3, 4, 5, 
              6, 0.549414`, \(-0.395728`\), \(-0.079719`\), 0.147677`, 
              0.123179`, \(-0.102266`\), 0.925525`, 0.865442`, 0.906924`, 
              0.952144`, 1.196139`, 1.170067`, 24.585045`, 24.601148`, 
              0.985493`, 45.96241`, 45.94267`, \(-1.208094`\), 49.496877`, 
              49.511166`, 0.874489`, 73.680105`, 73.681553`, 0.088607`, 
              38.68326`, 38.695451`, 0.746102`, 79.593277`, 79.607891`, 
              0.89439`, 79.648443`, 79.663367`, 0.913341`, 89.355917`, 
              89.329278`, \(-1.630309`\), 24.585045`, 45.96241`, 49.496877`, 
              81.040886`, 94.324494`, 112.852671`, 137.798416`, 139.779796`, 
              38.68326`, 79.648443`, 79.593277`, 89.355917`, 73.680105`, 
              100.909358`, 110.110061`, 111.67864`}, {2, 7, 
              17.41253947`, \(-0.8415201463`\), 6.962079971`, 0.1921681542`, 
              1, \(-0.4302997803`\), \(-0.1602216036`\), 50, 
              2.034154234`, \(-1.914967656`\), 2, 3, 4, 5, 6, 
              0.549414`, \(-0.395728`\), \(-0.079719`\), 0.147677`, 
              0.123179`, \(-0.102266`\), 0.925525`, 0.865442`, 0.906924`, 
              0.952144`, 1.196139`, 1.170067`, 24.585045`, 24.601148`, 
              0.985493`, 45.96241`, 45.94267`, \(-1.208094`\), 49.496877`, 
              49.511166`, 0.874489`, 73.680105`, 73.681553`, 0.088607`, 
              38.68326`, 38.695451`, 0.746102`, 79.593277`, 79.607891`, 
              0.89439`, 79.648443`, 79.663367`, 0.913341`, 89.355917`, 
              89.329278`, \(-1.630309`\), 24.585045`, 45.96241`, 49.496877`, 
              81.040886`, 94.324494`, 112.852671`, 137.798416`, 139.779796`, 
              38.68326`, 79.648443`, 79.593277`, 89.355917`, 73.680105`, 
              100.909358`, 110.110061`, 111.67864`}, {2, 9, 
              257846.1212`, \(-1.420957265`\), \(-0.1590134405`\), 0.25`, 
              1, \(-0.3652185139`\), \(-0.1797844676`\), 3.124025944`, 
              50, \(-1.472275693`\), 2, 3, 4, 5, 6, 
              0.689346`, \(-0.303652`\), \(-0.357552`\), \(-6.307285`\), \
\(-2.945815`\), 4.868702`, 1.000262`, 33.098904`, 1.077941`, \(-16.791228`\), 
              1.450455`, \(-33.51983`\), \(-0.632193`\), \(-0.633002`\), 
              0.001419`, \(-1.287067`\), \(-1.287456`\), 
              0.000683`, \(-1.521486`\), \(-1.521894`\), 
              0.000715`, \(-1.537392`\), \(-1.536973`\), \(-0.000736`\), \
\(-0.992559`\), \(-0.992747`\), 0.000331`, \(-1.980341`\), \(-1.980461`\), 
              0.00021`, \(-2.294707`\), \(-2.295117`\), 
              0.00072`, \(-2.576166`\), \(-2.575687`\), \(-0.00084`\), \
\(-0.632193`\), \(-1.287067`\), \(-1.521486`\), \(-1.537392`\), \
\(-2.641834`\), \(-3.076166`\), \(-3.727931`\), \(-3.760525`\), \
\(-0.992559`\), \(-1.980341`\), \(-2.294707`\), \(-2.576166`\), \
\(-2.203394`\), \(-2.862011`\), \(-3.158433`\), \(-3.335422`\)}, {2, 22, 
              258144.039`, \(-1.420957265`\), \(-0.1590134405`\), 0.25`, 
              1, \(-0.3890014391`\), \(-0.1778149707`\), \(-0.349779597`\), 
              49.91178637`, \(-2.188430419`\), 2, 3, 4, 5, 6, 
              0.689346`, \(-0.303652`\), \(-0.357552`\), \(-6.307285`\), \
\(-2.945815`\), 4.868702`, 1.000262`, 33.098904`, 1.077941`, \(-16.791228`\), 
              1.450455`, \(-33.51983`\), \(-0.632193`\), \(-0.633002`\), 
              0.001419`, \(-1.287067`\), \(-1.287456`\), 
              0.000683`, \(-1.521486`\), \(-1.521894`\), 
              0.000715`, \(-1.537392`\), \(-1.536973`\), \(-0.000736`\), \
\(-0.992559`\), \(-0.992747`\), 0.000331`, \(-1.980341`\), \(-1.980461`\), 
              0.00021`, \(-2.294707`\), \(-2.295117`\), 
              0.00072`, \(-2.576166`\), \(-2.575687`\), \(-0.00084`\), \
\(-0.632193`\), \(-1.287067`\), \(-1.521486`\), \(-1.537392`\), \
\(-2.641834`\), \(-3.076166`\), \(-3.727931`\), \(-3.760525`\), \
\(-0.992559`\), \(-1.980341`\), \(-2.294707`\), \(-2.576166`\), \
\(-2.203394`\), \(-2.862011`\), \(-3.158433`\), \(-3.335422`\)}}};\)\)], \
"Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data dtraj1*.out", "Subsection"],

Cell[BoxData[
    \(\(data = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from fitting \
lowest state to lattice value.\n2 parity doublets with fractional errors 1 \
0.1.\nSpectrum for P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, \
c^2 error for each) =\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & \
1, 0.25 0.25 1 1).\nSpectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  \
(26/2,6)  (32/2,6) .\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( \
2,20/2,6) \n  ( 2,24/2,4)  ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \
\n  ( 3,35/2,5)  ( 4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy \
potential determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  \
( 1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{2, 35, 6.463757838`, \(-1.586701318`\), 7.388765066`, 
              0, 1, \(-0.006118468428`\), \(-0.1223884029`\), 
              1328.031904`, \(-0.6191892896`\), \(-0.6448791526`\), 2, 3, 4, 
              5, 6, 0.185084`, \(-0.42534`\), 0.259678`, 0.136324`, 
              0.232599`, \(-0.226242`\), 0.619684`, 0.594766`, 0.713417`, 
              0.727959`, 1.010486`, 1.002122`, 16.5242`, 16.572037`, 
              1.046701`, 27.380553`, 27.349783`, \(-0.673259`\), 34.840339`, 
              34.866382`, 0.569836`, 44.774649`, 44.782409`, 0.169787`, 
              23.311982`, 23.346377`, 0.752579`, 44.035034`, 44.073939`, 
              0.851261`, 58.218027`, 58.22371`, 0.124351`, 62.284301`, 
              62.322227`, 0.82985`, 16.5242`, 27.380553`, 34.840339`, 
              49.042912`, 58.218027`, 74.724059`, 88.685196`, 95.269282`, 
              23.311982`, 44.035034`, 62.284301`, 64.985903`, 44.774649`, 
              65.467469`, 72.9797`, 79.566949`}, {5, 100, 
              6.652317964`, \(-1.586701318`\), 7.388765066`, 0, 
              1, \(-0.01939958609`\), \(-0.1120346005`\), 284.948311`, 
              1.048412263`, \(-0.7995526772`\), 2, 3, 4, 5, 6, 
              0.185084`, \(-0.42534`\), 0.259678`, 0.136324`, 
              0.232599`, \(-0.226242`\), 0.619684`, 0.594766`, 0.713417`, 
              0.727959`, 1.010486`, 1.002122`, 16.5242`, 16.572037`, 
              1.046701`, 27.380553`, 27.349783`, \(-0.673259`\), 34.840339`, 
              34.866382`, 0.569836`, 44.774649`, 44.782409`, 0.169787`, 
              23.311982`, 23.346377`, 0.752579`, 44.035034`, 44.073939`, 
              0.851261`, 58.218027`, 58.22371`, 0.124351`, 62.284301`, 
              62.322227`, 0.82985`, 16.5242`, 27.380553`, 34.840339`, 
              49.042912`, 58.218027`, 74.724059`, 88.685196`, 95.269282`, 
              23.311982`, 44.035034`, 62.284301`, 64.985903`, 44.774649`, 
              65.467469`, 72.9797`, 79.566949`}, {1, 56, 
              7.239690877`, \(-1.383769551`\), 7.207947949`, 0.001967542671`, 
              1, \(-0.05307870473`\), \(-0.1141404246`\), 
              2798.874143`, \(-0.7012994607`\), \(-0.7524379213`\), 2, 3, 4, 
              5, 6, 0.216659`, \(-0.451172`\), 0.293863`, 0.140002`, 
              0.219449`, \(-0.211372`\), 0.6549`, 0.626164`, 0.738868`, 
              0.75471`, 1.036536`, 1.027345`, 16.5242`, 16.566389`, 
              1.054161`, 26.394095`, 26.362216`, \(-0.796553`\), 35.613128`, 
              35.636899`, 0.593958`, 45.3781`, 45.384255`, 0.153795`, 
              24.599292`, 24.629529`, 0.755535`, 45.803578`, 45.836475`, 
              0.821997`, 59.814636`, 59.814706`, 0.001756`, 63.377198`, 
              63.396923`, 0.492859`, 16.5242`, 26.394095`, 35.613128`, 
              49.570474`, 59.814636`, 76.240965`, 89.49991`, 98.873036`, 
              24.599292`, 45.803578`, 63.377198`, 65.186077`, 45.3781`, 
              64.81508`, 73.468004`, 77.227451`}, {1, 101, 
              7.541365167`, \(-1.383769551`\), 7.207947949`, 0.001967542671`, 
              1, \(-0.05431675338`\), \(-0.1129367204`\), 161.3392068`, 
              1.089181049`, \(-0.8759773399`\), 2, 3, 4, 5, 6, 
              0.216659`, \(-0.451172`\), 0.293863`, 0.140002`, 
              0.219449`, \(-0.211372`\), 0.6549`, 0.626164`, 0.738868`, 
              0.75471`, 1.036536`, 1.027345`, 16.5242`, 16.566389`, 
              1.054161`, 26.394095`, 26.362216`, \(-0.796553`\), 35.613128`, 
              35.636899`, 0.593958`, 45.3781`, 45.384255`, 0.153795`, 
              24.599292`, 24.629529`, 0.755535`, 45.803578`, 45.836475`, 
              0.821997`, 59.814636`, 59.814706`, 0.001756`, 63.377198`, 
              63.396923`, 0.492859`, 16.5242`, 26.394095`, 35.613128`, 
              49.570474`, 59.814636`, 76.240965`, 89.49991`, 98.873036`, 
              24.599292`, 45.803578`, 63.377198`, 65.186077`, 45.3781`, 
              64.81508`, 73.468004`, 77.227451`}, {1, 38, 
              8.111494715`, \(-1.24167337`\), 6.927609382`, 0.007919222016`, 
              1, \(-0.08722034695`\), \(-0.1182137623`\), 
              13804.19238`, \(-0.7712155125`\), \(-0.844072778`\), 2, 3, 4, 
              5, 6, 0.24557`, \(-0.473244`\), 0.317899`, 0.146276`, 
              0.201536`, \(-0.193235`\), 0.686032`, 0.647412`, 0.763995`, 
              0.777242`, 1.064574`, 1.056856`, 16.5242`, 16.562934`, 
              1.054328`, 26.289197`, 26.258345`, \(-0.839767`\), 35.828446`, 
              35.848934`, 0.55768`, 45.937088`, 45.942135`, 0.137375`, 
              25.208832`, 25.235818`, 0.734528`, 46.90663`, 46.936194`, 
              0.804725`, 60.647863`, 60.644706`, \(-0.085926`\), 63.04669`, 
              63.054367`, 0.208952`, 16.5242`, 26.289197`, 35.828446`, 
              49.606937`, 60.647863`, 76.829715`, 91.117225`, 99.720468`, 
              25.208832`, 46.90663`, 63.04669`, 65.013161`, 45.937088`, 
              65.826893`, 74.642457`, 75.896437`}, {5, 105, 
              8.404935607`, \(-1.24167337`\), 6.927609382`, 0.007919222016`, 
              1, \(-0.08817501252`\), \(-0.117339114`\), 1198.521849`, 
              1.137612802`, \(-0.9523255824`\), 2, 3, 4, 5, 6, 
              0.24557`, \(-0.473244`\), 0.317899`, 0.146276`, 
              0.201536`, \(-0.193235`\), 0.686032`, 0.647412`, 0.763995`, 
              0.777242`, 1.064574`, 1.056856`, 16.5242`, 16.562934`, 
              1.054328`, 26.289197`, 26.258345`, \(-0.839767`\), 35.828446`, 
              35.848934`, 0.55768`, 45.937088`, 45.942135`, 0.137375`, 
              25.208832`, 25.235818`, 0.734528`, 46.90663`, 46.936194`, 
              0.804725`, 60.647863`, 60.644706`, \(-0.085926`\), 63.04669`, 
              63.054367`, 0.208952`, 16.5242`, 26.289197`, 35.828446`, 
              49.606937`, 60.647863`, 76.829715`, 91.117225`, 99.720468`, 
              25.208832`, 46.90663`, 63.04669`, 65.013161`, 45.937088`, 
              65.826893`, 74.642457`, 75.896437`}, {1, 69, 
              8.341489309`, \(-1.496426973`\), 7.058025858`, 0.01800590693`, 
              1, \(-0.1100994274`\), \(-0.08537929229`\), 
              2.019952108`, \(-0.9794565415`\), \(-0.9904684232`\), 2, 3, 4, 
              5, 6, 0.306315`, \(-0.461412`\), 0.271927`, 0.142445`, 
              0.194826`, \(-0.182518`\), 0.722984`, 0.704537`, 0.787083`, 
              0.816104`, 1.08671`, 1.069876`, 16.5242`, 16.554804`, 
              1.058647`, 25.831813`, 25.847171`, 0.531263`, 30.0951`, 
              30.076875`, \(-0.63045`\), 48.179144`, 48.203142`, 0.830135`, 
              27.945079`, 27.962181`, 0.591614`, 54.936827`, 54.958023`, 
              0.733228`, 67.330158`, 67.317602`, \(-0.434318`\), 68.114692`, 
              68.117218`, 0.087371`, 16.5242`, 25.831813`, 30.0951`, 
              48.179144`, 68.114692`, 85.217886`, 103.263654`, 109.474377`, 
              27.945079`, 54.936827`, 67.330158`, 70.719656`, 52.920059`, 
              75.548947`, 81.982874`, 87.241563`}, {2, 64, 
              7.759812946`, \(-1.33250848`\), 6.969134953`, 0.01800590693`, 
              1, \(-0.1204097705`\), \(-0.09335395852`\), 2.506091432`, 
              1.275189829`, \(-1.062890758`\), 2, 3, 4, 5, 6, 
              0.297292`, \(-0.471641`\), 0.297082`, 0.14619`, 
              0.15077`, \(-0.130738`\), 0.719948`, 0.671868`, 0.76968`, 
              0.786975`, 1.057988`, 1.048032`, 16.5242`, 16.555871`, 
              1.049876`, 25.603637`, 25.620072`, 0.544806`, 28.45381`, 
              28.433809`, \(-0.663048`\), 47.980106`, 48.0065`, 0.874959`, 
              27.402191`, 27.421168`, 0.629088`, 52.743663`, 52.766255`, 
              0.748929`, 66.026303`, 66.020247`, \(-0.200744`\), 66.089963`, 
              66.084043`, \(-0.196243`\), 16.5242`, 25.603637`, 28.45381`, 
              47.980106`, 66.026303`, 82.821256`, 99.617136`, 106.616358`, 
              27.402191`, 52.743663`, 66.089963`, 68.715935`, 50.711665`, 
              72.186759`, 79.42796`, 84.748288`}, {1, 63, 
              9.4115474`, \(-1.019658861`\), 6.830897546`, 0.03249196962`, 
              1, \(-0.1673942526`\), \(-0.1209521116`\), 
              9.419471443`, \(-1.022668024`\), \(-1.098150327`\), 2, 3, 4, 5, 
              6, 0.312142`, \(-0.503509`\), 0.340742`, 0.148937`, 
              0.174328`, \(-0.158249`\), 0.745152`, 0.701267`, 0.799576`, 
              0.819145`, 1.097323`, 1.085996`, 16.5242`, 16.554124`, 
              1.020854`, 28.091265`, 28.067613`, \(-0.806888`\), 35.515801`, 
              35.530583`, 0.504295`, 50.317778`, 50.320959`, 0.108508`, 
              28.033306`, 28.054764`, 0.732042`, 52.540276`, 52.565648`, 
              0.865575`, 66.250188`, 66.227542`, \(-0.772563`\), 66.441621`, 
              66.453898`, 0.418825`, 16.5242`, 28.091265`, 35.515801`, 
              52.804391`, 66.441621`, 83.148406`, 101.037201`, 106.653823`, 
              28.033306`, 52.540276`, 66.250188`, 68.531852`, 50.317778`, 
              71.040614`, 80.378896`, 85.059709`}, {1, 63, 
              9.6231671`, \(-1.019658861`\), 6.830897546`, 0.03249196962`, 
              1, \(-0.1680357685`\), \(-0.1199906121`\), 8.930726944`, 
              1.342931924`, \(-1.179533243`\), 2, 3, 4, 5, 6, 
              0.312142`, \(-0.503509`\), 0.340742`, 0.148937`, 
              0.174328`, \(-0.158249`\), 0.745152`, 0.701267`, 0.799576`, 
              0.819145`, 1.097323`, 1.085996`, 16.5242`, 16.554124`, 
              1.020854`, 28.091265`, 28.067613`, \(-0.806888`\), 35.515801`, 
              35.530583`, 0.504295`, 50.317778`, 50.320959`, 0.108508`, 
              28.033306`, 28.054764`, 0.732042`, 52.540276`, 52.565648`, 
              0.865575`, 66.250188`, 66.227542`, \(-0.772563`\), 66.441621`, 
              66.453898`, 0.418825`, 16.5242`, 28.091265`, 35.515801`, 
              52.804391`, 66.441621`, 83.148406`, 101.037201`, 106.653823`, 
              28.033306`, 52.540276`, 66.250188`, 68.531852`, 50.317778`, 
              71.040614`, 80.378896`, 85.059709`}, {1, 63, 
              8.300556925`, \(-0.9048407402`\), 6.663942561`, 0.0517766953`, 
              1, \(-0.2097078591`\), \(-0.1388002796`\), 
              1425.934394`, \(-1.115421576`\), \(-1.217696578`\), 2, 3, 4, 5, 
              6, 0.34115`, \(-0.515685`\), 0.335753`, 0.153619`, 
              0.157164`, \(-0.139513`\), 0.774798`, 0.71925`, 0.820095`, 
              0.837648`, 1.118621`, 1.108539`, 16.5242`, 16.552588`, 
              1.032611`, 29.060814`, 29.036915`, \(-0.869315`\), 39.159538`, 
              39.173223`, 0.497783`, 51.578301`, 51.581464`, 0.11506`, 
              28.820904`, 28.84156`, 0.751338`, 54.038504`, 54.064413`, 
              0.942446`, 66.657148`, 66.646134`, \(-0.40062`\), 68.10033`, 
              68.106625`, 0.228984`, 16.5242`, 29.060814`, 39.159538`, 
              55.197298`, 68.10033`, 84.728219`, 104.302178`, 108.18104`, 
              28.820904`, 54.038504`, 66.657148`, 69.19293`, 51.578301`, 
              69.916344`, 84.799791`, 86.95024`}, {5, 100, 
              8.520252758`, \(-0.9038335306`\), 6.678375773`, 0.0517766953`, 
              1, \(-0.2099653814`\), \(-0.1384837461`\), 599.154607`, 
              1.408231914`, \(-1.281339115`\), 2, 3, 4, 5, 6, 
              0.341208`, \(-0.51591`\), 0.336465`, 0.154459`, 
              0.129551`, \(-0.108639`\), 0.774754`, 0.704255`, 0.80897`, 
              0.821772`, 1.099323`, 1.091968`, 16.5242`, 16.552524`, 
              1.032672`, 29.096244`, 29.072412`, \(-0.868903`\), 39.21994`, 
              39.233635`, 0.499311`, 51.670182`, 51.673339`, 0.115118`, 
              28.882929`, 28.903604`, 0.753788`, 54.145343`, 54.171272`, 
              0.94536`, 66.794338`, 66.783184`, \(-0.406652`\), 68.233054`, 
              68.239051`, 0.218672`, 16.5242`, 29.096244`, 39.21994`, 
              55.268928`, 68.233054`, 84.896197`, 104.515852`, 108.368307`, 
              28.882929`, 54.145343`, 66.794338`, 69.318625`, 51.670182`, 
              70.010544`, 84.966418`, 87.11757`}, {2, 46, 
              9.332606961`, \(-0.9032653951`\), 6.038734053`, 0.07642881742`, 
              1, \(-0.2457236507`\), \(-0.1582036385`\), 
              13.76009681`, \(-1.020672186`\), \(-1.17477871`\), 2, 3, 4, 5, 
              6, 0.38074`, \(-0.497446`\), 0.256279`, 0.173686`, 
              0.12641`, \(-0.117167`\), 0.809578`, 0.742145`, 0.874785`, 
              0.879646`, 1.196608`, 1.190676`, 16.5242`, 16.552468`, 
              1.039882`, 29.580266`, 29.557503`, \(-0.837379`\), 35.352203`, 
              35.362595`, 0.382287`, 50.770367`, 50.773093`, 0.100261`, 
              27.741169`, 27.758585`, 0.640658`, 53.140517`, 53.164808`, 
              0.893566`, 62.707317`, 62.706009`, \(-0.048114`\), 66.230924`, 
              66.238472`, 0.277679`, 16.5242`, 29.580266`, 35.352203`, 
              54.743833`, 66.230924`, 81.612361`, 93.952056`, 104.929212`, 
              27.741169`, 53.140517`, 62.707317`, 66.414511`, 50.770367`, 
              71.506664`, 76.527348`, 87.685913`}, {5, 103, 
              9.846042963`, \(-0.9032653951`\), 6.038734053`, 0.07642881742`, 
              1, \(-0.2446994621`\), \(-0.1565652944`\), 9.068504546`, 
              1.328164887`, \(-1.264474114`\), 2, 3, 4, 5, 6, 
              0.38074`, \(-0.497446`\), 0.256279`, 0.173686`, 
              0.12641`, \(-0.117167`\), 0.809578`, 0.742145`, 0.874785`, 
              0.879646`, 1.196608`, 1.190676`, 16.5242`, 16.552468`, 
              1.039882`, 29.580266`, 29.557503`, \(-0.837379`\), 35.352203`, 
              35.362595`, 0.382287`, 50.770367`, 50.773093`, 0.100261`, 
              27.741169`, 27.758585`, 0.640658`, 53.140517`, 53.164808`, 
              0.893566`, 62.707317`, 62.706009`, \(-0.048114`\), 66.230924`, 
              66.238472`, 0.277679`, 16.5242`, 29.580266`, 35.352203`, 
              54.743833`, 66.230924`, 81.612361`, 93.952056`, 104.929212`, 
              27.741169`, 53.140517`, 62.707317`, 66.414511`, 50.770367`, 
              71.506664`, 76.527348`, 87.685913`}, {5, 100, 
              11.70458489`, \(-0.7821342608`\), 6.360673415`, 0.1072401379`, 
              1, \(-0.3119820943`\), \(-0.164496886`\), 
              528.4818481`, \(-1.318213571`\), \(-1.469103933`\), 2, 3, 4, 5, 
              6, 0.41988`, \(-0.504512`\), 0.239424`, 0.16275`, 
              0.124816`, \(-0.10569`\), 0.841072`, 0.769966`, 0.869807`, 
              0.887436`, 1.173127`, 1.163093`, 16.5242`, 16.547691`, 
              1.003822`, 32.209406`, 32.189642`, \(-0.844522`\), 40.768135`, 
              40.778095`, 0.425623`, 55.64378`, 55.646252`, 0.105628`, 
              30.819212`, 30.836067`, 0.720261`, 58.932365`, 58.958675`, 
              1.124237`, 67.889033`, 67.887489`, \(-0.066002`\), 71.583424`, 
              71.562148`, \(-0.90917`\), 16.5242`, 32.209406`, 40.768135`, 
              60.430885`, 72.989269`, 89.445591`, 104.424399`, 111.969634`, 
              30.819212`, 58.932365`, 67.889033`, 71.583424`, 55.64378`, 
              79.346991`, 79.680021`, 95.792696`}, {5, 100, 
              11.87717384`, \(-0.7821342608`\), 6.360673415`, 0.1072401379`, 
              1, \(-0.3121782428`\), \(-0.1641999204`\), 433.2546802`, 
              1.567746709`, \(-1.509329198`\), 2, 3, 4, 5, 6, 
              0.41988`, \(-0.504512`\), 0.239424`, 0.16275`, 
              0.124816`, \(-0.10569`\), 0.841072`, 0.769966`, 0.869807`, 
              0.887436`, 1.173127`, 1.163093`, 16.5242`, 16.547691`, 
              1.003822`, 32.209406`, 32.189642`, \(-0.844522`\), 40.768135`, 
              40.778095`, 0.425623`, 55.64378`, 55.646252`, 0.105628`, 
              30.819212`, 30.836067`, 0.720261`, 58.932365`, 58.958675`, 
              1.124237`, 67.889033`, 67.887489`, \(-0.066002`\), 71.583424`, 
              71.562148`, \(-0.90917`\), 16.5242`, 32.209406`, 40.768135`, 
              60.430885`, 72.989269`, 89.445591`, 104.424399`, 111.969634`, 
              30.819212`, 58.932365`, 67.889033`, 71.583424`, 55.64378`, 
              79.346991`, 79.680021`, 95.792696`}, {3, 79, 
              15.75843486`, \(-0.7737100304`\), 5.574004866`, 0.1453085056`, 
              1, \(-0.3617587894`\), \(-0.1972666616`\), 
              3852.154422`, \(-1.105785389`\), \(-1.421998508`\), 2, 3, 4, 5, 
              6, 0.464625`, \(-0.475019`\), 0.11878`, 0.188326`, 
              0.090637`, \(-0.081424`\), 0.880628`, 0.784738`, 0.930756`, 
              0.929269`, 1.259439`, 1.260317`, 16.5242`, 16.548423`, 
              1.003718`, 31.813859`, 31.793283`, \(-0.852617`\), 37.634518`, 
              37.642286`, 0.321883`, 53.024739`, 53.027062`, 0.09626`, 
              28.733076`, 28.747416`, 0.594232`, 56.291567`, 56.321557`, 
              1.242738`, 61.428382`, 61.425642`, \(-0.113536`\), 66.986073`, 
              66.961133`, \(-1.033462`\), 16.5242`, 31.813859`, 37.634518`, 
              58.406044`, 68.803032`, 83.484257`, 102.244134`, 103.297372`, 
              28.733076`, 56.291567`, 61.428382`, 66.986073`, 53.024739`, 
              75.706051`, 77.217561`, 82.353405`}, {1, 39, 
              15.91646958`, \(-0.7737100304`\), 5.574004866`, 0.1453085056`, 
              1, \(-0.3615279472`\), \(-0.197534793`\), 4765.560566`, 
              1.327145906`, \(-1.439827774`\), 2, 3, 4, 5, 6, 
              0.464625`, \(-0.475019`\), 0.11878`, 0.188326`, 
              0.090637`, \(-0.081424`\), 0.880628`, 0.784738`, 0.930756`, 
              0.929269`, 1.259439`, 1.260317`, 16.5242`, 16.548423`, 
              1.003718`, 31.813859`, 31.793283`, \(-0.852617`\), 37.634518`, 
              37.642286`, 0.321883`, 53.024739`, 53.027062`, 0.09626`, 
              28.733076`, 28.747416`, 0.594232`, 56.291567`, 56.321557`, 
              1.242738`, 61.428382`, 61.425642`, \(-0.113536`\), 66.986073`, 
              66.961133`, \(-1.033462`\), 16.5242`, 31.813859`, 37.634518`, 
              58.406044`, 68.803032`, 83.484257`, 102.244134`, 103.297372`, 
              28.733076`, 56.291567`, 61.428382`, 66.986073`, 53.024739`, 
              75.706051`, 77.217561`, 82.353405`}, {2, 40, 
              28.80204336`, \(-0.787267923`\), 4.943818432`, 0.1921681542`, 
              1, \(-0.4234107898`\), \(-0.217082125`\), 
              24.12743121`, \(-0.7494070055`\), \(-1.360576748`\), 2, 3, 4, 
              5, 6, 0.524957`, \(-0.424866`\), \(-0.044819`\), 0.214489`, 
              0.0533`, \(-0.052378`\), 0.924397`, 0.806973`, 0.997695`, 
              0.978225`, 1.352787`, 1.364031`, 16.5242`, 16.5461`, 0.909383`, 
              31.294242`, 31.293652`, \(-0.024476`\), 34.967954`, 
              34.956354`, \(-0.481684`\), 51.602385`, 51.604172`, 0.074181`, 
              27.283185`, 27.294503`, 0.469993`, 55.609038`, 55.613993`, 
              0.205745`, 55.897899`, 55.918951`, 0.874182`, 63.727049`, 
              63.701923`, \(-1.043341`\), 16.5242`, 31.294242`, 34.967954`, 
              55.755055`, 66.192934`, 79.421504`, 97.57947`, 98.444182`, 
              27.283185`, 55.897899`, 55.609038`, 63.727049`, 51.602385`, 
              71.602448`, 78.046515`, 78.220387`}, {2, 7, 
              80.93423921`, \(-0.787267923`\), 4.943818432`, 0.1921681542`, 
              1, \(-0.4302997803`\), \(-0.1602216036`\), 50, 
              2.034154234`, \(-1.914967656`\), 2, 3, 4, 5, 6, 
              0.524957`, \(-0.424866`\), \(-0.044819`\), 0.214489`, 
              0.0533`, \(-0.052378`\), 0.924397`, 0.806973`, 0.997695`, 
              0.978225`, 1.352787`, 1.364031`, 16.5242`, 16.5461`, 0.909383`, 
              31.294242`, 31.293652`, \(-0.024476`\), 34.967954`, 
              34.956354`, \(-0.481684`\), 51.602385`, 51.604172`, 0.074181`, 
              27.283185`, 27.294503`, 0.469993`, 55.609038`, 55.613993`, 
              0.205745`, 55.897899`, 55.918951`, 0.874182`, 63.727049`, 
              63.701923`, \(-1.043341`\), 16.5242`, 31.294242`, 34.967954`, 
              55.755055`, 66.192934`, 79.421504`, 97.57947`, 98.444182`, 
              27.283185`, 55.897899`, 55.609038`, 63.727049`, 51.602385`, 
              71.602448`, 78.046515`, 78.220387`}, {2, 73, 
              14.3933885`, \(-7.263677677`\), 22.70260985`, 0.25`, 1, 
              0.01307421984`, 0.007134481757`, \(-2.04170849`\), 
              3.964286687`, \(-2.725204359`\), 2, 3, 4, 5, 6, 
              0.808058`, \(-0.388575`\), \(-0.140322`\), 0.039809`, 
              0.27714`, \(-0.293726`\), 1.025231`, 1.075153`, 0.930244`, 
              1.081333`, 1.186937`, 1.099594`, 16.5242`, 16.527643`, 
              1.010615`, 207.659675`, 207.676025`, 4.799179`, 285.04826`, 
              285.046185`, \(-0.60914`\), 285.165255`, 285.166942`, 
              0.495121`, 146.822079`, 146.82526`, 0.93377`, 290.846822`, 
              290.848907`, 0.611772`, 388.324498`, 388.32528`, 0.229791`, 
              433.487909`, 433.489772`, 0.546701`, 16.5242`, 285.04826`, 
              207.659675`, 285.165255`, 434.230148`, 494.915`, 594.567468`, 
              605.63102`, 146.822079`, 290.846822`, 388.324498`, 433.487909`, 
              385.255065`, 486.340691`, 514.395497`, 549.519272`}, {1, 67, 
              31.94321076`, \(-7.263677677`\), 22.70260985`, 0.25`, 
              1, \(-0.4089850132`\), \(-0.05444673811`\), 1.217524469`, 
              2.310851381`, \(-2.108328446`\), 2, 3, 4, 5, 6, 
              0.808058`, \(-0.388575`\), \(-0.140322`\), 0.039809`, 
              0.27714`, \(-0.293726`\), 1.025231`, 1.075153`, 0.930244`, 
              1.081333`, 1.186937`, 1.099594`, 16.5242`, 16.527643`, 
              1.010615`, 207.659675`, 207.676025`, 4.799179`, 285.04826`, 
              285.046185`, \(-0.60914`\), 285.165255`, 285.166942`, 
              0.495121`, 146.822079`, 146.82526`, 0.93377`, 290.846822`, 
              290.848907`, 0.611772`, 388.324498`, 388.32528`, 0.229791`, 
              433.487909`, 433.489772`, 0.546701`, 16.5242`, 285.04826`, 
              207.659675`, 285.165255`, 434.230148`, 494.915`, 594.567468`, 
              605.63102`, 146.822079`, 290.846822`, 388.324498`, 433.487909`, 
              385.255065`, 486.340691`, 514.395497`, 
              549.519272`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["analyze", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(data[\([1]\)]\)], "Input"],

Cell[BoxData[
    \("********** Eigenvalues for the 2+1 transverse lattice **********\n\
Couplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n            \
 0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 fit with 14 \
criteria, and tolerance 0.001.  \nOverall scale from fitting lowest state to \
lattice value.\n2 parity doublets with fractional errors 1 0.1.\nSpectrum for \
P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, c^2 error for each) \
=\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & 1, 0.25 0.25 1 1).\n\
Spectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  (26/2,6)  (32/2,6) \
.\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( 2,20/2,6) \n  ( 2,24/2,4) \
 ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \n  ( 3,35/2,5)  ( \
4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy potential \
determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  ( \
1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra."\)], "Output"]
}, Open  ]],

Cell["Chi^2", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot[Map[{Sqrt[#[\([6]\)]], #[\([3]\)]} &, data[\([2]\)]], 
        PlotJoined -> True, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        FrameLabel -> {"\<m\>", "\<\>"}, PlotRange -> {0, 20}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Select which points to plot (gooddata) and which points to use for \
fitting (gooddata).\
\>", "Text"],

Cell[BoxData[
    \(\(gooddata = 
        Select[data[\([2]\)], \((#[\([3]\)] < 12\  && \ #[\([11]\)] < 
                  0)\) &];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((#[\([6]\)])\) &, gooddata]\)], "Input"],

Cell[BoxData[
    \({0, 0.001967542671`, 0.007919222016`, 0.01800590693`, 0.03249196962`, 
      0.0517766953`, 0.07642881742`, 0.1072401379`}\)], "Output"]
}, Open  ]],

Cell["\<\
Using gooddata, fit all couplings to either a linear or constant \
function.  Here, x is the mass.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting = 
      Chop[Table[
          Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([i]\)]/#[\([7]\)]} &, 
              gooddata], If[MemberQ[{10}, i], {1}, {1, x, x^2}], x], {i, 6, 
            12}]]\)], "Input"],

Cell[BoxData[
    \({0.9999999999999999`\ x\^2, 
      0.9999999999999997`, \(-0.011563807191840343`\) - 
        0.7527835558565635`\ x - 
        0.45905942887563184`\ x\^2, \(-0.12118436689655629`\) + 
        0.24407606336027804`\ x - 1.234958569597425`\ x\^2, 
      2488.839273682625`, \(-0.6133350761446964`\) - 2.391924111090087`\ x + 
        1.4145016744401746`\ x\^2, \(-0.6484585493026624`\) - 
        2.399033499434755`\ x + 0.22000237846197143`\ x\^2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pc = 
      Table[Show[
          ListPlot[
            Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([i]\)]/#[\([7]\)]} &, 
              gooddata], PlotStyle -> {PointSize[ .02], Hue[2/3]}, 
            DisplayFunction -> Identity], \n\t (*\(ListPlot[tau1, 
              PlotStyle -> {PointSize[ .02]}, 
              DisplayFunction -> Identity]\)\(,\)*) Plot[
            couplefitting[\([i - 5]\)], {x, 0, 1/2}, PlotStyle -> {Hue[0]}, 
            DisplayFunction -> Identity], 
          DisplayFunction -> $DisplayFunction, Axes -> False, 
          RotateLabel -> False, FormatType -> OutputForm, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
          Frame -> True, 
          FrameLabel -> {"\<m\>", {\*"\"\<\!\(l\_1\)\>\"", \
\*"\"\<\!\(l\_2\)\>\"", \*"\"\<\!\(l\_3\)\>\"", \*"\"\<\!\(t\_1\)\>\"", \*"\"\
\<\!\(t\_2\)\>\""}[\([i - 7]\)]}], {i, 8, 12}]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False], ",", 
        TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
          False,
          Editable->False]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
a^2 sigma vs m using two different methods: using the overall scale \
and using the longitudinal potential. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita2 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          Drop[gooddata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(4.659782978466963`\)\(\[InvisibleSpace]\)\) + 
      1.8685557940036552`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fita3 = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          Drop[gooddata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(0.8538431364367168`\)\(\[InvisibleSpace]\)\) + 
      0.2798955236411893`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fita2, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t\tPlot[fita3, {x, 0,  .5}, 
        DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0]}], \n\t\t (*\(Plot[1 + 2\ x, {x, 0,  .5}, 
          DisplayFunction -> Identity]\)\(,\)*) \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]*#[\([5]\)]} &, 
          gooddata], PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([24]\)]/#[\([27]\)]} &, 
          gooddata], PlotStyle -> {PointSize[0.01], Hue[0]}, 
        DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fitag = 
      Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]]*#[\([5]\)]} &, 
          Drop[gooddata, {2}]], {1, x}, x]\)], "Input"],

Cell[BoxData[
    \(\(\(5.876458136735308`\)\(\[InvisibleSpace]\)\) - 
      0.445645324002019`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[\n\t\tPlot[fitag, {x, 0,  .5}, DisplayFunction -> Identity, 
        PlotStyle -> {Hue[0.6]}], \n\t
      ListPlot[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], 
              Sqrt[#[\([24]\)]]*#[\([5]\)]} &, gooddata], 
        PlotStyle -> {PointSize[0.01], Hue[0.6]}, 
        DisplayFunction -> Identity], \nFrameLabel -> {"\<m\>", "\<\>"}, 
      PlotRange -> All, Axes -> False, Frame -> True, 
      TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
      DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(exam = Sqrt[Partition[Take[data[\([2, 2]\)], \(-40\)], 4]]\)], "Input"],

Cell[BoxData[
    \({{4.064996924968087`, 4.070876686906643`, 1.0230840630173066`, 
        5.2326430224122875`}, {5.229701999158269`, \(\(0.`\)\(\
\[InvisibleSpace]\)\) + 0.8205236133104276`\ \[ImaginaryI], 
        5.902570541721632`, 5.904776202363642`}, {0.7548748240602544`, 
        6.691386179260617`, 6.6919660040977496`, 
        0.4120521811615611`}, {4.828248336612358`, 4.831808874531359`, 
        0.8675131122928345`, 6.635889842364775`}, {6.638820603089076`, 
        0.9226380655489996`, 7.630073852853588`, 
        7.630446251694589`}, {0.3526343715521787`, 7.892040357220685`, 
        7.8944427922431615`, 0.9109610309996801`}, {4.064996924968087`, 
        5.2326430224122875`, 5.902570541721632`, 
        7.003064472072208`}, {7.630073852853588`, 8.644307895950954`, 
        9.417281773420608`, 9.760598444767616`}, {4.828248336612358`, 
        6.635889842364775`, 7.892040357220685`, 
        8.061383442065015`}, {6.691386179260617`, 8.091197006623927`, 
        8.542815695073843`, 8.920030773489517`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec1 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(\(0\^++\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(0\^\(-+\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(\(1\^++\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(1\^\(-+\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(\(2\^++\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(2\^\(-+\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([1]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([3]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([10]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([5]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([7]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 7}}, 
          FormatType -> OutputForm, FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pspec2 = 
      Block[{bs = 2, err = 0.05}, 
        Show[Graphics[{Text[\*"\"\<\!\(0\^\(+-\)\)\>\"", {1, bs}], 
              Text[\*"\"\<\!\(\(0\^--\)\)\>\"", {2, bs}], 
              Text[\*"\"\<\!\(1\^\(+-\)\)\>\"", {3, bs + 0.25}], 
              Text[\*"\"\<\!\(\(1\^--\)\)\>\"", {3, bs - 0.25}], 
              Text[\*"\"\<\!\(2\^\(+-\)\)\>\"", {4, bs}], 
              Text[\*"\"\<\!\(\(2\^--\)\)\>\"", {5, bs}], {Thickness[0.008], 
                Map[cross[1, #, #\ err] &, exam[\([2]\)]], 
                Map[cross[2, #, #\ err] &, exam[\([4]\)]], 
                Map[cross[3, #, #\ err] &, exam[\([9]\)]], 
                Map[cross[4, #, #\ err] &, exam[\([6]\)]], 
                Map[cross[5, #, #\ err] &, exam[\([8]\)]]}}], Frame -> True, 
          AspectRatio -> 1, PlotRange -> {{1/2, 5 + 1/2}, {0, 9.5}}, 
          FrameTicks -> {None, Automatic}, 
          TextStyle -> {FontFamily -> "\<Courier\>", 
              FontSize -> 12}]]\)], "Input"],

Cell[BoxData[
    \(Graphics::"gptn" \(\(:\)\(\ \)\) 
      "Coordinate \!\(\(\(0.` \[InvisibleSpace]\)\) + \(\(0.7794974326449062`\
\\ \[ImaginaryI]\)\)\) in \!\({1, \(\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.7794974326449062`\\ \[ImaginaryI]\)\)\)\)}\) is not a floating-point \
number."\)], "Message"],

Cell[BoxData[
    \(Graphics::"gptn" \(\(:\)\(\ \)\) 
      "Coordinate \!\(\(\(0.` \[InvisibleSpace]\)\) + \(\(0.8615497939759489`\
\\ \[ImaginaryI]\)\)\) in \!\({1, \(\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.8615497939759489`\\ \[ImaginaryI]\)\)\)\)}\) is not a floating-point \
number."\)], "Message"],

Cell[BoxData[
    \(Graphics::"gptn" \(\(:\)\(\ \)\) 
      "Coordinate \!\(\(\(0.` \[InvisibleSpace]\)\) + \(\(0.8205236133104276`\
\\ \[ImaginaryI]\)\)\) in \!\({0.9`, \(\(\(\(0.` \[InvisibleSpace]\)\) + \
\(\(0.8205236133104276`\\ \[ImaginaryI]\)\)\)\)}\) is not a floating-point \
number."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(Graphics :: \"gptn\"\) will be suppressed during \
this calculation."\)], "Message"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Put plots", "Subsection"],

Cell["Export[] is the new improved command.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<~/three/ttraj60.spec1.eps\>", pspec1, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<~/three/ttraj60.spec2.eps\>", pspec2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<~/three/ttraj60.chi2.eps\>", chi2, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pass = 
      Show[\n\t\t (*\(Plot[Sqrt[fita2], {x, 0,  .5}, 
            DisplayFunction -> Identity]\)\(,\)*) \n\t
        ListPlot[
          Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], Sqrt[#[\([24]\)]*#[\([5]\)]]} &, 
            gooddata], PlotStyle -> {PointSize[0.015]}, 
          DisplayFunction -> Identity], FrameLabel -> {"\<m\>", "\<\>"}, 
        PlotRange -> All, Axes -> False, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        DisplayFunction -> $DisplayFunction]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Display["\<~/three/ttraj60.ass.eps\>", pass, "\<EPS\>", 
      ImageSize -> 360, ImageOffset -> {0, 0}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This was printed to a postscript file \"ttraj60.scaling.ps\", size \
616 by 779, and moved to the left such that left=0. (You need to adjust the \
Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\(\ \)\(Show[GraphicsArray[Partition[pc, \ 2]]]\)\)\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  GraphicsArray  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
This prints out scaling graphs to separate files (You need to \
adjust the Font size on the graph labels, above)\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{couplelist = {"\<b\>", "\<l1\>", "\<l2\>", "\<l3\>", "\<l4\>", "\
\<l5\>", "\<t1\>", "\<t2\>"}}, \[IndentingNewLine]Do[
        Print["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>"]; 
        Display["\<~/three/ttraj60.\>" <> couplelist[\([i]\)] <> "\<.eps\>", 
          pc[\([i]\)], "\<EPS\>", ImageSize -> 360, 
          ImageOffset -> {0, 0}], {i, Length[pc]}]]\)], "Input"],

Cell[BoxData[
    \("~/three/ttraj60.b.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l2.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l3.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l4.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.l5.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t1.eps"\)], "Print"],

Cell[BoxData[
    \("~/three/ttraj60.t2.eps"\)], "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Create data file", "Subsection"],

Cell["\<\
Create input file for scans of parameter space.
This uses the fitted coupling constants as determined above.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefitting\)], "Input"],

Cell[BoxData[
    \({0.9999999999999999`\ x\^2, 
      0.9999999999999997`, \(-0.011563807191840343`\) - 
        0.7527835558565635`\ x - 
        0.45905942887563184`\ x\^2, \(-0.12118436689655629`\) + 
        0.24407606336027804`\ x - 1.234958569597425`\ x\^2, 
      2488.839273682625`, \(-0.6133350761446964`\) - 2.391924111090087`\ x + 
        1.4145016744401746`\ x\^2, \(-0.6484585493026624`\) - 
        2.399033499434755`\ x + 0.22000237846197143`\ x\^2}\)], "Output"]
}, Open  ]],

Cell["Trajectory case", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr2 = 
      Block[{ni = 11, nj = 11, cc, y}, \[IndentingNewLine]cc = 
          couplefitting; \[IndentingNewLine]cc[\([5]\)] = 
          50; \[IndentingNewLine]TableForm[
          Flatten[Table[\[IndentingNewLine]{5, 
                N[cc /. {x \[Rule] 
                        0.5\ \((i - 1)\)/\((ni - 1)\)}\[IndentingNewLine], 
                  12], {2, 3, 4, 5, 6}}, {i, 6, 11}], 1], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.06249999999999999`", 
            "0.9999999999999997`", \(-0.2284509104607082`\), \
\(-0.13735026165632586`\), 
            "50.`", \(-1.122909749264707`\), \(-1.234466775507478`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"},
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.08999999999999998`", 
            "0.9999999999999997`", \(-0.27871422254761624`\), \
\(-0.15910781915224115`\), 
            "50.`", \(-1.2036071587721069`\), \(-1.3483683850715116`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"},
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.12249999999999997`", 
            "0.9999999999999997`", \(-0.3312728317789025`\), \
\(-0.18704016949614352`\), 
            "50.`", \(-1.2772320599073053`\), \(-1.4611699827432352`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"},
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.16`", 
            "0.9999999999999997`", \(-0.3861267381545669`\), \
\(-0.22114731268803312`\), 
            "50.`", \(-1.3437844526703033`\), \(-1.572871568522649`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"},
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.20249999999999999`", 
            "0.9999999999999997`", \(-0.44327594167460943`\), \
\(-0.26142924872790974`\), 
            "50.`", \(-1.4032643370611`\), \(-1.683473142409753`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"},
          {
            "5", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", "\<\"\"\>", \
"\<\"\"\>"},
          {"0.24999999999999997`", 
            "0.9999999999999997`", \(-0.5027204423390301`\), \
\(-0.3078859776157735`\), 
            "50.`", \(-1.4556717130796963`\), \(-1.7929747044045472`\)},
          {"2", "3", "4", "5", "6", "\<\"\"\>", "\<\"\"\>"}
          },
        RowSpacings->0,
        ColumnSpacings->1,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {5, {0.062499999999999993, 
        0.99999999999999967, -0.2284509104607082, -0.13735026165632586, 
        50.0, -1.1229097492647071, -1.2344667755074781}, {2, 3, 4, 5, 6}, 5, {
        0.089999999999999983, 
        0.99999999999999967, -0.27871422254761624, -0.15910781915224115, 
        50.0, -1.2036071587721069, -1.3483683850715116}, {2, 3, 4, 5, 6}, 5, {
        0.12249999999999997, 
        0.99999999999999967, -0.33127283177890249, -0.18704016949614352, 
        50.0, -1.2772320599073053, -1.4611699827432352}, {2, 3, 4, 5, 6}, 5, {
        0.16, 0.99999999999999967, -0.38612673815456688, -0.22114731268803312,
         50.0, -1.3437844526703033, -1.5728715685226491}, {2, 3, 4, 5, 6}, 
        5, {0.20249999999999999, 
        0.99999999999999967, -0.44327594167460943, -0.26142924872790974, 
        50.0, -1.4032643370611, -1.6834731424097531}, {2, 3, 4, 5, 6}, 5, {
        0.24999999999999997, 
        0.99999999999999967, -0.50272044233903013, -0.30788597761577352, 
        50.0, -1.4556717130796963, -1.7929747044045472}, {2, 3, 4, 5, 6}}, 
        TableSpacing -> {0, 1}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/dave/btraj21.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<btraj21.out\>"]; \n\t\tWrite[
        fp, "\<100\>"]; \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
Write[fp, "\<3\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/dave/btraj21.in"\)], "Output"]
}, Open  ]],

Cell["lambda_1 case", "Text"],

Cell[BoxData[
    \(expr2 = 
      Block[{ni = 11, nj = 11, cc, y}, \[IndentingNewLine]cc = 
          couplefitting; \[IndentingNewLine]cc[\([3]\)] = 
          y; \[IndentingNewLine]cc[\([5]\)] = 
          50; \[IndentingNewLine]TableForm[
          Flatten[Table[\[IndentingNewLine]{4, 
                N[cc /. {x \[Rule] 0.5\ \((i - 1)\)/\((ni - 1)\), 
                      y \[Rule] \(-0.6\) + \((j - 1)\)/\((nj - 
                                1)\)}\[IndentingNewLine], 12], {3, 4, 5, 
                  6}}, {i, 9, 11}, {j, nj}], 2], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/dave/l1_dscan12.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<l1_dscan12.out\>"]; \n\t\tWrite[fp, "\<25\>"]; \n\t\tWrite[
        fp, "\<1\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/dave/l1_dscan12.in"\)], "Output"]
}, Open  ]],

Cell["lambda_2 case", "Text"],

Cell[BoxData[
    \(expr3 = 
      Block[{ni = 11, nj = 11, cc, y}, \[IndentingNewLine]cc = 
          couplefitting; \[IndentingNewLine]cc[\([4]\)] = 
          y; \[IndentingNewLine]cc[\([5]\)] = 
          50; \[IndentingNewLine]TableForm[
          Flatten[Table[\[IndentingNewLine]{4, 
                N[cc /. {x \[Rule] 0.5\ \((i - 1)\)/\((ni - 1)\), 
                      y \[Rule] \(-0.35\) + 
                          0.5*\((j - 1)\)/\((nj - 1)\)}\[IndentingNewLine], 
                  12], {2, 4, 5, 6}}, {i, ni}, {j, nj}], 2], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/dave/l2_dscan12.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<l2_dscan12.out\>"]; \n\t\tWrite[fp, "\<25\>"]; \n\t\tWrite[
        fp, "\<1\>"]; \n\t\tWrite[fp, expr3]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/dave/l2_dscan12.in"\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Scanalyze lambda_1", "Section"],

Cell[TextData[{
  "The postscript plots bvalley.ps and dvalley.ps were created with a left \
margin of 50pts and width 400pts.\nThe print style was chosen to have no \
headers or footers.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " fonts included in the document."
}], "Text"],

Cell[CellGroupData[{

Cell["data l1_dscan1*.out", "Subsection"],

Cell[BoxData[
    \(\(datal1 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from fitting \
lowest state to lattice value.\n2 parity doublets with fractional errors 1 \
0.1.\nSpectrum for P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, \
c^2 error for each) =\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & \
1, 0.25 0.25 1 1).\nSpectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  \
(26/2,6)  (32/2,6) .\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( \
2,20/2,6) \n  ( 2,24/2,4)  ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \
\n  ( 3,35/2,5)  ( 4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy \
potential determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  \
( 1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{5, 28, 
              8.485293419`*^7, \(-0.25`\), \(-0.1793862218`\), 0, 
              1, \(-0.6`\), \(-0.08859830215`\), \(-6.577137323`\), \
\(-0.9752786396`\), \(-0.7263331016`\), 3, 4, 5, 6, 
              0.326118`, \(-63.105456`\), 
              49.791633`, \(-0.036057`\), \(-7.50715`\), \(-0.284654`\), \
\(-7.691867`\), \(-10.28954`\), \(-7.680675`\), \(-7.711136`\), \
\(-7.66862`\), \(-7.744394`\), 16.5242`, 16.524381`, \(-0.000169`\), 
              14.305204`, 14.305228`, \(-0.000022`\), 13.034417`, 12.948919`, 
              0.080027`, 12.948964`, 12.63845`, 0.290646`, 15.058501`, 
              15.058696`, \(-0.000182`\), 13.464345`, 
              13.46504`, \(-0.000651`\), 12.919715`, 12.763333`, 0.146377`, 
              12.764264`, 12.362051`, 0.376479`, 16.5242`, 14.305204`, 
              12.948964`, 12.360534`, 12.764264`, 12.36209`, 12.668463`, 
              11.573382`, 15.058501`, 13.464345`, 12.919715`, 12.283308`, 
              12.856477`, 12.535466`, 13.034417`, 12.293522`}, {5, 28, 
              4.517415644`*^7, \(-0.25`\), \(-0.4491166152`\), 0, 
              1, \(-0.5`\), \(-0.09065077425`\), \(-15.49162097`\), \
\(-0.9779794122`\), \(-0.7005946712`\), 3, 4, 5, 6, 
              0.164049`, \(-44.707835`\), 
              40.173725`, \(-0.015117`\), \(-4.840458`\), 
              0.03014`, \(-4.885766`\), \(-4.677518`\), \(-4.872565`\), \
\(-4.866326`\), \(-4.855953`\), \(-4.910067`\), 30.185854`, 30.184188`, 
              0.001964`, 26.524085`, 26.524173`, \(-0.000103`\), 23.67575`, 
              23.405697`, 0.318349`, 23.405383`, 23.357463`, 0.056489`, 
              26.876686`, 26.877108`, \(-0.000498`\), 23.926495`, 23.92421`, 
              0.002694`, 22.971972`, 22.972198`, \(-0.000266`\), 22.266727`, 
              22.135408`, 0.154803`, 16.5242`, 30.185854`, 26.524085`, 
              23.405383`, 21.984824`, 22.971972`, 22.266727`, 19.186872`, 
              26.876686`, 23.926495`, 22.131417`, 22.112029`, 23.352713`, 
              23.67575`, 22.853973`, 19.844599`}, {1, 22, 
              2.030531288`*^7, \(-0.25`\), \(-0.3667137891`\), 0, 
              1, \(-0.4`\), 0.0006222539087`, 
              4.078362252`*^6, \(-0.8852163645`\), \(-0.6300414282`\), 3, 4, 
              5, 6, 0.231295`, \(-25.729052`\), 21.576436`, 
              0.012649`, \(-2.460507`\), 0.575645`, \(-2.277953`\), 
              0.805678`, \(-2.256862`\), \(-2.199119`\), \(-2.217281`\), \
\(-2.282166`\), 16.5242`, 16.524204`, \(-5.`*^-6\), 14.367468`, 14.367453`, 
              0.000021`, 12.657314`, 13.092165`, \(-0.590138`\), 12.619682`, 
              11.645106`, 1.322602`, 13.233644`, 13.233836`, \(-0.00026`\), 
              11.370088`, 11.369985`, 0.000139`, 11.15456`, 
              11.154636`, \(-0.000103`\), 10.805241`, 
              10.999807`, \(-0.264047`\), 16.5242`, 14.367468`, 12.619682`, 
              11.338711`, 10.722131`, 11.370088`, 8.890857`, 8.143524`, 
              13.233644`, 11.15456`, 10.805241`, 9.789458`, 11.645109`, 
              12.657314`, 9.53121`, 8.231636`}, {\(-3\), 1, 
              6.87456437`*^6, \(-0.25`\), \(-0.3667137891`\), 0, 
              1, \(-0.3`\), \(-0.114676`\), 
              50, \(-0.97786`\), \(-0.676558`\), 3, 4, 5, 6, 
              0.231295`, \(-25.729052`\), 21.576436`, 
              0.012649`, \(-2.460507`\), 0.575645`, \(-2.277953`\), 
              0.805678`, \(-2.256862`\), \(-2.199119`\), \(-2.217281`\), \
\(-2.282166`\), 16.5242`, 16.524204`, \(-5.`*^-6\), 14.367468`, 14.367453`, 
              0.000021`, 12.657314`, 13.092165`, \(-0.590138`\), 12.619682`, 
              11.645106`, 1.322602`, 13.233644`, 13.233836`, \(-0.00026`\), 
              11.370088`, 11.369985`, 0.000139`, 11.15456`, 
              11.154636`, \(-0.000103`\), 10.805241`, 
              10.999807`, \(-0.264047`\), 16.5242`, 14.367468`, 12.619682`, 
              11.338711`, 10.722131`, 11.370088`, 8.890857`, 8.143524`, 
              13.233644`, 11.15456`, 10.805241`, 9.789458`, 11.645109`, 
              12.657314`, 9.53121`, 8.231636`}, {5, 29, 
              1.018704215`*^6, \(-0.25`\), \(-1.637443708`\), 0, 
              1, \(-0.2`\), \(-0.1116892459`\), 
              0.7169711554`, \(-0.9790475706`\), \(-0.6917093345`\), 3, 4, 5, 
              6, 0.051219`, \(-5.679863`\), 6.175618`, 
              0.071103`, \(-0.143033`\), 0.457651`, 0.46263`, 1.11827`, 
              0.518213`, 0.217748`, 0.755199`, 0.304736`, 16.5242`, 
              16.520385`, 0.005119`, 11.18742`, 11.195256`, \(-0.010516`\), 
              7.925327`, 7.460273`, 0.624056`, 7.463786`, 7.282227`, 
              0.243634`, 6.721626`, 6.711775`, 0.013219`, 3.007284`, 
              3.783909`, \(-1.042154`\), 1.641921`, 3.122633`, \(-1.98697`\), 
              1.014374`, 1.565833`, \(-0.740004`\), 16.5242`, 11.18742`, 
              7.463786`, 5.568332`, 3.007284`, 
              0.56493`, \(-1.478819`\), \(-1.92364`\), 
              6.721626`, \(-1.830518`\), 1.641921`, 1.014374`, 7.382018`, 
              7.925327`, 3.18193`, \(-0.547522`\)}, {5, 28, 
              8631.475839`, \(-0.25`\), \(-23.67927756`\), 0, 
              1, \(-0.1`\), \(-0.1167440843`\), 
              12.96120516`, \(-1.322025314`\), \(-0.9867866322`\), 3, 4, 5, 
              6, 0.128109`, \(-0.966803`\), 1.362174`, 0.036949`, 
              0.182064`, \(-0.032251`\), 0.594273`, 0.491413`, 0.606198`, 
              0.504908`, 0.832558`, 0.542374`, 16.5242`, 16.341816`, 
              8.852232`, \(-26.675742`\), \(-27.01507`\), 
              16.469731`, \(-34.982471`\), \(-34.829156`\), \(-7.441349`\), \
\(-84.406065`\), \(-84.187394`\), \(-10.613525`\), \(-66.944429`\), \
\(-67.180863`\), 11.475646`, \(-89.59019`\), \(-89.701076`\), 
              5.381996`, \(-137.878266`\), \(-137.488426`\), \(-18.921407`\), \
\(-140.810237`\), \(-141.200162`\), 18.925555`, 
              16.5242`, \(-34.982471`\), \(-26.675742`\), \(-94.130829`\), \
\(-145.024512`\), \(-142.826682`\), \(-201.853965`\), \(-218.020799`\), \
\(-66.944429`\), \(-89.59019`\), \(-137.878266`\), \(-140.810237`\), \
\(-84.406065`\), \(-92.339853`\), \(-119.783186`\), \(-171.926154`\)}, {5, 
              28, 11.24580645`, \(-1.694596901`\), 7.15235034`, 0, 1, 
              0, \(-0.1202695117`\), 
              5.887960038`, \(-0.8174729179`\), \(-0.8399298072`\), 3, 4, 5, 
              6, 0.188407`, \(-0.42274`\), 0.253201`, 0.115557`, 
              0.245151`, \(-0.21689`\), 0.620484`, 0.534041`, 0.672446`, 
              0.651349`, 0.938848`, 0.888402`, 16.5242`, 16.571963`, 
              1.029801`, 26.848504`, 26.828758`, \(-0.425735`\), 30.125112`, 
              30.136327`, 0.241806`, 44.032743`, 44.040438`, 0.165899`, 
              22.624571`, 22.656289`, 0.683871`, 43.246657`, 43.283441`, 
              0.793086`, 56.822856`, 56.827711`, 0.104664`, 60.42753`, 
              60.462399`, 0.751803`, 16.5242`, 26.848504`, 30.125112`, 
              48.385`, 56.822856`, 72.888256`, 87.031517`, 92.800058`, 
              22.624571`, 43.246657`, 60.42753`, 63.328062`, 44.032743`, 
              64.510948`, 71.387197`, 78.15257`}, {5, 28, 
              23.12784023`, \(-2.051249212`\), 5.938416276`, 0, 1, 
              0.1`, \(-0.1428636852`\), 
              2.850387171`, \(-0.829668744`\), \(-0.781936175`\), 3, 4, 5, 6, 
              0.186451`, \(-0.446684`\), 0.305994`, 0.1257`, 
              0.225604`, \(-0.204793`\), 0.610369`, 0.468059`, 0.675212`, 
              0.618579`, 0.952383`, 0.89549`, 16.5242`, 16.571301`, 0.83442`, 
              20.164116`, 20.182376`, 0.323488`, 27.381703`, 
              27.354773`, \(-0.477081`\), 39.848216`, 39.893567`, 0.803428`, 
              18.034059`, 18.056577`, 0.398916`, 37.614782`, 37.645113`, 
              0.537323`, 48.563741`, 48.569203`, 0.096779`, 50.008757`, 
              50.03839`, 0.524962`, 16.5242`, 20.164116`, 27.381703`, 
              39.848216`, 48.563741`, 62.28021`, 80.305632`, 81.196504`, 
              18.034059`, 37.614782`, 50.008757`, 54.558074`, 40.011159`, 
              63.569742`, 62.401773`, 71.303506`}, {5, 29, 
              98.21764755`, \(-1.564160637`\), 4.818596591`, 0, 1, 
              0.2`, \(-0.1201514923`\), 
              7.90388727`, \(-0.9655508548`\), \(-0.6845963573`\), 3, 4, 5, 
              6, 0.184203`, \(-0.501821`\), 0.455797`, 0.123564`, 
              0.211074`, \(-0.184313`\), 0.591147`, 0.377494`, 0.675785`, 
              0.555175`, 0.967161`, 0.845595`, 16.5242`, 16.545443`, 
              0.301687`, 19.396133`, 19.410751`, 0.207609`, 24.012752`, 
              24.000608`, \(-0.17247`\), 33.870339`, 33.872882`, 0.036114`, 
              13.878635`, 13.891459`, 0.182118`, 31.170548`, 31.188847`, 
              0.25987`, 39.513818`, 39.511669`, \(-0.030522`\), 40.354652`, 
              40.375251`, 0.292548`, 16.5242`, 24.012752`, 19.396133`, 
              38.822122`, 39.513818`, 50.710551`, 63.822897`, 71.258652`, 
              13.878635`, 31.170548`, 40.354652`, 44.544315`, 33.870339`, 
              52.810658`, 51.106535`, 60.845602`}, {2, 29, 
              16.01965141`, \(-1.14191558`\), 7.372469323`, 0, 1, 
              0.3`, \(-0.2751476413`\), 
              2440.620529`, \(-0.8256067067`\), \(-0.6291305369`\), 3, 4, 5, 
              6, 0.100887`, \(-0.651811`\), 0.808105`, 0.13861`, 
              0.174378`, \(-0.138945`\), 0.535951`, 0.439627`, 0.623006`, 
              0.608615`, 0.907338`, 0.914507`, 16.5242`, 16.600991`, 
              0.913859`, 29.194752`, 29.226762`, 0.380937`, 37.024297`, 
              36.995233`, \(-0.345879`\), 45.371215`, 
              45.354646`, \(-0.197182`\), 17.298496`, 17.339217`, 0.484601`, 
              37.87979`, 37.933032`, 0.633612`, 52.847575`, 52.87305`, 
              0.303167`, 57.569874`, 57.617192`, 0.563122`, 16.5242`, 
              37.024297`, 29.194752`, 46.419692`, 52.847575`, 69.141564`, 
              91.285045`, 92.529021`, 17.298496`, 37.87979`, 57.569874`, 
              64.233656`, 45.371215`, 67.228517`, 78.440492`, 
              87.930214`}, {5, 29, 78.71303424`, \(-1.021524179`\), 
              4.788474592`, 0, 1, 0.4`, \(-0.09497321707`\), 
              884.3701359`, \(-0.3144327061`\), \(-0.1793237534`\), 3, 4, 5, 
              6, 0.135909`, \(-0.721982`\), 0.997504`, 0.188043`, 
              0.108756`, \(-0.108279`\), 0.522551`, 0.379375`, 0.708701`, 
              0.648873`, 1.068932`, 1.085433`, 3.675217`, 3.709841`, 
              0.360526`, 16.5242`, 16.530079`, 0.061214`, 32.10049`, 
              32.101691`, 0.012511`, 38.614643`, 38.62506`, 0.108479`, 
              11.175001`, 11.183848`, 0.092113`, 27.882699`, 27.894865`, 
              0.126677`, 35.893253`, 35.952046`, 0.612197`, 38.311593`, 
              38.263286`, \(-0.503007`\), 16.5242`, 38.794442`, 3.675217`, 
              38.614643`, 35.893253`, 46.50803`, 60.111565`, 63.745243`, 
              11.175001`, 27.882699`, 38.311593`, 41.952691`, 32.10049`, 
              50.539704`, 49.390089`, 56.298562`}, {5, 28, 
              5.944748876`*^7, \(-0.25`\), \(-0.2143161086`\), 0.0025`, 
              1, \(-0.6`\), \(-0.07954130604`\), \(-4.35404774`\), \
\(-0.9786931686`\), \(-0.8109743705`\), 3, 4, 5, 6, 
              0.224482`, \(-49.38044`\), 
              40.22676`, \(-0.009455`\), \(-5.57983`\), 
              0.133956`, \(-5.57051`\), \(-4.261571`\), \(-5.550514`\), \
\(-5.549948`\), \(-5.525006`\), \(-5.600328`\), 16.5242`, 
              16.524342`, \(-0.000109`\), 14.322716`, 
              14.322758`, \(-0.000032`\), 12.743392`, 
              12.904402`, \(-0.123939`\), 12.62467`, 12.502954`, 0.093693`, 
              14.58148`, 14.581686`, \(-0.000158`\), 13.249761`, 
              13.250545`, \(-0.000604`\), 12.245562`, 12.086269`, 0.122617`, 
              12.086181`, 12.051447`, 0.026737`, 16.5242`, 14.322716`, 
              12.62467`, 12.503191`, 12.051753`, 12.086181`, 12.245562`, 
              10.869794`, 14.58148`, 13.249761`, 11.952963`, 11.584431`, 
              12.422916`, 12.438108`, 12.743392`, 11.434598`}, {5, 29, 
              3.123023593`*^7, \(-0.25`\), \(-0.2957016938`\), 0.0025`, 
              1, \(-0.5`\), \(-0.1018232199`\), 
              5.772211537`, \(-0.9785446784`\), \(-0.8179869271`\), 3, 4, 5, 
              6, 0.087011`, \(-34.428323`\), 32.906541`, 
              0.02132`, \(-3.425241`\), 0.489655`, \(-3.176734`\), 
              2.19032`, \(-3.142077`\), \(-3.176166`\), \(-3.098658`\), \
\(-3.220709`\), 16.5242`, 16.524294`, \(-0.000039`\), 14.331396`, 
              14.331488`, \(-0.000038`\), 12.903601`, 
              13.438268`, \(-0.220106`\), 12.893603`, 12.310733`, 0.239949`, 
              13.742203`, 13.74222`, \(-7.`*^-6\), 12.12562`, 12.12384`, 
              0.000733`, 12.036607`, 12.035733`, 0.00036`, 11.44161`, 
              11.445761`, \(-0.001709`\), 16.5242`, 14.331396`, 12.893603`, 
              12.310557`, 11.168327`, 12.036607`, 11.102701`, 9.128733`, 
              13.742203`, 12.12562`, 11.44161`, 10.592769`, 12.084439`, 
              12.903601`, 11.415448`, 9.530608`}, {1, 18, 
              1.238422843`*^7, \(-0.25`\), \(-0.4695965132`\), 0.0025`, 
              1, \(-0.4`\), 0.001665542701`, 
              98394.6924`, \(-1.07969787`\), \(-0.7265978113`\), 3, 4, 5, 6, 
              0.224924`, \(-18.246755`\), 16.089382`, 
              0.072697`, \(-1.573117`\), 1.133881`, \(-1.016215`\), 
              3.534328`, \(-0.954579`\), \(-0.938892`\), \(-0.840268`\), \
\(-0.982722`\), 16.5242`, 16.524204`, \(-7.`*^-6\), 14.224885`, 14.224867`, 
              0.000031`, 12.500578`, 13.328826`, \(-1.39972`\), 11.618789`, 
              10.848056`, 1.302521`, 11.807381`, 11.807625`, \(-0.000412`\), 
              10.450272`, 10.448421`, 0.003128`, 9.535027`, 
              10.041296`, \(-0.855583`\), 9.530899`, 
              9.535901`, \(-0.008454`\), 16.5242`, 14.224885`, 11.618789`, 
              10.668523`, 9.428128`, 10.450272`, 7.737994`, 5.76851`, 
              11.807381`, 9.535027`, 9.530899`, 7.568103`, 10.84806`, 
              12.500578`, 8.961401`, 5.659605`}, {5, 29, 
              3.568702532`*^6, \(-0.25`\), \(-0.8748964064`\), 0.0025`, 
              1, \(-0.3`\), \(-0.1020150148`\), 
              3.206605846`, \(-0.9829091677`\), \(-0.8252355185`\), 3, 4, 5, 
              6, 0.067302`, \(-10.004735`\), 10.187125`, 
              0.09385`, \(-0.579132`\), 1.002705`, 0.257808`, 3.859947`, 
              0.319636`, 0.055896`, 0.568026`, 0.090933`, 16.5242`, 
              16.524114`, 0.000081`, 12.816498`, 12.816464`, 0.000032`, 
              11.444255`, 11.445099`, \(-0.000795`\), 10.696183`, 10.319268`, 
              0.355096`, 9.351034`, 9.349435`, 0.001506`, 6.995817`, 
              6.43607`, 0.527343`, 6.879425`, 6.412542`, 0.439855`, 6.39604`, 
              5.308371`, 1.024705`, 16.5242`, 11.444255`, 12.816498`, 
              6.759934`, 6.995817`, 6.879425`, 3.738235`, 3.277388`, 
              9.351034`, 6.39604`, 3.810758`, 2.789775`, 9.245563`, 
              10.696183`, 5.867198`, 3.826141`}, {5, 29, 
              319290.1185`, \(-0.25`\), \(-2.926380058`\), 0.0025`, 
              1, \(-0.2`\), \(-0.1021205677`\), 
              1.837166081`, \(-0.9789469624`\), \(-0.833343108`\), 3, 4, 5, 
              6, 0.122653`, \(-2.75977`\), 3.7002`, 0.092297`, 0.060762`, 
              0.142034`, 0.576604`, 0.293941`, 0.62909`, 0.362709`, 
              0.885784`, 0.55967`, 16.5242`, 16.518215`, 0.034371`, 
              7.688176`, 7.696639`, \(-0.048601`\), 2.941312`, 2.932921`, 
              0.048184`, 2.143928`, 1.396576`, 
              4.291924`, \(-4.236613`\), \(-4.282667`\), 
              0.264481`, \(-4.559511`\), \(-5.290927`\), 
              4.20041`, \(-5.297049`\), \(-6.307961`\), 
              5.80551`, \(-9.071279`\), \(-9.880635`\), 4.648007`, 16.5242`, 
              7.688176`, 2.941312`, 
              0.86932`, \(-10.265366`\), \(-4.559511`\), \(-14.663083`\), \
\(-20.204343`\), \(-4.236613`\), \(-5.297049`\), \(-9.071279`\), \(-9.923007`\
\), 0.017701`, 2.143928`, \(-5.828435`\), \(-14.644419`\)}, {5, 28, 
              398.3990433`, \(-0.8705199096`\), 12.24429062`, 0.0025`, 
              1, \(-0.1`\), \(-0.1035449935`\), 
              5.339015867`, \(-1.199743322`\), \(-1.189413285`\), 3, 4, 5, 6, 
              0.20724`, \(-0.522298`\), 0.467538`, 0.081213`, 
              0.242662`, \(-0.174434`\), 0.649777`, 0.664055`, 0.643008`, 
              0.710242`, 0.87053`, 0.827492`, 16.5242`, 16.590012`, 
              2.671977`, 35.166071`, 35.114521`, \(-2.092903`\), 51.01105`, 
              51.051129`, 1.62722`, 64.169104`, 64.224886`, 2.264755`, 
              41.524721`, 41.583616`, 2.391133`, 71.217658`, 71.269065`, 
              2.087132`, 95.68737`, 95.682578`, \(-0.194563`\), 102.85262`, 
              102.789648`, \(-2.556676`\), 16.5242`, 35.166071`, 51.01105`, 
              64.169104`, 95.68737`, 121.929079`, 137.956355`, 156.476998`, 
              41.524721`, 71.217658`, 102.85262`, 107.113544`, 68.47227`, 
              93.211673`, 111.323157`, 123.723566`}, {5, 28, 
              24.3201141`, \(-2.362894729`\), 8.212521613`, 0.0025`, 1, 
              0, \(-0.09414653688`\), 
              1.263612736`, \(-0.9718736764`\), \(-0.8524473813`\), 3, 4, 5, 
              6, 0.248972`, \(-0.431472`\), 0.237868`, 0.12114`, 
              0.231655`, \(-0.211374`\), 0.66734`, 0.67648`, 0.735829`, 
              0.769117`, 1.026943`, 0.982087`, 16.5242`, 16.567835`, 
              1.42751`, 28.914094`, 28.939301`, 0.824654`, 36.130151`, 
              36.102254`, \(-0.912658`\), 51.972495`, 52.008385`, 1.174134`, 
              28.785499`, 28.808105`, 0.739555`, 58.866383`, 58.895894`, 
              0.965449`, 73.592068`, 73.567562`, \(-0.801693`\), 73.662105`, 
              73.702067`, 1.307357`, 16.5242`, 28.914094`, 36.130151`, 
              51.972495`, 73.592068`, 93.033861`, 114.921282`, 120.727091`, 
              28.785499`, 58.866383`, 73.662105`, 79.446609`, 59.054656`, 
              86.568067`, 92.740144`, 96.010337`}, {2, 28, 
              63.01307399`, \(-2.285235713`\), 4.88303958`, 0.0025`, 1, 
              0.1`, \(-0.1241233197`\), 
              3.290789245`, \(-0.846269539`\), \(-0.8913461997`\), 3, 4, 5, 
              6, 0.23885`, \(-0.452659`\), 0.282056`, 0.142604`, 
              0.200803`, \(-0.192617`\), 0.657828`, 0.460408`, 0.71778`, 
              0.637937`, 1.002435`, 0.956109`, 16.5242`, 16.555087`, 
              0.576375`, 19.581001`, 19.59282`, 0.220548`, 25.12592`, 
              25.107438`, \(-0.344885`\), 35.842132`, 35.992616`, 2.808192`, 
              16.42409`, 16.437488`, 0.250018`, 35.561959`, 35.581206`, 
              0.359179`, 43.483514`, 43.47476`, \(-0.163361`\), 44.419976`, 
              44.444412`, 0.455999`, 16.5242`, 19.581001`, 25.12592`, 
              35.842132`, 44.419976`, 56.088288`, 72.17786`, 75.286466`, 
              16.42409`, 35.561959`, 43.483514`, 48.675318`, 37.180905`, 
              55.636832`, 59.432523`, 59.985872`}, {5, 29, 
              71.90972106`, \(-1.704671983`\), 4.022163061`, 0.0025`, 1, 
              0.2`, \(-0.1113345372`\), 
              12.9266517`, \(-0.9579676368`\), \(-0.8286064714`\), 3, 4, 5, 
              6, 0.230721`, \(-0.493143`\), 0.393985`, 0.141285`, 
              0.189455`, \(-0.175735`\), 0.640131`, 0.371453`, 0.710806`, 
              0.580225`, 1.001839`, 0.913994`, 3.422815`, 3.480582`, 
              0.857732`, 16.5242`, 16.540924`, 0.24832`, 31.393528`, 
              31.394953`, 0.02115`, 37.007944`, 36.949879`, \(-0.862141`\), 
              12.859079`, 12.867726`, 0.128392`, 29.330697`, 29.343735`, 
              0.193582`, 35.472366`, 35.474042`, 0.024877`, 36.411992`, 
              36.419312`, 0.108676`, 16.5242`, 37.007944`, 3.422815`, 
              37.593934`, 36.411992`, 45.958443`, 59.799709`, 61.110774`, 
              12.859079`, 29.330697`, 35.472366`, 40.226344`, 31.393528`, 
              47.889133`, 48.117856`, 51.64233`}, {\(-3\), 6, 
              126.1460807`, \(-1.363484096`\), 3.936058713`, 0.0025`, 1, 
              0.3`, \(-0.104271`\), 50, \(-0.97786`\), \(-0.818537`\), 3, 4, 
              5, 6, 0.209108`, \(-0.560403`\), 0.569211`, 0.146771`, 
              0.169204`, \(-0.151652`\), 0.613227`, 0.349121`, 0.68188`, 
              0.572107`, 0.968056`, 0.920622`, 16.5242`, 16.537272`, 
              0.172142`, 17.710202`, 17.719607`, 0.123854`, 22.022703`, 
              22.016165`, \(-0.086096`\), 30.36176`, 30.362815`, 0.013901`, 
              11.653844`, 11.661132`, 0.095973`, 27.654372`, 27.665404`, 
              0.145278`, 34.056496`, 34.057982`, 0.019574`, 34.519811`, 
              34.525167`, 0.070527`, 16.5242`, 17.710202`, 22.022703`, 
              37.024261`, 34.519811`, 43.64902`, 57.526354`, 58.974467`, 
              11.653844`, 27.654372`, 34.056496`, 38.690624`, 30.36176`, 
              46.306925`, 46.976041`, 51.239727`}, {\(-3\), 6, 
              119.641175`, \(-1.135229108`\), 4.010679621`, 0.0025`, 1, 
              0.4`, \(-0.104271`\), 50, \(-0.97786`\), \(-0.818537`\), 3, 4, 
              5, 6, 0.179573`, \(-0.66025`\), 0.815786`, 0.152658`, 
              0.147888`, \(-0.125209`\), 0.579696`, 0.333669`, 0.643065`, 
              0.567953`, 0.921064`, 0.930357`, 16.5242`, 16.540819`, 
              0.191506`, 16.665144`, 16.670099`, 0.057092`, 22.328951`, 
              22.32401`, \(-0.056929`\), 29.886291`, 29.887264`, 0.011211`, 
              10.73394`, 10.740856`, 0.079695`, 26.455636`, 26.465747`, 
              0.116513`, 33.455367`, 33.46015`, 0.055118`, 33.804512`, 
              33.806124`, 0.018578`, 16.5242`, 16.665144`, 22.328951`, 
              36.379616`, 33.455367`, 42.467605`, 56.882037`, 57.850323`, 
              10.73394`, 26.455636`, 33.804512`, 38.197426`, 29.886291`, 
              45.83386`, 46.911132`, 51.142969`}, {5, 28, 
              3.552783101`*^7, \(-0.25`\), \(-0.2772383921`\), 0.01`, 
              1, \(-0.6`\), \(-0.07800406312`\), \(-8.444860366`\), \
\(-0.9817619299`\), \(-0.9400580719`\), 3, 4, 5, 6, 
              0.160965`, \(-37.898683`\), 33.036811`, 
              0.024134`, \(-3.994954`\), 0.610657`, \(-3.686512`\), 
              1.497805`, \(-3.643379`\), \(-3.690538`\), \(-3.586747`\), \
\(-3.752154`\), 16.5242`, 16.525671`, \(-0.00105`\), 15.548422`, 
              15.548495`, \(-0.000052`\), 13.287985`, 12.478101`, 0.578266`, 
              12.847212`, 10.993116`, 1.323846`, 14.748924`, 
              14.749171`, \(-0.000176`\), 13.341807`, 13.341194`, 0.000438`, 
              12.468743`, 12.468545`, 0.000142`, 12.00067`, 11.739222`, 
              0.186677`, 16.5242`, 15.548422`, 10.674782`, 8.935299`, 
              11.739718`, 12.468743`, 12.00067`, 10.570523`, 14.748924`, 
              13.341807`, 11.69698`, 11.191975`, 12.586797`, 13.287985`, 
              12.847212`, 10.63549`}, {1, 17, 
              1.926366375`*^7, \(-0.25`\), \(-0.376519058`\), 0.01`, 
              1, \(-0.5`\), 0.002008900308`, 
              94730.76694`, \(-1.033359676`\), \(-0.9058218306`\), 3, 4, 5, 
              6, 0.2517`, \(-23.101391`\), 19.457854`, 
              0.063641`, \(-2.147758`\), 1.12655`, \(-1.63101`\), 
              
              3.828051`, \(-1.555148`\), \(-1.538863`\), \(-1.437404`\), \
\(-1.604178`\), 16.5242`, 16.524204`, \(-6.`*^-6\), 14.290287`, 14.290272`, 
              0.000023`, 12.286273`, 13.218754`, \(-1.413936`\), 12.079728`, 
              10.830908`, 1.893606`, 12.478519`, 12.478676`, \(-0.000239`\), 
              10.753598`, 10.752854`, 0.001129`, 10.616112`, 
              10.616724`, \(-0.000928`\), 9.755693`, 9.755248`, 0.000675`, 
              16.5242`, 14.290287`, 12.079728`, 10.830906`, 9.517223`, 
              10.753598`, 9.30877`, 6.919673`, 12.478519`, 10.616112`, 
              9.755693`, 8.47205`, 10.760305`, 12.286273`, 8.583321`, 
              8.308594`}, {5, 29, 
              7.428522539`*^6, \(-0.25`\), \(-0.606470241`\), 0.01`, 
              1, \(-0.4`\), \(-0.09889283434`\), 
              5.154835674`, \(-0.9855949008`\), \(-0.9434009379`\), 3, 4, 5, 
              6, 0.090064`, \(-14.295964`\), 14.056073`, 
              0.105173`, \(-1.056319`\), 1.401629`, \(-0.142341`\), 
              6.663825`, \(-0.038212`\), \(-0.23352`\), 
              0.226737`, \(-0.249966`\), 16.5242`, 
              16.524315`, \(-0.000101`\), 13.793614`, 12.475563`, 1.151898`, 
              12.478387`, 12.400632`, 0.067953`, 12.397435`, 12.184635`, 
              0.185974`, 10.436147`, 10.435411`, 0.000643`, 9.815252`, 
              8.644305`, 1.023338`, 8.637345`, 7.513922`, 0.981806`, 
              8.047136`, 6.998355`, 0.916573`, 16.5242`, 12.478387`, 
              12.397435`, 8.824034`, 8.047136`, 9.815252`, 6.790989`, 
              3.370692`, 10.436147`, 8.637345`, 6.266604`, 4.759241`, 
              9.726462`, 13.793614`, 6.694752`, 3.830599`}, {5, 28, 
              1.495797595`*^6, \(-0.25`\), \(-1.351299931`\), 0.01`, 
              1, \(-0.3`\), \(-0.07748977101`\), 
              0.1082239188`, \(-0.956829222`\), \(-0.9822738604`\), 3, 4, 5, 
              6, 0.148796`, \(-5.244173`\), 5.851735`, 
              0.10926`, \(-0.139812`\), 0.46529`, 0.514277`, 0.80978`, 
              0.550142`, 0.321408`, 0.795423`, 0.502003`, 16.5242`, 
              16.525127`, \(-0.002983`\), 9.703186`, 9.695762`, 0.023884`, 
              7.444968`, 7.440017`, 0.01593`, 7.157944`, 6.397743`, 
              2.445624`, 4.337875`, 3.378537`, 3.086268`, 3.388147`, 
              1.946696`, 4.637261`, 1.686699`, 1.70723`, \(-0.06605`\), 
              1.071959`, 1.072015`, \(-0.000182`\), 16.5242`, 9.703186`, 
              7.444968`, 5.86825`, 1.071959`, 
              4.337875`, \(-2.175229`\), \(-5.699082`\), 
              3.388147`, \(-1.350344`\), 1.686699`, \(-0.768343`\), 
              6.398878`, 7.157944`, 0.852793`, \(-0.609456`\)}, {5, 27, 
              49830.19425`, \(-0.25`\), \(-8.336193114`\), 0.01`, 
              1, \(-0.2`\), \(-0.06631598411`\), 
              24.14568014`, \(-2.533429126`\), \(-4.231994056`\), 3, 4, 5, 6, 
              0.221277`, \(-1.056943`\), 1.557141`, \(-0.071827`\), 
              0.200041`, \(-0.07545`\), 0.65429`, \(-0.100858`\), 
              0.009599`, \(-0.150213`\), \(-0.310321`\), \(-0.268242`\), 
              16.5242`, 16.517579`, 
              0.195401`, \(-6.946579`\), \(-6.941366`\), \(-0.153872`\), \
\(-14.846705`\), \(-14.835378`\), \(-0.334301`\), \(-25.122769`\), \
\(-26.096801`\), 28.74731`, \(-28.667353`\), \(-28.708905`\), 
              1.226343`, \(-37.488868`\), \(-37.510435`\), 
              0.636525`, \(-52.125991`\), \(-55.044823`\), 
              86.145616`, \(-55.216863`\), \(-55.634935`\), 12.338871`, 
              16.5242`, \(-6.946579`\), \(-14.846705`\), \(-25.122769`\), \
\(-55.530098`\), \(-52.125991`\), \(-82.547644`\), \(-82.552119`\), \
\(-28.667353`\), \(-37.488868`\), \(-55.216863`\), \(-67.977397`\), \
\(-31.48102`\), \(-25.792841`\), \(-55.046064`\), \(-58.247323`\)}, {5, 28, 
              9.008209257`, \(-1.236151281`\), 6.97969833`, 0.01`, 
              1, \(-0.1`\), \(-0.1091597113`\), 
              7.07716182`, \(-0.8509371612`\), \(-0.9433926624`\), 3, 4, 5, 
              6, 0.258085`, \(-0.476865`\), 0.324416`, 0.142376`, 
              0.202307`, \(-0.189737`\), 0.693812`, 0.653638`, 0.755688`, 
              0.776278`, 1.046442`, 1.044071`, 16.5242`, 16.560397`, 
              1.043269`, 26.361043`, 26.333356`, \(-0.797991`\), 32.727483`, 
              32.745281`, 0.51297`, 46.971547`, 46.975731`, 0.12057`, 
              25.908451`, 25.933304`, 0.716318`, 48.359397`, 48.386772`, 
              0.789009`, 62.033474`, 62.027647`, \(-0.16794`\), 64.071484`, 
              64.07078`, \(-0.020301`\), 16.5242`, 26.361043`, 32.727483`, 
              49.244703`, 62.033474`, 78.447709`, 92.892727`, 101.669884`, 
              25.908451`, 48.359397`, 64.071484`, 65.962198`, 46.971547`, 
              66.777913`, 75.847514`, 78.319155`}, {5, 28, 
              29.51688771`, \(-2.141412274`\), 4.996720193`, 0.01`, 1, 
              0, \(-0.133348195`\), 
              3.380622694`, \(-0.8339478863`\), \(-1.025519242`\), 3, 4, 5, 
              6, 0.274783`, \(-0.441694`\), 0.215831`, 0.151641`, 
              0.189308`, \(-0.186316`\), 0.703197`, 0.515363`, 0.752514`, 
              0.686921`, 1.037838`, 1.012239`, 16.5242`, 16.559581`, 
              0.777257`, 20.917738`, 20.930313`, 0.276238`, 25.367572`, 
              25.344554`, \(-0.505665`\), 37.377366`, 37.409974`, 0.716334`, 
              18.611904`, 18.626943`, 0.330375`, 38.582632`, 38.603279`, 
              0.45357`, 46.56468`, 46.571154`, 0.142223`, 48.186832`, 
              48.195532`, 0.191129`, 16.5242`, 20.917738`, 25.367572`, 
              37.377366`, 48.186832`, 60.294267`, 77.203787`, 77.522281`, 
              18.611904`, 38.582632`, 46.56468`, 51.859364`, 39.140412`, 
              59.682704`, 60.086675`, 62.48312`}, {2, 28, 
              66.06242648`, \(-2.467059518`\), 3.718715546`, 0.01`, 1, 
              0.1`, \(-0.1149946323`\), 
              9.89566599`, \(-0.8176645768`\), \(-1.01729208`\), 3, 4, 5, 6, 
              0.287949`, \(-0.454716`\), 0.252961`, 0.155555`, 
              0.176801`, \(-0.174592`\), 0.70019`, 0.399718`, 0.756165`, 
              0.619803`, 1.047444`, 0.981802`, 8.45066`, 8.508961`, 
              0.998862`, 16.5242`, 16.543321`, 0.327593`, 31.455679`, 
              31.428046`, \(-0.473429`\), 31.492008`, 
              31.456972`, \(-0.600261`\), 13.648176`, 13.656177`, 0.137084`, 
              30.346621`, 30.35899`, 0.211923`, 34.751834`, 34.753995`, 
              0.037024`, 37.16484`, 37.169504`, 0.079907`, 16.5242`, 
              31.492008`, 8.45066`, 35.939133`, 37.16484`, 46.260494`, 
              59.122439`, 61.069775`, 13.648176`, 30.346621`, 34.751834`, 
              40.080894`, 31.455679`, 47.010604`, 47.524402`, 
              51.072822`}, {\(-3\), 6, 129.1806281`, \(-1.839562728`\), 
              3.428841837`, 0.01`, 1, 0.2`, \(-0.10115`\), 
              50, \(-0.97786`\), \(-0.947302`\), 3, 4, 5, 6, 
              0.279799`, \(-0.480612`\), 0.329503`, 0.154282`, 
              0.168877`, \(-0.162412`\), 0.685397`, 0.359155`, 0.748336`, 
              0.592811`, 1.043094`, 0.958934`, 16.5242`, 16.538395`, 
              0.217901`, 17.710507`, 17.715386`, 0.074886`, 20.835689`, 
              20.828274`, \(-0.113824`\), 29.597346`, 29.59813`, 0.012024`, 
              12.066521`, 12.072416`, 0.090488`, 27.947639`, 27.957439`, 
              0.150432`, 31.750424`, 31.750913`, 0.007493`, 34.134054`, 
              34.137173`, 0.047883`, 16.5242`, 17.710507`, 20.835689`, 
              35.526027`, 34.134054`, 42.408543`, 54.590488`, 56.98123`, 
              12.066521`, 27.947639`, 31.750424`, 37.119495`, 29.597346`, 
              44.073313`, 44.342662`, 48.006115`}, {\(-3\), 6, 
              127.8770733`, \(-1.487542317`\), 3.454070038`, 0.01`, 1, 
              0.3`, \(-0.10115`\), 50, \(-0.97786`\), \(-0.947302`\), 3, 4, 
              5, 6, 0.257329`, \(-0.528199`\), 0.457073`, 0.15973`, 
              0.151303`, \(-0.141769`\), 0.662203`, 0.350265`, 0.721074`, 
              0.592501`, 1.009577`, 0.970813`, 16.5242`, 16.538525`, 
              0.203723`, 17.301724`, 17.30501`, 0.046738`, 21.437317`, 
              21.431576`, \(-0.081647`\), 29.482225`, 29.482924`, 0.009944`, 
              11.396843`, 11.40229`, 0.077463`, 27.189383`, 27.198486`, 
              0.129468`, 31.397685`, 31.397772`, 0.001236`, 33.431769`, 
              33.434765`, 0.042607`, 16.5242`, 17.301724`, 21.437317`, 
              35.284795`, 33.431769`, 41.550848`, 54.026969`, 56.39213`, 
              11.396843`, 27.189383`, 31.397685`, 36.847427`, 29.482225`, 
              43.936304`, 44.164127`, 49.727813`}, {\(-3\), 6, 
              125.8553012`, \(-1.250953521`\), 3.582417066`, 0.01`, 1, 
              0.4`, \(-0.10115`\), 50, \(-0.97786`\), \(-0.947302`\), 3, 4, 
              5, 6, 0.229115`, \(-0.600675`\), 0.64144`, 0.164362`, 
              0.134246`, \(-0.120378`\), 0.63331`, 0.345885`, 0.687065`, 
              0.592811`, 0.9678`, 0.980266`, 16.5242`, 16.538266`, 0.184719`, 
              17.204253`, 17.207389`, 0.041176`, 22.196806`, 
              22.192185`, \(-0.060691`\), 29.709838`, 29.710515`, 0.00889`, 
              10.889848`, 10.895151`, 0.069654`, 26.758993`, 26.767498`, 
              0.111705`, 31.781816`, 31.781827`, 0.000152`, 33.234909`, 
              33.237884`, 0.03906`, 16.5242`, 17.204253`, 22.196806`, 
              35.797779`, 33.234909`, 41.381481`, 54.560149`, 56.844082`, 
              10.889848`, 26.758993`, 31.781816`, 37.19175`, 29.709838`, 
              44.423198`, 44.952653`, 50.692421`}, {5, 29, 
              2.633548841`*^7, \(-0.25`\), \(-0.3220594993`\), 0.0225`, 
              1, \(-0.6`\), \(-0.1031701646`\), 
              6.662114829`, \(-0.9816761367`\), \(-1.061918988`\), 3, 4, 5, 
              6, 0.109639`, \(-28.903497`\), 26.876045`, 
              0.060099`, \(-2.750474`\), 1.149188`, \(-2.126509`\), 
              8.032322`, \(-2.040458`\), \(-2.140833`\), \(-1.911901`\), \
\(-2.220126`\), 16.5242`, 16.524279`, \(-0.000044`\), 14.260745`, 
              14.260835`, \(-0.000051`\), 12.603638`, 
              13.566353`, \(-0.543898`\), 12.294865`, 11.757394`, 0.303651`, 
              12.85116`, 12.851296`, \(-0.000077`\), 11.410658`, 11.40985`, 
              0.000456`, 11.381912`, 11.380638`, 0.000719`, 10.385937`, 
              10.388185`, \(-0.00127`\), 16.5242`, 14.260745`, 12.294865`, 
              11.757314`, 10.04882`, 11.410658`, 9.693331`, 9.426902`, 
              12.85116`, 11.381912`, 10.385937`, 8.934024`, 11.132226`, 
              12.603638`, 11.077986`, 8.696344`}, {5, 29, 
              1.192946891`*^7, \(-0.25`\), \(-0.4785834977`\), 0.0225`, 
              1, \(-0.5`\), \(-0.1031849794`\), 
              7.998429039`, \(-0.9662418898`\), \(-1.061252067`\), 3, 4, 5, 
              6, 0.109412`, \(-18.037785`\), 17.219791`, 
              0.10096`, \(-1.445703`\), 1.550946`, \(-0.556707`\), 
              8.367572`, \(-0.4327`\), \(-0.57366`\), \(-0.183373`\), \
\(-0.630619`\), 16.5242`, 16.524299`, \(-0.000083`\), 13.456927`, 13.45438`, 
              0.002134`, 13.020319`, 12.686299`, 0.279844`, 11.620509`, 
              11.623289`, \(-0.002329`\), 11.025884`, 11.025206`, 0.000568`, 
              10.831891`, 9.358379`, 1.234517`, 9.35467`, 8.555019`, 
              0.669952`, 8.410772`, 8.442445`, \(-0.026536`\), 16.5242`, 
              13.456927`, 11.620509`, 9.805998`, 8.410772`, 10.831891`, 
              7.008472`, 6.358617`, 11.025884`, 9.35467`, 8.182877`, 
              6.270082`, 9.956577`, 13.020319`, 7.26279`, 6.825762`}, {5, 28, 
              3.620314545`*^6, \(-0.25`\), \(-0.8685398007`\), 0.0225`, 
              1, \(-0.4`\), \(-0.08203894327`\), \(-1.032995965`\), \
\(-0.9406980734`\), \(-1.083271536`\), 3, 4, 5, 6, 0.158635`, \(-8.583046`\), 
              8.567618`, 0.119917`, \(-0.418026`\), 0.905181`, 0.382732`, 
              2.240106`, 0.430997`, 0.243278`, 0.688804`, 0.363598`, 
              16.5242`, 16.525228`, \(-0.002267`\), 12.647554`, 12.185739`, 
              1.01807`, 12.193603`, 8.984331`, 7.074836`, 8.989446`, 
              8.044656`, 2.082789`, 7.069767`, 7.067943`, 0.004022`, 
              6.612347`, 4.748631`, 4.108558`, 4.7249`, 4.378388`, 0.763886`, 
              4.456848`, 4.348419`, 0.23903`, 16.5242`, 12.193603`, 
              8.989446`, 5.738777`, 4.456848`, 6.612347`, 
              3.939628`, \(-0.951368`\), 7.069767`, 4.7249`, 2.287955`, 
              1.821015`, 8.045241`, 12.647554`, 
              4.473932`, \(-0.778442`\)}, {2, 8, 
              643653.8747`, \(-0.25`\), \(-2.067491711`\), 0.0225`, 
              1, \(-0.3`\), \(-0.105313`\), 50, \(-0.97786`\), \(-1.06285`\), 
              3, 4, 5, 6, 0.179009`, \(-3.092297`\), 4.164351`, 0.126034`, 
              0.024724`, 0.164128`, 0.623242`, 0.322601`, 0.660565`, 
              0.420507`, 0.924448`, 0.696798`, 16.5242`, 16.522739`, 
              0.008649`, 8.611352`, 8.611101`, 0.001488`, 6.854668`, 
              6.856493`, \(-0.010811`\), 5.104672`, 0.408385`, 
              27.809558`, \(-3.303127`\), \(-3.295065`\), \(-0.04774`\), \
\(-3.742161`\), \(-4.318609`\), 3.413498`, \(-4.287562`\), \(-6.163434`\), 
              11.108175`, \(-6.188413`\), \(-7.468716`\), 7.581452`, 
              16.5242`, 6.854668`, 
              8.611352`, \(-4.236226`\), \(-7.466303`\), \(-3.742161`\), \
\(-13.938811`\), \(-14.286066`\), \(-4.287562`\), \(-3.303127`\), \
\(-6.188413`\), \(-10.171255`\), 0.412567`, 
              5.104672`, \(-6.256289`\), \(-10.073558`\)}, {5, 26, 
              943060.0478`, \(-0.494918491`\), 30.07441666`, 0.0225`, 
              1, \(-0.2`\), \(-0.09401077099`\), 
              4194.792665`, \(-1.959242418`\), \(-1.633661811`\), 3, 4, 5, 6, 
              0.269532`, \(-0.67241`\), 0.750384`, 0.032962`, 
              0.224982`, \(-0.136711`\), 0.706814`, 0.730875`, 0.647926`, 
              0.737541`, 0.851239`, 0.757037`, 16.5242`, 16.591984`, 
              8.791343`, 78.471786`, 78.414545`, \(-7.424019`\), 142.337634`, 
              154.058475`, 1520.150051`, 153.987201`, 156.283726`, 
              297.850887`, 115.12916`, 115.243595`, 14.841819`, 189.835577`, 
              189.93905`, 13.420031`, 251.750473`, 251.887037`, 17.711797`, 
              252.501564`, 252.189408`, \(-40.485606`\), 16.5242`, 
              78.471786`, 142.337634`, 153.987201`, 251.750473`, 320.550897`, 
              353.894432`, 416.10425`, 115.12916`, 189.835577`, 252.501564`, 
              280.884381`, 174.738236`, 222.531368`, 294.898233`, 
              298.505395`}, {5, 28, 10.94658073`, \(-1.453007445`\), 
              5.784950061`, 0.0225`, 1, \(-0.1`\), \(-0.1259055737`\), 
              3.209367586`, \(-0.8195469802`\), \(-1.092366238`\), 3, 4, 5, 
              6, 0.303662`, \(-0.458268`\), 0.238924`, 0.161924`, 
              0.169067`, \(-0.167207`\), 0.734149`, 0.6328`, 0.783837`, 
              0.78378`, 1.07563`, 1.101041`, 16.5242`, 16.560755`, 1.027452`, 
              23.902627`, 23.9175`, 0.418036`, 26.919826`, 
              26.89519`, \(-0.692437`\), 42.527092`, 42.555192`, 0.789788`, 
              23.146091`, 23.163524`, 0.489982`, 45.661529`, 45.683714`, 
              0.623527`, 55.778237`, 55.775443`, \(-0.078522`\), 57.148697`, 
              57.162344`, 0.38358`, 16.5242`, 26.919826`, 23.902627`, 
              42.527092`, 57.148697`, 71.280722`, 88.503599`, 91.310631`, 
              23.146091`, 45.661529`, 55.778237`, 59.987168`, 44.791111`, 
              65.978227`, 69.265631`, 73.230265`}, {5, 28, 
              63.73287396`, \(-2.276069568`\), 4.023839166`, 0.0225`, 1, 
              0, \(-0.1452608586`\), 
              5.573721298`, \(-0.8338982593`\), \(-1.12877363`\), 3, 4, 5, 6, 
              0.319605`, \(-0.444055`\), 0.183553`, 0.166693`, 
              0.159697`, \(-0.1615`\), 0.741513`, 0.4679`, 0.78704`, 
              0.675351`, 1.078548`, 1.04597`, 16.5242`, 16.550997`, 
              0.551401`, 20.008208`, 20.016468`, 0.169956`, 23.337831`, 
              23.319217`, \(-0.383008`\), 35.012502`, 35.014545`, 0.042041`, 
              16.130178`, 16.140692`, 0.21634`, 34.482137`, 34.497724`, 
              0.320733`, 39.172721`, 39.176702`, 0.081923`, 42.405085`, 
              42.411108`, 0.12392`, 16.5242`, 20.008208`, 23.337831`, 
              35.132353`, 42.405085`, 52.402182`, 66.281264`, 69.230799`, 
              16.130178`, 34.482137`, 39.172721`, 45.144748`, 35.012502`, 
              52.038583`, 54.001417`, 55.027584`}, {5, 29, 
              108.8296774`, \(-2.630364607`\), 3.510792405`, 0.0225`, 1, 
              0.1`, \(-0.1269689286`\), 
              7.317580709`, \(-0.9361354211`\), \(-1.082444083`\), 3, 4, 5, 
              6, 0.331848`, \(-0.454705`\), 0.216088`, 0.164342`, 
              0.157735`, \(-0.157869`\), 0.73928`, 0.412048`, 0.793739`, 
              0.637105`, 1.092476`, 1.014852`, 16.5242`, 16.540806`, 
              0.309541`, 19.059323`, 19.066401`, 0.131946`, 22.600241`, 
              22.588559`, \(-0.217754`\), 32.565708`, 32.566777`, 0.019935`, 
              13.902493`, 13.909362`, 0.128036`, 31.425895`, 31.437485`, 
              0.216044`, 34.280911`, 34.282126`, 0.022662`, 38.160973`, 
              38.164752`, 0.070439`, 16.5242`, 19.059323`, 22.600241`, 
              33.356992`, 38.160973`, 46.888237`, 59.482513`, 62.019631`, 
              13.902493`, 31.425895`, 34.280911`, 40.792879`, 32.565708`, 
              47.484793`, 48.657934`, 53.164748`}, {5, 29, 
              112.9808971`, \(-1.994279081`\), 3.482257728`, 0.0225`, 1, 
              0.2`, \(-0.1479535464`\), 
              8.096042444`, \(-0.9287950319`\), \(-1.081005278`\), 3, 4, 5, 
              6, 0.314761`, \(-0.478668`\), 0.278026`, 0.16923`, 
              0.143803`, \(-0.142326`\), 0.725172`, 0.401571`, 0.777257`, 
              0.635881`, 1.07227`, 1.025736`, 16.5242`, 16.539489`, 
              0.268129`, 19.151355`, 19.159893`, 0.149742`, 23.653739`, 
              23.642545`, \(-0.196314`\), 32.614588`, 32.615757`, 0.020498`, 
              13.227013`, 13.233897`, 0.120722`, 30.674396`, 30.686344`, 
              0.209541`, 33.595018`, 33.596124`, 0.019381`, 37.483927`, 
              37.488094`, 0.073082`, 16.5242`, 19.151355`, 23.653739`, 
              33.77288`, 37.483927`, 45.984458`, 58.642161`, 61.216077`, 
              13.227013`, 30.674396`, 33.595018`, 40.588852`, 32.614588`, 
              47.496589`, 48.158367`, 53.901579`}, {\(-3\), 6, 
              136.7511508`, \(-1.611212376`\), 3.185194472`, 0.0225`, 1, 
              0.3`, \(-0.105313`\), 50, \(-0.97786`\), \(-1.06285`\), 3, 4, 
              5, 6, 0.30677`, \(-0.498904`\), 0.352587`, 0.171215`, 
              0.133181`, \(-0.129012`\), 0.708302`, 0.362106`, 0.76076`, 
              0.6149`, 1.053897`, 1.016649`, 16.5242`, 16.535361`, 0.174497`, 
              18.189527`, 18.193229`, 0.057868`, 22.011861`, 
              22.006475`, \(-0.084206`\), 29.958669`, 29.959204`, 0.008369`, 
              11.59574`, 11.600216`, 0.069965`, 27.808921`, 27.817648`, 
              0.136436`, 30.386256`, 30.384914`, \(-0.020991`\), 33.868679`, 
              33.870976`, 0.035905`, 16.5242`, 18.189527`, 22.011861`, 
              34.635567`, 33.868679`, 41.438657`, 53.223792`, 55.399726`, 
              11.59574`, 27.808921`, 30.386256`, 36.869365`, 29.958669`, 
              43.671646`, 43.67654`, 49.462058`}, {\(-3\), 6, 
              133.8619587`, \(-1.366491062`\), 3.29380021`, 0.0225`, 1, 
              0.4`, \(-0.105313`\), 50, \(-0.97786`\), \(-1.06285`\), 3, 4, 
              5, 6, 0.280213`, \(-0.547926`\), 0.483291`, 0.174836`, 
              0.119788`, \(-0.112124`\), 0.683672`, 0.360321`, 0.731337`, 
              0.615895`, 1.017388`, 1.024422`, 16.5242`, 16.535225`, 
              0.162806`, 18.312974`, 18.316438`, 0.051155`, 22.787849`, 
              22.783334`, \(-0.066684`\), 30.272172`, 30.272701`, 0.007813`, 
              11.221133`, 11.225501`, 0.064514`, 27.588882`, 27.597055`, 
              0.120697`, 30.685657`, 30.684274`, \(-0.02042`\), 33.809696`, 
              33.81202`, 0.034328`, 16.5242`, 18.312974`, 22.787849`, 
              35.417148`, 33.809696`, 41.377227`, 53.771902`, 55.837078`, 
              11.221133`, 27.588882`, 30.685657`, 37.274693`, 30.272172`, 
              44.2189`, 44.390805`, 50.064773`}, {5, 29, 
              1.643046719`*^7, \(-0.25`\), \(-0.4077934502`\), 0.04`, 
              1, \(-0.6`\), \(-0.1142928815`\), 
              9.903882531`, \(-0.9743707185`\), \(-1.163489327`\), 3, 4, 5, 
              6, 0.122745`, \(-21.012492`\), 19.561976`, 
              0.094539`, \(-1.745075`\), 1.607799`, \(-0.868499`\), 
              9.521999`, \(-0.735253`\), \(-0.86627`\), \(-0.483834`\), \
\(-0.950758`\), 16.5242`, 16.524287`, \(-0.00007`\), 13.837353`, 
              13.837406`, \(-0.000043`\), 12.430278`, 
              12.836223`, \(-0.325109`\), 11.270372`, 
              11.270514`, \(-0.000113`\), 11.376301`, 11.376169`, 0.000106`, 
              10.817957`, 9.586791`, 0.986006`, 9.586938`, 8.946283`, 
              0.513083`, 8.945107`, 8.828676`, 0.093247`, 16.5242`, 
              13.837353`, 11.270372`, 10.379192`, 8.552656`, 10.817957`, 
              8.355228`, 7.374395`, 11.376301`, 9.586938`, 8.945107`, 
              6.929133`, 10.043655`, 12.430278`, 9.449583`, 7.215543`}, {5, 
              28, 6.393883038`*^6, \(-0.25`\), \(-0.6535739388`\), 0.04`, 
              1, \(-0.5`\), \(-0.0891929837`\), \(-0.9005279395`\), \
\(-0.9378668743`\), \(-1.191775225`\), 3, 4, 5, 6, 
              0.171811`, \(-11.506438`\), 10.879187`, 
              0.123135`, \(-0.683719`\), 1.276948`, 0.204127`, 4.063238`, 
              0.283433`, 0.133727`, 0.56984`, 0.187873`, 16.5242`, 
              16.524804`, \(-0.001085`\), 12.248696`, 12.244537`, 0.007473`, 
              12.091322`, 8.490295`, 6.469832`, 8.491809`, 7.993285`, 
              0.89568`, 8.492043`, 8.491262`, 0.001405`, 7.449067`, 
              6.781036`, 1.200226`, 6.77035`, 5.64181`, 2.027606`, 6.169016`, 
              4.752647`, 2.544737`, 16.5242`, 12.248696`, 7.985728`, 
              7.094805`, 5.657721`, 7.449067`, 6.169016`, 2.697677`, 
              8.492043`, 6.77035`, 3.746935`, 2.750533`, 8.491809`, 
              12.091322`, 4.615606`, 3.785731`}, {5, 29, 
              1.792533943`*^6, \(-0.25`\), \(-1.236050521`\), 0.04`, 
              1, \(-0.4`\), \(-0.1162524213`\), 
              1.691846256`, \(-0.9732917658`\), \(-1.179548744`\), 3, 4, 5, 
              6, 0.176331`, \(-5.356492`\), 6.057635`, 
              0.135333`, \(-0.109721`\), 0.393824`, 0.57642`, 0.719277`, 
              0.605607`, 0.390066`, 0.866431`, 0.648923`, 16.5242`, 
              16.524567`, \(-0.00128`\), 10.867591`, 8.822473`, 7.131871`, 
              8.822104`, 6.860483`, 6.840694`, 6.858411`, 4.17093`, 
              9.371958`, 1.869221`, 1.445948`, 1.476065`, 1.463242`, 
              1.11053`, 1.229999`, 1.097367`, \(-0.609092`\), 5.950876`, 
              0.091761`, \(-0.667514`\), 2.647795`, 16.5242`, 8.822104`, 
              6.858411`, 3.421425`, \(-0.608699`\), 1.869221`, 
              0.091761`, \(-8.626441`\), 1.463242`, \(-0.668778`\), 
              1.097367`, \(-2.334661`\), 4.720769`, 10.867591`, 
              0.528517`, \(-6.009758`\)}, {5, 28, 
              123265.3162`, \(-0.25`\), \(-4.796075652`\), 0.04`, 
              1, \(-0.3`\), 0.04245561783`, 
              244.034081`, \(-2.405016816`\), \(-3.149824412`\), 3, 4, 5, 6, 
              0.352663`, \(-0.848453`\), 1.107367`, 0.012923`, 
              0.118809`, \(-0.003832`\), 0.709158`, 0.154271`, 0.375831`, 
              0.166773`, 0.361468`, 0.192507`, 16.5242`, 
              16.524449`, \(-0.006748`\), \(-9.483513`\), \(-9.484159`\), 
              0.017482`, \(-11.932493`\), \(-12.776262`\), 
              22.834377`, \(-12.454208`\), \(-13.696588`\), 
              33.621745`, \(-19.19554`\), \(-19.195664`\), 
              0.00337`, \(-28.021829`\), \(-28.027663`\), 
              0.157875`, \(-31.697147`\), \(-31.672986`\), \(-0.653862`\), \
\(-37.567533`\), \(-37.62225`\), 1.480767`, 
              16.5242`, \(-9.483513`\), \(-11.932493`\), \(-12.789187`\), \
\(-37.567533`\), \(-42.310465`\), \(-46.01705`\), \(-59.761923`\), \
\(-19.19554`\), \(-28.021829`\), \(-31.697147`\), \(-43.536072`\), \
\(-21.973395`\), \(-12.454208`\), \(-41.218747`\), \(-46.699248`\)}, {5, 27, 
              11.31389915`, \(-0.8751898375`\), 7.692328041`, 0.04`, 
              1, \(-0.2`\), \(-0.11530574`\), 
              3598.750946`, \(-1.266607318`\), \(-1.300919284`\), 3, 4, 5, 6, 
              0.323695`, \(-0.529536`\), 0.395481`, 0.131402`, 
              0.181303`, \(-0.150695`\), 0.754013`, 0.721949`, 0.776526`, 
              0.813697`, 1.053106`, 1.031766`, 16.5242`, 16.549415`, 
              1.00457`, 30.313825`, 30.292781`, \(-0.838399`\), 43.796651`, 
              43.813265`, 0.661924`, 56.068638`, 56.071707`, 0.122257`, 
              32.163988`, 32.187246`, 0.926592`, 59.271503`, 59.298939`, 
              1.093055`, 74.682754`, 74.654952`, \(-1.107619`\), 75.007551`, 
              74.994343`, \(-0.526211`\), 16.5242`, 30.313825`, 43.796651`, 
              57.999138`, 75.007551`, 93.908656`, 113.291752`, 120.391695`, 
              32.163988`, 59.271503`, 74.682754`, 76.732887`, 56.068638`, 
              74.648699`, 93.235038`, 95.93696`}, {5, 29, 
              77.33960345`, \(-1.740557201`\), 3.919426953`, 0.04`, 
              1, \(-0.1`\), \(-0.1243328603`\), 
              11.55419182`, \(-0.9147696679`\), \(-1.198856959`\), 3, 4, 5, 
              6, 0.362641`, \(-0.441628`\), 0.164967`, 0.169099`, 
              0.150429`, \(-0.15045`\), 0.775365`, 0.485804`, 0.821634`, 
              0.688056`, 1.120835`, 1.05818`, 16.5242`, 16.543989`, 
              0.450039`, 21.838278`, 21.820431`, \(-0.405875`\), 21.928628`, 
              21.93803`, 0.213812`, 34.796398`, 34.7977`, 0.029614`, 
              16.952739`, 16.961651`, 0.202659`, 35.366699`, 35.379763`, 
              0.297097`, 39.62172`, 39.623778`, 0.046789`, 43.117657`, 
              43.121352`, 0.084016`, 16.5242`, 21.838278`, 21.928628`, 
              39.296614`, 43.117657`, 52.988395`, 67.148048`, 68.070398`, 
              16.952739`, 35.366699`, 39.62172`, 44.661787`, 34.796398`, 
              50.649673`, 53.27294`, 54.713627`}, {5, 28, 
              60.03916606`, \(-2.362904023`\), 3.971974175`, 0.04`, 1, 
              0, \(-0.160658581`\), 
              4.306059155`, \(-0.8397925747`\), \(-1.222272465`\), 3, 4, 5, 
              6, 0.3666`, \(-0.44181`\), 0.140362`, 0.178904`, 
              0.13384`, \(-0.137728`\), 0.779423`, 0.506822`, 0.823065`, 
              0.714747`, 1.121925`, 1.10163`, 16.5242`, 16.547373`, 
              0.539884`, 21.708191`, 21.7213`, 0.305433`, 26.158873`, 
              26.140412`, \(-0.430105`\), 33.78616`, 33.804946`, 0.437685`, 
              17.08482`, 17.094035`, 0.214691`, 37.477899`, 37.492808`, 
              0.347338`, 40.369665`, 40.372809`, 0.073249`, 45.59803`, 
              45.602792`, 0.110953`, 16.5242`, 21.708191`, 26.158873`, 
              33.78616`, 45.59803`, 55.656983`, 70.0644`, 72.826013`, 
              17.08482`, 37.477899`, 40.369665`, 48.148556`, 38.167625`, 
              55.250186`, 57.418009`, 61.236348`}, {5, 28, 
              69.26727761`, \(-2.592109115`\), 3.805378611`, 0.04`, 1, 
              0.1`, \(-0.1923747589`\), 
              4.601575921`, \(-0.841050672`\), \(-1.214560302`\), 3, 4, 5, 6, 
              0.35951`, \(-0.457226`\), 0.166854`, 0.182087`, 
              0.124991`, \(-0.128331`\), 0.77406`, 0.484211`, 0.818223`, 
              0.703382`, 1.117048`, 1.102393`, 16.5242`, 16.546087`, 
              0.479081`, 21.951053`, 21.96632`, 0.334176`, 27.452964`, 
              27.434886`, \(-0.395716`\), 33.186398`, 33.206225`, 0.434001`, 
              16.011354`, 16.020353`, 0.196992`, 36.186888`, 36.202139`, 
              0.333826`, 38.5491`, 38.552254`, 0.069025`, 44.214532`, 
              44.218862`, 0.094783`, 16.5242`, 21.951053`, 27.452964`, 
              33.186398`, 44.214532`, 53.769586`, 67.742161`, 70.628101`, 
              16.011354`, 36.186888`, 38.5491`, 47.309742`, 37.901607`, 
              54.340776`, 55.986852`, 61.833961`}, {\(-3\), 6, 
              146.6684051`, \(-2.11371944`\), 2.926449054`, 0.04`, 1, 
              0.2`, \(-0.116761`\), 50, \(-0.97786`\), \(-1.16519`\), 3, 4, 
              5, 6, 0.376631`, \(-0.45669`\), 0.201605`, 0.178062`, 
              0.126577`, \(-0.128046`\), 0.768796`, 0.375075`, 0.82118`, 
              0.633096`, 1.125939`, 1.048477`, 16.5242`, 16.533488`, 
              0.163788`, 18.921959`, 18.92678`, 0.085024`, 22.044205`, 
              22.037742`, \(-0.113972`\), 30.350682`, 30.35116`, 0.008432`, 
              12.162254`, 12.166349`, 0.072213`, 28.520223`, 28.528826`, 
              0.151714`, 29.958269`, 29.95684`, \(-0.025188`\), 34.654956`, 
              34.657022`, 0.036421`, 16.5242`, 18.921959`, 22.044205`, 
              33.435489`, 34.654956`, 41.88068`, 52.816191`, 54.949921`, 
              12.162254`, 28.520223`, 29.958269`, 36.998612`, 30.350682`, 
              43.027052`, 43.749508`, 48.952099`}, {\(-3\), 6, 
              144.7869112`, \(-1.735045092`\), 2.981144049`, 0.04`, 1, 
              0.3`, \(-0.116761`\), 50, \(-0.97786`\), \(-1.16519`\), 3, 4, 
              5, 6, 0.357865`, \(-0.47309`\), 0.254884`, 0.181272`, 
              0.116035`, \(-0.115404`\), 0.752647`, 0.3746`, 0.801673`, 
              0.635123`, 1.101623`, 1.056583`, 16.5242`, 16.533016`, 
              0.150485`, 19.325038`, 19.329447`, 0.075259`, 22.952318`, 
              22.946924`, \(-0.092072`\), 30.784201`, 30.784657`, 0.007787`, 
              11.880109`, 11.884`, 0.066411`, 28.515562`, 28.524729`, 
              0.156475`, 29.964329`, 29.961639`, \(-0.045913`\), 34.68644`, 
              34.688483`, 0.034872`, 16.5242`, 19.325038`, 22.952318`, 
              33.950323`, 34.68644`, 41.8391`, 53.145189`, 55.156712`, 
              11.880109`, 28.515562`, 29.964329`, 37.407875`, 30.784201`, 
              43.898303`, 43.90261`, 49.463007`}, {\(-3\), 6, 
              141.2051237`, \(-1.482197156`\), 3.065371222`, 0.04`, 1, 
              0.4`, \(-0.116761`\), 50, \(-0.97786`\), \(-1.16519`\), 3, 4, 
              5, 6, 0.333223`, \(-0.502668`\), 0.340553`, 0.184088`, 
              0.105683`, \(-0.102342`\), 0.731825`, 0.37359`, 0.776483`, 
              0.636106`, 1.070174`, 1.062769`, 16.5242`, 16.533001`, 
              0.143839`, 19.536427`, 19.540411`, 0.065108`, 23.689121`, 
              23.684492`, \(-0.075656`\), 31.094131`, 31.094593`, 0.007562`, 
              11.578194`, 11.581985`, 0.061965`, 28.530563`, 28.540609`, 
              0.164184`, 29.97773`, 29.973623`, \(-0.067116`\), 34.651937`, 
              34.653987`, 0.033511`, 16.5242`, 19.536427`, 23.689121`, 
              34.654718`, 34.651937`, 41.760192`, 53.562023`, 55.444579`, 
              11.578194`, 28.530563`, 29.97773`, 37.773937`, 31.094131`, 
              44.44334`, 44.395965`, 49.943331`}, {5, 28, 
              9.320991871`*^6, \(-0.25`\), \(-0.5413412807`\), 0.0625`, 
              1, \(-0.6`\), \(-0.1078192777`\), \(-0.3484374398`\), \
\(-0.9521417165`\), \(-1.295891365`\), 3, 4, 5, 6, 
              0.176207`, \(-13.982465`\), 12.790641`, 
              0.122383`, \(-0.898777`\), 1.51524`, 0.031714`, 5.797078`, 
              0.140249`, 0.012049`, 0.456361`, 0.016503`, 16.5242`, 
              16.524637`, \(-0.000666`\), 11.716833`, 
              11.720092`, \(-0.004974`\), 11.537258`, 10.195394`, 2.047967`, 
              10.196012`, 7.730907`, 3.762271`, 9.229892`, 9.229684`, 
              0.000317`, 8.047336`, 7.414824`, 0.965347`, 7.408648`, 
              6.241557`, 1.781228`, 6.24186`, 5.289193`, 1.453972`, 16.5242`, 
              11.716833`, 10.196012`, 5.710827`, 6.24186`, 8.047336`, 
              6.191916`, 6.071148`, 9.229892`, 7.408648`, 5.296312`, 
              3.236042`, 8.705021`, 11.537258`, 7.170596`, 5.173401`}, {5, 
              28, 3.275062578`*^6, \(-0.25`\), \(-0.9131434127`\), 0.0625`, 
              1, \(-0.5`\), \(-0.1073754241`\), 
              0.760786388`, \(-0.9700368382`\), \(-1.269370403`\), 3, 4, 5, 
              6, 0.201281`, \(-7.002424`\), 7.17638`, 
              0.142611`, \(-0.216505`\), 0.577889`, 0.524545`, 1.25252`, 
              0.557434`, 0.372891`, 0.82606`, 0.614132`, 16.5242`, 
              16.524895`, \(-0.002045`\), 10.162136`, 9.069915`, 3.211973`, 
              9.070238`, 7.662652`, 4.139389`, 7.659899`, 5.99797`, 
              4.887354`, 3.856933`, 3.849319`, 0.022392`, 3.697074`, 
              2.38267`, 3.865364`, 3.373565`, 1.716836`, 4.872065`, 
              2.389716`, 1.527635`, 2.535184`, 16.5242`, 9.070238`, 
              7.659899`, 5.421135`, 1.717675`, 3.373565`, 
              3.697074`, \(-2.501814`\), 3.856933`, 1.5155`, 2.389716`, 
              0.391449`, 6.001791`, 10.162136`, 1.142104`, 0.350827`}, {5, 
              28, 535695.7342`, \(-0.25`\), \(-2.659945902`\), 0.0625`, 
              1, \(-0.4`\), \(-0.1119807977`\), \(-3.066822562`\), \
\(-1.000986975`\), \(-1.300119126`\), 3, 4, 5, 6, 0.249026`, \(-2.688157`\), 
              3.780767`, 0.154372`, 0.023379`, 0.11555`, 0.691734`, 
              0.315341`, 0.715256`, 0.501158`, 0.988393`, 0.844312`, 
              19.389276`, 19.382304`, 0.073892`, 16.5242`, 16.514931`, 
              0.098234`, 6.506096`, 6.507581`, \(-0.015736`\), 6.394657`, 
              4.033075`, 25.028818`, \(-6.475281`\), \(-6.476451`\), 
              0.012407`, \(-7.76538`\), \(-8.38753`\), 
              6.593753`, \(-8.363537`\), \(-10.15916`\), 
              19.030595`, \(-10.1825`\), \(-13.17542`\), 31.719941`, 
              16.5242`, 19.389276`, 6.506096`, 
              4.033961`, \(-13.170136`\), \(-14.197193`\), \(-7.76538`\), \
\(-31.655609`\), \(-8.363537`\), \(-6.475281`\), \(-10.1825`\), \(-18.487483`\
\), \(-2.942899`\), 6.394657`, \(-13.44461`\), \(-19.345031`\)}, {5, 28, 
              50194.02263`, \(-0.25`\), \(-13.17266784`\), 0.0625`, 
              1, \(-0.3`\), \(-0.1576195314`\), 
              9.846215792`, \(-2.937476654`\), \(-5.0311637`\), 3, 4, 5, 6, 
              0.289892`, \(-1.116986`\), 1.520098`, \(-0.033631`\), 
              0.174332`, \(-0.084618`\), 0.759122`, \(-0.076123`\), 
              0.086981`, \(-0.089829`\), \(-0.207063`\), \(-0.127259`\), 
              16.5242`, 16.475113`, 
              2.99913`, \(-25.792195`\), \(-25.748353`\), \(-2.678731`\), \
\(-52.432227`\), \(-58.484357`\), 
              369.775532`, \(-62.04861`\), \(-62.099348`\), 
              3.100007`, \(-54.255036`\), \(-54.320296`\), 
              3.987317`, \(-80.250233`\), \(-80.313463`\), 
              3.863253`, \(-112.183043`\), \(-112.211848`\), 
              1.75997`, \(-112.350252`\), \(-112.24091`\), \(-6.680613`\), 
              16.5242`, \(-25.792195`\), \(-52.432227`\), \(-62.062528`\), \
\(-112.183043`\), \(-145.160308`\), \(-118.569946`\), \(-197.969937`\), \
\(-54.255036`\), \(-80.250233`\), \(-112.350252`\), \(-125.648862`\), \
\(-73.172321`\), \(-62.04861`\), \(-135.303604`\), \(-141.683415`\)}, {5, 29, 
              12.9314213`, \(-1.080745575`\), 5.479294116`, 0.0625`, 
              1, \(-0.2`\), \(-0.1543589427`\), 
              5.936100587`, \(-0.8574478199`\), \(-1.19306886`\), 3, 4, 5, 6, 
              0.36886`, \(-0.473193`\), 0.213843`, 0.181017`, 
              0.123897`, \(-0.12187`\), 0.796898`, 0.689661`, 0.853841`, 
              0.848051`, 1.166097`, 1.190826`, 16.5242`, 16.554962`, 
              0.994754`, 27.83972`, 27.812895`, \(-0.867448`\), 28.33176`, 
              28.347398`, 0.505701`, 46.50428`, 46.506997`, 0.087839`, 
              24.623008`, 24.638627`, 0.505075`, 48.166343`, 48.187926`, 
              0.697949`, 56.353805`, 56.354953`, 0.037107`, 59.813129`, 
              59.821124`, 0.258561`, 16.5242`, 27.83972`, 28.33176`, 
              46.683462`, 59.813129`, 73.682352`, 86.932285`, 98.761968`, 
              24.623008`, 48.166343`, 56.353805`, 60.79832`, 46.50428`, 
              66.112665`, 71.953514`, 77.206383`}, {5, 28, 
              46.59174731`, \(-1.757755417`\), 4.043124662`, 0.0625`, 
              1, \(-0.1`\), \(-0.1652685642`\), 
              4.421660942`, \(-0.8268791285`\), \(-1.315809494`\), 3, 4, 5, 
              6, 0.402679`, \(-0.429753`\), 0.087978`, 0.187229`, 
              0.116132`, \(-0.119896`\), 0.81291`, 0.549764`, 0.853266`, 
              0.753365`, 1.157043`, 1.146995`, 16.5242`, 16.549029`, 
              0.64679`, 22.342577`, 22.355018`, 0.324068`, 26.304099`, 
              26.284513`, \(-0.510192`\), 34.867063`, 34.885224`, 0.473086`, 
              18.610743`, 18.620103`, 0.243809`, 39.811795`, 39.826866`, 
              0.392573`, 42.524541`, 42.527686`, 0.081942`, 48.321741`, 
              48.325911`, 0.108623`, 16.5242`, 22.342577`, 26.304099`, 
              34.867063`, 48.321741`, 58.696883`, 73.536381`, 75.445277`, 
              18.610743`, 39.811795`, 42.524541`, 50.059169`, 39.66807`, 
              57.040564`, 59.703653`, 63.235787`}, {5, 28, 
              67.62208013`, \(-2.296645096`\), 3.665722784`, 0.0625`, 1, 
              0, \(-0.1935570147`\), 
              4.814318899`, \(-0.8125925921`\), \(-1.317520042`\), 3, 4, 5, 
              6, 0.410125`, \(-0.436061`\), 0.08231`, 0.190984`, 
              0.108579`, \(-0.113488`\), 0.816436`, 0.508583`, 0.857772`, 
              0.730455`, 1.1635`, 1.142968`, 16.5242`, 16.545512`, 0.51266`, 
              22.462999`, 22.477877`, 0.357895`, 27.271471`, 
              27.252518`, \(-0.455901`\), 32.956899`, 32.974542`, 0.42438`, 
              16.825129`, 16.833389`, 0.198682`, 37.393189`, 37.406667`, 
              0.324215`, 38.970305`, 38.973975`, 0.088301`, 45.508692`, 
              45.51184`, 0.075722`, 16.5242`, 22.462999`, 27.271471`, 
              32.956899`, 45.508692`, 54.920056`, 68.572356`, 71.413213`, 
              16.825129`, 37.393189`, 38.970305`, 47.872411`, 38.500667`, 
              54.507825`, 56.809087`, 61.524043`}, {5, 28, 
              81.65941598`, \(-2.576108331`\), 3.476058156`, 0.0625`, 1, 
              0.1`, \(-0.2259016312`\), 
              5.443923963`, \(-0.829185865`\), \(-1.306793215`\), 3, 4, 5, 6, 
              0.403832`, \(-0.450475`\), 0.105696`, 0.192272`, 
              0.103856`, \(-0.108097`\), 0.8119`, 0.479642`, 0.854245`, 
              0.712892`, 1.160335`, 1.135083`, 16.5242`, 16.54358`, 
              0.435278`, 22.39985`, 22.416057`, 0.364019`, 27.920407`, 
              27.902723`, \(-0.397188`\), 32.454998`, 32.472454`, 0.392059`, 
              15.670143`, 15.678135`, 0.179502`, 35.515594`, 35.527929`, 
              0.277056`, 37.039946`, 37.044608`, 0.104705`, 43.640747`, 
              43.643054`, 0.051804`, 16.5242`, 22.39985`, 27.920407`, 
              32.454998`, 43.640747`, 52.463397`, 65.566408`, 68.130193`, 
              15.670143`, 35.515594`, 37.039946`, 46.474428`, 37.670877`, 
              52.916316`, 54.668484`, 60.386411`}, {\(-3\), 6, 
              156.0900663`, \(-2.251545636`\), 2.765206353`, 0.0625`, 1, 
              0.2`, \(-0.135494`\), 50, \(-0.97786`\), \(-1.25431`\), 3, 4, 
              5, 6, 0.427556`, \(-0.444813`\), 0.134998`, 0.187539`, 
              0.10849`, \(-0.111772`\), 0.80934`, 0.385763`, 0.860549`, 
              0.650882`, 1.173375`, 1.08459`, 16.5242`, 16.531616`, 
              0.140278`, 20.020973`, 20.026885`, 0.111834`, 23.165619`, 
              23.158846`, \(-0.128122`\), 31.282989`, 31.283425`, 0.008241`, 
              12.396916`, 12.400615`, 0.069976`, 28.556758`, 28.561999`, 
              0.099145`, 30.411509`, 30.412978`, 0.027774`, 35.546945`, 
              35.548857`, 0.036179`, 16.5242`, 20.020973`, 23.165619`, 
              32.831513`, 35.546945`, 42.435444`, 53.04866`, 55.04044`, 
              12.396916`, 28.556758`, 30.411509`, 37.723319`, 31.282989`, 
              43.393064`, 44.366868`, 49.241849`}, {\(-3\), 6, 
              154.0346672`, \(-1.859306883`\), 2.808583103`, 0.0625`, 1, 
              0.3`, \(-0.135494`\), 50, \(-0.97786`\), \(-1.25431`\), 3, 4, 
              5, 6, 0.410874`, \(-0.450817`\), 0.162939`, 0.189991`, 
              0.100523`, \(-0.102212`\), 0.795977`, 0.385785`, 0.84425`, 
              0.652659`, 1.153042`, 1.090943`, 16.5242`, 16.531352`, 
              0.13206`, 20.49886`, 20.504111`, 0.096967`, 24.052035`, 
              24.046301`, \(-0.105872`\), 31.732095`, 31.732538`, 0.008193`, 
              12.173723`, 12.177248`, 0.065079`, 28.594723`, 28.600138`, 
              0.099974`, 30.403226`, 30.403893`, 0.012311`, 35.629222`, 
              35.63109`, 0.034481`, 16.5242`, 20.49886`, 24.052035`, 
              33.25849`, 35.629222`, 42.426416`, 53.350429`, 55.206096`, 
              12.173723`, 28.594723`, 30.403226`, 38.149616`, 31.732095`, 
              44.242633`, 44.485067`, 49.711859`}, {\(-3\), 6, 
              150.644457`, \(-1.598138088`\), 2.87341885`, 0.0625`, 1, 
              0.4`, \(-0.135494`\), 50, \(-0.97786`\), \(-1.25431`\), 3, 4, 
              5, 6, 0.38833`, \(-0.46488`\), 0.212073`, 0.192172`, 
              0.092624`, \(-0.092275`\), 0.778487`, 0.385195`, 0.822871`, 
              0.653566`, 1.126276`, 1.095868`, 16.5242`, 16.531357`, 
              0.12778`, 20.776034`, 20.780736`, 0.083949`, 24.757813`, 
              24.752834`, \(-0.08889`\), 32.041228`, 32.041782`, 0.0099`, 
              11.927779`, 11.931208`, 0.061216`, 28.703596`, 28.709581`, 
              0.106856`, 30.285015`, 30.284618`, \(-0.007093`\), 35.616033`, 
              35.61789`, 0.033143`, 16.5242`, 20.776034`, 24.757813`, 
              33.83582`, 35.616033`, 42.338303`, 53.661098`, 55.380656`, 
              11.927779`, 28.703596`, 30.285015`, 38.488986`, 32.041228`, 
              44.860507`, 44.738049`, 50.100092`}, {5, 28, 
              4.992060696`*^6, \(-0.25`\), \(-0.7396127628`\), 0.09`, 
              1, \(-0.6`\), \(-0.1278174193`\), 
              0.5772807888`, \(-0.9689673954`\), \(-1.345662794`\), 3, 4, 5, 
              6, 0.206408`, \(-8.822704`\), 8.433467`, 
              0.146827`, \(-0.31321`\), 0.746166`, 0.467681`, 1.956714`, 
              0.510458`, 0.352966`, 0.791936`, 0.57148`, 16.5242`, 
              16.524888`, \(-0.00168`\), 9.876699`, 
              9.880944`, \(-0.010369`\), 9.711715`, 8.145213`, 3.826322`, 
              8.146538`, 6.385355`, 4.301849`, 5.518947`, 5.515308`, 
              0.008891`, 4.68653`, 3.094876`, 3.887758`, 3.949522`, 
              3.041653`, 2.217552`, 3.087156`, 2.874975`, 0.518272`, 
              16.5242`, 9.876699`, 8.146538`, 5.504646`, 3.042355`, 
              3.949522`, 4.68653`, 2.355711`, 5.518947`, 3.087156`, 
              2.881856`, 1.536327`, 6.695262`, 9.711715`, 4.811968`, 
              2.245713`}, {5, 28, 
              1.418180028`*^6, \(-0.25`\), \(-1.388179807`\), 0.09`, 
              1, \(-0.5`\), \(-0.1256026267`\), 
              2.1789097`, \(-0.9850943698`\), \(-1.361545163`\), 3, 4, 5, 6, 
              0.251902`, \(-3.942256`\), 4.929658`, 
              0.163561`, \(-0.034077`\), 0.207594`, 0.677444`, 0.377837`, 
              0.702002`, 0.470398`, 0.98142`, 0.832752`, 16.5242`, 
              16.524782`, \(-0.003254`\), 6.814092`, 
              6.814377`, \(-0.001595`\), 5.945242`, 4.08869`, 10.387337`, 
              4.090332`, 1.807728`, 
              12.771083`, \(-0.595227`\), \(-1.141019`\), 
              3.053688`, \(-1.144822`\), \(-2.123992`\), 
              5.47842`, \(-2.135281`\), \(-3.002427`\), 
              4.851647`, \(-2.985135`\), \(-4.430077`\), 8.084395`, 16.5242`, 
              6.814092`, 4.090332`, 
              1.810025`, \(-4.427145`\), \(-5.034704`\), \(-0.595227`\), \
\(-9.886471`\), \(-2.985135`\), \(-1.144822`\), \(-2.135281`\), \
\(-6.998095`\), 1.567548`, 5.945242`, \(-3.362247`\), \(-5.846104`\)}, {5, 
              29, 194665.5494`, \(-0.25`\), \(-3.884316942`\), 0.09`, 
              1, \(-0.4`\), \(-0.1586078124`\), 
              11.87909095`, \(-0.9996342271`\), \(-1.340731794`\), 3, 4, 5, 
              6, 0.298087`, \(-1.695003`\), 2.439037`, 0.171128`, 0.058086`, 
              0.016905`, 0.766421`, 0.428974`, 0.800112`, 0.624258`, 
              1.095014`, 0.990622`, 16.5242`, 16.516763`, 
              0.137771`, \(-1.241074`\), \(-1.234285`\), \(-0.125772`\), \
\(-10.048618`\), \(-12.819982`\), 
              51.34183`, \(-11.921595`\), \(-13.610023`\), 
              31.279544`, \(-15.773203`\), \(-15.795333`\), 
              0.409981`, \(-19.541375`\), \(-19.557885`\), 
              0.30587`, \(-22.639868`\), \(-27.214466`\), 
              84.748267`, \(-27.245685`\), \(-29.756446`\), 46.513954`, 
              16.5242`, \(-1.241074`\), \(-10.048618`\), \(-15.052506`\), \
\(-29.591441`\), \(-39.315959`\), \(-22.639868`\), \(-50.677237`\), \
\(-15.773203`\), \(-19.541375`\), \(-27.245685`\), \(-36.099905`\), \
\(-16.376041`\), \(-11.921595`\), \(-27.770767`\), \(-41.387677`\)}, {5, 27, 
              14.1864651`, \(-0.7460643617`\), 7.675353625`, 0.09`, 
              1, \(-0.3`\), \(-0.1323775874`\), 
              4320.672671`, \(-1.596175295`\), \(-1.626089848`\), 3, 4, 5, 6, 
              0.401274`, \(-0.527642`\), 0.326607`, 0.13265`, 
              0.153137`, \(-0.119483`\), 0.818845`, 0.773514`, 0.813305`, 
              0.856499`, 1.085646`, 1.061135`, 16.5242`, 16.543526`, 
              0.952365`, 35.181489`, 35.165586`, \(-0.783671`\), 47.684334`, 
              47.696863`, 0.61739`, 63.152276`, 63.154587`, 0.113881`, 
              35.955796`, 35.975003`, 0.946482`, 67.569315`, 67.59761`, 
              1.394298`, 79.894919`, 79.883923`, \(-0.541853`\), 82.123465`, 
              82.101842`, \(-1.065563`\), 16.5242`, 35.181489`, 47.684334`, 
              66.362226`, 83.96345`, 103.572807`, 115.790209`, 130.996307`, 
              35.955796`, 67.569315`, 79.894919`, 82.123465`, 63.152276`, 
              86.51423`, 93.485069`, 112.520076`}, {5, 28, 
              19.69999138`, \(-1.050052415`\), 4.439425368`, 0.09`, 
              1, \(-0.2`\), \(-0.2462739607`\), 
              1997.448659`, \(-0.1131109171`\), \(-1.003239689`\), 3, 4, 5, 
              6, 0.386524`, \(-0.458947`\), 0.096309`, 0.225116`, 
              0.03944`, \(-0.043422`\), 0.833455`, 0.645037`, 0.933731`, 
              0.866102`, 1.286142`, 1.317521`, 16.5242`, 16.564396`, 
              1.103595`, 27.140719`, 27.107012`, \(-0.925419`\), 28.459195`, 
              28.468258`, 0.248821`, 41.3137`, 41.317492`, 0.104113`, 
              20.999887`, 21.015432`, 0.426787`, 41.696399`, 41.719733`, 
              0.640614`, 47.220247`, 47.226161`, 0.162381`, 52.153045`, 
              52.151356`, \(-0.046372`\), 16.5242`, 27.140719`, 28.459195`, 
              46.825334`, 52.153045`, 63.601853`, 79.213982`, 80.014104`, 
              20.999887`, 41.696399`, 47.220247`, 53.887647`, 41.3137`, 
              60.553494`, 62.246527`, 67.313708`}, {5, 28, 
              62.69109734`, \(-1.808545599`\), 3.59602561`, 0.09`, 
              1, \(-0.1`\), \(-0.2002609118`\), 
              5.169380288`, \(-0.7894952022`\), \(-1.404466979`\), 3, 4, 5, 
              6, 0.451095`, \(-0.415817`\), 0.007218`, 0.198755`, 
              0.092252`, \(-0.096914`\), 0.851629`, 0.532162`, 0.891462`, 
              0.755264`, 1.204034`, 1.178439`, 16.5242`, 16.546251`, 
              0.572328`, 22.582693`, 22.596561`, 0.359921`, 26.967993`, 
              26.948194`, \(-0.513882`\), 33.255598`, 33.272315`, 0.433881`, 
              17.651841`, 17.659785`, 0.206167`, 38.441893`, 38.45379`, 
              0.308803`, 39.71619`, 39.72116`, 0.129005`, 46.748143`, 
              46.749813`, 0.043342`, 16.5242`, 22.582693`, 26.967993`, 
              33.255598`, 46.748143`, 56.118813`, 69.693962`, 72.37026`, 
              17.651841`, 38.441893`, 39.71619`, 48.346448`, 38.977403`, 
              54.680565`, 57.799958`, 61.078619`}, {5, 29, 
              144.9048803`, \(-2.615498034`\), 2.764858299`, 0.09`, 1, 
              0, \(-0.1790264216`\), 
              16.20122941`, \(-0.9402252625`\), \(-1.34669405`\), 3, 4, 5, 6, 
              0.478098`, \(-0.427168`\), 0.035309`, 0.192734`, 
              0.101738`, \(-0.106743`\), 0.85764`, 0.414361`, 0.908796`, 
              0.67439`, 1.231121`, 1.113503`, 16.5242`, 16.533999`, 
              0.207239`, 19.983683`, 19.9932`, 0.201278`, 23.108895`, 
              23.096715`, \(-0.257614`\), 32.153586`, 32.160479`, 0.145783`, 
              13.667776`, 13.672518`, 0.100297`, 30.07566`, 30.080436`, 
              0.101008`, 32.099733`, 32.104874`, 0.10873`, 37.637588`, 
              37.639311`, 0.036441`, 16.5242`, 19.983683`, 23.108895`, 
              32.153586`, 37.637588`, 44.833146`, 55.454493`, 57.64005`, 
              13.667776`, 30.07566`, 32.099733`, 39.175349`, 32.247811`, 
              44.426905`, 46.396502`, 50.783984`}, {5, 28, 
              92.65059249`, \(-2.493006392`\), 3.219578764`, 0.09`, 1, 
              0.1`, \(-0.2692104385`\), 
              6.263814287`, \(-0.804583226`\), \(-1.390928815`\), 3, 4, 5, 6, 
              0.448949`, \(-0.440572`\), 0.032244`, 0.201399`, 
              0.0851`, \(-0.089711`\), 0.849767`, 0.477979`, 0.891828`, 
              0.722966`, 1.206453`, 1.165348`, 16.5242`, 16.542019`, 0.4121`, 
              22.955349`, 22.973212`, 0.413113`, 28.672041`, 
              28.655`, \(-0.394113`\), 31.957569`, 31.972408`, 0.343173`, 
              15.504413`, 15.511765`, 0.170015`, 34.584309`, 34.591862`, 
              0.17468`, 36.658667`, 36.667521`, 0.20476`, 43.543433`, 
              43.543214`, \(-0.005086`\), 16.5242`, 22.955349`, 28.672041`, 
              31.957569`, 43.543433`, 51.787098`, 64.250999`, 66.455364`, 
              15.504413`, 34.584309`, 36.658667`, 46.222801`, 37.82671`, 
              52.153453`, 54.017298`, 59.310657`}, {\(-3\), 6, 
              164.0140461`, \(-2.390877056`\), 2.622325889`, 0.09`, 1, 
              0.2`, \(-0.16151`\), 50, \(-0.97786`\), \(-1.33022`\), 3, 4, 5, 
              6, 0.48052`, \(-0.433205`\), 0.06603`, 0.195622`, 
              0.092868`, \(-0.097243`\), 0.849783`, 0.394971`, 0.902236`, 
              0.666077`, 1.225007`, 1.115334`, 16.5242`, 16.530325`, 
              0.123495`, 21.125811`, 21.132906`, 0.143051`, 24.371078`, 
              24.36367`, \(-0.149357`\), 32.265902`, 32.270508`, 0.092858`, 
              12.635638`, 12.639082`, 0.069433`, 28.298072`, 28.30149`, 
              0.068911`, 31.26515`, 31.268299`, 0.063476`, 36.494132`, 
              36.495719`, 0.031986`, 16.5242`, 21.125811`, 24.371078`, 
              32.265902`, 36.494132`, 43.080241`, 53.421944`, 55.256847`, 
              12.635638`, 28.298072`, 31.26515`, 38.550581`, 32.270133`, 
              43.902246`, 45.063915`, 49.650484`}, {\(-3\), 6, 
              162.0124827`, \(-1.984637078`\), 2.656214054`, 0.09`, 1, 
              0.3`, \(-0.16151`\), 50, \(-0.97786`\), \(-1.33022`\), 3, 4, 5, 
              6, 0.465964`, \(-0.431777`\), 0.075444`, 0.197493`, 
              0.086912`, \(-0.090123`\), 0.838764`, 0.395296`, 0.888699`, 
              0.667647`, 1.208153`, 1.120354`, 16.5242`, 16.530137`, 
              0.117582`, 21.656638`, 21.662927`, 0.124541`, 25.230242`, 
              25.223904`, \(-0.125513`\), 32.610299`, 32.615075`, 0.094589`, 
              12.457234`, 12.460523`, 0.065137`, 28.319092`, 28.322507`, 
              0.067628`, 31.295149`, 31.29772`, 0.05092`, 36.6149`, 
              36.616461`, 0.030922`, 16.5242`, 21.656638`, 25.230242`, 
              32.610299`, 36.6149`, 43.097624`, 53.696104`, 55.394623`, 
              12.457234`, 28.319092`, 31.295149`, 38.987097`, 32.728705`, 
              44.65772`, 45.228114`, 50.087357`}, {\(-3\), 6, 
              159.6336536`, \(-1.714495541`\), 2.706428602`, 0.09`, 1, 
              0.4`, \(-0.16151`\), 50, \(-0.97786`\), \(-1.33022`\), 3, 4, 5, 
              6, 0.445629`, \(-0.433996`\), 0.09611`, 0.199179`, 
              0.080941`, \(-0.082657`\), 0.824143`, 0.395046`, 0.870687`, 
              0.668511`, 1.185595`, 1.12431`, 16.5242`, 16.530127`, 0.11437`, 
              21.993171`, 21.998811`, 0.108822`, 25.913524`, 
              25.907991`, \(-0.106773`\), 33.047611`, 33.047934`, 0.006248`, 
              12.258068`, 12.261264`, 0.061683`, 28.403028`, 28.406581`, 
              0.068567`, 31.193762`, 31.195735`, 0.038085`, 36.631358`, 
              36.63292`, 0.030141`, 16.5242`, 21.993171`, 25.913524`, 
              33.078266`, 36.631358`, 43.01993`, 53.938831`, 55.50252`, 
              12.258068`, 28.403028`, 31.193762`, 39.319392`, 33.047611`, 
              45.343706`, 45.314228`, 50.421777`}, {2, 7, 
              2.90674716`*^6, \(-0.25`\), \(-1.005805416`\), 0.1225`, 
              1, \(-0.6`\), \(-0.194811`\), 50, \(-0.97786`\), \(-1.39291`\), 
              3, 4, 5, 6, 0.222374`, \(-5.896113`\), 6.445287`, 
              0.169984`, \(-0.096824`\), 0.314267`, 0.655294`, 0.646386`, 
              0.687247`, 0.459221`, 0.977838`, 0.819486`, 16.5242`, 
              16.523445`, 0.0027`, 9.081859`, 9.081295`, 0.002018`, 
              8.094926`, 8.095934`, \(-0.003605`\), 6.283391`, 3.68292`, 
              9.30615`, 1.954346`, 0.872952`, 3.86992`, 0.86401`, 0.290578`, 
              2.052107`, 0.286071`, 0.150852`, 0.483899`, 
              0.168741`, \(-0.98857`\), 4.141598`, 16.5242`, 8.094926`, 
              9.081859`, \(-1.332932`\), \(-0.987313`\), \(-2.127002`\), 
              1.954346`, \(-4.448607`\), 0.168741`, 0.286071`, 
              0.86401`, \(-3.076253`\), 3.686431`, 6.283391`, 
              1.381886`, \(-1.125215`\)}, {5, 28, 
              474796.7131`, \(-0.25`\), \(-2.409606684`\), 0.1225`, 
              1, \(-0.5`\), \(-0.1263318509`\), 
              6.700236035`, \(-1.105203723`\), \(-1.566887957`\), 3, 4, 5, 6, 
              0.338182`, \(-2.00189`\), 2.841746`, 0.175038`, 0.013114`, 
              0.084687`, 0.777645`, 0.318826`, 0.785323`, 0.544311`, 
              1.066981`, 0.938335`, 16.5242`, 16.523428`, 0.010067`, 
              1.499605`, 
              1.499913`, \(-0.004012`\), \(-5.475202`\), \(-5.478936`\), 
              0.048684`, \(-6.000695`\), \(-5.998444`\), \(-0.029355`\), \
\(-9.797195`\), \(-9.808512`\), 0.147549`, \(-10.575225`\), \(-10.583367`\), 
              0.106164`, \(-13.225017`\), \(-13.992197`\), 
              10.002619`, \(-14.009696`\), \(-19.045281`\), 65.654777`, 
              16.5242`, 
              1.499605`, \(-5.475202`\), \(-6.000695`\), \(-17.16616`\), \
\(-23.007689`\), \(-13.225017`\), \(-28.207909`\), \(-9.797195`\), \
\(-10.575225`\), \(-14.009696`\), \(-21.624908`\), \(-7.996865`\), \
\(-6.018471`\), \(-11.666121`\), \(-21.469639`\)}, {5, 27, 
              10162.30359`, \(-0.2642637365`\), \(-20.12983885`\), 0.1225`, 
              1, \(-0.4`\), \(-0.1821298804`\), 
              4942.60866`, \(-3.34974778`\), \(-5.963491976`\), 3, 4, 5, 6, 
              0.377727`, \(-0.983367`\), 1.172378`, \(-0.021409`\), 
              0.156449`, \(-0.086338`\), 0.830987`, \(-0.115399`\), 
              0.06121`, \(-0.120614`\), \(-0.288254`\), \(-0.135704`\), 
              16.5242`, 16.461767`, 
              7.595437`, \(-59.945351`\), \(-59.889934`\), \(-6.741942`\), \
\(-118.091312`\), \(-118.129173`\), 
              4.60608`, \(-121.628042`\), \(-121.662009`\), 
              4.132256`, \(-94.605158`\), \(-94.678599`\), 
              8.934717`, \(-152.640448`\), \(-152.744738`\), 
              12.687667`, \(-198.626586`\), \(-198.6786`\), 
              6.327853`, \(-202.018653`\), \(-202.060642`\), 5.108303`, 
              16.5242`, \(-59.945351`\), \(-118.091312`\), \(-121.628042`\), \
\(-202.018653`\), \(-255.686463`\), \(-239.153292`\), \(-320.357613`\), \
\(-94.605158`\), \(-152.640448`\), \(-198.626586`\), \(-202.900443`\), \
\(-139.282083`\), \(-180.186135`\), \(-177.010576`\), \(-231.943305`\)}, {5, 
              28, 13.80951796`, \(-0.9285822914`\), 5.048472003`, 0.1225`, 
              1, \(-0.3`\), \(-0.196525602`\), 
              7.990917082`, \(-0.6111163944`\), \(-1.128356924`\), 3, 4, 5, 
              6, 0.44625`, \(-0.451804`\), 0.086714`, 0.211417`, 
              0.065175`, \(-0.06693`\), 0.864683`, 0.758329`, 0.954892`, 
              0.936771`, 1.307946`, 1.329145`, 16.5242`, 16.553268`, 
              1.047795`, 29.82323`, 29.799443`, \(-0.857422`\), 29.878886`, 
              29.889555`, 0.384585`, 48.18883`, 48.212883`, 0.867017`, 
              25.395125`, 25.408379`, 0.477739`, 50.535977`, 50.559624`, 
              0.852382`, 55.217194`, 55.218618`, 0.051319`, 61.342513`, 
              61.316413`, \(-0.940795`\), 16.5242`, 29.82323`, 29.878886`, 
              48.18883`, 61.861029`, 75.056237`, 92.481383`, 93.390189`, 
              25.395125`, 50.535977`, 55.217194`, 61.342513`, 48.368493`, 
              69.30457`, 70.970559`, 75.022896`}, {5, 28, 
              38.80945445`, \(-1.459149929`\), 3.913162475`, 0.1225`, 
              1, \(-0.2`\), \(-0.2035079095`\), 
              4.476276265`, \(-0.6650493085`\), \(-1.496086127`\), 3, 4, 5, 
              6, 0.489351`, \(-0.395578`\), \(-0.061403`\), 0.210658`, 
              0.068247`, \(-0.072372`\), 0.883195`, 0.618462`, 0.920078`, 
              0.829675`, 1.238146`, 1.25636`, 16.5242`, 16.549366`, 
              0.771049`, 24.466673`, 24.48112`, 0.442642`, 29.190201`, 
              29.169642`, \(-0.6299`\), 35.445493`, 35.462625`, 0.52491`, 
              20.315694`, 20.324013`, 0.254891`, 43.392435`, 43.402828`, 
              0.318431`, 44.56835`, 44.576158`, 0.239217`, 52.690225`, 
              52.687964`, \(-0.069256`\), 16.5242`, 24.466673`, 29.190201`, 
              35.445493`, 52.690225`, 62.997767`, 77.939831`, 80.336904`, 
              20.315694`, 43.392435`, 44.56835`, 53.416578`, 43.286593`, 
              60.138652`, 64.653406`, 66.62189`}, {5, 28, 
              77.90882828`, \(-1.812763819`\), 3.261878242`, 0.1225`, 
              1, \(-0.1`\), \(-0.2468410317`\), 
              5.999484725`, \(-0.7426143292`\), \(-1.485413318`\), 3, 4, 5, 
              6, 0.499304`, \(-0.400469`\), \(-0.082733`\), 0.208291`, 
              0.072624`, \(-0.077449`\), 0.890346`, 0.51911`, 0.930959`, 
              0.758114`, 1.253244`, 1.20534`, 16.5242`, 16.544539`, 
              0.529998`, 22.966324`, 22.982275`, 0.415678`, 27.873326`, 
              27.852648`, \(-0.538851`\), 32.207882`, 32.223776`, 0.414197`, 
              17.031797`, 17.038925`, 0.185737`, 36.609693`, 36.615845`, 
              0.160321`, 38.736981`, 38.747183`, 0.265854`, 45.880434`, 
              45.87843`, \(-0.052242`\), 16.5242`, 22.966324`, 27.873326`, 
              32.207882`, 45.880434`, 54.482875`, 67.144516`, 69.353494`, 
              17.031797`, 36.609693`, 38.736981`, 47.453762`, 38.743285`, 
              53.217466`, 56.246969`, 60.98716`}, {5, 28, 
              97.36173401`, \(-2.145113726`\), 3.051847478`, 0.1225`, 1, 
              0, \(-0.2812899319`\), 
              6.843057231`, \(-0.7713122767`\), \(-1.472225816`\), 3, 4, 5, 
              6, 0.502975`, \(-0.414237`\), \(-0.070314`\), 0.208195`, 
              0.072245`, \(-0.077268`\), 0.891851`, 0.487161`, 0.935126`, 
              0.737328`, 1.259882`, 1.192385`, 16.5242`, 16.541439`, 
              0.423382`, 23.087412`, 23.105274`, 0.4387`, 28.472705`, 
              28.453814`, \(-0.463959`\), 31.492966`, 31.507002`, 0.344708`, 
              15.878108`, 15.884826`, 0.164995`, 34.296982`, 34.30246`, 
              0.134558`, 37.172564`, 37.182783`, 0.250996`, 44.009404`, 
              44.006987`, \(-0.059346`\), 16.5242`, 23.087412`, 28.472705`, 
              31.492966`, 44.009404`, 52.003012`, 64.005676`, 66.023753`, 
              15.878108`, 34.296982`, 37.172564`, 46.10921`, 37.906896`, 
              51.622724`, 53.973611`, 58.763951`}, {5, 28, 
              103.632432`, \(-2.406832356`\), 2.997207015`, 0.1225`, 1, 
              0.1`, \(-0.3165574012`\), 
              7.126560467`, \(-0.7773218413`\), \(-1.464709369`\), 3, 4, 5, 
              6, 0.497397`, \(-0.427308`\), \(-0.051913`\), 0.209045`, 
              0.069631`, \(-0.07435`\), 0.88834`, 0.476342`, 0.932046`, 
              0.731309`, 1.256797`, 1.19073`, 16.5242`, 16.540523`, 0.38936`, 
              23.582445`, 23.602042`, 0.467455`, 29.422951`, 
              29.409337`, \(-0.324748`\), 31.612237`, 31.621651`, 0.224548`, 
              15.399412`, 15.406166`, 0.161106`, 33.518707`, 33.524184`, 
              0.130638`, 36.771715`, 36.782108`, 0.247904`, 43.622261`, 
              43.619153`, \(-0.074141`\), 16.5242`, 23.582445`, 29.422951`, 
              31.612237`, 43.622261`, 51.357355`, 63.274823`, 65.114813`, 
              15.399412`, 33.518707`, 36.771715`, 46.180784`, 38.131331`, 
              51.655392`, 53.613232`, 58.5263`}, {5, 28, 
              104.0759802`, \(-2.36173921`\), 3.013134777`, 0.1225`, 1, 
              0.2`, \(-0.3510469823`\), 
              7.037292525`, \(-0.7728043581`\), \(-1.459864017`\), 3, 4, 5, 
              6, 0.484898`, \(-0.438535`\), \(-0.031123`\), 0.210253`, 
              0.066019`, \(-0.070093`\), 0.88078`, 0.473763`, 0.923424`, 
              0.730768`, 1.246403`, 1.19317`, 16.5242`, 16.540774`, 
              0.387449`, 24.136863`, 24.158139`, 0.497377`, 30.108423`, 
              30.106179`, \(-0.052467`\), 32.334214`, 
              32.333462`, \(-0.017571`\), 15.174922`, 15.181903`, 0.163201`, 
              33.385799`, 33.391487`, 0.13297`, 36.7515`, 36.762175`, 
              0.249562`, 43.763278`, 43.7594`, \(-0.090655`\), 16.5242`, 
              24.136863`, 30.108423`, 32.334214`, 43.763278`, 51.374349`, 
              63.474893`, 65.141393`, 15.174922`, 33.385799`, 36.7515`, 
              46.747977`, 38.702263`, 52.295177`, 53.95861`, 
              59.012593`}, {\(-3\), 6, 170.9032424`, \(-2.112781251`\), 
              2.518570726`, 0.1225`, 1, 0.3`, \(-0.194811`\), 
              50, \(-0.97786`\), \(-1.39291`\), 3, 4, 5, 6, 
              0.523257`, \(-0.415437`\), \(-0.009126`\), 0.203917`, 
              0.075209`, \(-0.079418`\), 0.881297`, 0.403122`, 0.935077`, 
              0.680331`, 1.266721`, 1.145443`, 16.5242`, 16.529237`, 
              0.106198`, 22.777838`, 22.785358`, 0.158568`, 26.441789`, 
              26.434693`, \(-0.149643`\), 32.015305`, 32.018885`, 0.075473`, 
              12.724702`, 12.727833`, 0.066017`, 28.000843`, 28.003493`, 
              0.055887`, 32.268789`, 32.272211`, 0.07217`, 37.605594`, 
              37.606587`, 0.020942`, 16.5242`, 22.777838`, 26.441789`, 
              32.015305`, 37.605594`, 43.806497`, 54.115541`, 55.649009`, 
              12.724702`, 28.000843`, 32.268789`, 39.867846`, 33.739572`, 
              45.157436`, 46.002717`, 50.535998`}, {\(-3\), 6, 
              168.1179645`, \(-1.832026386`\), 2.557766472`, 0.1225`, 1, 
              0.4`, \(-0.194811`\), 50, \(-0.97786`\), \(-1.39291`\), 3, 4, 
              5, 6, 0.505173`, \(-0.409086`\), \(-0.009587`\), 0.20522`, 
              0.070725`, \(-0.073858`\), 0.869117`, 0.403178`, 0.919992`, 
              0.681189`, 1.247857`, 1.148664`, 16.5242`, 16.529203`, 
              0.103434`, 23.168246`, 23.175031`, 0.140277`, 27.101706`, 
              27.095515`, \(-0.12798`\), 32.40304`, 32.406531`, 0.072167`, 
              12.564799`, 12.567841`, 0.06289`, 28.054103`, 28.056755`, 
              0.054826`, 32.21145`, 32.214437`, 0.061751`, 37.656862`, 
              37.657911`, 0.021685`, 16.5242`, 23.168246`, 27.101706`, 
              32.40304`, 37.656862`, 43.752993`, 54.319857`, 55.726467`, 
              12.564799`, 28.054103`, 32.21145`, 40.20592`, 34.074754`, 
              45.83836`, 46.044707`, 50.844553`}, {5, 25, 
              719757.41`, \(-0.25`\), \(-1.951055963`\), 0.16`, 
              1, \(-0.6`\), \(-0.2037899205`\), \(-0.682205834`\), \
\(-1.055157537`\), \(-1.582531182`\), 3, 4, 5, 6, 0.306607`, \(-3.218044`\), 
              4.192673`, 0.185191`, \(-0.02388`\), 0.136829`, 0.778634`, 
              0.302843`, 0.794473`, 0.530718`, 1.08813`, 0.950191`, 16.5242`, 
              16.525348`, \(-0.010988`\), 16.361442`, 16.324687`, 
              0.351796`, \(-1.890703`\), \(-1.870558`\), \(-0.192812`\), \
\(-2.371906`\), \(-2.842598`\), 4.505139`, \(-5.189517`\), \(-5.193836`\), 
              0.041343`, \(-6.947122`\), \(-6.963965`\), 
              0.161203`, \(-7.981222`\), \(-8.211862`\), 
              2.207527`, \(-8.22783`\), \(-13.172367`\), 47.325673`, 
              16.5242`, \(-1.890703`\), 
              16.361442`, \(-3.158654`\), \(-11.020928`\), \(-16.265202`\), \
\(-7.981222`\), \(-20.707896`\), \(-6.947122`\), \(-5.189517`\), \
\(-8.22783`\), \(-15.667505`\), \(-2.849848`\), \(-2.371906`\), \
\(-5.984136`\), \(-13.298735`\)}, {5, 29, 
              82879.88859`, \(-0.25`\), \(-5.755300155`\), 0.16`, 
              1, \(-0.5`\), \(-0.2175271574`\), 
              2.477321941`, \(-1.121681873`\), \(-1.458605998`\), 3, 4, 5, 6, 
              0.374305`, \(-1.478456`\), 1.945584`, 0.185824`, 0.03276`, 
              0.026924`, 0.845495`, 0.694716`, 0.886818`, 0.841452`, 
              1.206712`, 1.173554`, 16.5242`, 16.506582`, 
              0.607254`, \(-8.643716`\), \(-8.687431`\), 
              1.506773`, \(-9.609755`\), \(-9.591099`\), \(-0.643026`\), \
\(-33.848695`\), \(-33.856199`\), 
              0.258627`, \(-27.055986`\), \(-27.082081`\), 
              0.899436`, \(-38.279187`\), \(-38.312316`\), 
              1.141897`, \(-53.890123`\), \(-53.901884`\), 
              0.405358`, \(-54.325086`\), \(-54.261838`\), \(-2.18004`\), 
              16.5242`, \(-9.609755`\), \(-8.643716`\), \(-36.45779`\), \
\(-53.890123`\), \(-69.854203`\), \(-63.313755`\), \(-87.806113`\), \
\(-27.055986`\), \(-38.279187`\), \(-54.325086`\), \(-54.364854`\), \
\(-33.848695`\), \(-46.554258`\), \(-42.5737`\), \(-60.083078`\)}, {5, 27, 
              27.02931716`, \(-0.7188903391`\), 6.184428676`, 0.16`, 
              1, \(-0.4`\), \(-0.1819022988`\), 
              4193.445052`, \(-1.463309214`\), \(-1.659649315`\), 3, 4, 5, 6, 
              0.484938`, \(-0.476935`\), 0.118355`, 0.16875`, 
              0.104262`, \(-0.084729`\), 0.891193`, 0.811341`, 0.908381`, 
              0.927574`, 1.215896`, 1.205014`, 16.5242`, 16.544147`, 
              0.957173`, 35.019661`, 35.002925`, \(-0.803113`\), 42.172539`, 
              42.180548`, 0.384328`, 59.090917`, 59.092949`, 0.097493`, 
              32.447918`, 32.462297`, 0.68997`, 63.840977`, 63.879472`, 
              1.847182`, 68.245812`, 68.23551`, \(-0.494299`\), 73.92372`, 
              73.898288`, \(-1.220339`\), 16.5242`, 35.019661`, 42.172539`, 
              64.216271`, 77.174036`, 93.517632`, 113.841557`, 115.7859`, 
              32.447918`, 63.840977`, 68.245812`, 73.92372`, 59.090917`, 
              83.733104`, 85.533449`, 91.777366`}, {5, 28, 
              37.85271522`, \(-1.129920739`\), 4.010191964`, 0.16`, 
              1, \(-0.3`\), \(-0.2326398705`\), 
              5.603445491`, \(-0.5498740334`\), \(-1.573851168`\), 3, 4, 5, 
              6, 0.512594`, \(-0.384892`\), \(-0.121906`\), 0.21955`, 
              0.049788`, \(-0.052873`\), 0.91125`, 0.661726`, 0.946638`, 
              0.871728`, 1.270802`, 1.306476`, 16.5242`, 16.552822`, 
              0.941363`, 24.924812`, 24.936719`, 0.39159`, 29.407227`, 
              29.385267`, \(-0.722269`\), 37.973044`, 37.992601`, 0.643219`, 
              21.737609`, 21.746864`, 0.304377`, 45.535666`, 45.546479`, 
              0.355632`, 46.201367`, 46.211736`, 0.341019`, 54.863789`, 
              54.799021`, \(-2.130181`\), 16.5242`, 24.924812`, 29.407227`, 
              37.973044`, 55.023284`, 65.684087`, 81.055982`, 82.496998`, 
              21.737609`, 45.535666`, 46.201367`, 54.863789`, 44.416422`, 
              61.426448`, 67.145804`, 67.692788`}, {5, 28, 
              90.44194043`, \(-1.431140213`\), 2.858889855`, 0.16`, 
              1, \(-0.2`\), \(-0.2887588428`\), 
              15.29491681`, \(-0.5547138518`\), \(-1.614692265`\), 3, 4, 5, 
              6, 0.5254`, \(-0.372393`\), \(-0.192875`\), 0.220052`, 
              0.049635`, \(-0.053688`\), 0.921875`, 0.47973`, 0.95385`, 
              0.747651`, 1.277546`, 1.230727`, 16.5242`, 16.546741`, 
              0.541735`, 20.585487`, 20.593758`, 0.198776`, 24.64426`, 
              24.622922`, \(-0.512816`\), 34.232542`, 34.242282`, 0.234085`, 
              15.699118`, 15.706004`, 0.165512`, 33.024419`, 33.029657`, 
              0.125886`, 34.753121`, 34.764549`, 0.274644`, 41.426381`, 
              41.416834`, \(-0.22944`\), 16.5242`, 20.585487`, 24.64426`, 
              34.232542`, 41.426381`, 49.04996`, 60.242478`, 61.702824`, 
              15.699118`, 33.024419`, 34.753121`, 42.395597`, 34.5064`, 
              47.157095`, 50.51361`, 54.287941`}, {5, 28, 
              98.27899324`, \(-1.810808102`\), 2.93365759`, 0.16`, 
              1, \(-0.1`\), \(-0.2988845555`\), 
              7.418299781`, \(-0.7269754941`\), \(-1.544823427`\), 3, 4, 5, 
              6, 0.550626`, \(-0.383674`\), \(-0.179645`\), 0.214572`, 
              0.059622`, \(-0.064357`\), 0.929662`, 0.49716`, 0.97452`, 
              0.751504`, 1.308964`, 1.217955`, 16.5242`, 16.541897`, 
              0.457396`, 23.028709`, 23.046393`, 0.457054`, 28.300611`, 
              28.279864`, \(-0.536221`\), 31.350127`, 31.364024`, 0.359152`, 
              16.280632`, 16.286965`, 0.163693`, 34.216268`, 34.220881`, 
              0.119222`, 37.718303`, 37.729301`, 0.284256`, 44.541651`, 
              44.535536`, \(-0.158052`\), 16.5242`, 23.028709`, 28.300611`, 
              31.350127`, 44.541651`, 52.359814`, 64.068628`, 65.804239`, 
              16.280632`, 34.216268`, 37.718303`, 46.085002`, 38.034275`, 
              51.249195`, 54.149471`, 58.472467`}, {5, 28, 
              116.5604574`, \(-2.105563687`\), 2.789484256`, 0.16`, 1, 
              0, \(-0.3297227282`\), 
              8.388512148`, \(-0.7611451624`\), \(-1.527938767`\), 3, 4, 5, 
              6, 0.555545`, \(-0.399405`\), \(-0.159456`\), 0.214109`, 
              0.06011`, \(-0.065047`\), 0.930945`, 0.473986`, 0.978847`, 
              0.736299`, 1.315892`, 1.207509`, 16.5242`, 16.538815`, 
              0.362376`, 23.362649`, 23.381957`, 0.478764`, 28.907861`, 
              28.890579`, \(-0.428489`\), 31.022562`, 31.032633`, 0.249713`, 
              15.441297`, 15.447287`, 0.148539`, 32.550761`, 32.55517`, 
              0.109304`, 36.720133`, 36.730572`, 0.258848`, 43.305657`, 
              43.300088`, \(-0.138094`\), 16.5242`, 23.362649`, 28.907861`, 
              31.022562`, 43.305657`, 50.676207`, 61.95214`, 63.56914`, 
              15.441297`, 32.550761`, 36.720133`, 45.297081`, 37.605683`, 
              50.311636`, 52.655165`, 57.047977`}, {5, 28, 
              120.0319515`, \(-2.352319649`\), 2.777367805`, 0.16`, 1, 
              0.1`, \(-0.3637420716`\), 
              8.328735046`, \(-0.7559998273`\), \(-1.524646296`\), 3, 4, 5, 
              6, 0.551071`, \(-0.411439`\), \(-0.142927`\), 0.215107`, 
              0.057648`, \(-0.0624`\), 0.927994`, 0.47088`, 0.97574`, 
              0.735445`, 1.312414`, 1.209495`, 16.5242`, 16.538424`, 
              0.348315`, 24.138246`, 24.159172`, 0.512451`, 29.823817`, 
              29.81887`, \(-0.12115`\), 31.447334`, 
              31.445825`, \(-0.036949`\), 15.218901`, 15.224971`, 0.148633`, 
              32.26775`, 32.272244`, 0.11004`, 36.830971`, 36.841507`, 
              0.25803`, 43.50488`, 43.498965`, \(-0.144848`\), 16.5242`, 
              24.138246`, 29.823817`, 31.447334`, 43.50488`, 50.730457`, 
              62.072925`, 63.563449`, 15.218901`, 32.26775`, 36.830971`, 
              45.92938`, 38.267417`, 50.969484`, 52.983444`, 57.52969`}, {5, 
              28, 120.827837`, \(-2.366453712`\), 2.782160467`, 0.16`, 1, 
              0.2`, \(-0.4036345055`\), 
              8.401536601`, \(-0.725348451`\), \(-1.528293645`\), 3, 4, 5, 6, 
              0.537779`, \(-0.419797`\), \(-0.132572`\), 0.216894`, 
              0.053558`, \(-0.057838`\), 0.921181`, 0.467728`, 0.966669`, 
              0.73552`, 1.300825`, 1.214127`, 16.5242`, 16.538611`, 
              0.344997`, 24.532932`, 24.555518`, 0.540707`, 30.109169`, 
              30.114065`, 0.117191`, 32.379688`, 32.369008`, \(-0.255651`\), 
              14.982125`, 14.988455`, 0.15152`, 32.044716`, 32.049403`, 
              0.112205`, 36.659922`, 36.670869`, 0.262055`, 43.487097`, 
              43.480346`, \(-0.161606`\), 16.5242`, 24.532932`, 30.109169`, 
              32.379688`, 43.487097`, 50.569546`, 62.0286`, 63.344808`, 
              14.982125`, 32.044716`, 36.659922`, 46.310254`, 38.642578`, 
              51.363852`, 53.114724`, 57.754514`}, {5, 28, 
              126.8525169`, \(-2.185633257`\), 2.770685738`, 0.16`, 1, 
              0.3`, \(-0.4343611197`\), 
              8.934044387`, \(-0.8472742019`\), \(-1.521114095`\), 3, 4, 5, 
              6, 0.522457`, \(-0.425817`\), \(-0.118805`\), 0.213206`, 
              0.058106`, \(-0.061675`\), 0.912049`, 0.454293`, 0.956329`, 
              0.721801`, 1.288326`, 1.195016`, 16.5242`, 16.537855`, 
              0.316262`, 24.617165`, 24.640532`, 0.541208`, 30.176896`, 
              30.183249`, 0.147158`, 33.071671`, 33.059787`, \(-0.275228`\), 
              14.602843`, 14.609262`, 0.148666`, 31.567064`, 31.571812`, 
              0.109965`, 36.087244`, 36.098194`, 0.25363`, 42.983947`, 
              42.976812`, \(-0.165234`\), 16.5242`, 24.617165`, 30.176896`, 
              33.071671`, 42.983947`, 49.853835`, 61.359948`, 62.503659`, 
              14.602843`, 31.567064`, 36.087244`, 46.105656`, 38.535696`, 
              51.170127`, 52.667228`, 57.326318`}, {\(-3\), 6, 
              178.0685538`, \(-1.952821708`\), 2.423316148`, 0.16`, 1, 
              0.4`, \(-0.235397`\), 50, \(-0.97786`\), \(-1.44239`\), 3, 4, 
              5, 6, 0.567006`, \(-0.389037`\), \(-0.107559`\), 0.210414`, 
              0.061917`, \(-0.06601`\), 0.913625`, 0.409695`, 0.970757`, 
              0.691844`, 1.312661`, 1.169456`, 16.5242`, 16.528512`, 
              0.094791`, 24.291713`, 24.299827`, 0.178381`, 28.271126`, 
              28.264467`, \(-0.146416`\), 31.825084`, 31.827315`, 0.04904`, 
              12.847436`, 12.850368`, 0.064468`, 27.71928`, 27.721549`, 
              0.049874`, 33.236352`, 33.239941`, 0.078901`, 38.668301`, 
              38.668549`, 0.005431`, 16.5242`, 24.291713`, 28.271126`, 
              31.825084`, 38.668301`, 44.507269`, 54.758762`, 56.002371`, 
              12.847436`, 27.71928`, 33.236352`, 41.119023`, 35.099634`, 
              46.351325`, 46.841`, 51.331077`}, {5, 28, 
              188571.9674`, \(-0.25`\), \(-3.807279951`\), 0.2025`, 
              1, \(-0.6`\), \(-0.003480537915`\), 
              25.4331295`, \(-3.829465274`\), \(-5.749636238`\), 3, 4, 5, 6, 
              0.534595`, \(-0.86375`\), 0.886971`, \(-0.012587`\), 0.053941`, 
              0.013584`, 0.878218`, 0.023782`, 0.221161`, 
              0.010477`, \(-0.00889`\), \(-0.015717`\), 16.5242`, 
              16.524342`, \(-0.004622`\), \(-12.554901`\), \(-12.554684`\), \
\(-0.007057`\), \(-17.186015`\), \(-17.18624`\), 
              0.007342`, \(-22.167082`\), \(-22.171111`\), 
              0.131199`, \(-19.463732`\), \(-19.465731`\), 
              0.065103`, \(-30.395881`\), \(-30.400187`\), 
              0.140214`, \(-35.667113`\), \(-35.670432`\), 
              0.108084`, \(-38.570519`\), \(-38.557378`\), \(-0.427936`\), 
              16.5242`, \(-12.554901`\), \(-17.186015`\), \(-22.167082`\), \
\(-40.709378`\), \(-50.369676`\), \(-52.874841`\), \(-57.650436`\), \
\(-19.463732`\), \(-30.395881`\), \(-35.667113`\), \(-38.570519`\), \
\(-26.113198`\), \(-39.229654`\), \(-24.990785`\), \(-47.06637`\)}, {5, 26, 
              248691.8872`, \(-0.3841432958`\), 31.95311035`, 0.2025`, 
              1, \(-0.5`\), \(-0.2523738587`\), 
              6.703731727`, \(-3.11910464`\), \(-3.825924241`\), 3, 4, 5, 6, 
              0.470371`, \(-0.742695`\), 0.55526`, 0.047508`, 
              0.132634`, \(-0.074974`\), 0.911024`, 1.17034`, 0.592619`, 
              1.177115`, 0.647517`, 1.19718`, 16.5242`, 16.696206`, 
              41.363447`, 142.521411`, 142.952508`, 103.668902`, 179.955681`, 
              179.493935`, \(-111.039302`\), 269.741949`, 269.900574`, 
              38.145615`, 170.273321`, 170.375821`, 24.649082`, 306.280285`, 
              306.545181`, 63.701458`, 353.783783`, 
              353.706078`, \(-18.686173`\), 364.503068`, 
              364.314854`, \(-45.261134`\), 16.5242`, 142.521411`, 
              179.955681`, 269.741949`, 384.394216`, 472.150353`, 
              568.717118`, 585.18638`, 170.273321`, 306.280285`, 353.783783`, 
              364.503068`, 281.269881`, 413.657699`, 401.913808`, 
              436.255054`}, {5, 28, 29.8820947`, \(-0.8712074687`\), 
              4.127454828`, 0.2025`, 1, \(-0.4`\), \(-0.2682058789`\), 
              15.86897436`, 0.1997240489`, \(-1.349282467`\), 3, 4, 5, 6, 
              0.532898`, \(-0.397558`\), \(-0.145085`\), 0.233675`, 
              0.022671`, \(-0.026294`\), 0.938821`, 0.718016`, 1.01761`, 
              0.930827`, 1.379219`, 1.382284`, 16.5242`, 16.553073`, 
              1.016114`, 28.726257`, 28.732234`, 0.210338`, 29.54777`, 
              29.523502`, \(-0.854028`\), 45.541191`, 45.543114`, 0.067671`, 
              23.238608`, 23.249039`, 0.367111`, 47.755418`, 47.769241`, 
              0.486454`, 47.833668`, 47.844799`, 0.391708`, 56.371954`, 
              56.33674`, \(-1.239237`\), 16.5242`, 29.54777`, 28.726257`, 
              47.618636`, 57.440473`, 68.55182`, 84.265314`, 84.832062`, 
              23.238608`, 47.833668`, 47.755418`, 56.371954`, 45.541191`, 
              62.799674`, 69.219779`, 69.763446`}, {2, 23, 
              84.15124504`, \(-1.248984838`\), 3.045708726`, 0.2025`, 
              1, \(-0.3`\), \(-0.2891537947`\), 
              9.710714706`, \(-0.7717526105`\), \(-1.574362012`\), 3, 4, 5, 
              6, 0.572384`, \(-0.343931`\), \(-0.283686`\), 0.216013`, 
              0.054267`, \(-0.057192`\), 0.955582`, 0.527198`, 1.005224`, 
              0.771879`, 1.349365`, 1.233122`, 16.5242`, 16.547193`, 
              0.641357`, 22.20902`, 22.218347`, 0.26015`, 26.558549`, 
              26.537285`, \(-0.593104`\), 34.33546`, 34.349401`, 0.388852`, 
              17.648331`, 17.655115`, 0.189213`, 36.287951`, 36.29244`, 
              0.125222`, 38.690075`, 38.702961`, 0.359438`, 45.961704`, 
              45.910285`, \(-1.434241`\), 16.5242`, 22.20902`, 26.558549`, 
              34.33546`, 45.961704`, 54.182594`, 66.238401`, 67.303319`, 
              17.648331`, 36.287951`, 38.690075`, 46.167519`, 37.930148`, 
              51.159424`, 55.643887`, 59.291563`}, {5, 28, 
              89.16551892`, \(-1.529129377`\), 2.898961547`, 0.2025`, 
              1, \(-0.2`\), \(-0.3298214965`\), 
              7.683008717`, \(-0.635719627`\), \(-1.624190684`\), 3, 4, 5, 6, 
              0.590115`, \(-0.348215`\), \(-0.304873`\), 0.221519`, 
              0.046089`, \(-0.05023`\), 0.964507`, 0.516846`, 1.009207`, 
              0.77274`, 1.351519`, 1.2494`, 16.5242`, 16.544822`, 0.564468`, 
              23.107573`, 23.12503`, 0.47781`, 28.674787`, 
              28.651632`, \(-0.633795`\), 31.683568`, 31.698563`, 0.41046`, 
              16.994585`, 17.000982`, 0.175094`, 34.890886`, 34.895216`, 
              0.118531`, 38.803171`, 38.815624`, 0.340852`, 45.846104`, 
              45.831231`, \(-0.407094`\), 16.5242`, 23.107573`, 28.674787`, 
              31.683568`, 45.846104`, 53.678879`, 65.382714`, 66.703383`, 
              16.994585`, 34.890886`, 38.803171`, 46.910358`, 38.795314`, 
              51.803483`, 55.366046`, 59.315758`}, {5, 28, 
              117.3920025`, \(-1.798521866`\), 2.684762073`, 0.2025`, 
              1, \(-0.1`\), \(-0.359226426`\), 
              8.950048907`, \(-0.7071305428`\), \(-1.591125248`\), 3, 4, 5, 
              6, 0.60396`, \(-0.365401`\), \(-0.285047`\), 0.219794`, 
              0.049245`, \(-0.053875`\), 0.969402`, 0.482016`, 1.020355`, 
              0.748411`, 1.367993`, 1.230119`, 16.5242`, 16.539749`, 0.4034`, 
              23.298285`, 23.318273`, 0.518568`, 28.990762`, 
              28.970732`, \(-0.51967`\), 30.823936`, 30.834618`, 0.277135`, 
              15.815347`, 15.821117`, 0.149693`, 32.470753`, 32.474785`, 
              0.104611`, 37.287024`, 37.298247`, 0.291172`, 43.87443`, 
              43.86396`, \(-0.271626`\), 16.5242`, 23.298285`, 28.990762`, 
              30.823936`, 43.87443`, 51.101119`, 62.110162`, 63.425283`, 
              15.815347`, 32.470753`, 37.287024`, 45.432401`, 37.833229`, 
              50.105037`, 52.920302`, 56.916966`}, {5, 28, 
              130.4015483`, \(-2.051824164`\), 2.614150295`, 0.2025`, 1, 
              0, \(-0.389700061`\), 
              9.433019328`, \(-0.7266580211`\), \(-1.578937866`\), 3, 4, 5, 
              6, 0.608823`, \(-0.382171`\), \(-0.260888`\), 0.219755`, 
              0.049138`, \(-0.053932`\), 0.970333`, 0.470942`, 1.02342`, 
              0.741471`, 1.372866`, 1.22584`, 16.5242`, 16.537719`, 0.34426`, 
              24.021847`, 24.043478`, 0.550831`, 29.748793`, 
              29.741491`, \(-0.185955`\), 30.888377`, 
              30.886329`, \(-0.05216`\), 15.36583`, 15.371415`, 0.14221`, 
              31.620143`, 31.624119`, 0.10124`, 37.056422`, 37.067195`, 
              0.274314`, 43.55486`, 43.545551`, \(-0.23705`\), 16.5242`, 
              24.021847`, 29.748793`, 30.888377`, 43.55486`, 50.522722`, 
              61.371227`, 62.623352`, 15.36583`, 31.620143`, 37.056422`, 
              45.531925`, 38.099452`, 50.167471`, 52.545334`, 
              56.667904`}, {5, 28, 137.9645959`, \(-2.238566252`\), 
              2.567884907`, 0.2025`, 1, 0.1`, \(-0.4325634135`\), 
              10.28735396`, \(-0.679789127`\), \(-1.590226335`\), 3, 4, 5, 6, 
              0.601463`, \(-0.392891`\), \(-0.252082`\), 0.221722`, 
              0.045288`, \(-0.049859`\), 0.967114`, 0.461812`, 1.017775`, 
              0.738371`, 1.365186`, 1.229336`, 16.5242`, 16.536778`, 
              0.310822`, 24.262938`, 24.286101`, 0.572381`, 29.909521`, 
              29.912472`, 0.072944`, 31.565146`, 31.552412`, \(-0.314687`\), 
              14.945111`, 14.950809`, 0.140805`, 30.91203`, 30.916106`, 
              0.100745`, 36.521343`, 36.532296`, 0.270675`, 43.047579`, 
              43.037947`, \(-0.238034`\), 16.5242`, 24.262938`, 29.909521`, 
              31.565146`, 43.047579`, 49.780971`, 60.530748`, 61.632041`, 
              14.945111`, 30.91203`, 36.521343`, 45.414378`, 38.069589`, 
              49.968416`, 52.03006`, 56.2081`}, {5, 28, 
              141.9570353`, \(-2.338779908`\), 2.553113026`, 0.2025`, 1, 
              0.2`, \(-0.4646261401`\), 
              10.79728942`, \(-0.6669965836`\), \(-1.589330557`\), 3, 4, 5, 
              6, 0.59255`, \(-0.399405`\), \(-0.242884`\), 0.222559`, 
              0.04324`, \(-0.047547`\), 0.961824`, 0.456446`, 1.011284`, 
              0.736029`, 1.357167`, 1.230063`, 16.5242`, 16.536037`, 
              0.286525`, 24.635381`, 24.659373`, 0.580731`, 29.996975`, 
              30.003433`, 0.15632`, 32.374108`, 32.358019`, \(-0.389442`\), 
              14.664569`, 14.670314`, 0.139055`, 30.524`, 30.52811`, 
              0.099479`, 36.25799`, 36.26891`, 0.264344`, 42.843419`, 
              42.833709`, \(-0.235052`\), 16.5242`, 24.635381`, 29.996975`, 
              32.374108`, 42.843419`, 49.398725`, 60.173719`, 61.146523`, 
              14.664569`, 30.524`, 36.25799`, 45.533998`, 38.248804`, 
              50.076643`, 51.884683`, 56.121986`}, {5, 28, 
              142.4629862`, \(-2.268993607`\), 2.573779442`, 0.2025`, 1, 
              0.3`, \(-0.4770231586`\), 
              10.48011997`, \(-0.6809658681`\), \(-1.579514709`\), 3, 4, 5, 
              6, 0.585351`, \(-0.400917`\), \(-0.231581`\), 0.222648`, 
              0.04238`, \(-0.04643`\), 0.95543`, 0.457229`, 1.004727`, 
              0.736341`, 1.349632`, 1.230201`, 16.5242`, 16.535581`, 
              0.274335`, 25.385633`, 25.409368`, 0.572142`, 30.07141`, 
              30.080433`, 0.217487`, 33.345923`, 33.328653`, \(-0.416293`\), 
              14.568165`, 14.573801`, 0.135853`, 30.533777`, 30.537803`, 
              0.097047`, 36.481114`, 36.491568`, 0.252006`, 43.144383`, 
              43.135332`, \(-0.218174`\), 16.5242`, 25.385633`, 30.07141`, 
              33.345923`, 43.144383`, 49.591468`, 60.543786`, 61.440257`, 
              14.568165`, 30.533777`, 36.481114`, 46.279566`, 38.854035`, 
              50.766003`, 52.318008`, 56.666138`}, {5, 28, 
              135.7202441`, \(-2.068744039`\), 2.642660472`, 0.2025`, 1, 
              0.4`, \(-0.4861103499`\), 
              9.303219988`, \(-0.6781143012`\), \(-1.574018052`\), 3, 4, 5, 
              6, 0.573889`, \(-0.397913`\), \(-0.226653`\), 0.223187`, 
              0.040673`, \(-0.04437`\), 0.946917`, 0.46532`, 0.994861`, 
              0.741638`, 1.337599`, 1.234494`, 16.5242`, 16.536297`, 
              0.293531`, 26.398525`, 26.422095`, 0.571937`, 30.148371`, 
              30.160511`, 0.29457`, 34.701331`, 34.683477`, \(-0.433249`\), 
              14.68043`, 14.686061`, 0.136641`, 31.039422`, 31.043427`, 
              0.097179`, 37.153262`, 37.163421`, 0.246533`, 44.014054`, 
              44.005515`, \(-0.207209`\), 16.5242`, 26.398525`, 30.148371`, 
              34.701331`, 44.014054`, 50.445659`, 61.773158`, 62.595739`, 
              14.68043`, 31.039422`, 37.153262`, 47.349906`, 39.94357`, 
              52.144508`, 53.474005`, 57.971233`}, {2, 13, 
              47213.00977`, \(-0.25`\), \(-7.790232992`\), 0.25`, 
              1, \(-0.6`\), \(-0.336801654`\), 
              29.04950453`, \(-1.012050689`\), \(-1.492470525`\), 3, 4, 5, 6, 
              0.443845`, \(-1.359901`\), 1.522307`, 0.207062`, 0.022555`, 
              0.010058`, 0.928829`, 1.099144`, 0.990492`, 1.217131`, 
              1.343046`, 1.516179`, 16.5242`, 16.468362`, 
              3.089096`, \(-24.734706`\), \(-24.68457`\), \(-2.773655`\), \
\(-50.838186`\), \(-50.849364`\), 0.618402`, \(-57.992987`\), \(-58.0524`\), 
              3.286912`, \(-41.812832`\), \(-41.846874`\), 
              1.883294`, \(-65.749641`\), \(-65.828659`\), 
              4.371467`, \(-82.630082`\), \(-82.56685`\), \(-3.498145`\), \
\(-86.344954`\), \(-86.321366`\), \(-1.304944`\), 
              16.5242`, \(-24.734706`\), \(-50.838186`\), \(-57.992987`\), \
\(-87.780395`\), \(-110.713859`\), \(-133.552993`\), \(-140.095055`\), \
\(-41.812832`\), \(-65.749641`\), \(-82.630082`\), \(-86.344954`\), \
\(-59.999977`\), \(-95.327313`\), \(-87.787718`\), \(-93.892818`\)}, {5, 28, 
              21.94476766`, \(-0.730740187`\), 4.555424181`, 0.25`, 
              1, \(-0.5`\), \(-0.2730221749`\), 
              204.8547443`, \(-0.2663007537`\), \(-1.432606484`\), 3, 4, 5, 
              6, 0.574527`, \(-0.400932`\), \(-0.182529`\), 0.232425`, 
              0.025062`, \(-0.027664`\), 0.969778`, 0.819591`, 1.046125`, 
              1.009383`, 1.412973`, 1.432815`, 16.5242`, 16.5493`, 1.051079`, 
              32.385689`, 32.378674`, \(-0.293768`\), 34.285831`, 
              34.276256`, \(-0.400947`\), 51.180289`, 51.182339`, 0.085847`, 
              26.800379`, 26.811177`, 0.45216`, 54.091759`, 54.112408`, 
              0.864678`, 54.275415`, 54.282962`, 0.315994`, 62.691945`, 
              62.660363`, \(-1.322502`\), 16.5242`, 32.385689`, 34.285831`, 
              56.362403`, 65.33136`, 77.771476`, 94.282101`, 95.475914`, 
              26.800379`, 54.275415`, 54.091759`, 62.691945`, 51.180289`, 
              69.702694`, 77.935543`, 78.969613`}, {5, 28, 
              51.15668912`, \(-1.038611658`\), 3.514952918`, 0.25`, 
              1, \(-0.4`\), \(-0.3247650838`\), 
              6.912623743`, \(-0.01334407639`\), \(-1.867459386`\), 3, 4, 5, 
              6, 0.603098`, \(-0.324643`\), \(-0.378612`\), 0.235145`, 
              0.021067`, \(-0.024485`\), 0.986881`, 0.653969`, 1.004594`, 
              0.884843`, 1.333737`, 1.356293`, 16.5242`, 16.555851`, 
              1.07353`, 25.156179`, 25.168849`, 0.429722`, 30.863572`, 
              30.83768`, \(-0.878209`\), 36.813537`, 36.834003`, 0.694157`, 
              21.282104`, 21.2903`, 0.277978`, 42.932777`, 42.937715`, 
              0.167499`, 45.685029`, 45.702471`, 0.591571`, 54.080215`, 
              54.012551`, \(-2.295025`\), 16.5242`, 25.156179`, 30.863572`, 
              36.813537`, 54.530571`, 64.138514`, 78.095674`, 78.527901`, 
              21.282104`, 42.932777`, 45.685029`, 54.080215`, 44.499448`, 
              59.55231`, 65.710141`, 69.474551`}, {5, 28, 
              79.47449908`, \(-1.292598233`\), 2.89333259`, 0.25`, 
              1, \(-0.3`\), \(-0.3727180107`\), 
              8.238114356`, \(-0.4984435497`\), \(-1.715862141`\), 3, 4, 5, 
              6, 0.623798`, \(-0.312902`\), \(-0.438418`\), 0.228177`, 
              0.033608`, \(-0.037195`\), 0.997877`, 0.538756`, 1.040383`, 
              0.795471`, 1.388727`, 1.281235`, 16.5242`, 16.549446`, 
              0.729058`, 23.181802`, 23.198135`, 0.47165`, 29.222496`, 
              29.196499`, \(-0.750738`\), 32.53722`, 32.553829`, 0.47962`, 
              17.805365`, 17.812072`, 0.193691`, 35.804425`, 35.808711`, 
              0.123794`, 39.943581`, 39.95807`, 0.418427`, 47.34175`, 
              47.303376`, \(-1.108153`\), 16.5242`, 23.181802`, 29.222496`, 
              32.53722`, 47.34175`, 55.26203`, 67.032897`, 67.80548`, 
              17.805365`, 35.804425`, 39.943581`, 47.977246`, 39.698784`, 
              52.589965`, 56.865921`, 60.476079`}, {5, 28, 
              113.1117461`, \(-1.551834464`\), 2.61735281`, 0.25`, 
              1, \(-0.2`\), \(-0.3989341174`\), 
              9.417580908`, \(-0.6090946787`\), \(-1.661733911`\), 3, 4, 5, 
              6, 0.646636`, \(-0.325541`\), \(-0.428538`\), 0.225758`, 
              0.038104`, \(-0.042231`\), 1.005701`, 0.493194`, 1.058039`, 
              0.762958`, 1.414502`, 1.255039`, 16.5242`, 16.542081`, 
              0.484224`, 23.181901`, 23.202233`, 0.550582`, 29.261459`, 
              29.238479`, \(-0.62229`\), 30.916536`, 30.928392`, 0.321038`, 
              16.286219`, 16.291976`, 0.155899`, 32.674345`, 32.678229`, 
              0.105176`, 37.965851`, 37.97828`, 0.336588`, 44.670849`, 
              44.651201`, \(-0.532064`\), 16.5242`, 23.181901`, 29.261459`, 
              30.916536`, 44.670849`, 51.829643`, 62.686427`, 63.605303`, 
              16.286219`, 32.674345`, 37.965851`, 45.847559`, 38.253559`, 
              50.198709`, 53.530832`, 57.175706`}, {5, 28, 
              138.608032`, \(-1.790257501`\), 2.481213838`, 0.25`, 
              1, \(-0.1`\), \(-0.4247551068`\), 
              10.65885578`, \(-0.685261057`\), \(-1.624231635`\), 3, 4, 5, 6, 
              0.660921`, \(-0.346111`\), \(-0.39626`\), 0.224099`, 
              0.041059`, \(-0.045593`\), 1.009778`, 0.470498`, 1.068565`, 
              0.74674`, 1.430173`, 1.240846`, 16.5242`, 16.537752`, 
              0.355585`, 23.706203`, 23.728631`, 0.588479`, 29.689304`, 
              29.677071`, \(-0.320989`\), 30.538247`, 30.538422`, 0.004578`, 
              15.504374`, 15.509659`, 0.138674`, 31.099874`, 31.103561`, 
              0.096731`, 37.17528`, 37.186564`, 0.296073`, 43.570915`, 
              43.556269`, \(-0.384289`\), 16.5242`, 23.706203`, 29.689304`, 
              30.538247`, 43.570915`, 50.322158`, 60.766624`, 61.712639`, 
              15.504374`, 31.099874`, 37.17528`, 45.153857`, 37.907973`, 
              49.410765`, 52.162629`, 55.895048`}, {5, 28, 
              155.4299829`, \(-1.984497317`\), 2.388474729`, 0.25`, 1, 
              0, \(-0.4637315136`\), 
              12.79882341`, \(-0.6469925766`\), \(-1.633809537`\), 3, 4, 5, 
              6, 0.661944`, \(-0.362024`\), \(-0.377613`\), 0.225507`, 
              0.038577`, \(-0.043103`\), 1.00984`, 0.454615`, 1.067363`, 
              0.739015`, 1.428263`, 1.240239`, 16.5242`, 16.535284`, 
              0.280387`, 23.862253`, 23.885996`, 0.600614`, 29.753135`, 
              29.75151`, \(-0.041093`\), 30.86401`, 
              30.852146`, \(-0.300108`\), 14.876726`, 14.881898`, 0.130831`, 
              29.910155`, 29.913818`, 0.092658`, 36.29773`, 36.308674`, 
              0.276845`, 42.56379`, 42.550555`, \(-0.334801`\), 16.5242`, 
              23.862253`, 29.753135`, 30.86401`, 42.56379`, 48.990779`, 
              59.149018`, 60.00862`, 14.876726`, 29.910155`, 36.29773`, 
              44.593058`, 37.477786`, 48.652367`, 50.991846`, 
              54.762855`}, {5, 28, 164.4129775`, \(-2.14744902`\), 
              2.338040528`, 0.25`, 1, 0.1`, \(-0.5075149351`\), 
              15.22261744`, \(-0.6228539511`\), \(-1.63964903`\), 3, 4, 5, 6, 
              0.654893`, \(-0.371699`\), \(-0.37192`\), 0.226429`, 
              0.036708`, \(-0.041091`\), 1.006872`, 0.443409`, 1.062839`, 
              0.733516`, 1.422339`, 1.239513`, 16.5242`, 16.533947`, 
              0.23879`, 23.960835`, 23.985794`, 0.611461`, 29.828987`, 
              29.830478`, 0.036521`, 31.412511`, 31.39631`, \(-0.396904`\), 
              14.438291`, 14.443528`, 0.128285`, 29.149043`, 29.152767`, 
              0.091236`, 35.634534`, 35.645539`, 0.269607`, 41.897412`, 
              41.884295`, \(-0.321332`\), 16.5242`, 23.960835`, 29.828987`, 
              31.412511`, 41.897412`, 48.085365`, 58.107094`, 58.84048`, 
              14.438291`, 29.149043`, 35.634534`, 43.768745`, 37.248723`, 
              48.221844`, 50.279232`, 54.072986`}, {5, 28, 
              158.3088197`, \(-2.273387673`\), 2.388012465`, 0.25`, 1, 
              0.2`, \(-0.539622336`\), 
              12.93115355`, \(-0.5999250768`\), \(-1.644536878`\), 3, 4, 5, 
              6, 0.647222`, \(-0.376982`\), \(-0.366213`\), 0.22727`, 
              0.034914`, \(-0.039116`\), 1.002342`, 0.451175`, 1.056462`, 
              0.739069`, 1.414105`, 1.244956`, 16.5242`, 16.534805`, 
              0.262255`, 24.995492`, 25.02181`, 0.650819`, 29.911025`, 
              29.919454`, 0.208447`, 32.859104`, 32.837763`, \(-0.527765`\), 
              14.586457`, 14.591864`, 0.133697`, 29.593843`, 29.597687`, 
              0.095044`, 36.386816`, 36.398039`, 0.277536`, 42.87163`, 
              42.858388`, \(-0.327457`\), 16.5242`, 24.995492`, 29.911025`, 
              32.859104`, 42.87163`, 49.063378`, 59.373757`, 60.023837`, 
              14.586457`, 29.593843`, 36.386816`, 44.520241`, 38.442288`, 
              49.644963`, 51.535007`, 55.4805`}, {5, 28, 
              142.8180988`, \(-2.360220205`\), 2.50367883`, 0.25`, 1, 
              0.3`, \(-0.5171413414`\), 
              9.338403027`, \(-0.5921394168`\), \(-1.640353648`\), 3, 4, 5, 
              6, 0.654104`, \(-0.377716`\), \(-0.339089`\), 0.227796`, 
              0.033764`, \(-0.037912`\), 0.999406`, 0.476304`, 1.053429`, 
              0.754839`, 1.4107`, 1.255572`, 16.5242`, 16.536489`, 0.322002`, 
              27.373887`, 27.398933`, 0.656263`, 29.805671`, 29.81898`, 
              0.348738`, 35.182257`, 35.161129`, \(-0.553607`\), 15.190457`, 
              15.195584`, 0.134343`, 30.917524`, 30.921185`, 0.095938`, 
              38.610568`, 38.620761`, 0.267102`, 45.309958`, 
              45.298968`, \(-0.287977`\), 16.5242`, 27.373887`, 29.805671`, 
              35.182257`, 45.309958`, 51.661916`, 62.578359`, 63.299733`, 
              15.190457`, 30.917524`, 38.610568`, 46.567634`, 40.993553`, 
              52.757922`, 54.416402`, 58.692759`}, {5, 28, 
              142.9727131`, \(-2.194809978`\), 2.527316558`, 0.25`, 1, 
              0.4`, \(-0.5006760704`\), 
              8.953386135`, \(-0.6136286055`\), \(-1.6255117`\), 3, 4, 5, 6, 
              0.651978`, \(-0.36949`\), \(-0.335563`\), 0.227656`, 
              0.033554`, \(-0.037595`\), 0.993837`, 0.479707`, 1.048583`, 
              0.756583`, 1.405597`, 1.25603`, 16.5242`, 16.535741`, 
              0.304261`, 28.391149`, 28.414039`, 0.603482`, 29.809674`, 
              29.822827`, 0.346773`, 36.10374`, 36.084757`, \(-0.500456`\), 
              15.153036`, 15.157822`, 0.126181`, 31.002878`, 31.00629`, 
              0.089949`, 39.045847`, 39.054949`, 0.239959`, 45.751652`, 
              45.74267`, \(-0.236806`\), 16.5242`, 28.391149`, 29.809674`, 
              36.10374`, 45.751652`, 51.995691`, 63.098811`, 63.812311`, 
              15.153036`, 31.002878`, 39.045847`, 46.796383`, 41.707834`, 
              53.598167`, 54.938231`, 59.350749`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["data l1_dcsan2*.out (This contains points from dtraj1*.out)", \
"Subsection"],

Cell[BoxData[
    \(\(datal1 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from fitting \
lowest state to lattice value.\n2 parity doublets with fractional errors 1 \
0.1.\nSpectrum for P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, \
c^2 error for each) =\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & \
1, 0.25 0.25 1 1).\nSpectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  \
(26/2,6)  (32/2,6) .\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( \
2,20/2,6) \n  ( 2,24/2,4)  ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \
\n  ( 3,35/2,5)  ( 4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy \
potential determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  \
( 1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{5, 28, 
              8.505592857`*^7, \(-0.25`\), \(-0.1791727179`\), 0, 
              1, \(-0.6`\), \(-0.0546190298`\), \(-3.237925911`\), \
\(-1.368713247`\), \(-1.628938322`\), 3, 4, 5, 6, 0.382899`, \(-61.708114`\), 
              44.599608`, \(-0.020335`\), \(-7.185211`\), \(-0.45285`\), \
\(-7.640614`\), \(-11.271982`\), \(-7.586359`\), \(-7.417061`\), \(-7.575458`\
\), \(-7.37746`\), 16.5242`, 16.524248`, \(-0.000053`\), 14.235972`, 
              14.235976`, \(-4.`*^-6\), 13.245118`, 12.958773`, 0.314317`, 
              12.904304`, 12.953032`, \(-0.053487`\), 14.979309`, 
              14.979441`, \(-0.000145`\), 13.175968`, 
              13.176031`, \(-0.00007`\), 12.828248`, 12.678661`, 0.164198`, 
              12.81084`, 12.391117`, 0.460722`, 16.5242`, 14.235972`, 
              13.245118`, 12.899083`, 12.828248`, 12.005296`, 12.472529`, 
              11.418297`, 14.979309`, 13.175968`, 12.81084`, 12.241295`, 
              12.666594`, 12.461902`, 12.904304`, 12.091148`}, {5, 28, 
              4.598422702`*^7, \(-0.25`\), \(-0.2436783132`\), 0, 
              1, \(-0.5`\), \(-0.06369534909`\), \(-3.857698506`\), \
\(-1.291006985`\), \(-1.500701684`\), 3, 4, 5, 6, 0.20831`, \(-43.604848`\), 
              36.735121`, \(-0.000537`\), \(-4.6058`\), \(-0.101569`\), \
\(-4.836822`\), \(-5.519107`\), \(-4.757851`\), \(-4.647732`\), \
\(-4.730698`\), \(-4.628797`\), 16.5242`, 16.524331`, \(-0.000106`\), 
              14.299335`, 14.299375`, \(-0.000033`\), 12.670587`, 12.66695`, 
              0.002954`, 12.471433`, 12.433108`, 0.031127`, 14.490531`, 
              14.490784`, \(-0.000206`\), 12.893019`, 12.892373`, 0.000525`, 
              12.281099`, 12.281402`, \(-0.000246`\), 11.949426`, 11.818178`, 
              0.106596`, 16.5242`, 14.299335`, 12.433012`, 11.946471`, 
              11.81869`, 12.281099`, 11.708411`, 10.36707`, 14.490531`, 
              12.893019`, 11.949426`, 11.755489`, 12.471433`, 12.670587`, 
              12.193523`, 10.714059`}, {3, 18, 
              2.030514653`*^7, \(-0.25`\), \(-0.3667133171`\), 0, 
              1, \(-0.4`\), \(-0.0009998522833`\), 
              263554.7326`, \(-1.234902323`\), \(-1.360155955`\), 3, 4, 5, 6, 
              0.227998`, \(-25.754959`\), 21.680785`, 
              0.034047`, \(-2.370213`\), 0.488455`, \(-2.277984`\), 
              0.42529`, \(-2.138765`\), \(-2.089982`\), \(-2.033756`\), \
\(-2.102242`\), 16.5242`, 16.524204`, \(-5.`*^-6\), 14.367476`, 14.367461`, 
              0.000021`, 12.674958`, 13.110068`, \(-0.582072`\), 12.619812`, 
              11.650104`, 1.297233`, 13.233941`, 13.234143`, \(-0.00027`\), 
              11.372472`, 11.372332`, 0.000188`, 11.153432`, 
              11.153483`, \(-0.000069`\), 10.823243`, 
              11.029721`, \(-0.276217`\), 16.5242`, 14.367476`, 12.619812`, 
              11.355107`, 10.729231`, 11.372472`, 8.912506`, 8.154013`, 
              13.233941`, 11.153432`, 10.823243`, 9.797909`, 11.650108`, 
              12.674958`, 9.518764`, 8.250221`}, {2, 9, 
              6.734188685`*^6, \(-0.25`\), \(-0.6378957771`\), 0, 
              1, \(-0.3`\), \(-0.102364`\), 50, \(-1.13357`\), \(-1.24487`\), 
              3, 4, 5, 6, 0.016099`, \(-15.102744`\), 15.488583`, 
              0.065482`, \(-1.073795`\), 1.01793`, \(-0.37668`\), 
              11.508437`, \(-0.219601`\), \(-0.503044`\), 
              0.024535`, \(-0.542781`\), 16.5242`, 16.524082`, 0.000019`, 
              13.218014`, 13.223723`, \(-0.000938`\), 13.04304`, 13.038128`, 
              0.000807`, 12.088007`, 12.635564`, \(-0.089968`\), 11.823534`, 
              11.822523`, 0.000166`, 9.816315`, 9.801591`, 0.002419`, 
              9.460985`, 9.475671`, \(-0.002413`\), 9.012793`, 8.674382`, 
              0.055603`, 16.5242`, 13.04304`, 13.218014`, 9.990004`, 
              9.816315`, 9.012793`, 7.318114`, 6.43763`, 11.823534`, 
              9.460985`, 7.910325`, 6.505385`, 11.132862`, 12.088007`, 
              8.155571`, 6.605444`}, {5, 29, 
              1.001541234`*^6, \(-0.25`\), \(-1.651403285`\), 0, 
              1, \(-0.2`\), \(-0.1086698821`\), 
              0.8281456595`, \(-1.056723989`\), \(-1.128476618`\), 3, 4, 5, 
              6, 0.055227`, \(-5.567835`\), 6.071214`, 
              0.063803`, \(-0.141743`\), 0.440171`, 0.465036`, 1.017231`, 
              0.43904`, 0.19366`, 0.609595`, 0.266003`, 16.5242`, 16.520722`, 
              0.005076`, 11.108681`, 11.115882`, \(-0.010508`\), 7.826362`, 
              7.674406`, 0.221737`, 7.677099`, 7.081938`, 0.86847`, 
              6.511451`, 6.501778`, 0.014116`, 2.907856`, 
              3.632663`, \(-1.057654`\), 1.488887`, 
              2.991588`, \(-2.192773`\), 0.867849`, 
              1.444152`, \(-0.840954`\), 16.5242`, 11.108681`, 7.677099`, 
              5.171322`, 2.907856`, 
              0.297068`, \(-1.645561`\), \(-2.019692`\), 
              6.511451`, \(-1.928803`\), 1.488887`, 0.867849`, 7.273032`, 
              7.826362`, 3.05647`, \(-0.809035`\)}, {2, 28, 
              8240.090929`, \(-0.25`\), \(-20.18332137`\), 0, 
              1, \(-0.1`\), \(-0.1203625276`\), 
              9.821136837`, \(-1.635938797`\), \(-2.216519213`\), 3, 4, 5, 6, 
              0.123918`, \(-1.011803`\), 1.436268`, \(-0.005358`\), 
              0.182925`, \(-0.025246`\), 0.59306`, 0.141304`, 0.325095`, 
              0.139217`, 0.319183`, 0.13347`, 16.5242`, 16.361409`, 
              6.514408`, \(-21.347603`\), \(-21.631381`\), 
              11.355956`, \(-28.999206`\), \(-28.863797`\), \(-5.418707`\), \
\(-70.038812`\), \(-69.854811`\), \(-7.363187`\), \(-56.296147`\), \
\(-56.508647`\), 8.503656`, \(-74.508301`\), \(-74.602102`\), 
              3.753639`, \(-116.777645`\), \(-116.629232`\), \(-5.939073`\), \
\(-116.930337`\), \(-117.116626`\), 7.454744`, 
              16.5242`, \(-28.999206`\), \(-21.347603`\), \(-77.285501`\), \
\(-122.265074`\), \(-119.082928`\), \(-169.60624`\), \(-185.031882`\), \
\(-56.296147`\), \(-74.508301`\), \(-116.777645`\), \(-116.930337`\), \
\(-70.038812`\), \(-77.272338`\), \(-99.969993`\), \(-143.829449`\)}, {5, 27, 
              6.48760648`, \(-1.605811226`\), 7.317176744`, 0, 1, 
              0, \(-0.1269419192`\), 
              3298.694781`, \(-0.6100093656`\), \(-0.637282721`\), 3, 4, 5, 
              6, 0.183646`, \(-0.425253`\), 0.257634`, 0.1378`, 
              0.230623`, \(-0.225544`\), 0.619163`, 0.590794`, 0.714542`, 
              0.727002`, 1.01287`, 1.00569`, 16.5242`, 16.572939`, 1.047914`, 
              27.575936`, 27.545732`, \(-0.649394`\), 34.523312`, 34.548577`, 
              0.543189`, 44.584173`, 44.592243`, 0.173527`, 23.001774`, 
              23.036265`, 0.741561`, 43.640684`, 43.680151`, 0.848562`, 
              57.738851`, 57.745493`, 0.142797`, 61.62535`, 61.664317`, 
              0.837816`, 16.5242`, 27.575936`, 34.523312`, 48.905455`, 
              57.738851`, 74.111648`, 88.564044`, 94.845903`, 23.001774`, 
              43.640684`, 61.62535`, 64.578006`, 44.584173`, 65.765316`, 
              72.758186`, 78.780881`}, {5, 28, 
              16.8599107`, \(-1.978616626`\), 6.159570992`, 0, 1, 
              0.1`, \(-0.1576522128`\), 
              3.175031293`, \(-0.7144879676`\), \(-0.7738504657`\), 3, 4, 5, 
              6, 0.178513`, \(-0.450108`\), 0.310688`, 0.136824`, 
              0.216268`, \(-0.204175`\), 0.607819`, 0.493893`, 0.673726`, 
              0.653448`, 0.950943`, 0.951285`, 16.5242`, 16.575964`, 
              0.910682`, 21.282561`, 21.304289`, 0.382251`, 28.459805`, 
              28.429482`, \(-0.533467`\), 40.877168`, 40.885071`, 0.139033`, 
              18.504288`, 18.530266`, 0.457022`, 38.174541`, 38.209026`, 
              0.606703`, 49.81617`, 49.824699`, 0.150052`, 51.592328`, 
              51.625866`, 0.590036`, 16.5242`, 28.459805`, 21.282561`, 
              41.577078`, 49.81617`, 63.964952`, 82.803336`, 83.273165`, 
              18.504288`, 38.174541`, 51.592328`, 56.315261`, 40.877168`, 
              64.327354`, 65.663891`, 72.450382`}, {5, 29, 
              64.38017244`, \(-1.563904091`\), 5.175801666`, 0, 1, 
              0.2`, \(-0.1526204664`\), 
              7.982803611`, \(-0.727770436`\), \(-0.617451612`\), 3, 4, 5, 6, 
              0.171828`, \(-0.508954`\), 0.468412`, 0.145203`, 
              0.192053`, \(-0.179706`\), 0.58734`, 0.419822`, 0.685533`, 
              0.61586`, 0.985909`, 0.949054`, 16.5242`, 16.554148`, 
              0.426148`, 20.652406`, 20.669431`, 0.242269`, 25.90603`, 
              25.888701`, \(-0.246586`\), 35.67463`, 35.678874`, 0.060392`, 
              14.655226`, 14.672292`, 0.242846`, 32.43012`, 32.454178`, 
              0.342334`, 41.774224`, 41.778532`, 0.061298`, 42.988725`, 
              43.012134`, 0.333103`, 16.5242`, 25.90603`, 20.652406`, 
              40.289202`, 41.774224`, 53.721455`, 67.333242`, 77.018095`, 
              14.655226`, 32.43012`, 42.988725`, 47.53576`, 35.67463`, 
              55.170535`, 58.228122`, 60.907572`}, {5, 29, 
              76.56907708`, \(-1.230354547`\), 5.156542178`, 0, 1, 
              0.3`, \(-0.1628526683`\), 
              8.731220557`, \(-0.6509953982`\), \(-0.4887938623`\), 3, 4, 5, 
              6, 0.146159`, \(-0.605162`\), 0.713967`, 0.158494`, 
              0.162585`, \(-0.151101`\), 0.55502`, 0.39839`, 0.677144`, 
              0.619732`, 0.991969`, 0.986312`, 16.5242`, 16.551359`, 
              0.327502`, 19.409538`, 19.42698`, 0.210324`, 26.328159`, 
              26.313581`, \(-0.175796`\), 34.949883`, 34.953409`, 0.042523`, 
              13.213976`, 13.229683`, 0.189408`, 30.683669`, 30.70592`, 
              0.268321`, 39.943157`, 39.954342`, 0.13488`, 41.858204`, 
              41.87289`, 0.17709`, 16.5242`, 26.328159`, 19.409538`, 
              39.346483`, 39.943157`, 51.630925`, 63.932658`, 73.496628`, 
              13.213976`, 30.683669`, 41.858204`, 46.384251`, 34.949883`, 
              54.555765`, 51.714252`, 65.862554`}, {5, 29, 
              17.8919222`, \(-0.8932092843`\), 8.378772683`, 0, 1, 
              0.4`, \(-0.3092720076`\), 
              1321.038929`, \(-1.07397727`\), \(-0.7601159105`\), 3, 4, 5, 6, 
              0.065605`, \(-0.795558`\), 1.171588`, 0.123273`, 
              0.150813`, \(-0.08627`\), 0.494249`, 0.375169`, 0.546618`, 
              0.530788`, 0.800294`, 0.805698`, 16.5242`, 16.614728`, 
              0.796201`, 30.757687`, 30.797142`, 0.347012`, 41.218399`, 
              41.189521`, \(-0.253981`\), 47.558421`, 47.688066`, 1.140235`, 
              16.839978`, 16.886726`, 0.411154`, 38.619054`, 38.679714`, 
              0.533507`, 55.648524`, 55.680356`, 0.279962`, 63.074838`, 
              63.127455`, 0.462771`, 16.5242`, 41.218399`, 30.757687`, 
              47.558421`, 55.648524`, 73.737764`, 94.114007`, 101.112355`, 
              16.839978`, 38.619054`, 63.074838`, 67.43449`, 48.7175`, 
              72.205177`, 92.102826`, 93.28714`}, {5, 28, 
              5.956982876`*^7, \(-0.25`\), \(-0.2140956987`\), 0.0025`, 
              1, \(-0.6`\), \(-0.03476263172`\), 
              0.4024426615`, \(-1.493661389`\), \(-1.484313024`\), 3, 4, 5, 
              6, 0.298962`, \(-47.993747`\), 35.159157`, 
              0.016411`, \(-5.2454`\), \(-0.103526`\), \(-5.523914`\), \
\(-6.127848`\), \(-5.457377`\), \(-5.245693`\), \(-5.431364`\), \
\(-5.184022`\), 16.5242`, 16.524221`, \(-0.000022`\), 14.267604`, 14.267602`, 
              3.`*^-6, 13.046584`, 12.635055`, 0.421448`, 12.65977`, 
              12.594015`, 0.06734`, 14.496434`, 14.496552`, \(-0.000121`\), 
              12.823569`, 12.823803`, \(-0.000239`\), 12.294156`, 12.234537`, 
              0.061056`, 11.925556`, 11.653171`, 0.278951`, 16.5242`, 
              14.267604`, 13.046584`, 12.429646`, 12.294156`, 11.477586`, 
              11.925556`, 10.706926`, 14.496434`, 12.823569`, 11.759348`, 
              11.593406`, 12.182479`, 12.213531`, 12.65977`, 11.23504`}, {5, 
              28, 3.034167278`*^7, \(-0.25`\), \(-0.4358994754`\), 0.0025`, 
              1, \(-0.5`\), \(-0.0488944857`\), \(-17.46593383`\), \
\(-1.398320065`\), \(-1.3901566`\), 3, 4, 5, 6, 0.17595`, \(-32.75941`\), 
              28.584608`, 0.04543`, \(-3.172137`\), 
              0.399441`, \(-3.098787`\), \(-0.979229`\), \(-2.994125`\), \
\(-2.898907`\), \(-2.92412`\), \(-2.864998`\), 24.010537`, 24.01053`, 
              8.`*^-6, 20.983946`, 20.983956`, \(-0.000012`\), 18.484071`, 
              17.295979`, 1.457959`, 17.295949`, 17.256725`, 0.048133`, 
              19.94607`, 19.946463`, \(-0.000483`\), 17.570535`, 17.569844`, 
              0.000848`, 17.089477`, 17.088821`, 0.000805`, 16.125368`, 
              16.127023`, \(-0.00203`\), 16.5242`, 24.010537`, 20.983946`, 
              17.295949`, 15.90119`, 17.089477`, 15.687949`, 12.918523`, 
              19.94607`, 17.570535`, 16.125368`, 15.130506`, 17.19874`, 
              18.484071`, 15.775824`, 13.897468`}, {2, 24, 
              1.238398414`*^7, \(-0.25`\), \(-0.4695987022`\), 0.0025`, 
              1, \(-0.4`\), 0.0003522080079`, 
              34035.94248`, \(-1.32767921`\), \(-0.8128614238`\), 3, 4, 5, 6, 
              0.222545`, \(-18.267718`\), 16.1436`, 
              0.079583`, \(-1.560241`\), 1.092804`, \(-1.016217`\), 
              3.390333`, \(-0.940098`\), \(-0.925088`\), \(-0.816952`\), \
\(-0.943588`\), 16.5242`, 16.524204`, \(-7.`*^-6\), 14.224906`, 14.224888`, 
              0.000031`, 12.521885`, 13.338133`, \(-1.364857`\), 11.609265`, 
              10.850318`, 1.269044`, 11.808185`, 11.808436`, \(-0.00042`\), 
              10.465772`, 10.463881`, 0.003162`, 9.547821`, 
              10.05515`, \(-0.848311`\), 9.533651`, 
              9.548747`, \(-0.025241`\), 16.5242`, 14.224906`, 11.609265`, 
              10.685175`, 9.43612`, 10.465772`, 7.74209`, 5.775401`, 
              11.808185`, 9.547821`, 9.533651`, 7.578462`, 10.850323`, 
              12.521885`, 8.956652`, 5.665862`}, {5, 29, 
              3.475369645`*^6, \(-0.25`\), \(-0.8864886872`\), 0.0025`, 
              1, \(-0.3`\), \(-0.08511868988`\), 
              5.159717072`, \(-1.201831604`\), \(-1.209889827`\), 3, 4, 5, 6, 
              0.091129`, \(-9.458494`\), 9.601632`, 
              0.080424`, \(-0.543714`\), 0.883355`, 0.276236`, 2.778719`, 
              0.296602`, 0.009831`, 0.484631`, 0.035402`, 16.5242`, 
              16.524268`, \(-0.000088`\), 12.835076`, 
              12.835102`, \(-0.000034`\), 11.033099`, 
              11.417346`, \(-0.496661`\), 9.83791`, 
              11.033443`, \(-1.545294`\), 8.8803`, 8.879705`, 0.000769`, 
              6.771424`, 6.186912`, 0.755516`, 6.251693`, 6.175115`, 
              0.098981`, 6.151009`, 4.931221`, 1.576646`, 16.5242`, 
              11.033099`, 12.835076`, 8.02623`, 6.771424`, 6.251693`, 
              3.669714`, 3.04048`, 8.8803`, 6.151009`, 3.582883`, 2.305116`, 
              9.239158`, 9.83791`, 5.552344`, 3.841968`}, {5, 29, 
              307828.5616`, \(-0.25`\), \(-2.979851399`\), 0.0025`, 
              1, \(-0.2`\), \(-0.09606131965`\), 
              2.340399182`, \(-1.10409012`\), \(-1.130050596`\), 3, 4, 5, 6, 
              0.129209`, \(-2.620822`\), 3.569131`, 0.080935`, 0.051344`, 
              0.15166`, 0.578672`, 0.279624`, 0.572897`, 0.327231`, 
              0.785364`, 0.494858`, 16.5242`, 16.519125`, 0.031264`, 
              7.356855`, 7.363809`, \(-0.042839`\), 3.533713`, 3.528652`, 
              0.031179`, 1.653711`, 0.880219`, 
              4.764987`, \(-4.930779`\), \(-4.976483`\), 
              0.281551`, \(-5.314316`\), \(-5.504237`\), 
              1.169987`, \(-5.511691`\), \(-6.854663`\), 
              8.273194`, \(-9.586531`\), \(-10.479925`\), 5.503628`, 
              16.5242`, 7.356855`, 
              3.533713`, \(-1.247947`\), \(-10.816689`\), \(-5.314316`\), \
\(-15.295181`\), \(-20.737353`\), \(-4.930779`\), \(-5.511691`\), \
\(-9.586531`\), \(-10.533097`\), \(-0.340799`\), 
              1.653711`, \(-6.40275`\), \(-15.207081`\)}, {5, 28, 
              16890.51171`, \(-0.8466913159`\), 12.32412494`, 0.0025`, 
              1, \(-0.1`\), \(-0.1054218736`\), 
              10.02281893`, \(-1.037497677`\), \(-0.9758813747`\), 3, 4, 5, 
              6, 0.205033`, \(-0.527089`\), 0.47715`, 0.099905`, 
              0.235619`, \(-0.180997`\), 0.649302`, 0.7607`, 0.686517`, 
              0.816924`, 0.947321`, 0.960028`, 16.5242`, 16.591086`, 
              2.704166`, 35.277363`, 35.223654`, \(-2.171423`\), 55.852035`, 
              55.894811`, 1.729392`, 65.470868`, 63.906076`, \(-63.263881`\), 
              41.678521`, 41.739488`, 2.46486`, 71.199728`, 71.252472`, 
              2.132426`, 95.994178`, 95.990704`, \(-0.140458`\), 103.375109`, 
              103.310616`, \(-2.607435`\), 16.5242`, 35.277363`, 55.852035`, 
              65.470868`, 95.994178`, 122.295763`, 138.484549`, 156.784241`, 
              41.678521`, 71.199728`, 103.375109`, 107.634428`, 68.533397`, 
              93.458024`, 111.226592`, 124.584953`}, {5, 28, 
              12.19410144`, \(-1.741650786`\), 6.150516234`, 0.0025`, 1, 
              0, \(-0.1427384314`\), 
              3.902830427`, \(-0.6847613955`\), \(-0.9005255551`\), 3, 4, 5, 
              6, 0.220235`, \(-0.438311`\), 0.241942`, 0.148216`, 
              0.205133`, \(-0.202935`\), 0.660595`, 0.562635`, 0.717329`, 
              0.717958`, 0.997594`, 1.025683`, 16.5242`, 16.572728`, 
              1.051732`, 24.242949`, 24.263616`, 0.447911`, 27.24394`, 
              27.209934`, \(-0.737011`\), 42.157754`, 42.164464`, 0.145438`, 
              20.940126`, 20.965424`, 0.548285`, 41.22526`, 41.256783`, 
              0.6832`, 53.292587`, 53.29143`, \(-0.025091`\), 54.409542`, 
              54.441133`, 0.684653`, 16.5242`, 27.24394`, 24.242949`, 
              43.441675`, 53.292587`, 67.62553`, 83.918607`, 87.991754`, 
              20.940126`, 41.22526`, 54.409542`, 58.623949`, 42.157754`, 
              63.81719`, 67.264726`, 68.896791`}, {5, 28, 
              30.89468027`, \(-2.239013266`\), 5.120250466`, 0.0025`, 1, 
              0.1`, \(-0.1449431441`\), 
              3.121153682`, \(-0.6931903726`\), \(-0.8724593878`\), 3, 4, 5, 
              6, 0.229482`, \(-0.456125`\), 0.284766`, 0.156218`, 
              0.184448`, \(-0.184063`\), 0.655358`, 0.492582`, 0.718483`, 
              0.679022`, 1.00476`, 1.021414`, 16.5242`, 16.561806`, 
              0.707005`, 19.874834`, 19.889336`, 0.272639`, 26.4059`, 
              26.383217`, \(-0.426432`\), 36.789469`, 36.825206`, 0.671854`, 
              17.052162`, 17.068377`, 0.304849`, 36.524281`, 36.547299`, 
              0.432742`, 45.378178`, 45.399082`, 0.392996`, 46.089671`, 
              46.089953`, 0.005293`, 16.5242`, 19.874834`, 26.4059`, 
              36.789469`, 46.089671`, 58.270551`, 75.056662`, 78.280563`, 
              17.052162`, 36.524281`, 45.378178`, 50.808103`, 38.455715`, 
              58.032624`, 61.761928`, 62.361178`}, {5, 29, 
              80.01482926`, \(-1.712877507`\), 4.304294452`, 0.0025`, 1, 
              0.2`, \(-0.1510506182`\), 
              9.118035697`, \(-0.6920763756`\), \(-0.7567231508`\), 3, 4, 5, 
              6, 0.217016`, \(-0.502598`\), 0.407952`, 0.162789`, 
              0.164302`, \(-0.161776`\), 0.636471`, 0.41004`, 0.720897`, 
              0.635528`, 1.021075`, 1.011232`, 16.5242`, 16.549014`, 
              0.370861`, 18.945576`, 18.956816`, 0.167989`, 23.752966`, 
              23.737625`, \(-0.229285`\), 32.998914`, 33.001561`, 0.039559`, 
              13.55101`, 13.562951`, 0.178454`, 30.473234`, 30.490873`, 
              0.263618`, 37.68451`, 37.687833`, 0.049664`, 38.394008`, 
              38.406449`, 0.185937`, 16.5242`, 23.752966`, 18.945576`, 
              38.380743`, 38.394008`, 48.561334`, 62.661766`, 64.592496`, 
              13.55101`, 30.473234`, 37.68451`, 42.792704`, 32.998914`, 
              50.457231`, 51.231459`, 56.656557`}, {2, 24, 
              49.57830722`, \(-1.364659009`\), 4.244112667`, 0.0025`, 1, 
              0.3`, \(-0.1592097301`\), 
              20.43087215`, \(-0.601769787`\), \(-0.6617899157`\), 3, 4, 5, 
              6, 0.193358`, \(-0.575176`\), 0.596096`, 0.174924`, 
              0.136683`, \(-0.134008`\), 0.608953`, 0.391327`, 0.708893`, 
              0.639448`, 1.017269`, 1.045132`, 3.913913`, 3.966554`, 
              0.691188`, 16.5242`, 16.545657`, 0.281728`, 32.103483`, 
              32.10579`, 0.030285`, 38.404213`, 38.427592`, 0.306968`, 
              12.324814`, 12.335751`, 0.143598`, 28.80132`, 28.817539`, 
              0.212952`, 36.403258`, 36.407723`, 0.058622`, 36.594878`, 
              36.604787`, 0.130112`, 16.5242`, 38.542712`, 3.913913`, 
              38.404213`, 36.594878`, 46.403769`, 60.544483`, 62.209078`, 
              12.324814`, 28.80132`, 36.403258`, 41.449345`, 32.103483`, 
              49.097475`, 50.154391`, 53.535426`}, {5, 28, 
              71.59242833`, \(-1.118249183`\), 4.609546775`, 0.0025`, 1, 
              0.4`, \(-0.2137856345`\), 
              12.18923344`, \(-0.4688993949`\), \(-0.5559607489`\), 3, 4, 5, 
              6, 0.150357`, \(-0.69441`\), 0.885001`, 0.185747`, 
              0.112023`, \(-0.108466`\), 0.570526`, 0.383342`, 0.686462`, 
              0.647574`, 1.002046`, 1.07789`, 16.5242`, 16.558492`, 
              0.380268`, 17.695791`, 17.704944`, 0.101496`, 25.85507`, 
              25.838812`, \(-0.18029`\), 32.995242`, 32.998791`, 0.039354`, 
              11.814377`, 11.828508`, 0.156706`, 28.299306`, 28.319829`, 
              0.227587`, 37.118016`, 37.130071`, 0.133688`, 38.170776`, 
              38.178572`, 0.08645`, 16.5242`, 17.695791`, 25.85507`, 
              38.871775`, 37.118016`, 47.429387`, 60.92756`, 64.67802`, 
              11.814377`, 28.299306`, 38.170776`, 43.252241`, 32.995242`, 
              50.977712`, 50.201241`, 56.838291`}, {5, 28, 
              4.004243597`*^7, \(-0.25`\), \(-0.2611360457`\), 0.01`, 
              1, \(-0.6`\), \(-0.02469927932`\), 
              3.170148461`, \(-1.56622243`\), \(-1.366838733`\), 3, 4, 5, 6, 
              0.249369`, \(-36.602272`\), 28.94977`, 
              0.062008`, \(-3.733987`\), 
              0.438152`, \(-3.645217`\), \(-0.365917`\), \(-3.559551`\), \
\(-3.403734`\), \(-3.493798`\), \(-3.336242`\), 16.5242`, 
              16.524213`, \(-0.000014`\), 14.328997`, 14.328991`, 6.`*^-6, 
              12.958844`, 12.301726`, 0.684655`, 12.248754`, 12.220043`, 
              0.029914`, 13.80746`, 13.807596`, \(-0.000141`\), 12.40383`, 
              12.404508`, \(-0.000706`\), 11.33915`, 11.339083`, 0.00007`, 
              11.036146`, 11.035919`, 0.000237`, 16.5242`, 14.328997`, 
              12.958844`, 11.6262`, 11.036146`, 11.33915`, 10.942174`, 
              9.702934`, 13.80746`, 12.40383`, 10.681499`, 10.364439`, 
              11.562953`, 12.248754`, 11.742046`, 10.058547`}, {1, 11, 
              1.927266866`*^7, \(-0.25`\), \(-0.3764351212`\), 0.01`, 
              1, \(-0.5`\), \(-0.01395649305`\), 
              21421.853`, \(-1.496181322`\), \(-1.062187764`\), 3, 4, 5, 6, 
              0.224208`, \(-23.360925`\), 20.189361`, 
              0.092622`, \(-2.13834`\), 1.083511`, \(-1.632543`\), 
              3.956969`, \(-1.534595`\), \(-1.473891`\), \(-1.405602`\), \
\(-1.458372`\), 16.5242`, 19.98034`, \(-4.66715`\), 14.29065`, 
              18.894231`, \(-6.216649`\), 12.443274`, 
              18.311279`, \(-7.924119`\), 10.796235`, 
              16.524206`, \(-7.735018`\), 12.481808`, 
              12.48205`, \(-0.000327`\), 10.882955`, 10.882104`, 0.001149`, 
              10.622976`, 10.623305`, \(-0.000444`\), 9.901342`, 
              9.901343`, \(-1.`*^-6\), 16.5242`, 14.29065`, 4.387382`, 
              3.099783`, 9.59653`, 10.882955`, 9.325007`, 7.081594`, 
              12.481808`, 10.622976`, 9.901342`, 8.5422`, 10.796235`, 
              12.443274`, 8.558149`, 8.482661`}, {5, 28, 
              6.159214505`*^6, \(-0.25`\), \(-0.6659126284`\), 0.01`, 
              1, \(-0.4`\), \(-0.05189547905`\), \(-3.643427767`\), \
\(-1.350754593`\), \(-1.281570554`\), 3, 4, 5, 6, 0.157859`, \(-13.046462`\), 
              12.449634`, 0.101147`, \(-0.993606`\), 
              1.244265`, \(-0.095658`\), 3.73601`, 0.022881`, \(-0.244385`\), 
              0.288211`, \(-0.238659`\), 16.5242`, 
              16.525857`, \(-0.002788`\), 13.691765`, 13.533873`, 0.265561`, 
              13.537578`, 12.163634`, 2.310865`, 12.164785`, 10.47163`, 
              2.847753`, 10.661134`, 10.660903`, 0.00039`, 10.264149`, 
              8.870987`, 2.343189`, 8.864689`, 7.402028`, 2.460079`, 
              8.256757`, 7.110425`, 1.92804`, 16.5242`, 13.537578`, 
              12.164785`, 7.439888`, 8.256757`, 10.264149`, 6.520319`, 
              3.274635`, 10.661134`, 8.864689`, 6.258065`, 4.593101`, 
              10.471878`, 13.691765`, 7.622814`, 3.428845`}, {5, 28, 
              1.483867549`*^6, \(-0.25`\), \(-1.356759154`\), 0.01`, 
              1, \(-0.3`\), \(-0.06443197031`\), 
              0.4338698521`, \(-1.225656298`\), \(-1.212215345`\), 3, 4, 5, 
              6, 0.162375`, \(-4.926705`\), 5.55751`, 
              0.092564`, \(-0.181354`\), 0.48039`, 0.518223`, 0.747933`, 
              0.514053`, 0.243243`, 0.725039`, 0.379731`, 16.5242`, 
              16.524904`, \(-0.002483`\), 8.757574`, 
              8.76189`, \(-0.015215`\), 6.535682`, 6.530672`, 0.01766`, 
              6.515285`, 6.51218`, 0.010947`, 3.991672`, 2.705611`, 4.53319`, 
              2.715646`, 1.554873`, 4.091565`, 1.449501`, 
              1.468638`, \(-0.067455`\), 0.785134`, 
              0.785564`, \(-0.001513`\), 16.5242`, 8.757574`, 6.535682`, 
              6.515285`, 0.785134`, 
              3.991672`, \(-2.362796`\), \(-6.079492`\), 
              2.715646`, \(-1.287352`\), 1.449501`, \(-1.069852`\), 
              6.279911`, 6.433357`, 0.713234`, \(-0.850868`\)}, {5, 28, 
              48779.59938`, \(-0.25`\), \(-8.358483512`\), 0.01`, 
              1, \(-0.2`\), \(-0.06578402854`\), 
              28.41192962`, \(-2.418526817`\), \(-3.867813095`\), 3, 4, 5, 6, 
              0.221746`, \(-1.053367`\), 1.550972`, \(-0.055548`\), 
              0.191326`, \(-0.064548`\), 0.654358`, \(-0.04371`\), 
              0.093018`, \(-0.081528`\), \(-0.149332`\), \(-0.172287`\), 
              16.5242`, 16.517709`, 
              0.1925`, \(-7.03491`\), \(-7.029816`\), \(-0.151073`\), \
\(-14.782584`\), \(-14.770727`\), \(-0.351603`\), \(-25.389184`\), \
\(-26.232536`\), 25.009942`, \(-28.764475`\), \(-28.805832`\), 
              1.226457`, \(-37.663614`\), \(-37.685211`\), 
              0.640465`, \(-52.326686`\), \(-55.212945`\), 
              85.59323`, \(-55.383961`\), \(-55.830838`\), 13.25235`, 
              16.5242`, \(-7.03491`\), \(-14.782584`\), \(-25.389184`\), \
\(-55.727038`\), \(-52.326686`\), \(-82.828367`\), \(-82.841019`\), \
\(-28.764475`\), \(-37.663614`\), \(-55.383961`\), \(-68.263551`\), \
\(-31.620071`\), \(-25.930642`\), \(-55.194741`\), \(-58.515298`\)}, {5, 28, 
              8.622903251`, \(-1.20270192`\), 7.019190474`, 0.01`, 
              1, \(-0.1`\), \(-0.1137729609`\), 
              15.60140799`, \(-0.8544858859`\), \(-0.9670858735`\), 3, 4, 5, 
              6, 0.254851`, \(-0.479775`\), 0.329655`, 0.14177`, 
              0.202739`, \(-0.189384`\), 0.693188`, 0.651652`, 0.750324`, 
              0.77387`, 1.037434`, 1.040646`, 16.5242`, 16.561305`, 1.06199`, 
              26.448697`, 26.41935`, \(-0.839951`\), 35.269708`, 35.288583`, 
              0.540246`, 46.961147`, 46.965686`, 0.129924`, 25.977135`, 
              26.003191`, 0.745764`, 48.264026`, 48.292502`, 0.815029`, 
              62.145904`, 62.141306`, \(-0.131603`\), 64.349922`, 
              64.349811`, \(-0.003179`\), 16.5242`, 26.448697`, 35.269708`, 
              50.032953`, 62.145904`, 78.631277`, 93.141269`, 101.932312`, 
              25.977135`, 48.264026`, 64.349922`, 66.219089`, 46.961147`, 
              66.955569`, 76.048132`, 78.155961`}, {5, 28, 
              42.6448837`, \(-2.0944137`\), 4.717747391`, 0.01`, 1, 
              0, \(-0.1374515649`\), 
              4.926686205`, \(-0.8218657349`\), \(-1.045721806`\), 3, 4, 5, 
              6, 0.272547`, \(-0.44213`\), 0.215563`, 0.153149`, 
              0.187408`, \(-0.185132`\), 0.702701`, 0.489438`, 0.747954`, 
              0.672119`, 1.030138`, 1.007796`, 16.5242`, 16.557289`, 
              0.680739`, 21.167093`, 21.178592`, 0.236557`, 23.977849`, 
              23.954765`, \(-0.474917`\), 36.836342`, 36.839509`, 0.065155`, 
              17.538907`, 17.55353`, 0.300842`, 36.263497`, 36.283511`, 
              0.411753`, 43.924221`, 43.930849`, 0.136358`, 45.389482`, 
              45.397995`, 0.175127`, 16.5242`, 23.977849`, 21.167093`, 
              38.39321`, 45.389482`, 56.810419`, 72.767518`, 73.070314`, 
              17.538907`, 36.263497`, 43.924221`, 48.911241`, 36.836342`, 
              56.285452`, 56.614174`, 58.907584`}, {5, 28, 
              37.48072622`, \(-2.326386981`\), 4.590189381`, 0.01`, 1, 
              0.1`, \(-0.1704926106`\), 
              3.712825258`, \(-0.6822777184`\), \(-0.9764509177`\), 3, 4, 5, 
              6, 0.26467`, \(-0.463799`\), 0.257128`, 0.170566`, 
              0.15556`, \(-0.158852`\), 0.694615`, 0.491079`, 0.752498`, 
              0.695941`, 1.043596`, 1.069996`, 16.5242`, 16.558597`, 
              0.668617`, 20.143849`, 20.158418`, 0.283194`, 26.52706`, 
              26.504511`, \(-0.438324`\), 35.447305`, 35.477392`, 0.584822`, 
              16.499431`, 16.513509`, 0.273653`, 35.83033`, 35.851174`, 
              0.405166`, 42.456364`, 42.463196`, 0.13281`, 44.818894`, 
              44.828821`, 0.192962`, 16.5242`, 20.143849`, 26.52706`, 
              35.447305`, 44.818894`, 55.957851`, 71.423827`, 74.167461`, 
              16.499431`, 35.83033`, 42.456364`, 48.959441`, 37.69718`, 
              56.798472`, 57.775716`, 62.678804`}, {5, 28, 
              14.77855901`, \(-1.858496958`\), 6.513660162`, 0.01`, 1, 
              0.2`, \(-0.1617987206`\), 
              1.614178306`, \(-0.6828268425`\), \(-0.8130905319`\), 3, 4, 5, 
              6, 0.260263`, \(-0.495433`\), 0.348811`, 0.175707`, 
              0.139561`, \(-0.140937`\), 0.680793`, 0.681024`, 0.768553`, 
              0.836331`, 1.080248`, 1.170948`, 16.5242`, 16.566916`, 
              1.158646`, 29.832171`, 29.858546`, 0.715391`, 39.905153`, 
              39.879168`, \(-0.704819`\), 44.02221`, 44.057518`, 0.957685`, 
              22.50949`, 22.525839`, 0.44345`, 51.229386`, 51.254826`, 
              0.69004`, 59.773075`, 59.779777`, 0.181782`, 63.663757`, 
              63.675178`, 0.309784`, 16.5242`, 29.832171`, 39.905153`, 
              44.02221`, 63.663757`, 79.318704`, 102.394977`, 106.788467`, 
              22.50949`, 51.229386`, 59.773075`, 69.988326`, 55.012094`, 
              82.140398`, 83.67293`, 90.864983`}, {5, 28, 
              59.29904956`, \(-1.495245059`\), 4.370973253`, 0.01`, 1, 
              0.3`, \(-0.2198065369`\), 
              4.743790593`, \(-0.4898773729`\), \(-0.8223394741`\), 3, 4, 5, 
              6, 0.222718`, \(-0.566755`\), 0.518092`, 0.191145`, 
              0.107216`, \(-0.109044`\), 0.65251`, 0.448647`, 0.733456`, 
              0.693104`, 1.036065`, 1.120921`, 16.5242`, 16.552967`, 
              0.448078`, 18.990972`, 19.008914`, 0.27946`, 27.364155`, 
              27.345209`, \(-0.295096`\), 35.401162`, 35.433171`, 0.498578`, 
              13.883027`, 13.896106`, 0.20371`, 32.112736`, 32.132779`, 
              0.312194`, 39.045415`, 39.052351`, 0.108034`, 40.82897`, 
              40.837329`, 0.13021`, 16.5242`, 18.990972`, 27.364155`, 
              35.401162`, 40.82897`, 51.049668`, 66.747389`, 68.963689`, 
              13.883027`, 32.112736`, 39.045415`, 45.881243`, 35.786497`, 
              53.613043`, 55.120665`, 59.909564`}, {5, 29, 
              47.6972641`, \(-1.241379469`\), 4.057617535`, 0.01`, 1, 
              0.4`, \(-0.2485512306`\), 
              158.9369322`, \(-0.3464134445`\), \(-0.7346783489`\), 3, 4, 5, 
              6, 0.190721`, \(-0.654222`\), 0.736557`, 0.200733`, 
              0.084374`, \(-0.085197`\), 0.621569`, 0.391999`, 0.714712`, 
              0.671681`, 1.022611`, 1.133144`, 3.575038`, 3.629926`, 
              0.679616`, 16.5242`, 16.556458`, 0.399424`, 32.070148`, 
              32.073528`, 0.041846`, 38.046614`, 38.071868`, 0.31269`, 
              11.749776`, 11.762241`, 0.154341`, 27.901691`, 27.920585`, 
              0.233945`, 35.281377`, 35.288201`, 0.0845`, 36.102849`, 
              36.11021`, 0.091147`, 16.5242`, 40.230547`, 3.575038`, 
              38.046614`, 36.102849`, 45.303831`, 60.108383`, 61.211175`, 
              11.749776`, 27.901691`, 35.281377`, 41.309516`, 32.070148`, 
              48.324792`, 50.007559`, 53.685422`}, {2, 13, 
              2.375440325`*^7, \(-0.25`\), \(-0.5416140298`\), 0.0225`, 
              1, \(-0.6`\), \(-0.03441358872`\), \(-14.01300973`\), \
\(-1.581518716`\), \(-1.279486542`\), 3, 4, 5, 6, 0.214946`, \(-27.338841`\), 
              23.464671`, 0.107112`, \(-2.631019`\), 1.146481`, \(-2.0697`\), 
              1.961632`, \(-1.95493`\), \(-1.905595`\), \(-1.810028`\), \
\(-1.865775`\), 26.294028`, 26.292839`, 0.002214`, 23.880564`, 23.880541`, 
              0.000044`, 20.583202`, 18.644556`, 3.611083`, 18.644549`, 
              17.882766`, 1.418959`, 21.309521`, 21.309911`, \(-0.000727`\), 
              18.766205`, 18.766733`, \(-0.000983`\), 18.439957`, 18.439157`, 
              0.001489`, 16.523509`, 16.52369`, \(-0.000336`\), 16.5242`, 
              26.294028`, 23.880564`, 18.644549`, 16.095209`, 18.439957`, 
              16.124455`, 14.489032`, 21.309521`, 18.766205`, 16.523509`, 
              14.53703`, 17.882843`, 20.583202`, 17.357627`, 14.502476`}, {5, 
              28, 1.153425887`*^7, \(-0.25`\), \(-0.4866259431`\), 0.0225`, 
              1, \(-0.5`\), \(-0.037195166`\), 
              1.577904253`, \(-1.470688041`\), \(-1.24390765`\), 3, 4, 5, 6, 
              0.202783`, \(-16.493355`\), 14.923191`, 
              0.121292`, \(-1.434044`\), 1.495546`, \(-0.504866`\), 
              5.417322`, \(-0.359145`\), \(-0.533718`\), \(-0.080769`\), \
\(-0.528181`\), 16.5242`, 16.524259`, \(-0.000092`\), 13.59743`, 
              13.59754`, \(-0.000174`\), 12.347688`, 10.510837`, 2.900148`, 
              10.510829`, 10.197159`, 0.495243`, 10.780935`, 
              10.781299`, \(-0.000575`\), 10.353966`, 8.744492`, 2.541149`, 
              8.74264`, 7.955302`, 1.243104`, 7.956374`, 7.954754`, 
              0.002558`, 16.5242`, 13.59743`, 10.510829`, 7.965377`, 
              7.956374`, 10.353966`, 6.957457`, 5.228067`, 10.780935`, 
              8.74264`, 7.955519`, 5.588121`, 9.743872`, 12.347688`, 
              7.009504`, 6.154984`}, {5, 28, 
              3.704559742`*^6, \(-0.25`\), \(-0.8586285228`\), 0.0225`, 
              1, \(-0.4`\), \(-0.05172666732`\), 
              0.9133190561`, \(-1.349851111`\), \(-1.231065768`\), 3, 4, 5, 
              6, 0.191399`, \(-7.860507`\), 7.774097`, 
              0.109464`, \(-0.498764`\), 0.886897`, 0.393727`, 1.903367`, 
              0.421578`, 0.128788`, 0.656523`, 0.226979`, 16.5242`, 
              16.524464`, \(-0.000694`\), 11.590726`, 10.137398`, 3.821454`, 
              10.133494`, 9.65638`, 1.254549`, 9.658139`, 6.4038`, 8.557126`, 
              6.343799`, 6.342542`, 0.003305`, 6.089619`, 4.208053`, 
              4.947484`, 4.191834`, 3.978112`, 0.561971`, 3.982247`, 
              3.794163`, 0.494557`, 16.5242`, 9.658139`, 10.133494`, 
              4.497805`, 3.982247`, 6.089619`, 3.325548`, \(-1.072233`\), 
              6.343799`, 4.191834`, 2.140641`, 1.676281`, 7.750859`, 
              11.590726`, 4.034505`, \(-0.936062`\)}, {5, 28, 
              570104.9787`, \(-0.25`\), \(-2.199956218`\), 0.0225`, 
              1, \(-0.3`\), \(-0.06069021902`\), 
              7.560081897`, \(-1.221693953`\), \(-1.239237779`\), 3, 4, 5, 6, 
              0.218062`, \(-2.402239`\), 3.463951`, 0.110473`, 0.002348`, 
              0.19065`, 0.631276`, 0.30142`, 0.637608`, 0.371477`, 0.878167`, 
              0.603603`, 16.5242`, 16.524128`, 0.000556`, 8.919176`, 
              8.919054`, 0.000941`, 5.041258`, 5.041071`, 0.001433`, 
              2.519662`, \(-2.216881`\), 
              36.355973`, \(-4.493151`\), \(-4.48997`\), \(-0.024415`\), \
\(-5.364909`\), \(-6.184869`\), 6.293719`, \(-6.164676`\), \(-7.386813`\), 
              9.380674`, \(-7.411078`\), \(-9.401934`\), 15.281082`, 
              16.5242`, 5.041258`, 
              8.919176`, \(-6.842628`\), \(-9.397649`\), \(-5.364909`\), \
\(-16.310417`\), \(-16.773246`\), \(-6.164676`\), \(-4.493151`\), \
\(-7.411078`\), \(-12.575442`\), \(-0.780438`\), 
              2.519662`, \(-7.075791`\), \(-11.988894`\)}, {5, 26, 
              944183.2755`, \(-0.4947264965`\), 30.10403542`, 0.0225`, 
              1, \(-0.2`\), \(-0.09404698391`\), 
              3332.019908`, \(-1.95276879`\), \(-1.50420985`\), 3, 4, 5, 6, 
              0.269503`, \(-0.672533`\), 0.750624`, 0.033751`, 
              0.224492`, \(-0.136328`\), 0.706809`, 0.743228`, 0.672617`, 
              0.750044`, 0.894886`, 0.769981`, 16.5242`, 16.592108`, 
              8.815077`, 78.537747`, 78.480397`, \(-7.444617`\), 142.451241`, 
              154.200545`, 1525.17666`, 154.129188`, 156.414684`, 
              296.680067`, 115.239436`, 115.354027`, 14.875087`, 190.006062`, 
              190.109663`, 13.448448`, 251.98733`, 252.124183`, 17.764889`, 
              252.746046`, 252.433394`, \(-40.585341`\), 16.5242`, 
              78.537747`, 142.451241`, 154.129188`, 251.98733`, 320.855109`, 
              354.215399`, 416.499068`, 115.239436`, 190.006062`, 
              252.746046`, 281.156318`, 174.897192`, 222.721345`, 
              295.159638`, 298.792217`}, {5, 28, 
              9.803800087`, \(-1.378962756`\), 5.769625578`, 0.0225`, 
              1, \(-0.1`\), \(-0.1374650116`\), 
              3.840069026`, \(-0.6908374617`\), \(-0.9158362355`\), 3, 4, 5, 
              6, 0.296401`, \(-0.462087`\), 0.243712`, 0.172608`, 
              0.154305`, \(-0.157689`\), 0.732804`, 0.64629`, 0.814507`, 
              0.807972`, 1.129081`, 1.146816`, 16.5242`, 16.563634`, 
              1.079003`, 24.826164`, 24.8412`, 0.411433`, 26.76692`, 
              26.739306`, \(-0.755567`\), 43.492334`, 43.530539`, 1.045361`, 
              22.957195`, 22.97618`, 0.519465`, 44.886804`, 44.910653`, 
              0.652554`, 55.486668`, 55.481131`, \(-0.151498`\), 56.568498`, 
              56.587741`, 0.526538`, 16.5242`, 26.76692`, 24.826164`, 
              43.492334`, 56.568498`, 70.632659`, 87.78743`, 90.564016`, 
              22.957195`, 44.886804`, 55.486668`, 59.625284`, 44.191811`, 
              65.417422`, 68.72705`, 72.634275`}, {5, 28, 
              27.16521891`, \(-2.072325474`\), 4.649920371`, 0.0225`, 1, 
              0, \(-0.165458343`\), 
              3.522405817`, \(-0.7277358416`\), \(-1.135104535`\), 3, 4, 5, 
              6, 0.309317`, \(-0.445787`\), 0.180281`, 0.174801`, 
              0.147335`, \(-0.151735`\), 0.739347`, 0.540718`, 0.783097`, 
              0.733901`, 1.072717`, 1.104374`, 16.5242`, 16.559082`, 
              0.80273`, 21.52496`, 21.539923`, 0.344334`, 26.965201`, 
              26.940928`, \(-0.558584`\), 36.265002`, 36.291409`, 0.607686`, 
              18.497012`, 18.510811`, 0.317552`, 39.124986`, 39.145117`, 
              0.463273`, 45.090488`, 45.096519`, 0.1388`, 48.520932`, 
              48.52904`, 0.186592`, 16.5242`, 21.52496`, 26.965201`, 
              36.265002`, 48.520932`, 60.036392`, 75.735931`, 79.708251`, 
              18.497012`, 39.124986`, 45.090488`, 51.923364`, 39.935791`, 
              59.651235`, 61.77331`, 63.486178`}, {5, 28, 
              56.40087264`, \(-2.41444683`\), 3.957489271`, 0.0225`, 1, 
              0.1`, \(-0.190566477`\), 
              5.069882185`, \(-0.6293168384`\), \(-1.088005298`\), 3, 4, 5, 
              6, 0.306311`, \(-0.464429`\), 0.216821`, 0.186116`, 
              0.123365`, \(-0.128958`\), 0.733573`, 0.472136`, 0.785405`, 
              0.703171`, 1.08072`, 1.115693`, 16.5242`, 16.55226`, 0.544234`, 
              19.903276`, 19.915954`, 0.245903`, 25.578291`, 
              25.558046`, \(-0.392665`\), 34.04522`, 34.070313`, 0.486684`, 
              15.365613`, 15.376807`, 0.217123`, 33.964221`, 33.981716`, 
              0.339316`, 38.221933`, 38.227201`, 0.102168`, 42.026162`, 
              42.032486`, 0.12266`, 16.5242`, 19.903276`, 25.578291`, 
              34.04522`, 42.026162`, 51.803148`, 65.952546`, 68.689477`, 
              15.365613`, 33.964221`, 38.221933`, 45.480463`, 35.664045`, 
              52.459541`, 54.053371`, 59.408657`}, {5, 28, 
              54.99684063`, \(-1.993088619`\), 4.063255887`, 0.0225`, 1, 
              0.2`, \(-0.2209433802`\), 
              4.587346919`, \(-0.5169541154`\), \(-1.039281334`\), 3, 4, 5, 
              6, 0.289277`, \(-0.498096`\), 0.297912`, 0.195912`, 
              0.100693`, \(-0.105421`\), 0.718588`, 0.4806`, 0.776683`, 
              0.719478`, 1.074518`, 1.150414`, 16.5242`, 16.551577`, 
              0.51486`, 20.490828`, 20.508506`, 0.332452`, 27.663948`, 
              27.643808`, \(-0.378775`\), 34.114462`, 34.141177`, 0.502403`, 
              15.093674`, 15.105289`, 0.218436`, 34.277755`, 34.296362`, 
              0.349941`, 38.727512`, 38.733045`, 0.104059`, 42.692536`, 
              42.698513`, 0.1124`, 16.5242`, 20.490828`, 27.663948`, 
              34.114462`, 42.692536`, 52.564585`, 67.284612`, 70.140397`, 
              15.093674`, 34.277755`, 38.727512`, 46.786161`, 36.961872`, 
              54.116391`, 55.512962`, 61.977286`}, {5, 28, 
              78.0546061`, \(-1.627717723`\), 3.740342973`, 0.0225`, 1, 
              0.3`, \(-0.2502634828`\), 
              7.500879041`, \(-0.3912087698`\), \(-0.995026099`\), 3, 4, 5, 
              6, 0.264935`, \(-0.546778`\), 0.421015`, 0.204263`, 
              0.080543`, \(-0.083898`\), 0.697225`, 0.430217`, 0.759524`, 
              0.698484`, 1.057448`, 1.159293`, 16.5242`, 16.548449`, 
              0.384474`, 18.697166`, 18.712307`, 0.240058`, 25.971443`, 
              25.953357`, \(-0.286756`\), 33.608489`, 33.611439`, 0.046766`, 
              13.090099`, 13.100982`, 0.172557`, 30.388668`, 30.406168`, 
              0.277466`, 34.967041`, 34.972197`, 0.081755`, 38.242807`, 
              38.247762`, 0.078559`, 16.5242`, 18.697166`, 25.971443`, 
              34.945832`, 38.242807`, 47.102119`, 60.851176`, 63.29437`, 
              13.090099`, 30.388668`, 34.967041`, 42.472006`, 33.608489`, 
              49.222622`, 50.485512`, 56.352691`}, {5, 28, 
              83.55621262`, \(-1.366936963`\), 3.768151465`, 0.0225`, 1, 
              0.4`, \(-0.2803182981`\), 
              9.325875098`, \(-0.3016203797`\), \(-0.9498993896`\), 3, 4, 5, 
              6, 0.235001`, \(-0.614632`\), 0.591549`, 0.209147`, 
              0.067605`, \(-0.069475`\), 0.670496`, 0.411616`, 0.736336`, 
              0.692737`, 1.033342`, 1.167889`, 16.5242`, 16.548154`, 
              0.339384`, 18.030223`, 18.046596`, 0.231977`, 26.13718`, 
              26.119168`, \(-0.255203`\), 32.897061`, 32.900174`, 0.044109`, 
              12.228208`, 12.239468`, 0.15954`, 28.979653`, 28.997592`, 
              0.254175`, 34.371796`, 34.377106`, 0.075235`, 36.991249`, 
              36.995717`, 0.063308`, 16.5242`, 18.030223`, 26.13718`, 
              35.962503`, 36.991249`, 45.640901`, 59.710062`, 61.910738`, 
              12.228208`, 28.979653`, 34.371796`, 41.690081`, 32.897061`, 
              48.370756`, 49.821155`, 55.438705`}, {5, 28, 
              1.59996609`*^7, \(-0.25`\), \(-0.4131812715`\), 0.04`, 
              1, \(-0.6`\), \(-0.02623739138`\), 
              5.163188799`, \(-1.554686159`\), \(-1.219873991`\), 3, 4, 5, 6, 
              0.24173`, \(-19.117057`\), 16.490232`, 
              0.139143`, \(-1.79781`\), 1.689543`, \(-0.812164`\), 
              6.547389`, \(-0.670668`\), \(-0.775208`\), \(-0.407651`\), \
\(-0.763889`\), 16.5242`, 16.52422`, \(-0.000033`\), 13.774364`, 
              13.774369`, \(-8.`*^-6\), 11.856236`, 
              12.402877`, \(-0.873563`\), 10.916108`, 10.385805`, 0.847453`, 
              11.22716`, 11.227477`, \(-0.000507`\), 10.295542`, 9.261071`, 
              1.653142`, 9.259663`, 8.313339`, 1.512276`, 8.314707`, 
              8.305445`, 0.014801`, 16.5242`, 13.774364`, 10.916108`, 
              10.3858`, 7.962641`, 10.295542`, 7.414562`, 7.144649`, 
              11.22716`, 9.259663`, 8.314707`, 6.423198`, 9.603627`, 
              11.856236`, 8.399648`, 7.29342`}, {5, 28, 
              6.542829808`*^6, \(-0.25`\), \(-0.6460723266`\), 0.04`, 
              1, \(-0.5`\), \(-0.04381444748`\), 
              5.925610169`, \(-1.431352861`\), \(-1.226843585`\), 3, 4, 5, 6, 
              0.220985`, \(-10.494199`\), 9.631354`, 
              0.130543`, \(-0.820561`\), 1.266146`, 0.222142`, 3.387985`, 
              0.296976`, 0.010959`, 0.577627`, 0.086122`, 16.5242`, 
              16.524264`, \(-0.000146`\), 11.271005`, 
              11.271974`, \(-0.002215`\), 11.197453`, 10.741804`, 1.040865`, 
              10.742729`, 8.538518`, 5.035198`, 8.003021`, 
              8.00337`, \(-0.000797`\), 7.195603`, 5.88517`, 2.993492`, 
              5.878438`, 5.096153`, 1.787017`, 5.279582`, 4.218453`, 
              2.423995`, 16.5242`, 10.742729`, 11.271005`, 6.033952`, 
              5.096752`, 7.195603`, 5.279582`, 2.089231`, 8.003021`, 
              5.878438`, 3.65313`, 2.473498`, 8.130962`, 11.197453`, 
              4.65489`, 2.849961`}, {5, 28, 
              1.695337519`*^6, \(-0.25`\), \(-1.27003622`\), 0.04`, 
              1, \(-0.4`\), \(-0.06293681143`\), 
              2.322302552`, \(-1.278538332`\), \(-1.272556567`\), 3, 4, 5, 6, 
              0.22236`, \(-4.320037`\), 5.127857`, 0.123773`, \(-0.172698`\), 
              0.455385`, 0.588168`, 0.661814`, 0.602165`, 0.322632`, 
              0.851035`, 0.541022`, 16.5242`, 16.524552`, \(-0.001589`\), 
              9.679944`, 7.346697`, 10.542759`, 7.347048`, 7.342292`, 
              0.02149`, 7.339154`, 3.087799`, 19.209712`, 1.02264`, 
              0.336282`, 3.101302`, 0.324582`, \(-0.386536`\), 
              3.213178`, \(-0.390988`\), \(-0.508285`\), 
              0.530004`, \(-0.492975`\), \(-1.734673`\), 5.6106`, 16.5242`, 
              7.347048`, 7.339154`, 1.898927`, \(-1.733164`\), 
              1.02264`, \(-1.34658`\), \(-9.312543`\), \(-0.492975`\), \
\(-0.390988`\), 0.324582`, \(-3.090816`\), 4.024231`, 
              9.679944`, \(-0.447811`\), \(-6.922243`\)}, {5, 27, 
              121377.2592`, \(-0.25`\), \(-4.767856149`\), 0.04`, 
              1, \(-0.3`\), 0.04968643868`, 
              499.9594513`, \(-2.924763191`\), \(-4.363581163`\), 3, 4, 5, 6, 
              0.356422`, \(-0.832652`\), 1.072352`, \(-0.051886`\), 
              0.166993`, \(-0.071262`\), 0.708817`, \(-0.005735`\), 
              0.129323`, \(-0.048386`\), \(-0.09876`\), \(-0.14392`\), 
              16.5242`, 
              16.524506`, \(-0.008329`\), \(-9.704324`\), \(-9.705258`\), 
              0.025402`, \(-11.906148`\), \(-12.576341`\), 
              18.222456`, \(-12.577436`\), \(-13.207885`\), 
              17.141839`, \(-19.107336`\), \(-19.106554`\), \(-0.02128`\), \
\(-27.941004`\), \(-27.945465`\), 
              0.121296`, \(-31.579038`\), \(-31.557054`\), \(-0.597738`\), \
\(-37.479118`\), \(-37.540181`\), 1.660284`, 
              16.5242`, \(-9.704324`\), \(-11.906148`\), \(-12.589361`\), \
\(-37.479118`\), \(-42.16321`\), \(-45.78183`\), \(-59.545316`\), \
\(-19.107336`\), \(-27.941004`\), \(-31.579038`\), \(-43.272103`\), \
\(-21.932546`\), \(-12.577436`\), \(-41.237215`\), \(-46.447268`\)}, {5, 27, 
              11.31459032`, \(-0.875204578`\), 7.692490482`, 0.04`, 
              1, \(-0.2`\), \(-0.1153021249`\), 
              2581.379746`, \(-1.26665241`\), \(-1.300997665`\), 3, 4, 5, 6, 
              0.323698`, \(-0.529532`\), 0.395474`, 0.131398`, 
              0.181307`, \(-0.150696`\), 0.754013`, 0.721949`, 0.776513`, 
              0.813692`, 1.053082`, 1.031752`, 16.5242`, 16.549416`, 
              1.004606`, 30.314597`, 30.293553`, \(-0.838385`\), 43.795108`, 
              43.811721`, 0.661893`, 56.07004`, 56.073108`, 0.122254`, 
              32.164722`, 32.18798`, 0.926606`, 59.273049`, 59.300485`, 
              1.093084`, 74.684384`, 74.656583`, \(-1.107607`\), 75.009329`, 
              74.996116`, \(-0.526411`\), 16.5242`, 30.314597`, 43.795108`, 
              58.000181`, 75.009329`, 93.910836`, 113.294416`, 120.394469`, 
              32.164722`, 59.273049`, 74.684384`, 76.73459`, 56.07004`, 
              74.650659`, 93.237012`, 95.939183`}, {5, 25, 
              59.60335535`, \(-1.61836058`\), 4.093841878`, 0.04`, 
              1, \(-0.1`\), \(-0.1453169335`\), 
              12.29618996`, \(-0.8847007707`\), \(-1.226969957`\), 3, 4, 5, 
              6, 0.351292`, \(-0.443681`\), 0.162586`, 0.171426`, 
              0.146986`, \(-0.147385`\), 0.773518`, 0.502196`, 0.8143`, 
              0.701912`, 1.109039`, 1.073195`, 16.5242`, 16.548755`, 
              0.565005`, 22.648094`, 22.680798`, 0.752521`, 23.012181`, 
              22.967881`, \(-1.019341`\), 35.857968`, 35.85982`, 0.04262`, 
              17.585617`, 17.596385`, 0.247785`, 36.262097`, 36.277525`, 
              0.355`, 41.235602`, 41.238761`, 0.072691`, 44.586758`, 
              44.592339`, 0.128428`, 16.5242`, 22.648094`, 23.012181`, 
              40.670408`, 44.586758`, 54.871834`, 69.143424`, 71.094897`, 
              17.585617`, 36.262097`, 41.235602`, 46.45281`, 35.857968`, 
              52.461238`, 55.25475`, 56.823202`}, {5, 28, 
              46.09400111`, \(-2.097528049`\), 3.975843204`, 0.04`, 1, 
              0, \(-0.190665725`\), 
              4.68656106`, \(-0.6695488423`\), \(-1.229229538`\), 3, 4, 5, 6, 
              0.352278`, \(-0.443547`\), 0.132639`, 0.190301`, 
              0.114569`, \(-0.120534`\), 0.776582`, 0.512561`, 0.818199`, 
              0.733729`, 1.114877`, 1.144923`, 16.5242`, 16.553066`, 
              0.64688`, 21.091874`, 21.105276`, 0.300333`, 26.179603`, 
              26.157557`, \(-0.49403`\), 34.178254`, 34.200533`, 0.49928`, 
              16.943145`, 16.954057`, 0.244542`, 36.705746`, 36.722844`, 
              0.383157`, 40.180672`, 40.185542`, 0.109138`, 45.072846`, 
              45.077947`, 0.114307`, 16.5242`, 21.091874`, 26.179603`, 
              34.178254`, 45.072846`, 55.107646`, 69.497162`, 72.10815`, 
              16.943145`, 36.705746`, 40.180672`, 47.898311`, 37.59716`, 
              54.740846`, 56.952436`, 60.938086`}, {5, 28, 
              47.75713647`, \(-2.37897164`\), 3.925894276`, 0.04`, 1, 
              0.1`, \(-0.2277890361`\), 
              4.460423527`, \(-0.5423616692`\), \(-1.205672775`\), 3, 4, 5, 
              6, 0.344625`, \(-0.462423`\), 0.164257`, 0.200424`, 
              0.092656`, \(-0.098553`\), 0.770611`, 0.511808`, 0.815406`, 
              0.74516`, 1.113871`, 1.178208`, 16.5242`, 16.55228`, 0.607848`, 
              21.71599`, 21.734162`, 0.393379`, 28.303484`, 
              28.281347`, \(-0.479219`\), 33.702378`, 33.726352`, 0.518959`, 
              16.342886`, 16.353883`, 0.238065`, 36.524683`, 36.54269`, 
              0.3898`, 39.483528`, 39.488714`, 0.112267`, 45.010722`, 
              45.014912`, 0.090717`, 16.5242`, 21.71599`, 28.303484`, 
              33.702378`, 45.010722`, 54.837182`, 69.224738`, 72.053801`, 
              16.342886`, 36.524683`, 39.483528`, 48.460189`, 38.463204`, 
              55.402433`, 57.255458`, 63.289259`}, {5, 28, 
              88.24952787`, \(-2.096924316`\), 3.169752643`, 0.04`, 1, 
              0.2`, \(-0.264525686`\), 
              36.95386467`, \(-0.4141908428`\), \(-1.196063514`\), 3, 4, 5, 
              6, 0.327487`, \(-0.491533`\), 0.229898`, 0.208349`, 
              0.07411`, \(-0.078854`\), 0.757473`, 0.408571`, 0.801238`, 
              0.692119`, 1.097054`, 1.167468`, 16.5242`, 16.549041`, 
              0.412577`, 18.466928`, 18.475008`, 0.134204`, 23.965974`, 
              23.947703`, \(-0.303457`\), 31.29658`, 31.2989`, 0.038533`, 
              12.705488`, 12.714657`, 0.152294`, 29.00246`, 29.01776`, 
              0.254113`, 31.484174`, 31.488497`, 0.0718`, 36.005208`, 
              36.007808`, 0.043196`, 16.5242`, 18.466928`, 23.965974`, 
              35.057881`, 36.005208`, 43.790599`, 55.553232`, 57.687391`, 
              12.705488`, 29.00246`, 31.484174`, 39.231323`, 31.29658`, 
              44.904078`, 46.199825`, 51.331174`}, {5, 28, 
              89.90412399`, \(-1.749888676`\), 3.502424011`, 0.04`, 1, 
              0.3`, \(-0.3084807836`\), 
              8.291915819`, \(-0.3124954719`\), \(-1.163489252`\), 3, 4, 5, 
              6, 0.300736`, \(-0.535001`\), 0.332156`, 0.213315`, 
              0.061601`, \(-0.065068`\), 0.737786`, 0.43744`, 0.783187`, 
              0.713467`, 1.077259`, 1.192016`, 16.5242`, 16.5484`, 0.407839`, 
              19.267545`, 19.285651`, 0.30514`, 26.910901`, 
              26.891235`, \(-0.331438`\), 34.088432`, 34.263267`, 2.946477`, 
              13.307719`, 13.318516`, 0.18196`, 30.837277`, 30.855344`, 
              0.304492`, 34.095657`, 34.100783`, 0.086376`, 38.735478`, 
              38.737261`, 0.030044`, 16.5242`, 19.267545`, 26.910901`, 
              34.423915`, 38.735478`, 47.11021`, 60.278317`, 62.369257`, 
              13.307719`, 30.837277`, 34.095657`, 42.764963`, 34.088432`, 
              48.942361`, 50.380317`, 55.924733`}, {5, 29, 
              106.7966592`, \(-1.494117079`\), 3.317197429`, 0.04`, 1, 
              0.4`, \(-0.2767195172`\), 
              14.14294561`, \(-0.2873290501`\), \(-1.175004257`\), 3, 4, 5, 
              6, 0.293296`, \(-0.560156`\), 0.427171`, 0.21506`, 
              0.056152`, \(-0.058695`\), 0.721485`, 0.409832`, 0.761641`, 
              0.699154`, 1.049809`, 1.187704`, 16.5242`, 16.539838`, 
              0.243428`, 18.837371`, 18.852803`, 0.240229`, 25.673275`, 
              25.657898`, \(-0.239364`\), 32.357692`, 32.359758`, 0.032153`, 
              12.096469`, 12.104881`, 0.130944`, 28.969748`, 28.98463`, 
              0.231663`, 31.888343`, 31.890569`, 0.03464`, 36.227684`, 
              36.23016`, 0.038551`, 16.5242`, 18.837371`, 25.673275`, 
              35.035926`, 36.227684`, 43.931614`, 56.617347`, 58.54438`, 
              12.096469`, 28.969748`, 31.888343`, 40.156314`, 32.357692`, 
              46.393112`, 47.368228`, 52.747747`}, {5, 27, 
              8.705607198`*^6, \(-0.25`\), \(-0.5601120756`\), 0.0625`, 
              1, \(-0.6`\), \(-0.00125911188`\), \(-2.629553377`\), \
\(-1.546756495`\), \(-1.296269314`\), 3, 4, 5, 6, 0.285779`, \(-11.81754`\), 
              9.819142`, 0.148689`, \(-1.077849`\), 1.543799`, 0.064353`, 
              4.122524`, 0.170427`, \(-0.090333`\), 
              0.485444`, \(-0.024866`\), 16.5242`, 
              16.524236`, \(-0.000092`\), 15.534928`, 15.534832`, 0.000246`, 
              10.65915`, 9.180903`, 3.78593`, 9.180914`, 8.448371`, 
              1.876114`, 9.181783`, 9.182163`, \(-0.000974`\), 8.268209`, 
              6.060742`, 5.653532`, 6.058642`, 5.491304`, 1.453006`, 
              5.492307`, 5.412637`, 0.204041`, 16.5242`, 15.534928`, 
              9.180914`, 3.056242`, 5.492307`, 8.268209`, 5.155607`, 
              4.898346`, 9.181783`, 6.058642`, 5.415519`, 2.767509`, 
              8.448371`, 10.65915`, 6.001443`, 5.262791`}, {5, 28, 
              3.323729121`*^6, \(-0.25`\), \(-0.9064524112`\), 0.0625`, 
              1, \(-0.5`\), \(-0.05651862575`\), 
              6.471515026`, \(-1.331787809`\), \(-1.220026888`\), 3, 4, 5, 6, 
              0.242969`, \(-6.013817`\), 6.205688`, 
              0.138365`, \(-0.347909`\), 0.713877`, 0.535167`, 1.311642`, 
              0.575126`, 0.28458`, 0.85129`, 0.48888`, 16.5242`, 
              16.524316`, \(-0.00041`\), 10.887233`, 
              10.887505`, \(-0.000958`\), 9.154165`, 7.83575`, 4.645876`, 
              7.835916`, 5.456274`, 8.385461`, 2.865517`, 2.860704`, 
              0.016961`, 2.832215`, 2.063623`, 2.708388`, 2.830355`, 
              1.226878`, 5.650388`, 2.063105`, 0.985972`, 3.795638`, 
              16.5242`, 7.835916`, 10.887233`, 1.202302`, 0.987454`, 
              2.832215`, 2.830355`, \(-3.278016`\), 2.865517`, 1.222729`, 
              2.063105`, 0.134804`, 5.456775`, 9.154165`, 
              0.816039`, \(-0.781059`\)}, {5, 28, 
              658737.2606`, \(-0.25`\), \(-2.066804942`\), 0.0625`, 
              1, \(-0.4`\), \(-0.07200974867`\), 
              24.7617606`, \(-1.222620619`\), \(-1.329224916`\), 3, 4, 5, 6, 
              0.279974`, \(-2.242504`\), 3.263289`, 
              0.142417`, \(-0.006997`\), 0.166573`, 0.696671`, 0.301068`, 
              0.715812`, 0.442666`, 0.986117`, 0.754469`, 16.5242`, 
              16.524066`, 0.00124`, 4.947631`, 4.951181`, \(-0.032867`\), 
              3.744477`, 3.504169`, 2.224867`, 3.504277`, \(-3.957555`\), 
              69.084742`, \(-5.990509`\), \(-5.993396`\), 
              0.02673`, \(-7.059394`\), \(-7.071211`\), 
              0.109407`, \(-7.458334`\), \(-8.464068`\), 
              9.311505`, \(-8.480404`\), \(-11.222724`\), 25.389539`, 
              16.5242`, 3.504277`, 
              4.947631`, \(-6.862436`\), \(-11.216044`\), \(-11.894133`\), \
\(-7.458334`\), \(-25.593089`\), \(-7.059394`\), \(-5.990509`\), \(-8.480404`\
\), \(-15.019211`\), \(-3.235897`\), 
              3.744477`, \(-11.453634`\), \(-15.861577`\)}, {5, 28, 
              157657.2688`, \(-0.2692085322`\), \(-21.561416`\), 0.0625`, 
              1, \(-0.3`\), \(-0.1258853844`\), 
              158.8719801`, \(-2.859439477`\), \(-5.092110954`\), 3, 4, 5, 6, 
              0.316243`, \(-0.930798`\), 1.196232`, \(-0.023056`\), 
              0.186417`, \(-0.099298`\), 0.763343`, \(-0.101335`\), 
              0.079298`, \(-0.107047`\), \(-0.224867`\), \(-0.123553`\), 
              16.5242`, 16.479334`, 
              4.8948`, \(-51.279577`\), \(-51.240598`\), \(-4.252535`\), \
\(-87.921171`\), \(-94.422052`\), 
              709.234578`, \(-116.287733`\), \(-116.699858`\), 
              44.962106`, \(-90.839106`\), \(-90.920952`\), 
              8.929268`, \(-142.178777`\), \(-142.269315`\), 
              9.877523`, \(-187.130564`\), \(-186.970618`\), \(-17.449814`\), \
\(-191.247744`\), \(-192.413984`\), 127.23477`, 
              16.5242`, \(-51.279577`\), \(-87.921171`\), \(-116.658`\), \
\(-191.247744`\), \(-244.717618`\), \(-216.966033`\), \(-329.900727`\), \
\(-90.839106`\), \(-142.178777`\), \(-187.130564`\), \(-208.906372`\), \
\(-128.607248`\), \(-116.287733`\), \(-231.46168`\), \(-245.662124`\)}, {5, 
              28, 11.98900331`, \(-1.060731083`\), 5.556669013`, 0.0625`, 
              1, \(-0.2`\), \(-0.1596851206`\), 
              6.071608524`, \(-0.7760231552`\), \(-1.022611921`\), 3, 4, 5, 
              6, 0.365536`, \(-0.476488`\), 0.218845`, 0.186704`, 
              0.115755`, \(-0.11585`\), 0.796375`, 0.708509`, 0.881962`, 
              0.869942`, 1.214893`, 1.221194`, 16.5242`, 16.556943`, 
              1.064108`, 28.108738`, 28.102517`, \(-0.202171`\), 28.858381`, 
              28.852034`, \(-0.206269`\), 46.951518`, 46.954372`, 0.092741`, 
              24.92032`, 24.93673`, 0.533311`, 48.563132`, 48.585644`, 
              0.731606`, 57.092902`, 57.094135`, 0.040069`, 60.466262`, 
              60.474977`, 0.28324`, 16.5242`, 28.108738`, 28.858381`, 
              47.373201`, 60.466262`, 74.52726`, 87.652566`, 100.068214`, 
              24.92032`, 48.563132`, 57.092902`, 61.570729`, 46.951518`, 
              66.826286`, 72.67029`, 78.272178`}, {5, 29, 
              71.56251423`, \(-1.703639201`\), 3.544007008`, 0.0625`, 
              1, \(-0.1`\), \(-0.1756042946`\), 
              9.810403101`, \(-0.8154106919`\), \(-1.307314953`\), 3, 4, 5, 
              6, 0.397339`, \(-0.430311`\), 0.085176`, 0.187809`, 
              0.115133`, \(-0.118859`\), 0.812028`, 0.481485`, 0.853584`, 
              0.709348`, 1.158027`, 1.12194`, 16.5242`, 16.548045`, 
              0.537256`, 20.832447`, 20.838563`, 0.13781`, 23.036462`, 
              23.018068`, \(-0.414432`\), 34.57271`, 34.574281`, 0.035401`, 
              16.265834`, 16.274544`, 0.196254`, 34.635632`, 34.649484`, 
              0.312087`, 37.203148`, 37.206366`, 0.072488`, 42.170119`, 
              42.173774`, 0.082349`, 16.5242`, 23.036462`, 20.832447`, 
              36.27824`, 42.170119`, 51.256129`, 64.249271`, 65.856772`, 
              16.265834`, 34.635632`, 37.203148`, 43.79013`, 34.57271`, 
              49.837312`, 52.137334`, 55.302203`}, {5, 28, 
              46.55711748`, \(-2.036725846`\), 3.771846117`, 0.0625`, 1, 
              0, \(-0.2302028113`\), 
              4.7590526`, \(-0.5634932456`\), \(-1.335203324`\), 3, 4, 5, 6, 
              0.393063`, \(-0.4375`\), 0.069815`, 0.204961`, 
              0.083149`, \(-0.089073`\), 0.813115`, 0.530163`, 0.850517`, 
              0.762581`, 1.152635`, 1.200683`, 16.5242`, 16.55199`, 
              0.659204`, 22.174755`, 22.191833`, 0.405107`, 28.021939`, 
              27.998681`, \(-0.551715`\), 33.532837`, 33.554442`, 0.512483`, 
              17.140235`, 17.150381`, 0.240661`, 37.710739`, 37.727345`, 
              0.393919`, 39.704991`, 39.710008`, 0.119005`, 46.171933`, 
              46.174348`, 0.057284`, 16.5242`, 22.174755`, 28.021939`, 
              33.532837`, 46.171933`, 55.824163`, 69.83498`, 72.55652`, 
              17.140235`, 37.710739`, 39.704991`, 48.896072`, 38.925278`, 
              55.440543`, 57.848355`, 62.657832`}, {5, 28, 
              53.3412934`, \(-2.270397081`\), 3.623428077`, 0.0625`, 1, 
              0.1`, \(-0.2757930594`\), 
              5.14459971`, \(-0.4336662885`\), \(-1.337433187`\), 3, 4, 5, 6, 
              0.382808`, \(-0.456434`\), 0.09701`, 0.212273`, 
              0.066854`, \(-0.072063`\), 0.807172`, 0.510597`, 0.842978`, 
              0.758199`, 1.143263`, 1.216871`, 16.5242`, 16.551233`, 
              0.599948`, 22.108357`, 22.12841`, 0.445048`, 29.013792`, 
              28.991663`, \(-0.491113`\), 33.06778`, 33.08973`, 0.487134`, 
              16.114176`, 16.124437`, 0.227724`, 36.227317`, 36.244097`, 
              0.372407`, 37.991852`, 37.99765`, 0.128681`, 44.68895`, 
              44.689837`, 0.019701`, 16.5242`, 22.108357`, 29.013792`, 
              33.06778`, 44.68895`, 53.850747`, 67.468917`, 69.912516`, 
              16.114176`, 36.227317`, 37.991852`, 47.957001`, 38.425538`, 
              54.298606`, 56.289784`, 62.037743`}, {5, 28, 
              63.55335858`, \(-2.155487269`\), 3.504606`, 0.0625`, 1, 
              0.2`, \(-0.3129415699`\), 
              5.819113098`, \(-0.3418013181`\), \(-1.339007735`\), 3, 4, 5, 
              6, 0.367798`, \(-0.480934`\), 0.149911`, 0.216547`, 
              0.056504`, \(-0.060784`\), 0.796083`, 0.488158`, 0.829097`, 
              0.748552`, 1.125733`, 1.221815`, 16.5242`, 16.549195`, 
              0.515496`, 21.70945`, 21.731132`, 0.447168`, 29.1568`, 
              29.137963`, \(-0.388492`\), 33.095832`, 33.115927`, 0.414435`, 
              15.111158`, 15.121339`, 0.209964`, 34.592535`, 34.60973`, 
              0.354625`, 36.357087`, 36.362451`, 0.110622`, 42.955`, 
              42.954743`, \(-0.005295`\), 16.5242`, 21.70945`, 29.1568`, 
              33.095832`, 42.955`, 51.641107`, 64.971714`, 67.134545`, 
              15.111158`, 34.592535`, 36.357087`, 46.584393`, 37.495659`, 
              52.774755`, 54.461968`, 60.140892`}, {5, 28, 
              71.28152917`, \(-1.862341825`\), 3.453500851`, 0.0625`, 1, 
              0.3`, \(-0.350171906`\), 
              6.581582137`, \(-0.2405117826`\), \(-1.343550581`\), 3, 4, 5, 
              6, 0.34602`, \(-0.512753`\), 0.225882`, 0.220085`, 
              0.047811`, \(-0.051305`\), 0.780044`, 0.469651`, 0.808538`, 
              0.740847`, 1.099396`, 1.226088`, 16.5242`, 16.547806`, 
              0.451346`, 21.196441`, 21.219803`, 0.446667`, 29.177961`, 
              29.161327`, \(-0.318035`\), 33.571242`, 33.590274`, 0.363893`, 
              14.281357`, 14.291713`, 0.198015`, 33.235237`, 33.253947`, 
              0.357723`, 35.167069`, 35.171219`, 0.079356`, 41.530741`, 
              41.52942`, \(-0.025242`\), 16.5242`, 21.196441`, 29.177961`, 
              33.571242`, 41.530741`, 49.864833`, 63.163374`, 65.045602`, 
              14.281357`, 33.235237`, 35.167069`, 45.509665`, 36.67882`, 
              51.594068`, 53.148858`, 58.694871`}, {5, 29, 
              117.2942917`, \(-1.616290407`\), 3.054681347`, 0.0625`, 1, 
              0.4`, \(-0.3155062771`\), 
              18.54402728`, \(-0.2310253818`\), \(-1.378141506`\), 3, 4, 5, 
              6, 0.342375`, \(-0.525462`\), 0.293591`, 0.220811`, 
              0.045282`, \(-0.048291`\), 0.767167`, 0.412937`, 0.78681`, 
              0.70817`, 1.069404`, 1.208664`, 16.5242`, 16.537142`, 
              0.216572`, 19.600597`, 19.61751`, 0.283013`, 26.287325`, 
              26.271688`, \(-0.261661`\), 32.673762`, 32.675963`, 0.036841`, 
              12.245107`, 12.252706`, 0.127155`, 29.380159`, 29.395398`, 
              0.255003`, 30.768338`, 30.769033`, 0.01163`, 36.493175`, 
              36.493496`, 0.005378`, 16.5242`, 19.600597`, 26.287325`, 
              34.303318`, 36.493175`, 43.652772`, 55.600091`, 57.243493`, 
              12.245107`, 29.380159`, 30.768338`, 40.096493`, 32.673762`, 
              45.859518`, 46.809448`, 51.865419`}, {5, 27, 
              4.546208295`*^6, \(-0.25`\), \(-0.7750389164`\), 0.09`, 
              1, \(-0.6`\), 
              0.002150254081`, \(-1.358833654`\), \(-1.254887627`\), \
\(-1.249771332`\), 3, 4, 5, 6, 0.303942`, \(-6.464175`\), 5.854875`, 
              0.154666`, \(-0.422675`\), 0.846177`, 0.488174`, 1.623762`, 
              0.542707`, 0.303231`, 0.837729`, 0.522096`, 16.5242`, 
              16.524226`, \(-0.000097`\), 10.484339`, 10.484257`, 0.000307`, 
              8.686371`, 8.686351`, 0.000078`, 8.36718`, 6.072627`, 
              8.648344`, 4.812495`, 4.812698`, \(-0.000768`\), 3.250352`, 
              2.817526`, 1.631353`, 3.231887`, 1.784544`, 5.455144`, 
              2.813511`, 1.565543`, 4.703682`, 16.5242`, 10.484339`, 
              8.686371`, 4.524329`, 1.786916`, 3.231887`, 3.250352`, 
              1.327988`, 4.812495`, 1.568893`, 2.813511`, 1.316198`, 
              6.072621`, 8.36718`, 3.381232`, 1.507268`}, {5, 28, 
              1.445172391`*^6, \(-0.25`\), \(-1.377700057`\), 0.09`, 
              1, \(-0.5`\), \(-0.06724133883`\), 
              11.89287202`, \(-1.202193122`\), \(-1.261923865`\), 3, 4, 5, 6, 
              0.294585`, \(-3.185827`\), 4.196237`, 0.155795`, \(-0.07931`\), 
              0.276041`, 0.68486`, 0.406347`, 0.725995`, 0.433013`, 
              1.019699`, 0.76293`, 16.5242`, 16.524274`, \(-0.00048`\), 
              8.411655`, 8.411508`, 0.000955`, 5.145542`, 5.145347`, 
              0.001265`, 4.683999`, \(-1.29008`\), 
              38.793277`, \(-1.530684`\), \(-1.529282`\), \(-0.009102`\), \
\(-1.888183`\), \(-2.9479`\), 6.881377`, \(-2.9562`\), \(-3.775036`\), 
              5.317191`, \(-3.765762`\), \(-5.39008`\), 10.547673`, 16.5242`, 
              5.145542`, 
              8.411655`, \(-4.598577`\), \(-5.383717`\), \(-5.748461`\), \
\(-1.888183`\), \(-10.868514`\), \(-3.765762`\), \(-1.530684`\), \
\(-2.9562`\), \(-7.496191`\), 0.673138`, 
              4.683999`, \(-4.739621`\), \(-6.861646`\)}, {5, 29, 
              174931.8694`, \(-0.25`\), \(-4.223267213`\), 0.09`, 
              1, \(-0.4`\), \(-0.1202332899`\), 
              15.96647146`, \(-1.601065587`\), \(-1.791746856`\), 3, 4, 5, 6, 
              0.327443`, \(-1.421489`\), 2.033811`, 0.1298`, 0.029232`, 
              0.090222`, 0.77099`, 0.380249`, 0.729707`, 0.506509`, 
              0.967236`, 0.766312`, 16.5242`, 16.520641`, 
              0.078755`, \(-3.690997`\), \(-3.688307`\), \(-0.059521`\), \
\(-10.934858`\), \(-15.043445`\), 
              90.906868`, \(-14.959323`\), \(-15.528157`\), 
              12.586073`, \(-17.704921`\), \(-17.722293`\), 
              0.38437`, \(-23.461312`\), \(-23.478285`\), 
              0.375558`, \(-27.145022`\), \(-30.38061`\), 
              71.590821`, \(-30.412515`\), \(-35.809922`\), 119.423384`, 
              16.5242`, \(-3.690997`\), \(-10.934858`\), \(-18.145068`\), \
\(-33.851625`\), \(-44.145171`\), \(-27.145022`\), \(-57.059645`\), \
\(-17.704921`\), \(-23.461312`\), \(-30.412515`\), \(-39.997675`\), \
\(-19.632198`\), \(-14.959323`\), \(-32.598651`\), \(-47.652862`\)}, {5, 27, 
              14.18625471`, \(-0.7461192054`\), 7.67472108`, 0.09`, 
              1, \(-0.3`\), \(-0.1323575363`\), 
              4690.589124`, \(-1.596018519`\), \(-1.625689251`\), 3, 4, 5, 6, 
              0.401286`, \(-0.527619`\), 0.326565`, 0.132665`, 
              0.153128`, \(-0.119482`\), 0.818847`, 0.77353`, 0.813374`, 
              0.856528`, 1.085766`, 1.061195`, 16.5242`, 16.543518`, 
              0.951927`, 35.179514`, 35.163618`, \(-0.783295`\), 47.681067`, 
              47.693594`, 0.617291`, 63.148149`, 63.150459`, 0.113832`, 
              35.95307`, 35.972273`, 0.946221`, 67.56523`, 67.593519`, 
              1.393974`, 79.888576`, 79.877582`, \(-0.541733`\), 82.117078`, 
              82.095456`, \(-1.065455`\), 16.5242`, 35.179514`, 47.681067`, 
              66.358403`, 83.957527`, 103.565235`, 115.782404`, 130.986892`, 
              35.95307`, 67.56523`, 79.888576`, 82.117078`, 63.148149`, 
              86.50837`, 93.479038`, 112.511134`}, {5, 28, 
              19.64400675`, \(-1.055326036`\), 4.4208135`, 0.09`, 
              1, \(-0.2`\), \(-0.2466071549`\), 595.6940323`, 
              0.1550365336`, \(-0.977541228`\), 3, 4, 5, 6, 
              0.387442`, \(-0.457904`\), 0.094851`, 0.227844`, 
              0.031742`, \(-0.035539`\), 0.833622`, 0.645791`, 0.938067`, 
              0.86926`, 1.293465`, 1.325864`, 16.5242`, 16.563932`, 
              1.088859`, 27.043137`, 27.009798`, \(-0.913653`\), 28.330705`, 
              28.339686`, 0.246145`, 41.189606`, 41.19333`, 0.102053`, 
              20.923568`, 20.938937`, 0.421208`, 41.585288`, 41.608397`, 
              0.633298`, 47.035755`, 47.041596`, 0.160078`, 51.979475`, 
              51.977889`, \(-0.043467`\), 16.5242`, 27.043137`, 28.330705`, 
              46.699452`, 51.979475`, 63.381268`, 78.945482`, 79.736947`, 
              20.923568`, 41.585288`, 47.035755`, 53.686919`, 41.189606`, 
              60.340433`, 62.038624`, 67.156028`}, {5, 28, 
              37.07474059`, \(-1.603850317`\), 3.747269215`, 0.09`, 
              1, \(-0.1`\), \(-0.2455020282`\), 
              5.163981724`, \(-0.4874676449`\), \(-1.433774316`\), 3, 4, 5, 
              6, 0.427982`, \(-0.417515`\), \(-0.009641`\), 0.213513`, 
              0.064908`, \(-0.07008`\), 0.847674`, 0.559744`, 0.881614`, 
              0.792487`, 1.189085`, 1.241301`, 16.5242`, 16.555336`, 
              0.798964`, 22.471313`, 22.486968`, 0.401722`, 27.976495`, 
              27.950795`, \(-0.659481`\), 34.259808`, 34.281562`, 0.558201`, 
              18.179101`, 18.189373`, 0.263575`, 39.137329`, 39.1545`, 
              0.440616`, 40.775793`, 40.780844`, 0.129614`, 47.832531`, 
              47.832091`, \(-0.011303`\), 16.5242`, 22.471313`, 27.976495`, 
              34.259808`, 47.832531`, 57.562965`, 71.643772`, 74.107769`, 
              18.179101`, 39.137329`, 40.775793`, 49.909015`, 39.686037`, 
              56.173837`, 59.369065`, 62.677214`}, {5, 28, 
              50.00374924`, \(-1.923767294`\), 3.521749311`, 0.09`, 1, 
              0, \(-0.2863677642`\), 
              5.278008372`, \(-0.3976637494`\), \(-1.455530139`\), 3, 4, 5, 
              6, 0.431145`, \(-0.427715`\), \(-0.010755`\), 0.217922`, 
              0.055327`, \(-0.060298`\), 0.849161`, 0.533182`, 0.880146`, 
              0.779752`, 1.186135`, 1.244879`, 16.5242`, 16.552316`, 
              0.683057`, 22.727884`, 22.74782`, 0.484334`, 29.308374`, 
              29.283925`, \(-0.593967`\), 32.950528`, 32.971633`, 0.512722`, 
              16.997649`, 17.007361`, 0.235949`, 37.482552`, 37.495806`, 
              0.321991`, 38.817014`, 38.825733`, 0.21183`, 46.278812`, 
              46.277104`, \(-0.04149`\), 16.5242`, 22.727884`, 29.308374`, 
              32.950528`, 46.278812`, 55.381895`, 68.804169`, 71.092352`, 
              16.997649`, 37.482552`, 38.817014`, 48.972129`, 39.350883`, 
              55.000117`, 57.52326`, 62.865856`}, {5, 28, 
              63.73924935`, \(-2.155856555`\), 3.336183577`, 0.09`, 1, 
              0.1`, \(-0.329853888`\), 
              6.039179558`, \(-0.3126531892`\), \(-1.478345336`\), 3, 4, 5, 
              6, 0.423194`, \(-0.445731`\), 0.014755`, 0.221195`, 
              0.047721`, \(-0.052136`\), 0.844153`, 0.503758`, 0.870248`, 
              0.764265`, 1.172104`, 1.243825`, 16.5242`, 16.54969`, 
              0.575822`, 22.372938`, 22.394815`, 0.494196`, 29.526629`, 
              29.506398`, \(-0.457009`\), 32.543792`, 32.561686`, 0.404207`, 
              15.825556`, 15.83506`, 0.214677`, 35.389631`, 35.401282`, 
              0.263193`, 36.850005`, 36.86025`, 0.231442`, 44.199688`, 
              44.19668`, \(-0.067936`\), 16.5242`, 22.372938`, 29.526629`, 
              32.543792`, 44.199688`, 52.70555`, 65.568939`, 67.570991`, 
              15.825556`, 35.389631`, 36.850005`, 47.31025`, 38.234336`, 
              53.058163`, 55.155223`, 60.431333`}, {5, 28, 
              70.45500751`, \(-2.141862549`\), 3.274770691`, 0.09`, 1, 
              0.2`, \(-0.3738527227`\), 
              6.566168353`, \(-0.2163218027`\), \(-1.49960591`\), 3, 4, 5, 6, 
              0.407333`, \(-0.467694`\), 0.056749`, 0.223961`, 
              0.041126`, \(-0.044984`\), 0.834053`, 0.487691`, 0.854102`, 
              0.757113`, 1.150002`, 1.246525`, 16.5242`, 16.548685`, 
              0.522574`, 22.184023`, 22.208229`, 0.516633`, 29.857221`, 
              29.842904`, \(-0.305567`\), 32.856212`, 32.869306`, 0.279465`, 
              15.117312`, 15.127029`, 0.207386`, 34.311684`, 34.323922`, 
              0.261192`, 35.762673`, 35.772921`, 0.218726`, 43.11957`, 
              43.115206`, \(-0.093133`\), 16.5242`, 22.184023`, 29.857221`, 
              32.856212`, 43.11957`, 51.293646`, 64.057919`, 65.797741`, 
              15.117312`, 34.311684`, 35.762673`, 46.625903`, 37.78514`, 
              52.256387`, 54.125635`, 59.366585`}, {5, 28, 
              126.2416685`, \(-2.000343091`\), 2.76751259`, 0.09`, 1, 
              0.3`, \(-0.3375762402`\), 
              28.96396288`, \(-0.02262173013`\), \(-1.47244334`\), 3, 4, 5, 
              6, 0.414799`, \(-0.47282`\), 0.10475`, 0.227177`, 
              0.03361`, \(-0.037191`\), 0.82771`, 0.417668`, 0.85091`, 
              0.719008`, 1.14849`, 1.232505`, 16.5242`, 16.535337`, 
              0.204556`, 20.076266`, 20.093491`, 0.316391`, 26.24361`, 
              26.227387`, \(-0.297963`\), 32.688432`, 32.746959`, 1.074983`, 
              12.585209`, 12.591964`, 0.124073`, 28.962026`, 28.969415`, 
              0.135734`, 30.64049`, 30.648111`, 0.139984`, 36.750836`, 
              36.749212`, \(-0.029829`\), 16.5242`, 20.076266`, 26.24361`, 
              33.130207`, 36.750836`, 43.502426`, 54.451632`, 55.974928`, 
              12.585209`, 28.962026`, 30.64049`, 39.77311`, 32.688432`, 
              44.950349`, 46.050873`, 50.736307`}, {5, 25, 
              134.1485176`, \(-1.735742857`\), 2.773882304`, 0.09`, 1, 
              0.4`, \(-0.3547135309`\), 52.61515672`, 
              0.08981295465`, \(-1.447642303`\), 3, 4, 5, 6, 
              0.395043`, \(-0.4913`\), 0.160521`, 0.227905`, 
              0.031385`, \(-0.034914`\), 0.812403`, 0.4087`, 0.836019`, 
              0.714559`, 1.131628`, 1.231604`, 16.5242`, 16.534387`, 
              0.178607`, 20.035767`, 20.053397`, 0.309114`, 26.528318`, 
              26.512588`, \(-0.27579`\), 32.469113`, 32.584204`, 2.017874`, 
              12.15248`, 12.159162`, 0.117152`, 28.535179`, 28.542975`, 
              0.136688`, 30.025393`, 30.032144`, 0.118365`, 36.156802`, 
              36.15508`, \(-0.030186`\), 16.5242`, 20.035767`, 26.528318`, 
              33.506913`, 36.156802`, 42.714115`, 53.813794`, 55.169679`, 
              12.15248`, 28.535179`, 30.025393`, 39.433282`, 32.469113`, 
              44.667161`, 45.61555`, 50.261956`}, {5, 27, 
              1.723850258`*^6, \(-0.25`\), \(-1.259128991`\), 0.1225`, 
              1, \(-0.6`\), \(-0.02361346248`\), \(-1.996773276`\), \
\(-1.119398403`\), \(-1.269352342`\), 3, 4, 5, 6, 0.340314`, \(-3.52861`\), 
              4.283996`, 0.170634`, \(-0.103385`\), 0.305313`, 0.678684`, 
              0.437609`, 0.730287`, 0.458404`, 1.03907`, 0.819357`, 16.5242`, 
              16.525028`, \(-0.005678`\), 16.375145`, 16.372414`, 0.018727`, 
              7.293432`, 7.29364`, \(-0.001426`\), 4.713459`, 2.271063`, 
              16.745031`, 0.390456`, 
              0.391783`, \(-0.009097`\), \(-1.009376`\), \(-1.649395`\), 
              4.387963`, \(-1.653568`\), \(-2.518268`\), 
              5.928366`, \(-2.513862`\), \(-4.230124`\), 11.766671`, 
              16.5242`, \(-1.081541`\), 16.375145`, 
              7.293432`, \(-3.916233`\), \(-4.472392`\), \(-1.009376`\), \
\(-6.15328`\), \(-2.513862`\), 0.390456`, \(-1.653568`\), \(-5.166312`\), 
              2.270982`, 4.713459`, 0.706222`, \(-5.388477`\)}, {5, 28, 
              408558.0441`, \(-0.25`\), \(-2.590632357`\), 0.1225`, 
              1, \(-0.5`\), 0.003673480579`, 
              4.781691847`, \(-1.167238256`\), \(-1.502992563`\), 3, 4, 5, 6, 
              0.420803`, \(-1.266088`\), 1.72528`, 0.174151`, 0.017235`, 
              0.074176`, 0.784029`, 0.354039`, 0.803018`, 0.566462`, 
              1.094433`, 0.949922`, 16.5242`, 
              16.524355`, \(-0.002708`\), \(-2.884101`\), \(-2.883944`\), \
\(-0.002743`\), \(-5.123515`\), \(-5.12418`\), 
              0.011602`, \(-8.100472`\), \(-8.103062`\), 
              0.045168`, \(-11.227735`\), \(-11.229818`\), 
              0.036328`, \(-13.428375`\), \(-13.429847`\), 
              0.025679`, \(-17.113706`\), \(-17.105961`\), \(-0.135092`\), \
\(-18.819647`\), \(-21.049347`\), 38.891137`, 
              16.5242`, \(-2.884101`\), \(-5.123515`\), \(-8.100472`\), \
\(-21.033027`\), \(-26.489246`\), \(-18.819647`\), \(-30.044389`\), \
\(-11.227735`\), \(-13.428375`\), \(-17.113706`\), \(-24.047171`\), \
\(-11.023854`\), \(-10.065197`\), \(-13.549003`\), \(-28.005846`\)}, {5, 26, 
              10554.29463`, \(-0.2586063696`\), \(-18.81196205`\), 0.1225`, 
              1, \(-0.4`\), \(-0.1797393964`\), 
              3060.767588`, \(-3.300032502`\), \(-5.793411166`\), 3, 4, 5, 6, 
              0.375921`, \(-1.000386`\), 1.203429`, \(-0.010482`\), 
              0.143575`, \(-0.069198`\), 0.830695`, 0.016681`, 0.095444`, 
              0.01401`, \(-0.223521`\), 0.006305`, 16.5242`, 16.464432`, 
              6.76264`, \(-55.210407`\), \(-55.157205`\), \(-6.019742`\), \
\(-109.145285`\), \(-109.181553`\), 
              4.103734`, \(-113.459438`\), \(-113.491409`\), 
              3.617488`, \(-88.327767`\), \(-88.397305`\), 
              7.868149`, \(-141.900908`\), \(-141.999264`\), 
              11.128983`, \(-185.268264`\), \(-185.319388`\), 
              5.784578`, \(-188.215728`\), \(-188.255196`\), 4.465774`, 
              16.5242`, \(-55.210407`\), \(-109.145285`\), \(-113.459438`\), \
\(-188.215728`\), \(-238.444655`\), \(-221.612383`\), \(-298.713854`\), \
\(-88.327767`\), \(-141.900908`\), \(-185.268264`\), \(-189.464806`\), \
\(-129.500416`\), \(-167.159882`\), \(-163.757176`\), \(-215.168116`\)}, {5, 
              28, 17.72684632`, \(-0.9154157745`\), 4.935650506`, 0.1225`, 
              1, \(-0.3`\), \(-0.2015029857`\), 
              12.8443761`, \(-0.5735172345`\), \(-1.116429063`\), 3, 4, 5, 6, 
              0.443252`, \(-0.455625`\), 0.092413`, 0.213099`, 
              0.062306`, \(-0.064335`\), 0.864258`, 0.742975`, 0.956235`, 
              0.926592`, 1.310474`, 1.326555`, 16.5242`, 16.552636`, 
              0.995369`, 29.048267`, 29.022867`, \(-0.889098`\), 30.870823`, 
              30.880799`, 0.349199`, 47.121698`, 47.123997`, 0.080467`, 
              24.791159`, 24.804477`, 0.466177`, 49.17128`, 49.194651`, 
              0.818052`, 53.952004`, 53.953725`, 0.060227`, 59.901704`, 
              59.87241`, \(-1.025418`\), 16.5242`, 29.048267`, 30.870823`, 
              50.365917`, 60.323148`, 73.225292`, 90.202151`, 91.111752`, 
              24.791159`, 49.17128`, 53.952004`, 59.901704`, 47.121698`, 
              67.63879`, 69.194023`, 73.134665`}, {5, 28, 
              39.22692648`, \(-1.332102434`\), 3.646155984`, 0.1225`, 
              1, \(-0.2`\), \(-0.244317561`\), 
              6.512733686`, \(-0.5214868773`\), \(-1.496207703`\), 3, 4, 5, 
              6, 0.467648`, \(-0.400819`\), \(-0.068949`\), 0.216527`, 
              0.057401`, \(-0.061547`\), 0.879957`, 0.572609`, 0.915946`, 
              0.8045`, 1.232736`, 1.25617`, 16.5242`, 16.553943`, 0.811434`, 
              22.695017`, 22.706041`, 0.300758`, 26.980678`, 
              26.956443`, \(-0.661164`\), 35.474515`, 35.494388`, 0.54217`, 
              18.746476`, 18.755902`, 0.257152`, 39.786461`, 39.802702`, 
              0.443086`, 40.797423`, 40.801904`, 0.12226`, 48.295931`, 
              48.290651`, \(-0.144032`\), 16.5242`, 22.695017`, 26.980678`, 
              35.474515`, 48.295931`, 57.883186`, 71.729464`, 73.74682`, 
              18.746476`, 39.786461`, 40.797423`, 49.377663`, 39.493077`, 
              55.37084`, 59.427735`, 61.135114`}, {5, 29, 
              99.77966453`, \(-1.734366443`\), 2.844438221`, 0.1225`, 
              1, \(-0.1`\), \(-0.2663468885`\), 
              13.98732443`, \(-0.5783743309`\), \(-1.515257495`\), 3, 4, 5, 
              6, 0.489901`, \(-0.39996`\), \(-0.09323`\), 0.2153`, 
              0.059886`, \(-0.064714`\), 0.888808`, 0.456023`, 0.924284`, 
              0.727711`, 1.242575`, 1.206561`, 16.5242`, 16.543289`, 
              0.425604`, 20.024159`, 20.034506`, 0.230711`, 24.271284`, 
              24.251735`, \(-0.435871`\), 33.060415`, 33.106862`, 1.035561`, 
              14.78947`, 14.796213`, 0.150353`, 31.829581`, 31.835623`, 
              0.13472`, 33.410126`, 33.419684`, 0.213088`, 39.737378`, 
              39.734609`, \(-0.061742`\), 16.5242`, 20.024159`, 24.271284`, 
              33.060415`, 39.737378`, 47.229823`, 58.249856`, 60.082532`, 
              14.78947`, 31.829581`, 33.410126`, 41.231789`, 33.502566`, 
              46.158941`, 48.782393`, 52.888265`}, {5, 28, 
              42.14434238`, \(-1.816988604`\), 3.650097848`, 0.1225`, 1, 
              0, \(-0.3532951666`\), 
              4.388088305`, \(-0.2019734946`\), \(-1.602608055`\), 3, 4, 5, 
              6, 0.469619`, \(-0.413821`\), \(-0.10832`\), 0.226904`, 
              0.036313`, \(-0.040345`\), 0.8855`, 0.587754`, 0.906329`, 
              0.827351`, 1.213647`, 1.297667`, 16.5242`, 16.559776`, 
              0.975736`, 25.539888`, 25.567441`, 0.755672`, 33.248296`, 
              33.252227`, 0.107803`, 34.414396`, 34.405065`, \(-0.255936`\), 
              18.679271`, 18.689652`, 0.284715`, 40.58678`, 40.595915`, 
              0.250541`, 42.704591`, 42.720379`, 0.433003`, 51.343158`, 
              51.33542`, \(-0.212251`\), 16.5242`, 25.539888`, 33.248296`, 
              34.414396`, 51.343158`, 60.852941`, 75.120397`, 77.1104`, 
              18.679271`, 40.58678`, 42.704591`, 54.343864`, 43.98869`, 
              60.431136`, 63.343498`, 68.856167`}, {2, 19, 
              134.5188684`, \(-2.358227955`\), 2.545776505`, 0.1225`, 1, 
              0.1`, \(-0.3259579559`\), 48.75408591`, 
              0.2680555462`, \(-1.791296026`\), 3, 4, 5, 6, 
              0.493604`, \(-0.427578`\), \(-0.055837`\), 0.229693`, 
              0.028329`, \(-0.031799`\), 0.887586`, 0.421377`, 0.878991`, 
              0.724045`, 1.16561`, 1.242076`, 16.5242`, 16.53487`, 0.214532`, 
              19.928517`, 19.94452`, 0.321761`, 25.191566`, 
              25.174361`, \(-0.345919`\), 32.123104`, 32.171756`, 0.978184`, 
              13.05496`, 13.060881`, 0.119043`, 28.438483`, 28.443293`, 
              0.096705`, 31.092073`, 31.101248`, 0.184468`, 36.950295`, 
              36.947267`, \(-0.06088`\), 16.5242`, 19.928517`, 25.191566`, 
              32.123104`, 36.950295`, 43.516977`, 53.632835`, 55.164972`, 
              13.05496`, 28.438483`, 31.092073`, 39.160544`, 32.283298`, 
              43.770295`, 45.444757`, 49.600356`}, {5, 28, 
              76.82579997`, \(-2.06101275`\), 3.087844459`, 0.1225`, 1, 
              0.2`, \(-0.4476124318`\), 
              7.260196518`, \(-0.04730583667`\), \(-1.666933425`\), 3, 4, 5, 
              6, 0.445975`, \(-0.451028`\), \(-0.052697`\), 0.229353`, 
              0.030359`, \(-0.033958`\), 0.871653`, 0.486842`, 0.878508`, 
              0.763151`, 1.173591`, 1.264547`, 16.5242`, 16.549114`, 
              0.548953`, 22.643788`, 22.671181`, 0.603563`, 30.475666`, 
              30.473358`, \(-0.050868`\), 32.98372`, 
              32.982433`, \(-0.028354`\), 15.20059`, 15.210051`, 0.20844`, 
              33.753079`, 33.76125`, 0.180043`, 35.854012`, 35.868827`, 
              0.32644`, 43.500618`, 43.491269`, \(-0.205986`\), 16.5242`, 
              22.643788`, 30.475666`, 32.98372`, 43.500618`, 51.251178`, 
              63.572603`, 64.86523`, 15.20059`, 33.753079`, 35.854012`, 
              46.964109`, 38.254857`, 52.061945`, 54.141353`, 
              58.976483`}, {5, 28, 78.45179439`, \(-1.938995358`\), 
              3.103015629`, 0.1225`, 1, 0.3`, \(-0.5003035752`\), 
              7.544450558`, 0.05710589274`, \(-1.699331879`\), 3, 4, 5, 6, 
              0.422814`, \(-0.475308`\), \(-0.006603`\), 0.230083`, 
              0.028341`, \(-0.031881`\), 0.858285`, 0.476782`, 0.856187`, 
              0.757596`, 1.142446`, 1.262984`, 16.5242`, 16.54981`, 
              0.537597`, 22.454142`, 22.484527`, 0.637849`, 30.676561`, 
              30.680905`, 0.091174`, 33.807228`, 33.800604`, \(-0.139059`\), 
              14.785445`, 14.7955`, 0.211092`, 33.36983`, 33.378958`, 
              0.191621`, 35.157874`, 35.173068`, 0.318939`, 43.00052`, 
              42.989287`, \(-0.235805`\), 16.5242`, 22.454142`, 30.676561`, 
              33.807228`, 43.00052`, 50.575687`, 63.069117`, 64.087739`, 
              14.785445`, 33.36983`, 35.157874`, 46.860125`, 38.149053`, 
              51.864352`, 53.929565`, 58.682225`}, {5, 28, 
              77.97563945`, \(-1.757446534`\), 3.151202006`, 0.1225`, 1, 
              0.4`, \(-0.5534286064`\), 7.725028086`, 
              0.131668173`, \(-1.726646938`\), 3, 4, 5, 6, 
              0.395125`, \(-0.506379`\), 0.058717`, 0.230236`, 
              0.027586`, \(-0.03119`\), 0.841436`, 0.467895`, 0.830191`, 
              0.752172`, 1.106838`, 1.260013`, 16.5242`, 16.551111`, 
              0.536117`, 22.168874`, 22.202469`, 0.669268`, 30.900041`, 
              30.908837`, 0.175235`, 34.685968`, 34.676713`, \(-0.184386`\), 
              14.419518`, 14.430317`, 0.21514`, 33.191759`, 33.202632`, 
              0.216602`, 34.522961`, 34.537958`, 0.298778`, 42.60862`, 
              42.595319`, \(-0.264967`\), 16.5242`, 22.168874`, 30.900041`, 
              34.685968`, 42.60862`, 50.055586`, 62.855102`, 63.579557`, 
              14.419518`, 33.191759`, 34.522961`, 46.864112`, 38.075111`, 
              51.780645`, 53.966272`, 58.578637`}, {5, 28, 
              866579.3926`, \(-0.25`\), \(-1.775787208`\), 0.16`, 
              1, \(-0.6`\), \(-0.100626094`\), 
              6.072215161`, \(-0.8804385217`\), \(-1.298345544`\), 3, 4, 5, 
              6, 0.377864`, \(-2.258257`\), 3.092339`, 0.195792`, 0.010546`, 
              0.062063`, 0.790231`, 0.281125`, 0.851073`, 0.567401`, 
              1.180525`, 1.024689`, 16.5242`, 16.524395`, \(-0.002089`\), 
              2.380375`, 2.379946`, 
              0.004616`, \(-1.607672`\), \(-1.608627`\), 
              0.010255`, \(-4.198231`\), \(-4.197803`\), \(-0.004592`\), \
\(-6.314907`\), \(-6.31759`\), 0.028801`, \(-7.128005`\), \(-7.134632`\), 
              0.071141`, \(-9.026408`\), \(-9.017511`\), \(-0.095513`\), \
\(-9.976663`\), \(-12.013183`\), 21.86429`, 16.5242`, 
              2.380375`, \(-1.607672`\), \(-4.198231`\), \(-11.973218`\), \
\(-16.241225`\), \(-9.976663`\), \(-19.404173`\), \(-7.128005`\), \
\(-6.314907`\), \(-9.026408`\), \(-15.218677`\), \(-4.506474`\), \(-4.436675`\
\), \(-5.831828`\), \(-14.776759`\)}, {5, 28, 
              68833.30004`, \(-0.25`\), \(-6.345206743`\), 0.16`, 
              1, \(-0.5`\), \(-0.1186143331`\), 
              37.2384873`, \(-1.185838395`\), \(-1.332857425`\), 3, 4, 5, 6, 
              0.442203`, \(-1.016016`\), 1.218178`, 0.183769`, 0.039248`, 
              0.015575`, 0.854285`, 0.818347`, 0.916467`, 0.943715`, 
              1.252855`, 1.243933`, 16.5242`, 16.520492`, 
              0.166472`, \(-17.524814`\), \(-17.522344`\), \(-0.110878`\), \
\(-32.365065`\), \(-33.403081`\), 
              46.600568`, \(-37.579004`\), \(-37.586635`\), 
              0.342586`, \(-31.137171`\), \(-31.151824`\), 
              0.657803`, \(-49.390294`\), \(-49.420091`\), 
              1.337709`, \(-61.63169`\), \(-61.557106`\), \(-3.348363`\), \
\(-62.027374`\), \(-62.064043`\), 1.646228`, 
              16.5242`, \(-17.524814`\), \(-32.365065`\), \(-37.579004`\), \
\(-64.709361`\), \(-81.576034`\), \(-77.872751`\), \(-99.307044`\), \
\(-31.137171`\), \(-49.390294`\), \(-61.63169`\), \(-62.027374`\), \
\(-42.852787`\), \(-58.073829`\), \(-48.030815`\), \(-73.096289`\)}, {5, 27, 
              27.03428549`, \(-0.7190986503`\), 6.183076506`, 0.16`, 
              1, \(-0.4`\), \(-0.1818024471`\), 
              2909.27265`, \(-1.463401019`\), \(-1.660199866`\), 3, 4, 5, 6, 
              0.484997`, \(-0.476819`\), 0.118164`, 0.168746`, 
              0.104274`, \(-0.084747`\), 0.8912`, 0.811212`, 0.908304`, 
              0.927459`, 1.215758`, 1.204918`, 16.5242`, 16.544121`, 
              0.95583`, 35.0156`, 34.998888`, \(-0.801849`\), 42.161432`, 
              42.169436`, 0.384073`, 59.082065`, 59.084094`, 0.09734`, 
              32.441645`, 32.456012`, 0.689354`, 63.83401`, 63.8725`, 
              1.846759`, 68.230386`, 68.220071`, \(-0.494884`\), 73.909118`, 
              73.883705`, \(-1.219356`\), 16.5242`, 35.0156`, 42.161432`, 
              64.205663`, 77.161021`, 93.500841`, 113.821148`, 115.765673`, 
              32.441645`, 63.83401`, 68.230386`, 73.909118`, 59.082065`, 
              83.71733`, 85.519919`, 91.762911`}, {5, 28, 
              38.99928338`, \(-1.122780762`\), 3.972527823`, 0.16`, 
              1, \(-0.3`\), \(-0.2356706847`\), 
              5.881418548`, \(-0.1943789734`\), \(-1.528255224`\), 3, 4, 5, 
              6, 0.510927`, \(-0.385915`\), \(-0.121199`\), 0.229284`, 
              0.031702`, \(-0.035242`\), 0.911024`, 0.670349`, 0.953355`, 
              0.889824`, 1.282464`, 1.343964`, 16.5242`, 16.552929`, 
              0.932967`, 24.796889`, 24.808291`, 0.370288`, 29.099475`, 
              29.07726`, \(-0.721413`\), 37.98994`, 38.009695`, 0.641509`, 
              21.518672`, 21.52797`, 0.30197`, 45.069207`, 45.080387`, 
              0.36306`, 45.69133`, 45.701412`, 0.327417`, 54.317122`, 
              54.241827`, \(-2.445206`\), 16.5242`, 24.796889`, 29.099475`, 
              37.98994`, 54.439011`, 64.999738`, 80.219383`, 81.642239`, 
              21.518672`, 45.069207`, 45.69133`, 54.317122`, 43.928557`, 
              60.79754`, 66.444695`, 66.973275`}, {5, 28, 
              37.83683621`, \(-1.350611781`\), 3.628216808`, 0.16`, 
              1, \(-0.2`\), \(-0.3164910228`\), 5.16882076`, 
              0.04601550108`, \(-1.625068378`\), 3, 4, 5, 6, 
              0.510538`, \(-0.375197`\), \(-0.201908`\), 0.232002`, 
              0.025851`, \(-0.029406`\), 0.91953`, 0.615957`, 0.949271`, 
              0.853238`, 1.270928`, 1.327291`, 16.5242`, 16.55959`, 
              1.048868`, 24.444732`, 24.464741`, 0.593013`, 31.125179`, 
              31.09505`, \(-0.892934`\), 34.71221`, 34.734818`, 0.670041`, 
              19.7968`, 19.806527`, 0.288281`, 41.725076`, 41.732924`, 
              0.232622`, 43.339828`, 43.355504`, 0.464598`, 52.008426`, 
              51.993969`, \(-0.428457`\), 16.5242`, 24.444732`, 31.125179`, 
              34.71221`, 52.008426`, 61.66983`, 75.82121`, 77.508846`, 
              19.7968`, 41.725076`, 43.339828`, 53.488122`, 43.198737`, 
              59.338128`, 63.545244`, 68.288967`}, {5, 28, 
              118.0661215`, \(-1.698535089`\), 2.56237359`, 0.16`, 
              1, \(-0.1`\), \(-0.3311741299`\), 26.80171151`, 
              0.06406279255`, \(-1.762183466`\), 3, 4, 5, 6, 
              0.535276`, \(-0.381756`\), \(-0.200817`\), 0.232076`, 
              0.025676`, \(-0.029209`\), 0.927164`, 0.445167`, 0.938058`, 
              0.741523`, 1.247762`, 1.259466`, 16.5242`, 16.54087`, 
              0.365831`, 19.738147`, 19.751225`, 0.287008`, 24.661037`, 
              24.640478`, \(-0.451172`\), 32.514703`, 32.516151`, 0.031772`, 
              14.129533`, 14.135739`, 0.136183`, 29.77318`, 29.777702`, 
              0.099225`, 32.373526`, 32.384454`, 0.239805`, 38.497575`, 
              38.490364`, \(-0.158248`\), 16.5242`, 19.738147`, 24.661037`, 
              32.514703`, 38.497575`, 45.313042`, 55.506562`, 56.889406`, 
              14.129533`, 29.77318`, 32.373526`, 40.015033`, 32.799374`, 
              44.380246`, 46.898796`, 50.633431`}, {5, 28, 
              73.14441445`, \(-1.758863576`\), 2.980116341`, 0.16`, 1, 
              0, \(-0.41971656`\), 
              7.338959657`, \(-0.007114872394`\), \(-1.74643075`\), 3, 4, 5, 
              6, 0.514249`, \(-0.395733`\), \(-0.216968`\), 0.231754`, 
              0.026501`, \(-0.030084`\), 0.923332`, 0.507699`, 0.935611`, 
              0.779577`, 1.245772`, 1.281144`, 16.5242`, 16.550027`, 
              0.633278`, 22.973596`, 22.998072`, 0.60015`, 30.40875`, 
              30.39443`, \(-0.351144`\), 31.994399`, 32.001211`, 0.167036`, 
              16.190577`, 16.199045`, 0.207656`, 34.398257`, 34.404448`, 
              0.151792`, 37.403398`, 37.418758`, 0.376639`, 44.943228`, 
              44.931422`, \(-0.289492`\), 16.5242`, 22.973596`, 30.40875`, 
              31.994399`, 44.943228`, 52.767223`, 64.715259`, 66.019426`, 
              16.190577`, 34.398257`, 37.403398`, 47.54182`, 38.803406`, 
              52.398663`, 55.009138`, 59.496121`}, {5, 28, 
              83.39748003`, \(-1.900064287`\), 2.901442511`, 0.16`, 1, 
              0.1`, \(-0.4739829439`\), 8.024356218`, 
              0.0480727038`, \(-1.795975883`\), 3, 4, 5, 6, 
              0.503337`, \(-0.412128`\), \(-0.201892`\), 0.232049`, 
              0.02564`, \(-0.029167`\), 0.918321`, 0.489135`, 0.921524`, 
              0.768102`, 1.224373`, 1.27497`, 16.5242`, 16.548622`, 
              0.570649`, 22.87212`, 22.899454`, 0.638684`, 30.506086`, 
              30.506135`, 0.001158`, 32.455592`, 32.448159`, \(-0.173674`\), 
              15.533313`, 15.541965`, 0.202163`, 33.26251`, 33.268844`, 
              0.148005`, 36.388778`, 36.404596`, 0.369602`, 43.922936`, 
              43.909975`, \(-0.302844`\), 16.5242`, 22.87212`, 30.506086`, 
              32.455592`, 43.922936`, 51.411921`, 63.154868`, 64.234021`, 
              15.533313`, 33.26251`, 36.388778`, 46.939801`, 38.392744`, 
              51.60533`, 53.959737`, 58.413428`}, {5, 28, 
              89.05170378`, \(-1.946890496`\), 2.864495424`, 0.16`, 1, 
              0.2`, \(-0.5326610399`\), 8.797703207`, 
              0.1211065883`, \(-1.845978643`\), 3, 4, 5, 6, 
              0.484451`, \(-0.430113`\), \(-0.180367`\), 0.232188`, 
              0.025006`, \(-0.02852`\), 0.909245`, 0.473628`, 0.90198`, 
              0.758547`, 1.195667`, 1.269541`, 16.5242`, 16.548251`, 
              0.534022`, 22.58793`, 22.618028`, 0.668285`, 30.550538`, 
              30.555445`, 0.108951`, 33.135292`, 33.122946`, \(-0.27413`\), 
              14.99368`, 15.002758`, 0.201563`, 32.467273`, 32.473933`, 
              0.14787`, 35.398614`, 35.41518`, 0.367813`, 43.041634`, 
              43.0269`, \(-0.32715`\), 16.5242`, 22.58793`, 30.550538`, 
              33.135292`, 43.041634`, 50.265895`, 61.950917`, 62.769072`, 
              14.99368`, 32.467273`, 35.398614`, 46.435836`, 37.980566`, 
              50.92168`, 53.172`, 57.53545`}, {5, 28, 
              90.32616987`, \(-1.892519282`\), 2.877914161`, 0.16`, 1, 
              0.3`, \(-0.590566627`\), 9.197235399`, 
              0.1559502832`, \(-1.891499706`\), 3, 4, 5, 6, 
              0.461045`, \(-0.451077`\), \(-0.147113`\), 0.232169`, 
              0.024807`, \(-0.028361`\), 0.896979`, 0.463979`, 0.878875`, 
              0.752566`, 1.162447`, 1.265878`, 16.5242`, 16.548932`, 
              0.525045`, 22.389676`, 22.422795`, 0.703093`, 30.713499`, 
              30.722353`, 0.18798`, 33.964357`, 33.94869`, \(-0.332618`\), 
              14.634799`, 14.644465`, 0.205195`, 32.141349`, 32.148482`, 
              0.151447`, 34.722427`, 34.739867`, 0.370252`, 42.591078`, 
              42.574282`, \(-0.356576`\), 16.5242`, 22.389676`, 30.713499`, 
              33.964357`, 42.591078`, 49.647539`, 61.473185`, 62.025833`, 
              14.634799`, 32.141349`, 34.722427`, 46.352605`, 37.875322`, 
              50.713041`, 52.988499`, 57.247901`}, {5, 28, 
              88.95488433`, \(-1.774044152`\), 2.928081187`, 0.16`, 1, 
              0.4`, \(-0.6455356189`\), 9.212875604`, 
              0.07567438624`, \(-1.93143286`\), 3, 4, 5, 6, 
              0.4348`, \(-0.476269`\), \(-0.098797`\), 0.232174`, 
              0.024884`, \(-0.028427`\), 0.882036`, 0.458432`, 0.85308`, 
              0.749268`, 1.125975`, 1.263994`, 16.5242`, 16.55031`, 
              0.531869`, 22.25785`, 22.294192`, 0.740291`, 30.924507`, 
              30.937472`, 0.264099`, 34.890654`, 34.872505`, \(-0.369686`\), 
              14.383096`, 14.39344`, 0.210703`, 32.13242`, 32.140183`, 
              0.158128`, 34.258603`, 34.276865`, 0.371993`, 42.428828`, 
              42.409868`, \(-0.386209`\), 16.5242`, 22.25785`, 30.924507`, 
              34.890654`, 42.428828`, 49.379499`, 61.496585`, 61.781644`, 
              14.383096`, 32.13242`, 34.258603`, 46.558062`, 37.982428`, 
              50.831932`, 53.234947`, 57.368505`}, {5, 28, 
              186111.8218`, \(-0.25`\), \(-3.833205738`\), 0.2025`, 
              1, \(-0.6`\), \(-0.002557026828`\), 
              16.118331`, \(-2.367887516`\), \(-2.947348492`\), 3, 4, 5, 6, 
              0.535053`, \(-0.861582`\), 0.882907`, 
              0.119736`, \(-0.015088`\), 0.122843`, 0.878221`, 0.364451`, 
              0.699352`, 0.46741`, 0.858581`, 0.692732`, 16.5242`, 
              16.524376`, \(-0.005769`\), \(-12.674508`\), \(-12.674208`\), \
\(-0.009847`\), \(-18.129892`\), \(-18.130226`\), 
              0.010955`, \(-21.357277`\), \(-21.361394`\), 
              0.135085`, \(-19.598233`\), \(-19.600202`\), 
              0.06461`, \(-30.611796`\), \(-30.616009`\), 
              0.138235`, \(-35.900082`\), \(-35.902976`\), 
              0.094961`, \(-38.858146`\), \(-38.845478`\), \(-0.415713`\), 
              16.5242`, \(-12.674508`\), \(-18.129892`\), \(-21.357277`\), \
\(-41.002467`\), \(-50.723416`\), \(-53.268514`\), \(-58.031158`\), \
\(-19.598233`\), \(-30.611796`\), \(-35.900082`\), \(-38.858146`\), \
\(-26.304557`\), \(-39.516572`\), \(-25.157008`\), \(-47.412401`\)}, {5, 28, 
              110.7178352`, \(-0.5830882091`\), 9.630732469`, 0.2025`, 
              1, \(-0.5`\), \(-0.1497423925`\), 
              257.6376147`, \(-2.440065254`\), \(-2.266028013`\), 3, 4, 5, 6, 
              0.536344`, \(-0.486663`\), 0.111625`, 0.108548`, 
              0.1276`, \(-0.085796`\), 0.919248`, 0.881037`, 0.854593`, 
              0.928407`, 1.108934`, 1.056139`, 16.5242`, 16.538353`, 
              1.169701`, 54.202233`, 54.19071`, \(-0.952406`\), 67.090812`, 
              67.101761`, 0.904931`, 92.90133`, 92.902995`, 0.13763`, 
              52.728881`, 52.746293`, 1.439061`, 105.329145`, 
              105.316332`, \(-1.059004`\), 105.630715`, 105.683706`, 
              4.379557`, 113.713728`, 113.674825`, \(-3.215153`\), 16.5242`, 
              54.202233`, 67.090812`, 96.67838`, 123.502773`, 149.168632`, 
              179.495252`, 185.162883`, 52.728881`, 105.630715`, 105.329145`, 
              113.713728`, 92.90133`, 129.36947`, 133.782812`, 
              144.484187`}, {5, 27, 26.63225064`, \(-0.9251833264`\), 
              4.353437954`, 0.2025`, 1, \(-0.4`\), \(-0.2409051304`\), 
              6.582782365`, 0.3661416886`, \(-1.222171263`\), 3, 4, 5, 6, 
              0.548422`, \(-0.378976`\), \(-0.169163`\), 0.232184`, 
              0.024731`, \(-0.028346`\), 0.940822`, 0.76449`, 1.038653`, 
              0.965022`, 1.413525`, 1.401292`, 16.5242`, 16.55205`, 
              1.063872`, 27.666934`, 27.67742`, 0.400575`, 31.629498`, 
              31.608868`, \(-0.788066`\), 42.753428`, 42.774278`, 0.796506`, 
              24.660708`, 24.670446`, 0.371991`, 50.770919`, 50.779271`, 
              0.319062`, 51.237573`, 51.25304`, 0.590812`, 59.783828`, 
              59.757027`, \(-1.023795`\), 16.5242`, 31.629498`, 27.666934`, 
              42.753428`, 61.293384`, 72.995229`, 89.667485`, 90.207453`, 
              24.660708`, 50.770919`, 51.237573`, 59.783828`, 48.774282`, 
              66.753691`, 73.929948`, 74.331135`}, {5, 29, 
              77.43742101`, \(-1.202698908`\), 2.95061256`, 0.2025`, 
              1, \(-0.3`\), \(-0.3097757476`\), 
              15.71552733`, \(-0.2522393516`\), \(-1.640118562`\), 3, 4, 5, 
              6, 0.561391`, \(-0.347704`\), \(-0.286886`\), 0.230718`, 
              0.029244`, \(-0.032765`\), 0.95406`, 0.52449`, 0.99358`, 
              0.789401`, 1.330357`, 1.284577`, 16.5242`, 16.54948`, 0.66999`, 
              21.722722`, 21.729399`, 0.176957`, 25.607409`, 
              25.584748`, \(-0.600582`\), 36.054157`, 36.06522`, 0.293201`, 
              17.02651`, 17.033625`, 0.18856`, 35.064777`, 35.069525`, 
              0.125853`, 37.011443`, 37.024842`, 0.355129`, 44.192132`, 
              44.152943`, \(-1.038639`\), 16.5242`, 25.607409`, 21.722722`, 
              36.054157`, 44.192132`, 52.153768`, 63.796628`, 64.766747`, 
              17.02651`, 35.064777`, 37.011443`, 44.553866`, 36.397826`, 
              49.285561`, 53.566899`, 57.092926`}, {5, 28, 
              113.53913`, \(-1.476004774`\), 2.467908405`, 0.2025`, 
              1, \(-0.2`\), \(-0.3499262497`\), 
              49.55885991`, \(-0.1008316774`\), \(-2.08891416`\), 3, 4, 5, 6, 
              0.580066`, \(-0.347703`\), \(-0.317228`\), 0.23286`, 
              0.025235`, \(-0.028829`\), 0.963014`, 0.447584`, 0.937969`, 
              0.744351`, 1.23026`, 1.263689`, 16.5242`, 16.542506`, 
              0.419303`, 19.654159`, 19.664995`, 0.248189`, 24.3537`, 
              24.332474`, \(-0.486181`\), 32.76077`, 32.809966`, 1.126833`, 
              14.413095`, 14.418927`, 0.133578`, 29.636789`, 29.640719`, 
              0.090034`, 32.671487`, 32.682885`, 0.261069`, 38.772303`, 
              38.758471`, \(-0.316815`\), 16.5242`, 19.654159`, 24.3537`, 
              32.887473`, 38.772303`, 45.434067`, 55.373366`, 56.422582`, 
              14.413095`, 29.636789`, 32.671487`, 39.789602`, 32.76077`, 
              43.868904`, 46.877759`, 50.222873`}, {5, 28, 
              67.31096761`, \(-1.522121515`\), 2.991809978`, 0.2025`, 
              1, \(-0.1`\), \(-0.4527635876`\), 6.897759694`, 
              0.1280596074`, \(-1.885182731`\), 3, 4, 5, 6, 
              0.558257`, \(-0.359476`\), \(-0.353299`\), 0.233898`, 
              0.022594`, \(-0.026031`\), 0.961781`, 0.5338`, 0.967026`, 
              0.799243`, 1.281058`, 1.298889`, 16.5242`, 16.55506`, 
              0.824687`, 23.840563`, 23.866311`, 0.688067`, 31.48235`, 
              31.484835`, 0.066407`, 32.464998`, 32.453261`, \(-0.31365`\), 
              17.302911`, 17.311405`, 0.227`, 35.794946`, 35.800733`, 
              0.154646`, 39.522348`, 39.539302`, 0.453057`, 47.427412`, 
              47.408686`, \(-0.50044`\), 16.5242`, 23.840563`, 31.48235`, 
              32.464998`, 47.427412`, 55.42943`, 67.571803`, 68.581238`, 
              17.302911`, 35.794946`, 39.522348`, 49.705179`, 40.639028`, 
              54.38748`, 57.558934`, 61.832465`}, {5, 28, 
              85.92445192`, \(-1.662797309`\), 2.787454706`, 0.2025`, 1, 
              0, \(-0.5087698625`\), 8.455665133`, 
              0.1385726106`, \(-1.923978378`\), 3, 4, 5, 6, 
              0.553778`, \(-0.373977`\), \(-0.349303`\), 0.233894`, 
              0.022541`, \(-0.025971`\), 0.960415`, 0.495191`, 0.959463`, 
              0.774352`, 1.268942`, 1.283859`, 16.5242`, 16.550275`, 
              0.643996`, 23.098006`, 23.12578`, 0.685974`, 30.740377`, 
              30.746803`, 0.15872`, 32.32092`, 32.304282`, \(-0.410911`\), 
              16.00849`, 16.016713`, 0.203081`, 33.260656`, 33.266324`, 
              0.139995`, 37.215494`, 37.232046`, 0.408818`, 44.767992`, 
              44.749981`, \(-0.444843`\), 16.5242`, 23.098006`, 30.740377`, 
              32.32092`, 44.767992`, 52.132154`, 63.565335`, 64.377606`, 
              16.00849`, 33.260656`, 37.215494`, 47.43135`, 38.888936`, 
              51.75159`, 54.451855`, 58.583936`}, {5, 28, 
              96.24204354`, \(-1.74299635`\), 2.688703911`, 0.2025`, 1, 
              0.1`, \(-0.5767762193`\), 10.17590226`, 
              0.1109064485`, \(-1.985481819`\), 3, 4, 5, 6, 
              0.537131`, \(-0.38948`\), \(-0.345521`\), 0.233916`, 
              0.022538`, \(-0.025993`\), 0.9544`, 0.470285`, 0.942329`, 
              0.758981`, 1.242773`, 1.274832`, 16.5242`, 16.548551`, 
              0.562668`, 22.466065`, 22.496403`, 0.701025`, 30.578334`, 
              30.583426`, 0.117664`, 32.700209`, 32.683103`, \(-0.395273`\), 
              15.196965`, 15.205478`, 0.196718`, 31.820685`, 31.826548`, 
              0.135489`, 35.605625`, 35.622808`, 0.397057`, 43.132226`, 
              43.112853`, \(-0.447655`\), 16.5242`, 22.466065`, 30.578334`, 
              32.700209`, 43.132226`, 50.101138`, 61.208906`, 61.762781`, 
              15.196965`, 31.820685`, 35.605625`, 46.178767`, 37.842303`, 
              50.197965`, 52.704524`, 56.696283`}, {5, 28, 
              99.10624641`, \(-1.78996237`\), 2.678828121`, 0.2025`, 1, 
              0.2`, \(-0.6408733774`\), 10.86699435`, 
              0.02450756495`, \(-2.040120351`\), 3, 4, 5, 6, 
              0.517258`, \(-0.406174`\), \(-0.33198`\), 0.233774`, 
              0.022926`, \(-0.026406`\), 0.945648`, 0.459616`, 0.922506`, 
              0.752408`, 1.213334`, 1.27064`, 16.5242`, 16.548768`, 
              0.544684`, 22.316107`, 22.349616`, 0.742897`, 30.699249`, 
              30.708327`, 0.201265`, 33.501393`, 33.480012`, \(-0.474012`\), 
              14.828852`, 14.837862`, 0.19977`, 31.355768`, 31.361935`, 
              0.136722`, 34.923507`, 34.941636`, 0.401924`, 42.636201`, 
              42.614879`, \(-0.472729`\), 16.5242`, 22.316107`, 30.699249`, 
              33.501393`, 42.636201`, 49.414989`, 60.551629`, 60.864426`, 
              14.828852`, 31.355768`, 34.923507`, 46.056523`, 37.715964`, 
              49.905849`, 52.389254`, 56.294657`}, {5, 28, 
              97.05201176`, \(-1.783014525`\), 2.717357168`, 0.2025`, 1, 
              0.3`, \(-0.7017629133`\), 10.70931021`, 
              0.04542106635`, \(-2.090357965`\), 3, 4, 5, 6, 
              0.494432`, \(-0.424818`\), \(-0.307999`\), 0.233852`, 
              0.022617`, \(-0.026108`\), 0.934408`, 0.455782`, 0.899943`, 
              0.750151`, 1.180405`, 1.269468`, 16.5242`, 16.550242`, 
              0.559809`, 22.358347`, 22.395318`, 0.79476`, 30.86647`, 
              30.880513`, 0.301885`, 34.538245`, 34.513191`, \(-0.538585`\), 
              14.661625`, 14.671264`, 0.207199`, 31.377352`, 31.383887`, 
              0.140485`, 34.644755`, 34.663994`, 0.413578`, 42.662635`, 
              42.639015`, \(-0.507755`\), 16.5242`, 22.358347`, 30.86647`, 
              34.538245`, 42.662635`, 49.346466`, 60.705909`, 60.77981`, 
              14.661625`, 31.377352`, 34.644755`, 46.469556`, 38.003152`, 
              50.196992`, 52.767454`, 56.589495`}, {5, 28, 
              92.79763043`, \(-1.727669846`\), 2.786187718`, 0.2025`, 1, 
              0.4`, \(-0.7575354194`\), 10.06798332`, 
              0.2879845469`, \(-2.134379038`\), 3, 4, 5, 6, 
              0.470035`, \(-0.445838`\), \(-0.271954`\), 0.233327`, 
              0.02276`, \(-0.026364`\), 0.921137`, 0.454656`, 0.875547`, 
              0.748483`, 1.145333`, 1.266709`, 16.5242`, 16.552424`, 
              0.591397`, 22.507251`, 22.547844`, 0.850572`, 31.015015`, 
              31.034315`, 0.404422`, 35.718211`, 35.690653`, \(-0.577424`\), 
              14.592158`, 14.602474`, 0.216157`, 31.674654`, 31.681584`, 
              0.145201`, 34.578967`, 34.599314`, 0.426354`, 42.967234`, 
              42.941247`, \(-0.544543`\), 16.5242`, 22.507251`, 31.015015`, 
              35.718211`, 42.967234`, 49.60373`, 61.153843`, 61.308343`, 
              14.592158`, 31.674654`, 34.578967`, 47.19611`, 38.512539`, 
              50.80708`, 53.538823`, 57.264544`}, {5, 26, 
              21547.86413`, \(-0.25`\), \(-12.90005839`\), 0.25`, 
              1, \(-0.6`\), \(-0.2832285412`\), 
              5.408493055`, \(-4.265785168`\), \(-6.991834716`\), 3, 4, 5, 6, 
              0.481089`, \(-1.112091`\), 1.149693`, \(-0.038712`\), 
              0.101433`, \(-0.057078`\), 0.934482`, \(-0.251325`\), 
              0.099233`, \(-0.264356`\), \(-0.265279`\), \(-0.300928`\), 
              16.5242`, 16.461892`, 
              6.187015`, \(-48.723609`\), \(-48.672271`\), \(-5.097681`\), \
\(-53.932528`\), \(-54.004903`\), 
              7.186671`, \(-103.285441`\), \(-103.336942`\), 
              5.113887`, \(-70.707566`\), \(-70.751779`\), 
              4.390205`, \(-120.543067`\), \(-120.720336`\), 
              17.602339`, \(-140.272701`\), \(-140.188213`\), \(-8.389444`\), \
\(-141.121815`\), \(-141.029349`\), \(-9.181691`\), 
              
              16.5242`, \(-48.723609`\), \(-53.932528`\), \(-103.285441`\), \
\(-151.572313`\), \(-188.918459`\), \(-227.885894`\), \(-237.575736`\), \
\(-70.707566`\), \(-120.543067`\), \(-140.272701`\), \(-141.121815`\), \
\(-105.877173`\), \(-162.094559`\), \(-151.262846`\), \(-162.983681`\)}, {2, 
              22, 21.7549659`, \(-0.7380321377`\), 4.554094781`, 0.25`, 
              1, \(-0.5`\), \(-0.2687758081`\), 
              60.27357208`, \(-0.246622416`\), \(-1.472358066`\), 3, 4, 5, 6, 
              0.576977`, \(-0.396478`\), \(-0.18911`\), 0.232742`, 
              0.024602`, \(-0.027298`\), 0.970077`, 0.82182`, 1.04229`, 
              1.011562`, 1.406388`, 1.43518`, 16.5242`, 16.548748`, 
              1.032056`, 32.872505`, 32.857926`, \(-0.612899`\), 33.643203`, 
              33.641778`, \(-0.05991`\), 51.290262`, 51.292219`, 0.082253`, 
              26.816672`, 26.827272`, 0.445637`, 54.234005`, 54.25455`, 
              0.863754`, 54.298392`, 54.305669`, 0.305939`, 62.729566`, 
              62.698904`, \(-1.289098`\), 16.5242`, 32.872505`, 33.643203`, 
              56.068338`, 65.432463`, 77.862874`, 94.371072`, 95.573104`, 
              26.816672`, 54.298392`, 54.234005`, 62.729566`, 51.290262`, 
              69.764728`, 78.074033`, 79.073207`}, {5, 29, 
              66.8724909`, \(-1.037667446`\), 3.122504197`, 0.25`, 
              1, \(-0.4`\), \(-0.3252717473`\), 13.32358155`, 
              0.09225368198`, \(-1.749085514`\), 3, 4, 5, 6, 
              0.602822`, \(-0.324836`\), \(-0.378504`\), 0.235423`, 
              0.020388`, \(-0.023838`\), 0.986845`, 0.581305`, 1.021679`, 
              0.833393`, 1.362788`, 1.324285`, 16.5242`, 16.550821`, 
              0.801735`, 23.398709`, 23.405058`, 0.191217`, 27.413279`, 
              27.390203`, \(-0.694956`\), 37.831722`, 37.845028`, 0.400723`, 
              18.904077`, 18.911371`, 0.219676`, 38.137027`, 38.141422`, 
              0.132365`, 40.571491`, 40.587008`, 0.467345`, 48.037542`, 
              47.976757`, \(-1.83065`\), 16.5242`, 27.413279`, 23.398709`, 
              37.831722`, 48.433132`, 56.968414`, 69.3661`, 69.749436`, 
              18.904077`, 38.137027`, 40.571491`, 48.037542`, 39.521652`, 
              52.896072`, 58.364311`, 61.708481`}, {5, 28, 
              59.40974253`, \(-1.222542137`\), 3.095408038`, 0.25`, 
              1, \(-0.3`\), \(-0.4055101144`\), 7.100819262`, 
              0.4715637412`, \(-1.963036691`\), 3, 4, 5, 6, 
              0.606338`, \(-0.3167`\), \(-0.450629`\), 0.232626`, 
              0.023964`, \(-0.027279`\), 0.995382`, 0.573352`, 1.00159`, 
              0.823951`, 1.324194`, 1.310277`, 16.5242`, 16.557266`, 
              0.992965`, 24.07315`, 24.092144`, 0.570383`, 31.073736`, 
              31.043006`, \(-0.922819`\), 33.427679`, 33.448539`, 0.626414`, 
              18.929799`, 18.937758`, 0.239024`, 38.162748`, 38.167776`, 
              0.150988`, 41.93219`, 41.949295`, 0.513668`, 50.07653`, 
              50.036892`, \(-1.190307`\), 16.5242`, 24.07315`, 31.073736`, 
              33.427679`, 50.07653`, 58.541401`, 71.07397`, 71.766942`, 
              18.929799`, 38.162748`, 41.93219`, 51.002802`, 41.877071`, 
              55.755606`, 60.26566`, 64.098079`}, {5, 28, 
              73.17838465`, \(-1.357973063`\), 2.977000228`, 0.25`, 
              1, \(-0.2`\), \(-0.4802604441`\), 6.834816078`, 
              0.3634490872`, \(-2.070431604`\), 3, 4, 5, 6, 
              0.605145`, \(-0.322926`\), \(-0.485585`\), 0.234076`, 
              0.021931`, \(-0.025262`\), 0.999352`, 0.553104`, 0.991132`, 
              0.812201`, 1.30433`, 1.307167`, 16.5242`, 16.5586`, 0.991548`, 
              24.537957`, 24.563866`, 0.74681`, 32.170303`, 32.187834`, 
              0.505318`, 33.088455`, 33.059931`, \(-0.822184`\), 18.245909`, 
              18.254156`, 0.237694`, 36.829289`, 36.834691`, 0.15573`, 
              41.349084`, 41.366992`, 0.516181`, 49.487308`, 
              49.459104`, \(-0.812982`\), 16.5242`, 24.537957`, 32.170303`, 
              33.088455`, 49.487308`, 57.591832`, 69.813683`, 70.480046`, 
              18.245909`, 36.829289`, 41.349084`, 51.333036`, 42.137371`, 
              55.832209`, 59.588872`, 63.609453`}, {5, 28, 
              87.88410155`, \(-1.454836975`\), 2.690335699`, 0.25`, 
              1, \(-0.1`\), \(-0.5539771315`\), 9.178351665`, 
              0.1738578669`, \(-2.101723061`\), 3, 4, 5, 6, 
              0.597704`, \(-0.334853`\), \(-0.503239`\), 0.235332`, 
              0.020379`, \(-0.023743`\), 0.999397`, 0.498438`, 0.986626`, 
              0.778399`, 1.296687`, 1.2904`, 16.5242`, 16.553123`, 0.744142`, 
              23.036293`, 23.063915`, 0.710656`, 31.017705`, 31.026689`, 
              0.231143`, 32.313309`, 32.291167`, \(-0.569674`\), 16.414565`, 
              16.422562`, 0.205755`, 33.240216`, 33.245558`, 0.137439`, 
              37.768947`, 37.786428`, 0.449756`, 45.392694`, 
              45.367675`, \(-0.643705`\), 16.5242`, 23.036293`, 31.017705`, 
              32.313309`, 45.392694`, 52.644922`, 63.80292`, 64.2715`, 
              16.414565`, 33.240216`, 37.768947`, 47.743963`, 39.182405`, 
              51.699568`, 54.780627`, 58.583235`}, {5, 28, 
              104.7767012`, \(-1.543207979`\), 2.538255945`, 0.25`, 1, 
              0, \(-0.6218992064`\), 11.93598228`, 
              0.4865394258`, \(-2.135810888`\), 3, 4, 5, 6, 
              0.587285`, \(-0.348477`\), \(-0.508732`\), 0.23269`, 
              0.023746`, \(-0.027052`\), 0.996618`, 0.462462`, 0.977956`, 
              0.752275`, 1.283369`, 1.267094`, 16.5242`, 16.549094`, 
              0.593737`, 22.229991`, 22.260112`, 0.718399`, 30.647727`, 
              30.650559`, 0.067548`, 32.319601`, 32.300241`, \(-0.461754`\), 
              15.346654`, 15.354639`, 0.190445`, 31.222848`, 31.228225`, 
              0.128226`, 35.713686`, 35.731176`, 0.417161`, 43.127251`, 
              43.102717`, \(-0.585143`\), 16.5242`, 22.229991`, 30.647727`, 
              32.319601`, 43.127251`, 49.869517`, 60.483282`, 60.759401`, 
              15.346654`, 31.222848`, 35.713686`, 45.850029`, 37.640234`, 
              49.463763`, 52.212907`, 55.867166`}, {5, 28, 
              111.1580024`, \(-1.604219818`\), 2.507451303`, 0.25`, 1, 
              0.1`, \(-0.6930380969`\), 13.23049071`, 
              0.6601194211`, \(-2.188522867`\), 3, 4, 5, 6, 
              0.569861`, \(-0.362786`\), \(-0.510804`\), 0.229834`, 
              0.027551`, \(-0.030763`\), 0.990567`, 0.44634`, 0.962143`, 
              0.73844`, 1.25944`, 1.250555`, 16.5242`, 16.548776`, 0.561858`, 
              22.055403`, 22.088957`, 0.767131`, 30.732393`, 30.73961`, 
              0.164983`, 33.040039`, 33.015021`, \(-0.571987`\), 14.934052`, 
              14.942462`, 0.192287`, 30.593183`, 30.598824`, 0.128971`, 
              34.951141`, 34.969522`, 0.420239`, 42.506928`, 
              42.480855`, \(-0.596092`\), 16.5242`, 22.055403`, 30.732393`, 
              33.040039`, 42.506928`, 49.034408`, 59.578802`, 59.639744`, 
              14.934052`, 30.593183`, 34.951141`, 45.822421`, 37.419644`, 
              49.026124`, 51.702323`, 55.279177`}, {5, 28, 
              121.2426107`, \(-1.655383952`\), 2.544465276`, 0.25`, 1, 
              0.2`, \(-0.7570511874`\), 
              12.43348227`, \(-1.289395471`\), \(-2.235215001`\), 3, 4, 5, 6, 
              0.55137`, \(-0.377486`\), \(-0.503146`\), 0.203439`, 
              0.066443`, \(-0.067755`\), 0.982604`, 0.414954`, 0.944562`, 
              0.682326`, 1.233397`, 1.14116`, 16.5242`, 16.550237`, 
              0.584452`, 22.351085`, 22.388591`, 0.841884`, 30.82568`, 
              30.839764`, 0.316158`, 34.264474`, 34.2341`, \(-0.681803`\), 
              14.88039`, 14.889357`, 0.201297`, 30.733352`, 30.739314`, 
              0.133829`, 34.986477`, 35.005968`, 0.437526`, 42.850543`, 
              42.822342`, \(-0.633025`\), 16.5242`, 22.351085`, 30.82568`, 
              34.264474`, 42.850543`, 49.316311`, 59.930658`, 60.066017`, 
              14.88039`, 30.733352`, 34.986477`, 46.527978`, 38.005905`, 
              49.656124`, 52.387327`, 55.932658`}, {5, 28, 
              119.0095115`, \(-1.685859957`\), 2.627674157`, 0.25`, 1, 
              0.3`, \(-0.8131773553`\), 10.65141015`, 
              1.739998296`, \(-2.280222347`\), 3, 4, 5, 6, 
              0.532518`, \(-0.392222`\), \(-0.486443`\), 0.192855`, 
              0.070416`, \(-0.06876`\), 0.973113`, 0.404359`, 0.925082`, 
              0.65622`, 1.204815`, 1.090331`, 16.5242`, 16.553257`, 
              0.650547`, 22.980187`, 23.021984`, 0.935789`, 30.84344`, 
              30.864899`, 0.480425`, 35.925344`, 35.891658`, \(-0.754178`\), 
              15.04877`, 15.058365`, 0.214835`, 31.374337`, 31.380642`, 
              0.14116`, 35.540174`, 35.560873`, 0.463416`, 43.823233`, 
              43.79265`, \(-0.684688`\), 16.5242`, 22.980187`, 30.84344`, 
              35.925344`, 43.823233`, 50.318872`, 61.15484`, 61.459442`, 
              15.04877`, 31.374337`, 35.540174`, 47.69632`, 39.133851`, 
              51.002222`, 53.865617`, 57.405961`}, {5, 28, 
              92.92737099`, \(-1.669628706`\), 2.694568044`, 0.25`, 1, 
              0.4`, \(-0.8691358119`\), 
              9.905247312`, \(-0.006952082833`\), \(-2.338489726`\), 3, 4, 5, 
              6, 0.509905`, \(-0.408702`\), \(-0.462834`\), 0.235269`, 
              0.020481`, \(-0.023928`\), 0.961395`, 0.460733`, 0.900494`, 
              0.755104`, 1.168484`, 1.276456`, 16.5242`, 16.555798`, 
              0.694641`, 23.246724`, 23.29223`, 1.000396`, 30.914153`, 
              30.940057`, 0.569459`, 37.203645`, 37.168773`, \(-0.766603`\), 
              15.043913`, 15.054122`, 0.224424`, 31.72888`, 31.735477`, 
              0.145042`, 35.613099`, 35.634909`, 0.479461`, 44.270801`, 
              44.237814`, \(-0.72518`\), 16.5242`, 23.246724`, 30.914153`, 
              37.203645`, 44.270801`, 50.724662`, 61.712557`, 62.152595`, 
              15.043913`, 31.72888`, 35.613099`, 48.44807`, 39.765258`, 
              51.73226`, 54.768741`, 58.213174`}, {2, 35, 
              6.463757838`, \(-1.586701318`\), 7.388765066`, 0, 
              1, \(-0.006118468428`\), \(-0.1223884029`\), 
              1328.031904`, \(-0.6191892896`\), \(-0.6448791526`\), 2, 3, 4, 
              5, 6, 0.185084`, \(-0.42534`\), 0.259678`, 0.136324`, 
              0.232599`, \(-0.226242`\), 0.619684`, 0.594766`, 0.713417`, 
              0.727959`, 1.010486`, 1.002122`, 16.5242`, 16.572037`, 
              1.046701`, 27.380553`, 27.349783`, \(-0.673259`\), 34.840339`, 
              34.866382`, 0.569836`, 44.774649`, 44.782409`, 0.169787`, 
              23.311982`, 23.346377`, 0.752579`, 44.035034`, 44.073939`, 
              0.851261`, 58.218027`, 58.22371`, 0.124351`, 62.284301`, 
              62.322227`, 0.82985`, 16.5242`, 27.380553`, 34.840339`, 
              49.042912`, 58.218027`, 74.724059`, 88.685196`, 95.269282`, 
              23.311982`, 44.035034`, 62.284301`, 64.985903`, 44.774649`, 
              65.467469`, 72.9797`, 79.566949`}, {1, 56, 
              7.239690877`, \(-1.383769551`\), 7.207947949`, 0.001967542671`, 
              1, \(-0.05307870473`\), \(-0.1141404246`\), 
              2798.874143`, \(-0.7012994607`\), \(-0.7524379213`\), 2, 3, 4, 
              5, 6, 0.216659`, \(-0.451172`\), 0.293863`, 0.140002`, 
              0.219449`, \(-0.211372`\), 0.6549`, 0.626164`, 0.738868`, 
              0.75471`, 1.036536`, 1.027345`, 16.5242`, 16.566389`, 
              1.054161`, 26.394095`, 26.362216`, \(-0.796553`\), 35.613128`, 
              35.636899`, 0.593958`, 45.3781`, 45.384255`, 0.153795`, 
              24.599292`, 24.629529`, 0.755535`, 45.803578`, 45.836475`, 
              0.821997`, 59.814636`, 59.814706`, 0.001756`, 63.377198`, 
              63.396923`, 0.492859`, 16.5242`, 26.394095`, 35.613128`, 
              49.570474`, 59.814636`, 76.240965`, 89.49991`, 98.873036`, 
              24.599292`, 45.803578`, 63.377198`, 65.186077`, 45.3781`, 
              64.81508`, 73.468004`, 77.227451`}, {1, 38, 
              8.111494715`, \(-1.24167337`\), 6.927609382`, 0.007919222016`, 
              1, \(-0.08722034695`\), \(-0.1182137623`\), 
              13804.19238`, \(-0.7712155125`\), \(-0.844072778`\), 2, 3, 4, 
              5, 6, 0.24557`, \(-0.473244`\), 0.317899`, 0.146276`, 
              0.201536`, \(-0.193235`\), 0.686032`, 0.647412`, 0.763995`, 
              0.777242`, 1.064574`, 1.056856`, 16.5242`, 16.562934`, 
              1.054328`, 26.289197`, 26.258345`, \(-0.839767`\), 35.828446`, 
              35.848934`, 0.55768`, 45.937088`, 45.942135`, 0.137375`, 
              25.208832`, 25.235818`, 0.734528`, 46.90663`, 46.936194`, 
              0.804725`, 60.647863`, 60.644706`, \(-0.085926`\), 63.04669`, 
              63.054367`, 0.208952`, 16.5242`, 26.289197`, 35.828446`, 
              49.606937`, 60.647863`, 76.829715`, 91.117225`, 99.720468`, 
              25.208832`, 46.90663`, 63.04669`, 65.013161`, 45.937088`, 
              65.826893`, 74.642457`, 75.896437`}, {1, 69, 
              8.341489309`, \(-1.496426973`\), 7.058025858`, 0.01800590693`, 
              1, \(-0.1100994274`\), \(-0.08537929229`\), 
              2.019952108`, \(-0.9794565415`\), \(-0.9904684232`\), 2, 3, 4, 
              5, 6, 0.306315`, \(-0.461412`\), 0.271927`, 0.142445`, 
              0.194826`, \(-0.182518`\), 0.722984`, 0.704537`, 0.787083`, 
              0.816104`, 1.08671`, 1.069876`, 16.5242`, 16.554804`, 
              1.058647`, 25.831813`, 25.847171`, 0.531263`, 30.0951`, 
              30.076875`, \(-0.63045`\), 48.179144`, 48.203142`, 0.830135`, 
              27.945079`, 27.962181`, 0.591614`, 54.936827`, 54.958023`, 
              0.733228`, 67.330158`, 67.317602`, \(-0.434318`\), 68.114692`, 
              68.117218`, 0.087371`, 16.5242`, 25.831813`, 30.0951`, 
              48.179144`, 68.114692`, 85.217886`, 103.263654`, 109.474377`, 
              27.945079`, 54.936827`, 67.330158`, 70.719656`, 52.920059`, 
              75.548947`, 81.982874`, 87.241563`}, {1, 63, 
              9.4115474`, \(-1.019658861`\), 6.830897546`, 0.03249196962`, 
              1, \(-0.1673942526`\), \(-0.1209521116`\), 
              9.419471443`, \(-1.022668024`\), \(-1.098150327`\), 2, 3, 4, 5, 
              6, 0.312142`, \(-0.503509`\), 0.340742`, 0.148937`, 
              0.174328`, \(-0.158249`\), 0.745152`, 0.701267`, 0.799576`, 
              0.819145`, 1.097323`, 1.085996`, 16.5242`, 16.554124`, 
              1.020854`, 28.091265`, 28.067613`, \(-0.806888`\), 35.515801`, 
              35.530583`, 0.504295`, 50.317778`, 50.320959`, 0.108508`, 
              28.033306`, 28.054764`, 0.732042`, 52.540276`, 52.565648`, 
              0.865575`, 66.250188`, 66.227542`, \(-0.772563`\), 66.441621`, 
              66.453898`, 0.418825`, 16.5242`, 28.091265`, 35.515801`, 
              52.804391`, 66.441621`, 83.148406`, 101.037201`, 106.653823`, 
              28.033306`, 52.540276`, 66.250188`, 68.531852`, 50.317778`, 
              71.040614`, 80.378896`, 85.059709`}, {1, 63, 
              8.300556925`, \(-0.9048407402`\), 6.663942561`, 0.0517766953`, 
              1, \(-0.2097078591`\), \(-0.1388002796`\), 
              1425.934394`, \(-1.115421576`\), \(-1.217696578`\), 2, 3, 4, 5, 
              6, 0.34115`, \(-0.515685`\), 0.335753`, 0.153619`, 
              0.157164`, \(-0.139513`\), 0.774798`, 0.71925`, 0.820095`, 
              0.837648`, 1.118621`, 1.108539`, 16.5242`, 16.552588`, 
              1.032611`, 29.060814`, 29.036915`, \(-0.869315`\), 39.159538`, 
              39.173223`, 0.497783`, 51.578301`, 51.581464`, 0.11506`, 
              28.820904`, 28.84156`, 0.751338`, 54.038504`, 54.064413`, 
              0.942446`, 66.657148`, 66.646134`, \(-0.40062`\), 68.10033`, 
              68.106625`, 0.228984`, 16.5242`, 29.060814`, 39.159538`, 
              55.197298`, 68.10033`, 84.728219`, 104.302178`, 108.18104`, 
              28.820904`, 54.038504`, 66.657148`, 69.19293`, 51.578301`, 
              69.916344`, 84.799791`, 86.95024`}, {2, 46, 
              9.332606961`, \(-0.9032653951`\), 6.038734053`, 0.07642881742`, 
              1, \(-0.2457236507`\), \(-0.1582036385`\), 
              13.76009681`, \(-1.020672186`\), \(-1.17477871`\), 2, 3, 4, 5, 
              6, 0.38074`, \(-0.497446`\), 0.256279`, 0.173686`, 
              0.12641`, \(-0.117167`\), 0.809578`, 0.742145`, 0.874785`, 
              0.879646`, 1.196608`, 1.190676`, 16.5242`, 16.552468`, 
              1.039882`, 29.580266`, 29.557503`, \(-0.837379`\), 35.352203`, 
              35.362595`, 0.382287`, 50.770367`, 50.773093`, 0.100261`, 
              27.741169`, 27.758585`, 0.640658`, 53.140517`, 53.164808`, 
              0.893566`, 62.707317`, 62.706009`, \(-0.048114`\), 66.230924`, 
              66.238472`, 0.277679`, 16.5242`, 29.580266`, 35.352203`, 
              54.743833`, 66.230924`, 81.612361`, 93.952056`, 104.929212`, 
              27.741169`, 53.140517`, 62.707317`, 66.414511`, 50.770367`, 
              71.506664`, 76.527348`, 87.685913`}, {5, 100, 
              11.70458489`, \(-0.7821342608`\), 6.360673415`, 0.1072401379`, 
              1, \(-0.3119820943`\), \(-0.164496886`\), 
              528.4818481`, \(-1.318213571`\), \(-1.469103933`\), 2, 3, 4, 5, 
              6, 0.41988`, \(-0.504512`\), 0.239424`, 0.16275`, 
              0.124816`, \(-0.10569`\), 0.841072`, 0.769966`, 0.869807`, 
              0.887436`, 1.173127`, 1.163093`, 16.5242`, 16.547691`, 
              1.003822`, 32.209406`, 32.189642`, \(-0.844522`\), 40.768135`, 
              40.778095`, 0.425623`, 55.64378`, 55.646252`, 0.105628`, 
              30.819212`, 30.836067`, 0.720261`, 58.932365`, 58.958675`, 
              1.124237`, 67.889033`, 67.887489`, \(-0.066002`\), 71.583424`, 
              71.562148`, \(-0.90917`\), 16.5242`, 32.209406`, 40.768135`, 
              60.430885`, 72.989269`, 89.445591`, 104.424399`, 111.969634`, 
              30.819212`, 58.932365`, 67.889033`, 71.583424`, 55.64378`, 
              79.346991`, 79.680021`, 95.792696`}, {3, 79, 
              15.75843486`, \(-0.7737100304`\), 5.574004866`, 0.1453085056`, 
              1, \(-0.3617587894`\), \(-0.1972666616`\), 
              3852.154422`, \(-1.105785389`\), \(-1.421998508`\), 2, 3, 4, 5, 
              6, 0.464625`, \(-0.475019`\), 0.11878`, 0.188326`, 
              0.090637`, \(-0.081424`\), 0.880628`, 0.784738`, 0.930756`, 
              0.929269`, 1.259439`, 1.260317`, 16.5242`, 16.548423`, 
              1.003718`, 31.813859`, 31.793283`, \(-0.852617`\), 37.634518`, 
              37.642286`, 0.321883`, 53.024739`, 53.027062`, 0.09626`, 
              28.733076`, 28.747416`, 0.594232`, 56.291567`, 56.321557`, 
              1.242738`, 61.428382`, 61.425642`, \(-0.113536`\), 66.986073`, 
              66.961133`, \(-1.033462`\), 16.5242`, 31.813859`, 37.634518`, 
              58.406044`, 68.803032`, 83.484257`, 102.244134`, 103.297372`, 
              28.733076`, 56.291567`, 61.428382`, 66.986073`, 53.024739`, 
              75.706051`, 77.217561`, 82.353405`}, {2, 40, 
              28.80204336`, \(-0.787267923`\), 4.943818432`, 0.1921681542`, 
              1, \(-0.4234107898`\), \(-0.217082125`\), 
              24.12743121`, \(-0.7494070055`\), \(-1.360576748`\), 2, 3, 4, 
              5, 6, 0.524957`, \(-0.424866`\), \(-0.044819`\), 0.214489`, 
              0.0533`, \(-0.052378`\), 0.924397`, 0.806973`, 0.997695`, 
              0.978225`, 1.352787`, 1.364031`, 16.5242`, 16.5461`, 0.909383`, 
              31.294242`, 31.293652`, \(-0.024476`\), 34.967954`, 
              34.956354`, \(-0.481684`\), 51.602385`, 51.604172`, 0.074181`, 
              27.283185`, 27.294503`, 0.469993`, 55.609038`, 55.613993`, 
              0.205745`, 55.897899`, 55.918951`, 0.874182`, 63.727049`, 
              63.701923`, \(-1.043341`\), 16.5242`, 31.294242`, 34.967954`, 
              55.755055`, 66.192934`, 79.421504`, 97.57947`, 98.444182`, 
              27.283185`, 55.897899`, 55.609038`, 63.727049`, 51.602385`, 
              71.602448`, 78.046515`, 78.220387`}, {2, 73, 
              14.3933885`, \(-7.263677677`\), 22.70260985`, 0.25`, 1, 
              0.01307421984`, 0.007134481757`, \(-2.04170849`\), 
              3.964286687`, \(-2.725204359`\), 2, 3, 4, 5, 6, 
              0.808058`, \(-0.388575`\), \(-0.140322`\), 0.039809`, 
              0.27714`, \(-0.293726`\), 1.025231`, 1.075153`, 0.930244`, 
              1.081333`, 1.186937`, 1.099594`, 16.5242`, 16.527643`, 
              1.010615`, 207.659675`, 207.676025`, 4.799179`, 285.04826`, 
              285.046185`, \(-0.60914`\), 285.165255`, 285.166942`, 
              0.495121`, 146.822079`, 146.82526`, 0.93377`, 290.846822`, 
              290.848907`, 0.611772`, 388.324498`, 388.32528`, 0.229791`, 
              433.487909`, 433.489772`, 0.546701`, 16.5242`, 285.04826`, 
              207.659675`, 285.165255`, 434.230148`, 494.915`, 594.567468`, 
              605.63102`, 146.822079`, 290.846822`, 388.324498`, 433.487909`, 
              385.255065`, 486.340691`, 514.395497`, 
              549.519272`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Contour plot", "Subsection"],

Cell[BoxData[
    \(\(Length[
        data = Map[{Sqrt[#[\([6]\)]], #[\([8]\)], #[\([3]\)]} &, 
            datal1[\([2]\)]]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[
      d2 = Map[Sort[#, Function[{a, b}, OrderedQ[{Rest[a], Rest[b]}]]] &, 
          Partition[Sort[data], 12]]]\)], "Input"],

Cell[BoxData[
    \({11, 12, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ColumnForm[d2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\({{0, \(-0.6`\), 8.505592857`*^7}, {0, \(-0.5`\), 
                4.598422702`*^7}, {0, \(-0.4`\), 
                2.030514653`*^7}, {0, \(-0.3`\), 
                6.734188685`*^6}, {0, \(-0.2`\), 
                1.001541234`*^6}, {0, \(-0.1`\), 
                8240.090929`}, {0, \(-0.006118468428`\), 6.463757838`}, {0, 
                0, 6.48760648`}, {0, 0.1`, 16.8599107`}, {0, 0.2`, 
                64.38017244`}, {0, 0.3`, 76.56907708`}, {0, 0.4`, 
                17.8919222`}}\)},
          {\({{0.05`, \(-0.6`\), 5.956982876`*^7}, {0.05`, \(-0.5`\), 
                3.034167278`*^7}, {0.05`, \(-0.4`\), 
                1.238398414`*^7}, {0.05`, \(-0.3`\), 
                3.475369645`*^6}, {0.05`, \(-0.2`\), 
                307828.5616`}, {0.05`, \(-0.1`\), 
                16890.51171`}, {0.04435699123024464`, \(-0.05307870473`\), 
                7.239690877`}, {0.05`, 0, 12.19410144`}, {0.05`, 0.1`, 
                30.89468027`}, {0.05`, 0.2`, 80.01482926`}, {0.05`, 0.3`, 
                49.57830722`}, {0.05`, 0.4`, 71.59242833`}}\)},
          {\({{0.1`, \(-0.6`\), 4.004243597`*^7}, {0.1`, \(-0.5`\), 
                1.927266866`*^7}, {0.1`, \(-0.4`\), 
                6.159214505`*^6}, {0.1`, \(-0.3`\), 
                1.483867549`*^6}, {0.1`, \(-0.2`\), 
                48779.59938`}, {0.1`, \(-0.1`\), 
                8.622903251`}, {0.08899001076525387`, \(-0.08722034695`\), 
                8.111494715`}, {0.1`, 0, 42.6448837`}, {0.1`, 0.1`, 
                37.48072622`}, {0.1`, 0.2`, 14.77855901`}, {0.1`, 0.3`, 
                59.29904956`}, {0.1`, 0.4`, 47.6972641`}}\)},
          {\({{0.15`, \(-0.6`\), 2.375440325`*^7}, {0.15`, \(-0.5`\), 
                1.153425887`*^7}, {0.15`, \(-0.4`\), 
                3.704559742`*^6}, {0.15`, \(-0.3`\), 
                570104.9787`}, {0.15`, \(-0.2`\), 
                944183.2755`}, {0.13418609067261777`, \(-0.1100994274`\), 
                8.341489309`}, {0.15`, \(-0.1`\), 9.803800087`}, {0.15`, 0, 
                27.16521891`}, {0.15`, 0.1`, 56.40087264`}, {0.15`, 0.2`, 
                54.99684063`}, {0.15`, 0.3`, 78.0546061`}, {0.15`, 0.4`, 
                83.55621262`}}\)},
          {\({{0.2`, \(-0.6`\), 1.59996609`*^7}, {0.2`, \(-0.5`\), 
                6.542829808`*^6}, {0.2`, \(-0.4`\), 
                1.695337519`*^6}, {0.2`, \(-0.3`\), 
                121377.2592`}, {0.2`, \(-0.2`\), 
                11.31459032`}, {0.18025529013041475`, \(-0.1673942526`\), 
                9.4115474`}, {0.2`, \(-0.1`\), 59.60335535`}, {0.2`, 0, 
                46.09400111`}, {0.2`, 0.1`, 47.75713647`}, {0.2`, 0.2`, 
                88.24952787`}, {0.2`, 0.3`, 89.90412399`}, {0.2`, 0.4`, 
                106.7966592`}}\)},
          {\({{0.25`, \(-0.6`\), 8.705607198`*^6}, {0.25`, \(-0.5`\), 
                3.323729121`*^6}, {0.25`, \(-0.4`\), 
                658737.2606`}, {0.25`, \(-0.3`\), 
                157657.2688`}, {0.22754493028850367`, \(-0.2097078591`\), 
                8.300556925`}, {0.25`, \(-0.2`\), 
                11.98900331`}, {0.25`, \(-0.1`\), 71.56251423`}, {0.25`, 0, 
                46.55711748`}, {0.25`, 0.1`, 53.3412934`}, {0.25`, 0.2`, 
                63.55335858`}, {0.25`, 0.3`, 71.28152917`}, {0.25`, 0.4`, 
                117.2942917`}}\)},
          {\({{0.3`, \(-0.6`\), 4.546208295`*^6}, {0.3`, \(-0.5`\), 
                1.445172391`*^6}, {0.3`, \(-0.4`\), 
                174931.8694`}, {0.3`, \(-0.3`\), 
                14.18625471`}, {0.2764576231902459`, \(-0.2457236507`\), 
                9.332606961`}, {0.3`, \(-0.2`\), 
                19.64400675`}, {0.3`, \(-0.1`\), 37.07474059`}, {0.3`, 0, 
                50.00374924`}, {0.3`, 0.1`, 63.73924935`}, {0.3`, 0.2`, 
                70.45500751`}, {0.3`, 0.3`, 126.2416685`}, {0.3`, 0.4`, 
                134.1485176`}}\)},
          {\({{0.35`, \(-0.6`\), 1.723850258`*^6}, {0.35`, \(-0.5`\), 
                408558.0441`}, {0.35`, \(-0.4`\), 
                10554.29463`}, {0.3274754004501712`, \(-0.3119820943`\), 
                11.70458489`}, {0.35`, \(-0.3`\), 
                17.72684632`}, {0.35`, \(-0.2`\), 
                39.22692648`}, {0.35`, \(-0.1`\), 99.77966453`}, {0.35`, 0, 
                42.14434238`}, {0.35`, 0.1`, 134.5188684`}, {0.35`, 0.2`, 
                76.82579997`}, {0.35`, 0.3`, 78.45179439`}, {0.35`, 0.4`, 
                77.97563945`}}\)},
          {\({{0.4`, \(-0.6`\), 866579.3926`}, {0.4`, \(-0.5`\), 
                68833.30004`}, {0.4`, \(-0.4`\), 
                27.03428549`}, {0.38119352775198057`, \(-0.3617587894`\), 
                15.75843486`}, {0.4`, \(-0.3`\), 
                38.99928338`}, {0.4`, \(-0.2`\), 
                37.83683621`}, {0.4`, \(-0.1`\), 118.0661215`}, {0.4`, 0, 
                73.14441445`}, {0.4`, 0.1`, 83.39748003`}, {0.4`, 0.2`, 
                89.05170378`}, {0.4`, 0.3`, 90.32616987`}, {0.4`, 0.4`, 
                88.95488433`}}\)},
          {\({{0.45`, \(-0.6`\), 186111.8218`}, {0.45`, \(-0.5`\), 
                110.7178352`}, {0.43836988286149403`, \(-0.4234107898`\), 
                28.80204336`}, {0.45`, \(-0.4`\), 
                26.63225064`}, {0.45`, \(-0.3`\), 
                77.43742101`}, {0.45`, \(-0.2`\), 
                113.53913`}, {0.45`, \(-0.1`\), 67.31096761`}, {0.45`, 0, 
                85.92445192`}, {0.45`, 0.1`, 96.24204354`}, {0.45`, 0.2`, 
                99.10624641`}, {0.45`, 0.3`, 97.05201176`}, {0.45`, 0.4`, 
                92.79763043`}}\)},
          {\({{0.5`, \(-0.6`\), 21547.86413`}, {0.5`, \(-0.5`\), 
                21.7549659`}, {0.5`, \(-0.4`\), 
                66.8724909`}, {0.5`, \(-0.3`\), 
                59.40974253`}, {0.5`, \(-0.2`\), 
                73.17838465`}, {0.5`, \(-0.1`\), 87.88410155`}, {0.5`, 0, 
                104.7767012`}, {0.5`, 0.01307421984`, 14.3933885`}, {0.5`, 
                0.1`, 111.1580024`}, {0.5`, 0.2`, 121.2426107`}, {0.5`, 0.3`, 
                119.0095115`}, {0.5`, 0.4`, 92.92737099`}}\)}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      ColumnForm[ {{{0, -0.59999999999999998, 85055928.569999993}, {0, -0.5, 
        45984227.020000003}, {0, -0.40000000000000002, 20305146.530000001}, {
        0, -0.29999999999999999, 6734188.6849999996}, {
        0, -0.20000000000000001, 1001541.2340000001}, {
        0, -0.10000000000000001, 8240.090929}, {0, -0.0061184684280000002, 
        6.4637578380000003}, {0, 0, 6.4876064800000002}, {0, 
        0.10000000000000001, 16.8599107}, {0, 0.20000000000000001, 
        64.380172439999996}, {0, 0.29999999999999999, 76.56907708}, {0, 
        0.40000000000000002, 17.8919222}}, {{
        0.050000000000000003, -0.59999999999999998, 59569828.759999998}, {
        0.050000000000000003, -0.5, 30341672.780000001}, {
        0.050000000000000003, -0.40000000000000002, 12383984.140000001}, {
        0.050000000000000003, -0.29999999999999999, 3475369.645}, {
        0.050000000000000003, -0.20000000000000001, 307828.56160000002}, {
        0.050000000000000003, -0.10000000000000001, 16890.511709999999}, {
        0.044356991230244641, -0.053078704730000002, 7.2396908770000001}, {
        0.050000000000000003, 0, 12.194101440000001}, {0.050000000000000003, 
        0.10000000000000001, 30.894680269999999}, {0.050000000000000003, 
        0.20000000000000001, 80.014829259999999}, {0.050000000000000003, 
        0.29999999999999999, 49.578307219999999}, {0.050000000000000003, 
        0.40000000000000002, 71.592428330000004}}, {{
        0.10000000000000001, -0.59999999999999998, 40042435.969999999}, {
        0.10000000000000001, -0.5, 19272668.66}, {
        0.10000000000000001, -0.40000000000000002, 6159214.5049999999}, {
        0.10000000000000001, -0.29999999999999999, 1483867.5490000001}, {
        0.10000000000000001, -0.20000000000000001, 48779.59938}, {
        0.10000000000000001, -0.10000000000000001, 8.6229032510000003}, {
        0.088990010765253869, -0.087220346949999999, 8.1114947149999992}, {
        0.10000000000000001, 0, 42.644883700000001}, {0.10000000000000001, 
        0.10000000000000001, 37.480726220000001}, {0.10000000000000001, 
        0.20000000000000001, 14.77855901}, {0.10000000000000001, 
        0.29999999999999999, 59.29904956}, {0.10000000000000001, 
        0.40000000000000002, 47.697264099999998}}, {{
        0.14999999999999999, -0.59999999999999998, 23754403.25}, {
        0.14999999999999999, -0.5, 11534258.869999999}, {
        0.14999999999999999, -0.40000000000000002, 3704559.7420000001}, {
        0.14999999999999999, -0.29999999999999999, 570104.97869999998}, {
        0.14999999999999999, -0.20000000000000001, 944183.27549999999}, {
        0.13418609067261777, -0.11009942740000001, 8.341489309}, {
        0.14999999999999999, -0.10000000000000001, 9.8038000870000008}, {
        0.14999999999999999, 0, 27.16521891}, {0.14999999999999999, 
        0.10000000000000001, 56.400872640000003}, {0.14999999999999999, 
        0.20000000000000001, 54.996840630000001}, {0.14999999999999999, 
        0.29999999999999999, 78.054606100000001}, {0.14999999999999999, 
        0.40000000000000002, 83.556212619999997}}, {{
        0.20000000000000001, -0.59999999999999998, 15999660.9}, {
        0.20000000000000001, -0.5, 6542829.8080000002}, {
        0.20000000000000001, -0.40000000000000002, 1695337.5190000001}, {
        0.20000000000000001, -0.29999999999999999, 121377.2592}, {
        0.20000000000000001, -0.20000000000000001, 11.314590320000001}, {
        0.18025529013041475, -0.16739425259999999, 9.4115473999999999}, {
        0.20000000000000001, -0.10000000000000001, 59.603355350000001}, {
        0.20000000000000001, 0, 46.094001110000001}, {0.20000000000000001, 
        0.10000000000000001, 47.757136469999999}, {0.20000000000000001, 
        0.20000000000000001, 88.249527869999994}, {0.20000000000000001, 
        0.29999999999999999, 89.904123990000002}, {0.20000000000000001, 
        0.40000000000000002, 106.79665919999999}}, {{
        0.25, -0.59999999999999998, 8705607.1980000008}, {0.25, -0.5, 
        3323729.1209999998}, {0.25, -0.40000000000000002, 
        658737.26060000004}, {0.25, -0.29999999999999999, 
        157657.26879999999}, {0.22754493028850367, -0.2097078591, 
        8.3005569250000004}, {0.25, -0.20000000000000001, 
        11.989003309999999}, {0.25, -0.10000000000000001, 
        71.562514230000005}, {0.25, 0, 46.557117480000002}, {0.25, 
        0.10000000000000001, 53.341293399999998}, {0.25, 0.20000000000000001, 
        63.553358580000001}, {0.25, 0.29999999999999999, 
        71.281529169999999}, {0.25, 0.40000000000000002, 117.2942917}}, {{
        0.29999999999999999, -0.59999999999999998, 4546208.2949999999}, {
        0.29999999999999999, -0.5, 1445172.3910000001}, {
        0.29999999999999999, -0.40000000000000002, 174931.8694}, {
        0.29999999999999999, -0.29999999999999999, 14.18625471}, {
        0.27645762319024592, -0.24572365069999999, 9.3326069609999998}, {
        0.29999999999999999, -0.20000000000000001, 19.644006749999999}, {
        0.29999999999999999, -0.10000000000000001, 37.074740589999998}, {
        0.29999999999999999, 0, 50.003749239999998}, {0.29999999999999999, 
        0.10000000000000001, 63.739249350000001}, {0.29999999999999999, 
        0.20000000000000001, 70.455007510000002}, {0.29999999999999999, 
        0.29999999999999999, 126.2416685}, {0.29999999999999999, 
        0.40000000000000002, 134.14851759999999}}, {{
        0.34999999999999998, -0.59999999999999998, 1723850.2579999999}, {
        0.34999999999999998, -0.5, 408558.0441}, {
        0.34999999999999998, -0.40000000000000002, 10554.29463}, {
        0.32747540045017121, -0.31198209430000001, 11.70458489}, {
        0.34999999999999998, -0.29999999999999999, 17.72684632}, {
        0.34999999999999998, -0.20000000000000001, 39.226926480000003}, {
        0.34999999999999998, -0.10000000000000001, 99.779664530000005}, {
        0.34999999999999998, 0, 42.144342379999998}, {0.34999999999999998, 
        0.10000000000000001, 134.5188684}, {0.34999999999999998, 
        0.20000000000000001, 76.825799970000006}, {0.34999999999999998, 
        0.29999999999999999, 78.451794390000003}, {0.34999999999999998, 
        0.40000000000000002, 77.975639450000003}}, {{
        0.40000000000000002, -0.59999999999999998, 866579.39260000002}, {
        0.40000000000000002, -0.5, 68833.300040000002}, {
        0.40000000000000002, -0.40000000000000002, 27.034285489999998}, {
        0.38119352775198057, -0.3617587894, 15.758434859999999}, {
        0.40000000000000002, -0.29999999999999999, 38.999283380000001}, {
        0.40000000000000002, -0.20000000000000001, 37.836836210000001}, {
        0.40000000000000002, -0.10000000000000001, 118.06612149999999}, {
        0.40000000000000002, 0, 73.144414449999999}, {0.40000000000000002, 
        0.10000000000000001, 83.397480029999997}, {0.40000000000000002, 
        0.20000000000000001, 89.051703779999997}, {0.40000000000000002, 
        0.29999999999999999, 90.326169870000001}, {0.40000000000000002, 
        0.40000000000000002, 88.954884329999999}}, {{
        0.45000000000000001, -0.59999999999999998, 186111.82180000001}, {
        0.45000000000000001, -0.5, 110.7178352}, {
        0.43836988286149403, -0.42341078980000002, 28.802043359999999}, {
        0.45000000000000001, -0.40000000000000002, 26.632250639999999}, {
        0.45000000000000001, -0.29999999999999999, 77.437421009999994}, {
        0.45000000000000001, -0.20000000000000001, 113.53913}, {
        0.45000000000000001, -0.10000000000000001, 67.310967610000006}, {
        0.45000000000000001, 0, 85.924451919999996}, {0.45000000000000001, 
        0.10000000000000001, 96.242043539999997}, {0.45000000000000001, 
        0.20000000000000001, 99.106246409999997}, {0.45000000000000001, 
        0.29999999999999999, 97.052011759999999}, {0.45000000000000001, 
        0.40000000000000002, 92.797630429999998}}, {{
        0.5, -0.59999999999999998, 21547.864130000002}, {0.5, -0.5, 
        21.754965899999998}, {0.5, -0.40000000000000002, 
        66.872490900000003}, {0.5, -0.29999999999999999, 
        59.409742530000003}, {0.5, -0.20000000000000001, 
        73.178384649999998}, {0.5, -0.10000000000000001, 
        87.884101549999997}, {0.5, 0, 104.77670120000001}, {0.5, 
        0.01307421984, 14.3933885}, {0.5, 0.10000000000000001, 111.1580024}, {
        0.5, 0.20000000000000001, 121.2426107}, {0.5, 0.29999999999999999, 
        119.0095115}, {0.5, 0.40000000000000002, 92.92737099}}}],
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppd1 = 
      Block[{theta = 0\ 2\ Pi}, 
        ListSurfacePlot3D[d2, 
          PlotRange \[Rule] {{0, 0.55}, {\(- .55\), 0.3}, {0, 110}}, 
          BoxRatios \[Rule] {1, 1, 0.5}, 
          ViewPoint \[Rule] {\(-1.2\), 1.15, 1.75}, Axes \[Rule] True, 
          AxesLabel \[Rule] {"\<m\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\(\
\[Chi]\^2\)\>\""}, 
          TextStyle \[Rule] {FontSize \[Rule] 
                12}, \[IndentingNewLine]LightSources \[Rule] {{{Cos[theta], 
                  Sin[theta], 1}, 
                Hue[0.1, 0.8, 1]}, {{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                Hue[0.6, 0.8, 1]}} (*\(,\)\(AmbientLight \[Rule] 
              GrayLevel[0.05]\)*) ]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<dvalley1.jpg\>", pppd1, "\<JPEG\>", 
      ImageSize \[Rule] 72*4]\)], "Input"],

Cell[BoxData[
    \("dvalley1.jpg"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppd13 = 
      Block[{theta = Pi/2}, 
        Show[pppd1, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize \[Rule] 12}, 
          AxesLabel \[Rule] None, ColorOutput \[Rule] GrayLevel, 
          FormatType -> OutputForm, 
          LightSources \[Rule] {{{Cos[theta], Sin[theta], 2}, 
                GrayLevel[
                  0.75]} (*\(,\)\({{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                  GrayLevel[0.5]}\)*) }, 
          AmbientLight \[Rule] GrayLevel[0.25]]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<dvalley1.eps\>", pppd13, "\<EPS\>", 
      ImageSize \[Rule] 72*5.5]\)], "Input"],

Cell[BoxData[
    \("dvalley1.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["data l1_bcsan1*.out (This contains points from btraj2*.out)", \
"Subsection"],

Cell[BoxData[
    \(\(datal1 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from minimizing \
chi^2.\n2 parity doublets with fractional errors 1 0.1.\nSpectrum for P_perp \
a = (0)  ( 0.25) \n using (# states, o, multiplet, c^2 error for each) =\n  \
(4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & 1, 0.25 0.25 1 1).\n\
Spectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  (26/2,6)  (32/2,6) \
.\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( 2,20/2,6) \n  ( 2,24/2,4) \
 ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \n  ( 3,35/2,5)  ( \
4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy potential \
determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  ( \
1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{3, 47, 6.369872612`, \(-1.710097193`\), 7.578522352`, 
              0, 1, \(-0.004693500257`\), \(-0.114567563`\), 
              2738.711613`, \(-0.6630839669`\), \(-0.6714368651`\), 2, 3, 4, 
              5, 6, 0.191087`, \(-0.422064`\), 0.253378`, 0.132402`, 
              0.235192`, \(-0.224878`\), 0.621288`, 0.605939`, 0.709826`, 
              0.730517`, 1.003683`, 0.991837`, 17.95654`, 18.001823`, 
              1.049223`, 28.275967`, 28.246208`, \(-0.689542`\), 35.851152`, 
              35.877232`, 0.604289`, 46.617879`, 46.625342`, 0.172928`, 
              24.087495`, 24.120067`, 0.75473`, 46.002772`, 46.040372`, 
              0.871213`, 60.272327`, 60.276033`, 0.085884`, 64.134569`, 
              64.169114`, 0.800426`, 17.95654`, 28.275967`, 35.851152`, 
              51.999544`, 60.272327`, 77.2951`, 91.782172`, 98.074476`, 
              24.087495`, 46.002772`, 64.134569`, 66.994053`, 46.617879`, 
              67.694651`, 75.316992`, 83.250334`}, {1, 53, 
              7.349227765`, \(-1.454957284`\), 7.314588575`, 0.0025`, 
              1, \(-0.05543702533`\), \(-0.1081262544`\), 
              1062.96234`, \(-0.7481706764`\), \(-0.7871380194`\), 2, 3, 4, 
              5, 6, 0.225674`, \(-0.449297`\), 0.287104`, 0.137514`, 
              0.219964`, \(-0.209276`\), 0.660361`, 0.637571`, 0.739298`, 
              0.759729`, 1.034728`, 1.022897`, 17.785889`, 17.825676`, 
              1.050836`, 27.169455`, 27.138941`, \(-0.805913`\), 36.450549`, 
              36.473755`, 0.6129`, 47.002199`, 47.007609`, 0.142865`, 
              25.316896`, 25.345251`, 0.748889`, 47.613518`, 47.644962`, 
              0.830489`, 61.641914`, 61.639827`, \(-0.05513`\), 64.796446`, 
              64.813804`, 0.458461`, 17.785889`, 27.169455`, 36.450549`, 
              52.132068`, 61.641914`, 78.430141`, 92.271577`, 102.098391`, 
              25.316896`, 47.613518`, 64.796446`, 66.823859`, 47.002199`, 
              66.882649`, 76.751054`, 76.926475`}, {2, 43, 
              8.135087732`, \(-1.315172513`\), 7.251136348`, 0.01`, 
              1, \(-0.09526007516`\), \(-0.1061584742`\), 
              3527.743867`, \(-0.886839406`\), \(-0.9286899822`\), 2, 3, 4, 
              5, 6, 0.262328`, \(-0.469771`\), 0.307501`, 0.138867`, 
              0.205737`, \(-0.191172`\), 0.69532`, 0.669999`, 0.76045`, 
              0.784735`, 1.054031`, 1.039964`, 18.756777`, 18.791116`, 
              1.045099`, 28.02928`, 28.001499`, \(-0.845525`\), 38.066859`, 
              38.086895`, 0.609806`, 49.588918`, 49.592895`, 0.121062`, 
              27.020287`, 27.044846`, 0.747439`, 51.002834`, 51.030515`, 
              0.842464`, 65.064881`, 65.056319`, \(-0.260606`\), 66.794897`, 
              66.801071`, 0.187886`, 18.756777`, 28.02928`, 38.066859`, 
              54.573381`, 65.064881`, 82.173756`, 97.701419`, 106.42545`, 
              27.020287`, 51.002834`, 66.794897`, 69.054148`, 49.588918`, 
              70.599356`, 79.57257`, 82.852506`}, {3, 67, 
              8.339396601`, \(-1.409207614`\), 7.64254067`, 0.0225`, 
              1, \(-0.1213305738`\), \(-0.09585391155`\), 
              4205.936202`, \(-1.142357493`\), \(-1.133311771`\), 2, 3, 4, 5, 
              6, 0.313229`, \(-0.464664`\), 0.268646`, 0.13112`, 
              0.201963`, \(-0.181753`\), 0.732891`, 0.720308`, 0.774663`, 
              0.814886`, 1.060319`, 1.037097`, 24.170957`, 24.197983`, 
              1.035141`, 33.352044`, 33.330111`, \(-0.840083`\), 42.534163`, 
              42.551573`, 0.666823`, 58.174656`, 58.177133`, 0.094874`, 
              30.775919`, 30.795345`, 0.744057`, 60.338882`, 60.363153`, 
              0.929623`, 73.660164`, 73.650816`, \(-0.358055`\), 74.929454`, 
              74.933174`, 0.14249`, 24.170957`, 33.352044`, 42.534163`, 
              66.338217`, 74.929454`, 93.559979`, 114.181846`, 119.958127`, 
              30.775919`, 60.338882`, 73.660164`, 77.610039`, 58.174656`, 
              83.702795`, 89.86553`, 95.809009`}, {2, 47, 
              8.133791511`, \(-1.110162749`\), 7.081290114`, 0.04`, 
              1, \(-0.173629971`\), \(-0.1186141863`\), 
              865.4865301`, \(-1.163730934`\), \(-1.21010522`\), 2, 3, 4, 5, 
              6, 0.336106`, \(-0.485057`\), 0.286535`, 0.142898`, 
              0.177012`, \(-0.158381`\), 0.761515`, 0.72508`, 0.802064`, 
              0.830923`, 1.093649`, 1.077004`, 20.46555`, 20.492582`, 
              1.029412`, 31.581239`, 31.558863`, \(-0.852135`\), 40.977767`, 
              40.992552`, 0.563006`, 55.270047`, 55.272771`, 0.103731`, 
              30.006111`, 30.025699`, 0.745926`, 57.695198`, 57.720032`, 
              0.945706`, 70.165117`, 70.157796`, \(-0.278804`\), 71.987414`, 
              71.990164`, 0.104715`, 20.46555`, 31.581239`, 40.977767`, 
              61.043323`, 71.987414`, 89.5252`, 110.471154`, 114.273568`, 
              30.006111`, 57.695198`, 70.165117`, 73.549445`, 55.270047`, 
              75.963954`, 89.363369`, 91.686451`}, {2, 46, 
              8.325858404`, \(-0.8902887089`\), 6.488769654`, 0.0625`, 
              1, \(-0.2285961599`\), \(-0.1467998049`\), 
              1450.518496`, \(-1.135220771`\), \(-1.26481196`\), 2, 3, 4, 5, 
              6, 0.358613`, \(-0.511499`\), 0.310217`, 0.157993`, 
              0.147817`, \(-0.131384`\), 0.790127`, 0.727926`, 0.833048`, 
              0.849389`, 1.133698`, 1.12754`, 16.730731`, 16.758272`, 
              1.025383`, 29.581138`, 29.557926`, \(-0.864222`\), 38.962676`, 
              38.975216`, 0.466911`, 51.934285`, 51.937265`, 0.110938`, 
              28.833682`, 28.853104`, 0.723103`, 54.458057`, 54.48353`, 
              0.9484`, 65.987463`, 65.981135`, \(-0.235612`\), 68.30968`, 
              68.314361`, 0.17428`, 16.730731`, 29.581138`, 38.962676`, 
              56.109421`, 68.30968`, 84.655208`, 99.697386`, 111.76613`, 
              28.833682`, 54.458057`, 65.987463`, 68.892926`, 51.934285`, 
              71.411538`, 83.225572`, 87.762145`}, {1, 26, 
              10.42272203`, \(-0.592333783`\), 6.153766281`, 0.09`, 
              1, \(-0.3032787178`\), \(-0.1748147515`\), 
              4712.170652`, \(-1.140681792`\), \(-1.362717144`\), 2, 3, 4, 5, 
              6, 0.370917`, \(-0.609799`\), 0.477798`, 0.168006`, 
              0.11823`, \(-0.0959`\), 0.813768`, 0.7223`, 0.850991`, 
              0.853833`, 1.15433`, 1.152703`, 9.90408`, 9.931327`, 0.995085`, 
              25.997692`, 25.974183`, \(-0.858567`\), 37.7311`, 37.742483`, 
              0.415736`, 48.033231`, 48.036686`, 0.126199`, 28.293726`, 
              28.314672`, 0.764952`, 50.927077`, 50.954873`, 1.015116`, 
              63.379592`, 63.365109`, \(-0.528925`\), 64.587719`, 
              64.575236`, \(-0.455868`\), 9.90408`, 25.997692`, 37.7311`, 
              48.982458`, 64.982183`, 80.768752`, 88.031497`, 101.852415`, 
              28.293726`, 50.927077`, 63.379592`, 64.587719`, 48.033231`, 
              65.698905`, 71.126432`, 88.710296`}, {2, 37, 
              8.52311789`, \(-0.4421933085`\), 5.837450747`, 0.1225`, 
              1, \(-0.3711845869`\), \(-0.2015905316`\), 
              546.4384415`, \(-1.124026409`\), \(-1.451080118`\), 2, 3, 4, 5, 
              6, 0.393565`, \(-0.710751`\), 0.627048`, 0.179116`, 
              0.089896`, \(-0.064081`\), 0.842068`, 0.728341`, 0.87647`, 
              0.869659`, 1.185745`, 1.189659`, 5.324506`, 5.351116`, 
              0.978162`, 23.933439`, 23.910184`, \(-0.854829`\), 36.739305`, 
              36.749264`, 0.366088`, 45.654601`, 45.674961`, 0.748401`, 
              27.988739`, 28.009362`, 0.75808`, 49.050565`, 49.080199`, 
              1.089309`, 61.413593`, 61.400355`, \(-0.486605`\), 61.689097`, 
              61.674527`, \(-0.535546`\), 5.324506`, 23.933439`, 36.739305`, 
              45.654601`, 63.097867`, 78.450899`, 85.458783`, 98.168414`, 
              27.988739`, 49.050565`, 61.413593`, 61.689097`, 45.907508`, 
              64.65393`, 64.146348`, 74.733789`}, {2, 34, 
              10.94888151`, \(-0.3796124889`\), 5.591995421`, 0.16`, 
              1, \(-0.4383723302`\), \(-0.2230772747`\), 
              147.4570973`, \(-1.192892929`\), \(-1.643612253`\), 2, 3, 4, 5, 
              6, 0.427594`, \(-0.768733`\), 0.679329`, 0.184094`, 
              0.070158`, \(-0.040229`\), 0.874178`, 0.732322`, 0.890553`, 
              0.876531`, 1.195045`, 1.204021`, 2.76621`, 2.791108`, 
              0.952515`, 23.482012`, 23.46027`, \(-0.83181`\), 36.318867`, 
              36.327288`, 0.322195`, 45.517041`, 45.537768`, 0.792942`, 
              28.170015`, 28.189001`, 0.726366`, 49.164842`, 49.196947`, 
              1.228238`, 60.435166`, 60.427088`, \(-0.309021`\), 60.45109`, 
              60.428341`, \(-0.870332`\), 2.76621`, 23.482012`, 36.318867`, 
              45.517041`, 63.028432`, 78.099926`, 88.082782`, 97.34302`, 
              28.170015`, 49.164842`, 60.435166`, 60.45109`, 45.640288`, 
              67.499908`, 61.436363`, 72.379986`}, {2, 33, 
              16.64275753`, \(-0.7862275025`\), 6.778186335`, 0.2025`, 
              1, \(-0.454417308`\), \(-0.1731850841`\), 
              5964.667405`, \(-1.880616501`\), \(-1.989617105`\), 2, 3, 4, 5, 
              6, 0.553424`, \(-0.403181`\), \(-0.07697`\), 0.151857`, 
              0.114758`, \(-0.093055`\), 0.931717`, 0.862035`, 0.912786`, 
              0.952901`, 1.203459`, 1.180012`, 22.391021`, 22.407348`, 
              0.979881`, 44.345512`, 44.327159`, \(-1.101518`\), 48.583071`, 
              48.595396`, 0.739744`, 71.540562`, 71.542087`, 0.091535`, 
              37.950586`, 37.962922`, 0.740424`, 77.413824`, 77.429359`, 
              0.932389`, 77.826079`, 77.840959`, 0.89311`, 86.848951`, 
              86.821184`, \(-1.666574`\), 22.391021`, 44.345512`, 48.583071`, 
              78.368229`, 92.021899`, 110.122941`, 133.90166`, 136.403334`, 
              37.950586`, 77.826079`, 77.413824`, 86.848951`, 71.540562`, 
              97.983419`, 107.2518`, 108.497392`}, {\(-2\), 7, 
              43.4135314`, \(-0.6599962362`\), 4.229561016`, 0.25`, 
              1, \(-0.50272`\), \(-0.307886`\), 
              50, \(-1.45567`\), \(-1.79297`\), 2, 3, 4, 5, 6, 
              0.553185`, \(-0.448256`\), \(-0.107535`\), 0.187429`, 
              0.073234`, \(-0.059361`\), 0.96658`, 0.643977`, 0.990132`, 
              0.81159`, 1.319973`, 1.170382`, 13.516802`, 13.546507`, 
              1.112021`, 29.166656`, 29.148763`, \(-0.669829`\), 31.191782`, 
              31.18914`, \(-0.098904`\), 46.325922`, 46.328361`, 0.091318`, 
              24.663032`, 24.674602`, 0.433133`, 48.883293`, 48.901709`, 
              0.689429`, 49.974225`, 49.984704`, 0.392291`, 57.517343`, 
              57.479834`, \(-1.40418`\), 13.516802`, 29.166656`, 31.191782`, 
              50.496819`, 59.540379`, 71.142359`, 86.307301`, 87.537013`, 
              24.663032`, 49.974225`, 48.883293`, 57.517343`, 46.325922`, 
              63.8279`, 70.662345`, 72.097573`}, {5, 32, 
              7.87749082`*^7, \(-0.25`\), \(-0.2606881175`\), 0, 
              1, \(-0.6`\), 
              0.0136985993`, \(-10.68196147`\), \(-0.1917997572`\), \
\(-0.7662876791`\), 3, 4, 5, 6, 0.517261`, \(-60.107044`\), 
              36.654469`, \(-0.029246`\), \(-8.049597`\), 
              0.476656`, \(-7.611972`\), \(-5.512764`\), \(-7.599776`\), \
\(-7.932787`\), \(-7.587287`\), \(-8.100663`\), 23.137344`, 23.137131`, 
              0.000458`, 20.710999`, 20.711001`, \(-5.`*^-6\), 18.542997`, 
              18.368331`, 0.376841`, 18.317329`, 18.317439`, \(-0.000237`\), 
              21.747489`, 21.747493`, \(-8.`*^-6\), 19.125711`, 
              19.125716`, \(-0.000011`\), 18.631976`, 18.158827`, 1.020818`, 
              18.327659`, 17.705271`, 1.342803`, 23.137344`, 20.710999`, 
              18.317329`, 11.886034`, 18.631976`, 17.080578`, 17.691818`, 
              16.249056`, 21.747489`, 19.125711`, 18.327659`, 17.231972`, 
              18.100989`, 18.077378`, 18.542997`, 17.15936`}, {\(-2\), 1, 
              4.636975773`*^7, \(-0.25`\), \(-0.2606881175`\), 0, 
              1, \(-0.5`\), 0.029437`, 50, \(-0.225481`\), \(-1.01917`\), 3, 
              4, 5, 6, 0.517261`, \(-60.107044`\), 
              36.654469`, \(-0.029246`\), \(-8.049597`\), 
              0.476656`, \(-7.611972`\), \(-5.512764`\), \(-7.599776`\), \
\(-7.932787`\), \(-7.587287`\), \(-8.100663`\), 23.137344`, 23.137131`, 
              0.000458`, 20.710999`, 20.711001`, \(-5.`*^-6\), 18.542997`, 
              18.368331`, 0.376841`, 18.317329`, 18.317439`, \(-0.000237`\), 
              21.747489`, 21.747493`, \(-8.`*^-6\), 19.125711`, 
              19.125716`, \(-0.000011`\), 18.631976`, 18.158827`, 1.020818`, 
              18.327659`, 17.705271`, 1.342803`, 23.137344`, 20.710999`, 
              18.317329`, 11.886034`, 18.631976`, 17.080578`, 17.691818`, 
              16.249056`, 21.747489`, 19.125711`, 18.327659`, 17.231972`, 
              18.100989`, 18.077378`, 18.542997`, 17.15936`}, {1, 17, 
              2.031149304`*^7, \(-0.25`\), \(-0.3139782845`\), 0, 
              1, \(-0.4`\), \(-0.0009928893137`\), 
              5116.43474`, \(-0.2798696399`\), \(-0.9040070072`\), 3, 4, 5, 
              6, 0.228012`, \(-25.754846`\), 
              21.680336`, \(-0.013485`\), \(-2.895982`\), 
              1.346335`, \(-2.277984`\), 
              6.081965`, \(-2.219205`\), \(-2.392187`\), \(-2.173694`\), \
\(-2.694291`\), 14.147967`, 14.14797`, \(-4.`*^-6\), 12.301334`, 12.30132`, 
              0.000015`, 10.852176`, 11.224704`, \(-0.426713`\), 10.805036`, 
              9.974748`, 0.951055`, 11.330839`, 11.331012`, \(-0.000198`\), 
              9.73705`, 9.73693`, 0.000138`, 9.549522`, 
              9.549566`, \(-0.00005`\), 9.266746`, 9.443487`, \(-0.202449`\), 
              14.147967`, 12.301334`, 10.805036`, 9.72213`, 9.186293`, 
              9.73705`, 7.630767`, 6.981391`, 11.330839`, 9.549522`, 
              9.266746`, 8.388894`, 9.974751`, 10.852176`, 8.149968`, 
              7.063733`}, {1, 9, 
              6.209914282`*^6, \(-0.25`\), \(-1.049975284`\), 0, 
              1, \(-0.3`\), \(-0.01851916829`\), 
              43.28959325`, \(-0.3036738246`\), \(-0.9230649911`\), 3, 4, 5, 
              6, 0.160723`, \(-12.594519`\), 12.035319`, 
              0.009001`, \(-0.815206`\), 1.328611`, \(-0.252501`\), 
              2.344888`, \(-0.144894`\), \(-0.26832`\), 
              0.074678`, \(-0.505257`\), 26.163005`, 
              26.163036`, \(-0.000083`\), 21.313724`, 21.313471`, 0.000684`, 
              17.510828`, 17.511051`, \(-0.000601`\), 17.102944`, 17.102857`, 
              0.000237`, 16.620583`, 16.621448`, \(-0.002334`\), 14.315355`, 
              13.952073`, 0.980893`, 13.706047`, 13.714532`, \(-0.02291`\), 
              11.466278`, 11.25568`, 0.568636`, 26.163005`, 21.313724`, 
              17.510828`, 14.928502`, 14.315355`, 11.466278`, 11.317982`, 
              8.600371`, 16.620583`, 13.706047`, 10.339`, 8.430574`, 
              17.102944`, 14.807045`, 12.556745`, 10.39821`}, {5, 34, 
              749669.0827`, \(-0.25`\), \(-0.754913903`\), 0, 
              1, \(-0.2`\), \(-0.04188598053`\), 
              0.394844671`, \(-0.3465666692`\), \(-0.8782459248`\), 3, 4, 5, 
              6, 0.138323`, \(-3.486626`\), 4.186943`, 0.066971`, 
              0.451305`, \(-0.385541`\), 0.498232`, \(-0.903064`\), 
              0.522328`, 0.466533`, 0.739496`, 0.709701`, 6.517234`, 
              6.51749`, \(-0.000427`\), 3.305179`, 3.306107`, \(-0.001551`\), 
              2.765653`, 2.739754`, 0.043271`, 2.692539`, 
              2.716202`, \(-0.039535`\), 0.533945`, 0.529604`, 0.007254`, 
              0.27431`, \(-0.076536`\), 
              0.586178`, \(-0.52649`\), \(-0.161027`\), \(-0.610598`\), \
\(-0.728669`\), \(-0.644616`\), \(-0.140432`\), 6.517234`, 3.305179`, 
              2.692539`, 2.152826`, 
              0.27431`, \(-1.774907`\), \(-1.798087`\), \(-1.813305`\), 
              0.533945`, \(-1.116711`\), \(-0.52649`\), \(-0.728669`\), 
              2.765653`, 1.276157`, 0.115147`, \(-1.43669`\)}, {5, 32, 
              34.54916505`, \(-0.3096009375`\), 7.600025626`, 0, 
              1, \(-0.1`\), \(-0.09720544145`\), 3900.248836`, 
              0.866333714`, \(-0.7362485962`\), 3, 4, 5, 6, 
              0.151817`, \(-0.734705`\), 0.945536`, 0.138243`, 
              0.163052`, \(-0.121435`\), 0.600177`, 0.531416`, 0.6685`, 
              0.66747`, 0.938726`, 0.945501`, \(-0.063332`\), \(-0.02039`\), 
              0.792749`, 13.162131`, 13.12482`, \(-0.688796`\), 16.651734`, 
              16.716413`, 1.194038`, 31.186616`, 31.12695`, \(-1.101495`\), 
              22.787949`, 22.844396`, 1.042061`, 33.09363`, 33.126607`, 
              0.608789`, 47.636382`, 47.51874`, \(-2.171783`\), 49.497436`, 
              49.481458`, \(-0.294955`\), \(-0.063332`\), 13.162131`, 
              16.651734`, 33.766927`, 49.497436`, 53.82597`, 72.120572`, 
              72.793833`, 22.787949`, 33.09363`, 47.636382`, 52.488898`, 
              31.186616`, 35.579363`, 46.046556`, 62.510802`}, {5, 33, 
              6.724709295`, \(-1.730584884`\), 7.409184321`, 0, 1, 
              0, \(-0.1176008093`\), 
              12.61009021`, \(-0.6155233757`\), \(-0.6164392438`\), 3, 4, 5, 
              6, 0.190268`, \(-0.421902`\), 0.251766`, 0.137939`, 
              0.230187`, \(-0.225577`\), 0.620988`, 0.606189`, 0.721357`, 
              0.73859`, 1.023949`, 1.013526`, 17.594575`, 17.640962`, 
              1.046278`, 27.9672`, 27.940239`, \(-0.608114`\), 33.539216`, 
              33.560094`, 0.470918`, 45.793839`, 45.80145`, 0.171669`, 
              23.492968`, 23.525028`, 0.723144`, 45.036456`, 45.073819`, 
              0.842739`, 59.0166`, 59.020933`, 0.097735`, 62.672508`, 
              62.707339`, 0.785625`, 17.594575`, 27.9672`, 33.539216`, 
              51.019366`, 59.0166`, 75.683315`, 90.340084`, 96.202569`, 
              23.492968`, 45.036456`, 62.672508`, 65.681206`, 45.793839`, 
              66.91616`, 74.037554`, 81.441968`}, {1, 23, 
              6.555084342`, \(-2.099194556`\), 8.004893473`, 0, 1, 
              0.1`, \(-0.1270939969`\), 
              4775.854015`, \(-0.8456016975`\), \(-0.733583403`\), 3, 4, 5, 
              6, 0.194392`, \(-0.44386`\), 0.302428`, 0.124958`, 
              0.226582`, \(-0.206217`\), 0.61276`, 0.609172`, 0.688814`, 
              0.719355`, 0.975171`, 0.957648`, 25.062974`, 25.105214`, 
              1.05165`, 35.172491`, 35.173917`, 0.035484`, 38.710805`, 
              38.71017`, \(-0.015822`\), 54.70133`, 54.707831`, 0.161861`, 
              24.560673`, 24.587685`, 0.67251`, 51.774948`, 51.811771`, 
              0.916793`, 66.154355`, 66.157428`, 0.076526`, 67.758765`, 
              67.795252`, 0.908441`, 25.062974`, 35.172491`, 38.710805`, 
              66.500257`, 66.154355`, 84.741999`, 109.251574`, 110.607884`, 
              24.560673`, 51.774948`, 67.758765`, 73.866662`, 54.70133`, 
              86.534996`, 84.98687`, 94.235363`}, {5, 32, 
              7.121482369`, \(-1.563161134`\), 8.309330894`, 0, 1, 
              0.2`, \(-0.1574928754`\), 
              3437.192941`, \(-0.959213735`\), \(-0.7488003911`\), 3, 4, 5, 
              6, 0.170303`, \(-0.509917`\), 0.470083`, 0.122356`, 
              0.210835`, \(-0.18098`\), 0.58683`, 0.577624`, 0.656222`, 
              0.687422`, 0.934866`, 0.922812`, 26.4379`, 26.484189`, 
              1.048059`, 36.094749`, 36.121262`, 0.600292`, 41.801774`, 
              41.777936`, \(-0.539738`\), 57.122971`, 57.130091`, 0.161215`, 
              23.475616`, 23.503671`, 0.635209`, 51.850386`, 51.889865`, 
              0.893853`, 66.925453`, 66.933383`, 0.179536`, 68.941332`, 
              68.979371`, 0.861274`, 26.4379`, 41.801774`, 36.094749`, 
              67.143518`, 66.925453`, 86.089066`, 108.202502`, 123.18724`, 
              23.475616`, 51.850386`, 68.941332`, 76.243275`, 57.122971`, 
              88.178406`, 94.250411`, 96.807861`}, {5, 33, 
              6.679296838`, \(-1.232667001`\), 9.935173442`, 0, 1, 
              0.3`, \(-0.151938577`\), 
              3488.858867`, \(-1.276728498`\), \(-0.8894611031`\), 3, 4, 5, 
              6, 0.149796`, \(-0.602333`\), 0.708013`, 0.100545`, 
              0.204772`, \(-0.154979`\), 0.55625`, 0.551091`, 0.590421`, 
              0.632133`, 0.839017`, 0.814403`, 32.300707`, 32.344636`, 
              1.04604`, 41.312443`, 41.348743`, 0.864368`, 50.679346`, 
              50.654727`, \(-0.586231`\), 67.738419`, 67.7445`, 0.144802`, 
              25.608221`, 25.636645`, 0.676841`, 59.712992`, 59.753345`, 
              0.960899`, 77.345473`, 77.37886`, 0.795021`, 80.862424`, 
              80.874695`, 0.292215`, 32.300707`, 50.679346`, 41.312443`, 
              77.477692`, 77.345473`, 99.899935`, 124.456499`, 140.207281`, 
              25.608221`, 59.712992`, 80.862424`, 89.562705`, 67.738419`, 
              105.951098`, 100.600368`, 124.976948`}, {5, 32, 
              7.832289531`, \(-0.9910907483`\), 10.63091017`, 0, 1, 
              0.4`, \(-0.2006369519`\), 
              3318.765626`, \(-1.416836191`\), \(-0.9532793011`\), 3, 4, 5, 
              6, 0.106119`, \(-0.746132`\), 1.060653`, 0.094796`, 
              0.177794`, \(-0.112479`\), 0.513063`, 0.473603`, 0.526045`, 
              0.556351`, 0.752883`, 0.735867`, 30.226271`, 30.284605`, 
              1.052953`, 40.79787`, 40.842473`, 0.805083`, 53.710265`, 
              53.678435`, \(-0.574528`\), 68.077025`, 68.084766`, 0.139727`, 
              23.552431`, 23.587879`, 0.639851`, 56.905314`, 56.954719`, 
              0.891764`, 76.48739`, 76.526945`, 0.713981`, 83.214559`, 
              83.236063`, 0.38815`, 30.226271`, 53.710265`, 40.79787`, 
              75.42646`, 76.48739`, 99.866493`, 121.123044`, 141.613004`, 
              23.552431`, 56.905314`, 83.214559`, 91.663531`, 68.077025`, 
              106.829752`, 98.640936`, 126.811206`}, {5, 33, 
              5.917227444`*^7, \(-0.25`\), \(-0.2766899437`\), 0.0025`, 
              1, \(-0.6`\), 
              0.03133663851`, \(-3.443167326`\), \(-0.6625851301`\), \
\(-1.071960427`\), 3, 4, 5, 6, 0.424385`, \(-46.888536`\), 
              29.23748`, \(-0.029579`\), \(-5.702561`\), 
              0.483282`, \(-5.521083`\), \(-3.026706`\), \(-5.485874`\), \
\(-5.583887`\), \(-5.463172`\), \(-5.753956`\), 21.283825`, 21.283811`, 
              0.000027`, 18.424382`, 18.42437`, 0.000023`, 16.169594`, 
              16.169606`, \(-0.000021`\), 16.110916`, 16.072016`, 0.073084`, 
              18.73124`, 18.731219`, 0.00004`, 16.464898`, 16.464879`, 
              0.000036`, 16.02948`, 15.395105`, 1.191844`, 15.040499`, 
              15.220352`, \(-0.337902`\), 21.283825`, 18.424382`, 16.169594`, 
              16.110916`, 16.02948`, 14.381869`, 15.040499`, 13.44373`, 
              18.73124`, 16.464898`, 15.015482`, 14.585762`, 15.540209`, 
              15.428925`, 16.107911`, 14.479686`}, {5, 32, 
              1.682173566`*^6, \(-0.25`\), \(-0.4895289177`\), 0.0025`, 
              1, \(-0.5`\), 
              0.01467927876`, \(-1296.985843`\), \(-0.1694625399`\), 
              0.1044261824`, 3, 4, 5, 6, 0.265674`, \(-31.73445`\), 
              24.980508`, \(-0.009827`\), \(-4.257226`\), 
              2.590032`, \(-3.076443`\), 
              6.005141`, \(-3.102397`\), \(-3.251144`\), \(-3.01751`\), \
\(-3.789078`\), 26.961928`, 26.961935`, \(-0.000014`\), 23.392739`, 
              23.392718`, 0.000044`, 20.802875`, 20.063805`, 1.537916`, 
              20.178827`, 19.696856`, 1.002926`, 22.31075`, 
              22.310921`, \(-0.000355`\), 19.662311`, 
              19.662757`, \(-0.000928`\), 18.576139`, 18.575951`, 0.000391`, 
              17.888877`, 17.88853`, 0.000722`, 26.961928`, 23.392739`, 
              20.802875`, 18.720105`, 17.888877`, 18.576139`, 16.646306`, 
              14.205673`, 22.31075`, 19.662311`, 17.388908`, 16.625824`, 
              18.948379`, 20.178827`, 16.864521`, 15.940277`}, {1, 13, 
              1.238967622`*^7, \(-0.25`\), \(-0.2734701112`\), 0.0025`, 
              1, \(-0.4`\), 0.002400907838`, 
              1834.156811`, \(-0.6653868663`\), \(-0.9921702055`\), 3, 4, 5, 
              6, 0.226726`, \(-18.231083`\), 16.048434`, 
              0.032669`, \(-1.533321`\), 1.170372`, \(-1.016232`\), 
              7.458944`, \(-0.907989`\), \(-0.984023`\), \(-0.759236`\), \
\(-1.135926`\), 9.623012`, 9.623015`, \(-3.`*^-6\), 8.283845`, 8.283835`, 
              0.00001`, 7.270403`, 7.75744`, \(-0.483162`\), 6.770136`, 
              6.316434`, 0.450092`, 6.87578`, 6.875918`, \(-0.000137`\), 
              6.078961`, 6.077901`, 0.001052`, 5.549034`, 
              5.841346`, \(-0.289986`\), 5.547292`, 
              5.547778`, \(-0.000482`\), 9.623012`, 8.283845`, 6.770136`, 
              6.205557`, 5.487008`, 6.078961`, 4.504497`, 3.356313`, 
              6.87578`, 5.547292`, 5.549034`, 4.402816`, 6.316436`, 
              7.270403`, 5.220676`, 3.293305`}, {5, 32, 
              1.086518108`*^6, \(-0.25`\), \(-0.6508175049`\), 0.0025`, 
              1, \(-0.3`\), 
              0.003718618343`, \(-1042.36063`\), \(-0.6670129476`\), \
\(-0.9434026857`\), 3, 4, 5, 6, 0.17065`, \(-7.869253`\), 7.803355`, 
              0.08674`, \(-0.314944`\), 0.839998`, 0.310871`, 2.832767`, 
              0.356703`, 0.237228`, 0.586688`, 0.287138`, 11.830857`, 
              11.830888`, \(-0.000054`\), 9.128222`, 
              9.128266`, \(-0.000079`\), 6.817057`, 6.81698`, 0.000136`, 
              6.566669`, 6.566563`, 0.000189`, 5.356012`, 
              5.356998`, \(-0.001752`\), 4.31551`, 3.873292`, 0.785819`, 
              3.858435`, 3.814099`, 0.078784`, 3.129872`, 2.864629`, 
              0.471335`, 11.830857`, 6.817057`, 9.128222`, 5.5311`, 4.31551`, 
              3.129872`, 2.385431`, 1.880492`, 5.356012`, 3.858435`, 
              2.167912`, 0.91833`, 6.566669`, 5.213414`, 3.454023`, 
              2.667322`}, {5, 34, 248786.4596`, \(-0.25`\), 1.884857302`, 
              0.0025`, 1, \(-0.2`\), \(-0.05590456103`\), 
              4.425490538`, \(-0.6777137412`\), \(-0.9157221577`\), 3, 4, 5, 
              6, 0.170653`, \(-1.87025`\), 2.783291`, 0.116141`, 
              0.171576`, \(-0.049211`\), 0.588796`, 0.198807`, 0.627222`, 
              0.456893`, 0.875848`, 
              0.749557`, \(-9.398932`\), \(-9.398477`\), 
              0.002341`, \(-3.874762`\), \(-3.879793`\), \(-0.025891`\), \
\(-2.728901`\), \(-2.72918`\), \(-0.001437`\), 1.128642`, 1.542512`, 
              2.12999`, 4.314885`, 4.31142`, \(-0.017835`\), 4.931191`, 
              4.950332`, 0.098512`, 5.813747`, 5.961331`, 0.759541`, 
              7.611831`, 7.872899`, 
              1.34359`, \(-9.398932`\), \(-2.728901`\), \(-3.874762`\), 
              4.000138`, 8.135501`, 5.813747`, 11.088343`, 13.655019`, 
              4.931191`, 4.314885`, 7.611831`, 7.91531`, 1.128642`, 
              1.279244`, 5.888654`, 10.337072`}, {5, 33, 
              15.94250349`, \(-1.197037181`\), 9.967937565`, 0.0025`, 
              1, \(-0.1`\), \(-0.06238445593`\), 
              1896.115873`, \(-1.076268406`\), \(-0.7778778543`\), 3, 4, 5, 
              6, 0.238067`, \(-0.480889`\), 0.382531`, 0.099519`, 
              0.241886`, \(-0.194174`\), 0.655473`, 0.685979`, 0.734749`, 
              0.750596`, 1.028634`, 0.90822`, 19.333685`, 19.357486`, 
              0.90369`, 29.903781`, 29.88467`, \(-0.725587`\), 49.063684`, 
              49.093891`, 1.146921`, 59.660076`, 59.661929`, 0.070344`, 
              34.979907`, 35.010144`, 1.148071`, 63.153433`, 63.182795`, 
              1.114846`, 81.159488`, 81.129878`, \(-1.124244`\), 84.900482`, 
              84.875484`, \(-0.949149`\), 19.333685`, 29.903781`, 49.063684`, 
              62.938134`, 81.159488`, 103.419087`, 115.908729`, 133.829435`, 
              34.979907`, 63.153433`, 84.900482`, 88.915321`, 59.660076`, 
              79.000711`, 98.939543`, 99.101726`}, {1, 23, 
              8.176644017`, \(-1.790776453`\), 6.708897925`, 0.0025`, 1, 
              0, \(-0.138461567`\), 
              2749.567766`, \(-0.6634787387`\), \(-0.7195747486`\), 3, 4, 5, 
              6, 0.223003`, \(-0.437128`\), 0.240493`, 0.150588`, 
              0.202363`, \(-0.202156`\), 0.661317`, 0.61564`, 0.754994`, 
              0.760005`, 1.062529`, 1.059574`, 19.303706`, 19.347907`, 
              1.058064`, 29.387789`, 29.36083`, \(-0.645333`\), 33.718417`, 
              33.735528`, 0.409575`, 46.219414`, 46.225993`, 0.157491`, 
              22.910838`, 22.937527`, 0.638868`, 45.278028`, 45.311466`, 
              0.800429`, 58.334574`, 58.332103`, \(-0.059169`\), 59.434694`, 
              59.46863`, 0.812367`, 19.303706`, 29.387789`, 33.718417`, 
              53.04934`, 58.334574`, 73.993903`, 91.781916`, 96.372314`, 
              22.910838`, 45.278028`, 59.434694`, 64.055511`, 46.219414`, 
              69.73335`, 73.628204`, 74.954896`}, {1, 28, 
              8.382634368`, \(-2.242308736`\), 7.109854404`, 0.0025`, 1, 
              0.1`, \(-0.1439388969`\), 
              7136.093032`, \(-0.8469557806`\), \(-0.7818299421`\), 3, 4, 5, 
              6, 0.22995`, \(-0.45594`\), 0.284614`, 0.141613`, 
              0.201745`, \(-0.192167`\), 0.655487`, 0.628196`, 0.73718`, 
              0.754998`, 1.037206`, 1.027001`, 25.784879`, 25.825467`, 
              1.061747`, 34.581037`, 34.597748`, 0.437147`, 37.235286`, 
              37.212495`, \(-0.5962`\), 53.436843`, 53.442098`, 0.137462`, 
              23.690295`, 23.71266`, 0.585048`, 50.770608`, 50.802375`, 
              0.830985`, 63.026472`, 63.040703`, 0.372268`, 64.033819`, 
              64.048693`, 0.389098`, 25.784879`, 37.235286`, 34.581037`, 
              64.536601`, 64.033819`, 80.951775`, 104.267383`, 108.747505`, 
              23.690295`, 50.770608`, 63.026472`, 70.568216`, 53.436843`, 
              80.603849`, 85.848401`, 86.634136`}, {5, 33, 
              8.135028131`, \(-1.701270405`\), 9.274625113`, 0.0025`, 1, 
              0.2`, \(-0.1000249053`\), 
              6377.207328`, \(-1.306761272`\), \(-0.9953524085`\), 3, 4, 5, 
              6, 0.234317`, \(-0.490796`\), 0.390327`, 0.106816`, 
              0.221498`, \(-0.190557`\), 0.640952`, 0.658605`, 0.677313`, 
              0.733305`, 0.942794`, 0.910923`, 38.871998`, 38.902129`, 
              1.047704`, 44.734853`, 44.76074`, 0.900118`, 50.848761`, 
              50.829582`, \(-0.666848`\), 72.691775`, 72.694502`, 0.094824`, 
              29.76245`, 29.780849`, 0.639776`, 68.128416`, 68.156385`, 
              0.972488`, 81.942304`, 81.944932`, 0.091362`, 84.258789`, 
              84.272892`, 0.490378`, 38.871998`, 50.848761`, 44.734853`, 
              89.537022`, 84.258789`, 106.295381`, 138.317341`, 141.609074`, 
              29.76245`, 68.128416`, 81.942304`, 92.866625`, 72.691775`, 
              110.847653`, 111.032762`, 119.782134`}, {\(-2\), 22, 
              8.557305533`, \(-1.364229023`\), 8.348811803`, 0.0025`, 1, 
              0.3`, \(-0.1691262605`\), 
              615.1367757`, \(-1.202037779`\), \(-0.8587407616`\), 3, 4, 5, 
              6, 0.19063`, \(-0.577916`\), 0.600879`, 0.123576`, 
              0.18803`, \(-0.155406`\), 0.60809`, 0.595853`, 0.666963`, 
              0.69909`, 0.944434`, 0.927562`, 31.920269`, 31.9614`, 
              1.047386`, 37.914369`, 37.941742`, 0.697033`, 47.029452`, 
              47.002571`, \(-0.684512`\), 62.908935`, 62.913932`, 0.127228`, 
              24.156273`, 24.178825`, 0.574269`, 56.292897`, 56.326262`, 
              0.849616`, 71.494807`, 71.504426`, 0.244942`, 71.753516`, 
              71.774232`, 0.527532`, 31.920269`, 47.029452`, 37.914369`, 
              75.550499`, 71.753516`, 91.034228`, 118.314913`, 122.162024`, 
              24.156273`, 56.292897`, 71.494807`, 81.418597`, 62.908935`, 
              96.270499`, 97.878215`, 105.306501`}, {5, 33, 
              8.917290277`, \(-1.105105359`\), 7.67014115`, 0.0025`, 1, 
              0.4`, \(-0.2504376375`\), 
              2.421391247`, \(-1.157697218`\), \(-0.8092627818`\), 3, 4, 5, 
              6, 0.13868`, \(-0.710321`\), 0.915377`, 0.132491`, 
              0.157439`, \(-0.115501`\), 0.565786`, 0.495442`, 0.627731`, 
              0.629931`, 0.90161`, 0.900312`, 19.450096`, 19.512038`, 
              1.054208`, 24.960318`, 25.005861`, 0.775098`, 43.872684`, 
              43.836372`, \(-0.617997`\), 47.82335`, 47.895981`, 1.236121`, 
              19.256692`, 19.284345`, 0.470625`, 45.552051`, 45.591893`, 
              0.678066`, 60.70417`, 60.72874`, 0.418165`, 62.994611`, 
              63.00969`, 0.256626`, 19.450096`, 24.960318`, 43.872684`, 
              47.82335`, 60.70417`, 77.809397`, 98.105914`, 106.586474`, 
              19.256692`, 45.552051`, 62.994611`, 71.318856`, 53.778838`, 
              83.439854`, 80.246816`, 94.18695`}, {\(-2\), 1, 
              4.051430028`*^7, \(-1.105105359`\), 7.67014115`, 0.01`, 
              1, \(-0.6`\), 0.0663245`, 50, \(-1.04119`\), \(-1.09806`\), 3, 
              4, 5, 6, 0.13868`, \(-0.710321`\), 0.915377`, 0.132491`, 
              0.157439`, \(-0.115501`\), 0.565786`, 0.495442`, 0.627731`, 
              0.629931`, 0.90161`, 0.900312`, 19.450096`, 19.512038`, 
              1.054208`, 24.960318`, 25.005861`, 0.775098`, 43.872684`, 
              43.836372`, \(-0.617997`\), 47.82335`, 47.895981`, 1.236121`, 
              19.256692`, 19.284345`, 0.470625`, 45.552051`, 45.591893`, 
              0.678066`, 60.70417`, 60.72874`, 0.418165`, 62.994611`, 
              63.00969`, 0.256626`, 19.450096`, 24.960318`, 43.872684`, 
              47.82335`, 60.70417`, 77.809397`, 98.105914`, 106.586474`, 
              19.256692`, 45.552051`, 62.994611`, 71.318856`, 53.778838`, 
              83.439854`, 80.246816`, 94.18695`}, {\(-2\), 11, 
              1.929977044`*^7, \(-0.25`\), \(-0.2091515326`\), 0.01`, 
              1, \(-0.5`\), 0.02640794938`, 
              1934.263373`, \(-1.007392784`\), \(-1.057491827`\), 3, 4, 5, 6, 
              0.294076`, \(-22.77579`\), 18.383149`, 
              0.057163`, \(-2.13312`\), 1.127732`, \(-1.635242`\), 
              7.816311`, \(-1.537071`\), \(-1.539437`\), \(-1.40733`\), \
\(-1.621745`\), 9.186172`, 9.186177`, \(-5.`*^-6\), 7.944408`, 7.944405`, 
              3.`*^-6, 6.716953`, 7.212801`, \(-0.487965`\), 6.695153`, 
              5.95679`, 0.726625`, 6.939551`, 6.939558`, \(-7.`*^-6\), 
              5.902933`, 5.903464`, \(-0.000523`\), 5.879209`, 5.878836`, 
              0.000368`, 5.302216`, 5.301665`, 0.000542`, 9.186172`, 
              7.944408`, 6.716953`, 5.89617`, 5.228788`, 5.879209`, 
              5.153384`, 3.722115`, 6.939551`, 5.902933`, 5.302216`, 
              4.656042`, 5.95679`, 6.695153`, 4.79311`, 4.489964`}, {5, 32, 
              6.980307701`*^6, \(-0.25`\), \(-0.3197867866`\), 0.01`, 
              1, \(-0.4`\), \(-0.00133357197`\), \(-0.4620923815`\), \
\(-1.140637653`\), \(-0.9544111576`\), 3, 4, 5, 6, 0.23051`, \(-11.976237`\), 
              10.835893`, 0.108254`, \(-0.958398`\), 
              1.33139`, \(-0.078682`\), 7.729833`, 0.02582`, \(-0.155698`\), 
              0.300293`, \(-0.15072`\), 8.44507`, 8.44508`, \(-0.000011`\), 
              6.375141`, 6.375196`, \(-0.000064`\), 5.931689`, 5.016536`, 
              1.079356`, 5.01653`, 4.257246`, 0.89552`, 4.94116`, 
              4.941292`, \(-0.000155`\), 4.701393`, 3.907846`, 0.93593`, 
              3.905672`, 3.397538`, 0.599306`, 3.736976`, 3.237476`, 
              0.589124`, 8.44507`, 6.375141`, 5.01653`, 2.821687`, 3.736976`, 
              4.701393`, 2.980731`, 1.295225`, 4.94116`, 3.905672`, 
              2.899629`, 1.966025`, 4.969286`, 5.931689`, 3.624604`, 
              1.626398`}, {5, 33, 
              1.453006837`*^6, \(-0.25`\), \(-0.4304550259`\), 0.01`, 
              1, \(-0.3`\), \(-0.02311408767`\), 
              1.025821083`, \(-0.9340267853`\), \(-1.01342891`\), 3, 4, 5, 6, 
              0.202452`, \(-4.044351`\), 4.711458`, 
              0.111737`, \(-0.087562`\), 0.3844`, 0.525768`, 1.918779`, 
              0.557`, 0.345918`, 0.799723`, 0.548364`, 5.187426`, 
              5.187493`, \(-0.000094`\), 2.322657`, 2.322609`, 0.000066`, 
              2.245669`, 2.246301`, \(-0.00088`\), 1.889493`, 1.80959`, 
              0.111413`, 0.882748`, 0.256804`, 0.87278`, 0.252346`, 
              0.241795`, 0.014712`, 0.245146`, 0.138465`, 0.14875`, 
              0.000284`, 0.000947`, \(-0.000925`\), 5.187426`, 2.322657`, 
              2.245669`, 1.479836`, 0.000284`, 
              0.882748`, \(-0.895947`\), \(-2.125562`\), 
              0.245146`, \(-0.267973`\), 0.252346`, \(-0.535346`\), 
              1.889493`, 1.412628`, 0.145213`, \(-0.535544`\)}, {1, 34, 
              14929.25603`, \(-0.4247778026`\), 2.834113293`, 0.01`, 
              1, \(-0.2`\), 0.05413377356`, 1125.469207`, 
              0.1343506695`, \(-0.6132483813`\), 3, 4, 5, 6, 
              0.305143`, \(-0.633607`\), 0.701918`, 0.214462`, 
              0.052931`, \(-0.056721`\), 0.657394`, 0.352453`, 0.774855`, 
              0.666308`, 1.098984`, 
              1.165748`, \(-3.454849`\), \(-3.454834`\), 0.000202`, 
              6.673309`, 6.674024`, 0.009893`, 7.916949`, 
              7.910905`, \(-0.083634`\), 11.212166`, 11.379048`, 2.309141`, 
              10.532267`, 10.531227`, \(-0.014402`\), 17.164987`, 17.167904`, 
              0.040365`, 19.046996`, 19.027045`, \(-0.276055`\), 21.605711`, 
              21.61781`, 0.167417`, \(-3.454849`\), 6.673309`, 7.916949`, 
              11.212166`, 21.605711`, 23.5303`, 31.860732`, 33.065805`, 
              10.532267`, 17.164987`, 19.046996`, 25.246232`, 13.671725`, 
              16.211506`, 23.218407`, 23.950688`}, {1, 28, 
              8.225594084`, \(-1.255278732`\), 7.279460237`, 0.01`, 
              1, \(-0.1`\), \(-0.106496461`\), 
              1807.357256`, \(-0.8867586602`\), \(-0.9319463674`\), 3, 4, 5, 
              6, 0.259926`, \(-0.47534`\), 0.321693`, 0.138344`, 
              0.205974`, \(-0.190405`\), 0.694159`, 0.668103`, 0.758336`, 
              0.782482`, 1.050868`, 1.036843`, 18.015478`, 18.049904`, 
              1.042211`, 27.528765`, 27.50079`, \(-0.846911`\), 38.114431`, 
              38.134769`, 0.615715`, 49.149716`, 49.153809`, 0.123911`, 
              27.066413`, 27.091712`, 0.7659`, 50.652788`, 50.680776`, 
              0.847307`, 64.836847`, 64.829977`, \(-0.207993`\), 66.871618`, 
              66.87051`, \(-0.033523`\), 18.015478`, 27.528765`, 38.114431`, 
              53.462845`, 64.836847`, 81.968719`, 97.042764`, 106.217801`, 
              27.066413`, 50.652788`, 66.871618`, 68.861585`, 49.149716`, 
              69.760852`, 79.236191`, 82.037168`}, {1, 31, 
              9.92418033`, \(-2.267241046`\), 6.677269501`, 0.01`, 1, 
              0, \(-0.1228998723`\), 
              9994.620395`, \(-0.8443096778`\), \(-0.8632841183`\), 3, 4, 5, 
              6, 0.28033`, \(-0.440806`\), 0.216864`, 0.151112`, 
              0.190007`, \(-0.187058`\), 0.704384`, 0.67133`, 0.78533`, 
              0.801117`, 1.094016`, 1.08481`, 25.580374`, 25.615915`, 
              1.064437`, 33.774492`, 33.753205`, \(-0.63754`\), 35.804801`, 
              35.816123`, 0.339099`, 52.71216`, 52.715617`, 0.103527`, 
              24.988123`, 25.006736`, 0.557452`, 52.13252`, 52.158274`, 
              0.771319`, 62.366134`, 62.373081`, 0.208057`, 64.762217`, 
              64.772511`, 0.308305`, 25.580374`, 33.774492`, 35.804801`, 
              63.692104`, 64.762217`, 80.977989`, 103.600081`, 104.021871`, 
              24.988123`, 52.13252`, 62.366134`, 69.476281`, 52.71216`, 
              79.971271`, 80.697325`, 83.800869`}, {5, 33, 
              10.96256292`, \(-2.456834675`\), 6.943655991`, 0.01`, 1, 
              0.1`, \(-0.1283513663`\), 
              5318.547453`, \(-0.9906246599`\), \(-0.9059886833`\), 3, 4, 5, 
              6, 0.282726`, \(-0.456641`\), 0.253841`, 0.144936`, 
              0.189975`, \(-0.182419`\), 0.699066`, 0.675682`, 0.771843`, 
              0.795312`, 1.075074`, 1.061463`, 30.852607`, 30.886473`, 
              1.06373`, 36.800694`, 36.818443`, 0.557505`, 40.076963`, 
              40.051564`, \(-0.797775`\), 58.378413`, 58.38133`, 0.091599`, 
              25.372412`, 25.388789`, 0.514406`, 56.129429`, 56.154412`, 
              0.784723`, 64.745526`, 64.751056`, 0.173723`, 69.056888`, 
              69.067406`, 0.330391`, 30.852607`, 40.076963`, 36.800694`, 
              71.214319`, 69.056888`, 86.013496`, 109.936537`, 113.675039`, 
              25.372412`, 56.129429`, 64.745526`, 74.693229`, 58.378413`, 
              87.414996`, 88.453929`, 95.251305`}, {5, 33, 
              11.39879853`, \(-1.857260631`\), 6.95847209`, 0.01`, 1, 
              0.2`, \(-0.1548259546`\), 
              4224.82961`, \(-1.058161236`\), \(-0.8962031768`\), 3, 4, 5, 6, 
              0.26269`, \(-0.493567`\), 0.346501`, 0.144952`, 
              0.179359`, \(-0.167282`\), 0.681441`, 0.649419`, 0.753224`, 
              0.772276`, 1.053396`, 1.042363`, 31.542609`, 31.579012`, 
              1.064668`, 35.929853`, 35.946922`, 0.499225`, 42.567504`, 
              42.542432`, \(-0.733296`\), 58.945006`, 58.948316`, 0.0968`, 
              24.104802`, 24.121596`, 0.491154`, 54.985246`, 55.011507`, 
              0.76807`, 63.929804`, 63.93623`, 0.187947`, 68.179244`, 
              68.190933`, 0.34188`, 31.542609`, 42.567504`, 35.929853`, 
              70.999091`, 68.179244`, 84.913631`, 109.575065`, 114.337619`, 
              24.104802`, 54.985246`, 63.929804`, 74.852952`, 58.945006`, 
              87.972608`, 89.478571`, 97.134006`}, {5, 33, 
              12.39463084`, \(-1.495720085`\), 6.610116917`, 0.01`, 1, 
              0.3`, \(-0.2146048765`\), 
              839.4333597`, \(-1.008427543`\), \(-0.8149484013`\), 3, 4, 5, 
              6, 0.224471`, \(-0.564702`\), 0.514968`, 0.153554`, 
              0.156826`, \(-0.141243`\), 0.653106`, 0.592622`, 0.735647`, 
              0.736908`, 1.039568`, 1.03883`, 28.129269`, 28.1742`, 
              1.066682`, 32.366465`, 32.383713`, 0.409455`, 41.352167`, 
              41.323281`, \(-0.685778`\), 54.255387`, 54.260242`, 0.115245`, 
              21.040801`, 21.060115`, 0.458531`, 48.751942`, 48.781601`, 
              0.70414`, 59.104455`, 59.114455`, 0.237391`, 61.872714`, 
              61.885213`, 0.296728`, 28.129269`, 41.352167`, 32.366465`, 
              64.628075`, 61.872714`, 77.335733`, 101.08205`, 104.632242`, 
              21.040801`, 48.751942`, 59.104455`, 69.461923`, 54.255387`, 
              81.246458`, 83.438773`, 90.977168`}, {5, 34, 
              13.884388`, \(-1.23128853`\), 6.471503885`, 0.01`, 1, 
              0.4`, \(-0.2847103438`\), 
              349.8290983`, \(-0.991639224`\), \(-0.7430568169`\), 3, 4, 5, 
              6, 0.179084`, \(-0.672157`\), 0.767137`, 0.158319`, 
              0.13674`, \(-0.11379`\), 0.616908`, 0.528767`, 0.707368`, 
              0.693117`, 1.012393`, 1.019338`, 24.55109`, 24.608546`, 
              1.065427`, 29.527365`, 29.545038`, 0.327713`, 40.268253`, 
              40.23494`, \(-0.617728`\), 50.185959`, 50.193022`, 0.13097`, 
              18.409443`, 18.432398`, 0.425658`, 43.214133`, 43.248587`, 
              0.638877`, 55.869936`, 55.883929`, 0.259475`, 56.683769`, 
              56.695804`, 0.223171`, 24.55109`, 40.268253`, 29.527365`, 
              59.216665`, 56.683769`, 71.323856`, 93.854074`, 96.122376`, 
              18.409443`, 43.214133`, 55.869936`, 65.297091`, 50.185959`, 
              75.916408`, 78.363878`, 83.748481`}, {5, 31, 
              5.069441347`*^6, \(-0.25`\), \(-0.6546849949`\), 0.0225`, 
              1, \(-0.6`\), \(-0.05799941731`\), \(-2251.541124`\), \
\(-1.293933647`\), \(-0.9353306995`\), 3, 4, 5, 6, 
              0.271368`, \(-26.740358`\), 
              21.804755`, \(-0.17025`\), \(-1.906767`\), 
              0.417354`, \(-2.062402`\), \(-0.74108`\), \(-2.072975`\), \
\(-2.170924`\), \(-1.917034`\), \(-2.676717`\), 33.167117`, 
              33.167128`, \(-0.00003`\), 28.508951`, 28.508926`, 0.000072`, 
              24.40221`, 23.646538`, 2.148046`, 22.649341`, 22.069814`, 
              1.647342`, 25.716747`, 25.716966`, \(-0.000621`\), 22.564721`, 
              22.566749`, \(-0.005766`\), 21.830391`, 21.829829`, 0.001598`, 
              19.488241`, 19.475685`, 0.035692`, 33.167117`, 28.508951`, 
              22.649341`, 21.945715`, 19.127945`, 21.830391`, 19.488241`, 
              16.786708`, 25.716747`, 22.564721`, 19.477177`, 17.446484`, 
              21.357287`, 24.40221`, 20.337217`, 17.555312`}, {5, 33, 
              1.154897496`*^7, \(-0.25`\), \(-0.2645737496`\), 0.0225`, 
              1, \(-0.5`\), 0.009628577539`, 
              3.51714044`, \(-1.279744631`\), \(-1.094258199`\), 3, 4, 5, 6, 
              0.269166`, \(-15.645194`\), 13.369528`, 
              0.120923`, \(-1.405507`\), 1.566049`, \(-0.497543`\), 
              10.020112`, \(-0.368011`\), \(-0.477266`\), \(-0.106272`\), \
\(-0.487572`\), 8.986854`, 8.986857`, \(-3.`*^-6\), 7.439147`, 7.439136`, 
              0.000013`, 6.360652`, 6.556455`, \(-0.223104`\), 5.524925`, 
              5.396588`, 0.146231`, 5.830797`, 5.830897`, \(-0.000114`\), 
              5.382356`, 4.559432`, 0.937664`, 4.558421`, 4.19811`, 
              0.410549`, 4.199534`, 4.158436`, 0.046828`, 8.986854`, 
              7.439147`, 5.524925`, 5.39659`, 4.158409`, 5.382356`, 
              3.824809`, 2.45082`, 5.830797`, 4.558421`, 4.199534`, 
              2.847022`, 5.259351`, 6.360652`, 3.931099`, 3.022977`}, {5, 32, 
              3.662739179`*^6, \(-0.25`\), \(-0.3095286122`\), 0.0225`, 
              1, \(-0.4`\), \(-0.01709152976`\), 
              0.8820937536`, \(-1.186589231`\), \(-1.108877762`\), 3, 4, 5, 
              6, 0.226749`, \(-7.116475`\), 6.892559`, 
              0.118522`, \(-0.450886`\), 0.876222`, 0.400095`, 5.511431`, 
              0.444047`, 0.195814`, 0.698378`, 0.317148`, 5.921535`, 
              5.921568`, \(-0.000037`\), 3.831648`, 3.460971`, 0.416257`, 
              3.461207`, 3.340666`, 0.135364`, 3.340079`, 2.359663`, 
              1.100972`, 2.108314`, 2.108155`, 0.000179`, 2.037733`, 
              1.289632`, 0.840091`, 1.289561`, 1.289283`, 0.000313`, 
              1.286647`, 1.177744`, 0.122294`, 5.921535`, 3.461207`, 
              3.340079`, 1.612723`, 1.289561`, 2.037733`, 
              1.024215`, \(-0.434112`\), 2.108314`, 1.286647`, 0.747227`, 
              0.619669`, 2.739092`, 3.831648`, 
              1.345427`, \(-0.364327`\)}, {5, 33, 
              499321.3356`, \(-0.25`\), \(-0.5563967639`\), 0.0225`, 
              1, \(-0.3`\), \(-0.03487580544`\), 
              1.201625971`, \(-1.108496152`\), \(-1.090142065`\), 3, 4, 5, 6, 
              0.239311`, \(-2.08699`\), 3.093922`, 0.120253`, 0.026793`, 
              0.151231`, 0.633956`, 0.61514`, 0.667806`, 0.389339`, 
              0.930629`, 0.659149`, 3.930903`, 3.931053`, \(-0.00032`\), 
              1.205406`, 1.202281`, 0.006657`, 0.714549`, 0.714058`, 
              0.001044`, 0.377003`, 
              0.379585`, \(-0.0055`\), \(-1.297863`\), \(-1.297786`\), \
\(-0.000165`\), \(-1.528056`\), \(-1.700177`\), 
              0.366692`, \(-1.696735`\), \(-1.951304`\), 
              0.54234`, \(-1.956821`\), \(-2.551978`\), 1.267939`, 3.930903`, 
              1.205406`, 0.714549`, 
              0.377003`, \(-2.550726`\), \(-1.528056`\), \(-4.253575`\), \
\(-4.421825`\), \(-1.696735`\), \(-1.297863`\), \(-1.956821`\), \
\(-3.330049`\), \(-0.313016`\), 
              0.272168`, \(-1.805188`\), \(-3.230691`\)}, {2, 8, 
              231.7476679`, \(-0.4586635835`\), 2.093481282`, 0.0225`, 
              1, \(-0.2`\), \(-0.101403`\), 50, \(-1.04575`\), \(-1.03912`\), 
              3, 4, 5, 6, 0.264019`, \(-0.697226`\), 0.79864`, 0.132281`, 
              0.174658`, \(-0.127561`\), 0.705951`, 0.198366`, 0.758628`, 
              0.487804`, 1.045919`, 0.826256`, 0.91927`, 0.924763`, 
              0.048579`, 5.311017`, 5.306334`, \(-0.041414`\), 9.542074`, 
              10.542016`, 8.842999`, 10.536873`, 10.566909`, 0.265626`, 
              7.972875`, 7.981415`, 0.075523`, 12.994889`, 13.002439`, 
              0.066766`, 17.375895`, 17.389219`, 0.117832`, 17.518107`, 
              17.492138`, \(-0.229656`\), 0.91927`, 5.311017`, 9.542074`, 
              10.536873`, 17.375895`, 22.159164`, 24.250017`, 28.761417`, 
              7.972875`, 12.994889`, 17.518107`, 19.488952`, 11.986852`, 
              15.080102`, 20.131022`, 20.681879`}, {1, 28, 
              8.986510587`, \(-1.66473149`\), 7.528833947`, 0.0225`, 
              1, \(-0.1`\), \(-0.09224040519`\), 
              9706.973055`, \(-1.144245056`\), \(-1.119434277`\), 3, 4, 5, 6, 
              0.323262`, \(-0.452161`\), 0.23426`, 0.132974`, 
              0.201963`, \(-0.185515`\), 0.737346`, 0.727834`, 0.78294`, 
              0.823715`, 1.07266`, 1.049022`, 27.262543`, 27.289467`, 
              1.048437`, 35.28437`, 35.26281`, \(-0.839567`\), 42.25439`, 
              42.270839`, 0.640557`, 59.874629`, 59.876746`, 0.082443`, 
              30.538569`, 30.555747`, 0.668898`, 61.655122`, 61.677814`, 
              0.883654`, 73.060477`, 73.058949`, \(-0.059486`\), 75.800723`, 
              75.801679`, 0.037257`, 27.262543`, 35.28437`, 42.25439`, 
              70.756669`, 75.800723`, 94.293583`, 116.750426`, 120.505731`, 
              30.538569`, 61.655122`, 73.060477`, 78.654768`, 59.874629`, 
              87.011501`, 91.284403`, 96.693414`}, {5, 32, 
              12.89645198`, \(-2.500880919`\), 6.263031685`, 0.0225`, 1, 
              0, \(-0.1261432904`\), 
              7581.469286`, \(-0.9008957139`\), \(-0.9390451106`\), 3, 4, 5, 
              6, 0.32876`, \(-0.443045`\), 0.187429`, 0.161609`, 
              0.166993`, \(-0.167223`\), 0.74328`, 0.700777`, 0.824143`, 
              0.835911`, 1.141619`, 1.134687`, 28.707299`, 28.739612`, 
              1.064533`, 35.560127`, 35.574597`, 0.476709`, 36.32863`, 
              36.302185`, \(-0.87122`\), 55.091042`, 55.093366`, 0.076533`, 
              25.269078`, 25.28347`, 0.474127`, 54.511547`, 54.533291`, 
              0.716368`, 61.17825`, 61.182605`, 0.143449`, 66.555234`, 
              66.562653`, 0.244426`, 28.707299`, 36.32863`, 35.560127`, 
              66.664075`, 66.555234`, 82.156905`, 104.163744`, 108.031163`, 
              25.269078`, 54.511547`, 61.17825`, 70.524244`, 55.091042`, 
              81.514001`, 84.707767`, 85.836306`}, {5, 33, 
              14.114535`, \(-2.582102713`\), 6.051051345`, 0.0225`, 1, 
              0.1`, \(-0.1532069162`\), 
              2860.206268`, \(-0.8852743183`\), \(-0.8981300259`\), 3, 4, 5, 
              6, 0.321882`, \(-0.458421`\), 0.216453`, 0.167732`, 
              0.152805`, \(-0.153592`\), 0.73723`, 0.683895`, 0.824999`, 
              0.82897`, 1.147216`, 1.144794`, 30.082492`, 30.116641`, 
              1.064229`, 34.278659`, 34.290462`, 0.367826`, 39.05647`, 
              39.03179`, \(-0.769123`\), 55.539465`, 55.541988`, 0.078626`, 
              23.78707`, 23.800986`, 0.433668`, 53.322381`, 53.344923`, 
              0.702506`, 58.834958`, 58.839517`, 0.142091`, 65.20728`, 
              65.215541`, 0.25743`, 30.082492`, 39.05647`, 34.278659`, 
              65.910307`, 65.20728`, 80.217273`, 101.897463`, 106.266387`, 
              23.78707`, 53.322381`, 58.834958`, 70.045013`, 55.539465`, 
              81.270691`, 83.391907`, 91.376678`}, {5, 33, 
              14.15072367`, \(-1.987218745`\), 6.780079219`, 0.0225`, 1, 
              0.2`, \(-0.1297557661`\), 
              207.8061508`, \(-1.189266299`\), \(-1.012437011`\), 3, 4, 5, 6, 
              0.320378`, \(-0.474338`\), 0.273124`, 0.148823`, 
              0.170152`, \(-0.162062`\), 0.726372`, 0.699053`, 0.791552`, 
              0.816396`, 1.096469`, 1.082467`, 35.796451`, 35.827018`, 
              1.06238`, 38.521678`, 38.531222`, 0.331727`, 45.985681`, 
              45.967029`, \(-0.648261`\), 63.852731`, 63.854477`, 0.060694`, 
              25.867078`, 25.879123`, 0.418626`, 60.235674`, 60.257344`, 
              0.753132`, 65.584507`, 65.584902`, 0.013719`, 73.328265`, 
              73.335262`, 0.243158`, 35.796451`, 38.521678`, 45.985681`, 
              74.404461`, 73.328265`, 89.889166`, 114.553226`, 119.546896`, 
              25.867078`, 60.235674`, 65.584507`, 79.186276`, 63.852731`, 
              92.889797`, 93.983961`, 105.0576`}, {\(-2\), 28, 
              17.68847471`, \(-1.61488093`\), 5.287587543`, 0.0225`, 1, 
              0.3`, \(-0.2930629861`\), 
              50.44028799`, \(-0.6962892692`\), \(-0.9209830874`\), 3, 4, 5, 
              6, 0.249339`, \(-0.565266`\), 0.44623`, 0.186271`, 
              0.111234`, \(-0.109235`\), 0.691879`, 0.561672`, 0.766466`, 
              0.75849`, 1.072441`, 1.145146`, 23.65599`, 23.707262`, 
              1.081559`, 27.265146`, 27.27655`, 0.240566`, 36.641373`, 
              36.610177`, \(-0.658053`\), 46.488136`, 46.493687`, 0.117091`, 
              18.182599`, 18.200856`, 0.385124`, 41.618432`, 41.647187`, 
              0.606565`, 49.030764`, 49.041035`, 0.216651`, 53.110484`, 
              53.116784`, 0.132882`, 23.65599`, 36.641373`, 27.265146`, 
              53.859569`, 53.110484`, 65.592857`, 84.986686`, 88.176967`, 
              18.182599`, 41.618432`, 49.030764`, 59.411738`, 46.488136`, 
              68.352606`, 70.718852`, 78.068189`}, {2, 12, 
              19.88521517`, \(-1.352992937`\), 5.551417647`, 0.0225`, 1, 
              0.4`, \(-0.3365752895`\), 
              13.30701909`, \(-1.102684575`\), \(-0.9319655001`\), 3, 4, 5, 
              6, 0.217166`, \(-0.642523`\), 0.635127`, 0.160846`, 
              0.130407`, \(-0.110091`\), 0.663596`, 0.503964`, 0.731273`, 
              0.678465`, 1.028413`, 1.016356`, 23.676297`, 23.734133`, 
              1.115608`, 25.754286`, 25.767707`, 0.258882`, 38.331914`, 
              38.299673`, \(-0.62189`\), 47.18288`, 47.189409`, 0.125938`, 
              17.596014`, 17.616128`, 0.387968`, 41.02434`, 41.055766`, 
              0.606188`, 50.138169`, 50.149484`, 0.218253`, 53.306995`, 
              53.312265`, 0.101657`, 23.676297`, 25.754286`, 38.331914`, 
              53.498974`, 53.306995`, 66.011208`, 86.682024`, 89.566592`, 
              17.596014`, 41.02434`, 50.138169`, 60.595213`, 47.18288`, 
              69.729222`, 72.553893`, 79.369528`}, {5, 33, 
              1.595120815`*^7, \(-0.25`\), \(-0.2947710718`\), 0.04`, 
              1, \(-0.6`\), 0.0156875879`, 
              3.098203096`, \(-1.540354925`\), \(-1.133862525`\), 3, 4, 5, 6, 
              0.298353`, \(-18.424025`\), 15.07608`, 
              0.13882`, \(-1.780636`\), 1.70725`, \(-0.809771`\), 
              8.845171`, \(-0.677665`\), \(-0.751378`\), \(-0.422032`\), \
\(-0.744902`\), 11.769359`, 11.769361`, \(-3.`*^-6\), 9.782536`, 9.782517`, 
              0.000027`, 8.150907`, 8.584978`, \(-0.610795`\), 7.385606`, 
              7.118534`, 0.375806`, 8.003011`, 8.00307`, \(-0.000083`\), 
              7.150772`, 6.514725`, 0.895001`, 6.513038`, 5.757517`, 
              1.063118`, 5.74014`, 5.738334`, 0.002542`, 11.769359`, 
              9.782536`, 7.385606`, 7.118535`, 5.504785`, 7.150772`, 
              5.341617`, 4.729672`, 8.003011`, 6.513038`, 5.74014`, 
              4.499769`, 6.808941`, 8.150907`, 5.721977`, 5.220233`}, {5, 33, 
              6.422746072`*^6, \(-0.25`\), \(-0.2646692592`\), 0.04`, 
              1, \(-0.5`\), \(-0.002044947432`\), 
              1.536293166`, \(-1.43291709`\), \(-1.151752077`\), 3, 4, 5, 6, 
              0.263617`, \(-9.671315`\), 8.490239`, 
              0.130835`, \(-0.796284`\), 1.24761`, 0.22792`, 8.402457`, 
              0.308185`, 0.029594`, 0.599415`, 0.107562`, 6.703975`, 
              6.703983`, \(-0.00001`\), 4.879592`, 4.879626`, \(-0.000038`\), 
              4.316982`, 3.591583`, 0.809794`, 3.591582`, 2.983313`, 
              0.679037`, 3.228704`, 3.228912`, \(-0.000232`\), 2.940052`, 
              2.074044`, 0.966761`, 2.072877`, 1.936911`, 0.151784`, 
              1.936898`, 1.56465`, 0.415557`, 6.703975`, 4.879592`, 
              3.591582`, 1.744223`, 1.936898`, 2.940052`, 1.853119`, 
              0.704501`, 3.228704`, 2.072877`, 1.55866`, 0.96269`, 3.274792`, 
              4.316982`, 2.028825`, 0.917766`}, {5, 33, 
              1.676578381`*^6, \(-0.25`\), \(-0.3115875346`\), 0.04`, 
              1, \(-0.4`\), \(-0.02709238851`\), 
              3.147529048`, \(-1.2793697`\), \(-1.144747592`\), 3, 4, 5, 6, 
              0.25057`, \(-3.746041`\), 4.578447`, 0.124572`, \(-0.156879`\), 
              0.430219`, 0.592045`, 2.620873`, 0.627667`, 0.32691`, 
              0.895605`, 0.552285`, 4.033489`, 4.033523`, \(-0.000042`\), 
              2.140739`, 2.049606`, 0.113843`, 2.049093`, 1.548429`, 
              0.625425`, 1.54847`, 0.551392`, 1.245545`, 
              0.132171`, \(-0.043563`\), 
              0.219527`, \(-0.045217`\), \(-0.044298`\), \(-0.001148`\), \
\(-0.045922`\), \(-0.351721`\), 0.382002`, \(-0.348996`\), \(-0.571779`\), 
              0.278299`, 4.033489`, 1.54847`, 2.049093`, 
              0.079823`, \(-0.571277`\), 
              0.132171`, \(-0.487914`\), \(-2.341354`\), \(-0.348996`\), \
\(-0.045217`\), \(-0.045922`\), \(-0.815682`\), 0.90462`, 
              2.140739`, \(-0.248045`\), \(-1.761741`\)}, {5, 34, 
              132807.0503`, \(-0.25`\), 0.9610848002`, 0.04`, 
              1, \(-0.3`\), \(-0.08219101985`\), 
              7.320878636`, \(-1.182100266`\), \(-1.140202831`\), 3, 4, 5, 6, 
              0.269513`, \(-1.373142`\), 2.054192`, 0.132146`, 0.080224`, 
              0.032868`, 0.705448`, 0.212033`, 0.748071`, 0.429997`, 
              1.031944`, 0.750775`, \(-3.500367`\), \(-3.4999`\), 0.001937`, 
              0.413869`, 0.4136`, \(-0.001114`\), 1.009268`, 2.348434`, 
              5.550043`, 2.192835`, 2.426301`, 0.967576`, 3.599659`, 
              3.604124`, 0.018505`, 4.4783`, 4.48112`, 0.011686`, 6.133147`, 
              6.12467`, \(-0.035133`\), 6.695988`, 6.698925`, 
              0.012173`, \(-3.500367`\), 0.413869`, 2.192835`, 2.426158`, 
              6.695988`, 7.665117`, 8.385718`, 11.409802`, 3.599659`, 
              4.4783`, 6.133147`, 8.422492`, 3.609343`, 1.009268`, 6.722655`, 
              9.303579`}, {1, 21, 10.31076161`, \(-0.8270936664`\), 
              7.093756186`, 0.04`, 1, \(-0.2`\), \(-0.1267540017`\), 
              2759.045373`, \(-1.134852037`\), \(-1.217307382`\), 3, 4, 5, 6, 
              0.315908`, \(-0.542805`\), 0.419818`, 0.143158`, 
              0.170757`, \(-0.145844`\), 0.752813`, 0.704963`, 0.789938`, 
              0.813475`, 1.077046`, 1.063457`, 14.409247`, 14.43696`, 
              0.99368`, 27.564285`, 27.540894`, \(-0.838722`\), 40.262959`, 
              40.278819`, 0.568666`, 51.045687`, 51.049144`, 0.123924`, 
              29.494886`, 29.518457`, 0.845158`, 53.775189`, 53.802341`, 
              0.973533`, 68.584017`, 68.585331`, 0.047097`, 68.705611`, 
              68.673301`, \(-1.158496`\), 14.409247`, 27.564285`, 40.262959`, 
              52.281932`, 68.584017`, 86.003255`, 103.636001`, 110.321827`, 
              29.494886`, 53.775189`, 68.705611`, 70.503499`, 51.045687`, 
              67.660733`, 85.948524`, 87.87811`}, {5, 32, 
              11.57583103`, \(-1.932069646`\), 7.211573783`, 0.04`, 
              1, \(-0.1`\), \(-0.09172297512`\), 
              8371.426987`, \(-1.275317449`\), \(-1.230476774`\), 3, 4, 5, 6, 
              0.378927`, \(-0.440552`\), 0.171781`, 0.13894`, 
              0.18825`, \(-0.177232`\), 0.777637`, 0.765115`, 0.818976`, 
              0.861334`, 1.115218`, 1.090743`, 33.016299`, 33.040393`, 
              1.05348`, 40.284429`, 40.265612`, \(-0.822704`\), 43.110202`, 
              43.12271`, 0.54687`, 65.164559`, 65.165913`, 0.059183`, 
              31.474296`, 31.487013`, 0.555997`, 66.715288`, 66.734942`, 
              0.859323`, 73.263027`, 73.263873`, 0.036995`, 80.401053`, 
              80.399031`, \(-0.088415`\), 33.016299`, 40.284429`, 43.110202`, 
              78.322096`, 80.401053`, 98.618059`, 125.250973`, 125.850614`, 
              31.474296`, 66.715288`, 73.263027`, 82.563242`, 65.164559`, 
              94.175491`, 98.779916`, 101.566497`}, {1, 32, 
              15.60724641`, \(-2.939285092`\), 6.154057954`, 0.04`, 1, 
              0, \(-0.1142034576`\), 
              6530.925736`, \(-1.041047266`\), \(-1.060385832`\), 3, 4, 5, 6, 
              0.386547`, \(-0.440467`\), 0.15268`, 0.164499`, 
              0.155569`, \(-0.156276`\), 0.782795`, 0.744123`, 0.855851`, 
              0.873219`, 1.176722`, 1.166431`, 33.431105`, 33.458998`, 
              1.061624`, 37.591381`, 37.599691`, 0.316258`, 40.384144`, 
              40.364085`, \(-0.763505`\), 60.324322`, 60.325638`, 0.050094`, 
              26.781981`, 26.792587`, 0.403677`, 59.659402`, 59.678225`, 
              0.716436`, 63.077385`, 63.077923`, 0.020493`, 71.784181`, 
              71.788474`, 0.16341`, 33.431105`, 40.384144`, 37.591381`, 
              70.431007`, 71.784181`, 87.428476`, 109.814577`, 114.144645`, 
              26.781981`, 59.659402`, 63.077385`, 75.104181`, 60.324322`, 
              86.603183`, 90.054526`, 95.521018`}, {5, 32, 
              14.73286189`, \(-2.794823287`\), 6.864433803`, 0.04`, 1, 
              0.1`, \(-0.1068604623`\), 
              181.158633`, \(-1.332372393`\), \(-1.191001442`\), 3, 4, 5, 6, 
              0.390291`, \(-0.446449`\), 0.17021`, 0.146041`, 
              0.17593`, \(-0.171388`\), 0.779812`, 0.762253`, 0.829808`, 
              0.866343`, 1.134335`, 1.111908`, 39.393874`, 39.418497`, 
              1.055453`, 43.580383`, 43.590155`, 0.418895`, 49.216212`, 
              49.19924`, \(-0.72752`\), 70.366064`, 70.367157`, 0.04684`, 
              29.444805`, 29.454664`, 0.42263`, 67.636877`, 67.65657`, 
              0.844186`, 70.850624`, 70.848456`, \(-0.092944`\), 81.705933`, 
              81.710045`, 0.176285`, 39.393874`, 43.580383`, 49.216212`, 
              79.167681`, 81.705933`, 99.0112`, 124.320134`, 129.530416`, 
              29.444805`, 67.636877`, 70.850624`, 86.261524`, 70.366064`, 
              99.839246`, 102.761922`, 112.724194`}, {\(-2\), 31, 
              19.1431102`, \(-2.057446522`\), 4.79010388`, 0.04`, 1, 
              0.2`, \(-0.2859317135`\), 
              3078.813972`, \(-0.3022579183`\), \(-0.7367789379`\), 3, 4, 5, 
              6, 0.318438`, \(-0.498003`\), 0.234643`, 0.213734`, 
              0.062547`, \(-0.067178`\), 0.754884`, 0.615434`, 0.880846`, 
              0.831256`, 1.235852`, 1.265569`, 24.536275`, 24.579812`, 
              1.062557`, 27.95222`, 27.96304`, 0.264071`, 36.158496`, 
              36.128394`, \(-0.734651`\), 46.776386`, 46.780572`, 0.102161`, 
              19.05183`, 19.066967`, 0.369419`, 43.179586`, 43.204488`, 
              0.607748`, 47.362549`, 47.370444`, 0.192677`, 53.925322`, 
              53.928519`, 0.078014`, 24.536275`, 36.158496`, 27.95222`, 
              53.352956`, 53.925322`, 65.667615`, 83.42255`, 86.50785`, 
              19.05183`, 43.179586`, 47.362549`, 58.968544`, 46.776386`, 
              67.264674`, 69.452864`, 76.995981`}, {5, 30, 
              11.54917753`, \(-1.74239671`\), 5.422944838`, 0.04`, 1, 
              0.3`, \(-0.3213444144`\), 
              6.631339126`, \(-0.7186422985`\), \(-0.9274075343`\), 3, 4, 5, 
              6, 0.295968`, \(-0.540196`\), 0.338292`, 0.193682`, 
              0.096387`, \(-0.096085`\), 0.736231`, 0.635226`, 0.82366`, 
              0.820512`, 1.148583`, 1.204256`, 24.452399`, 24.49297`, 
              1.041859`, 29.274667`, 29.305689`, 0.796653`, 41.547104`, 
              41.516927`, \(-0.774956`\), 50.758918`, 50.796968`, 0.977135`, 
              20.50833`, 20.525824`, 0.449264`, 47.341294`, 47.370343`, 
              0.745993`, 52.661519`, 52.670286`, 0.225126`, 59.674358`, 
              59.67654`, 0.056048`, 24.452399`, 29.274667`, 41.547104`, 
              50.758918`, 59.674358`, 72.629713`, 93.005946`, 96.156926`, 
              20.50833`, 47.341294`, 52.661519`, 66.00639`, 52.457636`, 
              75.396617`, 77.789163`, 86.238502`}, {5, 34, 
              15.32954057`, \(-1.479579248`\), 5.480462194`, 0.04`, 1, 
              0.4`, \(-0.3656639314`\), 
              11.90484039`, \(-0.6931793947`\), \(-0.8854560736`\), 3, 4, 5, 
              6, 0.264661`, \(-0.601496`\), 0.485672`, 0.19674`, 
              0.086342`, \(-0.083392`\), 0.711491`, 0.611461`, 0.801856`, 
              0.805565`, 1.12591`, 1.201131`, 25.765387`, 25.809866`, 
              1.032247`, 28.523328`, 28.550868`, 0.639141`, 42.065617`, 
              42.032278`, \(-0.773722`\), 51.507502`, 51.513513`, 0.139498`, 
              19.372384`, 19.391107`, 0.434517`, 45.337337`, 45.368258`, 
              0.717593`, 51.918658`, 51.927916`, 0.214839`, 58.022409`, 
              58.022791`, 0.008854`, 25.765387`, 28.523328`, 42.065617`, 
              55.758861`, 58.022409`, 70.704295`, 91.584359`, 94.279588`, 
              19.372384`, 45.337337`, 51.918658`, 65.063272`, 51.507502`, 
              74.289996`, 76.982327`, 85.086781`}, {5, 33, 
              1.034388894`*^6, \(-0.25`\), \(-0.2114057659`\), 0.0625`, 
              1, \(-0.6`\), \(-0.01364990998`\), \(-1017.04646`\), \
\(-1.670114844`\), \(-1.113330424`\), 3, 4, 5, 6, 0.296481`, \(-11.621112`\), 
              9.500427`, 0.146653`, \(-1.079265`\), 1.522627`, 0.064258`, 
              12.165125`, 0.174823`, \(-0.103839`\), 
              0.503707`, \(-0.041363`\), 6.496513`, 6.496516`, \(-3.`*^-6\), 
              4.922717`, 4.922734`, \(-0.000017`\), 3.974747`, 3.974373`, 
              0.000375`, 3.604508`, 3.604482`, 0.000026`, 3.466463`, 
              3.466561`, \(-0.000098`\), 3.127418`, 2.230227`, 0.899744`, 
              2.22954`, 2.052202`, 0.177843`, 2.05315`, 2.042632`, 0.010547`, 
              6.496513`, 4.922717`, 3.604508`, 3.128002`, 2.043104`, 
              3.127418`, 1.886196`, 1.823362`, 3.466463`, 2.22954`, 2.05315`, 
              1.035197`, 3.178065`, 3.974747`, 2.228391`, 1.987626`}, {5, 32, 
              2.895160895`*^6, \(-0.25`\), \(-0.4966423953`\), 0.0625`, 
              1, \(-0.5`\), \(-0.01211459018`\), \(-1.322604208`\), \
\(-1.328050569`\), \(-1.242622385`\), 3, 4, 5, 6, 0.275914`, \(-5.26889`\), 
              5.421069`, 0.139085`, \(-0.326291`\), 0.682665`, 0.539063`, 
              2.326828`, 0.575749`, 0.294901`, 0.849184`, 0.506429`, 
              8.44957`, 8.449714`, \(-0.000316`\), 4.637173`, 4.636943`, 
              0.000505`, 4.588837`, 3.814285`, 1.6982`, 3.814288`, 3.630648`, 
              0.402629`, 1.355742`, 1.313044`, 0.093617`, 1.314111`, 
              0.998832`, 0.691248`, 1.197315`, 0.517867`, 1.489685`, 
              0.996105`, 0.261223`, 1.611223`, 8.44957`, 3.814288`, 
              4.637173`, 3.630878`, 0.262273`, 1.355742`, 
              1.197315`, \(-2.047651`\), 1.314111`, 0.518872`, 0.996105`, 
              0.02165`, 2.837511`, 4.588837`, 0.383162`, \(-0.779435`\)}, {5, 
              33, 631162.7368`, \(-0.25`\), \(-0.172747921`\), 0.0625`, 
              1, \(-0.4`\), \(-0.06498090853`\), 
              7.981933904`, \(-1.26729542`\), \(-1.305099439`\), 3, 4, 5, 6, 
              0.285266`, \(-2.17344`\), 3.176591`, 0.138123`, \(-0.014607`\), 
              0.17903`, 0.697319`, 1.938526`, 0.731937`, 0.402499`, 
              1.014626`, 0.711925`, 1.361981`, 1.361982`, \(-1.`*^-6\), 
              0.297059`, 0.29776`, \(-0.000553`\), 0.29542`, 0.271319`, 
              0.019003`, 0.271342`, \(-0.326736`\), 
              0.471564`, \(-0.513259`\), \(-0.513512`\), 
              0.0002`, \(-0.596383`\), \(-0.597286`\), 
              0.000712`, \(-0.641203`\), \(-0.715323`\), 
              0.058441`, \(-0.716651`\), \(-0.951098`\), 0.184854`, 
              1.361981`, 0.271342`, 
              0.297059`, \(-0.501277`\), \(-0.950496`\), \(-1.005495`\), \
\(-0.641203`\), \(-2.151984`\), \(-0.596383`\), \(-0.513259`\), \
\(-0.716651`\), \(-1.262624`\), \(-0.282366`\), 
              0.29542`, \(-0.972445`\), \(-1.334024`\)}, {5, 33, 
              121.5757198`, \(-0.4053987812`\), 1.883177127`, 0.0625`, 
              1, \(-0.3`\), \(-0.08863716817`\), 440.5800944`, 
              0.1935781001`, \(-1.278723061`\), 3, 4, 5, 6, 
              0.346984`, \(-0.721311`\), 0.797185`, 0.224908`, 
              0.03592`, \(-0.040455`\), 0.76766`, 0.248939`, 0.803765`, 
              0.636263`, 1.095638`, 1.179902`, \(-0.06522`\), \(-0.063407`\), 
              0.018954`, 5.644011`, 5.64265`, \(-0.014223`\), 9.779822`, 
              10.290399`, 5.338046`, 10.51641`, 10.519612`, 0.033478`, 
              8.095981`, 8.101014`, 0.052622`, 13.613369`, 13.6197`, 
              0.066195`, 16.693127`, 16.681086`, \(-0.125887`\), 17.566842`, 
              17.921618`, 3.709164`, \(-0.06522`\), 5.644011`, 9.779822`, 
              10.51641`, 17.566842`, 22.183038`, 22.664524`, 29.538351`, 
              8.095981`, 13.613369`, 16.693127`, 18.577174`, 12.239741`, 
              13.214092`, 21.569402`, 22.646886`}, {5, 33, 
              9.036641995`, \(-1.165151366`\), 6.649656975`, 0.0625`, 
              1, \(-0.2`\), \(-0.1295479436`\), 
              1272.349249`, \(-1.237504076`\), \(-1.302105931`\), 3, 4, 5, 6, 
              0.383009`, \(-0.461591`\), 0.19719`, 0.15295`, 
              0.158089`, \(-0.144682`\), 0.798984`, 0.752373`, 0.837997`, 
              0.864525`, 1.137599`, 1.126223`, 23.135469`, 23.160904`, 
              1.036492`, 34.229059`, 34.208062`, \(-0.855629`\), 40.669888`, 
              40.681683`, 0.480623`, 57.467373`, 57.469578`, 0.089857`, 
              30.127026`, 30.14312`, 0.655824`, 59.871187`, 59.894182`, 
              0.937078`, 68.667488`, 68.668229`, 0.030207`, 73.53576`, 
              73.537722`, 0.079959`, 23.135469`, 34.229059`, 40.669888`, 
              65.55835`, 73.53576`, 90.386857`, 107.996523`, 119.803633`, 
              30.127026`, 59.871187`, 68.667488`, 74.17938`, 57.467373`, 
              81.297175`, 88.660884`, 93.921676`}, {5, 32, 
              14.43910715`, \(-2.155015267`\), 6.810942949`, 0.0625`, 
              1, \(-0.1`\), \(-0.09358223077`\), 
              1671.291147`, \(-1.367094366`\), \(-1.323322431`\), 3, 4, 5, 6, 
              0.436012`, \(-0.42982`\), 0.111615`, 0.147503`, 
              0.171843`, \(-0.166141`\), 0.817482`, 0.798274`, 0.857879`, 
              0.898531`, 1.162814`, 1.139101`, 37.315738`, 37.337863`, 
              1.051249`, 43.58665`, 43.596134`, 0.450644`, 44.422775`, 
              44.405636`, \(-0.814343`\), 68.985157`, 68.986066`, 0.043167`, 
              31.860707`, 31.870387`, 0.459976`, 69.815111`, 69.833218`, 
              0.860348`, 72.667421`, 72.666434`, \(-0.046885`\), 83.467438`, 
              83.465779`, \(-0.078846`\), 37.315738`, 44.422775`, 43.58665`, 
              80.097074`, 83.467438`, 101.022561`, 126.185074`, 130.011927`, 
              31.860707`, 69.815111`, 72.667421`, 85.135099`, 68.985157`, 
              97.664783`, 102.751338`, 107.890303`}, {5, 34, 
              16.09731379`, \(-3.213821229`\), 6.605382848`, 0.0625`, 1, 
              0, \(-0.1126497193`\), 
              3120.584511`, \(-1.3652209`\), \(-1.300842598`\), 3, 4, 5, 6, 
              0.442789`, \(-0.434847`\), 0.107979`, 0.152453`, 
              0.165638`, \(-0.163875`\), 0.821598`, 0.798442`, 0.867256`, 
              0.904124`, 1.177351`, 1.155942`, 39.845018`, 39.86783`, 
              1.067522`, 44.514802`, 44.526198`, 0.533301`, 48.910434`, 
              48.890931`, \(-0.912673`\), 71.412081`, 71.413054`, 0.045511`, 
              30.82569`, 30.834962`, 0.4339`, 69.010353`, 69.02453`, 
              0.663454`, 72.211672`, 72.214678`, 0.140692`, 83.986463`, 
              83.989995`, 0.165272`, 39.845018`, 44.514802`, 48.910434`, 
              79.161076`, 83.986463`, 101.015024`, 125.740765`, 130.970069`, 
              30.82569`, 69.010353`, 72.211672`, 87.16339`, 71.412081`, 
              99.913441`, 104.293391`, 115.060578`}, {5, 33, 
              14.0372776`, \(-2.291816609`\), 4.99379682`, 0.0625`, 1, 
              0.1`, \(-0.2722689278`\), 
              8.626732745`, \(-0.6162844581`\), \(-0.9906347731`\), 3, 4, 5, 
              6, 0.384371`, \(-0.45595`\), 0.097578`, 0.203691`, 
              0.083602`, \(-0.088573`\), 0.807542`, 0.685628`, 0.902169`, 
              0.871907`, 1.244998`, 1.266434`, 25.557488`, 25.592119`, 
              1.063557`, 30.943888`, 30.966164`, 0.684117`, 40.052269`, 
              40.020901`, \(-0.96337`\), 51.169038`, 51.196326`, 0.838056`, 
              22.231903`, 22.245847`, 0.428238`, 50.022763`, 50.045517`, 
              0.698792`, 52.413603`, 52.421477`, 0.241815`, 61.674787`, 
              61.676191`, 0.04313`, 25.557488`, 30.943888`, 40.052269`, 
              51.169038`, 61.674787`, 74.305562`, 93.07819`, 96.472332`, 
              22.231903`, 50.022763`, 52.413603`, 66.147702`, 53.047615`, 
              74.927946`, 77.650671`, 85.597522`}, {5, 33, 
              11.57795483`, \(-2.194475247`\), 5.445659661`, 0.0625`, 1, 
              0.2`, \(-0.2949504752`\), 
              9.91024033`, \(-0.9160295952`\), \(-1.009220212`\), 3, 4, 5, 6, 
              0.374942`, \(-0.476717`\), 0.148688`, 0.188663`, 
              0.105579`, \(-0.106887`\), 0.797969`, 0.702623`, 0.887778`, 
              0.866205`, 1.226208`, 1.22164`, 28.601676`, 28.633779`, 
              1.048778`, 34.593247`, 34.621866`, 0.934977`, 45.577279`, 
              45.545304`, \(-1.044591`\), 57.473929`, 57.501231`, 0.891941`, 
              23.605317`, 23.620145`, 0.484405`, 54.213351`, 54.238182`, 
              0.811207`, 56.786913`, 56.794623`, 0.251879`, 67.177844`, 
              67.178508`, 0.021681`, 28.601676`, 34.593247`, 45.577279`, 
              57.473929`, 67.177844`, 80.691848`, 101.425938`, 104.913003`, 
              23.605317`, 54.213351`, 56.786913`, 72.672819`, 58.719162`, 
              82.514514`, 84.968472`, 93.941909`}, {5, 33, 
              9.540882195`, \(-1.885046805`\), 6.266222187`, 0.0625`, 1, 
              0.3`, \(-0.287411533`\), 
              11.91711071`, \(-1.403222256`\), \(-1.221891456`\), 3, 4, 5, 6, 
              0.367977`, \(-0.492544`\), 0.207421`, 0.159341`, 
              0.139153`, \(-0.131612`\), 0.786341`, 0.710209`, 0.83759`, 
              0.835161`, 1.146569`, 1.118991`, 33.57284`, 33.600958`, 
              1.037366`, 41.031399`, 41.064268`, 1.212622`, 53.609305`, 
              53.577807`, \(-1.162063`\), 68.137812`, 68.163449`, 0.945821`, 
              26.378277`, 26.393495`, 0.56146`, 61.995768`, 62.022653`, 
              0.991881`, 64.830323`, 64.83577`, 0.200981`, 76.897159`, 
              76.898841`, 0.062065`, 33.57284`, 41.031399`, 53.609305`, 
              68.137812`, 76.897159`, 92.063245`, 116.26554`, 120.101952`, 
              26.378277`, 61.995768`, 64.830323`, 83.628115`, 68.174253`, 
              95.535413`, 97.558036`, 108.22136`}, {5, 34, 
              11.67771023`, \(-1.598273863`\), 5.399950781`, 0.0625`, 1, 
              0.4`, \(-0.4064560862`\), 
              8.49379086`, \(-0.835483499`\), \(-0.9801718067`\), 3, 4, 5, 6, 
              0.312432`, \(-0.56364`\), 0.340748`, 0.196518`, 
              0.083983`, \(-0.080934`\), 0.75744`, 0.650327`, 0.844168`, 
              0.833532`, 1.175191`, 1.217872`, 26.226098`, 26.264012`, 
              1.023441`, 31.613237`, 31.65288`, 1.070113`, 45.70649`, 
              45.676915`, \(-0.798342`\), 54.769854`, 54.802666`, 0.885707`, 
              21.050771`, 21.068489`, 0.478288`, 49.494723`, 49.527478`, 
              0.884188`, 53.520897`, 53.527105`, 0.167573`, 62.625631`, 
              62.620669`, \(-0.133952`\), 26.226098`, 31.613237`, 45.70649`, 
              54.769854`, 62.625631`, 75.235678`, 96.275609`, 98.643013`, 
              21.050771`, 49.494723`, 53.520897`, 69.52519`, 55.760146`, 
              78.662523`, 81.39741`, 89.591216`}, {5, 32, 
              4.565945047`*^6, \(-0.25`\), \(-0.2974658506`\), 0.09`, 
              1, \(-0.6`\), \(-0.008150280401`\), \(-1.258195576`\), \
\(-1.708793034`\), \(-1.253944285`\), 3, 4, 5, 6, 0.297147`, \(-6.625055`\), 
              6.051253`, 0.13885`, \(-0.553361`\), 0.94259`, 0.488019`, 
              5.282154`, 0.541948`, 0.170739`, 0.836344`, 0.329673`, 
              6.35454`, 6.354591`, \(-0.000073`\), 5.607432`, 5.607059`, 
              0.000528`, 3.287809`, 3.287639`, 0.00024`, 3.262813`, 
              2.347412`, 1.294611`, 1.852128`, 1.852186`, \(-0.000083`\), 
              1.290682`, 1.08803`, 0.286602`, 1.260565`, 0.718182`, 
              0.767067`, 1.086649`, 0.644898`, 0.624748`, 6.35454`, 
              5.607432`, 3.287809`, 0.216209`, 0.719043`, 1.260565`, 
              1.290682`, 0.532223`, 1.852128`, 0.646071`, 1.086649`, 
              0.50674`, 2.34741`, 3.262813`, 1.333724`, 0.582735`}, {5, 32, 
              1.0838068`*^6, \(-0.25`\), \(-0.4072311228`\), 0.09`, 
              1, \(-0.5`\), 
              0.02024621386`, \(-4.181742079`\), \(-1.386696956`\), \
\(-1.338854999`\), 3, 4, 5, 6, 0.350517`, \(-2.360067`\), 3.220077`, 
              0.147808`, \(-0.101858`\), 0.305151`, 0.687807`, 1.199445`, 
              0.717003`, 0.390874`, 1.001438`, 0.699002`, 4.225615`, 
              4.225874`, \(-0.000591`\), 3.478431`, 
              3.478446`, \(-0.000034`\), 1.716743`, 
              1.717083`, \(-0.000777`\), 0.939458`, 0.939452`, 
              0.000014`, \(-0.500363`\), \(-0.500334`\), \(-0.000066`\), \
\(-1.05014`\), \(-1.242517`\), 0.439362`, \(-1.243707`\), \(-1.267316`\), 
              0.05392`, \(-1.266868`\), \(-1.92169`\), 1.495524`, 3.478431`, 
              4.225615`, 1.716743`, 
              0.939458`, \(-1.907688`\), \(-1.923112`\), \(-1.05014`\), \
\(-3.45471`\), \(-1.266868`\), \(-0.500363`\), \(-1.243707`\), \
\(-2.293588`\), \(-0.033718`\), 
              0.90355`, \(-1.791215`\), \(-2.272398`\)}, {5, 34, 
              143585.0156`, \(-0.25`\), 0.5769569181`, 0.09`, 
              1, \(-0.4`\), \(-0.1050693303`\), 
              11.03713807`, \(-1.217666538`\), \(-1.281111194`\), 3, 4, 5, 6, 
              0.338526`, \(-1.33166`\), 1.893083`, 0.158357`, 0.050329`, 
              0.039459`, 0.772424`, 0.221034`, 0.816909`, 0.465418`, 
              1.120889`, 0.851771`, \(-2.184949`\), \(-2.184628`\), 
              0.001005`, 0.623745`, 0.623552`, \(-0.000605`\), 1.650006`, 
              1.875753`, 0.705469`, 2.145192`, 2.392376`, 0.77246`, 
              2.442791`, 2.444858`, 0.006459`, 3.312769`, 3.315006`, 
              0.006991`, 3.842215`, 4.184813`, 1.070631`, 4.18905`, 
              5.092258`, 2.822562`, \(-2.184949`\), 0.623745`, 1.650006`, 
              2.391717`, 4.705194`, 6.096058`, 3.842215`, 7.880921`, 
              2.442791`, 3.312769`, 4.18905`, 5.495991`, 2.767811`, 
              2.145192`, 4.57331`, 6.588904`}, {5, 30, 
              8.140432639`, \(-0.5076145742`\), 5.45282897`, 0.09`, 
              1, \(-0.3`\), \(-0.2074708064`\), 
              1759.001869`, \(-0.924350609`\), \(-1.3291559`\), 3, 4, 5, 6, 
              0.349821`, \(-0.680806`\), 0.600233`, 0.182344`, 
              0.103287`, \(-0.087409`\), 0.811117`, 0.664266`, 0.853412`, 
              0.825943`, 1.160097`, 1.173529`, 7.099811`, 7.134016`, 
              1.04396`, 22.235339`, 22.205445`, \(-0.912373`\), 33.163109`, 
              33.174374`, 0.343831`, 41.224596`, 41.250205`, 0.781603`, 
              24.748759`, 24.771164`, 0.683833`, 43.359122`, 43.387201`, 
              0.856971`, 55.889289`, 55.873353`, \(-0.48638`\), 56.410475`, 
              56.458707`, 1.47207`, 7.099811`, 22.235339`, 33.163109`, 
              41.224596`, 56.410475`, 70.458806`, 75.788065`, 88.589381`, 
              24.748759`, 43.359122`, 55.889289`, 56.817186`, 41.294762`, 
              56.843021`, 61.023986`, 77.372147`}, {5, 30, 
              12.46394067`, \(-1.427079458`\), 6.186710732`, 0.09`, 
              1, \(-0.2`\), \(-0.1361145972`\), 
              631.2457404`, \(-1.258143284`\), \(-1.353769756`\), 3, 4, 5, 6, 
              0.450493`, \(-0.419928`\), 0.060541`, 0.165104`, 
              0.139621`, \(-0.13418`\), 0.843381`, 0.792893`, 0.889213`, 
              0.912187`, 1.204318`, 1.192061`, 29.522429`, 29.54568`, 
              1.036845`, 39.261617`, 39.242437`, \(-0.855332`\), 40.497493`, 
              40.506456`, 0.399655`, 61.917371`, 61.918805`, 0.063945`, 
              30.261349`, 30.272894`, 0.514794`, 63.927798`, 63.948934`, 
              0.942533`, 67.061295`, 67.062096`, 0.035691`, 76.734138`, 
              76.737132`, 0.133483`, 29.522429`, 39.261617`, 40.497493`, 
              71.711513`, 76.734138`, 92.773536`, 115.752771`, 116.786506`, 
              30.261349`, 63.927798`, 67.061295`, 77.026576`, 61.917371`, 
              87.70488`, 91.377892`, 99.809754`}, {5, 34, 
              17.60299526`, \(-2.282894752`\), 6.725645006`, 0.09`, 
              1, \(-0.1`\), \(-0.1080961134`\), 
              36.3162816`, \(-1.500706608`\), \(-1.452730724`\), 3, 4, 5, 6, 
              0.491451`, \(-0.418638`\), 0.04694`, 0.151449`, 
              0.161442`, \(-0.158826`\), 0.857055`, 0.841825`, 0.890563`, 
              0.939919`, 1.199803`, 1.179154`, 40.745756`, 40.765915`, 
              1.066139`, 46.757489`, 46.768992`, 0.60834`, 50.5004`, 
              50.481712`, \(-0.988308`\), 75.450936`, 75.451738`, 0.042458`, 
              33.589358`, 33.597959`, 0.454882`, 73.282727`, 73.29255`, 
              0.519464`, 77.334971`, 77.341597`, 0.350418`, 89.915123`, 
              89.91674`, 0.085548`, 40.745756`, 46.757489`, 50.5004`, 
              81.158935`, 89.915123`, 107.507768`, 133.099175`, 138.530838`, 
              33.589358`, 73.282727`, 77.334971`, 91.461079`, 75.450936`, 
              104.194992`, 110.629572`, 119.753236`}, {5, 33, 
              13.73519764`, \(-2.511500094`\), 6.020610748`, 0.09`, 1, 
              0, \(-0.1917216239`\), 
              11.60122107`, \(-1.310317765`\), \(-1.278602552`\), 3, 4, 5, 6, 
              0.473092`, \(-0.427046`\), 0.030151`, 0.168273`, 
              0.138271`, \(-0.139531`\), 0.856859`, 0.801407`, 0.918844`, 
              0.923395`, 1.248735`, 1.209237`, 34.750663`, 34.774109`, 
              1.068494`, 43.002361`, 43.024817`, 1.023395`, 50.337047`, 
              50.30861`, \(-1.295975`\), 67.244894`, 67.264667`, 0.901111`, 
              29.694083`, 29.705085`, 0.501404`, 65.38852`, 65.399724`, 
              0.510582`, 69.49992`, 69.512052`, 0.552889`, 81.675492`, 
              81.678876`, 0.154192`, 34.750663`, 43.002361`, 50.337047`, 
              67.244894`, 81.675492`, 97.336592`, 120.445359`, 125.144808`, 
              29.694083`, 65.38852`, 69.49992`, 85.168281`, 69.934244`, 
              96.497892`, 100.75472`, 110.286868`}, {5, 34, 
              17.64664087`, \(-2.242596628`\), 4.968623963`, 0.09`, 1, 
              0.1`, \(-0.3136087353`\), 
              13.25409145`, \(-0.5379378004`\), \(-1.288692504`\), 3, 4, 5, 
              6, 0.430397`, \(-0.444102`\), 0.019264`, 0.213415`, 
              0.063303`, \(-0.068037`\), 0.845787`, 0.738088`, 0.9035`, 
              0.92344`, 1.22864`, 1.325713`, 28.320065`, 28.350752`, 
              1.04998`, 34.172801`, 34.199639`, 0.918266`, 44.247304`, 
              44.213537`, \(-1.155361`\), 56.204086`, 56.221389`, 0.592016`, 
              23.672433`, 23.6858`, 0.457361`, 52.924453`, 52.939793`, 
              0.524875`, 55.333921`, 55.34918`, 0.522105`, 66.219654`, 
              66.216398`, \(-0.111405`\), 28.320065`, 34.172801`, 44.247304`, 
              56.204086`, 66.219654`, 78.904354`, 98.083493`, 101.190478`, 
              23.672433`, 52.924453`, 55.333921`, 70.717137`, 57.356001`, 
              79.450263`, 82.487925`, 90.443974`}, {5, 33, 
              12.9458856`, \(-2.293745308`\), 5.74591947`, 0.09`, 1, 
              0.2`, \(-0.320301868`\), 
              16.04856627`, \(-1.168090251`\), \(-1.234636378`\), 3, 4, 5, 6, 
              0.429543`, \(-0.457543`\), 0.060991`, 0.182289`, 
              0.111307`, \(-0.11188`\), 0.839558`, 0.76807`, 0.904884`, 
              0.910525`, 1.234922`, 1.234474`, 33.610668`, 33.637236`, 
              1.049156`, 41.260441`, 41.294771`, 1.35568`, 53.302186`, 
              53.267376`, \(-1.374625`\), 66.625377`, 66.644969`, 0.773694`, 
              26.913298`, 26.92753`, 0.562053`, 60.926201`, 60.941332`, 
              0.597521`, 64.460936`, 64.477907`, 0.670162`, 77.078509`, 
              77.075548`, \(-0.116916`\), 33.610668`, 41.260441`, 53.302186`, 
              66.625377`, 77.078509`, 91.472445`, 113.943192`, 117.424811`, 
              26.913298`, 60.926201`, 64.460936`, 82.778867`, 67.787977`, 
              93.324708`, 96.148219`, 105.755726`}, {5, 34, 
              17.37097357`, \(-1.948438502`\), 5.279937933`, 0.09`, 1, 
              0.3`, \(-0.4054099314`\), 18.53252387`, 
              1.016063185`, \(-1.292554299`\), 3, 4, 5, 6, 
              0.390256`, \(-0.491163`\), 0.114343`, 0.206795`, 
              0.060598`, \(-0.062153`\), 0.821044`, 0.723847`, 0.872479`, 
              0.901457`, 1.189639`, 1.288911`, 30.240166`, 30.271161`, 
              1.021856`, 36.04592`, 36.083962`, 1.254192`, 49.670567`, 
              49.634059`, \(-1.203629`\), 60.822702`, 60.829432`, 0.221897`, 
              23.5912`, 23.60699`, 0.520571`, 54.478831`, 54.499859`, 
              0.693259`, 56.705943`, 56.721166`, 0.501857`, 68.644749`, 
              68.636519`, \(-0.271352`\), 30.240166`, 36.04592`, 49.670567`, 
              61.414199`, 68.644749`, 81.481408`, 102.296963`, 104.770757`, 
              23.5912`, 54.478831`, 56.705943`, 74.841754`, 60.822702`, 
              83.969733`, 86.723388`, 95.163092`}, {2, 34, 
              12.43376154`, \(-1.712330567`\), 5.531372362`, 0.09`, 1, 
              0.4`, \(-0.4452646021`\), 27.15803603`, 
              1.393598592`, \(-1.201625082`\), 3, 4, 5, 6, 
              0.364805`, \(-0.523439`\), 0.191962`, 0.188026`, 
              0.08055`, \(-0.076571`\), 0.803374`, 0.685807`, 0.86603`, 
              0.84678`, 1.189041`, 1.198503`, 31.571878`, 31.603645`, 
              1.025606`, 36.931898`, 36.974947`, 1.389871`, 52.353544`, 
              52.314491`, \(-1.260846`\), 62.706205`, 62.737232`, 1.00172`, 
              23.655589`, 23.672735`, 0.55358`, 55.725996`, 55.752896`, 
              0.868512`, 57.70417`, 57.71623`, 0.389357`, 70.161152`, 
              70.150622`, \(-0.339954`\), 31.571878`, 36.931898`, 52.353544`, 
              65.085813`, 70.161152`, 83.187752`, 105.231985`, 107.363258`, 
              23.655589`, 55.725996`, 57.70417`, 77.216159`, 62.706205`, 
              86.627203`, 89.532968`, 98.088034`}, {5, 32, 
              1.901108635`*^6, \(-0.25`\), \(-0.4133717207`\), 0.1225`, 
              1, \(-0.6`\), \(-0.002714447014`\), \(-1.056228114`\), \
\(-1.469444242`\), \(-1.321987068`\), 3, 4, 5, 6, 0.352641`, \(-3.322548`\), 
              4.056108`, 0.156779`, \(-0.191575`\), 0.436425`, 0.67909`, 
              1.62479`, 0.722708`, 0.375991`, 1.024998`, 0.680442`, 
              5.699137`, 5.699194`, \(-0.000133`\), 2.481282`, 2.481115`, 
              0.00039`, 2.181488`, 2.181653`, \(-0.000385`\), 1.801306`, 
              1.801306`, 2.`*^-6, 0.134992`, 
              0.135235`, \(-0.000566`\), \(-0.445483`\), \(-0.638558`\), 
              0.450319`, \(-0.639744`\), \(-0.860483`\), 
              0.514842`, \(-0.859552`\), \(-1.500006`\), 1.493764`, 
              5.699137`, 1.801306`, 2.481282`, 
              2.181488`, \(-1.35894`\), \(-1.511958`\), \(-0.445483`\), \
\(-1.999801`\), \(-0.859552`\), 0.134992`, \(-0.639744`\), \(-1.705963`\), 
              0.695095`, 1.436548`, 0.202296`, \(-1.842614`\)}, {5, 33, 
              385065.1712`, \(-0.25`\), 0.5757920496`, 0.1225`, 
              1, \(-0.5`\), \(-0.07063710209`\), 
              2.387698487`, \(-1.27775275`\), \(-1.348211999`\), 3, 4, 5, 6, 
              0.376085`, \(-1.623668`\), 2.30554`, 0.166149`, 0.002192`, 
              0.107378`, 0.781961`, 0.364954`, 0.825354`, 0.464216`, 
              1.135189`, 
              0.856427`, \(-3.57224`\), \(-3.572449`\), \(-0.000724`\), 
              0.153225`, 0.152879`, \(-0.0012`\), 0.239012`, 0.240765`, 
              0.006073`, 1.807426`, 2.062481`, 0.883701`, 2.429569`, 
              2.431141`, 0.005445`, 2.810783`, 2.812192`, 0.00488`, 
              3.538028`, 3.534788`, \(-0.011226`\), 3.624998`, 4.419097`, 
              2.751352`, \(-3.57224`\), 0.153225`, 0.239012`, 2.062608`, 
              4.38784`, 5.705116`, 3.624998`, 6.776224`, 2.429569`, 
              2.810783`, 3.538028`, 5.277641`, 2.192348`, 1.807426`, 
              2.894225`, 5.739223`}, {5, 33, 
              18.38039162`, \(-0.3751813597`\), 7.182574566`, 0.1225`, 
              1, \(-0.4`\), \(-0.1429334508`\), 
              139.584386`, \(-1.411151635`\), \(-1.408022611`\), 3, 4, 5, 6, 
              0.410147`, \(-0.747582`\), 0.740589`, 0.159249`, 
              0.088975`, \(-0.041389`\), 0.835785`, 0.812511`, 0.87608`, 
              0.9148`, 1.189014`, 1.164832`, \(-0.007345`\), 0.005445`, 
              0.602823`, 26.158533`, 26.148056`, \(-0.493823`\), 44.150808`, 
              44.161426`, 0.50047`, 48.880508`, 48.888473`, 0.375406`, 
              34.333254`, 34.353074`, 0.934228`, 59.132717`, 59.164112`, 
              1.479805`, 72.860149`, 72.866462`, 0.297597`, 73.28089`, 
              73.23693`, \(-2.072051`\), \(-0.007345`\), 26.158533`, 
              44.150808`, 48.880508`, 75.603187`, 94.338951`, 95.699901`, 
              118.085864`, 34.333254`, 59.132717`, 72.860149`, 73.28089`, 
              53.677086`, 71.124976`, 72.063669`, 92.587954`}, {1, 28, 
              14.74512804`, \(-0.9863546739`\), 5.703302138`, 0.1225`, 
              1, \(-0.3`\), \(-0.1737429417`\), 
              7506.377767`, \(-1.14982818`\), \(-1.397550364`\), 3, 4, 5, 6, 
              0.459563`, \(-0.436919`\), 0.065254`, 0.180932`, 
              0.107787`, \(-0.100753`\), 0.866478`, 0.78127`, 0.914705`, 
              0.918947`, 1.237923`, 1.235422`, 21.286412`, 21.310062`, 
              0.991829`, 34.164656`, 34.144847`, \(-0.830724`\), 38.11434`, 
              38.122356`, 0.336178`, 55.470853`, 55.472823`, 0.08263`, 
              28.86886`, 28.882007`, 0.551358`, 58.301101`, 58.326854`, 
              1.07997`, 62.513815`, 62.513322`, \(-0.020679`\), 69.630061`, 
              69.608705`, \(-0.895605`\), 21.286412`, 34.164656`, 38.11434`, 
              62.627494`, 70.664827`, 85.561991`, 105.529635`, 106.461192`, 
              28.86886`, 58.301101`, 62.513815`, 69.630061`, 55.470853`, 
              78.862878`, 81.118825`, 85.864032`}, {5, 32, 
              19.2459122`, \(-1.641522516`\), 5.979595501`, 0.1225`, 
              1, \(-0.2`\), \(-0.1420189055`\), 
              1209.561958`, \(-1.348775552`\), \(-1.398857272`\), 3, 4, 5, 6, 
              0.519228`, \(-0.394053`\), \(-0.040355`\), 0.171491`, 
              0.127494`, \(-0.126153`\), 0.886933`, 0.837128`, 0.940175`, 
              0.956453`, 1.270742`, 1.240293`, 35.435118`, 35.456528`, 
              1.063566`, 42.222973`, 42.231297`, 0.413519`, 45.090454`, 
              45.071681`, \(-0.93255`\), 67.876118`, 67.877125`, 0.050047`, 
              31.408108`, 31.417092`, 0.446294`, 67.047812`, 67.056424`, 
              0.427812`, 70.29935`, 70.309735`, 0.515896`, 82.190136`, 
              82.19368`, 0.176055`, 35.435118`, 45.090454`, 42.222973`, 
              75.360092`, 82.190136`, 97.964379`, 120.96359`, 124.784252`, 
              31.408108`, 67.047812`, 70.29935`, 82.307101`, 67.876118`, 
              93.133251`, 100.5377`, 103.678363`}, {5, 33, 
              18.42817601`, \(-1.940626088`\), 5.217453999`, 0.1225`, 
              1, \(-0.1`\), \(-0.2179758112`\), 
              8.779616132`, \(-0.910435242`\), \(-1.241411255`\), 3, 4, 5, 6, 
              0.513263`, \(-0.401948`\), \(-0.065747`\), 0.200166`, 
              0.08655`, \(-0.09117`\), 0.892486`, 0.810361`, 0.971822`, 
              0.966399`, 1.32197`, 1.321532`, 29.40119`, 29.425728`, 
              1.051366`, 37.95545`, 37.976879`, 0.918165`, 44.657721`, 
              44.628875`, \(-1.235953`\), 56.395424`, 56.414585`, 0.820984`, 
              27.405457`, 27.415423`, 0.427043`, 58.786735`, 58.79507`, 
              0.357124`, 62.967884`, 62.982015`, 0.605439`, 74.104019`, 
              74.103421`, \(-0.0256`\), 29.40119`, 37.95545`, 44.657721`, 
              56.395424`, 74.104019`, 87.885321`, 108.195269`, 111.976528`, 
              27.405457`, 58.786735`, 62.967884`, 76.275078`, 62.71098`, 
              85.750169`, 90.680673`, 98.315882`}, {5, 34, 
              20.67875333`, \(-2.029405296`\), 4.896835921`, 0.1225`, 1, 
              0, \(-0.3051204161`\), 
              16.62174803`, \(-0.5088699402`\), \(-1.449780873`\), 3, 4, 5, 
              6, 0.492418`, \(-0.413613`\), \(-0.083376`\), 0.218661`, 
              0.053102`, \(-0.057903`\), 0.889933`, 0.787623`, 0.93621`, 
              0.968664`, 1.262745`, 1.370163`, 29.972405`, 29.999333`, 
              1.038882`, 36.448137`, 36.474878`, 1.031701`, 45.682813`, 
              45.648242`, \(-1.333782`\), 58.511882`, 58.527249`, 0.592843`, 
              25.349859`, 25.36163`, 0.45413`, 54.862993`, 54.872709`, 
              0.374833`, 58.901776`, 58.919897`, 0.699144`, 70.0795`, 
              70.073625`, \(-0.22669`\), 29.972405`, 36.448137`, 45.682813`, 
              58.511882`, 70.0795`, 82.886233`, 102.107929`, 105.173167`, 
              25.349859`, 54.862993`, 58.901776`, 73.663134`, 60.268937`, 
              82.304336`, 86.095473`, 93.705394`}, {2, 7, 
              30.56712456`, \(-2.282540705`\), 5.170058654`, 0.1225`, 1, 
              0.1`, \(-0.343449`\), 50, 0.199666`, \(-1.69987`\), 3, 4, 5, 6, 
              0.487394`, \(-0.428073`\), \(-0.062162`\), 0.230067`, 
              0.028002`, \(-0.031495`\), 0.886326`, 0.849682`, 0.892029`, 
              1.030974`, 1.18885`, 1.441644`, 33.412805`, 33.436669`, 
              0.962144`, 39.960246`, 39.993526`, 1.34178`, 51.106154`, 
              51.069518`, \(-1.477072`\), 65.181031`, 65.193413`, 0.499206`, 
              26.427886`, 26.440522`, 0.50944`, 57.645594`, 57.655903`, 
              0.415607`, 62.671121`, 62.690882`, 0.796741`, 74.697826`, 
              74.690361`, \(-0.300993`\), 33.412805`, 39.960246`, 51.106154`, 
              65.181031`, 74.697826`, 88.021655`, 108.543309`, 111.549696`, 
              26.427886`, 57.645594`, 62.671121`, 79.307919`, 65.208183`, 
              88.532333`, 91.978679`, 100.354937`}, {5, 33, 
              12.95587852`, \(-2.455166892`\), 6.43597882`, 0.1225`, 1, 
              0.2`, \(-0.3089044429`\), 22.49822047`, 
              1.945922819`, \(-1.551152614`\), 3, 4, 5, 6, 
              0.500007`, \(-0.434181`\), \(-0.021801`\), 0.158282`, 
              0.125023`, \(-0.122129`\), 0.883963`, 0.818519`, 0.912984`, 
              0.923366`, 1.226724`, 1.177102`, 40.959954`, 40.980363`, 
              1.050835`, 53.328551`, 53.367118`, 1.985729`, 66.086662`, 
              66.051127`, \(-1.829669`\), 79.356442`, 79.370054`, 0.700857`, 
              32.674778`, 32.687815`, 0.671271`, 71.634542`, 71.64514`, 
              0.545652`, 79.929503`, 79.948638`, 0.985233`, 94.501712`, 
              94.497384`, \(-0.222857`\), 40.959954`, 53.328551`, 66.086662`, 
              79.356442`, 94.501712`, 110.7845`, 136.696963`, 140.543077`, 
              32.674778`, 71.634542`, 79.929503`, 100.520923`, 83.715716`, 
              112.79397`, 116.158995`, 127.154296`}, {5, 33, 
              12.9878404`, \(-2.119625639`\), 6.476483484`, 0.1225`, 1, 
              0.3`, \(-0.3540126067`\), 23.89792539`, 
              1.992125985`, \(-1.546856918`\), 3, 4, 5, 6, 
              0.477194`, \(-0.444154`\), \(-0.003867`\), 0.157732`, 
              0.121264`, \(-0.116925`\), 0.872173`, 0.800807`, 0.898803`, 
              0.906829`, 1.208979`, 1.162122`, 40.739138`, 40.760278`, 
              1.045348`, 53.26854`, 53.311743`, 2.136343`, 67.74323`, 
              67.706909`, \(-1.796041`\), 79.925853`, 79.939017`, 0.650936`, 
              31.948435`, 31.962572`, 0.699091`, 71.081284`, 71.092896`, 
              0.574202`, 78.735598`, 78.756284`, 1.022856`, 93.770526`, 
              93.763601`, \(-0.342448`\), 40.739138`, 53.26854`, 67.74323`, 
              79.925853`, 93.770526`, 109.712106`, 136.010478`, 139.35611`, 
              31.948435`, 71.081284`, 78.735598`, 100.716629`, 83.825828`, 
              113.045985`, 115.806668`, 126.939055`}, {5, 33, 
              13.05075605`, \(-1.809896082`\), 6.590637442`, 0.1225`, 1, 
              0.4`, \(-0.397479088`\), 28.66930448`, 
              2.073857011`, \(-1.567040138`\), 3, 4, 5, 6, 
              0.450616`, \(-0.458771`\), 0.029097`, 0.15475`, 
              0.119025`, \(-0.112351`\), 0.857224`, 0.778267`, 0.876681`, 
              0.883299`, 1.179246`, 1.135354`, 40.969674`, 40.991567`, 
              1.040288`, 53.207753`, 53.255881`, 2.286888`, 69.424121`, 
              69.386312`, \(-1.796604`\), 81.568954`, 81.581903`, 0.615309`, 
              31.378954`, 31.394301`, 0.729233`, 71.098502`, 71.111373`, 
              0.611571`, 77.808965`, 77.83103`, 1.04844`, 93.431579`, 
              93.422043`, \(-0.453124`\), 40.969674`, 53.207753`, 69.424121`, 
              81.568954`, 93.431579`, 109.145091`, 136.128932`, 138.978044`, 
              31.378954`, 71.098502`, 77.808965`, 101.275855`, 84.175861`, 
              113.652847`, 116.216466`, 127.314943`}, {5, 33, 
              751259.8744`, \(-0.25`\), 0.6707205571`, 0.16`, 
              1, \(-0.6`\), \(-0.07385343942`\), 
              1.839759316`, \(-1.329434026`\), \(-1.321310435`\), 3, 4, 5, 6, 
              0.395193`, \(-2.065207`\), 2.83708`, 0.173637`, \(-0.042817`\), 
              0.170231`, 0.791934`, 0.434126`, 0.849473`, 0.46455`, 
              1.176626`, 
              0.862374`, \(-5.811824`\), \(-5.812212`\), \(-0.001646`\), \
\(-0.661911`\), \(-0.659927`\), 
              0.008414`, \(-0.603102`\), \(-0.60336`\), \(-0.001092`\), 
              1.826006`, 1.825871`, \(-0.000572`\), 2.475273`, 2.47595`, 
              0.002871`, 2.739441`, 2.741337`, 0.00804`, 3.562327`, 
              3.559584`, \(-0.011634`\), 4.017279`, 4.682658`, 
              2.82189`, \(-5.811824`\), \(-0.603102`\), \(-0.661911`\), 
              1.826006`, 4.677018`, 6.238948`, 4.017279`, 7.323802`, 
              2.739441`, 2.475273`, 3.562327`, 5.801573`, 1.844312`, 
              1.880738`, 2.199408`, 5.790555`}, {5, 34, 
              67963.86929`, \(-0.25`\), 5.710377553`, 0.16`, 
              1, \(-0.5`\), \(-0.1408819803`\), 
              6.952276307`, \(-0.9599441625`\), \(-1.415245093`\), 3, 4, 5, 
              6, 0.427807`, \(-1.098521`\), 1.354943`, 0.195959`, 
              0.042472`, \(-0.00293`\), 0.85279`, 0.77359`, 0.902`, 
              0.922547`, 1.228677`, 
              1.264981`, \(-14.884336`\), \(-14.879352`\), 0.194793`, 
              14.300825`, 14.298155`, \(-0.104371`\), 25.775745`, 25.788408`, 
              0.494948`, 34.898482`, 34.902256`, 0.147542`, 27.815594`, 
              27.831201`, 0.610049`, 43.155774`, 43.183993`, 1.102971`, 
              55.247476`, 55.174966`, \(-2.834193`\), 55.46305`, 55.500592`, 
              1.467424`, \(-14.884336`\), 14.300825`, 25.775745`, 34.898482`, 
              57.300193`, 72.628641`, 68.509333`, 89.184752`, 27.815594`, 
              43.155774`, 55.247476`, 55.46305`, 37.611327`, 50.99851`, 
              43.160409`, 64.515988`}, {5, 32, 
              15.98940159`, \(-0.614138354`\), 5.057792863`, 0.16`, 
              1, \(-0.4`\), \(-0.2283777347`\), 
              5297.100292`, \(-0.8067025776`\), \(-1.353824984`\), 3, 4, 5, 
              6, 0.456`, \(-0.545769`\), 0.233761`, 0.206837`, 
              0.063846`, \(-0.058695`\), 0.88737`, 0.753093`, 0.951056`, 
              0.925439`, 1.2917`, 1.306583`, 11.342782`, 11.369193`, 
              0.974579`, 27.195092`, 27.172292`, \(-0.841336`\), 34.18377`, 
              34.191085`, 0.269961`, 46.605411`, 46.608176`, 0.102036`, 
              26.184503`, 26.199665`, 0.559537`, 49.509441`, 49.539373`, 
              1.104533`, 55.821797`, 55.819493`, \(-0.085049`\), 59.725943`, 
              59.697983`, \(-1.031778`\), 11.342782`, 27.195092`, 34.18377`, 
              50.232862`, 61.496548`, 74.949521`, 91.06675`, 92.647034`, 
              26.184503`, 49.509441`, 55.821797`, 59.725943`, 46.605411`, 
              67.359504`, 67.690013`, 72.635146`}, {5, 32, 
              19.2823002`, \(-1.386249897`\), 8.371273842`, 0.16`, 
              1, \(-0.3`\), \(-0.104990843`\), 
              1310.664391`, \(-2.261170335`\), \(-2.023001992`\), 3, 4, 5, 6, 
              0.574683`, \(-0.364701`\), \(-0.116283`\), 0.114347`, 
              0.182674`, \(-0.169577`\), 0.917988`, 0.881611`, 0.885513`, 
              0.937748`, 1.16154`, 1.085515`, 47.634045`, 47.647706`, 
              1.051543`, 60.707024`, 60.716493`, 0.728922`, 64.37036`, 
              64.3592`, \(-0.859064`\), 97.705789`, 97.706522`, 0.056374`, 
              46.363198`, 46.372446`, 0.711877`, 96.763344`, 96.770441`, 
              0.546304`, 103.181228`, 103.195201`, 1.075568`, 116.246963`, 
              116.227783`, \(-1.476368`\), 47.634045`, 64.37036`, 60.707024`, 
              107.82156`, 119.714686`, 141.921309`, 174.580527`, 176.672384`, 
              46.363198`, 96.763344`, 103.181228`, 116.246963`, 97.705789`, 
              131.447671`, 145.294033`, 147.093033`}, {5, 33, 
              19.67553708`, \(-1.730463686`\), 5.522816762`, 0.16`, 
              1, \(-0.2`\), \(-0.1833989801`\), 
              8.494592238`, \(-1.322353734`\), \(-1.413268872`\), 3, 4, 5, 6, 
              0.575603`, \(-0.374104`\), \(-0.140463`\), 0.183709`, 
              0.107269`, \(-0.109331`\), 0.92864`, 0.850933`, 0.993061`, 
              0.98122`, 1.340961`, 1.288948`, 31.631479`, 31.652306`, 
              1.059341`, 42.096798`, 42.11425`, 0.887646`, 48.201222`, 
              48.177749`, \(-1.193888`\), 60.468126`, 60.485144`, 0.865559`, 
              30.902251`, 30.910594`, 0.424369`, 64.538432`, 64.544485`, 
              0.307867`, 71.020748`, 71.034142`, 0.681268`, 82.734955`, 
              82.729632`, \(-0.270728`\), 31.631479`, 42.096798`, 48.201222`, 
              60.468126`, 82.734955`, 97.510028`, 119.392706`, 122.924529`, 
              30.902251`, 64.538432`, 71.020748`, 83.204306`, 69.446574`, 
              93.297195`, 100.336752`, 107.691441`}, {5, 33, 
              18.29097277`, \(-1.970226649`\), 5.424498012`, 0.16`, 
              1, \(-0.1`\), \(-0.2558685303`\), 14.29170836`, 
              1.404881962`, \(-1.4306819`\), 3, 4, 5, 6, 
              0.56979`, \(-0.387355`\), \(-0.150654`\), 0.19425`, 
              0.080889`, \(-0.082494`\), 0.932546`, 0.856127`, 0.995449`, 
              0.99561`, 1.343087`, 1.323497`, 34.652388`, 34.673429`, 
              1.04055`, 44.248395`, 44.276346`, 1.382283`, 52.473325`, 
              52.440048`, \(-1.64568`\), 65.767014`, 65.780914`, 0.687381`, 
              30.328943`, 30.338882`, 0.491489`, 63.543066`, 63.550333`, 
              0.35935`, 71.22307`, 71.239782`, 0.826496`, 83.395864`, 
              83.389477`, \(-0.315868`\), 34.652388`, 44.248395`, 52.473325`, 
              65.767014`, 83.395864`, 97.880897`, 119.618425`, 123.169508`, 
              30.328943`, 63.543066`, 71.22307`, 85.780055`, 71.388365`, 
              95.689085`, 101.152686`, 109.215816`}, {5, 33, 
              14.57533723`, \(-2.513174762`\), 6.475795448`, 0.16`, 1, 
              0, \(-0.2484022363`\), 19.86910433`, 
              2.061243629`, \(-1.738004783`\), 3, 4, 5, 6, 
              0.587926`, \(-0.405186`\), \(-0.108595`\), 0.155415`, 
              0.132302`, \(-0.131856`\), 0.936062`, 0.877443`, 0.953461`, 
              0.972572`, 1.269848`, 1.208742`, 43.46039`, 43.477695`, 
              1.054139`, 57.750322`, 57.784444`, 2.07861`, 67.468021`, 
              67.433246`, \(-2.118373`\), 81.775666`, 81.78735`, 0.711743`, 
              36.308962`, 36.319349`, 0.632778`, 76.123912`, 76.131652`, 
              0.471515`, 88.219486`, 88.236133`, 1.014091`, 102.616928`, 
              102.613314`, \(-0.220137`\), 43.46039`, 57.750322`, 67.468021`, 
              81.775666`, 102.616928`, 119.781887`, 146.12508`, 150.518332`, 
              36.308962`, 76.123912`, 88.219486`, 106.35132`, 89.411554`, 
              118.728873`, 124.267958`, 134.633994`}, {5, 33, 
              14.41741523`, \(-2.696170292`\), 6.346754656`, 0.16`, 1, 
              0.1`, \(-0.2973861999`\), 18.76388977`, 
              2.011157153`, \(-1.692257696`\), 3, 4, 5, 6, 
              0.576053`, \(-0.412707`\), \(-0.108744`\), 0.161401`, 
              0.123181`, \(-0.123282`\), 0.932413`, 0.875066`, 0.955863`, 
              0.976414`, 1.276115`, 1.226016`, 42.350421`, 42.368236`, 
              1.042093`, 57.92099`, 57.959353`, 2.244133`, 69.472682`, 
              69.437316`, \(-2.068783`\), 79.932316`, 79.943774`, 0.670262`, 
              35.153558`, 35.164779`, 0.656427`, 74.189192`, 74.197588`, 
              0.491154`, 86.453483`, 86.471737`, 1.067804`, 101.038952`, 
              101.032704`, \(-0.365485`\), 42.350421`, 57.92099`, 69.472682`, 
              79.932316`, 101.038952`, 117.593488`, 143.634715`, 147.51924`, 
              35.153558`, 74.189192`, 86.453483`, 105.966872`, 89.09338`, 
              118.075952`, 122.611945`, 133.194054`}, {5, 33, 
              13.96155958`, \(-2.579433648`\), 6.587805842`, 0.16`, 1, 
              0.2`, \(-0.3229906099`\), 22.12380881`, 
              2.139410881`, \(-1.737970714`\), 3, 4, 5, 6, 
              0.566227`, \(-0.415892`\), \(-0.10377`\), 0.155422`, 
              0.128334`, \(-0.127899`\), 0.926708`, 0.872992`, 0.941595`, 
              0.968189`, 1.254899`, 1.204476`, 44.195623`, 44.213004`, 
              1.037357`, 61.436152`, 61.4771`, 2.443896`, 74.444935`, 
              74.410621`, \(-2.04794`\), 84.063174`, 84.072473`, 0.555005`, 
              35.949561`, 35.961396`, 0.706347`, 76.444216`, 76.453092`, 
              0.529775`, 89.552033`, 89.570977`, 1.130624`, 104.933381`, 
              104.926277`, \(-0.423967`\), 44.195623`, 61.436152`, 
              74.444935`, 84.063174`, 104.933381`, 121.751651`, 149.025909`, 
              152.70641`, 35.949561`, 76.444216`, 89.552033`, 110.947679`, 
              93.492572`, 123.664881`, 127.556191`, 138.853704`}, {5, 33, 
              11.42664034`, \(-2.247908421`\), 7.938096917`, 0.16`, 1, 
              0.3`, \(-0.234122292`\), 84.00577993`, 
              2.619974948`, \(-2.024166003`\), 3, 4, 5, 6, 
              0.583182`, \(-0.400991`\), \(-0.092344`\), 0.125778`, 
              0.162764`, \(-0.162765`\), 0.923799`, 0.898383`, 0.893402`, 
              0.962705`, 1.173487`, 1.130413`, 55.280215`, 55.294488`, 
              1.057167`, 79.191778`, 79.221149`, 2.175499`, 91.813018`, 
              91.78693`, \(-1.932305`\), 105.189017`, 105.197467`, 0.625865`, 
              43.054811`, 43.064772`, 0.737828`, 91.873199`, 91.880841`, 
              0.566081`, 110.342055`, 110.355094`, 0.965823`, 128.032346`, 
              128.032714`, 0.027243`, 55.280215`, 79.191778`, 91.813018`, 
              105.189017`, 128.032346`, 147.767183`, 181.092575`, 
              185.599099`, 43.054811`, 91.873199`, 110.342055`, 135.259477`, 
              115.306896`, 151.713605`, 155.193985`, 169.336812`}, {5, 33, 
              11.21156772`, \(-1.952770793`\), 8.19031952`, 0.16`, 1, 
              0.4`, \(-0.2349897347`\), 2240.001753`, 
              2.720327316`, \(-2.02997987`\), 3, 4, 5, 6, 
              0.567102`, \(-0.388958`\), \(-0.107611`\), 0.122265`, 
              0.162014`, \(-0.161229`\), 0.913641`, 0.889985`, 0.879131`, 
              0.951492`, 1.154374`, 1.112483`, 56.916768`, 56.931029`, 
              1.059802`, 82.118392`, 82.145726`, 2.031397`, 95.820674`, 
              95.79702`, \(-1.757883`\), 109.132744`, 109.140879`, 0.604572`, 
              43.423515`, 43.433411`, 0.735427`, 93.687592`, 93.695248`, 
              0.569019`, 112.34272`, 112.35482`, 0.899194`, 130.698012`, 
              130.698867`, 0.063538`, 56.916768`, 82.118392`, 95.820674`, 
              109.132744`, 130.698012`, 150.43236`, 185.080936`, 189.28506`, 
              43.423515`, 93.687592`, 112.34272`, 138.977924`, 118.636426`, 
              156.665426`, 158.318554`, 173.495383`}, {5, 33, 
              177181.3343`, \(-0.25`\), 4.886444289`, 0.2025`, 
              1, \(-0.6`\), \(-0.164530078`\), 
              2.914185139`, \(-1.012917421`\), \(-1.307245698`\), 3, 4, 5, 6, 
              0.442375`, \(-1.415733`\), 1.802171`, 0.200037`, 0.019502`, 
              0.028927`, 0.871939`, 0.67853`, 0.947004`, 0.846775`, 
              1.299031`, 1.217263`, \(-20.532916`\), \(-20.531098`\), 
              0.062885`, 7.176268`, 7.205231`, 1.001728`, 8.347873`, 
              8.341425`, \(-0.223036`\), 28.914723`, 28.91763`, 0.100537`, 
              24.059975`, 24.074855`, 0.514658`, 33.870343`, 33.895519`, 
              0.870737`, 45.608162`, 45.523679`, \(-2.92194`\), 45.94267`, 
              45.996718`, 1.86932`, \(-20.532916`\), 8.347873`, 7.176268`, 
              30.922867`, 47.386862`, 60.894604`, 59.185254`, 74.704459`, 
              24.059975`, 33.870343`, 45.608162`, 45.94267`, 28.914723`, 
              43.677922`, 32.616808`, 53.449939`}, {5, 33, 
              19.71149971`, \(-0.3543297385`\), 4.739118092`, 0.2025`, 
              1, \(-0.5`\), \(-0.2653751267`\), 
              24.23876679`, \(-0.6183664842`\), \(-1.426480787`\), 3, 4, 5, 
              6, 0.460264`, \(-0.803783`\), 0.661174`, 0.21941`, 
              0.038249`, \(-0.031617`\), 0.909506`, 0.73385`, 0.97139`, 
              0.924084`, 1.31748`, 1.337213`, 1.729031`, 1.755294`, 
              0.916579`, 21.456599`, 21.434118`, \(-0.784581`\), 31.109007`, 
              31.115549`, 0.228327`, 40.991158`, 40.993589`, 0.084846`, 
              25.137497`, 25.153735`, 0.566704`, 44.411252`, 44.449302`, 
              1.327938`, 52.475135`, 52.465872`, \(-0.323295`\), 53.649208`, 
              53.619476`, \(-1.037673`\), 1.729031`, 21.456599`, 31.109007`, 
              41.714654`, 56.344588`, 69.440741`, 83.554041`, 86.032547`, 
              25.137497`, 44.411252`, 52.475135`, 53.649208`, 40.991158`, 
              60.91254`, 58.323988`, 63.425291`}, {5, 33, 
              20.91819201`, \(-1.080178713`\), 7.066525039`, 0.2025`, 
              1, \(-0.4`\), \(-0.149679551`\), 
              568.8083973`, \(-1.997666119`\), \(-1.970068334`\), 3, 4, 5, 6, 
              0.59519`, \(-0.342357`\), \(-0.207408`\), 0.145454`, 
              0.135197`, \(-0.122112`\), 0.945827`, 0.905773`, 0.933661`, 
              0.987213`, 1.231908`, 1.194669`, 35.103803`, 35.118935`, 
              1.018294`, 52.228197`, 52.236519`, 0.560064`, 53.92119`, 
              53.907749`, \(-0.904517`\), 82.470886`, 82.471932`, 0.070412`, 
              40.662549`, 40.672376`, 0.661307`, 83.379145`, 83.385762`, 
              0.445271`, 87.681448`, 87.700489`, 1.281353`, 98.213099`, 
              98.190721`, \(-1.505906`\), 35.103803`, 53.92119`, 52.228197`, 
              91.133864`, 102.703043`, 121.59972`, 148.911323`, 149.391444`, 
              40.662549`, 83.379145`, 87.681448`, 98.213099`, 82.470886`, 
              110.430799`, 124.055462`, 124.066736`}, {2, 8, 
              45.93651044`, \(-1.251453849`\), 3.845348189`, 0.2025`, 
              1, \(-0.3`\), \(-0.288039`\), 
              50, \(-0.0108868`\), \(-1.67907`\), 3, 4, 5, 6, 
              0.572967`, \(-0.343763`\), \(-0.283454`\), 0.233542`, 
              0.023658`, \(-0.027156`\), 0.955661`, 0.694102`, 0.98983`, 
              0.912671`, 1.323121`, 1.370124`, 23.092575`, 23.121713`, 
              1.027158`, 29.142598`, 29.149662`, 0.249049`, 33.53976`, 
              33.513057`, \(-0.941351`\), 47.912329`, 47.914056`, 0.060871`, 
              22.286631`, 22.295158`, 0.300601`, 45.821249`, 45.826891`, 
              0.19888`, 48.880461`, 48.896665`, 0.57123`, 58.05168`, 
              57.985555`, \(-2.331046`\), 23.092575`, 33.53976`, 29.142598`, 
              50.491602`, 58.05168`, 68.431095`, 83.654606`, 85.003107`, 
              22.286631`, 45.821249`, 48.880461`, 58.300136`, 47.912329`, 
              64.609784`, 70.276313`, 74.882021`}, {5, 33, 
              20.46288739`, \(-1.815640985`\), 5.70124242`, 0.2025`, 
              1, \(-0.2`\), \(-0.2227545064`\), 13.33946056`, 
              1.774374094`, \(-1.663038377`\), 3, 4, 5, 6, 
              0.638515`, \(-0.357333`\), \(-0.232506`\), 0.17926`, 
              0.098895`, \(-0.099153`\), 0.970823`, 0.893786`, 1.011574`, 
              1.011354`, 1.352291`, 1.296789`, 37.50572`, 37.523588`, 
              1.040715`, 49.325506`, 49.350012`, 1.427309`, 56.896004`, 
              56.866906`, \(-1.694845`\), 70.281083`, 70.293945`, 0.749205`, 
              33.968162`, 33.976497`, 0.485454`, 69.272373`, 69.278131`, 
              0.335371`, 80.204988`, 80.220251`, 0.888979`, 92.858356`, 
              92.842618`, \(-0.916709`\), 37.50572`, 49.325506`, 56.896004`, 
              70.281083`, 92.858356`, 108.310998`, 131.590742`, 134.952107`, 
              33.968162`, 69.272373`, 80.204988`, 93.615452`, 79.042837`, 
              104.118289`, 111.620916`, 119.44117`}, {5, 33, 
              16.88297449`, \(-2.187686605`\), 6.207464142`, 0.2025`, 
              1, \(-0.1`\), \(-0.2484526582`\), 16.36656672`, 
              2.054933769`, \(-1.834473528`\), 3, 4, 5, 6, 
              0.650286`, \(-0.379115`\), \(-0.200225`\), 0.163878`, 
              0.120114`, \(-0.121167`\), 0.975888`, 0.916235`, 0.992945`, 
              1.015402`, 1.318127`, 1.262442`, 42.764436`, 42.780563`, 
              1.041534`, 58.480924`, 58.514478`, 2.167154`, 67.421708`, 
              67.38704`, \(-2.23908`\), 79.186292`, 79.197791`, 0.742667`, 
              37.150833`, 37.159894`, 0.585234`, 75.77932`, 75.785785`, 
              0.417546`, 90.26142`, 90.277544`, 1.041427`, 104.273842`, 
              104.265369`, \(-0.547231`\), 42.764436`, 58.480924`, 
              67.421708`, 79.186292`, 104.273842`, 121.050847`, 146.760288`, 
              150.711605`, 37.150833`, 75.77932`, 90.26142`, 106.600155`, 
              90.339811`, 118.356303`, 125.183189`, 134.556836`}, {5, 33, 
              15.33917849`, \(-2.78966376`\), 6.645177824`, 0.2025`, 1, 
              0, \(-0.2463616276`\), 15.87589203`, 
              2.291287862`, \(-1.948811513`\), 3, 4, 5, 6, 
              0.663437`, \(-0.396707`\), \(-0.160311`\), 0.151005`, 
              0.137372`, \(-0.139685`\), 0.978371`, 0.928897`, 0.97937`, 
              1.014248`, 1.293862`, 1.231114`, 46.172346`, 46.187197`, 
              1.047542`, 67.253216`, 67.287763`, 2.436894`, 76.716454`, 
              76.683836`, \(-2.300819`\), 84.976283`, 84.987101`, 0.763032`, 
              39.814565`, 39.823411`, 0.623976`, 81.289155`, 81.295599`, 
              0.454536`, 99.289594`, 99.304417`, 1.045598`, 114.293388`, 
              114.288457`, \(-0.347836`\), 46.172346`, 67.253216`, 
              76.716454`, 84.976283`, 114.293388`, 132.088009`, 159.975752`, 
              164.257237`, 39.814565`, 81.289155`, 99.289594`, 117.808871`, 
              100.439234`, 130.809775`, 137.083923`, 147.785701`}, {5, 33, 
              13.83461228`, \(-6.783655593`\), 7.430471159`, 0.2025`, 1, 
              0.1`, \(-0.2194498837`\), 20.14124052`, 
              2.609988164`, \(-2.082946952`\), 3, 4, 5, 6, 
              0.676064`, \(-0.413094`\), \(-0.105118`\), 0.132172`, 
              0.161064`, \(-0.16481`\), 0.978483`, 0.941604`, 0.959646`, 
              1.008603`, 1.260231`, 1.183653`, 52.815827`, 52.829036`, 
              1.061688`, 79.841112`, 79.871791`, 2.465838`, 89.53629`, 
              89.508967`, \(-2.196129`\), 97.481397`, 97.489529`, 0.653589`, 
              44.419782`, 44.428133`, 0.671221`, 90.874`, 90.880205`, 
              0.498707`, 113.406671`, 113.419117`, 1.000394`, 130.048942`, 
              130.047726`, \(-0.097698`\), 52.815827`, 79.841112`, 89.53629`, 
              97.481397`, 130.048942`, 149.616922`, 181.167927`, 185.858978`, 
              44.419782`, 90.874`, 113.406671`, 134.774875`, 115.609225`, 
              149.738436`, 155.862182`, 168.310154`}, {5, 33, 
              12.37438378`, \(-2.837170413`\), 8.3258391`, 0.2025`, 1, 
              0.2`, \(-0.191985171`\), 491.7997941`, 
              2.846052767`, \(-2.239902539`\), 3, 4, 5, 6, 
              0.678272`, \(-0.394696`\), \(-0.137794`\), 0.118901`, 
              0.176105`, \(-0.18111`\), 0.975562`, 0.964989`, 0.932216`, 
              1.019085`, 1.21471`, 1.163796`, 62.382601`, 62.394339`, 
              1.06065`, 92.934773`, 92.960248`, 2.301745`, 103.495051`, 
              103.471552`, \(-2.123228`\), 115.501201`, 115.50804`, 
              0.617938`, 49.392958`, 49.400849`, 0.71302`, 101.415234`, 
              101.421103`, 0.530299`, 128.211039`, 128.221548`, 0.949564`, 
              146.702525`, 146.703645`, 0.101131`, 62.382601`, 92.934773`, 
              103.495051`, 115.501201`, 146.702525`, 168.161183`, 
              203.891094`, 208.844903`, 49.392958`, 101.415234`, 128.211039`, 
              152.858178`, 131.663279`, 169.950367`, 175.870856`, 
              190.227917`}, {5, 33, 12.20220324`, \(-2.390917194`\), 
              8.344211386`, 0.2025`, 1, 0.3`, \(-0.2127607388`\), 
              1232.070363`, 2.891696372`, \(-2.232224154`\), 3, 4, 5, 6, 
              0.662704`, \(-0.381288`\), \(-0.167758`\), 0.119001`, 
              0.172535`, \(-0.177211`\), 0.968686`, 0.954787`, 0.924169`, 
              1.0094`, 1.20454`, 1.155239`, 62.038784`, 62.050767`, 
              1.060211`, 93.781345`, 93.807015`, 2.271223`, 105.842824`, 
              105.820924`, \(-1.937612`\), 115.701563`, 115.707245`, 
              0.502773`, 48.721958`, 48.730249`, 0.73357`, 100.756343`, 
              100.762511`, 0.545724`, 127.366139`, 127.376993`, 0.960274`, 
              146.244892`, 146.245456`, 0.049875`, 62.038784`, 93.781345`, 
              105.842824`, 115.701563`, 146.244892`, 167.201299`, 
              203.230619`, 207.741515`, 48.721958`, 100.756343`, 127.366139`, 
              152.499606`, 132.28755`, 170.856745`, 175.547825`, 
              190.286341`}, {5, 32, 12.55393782`, \(-2.067516676`\), 
              8.527330425`, 0.2025`, 1, 0.4`, \(-0.2100634216`\), 
              768.6118994`, 2.97818427`, \(-2.231622226`\), 3, 4, 5, 6, 
              0.648152`, \(-0.361601`\), \(-0.202232`\), 0.1165`, 
              0.171376`, \(-0.17568`\), 0.960246`, 0.94558`, 0.912968`, 
              0.998504`, 1.189757`, 1.140123`, 62.679428`, 62.691499`, 
              1.067453`, 96.463123`, 96.4859`, 2.014159`, 109.237241`, 
              109.219856`, \(-1.537417`\), 118.044028`, 118.04813`, 
              0.362755`, 48.887681`, 48.895776`, 0.715911`, 101.931652`, 
              101.937691`, 0.534062`, 128.835789`, 128.845647`, 0.871738`, 
              148.250008`, 148.251112`, 0.097596`, 62.679428`, 96.463123`, 
              109.237241`, 118.044028`, 148.250008`, 169.027498`, 
              206.072661`, 210.248157`, 48.887681`, 101.931652`, 128.835789`, 
              154.836687`, 135.066619`, 174.920114`, 177.818206`, 
              193.449892`}, {5, 33, 914.629817`, \(-0.335830962`\), 
              4.425404614`, 0.25`, 1, \(-0.6`\), \(-0.2129818622`\), 
              2304.993205`, 0.1571873383`, \(-1.384078627`\), 3, 4, 5, 6, 
              0.532996`, \(-0.748775`\), 0.532236`, 0.235312`, 
              0.019933`, \(-0.023943`\), 0.941994`, 0.772776`, 1.020766`, 
              0.975352`, 1.386152`, 1.416724`, \(-1.28621`\), \(-1.276124`\), 
              0.380621`, 20.870723`, 20.861992`, \(-0.329485`\), 31.035472`, 
              31.040474`, 0.188758`, 39.781778`, 39.791835`, 0.379526`, 
              24.76465`, 24.775365`, 0.404365`, 47.2284`, 
              47.207094`, \(-0.804099`\), 47.363722`, 47.40936`, 1.722371`, 
              50.322098`, 50.298428`, \(-0.893296`\), \(-1.28621`\), 
              20.870723`, 31.035472`, 39.781778`, 55.332967`, 67.656914`, 
              81.055622`, 84.146244`, 24.76465`, 47.363722`, 47.2284`, 
              50.322098`, 39.920809`, 57.589449`, 57.702719`, 
              61.955647`}, {5, 30, 22.11318881`, \(-0.7360129827`\), 
              4.489464494`, 0.25`, 1, \(-0.5`\), \(-0.2705737822`\), 
              87.58740395`, \(-0.075904386`\), \(-1.477955065`\), 3, 4, 5, 6, 
              0.576298`, \(-0.3977`\), \(-0.187307`\), 0.234765`, 
              0.021419`, \(-0.024651`\), 0.969995`, 0.8143`, 1.039447`, 
              1.007882`, 1.401661`, 1.437926`, 16.31265`, 16.336957`, 
              1.006234`, 31.885697`, 31.885471`, \(-0.009336`\), 33.764849`, 
              33.749147`, \(-0.650009`\), 50.528315`, 50.530273`, 0.081063`, 
              26.429531`, 26.440034`, 0.434768`, 53.421112`, 53.441394`, 
              0.839609`, 53.517317`, 53.524562`, 0.299908`, 61.824149`, 
              61.793677`, \(-1.261422`\), 16.31265`, 31.885697`, 33.764849`, 
              55.425909`, 64.471093`, 76.726752`, 93.000203`, 94.182559`, 
              26.429531`, 53.517317`, 53.421112`, 61.824149`, 50.528315`, 
              68.752234`, 76.922101`, 77.916483`}, {5, 34, 
              42.87444482`, \(-1.082758856`\), 3.840120463`, 0.25`, 
              1, \(-0.4`\), \(-0.3005941627`\), 211.0437943`, 
              0.02171007704`, \(-1.711428175`\), 3, 4, 5, 6, 
              0.616067`, \(-0.31657`\), \(-0.381866`\), 0.23529`, 
              0.020775`, \(-0.024206`\), 0.988528`, 0.721934`, 1.02922`, 
              0.935984`, 1.374682`, 1.390305`, 22.574757`, 22.60253`, 
              1.051301`, 30.152153`, 30.15755`, 0.204272`, 33.97761`, 
              33.95265`, \(-0.944803`\), 49.148541`, 49.150207`, 0.063092`, 
              23.35527`, 23.363453`, 0.309738`, 47.030447`, 47.035392`, 
              0.187158`, 50.641809`, 50.659471`, 0.668561`, 59.335461`, 
              59.288959`, \(-1.760217`\), 22.574757`, 33.97761`, 30.152153`, 
              52.418993`, 60.08954`, 70.58168`, 85.89409`, 86.396114`, 
              23.35527`, 47.030447`, 50.641809`, 59.335461`, 49.148541`, 
              65.462232`, 72.315066`, 76.420491`}, {2, 33, 
              41.11209723`, \(-1.362728101`\), 3.899287129`, 0.25`, 
              1, \(-0.3`\), \(-0.3392133725`\), 13.53360703`, 
              0.1059371165`, \(-1.829747937`\), 3, 4, 5, 6, 
              0.64082`, \(-0.311583`\), \(-0.421797`\), 0.235424`, 
              0.020433`, \(-0.023823`\), 1.000174`, 0.74766`, 1.026994`, 
              0.955975`, 1.364921`, 1.404061`, 25.011946`, 25.037638`, 
              1.027178`, 32.225547`, 32.244237`, 0.747211`, 39.597336`, 
              39.566171`, \(-1.245969`\), 48.837012`, 48.850098`, 0.523175`, 
              24.134663`, 24.142757`, 0.323604`, 48.418362`, 48.423592`, 
              0.209082`, 54.799896`, 54.817398`, 0.699751`, 64.485137`, 
              64.427223`, \(-2.31538`\), 25.011946`, 32.225547`, 39.597336`, 
              48.837012`, 64.485137`, 75.168473`, 91.103718`, 92.300543`, 
              24.134663`, 48.418362`, 54.799896`, 65.027953`, 54.206988`, 
              71.460775`, 77.327518`, 82.214832`}, {5, 33, 
              23.43369672`, \(-1.900226958`\), 5.508778672`, 0.25`, 
              1, \(-0.2`\), \(-0.2610832257`\), 12.46388226`, 
              1.80091136`, \(-1.856505506`\), 3, 4, 5, 6, 
              0.707019`, \(-0.343863`\), \(-0.316315`\), 0.185909`, 
              0.089401`, \(-0.091704`\), 1.013453`, 0.930737`, 1.040314`, 
              1.050531`, 1.380778`, 1.343071`, 37.967702`, 37.983928`, 
              1.011158`, 53.793981`, 53.824037`, 1.873022`, 62.203863`, 
              62.172047`, \(-1.982673`\), 69.397732`, 69.410958`, 0.824175`, 
              34.919962`, 34.927632`, 0.477962`, 69.544666`, 69.54999`, 
              0.331763`, 84.596953`, 84.612385`, 0.961666`, 97.298992`, 
              97.274307`, \(-1.538323`\), 37.967702`, 53.793981`, 62.203863`, 
              69.397732`, 97.298992`, 112.43234`, 135.606976`, 138.497554`, 
              34.919962`, 69.544666`, 84.596953`, 98.245028`, 83.837228`, 
              108.469634`, 115.984817`, 123.74091`}, {5, 33, 
              17.2966442`, \(-2.66480306`\), 6.745905269`, 0.25`, 
              1, \(-0.1`\), \(-0.2080085199`\), 13.72678806`, 
              2.480820996`, \(-2.142837182`\), 3, 4, 5, 6, 
              0.744702`, \(-0.377861`\), \(-0.221145`\), 0.14739`, 
              0.141218`, \(-0.145445`\), 1.020384`, 0.974256`, 1.008481`, 
              1.052057`, 1.323427`, 1.253324`, 48.251068`, 48.264098`, 
              1.047318`, 73.819446`, 73.849882`, 2.446446`, 81.601221`, 
              81.571711`, \(-2.372036`\), 87.099308`, 87.110298`, 0.883374`, 
              43.212321`, 43.21926`, 0.557776`, 85.841722`, 85.846697`, 
              0.399948`, 108.858506`, 108.870717`, 0.981476`, 123.893176`, 
              123.88752`, \(-0.454604`\), 48.251068`, 73.819446`, 81.601221`, 
              87.099308`, 123.893176`, 142.351597`, 171.274792`, 175.582033`, 
              43.212321`, 85.841722`, 108.858506`, 125.688177`, 108.488452`, 
              139.034669`, 147.280505`, 157.555128`}, {5, 33, 
              15.73776957`, \(-7.101494941`\), 7.26111883`, 0.25`, 1, 
              0, \(-0.2123084379`\), 17.49226751`, 
              2.67210794`, \(-2.202276117`\), 3, 4, 5, 6, 
              0.752873`, \(-0.393497`\), \(-0.19126`\), 0.137291`, 
              0.154934`, \(-0.161011`\), 1.022044`, 0.994357`, 1.00221`, 
              1.061767`, 1.312049`, 1.239183`, 53.545023`, 53.557016`, 
              1.048993`, 83.965664`, 83.997071`, 2.747053`, 92.417837`, 
              92.39`, \(-2.434865`\), 96.873624`, 96.880955`, 0.641269`, 
              46.506617`, 46.513755`, 0.624358`, 92.507572`, 92.512809`, 
              0.458083`, 119.462012`, 119.473603`, 1.01377`, 135.845338`, 
              135.842154`, \(-0.278452`\), 53.545023`, 83.965664`, 
              92.417837`, 96.873624`, 135.845338`, 155.476767`, 186.946427`, 
              191.485417`, 46.506617`, 92.507572`, 119.462012`, 138.074192`, 
              120.315164`, 153.706211`, 161.399083`, 173.090181`}, {5, 34, 
              13.85121202`, \(-8.165634584`\), 8.385641944`, 0.25`, 1, 
              0.1`, \(-0.1889812974`\), 245.8120682`, 
              3.003603855`, \(-2.427641678`\), 3, 4, 5, 6, 
              0.761195`, \(-0.396574`\), \(-0.173496`\), 0.117375`, 
              0.180045`, \(-0.188521`\), 1.021818`, 1.013015`, 0.969258`, 
              1.06315`, 1.255913`, 1.198893`, 66.189242`, 66.19951`, 
              1.048601`, 101.59757`, 101.625572`, 2.859837`, 110.94461`, 
              110.918588`, \(-2.657635`\), 120.171796`, 120.177521`, 
              0.58471`, 53.567545`, 53.574604`, 0.720961`, 106.73628`, 
              106.741498`, 0.532889`, 140.098069`, 140.108316`, 1.046557`, 
              158.976555`, 158.976158`, \(-0.040578`\), 66.189242`, 
              101.59757`, 110.94461`, 120.171796`, 158.976555`, 181.278619`, 
              218.00084`, 223.026334`, 53.567545`, 106.73628`, 140.098069`, 
              159.36835`, 142.185967`, 180.998694`, 188.811961`, 
              202.853074`}, {5, 30, 12.73868403`, \(-7.515759486`\), 
              8.484985821`, 0.25`, 1, 0.2`, \(-0.2005069002`\), 1430.670962`, 
              3.063719328`, \(-2.427354315`\), 3, 4, 5, 6, 
              0.754097`, \(-0.386081`\), \(-0.198617`\), 0.116406`, 
              0.179653`, \(-0.188691`\), 1.018401`, 1.011751`, 0.964688`, 
              1.061134`, 1.24993`, 1.195004`, 66.929973`, 66.940296`, 
              1.056806`, 104.891805`, 104.917748`, 2.655954`, 115.47512`, 
              115.453493`, \(-2.214155`\), 122.036431`, 122.040131`, 
              0.378766`, 53.729748`, 53.736903`, 0.732509`, 107.475755`, 
              107.481061`, 0.54322`, 141.907348`, 141.917265`, 1.015251`, 
              160.716616`, 160.722273`, 0.57911`, 66.929973`, 104.891805`, 
              115.47512`, 122.036431`, 161.316624`, 183.418402`, 220.825537`, 
              225.51324`, 53.729748`, 107.475755`, 141.907348`, 160.716616`, 
              145.47268`, 184.973256`, 191.724534`, 206.36632`}, {5, 31, 
              14.70763349`, \(-2.611171547`\), 8.81413964`, 0.25`, 1, 
              0.3`, \(-0.1719834734`\), 5880.77969`, 
              3.187765886`, \(-2.442290267`\), 3, 4, 5, 6, 
              0.750254`, \(-0.365276`\), \(-0.229366`\), 0.111452`, 
              0.183575`, \(-0.193453`\), 1.013749`, 1.009121`, 0.956197`, 
              1.054837`, 1.237841`, 1.179502`, 69.186988`, 69.197127`, 
              1.072777`, 111.27255`, 111.291488`, 2.003764`, 121.691244`, 
              121.679229`, \(-1.271283`\), 127.175438`, 127.176019`, 
              0.061469`, 55.251313`, 55.257644`, 0.66988`, 110.972795`, 
              110.977467`, 0.494369`, 147.52048`, 147.528237`, 0.820685`, 
              166.241008`, 166.245949`, 0.522774`, 69.186988`, 111.27255`, 
              121.691244`, 127.175438`, 167.589649`, 189.990466`, 229.15318`, 
              233.663711`, 55.251313`, 110.972795`, 147.52048`, 166.241008`, 
              152.230316`, 193.297595`, 199.319282`, 214.845485`}, {5, 32, 
              18.40951838`, \(-2.193821434`\), 9.057503897`, 0.25`, 1, 
              0.4`, \(-0.138309748`\), 1504.589628`, 
              3.288120239`, \(-2.440864547`\), 3, 4, 5, 6, 
              0.742253`, \(-0.33515`\), \(-0.282604`\), 0.108376`, 
              0.184109`, \(-0.19432`\), 1.007156`, 1.004911`, 0.947452`, 
              1.048439`, 1.226303`, 1.167557`, 70.457097`, 70.467097`, 
              1.075613`, 115.848288`, 115.860195`, 1.280807`, 125.619953`, 
              125.616676`, \(-0.352458`\), 131.274619`, 
              131.272906`, \(-0.184235`\), 56.013048`, 56.018383`, 0.573843`, 
              113.126308`, 113.130263`, 0.425416`, 150.965283`, 150.970711`, 
              0.583917`, 169.89224`, 169.896294`, 0.43612`, 70.457097`, 
              115.848288`, 125.619953`, 131.274619`, 171.512348`, 
              193.847656`, 234.321299`, 238.502535`, 56.013048`, 113.126308`, 
              150.965283`, 169.89224`, 156.813663`, 198.997277`, 204.122271`, 
              220.245591`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Contour plot", "Subsection"],

Cell[BoxData[
    \(\(Length[
        data = Map[{Sqrt[#[\([6]\)]], #[\([8]\)], #[\([3]\)]} &, 
            datal1[\([2]\)]]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[
      d2 = Map[Sort[#, Function[{a, b}, OrderedQ[{Rest[a], Rest[b]}]]] &, 
          Partition[Sort[data], 12]]]\)], "Input"],

Cell[BoxData[
    \({11, 12, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ColumnForm[d2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\({{0, \(-0.6`\), 7.87749082`*^7}, {0, \(-0.5`\), 
                4.636975773`*^7}, {0, \(-0.4`\), 
                2.031149304`*^7}, {0, \(-0.3`\), 
                6.209914282`*^6}, {0, \(-0.2`\), 
                749669.0827`}, {0, \(-0.1`\), 
                34.54916505`}, {0, \(-0.004693500257`\), 6.369872612`}, {0, 
                0, 6.724709295`}, {0, 0.1`, 6.555084342`}, {0, 0.2`, 
                7.121482369`}, {0, 0.3`, 6.679296838`}, {0, 0.4`, 
                7.832289531`}}\)},
          {\({{0.05`, \(-0.6`\), 5.917227444`*^7}, {0.05`, \(-0.5`\), 
                1.682173566`*^6}, {0.05`, \(-0.4`\), 
                1.238967622`*^7}, {0.05`, \(-0.3`\), 
                1.086518108`*^6}, {0.05`, \(-0.2`\), 
                248786.4596`}, {0.05`, \(-0.1`\), 
                15.94250349`}, {0.05`, \(-0.05543702533`\), 
                7.349227765`}, {0.05`, 0, 8.176644017`}, {0.05`, 0.1`, 
                8.382634368`}, {0.05`, 0.2`, 8.135028131`}, {0.05`, 0.3`, 
                8.557305533`}, {0.05`, 0.4`, 8.917290277`}}\)},
          {\({{0.1`, \(-0.6`\), 4.051430028`*^7}, {0.1`, \(-0.5`\), 
                1.929977044`*^7}, {0.1`, \(-0.4`\), 
                6.980307701`*^6}, {0.1`, \(-0.3`\), 
                1.453006837`*^6}, {0.1`, \(-0.2`\), 
                14929.25603`}, {0.1`, \(-0.1`\), 
                8.225594084`}, {0.1`, \(-0.09526007516`\), 
                8.135087732`}, {0.1`, 0, 9.92418033`}, {0.1`, 0.1`, 
                10.96256292`}, {0.1`, 0.2`, 11.39879853`}, {0.1`, 0.3`, 
                12.39463084`}, {0.1`, 0.4`, 13.884388`}}\)},
          {\({{0.15`, \(-0.6`\), 5.069441347`*^6}, {0.15`, \(-0.5`\), 
                1.154897496`*^7}, {0.15`, \(-0.4`\), 
                3.662739179`*^6}, {0.15`, \(-0.3`\), 
                499321.3356`}, {0.15`, \(-0.2`\), 
                231.7476679`}, {0.15`, \(-0.1213305738`\), 
                8.339396601`}, {0.15`, \(-0.1`\), 8.986510587`}, {0.15`, 0, 
                12.89645198`}, {0.15`, 0.1`, 14.114535`}, {0.15`, 0.2`, 
                14.15072367`}, {0.15`, 0.3`, 17.68847471`}, {0.15`, 0.4`, 
                19.88521517`}}\)},
          {\({{0.2`, \(-0.6`\), 1.595120815`*^7}, {0.2`, \(-0.5`\), 
                6.422746072`*^6}, {0.2`, \(-0.4`\), 
                1.676578381`*^6}, {0.2`, \(-0.3`\), 
                132807.0503`}, {0.2`, \(-0.2`\), 
                10.31076161`}, {0.2`, \(-0.173629971`\), 
                8.133791511`}, {0.2`, \(-0.1`\), 11.57583103`}, {0.2`, 0, 
                15.60724641`}, {0.2`, 0.1`, 14.73286189`}, {0.2`, 0.2`, 
                19.1431102`}, {0.2`, 0.3`, 11.54917753`}, {0.2`, 0.4`, 
                15.32954057`}}\)},
          {\({{0.25`, \(-0.6`\), 1.034388894`*^6}, {0.25`, \(-0.5`\), 
                2.895160895`*^6}, {0.25`, \(-0.4`\), 
                631162.7368`}, {0.25`, \(-0.3`\), 
                121.5757198`}, {0.25`, \(-0.2285961599`\), 
                8.325858404`}, {0.25`, \(-0.2`\), 
                9.036641995`}, {0.25`, \(-0.1`\), 14.43910715`}, {0.25`, 0, 
                16.09731379`}, {0.25`, 0.1`, 14.0372776`}, {0.25`, 0.2`, 
                11.57795483`}, {0.25`, 0.3`, 9.540882195`}, {0.25`, 0.4`, 
                11.67771023`}}\)},
          {\({{0.3`, \(-0.6`\), 4.565945047`*^6}, {0.3`, \(-0.5`\), 
                1.0838068`*^6}, {0.3`, \(-0.4`\), 
                143585.0156`}, {0.3`, \(-0.3032787178`\), 
                10.42272203`}, {0.3`, \(-0.3`\), 
                8.140432639`}, {0.3`, \(-0.2`\), 
                12.46394067`}, {0.3`, \(-0.1`\), 17.60299526`}, {0.3`, 0, 
                13.73519764`}, {0.3`, 0.1`, 17.64664087`}, {0.3`, 0.2`, 
                12.9458856`}, {0.3`, 0.3`, 17.37097357`}, {0.3`, 0.4`, 
                12.43376154`}}\)},
          {\({{0.35`, \(-0.6`\), 1.901108635`*^6}, {0.35`, \(-0.5`\), 
                385065.1712`}, {0.35`, \(-0.4`\), 
                18.38039162`}, {0.35`, \(-0.3711845869`\), 
                8.52311789`}, {0.35`, \(-0.3`\), 
                14.74512804`}, {0.35`, \(-0.2`\), 
                19.2459122`}, {0.35`, \(-0.1`\), 18.42817601`}, {0.35`, 0, 
                20.67875333`}, {0.35`, 0.1`, 30.56712456`}, {0.35`, 0.2`, 
                12.95587852`}, {0.35`, 0.3`, 12.9878404`}, {0.35`, 0.4`, 
                13.05075605`}}\)},
          {\({{0.4`, \(-0.6`\), 751259.8744`}, {0.4`, \(-0.5`\), 
                67963.86929`}, {0.4`, \(-0.4383723302`\), 
                10.94888151`}, {0.4`, \(-0.4`\), 
                15.98940159`}, {0.4`, \(-0.3`\), 
                19.2823002`}, {0.4`, \(-0.2`\), 
                19.67553708`}, {0.4`, \(-0.1`\), 18.29097277`}, {0.4`, 0, 
                14.57533723`}, {0.4`, 0.1`, 14.41741523`}, {0.4`, 0.2`, 
                13.96155958`}, {0.4`, 0.3`, 11.42664034`}, {0.4`, 0.4`, 
                11.21156772`}}\)},
          {\({{0.45`, \(-0.6`\), 177181.3343`}, {0.45`, \(-0.5`\), 
                19.71149971`}, {0.45`, \(-0.454417308`\), 
                16.64275753`}, {0.45`, \(-0.4`\), 
                20.91819201`}, {0.45`, \(-0.3`\), 
                45.93651044`}, {0.45`, \(-0.2`\), 
                20.46288739`}, {0.45`, \(-0.1`\), 16.88297449`}, {0.45`, 0, 
                15.33917849`}, {0.45`, 0.1`, 13.83461228`}, {0.45`, 0.2`, 
                12.37438378`}, {0.45`, 0.3`, 12.20220324`}, {0.45`, 0.4`, 
                12.55393782`}}\)},
          {\({{0.5`, \(-0.6`\), 914.629817`}, {0.5`, \(-0.50272`\), 
                43.4135314`}, {0.5`, \(-0.5`\), 
                22.11318881`}, {0.5`, \(-0.4`\), 
                42.87444482`}, {0.5`, \(-0.3`\), 
                41.11209723`}, {0.5`, \(-0.2`\), 
                23.43369672`}, {0.5`, \(-0.1`\), 17.2966442`}, {0.5`, 0, 
                15.73776957`}, {0.5`, 0.1`, 13.85121202`}, {0.5`, 0.2`, 
                12.73868403`}, {0.5`, 0.3`, 14.70763349`}, {0.5`, 0.4`, 
                18.40951838`}}\)}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      ColumnForm[ {{{0, -0.59999999999999998, 78774908.200000003}, {0, -0.5, 
        46369757.729999997}, {0, -0.40000000000000002, 20311493.039999999}, {
        0, -0.29999999999999999, 6209914.2819999997}, {
        0, -0.20000000000000001, 749669.08270000003}, {
        0, -0.10000000000000001, 34.549165049999999}, {0, -0.004693500257, 
        6.369872612}, {0, 0, 6.7247092950000003}, {0, 0.10000000000000001, 
        6.5550843419999998}, {0, 0.20000000000000001, 7.1214823689999998}, {0,
         0.29999999999999999, 6.679296838}, {0, 0.40000000000000002, 
        7.8322895309999998}}, {{0.050000000000000003, -0.59999999999999998, 
        59172274.439999998}, {0.050000000000000003, -0.5, 
        1682173.5660000001}, {0.050000000000000003, -0.40000000000000002, 
        12389676.220000001}, {0.050000000000000003, -0.29999999999999999, 
        1086518.108}, {0.050000000000000003, -0.20000000000000001, 
        248786.4596}, {0.050000000000000003, -0.10000000000000001, 
        15.94250349}, {0.050000000000000003, -0.05543702533, 
        7.3492277650000002}, {0.050000000000000003, 0, 8.1766440169999992}, {
        0.050000000000000003, 0.10000000000000001, 8.3826343679999997}, {
        0.050000000000000003, 0.20000000000000001, 8.1350281310000003}, {
        0.050000000000000003, 0.29999999999999999, 8.5573055329999992}, {
        0.050000000000000003, 0.40000000000000002, 8.9172902769999993}}, {{
        0.10000000000000001, -0.59999999999999998, 40514300.280000001}, {
        0.10000000000000001, -0.5, 19299770.440000001}, {
        0.10000000000000001, -0.40000000000000002, 6980307.7010000004}, {
        0.10000000000000001, -0.29999999999999999, 1453006.8370000001}, {
        0.10000000000000001, -0.20000000000000001, 14929.25603}, {
        0.10000000000000001, -0.10000000000000001, 8.2255940840000008}, {
        0.10000000000000001, -0.095260075159999999, 8.1350877320000006}, {
        0.10000000000000001, 0, 9.9241803300000004}, {0.10000000000000001, 
        0.10000000000000001, 10.96256292}, {0.10000000000000001, 
        0.20000000000000001, 11.398798530000001}, {0.10000000000000001, 
        0.29999999999999999, 12.39463084}, {0.10000000000000001, 
        0.40000000000000002, 13.884388}}, {{
        0.14999999999999999, -0.59999999999999998, 5069441.3470000001}, {
        0.14999999999999999, -0.5, 11548974.960000001}, {
        0.14999999999999999, -0.40000000000000002, 3662739.179}, {
        0.14999999999999999, -0.29999999999999999, 499321.33559999999}, {
        0.14999999999999999, -0.20000000000000001, 231.74766790000001}, {
        0.14999999999999999, -0.12133057379999999, 8.3393966010000007}, {
        0.14999999999999999, -0.10000000000000001, 8.9865105869999997}, {
        0.14999999999999999, 0, 12.89645198}, {0.14999999999999999, 
        0.10000000000000001, 14.114535}, {0.14999999999999999, 
        0.20000000000000001, 14.15072367}, {0.14999999999999999, 
        0.29999999999999999, 17.688474710000001}, {0.14999999999999999, 
        0.40000000000000002, 19.885215169999999}}, {{
        0.20000000000000001, -0.59999999999999998, 15951208.15}, {
        0.20000000000000001, -0.5, 6422746.0719999997}, {
        0.20000000000000001, -0.40000000000000002, 1676578.3810000001}, {
        0.20000000000000001, -0.29999999999999999, 132807.0503}, {
        0.20000000000000001, -0.20000000000000001, 10.31076161}, {
        0.20000000000000001, -0.17362997099999999, 8.1337915110000001}, {
        0.20000000000000001, -0.10000000000000001, 11.57583103}, {
        0.20000000000000001, 0, 15.60724641}, {0.20000000000000001, 
        0.10000000000000001, 14.732861890000001}, {0.20000000000000001, 
        0.20000000000000001, 19.143110199999999}, {0.20000000000000001, 
        0.29999999999999999, 11.54917753}, {0.20000000000000001, 
        0.40000000000000002, 15.329540570000001}}, {{
        0.25, -0.59999999999999998, 1034388.894}, {0.25, -0.5, 2895160.895}, {
        0.25, -0.40000000000000002, 631162.73679999996}, {
        0.25, -0.29999999999999999, 121.5757198}, {0.25, -0.22859615990000001,
         8.3258584039999999}, {0.25, -0.20000000000000001, 
        9.0366419950000001}, {0.25, -0.10000000000000001, 14.43910715}, {0.25,
         0, 16.097313790000001}, {0.25, 0.10000000000000001, 
        14.037277599999999}, {0.25, 0.20000000000000001, 
        11.577954829999999}, {0.25, 0.29999999999999999, 9.540882195}, {0.25, 
        0.40000000000000002, 11.677710230000001}}, {{
        0.29999999999999999, -0.59999999999999998, 4565945.0470000003}, {
        0.29999999999999999, -0.5, 1083806.8}, {
        0.29999999999999999, -0.40000000000000002, 143585.01560000001}, {
        0.29999999999999999, -0.30327871779999999, 10.422722029999999}, {
        0.29999999999999999, -0.29999999999999999, 8.1404326390000001}, {
        0.29999999999999999, -0.20000000000000001, 12.463940669999999}, {
        0.29999999999999999, -0.10000000000000001, 17.60299526}, {
        0.29999999999999999, 0, 13.735197640000001}, {0.29999999999999999, 
        0.10000000000000001, 17.646640869999999}, {0.29999999999999999, 
        0.20000000000000001, 12.9458856}, {0.29999999999999999, 
        0.29999999999999999, 17.37097357}, {0.29999999999999999, 
        0.40000000000000002, 12.433761540000001}}, {{
        0.34999999999999998, -0.59999999999999998, 1901108.635}, {
        0.34999999999999998, -0.5, 385065.17119999998}, {
        0.34999999999999998, -0.40000000000000002, 18.380391620000001}, {
        0.34999999999999998, -0.3711845869, 8.52311789}, {
        0.34999999999999998, -0.29999999999999999, 14.745128040000001}, {
        0.34999999999999998, -0.20000000000000001, 19.245912199999999}, {
        0.34999999999999998, -0.10000000000000001, 18.428176010000001}, {
        0.34999999999999998, 0, 20.678753329999999}, {0.34999999999999998, 
        0.10000000000000001, 30.56712456}, {0.34999999999999998, 
        0.20000000000000001, 12.955878520000001}, {0.34999999999999998, 
        0.29999999999999999, 12.9878404}, {0.34999999999999998, 
        0.40000000000000002, 13.05075605}}, {{
        0.40000000000000002, -0.59999999999999998, 751259.87439999997}, {
        0.40000000000000002, -0.5, 67963.869290000002}, {
        0.40000000000000002, -0.43837233019999999, 10.94888151}, {
        0.40000000000000002, -0.40000000000000002, 15.98940159}, {
        0.40000000000000002, -0.29999999999999999, 19.282300200000002}, {
        0.40000000000000002, -0.20000000000000001, 19.675537080000002}, {
        0.40000000000000002, -0.10000000000000001, 18.29097277}, {
        0.40000000000000002, 0, 14.575337230000001}, {0.40000000000000002, 
        0.10000000000000001, 14.41741523}, {0.40000000000000002, 
        0.20000000000000001, 13.961559579999999}, {0.40000000000000002, 
        0.29999999999999999, 11.426640340000001}, {0.40000000000000002, 
        0.40000000000000002, 11.21156772}}, {{
        0.45000000000000001, -0.59999999999999998, 177181.33429999999}, {
        0.45000000000000001, -0.5, 19.711499709999998}, {
        0.45000000000000001, -0.45441730800000002, 16.642757530000001}, {
        0.45000000000000001, -0.40000000000000002, 20.918192009999999}, {
        0.45000000000000001, -0.29999999999999999, 45.936510439999999}, {
        0.45000000000000001, -0.20000000000000001, 20.462887389999999}, {
        0.45000000000000001, -0.10000000000000001, 16.882974489999999}, {
        0.45000000000000001, 0, 15.33917849}, {0.45000000000000001, 
        0.10000000000000001, 13.83461228}, {0.45000000000000001, 
        0.20000000000000001, 12.37438378}, {0.45000000000000001, 
        0.29999999999999999, 12.202203239999999}, {0.45000000000000001, 
        0.40000000000000002, 12.55393782}}, {{0.5, -0.59999999999999998, 
        914.629817}, {0.5, -0.50271999999999994, 43.413531399999997}, {
        0.5, -0.5, 22.11318881}, {0.5, -0.40000000000000002, 
        42.874444820000001}, {0.5, -0.29999999999999999, 
        41.112097230000003}, {0.5, -0.20000000000000001, 23.43369672}, {
        0.5, -0.10000000000000001, 17.296644199999999}, {0.5, 0, 
        15.737769569999999}, {0.5, 0.10000000000000001, 13.85121202}, {0.5, 
        0.20000000000000001, 12.73868403}, {0.5, 0.29999999999999999, 
        14.707633489999999}, {0.5, 0.40000000000000002, 
        18.409518380000002}}}],
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppb1 = 
      Block[{theta = 0\ 2\ Pi}, 
        ListSurfacePlot3D[d2, 
          PlotRange \[Rule] {{0, 0.5}, {\(- .55\), 0.4}, {0, 30}}, 
          BoxRatios \[Rule] {1, 1, 0.6}, 
          ViewPoint \[Rule] {\(-1.1\), 1.25, 1.45}, Axes \[Rule] True, 
          AxesLabel \[Rule] {"\<m\>", \*"\"\<\!\(l\_1\)\>\"", \*"\"\<\!\(\
\[Chi]\^2\)\>\""}, 
          TextStyle \[Rule] {FontSize \[Rule] 
                12}, \[IndentingNewLine]LightSources \[Rule] {{{Cos[theta], 
                  Sin[theta], 1}, 
                Hue[0.1, 0.8, 1]}, {{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                Hue[0.6, 0.8, 1]}} (*\(,\)\(AmbientLight \[Rule] 
              GrayLevel[0.05]\)*) ]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bvalley1.jpg\>", pppb1, "\<JPEG\>", 
      ImageSize \[Rule] 72*4]\)], "Input"],

Cell[BoxData[
    \("bvalley1.jpg"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppb13 = 
      Block[{theta = Pi/2}, 
        Show[pppb1, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize \[Rule] 10}, 
          AxesLabel \[Rule] None, ColorOutput \[Rule] GrayLevel, 
          FormatType -> OutputForm, 
          LightSources \[Rule] {{{Cos[theta], Sin[theta], 2}, 
                GrayLevel[
                  0.75]} (*\(,\)\({{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                  GrayLevel[0.5]}\)*) }, 
          AmbientLight \[Rule] GrayLevel[0.25]]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bvalley1.eps\>", pppb13, "\<EPS\>", 
      ImageSize \[Rule] 72*\((3 + 3/8)\)*1.0]\)], "Input"],

Cell[BoxData[
    \("bvalley1.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Analyze", "Subsection"],

Cell["Chi^2", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot[Map[{Sqrt[#[\([6]\)]], #[\([3]\)]} &, datal1[\([2]\)]], 
        PlotJoined \[Rule] False, Axes -> False, 
        PlotStyle \[Rule] {PointSize[0.015]}, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        FrameLabel -> {"\<m\>", "\<\>"}, PlotRange -> {0, 20}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Select which points to plot (gooddata) and which points to use for \
fitting (gooddata).\
\>", "Text"],

Cell[BoxData[
    \(\(gooddatal1 = 
        Select[datal1[\([2]\)], \((#[\([3]\)] < 30)\) &];\)\)], "Input"],

Cell["#[[8]] is for lambda_1 and #[[9]] is for lambda_2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[{#[\([6]\)], #[\([8]\)]} &, gooddatal1]\)], "Input"],

Cell[BoxData[
    \({{0, \(-0.004693500257`\)}, {0.0025`, \(-0.05543702533`\)}, {0.01`, \
\(-0.09526007516`\)}, {0.0225`, \(-0.1213305738`\)}, {0.04`, \
\(-0.173629971`\)}, {0.0625`, \(-0.2285961599`\)}, {0.09`, \
\(-0.3032787178`\)}, {0.1225`, \(-0.3711845869`\)}, {0.16`, \
\(-0.4383723302`\)}, {0.2025`, \(-0.454417308`\)}, {0, 0}, {0, 0.1`}, {0, 
        0.2`}, {0, 0.3`}, {0, 0.4`}, {0.0025`, \(-0.1`\)}, {0.0025`, 
        0}, {0.0025`, 0.1`}, {0.0025`, 0.2`}, {0.0025`, 0.3`}, {0.0025`, 
        0.4`}, {0.01`, \(-0.1`\)}, {0.01`, 0}, {0.01`, 0.1`}, {0.01`, 
        0.2`}, {0.01`, 0.3`}, {0.01`, 0.4`}, {0.0225`, \(-0.1`\)}, {0.0225`, 
        0}, {0.0225`, 0.1`}, {0.0225`, 0.2`}, {0.0225`, 0.3`}, {0.0225`, 
        0.4`}, {0.04`, \(-0.2`\)}, {0.04`, \(-0.1`\)}, {0.04`, 0}, {0.04`, 
        0.1`}, {0.04`, 0.2`}, {0.04`, 0.3`}, {0.04`, 
        0.4`}, {0.0625`, \(-0.2`\)}, {0.0625`, \(-0.1`\)}, {0.0625`, 
        0}, {0.0625`, 0.1`}, {0.0625`, 0.2`}, {0.0625`, 0.3`}, {0.0625`, 
        0.4`}, {0.09`, \(-0.3`\)}, {0.09`, \(-0.2`\)}, {0.09`, \(-0.1`\)}, \
{0.09`, 0}, {0.09`, 0.1`}, {0.09`, 0.2`}, {0.09`, 0.3`}, {0.09`, 
        0.4`}, {0.1225`, \(-0.4`\)}, {0.1225`, \(-0.3`\)}, {0.1225`, \
\(-0.2`\)}, {0.1225`, \(-0.1`\)}, {0.1225`, 0}, {0.1225`, 0.2`}, {0.1225`, 
        0.3`}, {0.1225`, 
        0.4`}, {0.16`, \(-0.4`\)}, {0.16`, \(-0.3`\)}, {0.16`, \(-0.2`\)}, \
{0.16`, \(-0.1`\)}, {0.16`, 0}, {0.16`, 0.1`}, {0.16`, 0.2`}, {0.16`, 
        0.3`}, {0.16`, 
        0.4`}, {0.2025`, \(-0.5`\)}, {0.2025`, \(-0.4`\)}, {0.2025`, \
\(-0.2`\)}, {0.2025`, \(-0.1`\)}, {0.2025`, 0}, {0.2025`, 0.1`}, {0.2025`, 
        0.2`}, {0.2025`, 0.3`}, {0.2025`, 
        0.4`}, {0.25`, \(-0.5`\)}, {0.25`, \(-0.2`\)}, {0.25`, \(-0.1`\)}, \
{0.25`, 0}, {0.25`, 0.1`}, {0.25`, 0.2`}, {0.25`, 0.3`}, {0.25`, 
        0.4`}}\)], "Output"]
}, Open  ]],

Cell["\<\
Using gooddata, fit all couplings to either a linear or constant \
function.  Here, x is the mass, y is the other coupling.
#[[8]] is for lambda_1 and #[[9]] is for lambda_2.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefittingl1 = 
      Chop[Table[
          Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([8]\)]/#[\([7]\)], \
#[\([i]\)]/#[\([7]\)]} &, gooddatal1], 
            If[MemberQ[{10}, i], {1}, {1, x, x^2, y, x\ y, \ x^2\ y, y^2, 
                x\ y^2, x^2\ y^2}], {x, y}], {i, 6, 12}]]\)], "Input"],

Cell[BoxData[
    \({1.0000000000000002`\ x\^2, 1.0000000000000002`, 
      1.0000000000000007`\ y, \(-0.08593190052901149`\) - 
        0.3777365961688942`\ x + 0.12196664970725002`\ x\^2 - 
        0.004444226210932166`\ y - 2.909192395240925`\ x\ y + 
        6.4195431711415685`\ x\^2\ y - 0.42565815972484117`\ y\^2 - 
        3.122209245848818`\ x\ y\^2 + 9.17951209949865`\ x\^2\ y\^2, 
      1818.6365718553707`, \(-0.36496962677312617`\) - 
        11.392013926177846`\ x + 36.224557502181604`\ x\^2 - 
        5.009909777166792`\ y + 40.19638246473106`\ x\ y - 
        42.24274454756199`\ x\^2\ y + 5.448185307404669`\ y\^2 - 
        8.216580552048857`\ x\ y\^2 - 
        28.05981746864605`\ x\^2\ y\^2, \(-0.7558408615768634`\) - 
        0.6438563926404453`\ x - 4.372036015898719`\ x\^2 - 
        0.05292231632682053`\ y + 5.1635454481215195`\ x\ y - 
        14.756564848849674`\ x\^2\ y - 0.9769112236781107`\ y\^2 + 
        4.8804136791241195`\ x\ y\^2 - 
        6.339900216204633`\ x\^2\ y\^2}\)], "Output"]
}, Open  ]],

Cell["Use 1/chi^2 as the weighting.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefittingl1 = 
      Chop[Table[
          BestFit /. 
            Regress[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([8]\)]/#[\([7]\)], \
#[\([i]\)]/#[\([7]\)]} &, gooddatal1], 
              If[MemberQ[{10}, i], {1}, {1, x, x^2, y, x\ y, \ x^2\ y, y^2, 
                  x\ y^2, x^2\ y^2}], {x, y}, 
              Weights \[Rule] Map[\((1/#[\([3]\)])\) &, gooddatal1], 
              RegressionReport \[Rule] BestFit], {i, 6, 12}]]\)], "Input"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \({1.0000000000000007`\ x\^2, 0.9999999999999997`, 
      0.9999999999999994`\ y, \(-0.09896375381199057`\) - 
        0.23466236518353745`\ x - 0.13619683636697447`\ x\^2 - 
        0.019891132372517085`\ y - 2.603126016391939`\ x\ y + 
        5.854909826297299`\ x\^2\ y - 0.35984887266831445`\ y\^2 - 
        4.465989195608687`\ x\ y\^2 + 11.905501775517909`\ x\^2\ y\^2, 
      2083.3386630210603`, \(-0.4187852213364225`\) - 
        10.720074585218311`\ x + 35.00922793794801`\ x\^2 - 
        4.377708933155972`\ y + 33.39379561128087`\ x\ y - 
        29.91185189171781`\ x\^2\ y + 4.710072586641288`\ y\^2 - 
        3.316452708743011`\ x\ y\^2 - 
        34.567812459446834`\ x\^2\ y\^2, \(-0.7269311340590704`\) - 
        1.0403214406981618`\ x - 3.651276644715743`\ x\^2 - 
        0.08162552668284404`\ y + 5.29109634220076`\ x\ y - 
        15.028855622007178`\ x\^2\ y - 1.2291901279513144`\ y\^2 + 
        9.063136719912356`\ x\ y\^2 - 
        14.696547192426484`\ x\^2\ y\^2}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Create data file", "Subsection"],

Cell[BoxData[
    \(expr2 = 
      Block[{ni = 11, nj = 11, cc, y}, \[IndentingNewLine]cc = 
          couplefittingl1; \[IndentingNewLine]cc[\([5]\)] = 
          50; \[IndentingNewLine]TableForm[
          Flatten[Table[\[IndentingNewLine]{4, 
                N[cc /. {x \[Rule] 0.5\ \((i - 1)\)/\((ni - 1)\), 
                      y \[Rule] \(-0.6\) + \((j - 1)\)/\((nj - 
                                1)\)}\[IndentingNewLine], 12], {3, 4, 5, 
                  6}}, {i, 10, 11}, {j, nj}], 2], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/dave/l1_bscan23.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<l1_bscan23.out\>"]; \n\t\tWrite[fp, "\<35\>"]; \n\t\tWrite[
        fp, "\<3\>"]; \n\t\tWrite[fp, expr2]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/dave/l1_bscan23.in"\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Scanalyze lambda_2", "Section"],

Cell[CellGroupData[{

Cell["datal2 l2_dscan1*.out", "Subsection"],

Cell[BoxData[
    \(\(datal2 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from fitting \
lowest state to lattice value.\n2 parity doublets with fractional errors 1 \
0.1.\nSpectrum for P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, \
c^2 error for each) =\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & \
1, 0.25 0.25 1 1).\nSpectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  \
(26/2,6)  (32/2,6) .\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( \
2,20/2,6) \n  ( 2,24/2,4)  ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \
\n  ( 3,35/2,5)  ( 4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy \
potential determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  \
( 1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{\(-3\), 1, 621352.9232`, 0, 0, 0, 
              1, \(-0.0174241`\), \(-0.35`\), 
              50, \(-0.97786`\), \(-0.676558`\), 2, 4, 5, 6, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0}, {\(-3\), 1, 215621.2937`, 0, 0, 0, 
              1, \(-0.0174241`\), \(-0.3`\), 
              50, \(-0.97786`\), \(-0.676558`\), 2, 4, 5, 6, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
              0, 0, 0, 0, 0}, {5, 27, 
              11710.5716`, \(-0.25`\), \(-16.20793733`\), 0, 1, 
              0.003953161123`, \(-0.25`\), 
              12641.78283`, \(-2.044146908`\), \(-3.33227701`\), 2, 4, 5, 6, 
              0.061888`, \(-1.169318`\), 1.59528`, \(-0.052177`\), 
              0.173885`, \(-0.021071`\), 0.576372`, \(-0.041724`\), 
              0.033422`, \(-0.078097`\), \(-0.222346`\), \(-0.16454`\), 
              16.5242`, 16.223158`, 
              4.831484`, \(-17.451507`\), \(-17.701654`\), 
              4.014661`, \(-37.628418`\), \(-37.98207`\), 
              5.675841`, \(-56.563376`\), \(-55.618701`\), \(-15.161267`\), \
\(-37.549444`\), \(-37.822865`\), 4.388188`, \(-53.61826`\), \(-53.76931`\), 
              2.424227`, \(-78.725105`\), \(-78.815645`\), 
              1.453096`, \(-94.400094`\), \(-94.528546`\), 2.061537`, 
              16.5242`, \(-37.628418`\), \(-17.451507`\), \(-62.325949`\), \
\(-94.400094`\), \(-122.704022`\), \(-135.883083`\), \(-150.720846`\), \
\(-37.549444`\), \(-53.61826`\), \(-78.725105`\), \(-100.732705`\), \
\(-56.563376`\), \(-91.306004`\), \(-105.118733`\), \(-127.020892`\)}, {5, 
              28, 7.318571287`*^6, \(-0.3923129608`\), 66.34744756`, 0, 
              1, \(-0.01682922532`\), \(-0.2`\), 
              5.828312272`, \(-1.118342575`\), 1.083075251`, 2, 4, 5, 6, 
              0.107109`, \(-0.6874`\), 0.768425`, 0.070457`, 
              0.215776`, \(-0.109328`\), 0.595057`, 1.740642`, 1.017597`, 
              1.750782`, 1.492227`, 1.780807`, 16.5242`, 17.426967`, 
              102.646867`, 204.053782`, 203.95949`, \(-10.721192`\), 
              222.066162`, 222.892175`, 93.91965`, 238.332238`, 
              227.480523`, \(-1233.866545`\), 181.459613`, 182.185248`, 
              82.506442`, 288.86654`, 289.412816`, 62.112939`, 437.859599`, 
              438.954155`, 124.453663`, 444.866281`, 445.116623`, 28.464482`, 
              16.5242`, 204.053782`, 222.066162`, 238.332238`, 444.866281`, 
              576.195934`, 662.357752`, 730.385396`, 181.459613`, 288.86654`, 
              437.859599`, 504.1765`, 303.65306`, 491.615418`, 495.455309`, 
              581.574742`}, {5, 28, 48654.19327`, \(-1.12866863`\), 
              10.33863552`, 0, 1, \(-0.01398895025`\), \(-0.15`\), 
              5.94986618`, \(-0.9714065316`\), \(-0.7073752116`\), 2, 4, 5, 
              6, 0.160071`, \(-0.454087`\), 0.31837`, 0.09321`, 
              0.249725`, \(-0.192762`\), 0.612436`, 0.588675`, 0.690544`, 
              0.660342`, 0.973501`, 0.824877`, 16.5242`, 16.616307`, 
              2.438876`, 36.417113`, 36.376121`, \(-1.08543`\), 42.95808`, 
              42.983305`, 0.667931`, 57.606791`, 
              53.489231`, \(-109.027657`\), 31.513904`, 31.579028`, 
              1.724393`, 56.584817`, 56.65031`, 1.734178`, 77.948285`, 
              77.965735`, 0.462068`, 85.3444`, 85.419764`, 1.995529`, 
              16.5242`, 36.417113`, 42.95808`, 57.606791`, 77.948285`, 
              100.433475`, 118.649954`, 130.395948`, 31.513904`, 56.584817`, 
              85.3444`, 88.259345`, 58.183276`, 87.606098`, 97.251842`, 
              103.413753`}, {5, 28, 18.15979392`, \(-1.707815166`\), 
              7.166042235`, 0, 1, \(-0.01924799748`\), \(-0.1`\), 
              5.244899926`, \(-0.8981755678`\), \(-0.7847896055`\), 2, 4, 5, 
              6, 0.196973`, \(-0.422604`\), 0.259581`, 0.104981`, 
              0.252389`, \(-0.214433`\), 0.622697`, 0.518848`, 0.686833`, 
              0.624068`, 0.96255`, 0.838298`, 16.5242`, 16.565213`, 0.92624`, 
              25.510162`, 25.486095`, \(-0.54353`\), 29.422645`, 29.440377`, 
              0.40046`, 43.660387`, 43.666343`, 0.134511`, 23.031101`, 
              23.059764`, 0.647334`, 43.670569`, 43.702841`, 0.728845`, 
              56.913416`, 56.913111`, \(-0.006893`\), 60.810962`, 60.860019`, 
              1.107911`, 16.5242`, 25.510162`, 29.422645`, 48.220828`, 
              56.913416`, 72.970151`, 85.145758`, 92.815184`, 23.031101`, 
              43.670569`, 60.810962`, 62.866802`, 43.660387`, 61.846035`, 
              69.891381`, 79.48534`}, {2, 8, 102.1145722`, \(-2.448559817`\), 
              5.593274373`, 0, 1, \(-0.0174241`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-0.676558`\), 2, 4, 5, 6, 
              0.227989`, \(-0.419351`\), 0.254032`, 0.100521`, 
              0.256932`, \(-0.219966`\), 0.628965`, 0.44303`, 0.71861`, 
              0.564532`, 1.015864`, 0.787939`, 16.5242`, 16.539283`, 
              0.307746`, 19.933678`, 19.922339`, \(-0.231368`\), 26.479511`, 
              26.497618`, 0.36945`, 36.111619`, 36.112913`, 0.02642`, 
              18.578185`, 18.590942`, 0.260277`, 36.944123`, 36.959373`, 
              0.311135`, 46.150063`, 46.135778`, \(-0.291462`\), 48.449502`, 
              48.45871`, 0.187857`, 16.5242`, 19.933678`, 26.479511`, 
              44.050492`, 46.150063`, 58.969496`, 68.582725`, 77.262333`, 
              18.578185`, 36.944123`, 48.449502`, 49.584333`, 36.111619`, 
              49.386893`, 58.529767`, 59.397182`}, {2, 8, 
              165.0753628`, \(-2.529488775`\), 4.990338632`, 0, 
              1, \(-0.0174241`\), 0, 50, \(-0.97786`\), \(-0.676558`\), 2, 4, 
              5, 6, 0.251646`, \(-0.42461`\), 0.261295`, 0.104128`, 
              0.254572`, \(-0.223026`\), 0.630974`, 0.426153`, 0.721764`, 
              0.55993`, 1.021281`, 0.796937`, 16.5242`, 16.525449`, 
              0.025088`, 17.482364`, 17.482615`, 0.005028`, 23.707249`, 
              23.723316`, 0.322824`, 33.176733`, 33.176744`, 0.000223`, 
              16.903675`, 16.908525`, 0.09746`, 34.65702`, 34.663305`, 
              0.126275`, 42.037412`, 42.003384`, \(-0.683725`\), 43.569165`, 
              43.553544`, \(-0.313871`\), 16.5242`, 17.482364`, 23.707249`, 
              42.540614`, 42.037412`, 53.63441`, 62.169`, 70.164783`, 
              16.903675`, 34.65702`, 43.569165`, 44.322269`, 33.176733`, 
              44.337548`, 52.765248`, 54.899539`}, {2, 8, 
              155.4561347`, \(-2.41132321`\), 5.135297525`, 0, 
              1, \(-0.0174241`\), 0.05`, 50, \(-0.97786`\), \(-0.676558`\), 
              2, 4, 5, 6, 0.269918`, \(-0.430838`\), 0.26301`, 0.107566`, 
              0.249894`, \(-0.221636`\), 0.629374`, 0.453805`, 0.720502`, 
              0.582563`, 1.02076`, 0.820394`, 16.5242`, 16.528801`, 
              0.102045`, 18.974687`, 18.971172`, \(-0.077959`\), 24.371598`, 
              24.388572`, 0.376439`, 34.553044`, 34.553035`, \(-0.000197`\), 
              17.577155`, 17.576438`, \(-0.015908`\), 36.789994`, 36.79061`, 
              0.013673`, 43.662315`, 43.576678`, \(-1.899231`\), 44.508987`, 
              44.538655`, 0.657967`, 16.5242`, 18.974687`, 24.371598`, 
              43.065716`, 43.662315`, 55.697648`, 64.44005`, 73.403184`, 
              17.577155`, 36.789994`, 44.508987`, 45.726983`, 34.553044`, 
              45.519212`, 54.083236`, 57.143554`}, {5, 28, 
              170.7033518`, \(-2.262797151`\), 5.41183598`, 0, 
              1, \(-0.007333101047`\), 0.1`, 
              3.771663998`, \(-0.8000059226`\), \(-0.7879648224`\), 2, 4, 5, 
              6, 0.283277`, \(-0.433969`\), 0.249327`, 0.13065`, 
              0.226093`, \(-0.214639`\), 0.625136`, 0.527897`, 0.692086`, 
              0.668115`, 0.974345`, 0.942787`, 16.5242`, 16.538393`, 
              0.348134`, 20.62021`, 20.628644`, 0.206889`, 21.315112`, 
              21.305055`, \(-0.246702`\), 36.819378`, 36.820004`, 0.01536`, 
              18.533217`, 18.527299`, \(-0.145172`\), 39.662703`, 
              39.658968`, \(-0.09163`\), 46.30113`, 
              46.031766`, \(-6.607165`\), 46.438628`, 46.641781`, 4.983094`, 
              16.5242`, 21.315112`, 20.62021`, 40.544712`, 46.30113`, 
              59.071652`, 69.110292`, 77.439217`, 18.533217`, 39.662703`, 
              46.438628`, 48.355231`, 36.819378`, 48.678808`, 56.826821`, 
              60.832238`}, {5, 28, 135.8926523`, \(-2.041923684`\), 
              6.20747929`, 0, 1, \(-0.01548837761`\), 0.15`, 
              4.841003349`, \(-0.7190911987`\), \(-0.622476598`\), 2, 4, 5, 
              6, 0.294163`, \(-0.441648`\), 0.235343`, 0.140585`, 
              0.211682`, \(-0.204972`\), 0.616632`, 0.624113`, 0.718043`, 
              0.752337`, 1.024606`, 1.025102`, 16.5242`, 16.546448`, 
              0.649987`, 24.202554`, 24.19924`, \(-0.096839`\), 25.443173`, 
              25.443535`, 0.010584`, 41.323506`, 41.325781`, 0.066469`, 
              21.239403`, 21.226537`, \(-0.375904`\), 45.505069`, 
              45.496934`, \(-0.237656`\), 51.551703`, 
              51.347471`, \(-5.966864`\), 52.463254`, 52.626794`, 4.778005`, 
              16.5242`, 24.202554`, 25.443173`, 46.127289`, 52.463254`, 
              67.040782`, 77.746327`, 86.746042`, 21.239403`, 45.505069`, 
              51.551703`, 55.253276`, 41.323506`, 53.909671`, 63.227553`, 
              69.223638`}, {\(-3\), 1, 348659.1623`, \(-2.041923684`\), 
              6.20747929`, 0.0025`, 1, \(-0.0454408`\), \(-0.35`\), 
              50, \(-0.97786`\), \(-0.818537`\), 2, 4, 5, 6, 
              0.294163`, \(-0.441648`\), 0.235343`, 0.140585`, 
              0.211682`, \(-0.204972`\), 0.616632`, 0.624113`, 0.718043`, 
              0.752337`, 1.024606`, 1.025102`, 16.5242`, 16.546448`, 
              0.649987`, 24.202554`, 24.19924`, \(-0.096839`\), 25.443173`, 
              25.443535`, 0.010584`, 41.323506`, 41.325781`, 0.066469`, 
              21.239403`, 21.226537`, \(-0.375904`\), 45.505069`, 
              45.496934`, \(-0.237656`\), 51.551703`, 
              51.347471`, \(-5.966864`\), 52.463254`, 52.626794`, 4.778005`, 
              16.5242`, 24.202554`, 25.443173`, 46.127289`, 52.463254`, 
              67.040782`, 77.746327`, 86.746042`, 21.239403`, 45.505069`, 
              51.551703`, 55.253276`, 41.323506`, 53.909671`, 63.227553`, 
              69.223638`}, {5, 29, 
              91377.93628`, \(-0.25`\), \(-5.47514631`\), 0.0025`, 
              1, \(-0.04445855738`\), \(-0.3`\), 
              1.758545858`, \(-0.9925283848`\), \(-0.8351236465`\), 2, 4, 5, 
              6, 0.034385`, \(-2.267964`\), 3.107372`, 0.098305`, 0.112065`, 
              0.04404`, 0.595313`, 0.255248`, 0.650701`, 0.39364`, 0.914219`, 
              0.62225`, 16.5242`, 16.405526`, 
              0.357466`, \(-1.425148`\), \(-1.43443`\), 
              0.027958`, \(-4.171327`\), \(-4.290549`\), 
              0.359117`, \(-8.403197`\), \(-8.460113`\), 
              0.17144`, \(-9.660747`\), \(-9.79966`\), 
              0.418429`, \(-13.586452`\), \(-13.618583`\), 
              0.096783`, \(-22.772718`\), \(-22.795332`\), 
              0.068117`, \(-27.340776`\), \(-27.514346`\), 0.522824`, 
              16.5242`, \(-8.403197`\), \(-1.425148`\), \(-4.171327`\), \
\(-27.819967`\), \(-37.243264`\), \(-37.230953`\), \(-40.953696`\), \
\(-9.660747`\), \(-13.586452`\), \(-22.772718`\), \(-27.340776`\), \
\(-11.776079`\), \(-18.343463`\), \(-28.218898`\), \(-42.582229`\)}, {5, 27, 
              19342.13554`, \(-0.25`\), \(-12.45866636`\), 0.0025`, 
              1, \(-0.0535448608`\), \(-0.25`\), 
              3791.338645`, \(-2.293169152`\), \(-3.813774325`\), 2, 4, 5, 6, 
              0.084771`, \(-1.365321`\), 1.895128`, \(-0.064853`\), 
              0.191086`, \(-0.051625`\), 0.618581`, \(-0.058393`\), 
              0.015777`, \(-0.106338`\), \(-0.280927`\), \(-0.217881`\), 
              16.5242`, 16.30371`, 
              3.725894`, \(-18.043229`\), \(-18.390045`\), 
              5.860571`, \(-31.727505`\), \(-31.740841`\), 
              0.225353`, \(-42.824414`\), \(-44.064848`\), 
              20.961134`, \(-32.328937`\), \(-32.527494`\), 
              3.355263`, \(-44.543999`\), \(-44.648648`\), 
              1.768378`, \(-74.214596`\), \(-74.298942`\), 
              1.425301`, \(-77.886007`\), \(-77.973171`\), 1.47292`, 
              16.5242`, \(-31.727505`\), \(-18.043229`\), \(-42.824414`\), \
\(-77.886007`\), \(-101.561729`\), \(-104.772636`\), \(-109.66219`\), \
\(-32.328937`\), \(-44.543999`\), \(-74.214596`\), \(-82.833153`\), \
\(-45.69237`\), \(-76.013677`\), \(-58.761705`\), \(-113.446187`\)}, {5, 28, 
              8661.446821`, \(-0.6181117423`\), 17.82769607`, 0.0025`, 
              1, \(-0.0450255167`\), \(-0.2`\), 
              10.7590957`, \(-1.191653777`\), \(-0.7404257693`\), 2, 4, 5, 6, 
              0.15319`, \(-0.595015`\), 0.557359`, 0.083268`, 
              0.228568`, \(-0.153314`\), 0.641139`, 0.787614`, 0.724696`, 
              0.825068`, 1.018223`, 0.925495`, 16.5242`, 16.736571`, 
              9.279843`, 63.588331`, 63.484741`, \(-4.526552`\), 79.062756`, 
              79.161396`, 4.310188`, 82.770392`, 82.447199`, \(-14.122357`\), 
              55.693782`, 55.842718`, 6.507978`, 93.862128`, 93.992278`, 
              5.687064`, 135.275707`, 135.317139`, 1.810447`, 150.246709`, 
              150.35693`, 4.81627`, 16.5242`, 63.588331`, 79.062756`, 
              82.770392`, 135.275707`, 174.155908`, 207.075815`, 211.91172`, 
              55.693782`, 93.862128`, 150.246709`, 150.866566`, 97.036802`, 
              152.606165`, 146.968718`, 191.388581`}, {5, 27, 
              11.31910381`, \(-1.383433776`\), 7.014007365`, 0.0025`, 
              1, \(-0.02525906564`\), \(-0.15`\), 
              16175.97468`, \(-0.6849988242`\), \(-0.7596724741`\), 2, 4, 5, 
              6, 0.207132`, \(-0.450367`\), 0.270491`, 0.144925`, 
              0.211042`, \(-0.205813`\), 0.657259`, 0.609197`, 0.741309`, 
              0.7479`, 1.040057`, 1.036059`, 16.5242`, 16.577501`, 1.238996`, 
              28.616048`, 28.581236`, \(-0.809201`\), 34.883817`, 34.905755`, 
              0.50994`, 45.448723`, 45.457714`, 0.208976`, 23.65788`, 
              23.691817`, 0.788878`, 44.669192`, 44.7081`, 0.90443`, 
              58.943264`, 58.947668`, 0.102381`, 61.601631`, 61.636827`, 
              0.818139`, 16.5242`, 28.616048`, 34.883817`, 49.222279`, 
              58.943264`, 75.039262`, 91.235215`, 96.670937`, 23.65788`, 
              44.669192`, 61.601631`, 65.130413`, 45.448723`, 68.302879`, 
              72.292789`, 78.779695`}, {5, 28, 
              12.63793829`, \(-1.52060813`\), 6.873277128`, 0.0025`, 
              1, \(-0.05672391479`\), \(-0.1`\), 
              4.147042994`, \(-0.8730681844`\), \(-0.9201131462`\), 2, 4, 5, 
              6, 0.230804`, \(-0.447165`\), 0.284301`, 0.124612`, 
              0.23019`, \(-0.208346`\), 0.661339`, 0.578573`, 0.713503`, 
              0.695992`, 0.98954`, 0.942092`, 16.5242`, 16.562586`, 
              0.974321`, 25.436874`, 25.411568`, \(-0.64231`\), 27.66419`, 
              27.680846`, 0.422751`, 44.49986`, 44.504429`, 0.115969`, 
              23.920308`, 23.945126`, 0.629915`, 45.262422`, 45.29016`, 
              0.704042`, 58.227889`, 58.223927`, \(-0.100583`\), 61.033656`, 
              61.046773`, 0.332948`, 16.5242`, 25.436874`, 27.66419`, 
              46.199706`, 58.227889`, 74.041352`, 86.91726`, 96.447387`, 
              23.920308`, 45.262422`, 61.033656`, 62.876984`, 44.49986`, 
              62.870037`, 72.616466`, 72.649893`}, {2, 8, 
              103.7340288`, \(-2.266123999`\), 5.1223202`, 0.0025`, 
              1, \(-0.0454408`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-0.818537`\), 2, 4, 5, 6, 
              0.264035`, \(-0.43718`\), 0.263973`, 0.118486`, 
              0.235762`, \(-0.213164`\), 0.668103`, 0.466639`, 0.743183`, 
              0.607161`, 1.038698`, 0.86754`, 16.5242`, 16.536578`, 
              0.267859`, 19.768407`, 19.75899`, \(-0.203787`\), 25.627463`, 
              25.64184`, 0.311097`, 35.604744`, 35.605658`, 0.019771`, 
              18.343496`, 18.353804`, 0.223059`, 36.751922`, 36.764554`, 
              0.273331`, 45.387239`, 45.370992`, \(-0.351579`\), 46.146943`, 
              46.149343`, 0.051922`, 16.5242`, 19.768407`, 25.627463`, 
              43.444519`, 45.387239`, 57.431766`, 67.831563`, 74.585934`, 
              18.343496`, 36.751922`, 46.146943`, 48.035361`, 35.604744`, 
              49.268256`, 56.315526`, 58.80643`}, {5, 29, 
              147.7770683`, \(-2.591950891`\), 4.918677832`, 0.0025`, 
              1, \(-0.04749893013`\), 0, 
              4.817226507`, \(-0.963797792`\), \(-0.8310802244`\), 2, 4, 5, 
              6, 0.287058`, \(-0.441191`\), 0.272565`, 0.121973`, 
              0.232094`, \(-0.213314`\), 0.669546`, 0.472587`, 0.742855`, 
              0.616433`, 1.038005`, 0.88374`, 16.5242`, 16.526698`, 
              0.056427`, 18.998285`, 18.999602`, 0.029769`, 20.959782`, 
              20.969214`, 0.213078`, 34.992368`, 34.992377`, 0.00019`, 
              17.900089`, 17.904369`, 0.096672`, 36.802241`, 36.808198`, 
              0.134567`, 44.333004`, 44.292327`, \(-0.91894`\), 44.555836`, 
              44.545569`, \(-0.23194`\), 16.5242`, 18.998285`, 20.959782`, 
              39.166079`, 44.333004`, 56.004555`, 65.810348`, 72.592901`, 
              17.900089`, 36.802241`, 44.555836`, 46.104362`, 34.992368`, 
              47.411874`, 54.377963`, 57.312392`}, {2, 8, 
              155.4499025`, \(-2.549202041`\), 4.831531261`, 0.0025`, 
              1, \(-0.0454408`\), 0.05`, 50, \(-0.97786`\), \(-0.818537`\), 
              2, 4, 5, 6, 0.305715`, \(-0.444494`\), 0.271698`, 0.124227`, 
              0.227793`, \(-0.210785`\), 0.66848`, 0.480751`, 0.744626`, 
              0.624696`, 1.042269`, 0.894839`, 16.5242`, 16.527022`, 
              0.066693`, 19.651457`, 19.649876`, \(-0.037376`\), 24.224738`, 
              24.238425`, 0.323467`, 34.905809`, 34.905771`, \(-0.000904`\), 
              17.734297`, 17.733666`, \(-0.014904`\), 37.274458`, 37.275599`, 
              0.026947`, 43.693624`, 43.664586`, \(-0.686251`\), 44.027589`, 
              43.968599`, \(-1.394129`\), 16.5242`, 19.651457`, 24.224738`, 
              42.62679`, 44.027589`, 55.567915`, 65.327848`, 72.973928`, 
              17.734297`, 37.274458`, 43.693624`, 45.257078`, 34.905809`, 
              46.813999`, 53.376724`, 56.965991`}, {2, 8, 
              130.568364`, \(-2.407668412`\), 5.145864011`, 0.0025`, 
              1, \(-0.0454408`\), 0.1`, 50, \(-0.97786`\), \(-0.818537`\), 2, 
              4, 5, 6, 0.320013`, \(-0.446973`\), 0.261525`, 0.126859`, 
              0.221864`, \(-0.205543`\), 0.664397`, 0.520541`, 0.740118`, 
              0.655077`, 1.037079`, 0.9203`, 16.5242`, 16.533034`, 0.232758`, 
              21.789126`, 21.781926`, \(-0.189688`\), 25.692473`, 25.70774`, 
              0.402261`, 37.211937`, 37.212386`, 0.011832`, 18.937176`, 
              18.931832`, \(-0.140815`\), 40.356736`, 
              40.354126`, \(-0.068759`\), 45.982808`, 
              45.923764`, \(-1.555692`\), 46.95587`, 
              46.909721`, \(-1.215915`\), 16.5242`, 21.789126`, 25.692473`, 
              43.84591`, 46.95587`, 59.266461`, 69.662907`, 76.987783`, 
              18.937176`, 40.356736`, 45.982808`, 48.016902`, 37.211937`, 
              49.5566`, 56.191296`, 60.898746`}, {2, 14, 
              96.73565451`, \(-2.211644728`\), 5.753031483`, 0.0025`, 
              1, \(-0.04554500723`\), 0.15`, 
              5.635549454`, \(-0.9718653369`\), \(-0.8226121244`\), 2, 4, 5, 
              6, 0.33086`, \(-0.447893`\), 0.239132`, 0.129759`, 
              0.214578`, \(-0.198068`\), 0.657645`, 0.584118`, 0.731755`, 
              0.704978`, 1.026545`, 0.959564`, 16.5242`, 16.5413`, 0.520792`, 
              22.57172`, 22.597805`, 0.794425`, 27.368599`, 
              27.341811`, \(-0.815841`\), 41.244758`, 41.246201`, 0.043928`, 
              21.114032`, 21.103112`, \(-0.332559`\), 45.409094`, 
              45.403104`, \(-0.182436`\), 50.318762`, 
              50.248448`, \(-2.141405`\), 52.209769`, 52.216464`, 0.203896`, 
              16.5242`, 22.57172`, 27.368599`, 44.966456`, 52.209769`, 
              65.934861`, 77.577942`, 84.797635`, 21.114032`, 45.409094`, 
              50.318762`, 53.42596`, 41.244758`, 54.758443`, 61.624863`, 
              67.925655`}, {5, 27, 
              20869.05867`, \(-0.25`\), \(-13.56124257`\), 0.01`, 
              1, \(-0.01069089575`\), \(-0.35`\), 
              19416.31114`, \(-2.930705843`\), \(-4.662701955`\), 2, 4, 5, 6, 
              0.086092`, \(-1.417681`\), 1.920196`, \(-0.114174`\), 
              0.212947`, \(-0.101564`\), 
              0.644607`, \(-0.266884`\), \(-0.104453`\), \(-0.345199`\), \
\(-0.520689`\), \(-0.532528`\), 16.5242`, 16.212148`, 
              5.829189`, \(-34.692634`\), \(-42.814031`\), 
              151.709116`, \(-42.926189`\), \(-43.342087`\), 
              7.769046`, \(-44.406523`\), \(-44.511592`\), 
              1.962693`, \(-36.888053`\), \(-37.10247`\), 
              4.005342`, \(-53.128554`\), \(-53.274767`\), 
              2.731279`, \(-88.339318`\), \(-88.545776`\), 
              3.856683`, \(-91.506398`\), \(-91.617688`\), 2.078919`, 
              16.5242`, \(-44.406523`\), \(-34.692634`\), \(-42.926189`\), \
\(-91.506398`\), \(-120.145975`\), \(-118.391275`\), \(-148.78576`\), \
\(-36.888053`\), \(-53.128554`\), \(-88.339318`\), \(-104.824679`\), \
\(-58.021917`\), \(-84.80715`\), \(-111.117834`\), \(-117.727916`\)}, {5, 27, 
              38957.31453`, \(-0.25`\), \(-8.911083281`\), 0.01`, 
              1, \(-0.07433142876`\), \(-0.3`\), 
              1727.293895`, \(-2.6818686`\), \(-4.381350012`\), 2, 4, 5, 6, 
              0.088351`, \(-1.665334`\), 2.317498`, \(-0.090013`\), 
              0.197068`, \(-0.076052`\), 
              0.647595`, \(-0.070066`\), \(-0.03324`\), \(-0.151661`\), \
\(-0.386742`\), \(-0.325691`\), 16.5242`, 16.343776`, 
              2.27278`, \(-18.051926`\), \(-18.331377`\), 
              3.520212`, \(-23.54888`\), \(-23.555244`\), 
              0.080163`, \(-26.149154`\), \(-28.888614`\), 
              34.508708`, \(-23.935636`\), \(-24.086867`\), 
              1.905036`, \(-32.25763`\), \(-32.334251`\), 
              0.965185`, \(-57.520963`\), \(-57.589415`\), 
              0.862287`, \(-59.05197`\), \(-59.349709`\), 3.750599`, 
              16.5242`, \(-23.54888`\), \(-18.051926`\), \(-26.149154`\), \
\(-57.520963`\), \(-75.864782`\), \(-81.589281`\), \(-85.847644`\), \
\(-23.935636`\), \(-32.25763`\), \(-59.05197`\), \(-62.650887`\), \
\(-33.17909`\), \(-45.091283`\), \(-61.878813`\), \(-72.226424`\)}, {5, 27, 
              199364.1274`, \(-0.25`\), \(-26.28615285`\), 0.01`, 
              1, \(-0.08897094788`\), \(-0.25`\), 
              214.7977291`, \(-2.230954114`\), \(-3.965451654`\), 2, 4, 5, 6, 
              0.130962`, \(-1.154003`\), 1.543895`, \(-0.027334`\), 
              0.197979`, \(-0.073604`\), 0.662767`, \(-0.06978`\), 
              0.079025`, \(-0.078355`\), \(-0.182791`\), \(-0.102622`\), 
              16.5242`, 16.103437`, 
              23.175611`, \(-73.688492`\), \(-80.535121`\), 
              377.11184`, \(-80.69981`\), \(-94.754299`\), 
              774.120227`, \(-96.430938`\), \(-96.95677`\), 
              28.962764`, \(-81.265442`\), \(-81.573848`\), 
              16.986987`, \(-117.963207`\), \(-118.192381`\), 
              12.622898`, \(-188.590472`\), \(-188.719017`\), 
              7.080241`, \(-205.672064`\), \(-206.302547`\), 34.726994`, 
              16.5242`, \(-80.69981`\), \(-73.688492`\), \(-96.430938`\), \
\(-188.590472`\), \(-245.061384`\), \(-262.967345`\), \(-288.67059`\), \
\(-81.265442`\), \(-117.963207`\), \(-205.672064`\), \(-216.542455`\), \
\(-121.835402`\), \(-165.140728`\), \(-194.796095`\), \(-226.408749`\)}, {5, 
              28, 978.2838361`, \(-0.8759446071`\), 9.383024022`, 0.01`, 
              1, \(-0.0660722549`\), \(-0.2`\), 
              189.9171075`, \(-1.156019197`\), \(-1.232546582`\), 2, 4, 5, 6, 
              0.205211`, \(-0.529682`\), 0.391689`, 0.108701`, 
              0.219351`, \(-0.174468`\), 0.684583`, 0.640339`, 0.687946`, 
              0.719998`, 0.93301`, 0.906118`, 16.5242`, 16.621076`, 
              2.984553`, 38.720446`, 38.657262`, \(-1.946566`\), 48.054003`, 
              48.086659`, 1.006064`, 58.165865`, 58.581535`, 12.805938`, 
              32.825581`, 32.884427`, 1.812921`, 58.699235`, 58.760363`, 
              1.883239`, 80.016209`, 80.02351`, 0.224926`, 84.433862`, 
              84.485362`, 1.586625`, 16.5242`, 38.720446`, 48.054003`, 
              58.165865`, 80.016209`, 101.750672`, 124.83485`, 131.817703`, 
              32.825581`, 58.699235`, 84.433862`, 88.502632`, 59.906089`, 
              93.102607`, 97.158944`, 98.206454`}, {5, 27, 
              9.806756036`, \(-1.325594182`\), 6.235031965`, 0.01`, 
              1, \(-0.0632014333`\), \(-0.15`\), 
              3347.872598`, \(-0.6361395684`\), \(-0.7718252495`\), 2, 4, 5, 
              6, 0.245922`, \(-0.463302`\), 0.269442`, 0.163908`, 
              0.177196`, \(-0.180822`\), 0.69516`, 0.625517`, 0.790446`, 
              0.783457`, 1.105933`, 1.110023`, 16.5242`, 16.568728`, 
              1.092423`, 26.893092`, 26.860263`, \(-0.805397`\), 32.766918`, 
              32.784433`, 0.429689`, 43.541395`, 43.547604`, 0.152344`, 
              22.840968`, 22.867303`, 0.64608`, 43.501541`, 43.532542`, 
              0.76055`, 56.324354`, 56.313691`, \(-0.261582`\), 57.309683`, 
              57.344752`, 0.860354`, 16.5242`, 26.893092`, 32.766918`, 
              47.582022`, 56.324354`, 71.066316`, 87.152671`, 92.027841`, 
              22.840968`, 43.501541`, 57.309683`, 60.900213`, 43.541395`, 
              65.025292`, 69.590247`, 71.566565`}, {5, 28, 
              22.68094548`, \(-1.453093682`\), 6.111328363`, 0.01`, 
              1, \(-0.08766622212`\), \(-0.1`\), 
              5.320493277`, \(-0.9281946993`\), \(-0.9913113519`\), 2, 4, 5, 
              6, 0.269605`, \(-0.460481`\), 0.28531`, 0.135831`, 
              0.208928`, \(-0.193336`\), 0.6977`, 0.579021`, 0.75143`, 
              0.709787`, 1.037433`, 0.980662`, 16.5242`, 16.554064`, 
              0.787293`, 24.444801`, 24.423045`, \(-0.57353`\), 27.36478`, 
              27.379291`, 0.382522`, 42.856516`, 42.859499`, 0.078642`, 
              22.914984`, 22.933893`, 0.498494`, 44.032574`, 44.054642`, 
              0.581786`, 55.67106`, 55.658002`, \(-0.34424`\), 56.564271`, 
              56.578019`, 0.362443`, 16.5242`, 24.444801`, 27.36478`, 
              45.442445`, 55.67106`, 70.167732`, 83.933677`, 90.754409`, 
              22.914984`, 44.032574`, 56.564271`, 58.923282`, 42.856516`, 
              61.103131`, 68.101409`, 71.860241`}, {2, 8, 
              107.7901886`, \(-1.92183386`\), 4.863877912`, 0.01`, 
              1, \(-0.0803835`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-0.947302`\), 2, 4, 5, 6, 
              0.30078`, \(-0.450544`\), 0.268217`, 0.134782`, 
              0.210577`, \(-0.197159`\), 0.703243`, 0.496198`, 0.76717`, 
              0.649397`, 1.062482`, 0.939603`, 16.5242`, 16.533409`, 
              0.215564`, 20.199142`, 20.192367`, \(-0.158589`\), 25.637567`, 
              25.6494`, 0.276972`, 36.066142`, 36.066813`, 0.015705`, 
              18.66614`, 18.674823`, 0.203258`, 37.570874`, 37.581916`, 
              0.258446`, 45.539851`, 45.512361`, \(-0.643473`\), 45.968014`, 
              45.975759`, 0.181308`, 16.5242`, 20.199142`, 25.637567`, 
              43.03077`, 45.968014`, 57.664531`, 69.116114`, 74.275134`, 
              18.66614`, 37.570874`, 45.539851`, 47.77383`, 36.066142`, 
              50.477456`, 55.779791`, 59.011937`}, {5, 29, 
              144.3668351`, \(-2.1565176`\), 4.80939875`, 0.01`, 
              1, \(-0.08159053969`\), 0, 
              5.428348942`, \(-0.9630644603`\), \(-0.9584837465`\), 2, 4, 5, 
              6, 0.323662`, \(-0.452512`\), 0.274521`, 0.137834`, 
              0.206554`, \(-0.195828`\), 0.704467`, 0.512114`, 0.76685`, 
              0.664566`, 1.06175`, 0.957806`, 16.5242`, 16.526277`, 
              0.051727`, 20.369215`, 20.370987`, 0.044136`, 22.019672`, 
              22.027506`, 0.195127`, 36.469056`, 36.469073`, 0.000425`, 
              18.701581`, 18.70531`, 0.09289`, 38.597934`, 38.603669`, 
              0.142834`, 45.208945`, 45.205353`, \(-0.089445`\), 46.202882`, 
              46.140922`, \(-1.543181`\), 16.5242`, 20.369215`, 22.019672`, 
              39.815011`, 46.202882`, 57.839968`, 69.075166`, 74.362839`, 
              18.701581`, 38.597934`, 45.208945`, 47.242369`, 36.469056`, 
              50.189967`, 55.420599`, 59.201258`}, {5, 29, 
              163.7215606`, \(-2.254571566`\), 4.804332014`, 0.01`, 
              1, \(-0.07978697135`\), 0.05`, 
              5.800267825`, \(-0.9526790137`\), \(-0.9642919795`\), 2, 4, 5, 
              6, 0.342635`, \(-0.453802`\), 0.272112`, 0.141239`, 
              0.200926`, \(-0.192164`\), 0.703704`, 0.529789`, 0.765161`, 
              0.681308`, 1.059676`, 0.977659`, 16.5242`, 16.526781`, 
              0.067988`, 21.259803`, 21.261539`, 0.045715`, 22.292238`, 
              22.298838`, 0.173823`, 36.995652`, 36.995592`, \(-0.001585`\), 
              18.82011`, 18.819544`, \(-0.014908`\), 39.678696`, 39.680408`, 
              0.045076`, 45.140278`, 45.131706`, \(-0.225761`\), 46.669306`, 
              46.518262`, \(-3.978225`\), 16.5242`, 21.259803`, 22.292238`, 
              40.514405`, 46.669306`, 58.338631`, 69.688905`, 76.011292`, 
              18.82011`, 39.678696`, 45.140278`, 47.11232`, 36.995652`, 
              50.466373`, 55.346602`, 59.802111`}, {5, 28, 
              145.8537584`, \(-1.986849653`\), 5.657336292`, 0.01`, 
              1, \(-0.0967103854`\), 0.1`, 
              6.54195279`, \(-0.8805251546`\), \(-0.9649330854`\), 2, 4, 5, 
              6, 0.3554`, \(-0.460517`\), 0.27729`, 0.148703`, 
              0.189385`, \(-0.182621`\), 0.696913`, 0.636759`, 0.75686`, 
              0.770166`, 1.048882`, 1.055954`, 16.5242`, 16.531383`, 
              0.231061`, 25.067568`, 25.062776`, \(-0.154145`\), 26.466794`, 
              26.47804`, 0.361793`, 42.501122`, 42.501434`, 0.01005`, 
              22.175178`, 22.169554`, \(-0.18091`\), 46.681022`, 
              46.67903`, \(-0.06407`\), 51.800573`, 
              51.764284`, \(-1.16739`\), 54.207715`, 
              54.05776`, \(-4.824009`\), 16.5242`, 25.067568`, 26.466794`, 
              44.977838`, 54.207715`, 67.848971`, 79.968464`, 86.988601`, 
              22.175178`, 46.681022`, 51.800573`, 54.481759`, 42.501122`, 
              56.839333`, 62.958846`, 69.65201`}, {5, 28, 
              147.7855902`, \(-2.287031359`\), 5.039666178`, 0.01`, 
              1, \(-0.06754748312`\), 0.15`, 
              7.036844115`, \(-0.7336110272`\), \(-0.8851407307`\), 2, 4, 5, 
              6, 0.369505`, \(-0.446785`\), 0.224852`, 0.165799`, 
              0.16378`, \(-0.16561`\), 0.696873`, 0.61764`, 0.772887`, 
              0.777482`, 1.077483`, 1.107761`, 16.5242`, 16.537371`, 
              0.392438`, 23.85695`, 23.864622`, 0.228594`, 24.372427`, 
              24.365155`, \(-0.216674`\), 39.366967`, 39.367824`, 0.025522`, 
              19.746441`, 19.738164`, \(-0.24663`\), 43.062987`, 
              43.059456`, \(-0.105221`\), 46.733427`, 
              46.71226`, \(-0.630673`\), 49.151038`, 
              48.934538`, \(-6.450611`\), 16.5242`, 24.372427`, 23.85695`, 
              43.425549`, 49.405552`, 61.64536`, 74.404359`, 79.038977`, 
              19.746441`, 43.062987`, 46.733427`, 49.151038`, 39.366967`, 
              53.769826`, 57.467697`, 63.480489`}, {5, 28, 
              70052.45921`, \(-0.25`\), \(-6.408212565`\), 0.0225`, 
              1, \(-0.1068460175`\), \(-0.35`\), 
              28.97937759`, \(-1.442549747`\), \(-1.476863014`\), 2, 4, 5, 6, 
              0.091137`, \(-2.050885`\), 2.837826`, 0.091784`, 0.088317`, 
              0.049579`, 0.674423`, 0.291508`, 0.639936`, 0.394141`, 0.8558`, 
              0.58964`, 16.5242`, 16.378969`, 
              1.357088`, \(-15.729622`\), \(-15.952603`\), 
              2.083623`, \(-16.489273`\), \(-16.557122`\), 
              0.634009`, \(-16.559951`\), \(-19.729138`\), 
              29.614042`, \(-17.613079`\), \(-17.730154`\), 
              1.093993`, \(-22.902894`\), \(-22.9566`\), 
              0.501854`, \(-42.04276`\), \(-42.09326`\), 
              0.471889`, \(-43.569867`\), \(-43.76933`\), 1.863848`, 
              
              16.5242`, \(-16.559951`\), \(-15.729622`\), \(-16.489273`\), \
\(-42.04276`\), \(-56.178559`\), \(-48.148372`\), \(-76.430659`\), \
\(-17.613079`\), \(-22.902894`\), \(-43.569867`\), \(-49.732616`\), \
\(-23.389095`\), \(-37.553076`\), \(-39.807379`\), \(-49.006473`\)}, {5, 28, 
              19781.3754`, \(-0.25`\), \(-12.29730701`\), 0.0225`, 
              1, \(-0.1212041037`\), \(-0.3`\), 
              4.647011994`, \(-2.724141661`\), \(-4.654431069`\), 2, 4, 5, 6, 
              0.13212`, \(-1.491128`\), 2.044693`, \(-0.065865`\), 
              0.201445`, \(-0.092482`\), 0.689074`, \(-0.113655`\), 
              0.001105`, \(-0.153942`\), \(-0.341324`\), \(-0.254084`\), 
              16.5242`, 16.294364`, 
              5.974689`, \(-34.887464`\), \(-36.621101`\), 
              45.066737`, \(-36.619608`\), \(-39.007646`\), 
              62.078187`, \(-40.942181`\), \(-41.022318`\), 
              2.083208`, \(-39.00342`\), \(-39.16468`\), 
              4.192037`, \(-54.205779`\), \(-54.31478`\), 
              2.833551`, \(-89.531856`\), \(-89.605533`\), 
              1.915288`, \(-99.496172`\), \(-99.670161`\), 4.522915`, 
              16.5242`, \(-36.619608`\), \(-34.887464`\), \(-40.942181`\), \
\(-89.531856`\), \(-117.361284`\), \(-118.025921`\), \(-154.426617`\), \
\(-39.00342`\), \(-54.205779`\), \(-99.496172`\), \(-104.559888`\), \
\(-55.967559`\), \(-71.258819`\), \(-95.051176`\), \(-105.866278`\)}, {5, 26, 
              446784.4102`, \(-0.425959422`\), 23.89958452`, 0.0225`, 
              1, \(-0.1170852894`\), \(-0.25`\), 
              10017.59461`, \(-1.897510172`\), \(-2.105590106`\), 2, 4, 5, 6, 
              0.194289`, \(-0.780733`\), 0.837645`, 0.041304`, 
              0.214731`, \(-0.125617`\), 0.707128`, 0.637921`, 0.556641`, 
              0.650328`, 0.688051`, 0.685596`, 16.5242`, 16.844502`, 
              23.796693`, 92.225131`, 92.018414`, \(-15.357959`\), 
              113.566655`, 123.445722`, 733.961728`, 123.328486`, 
              126.740821`, 253.518227`, 85.250782`, 85.445273`, 14.449567`, 
              139.836827`, 140.016587`, 13.355187`, 200.619514`, 200.683295`, 
              4.738593`, 217.939962`, 218.02179`, 6.079311`, 16.5242`, 
              92.225131`, 113.566655`, 123.328486`, 200.619514`, 256.709372`, 
              304.464739`, 332.425244`, 85.250782`, 139.836827`, 217.939962`, 
              222.382728`, 142.775936`, 191.606587`, 230.576137`, 
              268.830332`}, {5, 26, 57.53422086`, \(-1.049861328`\), 
              6.645021074`, 0.0225`, 1, \(-0.09050857363`\), \(-0.2`\), 
              31185.82407`, \(-0.8728731733`\), \(-1.026300738`\), 2, 4, 5, 
              6, 0.257055`, \(-0.497583`\), 0.292683`, 0.154748`, 
              0.175544`, \(-0.165888`\), 0.725071`, 0.647664`, 0.784632`, 
              0.78522`, 1.081368`, 1.081031`, 16.5242`, 16.585908`, 
              1.686504`, 30.982806`, 30.938163`, \(-1.220111`\), 36.449786`, 
              36.46775`, 0.490964`, 48.209128`, 48.218313`, 0.251003`, 
              25.531313`, 25.563947`, 0.891878`, 47.794242`, 47.832697`, 
              1.050971`, 62.66814`, 62.624053`, \(-1.204904`\), 62.947686`, 
              63.021152`, 2.007861`, 16.5242`, 30.982806`, 36.449786`, 
              50.681213`, 62.66814`, 78.685979`, 98.900871`, 101.368236`, 
              25.531313`, 47.794242`, 62.947686`, 67.685824`, 48.209128`, 
              70.950421`, 80.139805`, 80.475941`}, {5, 27, 
              10.00028973`, \(-1.145745838`\), 6.186607539`, 0.0225`, 
              1, \(-0.116075451`\), \(-0.15`\), 
              3688.830259`, \(-0.7507564924`\), \(-0.9000860862`\), 2, 4, 5, 
              6, 0.280623`, \(-0.484804`\), 0.294634`, 0.165724`, 
              0.164046`, \(-0.162647`\), 0.727851`, 0.657541`, 0.811115`, 
              0.807695`, 1.125523`, 1.127594`, 16.5242`, 16.564718`, 
              1.125497`, 26.926724`, 26.894213`, \(-0.903079`\), 34.084355`, 
              34.100248`, 0.44148`, 45.137532`, 45.142443`, 0.136426`, 
              24.298246`, 24.322105`, 0.662726`, 45.912922`, 45.94106`, 
              0.781584`, 58.859573`, 58.865941`, 0.176879`, 59.01234`, 
              59.021407`, 0.251876`, 16.5242`, 26.926724`, 34.084355`, 
              48.912221`, 58.859573`, 73.822043`, 90.788011`, 94.960221`, 
              24.298246`, 45.912922`, 59.01234`, 62.274976`, 45.137532`, 
              66.723709`, 71.478755`, 75.90954`}, {5, 28, 
              19.09177274`, \(-1.276494696`\), 6.179577888`, 0.0225`, 
              1, \(-0.1316554889`\), \(-0.1`\), 
              4.401323656`, \(-0.9305509613`\), \(-1.088642355`\), 2, 4, 5, 
              6, 0.306244`, \(-0.474985`\), 0.294651`, 0.150335`, 
              0.182143`, \(-0.171969`\), 0.730111`, 0.645962`, 0.779385`, 
              0.778383`, 1.069764`, 1.064426`, 16.5242`, 16.550151`, 
              0.785773`, 25.855852`, 25.837691`, \(-0.549893`\), 27.335968`, 
              27.34984`, 0.420046`, 45.793385`, 45.795753`, 0.071711`, 
              24.746097`, 24.763086`, 0.514393`, 47.640699`, 47.661208`, 
              0.620993`, 59.251446`, 59.234379`, \(-0.516756`\), 59.597882`, 
              59.606961`, 0.274891`, 16.5242`, 25.855852`, 27.335968`, 
              46.732276`, 59.597882`, 74.59501`, 90.363629`, 95.820579`, 
              24.746097`, 47.640699`, 59.251446`, 61.774064`, 45.793385`, 
              65.606122`, 71.3856`, 76.340177`}, {2, 8, 
              111.3644914`, \(-1.620130462`\), 4.789326912`, 0.0225`, 
              1, \(-0.122252`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-1.06285`\), 2, 4, 5, 6, 
              0.338021`, \(-0.458959`\), 0.264318`, 0.149402`, 
              0.183879`, \(-0.175707`\), 0.735763`, 0.536787`, 0.79142`, 
              0.696006`, 1.08811`, 1.008216`, 16.5242`, 16.530618`, 
              0.166248`, 21.295767`, 21.291424`, \(-0.112493`\), 26.497785`, 
              26.507894`, 0.261841`, 37.56584`, 37.566352`, 0.013275`, 
              19.584968`, 19.592583`, 0.197255`, 39.505543`, 39.515805`, 
              0.265788`, 46.45346`, 46.449705`, \(-0.097256`\), 47.969088`, 
              47.938184`, \(-0.800468`\), 16.5242`, 21.295767`, 26.497785`, 
              43.635668`, 47.969088`, 59.707522`, 72.60943`, 76.374676`, 
              19.584968`, 39.505543`, 46.45346`, 48.892554`, 37.56584`, 
              53.143916`, 56.94357`, 61.097612`}, {5, 29, 
              158.5978783`, \(-1.816453745`\), 4.805442809`, 0.0225`, 
              1, \(-0.1198562913`\), 1\/100000, 
              5.136131096`, \(-0.9328815036`\), \(-1.087921957`\), 2, 4, 5, 
              6, 0.361695`, \(-0.457084`\), 0.262512`, 0.15493`, 
              0.176612`, \(-0.17208`\), 0.737468`, 0.564824`, 0.789171`, 
              0.723614`, 1.083711`, 1.041558`, 16.5242`, 16.526389`, 
              0.060869`, 22.105105`, 22.109542`, 0.123398`, 22.855025`, 
              22.858413`, 0.094222`, 38.724691`, 38.72471`, 0.000534`, 
              19.880386`, 19.883689`, 0.091873`, 41.227581`, 41.233403`, 
              0.161898`, 46.820849`, 46.815771`, \(-0.141219`\), 49.061779`, 
              48.901863`, \(-4.447208`\), 16.5242`, 22.105105`, 22.855025`, 
              39.911867`, 49.061779`, 60.892098`, 74.011862`, 77.769942`, 
              19.880386`, 41.227581`, 46.820849`, 49.255618`, 38.724691`, 
              54.160692`, 57.620566`, 62.357774`}, {5, 28, 
              151.7036514`, \(-1.773987675`\), 5.352637665`, 0.0225`, 
              1, \(-0.12928362`\), 0.05`, 
              3.88768724`, \(-0.8303768609`\), \(-1.032786101`\), 2, 4, 5, 6, 
              0.378399`, \(-0.460254`\), 0.269207`, 0.164889`, 
              0.161927`, \(-0.161296`\), 0.734573`, 0.655857`, 0.795937`, 
              0.804591`, 1.097074`, 1.122136`, 16.5242`, 16.526641`, 
              0.079104`, 23.37946`, 23.385061`, 0.181495`, 25.272738`, 
              25.271654`, \(-0.035102`\), 41.489848`, 41.490943`, 0.035484`, 
              22.236839`, 22.236248`, \(-0.019168`\), 46.564836`, 46.567354`, 
              0.081592`, 51.809259`, 51.798027`, \(-0.364014`\), 54.053804`, 
              53.92118`, \(-4.297949`\), 16.5242`, 25.272738`, 23.37946`, 
              41.489848`, 54.631258`, 67.770776`, 81.76834`, 87.214495`, 
              22.236839`, 46.564836`, 51.809259`, 54.053804`, 42.97786`, 
              59.232595`, 63.157665`, 69.477171`}, {2, 8, 
              137.815542`, \(-1.903154476`\), 4.884419943`, 0.0225`, 
              1, \(-0.122252`\), 0.1`, 50, \(-0.97786`\), \(-1.06285`\), 2, 
              4, 5, 6, 0.395338`, \(-0.455324`\), 0.247095`, 0.154741`, 
              0.172955`, \(-0.16709`\), 0.733269`, 0.593776`, 0.789104`, 
              0.743537`, 1.086336`, 1.052476`, 16.5242`, 16.528993`, 
              0.148099`, 24.192502`, 24.189309`, \(-0.098647`\), 26.973907`, 
              26.984549`, 0.328805`, 39.86712`, 39.86732`, 0.006159`, 
              20.356121`, 20.352015`, \(-0.126872`\), 43.55787`, 
              43.557596`, \(-0.008445`\), 47.187612`, 
              47.175656`, \(-0.369401`\), 49.389758`, 
              49.303742`, \(-2.65757`\), 16.5242`, 24.192502`, 26.973907`, 
              44.362445`, 50.408744`, 62.405444`, 75.642424`, 79.686378`, 
              20.356121`, 43.55787`, 47.187612`, 49.389758`, 39.86712`, 
              54.9259`, 57.734134`, 64.106146`}, {5, 29, 
              111.6927447`, \(-1.903118061`\), 5.371153106`, 0.0225`, 
              1, \(-0.1231571434`\), 0.15`, 
              9.650666949`, \(-0.9613784306`\), \(-1.059891064`\), 2, 4, 5, 
              6, 0.40735`, \(-0.450248`\), 0.219556`, 0.157073`, 
              0.167227`, \(-0.16099`\), 0.728307`, 0.658726`, 0.783795`, 
              0.795722`, 1.080016`, 1.092902`, 16.5242`, 16.534077`, 
              0.345777`, 26.45193`, 26.467798`, 0.555469`, 28.290433`, 
              28.278512`, \(-0.417319`\), 43.639768`, 43.640484`, 0.025079`, 
              22.333466`, 22.325191`, \(-0.289701`\), 48.30362`, 
              48.300991`, \(-0.09204`\), 51.127191`, 
              51.108837`, \(-0.642522`\), 53.826992`, 
              53.754997`, \(-2.520316`\), 16.5242`, 26.45193`, 28.290433`, 
              46.246511`, 55.29227`, 68.410909`, 82.886245`, 86.676356`, 
              22.333466`, 48.30362`, 51.127191`, 53.826992`, 43.639768`, 
              59.972683`, 62.490717`, 70.419902`}, {2, 23, 
              70885.09153`, \(-0.25`\), \(-6.81550277`\), 0.04`, 
              1, \(-0.1692036814`\), \(-0.35`\), 
              71.95355117`, \(-1.662730451`\), \(-1.739513606`\), 2, 4, 5, 6, 
              0.13014`, \(-1.984389`\), 2.734002`, 0.087872`, 0.080901`, 
              0.048668`, 0.712236`, 0.316745`, 0.64745`, 0.402898`, 
              0.849587`, 0.579242`, 16.5242`, 16.381443`, 
              2.025936`, \(-16.443278`\), \(-18.21118`\), 
              25.089162`, \(-18.2783`\), \(-22.247905`\), 
              56.334588`, \(-26.097516`\), \(-22.972396`\), \(-44.350099`\), \
\(-21.714559`\), \(-21.817291`\), 
              1.457924`, \(-28.208868`\), \(-28.266228`\), 
              0.814017`, \(-48.979392`\), \(-49.040451`\), 
              0.86651`, \(-51.807519`\), \(-55.1794`\), 47.852004`, 
              16.5242`, \(-18.2783`\), \(-16.443278`\), \(-26.097516`\), \
\(-48.979392`\), \(-65.233945`\), \(-51.807519`\), \(-89.933443`\), \
\(-21.714559`\), \(-28.208868`\), \(-55.109359`\), \(-58.449631`\), \
\(-28.753895`\), \(-41.888908`\), \(-49.522761`\), \(-55.619857`\)}, {5, 27, 
              37993.13815`, \(-0.25`\), \(-10.42108375`\), 0.04`, 
              1, \(-0.186261519`\), \(-0.3`\), 
              7.715897957`, \(-2.699770077`\), \(-5.059907042`\), 2, 4, 5, 6, 
              0.165123`, \(-1.594193`\), 2.193432`, \(-0.034305`\), 
              0.171577`, \(-0.06762`\), 0.721512`, 0.010339`, 
              0.003855`, \(-0.01087`\), \(-0.348991`\), \(-0.063519`\), 
              16.5242`, 16.345213`, 
              4.927894`, \(-28.455857`\), \(-28.335505`\), \(-3.313538`\), \
\(-32.183936`\), \(-40.381347`\), 
              225.691946`, \(-41.63115`\), \(-41.641575`\), 
              0.286999`, \(-36.147178`\), \(-36.268523`\), 
              3.340884`, \(-49.374142`\), \(-49.458553`\), 
              2.32403`, \(-79.669055`\), \(-79.772977`\), 
              2.8612`, \(-89.458485`\), \(-89.516691`\), 1.602525`, 
              16.5242`, \(-28.455857`\), \(-32.183936`\), \(-41.63115`\), \
\(-79.669055`\), \(-104.797528`\), \(-89.638245`\), \(-146.326263`\), \
\(-36.147178`\), \(-49.374142`\), \(-89.458485`\), \(-92.685345`\), \
\(-49.755582`\), \(-59.781478`\), \(-87.206295`\), \(-96.705692`\)}, {5, 26, 
              1217.980689`, \(-0.679775906`\), 9.339425576`, 0.04`, 
              1, \(-0.1441884736`\), \(-0.25`\), 
              6424.68309`, \(-1.478757276`\), \(-1.619961216`\), 2, 4, 5, 6, 
              0.256865`, \(-0.609577`\), 0.473062`, 0.10976`, 
              0.19019`, \(-0.143127`\), 0.749749`, 0.679491`, 0.713591`, 
              0.751004`, 0.945202`, 0.925117`, 16.5242`, 16.6329`, 4.172279`, 
              42.777521`, 42.696919`, \(-3.093784`\), 52.619264`, 52.647893`, 
              1.098866`, 65.074267`, 65.150756`, 2.935931`, 37.129058`, 
              37.18493`, 2.144547`, 65.892262`, 65.954005`, 2.36989`, 
              88.823959`, 89.060671`, 9.085847`, 90.331175`, 
              90.131189`, \(-7.676166`\), 16.5242`, 42.777521`, 52.619264`, 
              65.074267`, 88.823959`, 111.710551`, 132.994017`, 149.630532`, 
              37.129058`, 65.892262`, 90.331175`, 95.784179`, 66.468712`, 
              96.350844`, 112.468392`, 116.252323`}, {2, 21, 
              14.11824302`, \(-1.177726024`\), 5.16781565`, 0.04`, 
              1, \(-0.1173903323`\), \(-0.2`\), 
              18406.17968`, \(-0.4543397236`\), \(-0.8548474274`\), 2, 4, 5, 
              6, 0.310492`, \(-0.472712`\), 0.208381`, 0.199934`, 
              0.09889`, \(-0.106318`\), 0.76428`, 0.637699`, 0.86786`, 
              0.83344`, 1.205706`, 1.234068`, 16.5242`, 16.567667`, 
              1.115922`, 27.045959`, 27.012504`, \(-0.858884`\), 30.103479`, 
              30.114859`, 0.292179`, 41.991775`, 41.996769`, 0.128187`, 
              21.576037`, 21.596318`, 0.520684`, 42.030601`, 42.057181`, 
              0.682384`, 51.256814`, 51.261515`, 0.120671`, 53.490834`, 
              53.502809`, 0.307431`, 16.5242`, 27.045959`, 30.103479`, 
              46.904036`, 53.490834`, 66.328651`, 82.474475`, 87.149659`, 
              21.576037`, 42.030601`, 51.256814`, 56.587406`, 41.991775`, 
              61.654702`, 67.019296`, 69.280247`}, {5, 27, 
              9.66473887`, \(-1.020077071`\), 6.113373022`, 0.04`, 
              1, \(-0.1680884761`\), \(-0.15`\), 
              1100.422911`, \(-0.8637152484`\), \(-1.022651035`\), 2, 4, 5, 
              6, 0.318617`, \(-0.498804`\), 0.303076`, 0.168243`, 
              0.149457`, \(-0.143939`\), 0.759642`, 0.688313`, 0.832781`, 
              0.832596`, 1.147681`, 1.147873`, 16.5242`, 16.558565`, 
              1.070981`, 27.326753`, 27.298121`, \(-0.892342`\), 35.23701`, 
              35.250745`, 0.428026`, 46.912365`, 46.916141`, 0.117665`, 
              25.610389`, 25.631351`, 0.653308`, 48.418842`, 48.444572`, 
              0.801872`, 60.340875`, 60.332101`, \(-0.273447`\), 61.365715`, 
              61.383539`, 0.555494`, 16.5242`, 27.326753`, 35.23701`, 
              51.003415`, 61.365715`, 76.482266`, 94.83606`, 97.737333`, 
              25.610389`, 48.418842`, 60.340875`, 63.480049`, 46.912365`, 
              64.872853`, 77.269623`, 78.502784`}, {5, 28, 
              17.07971488`, \(-1.131129524`\), 6.367950886`, 0.04`, 
              1, \(-0.1794142866`\), \(-0.1`\), 
              4.130146791`, \(-0.9592043803`\), \(-1.151352442`\), 2, 4, 5, 
              6, 0.344582`, \(-0.482907`\), 0.289598`, 0.16118`, 
              0.158014`, \(-0.149488`\), 0.761569`, 0.720522`, 0.812569`, 
              0.849599`, 1.111849`, 1.140288`, 16.5242`, 16.545968`, 
              0.764252`, 27.751092`, 27.789312`, 1.341835`, 28.892541`, 
              28.853803`, \(-1.360044`\), 48.629695`, 48.65077`, 0.739908`, 
              27.104618`, 27.120058`, 0.542073`, 52.383098`, 52.402984`, 
              0.698175`, 63.101886`, 63.094502`, \(-0.259239`\), 64.891749`, 
              64.874175`, \(-0.617001`\), 16.5242`, 27.751092`, 28.892541`, 
              48.629695`, 64.891749`, 80.658455`, 99.057282`, 102.95006`, 
              27.104618`, 52.383098`, 63.101886`, 65.810479`, 49.797856`, 
              68.131058`, 79.778933`, 82.529174`}, {2, 8, 
              133.692861`, \(-1.377801116`\), 4.885511546`, 0.04`, 
              1, \(-0.171047`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-1.16519`\), 2, 4, 5, 6, 
              0.375675`, \(-0.462027`\), 0.24993`, 0.162371`, 
              0.157514`, \(-0.151828`\), 0.766685`, 0.593021`, 0.816642`, 
              0.751978`, 1.116476`, 1.077775`, 16.5242`, 16.528681`, 
              0.131588`, 23.11546`, 23.112808`, \(-0.0779`\), 28.256142`, 
              28.265109`, 0.263337`, 40.228924`, 40.229331`, 0.011953`, 
              21.178452`, 21.185391`, 0.203778`, 42.724644`, 42.734845`, 
              0.299563`, 48.927019`, 48.923021`, \(-0.117391`\), 51.46913`, 
              51.328719`, \(-4.123272`\), 16.5242`, 23.11546`, 28.256142`, 
              45.515714`, 51.558356`, 63.719815`, 78.526832`, 81.052114`, 
              21.178452`, 42.724644`, 48.927019`, 51.46913`, 40.228924`, 
              55.337259`, 62.039821`, 65.228738`}, {2, 8, 
              143.3317848`, \(-1.495439348`\), 4.74795212`, 0.04`, 
              1, \(-0.171047`\), 0.0001`, 50, \(-0.97786`\), \(-1.16519`\), 
              2, 4, 5, 6, 0.398647`, \(-0.457455`\), 0.243855`, 0.163866`, 
              0.155091`, \(-0.150579`\), 0.767726`, 0.596097`, 0.818108`, 
              0.756154`, 1.118696`, 1.084572`, 16.5242`, 16.52467`, 
              0.014226`, 23.392256`, 23.393109`, 0.025853`, 27.524824`, 
              27.533302`, 0.256734`, 39.950264`, 39.950276`, 0.000361`, 
              20.776644`, 20.779585`, 0.089067`, 42.943208`, 42.949233`, 
              0.182475`, 47.688648`, 47.683122`, \(-0.167351`\), 50.048847`, 
              49.992564`, \(-1.704451`\), 16.5242`, 23.392256`, 27.524824`, 
              44.870129`, 50.911615`, 62.739082`, 77.14873`, 80.229473`, 
              20.776644`, 42.943208`, 47.688648`, 50.048847`, 39.950264`, 
              54.633263`, 60.318302`, 64.281137`}, {5, 29, 
              134.9005236`, \(-1.546462537`\), 5.157225922`, 0.04`, 
              1, \(-0.1731222981`\), 0.05`, 
              5.831311911`, \(-0.9565465436`\), \(-1.168619602`\), 2, 4, 5, 
              6, 0.417214`, \(-0.454941`\), 0.237213`, 0.166308`, 
              0.150659`, \(-0.146895`\), 0.766502`, 0.660562`, 0.816166`, 
              0.808053`, 1.116203`, 1.125399`, 16.5242`, 16.525877`, 
              0.057742`, 25.687646`, 25.695157`, 0.258575`, 26.402267`, 
              26.401354`, \(-0.031435`\), 43.790397`, 
              43.790347`, \(-0.001748`\), 22.670053`, 
              22.669558`, \(-0.017044`\), 47.657531`, 47.660788`, 0.112147`, 
              51.68693`, 51.678514`, \(-0.28975`\), 54.010992`, 
              53.96388`, \(-1.621882`\), 16.5242`, 26.402267`, 25.687646`, 
              44.01526`, 55.718687`, 68.534751`, 83.987081`, 87.499478`, 
              22.670053`, 47.657531`, 51.68693`, 54.010992`, 43.790397`, 
              59.240063`, 65.349681`, 70.315689`}, {5, 29, 
              123.0774491`, \(-1.585869673`\), 5.252333357`, 0.04`, 
              1, \(-0.1736052746`\), 0.1`, 
              8.713582933`, \(-0.9480764478`\), \(-1.148922877`\), 2, 4, 5, 
              6, 0.433044`, \(-0.450053`\), 0.218661`, 0.168253`, 
              0.146381`, \(-0.142736`\), 0.763984`, 0.686626`, 0.816733`, 
              0.830639`, 1.118752`, 1.145598`, 16.5242`, 16.528469`, 
              0.155357`, 27.740604`, 27.745034`, 0.16121`, 27.812847`, 
              27.814406`, 0.056735`, 44.80971`, 44.809903`, 0.00702`, 
              23.119208`, 23.115207`, \(-0.145605`\), 49.320021`, 49.320799`, 
              0.028318`, 52.311858`, 52.300828`, \(-0.401385`\), 54.624953`, 
              54.583291`, \(-1.516187`\), 16.5242`, 27.740604`, 27.812847`, 
              46.675631`, 56.986302`, 69.982132`, 85.405479`, 88.915646`, 
              23.119208`, 49.320021`, 52.311858`, 54.624953`, 44.80971`, 
              59.761718`, 66.621345`, 71.928435`}, {5, 29, 
              108.8088802`, \(-1.615307184`\), 5.500023748`, 0.04`, 
              1, \(-0.1734840023`\), 0.15`, 
              15.83531094`, \(-0.9617982406`\), \(-1.09977098`\), 2, 4, 5, 6, 
              0.446002`, \(-0.441914`\), 0.186329`, 0.168395`, 
              0.143996`, \(-0.139308`\), 0.759985`, 0.7246`, 0.820707`, 
              0.860847`, 1.128006`, 1.16629`, 16.5242`, 16.531786`, 
              0.297723`, 29.595434`, 29.581587`, \(-0.543471`\), 30.546287`, 
              30.565735`, 0.763305`, 46.886189`, 46.886807`, 0.024234`, 
              24.164723`, 24.157077`, \(-0.300122`\), 52.17527`, 
              52.174403`, \(-0.034019`\), 54.13268`, 
              54.118325`, \(-0.563421`\), 56.765003`, 
              56.72619`, \(-1.523355`\), 16.5242`, 29.595434`, 30.546287`, 
              48.828263`, 59.68424`, 73.202097`, 88.533182`, 93.204625`, 
              24.164723`, 52.17527`, 54.13268`, 56.765003`, 46.886189`, 
              61.530435`, 69.847327`, 75.391023`}, {5, 28, 
              48016.35381`, \(-0.25`\), \(-8.269553441`\), 0.0625`, 
              1, \(-0.2255775482`\), \(-0.35`\), 
              9.155020368`, \(-1.842154314`\), \(-1.824110277`\), 2, 4, 5, 6, 
              0.177851`, \(-1.802034`\), 2.4496`, 0.089272`, 0.075614`, 
              0.04324`, 0.750803`, 0.399345`, 0.68917`, 0.468754`, 0.903719`, 
              0.626917`, 16.5242`, 16.365751`, 
              3.728605`, \(-23.492426`\), \(-23.381224`\), \(-2.616808`\), \
\(-28.461645`\), \(-34.739898`\), 
              147.73936`, \(-35.864352`\), \(-34.940985`\), \(-21.728579`\), \
\(-30.143031`\), \(-30.240683`\), 
              2.297943`, \(-40.571141`\), \(-40.641615`\), 
              1.658392`, \(-65.767999`\), \(-66.512453`\), 
              17.518433`, \(-73.36096`\), \(-73.37053`\), 0.225203`, 
              16.5242`, \(-23.492426`\), \(-28.461645`\), \(-35.864352`\), \
\(-65.767999`\), \(-86.746027`\), \(-74.761256`\), \(-109.28495`\), \
\(-30.143031`\), \(-40.571141`\), \(-73.36096`\), \(-76.29879`\), \
\(-41.118243`\), \(-58.962598`\), \(-67.13166`\), \(-73.379746`\)}, {5, 28, 
              55897.63595`, \(-0.25`\), \(-14.20477519`\), 0.0625`, 
              1, \(-0.2421492665`\), \(-0.3`\), 
              2.337356683`, \(-2.936692922`\), \(-4.968516806`\), 2, 4, 5, 6, 
              0.211721`, \(-1.485821`\), 2.003645`, \(-0.035126`\), 
              0.170353`, \(-0.079276`\), 0.757707`, \(-0.07136`\), 
              0.098095`, \(-0.08678`\), \(-0.184918`\), \(-0.128338`\), 
              16.5242`, 16.300508`, 
              10.76389`, \(-30.285455`\), \(-30.457619`\), 
              8.28441`, \(-39.685769`\), \(-39.510626`\), \(-8.427751`\), \
\(-76.856392`\), \(-76.875526`\), 0.92071`, \(-54.717919`\), \(-54.850329`\), 
              6.37146`, \(-77.340271`\), \(-77.451444`\), 
              5.34955`, \(-118.271885`\), \(-122.146226`\), 
              186.430139`, \(-127.834412`\), \(-129.604067`\), 85.154368`, 
              16.5242`, \(-39.685769`\), \(-30.285455`\), \(-78.72502`\), \
\(-118.271885`\), \(-154.443511`\), \(-127.834412`\), \(-196.357887`\), \
\(-54.717919`\), \(-77.340271`\), \(-129.64023`\), \(-133.442361`\), \
\(-76.856392`\), \(-96.539697`\), \(-123.400048`\), \(-153.326204`\)}, {5, 
              26, 146.9391121`, \(-0.8376934851`\), 6.372253145`, 0.0625`, 
              1, \(-0.1786432774`\), \(-0.25`\), 
              13588.46819`, \(-1.075761253`\), \(-1.281173567`\), 2, 4, 5, 6, 
              0.316051`, \(-0.534861`\), 0.289485`, 0.162775`, 
              0.142739`, \(-0.128538`\), 0.790144`, 0.689981`, 0.830441`, 
              0.824392`, 1.129156`, 1.123053`, 16.5242`, 16.590348`, 
              2.131511`, 33.165887`, 33.113612`, \(-1.684489`\), 38.276797`, 
              38.291876`, 0.485917`, 51.531565`, 51.539984`, 0.27131`, 
              27.661643`, 27.691389`, 0.958488`, 51.644039`, 51.681964`, 
              1.222084`, 64.621282`, 64.625755`, 0.144124`, 67.013352`, 
              67.024192`, 0.349325`, 16.5242`, 33.165887`, 38.276797`, 
              55.191122`, 67.013352`, 83.078014`, 97.880778`, 105.914018`, 
              27.661643`, 51.644039`, 64.621282`, 70.585315`, 51.531565`, 
              76.751021`, 78.182823`, 91.417507`}, {5, 26, 
              14.24640457`, \(-1.045205712`\), 5.111963048`, 0.0625`, 
              1, \(-0.1799228404`\), \(-0.2`\), 
              32769.27127`, \(-0.5130855073`\), \(-0.9313544743`\), 2, 4, 5, 
              6, 0.350059`, \(-0.479593`\), 0.20093`, 0.203635`, 
              0.087379`, \(-0.092345`\), 0.796583`, 0.672747`, 0.897489`, 
              0.862909`, 1.241461`, 1.261538`, 16.5242`, 16.562993`, 
              1.110716`, 27.16951`, 27.137632`, \(-0.912718`\), 31.126836`, 
              31.137219`, 0.297286`, 43.430525`, 43.434482`, 0.113296`, 
              22.749175`, 22.767252`, 0.517558`, 44.179604`, 44.204199`, 
              0.704199`, 52.448878`, 52.452884`, 0.114717`, 55.658682`, 
              55.666398`, 0.22091`, 16.5242`, 27.16951`, 31.126836`, 
              48.715121`, 55.658682`, 68.598767`, 81.361389`, 90.055952`, 
              22.749175`, 44.179604`, 52.448878`, 57.613658`, 43.430525`, 
              63.510901`, 65.61779`, 73.987377`}, {5, 27, 
              8.38891486`, \(-0.8871839557`\), 6.403429164`, 0.0625`, 
              1, \(-0.226739639`\), \(-0.15`\), 
              1165.8821`, \(-1.10290996`\), \(-1.234796605`\), 2, 4, 5, 6, 
              0.356781`, \(-0.51261`\), 0.310333`, 0.160641`, 
              0.1452`, \(-0.13015`\), 0.790144`, 0.725597`, 0.838364`, 
              0.850748`, 1.142911`, 1.135788`, 16.5242`, 16.552914`, 
              1.049616`, 29.274921`, 29.250679`, \(-0.886116`\), 38.44733`, 
              38.459814`, 0.456358`, 51.244115`, 51.247222`, 0.113562`, 
              28.430207`, 28.449823`, 0.717052`, 53.644543`, 53.670182`, 
              0.937206`, 65.121879`, 65.115976`, \(-0.215771`\), 67.387846`, 
              67.395921`, 0.295189`, 16.5242`, 29.274921`, 38.44733`, 
              55.387848`, 67.387846`, 83.521423`, 98.138201`, 110.551079`, 
              28.430207`, 53.644543`, 65.121879`, 68.079658`, 51.244115`, 
              70.644015`, 82.04139`, 86.724721`}, {5, 28, 
              22.01087351`, \(-0.9981535129`\), 6.786255705`, 0.0625`, 
              1, \(-0.2332237241`\), \(-0.1`\), 
              4.043917004`, \(-1.054286827`\), \(-1.156019029`\), 2, 4, 5, 6, 
              0.383912`, \(-0.48567`\), 0.272219`, 0.165624`, 
              0.141069`, \(-0.13021`\), 0.792426`, 0.80652`, 0.854647`, 
              0.924663`, 1.169973`, 1.203164`, 16.5242`, 16.542698`, 
              0.771096`, 30.836934`, 30.851907`, 0.624146`, 31.725024`, 
              31.713189`, \(-0.49337`\), 52.47488`, 52.494726`, 0.827282`, 
              30.56526`, 30.579791`, 0.605725`, 59.244269`, 59.264854`, 
              0.858102`, 69.317562`, 69.310663`, \(-0.287606`\), 72.160274`, 
              72.112835`, \(-1.977483`\), 16.5242`, 31.725024`, 30.836934`, 
              52.47488`, 72.789014`, 89.870591`, 108.486632`, 115.63409`, 
              30.56526`, 59.244269`, 69.317562`, 72.160274`, 55.71512`, 
              75.610036`, 88.432013`, 92.353129`}, {2, 8, 
              107.6760806`, \(-1.178333352`\), 5.169900644`, 0.0625`, 
              1, \(-0.226768`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-1.25431`\), 2, 4, 5, 6, 
              0.41379`, \(-0.459738`\), 0.223373`, 0.173751`, 
              0.1327`, \(-0.127713`\), 0.796752`, 0.672128`, 0.843398`, 
              0.824895`, 1.148299`, 1.154805`, 16.5242`, 16.527451`, 
              0.111283`, 25.847259`, 25.845665`, \(-0.054546`\), 31.130464`, 
              31.138752`, 0.283676`, 44.398068`, 44.398404`, 0.011502`, 
              23.650473`, 23.657047`, 0.225017`, 47.644008`, 47.654938`, 
              0.374128`, 53.289182`, 53.284569`, \(-0.157877`\), 55.853983`, 
              55.826098`, \(-0.954453`\), 16.5242`, 25.847259`, 31.130464`, 
              48.93009`, 57.203091`, 70.231596`, 85.773419`, 89.277244`, 
              23.650473`, 47.644008`, 53.289182`, 55.853983`, 44.398068`, 
              60.294664`, 68.650839`, 72.055388`}, {5, 29, 
              119.9795826`, \(-1.255740129`\), 5.580719756`, 0.0625`, 
              1, \(-0.228557424`\), 0, 
              5.229200543`, \(-0.9660629043`\), \(-1.239513346`\), 2, 4, 5, 
              6, 0.435863`, \(-0.451358`\), 0.210228`, 0.175194`, 
              0.130388`, \(-0.126284`\), 0.797159`, 0.741594`, 0.84655`, 
              0.882075`, 1.153699`, 1.198158`, 16.5242`, 16.525639`, 
              0.056002`, 27.912384`, 27.920633`, 0.321041`, 29.133778`, 
              29.132969`, \(-0.031472`\), 46.741363`, 46.74601`, 0.180863`, 
              25.716285`, 25.71941`, 0.12162`, 52.940957`, 52.948636`, 
              0.298853`, 57.592571`, 57.585695`, \(-0.267622`\), 60.13633`, 
              60.108605`, \(-1.079033`\), 16.5242`, 29.133778`, 27.912384`, 
              46.741363`, 62.560293`, 76.587299`, 92.470363`, 98.287586`, 
              25.716285`, 52.940957`, 57.592571`, 60.13633`, 48.74479`, 
              65.61683`, 74.265454`, 78.581723`}, {5, 27, 
              12402.52809`, \(-0.25`\), \(-18.3792656`\), 0.09`, 
              1, \(-0.2698105218`\), \(-0.35`\), 
              709.0213239`, \(-3.138170834`\), \(-5.518453665`\), 2, 4, 5, 6, 
              0.242504`, \(-1.415823`\), 1.812777`, \(-0.029748`\), 
              0.164028`, \(-0.083844`\), 0.795032`, \(-0.114482`\), 
              0.068964`, \(-0.12423`\), \(-0.257445`\), \(-0.15168`\), 
              16.5242`, 16.212245`, 
              22.246384`, \(-65.220779`\), \(-64.976409`\), \(-17.426725`\), \
\(-87.824872`\), \(-88.037836`\), 
              15.187028`, \(-104.75315`\), \(-104.829177`\), 
              5.421738`, \(-76.549451`\), \(-76.704329`\), 
              11.044811`, \(-113.451026`\), \(-113.609477`\), 
              11.299621`, \(-168.025452`\), \(-168.402071`\), 
              26.857796`, \(-181.349873`\), \(-181.306394`\), \(-3.100609`\), 
              16.5242`, \(-65.220779`\), \(-87.824872`\), \(-104.75315`\), \
\(-168.025452`\), \(-216.601979`\), \(-216.033955`\), \(-270.379297`\), \
\(-76.549451`\), \(-113.451026`\), \(-181.349873`\), \(-182.228174`\), \
\(-114.345995`\), \(-174.135144`\), \(-166.966586`\), \(-189.523542`\)}, {5, 
              27, 12615.24204`, \(-0.25`\), \(-18.5904931`\), 0.09`, 
              1, \(-0.2973689208`\), \(-0.3`\), 
              4316.303361`, \(-3.123355698`\), \(-5.520635387`\), 2, 4, 5, 6, 
              0.264424`, \(-1.335217`\), 1.727372`, \(-0.028317`\), 
              0.163057`, \(-0.082323`\), 0.795217`, \(-0.116257`\), 
              0.068809`, \(-0.125081`\), \(-0.25791`\), \(-0.150068`\), 
              16.5242`, 16.278998`, 
              19.285715`, \(-56.587591`\), \(-56.375222`\), \(-16.703337`\), \
\(-88.816888`\), \(-88.983598`\), 
              13.112178`, \(-106.068713`\), \(-106.1335`\), 
              5.095621`, \(-78.713078`\), \(-78.849452`\), 
              10.726137`, \(-116.986138`\), \(-117.126495`\), 
              11.039437`, \(-169.677999`\), \(-170.138387`\), 
              36.21066`, \(-179.64892`\), \(-179.566733`\), \(-6.464209`\), 
              16.5242`, \(-56.587591`\), \(-88.816888`\), \(-106.068713`\), \
\(-169.677999`\), \(-218.915986`\), \(-202.418474`\), \(-272.919206`\), \
\(-78.713078`\), \(-116.986138`\), \(-179.64892`\), \(-183.910265`\), \
\(-114.67709`\), \(-156.219028`\), \(-165.793928`\), \(-202.855266`\)}, {5, 
              26, 53.77056345`, \(-0.8647974204`\), 5.451654867`, 0.09`, 
              1, \(-0.2295639772`\), \(-0.25`\), 
              10920.94533`, \(-0.8388948215`\), \(-1.146225982`\), 2, 4, 5, 
              6, 0.368344`, \(-0.505249`\), 0.19845`, 0.190994`, 
              0.101123`, \(-0.098353`\), 0.826895`, 0.703334`, 0.902536`, 
              0.869451`, 1.236174`, 1.229842`, 16.5242`, 16.574729`, 
              1.623475`, 30.507684`, 30.465919`, \(-1.341897`\), 34.478153`, 
              34.488991`, 0.348226`, 47.927759`, 47.933127`, 0.172479`, 
              25.422696`, 25.444003`, 0.684571`, 48.634401`, 48.664478`, 
              0.96634`, 57.447682`, 57.453587`, 0.189715`, 61.737441`, 
              61.73557`, \(-0.060112`\), 16.5242`, 30.507684`, 34.478153`, 
              53.037593`, 61.737441`, 75.77043`, 91.879702`, 95.28453`, 
              25.422696`, 48.634401`, 57.447682`, 63.645247`, 47.927759`, 
              71.322321`, 70.990572`, 80.605757`}, {5, 26, 
              13.37073923`, \(-0.9210682031`\), 5.222200242`, 0.09`, 
              1, \(-0.2467357992`\), \(-0.2`\), 
              10575.96488`, \(-0.6887349238`\), \(-1.081276325`\), 2, 4, 5, 
              6, 0.390868`, \(-0.48455`\), 0.187425`, 0.200474`, 
              0.087495`, \(-0.088457`\), 0.828138`, 0.714926`, 0.914612`, 
              0.890354`, 1.256281`, 1.269822`, 16.5242`, 16.557883`, 
              1.100057`, 28.214131`, 28.185643`, \(-0.930416`\), 33.117477`, 
              33.126804`, 0.304593`, 46.221274`, 46.224516`, 0.105861`, 
              24.631156`, 24.647728`, 0.541198`, 47.735123`, 47.759231`, 
              0.787336`, 55.231709`, 55.234785`, 0.10045`, 59.590926`, 
              59.591643`, 0.023411`, 16.5242`, 28.214131`, 33.117477`, 
              51.665341`, 59.590926`, 73.031707`, 87.277212`, 91.873818`, 
              24.631156`, 47.735123`, 55.231709`, 60.285155`, 46.221274`, 
              67.482246`, 67.417841`, 79.730859`}, {3, 23, 
              11.12218907`, \(-0.7951982338`\), 6.715174452`, 0.09`, 
              1, \(-0.2869079816`\), \(-0.15`\), 
              206.3317144`, \(-1.352987104`\), \(-1.457828547`\), 2, 4, 5, 6, 
              0.398569`, \(-0.514531`\), 0.287154`, 0.153137`, 
              0.139051`, \(-0.116254`\), 0.82162`, 0.760845`, 0.844827`, 
              0.869121`, 1.139121`, 1.125187`, 16.5242`, 16.547336`, 
              0.990754`, 32.111604`, 32.09225`, \(-0.828795`\), 41.878476`, 
              41.889652`, 0.478604`, 56.412827`, 56.415357`, 0.108339`, 
              31.535102`, 31.553169`, 0.773703`, 59.776261`, 59.802554`, 
              1.12595`, 70.337724`, 70.333126`, \(-0.19689`\), 73.336316`, 
              73.315359`, \(-0.897467`\), 16.5242`, 32.111604`, 41.878476`, 
              60.649301`, 74.350206`, 91.552729`, 104.408121`, 115.475191`, 
              31.535102`, 59.776261`, 70.337724`, 73.336316`, 56.412827`, 
              78.760712`, 82.835548`, 99.856637`}, {5, 28, 
              19.55403506`, \(-0.8924756879`\), 7.15789053`, 0.09`, 
              1, \(-0.2913712567`\), \(-0.1`\), 
              4.491644115`, \(-1.279940766`\), \(-1.151076451`\), 2, 4, 5, 6, 
              0.425478`, \(-0.480342`\), 0.233652`, 0.159962`, 
              0.134805`, \(-0.11733`\), 0.823778`, 0.856537`, 0.898043`, 
              0.959838`, 1.230308`, 1.211956`, 16.5242`, 16.539127`, 
              0.727367`, 34.326243`, 34.341583`, 0.74752`, 35.613633`, 
              35.604137`, \(-0.462711`\), 57.270291`, 57.288476`, 0.886109`, 
              34.026406`, 34.039863`, 0.655731`, 66.307059`, 66.329003`, 
              1.069297`, 75.278135`, 75.271814`, \(-0.307987`\), 78.275309`, 
              78.252866`, \(-1.093593`\), 16.5242`, 35.613633`, 34.326243`, 
              57.270291`, 80.801186`, 99.071445`, 113.256958`, 125.262173`, 
              34.026406`, 66.307059`, 75.278135`, 78.275309`, 61.722289`, 
              85.065819`, 90.866027`, 106.497033`}, {2, 8, 
              103.2194112`, \(-1.00893519`\), 5.708562315`, 0.09`, 
              1, \(-0.289415`\), \(-0.05`\), 
              50, \(-0.97786`\), \(-1.33022`\), 2, 4, 5, 6, 
              0.452522`, \(-0.452763`\), 0.184235`, 0.18318`, 
              0.110704`, \(-0.105159`\), 0.826488`, 0.786756`, 0.873639`, 
              0.927124`, 1.186661`, 1.248882`, 16.5242`, 16.52669`, 
              0.102931`, 29.952384`, 29.951422`, \(-0.039768`\), 35.653686`, 
              35.661748`, 0.333212`, 50.844164`, 50.844452`, 0.011914`, 
              27.451992`, 27.458501`, 0.26901`, 55.219298`, 55.232501`, 
              0.545717`, 60.33516`, 60.329255`, \(-0.244043`\), 62.910492`, 
              62.892399`, \(-0.747815`\), 16.5242`, 29.952384`, 35.653686`, 
              54.586157`, 65.953279`, 80.47949`, 93.101919`, 101.114954`, 
              27.451992`, 55.219298`, 60.33516`, 62.910492`, 50.844164`, 
              69.72463`, 74.903458`, 85.121487`}, {5, 29, 
              107.9067633`, \(-1.101214258`\), 6.503132855`, 0.09`, 
              1, \(-0.2872526626`\), 0, 
              3.68673475`, \(-1.099136369`\), \(-1.167157772`\), 2, 4, 5, 6, 
              0.476139`, \(-0.43577`\), 0.151413`, 0.175838`, 
              0.119844`, \(-0.112221`\), 0.827905`, 0.883881`, 0.900663`, 
              1.001233`, 1.232736`, 1.284481`, 16.5242`, 16.526325`, 
              0.105259`, 30.240697`, 30.246256`, 0.275395`, 35.98464`, 
              35.984675`, 0.00173`, 51.782832`, 51.788272`, 0.26953`, 
              31.520234`, 31.523461`, 0.15986`, 65.222535`, 65.234608`, 
              0.598119`, 68.80043`, 68.790811`, \(-0.47651`\), 71.913739`, 
              71.893124`, \(-1.021335`\), 16.5242`, 35.98464`, 30.240697`, 
              51.782832`, 76.483531`, 92.968692`, 108.470282`, 115.556975`, 
              31.520234`, 65.222535`, 68.80043`, 71.913739`, 59.343023`, 
              81.046348`, 87.263486`, 97.165007`}, {2, 8, 
              129.4649014`, \(-1.156119113`\), 5.507808393`, 0.09`, 
              1, \(-0.289415`\), 0.05`, 50, \(-0.97786`\), \(-1.33022`\), 2, 
              4, 5, 6, 0.495585`, \(-0.424078`\), 0.125404`, 0.185037`, 
              0.107879`, \(-0.10381`\), 0.827396`, 0.798562`, 0.873413`, 
              0.939037`, 1.186023`, 1.262244`, 16.5242`, 16.524922`, 
              0.031544`, 31.62797`, 31.627819`, \(-0.006591`\), 34.46762`, 
              34.475783`, 0.356497`, 50.901985`, 50.901981`, \(-0.000143`\), 
              26.794614`, 26.794172`, \(-0.019282`\), 56.690616`, 56.698759`, 
              0.35561`, 58.046864`, 58.036214`, \(-0.465106`\), 60.659058`, 
              60.641806`, \(-0.753443`\), 16.5242`, 31.62797`, 34.46762`, 
              53.465797`, 65.435567`, 79.315606`, 92.86323`, 97.582492`, 
              26.794614`, 56.690616`, 58.046864`, 60.659058`, 50.901985`, 
              69.145235`, 74.582958`, 82.363294`}, {5, 28, 
              121.2181126`, \(-1.189671925`\), 5.967477746`, 0.09`, 
              1, \(-0.2882941828`\), 0.1`, 
              10.95425265`, \(-1.12136086`\), \(-1.105982873`\), 2, 4, 5, 6, 
              0.511917`, \(-0.412478`\), 0.094242`, 0.176153`, 
              0.117795`, \(-0.110092`\), 0.825439`, 0.844117`, 0.907857`, 
              0.967094`, 1.246821`, 1.259243`, 16.5242`, 16.527072`, 
              0.140357`, 34.984425`, 34.992168`, 0.378434`, 35.264491`, 
              35.26308`, \(-0.068966`\), 55.448477`, 55.448678`, 0.009826`, 
              29.048522`, 29.044847`, \(-0.179654`\), 62.101011`, 62.103311`, 
              0.112431`, 62.873641`, 62.864727`, \(-0.435653`\), 65.210319`, 
              65.191675`, \(-0.911263`\), 16.5242`, 35.264491`, 34.984425`, 
              55.515443`, 71.259915`, 86.179115`, 101.040189`, 105.084909`, 
              29.048522`, 62.101011`, 62.873641`, 65.210319`, 55.448477`, 
              74.92797`, 80.827725`, 89.2941`}, {2, 7, 
              127.1962156`, \(-1.22527098`\), 6.032865387`, 0.09`, 
              1, \(-0.289415`\), 0.15`, 50, \(-0.97786`\), \(-1.33022`\), 2, 
              4, 5, 6, 0.526354`, \(-0.398198`\), 0.05113`, 0.186611`, 
              0.103609`, \(-0.099207`\), 0.822858`, 0.897714`, 0.86803`, 
              1.024546`, 1.179514`, 1.328725`, 16.5242`, 16.529266`, 
              0.257387`, 36.668483`, 36.676338`, 0.399091`, 37.413531`, 
              37.411845`, \(-0.085628`\), 56.274263`, 56.274803`, 0.027446`, 
              29.335707`, 29.32896`, \(-0.342771`\), 62.813435`, 
              62.809949`, \(-0.177143`\), 63.824968`, 
              63.81948`, \(-0.278845`\), 65.562994`, 
              65.543315`, \(-0.999849`\), 16.5242`, 36.668483`, 37.413531`, 
              56.910533`, 72.314668`, 87.262363`, 102.544825`, 105.581797`, 
              29.335707`, 62.813435`, 63.824968`, 65.562994`, 56.274263`, 
              75.760414`, 81.568574`, 90.437182`}, {5, 28, 
              12561.1507`, \(-0.25`\), \(-18.38439001`\), 0.1225`, 
              1, \(-0.341004126`\), \(-0.35`\), 
              1732.74064`, \(-2.929672657`\), \(-2.876035286`\), 2, 4, 5, 6, 
              0.289898`, \(-1.385604`\), 1.725406`, 0.021003`, 
              0.10874`, \(-0.021362`\), 0.829572`, 0.314483`, 0.624839`, 
              0.321077`, 0.748068`, 0.339733`, 16.5242`, 16.249756`, 
              23.402831`, \(-63.757886`\), \(-63.522852`\), \(-20.042268`\), \
\(-105.607547`\), \(-105.816619`\), 
              17.828373`, \(-110.461319`\), \(-110.518347`\), 
              4.863012`, \(-82.569984`\), \(-82.700784`\), 
              11.153816`, \(-124.529677`\), \(-124.679934`\), 
              12.812911`, \(-178.79184`\), \(-178.901664`\), 
              9.365103`, \(-187.214708`\), \(-187.148512`\), \(-5.644769`\), 
              16.5242`, \(-63.757886`\), \(-105.607547`\), \(-110.461319`\), \
\(-178.79184`\), \(-229.091058`\), \(-242.006931`\), \(-284.747415`\), \
\(-82.569984`\), \(-124.529677`\), \(-187.214708`\), \(-189.019142`\), \
\(-122.956267`\), \(-193.572968`\), \(-171.585223`\), \(-196.320282`\)}, {2, 
              12, 14154.86869`, \(-0.25`\), \(-18.6201714`\), 0.1225`, 
              1, \(-0.3637421533`\), \(-0.3`\), 
              42.95493663`, \(-3.068881472`\), \(-6.3398474`\), 2, 4, 5, 6, 
              0.31346`, \(-1.284829`\), 1.604565`, 0.005962`, 
              0.121136`, \(-0.038484`\), 0.829784`, 
              0.179599`, \(-0.004277`\), 0.181463`, \(-0.411499`\), 
              0.186742`, 16.5242`, 16.317246`, 
              19.326774`, \(-57.733374`\), \(-57.548466`\), \(-17.267955`\), \
\(-108.212076`\), \(-108.374254`\), 
              15.145261`, \(-109.967945`\), \(-110.017047`\), 
              4.585547`, \(-84.795466`\), \(-84.909071`\), 
              10.609253`, \(-129.287955`\), \(-129.422687`\), 
              12.582262`, \(-181.630617`\), \(-182.018464`\), 
              36.219885`, \(-186.013454`\), \(-185.681671`\), \
\(-30.984205`\), 
              16.5242`, \(-57.733374`\), \(-108.212076`\), \(-109.967945`\), \
\(-181.630617`\), \(-232.487533`\), \(-232.881346`\), \(-289.1736`\), \
\(-84.795466`\), \(-129.287955`\), \(-186.013454`\), \(-191.195643`\), \
\(-124.490525`\), \(-181.805625`\), \(-169.252508`\), \(-199.772579`\)}, {5, 
              26, 68.17589904`, \(-0.7733067353`\), 5.590025021`, 0.1225`, 
              1, \(-0.302725339`\), \(-0.25`\), 
              8503.135785`, \(-1.01439734`\), \(-1.344801843`\), 2, 4, 5, 6, 
              0.411891`, \(-0.509013`\), 0.177498`, 0.188304`, 
              0.095789`, \(-0.088071`\), 0.859315`, 0.746801`, 0.914214`, 
              0.899054`, 1.241045`, 1.240288`, 16.5242`, 16.568813`, 
              1.643535`, 31.718113`, 31.680107`, \(-1.400112`\), 36.78077`, 
              36.790412`, 0.35521`, 51.254872`, 51.259293`, 0.162873`, 
              27.608939`, 27.628432`, 0.718102`, 52.849877`, 52.880237`, 
              1.118475`, 60.642824`, 60.647211`, 0.161628`, 66.363163`, 
              66.313155`, \(-1.842273`\), 16.5242`, 31.718113`, 36.78077`, 
              56.662247`, 66.363163`, 80.999992`, 99.366999`, 100.768552`, 
              27.608939`, 52.849877`, 60.642824`, 66.836235`, 51.254872`, 
              75.028144`, 75.820995`, 80.137576`}, {5, 26, 
              14.80098622`, \(-0.8215653355`\), 5.421398282`, 0.1225`, 
              1, \(-0.3162169401`\), \(-0.2`\), 
              9324.98502`, \(-0.933494568`\), \(-1.276940505`\), 2, 4, 5, 6, 
              0.434516`, \(-0.482435`\), 0.153995`, 0.193585`, 
              0.089875`, \(-0.084878`\), 0.860161`, 0.757931`, 0.926015`, 
              0.914698`, 1.260941`, 1.265801`, 16.5242`, 16.552402`, 
              1.062954`, 30.162193`, 30.138167`, \(-0.905549`\), 35.743987`, 
              35.75233`, 0.314431`, 50.098642`, 50.101326`, 0.101152`, 
              27.018542`, 27.033824`, 0.575972`, 52.511364`, 52.536864`, 
              0.96112`, 58.940192`, 58.941757`, 0.05899`, 64.129449`, 
              64.102756`, \(-1.006068`\), 16.5242`, 30.162193`, 35.743987`, 
              55.546769`, 64.877448`, 79.039489`, 95.800649`, 98.364746`, 
              27.018542`, 52.511364`, 58.940192`, 64.129449`, 50.098642`, 
              72.403642`, 72.606009`, 78.505828`}, {5, 28, 
              16.54314189`, \(-0.7233186289`\), 7.128318541`, 0.1225`, 
              1, \(-0.350987264`\), \(-0.15`\), 
              143.5308811`, \(-1.513998878`\), \(-1.396700298`\), 2, 4, 5, 6, 
              0.443154`, \(-0.506853`\), 0.238195`, 0.153343`, 
              0.125615`, \(-0.099811`\), 0.853953`, 0.83224`, 0.899458`, 
              0.929471`, 1.218655`, 1.168177`, 16.5242`, 16.542805`, 
              0.940369`, 36.087141`, 36.071908`, \(-0.769926`\), 46.171443`, 
              46.181592`, 0.512948`, 62.991071`, 62.99315`, 0.105082`, 
              35.334618`, 35.351353`, 0.845847`, 67.569326`, 67.598524`, 
              1.475743`, 76.76423`, 76.759816`, \(-0.223106`\), 80.078678`, 
              80.055765`, \(-1.158136`\), 16.5242`, 36.087141`, 46.171443`, 
              67.238809`, 83.12592`, 101.650496`, 118.210182`, 127.040175`, 
              35.334618`, 67.569326`, 76.76423`, 80.078678`, 62.991071`, 
              89.553149`, 88.78625`, 102.95845`}, {5, 28, 
              26.56675494`, \(-0.8420444692`\), 8.044972264`, 0.1225`, 
              1, \(-0.3489924967`\), \(-0.1`\), 
              3.213557622`, \(-1.627757663`\), \(-1.42045325`\), 2, 4, 5, 6, 
              0.472584`, \(-0.457787`\), 0.151703`, 0.145698`, 
              0.136878`, \(-0.112247`\), 0.857554`, 0.922365`, 0.90012`, 
              1.00267`, 1.217803`, 1.208094`, 16.5242`, 16.541977`, 
              1.081396`, 32.586131`, 32.60469`, 1.128939`, 46.514862`, 
              46.504766`, \(-0.614149`\), 63.326115`, 63.343254`, 1.042581`, 
              40.411365`, 40.42436`, 0.790509`, 80.218801`, 80.248333`, 
              1.796431`, 86.987268`, 86.978599`, \(-0.527383`\), 91.437292`, 
              91.41358`, \(-1.44246`\), 16.5242`, 46.514862`, 32.586131`, 
              63.326115`, 96.367465`, 117.16601`, 137.375989`, 146.399279`, 
              40.411365`, 80.218801`, 86.987268`, 91.437292`, 73.824735`, 
              103.820787`, 103.296035`, 122.71217`}, {2, 29, 
              104.1068994`, \(-0.8930165786`\), 6.253248095`, 0.1225`, 
              1, \(-0.3566965438`\), \(-0.05`\), 
              40.94533275`, \(-1.137297406`\), \(-1.225542051`\), 2, 4, 5, 6, 
              0.494742`, \(-0.436442`\), 0.118939`, 0.18194`, 
              0.102429`, \(-0.092574`\), 0.857753`, 0.881628`, 0.931366`, 
              1.004611`, 1.271711`, 1.300117`, 16.5242`, 16.526205`, 
              0.099248`, 34.802351`, 34.80171`, \(-0.031736`\), 40.386606`, 
              40.394488`, 0.390156`, 58.133548`, 58.133789`, 0.011914`, 
              31.592964`, 31.599265`, 0.311914`, 64.277697`, 64.299268`, 
              1.067754`, 67.328385`, 67.316719`, \(-0.577441`\), 70.625946`, 
              70.608953`, \(-0.84116`\), 16.5242`, 34.802351`, 40.386606`, 
              61.257288`, 75.724568`, 91.783943`, 107.456915`, 113.853032`, 
              31.592964`, 64.277697`, 67.328385`, 70.625946`, 58.133548`, 
              80.753049`, 80.639061`, 98.27891`}, {5, 29, 
              122.9300474`, \(-0.9480213713`\), 6.229349433`, 0.1225`, 
              1, \(-0.3559113289`\), 0, 
              52.00704302`, \(-1.297845502`\), \(-1.198646117`\), 2, 4, 5, 6, 
              0.516111`, \(-0.417007`\), 0.082916`, 0.171979`, 
              0.113128`, \(-0.099939`\), 0.857901`, 0.86044`, 0.935867`, 
              0.975226`, 1.279458`, 1.252259`, 16.5242`, 16.524432`, 
              0.011944`, 36.076233`, 36.07657`, 0.017337`, 40.379842`, 
              40.387956`, 0.417424`, 58.774256`, 58.774258`, 0.000069`, 
              31.625755`, 31.628513`, 0.141916`, 65.783171`, 65.797811`, 
              0.753084`, 66.992047`, 66.981644`, \(-0.535155`\), 70.093741`, 
              70.077915`, \(-0.814096`\), 16.5242`, 36.076233`, 40.379842`, 
              61.141097`, 76.33286`, 92.217506`, 108.232615`, 113.267792`, 
              31.625755`, 65.783171`, 66.992047`, 70.093741`, 58.774256`, 
              80.725889`, 81.155198`, 100.160344`}, {5, 29, 
              130.9753058`, \(-1.003196187`\), 6.187149221`, 0.1225`, 
              1, \(-0.3551554098`\), 0.05`, 
              84.90494602`, \(-1.247741163`\), \(-1.141768291`\), 2, 4, 5, 6, 
              0.536029`, \(-0.399187`\), 0.043857`, 0.176305`, 
              0.108571`, \(-0.097556`\), 0.85775`, 0.88567`, 0.944721`, 
              1.001905`, 1.294836`, 1.283608`, 16.5242`, 16.524806`, 
              0.032174`, 37.364102`, 37.362789`, \(-0.069668`\), 40.259375`, 
              40.26868`, 0.493758`, 59.229876`, 59.229881`, 0.000256`, 
              31.52488`, 31.524408`, \(-0.02503`\), 66.238744`, 66.244219`, 
              0.29054`, 67.200046`, 67.193843`, \(-0.329175`\), 69.431553`, 
              69.416681`, \(-0.789175`\), 16.5242`, 37.364102`, 40.259375`, 
              60.912791`, 76.685921`, 92.345082`, 108.809908`, 112.380589`, 
              31.52488`, 66.238744`, 67.200046`, 69.431553`, 59.229876`, 
              80.364465`, 81.824695`, 100.77554`}, {5, 29, 
              125.7686507`, \(-1.02790567`\), 6.674914597`, 0.1225`, 
              1, \(-0.3572659976`\), 0.1`, 
              28.34581964`, \(-1.587438957`\), \(-1.352834807`\), 2, 4, 5, 6, 
              0.552077`, \(-0.383824`\), 0.00532`, 0.153041`, 
              0.130634`, \(-0.111186`\), 0.855673`, 0.867237`, 0.908885`, 
              0.960615`, 1.234413`, 1.192647`, 16.5242`, 16.52656`, 0.13914`, 
              40.521867`, 40.539888`, 1.062528`, 43.395352`, 
              43.384844`, \(-0.619518`\), 64.121873`, 64.122076`, 0.011991`, 
              34.001847`, 33.998104`, \(-0.220713`\), 71.478146`, 71.478341`, 
              0.011517`, 73.14272`, 73.137063`, \(-0.333572`\), 74.144308`, 
              74.129183`, \(-0.891792`\), 16.5242`, 40.521867`, 43.395352`, 
              64.209089`, 83.070844`, 99.795692`, 117.482187`, 120.406905`, 
              34.001847`, 71.478146`, 73.14272`, 74.144308`, 64.121873`, 
              85.998494`, 88.661762`, 108.831354`}, {2, 29, 
              121.0824982`, \(-1.100634184`\), 6.439116217`, 0.1225`, 
              1, \(-0.3525770988`\), 0.15`, 
              22.9401961`, \(-1.283075398`\), \(-1.069869387`\), 2, 4, 5, 6, 
              0.569655`, \(-0.36415`\), \(-0.049578`\), 0.175814`, 
              0.107982`, \(-0.097053`\), 0.855501`, 0.94246`, 0.953468`, 
              1.051095`, 1.311288`, 1.319969`, 16.5242`, 16.52889`, 
              0.275238`, 40.900566`, 40.908918`, 0.490206`, 41.556693`, 
              41.554157`, \(-0.14881`\), 62.576325`, 62.578566`, 0.131496`, 
              32.87796`, 32.87154`, \(-0.376823`\), 69.246581`, 
              69.243803`, \(-0.163035`\), 71.523582`, 
              71.509787`, \(-0.809625`\), 72.14867`, 
              72.141908`, \(-0.396858`\), 16.5242`, 41.556693`, 40.900566`, 
              62.576325`, 81.164875`, 97.183518`, 114.762867`, 117.082112`, 
              32.87796`, 69.246581`, 71.523582`, 72.14867`, 62.953078`, 
              83.354944`, 87.712964`, 105.824619`}, {5, 28, 
              14479.59931`, \(-0.25`\), \(-17.26266149`\), 0.16`, 
              1, \(-0.4158248534`\), \(-0.35`\), 
              2294.29969`, \(-3.091563247`\), \(-2.801249259`\), 2, 4, 5, 6, 
              0.340361`, \(-1.344134`\), 1.609821`, 0.028709`, 
              0.085531`, \(-0.001711`\), 0.864376`, 0.374489`, 0.69292`, 
              0.383269`, 0.851943`, 0.408178`, 16.5242`, 16.305487`, 
              20.560864`, \(-59.850712`\), \(-59.655684`\), \(-18.334277`\), \
\(-108.762261`\), \(-108.803044`\), 
              3.833868`, \(-114.938986`\), \(-115.117703`\), 
              16.800889`, \(-83.059361`\), \(-83.162955`\), 
              9.738794`, \(-127.893396`\), \(-128.030101`\), 
              12.851468`, \(-178.49596`\), \(-178.64028`\), 
              13.567283`, \(-181.022296`\), \(-180.84306`\), \(-16.849716`\), 
              16.5242`, \(-59.850712`\), \(-108.762261`\), \(-114.938986`\), \
\(-178.49596`\), \(-227.053557`\), \(-255.581294`\), \(-281.250333`\), \
\(-83.059361`\), \(-127.893396`\), \(-181.022296`\), \(-183.583002`\), \
\(-123.817304`\), \(-201.305504`\), \(-168.170275`\), \(-192.612565`\)}, {2, 
              19, 13383.91762`, \(-0.25`\), \(-17.11549021`\), 0.16`, 
              1, \(-0.4350292667`\), \(-0.3`\), 
              1856.598739`, \(-3.623514865`\), \(-6.731974642`\), 2, 4, 5, 6, 
              0.364545`, \(-1.236351`\), 1.474218`, \(-0.027893`\), 
              0.132644`, \(-0.069281`\), 0.864526`, \(-0.162303`\), 
              0.000655`, \(-0.17043`\), \(-0.41722`\), \(-0.193601`\), 
              16.5242`, 16.371563`, 
              15.237714`, \(-55.324208`\), \(-55.185239`\), \(-13.873252`\), \
\(-107.869764`\), \(-107.904293`\), 
              3.446952`, \(-114.070621`\), \(-114.195356`\), 
              12.452241`, \(-83.26186`\), \(-83.349302`\), 
              8.72926`, \(-130.172637`\), \(-130.29463`\), 
              12.178561`, \(-176.274035`\), \(-176.123316`\), \
\(-15.046308`\), \(-177.953216`\), \(-178.076385`\), 12.295925`, 
              16.5242`, \(-55.324208`\), \(-107.869764`\), \(-114.070621`\), \
\(-177.953216`\), \(-225.853388`\), \(-244.607185`\), \(-280.560957`\), \
\(-83.26186`\), \(-130.172637`\), \(-176.274035`\), \(-181.887262`\), \
\(-123.186481`\), \(-192.332543`\), \(-159.920686`\), \(-192.92734`\)}, {5, 
              26, 276.4995557`, \(-0.5986513653`\), 7.398915171`, 0.16`, 
              1, \(-0.3935687854`\), \(-0.25`\), 
              5146.234359`, \(-1.805758542`\), \(-1.992012664`\), 2, 4, 5, 6, 
              0.446246`, \(-0.563792`\), 0.255396`, 0.143051`, 
              0.117789`, \(-0.083793`\), 0.887243`, 0.807877`, 0.851146`, 
              0.895002`, 1.118316`, 1.111601`, 16.5242`, 16.571409`, 
              2.493933`, 39.970677`, 39.929736`, \(-2.162824`\), 49.985434`, 
              49.996683`, 0.594244`, 67.980082`, 67.984998`, 0.259713`, 
              38.186797`, 38.211014`, 1.279336`, 71.652245`, 71.697013`, 
              2.36496`, 81.700737`, 81.699902`, \(-0.044137`\), 87.840442`, 
              87.787778`, \(-2.782168`\), 16.5242`, 39.970677`, 49.985434`, 
              73.385387`, 89.681986`, 109.407402`, 133.830366`, 135.041488`, 
              38.186797`, 71.652245`, 81.700737`, 87.840442`, 67.980082`, 
              98.812265`, 99.651173`, 105.746877`}, {5, 27, 
              99.58219097`, \(-0.6414939632`\), 7.255058977`, 0.16`, 
              1, \(-0.4093699771`\), \(-0.2`\), 
              251.9423638`, \(-1.923943149`\), \(-2.027176267`\), 2, 4, 5, 6, 
              0.46958`, \(-0.519727`\), 0.198327`, 0.134365`, 
              0.126686`, \(-0.091967`\), 0.887832`, 0.776197`, 0.846275`, 
              0.857872`, 1.109456`, 1.061054`, 16.5242`, 16.552349`, 
              1.53437`, 39.161038`, 39.137141`, \(-1.30258`\), 48.984059`, 
              48.993855`, 0.533958`, 67.325437`, 67.32841`, 0.162073`, 
              37.728736`, 37.747832`, 1.040956`, 72.276912`, 72.319551`, 
              2.324192`, 79.984531`, 79.977187`, \(-0.400349`\), 85.286027`, 
              85.253424`, \(-1.777197`\), 16.5242`, 39.161038`, 48.984059`, 
              72.342767`, 88.756741`, 107.985535`, 130.283078`, 133.79719`, 
              37.728736`, 72.276912`, 79.984531`, 85.286027`, 67.325437`, 
              96.490745`, 96.868585`, 105.110949`}, {5, 28, 
              41.83467719`, \(-0.7304290833`\), 6.433994653`, 0.16`, 
              1, \(-0.408879132`\), \(-0.15`\), 
              177.5855738`, \(-1.594863741`\), \(-1.731252431`\), 2, 4, 5, 6, 
              0.497582`, \(-0.463273`\), 0.106059`, 0.159886`, 
              0.111634`, \(-0.088787`\), 0.89061`, 0.817714`, 0.896485`, 
              0.922948`, 1.195594`, 1.178135`, 16.5242`, 16.537549`, 
              0.683801`, 36.361988`, 36.351266`, \(-0.549215`\), 43.677617`, 
              43.685501`, 0.403848`, 61.488222`, 61.48965`, 0.073165`, 
              33.835462`, 33.848139`, 0.649372`, 67.340541`, 67.379315`, 
              1.986102`, 70.672267`, 70.658626`, \(-0.698712`\), 76.094002`, 
              76.07075`, \(-1.191068`\), 16.5242`, 36.361988`, 43.677617`, 
              66.136604`, 80.438367`, 97.372477`, 117.277753`, 120.949889`, 
              33.835462`, 67.340541`, 70.672267`, 76.094002`, 61.488222`, 
              86.506173`, 87.981641`, 95.808425`}, {5, 28, 
              82.25169439`, \(-0.7494348184`\), 6.882856752`, 0.16`, 
              1, \(-0.4233846867`\), \(-0.1`\), 
              224.0313489`, \(-1.688879901`\), \(-1.425291303`\), 2, 4, 5, 6, 
              0.517268`, \(-0.442207`\), 0.082121`, 0.153674`, 
              0.116439`, \(-0.091527`\), 0.889552`, 0.858672`, 0.942781`, 
              0.952165`, 1.276333`, 1.184675`, 16.5242`, 16.529985`, 
              0.329526`, 39.230091`, 39.226016`, \(-0.232111`\), 46.642643`, 
              46.650706`, 0.459294`, 65.876564`, 65.877259`, 0.039617`, 
              36.302554`, 36.312368`, 0.559083`, 73.530257`, 73.563439`, 
              1.890153`, 75.106438`, 75.093427`, \(-0.741142`\), 80.149234`, 
              80.12759`, \(-1.232952`\), 16.5242`, 39.230091`, 46.642643`, 
              69.590613`, 86.363284`, 104.339767`, 124.301835`, 129.356067`, 
              36.302554`, 73.530257`, 75.106438`, 80.149234`, 65.876564`, 
              91.592766`, 92.845725`, 103.148356`}, {2, 13, 
              102.9745739`, \(-0.7938975793`\), 7.130128504`, 0.16`, 
              1, \(-0.4280906065`\), \(-0.05`\), 
              26.22058269`, \(-1.698764058`\), \(-1.351972618`\), 2, 4, 5, 6, 
              0.538915`, \(-0.415149`\), 0.037534`, 0.153753`, 
              0.117381`, \(-0.09363`\), 0.889632`, 0.904282`, 0.954204`, 
              0.993112`, 1.29596`, 1.217409`, 16.5242`, 16.525953`, 
              0.107753`, 42.053551`, 42.0521`, \(-0.089215`\), 47.123878`, 
              47.133001`, 0.560856`, 69.085028`, 69.085222`, 0.011921`, 
              37.777839`, 37.784183`, 0.389998`, 77.682825`, 77.701649`, 
              1.157306`, 78.085524`, 78.08127`, \(-0.261504`\), 82.537848`, 
              82.51773`, \(-1.236894`\), 16.5242`, 42.053551`, 47.123878`, 
              71.243897`, 90.40386`, 108.870367`, 129.416269`, 133.972995`, 
              37.777839`, 77.682825`, 78.085524`, 82.537848`, 69.085028`, 
              94.72368`, 96.696159`, 108.328091`}, {2, 29, 
              121.8993584`, \(-0.790147172`\), 8.23436971`, 0.16`, 
              1, \(-0.4356431429`\), 0, 
              47.09707091`, \(-2.107917935`\), \(-1.90879872`\), 2, 4, 5, 6, 
              0.554987`, \(-0.400903`\), 0.015397`, 0.122968`, 
              0.140436`, \(-0.10787`\), 0.886823`, 0.877189`, 0.863917`, 
              0.938727`, 1.141117`, 1.099974`, 16.5242`, 16.524376`, 
              0.012847`, 49.056396`, 49.055319`, \(-0.078758`\), 54.777272`, 
              54.788319`, 0.807716`, 79.275028`, 79.275016`, \(-0.000871`\), 
              43.570364`, 43.57365`, 0.240309`, 89.660709`, 89.672485`, 
              0.861053`, 90.312294`, 90.309997`, \(-0.167945`\), 93.419426`, 
              93.398772`, \(-1.510194`\), 16.5242`, 49.056396`, 54.777272`, 
              79.776576`, 104.23751`, 125.326111`, 147.670893`, 152.743375`, 
              43.570364`, 89.660709`, 90.312294`, 93.419426`, 79.275028`, 
              107.558129`, 109.533375`, 125.240372`}, {5, 29, 
              132.8482608`, \(-0.8743306375`\), 7.300675855`, 0.16`, 
              1, \(-0.4291435457`\), 0.05`, 
              77.89513286`, \(-1.913357296`\), \(-1.69139559`\), 2, 4, 5, 6, 
              0.578107`, \(-0.369892`\), \(-0.052551`\), 0.139674`, 
              0.130236`, \(-0.104229`\), 0.888646`, 0.886782`, 0.90046`, 
              0.963442`, 1.203853`, 1.159762`, 16.5242`, 16.524752`, 
              0.037275`, 46.565768`, 46.558144`, \(-0.514809`\), 48.676135`, 
              48.69218`, 1.083506`, 72.266326`, 72.266337`, 0.000751`, 
              38.886509`, 38.88597`, \(-0.036369`\), 80.471025`, 80.473931`, 
              0.196265`, 82.045016`, 82.04411`, \(-0.061183`\), 83.656848`, 
              83.641164`, \(-1.059148`\), 16.5242`, 46.565768`, 48.676135`, 
              72.732689`, 94.278438`, 112.833464`, 133.31579`, 137.436031`, 
              38.886509`, 80.471025`, 82.045016`, 83.656848`, 72.266326`, 
              96.492407`, 100.482441`, 113.607299`}, {5, 28, 
              124.2148256`, \(-0.871037912`\), 9.0607462`, 0.16`, 
              1, \(-0.4360488643`\), 0.1`, 
              13.30892722`, \(-2.38818393`\), \(-2.167104556`\), 2, 4, 5, 6, 
              0.591927`, \(-0.353184`\), \(-0.092773`\), 0.102123`, 
              0.159134`, \(-0.125741`\), 0.885192`, 0.852999`, 0.820656`, 
              0.898453`, 1.066246`, 1.020566`, 16.5242`, 16.526742`, 
              0.218135`, 57.320608`, 57.330088`, 0.813524`, 57.867793`, 
              57.866794`, \(-0.085716`\), 85.201877`, 85.204764`, 0.247741`, 
              48.10058`, 48.09589`, \(-0.402449`\), 99.141114`, 
              99.140438`, \(-0.058021`\), 101.237046`, 
              101.221639`, \(-1.322101`\), 102.981344`, 
              102.976308`, \(-0.432225`\), 16.5242`, 57.867793`, 57.320608`, 
              85.201877`, 116.763443`, 139.483633`, 163.25825`, 169.019843`, 
              48.10058`, 99.141114`, 101.237046`, 102.981344`, 89.097489`, 
              117.707411`, 123.005738`, 141.093663`}, {5, 29, 
              118.5316522`, \(-0.9684054953`\), 7.898000851`, 0.16`, 
              1, \(-0.4236605891`\), 0.15`, 
              9.085419295`, \(-1.902210155`\), \(-1.398928792`\), 2, 4, 5, 6, 
              0.612813`, \(-0.32564`\), \(-0.164237`\), 0.142891`, 
              0.128048`, \(-0.104485`\), 0.887177`, 0.994606`, 0.944199`, 
              1.065328`, 1.280435`, 1.251104`, 16.5242`, 16.529046`, 
              0.375305`, 48.097381`, 48.102702`, 0.411997`, 53.496504`, 
              53.494079`, \(-0.187723`\), 74.124174`, 74.124675`, 0.038786`, 
              42.177323`, 42.170192`, \(-0.552219`\), 87.229118`, 
              87.226345`, \(-0.214727`\), 89.497459`, 
              89.483394`, \(-1.089174`\), 92.383678`, 
              92.378275`, \(-0.418424`\), 16.5242`, 53.496504`, 48.097381`, 
              74.124174`, 103.933905`, 123.61685`, 144.315264`, 151.334854`, 
              42.177323`, 87.229118`, 89.497459`, 92.383678`, 80.051458`, 
              104.239538`, 111.7284`, 125.816385`}, {5, 28, 
              18631.37638`, \(-0.25`\), \(-19.6720811`\), 0.2025`, 
              1, \(-0.489648106`\), \(-0.35`\), 
              704.0604725`, \(-3.774279114`\), \(-5.258544929`\), 2, 4, 5, 6, 
              0.39791`, \(-1.244664`\), 1.386537`, \(-0.016448`\), 
              0.108554`, \(-0.048395`\), 0.901614`, \(-0.099456`\), 
              0.336964`, \(-0.103481`\), 0.189413`, \(-0.11512`\), 16.5242`, 
              16.317864`, 
              25.842307`, \(-73.765522`\), \(-73.579821`\), \(-23.257927`\), \
\(-130.144367`\), \(-130.180023`\), 
              4.465716`, \(-151.956752`\), \(-152.133847`\), 
              22.180089`, \(-101.411735`\), \(-101.509206`\), 
              12.207626`, \(-162.146233`\), \(-162.308219`\), 
              20.287641`, \(-215.369132`\), \(-215.069041`\), \
\(-37.584478`\), \(-216.421361`\), \(-216.41023`\), \(-1.394146`\), 
              16.5242`, \(-73.765522`\), \(-130.144367`\), \(-151.956752`\), \
\(-218.849471`\), \(-275.32636`\), \(-334.395463`\), \(-339.719299`\), \
\(-101.411735`\), \(-162.146233`\), \(-215.369132`\), \(-216.421361`\), \
\(-154.124218`\), \(-244.680069`\), \(-220.875692`\), \(-233.7163`\)}, {5, 
              27, 14430.22825`, \(-0.25`\), \(-16.19892374`\), 0.2025`, 
              1, \(-0.5091310006`\), \(-0.3`\), 
              4886.066818`, \(-3.81512225`\), \(-6.585312528`\), 2, 4, 5, 6, 
              0.419706`, \(-1.166103`\), 1.295958`, \(-0.020118`\), 
              0.109621`, \(-0.050841`\), 0.900577`, \(-0.106355`\), 
              0.098159`, \(-0.112132`\), \(-0.251115`\), \(-0.128623`\), 
              16.5242`, 16.409926`, 
              12.430839`, \(-57.161938`\), \(-57.058286`\), \(-11.27529`\), \
\(-106.949829`\), \(-106.975584`\), 
              2.801582`, \(-122.54228`\), \(-122.641541`\), 
              10.797783`, \(-83.966961`\), \(-84.035959`\), 
              7.505644`, \(-135.751991`\), \(-135.87794`\), 
              13.700747`, \(-173.138108`\), \(-173.024459`\), \
\(-12.362823`\), \(-177.587083`\), \(-177.563775`\), \(-2.535416`\), 
              16.5242`, \(-57.161938`\), \(-106.949829`\), \(-122.54228`\), \
\(-180.218422`\), \(-226.432747`\), \(-265.662289`\), \(-280.887475`\), \
\(-83.966961`\), \(-135.751991`\), \(-173.138108`\), \(-177.587083`\), \
\(-126.126904`\), \(-198.15124`\), \(-170.008063`\), \(-193.732257`\)}, {2, 
              17, 14483.09031`, \(-0.25`\), \(-15.65367889`\), 0.2025`, 
              1, \(-0.5218923459`\), \(-0.25`\), 
              4316.770054`, \(-3.965644341`\), \(-6.590503951`\), 2, 4, 5, 6, 
              0.445371`, \(-1.052759`\), 1.138347`, \(-0.032298`\), 
              0.122653`, \(-0.06975`\), 0.900992`, \(-0.221425`\), 
              0.097769`, \(-0.230769`\), \(-0.252116`\), \(-0.257425`\), 
              16.5242`, 16.457121`, 
              7.482494`, \(-55.786392`\), \(-55.726446`\), \(-6.686828`\), \
\(-103.413041`\), \(-103.434885`\), 
              2.436564`, \(-118.225613`\), \(-118.283943`\), 
              6.506554`, \(-81.841425`\), \(-81.896311`\), 
              6.122377`, \(-135.735376`\), \(-135.856288`\), 
              13.4874`, \(-165.406392`\), \(-165.328308`\), \(-8.710065`\), \
\(-171.007494`\), \(-170.968828`\), \(-4.313123`\), 
              16.5242`, \(-55.786392`\), \(-103.413041`\), \(-118.225613`\), \
\(-176.065809`\), \(-220.249519`\), \(-253.403136`\), \(-274.156992`\), \
\(-81.841425`\), \(-135.735376`\), \(-165.406392`\), \(-171.007494`\), \
\(-123.309275`\), \(-190.366141`\), \(-158.389212`\), \(-190.691419`\)}, {5, 
              27, 10991.02551`, \(-0.4729219692`\), 14.58917594`, 0.2025`, 
              1, \(-0.5070920754`\), \(-0.2`\), 
              169.8635494`, \(-2.918578682`\), \(-2.754219025`\), 2, 4, 5, 6, 
              0.503555`, \(-0.597619`\), 0.308842`, 0.066366`, 
              0.1411`, \(-0.090216`\), 0.914641`, 0.819449`, 0.771953`, 
              0.839442`, 0.965738`, 0.896251`, 16.5242`, 16.564714`, 
              4.762096`, 74.646325`, 74.612143`, \(-4.017872`\), 100.321102`, 
              100.338652`, 2.062895`, 134.098936`, 134.103598`, 0.547985`, 
              78.766304`, 78.80227`, 4.227491`, 149.894045`, 150.025526`, 
              15.454692`, 160.102104`, 160.043074`, \(-6.938548`\), 
              168.668434`, 168.599372`, \(-8.117817`\), 16.5242`, 74.646325`, 
              100.321102`, 138.397076`, 180.925439`, 220.328945`, 
              264.825507`, 273.456775`, 78.766304`, 149.894045`, 160.102104`, 
              168.668434`, 134.098936`, 191.699827`, 191.799434`, 
              209.075382`}, {5, 27, 134.9000388`, \(-0.5829469129`\), 
              9.635976711`, 0.2025`, 1, \(-0.5011873267`\), \(-0.15`\), 
              1300.6225`, \(-2.474807367`\), \(-2.390269309`\), 2, 4, 5, 6, 
              0.536235`, \(-0.486862`\), 0.111915`, 0.110079`, 
              0.126461`, \(-0.084804`\), 0.91925`, 0.891184`, 0.82345`, 
              0.939184`, 1.054136`, 1.068632`, 16.5242`, 16.53843`, 
              1.176452`, 54.226025`, 54.214441`, \(-0.957682`\), 67.095053`, 
              67.106011`, 0.905912`, 92.948897`, 92.950567`, 0.138043`, 
              52.75667`, 52.774118`, 1.442481`, 105.386691`, 
              105.373769`, \(-1.068339`\), 105.678888`, 105.732053`, 
              4.395354`, 113.782008`, 113.743056`, \(-3.220374`\), 16.5242`, 
              54.226025`, 67.095053`, 96.721126`, 123.565649`, 149.246771`, 
              179.598087`, 185.259784`, 52.75667`, 105.678888`, 105.386691`, 
              113.782008`, 92.948897`, 129.444486`, 133.860322`, 
              144.554335`}, {5, 28, 55.89552885`, \(-0.6194725092`\), 
              9.910504431`, 0.2025`, 1, \(-0.5069043786`\), \(-0.1`\), 
              122.9453137`, \(-2.529084572`\), \(-2.35107649`\), 2, 4, 5, 6, 
              0.558206`, \(-0.445555`\), 0.046203`, 0.101986`, 
              0.135396`, \(-0.09492`\), 0.919218`, 0.877725`, 0.841252`, 
              0.920279`, 1.085528`, 1.035966`, 16.5242`, 16.530177`, 
              0.529052`, 57.54617`, 57.541245`, \(-0.435936`\), 68.826445`, 
              68.837764`, 1.001866`, 96.627805`, 96.628546`, 0.065589`, 
              54.479853`, 54.492671`, 1.134548`, 109.01467`, 109.018878`, 
              0.372493`, 110.306843`, 110.335724`, 2.556395`, 116.250165`, 
              116.215041`, \(-3.108978`\), 16.5242`, 57.54617`, 68.826445`, 
              98.966031`, 128.316037`, 154.459014`, 184.304481`, 191.552363`, 
              54.479853`, 110.306843`, 109.01467`, 116.250165`, 96.627805`, 
              132.660896`, 137.866736`, 150.833669`}, {2, 28, 
              96.50889312`, \(-0.658655249`\), 10.14539814`, 0.2025`, 
              1, \(-0.5113201654`\), \(-0.05`\), 
              72.55401119`, \(-2.566995766`\), \(-2.300378929`\), 2, 4, 5, 6, 
              0.57964`, \(-0.408117`\), \(-0.019031`\), 0.099773`, 
              0.139901`, \(-0.101162`\), 0.919167`, 0.897462`, 0.849108`, 
              0.93746`, 1.099395`, 1.046938`, 16.5242`, 16.525822`, 
              0.152581`, 61.223283`, 61.220039`, \(-0.305253`\), 70.090092`, 
              70.103324`, 1.245004`, 100.118527`, 100.118717`, 0.017906`, 
              55.968057`, 55.976233`, 0.769225`, 112.984097`, 112.994557`, 
              0.984124`, 113.762382`, 113.777034`, 1.378539`, 118.575302`, 
              118.543988`, \(-2.9464`\), 16.5242`, 61.223283`, 70.090092`, 
              100.837723`, 132.725488`, 159.199255`, 188.406422`, 197.12612`, 
              55.968057`, 113.762382`, 112.984097`, 118.575302`, 100.118527`, 
              135.492738`, 142.039801`, 156.688164`}, {2, 12, 
              132.4418703`, \(-0.7469845844`\), 8.364494279`, 0.2025`, 
              1, \(-0.5082442166`\), 0, 
              61.17001249`, \(-2.240600569`\), \(-1.979169181`\), 2, 4, 5, 6, 
              0.605406`, \(-0.361245`\), \(-0.112652`\), 0.127166`, 
              0.125788`, \(-0.094512`\), 0.921968`, 0.938893`, 0.902342`, 
              0.998818`, 1.190946`, 1.157941`, 16.5242`, 16.524324`, 
              0.010013`, 50.928519`, 50.957115`, 2.3169`, 61.206914`, 
              61.187345`, \(-1.585537`\), 85.335436`, 
              85.335424`, \(-0.000934`\), 46.535805`, 46.538668`, 0.231901`, 
              94.920022`, 94.924548`, 0.366719`, 95.875674`, 95.883216`, 
              0.611032`, 99.394066`, 99.373481`, \(-1.667818`\), 16.5242`, 
              50.928519`, 61.206914`, 85.934413`, 112.040825`, 133.666483`, 
              157.28896`, 166.007965`, 46.535805`, 94.920022`, 95.875674`, 
              99.394066`, 85.335436`, 113.43885`, 121.786551`, 
              132.840419`}, {5, 28, 134.7773228`, \(-0.7562901807`\), 
              9.487312856`, 0.2025`, 1, \(-0.5108874653`\), 0.05`, 
              115.6549774`, \(-2.414996726`\), \(-1.705170431`\), 2, 4, 5, 6, 
              0.621351`, \(-0.340187`\), \(-0.156042`\), 0.114454`, 
              0.134303`, \(-0.100981`\), 0.919702`, 0.976738`, 0.941389`, 
              1.023995`, 1.259956`, 1.1527`, 16.5242`, 16.524733`, 0.050315`, 
              62.486917`, 62.497835`, 1.029774`, 65.865595`, 
              65.864465`, \(-0.106601`\), 95.192085`, 95.192925`, 0.079277`, 
              52.702309`, 52.701621`, \(-0.064899`\), 107.254617`, 
              107.256452`, 0.17308`, 108.472907`, 
              108.468525`, \(-0.413299`\), 112.303719`, 
              112.292531`, \(-1.055195`\), 16.5242`, 62.486917`, 65.865595`, 
              95.192085`, 127.277344`, 151.502627`, 176.561733`, 188.179874`, 
              52.702309`, 107.254617`, 108.472907`, 112.303719`, 96.663026`, 
              127.091059`, 137.054975`, 151.310476`}, {2, 18, 
              128.194252`, \(-0.8256993522`\), 8.686221116`, 0.2025`, 
              1, \(-0.5076590006`\), 0.1`, 
              27.21073873`, \(-2.419154429`\), \(-2.016799379`\), 2, 4, 5, 6, 
              0.642333`, \(-0.3084`\), \(-0.229294`\), 0.115431`, 
              0.13759`, \(-0.107188`\), 0.920957`, 0.925778`, 0.8953`, 
              0.976929`, 1.179329`, 1.114455`, 16.5242`, 16.526283`, 
              0.185949`, 59.45263`, 59.461529`, 0.794454`, 60.015358`, 
              60.014423`, \(-0.083455`\), 88.270393`, 88.272924`, 0.225931`, 
              48.480955`, 48.476995`, \(-0.353512`\), 98.707539`, 
              98.7067`, \(-0.074872`\), 100.529742`, 
              100.51934`, \(-0.928584`\), 105.061132`, 
              105.056177`, \(-0.44235`\), 16.5242`, 60.015358`, 59.45263`, 
              88.270393`, 118.52894`, 140.504069`, 162.769521`, 174.653958`, 
              48.480955`, 98.707539`, 100.529742`, 105.061132`, 90.60485`, 
              117.434467`, 129.12965`, 141.272408`}, {5, 27, 
              116.7855673`, \(-0.8933620424`\), 7.763328668`, 0.2025`, 
              1, \(-0.4959197732`\), 0.15`, 
              37.88333219`, \(-2.307501418`\), \(-2.013910633`\), 2, 4, 5, 6, 
              0.66156`, \(-0.280837`\), \(-0.298257`\), 0.126718`, 
              0.131168`, \(-0.104407`\), 0.921848`, 0.914784`, 0.896833`, 
              0.976376`, 1.181532`, 1.138832`, 16.5242`, 16.528131`, 
              0.32305`, 53.766171`, 53.774147`, 0.655472`, 56.128413`, 
              56.12643`, \(-0.162903`\), 80.661259`, 80.665164`, 0.320911`, 
              43.490685`, 43.484449`, \(-0.512451`\), 88.557843`, 
              88.555523`, \(-0.190693`\), 90.72`, 90.708543`, \(-0.941414`\), 
              95.941792`, 95.937861`, \(-0.323035`\), 16.5242`, 56.128413`, 
              53.766171`, 80.661259`, 107.574787`, 127.041639`, 146.382519`, 
              157.846863`, 43.490685`, 88.557843`, 90.72`, 95.941792`, 
              82.715818`, 105.850219`, 118.505847`, 128.468001`}, {2, 7, 
              79515.49954`, \(-0.25`\), \(-6.388416964`\), 0.25`, 
              1, \(-0.609262`\), \(-0.35`\), 50, \(-0.97786`\), \(-1.5017`\), 
              2, 4, 5, 6, 0.428569`, \(-1.497344`\), 1.729862`, 0.208331`, 
              0.020849`, 0.011966`, 0.92488`, 0.894989`, 0.984643`, 
              1.038385`, 1.335412`, 1.380687`, 16.5242`, 16.473891`, 
              2.203835`, \(-18.137669`\), \(-18.091673`\), \(-2.014891`\), \
\(-41.96298`\), \(-41.972181`\), 0.403069`, \(-44.481027`\), \(-44.53065`\), 
              2.173788`, \(-33.85534`\), \(-33.885908`\), 
              1.339044`, \(-51.331302`\), \(-51.390339`\), 
              2.586173`, \(-66.230066`\), \(-66.175661`\), \(-2.383291`\), \
\(-70.130951`\), \(-70.153858`\), 1.003475`, 
              16.5242`, \(-18.137669`\), \(-41.96298`\), \(-44.481027`\), \
\(-70.130951`\), \(-89.071874`\), \(-107.516662`\), \(-112.009722`\), \
\(-33.85534`\), \(-51.331302`\), \(-66.230066`\), \(-70.577355`\), \
\(-47.14012`\), \(-76.507186`\), \(-69.073983`\), \(-74.979826`\)}, {2, 7, 
              48704.01712`, \(-0.25`\), \(-8.416693312`\), 0.25`, 
              1, \(-0.609262`\), \(-0.3`\), 50, \(-0.97786`\), \(-1.5017`\), 
              2, 4, 5, 6, 0.464232`, \(-1.236325`\), 1.34564`, 0.208762`, 
              0.023959`, 0.005518`, 0.930624`, 1.222104`, 0.991242`, 
              1.33084`, 1.343301`, 1.613013`, 16.5242`, 16.479483`, 
              2.795581`, \(-28.121621`\), \(-28.081632`\), \(-2.500024`\), \
\(-56.109415`\), \(-56.120212`\), 0.674963`, \(-63.89507`\), \(-63.940631`\), 
              2.848369`, \(-45.597386`\), \(-45.629487`\), 
              2.006855`, \(-74.172822`\), \(-74.269802`\), 
              6.06293`, \(-89.541018`\), \(-89.483414`\), \(-3.601239`\), \
\(-92.786488`\), \(-92.745296`\), \(-2.575194`\), 
              16.5242`, \(-28.121621`\), \(-56.109415`\), \(-63.89507`\), \
\(-96.412253`\), \(-120.964863`\), \(-145.974464`\), \(-152.219377`\), \
\(-45.597386`\), \(-74.172822`\), \(-89.541018`\), \(-92.786488`\), \
\(-66.342099`\), \(-103.600571`\), \(-94.757098`\), \(-103.574738`\)}, {2, 
              23, 22749.59199`, \(-0.2519276172`\), \(-12.50849181`\), 0.25`, 
              1, \(-0.6026105646`\), \(-0.25`\), 
              3077.629046`, \(-4.654349912`\), \(-7.983398647`\), 2, 4, 5, 6, 
              0.500204`, \(-1.008009`\), 0.990448`, \(-0.076353`\), 
              0.144049`, \(-0.118749`\), 
              0.936314`, \(-0.544844`\), \(-0.077537`\), \(-0.570791`\), \
\(-0.597056`\), \(-0.643526`\), 16.5242`, 16.48325`, 
              4.099502`, \(-49.027759`\), \(-48.9915`\), \(-3.629843`\), \
\(-86.078094`\), \(-86.093002`\), 
              1.49241`, \(-102.810327`\), \(-102.850214`\), 
              3.993038`, \(-69.096993`\), \(-69.134258`\), 
              3.730594`, \(-123.015526`\), \(-123.196332`\), 
              18.100322`, \(-134.586676`\), \(-134.482425`\), \
\(-10.436423`\), \(-136.82267`\), \(-136.748609`\), \(-7.414244`\), 
              16.5242`, \(-49.027759`\), \(-86.078094`\), \(-102.810327`\), \
\(-149.407281`\), \(-185.274376`\), \(-223.561976`\), \(-231.746487`\), \
\(-69.096993`\), \(-123.015526`\), \(-134.586676`\), \(-136.82267`\), \
\(-105.074494`\), \(-158.312173`\), \(-147.978087`\), \(-161.468017`\)}, {2, 
              6, 29914.66318`, \(-0.3039380669`\), \(-19.23154662`\), 0.25`, 
              1, \(-0.609262`\), \(-0.2`\), 50, \(-0.97786`\), \(-1.5017`\), 
              2, 4, 5, 6, 0.531831`, \(-0.804632`\), 0.645692`, 0.209642`, 
              0.03272`, \(-0.011624`\), 0.938796`, 2.972105`, 1.001827`, 
              3.018464`, 1.356187`, 3.153413`, 16.5242`, 16.488136`, 
              5.901743`, \(-85.340024`\), \(-85.309678`\), \(-4.965985`\), \
\(-131.39966`\), \(-131.421062`\), 
              3.50243`, \(-164.096784`\), \(-164.137073`\), 
              6.593171`, \(-107.111922`\), \(-107.156928`\), 
              7.365145`, \(-201.940379`\), \(-202.148194`\), 
              34.008177`, \(-203.852623`\), \(-203.745211`\), \
\(-17.577654`\), \(-213.302308`\), \(-213.204063`\), \(-16.077512`\), 
              16.5242`, \(-85.340024`\), \(-131.39966`\), \(-164.096784`\), \
\(-236.546717`\), \(-290.266925`\), \(-347.373107`\), \(-361.469648`\), \
\(-107.111922`\), \(-201.940379`\), \(-203.852623`\), \(-213.302308`\), \
\(-168.543846`\), \(-245.502897`\), \(-237.552833`\), \(-263.096331`\)}, {2, 
              6, 720508.2936`, \(-0.3846914576`\), \(-93.06979208`\), 0.25`, 
              1, \(-0.609262`\), \(-0.15`\), 50, \(-0.97786`\), \(-1.5017`\), 
              2, 4, 5, 6, 0.565028`, \(-0.617779`\), 0.314459`, 0.210145`, 
              0.038632`, \(-0.022905`\), 0.942876`, 14.739811`, 1.00621`, 
              14.749195`, 1.360947`, 14.777312`, 16.5242`, 16.433105`, 
              76.646755`, \(-474.12855`\), \(-474.04907`\), \(-66.874411`\), \
\(-644.869306`\), \(-644.973714`\), 
              87.847844`, \(-841.337378`\), \(-841.44429`\), 
              89.955012`, \(-524.095892`\), \(-524.257146`\), 
              135.678307`, \(-992.84457`\), \(-992.794199`\), \
\(-42.381895`\), \(-1030.419156`\), \(-1030.876293`\), 
              384.631745`, \(-1059.736428`\), \(-1059.326231`\), \
\(-345.136196`\), 
              16.5242`, \(-474.12855`\), \(-644.869306`\), \(-841.337378`\), \
\(-1187.099886`\), \(-1440.712646`\), \(-1705.626169`\), \(-1790.812195`\), \
\(-524.095892`\), \(-1030.419156`\), \(-992.84457`\), \(-1059.736428`\), \
\(-861.13114`\), \(-1210.574776`\), \(-1231.445959`\), \(-1351.507376`\)}, \
{5, 26, 70.38046465`, \(-0.6620492294`\), 8.100984359`, 0.25`, 
              1, \(-0.5730282337`\), \(-0.1`\), 
              14.59361245`, \(-2.52027551`\), \(-2.445638049`\), 2, 4, 5, 6, 
              0.621632`, \(-0.3729`\), \(-0.157373`\), 0.118691`, 
              0.120092`, \(-0.087124`\), 0.959555`, 0.864544`, 0.884186`, 
              0.921485`, 1.140618`, 1.072104`, 16.5242`, 16.529077`, 
              0.392933`, 54.527862`, 54.520515`, \(-0.591961`\), 55.15641`, 
              55.169295`, 1.038162`, 86.382635`, 86.390766`, 0.655142`, 
              47.266265`, 47.274955`, 0.700235`, 95.249191`, 95.255457`, 
              0.504886`, 95.501276`, 95.523898`, 1.82277`, 102.129057`, 
              102.099443`, \(-2.386033`\), 16.5242`, 54.527862`, 55.15641`, 
              86.382635`, 113.403461`, 135.014234`, 159.296419`, 166.475173`, 
              47.266265`, 95.249191`, 95.501276`, 102.129057`, 86.574847`, 
              115.403416`, 129.557674`, 132.185629`}, {5, 27, 
              107.5328684`, \(-0.6572289846`\), 9.098606431`, 0.25`, 
              1, \(-0.5810366985`\), \(-0.05`\), 
              99.68179394`, \(-2.637281502`\), \(-2.458729767`\), 2, 4, 5, 6, 
              0.63699`, \(-0.354886`\), \(-0.182129`\), 0.110407`, 
              0.123542`, \(-0.088871`\), 0.956951`, 0.913052`, 0.878938`, 
              0.959901`, 1.132935`, 1.08688`, 16.5242`, 16.525398`, 
              0.111061`, 60.808257`, 60.820074`, 1.095835`, 65.891898`, 
              65.888137`, \(-0.348755`\), 96.502358`, 96.502492`, 0.012399`, 
              52.967478`, 52.973699`, 0.576908`, 106.55618`, 106.560917`, 
              0.439247`, 107.083077`, 107.100522`, 1.617696`, 112.983063`, 
              112.956874`, \(-2.428607`\), 16.5242`, 60.808257`, 65.891898`, 
              98.075074`, 127.078174`, 151.089944`, 176.559328`, 186.559282`, 
              52.967478`, 106.55618`, 107.083077`, 112.983063`, 96.502358`, 
              127.49394`, 142.821345`, 148.440033`}, {5, 28, 
              125.8075282`, \(-0.6326670152`\), 12.95312126`, 0.25`, 
              1, \(-0.5954123583`\), 0, 
              155.8989642`, \(-3.070591947`\), \(-2.656504474`\), 2, 4, 5, 6, 
              0.649351`, \(-0.342865`\), \(-0.200165`\), 0.075981`, 
              0.144258`, \(-0.107349`\), 0.952731`, 0.927058`, 0.843415`, 
              0.949771`, 1.072962`, 1.014364`, 16.5242`, 16.524251`, 
              0.006882`, 85.988684`, 86.001433`, 1.715673`, 93.205519`, 
              93.205255`, \(-0.035511`\), 132.819345`, 132.820032`, 
              0.092341`, 74.9817`, 74.985649`, 0.531446`, 149.846353`, 
              149.852349`, 0.80691`, 150.45658`, 150.460708`, 0.555459`, 
              158.424529`, 158.403908`, \(-2.775247`\), 16.5242`, 85.988684`, 
              93.205519`, 132.819345`, 179.428147`, 213.168646`, 246.434944`, 
              263.804903`, 74.9817`, 150.45658`, 149.846353`, 158.424529`, 
              135.045592`, 177.097181`, 197.371256`, 210.031602`}, {5, 28, 
              199.9055944`, \(-0.6474229774`\), 15.39095981`, 0.25`, 
              1, \(-0.6003909412`\), 0.05`, 
              134.1134673`, \(-3.195965553`\), \(-2.489996784`\), 2, 4, 5, 6, 
              0.66591`, \(-0.314611`\), \(-0.258409`\), 0.066355`, 
              0.151569`, \(-0.115752`\), 0.950902`, 0.97574`, 0.866606`, 
              0.992267`, 1.114813`, 1.040006`, 16.5242`, 16.524829`, 
              0.103076`, 105.21186`, 105.212583`, 0.118478`, 109.541695`, 
              109.555268`, 2.225611`, 154.554253`, 154.55542`, 0.191423`, 
              88.976946`, 88.975841`, \(-0.181343`\), 176.798678`, 
              176.791245`, \(-1.218989`\), 178.179326`, 178.180819`, 
              0.244855`, 189.193513`, 189.181734`, \(-1.931468`\), 16.5242`, 
              105.21186`, 109.541695`, 154.554253`, 213.911476`, 253.464426`, 
              290.356534`, 314.000621`, 88.976946`, 178.179326`, 176.798678`, 
              189.193513`, 160.665404`, 208.329435`, 233.706205`, 
              251.165693`}, {2, 26, 129.7664015`, \(-0.7957421278`\), 
              8.236521346`, 0.25`, 1, \(-0.5806469472`\), 0.1`, 
              33.12749767`, \(-2.694756989`\), \(-2.358757116`\), 2, 4, 5, 6, 
              0.697036`, \(-0.258708`\), \(-0.380637`\), 0.111433`, 
              0.131921`, \(-0.103363`\), 0.958129`, 0.883169`, 0.900195`, 
              0.933143`, 1.169489`, 1.067191`, 16.5242`, 16.525952`, 
              0.16091`, 59.066148`, 59.073682`, 0.692134`, 61.794407`, 
              61.793618`, \(-0.072443`\), 89.678393`, 89.683975`, 0.512707`, 
              48.374097`, 48.370793`, \(-0.303512`\), 96.98919`, 
              96.988048`, \(-0.104876`\), 99.585345`, 
              99.577823`, \(-0.690978`\), 106.261209`, 
              106.2573`, \(-0.359033`\), 16.5242`, 61.794407`, 59.066148`, 
              89.678393`, 119.271561`, 140.325961`, 160.5313`, 173.202848`, 
              48.374097`, 96.98919`, 99.585345`, 106.261209`, 91.465991`, 
              116.316403`, 134.979842`, 140.504259`}, {5, 26, 
              143.2694656`, \(-0.8697012809`\), 7.058966929`, 0.25`, 
              1, \(-0.5696704958`\), 0.15`, 
              40.14963879`, \(-2.424548904`\), \(-2.156103233`\), 2, 4, 5, 6, 
              0.717764`, \(-0.229784`\), \(-0.45025`\), 0.131388`, 
              0.120074`, \(-0.096093`\), 0.960023`, 0.889763`, 0.929415`, 
              0.956899`, 1.219288`, 1.131947`, 16.5242`, 16.527481`, 
              0.265978`, 51.130925`, 51.137307`, 0.517387`, 55.608014`, 
              55.606339`, \(-0.135816`\), 79.034889`, 79.041797`, 0.559983`, 
              41.676494`, 41.671544`, \(-0.401261`\), 83.499261`, 
              83.49712`, \(-0.173552`\), 86.755061`, 
              86.746624`, \(-0.683894`\), 93.270949`, 
              93.268009`, \(-0.238311`\), 16.5242`, 55.608014`, 51.130925`, 
              79.034889`, 104.085409`, 121.981266`, 138.937166`, 150.413897`, 
              41.676494`, 83.499261`, 86.755061`, 93.270949`, 80.40597`, 
              101.023209`, 119.1312`, 122.853206`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
datal2 l2_dscan2*.out (This contains points from dtraj1*.out)\
\>", \
"Subsection"],

Cell[BoxData[
    \(\(datal2 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from fitting \
lowest state to lattice value.\n2 parity doublets with fractional errors 1 \
0.1.\nSpectrum for P_perp a = (0)  ( 0.25) \n using (# states, o, multiplet, \
c^2 error for each) =\n  (4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & \
1, 0.25 0.25 1 1).\nSpectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  \
(26/2,6)  (32/2,6) .\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( \
2,20/2,6) \n  ( 2,24/2,4)  ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \
\n  ( 3,35/2,5)  ( 4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy \
potential determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  \
( 1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{5, 27, 8112.62384`, \(-0.25`\), \(-38.34754933`\), 0, 
              1, 0.1558137587`, \(-0.35`\), 
              7396.777546`, \(-2.058178842`\), \(-3.124041783`\), 2, 4, 5, 6, 
              0.042015`, \(-0.980651`\), 1.319708`, \(-0.02607`\), 0.113531`, 
              0.03395`, 0.538675`, \(-0.087064`\), 
              0.040242`, \(-0.097382`\), \(-0.171585`\), \(-0.125683`\), 
              16.5242`, 15.655633`, 
              22.390443`, \(-75.766527`\), \(-77.036578`\), 
              32.740137`, \(-110.884964`\), \(-112.020976`\), 
              29.284804`, \(-119.149773`\), \(-119.59227`\), 
              11.406954`, \(-79.307936`\), \(-79.893642`\), 
              15.098672`, \(-125.912969`\), \(-126.37256`\), 
              11.847632`, \(-194.059313`\), \(-194.557989`\), 
              12.855162`, \(-214.639375`\), \(-215.110828`\), 12.153409`, 
              16.5242`, \(-119.149773`\), \(-75.766527`\), \(-110.884964`\), \
\(-214.639375`\), \(-286.900033`\), \(-326.791081`\), \(-330.873845`\), \
\(-79.307936`\), \(-125.912969`\), \(-194.059313`\), \(-234.529637`\), \
\(-148.403683`\), \(-215.741586`\), \(-281.322143`\), \(-325.88145`\)}, {2, 
              23, 6220.26202`, \(-0.25`\), \(-28.22240415`\), 0, 1, 
              0.07667457967`, \(-0.3`\), 
              5.078752004`, \(-1.791329377`\), \(-2.191228745`\), 2, 4, 5, 6, 
              0.055812`, \(-1.010808`\), 1.333765`, \(-0.003757`\), 
              0.140378`, 0.012714`, 0.562402`, 0.117235`, 0.295538`, 
              0.115411`, 0.290599`, 0.110593`, 16.5242`, 15.928682`, 
              15.008461`, \(-27.532318`\), \(-28.477644`\), 
              23.824445`, \(-70.736815`\), \(-70.827199`\), 
              2.277885`, \(-108.621524`\), \(-108.495936`\), \(-3.165128`\), \
\(-63.526648`\), \(-63.969782`\), 
              11.16802`, \(-96.728396`\), \(-97.043449`\), 
              7.940084`, \(-142.584316`\), \(-142.78447`\), 
              5.044352`, \(-165.09392`\), \(-165.391897`\), 7.509717`, 
              16.5242`, \(-27.532318`\), \(-70.736815`\), \(-113.117481`\), \
\(-165.09392`\), \(-217.256807`\), \(-243.655793`\), \(-266.450161`\), \
\(-63.526648`\), \(-96.728396`\), \(-142.584316`\), \(-179.844524`\), \
\(-108.621524`\), \(-168.152218`\), \(-216.555647`\), \(-233.807327`\)}, {5, 
              28, 146107.054`, \(-0.6847234849`\), 21.00437403`, 0, 1, 
              0.05852199281`, \(-0.25`\), 
              6.048664414`, \(-0.6461799595`\), \(-0.1048229281`\), 2, 4, 5, 
              6, 0.101098`, \(-0.578857`\), 0.543386`, 0.130772`, 
              0.22126`, \(-0.19155`\), 0.584594`, 1.065942`, 0.784508`, 
              1.126948`, 1.149027`, 1.289308`, 16.5242`, 16.844811`, 
              10.893032`, 74.355405`, 74.422875`, 2.292338`, 85.926812`, 
              85.841033`, \(-2.914414`\), 91.422088`, 
              86.048365`, \(-182.576978`\), 55.533644`, 55.74888`, 7.312825`, 
              96.999128`, 97.189526`, 6.468935`, 145.412955`, 145.514056`, 
              3.435014`, 152.016011`, 152.251952`, 8.016299`, 16.5242`, 
              74.355405`, 85.926812`, 91.422088`, 145.412955`, 189.483133`, 
              235.634989`, 257.493085`, 55.533644`, 96.999128`, 152.016011`, 
              166.024349`, 107.59895`, 181.406893`, 180.518187`, 
              208.501968`}, {5, 28, 45012.90456`, \(-1.100124454`\), 
              10.96476161`, 0, 1, 0.03616117706`, \(-0.2`\), 
              6.013199404`, \(-0.6454535391`\), \(-0.4698266925`\), 2, 4, 5, 
              6, 0.137835`, \(-0.475295`\), 0.345231`, 0.132938`, 
              0.228241`, \(-0.211`\), 0.601716`, 0.71757`, 0.729189`, 
              0.819138`, 1.046861`, 1.052642`, 16.5242`, 16.650849`, 
              3.062534`, 41.054128`, 41.063955`, 0.237616`, 47.013976`, 
              46.994321`, \(-0.475296`\), 60.021305`, 
              55.699491`, \(-104.506629`\), 31.676555`, 31.758165`, 
              1.973422`, 57.871387`, 57.956073`, 2.047792`, 81.555861`, 
              81.588959`, 0.80037`, 88.740216`, 88.859623`, 2.887401`, 
              16.5242`, 41.054128`, 47.013976`, 60.021305`, 81.555861`, 
              105.433911`, 130.781749`, 142.163525`, 31.676555`, 57.871387`, 
              88.740216`, 92.545519`, 62.186735`, 101.956686`, 105.031671`, 
              108.019887`}, {5, 26, 6.829128776`, \(-1.714796131`\), 
              6.987764666`, 0, 1, 0.03476567279`, \(-0.15`\), 
              10307.29769`, \(-0.56505539`\), \(-0.5932496702`\), 2, 4, 5, 6, 
              0.176206`, \(-0.430415`\), 0.261732`, 0.145284`, 
              0.218895`, \(-0.219207`\), 0.614936`, 0.570318`, 0.719614`, 
              0.721956`, 1.024803`, 1.023278`, 16.5242`, 16.576651`, 
              1.033317`, 28.308913`, 28.28525`, \(-0.466176`\), 33.076257`, 
              33.094615`, 0.361667`, 43.675524`, 43.684839`, 0.18351`, 
              21.493825`, 21.52835`, 0.680156`, 41.743774`, 41.785417`, 
              0.820405`, 55.406851`, 55.417368`, 0.207195`, 58.539217`, 
              58.581296`, 0.828993`, 16.5242`, 28.308913`, 33.076257`, 
              48.38262`, 55.406851`, 71.162042`, 88.105153`, 92.586385`, 
              21.493825`, 41.743774`, 58.539217`, 62.557737`, 43.675524`, 
              67.407116`, 71.279419`, 75.666229`}, {5, 28, 
              13.48393541`, \(-1.692948377`\), 7.236778344`, 0, 
              1, \(-0.02027661683`\), \(-0.1`\), 
              4.812729483`, \(-0.8464874498`\), \(-0.8820259779`\), 2, 4, 5, 
              6, 0.196657`, \(-0.423012`\), 0.260753`, 0.110792`, 
              0.249679`, \(-0.217244`\), 0.622594`, 0.537541`, 0.665508`, 
              0.646984`, 0.925076`, 0.871361`, 16.5242`, 16.566044`, 
              0.952821`, 25.658361`, 25.63458`, \(-0.541499`\), 29.175682`, 
              29.19273`, 0.388199`, 43.990142`, 43.996192`, 0.137758`, 
              23.255291`, 23.284383`, 0.662451`, 44.021`, 44.053631`, 
              0.743021`, 57.408898`, 57.408593`, \(-0.006925`\), 61.412033`, 
              61.470697`, 1.335825`, 16.5242`, 25.658361`, 29.175682`, 
              48.328673`, 57.408898`, 73.611583`, 85.796767`, 93.602191`, 
              23.255291`, 44.021`, 61.412033`, 63.402457`, 43.990142`, 
              62.26178`, 70.455326`, 80.130819`}, {5, 28, 
              39.5757694`, \(-1.685558778`\), 7.816099297`, 0, 
              1, \(-0.05792357358`\), \(-0.05`\), 
              2.575255177`, \(-1.025109588`\), \(-1.031501628`\), 2, 4, 5, 6, 
              0.21809`, \(-0.436319`\), 0.304492`, 0.08775`, 
              0.260341`, \(-0.206079`\), 0.623396`, 0.524184`, 0.633243`, 
              0.601986`, 0.866911`, 0.77028`, 16.5242`, 16.547823`, 
              0.644283`, 23.525274`, 23.510278`, \(-0.408989`\), 25.501398`, 
              25.519569`, 0.49559`, 46.077297`, 46.079476`, 0.059427`, 
              25.895653`, 25.917116`, 0.585385`, 48.228205`, 48.250224`, 
              0.600531`, 61.50816`, 61.484111`, \(-0.655923`\), 65.706406`, 
              65.689585`, \(-0.45877`\), 16.5242`, 23.525274`, 25.501398`, 
              46.787488`, 61.50816`, 78.811256`, 87.441334`, 101.960835`, 
              25.895653`, 48.228205`, 65.706406`, 67.322003`, 46.077297`, 
              60.330785`, 75.266288`, 82.300338`}, {2, 8, 
              167.36759`, \(-1.827953687`\), 7.192146386`, 0, 
              1, \(-0.0678817`\), 0, 50, \(-1.24395`\), \(-1.13649`\), 2, 4, 
              5, 6, 0.244089`, \(-0.445419`\), 0.320339`, 0.064286`, 
              0.269205`, \(-0.199783`\), 0.623551`, 0.441407`, 0.610548`, 
              0.50252`, 0.826748`, 0.630964`, 16.5242`, 16.524793`, 
              0.016669`, 21.468389`, 21.469328`, 0.026371`, 33.713101`, 
              33.736032`, 0.644095`, 43.127852`, 43.127864`, 0.000325`, 
              24.405576`, 24.413961`, 0.235526`, 46.521409`, 46.530351`, 
              0.251173`, 57.35123`, 57.278103`, \(-2.054022`\), 59.30142`, 
              59.341753`, 1.132889`, 16.5242`, 21.468389`, 33.713101`, 
              47.75346`, 57.35123`, 72.397636`, 81.090246`, 93.490191`, 
              24.405576`, 46.521409`, 59.30142`, 62.726509`, 43.127852`, 
              54.13597`, 70.693047`, 73.161936`}, {5, 27, 
              129.1662164`, \(-2.41296374`\), 5.397526987`, 0, 
              1, \(-0.02761175993`\), 0.05`, 
              2122.033611`, \(-0.1915066663`\), \(-0.261806849`\), 2, 4, 5, 
              6, 0.269839`, \(-0.433671`\), 0.270128`, 0.185903`, 
              0.138098`, \(-0.155624`\), 0.628574`, 0.603828`, 0.807022`, 
              0.795015`, 1.168623`, 1.176635`, 16.5242`, 16.528453`, 
              0.099104`, 19.501947`, 19.498766`, \(-0.074133`\), 25.865711`, 
              25.883493`, 0.41439`, 35.798905`, 35.798902`, \(-0.00007`\), 
              18.514031`, 18.513185`, \(-0.019716`\), 38.352897`, 38.35339`, 
              0.011481`, 45.559869`, 45.460128`, \(-2.324308`\), 46.411927`, 
              46.467413`, 1.293019`, 16.5242`, 19.501947`, 25.865711`, 
              43.569929`, 45.559869`, 58.154714`, 66.466692`, 76.663189`, 
              18.514031`, 38.352897`, 46.411927`, 47.909173`, 35.798905`, 
              46.585769`, 56.195834`, 59.628032`}, {5, 27, 
              43881.34237`, \(-1.083248069`\), 51.74673481`, 0, 
              1, \(-0.1115107564`\), 0.1`, 
              88.26823455`, \(-1.620211276`\), \(-1.352104132`\), 2, 4, 5, 6, 
              0.275735`, \(-0.508508`\), 0.415609`, 0.016329`, 
              0.260654`, \(-0.157039`\), 0.596314`, 0.698578`, 0.53052`, 
              0.700477`, 0.699203`, 0.706123`, 16.5242`, 16.546975`, 
              5.199323`, 135.78067`, 135.818951`, 8.739283`, 175.332664`, 
              175.273537`, \(-13.498376`\), 255.447192`, 255.653167`, 
              47.022776`, 177.117215`, 177.019571`, \(-22.291561`\), 
              322.322438`, 322.290728`, \(-7.23922`\), 350.528688`, 
              350.00289`, \(-120.036692`\), 383.041192`, 383.464683`, 
              96.680327`, 16.5242`, 135.78067`, 175.332664`, 255.447192`, 
              383.041192`, 505.059188`, 493.019233`, 591.761852`, 
              177.117215`, 322.322438`, 350.528688`, 447.955286`, 
              264.288992`, 317.484185`, 437.745083`, 489.239297`}, {2, 24, 
              12712.43577`, \(-0.7766130723`\), \(-16.33999862`\), 0, 
              1, \(-0.1259988114`\), 0.15`, 
              13892.81568`, \(-2.041730955`\), \(-3.329558306`\), 2, 4, 5, 6, 
              0.286143`, \(-0.540238`\), 0.421706`, \(-0.038472`\), 
              0.255292`, \(-0.135392`\), 0.5721`, \(-0.096472`\), 
              0.033096`, \(-0.109359`\), \(-0.219055`\), \(-0.14556`\), 
              16.5242`, 
              16.524766`, \(-0.042378`\), \(-30.878473`\), \(-30.95455`\), 
              5.691248`, \(-36.144056`\), \(-36.076985`\), \(-5.017528`\), \
\(-70.435871`\), \(-70.452904`\), 
              1.274238`, \(-55.205377`\), \(-55.144906`\), \(-4.523822`\), \
\(-94.232057`\), \(-94.099722`\), \(-9.899859`\), \(-96.426329`\), \
\(-96.491465`\), 4.872819`, \(-113.671457`\), \(-113.706234`\), 2.601688`, 
              16.5242`, \(-30.878473`\), \(-36.144056`\), \(-74.507922`\), \
\(-113.671457`\), \(-148.124037`\), \(-145.472773`\), \(-169.138199`\), \
\(-55.205377`\), \(-94.232057`\), \(-96.426329`\), \(-133.747514`\), \
\(-70.435871`\), \(-92.027976`\), \(-122.764905`\), \(-144.249036`\)}, {5, 
              28, 18553.30596`, \(-0.25`\), \(-26.03014187`\), 0.0025`, 1, 
              0.07388589996`, \(-0.35`\), 
              275.9874695`, \(-2.169581992`\), \(-3.419374984`\), 2, 4, 5, 6, 
              0.06918`, \(-1.129057`\), 1.486769`, \(-0.027982`\), 
              0.149535`, \(-0.013267`\), 0.599226`, \(-0.043983`\), 
              0.084914`, \(-0.056238`\), \(-0.127413`\), \(-0.08926`\), 
              16.5242`, 15.924965`, 
              17.265286`, \(-69.751691`\), \(-75.522932`\), 
              166.282123`, \(-74.661837`\), \(-79.418051`\), 
              137.036964`, \(-85.98455`\), \(-86.283256`\), 
              8.606375`, \(-63.915406`\), \(-64.311057`\), 
              11.399588`, \(-97.699947`\), \(-98.005038`\), 
              8.790332`, \(-165.537192`\), \(-165.789719`\), 
              7.27587`, \(-165.978142`\), \(-166.243195`\), 7.63675`, 
              16.5242`, \(-85.98455`\), \(-69.751691`\), \(-74.661837`\), \
\(-165.537192`\), \(-217.920365`\), \(-235.670793`\), \(-248.71369`\), \
\(-63.915406`\), \(-97.699947`\), \(-165.978142`\), \(-173.324828`\), \
\(-110.819923`\), \(-140.687648`\), \(-207.42626`\), \(-254.539926`\)}, {5, 
              28, 9030.662981`, \(-0.6637747448`\), 17.59215782`, 0.0025`, 1, 
              0.06309601266`, \(-0.3`\), 
              223.9498247`, \(-1.394782924`\), \(-1.292138974`\), 2, 4, 5, 6, 
              0.117532`, \(-0.634512`\), 0.590397`, 0.069054`, 
              0.213509`, \(-0.126058`\), 0.620449`, 0.609082`, 0.587008`, 
              0.645038`, 0.789901`, 0.738402`, 16.5242`, 16.817296`, 
              9.69625`, 73.569092`, 73.579076`, 0.330262`, 82.77975`, 
              81.925149`, \(-28.272031`\), 83.16279`, 
              82.851555`, \(-10.296311`\), 49.868748`, 50.045811`, 5.857636`, 
              88.020184`, 88.187345`, 5.530047`, 131.009393`, 131.079256`, 
              2.311215`, 143.588657`, 143.783703`, 6.452542`, 16.5242`, 
              73.569092`, 82.77975`, 83.16279`, 131.009393`, 169.565915`, 
              213.4007`, 214.262536`, 49.868748`, 88.020184`, 143.588657`, 
              145.178051`, 98.357049`, 145.447346`, 185.420302`, 
              192.891737`}, {5, 27, 120.1819142`, \(-1.387598946`\), 
              7.580549999`, 0.0025`, 1, 0.08650221666`, \(-0.25`\), 
              5.581052502`, \(-0.6200216714`\), \(-0.7236752022`\), 2, 4, 5, 
              6, 0.166901`, \(-0.498407`\), 0.339295`, 0.156557`, 
              0.186096`, \(-0.183034`\), 0.636321`, 0.611839`, 0.72469`, 
              0.763216`, 1.024097`, 1.075751`, 16.5242`, 16.620398`, 
              1.947357`, 30.731845`, 30.764007`, 0.651057`, 37.995008`, 
              37.945885`, \(-0.994401`\), 49.497106`, 49.667325`, 3.445795`, 
              23.375429`, 23.423771`, 0.978598`, 45.63083`, 45.689942`, 
              1.196627`, 62.389051`, 62.403086`, 0.284121`, 64.706852`, 
              64.762776`, 1.132084`, 16.5242`, 37.995008`, 30.731845`, 
              49.497106`, 62.389051`, 79.757781`, 103.377629`, 103.997895`, 
              23.375429`, 45.63083`, 64.706852`, 71.755983`, 49.99974`, 
              76.260376`, 86.946347`, 90.98475`}, {5, 26, 
              52.11061983`, \(-1.300618396`\), 7.495042017`, 0.0025`, 1, 
              0.01688336714`, \(-0.2`\), 
              5099.508319`, \(-0.7843897659`\), \(-0.8564647657`\), 2, 4, 5, 
              6, 0.184623`, \(-0.467092`\), 0.28461`, 0.136427`, 
              0.213372`, \(-0.197034`\), 0.649422`, 0.591547`, 0.712546`, 
              0.721245`, 0.994795`, 0.991624`, 16.5242`, 16.599386`, 
              1.664639`, 32.976464`, 32.946438`, \(-0.664779`\), 37.206821`, 
              37.221364`, 0.321981`, 48.57151`, 48.563514`, \(-0.177032`\), 
              24.297577`, 24.341833`, 0.979829`, 46.081872`, 46.133567`, 
              1.144527`, 62.228298`, 62.238786`, 0.232209`, 64.920406`, 
              64.970651`, 1.112415`, 16.5242`, 32.976464`, 37.206821`, 
              50.955599`, 62.228298`, 79.381748`, 100.244312`, 100.889938`, 
              24.297577`, 46.081872`, 64.920406`, 70.181193`, 48.57151`, 
              78.201718`, 75.201425`, 88.415714`}, {5, 27, 
              8.074393059`, \(-1.504712437`\), 6.538905905`, 0.0025`, 
              1, \(-0.01430058976`\), \(-0.15`\), 
              2614.957258`, \(-0.5903682019`\), \(-0.6830348675`\), 2, 4, 5, 
              6, 0.21106`, \(-0.444717`\), 0.25599`, 0.155617`, 
              0.198147`, \(-0.202015`\), 0.65835`, 0.598383`, 0.758317`, 
              0.754648`, 1.069127`, 1.071236`, 16.5242`, 16.572835`, 
              1.073947`, 27.543693`, 27.513127`, \(-0.67495`\), 32.638803`, 
              32.657731`, 0.417965`, 43.31857`, 43.326606`, 0.177442`, 
              22.106704`, 22.136842`, 0.665502`, 42.420697`, 42.456479`, 
              0.790139`, 55.590115`, 55.592951`, 0.062631`, 57.561351`, 
              57.594756`, 0.737634`, 16.5242`, 27.543693`, 32.638803`, 
              47.799074`, 55.590115`, 70.684213`, 86.749513`, 91.342278`, 
              22.106704`, 42.420697`, 57.561351`, 61.385908`, 43.31857`, 
              65.470279`, 68.940743`, 73.735044`}, {5, 28, 
              9.692174315`, \(-1.456796482`\), 7.07076636`, 0.0025`, 
              1, \(-0.06134995128`\), \(-0.1`\), 
              4.767224836`, \(-0.8045896751`\), \(-0.9197209686`\), 2, 4, 5, 
              6, 0.228996`, \(-0.450777`\), 0.294004`, 0.131359`, 
              0.225204`, \(-0.20893`\), 0.660493`, 0.607923`, 0.712457`, 
              0.728167`, 0.988017`, 0.983647`, 16.5242`, 16.562874`, 
              1.001913`, 25.659708`, 25.632645`, \(-0.701128`\), 29.497527`, 
              29.515992`, 0.478348`, 45.256292`, 45.261028`, 0.122685`, 
              24.573556`, 24.599672`, 0.676574`, 46.129789`, 46.15845`, 
              0.742526`, 59.526207`, 59.52242`, \(-0.09813`\), 62.702921`, 
              62.711039`, 0.210327`, 16.5242`, 25.659708`, 29.497527`, 
              47.64548`, 59.526207`, 75.745416`, 88.509143`, 98.642656`, 
              24.573556`, 46.129789`, 62.702921`, 64.275287`, 45.256292`, 
              63.718389`, 73.919872`, 74.154755`}, {5, 28, 
              27.05934154`, \(-1.430568356`\), 8.058260445`, 0.0025`, 
              1, \(-0.09094793735`\), \(-0.05`\), 
              2.441329684`, \(-1.015053516`\), \(-1.091423263`\), 2, 4, 5, 6, 
              0.250206`, \(-0.464382`\), 0.341346`, 0.10773`, 
              0.240205`, \(-0.202633`\), 0.659306`, 0.624172`, 0.67559`, 
              0.708463`, 0.923874`, 0.900542`, 16.5242`, 16.544014`, 
              0.639182`, 25.783084`, 25.774165`, \(-0.287727`\), 26.529459`, 
              26.543688`, 0.459028`, 50.031651`, 50.047785`, 0.520493`, 
              28.607828`, 28.627629`, 0.638753`, 53.336589`, 53.357221`, 
              0.665556`, 67.337094`, 67.305736`, \(-1.0116`\), 69.99031`, 
              69.97587`, \(-0.465841`\), 16.5242`, 25.783084`, 26.529459`, 
              50.031651`, 67.337094`, 85.744169`, 96.638614`, 111.51785`, 
              28.607828`, 53.336589`, 69.99031`, 72.523215`, 50.422045`, 
              66.903853`, 82.579997`, 83.576444`}, {5, 28, 
              135.5622059`, \(-1.532969668`\), 7.294071054`, 0.0025`, 
              1, \(-0.09924890955`\), 0, 
              5588.000492`, \(-0.7815518555`\), \(-0.7061044553`\), 2, 4, 5, 
              6, 0.275529`, \(-0.468844`\), 0.350426`, 0.135179`, 
              0.220306`, \(-0.20692`\), 0.658994`, 0.683825`, 0.754231`, 
              0.79318`, 1.061447`, 1.038745`, 16.5242`, 16.524697`, 
              0.015996`, 23.788249`, 23.789175`, 0.029781`, 36.287427`, 
              36.307703`, 0.652002`, 46.408347`, 46.408344`, \(-0.000082`\), 
              26.383003`, 26.390672`, 0.246615`, 50.366158`, 50.375092`, 
              0.287281`, 61.714686`, 61.620511`, \(-3.028262`\), 62.222874`, 
              62.279667`, 1.826231`, 16.5242`, 23.788249`, 36.287427`, 
              49.71452`, 61.714686`, 78.304221`, 87.540055`, 101.21933`, 
              26.383003`, 50.366158`, 62.222874`, 66.194132`, 46.408347`, 
              59.616453`, 74.290919`, 77.730145`}, {5, 28, 
              115209.1866`, \(-1.061271047`\), 18.5061548`, 0.0025`, 
              1, \(-0.1333483604`\), 0.05`, 
              6.848252901`, \(-1.454138633`\), \(-1.241788913`\), 2, 4, 5, 6, 
              0.287799`, \(-0.511185`\), 0.442679`, 0.057308`, 
              0.250205`, \(-0.172454`\), 0.643978`, 0.801789`, 0.626068`, 
              0.820076`, 0.844858`, 0.871727`, 16.5242`, 16.528088`, 
              0.331358`, 54.231413`, 54.231142`, \(-0.02305`\), 78.41769`, 
              78.453364`, 3.039975`, 89.381828`, 
              87.407706`, \(-168.228196`\), 66.890688`, 
              66.886055`, \(-0.394733`\), 122.239241`, 122.245146`, 
              0.503196`, 140.809554`, 140.587227`, \(-18.946011`\), 
              148.341445`, 148.506395`, 14.056532`, 16.5242`, 54.231413`, 
              78.41769`, 89.381828`, 148.341445`, 180.543003`, 211.267738`, 
              234.776998`, 66.890688`, 122.239241`, 140.809554`, 166.542971`, 
              106.483811`, 130.096048`, 171.744017`, 178.011325`}, {5, 26, 
              5787.671968`, \(-0.6925781294`\), \(-26.45028029`\), 0.0025`, 
              1, \(-0.1572055206`\), 0.1`, 
              61752.5145`, \(-2.074840366`\), \(-3.59911318`\), 2, 4, 5, 6, 
              0.299133`, \(-0.556441`\), 0.513208`, \(-0.022337`\), 
              0.258535`, \(-0.149706`\), 0.623515`, \(-0.074945`\), 
              0.073064`, \(-0.079584`\), \(-0.173282`\), \(-0.093133`\), 
              16.5242`, 16.520409`, 
              0.479875`, \(-66.346629`\), \(-66.383477`\), 
              
              4.664749`, \(-76.091299`\), \(-76.028859`\), \(-7.904459`\), \
\(-120.268918`\), \(-120.346455`\), 
              9.815814`, \(-94.621849`\), \(-94.573186`\), \(-6.160491`\), \
\(-163.964894`\), \(-163.959656`\), \(-0.663079`\), \(-172.943813`\), \
\(-172.785891`\), \(-19.992121`\), \(-198.377868`\), \(-198.49022`\), 
              14.223157`, 
              16.5242`, \(-66.346629`\), \(-76.091299`\), \(-120.268918`\), \
\(-198.377868`\), \(-239.378513`\), \(-276.198268`\), \(-300.811915`\), \
\(-94.621849`\), \(-163.964894`\), \(-172.943813`\), \(-231.374892`\), \
\(-128.962113`\), \(-162.644617`\), \(-218.620514`\), \(-231.392658`\)}, {2, 
              8, 241134.2592`, \(-0.25`\), \(-3.466060818`\), 0.0025`, 
              1, \(-0.190343`\), 0.15`, 50, \(-1.80067`\), \(-1.74454`\), 2, 
              4, 5, 6, 0.301791`, \(-0.692694`\), 0.689524`, 0.020918`, 
              0.090487`, 0.06741`, 0.57991`, 0.16572`, 0.449253`, 0.177675`, 
              0.56626`, 0.215005`, 16.5242`, 16.528304`, \(-0.068693`\), 
              3.421655`, 3.447382`, \(-0.430566`\), 0.550357`, 0.520018`, 
              0.507766`, \(-5.983927`\), \(-6.005686`\), 
              0.364157`, \(-11.870854`\), \(-11.855715`\), \(-0.253375`\), \
\(-13.445279`\), \(-13.455113`\), 
              0.164591`, \(-17.705016`\), \(-17.697447`\), \(-0.126678`\), \
\(-20.891396`\), \(-20.879341`\), \(-0.201765`\), 16.5242`, 0.550357`, 
              3.421655`, \(-9.55822`\), \(-20.891396`\), \(-21.033552`\), \
\(-27.317792`\), \(-28.338025`\), \(-11.870854`\), \(-13.445279`\), \
\(-17.705016`\), \(-22.090994`\), \(-5.983927`\), \(-14.194313`\), \
\(-19.240062`\), \(-21.029261`\)}, {2, 13, 27793.11091`, \(-0.5731197742`\), 
              18.38461037`, 0.01`, 1, 0.05146108157`, \(-0.35`\), 
              185.6241064`, 2.041426198`, \(-1.95808018`\), 2, 4, 5, 6, 
              0.132839`, \(-0.712388`\), 0.676002`, 0.048802`, 
              0.177101`, \(-0.091235`\), 0.65516`, 0.490491`, 0.505257`, 
              0.513368`, 0.624991`, 0.5742`, 16.5242`, 16.870238`, 
              13.521413`, 74.913304`, 75.111659`, 7.750734`, 88.580128`, 
              93.877732`, 207.00379`, 100.792057`, 
              100.631777`, \(-6.262957`\), 55.839131`, 56.026633`, 7.326639`, 
              97.778085`, 97.961038`, 7.148881`, 145.633921`, 145.686888`, 
              2.06968`, 156.805174`, 156.974666`, 6.622917`, 16.5242`, 
              74.913304`, 88.580128`, 100.792057`, 145.633921`, 187.198583`, 
              217.717237`, 245.748581`, 55.839131`, 97.778085`, 156.805174`, 
              165.476086`, 109.401206`, 167.57751`, 189.951838`, 
              202.685736`}, {5, 28, 4841.231472`, \(-0.8522406781`\), 
              10.13830139`, 0.01`, 1, 0.02835566978`, \(-0.3`\), 
              107.192926`, \(-0.4403161739`\), \(-0.1566616631`\), 2, 4, 5, 
              6, 0.16739`, \(-0.573663`\), 0.42388`, 0.180269`, 
              0.148217`, \(-0.157296`\), 0.670103`, 0.858035`, 0.875854`, 
              0.98879`, 1.264995`, 1.295131`, 16.5242`, 16.67902`, 4.203796`, 
              33.227934`, 31.711543`, \(-41.17414`\), 61.048071`, 62.174697`, 
              30.590982`, 65.710516`, 64.735307`, \(-26.479592`\), 
              33.002768`, 33.084234`, 2.212042`, 60.309681`, 60.398399`, 
              2.408935`, 85.279541`, 85.292334`, 0.347367`, 89.088059`, 
              89.169013`, 2.198129`, 16.5242`, 33.227934`, 61.048071`, 
              67.231716`, 85.279541`, 108.65871`, 135.547726`, 142.968449`, 
              33.002768`, 60.309681`, 89.088059`, 97.578004`, 65.710516`, 
              102.434968`, 114.851071`, 116.855666`}, {5, 29, 
              29.29295108`, \(-1.435197513`\), 5.957230847`, 0.01`, 1, 
              0.04764052826`, \(-0.25`\), 
              240.6851907`, \(-0.5830709269`\), \(-0.7500217071`\), 2, 4, 5, 
              6, 0.213256`, \(-0.483709`\), 0.262587`, 0.17265`, 
              0.156946`, \(-0.161328`\), 0.683909`, 0.573269`, 0.781991`, 
              0.755336`, 1.098571`, 1.113967`, 16.5242`, 16.591396`, 
              1.365871`, 30.037601`, 30.052901`, 0.310985`, 32.441938`, 
              32.407209`, \(-0.705914`\), 43.551791`, 43.563863`, 0.245393`, 
              20.572454`, 20.605106`, 0.663689`, 40.720007`, 40.762406`, 
              0.861831`, 53.828086`, 53.746898`, \(-1.650284`\), 54.072286`, 
              54.192886`, 2.451384`, 16.5242`, 32.441938`, 30.037601`, 
              46.74752`, 54.072286`, 68.175637`, 86.447757`, 91.457349`, 
              20.572454`, 40.720007`, 53.828086`, 60.531681`, 43.551791`, 
              67.950183`, 70.597755`, 71.316751`}, {5, 27, 
              18.14269093`, \(-1.355928404`\), 6.095506628`, 0.01`, 
              1, \(-0.01496310689`\), \(-0.2`\), 
              2277.138707`, \(-0.6093471308`\), \(-0.7727467796`\), 2, 4, 5, 
              6, 0.229133`, \(-0.465189`\), 0.244913`, 0.16802`, 
              0.16851`, \(-0.173117`\), 0.691899`, 0.599426`, 0.78671`, 
              0.769044`, 1.101713`, 1.11129`, 16.5242`, 16.581473`, 
              1.279871`, 29.366036`, 29.336455`, \(-0.66105`\), 32.093063`, 
              32.103498`, 0.233187`, 43.648422`, 43.657878`, 0.211305`, 
              21.779398`, 21.809693`, 0.677005`, 42.177732`, 42.215342`, 
              0.840461`, 55.369792`, 55.331741`, \(-0.85031`\), 55.638663`, 
              55.711907`, 1.63678`, 16.5242`, 29.366036`, 32.093063`, 
              47.175488`, 55.369792`, 69.807937`, 89.039683`, 90.512386`, 
              21.779398`, 42.177732`, 55.638663`, 61.011318`, 43.648422`, 
              67.939934`, 70.101788`, 70.192492`}, {5, 28, 
              9.81270008`, \(-1.325757644`\), 6.234945672`, 0.01`, 
              1, \(-0.06318553711`\), \(-0.15`\), 
              1091.628266`, \(-0.6362868449`\), \(-0.7721210488`\), 2, 4, 5, 
              6, 0.245928`, \(-0.463288`\), 0.269409`, 0.163896`, 
              0.177212`, \(-0.180829`\), 0.695162`, 0.625493`, 0.790393`, 
              0.783422`, 1.105842`, 1.109966`, 16.5242`, 16.568737`, 
              1.092659`, 26.894142`, 26.861319`, \(-0.805263`\), 32.756268`, 
              32.773762`, 0.429188`, 43.542289`, 43.548499`, 0.152338`, 
              22.840771`, 22.867104`, 0.646036`, 43.502202`, 43.533202`, 
              0.760536`, 56.324665`, 56.313989`, \(-0.261931`\), 57.309125`, 
              57.344209`, 0.860725`, 16.5242`, 26.894142`, 32.756268`, 
              47.581252`, 56.324665`, 71.066526`, 87.154015`, 92.027969`, 
              22.840771`, 43.502202`, 57.309125`, 60.900545`, 43.542289`, 
              65.027148`, 69.590778`, 71.568114`}, {5, 28, 
              12.77802438`, \(-1.325723636`\), 6.633916289`, 0.01`, 
              1, \(-0.09802499445`\), \(-0.1`\), 
              5.501096619`, \(-0.8446047182`\), \(-0.9871744264`\), 2, 4, 5, 
              6, 0.265221`, \(-0.46991`\), 0.310322`, 0.143117`, 
              0.201836`, \(-0.190585`\), 0.695434`, 0.635001`, 0.749312`, 
              0.762694`, 1.034712`, 1.037042`, 16.5242`, 16.555757`, 
              0.888361`, 25.392074`, 25.368559`, \(-0.661974`\), 29.781508`, 
              29.797921`, 0.462033`, 45.368076`, 45.371423`, 0.094216`, 
              24.782835`, 24.804342`, 0.605475`, 46.826689`, 46.850906`, 
              0.681719`, 59.562446`, 59.553706`, \(-0.246057`\), 61.115385`, 
              61.116369`, 0.027712`, 16.5242`, 25.392074`, 29.781508`, 
              47.44884`, 59.562446`, 75.218445`, 89.165958`, 97.41232`, 
              24.782835`, 46.826689`, 61.115385`, 63.063189`, 45.368076`, 
              64.233524`, 72.725946`, 75.942793`}, {2, 8, 
              75.40294632`, \(-1.315727033`\), 6.774287165`, 0.01`, 
              1, \(-0.121627`\), \(-0.05`\), 
              50, \(-1.08318`\), \(-1.15045`\), 2, 4, 5, 6, 
              0.286388`, \(-0.479177`\), 0.348374`, 0.119587`, 
              0.220304`, \(-0.190853`\), 0.693851`, 0.601194`, 0.715646`, 
              0.70359`, 0.976968`, 0.927967`, 16.5242`, 16.533326`, 
              0.283288`, 23.985505`, 23.978936`, \(-0.203923`\), 35.093632`, 
              35.110737`, 0.530956`, 45.708107`, 45.709131`, 0.031781`, 
              25.701592`, 25.715916`, 0.444648`, 48.62858`, 48.644627`, 
              0.49811`, 60.58421`, 60.570207`, \(-0.434674`\), 61.360159`, 
              61.330142`, \(-0.931776`\), 16.5242`, 23.985505`, 35.093632`, 
              49.267213`, 60.58421`, 76.532076`, 88.472245`, 99.175592`, 
              25.701592`, 48.62858`, 61.360159`, 63.640464`, 45.708107`, 
              61.947593`, 72.700805`, 77.196649`}, {5, 28, 
              150.3414383`, \(-1.116445762`\), 9.824512802`, 0.01`, 
              1, \(-0.148760086`\), 0, 
              14.77388634`, \(-1.270106643`\), \(-1.136352662`\), 2, 4, 5, 6, 
              0.302676`, \(-0.5038`\), 0.414446`, 0.098332`, 
              0.228196`, \(-0.180255`\), 0.686423`, 0.726336`, 0.709364`, 
              0.783515`, 0.969858`, 0.927738`, 16.5242`, 16.524827`, 
              0.029816`, 32.450388`, 32.451521`, 0.053906`, 48.502631`, 
              48.525937`, 1.108857`, 59.892731`, 59.925823`, 1.574451`, 
              37.474831`, 37.484748`, 0.471845`, 69.776648`, 69.789444`, 
              0.608804`, 83.595083`, 83.48102`, \(-5.426939`\), 85.674233`, 
              85.743731`, 3.306588`, 16.5242`, 32.450388`, 48.502631`, 
              59.892731`, 85.674233`, 107.9341`, 122.41102`, 139.728315`, 
              37.474831`, 69.776648`, 83.595083`, 91.776048`, 63.21829`, 
              82.000758`, 100.12412`, 106.910087`}, {5, 26, 
              7992.677415`, \(-1.43987867`\), 7.795991956`, 0.01`, 
              1, \(-0.1324201825`\), 0.05`, 
              5.499290854`, \(-1.057542358`\), \(-0.9216319561`\), 2, 4, 5, 
              6, 0.331347`, \(-0.481728`\), 0.347978`, 0.12563`, 
              0.212152`, \(-0.187021`\), 0.690951`, 0.734253`, 0.757161`, 
              0.821103`, 1.05139`, 1.028363`, 16.5242`, 16.526532`, 
              0.096377`, 29.734432`, 29.733909`, \(-0.021594`\), 34.773469`, 
              34.788623`, 0.626322`, 51.724268`, 52.821211`, 45.337574`, 
              30.242354`, 30.241047`, \(-0.054032`\), 59.776729`, 59.779214`, 
              0.102722`, 68.208762`, 68.137031`, \(-2.964709`\), 71.05233`, 
              71.080326`, 1.157103`, 16.5242`, 29.734432`, 34.773469`, 
              51.724268`, 71.05233`, 89.377022`, 102.266325`, 114.546006`, 
              30.242354`, 59.776729`, 68.208762`, 73.764909`, 53.924763`, 
              70.308416`, 82.229808`, 91.487364`}, {2, 23, 
              5717.968232`, \(-0.6450902889`\), \(-24.84524197`\), 0.01`, 
              1, \(-0.1913405868`\), 0.1`, 
              20064.52881`, \(-2.287937289`\), \(-4.009879053`\), 2, 4, 5, 6, 
              0.328966`, \(-0.565417`\), 0.523387`, \(-0.023887`\), 
              0.25911`, \(-0.161827`\), 0.658361`, \(-0.092634`\), 
              0.070325`, \(-0.097662`\), \(-0.194502`\), \(-0.112339`\), 
              16.5242`, 16.520386`, 
              0.498746`, \(-69.983228`\), \(-70.004546`\), 
              2.78771`, \(-88.769132`\), \(-88.728559`\), \(-5.305735`\), \
\(-107.88168`\), \(-107.938361`\), 
              7.41225`, \(-93.536884`\), \(-93.49757`\), \(-5.141223`\), \
\(-163.649453`\), \(-163.641275`\), \(-1.069463`\), \(-171.537018`\), \
\(-171.424893`\), \(-14.662774`\), \(-198.727561`\), \(-198.820135`\), 
              12.106049`, 
              16.5242`, \(-69.983228`\), \(-88.769132`\), \(-107.88168`\), \
\(-198.727561`\), \(-230.766576`\), \(-290.670069`\), \(-302.809026`\), \
\(-93.536884`\), \(-163.649453`\), \(-171.537018`\), \(-225.027724`\), \
\(-133.166131`\), \(-166.906015`\), \(-215.069994`\), \(-231.727534`\)}, {2, 
              7, 364115.2798`, \(-0.25`\), \(-2.840649597`\), 0.01`, 
              1, \(-0.237185`\), 0.15`, 50, \(-1.92859`\), \(-1.82561`\), 2, 
              4, 5, 6, 0.324984`, \(-0.783297`\), 0.855642`, 0.029244`, 
              0.05854`, 0.097879`, 0.607068`, 0.166339`, 0.483194`, 
              0.178507`, 0.61407`, 0.23086`, 16.5242`, 
              16.527098`, \(-0.042812`\), 4.520392`, 
              4.542011`, \(-0.319329`\), 0.263739`, 0.240123`, 
              0.348832`, \(-4.872401`\), \(-4.88713`\), 
              0.21755`, \(-9.977735`\), \(-9.965935`\), \(-0.174291`\), \
\(-10.131954`\), \(-10.137729`\), 
              0.085307`, \(-13.803015`\), \(-14.460943`\), 
              9.718017`, \(-14.465378`\), \(-17.358749`\), 42.736967`, 
              16.5242`, 0.263739`, 
              4.520392`, \(-7.862031`\), \(-17.361164`\), \(-13.803015`\), \
\(-22.977848`\), \(-26.825815`\), \(-9.977735`\), \(-10.131954`\), \
\(-14.465378`\), \(-18.270958`\), \(-4.872401`\), \(-10.129564`\), \
\(-15.126769`\), \(-16.923848`\)}, {5, 26, 63.05780077`, \(-1.292737884`\), 
              5.803934411`, 0.0225`, 1, 0.09524264254`, \(-0.35`\), 
              11050.0638`, 0.9831007678`, \(-0.858161068`\), 2, 4, 5, 6, 
              0.216833`, \(-0.536445`\), 0.296805`, 0.181415`, 
              0.104257`, \(-0.100494`\), 0.706712`, 0.557371`, 0.7944`, 
              0.744859`, 1.109884`, 1.117682`, 16.5242`, 16.610669`, 
              1.741116`, 30.762047`, 30.78536`, 0.469417`, 35.917661`, 
              35.869873`, \(-0.962232`\), 45.24405`, 45.259594`, 0.312995`, 
              20.577081`, 20.612819`, 0.719616`, 41.090322`, 41.138375`, 
              0.967589`, 53.718033`, 53.737855`, 0.399125`, 55.367438`, 
              55.379365`, 0.240173`, 16.5242`, 35.917661`, 30.762047`, 
              47.967167`, 55.367438`, 69.252449`, 87.716322`, 90.783544`, 
              20.577081`, 41.090322`, 53.718033`, 62.482535`, 45.24405`, 
              69.182582`, 71.510415`, 79.637793`}, {5, 26, 
              34.60910919`, \(-1.31482134`\), 5.526425594`, 0.0225`, 1, 
              0.04003846848`, \(-0.3`\), 
              8296.880292`, \(-0.5038671442`\), \(-0.7304533391`\), 2, 4, 5, 
              6, 0.236687`, \(-0.498904`\), 0.23476`, 0.189787`, 
              0.119889`, \(-0.126181`\), 0.71805`, 0.583225`, 0.831067`, 
              0.782292`, 1.166332`, 1.175089`, 16.5242`, 16.595985`, 
              1.502356`, 29.797772`, 29.817871`, 0.42064`, 32.960486`, 
              32.915735`, \(-0.936571`\), 43.430086`, 43.442249`, 0.254538`, 
              20.361826`, 20.392652`, 0.645139`, 40.474086`, 40.51553`, 
              0.867359`, 51.840662`, 51.856211`, 0.325421`, 53.620826`, 
              53.634531`, 0.286828`, 16.5242`, 32.960486`, 29.797772`, 
              46.768711`, 53.620826`, 66.960128`, 84.002929`, 87.717231`, 
              20.361826`, 40.474086`, 51.840662`, 59.688989`, 43.430086`, 
              66.534482`, 69.608312`, 75.069794`}, {5, 26, 
              15.17633094`, \(-1.454195447`\), 4.983857092`, 0.0225`, 1, 
              0.007883666104`, \(-0.25`\), 
              39884.07158`, \(-0.2464532948`\), \(-0.6854850374`\), 2, 4, 5, 
              6, 0.261085`, \(-0.469928`\), 0.196457`, 0.206405`, 
              0.08411`, \(-0.09249`\), 0.727015`, 0.573417`, 0.84985`, 
              0.794774`, 1.193435`, 1.225798`, 16.5242`, 16.577509`, 
              1.109865`, 27.280568`, 27.295618`, 0.313328`, 28.987398`, 
              28.95108`, \(-0.756124`\), 40.038825`, 40.046686`, 0.163668`, 
              18.977564`, 19.000987`, 0.487645`, 38.174665`, 38.206737`, 
              0.667732`, 47.353961`, 47.365682`, 0.24403`, 49.443891`, 
              49.455826`, 0.248469`, 16.5242`, 28.987398`, 27.280568`, 
              44.627294`, 49.443891`, 61.580123`, 76.768892`, 81.99751`, 
              18.977564`, 38.174665`, 47.353961`, 54.224247`, 40.038825`, 
              61.084606`, 62.276085`, 68.746941`}, {5, 26, 
              12.92184563`, \(-1.329073748`\), 5.323922023`, 0.0225`, 
              1, \(-0.05680445917`\), \(-0.2`\), 
              21260.81331`, \(-0.4303566426`\), \(-0.7434058382`\), 2, 4, 5, 
              6, 0.272458`, \(-0.464573`\), 0.213888`, 0.193525`, 
              0.116125`, \(-0.125852`\), 0.73042`, 0.608633`, 0.843025`, 
              0.80585`, 1.179347`, 1.201155`, 16.5242`, 16.571551`, 
              1.098948`, 27.391148`, 27.359906`, \(-0.725087`\), 29.586564`, 
              29.597307`, 0.249345`, 41.280238`, 41.28659`, 0.147419`, 
              20.702737`, 20.725669`, 0.532212`, 40.570106`, 40.59973`, 
              0.687543`, 50.851673`, 50.855154`, 0.080792`, 52.189325`, 
              52.208618`, 0.447747`, 16.5242`, 27.391148`, 29.586564`, 
              45.789833`, 52.189325`, 65.146719`, 83.612034`, 83.647129`, 
              20.702737`, 40.570106`, 50.851673`, 56.350485`, 41.280238`, 
              61.144422`, 67.366928`, 67.866666`}, {5, 27, 
              12.24763413`, \(-1.278473216`\), 5.725545682`, 0.0225`, 
              1, \(-0.1020975211`\), \(-0.15`\), 
              8.95484164`, \(-0.6009986882`\), \(-0.8013293416`\), 2, 4, 5, 
              6, 0.28727`, \(-0.469488`\), 0.256343`, 0.179433`, 
              0.143811`, \(-0.150076`\), 0.730792`, 0.645541`, 0.832767`, 
              0.816123`, 1.161301`, 1.170893`, 16.5242`, 16.563745`, 
              1.040693`, 26.263017`, 26.233732`, \(-0.770678`\), 29.176094`, 
              29.188395`, 0.323726`, 43.108001`, 43.112323`, 0.113739`, 
              22.615849`, 22.636683`, 0.548269`, 43.638459`, 43.664089`, 
              0.67449`, 54.862711`, 54.846141`, \(-0.436072`\), 55.497792`, 
              55.530545`, 0.861935`, 16.5242`, 26.263017`, 29.176094`, 
              46.70827`, 55.497792`, 69.410205`, 86.236508`, 89.116016`, 
              22.615849`, 43.638459`, 54.862711`, 58.76067`, 43.108001`, 
              64.095239`, 67.597253`, 71.455117`}, {5, 28, 
              14.24749435`, \(-1.199863832`\), 6.596412808`, 0.0225`, 
              1, \(-0.1387998749`\), \(-0.1`\), 
              4.347204326`, \(-0.9072292786`\), \(-1.058902245`\), 2, 4, 5, 
              6, 0.302904`, \(-0.48298`\), 0.315341`, 0.151872`, 
              0.180075`, \(-0.170049`\), 0.72829`, 0.6846`, 0.78257`, 
              0.810945`, 1.076123`, 1.090975`, 16.5242`, 16.551137`, 
              0.86116`, 26.765732`, 26.746776`, \(-0.606014`\), 28.886444`, 
              28.90169`, 0.48741`, 48.03024`, 48.032808`, 0.082113`, 
              26.334222`, 26.352921`, 0.597789`, 50.124506`, 50.146592`, 
              0.706064`, 62.949757`, 62.960768`, 0.352025`, 62.989321`, 
              62.963717`, \(-0.818528`\), 16.5242`, 26.765732`, 28.886444`, 
              48.897799`, 62.949757`, 78.913401`, 95.05565`, 101.483274`, 
              26.334222`, 50.124506`, 62.989321`, 65.255496`, 48.03024`, 
              68.503525`, 75.290083`, 80.714081`}, {2, 8, 
              76.70327831`, \(-1.112632559`\), 7.238560442`, 0.0225`, 
              1, \(-0.1657`\), \(-0.05`\), 50, \(-1.16256`\), \(-1.22527`\), 
              2, 4, 5, 6, 0.320286`, \(-0.497461`\), 0.368467`, 0.127966`, 
              0.199387`, \(-0.17182`\), 0.724167`, 0.681668`, 0.746677`, 
              0.777715`, 1.01598`, 0.999965`, 16.5242`, 16.530815`, 
              0.245372`, 27.155136`, 27.150804`, \(-0.160716`\), 39.148952`, 
              39.164829`, 0.588965`, 51.178394`, 51.179227`, 0.030907`, 
              29.115446`, 29.129146`, 0.508178`, 55.022813`, 55.039324`, 
              0.612476`, 67.311556`, 67.271959`, \(-1.468866`\), 68.084202`, 
              68.071077`, \(-0.486867`\), 16.5242`, 27.155136`, 39.148952`, 
              52.827765`, 68.084202`, 85.47529`, 100.489069`, 110.213095`, 
              29.115446`, 55.022813`, 67.311556`, 70.210384`, 51.178394`, 
              70.26141`, 79.794742`, 87.235349`}, {5, 28, 
              108.5765528`, \(-1.121227347`\), 9.779327976`, 0.0225`, 
              1, \(-0.176984652`\), 0, 
              2.42209267`, \(-1.249061877`\), \(-1.135068536`\), 2, 4, 5, 6, 
              0.341814`, \(-0.498166`\), 0.375969`, 0.119779`, 
              0.203641`, \(-0.170512`\), 0.721735`, 0.858654`, 0.760652`, 
              0.923758`, 1.04177`, 1.09096`, 16.5242`, 16.52737`, 0.169522`, 
              33.166668`, 33.177192`, 0.562908`, 37.299868`, 
              37.299742`, \(-0.006741`\), 64.462458`, 64.472925`, 0.559832`, 
              39.677747`, 39.685993`, 0.441046`, 76.024927`, 76.037903`, 
              0.694021`, 88.55086`, 88.487343`, \(-3.397083`\), 92.183791`, 
              92.241398`, 3.081029`, 16.5242`, 37.299868`, 33.166668`, 
              64.462458`, 92.183791`, 115.511362`, 134.592497`, 148.268393`, 
              39.677747`, 76.024927`, 88.55086`, 94.882157`, 69.111589`, 
              92.79591`, 105.876323`, 118.050359`}, {2, 6, 
              4.706222998`*^7, \(-0.6243154913`\), \(-88.29434411`\), 
              0.0225`, 1, \(-0.223945`\), 0.05`, 
              50, \(-1.6351`\), \(-1.54482`\), 2, 4, 5, 6, 
              0.343102`, \(-0.576459`\), 0.557651`, 0.077369`, 
              0.200063`, \(-0.126555`\), 0.69934`, 4.198989`, 0.656382`, 
              4.203664`, 0.870979`, 4.217656`, 16.5242`, 16.518192`, 
              2.911915`, \(-271.327165`\), \(-271.33235`\), 
              2.513355`, \(-387.12476`\), \(-415.387703`\), 
              13699.148484`, \(-415.557536`\), \(-415.605379`\), 
              
              23.189469`, \(-350.164746`\), \(-350.148275`\), \(-7.983552`\), \
\(-618.192831`\), \(-618.279941`\), 
              42.222393`, \(-674.171594`\), \(-673.666491`\), \
\(-244.824958`\), \(-758.010491`\), \(-758.480498`\), 227.814047`, 
              16.5242`, \(-271.327165`\), \(-387.12476`\), \(-415.557536`\), \
\(-758.010491`\), \(-921.255131`\), \(-1088.611963`\), \(-1178.842869`\), \
\(-350.164746`\), \(-618.192831`\), \(-674.171594`\), \(-832.593018`\), \
\(-525.900004`\), \(-660.862144`\), \(-831.953632`\), \(-916.677634`\)}, {2, 
              8, 111755.9926`, \(-0.253527188`\), \(-5.225035074`\), 0.0225`, 
              1, \(-0.253067`\), 0.1`, 50, \(-1.87138`\), \(-1.7046`\), 2, 4, 
              5, 6, 0.34946`, \(-0.707082`\), 0.803787`, 0.054856`, 
              0.110837`, 0.016264`, 0.675338`, 0.285541`, 0.598939`, 
              0.332913`, 0.783352`, 0.435999`, 16.5242`, 
              16.525742`, \(-0.045046`\), \(-8.589718`\), \(-8.565437`\), \
\(-0.70937`\), \(-11.438788`\), \(-11.449673`\), 
              0.317994`, \(-15.249348`\), \(-15.267144`\), 
              0.519887`, \(-20.168883`\), \(-20.160667`\), \(-0.240022`\), \
\(-29.809292`\), \(-29.808429`\), \(-0.025201`\), \(-33.854508`\), \
\(-33.838979`\), \(-0.453682`\), \(-39.883407`\), \(-39.911239`\), 0.813135`, 
              16.5242`, \(-11.438788`\), \(-8.589718`\), \(-15.249348`\), \
\(-39.883407`\), \(-41.10069`\), \(-55.287177`\), \(-59.222504`\), \
\(-20.168883`\), \(-29.809292`\), \(-33.854508`\), \(-45.8175`\), \
\(-22.787657`\), \(-23.031014`\), \(-40.971662`\), \(-42.051111`\)}, {5, 29, 
              268455.8884`, \(-0.25`\), \(-3.258475288`\), 0.0225`, 
              1, \(-0.2779742188`\), 
              0.15`, \(-0.2356732596`\), \(-2.095750511`\), \
\(-1.999035782`\), 2, 4, 5, 6, 0.353985`, \(-0.802596`\), 0.898661`, 
              0.033286`, 0.050639`, 0.095862`, 0.643674`, 0.164618`, 
              0.512485`, 0.186246`, 0.647473`, 0.24702`, 16.5242`, 
              16.519007`, 0.095833`, 6.107805`, 6.120942`, \(-0.242463`\), 
              2.593309`, 
              2.611043`, \(-0.327283`\), \(-4.604992`\), \(-4.611239`\), 
              0.115287`, \(-11.968599`\), \(-11.957255`\), \(-0.209364`\), \
\(-12.834778`\), \(-12.838214`\), 
              0.063416`, \(-18.112864`\), \(-18.108745`\), \(-0.076003`\), \
\(-18.714779`\), \(-21.697878`\), 55.053732`, 16.5242`, \(-4.604992`\), 
              6.107805`, 
              2.593309`, \(-21.688359`\), \(-18.714779`\), \(-31.00616`\), \
\(-31.11279`\), \(-11.968599`\), \(-12.834778`\), \(-18.112864`\), \
\(-23.580947`\), \(-8.313172`\), \(-10.335317`\), \(-20.123379`\), \
\(-21.811018`\)}, {5, 26, 23.52789225`, \(-1.499116736`\), 4.349865918`, 
              0.04`, 1, 0.0781305325`, \(-0.35`\), 
              19633.70834`, \(-0.1331611813`\), \(-0.5410003382`\), 2, 4, 5, 
              6, 0.277785`, \(-0.497042`\), 0.175995`, 0.217545`, 
              0.054683`, \(-0.059474`\), 0.753654`, 0.527488`, 0.910321`, 
              0.775077`, 1.2856`, 1.240491`, 16.5242`, 16.583366`, 1.143868`, 
              24.969106`, 24.983042`, 0.269433`, 30.108957`, 
              30.071092`, \(-0.732042`\), 38.567266`, 38.575641`, 0.161907`, 
              17.184519`, 17.205921`, 0.413772`, 35.702934`, 35.734749`, 
              0.615095`, 42.510576`, 42.52383`, 0.256242`, 46.350218`, 
              46.35095`, 0.014149`, 16.5242`, 30.108957`, 24.969106`, 
              43.343396`, 46.350218`, 57.049102`, 72.779754`, 74.665331`, 
              17.184519`, 35.702934`, 42.510576`, 51.198067`, 38.567266`, 
              57.050972`, 60.455107`, 64.444744`}, {5, 28, 
              20.02711043`, \(-1.397155283`\), 4.475810204`, 0.04`, 1, 
              0.00498674048`, \(-0.3`\), 5740.022203`, 
              0.1649836876`, \(-0.7887486003`\), 2, 4, 5, 6, 
              0.29084`, \(-0.472702`\), 0.143708`, 0.222235`, 
              0.040778`, \(-0.044839`\), 0.761548`, 0.55863`, 0.876626`, 
              0.800166`, 1.223019`, 1.266585`, 16.5242`, 16.578635`, 
              1.133759`, 25.964915`, 25.978834`, 0.289899`, 29.069616`, 
              29.031296`, \(-0.798136`\), 39.048946`, 39.056306`, 0.153289`, 
              18.165393`, 18.186225`, 0.433898`, 37.051349`, 37.081482`, 
              0.627614`, 44.129261`, 44.141183`, 0.248309`, 47.71837`, 
              47.721936`, 0.074279`, 16.5242`, 29.069616`, 25.964915`, 
              43.960903`, 47.71837`, 58.815022`, 74.782332`, 76.46151`, 
              18.165393`, 37.051349`, 44.129261`, 52.094408`, 39.048946`, 
              58.272896`, 61.596333`, 64.922095`}, {5, 27, 
              17.39983703`, \(-1.296419103`\), 4.711182955`, 0.04`, 
              1, \(-0.05946385642`\), \(-0.25`\), 
              3797.838901`, \(-0.06717170754`\), \(-0.7508064946`\), 2, 4, 5, 
              6, 0.301633`, \(-0.46414`\), 0.154323`, 0.21899`, 
              0.051679`, \(-0.056756`\), 0.76503`, 0.596366`, 0.887019`, 
              0.823806`, 1.238416`, 1.274471`, 16.5242`, 16.573538`, 
              1.121791`, 27.355391`, 27.367385`, 0.27272`, 28.068595`, 
              28.032475`, \(-0.821249`\), 39.959871`, 39.966072`, 0.140999`, 
              19.46962`, 19.489978`, 0.462866`, 38.900037`, 38.928299`, 
              0.642579`, 46.683768`, 46.693372`, 0.218376`, 49.764464`, 
              49.771406`, 0.157836`, 16.5242`, 28.068595`, 27.355391`, 
              44.980748`, 49.764464`, 61.480614`, 76.244817`, 79.924799`, 
              19.46962`, 38.900037`, 46.683768`, 53.582243`, 39.959871`, 
              60.289279`, 61.331857`, 67.723982`}, {5, 27, 
              14.110022`, \(-1.182570572`\), 5.148272838`, 0.04`, 
              1, \(-0.1167483566`\), \(-0.2`\), 
              2391.367688`, \(-0.4366414081`\), \(-0.8242517066`\), 2, 4, 5, 
              6, 0.310796`, \(-0.472101`\), 0.206889`, 0.201076`, 
              0.096491`, \(-0.10404`\), 0.764396`, 0.637353`, 0.873302`, 
              0.834503`, 1.214966`, 1.237689`, 16.5242`, 16.567491`, 
              1.108296`, 26.995016`, 26.961732`, \(-0.852109`\), 29.990954`, 
              30.002249`, 0.289151`, 41.883581`, 41.888542`, 0.127014`, 
              21.499494`, 21.519653`, 0.51609`, 41.915358`, 41.941817`, 
              0.677364`, 51.071361`, 51.076147`, 0.122528`, 53.327771`, 
              53.339592`, 0.302617`, 16.5242`, 26.995016`, 29.990954`, 
              46.819139`, 53.327771`, 66.118699`, 82.193355`, 86.91506`, 
              21.499494`, 41.915358`, 51.071361`, 56.417028`, 41.883581`, 
              61.526175`, 66.793095`, 69.076961`}, {5, 28, 
              9.955953787`, \(-1.014919203`\), 6.18149902`, 0.04`, 
              1, \(-0.1686706036`\), \(-0.15`\), 
              50.41403142`, \(-0.8568992938`\), \(-1.010097557`\), 2, 4, 5, 
              6, 0.318297`, \(-0.499772`\), 0.305394`, 0.168763`, 
              0.148795`, \(-0.143508`\), 0.759488`, 0.6962`, 0.83479`, 
              0.839507`, 1.151226`, 1.154049`, 16.5242`, 16.559202`, 
              1.101881`, 27.561287`, 27.532349`, \(-0.911`\), 35.231765`, 
              35.245366`, 0.428167`, 47.366075`, 47.369901`, 0.120426`, 
              25.887867`, 25.909116`, 0.668926`, 48.897639`, 48.923674`, 
              0.819617`, 60.996613`, 60.987324`, \(-0.292424`\), 61.994257`, 
              62.012506`, 0.574508`, 16.5242`, 27.561287`, 35.231765`, 
              51.319264`, 61.994257`, 77.276976`, 95.778496`, 98.76186`, 
              25.887867`, 48.897639`, 60.996613`, 64.127194`, 47.366075`, 
              65.454271`, 78.074457`, 79.313665`}, {5, 28, 
              23.01951946`, \(-1.052923238`\), 6.569862753`, 0.04`, 
              1, \(-0.187744898`\), \(-0.1`\), 
              6.689192619`, \(-1.027834252`\), \(-1.147913657`\), 2, 4, 5, 6, 
              0.340334`, \(-0.494163`\), 0.317882`, 0.154667`, 
              0.16414`, \(-0.15078`\), 0.759208`, 0.717598`, 0.811667`, 
              0.838948`, 1.11149`, 1.114607`, 16.5242`, 16.543693`, 
              0.697375`, 27.979694`, 27.965678`, \(-0.50143`\), 33.074748`, 
              33.087896`, 0.470375`, 50.314766`, 50.316648`, 0.067323`, 
              27.84873`, 27.865214`, 0.589726`, 53.121488`, 53.142272`, 
              0.743573`, 64.738581`, 64.727002`, \(-0.414233`\), 66.09277`, 
              66.068398`, \(-0.8719`\), 16.5242`, 27.979694`, 33.074748`, 
              52.298995`, 66.09277`, 82.316033`, 100.457439`, 105.219839`, 
              27.84873`, 53.121488`, 64.738581`, 67.015173`, 50.314766`, 
              68.287755`, 81.249357`, 84.169719`}, {2, 23, 
              96.54642829`, \(-1.001223031`\), 7.374116779`, 0.04`, 
              1, \(-0.2112291247`\), \(-0.05`\), 
              103.7506156`, \(-1.291776461`\), \(-1.277572758`\), 2, 4, 5, 6, 
              0.358086`, \(-0.501817`\), 0.354632`, 0.130873`, 
              0.183228`, \(-0.154715`\), 0.755438`, 0.725671`, 0.782614`, 
              0.81647`, 1.063156`, 1.032787`, 16.5242`, 16.528999`, 
              0.202754`, 30.033619`, 30.030791`, \(-0.119456`\), 41.930592`, 
              41.944663`, 0.594495`, 55.296655`, 55.297301`, 0.027288`, 
              31.412019`, 31.424169`, 0.513312`, 59.877155`, 59.893761`, 
              0.701618`, 71.072086`, 71.044513`, \(-1.164936`\), 73.440118`, 
              73.496361`, 2.376185`, 16.5242`, 30.033619`, 41.930592`, 
              56.50407`, 73.440118`, 91.515513`, 109.69699`, 117.336517`, 
              31.412019`, 59.877155`, 71.072086`, 73.782962`, 55.296655`, 
              73.603655`, 87.765978`, 93.485426`}, {5, 27, 
              239.7708473`, \(-0.9805567241`\), 8.883053678`, 0.04`, 
              1, \(-0.2247570973`\), 0, 
              1438.348287`, \(-1.555009301`\), \(-1.397428329`\), 2, 4, 5, 6, 
              0.377263`, \(-0.502038`\), 0.363534`, 0.106193`, 
              0.199483`, \(-0.158076`\), 0.751874`, 0.749912`, 0.756852`, 
              0.810472`, 1.020291`, 0.964017`, 16.5242`, 16.524544`, 
              0.018461`, 36.337095`, 36.337967`, 0.046761`, 50.343637`, 
              50.360004`, 0.877618`, 63.462286`, 63.463444`, 0.062083`, 
              37.997169`, 38.003802`, 0.355677`, 73.040477`, 73.053391`, 
              0.692447`, 82.9399`, 82.896433`, \(-2.330702`\), 88.161776`, 
              88.348991`, 10.038428`, 16.5242`, 36.337095`, 50.343637`, 
              63.462286`, 88.161776`, 109.705866`, 130.155933`, 139.30537`, 
              37.997169`, 73.040477`, 82.9399`, 88.571012`, 65.964403`, 
              86.22881`, 103.077538`, 112.244546`}, {2, 6, 
              7.188043967`*^6, \(-0.5569964677`\), \(-38.96032531`\), 0.04`, 
              1, \(-0.2692`\), 0.05`, 50, \(-1.81649`\), \(-1.57616`\), 2, 4, 
              5, 6, 0.376526`, \(-0.58396`\), 0.551972`, 0.078392`, 
              0.181793`, \(-0.109215`\), 0.730042`, 2.051329`, 0.700086`, 
              2.061573`, 0.93275`, 2.091976`, 16.5242`, 16.522157`, 
              0.479501`, \(-129.312381`\), \(-129.3142`\), 
              0.42678`, \(-174.670362`\), \(-193.807633`\), 
              4491.772773`, \(-197.4646`\), \(-197.509332`\), 
              10.499065`, \(-162.209455`\), \(-162.203293`\), \(-1.446302`\), \
\(-287.355141`\), \(-287.398177`\), 
              10.101211`, \(-308.94751`\), \(-308.76253`\), \(-43.417209`\), \
\(-352.977154`\), \(-359.211957`\), 1463.391446`, 
              16.5242`, \(-129.312381`\), \(-174.670362`\), \(-197.4646`\), \
\(-352.977154`\), \(-436.07355`\), \(-495.397183`\), \(-548.5037`\), \
\(-162.209455`\), \(-287.355141`\), \(-308.94751`\), \(-377.168584`\), \
\(-245.406497`\), \(-277.926717`\), \(-413.012811`\), \(-441.092533`\)}, {2, 
              7, 120618.5886`, \(-0.2616236816`\), \(-5.15837035`\), 0.04`, 
              1, \(-0.297279`\), 0.1`, 50, \(-2.07718`\), \(-1.7185`\), 2, 4, 
              5, 6, 0.382236`, \(-0.708197`\), 0.787452`, 0.054523`, 
              0.101499`, 0.016625`, 0.708621`, 0.280594`, 0.648041`, 
              0.326542`, 0.853641`, 0.427649`, 16.5242`, 
              16.525171`, \(-0.030619`\), \(-12.986163`\), \(-12.9927`\), 
              0.206209`, \(-13.324282`\), \(-13.339061`\), 
              0.466248`, \(-15.379329`\), \(-15.364318`\), \(-0.47355`\), \
\(-20.897548`\), \(-20.890683`\), \(-0.216568`\), \(-31.361115`\), \
\(-31.359462`\), \(-0.052161`\), \(-35.886032`\), \(-35.873637`\), \
\(-0.391032`\), \(-42.227718`\), \(-42.603421`\), 11.852503`, 
              16.5242`, \(-12.986163`\), \(-13.324282`\), \(-15.379329`\), \
\(-42.227718`\), \(-48.113291`\), \(-51.129936`\), \(-66.605966`\), \
\(-20.897548`\), \(-31.361115`\), \(-35.886032`\), \(-47.215697`\), \
\(-25.515428`\), \(-17.238839`\), \(-48.11228`\), \(-51.191639`\)}, {5, 28, 
              17459.99314`, \(-0.6190330315`\), \(-17.46532799`\), 0.04`, 
              1, \(-0.2693743523`\), 0.15`, 
              194.8478422`, \(-2.752439066`\), \(-4.756565175`\), 2, 4, 5, 6, 
              0.411888`, \(-0.527658`\), 0.379365`, \(-0.08074`\), 
              0.29292`, \(-0.224338`\), 0.719584`, \(-0.632102`\), 
              0.115395`, \(-0.653859`\), \(-0.131811`\), \(-0.716323`\), 
              16.5242`, 16.519818`, 
              0.504426`, \(-62.610583`\), \(-62.633153`\), 
              2.597761`, \(-74.633724`\), \(-74.67644`\), 
              4.916632`, \(-85.040643`\), \(-85.032896`\), \(-0.891614`\), \
\(-72.487671`\), \(-72.45172`\), \(-4.137959`\), \(-125.844874`\), \
\(-125.815187`\), \(-3.416962`\), \(-137.82966`\), \(-137.812273`\), \
\(-2.001223`\), \(-159.343348`\), \(-160.09098`\), 86.05256`, 
              16.5242`, \(-62.610583`\), \(-74.633724`\), \(-85.040643`\), \
\(-159.343348`\), \(-194.055057`\), \(-216.911445`\), \(-248.759993`\), \
\(-72.487671`\), \(-125.844874`\), \(-137.82966`\), \(-167.439069`\), \
\(-110.01656`\), \(-119.763985`\), \(-186.979352`\), \(-198.933702`\)}, {5, 
              27, 30.25118972`, \(-1.275473924`\), 4.208604718`, 0.0625`, 
              1, \(-0.01933165125`\), \(-0.35`\), 
              2050.393823`, \(-0.1983156655`\), \(-0.5786434389`\), 2, 4, 5, 
              6, 0.319797`, \(-0.47066`\), 0.078035`, 0.219469`, 
              0.051747`, \(-0.056425`\), 0.7963`, 0.550372`, 0.955979`, 
              0.792944`, 1.342012`, 1.256874`, 16.5242`, 16.582028`, 
              1.245297`, 25.695507`, 25.70755`, 0.259339`, 29.797413`, 
              29.757198`, \(-0.865993`\), 39.209137`, 39.21646`, 0.157702`, 
              18.201096`, 18.220853`, 0.425458`, 37.193976`, 37.223739`, 
              0.640913`, 42.980869`, 42.992276`, 0.245641`, 47.782966`, 
              47.779547`, \(-0.073624`\), 16.5242`, 29.797413`, 25.695507`, 
              43.933256`, 47.782966`, 58.373443`, 73.758884`, 75.505276`, 
              18.201096`, 37.193976`, 42.980869`, 51.915439`, 39.209137`, 
              57.58732`, 61.135578`, 64.706199`}, {5, 26, 
              55.16029944`, \(-1.017877446`\), 5.136042766`, 0.0625`, 
              1, \(-0.1121265411`\), \(-0.3`\), 
              2142.965445`, \(-0.5103160881`\), \(-0.9609771755`\), 2, 4, 5, 
              6, 0.315333`, \(-0.491381`\), 0.156081`, 0.20419`, 
              0.08558`, \(-0.090191`\), 0.794665`, 0.643218`, 0.890334`, 
              0.841308`, 1.23042`, 1.248579`, 16.5242`, 16.588436`, 1.66456`, 
              31.05595`, 31.070761`, 0.38379`, 31.590636`, 
              31.541788`, \(-1.26579`\), 44.519433`, 44.527766`, 0.215936`, 
              22.308517`, 22.33326`, 0.641177`, 43.394366`, 43.428591`, 
              0.88687`, 52.333091`, 52.344162`, 0.286902`, 56.127556`, 
              56.128244`, 0.017829`, 16.5242`, 31.590636`, 31.05595`, 
              48.956646`, 56.127556`, 69.07835`, 86.3831`, 87.896227`, 
              22.308517`, 43.394366`, 52.333091`, 60.197685`, 44.519433`, 
              67.033971`, 69.265115`, 75.060538`}, {5, 27, 
              19.08044313`, \(-1.170020048`\), 4.518773423`, 0.0625`, 
              1, \(-0.1269241597`\), \(-0.25`\), 1734.046058`, 
              0.248387539`, \(-0.832864827`\), 2, 4, 5, 6, 
              0.343034`, \(-0.461322`\), 0.123151`, 0.2247`, 
              0.036492`, \(-0.040671`\), 0.80007`, 0.615816`, 0.918233`, 
              0.843379`, 1.275202`, 1.300485`, 16.5242`, 16.569416`, 
              1.121413`, 27.50393`, 27.482395`, \(-0.534122`\), 27.544789`, 
              27.540923`, \(-0.095862`\), 40.269022`, 40.273888`, 0.120696`, 
              20.050939`, 20.068663`, 0.43958`, 39.916156`, 39.941458`, 
              0.627533`, 46.483344`, 46.491014`, 0.19024`, 50.484416`, 
              50.48735`, 0.072768`, 16.5242`, 27.50393`, 27.544789`, 
              45.614412`, 50.484416`, 61.939456`, 76.903874`, 78.798124`, 
              20.050939`, 39.916156`, 46.483344`, 53.319206`, 40.269022`, 
              59.943017`, 61.162874`, 67.778385`}, {5, 28, 
              14.25847789`, \(-1.045624186`\), 5.111107346`, 0.0625`, 
              1, \(-0.1798640188`\), \(-0.2`\), 
              1198.876407`, \(-0.5136080677`\), \(-0.9323780238`\), 2, 4, 5, 
              6, 0.35009`, \(-0.479515`\), 0.200746`, 0.203606`, 
              0.087433`, \(-0.092396`\), 0.796596`, 0.672619`, 0.897334`, 
              0.862779`, 1.241188`, 1.261378`, 16.5242`, 16.563002`, 
              1.110884`, 27.170316`, 27.138444`, \(-0.912472`\), 31.108568`, 
              31.118933`, 0.29674`, 43.428435`, 43.43239`, 0.11324`, 
              22.745963`, 22.764033`, 0.517324`, 44.176763`, 44.201352`, 
              0.703972`, 52.441076`, 52.445089`, 0.114871`, 55.653534`, 
              55.661242`, 0.220694`, 16.5242`, 27.170316`, 31.108568`, 
              48.709784`, 55.653534`, 68.591561`, 81.358325`, 90.042039`, 
              22.745963`, 44.176763`, 52.441076`, 57.608756`, 43.428435`, 
              63.510624`, 65.610536`, 73.980646`}, {5, 28, 
              8.929737544`, \(-0.9123728098`\), 6.327285527`, 0.0625`, 
              1, \(-0.2236278422`\), \(-0.15`\), 
              16.97263769`, \(-1.038420724`\), \(-1.161672082`\), 2, 4, 5, 6, 
              0.358614`, \(-0.506335`\), 0.295732`, 0.166015`, 
              0.140128`, \(-0.128459`\), 0.791051`, 0.735264`, 0.851923`, 
              0.865451`, 1.165921`, 1.161146`, 16.5242`, 16.553722`, 
              1.071785`, 29.337764`, 29.313684`, \(-0.874217`\), 36.571774`, 
              36.583472`, 0.424699`, 51.042039`, 51.045055`, 0.109495`, 
              28.142718`, 28.161857`, 0.69483`, 53.369529`, 53.394743`, 
              0.915393`, 64.458304`, 64.453755`, \(-0.165148`\), 66.920368`, 
              66.928755`, 0.304493`, 16.5242`, 29.337764`, 36.571774`, 
              54.930098`, 66.920368`, 82.870912`, 97.521706`, 109.809632`, 
              28.142718`, 53.369529`, 64.458304`, 67.635696`, 51.042039`, 
              70.656371`, 81.374237`, 86.109627`}, {5, 28, 
              18.25811718`, \(-0.9292842469`\), 7.477553579`, 0.0625`, 
              1, \(-0.2415031839`\), \(-0.1`\), 
              4.065686206`, \(-1.264367838`\), \(-1.211104563`\), 2, 4, 5, 6, 
              0.379375`, \(-0.498865`\), 0.304315`, 0.147778`, 
              0.157386`, \(-0.136056`\), 0.789908`, 0.808464`, 0.842178`, 
              0.905846`, 1.149742`, 1.141961`, 16.5242`, 16.543432`, 
              0.872922`, 33.443159`, 33.458297`, 0.687069`, 33.845837`, 
              33.835283`, \(-0.479043`\), 56.987556`, 57.00899`, 0.972824`, 
              33.525927`, 33.542467`, 0.750728`, 64.15438`, 64.177435`, 
              1.04646`, 75.921013`, 75.909423`, \(-0.526018`\), 78.408486`, 
              78.358571`, \(-2.265571`\), 16.5242`, 33.845837`, 33.443159`, 
              56.987556`, 79.15305`, 97.934527`, 117.611196`, 125.91587`, 
              33.525927`, 64.15438`, 75.921013`, 78.408486`, 60.111916`, 
              80.634511`, 96.394543`, 100.552919`}, {2, 8, 
              85.89514759`, \(-0.8833458624`\), 8.068044178`, 0.0625`, 
              1, \(-0.263653`\), \(-0.05`\), 
              50, \(-1.48084`\), \(-1.34904`\), 2, 4, 5, 6, 
              0.396238`, \(-0.502946`\), 0.332706`, 0.129003`, 
              0.17005`, \(-0.137755`\), 0.785838`, 0.798084`, 0.81365`, 
              0.87595`, 1.102497`, 1.070111`, 16.5242`, 16.527995`, 
              0.194114`, 35.158128`, 35.156254`, \(-0.095844`\), 47.444706`, 
              47.458109`, 0.685559`, 63.527938`, 63.528467`, 0.027093`, 
              36.241477`, 36.253157`, 0.597451`, 69.371339`, 69.39001`, 
              0.954984`, 80.251794`, 80.225179`, \(-1.361371`\), 82.763593`, 
              82.729414`, \(-1.748248`\), 16.5242`, 35.158128`, 47.444706`, 
              64.000727`, 84.541893`, 104.624949`, 123.229485`, 135.213536`, 
              36.241477`, 69.371339`, 80.251794`, 82.763593`, 63.527938`, 
              82.049192`, 102.320124`, 107.12124`}, {5, 28, 
              7389.40144`, \(-0.8143526554`\), 11.68008206`, 0.0625`, 
              1, \(-0.2809625822`\), 0, 
              200.432178`, \(-1.982077307`\), \(-1.721015166`\), 2, 4, 5, 6, 
              0.411453`, \(-0.508856`\), 0.35678`, 0.079993`, 
              0.201459`, \(-0.15028`\), 0.779956`, 0.780723`, 0.755148`, 
              0.814239`, 1.003508`, 0.905302`, 16.5242`, 16.52452`, 
              0.024612`, 49.817164`, 49.818099`, 0.071931`, 68.275992`, 
              68.294869`, 1.451555`, 83.227708`, 83.611989`, 29.548518`, 
              52.358337`, 52.366194`, 0.604187`, 99.968456`, 99.988089`, 
              1.509684`, 111.050025`, 110.996888`, \(-4.08582`\), 118.75078`, 
              118.711543`, \(-3.017016`\), 16.5242`, 49.817164`, 68.275992`, 
              83.227708`, 120.593764`, 149.204955`, 169.423314`, 195.666831`, 
              52.358337`, 99.968456`, 111.050025`, 118.75078`, 89.388935`, 
              110.951718`, 144.706434`, 152.901632`}, {2, 6, 
              3.591456048`*^7, \(-0.549407118`\), \(-42.4146353`\), 0.0625`, 
              1, \(-0.315171`\), 0.05`, 50, \(-2.05004`\), \(-1.58203`\), 2, 
              4, 5, 6, 0.414777`, \(-0.566482`\), 0.48678`, 0.074653`, 
              0.172942`, \(-0.104268`\), 0.763431`, 2.20836`, 0.746262`, 
              2.216899`, 0.997399`, 2.242302`, 16.5242`, 16.522127`, 
              0.583543`, \(-157.44731`\), \(-157.448388`\), 
              0.303426`, \(-214.657747`\), \(-230.93905`\), 
              4582.886814`, \(-230.87951`\), \(-240.705191`\), 
              2765.748103`, \(-186.037492`\), \(-186.031795`\), \
\(-1.603683`\), \(-334.752337`\), \(-334.792302`\), 
              11.24953`, \(-357.603941`\), \(-357.436783`\), \(-47.051733`\), \
\(-410.688338`\), \(-411.112753`\), 119.465002`, 
              16.5242`, \(-157.44731`\), \(-214.657747`\), \(-230.87951`\), \
\(-410.688338`\), \(-508.229005`\), \(-521.044944`\), \(-687.930813`\), \
\(-186.037492`\), \(-334.752337`\), \(-357.603941`\), \(-422.599418`\), \
\(-288.991397`\), \(-320.152235`\), \(-488.688072`\), \(-524.437663`\)}, {2, 
              28, 41338.145`, \(-0.4025534166`\), \(-8.916750103`\), 0.0625`, 
              1, \(-0.3335396771`\), 0.1`, 
              1132.987181`, \(-2.856924368`\), \(-3.891096586`\), 2, 4, 5, 6, 
              0.424428`, \(-0.602241`\), 0.544775`, \(-0.012805`\), 
              0.198763`, \(-0.121277`\), 0.750067`, 0.103503`, 0.301988`, 
              0.098607`, 0.20419`, 0.084798`, 16.5242`, 16.523758`, 
              0.02675`, \(-30.168477`\), \(-30.172046`\), 
              0.216152`, \(-32.797807`\), \(-35.530663`\), 
              165.480755`, \(-44.303137`\), \(-44.306264`\), 
              0.189336`, \(-38.440159`\), \(-38.430705`\), \(-0.572418`\), \
\(-63.728022`\), \(-63.720894`\), \(-0.431629`\), \(-70.729044`\), \
\(-70.713571`\), \(-0.936913`\), \(-82.46713`\), \(-82.566499`\), 6.017067`, 
              16.5242`, \(-30.168477`\), \(-32.797807`\), \(-44.303137`\), \
\(-82.46713`\), \(-100.746744`\), \(-94.43185`\), \(-140.996545`\), \
\(-38.440159`\), \(-63.728022`\), \(-70.729044`\), \(-86.771198`\), \
\(-55.098584`\), \(-54.546333`\), \(-95.858995`\), \(-104.10299`\)}, {5, 27, 
              114.6886096`, \(-1.558770633`\), 4.749195308`, 0.0625`, 
              1, \(-0.2082420375`\), 0.15`, 
              21127.20998`, \(-0.4619022285`\), \(-0.6030423871`\), 2, 4, 5, 
              6, 0.491038`, \(-0.420417`\), 0.118481`, 0.209469`, 
              0.072975`, \(-0.077629`\), 0.796682`, 0.746752`, 0.952855`, 
              0.926211`, 1.336968`, 1.318177`, 16.5242`, 16.529706`, 
              0.205458`, 28.263121`, 28.266882`, 0.140314`, 29.135517`, 
              29.136153`, 0.023743`, 43.864231`, 43.864639`, 0.015221`, 
              22.129383`, 22.123776`, \(-0.209228`\), 48.637443`, 
              48.637191`, \(-0.009407`\), 49.046665`, 
              49.03967`, \(-0.261001`\), 51.660503`, 
              51.633924`, \(-0.991736`\), 16.5242`, 28.263121`, 29.135517`, 
              47.193284`, 55.400658`, 67.232037`, 80.298988`, 86.715682`, 
              22.129383`, 48.637443`, 49.046665`, 51.660503`, 43.864231`, 
              59.270334`, 63.991502`, 69.314635`}, {5, 28, 
              195.8741992`, \(-0.8318920673`\), 5.549970821`, 0.09`, 
              1, \(-0.1646340068`\), \(-0.35`\), 350.0343487`, 
              1.061950459`, \(-1.162019447`\), 2, 4, 5, 6, 
              0.332878`, \(-0.529453`\), 0.172447`, 0.194877`, 
              0.080697`, \(-0.077811`\), 0.825217`, 0.680411`, 0.897978`, 
              0.853219`, 1.229367`, 1.224668`, 16.5242`, 16.60405`, 
              2.360313`, 34.878722`, 34.896331`, 0.520498`, 35.39952`, 
              35.336803`, \(-1.853882`\), 49.521981`, 49.532314`, 0.305445`, 
              25.334675`, 25.363197`, 0.843081`, 48.321611`, 48.361776`, 
              1.187248`, 58.129788`, 58.142099`, 0.363892`, 62.958337`, 
              62.947736`, \(-0.313365`\), 16.5242`, 35.39952`, 34.878722`, 
              53.733644`, 62.958337`, 77.183658`, 97.220243`, 97.994725`, 
              25.334675`, 48.321611`, 58.129788`, 67.197252`, 49.521981`, 
              74.20473`, 78.427573`, 78.465976`}, {5, 28, 
              29.24473859`, \(-1.047186325`\), 4.403795834`, 0.09`, 
              1, \(-0.1633980148`\), \(-0.3`\), 400.3323828`, 
              0.07605003042`, \(-1.025466363`\), 2, 4, 5, 6, 
              0.369519`, \(-0.463135`\), 0.067218`, 0.227694`, 
              0.032551`, \(-0.036277`\), 0.833192`, 0.628533`, 0.929879`, 
              0.856649`, 1.279758`, 1.317529`, 16.5242`, 16.575389`, 
              1.332786`, 28.18462`, 28.195453`, 0.282049`, 28.879797`, 
              28.838229`, \(-1.082297`\), 41.497078`, 41.502467`, 0.140336`, 
              20.640974`, 20.658907`, 0.466916`, 41.037582`, 41.064571`, 
              0.702708`, 46.728471`, 46.736755`, 0.215673`, 51.937021`, 
              51.931703`, \(-0.138468`\), 16.5242`, 28.879797`, 28.18462`, 
              46.622211`, 51.937021`, 63.273751`, 79.400268`, 80.912046`, 
              20.640974`, 41.037582`, 46.728471`, 54.641329`, 41.497078`, 
              60.918197`, 64.460841`, 64.790202`}, {5, 27, 
              19.99020407`, \(-1.036702759`\), 4.492185206`, 0.09`, 
              1, \(-0.1998918519`\), \(-0.25`\), 
              1962.655141`, \(-0.1103280359`\), \(-0.9987849613`\), 2, 4, 5, 
              6, 0.384143`, \(-0.461683`\), 0.10007`, 0.225169`, 
              0.039298`, \(-0.043263`\), 0.833029`, 0.650681`, 0.933927`, 
              0.870299`, 1.286713`, 1.320365`, 16.5242`, 16.565736`, 
              1.146824`, 27.427803`, 27.392758`, \(-0.967616`\), 28.771592`, 
              28.781155`, 0.264056`, 41.679334`, 41.683313`, 0.109846`, 
              21.218524`, 21.234546`, 0.442386`, 42.026425`, 42.050373`, 
              0.661219`, 47.74632`, 47.752435`, 0.168828`, 52.657059`, 
              52.655095`, \(-0.05424`\), 16.5242`, 27.427803`, 28.771592`, 
              47.160553`, 52.657059`, 64.239373`, 79.997953`, 80.815134`, 
              21.218524`, 42.026425`, 47.74632`, 54.466633`, 41.679334`, 
              61.170399`, 62.857998`, 67.773777`}, {5, 28, 
              13.382742`, \(-0.921156899`\), 5.223215611`, 0.09`, 
              1, \(-0.2467224684`\), \(-0.2`\), 
              1149.379991`, \(-0.6922170151`\), \(-1.086375656`\), 2, 4, 5, 
              6, 0.390875`, \(-0.484528`\), 0.187373`, 0.20027`, 
              0.087816`, \(-0.088718`\), 0.828141`, 0.714643`, 0.91379`, 
              0.889886`, 1.254865`, 1.26896`, 16.5242`, 16.557903`, 
              1.100958`, 28.220984`, 28.19249`, \(-0.930798`\), 33.10975`, 
              33.119066`, 0.304312`, 46.23158`, 46.234822`, 0.105892`, 
              24.636112`, 24.652685`, 0.541395`, 47.745583`, 47.769696`, 
              0.787644`, 55.242722`, 55.245799`, 0.100529`, 59.6036`, 
              59.604318`, 0.023443`, 16.5242`, 28.220984`, 33.10975`, 
              51.669505`, 59.6036`, 73.04701`, 87.296835`, 91.893081`, 
              24.636112`, 47.745583`, 55.242722`, 60.298133`, 46.23158`, 
              67.497636`, 67.432814`, 79.747361`}, {2, 26, 
              11.12250316`, \(-0.797802699`\), 6.70496231`, 0.09`, 
              1, \(-0.2865557795`\), \(-0.15`\), 
              86.07868147`, \(-1.349621388`\), \(-1.455285002`\), 2, 4, 5, 6, 
              0.398789`, \(-0.513693`\), 0.285264`, 0.153425`, 
              0.138899`, \(-0.116323`\), 0.82173`, 0.761054`, 0.845384`, 
              0.869654`, 1.140031`, 1.12638`, 16.5242`, 16.547425`, 
              0.993607`, 32.114905`, 32.095568`, \(-0.827277`\), 41.659848`, 
              41.67092`, 0.473702`, 56.379158`, 56.381678`, 0.107791`, 
              31.493997`, 31.512012`, 0.77072`, 59.732404`, 59.758641`, 
              1.122449`, 70.245567`, 70.241096`, \(-0.191295`\), 73.272843`, 
              73.251868`, \(-0.89736`\), 16.5242`, 32.114905`, 41.659848`, 
              60.592027`, 74.280509`, 91.457503`, 104.36193`, 115.351589`, 
              31.493997`, 59.732404`, 70.245567`, 73.272843`, 56.379158`, 
              78.745148`, 82.769`, 99.7491`}, {5, 28, 
              14.01795775`, \(-0.8458491567`\), 7.878591992`, 0.09`, 
              1, \(-0.2976602321`\), \(-0.1`\), 
              4.260796122`, \(-1.553900326`\), \(-1.405643855`\), 2, 4, 5, 6, 
              0.421854`, \(-0.491458`\), 0.259906`, 0.137314`, 
              0.153449`, \(-0.125035`\), 0.821801`, 0.831675`, 0.853766`, 
              0.913529`, 1.154661`, 1.118374`, 16.5242`, 16.540265`, 
              0.854321`, 36.671969`, 36.689608`, 0.937977`, 38.183436`, 
              38.173235`, \(-0.542489`\), 61.812144`, 61.831749`, 1.042555`, 
              37.315654`, 37.330813`, 0.806132`, 72.028458`, 72.052932`, 
              1.30149`, 82.501499`, 82.492142`, \(-0.497606`\), 85.180924`, 
              85.157419`, \(-1.249983`\), 16.5242`, 38.183436`, 36.671969`, 
              61.812144`, 88.048364`, 108.133063`, 122.378521`, 136.808505`, 
              37.315654`, 72.028458`, 82.501499`, 85.180924`, 66.868298`, 
              91.497753`, 98.982439`, 116.185912`}, {2, 8, 
              86.14123258`, \(-0.7989032884`\), 8.732167911`, 0.09`, 
              1, \(-0.317533`\), \(-0.05`\), 
              50, \(-1.71976`\), \(-1.39799`\), 2, 4, 5, 6, 
              0.437306`, \(-0.493592`\), 0.283398`, 0.123335`, 
              0.160316`, \(-0.124182`\), 0.81733`, 0.85101`, 0.849627`, 
              0.91666`, 1.150087`, 1.086139`, 16.5242`, 16.527238`, 
              0.185626`, 41.004099`, 41.002864`, \(-0.075488`\), 53.421227`, 
              53.433947`, 0.777135`, 72.466523`, 72.466948`, 0.026002`, 
              41.319526`, 41.330525`, 0.672038`, 79.732462`, 79.753993`, 
              1.3155`, 89.91512`, 89.891679`, \(-1.432213`\), 91.823575`, 
              91.805835`, \(-1.083882`\), 16.5242`, 41.004099`, 53.421227`, 
              72.726174`, 96.48423`, 118.517542`, 131.040137`, 149.058573`, 
              41.319526`, 79.732462`, 89.91512`, 91.823575`, 72.466523`, 
              96.072053`, 109.766037`, 124.848353`}, {5, 28, 
              128.5835457`, \(-0.8043580549`\), 10.4091602`, 0.09`, 
              1, \(-0.3272886546`\), 0, 
              62.10515044`, \(-2.054714332`\), \(-1.810526238`\), 2, 4, 5, 6, 
              0.456114`, \(-0.480443`\), 0.262988`, 0.093483`, 
              0.181806`, \(-0.137482`\), 0.814662`, 0.819432`, 0.777126`, 
              0.860882`, 1.024752`, 0.972323`, 16.5242`, 16.524474`, 
              0.020779`, 49.792249`, 49.793018`, 0.058423`, 63.437438`, 
              63.452508`, 1.144851`, 82.892016`, 82.892938`, 0.070003`, 
              49.335541`, 49.341424`, 0.446937`, 96.372642`, 96.391897`, 
              1.462713`, 105.161766`, 105.124602`, \(-2.823177`\), 
              108.607531`, 108.593987`, \(-1.02887`\), 16.5242`, 49.792249`, 
              63.437438`, 82.892016`, 115.097804`, 141.061063`, 154.806199`, 
              176.341626`, 49.335541`, 96.372642`, 105.161766`, 108.607531`, 
              86.116602`, 112.160687`, 131.548885`, 146.764394`}, {2, 23, 
              15929.76771`, \(-0.598871862`\), \(-18.5560841`\), 0.09`, 
              1, \(-0.3599316909`\), 
              0.05`, \(-0.2164005685`\), \(-5.657148927`\), \
\(-7.677147057`\), 2, 4, 5, 6, 0.45956`, \(-0.519532`\), 
              0.35279`, \(-0.349492`\), 0.512126`, \(-0.496966`\), 
              0.800488`, \(-3.923759`\), \(-0.3703`\), \(-4.008982`\), \
\(-1.1072`\), \(-4.255524`\), 16.5242`, 
              16.535769`, \(-1.578525`\), \(-20.464906`\), \(-20.453008`\), \
\(-1.623352`\), \(-79.931583`\), \(-79.93184`\), 
              0.035075`, \(-119.177993`\), \(-119.194622`\), 
              2.268958`, \(-86.211772`\), \(-86.209671`\), \(-0.286561`\), \
\(-160.416196`\), \(-160.430236`\), 
              1.915661`, \(-170.424193`\), \(-170.358911`\), \(-8.907248`\), \
\(-190.44099`\), \(-190.41876`\), \(-3.033144`\), 
              16.5242`, \(-20.464906`\), \(-79.931583`\), \(-119.177993`\), \
\(-194.848116`\), \(-239.433888`\), \(-246.243618`\), \(-302.493205`\), \
\(-86.211772`\), \(-160.416196`\), \(-170.424193`\), \(-190.44099`\), \
\(-140.117276`\), \(-171.239844`\), \(-219.213621`\), \(-248.332469`\)}, {2, 
              13, 26866.35127`, \(-0.4604488142`\), \(-11.61906771`\), 0.09`, 
              1, \(-0.383245754`\), 0.1`, 
              5.367881865`, \(-2.984980682`\), \(-3.809145763`\), 2, 4, 5, 6, 
              0.467179`, \(-0.547806`\), 0.399823`, \(-0.001327`\), 
              0.197542`, \(-0.131355`\), 0.787789`, 0.170464`, 0.386924`, 
              0.170718`, 0.340766`, 0.171093`, 16.5242`, 16.522308`, 
              0.164293`, \(-45.926955`\), \(-45.923902`\), \(-0.265192`\), \
\(-49.611267`\), \(-49.616664`\), 
              0.468676`, \(-62.317135`\), \(-62.327923`\), 
              0.936977`, \(-52.999345`\), \(-52.989155`\), \(-0.885011`\), \
\(-91.497887`\), \(-91.486339`\), \(-1.002946`\), \(-100.994309`\), \
\(-100.978606`\), \(-1.363816`\), \(-116.982477`\), \(-117.02243`\), 
              3.469975`, 
              16.5242`, \(-45.926955`\), \(-49.611267`\), \(-62.317135`\), \
\(-116.982477`\), \(-143.299491`\), \(-139.752843`\), \(-181.559028`\), \
\(-52.999345`\), \(-91.497887`\), \(-100.994309`\), \(-117.231424`\), \
\(-80.745796`\), \(-93.15898`\), \(-126.989846`\), \(-149.392181`\)}, {2, 8, 
              194211.5421`, \(-0.2570801776`\), \(-4.01692299`\), 0.09`, 
              1, \(-0.406182`\), 0.15`, 50, \(-2.95176`\), \(-1.7269`\), 2, 
              4, 5, 6, 0.469967`, \(-0.65174`\), 0.571891`, 0.009659`, 
              0.08472`, 0.011101`, 0.767747`, 0.115351`, 0.737356`, 
              0.123631`, 0.983443`, 0.141802`, 16.5242`, 
              16.525156`, \(-0.028869`\), \(-9.44405`\), \(-9.444`\), \
\(-0.00151`\), \(-12.578696`\), \(-12.586955`\), 
              0.249472`, \(-18.219325`\), \(-18.214478`\), \(-0.146376`\), \
\(-17.757017`\), \(-17.749939`\), \(-0.21379`\), \(-25.665166`\), \
\(-25.661787`\), \(-0.102075`\), \(-32.023678`\), \(-32.020706`\), \
\(-0.089779`\), \(-37.10511`\), \(-37.114678`\), 0.289`, 
              16.5242`, \(-12.578696`\), \(-9.44405`\), \(-18.219325`\), \
\(-37.10511`\), \(-44.676782`\), \(-38.166266`\), \(-55.246943`\), \
\(-17.757017`\), \(-25.665166`\), \(-32.023678`\), \(-38.652049`\), \
\(-22.918941`\), \(-22.622207`\), \(-34.522278`\), \(-46.125339`\)}, {5, 28, 
              250.4055322`, \(-0.7459099514`\), 5.569967307`, 0.1225`, 
              1, \(-0.2469966949`\), \(-0.35`\), 
              374.3591314`, \(-0.878144637`\), \(-1.244388584`\), 2, 4, 5, 6, 
              0.375602`, \(-0.539405`\), 0.163002`, 0.202717`, 
              0.077309`, \(-0.075229`\), 0.859301`, 0.747273`, 0.949824`, 
              0.917263`, 1.3023`, 1.292167`, 16.5242`, 16.599493`, 2.520325`, 
              35.494405`, 35.435403`, \(-1.975019`\), 36.518109`, 36.529251`, 
              0.37297`, 51.732793`, 51.741373`, 0.287199`, 27.092737`, 
              27.11869`, 0.868727`, 51.444564`, 51.483401`, 1.300004`, 
              60.245531`, 60.25548`, 0.333017`, 66.249714`, 
              66.226935`, \(-0.762469`\), 16.5242`, 35.494405`, 36.518109`, 
              56.54591`, 66.249714`, 80.822983`, 101.101487`, 102.553949`, 
              27.092737`, 51.444564`, 60.245531`, 69.208829`, 51.732793`, 
              76.538896`, 80.565136`, 81.272089`}, {5, 27, 
              33.99544929`, \(-0.9899932195`\), 4.141292398`, 0.1225`, 
              1, \(-0.232688584`\), \(-0.3`\), 1212.816527`, 
              0.781212405`, \(-0.8212466505`\), 2, 4, 5, 6, 
              0.418621`, \(-0.447266`\), 0.005262`, 0.215614`, 
              0.051056`, \(-0.054014`\), 0.868799`, 0.608625`, 1.007351`, 
              0.82646`, 1.394537`, 1.264646`, 16.5242`, 16.568586`, 
              1.231177`, 27.797071`, 27.804774`, 0.213646`, 28.096196`, 
              28.059598`, \(-1.015167`\), 41.308081`, 41.312132`, 0.112364`, 
              20.713411`, 20.728274`, 0.412278`, 41.517404`, 41.54241`, 
              0.693598`, 45.435977`, 45.441554`, 0.154682`, 51.772096`, 
              51.758995`, \(-0.363389`\), 16.5242`, 28.096196`, 27.797071`, 
              46.505915`, 51.772096`, 62.616142`, 78.017878`, 79.622971`, 
              20.713411`, 41.517404`, 45.435977`, 53.407848`, 41.308081`, 
              59.497894`, 63.68931`, 63.837397`}, {5, 29, 
              20.26491933`, \(-0.9326134347`\), 4.487498713`, 0.1225`, 
              1, \(-0.2730340158`\), \(-0.25`\), 1240.775591`, 
              0.07271680645`, \(-1.074114545`\), 2, 4, 5, 6, 
              0.428567`, \(-0.456622`\), 0.062292`, 0.230043`, 
              0.028652`, \(-0.03227`\), 0.866021`, 0.693474`, 0.964961`, 
              0.907677`, 1.324361`, 1.357143`, 16.5242`, 16.560728`, 
              1.124001`, 27.931357`, 27.900618`, \(-0.945865`\), 29.942873`, 
              29.950171`, 0.224572`, 43.482024`, 43.485248`, 0.099228`, 
              22.471062`, 22.485396`, 0.441086`, 44.564888`, 44.588956`, 
              0.740603`, 49.122557`, 49.126532`, 0.122323`, 55.220722`, 
              55.20163`, \(-0.587468`\), 16.5242`, 27.931357`, 29.942873`, 
              48.986118`, 55.220722`, 66.971425`, 83.552234`, 84.015789`, 
              22.471062`, 44.564888`, 49.122557`, 55.898707`, 43.482024`, 
              62.71769`, 65.583115`, 67.044277`}, {5, 27, 
              14.8144333`, \(-0.822288876`\), 5.417842909`, 0.1225`, 
              1, \(-0.3160995738`\), \(-0.2`\), 
              909.5338314`, \(-0.9321705316`\), \(-1.281588092`\), 2, 4, 5, 
              6, 0.434588`, \(-0.482198`\), 0.153468`, 0.193668`, 
              0.089782`, \(-0.084831`\), 0.860193`, 0.75774`, 0.925319`, 
              0.914648`, 1.259726`, 1.265996`, 16.5242`, 16.552407`, 
              1.062634`, 30.155754`, 30.131739`, \(-0.904675`\), 35.702888`, 
              35.711209`, 0.313483`, 50.078712`, 50.081392`, 0.100979`, 
              27.002546`, 27.017811`, 0.575087`, 52.488823`, 52.514306`, 
              0.959988`, 58.904168`, 58.905737`, 0.059112`, 64.100054`, 
              64.07336`, \(-1.005644`\), 16.5242`, 30.155754`, 35.702888`, 
              55.518162`, 64.845724`, 78.998518`, 95.763412`, 98.313491`, 
              27.002546`, 52.488823`, 58.904168`, 64.100054`, 50.078712`, 
              72.371177`, 72.584273`, 78.468729`}, {2, 7, 
              19.48096544`, \(-0.7950575403`\), 6.166832709`, 0.1225`, 
              1, \(-0.340104`\), \(-0.15`\), 
              50, \(-1.35005`\), \(-1.35924`\), 2, 4, 5, 6, 
              0.450036`, \(-0.48147`\), 0.181739`, 0.166663`, 
              0.117234`, \(-0.099653`\), 0.857323`, 0.782633`, 0.909536`, 
              0.902221`, 1.234167`, 1.183456`, 16.5242`, 16.541203`, 
              0.75501`, 32.715747`, 32.702028`, \(-0.609187`\), 39.917138`, 
              39.925664`, 0.378611`, 55.992597`, 55.994341`, 0.077414`, 
              30.771584`, 30.785515`, 0.61858`, 59.84746`, 59.872665`, 
              1.119202`, 66.772734`, 66.770334`, \(-0.106561`\), 70.727801`, 
              70.707317`, \(-0.90956`\), 16.5242`, 32.715747`, 39.917138`, 
              60.630535`, 73.170765`, 89.206072`, 105.477058`, 111.29161`, 
              30.771584`, 59.84746`, 66.772734`, 70.727801`, 55.992597`, 
              79.918427`, 79.239116`, 90.547604`}, {5, 28, 
              28.13177892`, \(-0.7859904627`\), 7.851847232`, 0.1225`, 
              1, \(-0.3575056012`\), \(-0.1`\), 
              5.678619348`, \(-1.816798904`\), \(-1.765606031`\), 2, 4, 5, 6, 
              0.467548`, \(-0.473011`\), 0.186859`, 0.12998`, 
              0.147003`, \(-0.11517`\), 0.854885`, 0.8234`, 0.841163`, 
              0.897352`, 1.116635`, 1.084909`, 16.5242`, 16.535744`, 
              0.678052`, 40.928864`, 40.943411`, 0.854445`, 41.454891`, 
              41.448034`, \(-0.40273`\), 66.1356`, 66.152805`, 1.010588`, 
              39.244708`, 39.257761`, 0.766725`, 76.895744`, 76.92398`, 
              1.658519`, 84.545715`, 84.536491`, \(-0.54176`\), 87.817017`, 
              87.794588`, \(-1.317411`\), 16.5242`, 41.454891`, 40.928864`, 
              66.1356`, 92.826574`, 113.100622`, 131.017114`, 141.334709`, 
              39.244708`, 76.895744`, 84.545715`, 87.817017`, 70.586443`, 
              98.905199`, 98.501427`, 118.367073`}, {5, 28, 
              37.01794732`, \(-0.9485364531`\), 8.58177982`, 0.1225`, 
              1, \(-0.3455032908`\), \(-0.05`\), 
              2.244805211`, \(-1.928389555`\), \(-1.829003627`\), 2, 4, 5, 6, 
              0.499368`, \(-0.426734`\), 0.095116`, 0.122336`, 
              0.161389`, \(-0.133446`\), 0.860371`, 0.881279`, 0.837739`, 
              0.943508`, 1.107641`, 1.105952`, 16.5242`, 16.536218`, 
              0.824043`, 35.854587`, 35.863024`, 0.578492`, 49.215221`, 
              49.213348`, \(-0.128408`\), 66.419667`, 66.431563`, 0.815654`, 
              43.558195`, 43.566617`, 0.577469`, 89.649305`, 89.678622`, 
              2.010138`, 92.778675`, 92.763778`, \(-1.021427`\), 98.460854`, 
              98.437056`, \(-1.631822`\), 16.5242`, 35.854587`, 49.215221`, 
              66.419667`, 105.200743`, 127.284132`, 150.592647`, 158.02771`, 
              43.558195`, 89.649305`, 92.778675`, 98.460854`, 81.313173`, 
              112.928638`, 113.433791`, 136.249586`}, {5, 28, 
              128.6698981`, \(-0.8230100292`\), 8.408785385`, 0.1225`, 
              1, \(-0.3750560212`\), 0, 
              257.3116851`, \(-2.05105049`\), \(-1.974992639`\), 2, 4, 5, 6, 
              0.506069`, \(-0.438362`\), 0.134914`, 0.111864`, 
              0.160041`, \(-0.123863`\), 0.851646`, 0.808493`, 0.802895`, 
              0.866183`, 1.051472`, 1.016285`, 16.5242`, 16.524392`, 
              0.013094`, 45.618028`, 45.618516`, 0.033181`, 54.284709`, 
              54.295558`, 0.738698`, 75.85857`, 75.85856`, \(-0.000735`\), 
              42.222979`, 42.226922`, 0.268455`, 85.683138`, 85.706062`, 
              1.560834`, 89.016844`, 88.995474`, \(-1.455005`\), 91.441014`, 
              91.423408`, \(-1.198769`\), 16.5242`, 45.618028`, 54.284709`, 
              75.927606`, 100.149482`, 121.456193`, 139.057894`, 148.70697`, 
              42.222979`, 85.683138`, 89.016844`, 91.441014`, 75.85857`, 
              103.509426`, 104.017143`, 131.891179`}, {5, 27, 
              4300.243371`, \(-0.6271182158`\), \(-32.87151483`\), 0.1225`, 
              1, \(-0.410714451`\), 0.05`, 
              0.2778012723`, \(-3.496156882`\), \(-5.494332974`\), 2, 4, 5, 
              6, 0.506887`, \(-0.47172`\), 0.21302`, \(-0.032512`\), 
              0.258233`, \(-0.215666`\), 0.837566`, \(-0.511879`\), 
              0.156937`, \(-0.516156`\), \(-0.112101`\), \(-0.528847`\), 
              16.5242`, 
              16.539608`, \(-4.107565`\), \(-56.735799`\), \(-56.714187`\), \
\(-5.761444`\), \(-161.181607`\), \(-161.181946`\), 
              0.09054`, \(-224.851402`\), \(-224.877383`\), 
              6.926458`, \(-161.547912`\), \(-161.544694`\), \(-0.857903`\), \
\(-310.65043`\), \(-310.671435`\), 
              5.599687`, \(-327.827047`\), \(-327.715502`\), \(-29.737318`\), \
\(-343.481624`\), \(-343.488455`\), 1.82123`, 
              16.5242`, \(-56.735799`\), \(-161.181607`\), \(-224.851402`\), \
\(-372.146796`\), \(-453.046851`\), \(-492.295219`\), \(-542.909753`\), \
\(-161.547912`\), \(-310.65043`\), \(-327.827047`\), \(-343.481624`\), \
\(-271.751109`\), \(-356.43425`\), \(-369.79363`\), \(-493.116031`\)}, {5, 
              27, 11184.43927`, \(-0.548542909`\), \(-22.57323948`\), 
              0.1225`, 1, \(-0.429970337`\), 0.1`, 
              2.664370117`, \(-1.433009546`\), \(-4.597714413`\), 2, 4, 5, 6, 
              0.51653`, \(-0.474102`\), 0.208459`, 0.161839`, 
              0.094204`, \(-0.055799`\), 0.828583`, 2.716342`, 0.31346`, 
              2.747373`, 0.184537`, 2.838362`, 16.5242`, 16.518295`, 
              1.101522`, \(-69.927567`\), \(-69.894102`\), \(-6.242954`\), \
\(-107.137423`\), \(-107.13443`\), \(-0.558239`\), \(-150.77943`\), \
\(-150.794983`\), 
              2.901434`, \(-109.508395`\), \(-109.492231`\), \(-3.015531`\), \
\(-200.303815`\), \(-200.276352`\), \(-5.123368`\), \(-217.21263`\), \
\(-217.1845`\), \(-5.247686`\), \(-235.140208`\), \(-235.131509`\), \
\(-1.622835`\), 
              16.5242`, \(-107.137423`\), \(-69.927567`\), \(-150.77943`\), \
\(-249.454033`\), \(-303.271472`\), \(-321.910102`\), \(-355.74155`\), \
\(-109.508395`\), \(-200.303815`\), \(-217.21263`\), \(-235.140208`\), \
\(-178.026789`\), \(-228.781342`\), \(-240.593677`\), \(-329.124347`\)}, {2, 
              24, 25566.65145`, \(-0.5049158009`\), \(-12.03378311`\), 
              0.1225`, 1, \(-0.4396818034`\), 0.15`, 
              167.8430283`, \(-3.271612397`\), \(-3.844868656`\), 2, 4, 5, 6, 
              0.527594`, \(-0.460284`\), 0.157368`, \(-0.004603`\), 
              0.196436`, \(-0.141604`\), 0.820691`, 0.140001`, 0.457794`, 
              0.139012`, 0.452769`, 0.136021`, 16.5242`, 16.521209`, 
              0.303879`, \(-55.84061`\), \(-55.8479`\), 
              0.740546`, \(-66.204336`\), \(-66.203569`\), \(-0.077865`\), \
\(-70.026536`\), \(-70.036042`\), 
              0.965648`, \(-57.702459`\), \(-57.686421`\), \(-1.629155`\), \
\(-100.396185`\), \(-100.377191`\), \(-1.929444`\), \(-115.403924`\), \
\(-115.408501`\), 
              0.46497`, \(-123.308735`\), \(-123.288015`\), \(-2.104753`\), 
              16.5242`, \(-55.84061`\), \(-66.204336`\), \(-70.026536`\), \
\(-130.832458`\), \(-158.502509`\), \(-166.530457`\), \(-182.019632`\), \
\(-57.702459`\), \(-100.396185`\), \(-115.403924`\), \(-123.308735`\), \
\(-91.600476`\), \(-116.667405`\), \(-122.978651`\), \(-171.00608`\)}, {5, 
              27, 54.38973181`, \(-0.8627025607`\), 4.235857358`, 0.16`, 
              1, \(-0.2912493057`\), \(-0.35`\), 1372.080853`, 
              0.2234122357`, \(-0.9626949275`\), 2, 4, 5, 6, 
              0.446006`, \(-0.455082`\), \(-0.045094`\), 0.231796`, 
              0.025421`, \(-0.02915`\), 0.901734`, 0.670835`, 1.027457`, 
              0.892785`, 1.413153`, 1.351962`, 16.5242`, 16.57561`, 1.55401`, 
              29.540679`, 29.547959`, 0.220034`, 30.236252`, 
              30.193962`, \(-1.278324`\), 44.001377`, 44.006064`, 0.141691`, 
              22.267081`, 22.282886`, 0.477729`, 44.493852`, 44.527315`, 
              1.01149`, 47.519032`, 47.519652`, 0.01877`, 55.315711`, 
              55.28292`, \(-0.991192`\), 16.5242`, 30.236252`, 29.540679`, 
              49.03582`, 55.315711`, 66.574851`, 82.46563`, 83.595625`, 
              22.267081`, 44.493852`, 47.519032`, 56.745606`, 44.001377`, 
              62.724144`, 68.16533`, 68.482724`}, {5, 27, 
              34.88784785`, \(-0.8878458468`\), 4.17485577`, 0.16`, 
              1, \(-0.3152633678`\), \(-0.3`\), 1521.352211`, 
              0.4072113549`, \(-0.8511921459`\), 2, 4, 5, 6, 
              0.464598`, \(-0.439678`\), \(-0.040909`\), 0.229555`, 
              0.028839`, \(-0.032663`\), 0.902271`, 0.670591`, 1.044629`, 
              0.88973`, 1.441432`, 1.343803`, 16.5242`, 16.564168`, 1.24037`, 
              28.68515`, 28.651579`, \(-1.041848`\), 29.150638`, 29.156916`, 
              0.194836`, 43.319203`, 43.322582`, 0.104866`, 22.096141`, 
              22.109654`, 0.419351`, 44.597486`, 44.629126`, 0.981927`, 
              46.792425`, 46.789918`, \(-0.077801`\), 54.638849`, 
              54.58767`, \(-1.588294`\), 16.5242`, 28.68515`, 29.150638`, 
              48.540542`, 54.638849`, 65.738364`, 81.416176`, 82.575022`, 
              22.096141`, 44.597486`, 46.792425`, 55.181801`, 43.319203`, 
              61.398919`, 66.210961`, 66.843177`}, {5, 26, 
              23.54076314`, \(-0.8440546484`\), 4.551320214`, 0.16`, 
              1, \(-0.3493200712`\), \(-0.25`\), 5374.110113`, 
              0.01469182499`, \(-1.133814139`\), 2, 4, 5, 6, 
              0.475669`, \(-0.4458`\), 0.007582`, 0.231827`, 
              0.026008`, \(-0.029485`\), 0.899311`, 0.744316`, 0.998916`, 
              0.948954`, 1.366498`, 1.389641`, 16.5242`, 16.555304`, 
              1.077409`, 29.139371`, 29.112005`, \(-0.947927`\), 31.586821`, 
              31.594021`, 0.249405`, 46.086692`, 46.089341`, 0.091758`, 
              24.081991`, 24.094924`, 0.447978`, 48.372329`, 48.404432`, 
              1.112024`, 50.787859`, 50.782953`, \(-0.169967`\), 58.216145`, 
              58.169551`, \(-1.613949`\), 16.5242`, 29.139371`, 31.586821`, 
              51.538802`, 58.775041`, 70.861509`, 87.834083`, 88.833512`, 
              24.081991`, 48.372329`, 50.787859`, 58.216145`, 46.086692`, 
              65.235274`, 69.932046`, 70.233652`}, {5, 28, 
              20.54258057`, \(-0.7781080674`\), 5.373108435`, 0.16`, 
              1, \(-0.3828495339`\), \(-0.2`\), 
              73.99373952`, \(-1.027070412`\), \(-1.407620068`\), 2, 4, 5, 6, 
              0.485206`, \(-0.45846`\), 0.06749`, 0.196072`, 
              0.079645`, \(-0.073114`\), 0.894772`, 0.793955`, 0.951771`, 
              0.9459`, 1.288727`, 1.292534`, 16.5242`, 16.546687`, 0.938014`, 
              31.91865`, 31.899663`, \(-0.791991`\), 36.630614`, 36.637662`, 
              0.294008`, 52.678318`, 52.680387`, 0.086317`, 28.334251`, 
              28.347346`, 0.546242`, 56.4503`, 56.485371`, 1.462895`, 
              59.543352`, 59.534909`, \(-0.352176`\), 65.956868`, 
              65.932296`, \(-1.024941`\), 16.5242`, 31.91865`, 36.630614`, 
              57.87006`, 68.114931`, 82.338136`, 101.715196`, 101.955279`, 
              28.334251`, 56.4503`, 59.543352`, 65.956868`, 52.678318`, 
              74.467275`, 77.587728`, 81.142619`}, {2, 22, 
              33.90971015`, \(-0.7660307576`\), 6.475603445`, 0.16`, 
              1, \(-0.4026828846`\), \(-0.15`\), 
              8.649776717`, \(-1.600209054`\), \(-1.704387148`\), 2, 4, 5, 6, 
              0.50143`, \(-0.450455`\), 0.077822`, 0.158905`, 
              0.114473`, \(-0.092766`\), 0.892444`, 0.8229`, 0.910138`, 
              0.926627`, 1.218254`, 1.17886`, 16.5242`, 16.540366`, 
              0.839896`, 36.970394`, 36.988784`, 0.955388`, 39.665059`, 
              39.645558`, \(-1.013173`\), 61.482096`, 61.501193`, 0.99217`, 
              34.175526`, 34.188041`, 0.650195`, 68.682381`, 68.720759`, 
              1.993882`, 71.232634`, 71.219698`, \(-0.672105`\), 77.464355`, 
              77.441045`, \(-1.211028`\), 16.5242`, 36.970394`, 39.665059`, 
              61.482096`, 81.695163`, 98.741534`, 119.776456`, 122.517561`, 
              34.175526`, 68.682381`, 71.232634`, 77.464355`, 62.759003`, 
              88.017733`, 90.358557`, 97.409017`}, {2, 18, 
              80.89311339`, \(-0.7541503834`\), 6.948989096`, 0.16`, 
              1, \(-0.4225828297`\), \(-0.1`\), 
              29.24448426`, \(-1.957613786`\), \(-1.626252139`\), 2, 4, 5, 6, 
              0.517754`, \(-0.440698`\), 0.078742`, 0.13328`, 
              0.133457`, \(-0.102969`\), 0.889802`, 0.779284`, 0.911937`, 
              0.860114`, 1.22293`, 1.061341`, 16.5242`, 16.530318`, 
              0.352214`, 39.727021`, 39.722794`, \(-0.243325`\), 46.138276`, 
              46.146531`, 0.475216`, 66.634334`, 66.635009`, 0.038834`, 
              36.668804`, 36.678688`, 0.568993`, 74.352631`, 74.385749`, 
              1.906476`, 75.860039`, 75.847321`, \(-0.732162`\), 81.044227`, 
              81.022375`, \(-1.25795`\), 16.5242`, 39.727021`, 46.138276`, 
              69.881011`, 87.298035`, 105.448648`, 125.735696`, 130.738443`, 
              36.668804`, 74.352631`, 75.860039`, 81.044227`, 66.634334`, 
              92.610979`, 93.997783`, 104.275824`}, {5, 29, 
              69.11803596`, \(-0.7810306805`\), 9.240830182`, 0.16`, 
              1, \(-0.429073087`\), \(-0.05`\), 
              3.74281631`, \(-2.225240974`\), \(-1.928461074`\), 2, 4, 5, 6, 
              0.537651`, \(-0.418425`\), 0.04506`, 0.113318`, 
              0.150008`, \(-0.116201`\), 0.888934`, 0.900679`, 0.861342`, 
              0.952335`, 1.135346`, 1.090448`, 16.5242`, 16.530416`, 
              0.494133`, 43.192504`, 43.205583`, 1.039714`, 54.105754`, 
              54.103907`, \(-0.146772`\), 75.280535`, 75.291849`, 0.899324`, 
              48.897751`, 48.906026`, 0.657815`, 100.364593`, 100.390356`, 
              2.047984`, 100.969537`, 100.96254`, \(-0.556156`\), 
              106.522447`, 106.496403`, \(-2.070332`\), 16.5242`, 54.105754`, 
              43.192504`, 75.280535`, 116.788308`, 140.711671`, 166.889907`, 
              173.078575`, 48.897751`, 100.364593`, 100.969537`, 106.522447`, 
              89.085233`, 122.260543`, 124.387702`, 139.920276`}, {5, 29, 
              131.4001932`, \(-0.7384284266`\), 10.22003104`, 0.16`, 
              1, \(-0.4442713424`\), 0, 
              21.67893776`, \(-2.541370585`\), \(-2.13108674`\), 2, 4, 5, 6, 
              0.55009`, \(-0.413466`\), 0.044125`, 0.085442`, 
              0.166413`, \(-0.126823`\), 0.883888`, 0.797333`, 0.824815`, 
              0.832491`, 1.074196`, 0.928308`, 16.5242`, 16.524419`, 
              0.019682`, 59.085855`, 59.083719`, \(-0.192144`\), 66.116524`, 
              66.131169`, 1.317355`, 94.700404`, 94.701813`, 0.126764`, 
              53.783657`, 53.78784`, 0.376194`, 109.852214`, 109.870093`, 
              1.608231`, 110.8731`, 110.865831`, \(-0.653918`\), 114.035428`, 
              114.010738`, \(-2.220888`\), 16.5242`, 59.085855`, 66.116524`, 
              94.700404`, 127.669513`, 153.779778`, 179.556435`, 186.936643`, 
              53.783657`, 109.852214`, 110.8731`, 114.035428`, 96.342836`, 
              131.20457`, 131.664459`, 153.331206`}, {2, 7, 
              207.3581082`, \(-0.7066966808`\), 15.07052889`, 0.16`, 
              1, \(-0.457379`\), 0.05`, 50, \(-3.06369`\), \(-1.4469`\), 2, 
              4, 5, 6, 0.562893`, \(-0.402772`\), 0.023022`, 0.039152`, 
              0.199037`, \(-0.158223`\), 0.878883`, 0.627728`, 0.926718`, 
              0.63875`, 1.254977`, 0.670276`, 16.5242`, 16.525065`, 
              0.117445`, 86.704603`, 86.701778`, \(-0.383542`\), 98.554689`, 
              98.575052`, 2.763866`, 131.449439`, 131.450417`, 0.132761`, 
              78.846394`, 78.845143`, \(-0.169798`\), 159.420613`, 
              159.429381`, 1.190042`, 162.60693`, 
              162.596514`, \(-1.413776`\), 164.121177`, 
              164.094795`, \(-3.580744`\), 16.5242`, 86.704603`, 98.554689`, 
              131.449439`, 186.500337`, 224.391036`, 258.90807`, 269.449686`, 
              78.846394`, 159.420613`, 162.60693`, 164.121177`, 139.357662`, 
              188.144056`, 187.083377`, 224.719362`}, {2, 17, 
              150.6750924`, \(-0.8219103367`\), 12.37554717`, 0.16`, 
              1, \(-0.4477096655`\), 0.1`, 
              5.447484783`, \(-2.82619174`\), \(-2.268963382`\), 2, 4, 5, 6, 
              0.587654`, \(-0.359779`\), \(-0.07744`\), 0.06226`, 
              0.188989`, \(-0.151986`\), 0.882291`, 0.763628`, 0.800585`, 
              0.784321`, 1.032525`, 0.84249`, 16.5242`, 16.527162`, 
              0.344626`, 65.062808`, 65.062079`, \(-0.084765`\), 77.044878`, 
              77.043617`, \(-0.146735`\), 103.025862`, 
              103.024504`, \(-0.157924`\), 65.354191`, 
              65.347618`, \(-0.764782`\), 133.929924`, 
              133.928396`, \(-0.177903`\), 136.299598`, 
              136.280038`, \(-2.276098`\), 138.8284`, 
              138.821025`, \(-0.858167`\), 16.5242`, 77.044878`, 65.062808`, 
              103.025862`, 157.557852`, 188.46937`, 219.627212`, 226.945789`, 
              65.354191`, 133.929924`, 136.299598`, 138.8284`, 119.35682`, 
              158.089164`, 163.300348`, 190.451803`}, {2, 6, 
              6275535789, \(-0.6354573437`\), \(-107.5801946`\), 0.16`, 
              1, \(-0.4797778055`\), 0.15`, 43.99072317`, \(-3.376094163`\), 
              6.787312456`, 2, 4, 5, 6, 
              0.584818`, \(-0.373084`\), \(-0.061694`\), 0.008883`, 
              0.205527`, \(-0.163688`\), 0.866433`, 0.934361`, 1.734479`, 
              0.9347`, 2.288692`, 0.935713`, 16.5242`, 16.485767`, 
              38.688157`, \(-606.706488`\), \(-606.737548`\), 
              31.26653`, \(-685.091897`\), \(-685.180434`\), 
              89.124266`, \(-798.811729`\), \(-838.158522`\), 
              39607.936154`, \(-552.895921`\), \(-552.78103`\), \
\(-115.653425`\), \(-1055.535158`\), \(-1055.36579`\), \(-170.492158`\), \
\(-1138.036243`\), \(-1138.021237`\), \(-15.105912`\), \(-1148.458704`\), \
\(-1148.378723`\), \(-80.512067`\), 
              16.5242`, \(-606.706488`\), \(-685.091897`\), \(-798.811729`\), \
\(-1299.98905`\), \(-1558.89836`\), \(-1761.279459`\), \(-1824.024736`\), \
\(-552.895921`\), \(-1055.535158`\), \(-1138.036243`\), \(-1148.458704`\), \
\(-947.897773`\), \(-1258.120697`\), \(-1231.004781`\), \(-1579.523718`\)}, \
{2, 14, 1.320052612`*^6, \(-0.2833405138`\), \(-1.450322949`\), 0.2025`, 
              1, \(-0.4764928214`\), \(-0.35`\), \(-8.57240906`\), \
\(-3.585459144`\), 18.53363442`, 2, 4, 5, 6, 0.412633`, \(-1.055762`\), 
              1.052716`, 0.0021`, 0.125917`, \(-0.066278`\), 0.907097`, 
              0.056731`, 1.945422`, 0.106676`, 2.527088`, 0.123564`, 
              16.5242`, 16.497305`, 
              0.257522`, \(-0.232645`\), \(-0.247204`\), 
              0.139402`, \(-6.313062`\), \(-6.299374`\), \(-0.131061`\), \
\(-10.226676`\), \(-10.230074`\), 0.032531`, \(-7.571258`\), \(-7.578022`\), 
              0.064767`, \(-12.651199`\), \(-12.663191`\), 
              0.114819`, \(-16.08678`\), \(-16.086331`\), \(-0.004293`\), \
\(-16.523191`\), \(-16.496877`\), \(-0.251958`\), 
              16.5242`, \(-0.232645`\), \(-6.313062`\), \(-10.226676`\), \
\(-16.743662`\), \(-20.850194`\), \(-25.560004`\), \(-26.028868`\), \
\(-7.571258`\), \(-12.651199`\), \(-16.08678`\), \(-16.523191`\), \
\(-12.075568`\), \(-18.645988`\), \(-17.910485`\), \(-18.223902`\)}, {5, 26, 
              55.73306761`, \(-0.7679661641`\), 4.503046148`, 0.2025`, 
              1, \(-0.4106620247`\), \(-0.3`\), 
              2454.111283`, \(-0.2456782739`\), \(-1.156815393`\), 2, 4, 5, 
              6, 0.50899`, \(-0.441018`\), \(-0.073821`\), 0.2306`, 
              0.028574`, \(-0.031356`\), 0.93435`, 0.759647`, 1.040353`, 
              0.959586`, 1.419765`, 1.393787`, 16.5242`, 16.560839`, 
              1.343624`, 30.926211`, 30.896954`, \(-1.072905`\), 32.536298`, 
              32.540087`, 0.138973`, 48.038545`, 48.041608`, 0.112337`, 
              25.058359`, 25.071663`, 0.487881`, 50.862719`, 50.866603`, 
              0.14244`, 51.134368`, 51.160996`, 0.976511`, 60.374717`, 
              60.313186`, \(-2.256453`\), 16.5242`, 30.926211`, 32.536298`, 
              53.391753`, 61.171029`, 73.323675`, 90.207011`, 90.881339`, 
              25.058359`, 51.134368`, 50.862719`, 60.374717`, 48.038545`, 
              67.1098`, 73.048892`, 74.341464`}, {5, 27, 
              37.38460392`, \(-0.7174142274`\), 5.175339818`, 0.2025`, 
              1, \(-0.4401198641`\), \(-0.25`\), 
              1820.44574`, \(-0.9429933694`\), \(-1.451116148`\), 2, 4, 5, 6, 
              0.518778`, \(-0.450998`\), \(-0.019171`\), 0.206933`, 
              0.060943`, \(-0.056388`\), 0.930424`, 0.817181`, 0.992357`, 
              0.977162`, 1.341023`, 1.342459`, 16.5242`, 16.552329`, 
              1.208369`, 33.250594`, 33.223558`, \(-1.161422`\), 37.042103`, 
              37.051364`, 0.397838`, 53.691109`, 53.693613`, 0.107597`, 
              28.709523`, 28.722939`, 0.576328`, 57.981089`, 57.982046`, 
              0.041113`, 58.345849`, 58.375502`, 1.273838`, 66.892082`, 
              66.858883`, \(-1.426149`\), 16.5242`, 33.250594`, 37.042103`, 
              59.170053`, 69.195653`, 83.1323`, 101.971576`, 103.034476`, 
              28.709523`, 58.345849`, 57.981089`, 66.892082`, 53.691109`, 
              74.905872`, 81.360947`, 82.044423`}, {5, 28, 
              44.17145461`, \(-0.653899119`\), 6.53820485`, 0.2025`, 
              1, \(-0.4715574698`\), \(-0.2`\), 
              183.9048726`, \(-1.784906986`\), \(-1.942115949`\), 2, 4, 5, 6, 
              0.527661`, \(-0.46661`\), 0.04385`, 0.155227`, 
              0.103238`, \(-0.076453`\), 0.925323`, 0.824371`, 0.925613`, 
              0.922124`, 1.22931`, 1.162617`, 16.5242`, 16.544645`, 
              1.128524`, 39.243597`, 39.225846`, \(-0.979841`\), 46.141857`, 
              46.150015`, 0.450311`, 65.570522`, 65.57257`, 0.113052`, 
              36.069677`, 36.084155`, 0.799204`, 72.452491`, 
              72.448468`, \(-0.222014`\), 72.86956`, 72.906657`, 2.047733`, 
              80.957419`, 80.926429`, \(-1.710613`\), 16.5242`, 39.243597`, 
              46.141857`, 70.702271`, 85.727105`, 103.263316`, 125.540346`, 
              128.348202`, 36.069677`, 72.86956`, 72.452491`, 80.957419`, 
              65.570522`, 91.379203`, 97.564303`, 100.439745`}, {2, 12, 
              35.93993287`, \(-0.6602475339`\), 7.354224819`, 0.2025`, 
              1, \(-0.4893951991`\), \(-0.15`\), 
              34.7279575`, \(-2.022927982`\), \(-1.720922211`\), 2, 4, 5, 6, 
              0.545853`, \(-0.445507`\), 0.025507`, 0.141214`, 
              0.112755`, \(-0.08174`\), 0.9238`, 0.864991`, 0.943906`, 
              0.943691`, 1.261823`, 1.144401`, 16.5242`, 16.536132`, 
              0.766403`, 43.971447`, 43.960778`, \(-0.685296`\), 50.838764`, 
              50.848241`, 0.608655`, 73.476745`, 73.477963`, 0.078251`, 
              40.62731`, 40.640055`, 0.818573`, 81.546635`, 81.550765`, 
              0.265253`, 82.537566`, 82.564243`, 1.713446`, 89.35854`, 
              89.329239`, \(-1.881922`\), 16.5242`, 43.971447`, 50.838764`, 
              77.378133`, 96.451269`, 116.031425`, 140.219261`, 143.907526`, 
              40.62731`, 82.537566`, 81.546635`, 89.35854`, 73.476745`, 
              101.412561`, 107.437557`, 113.257919`}, {2, 7, 
              74.08461976`, \(-0.6845537774`\), 7.728088662`, 0.2025`, 
              1, \(-0.495072`\), \(-0.1`\), 50, \(-2.38038`\), \(-1.56419`\), 
              2, 4, 5, 6, 0.565915`, \(-0.416996`\), \(-0.014737`\), 
              0.115722`, 0.132885`, \(-0.097993`\), 0.923108`, 0.788794`, 
              0.966706`, 0.849732`, 1.301432`, 1.007462`, 16.5242`, 
              16.529277`, 0.355283`, 47.086329`, 47.080958`, \(-0.375858`\), 
              53.656354`, 53.666428`, 0.70491`, 77.575233`, 77.575811`, 
              0.040473`, 42.804475`, 42.81412`, 0.67486`, 86.435241`, 
              86.445403`, 0.711066`, 87.274774`, 87.290508`, 1.100992`, 
              92.846749`, 92.819719`, \(-1.891459`\), 16.5242`, 47.086329`, 
              53.656354`, 80.661795`, 101.932157`, 122.324567`, 146.492576`, 
              151.791197`, 42.804475`, 87.274774`, 86.435241`, 92.846749`, 
              77.575233`, 105.786176`, 112.115909`, 120.086744`}, {2, 7, 
              159.8667665`, \(-0.7266819613`\), 7.741210251`, 0.2025`, 
              1, \(-0.498786`\), \(-0.05`\), 50, \(-2.75556`\), \(-1.4931`\), 
              2, 4, 5, 6, 0.58734`, \(-0.38468`\), \(-0.070133`\), 0.085549`, 
              0.160167`, \(-0.125168`\), 0.92329`, 0.646607`, 0.9776`, 
              0.691485`, 1.319993`, 0.807782`, 16.5242`, 16.525586`, 
              0.100818`, 49.096541`, 49.091135`, \(-0.39334`\), 53.526679`, 
              53.539626`, 0.941882`, 78.700774`, 78.700934`, 0.01163`, 
              43.042645`, 43.048664`, 0.437812`, 87.892366`, 87.900451`, 
              0.588151`, 87.990947`, 88.002237`, 0.821279`, 92.710458`, 
              92.686691`, \(-1.729019`\), 16.5242`, 49.096541`, 53.526679`, 
              80.761187`, 103.205347`, 123.437779`, 146.601308`, 153.200282`, 
              43.042645`, 87.892366`, 87.990947`, 92.710458`, 78.700774`, 
              105.858245`, 113.097426`, 122.017344`}, {5, 26, 
              125.9312698`, \(-0.6892810011`\), 10.58447726`, 0.2025`, 
              1, \(-0.5153447511`\), 0, 
              6042.88317`, \(-2.654723119`\), \(-2.357349202`\), 2, 4, 5, 6, 
              0.599126`, \(-0.377451`\), \(-0.076899`\), 0.093241`, 
              0.146392`, \(-0.108541`\), 0.918394`, 0.897043`, 0.840376`, 
              0.932393`, 1.084517`, 1.030007`, 16.5242`, 16.524296`, 
              0.009779`, 66.138011`, 66.132946`, \(-0.513868`\), 73.503185`, 
              73.519559`, 1.661335`, 104.383657`, 104.384281`, 0.063323`, 
              58.48962`, 58.493346`, 0.378068`, 118.887791`, 118.895134`, 
              0.7451`, 119.019328`, 119.026004`, 0.677357`, 123.339353`, 
              123.314367`, \(-2.535201`\), 16.5242`, 66.138011`, 73.503185`, 
              104.383657`, 139.549465`, 166.850564`, 195.738682`, 
              206.303291`, 58.48962`, 118.887791`, 119.019328`, 123.339353`, 
              105.308602`, 140.63596`, 148.690104`, 165.335624`}, {5, 28, 
              187.0043204`, \(-0.6952179282`\), 17.09966572`, 0.2025`, 
              1, \(-0.5222422653`\), 0.05`, 
              4.912219258`, \(-3.33378912`\), \(-2.169697955`\), 2, 4, 5, 6, 
              0.614932`, \(-0.354928`\), \(-0.123634`\), 0.032971`, 
              0.196`, \(-0.161596`\), 0.915806`, 0.626062`, 0.865293`, 
              0.633884`, 1.129803`, 0.656557`, 16.5242`, 16.524899`, 
              0.117637`, 86.952209`, 86.9543`, 0.351714`, 108.155443`, 
              108.155847`, 0.067975`, 142.845098`, 142.84798`, 0.484888`, 
              94.291331`, 94.290035`, \(-0.21802`\), 191.008691`, 191.01302`, 
              0.728271`, 191.23912`, 191.228514`, \(-1.784401`\), 
              198.638759`, 198.620349`, \(-3.097291`\), 16.5242`, 
              108.155443`, 86.952209`, 142.845098`, 225.585319`, 269.093742`, 
              312.535174`, 331.514786`, 94.291331`, 191.008691`, 191.23912`, 
              198.638759`, 169.608038`, 224.001919`, 237.723679`, 
              268.086557`}, {5, 29, 386.2305933`, \(-0.8313896954`\), 
              8.503587753`, 0.2025`, 1, \(-0.5029553977`\), 0.1`, 
              26.42844134`, \(-3.660056825`\), \(-2.229548901`\), 2, 4, 5, 6, 
              0.642907`, \(-0.307642`\), \(-0.23101`\), 0.002995`, 
              0.23976`, \(-0.215881`\), 0.921316`, 0.228517`, 0.86279`, 
              0.231781`, 1.122281`, 0.239548`, 16.5242`, 16.526257`, 
              0.179891`, 58.195066`, 58.203762`, 0.760627`, 58.932073`, 
              58.93115`, \(-0.080721`\), 86.733285`, 86.735816`, 0.221416`, 
              47.493048`, 47.489183`, \(-0.338069`\), 96.718047`, 
              96.71721`, \(-0.073236`\), 98.634066`, 
              98.623869`, \(-0.891952`\), 103.016129`, 
              103.011282`, \(-0.423991`\), 16.5242`, 58.932073`, 58.195066`, 
              86.733285`, 116.207665`, 137.729625`, 159.608265`, 171.175445`, 
              47.493048`, 96.718047`, 98.634066`, 103.016129`, 88.906741`, 
              115.201337`, 126.816734`, 138.504066`}, {5, 28, 
              174.1819358`, \(-0.8620778089`\), 8.803035701`, 0.2025`, 
              1, \(-0.5021736732`\), 0.15`, 
              29.71962863`, \(-3.273943124`\), \(-4.170489603`\), 2, 4, 5, 6, 
              0.658524`, \(-0.2834`\), \(-0.29256`\), 0.042585`, 
              0.202368`, \(-0.173974`\), 0.919857`, 0.482227`, 0.545089`, 
              0.501685`, 0.557201`, 0.553742`, 16.5242`, 16.528535`, 
              0.402068`, 60.349784`, 60.358826`, 0.838592`, 62.654652`, 
              62.652534`, \(-0.196522`\), 89.200073`, 89.206399`, 0.586737`, 
              49.136004`, 49.128819`, \(-0.666388`\), 99.934564`, 
              99.931992`, \(-0.238501`\), 101.613787`, 
              101.601012`, \(-1.184953`\), 107.878187`, 
              107.873725`, \(-0.413895`\), 16.5242`, 62.654652`, 60.349784`, 
              89.200073`, 121.023543`, 143.038467`, 164.495911`, 177.90956`, 
              49.136004`, 99.934564`, 101.613787`, 107.878187`, 92.628168`, 
              118.669957`, 132.148661`, 144.557797`}, {5, 27, 
              19009.47476`, \(-0.25`\), \(-14.27954826`\), 0.25`, 
              1, \(-0.5762735139`\), \(-0.35`\), 
              2232.274743`, \(-4.334241898`\), \(-7.625417168`\), 2, 4, 5, 6, 
              0.451651`, \(-1.21843`\), 1.276412`, \(-0.047188`\), 
              0.112712`, \(-0.072403`\), 
              0.936281`, \(-0.347678`\), \(-0.012353`\), \(-0.362549`\), \
\(-0.47466`\), \(-0.404604`\), 16.5242`, 16.404858`, 
              
              12.314888`, \(-55.050271`\), \(-54.942156`\), \(-11.156388`\), \
\(-98.330861`\), \(-98.351587`\), 
              2.138747`, \(-118.162293`\), \(-118.175118`\), 
              1.323372`, \(-77.920602`\), \(-77.981335`\), 
              6.267001`, \(-128.736502`\), \(-128.911381`\), 
              18.045741`, \(-159.315224`\), \(-159.253971`\), \(-6.320628`\), \
\(-160.179788`\), \(-160.037512`\), \(-14.681464`\), 
              16.5242`, \(-55.050271`\), \(-98.330861`\), \(-118.887877`\), \
\(-167.67004`\), \(-209.506807`\), \(-252.396392`\), \(-265.921849`\), \
\(-77.920602`\), \(-128.736502`\), \(-159.315224`\), \(-160.179788`\), \
\(-118.162293`\), \(-183.073024`\), \(-178.508581`\), \(-179.986597`\)}, {2, 
              18, 21302.03328`, \(-0.25`\), \(-13.23689718`\), 0.25`, 
              1, \(-0.5909048863`\), \(-0.3`\), 
              1536.807775`, \(-4.121224307`\), \(-7.271225118`\), 2, 4, 5, 6, 
              0.475376`, \(-1.116339`\), 1.143059`, \(-0.024029`\), 
              0.09272`, \(-0.042073`\), 0.936182`, \(-0.131194`\), 
              0.060695`, \(-0.139109`\), \(-0.338108`\), \(-0.161382`\), 
              16.5242`, 16.451563`, 
              7.313154`, \(-50.596708`\), \(-50.531226`\), \(-6.592669`\), \
\(-91.077712`\), \(-91.094985`\), 
              1.738968`, \(-109.442213`\), \(-109.510611`\), 
              6.88634`, \(-72.677637`\), \(-72.72541`\), 
              4.809792`, \(-124.008811`\), \(-124.193391`\), 
              18.583605`, \(-145.58695`\), \(-145.495191`\), \(-9.238298`\), \
\(-146.293962`\), \(-146.19836`\), \(-9.625289`\), 
              16.5242`, \(-50.596708`\), \(-91.077712`\), \(-109.442213`\), \
\(-156.510818`\), \(-194.93778`\), \(-234.88886`\), \(-245.991559`\), \
\(-72.677637`\), \(-124.008811`\), \(-145.58695`\), \(-146.293962`\), \
\(-110.083545`\), \(-168.404573`\), \(-160.70991`\), \(-167.688824`\)}, {5, 
              27, 2230.491095`, \(-0.4575703517`\), 12.65673803`, 0.25`, 
              1, \(-0.5659133524`\), \(-0.25`\), 
              328.4836512`, \(-2.984110387`\), \(-3.586954495`\), 2, 4, 5, 6, 
              0.540417`, \(-0.593149`\), 0.212637`, 0.079441`, 
              0.118722`, \(-0.070078`\), 0.952261`, 0.850342`, 0.69727`, 
              0.876817`, 0.813689`, 0.951206`, 16.5242`, 16.57318`, 5.36034`, 
              71.101243`, 71.057332`, \(-4.80556`\), 91.08348`, 91.09941`, 
              1.74337`, 124.445728`, 124.45079`, 0.553963`, 72.026401`, 
              72.059097`, 3.57826`, 138.329892`, 138.323521`, \(-0.697231`\), 
              143.102122`, 143.186293`, 9.211523`, 155.40518`, 
              155.32748`, \(-8.503327`\), 16.5242`, 71.101243`, 91.08348`, 
              130.981255`, 166.570403`, 201.505238`, 241.221584`, 
              250.722876`, 72.026401`, 143.102122`, 138.329892`, 155.40518`, 
              124.445728`, 175.157877`, 189.808713`, 189.953859`}, {5, 27, 
              21.7666919`, \(-0.642678198`\), 6.270883073`, 0.25`, 
              1, \(-0.5452726352`\), \(-0.2`\), 
              2726.392414`, \(-1.801194062`\), \(-2.022246692`\), 2, 4, 5, 6, 
              0.585849`, \(-0.420481`\), \(-0.100511`\), 0.167505`, 
              0.085606`, \(-0.062874`\), 0.962279`, 0.876493`, 0.951105`, 
              0.980495`, 1.255393`, 1.237499`, 16.5242`, 16.539811`, 
              0.917633`, 41.475587`, 41.44904`, \(-1.560455`\), 45.963805`, 
              45.983691`, 1.168887`, 67.195748`, 67.197292`, 0.090764`, 
              36.54673`, 36.558619`, 0.698858`, 72.80052`, 72.823487`, 
              1.35001`, 73.852597`, 73.862162`, 0.562264`, 81.311117`, 
              81.280232`, \(-1.815445`\), 16.5242`, 41.475587`, 45.963805`, 
              72.381858`, 87.483583`, 104.502411`, 124.943562`, 129.055459`, 
              36.54673`, 73.852597`, 72.80052`, 81.311117`, 67.195748`, 
              91.331518`, 101.633781`, 102.923851`}, {5, 28, 
              40.17585357`, \(-0.6289150167`\), 7.468178222`, 0.25`, 
              1, \(-0.5629512517`\), \(-0.15`\), 
              115.1987532`, \(-2.266900338`\), \(-2.322188889`\), 2, 4, 5, 6, 
              0.600672`, \(-0.408245`\), \(-0.102222`\), 0.136708`, 
              0.104705`, \(-0.07271`\), 0.959533`, 0.883419`, 0.902847`, 
              0.955104`, 1.173221`, 1.140831`, 16.5242`, 16.53353`, 
              0.669663`, 48.788826`, 48.770712`, \(-1.300133`\), 54.016485`, 
              54.034563`, 1.297524`, 79.047235`, 79.048231`, 0.07148`, 
              43.438424`, 43.449619`, 0.80351`, 86.671833`, 86.695984`, 
              1.733442`, 87.644137`, 87.653054`, 0.64001`, 94.547746`, 
              94.515898`, \(-2.285919`\), 16.5242`, 48.788826`, 54.016485`, 
              83.321932`, 103.628359`, 123.742165`, 147.144767`, 152.561992`, 
              43.438424`, 87.644137`, 86.671833`, 94.547746`, 79.047235`, 
              106.685339`, 119.282374`, 120.389015`}, {2, 21, 
              73.96341681`, \(-0.6552529993`\), 7.894215985`, 0.25`, 
              1, \(-0.5703797979`\), \(-0.1`\), 
              47.19222633`, \(-2.395636595`\), \(-2.336902835`\), 2, 4, 5, 6, 
              0.62073`, \(-0.375801`\), \(-0.151273`\), 0.129245`, 
              0.111342`, \(-0.079159`\), 0.959119`, 0.902463`, 0.900107`, 
              0.965889`, 1.168692`, 1.132819`, 16.5242`, 16.528366`, 
              0.326604`, 49.272099`, 49.307554`, 2.779766`, 59.985256`, 
              59.954511`, \(-2.410463`\), 84.103578`, 84.10401`, 0.03391`, 
              46.021163`, 46.029664`, 0.666474`, 92.735835`, 92.742059`, 
              0.487985`, 92.825335`, 92.84727`, 1.7198`, 99.266954`, 
              99.238053`, \(-2.265967`\), 16.5242`, 49.272099`, 59.985256`, 
              87.070781`, 110.288607`, 131.347369`, 154.932109`, 161.986902`, 
              46.021163`, 92.735835`, 92.825335`, 99.266954`, 84.103578`, 
              112.182666`, 125.753999`, 128.523115`}, {5, 28, 
              53.92311192`, \(-0.676860662`\), 11.14776459`, 0.25`, 
              1, \(-0.5769982675`\), \(-0.05`\), 
              4.227606773`, \(-3.014748725`\), \(-2.162938309`\), 2, 4, 5, 6, 
              0.639507`, \(-0.347525`\), \(-0.19773`\), 0.079773`, 
              0.150939`, \(-0.117479`\), 0.958239`, 0.84892`, 0.925175`, 
              0.876844`, 1.212852`, 0.954823`, 16.5242`, 16.529738`, 
              0.631687`, 56.448437`, 56.463097`, 1.672195`, 75.452725`, 
              75.451273`, \(-0.165645`\), 96.913712`, 96.925089`, 1.297729`, 
              65.057485`, 65.065028`, 0.860379`, 130.903513`, 130.909005`, 
              0.62652`, 132.236632`, 132.258722`, 2.519636`, 139.418417`, 
              139.386037`, \(-3.693451`\), 16.5242`, 75.452725`, 56.448437`, 
              96.913712`, 156.596484`, 186.026353`, 217.561001`, 229.553867`, 
              65.057485`, 130.903513`, 132.236632`, 139.418417`, 119.308562`, 
              157.346268`, 177.03848`, 183.013466`}, {2, 8, 
              400.0740305`, \(-0.7975613791`\), 6.228370843`, 0.25`, 
              1, \(-0.560753`\), 0, 50, \(-3.60373`\), \(-1.36853`\), 2, 4, 
              5, 6, 0.669708`, \(-0.296293`\), \(-0.30031`\), 0.031103`, 
              0.208984`, \(-0.186571`\), 0.963633`, 0.330911`, 1.048063`, 
              0.352694`, 1.419625`, 0.404927`, 16.5242`, 16.524311`, 
              0.00742`, 45.414913`, 45.422147`, 0.482773`, 45.959529`, 
              45.958299`, \(-0.082118`\), 69.910684`, 69.910685`, 0.000043`, 
              36.80956`, 36.811311`, 0.116842`, 73.955112`, 73.956322`, 
              0.080747`, 77.250941`, 77.257535`, 0.440091`, 80.46387`, 
              80.451254`, \(-0.841977`\), 16.5242`, 45.959529`, 45.414913`, 
              71.824669`, 90.424909`, 106.744939`, 124.368006`, 131.398123`, 
              36.80956`, 73.955112`, 77.250941`, 80.46387`, 69.910684`, 
              90.556641`, 104.319258`, 106.139443`}, {5, 28, 
              298.6341609`, \(-0.7553377187`\), 8.541302949`, 0.25`, 
              1, \(-0.5779572002`\), 0.05`, 
              21.52604693`, \(-3.597968585`\), \(-2.351186037`\), 2, 4, 5, 6, 
              0.678858`, \(-0.286157`\), \(-0.318833`\), 0.030369`, 
              0.200939`, \(-0.176216`\), 0.958292`, 0.38962`, 0.896876`, 
              0.404149`, 1.163583`, 0.442697`, 16.5242`, 16.524659`, 
              0.042548`, 60.311565`, 60.319928`, 0.775798`, 61.998009`, 
              61.997557`, \(-0.041988`\), 92.025555`, 92.027284`, 0.160378`, 
              50.082534`, 50.081967`, \(-0.05263`\), 100.516431`, 
              100.516731`, 0.027905`, 102.993294`, 
              102.990154`, \(-0.291286`\), 108.653188`, 
              108.646136`, \(-0.65423`\), 16.5242`, 61.998009`, 60.311565`, 
              92.025555`, 122.468633`, 144.540321`, 166.502849`, 178.388122`, 
              50.082534`, 100.516431`, 102.993294`, 108.653188`, 93.702024`, 
              120.550487`, 138.323086`, 143.944375`}, {5, 28, 
              220.8741723`, \(-0.9696548901`\), 4.736110893`, 0.25`, 
              1, \(-0.5364246246`\), 0.1`, 
              478.0847465`, \(-3.262738944`\), \(-4.043612008`\), 2, 4, 5, 6, 
              0.716839`, \(-0.244144`\), \(-0.413755`\), 0.065113`, 
              0.191162`, \(-0.171913`\), 0.969381`, 0.409386`, 0.64617`, 
              0.462465`, 0.712932`, 0.581669`, 16.5242`, 16.525383`, 
              0.064281`, 35.505795`, 35.510157`, 0.236977`, 38.852189`, 
              38.851448`, \(-0.040287`\), 56.382016`, 56.38211`, 0.00507`, 
              28.394015`, 28.392285`, \(-0.093969`\), 56.857432`, 
              56.856557`, \(-0.047553`\), 61.719659`, 
              61.715136`, \(-0.245655`\), 64.112376`, 
              64.110133`, \(-0.121855`\), 16.5242`, 38.852189`, 35.505795`, 
              58.34935`, 71.725516`, 83.972672`, 97.037324`, 103.095604`, 
              28.394015`, 56.857432`, 61.719659`, 64.112376`, 56.382016`, 
              71.254668`, 84.460611`, 84.650336`}, {5, 28, 
              123.947963`, \(-0.8821712108`\), 6.695225931`, 0.25`, 
              1, \(-0.5622141312`\), 0.15`, 
              61.61582502`, \(-2.251876258`\), \(-2.857650485`\), 2, 4, 5, 6, 
              0.719152`, \(-0.228968`\), \(-0.451979`\), 0.142646`, 
              0.111779`, \(-0.089883`\), 0.960857`, 0.905857`, 0.825658`, 
              0.982235`, 1.0371`, 1.179088`, 16.5242`, 16.527311`, 0.239644`, 
              48.910704`, 48.916794`, 0.469161`, 53.079287`, 
              53.077633`, \(-0.127403`\), 75.887591`, 75.891487`, 0.300094`, 
              39.589877`, 39.585212`, \(-0.359349`\), 79.316446`, 
              79.314387`, \(-0.158636`\), 82.742543`, 
              82.734499`, \(-0.61967`\), 88.770332`, 
              88.767545`, \(-0.214721`\), 16.5242`, 53.079287`, 48.910704`, 
              75.887591`, 99.04279`, 116.033456`, 132.266101`, 143.021713`, 
              39.589877`, 79.316446`, 82.742543`, 88.770332`, 76.651575`, 
              96.269134`, 113.69856`, 116.903346`}, {2, 35, 
              6.463757838`, \(-1.586701318`\), 7.388765066`, 0, 
              1, \(-0.006118468428`\), \(-0.1223884029`\), 
              1328.031904`, \(-0.6191892896`\), \(-0.6448791526`\), 2, 3, 4, 
              5, 6, 0.185084`, \(-0.42534`\), 0.259678`, 0.136324`, 
              0.232599`, \(-0.226242`\), 0.619684`, 0.594766`, 0.713417`, 
              0.727959`, 1.010486`, 1.002122`, 16.5242`, 16.572037`, 
              1.046701`, 27.380553`, 27.349783`, \(-0.673259`\), 34.840339`, 
              34.866382`, 0.569836`, 44.774649`, 44.782409`, 0.169787`, 
              23.311982`, 23.346377`, 0.752579`, 44.035034`, 44.073939`, 
              0.851261`, 58.218027`, 58.22371`, 0.124351`, 62.284301`, 
              62.322227`, 0.82985`, 16.5242`, 27.380553`, 34.840339`, 
              49.042912`, 58.218027`, 74.724059`, 88.685196`, 95.269282`, 
              23.311982`, 44.035034`, 62.284301`, 64.985903`, 44.774649`, 
              65.467469`, 72.9797`, 79.566949`}, {1, 56, 
              7.239690877`, \(-1.383769551`\), 7.207947949`, 0.001967542671`, 
              1, \(-0.05307870473`\), \(-0.1141404246`\), 
              2798.874143`, \(-0.7012994607`\), \(-0.7524379213`\), 2, 3, 4, 
              5, 6, 0.216659`, \(-0.451172`\), 0.293863`, 0.140002`, 
              0.219449`, \(-0.211372`\), 0.6549`, 0.626164`, 0.738868`, 
              0.75471`, 1.036536`, 1.027345`, 16.5242`, 16.566389`, 
              1.054161`, 26.394095`, 26.362216`, \(-0.796553`\), 35.613128`, 
              35.636899`, 0.593958`, 45.3781`, 45.384255`, 0.153795`, 
              24.599292`, 24.629529`, 0.755535`, 45.803578`, 45.836475`, 
              0.821997`, 59.814636`, 59.814706`, 0.001756`, 63.377198`, 
              63.396923`, 0.492859`, 16.5242`, 26.394095`, 35.613128`, 
              49.570474`, 59.814636`, 76.240965`, 89.49991`, 98.873036`, 
              24.599292`, 45.803578`, 63.377198`, 65.186077`, 45.3781`, 
              64.81508`, 73.468004`, 77.227451`}, {1, 38, 
              8.111494715`, \(-1.24167337`\), 6.927609382`, 0.007919222016`, 
              1, \(-0.08722034695`\), \(-0.1182137623`\), 
              13804.19238`, \(-0.7712155125`\), \(-0.844072778`\), 2, 3, 4, 
              5, 6, 0.24557`, \(-0.473244`\), 0.317899`, 0.146276`, 
              0.201536`, \(-0.193235`\), 0.686032`, 0.647412`, 0.763995`, 
              0.777242`, 1.064574`, 1.056856`, 16.5242`, 16.562934`, 
              1.054328`, 26.289197`, 26.258345`, \(-0.839767`\), 35.828446`, 
              35.848934`, 0.55768`, 45.937088`, 45.942135`, 0.137375`, 
              25.208832`, 25.235818`, 0.734528`, 46.90663`, 46.936194`, 
              0.804725`, 60.647863`, 60.644706`, \(-0.085926`\), 63.04669`, 
              63.054367`, 0.208952`, 16.5242`, 26.289197`, 35.828446`, 
              49.606937`, 60.647863`, 76.829715`, 91.117225`, 99.720468`, 
              25.208832`, 46.90663`, 63.04669`, 65.013161`, 45.937088`, 
              65.826893`, 74.642457`, 75.896437`}, {1, 69, 
              8.341489309`, \(-1.496426973`\), 7.058025858`, 0.01800590693`, 
              1, \(-0.1100994274`\), \(-0.08537929229`\), 
              2.019952108`, \(-0.9794565415`\), \(-0.9904684232`\), 2, 3, 4, 
              5, 6, 0.306315`, \(-0.461412`\), 0.271927`, 0.142445`, 
              0.194826`, \(-0.182518`\), 0.722984`, 0.704537`, 0.787083`, 
              0.816104`, 1.08671`, 1.069876`, 16.5242`, 16.554804`, 
              1.058647`, 25.831813`, 25.847171`, 0.531263`, 30.0951`, 
              30.076875`, \(-0.63045`\), 48.179144`, 48.203142`, 0.830135`, 
              27.945079`, 27.962181`, 0.591614`, 54.936827`, 54.958023`, 
              0.733228`, 67.330158`, 67.317602`, \(-0.434318`\), 68.114692`, 
              68.117218`, 0.087371`, 16.5242`, 25.831813`, 30.0951`, 
              48.179144`, 68.114692`, 85.217886`, 103.263654`, 109.474377`, 
              27.945079`, 54.936827`, 67.330158`, 70.719656`, 52.920059`, 
              75.548947`, 81.982874`, 87.241563`}, {1, 63, 
              9.4115474`, \(-1.019658861`\), 6.830897546`, 0.03249196962`, 
              1, \(-0.1673942526`\), \(-0.1209521116`\), 
              9.419471443`, \(-1.022668024`\), \(-1.098150327`\), 2, 3, 4, 5, 
              6, 0.312142`, \(-0.503509`\), 0.340742`, 0.148937`, 
              0.174328`, \(-0.158249`\), 0.745152`, 0.701267`, 0.799576`, 
              0.819145`, 1.097323`, 1.085996`, 16.5242`, 16.554124`, 
              1.020854`, 28.091265`, 28.067613`, \(-0.806888`\), 35.515801`, 
              35.530583`, 0.504295`, 50.317778`, 50.320959`, 0.108508`, 
              28.033306`, 28.054764`, 0.732042`, 52.540276`, 52.565648`, 
              0.865575`, 66.250188`, 66.227542`, \(-0.772563`\), 66.441621`, 
              66.453898`, 0.418825`, 16.5242`, 28.091265`, 35.515801`, 
              52.804391`, 66.441621`, 83.148406`, 101.037201`, 106.653823`, 
              28.033306`, 52.540276`, 66.250188`, 68.531852`, 50.317778`, 
              71.040614`, 80.378896`, 85.059709`}, {1, 63, 
              8.300556925`, \(-0.9048407402`\), 6.663942561`, 0.0517766953`, 
              1, \(-0.2097078591`\), \(-0.1388002796`\), 
              1425.934394`, \(-1.115421576`\), \(-1.217696578`\), 2, 3, 4, 5, 
              6, 0.34115`, \(-0.515685`\), 0.335753`, 0.153619`, 
              0.157164`, \(-0.139513`\), 0.774798`, 0.71925`, 0.820095`, 
              0.837648`, 1.118621`, 1.108539`, 16.5242`, 16.552588`, 
              1.032611`, 29.060814`, 29.036915`, \(-0.869315`\), 39.159538`, 
              39.173223`, 0.497783`, 51.578301`, 51.581464`, 0.11506`, 
              28.820904`, 28.84156`, 0.751338`, 54.038504`, 54.064413`, 
              0.942446`, 66.657148`, 66.646134`, \(-0.40062`\), 68.10033`, 
              68.106625`, 0.228984`, 16.5242`, 29.060814`, 39.159538`, 
              55.197298`, 68.10033`, 84.728219`, 104.302178`, 108.18104`, 
              28.820904`, 54.038504`, 66.657148`, 69.19293`, 51.578301`, 
              69.916344`, 84.799791`, 86.95024`}, {2, 46, 
              9.332606961`, \(-0.9032653951`\), 6.038734053`, 0.07642881742`, 
              1, \(-0.2457236507`\), \(-0.1582036385`\), 
              13.76009681`, \(-1.020672186`\), \(-1.17477871`\), 2, 3, 4, 5, 
              6, 0.38074`, \(-0.497446`\), 0.256279`, 0.173686`, 
              0.12641`, \(-0.117167`\), 0.809578`, 0.742145`, 0.874785`, 
              0.879646`, 1.196608`, 1.190676`, 16.5242`, 16.552468`, 
              1.039882`, 29.580266`, 29.557503`, \(-0.837379`\), 35.352203`, 
              35.362595`, 0.382287`, 50.770367`, 50.773093`, 0.100261`, 
              27.741169`, 27.758585`, 0.640658`, 53.140517`, 53.164808`, 
              0.893566`, 62.707317`, 62.706009`, \(-0.048114`\), 66.230924`, 
              66.238472`, 0.277679`, 16.5242`, 29.580266`, 35.352203`, 
              54.743833`, 66.230924`, 81.612361`, 93.952056`, 104.929212`, 
              27.741169`, 53.140517`, 62.707317`, 66.414511`, 50.770367`, 
              71.506664`, 76.527348`, 87.685913`}, {5, 100, 
              11.70458489`, \(-0.7821342608`\), 6.360673415`, 0.1072401379`, 
              1, \(-0.3119820943`\), \(-0.164496886`\), 
              528.4818481`, \(-1.318213571`\), \(-1.469103933`\), 2, 3, 4, 5, 
              6, 0.41988`, \(-0.504512`\), 0.239424`, 0.16275`, 
              0.124816`, \(-0.10569`\), 0.841072`, 0.769966`, 0.869807`, 
              0.887436`, 1.173127`, 1.163093`, 16.5242`, 16.547691`, 
              1.003822`, 32.209406`, 32.189642`, \(-0.844522`\), 40.768135`, 
              40.778095`, 0.425623`, 55.64378`, 55.646252`, 0.105628`, 
              30.819212`, 30.836067`, 0.720261`, 58.932365`, 58.958675`, 
              1.124237`, 67.889033`, 67.887489`, \(-0.066002`\), 71.583424`, 
              71.562148`, \(-0.90917`\), 16.5242`, 32.209406`, 40.768135`, 
              60.430885`, 72.989269`, 89.445591`, 104.424399`, 111.969634`, 
              30.819212`, 58.932365`, 67.889033`, 71.583424`, 55.64378`, 
              79.346991`, 79.680021`, 95.792696`}, {3, 79, 
              15.75843486`, \(-0.7737100304`\), 5.574004866`, 0.1453085056`, 
              1, \(-0.3617587894`\), \(-0.1972666616`\), 
              3852.154422`, \(-1.105785389`\), \(-1.421998508`\), 2, 3, 4, 5, 
              6, 0.464625`, \(-0.475019`\), 0.11878`, 0.188326`, 
              0.090637`, \(-0.081424`\), 0.880628`, 0.784738`, 0.930756`, 
              0.929269`, 1.259439`, 1.260317`, 16.5242`, 16.548423`, 
              1.003718`, 31.813859`, 31.793283`, \(-0.852617`\), 37.634518`, 
              37.642286`, 0.321883`, 53.024739`, 53.027062`, 0.09626`, 
              28.733076`, 28.747416`, 0.594232`, 56.291567`, 56.321557`, 
              1.242738`, 61.428382`, 61.425642`, \(-0.113536`\), 66.986073`, 
              66.961133`, \(-1.033462`\), 16.5242`, 31.813859`, 37.634518`, 
              58.406044`, 68.803032`, 83.484257`, 102.244134`, 103.297372`, 
              28.733076`, 56.291567`, 61.428382`, 66.986073`, 53.024739`, 
              75.706051`, 77.217561`, 82.353405`}, {2, 40, 
              28.80204336`, \(-0.787267923`\), 4.943818432`, 0.1921681542`, 
              1, \(-0.4234107898`\), \(-0.217082125`\), 
              24.12743121`, \(-0.7494070055`\), \(-1.360576748`\), 2, 3, 4, 
              5, 6, 0.524957`, \(-0.424866`\), \(-0.044819`\), 0.214489`, 
              0.0533`, \(-0.052378`\), 0.924397`, 0.806973`, 0.997695`, 
              0.978225`, 1.352787`, 1.364031`, 16.5242`, 16.5461`, 0.909383`, 
              31.294242`, 31.293652`, \(-0.024476`\), 34.967954`, 
              34.956354`, \(-0.481684`\), 51.602385`, 51.604172`, 0.074181`, 
              27.283185`, 27.294503`, 0.469993`, 55.609038`, 55.613993`, 
              0.205745`, 55.897899`, 55.918951`, 0.874182`, 63.727049`, 
              63.701923`, \(-1.043341`\), 16.5242`, 31.294242`, 34.967954`, 
              55.755055`, 66.192934`, 79.421504`, 97.57947`, 98.444182`, 
              27.283185`, 55.897899`, 55.609038`, 63.727049`, 51.602385`, 
              71.602448`, 78.046515`, 78.220387`}, {2, 73, 
              14.3933885`, \(-7.263677677`\), 22.70260985`, 0.25`, 1, 
              0.01307421984`, 0.007134481757`, \(-2.04170849`\), 
              3.964286687`, \(-2.725204359`\), 2, 3, 4, 5, 6, 
              0.808058`, \(-0.388575`\), \(-0.140322`\), 0.039809`, 
              0.27714`, \(-0.293726`\), 1.025231`, 1.075153`, 0.930244`, 
              1.081333`, 1.186937`, 1.099594`, 16.5242`, 16.527643`, 
              1.010615`, 207.659675`, 207.676025`, 4.799179`, 285.04826`, 
              285.046185`, \(-0.60914`\), 285.165255`, 285.166942`, 
              0.495121`, 146.822079`, 146.82526`, 0.93377`, 290.846822`, 
              290.848907`, 0.611772`, 388.324498`, 388.32528`, 0.229791`, 
              433.487909`, 433.489772`, 0.546701`, 16.5242`, 285.04826`, 
              207.659675`, 285.165255`, 434.230148`, 494.915`, 594.567468`, 
              605.63102`, 146.822079`, 290.846822`, 388.324498`, 433.487909`, 
              385.255065`, 486.340691`, 514.395497`, 
              549.519272`}}};\)\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Contour plot", "Subsection"],

Cell[BoxData[
    \(\(Length[
        data = Map[{Sqrt[#[\([6]\)]], #[\([9]\)], #[\([3]\)]} &, 
            datal2[\([2]\)]]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[
      d2 = Map[Sort[#, Function[{a, b}, OrderedQ[{Rest[a], Rest[b]}]]] &, 
          Partition[Sort[data], 12]]]\)], "Input"],

Cell[BoxData[
    \({11, 12, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ColumnForm[d2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\({{0, \(-0.35`\), 8112.62384`}, {0, \(-0.3`\), 
                6220.26202`}, {0, \(-0.25`\), 146107.054`}, {0, \(-0.2`\), 
                45012.90456`}, {0, \(-0.15`\), 
                6.829128776`}, {0, \(-0.1223884029`\), 
                6.463757838`}, {0, \(-0.1`\), 13.48393541`}, {0, \(-0.05`\), 
                39.5757694`}, {0, 0, 167.36759`}, {0, 0.05`, 
                129.1662164`}, {0, 0.1`, 43881.34237`}, {0, 0.15`, 
                12712.43577`}}\)},
          {\({{0.05`, \(-0.35`\), 18553.30596`}, {0.05`, \(-0.3`\), 
                9030.662981`}, {0.05`, \(-0.25`\), 
                120.1819142`}, {0.05`, \(-0.2`\), 
                52.11061983`}, {0.05`, \(-0.15`\), 
                8.074393059`}, {0.04435699123024464`, \(-0.1141404246`\), 
                7.239690877`}, {0.05`, \(-0.1`\), 
                9.692174315`}, {0.05`, \(-0.05`\), 27.05934154`}, {0.05`, 0, 
                135.5622059`}, {0.05`, 0.05`, 115209.1866`}, {0.05`, 0.1`, 
                5787.671968`}, {0.05`, 0.15`, 241134.2592`}}\)},
          {\({{0.1`, \(-0.35`\), 27793.11091`}, {0.1`, \(-0.3`\), 
                4841.231472`}, {0.1`, \(-0.25`\), 
                29.29295108`}, {0.1`, \(-0.2`\), 
                18.14269093`}, {0.1`, \(-0.15`\), 
                9.81270008`}, {0.08899001076525387`, \(-0.1182137623`\), 
                8.111494715`}, {0.1`, \(-0.1`\), 
                12.77802438`}, {0.1`, \(-0.05`\), 75.40294632`}, {0.1`, 0, 
                150.3414383`}, {0.1`, 0.05`, 7992.677415`}, {0.1`, 0.1`, 
                5717.968232`}, {0.1`, 0.15`, 364115.2798`}}\)},
          {\({{0.15`, \(-0.35`\), 63.05780077`}, {0.15`, \(-0.3`\), 
                34.60910919`}, {0.15`, \(-0.25`\), 
                15.17633094`}, {0.15`, \(-0.2`\), 
                12.92184563`}, {0.15`, \(-0.15`\), 
                12.24763413`}, {0.15`, \(-0.1`\), 
                14.24749435`}, {0.13418609067261777`, \(-0.08537929229`\), 
                8.341489309`}, {0.15`, \(-0.05`\), 76.70327831`}, {0.15`, 0, 
                108.5765528`}, {0.15`, 0.05`, 4.706222998`*^7}, {0.15`, 0.1`, 
                111755.9926`}, {0.15`, 0.15`, 268455.8884`}}\)},
          {\({{0.2`, \(-0.35`\), 23.52789225`}, {0.2`, \(-0.3`\), 
                20.02711043`}, {0.2`, \(-0.25`\), 
                17.39983703`}, {0.2`, \(-0.2`\), 
                14.110022`}, {0.2`, \(-0.15`\), 
                9.955953787`}, {0.18025529013041475`, \(-0.1209521116`\), 
                9.4115474`}, {0.2`, \(-0.1`\), 
                23.01951946`}, {0.2`, \(-0.05`\), 96.54642829`}, {0.2`, 0, 
                239.7708473`}, {0.2`, 0.05`, 7.188043967`*^6}, {0.2`, 0.1`, 
                120618.5886`}, {0.2`, 0.15`, 17459.99314`}}\)},
          {\({{0.25`, \(-0.35`\), 30.25118972`}, {0.25`, \(-0.3`\), 
                55.16029944`}, {0.25`, \(-0.25`\), 
                19.08044313`}, {0.25`, \(-0.2`\), 
                14.25847789`}, {0.25`, \(-0.15`\), 
                8.929737544`}, {0.22754493028850367`, \(-0.1388002796`\), 
                8.300556925`}, {0.25`, \(-0.1`\), 
                18.25811718`}, {0.25`, \(-0.05`\), 85.89514759`}, {0.25`, 0, 
                7389.40144`}, {0.25`, 0.05`, 3.591456048`*^7}, {0.25`, 0.1`, 
                41338.145`}, {0.25`, 0.15`, 114.6886096`}}\)},
          {\({{0.3`, \(-0.35`\), 195.8741992`}, {0.3`, \(-0.3`\), 
                29.24473859`}, {0.3`, \(-0.25`\), 
                19.99020407`}, {0.3`, \(-0.2`\), 
                13.382742`}, {0.2764576231902459`, \(-0.1582036385`\), 
                9.332606961`}, {0.3`, \(-0.15`\), 
                11.12250316`}, {0.3`, \(-0.1`\), 
                14.01795775`}, {0.3`, \(-0.05`\), 86.14123258`}, {0.3`, 0, 
                128.5835457`}, {0.3`, 0.05`, 15929.76771`}, {0.3`, 0.1`, 
                26866.35127`}, {0.3`, 0.15`, 194211.5421`}}\)},
          {\({{0.35`, \(-0.35`\), 250.4055322`}, {0.35`, \(-0.3`\), 
                33.99544929`}, {0.35`, \(-0.25`\), 
                20.26491933`}, {0.35`, \(-0.2`\), 
                14.8144333`}, {0.3274754004501712`, \(-0.164496886`\), 
                11.70458489`}, {0.35`, \(-0.15`\), 
                19.48096544`}, {0.35`, \(-0.1`\), 
                28.13177892`}, {0.35`, \(-0.05`\), 37.01794732`}, {0.35`, 0, 
                128.6698981`}, {0.35`, 0.05`, 4300.243371`}, {0.35`, 0.1`, 
                11184.43927`}, {0.35`, 0.15`, 25566.65145`}}\)},
          {\({{0.4`, \(-0.35`\), 54.38973181`}, {0.4`, \(-0.3`\), 
                34.88784785`}, {0.4`, \(-0.25`\), 
                23.54076314`}, {0.4`, \(-0.2`\), 
                20.54258057`}, {0.38119352775198057`, \(-0.1972666616`\), 
                15.75843486`}, {0.4`, \(-0.15`\), 
                33.90971015`}, {0.4`, \(-0.1`\), 
                80.89311339`}, {0.4`, \(-0.05`\), 69.11803596`}, {0.4`, 0, 
                131.4001932`}, {0.4`, 0.05`, 207.3581082`}, {0.4`, 0.1`, 
                150.6750924`}, {0.4`, 0.15`, 6275535789}}\)},
          {\({{0.45`, \(-0.35`\), 1.320052612`*^6}, {0.45`, \(-0.3`\), 
                55.73306761`}, {0.45`, \(-0.25`\), 
                37.38460392`}, {0.43836988286149403`, \(-0.217082125`\), 
                28.80204336`}, {0.45`, \(-0.2`\), 
                44.17145461`}, {0.45`, \(-0.15`\), 
                35.93993287`}, {0.45`, \(-0.1`\), 
                74.08461976`}, {0.45`, \(-0.05`\), 159.8667665`}, {0.45`, 0, 
                125.9312698`}, {0.45`, 0.05`, 187.0043204`}, {0.45`, 0.1`, 
                386.2305933`}, {0.45`, 0.15`, 174.1819358`}}\)},
          {\({{0.5`, \(-0.35`\), 19009.47476`}, {0.5`, \(-0.3`\), 
                21302.03328`}, {0.5`, \(-0.25`\), 
                2230.491095`}, {0.5`, \(-0.2`\), 
                21.7666919`}, {0.5`, \(-0.15`\), 
                40.17585357`}, {0.5`, \(-0.1`\), 
                73.96341681`}, {0.5`, \(-0.05`\), 53.92311192`}, {0.5`, 0, 
                400.0740305`}, {0.5`, 0.007134481757`, 14.3933885`}, {0.5`, 
                0.05`, 298.6341609`}, {0.5`, 0.1`, 220.8741723`}, {0.5`, 
                0.15`, 123.947963`}}\)}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      ColumnForm[ {{{0, -0.34999999999999998, 8112.6238400000002}, {
        0, -0.29999999999999999, 6220.2620200000001}, {0, -0.25, 
        146107.054}, {0, -0.20000000000000001, 45012.904560000003}, {
        0, -0.14999999999999999, 6.8291287760000001}, {0, -0.1223884029, 
        6.4637578380000003}, {0, -0.10000000000000001, 13.483935410000001}, {
        0, -0.050000000000000003, 39.575769399999999}, {0, 0, 
        167.36759000000001}, {0, 0.050000000000000003, 129.1662164}, {0, 
        0.10000000000000001, 43881.342369999998}, {0, 0.14999999999999999, 
        12712.43577}}, {{0.050000000000000003, -0.34999999999999998, 
        18553.305960000002}, {0.050000000000000003, -0.29999999999999999, 
        9030.6629809999995}, {0.050000000000000003, -0.25, 
        120.18191419999999}, {0.050000000000000003, -0.20000000000000001, 
        52.110619829999997}, {0.050000000000000003, -0.14999999999999999, 
        8.0743930590000002}, {0.044356991230244641, -0.1141404246, 
        7.2396908770000001}, {0.050000000000000003, -0.10000000000000001, 
        9.6921743150000008}, {0.050000000000000003, -0.050000000000000003, 
        27.059341539999998}, {0.050000000000000003, 0, 135.56220590000001}, {
        0.050000000000000003, 0.050000000000000003, 115209.1866}, {
        0.050000000000000003, 0.10000000000000001, 5787.6719679999997}, {
        0.050000000000000003, 0.14999999999999999, 241134.2592}}, {{
        0.10000000000000001, -0.34999999999999998, 27793.110909999999}, {
        0.10000000000000001, -0.29999999999999999, 4841.2314720000004}, {
        0.10000000000000001, -0.25, 29.292951080000002}, {
        0.10000000000000001, -0.20000000000000001, 18.142690930000001}, {
        0.10000000000000001, -0.14999999999999999, 9.8127000800000008}, {
        0.088990010765253869, -0.1182137623, 8.1114947149999992}, {
        0.10000000000000001, -0.10000000000000001, 12.77802438}, {
        0.10000000000000001, -0.050000000000000003, 75.402946319999998}, {
        0.10000000000000001, 0, 150.34143829999999}, {0.10000000000000001, 
        0.050000000000000003, 7992.6774150000001}, {0.10000000000000001, 
        0.10000000000000001, 5717.9682320000002}, {0.10000000000000001, 
        0.14999999999999999, 364115.27980000002}}, {{
        0.14999999999999999, -0.34999999999999998, 63.05780077}, {
        0.14999999999999999, -0.29999999999999999, 34.609109189999998}, {
        0.14999999999999999, -0.25, 15.17633094}, {
        0.14999999999999999, -0.20000000000000001, 12.92184563}, {
        0.14999999999999999, -0.14999999999999999, 12.24763413}, {
        0.14999999999999999, -0.10000000000000001, 14.24749435}, {
        0.13418609067261777, -0.085379292290000006, 8.341489309}, {
        0.14999999999999999, -0.050000000000000003, 76.703278310000002}, {
        0.14999999999999999, 0, 108.5765528}, {0.14999999999999999, 
        0.050000000000000003, 47062229.979999997}, {0.14999999999999999, 
        0.10000000000000001, 111755.9926}, {0.14999999999999999, 
        0.14999999999999999, 268455.8884}}, {{
        0.20000000000000001, -0.34999999999999998, 23.527892250000001}, {
        0.20000000000000001, -0.29999999999999999, 20.02711043}, {
        0.20000000000000001, -0.25, 17.39983703}, {
        0.20000000000000001, -0.20000000000000001, 14.110022000000001}, {
        0.20000000000000001, -0.14999999999999999, 9.9559537870000003}, {
        0.18025529013041475, -0.1209521116, 9.4115473999999999}, {
        0.20000000000000001, -0.10000000000000001, 23.019519460000001}, {
        0.20000000000000001, -0.050000000000000003, 96.546428289999994}, {
        0.20000000000000001, 0, 239.77084730000001}, {0.20000000000000001, 
        0.050000000000000003, 7188043.9670000002}, {0.20000000000000001, 
        0.10000000000000001, 120618.5886}, {0.20000000000000001, 
        0.14999999999999999, 17459.993139999999}}, {{
        0.25, -0.34999999999999998, 30.251189719999999}, {
        0.25, -0.29999999999999999, 55.160299440000003}, {0.25, -0.25, 
        19.080443129999999}, {0.25, -0.20000000000000001, 14.25847789}, {
        0.25, -0.14999999999999999, 8.929737544}, {
        0.22754493028850367, -0.13880027959999999, 8.3005569250000004}, {
        0.25, -0.10000000000000001, 18.258117179999999}, {
        0.25, -0.050000000000000003, 85.895147589999993}, {0.25, 0, 
        7389.4014399999996}, {0.25, 0.050000000000000003, 
        35914560.479999997}, {0.25, 0.10000000000000001, 
        41338.144999999997}, {0.25, 0.14999999999999999, 
        114.68860960000001}}, {{0.29999999999999999, -0.34999999999999998, 
        195.87419919999999}, {0.29999999999999999, -0.29999999999999999, 
        29.244738590000001}, {0.29999999999999999, -0.25, 
        19.990204070000001}, {0.29999999999999999, -0.20000000000000001, 
        13.382742}, {0.27645762319024592, -0.15820363849999999, 
        9.3326069609999998}, {0.29999999999999999, -0.14999999999999999, 
        11.122503160000001}, {0.29999999999999999, -0.10000000000000001, 
        14.017957750000001}, {0.29999999999999999, -0.050000000000000003, 
        86.141232579999993}, {0.29999999999999999, 0, 128.5835457}, {
        0.29999999999999999, 0.050000000000000003, 15929.76771}, {
        0.29999999999999999, 0.10000000000000001, 26866.351269999999}, {
        0.29999999999999999, 0.14999999999999999, 194211.54209999999}}, {{
        0.34999999999999998, -0.34999999999999998, 250.40553220000001}, {
        0.34999999999999998, -0.29999999999999999, 33.995449290000003}, {
        0.34999999999999998, -0.25, 20.264919330000001}, {
        0.34999999999999998, -0.20000000000000001, 14.814433299999999}, {
        0.32747540045017121, -0.16449688600000001, 11.70458489}, {
        0.34999999999999998, -0.14999999999999999, 19.480965439999999}, {
        0.34999999999999998, -0.10000000000000001, 28.131778919999999}, {
        0.34999999999999998, -0.050000000000000003, 37.017947319999998}, {
        0.34999999999999998, 0, 128.66989810000001}, {0.34999999999999998, 
        0.050000000000000003, 4300.2433709999996}, {0.34999999999999998, 
        0.10000000000000001, 11184.439270000001}, {0.34999999999999998, 
        0.14999999999999999, 25566.651450000001}}, {{
        0.40000000000000002, -0.34999999999999998, 54.389731810000001}, {
        0.40000000000000002, -0.29999999999999999, 34.88784785}, {
        0.40000000000000002, -0.25, 23.540763139999999}, {
        0.40000000000000002, -0.20000000000000001, 20.542580569999998}, {
        0.38119352775198057, -0.1972666616, 15.758434859999999}, {
        0.40000000000000002, -0.14999999999999999, 33.909710150000002}, {
        0.40000000000000002, -0.10000000000000001, 80.893113389999996}, {
        0.40000000000000002, -0.050000000000000003, 69.11803596}, {
        0.40000000000000002, 0, 131.40019319999999}, {0.40000000000000002, 
        0.050000000000000003, 207.3581082}, {0.40000000000000002, 
        0.10000000000000001, 150.67509240000001}, {0.40000000000000002, 
        0.14999999999999999, 6275535789}}, {{
        0.45000000000000001, -0.34999999999999998, 1320052.612}, {
        0.45000000000000001, -0.29999999999999999, 55.733067609999999}, {
        0.45000000000000001, -0.25, 37.384603920000004}, {
        0.43836988286149403, -0.21708212499999999, 28.802043359999999}, {
        0.45000000000000001, -0.20000000000000001, 44.171454609999998}, {
        0.45000000000000001, -0.14999999999999999, 35.93993287}, {
        0.45000000000000001, -0.10000000000000001, 74.084619759999995}, {
        0.45000000000000001, -0.050000000000000003, 159.86676650000001}, {
        0.45000000000000001, 0, 125.9312698}, {0.45000000000000001, 
        0.050000000000000003, 187.00432040000001}, {0.45000000000000001, 
        0.10000000000000001, 386.23059330000001}, {0.45000000000000001, 
        0.14999999999999999, 174.18193579999999}}, {{
        0.5, -0.34999999999999998, 19009.474760000001}, {
        0.5, -0.29999999999999999, 21302.03328}, {0.5, -0.25, 
        2230.4910949999999}, {0.5, -0.20000000000000001, 
        21.766691900000001}, {0.5, -0.14999999999999999, 
        40.175853570000001}, {0.5, -0.10000000000000001, 
        73.963416809999998}, {0.5, -0.050000000000000003, 
        53.923111919999997}, {0.5, 0, 400.07403049999999}, {0.5, 
        0.0071344817569999998, 14.3933885}, {0.5, 0.050000000000000003, 
        298.63416089999998}, {0.5, 0.10000000000000001, 220.8741723}, {0.5, 
        0.14999999999999999, 123.947963}}}],
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppd2 = 
      ListSurfacePlot3D[Transpose[d2], 
        PlotRange \[Rule] {{0, 0.5}, {\(- .35\), 0}, {0, 110}}, 
        BoxRatios \[Rule] {1, 1, 0.5}, 
        ViewPoint \[Rule] {\(-1.6\), 0.6, 1.65}, Axes \[Rule] True, 
        AxesLabel \[Rule] {"\<m\>", \*"\"\<\!\(l\_2\)\>\"", \*"\"\<\!\(\[Chi]\
\^2\)\>\""}, 
        TextStyle -> {FontSize \[Rule] 
              12}, \[IndentingNewLine]LightSources \[Rule] {{{0, 1, 1}, 
              Hue[0.1, 0.8, 1]}, {{0, \(-1\), 1}, 
              Hue[0.6, 0.8, 1]}} (*\(,\)\(AmbientLight \[Rule] 
            GrayLevel[0.05]\)*) ]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<dvalley2.jpg\>", pppd2, "\<JPEG\>", 
      ImageSize \[Rule] 72*4]\)], "Input"],

Cell[BoxData[
    \("dvalley2.jpg"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppd23 = 
      Block[{theta = Pi/2}, 
        Show[pppd2, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize \[Rule] 12}, 
          FormatType -> OutputForm, AxesLabel \[Rule] None, 
          ColorOutput \[Rule] GrayLevel, 
          LightSources \[Rule] {{{Cos[theta], Sin[theta], 2}, 
                GrayLevel[
                  0.75]} (*\(,\)\({{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                  GrayLevel[0.5]}\)*) }, 
          AmbientLight \[Rule] GrayLevel[0.25]]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<dvalley2.eps\>", pppd23, "\<EPS\>", 
      ImageSize \[Rule] 72*5.5]\)], "Input"],

Cell[BoxData[
    \("dvalley2.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
datal2 l2_bscan1*.out (This contains points from btraj2*.out)\
\>", \
"Subsection"],

Cell[BoxData[
    \(\(datal2 = {"\<********** Eigenvalues for the 2+1 transverse lattice \
**********\nCouplings:  m^2, G^2 N, la_1, la_2, la_3, tau_1, tau_2          \n\
             0      1     2     3     4      5      6   (2-6 /a)\nUse chi^2 \
fit with 14 criteria, and tolerance 0.001.  \nOverall scale from minimizing \
chi^2.\n2 parity doublets with fractional errors 1 0.1.\nSpectrum for P_perp \
a = (0)  ( 0.25) \n using (# states, o, multiplet, c^2 error for each) =\n  \
(4, 1 & -1, 1 & 2, 0.1 2 2 0.5)\n  (4, 1 & -1, 2 & 1, 0.25 0.25 1 1).\n\
Spectra extrapolated using (K,p) = (20/2,6)  (20/2,8) \n  (26/2,6)  (32/2,6) \
.\nWinding potential using (n,K,p) = ( 2,20/2,4)  ( 2,20/2,6) \n  ( 2,24/2,4) \
 ( 2,34/2,4)  ( 3,21/2,5)  ( 3,21/2,7)  ( 3,25/2,5) \n  ( 3,35/2,5)  ( \
4,20/2,6)  ( 4,20/2,8)  ( 4,24/2,6)  ( 4,34/2,6) .\nHeavy potential \
determined using (n,K,p,K_max) = ( 1,-40/2,2,4) \n  ( 1,-40/2,4,4)  ( \
1,-40/2,2,3.5)  ( 1,-50/2,2,3.75)  ( 1,-50/2,2,4.25) \n  ( 1,-70/2,2,4)  ( \
1,-70/2,2,4.5) ,\n L = 3 4 6 (all in G^2 N units); relative scale error 0.1.\n\
Roundness determined using (n,K,p,K_max) = ( 1,-21/2,3,4) \n  ( 1,-21/2,5,4)  \
( 1,-21/2,3,3.5)  ( 1,-35/2,3,4)  ( 1,-35/2,3,5) \n  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=0 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=2.5 and error 0.1; \n  ( 1,-21/2,3,4)  ( 1,-21/2,5,4)  ( \
1,-21/2,3,3.5)  ( 1,-35/2,3,4) \n  ( 1,-35/2,3,5)  ( 1,-69/2,3,5)  ( \
1,-69/2,3,6) \n L=5 and error 0.1; all in G^2 N units.\np-extrapolation using \
n=( 1) and (K,p) = (21/2,3)  (27/2,3) \n  (39/2,3)  (21/2,5)  (27/2,5)  \
(21/2,7)  (23/2,7) .\nResult format:\n Fit info, # steps, chi^2, p damping, \
and scale G^2 N/sigma.\n The 7 couplings (G^2 N units) and which--if \
any--were fit.\n Winding potential and heavy source potential fits.\n \
Roundness with calculated and derived values (G^2 N units).\n The rescaled \
spectrum for each P_perp*a and c^2 values.\n Finally come the states for the \
ordinary spectra.\>", {{3, 47, 6.369872612`, \(-1.710097193`\), 7.578522352`, 
              0, 1, \(-0.004693500257`\), \(-0.114567563`\), 
              2738.711613`, \(-0.6630839669`\), \(-0.6714368651`\), 2, 3, 4, 
              5, 6, 0.191087`, \(-0.422064`\), 0.253378`, 0.132402`, 
              0.235192`, \(-0.224878`\), 0.621288`, 0.605939`, 0.709826`, 
              0.730517`, 1.003683`, 0.991837`, 17.95654`, 18.001823`, 
              1.049223`, 28.275967`, 28.246208`, \(-0.689542`\), 35.851152`, 
              35.877232`, 0.604289`, 46.617879`, 46.625342`, 0.172928`, 
              24.087495`, 24.120067`, 0.75473`, 46.002772`, 46.040372`, 
              0.871213`, 60.272327`, 60.276033`, 0.085884`, 64.134569`, 
              64.169114`, 0.800426`, 17.95654`, 28.275967`, 35.851152`, 
              51.999544`, 60.272327`, 77.2951`, 91.782172`, 98.074476`, 
              24.087495`, 46.002772`, 64.134569`, 66.994053`, 46.617879`, 
              67.694651`, 75.316992`, 83.250334`}, {1, 53, 
              7.349227765`, \(-1.454957284`\), 7.314588575`, 0.0025`, 
              1, \(-0.05543702533`\), \(-0.1081262544`\), 
              1062.96234`, \(-0.7481706764`\), \(-0.7871380194`\), 2, 3, 4, 
              5, 6, 0.225674`, \(-0.449297`\), 0.287104`, 0.137514`, 
              0.219964`, \(-0.209276`\), 0.660361`, 0.637571`, 0.739298`, 
              0.759729`, 1.034728`, 1.022897`, 17.785889`, 17.825676`, 
              1.050836`, 27.169455`, 27.138941`, \(-0.805913`\), 36.450549`, 
              36.473755`, 0.6129`, 47.002199`, 47.007609`, 0.142865`, 
              25.316896`, 25.345251`, 0.748889`, 47.613518`, 47.644962`, 
              0.830489`, 61.641914`, 61.639827`, \(-0.05513`\), 64.796446`, 
              64.813804`, 0.458461`, 17.785889`, 27.169455`, 36.450549`, 
              52.132068`, 61.641914`, 78.430141`, 92.271577`, 102.098391`, 
              25.316896`, 47.613518`, 64.796446`, 66.823859`, 47.002199`, 
              66.882649`, 76.751054`, 76.926475`}, {2, 43, 
              8.135087732`, \(-1.315172513`\), 7.251136348`, 0.01`, 
              1, \(-0.09526007516`\), \(-0.1061584742`\), 
              3527.743867`, \(-0.886839406`\), \(-0.9286899822`\), 2, 3, 4, 
              5, 6, 0.262328`, \(-0.469771`\), 0.307501`, 0.138867`, 
              0.205737`, \(-0.191172`\), 0.69532`, 0.669999`, 0.76045`, 
              0.784735`, 1.054031`, 1.039964`, 18.756777`, 18.791116`, 
              1.045099`, 28.02928`, 28.001499`, \(-0.845525`\), 38.066859`, 
              38.086895`, 0.609806`, 49.588918`, 49.592895`, 0.121062`, 
              27.020287`, 27.044846`, 0.747439`, 51.002834`, 51.030515`, 
              0.842464`, 65.064881`, 65.056319`, \(-0.260606`\), 66.794897`, 
              66.801071`, 0.187886`, 18.756777`, 28.02928`, 38.066859`, 
              54.573381`, 65.064881`, 82.173756`, 97.701419`, 106.42545`, 
              27.020287`, 51.002834`, 66.794897`, 69.054148`, 49.588918`, 
              70.599356`, 79.57257`, 82.852506`}, {3, 67, 
              8.339396601`, \(-1.409207614`\), 7.64254067`, 0.0225`, 
              1, \(-0.1213305738`\), \(-0.09585391155`\), 
              4205.936202`, \(-1.142357493`\), \(-1.133311771`\), 2, 3, 4, 5, 
              6, 0.313229`, \(-0.464664`\), 0.268646`, 0.13112`, 
              0.201963`, \(-0.181753`\), 0.732891`, 0.720308`, 0.774663`, 
              0.814886`, 1.060319`, 1.037097`, 24.170957`, 24.197983`, 
              1.035141`, 33.352044`, 33.330111`, \(-0.840083`\), 42.534163`, 
              42.551573`, 0.666823`, 58.174656`, 58.177133`, 0.094874`, 
              30.775919`, 30.795345`, 0.744057`, 60.338882`, 60.363153`, 
              0.929623`, 73.660164`, 73.650816`, \(-0.358055`\), 74.929454`, 
              74.933174`, 0.14249`, 24.170957`, 33.352044`, 42.534163`, 
              66.338217`, 74.929454`, 93.559979`, 114.181846`, 119.958127`, 
              30.775919`, 60.338882`, 73.660164`, 77.610039`, 58.174656`, 
              83.702795`, 89.86553`, 95.809009`}, {2, 47, 
              8.133791511`, \(-1.110162749`\), 7.081290114`, 0.04`, 
              1, \(-0.173629971`\), \(-0.1186141863`\), 
              865.4865301`, \(-1.163730934`\), \(-1.21010522`\), 2, 3, 4, 5, 
              6, 0.336106`, \(-0.485057`\), 0.286535`, 0.142898`, 
              0.177012`, \(-0.158381`\), 0.761515`, 0.72508`, 0.802064`, 
              0.830923`, 1.093649`, 1.077004`, 20.46555`, 20.492582`, 
              1.029412`, 31.581239`, 31.558863`, \(-0.852135`\), 40.977767`, 
              40.992552`, 0.563006`, 55.270047`, 55.272771`, 0.103731`, 
              30.006111`, 30.025699`, 0.745926`, 57.695198`, 57.720032`, 
              0.945706`, 70.165117`, 70.157796`, \(-0.278804`\), 71.987414`, 
              71.990164`, 0.104715`, 20.46555`, 31.581239`, 40.977767`, 
              61.043323`, 71.987414`, 89.5252`, 110.471154`, 114.273568`, 
              30.006111`, 57.695198`, 70.165117`, 73.549445`, 55.270047`, 
              75.963954`, 89.363369`, 91.686451`}, {2, 46, 
              8.325858404`, \(-0.8902887089`\), 6.488769654`, 0.0625`, 
              1, \(-0.2285961599`\), \(-0.1467998049`\), 
              1450.518496`, \(-1.135220771`\), \(-1.26481196`\), 2, 3, 4, 5, 
              6, 0.358613`, \(-0.511499`\), 0.310217`, 0.157993`, 
              0.147817`, \(-0.131384`\), 0.790127`, 0.727926`, 0.833048`, 
              0.849389`, 1.133698`, 1.12754`, 16.730731`, 16.758272`, 
              1.025383`, 29.581138`, 29.557926`, \(-0.864222`\), 38.962676`, 
              38.975216`, 0.466911`, 51.934285`, 51.937265`, 0.110938`, 
              28.833682`, 28.853104`, 0.723103`, 54.458057`, 54.48353`, 
              0.9484`, 65.987463`, 65.981135`, \(-0.235612`\), 68.30968`, 
              68.314361`, 0.17428`, 16.730731`, 29.581138`, 38.962676`, 
              56.109421`, 68.30968`, 84.655208`, 99.697386`, 111.76613`, 
              28.833682`, 54.458057`, 65.987463`, 68.892926`, 51.934285`, 
              71.411538`, 83.225572`, 87.762145`}, {1, 26, 
              10.42272203`, \(-0.592333783`\), 6.153766281`, 0.09`, 
              1, \(-0.3032787178`\), \(-0.1748147515`\), 
              4712.170652`, \(-1.140681792`\), \(-1.362717144`\), 2, 3, 4, 5, 
              6, 0.370917`, \(-0.609799`\), 0.477798`, 0.168006`, 
              0.11823`, \(-0.0959`\), 0.813768`, 0.7223`, 0.850991`, 
              0.853833`, 1.15433`, 1.152703`, 9.90408`, 9.931327`, 0.995085`, 
              25.997692`, 25.974183`, \(-0.858567`\), 37.7311`, 37.742483`, 
              0.415736`, 48.033231`, 48.036686`, 0.126199`, 28.293726`, 
              28.314672`, 0.764952`, 50.927077`, 50.954873`, 1.015116`, 
              63.379592`, 63.365109`, \(-0.528925`\), 64.587719`, 
              64.575236`, \(-0.455868`\), 9.90408`, 25.997692`, 37.7311`, 
              48.982458`, 64.982183`, 80.768752`, 88.031497`, 101.852415`, 
              28.293726`, 50.927077`, 63.379592`, 64.587719`, 48.033231`, 
              65.698905`, 71.126432`, 88.710296`}, {2, 37, 
              8.52311789`, \(-0.4421933085`\), 5.837450747`, 0.1225`, 
              1, \(-0.3711845869`\), \(-0.2015905316`\), 
              546.4384415`, \(-1.124026409`\), \(-1.451080118`\), 2, 3, 4, 5, 
              6, 0.393565`, \(-0.710751`\), 0.627048`, 0.179116`, 
              0.089896`, \(-0.064081`\), 0.842068`, 0.728341`, 0.87647`, 
              0.869659`, 1.185745`, 1.189659`, 5.324506`, 5.351116`, 
              0.978162`, 23.933439`, 23.910184`, \(-0.854829`\), 36.739305`, 
              36.749264`, 0.366088`, 45.654601`, 45.674961`, 0.748401`, 
              27.988739`, 28.009362`, 0.75808`, 49.050565`, 49.080199`, 
              1.089309`, 61.413593`, 61.400355`, \(-0.486605`\), 61.689097`, 
              61.674527`, \(-0.535546`\), 5.324506`, 23.933439`, 36.739305`, 
              45.654601`, 63.097867`, 78.450899`, 85.458783`, 98.168414`, 
              27.988739`, 49.050565`, 61.413593`, 61.689097`, 45.907508`, 
              64.65393`, 64.146348`, 74.733789`}, {2, 34, 
              10.94888151`, \(-0.3796124889`\), 5.591995421`, 0.16`, 
              1, \(-0.4383723302`\), \(-0.2230772747`\), 
              147.4570973`, \(-1.192892929`\), \(-1.643612253`\), 2, 3, 4, 5, 
              6, 0.427594`, \(-0.768733`\), 0.679329`, 0.184094`, 
              0.070158`, \(-0.040229`\), 0.874178`, 0.732322`, 0.890553`, 
              0.876531`, 1.195045`, 1.204021`, 2.76621`, 2.791108`, 
              0.952515`, 23.482012`, 23.46027`, \(-0.83181`\), 36.318867`, 
              36.327288`, 0.322195`, 45.517041`, 45.537768`, 0.792942`, 
              28.170015`, 28.189001`, 0.726366`, 49.164842`, 49.196947`, 
              1.228238`, 60.435166`, 60.427088`, \(-0.309021`\), 60.45109`, 
              60.428341`, \(-0.870332`\), 2.76621`, 23.482012`, 36.318867`, 
              45.517041`, 63.028432`, 78.099926`, 88.082782`, 97.34302`, 
              28.170015`, 49.164842`, 60.435166`, 60.45109`, 45.640288`, 
              67.499908`, 61.436363`, 72.379986`}, {2, 33, 
              16.64275753`, \(-0.7862275025`\), 6.778186335`, 0.2025`, 
              1, \(-0.454417308`\), \(-0.1731850841`\), 
              5964.667405`, \(-1.880616501`\), \(-1.989617105`\), 2, 3, 4, 5, 
              6, 0.553424`, \(-0.403181`\), \(-0.07697`\), 0.151857`, 
              0.114758`, \(-0.093055`\), 0.931717`, 0.862035`, 0.912786`, 
              0.952901`, 1.203459`, 1.180012`, 22.391021`, 22.407348`, 
              0.979881`, 44.345512`, 44.327159`, \(-1.101518`\), 48.583071`, 
              48.595396`, 0.739744`, 71.540562`, 71.542087`, 0.091535`, 
              37.950586`, 37.962922`, 0.740424`, 77.413824`, 77.429359`, 
              0.932389`, 77.826079`, 77.840959`, 0.89311`, 86.848951`, 
              86.821184`, \(-1.666574`\), 22.391021`, 44.345512`, 48.583071`, 
              78.368229`, 92.021899`, 110.122941`, 133.90166`, 136.403334`, 
              37.950586`, 77.826079`, 77.413824`, 86.848951`, 71.540562`, 
              97.983419`, 107.2518`, 108.497392`}, {\(-2\), 7, 
              43.4135314`, \(-0.6599962362`\), 4.229561016`, 0.25`, 
              1, \(-0.50272`\), \(-0.307886`\), 
              50, \(-1.45567`\), \(-1.79297`\), 2, 3, 4, 5, 6, 
              0.553185`, \(-0.448256`\), \(-0.107535`\), 0.187429`, 
              0.073234`, \(-0.059361`\), 0.96658`, 0.643977`, 0.990132`, 
              0.81159`, 1.319973`, 1.170382`, 13.516802`, 13.546507`, 
              1.112021`, 29.166656`, 29.148763`, \(-0.669829`\), 31.191782`, 
              31.18914`, \(-0.098904`\), 46.325922`, 46.328361`, 0.091318`, 
              24.663032`, 24.674602`, 0.433133`, 48.883293`, 48.901709`, 
              0.689429`, 49.974225`, 49.984704`, 0.392291`, 57.517343`, 
              57.479834`, \(-1.40418`\), 13.516802`, 29.166656`, 31.191782`, 
              50.496819`, 59.540379`, 71.142359`, 86.307301`, 87.537013`, 
              24.663032`, 49.974225`, 48.883293`, 57.517343`, 46.325922`, 
              63.8279`, 70.662345`, 72.097573`}, {5, 28, 
              141.627678`, \(-0.5345067416`\), 4.847761783`, 0, 1, 
              0.1914031551`, \(-0.35`\), 
              10.96199988`, \(-0.6549762575`\), \(-0.2727311781`\), 2, 4, 5, 
              6, 0.053457`, \(-0.739537`\), 0.90292`, 0.137116`, 
              0.182681`, \(-0.134745`\), 0.53654`, 0.216149`, 0.696582`, 
              0.509165`, 1.030336`, 0.858979`, 1.964622`, 2.059809`, 
              0.39468`, 16.818325`, 16.845639`, 0.113252`, 17.383864`, 
              17.46868`, 0.351681`, 19.75832`, 19.096858`, \(-2.742659`\), 
              10.478651`, 10.537548`, 0.244209`, 18.92151`, 18.972102`, 
              0.209771`, 30.070962`, 30.109056`, 0.157948`, 31.631518`, 
              31.700641`, 0.286608`, 1.964622`, 17.383864`, 16.818325`, 
              19.75832`, 30.070962`, 40.020313`, 50.873279`, 51.129105`, 
              10.478651`, 18.92151`, 31.631518`, 33.796154`, 22.904641`, 
              36.02353`, 44.152671`, 45.877137`}, {\(-3\), 6, 
              277.0667036`, \(-0.9955688986`\), 1.35205971`, 0, 1, 
              0.169532`, \(-0.3`\), 50, \(-0.793728`\), \(-0.276775`\), 2, 4, 
              5, 6, 0.090301`, \(-0.592613`\), 0.606084`, 0.127801`, 
              0.198704`, \(-0.156021`\), 0.559368`, \(-0.133381`\), 
              0.722717`, 0.460799`, 1.062197`, 0.808715`, 1.721297`, 
              1.741888`, 0.040224`, 5.297555`, 5.302705`, 0.010059`, 
              6.238669`, 6.241198`, 0.00494`, 7.145644`, 
              6.485484`, \(-1.289606`\), 3.31743`, 3.329593`, 0.023761`, 
              6.337716`, 6.350328`, 0.024637`, 9.361651`, 9.368539`, 
              0.013455`, 10.394314`, 10.410518`, 0.031655`, 1.721297`, 
              6.238669`, 5.297555`, 7.145644`, 9.361651`, 12.27664`, 
              16.047784`, 16.087777`, 3.31743`, 6.337716`, 10.394314`, 
              10.495926`, 7.441223`, 11.993673`, 12.996758`, 13.672488`}, {5, 
              25, 199.0188878`, \(-1.302630847`\), 1.808939205`, 0, 1, 
              0.1295821284`, \(-0.25`\), 40958.53166`, 
              0.1818998942`, \(-0.4219852542`\), 2, 4, 5, 6, 
              0.121988`, \(-0.516315`\), 0.440169`, 0.205006`, 
              0.070402`, \(-0.075695`\), 0.5812`, 0.080119`, 0.725929`, 
              0.569674`, 1.055162`, 1.088561`, 3.232613`, 3.254833`, 
              0.078452`, 3.23367`, 3.281741`, 0.169724`, 10.703299`, 
              9.971867`, \(-2.582477`\), 11.083223`, 11.845537`, 2.691514`, 
              4.867934`, 4.881095`, 0.046466`, 9.462566`, 9.477654`, 
              0.053271`, 13.354011`, 13.360871`, 0.024221`, 14.44449`, 
              14.460562`, 0.056744`, 3.232613`, 3.23367`, 11.083223`, 
              12.845078`, 13.354011`, 17.346693`, 22.625236`, 22.90537`, 
              4.867934`, 9.462566`, 14.44449`, 15.501749`, 10.703299`, 
              17.409327`, 18.732065`, 19.177997`}, {5, 27, 
              7.88096895`, \(-1.660600703`\), 7.182079336`, 0, 1, 
              0.1610503156`, \(-0.2`\), 
              28953.30702`, \(-0.7115666387`\), \(-0.6310892183`\), 2, 4, 5, 
              6, 0.154386`, \(-0.496882`\), 0.421248`, 0.140853`, 
              0.202268`, \(-0.187251`\), 0.588747`, 0.533399`, 0.68286`, 
              0.68363`, 0.978686`, 0.978347`, 19.076967`, 19.136811`, 
              1.061688`, 30.877627`, 30.898262`, 0.366089`, 35.55326`, 
              35.52949`, \(-0.421704`\), 47.025367`, 47.035838`, 0.185772`, 
              20.201807`, 20.236187`, 0.60994`, 42.345384`, 42.391213`, 
              0.81305`, 56.389443`, 56.405832`, 0.290755`, 58.96843`, 
              59.011636`, 0.766524`, 19.076967`, 35.55326`, 30.877627`, 
              54.176646`, 56.389443`, 72.745519`, 96.016416`, 96.060057`, 
              20.201807`, 42.345384`, 58.96843`, 65.029144`, 47.025367`, 
              71.940165`, 74.553981`, 85.315373`}, {2, 18, 
              8.65228068`, \(-1.863388003`\), 7.740170749`, 0, 1, 
              0.1372743036`, \(-0.15`\), 
              8097.116586`, \(-0.8194157262`\), \(-0.6984230654`\), 2, 4, 5, 
              6, 0.181137`, \(-0.466923`\), 0.35777`, 0.130471`, 
              0.215738`, \(-0.196503`\), 0.602615`, 0.585888`, 0.685933`, 
              0.707776`, 0.976455`, 0.964132`, 23.788973`, 23.835216`, 
              1.037347`, 26.309211`, 26.396898`, 1.967041`, 45.779748`, 
              45.691764`, \(-1.973705`\), 52.775179`, 52.782782`, 0.170546`, 
              22.935493`, 22.963753`, 0.633944`, 48.921828`, 48.960744`, 
              0.872993`, 63.11709`, 63.124442`, 0.164921`, 64.861297`, 
              64.899368`, 0.854022`, 23.788973`, 26.309211`, 45.779748`, 
              63.558878`, 63.11709`, 81.02356`, 105.782123`, 107.761623`, 
              22.935493`, 48.921828`, 64.861297`, 71.252464`, 52.775179`, 
              80.48421`, 89.195396`, 91.016137`}, {5, 26, 
              6.36215481`, \(-1.727403392`\), 7.909060354`, 0, 
              1, \(-0.01788763497`\), \(-0.1`\), 
              7741.292925`, \(-0.7107630978`\), \(-0.7037398962`\), 2, 4, 5, 
              6, 0.197382`, \(-0.422096`\), 0.258112`, 0.126425`, 
              0.240775`, \(-0.224726`\), 0.622828`, 0.621053`, 0.70455`, 
              0.733859`, 0.993469`, 0.97661`, 18.803131`, 18.844909`, 
              1.043505`, 28.339137`, 28.309461`, \(-0.741243`\), 37.440708`, 
              37.468442`, 0.692725`, 48.331719`, 48.338061`, 0.158407`, 
              25.423248`, 25.454675`, 0.784977`, 48.313282`, 48.348842`, 
              0.888204`, 62.908778`, 62.908433`, \(-0.008595`\), 67.132434`, 
              67.172855`, 1.009644`, 18.803131`, 28.339137`, 37.440708`, 
              54.2261`, 62.908778`, 80.648241`, 94.245914`, 102.622442`, 
              25.423248`, 48.313282`, 67.132434`, 69.488255`, 48.331719`, 
              68.537739`, 77.316895`, 87.926373`}, {5, 28, 
              13.76231161`, \(-1.677843521`\), 9.834101736`, 0, 
              1, \(-0.05830486161`\), \(-0.05`\), 
              3.15959351`, \(-0.8700688547`\), \(-0.881326295`\), 2, 4, 5, 6, 
              0.21796`, \(-0.436629`\), 0.30535`, 0.106096`, 
              0.253294`, \(-0.21582`\), 0.623311`, 0.693389`, 0.666132`, 
              0.766645`, 0.925259`, 0.941692`, 21.175087`, 21.203125`, 
              0.961555`, 29.538485`, 29.519635`, \(-0.646451`\), 34.793578`, 
              34.818742`, 0.862989`, 57.908923`, 57.911639`, 0.093146`, 
              32.578413`, 32.605476`, 0.928102`, 60.627077`, 60.654785`, 
              0.950272`, 77.342233`, 77.311898`, \(-1.04035`\), 82.619405`, 
              82.598423`, \(-0.719557`\), 21.175087`, 29.538485`, 34.793578`, 
              61.483868`, 77.342233`, 99.035489`, 109.971954`, 128.165454`, 
              32.578413`, 60.627077`, 82.619405`, 84.691845`, 57.908923`, 
              75.788902`, 94.577518`, 103.5525`}, {5, 28, 
              137.1436763`, \(-0.6126638048`\), 1.95944593`, 0, 
              1, \(-0.1233848335`\), 0, 
              3.274081464`, \(-0.7603690288`\), \(-1.163866785`\), 2, 4, 5, 
              6, 0.222167`, \(-0.558282`\), 0.599888`, 0.110671`, 
              0.237589`, \(-0.184941`\), 0.599849`, 0.139557`, 0.574972`, 
              0.463151`, 0.774992`, 0.763933`, \(-0.031686`\), \(-0.03157`\), 
              0.000807`, 3.707615`, 3.707831`, 0.001506`, 5.754464`, 
              5.756217`, 0.012212`, 6.626071`, 6.478997`, \(-1.024398`\), 
              6.463404`, 6.466855`, 0.024035`, 10.322294`, 10.324868`, 
              0.017927`, 13.468234`, 13.368511`, \(-0.694589`\), 13.504561`, 
              13.590908`, 0.601417`, \(-0.031686`\), 3.707615`, 5.754464`, 
              6.626071`, 13.504561`, 17.183818`, 17.417183`, 20.89128`, 
              6.463404`, 10.322294`, 13.468234`, 14.765542`, 8.617614`, 
              10.359677`, 14.153504`, 18.361077`}, {2, 6, 
              107233.2632`, \(-0.25`\), 0.9794657355`, 0, 1, \(-0.154035`\), 
              0.05`, 50, \(-0.841807`\), \(-1.23643`\), 2, 4, 5, 6, 
              0.237156`, \(-0.757672`\), 0.99073`, 0.096995`, 
              0.17071`, \(-0.040204`\), 0.570249`, 0.132688`, 0.526067`, 
              0.401775`, 0.705425`, 
              0.649883`, \(-3.184175`\), \(-3.184414`\), \(-0.000887`\), \
\(-1.636116`\), \(-1.636967`\), \(-0.003163`\), 0.912902`, 0.914645`, 
              0.00648`, 1.470442`, 1.863195`, 1.459699`, 3.192149`, 
              3.191814`, \(-0.001244`\), 4.01761`, 4.018966`, 0.005043`, 
              4.965068`, 4.952841`, \(-0.045441`\), 5.383084`, 
              5.157664`, \(-0.837791`\), \(-3.184175`\), 
              0.912902`, \(-1.636116`\), 3.291586`, 5.420264`, 5.997115`, 
              7.056894`, 7.070224`, 3.192149`, 4.01761`, 4.965068`, 
              5.383084`, 1.470442`, 3.677195`, 4.626502`, 6.924223`}, {2, 8, 
              898247.4609`, \(-0.25`\), 0.8928940926`, 0, 1, \(-0.200258`\), 
              0.1`, 50, \(-0.848676`\), \(-1.37352`\), 2, 4, 5, 6, 
              0.248075`, \(-1.390309`\), 1.547267`, 0.090541`, 0.020206`, 
              0.236723`, 0.483602`, 0.592761`, 0.409863`, 0.343336`, 
              0.54894`, 
              0.521854`, \(-8.286708`\), \(-8.287361`\), \(-0.002314`\), \
\(-3.607007`\), \(-3.608569`\), \(-0.005536`\), \(-3.321832`\), \
\(-3.318919`\), 0.010324`, \(-2.790552`\), \(-2.791819`\), \(-0.00449`\), 
              0.221875`, 0.22849`, 0.023445`, 0.898173`, 
              0.888994`, \(-0.032532`\), 1.384005`, 1.381522`, \(-0.0088`\), 
              1.607456`, 2.279163`, 
              2.380585`, \(-8.286708`\), \(-3.321832`\), \(-3.607007`\), \
\(-1.563817`\), 0.898173`, 3.699513`, 2.130531`, 3.344697`, 2.443094`, 
              0.221875`, 1.384005`, 1.607456`, \(-2.790552`\), 1.447651`, 
              1.556667`, 2.344503`}, {2, 25, 129.6187294`, \(-1.123490617`\), 
              3.838587274`, 0, 1, \(-0.1101234991`\), 0.15`, 240592.7812`, 
              0.35320859`, 1.677729624`, 2, 4, 5, 6, 
              0.290029`, \(-0.51145`\), 0.368707`, 0.196979`, 
              0.105594`, \(-0.133084`\), 0.584625`, 0.44842`, 1.107921`, 
              0.703568`, 1.592391`, 
              1.146751`, \(-0.035821`\), \(-0.034116`\), 0.030373`, 
              9.672681`, 9.683792`, 0.197917`, 12.445825`, 
              12.435277`, \(-0.187885`\), 18.491063`, 18.51191`, 0.371344`, 
              13.086661`, 13.074047`, \(-0.224689`\), 24.361675`, 
              24.339469`, \(-0.395558`\), 24.51974`, 
              24.516424`, \(-0.059058`\), 28.235354`, 28.252905`, 
              0.312626`, \(-0.035821`\), 9.672681`, 12.445825`, 18.491063`, 
              28.235354`, 37.341872`, 36.610409`, 43.439527`, 13.086661`, 
              24.51974`, 24.361675`, 32.966889`, 19.367621`, 23.623804`, 
              31.33587`, 36.406083`}, {5, 26, 
              13.43585703`, \(-1.058973916`\), 7.165638242`, 0.0025`, 1, 
              0.3772649539`, \(-0.35`\), 
              3247.10839`, \(-0.9813171025`\), \(-0.7585671159`\), 2, 4, 5, 
              6, 0.105078`, \(-0.7468`\), 0.967793`, 0.142483`, 
              0.147656`, \(-0.104956`\), 0.554942`, 0.43343`, 0.624721`, 
              0.603127`, 0.901907`, 0.914155`, 16.918436`, 17.007104`, 
              1.068208`, 29.412814`, 29.44426`, 0.37883`, 39.164956`, 
              39.128488`, \(-0.439344`\), 46.381833`, 46.390973`, 0.110121`, 
              17.007772`, 17.048354`, 0.488902`, 37.932034`, 37.986439`, 
              0.655424`, 53.502106`, 53.534189`, 0.386508`, 57.345684`, 
              57.371816`, 0.314815`, 16.918436`, 39.164956`, 29.412814`, 
              48.525237`, 53.502106`, 69.342853`, 83.715686`, 97.504949`, 
              17.007772`, 37.932034`, 57.345684`, 65.383132`, 46.381833`, 
              71.444957`, 75.881586`, 87.942712`}, {5, 27, 
              15.04115415`, \(-1.368601047`\), 5.853949456`, 0.0025`, 1, 
              0.1702759189`, \(-0.3`\), 
              150.9379228`, \(-0.4650456834`\), \(-0.5358166214`\), 2, 4, 5, 
              6, 0.14521`, \(-0.560418`\), 0.479449`, 0.173825`, 
              0.154611`, \(-0.158731`\), 0.614724`, 0.471211`, 0.738211`, 
              0.689645`, 1.060849`, 1.077319`, 12.980536`, 13.058426`, 
              1.059372`, 26.258169`, 26.281992`, 0.324017`, 31.026325`, 
              30.995727`, \(-0.416159`\), 38.521897`, 
              38.5111`, \(-0.146846`\), 16.795375`, 16.834456`, 0.531539`, 
              33.788059`, 33.836919`, 0.664548`, 46.925898`, 46.940056`, 
              0.192558`, 48.945995`, 48.988855`, 0.582924`, 12.980536`, 
              31.026325`, 26.258169`, 39.693263`, 46.925898`, 60.24815`, 
              77.150522`, 80.906148`, 16.795375`, 33.788059`, 48.945995`, 
              55.509636`, 38.521897`, 58.303684`, 64.468502`, 
              71.311239`}, {5, 26, 10.82764979`, \(-1.642182489`\), 
              5.974806104`, 0.0025`, 1, 0.1388675802`, \(-0.25`\), 
              5768.412651`, \(-0.5157025456`\), \(-0.5950753308`\), 2, 4, 5, 
              6, 0.174186`, \(-0.509173`\), 0.376407`, 0.17005`, 
              0.162373`, \(-0.166458`\), 0.631546`, 0.519661`, 0.746076`, 
              0.715683`, 1.064978`, 1.082706`, 16.069392`, 16.133479`, 
              1.067159`, 27.770516`, 27.791322`, 0.346445`, 31.781611`, 
              31.750118`, \(-0.524405`\), 41.366352`, 41.378174`, 0.196861`, 
              18.23677`, 18.269383`, 0.543069`, 37.365188`, 37.408373`, 
              0.719107`, 50.115575`, 50.121057`, 0.091284`, 51.119354`, 
              51.161932`, 0.708987`, 16.069392`, 31.781611`, 27.770516`, 
              45.307245`, 50.115575`, 63.935673`, 81.219727`, 87.448634`, 
              18.23677`, 37.365188`, 51.119354`, 57.422005`, 41.366352`, 
              63.497724`, 67.322461`, 73.125126`}, {5, 27, 
              9.345748316`, \(-1.539018`\), 6.039451849`, 0.0025`, 1, 
              0.04236538788`, \(-0.2`\), 
              4891.600991`, \(-0.4879365697`\), \(-0.6247233319`\), 2, 4, 5, 
              6, 0.192149`, \(-0.460552`\), 0.270857`, 0.167816`, 
              0.175837`, \(-0.185143`\), 0.64993`, 0.550176`, 0.760604`, 
              0.736049`, 1.078291`, 1.092551`, 15.283704`, 15.341117`, 
              1.066026`, 27.724636`, 27.713173`, \(-0.212833`\), 30.365975`, 
              30.365198`, \(-0.014427`\), 40.753591`, 40.764322`, 0.199252`, 
              19.624662`, 19.656937`, 0.599273`, 38.425255`, 38.465464`, 
              0.746575`, 51.145756`, 51.151269`, 0.102359`, 52.52539`, 
              52.565286`, 0.740779`, 15.283704`, 27.724636`, 30.365975`, 
              44.418219`, 51.145756`, 65.112426`, 83.514843`, 83.646831`, 
              19.624662`, 38.425255`, 52.52539`, 57.557861`, 40.753591`, 
              64.533175`, 64.941394`, 72.447194`}, {5, 28, 
              8.00599147`, \(-1.276821834`\), 6.494304743`, 0.0025`, 
              1, \(-0.03468963598`\), \(-0.15`\), 
              1037.995015`, \(-0.5749627451`\), \(-0.7057843714`\), 2, 4, 5, 
              6, 0.203313`, \(-0.457688`\), 0.288715`, 0.154816`, 
              0.201666`, \(-0.205553`\), 0.656051`, 0.584818`, 0.750491`, 
              0.744371`, 1.056182`, 1.063256`, 14.240853`, 14.291139`, 
              1.062323`, 25.680317`, 25.646628`, \(-0.711718`\), 32.160166`, 
              32.181664`, 0.454183`, 41.186617`, 41.19508`, 0.178801`, 
              21.84169`, 21.87457`, 0.694638`, 40.614403`, 40.650805`, 
              0.769024`, 53.959433`, 53.964393`, 0.104785`, 56.901542`, 
              56.933554`, 0.676301`, 14.240853`, 25.680317`, 32.160166`, 
              43.784924`, 53.959433`, 68.778621`, 82.925426`, 88.331236`, 
              21.84169`, 40.614403`, 56.901542`, 59.658193`, 41.186617`, 
              61.598304`, 65.47647`, 72.533415`}, {1, 29, 
              7.385711701`, \(-1.536505127`\), 7.545784081`, 0.0025`, 
              1, \(-0.05556851113`\), \(-0.1`\), 
              896.211886`, \(-0.7961537034`\), \(-0.8150747263`\), 2, 4, 5, 
              6, 0.231241`, \(-0.446349`\), 0.282086`, 0.132984`, 
              0.223802`, \(-0.209475`\), 0.661539`, 0.648615`, 0.735157`, 
              0.762246`, 1.027094`, 1.011457`, 19.137445`, 19.174999`, 
              1.048445`, 28.065736`, 28.03662`, \(-0.812854`\), 37.686883`, 
              37.710334`, 0.654707`, 48.989206`, 48.993984`, 0.133392`, 
              26.269038`, 26.296134`, 0.756459`, 49.802736`, 49.833152`, 
              0.849174`, 64.021699`, 64.017261`, \(-0.123896`\), 67.024944`, 
              67.040324`, 0.429397`, 19.137445`, 28.065736`, 37.686883`, 
              55.104535`, 64.021699`, 81.39501`, 95.658314`, 106.02905`, 
              26.269038`, 49.802736`, 67.024944`, 69.136177`, 48.989206`, 
              69.271808`, 79.859421`, 79.984209`}, {5, 28, 
              21.23146772`, \(-1.353795457`\), 10.92409733`, 0.0025`, 
              1, \(-0.0955625528`\), \(-0.05`\), 
              1043.084308`, \(-1.154466079`\), \(-1.018714856`\), 2, 4, 5, 6, 
              0.248304`, \(-0.46977`\), 0.355632`, 0.091084`, 
              0.247968`, \(-0.196598`\), 0.657986`, 0.707665`, 0.688912`, 
              0.76075`, 0.947878`, 0.894598`, 23.742977`, 23.761921`, 
              0.822185`, 34.101491`, 34.08694`, \(-0.631491`\), 54.119204`, 
              54.151324`, 1.394019`, 67.377568`, 67.379706`, 0.092791`, 
              38.710397`, 38.73793`, 1.194919`, 71.484238`, 71.512295`, 
              1.217666`, 90.545578`, 90.500969`, \(-1.936041`\), 94.062765`, 
              94.046943`, \(-0.686691`\), 23.742977`, 34.101491`, 54.119204`, 
              73.097051`, 90.545578`, 115.365751`, 129.413784`, 149.68304`, 
              38.710397`, 71.484238`, 94.062765`, 98.086148`, 67.377568`, 
              88.976069`, 110.90828`, 111.457277`}, {2, 28, 
              181.5478697`, \(-0.578147215`\), 1.742013433`, 0.0025`, 
              1, \(-0.1530921045`\), 0, 
              73.84326756`, \(-0.2074927336`\), \(-0.9188694949`\), 2, 4, 5, 
              6, 0.251944`, \(-0.588477`\), 0.642605`, 0.169178`, 
              0.198069`, \(-0.224634`\), 0.635486`, 0.089092`, 0.670258`, 
              0.561322`, 0.926325`, 
              1.012543`, \(-0.043051`\), \(-0.043006`\), 0.000317`, 
              3.747953`, 3.748106`, 0.001075`, 5.93823`, 
              5.936148`, \(-0.014616`\), 7.847687`, 
              7.673649`, \(-1.222136`\), 6.103199`, 6.105845`, 0.01858`, 
              9.948167`, 9.95066`, 0.017507`, 12.434598`, 
              12.405092`, \(-0.207195`\), 12.842944`, 12.863808`, 
              0.146515`, \(-0.043051`\), 3.747953`, 5.93823`, 7.847687`, 
              12.842944`, 14.763053`, 18.564548`, 19.54042`, 6.103199`, 
              9.948167`, 12.434598`, 15.190153`, 8.509906`, 10.12039`, 
              14.352649`, 15.215255`}, {2, 8, 311606.9456`, \(-0.25`\), 
              0.7976684861`, 0.0025`, 1, \(-0.206043`\), 0.05`, 
              50, \(-0.960122`\), \(-1.23009`\), 2, 4, 5, 6, 
              0.253014`, \(-1.062947`\), 1.545337`, 0.102513`, 0.106616`, 
              0.053094`, 0.585369`, 0.280076`, 0.561726`, 0.387148`, 
              0.763517`, 
              0.631415`, \(-4.321783`\), \(-4.321947`\), \(-0.000529`\), \
\(-2.411502`\), \(-2.412383`\), \(-0.002844`\), 0.087252`, 0.088461`, 
              0.003902`, 0.431786`, 0.433155`, 0.004421`, 2.599523`, 
              2.600554`, 0.003327`, 2.616053`, 2.615895`, \(-0.000509`\), 
              3.31765`, 3.311394`, \(-0.020199`\), 3.837832`, 
              3.373813`, \(-1.498377`\), \(-4.321783`\), 
              0.087252`, \(-2.411502`\), 1.843336`, 3.95965`, 3.837832`, 
              5.578175`, 5.750381`, 2.616053`, 2.599523`, 3.31765`, 
              4.605566`, 0.431786`, 2.14732`, 3.764739`, 3.936026`}, {2, 8, 
              1.482468284`*^6, \(-0.25`\), 3.458049708`, 0.0025`, 
              1, \(-0.255089`\), 0.1`, 50, \(-1.01755`\), \(-1.35086`\), 2, 
              4, 5, 6, 0.268447`, \(-2.116782`\), 2.0624`, 
              0.096733`, \(-0.101449`\), 0.400457`, 0.482213`, 0.295375`, 
              0.437087`, 0.326275`, 0.603361`, 
              0.487728`, \(-41.225773`\), \(-41.227845`\), \(-0.030777`\), \
\(-19.92248`\), \(-19.931289`\), \(-0.130834`\), \(-18.594688`\), \
\(-18.582718`\), 0.177789`, \(-15.695217`\), \(-15.692214`\), 
              0.044594`, \(-0.097489`\), \(-0.090389`\), 0.105456`, 
              0.441295`, 0.414052`, \(-0.404641`\), 1.316319`, 
              1.300793`, \(-0.230597`\), 2.080406`, 
              2.066002`, \(-0.213941`\), \(-41.225773`\), \(-18.594688`\), \
\(-19.92248`\), \(-10.172047`\), 0.441295`, 7.766194`, 4.341031`, 10.927197`, 
              3.370388`, \(-0.097489`\), 1.316319`, 
              2.080406`, \(-15.695217`\), 0.309174`, 1.64462`, 
              4.280139`}, {5, 27, 
              344.2900439`, \(-1.032815614`\), \(-3.571316884`\), 0.0025`, 
              1, \(-0.1422784893`\), 0.15`, 24328.50148`, 
              3.936281445`, \(-6.867861294`\), 2, 4, 5, 6, 
              0.319756`, \(-0.517707`\), 0.38391`, \(-0.245759`\), 
              0.352502`, \(-0.237556`\), 
              0.62452`, \(-0.261433`\), \(-0.815179`\), \(-0.511226`\), \
\(-1.916512`\), \(-1.016787`\), \(-0.110815`\), \(-0.112523`\), 
              0.031212`, \(-10.392173`\), \(-10.399741`\), 
              0.138284`, \(-13.645173`\), \(-13.637349`\), \(-0.142955`\), \
\(-16.915013`\), \(-16.926873`\), 
              0.216705`, \(-12.873897`\), \(-12.863554`\), \(-0.188973`\), \
\(-23.954241`\), \(-23.939298`\), \(-0.273026`\), \(-24.432895`\), \
\(-24.428387`\), \(-0.082363`\), \(-28.127195`\), \(-28.140685`\), 
              0.246472`, \(-0.110815`\), \(-10.392173`\), \(-13.645173`\), \
\(-16.915013`\), \(-28.127195`\), \(-34.53382`\), \(-39.790638`\), \
\(-43.183167`\), \(-12.873897`\), \(-24.432895`\), \(-23.954241`\), \
\(-31.800394`\), \(-19.675701`\), \(-24.456277`\), \(-30.692858`\), \
\(-34.524191`\)}, {5, 28, 14.40401407`, \(-1.220347827`\), 6.089950107`, 
              0.01`, 1, 0.3864559322`, \(-0.35`\), 
              3484.502024`, \(-0.8330377476`\), \(-0.6793359873`\), 2, 4, 5, 
              6, 0.157921`, \(-0.700101`\), 0.804313`, 0.167788`, 
              0.127957`, \(-0.108818`\), 0.610196`, 0.489058`, 0.711598`, 
              0.680502`, 1.0234`, 1.040415`, 19.716586`, 19.785996`, 
              1.068061`, 27.644491`, 27.666393`, 0.337015`, 37.487561`, 
              37.450361`, \(-0.572433`\), 45.291631`, 45.302038`, 0.160128`, 
              16.833892`, 16.862009`, 0.43265`, 38.292992`, 38.333899`, 
              0.629473`, 51.723683`, 51.735549`, 0.182581`, 51.903085`, 
              51.922114`, 0.292813`, 19.716586`, 37.487561`, 27.644491`, 
              52.053243`, 51.723683`, 65.464069`, 83.642581`, 88.71562`, 
              16.833892`, 38.292992`, 51.903085`, 60.318676`, 45.291631`, 
              69.178089`, 68.92256`, 77.71259`}, {5, 27, 
              15.18244943`, \(-1.185202606`\), 5.105824582`, 0.01`, 1, 
              0.07176584075`, \(-0.3`\), 
              1914.482037`, \(-0.2562448356`\), \(-0.6120492625`\), 2, 4, 5, 
              6, 0.183838`, \(-0.529378`\), 0.339192`, 0.197342`, 
              0.108`, \(-0.119112`\), 0.671625`, 0.48561`, 0.793585`, 
              0.727929`, 1.125116`, 1.163319`, 11.597058`, 11.66848`, 
              1.072648`, 25.186932`, 25.204742`, 0.267477`, 27.893972`, 
              27.861736`, \(-0.484135`\), 35.632715`, 35.645437`, 0.191075`, 
              16.826127`, 16.86077`, 0.520279`, 32.489685`, 32.532171`, 
              0.638078`, 44.615347`, 44.604957`, \(-0.156046`\), 45.285312`, 
              45.336298`, 0.765727`, 11.597058`, 27.893972`, 25.186932`, 
              35.995758`, 44.615347`, 56.542369`, 69.641111`, 76.049329`, 
              16.826127`, 32.489685`, 45.285312`, 50.93418`, 35.632715`, 
              55.74136`, 59.266324`, 61.827768`}, {5, 27, 
              12.93354136`, \(-1.051312096`\), 5.255703706`, 0.01`, 
              1, \(-0.0005172598428`\), \(-0.25`\), 
              3950.817781`, \(-0.2855481905`\), \(-0.6127497388`\), 2, 4, 5, 
              6, 0.196563`, \(-0.509409`\), 0.315448`, 0.193507`, 
              0.12096`, \(-0.13475`\), 0.681752`, 0.514029`, 0.804717`, 
              0.742849`, 1.137648`, 1.163976`, 11.101785`, 11.166522`, 
              1.070047`, 25.141648`, 25.121483`, \(-0.333306`\), 27.347759`, 
              27.351075`, 0.054804`, 35.639785`, 35.654091`, 0.236459`, 
              17.99015`, 18.024495`, 0.567688`, 33.629843`, 33.669602`, 
              0.657188`, 45.873066`, 45.872158`, \(-0.015012`\), 47.111858`, 
              47.1525`, 0.67178`, 11.101785`, 25.141648`, 27.347759`, 
              35.639785`, 45.873066`, 58.145044`, 74.778586`, 75.635024`, 
              17.99015`, 33.629843`, 47.111858`, 51.558368`, 35.647251`, 
              54.875906`, 58.577393`, 61.033791`}, {5, 26, 
              11.05187693`, \(-1.445278528`\), 5.578668109`, 0.01`, 
              1, \(-0.004601056997`\), \(-0.2`\), 
              11815.26701`, \(-0.4559938368`\), \(-0.6891575662`\), 2, 4, 5, 
              6, 0.232651`, \(-0.461191`\), 0.235286`, 0.181961`, 
              0.14432`, \(-0.155262`\), 0.69266`, 0.57624`, 0.803689`, 
              0.770377`, 1.130609`, 1.15016`, 15.932473`, 15.983951`, 
              1.068988`, 27.441334`, 27.418762`, \(-0.468735`\), 29.496308`, 
              29.501668`, 0.111307`, 40.609886`, 40.618176`, 0.172152`, 
              19.967212`, 19.993923`, 0.554697`, 39.145973`, 39.179933`, 
              0.705221`, 51.00554`, 51.021169`, 0.324563`, 51.124859`, 
              51.140702`, 0.329018`, 15.932473`, 27.441334`, 29.496308`, 
              44.498463`, 51.124859`, 64.381794`, 82.63949`, 83.424132`, 
              19.967212`, 39.145973`, 51.00554`, 56.304548`, 40.609886`, 
              62.323659`, 65.265344`, 66.018288`}, {5, 25, 
              9.008659793`, \(-1.035081321`\), 6.216131665`, 0.01`, 
              1, \(-0.09042570761`\), \(-0.15`\), 
              1432.061356`, \(-0.6099761667`\), \(-0.7744315712`\), 2, 4, 5, 
              6, 0.23276`, \(-0.498334`\), 0.354695`, 0.163571`, 
              0.179538`, \(-0.182692`\), 0.689697`, 0.609168`, 0.78289`, 
              0.771385`, 1.095162`, 1.101895`, 12.946453`, 12.992166`, 
              1.058252`, 24.017032`, 23.981807`, \(-0.815442`\), 32.223963`, 
              32.243601`, 0.454632`, 40.530372`, 40.53738`, 0.162235`, 
              22.496838`, 22.526766`, 0.692845`, 40.899883`, 40.9316`, 
              0.734251`, 54.0159`, 54.017011`, 0.025715`, 56.592605`, 
              56.608099`, 0.358682`, 12.946453`, 24.017032`, 32.223963`, 
              41.708541`, 54.0159`, 68.518637`, 82.083783`, 88.897083`, 
              22.496838`, 40.899883`, 56.592605`, 58.463519`, 40.530372`, 
              59.627382`, 66.438091`, 66.525399`}, {5, 27, 
              8.172929207`, \(-1.358482237`\), 7.467317946`, 0.01`, 
              1, \(-0.09536466895`\), \(-0.1`\), 
              2847.950914`, \(-0.9331093735`\), \(-0.9572715651`\), 2, 4, 5, 
              6, 0.266389`, \(-0.467212`\), 0.303254`, 0.134514`, 
              0.209746`, \(-0.192285`\), 0.696046`, 0.678285`, 0.755872`, 
              0.785836`, 1.04581`, 1.028476`, 19.836397`, 19.869113`, 
              1.041285`, 28.922069`, 28.895559`, \(-0.843749`\), 39.259633`, 
              39.279952`, 0.646707`, 51.411845`, 51.415483`, 0.115779`, 
              27.924908`, 27.948821`, 0.761064`, 53.000957`, 53.028142`, 
              0.865235`, 67.305864`, 67.294926`, \(-0.348144`\), 68.890093`, 
              68.895354`, 0.16743`, 19.836397`, 28.922069`, 39.259633`, 
              57.052901`, 67.305864`, 84.952616`, 100.944144`, 109.983315`, 
              27.924908`, 53.000957`, 68.890093`, 71.244935`, 51.411845`, 
              72.92404`, 82.219696`, 86.102554`}, {5, 26, 
              95.23262854`, \(-0.9795508005`\), 6.995490212`, 0.01`, 
              1, \(-0.1458281911`\), \(-0.05`\), 
              9248.061537`, \(-0.8146589148`\), \(-0.8227775819`\), 2, 4, 5, 
              6, 0.274827`, \(-0.51819`\), 0.447368`, 0.143366`, 
              0.198887`, \(-0.183222`\), 0.685696`, 0.674692`, 0.768766`, 
              0.793916`, 1.072836`, 1.058311`, 11.666118`, 11.673488`, 
              0.226703`, 21.516495`, 21.51123`, \(-0.161962`\), 35.86517`, 
              35.88277`, 0.541392`, 42.498466`, 42.538586`, 1.234129`, 
              26.201064`, 26.21794`, 0.519127`, 47.080474`, 47.097537`, 
              0.524875`, 59.695818`, 59.786075`, 2.776381`, 60.113982`, 
              59.98651`, \(-3.921129`\), 11.666118`, 21.516495`, 35.86517`, 
              42.498466`, 59.695818`, 75.671239`, 85.615633`, 98.176147`, 
              26.201064`, 47.080474`, 60.113982`, 64.756239`, 43.549566`, 
              57.5877`, 71.147242`, 72.98522`}, {5, 26, 
              131.5299108`, \(-1.701392243`\), 6.170752196`, 0.01`, 
              1, \(-0.1082038274`\), 0, 
              1.538553997`*^6, \(-0.6360149633`\), \(-0.6351884604`\), 2, 4, 
              5, 6, 0.31717`, \(-0.464497`\), 0.309481`, 0.166506`, 
              0.171481`, \(-0.176564`\), 0.699048`, 0.699322`, 0.822353`, 
              0.843055`, 1.159038`, 1.156357`, 18.282573`, 18.283113`, 
              0.016926`, 24.275811`, 24.2767`, 0.027837`, 32.681589`, 
              32.696607`, 0.470288`, 44.465657`, 44.465673`, 0.00052`, 
              23.89074`, 23.895947`, 0.163033`, 47.733964`, 47.741341`, 
              0.231024`, 56.931502`, 56.882295`, \(-1.540897`\), 57.529455`, 
              57.503977`, \(-0.797863`\), 18.282573`, 24.275811`, 32.681589`, 
              50.038763`, 57.529455`, 72.278507`, 84.404282`, 93.418182`, 
              23.89074`, 47.733964`, 56.931502`, 59.000296`, 44.465657`, 
              59.88807`, 68.368515`, 73.883764`}, {2, 8, 
              493852.3303`, \(-0.25`\), \(-0.58844881`\), 0.01`, 
              1, \(-0.254995`\), 0.05`, 50, \(-1.15716`\), \(-1.26943`\), 2, 
              4, 5, 6, 0.273081`, \(-1.31184`\), 1.942934`, 0.103138`, 
              0.04576`, 0.133857`, 0.60556`, 0.512772`, 0.591378`, 0.35869`, 
              0.807952`, 0.589144`, 4.017978`, 4.018074`, \(-0.000245`\), 
              2.186896`, 2.187332`, \(-0.001122`\), 0.216359`, 0.215759`, 
              0.001544`, \(-0.22066`\), \(-0.221563`\), 
              0.002322`, \(-1.649422`\), \(-1.649999`\), 
              0.001482`, \(-1.739887`\), \(-1.944816`\), 
              0.526896`, \(-1.944897`\), \(-2.222637`\), 
              0.714099`, \(-2.226034`\), \(-2.852273`\), 1.610127`, 
              4.017978`, 0.216359`, 
              2.186896`, \(-1.232426`\), \(-2.902669`\), \(-1.739887`\), \
\(-4.145356`\), \(-5.069975`\), \(-1.944897`\), \(-1.649422`\), \
\(-2.226034`\), \(-3.385485`\), \(-0.22066`\), \(-1.070291`\), \
\(-2.026993`\), \(-3.184132`\)}, {2, 8, 
              1.81117411`*^6, \(-0.25`\), \(-0.8877621855`\), 0.01`, 
              1, \(-0.305179`\), 0.1`, 50, \(-1.27206`\), \(-1.38086`\), 2, 
              4, 5, 6, 0.28678`, \(-2.462554`\), 2.493149`, 
              0.097204`, \(-0.175135`\), 0.469835`, 0.503552`, 0.859343`, 
              0.471607`, 0.256854`, 0.657774`, 0.40662`, 11.696232`, 
              11.696682`, \(-0.001834`\), 5.960421`, 
              5.962934`, \(-0.010237`\), 5.047981`, 5.044907`, 0.012522`, 
              4.029677`, 4.02868`, 0.004063`, 0.231731`, 0.008303`, 
              0.910133`, 0.008518`, \(-0.169789`\), 
              0.72633`, \(-0.173236`\), \(-0.303402`\), 
              0.530228`, \(-0.303652`\), \(-0.5163`\), 0.86622`, 11.696232`, 
              5.047981`, 5.960421`, 2.490683`, \(-0.521453`\), 
              0.231731`, \(-2.095729`\), \(-3.519033`\), \(-0.303652`\), 
              0.008518`, \(-0.173236`\), \(-0.52691`\), 4.029677`, 
              1.002305`, \(-0.121439`\), \(-1.119979`\)}, {2, 8, 
              4.394548546`*^6, \(-0.25`\), \(-0.3426948022`\), 0.01`, 
              1, \(-0.355362`\), 0.15`, 50, \(-1.38697`\), \(-1.4923`\), 2, 
              4, 5, 6, 0.350287`, \(-5.145902`\), 1.818305`, 
              0.099506`, \(-0.572334`\), 0.956693`, 0.264168`, 4.164709`, 
              0.26999`, 0.058672`, 0.443358`, 0.116785`, 7.057088`, 
              7.057323`, \(-0.000451`\), 5.455527`, 
              5.455672`, \(-0.000278`\), 3.473748`, 3.473334`, 0.000795`, 
              3.176857`, 3.173742`, 0.005983`, 3.151196`, 3.149203`, 
              0.003828`, 2.172049`, 1.696484`, 0.913401`, 1.689825`, 
              1.446633`, 0.467091`, 1.444761`, 1.07874`, 0.703004`, 
              7.057088`, 5.455527`, 3.176857`, 2.670111`, 1.689825`, 
              2.172049`, 1.382461`, \(-0.248781`\), 3.151196`, 0.633149`, 
              1.444761`, 1.048485`, 3.473748`, 2.742561`, 1.819556`, 
              0.723756`}, {5, 28, 17.49123999`, \(-1.526078127`\), 
              5.080645372`, 0.0225`, 1, 0.3179316914`, \(-0.35`\), 
              2078.058144`, \(-0.457346619`\), \(-0.6202378172`\), 2, 4, 5, 
              6, 0.224405`, \(-0.603552`\), 0.513892`, 0.199994`, 
              0.087824`, \(-0.089062`\), 0.679749`, 0.532161`, 0.808257`, 
              0.756189`, 1.152222`, 1.181565`, 20.514322`, 20.572918`, 
              1.068913`, 25.990886`, 26.005436`, 0.265427`, 34.966617`, 
              34.932194`, \(-0.62795`\), 43.10674`, 43.114406`, 0.139833`, 
              16.820962`, 16.842179`, 0.387026`, 37.896916`, 37.92954`, 
              0.59513`, 46.406713`, 46.419963`, 0.241706`, 49.441894`, 
              49.446412`, 0.082431`, 20.514322`, 34.966617`, 25.990886`, 
              49.866736`, 49.441894`, 61.313531`, 79.945366`, 82.433335`, 
              16.820962`, 37.896916`, 46.406713`, 55.966533`, 43.10674`, 
              63.820798`, 66.798862`, 71.578324`}, {5, 27, 
              17.14287462`, \(-1.327432424`\), 4.679749968`, 0.0225`, 1, 
              0.04227852595`, \(-0.3`\), 
              3810.92519`, \(-0.06919582096`\), \(-0.6736713493`\), 2, 4, 5, 
              6, 0.237268`, \(-0.498406`\), 0.233906`, 0.214481`, 
              0.06112`, \(-0.066845`\), 0.718094`, 0.520708`, 0.841427`, 
              0.766999`, 1.184099`, 1.227552`, 14.100492`, 14.161098`, 
              1.076703`, 25.241187`, 25.258207`, 0.302379`, 27.981176`, 
              27.943296`, \(-0.67296`\), 36.859565`, 36.86979`, 0.181639`, 
              17.245284`, 17.271242`, 0.461152`, 34.341322`, 34.37633`, 
              0.621932`, 43.907191`, 43.920521`, 0.236814`, 45.460522`, 
              45.471848`, 0.201218`, 14.100492`, 27.981176`, 25.241187`, 
              39.773391`, 45.460522`, 56.758874`, 71.279149`, 74.341088`, 
              17.245284`, 34.341322`, 43.907191`, 50.599625`, 36.859565`, 
              56.421962`, 59.085092`, 63.637454`}, {5, 25, 
              15.0605637`, \(-1.183574923`\), 4.88639105`, 0.0225`, 
              1, \(-0.02985825912`\), \(-0.25`\), 
              618.8191964`, \(-0.2235979459`\), \(-0.7096485453`\), 2, 4, 5, 
              6, 0.247725`, \(-0.486041`\), 0.232588`, 0.206687`, 
              0.084434`, \(-0.093376`\), 0.724254`, 0.548715`, 0.841737`, 
              0.778551`, 1.18052`, 1.216846`, 13.643191`, 13.698629`, 
              1.073704`, 25.956934`, 25.945591`, \(-0.219679`\), 27.065148`, 
              27.054645`, \(-0.203431`\), 37.217192`, 37.226158`, 0.173655`, 
              18.481614`, 18.507504`, 0.501437`, 35.729752`, 35.762746`, 
              0.639021`, 46.171691`, 46.17339`, 0.032902`, 47.053411`, 
              47.076919`, 0.455294`, 13.643191`, 25.956934`, 27.065148`, 
              39.712026`, 47.053411`, 58.867343`, 73.965712`, 78.305811`, 
              18.481614`, 35.729752`, 46.171691`, 51.683032`, 37.217192`, 
              57.325772`, 60.329046`, 62.149633`}, {5, 25, 
              13.01551971`, \(-1.299820908`\), 5.246982403`, 0.0225`, 
              1, \(-0.06045740157`\), \(-0.2`\), 
              1854.332252`, \(-0.4207178105`\), \(-0.7537203431`\), 2, 4, 5, 
              6, 0.270982`, \(-0.466819`\), 0.219454`, 0.194034`, 
              0.115217`, \(-0.12508`\), 0.729948`, 0.599399`, 0.840553`, 
              0.799894`, 1.175322`, 1.19875`, 15.971768`, 16.018669`, 
              1.066954`, 26.749954`, 26.718571`, \(-0.713935`\), 29.116637`, 
              29.127699`, 0.25166`, 40.429927`, 40.436282`, 0.144585`, 
              20.383838`, 20.406731`, 0.520803`, 39.769814`, 39.799143`, 
              0.66721`, 50.080636`, 50.082159`, 0.034651`, 51.249279`, 
              51.270328`, 0.478858`, 15.971768`, 26.749954`, 29.116637`, 
              44.629866`, 51.249279`, 64.007489`, 81.991616`, 82.202687`, 
              20.383838`, 39.769814`, 50.080636`, 55.335874`, 40.429927`, 
              59.746194`, 66.19319`, 66.648921`}, {1, 25, 
              10.83108408`, \(-1.356901358`\), 5.908164716`, 0.0225`, 
              1, \(-0.09366639454`\), \(-0.15`\), 
              1352.562899`, \(-0.6937949679`\), \(-0.8650857529`\), 2, 4, 5, 
              6, 0.290942`, \(-0.46269`\), 0.238773`, 0.172341`, 
              0.154594`, \(-0.157733`\), 0.732354`, 0.654314`, 0.823198`, 
              0.813699`, 1.144224`, 1.149837`, 18.482465`, 18.521014`, 
              1.060205`, 27.876392`, 27.846245`, \(-0.82915`\), 32.847334`, 
              32.861462`, 0.388566`, 45.248218`, 45.252497`, 0.117689`, 
              23.403447`, 23.424259`, 0.57239`, 45.679754`, 45.705937`, 
              0.720117`, 56.739973`, 56.734791`, \(-0.142515`\), 57.848988`, 
              57.871368`, 0.615522`, 18.482465`, 27.876392`, 32.847334`, 
              50.966563`, 57.848988`, 72.243083`, 90.271543`, 92.651127`, 
              23.403447`, 45.679754`, 56.739973`, 61.28343`, 45.248218`, 
              67.448385`, 70.636904`, 74.421701`}, {2, 17, 
              8.429733098`, \(-1.203553385`\), 7.69566534`, 0.0225`, 
              1, \(-0.1384570855`\), \(-0.1`\), 
              8466.91459`, \(-1.140160951`\), \(-1.145680237`\), 2, 4, 5, 6, 
              0.303068`, \(-0.482562`\), 0.31427`, 0.130383`, 
              0.200317`, \(-0.176672`\), 0.728381`, 0.710995`, 0.766646`, 
              0.805784`, 1.048503`, 1.027827`, 20.931048`, 20.958397`, 
              1.020587`, 31.275068`, 31.252622`, \(-0.837631`\), 42.449478`, 
              42.46765`, 0.678132`, 56.083113`, 56.085982`, 0.107069`, 
              30.727439`, 30.749221`, 0.81284`, 58.519522`, 58.545278`, 
              0.961111`, 73.478429`, 73.469951`, \(-0.316369`\), 73.502556`, 
              73.494399`, \(-0.304372`\), 20.931048`, 31.275068`, 42.449478`, 
              61.452815`, 73.478429`, 92.104902`, 110.97648`, 118.441123`, 
              30.727439`, 58.519522`, 73.502556`, 76.167926`, 56.083113`, 
              80.006797`, 87.888847`, 94.209188`}, {5, 28, 
              13.88767786`, \(-2.644587832`\), 9.139202273`, 0.0225`, 
              1, \(-0.05716896311`\), \(-0.05`\), 
              3231.423518`, \(-1.449774961`\), \(-1.259837627`\), 2, 4, 5, 6, 
              0.354163`, \(-0.443069`\), 0.211895`, 0.107826`, 
              0.229172`, \(-0.206622`\), 0.745213`, 0.777638`, 0.767243`, 
              0.840097`, 1.042229`, 0.997819`, 41.824049`, 41.843731`, 
              1.01928`, 47.526833`, 47.513107`, \(-0.710863`\), 52.069486`, 
              52.086856`, 0.899544`, 78.88214`, 78.882978`, 0.043412`, 
              37.694756`, 37.7069`, 0.628913`, 80.857797`, 80.877039`, 
              0.996499`, 89.840173`, 89.837206`, \(-0.153642`\), 96.885444`, 
              96.864573`, \(-1.080901`\), 41.824049`, 47.526833`, 52.069486`, 
              97.18499`, 96.885444`, 119.674324`, 151.739127`, 151.913214`, 
              37.694756`, 80.857797`, 89.840173`, 100.034922`, 78.88214`, 
              115.355249`, 117.319024`, 122.962978`}, {2, 8, 
              80794.43708`, \(-0.25`\), 0.8212720588`, 0.0225`, 
              1, \(-0.251251`\), 0, 50, \(-1.2536`\), \(-1.24533`\), 2, 4, 5, 
              6, 0.298392`, \(-0.890752`\), 1.23396`, 0.112609`, 
              0.11391`, \(-0.002642`\), 0.682925`, 0.158539`, 0.694023`, 
              0.398889`, 0.946756`, 
              0.678689`, \(-2.214398`\), \(-2.214405`\), \(-0.000025`\), 
              1.278768`, 1.278808`, 0.000153`, 1.79285`, 
              1.790209`, \(-0.010355`\), 2.249811`, 2.672593`, 1.657717`, 
              3.111723`, 3.1129`, 0.004614`, 4.45324`, 4.455`, 0.0069`, 
              5.359419`, 5.352469`, \(-0.02725`\), 6.02342`, 6.02722`, 
              0.014901`, \(-2.214398`\), 1.278768`, 1.79285`, 2.249811`, 
              6.02342`, 6.2317`, 8.319695`, 9.487963`, 3.111723`, 4.45324`, 
              5.359419`, 7.270113`, 3.497052`, 2.968675`, 5.849223`, 
              6.849254`}, {5, 25, 101.7893908`, \(-0.6629030726`\), 
              2.16970382`, 0.0225`, 1, \(-0.2211760171`\), 0.05`, 
              5044.673631`, 0.0820763293`, \(-0.3971723369`\), 2, 4, 5, 6, 
              0.344554`, \(-0.567316`\), 0.536855`, 0.218028`, 
              0.049024`, \(-0.053074`\), 0.700892`, 0.285029`, 0.869282`, 
              0.641832`, 1.240477`, 
              1.163688`, \(-0.019021`\), \(-0.018864`\), 0.001877`, 
              6.827765`, 6.827868`, 0.001233`, 9.862871`, 10.508638`, 
              7.724196`, 10.529855`, 10.531419`, 0.018706`, 8.626007`, 
              8.625608`, \(-0.004776`\), 15.37277`, 15.374824`, 0.024572`, 
              16.771847`, 16.75943`, \(-0.148524`\), 18.786343`, 18.798455`, 
              0.144867`, \(-0.019021`\), 6.827765`, 9.862871`, 10.529855`, 
              18.786343`, 22.96105`, 26.981943`, 29.255512`, 8.626007`, 
              15.37277`, 16.771847`, 20.520201`, 13.138212`, 16.626092`, 
              20.652345`, 22.883816`}, {2, 8, 
              1.819685403`*^6, \(-0.25`\), \(-0.448729978`\), 0.0225`, 
              1, \(-0.350528`\), 0.1`, 50, \(-1.61222`\), \(-1.46355`\), 2, 
              4, 5, 6, 0.305895`, \(-2.372478`\), 2.703777`, 
              0.086264`, \(-0.184326`\), 0.44098`, 0.547205`, 1.61392`, 
              0.515135`, 0.207924`, 0.713515`, 0.335614`, 5.917873`, 
              5.918066`, \(-0.000424`\), 3.127521`, 3.128559`, \(-0.00228`\), 
              2.106926`, 2.105663`, 0.002773`, 1.552738`, 1.552136`, 
              0.001321`, 0.316205`, 0.11309`, 0.446087`, 
              0.113805`, \(-0.400386`\), 
              1.12928`, \(-0.401568`\), \(-0.60039`\), 
              0.436657`, \(-0.601531`\), \(-0.712044`\), 0.242713`, 
              5.917873`, 2.106926`, 3.127521`, 0.687658`, \(-0.91377`\), 
              0.316205`, \(-1.758754`\), \(-2.284985`\), \(-0.601531`\), 
              0.113805`, \(-0.401568`\), \(-0.712062`\), 1.552738`, 
              1.498836`, \(-0.429199`\), \(-1.761449`\)}, {5, 28, 
              4961.920201`, \(-0.7239539657`\), 4.146153657`, 0.0225`, 
              1, \(-0.2253238464`\), 0.15`, 1284.3073`, 
              1.0004714`, \(-0.70809702`\), 2, 4, 5, 6, 
              0.379636`, \(-0.529993`\), 0.398168`, 0.174137`, 
              0.113353`, \(-0.102589`\), 0.688612`, 0.518052`, 0.79983`, 
              0.707083`, 1.128784`, 1.072829`, \(-2.49426`\), \(-2.493006`\), 
              0.031572`, 13.902269`, 13.908879`, 0.166466`, 18.112252`, 
              18.122174`, 0.249889`, 18.628899`, 18.62294`, \(-0.150091`\), 
              16.448313`, 16.438657`, \(-0.243183`\), 29.176322`, 
              29.167704`, \(-0.217026`\), 31.10503`, 
              31.100095`, \(-0.124295`\), 36.023843`, 36.118315`, 
              2.37923`, \(-2.49426`\), 13.902269`, 18.112252`, 18.628899`, 
              36.023843`, 43.535224`, 52.143843`, 54.116906`, 16.448313`, 
              29.176322`, 31.10503`, 38.821153`, 24.969481`, 32.385838`, 
              37.538086`, 44.334082`}, {5, 27, 
              21.3915725`, \(-0.7290645183`\), 4.305582531`, 0.04`, 
              1, \(-0.05401767882`\), \(-0.35`\), 1772.67798`, 
              0.5584457208`, \(-0.7982259081`\), 2, 4, 5, 6, 
              0.227694`, \(-0.611385`\), 0.399667`, 0.21097`, 
              0.064747`, \(-0.072924`\), 0.745256`, 0.462692`, 0.85478`, 
              0.724888`, 1.193993`, 1.191686`, 8.304509`, 8.375581`, 
              1.114817`, 23.669141`, 23.678362`, 0.144629`, 24.613458`, 
              24.58032`, \(-0.519789`\), 31.243512`, 31.293267`, 0.780449`, 
              16.516575`, 16.547735`, 0.488757`, 30.113735`, 30.150532`, 
              0.577183`, 41.265276`, 41.265391`, 0.001804`, 41.526556`, 
              41.546371`, 0.310816`, 8.304509`, 24.613458`, 23.669141`, 
              31.243512`, 41.526556`, 52.013169`, 63.320673`, 67.028584`, 
              16.516575`, 30.113735`, 41.265276`, 46.234228`, 32.049057`, 
              50.581752`, 51.548163`, 54.825711`}, {5, 28, 
              19.71305313`, \(-0.9601243503`\), 4.391273272`, 0.04`, 
              1, \(-0.06586627403`\), \(-0.3`\), 311.5656315`, 
              0.1062396627`, \(-0.7739640161`\), 2, 4, 5, 6, 
              0.262502`, \(-0.521485`\), 0.248565`, 0.222017`, 
              0.041851`, \(-0.045914`\), 0.755`, 0.520953`, 0.870744`, 
              0.774914`, 1.215925`, 1.250768`, 11.573756`, 11.632814`, 
              1.08923`, 24.703631`, 24.707734`, 0.075685`, 25.391003`, 
              25.361238`, \(-0.548962`\), 34.679779`, 34.689307`, 0.175732`, 
              17.521133`, 17.546416`, 0.466309`, 33.250468`, 33.282681`, 
              0.594125`, 42.798136`, 42.806428`, 0.15294`, 44.167348`, 
              44.176763`, 0.173631`, 11.573756`, 25.391003`, 24.703631`, 
              36.366997`, 44.167348`, 54.965785`, 67.247848`, 71.05198`, 
              17.521133`, 33.250468`, 42.798136`, 48.347221`, 34.679779`, 
              53.630939`, 54.059688`, 61.759322`}, {2, 29, 
              17.46583367`, \(-1.199302957`\), 4.636652847`, 0.04`, 
              1, \(-0.07420390182`\), \(-0.25`\), 
              292.5643648`, \(-0.2620280138`\), \(-0.8096404695`\), 2, 4, 5, 
              6, 0.295555`, \(-0.473077`\), 0.175475`, 0.211335`, 
              0.072341`, \(-0.079291`\), 0.763245`, 0.5736`, 0.874599`, 
              0.80029`, 1.218027`, 1.241542`, 15.1728`, 15.222327`, 
              1.085936`, 26.543242`, 26.537625`, \(-0.123177`\), 26.996828`, 
              26.978186`, \(-0.408757`\), 38.464577`, 38.471045`, 0.141808`, 
              19.088691`, 19.109658`, 0.459726`, 37.549832`, 37.57816`, 
              0.621118`, 45.821962`, 45.830756`, 0.19283`, 48.332565`, 
              48.340475`, 0.173443`, 15.1728`, 26.996828`, 26.543242`, 
              42.675374`, 48.332565`, 59.837709`, 73.352436`, 78.715888`, 
              19.088691`, 37.549832`, 45.821962`, 52.039362`, 38.464577`, 
              58.468528`, 58.853418`, 65.775101`}, {5, 27, 
              13.42216434`, \(-0.9414271186`\), 5.131071538`, 0.04`, 
              1, \(-0.1474361778`\), \(-0.2`\), 
              1058.146563`, \(-0.4472733956`\), \(-0.8508643233`\), 2, 4, 5, 
              6, 0.29476`, \(-0.515104`\), 0.307913`, 0.199052`, 
              0.101406`, \(-0.108615`\), 0.757955`, 0.616927`, 0.860678`, 
              0.817451`, 1.196599`, 1.22174`, 13.063739`, 13.107634`, 
              1.062199`, 24.208902`, 24.173942`, \(-0.845979`\), 29.469631`, 
              29.482392`, 0.30879`, 39.082682`, 39.088312`, 0.136256`, 
              21.132844`, 21.155476`, 0.547678`, 39.444476`, 39.471811`, 
              0.661485`, 50.418217`, 50.410352`, \(-0.19033`\), 51.063487`, 
              51.087451`, 0.579904`, 13.063739`, 24.208902`, 29.469631`, 
              41.611101`, 51.063487`, 63.729596`, 79.517744`, 82.269734`, 
              21.132844`, 39.444476`, 50.418217`, 53.933646`, 39.082682`, 
              55.783875`, 64.873328`, 66.065582`}, {5, 29, 
              9.644386415`, \(-0.983118398`\), 6.085798571`, 0.04`, 
              1, \(-0.1722379033`\), \(-0.15`\), 
              1583.180249`, \(-0.8634640081`\), \(-1.033161576`\), 2, 4, 5, 
              6, 0.316313`, \(-0.506014`\), 0.320253`, 0.167967`, 
              0.149517`, \(-0.14339`\), 0.758528`, 0.682533`, 0.829552`, 
              0.827621`, 1.14268`, 1.143615`, 15.786158`, 15.820149`, 
              1.046929`, 26.706267`, 26.677861`, \(-0.87491`\), 34.998613`, 
              35.012422`, 0.425347`, 46.210014`, 46.213836`, 0.117717`, 
              25.43808`, 25.459323`, 0.654277`, 47.767955`, 47.793701`, 
              0.792999`, 59.946743`, 59.934789`, \(-0.36818`\), 60.695597`, 
              60.71507`, 0.599759`, 15.786158`, 26.706267`, 34.998613`, 
              49.799056`, 60.695597`, 75.727859`, 93.59353`, 96.836221`, 
              25.43808`, 47.767955`, 59.946743`, 62.76822`, 46.210014`, 
              63.573479`, 76.521263`, 77.694552`}, {5, 28, 
              17.08376039`, \(-0.9681021988`\), 7.59082639`, 0.04`, 
              1, \(-0.1966136112`\), \(-0.1`\), 
              5.640807899`, \(-1.08333782`\), \(-1.100349958`\), 2, 4, 5, 6, 
              0.335556`, \(-0.508907`\), 0.354008`, 0.149103`, 
              0.168179`, \(-0.149937`\), 0.75651`, 0.789708`, 0.815286`, 
              0.892722`, 1.119146`, 1.138454`, 16.872657`, 16.894651`, 
              0.896359`, 30.989161`, 30.973808`, \(-0.625675`\), 36.522088`, 
              36.538026`, 0.649518`, 56.746807`, 56.748971`, 0.088206`, 
              32.019382`, 32.03918`, 0.806841`, 60.161259`, 60.185618`, 
              0.992748`, 74.190706`, 74.170899`, \(-0.807228`\), 75.240531`, 
              75.207857`, \(-1.331621`\), 16.872657`, 30.989161`, 36.522088`, 
              57.575761`, 75.240531`, 93.928595`, 113.775513`, 120.25694`, 
              32.019382`, 60.161259`, 74.190706`, 76.401992`, 56.746807`, 
              76.240097`, 92.527183`, 95.973779`}, {5, 27, 
              45.79062963`, \(-1.86782627`\), 9.464635057`, 0.04`, 
              1, \(-0.1246769432`\), \(-0.05`\), 
              1570.906186`, \(-1.656372567`\), \(-1.036132598`\), 2, 4, 5, 6, 
              0.391572`, \(-0.444212`\), 0.194689`, 0.101785`, 
              0.224997`, \(-0.199532`\), 0.776186`, 0.794132`, 0.851331`, 
              0.848289`, 1.171617`, 0.988178`, 41.851131`, 41.863755`, 
              0.748558`, 50.664231`, 50.656489`, \(-0.459035`\), 56.405069`, 
              56.421101`, 0.950675`, 84.412428`, 84.41314`, 0.042225`, 
              41.534666`, 41.54639`, 0.695186`, 87.924701`, 87.944064`, 
              1.148143`, 96.199078`, 96.195402`, \(-0.217936`\), 104.971634`, 
              104.902332`, \(-4.10941`\), 41.851131`, 50.664231`, 56.405069`, 
              101.343623`, 104.971634`, 128.833817`, 162.771342`, 
              162.994509`, 41.534666`, 87.924701`, 96.199078`, 106.072128`, 
              84.412428`, 119.454859`, 127.508885`, 132.258812`}, {2, 8, 
              96377.14665`, \(-0.25`\), 0.6976196293`, 0.04`, 
              1, \(-0.296319`\), 0, 50, \(-1.53675`\), \(-1.37135`\), 2, 4, 
              5, 6, 0.332155`, \(-0.907416`\), 1.236329`, 0.10456`, 
              0.088487`, 0.028254`, 0.714045`, 0.2008`, 0.717105`, 0.364405`, 
              0.972384`, 
              0.618608`, \(-2.051083`\), \(-2.051092`\), \(-0.000032`\), 
              1.253441`, 1.253468`, 0.000098`, 1.672315`, 2.170879`, 
              1.848417`, 1.886186`, 2.295172`, 1.516309`, 2.784378`, 
              2.785212`, 0.00309`, 4.082374`, 4.083982`, 0.005962`, 
              4.753055`, 4.748239`, \(-0.017854`\), 5.4648`, 5.470227`, 
              0.020121`, \(-2.051083`\), 1.253441`, 1.672315`, 2.296613`, 
              5.4648`, 6.33291`, 6.75049`, 8.764943`, 2.784378`, 4.082374`, 
              4.753055`, 6.405716`, 3.279339`, 1.886186`, 5.998064`, 
              6.877447`}, {2, 8, 
              535587.7757`, \(-0.25`\), \(-0.2579915471`\), 0.04`, 
              1, \(-0.343728`\), 0.05`, 50, \(-1.78739`\), \(-1.48513`\), 2, 
              4, 5, 6, 0.330788`, \(-1.307629`\), 1.908634`, 
              0.082509`, \(-0.001667`\), 0.167731`, 0.6728`, 1.140979`, 
              0.651276`, 0.271943`, 0.881275`, 0.444592`, 1.827389`, 
              1.827414`, \(-0.000034`\), 0.633466`, 
              0.634019`, \(-0.000754`\), 0.16944`, \(-0.040691`\), 
              0.286924`, \(-0.040598`\), \(-0.544173`\), 
              0.687605`, \(-0.915246`\), \(-0.915398`\), 
              0.000208`, \(-0.94271`\), \(-0.942661`\), \(-0.000066`\), \
\(-1.225412`\), \(-1.224434`\), \(-0.001336`\), \(-1.411273`\), \
\(-1.582701`\), 0.234075`, 1.827389`, \(-0.040598`\), 
              0.633466`, \(-0.836036`\), \(-1.581523`\), \(-1.411273`\), \
\(-1.863722`\), \(-2.775831`\), \(-0.94271`\), \(-0.915246`\), \
\(-1.225412`\), \(-1.875369`\), \(-0.538577`\), 
              0.16944`, \(-1.466474`\), \(-2.221405`\)}, {5, 28, 
              158.0702871`, \(-0.7160908934`\), 1.661291691`, 0.04`, 
              1, \(-0.260752779`\), 0.1`, 5548.46003`, 0.4235014263`, 
              0.7685944228`, 2, 4, 5, 6, 0.401201`, \(-0.52756`\), 0.408824`, 
              0.216542`, 0.054373`, \(-0.062946`\), 0.731449`, 0.222414`, 
              1.09553`, 0.61852`, 1.591274`, 
              1.14848`, \(-0.038112`\), \(-0.037716`\), 0.004222`, 6.26814`, 
              6.268372`, 0.002471`, 8.301371`, 8.76442`, 4.938044`, 
              8.762183`, 8.939393`, 1.889798`, 6.988107`, 
              6.986307`, \(-0.019196`\), 12.921124`, 
              12.920122`, \(-0.010691`\), 13.510719`, 
              13.505983`, \(-0.050508`\), 15.58551`, 15.655491`, 
              0.746293`, \(-0.038112`\), 6.26814`, 8.301371`, 8.762183`, 
              15.58551`, 19.233588`, 22.065507`, 24.049739`, 6.988107`, 
              12.921124`, 13.510719`, 16.210712`, 11.099218`, 13.051796`, 
              18.025296`, 19.800882`}, {2, 8, 
              3.392014334`*^6, \(-0.25`\), \(-0.2981788869`\), 0.04`, 
              1, \(-0.438547`\), 0.15`, 50, \(-2.28866`\), \(-1.71269`\), 2, 
              4, 5, 6, 0.342594`, \(-3.188527`\), 2.245097`, 
              0.046977`, \(-0.34466`\), 0.552502`, 0.485`, 2.829218`, 
              0.438106`, \(-0.006469`\), 0.618836`, 0.000799`, 5.377774`, 
              5.37797`, \(-0.00032`\), 3.221304`, 3.220423`, 0.001439`, 
              2.155154`, 2.037534`, 0.192246`, 2.036868`, 1.761432`, 
              0.450191`, 1.100514`, 1.099097`, 0.002316`, 0.916344`, 
              0.282755`, 1.035581`, 0.282458`, 0.271324`, 0.018198`, 
              0.276831`, 0.000445`, 0.451745`, 5.377774`, 3.221304`, 
              2.036868`, 1.132253`, \(-0.056236`\), 0.916344`, 
              0.169169`, \(-1.393826`\), 1.100514`, \(-0.001898`\), 
              0.282458`, 0.276831`, 1.761964`, 2.155154`, 
              0.33344`, \(-0.68626`\)}, {5, 28, 
              25.54751593`, \(-0.6927238062`\), 4.043811203`, 0.0625`, 
              1, \(-0.1231886633`\), \(-0.35`\), 1808.887851`, 
              0.1178401236`, \(-0.8622537416`\), 2, 4, 5, 6, 
              0.269743`, \(-0.613239`\), 0.373036`, 0.225172`, 
              0.036021`, \(-0.039829`\), 0.783084`, 0.489968`, 0.89275`, 
              0.759033`, 1.241082`, 1.249825`, 8.544817`, 8.608588`, 
              1.112979`, 23.414692`, 23.401546`, \(-0.229428`\), 23.72518`, 
              23.708917`, \(-0.283835`\), 32.029967`, 32.040564`, 0.184944`, 
              16.865514`, 16.891608`, 0.455401`, 30.767436`, 30.799738`, 
              0.563741`, 40.416572`, 40.423791`, 0.126007`, 41.595465`, 
              41.598321`, 0.049845`, 8.544817`, 23.72518`, 23.414692`, 
              32.476533`, 41.595465`, 51.715955`, 63.422752`, 65.723694`, 
              16.865514`, 30.767436`, 40.416572`, 45.382023`, 32.029967`, 
              49.875146`, 50.575942`, 53.178941`}, {5, 28, 
              22.83356449`, \(-0.9540701761`\), 4.184939742`, 0.0625`, 
              1, \(-0.1227107247`\), \(-0.3`\), 251.8126404`, 
              0.004779616417`, \(-0.8364178588`\), 2, 4, 5, 6, 
              0.309852`, \(-0.505866`\), 0.187845`, 0.224099`, 
              0.039806`, \(-0.043772`\), 0.792925`, 0.543057`, 0.908959`, 
              0.792766`, 1.263311`, 1.268006`, 12.598938`, 12.652011`, 
              1.101127`, 25.050773`, 25.055099`, 0.089759`, 25.191465`, 
              25.159002`, \(-0.673522`\), 35.611619`, 35.618704`, 0.147003`, 
              18.10094`, 18.121813`, 0.433065`, 34.771667`, 34.799904`, 
              0.585827`, 42.532415`, 42.540969`, 0.17747`, 45.214689`, 
              45.215869`, 0.024483`, 12.598938`, 25.191465`, 25.050773`, 
              38.716668`, 45.214689`, 55.757322`, 69.120587`, 70.919071`, 
              18.10094`, 34.771667`, 42.532415`, 48.494559`, 35.611619`, 
              53.964288`, 55.103916`, 60.948635`}, {5, 27, 
              14.39445119`, \(-0.5364821037`\), 4.644070458`, 0.0625`, 
              1, \(-0.2162879383`\), \(-0.25`\), 
              465.820492`, \(-0.1706569768`\), \(-0.8901859611`\), 2, 4, 5, 
              6, 0.288655`, \(-0.691195`\), 0.61412`, 0.218754`, 
              0.053837`, \(-0.05827`\), 0.779318`, 0.576297`, 0.883347`, 
              0.809995`, 1.226832`, 1.265804`, 6.629021`, 6.678539`, 
              1.062093`, 20.045443`, 20.004806`, \(-0.8716`\), 27.083751`, 
              27.096781`, 0.279485`, 32.692104`, 32.727806`, 0.765752`, 
              19.60484`, 19.630711`, 0.554913`, 33.997578`, 34.026619`, 
              0.622883`, 45.607392`, 45.643495`, 0.774347`, 46.22771`, 
              46.202303`, \(-0.544942`\), 6.629021`, 20.045443`, 27.083751`, 
              32.692104`, 45.607392`, 57.301493`, 62.190487`, 74.83894`, 
              19.60484`, 33.997578`, 46.22771`, 48.032425`, 33.572482`, 
              47.566587`, 52.800702`, 62.962073`}, {5, 29, 
              9.777954177`, \(-0.5920016364`\), 5.332322467`, 0.0625`, 
              1, \(-0.2393509748`\), \(-0.2`\), 
              802.8575159`, \(-0.6421574618`\), \(-1.017803532`\), 2, 4, 5, 
              6, 0.313493`, \(-0.639001`\), 0.550193`, 0.192599`, 
              0.104603`, \(-0.102732`\), 0.780495`, 0.645217`, 0.863333`, 
              0.828216`, 1.191551`, 1.208574`, 8.366705`, 8.40575`, 
              1.044313`, 21.621724`, 21.588193`, \(-0.896838`\), 31.231834`, 
              31.244628`, 0.342192`, 38.671781`, 38.7003`, 0.762765`, 
              22.893584`, 22.917659`, 0.643913`, 40.274823`, 40.302487`, 
              0.73991`, 52.803882`, 52.835533`, 0.846536`, 53.155088`, 
              53.128478`, \(-0.711707`\), 8.366705`, 21.621724`, 31.231834`, 
              38.671781`, 52.803882`, 66.222037`, 72.716348`, 88.778232`, 
              22.893584`, 40.274823`, 53.155088`, 54.448302`, 38.857369`, 
              52.633666`, 63.492095`, 70.239937`}, {5, 25, 
              8.370217952`, \(-0.8555637339`\), 6.390463203`, 0.0625`, 
              1, \(-0.2306013987`\), \(-0.15`\), 
              1274.383894`, \(-1.100566877`\), \(-1.23890187`\), 2, 4, 5, 6, 
              0.354461`, \(-0.521053`\), 0.329806`, 0.160588`, 
              0.144538`, \(-0.128713`\), 0.788989`, 0.721692`, 0.836238`, 
              0.847347`, 1.139849`, 1.13302`, 15.737491`, 15.765827`, 
              1.026951`, 28.687481`, 28.663515`, \(-0.868577`\), 38.271556`, 
              38.284106`, 0.454836`, 50.617939`, 50.621084`, 0.11399`, 
              28.306983`, 28.326877`, 0.72102`, 53.06912`, 53.094859`, 
              0.932861`, 64.840071`, 64.832315`, \(-0.28112`\), 66.822684`, 
              66.829906`, 0.261718`, 15.737491`, 28.687481`, 38.271556`, 
              54.250117`, 66.822684`, 82.912709`, 97.202999`, 109.607435`, 
              28.306983`, 53.06912`, 64.840071`, 67.480142`, 50.617939`, 
              69.401507`, 81.471303`, 86.021356`}, {5, 28, 
              20.64756111`, \(-0.9035227478`\), 7.63510874`, 0.0625`, 
              1, \(-0.2445533183`\), \(-0.1`\), 
              5.028885532`, \(-1.279376062`\), \(-1.245744403`\), 2, 4, 5, 6, 
              0.377651`, \(-0.504397`\), 0.317563`, 0.146297`, 
              0.157961`, \(-0.135189`\), 0.788943`, 0.815577`, 0.83508`, 
              0.910319`, 1.13795`, 1.141366`, 16.761616`, 16.779542`, 
              0.827005`, 33.814743`, 33.803598`, \(-0.514152`\), 36.756593`, 
              36.771895`, 0.705932`, 59.646578`, 59.667367`, 0.959129`, 
              34.171526`, 34.188626`, 0.788873`, 65.064905`, 65.088593`, 
              1.092868`, 77.317109`, 77.303382`, \(-0.633282`\), 79.654828`, 
              79.604191`, \(-2.336117`\), 16.761616`, 33.814743`, 36.756593`, 
              59.646578`, 80.410247`, 99.571999`, 119.268952`, 128.039611`, 
              34.171526`, 65.064905`, 77.317109`, 79.654828`, 60.879579`, 
              81.281977`, 98.013719`, 102.202946`}, {5, 27, 
              88.52067179`, \(-0.7027294261`\), 8.596730664`, 0.0625`, 
              1, \(-0.281616568`\), \(-0.05`\), 
              2690.857772`, \(-1.518129337`\), \(-1.433856887`\), 2, 4, 5, 6, 
              0.384672`, \(-0.549279`\), 0.441209`, 0.12437`, 
              0.163784`, \(-0.121951`\), 0.778336`, 0.804033`, 0.791568`, 
              0.875471`, 1.068108`, 1.056179`, 10.785504`, 10.788798`, 
              0.17428`, 33.645471`, 33.64381`, \(-0.087869`\), 50.12003`, 
              50.134258`, 0.752825`, 59.285137`, 59.287285`, 0.11369`, 
              38.092222`, 38.10582`, 0.719496`, 70.17692`, 70.198263`, 
              1.129298`, 81.663006`, 81.619373`, \(-2.308628`\), 86.6246`, 
              86.674687`, 2.650112`, 10.785504`, 33.645471`, 50.12003`, 
              59.285137`, 86.6246`, 107.816183`, 122.396942`, 140.933302`, 
              38.092222`, 70.17692`, 81.663006`, 86.72629`, 63.487472`, 
              78.290991`, 105.428743`, 110.292092`}, {\(-2\), 1, 
              84060.60851`, \(-0.7027294261`\), 8.596730664`, 0.0625`, 
              1, \(-0.340013`\), 0, 50, \(-1.8917`\), \(-1.53604`\), 2, 4, 5, 
              6, 0.384672`, \(-0.549279`\), 0.441209`, 0.12437`, 
              0.163784`, \(-0.121951`\), 0.778336`, 0.804033`, 0.791568`, 
              0.875471`, 1.068108`, 1.056179`, 10.785504`, 10.788798`, 
              0.17428`, 33.645471`, 33.64381`, \(-0.087869`\), 50.12003`, 
              50.134258`, 0.752825`, 59.285137`, 59.287285`, 0.11369`, 
              38.092222`, 38.10582`, 0.719496`, 70.17692`, 70.198263`, 
              1.129298`, 81.663006`, 81.619373`, \(-2.308628`\), 86.6246`, 
              86.674687`, 2.650112`, 10.785504`, 33.645471`, 50.12003`, 
              59.285137`, 86.6246`, 107.816183`, 122.396942`, 140.933302`, 
              38.092222`, 70.17692`, 81.663006`, 86.72629`, 63.487472`, 
              78.290991`, 105.428743`, 110.292092`}, {2, 8, 
              412838.9561`, \(-0.25`\), \(-0.2904733272`\), 0.0625`, 
              1, \(-0.38351`\), 0.05`, 50, \(-2.22058`\), \(-1.6615`\), 2, 4, 
              5, 6, 0.371173`, \(-1.133197`\), 1.585531`, 0.059399`, 
              0.015854`, 0.124932`, 0.71694`, 0.732321`, 0.680688`, 0.21457`, 
              0.910092`, 0.338005`, 1.797813`, 
              1.797834`, \(-0.000035`\), \(-0.151699`\), \(-0.150643`\), \
\(-0.001822`\), \(-0.292699`\), \(-0.292758`\), 
              0.000102`, \(-0.312509`\), \(-0.773012`\), 
              0.794393`, \(-1.152263`\), \(-1.152213`\), \(-0.000087`\), \
\(-1.331899`\), \(-1.331989`\), 
              0.000156`, \(-1.692479`\), \(-1.691599`\), \(-0.001517`\), \
\(-1.742132`\), \(-2.116722`\), 0.646188`, 
              1.797813`, \(-0.292699`\), \(-0.151699`\), \(-0.771894`\), \
\(-2.097031`\), \(-2.333269`\), \(-1.742132`\), \(-4.016717`\), \
\(-1.152263`\), \(-1.331899`\), \(-1.692479`\), \(-2.459187`\), \
\(-1.004321`\), \(-0.312509`\), \(-2.343526`\), \(-2.528343`\)}, {5, 28, 
              295.2454032`, \(-0.680242332`\), 0.6634930987`, 0.0625`, 
              1, \(-0.3077473303`\), 0.1`, 7614.067812`, 
              2.376797039`, \(-5.585152272`\), 2, 4, 5, 6, 
              0.438799`, \(-0.508933`\), 0.345764`, 0.068897`, 
              0.170277`, \(-0.111728`\), 
              0.764742`, \(-0.053651`\), \(-0.01461`\), 
              0.301146`, \(-0.404104`\), 
              0.493832`, \(-0.016178`\), \(-0.016032`\), 0.000679`, 
              2.753621`, 2.753662`, 0.000193`, 3.667824`, 3.712121`, 
              0.206342`, 3.711171`, 4.010905`, 1.396237`, 2.935736`, 
              2.935121`, \(-0.002869`\), 5.454173`, 
              5.453603`, \(-0.002655`\), 5.745034`, 
              5.743655`, \(-0.006422`\), 6.624341`, 6.628288`, 
              0.018385`, \(-0.016178`\), 2.753621`, 3.667824`, 3.711171`, 
              6.624341`, 8.151135`, 8.528161`, 11.012066`, 2.935736`, 
              5.454173`, 5.745034`, 6.653638`, 4.750418`, 5.50218`, 
              7.770693`, 8.428727`}, {2, 8, 
              2.421483677`*^6, \(-0.25`\), \(-0.4031004004`\), 0.0625`, 
              1, \(-0.470504`\), 0.15`, 50, \(-2.87835`\), \(-1.91241`\), 2, 
              4, 5, 6, 0.369041`, \(-2.239567`\), 
              2.380428`, \(-0.001583`\), \(-0.187146`\), 0.319754`, 
              0.592326`, 1.118235`, 0.514437`, \(-0.063829`\), 
              0.691202`, \(-0.131194`\), 6.125649`, 
              6.125885`, \(-0.000563`\), 2.85839`, 2.860375`, \(-0.004723`\), 
              2.083557`, 2.081308`, 0.005353`, 1.671656`, 1.070924`, 
              1.429844`, 0.244898`, 0.244892`, 
              0.000014`, \(-0.378932`\), \(-0.440927`\), 
              0.147559`, \(-0.441951`\), \(-0.685973`\), 
              0.580813`, \(-0.668808`\), \(-0.814423`\), 0.346589`, 
              6.125649`, 2.083557`, 2.85839`, 
              0.11235`, \(-1.195955`\), \(-0.378932`\), \(-0.668808`\), \
\(-3.044757`\), \(-0.441951`\), 0.244898`, \(-0.686272`\), \(-0.841406`\), 
              1.074509`, 1.671656`, \(-0.921662`\), \(-1.486295`\)}, {5, 28, 
              24.26563901`, \(-0.4035191394`\), 3.954148043`, 0.09`, 
              1, \(-0.2327346795`\), \(-0.35`\), 2055.524517`, 
              0.02934609388`, \(-1.17389382`\), 2, 4, 5, 6, 
              0.282667`, \(-0.86358`\), 0.826771`, 0.227289`, 
              0.032543`, \(-0.035653`\), 0.808645`, 0.493409`, 0.876008`, 
              0.764052`, 1.200973`, 1.258781`, 3.903931`, 3.966256`, 
              1.114577`, 19.371465`, 19.324114`, \(-0.846786`\), 23.845219`, 
              23.858238`, 0.232826`, 28.204987`, 28.251084`, 0.824359`, 
              17.234593`, 17.261345`, 0.478424`, 29.147606`, 29.179236`, 
              0.565655`, 40.217712`, 40.223597`, 0.105243`, 40.328617`, 
              40.326949`, \(-0.029836`\), 3.903931`, 19.371465`, 23.845219`, 
              28.204987`, 40.217712`, 50.447852`, 58.43222`, 63.228492`, 
              17.234593`, 29.147606`, 40.328617`, 43.046149`, 29.58825`, 
              47.010187`, 43.966955`, 48.051992`}, {5, 28, 
              21.81026268`, \(-0.5084469919`\), 4.184526627`, 0.09`, 
              1, \(-0.2503904925`\), \(-0.3`\), 243.8145379`, 
              0.1346729664`, \(-0.9833834711`\), 2, 4, 5, 6, 
              0.313841`, \(-0.717096`\), 0.592334`, 0.227838`, 
              0.031368`, \(-0.035149`\), 0.813167`, 0.550481`, 0.912356`, 
              0.80195`, 1.261017`, 1.283186`, 6.365639`, 6.417695`, 
              1.093811`, 20.216986`, 20.174679`, \(-0.888965`\), 25.532905`, 
              25.543823`, 0.229406`, 32.51093`, 32.518315`, 0.15518`, 
              18.685159`, 18.708525`, 0.490974`, 32.69182`, 32.720432`, 
              0.601201`, 43.079671`, 43.078589`, \(-0.022726`\), 43.703516`, 
              43.707638`, 0.086607`, 6.365639`, 20.216986`, 25.532905`, 
              32.730021`, 43.703516`, 54.503533`, 62.833667`, 68.372777`, 
              18.685159`, 32.69182`, 43.079671`, 45.809745`, 32.51093`, 
              49.24429`, 48.360162`, 54.275886`}, {5, 28, 
              12.35629126`, \(-0.3949769214`\), 4.737965574`, 0.09`, 
              1, \(-0.2927910853`\), \(-0.25`\), 
              122.3377116`, \(-0.3740631855`\), \(-1.028491585`\), 2, 4, 5, 
              6, 0.319512`, \(-0.822185`\), 0.841471`, 0.214373`, 
              0.060809`, \(-0.061613`\), 0.807229`, 0.611928`, 0.898012`, 
              0.829385`, 1.239898`, 1.26589`, 3.717093`, 3.760791`, 
              1.058404`, 18.521533`, 18.483555`, \(-0.919886`\), 28.302423`, 
              28.314044`, 0.281484`, 32.616954`, 32.649569`, 0.789977`, 
              21.077376`, 21.101923`, 0.594559`, 35.436553`, 35.465494`, 
              0.701003`, 47.536134`, 47.564792`, 0.69414`, 48.213627`, 
              48.19367`, \(-0.483364`\), 3.717093`, 18.521533`, 28.302423`, 
              32.616954`, 47.536134`, 59.833752`, 62.828389`, 74.946105`, 
              21.077376`, 35.436553`, 48.213627`, 48.829683`, 34.264838`, 
              47.760072`, 50.313451`, 62.758527`}, {5, 27, 
              9.732090708`, \(-0.5506068316`\), 5.521371548`, 0.09`, 
              1, \(-0.2976824322`\), \(-0.2`\), 
              5532.408618`, \(-0.8501881933`\), \(-1.202434026`\), 2, 4, 5, 
              6, 0.356786`, \(-0.64568`\), 0.533847`, 0.188004`, 
              0.099166`, \(-0.088483`\), 0.812817`, 0.692375`, 0.876252`, 
              0.855832`, 1.198806`, 1.210487`, 8.302969`, 8.335584`, 
              1.028017`, 23.195838`, 23.167447`, \(-0.894847`\), 33.756397`, 
              33.767442`, 0.348109`, 42.56742`, 42.571606`, 0.131941`, 
              25.196253`, 25.217775`, 0.678362`, 44.729165`, 44.756691`, 
              0.867606`, 56.807825`, 56.795047`, \(-0.402741`\), 57.771413`, 
              57.803147`, 1.000244`, 8.302969`, 23.195838`, 33.756397`, 
              43.166989`, 57.771413`, 71.976126`, 78.447296`, 90.542593`, 
              25.196253`, 44.729165`, 56.807825`, 58.014538`, 42.56742`, 
              58.884`, 62.943964`, 79.156021`}, {5, 28, 
              11.55752344`, \(-0.8413759822`\), 6.388428195`, 0.09`, 
              1, \(-0.2806009401`\), \(-0.15`\), 
              11.7088463`, \(-1.190932117`\), \(-1.273922226`\), 2, 4, 5, 6, 
              0.402446`, \(-0.500491`\), 0.255241`, 0.166091`, 
              0.128477`, \(-0.113138`\), 0.823542`, 0.773685`, 0.87758`, 
              0.89561`, 1.194965`, 1.179896`, 16.336577`, 16.360458`, 
              0.982389`, 31.42077`, 31.402153`, \(-0.765857`\), 37.466104`, 
              37.47645`, 0.425575`, 54.540677`, 54.542966`, 0.09417`, 
              30.114649`, 30.131427`, 0.690197`, 57.653836`, 57.678577`, 
              1.017759`, 67.154615`, 67.15204`, \(-0.10593`\), 70.581369`, 
              70.560732`, \(-0.848931`\), 16.336577`, 31.42077`, 37.466104`, 
              57.77342`, 71.458354`, 87.83516`, 101.191916`, 110.733437`, 
              30.114649`, 57.653836`, 67.154615`, 70.581369`, 54.540677`, 
              76.665855`, 79.794178`, 95.735655`}, {5, 28, 
              17.07386773`, \(-0.8615827767`\), 8.022500985`, 0.09`, 
              1, \(-0.2955510219`\), \(-0.1`\), 
              5.608466885`, \(-1.547681791`\), \(-1.409423782`\), 2, 4, 5, 6, 
              0.423082`, \(-0.487554`\), 0.250737`, 0.137952`, 
              0.153636`, \(-0.126035`\), 0.822473`, 0.849346`, 0.853967`, 
              0.930067`, 1.154635`, 1.133177`, 18.170195`, 18.185039`, 
              0.806157`, 39.201292`, 39.186548`, \(-0.800705`\), 41.003316`, 
              41.023996`, 1.123057`, 66.06035`, 66.080103`, 1.072697`, 
              38.044776`, 38.060092`, 0.831712`, 73.67489`, 73.699698`, 
              1.347246`, 84.136089`, 84.127461`, \(-0.468563`\), 87.069672`, 
              87.045274`, \(-1.32499`\), 18.170195`, 39.201292`, 41.003316`, 
              66.06035`, 89.963964`, 110.424108`, 125.407803`, 139.676091`, 
              38.044776`, 73.67489`, 84.136089`, 87.069672`, 68.459261`, 
              93.908147`, 101.150066`, 118.665331`}, {5, 28, 
              82.512688`, \(-0.7179348419`\), 9.234775618`, 0.09`, 
              1, \(-0.3277355376`\), \(-0.05`\), 
              9.725460911`, \(-1.874907104`\), \(-1.668934877`\), 2, 4, 5, 6, 
              0.431246`, \(-0.516429`\), 0.33614`, 0.108761`, 
              0.165806`, \(-0.121529`\), 0.813577`, 0.805339`, 0.7997`, 
              0.860698`, 1.064916`, 1.005189`, 12.775766`, 12.779196`, 
              0.218563`, 41.231088`, 41.230261`, \(-0.052687`\), 51.646087`, 
              51.659741`, 0.869985`, 71.73242`, 71.738828`, 0.408305`, 
              43.395136`, 43.407252`, 0.772024`, 82.230673`, 82.254257`, 
              1.502783`, 93.459808`, 93.424567`, \(-2.245562`\), 95.813094`, 
              95.801537`, \(-0.736392`\), 12.775766`, 41.231088`, 51.646087`, 
              71.73242`, 100.126857`, 123.345323`, 133.436365`, 155.346743`, 
              43.395136`, 82.230673`, 93.459808`, 95.813094`, 74.331534`, 
              96.9297`, 113.722658`, 129.82332`}, {5, 29, 
              201.6289145`, \(-0.5235757493`\), 0.7409357215`, 0.09`, 
              1, \(-0.3605305393`\), 0, 370.842657`, 
              0.7491295895`, \(-1.31422714`\), 2, 4, 5, 6, 
              0.436154`, \(-0.57291`\), 0.470794`, 0.210201`, 
              0.061309`, \(-0.064157`\), 0.800614`, 0.033055`, 0.843538`, 
              0.571599`, 1.149451`, 
              1.104566`, \(-0.037097`\), \(-0.037091`\), 0.00003`, 2.936459`, 
              2.936493`, 0.000178`, 4.356237`, 4.357258`, 0.005278`, 
              4.47122`, 4.836594`, 1.889203`, 3.421668`, 3.42215`, 0.002493`, 
              6.210684`, 6.212204`, 0.007861`, 6.793863`, 
              6.790192`, \(-0.018983`\), 7.566216`, 
              7.565352`, \(-0.004467`\), \(-0.037097`\), 2.936459`, 
              4.356237`, 4.47122`, 7.631035`, 9.436977`, 9.444579`, 
              11.953813`, 3.421668`, 6.210684`, 6.793863`, 7.566216`, 
              5.441434`, 6.570204`, 8.561448`, 9.822598`}, {\(-2\), 8, 
              255465.9259`, \(-0.25`\), 0.4808316766`, 0.09`, 
              1, \(-0.420235`\), 0.05`, 50, \(-2.7325`\), \(-1.88354`\), 2, 
              4, 5, 6, 0.420393`, \(-0.927747`\), 1.174898`, 0.028072`, 
              0.054315`, 0.056027`, 0.765011`, 0.242777`, 0.70779`, 
              0.147277`, 0.932317`, 
              0.206131`, \(-2.31991`\), \(-2.319933`\), \(-0.000074`\), 
              0.989982`, 0.990961`, 0.003167`, 1.014466`, 1.014516`, 
              0.000159`, 1.873417`, 1.872967`, \(-0.001457`\), 2.081732`, 
              2.081664`, \(-0.000222`\), 2.856186`, 2.85624`, 0.000174`, 
              3.424196`, 3.422982`, \(-0.003926`\), 3.655471`, 4.103176`, 
              1.447971`, \(-2.31991`\), 1.014466`, 0.989982`, 1.873417`, 
              4.098389`, 5.024505`, 3.655471`, 6.50815`, 2.081732`, 
              2.856186`, 3.424196`, 4.543738`, 2.380647`, 1.919463`, 
              4.00473`, 5.284875`}, {2, 8, 
              693570.1306`, \(-0.25`\), \(-0.3878922185`\), 0.09`, 
              1, \(-0.458136`\), 0.1`, 50, \(-3.14656`\), \(-2.02767`\), 2, 
              4, 5, 6, 0.418892`, \(-1.154561`\), 1.506264`, \(-0.014768`\), 
              0.014292`, 0.087778`, 0.731972`, 0.360225`, 0.647273`, 
              0.012122`, 0.844906`, \(-0.042107`\), 3.120793`, 
              3.120866`, \(-0.000191`\), 0.178401`, 
              0.179809`, \(-0.003659`\), \(-0.339712`\), \(-0.340153`\), 
              0.001146`, \(-0.709068`\), \(-1.071801`\), 
              0.943019`, \(-1.476062`\), \(-1.476112`\), 
              0.00013`, \(-1.547925`\), \(-1.547386`\), \(-0.001401`\), \
\(-2.288507`\), \(-2.287961`\), \(-0.001419`\), \(-2.302394`\), \
\(-2.818411`\), 1.341518`, 3.120793`, \(-0.339712`\), 
              0.178401`, \(-1.070843`\), \(-2.813103`\), \(-3.223289`\), \
\(-2.302394`\), \(-4.480736`\), \(-1.547925`\), \(-1.476062`\), \
\(-2.288507`\), \(-3.16179`\), \(-1.208079`\), \(-0.709068`\), \
\(-2.638628`\), \(-3.347809`\)}, {2, 8, 
              1.456994717`*^6, \(-0.25`\), \(-0.3930393327`\), 0.09`, 
              1, \(-0.496036`\), 0.15`, 50, \(-3.56061`\), \(-2.1718`\), 2, 
              4, 5, 6, 0.415899`, \(-1.477291`\), 
              1.808372`, \(-0.059311`\), \(-0.034452`\), 0.107688`, 
              0.686004`, 0.375364`, 0.575672`, \(-0.140634`\), 
              0.745189`, \(-0.309877`\), 4.611825`, 
              4.612009`, \(-0.000481`\), 1.929069`, 
              1.931153`, \(-0.005451`\), 0.425702`, 0.423477`, 0.00582`, 
              0.135051`, \(-0.310452`\), 
              1.165184`, \(-0.506865`\), \(-0.507352`\), 
              0.001272`, \(-1.302538`\), \(-1.301254`\), \(-0.003358`\), \
\(-1.693621`\), \(-1.729443`\), 0.09369`, \(-1.729726`\), \(-2.222386`\), 
              1.288522`, 4.611825`, 0.425702`, 
              1.929069`, \(-1.125984`\), \(-2.220846`\), \(-2.190918`\), \
\(-1.693621`\), \(-3.478721`\), \(-1.302538`\), \(-0.506865`\), \
\(-1.729726`\), \(-2.258141`\), \(-0.306614`\), 
              0.135051`, \(-1.885114`\), \(-2.326023`\)}, {5, 28, 
              29.82555823`, \(-0.421281993`\), 3.840866243`, 0.1225`, 
              1, \(-0.3010967342`\), \(-0.35`\), 286.9600864`, 
              0.2946568933`, \(-1.041641572`\), 2, 4, 5, 6, 
              0.333531`, \(-0.820427`\), 0.711188`, 0.228741`, 
              0.030345`, \(-0.034985`\), 0.844726`, 0.521961`, 0.944514`, 
              0.784375`, 1.301717`, 1.275148`, 4.689767`, 4.744261`, 
              1.11695`, 19.358687`, 19.3141`, \(-0.913876`\), 24.301186`, 
              24.310959`, 0.200301`, 30.96116`, 30.968637`, 0.153255`, 
              18.007954`, 18.030025`, 0.452367`, 31.061037`, 31.089916`, 
              0.591921`, 40.632075`, 40.634732`, 0.054454`, 41.756238`, 
              41.745326`, \(-0.223663`\), 4.689767`, 19.358687`, 24.301186`, 
              31.324441`, 41.756238`, 51.939907`, 62.267461`, 64.558085`, 
              18.007954`, 31.061037`, 40.632075`, 43.543549`, 30.96116`, 
              48.254237`, 46.146316`, 48.895893`}, {5, 28, 
              26.43922532`, \(-0.8464433826`\), 3.973860254`, 0.1225`, 
              1, \(-0.2599403828`\), \(-0.3`\), 342.3053502`, 
              0.2430421538`, \(-1.185670587`\), 2, 4, 5, 6, 
              0.403093`, \(-0.488797`\), 0.096374`, 0.229626`, 
              0.028719`, \(-0.032633`\), 0.863199`, 0.595128`, 0.944085`, 
              0.835123`, 1.290259`, 1.30891`, 13.406402`, 13.449392`, 
              1.101804`, 25.011589`, 24.979038`, \(-0.834273`\), 26.344276`, 
              26.348772`, 0.115234`, 37.811951`, 37.81623`, 0.109677`, 
              19.644368`, 19.659809`, 0.39575`, 38.179491`, 38.203785`, 
              0.622645`, 43.280138`, 43.285627`, 0.140689`, 48.173027`, 
              48.158405`, \(-0.374763`\), 13.406402`, 25.011589`, 26.344276`, 
              42.080039`, 48.173027`, 58.577458`, 73.16848`, 74.01958`, 
              19.644368`, 38.179491`, 43.280138`, 49.542096`, 37.811951`, 
              55.209335`, 58.139555`, 58.511985`}, {5, 29, 
              20.34176284`, \(-0.8549744555`\), 4.400989506`, 0.1225`, 
              1, \(-0.2867200061`\), \(-0.25`\), 
              533.8742756`, \(-0.0009555392462`\), \(-1.117587748`\), 2, 4, 
              5, 6, 0.420535`, \(-0.479185`\), 0.112628`, 0.229536`, 
              0.029938`, \(-0.033465`\), 0.862821`, 0.673304`, 0.954286`, 
              0.892001`, 1.307924`, 1.345608`, 14.676598`, 14.712136`, 
              1.052349`, 26.234033`, 26.203965`, \(-0.890368`\), 29.148652`, 
              29.156007`, 0.217789`, 41.544654`, 41.54797`, 0.098176`, 
              21.895776`, 21.910438`, 0.43418`, 42.702148`, 42.72583`, 
              0.701298`, 47.974384`, 47.978249`, 0.114452`, 53.242257`, 
              53.216611`, \(-0.75943`\), 14.676598`, 26.234033`, 29.148652`, 
              46.423344`, 53.242257`, 64.763716`, 80.529991`, 80.573409`, 
              21.895776`, 42.702148`, 47.974384`, 53.762269`, 41.544654`, 
              60.34245`, 62.119515`, 64.482005`}, {5, 28, 
              8.609306924`, \(-0.4356531873`\), 5.901930228`, 0.1225`, 
              1, \(-0.3753058061`\), \(-0.2`\), 
              127.5472036`, \(-1.190206832`\), \(-1.511779946`\), 2, 4, 5, 6, 
              0.39344`, \(-0.716759`\), 0.640153`, 0.174661`, 
              0.091947`, \(-0.062459`\), 0.841664`, 0.721663`, 0.866235`, 
              0.858183`, 1.16815`, 1.168523`, 5.021804`, 5.048191`, 
              0.980366`, 23.979875`, 23.956904`, \(-0.853419`\), 36.963211`, 
              36.973178`, 0.370304`, 45.777429`, 45.797748`, 0.754903`, 
              28.280843`, 28.301637`, 0.77257`, 49.456282`, 49.486165`, 
              1.110244`, 62.023395`, 62.009075`, \(-0.532047`\), 62.189152`, 
              62.175342`, \(-0.513105`\), 5.021804`, 23.979875`, 36.963211`, 
              45.777429`, 63.651169`, 79.175925`, 85.892882`, 99.102793`, 
              28.280843`, 49.456282`, 62.023395`, 62.189152`, 46.235596`, 
              64.948907`, 64.489724`, 75.367491`}, {5, 28, 
              14.2861495`, \(-0.8100935527`\), 6.534666672`, 0.1225`, 
              1, \(-0.3377746695`\), \(-0.15`\), 
              9.941182325`, \(-1.441910257`\), \(-1.518977928`\), 2, 4, 5, 6, 
              0.451464`, \(-0.476764`\), 0.171113`, 0.159328`, 
              0.124635`, \(-0.104566`\), 0.858017`, 0.800385`, 0.884871`, 
              0.908956`, 1.191021`, 1.169123`, 17.102605`, 17.122388`, 
              0.933813`, 34.983229`, 34.968337`, \(-0.702928`\), 39.190199`, 
              39.200306`, 0.477075`, 59.659219`, 59.6608`, 0.074612`, 
              32.650845`, 32.66549`, 0.691255`, 63.721075`, 63.747806`, 
              1.261751`, 70.828939`, 70.826585`, \(-0.111118`\), 75.267813`, 
              75.246038`, \(-1.027857`\), 17.102605`, 34.983229`, 39.190199`, 
              61.538926`, 77.809282`, 94.80379`, 112.457383`, 118.235459`, 
              32.650845`, 63.721075`, 70.828939`, 75.267813`, 59.659219`, 
              85.164213`, 84.538541`, 96.258354`}, {2, 28, 
              31.36822567`, \(-0.8096100341`\), 7.950581641`, 0.1225`, 
              1, \(-0.3539444896`\), \(-0.1`\), 
              7.170982442`, \(-1.835593464`\), \(-1.732042834`\), 2, 4, 5, 6, 
              0.469677`, \(-0.466299`\), 0.171449`, 0.128548`, 
              0.149729`, \(-0.118543`\), 0.856017`, 0.828403`, 0.848031`, 
              0.900592`, 1.128006`, 1.08434`, 18.22187`, 18.232857`, 
              0.656446`, 42.349883`, 42.338361`, \(-0.68838`\), 44.506121`, 
              44.523787`, 1.055511`, 70.259628`, 70.276048`, 0.981026`, 
              39.822964`, 39.836021`, 0.780139`, 78.462032`, 78.490833`, 
              1.720784`, 85.767423`, 85.758528`, \(-0.531441`\), 89.527993`, 
              89.504909`, \(-1.379174`\), 18.22187`, 42.349883`, 44.506121`, 
              70.259628`, 94.521132`, 115.060639`, 133.982244`, 143.783629`, 
              39.822964`, 78.462032`, 85.767423`, 89.527993`, 72.103754`, 
              101.222851`, 100.707938`, 120.463961`}, {5, 28, 
              101.4310464`, \(-0.6680778598`\), 8.478162743`, 0.1225`, 
              1, \(-0.387275569`\), \(-0.05`\), 
              10.98360161`, \(-2.197277016`\), \(-2.030967212`\), 2, 4, 5, 6, 
              0.475429`, \(-0.49707`\), 0.25873`, 0.096822`, 
              0.160764`, \(-0.114743`\), 0.846472`, 0.707135`, 0.787193`, 
              0.758303`, 1.026694`, 0.89064`, 10.688817`, 10.69122`, 
              0.155033`, 41.187118`, 41.186529`, \(-0.038032`\), 49.843589`, 
              49.855018`, 0.737063`, 69.974171`, 69.981955`, 0.502016`, 
              41.949478`, 41.959046`, 0.617101`, 80.727017`, 80.753502`, 
              1.708095`, 89.075331`, 89.049509`, \(-1.665302`\), 90.098652`, 
              90.086007`, \(-0.815537`\), 10.688817`, 41.187118`, 49.843589`, 
              69.974171`, 97.235565`, 118.83281`, 132.156393`, 146.70821`, 
              41.949478`, 80.727017`, 89.075331`, 90.098652`, 72.300055`, 
              97.994695`, 99.564788`, 126.843128`}, {2, 23, 
              156.0417465`, \(-0.519393006`\), 1.162867416`, 0.1225`, 
              1, \(-0.4168264097`\), 0, 
              533.0203498`, \(-1.020795648`\), \(-2.399179141`\), 2, 4, 5, 6, 
              0.480943`, \(-0.540313`\), 0.361814`, 0.188059`, 
              0.081987`, \(-0.062385`\), 0.835142`, 0.156289`, 0.711749`, 
              0.552241`, 0.899765`, 
              1.022116`, \(-0.026363`\), \(-0.026358`\), 0.000047`, 
              5.148638`, 5.148675`, 0.000337`, 7.180458`, 7.181932`, 
              0.013188`, 7.908023`, 8.140158`, 2.077232`, 5.661205`, 
              5.661839`, 0.00568`, 10.494272`, 10.496837`, 0.022953`, 
              11.420407`, 11.414175`, \(-0.055772`\), 12.036312`, 12.03734`, 
              0.009205`, \(-0.026363`\), 5.148638`, 7.180458`, 7.908023`, 
              12.784203`, 15.67677`, 16.52195`, 18.9394`, 5.661205`, 
              10.494272`, 11.420407`, 12.036312`, 9.194496`, 11.933131`, 
              12.399685`, 16.879`}, {2, 7, 
              108707.2851`, \(-0.287658203`\), \(-0.5923582986`\), 0.1225`, 
              1, \(-0.453904`\), 0.05`, 50, \(-3.32314`\), \(-2.15126`\), 2, 
              4, 5, 6, 0.479395`, \(-0.695107`\), 0.67787`, \(-0.012444`\), 
              0.132746`, \(-0.060551`\), 0.816112`, \(-0.019992`\), 
              0.731999`, 0.07732`, 0.947512`, 0.058249`, 1.651036`, 
              1.651044`, \(-0.000038`\), \(-2.076674`\), \(-2.076687`\), 
              0.000057`, \(-2.369202`\), \(-2.768002`\), 
              1.811979`, \(-3.296568`\), \(-3.297221`\), 
              0.002968`, \(-2.797751`\), \(-2.797683`\), \(-0.000308`\), \
\(-4.547142`\), \(-4.547132`\), \(-0.000045`\), \(-5.129512`\), \
\(-5.127984`\), \(-0.006941`\), \(-5.965726`\), \(-5.967173`\), 0.006574`, 
              1.651036`, \(-2.076674`\), \(-2.369202`\), \(-3.296568`\), \
\(-5.965726`\), \(-7.349857`\), \(-6.798214`\), \(-8.353362`\), \
\(-2.797751`\), \(-4.547142`\), \(-5.129512`\), \(-6.00574`\), \
\(-3.947121`\), \(-4.662387`\), \(-4.666823`\), \(-7.863061`\)}, {\(-2\), 8, 
              319357.2343`, \(-0.25`\), \(-6.25688774`\), 0.1225`, 
              1, \(-0.484524`\), 0.1`, 50, \(-3.82929`\), \(-2.32106`\), 2, 
              4, 5, 6, 0.480131`, \(-0.821503`\), 0.890832`, \(-0.066387`\), 
              0.110491`, \(-0.052419`\), 0.794496`, \(-0.189419`\), 
              0.679532`, \(-0.232226`\), 0.867352`, \(-0.33677`\), 
              33.750161`, 
              33.750768`, \(-0.029162`\), \(-12.997474`\), \(-13.001098`\), 
              0.174228`, \(-16.588841`\), \(-16.59211`\), 
              0.157138`, \(-28.365083`\), \(-28.363835`\), \(-0.059962`\), \
\(-28.356398`\), \(-28.350591`\), \(-0.279129`\), \(-38.008066`\), \
\(-38.004698`\), \(-0.161903`\), \(-48.813449`\), \(-48.806251`\), \
\(-0.345989`\), \(-57.414338`\), \(-57.427476`\), 0.631498`, 
              33.750161`, \(-16.588841`\), \(-12.997474`\), \(-28.365083`\), \
\(-57.414338`\), \(-70.226432`\), \(-59.548974`\), \(-75.269335`\), \
\(-28.356398`\), \(-38.008066`\), \(-48.813449`\), \(-60.552317`\), \
\(-33.765431`\), \(-36.962471`\), \(-38.221132`\), \(-74.312962`\)}, {2, 8, 
              682178.1949`, \(-0.25`\), \(-1.405518775`\), 0.1225`, 
              1, \(-0.515145`\), 0.15`, 50, \(-4.33544`\), \(-2.49087`\), 2, 
              4, 5, 6, 0.479589`, \(-0.938333`\), 1.021844`, \(-0.124814`\), 
              0.11133`, \(-0.085818`\), 0.767868`, \(-0.098325`\), 
              0.621704`, \(-0.255508`\), 0.780061`, \(-0.541301`\), 
              11.189597`, 
              11.190017`, \(-0.004529`\), \(-2.36793`\), \(-2.371213`\), 
              0.03541`, \(-2.440673`\), \(-2.435927`\), \(-0.051183`\), \
\(-2.893262`\), \(-2.895151`\), 
              0.020382`, \(-5.96476`\), \(-5.962014`\), \(-0.029615`\), \
\(-6.053888`\), \(-6.053835`\), \(-0.000561`\), \(-9.746865`\), \
\(-9.745964`\), \(-0.009716`\), \(-11.254974`\), \(-11.539464`\), 3.068261`, 
              11.189597`, \(-2.36793`\), \(-2.440673`\), \(-2.893262`\), \
\(-11.538098`\), \(-13.778299`\), \(-11.254974`\), \(-14.11989`\), \
\(-5.96476`\), \(-6.053888`\), \(-9.746865`\), \(-12.35464`\), \
\(-5.621285`\), \(-5.956769`\), \(-6.727252`\), \(-13.75425`\)}, {5, 28, 
              35.11282161`, \(-0.3108672981`\), 3.901362794`, 0.16`, 
              1, \(-0.3926581312`\), \(-0.35`\), 3474.865101`, 
              1.02959821`, \(-1.612577775`\), 2, 4, 5, 6, 
              0.365556`, \(-1.009496`\), 1.019387`, 0.204737`, 
              0.053637`, \(-0.04363`\), 0.873763`, 0.518078`, 0.895106`, 
              0.744656`, 1.203352`, 1.17763`, 1.465917`, 1.515205`, 1.12469`, 
              17.424984`, 17.382246`, \(-0.975241`\), 25.342909`, 25.351461`, 
              0.195147`, 31.234916`, 31.241254`, 0.144616`, 19.231683`, 
              19.252513`, 0.4753`, 32.017369`, 32.047244`, 0.681712`, 
              42.198096`, 42.198885`, 0.018006`, 43.092085`, 
              43.055273`, \(-0.839998`\), 1.465917`, 17.424984`, 25.342909`, 
              31.287726`, 43.092085`, 53.836558`, 64.497069`, 66.405939`, 
              19.231683`, 32.017369`, 42.198096`, 43.752013`, 31.234916`, 
              48.940761`, 45.242581`, 48.003287`}, {5, 28, 
              30.6263805`, \(-0.9199945321`\), 3.859376891`, 0.16`, 
              1, \(-0.3093931652`\), \(-0.3`\), 256.6080161`, 
              0.1157635269`, \(-1.407860459`\), 2, 4, 5, 6, 
              0.468436`, \(-0.430316`\), \(-0.061419`\), 0.232137`, 
              0.02524`, \(-0.028821`\), 0.903674`, 0.627508`, 0.973968`, 
              0.861583`, 1.321731`, 1.332844`, 15.849466`, 15.886544`, 
              1.072506`, 26.971067`, 26.956862`, \(-0.410889`\), 26.987946`, 
              26.976831`, \(-0.321517`\), 40.487753`, 40.490809`, 0.088391`, 
              20.483748`, 20.49602`, 0.354958`, 41.668879`, 41.697959`, 
              0.841162`, 43.29791`, 43.295605`, \(-0.066664`\), 50.879647`, 
              50.837678`, \(-1.213992`\), 15.849466`, 26.987946`, 26.971067`, 
              45.350592`, 50.879647`, 61.139496`, 75.681499`, 76.803461`, 
              20.483748`, 41.668879`, 43.29791`, 51.440782`, 40.487753`, 
              57.222378`, 62.023566`, 62.335178`}, {5, 29, 
              14.36967753`, \(-0.3636493728`\), 4.980051971`, 0.16`, 
              1, \(-0.4302807468`\), \(-0.25`\), 
              176.1652612`, \(-0.5396180746`\), \(-1.216522475`\), 2, 4, 5, 
              6, 0.414373`, \(-0.814698`\), 0.746377`, 0.217659`, 
              0.045381`, \(-0.040491`\), 0.87422`, 0.730511`, 0.956661`, 
              0.920275`, 1.309183`, 1.331334`, 2.498271`, 2.527773`, 
              0.974071`, 20.905587`, 20.879527`, \(-0.860444`\), 32.354777`, 
              32.362856`, 0.266761`, 40.452091`, 40.455766`, 0.121326`, 
              24.987253`, 25.006168`, 0.624503`, 43.140036`, 43.170337`, 
              1.000468`, 53.87637`, 53.871094`, \(-0.174193`\), 54.219694`, 
              54.193746`, \(-0.856712`\), 2.498271`, 20.905587`, 32.354777`, 
              40.575641`, 55.865308`, 69.364579`, 79.001065`, 86.227957`, 
              24.987253`, 43.140036`, 53.87637`, 54.219694`, 40.452091`, 
              60.651526`, 54.950758`, 63.745786`}, {5, 28, 
              19.82408422`, \(-0.8681955164`\), 5.140798299`, 0.16`, 
              1, \(-0.3663590167`\), \(-0.2`\), 
              23.45732957`, \(-0.950135866`\), \(-1.430433656`\), 2, 4, 5, 6, 
              0.495229`, \(-0.429853`\), 0.003956`, 0.200686`, 
              0.075726`, \(-0.072424`\), 0.899129`, 0.781728`, 0.953605`, 
              0.941767`, 1.289436`, 1.302528`, 18.040858`, 18.063866`, 
              0.937237`, 32.319114`, 32.294095`, \(-1.019122`\), 34.588765`, 
              34.60138`, 0.513844`, 52.097141`, 52.099011`, 0.076186`, 
              27.339061`, 27.350993`, 0.486041`, 55.716625`, 55.74819`, 
              1.285787`, 57.16601`, 57.159761`, \(-0.254538`\), 64.81783`, 
              64.794312`, \(-0.957989`\), 18.040858`, 32.319114`, 34.588765`, 
              56.819709`, 66.600556`, 80.210901`, 99.351626`, 99.999209`, 
              27.339061`, 55.716625`, 57.16601`, 64.81783`, 52.097141`, 
              73.074504`, 77.625266`, 79.565181`}, {5, 28, 
              27.05241317`, \(-0.8468375273`\), 6.257550227`, 0.16`, 
              1, \(-0.3884887624`\), \(-0.15`\), 
              11.89583812`, \(-1.557123504`\), \(-1.674154023`\), 2, 4, 5, 6, 
              0.510052`, \(-0.426058`\), 0.022946`, 0.163202`, 
              0.114534`, \(-0.097239`\), 0.896505`, 0.822128`, 0.912702`, 
              0.931262`, 1.220403`, 1.194487`, 19.319124`, 19.334709`, 
              0.79589`, 38.502993`, 38.481244`, \(-1.110655`\), 39.730054`, 
              39.748459`, 0.939891`, 62.516148`, 62.517245`, 0.056046`, 
              33.28093`, 33.292515`, 0.591599`, 68.131648`, 68.163215`, 
              1.612031`, 69.205896`, 69.198932`, \(-0.355616`\), 76.76149`, 
              76.739418`, \(-1.127156`\), 19.319124`, 38.502993`, 39.730054`, 
              64.536712`, 80.521129`, 97.005137`, 119.424435`, 119.994918`, 
              33.28093`, 68.131648`, 69.205896`, 76.76149`, 62.516148`, 
              87.100606`, 91.126433`, 96.201109`}, {2, 7, 
              85.50135279`, \(-0.7719204865`\), 7.20515208`, 0.16`, 
              1, \(-0.419544`\), \(-0.1`\), 50, \(-2.17698`\), \(-1.8572`\), 
              2, 4, 5, 6, 0.519585`, \(-0.435206`\), 0.06639`, 0.11549`, 
              0.149559`, \(-0.116389`\), 0.890742`, 0.726962`, 0.876875`, 
              0.795052`, 1.161405`, 0.966015`, 18.120023`, 18.126333`, 
              0.378019`, 41.655079`, 41.650665`, \(-0.264377`\), 48.459918`, 
              48.468376`, 0.506668`, 69.576091`, 69.576807`, 0.042885`, 
              38.088247`, 38.0984`, 0.608156`, 77.531091`, 77.563811`, 
              1.959899`, 78.791714`, 78.780259`, \(-0.686143`\), 84.519611`, 
              84.496972`, \(-1.356045`\), 18.120023`, 41.655079`, 48.459918`, 
              73.61947`, 90.92431`, 109.749726`, 131.299345`, 136.099676`, 
              38.088247`, 77.531091`, 78.791714`, 84.519611`, 69.576091`, 
              96.562311`, 98.470499`, 108.648755`}, {5, 28, 
              131.2128381`, \(-0.7651607752`\), 8.514382827`, 0.16`, 
              1, \(-0.4299252721`\), \(-0.05`\), 
              11.83064442`, \(-2.6785296`\), \(-2.293384944`\), 2, 4, 5, 6, 
              0.536087`, \(-0.422658`\), 0.054727`, 0.071763`, 
              0.183823`, \(-0.145968`\), 0.888069`, 0.607785`, 0.803905`, 
              0.6441`, 1.034936`, 0.739051`, 17.469271`, 17.471622`, 
              0.171656`, 49.712925`, 49.707621`, \(-0.387363`\), 53.02036`, 
              53.035289`, 1.090321`, 81.289478`, 81.296862`, 0.539217`, 
              44.981192`, 44.988878`, 0.561359`, 92.096189`, 92.121508`, 
              1.849042`, 92.782616`, 92.77442`, \(-0.598618`\), 97.639435`, 
              97.615505`, \(-1.747626`\), 17.469271`, 49.712925`, 53.02036`, 
              81.289478`, 107.175869`, 129.20765`, 152.806627`, 158.844708`, 
              44.981192`, 92.096189`, 92.782616`, 97.639435`, 81.56667`, 
              112.073975`, 113.537918`, 128.375444`}, {5, 28, 
              122.2241781`, \(-0.746479356`\), 8.325584527`, 0.16`, 
              1, \(-0.4429433425`\), 0, 
              1295.693106`, \(-2.178637684`\), \(-2.044093995`\), 2, 4, 5, 6, 
              0.550854`, \(-0.411366`\), 0.039364`, 0.118226`, 
              0.140424`, \(-0.104712`\), 0.884349`, 0.854022`, 0.839319`, 
              0.912784`, 1.099424`, 1.067`, 14.534041`, 14.534166`, 
              0.009124`, 48.290543`, 48.290352`, \(-0.014022`\), 55.856649`, 
              55.866802`, 0.744996`, 78.355587`, 78.356138`, 0.040408`, 
              43.851672`, 43.855066`, 0.249021`, 89.686583`, 89.700725`, 
              1.037724`, 90.467236`, 90.461923`, \(-0.389907`\), 93.136969`, 
              93.116699`, \(-1.487355`\), 14.534041`, 48.290543`, 55.856649`, 
              78.355587`, 104.221887`, 125.50029`, 146.755013`, 152.617912`, 
              43.851672`, 89.686583`, 90.467236`, 93.136969`, 78.746427`, 
              107.176061`, 107.810921`, 125.179793`}, {2, 18, 
              235.4344981`, \(-0.5560836347`\), \(-1.124036751`\), 0.16`, 
              1, \(-0.4805399879`\), 0.05`, 
              545.6146476`, \(-4.7003589`\), \(-6.882182797`\), 2, 4, 5, 6, 
              0.54908`, \(-0.453744`\), 0.133307`, \(-0.143337`\), 
              0.343901`, \(-0.334358`\), 
              0.869563`, \(-0.176919`\), \(-0.034062`\), \(-0.160692`\), \
\(-0.485291`\), \(-0.448622`\), 0.042282`, 0.042239`, 
              0.000424`, \(-5.833636`\), \(-5.833575`\), \(-0.000602`\), \
\(-7.258888`\), \(-7.26023`\), 0.013254`, \(-8.466085`\), \(-8.658293`\), 
              1.898049`, \(-5.77693`\), \(-5.776827`\), \(-0.001017`\), \
\(-11.118346`\), \(-11.1187`\), 
              0.003499`, \(-11.722843`\), \(-11.721409`\), \(-0.014158`\), \
\(-11.83441`\), \(-11.833372`\), \(-0.010249`\), 
              0.042282`, \(-5.833636`\), \(-7.258888`\), \(-8.466085`\), \
\(-13.328576`\), \(-16.123684`\), \(-18.018245`\), \(-19.156201`\), \
\(-5.77693`\), \(-11.118346`\), \(-11.722843`\), \(-11.83441`\), \(-9.681453`\
\), \(-13.239398`\), \(-12.370881`\), \(-16.036314`\)}, {\(-2\), 16, 
              823.8308971`, \(-0.5968308544`\), \(-1.944086226`\), 0.16`, 
              1, \(-0.4791996937`\), 0.1`, 24.25201413`, 3.51116501`, 
              3.739389261`, 2, 4, 5, 6, 0.567874`, \(-0.411684`\), 
              0.035679`, \(-0.511912`\), 0.656049`, \(-0.703856`\), 
              0.86825`, \(-0.574353`\), \(-1.08551`\), \(-1.070192`\), \
\(-2.526627`\), \(-2.146213`\), 0.359479`, 0.359115`, 
              0.00643`, \(-10.606146`\), \(-10.606116`\), \(-0.000535`\), \
\(-11.167669`\), \(-11.169269`\), 
              0.028267`, \(-14.640755`\), \(-14.641373`\), 
              0.01093`, \(-9.998476`\), \(-9.997311`\), \(-0.020562`\), \
\(-19.211971`\), \(-19.210091`\), \(-0.033203`\), \(-20.410618`\), \
\(-20.409297`\), \(-0.023335`\), \(-20.602687`\), \(-20.601625`\), \
\(-0.018751`\), 
              0.359479`, \(-10.606146`\), \(-11.167669`\), \(-14.640755`\), \
\(-23.290998`\), \(-28.047281`\), \(-31.550784`\), \(-33.020628`\), \
\(-9.998476`\), \(-19.211971`\), \(-20.410618`\), \(-20.602687`\), \
\(-16.955807`\), \(-22.86701`\), \(-21.803571`\), \(-28.173447`\)}, {\(-2\), 
              6, 169545.6966`, \(-0.3081753571`\), \(-4.126549126`\), 0.16`, 
              1, \(-0.527829`\), 0.15`, 50, \(-5.20283`\), \(-2.86962`\), 2, 
              4, 5, 6, 0.557016`, \(-0.5478`\), 0.283415`, \(-0.20073`\), 
              0.293511`, \(-0.306659`\), 0.842372`, \(-0.424267`\), 
              0.654838`, \(-0.580043`\), 0.797078`, \(-0.937478`\), 
              14.699505`, 14.699428`, 
              0.002825`, \(-17.318379`\), \(-17.321832`\), 
              0.126975`, \(-17.338334`\), \(-17.336428`\), \(-0.070109`\), \
\(-23.77044`\), \(-23.772818`\), 
              0.087452`, \(-20.259603`\), \(-20.25439`\), \(-0.191737`\), \
\(-31.639328`\), \(-31.633818`\), \(-0.202633`\), \(-39.068512`\), \
\(-39.062734`\), \(-0.212502`\), \(-42.149286`\), \(-42.152098`\), 0.103422`, 
              14.699505`, \(-17.318379`\), \(-17.338334`\), \(-23.77044`\), \
\(-44.438457`\), \(-53.737274`\), \(-56.149236`\), \(-58.573573`\), \
\(-20.259603`\), \(-31.639328`\), \(-39.068512`\), \(-42.149286`\), \
\(-29.228506`\), \(-40.163181`\), \(-29.124935`\), \(-53.680051`\)}, {5, 29, 
              32.34800167`, \(-0.3064113622`\), 3.84648947`, 0.2025`, 
              1, \(-0.4726225099`\), \(-0.35`\), 104.9623392`, 
              0.1117439387`, \(-1.392714498`\), 2, 4, 5, 6, 
              0.417948`, \(-0.987398`\), 0.928672`, 0.233897`, 
              0.021787`, \(-0.025593`\), 0.908804`, 0.59313`, 0.97574`, 
              0.839285`, 1.325453`, 1.323341`, 0.992453`, 1.033333`, 
              1.051524`, 17.545632`, 17.509642`, \(-0.925742`\), 25.869158`, 
              25.87574`, 0.169295`, 32.660536`, 32.665071`, 0.116646`, 
              20.151571`, 20.169182`, 0.453005`, 34.176087`, 34.207763`, 
              0.814761`, 42.766731`, 42.76571`, \(-0.026245`\), 44.386402`, 
              44.313031`, \(-1.887256`\), 0.992453`, 17.545632`, 25.869158`, 
              33.542025`, 44.952043`, 55.777`, 68.366113`, 69.958236`, 
              20.151571`, 34.176087`, 42.766731`, 44.386402`, 32.660536`, 
              49.95102`, 48.989776`, 49.457951`}, {5, 28, 
              30.2281566`, \(-0.616107674`\), 3.981627084`, 0.2025`, 
              1, \(-0.4389202446`\), \(-0.3`\), 
              104.7604253`, \(-0.01405086349`\), \(-1.261039128`\), 2, 4, 5, 
              6, 0.487879`, \(-0.525279`\), 0.098106`, 0.233507`, 
              0.023129`, \(-0.026429`\), 0.926299`, 0.663033`, 1.01548`, 
              0.889283`, 1.382421`, 1.3545`, 10.682924`, 10.714466`, 
              0.980339`, 24.555672`, 24.527745`, \(-0.868016`\), 28.088741`, 
              28.094948`, 0.192914`, 39.889173`, 39.892101`, 0.091015`, 
              21.79195`, 21.804828`, 0.40025`, 42.675219`, 42.715769`, 
              1.26033`, 44.589049`, 44.576093`, \(-0.40268`\), 50.83399`, 
              50.790451`, \(-1.353231`\), 10.682924`, 24.555672`, 28.088741`, 
              43.530083`, 51.887037`, 62.678797`, 76.997194`, 77.953148`, 
              21.79195`, 42.675219`, 44.589049`, 50.83399`, 39.889173`, 
              56.694659`, 60.636646`, 61.636009`}, {5, 29, 
              23.69461664`, \(-0.8715602621`\), 4.554468165`, 0.2025`, 
              1, \(-0.4078737549`\), \(-0.25`\), 
              4226.507505`, \(-0.298778779`\), \(-1.323783696`\), 2, 4, 5, 6, 
              0.538775`, \(-0.39417`\), \(-0.140885`\), 0.229633`, 
              0.030483`, \(-0.033324`\), 0.938405`, 0.788186`, 1.020855`, 
              0.981087`, 1.38492`, 1.4061`, 18.86024`, 18.886463`, 1.029555`, 
              32.084934`, 32.083377`, \(-0.061147`\), 33.363901`, 
              33.348436`, \(-0.607203`\), 50.128227`, 50.130254`, 0.079559`, 
              25.66261`, 25.673438`, 0.425094`, 52.792028`, 52.806974`, 
              0.586788`, 52.807214`, 52.818312`, 0.435722`, 61.777627`, 
              61.747156`, \(-1.196358`\), 18.86024`, 32.084934`, 33.363901`, 
              55.915494`, 63.335743`, 75.584014`, 92.887523`, 93.442865`, 
              25.66261`, 52.807214`, 52.792028`, 61.777627`, 50.128227`, 
              68.966816`, 75.838341`, 76.751981`}, {2, 25, 
              18.23453077`, \(-0.8026459193`\), 5.839400915`, 0.2025`, 
              1, \(-0.4419125974`\), \(-0.2`\), 
              6187.225296`, \(-1.482331698`\), \(-1.761084096`\), 2, 4, 5, 6, 
              0.54649`, \(-0.404532`\), \(-0.088373`\), 0.178095`, 
              0.09269`, \(-0.079591`\), 0.933469`, 0.843047`, 0.949692`, 
              0.96515`, 1.26632`, 1.257071`, 20.63777`, 20.657608`, 
              1.012922`, 38.937292`, 38.91008`, \(-1.389415`\), 41.955151`, 
              41.972241`, 0.872619`, 62.228443`, 62.230177`, 0.088501`, 
              32.731865`, 32.743759`, 0.607294`, 66.804863`, 66.819353`, 
              0.739844`, 67.173076`, 67.187372`, 0.729956`, 76.02931`, 
              76.002588`, \(-1.364401`\), 20.63777`, 38.937292`, 41.955151`, 
              68.700928`, 79.675456`, 95.31652`, 116.500403`, 117.890206`, 
              32.731865`, 67.173076`, 66.804863`, 76.02931`, 62.228443`, 
              85.483661`, 94.020049`, 94.211751`}, {2, 21, 
              18.48934052`, \(-0.7504087272`\), 7.776910096`, 0.2025`, 
              1, \(-0.468776396`\), \(-0.15`\), 
              55.73105106`, \(-2.163031761`\), \(-2.157358461`\), 2, 4, 5, 6, 
              0.556865`, \(-0.408861`\), \(-0.053158`\), 0.131108`, 
              0.128815`, \(-0.100641`\), 0.928923`, 0.872344`, 0.883498`, 
              0.941355`, 1.154016`, 1.119996`, 22.579497`, 22.59285`, 
              0.925212`, 49.566404`, 49.551321`, \(-1.045133`\), 54.749977`, 
              54.762902`, 0.895558`, 80.702339`, 80.703652`, 0.090967`, 
              43.390989`, 43.40381`, 0.888372`, 88.170729`, 88.185776`, 
              1.042656`, 88.837684`, 88.854784`, 1.184874`, 97.542239`, 
              97.512252`, \(-2.077836`\), 22.579497`, 49.566404`, 54.749977`, 
              86.787847`, 104.53555`, 125.237869`, 151.733094`, 155.096741`, 
              43.390989`, 88.837684`, 88.170729`, 97.542239`, 80.702339`, 
              110.425447`, 119.60679`, 123.155312`}, {2, 22, 
              29.58947543`, \(-0.7783830982`\), 9.972784496`, 0.2025`, 
              1, \(-0.4772337031`\), \(-0.1`\), 
              14.0523304`, \(-2.556573668`\), \(-2.312694203`\), 2, 4, 5, 6, 
              0.576881`, \(-0.385439`\), \(-0.083947`\), 0.100275`, 
              0.155465`, \(-0.124436`\), 0.928535`, 0.898578`, 0.858926`, 
              0.939706`, 1.111258`, 1.051853`, 26.840323`, 26.848434`, 
              0.746644`, 65.69332`, 65.687433`, \(-0.541857`\), 65.69867`, 
              65.710662`, 1.103803`, 104.098908`, 104.106123`, 0.664217`, 
              55.811113`, 55.822948`, 1.089381`, 114.320716`, 114.333783`, 
              1.202779`, 114.568824`, 114.588578`, 1.818349`, 123.860171`, 
              123.826627`, \(-3.087763`\), 26.840323`, 65.69332`, 65.69867`, 
              105.917911`, 134.903156`, 161.249569`, 193.884442`, 
              199.612473`, 55.811113`, 114.320716`, 114.568824`, 123.860171`, 
              104.098908`, 140.811964`, 152.665246`, 159.314068`}, {2, 7, 
              191.0910791`, \(-0.7726323668`\), 12.17660551`, 0.2025`, 
              1, \(-0.490051`\), \(-0.05`\), 50, \(-3.31835`\), \(-2.3394`\), 
              2, 4, 5, 6, 0.592502`, \(-0.37174`\), \(-0.098975`\), 
              0.034311`, 0.211594`, \(-0.181791`\), 0.92601`, 0.513792`, 
              0.851577`, 0.52569`, 1.099823`, 0.558972`, 30.091527`, 
              30.093853`, 0.268516`, 79.941112`, 79.932668`, \(-0.974783`\), 
              84.218148`, 84.238317`, 2.328253`, 126.183332`, 126.183599`, 
              0.030805`, 68.051823`, 68.061067`, 1.067061`, 139.195399`, 
              139.205931`, 1.215757`, 140.380245`, 140.40002`, 2.282711`, 
              148.204686`, 148.16779`, \(-4.259072`\), 30.091527`, 
              79.941112`, 84.218148`, 131.014491`, 164.342888`, 196.206945`, 
              233.530242`, 243.218327`, 68.051823`, 139.195399`, 140.380245`, 
              148.204686`, 126.183332`, 169.119022`, 182.674826`, 
              194.452334`}, {5, 29, 
              280.9015528`, \(-0.7650901215`\), \(-12.60667354`\), 0.2025`, 
              1, \(-0.5012168366`\), 0, 
              15.32399273`, \(-3.815772668`\), \(-4.902226378`\), 2, 4, 5, 6, 
              0.607369`, \(-0.356834`\), \(-0.122539`\), \(-0.01682`\), 
              0.256517`, \(-0.233761`\), 0.923072`, 0.067473`, 0.424684`, 
              0.062942`, 0.33709`, 
              0.049893`, \(-25.368745`\), \(-25.369085`\), 
              0.041575`, \(-82.720666`\), \(-82.716893`\), \(-0.462231`\), \
\(-83.538344`\), \(-83.555907`\), 
              2.15164`, \(-128.084969`\), \(-128.08925`\), 
              0.524463`, \(-70.282391`\), \(-70.286667`\), 
              0.523863`, \(-143.461346`\), \(-143.467687`\), 
              0.776829`, \(-145.377433`\), \(-145.389646`\), 
              1.496258`, \(-150.718051`\), \(-150.686722`\), \(-3.838202`\), \
\(-25.368745`\), \(-82.720666`\), \(-83.538344`\), \(-128.084969`\), \
\(-169.684088`\), \(-202.302109`\), \(-238.275049`\), \(-251.15228`\), \
\(-70.282391`\), \(-143.461346`\), \(-145.377433`\), \(-150.718051`\), \
\(-129.597785`\), \(-172.053732`\), \(-185.515912`\), \(-201.222881`\)}, \
{\(-2\), 17, 428.8210439`, \(-0.7490612441`\), \(-13.38209913`\), 0.2025`, 
              1, \(-0.5122516686`\), 0.05`, 
              394.8266407`, \(-4.2373894`\), \(-4.870095809`\), 2, 4, 5, 6, 
              0.620592`, \(-0.341754`\), \(-0.152557`\), \(-0.057052`\), 
              0.290442`, \(-0.275378`\), 0.919246`, \(-0.337126`\), 
              0.424822`, \(-0.353102`\), 
              0.339613`, \(-0.398825`\), \(-22.719876`\), \(-22.720604`\), 
              0.096668`, \(-84.403004`\), \(-84.446229`\), 
              5.743584`, \(-96.531367`\), \(-96.501888`\), \(-3.917084`\), \
\(-133.676068`\), \(-133.67715`\), 
              0.143797`, \(-74.274293`\), \(-74.273317`\), \(-0.129628`\), \
\(-151.092747`\), \(-151.095428`\), 
              0.356268`, \(-152.5963`\), \(-152.589837`\), \(-0.858762`\), \
\(-158.059558`\), \(-158.043976`\), \(-2.070552`\), \(-22.719876`\), \
\(-84.403004`\), \(-96.531367`\), \(-133.676068`\), \(-179.178172`\), \
\(-213.334274`\), \(-248.521401`\), \(-264.794814`\), \(-74.274293`\), \
\(-151.092747`\), \(-152.5963`\), \(-158.059558`\), \(-135.923368`\), \
\(-178.797507`\), \(-192.477153`\), \(-213.004267`\)}, {\(-2\), 21, 
              295.3971875`, \(-0.7338980269`\), \(-13.16063339`\), 0.2025`, 
              1, \(-0.5253236621`\), 0.1`, 
              91498.95888`, \(-4.109380459`\), \(-6.606574007`\), 2, 4, 5, 6, 
              0.633032`, \(-0.323827`\), \(-0.195275`\), \(-0.044476`\), 
              0.272029`, \(-0.253773`\), 0.915039`, \(-0.217913`\), 
              0.109665`, \(-0.23034`\), \(-0.238532`\), \(-0.265918`\), \
\(-16.542773`\), \(-16.545392`\), 
              0.349059`, \(-86.502893`\), \(-86.508031`\), 
              0.684872`, \(-91.113938`\), \(-91.121103`\), 
              0.955073`, \(-125.465464`\), \(-125.467554`\), 
              0.278536`, \(-72.644577`\), \(-72.638271`\), \(-0.840565`\), \
\(-146.990596`\), \(-146.989449`\), \(-0.15281`\), \(-147.127649`\), \
\(-147.112477`\), \(-2.022351`\), \(-155.00875`\), \(-155.001145`\), \
\(-1.01376`\), \(-16.542773`\), \(-86.502893`\), \(-91.113938`\), \
\(-125.465464`\), \(-175.240327`\), \(-208.308157`\), \(-240.021531`\), \
\(-257.264144`\), \(-72.644577`\), \(-146.990596`\), \(-147.127649`\), \
\(-155.00875`\), \(-132.000284`\), \(-171.995296`\), \(-185.259905`\), \
\(-208.892726`\)}, {\(-2\), 6, 
              911.1087333`, \(-0.7152195899`\), \(-0.8541764588`\), 0.2025`, 
              1, \(-0.53409`\), 0.15`, 50, \(-6.16279`\), \(-3.30804`\), 2, 
              4, 5, 6, 
              0.644106`, \(-0.303563`\), \(-0.249852`\), \(-0.286006`\), 
              0.502131`, \(-0.542579`\), 0.910116`, \(-0.485409`\), 
              0.675911`, \(-0.448585`\), 
              0.797416`, \(-1.048789`\), \(-0.628226`\), \(-0.628564`\), 
              0.002977`, \(-5.61666`\), \(-5.616481`\), \(-0.001578`\), \
\(-5.779482`\), \(-5.780424`\), 0.008289`, \(-7.690401`\), \(-7.929213`\), 
              2.102228`, \(-4.681085`\), \(-4.680333`\), \(-0.006622`\), \
\(-9.348852`\), \(-9.347815`\), \(-0.009124`\), \(-9.383628`\), \
\(-9.383296`\), \(-0.002926`\), \(-10.037304`\), \(-10.036867`\), \
\(-0.003843`\), \(-0.628226`\), \(-5.61666`\), \(-5.779482`\), \
\(-7.690401`\), \(-11.291816`\), \(-13.399221`\), \(-15.268311`\), \
\(-16.467772`\), \(-4.681085`\), \(-9.383628`\), \(-9.348852`\), \
\(-10.037304`\), \(-8.435687`\), \(-10.888502`\), \(-11.74609`\), \
\(-13.500185`\)}, {2, 19, 46.73352636`, \(-0.2962718483`\), 3.748567523`, 
              0.25`, 1, \(-0.5582540993`\), \(-0.35`\), 85.6977019`, 
              0.7433834825`, \(-1.118774187`\), 2, 4, 5, 6, 
              0.471767`, \(-0.974258`\), 0.845766`, 0.224676`, 
              0.030424`, \(-0.029787`\), 0.943869`, 0.597332`, 1.06079`, 
              0.828432`, 1.45199`, 1.288703`, 0.016076`, 0.04857`, 0.919418`, 
              17.587268`, 17.55838`, \(-0.817384`\), 26.146508`, 26.151817`, 
              0.1502`, 33.569747`, 33.57291`, 0.089487`, 20.785332`, 
              20.800126`, 0.418596`, 36.777556`, 36.832958`, 1.567623`, 
              41.781391`, 41.756122`, \(-0.715005`\), 44.425081`, 
              44.384461`, \(-1.149346`\), 0.016076`, 17.587268`, 26.146508`, 
              34.872063`, 46.205478`, 56.976383`, 68.589663`, 71.821903`, 
              20.785332`, 36.777556`, 41.781391`, 44.425081`, 33.569747`, 
              50.171982`, 50.392657`, 52.513257`}, {2, 24, 
              29.06467183`, \(-0.6072980291`\), 4.282963992`, 0.25`, 
              1, \(-0.5203624235`\), \(-0.3`\), 523.0113991`, 
              0.0611698188`, \(-1.47669566`\), 2, 4, 5, 6, 
              0.546894`, \(-0.480971`\), \(-0.038585`\), 0.2354`, 
              0.020193`, \(-0.023704`\), 0.963317`, 0.758303`, 1.032046`, 
              0.964183`, 1.392819`, 1.409471`, 12.113402`, 12.141273`, 
              1.044556`, 27.975876`, 27.960634`, \(-0.571216`\), 32.014144`, 
              32.010253`, \(-0.145808`\), 45.778305`, 45.780783`, 0.092898`, 
              24.821348`, 24.833241`, 0.44571`, 48.689316`, 48.704723`, 
              0.577431`, 50.195334`, 50.209125`, 0.516865`, 57.022094`, 
              56.986528`, \(-1.332942`\), 12.113402`, 27.975876`, 32.014144`, 
              50.056891`, 59.340739`, 71.109836`, 85.982061`, 87.80058`, 
              24.821348`, 50.195334`, 48.689316`, 57.022094`, 45.778305`, 
              63.424415`, 69.411624`, 71.614718`}, {5, 25, 
              19.93996049`, \(-0.5880958092`\), 5.162831172`, 0.25`, 
              1, \(-0.5398310786`\), \(-0.25`\), 
              481.3749348`, \(-1.103066645`\), \(-1.687098196`\), 2, 4, 5, 6, 
              0.560874`, \(-0.474429`\), \(-0.019548`\), 0.205077`, 
              0.054207`, \(-0.044743`\), 0.960629`, 0.835569`, 0.998382`, 
              0.988734`, 1.337356`, 1.343337`, 12.661903`, 12.683416`, 
              0.996706`, 32.936657`, 32.911188`, \(-1.180027`\), 37.845993`, 
              37.858648`, 0.58634`, 54.298023`, 54.30007`, 0.094858`, 
              29.879029`, 29.891335`, 0.57015`, 58.550479`, 58.567981`, 
              0.810886`, 60.349399`, 60.363059`, 0.632884`, 66.873204`, 
              66.841729`, \(-1.458279`\), 12.661903`, 32.936657`, 37.845993`, 
              58.676886`, 70.963741`, 85.0922`, 102.066022`, 105.422193`, 
              29.879029`, 60.349399`, 58.550479`, 66.873204`, 54.298023`, 
              74.873141`, 82.131744`, 84.036177`}, {2, 19, 
              21.20681857`, \(-0.6457148562`\), 6.039209809`, 0.25`, 
              1, \(-0.5446322948`\), \(-0.2`\), 
              54.80529799`, \(-1.555145978`\), \(-1.756129725`\), 2, 4, 5, 6, 
              0.586284`, \(-0.418845`\), \(-0.103868`\), 0.182163`, 
              0.074877`, \(-0.05742`\), 0.962465`, 0.904811`, 0.990525`, 
              1.02153`, 1.322954`, 1.307065`, 15.786677`, 15.802064`, 
              0.871661`, 37.931451`, 37.934989`, 0.200466`, 45.858855`, 
              45.84877`, \(-0.571323`\), 64.802101`, 64.803539`, 0.08148`, 
              35.209415`, 35.220845`, 0.647516`, 70.186152`, 70.208348`, 
              1.257436`, 71.152406`, 71.161517`, 0.516134`, 78.396786`, 
              78.367072`, \(-1.683329`\), 15.786677`, 37.931451`, 45.858855`, 
              69.385394`, 84.327143`, 100.716109`, 120.42161`, 124.373656`, 
              35.209415`, 71.152406`, 70.186152`, 78.396786`, 64.802101`, 
              88.049138`, 97.989357`, 99.280256`}, {5, 27, 
              20.52044463`, \(-0.704931286`\), 6.80178474`, 0.25`, 
              1, \(-0.5469797105`\), \(-0.15`\), 
              5.333906338`, \(-2.064949488`\), \(-2.162807803`\), 2, 4, 5, 6, 
              0.611112`, \(-0.373535`\), \(-0.174711`\), 0.15099`, 
              0.101807`, \(-0.077269`\), 0.964263`, 0.890118`, 0.932641`, 
              0.97581`, 1.222358`, 1.193265`, 15.643025`, 15.657945`, 
              0.992312`, 39.007708`, 39.022848`, 1.006884`, 46.540074`, 
              46.531792`, \(-0.550803`\), 62.479184`, 62.496111`, 1.125766`, 
              39.926425`, 39.936194`, 0.64971`, 80.577849`, 80.584393`, 
              0.435216`, 81.133469`, 81.156469`, 1.529614`, 88.616497`, 
              88.588404`, \(-1.868334`\), 15.643025`, 46.540074`, 39.007708`, 
              62.479184`, 96.492524`, 114.800236`, 136.81657`, 141.256668`, 
              39.926425`, 80.577849`, 81.133469`, 88.616497`, 74.48593`, 
              99.778595`, 112.53959`, 113.243354`}, {2, 7, 
              66.92996553`, \(-0.794519059`\), 10.69764948`, 0.25`, 
              1, \(-0.540532`\), \(-0.1`\), 50, \(-3.09544`\), \(-2.36195`\), 
              2, 4, 5, 6, 0.63902`, \(-0.328837`\), \(-0.252141`\), 
              0.074319`, 0.173206`, \(-0.147202`\), 0.967785`, 0.791534`, 
              0.907212`, 0.818828`, 1.17621`, 0.89455`, 34.34767`, 
              34.354198`, 0.713954`, 78.136401`, 78.127124`, \(-1.014688`\), 
              78.251167`, 78.265927`, 1.614397`, 121.019205`, 121.019847`, 
              0.0702`, 63.400238`, 63.410922`, 1.16859`, 127.668179`, 
              127.674595`, 0.701792`, 132.296256`, 132.326784`, 3.339003`, 
              141.554981`, 141.517947`, \(-4.050655`\), 34.34767`, 
              78.136401`, 78.251167`, 128.489596`, 155.401924`, 183.980367`, 
              218.166048`, 226.006902`, 63.400238`, 127.668179`, 132.296256`, 
              141.554981`, 121.019205`, 159.601476`, 182.07689`, 
              183.360954`}, {5, 27, 21.9641246`, \(-1.57252971`\), 
              10.42960249`, 0.25`, 1, \(-0.3647217199`\), \(-0.05`\), 
              25099.11073`, \(-2.938496661`\), \(-2.499685726`\), 2, 4, 5, 6, 
              0.737311`, \(-0.303338`\), \(-0.318472`\), 0.092876`, 
              0.203087`, \(-0.201366`\), 1.002943`, 1.012356`, 0.932789`, 
              1.044985`, 1.202006`, 1.136063`, 75.86595`, 75.87435`, 
              1.033537`, 88.939203`, 88.943791`, 0.564453`, 104.120621`, 
              104.117924`, \(-0.331795`\), 146.81083`, 146.817842`, 
              0.862679`, 65.705676`, 65.710489`, 0.592229`, 130.544237`, 
              130.547195`, 0.363958`, 156.808616`, 156.818728`, 1.244221`, 
              169.334166`, 169.318337`, \(-1.947588`\), 75.86595`, 
              88.939203`, 104.120621`, 146.81083`, 177.131075`, 205.445216`, 
              248.598131`, 248.691294`, 65.705676`, 130.544237`, 156.808616`, 
              169.334166`, 148.200538`, 189.343198`, 211.21016`, 
              222.074336`}, {5, 27, 44.53715022`, \(-1.904589321`\), 
              10.18212025`, 0.25`, 1, \(-0.3024700348`\), 0, 
              8200.616348`, \(-3.257322594`\), \(-3.700350555`\), 2, 4, 5, 6, 
              0.770672`, \(-0.325194`\), \(-0.266469`\), 0.072148`, 
              0.234366`, \(-0.238688`\), 1.010661`, 0.852574`, 0.760633`, 
              0.878309`, 0.896137`, 0.950009`, 77.873626`, 77.882339`, 
              1.093968`, 95.354158`, 95.355263`, 0.138717`, 109.196387`, 
              109.196144`, \(-0.030453`\), 144.594783`, 144.601098`, 
              0.792929`, 64.945227`, 64.947217`, 0.24986`, 128.631972`, 
              128.633112`, 0.143201`, 160.313194`, 160.316204`, 0.377937`, 
              173.053846`, 173.038991`, \(-1.86511`\), 77.873626`, 
              95.354158`, 109.196387`, 144.594783`, 179.548977`, 207.241403`, 
              250.137216`, 251.061128`, 64.945227`, 128.631972`, 160.313194`, 
              173.053846`, 152.475306`, 193.336819`, 213.510793`, 
              225.101017`}, {5, 25, 
              364.3499999`, \(-0.6054986657`\), \(-3.107350198`\), 0.25`, 
              1, \(-0.613045371`\), 0.05`, 
              3189.652155`, \(-6.688216454`\), \(-10.53688932`\), 2, 4, 5, 6, 
              0.660853`, \(-0.3296`\), \(-0.2273`\), \(-0.317376`\), 
              0.519599`, \(-0.582463`\), 
              0.947933`, \(-0.512691`\), \(-0.548194`\), \(-0.772245`\), \
\(-1.503333`\), \(-1.362274`\), \(-2.164968`\), \(-2.16508`\), 
              0.003695`, \(-20.490227`\), \(-20.491822`\), 
              0.052419`, \(-22.213275`\), \(-22.214551`\), 
              0.041921`, \(-30.086277`\), \(-30.086471`\), 
              0.006401`, \(-17.860594`\), \(-17.860364`\), \(-0.007556`\), \
\(-35.028541`\), \(-35.026907`\), \(-0.053683`\), \(-35.685371`\), \
\(-35.685769`\), 
              0.013068`, \(-37.664291`\), \(-37.661952`\), \(-0.076854`\), \
\(-2.164968`\), \(-20.490227`\), \(-22.213275`\), \(-30.086277`\), \
\(-42.638905`\), \(-50.608184`\), \(-57.857315`\), \(-62.795169`\), \
\(-17.860594`\), \(-35.685371`\), \(-35.028541`\), \(-37.664291`\), \
\(-31.770487`\), \(-41.358009`\), \(-45.880232`\), \(-50.038467`\)}, {5, 28, 
              616.6305741`, \(-0.6560414659`\), \(-5.634645397`\), 0.25`, 
              1, \(-0.6053026428`\), 0.1`, 
              1030.815886`, \(-5.480325496`\), \(-4.847462577`\), 2, 4, 5, 6, 
              0.680778`, \(-0.287836`\), \(-0.319594`\), \(-0.162768`\), 
              0.372417`, \(-0.393935`\), 0.948382`, \(-0.469043`\), 
              0.484693`, \(-0.561408`\), 
              0.433546`, \(-0.796575`\), \(-4.708193`\), \(-4.709128`\), 
              0.057405`, \(-38.886526`\), \(-38.88607`\), \(-0.028014`\), \
\(-40.390272`\), \(-40.395495`\), 
              0.320561`, \(-55.161469`\), \(-55.162631`\), 
              0.071366`, \(-32.48032`\), \(-32.477877`\), \(-0.149926`\), \
\(-63.929489`\), \(-63.924411`\), \(-0.311612`\), \(-64.873726`\), \
\(-64.873264`\), \(-0.028363`\), \(-69.634311`\), \(-69.631578`\), \
\(-0.167752`\), \(-4.708193`\), \(-38.886526`\), \(-40.390272`\), \
\(-55.161469`\), \(-78.404395`\), \(-92.670897`\), \(-105.177056`\), \
\(-114.961644`\), \(-32.48032`\), \(-64.873726`\), \(-63.929489`\), \
\(-69.634311`\), \(-58.695778`\), \(-75.296565`\), \(-85.046922`\), \
\(-92.322911`\)}, {5, 28, 581.9692231`, \(-0.750592352`\), \(-5.367825767`\), 
              0.25`, 1, \(-0.5912057154`\), 0.15`, 
              960.1292144`, \(-5.800098769`\), \(-6.014200888`\), 2, 4, 5, 6, 
              0.70437`, \(-0.242935`\), \(-0.423192`\), \(-0.201743`\), 
              0.426645`, \(-0.46073`\), 0.951791`, \(-0.584485`\), 
              0.289982`, \(-0.702203`\), 
              0.076846`, \(-0.999579`\), \(-7.3871`\), \(-7.389148`\), 
              0.123891`, \(-38.838145`\), \(-38.843085`\), 
              0.298788`, \(-39.647795`\), \(-39.646777`\), \(-0.061568`\), \
\(-55.218128`\), \(-55.220632`\), 
              0.151458`, \(-31.203334`\), \(-31.199327`\), \(-0.2424`\), \
\(-62.423416`\), \(-62.42205`\), \(-0.08262`\), \(-62.488835`\), \
\(-62.482685`\), \(-0.372016`\), \(-68.524918`\), \(-68.522671`\), \
\(-0.135931`\), \(-7.3871`\), \(-39.647795`\), \(-38.838145`\), \(-55.218128`\
\), \(-76.630953`\), \(-90.10594`\), \(-101.866775`\), \(-111.571565`\), \
\(-31.203334`\), \(-62.423416`\), \(-62.488835`\), \(-68.524918`\), \
\(-58.078922`\), \(-73.325899`\), \(-84.965361`\), \(-90.462195`\)}}};\)\)], \
"Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Contour plot", "Subsection"],

Cell[BoxData[
    \(\(Length[
        data = Map[{Sqrt[#[\([6]\)]], #[\([9]\)], #[\([3]\)]} &, 
            datal2[\([2]\)]]];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[
      d2 = Map[Sort[#, Function[{a, b}, OrderedQ[{Rest[a], Rest[b]}]]] &, 
          Partition[Sort[data], 12]]]\)], "Input"],

Cell[BoxData[
    \({11, 12, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ColumnForm[d2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\({{0, \(-0.35`\), 141.627678`}, {0, \(-0.3`\), 
                277.0667036`}, {0, \(-0.25`\), 199.0188878`}, {0, \(-0.2`\), 
                7.88096895`}, {0, \(-0.15`\), 
                8.65228068`}, {0, \(-0.114567563`\), 
                6.369872612`}, {0, \(-0.1`\), 6.36215481`}, {0, \(-0.05`\), 
                13.76231161`}, {0, 0, 137.1436763`}, {0, 0.05`, 
                107233.2632`}, {0, 0.1`, 898247.4609`}, {0, 0.15`, 
                129.6187294`}}\)},
          {\({{0.05`, \(-0.35`\), 13.43585703`}, {0.05`, \(-0.3`\), 
                15.04115415`}, {0.05`, \(-0.25`\), 
                10.82764979`}, {0.05`, \(-0.2`\), 
                9.345748316`}, {0.05`, \(-0.15`\), 
                8.00599147`}, {0.05`, \(-0.1081262544`\), 
                7.349227765`}, {0.05`, \(-0.1`\), 
                7.385711701`}, {0.05`, \(-0.05`\), 21.23146772`}, {0.05`, 0, 
                181.5478697`}, {0.05`, 0.05`, 311606.9456`}, {0.05`, 0.1`, 
                1.482468284`*^6}, {0.05`, 0.15`, 344.2900439`}}\)},
          {\({{0.1`, \(-0.35`\), 14.40401407`}, {0.1`, \(-0.3`\), 
                15.18244943`}, {0.1`, \(-0.25`\), 
                12.93354136`}, {0.1`, \(-0.2`\), 
                11.05187693`}, {0.1`, \(-0.15`\), 
                9.008659793`}, {0.1`, \(-0.1061584742`\), 
                8.135087732`}, {0.1`, \(-0.1`\), 
                8.172929207`}, {0.1`, \(-0.05`\), 95.23262854`}, {0.1`, 0, 
                131.5299108`}, {0.1`, 0.05`, 493852.3303`}, {0.1`, 0.1`, 
                1.81117411`*^6}, {0.1`, 0.15`, 4.394548546`*^6}}\)},
          {\({{0.15`, \(-0.35`\), 17.49123999`}, {0.15`, \(-0.3`\), 
                17.14287462`}, {0.15`, \(-0.25`\), 
                15.0605637`}, {0.15`, \(-0.2`\), 
                13.01551971`}, {0.15`, \(-0.15`\), 
                10.83108408`}, {0.15`, \(-0.1`\), 
                8.429733098`}, {0.15`, \(-0.09585391155`\), 
                8.339396601`}, {0.15`, \(-0.05`\), 13.88767786`}, {0.15`, 0, 
                80794.43708`}, {0.15`, 0.05`, 101.7893908`}, {0.15`, 0.1`, 
                1.819685403`*^6}, {0.15`, 0.15`, 4961.920201`}}\)},
          {\({{0.2`, \(-0.35`\), 21.3915725`}, {0.2`, \(-0.3`\), 
                19.71305313`}, {0.2`, \(-0.25`\), 
                17.46583367`}, {0.2`, \(-0.2`\), 
                13.42216434`}, {0.2`, \(-0.15`\), 
                9.644386415`}, {0.2`, \(-0.1186141863`\), 
                8.133791511`}, {0.2`, \(-0.1`\), 
                17.08376039`}, {0.2`, \(-0.05`\), 45.79062963`}, {0.2`, 0, 
                96377.14665`}, {0.2`, 0.05`, 535587.7757`}, {0.2`, 0.1`, 
                158.0702871`}, {0.2`, 0.15`, 3.392014334`*^6}}\)},
          {\({{0.25`, \(-0.35`\), 25.54751593`}, {0.25`, \(-0.3`\), 
                22.83356449`}, {0.25`, \(-0.25`\), 
                14.39445119`}, {0.25`, \(-0.2`\), 
                9.777954177`}, {0.25`, \(-0.15`\), 
                8.370217952`}, {0.25`, \(-0.1467998049`\), 
                8.325858404`}, {0.25`, \(-0.1`\), 
                20.64756111`}, {0.25`, \(-0.05`\), 88.52067179`}, {0.25`, 0, 
                84060.60851`}, {0.25`, 0.05`, 412838.9561`}, {0.25`, 0.1`, 
                295.2454032`}, {0.25`, 0.15`, 2.421483677`*^6}}\)},
          {\({{0.3`, \(-0.35`\), 24.26563901`}, {0.3`, \(-0.3`\), 
                21.81026268`}, {0.3`, \(-0.25`\), 
                12.35629126`}, {0.3`, \(-0.2`\), 
                9.732090708`}, {0.3`, \(-0.1748147515`\), 
                10.42272203`}, {0.3`, \(-0.15`\), 
                11.55752344`}, {0.3`, \(-0.1`\), 
                17.07386773`}, {0.3`, \(-0.05`\), 82.512688`}, {0.3`, 0, 
                201.6289145`}, {0.3`, 0.05`, 255465.9259`}, {0.3`, 0.1`, 
                693570.1306`}, {0.3`, 0.15`, 1.456994717`*^6}}\)},
          {\({{0.35`, \(-0.35`\), 29.82555823`}, {0.35`, \(-0.3`\), 
                26.43922532`}, {0.35`, \(-0.25`\), 
                20.34176284`}, {0.35`, \(-0.2015905316`\), 
                8.52311789`}, {0.35`, \(-0.2`\), 
                8.609306924`}, {0.35`, \(-0.15`\), 
                14.2861495`}, {0.35`, \(-0.1`\), 
                31.36822567`}, {0.35`, \(-0.05`\), 101.4310464`}, {0.35`, 0, 
                156.0417465`}, {0.35`, 0.05`, 108707.2851`}, {0.35`, 0.1`, 
                319357.2343`}, {0.35`, 0.15`, 682178.1949`}}\)},
          {\({{0.4`, \(-0.35`\), 35.11282161`}, {0.4`, \(-0.3`\), 
                30.6263805`}, {0.4`, \(-0.25`\), 
                14.36967753`}, {0.4`, \(-0.2230772747`\), 
                10.94888151`}, {0.4`, \(-0.2`\), 
                19.82408422`}, {0.4`, \(-0.15`\), 
                27.05241317`}, {0.4`, \(-0.1`\), 
                85.50135279`}, {0.4`, \(-0.05`\), 131.2128381`}, {0.4`, 0, 
                122.2241781`}, {0.4`, 0.05`, 235.4344981`}, {0.4`, 0.1`, 
                823.8308971`}, {0.4`, 0.15`, 169545.6966`}}\)},
          {\({{0.45`, \(-0.35`\), 32.34800167`}, {0.45`, \(-0.3`\), 
                30.2281566`}, {0.45`, \(-0.25`\), 
                23.69461664`}, {0.45`, \(-0.2`\), 
                18.23453077`}, {0.45`, \(-0.1731850841`\), 
                16.64275753`}, {0.45`, \(-0.15`\), 
                18.48934052`}, {0.45`, \(-0.1`\), 
                29.58947543`}, {0.45`, \(-0.05`\), 191.0910791`}, {0.45`, 0, 
                280.9015528`}, {0.45`, 0.05`, 428.8210439`}, {0.45`, 0.1`, 
                295.3971875`}, {0.45`, 0.15`, 911.1087333`}}\)},
          {\({{0.5`, \(-0.35`\), 46.73352636`}, {0.5`, \(-0.307886`\), 
                43.4135314`}, {0.5`, \(-0.3`\), 
                29.06467183`}, {0.5`, \(-0.25`\), 
                19.93996049`}, {0.5`, \(-0.2`\), 
                21.20681857`}, {0.5`, \(-0.15`\), 
                20.52044463`}, {0.5`, \(-0.1`\), 
                66.92996553`}, {0.5`, \(-0.05`\), 21.9641246`}, {0.5`, 0, 
                44.53715022`}, {0.5`, 0.05`, 364.3499999`}, {0.5`, 0.1`, 
                616.6305741`}, {0.5`, 0.15`, 581.9692231`}}\)}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      ColumnForm[ {{{0, -0.34999999999999998, 141.627678}, {
        0, -0.29999999999999999, 277.06670359999998}, {0, -0.25, 
        199.01888779999999}, {0, -0.20000000000000001, 7.8809689499999998}, {
        0, -0.14999999999999999, 8.6522806800000005}, {0, -0.114567563, 
        6.369872612}, {0, -0.10000000000000001, 6.3621548099999998}, {
        0, -0.050000000000000003, 13.762311609999999}, {0, 0, 
        137.14367630000001}, {0, 0.050000000000000003, 107233.2632}, {0, 
        0.10000000000000001, 898247.46089999995}, {0, 0.14999999999999999, 
        129.61872940000001}}, {{0.050000000000000003, -0.34999999999999998, 
        13.435857029999999}, {0.050000000000000003, -0.29999999999999999, 
        15.041154150000001}, {0.050000000000000003, -0.25, 
        10.827649790000001}, {0.050000000000000003, -0.20000000000000001, 
        9.3457483159999999}, {0.050000000000000003, -0.14999999999999999, 
        8.0059914699999997}, {0.050000000000000003, -0.1081262544, 
        7.3492277650000002}, {0.050000000000000003, -0.10000000000000001, 
        7.385711701}, {0.050000000000000003, -0.050000000000000003, 
        21.231467720000001}, {0.050000000000000003, 0, 181.54786970000001}, {
        0.050000000000000003, 0.050000000000000003, 311606.94559999998}, {
        0.050000000000000003, 0.10000000000000001, 1482468.284}, {
        0.050000000000000003, 0.14999999999999999, 344.2900439}}, {{
        0.10000000000000001, -0.34999999999999998, 14.404014070000001}, {
        0.10000000000000001, -0.29999999999999999, 15.18244943}, {
        0.10000000000000001, -0.25, 12.93354136}, {
        0.10000000000000001, -0.20000000000000001, 11.051876930000001}, {
        0.10000000000000001, -0.14999999999999999, 9.0086597929999996}, {
        0.10000000000000001, -0.10615847420000001, 8.1350877320000006}, {
        0.10000000000000001, -0.10000000000000001, 8.1729292069999993}, {
        0.10000000000000001, -0.050000000000000003, 95.232628539999993}, {
        0.10000000000000001, 0, 131.52991080000001}, {0.10000000000000001, 
        0.050000000000000003, 493852.33029999997}, {0.10000000000000001, 
        0.10000000000000001, 1811174.1100000001}, {0.10000000000000001, 
        0.14999999999999999, 4394548.5460000001}}, {{
        0.14999999999999999, -0.34999999999999998, 17.49123999}, {
        0.14999999999999999, -0.29999999999999999, 17.142874620000001}, {
        0.14999999999999999, -0.25, 15.060563699999999}, {
        0.14999999999999999, -0.20000000000000001, 13.01551971}, {
        0.14999999999999999, -0.14999999999999999, 10.83108408}, {
        0.14999999999999999, -0.10000000000000001, 8.4297330979999998}, {
        0.14999999999999999, -0.095853911550000004, 8.3393966010000007}, {
        0.14999999999999999, -0.050000000000000003, 13.88767786}, {
        0.14999999999999999, 0, 80794.437080000003}, {0.14999999999999999, 
        0.050000000000000003, 101.78939080000001}, {0.14999999999999999, 
        0.10000000000000001, 1819685.4029999999}, {0.14999999999999999, 
        0.14999999999999999, 4961.9202009999999}}, {{
        0.20000000000000001, -0.34999999999999998, 21.391572499999999}, {
        0.20000000000000001, -0.29999999999999999, 19.713053129999999}, {
        0.20000000000000001, -0.25, 17.465833669999999}, {
        0.20000000000000001, -0.20000000000000001, 13.42216434}, {
        0.20000000000000001, -0.14999999999999999, 9.6443864149999996}, {
        0.20000000000000001, -0.11861418629999999, 8.1337915110000001}, {
        0.20000000000000001, -0.10000000000000001, 17.083760389999998}, {
        0.20000000000000001, -0.050000000000000003, 45.790629629999998}, {
        0.20000000000000001, 0, 96377.146649999995}, {0.20000000000000001, 
        0.050000000000000003, 535587.7757}, {0.20000000000000001, 
        0.10000000000000001, 158.0702871}, {0.20000000000000001, 
        0.14999999999999999, 3392014.3339999998}}, {{
        0.25, -0.34999999999999998, 25.547515929999999}, {
        0.25, -0.29999999999999999, 22.833564490000001}, {0.25, -0.25, 
        14.39445119}, {0.25, -0.20000000000000001, 9.7779541769999998}, {
        0.25, -0.14999999999999999, 8.3702179520000008}, {0.25, -0.1467998049,
         8.3258584039999999}, {0.25, -0.10000000000000001, 
        20.647561110000002}, {0.25, -0.050000000000000003, 
        88.520671789999994}, {0.25, 0, 84060.608510000005}, {0.25, 
        0.050000000000000003, 412838.95610000001}, {0.25, 0.10000000000000001,
         295.2454032}, {0.25, 0.14999999999999999, 2421483.6770000001}}, {{
        0.29999999999999999, -0.34999999999999998, 24.265639010000001}, {
        0.29999999999999999, -0.29999999999999999, 21.810262680000001}, {
        0.29999999999999999, -0.25, 12.356291260000001}, {
        0.29999999999999999, -0.20000000000000001, 9.7320907079999994}, {
        0.29999999999999999, -0.1748147515, 10.422722029999999}, {
        0.29999999999999999, -0.14999999999999999, 11.557523440000001}, {
        0.29999999999999999, -0.10000000000000001, 17.07386773}, {
        0.29999999999999999, -0.050000000000000003, 82.512687999999997}, {
        0.29999999999999999, 0, 201.62891450000001}, {0.29999999999999999, 
        0.050000000000000003, 255465.9259}, {0.29999999999999999, 
        0.10000000000000001, 693570.13060000003}, {0.29999999999999999, 
        0.14999999999999999, 1456994.7169999999}}, {{
        0.34999999999999998, -0.34999999999999998, 29.825558229999999}, {
        0.34999999999999998, -0.29999999999999999, 26.439225319999998}, {
        0.34999999999999998, -0.25, 20.341762840000001}, {
        0.34999999999999998, -0.20159053160000001, 8.52311789}, {
        0.34999999999999998, -0.20000000000000001, 8.6093069240000002}, {
        0.34999999999999998, -0.14999999999999999, 14.286149500000001}, {
        0.34999999999999998, -0.10000000000000001, 31.368225670000001}, {
        0.34999999999999998, -0.050000000000000003, 101.4310464}, {
        0.34999999999999998, 0, 156.04174649999999}, {0.34999999999999998, 
        0.050000000000000003, 108707.28509999999}, {0.34999999999999998, 
        0.10000000000000001, 319357.23430000001}, {0.34999999999999998, 
        0.14999999999999999, 682178.1949}}, {{
        0.40000000000000002, -0.34999999999999998, 35.112821609999997}, {
        0.40000000000000002, -0.29999999999999999, 30.6263805}, {
        0.40000000000000002, -0.25, 14.369677530000001}, {
        0.40000000000000002, -0.22307727469999999, 10.94888151}, {
        0.40000000000000002, -0.20000000000000001, 19.82408422}, {
        0.40000000000000002, -0.14999999999999999, 27.052413170000001}, {
        0.40000000000000002, -0.10000000000000001, 85.501352789999999}, {
        0.40000000000000002, -0.050000000000000003, 131.2128381}, {
        0.40000000000000002, 0, 122.2241781}, {0.40000000000000002, 
        0.050000000000000003, 235.43449810000001}, {0.40000000000000002, 
        0.10000000000000001, 823.83089710000002}, {0.40000000000000002, 
        0.14999999999999999, 169545.6966}}, {{
        0.45000000000000001, -0.34999999999999998, 32.348001670000002}, {
        0.45000000000000001, -0.29999999999999999, 30.228156599999998}, {
        0.45000000000000001, -0.25, 23.69461664}, {
        0.45000000000000001, -0.20000000000000001, 18.234530769999999}, {
        0.45000000000000001, -0.1731850841, 16.642757530000001}, {
        0.45000000000000001, -0.14999999999999999, 18.489340519999999}, {
        0.45000000000000001, -0.10000000000000001, 29.58947543}, {
        0.45000000000000001, -0.050000000000000003, 191.0910791}, {
        0.45000000000000001, 0, 280.90155279999999}, {0.45000000000000001, 
        0.050000000000000003, 428.82104390000001}, {0.45000000000000001, 
        0.10000000000000001, 295.39718749999997}, {0.45000000000000001, 
        0.14999999999999999, 911.10873330000004}}, {{
        0.5, -0.34999999999999998, 46.733526359999999}, {
        0.5, -0.30788599999999999, 43.413531399999997}, {
        0.5, -0.29999999999999999, 29.064671830000002}, {0.5, -0.25, 
        19.939960490000001}, {0.5, -0.20000000000000001, 
        21.206818569999999}, {0.5, -0.14999999999999999, 20.52044463}, {
        0.5, -0.10000000000000001, 66.929965530000004}, {
        0.5, -0.050000000000000003, 21.964124600000002}, {0.5, 0, 
        44.537150220000001}, {0.5, 0.050000000000000003, 364.3499999}, {0.5, 
        0.10000000000000001, 616.63057409999999}, {0.5, 0.14999999999999999, 
        581.96922310000002}}}],
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(6*72\)], "Input"],

Cell[BoxData[
    \(432\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppb2 = 
      ListSurfacePlot3D[Transpose[d2], 
        PlotRange \[Rule] {{0, 0.5}, {\(- .35\), 0}, {0, 30}}, 
        BoxRatios \[Rule] {1, 1, 0.65}, 
        ViewPoint \[Rule] {\(-1.45\), \(-0.26\), 1.75}, Axes \[Rule] True, 
        AxesLabel \[Rule] {"\<m\>", \*"\"\<\!\(l\_2\)\>\"", \*"\"\<\!\(\[Chi]\
\^2\)\>\""}, 
        TextStyle -> {FontSize \[Rule] 
              12}, \[IndentingNewLine]LightSources \[Rule] {{{0, 1, 1}, 
              Hue[0.1, 0.8, 1]}, {{0, \(-1\), 1}, 
              Hue[0.6, 0.8, 1]}} (*\(,\)\(AmbientLight \[Rule] 
            GrayLevel[0.05]\)*) ]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bvalley2.jpg\>", pppb2, "\<JPEG\>", 
      ImageSize \[Rule] 72*4]\)], "Input"],

Cell[BoxData[
    \("bvalley2.jpg"\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pppb23 = 
      Block[{theta = Pi/2}, 
        Show[pppb2, 
          TextStyle -> {FontFamily -> "\<Courier\>", FontSize \[Rule] 10}, 
          AxesLabel \[Rule] None, ColorOutput \[Rule] GrayLevel, 
          FormatType -> OutputForm, 
          LightSources \[Rule] {{{Cos[theta], Sin[theta], 2}, 
                GrayLevel[
                  0.75]} (*\(,\)\({{\(-Cos[theta]\), \(-Sin[theta]\), 1}, 
                  GrayLevel[0.5]}\)*) }, 
          AmbientLight \[Rule] 
            GrayLevel[
              0.25] (*\(,\)\(FaceGrids \[Rule] {{0, \(-1\), 0}, {0, 1, 
                  0}, {0, 0, \(-1\)}, {1, 0, 0}}\)*) ]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Export["\<bvalley2.eps\>", pppb23, "\<EPS\>", 
      ImageSize \[Rule] 72*\((3 + 3/8)\)*1.0]\)], "Input"],

Cell[BoxData[
    \("bvalley2.eps"\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Analyze", "Subsection"],

Cell["Chi^2", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(chi2 = 
      ListPlot[Map[{Sqrt[#[\([6]\)]], #[\([3]\)]} &, datal2[\([2]\)]], 
        PlotJoined \[Rule] False, Axes -> False, 
        PlotStyle \[Rule] {PointSize[0.015]}, Frame -> True, 
        TextStyle -> {FontFamily -> "\<Courier\>", FontSize -> 12}, 
        FrameLabel -> {"\<m\>", "\<\>"}, PlotRange -> {0, 40}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell["\<\
Select which points to plot (gooddata) and which points to use for \
fitting (gooddata).\
\>", "Text"],

Cell[BoxData[
    \(\(gooddatal2 = 
        Select[datal2[\([2]\)], \((#[\([3]\)] < 40)\) &];\)\)], "Input"],

Cell["#[[8]] is for lambda_1 and #[[9]] is for lambda_2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[{#[\([6]\)], #[\([9]\)]} &, gooddatal2]\)], "Input"],

Cell[BoxData[
    \({{0, \(-0.114567563`\)}, {0.0025`, \(-0.1081262544`\)}, {0.01`, \
\(-0.1061584742`\)}, {0.0225`, \(-0.09585391155`\)}, {0.04`, \(-0.1186141863`\
\)}, {0.0625`, \(-0.1467998049`\)}, {0.09`, \(-0.1748147515`\)}, {0.1225`, \
\(-0.2015905316`\)}, {0.16`, \(-0.2230772747`\)}, {0.2025`, \
\(-0.1731850841`\)}, {0, \(-0.2`\)}, {0, \(-0.15`\)}, {0, \(-0.1`\)}, {0, \
\(-0.05`\)}, {0.0025`, \(-0.35`\)}, {0.0025`, \(-0.3`\)}, {0.0025`, \
\(-0.25`\)}, {0.0025`, \(-0.2`\)}, {0.0025`, \(-0.15`\)}, {0.0025`, \
\(-0.1`\)}, {0.0025`, \(-0.05`\)}, {0.01`, \(-0.35`\)}, {0.01`, \(-0.3`\)}, \
{0.01`, \(-0.25`\)}, {0.01`, \(-0.2`\)}, {0.01`, \(-0.15`\)}, {0.01`, \(-0.1`\
\)}, {0.0225`, \(-0.35`\)}, {0.0225`, \(-0.3`\)}, {0.0225`, \(-0.25`\)}, \
{0.0225`, \(-0.2`\)}, {0.0225`, \(-0.15`\)}, {0.0225`, \(-0.1`\)}, {0.0225`, \
\(-0.05`\)}, {0.04`, \(-0.35`\)}, {0.04`, \(-0.3`\)}, {0.04`, \(-0.25`\)}, \
{0.04`, \(-0.2`\)}, {0.04`, \(-0.15`\)}, {0.04`, \(-0.1`\)}, {0.0625`, \
\(-0.35`\)}, {0.0625`, \(-0.3`\)}, {0.0625`, \(-0.25`\)}, {0.0625`, \
\(-0.2`\)}, {0.0625`, \(-0.15`\)}, {0.0625`, \(-0.1`\)}, {0.09`, \(-0.35`\)}, \
{0.09`, \(-0.3`\)}, {0.09`, \(-0.25`\)}, {0.09`, \(-0.2`\)}, {0.09`, \(-0.15`\
\)}, {0.09`, \(-0.1`\)}, {0.1225`, \(-0.35`\)}, {0.1225`, \(-0.3`\)}, \
{0.1225`, \(-0.25`\)}, {0.1225`, \(-0.2`\)}, {0.1225`, \(-0.15`\)}, {0.1225`, \
\(-0.1`\)}, {0.16`, \(-0.35`\)}, {0.16`, \(-0.3`\)}, {0.16`, \(-0.25`\)}, \
{0.16`, \(-0.2`\)}, {0.16`, \(-0.15`\)}, {0.2025`, \(-0.35`\)}, {0.2025`, \
\(-0.3`\)}, {0.2025`, \(-0.25`\)}, {0.2025`, \(-0.2`\)}, {0.2025`, \
\(-0.15`\)}, {0.2025`, \(-0.1`\)}, {0.25`, \(-0.3`\)}, {0.25`, \(-0.25`\)}, \
{0.25`, \(-0.2`\)}, {0.25`, \(-0.15`\)}, {0.25`, \(-0.05`\)}}\)], "Output"]
}, Open  ]],

Cell["\<\
Using gooddata, fit all couplings to either a linear or constant \
function.  Here, x is the mass, y is the other coupling.
#[[8]] is for lambda_1 and #[[9]] is for lambda_2.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefittingl2 = 
      Chop[Table[
          Fit[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([9]\)]/#[\([7]\)], \
#[\([i]\)]/#[\([7]\)]} &, gooddatal2], 
            If[MemberQ[{10}, i], {1}, {1, x, x^2, y, x\ y, \ x^2\ y, y^2, 
                x\ y^2, x^2\ y^2}], {x, y}], {i, 6, 12}]]\)], "Input"],

Cell[BoxData[
    \({0.9999999999999998`\ x\^2, 
      1.0000000000000004`, \(-0.034398426747347895`\) - 
        0.2733652058134204`\ x - 0.8013495684885832`\ x\^2 + 
        0.3147110791550564`\ y + 4.650391547396235`\ x\ y - 
        5.715533018537675`\ x\^2\ y + 5.658063548227542`\ y\^2 - 
        8.172888252857462`\ x\ y\^2 + 2.922505020489585`\ x\^2\ y\^2, 
      0.9999999999999992`\ y, 
      2616.5144899739325`, \(-1.287639791676958`\) - 5.653047319700223`\ x + 
        3.0233974747409587`\ x\^2 - 8.256972136298572`\ y - 
        35.549562158728016`\ x\ y + 85.4860505473433`\ x\^2\ y - 
        23.690352803629096`\ y\^2 + 14.223087472101428`\ x\ y\^2 + 
        143.42784710748194`\ x\^2\ y\^2, \(-1.0499368117033658`\) - 
        3.1179558283495825`\ x - 1.4754867067084718`\ x\^2 - 
        3.8733111156611044`\ y - 15.907265082193236`\ x\ y + 
        20.59696700997102`\ x\^2\ y - 6.987938476757359`\ y\^2 - 
        29.00536576864658`\ x\ y\^2 + 
        58.2013320501871`\ x\^2\ y\^2}\)], "Output"]
}, Open  ]],

Cell["Use 1/chi^2 as the weighting.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(couplefittingl2 = 
      Chop[Table[
          BestFit /. 
            Regress[Map[{Sqrt[#[\([6]\)]/#[\([7]\)]], #[\([9]\)]/#[\([7]\)], \
#[\([i]\)]/#[\([7]\)]} &, gooddatal2], 
              If[MemberQ[{10}, i], {1}, {1, x, x^2, y, x\ y, \ x^2\ y, y^2, 
                  x\ y^2, x^2\ y^2}], {x, y}, 
              Weights \[Rule] Map[\((1/#[\([3]\)])\) &, gooddatal2], 
              RegressionReport \[Rule] BestFit], {i, 6, 12}]]\)], "Input"],

Cell[BoxData[
    \(DesignedRegress::"badfit" \(\(:\)\(\ \)\) 
      "Warning: unable to find a fit that is better than the mean \
response."\)], "Message"],

Cell[BoxData[
    \({1.0000000000000009`\ x\^2, 
      1.000000000000001`, \(-0.07315544469175292`\) + 
        0.3447424943304787`\ x - 1.8679240098635377`\ x\^2 - 
        0.18561180367319982`\ y + 12.035908544600613`\ x\ y - 
        18.648901455214144`\ x\^2\ y + 4.387254370832463`\ y\^2 + 
        10.043571881228338`\ x\ y\^2 - 29.47256161884147`\ x\^2\ y\^2, 
      0.9999999999999993`\ y, 
      3129.829822433635`, \(-1.2792513622677677`\) - 5.481608723521813`\ x + 
        3.2371829320852`\ x\^2 - 8.059063318438842`\ y - 
        33.52157081945379`\ x\ y + 89.81938666699536`\ x\^2\ y - 
        23.28716834581781`\ y\^2 + 19.29918324995366`\ x\ y\^2 + 
        154.95951882910308`\ x\^2\ y\^2, \(-1.0402225722211398`\) - 
        3.2612626456514757`\ x - 0.8566497884134436`\ x\^2 - 
        3.905597661220756`\ y - 15.207988131986943`\ x\ y + 
        24.53490994677505`\ x\^2\ y - 7.584841330379188`\ y\^2 - 
        21.745172635612747`\ x\ y\^2 + 
        59.02815499564082`\ x\^2\ y\^2}\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Create data file", "Subsection"],

Cell[BoxData[
    \(expr3 = 
      Block[{ni = 11, nj = 11, cc, y}, \[IndentingNewLine]cc = 
          couplefittingl2; \[IndentingNewLine]cc[\([5]\)] = 
          50; \[IndentingNewLine]TableForm[
          Flatten[Table[\[IndentingNewLine]{4, 
                N[cc /. {x \[Rule] 0.5\ \((i - 1)\)/\((ni - 1)\), 
                      y \[Rule] \(-0.35\) + 
                          0.5*\((j - 1)\)/\((nj - 1)\)}\[IndentingNewLine], 
                  12], {2, 4, 5, 6}}, {i, 10, 11}, {j, nj}], 2], \n
          TableSpacing -> {0, 1}]]\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{fp = 
          OpenWrite["\<~/dave/l2_bscan23.in\>", FormatType -> OutputForm, 
            PageWidth -> Infinity]}, \n\t\tWrite[
        fp, "\<l2_bscan23.out\>"]; \n\t\tWrite[fp, "\<35\>"]; \n\t\tWrite[
        fp, "\<3\>"]; \n\t\tWrite[fp, expr3]; Write[fp, \(-1\)]; 
      Close[fp]]\)], "Input"],

Cell[BoxData[
    \("/home/bvds/dave/l2_bscan23.in"\)], "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1600}, {0, 1200}},
AutoGeneratedPackage->Automatic,
WindowSize->{933, 950},
WindowMargins->{{87, Automatic}, {Automatic, 47}},
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"], Inherited, Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
        TextData[ {
          ValueBox[ "FileName"]}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}], "PageNumber"]}},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"FirstPageHeader"->False,
"FirstPageFooter"->False,
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "bvds", \
"dave"}, "dvalley.ps", CharacterEncoding -> "ISO8859-1"],
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 54, 0, 132, "Subtitle"],
Cell[1796, 53, 106, 4, 194, "Subsubtitle"],
Cell[1905, 59, 134, 3, 92, "Text"],

Cell[CellGroupData[{
Cell[2064, 66, 159, 3, 112, "Input",
  InitializationCell->True],
Cell[2226, 71, 183, 3, 112, "Message"]
}, Open  ]],
Cell[2424, 77, 84, 2, 52, "Input",
  InitializationCell->True],
Cell[2511, 81, 92, 2, 52, "Input",
  InitializationCell->True],
Cell[2606, 85, 39, 1, 52, "Input"],
Cell[2648, 88, 359, 7, 292, "Input"],
Cell[3010, 97, 46, 1, 52, "Input"],
Cell[3059, 100, 47, 1, 52, "Input"],
Cell[3109, 103, 47, 1, 52, "Input"],
Cell[3159, 106, 183, 3, 142, "Input"],

Cell[CellGroupData[{
Cell[3367, 113, 37, 0, 106, "Section"],

Cell[CellGroupData[{
Cell[3429, 117, 38, 0, 96, "Subsection"],
Cell[3470, 119, 22425, 312, 2971, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[25932, 436, 38, 0, 60, "Subsection"],
Cell[25973, 438, 24269, 336, 3243, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[50279, 779, 29, 0, 60, "Subsection"],

Cell[CellGroupData[{
Cell[50333, 783, 46, 1, 27, "Input"],
Cell[50382, 786, 2105, 28, 619, "Output"]
}, Open  ]],
Cell[52502, 817, 21, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[52548, 821, 301, 5, 43, "Input"],
Cell[52852, 828, 130, 3, 27, "Output"]
}, Open  ]],
Cell[52997, 834, 112, 3, 32, "Text"],
Cell[53112, 839, 146, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[53283, 846, 66, 1, 27, "Input"],
Cell[53352, 849, 156, 2, 27, "Output"]
}, Open  ]],
Cell[53523, 854, 122, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[53670, 861, 237, 5, 27, "Input"],
Cell[53910, 868, 480, 8, 29, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[54427, 881, 915, 17, 75, "Input"],
Cell[55345, 900, 680, 17, 27, "Output"]
}, Open  ]],
Cell[56040, 920, 132, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[56197, 927, 156, 3, 27, "Input"],
Cell[56356, 932, 113, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56506, 939, 157, 3, 27, "Input"],
Cell[56666, 944, 114, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56817, 951, 888, 14, 139, "Input"],
Cell[57708, 967, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57875, 975, 162, 3, 27, "Input"],
Cell[58040, 980, 112, 2, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58189, 987, 536, 9, 91, "Input"],
Cell[58728, 998, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58895, 1006, 91, 1, 27, "Input"],
Cell[58989, 1009, 1044, 16, 59, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[60070, 1030, 1016, 17, 107, "Input"],
Cell[61089, 1049, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[61256, 1057, 991, 17, 107, "Input"],
Cell[62250, 1076, 305, 5, 23, "Message"],
Cell[62558, 1083, 305, 5, 23, "Message"],
Cell[62866, 1090, 308, 5, 23, "Message"],
Cell[63177, 1097, 166, 3, 23, "Message"],
Cell[63346, 1102, 130, 3, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[63525, 1111, 31, 0, 60, "Subsection"],
Cell[63559, 1113, 53, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[63637, 1117, 139, 2, 27, "Input"],
Cell[63779, 1121, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[63946, 1129, 139, 2, 27, "Input"],
Cell[64088, 1133, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64255, 1141, 136, 2, 27, "Input"],
Cell[64394, 1145, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[64561, 1153, 547, 10, 107, "Input"],
Cell[65111, 1165, 130, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65278, 1173, 136, 2, 27, "Input"],
Cell[65417, 1177, 130, 3, 27, "Output"]
}, Open  ]],
Cell[65562, 1183, 208, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[65795, 1191, 86, 1, 27, "Input"],
Cell[65884, 1194, 135, 3, 27, "Output"]
}, Open  ]],
Cell[66034, 1200, 136, 3, 32, "Text"],

Cell[CellGroupData[{
Cell[66195, 1207, 419, 6, 75, "Input"],
Cell[66617, 1215, 56, 1, 23, "Print"],
Cell[66676, 1218, 57, 1, 23, "Print"],
Cell[66736, 1221, 57, 1, 23, "Print"],
Cell[66796, 1224, 57, 1, 23, "Print"],
Cell[66856, 1227, 57, 1, 23, "Print"],
Cell[66916, 1230, 57, 1, 23, "Print"],
Cell[66976, 1233, 57, 1, 23, "Print"],
Cell[67036, 1236, 57, 1, 23, "Print"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[67142, 1243, 38, 0, 60, "Subsection"],
Cell[67183, 1245, 132, 3, 92, "Text"],

Cell[CellGroupData[{
Cell[67340, 1252, 46, 1, 52, "Input"],
Cell[67389, 1255, 480, 8, 165, "Output"]
}, Open  ]],
Cell[67884, 1266, 31, 0, 60, "Text"],

Cell[CellGroupData[{
Cell[67940, 1270, 464, 9, 232, "Input"],
Cell[68407, 1281, 3473, 72, 406, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[71917, 1358, 370, 7, 292, "Input"],
Cell[72290, 1367, 62, 1, 52, "Output"]
}, Open  ]],
Cell[72367, 1371, 29, 0, 60, "Text"],
Cell[72399, 1373, 595, 11, 292, "Input"],

Cell[CellGroupData[{
Cell[73019, 1388, 328, 6, 292, "Input"],
Cell[73350, 1396, 65, 1, 52, "Output"]
}, Open  ]],
Cell[73430, 1400, 29, 0, 60, "Text"],
Cell[73462, 1402, 591, 11, 292, "Input"],

Cell[CellGroupData[{
Cell[74078, 1417, 328, 6, 292, "Input"],
Cell[74409, 1425, 65, 1, 52, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[74535, 1433, 37, 0, 106, "Section"],
Cell[74575, 1435, 292, 7, 156, "Text"],

Cell[CellGroupData[{
Cell[74892, 1446, 41, 0, 96, "Subsection"],
Cell[74936, 1448, 122767, 1763, 13963, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[197740, 3216, 83, 1, 60, "Subsection"],
Cell[197826, 3219, 134553, 1938, 18539, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[332416, 5162, 34, 0, 60, "Subsection"],
Cell[332453, 5164, 141, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[332619, 5171, 157, 3, 27, "Input"],
Cell[332779, 5176, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[332861, 5182, 47, 1, 27, "Input"],
Cell[332911, 5185, 14719, 227, 223, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[347667, 5417, 722, 14, 101, "Input"],
Cell[348392, 5433, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[348561, 5441, 109, 2, 27, "Input"],
Cell[348673, 5445, 48, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[348758, 5451, 527, 11, 75, "Input"],
Cell[349288, 5464, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[349457, 5472, 111, 2, 27, "Input"],
Cell[349571, 5476, 48, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[349668, 5483, 83, 1, 60, "Subsection"],
Cell[349754, 5486, 133263, 1901, 18651, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[483054, 7392, 34, 0, 60, "Subsection"],
Cell[483091, 7394, 141, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[483257, 7401, 157, 3, 27, "Input"],
Cell[483417, 7406, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[483499, 7412, 47, 1, 27, "Input"],
Cell[483549, 7415, 14433, 226, 223, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[498019, 7646, 720, 14, 101, "Input"],
Cell[498742, 7662, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[498911, 7670, 109, 2, 27, "Input"],
Cell[499023, 7674, 48, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[499108, 7680, 527, 11, 75, "Input"],
Cell[499638, 7693, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[499807, 7701, 125, 2, 27, "Input"],
Cell[499935, 7705, 48, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[500032, 7712, 29, 0, 60, "Subsection"],
Cell[500064, 7714, 21, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[500110, 7718, 356, 6, 59, "Input"],
Cell[500469, 7726, 130, 3, 27, "Output"]
}, Open  ]],
Cell[500614, 7732, 112, 3, 32, "Text"],
Cell[500729, 7737, 108, 2, 27, "Input"],
Cell[500840, 7741, 66, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[500931, 7745, 76, 1, 27, "Input"],
Cell[501010, 7748, 1841, 28, 203, "Output"]
}, Open  ]],
Cell[502866, 7779, 198, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[503089, 7787, 315, 6, 59, "Input"],
Cell[503407, 7795, 1030, 17, 83, "Output"]
}, Open  ]],
Cell[504452, 7815, 45, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[504522, 7819, 466, 9, 91, "Input"],
Cell[504991, 7830, 156, 3, 23, "Message"],
Cell[505150, 7835, 1029, 17, 83, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[506228, 7858, 38, 0, 60, "Subsection"],
Cell[506269, 7860, 551, 10, 123, "Input"],

Cell[CellGroupData[{
Cell[506845, 7874, 328, 6, 91, "Input"],
Cell[507176, 7882, 65, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[507302, 7890, 37, 0, 106, "Section"],

Cell[CellGroupData[{
Cell[507364, 7894, 43, 0, 96, "Subsection"],
Cell[507410, 7896, 122018, 1759, 17391, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[629465, 9660, 93, 3, 60, "Subsection"],
Cell[629561, 9665, 136570, 1956, 17819, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[766168, 11626, 34, 0, 60, "Subsection"],
Cell[766205, 11628, 141, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[766371, 11635, 157, 3, 27, "Input"],
Cell[766531, 11640, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[766613, 11646, 47, 1, 27, "Input"],
Cell[766663, 11649, 14762, 227, 215, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[781462, 11881, 610, 12, 68, "Input"],
Cell[782075, 11895, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[782244, 11903, 109, 2, 27, "Input"],
Cell[782356, 11907, 48, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[782441, 11913, 527, 11, 75, "Input"],
Cell[782971, 11926, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[783140, 11934, 111, 2, 27, "Input"],
Cell[783254, 11938, 48, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[783351, 11945, 93, 3, 60, "Subsection"],
Cell[783447, 11950, 134960, 1940, 18731, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[918444, 13895, 34, 0, 60, "Subsection"],
Cell[918481, 13897, 141, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[918647, 13904, 157, 3, 27, "Input"],
Cell[918807, 13909, 45, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[918889, 13915, 47, 1, 27, "Input"],
Cell[918939, 13918, 14630, 226, 219, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[933606, 14149, 37, 1, 27, "Input"],
Cell[933646, 14152, 37, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[933720, 14158, 617, 12, 68, "Input"],
Cell[934340, 14172, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[934509, 14180, 109, 2, 27, "Input"],
Cell[934621, 14184, 48, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[934706, 14190, 661, 14, 75, "Input"],
Cell[935370, 14206, 132, 3, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[935539, 14214, 125, 2, 27, "Input"],
Cell[935667, 14218, 48, 1, 27, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[935764, 14225, 29, 0, 60, "Subsection"],
Cell[935796, 14227, 21, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[935842, 14231, 356, 6, 59, "Input"],
Cell[936201, 14239, 130, 3, 27, "Output"]
}, Open  ]],
Cell[936346, 14245, 112, 3, 32, "Text"],
Cell[936461, 14250, 108, 2, 27, "Input"],
Cell[936572, 14254, 66, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[936663, 14258, 76, 1, 27, "Input"],
Cell[936742, 14261, 1736, 23, 187, "Output"]
}, Open  ]],
Cell[938493, 14287, 198, 4, 50, "Text"],

Cell[CellGroupData[{
Cell[938716, 14295, 315, 6, 59, "Input"],
Cell[939034, 14303, 1020, 17, 83, "Output"]
}, Open  ]],
Cell[940069, 14323, 45, 0, 32, "Text"],

Cell[CellGroupData[{
Cell[940139, 14327, 466, 9, 91, "Input"],
Cell[940608, 14338, 156, 3, 23, "Message"],
Cell[940767, 14343, 1018, 17, 83, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[941834, 14366, 38, 0, 60, "Subsection"],
Cell[941875, 14368, 550, 10, 123, "Input"],

Cell[CellGroupData[{
Cell[942450, 14382, 328, 6, 91, "Input"],
Cell[942781, 14390, 65, 1, 27, "Output"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

